
/*********************************************************************
 *  made by : Kees Lemmens
 *  version : 0.2
 *  date    : June 1999
 *
 *  class name  : FileViewer
 *  function    : Pops up a separate window with scrollbars to show a file
 *                from disk or URL (i.e. a local or remote Help text) 
 *********************************************************************/

import java.io.*;
import java.net.*;
import java.awt.*;
import java.awt.event.*;

class FileViewer
extends WindowAdapter // defines all Windowevents as empty methods
implements ActionListener
{
   // Some predefined settings :
   static final int USEFILE=0, USEURL=1; 
   static final int BUFFERSIZE = 1024;

   // And the class variables + methods :
   String s;
   BufferedReader bin;
   Frame frame;
   TextArea text;

   public FileViewer(String filename,String title,int mode) // constructor
   { 
      createWidgets(title);
      frame.setSize(800,600);
      frame.addWindowListener(this);
      
      switch(mode)
      {
       case USEURL : // reads from an arbitrary URL :
	 openUrl(filename);
	 break;
       case USEFILE: // reads only from the local filesystem
       default   :
	 openFile(filename);
      }
      try
      {
	 while((s = bin.readLine()) != null)
	 {
	    if(Debug.LEVEL >=1) 
	      Debug.print(s);
	    text.append(s + "\n");
	 }
	 bin.close();
      }
      catch(IOException io) {} // EOF      
      
      frame.setVisible(true); // now show the window (this is faster !)
   }
   
   private void createWidgets(String title)
   {
      Panel panel;
      Button button;
      Font myfont = new Font("MonoSpaced",Font.PLAIN,14);
      
      frame  = new Frame(title);
      panel  = new Panel();
      text   = new TextArea(); // Omit Dimension to allow for resizing 
      button = new Button("Dismiss");
      
      frame.add(text,"Center");
      panel.add(button);
      frame.add(panel,"South");
      
      button.addActionListener(this);
      
      text.setEditable(false);
      text.setFont(myfont);
   }

   private void openUrl(String urlname)
   {
      InputStream sin;
      InputStreamReader uin;
      
      if(Debug.LEVEL >=1) 
	Debug.print("Now using a URL ...");
      try
      {
	 sin = (new URL(urlname)).openStream();
	 uin = new InputStreamReader(sin);
	 bin = new BufferedReader(uin,BUFFERSIZE);
      }
      catch(IOException io)
      {
	 System.err.println("IO Error for " + urlname + ": " + io );
	 return;
      }
   }
   
   private void openFile(String filename)
   {
      FileReader fin;
      
      if(Debug.LEVEL >=1) 
	Debug.print("Now using a local file ...");
      try
      {
	 	fin = new FileReader(filename);
	 	bin = new BufferedReader(fin,BUFFERSIZE);
      }
	  catch(FileNotFoundException fn)
	  {
		System.err.println("File " + filename + " not found :" + fn);
		return;
	  }  
	  catch(IOException io)
	  {
		System.err.println("IO Error for " + filename + ": " + io );
		return;	    
	  }      
   }
   
   private void removeWindow()
   {
      frame.setVisible(false);
      frame.dispose();
   }
   
   // and the event listeners :
   
   public void windowClosing(WindowEvent we)
   { removeWindow();}
   
   public void actionPerformed(ActionEvent ev) // JDK 1.1
   { removeWindow();}
   
    // just for testing :
/*    public static void main(String argv[])
    {  new FileViewer(argv[0],argv[0],USEFILE); }
 */  
}
