
/*********************************************************************
 *  made by : Kees Lemmens
 *  version : 0.2
 *  date    : June 1999
 *
 *  class name  : Player
 *  function    : Contains a separate thread to start the canvas player
 *                while retaining all other functionality. Also provides
 *                a scrollbar with value and stopbutton.
 *  
 *  Modified : January 2004 JB
 *  In method run() updating of current is placed to the end of the method
 *  in order to prevent skipping to the next file before the sleeping of the
 *  play-thread (as controlled by the slider) takes effect
 * 
 *********************************************************************/

import java.awt.*;
import java.text.*;
import java.awt.event.*;

public class Player extends Dialog // extends Frame
implements Runnable, AdjustmentListener, ActionListener
{
   final static int SBMIN  =    5; //  0.05 sec
   final static int SBMAX  = 1000; // 10    sec
   final static int SBDEF  =  400; //  4    sec
   final static int SBSIZE = (SBMAX-SBMIN)/5; // length of the adjuster
   
   JsepView   applet;
   Thread thread;
   Scrollbar slider;
   Label value;
   Button button;
   NumberFormat n;
   
   public Player(JsepView applet)
   {
      //      super("Playing speed"); // Frame 
      super(applet.getParentFrame(applet),"Playing speed"); // Dialog
      this.applet = applet;

      thread = new Thread(this);
      
      n = NumberFormat.getInstance(); // formats the slider value display
      n.setMinimumFractionDigits(2);
      n.setMaximumFractionDigits(2);
      
      createSlider();
      thread.start();
   }
   
   private void createSlider()
   {
      Panel panel = new Panel();
      
      slider = new Scrollbar(Scrollbar.HORIZONTAL, SBDEF,
			     SBSIZE, SBMIN, SBMAX + SBSIZE);
      
      button = new Button("Stop");
      value  = new Label(n.format(SBDEF/100.0) + " sec");
      
      slider.setBlockIncrement(20);
      slider.addAdjustmentListener(this);
      button.addActionListener(this);
      
      setSize(200,100);
      setLayout(new BorderLayout());
      
      add(slider,"North");
      panel.add(value,"East");
      panel.add(button,"West");
      add(panel,"South");
      
      setVisible(true);
   }
   
   public void run()
   {
      try
      {
	 while(true)
	 {	    
	    applet.header.setCurrentName(applet.playlist[applet.current]);
	    System.err.println("(Player) applet.playlist["+applet.current+"]: " + applet.playlist[applet.current]);
	    applet.canvas.repaint();
	    Thread.sleep(slider.getValue()*10);
		if(applet.current<applet.playlist.length-1)
		   applet.current++;
		else
		   applet.current=0;
	 }
      }
      catch (InterruptedException i)
      {
	 ;
      }
   }

   public void stop()
   {
      thread.stop();
      setVisible(false);
      dispose();
   }

   public void adjustmentValueChanged(AdjustmentEvent ae) 
   {
      value.setText(n.format(slider.getValue()/100.0) + " sec");
   }

   public void actionPerformed(ActionEvent ae) 
   {
      applet.togglePlay(); // stops thread and changes button color
   }
}
