
/*********************************************************************
 *  made by : Kees Lemmens
 *  version : 0.1
 *  date    : May 1999
 *
 *  class name  : ASCII and Binary Record Readers for sepplots.
 *  function    : Contains all routines to read the plot from a file
 *                (either in ASCII or Java portable binary format)
 *********************************************************************/

import java.io.*;
import java.net.URL;

class RecordReader
{
   public static final byte AUTO=0,ASCII=1,BINARY=2,NETBINARY=3,
     FORTRANBE=4,FORTRANLE=5;
   public static int status=0;
   public JSepRecord record=null;
   
   public int readInt() throws IOException
   {return 0;}
   public String readString() throws IOException
   {return null;}
   public Vertex readVertex() throws IOException
   {return null;}
   public Vertex[] readVertices(int n) throws IOException
   {return null;}
   public JSepRecord readRecord() throws IOException
   {return null;}
   public void openFile(String plotfile) throws IOException
   {}
   public void close() throws IOException
   {throw new IOException(); }

   public RecordReader setReaderType(byte choice)
   {
      switch(choice)
      {
       case AUTO      : return new AutoDetectingReader();
       case ASCII     : return new AsciiRecordReader();
       case BINARY    : return new BinaryRecordReader();
       case NETBINARY : return new NetRecordReader();
       case FORTRANBE : return new BigEndianFortranReader();
       case FORTRANLE : return new LittleEndianFortranReader();
       default: return this; // obviously must fail ....
      }
   }
}

final class AutoDetectingReader extends RecordReader
{
   public static int status;
   private static FileInputStream fin;
   RecordReader realReader;
   
   public int readInt()
     throws IOException
   {
      return realReader.readInt();
   }
   
   public String readString()
     throws IOException
   {
      return realReader.readString();
   }
   
   public Vertex readVertex()
     throws IOException
   {
      return realReader.readVertex();
   }
   
   public Vertex[] readVertices(int n)
     throws IOException
   {
      return realReader.readVertices(n);
   }
   
   public JSepRecord readRecord()
     throws IOException
   {
      return realReader.readRecord();
   }
   
   public void openFile(String plotfile)
   {
      FileInputStream fin;
      RecordReader reader;
      byte b[] = new byte[4];

      try
      {  fin = new FileInputStream(plotfile);
	 fin.read(b);
	 fin.close();
	 if(Debug.LEVEL >= 2)
	   Debug.print("Header read : " + b);
	 switch(b[0])
	 {
	  case 0x18 : realReader = new LittleEndianFortranReader(); break;
	  case 0x31 : realReader = new AsciiRecordReader();         break;
	  case 0x00 :
	    if(b[1] == 0x07)
	    {
	       realReader = new BinaryRecordReader();	      break;
	    }
	    if(b[3] == 0x18)
	    {
	       realReader = new BigEndianFortranReader();     break;
	    }
	    else ; // goto default, which is fail 
	  default   : realReader = null;
	    System.err.println("Autodetect failed, format not recognized");
	 }
	 realReader.openFile(plotfile);
      }
      catch (FileNotFoundException f)
      {
	 System.err.println("Autodetect failed, file not found : " + f);
	 return;
      }
      catch (IOException i)
      {
	 System.err.println("Autdetect failed, error opening file : " + i);
	 return;
      }
   }
   
   public void close()
     throws IOException
   {
      realReader.close();
   }
}

final class AsciiRecordReader extends RecordReader
// standard ASCII Sepran plot files
{
   public static int status;
   private JSepRecord record = new JSepRecord();
   private static BufferedReader  bin;
   private static StreamTokenizer tin;
   
   public int readInt()
     throws IOException
   {
      status = tin.nextToken();
      return (int)tin.nval;
   }
   
   public  String readString()
     throws IOException
   {
      return bin.readLine();
   }
   
   public Vertex readVertex()
     throws IOException
   {
      Vertex vertex = new Vertex();

      tin.nextToken(); vertex.x = (float)tin.nval;
      tin.nextToken(); vertex.y = (float)tin.nval;

      return vertex;
   }
   
   public Vertex[] readVertices(int n)
     throws IOException
   {
      Vertex[] vertices = new Vertex[n];

      for(int x=0;x<n;x++)
      {
	 vertices[x] = new Vertex();
	 tin.nextToken(); vertices[x].x = (float)tin.nval;
	 tin.nextToken(); vertices[x].y = (float)tin.nval;
      }
      
      return vertices;
   }
   
   public JSepRecord readRecord()
     throws IOException
   {
      tin.nextToken(); record.type   = (byte)tin.nval;
      tin.nextToken(); record.xcrd   = (float)tin.nval; 
      tin.nextToken(); record.ycrd   = (float)tin.nval;
      tin.nextToken(); record.angle  = (float)tin.nval;
      tin.nextToken(); record.height = (float)tin.nval;
      tin.nextToken(); record.nchar  = (byte)tin.nval;

      return record;
   }
   
   public void openFile(String plotfile)
   {
      FileReader fin;
      
      try
      {
	 fin = new FileReader(plotfile);
	 bin = new BufferedReader(fin,100000);
	 tin = new StreamTokenizer(bin);
	 
	 tin.slashSlashComments(true); // skip C++ // comments
	 tin.parseNumbers();           // (just for sure: already default) 
	 tin.whitespaceChars(' ',',');
	 // all chars between space and comma serve as white characters

	 System.err.println("Opening a Standard ASCII sepplot ...");
      }
      catch (FileNotFoundException f)
      {
	 System.err.println("Ascii file not found : " + f);
	 return;
      }
      catch (IOException i)
      {
	 System.err.println("Error opening Ascii file : " + i);
	 return;
      }
   }
   
   public void close()
     throws IOException
   {
      bin.close();
   }
}

final class BinaryRecordReader extends RecordReader
// Portable Java Binary sepran plot files
{
   public static int status;
   private JSepRecord record = new JSepRecord();
   private static FileInputStream fin;
   private static BufferedInputStream bin;
   private static DataInputStream din;

   public int readInt()
     throws IOException
   {
      return din.readInt();
   }
   
   public String readString()
     throws IOException
   {
      return din.readUTF();
   }
   
   public Vertex readVertex()
     throws IOException
   {
      Vertex vertex = new Vertex();
      
      vertex.x = din.readFloat();
      vertex.y = din.readFloat();
      return vertex;
   }
   
   public Vertex[] readVertices(int n)
     throws IOException
   {
      Vertex[] vertices = new Vertex[n];

      for(int x=0;x<n;x++)
      {
	 vertices[x] = new Vertex();
	 vertices[x].x = din.readFloat();
	 vertices[x].y = din.readFloat();
      }
      return vertices;
   }
   
   public JSepRecord readRecord()
     throws IOException
   {
      record.type   = din.readByte();
      record.xcrd   = din.readFloat();
      record.ycrd   = din.readFloat();
      record.angle  = din.readFloat();
      record.height = din.readFloat();
      record.nchar  = din.readByte();

      return record;
   }
   
   public void openFile(String plotfile)
   {
      try
      {
	 fin = new FileInputStream(plotfile);
	 bin = new BufferedInputStream(fin,100000);
	 din = new DataInputStream(bin);

	 if(din.readUTF().equals("jbinsep"))
	    System.err.println("Opening a Java binary sepplot ...");
	 else
	 {
	    System.err.println("Not a Java binary sepplot: skipping " 
			       + plotfile);
	    status = -1;
	    din = null; // force read errors
	 }
      }
      
      catch (FileNotFoundException f)
      {
	 System.err.println("Binary file not found : " + f);
	 return;
      }
      catch (IOException i)
      {
	 System.err.println("Error opening binary file : " + i);
	 din = null;
	 return;
      }
   }

   public void close()
     throws IOException
   {
      fin.close();
   }
}

class BigEndianFortranReader extends RecordReader
// Fortran binary sepran plot files
{
   public static int status;
   private JSepRecord record = new JSepRecord();
   private static FileInputStream fin;
   private static BufferedInputStream bin;
   protected static DataInputStream din;

   public int readInt()
     throws IOException
   {
      return din.readInt();
   }
   
   public String readString()
     throws IOException
   {
      byte data[];

      int len = readInt();
      data = new byte[len];
      din.read(data,0,len);
      
      din.readInt(); // discard
      return new String(data);
   }

   public float readFloat()
     throws IOException
   {
      int flt = readInt();
      
      return Float.intBitsToFloat(flt);
   }
   
   public Vertex readVertex()
     throws IOException
   {
      Vertex vertex = new Vertex();
      
      vertex.x = readFloat();
      vertex.y = readFloat();
      return vertex;
   }
   
   public Vertex[] readVertices(int n)
     throws IOException
   {
      Vertex[] vertices = new Vertex[n];

      din.readInt(); // discard 

      for(int x=0;x<n;x++)
      {
	 vertices[x] = new Vertex();
	 vertices[x].x = readFloat();
	 vertices[x].y = readFloat();
      }

      din.readInt(); // discard 

      return vertices;
   }
   
   public JSepRecord readRecord()
     throws IOException
   {
      din.readInt(); // discard 
      
      record.type   = (byte)readInt();
      record.xcrd   = readFloat();
      record.ycrd   = readFloat();
      record.angle  = readFloat();
      record.height = readFloat();
      record.nchar  = (byte)readInt();
      
      din.readInt(); // discard 
      
      return record;
   }
   
   public void openFile(String name)
   {
      report();
      openBEFile(name);
   }
   
   private void report()
   {
      Debug.print("Opening Big Endian Fortran meshoutput ...");
   }

   public void openBEFile(String plotfile)
   {
      try
      {
	 fin = new FileInputStream(plotfile);
	 bin = new BufferedInputStream(fin,100000);
	 din = new DataInputStream(bin);

      	 System.err.println("Opening a Fortran binary sepplot ...");
      }
      
      catch (FileNotFoundException f)
      {
	 System.err.println("Fortran binary file not found : " + f);
	 return;
      }
      catch (IOException i)
      {
	 System.err.println("Error opening Fortran binary file : " + i);
	 return;
      }
   }

   public void close()
     throws IOException
   {
      bin.close();
   }
}

final class LittleEndianFortranReader extends BigEndianFortranReader
{
   public void openFile(String name)
   {
      report();
      openBEFile(name);
   }
   
   private void report()
   {
      Debug.print("Opening Little Endian Fortran meshoutput ...");
   }

   public int readInt()
     throws IOException
   {
      int bytes[] = new int[4];
      
      int littleEndian = din.readInt();
      
      bytes[0] =  littleEndian      & 0xff;
      bytes[1] = (littleEndian>>8 ) & 0xff;
      bytes[2] = (littleEndian>>16) & 0xff;
      bytes[3] = (littleEndian>>24) & 0xff;
      
      // Now swap the byteorder for a little endian file 
      return (bytes[0] << 24 | bytes[1] << 16 |
	      bytes[2] <<  8 | bytes[3]);
   }
}

final class NetRecordReader extends RecordReader
// Portable Java Binary sepran plot files using a URL
{
   public static int status;
   private JSepRecord record = new JSepRecord();
   private static InputStream iin;
   private static BufferedInputStream bin;
   private static DataInputStream din;

   public int readInt()
     throws IOException
   {
      return din.readInt();
   }
   
   public String readString()
     throws IOException
   {
      return din.readUTF();
   }
   
   public Vertex readVertex()
     throws IOException
   {
      Vertex vertex = new Vertex();
      
      vertex.x = din.readFloat();
      vertex.y = din.readFloat();
      return vertex;
   }
   
   public Vertex[] readVertices(int n)
     throws IOException
   {
      Vertex[] vertices = new Vertex[n];

      for(int x=0;x<n;x++)
      {
	 vertices[x] = new Vertex();
	 vertices[x].x = din.readFloat();
	 vertices[x].y = din.readFloat();
      }
      return vertices;
   }
   
   public JSepRecord readRecord()
     throws IOException
   {
      record.type   = din.readByte();
      record.xcrd   = din.readFloat();
      record.ycrd   = din.readFloat();
      record.angle  = din.readFloat();
      record.height = din.readFloat();
      record.nchar  = din.readByte();

      return record;
   }
   
   public void openFile(String plotfile) // override superclass method
   {
      try
      {
	 iin = (new URL(plotfile)).openStream();
	 bin = new BufferedInputStream(iin,100000);
	 din = new DataInputStream(bin);
	 
	 if(din.readUTF().equals("jbinsep"))
	    System.err.println("Opening a remote Java binary sepplot ...");
	 else
	 {
	    System.err.println("Remote not a Java binary sepplot : Skipping :" 
			       + plotfile);
	    status = -1;
	    din = null;
	 }
      }
      
      catch (FileNotFoundException f)
      {
	 System.err.println("Binary network file not found : " + f);
	 return;
      }
      catch (IOException i)
      {
	 System.err.println("Error opening binary network file : " + i);
	 return;
      }
   }
   
   public void close()
     throws IOException
   {
      din.close();
   }
}

/*
 * This does not work because every object is only serialized once, no matter 
 * if its fields may have changed later : subsequent call only result in a 
 * reference to the first instance !!!
 *
 final class SerializedRecordReader extends RecordReader
// Java Serialized sepran plot files
{
   public static int status;
   private static FileInputStream fin;
   private static BufferedInputStream bin;
   private static ObjectInputStream oin;

   public String readString()
     throws IOException
   {
      return oin.readUTF();
   }
   
   public Vertex readVertex()
     throws IOException
   {
      try
      {
	 return (Vertex)oin.readObject();
      }
      catch (ClassNotFoundException cn)
      {
	 System.err.println("ClassNotFoundException " + cn);
	 return null;
      }
   }

   public Vertex[] readVertices(int n)
     throws IOException
   {
      Vertex[] vertices = new Vertex[n];

      for(int x=0;x<n;x++)
      {
	 vertices[x] = readVertex();
      }
      return vertices;
   }
   
   public JSepRecord readRecord()
     throws IOException
   {
      try
      {
	 return (JSepRecord)oin.readObject();
      }
      catch (ClassNotFoundException cn)
      {
	 System.err.println("ClassNotFoundException " + cn);
	 return null;
      }
   }
   
   public void openFile(String plotfile)
   {
      try
      {
	 fin = new FileInputStream(plotfile);
	 bin = new BufferedInputStream(fin,100000);
	 oin = new ObjectInputStream(bin);

	 if(oin.readUTF().equals("jsersep"))
	    System.err.println("Opening a Java serialized sepplot ...");
	 else
	 {
	    System.err.println("Not a Java serialized sepplot : Skipping :" 
			       + plotfile);
	    status = -1;
	 }
      }
      
      catch (FileNotFoundException f)
      {
	 System.err.println("Serialized file not found : " + f);
      }
      catch (IOException i)
      {
	 System.err.println("Error opening binary file : " + i);
      }
   }
   
   public void close()
     throws IOException
   {
      bin.close();
   }
}
*/
