      subroutine askscl ( number, minnum, maxnum, code, icode )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  2.9  date 08-12-2013 Extension with integer scalars
!        version  2.8  date 20-04-2012 Indexed scalars
!        version  2.7  date 22-02-2012 Correction name
!
!   copyright (c) 1993-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the integer sequence number after a keyword and check if it is
!     in the correct range
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cseprdf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cpost1f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer number, minnum, maxnum, icode
      character (len=*) code

!     code           i    name of keyword (for error messages)
!     icode          i    Type of keyword before the number
!                         See READSC
!     maxnum         i    maximum value of integer
!     minnum         i    minimum value of integer
!     number         o    integer number found
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numchar, nh
      character (len=80) keyword
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     keyword        contains the keyword without number
!     nh             Help variable
!     numchar        Number of characters in keyword
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     READADDVEC     Reread name of new vector and add to list of vectors
!     READNAMEVEC    Read name of vector in common cpost5
!     READSPLITKEYW  check whether the keyword contains numbers
!                    If so split the keyword into a part without number and
!     READSTRSCALS   Check if name read corresponds to one the scalars
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     574   Not an integer number found after the keyword
!    1881   The sequence number found after the keyword is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'askscl' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askscl'
         call prinin ( typeitems, nitems, 'typeitems' )
         write(irefwr,1) 'ityprd, lastitem, icode', &
                          ityprd, lastitem, icode
         write(irefwr,2) 'realrd', realrd
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ityprd==102 ) then

!     --- New vector found, sequence number in realrd

         number = nint(realrd)
         go to 1000

      end if  ! ( ityprd==102 )

      if ( ityprd==4 .and. typeitems(lastitem+1)/=13 .and. &
           ityprd/=2 .and. ityprd/=11 ) then

!     --- The scalar or vector number has not yet been found
!         Read next item

         ispecl = 9
         call asknxt
         ispecl = 0
         if ( debug ) then
            write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
            write(irefwr,2) 'realrd', realrd
         end if  ! ( debug )

      end if  ! ( ityprd/=2 .and. ityprd/=11 )

      if ( ityprd==4 ) then

!     --- text found, check if it is vector or scalar

         call readsplitkeyw ( numchar, keyword )
         if ( debug ) then
            write(irefwr,1) 'ityprd', ityprd
            write(irefwr,*) 'string ', string(1:nchar)
            write(irefwr,*) 'keyword ', keyword(1:numchar)
         end if  ! ( debug )
         if ( icode==6 .or. icode==12 .or. icode==82 ) then

!        --- icode = 6 or 12, scalar

            if ( keyword(1:numchar)=='S' .and. &
                 typeitems(lastitem+1)/=13 .or. &
                 keyword(1:numchar)=='SCALAR' ) then

!           --- Text S or SCALAR found, read next item

               if ( debug ) write(irefwr,1) 'lastitembef', lastitembef
               lastitem = lastitembef+1
               call asknxt
               if ( debug ) then
                  write(irefwr,1) 'ityprd', ityprd
                  write(irefwr,1) 'lastitem', lastitem
                  write(irefwr,*) 'string ', string(1:nchar)
                  write(irefwr,*) 'keyword ', keyword(1:numchar)
               end if  ! ( debug )

            end if
            if ( ityprd/=2 .and. ityprd/=11 ) then

!           --- Not a number found, interprete text

               call readstrscals  ! check if it is a scalar

            end if  ! ( ityprd/=2 .and. ityprd/=11 )

         else

!        --- vector found

            if ( debug ) then
               write(irefwr,*) 'vector found'
               write(irefwr,1) 'ityprd', ityprd
               write(irefwr,*) 'keyword ', keyword(1:numchar)
            end if  ! ( debug )

            if ( ityprd/=2 .and. ityprd/=11 ) then

!           --- Not a number found, interprete text

               call readnamevec  ! check if it is a vector

            end if  ! ( ityprd/=2 .and. ityprd/=11 )

            if ( debug ) write(irefwr,1) 'ityprd', ityprd

            if ( ityprd==4 .and. icode/=67 .and. &
                      ( keyword(1:numchar)=='V' .or. &
                        keyword(1:numchar)=='VECT' .or. &
                        keyword(1:numchar)=='VECTOR' ) ) then

!           --- Text V or VECTOR found, read next item
!               In case the number is part of the text we have to
!               decrease lastitem to read the number

               if ( numchar<nchar ) lastitem = lastitembef+1
               call asknxt
               if ( debug ) write(irefwr,1) 'ityprd', ityprd

               if ( ityprd/=2 .and. ityprd/=11 ) then

!              --- Not a number found, interprete text

                  call readnamevec  ! check if it is a vector

               end if  ! ( ityprd/=2 .and. ityprd/=11 )

            end if  ! ( ityprd==4 .and.... )

         end if  ! ( icode==6 .or. icode==12 )

      end if  ! ( ityprd==4 )

      if ( ityprd==4 .and. icode==67 ) then

!     --- Vector found, it concerns a new vector

         call readaddvec
         number = numvec

      else if ( ityprd/=2 .and. ityprd/=11 ) then

!     --- item read is not an integer

         call errchr ( code, 1 )
         call errsub ( 574, 0, 0, 1 )
         call errcrd
         go to 1000

      else

         number = mod(nint(realrd),100000000)
         nh = mod(number,10000)
         nh = mod(number,5000)

         if ( debug ) &
            write(irefwr,1) 'number, nh, maxcls', number, nh, maxcls

!        --- integer found, check value

         if ( number<=maxcls ) then

!        --- real scalar

            if ( number<=-1000 ) then

!           --- number <=-1000, hence special vector

               number = index_rdarr(1)

            else if ( number>-1000 .and. nh<minnum .or. nh>maxnum ) &
               then

!           --- integer out of range

               call errint ( nh, 1 )
               call errint ( minnum, 2 )
               call errint ( maxnum, 3 )
               call errchr ( code, 1 )
               call errsub ( 1881, 3, 0, 1 )
               call errcrd
               go to 1000

            end if

         else

!        --- integer scalar

            nh = nh-maxcls
            if ( nh>maxintscals ) then

!           --- integer out of range

               call errint ( nh, 1 )
               call errint ( 0, 2 )
               call errint ( maxintscals, 3 )
               call errchr ( code, 1 )
               call errsub ( 1881, 3, 0, 1 )
               call errcrd
               go to 1000

            end if

         end if  ! ( number>maxcls )

      end if

1000  call erclos ( 'askscl' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'number, lastitem', number, lastitem
         write(irefwr,*) 'End askscl'

      end if

      end
