      subroutine askvec ( number, inpsep, rinsep, numvec,
     +                    istrin, kmesh, istart, nprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 26-05-2011 Extra parameter nprob
!        version  2.9  date 02-03-2011 Make vector from vectors
!        version  2.8  date 03-05-2010 Extension with INTEGRATED_TO_NODAL
!
!   copyright (c) 1993-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret the options after the keyword compute_vector i and store in
!     inpsep and rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpi'
      include 'SPcommon/cconst'
      include 'SPcommon/cseprdnew'
      include 'SPcommon/cseprd'
      include 'SPcommon/cscalvec'
      include 'SPcommon/cinout'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(0:*), number, numvec, istrin, istart, kmesh(*),
     +        nprob
      double precision rinsep(*)

!     inpsep         o    The integer input read is stored in array INPSEP of
!                         length 20+nelgrp
!                         Array INPSEP is filled as follows:
!                          0  Type of function to be called
!                             6:  manvec
!                             8:  derivatives
!                          1  ISEQNR       Sequence number corresponding to
!                                          input.  If iseqnr = -1,
!                                          the next one will be used
!                          2  IPROB        Problem sequence number
!                                          If iprob = -1, the next one will be
!                                          used
!                          3  IVECTOR      Vector sequence number
!                                          (output vector)
!                                          If ivector = -1, the same value as
!                                          iprob will be used
!                          4: ICHOIS (See prmanv/manvec)
!                          5: IDEGFD (See manvec)
!                          6: IPOINT (See manvec)
!                          7: ISPEC  (See manvec)
!                          8: ICHNRM (See manvec)
!                          9: Vector sequence number first input vector
!                         10: Vector sequence number second input vector
!                         11: Scalar sequence number first scalar
!                         12: Scalar sequence number second scalar
!                         13: Sequence number first position in RINSEP
!                         14: iskip  defines if element groups must be skipped
!                             (>0) or not (0)
!                             If >0 (21) positions iskip to iskip+nelgrp-1 are
!                             used to indicate if an element group is skipped
!                             or not
!                             0: do not skip
!                             1: skip
!                         used (1) or not (0)
!                         15: ncurves  number of curves given
!                             if>0, these curves are stored from position 21
!     istart        i/o   Next free position in inpsep
!     istrin        i/o   Starting address of next positions in rinsep
!                         used (1) or not (0)
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     nprob          i    Number of various problems
!     number         i    Vector sequence number
!     numvec        i/o   Number of vectors read
!     rinsep         o    Contains real input at output. The storage depends on
!                         the storage of inpsep
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nncode, nparm, nout, nextra
      parameter ( nncode=35, nparm=20, nout=25, nextra=1 )
      integer ncode(nncode), itype(nncode), ncodmx(nncode),
     +        iseqkeyw(10), iretur, iscal1, nmcode(nextra),
     +        idegfd, nseqkeyw, isignvec(2), minussign, nelgrp,
     +        ncurvs
      double precision table(nncode), tabmin(nncode), tabmax(nncode),
     +                 tabout(nout)
      character (len=22) code(nncode)
      character (len=1) strout(1)
      character (len=1) codemn(nextra)
      logical found, debug

!     code          Table with keywords corresponding to options of
!                    COMPUTE_VECTOR
!     codemn         Table with main keywords
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     found          Indicates if one of the possibilities modulus, length,
!                    phase or phase or subtract has been found (true) or not
!                    (false)
!     idegfd         Sequence number of degree of freedom
!     iretur         Return value of subroutine INTX02
!                    Possible values:
!                    -1:  error has been found
!                     0:  normal return
!     iscal1         First scalar
!     iseqkeyw       Array of size nseqkeyw that contains the sequence in
!                    which keywords are found in a record
!                    isekeyw(i)>0, means i-th keyword found is the
!                                    iseqkeyw(i)-th keyword in the table
!                    isekeyw(i)<0, means keyword refers to number found
!                                    -isekeyw(i) refers to position in tabout
!     isignvec       Contains the signs of the two vectors in case of
!                    linear combination
!     itype         Indicates the type of item expected after a keyword in the
!                    options table for code. See subroutine INTX02
!     minussign      Sign to be used for first scalar
!     ncode         Table containing the number of significant characters in
!                    the keyword table code
!     ncodmx         Table containing the number of maximum characters in
!                    the keyword table code
!     ncurvs         Number of curves in mesh
!     nelgrp         Number of element groups
!     nextra         Number of extra keywords in codenm
!     nmcode         Table containing the number of significant characters
!                    in the main keyword table codemn, including those of
!                    the extra texts
!     nncode         Number of keywords in code
!     nout           Length of array tabout
!     nparm          Number of parameters in inpsep before information
!                    concerning the element groups
!     nseqkeyw       Number of entries in iseqkeyw
!     strout         Dummy parameter
!     table         Contains the values of the options corresponding to code
!     tabmax         Contains the maximum values of the options corresponding
!                    to code
!     tabmin         Contains the minimum values of the options corresponding
!                    to code
!     tabout         Contains the values found at output of intx02
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKVEC01       Subroutine corresponding to askvec dealing with iretur<-10,
!                    nseqkeyw==1 and iseqkeyw(1)<0
!     ASKVEC24       Interpret part following mesh_velocity in subroutine askvec
!     ASKVEC25       Store information about skip element groups
!     ASKVECCURVS    Interpret information about curves
!     ASKVECDERIV    Interpret input for derivatives following vi = derivatives
!     ASKVECLINCOM   Store coefficients for subroutine askvec
!     ASKVECNOTFOUND Interpret input of askvec if none of the standard options
!                    is found
!     ASKVECSIGN     Interpret the options of the form v = sign(V1) * V2
!     ASKVECSTSCAL   Store scalar numbers for subroutine askvec
!     ASKVECSTVEC    Store vector numbers for subroutine askvec
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INTX02EXT      Interpret a record containing keywords followed by numbers
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
!         askvec requires input from the standard input file
!         The following input is recognised:
!         STRUCTURE    (has been read already be readal)
!         start of input
!         the next records are interpreted in the sequence they are read
!         the program is carried out in the same sequence
!         [COMPUTE_]VECTOR i, options (all in one line)
!         where options may have the following contents
!         MODULUS VECTOR j             compute modulus of vector j
!         PHASE VECTOR j               compute phase of vector j
!         LENGTH VECTOR j              compute length of vector j
!         CONJUGATE VECTOR j           compute complex conjugate of
!                                      vector j
!         FUNC VECTOR j1 VECTOR j2     Vi = f ( Vj1, Vj2)
!         SIGN ( Vi ) or  SIGN ( Vi ) * Vj vector is replaced by sign
!                                      componentwise
!         Vi * Vj                      componentwise
!         MULTIPLY = m                 multiply result by m
!         SUBTRACT                     subtract constant from vector j
!         [LIN_COMBINATION] SCALAR1 VECTOR1 + SCALAR2 VECTOR2
!                                   Vi = S1 V1 + S2 V2
!           or:
!         [LIN_COMBINATION] COEF1 = i/%i VECTOR1 COEF2 = j/%j VECTOR2
!           or:
!         [LIN_COMBINATION] alpha VECTOR1 + beta VECTOR2
!         EXTRACT                      extract degree of freedom from vector
!         DEGFD j                      sequence number of degree of freedom
!         ELEMENT_GROUPS = (s1, s2, ...)           (Default all)
!         CURVES = (c1, c2, ...)                   (Default nonel)
!         SKIP_ELEMENT_GROUPS = (s1, s2, ... )     (Default none)
!         REAL VECTOR j                compute real part of vector j
!         IMAGINARY VECTOR j           compute imaginary part of vector j
!         CONSTANT = c                 this constant may be either a
!         SCALAR J                     constant or a scalar number
!         INNER_PRODUCT VECTOR j1 VECTOR j2 Component wise inner product
!         MAP VECTOR i, PROBLEM = p, TYPE = t
!             t may be: 0, i.e. SOLUTION_VECTOR
!                       k, i.e. VECTOR_OF_SPECIAL_STRUCTURE k
!         MESH_VELOCITY ( VECTOR1, VECTOR2 )
!         INTEGRATED_TO_NODAL Vj [ZERO_LEVELSET i]
!         TRANSFORM_TO_NORMALDIR Vj
!         SMOOTH_END_POINTS
!         STRESS_VECTOR Vj
!         = ( V1, V2 )                Make vector from V1 and V2
!         DERIVATIVES ( V1, S1, icheld = i, problem =  p, seq_coef = s1,
!                       seq_deriv  = s2 )
!         This program only interprets the part options
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1903   From the possibilities: PHASE, MODULUS and LENGTH at least two are
!           found in the record
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code / 'MODULUS', 'LENGTH', 'VECTOR', 'PHASE', 'MULTIPLY',
     +            'SUBTRACT', 'CONSTANT', 'SCALAR', 'LIN_COMBINATION',
     +            'EXTRACT', 'DEGFD', 'REAL', 'IMAGINARY', 'CONJUGATE',
     +            'FUNC', 'INNER_PRODUCT', 'MAP', 'PROBLEM',
     +            'TYPE', 'COEF', 'COEF', 'ELEMENT_GROUPS',
     +            'SKIP_ELEMENT_GROUPS', 'MESH_VELOCITY',
     +            'INTEGRATED_TO_NODAL', 'TRANSFORM_TO_NORMALDIR',
     +            'CURVES', 'SMOOTH_END_POINTS', 'STRESS_VECTOR',
     +            'ZERO_LEVELSET', 'ICHELD', 'SEQ_COEF', 'SEQ_DERIV',
     +            'DERIVATIVES', 'SIGN' /
      data ncode / 3, 3, 1, 3, 3,    3, 3, 3, 3, 3,
     +              3, 4, 4, 4, 3,    5, 3, 4, 3, 4,
     +              4, 4, 4, 8, 5,    5, 4, 5, 5, 8,
     +              4, 6, 7, 4, 4 /
      data ncodmx / 7, 6, 6, 5, 8,    8, 8, 6,15, 7,
     +              5, 4, 9, 9, 4,   13, 3, 7, 4, 4,
     +              4,14,19,13,19,   22, 6,17,13,13,
     +              6, 8, 9,11, 4 /
      data codemn / 'C' /
      data nmcode / 1 /

!     code   Table with keywords corresponding to options of COMPUTE_VECTOR
!             Values:
!              1:  MODULUS
!              2:  LENGTH
!              3:  VECTOR
!              4:  PHASE
!              5:  MULTIPLY
!              6:  SUBTRACT
!              7:  CONSTANT
!              8:  SCALAR
!              9:  LIN_COMBINATION
!             10:  EXTRACT
!             11:  DEGFD
!             12:  REAL
!             13:  IMAGINARY
!             14:  CONJUGATE
!             15:  FUNC
!             16:  INNER_PRODUCT
!             17:  MAP
!             18:  PROBLEM
!             19:  TYPE
!             20:  COEF1
!             21:  COEF2
!             22:  ELEMENT_GROUPS
!             23:  SKIP_ELEMENT_GROUPS
!             24:  MESH_VELOCITY
!             25:  INTEGRATED_TO_NODAL
!             26:  TRANSFORM_TO_NORMALDIR
!             27:  CURVES
!             28:  SMOOTH_END_POINTS
!             29:  STRESS_VECTOR
!             30:  ZERO_LEVELSET
!             31:  ICHELD
!             32:  SEQ_COEF
!             33:  SEQ_DERIV
!             34:  DERIVATIVES
!             35:  SIGN
! ======================================================================
!
      call eropen ( 'askvec' )

      debug = .false. .and. ioutp>=0

      if ( debug ) then

!     --- debug information

         write(irefwr,*) 'Debug information from askvec'
         call prinin ( typeitems, nitems, 'typeitems' )
         write(irefwr,1) 'ityprd, lastitem, istrin, number',
     +                    ityprd, lastitem, istrin, number
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Set parameters

      nelgrp = kmesh(5)      ! number of element groups
      ncurvs = kmesh(11)     ! number of curves

      inpsep(0) = 6          ! Use manvec

!     --- Initialize arrays

      table = -1d0
      tabmin = 0d0
      tabmax = 0d0
      itype = 2

      inpsep(1:nparm) = 0

!     --- MODULUS and LENGTH

      tabmin(1:2) = 1d0
      tabmax(1:2) = 1d0

!     --- VECTOR

      tabmin(3) = 2d0
      tabmax(3) = dble(maxvectors)
      itype(3) = 15

!     --- PHASE

      tabmin(4) = 1d0
      tabmax(4) = 1d0

!     --- MULTIPLY

      tabmin(5) = -1d20
      tabmax(5) = 1d20
      itype(5) = 1
      table(5) = 0d0

!     --- CONSTANT

      tabmin(7) = -1d20
      tabmax(7) = 1d20
      itype(7) = 1
      table(7) = -1d30

!     --- SCALAR

      tabmin(8) = 2d0
      tabmax(8) = dble(maxscalars)
      itype(8) = 15

!     --- DEGFD

      tabmin(11) = 1d0
      tabmax(11) = 1d2
      itype(11) = 0

!     --- MAP, PROBLEM and TYPE

      tabmin(18) = 1d0
      tabmin(19) = -1d2
      tabmax(18:19) = 1d3
      itype(18:19) = 0
      table(17:19) = 0d0

!     --- COEF1

      tabmin(20) = 1d0
      tabmax(20) = 2d0
      itype(20) = 34

!     --- COEF2

      tabmin(21) = -1d25
      tabmax(21) = 1d25
      itype(21) = 1

!     --- ELEMENT_GROUPS and SKIP_ELEMENT_GROUPS

      tabmin(22:23) = nelgrp
      tabmax(22:23) = nelgrp
      itype(22:23) = 11

!     --- curves

      itype(27) = 18
      table(27) = -1d0  ! minus position in codemn
      tabmin(27) = ncurvs
      tabmax(27) = ncurvs

!     --- zero_level_set

      itype(30) = 0  ! integer
      tabmax(30) = 1d0

!     --- ICHELD

      itype(31) = 0  ! integer
      tabmin(31) = 1d0
      tabmax(31) = 1d3

!     --- SEQ_COEF

      itype(32) = 0  ! integer
      tabmin(32) = 1d0
      tabmax(32) = 25d0
      table(32) = 0d0

!     --- SEQ_DERIV

      itype(33) = 0  ! integer
      tabmin(33) = 1d0
      tabmax(33) = 100d0

      iretur = -10
      ispecl = 4
      minussign = 1
      call intx02ext ( code, ncode, nncode, itype, table, codemn,
     +                 nmcode, 0, iretur, tabmin, tabmax, tabout,
     +                 nout, strout, 1, ncodmx, iseqkeyw, nseqkeyw )
      ispecl = 0
      if ( ierror/=0 ) go to 1000

      if ( debug ) then

!     --- Debug information

         call prinin ( iseqkeyw, nseqkeyw, 'iseqkeyw' )
         write(irefwr,1) 'iretur', iretur
         call prinrl ( table, nncode, 'table' )
         call prinrl ( tabmax, nncode, 'tabmax' )
         call prinrl ( tabout, 4, 'tabout' )

      end if
      if ( iretur<-10 .and. nseqkeyw==1 .and. iseqkeyw(1)<0 ) then

!     --- iretur<-10 and no keyword found, hence number found
!         This must be the situation vi = c

         call askvec01 ( iretur, inpsep(1), tabout, rinsep, istrin,
     +                   number )
         go to 900

      end if  ! ( iretur<-10 )

!     --- Check array length

      call inichkarlen ( nseqkeyw, 10, 'iseqkeyw' )
      if ( ierror/=0 ) go to 1000

!     --- First check the operation

      found = .false.

      if ( table(1)>0d0 ) then

!     --- modulus found

         inpsep(4) = 29
         found = .true.

      end if

      if ( table(2)>0d0 ) then

!     --- length found

         inpsep(4) = 33
         if ( found )
     +      call errsub ( 1903, 0, 0, 0 )
         found = .true.

      end if

      if ( table(4)>0d0 ) then

!     --- phase found

         inpsep(4) = 30
         if ( found )
     +      call errsub ( 1903, 0, 0, 0 )
         found = .true.

      end if

      if ( table(6)>0d0 ) then

!     --- subtract found

         inpsep(4) = 26
         if ( found )
     +      call errsub ( 1903, 0, 0, 0 )
         found = .true.

      end if

      if ( table(9)>0d0 ) then

!     --- linear combination found

          inpsep(4) = 27
         if ( found )
     +      call errsub ( 1903, 0, 0, 0 )
          found = .true.

      end if

      if ( table(10)>0d0 ) then

!     --- extract found

         inpsep(4) = 36
         if ( found ) call errsub ( 1903, 0, 0, 0 )
         found = .true.

      end if

      if ( table(12)>0d0 ) then

!     --- real found

         inpsep(4) = 37
         if ( found )
     +      call errsub ( 1903, 0, 0, 0 )
         found = .true.

      end if

      if ( table(13)>0d0 ) then

!     --- imaginary found

         inpsep(4) = 38
         if ( found ) call errsub ( 1903, 0, 0, 0 )
         found = .true.

      end if

      if ( table(14)>0d0 ) then

!     --- conjugate found

         inpsep(4) = 34
         if ( found )
     +      call errsub ( 1903, 0, 0, 0 )
         found = .true.

      end if

      if ( table(15)>0d0 ) then

!     --- FUNC found

         inpsep(4) = 31
         if ( found )
     +      call errsub ( 1903, 0, 0, 0 )
         found = .true.

      end if

      if ( table(16)>0d0 ) then

!     --- INNER_PRODUCT found

         inpsep(4) = 41
         if ( found )
     +      call errsub ( 1903, 0, 0, 0 )
         found = .true.

      end if

      if ( table(17)>0d0 ) then

!     --- MAP found

         inpsep(4) = 51
         if ( found )
     +      call errsub ( 1903, 0, 0, 0 )
         found = .true.
         inpsep(2) = nint(table(18)) ! problem number
         inpsep(7)  = nint(table(19)) ! type of vector

      end if

      if ( table(18)>0d0 ) inpsep(2) = nint(table(18)) ! problem number

      if ( table(24)>0d0 ) ! MESH_VELOCITY found
     +   call askvec24 ( inpsep(1), found, tabmax, 2, 101, code(24) )

      if ( table(25)>0d0 ) then

!     --- INTEGRATED_TO_NODAL found

         call askvec24 ( inpsep(1), found, tabmax, 1, 103, code(25) )
         if ( table(30)>0 ) then

!        --- table(30)>0, hence zero_level_set i found

            inpsep(7) = 1   ! zero_level set
            inpsep(8) = nint(table(30))  ! sequence number

         else

!        --- table(30)=0, use all outer boundaries

            inpsep(7) = 0   ! all outer boundaries

         end if  ! ( table(30)>0 )

      end if  ! ( table(25)>0d0 )

      if ( table(26)>0d0 ) ! TRANSFORM_TO_NORMALDIR found
     +   call askvec24 ( inpsep(1), found, tabmax, 1, 104, code(26) )
      if ( table(29)>0d0 ) ! STRESS_VECTOR found
     +   call askvec24 ( inpsep(1), found, tabmax, 1, 105, code(29) )

      if ( table(34)>0d0 ) then

!     --- DERIVATIVES found, special case

         call askvecderiv ( inpsep, istart, table, tabmax, tabout,
     +                      nprob, number )
         go to 1000

      end if   ! ( table(34)>0d0 )

      if ( table(35)>0d0 ) then

!     --- SIGN found

         call askvecsign ( inpsep(1), found )

      end if

!     --- Store vector numbers

      call askvecstvec ( inpsep(1), tabmax, table, tabout, isignvec,
     +                   number, numvec, rinsep )

!     --- Store scalar numbers

      call askvecstscal ( inpsep(1), tabmax, table, tabout,
     +                    minussign, iscal1 )

      if ( debug ) write(irefwr,*) 'found', found

      if ( .not. found ) then

!     --- None of the standard keywords is found
!         It might be possible that VECTOR is found, so that this case is
!         interpreted as LIN_COMBINATION

         call askvecnotfound ( inpsep(1), tabmax, table, nseqkeyw,
     +                         iseqkeyw, minussign )

      end if  ! ( .not. found )
      if ( ierror/=0 ) go to 1000

      if ( table(28)>0 ) inpsep(7) = 1   ! set ispec in case of smoothing

!     --- Store coefficients (only for LIN_COMBINATION)

      if ( inpsep(4)==27 )
     +   call askveclincom ( code, ncodmx, table, iscal1, iretur,
     +                       tabout, inpsep(1), rinsep, istrin,
     +                       nseqkeyw, iseqkeyw, isignvec )

      if ( inpsep(4)==26 .and. table(7)>-1d25 ) then

         rinsep(istrin) = table(7)
         rinsep(istrin+1) = 0d0
         inpsep(13) = istrin
         istrin = istrin + 2

      end if

!     --- Store degree of freedom

      idegfd = nint(table(11))
      if ( idegfd>0 ) inpsep(5) = idegfd

!     --- information about skip element groups

      if ( table(22)>0d0 .or. table(23)>0d0 )
     +   call askvec25 ( inpsep(1), nparm, table, nelgrp, tabout,
     +                   tabmax )

!     --- information about curves

      if ( table(27)>0d0 )
     +   call  askveccurvs ( inpsep(1), table, tabmax, ncurvs, nparm,
     +                       tabout )

!     --- raise istart

 900  istart = istart+21
      if ( inpsep(14)>0 ) then
         istart = istart+nelgrp      ! iskip (nelgrp elements)
      else if ( inpsep(15)>0 ) then
         istart = istart+inpsep(15)  ! ncurves curves
      end if   ! ( inpsep(14)>0 )

1000  call erclos ( 'askvec' )

      if ( debug ) then

!     --- debug information

         call prinin ( inpsep(1), nparm+nelgrp, 'inpsep' )
         if ( istrin>1 ) call prinrl ( rinsep, istrin-1, 'rinsep' )
         write(irefwr,*) 'End askvec'

      end if

      end
