      subroutine chsort2darr ( intarray, narray, ndim )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.0  date 25-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Sort a ndim-d array in increasing sequence
!     First first entry, next second entry (if first are equal) etc
! **********************************************************************
!
!                       KEYWORDS
!
!     sort
!     2d
!     array
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer narray, ndim, intarray(ndim,narray)

!     intarray      i/o   Two-dimensional array with integer values to be sorted
!                         At output the array is sorted in increasing
!                         sequence, first for the components (1,.)
!                         and in case these are equal for (2,.) etc
!     narray         i    length of the second dimension of iarray
!     ndim           i    length of the first dimension of iarray
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iarray(narray,ndim), i
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     iarray         Contains copy of intarray with the components
!                    switched
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHSORT2DARR1   Sort part of a ndim-d array in increasing sequence
!                    recursively
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'chsort2darr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from chsort2darr'
         call prinin1 ( intarray, narray, ndim, 'intarray/in' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Copy intarray in iarray

      do i = 1, ndim
         iarray(1:narray,i) = intarray(i,1:narray)
      end do  ! i = 1, ndim

!     --- sort recursively

      call chsort2darr1 ( iarray, narray, ndim, 1 )

!     --- Store ordered array back

      do i = 1, ndim
         intarray(i,1:narray) = iarray(1:narray,i)
      end do  ! i = 1, ndim

1000  call erclos ( 'chsort2darr' )
      if ( debug ) then

!     --- Debug information

         call prinin1 ( intarray, narray, ndim, 'intarray/sorted' )
         write(irefwr,*) 'End chsort2darr'

      end if  ! ( debug )

      end
