      subroutine ercompnormvec ( t1, t2, normal )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.0  date 03-01-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the normal vector with respect to a plane, given
!     two orthogonal tangential vectors in this plane (all in 3d)
! **********************************************************************
!
!                       KEYWORDS
!
!     tangential_vector
!     normal_vector
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision t1(3), t2(3), normal(3)

!     normal         o    normal vector
!     t1             i    First tangential vector
!     t2             i    Second tangential vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k
      double precision h
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     h              length of vector
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELOUTR         Compute outer product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The normal vector is defined as
!     n = t1 x t2 / ||t1 x t2||
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'ercompnormvec' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercompnormvec'

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

!     --- Compute normal direction by outer product

      call eloutr ( t1, t2, normal )
      h = 1d0/sqrt(normal(1)**2+normal(2)**2+normal(3)**2) !length of the vector
      do k = 1, 3
         normal(k) = normal(k)*h   ! normalized vector
      end do  ! k = 1, 3

1000  call erclos ( 'ercompnormvec' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ercompnormvec'

      end if

      end
