      subroutine errcrd
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  2.1  date 09-03-2013 Correction negative values of columns
!        version  2.0  date 02-01-2003 Complete revision
!        version  1.4  date 28-12-2000 Length of records extended to 240
!
!   copyright (c) 1990-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Print the last record read and underline the columns between
!   kcolum and icolum
!   This subroutine is meant to indicate the positions that are incorrect
!   during input. So it is an extra subroutine providing error messages
!   This subroutine replaces the old subroutine ercard
! **********************************************************************
!
!                       KEYWORDS
!
!     error_manipulation
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/csepchnewf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      character (len=480) text
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              General loop variable
!     text           Variable to store a text (is used for underlining)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Print first part of error message
!     Compute column numbers for the printing
!     Print *** under the computed column numbers
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'errcrd' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from errcrd'
         write(irefwr,1) 'numrecord, lastitem', numrecord, lastitem
         call prinin ( lastcolumn, numrecord, 'lastcolumn' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )

!     --- Print first part of error message

      write(irefwr,110)
110   format(//' Last record read:  ( The underlined positions could ', &
               'not be interpreted)')
      do i = 1, numrecord
         write(irefwr,120) cardimage(i)(1:lastcolumn(i))
120      format(1x,a)
      end do
      firstcol = max(1,firstcolitems(lastitem)-480*(numrecord-1))
      lastcol = max(1,lastcolitems(lastitem)-480*(numrecord-1))
      if ( debug ) &
         write(irefwr,1) 'firstcol, lastcol', firstcol, lastcol

!     --- Compute Column numbers for the printing

      text = ' '
      do i = firstcol, lastcol
         text(i:i) = '*'
      end do
      write(irefwr,130) text(1:lastcol)
130   format(1x,a//)

      call erclos ( 'errcrd' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End errcrd'

      end if  ! ( debug )

      end
