      subroutine errkeywexclus ( table, code, ncodmx, i1, i2 )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.0  date 09-11-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Check if two keywords are mutually exclusive
!
! **********************************************************************
!
!                       KEYWORDS
!
!     error_manipulation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncodmx(*), i1, i2
      double precision table(*)
      character (len=*) code(*)

!     code           i    List of  keywords
!     i1             i    Index of first keyword
!     i2             i    Index of second keyword
!     ncodmx         i    Number of characters per keyword
!     table          i    Contains the numbers read for the various keywords
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'errkeywexclus' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from errkeywexclus'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- in case one of the 2 is not used, return

      if ( table(i1)<=0d0 .or. table(i2)<=0d0 ) go to 1000

      call errkeywexcl ( code, ncodmx, i1, i2 )

1000  call erclos ( 'errkeywexclus' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End errkeywexclus'

      end if  ! ( debug )

      end
