      subroutine fillpc ( kmesh1c, kmesh2c, kmesh1, kmesh2 )
! ======================================================================
!
!        programmer    Luc Timmermans/Frans van de Vosse
!        version  2.8  date 29-05-2014 Remove time routine
!        version  2.7  date 21-03-2007 Fill kmesh2 part t
!        version  2.6  date 12-08-1999 New call to erclmn
!
!   copyright (c) 1992-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       fillpc fills kmesh2 part a and c
!
! **********************************************************************
!
!                       KEYWORDS
!
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh1c(*), kmesh2c(*), kmesh1(*), kmesh2(*)

!     kmesh1    i   standard sepran array
!     kmesh1c   i   kmesh2 part c (spectral element mesh)
!     kmesh2    i   standard sepran array
!     kmesh2c  i/o  kmesh2 part c (linear element mesh)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp, nelgrp, meshtp, nelm, irelem, nnpe, mone, nn, &
              ipk1c, ipk2c, mhelp, icount, ipa, ipg, jcount, mtwo, &
              n1, n2, n3, ielemorig, kelmt, kelmtmax, numrowkelmt, i

!     i              loop variable
!     icount         counter
!     ielemorig      Element sequence number in original mesh
!     ielgrp         element group
!     ipa            pointer to kmesh2 part a
!     ipg            pointer to kmesh2 part g
!     ipk1c          pointer in kmesh1 part c
!     ipk2c          pointer in kmesh2 part c
!     irelem         relative element number with respect to an element group
!     jcount
!     kelmt          Indicates if array KMESH part t has been filled >0, &
!                    or not 0
!                    If kelmt>0, kelmt denotes the starting address of
!                    KMESH part t in KMESH
!     kelmtmax       Last position of kmesh part t in kmesh
!     meshtp         type of the elements in an element group
!     mhelp          help parameter
!     mone           square root of nnpe
!     mtwo           mtwo = mone*mone
!     n1             loop variable
!     n2             loop variable
!     n3             loop variable
!     nelgrp         number of element groups
!     nelm           number of elements in a group
!     nn             relative node number per element
!     nnpe           number of nodes per element in an element group
!     numrowkelmt    Number of rows in kmesh part t
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     FMSHTP
!     FNELM
!     FNNPE
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1696   Number of nodes per element incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('fillpc')

!     --- Find information concerning kmesh part t

      if ( kmesh1(13)>0 ) then

!     --- Volumes found

         numrowkelmt = kmesh1(13)

      else if ( kmesh1(12)>0 ) then

!     --- Surfaces found

         numrowkelmt = kmesh1(12)

      else

!     --- Curves found

         numrowkelmt = kmesh1(11)

      end if  ! ( kmesh1(13)>0 )
      kelmt = kmesh1(36)
      kelmtmax = kelmt+2*numrowkelmt-1

!     --- fetch the number of element groups in the spectral mesh

      nelgrp = kmesh1(5)
      ipk1c = 1
      ipk2c = 1
      ipa = kmesh2(15)
      if (nelgrp>1) kmesh2(ipa) = 0
      if (nelgrp>1) ipg = kmesh2(21)

!     --- loop over the element groups

      icount = 0
      jcount = 0
      ielemorig = 0

      do ielgrp = 1, nelgrp

!     --- check if the mesh is a spectral element mesh

         call fmshtp(kmesh1, ielgrp, meshtp)

         if (ierror/=0) goto 1000

!        --- fetch the number of elements in this group

         call fnelm(kmesh1, ielgrp, nelm)

!        --- fetch the number of nodes per element in this group

         call fnnpe(kmesh1, ielgrp, nnpe)
         if (meshtp==-10001) then

            if (nelgrp>1) kmesh2(ipa+ielgrp) = &
                                kmesh2(ipa+ielgrp-1)+2
            do irelem = 1, nelm
               kmesh2c(ipk2c) = kmesh1c(ipk1c)
               kmesh2c(ipk2c+1) = kmesh1c(ipk1c+1)
               ipk1c = ipk1c + 2
               ipk2c = ipk2c + 2
               jcount = jcount + 1
               icount = icount + 1
            end do
            ielemorig = ielemorig+nelm

         else if (meshtp==1) then

            if (nelgrp>1) kmesh2(ipa+ielgrp) = &
                                kmesh2(ipa+ielgrp-1)+2
            mone = nnpe
            mhelp = mone-1
            do irelem = 1, nelm

!           --- Raise ielemorig and adapt kmesh part 2 if necessary

               ielemorig = ielemorig+1
               do i = 0, numrowkelmt-1
                  if ( kmesh1(kelmt+2*i)==ielemorig ) &
                      kmesh2(kelmt+2*i) = jcount+1
               end do  ! i = 0, numrowkelmt-1

               do nn = 1, mhelp
                  kmesh2c(ipk2c) = kmesh1c(ipk1c)
                  kmesh2c(ipk2c+1) = kmesh1c(ipk1c+1)
                  ipk1c = ipk1c + 1
                  ipk2c = ipk2c + 2
                  jcount = jcount + 1
                  icount = icount + 1
               end do
               ipk1c = ipk1c + 1

               do i = 0, numrowkelmt-1
                  if ( kmesh1(kelmt+2*i+1)==ielemorig ) &
                      kmesh2(kelmt+2*i+1) = jcount
               end do  ! i = 0, numrowkelmt-1

            end do

         else if (meshtp==5) then

            if (nelgrp>1) kmesh2(ipa+ielgrp) = &
                                kmesh2(ipa+ielgrp-1)+4
            mone = int(sqrt(dble(nnpe)))
            mhelp = mone*(mone - 1) - 1

!           --- loop over the elements

            do irelem = 1, nelm

!           --- Raise ielemorig and adapt kmesh part 2 if necessary

               ielemorig = ielemorig+1
               do i = 0, numrowkelmt-1
                  if ( kmesh1(kelmt+2*i)==ielemorig ) &
                      kmesh2(kelmt+2*i) = jcount+1
               end do  ! i = 0, numrowkelmt-1
               if ( kelmt<kelmtmax .and. &
                    kmesh1(kelmt)==ielemorig ) kmesh2(kelmt) = jcount+1

!              --- loop over the nodes

               do nn = 1, mhelp

!              --- fill kmesh 2 part c

                  if (mod(nn,mone)/=0) then

                     kmesh2c(ipk2c) = kmesh1c(ipk1c)
                     kmesh2c(ipk2c+1) = kmesh1c(ipk1c+1)
                     kmesh2c(ipk2c+2) = kmesh1c(ipk1c+mone+1)
                     kmesh2c(ipk2c+3) = kmesh1c(ipk1c+mone)
                     ipk2c = ipk2c + 4
                     ipk1c = ipk1c + 1
                     jcount = jcount + 1
                     icount = icount + 1
                  else

                     ipk1c = ipk1c + 1
                  end if
               end do

               ipk1c = ipk1c + mone + 1

               do i = 0, numrowkelmt-1
                  if ( kmesh1(kelmt+2*i+1)==ielemorig ) &
                      kmesh2(kelmt+2*i+1) = jcount
               end do  ! i = 0, numrowkelmt-1

            end do

         else if (meshtp==13 .or. meshtp==14) then

            if (nelgrp>1) kmesh2(ipa+ielgrp) = &
                                kmesh2(ipa+ielgrp-1)+8
            mone = int(nnpe**(1d0/3d0)+.5d0)
            if( mone**3/=nnpe ) then
               call errint ( nnpe, 1 )
               call errint ( mone, 2 )
               call errint ( mone**3, 3 )
               call errsub ( 1696, 3, 0, 0 )
               go to 1000
            end if

            mtwo = mone*mone

!           --- loop over the elements

            do irelem = 1, nelm

!           --- Raise ielemorig and adapt kmesh part 2 if necessary

               ielemorig = ielemorig+1
               do i = 0, numrowkelmt-1
                  if ( kmesh1(kelmt+2*i)==ielemorig ) &
                      kmesh2(kelmt+2*i) = jcount+1
               end do  ! i = 0, numrowkelmt-1

!              --- loop over the nodes

               do n3 = 1, mone-1
               do n2 = 1, mone-1
               do n1 = 1, mone-1

!              --- fill kmesh 2 part c

                     kmesh2c(ipk2c) = kmesh1c(ipk1c)
                     kmesh2c(ipk2c+1) = kmesh1c(ipk1c+1)
                     kmesh2c(ipk2c+2) = kmesh1c(ipk1c+mone+1)
                     kmesh2c(ipk2c+3) = kmesh1c(ipk1c+mone)
                     kmesh2c(ipk2c+4) = kmesh1c(ipk1c+mtwo)
                     kmesh2c(ipk2c+5) = kmesh1c(ipk1c+mtwo+1)
                     kmesh2c(ipk2c+6) = kmesh1c(ipk1c+mtwo+mone+1)
                     kmesh2c(ipk2c+7) = kmesh1c(ipk1c+mtwo+mone)
                     ipk2c = ipk2c + 8
                     jcount = jcount + 1
                     icount = icount + 1
                     ipk1c = ipk1c + 1

               end do

               ipk1c = ipk1c + 1
               end do

               ipk1c = ipk1c + mone
               end do

               ipk1c = ipk1c + mtwo

               do i = 0, numrowkelmt-1
                  if ( kmesh1(kelmt+2*i+1)==ielemorig ) &
                      kmesh2(kelmt+2*i+1) = jcount
               end do  ! i = 0, numrowkelmt-1

            end do

         end if
         if (nelgrp>1) kmesh2(ipg+ielgrp-1) = jcount
         jcount = 0
      end do

      kmesh2(9) = icount

1000  call erclos('fillpc')
      end
