      function getelgrpelem ( kmesh, ielem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-02-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find element group corresponding to element ielem
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_group
!     element
!     element_shape
!     shape
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer getelgrpelem, ielem, kmesh(*)

!     getelgrpelem   o    element group number found
!     ielem         i    element sequence number
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kelmg, inelem, ielgrp
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ielgrp         element group number
!     inelem         Number of elements in a specific element group
!     kelmg          Indicates if KMESH part g has been filled (>0)
!                    or not (0)
!                    If kelmg>0, kelmg is the starting address of KMESH
!                    part g in KMESH
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getelgrpelem' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getelgrpelem'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kelmg = kmesh(21)  !starting address kmesh part g
      inelem = 0

      if ( kelmg==0 ) then

!     --- kelmg = 0, only one element group

         getelgrpelem = 1

      else

!     --- kelmg>0, more element groups

         do ielgrp = 1, kmesh(5)

            inelem = inelem + kmesh(kelmg-1+ielgrp)
            if ( ielem<=inelem ) then
               getelgrpelem = ielgrp
               exit
            end if  ! ( ielem<=inelem )

         end do  ! ielgrp = 1, kmesh(5)

      end if  ! ( kelmg==0 )

1000  call erclos ( 'getelgrpelem' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End getelgrpelem'

      end if  ! ( debug )

      end
