      subroutine getelmsnode ( ibuffr, kmesh, inode, nelms, ielems )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 10-02-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all elements corresponding to nodal point inode
!
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     element
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), kmesh(*), inode, nelms, ielems(*)

!     ibuffr         i    Integer buffer array in which all large data is stored
!     ielems         o    Array of size nelms containg the elements containing
!                         inode
!     inode          i    Node sequence number
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     nelms          i    Number of elements found
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipkmsd
      integer npoint
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipkmsd         Starting address of array KMESH part d
!     npoint         Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETELMSNODE1   Find all elements corresponding to nodal point inode
!     INIACTMK       General memory management subroutine to activate or create
!                    a permanent array
!     INIGETTEXT     Compute pointer of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getelmsnode' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getelmsnode'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate arrays

      call iniactmk ( ibuffr, 1, 18, kmesh, kmesh, kmesh(18) )  ! kmesh d

!     --- Compute starting addresses

      ipkmsd = inigettext ( kmesh(18), 'kmeshd' )
      npoint = kmesh(8)

      call getelmsnode1 ( ibuffr(ipkmsd), inode, ielems, nelms, &
                           npoint )

1000  call erclos ( 'getelmsnode' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End getelmsnode'

      end if  ! ( debug )

      end
