      subroutine getelmsnode1 ( kmeshd, inode, ielems, nelms, npoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-02-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all elements corresponding to nodal point inode
!
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     element
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshd(*), inode, ielems(*), nelms, npoint

!     ielems         o    Array of size nelms with the elements containing
!                         inode
!     inode          i    Node sequence number
!     kmeshd         i    KMESH part d
!     nelms          i    Number of elements found
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipstart
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipstart        Starting address in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getelmsnode1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getelmsnode1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute starting position and number of elements

      ipstart = kmeshd(inode)+npoint+1
      nelms = kmeshd(inode+1)-kmeshd(inode)
      ielems(1:nelms) = kmeshd(ipstart+1:ipstart+nelms)

1000  call erclos ( 'getelmsnode1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End getelmsnode1'

      end if  ! ( debug )

      end
