      function getintvar ( varname )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-12-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Get the value of the integer in the block constants with name varname
!     If the name does not exist an error message is given
! **********************************************************************
!
!                       KEYWORDS
!
!     name
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/cusconsf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*) varname
      double precision getintvar

!     getintvar         o    Value of variable
!     varname        i    Name of variable
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iseq           Sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getnamevar

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRWAR         Print warnings
!     GETNAMEVAR     Return with sequence number of variable in scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2157   variable name not found
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find position of name in chscalars
!     If ( name in chscalars ) then
!        getintvar := value
!     else
!        error message
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getintvar' )
      debug = .false. !.and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getintvar'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseq = getnamevar ( varname )

      if ( debug ) write(irefwr,1) 'iseq', iseq

      if ( iseq>0 ) then
         getintvar = scalars(iseq)
      else
         getintvar = 0
         call errchr ( varname, 1 )
         call errwar ( 2157, 0, 0, 1 )
      end if

1000  call erclos ( 'getintvar' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'getintvar', getintvar
         write(irefwr,*) 'End getintvar'

      end if  ! ( debug )

      end
