      function getkproba ( ivec, kprob, kmesh, ielgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with starting address in kproba or kprobe for
!     vector of type ivec and element group ielgrp
!     if 0, all nodes have the same number of unknowns
! **********************************************************************
!
!                       KEYWORDS
!
!     element_group
!     problem_definition
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ivec, kprob(*), kmesh(*), ielgrp
      integer :: getkproba

!     getkproba      o    Value of specific kproba
!     ielgrp         i    element group number
!     ivec           i    Sequence number of vector of special structure
!                         0 means solution vector
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ipkpre, ikelm, kprelgrp
      logical :: debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ikelm          Last position used in kmeshc
!     ipkpre         Starting position of array kprob part e in array KPROB
!     kprelgrp       See kprob(39)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: getikelma

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETIKELMA      Return with ikelma as defined for kprobe
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getkproba' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getkproba'
         write(irefwr,1) 'ielgrp, ivec', ielgrp, ivec
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ikelm =  getikelma ( kmesh, ielgrp )
      if ( ivec==0 ) then

!     --- solution vector

         getkproba = kprob(14)+ikelm

      else

!     --- vector of special structure with sequence number ivec

         ipkpre = kprob(18)
         kprelgrp = kprob(39)
         getkproba = ipkpre+ikelm+(ivec-1)*kprelgrp

      end if  ! ( ivec==0 )

1000  call erclos ( 'getkproba' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'getkproba', getkproba
         write(irefwr,*) 'End getkproba'

      end if  ! ( debug )

      end function getkproba
