      double precision function getparm ( inpsep, rinsep, iseq, name )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 28-10-2015 Remove error
!        version  1.2  date 17-04-2014 Extra debug
!        version  1.1  date 01-05-2013 Use inpsep array
!
!   copyright (c) 2011-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with real defined by inpsep/rinsep
!     In case of a scalar, return with the scalar
! **********************************************************************
!
!                       KEYWORDS
!
!     real
!     variable
!     value
!     evaluate
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/cusconsf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), iseq
      double precision rinsep(*)
      character (len=*) name

!     getparm        o    Returned value
!     inpsep         i    Entry of integer input array that defines the value
!                         If 0, no value found: error message
!                         If>0, the value is stored in rinsep(inpsep)
!                         If<0, the value is stored in scalars(-inpsep)
!     iseq           i    Sequence number of entry in array inpsep
!     name           i    Name of variable
!     rinsep         i    Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpseploc
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     inpseploc      Local value of inpsep
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getparm' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getparm'
         write(irefwr,1) 'iseq', iseq
  1      format ( a, 1x, (10i10) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpseploc = inpsep(iseq)
      if ( debug ) write(irefwr,1) 'inpseploc', inpseploc
      if ( inpseploc==0 ) then

!     --- Error: no value found

         getparm = 0d0

      else if ( inpseploc<0 ) then

!     --- Real value in rinsep

         getparm = rinsep(-inpseploc)

      else

!     --- inpsep<0, hence variable or constant found

         inpseploc = mod(inpseploc,100000000)
         if ( debug ) write(irefwr,1) 'inpseploc/2', inpseploc

         if ( inpseploc<=maxints ) then
            getparm = incons(inpseploc)
         else
            inpseploc = inpseploc-maxints
            if ( inpseploc<=maxreals ) then
               getparm = rlcons(inpseploc)
            else
               getparm = scalars(inpseploc-maxreals)
            end if  ! ( inpseploc<=maxreals ) then
         end if  ! ( inpseploc<=maxints )

      end if  !  ( inpsep==0 )

1000  call erclos ( 'getparm' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'getparm', getparm
         write(irefwr,*) 'End getparm'

      end if  ! ( debug )

      end
