      function getshapeelem ( kmesh, ielem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-02-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with shape number of element ielem
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     shape
!     element_shape
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer getshapeelem, kmesh(*), ielem

!     getshapeelem   o    shape number found
!     ielem          i    Element number
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ielgrp         element group number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getelgrpelem, getshapeelgrp

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETELGRPELEM   Find element group corresponding to element ielem1
!     GETSHAPEELGRP  Get the shape of an element in group ielgrp
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getshapeelem' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getshapeelem'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ielgrp = getelgrpelem ( kmesh, ielem )
      getshapeelem = getshapeelgrp ( kmesh, ielgrp )

1000  call erclos ( 'getshapeelem' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End getshapeelem'

      end if  ! ( debug )

      end
