      function getstartkmeshc ( kmesh, inpelm, ishape, ielem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-02-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute starting address in kmeshc and information of element
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), inpelm, ishape, ielem, getstartkmeshc

!     getstartkmeshc o    starting address of element ielem in kmesh part c
!     ielem          i    Element number from which information is required
!     inpelm         o    number of points per element
!     ishape         o    Shape number of elements
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nelgrp, kelmf, kelma, kelmg, nelembef, inelem, ielgrp, &
              ikelmc
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ielgrp         element group number
!     ikelmc         starting address of element ielem in kmesh part c
!     inelem         Number of elements in a specific element group
!     kelma          Indicates if KMESH part a has been filled (>0)
!                    or not (0)
!                    If kelma>0, kelma is the starting address of KMESH
!                    part a in KMESH
!     kelmf          Indicates if KMESH part f has been filled (>0) or
!                    not (0)
!                    If kelmf>0, kelmf is the starting address of KMESH
!                    part f in KMESH
!     kelmg          Indicates if KMESH part g has been filled (>0)
!                    or not (0)
!                    If kelmg>0, kelmg is the starting address of KMESH
!                    part g in KMESH
!     nelembef       number of elements in all previous element groups
!     nelgrp         Number of element groups
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getstartkmeshc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getstartkmeshc'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Set constants

      nelgrp = kmesh(5)   ! number of element groups
      inpelm = kmesh(4)

!     --- Compute starting addresses

      kelmf = kmesh(20) ! starting address of kmesh part f
      if ( nelgrp==1 ) then

!     --- nelgrp = 1, only one element group

         ikelmc = (ielem-1)*inpelm+1
         ishape = kmesh(kelmf)

      else

!     --- nelgrp>1, more than one element group

         ikelmc = 0        ! starting address
         kelma = kmesh(15) ! starting address of kmesh part a
         kelmg = kmesh(21) ! starting address of kmesh part g
         nelembef = 0      ! number fo elements treated before

!        --- For all element groups do

         do ielgrp = 1, nelgrp

            inelem = kmesh(kelmg+ielgrp-1)  ! number of elements in group
            if ( kelma>0 ) &
               inpelm = kmesh(kelma+ielgrp)-kmesh(kelma+ielgrp-1)
            nelembef = nelembef+inelem
            if ( ielem<=nelembef+inelem ) then

!           --- ielem<=nelembef+inelem, hence element belongs to this group

               ikelmc = ikelmc + (ielem-1-nelembef)*inpelm + 1
               ishape = kmesh(kelmf+ielgrp-1)
               go to 1000

            else

               nelembef = nelembef+inelem
               ikelmc = ikelmc+inelem*inpelm

            end if  ! ( ielem<=nelembef )

         end do  ! ielgrp = 1, nelgrp

      end if  ! ( nelgrp==1 )

      getstartkmeshc = ikelmc

1000  call erclos ( 'getstartkmeshc' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End getstartkmeshc'

      end if

      end
