/*
 *  Copyright (c) 1999 TU Delft
 * 
 *  Author    : Kees Lemmens
 *
 *  Date      : June, 1999
 *
 *  Function  : Translates plotting primitives into Java binary portable
 *              format.
 * 
 * Variable name : start |  cmd |   x   |   y   |   a   |   h   | nchar | end 
 * Java type     :   -   | byte | float | float | float | float | byte  |  -  
 * Java length   :   0B  |  1B  |  4B   |   4B  |  4B   |  4B   |  1B   | 0B  
 * Fortran type  :  int  |  int | float | float | float | float | int   | int 
 * Fortran length:  4B   |  4B  |  4B   |   4B  |  4B   |  4B   |  4B   | 4B  
 * 
 * Total length Fortran binary record format : 32 bytes
 * Total length Java  portable binary format : 18 bytes
 * 
 * Note1 : The Java binary format ALWAYS uses big-endian, no matter on what
 * architecture the Java Virtual Machine is being run. This even applies for 
 * Java programs converted to Native executable format with a native compiler.
 *
 * Note2 : The Java portable binary format as used in my Java sepviewer
 * Jsepview always expects the first 9 bytes of the file to contain the
 * magic word "jbinsep" in UTF format to make it easy to distinguish Java
 * binary files from Fortran binary files.
 * 
 *  Modified  :
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/file.h>
#include <unistd.h>
#include "plotfun.h"
#include "readfun.h"

/* Global variables : unavoidable due to the constraints to the write
 * 		      routines : they should be callable from Fortran
 *                    as well.
 */
		     
static seprec_t *rec;
static int ios;

void jb_header()
{
   jplwr01(&rec->command,&rec->x,&rec->y,&rec->a,&rec->h,&rec->nchar,&ios);
}

void jb_trailer()
{
   jplwr01(&rec->command,&rec->x,&rec->y,&rec->a,&rec->h,&rec->nchar,&ios);
   jplwrcl(&ios);
   
   infoMessage("");
}

void jb_finish()
{
   jb_trailer();
}

int  jb_init(seprec_t *nrec,filnam_t *fileinfo)
{
   int  fillen, number = 0;
   char string[160], jbfile[80];
   
   rec = nrec; // remember it for the other plot functions 
   
   if(strlen(fileinfo->outname) == 0)
   {
      do
	sprintf(jbfile, "j%s%03d_%02d", fileinfo->name, 
		fileinfo->version, ++number);
      while (access(jbfile, R_OK) == F_OK && number < 100);
   }
   else
     strncpy(jbfile,fileinfo->outname,79);
   
   fillen = strlen(jbfile);

   jplwrop(jbfile,&fillen,&ios);
   
   if (ios)
   {
      sprintf(string, "Unable to open Java binary file '%s'", jbfile);
      errorMessage(string);
      return(FALSE);
   }
   sprintf(string, "Creating Java binary file: %s...", jbfile);
   infoMessage(string);
   
   jb_header();
   
   return(TRUE);
}

void jb_move()
{
   jplwr01(&rec->command,&rec->x,&rec->y,&rec->a,&rec->h,&rec->nchar,&ios);
}

void jb_line()
{
   jplwr01(&rec->command,&rec->x,&rec->y,&rec->a,&rec->h,&rec->nchar,&ios);
}

void jb_polyline(double x[], double y[],int n)
{
   int p;
   float *polygon;
   
   polygon = (float *)malloc(2*n*sizeof(float));
   
   for(p=0;p<n;p++) /* copy all coordinates into one array */
   {
      polygon[2*p  ] = (float) x[p];
      polygon[2*p+1] = (float) y[p];
   }
     
   jplwr01(&rec->command,&rec->x,&rec->y,&rec->a,&rec->h,&rec->nchar,&ios);
   jplwr07(&n,polygon,&ios);
   free(polygon);
}

void jb_polyfill(double x[], double y[],int n)
{
   int p;
   float *polygon;

   polygon = (float *)malloc(2*n*sizeof(float));
   
   for(p=0;p<n;p++) /* copy all coordinates into one array */
   {
      polygon[2*p  ] = (float) x[p];
      polygon[2*p+1] = (float) y[p];
   }
     
   jplwr01(&rec->command,&rec->x,&rec->y,&rec->a,&rec->h,&rec->nchar,&ios);
   jplwr07(&n,polygon,&ios);

   free(polygon);
}
   
void jb_text(char * string)
{
   int len = strlen(string);
   
   jplwr01(&rec->command,&rec->x,&rec->y,&rec->a,&rec->h,&rec->nchar,&ios);
   jplwr06(&len,string,&ios);
}

void jb_symbol()
{
   jplwr01(&rec->command,&rec->x,&rec->y,&rec->a,&rec->h,&rec->nchar,&ios);
}

