/*
 *   Copyright (c) 1999 Kees Lemmens, TU Delft
 * 
 * 
 * Date     : June , 1999
 * 
 * Modified : 
 * 
 * Function : Commandline oriented program that converts sepplots to
 *            Postscript, Fig or HPGL format.
 *            Loosely based on existing code (sepview) by G.F.M. Haenen
 *            and F.W.G.M. Wouters at Philips Research Laboratories.
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/fcntl.h>
#include <sys/file.h>
#include "plotfun.h"
#include "readfun.h"

#define VERSION         "0.4"

/* Local function declarations */

int  parseArgs(int argc,char *argv[],filnam_t *fileinfo);
void errorMessage(char *);
void infoMessage(char *);
Boolean sepscan(window_t *, int);
void sepplot(filnam_t *, int);
void setInputFile(char *,filnam_t *);
void setOutputFile(char *,filnam_t *);
void zoom_status(int s);
void color_status(int s);

/* global variables */

// None !!!!!!!!!!!!!!!!

void usage(char *name)
{
   fprintf(stderr, "version: %s (June 12, 1999; K. Lemmens)\n", VERSION);
   fprintf(stderr, "usage:   %s -h\n", name);
   fprintf(stderr, "or   :   %s [-f <fmt> ] [<sepplot file> [<outputfile>]]\n",name);
   fprintf(stderr, "          fmt = 1 : Currently unused (used to be X)\n");
   fprintf(stderr, "          fmt = 2 : Postscript formatted\n");
   fprintf(stderr, "          fmt = 3 : Hpgl plotter language\n");
   fprintf(stderr, "          fmt = 4 : Fig format for xfig 3.x\n");
   fprintf(stderr, "          fmt = 5 : Java portable binary format\n");
   exit(0);
}

// This parses the commandline options 

int parseArgs(int argc, char *argv[],filnam_t *fileinfo)
{
   int c;
   int type = PS_OUTPUT; // default 
   extern char * optarg;
   extern int optind, optopt;
   
   strcpy(fileinfo->basedir,  "");
   strcpy(fileinfo->name,     "");
   fileinfo->outname[0] = '\0';  // generate a unique filename
   fileinfo->version = 0;
   fileinfo->new     = TRUE;
   
   while ((c=getopt(argc,argv,"hf:"))!= -1)
   {
      switch(c)
      {
       case 'f':
	 type = atoi(optarg);
	 break;
       case '?':
       case 'h':
	 usage(argv[0]);
      }
   }
   if (optind < argc)
     setInputFile(argv[optind++],fileinfo);
   if (optind < argc)
     setOutputFile(argv[optind++],fileinfo);
   return type;
}

// Some simple (or dummy) helpers to avoid extra work :

void errorMessage(char *message)
{
   fprintf(stderr,"Error : %s\n",message);
}

void infoMessage(char *message)
{
   fprintf(stderr,"%s\n",message);
}

Boolean sepscan( window_t *d , int type)
{
   struct seprec_t rec;
   int    i;
   int    ios;			/* Fortran I/O status field */
   float  polyg[2*MAXPOLY];
   char   string[256];

   d->xmin = 1E10;    d->xmax = -d->xmin;
   d->ymin = d->xmin; d->ymax =  d->xmax;

   // fprintf(stderr,"Scanning the input ...\n");

   ios = readrecord(&rec,type);
   
   while (ios == 0 && rec.command != 5)
   {
      switch (rec.command)
      {
       case 2: // PLOT
       case 3: // MOVE
	 if (rec.x < d->xmin) { d->xmin = rec.x; };
	 if (rec.x > d->xmax) { d->xmax = rec.x; };
	 if (rec.y < d->ymin) { d->ymin = rec.y; };
	 if (rec.y > d->ymax) { d->ymax = rec.y; };
	 break;
       case 6: // TEXT
	 rec.nchar %= 100;                /* strip dcolor out of value */
	 ios = readstring(rec.nchar,string,type);
	 if (ios != 0) { return FALSE; };
	 if (rec.x < d->xmin) { d->xmin = rec.x; };
	 if (rec.x + rec.nchar*rec.h > d->xmax) { d->xmax = rec.x + rec.nchar*rec.h; };
	 if (rec.y < d->ymin) { d->ymin = rec.y; };
	 if (rec.y > d->ymax) { d->ymax = rec.y; };
	 break;
       case 7: // POLYGON
	 ios = readpolygon(rec.nchar,polyg,type);
	 if (ios != 0) { return FALSE; };
	 for (i = 0; i < rec.nchar; i++) 
	 {
	    if (polyg[2*i] < d->xmin) { d->xmin = polyg[2*i]; };
	    if (polyg[2*i] > d->xmax) { d->xmax = polyg[2*i]; };
	    if (polyg[2*i+1] < d->ymin) { d->ymin = polyg[2*i+1]; };
	    if (polyg[2*i+1] > d->ymax) { d->ymax = polyg[2*i+1]; };
	 }
	 break;
       case 8: // SYMBOL
	 if (rec.x < d->xmin) { d->xmin = rec.x; };
	 if (rec.x > d->xmax) { d->xmax = rec.x; };
	 if (rec.y < d->ymin) { d->ymin = rec.y; };
	 if (rec.y > d->ymax) { d->ymax = rec.y; };
	 break;
       case 9: // ?????????
	 break;
      }
      ios = readrecord(&rec,type);
   }
   if (rec.command == 5) 
     return TRUE; /* last record was plot close */
   else              
     return FALSE; /* must be some wrong */
}

void sepplot(filnam_t *fileinfo, int outtype)
{
   struct seprec_t rec;
   int    i, col, intype, ios;
   float  polyg[2*MAXPOLY];  /* REAL's from Fortran files */
   double xcoord, ycoord, angle, height;
   double xstart = 0, ystart = 0;
   double polyx[MAXPOLY], polyy[MAXPOLY];
   char   string[256], fileid[MAXFILENAME];
   struct window_t usr;
   
   sprintf(fileid, "%s%s%03d", fileinfo->basedir, fileinfo->name, 
	   fileinfo->version);
   
   ios = openinput(fileid, fileinfo); /* also sets the input type */
   intype = fileinfo->intype;
   
   /* First scan for the dimensions of the plot */
   if ( sepscan( &usr,intype) == FALSE)
   {
      fprintf(stderr,"error scanning the file\n");
      return;
   }
   
   closeinput(intype);
   ios = openinput(fileid, fileinfo); /* rewind */
   
   ios = readrecord(&rec,intype);
   
   // fprintf(stderr,"Plotting the output ...\n");
   
   while (ios == 0)
   {
      switch (rec.command)
      {
       case 1: // START
	 // x-ymin and x-ymax already set in sepscan !
	 usr.width  = usr.xmax - usr.xmin;  
	 usr.height = usr.ymax - usr.ymin;  

	 if (!init(&usr,&rec,fileinfo,outtype))
	   goto error;
	 color(-1);
	 break;
       case 2: // PLOT
	 color(rec.nchar);
	 xcoord = rec.x;
	 ycoord = rec.y;
	 line(xstart, ystart, xcoord, ycoord);
	 xstart = xcoord;
	 ystart = ycoord;
	 break;
       case 3: // MOVE
	 move(rec.x,rec.y);
	 xstart = rec.x;
	 ystart = rec.y;
	 break;
       case 5: // CLOSE
	  pl_finish();
	 break;
       case 6: // TEXT
	 xcoord = rec.x;
	 ycoord = rec.y;
	 angle  = rec.a;
	 height = rec.h;
	 col    = rec.nchar/100;
	 rec.nchar %= 100;
	 ios = readstring(rec.nchar,string,intype);

	 *(string + rec.nchar) = '\0';
	 color(col);
	 text(xcoord, ycoord, height, angle, string);
	 break;
       case 7: // POLYGON
	 col = (int) rec.x;
	 ios = readpolygon(rec.nchar, polyg,intype);
	 for (i = 0; i < rec.nchar; i++)
	 {
	    polyx[i] = polyg[2*i];
	    polyy[i] = polyg[2*i+1];
	 }
	 color(col);
	 polyfill(polyx, polyy, rec.nchar);
	 break;
       case 8: // SYMBOL
	 xcoord = rec.x;
	 ycoord = rec.y;
	 color(rec.nchar / 100);
	 symbol(xcoord, ycoord, rec.nchar % 100);
	 break;
       case 9: // ?????????
	 ios = readpolygon(rec.nchar, polyg,intype);
	 for (i = 0; i < rec.nchar; i++)
	 {
	    polyx[i] = polyg[2*i];
	    polyy[i] = polyg[2*i+1];
	 }
	 color((int) rec.x);
	 polyline(polyx, polyy, rec.nchar);
	 break;
       default:
	 fprintf(stderr, "Unknown command: %d\n", rec.command);
	 break;
      }
      ios = readrecord(&rec,intype);
   }
   error:
   closeinput(intype);
}

void setOutputFile(char *outname,filnam_t *fileinfo)
{
   if(outname != NULL)
     strcpy(fileinfo->outname, outname);
}

void setInputFile(char *name,filnam_t *fileinfo)
{
   char *sp;
   char s1[MAXCOMMAND];
   
   sp = strrchr(name, '/'); // Argument contains a path name ?

   if (sp != NULL)
   {
      strcpy(s1, sp + 1);
      *(sp+1) = '\0';    // truncate filename and keep pathname
      strcpy(fileinfo->basedir, name);
   }
   else
   {
      strcpy(s1, name);
      strcpy(fileinfo->basedir, "");
   }
   
   fileinfo->version = 0; // number counter
   
   if (sscanf(s1, "%[^.].plot%3d", fileinfo->name, &fileinfo->version) == 2)
     strcat(fileinfo->name, ".plot");
   else if (sscanf(s1, "%[^.].%3d", fileinfo->name, &fileinfo->version) == 2)
     strcat(fileinfo->name, ".");
   else {
      sprintf(fileinfo->name, "%s.plot", s1);
      fileinfo->version = 1;
   }
}

/* main */


int main(int argc, char *argv[])
{
   static struct filnam_t fileinfo;
   int  istart, irotat, ioutp, itime;
   int type;

   /* Next 2 calls are useless for the program if using the Fortran IO
    * emulation, but necessary if we want to link with the Sepran routines
    * as they won't work without calling these.
    */

   istart = 110100;
   irotat = 1;
   ioutp = -1;
   itime = 0;
   
   initmd ();                                 // Sepran
   start  (&istart, &irotat, &ioutp, &itime); // Sepran
   
   type = parseArgs(argc, argv,&fileinfo);

   sepplot(&fileinfo, type);

   exit(0);
}
