      subroutine ini015 ( ibuffr, buffer, isubr, iseqnr, mmiinput, &
                          mmirnput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 17-08-2013 Replaced by ini015ext
!        version  3.7  date 21-11-2012 Long integers
!        version  3.6  date 28-05-2011 Extra debug
!
!   copyright (c) 1993-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Extract the infor entries of the arrays IINPUT and RINPUT for
!   subroutine ISUBR and sequence number ISEQNR
!   The corresponding arrays are also activated
!   Warning: This subroutine uses memory managment subroutines
!            Therefore pointers computed before must be recomputed
!   Remark:  The corresponding information is filled in ISEPIN by INI009
! **********************************************************************
!
!                       KEYWORDS
!
!     extract
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isubr, iseqnr, mmiinput, mmirnput, ibuffr(*)
      double precision buffer(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     iseqnr         i    Sequence number with respect to the storage in array
!                         ISEPIN
!     isubr          i    Indicates the type of calling subroutine
!                         Possible values
!                          1:  START
!                          2:  PROBDF
!                          3:  COMMAT
!                          4:  CREATE
!                          5:  PRESDF
!                          6:  FILCOF
!                          7:  CHNCOF
!                          8:  SOLVE
!                          9:  OUTPUT
!                         10:  SEPCOMP
!                         11:  DERIV
!                         12:  NLNPRB
!                         13:  INTEGR
!                         14:  INTBND
!                         15:  TIMPRB
!                         16:  ADAPBOUN
!                         17:  ADAPMESH
!                         18:  STATFREE
!                         19:  INSTFREE
!                         20:  CONTACT
!                         21:  LOOP
!                         22:  CHANGE_COORDINATES
!                         23:  EIGENVAL
!                         24:  CAPACITY
!                         25:  INVERSE_PROBLEM
!                         26:  ENTHALPY_INTEGRATION
!                         27:  New problem
!     mmiinput       o    Memory management sequence number with respect to
!                         iinput
!     mmirnput       o    Memory management sequence number with respect to
!                         rinput
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxcal

!     maxcal         Maximum value of icall
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI015EXT      Extract infor entries of integer and real input arrays from
!                    ISEPIN
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ini015' )
      call ini015ext ( ibuffr, buffer, isubr, iseqnr, mmiinput, &
                       mmirnput, maxcal )
      call erclos ( 'ini015' )
      end
