      subroutine ini015ext ( ibuffr, buffer, isubr, iseqnr, mmiinput, &
                             mmirnput, maxcal )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 28-10-2015 Extra debug
!        version  1.1  date 24-12-2014 New definition clinsl
!        version  1.0  date 17-08-2013 Extension of ini015
!
!   copyright (c) 2013-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Extract the infor entries of the arrays IINPUT and RINPUT for
!   subroutine ISUBR and sequence number ISEQNR
!   The corresponding arrays are also activated
!   Warning: This subroutine uses memory managment subroutines
!            Therefore pointers computed before must be recomputed
!   Remark:  The corresponding information is filled in ISEPIN by INI009
! **********************************************************************
!
!                       KEYWORDS
!
!     extract
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/clinslf90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/cusconsf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isubr, iseqnr, mmiinput, mmirnput, ibuffr(*), maxcal
      double precision buffer(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     iseqnr         i    Sequence number with respect to the storage in array
!                         ISEPIN
!     isubr          i    Indicates the type of calling subroutine
!                         Possible values
!                          1:  START
!                          2:  PROBDF
!                          3:  COMMAT
!                          4:  CREATE
!                          5:  PRESDF
!                          6:  FILCOF
!                          7:  CHNCOF
!                          8:  SOLVE
!                          9:  OUTPUT
!                         10:  SEPCOMP
!                         11:  DERIV
!                         12:  NLNPRB
!                         13:  INTEGR
!                         14:  INTBND
!                         15:  TIMPRB
!                         16:  ADAPBOUN
!                         17:  ADAPMESH
!                         18:  STATFREE
!                         19:  INSTFREE
!                         20:  CONTACT
!                         21:  LOOP
!                         22:  CHANGE_COORDINATES
!                         23:  EIGENVAL
!                         24:  CAPACITY
!                         25:  INVERSE_PROBLEM
!                         26:  ENTHALPY_INTEGRATION
!                         27:  New problem
!     maxcal         o    Maximum number of items read
!     mmiinput       o    Memory management sequence number with respect to
!                         iinput
!     mmirnput       o    Memory management sequence number with respect to
!                         rinput
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipiinputvar, iprinput, ipblc2, ipblc3
      integer icall, istart, lenrinp, mmiinputvar, iseqnrcp
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     icall          Call sequence number for present subroutine
!     ipblc2         Starting address of Block 2 of ISEPIN in IBUFFR
!     ipblc3         Starting address of Block 3 of ISEPIN in IBUFFR
!     ipiinputvar    Starting address of array iinputvar in IBUFFR
!     iprinput       Starting address of array rinput in array BUFFER
!     iseqnrcp       Copy of iseqnr
!                    In case maxcal=1, iseqnrcp is set to 1
!     istart         First position used at a part
!     lenrinp        Length of array rinput
!     mmiinputvar    Memory management sequence number of iinputvar
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext, inidgttext
      integer inilen

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI0151        Replace values in rinput corresponding to variables by
!                    actual value
!     INI0152        Compute information concerning input read by readal
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     INILEN         Compute length of array in IBUFFR
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1335   SEPSTN has not been called with iinstr(2) = 1
!    1883   Sequence number more than maximum
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Activate isepin
!   Compute starting addresses of block2 and block 3 in IBUFFR
!   Compute ICALL, MAXCAL and ISTART
!   If ( ICALL>MAXCAL ) then
!      give warning
!      ICALL := MAXCAL
!   Fill IINPUT and IRNPUT
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ini015ext' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ini015ext'
         write(irefwr,1) 'isubr, iseqnr', isubr, iseqnr
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      mmiinput = 0
      mmirnput = 0
      if ( isepin<=0 ) then

!     --- isepin<=0, input has not been read, i.e. sepstn has not been
!                      called with iinstr(2) = 1

         call errsub ( 1335, 0, 0, 0 )
         go to 1000

      end if

!     --- Compute certain start addresses

      call ini0152 ( ibuffr, isepin, icall, maxcal, istart, &
                     ipblc2, ipblc3, isubr )
      if ( debug ) &
         write(irefwr,1) 'icall, maxcal, istart', icall, maxcal, istart

!     --- Set iseqnrcp equal to iseqnr

      if ( maxcal==1 ) then

!     --- maxcal = 1, so maximal 1 input found
!         in this case we set iseqnrcp equal to 1

         iseqnrcp = 1

      else

!     --- Standard case

         iseqnrcp = iseqnr

      end if  ! ( maxcal==1 )

      if ( maxcal>0 .and. iseqnrcp>0 ) then
         if ( iseqnr>100000000 ) then
            icall = iseqnr-100000000
            icall = nint(scalars(icall))
         else
            icall = iseqnrcp
         end if  ! ( iseqnr>100000000 )
         if ( icall>maxcal .and. (isubr/=3 .or. iseqnrcp/=1) ) then

            select case ( isubr )
               case(1)
                  call errchr ( 'START', 1 )
               case(2,27)
                  call errchr ( 'PROBDF (problem definition)', 1 )
               case(3)
                  call errchr ( 'COMMAT (structure of matrix)', 1 )
               case(4)
                  call errchr ( 'CREATE (create vector)', 1 )
               case(5)
                  call errchr ( 'PRESDF(essential boundary conditions)', &
                                1 )
               case(6)
                  call errchr ( 'FILCOF (coefficients)', 1 )
               case(7)
                  call errchr ( 'CHNCOF (change coefficients)', 1 )
               case(8)
                  call errchr ( 'SOLVE (solve linear system)', 1 )
               case(9)
                  call errchr ( 'OUTPUT', 1 )
               case(10)
                  call errchr ( 'SEPCOMP', 1 )
               case(11)
                  call errchr ( 'DERIVS (derivatives)', 1 )
               case(12)
                  call errchr ( 'NLNPRB (non-linear problem)', 1 )
               case(13)
                  call errchr ( 'INTEGR (integral)', 1 )
               case(14)
                  call errchr ( 'INTBND (boundary integral)', 1 )
               case(15)
                  call errchr ( 'TIMPRB (time integration)', 1 )
               case(16)
                  call errchr ( 'ADAPBOUN (adapt boundary)', 1 )
               case(17)
                  call errchr ( 'ADAPMESH (adapt mesh)', 1 )
               case(18)
                  call errchr ( 'STATFREE (stationary free surface)', &
                                1 )
               case(19)
                  call errchr ( 'INSTFREE (instationary free surface)', &
                        1 )
               case(20)
                  call errchr ( 'CONTACT', 1 )
               case(23)
                  call errchr ( 'EIGENVAL (eigenvalues)', 1 )
               case(24)
                  call errchr ( 'CAPACITY', 1 )
               case(25)
                  call errchr ( 'INVERSE_PROBLEM', 1 )
               case(26)
                  call errchr ( 'ENTHALPY_INTEGRATION', 1 )
            end select  ! case ( isubr )

            call errint ( icall, 1 )
            call errint ( maxcal, 2 )
            call errsub ( 1883, 2, 0, 1 )
            icall = maxcal

         end if

      end if
      if ( debug ) write(irefwr,1) 'maxcal', maxcal
      if ( ierror/=0 ) go to 1000

      if ( maxcal>0) then
         mmiinput = ibuffr(ipblc3+(istart+icall-2)*3)
         mmirnput = ibuffr(ipblc3+(istart+icall-2)*3+1)
         mmiinputvar = ibuffr(ipblc3+(istart+icall-2)*3+2)
         if ( debug ) &
            write(irefwr,1) 'mmiinput, mmirnput, mmiinputvar', &
                             mmiinput, mmirnput, mmiinputvar
         if ( mmiinput>0 ) &
            call ini060text ( ibuffr, -mmiinput, 'iinput' )
         ibuffr(ipblc2+4*(isubr-1)+1) = min ( maxcal, icall+1 )
         if ( mmirnput>0 ) &
            call ini060text ( ibuffr, mmirnput, 'irnput' )
         if ( mmiinputvar>0 ) then

!        --- mmiinputvar>0, hence variables in rinput must be replaced by
!            actual values

            call ini060text ( ibuffr, mmiinputvar, 'iinputvar' )
            iprinput = inidgttext ( mmirnput, 'irnput' )
            ipiinputvar = inigettext ( mmiinputvar, 'iinputvar' )
            lenrinp = inilen(mmiinputvar)

            if ( debug ) write(irefwr,1) 'lenrinp', lenrinp
            call ini0151 ( buffer(iprinput), ibuffr(ipiinputvar), &
                           lenrinp )
            if ( debug ) &
               call prinrl ( buffer(iprinput), lenrinp, 'rinput' )

         end if  ! ( mmiinputvar>0 )

      end if  ! ( maxcal>0)

1000  call erclos ( 'ini015ext' )

      if ( debug) then

!     --- Debug information

         write(irefwr,*) 'End ini015ext'

      end if  ! ( debug)

      end
