      subroutine ini049 ( ibuffr, length, ipstar )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.4  date 21-11-2012 Long integers
!        version  2.3  date 15-04-2005 Extra information in error message
!        version  2.2  date 22-11-2002 Test on swap
!
!   copyright (c) 1988-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  SEPRAN subroutine to create space in array ibuffr by writing low priority
!  arrays to backing storage or by compressing
!       Steps to be carried out
!       -  Check if there is enough contiguous space in IBUFFR
!       -  if not create space by writing low priority arrays to file
!       -  if this is not sufficient by compressing
!          array ibuffr after writing of all arrays with priority
!          smaller than 10 to backing storage
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons3f90'
      include 'SPcommon/cbuffrf90'
      include 'SPcommon/cdebugf90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cmcdpif90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer (kind=8) ipstar
      integer length, ibuffr(*)

!    ibuffr   i/o  Integer array in which all large data is stored
!    ipstar    o   starting position if there is space free
!    length    i   free space needed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iextra, iwork(numarrs)

!    iextra   Extra space neede for contrl integers at each side of the array
!    iwork    Integer work array to be used for sorting of free space
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRINTLONG     Put long integer in error message
!     ERRSUB         Error messages
!     INI003         Clear array IBUFFR by writing all low priority arrays to
!                    file 3 and compressing array IBUFFR
!     INI101         Try to create free space in IBUFFR by writing sufficient
!                    low priority arrays to file 3
!     PRINBF         Print and check length of array IBUFFR
!     PRINBUFARS     Print the active arrays present in array buffer
! **********************************************************************
!
!                       I/O
!
!     If necessary arrays are written to and/or read from file 3 (iref3)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2375   Swapping must be carried out, but noswap activated
!    3116   Memory needed exceeds largest integer
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Try to make space by writing as much low priority arrays to file 3 as
!     necessary to get length free positions in IBUFFR
!     If ( not sufficient space available ) then
!        Create space by writing all arrays with priority<10 to file 3 and
!        compressing IBUFFR
!        Check length of IBUFFR
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- try to create free space by writing low priority arrays
!         to backing storage  (file 3)

      call eropen ( 'ini049' )
      if ( ierror/=0 ) go to 1000

      call ini101 ( ibuffr, length, ipstar, iwork, nbuffr )
      if ( ierror/=0 ) go to 1000

      if ( ipstar==0 ) then

!     --- ipstar = 0  write all parts in
!         ibuffr with priority smaller than 10 to file 3
!         compress array ibuffr
!         Remark:  In this case all pointers must be recomputed!!!

         if ( noswap==1 ) then

!        --- Swapping is not permitted

            call errintlong ( ibfree, 1 )
            call errint ( length, 2 )
            call errintlong ( nbuffr, 3 )
            call errintlong ( ibfree+length-nbuffr, 4 )
            call errsub ( 2375, 4, 0, 0 )
            call prinbufars
            write(irefwr,200) length
200         format ( 'length required = ', i10 )
            go to 1000

         end if
         call ini003 ( ibuffr )
         if ( ierror/=0 ) go to 1000

         if ( intlen==2 .and. mod(ibfree,2_8)==0 ) ibfree = ibfree+1
         if ( idebug>0 ) then
            iextra = intlen
         else
            iextra = 0
         end if

         if ( ibfree+length+iextra*2-1<0 ) then

!        --- Error not enough space in ibuffr and extension is not
!            possible

            call errintlong ( ibfree, 1 )
            call errint ( length, 2 )
            call errsub ( 3116, 2, 0, 0 )
            go to 1000

         end if  ! ( ibfree+length+iextra*2-1<0 )

         call prinbf ( ibfree+length+iextra*2-1 )
         ipstar = ibfree
         ibfree = ibfree+length+iextra*2-1
         if ( intlen==2 .and. mod(ibfree,2_8)==0 ) ibfree = ibfree+1

      end if

1000  call erclos ( 'ini049' )

      end
