      subroutine ini058 ( ibuffr, ipstar, maxlen )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 21-11-2012 Long integers
!        version  2.0  date 16-04-1999 Extra parameter ibuffr
!        version  1.4  date 26-02-1999 Use parameter numarrs
!
!   copyright (c) 1989-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check how much contiguous space is available in array IBUFFR
!     from position IPSTAR
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons3f90'
      include 'SPcommon/cbuffrf90'
      include 'SPcommon/cdebugf90'
      include 'SPcommon/carrayf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer (kind=8) ipstar, maxlen
      integer ibuffr(*)

!     ibuffr    i/o   Integer buffer array in which all large data is stored
!     ipstar     i    Position in array IBUFFR from which the available space
!                     must be found
!     maxlen     o    Number of words available in array IBUFFR from position
!                     IPSTAR
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) last
      integer i, iextra, ifirsv

!     i         Loop variable
!     iextra    Extra space needed at both sides of the array for control
!               integers
!     ifirsv    Help variable to save the value of ifirsw temporarily
!     last      Last free position in IBUFFR immediately after IPSTAR
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     INICHK1    Check IBUFFR
! **********************************************************************
!
!                       I/O
!
!     none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!     If ( idebug>0 ) then
!        Set iextra = extra space needed
!        check all control integers
!     if ( ibfree=ipstar ) then
!        maxlen = nbuffr-ipstar+1
!     else
!        Find smallest starting address in IBUFFR that may not be used
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( idebug==0 ) then
         iextra = 0
      else
         iextra = intlen
         ifirsv = ifirws
         ifirws = 1
         call inichk1 ( ibuffr )
         ifirws = ifirsv
      end if

      if ( ibfree==ipstar ) then

!     --- ipstar is ibfree, hence all space after ipstar is available

         maxlen = nbuffr - ipstar + 1 - iextra

      else

!     --- Find smallest starting address in IBUFFR that may not be used

         last = nbuffr + 1
         do i = 1, iinfor
            if ( infor(i)>ipstar ) last = min ( last,infor(i) )
         end do
         maxlen = last - ipstar - 2* iextra

      end if

      if ( idebug>0 ) then
         nbufw3 = ipstar + maxlen
         ibuffr(nbufw3) = numarrs+1
      end if
      end
