      subroutine ini080extvec ( ibuffr, buffer, isol, length, ivec, &
                                iprob, name )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 21-11-2012 Long integers
!        version  1.1  date 28-09-2012 Jump in case ierror#0
!        version  1.0  date 29-03-2006
!
!   copyright (c) 2006-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create new sepran double precision array of type isol
!     Clear this array
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     clear
!     solution_vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), isol(5), length, ivec, iprob
      double precision buffer(*)
      character (len=*) name

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     iprob          i    Actual problem number
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution array
!     ivec           i    Sequence number of vector of special structure
!                         if 0, the vector is of type solution vector
!     length         i    Help parameter to indicate the length of arrays
!                         In this case it contains the length of the vector in
!                         doubles
!     name           i    Name of array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipusol
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipusol         Starting address of solution array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidgttext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI006         clear real space
!     INI080         Create permanent space in array BUFFER
!     INIDGTTEXT     Compute pointer of array in BUFFER
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ini080extvec' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ini080extvec'

      end if
      if ( ierror/=0 ) go to 1000

      call ini080 ( ibuffr, isol(1), length, name )
      if ( ierror/=0 ) go to 1000

      if ( ivec==0 ) then
         isol(2) = 110
      else
         isol(2) = 115
      end if  ! ( ivec==0 )
      isol(3) = 0
      isol(4) = ivec+1000*(iprob-1)
      isol(5) = length

!     --- Clear this array

      ipusol = inidgttext ( isol(1), name )
      call ini006 ( buffer(ipusol), length )

1000  call erclos ( 'ini080extvec' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ini080extvec'

      end if

      end
