      subroutine ini100 ( ibuffr, index )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 21-11-2012 Long integers
!        version  2.2  date 19-06-2002 Adapt write
!        version  2.1  date 06-02-2002 Extension with eropen/erclos
!
!   copyright (c) 1986-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   write array with reference number index from ibuffr to file 3
!   change contents of infor, inback and lenbac
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     backing_storage
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons3f90'
      include 'SPcommon/carrayf90'
      include 'SPcommon/cbuffrf90'
      include 'SPcommon/cfile3f90'
      include 'SPcommon/carbacf90'
      include 'SPcommon/cdebugf90'
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cnamtbf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index, ibuffr(*)

!     ibuffr i/o  Integer array in which all large data is stored
!     index   i   Sequence number of array to be written with respect to infor
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipstar
      integer iextra, length, irecnr, irecn, numrec

!    iextra    Extra space needed at both sides of the array for control
!              integers
!    ipstar    Starting address of array in IBUFFR
!    irecn     Local record number
!    irecnr    Record number in direct access file
!    length    Computed length of an array
!    numrec    number of records to be written
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     WRITFL         Write to SEPRAN direct access file
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!      ipstar := infor(index)
!      if ( ipstar>0 ) then
!         irecnr := inback(index)
!         if ( irecnr>0 ) then
!            infor(index) := -inback(index)
!         else if ( irecnr=0 ) then
!            write array to backing storage from position ifree3
!         else ( irecnr<0 )
!            write array to backing storage and check the start position
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ini100' )
      ipstar = infor(index)
      if ( ipstar>0 ) then

!     --- ipstar>0, array is in IBUFFR

         if ( idebug==0 ) then
            iextra = 0
         else
            iextra = intlen
         end if
         length = infor1(1,index)+iextra*2
         irecnr = inback(index)
         if ( irecnr>0 ) then

!        --- irecnr>0,  no need of writing of array, since array is already
!                         on file 3
!                         positions in ibuffr are free to be used

            infor(index) = -inback(index)
            go to 1000

         else if ( irecnr==0 ) then

!        --- irecnr  =  0   write array to backing storage from position ifree3

            irecn = ifree3
            inback(index) = irecn
            numrec = (length-1)/nrec3+1
            lenbac(index) = numrec

         else

!        --- irecnr<0    write array to backing storage
!                          check the start position

            numrec = (length-1)/nrec3+1
            if ( numrec<=lenbac(index) ) then

!           --- number of records is not larger than before
!               write on old positions

               irecn = -inback(index)
               inback(index) = irecn

            else

!           --- number of records is too large to store on old positions
!               store from record ifree3

               irecn = ifree3
               inback(index) = irecn
               lenbac(index) = numrec

            end if

         end if

         infor(index) = -irecn
         if (ioutp>=2) &
           write(irefwr,*) ' writing array ', namarr(index), &
                           ' number ', index, &
                           ' to backing storage'
         call writfl ( irecn, 1, length-1, &
                       1, 3, ibuffr(ipstar-iextra) )
         if ( idebug>=2 ) then
            ibuffr(ipstar-iextra:ipstar+length-1-iextra) = iinfin
         end if

      end if

1000  call erclos ( 'ini100' )

      end
