      subroutine ini102 ( iwork, numarr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.9  date 21-11-2012 Long integers
!        version  1.8  date 06-02-2002 Extension with eropen/erclos
!        version  1.7  date 28-01-2000 Do not sort if numarr = 0
!
!   copyright (c) 1986-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   sort the sequence in array infor and store in array iwork
!   at output numarr is the number of arrays stored in iwork
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     sort
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons3f90'
      include 'SPcommon/carrayf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*)
      integer numarr

!    iwork  o  sorted array of index numbers
!              Array iwork is sorted as follows:
!              iwork(1) contains the index number of the array with the
!                       smallest start number in ibuffr
!              iwork(2) contains the index number of the array with the
!                        next smallest start number in ibuffr   etc. etc.
!    numarr o  number of arrays in iwork
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) keysrt(numarrs)
      integer ispace(numarrs), i, ksort(numarrs)

!    i          General loop variable
!    ispace     Integer work space for subroutine chsort
!    keysrt     Contains the starting addresses of the arrays in IBUFFR
!    ksort      Contais the sorted sequence of the arrays
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHSORTLONG     Sort long integer array in increasing order
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Store sequence numbers of all arrays in IBUFFR in iwork and corresponding
!   starting addresses in keysrt
!   Sort keysrt, the result is stored in KSORT
!   Copy iwork into ISPACE
!   Put final ordering in IWORK
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Store sequence numbers into IWORK and starting addresses in KEYSRT

      call eropen ( 'ini102' )
      numarr = 0
      do i = 1, iinfor
         if ( infor(i)>0 .and. infor1(1,i)>0 ) then

!        --- infor(i)>0   array "i" is stored in ibuffr

            numarr = numarr+1
            iwork(numarr)  = i
            keysrt(numarr) = infor(i)

         end if

      end do

!     --- Sort KEYSRT

      if ( numarr>0 ) call chsortlong ( keysrt, ksort, numarr )

!     --- Copy iwork into ispace

      ispace(1:numarr) = iwork(1:numarr)

!     --- Fill iwork finally

      iwork(1:numarr) = ispace(ksort(1:numarr))

      call erclos ( 'ini102' )

      end
