      subroutine iniaccumularr ( iarray, npoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-08-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Accumulate contents of array iarray
!     iarray(1) := 0
!     iarray(i) := iarray(i-1)+iarray(i)
! **********************************************************************
!
!                       KEYWORDS
!
!     accumulate
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, iarray(npoint+1)

!     iarray        i/o   Array whose values must be accumulated
!     npoint         i    length of iarray
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isum, i, ih
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     isum           Integer variable to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'iniaccumularr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from iniaccumularr'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      isum = 0
      do i = 1, npoint
         ih = iarray(i)
         iarray(i) = isum
         isum = isum+ih
      end do

      iarray(npoint+1) = isum

1000  call erclos ( 'iniaccumularr' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End iniaccumularr'

      end if  ! ( debug )

      end
