      subroutine iniactelgrps ( ibuffr, kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 28-12-2002
!
!   copyright (c) 2002-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill part elgrps of kmesh part AB
!     This means information per element group
! **********************************************************************
!
!                       KEYWORDS
!
!     element_group
!     mesh
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), kmesh(*)

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     kmesh         i/o   Standard SEPRAN array containing information of the
!                         mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipelgrps
      integer kelmab, mmelgrps, ninpgr, nelgrp, lnelgrps

!     ipelgrps       Starting address of array ELGRPS in IBUFFR
!                    This array contains ordered information about the
!                    element groups
!     kelmab         Starting position of KMESH part AB in array KMESH
!     lnelgrps       length of array elgrps
!     mmelgrps       memory management sequence number of array ELGRPS
!     nelgrp         Number of element groups
!     ninpgr         Number of positions per element group that are stored
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) iniget

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI081         Create permanent space in array IBUFFR
!     INIGET         Compute pointer of array in IBUFFR
!     TOPGR02        Creates part elgrps of KMESH part AB
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     create array elgrps
!     Compute starting address
!     Actually fill elgrps
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'iniactelgrps' )
      kelmab = kmesh(49)-1
      mmelgrps = 0

!     --- Array elgrps does not exist, create

      ninpgr = 8
      nelgrp = kmesh(5)     ! number of element groups
      lnelgrps = (2+ninpgr)*nelgrp  ! length of elgrps
      call ini081 ( ibuffr, mmelgrps, lnelgrps, 'elgrps' )
      kmesh(kelmab+1) = mmelgrps    ! store memory management number

!     --- Compute starting address

      ipelgrps = iniget(mmelgrps)

!     --- Actually fill elgrps

      call topgr02 ( ibuffr(ipelgrps), ninpgr, kmesh )

      call erclos ( 'iniactelgrps' )
      end
