      subroutine iniactm0 ( ibuffr, buffer, jchoice, intar, intarex )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.28 date 21-11-2012 Long integers
!        version  1.27 date 16-07-2012 Remove ini062
!        version  1.26 date 24-02-2012 extra debug
!
!   copyright (c) 1998-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     General memory management subroutine that is used to activate
!     a permanent array. If this array does not exist
!     the new array is created if possible. If it is not possible to create
!     the new array an error message is given.
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cconstf90'
      include 'SPcommon/cmcdpif90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), intar(*), intarex(*), jchoice
      double precision buffer(*)

!     buffer    i/o   Buffer array
!     ibuffr    i/o   Integer buffer array in which all large data is stored
!     intar      i    Contains the main array from which a part must be
!                     created or activated.
!                     Which array it concerns depends on JCHOICE
!     intarex    i    Extra array that may be necessary to create a part of
!                     intar.
!     jchoice    i    Choice parameter indicating which array must be
!                     activated or created
!                     Possible values:
!                     1:  kmesh part AB: part ELGRPS
!                     2:  kmesh part AA: part NMESHE
!                     3:  kmesh part d
!                     4:  kmesh part AA: parts NMESHC and D
!                     5:  kmesh part AA: part NMESHH
!                     6:  kmesh part AA: part NMESHL
!                     7:  kmesh part AA: part NMESHA and B
!                     8:  kmesh part AA: part NMESHF
!                     9:  kmesh part AA: part NMESHG
!                    10:  kmesh part AB: part ELSCURVS
!                    11:  kmesh part AB: part ELSSURFS
!                    12:  kmesh part AB: part INFVOLMS
!                    13:  kmesh part AB: part INFSURFS
!                    14:  kmesh part AB: part INFCURVS
!                    15:  kmesh part AB: part INFUSPNT
!                    16:  kmesh part AB: part ELSUSPNT
!                    17:
!                    18:
!                    19:
!                    20:  kmesh part e
!                    21:  kmesh part AD: parts ipointobst
!                    22:  kmesh part AD: parts ielemobst
!                    23:  kmesh part AD: parts icrossobst,rcrossobst
!                    24:  kmesh part ae
!                    25:  kmesh part af
!                    26:  kmesh part AD: part obstfaces
!                    27:  kmesh part AA: part NMESHK
!                    28:  kmesh part o
!                    29:  kmesh part AH: outer curves/surfaces
!                    30:  kmesh part AA: part NMESH J
!                    31:  kmesh part AA: part NMESH i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipkmsc, ipkmsm, ipnmpf, ipelscur, ipelssur, &
                       ipnmpc, ipiinp, ipielgrp_nodes, ipelsupt, &
                       ipinfvolms, ipinfsurfs, ipinfcurvs, ipkmpl
      integer iseqrec(100), nseqrec, i, &
              kelmaa, kelmab, kelmd, &
              mmelgrps, mmnmesha, mmnmeshc, mmnmeshj, mmnmeshi, &
              mmnmeshd, mmnmeshe, mmnmeshf, mmnmeshg, mmnmeshh, &
              mmnmeshl, mmelscur, mmelssur, &
              ncurvs, nedges, nfaces, npoint, nsurfs, nvolms, &
              mminfvolms, lninfvolms, ninpvo, &
              mminfsurfs, lninfsurfs, ninpsu, mmelsupt, &
              mminfcurvs, lninfcurvs, ninpcu, &
              mminfuspnt, iinp, icurve, isurf, lnelsupt, &
              nuspnt, mmnmeshb, kelme, kelmad, &
              lenielgrpnods, lennodeelgrps, mmnmeshk, error
      integer, allocatable :: ielgrp_nodes(:), node_elgrps(:)
      logical debug, exist

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     error          Return value of allocate or deallocate
!     exist          If true, an array already exists
!     i              Counting variable
!     icurve         Type of curve
!     ielgrp_nodes   In this array all nodes corresponding to the
!                    element groups are stored in the following way:
!                    Pos. 1 to nelgrp starting addresses istart_ielgrp
!                    Pos. istart_ielgrp: nnodes (number of nodes in
!                                        element group ielgrp)
!                    Pos. istart_ielgrp+1 to istart_ielgrp+nnodes:
!                                        node numbers
!     iinp           last position used in array iinput
!     ipelscur       Pointer to elscurvs
!     ipelssur       Pointer to elssurfs
!     ipelsupt       Pointer to elsuserpoints
!     ipielgrp_nodes Starting address of array ielgrp_nodes in array IBUFFR
!     ipiinp         Starting address of array IINPUT in array IBUFFR
!     ipinfcurvs     Pointer to infcurvs
!     ipinfsurfs     Pointer to infsurfs
!     ipinfvolms     Pointer to infvolms
!     ipkmpl         pointer to kmesh part l
!     ipkmsc         Starting address of array KMESH part c in array IBUFFR
!     ipkmsm         Starting address of array KMESH part m in array IBUFFR
!     ipnmpc         Pointer to nmesh part c
!     ipnmpf         Pointer to nmesh part f
!     iseqrec        Array that is used to define recursion
!                    In fact the actual parameter JCHOICE is stored in position
!                    nseqrec
!                    In the positions 1 to nseqrec-1 the previous values of
!                    JCHOICE are stored
!                    As soon as nseqrec=0 all necessary actions have been
!                    performed
!     isurf          Type of surface with respect to mesh generator
!     kelmaa         Starting position of KMESH part AA in array KMESH
!     kelmab         Starting position of KMESH part AB in array KMESH
!     kelmad         Starting position of KMESH part AD in array KMESH
!     kelmd          Memory management sequence number of KMESH part d
!     kelme          Memory management sequence number of KMESH part e
!     lenielgrpnods  Length of array ielgrp_nodes
!     lennodeelgrps  Length of array nodeelgrps
!     lnelsupt       length of array userpoints
!     lninfcurvs     length of array infcurvs
!     lninfsurfs     length of array infsurfs
!     lninfvolms     length of array infvolms
!     mmelgrps       memory management sequence number of array elgrps
!     mmelscur       memory management sequence number of array elscurvs
!     mmelssur       memory management sequence number of array elssur
!     mmelsupt       memory management sequence number of array elsupt
!     mminfcurvs     memory management sequence number of array infcurvs
!     mminfsurfs     memory management sequence number of array infsurfs
!     mminfuspnt     memory management sequence number of array infuspnt
!     mminfvolms     memory management sequence number of array infvolms
!     mmnmesha       memory management sequence number of array nmesha
!     mmnmeshb       memory management sequence number of array nmeshb
!     mmnmeshc       memory management sequence number of array nmeshc
!     mmnmeshd       memory management sequence number of array nmeshd
!     mmnmeshe       memory management sequence number of array nmeshe
!     mmnmeshf       memory management sequence number of array nmeshf
!     mmnmeshg       memory management sequence number of array nmeshg
!     mmnmeshh       memory management sequence number of array nmeshh
!     mmnmeshi       memory management sequence number of array nmeshi
!     mmnmeshj       memory management sequence number of array nmeshj
!     mmnmeshk       memory management sequence number of array nmeshk
!     mmnmeshl       memory management sequence number of array nmeshl
!     ncurvs         Number of curves in mesh
!     nedges         number of edges
!     nfaces         Number of faces
!     ninpcu         number of info positions per curve
!     ninpsu         number of info positions per surface
!     ninpvo         number of info positions per volume
!     node_elgrps    In this array all element groups corresponding to the
!                    nodes are stored in the following way:
!                    Pos. 1 to npoint if negative -starting addresses
!                                     istart_node
!                                     if positive element group number
!                    Pos. istart_node: number of element groups (>1)
!                    Next positions element group numbers
!     npoint         Number of nodal points
!     nseqrec        Number of positions that are used is iseqrec
!     nsurfs         Number of surfaces in mesh
!     nuspnt         Number of user points
!     nvolms         number of volumes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060CHECK    Reactivate existing array in IBUFFR and check first
!                    position
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INI081         Create permanent space in array IBUFFR
!     INIACTELGRPS   Fill array elgrps
!     INIACTKMESHD   Fill array kmesh part d
!     INIACTM26      Fill kmesh part ad subpart 9: iobstfaces
!     INIACTNMESHA   Fill array nmesh parts a and b
!     INIACTNMESHE   Fill array nmesh part e
!     INIACTNMESHH   Fill array nmesh part h
!     INIACTNMESHJ   Fill array nmesh part j
!     INIACTNMESHL   Fill array nmesh part l
!     INICHK1        Check IBUFFR
!     INICOPBUF      Activate or create the array referred to by the memory
!                    management index mmindex and copy the contents of
!                    iarray to ibuffr
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     MSH046         Raise iinp as function of icurve
!     MSH049         Compute starting address of specific surface in array
!                    IINPUT
!     TOPCU03        fill array kmeshab with boundaries of surfaces
!     TOPELGRPNODES  Fill array kmesh part ae with all nodes corresponding
!                    to element groups
!     TOPFACEEDGE    Fill array nmeshi with the face numbers of the faces
!                    connected to the edges
!     TOPFILLELSCUR  Fill array elscurvs with edges per curve
!     TOPFILLELSSUR  Fill array elssurfs with edges per surface
!     TOPFILLKMAD1   Fill array kmesh ad  part 1: ipointobst
!     TOPFILLKMAD2   Fill array kmesh ad  part 2: ielemobst
!     TOPFILLKMAD3   Fill array kmesh ad  part 3: icrossobst,rcrossobst
!     TOPFILLKMESHE  Fill array kmesh part e
!     TOPFILLKMESHO  Fill array kmesh part o
!     TOPFILLNMESHC  Fill array nmesh parts c and d
!     TOPFILLNMESHF  Fill array nmesh part f
!     TOPFILLNMESHG  Fill array nmesh part g
!     TOPFILLOUTERB  Fill outer boundary of mesh
!     TOPNODEELGRPS  Fill array kmesh part af with all element groups
!                    corresponding to nodes
!     TOPNODFACES    Fill array nmeshk with all faces connected to the nodes
!     TOPSU03        fill array kmeshab with boundaries of surfaces
!     TOPVL01        fill array kmeshab with boundaries of volumes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1691   NSEQREC too small
! **********************************************************************
!
!                       PSEUDO CODE
!
!    iseqrec(1) := jchoice
!    nseqrec := 1
!    While ( nseqrec>0 ) do
!       check upper bound nseqrec
!       If (necessary array not available) then
!          nseqrec := nseqrec+1
!          put action to create this array in iseqrec(nseqrec)
!       else (all arrays are available)
!          Perform action depending on iseqrec(nseqrec)
!          nseqrec := nseqrec-1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'iniactm0' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from iniactm0'
         write(irefwr,1) 'jchoice', jchoice
  1      format ( a, 1x, (10i6) )

      end if

!     --- Initialize iseqrec

      nseqrec = 1
      iseqrec(1) = jchoice

!     --- Fill arrays recursively if needed

100   if ( ierror/=0 ) go to 1000
      if ( nseqrec>100 ) then

!     --- nseqrec too large

         call errint ( nseqrec, 1 )
         call errint ( 100, 2 )
         call errsub ( 1691, 2, 0, 0 )
         go to 100

      else if ( nseqrec>0 ) then

!     --- nseqrec>0, still a part of the array must be filled
!         Jump to correct position

         if ( debug ) &
            write(irefwr,1) 'nseqrec, iseqrec(nseqrec)', &
                             nseqrec, iseqrec(nseqrec)

         select case ( iseqrec(nseqrec) )

            case(1)

!           --- iseqrec() = 1, Fill kmesh AB part elgrps

               kelmab = intar(49)-1
               mmelgrps = intar(kelmab+1)
               if ( mmelgrps>0 ) then

!              --- Array elgrps already exists: activate

                  call ini060text ( ibuffr, mmelgrps, 'elgrps' )

               else

!              --- Array elgrps does not exist, create

                  call iniactelgrps ( ibuffr, intar )

               end if

            case(2)

!           --- iseqrec() = 2, Fill kmesh AA part nmeshe

               kelmaa = intar(48)-1
               mmnmeshe = intar(kelmaa+35)
               if ( mmnmeshe>0 ) then

!              --- Array nmeshe already exists: activate

                  call ini060check ( ibuffr, mmnmeshe, intar(9)+1 )

               else

!              --- Array nmeshe does not exist, create

                  call iniactnmeshe ( ibuffr, intar )

               end if

            case(3)

!           --- iseqrec() = 3, Fill kmesh part d

               kelmd = intar(18)
               if ( debug ) write(irefwr,1) 'kelmd', kelmd
               if ( kelmd>0 ) then

!              --- Array kmesh d already exists: activate

                  call ini060check ( ibuffr, kelmd, 0 )

               else

!              --- Array kmesh d does not exist, create

                  call iniactkmeshd ( ibuffr, intar )

               end if

            case(4)

!           --- iseqrec() = 4, Fill kmesh AA parts nmeshc and nmeshd

               kelmaa = intar(48)-1
               if ( debug ) write(irefwr,1) 'kelmaa', kelmaa

               mmnmeshc = intar(kelmaa+33)
               mmnmeshd = intar(kelmaa+34)
               if ( debug ) &
                  write(irefwr,1) 'mmnmeshc, mmnmeshd', &
                                   mmnmeshc, mmnmeshd

               if ( mmnmeshc>0 .and. mmnmeshd>0 ) then

!              --- Array nmeshc already exists: activate

                  nedges = intar(kelmaa+4)
                  if ( debug ) write(irefwr,1) 'nedges', nedges
                  call ini060check ( ibuffr, mmnmeshc, nedges+1 )
                  call ini060check ( ibuffr, mmnmeshd, intar(9)+1 )

               else

!              --- Array nmeshc does not exist, create

                  call topfillnmeshc ( ibuffr, buffer, intar )

               end if

            case(5)

!           --- iseqrec() = 5, Fill kmesh AA part nmeshh

               kelmaa = intar(48)-1
               mmnmeshh = intar(kelmaa+38)
               if ( mmnmeshh>0 ) then

!              --- Array nmeshh already exists: activate

                  nedges = intar(kelmaa+4)
                  call ini060check ( ibuffr, mmnmeshh, nedges+1 )

               else

!              --- Array nmeshh does not exist, create

                  if ( debug ) write(irefwr,*) 'before iniactnmeshh'
                  call iniactnmeshh ( ibuffr, buffer, intar )
                  if ( debug ) write(irefwr,*) 'after iniactnmeshh'

               end if

            case(6)

!           --- iseqrec() = 6, Fill kmesh AA part nmeshl

               kelmaa = intar(48)-1
               mmnmeshl = intar(kelmaa+42)
               if ( mmnmeshl>0 ) then

!              --- Array nmeshl already exists: activate

                  call ini060check ( ibuffr, mmnmeshl, intar(8)+1 )

               else

!              --- Array nmeshl does not exist, create

                  call iniactnmeshl ( ibuffr, buffer, intar )

               end if

            case(7)

!           --- iseqrec() = 7, Fill kmesh AA part nmesha, b

               kelmaa = intar(48)-1
               mmnmesha = intar(kelmaa+31)
               mmnmeshb = intar(kelmaa+32)
               if ( mmnmesha>0 ) then

!              --- Array nmesha, b already exists: activate

                  nfaces = intar(kelmaa+5)
                  call ini060check ( ibuffr, mmnmesha, intar(9)+1 )
                  call ini060check ( ibuffr, mmnmeshb, nfaces+1 )

               else

!              --- Array nmesha, b does not exist, create

                  call iniactnmesha ( ibuffr, buffer, intar )

               end if

            case(8)

!           --- iseqrec() = 8, Fill kmesh AA part nmeshf

               kelmaa = intar(48)-1
               mmnmeshf = intar(kelmaa+36)
               if ( mmnmeshf>0 ) then

!              --- Array nmeshf already exists: activate

                  nfaces = intar(kelmaa+5)
                  call ini060check ( ibuffr, mmnmeshf, nfaces+1 )

               else

!              --- Array nmeshf does not exist, create

                  call topfillnmeshf ( ibuffr, buffer, intar )

               end if

            case(9)

!           --- iseqrec() = 9, Fill kmesh AA part nmeshg

               kelmaa = intar(48)-1
               mmnmeshg = intar(kelmaa+37)
               if ( mmnmeshg>0 ) then

!              --- Array nmeshg already exists: activate

                  nfaces = intar(kelmaa+5)
                  call ini060check ( ibuffr, mmnmeshg, nfaces+1 )

               else

!              --- Array nmeshg does not exist, create

                  call topfillnmeshg ( ibuffr, buffer, intar )

               end if

            case(10)

!           --- iseqrec() = 10, Fill kmesh AB part elscurvs

               kelmab = intar(49)-1
               mmelscur = intar(kelmab+11)
               if ( mmelscur>0 ) then

!              --- Array elscurvs already exists: activate

                  call ini060text ( ibuffr, mmelscur, 'elscur' )

               else

!              --- Array elscurvs does not exist, create

                  call topfillelscur ( ibuffr, buffer, intar )

               end if

            case(11)

!           --- iseqrec() = 11, Fill kmesh AB part elssurfs

               kelmab = intar(49)-1
               mmelssur = intar(kelmab+10)
               if ( mmelssur>0 ) then

!              --- Array elssur already exists: activate

                  call ini060text ( ibuffr, mmelssur, 'elssur' )

               else

!              --- Array elssur does not exist, create

                  call topfillelssur ( ibuffr, buffer, intar )

               end if

            case(12)

!           --- iseqrec() = 12, Fill kmesh AB part infvolms

               kelmaa = intar(48)-1
               kelmab = intar(49)-1
               mminfvolms = intar(kelmab+2)
               if ( mminfvolms>0 ) then

!              --- Array infvolms already exists: activate

                  call ini060text ( ibuffr, mminfvolms, 'infvolms' )

               else

!              --- Array infvolms does not exist, create

                  nvolms = intar(13)
                  if ( nvolms>0 ) then
                     call ini060text ( ibuffr, intar(34), 'iinput' )
                     ipiinp = inigettext ( intar(34), 'iinput' )-1

!                    --- compute the starting address in iinput of volumes

                     ncurvs = intar(11)
                     nsurfs = intar(12)
                     iinp = 16

!                    --- first curves

                     do i = 1,ncurvs
                        icurve = ibuffr(ipiinp+iinp)
                        call msh046(ibuffr(ipiinp+1),iinp,icurve)
                     end do

!                    --- next surfaces

                     do i = 1,nsurfs
                        isurf = ibuffr(ipiinp+iinp)
                        if ( isurf==0 ) then
                           iinp = iinp + 1
                        else
                           call msh049(ibuffr(ipiinp+1),iinp,isurf)
                        end if
                     end do

                     ninpvo = 9
                     lninfvolms = (2+ninpvo)*nvolms
                     call ini081 ( ibuffr, mminfvolms, lninfvolms, &
                                   'volms' )
                     call ini060text ( ibuffr, intar(34), 'iinput' )
                     ipiinp = inigettext ( intar(34), 'iinput' )-1
                     ipinfvolms = inigettext ( mminfvolms, 'infvolms' )
                     call topvl01 ( ibuffr(ipinfvolms), &
                                    ibuffr(ipiinp+1), intar, lninfvolms, &
                                    ninpvo, iinp )
                  else
                     mminfvolms = 0
                  end if

                  intar(kelmab+2) = mminfvolms

               end if

            case(13)

!           --- iseqrec() = 13, Fill kmesh AB part infsurfs

               kelmaa = intar(48)-1
               kelmab = intar(49)-1
               mminfsurfs = intar(kelmab+3)
               if ( mminfsurfs>0 ) then

!              --- Array infsurfs already exists: activate

                  call ini060text ( ibuffr, mminfsurfs, 'infsurfs' )

               else

!              --- Array infsurfs does not exist, create

                  nsurfs = intar(12)
                  if ( nsurfs>0 ) then
                     if ( intar(kelmab+10)==0 ) then

!                    --- kmeshab part surfaces does not exist: create

                        nseqrec = nseqrec+1
                        iseqrec(nseqrec) = 11
                        go to 100

                     end if

                     if ( intar(kelmaa+36)==0 ) then

!                    --- kmeshab part surfaces does not exist: create

                        nseqrec = nseqrec+1
                        iseqrec(nseqrec) = 8
                        go to 100

                     end if

                     if (intar(34)>0) then
                        call ini060text ( ibuffr, intar(34), 'iinput' )
                        ipiinp = inigettext ( intar(34), 'iinput' )-1

!                       --- compute the starting address in iinput of surfaces

                        ncurvs = intar(11)
                        iinp = 16

!                       --- first curves

                        do i = 1,ncurvs
                           icurve = ibuffr(ipiinp+iinp)
                           call msh046(ibuffr(ipiinp+1),iinp,icurve)
                        end do
                     else
                        iinp = -1
                        ipiinp = 0
                     end if

                     ninpsu = 9
                     lninfsurfs = (2+ninpsu)*nsurfs
                     call ini081 ( ibuffr, mminfsurfs, lninfsurfs, &
                                   'surfs' )
                     if (intar(34)>0) &
                        call ini060text ( ibuffr, intar(34), 'iinput' )
                     call ini060text ( ibuffr, intar(kelmab+10), &
                                       'elssur' )
                     call ini060text ( ibuffr, intar(kelmaa+36), &
                                       'nmeshf' )
                     if (intar(34)>0) &
                        ipiinp = inigettext ( intar(34), 'iinput' )-1
                     ipinfsurfs = inigettext ( mminfsurfs, 'infsurfs' )
                     ipelssur = inigettext ( intar(kelmab+10), &
                                             'elssur' )
                     ipnmpf = inigettext ( intar(kelmaa+36), 'nmeshf' )
                     call topsu03 ( intar, ibuffr(ipinfsurfs), &
                                    ibuffr(ipelssur), ibuffr(ipnmpf), &
                                    ibuffr(ipiinp+1), &
                                    lninfsurfs, ninpsu, nsurfs, iinp)
                  else

                     mminfsurfs = 0
                  end if

                  intar(kelmab+3) = mminfsurfs

               end if

            case(14)

!           --- iseqrec() = 14, Fill kmesh AB part infcurvs

               kelmaa = intar(48)-1
               kelmab = intar(49)-1
               mminfcurvs = intar(kelmab+4)
               if ( mminfcurvs>0 ) then

!              --- Array infcurvs already exists: activate

                  call ini060text ( ibuffr, mminfcurvs, 'infcurvs' )

               else

!              --- Array infcurvs does not exist, create

                  ncurvs = intar(11)
                  if (ncurvs>0) then
                     if ( intar(kelmab+11)==0 ) then

!                    --- kmeshab part curves does not exist: create

                        nseqrec = nseqrec+1
                        iseqrec(nseqrec) = 10
                        go to 100

                     end if

                     ninpcu = 9
                     lninfcurvs = (2+ninpcu)*ncurvs
                     call ini081 ( ibuffr, mminfcurvs, lninfcurvs, &
                                   'curvs' )
                     ipkmsm = 0
                     if (intar(27)>0) then
                        call ini060text ( ibuffr, intar(27), 'kmeshm' )
                        ipkmsm = inigettext ( intar(27), 'kmeshm' )-1
                     end if
                     call ini060text ( ibuffr, intar(kelmab+11), &
                                       'elscur' )
                     call ini060text ( ibuffr, intar(kelmaa+33), &
                                       'nmeshc' )
                     if (intar(34)>0) then
                        call ini060text ( ibuffr, intar(34), 'iinput' )
                        ipiinp = inigettext ( intar(34), 'iinput' )-1
                        iinp = 16
                     else
                        ipiinp = 0
                        iinp = -1
                     end if
                     ipinfcurvs = inigettext ( mminfcurvs, 'infcurvs' )
                     ipelscur = inigettext ( intar(kelmab+11), &
                                             'elscur' )
                     ipnmpc   = inigettext ( intar(kelmaa+33), &
                                             'nmeshc' )
                     call topcu03 ( ibuffr, ibuffr(ipinfcurvs), &
                                    ibuffr(ipelscur), ibuffr(ipnmpc), &
                                    ibuffr(ipiinp+1), &
                                    lninfcurvs, ninpcu, ncurvs, iinp, &
                                    ipkmsm )
                  else
                     mminfcurvs = 0
                  end if

                  intar(kelmab+4) = mminfcurvs

               end if

            case(15)

!           --- iseqrec() = 15, Fill kmesh AB part infuspnt

               kelmaa = intar(48)-1
               kelmab = intar(49)-1
               mminfuspnt = intar(kelmab+5)
               if ( mminfuspnt>0 ) then

!              --- Array infuspnt already exists: activate

                  call ini060text ( ibuffr, mminfuspnt, 'infuspnt' )

               else

!              --- Array infuspnt does not exist, create

               end if

            case(16)

!           --- iseqrec() = 16, Fill kmesh AB part elsuspnt

               kelmaa = intar(48)-1
               kelmab = intar(49)-1
               mmelsupt = intar(kelmab+12)
               if ( mmelsupt>0 ) then

!              --- Array elsupnt already exists: activate

                  call ini060text ( ibuffr, mmelsupt, 'elsupnt' )

               else

!              --- Array elsupnt does not exist, create

                  nuspnt = intar(10)
                  lnelsupt = 2*nuspnt+nuspnt

!                 --- Create space for elsupt

                  call ini081 ( ibuffr, mmelsupt, lnelsupt, 'uspnts' )
                  ipelsupt = inigettext ( mmelsupt, 'uspnts' )-1
                  call ini060text ( ibuffr, -intar(26), 'kmeshl' )
                  ipkmpl = inigettext ( intar(26), 'kmeshl' )-1
                  do i = 1, nuspnt
                     ibuffr(ipelsupt+i) = nuspnt + 2*i-1
                     ibuffr(ipelsupt+nuspnt+2*i-1) = 1
                     ibuffr(ipelsupt+nuspnt+2*i) = ibuffr(ipkmpl+i)
                  end do

                  intar(kelmab+12) = mmelsupt

               end if

            case(20)

!           --- iseqrec() = 20, Fill kmesh part e

               kelme = intar(19)
               if ( debug ) write(irefwr,1) 'kelme', kelme
               if ( kelme>0 ) then

!              --- Array kmesh e already exists: activate

                  call ini060check ( ibuffr, kelme, 0 )

               else

!              --- Array kmesh e does not exist, create

                  call topfillkmeshe ( ibuffr, intar )

               end if

            case(21)

!           --- iseqrec() = 21, Fill kmesh part ad pos. 1

               kelmad = intar(52)
               exist = .false.
               if ( kelmad>0 ) then

!              --- Array kmesh ad already exists: check if sequence number is
!                  correct

                  if ( intar(kelmad-1+5)==intar(kelmad-1+6) ) then

!                 --- Sequence numbers equal, activate

                     call ini060text ( ibuffr, intar(kelmad), &
                                       'ipointobst' )
                     exist = .true.

                  end if

               end if

               if ( .not. exist ) then

!              --- Array kmesh ad does not exist, create

                  call topfillkmad1 ( ibuffr, buffer, intar )

               end if

            case(22)

!           --- iseqrec() = 22, Fill kmesh part ad pos. 2

               kelmad = intar(52)
               exist = .false.
               if ( kelmad>0 .and. intar(kelmad+1)>0 ) then

!              --- Array kmesh ad already exists: check if sequence number is
!                  correct

                  if ( intar(kelmad-1+5)==intar(kelmad-1+7) ) then

!                 --- Sequence numbers equal, activate

                     call ini060text ( ibuffr, intar(kelmad+1), &
                                       'ielemobst' )
                     exist = .true.

                  end if

               end if

               if ( .not. exist ) then

!              --- Array kmesh ad 2 does not exist, create

                  call topfillkmad2 ( ibuffr, buffer, intar )

               end if

            case(23)

!           --- iseqrec() = 23, Fill kmesh part ad pos. 3,4

               kelmad = intar(52)
               exist = .false.
               if ( kelmad>0 .and. intar(kelmad+2)>0 ) then

!              --- Array kmesh ad already exists: check if sequence number is
!                  correct

                  if ( intar(kelmad-1+5)==intar(kelmad-1+8) ) then

!                 --- Sequence numbers equal, activate

                     call ini060text ( ibuffr, intar(kelmad+2), &
                                       'icrossobst' )
                     call ini060text ( ibuffr, intar(kelmad+3), &
                                       'crossobst' )
                     if ( intar(kelmad+9)>0 ) call ini060text &
                        ( ibuffr, intar(kelmad+9), 'obstfaces' )
                     exist = .true.

                  end if

               end if

               if ( .not. exist ) then

!              --- Array kmesh ad 3, 4 does not exist, create

                  call topfillkmad3 ( ibuffr, buffer, intar )

               end if

            case(24)

!           --- iseqrec() = 24, Fill kmesh part ae

               if ( intar(55)>0 ) then

!              --- Array kmesh part ae already exists: activate

                  call ini060text ( ibuffr, intar(55), 'kmeshae' )
                  nseqrec = nseqrec-1
                  go to 100

               end if

!              --- Array does not exist, create
!                  First activate kmesh part c

               call ini060text ( ibuffr, intar(17), 'kmeshc' )

!              --- Next reserve work space

               npoint = intar(8)
               lenielgrpnods = 3*(npoint+intar(5))

               allocate ( ielgrp_nodes(lenielgrpnods), stat = error )
               if ( error/=0 ) &
                  call eralloc ( error, lenielgrpnods, 'ielgrp_nodes' )
               if ( ierror/=0 ) go to 1000

!              --- Set pointers

               ipkmsc = inigettext ( intar(17), 'kmeshc' )

!              --- Actual call

               call topelgrpnodes ( intar, ibuffr(ipkmsc), &
                                    ielgrp_nodes, lenielgrpnods )

!              --- Make array ielgrp_nodes permanent

               call inicopbuf ( ibuffr, intar(55), ielgrp_nodes, &
                                lenielgrpnods, 'ielgrp_nodes' )

               deallocate ( ielgrp_nodes, stat = error )
               if ( error/=0 ) &
                  call erdealloc ( error, 'ielgrp_nodes' )
               if ( ierror/=0 ) go to 1000

            case(25)

!           --- iseqrec() = 25, Fill kmesh part af

               if ( intar(56)>0 ) then

!              --- Array kmesh part af already exists: activate

                  call ini060text ( ibuffr, intar(56), 'kmeshaf' )
                  nseqrec = nseqrec-1
                  go to 100

               end if

!              --- Array kmesh part af does not exist, create
!                  First check if kmesh part ae exists

               if ( intar(55)==0 ) then

!              --- kmesh part ae does not exist: create

                  nseqrec = nseqrec+1
                  iseqrec(nseqrec) = 24
                  go to 100

               end if

!              --- First activate kmesh part ae

               call ini060text ( ibuffr, intar(55), 'kmeshae' )

!              --- Next reserve work space

               npoint = intar(8)
               lennodeelgrps = 4*npoint

               allocate ( node_elgrps(lennodeelgrps), stat = error )
               if ( error/=0 ) &
                  call eralloc ( error, lennodeelgrps, 'node_elgrps' )
               if ( ierror/=0 ) go to 1000

!              --- Set pointers

               ipielgrp_nodes = inigettext ( intar(55), 'ielgrp_nodes' )

!              --- Actual call

               call topnodeelgrps ( intar, ibuffr(ipielgrp_nodes), &
                                    node_elgrps, lennodeelgrps )

!              --- Make array node_elgrps permanent

               call inicopbuf ( ibuffr, intar(56), node_elgrps, &
                                lennodeelgrps, 'node_elgrps' )

               deallocate ( node_elgrps, stat = error )
               if ( error/=0 ) &
                  call erdealloc ( error, 'node_elgrps' )
               if ( ierror/=0 ) go to 1000

            case(26)

!           --- iseqrec() = 26, Fill kmesh part ad pos. 9

               call iniactm26 ( ibuffr, buffer, intar )

            case(27)

!           --- iseqrec() = 27, Fill nmeshk

               kelmaa = intar(48)-1
               mmnmeshk = intar(kelmaa+41)
               if ( mmnmeshk>0 ) then

!              --- Array nmeshk already exists: activate

                  call ini060check ( ibuffr, mmnmeshk, intar(8)+1 )

               else

!              --- Array nmeshk does not exist, create

                  call topnodfaces ( ibuffr, intar )

               end if

            case(28)

!           --- iseqrec() = 28, Fill kmesh part o

               kelmd = intar(29)
               if ( kelmd>0 ) then

!              --- Array kmesh o already exists: activate

                  call ini060text ( ibuffr, kelmd, 'kmesho' )

               else

!              --- Array kmesh o does not exist, create

                  call topfillkmesho ( ibuffr, intar )

               end if

            case(29)

!           --- iseqrec() = 29, Fill kmesh part ah

              kelmd = intar(58)
              if ( kelmd>0 ) then

!             --- Array kmesh o already exists: activate

                 call ini060text ( ibuffr, kelmd, 'outerboun' )

              else

!             --- Array kmesh ah does not exist, create

                 call topfillouterb ( ibuffr, buffer, intar )

              end if

            case(30)

!           --- iseqrec() = 30, Fill kmesh aa part nmeshj

               kelmaa = intar(48)-1
               mmnmeshj = intar(kelmaa+40)
               if ( mmnmeshj>0 ) then

!              --- Array nmeshe already exists: activate

                  call ini060text ( ibuffr, mmnmeshj, 'nmeshj' )

               else

!              --- Array nmeshj does not exist, create

                  call iniactnmeshj ( ibuffr, intar )

               end if

            case(31)

!           --- iseqrec() = 31, Fill nmeshi

               kelmaa = intar(48)-1
               mmnmeshi = intar(kelmaa+39)
               if ( mmnmeshi>0 ) then

!              --- Array nmeshi already exists: activate

                  call ini060check ( ibuffr, mmnmeshi, &
                                     intar(kelmaa+4)+1 )

               else

!              --- Array nmeshk does not exist, create

                  call topfaceedge ( ibuffr, intar )

               end if

         end select  ! case ( iseqrec(nseqrec) )
         nseqrec = nseqrec-1

         go to 100

      end if

1000  if ( debug ) call inichk1(ibuffr)

      call erclos ( 'iniactm0' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End iniactm0'

      end if

      end
