      subroutine iniactnmesha ( ibuffr, buffer, kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 11-06-2014 Adaptation for kmesh(48) = 0
!        version  1.2  date 21-11-2012 Long integers
!        version  1.1  date 01-01-2004 Use ini060text etc
!
!   copyright (c) 2002-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create nmesh parts a and b
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     topology
!     face
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), kmesh(*)
      double precision buffer(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     kmesh         i/o   Standard SEPRAN array containing information of the
!                         mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipnmpa, ipnmpd, ipnmph, ipnmpb
      integer kelmaa, nfaces, lpnmpb, medpfa, nelem, mfapel, lpnmpa
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipnmpa         Pointer to nmesh part a
!     ipnmpb         Pointer to nmesh part b
!     ipnmpd         Pointer to nmesh part d
!     ipnmph         Pointer to nmesh part h
!     kelmaa         Starting position of KMESH part AA in array KMESH
!     lpnmpa         length of array nmesha
!     lpnmpb         length of array nmeshb
!     medpfa         maximum number of edges per face
!     mfapel         maximum number of faces in one element
!     nelem          Number of elements
!     nfaces         Number of faces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inigettext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INI062         reserve space for temporary arrays
!     INI083         Create new array in IBUFFR in the case that already a
!                    temporary copy has been stored in IBUFFR
!     INIACTNMESHH   Fill array nmesh part h
!     INIGETTEXT     Compute pointer of array in IBUFFR
!     PRININ         print 1d integer array
!     PRINKMESH      Print array KMESH
!     TOPFA01        creates the faces part 1 pointers and initialization
!     TOPFA02        creates the faces part 2 actual face labels and lmeshb
!     TOPFILLKMESHAA Fill first part of array KMESH part aa
!     TOPFILLNMESHC  Fill array nmesh parts c and d
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( maximum number of faces per element>0 ) then
!        activate old arrays nmeshd, nmeshh
!        Create temporary space and initialize first part (nmesha)
!        Initialize array nmesha=ibuffr(ipnmpa)
!        Make the array permanent
!        Fetch nmesh part a, d and h again
!        Create temorary space for nmeshb with medpfa extra space
!        Fetch pointers to nmesh part a, d and h
!        Fill nmesha with actual data and create face information
!        array nmeshb
!        Make nmeshb permanent
!     else
!        Fill information with zero faces
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'iniactnmesha' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from iniactnmesha'
         call  prinkmesh ( ibuffr, buffer, kmesh, 1 )
         call  prinkmesh ( ibuffr, buffer, kmesh, 38 )

      end if

      if ( kmesh(48)==0 ) then

!     --- Kmesh part aa has not been filled, call topfillkmeshaa

         if ( debug ) write(irefwr,*) 'fill kmeshaa'
         call topfillkmeshaa ( kmesh )

      end if
      if ( ierror/=0 ) go to 1000

      kelmaa = kmesh(48)-1

!     --- Arrays nmesha, b do not exist, create

      mfapel = kmesh(kelmaa+19)  ! maximum number of faces per element
      nelem = kmesh(9)           ! number of elements
      if ( debug ) write(irefwr,*) 'mfapel, nelem ', mfapel, nelem
      if ( mfapel>0 ) then

!     --- activate old arrays nmeshd, nmeshh

         if ( kmesh(kelmaa+34)==0 ) &
            call topfillnmeshc ( ibuffr, buffer, kmesh )  ! nmeshc/d
         if ( ierror/=0 ) go to 1000
         if ( kmesh(kelmaa+38)==0 ) &
            call iniactnmeshh ( ibuffr, buffer, kmesh )   ! nmeshh
         if ( ierror/=0 ) go to 1000
         call ini060text ( ibuffr, kmesh(kelmaa+34), 'nmeshd'  )
         call ini060text ( ibuffr, kmesh(kelmaa+38), 'nmeshh'  )
         if ( ierror/=0 ) go to 1000

!        --- Create temporary space and initialize first part (nmesha)

         lpnmpa = 2*nelem+2*nelem*mfapel  ! overestimated length of nmesha
         call ini062 ( ibuffr, lpnmpa, ipnmpa )
         if ( ierror/=0 ) go to 1000

!        --- Initialize array nmesha=ibuffr(ipnmpa)
!            For each element faces are created and face labels and
!            rotations are set to zero

         call topfa01 ( kmesh, ibuffr(ipnmpa), lpnmpa )
         if ( ierror/=0 ) go to 1000
         if ( debug ) then

!        --- Debug information

            write(irefwr,*) 'lpnmpa ',lpnmpa
            call prinin ( ibuffr(ipnmpa), nelem, 'nmesha part 1' )

         end if

!        --- Make the array permanent (store information in nmesh(31))

         call ini083 ( ibuffr, kmesh(kelmaa+31), lpnmpa, ipnmpa, &
                       'nmesha' )
         if ( ierror/=0 ) go to 1000

!        --- Fetch nmesh part a, d and h again

         call ini060text ( ibuffr, kmesh(kelmaa+31), 'nmesha' )
         call ini060text ( ibuffr, kmesh(kelmaa+34), 'nmeshd' )
         call ini060text ( ibuffr, kmesh(kelmaa+38), 'nmeshh' )
         if ( ierror/=0 ) go to 1000

!        --- Create temorary space for nmeshb with medpfa extra space
!            for help array

         medpfa = kmesh(kelmaa+21)  ! maximum number of edges per face
         lpnmpb = nelem*mfapel*(medpfa+2)  ! overestimated length of nmeshb
         call ini062 ( ibuffr, lpnmpb+2*medpfa, ipnmpb )
         if ( debug ) write(irefwr,*) 'medpfa, lpnmpb ',medpfa, lpnmpb

!        --- Fetch pointers to nmesh part a, d and h

         ipnmpa = inigettext ( kmesh(kelmaa+31), 'nmesha' )
         ipnmpd = inigettext ( kmesh(kelmaa+34), 'nmeshd' )
         ipnmph = inigettext ( kmesh(kelmaa+38), 'nmeshh' )
         if ( ierror/=0 ) go to 1000

!        --- Fill nmesha with actual data and create face information
!            array nmeshb

         call topfa02 ( kmesh, ibuffr(ipnmpd), ibuffr(ipnmpa), &
                        ibuffr(ipnmpb), ibuffr(ipnmph), medpfa, &
                        nfaces, lpnmpb, ibuffr(ipnmpb+lpnmpb) )
         if ( ierror/=0 ) go to 1000

!        --- Make nmeshb permanent

         call ini083 ( ibuffr, kmesh(kelmaa+32), lpnmpb, &
                       ipnmpb, 'nmeshb' )
         kmesh(kelmaa+5) = nfaces
         if ( debug ) then

!        --- Debug information

            call  prinkmesh ( ibuffr, buffer, kmesh, 39 )
            write(irefwr,*) 'nfaces ', nfaces
            call prinkmesh ( ibuffr, buffer, kmesh, 44 )
            call prinkmesh ( ibuffr, buffer, kmesh, 45 )
            write(irefwr,*) 'End iniactnmesha'

         end if

      else

!     --- no faces exist

         kmesh(kelmaa+5) = 0
         kmesh(kelmaa+31) = 0
         kmesh(kelmaa+32) = 0

      end if

1000  call erclos ( 'iniactnmesha' )
      end
