      subroutine iniclearboun ( usol, kprobf, indprf, nunkp, nodes, &
                                nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-02-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make solution array equal to 0 for all nodes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     clear
!     solution_vector
!     outer_boundary
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kprobf(*), indprf, nunkp, nodes(*), nnodes
      double precision usol(*)

!     indprf         i    indication if kprob part f has been filled (>0)
!                         or not (0)
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     nnodes         i    Number of nodes in array nodes
!     nodes          i    Contains the node numbers of the nodes that must
!                         be cleared
!     nunkp          i    Maximum number of unknowns in a point
!     usol          i/o   Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nodp, istart, inunkp, k
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     inunkp         Number of degrees of freedom in point
!     istart         First position used at a part
!     k              Counting variable
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'iniclearboun' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from iniclearboun'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( indprf==0 ) then

!     --- indprf = 0, number of unknowns per point is constant

         do i = 1, nnodes

            nodp = nodes(i)  ! node number
            istart = (nodp-1)*nunkp
            do k = 1, nunkp
               usol(istart+k) = 0d0
            end do  ! k = 1, nunkp

         end do  ! i = 1, nnodes

      else

!     --- indprf>0, number of unknowns per point is variable

         do i = 1, nnodes

            nodp = nodes(i)  ! node number
            istart = kprobf(nodp)
            inunkp = kprobf(nodp+1)-istart
            do k = 1, inunkp
               usol(istart+k) = 0d0
            end do  ! k = 1, nunkp

         end do  ! i = 1, nnodes

      end if  ! ( indprf==0 )

1000  call erclos ( 'iniclearboun' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End iniclearboun'

      end if  ! ( debug )

      end
