      subroutine inicreatvec1 ( ibuffr, kmesh, kprob, isolut, name )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-02-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create space for a vector with one degree of freedom per point
!
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), kmesh(*), kprob(*), isolut(*)
      character (len=*) name

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     isolut        i/o   Standard SEPRAN array containing information of
!                         the vector to be created
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     name           i    Name of array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer itypvc, ivec, npoint
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     npoint         Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI080         Create permanent space in array BUFFER
!     INISTR         find structure of vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inicreatvec1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inicreatvec1'

      end if
      if ( ierror/=0 ) go to 1000

!     --- First find the vector of special structure with one degree
!         of freedom per point

      itypvc = 0
      call inistr ( 1, kprob, itypvc, ivec, 1 )
      if ( ierror/=0 ) go to 1000

!     --- Next we create the vector

      npoint = kmesh(8) ! number of nodes
      isolut(2) = 115  !vector of special structure
      isolut(3) = 0    !real vector
      isolut(4) = ivec    !vector with sequence number ivec
      isolut(5) = npoint    !number of defrees of freedom
      call ini080 ( ibuffr, isolut(1), npoint, name )

1000  call erclos ( 'inicreatvec1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inicreatvec1'

      end if

      end
