      subroutine inicurvusp1 ( kmesh, coor, userpoint, &
                               maxusp, numusp, ix, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-11-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all user points along ix axis with coordinates smaller than
!     that of userpoint and more than 0
!     order these user points in increasing sequence
!     1d case
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     user_point
!     1d
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), userpoint(0:*), maxusp, numusp, ix, ndim
      double precision coor(ndim,*)

!     coor           i    contains he coordinates of the user points
!     ix             i    Defines direction of axis
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     maxusp         i    Declared length of array userpoint
!     ndim           i    Dimension of the space.
!     numusp         o    Number of entries in array userpoint
!     userpoint     i/o   Contains the user point numbers along the
!                         axis sorted in ascending order
!                         userpoint(0) must contain userpoint at input
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nuspnt, i, iseq(maxusp), iwork(maxusp)
      double precision xcoor, x, work(maxusp)
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     iseq           Work array to store sequence numbers
!     iwork          integer work array to copy aray
!     nuspnt         Number of user points
!     work           Work array containing the coordinates of the points
!     x              x co-ordinate of point
!     xcoor          x-coordinate of userpoint
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRSORT         Sort real array for increasing sequence
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inicurvusp1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inicurvusp1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nuspnt = kmesh(10)       ! number of user points
      xcoor = coor(ix,userpoint(0))  ! x-coordinate of userpoint

!     --- Loop over all user points

      numusp = 0
      do i = 1, nuspnt

         x = coor(ix,i)

         if ( x>0d0 .and. x<xcoor ) then

!        --- User point found
!            Store number inn userpoint and coordinate in work

            numusp = numusp+1
            call inichkarlen ( numusp, maxusp, 'userpoint' )
            if ( ierror/=0 ) go to 1000
            userpoint(numusp) = i
            work(numusp) = x

         end if  ! ( x>0d0 .and. x<xcoor )

      end do  ! i = 1, nuspnt

      if ( numusp>1 ) then

!     --- More than one user point found: sort

         call prsort ( work, iseq, numusp )
         iwork(1:numusp) = userpoint(1:numusp)

         do i = 1, numusp
            userpoint(i) = iwork(iseq(i))
         end do  ! i = 1, numusp

      end if  ! ( numusp>1 )

1000  call erclos ( 'inicurvusp1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inicurvusp1'

      end if  ! ( debug )

      end
