      subroutine inidcopbuf ( buffer, ibuffr, mmindex, array, &
                              length, namear)
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 22-11-2011
!
!   copyright (c) 2011-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Activate or create the real array referred to by the memory management
!     index mmindex and copy the contents of array to buffer
!     The inverse of this subroutine is inidbufcop
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     memory_management
!     user_vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), mmindex, length
      double precision buffer(*), array(*)
      character (len=*) namear

!     array          i    array to be copied
!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     length         i    length of the array in words
!     mmindex        i    Memory management sequence number of array to be
!                         copied
!     namear         i    Name of array
!                         This name is used in the table if the array did not
!                         yet exist
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipindex
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ipindex        Starting address of array with memory management sequence
!                    number mmindex in IBUFFR
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidgttext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI080         Create permanent space in array BUFFER
!     INIDGTTEXT     Compute pointer of array in BUFFER
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inidcopbuf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inidcopbuf'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- create array

      call ini080 ( ibuffr, mmindex, length, namear )

!     --- compute starting address

      ipindex = inidgttext ( mmindex, namear )

!     --- Copy array

      buffer(ipindex:ipindex+length-1) = array(1:length)

1000  call erclos ( 'inidcopbuf' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inidcopbuf'

      end if  ! ( debug )

      end
