      subroutine inidelmatr ( ibuffr, matr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-06-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Delete all parts of a matrix from array IBUFFR
!     If the LU decomposition is stored that decomposition is also deleted
! **********************************************************************
!
!                       KEYWORDS
!
!     delete
!     matrix
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), matr(5)

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     matr          i/o   Standard SEPRAN array, containing information of the
!                         large matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Help parameter to store a variable temporarily
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI066         delete temporary array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      95   array matr has not been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inidelmatr' )
      if ( matr(2).eq.103 ) then

!     --- matrix is of type matrix

         i = abs(matr(4))
         if ( i.eq.matr(3) ) then
            matr(3) = 0
         else
            call ini066 ( ibuffr, matr(3) )
         end if
         if ( i.gt.0 ) call ini066 ( ibuffr, i )
         matr(4) = 0
         if ( matr(5).gt.0 ) call ini066 ( ibuffr, matr(5) )

      else if ( matr(2).eq.110 ) then

!     --- matrix is diagonal matrix

         if ( matr(1).gt.0 ) call ini066 ( ibuffr, matr(1) )

      else

!     --- matrix is of wrong type

         call errint ( matr(2), 1 )
         call errsub ( 95, 1, 0, 0 )

      end if

      call erclos ( 'inidelmatr' )
      end
