      function iniedge ( inode1, inode2, nmeshl, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 25-04-2012 Fortran 90
!        version  1.0  date 07-07-2004
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find edge number corresponding to nodes inode1 and inode2
!
! **********************************************************************
!
!                       KEYWORDS
!
!     edge
!     node
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iniedge, inode1, inode2, nmeshl(*), iwork(*)

!     iniedge        o    edge number corresponding to inode1 and inode2
!     inode1         i    First node in edge
!     inode2         i    Second node in edge
!     iwork         i/o   integer work array of length nedges
!                         At start iwork must be set to zero
!                         After each call iwork is reset to 0
!     nmeshl         i    nmesh part l contains edge numbers connected to nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipstart, jpstart, i, iedge, nnode_edge, jnode_edge
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     iedge          edge sequence number
!     ipstart        Starting address in array
!     jnode_edge     number of edges connected to inode2
!     jpstart        Starting address in array
!     nnode_edge     number of edges connected to inode1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Set positions in iwork connected to inode1 to 1
!     found := false
!     For all edges iedge connected to node inode2 while not found do
!        if ( iwork(iedge) = 1 ) then
!           found := true
!     Clear iwork
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'iniedge' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from iniedge'
         write(irefwr,1) 'inode1, inode2', inode1, inode2
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Positions in iwork corresponding to edges connected to
!         inode1 are set to 1

      ipstart = nmeshl(inode1)       ! start position of inode1 in nmeshl
      nnode_edge = nmeshl(ipstart)   ! number of edges connected to inode1
      iwork(nmeshl(ipstart+1:ipstart+nnode_edge)) = 1

      if ( debug ) then
         write(irefwr,1) 'ipstart, nnode_edge', ipstart, nnode_edge
         call prinin ( nmeshl(ipstart+1), nnode_edge, 'edges' )
      end if  ! ( debug )

!     --- The edge is found by finding the common edge in the set
!         of all edges connected to inode2

      jpstart = nmeshl(inode2)     ! start position of inode2 in nmeshl
      jnode_edge = nmeshl(jpstart) ! number of edges connected to inode2
      if ( debug ) then
         write(irefwr,1) 'ipstart, jnode_edge', jpstart, jnode_edge
         call prinin ( nmeshl(jpstart+1), jnode_edge, 'edges' )
      end if  ! ( debug )

      iedge = 0
      do i = 1, jnode_edge
         iedge = nmeshl(jpstart+i)
         if ( iwork(iedge)==1 ) go to 100
      end do
      iedge = 0

!     --- iwork is cleared

100   iwork(nmeshl(ipstart+1:ipstart+nnode_edge)) = 0

      iniedge = iedge

1000  call erclos ( 'iniedge' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End iniedge'

      end if

      end
