      subroutine inielgrpsurfs ( iinput, ielgrps, numelgrps, elgrpsurfs, &
                                 numelgrpsurfs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-11-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find surface numbers corresponding to the element groups stored in ielgrps
!     This is only possible if the element group consists of surface elements
! **********************************************************************
!
!                       KEYWORDS
!
!     element_group
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cmeshf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), numelgrps, ielgrps(numelgrps), elgrpsurfs(*), &
              numelgrpsurfs

!     elgrpsurfs     o    Array with surface numbers corresponding to ielgrp
!     ielgrps        i    Array containing the element groups
!     iinput         i    integer input array for subroutine mesh
!     numelgrps      i    Number of element groups in ielgrps
!     numelgrpsurfs  o    number of surfaces in elgrpsurfs
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinp, iinp4, ilast, i, j, ifirst, jelgrp, nsurel, k, &
              ielgrp, nlines
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     ielgrp         element group number
!     ifirst         First surface number
!     iinp           last position used in array iinput
!     iinp4          first pos. iinput definition surface elements
!     ilast          Last surface number
!     j              Counting variable
!     jelgrp         Actual element group number
!     k              Counting variable
!     nlines         number of parts where line elements are created
!     nsurel         number of parts where surface elements are created
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      67   ielgrp has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inielgrpsurfs' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inielgrpsurfs'
         call prinin ( ielgrps, numelgrps, 'ielgrps' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nlines = iinput(6)      ! number of line elements
      nsurel = iinput(7)      ! number of surface elements
      iinp4 = intarmsh(12)    ! first pos. iinput definition surface elements
      if ( debug ) write(irefwr,1) 'nsurel, iinp4', nsurel, iinp4
      numelgrpsurfs = 0  ! number of surfaces stored

!     --- For all element groups stored in ielgrps do

      do k = 1, numelgrps

         ielgrp = ielgrps(k)   ! element group number

         if ( ielgrp<nlines .or. ielgrp>nsurel+nlines ) then

!        --- Error ielgrp out of range

            call errint ( ielgrp, 1 )
            call errint ( nsurel, 2 )
            call errsub ( 67, 2, 0, 0 )
            go to 1000

         end if  ! ( ielgrp <1 .or. ielgrp>nsurel )

!        --- Jump over first ielgrp-1 element groups

         iinp = iinp4       ! start information of line elements

         do i = 1, nsurel

            jelgrp = iinput(iinp+1)
            if ( debug ) write(irefwr,1) 'i, jelgrp', i, jelgrp
            if ( jelgrp>ielgrp ) then

!           --- jelgrp>ielgrp: finished

               go to 500

            else if ( jelgrp==ielgrp ) then

!           --- jelgrp = ielgrp, store surfaces

               ifirst = iinput(iinp+2)
               ilast = iinput(iinp+3)
               if ( debug ) &
                  write(irefwr,1) 'ifirst, ilast', ifirst, ilast
               do j = ifirst, ilast
                  numelgrpsurfs = numelgrpsurfs+1
                  elgrpsurfs(numelgrpsurfs) = j
               end do  ! j = ifirst, ilast

            end if  ! ( jelgrp>ielgrp )
            iinp = iinp+4
         end do  ! i = 1, nsurel

500   end do  ! k = 1, numelgrps

1000  call erclos ( 'inielgrpsurfs' )
      if ( debug ) then

!     --- Debug information

         call prinin ( elgrpsurfs, numelgrpsurfs, 'elgrpsurfs' )
         write(irefwr,*) 'End inielgrpsurfs'

      end if  ! ( debug )

      end
