      subroutine iniext ( ibuffr, index, lenext )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 21-11-2012 Long integers
!        version  2.0  date 01-04-1999 Extra parameter ibuffr
!        version  1.3  date 24-07-1998 include commons
!
!   copyright (c) 1995-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Extend the length of an existing integer array
!     in buffer array ibuffr
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons3f90'
      include 'SPcommon/cbuffrf90'
      include 'SPcommon/carrayf90'
      include 'SPcommon/cdebugf90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index, lenext, ibuffr(*)

!     ibuffr    i/o   Integer buffer array in which all large data is stored
!     index      i    Integer variable containing the sequence number of the
!                     array to be processed
!     lenext     i    Extra length required for array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipcopy, ipindx, iplast, ipstar
      integer iextra, isave, itemp, k, lennew, lenold, i

!     i         General loop variable
!     iextra    Extra space needed at both sides of the array for control
!               integers
!     ipcopy    Starting address in IBUFFR of new array
!     ipindx    Help variable to store a starting address
!     iplast    Ending address of array in IBUFFR
!     ipstar    Computed starting position in array IBUFFR
!     isave     Help variable to store the value of index
!     itemp     Temporary infor entry of array
!     k         General loop variable
!     lennew    New length of array in integers
!     lenold    Old length of array in integers
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) iniget
      integer inilen

!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     INI060  Reactivate existing array in IBUFFR
!     INI066  delete temporay array
!     INI081  Create permanent space in array IBUFFR
!     INIGET  Compute pointer of array in IBUFFR
!     INILEN  Compute length of array in IBUFFR
!     INSTOP  Stop the program
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1557   lenext<=0
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'iniext' )
      if ( idebug==0 ) then
         iextra = 0
      else
         iextra = intlen
      end if
      if ( lenext<=0 ) then

!     --- length<=0, error

         call errint ( lenext, 1 )
         call errsub ( 1557, 1, 0, 0 )

      end if
      call ini060 ( ibuffr, index )
      lenold = inilen(index)
      lennew = lenext + lenold
      if ( ierror/=0 ) go to 1000

!     --- Compute space after old array

      ipstar = iniget(index)
      iplast = ibfree-iextra-1
      do k = 1, iinfor
         ipindx = infor(k)
         if ( ipindx>ipstar ) iplast = min(iplast,ipindx-iextra-1)
      end do

      if ( lennew+iextra<iplast-ipstar ) then

!     --- sufficient space is available

         infor1(1,index) = lennew
         kbuffr = max(kbuffr,ipstar+lennew+2*iextra)
         if ( idebug>1 ) ibuffr(ipstar+lenold) = iinfin
         if ( idebug>0 ) ibuffr(ipstar+lennew) = index

      else

!     --- not enough space available
!         create new array with new length

         itemp = 0
         call ini081 ( ibuffr, itemp, lennew , 'itemp' )

!        --- copy old array to new array

         ipstar = iniget(index)
         ipcopy = iniget(itemp)
         if ( ierror/=0 ) go to 1000
         if ( ipcopy>ipstar ) then

!        --- Copy forwards

             do i = 0, lenold-1
                ibuffr(ipstar+i) = ibuffr(ipcopy+i)
             end do

         else if ( ipcopy<ipstar ) then

!        --- Copy backwards

             do i = lenold, 1, -1
                ibuffr(ipstar+lenold-i) = ibuffr(ipcopy+lenold-i)
             end do
         end if
         isave = index
         call ini066 ( ibuffr, isave )

!        --- Reset index and control integers

         infor(index) = ipcopy
         infor1(1,index) = lennew
         infor1(2,index) = 10
         infor(itemp) = 0
         infor1(1,itemp) = 0
         infor1(2,itemp) = 0
         if ( iinfor==itemp ) iinfor = iinfor - 1
         if ( idebug>0 ) then
            ibuffr(ipcopy-1) = index
            ibuffr(ipcopy+lennew) = index
         end if

      end if

1000  call erclos ( 'iniext' )
      if ( ierror/=0 ) call instop
      end
