      function inifirstelgrp ( kprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-11-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find first element group that has a non-zero type number
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_group
!     type_number
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inifirstelgrp, kprob(*)

!     inifirstelgrp  o    first element group that has a non-zero type number
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!                         In this case kprob must refer to problem iprob, &
!                         hence in the call one has to use kprob(ipkprb)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nelgrp, kprobc, itype, i
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     itype          Type number of element
!     kprobc         Starting address of KPROB part c in KPROB
!     nelgrp         Number of element groups
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inifirstelgrp' )
      inifirstelgrp = 0
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inifirstelgrp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kprobc = kprob(16)   ! starting address kprob part c
      nelgrp = kprob(8)    ! Number of element groups

      do i = 0, nelgrp-1
         itype = kprob(kprobc+i)
         if ( itype>0 ) then

!        --- type number>0 found, return

            inifirstelgrp = i+1
            go to 1000

         end if  ! ( itype>0 )

      end do  ! i = 1, nelgrp

1000  call erclos ( 'inifirstelgrp' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inifirstelgrp'

      end if  ! ( debug )

      end
