      subroutine inifpel ( kmesh, ielgrp, nfapel )
! ======================================================================
!
!        programmer    Frans van de Vosse
!        version  1.10 date 22-05-2014 Extension with shape 22
!        version  1.9  date 10-02-2012 use getnpelmelgrp
!        version  1.8  date 27-11-2011 use function getshapeelgrp
!
!   copyright (c) 1998-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fetch the number of faces per element in elementgroup ielgrp
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_group
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), ielgrp, nfapel

!     ielgrp     i    element group number
!     kmesh      i    Standard SEPRAN array containing information of the mesh
!     nfapel     o    number of faces in one element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ishape, dfapel(18)

!     dfapel     number of faces for each element type
!     ishape     Type number of elements in the mesh or element group
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getshapeelgrp

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     GETSHAPEELGRP  Determine type number for an element
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data dfapel / 0, 0, 1, 1, 1, &
                    1, 1, 1, 1, 1, &
                    4, 4, 6, 6, 4, &
                    4, 6, 4 /
! ======================================================================
!
      call eropen ( 'inifpel' )

!     --- Fetch the element type

      ishape = getshapeelgrp ( kmesh, ielgrp )
      if ( ishape<=0 .or.ishape==109 .or. &
           ishape>=50 .and. ishape<=60 .or. &
           ishape>70 .and. ishape<=80 .or. &
           ishape==134 ) then
         nfapel = 0
      else if ( 1<=ishape .and. ishape<=18 ) then
         nfapel = dfapel(ishape)
      else if ( ishape>=20 .and. ishape<=22 ) then
         nfapel = 5
      else if ( ishape<=0 ) then
         nfapel = -1
      else

!     --- Shape number not implemented

         call errint ( ishape, 1 )
         call errsub ( 800, 1, 0, 0 )
         nfapel = -1

      end if

      call erclos ( 'inifpel' )

      end
