      subroutine iniinfsurf ( kmeshn, istart, inpelm, inelem, nsurfs, &
                              isurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-01-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find starting address of surface isurnr in array kmeshn (minus 1)
!     Also find number of element and number of points per element in isurnr
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     element
!     node
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshn(*), istart, inpelm, inelem, nsurfs, isurnr

!     inelem         o    Number of elements in surface isurnr
!     inpelm         o    number of points per element in surface isurnr
!     istart         o    Last position in kmeshn before surface isurnr
!     isurnr         i    Surface sequence number
!     kmeshn         i    KMESH part n, contains nodal point numbers of the
!                         surfaces
!     nsurfs         i    number of surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'iniinfsurf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from iniinfsurf'

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

!     --- Find starting address minus 1

      istart = 2*nsurfs
      do i = 1, isurnr-1
         istart = istart+kmeshn(i)*kmeshn(i+nsurfs)
      end do  ! i = 1, isurnr-1

!     --- Find inpelm and inelem

      inpelm = kmeshn(isurnr)
      inelem = kmeshn(isurnr+nsurfs)

1000  call erclos ( 'iniinfsurf' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End iniinfsurf'

      end if

      end
