      subroutine iniipkc ( kmesh, ielem, ipkc )
! ======================================================================
!
!        programmer    Frans van de Vosse
!        version  1.2  date 10-02-2012 use getnpelmelgrp
!        version  1.1  date 27-11-2011 use function getnumelmelgrp
!        version  1.0  date 19-08-1998
!
!   copyright (c) 1998-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fetch starting adress in kmesh part c of an element
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), ipkc, ielem

!     ielem      i    Element number
!     ipkc       i    Pointer in kmesh part c
!     kmesh      i    Standard SEPRAN array containing information of the mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kelgrp, ielgrp, icount, nnopel, nelm

!     icount     number of unknowns per element
!     ielgrp     element group number
!     kelgrp     sum over all standard elements of number of points in standard
!                elements
!     nelm       number of elements along a curve
!     nnopel     Number of nodes per element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getnumelmelgrp, getnpelmelgrp, getelgrpelem

!     GETELGRPELEM   Find element group corresponding to element ielem1
!     GETNPELMELGRP   Determine the number of nodes in an element
!     GETNUMELMELGRP  Determine number of elements in a group
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      icount = 0
      ipkc = 1

!     --- Fetch the element group of ielem

      kelgrp = getelgrpelem ( kmesh, ielem )

!     --- Loop over preceeding element groups

      do ielgrp = 1, kelgrp-1

!     --- Fetch number of elements in this group

         nelm = getnumelmelgrp ( kmesh, ielgrp )

!        --- Fetch the number of nodes per element in this group

         nnopel = getnpelmelgrp ( kmesh, ielgrp )

!        --- count elements and update pointer

         icount = icount + nelm
         ipkc = ipkc + nelm*nnopel
      end do

!     --- Fetch the number of nodes per element in the current group

      nnopel = getnpelmelgrp ( kmesh, ielgrp )

!     --- count elements and update pointer

      nelm = ielem - icount - 1
      ipkc = ipkc + nelm*nnopel

      end
