      function inivec ( ivecin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-05-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with vector number corresponding to ivecin
!     In case of an array of vectors the vector number corresponding to
!     the loop is computed
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     number
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cseqnrf90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cpost5f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inivec, ivecin

!     inivec         o    Return value (reduced vector number)
!     ivecin         i    Original vector number containing extra information
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivectr, numarrays
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ivectr         intermediate vector number
!     numarrays      Number of indexed vectors in vector array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREDUCEVEC   Compute the reduced value of ivecin which indicates the
!                    first sequence number of a (series of) vectors, as well as
!                    the dimension of the series
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3141   vector belongs to an array of vectors but no loop active.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inivec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inivec'
         write(irefwr,1) 'ivecin', ivecin
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call inireducevec ( ivecin, ivectr, numarrays )

      if ( numarrays>1 ) then

!     --- dimension of array > 1

         if ( iseqarray>0 .and. iseqarray<=numarrays ) then

!        --- iseqarray in correct range

            inivec = ivectr+iseqarray-1

         else

!        --- iseqarray not in correct range

            call errint ( ivectr, 1 )
            call errint ( numarrays, 2 )
            call errint ( iseqarray, 3 )
            call errchr ( names(ivectr), 1 )
            call errsub ( 3141, 3, 0, 1 )
            inivec = ivectr

         end if  ! ( iseqarray>0 .and. iseqarray<=numarrays )

      else

!     --- not an array

         inivec = ivectr

      end if  ! ( numarrays>1 )

1000  call erclos ( 'inivec' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'inivec, ivectr, iseqarray, numarrays', &
                          inivec, ivectr, iseqarray, numarrays
         write(irefwr,*) 'End inivec'

      end if  ! ( debug )

      end
