      function intx00 ( code, ncode, nncode, ierpri, ncodmx )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.8  date 11-05-2014 New call to intx00keyw
!        version  2.7  date 08-12-2013 Extension with integer scalars
!        version  2.6  date 10-03-2008 New way of defining scalars + vectors
!
!   copyright (c) 1990-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     May be used to check whether a text read by ASKNXT
!     and stored in common CSEPCH, belongs to the table CODE
!     If this text belongs to CODE INTX00 returns with the sequence number
!     of this text in CODE, otherwise INTX00 gets the value 0
!     The number of characters that must be compared is stored in the
!     table NCODE
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer intx00, nncode, ncode(nncode), ierpri, ncodmx(nncode)
      character (len=*) code(nncode)

!     code           i    Table containing the texts to be compared
!                         nncode texts have been stored in code
!     ierpri         i    Indicates if intx00 must give an error message (1) or
!                         not (0)
!     intx00         o    Gives index number of the text that matches the code
!                         table
!                         So if string(1:ncode(i)) = code(1:ncode(i)), intx00
!                         gets the value i
!                         If none of the texts match, intx00 gets the value 0
!     ncode          i    Table containing the number of significant characters
!                         in the keyword table code
!     ncodmx         i    Table containing the maximal number of  characters in
!                         the keyword table code
!     nncode         i    Number of entries in code
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, numchar, itype(nncode), iposut, nout, icountkey(1)
      double precision tabout(1), table(nncode), tabmax(nncode)
      character (len=80) keyword
      logical debug, readnx

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              General loop variable
!     icountkey      dummy
!     iposut         dummy
!     itype          dummy
!     keyword        Text containing the string without numbers
!     nout           dummy
!     numchar        Number of characters in keyword
!     readnx         dummy
!     table          dummy
!     tabmax         dummy
!     tabout         dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRSUB         Error messages
!     INTX00KEYW     Check if string read is (main) keyword
!     PRININ         print 1d integer array
!     PRINSTR        Print an array of strings
!     READSPLITKEYW  check whether the keyword contains numbers
!                    If so split the keyword into a part without number and
!                    a number
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     648   string could not be recognized
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx00' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx00'
         write(irefwr,1) 'nncode, lastitem', nncode, lastitem
         call prinin ( ncode, nncode, 'ncode' )
         call prinstr ( code, nncode, 'code' )
         write(irefwr,*) 'string ', string
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- First we have to check whether the keyword contains numbers
!         If so we have to split the keyword into a part without number
!         and a number
!         This is necessary to check if it is a keyword, since keywords do
!         not contain numbers

      call readsplitkeyw ( numchar, keyword )
      if ( debug ) then
         write(irefwr,*) 'string ', string(1:nchar)
         write(irefwr,*) 'keyword ', keyword(1:numchar)
         write(irefwr,1) 'ierpri', ierpri
      end if  ! ( debug )

      itype = 0
      nout = 1
      call intx00keyw ( code, ncode, nncode, ncodmx, intx00, &
                        numchar, keyword, &
                        itype, nout, tabout, iposut, icountkey, &
                        table, readnx, tabmax )

      if ( debug ) &
         write(irefwr,1) 'intx00, ierpri, lastitem', &
                          intx00, ierpri, lastitem

      if ( intx00==0 .and. ierpri==1 ) then

!     --- Print error message

         call errchr ( string(1:nchar), 1 )
         do i = 1, nncode
            call errchr ( code(i)(1:ncodmx(i)), 1+i )
         end do
         call errsub ( 648, 0, 0, nncode+1 )
         call errcrd

      end if

1000  call erclos ( 'intx00' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'intx00, lastitem', intx00, lastitem
         write(irefwr,*) 'End intx00'

      end if

      end
