      subroutine intx0208 ( code, icode, numberkeyw, valuekeyw, &
                            tabmin, tabmax, readnx, finished )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 15-09-2008 Allow constants
!        version  1.0  date 17-01-2005
!
!   copyright (c) 2005-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword = number repeatedly
!     for subroutine intx02ext
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cseprdf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icode, numberkeyw
      character (len=*) code(*)
      double precision tabmin(*), tabmax(*), valuekeyw(*)
      logical finished, readnx

!     code           i    Table with keywords
!                         n texts have been stored in code
!     finished       i    Indicates if the loop has finished (true) or not
!                         (false)
!     icode          i    Position number of keyword found in table code
!     numberkeyw    i/o   Contains the number of values stored in array
!                         valuekeyw
!     readnx        i/o   Indicates if the next item has not been read (true)
!                         or has been (false)
!     tabmax        i/o   Contains maximum values allowed in table
!     tabmin        i/o   Contains minimum values allowed in table
!     valuekeyw     i/o   Array to store all values corresponding to one
!                         specific keyword
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERREAL         Put real in error message
!     ERRSUB         Error messages
!     READREALCONS   Check if last string read corresponds to constant
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1869   No number found after keyword
!    1871   Real number read out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx0208' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx0208'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call asknxt

!     --- in case of a string, check if it is a constant

      if ( ityprd==4 ) call readrealcons

      if ( debug ) &
         write(irefwr,1) 'ityprd, realrd', ityprd, realrd

      if ( ityprd<=1 .or. ityprd>=4 ) then

!     --- Not a real number found

         call errchr ( code(icode), 1 )
         call errsub ( 1869, 0, 0, 1 )
         call errcrd
         finished = .true.

      else

!     --- integer or real found, store

         if ( realrd<tabmin(icode) .or. &
              realrd>tabmax(icode) ) then
            call erreal ( realrd, 1 )
            call erreal ( tabmin(icode), 2 )
            call erreal ( tabmax(icode), 3 )
            call errsub ( 1871, 0, 3, 0 )
            call errcrd
         end if
         numberkeyw = numberkeyw+1
         valuekeyw(numberkeyw) = realrd
         readnx = .true.

      end if

1000  call erclos ( 'intx0208' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intx0208'

      end if

      end
