      subroutine intx0209 ( code, icode, numberkeyw, valuekeyw, &
                            tabmax, readnx, finished, minusg )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 03-10-2008 Allow constants
!        version  1.1  date 10-03-2008 Extra debug
!        version  1.0  date 17-01-2005
!
!   copyright (c) 2005-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword = number repeatedly
!     for subroutine intx02ext
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icode, numberkeyw, minusg
      character (len=*) code(*)
      double precision tabmax(*), valuekeyw(*)
      logical finished, readnx

!     code           i    Table with keywords
!                         n texts have been stored in code
!     finished       i    Indicates if the loop has finished (true) or not
!                         (false)
!     icode          i    Position number of keyword found in table code
!     minusg         i    Indicates if a minus sign has been found (-1) or
!                         not (1)
!     numberkeyw    i/o   Contains the number of values stored in array
!                         valuekeyw
!     readnx        i/o   Indicates if the next item has not been read (true)
!                         or has been (false)
!     tabmax        i/o   Contains maximum values allowed in table
!     valuekeyw     i/o   Array to store all values corresponding to one
!                         specific keyword
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivalrd, ivalmax
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ivalmax        Maximum integer value
!     ivalrd         Integer value read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READREALCONS   Check if last string read corresponds to constant
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     574   Not an integer number found after the keyword
!    1870   Integer number read out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx0209' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx0209'
         write(irefwr,1) 'minusg, lastitem, ityprd, lastitembef', &
                          minusg, lastitem, ityprd, lastitembef
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call asknxt
      if ( ityprd==4 ) call readrealcons
      if ( debug ) &
         write(irefwr,1) 'ityprd, realrd', ityprd, nint(realrd)

      if ( ityprd/=2 ) then

!     --- Not an integer number found

         call errchr ( code(icode), 1 )
         call errsub ( 574, 0, 0, 1 )
         call errcrd
         finished = .true.

      else

!     --- integer found, store

         ivalrd = nint(realrd)
         ivalmax = nint(tabmax(icode))
         if ( ivalrd<-ivalmax .or. &
              ivalrd>ivalmax ) then
            call errint ( ivalrd, 1 )
            call errint ( -ivalmax, 2 )
            call errint ( ivalmax, 3 )
            call errchr ( code(icode), 1 )
            call errsub ( 1870, 3, 0, 1 )
            call errcrd
         end if
         numberkeyw = numberkeyw+1
         valuekeyw(numberkeyw) = minusg*realrd
         readnx = .true.

      end if

1000  call erclos ( 'intx0209' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd
         write(irefwr,*) 'End intx0209'

      end if

      end
