      subroutine intx0227 ( icountkey, icode, table, nout, tabout, &
                            tabmin, code, iposut )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-11-2014 Check tabout
!        version  1.0  date 17-01-2005
!
!   copyright (c) 2005-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword followed bya series of values or value = v, &
!     or func = i
!     for subroutine intx02ext
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iposut, nout, icode, icountkey(*)
      character (len=*) code(*)
      double precision table(*), tabmin(*), tabout(nout)

!     code           i    Table with keywords
!     icode          i    Position number of keyword found in table code
!     icountkey     i/o   Array that counts the number of times a keyword has
!                         been found
!     iposut        i/o   last position used in array tabout
!     nout           i    Number of entries in tabout
!     table          i    Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabmin         i    Contains minimum values allowed in table
!     tabout        i/o   Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ifunc
      double precision value(1)
      logical debug, compl

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     ifunc          Parameter k after FUNC = &
!     value          Array containing the real or complex value read
!                    (2 positions)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READFN         Read items of the shape number, VALUE=number or FUNC=i
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1468   Array tabout too small
!    1628   Keyword found more than one for the same problem
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx0227' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx0227'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      compl = .false.
      if ( icountkey(icode)/=0 ) then
          call errchr ( code(icode), 1 )
          call errsub ( 1628, 0, 0, 1 )
      else
         icountkey(icode) = icountkey(icode)+1
      end if
      table(icode) = iposut+1

      do i = 1, nint(tabmin(icode))
         call readfn ( ifunc, value, compl )
         if ( iposut+2>nout ) then
            call errint ( nout, 1 )
            call errsub ( 1468, 1, 0, 0 )
            go to 1000
         end if  ! ( iposut>nout )
         tabout(iposut+1) = ifunc
         tabout(iposut+2) = 0d0
         if ( ifunc==0 ) tabout(iposut+2) = value(1)
         iposut = iposut + 2
      end do

1000  call erclos ( 'intx0227' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intx0227'

      end if

      end
