      subroutine intx02minus ( readnx, minusg )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-02-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check for the presence of minus sign and closing brackets
!     Closing brackets are skipped
!     Help subroutine for intx02ext1
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer minusg
      logical readnx

!     minusg         o    Indicates if a minus sign has been found (-1) or
!                         not (1)
!     readnx         o    Indicates if the next item has not been read (true)
!                         or has been (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx02minus' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx02minus'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      readnx = .true.  ! next item must be read
      if ( ityprd==6 ) then

!     --- minus sign found

         minusg = -1
         call asknxt  ! read next item
         if ( debug ) then
            write(irefwr,1) 'ityprd, istop, lastitem/2a', &
                             ityprd, istop, lastitem
            if ( ityprd==4 ) &
               write(irefwr,*) 'string ', string(1:nchar)
         end if  ! ( debug )

      end if

      do while ( ityprd==7 )

!     --- Closing bracket found

         call asknxt  ! read next item
         if ( ierror/=0 ) go to 1000
         if ( istop/=2 .and. lastitem==nitems .and. ityprd==7 ) &
            ityprd = 1  ! read next record

      end do  ! while ( ityprd==7 )

1000  call erclos ( 'intx02minus' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intx02minus'

      end if  ! ( debug )

      end
