      subroutine intx02next ( code, ncode, nncode, itype, table, codemn, &
                              nmcode, iretur, tabmin, tabmax, &
                              tabout, nout, strout, nstrng, iseqkeyw, &
                              nseqkeyw, icode, keywordfound, minusg, &
                              icountkey, pointar, finished, ihelp, &
                              maxrepeat, numberkeyw, readnx, &
                              maxmultiple, valuekeyw, iposut, kposut, &
                              lastitemsav, errorcode )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.2  date 27-03-2015 Extension with type 45
!        version  3.1  date 10-01-2015 Allow expressions
!        version  3.0  date 18-03-2014 Extra parameter errorcode
!
!   copyright (c) 2012-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Keyword found, read next item
!     Help subroutine for intx02ext1
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nncode, ncode(nncode), itype(*), nmcode(*), &
                             nstrng, icode, maxrepeat, maxmultiple, &
                             pointar(*)
      integer, intent(inout) :: iretur, nout, iseqkeyw(*), kposut, &
                                nseqkeyw, errorcode, minusg, iposut, &
                                icountkey(*), ihelp(*), &
                                numberkeyw(*), lastitemsav
      character (len=*), intent(in) :: code(nncode), codemn(*)
      character (len=*), intent(inout) :: strout(nstrng)
      double precision, intent(inout) :: tabmin(nncode), tabmax(nncode), &
                                         table(nncode), tabout(nout), &
                                        valuekeyw(maxrepeat,maxmultiple)
      logical, intent(inout) :: keywordfound, finished, readnx

!     code           i    Table with keywords
!     codemn         i    Table with main keywords
!     errorcode     i/o   If 0 at input an error message is given if a keyword
!                         is not recognized
!                         If>0, the subroutine does not give an error message, &
!                         but instead returns with -1
!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
!     icode          i    Return code with respect to subroutine INTX01
!                         Indicates position in array code or code1
!     icountkey     i/o   Array that counts the number of times a keyword has been
!                         found
!     ihelp         i/o   Help array to store number of appearances of keywords
!                         that may be used more than once or number of numbers
!                         read
!     iposut        i/o   Position in array tabout
!     iretur        i/o   Return value of subroutine INTX02
!                         See subroutine INTX02
!     iseqkeyw      i/o   Array of size nseqkeyw that contains the sequence in
!                         which keywords are found in a record
!                         isekeyw(i)>0, means i-th keyword found is the
!                                         iseqkeyw(i)-th keyword in the table
!                         isekeyw(i)<0, means keyword refers to number found
!                                         -isekeyw(i) refers to position in tabout
!     itype          i    Table containing information about the numbers behind
!                         the keywords in the table code
!     keywordfound  i/o   If true a keyword has been found
!     kposut        i/o   last position used in array strout
!     lastitemsav   i/o   Help parameter to save the value of lastitem
!                         This is used in case an expression is found, to
!                         restart with the begin of the expression
!     maxmultiple    i    Declared size of array pointar
!     maxrepeat      i    Declared size of first entry of array numberkeyw
!     minusg        i/o   Indicates if a minus sign has been found (-1) or
!                         not (1)
!     ncode          i    Table containing the number of significant characters
!                         in the keyword table code
!     nmcode         i    Table containing the number of significant characters
!                         in the main keyword table codemn, including those of
!                         the extra texts
!     nncode         i    Number of entries in code
!     nout          i/o   Number of entries in tabout
!     nseqkeyw       i    Number of entries in iseqkeyw
!     nstrng         i    Length of array strout
!     numberkeyw    i/o   Contains the number of values stored in array
!                         valuekeyw
!     pointar        i    array of size n containing the sequence numbers of
!                         all keywords with respect to arrays numberkeyw and
!                         valuekeyw
!                         if 0, no reference is necessary
!     readnx        i/o   Indicates if the next item has not been read (true)
!                         or has been (false)
!     strout        i/o   Contains series of strings
!     table         i/o   Contains the numbers read for the various keywords
!     tabmax         i    Contains maximum values allowed in table
!     tabmin        i/o   Contains minimum values allowed in table
!     tabout        i/o   Contains series of numbers
!     valuekeyw     i/o   Array to store all values corresponding to one
!                         specific keyword
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq, jump, m, nmax
      logical debug, vectorfound
      character (len=240) subkey

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iseq           Sequence number
!     jump           if 0 no error has been found, if 1 array tabout
!                    is too small
!     m              Number of integers read
!     nmax           maximum number of numbers allowed
!     subkey         Sub (second) keyword
!     vectorfound    If true the last item read is a vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INTX0201       Read keyword or keyword = number
!     INTX0205       Read keyword followed by one or two keywords
!     INTX0206       Read keyword = text
!     INTX0208       Read keyword = number repeatedly
!     INTX0209       Read keyword = number repeatedly
!     INTX0210       Read keyword followed by sequence number and number
!                    or number
!     INTX0211       Read keyword followed by a series of positive integers
!     INTX0215       Read keyword followed by an integer, repeatedly
!     INTX0216       Read keyword followed by a value or value = v, or func = i
!     INTX0217       Read keyword followed by a series of series of numbers
!     INTX0218       Interpret part of input record for subroutine intx02 for
!                    the possibilities itype = 14, 18 and 31
!     INTX0219       Read keyword followed by a series of numbers
!     INTX0220       Read single keyword inwhich case the table must be set
!                    to -100000 or keyword followed by agiven other keyword
!                    and a series of integers
!     INTX0221       Read keyword followed by (ax+by+cz=d)
!     INTX0222       Read keyword followed by possibly one or two special
!                    keywords that are skipped and then by a series of numbers
!     INTX0223       Read keyword followed by possibly a special keyword that
!                    is skipped
!                    table(icode) is set equal to 1
!     INTX0225       Read keyword followed by a sequence number followed by
!                    a value(1) or value(1) = v, or func = i
!     INTX0227       Read keyword followed bya series of values or value = v, &
!                    or func = i
!     INTX0229       Interpret part of input record for subroutine intx02 for
!                    the possibilities itype = 29, 33 and 37
!     INTX0232       Read keyword like:
!                    surfaces (i1, i2, ... in ) = s1 to s2 or
!                    surfaces = s1 to s2, nodes (i1, i2, ... in )
!     INTX0235       Read keyword followed by a sequence number followed by
!                              a value or value = v, or func = i
!                              and again the same keyword followed  by
!                              a sequence number followed by
!                              a value or value = v, or func = i
!     INTX0238       Interpret part of input record for subroutine intx02 for
!                    the possibilities itype = 38
!     INTX0239       Interpret part of input record for subroutine intx02 for
!                    the possibilities itype = 39
!     INTX0243       Interpret part of input record for subroutine intx02 for
!                    the possibilities itype = 43
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1468   Array tabout too small
!    1469   Array strout too small
!    1628   Keyword found more than once for the same problem
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intx02next' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intx02next'
         write(irefwr,1) 'iretur', iretur
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      vectorfound = ityprd==2

      nseqkeyw = nseqkeyw+1
      iseqkeyw(nseqkeyw) = icode
      keywordfound = .true.
      jump = 0

      if ( debug ) then
        write(irefwr,1) 'nseqkeyw, icode, itype(icode), minusg', &
                         nseqkeyw, icode, itype(icode), minusg
        write(irefwr,1) 'istop, lastitem/3', istop, lastitem
      end if  ! ( debug )

      select case ( itype(icode) )

         case(:-1,7)

!        --- itype(icode) = -1,7: skip

         case(0,1,3,4,36)

!        --- itype(icode) = 0, 1, 3, 4, 36: Read keyword or
!            keyword = number

            call intx0201 ( icountkey, icode, table, nout, &
                            tabout, tabmin, tabmax, code, &
                            finished, readnx, itype, iposut, errorcode )
            if ( ierror/=0 ) go to 1000
            if ( iposut>nout ) jump = 1

         case(2)

!        --- itype(icode) = 2: set table = 1

            if ( debug ) &
               write(irefwr,*) 'itype(icode) = 2: set table = 1'
            table(icode) = 1d0
            if ( icountkey(icode)/=0 ) then
                call errchr ( code(icode), 1 )
                call errsub ( 1628, 0, 0, 1 )
            else
               icountkey(icode) = icountkey(icode)+1
            end if

         case(5,13)

!        --- itype(icode) = 5 or 13, hence the keyword must be followed by
!            one or two keywords

            call intx0205 ( icountkey, icode, table, iretur, &
                            tabmin, tabmax, code, finished, &
                            readnx, itype, codemn, nmcode )

         case(6)

!        --- itype(icode) = 6, text

            call intx0206 ( icountkey, icode, table, &
                            tabmax, code, finished, &
                            kposut, nstrng, strout, iretur )
            if ( kposut>nstrng ) then

!           --- Error: kposut too large

               call errint ( nstrng, 1 )
               call errsub ( 1469, 1, 0, 0 )

            end if  ! ( kposut>nstrng )

         case(8)

!        --- itype(icode) = 8, keyword = real, repeatedly

            iseq = pointar(icode)
            call inichkarlen ( numberkeyw(iseq)+1, maxrepeat, &
                               'valuekeyw' )
            if ( ierror/=0 ) go to 1000
            call intx0208 ( code, icode, numberkeyw(iseq), &
                            valuekeyw(1,iseq), tabmin, tabmax, &
                            readnx, finished )

         case(9)

!        --- itype(icode) = 9, keyword = positive integer, repeatedly
!            keyword may be provided with minus sign

            iseq = pointar(icode)
            call inichkarlen ( numberkeyw(iseq)+1, maxrepeat, &
                               'valuekeyw' )
            if ( ierror/=0 ) go to 1000
            call intx0209 ( code, icode, numberkeyw(iseq), &
                            valuekeyw(1,iseq), &
                            tabmax, readnx, finished, minusg )

         case(10, 12, 30, 34)

!        --- itype(icode) = 10, hence the keyword must be followed by
!            a sequence number and a number
!            or itype(icode) = 12, 30:
!            the sequence number may be not present

            call intx0210 ( icode, table, nout, tabout, iretur, &
                            tabmin, tabmax, code, finished, &
                            readnx, itype, iposut, codemn, &
                            nmcode, subkey, vectorfound )

         case(11, 28)

!        --- itype(icode) = 11 or 28, &
!            hence the keyword must be followed by
!            a series of (positive) integers

            call intx0211 ( icountkey, icode, table, nout, &
                            tabout, tabmin, tabmax, code, readnx, &
                            itype, iposut, ihelp )
            if ( iposut>nout ) jump = 1

         case(14,18,31)

!        --- itype(icode) = 14, 18, 31 hence the keyword must be followed
!            by a positive integer and possibly the same keyword with
!            another integer and so on

            call intx0218 ( icode, code, ncode, itype, table, &
                            codemn, nmcode, tabmin, tabmax, &
                            tabout, nout, jump, iposut, &
                            ihelp, finished, readnx, minusg )

         case(15)

!        --- itype(icode) = 15, hence the keyword must be followed by
!            an integer. This situation may be repeated

            call intx0215 ( icode, table, nout, tabout, minusg, &
                            tabmin, tabmax, code, finished, &
                            iposut, ihelp )
            if ( debug ) then
               write(irefwr,*) 'after intx0215'
               write(irefwr,1) 'iposut', iposut
               write(irefwr,2) 'tabout', tabout(iposut-1)
            end if  ! ( debug )
            if ( iposut>nout ) jump = 1

         case(16,26)

!        --- itype(icode) = 16, hence the keyword must be followed by
!                               a value or value = v, or func = i

            call intx0216 ( icountkey, icode, table, nout, &
                            tabout, code, itype, iposut )
            if ( iposut>nout ) jump = 1

         case(17)

!        --- itype(icode) = 17, hence the keyword must be followed by
!                               a series of series of numbers

            call intx0217 ( icountkey, icode, table, nout, &
                            tabout, tabmin, tabmax, code, &
                            finished, readnx, iposut, ihelp, &
                            nmax )
            if ( iposut>nout ) jump = 1

         case(19)

!        --- itype(icode) = 19, hence the keyword must be followed by
!                               a series of numbers

            if ( debug ) call prinrl ( tabmax, icode, 'tabmax' )
            call intx0219 ( icountkey, icode, table, nout, &
                            tabout, tabmin, tabmax, code, readnx, &
                            iposut, ihelp, nmax )
            if ( nout<nmax+1 .or. iposut>nout ) jump = 1

         case(20)

!        --- itype(icode) = 20, hence the keyword is a single keyword in
!                               which case the table must be set to
!                               -100000 or the keyword is followed by a
!                               given other keyword and a series of
!                               integers

            call intx0220 ( icountkey, icode, table, nout, &
                            tabout, tabmin, tabmax, code, readnx, &
                            nmcode, itype, iposut, ihelp, subkey, &
                            codemn )

         case(21)

!        --- itype(icode) = 21, hence the keyword is followed by
!                               (ax+by+cz=d)

            call  intx0221 ( icountkey, icode, table, nout, &
                             tabout, code, iposut )
            if ( iposut>nout ) jump = 1

         case(22, 24)

!        --- itype(icode) = 22, 24 hence the keyword is followed by
!                                  possibly one or two special keywords
!                                  that are skipped and then by a
!                                  series of numbers

            call intx0222 ( icountkey, icode, table, nout, &
                            tabout, tabmin, tabmax, code, readnx, &
                            nmcode, itype, iposut, subkey, &
                            codemn, ihelp, nmax )

         case(23)

!        --- itype(icode) = 23, hence the keyword is followed by possibly
!                               a special keyword that is skipped
!                               table(icode) is set equal to 1

            call intx0223 ( icountkey, icode, table, code, &
                            readnx, subkey, codemn, nmcode )

         case(25)

!        --- itype(icode) = 25, hence the keyword must be followed by
!                               a sequence number followed by
!                               a value or value = v, or func = i

            call intx0225 ( icode, table, nout, tabout, tabmin, &
                            tabmax, code, finished, iretur, &
                            iposut )
            if ( iposut>nout ) jump = 1

         case(27)

!        --- itype(icode) = 27, hence the keyword must be followed by
!                               a series of values or value = v, &
!                               or func = i

            call intx0227 ( icountkey, icode, table, nout, &
                            tabout, tabmin, code, iposut )
            if ( iposut+2>nout ) jump = 1

         case(29,33,37,41,42)

!        --- itype(icode) = 29, 31, 33, 41, 42
!            hence the keyword must be followed by a sequence number
!            possibly a subkeyword and then a series of integers

            call intx0229 ( icode, code, itype, table, codemn, &
                            nmcode, tabmin, tabmax, tabout, nout, &
                            jump, iposut, ihelp, finished, &
                            readnx, iretur )
            if ( jump/=0 ) jump = 1

         case(32)

!        --- type = 32, keyword like:
!            surfaces (i1, i2, ... in ) = s1 to s2 or
!            surfaces = s1 to s2, nodes (i1, i2, ... in )

            call intx0232 ( icode, table, nout, tabout, codemn, &
                            tabmin, tabmax, code, finished, &
                            readnx, iposut, ihelp, subkey, &
                            nmcode, m )
            if ( iposut+m+2>nout ) jump = 1

         case(35)

!        --- itype(icode) = 35, hence the keyword must be followed by
!                               a sequence number followed by
!                               a value or value = v, or func = i
!                               and again the same keyword followed  by
!                               a sequence number followed by
!                               a value or value = v, or func = i

            call intx0235 ( icode, table, nout, tabout, &
                            tabmin, tabmax, code, finished, &
                            iposut, iretur )
            if ( iposut>nout ) jump = 1

         case(38, 40)

!        --- itype(icode) = 38, hence the keyword must be followed by
!            an integer number or a vector name

            call intx0238 ( icountkey, icode, code, table, &
                            tabout, nout, iposut, tabmax, itype, &
                            readnx, lastitemsav )

         case(39, 45)

!        --- itype(icode) = 39, hence the keyword must be followed by
!            an integer number or a scalar name

            call intx0239 ( icountkey, icode, code, table, tabout, &
                            nout, iposut, tabmax, itype )

         case(43)

            call intx0243 ( icountkey, icode, code, table )

      end select  ! case ( itype(icode) )

      if ( jump/=0 ) then

!     --- jump#0, hence tabout too small

         call errint ( nout, 1 )
         call errsub ( 1468, 1, 0, 0 )

      end if  ! ( jump/=0 )

1000  call erclos ( 'intx02next' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'table(icode)', table(icode)
         write(irefwr,1) 'icode, iretur, lastitem, ityprd', &
                          icode, iretur, lastitem, ityprd
         write(irefwr,*) 'readnx', readnx
         write(irefwr,*) 'End intx02next'

      end if  ! ( debug )

      end
