      subroutine intxkeyword ( code, ncode, nncode, ncodmx, icode, &
                               numchar, keyword )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 26-06-2014 extra debug
!        version  1.0  date 06-02-2012
!
!   copyright (c) 2012-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if string read is (main) keyword
!     If so return with icode >0
!     All characters are compared
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cusconsf90'
      include 'SPcommon/cpost5f90'
      include 'SPcommon/cpackagef90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nncode, ncode(nncode), ncodmx(nncode), icode, numchar
      character (len=*) code(nncode), keyword

!     code           i    Table with keywords
!     icode          o    Return code
!                         Indicates position of keyword in array code
!     keyword        i    Text containing the string without numbers
!     ncode          i    Table containing the number of significant characters
!                         in the keyword table code
!     ncodmx         i    Table containing the maximal number of  characters in
!                         the keyword table code
!     nncode         i    Number of entries in code
!     numchar        i    Number of characters in keyword
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, jcode
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     jcode          Special value of icode corresponding to vector or scalar
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INTX00KEYW1    Check information of vector name read
!     READNAMEVEC    Read name of vector in common cpost5
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intxkeyword' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intxkeyword'
         write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
         write(irefwr,1) 'nchar', nchar
         write(irefwr,*) 'namevfound', namevfound
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icode = 0

      do i = 1, nncode

!     --- First check if keyword is exactly equal to a keyword

         if ( nchar==1 .and. ncode(i)==1 .and. keyword(1:1)=='V' .and. &
              namevfound .and. nitems>lastitem .and. &
              typeitems(lastitem+1)==10 ) then

!        --- V is followed by %-sign, hence keyword V found
!            nothing to be done

            icode = i
            exit

         end if  ! ( nchar==1 .and. ncode(i)==1 .and. keyword(1:1)=='V' ... )

         if ( keyword(1:numchar)==code(i)(1:ncodmx(i)) ) then

!        --- Text found

            if ( ncode(i)==1 .and. keyword(1:1)=='V' .and. &
                 namevfound ) then

!           --- Special case, keyword V, check if this is meant as
!               V of vector or V as vector name
!               Check if this is a vector name
!               If so read name of vector and jump

               call readnamevec

               if ( debug ) write(irefwr,1) 'i, ityprd', i, ityprd
               if ( ityprd==2 ) then

!              --- vector name found

                  jcode = i
                  call intx00keyw1 ( icode, jcode )
                  exit

               end if  ! ( ityprd==2 )

            end if  ! ( ncode(i)==1 .and. keyword(1:1)=='V' )

            icode = i
            lastitem = lastitembef+1
            if ( debug ) then
               write(irefwr,1) 'text found, icode:', icode
               write(irefwr,1) 'lastitem, numchar, ncodmx(i), ityprd', &
                                lastitem, numchar, ncodmx(i), ityprd
            end if  ! ( debug )
            exit

         end if  ! ( string(1:ncode(i))==code(i)(1:ncode(i)) )

      end do  ! i = 1, nncode

1000  call erclos ( 'intxkeyword' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) &
            'icode, lastitem, ityprd', icode, lastitem, ityprd
         write(irefwr,*) 'End intxkeyword'

      end if  ! ( debug )

!     --- keyitems(lastitem) = icode

      end
