module sepmodulebasefn

   type basisfn

!        --- Defines information for the basis functions for one element

      integer :: cart                ! type of coordinate system
                                     ! 1:  Cartesian
                                     ! 2:  axi-symmetric
                                     ! 3:  polar
      integer :: inorm               ! Indicates which vector in qmat is equal
                                     ! to the normal vector
      integer :: itang               ! Indicates which vector in qmat is equal
                                     ! to the first tangential vector
      integer :: jdiag               ! if 1 the basis functions
                                     ! are 0 or 1 in the quadrature points
                                     ! if  0 they are general
      integer :: kstep               ! Indication if the elements must be
                                     ! treated normally i.e. all integration
                                     ! points are stored sequentially (1)
                                     ! or that a quadratic triangle with a
                                     ! 3-point newton cotes rule must be
                                     ! considered (2)
      integer :: m                   ! number of quadrature points
      integer :: n                   ! number of basis functions
      integer :: nd                  ! dimension of basis function
      integer :: nd2                 ! number of second derivs
      integer :: nvert               ! number of vertices
      integer :: rule                ! type of quadrature rule
      integer :: rule1               ! type of first quadrature rule
      integer :: rule2               ! type of second quadrature rule
      integer :: type_basisfunc      ! type of basis function


      double precision, pointer :: dxidx(:,:,:) ! contains dxsi/dx
      double precision, pointer :: dxdxi(:,:,:) ! contains dx/dxsi
      double precision, pointer :: dxdxig(:,:,:) ! contains dx/dxsi in
                                             ! Gauss points
      double precision, pointer :: Jac(:  )  ! Jacobian
      double precision, pointer :: lam(:,:)  ! linear basis functions
      double precision, pointer :: lamder(:,:,:)! first derivatives of lambda
                                             ! in quadrature points
      double precision, pointer :: lamderc(:,:,:)! first derivatives of lambda
                                             ! in reference element
      double precision, pointer :: lamn(:,:)  ! linear basis functions
                                             ! in nodal points
      double precision, pointer :: phi(:,:)  ! basis functions
                               ! first component: seq number of basis function
                               ! second component: quadrature point
      double precision, pointer :: philoc(:,:)  ! basis functions
      double precision, pointer :: phic(:,:) ! constant part basis funcs
      double precision, pointer :: phider(:,:,:)! first derivatives of phi
                                             ! in quadrature points
                                          !third component type of derivative
      double precision, pointer :: phiderc(:,:,:)! first derivatives of phi
                                             ! in reference element
      double precision, pointer :: phidern(:,:,:)! first derivatives of phi
                                             !     in nodal points
      double precision, pointer :: phidercn(:,:,:)! first derivatives of phi
                                             ! in nodes of reference element
      double precision, pointer :: phider2(:,:,:) ! second derivatives
      double precision, pointer :: phipress(:,:) ! pressure basis functions
      double precision, pointer :: psi(:,:)  ! upwind basis functions
      double precision, pointer :: qmat(:,:,:) ! contains transformation matrix
                                             ! of global to local coordinates
      double precision, pointer :: tang(:,:) ! contains tangential vector
      double precision, pointer :: w(:)      ! weights
      double precision, pointer :: w2(:)     ! second part of weights
      double precision, pointer :: wc(:)     ! constant part weights
      double precision, pointer :: xloc(:,:) ! Local coordinates of nodes
                                         ! might be the result of transformation
      double precision, pointer :: xyz(:,:)  ! coordinates of nodes
      double precision, pointer :: xyzg(:,:) ! Gauss coordinates of nodes
      double precision, pointer :: xyzgc(:,:) ! Gauss coordinates of nodes
                                             ! in reference element
      double precision, pointer :: xyzgcn(:,:) ! coordinates of nodes
                                             ! in reference element

      logical :: fillderiv         ! if true fill derivatives
      logical :: fillderiv2        ! if true fill second derivs
      logical :: fillderivn        ! if true fill phidern
      logical :: fillgauss         ! if true fill gauss points
      logical :: fillpsi           ! if true upwind basis functions are filled
      logical :: derivconst        ! if true the derivatives of
                                   ! the basis functions are constant
      logical :: secondupw         ! if true the second order term is
                                   ! significant for upwind
      logical :: comppsi           ! if true the linear basis functions and
                                   ! their derivatives must be computed
      logical :: fillqmat          ! if true the matrix qmat must be filled
                                   ! if ndim > nd
      logical :: compqmat          ! if true the matrix qmat must be filled
      logical :: callbyderiv       ! If true the subroutine is called by deriv

!     --- Special items for shell elements

      double precision, pointer :: fibjac(:,:) ! Jacobin the quadrature points
                                        ! of the fiber coordinate system
      double precision, pointer :: qfib(:,:,:) ! the fiber coordinate system
                                        ! qfib(j,k,i)  contains the k^th
                                        ! component of the j^th basis vector
                                        ! in the i^th nodal point
      double precision, pointer :: xhat(:,:) ! the normal vector in the nodes
                                        ! xhat(i,k) contains the k^th component
                                        ! in the i^th node

      double precision :: wzeta(2)      ! weights in quadrature points
      double precision :: zeta(2)       ! zeta coordinates in quadrature points

   end type basisfn

   type (basisfn), pointer :: B
   type (basisfn), target :: B1
   type (basisfn), target :: B2
   type (basisfn), target :: B3

end module sepmodulebasefn
