module sepmoduleelm

   type element

!     --- Defines information for the element

      double precision, pointer :: a1(:,:)      ! a1 tensor
      double precision, pointer :: Bb(:,:,:,:)  ! B^b matrix (bending stiffness)
      double precision, pointer :: Bl(:,:,:,:)  ! B matrix (linear
                         ! strain-displacement matrix for nonlinear analysis)
      double precision, pointer :: Blr(:,:,:,:) ! See Bl, reduced integration
      double precision, pointer :: Blloc(:,:,:,:)  ! Local Bl
      double precision, pointer :: Bmat(:,:,:,:)! B matrix
      double precision, pointer :: Bs(:,:,:,:)  ! B^s matrix (shear stiffness)
      double precision, pointer :: Bsh(:,:,:,:,:)  ! B matrix for shell element
      double precision, pointer :: cconv(:,:)   ! special coefficient c_conv
      double precision, pointer :: coeffcp(:,:) ! copy in coeffs
      double precision, pointer :: coeffs(:,:)  ! contains coefficients in
                                                ! quadrature points (m,.)
      double precision, pointer :: coeffsnode(:,:)  ! contains coefficients in
                                                ! nodal points (n,.)
      double precision, pointer :: coor(:,:)    ! coordinates
      double precision, pointer :: detdsc(:)    ! Derivative of second invariant
      double precision, pointer :: detf(:)      ! Deformation gradient
      double precision, pointer :: detf0(:,:,:) ! dphidx reduced integration
      double precision, pointer :: detfr(:)     ! Deformation gradient
                                                ! (reduced integration)
      double precision, pointer :: dfgrad(:,:,:) ! local grad matrix
      double precision, pointer :: difftensor(:,:)  ! diffusion tensor
      double precision, pointer :: diffvec(:)   ! diffusion
      double precision, pointer :: divgrmat(:,:)! either divamt or gradmat
      double precision, pointer :: divmat(:,:)  ! divergence matrix
      double precision, pointer :: Dmat(:,:,:)  ! D matrix (or Db)
                                                ! elasticity matrix
      double precision, pointer :: Dmats(:,:,:) ! Ds matrix
                                                ! elasticity matrix for shear
                                                ! stiffness
      double precision, pointer :: dphidx(:,:,:) ! local dphidx
      double precision, pointer :: dpressmmat(:) ! element diagonal
                                                ! pressure mass matrix
      double precision, pointer :: dvelmmat(:)  ! element diagonal
                                                ! velocity mass matrix
      double precision, pointer :: etaeff(:)    ! effective viscosity
      double precision, pointer :: gamma(:,:)   ! Vector gamma for rhs
      double precision, pointer :: gradmat(:,:) ! gradient matrix
      double precision, pointer :: gradred(:,:,:) ! reduced gradient vector
      double precision, pointer :: gradv(:,:)   ! gradient vector
      double precision, pointer :: mat(:,:)     ! element matrix
      double precision, pointer :: matshift(:)  ! element matrix for laplace
                                                ! shift (diagonal)
      double precision, pointer :: mmat(:,:)    ! element mass matrix
      double precision, pointer :: pksest(:,:)  ! Piola-Kirchoff stress
                                                ! estimate
      double precision, pointer :: pksestloc(:,:)  ! local pksest
      double precision, pointer :: pksr(:,:)    ! pksest for reduced
                                                ! integration
      double precision, pointer :: pressmat(:,:) ! pressure matrix
      double precision, pointer :: rhsdiv(:)    ! right-hand side divergence
      double precision, pointer :: rhsdivtr(:)  ! transformed right-hand side
                                                ! divergence
      double precision, pointer :: rho(:)       ! density
      double precision, pointer :: secinv(:)    ! second inverse tensor
      double precision, pointer :: ser(:,:,:)   ! Tangential matrix, reduced
                                                ! integration
      double precision, pointer :: submat(:,:)  ! sub matrix of element matrix
      double precision, pointer :: submass(:,:) ! sub matrix of element mass
                                                ! matrix
      double precision, pointer :: subvec(:)    ! sub vector of element vector
      double precision, pointer :: tmpmat(:,:)  ! temp (sub) element matrix
      double precision, pointer :: tmpsmat(:,:) ! temp (sub) element matrix
      double precision, pointer :: trans(:,:)   ! either transdiv or transpress
      double precision, pointer :: transdiv(:,:) ! divergence transformation
                                                ! matrix
      double precision, pointer :: transpress(:,:) ! pressure transformation
                                                ! matrix
      double precision, pointer :: ugauss(:,:)  ! old solution in Gauss points
      double precision, pointer :: ugaussloc(:,:)  ! local old solution
                                                ! in Gauss points
      double precision, pointer :: ugaussr(:,:) ! old solution in Gauss points
                                                ! reduced integration
      double precision, pointer :: user(:)      ! contains real user input
      double precision, pointer :: uvel(:)      ! u-component
      double precision, pointer :: vec(:)       ! element vector
      double precision, pointer :: veloc(:,:)   ! velocity vector
      double precision, pointer :: wpressmmat(:,:) ! element weighted
                                                ! pressure mass matrix
      double precision, pointer :: wvec(:)      ! element weight vector
      double precision, pointer :: wr(:)        ! reduced weight vector
      double precision, pointer :: xgaussr(:,:) ! gauss points (reduced int)
      double precision, pointer :: xgaussloc(:,:) ! local gauss points

      double complex, pointer :: ccoeffs(:,:)   ! contains coefficients in
                                                ! quadrature points (m,.)
      double complex, pointer :: ccoeffsnode(:,:)  ! contains coefficients in
                                                ! nodal points (n,.)
      double complex, pointer :: cdifftensor(:,:)  ! diffusion tensor
      double complex, pointer :: cdiffvec(:)    ! diffusion
      double complex, pointer :: cgamma(:,:)    ! Vector gamma for rhs
      double complex, pointer :: cmat(:,:)      ! element matrix
      double complex, pointer :: cmat2(:,:)     ! second element matrix
      double complex, pointer :: cugauss(:,:)   ! old solution in Gauss points
      double complex, pointer :: cuvel(:)       ! u-component
      double complex, pointer :: cvec(:)        ! element vector
      double complex, pointer :: cveloc(:,:)    ! velocity vector

      integer, pointer :: ielvec(:)  ! integer element vector
      integer, pointer :: iuser(:)   ! contains integer user input
      integer, pointer :: nodes(:)   ! node numbers of element
      integer, pointer :: islold(:)  ! sepran reference to all vectors

      integer :: applyupw            ! Defines on which terms upwind is applied
                                     !  0: all terms
                                     !  1: all terms except mass matrix
                                     !  2: convection only
      integer :: choicefnmater       ! choice parameter send to fnmateri
      integer :: compold             ! 1 or 3 means the old solution has to be
                                     ! computed in the integration points
                                     ! 2 or 3 means that array vecloc must be
                                     ! filled with all old solutions in the
                                     ! integration points
      integer :: dimD                ! Dimension of D matrix
      integer :: first               ! if 0 first element
      integer :: icent               ! Index of centroid in element
      integer :: ichfib              ! Indicates which fiber subroutine has to be used
                                     ! 0: fnfib...
                                     ! 1: fnfib...1
      integer :: icount              ! number of degrees of freedom in element
      integer :: ielgrp              ! element group number
      integer :: ielem               ! element number
      integer :: iflipflop           ! element of flip flop array
      integer :: iload               ! Indicates the type of boundary condition
                                     ! for elasticity problems
                                     ! Possible values:
                                     ! 2: Prescribed load in co-ordinate dir
                                     ! 3: Prescribed load given in r and theta dir
                                     ! 4: Prescribed load given in normal and
                                     !    tangential dir.
      integer :: imas                ! Indicates the type of mass matrix
                                     ! (1=diagonal, 2=consistent)
      integer :: imatlaw             ! Type of material constitutive equation
                                     ! to be used
                                     !  1: compressible elastic solid
                                     !  2: incompressible elastic solid
                                     !  4: incompressible Mooney-Rivlin
                                     ! 10: composite material
                                     ! 11: composite material
                                     ! 99: user-defined material
      integer :: indangvel           ! start coefficient of angular velocity
      integer :: indbeta             ! start coefficient of beta (zero order)
      integer :: indcconv            ! start coefficient of c_conv
      integer :: indcdiv             ! start coefficient of c_div
      integer :: indcgrad            ! start coefficient of c_grad
      integer :: indconst            ! start coefficient of constant
      integer :: indcp               ! start coefficient of cp
      integer :: indcvisc            ! start coefficient of c_visc
      integer :: indeta              ! start coefficient of eta
      integer :: inddiff             ! start coefficient of diffusion
      integer :: indgamma            ! start coefficient of gamma
      integer :: indgoertler         ! start coefficients for goertler
      integer :: indh                ! start coefficient of thickness
      integer :: indkappa            ! start coefficient of kappa
      integer :: indlambda           ! start coefficient of lambda
      integer :: indpower            ! start coefficient of power n
      integer :: indpress            ! start coefficient of pressure
      integer :: indrho              ! start coefficient of rho
      integer :: indrhocp            ! start coefficient of rho cp
      integer :: indrhscomp          ! start coefficient of right-hand side in
                                     ! continuity equation
      integer :: indrt               ! start coefficient of RT
      integer :: indsource           ! start coefficient of source
      integer :: indsqueeze          ! start coefficient of squeeze
      integer :: indstress           ! start coefficient of stress
      integer :: indvel              ! start coefficient of velocity
      integer :: induppvel           ! start coefficient of upper velocity
      integer :: indvisc             ! start coefficient of viscosity
      integer :: ipos                ! Position in iuser
      integer :: iqmat               ! Defines if the the qmat matrix must be
                                     ! filled
                                  ! 0: qmat must not be filled
                                  ! 1: qmat must be filled by subroutine qclsys
                                  ! 2: qmat must be filled assuming a linear
                                  !    transformation i.e. the surface elements
                                  !    are in a plane
                                  ! 3: qmat must be filled assuming a general
                                  !    transformation
      integer :: irelem              ! relative element number
      integer :: ishape              ! shape of element
      integer :: ishift              ! Shift in coefficient sequence number
      integer :: isubcfvar           ! Lowest space dependent coefficient
      integer :: isupcfvar           ! Highest space dependent coefficient
      integer :: itype               ! Type number of element
      integer :: jsol                ! Indication if the values of the local
                                     ! vector in the nodal points or the
                                     ! integration points must be computed (1)
                                     ! or not (0)
      integer :: jtransfo            ! Defines transformation
                                     ! 0, 1: No transformation
                                     ! 2: xsi = y, eta = z
                                     ! 3: xsi = x, eta = z
                                     ! 4: xsi = R*atan2(y,x), eta = z
                                     !    R = sqrt(x**2+y**2)
      integer :: ktang               ! Indicates if the tangential vector
                                     ! must be computed (1) or not (0)
      integer :: last                ! if 1 last element
      integer :: maxunk              ! Maximal number of unknowns per point for
                                     ! all "old" vectors
      integer :: mdiv                ! Defines if divergence right-hand side is
                                     ! used (#0) or not (0)
      integer :: methodupw           ! Type of upwind scheme
                                     ! 0: no upwind
                                     ! 1: Brooks and Hughes: classical upwind
                                     ! 2: Brooks and Hughes: Il'in scheme
                                     ! 3: Brooks and Hughes: Doubly assymptotic
                                     ! 4: Brooks and Hughes: Critical approx.
                                     ! 5: Hughes approximation
                                     ! 6: Time-dependent approximation
                                     ! 7: Hughes: Discontinuity capturing 1
                                     ! 8: Hughes: Discontinuity capturing 2
                                     ! 9: Mizukami triangles maximum principle
                                     !10: Mizukami triangles maximum principle,
                                     !             activate flip-flop detection
                                     !11: Mizukami triangles maximum principle,
                                     !             use flip-flop detection
                                     !12: Time-dependent approximation
                                     !    according to Tornberg and Enquist
      integer :: modelconv           ! defines how the convection coefficients
                                     ! are defined
                                     ! Old mcoefconv
                                     ! 0: standard convective terms
                                     ! 1: special case: convective terms are
                                     !    defined by:
                                     !    c_conv_1 u du/dx + c_conv_2 v du/dy
                                     !    c_conv_3 u dv/dx + c_conv_4 v dv/dy
      integer :: modelelastnl        ! Type of non-linearity in elasticity model
                                     ! 0: linear model
                                     ! 1: Geometrical and physical nonlinear
                                     !    behavior (3D)
                                     ! 2: 1 + reduced integration of
                                     !    compressibility term
                                     ! 3: nonlin + variable anisotropy
                                     ! 4: 3 + reduced integration of
                                     !    compressibility term
      integer :: modelrhsd           ! Type of right-hand-side model
                                     ! 0: if there is a given stress tensor the
                                     !    reference to this tensor is stored
                                     !    in position 19
                                     ! 1: It is supposed that there are 6 given
                                     !    stress tensor components stored in
                                     !    positions 19-24
      integer :: modelvisc           ! Type of viscosity model (modelv)
                                     ! 0,1: newtonian liquid
                                     !   2: power law liquid
                                     !   3: carreau liquid
                                     !   4: plastico viscous liquid
                                     !   5: special case: the viscous term is
                                     !      defined as
                                     !      c_visc_1 d^2 u /dx^2 +
                                     !      c_visc_2 d^2 u /dy^2
                                     !      c_visc_3 d^2 v /dx^2 +
                                     !      c_visc_4 d^2 v /dy^2
                                     ! 10:  special case: Newtonian fluid,
                                     !      with constant viscosity
                                     !      incompressible
                                     !      The viscous part is written as eta
                                     !      laplace u, hence the continuity
                                     !      equation is used to simplify
                                     !      the expression
                                     !      The natural boundary conditions are
                                     !      different from the standard ones.
                                     !100:  user provided model depending on
                                     !      grad u
                                     !101:  user provided model depending on the
                                     !      second invariant
                                     !102:  user provided model depending on the
                                     !      second invariant, the co-ordinates
                                     !      and the velocity
      integer :: mr                  ! number of reduced integration points
      integer :: ncoef               ! number of real coefficients
      integer :: ndim                ! dimension of space
      integer :: nelem               ! number of elements
      integer :: nelgrp              ! number of element groups
      integer :: notmat              ! indicates if element matrix is zero (1)
                                     ! or not (0)
      integer :: notvec              ! indicates if element vector is zero (1)
                                     ! or not (0)
      integer :: npelm               ! number of nodes in element
      integer :: npoint              ! number of nodes in domain
      integer :: npress              ! number of pressure unknowns in element
      integer :: nunkp_pel           ! number of degrees of freedom
                                     ! in the element
      integer :: nuser               ! Computed length of array user
      integer :: nveloc              ! number of velocity unknowns in element
      integer :: seq_meshvel         ! Sequence number of mesh velocity
      integer :: seq_sol             ! sequence number of solution vector
      integer :: seq_second_old_sol  ! Sequence number of second old solution
                                     ! in case of time integration
      integer :: special             ! Special option in stress-strain
      integer :: stress_strain       ! stress-strain relation
                     !  0: plane stress, isotropic material
                     !     or 3D stress, isotropic material (Cartesian)
                     !  1: plane strain, isotropic material (Cartesian)
                     !  2: axi-symmetric stress, isotropic material
                     !                                      (axi-symmetric)
                     !  3: plane stress, orthotropic material (Cartesian)
                     !  4: plane strain, orthotropic material (Cartesian)
                     !  5: axi-symmetric stress, orthotropic material
                     !     (axi-symmetric)
                     !  6: User defined elasticity matrix (3x3) (Cartesian)
                     !  7: User defined elasticity matrix (4x4) (axi-symmetric)
                     !  8: User defined elasticity matrix (6x6) (3d)
      integer :: tangstress          ! Deefines tangential stress
                                     ! 0  shear stress is taken into account
                                     ! 1  shear stress is not taken into account
      integer :: typebear            ! type of bearing.
                                     ! Possible values:
                                     ! 0:  incompressible bearing
                                     ! 1:  compressible bearing (adiabatic)
      integer :: typecompress        ! Defines type of compressibility
                                     !  0:  incompressible
                                     !  1:  compressible
      integer :: typeconv            ! Defines type of convection
                                     !  0:  incompressible
                                     !  1:  compressible
                                     !  2:  absolute
                                     !  3:  non_linear
      integer :: typeconvect         ! Defines type of convection linearization
                                     ! Old mconv
                                     !  0:  no convection
                                     !  1:  Picard iteration
                                     !  2:  Newton iteration
                                     !  3:  incorrect Picard iteration
                                     !  4:  successive substitution
      integer :: typecontinuity      ! Defines type of continuity equation
                                     ! Old mcont
                                     !  0:  continuity equation is div u = 0
                                     !  1:  special continuity equation
                                     !      div ( rho u ) = 0, which rho
                                     !      variable is solved.
                                     !  2:  Goertler equations are solved
                                     !  3:  continuity equation is equal to:
                                     !      c_div_1 du/dx +  c_div_2 dv/dy = 0
                                     !  4:  continuity equation contains a
                                     !      pressure term
      integer :: typediff            ! Defines type of diffusion tensor
                                     !  1:  diffusion is a scalar
                                     !  2:  diffusion is a diagonal tensor
                                     !  3:  diffusion is a symmetric tensor
      integer :: typediff1           ! type of diffusion tensor 1
      integer :: typediff2           ! type of diffusion tensor 2
      integer :: typelinear          ! Defines type of linearization
                                     !  0:  picard
                                     !  1:  newton
                                     !  2:  instationary Picard
                                     !  3:  instationary Newton
      integer :: typenatbnd          ! Defines type of natural boundary cond.
      integer :: typetime            ! Defines type of time integration
                                     !  0:  standard
                                     !  1:  theta method (in element)

      double precision :: costet     ! cos(theta)
      double precision :: dpndz      ! dp/dz
      double precision :: h0(2)      ! thickness at t=0
      double precision :: hn(2)      ! thickness at t=0
      double precision :: penalp     ! penalty parameter
      double precision :: sintet     ! sin(theta)
      double precision :: waxi(2)    ! axi-symmetric weights

      logical :: b2filled            ! if true B2 has been filled
      logical :: compressible        ! if true convection is in the
                                     ! compressible form or
                                     ! the continuity equation is compressible
      logical :: convection          ! if true convection must be computed
      logical :: convection1         ! see convection, first equation
      logical :: convection2         ! see convection, second equation
      logical :: diffusion           ! if true diffusion must be computed
      logical :: diffusion1          ! if true first diffusion must be computed
      logical :: diffusion2          ! if true 2nd diffusion must be computed
      logical :: dmatconst           ! if true the D-matrix is constant
                                     ! compressible form
      logical :: fillbmat            ! if true the B-matrix must be filled
      logical :: filloldderiv        ! if true the derivative of the prior
      logical :: filloldvec          ! if true the prior vector must be filled
      logical :: gammaterm           ! if true the gamma term is added to the
                                     ! element vector
      logical :: initial_strain      ! if true there is initial strain
      logical :: initial_stress      ! if true there is initial stress
      logical :: matelim             ! if true the eleent matrix must be built
                                     ! despite matrix is false.
                                     ! The reason is that some positions are
                                     ! used for the elimination of the centroid
      logical :: newton              ! if true newton iteration
      logical :: plane               ! if true the stress-strain relation
                                     ! corresponds to 2d oplane stress/strain
      logical :: rdint               ! if true reduced integration must be
                                     ! applied
      logical :: rmhy                ! if true hydrostatics must be removed
      logical :: rhoone              ! if true teh density rho cp = 1
      logical :: souza               ! if true souza incompressibility is used
      logical :: sourceterm          ! if true the source term is added to the
                                     ! element vector
      logical :: sourceterm1         ! sourceterm for first equation
      logical :: sourceterm2         ! sourceterm for second equation
      logical :: symm                ! Defines whether the matrix is
                                     ! symmetric (true) or not
      logical :: timedep             ! if true time-dependent iteration
      logical :: useetaeff           ! if etaeff is not the same as eta
      logical :: zeroterm            ! if true the zero order term is added to
                                     ! the element matrix
                                     ! vector must be filled
      logical :: zeroterm1           ! zeroterm for first equation
      logical :: zeroterm2           ! zeroterm for second equation


!     --- arrays from common celiar

      integer, pointer :: ich(:)      ! information concerning the
                                      ! coefficients is stored for the case
                                      ! IND(I)=3000 + IVEC
                                      ! The value of ich(i) has the following
                                      ! meaning:
                                      ! 1: Array VECOLD(IVEC) has exactly one
                                      !    degree of freedom in all nodal points
                                      ! 2: Array VECOLD(IVEC) has a constant
                                      !    number of degrees of freedom in all
                                      !    nodal points
                                      ! 3: Array VECOLD(IVEC) has exactly
                                      !    one degree of freedom in all vertices
                                      !    and none in the other nodes
                                      !    Two-dimensional elements only
                                      ! 4: Array VECOLD(IVEC) has a constant
                                      !    number of degrees of freedom in all
                                      !    vertices and none in the other nodes
                                      !    Two-dimensional elements only
                                      ! 5: Array VECOLD(IVEC) contains the
                                      !    degree of freedom in all nodes
                                      ! 6: Array VECOLD(IVEC) contains the
                                      !    degree of freedom in all vertices
                                      !    but not in all nodes
                                      !    Two-dimensional elements only
                                      ! 7: Array VECOLD(IVEC) does not
                                      !    contain the degree of freedom in
                                      !    the centroid
                                      ! 8: Array VECOLD(IVEC) does not
                                      !    contain the degree of freedom in
                                      !    the centroids of tetrahedrons
                                      ! >100: Array VECOLD(IVEC) corresponds
                                      !    to a vector of special structure
                                      !    of type 116 with ich(i)-100 degrees
                                      !    of freedom per element
                                      ! >1000: Array VECOLD(IVEC) corresponds
                                      !    to a vector of special structure of
                                      !    type 126 with ich(i)-1000 degrees
                                      !    of freedom per element per node
                                      ! >2000: Array VECOLD(IVEC) corresponds
                                      !    to a vector of special structure of
                                      !    type 129 with ich(i)-2000 degrees
                                      !    of freedom per element per
                                      !    integration point
      integer, pointer :: ind(:)      ! Of each coefficient it
                                      ! is stored whether the coefficient is 0
                                      ! (ind(i) = 0), constant (ind(i)<0)
                                      ! or variable (ind(i)>0)
      integer, pointer :: ist(:)      ! it is indicated if a coefficient
                                      ! is a copy of preceding coefficient (>0)
                                      ! or not (=0)
                                      ! If ist(i) > 0 the value of ist(i)
                                      ! indicates the coefficient number from
                                      ! which the coefficient must be copied.
      integer, pointer :: mst(:)      ! the start position of the real
                                      ! information of the ith coefficient in
                                      ! array user is stored. See elusr3
                                      ! If ist(i) = 3000+IVEC then mst(i)
                                      ! contains the degree of freedom j that
                                      ! must be used for this coefficient
      integer, pointer :: ivecnums(:) ! the values of ivec are stored
   end type element

   type (element) :: E

end module sepmoduleelm
