module sepmoduleproj

   use sepmodulemat
   use sepmodulematr
   use sepmodulesolve

   integer :: nnproj    ! length of array containing
                        ! the sparse projection vectors.
   integer :: nproj     ! Number of projection vectors
   integer :: nset      ! Number of sets
   integer :: nshale    ! Number of shale sets for which one or more subsystem
                        ! have to be solved.
   integer :: print_lv  ! Controls the amount of output produced when computing
                    !  the projection vectors
                    !  Possible levels: -1, 0, 1, 2, 3, 4, 5
                    ! -1 : no output
                    !  0 : standard output
                    !  1 : also shows which process is busy and number of
                    !      iterations in each subprocess is printed
                    !  2 : also information about each iteration in a subprocess
                    !      is printed
                    !  3 : also detailed information about the making of
                    !      the projection vectors is printed
                    !  4 : also the projection vectors are printed
                    !  5 : also the sub-matrices are printed
                    !  6 : in case of 2D also contour plots of the projection
                    !      vectors are made
                    !  PRINT_LV can be controlled by including
                    !  "PROJ_PRINT_LEVEL = i" in the solve block in the
                    !  problem file.
   integer :: plot_lv   ! Indicates if projection vectors must be plotted
                        ! 0: no plot
                        ! 1: contour of projection vectorsPlot level
   integer :: proj_type ! Type of creation of projection vectors
                        ! Possible values:
                        ! 1:  Classical method (sand=1, solution of equation in
                        !     shale)
                        ! 2:  Simple approximation (sand=1, shale=0)
                        ! 3:  Multigrid approximation (all layers 0 and 1)

   double precision :: epsproj    ! Accuracy with respect to the computation
                                  ! of the projection vectors
   double precision :: threshold  ! Entries in the projection vector that are
                                  ! smaller than threshold are ignored.

   integer, pointer :: conset(:,:) ! conset(i,j)=1 means that sets i and j are
                                   ! coupled
   integer, pointer :: dmap(:)     ! contains information on local
                      ! and global degrees of freedom and degrees of freedom
                      ! on internal boundaries.
                      ! DMAP consist of 3 parts, part I is of length nUSOL
                      ! If iDEGFD lies inside the current shale set then
                      ! DMAP(iDEGFD)=local number of degree of freedom for
                      ! global degree of freedom iDEGFD, iDEGFD=1,nUSOL.
                      ! Part II is of length nLOCDGDFD
                      ! DMAP(nUSOL+iLOCDGFD)=global number of degree of
                      ! freedom for local degree of freedom iLOCDGFD.
                      ! DMAP Part III is of length nINBOUND
                      ! DMAP(nUSOL+nLOCDGFD+iINBOUND)=global number of a degree
                      ! of freedom that lies on an internal border. They are not
                      ! ordered.
   integer, pointer :: dirichlt(:) ! DIRICHLT( iSET ) = 1 means that
                                   ! set iSET has a dirichlet boundary.
   integer, pointer :: groupset(:) ! element group i has SET number GROUPSET(i)
   integer, pointer :: inclusn(:)  ! if inclusn(i)=1 it has only one neighbor
                                   ! set
   integer, pointer :: itsprojv(:,:) ! contains information on the sparse
                           !  projection vectors.
                           !  ITSPROJV( iPROJ, iSET ) =
                           !  0     Projection vector iPROJ is zero in set iSET.
                           !  1     Projection vector iPROJ is one in set iSET.
                           ! -index The part of the projection vector iPROJ that
                           !        lies in set iSET is stored in array RTSPROJ,
                           !        starting at position INDEX. The sequence of
                           !        the unknowns is the same as in SETNODE.
   integer, pointer :: locintmt(:) ! local array intmat
   integer, pointer :: nconset(:)  ! contains number of sets coupled to a set
                                   ! accumulated (nconset(1)=0)
   integer, pointer :: nsetnode(:) ! Number of nodes per set, accumulated
                                   ! (nsetnode(1)=0)
   integer, pointer :: nodcheck(:) ! contains information on whether a node
                                   ! has been checked out (1)
                                   ! (in renumbering process) or not (0)
   integer, pointer :: nodeset(:)  ! contains information per node
                                   ! which set it belongs to.
                                   ! NODESET(iNODP) = set number node iNODP
                                   ! belongs to.
   integer, pointer :: nsol(:)     ! contains information of how much subsystems
                                   ! have to be solved on each set.
                                   ! abs( NSOL(iset)) = the number of subsystems
                                   ! that have to be solved on set iSET.
                                   ! If NSOL(iset) < 0 then the number of
                                   ! solutions is not equal to the number of
                                   ! sand neighbors.
   integer, pointer :: pntlist(:)  ! containis nodal points that do not have
                                   ! a local number of degree of freedom yet.
                                   ! Is filled during the process.
   integer, pointer :: projcols(:) ! Contains the degrees of freedom of the
                                   ! projection vectors, first of vector Aw_1
                                   ! etc. See also projrows
   integer, pointer :: projcolsa(:) ! See projcols, but now with extra
                                   ! information for parallel case
   integer, pointer :: projcolstmp(:) ! temporary version of projcols
   integer, pointer :: projcolsatmp(:) ! temporary version of projcolsa
   integer, pointer :: projrows(:) ! Contains the number of non-zero entries
                                   ! in the projection vectors w_i and also
                                   ! in Aw_i accumulated
                                   ! projrows(1)=0, projrows(nproj+1) is
                                   ! length of array projcols
                                   ! Note that the projection vectors are
                                   ! extended with neighboring elements in order
                                   ! that vectors Aw belong to the same
                                   ! structure
   integer, pointer :: projrowsa(:) ! See projrows, but now with extra
                                   ! information for parallel case
   integer, pointer :: projsand(:) ! contains information on sand sets
                                   ! for which a projection vector needs to be
                                   ! made.
                                   ! PROJSAND( 1 .. nPROJ ) = set numbers of
                                   ! sand sets
   integer, pointer :: sandproj(:) ! inverse of projsand
                                   ! SANDPROJ(iSET) = number of projection
                                   ! vector for sand set iSET.
   integer, pointer :: sandset(:)  ! indicates if a set is a sand set (1)
                                   ! or shale (0)
   integer, pointer :: setnode(:)  ! contains nodal point numbers of each set
                                   ! SETNODE( NSETNODE( iSET ) + 1 ..
                                   ! NSETNODE( iSET + 1 ) ) = nodal point
                                   ! numbers for set iSET.
   integer, pointer :: solshale(:) ! contains information on the shale sets
                                   ! for which a subsystem has to be solved.
                                   ! SOLSHALE(1..nSHALE) = set numbers of shale
                                   ! sets for which a subsystem has
                                   ! to be solved.
   integer, pointer :: spconsetsh(:) ! sparse version of conset (shale)
   integer, pointer :: spconsetsa(:) ! sparse version of conset (sand)

   double precision, pointer :: locmat(:)  ! local matrux
   double precision, pointer :: locrhsd(:) ! local right-hand side
   double precision, pointer :: locsol(:)  ! local solution

   double precision, pointer :: projU(:)   ! contains the projection
                                           ! vectors w1..wNPROJ
                                           ! For the description see projrows
                                           ! and projcols
   double precision, pointer :: projUtmp(:) ! temporary version of projU
   double precision, pointer :: projAU(:)  ! contains the projection
                                           ! vectors Aw_i
   double precision, pointer :: projE(:) ! contains the LU decomposition
                                           ! of the matrix E. The decomposition
                                           ! is stored as a matrix of
                                           ! size nPROJ x nPROJ
   double precision, pointer :: projE2d(:,:) ! 2d version
   double precision, pointer :: rtsprojv(:) ! contains the non-constant
                                   ! parts of the projection vectors,
                                   ! see ITSPROJV
end module sepmoduleproj
