module sepmodulesolve

   use sepmodulecons

   integer, target :: inpsol(ninpsol1,3+maxsubeq)
   double precision, target :: rinsol(nrinsol1,3+maxsubeq)

   integer :: ilimit     ! defines if the solution must be limited to the
                         ! minimum and maximum values in the input vector,
                         ! including boundary conditions.
                         ! Possible values:
                         ! limit = 0, no limiting
                         ! limit = 1, limiting
   integer :: imetproj   ! Type of method to compute to projection vectors
                         ! Possible values:
                         ! 0:  none
                         ! 1:  approximate eigenvectors
   integer :: ipos       ! indication whether the matrix to be decomposed is
                         ! positive definite (1) or not (0)
   integer :: irescomp   ! Indicates whether the residual must be
                         ! computed and printed after computation
                         ! of the solution. Possible values:
                         ! 0:  Do not compute the residual
                         ! 1:  Compute and print the 2-norm of the
                         !     residual
                         ! 2:  Compute and print the 2-norm of the
                         !     residual divided by the 2-norm of
                         !     the right-hand side
   integer :: iprecpetsc ! See ipreco, but now for petsc
   integer :: ipreco     ! type of preconditioner, see read03
   integer :: iprecq     ! type of preconditioner for Q-matrix
   integer :: iprecr     ! type of preconditioner for R-matrix
   integer :: ireadinpsol  ! With this parameter it is indicated if array INPSOL
                         ! is used or that input is read from the standard
                         ! INPUT file
                         ! Possible values:
                         ! 0   The input is read from the standard input file
                         ! >0  The input as read from the standard input file by
                         !     subroutine SEPSTN is used. The value of iread
                         !     defines the sequence number
                         !     If iread>1000, the start vector is given
                         ! -1  No input is read, all information from array
                         !     INPSOL is used
                         ! -2  The input is read from the standard input file
                         !     This input is stored in arrays INPSOL and RINSOL
                         !     Position 1 of both arrays must contain the
                         !     declared length
   integer :: iscaling   ! Choice of diagonal scaling. Possible values:
                         ! 0: No scaling
                         ! 1: Only row scaling
                         ! 2: Row and column scaling
                         ! 3: Only column scaling
   integer :: iseq_end_res ! Contains position of end residual in ISOL
   integer :: iseq_exact ! Sequence number of exact solution in ISOL
                         ! 0: no exact solution
   integer :: iseqdgpre  ! Sequence number of preconditioning for DG-matrix
   integer :: iseqintmat ! Sequence number with respect to array intmat
   integer :: iseqpmpre  ! Sequence number of preconditioning for PM-matrix
   integer :: iseqpre    ! Sequence number of preconditioning matrix
   integer :: iseqprerd  ! Sequence number of preconditioning matrix as
                         ! stored in inpsol
   integer :: iseqqpre   ! Sequence number of preconditioning for Q-matrix
   integer :: iseqreac   ! Sequence number of reaction vector in array ISOL
                         ! If <0 the right-hand side contribution is not
                         ! subtracted
   integer :: iseqres_exact ! Sequence number of residual of exact solution
                         ! If zero the residual is not computed.
   integer :: iseqsol    ! Sequence number of solution array in array ISOL
   integer :: iseqsolut  ! Sequence number of solution array in array isolut,
                         ! i.e. global solution array
   integer :: iseq_start_res ! Contains position of start residual in ISOL
   integer :: itermethod ! iteration method for petsc
   integer :: itrans     ! indication how local transforms must be carried out
                         ! Possible values:
                         ! 0  Standard situation, back transformation if
                         !    necessary
                         ! 1  The back transformation is skipped
                         ! 2  The solution process is skipped, only the back
                         !    transformation is carried out
   integer :: keep       ! With this parameter the user may indicate whether
                         ! the preconditioning matrix must be destroyed, kept or
                         ! that an old preconditioning matrix must be used.
                         ! Possibilities:
                         ! 0 The preconditioning matrix is destroyed.
                         ! 1 The preconditioning matrix is kept.
                         !   A reference is written in MATR(4).
                         ! 2 The preconditioning matrix as computed
                         !   before is used.
   integer ::  keepmat   ! indicates if the matrix must be kept, destroyed or
                         ! reused.  Possible values:
                         ! 0: new and destroy
                         ! 1: new and keep
                         ! 2: old and destroy
                         ! 3: old and keep
   integer :: lenpremat  ! Length of preconditioning matrix
   integer :: mumps_ren  ! Type of renumbering scheme for mumps
   integer :: niterout   ! Number of global iterations performed
   integer :: niterpout  ! Number of pressure iterations performed
   integer :: niteruout  ! Number of velocity iterations performed
   integer :: numdef_corr ! Number of defect correction steps
   integer :: nmax       ! maximum number of iterations
   integer :: num_levels ! Number of levels for petsc
   integer :: printlev   ! print level
   integer :: typeparsolv ! type of parallel solver
                         ! 1: block jacobi  (default)
                         ! 2: gcr
   integer :: typesolv   ! type of linear solver
                         !  0: A profile method is used
                         !     (direct solver)
                         !  1: CG (conjugate gradients) or BiCGstab
                         !     In case of a non-symmetric matrix
                         !     Bi-CGSTAB
                         !  2: CGS
                         !  3: GMRES
                         !  4: GMRESR
                         !  5: OVERRL or OVERCS
                         !  6: SIMPLE_GCR
                         !  7: SYMMLQ
                         !  8: SIMPLER_GCR
                         !  9: ML_multigrid
                         ! 10: IDR
                         ! 11: AL_GCR
                         ! 12: HSIMPLER_GCR
                         ! 13: MSIMPLER_GCR
                         ! 14: LSC_GCR
                         ! 15: QLSC_GCR
                         ! 16: SCHUR
                         ! 17: Bi_CGSTab(l)
                         ! 18: BLOCK_TRIANGULAR
                         ! 19: MUMPS
                         ! 20: PETSC
                         ! 21: RICHARDSON
                         ! 22: CHEBYSHEV
                         ! 23: FGMRES
                         ! 24: DGMRES
                         ! 25: BICG
                         ! 26: TFQMR
                         ! 27: TCQMR
                         ! 28: CR
                         ! 29: LSQR
                         ! 30: PREONLY
                         ! 31: GCR

   double precision :: abs_accuray ! parameter for petsc
   double precision :: algamma     ! Parameter gamma in al method
   double precision :: div_accuray ! parameter for petsc
   double precision :: dampingfact ! damping factor for petsc
   double precision :: emax        ! parameter for petsc
   double precision :: emin        ! parameter for petsc
   double precision :: epsacc      ! accuracy
   double precision :: lapshift    ! Real part of shift (laplace shift)
   double precision :: lapshiftim  ! Imaginary part of shift (laplace shift)
   double precision :: rel_accuray ! parameter for petsc
   double precision :: valmin      ! minimum value of input vector
   double precision :: valmax      ! maximum value of input vector

   logical :: copyprec   ! if true the matrix must be copied into the
                         ! preconditioning matrix
   logical :: matsym     ! if true the matrix is symmetric
                         ! (non-symmetric storage)
   logical :: newversion ! if true the new version is used
   logical :: posdev     ! if true the matrix is positive definite
   logical :: schur      ! If true the Schur method is applied
   logical :: skipzeros  ! if true zero pivots are replaced by machine accuracy

   double precision, pointer :: usolbuf(:) ! solution vector
   double precision, pointer :: rhsdbuf(:) ! rhs vector
   double precision, pointer :: rhsdbuf2(:) ! second rhs vector

!  Old common citer

   integer :: niter           ! The number of iteration performed
   integer :: niter1          ! Help factor N1, see description of OVERRL
   integer :: niter2          ! Help factor N2, see description of OVERRL
   integer :: numiters(10)    ! Number of iterations per problem

   double precision :: alam   ! Approximation of eigenvalue(Largest or smallest)
   double precision :: alpha  ! Help factor alpha, see description of OVERRL
   double precision :: beta   ! Help factor beta , see description of OVERRL
   double precision :: omega  ! Overrelaxation parameter

!  --- Complex variant

   double complex   :: clapshift   ! complex lapshift

   integer :: projdata(5)  ! temporary; must be removed
!     projdata       Integer array of length 5 containing information on
!                    the projection vectors
!                    Pos 1. number of projection vectors
!                    Pos 2. 130
!                    Pos 3. memory management sequence nr. of array INTPRJ
!                           containing an integer description of the projection
!                           vectors
!                    Pos 4. memory management sequence nr. of array PROJVC
!                           containing the projection vectors
!                    Pos 5. nnproj

end module sepmodulesolve
