module sepmodulevecs

   use sepmodulesol

   double precision, pointer :: capacity(:)  ! capacity vector
   double precision, pointer :: coornew1d(:) ! new coordinates (1d)
   double precision, pointer :: coornew(:,:) ! new coordinates (2d)
   double precision, pointer :: coorold1d(:) ! old coordinates (1d)
   double precision, pointer :: coorold(:,:) ! old coordinates (2d)
   double precision, pointer :: density(:)   ! density vector
   double precision, pointer :: deltatemp(:) ! temperature difference
   double precision, pointer :: diffvec(:)   ! difference vector
   double precision, pointer :: displacement(:) ! displacement vector
   double precision, pointer :: distvec(:)   ! distance vector
   double precision, pointer :: eigval(:)    ! eigenvalues
   double precision, pointer :: eigvec(:)    ! eigenvector
   double precision, pointer :: endres(:)    ! residual of end solution
   double precision, pointer :: enthalpy(:)  ! residual of end solution
   double precision, pointer :: forcevec(:)  ! force vector
   double precision, pointer :: grad(:)      ! gradient vector
   double precision, pointer :: help(:)      ! help vector
   double precision, pointer :: help1(:)     ! help vector
   double precision, pointer :: machno(:)    ! vector with mach numbers
   double precision, pointer :: matrixm(:)   ! m-matrix
   double precision, pointer :: matrixs(:)   ! s-matrix
   double precision, pointer :: meshvel(:)   ! mesh velocity (1d)
   double precision, pointer :: meshvel2d(:,:) ! mesh velocity (2d)
   double precision, pointer :: normalvec(:,:) ! normal vector per node
                                             ! first component dimension
                                             ! second component node
   double precision, pointer :: normalvec1d(:) ! normal vector (1d)
   double precision, pointer :: normalstress(:,:)  !normal stress vector along a
                                             ! set of curves and zero elsewhere
   double precision, pointer :: phi1(:)      ! first phi vector
   double precision, pointer :: phi2(:)      ! second phi vector
   double precision, pointer :: partics(:)   ! contains for each particle a
                                             ! (ndim+1) x  nsteps real array,
                                             ! containing for each position
                                             ! t,x,y,z
   double precision, pointer :: pressure(:)  ! pressure vector
   double precision, pointer :: qvec(:)      ! q vector
   double precision, pointer :: reac(:)      ! reaction vector
   double precision, pointer :: resusolex(:) ! residual of exact solution
   double precision, pointer :: res2(:)      ! rhs vector
   double precision, pointer :: resvec(:)    ! residual vector
   double precision, pointer :: rhsd(:)      ! rhs vector
   double precision, pointer :: rhsd1(:)     ! rhs vector
   double precision, pointer :: rhsd2(:)     ! rhs vector
   double precision, pointer :: rhsdcp(:)    ! rhs vector
   double precision, pointer :: rhsdbc(:)    ! rhs vector wrt to boundary
                                             ! conditions
   double precision, pointer :: rhsdsp(:)    ! special rhs vector
   double precision, pointer :: solut(:)     ! solution vector for mumps only
   double precision, pointer :: stagpres(:)  ! stagnation pressure vector
   double precision, pointer :: startres(:)  ! residual of start solution
   double precision, pointer :: stress1d(:)  ! stress vector
   double precision, pointer :: stress(:,:)  ! stress vector
   double precision, pointer :: tang(:,:)    ! tangential vector per node
                                             ! first component dimension
                                             ! second component node
   double precision, pointer :: temperature(:) ! temperature vector
   double precision, pointer :: tempnew(:)   ! new temperature vector
   double precision, pointer :: tempold(:)   ! old temperature vector
   double precision, pointer :: u1(:)        ! first vector
   double precision, pointer :: u2(:)        ! second vector
   double precision, pointer :: u3(:)        ! third vector
   double precision, pointer :: unew(:)      ! new solution vector
   double precision, pointer :: uold(:)      ! old solution vector
   double precision, pointer :: usol(:)      ! solution vector
   double precision, pointer :: usolex(:)    ! exact solution vector
   double precision, pointer :: usolglob(:)  ! global solution vector
   double precision, pointer :: usolsymm(:)  ! extended solution for symmetry
   double precision, pointer :: usolt(:)     ! temporary solution vector
   double precision, pointer :: vecc0(:)     ! c0 vector
   double precision, pointer :: vecc1(:)     ! c1 vector
   double precision, pointer :: veceps(:)    ! epsilon vector
   double precision, pointer :: vectin(:)    ! input vector
   double precision, pointer :: vectin2(:)   ! second input vector
   double precision, pointer :: vectout(:)   ! output vector
   double precision, pointer :: vectout2(:)  ! output vector
   double precision, pointer :: vectout2d(:,:) ! output vector
   double precision, pointer :: vecx(:)      ! x vector
   double precision, pointer :: vecy(:)      ! y vector
   double precision, pointer :: velocity(:)  ! velocity vector
   double precision, pointer :: xreal(:)     ! real x vector
   double precision, pointer :: xrc(:)       ! real x vector ( periodical part)
   double precision, pointer :: xri(:)       ! real x vector ( internal part)
   double precision, pointer :: xrp(:)       ! real x vector ( bc part)
   double precision, pointer :: yreal(:)     ! real y vector
   double precision, pointer :: yrc(:)       ! real y vector ( periodical part)
   double precision, pointer :: yri(:)       ! real y vector ( internal part)
   double precision, pointer :: yrp(:)       ! real y vector ( bc part)
   double precision, pointer :: w0(:)        ! w0 vector
   double precision, pointer :: w1(:)        ! w1 vector
   double precision, pointer :: workv(:)     ! work vector
   double precision, pointer :: xcoor(:)     ! x coordinates
   double precision, pointer :: xfunc(:)     ! x function

   double precision, pointer :: velocity2d(:,:) ! velocity vector (2d)

   double complex, pointer :: vcompl(:)      ! complex vector
   double complex, pointer :: xcompl(:)      ! complex x vector
   double complex, pointer :: xcc(:)         ! cmplx x vector ( periodical part)
   double complex, pointer :: xci(:)         ! cmplx x vector ( internal part)
   double complex, pointer :: xcp(:)         ! cmplx x vector ( bc part)
   double complex, pointer :: ycompl(:)      ! complex y vector
   double complex, pointer :: ycc(:)         ! cmplx y vector ( periodical part)
   double complex, pointer :: yci(:)         ! cmplx y vector ( internal part)
   double complex, pointer :: ycp(:)         ! cmplx y vector ( bc part)
   double complex, pointer :: zresusolex(:)  ! residual of exact solution
   double complex, pointer :: zstartres(:)   ! residual of start solution
   double complex, pointer :: zrhsd(:)       ! rhs vector
   double complex, pointer :: zusolex(:)     ! exact solution vector
   double complex, pointer :: zunew(:)       ! new solution vector
   double complex, pointer :: zusol(:)       ! solution vector

   integer, pointer :: intnodesnew(:)
   integer, pointer :: intnodesold(:)
   integer, pointer :: ivectin(:)    ! input vector (integer)
   integer, pointer :: ivectout(:)   ! output vector (integer)
   integer, pointer :: nodes(:)      ! Contains node numbers
   integer, pointer :: parentnodes(:)
   integer, pointer :: particinf(:,:) ! 2 x npart+1 array with integer
                                      ! information of the particles
                                      ! (:,1) contains npart, ndim
                                      ! (:,i+1) nsteps, istart, where
                                      ! istart is starting address
                                      ! of the second dimension of partics
                                      ! (starting from 1)
   integer, pointer :: phasenew(:)  ! Array containing the phase of each element
                                    ! for the new situation
                                    ! 0: element is solid
                                    ! 1: element is liquid
                                    ! 2: element is mushy
   integer, pointer :: phaseold(:)  ! Array containing the phase of each element
                                    ! for the old situation
   integer, pointer :: refinenew(:)
   integer, pointer :: refinenewloc(:)
   integer, pointer :: refineold(:)
   integer, pointer :: refineoldloc(:)
   integer, pointer :: refmapnew(:)
   integer, pointer :: refmapold(:)
   integer, pointer :: update(:)

   double precision, pointer :: usoltemp(:)  ! temporary vector
   double precision, pointer :: usoltemp2d(:,:)  ! temporary vector 2d
   integer, pointer :: isoltemp(:)           ! temporary vector

end module sepmodulevecs
