      subroutine mshdel1 ( ibuffr, kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 21-11-2012 Long integers
!        version  1.1  date 08-12-2003 Separate subroutine for kmesh ad
!        version  1.0  date 22-11-2002
!
!   copyright (c) 2002-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Delete array KMESH and all of its corresponding arrays
!    Identical to mshdel, but not all parts are deleted
!    Special routine for prrefine
! **********************************************************************
!
!                       KEYWORDS
!
!     delete
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), ibuffr(*)

!     ibuffr    i/o   Integer buffer array in which all large data is stored
!     kmesh     i/o   KMESH array to be deleted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipkmsh
      integer i, kelmaa, kelmab, kelmy

!     i              Counting variable
!     ipkmsh         Starting address of array KMESH part h into array IBUFFR
!     kelmaa         Starting position of KMESH part AA in array KMESH
!     kelmab         Starting position of KMESH part AB in array KMESH
!     kelmy          Starting position of KMESH part Y in array KMESH
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) iniget

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060         Reactivate existing array in IBUFFR
!     INI066         delete temporary array
!     INIGET         Compute pointer of array in IBUFFR
!     MSHDELKMESHAD  Delete kmesh part ad
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      75   Array KMESH has not been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshdel1' )

      if ( kmesh(2)/=100 ) then

!     --- kmesh has not been filled

         call errint ( kmesh(2), 1 )
         call errsub ( 75, 1, 0, 0 )
         go to 1000

      end if
      do i = 17, 19
         if ( kmesh(i)>0 ) call ini066 ( ibuffr, kmesh(i) )
      end do
      if ( kmesh(22)>0 ) then
         call ini060 ( ibuffr, kmesh(22) )
         ipkmsh = iniget(kmesh(22))
         if ( ibuffr(ipkmsh)>0 ) &
            call ini066 ( ibuffr, ibuffr(ipkmsh) )
         if ( ibuffr(ipkmsh+1)>0 ) &
            call ini066 ( ibuffr, ibuffr(ipkmsh+1) )
         if ( ibuffr(ipkmsh+2)>0 ) &
            call ini066 ( ibuffr, ibuffr(ipkmsh+2) )
         if ( ibuffr(ipkmsh+4)>0 ) &
            call ini066 ( ibuffr, ibuffr(ipkmsh+4) )
         call ini066 ( ibuffr, kmesh(22) )
      end if
      do i = 23, 35
         if ( kmesh(i)>0 ) call ini066 ( ibuffr, kmesh(i) )
      end do
      do i = 47, 47
         if ( kmesh(i)>0 ) call ini066 ( ibuffr, kmesh(i) )
      end do
      kelmaa = kmesh(48)-1
      if ( kelmaa>0 ) then
         do i = 31, 42
            if ( kmesh(kelmaa+i)>0 ) &
               call ini066 ( ibuffr, kmesh(kelmaa+i) )
         end do
      end if
      kelmab = kmesh(49)-1
      if ( kelmab>0 ) then
         do i = 1, 15
            if ( kmesh(kelmab+i)>0 ) &
               call ini066 ( ibuffr, kmesh(kelmab+i) )
         end do
      end if
      kelmy = kmesh(50)-1
      if ( kelmy>0 ) then
         do i = 21, 25
            if ( kmesh(kelmy+i)>0 ) &
               call ini066 ( ibuffr, kmesh(kelmy+i) )
         end do
      end if
      do i = 51, 51
         if ( kmesh(i)>0 ) call ini066 ( ibuffr, kmesh(i) )
      end do

!     --- Delete kmeshad and related arrays

      call mshdelkmeshad ( ibuffr, kmesh )

      do i = 55, 56
         if ( kmesh(i)>0 ) call ini066 ( ibuffr, kmesh(i) )
      end do
      do i = 59, 61
         if ( kmesh(i)>0 ) call ini066 ( ibuffr, kmesh(i) )
      end do

      do i = 4, kmesh(3)
         kmesh(i) = 0
      end do
      kmesh(2) = 0
1000  call erclos ( 'mshdel1' )

      end
