      subroutine prcopyus2vec1 ( sepvector, inpvec, npoint, nunkp, &
                                 idegfd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy user vector to sepran vector
!     The user vector has 1 unknown per point in all nodes
!     The output vector is defined in all nodes and has exactly nunkp
!     degrees of freedom per point
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer  npoint, nunkp, idegfd
      double precision sepvector(nunkp,npoint), inpvec(npoint)

!     idegfd         i    Degree of freedom sequence number
!     inpvec         i    Input vector of length npoint containing the
!                         vector to be copied
!     npoint         i    Number of nodal points
!     nunkp          i    Maximum number of unknowns in a point
!     sepvector      o    sepran vector consisting of nunkp degrees of freedom
!                         per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcopyus2vec1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcopyus2vec1'
         write(irefwr,1) 'idegfd, nunkp, npoint', idegfd, nunkp, npoint
         call prinrl ( inpvec, npoint, 'inpvec' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      sepvector(idegfd,:) = inpvec

1000  call erclos ( 'prcopyus2vec1' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( sepvector, npoint, nunkp, 'sepvector' )
         write(irefwr,*) 'End prcopyus2vec1'

      end if  ! ( debug )

      end
