      subroutine prcopyvec2us1 ( sepvector, outvec, nunkp, idegfd, &
                                 npoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy part of sepran vector to user vector
!     The user vector has 1 unknown per point and is defined in all nodes
!     The sepran vector is defined in all nodes and has exactly nunkp
!     degrees of freedom per point
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nunkp, idegfd, npoint
      double precision, intent(in) :: sepvector(nunkp,npoint)
      double precision, intent(out) :: outvec(npoint)

!     idegfd         i    Degree of freedom sequence number
!     npoint         i    Number of nodal points
!     nunkp          i    Maximum number of unknowns in a point
!     outvec         o    Output vector of length npoint containing the
!                         copied vector
!     sepvector      i    sepran vector consisting of nunkp degrees of freedom
!                         per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcopyvec2us1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcopyvec2us1'
         write(irefwr,1) 'idegfd, nunkp, npoint', idegfd, nunkp, npoint
         call prinrl1 ( sepvector, npoint, nunkp, 'sepvector' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      outvec = sepvector(idegfd,:)

1000  call erclos ( 'prcopyvec2us1' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( outvec, npoint, 'outvec' )
         write(irefwr,*) 'End prcopyvec2us1'

      end if  ! ( debug )

      end
