      subroutine prevalveccexpp ( work, iwork, ncount, lenvec, &
                                  irefwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-12-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Evaluate powers in vector expression
!     (complex version)
! **********************************************************************
!
!                       KEYWORDS
!
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), ncount, lenvec, irefwork(ncount)
      double complex work(lenvec,*)

!     irefwork      i/o   integer work array of length ncount
!                         if irefwork(i) = 0, it concerns a constant
!                         otherwise (1) a vector
!     iwork         i/o   integer work array of length ncount
!                         iwork is used to store the integer information of the
!                         expression
!     lenvec         i    Length of all vectors
!     ncount         i    Length of iwork
!     work          i/o   Work array of length ncount
!                         work is used to store the real information of the
!                         expression
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision value
      logical debug, funcfound

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     funcfound      If true a function has been found
!     i              Counting variable
!     j              Counting variable
!     value          Real value if it concerns a scalar
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3164   Exponent is a vector instead of a number
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prevalveccexpp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prevalveccexpp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      funcfound = .false.

!     --- For i := 1(1)ncount do

150   do i = 1, ncount

!     --- Next all powers are evaluated in order to satisfy the
!         priority rules

         if ( iwork(i)==3 ) then

!        --- iwork(i)=3: power
!            work(i-1) := work(i-1)**work(i)
!            shift work and iwork

             if ( irefwork(i)/=0 ) then

!            --- The power must correspond to a number

                call errsub ( 3164, 0, 0, 0 )
                go to 1000

             end if  ! ( irefwork(i)/=0 )

             funcfound = .true.

             value = work(1,i)
             if ( irefwork(i-1)==0 ) then

!            --- work(i-1) is number

                value = work(1,i-1)**value
                work(1:lenvec,i-1) = value

             else

!            --- work(i-1) is vector

                work(1:lenvec,i-1) = work(1:lenvec,i-1)**value

             end if  ! ( irefwork(i-1)==0 )
             ncount = ncount-1
             do j = i, ncount
                work(1:lenvec,j) = work(1:lenvec,j+1)
                iwork(j) = iwork(j+1)
                irefwork(j) = irefwork(j+1)
             end do
             work(1:lenvec,ncount+1) = (0d0,0d0)
             iwork(ncount+1) = 0
             irefwork(ncount+1) = 0
             go to 150

         end if

      end do

1000  call erclos ( 'prevalveccexpp' )
      if ( debug ) then

!     --- Debug information

         if ( funcfound ) then
            call prinin ( iwork, ncount, &
                          'after evaluation of power' )
            call prinrl2 ( work, lenvec, ncount, &
                           'after evaluation of power' )
         end if  ! ( funcfound )
         write(irefwr,*) 'End prevalveccexpp'

      end if  ! ( debug )

      end
