      subroutine prevalvecexprm ( work, iwork, ncount, text, &
                                  lenvec, irefwork  )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-12-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Evaluate multiplications/subdivisions in vector expression
!
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     expression
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), ncount, lenvec, irefwork(ncount)
      double precision work(lenvec,*)
      character (len=*) text

!     irefwork      i/o   integer work array of length ncount
!                         if irefwork(i) = 0, it concerns a constant
!                         otherwise (1) a vector
!     iwork         i/o   integer work array of length ncount
!                         iwork is used to store the integer information of the
!                         expression
!     lenvec         i    Length of all vectors
!     ncount         i    Length of iwork
!     text           i    String variable containing some text for error message
!     work          i/o   Work array of length ncount
!                         work is used to store the real information of the
!                         expression
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision value
      logical debug, funcfound, extradebug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     extradebug     If true extra debugging statements are carried out
!     funcfound      If true a function has been found
!     i              Counting variable
!     j              Counting variable
!     value          Real value if it concerns a scalar
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1693   Division by 0
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prevalvecexprm' )
      debug = .false. .and. ioutp>=0
      extradebug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prevalvecexprm'
         call prinin ( iwork, ncount, 'iwork' )
         call prinin ( irefwork, ncount, 'irefwork' )
  4      format ( a, 1x, i3, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      funcfound = .false.

!     --- For i := 1(1)ncount do

200   do i = 2, ncount

!     --- Next all multiplications and subdivisions

         if  ( iwork(i)==2 ) then

!        --- iwork(i)=2: multiplication
!            work(i-1) := work(i)*work(i-1)
!            shift work and iwork

             if ( irefwork(i)==0 ) then

!            --- work(i) = number

               if ( irefwork(i-1)==0 ) then

!              --- work(i-1) = number

                  value = work(1,i)*work(1,i-1)
                  work(1:lenvec,i-1) = value

               else

!              --- work(i-1) = vector

                  value = work(1,i)
                  work(1:lenvec,i-1) = value*work(1:lenvec,i-1)

               end if  ! ( irefwork(i-1)==0 )

            else

!           --- work(i) = number

               if ( irefwork(i-1)==0 ) then

!              --- work(i-1) = number

                  value = work(1,i-1)
                  work(1:lenvec,i-1) = value*work(1:lenvec,i)
                  irefwork(i-1)=1

               else

!              --- work(i-1) = vector

                  work(1:lenvec,i-1) = &
                     work(1:lenvec,i)*work(1:lenvec,i-1)

               end if  ! ( irefwork(i-1)==0 )

            end if  ! ( irefwork(i)==0 )
            ncount = ncount-1
            do j = i, ncount
               work(1:lenvec,j) = work(1:lenvec,j+1)
               iwork(j) = iwork(j+1)
               irefwork(j) = irefwork(j+1)
            end do
            work(1:lenvec,ncount+1) = 0d0
            iwork(ncount+1) = 0
            irefwork(ncount+1) = 0
            funcfound = .true.
            go to 200

         else if ( iwork(i)==-2 ) then

!        --- iwork(i)=-2: division
!            work(i-1) := work(i-1)/work(i)
!            shift work and iwork

            if ( irefwork(i)==0 ) then

!           --- work(i) = number

               value = work(1,i)
               if ( abs(value)<=0d0 ) then

!              --- Division by zero

                  call errchr ( text, 1 )
                  call errsub ( 1693, 0, 0, 1 )

               else

!              --- Standard division

                  if ( irefwork(i-1)==0 ) then

!                 --- work(i-1) = number

                     value = work(1,i-1)/value
                     work(1:lenvec,i-1) = value

                  else

!                 --- work(i-1) = vector

                     work(1:lenvec,i-1) = work(1:lenvec,i-1)/value

                  end if  ! ( irefwork(i-1)==0 )

               end if  ! ( abs(value)<=0d0 )

            else

!           --- work(i) = vector

               if ( debug ) call prinrl ( work(1,i), 5, 'work(i)' )

               do j = 1, lenvec

                  if ( abs(work(j,i-1))<=0d0 ) then

!                 --- left-hand side = 0, no action

                  else if ( abs(work(j,i))<=0d0 ) then

!                 --- Division by zero

                     call errchr ( text, 1 )
                     call errsub ( 1693, 0, 0, 1 )

                  else

!                 --- Division

                     work(j,i-1) = work(j,i-1)/work(j,i)

                  end if  ! ( abs(work(k,i))<=0d0 )

               end do  ! j = 1, lenvec

               irefwork(i-1) = 1  !vector

            end if  ! ( irefwork(i)==0 ) then

            ncount = ncount-1
            do j = i, ncount
               work(1:lenvec,j) = work(1:lenvec,j+1)
               iwork(j) = iwork(j+1)
               irefwork(j) = irefwork(j+1)
            end do
            work(1:lenvec,ncount+1) = 0d0
            iwork(ncount+1) = 0
            irefwork(ncount+1) = 0
            funcfound = .true.
            go to 200

         end if

      end do

1000  call erclos ( 'prevalvecexprm' )
      if ( debug ) then

!     --- Debug information

         if ( funcfound ) then
            call prinin ( iwork, ncount, &
                          'after evaluation of multiplication' )
            call prinin ( irefwork, ncount, 'irefwork' )
            if ( extradebug ) then
               call prinrl2 ( work, lenvec, ncount, 'work' )
            else
               do i = 1, ncount
                  write(irefwr,4) 'work ', i, work(1:min(5,lenvec),i)
               end do  ! i = 1, ncount
            end if  ! ( extradebug )
         end if  ! ( funcfound )
         write(irefwr,*) 'End prevalvecexprm'

      end if  ! ( debug )

      end
