      subroutine prfillindex1 ( kmeshc, index1, index2, inpelm, indprp, &
                                kprobp, inunkp, npoint, indprf, kprobf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-12-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays index1 and index2 with information about
!     nodes and unknowns per element
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, kmeshc(inpelm), index1(inpelm), index2(*), indprp, &
              npoint, kprobp(npoint,*), inunkp, indprf, kprobf(*)

!     index1         o    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     index2         o    Array of length icount containing the positions of the
!                         degrees of freedom in the element with respect to USOL
!     indprf         i    indication if kprob part f has been filled (>0)
!                         or not (0)
!     indprp         i    Indication if array kprob part p has been filled (1)
!                         or not (0)
!     inpelm         i    number of points per element
!     inunkp         i    Number of degrees of freedom in point
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     kprobp         i    SEPRAN array KPROB part p containing the positions of
!                         the unknowns per point
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, icount, inodp, inusol
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     icount         Counter
!     inodp          Nodal point number
!     inusol         Position of unknown in array USOL
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillindex1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillindex1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, inpelm
         index1 = kmeshc
      end do

      if ( indprp/=0 ) then

!     --- indprp>0 use kprobp

         icount = 0
         do i = 1, inpelm
            inodp = index1(i)
            do j = 1, inunkp

               if ( kprobp(inodp,j)>0 ) then

!              --- unknown is present in point

                  icount = icount+1
                  index2(icount) = kprobp(inodp,j)

               end if

            end do
         end do

      else if ( indprf/=0 ) then

!     --- indprf>0 use kprobf

         icount = 0
         do i = 1, inpelm
            inodp = index1(i)
            inusol=kprobf(inodp)
            inunkp=kprobf(inodp+1)-inusol
            do j = 1, inunkp
               icount = icount+1
               index2(icount) = inusol+j
            end do
         end do

      else

!     --- indprp = 0 and indprf = 0

         if ( inunkp==1 ) then

!        --- inunkp = 1

            index2(1:inpelm) = index1(1:inpelm)

         else

!        --- inunkp>1

            do i = 1, inpelm
               do j = 1, inunkp
                  index2((i-1)*inunkp+j) = (index1(i)-1)*inunkp+j
               end do
            end do

         end if

      end if

1000  call erclos ( 'prfillindex1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfillindex1'

      end if  ! ( debug )

      end
