      subroutine prfillnodsurf5 ( kmesh, nsurfaces, surfaces, kmeshn, &
                                  nodes, nnodes, kmesht, kmeshc, numnod, &
                                  ihelp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nodes for set of surfaces
!     Only a restricted number of nodes per element is used
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), nsurfaces, surfaces(nsurfaces), kmeshn(*), &
              nodes(*), nnodes, kmesht(*), kmeshc(*), numnod, ihelp(*)

!     ihelp          i    contains the realtive node numbers per element
!                         that should be used in case numnod>0
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshn         i    KMESH part n, contains nodal point numbers of the
!                         surfaces
!     kmesht         i    kmesh part t, contains element numbers of elements
!                         with highest dimensionality
!     nnodes        i/o   Number of nodes
!     nodes         i/o   Contains the node numbers that must be filled
!     nsurfaces      i    Number of surfaces in array surfaces
!     numnod         i    If>0, only numnod nodes per element must be used
!                         In that case ihelp is used
!                         If 0 all nodes are used
!     surfaces       i    Contains the surface numbers to be used for filling
!                         nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERSURFSNOD3    Store all nodes in the surfaces stored in array surfaces
!                    in array nodes
!                    It is assumed that kmesh part n has been filled
!                    Only a restricted number of nodes per element is used
!     ERSURFSNOD4    Store all nodes in the surfaces stored in array surfaces
!                    in array nodes
!                    It is assumed that kmesh part n has not been filled
!                    Only a restricted number of nodes per element is used
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillnodsurf5' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillnodsurf5'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nnodes = 0

      if ( kmesh(28)>0 ) then

!     --- kmeshn has been filled

         call ersurfsnod3 ( nsurfaces, surfaces, kmeshn, kmesh, &
                            nodes, nnodes, numnod, ihelp )

      else

!     --- kmeshn has not been filled

         call ersurfsnod4 ( nsurfaces, surfaces, kmesht, kmesh, &
                            nodes, nnodes, kmeshc, numnod, ihelp )

      end if  ! ( kmesh(28)>0 )

1000  call erclos ( 'prfillnodsurf5' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfillnodsurf5'

      end if  ! ( debug )

      end
