      subroutine prfindelemnods ( kmeshd, nodes, nnodes, npoint, iwork, &
                                  elements, numelements, lenelements )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-12-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all elements that are connected to the nodes stored in
!     array nodes and store the elments in array elements
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     element
!     find
!     find_element
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshd(*), nodes(*), nnodes, npoint, iwork(*), &
              elements(*), numelements, lenelements

!     elements       o    Array of length lenelements in which the
!                         elements corresponding to all nodes in inodes
!                         are stored
!     iwork         i/o   integer work array of length nelem
!                         At input iwork must be equal to 0, at output
!                         it is reset to 0
!                         iwork(ielem) contains the number of nodes from array
!                         inodes connected to ielem
!     kmeshd         i    KMESH part d
!     lenelements    i    Declared length of array elements
!     nnodes         i    Number of nodes in nodes
!     nodes          i    Contains the node numbers form which the common
!                         elements must be found
!     npoint         i    Number of nodal points
!     numelements    o    Number of elements stored in elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRFINDELCLEAR  Clear array iwork
!     PRINNODEELMS2  Raise iwork for all elements corresponding to a node
!     PRINNODEELMS3  Find all elements corresponding to a node with
!                    iwork(ielem) = nnodes1
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     numelements := 0
!     for i := 1 (1) nnodes-1 do
!        For all elements ielem corresponding to nodes(i) do
!           iwork(ielem) := iwork(ielem)+1
!     For all elements ielem corresponding to nodes(nnodes) do
!        if ( iwork(ielem) = nnodes-1 ) then
!           numelements := numelements+1; elements(numelements) := ielem
!     Clear array iwork
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfindelemnods' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfindelemnods'

      end if
      if ( ierror/=0 ) go to 1000

!     --- for i := 1 (1) nnodes-1 do
!            For all elements ielem corresponding to nodes(i) do
!               iwork(ielem) := iwork(ielem)+1

      do i = 1, nnodes-1
         call prinnodeelms2 ( kmeshd, nodes(i), npoint, iwork )
      end do  ! i = 1, nnodes-1

!     --- For all elements ielem corresponding to nodes(nnodes) do
!            if ( iwork(ielem) = nnodes-1 ) then
!               numelements := numelements+1; elements(numelements) := ielem

      call prinnodeelms3 ( kmeshd, nodes(nnodes), npoint, iwork, &
                           nnodes-1, elements, numelements, &
                           lenelements )

!     --- Clear array iwork

      call prfindelclear ( nodes, nnodes, iwork, kmeshd, npoint )

1000  call erclos ( 'prfindelemnods' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfindelemnods'

      end if

      end
