      subroutine prfindsurfnod1 ( inode, kmeshn, nsurfs, numsurfaces, &
                                  surfacesfound, kmeshr, kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 05-08-2005 Correction single surfaces
!        version  1.0  date 08-04-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if node inode is in one or more of the surfaces
!     and return with corresponding surface numbers
! **********************************************************************
!
!                       KEYWORDS
!
!     find
!     node
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inode, kmeshn(*), nsurfs, numsurfaces, surfacesfound(*), &
              kmeshr(*), kmesh(*)

!     inode          i    Node sequence number
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kmeshn         i    KMESH part n, contains nodal point numbers of the
!                         surfaces
!     kmeshr         i    Kmesh part r, see the PG 13.1
!     nsurfs         i    Number of surfaces in mesh
!     numsurfaces    o    Number of surfaces stored in array surfacesfound
!     surfacesfound  o    Array with surface numbers in which inode can be found
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isurnr, numnodes, istart, inpelm, inelem
      logical single, debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     inelem         Number of elements in a surface
!     inpelm         number of points per element in surface
!     istart         First position used at a part
!     isurnr         surface sequence number
!     numnodes       Number of nodal points in array nodes
!     single         If true the surface is a single surface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inifindentry

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIFINDENTRY   Check if integer number can be found in integer array
!                    and return with the position in the array
!     INIINFSURF     Find starting address of surface isurnr in array kmeshn
!                    Also find number of element and number of points per
!                    element in isurnr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all single surfaces isurnr do
!        if ( node in isurnr ) then
!           store surface number
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfindsurfnod1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfindsurfnod1'

      end if
      if ( ierror/=0 ) go to 1000
      numsurfaces = 0

!     --- For all single surfaces isurnr do

      do isurnr = 1, nsurfs

         single = kmesh(32)==0
         if ( .not. single ) single = kmeshr(isurnr+1)-kmeshr(isurnr)==0
         if ( single ) then

!        --- Single surface
!            if ( node in isurnr ) then
!               store surface number

            call iniinfsurf ( kmeshn, istart, inpelm, inelem, nsurfs, &
                              isurnr )
            numnodes = inpelm*inelem   ! complete topology of surface
            istart = istart+1          ! now it is the first position in kmeshn

            if ( inifindentry ( inode, kmeshn(istart), numnodes )>0 ) &
               then

!           --- node found in surface; store

               numsurfaces = numsurfaces+1
               surfacesfound(numsurfaces) = isurnr

            end if  ! ( inifindentry ( inode, kmeshn(istart), numnodes )>0 )

         end if  ! ( single )

      end do  ! isurnr = 1, nsurfs

1000  call erclos ( 'prfindsurfnod1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfindsurfnod1'

      end if

      end
