      subroutine prinbf ( length )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.13 date 19-12-2014 Print buffer length if ioutp>=2
!        version  5.12 date 21-11-2012 Long integers
!        version  5.11 date 17-01-2007 New format for text
!
!   copyright (c) 1984-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    check length of array buffer
!    print if necessary
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmessgf90'
      include 'SPcommon/cmesscf90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cbuffrf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cdebugf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer (kind=8) length

!     length     length of array buffer at present
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nch, lentext
      character ( len=80 ) text

!     i              Counting variable
!     lentext        Number of characters in text
!     nch            Number of characters in a string
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINTLONG     Put long integer in error message
!     ERRSUB         Error messages
!     INSTOP         Stop the program
!     PRINBUFARS     Print the active arrays present in array buffer
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      17   length of ibuffr too small
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      nbufmx = max(nbufmx,nbuffr)
      if ( length>nbuffr ) then
         call eropen ( 'prinbf' )
         call errintlong ( length+nbufmx-nbuffr, 1 )
         call errintlong ( nbufmx, 2 )
         call errsub ( 17, 2, 0, 0 )
         call erclos ( 'prinbf' )
         call prinbufars
      end if
      if ( length>kbuffr .and. ioutp>=2 ) then

!     --- Length has increased, print last 4 subroutines used

         text = ' '
         lentext = 1
         do i = max(1,levsub-3), levsub
            nch = index(namsub(i),' ')-1  ! last non-blank
            if ( nch==-1 ) nch = len(namsub(i))
            text(lentext:lentext+nch) = namsub(i)(1:nch)//':'
            lentext = lentext+nch+1
         end do  ! i = max(1,levsub-3), levsub
         write(irefwr,100) length, text(1:lentext-2)
100      format(' length of buffer array:', i9, 1x, a )

      end if
      kbuffr = max(kbuffr,length)
      if ( ierror/=0 ) call instop

      end
