      subroutine prinslbf ( ibuffr, buffer, ivectr, kmesh, kprob, iref )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 29-05-2014 remove call to time subroutine
!        version  1.3  date 21-11-2012 Long integers
!        version  1.2  date 12-08-1999 New call to erclmn
!
!   copyright (c) 1999-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   print real/complex  sepran array except the large matrix
!   standard format 5d16.9 is used
!   This subroutine is meant to be used in combination with subroutine
!   prinslbf.
!   prinslbf writes a solution in character format to a file, &
!   readsl reads the solution
!         ivectr(2)=110,115 allowed
! **********************************************************************
!
!                       KEYWORDS
!
!     output
!     print
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cbuffrf90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*),kprob(*),ivectr(*),iref, ibuffr(*)
      double precision buffer(*)

!     buffer    i/o   Buffer array
!     ibuffr    i/o   Integer buffer array in which all large data is stored
!     iref       i    Reference number of file to which solution must be written
!                     possibilities:  see user manual
!                     if iref<0, the file is written unformatted, otherwise it
!                                  is supposed to be formatted
!     ivectr     i    integer array denoting the array to be printed
!     kmesh      i    array containing information of the mesh
!     kprob      i    array containing information of degrees of freedom
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipkprh, ipvect, ipwork
      integer i, indprf, indprh, indprp, iprob, &
              leng, nphys, nprob, nusol
      logical compl

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     i              Counting variable
!     indprf         indication if kprob part f has been filled (>0) or not (0)
!     indprh         indication if kprob part h has been filled (>0) or not (0)
!     indprp         Indication if array kprob part p has been filled (1)
!                    or not (0)
!     ipkprh         starting address of array kprob part h
!     iprob          Actual problem number
!     ipvect         ?
!     ipwork         Starting address in array IBUFFR of work array
!     leng           Help variable to compute the length of some arrays
!     nphys          Number of physical unknowns, or maximal degrees of freedom
!                    per point
!     nprob          Number of various problems
!     nusol          Total number of unknowns
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) iniget, inidgt, inidbl

!     ER0003         Reorder solution array into array with output sequence
!     ER0044          ?
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI060         Reactivate existing array in IBUFFR
!     INI062         reserve space for temporary arrays
!     INI065         compute pointers to degrees of freedom of vectors
!     INIDBL         Transform pointer for IBUFFR to pointer for BUFFER
!     INIDGT         Compute pointer of array in BUFFER
!     INIGET         Compute pointer of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      26   Array KPROB has not been filled
!      75   Array KMESH has not been filled
!     699   Problem number incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinslbf' )
      if(kmesh(2)/=100) then
         call errint ( kmesh(2), 1 )
         call errsub ( 75, 1, 0, 0)
      end if
      if(kprob(2)/=101) then
         call errint ( kprob(2), 1 )
         call errsub ( 26, 1, 0, 0)
      end if
      if ( ierror/=0 ) go to 1000
      iprob  = ivectr(4) / 1000
      iprob  = iprob + 1
      nprob  = max ( 1, kprob(40) )
      if ( iprob<1 .or. iprob>nprob ) then
         call errint ( iprob, 1 )
         call errint ( nprob, 2 )
         call errsub ( 699, 2, 0, 0)
      end if
      if ( ivectr(2)==116 .or. ivectr(2)==126 .or. &
           ivectr(2)==129 ) then
         call ini060 ( ibuffr, -ivectr(1) )
         indprh = 0
      else
         call ini065 ( ibuffr, ivectr, kprob, indprf, indprh, indprp, &
                       nphys )
      end if
      if ( ierror/=0 ) go to 1000
      nusol = ivectr(5)
      compl = mod(ivectr(3),10)==1
      leng=1
      if(compl) leng=2
      if (indprh/=0) then
         call ini062 ( ibuffr, nusol*intlen*leng,ipwork)
         ipwork = inidbl(ipwork)
         ipvect = inidgt(ivectr(1))
         ipkprh = iniget(indprh)
         call er0003 ( buffer(ipvect), ibuffr(ipkprh), nusol, &
                       buffer(ipwork), compl)
         ipvect = ipwork
      else
         ipvect = inidgt(ivectr(1))
      end if
      if ( ierror/=0 ) go to 1000

      if ( iref>0 ) then
         write(iref,100) (ivectr(i),i=1,5)
100      format(5i6)
      else
         write(-iref) (ivectr(i),i=1,5)
      end if
      call er0044 ( buffer(ipvect), nusol*leng, iref )
1000  call erclos ( 'prinslbf' )
      end
