      subroutine prmultadd2 ( nusol, u1, u2, a1, a2, ures )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-07-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute a1 u1 + a2 u2 and store in ures
!     u4 may be one of the three vectors
! **********************************************************************
!
!                       KEYWORDS
!
!     multiplication
!     linear_combination
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nusol
      double precision u1(nusol), u2(nusol), a1, a2, &
                       ures(nusol)

!     a1             i    multiplication constant
!     a2             i    multiplication constant
!     nusol          i    Total number of unknowns
!     u1             i    First vector
!     u2             i    Second vector
!     ures           i    Result vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmultadd2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmultadd2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, nusol
         ures(i) = a1*u1(i) + a2*u2(i)
      end do  ! i = 1, nusol

1000  call erclos ( 'prmultadd2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmultadd2'

      end if  ! ( debug )

      end
