      subroutine prnormtrian ( x1, x2, x3, normal )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-11-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the normal on a triangle
!     3d only
! **********************************************************************
!
!                       KEYWORDS
!
!     normal_vector
!     triangle
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision x1(3), x2(3), x3(3), normal(3)

!     normal         o    normal vector
!     x1             i    coordinates of first point
!     x2             i    coordinates of second point
!     x3             i    coordinates of thirdf point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision t1(3), t2(3)
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     t1             First tangential vector
!     t2             Second tangential vector
!                    This one is not perpendicular to t1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ININORMOUTPROD Compute the normalized outer product of vectors v1 and v2
!                    and store in v3
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     let the nodes of the trianlge have coordinates x1, x2 and x3
!     Then t1 = (x2-x1)/||x2-x1|| is the first tangential vector in P1
!          t2* = (x3-x2)/||x3-x2|| and t3* = (x1-x3)/||x1-x3||
!          t2 = a t2* + t3* &
!     From (t2,t1) = 0, we get a (t2*,t1) + (t3*,t1) = 0
!          Hence t2** = -(t3*,t1)/(t2*,t1) t2* + t3* &
!          t2 = t2**/||t2**||
!     The normal n1 = defined as n1 = t1 x t2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prnormtrian' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prnormtrian'
         write(irefwr,2) 'x1', x1
         write(irefwr,2) 'x2', x2
         write(irefwr,2) 'x3', x3
  2      format ( a, 1x, (5e20.12) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute tangential vectors

      t1 = x2-x1
      t2 = x3-x1

      if ( debug ) then
         write(irefwr,2) 't1', t1
         write(irefwr,2) 't2', t2
      end if  ! ( debug )

!     --- n = t1 x t2

      call ininormoutprod ( t1, t2, normal )  ! also normalized

1000  call erclos ( 'prnormtrian' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'n', normal
         write(irefwr,*) 'End prnormtrian'

      end if  ! ( debug )

      end
