      subroutine procopybf1 ( ibuffr, buffer, kprob1, kprob2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 05-05-2013 Correct kprob part g
!        version  1.1  date 19-02-2003 Changes for kprobfict (RvL)
!        version  1.0  date 21-11-2002
!
!   copyright (c) 2002-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Copy the essential parts of array kprob1 into kprob2
!    Only those parts needed for subroutine intmsh are copied
!    In 2D it concerns the parts f, h and p and also the part refering to
!    ficitious domain
!    This routine is identical to procopybf except for
!    the fictitious domain part
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     copy
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kprob1(*), kprob2(*), ibuffr(*)
      double precision buffer(*)

!     buffer    i/o   Buffer array
!     ibuffr    i/o   Integer buffer array in which all large data is stored
!     kprob1     i    kprob array to be copied
!     kprob2    i/o   Copy of array kprob
!                     All positions must have been initialized before
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxprob, maxstore
      parameter ( maxprob=10, maxstore=100 )
      integer length, i, indprf(maxprob), indprh(maxprob), icount, &
              indprp(maxprob), nprob, iprob, ipkprb, indpre(maxstore), &
              ndtvec, kprobe, kelgrp, ivec, iadres, mmiinput, &
              mmrinput, indprg(maxprob), indprfict(maxprob)
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     iadres         Adress in array
!     icount         Counting variable
!     indpre         Array to store memory management sequence numbers
!                    of parts e
!     indprf         Array to store memory management sequence numbers
!                    of parts f
!     indprfict      Array to store memory management sequence numbers
!                    of parts kprobfict
!     indprg         Array to store memory management sequence numbers of
!                    parts g
!     indprh         Array to store memory management sequence numbers
!                    of parts h
!     indprp         Array to store memory management sequence numbers
!                    of parts p
!     ipkprb         Starting address of array KPROB for actual problem
!                    number - 1
!     iprob          Actual problem number
!     ivec           Sequence number of vector of special structure
!     kelgrp         sum over all standard elements of number of points in
!                    standard elements
!     kprobe         starting address in kprob part e
!     length         Help parameter to indicate the length of arrays
!     maxprob        Maximum number of problems allowed
!     maxstore       Length of array indpre
!     mmiinput       Memory management sequence number of array iinput
!     mmrinput       Memory management sequence number of array rinput
!     ndtvec         Number of vectors of special structure
!     nprob          Number of various problems
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer iniprb

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI066TEXT     delete temporary array
!     INICOPARR      Copy one sepran array into the other, which may already
!                    have been created before.
!     INIPRB         Get value of ipkprb as function of iprob
!     PRININ         print 1d integer array
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     432   Too many problems submitted
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Check length of array kprob2
!    Save memory management positions of old arrays of kprob2
!    Copy parts of array kprob
!    Copy kprob part i (co-ordinates)
!    Copy kprob part c (topology)
!    Copy kprob part m (curve description)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'procopybf1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from procopybf1'

      end if

!     --- Check length of array kprob2

      nprob = kprob1(40)
      length = 0
      icount = 0
      i = kprob2(1)
      do iprob = 1, nprob
         ipkprb = iniprb(iprob,kprob1)
         kprob2(ipkprb+1) = i - length
         i = i - length
         length = kprob1(ipkprb+3)
         icount = icount+length
         call prinlg ( 'kprob2', length, buffer, kprob2(ipkprb+1) )
      end do
      length = icount

!     --- Save memory management positions of old arrays of kprob2

      nprob = kprob2(40)
      if ( nprob>maxprob ) then
         call errint ( nprob, 1 )
         call errint ( maxprob, 1 )
         call errsub ( 432, 2, 0, 0 )
         go to 1000
      end if
      icount = 0
      do iprob = 1, nprob
         ipkprb = iniprb(iprob,kprob2)
         indprf(iprob) = kprob2(ipkprb+19)
         indprg(iprob) = kprob2(ipkprb+20)
         indprp(iprob) = kprob2(ipkprb+35)
         indprh(iprob) = kprob2(ipkprb+50)
         if ( kprob2(ipkprb+51)>0 ) then

!        --- Fictitous unknowns

            indprfict(iprob) = kprob2(ipkprb+kprob2(ipkprb+51)+3)

         else

!        --- No fictitous unknowns

            indprfict(iprob) = 0

         end if
         ndtvec = kprob2(ipkprb+6)
         kprobe = kprob2(ipkprb+18)
         kelgrp = kprob2(ipkprb+39)
         do ivec = 1, ndtvec
            iadres = ipkprb+kprobe-1+kelgrp*ndtvec+ivec
            if ( kprob2(iadres)>0 ) then
               icount = icount+1
               if ( icount<=maxstore ) then
                  indpre(icount) = kprob2(iadres)
               else
                  call ini066text ( ibuffr, kprob2(iadres), 'kprobe' )
               end if
            end if
         end do
      end do
      mmiinput = kprob2(41)
      mmrinput = kprob2(42)

!     --- Copy parts of array kprob

      do i = 2, length
         kprob2(i) = kprob1(i)
      end do

!     --- restore positions old arrays in ibuffr

      icount = 0
      do iprob = 1, min(nprob,kprob1(40))
         ipkprb = iniprb(iprob,kprob2)
         kprob2(ipkprb+19) = indprf(iprob)
         kprob2(ipkprb+20) = indprg(iprob)
         kprob2(ipkprb+35) = indprp(iprob)
         kprob2(ipkprb+50) = indprh(iprob)
         if ( kprob2(ipkprb+51)>0 ) then

!        --- Fictitous unknowns

            kprob2(ipkprb+kprob2(ipkprb+51)+3) = indprfict(iprob)

         end if
         ndtvec = kprob1(ipkprb+6)
         kprobe = kprob1(ipkprb+18)
         kelgrp = kprob1(ipkprb+39)
         do ivec = 1, ndtvec
            iadres = ipkprb+kprobe-1+kelgrp*ndtvec+ivec
            if ( kprob2(iadres)>0 ) then
               icount = icount+1
               if ( icount<=maxstore ) then
                  kprob2(iadres) = indpre(icount)
               else
                  kprob2(iadres) = 0
               end if
            end if
         end do
      end do

      if ( nprob==0 ) then
         do iprob = 1, kprob1(40)
            ipkprb = iniprb(iprob,kprob1)
            kprob2(ipkprb+19) = 0
            kprob2(ipkprb+20) = 0
            kprob2(ipkprb+35) = 0
            kprob2(ipkprb+50) = 0
            if ( kprob2(ipkprb+51)>0 ) &
               kprob2(ipkprb+kprob2(ipkprb+51)+3) = 0
            ndtvec = kprob2(ipkprb+6)
            kprobe = kprob2(ipkprb+18)
            kelgrp = kprob2(ipkprb+39)
            do ivec = 1, ndtvec
               iadres = ipkprb+kprobe-1+kelgrp*ndtvec+ivec
               if ( kprob2(iadres)>0 ) kprob2(iadres) = 0
            end do
         end do
      end if
      nprob = kprob1(40)
      do iprob = 1, nprob
         ipkprb = iniprb(iprob,kprob2)
         kprob2(ipkprb+21) = 0
         kprob2(ipkprb+22) = 0
         kprob2(ipkprb+24:ipkprb+28) = 0
         kprob2(ipkprb+36:ipkprb+38) = 0
         kprob2(ipkprb+41:ipkprb+42) = 0
         kprob2(ipkprb+53:ipkprb+57) = 0
      end do
      kprob2(41) = mmiinput
      kprob2(42) = mmrinput

!     --- Copy arrays iinput and rinput if available

      if ( kprob1(41)>0 ) then

         call inicoparr ( ibuffr, kprob1(41), kprob2(41), 'iinput-1', &
                          'iinput-2' )

      end if
      if ( kprob1(42)>0 ) then

         call inicoparr ( ibuffr, kprob1(42), kprob2(42), 'rinput-1', &
                          'rinput-2' )

      end if

      do iprob = 1, nprob
         ipkprb = iniprb(iprob,kprob2)

         if ( kprob1(ipkprb+18)>0 ) then

!        --- Array kprob part e has been filled
!            Check if there are parts stored in ibuffr and copy those parts

            ndtvec = kprob1(ipkprb+6)
            kprobe = kprob1(ipkprb+18)
            kelgrp = kprob1(ipkprb+39)
            do ivec = 1, ndtvec
               iadres = ipkprb+kprobe-1+kelgrp*ndtvec+ivec
               if ( kprob1(iadres)>0 ) then

!              --- part is stored in ibuffr, copy

                  call inicoparr ( ibuffr, kprob1(iadres), &
                                   kprob2(iadres), 'kprobe-1', &
                                   'kprobe-2' )

               end if

            end do

         end if

         if ( kprob1(ipkprb+19)>0 ) then

!        --- Copy kprobf

            call inicoparr ( ibuffr, kprob1(ipkprb+19), &
                             kprob2(ipkprb+19), 'kprobf-1', &
                             'kprobf-2' )

         end if

         if ( kprob1(ipkprb+20)>0 ) then

!        --- copy kprobg

            call inicoparr ( ibuffr, kprob1(ipkprb+20), &
                             kprob2(ipkprb+20), 'kprobg-1', &
                             'kprobg-2' )

         end if

         if ( kprob1(ipkprb+35)>0 ) then

!        --- copy kprobp

            call inicoparr ( ibuffr, kprob1(ipkprb+35), &
                             kprob2(ipkprb+35), 'kprobp-1', &
                             'kprobp-2' )

         end if

         if ( kprob1(ipkprb+50)>0 ) then

!        --- copy kprobh

            call inicoparr ( ibuffr, kprob1(ipkprb+50), &
                             kprob2(ipkprb+50), 'kprobh-1', &
                             'kprobh-2' )

         end if

         if ( kprob1(ipkprb+51)>0 ) then

!        --- make sure old kprobfict is in ibuffr

            call inicoparr ( ibuffr, kprob1(ipkprb+kprob1(ipkprb+51)+3), &
                             kprob2(ipkprb+kprob2(ipkprb+51)+3), &
                             'kprobfict-1', 'kprobfict-2' )

         end if
      end do

1000  call erclos ( 'procopybf1' )
      if ( debug ) then

!     --- Debug information

         call prinin ( kprob2, kprob1(3), 'kprob2 end of procopybf1' )
         write(irefwr,*) 'End procopybf1'

      end if

      end
