      subroutine prodel ( ibuffr, kprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 12-04-2003 Extension of kprob
!        version  2.2  date 20-11-2002 Correction for fictitious part
!        version  2.1  date 24-09-1999 Do not delete part j
!        version  2.0  date 10-04-1999 Extra parameter ibuffr
!        version  1.0  date 27-02-1997
!
!   copyright (c) 1997-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Delete array KPROB and all of its corresponding arrays
!
! **********************************************************************
!
!                       KEYWORDS
!
!     delete
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kprob(*), ibuffr(*)

!     ibuffr    i/o  Integer buffer array in which all large data is stored
!     kprob     i/o   KPROB array to be deleted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nprob, iprob, ipkprb, kprobe, ndtvec, kelgrp

!     i          Counting variable
!     ipkprb     Starting address of array KPROB for actual problem number - 1
!     iprob      Actual problem number
!     kelgrp     sum over all standard elements of number of points in standard
!                elements
!     kprobe     starting address in kprob part e
!     ndtvec     Number of vectors of special structure
!     nprob      Number of various problems
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer iniprb

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INI066TEXT     delete temporary array
!     INIPRB         Get value of ipkprb as function of iprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      26   Array KPROB has not been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prodel' )
      if ( kprob(2).ne.101 ) then

!     --- kprob has not been filled

         call errint ( kprob(2), 1 )
         call errsub ( 26, 1, 0, 0 )
         go to 1000
      end if
      nprob = kprob(40)
      do iprob = 1, nprob
         ipkprb = iniprb(iprob,kprob)
         kprobe = kprob(ipkprb+18)
         if ( kprobe.gt.0 ) then

!        --- Delete KPROB part e

            ndtvec = kprob(ipkprb+6)
            kelgrp = kprob(ipkprb+39)
            kprobe = kprobe+ndtvec*kelgrp+ipkprb
            do i = 0, ndtvec-1
               if ( kprob(kprobe+i).gt.0 ) &
                  call ini066text ( ibuffr, kprob(kprobe+i), 'kprobe' )
            end do
         end if
         if ( kprob(ipkprb+19).gt.0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+19), 'kprobf' )
         if ( kprob(ipkprb+20).gt.0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+20), 'kprobg' )
         if ( kprob(ipkprb+21).gt.0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+21), 'kprobh' )
         if ( kprob(ipkprb+22).gt.0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+22), 'kprobi' )
         if ( kprob(ipkprb+24).gt.0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+24), 'kprobk' )
         if ( kprob(ipkprb+25).gt.0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+25), 'kprobl' )
         if ( kprob(ipkprb+26).gt.0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+26), 'kprobm' )
         if ( kprob(ipkprb+27).gt.0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+27), 'kprobn' )
         if ( kprob(ipkprb+35).gt.0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+35), 'kprobp' )
         if ( kprob(ipkprb+36).gt.0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+36), 'kprobq' )
         if ( kprob(ipkprb+37).gt.0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+37), 'kprobr' )
         if ( kprob(ipkprb+38).gt.0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+38), 'kprobs' )
         if ( kprob(ipkprb+41).gt.0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+41), 'iinput' )
         if ( kprob(ipkprb+42).gt.0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+42), 'rinput' )
         if ( kprob(ipkprb+43).gt.0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+43), 'kprobt' )
         if ( kprob(ipkprb+44).gt.0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+44), 'kprobu' )
         if ( kprob(ipkprb+46).gt.0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+46), 'kprobv' )
         if ( kprob(ipkprb+47).gt.0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+47), 'kprobw' )
         do i = ipkprb+51, ipkprb+51
            if ( kprob(i).gt.0 )  then

!           --- Fictitious domain part

               call ini066text ( ibuffr, kprob(ipkprb+kprob(i)+3), &
                                 'kprobfict' )

            end if
         end do
         if ( kprob(ipkprb+53).gt.0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+53), 'kprobaa' )
         if ( kprob(ipkprb+55).gt.0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+55), 'blockunkinfo' )
         if ( kprob(ipkprb+56).gt.0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+56), 'sendinfo' )
         if ( kprob(ipkprb+57).gt.0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+57), 'innercontr' )
         if ( kprob(ipkprb+58).gt.0 ) &
            call ini066text ( ibuffr, kprob(ipkprb+58), 'kprobab' )

      end do
      do i = 4, kprob(3)
         kprob(i) = 0
      end do
      kprob(2) = 0
1000  call erclos ( 'prodel' )
      end
