      subroutine prputvalarr ( buffer, ibuffr, kprob, isol, nodes, &
                               nnodes, idegfd, uservec, namear, iprob, &
                               kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-05-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy the contents of a user array into a sepran solution array
!     for a set of nodes and a specific degree of freedom
! **********************************************************************
!
!                       KEYWORDS
!
!     put
!     value
!     variable
!     array
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), kprob(*), nnodes, nodes(nnodes), idegfd, &
              isol(5), iprob, kmesh(*)
      double precision buffer(*), uservec(nnodes)
      character (len=*) namear

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     idegfd         i    Degree of freedom sequence number
!     iprob          i    Actual problem number
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution array
!                         The values are copied in the corresponding array
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     namear         i    Name of array
!     nnodes         i    Number of nodes in array nodes
!     nodes          i    Contains the node numbers that must be filled
!     uservec        i    Contains the values that must be copied
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipusol, ipkprf
      integer indprf, indprp, nunkp, nphys, itypvc, ivec
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     indprf         indication if kprob part f has been filled (>0)
!                    or not (0)
!     indprp         Indication if array kprob part p has been filled (1)
!                    or not (0)
!     ipkprf         Starting address of array KPROB part f
!     ipusol         Starting address of solution array
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     nphys          Number of physical unknowns, or maximal degrees of
!                    freedom per point
!     nunkp          Maximum number of unknowns in a point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidgttext, inigettext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLARRCOP   Copy the contents of array uold into array unew for
!                    all nodes stored in array nodes and the degrees of
!                    freedom indicated in kdeg
!                    uold is a user array, unew a sepran array
!     EROPEN         Produces concatenated name of local subroutine
!     INI061         compute pointers to degrees of freedom of vectors
!     INI086         compute pointers to degrees of freedom of vectors
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INIGETTEXT     Compute pointer of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prputvalarr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prputvalarr'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( isol(1)>0 ) then

!     --- Activate array isol and corresponding arrays

         call ini061 ( ibuffr, isol, kprob, indprf, indprp, &
                       nunkp, nphys )

      else

!     --- Create array isol and corresponding arrays

         itypvc = 110
         ivec = 0
         call ini086 ( ibuffr, isol, itypvc, ivec, iprob, kmesh, &
                       kprob, indprf, indprp, nphys, namear )

      end if  ! ( isol(1)>0 )

      if ( ierror/=0 ) go to 1000

!     --- Get starting addresses

      ipusol = inidgttext ( isol(1), 'usol' )
      ipkprf = 1
      if ( indprf>0 ) ipkprf = inigettext ( indprf, 'kprobf' )

      call erfillarrcop ( nodes, idegfd, uservec, buffer(ipusol), &
                          indprf, ibuffr(ipkprf), nphys, nnodes )

1000  call erclos ( 'prputvalarr' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prputvalarr'

      end if  ! ( debug )

      end
