      function prvaluserp ( ibuffr, buffer, isol, kmesh, kprob, iuserp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-11-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Get the value of the solution vector in a given user point
!
! **********************************************************************
!
!                       KEYWORDS
!
!     value
!     user_point
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), isol(*), kmesh(*), kprob(*), iuserp
      double precision prvaluserp, buffer(*)

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     iuserp         i    User point sequence number
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     prvaluserp     o    value of the solution vector in a given user point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer node
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     node           Node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer prnodeuserp
      double precision prvalnode

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRNODEUSERP    Return with nodal point number corresponding to user point
!     PRVALNODE      Get the value of the solution vector in a given node
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Get node number corresponding to user point
!     Get value in node
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prvaluserp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prvaluserp'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get node number corresponding to user point

      node = prnodeuserp ( ibuffr, kmesh, iuserp )
      if ( debug ) write(irefwr,1) 'node', node
      if ( ierror/=0 ) go to 1000

!     --- Get value in node

      prvaluserp = prvalnode ( ibuffr, buffer, isol, kprob, node )

1000  call erclos ( 'prvaluserp' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prvaluserp'

      end if  ! ( debug )

      end
