      subroutine prvecfind ( ibuffr, kmesh, kprob, inpsep, iprob, ivec, &
                             itypvc, iseqvec, vecs, isol, numvecrhs, &
                             complexvec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 12-02-2015 Use prvecfind1
!        version  1.0  date 22-12-2013
!
!   copyright (c) 2013-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find the type of vector to be created for the left-hand side
!     This depends on the right-hand side
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     expression
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cpost5f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), kmesh(*), inpsep(*), iprob, ivec, itypvc, &
              kprob(*), iseqvec, vecs(*), isol(5,*), numvecrhs
      logical complexvec

!     complexvec     o    If true the vector to be filled is complex
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine ASKVEC
!     iprob          o    Actual problem number
!     iseqvec        i    Starting address of sequence numbers of vectors
!     isol           i    Standard SEPRAN array containing information of the
!                         solution arrays
!     itypvc         o    Indication of the type of solution vector:
!                         110: solution vector
!                         115, 116: vectors of special structure
!     ivec           o    Sequence number of vector of special structure
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kprob          i    Standard SEPRAN array, containing information of
!                         the problem
!     numvecrhs      i    Number of vectors in vecs
!     vecs           i    In this array of size numvecrhs the sequence numbers
!                         of the right-hand side vectors are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ndim, npoint
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     ndim           Dimension of the space.
!     npoint         Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
!     PRVECFIND1     Find the type of vector to be created for the left-hand side
!                    This depends on the right-hand side
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prvecfind' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prvecfind'
         write(irefwr,1) 'iseqvec', iseqvec
         call prinin1 ( isol, 4, 5, 'isol' )
  1      format ( a, 1x, (10i8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iprob = inpsep(2)
      ivec = inpsep(7)

      ndim = kmesh(6)    ! dimension of space
      npoint = kmesh(8)  ! number of nodes
      complexvec = .false.

      if ( debug ) &
         write(irefwr,1) 'iprob, ivec, npoint, ndim', &
                          iprob, ivec, npoint, ndim

      if ( iprob==0 .and. ivec==0 ) then

!     --- Both iprob = 0 and ivec = 0, hence check the vectors in the
!         right-hand side of the expression

         call prvecfind1 ( ibuffr, kmesh, kprob, iprob, ivec, &
                           itypvc, iseqvec, vecs, isol, numvecrhs, &
                           complexvec )

      else

!     --- either iprob or ivec given

         iprob = max(1,iprob)
         itypvc = 110
         if ( ivec>0 ) itypvc = 115

      end if  ! ( iprob==0 .and. ivec==0 )

1000  call erclos ( 'prvecfind' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'complexvec ', complexvec
         write(irefwr,1) 'iprob, ivec, itypvc', iprob, ivec, itypvc
         write(irefwr,*) 'End prvecfind'

      end if  ! ( debug )

      end
