      subroutine readb2 ( ibuffr, numarr, kmesh )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  5.4  date 15-11-2012 Long integers
!        version  5.3  date 01-08-2001 new definition kmesh part h
!        version  5.2  date 04-04-2001 Adaptation for empty surfaces
!
!   copyright (c) 1982-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    readb2 reads array KMESH from backing storage (file 2)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     backing_storage
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cbuffrf90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cfile2f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numarr, kmesh(*), ibuffr(*)

!    ibuffr    i/o   Integer buffer array in which all large data is stored
!    kmesh      o    Standard SEPRAN array containing information of the mesh
!    numarr     i    Array number with respect to the index in common cfile2
!                    for array KMESH
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) nbufol, ipwork, ipkmsh, istart
      integer length, index, icheck, lenglc, &
              i, j, nsurfs, ncurvs, leng1, leng2, leng3, leng4, &
              leng5, npointlc
      double precision dum(1)
      character *9 text

!     dum           Dummy array
!     i             General loop variable
!     icheck        Indication of the last version of writbs writb2) has been
!                   used (<=-1000000) or not (>-1000000)
!     index         Record number with respect to file 2 from which array
!                   must be read
!     ipkmsh        Starting address of array KMESH part h in array
!     ipwork        Starting position of part of array in IBUFFR
!     istart        Start position in array IBUFFR with respect to part of KMESH
!     j             General loop variable
!     leng1         Length of first part of surface in kmeshh
!     leng2         Length of second part of surface in kmeshh
!     leng3         Length of third part of surface in kmeshh
!     leng4         Length of fourth part of surface in kmeshh
!     leng5         Length of fifth part of surface in kmeshh
!     lenglc        Length of local array
!     length        Length of array KMESH
!     nbufol        Old value of nbuffr
!     ncurvs        Number of curves
!     npointlc      Local number of points
!     nsurfs        Number of surfaces
!     text          String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) iniget
      integer inilen

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INI000         Copy integer array to other integer array possibly itself
!     INI069         Reserve space at the end of IBUFFR for a semi-permanent
!                    array
!     INI081         Create permanent space in array IBUFFR
!     INIGET         Compute pointer of array in IBUFFR
!     INILEN         Compute length of array in IBUFFR
!     INSTOP         Stop the program
!     PRINLG         Print and checks length of user arrays
!     READFL         Read integer array from a direct access SEPRAN file
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      87   KMESH(2) # 100
!     775   The array KMESH read has been written by an old version of WRITBS
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readb2' )
      length = iwork3 ( 2*numarr )
      index  = iwork3 ( 2*numarr-1 )

!     --- Reserve space in IBUFFR

      call ini069 ( ibuffr, ipwork, length, nbufol )
      if ( ierror/=0 ) go to 1000

!     --- Read all information from file 2

      istart = ipwork
      call readfl ( index, 1, length, 1, 2, ibuffr(ipwork) )
      if ( ierror/=0 ) go to 1000

!     --- Replace all information
!         First KMESH

      if ( ibuffr(ipwork+1)/=100 ) then
         call errint ( ibuffr(ipwork+1), 1 )
         call errsub ( 87, 1, 0, 0)
         go to 1000
      end if
      length = ibuffr ( ipwork+2 )
      call prinlg ( 'kmesh', length, dum, kmesh )
      if ( ierror/=0 ) go to 1000
      call ini000 ( ibuffr(ipwork), kmesh, 1, 1, length, 1 )
      istart = istart + length

!     --- Read kmesh parts c, d and e and put in array IBUFFR

      icheck = ibuffr( istart )
      if ( icheck<-1000003 .or. icheck>-1000000 ) &
         call errwar ( 775, 0, 0, 0 )

      do i = 17, 19

         lenglc = ibuffr( istart+1 )
         istart = istart + 2
         kmesh(i) = 0
         if ( i==17 ) then
            call ini081 ( ibuffr, kmesh(i), lenglc, 'kmeshc' )
         else if ( i==18 ) then
            call ini081 ( ibuffr, kmesh(i), lenglc, 'kmeshd' )
         else
            call ini081 ( ibuffr, kmesh(i), lenglc, 'kmeshe' )
         end if
         ipwork = iniget(kmesh(i))
         if ( ierror/=0 ) go to 1000
         call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, lenglc, 1 )
         istart = istart + lenglc

      end do

      if ( kmesh(22)/=0 ) then

!     --- Read kmesh part h and put in array IBUFFR

         icheck = ibuffr( istart )
         lenglc = ibuffr( istart+1 )
         istart = istart + 2
         kmesh(22) = 0
         if ( icheck>-1000000  .or. icheck<-1000003 ) then

!        --- icheck # -1000000, Old version, i.e. KMESH part H does not contain
!                               useful information

            istart = istart + lenglc

         else if ( icheck== -1000003 ) then

!        --- icheck = -1000003
!            new version of kmeshh

            kmesh(22) = 0
            call ini081 ( ibuffr, kmesh(22), lenglc, 'kmeshh' )
            ipwork = iniget(kmesh(22))
            ipkmsh = ipwork
            if ( ierror/=0 ) go to 1000
            call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, lenglc, &
                          1 )

!           --- First 6 positions

            istart = istart + lenglc
            ncurvs = kmesh(11)
            ibuffr ( ipwork ) = 0
            ibuffr ( ipwork+1 ) = 0
            ibuffr ( ipwork+2 ) = 0
            ibuffr ( ipwork+3 ) = 0
            ibuffr ( ipwork+4 ) = 0
            ibuffr ( ipwork+5 ) = 0
            length = 2*ncurvs
            write(text(1:7),130) 2
130         format('kmesh-',i1)
            call ini081 ( ibuffr, ibuffr(ipwork+2), length, text(1:7) )
            ipwork = iniget(ibuffr(ipwork+2))
            if ( ierror/=0 ) go to 1000
            call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, length, &
                          1 )
            istart = istart + length
            length = ibuffr(ipwork+ncurvs-1)*kmesh(6)*intlen
            ipwork = iniget(kmesh(22))
            write(text(1:7),130) 4
            call ini081 ( ibuffr, ibuffr(ipwork+4), length, text(1:7) )
            ipwork = iniget(ibuffr(ipwork+4))
            if ( ierror/=0 ) go to 1000
            call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, length, &
                          1 )
            istart = istart + length
            length = length/kmesh(6)
            ipwork = iniget(kmesh(22))
            write(text(1:7),130) 5
            call ini081 ( ibuffr, ibuffr(ipwork+5), length, text(1:7) )
            ipwork = iniget(ibuffr(ipwork+5))
            if ( ierror/=0 ) go to 1000
            call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, length, &
                          1 )
            istart = istart + length

!           --- Next nsurfs+nvolms positions

            do j = 1, kmesh(12) + kmesh(13)
               ipkmsh = iniget(kmesh(22))
               if ( ibuffr(ipkmsh+6*j)/=0 ) then

!              --- surface not empty

                  leng1 = ibuffr(istart)
                  leng2 = ibuffr(istart+1)
                  leng3 = ibuffr(istart+2)
                  leng4 = ibuffr(istart+3)
                  leng5 = ibuffr(istart+4)
                  istart = istart + 5
                  index = 0
                  call ini081 ( ibuffr, index, leng1, 'coor-loc' )
                  ipwork = iniget(index)
                  if ( ierror/=0 ) go to 1000
                  call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, &
                                leng1, 1 )
                  istart = istart + leng1
                  ipkmsh = iniget(kmesh(22))
                  ibuffr(ipkmsh+6*j) = index
                  index = 0
                  call ini081 ( ibuffr, index, leng2, 'kmeshc-loc' )
                  ipwork = iniget(index)
                  if ( ierror/=0 ) go to 1000
                  call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, &
                                leng2, 1 )
                  istart = istart + leng2
                  ipkmsh = iniget(kmesh(22))
                  ibuffr(ipkmsh+6*j+1) = index
                  index = 0
                  call ini081 ( ibuffr, index, leng3, 'mapar' )
                  ipwork = iniget(index)
                  if ( ierror/=0 ) go to 1000
                  call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, &
                                leng3, 1 )
                  istart = istart + leng3
                  ipkmsh = iniget(kmesh(22))
                  ibuffr(ipkmsh+6*j+2) = index
                  if ( leng4>0 ) then
                     index = 0
                     call ini081 ( ibuffr, index, leng4, 'kbndpt' )
                     ipwork = iniget(index)
                     if ( ierror/=0 ) go to 1000
                     call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, &
                                   leng4, 1 )
                     istart = istart + leng4
                     ipkmsh = iniget(kmesh(22))
                     ibuffr(ipkmsh+6*j+3) = index
                  end if
                  if ( leng5>0 ) then
                     index = 0
                     call ini081 ( ibuffr, index, leng5, 'extra' )
                     ipwork = iniget(index)
                     if ( ierror/=0 ) go to 1000
                     call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, &
                                   leng5, 1 )
                     istart = istart + leng5
                     ipkmsh = iniget(kmesh(22))
                     ibuffr(ipkmsh+6*j+4) = index
                  end if

               end if

            end do

         else

!        --- -100002<=icheck<=-1000000
!            new version, read old KMESHH
!            Start with various parts of KMESHH

            kmesh(22) = 0
            call ini081 ( ibuffr, kmesh(22), lenglc, 'kmeshh' )
            ipwork = iniget(kmesh(22))
            ipkmsh = ipwork
            if ( ierror/=0 ) go to 1000
            call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, &
                          lenglc, 1 )

!           --- First 6 positions

            istart = istart + lenglc
            ncurvs = kmesh(11)
            ibuffr ( ipwork ) = 0
            ibuffr ( ipwork+1 ) = 0
            ibuffr ( ipwork+2 ) = 0
            ibuffr ( ipwork+3 ) = 0
            ibuffr ( ipwork+4 ) = 0
            length = 2*ncurvs
            write(text(1:7),130) 2
            call ini081 ( ibuffr, ibuffr(ipwork+2), length, text(1:7) )
            ipwork = iniget(ibuffr(ipwork+2))
            if ( ierror/=0 ) go to 1000
            call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, length, &
                          1 )
            istart = istart + length
            ipwork = iniget(kmesh(22))
            length = ibuffr(ipwork+5)
            write(text(1:7),130) 4
            call ini081 ( ibuffr, ibuffr(ipwork+4), length, text(1:7) )
            ipwork = iniget(ibuffr(ipwork+4))
            if ( ierror/=0 ) go to 1000
            call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, length, &
                          1 )
            istart = istart + length

!           --- Next nsurfs+nvolms positions

            do j = 1, kmesh(12) + kmesh(13)
               ipkmsh = iniget(kmesh(22))
               if ( ibuffr(ipkmsh+6*j)/=0 ) then

!              --- surface not empty

                  index = 0
                  length = ibuffr ( ipkmsh+6*j+1 )
                  npointlc = length/( kmesh(6)*intlen )
                  call ini081 ( ibuffr, index, length, 'coor-loc' )
                  ipwork = iniget(index)
                  if ( ierror/=0 ) go to 1000
                  call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, &
                                length, 1 )
                  istart = istart + length
                  ipkmsh = iniget(kmesh(22))
                  ibuffr(ipkmsh+6*j) = index

                  index = 0
                  length = ibuffr ( ipkmsh+6*j+3 )
                  call ini081 ( ibuffr, index, length, 'kmeshc-loc' )
                  ipwork = iniget(index)
                  if ( ierror/=0 ) go to 1000
                  call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, &
                                length, 1 )
                  istart = istart + length
                  ipkmsh = iniget(kmesh(22))
                  ibuffr(ipkmsh+6*j+1) = index

                  index = 0
                  length = ibuffr ( ipwork+6*j+5 )
                  call ini081 ( ibuffr, index, length, 'mapar' )
                  ipwork = iniget(index)
                  if ( ierror/=0 ) go to 1000
                  call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, &
                                length, 1 )
                  istart = istart + length
                  ipkmsh = iniget(kmesh(22))
                  ibuffr(ipkmsh+6*j+2) = index
                  ibuffr(ipkmsh+6*j+4) = 0
                  ibuffr(ipkmsh+6*j+5) = 0
                  ibuffr(ipkmsh+6*j+6) = 0

!                 --- write extra part of kmesh part h corresponding to surface
!                     generator general, see readb2

                  if ( j<=kmesh(12) ) then

!                 --- This is only necessary for surfaces

                     length = npointlc
                     index = 0
                     call ini081 ( ibuffr, index, length, 'extra' )
                     ipwork = iniget(index)
                     if ( ierror/=0 ) go to 1000
                     call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, &
                                   length, 1 )
                     istart = istart + length
                     ipkmsh = iniget(kmesh(22))
                     ibuffr(ipkmsh+6*j+5) = index

                  end if

               end if

            end do

         end if

      end if

!     --- Read kmesh part i - s, iinput and rinput in array IBUFFR

      do i = 23, 35

         if ( i/=30 .and. kmesh(i)/=0 .and. ( i<=33 .or. &
              icheck<=-1000001 ) ) then

            lenglc = ibuffr( istart+1 )
            istart = istart + 2
            kmesh(i) = 0
            if ( i==23 ) then
               call ini081 ( ibuffr, kmesh(i), lenglc, 'coor' )
            else if ( i==24 ) then
               call ini081 ( ibuffr, kmesh(i), lenglc, 'kmeshj' )
            else if ( i==25 ) then
               call ini081 ( ibuffr, kmesh(i), lenglc, 'kmeshk' )
            else if ( i==26 ) then
               call ini081 ( ibuffr, kmesh(i), lenglc, 'kmeshl' )
            else if ( i==27 ) then
               call ini081 ( ibuffr, kmesh(i), lenglc, 'kmeshm' )
            else if ( i==28 ) then
               call ini081 ( ibuffr, kmesh(i), lenglc, 'kmeshn' )
            else if ( i==29 ) then
               call ini081 ( ibuffr, kmesh(i), lenglc, 'kmesho' )
            else if ( i==31 ) then
               call ini081 ( ibuffr, kmesh(i), lenglc, 'kmeshq' )
            else if ( i==32 ) then
               call ini081 ( ibuffr, kmesh(i), lenglc, 'kmeshr' )
            else if ( i==33 ) then
               call ini081 ( ibuffr, kmesh(i), lenglc, 'kmeshs' )
            else if ( i==34 ) then
               call ini081 ( ibuffr, kmesh(i), lenglc, 'iinput' )
            else
               call ini081 ( ibuffr, kmesh(i), lenglc, 'rinput' )
            end if
            ipwork = iniget(kmesh(i))
            if ( ierror/=0 ) go to 1000
            call ini000 ( ibuffr(istart), ibuffr(ipwork), 1, 1, &
                          lenglc, 1 )
            istart = istart + lenglc

         end if

      end do
      if ( ierror/=0 ) go to 1000

!     --- Set KMESH part R3 to -1 and give warning if KMESH part r exists

      if ( kmesh(32)/=0 ) then

!     --- KMESH part r 3

         ipwork = iniget(kmesh(32))
         nsurfs = kmesh(12)
         length = nsurfs+1+ibuffr(ipwork+nsurfs)
         lenglc = inilen(kmesh(32)) - length
         ipwork = ipwork+length-1
         do i = 1, lenglc
            ibuffr(ipwork+i) = -1
         end do
         ipwork = ipwork + lenglc

      end if

1000  if ( ierror/=0 ) call instop
      call erclos ( 'readb2' )
      nbuffr = nbufol
      end
