      subroutine readb4 ( ibuffr, buffer, numarr, isol, kprob )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  6.1  date 15-11-2012 Long integers
!        version  6.0  date 14-04-1999 Extra parameters ibuffr/buffer
!        version  5.4  date 24-07-1998 include commons
!
!   copyright (c) 1982-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   readb4 reads array of isol type from backing storage
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     backing_storage
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cfile2f90'
      include 'SPcommon/cbuffrf90'
      include 'SPcommon/cconstf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numarr, isol(*), kprob(*), ibuffr(*)
      double precision buffer(*)

!     buffer i/o   Real array in which all large data is stored
!     ibuffr i/o   Integer buffer array in which all large data is stored
!     isol    o    Standard SEPRAN array containing information of the solution
!     kprob   i    Standard SEPRAN array, containing information of the problem
!     numarr  i    Array number with respect to the index in common cfile2
!                  for array KMESH
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical compl
      integer (kind=8) nbufol, ipwork, ipusol, iptemp, ipkprh
      integer length, index, nusol, iprob, ipkprb, indprh

!     compl   indication whether the arithmetic is real (false)
!             or complex (true)
!     index   Index array to be used if 1000<=ind(number)<2000 and
!             ichois = 2
!     indprh  indication if kprob part h has been filled (>0) or not (0)
!     ipkprb  Starting address of array KPROB for actual problem number
!     ipkprh  starting address of array kprob part h
!     iprob   Actual problem number
!     iptemp  Starting address of temporary array in IBUFFR
!     ipusol  Starting address of solution array in IBUFFR
!     ipwork  Starting address in array work of work array
!     length  Help parameter to indicate the length of arrays
!     nbufol  help variable to store the old value of nbuffr
!     nusol   Total number of unknowns
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidbl, iniget
      integer inilen, iniprb

!     DCOPY   Copy double precision dx to double precision dy
!     ER0086  Rearrange non-renumbered solution vector into SEPRAN renumbered
!             sequence
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     INI000  Copy integer array to other integer array possibly itself
!     INI060  Reactivate existing array in IBUFFR
!     INI062  Create temporay space in IBUFFR
!     INI069  Reserve space at the end of IBUFFR for a semi-permanent array
!     INI081  Create permanent space in array IBUFFR
!     INIDBL  Transform pointer for IBUFFR to pointer for BUFFER
!     INIGET  Compute pointer of array in IBUFFR
!     INILEN  Compute length of array in IBUFFR
!     INIPRB  Get value of ipkprb as function of iprob
!     INSTOP  Stop the program
!     READFL  Read integer array from a direct access SEPRAN file
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readb4' )
      length = iwork3 ( 2*numarr )
      index  = iwork3 ( 2*numarr-1 )

!     --- Reserve space in IBUFFR

      call ini069 ( ibuffr, ipwork, length, nbufol )
      if ( ierror/=0 ) go to 1000

!     --- Read all information from file 2

      call readfl ( index, 1, length, 1, 2, ibuffr(ipwork) )

      if ( abs(isol(2))/=110 .and. isol(2)/=115 .and. &
           isol(2)/=116 ) isol(1) = 0
      call ini081 ( ibuffr, isol(1), length-7, 'isol' )
      ipusol = iniget(isol(1))
      if ( ierror/=0 ) go to 1000
      call ini000 ( ibuffr(ipwork+7), ibuffr(ipusol), 1, 1, &
                    length-7, 0 )

!     --- Replace all information

      call ini000 ( ibuffr(ipwork+1), isol, 1, 2, 4, 0 )

      if ( isol(2)==-110 ) then

!     --- usol has been renumbered to original nodal point numbering
!         It must be renumbered according KPROB part h

         iprob = isol(4)/1000+1
         ipkprb = iniprb ( iprob, kprob )
         indprh = kprob(ipkprb+21)
         if ( indprh>0 ) then

!        --- Solution should be renumbered

            call ini060 ( ibuffr, -indprh )
            call ini062 ( ibuffr, inilen(isol(1)), iptemp )
            ipusol = inidbl(ipusol)
            iptemp = inidbl(iptemp)
            ipkprh = iniget(indprh)
            compl = mod(isol(3),10)==1
            nusol = isol(5)
            if ( ierror/=0 ) go to 1000
            call er0086 ( buffer(ipusol), ibuffr(ipkprh), nusol, &
                          buffer(iptemp), compl )
            if ( ierror/=0 ) go to 1000

!           --- Copy back

            if ( compl ) nusol = 2*nusol
            call dcopy ( nusol, buffer(iptemp), 1, buffer(ipusol), 1 )

         end if
         isol(2) = abs(isol(2))

      end if

      nbuffr = nbufol

      call erclos ( 'readb4' )
1000  if ( ierror/=0 ) call instop
      end
