      subroutine readbsbf ( ibuffr, buffer, numarr, iarray, ihelp )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.0  date 14-04-1999
!
!   copyright (c) 1999-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    readbsbf reads array from backing storage
!
! **********************************************************************
!
!                       KEYWORDS
!
!     backing_storage
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cconstf90'
      include 'SPcommon/cfile2f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numarr, iarray(*), ihelp(*), ibuffr(*)
      double precision buffer(*)

!    buffer   i/o   Real array in which all large data is stored
!    iarray    o   Integer array containing information of the array that has
!                  been read
!    ibuffr   i/o  Integer buffer array in which all large data is stored
!    ihelp     i   Second integer user array, depending on the first one
!    numarr    i   Array number with respect to the index in common cfile2
!                  for array KMESH
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iwork(2)

!     iwork       Work array that is used to store the first two positions
!                 of the record to be read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     INSTOP  Stop the program
!     READB2  Reads arrays of KMESH type
!     READB3  Reads arrays of KPROB type
!     READB4  Reads arrays of type solution vector
!     READB5  Reads arrays of INTMAT type
!     READB6  Reads arrays of MATR type
!     READB7  Reads arrays of user type
!     READFL  Read integer array from a direct access SEPRAN file
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      82   array is not a sepran array
!     384   numarr has incorrect value
!    1242   Array IHELP is not of the type KPROB
!    1714   Array IHELP is not of the type KMESH
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readbsbf' )
      if ( numarr.le.0 .or. numarr.ge.ifree2 ) then

!     --- Error 384: NUMARR has incorrect value

         call errint ( numarr, 1 )
         call errint ( ifree2-1, 2 )
         call errsub ( 384, 2, 0, 0 )
         go to 1000

      end if

!     --- Read first two positions from record

      call readfl ( iwork3(2*numarr-1), 1, 2, 1, 2, iwork )

!     --- jump to right position

      if ( iwork(2).eq.100 ) then

!     --- iwork(2) = 100 , array is of type kmesh

         call readb2 ( ibuffr, numarr, iarray )

      else if ( iwork(2).eq.101 ) then

!     --- iwork(2) = 101 , array is of type kprob

         if ( ihelp(2).ne.100 ) then

!        --- Array IHELP does not correspond to KMESH

            call errint ( 1, ihelp(2) )
            call errsub ( 1714, 1, 0, 0 )

         else

            call readb3 ( ibuffr, numarr, iarray, ihelp )

         end if

      else if ( abs(iwork(2)).eq.110 .or. iwork(2).eq.115 .or. &
                iwork(2).eq.116 .or. iwork(2).eq.120 ) then

!     --- iwork(2) = 110, 115, 116 or 120   array is of type solution vector

         if ( abs(iwork(2)).eq.110 .and. ihelp(2).ne.101 ) then

!        --- Array IHELP does not correspond to KPROB

            call errint ( 1, ihelp(2) )
            call errsub ( 1242, 1, 0, 0 )

         else

            call readb4 ( ibuffr, buffer, numarr, iarray, ihelp )

         end if

      else if (iwork(2).eq.102 ) then

!     --- iwork(2) = 102 , array is of type intmat

         call readb5 ( ibuffr, numarr, iarray )

      else if ( iwork(2).eq.103 ) then

!     --- iwork(2) = 103 , array is of type matr

         call readb6 ( ibuffr, numarr, iarray )

      else if ( iwork(2).eq.200 .or. iwork(2).eq.300 ) then

!     --- iwork(2) = 200, 300 , array is user-array

         call readb7 ( ibuffr, numarr, iarray )

      else

!     --- Array is of unknown type

         call errsub ( 82, 0, 0, 0 )

      end if

1000  if ( ierror.ne.0 ) call instop
      call erclos ( 'readbsbf' )
      end
