      subroutine readc0 ( inpcre, rincre, numint, nreal, icode, icompl,
     +                    kmesh, codemn, nmain, mncode, kmeshn, nextra,
     +                    inforinpcre, iprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.8  date 13-11-2010 error 1488 --> errkeywexcl
!        version  5.7  date 25-01-2008 Extension with vector
!        version  5.6  date 15-08-2007 Extra debug statements
!
!   copyright (c) 1989-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Reads the part of the standard input file for subroutine CREATE
!    that corresponds to a usual data record
!    and stores the information in the arrays INPCRE and RINCRE
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/celwrk'
      include 'SPcommon/cseprd'
      include 'SPcommon/csepcm'
      include 'SPcommon/csepint'
      include 'SPcommon/celiar'
      include 'SPcommon/creadinfo'
      include 'SPcommon/comcons1'
      include 'SPcommon/cuscons'
      include 'SPcommon/cmcdpi'
      include 'SPcommon/cconst'
      integer iwork(200)
      equivalence (ind(1),iwork(1))

!     iwork is filled as follows
!           1:  Defines how the vector must be filled
!               0 value = v found
!               1<k<1000:  func = k
!               1000<k<2000:  special_function = k-1000
!               2000<k<3000:  old_function = k-2000
!               3000<k<4000:  old_vector = k-3000
!                  Position 111 contains m
!                           111+i i=1...m the vector numbers
!               4000<k<5000:  vector = k-4000
!               9001: quadratic bc
!           2:  Defines the region
!               1: points
!               2: curves
!               3: surfaces
!               4: nodes
!               5: elements
!               6: groups
!               7: volumes
!               8: user_points
!               9: obstacle
!              10: contact
!           3:  Nummber of points etc
!               Sequence numbers stored in work(100..99+m)
!               Or step
!           4:  First point number
!           5:  Last point number
!           100-99+ndegfd:  Degrees of freedom
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numint, nreal, inpcre(*), icode, icompl, kmesh(*),
     +        nmain, mncode(*), kmeshn(*), nextra, inforinpcre(*), iprob
      double precision rincre(*)
      character (len=*) codemn(*)

!     codemn         i    Table with main keywords
!     icode         i/o   Defines type of main keyword found.
!                         Possible values:
!                         1: CREATE
!                         2: ESSENTIAL
!                         3: END
!     icompl         i    Multiplication factor for complex numbers
!                         Real: icompl=1,   Complex: icompl=2
!     inforinpcre    o    Array of length of rincre that contains a 0 if the
!                         number stored in rincre is a real and a reference
!                         to scalars if it is a scalar
!     inpcre         o    The integer input read is stored in array INPCRE from
!                         position 1. Hence in the call INPCRE(ISTART), with
!                         ISTART the starting position must be used.
!                         INPCRE will be filled in correspondence with array
!                         INPCRE for subroutine CREATV
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kmeshn         i    KMESH part n, contains nodal point numbers of the
!                         surfaces
!     mncode         i    Number of significant letters in main keywords
!     nextra         i    Number of extra positions available in codemn to be
!                         used to store subkeywords
!     nmain          i    Number of main keywords that are recognized
!     nreal         i/o   Number of reals stored in RINCRE
!     numint         o    Number of integers that are stored in INPCRE
!     rincre        i/o   The value alpha of the real parameter in VALUE = alpha
!                         is stored in RINCRE
!                         The same remarks as for INPCRE hold
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nncode, nout, nstrng, ncodsb
      parameter ( ncodsb=6, nncode=39, nout=100, nstrng=5 )
      integer nread, ndegfd, i, inum, inpelm, nscods(ncodsb),
     +        ncode(nncode), itype(nncode), ncodmx(nncode), iposut, m,
     +        kmesha, ielgrp
      character (len=20) code(nncode)
      character (len=1) codesb(ncodsb)
      character (len=80) strout(nstrng)
      double precision tabmin(nncode), tabmax(nncode), table(nncode),
     +                 tabout(nout)
      logical debug

!     code           Table with keywords
!     codesb         Table with subkeywords
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     ielgrp         element group number
!     inpelm         number of points per element
!     inum           Number of numbers read in a record
!     iposut         Position in array tabout
!     itype          Table containing information about the numbers behind
!                    the keywords in the table code
!     kmesha         Starting address of kmesh part a in kmesh
!     m              number of integers read
!     ncode          Table containing the number of significant characters in
!                    the keyword table code
!     ncodmx         Table containing the maximal number of  characters in
!                    the keyword table code
!     ncodsb         Number of entries in codesb
!     ndegfd         Number of degrees of freedom per point
!     nncode         Number of entries in code
!     nout           Number of entries in tabout
!     nread          Number of items read
!     nscods         Number of significant letters in subkeywords
!     nstrng         Length of array strout
!     strout         Contains series of strings
!     table          Contains the numbers read for the various keywords
!                    Must be filled with the default values at input
!                    If keyword = text is considered, the number to be put in
!                    table is equal to the relative position in the table
!                    codemn, starting with 1
!                    Hence if the text is equal to codemn(tabmin(icode))
!                    table(icode) gets the value 1 etc.
!     tabmax         Contains maximum values allowed in table
!     tabmin         Contains minimum values allowed in table
!     tabout         Contains series of numbers
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inipelm, iniplm

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIPELM        Compute inpelm for element number
!     INIPLM         Compute inpelm
!     INTX02         Interpret a record containing keywords followed by numbers
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRTS00         Check icurv1, icurv2
!     PRTS01         Check isurf1, isurf2
!     PRTS02         Check ivolm1, ivolm2
!     READC0QUAD     Fill information concerning boundary conditions of the type
!                    (half)-quadratic into itypebc and rincre
!     READC301       Check the numrel numbers stored in array relnods
! **********************************************************************
!
!                       I/O
!
!     READC0 requires input from the standard input file
!     This input must be stored into array CARDS by the calling subroutine
!     The following input is recognized:
!     [functional description] [degrees of freedom] [location part]
!     in arbitrary order
!     functional description may be of the shape:
!     [VALUE = [alpha] [(alpha, beta)] ] [FUNC = k] [SPECIAL_FUNC = l]
!     [OLD_FUNC = k ] [OLD_VECTOR = m] [SEQ_VECTORS = V1, V2, ...]  [VECTOR = j]
!     [QUADRATIC, MAX = a, MEAN_VALUE = v, BOUNDARY_LAYER_WIDTH = b,
!                 DIFFERENCE = delta_T ]
!     [HALF_QUADRATIC, MAX = a]
!     only one of these possibilities is permitted
!     Default: VALUE = 0
!     Degrees of freedom has the following shape
!     DEGFD k [,DEGFD l [, DEGFD m ....] ]
!     If omitted DEGFD 0 is assumed
!     The location part has one of the following structures:
!     POINTS ( Pk, Pl, + . + , Pm)
!     USER POINTS ( Pk to Pl [,step = s] )
!     CURVES [l] (Cj [to Cm] )
!     OUTER_CURVES
!     SURFACES [i1, i2, i3, ... ] (Sj [to Sm] )
!     OUTER_SURFACES
!     VOLUMES [i1, i2, i3, ... ] (Vj [to Vm] )
!     NODES (Nj [to Nm [, step = s ] ] )
!     ELEMENTS i [to j] [(RN1, RN2, RN3, ... )]
!     GROUP ielgrp [(RN1, RN2, RN3, ... )]
!     GLOBAL_GROUP iglgrp
!     CONTACT i
!     NO_CONTACT i
!     OBSTACLE i
!     IN_ALL_OBSTACLE i
!     IN_INNER_OBSTACLE i
!     IN_BOUN_OBSTACLE i
!     ON_BOUN_OBSTACLE i
!     ZERO_LEVELSET i
!     FILE_NODAL_VALUES = 'file_name'
!     FILE_ELEMENT_VALUES = 'file_name'
!     FILE_CAPACITY_VALUES = 'file_name'
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     891   In one record more than one of the combinations:
!           POINTS, CURVES, SURFACES, NODES, ELEMENTS or GROUP is found
!    1043   SEQ_VECTORS may only be used in combination with OLD_VECTOR
!    1044   Not the keyword POINTS found after USER
!    1045   Too much points found after keyword
!    1067   MAX found but not in combination with QUADRATIC
!    1069   A quadratic profile may only be described for one degree of freedom
!    1104   Second nodal point number after NODES = incorrect
!    1106   VALUE = appears more than once in a record
!    1107   Both VALUE = and FUNC = found in record
!    1112   FUNC appears more than once in a record
!    1561   nextra too small
!    1887   Number of texts read too large
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Extend codemn with local subkeywords
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code / 'VALUE', 'FUNCTION', 'SPECIAL_FUNCTION',
     +            'OLD_FUNCTION', 'OLD_VECTOR', 'SEQ_VECTORS', 'DEGFD',
     +            'POINTS', 'USER', 'STEP', 'CURVES', 'TYPE',
     +            'SURFACES', 'VOLUMES', 'NODES', 'ELEMENTS',
     +            'GROUP', 'OBSTACLE', 'CONTACT', 'NO_CONTACT',
     +            'QUADRATIC', 'MAX', 'FILE_NODAL_VALUES',
     +            'FILE_ELEMENT_VALUES', 'HALF_QUADRATIC',
     +            'FILE_CAPACITY_VALUES', 'GLOBAL_GROUP',
     +            'IN_ALL_OBSTACLE', 'IN_INNER_OBSTACLE',
     +            'IN_BOUN_OBSTACLE', 'ON_BOUN_OBSTACLE',
     +            'OUTER_CURVES', 'OUTER_SURFACES', 'MEAN_VALUE',
     +            'BOUNDARY_LAYER_WIDTH', 'DIFFERENCE',
     +            'ZERO_LEVELSET', 'VECTOR', 'PROBLEM' /
      data itype / 19, 0, 0, 0, 0,   40,14,18, 2, 0,
     +             31, 0,32,32,18,   33,33, 0, 0, 0,
     +              2, 1, 6, 6, 2,    6, 0, 0, 0, 0,
     +              0, 2, 2, 1, 1,    1, 0, 0, 0/
      data ncode /  3, 3, 3, 5, 7,    3, 5, 4, 4, 4,
     +              4, 4, 4, 4, 5,    4, 4, 4, 4, 7,
     +              5, 3, 9, 9, 8,    9, 9,10,12,12,
     +             12, 9, 9, 4,12,    4, 8, 3, 4 /
      data ncodmx/  5, 8,16,12,10,   11, 5, 6, 4, 4,
     +              6, 4, 8, 7, 5,    8, 5, 8, 7,10,
     +              9, 3,17,19,14,   20,12,15,17,17,
     +             17,12,14,10,20,   10,13, 6, 6 /
      data tabmin/ 1d0, 1d0, 1d0, 1d0, 1d0,    1d1, 0d0, 0d0, 0d0, 0d0,
     +             0d0, 0d0, 0d0, 0d0, 2d0,    2d0, 0d0, 1d0, 1d0, 1d0,
     +             0d0,-1d9, 0d0, 0d0, 0d0,    0d0, 0d0, 1d0, 1d0, 1d0,
     +             1d0, 0d0, 0d0,-1d20,0d0,  1d-20, 1d0, 1d0, 1d0 /
      data codesb/ 'V', 'P', 'C', 'S', 'N', 'E' /
      data nscods /6*1/

!    code      Contains keywords that are recognized
!              Possible values:
!              1:  VALUE
!              2:  FUNCTION
!              3:  SPECIAL_FUNCTION
!              4:  OLD_FUNCTION
!              5:  OLD_VECTOR
!              6:  SEQ_VECTORS
!              7:  DEGFD
!              8:  POINTS
!              9:  USER
!             10:  STEP
!             11:  CURVES
!             12:  TYPE
!             13:  SURFACES
!             14:  VOLUMES
!             15:  NODES
!             16:  ELEMENT
!             17:  GROUP
!             18:  OBSTACLE
!             19:  CONTACT
!             20:  NO_CONTACT
!             21:  QUADRATIC
!             22:  MAX
!             23:  FILE_NODAL_VALUES
!             24:  FILE_ELEMENT_VALUES
!             25:  HALF_QUADRATIC
!             26:  FILE_CAPACITY_VALUES
!             27:  GLOBAL_GROUP
!             28:  IN_ALL_OBSTACLE
!             29:  IN_INNER_OBSTACLE
!             30:  IN_BOUN_OBSTACLE
!             31:  ON_BOUN_OBSTACLE
!             32:  OUTER_CURVES
!             33:  OUTER_SURFACES
!             34:  MEAN_VALUE
!             35:  BOUNDARY_LAYER_WIDTH
!             36:  DIFFERENCE
!             37:  ZERO_LEVELSET
!             38:  VECTOR
!             39:  PROBLEM
! ======================================================================
!
      call eropen ( 'readc0' )
      debug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readc0'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Extend codemn with local subkeywords

      if ( nextra<ncodsb ) then

!     --- nextra too small

         call errint( ncodsb, 1 )
         call errint( nextra, 2 )
         call errsub ( 1561, 2, 0, 0 )

      end if

!     --- Copy extra information into codemn

      do i = 1, ncodsb
         codemn(nmain+i) = codesb(i)
         mncode(nmain+i) = nscods(i)
      end do

!     --- Initialize some parameters

      numint = 0
      nread  = 0
      ndegfd = 0
      do i = 1, 10
         iwork(i) = 0
      end do

!     --- Set table values

      table = -1d20
      table(6) = -nmain-1
      table(8) = -nmain-2
      table(11) = -nmain-3
      table(13) = -nmain-4
      table(14) = -nmain-1
      table(15) = -nmain-5
      table(22) = -1d21
      table(39) = -1d0
      tabmax(1) = 1d10
      do i = 2, 5
         tabmax(i) = 999d0
      end do
      do i = 9, nncode
         tabmax(i) = 0d0
      end do
      tabmax(6) = 100d0
      tabmax(7) = 11d0
      tabmax(8) = kmesh(10)
      tabmax(10) = kmesh(10)
      tabmax(11) = kmesh(11)
      tabmax(12) = kmesh(11)
      tabmax(13) = kmesh(12)
      tabmax(14) = kmesh(13)
      tabmax(15) = kmesh(8)
      tabmax(16) = kmesh(4)
      tabmax(17) = kmesh(4)
      tabmax(18) = 100d0
      tabmax(19) = 10d0
      tabmax(20) = 10d0
      tabmax(22) = 1d9
      tabmin(1) = icompl+1
      tabmin(6) = 88d0
      tabmin(7) = 100d0
      tabmin(8) = 97d0
      tabmin(11) = 2d0
      tabmin(12) = kmesh(4)
      tabmin(13) = kmesh(4)
      tabmin(14) = kmesh(4)
      tabmin(15) = 2d0
      tabmin(16) = 2d0
      tabmin(17) = kmesh(5)
      tabmax(27) = 1d3
      tabmax(28) = kmesh(42)
      tabmax(29) = kmesh(42)
      tabmax(30) = kmesh(42)
      tabmax(31) = kmesh(42)
      tabmin(34) = -1d10
      tabmax(34) = 1d10
      tabmax(35) = 1d10
      tabmin(36) = -1d10
      tabmax(36) = 1d10
      tabmax(37) = 1d0
      tabmax(38) = 100d0
      tabmax(39) = iprob
      icode = 0

      call intx02 ( code, ncode, nncode, itype, table, codemn, mncode,
     +              nmain, icode, tabmin, tabmax, tabout, nout, strout,
     +              nstrng, ncodmx )
      if ( debug ) then
         call prinrl ( table, nncode, 'table' )
         call prinrl ( tabmax, nncode, 'tabmax' )
      end if  ! ( debug )
      if ( icode>0 ) go to 1000
      if ( table(1)>0d0 ) then

!     --- VALUE found, read number

         if ( iwork(1)/=0 ) then
            call errsub ( 1107, 0, 0, 0 )
            call errcrd
         end if
         if ( nread>=1 ) then
            call errsub ( 1106, 0, 0, 0 )
            call errcrd
         end if
         iposut = nint(table(1))
         if ( infotabout(iposut)==11 ) then
            inforinpcre(nreal+1) = nint(tabout(iposut))-100000000
            rincre(nreal+1) = scalars(inforinpcre(nreal+1))
         else
            inforinpcre(nreal+1) = 0
            rincre(nreal+1) = tabout(iposut)
         end if
         if ( debug ) then
            write(irefwr,*) 'value found'
            write(irefwr,1) 'iposut, infotabout(iposut), nreal',
     +                       iposut, infotabout(iposut), nreal
            write(irefwr,2) 'rincre(nreal+1)', rincre(nreal+1)
         end if  ! ( debug )

         if ( icompl==0 ) then

!        --- icompl = 0, real vector

            nread = 1

         else

!        --- icompl = 1, Complex vector
!                        Find next item

            nread = 2
            if ( tabmax(1)>1d0 ) then
               if ( infotabout(iposut+1)==11 ) then
                  inforinpcre(nreal+2) =
     +               nint(tabout(iposut+1))-100000000
                  rincre(nreal+2) = scalars(inforinpcre(nreal+2))
               else
                  inforinpcre(nreal+2) = 0
                  rincre(nreal+2) = tabout(iposut+1)
               end if
            else
               inforinpcre(nreal+2) = 0
               rincre(nreal+2) = 0d0
            end if

         end if

      end if
      if ( table(2)>0 ) then

!     --- FUNCTION found

         if ( iwork(1)/=0 ) then
            call errsub ( 1112, 0, 0, 0 )
            call errcrd
         end if
         if ( nread/=0 ) then
            call errsub ( 1107, 0, 0, 0 )
            call errcrd
         end if
         iwork(1) = nint(table(2))
         if ( debug ) then
            write(irefwr,*) 'function found'
            write(irefwr,1) 'iwork(1)', iwork(1)
         end if  ! ( debug )

      end if
      if ( table(3)>0 ) then

!     --- SPECIAL_FUNCTION found

         if ( iwork(1)/=0 ) then
            call errsub ( 1112, 0, 0, 0 )
            call errcrd
         end if
         if ( nread/=0 ) then
            call errsub ( 1107, 0, 0, 0 )
            call errcrd
         end if
         iwork(1) = nint(table(3))+1000
         if ( debug ) then
            write(irefwr,*) 'SPECIAL_FUNCTION found'
            write(irefwr,1) 'iwork(1)', iwork(1)
         end if  ! ( debug )

      end if
      if ( table(4)>0 ) then

!     --- OLD_FUNCTION found

         if ( iwork(1)/=0 ) then
            call errsub ( 1112, 0, 0, 0 )
            call errcrd
         end if
         if ( nread/=0 ) then
            call errsub ( 1107, 0, 0, 0 )
            call errcrd
         end if
         iwork(1) = nint(table(4))+2000
         if ( debug ) then
            write(irefwr,*) 'OLD_FUNCTION found'
            write(irefwr,1) 'iwork(1)', iwork(1)
         end if  ! ( debug )

      end if
      if ( table(5)>0 ) then

!     --- OLD_VECTOR found

         if ( iwork(1)/=0 ) then
            call errsub ( 1112, 0, 0, 0 )
            call errcrd
         end if
         if ( nread/=0 ) then
            call errsub ( 1107, 0, 0, 0 )
            call errcrd
         end if
         iwork(1) = nint(table(5))+3000
         if ( table(6)>0d0 ) then
            iposut = nint(table(6))
            m = nint(tabmax(6))
            iwork(111) = m
            do i = 0, m-1
               iwork(112+i) = nint(tabout(iposut+i))
            end do
         else
            iwork(111) = 1
            iwork(112) = 1
         end if
         if ( debug ) then
            write(irefwr,*) 'OLD_VECTOR found'
            write(irefwr,1) 'iwork(1), iwork(111), iwork(112)',
     +                       iwork(1), iwork(111), iwork(112)
         end if  ! ( debug )

      else if ( table(6)>0d0 ) then

!     --- SEQ_VECTORS may only be used in combination with OLD_VECTOR

         call errsub ( 1043, 0, 0, 0 )
         call errcrd

      end if
      if ( table(38)>0 ) then

!     --- VECTOR found

         if ( iwork(1)/=0 ) then
            call errsub ( 1112, 0, 0, 0 )
            call errcrd
         end if
         if ( nread/=0 ) then
            call errsub ( 1107, 0, 0, 0 )
            call errcrd
         end if
         iwork(1) = nint(table(38))+4000
         if ( debug ) then
            write(irefwr,*) 'VECTOR found'
            write(irefwr,1) 'iwork(1)', iwork(1)
         end if  ! ( debug )

      end if
      if ( table(7)>0 ) then

!     --- DEGFD found

         iposut = nint(table(7))
         ndegfd = nint(tabmax(7))
         do i = 1, ndegfd
            iwork(99+i) = nint(tabout(iposut+i-1))
         end do
         if ( debug ) then
            write(irefwr,*) 'DEGFD found'
            write(irefwr,1) 'ndegfd', ndegfd
            call prinin ( iwork(100), ndegfd, 'iwork' )
         end if  ! ( debug )

      end if
      if ( table(8)>0d0 .and. table(9)<0d0 ) then

!     --- POINTS found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 1
         iposut = nint(table(8))
         m = nint(tabmax(8))
         iwork(3) = m
         do i = 1, m
            work(99+i) = tabout(iposut+i-1)
         end do
         if ( debug ) then
            write(irefwr,*) 'POINTS found'
            write(irefwr,1) 'm', m
            call prinin ( iwork(100), m, 'iwork' )
         end if  ! ( debug )

      end if
      if ( table(9)>0d0 ) then

!     --- USER POINTS found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         if ( table(8)<0d0 ) then
            call errsub ( 1044, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 8
         iposut = nint(table(8))
         m = nint(tabmax(8))
         iwork(4) = nint(tabout(iposut))
         if ( m>2 ) then
            call errchr ( code(9), 1 )
            call errint ( m, 1 )
            call errint ( 2, 2 )
            call errsub ( 1045, 2, 0, 1 )
            call errcrd
         end if
         if ( m==2 ) then
            iwork(5) = nint(tabout(iposut+1))
         else
            iwork(5) = iwork(4)
         end if
         if ( table(10)>-1d20 ) iwork(3) = nint(table(10))
         if ( debug ) then
            write(irefwr,*) 'USER POINTS found'
            write(irefwr,1) 'm', m
            call prinin ( iwork, 5, 'iwork' )
         end if  ! ( debug )

      end if
      if ( table(11)>0d0 .or. table(32)>0 ) then

!     --- CURVES found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 2
         if ( table(11)>0d0 ) then

!        --- table(11) >0, standard case

            iwork(3) = nint(tabmin(11))
            if ( table(12)>0 ) iwork(3) = nint(table(12))
            iposut = nint(table(11))
            m = nint(tabmax(11))
            iwork(4) = nint(tabout(iposut))
            if ( m>2 ) then
               call errchr ( code(10), 1 )
               call errint ( m, 1 )
               call errint ( 2, 2 )
               call errsub ( 1045, 2, 0, 1 )
               call errcrd
            end if
            if ( m==2 ) then
               iwork(5) = nint(tabout(iposut+1))
            else
               iwork(5) = iwork(4)
            end if
            call prts00 ( iwork(4), iwork(5), kmesh(11) )

         else  ! ( table(32)>0d0 )

!        --- table(32) >0, special case: outer curves

            iwork(3) = -1
            iwork(4) = -1
            iwork(5) = -1

         end if  ! ( table(11)>0d0 )
         if ( debug ) then
            write(irefwr,*) 'CURVES found'
            write(irefwr,1) 'm', m
            call prinin ( iwork, 5, 'iwork' )
         end if  ! ( debug )

      end if
      if ( table(13)>0d0 .or. table(33)>0 ) then

!     --- SURFACES RN1, RN2, ... (S1 to S2) found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 3
         if ( table(13)>0d0 ) then

!        --- table(13) >0, standard case

            iposut = nint(table(13))
            m = nint(tabmax(13))
            iwork(3) = m
            iwork(4) = nint(tabout(iposut))
            inum = nint(tabout(iposut+1))
            if ( inum==0 ) inum = iwork(4)
            iwork(5) = inum
            call prts01 ( iwork(4), iwork(5), kmesh(12) )
            do i = 1, m
               work(99+i) = tabout(iposut+1+i)
            end do
            inpelm = iniplm ( iwork(4), 0, kmesh, kmeshn )

            if ( m>0 ) then

!           --- numrel>0,  copy contents of work numrel+1 positions forwards
!               clear work from position 101+numrel

               call readc301 ( m, inpelm, work(100), work(101+m) )

            end if

         else  ! ( table(33)>0d0 )

!        --- table(33) >0, special case: outer surfaces

            iwork(3) = 0
            iwork(4) = -1
            iwork(5) = -1

         end if  ! ( table(11)>0d0 )
         if ( debug ) then
            write(irefwr,*) 'SURFACES found'
            write(irefwr,1) 'm', m
            call prinin ( iwork(100), m, 'iwork' )
         end if  ! ( debug )

      end if
      if ( table(14)>0d0 ) then

!     --- VOLUMES RN1, RN1, ... (V1 to V2) found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 7
         iposut = nint(table(14))
         m = nint(tabmax(14))
         iwork(3) = m
         iwork(4) = nint(tabout(iposut))
         inum = nint(tabout(iposut+1))
         if ( inum==0 ) inum = iwork(4)
         iwork(5) = inum
         call prts02 ( iwork(4), iwork(5), kmesh(13) )
         do i = 1, m
            work(99+i) = tabout(iposut+1+i)
         end do
         inpelm = iniplm ( iwork(4), 1, kmesh, kmeshn )

         if ( m>0 ) then

!        --- numrel>0,  copy contents of work numrel+1 positions forwards
!            clear work from position 101+numrel

            call readc301 ( m, inpelm, work(100), work(101+m) )

         end if
         if ( debug ) then
            write(irefwr,*) 'VOLUMES found'
            write(irefwr,1) 'm', m
            call prinin ( iwork(100), m, 'iwork' )
         end if  ! ( debug )

      end if

      if ( table(15)>0d0 ) then

!     --- NODES found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 4
         iwork(3) = 1
         if ( table(10)>-1d20 ) iwork(3) = nint(table(10))
         iposut = nint(table(15))
         m = nint(tabmax(15))
         iwork(4) = nint(tabout(iposut))
         if ( m==2 ) then
            iwork(5) = nint(tabout(iposut+1))
            if ( iwork(5)<iwork(4) ) then
               call errint ( iwork(5), 1 )
               call errint ( iwork(4), 2 )
               call errint ( kmesh(8), 3 )
               call errsub ( 1104, 3, 0, 0 )
            end if
         else
            iwork(5) = iwork(4)
         end if
         if ( debug ) then
            write(irefwr,*) 'nodes found'
            write(irefwr,1) 'm', m
            call prinin ( iwork, 5, 'iwork' )
         end if  ! ( debug )

      end if

      if ( table(16)>0d0 ) then

!     --- ELEMENT i, j (RN1, RN1, ... ) found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 5
         iposut = nint(table(16))
         m = nint(tabmax(16))-2
         iwork(3) = m
         iwork(4) = nint(tabout(iposut))
         inum = nint(tabout(iposut+1))
         if ( inum==0 ) inum = iwork(4)
         iwork(5) = inum
         do i = 1, m
            work(99+i) = tabout(iposut+1+i)
         end do
         inpelm = inipelm ( iwork(4), kmesh )
         if ( m>0 ) then

!        --- numrel>0,  copy contents of work numrel+1 positions forwards
!            clear work from position 101+numrel

            call readc301 ( m, inpelm, work(100), work(101+m) )

         end if
         if ( debug ) then
            write(irefwr,*) 'elements found'
            write(irefwr,1) 'm', m
            call prinin ( iwork, 5, 'iwork' )
            call prinin ( iwork(100), m, 'iwork/1' )
         end if  ! ( debug )

      end if

      if ( table(17)>0d0 ) then

!     --- GROUP ielgrp ( RN1, RN2, RN, ... ) found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 6
         iposut = nint(table(17))
         m = nint(tabmax(17))
         iwork(4) = nint(tabout(iposut))
         m = m - 1
         do i = 1, m
            work(99+i) = tabout(iposut+i)
         end do

!        --- Compute inpelm

         kmesha = kmesh(15)
         if ( kmesha==0 ) then

!        --- all elements have the same number of nodes

            inpelm = kmesh(4)

         else

!        --- More element groups

            ielgrp = iwork(4)
            inpelm = kmesh(kmesha+ielgrp) - kmesh(kmesha+ielgrp-1)

         end if
         if ( m>0 ) then

!        --- numrel>0,  copy contents of work numrel+1 positions forwards
!            clear work from position 101+numrel

            call readc301 ( m, inpelm, work(100), work(101+m) )

         end if
         if ( debug ) then
            write(irefwr,*) 'group found'
            write(irefwr,1) 'm', m
            call prinin ( iwork, 5, 'iwork' )
            call prinin ( iwork(100), m, 'iwork/1' )
         end if  ! ( debug )

      end if

      if ( table(18)>0d0 ) then

!     --- OBSTACLE i found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 9
         iwork(3) = nint(table(18))
         if ( debug ) then
            write(irefwr,*) 'obstacle found'
            call prinin ( iwork, 3, 'iwork' )
         end if  ! ( debug )

      end if

      if ( table(19)>0d0 ) then

!     --- CONTACT i found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 10
         iwork(3) = nint(table(19))
         if ( debug ) then
            write(irefwr,*) 'CONTACT found'
            call prinin ( iwork, 3, 'iwork' )
         end if  ! ( debug )

      end if
      if ( table(20)>0d0 ) then

!     --- NO_CONTACT i found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 11
         iwork(3) = nint(table(20))
         if ( debug ) then
            write(irefwr,*) 'NO_CONTACT found'
            call prinin ( iwork, 3, 'iwork' )
         end if  ! ( debug )

      end if

      if ( table(23)>0d0 .or. table(24)>0d0 .or.
     +     table(26)>0d0 ) then

!     --- FILE_NODAL_VALUE or FILE_ELEMENT_VALUE or FILE_CAPACITY_VALUE found
!         Fill iwork as follows:
!         iwork(2) = 12/13/14
!         iwork(3) = nextxt (refers to texts)

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         if ( table(23)>0d0 ) then
            iwork(2) = 12
            iposut = nint(table(23))
            nchar = nint(tabmax(23))
         else if ( table(24)>0d0 ) then
            iwork(2) = 13
            iposut = nint(table(24))
            nchar = nint(tabmax(24))
         else
            iwork(2) = 14
            iposut = nint(table(26))
            nchar = nint(tabmax(26))
         end if
         if ( nextxt>ntexts ) then

!        --- Too many texts stored in texts

            call errint ( ntexts, 1 )
            call errsub ( 1887, 1, 0, 0 )
            call errcrd
            go to 1000

         end if
         iwork(3) = nextxt
         texts(nextxt) = strout(iposut) (1:nchar)
         nextxt = nextxt+1
         if ( debug ) then
            write(irefwr,*) 'FILE_NODAL_VALUE found'
            call prinin ( iwork, 3, 'iwork' )
         end if  ! ( debug )

      end if

      if ( table(27)>0d0 ) then

!     --- GLOBAL_GROUP iglgrp found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 15
         iwork(3) = nint(table(27))
         if ( debug ) then
            write(irefwr,*) 'GLOBAL_GROUP found'
            call prinin ( iwork, 3, 'iwork' )
         end if  ! ( debug )

      end if

      if ( table(28)>0d0 ) then

!     --- in_all_obstacle i found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 16
         iwork(3) = nint(table(28))
         if ( debug ) then
            write(irefwr,*) 'in_all_obstacle found'
            call prinin ( iwork, 3, 'iwork' )
         end if  ! ( debug )

      end if

      if ( table(29)>0d0 ) then

!     --- in_inn_obstacle i found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 17
         iwork(3) = nint(table(29))
         if ( debug ) then
            write(irefwr,*) 'in_inn_obstacle found'
            call prinin ( iwork, 3, 'iwork' )
         end if  ! ( debug )

      end if

      if ( table(30)>0d0 ) then

!     --- in_boun_obstacle i found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 18
         iwork(3) = nint(table(30))
         if ( debug ) then
            write(irefwr,*) 'in_boun_obstacle found'
            call prinin ( iwork, 3, 'iwork' )
         end if  ! ( debug )

      end if

      if ( table(31)>0d0 ) then

!     --- on_boun_obstacle i found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 19
         iwork(3) = nint(table(31))
         if ( debug ) then
            write(irefwr,*) 'on_boun_obstacle found'
            call prinin ( iwork, 3, 'iwork' )
         end if  ! ( debug )

      end if

      if ( table(37)>0d0 ) then

!     --- zero_levelset i found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 20
         iwork(3) = nint(table(37))
         if ( debug ) then
            write(irefwr,*) 'zero_levelset found'
            call prinin ( iwork, 3, 'iwork' )
         end if  ! ( debug )

      end if

      if ( table(21)>0d0 .or. table(25)>0d0 ) then

!     --- (HALF_)QUADRATIC found

         if ( iwork(1)/=0 ) then
            call errsub ( 1112, 0, 0, 0 )
            call errcrd
         end if
         if ( nread/=0 ) then
            call errsub ( 1107, 0, 0, 0 )
            call errcrd
         end if
         call readc0quad ( iwork, table, rincre, nreal, inforinpcre,
     +                     code, ncodmx, 21, 22, 25, 34, 35, 36 )
         if ( debug ) then
            write(irefwr,*) 'QUADRATIC found'
            call prinrl ( rincre, nreal+3, 'rincre' )
         end if  ! ( debug )

      else if ( table(22)>-1d20 ) then

!     --- MAX found, but not quadratic

         call errsub ( 1067, 0, 0, 0 )
         call errcrd

      end if

!     --- End of record, fill information in INPCRE
!         First per point

      if ( debug ) then
         write(irefwr,*) 'ndegfd, nread ', ndegfd, nread
         call prinin ( iwork, 2, 'iwork' )
      end if  ! ( debug )
      numint = 1
      if ( ndegfd==0 ) then

!     --- All degrees of freedom

         inpcre(1) = 0
         if ( iwork(1)>=9001 .and. iwork(1)<=9004 ) inpcre(1) = 1

      else if ( ndegfd==1 ) then

!     --- One degree of freedom

         inpcre(1) = iwork(100)

      else if ( iwork(1)>=9001 .and. iwork(1)<=9004 ) then

!     --- Quadratic profile may only be given for one degree of freedom

         call errsub ( 1069, 0, 0, 0 )

      else

!     --- Specific degrees of freedom

         inpcre(1) = -ndegfd
         do i = 1, ndegfd
            inpcre(i+1) = iwork(99+i)
         end do
         numint = 1 + ndegfd

      end if

!     --- Indication of how to be filled

      numint = numint + 1
      if ( nread>0 ) then

!     --- nread>0, value read

         inpcre(numint) = -(nreal+1)
         nreal = nreal + nread

      else if ( iwork(1)==0 ) then

!     --- VALUE nor FUNC found, Set VALUE equal to zero

         inpcre(numint) = -(nreal+1)
         inforinpcre(nreal+1) = 0
         rincre(nreal+1) = 0d0
         if ( icompl==1 ) then
            inforinpcre(nreal+2) = 0
            rincre(nreal+2) = 0d0
         end if
         nreal = nreal + 1 + icompl

      else

!     --- FUNC found

         inpcre(numint) = iwork(1)
         if ( iwork(1)>=9001 .and. iwork(1)<=9004 ) then

!        --- QUADRATIC found, store MAX

            inpcre(numint+1) = nreal+1
            nreal = nreal+1
            numint = numint+1
            if ( iwork(1)==9003 ) then

!           --- quadratic with mean value and band width

               nreal = nreal+1

            else if ( iwork(1)==9004 ) then

!           --- quadratic with mean value, difference and band width

               nreal = nreal+2

            end if  ! ( iwork(1)==9003 )

         else if ( iwork(1)>3000 .and. iwork(1)<4000 ) then

!        --- old vectors found, fill the vector sequence numbers

            m = iwork(111)
            numint = numint+1
            inpcre(numint) = m
            do i = 1, m
               inpcre(numint+i) = iwork(111+i)
            end do
            numint = numint+m

         end if

      end if

!     --- Fill indication in which nodal points the vector must be filled

      numint = numint + 1
      inpcre(numint) = iwork(2)
      if ( debug ) call prinin ( inpcre, numint, 'inpcre/1' )
      if ( iwork(2)==1 ) then

!     --- User POINTS

         numint = numint + 1
         inpcre(numint) = iwork(3)
         do i = 1, iwork(3)
            inpcre(numint+i) = nint(work(99+i))
         end do
         numint = numint + iwork(3)

      else if ( iwork(2)==2 ) then

!     --- curves

         inpcre(numint+1) = iwork(4)
         inpcre(numint+2) = iwork(5)
         inpcre(numint+3) = iwork(3)
         numint = numint + 3

      else if ( iwork(2)==3 ) then

!     --- surfaces

         inpcre(numint+1) = iwork(4)
         inpcre(numint+2) = iwork(5)
         inpcre(numint+3) = iwork(3)
         numint = numint + 3
         do i = 1, iwork(3)
            inpcre(numint+i) = nint(work(99+i))
         end do
         numint = numint + iwork(3)

      else if ( iwork(2)==4 ) then

!     --- nodes

         inpcre(numint+1) = iwork(4)
         inpcre(numint+2) = iwork(5)
         inpcre(numint+3) = max(1,iwork(3))
         numint = numint + 3

      else if ( iwork(2)==5 ) then

!     --- elements

         inpcre(numint+1) = iwork(4)
         inpcre(numint+2) = iwork(5)
         inpcre(numint+3) = iwork(3)
         numint = numint + 3
         do i = 1, iwork(3)
            inpcre(numint+i) = nint(work(99+i))
         end do
         numint = numint + iwork(3)

      else if ( iwork(2)==6 ) then

!     --- element groups

         inpcre(numint+1) = iwork(4)
         inpcre(numint+2) = iwork(3)
         numint = numint + 2
         do i = 1, iwork(3)
            inpcre(numint+i) = nint(work(99+i))
         end do
         numint = numint + iwork(3)

      else if ( iwork(2)==7 ) then

!     --- volumes

         inpcre(numint+1) = iwork(4)
         inpcre(numint+2) = iwork(5)
         inpcre(numint+3) = iwork(3)
         numint = numint + 3
         do i = 1, iwork(3)
            inpcre(numint+i) = nint(work(99+i))
         end do
         numint = numint + iwork(3)

      else if ( iwork(2)==8 ) then

!     --- USER POINTS with step

         inpcre(numint+1) = iwork(4)
         inpcre(numint+2) = iwork(5)
         inpcre(numint+3) = iwork(3)
         numint = numint + 3

      else if ( iwork(2)>=9 .and. iwork(2)<=20 ) then

!     --- obstacle i, contact i or no_contact i or file_nodal_value or
!         file_element_value or global group or in_xx_obstacles found or
!         zero_levelset

         inpcre(numint+1) = iwork(3)
         numint = numint + 1

      end if

      iprob = nint(table(39))

1000  call erclos ( 'readc0' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'numint, nreal ', numint, nreal
         call prinin ( inpcre, numint, 'inpcre' )
         write(irefwr,*) 'End readc0'

      end if

      ispecl = 0

      end
