      subroutine readc0expr ( iwork, work )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 12-02-2015 New call to readexpress
!        version  1.1  date 26-06-2014 Allow scalar
!        version  1.0  date 06-03-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if the next information found corresponds to an expression
!     If so store information into iwork/work
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: iwork(*)
      double precision, intent(inout) :: work(*)

!     iwork         i/o   integer work array to store information of the items
!                         read. Here only iwork(1) is filled
!                         See readc0
!     work          i/o   Work array to store information of the items
!                         read. Here only work(1) is filled
!                         See readc0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lastitemsav
      logical :: debug, openfound, closefound, vecfound

!     closefound     If true a closing bracket has been found, which
!                    corresponds to a closing bracket read earlier
!     debug          If true debug statements are carried out otherwise
!                    they are not
!     lastitemsav    Help parameter to save the value of lastitem
!     openfound      Indicates if an openings bracket is found before
!                    reading the number or expression
!     vecfound       If true the expression contains vectors
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     READEXPRESS    Interprete part of input as number or expression
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readc0expr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readc0expr'
         write(irefwr,1) 'lastitem', lastitem
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      lastitemsav = lastitem

      closefound = .false.
      openfound = .false.
      lastitem = lastitem+1

      call readexpress ( openfound, closefound, .true., .false., &
                         .true., vecfound )

      if ( debug ) then
         write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd
         write(irefwr,2) 'realrd', realrd
      end if  ! ( debug )

      if ( ityprd==3 .or. ityprd==11 ) then

!     --- Real found, store in iwork, work

         iwork(1) = 1
         work(1) = realrd

      else if ( ityprd==19 ) then

!     --- Expression found, store in iwork

         iwork(1) = nint(realrd)

      else

!     --- None of these found, reset

         lastitem = lastitemsav

      end if  ! ( ityprd==3 )

1000  call erclos ( 'readc0expr' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readc0expr'

      end if  ! ( debug )

      end
