      subroutine readcoint ( iseq, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 06-12-2013 Replace readsrin by readsrinnotext
!        version  1.2  date 20-04-2012 Indexed constants
!        version  1.1  date 06-02-2012 Allow arrays
!
!   copyright (c) 2011-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read part integers of input block CONSTANTS
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     constant
!     integer
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpif90'
      include 'SPcommon/cconstf90'
      include 'SPcommon/cinoutf90'
      include 'SPcommon/comcons1f90'
      include 'SPcommon/comcons2f90'
      include 'SPcommon/cusconsf90'
      include 'SPcommon/cusnamef90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cpost5f90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseq, iwork(*)

!     iseq           i    Sequence number of vector in VECOLD/ISLOLD
!     iwork         i/o   integer work array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inum, i, numbint
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     inum           Number of numbers read in a record
!     numbint        Number of entries in integer array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer readarrindex

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     READARRINDEX   Check if the name of a constant vector or varibale is
!                    followed by an index
!     READSRINNOTEXT Read a series of integer numbers from the input
!                    No text between numbers allowed
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1610   Too much integers read
!    2143   Sequence number already in use
!    3100   Name of constant, variable or vector too long
!    3101   Index too large
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readcoint' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readcoint'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iseq==0 ) iseq = numints+1  ! sequence number of integer

      if ( iwork(iseq)/=0 ) then

!     --- error, sequence number already in use

         call errint ( iseq, 1 )
         call errchr ( 'integers' , 1 )
         call errsub ( 2143, 1, 0, 1 )

      end if
      iwork(iseq) = 1

      numints = max(numints,iseq)

      if ( debug ) write(irefwr,1) 'iseq', iseq

      if ( numints>maxints ) then

!     --- Too many integers

         call errint ( numints, 1 )
         call errint ( maxints, 2 )
         call errsub ( 1610, 2, 0, 0 )
         call errcrd
         go to 1000

      end if  ! ( numints>maxints )

!     --- store name into chincons

      chincons(iseq) = string(1:nchar)

!     --- Check if the constant is indexed

      numbint = readarrindex()

      dimcons(iseq) = numbint   ! dimension of constant
      ncharcons(iseq) = nchar   ! number of characters of constant

      if ( numbint>1 ) then

!     --- numbint>1, hence indexed real

         if ( nchar>29 ) then

!        --- Too many characters

            call errchr ( string, 1 )
            call errsub ( 3100, 0, 0, 1 )
            go to 1000

         end if  ! ( nchar>29 )
         do i = 1, min(numbint,9)
            write ( chincons(iseq+i-1), 100 ) &
                string(1:nchar), i
100         format ( a,'(',i1,')' )
         end do  ! i = 1, min(numbint,9)

         if ( numbint>9 ) then

            if ( nchar>28 ) then

!           --- Too many characters

               call errchr ( string, 1 )
               call errsub ( 3100, 0, 0, 1 )
               go to 1000

            end if  ! ( nchar>29 )
            do i = 10, min(numbint,99)
               write ( chincons(iseq+i-1), 110 ) &
                string(1:nchar), i
110            format ( a, '(',i2,')' )
            end do  ! i = 10, min(numbint,99)

         end if  ! ( numbint>9 )

         if ( numbint>=100 ) then

!        --- The upper bound for index is 99 at this moment

            call errint ( numbint, 1 )
            call errint ( 99, 2 )
            call errsub ( 3101, 2, 0, 0 )
            go to 1000

         end if  ! ( numbint>100 )

      end if  ! ( numbint>1 )

!     --- read numbers

      call readsrinnotext ( incons(iseq), maxints-iseq+1, inum, &
                            .false., 2000000000, string(1:32) )
      if ( ierror/=0 ) go to 1000

      if ( debug ) &
         call prinin ( incons(iseq), inum, 'integers' )

      numbint = max(numbint, inum)
      numints = max(numints,iseq+numbint-1)

      if ( numints>maxints ) then

!     --- Too many integers

         call errint ( numints, 1 )
         call errint ( maxints, 2 )
         call errsub ( 1610, 2, 0, 0 )
         call errcrd
         go to 1000

      end if  ! ( numints>maxints )

      do i = 1, numbint-1

         if ( iwork(iseq+i)/=0 ) then

!        --- error, sequence number already in use

            call errint ( iseq+i, 1 )
            call errchr ( 'integers' , 1 )
            call errsub ( 2143, 1, 0, 1 )

         end if
         iwork(iseq+i) = 1

      end do

1000  call erclos ( 'readcoint' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readcoint'

      end if  ! ( debug )

      end
