      subroutine readm3 ( iref )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 02-04-2008 Fortran 90
!        version  2.0  date 02-01-2003 Complete revision
!        version  1.5  date 28-12-2000 Length of records extended to 240
!        version  1.4  date 10-02-1994 New comments
!        version  1.3  date 06-11-1992 Remove text in case of incorrect file
!        version  1.2  date 27-12-1991 New type of reading, extra message
!        version  1.1  date 25-03-1991 Adaption of string
!        version  1.0  date 03-11-1990
!
!   copyright (c) 1990-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       With the help of READM3 it is read from the next line of
!       the environment file if a file is formatted or unformatted
!       If the file is unformatted the reference number as given at the input
!       is made negative
!       subroutine READM3 is a help subroutine of READMD
! **********************************************************************
!
!                       KEYWORDS
!
!     environment_file
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/csepchnewf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iref

!     iref   i/o  Reference number
!                 If the file is unformatted, the reference number is made
!                 negative
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical found

!     found          If true already a possibility has been found
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT  Read next item from file
!     READCD  Read a record from the standard input file
! **********************************************************************
!
!                       I/O
!
!    One record from the environment file is read
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     No colon found
!     No text found after colon
!     The text found after colon is not (un) formatted
! **********************************************************************
!
!                       PSEUDO CODE
!
!      Read next record (comments excluded):  READCD
!      Find colon
!      Read text after colon
!      If text=unformatted then
!         iref := -iref
!      else if text#formatted then
!         error: wrong text found
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Read record

      call readcd

!     --- Find colon

      found = .false.
      do while ( .not. found .and. lastitem<nitems-1 )
         lastitem = lastitem+1
         if ( typeitems(lastitem)==16 ) found =.true.
      end do
      if ( .not. found ) then

!     --- No colon found:

         write(*,100) cardimage(1)(1:lastcolumn(1))
100      format ( ' Line in the environment file does not contain a', &
                  ' colon(:). '//' Execution is terminated'// &
                  ' The line read is:'/1x,a )
         stop

      end if

      call asknxt
      if ( typeitems(lastitem)==3 ) then

!     --- Text found after colon

         if ( string(1:4)=='UNFO' ) then

!        --- UNFORMATTED found, make iref negative

            iref = -iref

         else if ( string(1:4)/='FORM' ) then

!        --- Neither the text formatted nor the text unformatted found

            write(*,110) cardimage(1)(firstcol:lastcolumn(1))
            stop

         end if

      else

!     --- No text found after colon whereas number is expected

         write(*,110) cardimage(1)(1:lastcolumn(1))
110      format ( ' Line in the environment file does not contain the', &
                  ' text (un)formatted after the colon', &
                  ' (:). '//' Execution is terminated'// &
                  ' The line read is:'/1x,a )
         stop

      end if

      end
