      subroutine readnumb ( dnumbr, maxnum, inum )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-06-1996
!
!   copyright (c) 1996-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read a series of numbers until a text is found
!   The reading of numbers is stopped if
!    a ) sign is found
!    an end of record is found
!    maxnum numbers have been read
!    A text is found
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cseprdf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer maxnum, inum
      double precision dnumbr(maxnum)

!     dnumbr  o  Array to be filled with the numbers
!     inum    o  Number of numbers that have been actually read
!     maxnum  i  Maximum number of numbers to be read
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical ready

!    ready    Indicates if a while loop must be terminated (true) or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT  Read next item from the input file
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1983   more numbers read than allowed
! **********************************************************************
!
!                       PSEUDO CODE
!
!   ready := false
!   inum  := 0
!   istop := 2
!   while ( not ready ) do
!      Read next item (ASKNXT)
!      if ( ityprd = 2 or 3 ) then
!         inum := inum+1
!      else if ( ityprd = 0 or 4 ) then
!         ready := true
!      if ( inum = maxnum ) then
!         ready := true
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Make some initializations

      call eropen ( 'readnumb' )

      ready = .false.
      inum  = 0
      istop = 2

200   if ( .not. ready ) then

!     --- While loop
!         Read next item

         call asknxt
         if ( ierror.ne.0 ) go to 1000

         if ( ityprd.eq.2 .or. ityprd.eq.3 ) then

!        --- ityprd = 2 or 3, number found

            inum = inum + 1
            dnumbr(inum) = realrd

         else if ( ityprd.eq.0 .or. ityprd.eq.4 ) then

!        --- ityprd = 0 or 4, string found

            ready = .true.

         end if

         if ( inum.ge.maxnum ) then

!        --- inum = maxnum, stop reading

            ready = .true.
            call errint ( maxnum, 1 )
            call errsub ( 1983, 1, 0, 0 )

         end if

         go to 200

      end if
      istop = 0

1000  call erclos ( 'readnumb' )

      end
