      subroutine readsepar2file ( ibuffr, buffer, mmarray, namefile )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 20-06-2013 New definition file ref numbers
!        version  1.1  date 15-11-2012 Long integers
!        version  1.0  date 27-08-2004
!
!   copyright (c) 2004-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read sepran real array stored in buffer (unformatted) from file named
!     namefile
!     First record is length (integer)
!     Second record array
! **********************************************************************
!
!                       KEYWORDS
!
!     write
!     read
!     file
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mmarray, ibuffr(*)
      double precision buffer(*)
      character *(*) namefile

!     buffer        i/o   Buffer array
!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     mmarray        i    Memory management sequence number of array
!     namefile       i    String variable containing name of the file
!                         from which the array is read
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) iparray
      integer length, i, iref
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              Counting variable
!     iparray        Starting address of array
!     iref           File reference number
!     length         length of array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inidgttext
      integer inisetnewref

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI080         Create permanent space in array BUFFER
!     INIDGTTEXT     Compute pointer of array in BUFFER
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     INISETNEWREF   Create new reference number
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readsepar2file' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readsepar2file'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      iref = inisetnewref()
      open ( iref, file = namefile, form = 'unformatted' )
      read(iref) length
      call ini080 ( ibuffr, mmarray, length, namefile )
      iparray = inidgttext ( mmarray, namefile )
      read(iref) (buffer(iparray+i),i=0,length-1)
      close(iref)
      call inirmref ( iref )

1000  call erclos ( 'readsepar2file' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readsepar2file'

      end if

      end
