      subroutine readsrexpr ( openfound, closefound, notext, finished )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 12-02-2015 New call to readexpress
!        version  1.0  date 11-12-2013
!
!   copyright (c) 2013-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret part of input as number or expression
!     Translate to number
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cseprdf90'
      include 'SPcommon/csepchf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/cexpressionf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical openfound, closefound, notext, finished

!     closefound    i/o   If true a closing bracket has been found, which
!                         corresponds to a closing bracket read earlier
!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
!     notext         i    If true a text is considered as a separator
!     openfound     i/o   Indicates if an openings bracket is found before
!                         reading the number or expression
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lastrealexprsv, numexpresssv, kmesh(9)
      logical debug, vecfound

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     kmesh          Standard SEPRAN array containing information of the
!                    mesh
!     lastrealexprsv Help parameter to save the value of lastrealexpr
!     numexpresssv   Help parameter to save the value of numexpress
!     vecfound       If true the expression contains vectors
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRSUB         Error messages
!     PREVALEXPR     Evaluate expression and fill actual values of scalars
!     PRININ         print 1d integer array
!     READEXPRESS    Interprete part of input as number or expression
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3151   No string expected
!    3156   Text not recognized
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readsrexpr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readsrexpr'
         call prinin ( typeitems, nitems, 'typeitems' )
         write(irefwr,1) 'lastitem', lastitem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kmesh = 0

      lastrealexprsv = lastrealexpr
      numexpresssv = numexpress
      call readexpress ( openfound, closefound, .true., .false., &
                         .false., vecfound )
      if ( closefound .or. lastitem>=nitems ) finished = .true.

      if ( debug ) &
         write(irefwr,1) 'lastitem, nitems', lastitem, nitems
      if ( ityprd==20 ) then

!     --- Nothing read

         ityprd = 0
         finished = .true.
         if ( notext .and. lastitem<nitems ) then

            if ( typeitems(lastitem)==3 ) then

!           --- Error: string found

               call asknxt
               call errchr ( string(1:nchar), 1 )
               call errsub ( 3151, 0, 0, 1 )
               call errcrd
               go to 1000

            else

!           --- Error text not recognized

               call errsub ( 3156, 0, 0, 0 )
               call errcrd
               go to 1000

            end if  ! ( typeitems(lastitem)==3 )

         end if  ! ( lastitem<nitems )

      else if ( ityprd==19 ) then

!     --- Transform to number

         call prevalexpr ( numexpress, realrd, kmesh, &
                           cardim(1:lastp) )
         ityprd = 3
         numexpress = numexpresssv
         lastrealexpr = lastrealexprsv

      end if  ! ( ityprd==20 )

1000  call erclos ( 'readsrexpr' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readsrexpr'

      end if  ! ( debug )

      end
