      subroutine readstrscals
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.7  date 09-05-2014 Correction integer scalars
!        version  1.6  date 08-12-2013 Extension with integer scalars
!        version  1.5  date 29-04-2013 Set index_rdarr
!
!   copyright (c) 2008-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if name read corresponds to one the scalars
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     keyword
!     scalar
!     variable
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons1f90'
      include 'SPcommon/cusconsf90'
      include 'SPcommon/cusnamef90'
      include 'SPcommon/cseprdf90'
      include 'SPcommon/cseprdnewf90'
      include 'SPcommon/csepchf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nscals
      parameter ( nscals = 4 )
      integer i, lenscals(nscals)
      character (len=8) specialscals(nscals)
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     i              General loop variable
!     lenscals       Contains number of characters in names of scalars
!     nscals         Number of special scalars
!     specialscals   Contains names of special scalars
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     READINTSCALS   Check if name read corresponds to one the integer scalars
!     READREALSCALS  Check if name read corresponds to one the real scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data specialscals / 'NELEM', 'NPOINT', 'NPELM', 'SHAPE' /
      data lenscals / 5, 6, 5, 5 /
! ======================================================================
!
      call eropen ( 'readstrscals' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readstrscals'
         write(irefwr,1) 'lastitem', lastitem
  1      format ( a, 1x, (10i7) )
  2      format ( a, 1x, (5d18.10) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call readrealscals
      if ( ityprd==11 .or. ityprd==14 ) go to 1000  ! scalar found

!     --- No scalar found, check if it is an integer scalar

      call readintscals
      if ( ityprd==11 .or. ityprd==14 ) go to 1000  ! scalar found

!     --- No scalar found, check if it is one of the special scalars

      if ( string(1:nchar)=='TIME' ) then

!     --- string TIME found, special scalar (0)

         if ( ispecl==10 .or. ispecl==15 .or. ispecl==19 ) then
            index_rdarr(1) = 0
            ityprd = 14
         else
            realrd = 100000000d0
            ityprd = 11
         end if
         go to 1000

      end if  ! ( string(1:nchar)=='TIME' )

      do i = 1, nscals

        if ( string(1:nchar)==specialscals(i)(1:lenscals(i)) ) then

!       --- special string found, special scalar (maxcls+i)

           if ( ispecl==10 .or. ispecl==15 .or. ispecl==19 ) then
              index_rdarr(1) = i+maxcls+maxintscals
              ityprd = 14
           else
              realrd = 100000000d0+i+maxcls+maxintscals
              ityprd = 11
           end if
           go to 1000

        end if  ! ( string(1:nchar)==specialscals(i)(1:lenscals(i)) )

      end do  ! i = 1, nscals

1000  call erclos ( 'readstrscals' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'realrd', realrd
         write(irefwr,1) 'lastitem, index_rdarr', &
                          lastitem, index_rdarr(1:5)
         write(irefwr,*) 'End readstrscals'

      end if  ! ( debug )

      end
