      subroutine msh616 ( coor  , i1   , i2   , i3   , kdrie, &
                          kstapl, kstap, xn   , yn   , zn   , &
                          jpn   )
! ======================================================================
!
!        programmer    niek praagman
!        version  4.0  date 03-09-2010 New release
!        version  3.1  date 27-06-2008 Textual adjustments
!        version  3.0  date 06-02-2006 Add node number jpn (new==old)
!        version  2.0  date 13-04-2005 Update code
!
!   copyright (c) 1988-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check whether lines that form edges of new triangle have no points
!     in common with triangles of present boundary
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     mesh_generation
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
!    none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer i1, i2, i3, kdrie, kstap, jpn, kstapl(3,*)
      double precision coor(3,*), xn, yn, zn

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points
!     i1             i    First point of basis triangle
!     i2             i    Second point of basis triangle
!     i3             i    Third point of basis triangle
!     jpn            i    node number (0 eventually)
!     kdrie          o    number of triangle that has point(s) common with
!                         lines to new point
!     kstap          i    Number of pairs in kstapl
!     kstapl         i    Array containing the actual boundary build by
!                         pairs of nodes
!     xn             i    next x coordinate
!     yn             i    next y coordinate
!     zn             i    next z coordinate
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ii1, ii2, ii3, ji
      double precision x1, x2, x3, xi1, xi2, xi3, &
                       xm, xmi, xma, xmin, xmax, &
                       y1, y2, y3, yi1, yi2, yi3, &
                       ym, ymi, yma, ymin, ymax, &
                       z1, z2, z3, zi1, zi2, zi3, &
                       zm, zmi, zma, zmin, zmax, &
                       t, ttest

!     i            loop variable
!     ii1          first node boundary triangle
!     ii2          second node boundary triangle
!     ii3          third node boundary triangle
!     ji           helpvariable to indicate nodes of triangle
!     t            intersection parameter ( 0<t<1 yes intersect )
!     ttest        intersection indicator
!     x1           x-coordinate first node triangle
!     x2           x-coordinate second node triangle
!     x3           x-coordinate third node triangle
!     xi1          x-coordinate node i1
!     xi2          x-coordinate node i2
!     xi3          x-coordinate node i3
!     xm           x-coordinate barycentre
!     xma          maximal x-coordinate triangle
!     xmax         maximal x-coordinate tetrahedron
!     xmi          minimal x-coordinate triangle
!     xmin         minimal x-coordinate tetrahedron
!     y1           y-coordinate first node triangle
!     y2           y-coordinate second node triangle
!     y3           y-coordinate third node triangle
!     yi1          y-coordinate node i1
!     yi2          y-coordinate node i2
!     yi3          y-coordinate node i3
!     ym           y-coordinate barycentre
!     yma          maximal y-coordinate triangle
!     ymax         maximal y-coordinate tetrahedron
!     ymi          minimal y-coordinate triangle
!     ymin         minimal y-coordinate tetrahedron
!     z1           z-coordinate first node triangle
!     z2           z-coordinate second node triangle
!     z3           z-coordinate third node triangle
!     zi1          z-coordinate node i1
!     zi2          z-coordinate node i2
!     zi3          z-coordinate node i3
!     zm           z-coordinate barycentre
!     zma          maximal z-coordinate triangle
!     zmax         maximal z-coordinate tetrahedron
!     zmi          minimal z-coordinate triangle
!     zmin         minimal z-coordinate tetrahedron
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!    MSH610    Determine coordinate t of intersection point
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Check whether new point is allowed

      xi1 = coor( 1, i1 )
      yi1 = coor( 2, i1 )
      zi1 = coor( 3, i1 )

      xi2 = coor( 1, i2 )
      yi2 = coor( 2, i2 )
      zi2 = coor( 3, i2 )

      xi3 = coor( 1, i3 )
      yi3 = coor( 2, i3 )
      zi3 = coor( 3, i3 )

      xmin  = min(xi1,xi2,xi3,xn)
      xmax  = max(xi1,xi2,xi3,xn)
      ymin  = min(yi1,yi2,yi3,yn)
      ymax  = max(yi1,yi2,yi3,yn)
      zmin  = min(zi1,zi2,zi3,zn)
      zmax  = max(zi1,zi2,zi3,zn)

      ttest = 100d0
      kdrie = 0

      xm = ( xi1 + xi2 + xi3 ) / 3d0
      ym = ( yi1 + yi2 + yi3 ) / 3d0
      zm = ( zi1 + zi2 + zi3 ) / 3d0

      xm = 1d-5 * xn + (1d0 - 1d-5) * xm
      ym = 1d-5 * yn + (1d0 - 1d-5) * ym
      zm = 1d-5 * zn + (1d0 - 1d-5) * zm

      do i= 2, kstap
          ii1 = kstapl( 1, i )
          ii2 = kstapl( 2, i )
          ii3 = kstapl( 3, i )

          if ( ii1==jpn .or. ii2==jpn .or. ii3==jpn ) goto 100
          x1  = coor( 1, ii1 )
          y1  = coor( 2, ii1 )
          z1  = coor( 3, ii1 )
          x2  = coor( 1, ii2 )
          y2  = coor( 2, ii2 )
          z2  = coor( 3, ii2 )
          x3  = coor( 1, ii3 )
          y3  = coor( 2, ii3 )
          z3  = coor( 3, ii3 )

          xmi = min(x1,x2,x3)
          if ( xmi>xmax ) goto 100
          xma = max(x1,x2,x3)
          if ( xma<xmin ) goto 100
          ymi = min(y1,y2,y3)
          if ( ymi>ymax ) goto 100
          yma = max(y1,y2,y3)
          if ( yma<ymin ) goto 100
          if ( yma<=ymin ) goto 100
          zmi = min(z1,z2,z3)
          if ( zmi>zmax ) goto 100
          zma = max(z1,z2,z3)
          if ( zma<zmin ) goto 100

!         --- Check for point i1

          ji = 1
          if ( ii1==i1.or.ii2==i1.or.ii3==i1 ) ji = 0
          if ( ji==1 ) then
               call msh610(xn,yn,zn,xi1,yi1,zi1,x1,y1,z1,x2,y2,z2, &
                           x3,y3,z3,t)
               if ( t<ttest - 1d-6 ) then
                    ttest = t
                    kdrie = i
               end if
          end if

!         --- Check for point i2

          ji = 1
          if ( ii1==i2.or.ii2==i2.or.ii3==i2 ) ji = 0
          if ( ji==1 ) then
               call msh610(xn,yn,zn,xi2,yi2,zi2,x1,y1,z1,x2,y2,z2, &
                           x3,y3,z3,t)
               if ( t<ttest - 1d-6 ) then
                    ttest = t
                    kdrie = i
               end if
          end if

!         --- Check for point i3

          ji = 1
          if ( ii1==i3.or.ii2==i3.or.ii3==i3 ) ji = 0
          if ( ji==1 ) then
               call msh610(xn,yn,zn,xi3,yi3,zi3,x1,y1,z1,x2,y2,z2, &
                           x3,y3,z3,t)
               if ( t<(ttest - 1d-6) ) then
                    ttest = t
                    kdrie = i
               end if
          end if
          if ( ttest>1d0 ) then

!         --- Extra test for barycenter

                call msh610(xn,yn,zn,xm,ym,zm,x1,y1,z1,x2,y2,z2, &
                            x3,y3,z3,t)
                if ( t<(ttest - 1d-6) ) then
                     ttest = t
                     kdrie = i
                end if
          end if
100       continue
      end do

      end
