      subroutine msh632( coor, i1, i2, i3, i4, volume )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.0  date 08-09-2010 New release
!        version  2.0  date 25-01-2005 New norms
!        version  1.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  1.0  date 25-6-1988
!
!   copyright (c) 1988-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to compute volume of tetrahedron with nodes i1-i2-i3-i4
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     3d
!     tetrahedron
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
!     None
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer i1, i2, i3, i4
      double precision coor(3,*), volume

!     coor     i/o   array of dimension (3,npoint) containing the coor- &
!                    dinates of the nodes
!     i1        i    node number first node
!     i2        i    node number second node
!     i3        i    node number third node
!     i4        i    node number fourth node
!     volume    o    volume of tetrahedron i1-i2-i3-i4
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision a1, a2, a3, b1, b2, b3, c1, c2, c3, &
                       x1, x2, x3, x4, y1, y2, y3, y4, z1, z2, z3, z4

!     a1             help variable
!     a2             help variable
!     a3             help variable
!     b1             help variable
!     b2             help variable
!     b3             help variable
!     c1             help variable
!     c2             help variable
!     c3             help variable
!     x1             x-coordinate
!     x2             x-coordinate
!     x3             x-coordinate
!     x4             x-coordinate
!     y1             y-coordinate
!     y2             y-coordinate
!     y3             y-coordinate
!     y4             y-coordinate
!     z1             z-coordinate
!     z2             z-coordinate
!     z3             z-coordinate
!     z4             z-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     NONE
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Determine coordinates four nodes
!
!     Use general formula for volume of tetrahedron
!     (Order is important with respect to sign!)
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Determine coordinates of i1:

      x1 = coor(1,i1)
      y1 = coor(2,i1)
      z1 = coor(3,i1)

!     --- Coordinates i2:

      x2 = coor(1,i2)
      y2 = coor(2,i2)
      z2 = coor(3,i2)

!     --- Coordinates i3:

      x3 = coor(1,i3)
      y3 = coor(2,i3)
      z3 = coor(3,i3)

!     --- Coordinates i4:

      x4 = coor(1,i4)
      y4 = coor(2,i4)
      z4 = coor(3,i4)

!     --- Determine 1-2 differences ("distances")

      a1 = x2 - x1
      a2 = y2 - y1
      a3 = z2 - z1

!     --- 1-3 differences:

      b1 = x3 - x1
      b2 = y3 - y1
      b3 = z3 - z1

!     --- 1-4 differences:

      c1 = x4 - x1
      c2 = y4 - y1
      c3 = z4 - z1

!     --- Compute volume using standard formula:

      volume=a1*b2*c3-a1*c2*b3-a2*b1*c3+a3*b1*c2+a2*b3*c1-c1*a3*b2

!     --- Value computed: return

      end
