      subroutine msh638( kelem, nelem, ielem, kstapl, kstap, nipnt )
! ======================================================================
!
!        programmer    niek praagman
!        version  4.0  date 08-09-2010 New release
!        version  3.0  date 24-02-2005 New norms, simplification
!        version  2.0  date 04-03-1999 Rewrite, call to 646 (BvR)
!
!   copyright (c) 1988-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Place the boundary of the smallest subvolume that
!     contains and surrounds element ielem in array kstapl. Array kstapl
!     is used for a new division. Array kelem is updated : all
!     elements inside that subvolume are cancelled.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
!     None
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kelem(4,*), nelem, ielem, kstapl(3,*), kstap, nipnt

!     ielem      i    element to be considered
!     kelem     i/o   array containing the elements
!     kstap      o    length of array kstapl
!     kstapl     o    boundary array
!     nelem     i/o   Number of elements in kelem
!     nipnt      i    number of nodes in boundaries
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ia, ib, ic, id, i1, i2, i3, i4, jj, i, nextra

!     i          loop variable
!     i1         node number loopelement
!     i2         node number loopelement
!     i3         node number loopelement
!     i4         node number loopelement
!     ia         node number element ielem
!     ib         node number element ielem
!     ic         node number element ielem
!     id         node number element ielem
!     jj         indicator
!     nextra     counter "new" elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     MSH646    Add a new triangle to array KSTAPL or cancel the
!               triangle if it is already present in KSTAPL.
! **********************************************************************
!
!                       I/O
!
!     None
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     None
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Determine the nodes of the element to be canceled
!
!     Run through all elements
!     If element has point in common then
!        Check whether each of the four triangles of this element
!        has to be placed in or has to be removed from array kstapl, &
!        and handle accordingly:
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Investigate element ielem

      ia = kelem(1,ielem)
      ib = kelem(2,ielem)
      ic = kelem(3,ielem)
      id = kelem(4,ielem)

!     --- Boundary points (fixed!) are excluded

      if ( ia<=nipnt ) ia = 0
      if ( ib<=nipnt ) ib = 0
      if ( ic<=nipnt ) ic = 0
      if ( id<=nipnt ) id = 0

!     --- Set parameters kstap and nextra

      kstap  = 0
      nextra = 0

      do i = 1, nelem
         i1 = kelem(1,i)
         i2 = kelem(2,i)
         i3 = kelem(3,i)
         i4 = kelem(4,i)

         jj = 1

         if (i1==ia.or.i1==ib.or.i1==ic.or.i1==id) jj = 0
         if (i2==ia.or.i2==ib.or.i2==ic.or.i2==id) jj = 0
         if (i3==ia.or.i3==ib.or.i3==ic.or.i3==id) jj = 0
         if (i4==ia.or.i4==ib.or.i4==ic.or.i4==id) jj = 0

         if ( jj==0 ) then

!        --- Check whether each of the four triangles of this element
!            has to be placed in or has to be removed from array kstapl, &
!            and handle accordingly:

            call msh646( kstapl, kstap, i1, i2, i3 )
            call msh646( kstapl, kstap, i1, i4, i2 )
            call msh646( kstapl, kstap, i2, i4, i3 )
            call msh646( kstapl, kstap, i3, i4, i1 )
         else
            nextra = nextra + 1

            kelem(1,nextra) = i1
            kelem(2,nextra) = i2
            kelem(3,nextra) = i3
            kelem(4,nextra) = i4
         end if

      end do

!     --- Set number of remaining elements:

      nelem = nextra

      end
