      subroutine msh655( npoint, coor   , xstart , ystart, zstart, &
                         dismax, fix_pnt, nfixpnt, icube , &
                         nx    , ny     , kelem , nelem , &
                         kstapl, kstap  , itri , kbndpt, &
                         nbndpt  )
! ======================================================================
!
!        programmer    niek praagman
!        version  2.0  date 08-09-2010 New release
!        version  1.0  date 24-01-2005
!
!   copyright (c) 2005-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Place special fixed nodes and place elements around these
!   special nodes.
! **********************************************************************
!
!                       KEYWORDS
!
!     fixed_nodes
!     plaxis
!     mesh
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, nfixpnt, kelem(4,*), icube(*), &
              nelem, nx, ny, kstap, kstapl(3,*), itri(*), &
              kbndpt(*), nbndpt
      double precision coor(3,*), fix_pnt(4,nfixpnt), &
              xstart, ystart, zstart, dismax

!     coor          i   coordinate array
!     dismax        i   helpdistance for blockdetermination
!     fix_pnt       i   array with coordinates and coarseness of fixed points
!     icube         i   array with for each node the number of the
!                       quadrangle that node belongs to
!     itri         i/o  helparray containing information indicating whether
!                       a point is in the (actual) boundary or not
!     kelem        i/o  element array (created triangles)
!     kstap        i/o  length of boundary elements array
!     kstapl       i/o  array with nodes of boundary elements
!     nelem        i/o  number of created elements in interior in total
!     nfixpnt       i   number of fixed points
!     npoint        i   number of points in line elements of contour
!     nx            i   number of blocks in x-direction
!     ny            i   number of blocks in y-direction
!     xstart        i   x-coordinate left under enveloping block
!     ystart        i   y-coordinate left under enveloping block
!     zstart        i   z-coordinate left under enveloping block
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision dist, xp, yp, zp, xpoin, ypoin, zpoin
      integer i, ja, n1, n2, n3, nc, npoin, npoine

!     dist         helpvariable to store local meshsize
!     i            loop variable
!     ja           indicator whether point is inside area or not
!     n1           helpvariable x-number of block
!     n2           helpvariable y-number of block
!     n3           helpvariable z-number of block
!     nc           block number
!     npoin        number of points at start
!     npoine       number of points npoin plus number of fixed points
!     xp           x-coordinate
!     xpoin        x-coordinate new point to be placed
!     yp           y-coordinate
!     ypoin        y-coordinate new point to be placed
!     zp           z-coordinate
!     zpoin        z-coordinate new point to be placed
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     INSTOP         Stop the program
!     MSH607         check whether point is inside or outside volume
! **********************************************************************
!
!                       I/O
!
!    None
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    None
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Run through all (nfixpnt) special nodes
!
!    Check whether tetrahedrons with given coarseness are permitted.
!    If not: stop, error
!    Else: place nodes and elements and adjust arrys.
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
!    None
! ======================================================================
!
!     --- Set helpvalues

      npoin  = npoint
      npoint = npoint + nfixpnt

!     --- Run through all fixed nodes:

      do i = 1, nfixpnt

!     --- Start filling nodes in kbndpt:

         kbndpt(nbndpt+i) = npoin + i

!        --- Set helpvariable

         npoine = npoin + nfixpnt + (i-1) * 8

!        --- Check position of nodes:

         xp = fix_pnt(1,i)
         yp = fix_pnt(2,i)
         zp = fix_pnt(3,i)

!        --- Node p inside?

         call msh607( xp, yp, zp, xstart, coor, kstapl, kstap, ja )

         if ( ja==0 ) then

!        --- Stop: User submitted point that is not inside region
!            call errint ( userpoints(i), 1 )
!            call errint ( isurnr, 2 )
!            call erreal ( xp, 1 )
!            call erreal ( yp, 2 )
!            call erreal ( zp, 3 )

!            call errsub ( 1344, 2, 3, 0 )
!            go to 1000

            write(irefwr,*) 'Fixed node ',i,' is outside region: error'
            write(irefwr,*) 'execution of program is stopped'
            call instop
         end if

!        --- Node is in region:

         coor( 1, npoin+i ) = xp
         coor( 2, npoin+i ) = yp
         coor( 3, npoin+i ) = zp

         n1 = int ( (xp-xstart)/dismax )
         n2 = int ( (yp-ystart)/dismax )
         n3 = int ( (zp-zstart)/dismax )

!        --- This point belongs to quadrangle with number

         nc = 1 + n1 + n2*nx + n3*nx*ny

         icube( npoin+i ) = nc

!        --- Eight other points:

         dist = fix_pnt(4,i)

!        --- Point 1:

         xpoin = xp - 0.45 * dist
         ypoin = yp - 0.45 * dist
         zpoin = zp - 0.45 * dist

!        --- Inside ?

         call msh607( xpoin , ypoin, zpoin, xstart, coor, &
                      kstapl, kstap, ja )

         if ( ja==0 ) then

!        --- Stop: User submitted wrong coarseness:
!            call errint ( userpoints(i), 1 )
!            call errint ( isurnr, 2 )
!            call errint ( 1, 3 )
!            call erreal ( xp, 1 )
!            call erreal ( yp, 2 )
!            call erreal ( xpoin, 2 )
!            call erreal ( ypoin, 3 )
!            call errsub ( 1346, 3, 3, 0 )
!            go to 1000

            write(irefwr,*) 'Fixed node ',i,' is outside region: error'
            write(irefwr,*) 'execution of program is stopped'
            call instop
         end if

         npoint = npoint + 1

         coor(1,npoint) = xpoin
         coor(2,npoint) = ypoin
         coor(3,npoint) = zpoin

         icube( npoint ) = nc

!        --- Point 2:

         xpoin = xp + 0.45 * dist
         ypoin = yp - 0.45 * dist
         zpoin = zp - 0.45 * dist

!        --- Inside

         call msh607( xpoin , ypoin , zpoin , xstart, coor, &
                      kstapl, kstap , ja )

         if ( ja==0 ) then

!        --- Stop: User submitted wrong coarseness:
!            call errint ( userpoints(i), 1 )
!            call errint ( isurnr, 2 )
!            call errint ( 1, 3 )
!            call erreal ( xp, 1 )
!            call erreal ( yp, 2 )
!            call erreal ( xpoin, 2 )
!            call erreal ( ypoin, 3 )
!            call errsub ( 1346, 3, 3, 0 )
!            go to 1000

            write(irefwr,*) 'Fixed node ',i,' is outside region: error'
            write(irefwr,*) 'execution of program is stopped'
            call instop
         end if

         npoint = npoint + 1

         coor(1,npoint) = xpoin
         coor(2,npoint) = ypoin
         coor(3,npoint) = zpoin

         icube( npoint ) = nc

!        --- Point 3:

         xpoin = xp + 0.45 * dist
         ypoin = yp + 0.45 * dist
         zpoin = zp - 0.45 * dist

!        --- Inside ?

         call msh607( xpoin , ypoin , zpoin , xstart, coor  , &
                      kstapl, kstap , ja )

         if ( ja==0 ) then

!        --- Stop: User submitted wrong coarseness:
!            call errint ( userpoints(i), 1 )
!            call errint ( isurnr, 2 )
!            call errint ( 1, 3 )
!            call erreal ( xp, 1 )
!            call erreal ( yp, 2 )
!            call erreal ( xpoin, 2 )
!            call erreal ( ypoin, 3 )
!            call errsub ( 1346, 3, 3, 0 )
!            go to 1000

            write(irefwr,*) 'Fixed node ',i,' is outside region: error'
            write(irefwr,*) 'execution of program is stopped'
            call instop
         end if

         npoint = npoint + 1

         coor(1,npoint) = xpoin
         coor(2,npoint) = ypoin
         coor(3,npoint) = zpoin

         icube( npoint ) = nc

!        --- Point 4:

         xpoin = xp - 0.45 * dist
         ypoin = yp + 0.45 * dist
         zpoin = zp - 0.45 * dist

!        --- Inside ?

         call msh607( xpoin , ypoin , zpoin , xstart, coor  , &
                      kstapl, kstap , ja )

         if ( ja==0 ) then

!        --- Stop: User submitted wrong coarseness:
!            call errint ( userpoints(i), 1 )
!            call errint ( isurnr, 2 )
!            call errint ( 1, 3 )
!            call erreal ( xp, 1 )
!            call erreal ( yp, 2 )
!            call erreal ( xpoin, 2 )
!            call erreal ( ypoin, 3 )
!            call errsub ( 1346, 3, 3, 0 )
!            go to 1000

            write(irefwr,*) 'Fixed node ',i,' is outside region: error'
            write(irefwr,*) 'execution of program is stopped'
            call instop
         end if

         npoint = npoint + 1

         coor(1,npoint) = xpoin
         coor(2,npoint) = ypoin
         coor(3,npoint) = zpoin

         icube( npoint ) = nc

!        --- Point 5:

         xpoin = xp - 0.45 * dist
         ypoin = yp - 0.45 * dist
         zpoin = zp + 0.45 * dist

!        --- Inside ?

         call msh607( xpoin , ypoin , zpoin , xstart, coor  , &
                      kstapl, kstap , ja )

         if ( ja==0 ) then

!        --- Stop: User submitted wrong coarseness:
!            call errint ( userpoints(i), 1 )
!            call errint ( isurnr, 2 )
!            call errint ( 1, 3 )
!            call erreal ( xp, 1 )
!            call erreal ( yp, 2 )
!            call erreal ( xpoin, 2 )
!            call erreal ( ypoin, 3 )
!            call errsub ( 1346, 3, 3, 0 )
!            go to 1000

            write(irefwr,*) 'Fixed node ',i,' is outside region: error'
            write(irefwr,*) 'execution of program is stopped'
            call instop
         end if

         npoint = npoint + 1

         coor(1,npoint) = xpoin
         coor(2,npoint) = ypoin
         coor(3,npoint) = zpoin

         icube( npoint ) = nc

!        --- Point 6:

         xpoin = xp + 0.45 * dist
         ypoin = yp - 0.45 * dist
         zpoin = zp + 0.45 * dist

!        --- Inside ?

         call msh607( xpoin , ypoin , zpoin , xstart, coor  , &
                      kstapl, kstap , ja )

         if ( ja==0 ) then

!        --- Stop: User submitted wrong coarseness:
!            call errint ( userpoints(i), 1 )
!            call errint ( isurnr, 2 )
!            call errint ( 1, 3 )
!            call erreal ( xp, 1 )
!            call erreal ( yp, 2 )
!            call erreal ( xpoin, 2 )
!            call erreal ( ypoin, 3 )
!            call errsub ( 1346, 3, 3, 0 )
!            go to 1000

            write(irefwr,*) 'Fixed node ',i,' is outside region: error'
            write(irefwr,*) 'execution of program is stopped'
            call instop
         end if

         npoint = npoint + 1

         coor(1,npoint) = xpoin
         coor(2,npoint) = ypoin
         coor(3,npoint) = zpoin

         icube( npoint ) = nc

!        --- Point 7:

         xpoin = xp + 0.45 * dist
         ypoin = yp + 0.45 * dist
         zpoin = zp + 0.45 * dist

!        --- Inside ?

         call msh607( xpoin , ypoin , zpoin , xstart, coor  , &
                      kstapl, kstap , ja )

         if ( ja==0 ) then

!        --- Stop: User submitted wrong coarseness:
!            call errint ( userpoints(i), 1 )
!            call errint ( isurnr, 2 )
!            call errint ( 1, 3 )
!            call erreal ( xp, 1 )
!            call erreal ( yp, 2 )
!            call erreal ( xpoin, 2 )
!            call erreal ( ypoin, 3 )
!            call errsub ( 1346, 3, 3, 0 )
!            go to 1000

            write(irefwr,*) 'Fixed node ',i,' is outside region: error'
            write(irefwr,*) 'execution of program is stopped'
            call instop
         end if

         npoint = npoint + 1

         coor(1,npoint) = xpoin
         coor(2,npoint) = ypoin
         coor(3,npoint) = zpoin

         icube( npoint ) = nc

!        --- Point 8:

         xpoin = xp - 0.45 * dist
         ypoin = yp + 0.45 * dist
         zpoin = zp + 0.45 * dist

!        --- Inside ?

         call msh607( xpoin , ypoin , zpoin , xstart, coor  , &
                      kstapl, kstap , ja )

         if ( ja==0 ) then

!        --- Stop: User submitted wrong coarseness:
!            call errint ( userpoints(i), 1 )
!            call errint ( isurnr, 2 )
!            call errint ( 1, 3 )
!            call erreal ( xp, 1 )
!            call erreal ( yp, 2 )
!            call erreal ( xpoin, 2 )
!            call erreal ( ypoin, 3 )
!            call errsub ( 1346, 3, 3, 0 )
!            go to 1000

            write(irefwr,*) 'Fixed node ',i,' is outside region: error'
            write(irefwr,*) 'execution of program is stopped'
            call instop
         end if

         npoint = npoint + 1

         coor(1,npoint) = xpoin
         coor(2,npoint) = ypoin
         coor(3,npoint) = zpoin

         icube( npoint ) = nc

!        --- Add elements:

         nelem          = nelem  + 1
         kelem(1,nelem) = npoine + 1
         kelem(2,nelem) = npoine + 2
         kelem(3,nelem) = npoine + 4
         kelem(4,nelem) = npoin  + i

         nelem          = nelem  + 1
         kelem(1,nelem) = npoine + 2
         kelem(2,nelem) = npoine + 3
         kelem(3,nelem) = npoine + 4
         kelem(4,nelem) = npoin  + i

         nelem          = nelem  + 1
         kelem(1,nelem) = npoine + 1
         kelem(2,nelem) = npoine + 5
         kelem(3,nelem) = npoine + 6
         kelem(4,nelem) = npoin  + i

         nelem          = nelem  + 1
         kelem(1,nelem) = npoine + 1
         kelem(2,nelem) = npoine + 6
         kelem(3,nelem) = npoine + 2
         kelem(4,nelem) = npoin  + i

         nelem          = nelem  + 1
         kelem(1,nelem) = npoine + 4
         kelem(2,nelem) = npoine + 8
         kelem(3,nelem) = npoine + 5
         kelem(4,nelem) = npoin  + i

         nelem          = nelem  + 1
         kelem(1,nelem) = npoine + 1
         kelem(2,nelem) = npoine + 4
         kelem(3,nelem) = npoine + 5
         kelem(4,nelem) = npoin  + i

         nelem          = nelem  + 1
         kelem(1,nelem) = npoine + 3
         kelem(2,nelem) = npoine + 7
         kelem(3,nelem) = npoine + 4
         kelem(4,nelem) = npoin  + i

         nelem          = nelem  + 1
         kelem(1,nelem) = npoine + 4
         kelem(2,nelem) = npoine + 7
         kelem(3,nelem) = npoine + 8
         kelem(4,nelem) = npoin  + i

         nelem          = nelem  + 1
         kelem(1,nelem) = npoine + 2
         kelem(2,nelem) = npoine + 6
         kelem(3,nelem) = npoine + 3
         kelem(4,nelem) = npoin  + i

         nelem          = nelem  + 1
         kelem(1,nelem) = npoine + 3
         kelem(2,nelem) = npoine + 6
         kelem(3,nelem) = npoine + 7
         kelem(4,nelem) = npoin  + i

         nelem          = nelem  + 1
         kelem(1,nelem) = npoine + 5
         kelem(2,nelem) = npoine + 8
         kelem(3,nelem) = npoine + 6
         kelem(4,nelem) = npoin  + i

         nelem          = nelem  + 1
         kelem(1,nelem) = npoine + 6
         kelem(2,nelem) = npoine + 8
         kelem(3,nelem) = npoine + 7
         kelem(4,nelem) = npoin  + i

!        --- Adjust boundary array

         kstap           = kstap  + 1
         kstapl(1,kstap) = npoine + 1
         kstapl(2,kstap) = npoine + 4
         kstapl(3,kstap) = npoine + 2

         kstap           = kstap  + 1
         kstapl(1,kstap) = npoine + 2
         kstapl(2,kstap) = npoine + 4
         kstapl(3,kstap) = npoine + 3

         kstap           = kstap  + 1
         kstapl(1,kstap) = npoine + 1
         kstapl(2,kstap) = npoine + 6
         kstapl(3,kstap) = npoine + 5

         kstap           = kstap  + 1
         kstapl(1,kstap) = npoine + 1
         kstapl(2,kstap) = npoine + 2
         kstapl(3,kstap) = npoine + 6

         kstap           = kstap  + 1
         kstapl(1,kstap) = npoine + 4
         kstapl(2,kstap) = npoine + 5
         kstapl(3,kstap) = npoine + 8

         kstap           = kstap  + 1
         kstapl(1,kstap) = npoine + 4
         kstapl(2,kstap) = npoine + 1
         kstapl(3,kstap) = npoine + 5

         kstap           = kstap  + 1
         kstapl(1,kstap) = npoine + 3
         kstapl(2,kstap) = npoine + 4
         kstapl(3,kstap) = npoine + 7

         kstap           = kstap  + 1
         kstapl(1,kstap) = npoine + 4
         kstapl(2,kstap) = npoine + 8
         kstapl(3,kstap) = npoine + 7

         kstap           = kstap  + 1
         kstapl(1,kstap) = npoine + 2
         kstapl(2,kstap) = npoine + 3
         kstapl(3,kstap) = npoine + 6

         kstap           = kstap  + 1
         kstapl(1,kstap) = npoine + 3
         kstapl(2,kstap) = npoine + 7
         kstapl(3,kstap) = npoine + 6

         kstap           = kstap  + 1
         kstapl(1,kstap) = npoine + 5
         kstapl(2,kstap) = npoine + 6
         kstapl(3,kstap) = npoine + 8

         kstap           = kstap  + 1
         kstapl(1,kstap) = npoine + 6
         kstapl(2,kstap) = npoine + 7
         kstapl(3,kstap) = npoine + 8

!        --- Adjust array itri

         itri(npoine + 1) = 4
         itri(npoine + 2) = 4
         itri(npoine + 3) = 4
         itri(npoine + 4) = 6
         itri(npoine + 5) = 4
         itri(npoine + 6) = 6
         itri(npoine + 7) = 4
         itri(npoine + 8) = 4
      end do

      end
