      subroutine sepbuilddalloc ( indoldsols )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     De-allocate temporary arrays in for subroutine build
!
! **********************************************************************
!
!                       KEYWORDS
!
!     allocate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (indprevsols) :: indoldsols(numtypesols)

!     indoldsols    i/o   Contains information of the positions of the unknowns
!                         for the various vectors stored in oldsols
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: error, i
      logical :: debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     error          Return value of allocate or deallocate
!     i              general loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepbuilddalloc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepbuilddalloc'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      error = 0

      if ( associated(elemms) ) deallocate ( elemms, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'elemms' )
      if ( associated(coeffarr) ) deallocate ( coeffarr, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'coeffarr' )
      if ( associated(icoeffarr) ) deallocate ( icoeffarr, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'icoeffarr' )
      if ( associated(index1sub) ) deallocate ( index1sub, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'index1sub' )
      if ( associated(indexren) ) deallocate ( indexren, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'indexren' )
      if ( associated(index5) ) deallocate ( index5, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'index5' )
      if ( associated(index2) ) deallocate ( index2, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'index2' )
      if ( associated(index3) ) deallocate ( index3, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'index3' )
      if ( associated(index4) ) deallocate ( index4, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'index4' )

      if ( ierror/=0 ) go to 1000

      if ( associated(elemvc2d) ) deallocate ( elemvc2d, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'elemvc2d' )
      if ( associated(elemmt2d) ) deallocate ( elemmt2d, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'elemmt2d' )

      do i = 1, numtypesols
         nullify (indoldsols(i)%index3)
         nullify (indoldsols(i)%index4)
      end do  ! i = 1, numtypesols

1000  call erclos ( 'sepbuilddalloc' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepbuilddalloc'

      end if  ! ( debug )

      end subroutine sepbuilddalloc
