      function sepstartbuf ( indexv, lenarr, typearr, text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with pointer corresponding to indexv and corresponding length
!     It is checked if pointer is available
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     pointer
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulebuf
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer (kind=8) :: sepstartbuf
      integer, intent(out) :: lenarr
      integer, intent(in) :: indexv, typearr
      character (len=*) :: text

!     indexv        i    indexv refers to the array sequence number from which
!                        the starting position must be computed
!     sepstartbuf   o    Gives pointer in array IBUFFR corresponding to indexv
!     lenarr        o    Length of array
!     text          i    Text to be printed
!     typearr       i    indicates type of array
!                        1: integer
!                        2: double precision
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) :: istart
      integer :: iprio
      logical :: debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     iprio          Priority number
!     istart         Starting address in array IBUFFR
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRINTLONG     Put long integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     802   indexv out of range
!    1446   array has not been activated proerly
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Check if indexv between 1 and numarrs
!   Check if indexv has been activated
!   Set pointer
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepstartbuf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepstartbuf'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istart = numarrs+1
      if ( indexv<1 .or. indexv>numarrs ) then

!     --- indexv is not a valid index

         call errint ( indexv, 1 )
         call errint ( abs(indexv), 2 )
         call errint ( numarrs, 3 )
         call errchr ( text, 1 )
         call errsub ( 802, 3, 0, 1 )

      else

!     --- indexv in the correct range, check if array has been activated

         istart = infor(indexv)
         iprio  = infor1(2,indexv)
         if ( istart<0 .or. iprio<10 ) then

!        --- Array has not been activated properly

            call errint ( indexv, 1 )
            call errintlong ( istart, 2 )
            call errint ( iprio, 3 )
            call errchr ( namarr(indexv), 1 )
            call errchr ( text, 2 )
            call errsub ( 1446, 3, 0, 2 )

         end if

      end if

      if ( typearr==1 .or. intlen==1 ) then

!     --- Integer array

         sepstartbuf = istart
         lenarr = infor1(1,indexv)

      else

!     --- Double precision array

         sepstartbuf = (istart-1)/intlen+1
         lenarr = infor1(1,indexv)/intlen

      end if  ! ( typearr==1 )

1000  call erclos ( 'sepstartbuf' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepstartbuf'

      end if  ! ( debug )

      end function sepstartbuf
