      subroutine addstring
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.0  date 13-12-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Add string name to the list of strings
!
! **********************************************************************
!
!                       KEYWORDS
!
!     string
!     name
!     add
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecons
      use sepmoduleread
      use sepmoduleplot
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3159   No space for scalars left
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'addstring' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from addstring'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numstrings = numstrings+1   ! new sequence number of string

!     --- Check available space

      if ( numstrings>maxstrings ) then

!     --- numstrings>maxstrings

         call errint ( numstrings, 1 )
         call errint ( maxstrings, 2 )
         call errsub ( 3159, 2, 0, 0 )
         go to 1000

      end if  ! ( numstrings>maxstrings )

      chstrings(numstrings) = stringim(1:nchar)    ! name of string
      dimstrings(numstrings) = 1         ! dimension of string name
      ncharstrings(numstrings) = nchar   ! number of characters of string name
      index_rdarr(1) = numstrings

      instring(numstrings) = nextxt
      texts(nextxt) = ''
      nextxt = nextxt+1

1000  call erclos ( 'addstring' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End addstring'

      end if  ! ( debug )

      end subroutine addstring
