      subroutine askvecstr ( inpsep, istart, number, istrin, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret input following vi = stream_function
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     stream_function
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmodulecons
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(0:*), istart, number, istrin
      double precision :: rinsep(*)

!     inpsep         o    The integer input read is stored in array INPSEP of
!                         length 10
!                         Array INPSEP is filled as follows:
!                         0: Type of function to be called
!                            63:  stream function
!                         1: output vector
!                         2: input vector
!                         3: start_node
!                         4: stream_function value
!     istart        i/o   Next free position in inpsep
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     number         i    Vector sequence number
!     rinsep         i    Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivec, m

!     ivec           Sequence number of input vector
!     m              number of values stored in tabout for keyword
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRGETNAME      Get sequence number of vector with specific name
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3176   No vector found after the keyword stream_function
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'askvecstr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askvecstr'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iposut = nint(table(3))
      m = nint(tabmax(3))
      ivec = -1
      if ( m>0 ) ivec = nint(tabout(iposut))   ! input vector

      if ( ivec==-1 ) call prgetname ( 'velocity', ivec )

      if ( ivec==-1 ) then

!     --- No vector found

         call errsub ( 3176, 0, 0, 0 )

      end if  ! ( ivec==-1 )

      inpsep(0) = 63                ! icode for stream function
      inpsep(1) = number            ! output vector
      inpsep(2) = ivec              ! input vector
      inpsep(3:4) = 0
      if ( table(43)>0d0 ) inpsep(3) = nint(table(43))  ! start_node

      if ( table(44)>-1d29 ) then

!     --- start_value found

         inpsep(4) = istrin
         rinsep(istrin) = table(44)
         istrin = istrin+1

      end if  ! ( table(44)>0d0 )

      if ( number>0 .and. number<=maxvec ) ivectors(number) = 1
      istart = istart + 5

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpsep, 5, 'inpsep' )
         write(irefwr,*) 'End askvecstr'

      end if  ! ( debug )
      call erclos ( 'askvecstr' )

      end
