      subroutine ellegp(n, poly, pder, polym1, pderm1, polym2, &
                        pderm2, y)
! ====================================================================== &
!
!     programmer Luc Timmermans
!     version 1.1    date  06-03-1996 Layout
!     version 1.0    date  16-11-1990
!
!   copyright (c) 1990-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!***********************************************************************
!
!                         DESCRIPTION
!
!     computes the Legendre polynomial and its derivative of degree n
!
! ********************************************************************
!
!                      KEYWORDS
!
!   legendre
!   polynomial
!   derivative
! ********************************************************************
!
!                 INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer n
      double precision poly, pder, polym1, pderm1, polym2, pderm2, y

!     n          i   degree of the Legendre polynomial
!     pder       o   Legendre derivative of degree n
!     pderm1     o   Legendre derivative of degree n-1
!     pderm2     o   Legendre derivative of degree n-2
!     poly       o   Legendre polynomial of degree n
!     polym1     o   Legendre polynomial of degree n-1
!     polym2     o   Legendre polynomial of degree n-2
!     y          i   computing point
! ********************************************************************
!
!                    COMMON BLOCKS
!
!***********************************************************************
!
!     LOCAL PARAMETERS

      integer k
      double precision polyls, pderls, a1, a2, a3, b1, psave, pdsave, &
                       polyn, pdern, rk

!     a1         coefficients for the recurrence relation
!     a2         coefficients for the recurrence relation
!     a3         coefficients for the recurrence relation
!     b1         coefficients for the recurrence relation
!     k          index parameter
!     pderls     highest order derivative for the recurrence relation
!     pdern      desired derivative in the loop
!     pdsave     lowest order derivative for the recurrence relation
!     polyls     highest order polynomial for the recurrence relation
!     polyn      desired polynomial in the loop
!     psave      lowest order polynomial for the recurrence relation
!     rk         double precision value of K
! ********************************************************************
!
!             SUBROUTINES CALLED
!
! ********************************************************************
!
!                    I/O
!
! ********************************************************************
!
!                    ERROR MESSAGES
!
! ********************************************************************
!
!                    PSEUDO CODE
!
! ====================================================================== &
!
      poly = 1d0
      pder = 0d0
      if ( n.eq.0 ) return
      polyls = poly
      pderls = pder
      poly = y
      pder = 1d0
      if ( n.eq.1 ) return

      do 100 k = 2,n
         rk = dble(k)
         a1 = 4d0*rk*rk*(rk - 1d0)
         b1 = 2d0*rk - 2d0
         a2 = b1*(b1 + 1d0)*(b1 + 2d0)
         a3 = 4d0*rk*(rk - 1d0)*(rk - 1d0)
         polyn = (a2*y*poly - a3*polyls)/a1
         pdern = (a2*y*pder - a3*pderls + a2*poly)/a1
         psave = polyls
         pdsave = pderls
         polyls = poly
         poly = polyn
         pderls = pder
         pder = pdern
100   continue

      polym1 = polyls
      pderm1 = pderls
      polym2 = psave
      pderm2 = pdsave

      end
