      subroutine er0047 ( linear, coorol, n, anorm )
! ====================================================================== &
!
!        programmer   Guus Segal
!        version 1.3  date   17-08-1996 Normalize correctly
!        version 1.2  date   07-03-1996 Layout
!        version 1.1  date   22-08-1991 Zdenek: no dimension, cdc, ce
!        version 1.0  date   23-08-1987
!
!
!
!   copyright (c) 1987-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by an2 means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill for each node in array coorol the normal to the side
!     2D only
! **********************************************************************
!
!                       KEYWORDS
!
!    2d
!    fill
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      double precision coorol(2,*), anorm(2,*)
      integer linear, n

!    anorm   o   Array containing the normals to the boundary in the
!                sequence:
!                 nx , ny , nx , ny , . . . .  nx , ny
!                   1    1    2    2             n    n
!    coorol  i   Array containing the co-ordinates of the boundary in the
!                sequence:
!                 x , y , x , y , . . . .  x , y
!                  1   1   2   2            n   n
!    linear  i   Indication whether the elements are linear (linear=1)
!                or quadratic (linear=2)
!    n       i   Number of points in coorol
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision x1, y1, an1, an2, x2, y2, h, x, y, &
                       x3, y3, dxdksi, dydksi, xp, yp, bn1, bn2
      integer i

!     an1        First component of norm
!     an2        Second component of norm
!     bn1        First component of norm of pevious element
!     bn2        Second component of norm of pevious element
!     dxdksi     dx/dksi in a point
!     dydksi     dy/dksi in a point
!     h          Help variable to store a constant temporarily
!     i          Counting variable
!     x          x-coordinate of previous point
!     x1         x-coordinate of first point in quadratic element
!     x2         x-coordinate of second point in quadratic element
!     x3         x-coordinate of third point in quadratic element
!     xp         x-coordinate of next point
!     y          y-coordinate of previous point
!     y1         y-coordinate of first point in quadratic element
!     y2         y-coordinate of second point in quadratic element
!     y3         y-coordinate of third point in quadratic element
!     yp         y-coordinate of next point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      if ( linear.eq.1 ) then

!     --- linear = 1 ,  linear elements

         xp = coorol(1,1)
         yp = coorol(2,1)
         x  = coorol(1,2)
         y  = coorol(2,2)
         an1 = y-yp
         an2 = x-xp
         h = 1d0/sqrt(an1*an1+an2*an2)
         an1 = an1*h
         an2 = -an2*h
         anorm(1,1) = an1
         anorm(2,1) = an2
         do i = 2, n-1

!        --- Compute nx and ny from tangential vector

            xp= coorol(1,i)
            yp= coorol(2,i)
            x  = coorol(1,i+1)
            y  = coorol(2,i+1)
            bn1 = y-yp
            bn2 = x-xp
            h = 1d0/sqrt(bn1*bn1+bn2*bn2)
            bn1 = bn1*h
            bn2 = -bn2*h
            an1 = 0.5d0*(an1+bn1)
            an2 = 0.5d0*(an2+bn2)
            h = 1d0/sqrt(an1*an1+an2*an2)
            an1 = an1*h
            an2 = an2*h
            anorm(1,i) = an1
            anorm(2,i) = an2
            an1 = bn1
            an2 = bn2

         end do
         anorm(1,n) = an1
         anorm(2,n) = an2

      else

!     --- linear = 2   quadratic elements

         x1 = coorol(1,1)
         y1 = coorol(2,1)
         do i = 1, n-1, 2

            x2=coorol(1,i+1)
            y2=coorol(2,i+1)
            x3=coorol(1,i+2)
            y3=coorol(2,i+2)

!           --- compute the mean value of the normal

            dxdksi=-1.5d0*x1+2d0*x2-0.5d0*x3
            dydksi=-1.5d0*y1+2d0*y2-0.5d0*y3
            if ( i.eq.1 ) then
               h = 1d0/sqrt(dxdksi**2+dydksi**2)
               an1=dydksi*h
               an2=-dxdksi*h
            else
               an1=(an1+dydksi)*0.5d0
               an2=(an2-dxdksi)*0.5d0
               h=1d0/sqrt(an1**2+an2**2)
               an1=an1*h
               an2=an2*h
            end if
            anorm(1,i)=an1
            anorm(2,i)=an2
            dxdksi=-0.5d0*x1+0.5d0*x3
            dydksi=-0.5d0*y1+0.5d0*y3
            h = 1d0/sqrt(dxdksi**2+dydksi**2)
            an1=dydksi*h
            an2=-dxdksi*h
            anorm(1,i+1)=an1
            anorm(2,i+1)=an2
            dxdksi=0.5d0*x1-2d0*x2+1.5d0*x3
            dydksi=0.5d0*y1-2d0*y2+1.5d0*y3
            an1=dydksi
            an2=-dxdksi
            x1=x3
            y1=y3
         end do
         h = 1d0/sqrt(an1**2+an2**2)
         an1 = an1*h
         an2 = an2*h
         anorm(1,n)=an1
         anorm(2,n)=an2

      end if
      end
