      subroutine erfillboun ( iwork, ihelp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-01-2001
!
!   copyright (c) 2000-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iwork with a 1 for all boundary nodes and a 0 for all
!     internal nodes
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     node
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), ihelp(*)

!     ihelp               integer work array of length ncurvs
!                         ihelp(i) = 0, means curve is external
!                         ihelp(i) = 1, means curve is internal
!     iwork          o    integer work array of size npoint
!                         iwork1 is filled in the following way:
!                         iwork1(i) = 0:  node i is internal
!                         iwork1(i) = 1:  node i is on the boundary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icurnr, istar3, istar4

!     i              Counting variable
!     icurnr         Curve sequence number
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istar4         Starting address of KMESHM part 4 in KMESH part M
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Clear iwork
!    For all outer curves do
!       For all nodes i do
!          iwork(i) := 1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Clear iwork

      do i = 1, npoint
         iwork(i) = 0
      end do
      do i = 1, ncurvs
         ihelp(i) = 0
      end do

!     --- For all outer curves do

      ninner = kmeshm(1)
      istar3 = kmeshm(2)-1
      istar4 = kmeshm(3)-1
      do i = 1, ninner
         ihelp(kmeshm(istar4+i)) = 1
      end do

      do icurnr = 1, ncurvs

         if ( ihelp(icurnr)==0 ) then

!        --- For all nodes i do
!               iwork(i) := 1

            do i = kmeshm(icurnr), kmeshm(icurnr+1)-1
               iwork(kmeshm(istar3+i)) = 1
            end do

         end if

      end do

      end
