      subroutine errchkdim ( numcomps, iinput, itypeinp, name )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.0  date 24-04-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check dimension of array or scalar
!
! **********************************************************************
!
!                       KEYWORDS
!
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecons
      use sepmoduleplot
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numcomps, iinput, itypeinp
      character (len=*) name

!     iinput         i    integer to be checked (position in input array)
!     itypeinp       i    Indicates type of input
!                         1:  constant or scalar
!                         2:  vector
!     name           i    Name of parameter (to be used for error message)
!     numcomps       i    Dimension that is expected
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinputred
      character (len=32) namevar

!     iinputred      Reduced input variable iinput (-5000)
!     namevar        Name of variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3134   Constant, vector or scalar not indexed
!    3135   dimension of constant, vector or scalar not correct
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'errchkdim' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from errchkdim'
         write(irefwr,1) 'iinput, itypeinp', iinput, itypeinp
         write(irefwr,*) 'name', name
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iinputred = mod(iinput,100000000)

      if ( iinput>5000 ) then

!     --- iinput has dimension, correct with value 5000

         iinputred = iinputred-5000

      else if ( numcomps>0 ) then

!     --- iinputred has no dimension, error

         if ( itypeinp==1 ) then

!        --- constant or scalar found

            if ( iinputred<=maxints ) then
               namevar = chincons(iinputred)
            else if ( iinputred<=maxints+maxreals ) then
               namevar = chrlcons(iinputred-maxints)
            else
               namevar = chscalars(iinputred-maxints-maxreals)
            end if  ! ( iinputred<=maxints )

         else if ( itypeinp==2 ) then

!        --- vector found

            namevar = names(iinputred)

         else

            namevar = name

         end if  ! ( itypeinp==1 )

         call errchr ( namevar, 1 )
         call errchr ( name, 2 )
         call errint ( iinputred, 1 )
         call errsub ( 3134, 1, 0, 2 )
         go to 1000

      end if  ! ( iinput>5000 )

      if ( numcomps==0 ) go to 1000   ! jump because not indexed

      if ( itypeinp==1 ) then

!     --- iinput is constant or scalar

         if ( iinputred>maxints+maxreals ) then

!        --- scalar found

            iinputred = iinputred-maxints-maxreals
            if ( numcomps/=dimscals(iinputred) ) then

!           --- Error: dimensions are not equal

               call errint ( iinputred, 1 )
               call errint ( dimscals(iinputred), 2 )
               call errint ( numcomps, 3 )
               call errchr ( chscalars(iinputred), 1 )
               call errchr ( name, 2 )
               call errsub ( 3135, 3, 0, 2 )

            end if  ! ( numcomps/=dimscals(iinputred) )

         else

!        --- constant found

            if ( numcomps/=dimcons(iinputred) ) then

!           --- Error: dimensions are not equal

               if ( iinputred>maxints ) then
                  namevar = chrlcons(iinputred-maxints)
               else
                  namevar = chincons(iinputred)
               end if  ! ( iinpured>maxints )

               call errint ( iinputred, 1 )
               call errint ( dimcons(iinputred), 2 )
               call errint ( numcomps, 3 )
               call errchr ( namevar, 1 )
               call errchr ( name, 2 )
               call errsub ( 3135, 3, 0, 2 )

            end if  ! ( numcomps/=dimcons(iinputred) )

         end if  ! ( iinputred>maxints+maxreals )

      else

!     --- vector found

         if ( numcomps/=dimnames(iinputred) ) then

!        --- Error: dimensions are not equal

            call errint ( iinputred, 1 )
            call errint ( dimnames(iinputred), 2 )
            call errint ( numcomps, 3 )
            call errchr ( names(iinputred), 1 )
            call errchr ( name, 2 )
            call errsub ( 3135, 3, 0, 2 )

         end if  ! ( numcomps/=dimnames(iinputred) )

      end if  ! ( itypeinp==1 )

1000  call erclos ( 'errchkdim' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End errchkdim'

      end if  ! ( debug )

      end
