      subroutine errcontreal ( realfirst, reallast, realval, name )
! ====================================================================== &
!
!        programmer    Guus Segal
!        version  1.0  date 12-12-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check control integer around a real work array
!
! **********************************************************************
!
!                       KEYWORDS
!
!     check
!     array
!     array_length
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision realfirst, reallast, realval
      character *(*) name

!     name           i    Name of array
!     realfirst      i    Real value of first control integer
!     reallast       i    Real value of last control integer
!     realval        i    Real value of original control integer
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2475   Leading control value incorrect
!    2476   Trailing control value incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'errcontreal' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from errcontreal'

      end if
      if ( ierror.ne.0 ) go to 1000
      if ( abs(realfirst-realval).gt.1d-10 ) then

!     --- First control integer incorrect

         call erreal ( realfirst, 1 )
         call erreal ( realval, 2 )
         call errchr ( name, 1 )
         call errsub ( 2475, 0, 2, 1 )

      end if

      if ( abs(reallast-realval).gt.1d-10 ) then

!     --- last control integer incorrect

         call erreal ( reallast, 1 )
         call erreal ( realval, 2 )
         call errchr ( name, 1 )
         call errsub ( 2476, 0, 2, 1 )

      end if

1000  call erclos ( 'errcontreal' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End errcontreal'

      end if

      end
