      subroutine errleadingd ( ileading, ivalue, namearray )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-11-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check leading control integer in case of temporary array
!
! **********************************************************************
!
!                       KEYWORDS
!
!     check
!     integer
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivalue
      double precision ileading
      character (len=*) namearray

!     ileading       i    Real that should have the value ivalue
!     ivalue         i    Integer value
!     namearray      i    Name of array in error message
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3089   leading control integer incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'errleadingd' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from errleadingd'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( nint(ileading)/=ivalue ) then

!     --- leading control integer incorrect

         call erreal ( ileading, 1 )
         call errint ( ivalue, 2 )
         call errchr ( namearray, 1 )
         call errsub ( 3089, 1, 1, 1 )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End errleadingd'

      end if
      call erclos ( 'errleadingd' )

      end
