      subroutine fillpc
! ======================================================================
!
!        programmer    Luc Timmermans/Frans van de Vosse
!        version  2.8  date 29-05-2014 Remove time routine
!        version  2.7  date 21-03-2007 Fill kmesh2 part t
!        version  2.6  date 12-08-1999 New call to erclmn
!
!   copyright (c) 1992-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       fillpc fills kmesh2 part a and c
!
! **********************************************************************
!
!                       KEYWORDS
!
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp, irelem, inpelm, mone, nn, &
              ipk1c, ipk2c, mhelp, icount, jcount, mtwo, &
              n1, n2, n3, ielemorig, numrowkelmt, i

!     i              loop variable
!     icount         counter
!     ielemorig      Element sequence number in original mesh
!     ielgrp         element group
!     inpelm         number of points per element
!     ipk1c          pointer in kmesh1 part c
!     ipk2c          pointer in kmesh2 part c
!     irelem         relative element number with respect to an element group
!     jcount
!     mhelp          help parameter
!     mone           square root of inpelm
!     mtwo           mtwo = mone*mone
!     n1             loop variable
!     n2             loop variable
!     n3             loop variable
!     nn             relative node number per element
!     numrowkelmt    Number of rows in kmesh part t
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1696   Number of nodes per element incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('fillpc')

!     --- Find information concerning kmesh part t

      if ( nvolms>0 ) then

!     --- Volumes found

         numrowkelmt = nvolms

      else if ( nsurfs>0 ) then

!     --- Surfaces found

         numrowkelmt = nsurfs

      else

!     --- Curves found

         numrowkelmt = ncurvs

      end if  ! ( nvolms>0 )
      allocate ( km(iseqkm)%kmesht(2,numrowkelmt) )
      km(iseqkm)%kmesh(36) = numrowkelmt

!     --- fetch the number of element groups in the spectral mesh

      ipk1c = 1
      ipk2c = 1
      if (nelgrp>1) then
         allocate ( km(iseqkm)%kmeshg(nelgrp) )
         km(iseqkm)%kmesh(21) = nelgrp
         km(iseqkm)%kmesha(1) = 0
      end if  ! (nelgrp>1)

!     --- loop over the element groups

      icount = 0
      jcount = 0
      ielemorig = 0

      do ielgrp = 1, nelgrp

!     --- check if the mesh is a spectral element mesh

         ishape = kmeshf(ielgrp)

         if (ierror/=0) goto 1000

!        --- fetch the number of elements in this group

         inelem = Topgrp(ielgrp)%inelem

!        --- fetch the number of nodes per element in this group

         inpelm = Topgrp(ielgrp)%inpelm
         if (ishape==-10001) then

            if (nelgrp>1) km(iseqkm)%kmesha(1+ielgrp) = &
                                km(iseqkm)%kmesha(ielgrp)+2
            do irelem = 1, inelem
               km(iseqkm)%kmeshc(ipk2c) = kmeshc(ipk1c)
               km(iseqkm)%kmeshc(ipk2c+1) = kmeshc(ipk1c+1)
               ipk1c = ipk1c + 2
               ipk2c = ipk2c + 2
               jcount = jcount + 1
               icount = icount + 1
            end do
            ielemorig = ielemorig+inelem

         else if (ishape==1) then

            if (nelgrp>1) km(iseqkm)%kmesha(1+ielgrp) = &
                                km(iseqkm)%kmesha(ielgrp)+2
            mone = inpelm
            mhelp = mone-1
            do irelem = 1, inelem

!           --- Raise ielemorig and adapt kmesh part 2 if necessary

               ielemorig = ielemorig+1
               do i = 1, numrowkelmt
                  if ( kmesht(1,i)==ielemorig ) &
                      km(iseqkm)%kmesht(1,i) = jcount+1
               end do  ! i = 1, numrowkelmt

               do nn = 1, mhelp
                  km(iseqkm)%kmeshc(ipk2c) = kmeshc(ipk1c)
                  km(iseqkm)%kmeshc(ipk2c+1) = kmeshc(ipk1c+1)
                  ipk1c = ipk1c + 1
                  ipk2c = ipk2c + 2
                  jcount = jcount + 1
                  icount = icount + 1
               end do
               ipk1c = ipk1c + 1

               do i = 1, numrowkelmt
                  if ( kmesht(2,i)==ielemorig ) &
                      km(iseqkm)%kmesht(2,i) = jcount
               end do  ! i = 1, numrowkelmt

            end do

         else if (ishape==5) then

            if (nelgrp>1) km(iseqkm)%kmesha(1+ielgrp) = &
                                km(iseqkm)%kmesha(ielgrp)+4
            mone = int(sqrt(dble(inpelm)))
            mhelp = mone*(mone - 1) - 1

!           --- loop over the elements

            do irelem = 1, inelem

!           --- Raise ielemorig and adapt kmesh part 2 if necessary

               ielemorig = ielemorig+1
               do i = 1, numrowkelmt
                  if ( kmesht(1,i)==ielemorig ) &
                      km(iseqkm)%kmesht(1,i) = jcount+1
               end do  ! i = 1, numrowkelmt
               if ( kmesht(1,1)==ielemorig ) &
                  km(iseqkm)%kmesht(1,1) = jcount+1

!              --- loop over the nodes

               do nn = 1, mhelp

!              --- fill kmesh 2 part c

                  if (mod(nn,mone)/=0) then

                     km(iseqkm)%kmeshc(ipk2c) = kmeshc(ipk1c)
                     km(iseqkm)%kmeshc(ipk2c+1) = kmeshc(ipk1c+1)
                     km(iseqkm)%kmeshc(ipk2c+2) = kmeshc(ipk1c+mone+1)
                     km(iseqkm)%kmeshc(ipk2c+3) = kmeshc(ipk1c+mone)
                     ipk2c = ipk2c + 4
                     ipk1c = ipk1c + 1
                     jcount = jcount + 1
                     icount = icount + 1
                  else

                     ipk1c = ipk1c + 1
                  end if
               end do

               ipk1c = ipk1c + mone + 1

               do i = 1, numrowkelmt
                  if ( kmesht(2,i)==ielemorig ) &
                      km(iseqkm)%kmesht(2,i) = jcount
               end do  ! i = 1, numrowkelmt

            end do

         else if (ishape==13 .or. ishape==14) then

            if (nelgrp>1) km(iseqkm)%kmesha(1+ielgrp) = &
                                km(iseqkm)%kmesha(ielgrp)+8
            mone = int(inpelm**(1d0/3d0)+.5d0)
            if( mone**3/=inpelm ) then
               call errint ( inpelm, 1 )
               call errint ( mone, 2 )
               call errint ( mone**3, 3 )
               call errsub ( 1696, 3, 0, 0 )
               go to 1000
            end if

            mtwo = mone*mone

!           --- loop over the elements

            do irelem = 1, inelem

!           --- Raise ielemorig and adapt kmesh part 2 if necessary

               ielemorig = ielemorig+1
               do i = 1, numrowkelmt
                  if ( kmesht(1,i)==ielemorig ) &
                      km(iseqkm)%kmesht(1,i) = jcount+1
               end do  ! i = 1, numrowkelmt

!              --- loop over the nodes

               do n3 = 1, mone-1
               do n2 = 1, mone-1
               do n1 = 1, mone-1

!              --- fill kmesh 2 part c

                     km(iseqkm)%kmeshc(ipk2c) = kmeshc(ipk1c)
                     km(iseqkm)%kmeshc(ipk2c+1) = kmeshc(ipk1c+1)
                     km(iseqkm)%kmeshc(ipk2c+2) = kmeshc(ipk1c+mone+1)
                     km(iseqkm)%kmeshc(ipk2c+3) = kmeshc(ipk1c+mone)
                     km(iseqkm)%kmeshc(ipk2c+4) = kmeshc(ipk1c+mtwo)
                     km(iseqkm)%kmeshc(ipk2c+5) = kmeshc(ipk1c+mtwo+1)
                     km(iseqkm)%kmeshc(ipk2c+6) = kmeshc(ipk1c+mtwo+mone+1)
                     km(iseqkm)%kmeshc(ipk2c+7) = kmeshc(ipk1c+mtwo+mone)
                     ipk2c = ipk2c + 8
                     jcount = jcount + 1
                     icount = icount + 1
                     ipk1c = ipk1c + 1

               end do

               ipk1c = ipk1c + 1
               end do

               ipk1c = ipk1c + mone
               end do

               ipk1c = ipk1c + mtwo

               do i = 1, numrowkelmt
                  if ( kmesht(2,i)==ielemorig ) &
                      km(iseqkm)%kmesht(2,i) = jcount
               end do  ! i = 1, numrowkelmt

            end do

         end if
         if (nelgrp>1) km(iseqkm)%kmeshg(ielgrp) = jcount
         jcount = 0
      end do

      km(iseqkm)%kmesh(9) = icount

1000  call erclos('fillpc')
      end
