      function getindconst ( realname, loopname )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Get the value of the real in the block constants with name realname
!     If the name does not exist an error message is given
! **********************************************************************
!
!                       KEYWORDS
!
!     name
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons 
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*) realname, loopname
      double precision getindconst

!     getindconst    o    Value of constant
!     loopname       i    Name of loop variable
!     realname       i    Name of real
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq

!     iseq           Sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getnameint, getnameindreal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRWAR         Print warnings
!     GETNAMEINDREAL Get the sequence number corresponding to the real in the
!                    block constants with name realname
!     GETNAMEINT     Return with sequence number of integer in incons
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2156   real name not found
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find position of name in chrlcons
!     If ( name in chrlcons ) then
!        getindconst := value
!     else
!        Find position of name in chincons
!        If ( name in chincons ) then
!           getindconst := value
!        else
!           error message
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      iseq = getnameindreal ( realname, loopname )
      if ( iseq>0 ) then
         getindconst = rlcons(iseq)
      else
         iseq = getnameint ( realname )
         if ( iseq>0 ) then
            getindconst = incons(iseq)
         else
            getindconst = 0
            call eropen ( 'getindconst' )
            call errchr ( realname, 1 )
            call errwar ( 2156, 0, 0, 1 )
            call erclos ( 'getindconst' )
         end if
      end if
      end
