      subroutine inireducevec ( ivecin, ivectr, numarrays )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the reduced value of ivecin which indicates the first
!     sequence number of a (series of) vectors, as well as the dimension
!     of the series
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     number
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivecin, ivectr, numarrays

!     ivecin         i    Actual sequence number of vector after reduction
!     ivectr         o    Sequence number of vector before reduction
!     numarrays      o    Number of indexed vectors in vector array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal, inivectr

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIREPSCAL     Corrects integer value in case of scalars
!     INIVECTR       Return with sequence number of vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inireducevec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inireducevec'
         write(irefwr,1) 'ivecin', ivecin
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ivectr = inirepscal(ivecin)

      if ( debug ) write(irefwr,1) 'ivectr/1', ivectr
      if ( ivectr>0 ) ivectr = inivectr(ivectr)
      if ( debug ) write(irefwr,1) 'ivectr/2', ivectr

      if ( ivectr>5000 ) then

!     --- A series of arrays must be used

         ivectr = ivectr-5000
         numarrays = dimnames(ivectr)

      else

!     --- Only one vector must be filled

         numarrays = 1

      end if  ! ( ivectr>5000 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ivectr, numarrays', ivectr, numarrays
         write(irefwr,*) 'End inireducevec'

      end if  ! ( debug )
      call erclos ( 'inireducevec' )

      end
