      function inirepscal ( ivalue )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 08-12-2013 Extension with integer scalars
!        version  1.2  date 22-11-2004 Fortran 90
!        version  1.1  date 15-12-2000 Avoid error
!
!   copyright (c) 2000-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    This subroutine returns with ivalue if ivalue<100000000
!    If ivalue>100000000 it returns with the scalar corresponding to
!    ivalue - 100000000
! **********************************************************************
!
!                       KEYWORDS
!
!     variable
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons 
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inirepscal, ivalue

!     inirepscal     o    Result of operation
!     ivalue         i    Integer value to be transformed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2952   Sequence number of scalar out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inirepscal' )
      if ( ivalue>100000000 ) then

!     --- scalar found

        inirepscal = ivalue-100000000
        if ( inirepscal<maxcls+maxintscals ) then
           inirepscal = nint(scalars(ivalue-100000000))
        else
           call errint ( inirepscal, 1 )
           call errint ( maxcls+maxintscals, 2 )
           call errsub ( 2952, 2, 0, 0 )
        end if  ! ( inirepscal<maxcls+maxintscals )

      else

!     --- standard case

        inirepscal = ivalue

      end if
      call erclos ( 'inirepscal' )
      end
