      subroutine inistr1 ( number, itypvc, ivec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 04-03-2011 Extension with probextend
!        version  1.0  date 11-09-2008
!
!   copyright (c) 2008-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find the structure of the vector with exactly NUMBER unknowns per point
!     If this type does not exist an error message is given
! **********************************************************************
!
!                       KEYWORDS
!
!     find
!     structure
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer number, itypvc, ivec

!     itypvc        i/o   Indication of the type of solution vector
!                         If itypvc = 110 at input then the output vector may be
!                         a vector of type 110 or 115 else of type 115 only
!                         Output:
!                         110: solution vector
!                         115: vector of special structure
!     ivec           o    Sequence number of vector of special structure
!     number         i    The required number of unknowns per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jprob

!     jprob          Problem number corresponding to specific vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PROBEXTEND     Extend problem definition with vectors of special structure
!                    of exactly number degrees of freedom
!     SEPCREASTRUC1  Find the structure of the vector with exactly
!                    NUMBER unknowns per point
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inistr1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inistr1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      itypvc = 115

      do jprob = 1, nprob

         iprob = jprob
         call sepcreastruc1 ( itypvc, ivec, number )
         if ( ivec>0 ) go to 1000   ! vector found

      end do  ! iprob = 1, nprob

      if ( ivec==0 ) then

!     --- No vector found

         call probextend ( number )
         ivec = kprobloc(6)

      end if  ! ( ivec==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inistr1'

      end if  ! ( debug )
      call erclos ( 'inistr1' )

      end
