      subroutine msh021 ( irefc, lengpc, ihelp, nboun )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.11 date 30-03-2004 Extension with shape 134
!        version  4.10 date 14-03-2003 Extension with shapes > 100
!        version  4.9  date 18-03-2002 Extend first part of kmesh to lenkmeshfp
!
!   copyright (c) 1984-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill a part of array kmesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer irefc, lengpc, ihelp(*), nboun

!     ihelp         i   Work array in which only position 1 and position 4 are
!                       of interest
!                       if ihelp(1) > 0, then array KMESH part t must be filled
!                       if ihelp(4) > 0
!     irefc         i   dataset reference number for reading (>0 formatted, &
!                          <0 unformatted)
!     lengpc        i   length of kmesh part c
!     nboun         i   number of nodal points on the boundary of an element
!                       when nelgrp = 1
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ih, ielgrp, jnodp, jboun, inpelm, &
              inboun, kelem, ishapet, n, ios, iwork(4*nelgrp)
      double precision dummy(1)

!     dummy   Dummy parameter
!     i       Counting variable
!     ielgrp  element group number
!     ih      Help variable to store a constant temporarily
!     inboun  Number of points on boundary of element
!     inpelm  number of points per element
!     ios     IOSTAT variable, returns from reading or writing
!     ishapet Shape number read
!     iwork   work array of length nelgrp x 4
!     jboun
!     jnodp   ?
!     kelem   New KMESH part c with respect to the surface elements
!             All elements have the same number of nodes
!     n
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     190   Number of elements found is incorrect
!     192   ishape has incorrect value
!     193   the sum of the number of elements of the various standard elements
!           is not equal to number of elements
!     199   error in reading of dimension parameters
!     251   end of file found in reading of dimension parameters
!    1746   The number of nodes in the elements of element group is incorrect
!    1747   The number of internal nodes in the elements of element group
!           is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh021' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh021'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000


      allocate ( Topgrp(nelgrp), km(iseqkm)%kmeshf(nelgrp) )

      kelmf = nelgrp
      kmeshf => km(iseqkm)%kmeshf
      nextpnt = ihelp(2)

      if ( debug ) write(irefwr,1) 'kelmf, nelgrp', kelmf, nelgrp
      if ( nelgrp==1 ) then

!     --- nelgrp = 1  simple situation

         kelgrp = npelm
         kmeshf(1) = ishape
         if ( nboun<npelm ) then

!        --- nboun < npelm  fill kmesh part b

            allocate ( km(iseqkm)%kmeshb(nelgrp) )
            kmeshb => km(iseqkm)%kmeshb
            kelmb = nelgrp

            kmeshb(1) = nboun
         end if
         lengpc = npelm*nelem
         Topgrp(1)%inpelm = npelm
         Topgrp(1)%inelem = nelem

      else

!     --- nelgrp > 1,  read information

         ih = 0
         do ielgrp = 1, nelgrp
            if ( irefc>=0 ) then
               read ( irefc, *, iostat = ios ) (iwork(ih+i), &
                      i = 1, 4)
            else
               read ( -irefc, iostat=ios ) (iwork(ih+i), &
                      i = 1, 4)
            end if
            if ( ios>0 ) then

!           --- ios > 0, Error in reading file

               call errint ( ios, 1 )
               call errsub (199, 1, 0, 0)
               go to 1000

            else if ( ios<0 ) then

!           --- ios > 0, End of file reached when reading file 10

               call errsub ( 251, 0, 0, 0)
               go to 1000

            end if
            ih = ih+4

         end do

!        --- check information
!            first inpelm

         jnodp = 1
         jboun = 1
         do ielgrp = 1, nelgrp
            inpelm = iwork(ielgrp*4-3)
            inboun = iwork(ielgrp*4)
            Topgrp(ielgrp)%inpelm = inpelm
            if ( inpelm/=npelm ) jnodp = 0
            if ( inboun/=nboun ) jboun = 0
            if ( inpelm<0 .or. inpelm>npelm ) then
              call errint ( ielgrp, 1 )
              call errint ( inpelm, 2 )
              call errint ( npelm, 3 )
              call errsub ( 1746, 3, 0, 0 )
            end if
         end do
         kelgrp = inpelm*nelgrp

         if ( jnodp==0 ) then

!        --- jnodp = 0   fill kmesh part a

            allocate ( km(iseqkm)%kmesha(nelgrp+1) )
            kmesha => km(iseqkm)%kmesha
            kelma = nelgrp+1
            kmesha(1) = 0
            kelgrp = 0
            do ielgrp = 1, nelgrp
               kelgrp = kelgrp+iwork(ielgrp*4-3)
               kmesha(1+ielgrp) = kelgrp
            end do
         end if

!        --- then nboun

         if ( jboun==0 ) then

!        --- jboun = 0   fill kmesh part b

           allocate ( km(iseqkm)%kmeshb(nelgrp) )
           kmeshb => km(iseqkm)%kmeshb
           kelmb = nelgrp

           do ielgrp = 1, nelgrp
              inboun = iwork(4*ielgrp)
              inpelm = iwork(ielgrp*4-3)
              kmeshb(ielgrp) = inboun
              if ( inboun<0 .or. inboun>inpelm ) then
                 call errint ( ielgrp, 1 )
                 call errint ( inboun, 2 )
                 call errint ( inpelm, 3 )
                 call errsub ( 1747, 3, 0, 0 )
              end if
           end do

         end if

!        --- ishape

         allocate ( km(iseqkm)%kmeshg(nelgrp) )
         kmeshg => km(iseqkm)%kmeshg

         lengpc = 0
         kelmg = nelgrp
         kelem = 0
         do ielgrp = 1, nelgrp
            ishapet = iwork(4*ielgrp-2)
            kmeshf(ielgrp) = ishapet
            if ( ishapet>100 .and. ishapet/=109 .and. &
                 ishapet/=134 ) then
               call errint ( ishapet, 1 )
               call errint ( 100, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 192, 3, 0, 0 )
            end if
            inelem = iwork(ielgrp*4-1)
            kmeshg(ielgrp) = inelem
            Topgrp(ielgrp)%inelem = inelem
            kelem = kelem+inelem
            inpelm = iwork(4*ielgrp-3)
            if ( inelem<0 .or. inelem>nelem ) then
               call errint ( inelem, 1 )
               call errint ( nelem, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 190, 3, 0, 0 )
            end if
            lengpc = lengpc+inelem*inpelm
         end do
         if ( kelem/=nelem ) then
            call errint ( kelem, 1 )
            call errint ( nelem, 2 )
            call errsub ( 193, 2, 0, 0 )
         end if
      end if
      if ( debug ) call prinin ( ihelp, 4, 'ihelp' )
      if ( ihelp(1)/=0 ) then

!     --- ihelp(1) = 1, hence KMESH part t must be filled

         n = nvolms
         if ( n==0 ) n = nsurfs
         if ( n==0 ) n = ncurvs
         allocate ( km(iseqkm)%kmesht(2,n) )
         kmesht => km(iseqkm)%kmesht
         kmesht = 0
         kelmt = 2*n
         if ( debug ) write(irefwr,1) 'n, kelmt', n, kelmt
         km(iseqkm)%kmesh(36) = kelmt

      end if
      if ( ihelp(4)>0 ) then

!     --- ihelp(4)>0, hence KMESH part u must be filled

         allocate ( km(iseqkm)%kmeshu(5) )
         kmeshu => km(iseqkm)%kmeshu
         kelmu = 5

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh021'

      end if  ! ( debug )
      call erclos ( 'msh021' )

      end subroutine msh021
