      subroutine msh022 ( irefc, lengpc, ivers )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.8  date 26-09-2007 Replace part of body by subroutines
!        version  1.7  date 18-06-2005 Extra debug statements
!        version  1.6  date 28-02-2004 Debug statements
!
!   copyright (c) 1984-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read arrays kmeshc and coor from the input file
!     for example meshoutput
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer irefc, lengpc, ivers

!     irefc          i    dataset reference number for reading (>0 formatted, &
!                         <0 unformatted)
!     ivers          i    Version sequence number
!     lengpc         i    Length of KMESH part c
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ios, i, ielgrp, ikelmc, inpelm
      character (len=80) text

!     i              Counting variable
!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     ios            IOSTAT variable, returns from reading or writing
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHRDMESHCOOR  Read coordinates from file meshoutput
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     194   nodal point number read<0 or>npoint
!     197   error in reading of element records
!     198   end of file found when reading element records
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh022' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh022'
         write(irefwr,*) 'ndim, npoint ', ndim, npoint

      end if
      if ( ierror/=0 ) go to 1000
100   format(a)

!     --- Read coordinates

      call mshrdmeshcoor ( irefc, ndim, npoint, coor, ivers )

!     --- Read kmesh part c

      if ( irefc>=0 ) then

!     --- Formatted case

         ikelmc = 0
         if ( ivers>=7 ) then
            read (irefc,100,iostat=ios) text
            if ( debug ) write(irefwr,*) text(1:20)
            ikelmc = 0
            do ielgrp = 1, nelgrp
               inelem = Topgrp(ielgrp)%inelem
               inpelm = Topgrp(ielgrp)%inpelm
               if ( inelem>0 ) then
                  read(irefc,*,iostat = ios) &
                     (kmeshc(i+ikelmc), i = 1, inelem*inpelm)
                  if ( debug ) then
                     write(irefwr,210) ielgrp, &
                        (kmeshc(i+ikelmc),i=1,inpelm)
210                  format ( 'element groep ', i6, '  eerste element'/ &
                              (100i6) )
                     write(irefwr,220) &
                        (kmeshc(i+ikelmc+inpelm*(inelem-1)),i=1,inpelm)
220                  format ( 'laatste element'/ (100i6) )
                  end if ! ( debug ) then
                  ikelmc = ikelmc+inpelm*inelem
               end if
            end do
         else
            read(irefc,*,iostat = ios) (kmeshc(i), i = 1, lengpc)
         end if
         if ( ios<0 ) then
            call errsub ( 198, 0, 0, 0 )
         else if ( ios>0 ) then
            call errint ( ios, 1 )
            call errsub ( 197, 1, 0, 0 )
         end if

      else

!     --- Unformatted case

         if ( ivers>=7 ) then
            read (-irefc,iostat=ios) text
            if ( debug ) write(irefwr,*) text(1:10)
            ikelmc = 0
            do ielgrp = 1, nelgrp
               inelem = Topgrp(ielgrp)%inelem
               inpelm = Topgrp(ielgrp)%inpelm
               if ( inelem>0 ) then
                  read(-irefc,iostat = ios) &
                     (kmeshc(i+ikelmc), i = 1, inelem*inpelm)
                  if ( debug ) then
                     write(irefwr,210) ielgrp, &
                        (kmeshc(i+ikelmc),i=1,inpelm)
                     write(irefwr,220) &
                        (kmeshc(i+ikelmc+inpelm*(inelem-1)),i=1,inpelm)
                  end if ! ( debug ) then
                  ikelmc = ikelmc+inpelm*inelem
               end if
            end do
         else
            read(-irefc,iostat = ios) (kmeshc(i), i = 1, lengpc)
         end if
         if ( ios<0 ) then
            call errsub ( 198, 0, 0, 0 )
         else if ( ios>0 ) then
            call errint ( ios, 1 )
            call errsub ( 197, 1, 0, 0 )
         end if

      end if

      do i = 1, lengpc
         if ( kmeshc(i)<=0 .or. kmeshc(i)>npoint ) then
            call errint ( kmeshc(i), 1 )
            call errint ( npoint, 2 )
            call errsub ( 194, 2, 0, 0 )
         end if
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh022'

      end if
      call erclos ( 'msh022' )

      end
