      subroutine msh024 ( irefc, npoint, kmeshm, ncurvs, lenkmeshm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.4  date 11-04-2008 Correction kmeshm part 4
!        version  2.3  date 11-08-2004 ini060 --> ini060text
!        version  2.2  date 24-02-1996 Jump in case of error
!
!   copyright (c) 1984-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read kmesh part m
!     fill kmeshm part iv in iwork
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     fill
!     mesh
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer irefc, npoint, lenkmeshm, kmeshm(lenkmeshm), ncurvs

!     irefc    i   dataset reference number for reading (>0 formatted, &
!                     <0 unformatted)
!     kmeshm   o   KMESH part m, contains nodal point numbers of the curves
!     lenkmeshm o   length of kmesh part m
!     ncurvs   i   number of curves
!     npoint   i   number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ninner, istar3, ios, incurv, icurv, i, l, iinner, lengav
      integer :: iwork(ncurvs)

!     i       Counting variable
!     icurv   Curve number
!     iinner  ?
!     incurv  ?
!     ios     IOSTAT variable, returns from reading or writing
!     istar3  Starting address of KMESHM part 3 in KMESH part M
!     iwork   work array of length ncurvs
!     l       Summation variable
!     lengav  Maximal available space for array KMESHM
!     ninner  Number of inner curves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     255    nodal point number in curve has incorrect value
!     256    error found in reading of nodal point numbers of curves.
!     257    end of file found in reading of nodal point numbers of curves
!    1381    Too few space available in array KMESHM
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh024' )
      ninner = 0
      lengav = lenkmeshm
      kmeshm(2) = 6+ncurvs
      kmeshm(5) = 1
      istar3 = 5+ncurvs
      do icurv = 1, ncurvs
         if ( irefc>=0 ) then
            read ( irefc, *, iostat=ios ) incurv, iinner
         else
            read(-irefc, iostat=ios ) incurv, iinner
         end if
         if ( ios<0 ) then
            go to 900
         else if ( ios>0 ) then
            go to 901
         end if
         if ( iinner<=0 ) ninner = ninner+1  ! inner curve of composite curve
         iwork(icurv) = iinner
         if ( istar3+incurv>lengav ) then

!        --- Error 1381: Not enough space available

            call errint ( istar3+incurv, 1 )
            call errint ( lengav, 2 )
            call errsub ( 1381, 2, 0, 0 )
            go to 1000

         end if

         if ( irefc>=0 ) then
            read(irefc, *, iostat=ios ) (kmeshm(istar3+l), &
                 l = 1, incurv)
         else
            read(-irefc, iostat=ios ) (kmeshm(istar3+l), &
                 l = 1, incurv)
         end if
         if ( ios<0 ) then
            go to 900
         else if ( ios>0 ) then
            go to 901
         end if

         do l = 1, incurv
            if ( kmeshm(istar3+l)>npoint ) then

               call errint ( icurv, 1 )
               call errint ( kmeshm(istar3+l), 2 )
               call errint ( npoint, 3 )
               call errsub ( 255, 3, 0, 0 )
               go to 1000
            end if
         end do
         istar3 = istar3+incurv
         kmeshm(5+icurv) = kmeshm(4+icurv)+incurv

      end do ! icurv = 1, ncurvs

      kmeshm(1) = ninner
      kmeshm(3) = istar3+1
      kmeshm(4) = ninner
      if ( istar3+ninner>lengav ) then

!     --- Error 1381: Not enough space available

         call errint ( istar3+ninner, 1 )
         call errint ( lengav, 2 )
         call errsub ( 1381, 2, 0, 0 )
         go to 1000

      end if
      lenkmeshm = istar3
      if ( ninner>0 ) then

!     --- ninner>0   fill part 4

         istar3 = istar3+1
         do i = 1, ncurvs

            if (iwork(i)==0 ) then

!           --- iwork(i) = 0   inner curve

               kmeshm(istar3) = i
               istar3 = istar3+1

            else if (iwork(i)<0 ) then

!           --- iwork(i) = -1   composite curve

               kmeshm(istar3) = -i
               istar3 = istar3+1

            end if  ! (iwork(i)/=1 )

         end do
         lenkmeshm = istar3

      end if

      go to 1000

!     --- read errors:

900   call errint ( ios, 1 )
      call errsub ( 256, 1, 0, 0 )
      go to 1000
901   call errsub ( 257, 0, 0, 0 )
      go to 1000
1000  call erclos ( 'msh024' )
      end
