      subroutine mshcopy ( kmesh1, kmesh2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 25-04-2012 Fortran 90
!        version  1.4  date 20-09-2006 Only copy l and m if existing
!        version  1.3  date 23-05-2005 Fortran 90
!
!   copyright (c) 1999-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy the essential parts of array kmesh1 into kmesh2
!     Only those parts needed for subroutine intmsh are copied
!     In 2D it concerns the parts c, i, k, m, iinput/rinput
!     In 3d also kmeshn
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     copy
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh1, kmesh2

!     kmesh1     i    KMESH array to be copied
!     kmesh2    i/o   Copy of array KMESH
!                     All positions must have been initialized before
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqkm1, iseqkm2

!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCOP1        Copy part of kmesh1 to kmesh2 (a,b,f,g)
!     MSHCOP2        Copy part of kmesh1 to kmesh2 (coor, c,k,l,m,n)
!     MSHCOP3        Copy part of kmesh1 to kmesh2 (q,r,s,iinput,rinput,
!                    kmesht, kmeshquant)
!     SEPCRMESHINFO  Create permanent space for mesh
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Check length of array kmesh2
!    Save memory management positions of old arrays of kmesh2
!    Copy parts of array kmesh
!    Copy kmesh part i (co-ordinates)
!    Copy kmesh part c (topology)
!    Copy kmesh part k (levels)
!    Copy kmesh part m (curve description)
!    Copy kmesh part n (surface description)
!    Copy kmesh parts q, r and s (description of curves and so on)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcopy' )

      call sepfillkmesh ( kmesh1 )
      iseqkm1 = iseqkm

      call sepcrmeshinfo ( kmesh2 )
      iseqkm2 = iseqkm

      km(iseqkm2)%kmesh(4:14) = km(iseqkm1)%kmesh(4:14)
      km(iseqkm2)%kmesh(42) = km(iseqkm1)%kmesh(42)
      km(iseqkm2)%kmesh(44) = km(iseqkm1)%kmesh(44)
      km(iseqkm2)%kmesh(64) = km(iseqkm1)%kmesh(64)

      call mshcop1 ( iseqkm1, iseqkm2 )

!     --- Copy coordinates, kmeshc, kmeshk, kmeshl, kmeshm, kmeshn

      call mshcop2 ( iseqkm1, iseqkm2 )

!     --- Copy kmeshq, kmeshr, kmeshs iinput, rinput, kmesht

      call mshcop3 ( iseqkm1, iseqkm2 )

      if ( kelmag>0 ) then

!     --- Copy volume_surfs

         allocate ( km(iseqkm2)%volume_surfs(size(volume_surfs)) )
         km(iseqkm2)%volume_surfs = volume_surfs

      end if  ! ( kelmag>0 )

      if ( kelmak>0 ) then

!     --- Copy curveusrp

         allocate ( km(iseqkm2)%curveusrp(size(curveusrp)) )
         km(iseqkm2)%curveusrp = curveusrp

      end if  ! ( kelmak>0 )

      if ( associated(km(iseqkm1)%intprop) ) then
         allocate ( km(iseqkm2)%intprop(size(km(iseqkm1)%intprop,1),nelgrp) )
         km(iseqkm2)%intprop = km(iseqkm1)%intprop
      end if  ! ( associated(km(iseqkm1)%intprop) )
      if ( associated(km(iseqkm1)%realprop) ) then
         allocate ( km(iseqkm2)%realprop(size(km(iseqkm1)%realprop,1),nelgrp) )
         km(iseqkm2)%realprop = km(iseqkm1)%realprop
      end if  ! ( associated(km(iseqkm1)%realprop) )

      call erclos ( 'mshcopy' )

      end
