      subroutine mshsortel3d ( lenkmeshac )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 22-08-2012 Remove iwork form parameter list
!        version  1.2  date 29-02-2012 Use subroutine prflgridnod3d
!        version  1.1  date 16-03-2001 Allow nsplit=0
!
!   copyright (c) 2000-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmesh part ac with information of in which part of the
!     region an element is positioned
!     Three-dimensional grid
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     sort
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: lenkmeshac

!     lenkmeshac     o    Length of array kmeshac
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart, nx, nz, ny, ifirstnode, ilastnode, &
              isplit, nnodes
      integer, allocatable :: iwork(:)
      double precision x, y, z, xmin, xmax, ymin, ymax, zmin, zmax

!     i              Counting variable
!     ifirstnode     First node in a part
!     ilastnode      Last node in a part
!     isplit         Sequence number of discontinuous part
!     istart         First position used at a part
!     iwork          integer work array to store nodes in a set
!     nnodes         Number of nodes
!     nx             number of points in x direction
!     ny             number of points in y direction
!     nz             number of points in z direction
!     x              x co-ordinate of point
!     xmax           Maximum x-value
!     xmin           Minimum x-value
!     y              y co-ordinate of point
!     ymax           Maximum y-value
!     ymin           Minimum y-value
!     z              z co-ordinate of point
!     zmax           Maximum z-value
!     zmin           Minimum z-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRFLGRIDNOD3D  Fill information of a set of nodes in rectangular 3d grid
!     PRINKMESHAC    Print contents of kmesh part AC
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear pointer array
!     nsplit := kmesh(44)
!     For isplit := 1 (1) nsplit do
!        Compute xmin, xmax, ymin, ymax for region
!        Store all nodes of part into iwork(1,.)
!        nx := min(100,sqrt(number of nodes))
!        dx := (xmax-xmin)/dx
!        For all nodes in part do
!           Compute i,j index for node and store in iwork(2-3,.)
!           Adapt Pointer array pos k = (i-1)*nx+j
!        Accumulate pointer array
!        For all nodes in part do
!           Fill node in second part of kmeshac
!           Adapt pointer array
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsortel3d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsortel3d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ifirstnode = 0
      istart = 5*nsplit

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear kmeshac part 1

      kmeshac(1:istart) = 0
      istart = istart+1

!     --- For isplit := 1 (1) nsplit do

      do isplit = 1, max(1,nsplit)

!     --- iwork(1,.) := all nodes in region isplit

         if ( nsplit<=1 ) then

!        --- The region consists of one part only

            nnodes = npoint
            do i = 1, nnodes
               iwork(i) = i
            end do

         else

!        --- The region consists of more parts

            ilastnode = kmeshk(nlevel+isplit)
            nnodes = ilastnode-ifirstnode
            if ( kelmj==0 ) then

!           --- No renumbering of nodes

               do i = 1, nnodes
                  iwork(i) = ifirstnode+i
               end do

            else

!           --- Nodes are renumbered

               do i = 1, nnodes
                  iwork(i) = kmeshj(ifirstnode+i)
               end do

            end if
            ifirstnode = ilastnode

         end if

!        --- Compute xmin, xmax, ymin, ymax, zmin and zmax

         xmin = rinfin
         ymin = rinfin
         zmin = rinfin
         xmax =-rinfin
         ymax =-rinfin
         zmax =-rinfin
         do i = 1, nnodes
            x = coor(1,iwork(i))
            y = coor(2,iwork(i))
            z = coor(3,iwork(i))
            xmin = min(xmin,x)
            ymin = min(ymin,y)
            zmin = min(zmin,z)
            xmax = max(xmax,x)
            ymax = max(ymax,y)
            zmax = max(zmax,z)
         end do
         rmeshac(1,isplit) = xmin
         rmeshac(2,isplit) = xmax
         rmeshac(3,isplit) = ymin
         rmeshac(4,isplit) = ymax
         rmeshac(5,isplit) = zmin
         rmeshac(6,isplit) = zmax

!        --- nx := min(100,sqrt(number of nodes))
!            dx := (xmax-xmin)/dx

         x = nnodes
         nx = max(1,nint(x**(1d0/3d0)))
         nx = min(50,nx)
         ny = nx
         nz = nx
         kmeshac((isplit-1)*5+1) = nnodes
         kmeshac((isplit-1)*5+2) = nx
         kmeshac((isplit-1)*5+3) = ny
         kmeshac((isplit-1)*5+4) = nz
         kmeshac((isplit-1)*5+5) = istart

!        --- Fill information of a set of nodes in rectangular 3d grid

         call prflgridnod3d ( kmeshac, nx, ny, nz, nnodes, coor, &
                              iwork, istart, xmin, xmax, ymin, ymax, &
                              zmin, zmax )
         istart = istart+nx*ny*nz+1+nnodes

      end do
      lenkmeshac = istart-1
      if ( debug ) then

!     --- debug is true, print kmesh part ac

         call prinkmeshac ( 3, nsplit, kmeshac, rmeshac )

      end if

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshsortel3d'

      end if  ! ( debug )
      call erclos ( 'mshsortel3d' )

      end
