      subroutine prcompidir ( isol, idir )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 06-11-2009
!
!   copyright (c) 2009-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute direction of axis (1=x, 2=y)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     direction
!     axis
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol, idir

!     idir           o    direction of axis (1=x, 2=y)
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, ielgrp

!     ielgrp         first element group number with type number # 0
!     istart         First position used at a part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer prcompidir1, inifirstelgrp, inistelgkmeshc

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIFIRSTELGRP  Find first element group that has a non-zero type number
!     INISTELGKMESHC Find starting address of element group in kmeshc
!     PRCOMPIDIR1    Return with direction of element
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find first element group with type number>0
!     Find starting address of element in kmeshc
!     Activate arrays
!     Compute starting addresses
!     Find direction
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcompidir' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcompidir'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      idir = 1            ! default direction
      ielgrp = 1          ! default element group
      if ( ndim==1 ) go to 1000

!     --- Find first element group with type number>0

      iprob = ks(isol)%iprob
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      ielgrp = inifirstelgrp()  ! element group
      if ( debug ) &
         write(irefwr,1) 'iprob, ielgrp', iprob, ielgrp

!     --- Find starting address of element in kmeshc (minus 1)

      istart = inistelgkmeshc ( ielgrp )
      if ( debug ) write(irefwr,1) 'istart', istart

!     --- Find direction

      idir = prcompidir1 ( coor, kmeshc(istart), ndim )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcompidir'

      end if  ! ( debug )
      call erclos ( 'prcompidir' )

      end subroutine prcompidir
