      function prcompidir1 ( coor, kmeshc, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-11-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with direction of element
!
! **********************************************************************
!
!                       KEYWORDS
!
!     direction
!     axis
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer prcompidir1, kmeshc(*), ndim
      double precision coor(ndim,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     ndim           i    Dimension of the space.
!     prcompidir1    o    direction of element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcompidir1' )
      prcompidir1 = 1
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcompidir1'
         call prinin ( kmeshc, 2, 'kmeshc' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, ndim
         if ( abs(coor(i,kmeshc(2))-coor(i,kmeshc(1)))>0d0 ) then
            prcompidir1 = i
            go to 1000
         end if  ! ( abs(coor(i,kmesh(2))-coor(i,kmesh(1)))>0d0 )
      end do  ! i = 1, ndim

1000  call erclos ( 'prcompidir1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'prcompidir1', prcompidir1
         write(irefwr,*) 'End prcompidir1'

      end if  ! ( debug )

      end
