      subroutine prcopsepvec6 ( func, usol, idegfd  )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy one degree of freedom of sepran vector in user vector
!     The degree of freedom is not available in all nodes
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     vector
!     sepran_vector
!     user_vector
!     solution_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: idegfd
      double precision, intent(in) :: usol(*)
      double precision, intent(out) :: func(*)

!     func           o    Contains the function values
!     idegfd         i    Degree of freedom sequence number
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, icount, ipos

!     i              Loop variable
!     icount         Counter
!     ipos           Position in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcopsepvec6' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcopsepvec6'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icount = 0

      do i = 1, npoint

         ipos = kprobp(i,idegfd)
         if ( ipos>0 ) then

!        --- Degree of freedom is available

            icount = icount+1
            func(icount) = usol(ipos)

         end if  ! ( kprobp(i,idegfd)>0 )

      end do  ! i = 1, npoint

1000  call erclos ( 'prcopsepvec6' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcopsepvec6'

      end if  ! ( debug )

      end subroutine prcopsepvec6
