      subroutine princons
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.6  date 06-12-2013 Extension with int vars
!        version  1.5  date 01-06-2013 Extension with strings
!        version  1.4  date 20-11-2006 Remove trailing spaces
!
!   copyright (c) 2000-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Print all constants, variables and vector names to the standard
!    output file
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     write
!     variable
!     constant
!     vector_name
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulecomio
      use sepmoduleplot
      use sepmoduletexts
      use sepmodulepost
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, lenname, ipos

!     i              Counting variable
!     ipos           Position in array
!     lenname        Length of name
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( ioutp>=0 ) then

!     --- ioutp>0, print

         if ( numints>0 ) then

!        --- numints>0, print integers

            write(irefwr,50) 'List of integer constants'
 50         format(//a/)
            write(irefwr,*) 'Seq.    name                        ', &
                            '         value'
            write(irefwr,100) &
              (i, chincons(i), incons(i), i= 1, numints )
100         format ( i3,':',5x,a,3x, i8 )

         end if
         if ( numreals>0 ) then

!        --- numreals>0, print reals

            write(irefwr,50) 'List of real constants'
            write(irefwr,*) 'Seq.    name                        ', &
                            '         value'
            write(irefwr,110) &
              (i, chrlcons(i), rlcons(i), i= 1, numreals )
110         format ( i3,':',5x,a,3x, ss, 1pe13.5e2 )

         end if
         if ( numscals>0 ) then

!        --- numscals>0, print variables

            write(irefwr,50) 'List of variables'
            write(irefwr,*) 'Seq.    name                        ', &
                            '         value'
            write(irefwr,120) &
              (i, chscalars(i), scalars(i), i= 1, numscals )
120         format ( i3,':',5x,a,3x, ss, 1pe13.5e2 )

         end if
         if ( numvec>0 ) then

!        --- numsvec>0, print vector_names

            write(irefwr,50) 'List of vector names'
            write(irefwr,*) 'Seq.     name'
            do i = 1, numvec
               lenname = len_trim(names(i))
               write(irefwr,130) i, names(i)(1:lenname)
130            format ( i3,':',5x,a )
            end do  ! i = 1, numvec

         end if

         if ( numstrings>0 ) then

!        --- nums>0, print strings

            write(irefwr,50) 'List of strings'
            write(irefwr,140)
140         format( 'Seq.      name', 32x, 'contents' )
            do i = 1, numstrings
               ipos = instring(i)
               lenname = len_trim(texts(ipos))
               write(irefwr,150) i, chstrings(i), &
                                 texts(ipos)(1:lenname)
150            format ( i3,':',5x,a, 5x, a )
            end do  ! i = 1, numstrings

         end if  ! ( numstrings>0 )

         if ( numintscals>0 ) then

!        --- numintscals>0, print integer variables

            write(irefwr,50) 'List of integer variables'
            write(irefwr,*) 'Seq.    name                        ', &
                            '         value'
            write(irefwr,100) &
              (i, chscalars(maxcls+i), nint(scalars(maxcls+i)), &
               i= 1, numintscals )

         end if

         write(irefwr,200)
200      format(//)

      end if
      end
