      subroutine probmerge ( kprob1, kprob2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-02-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Merge arrays kprob1 and kprob2 into a new array kprob1
!
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kprob1, kprob2

!     kprob1        i/o   Standard SEPRAN array containing information of the
!                         problems
!                         At input the original kprob, at output extended
!                         with kprob2
!     kprob2         i    Contains problem description of new problem
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lastkprob, lastkprob2, iseqkp1, iseqkp2, i, leng

!     i              Counting variable
!     iseqkp1        Sequence number of kprob1
!     iseqkp2        Sequence number of kprob2
!     lastkprob      Last position in array kprob1
!     lastkprob2     Last position in array kprob2
!     leng           Help variable to compute the length of some arrays
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPRMPROBINFO  Remove array with sequence number kproblc from kp
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probmerge' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probmerge'
         write(irefwr,1) 'kprob1, kprob2', kprob1, kprob2
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkp1 = kprob1
      iseqkp2 = kprob2

      kp(iseqkp1)%nprob = kp(iseqkp1)%nprob+1  ! raise nprob
      nprob = kp(iseqkp1)%nprob ! number of problems
      if ( debug ) write(irefwr,1) 'nprob', nprob

      allocate ( kp(iseqkp1)%kprobsub(nprob)%kprobloc(lenkprobfp) )
      kp(iseqkp1)%kprobsub(nprob)%kprobloc = &
         kp(iseqkp2)%kprobsub(1)%kprobloc
      kprobloc =>  kp(iseqkp2)%kprobsub(1)%kprobloc

      length = size(kp(iseqkp2)%kprobsub(1)%iinputloc)
      allocate ( kp(iseqkp1)%kprobsub(nprob)%iinputloc(length) )
      kp(iseqkp1)%kprobsub(nprob)%iinputloc = &
         kp(iseqkp2)%kprobsub(1)%iinputloc

      length = size(kp(iseqkp2)%kprobsub(1)%intarprb)
      allocate ( kp(iseqkp1)%kprobsub(nprob)%intarprb(length) )
      kp(iseqkp1)%kprobsub(nprob)%intarprb = &
         kp(iseqkp2)%kprobsub(1)%intarprb

      if ( kp(iseqkp2)%kprobsub(1)%kprobloc(14)>0 ) then

!     --- kproba

         length = size(kp(iseqkp2)%kprobsub(1)%kproba)
         allocate ( kp(iseqkp1)%kprobsub(nprob)%kproba(length) )
         indpra = length
         kprobloc(14) = indpra

         do i = 1, length

            if ( associated(kp(iseqkp2)%kprobsub(1)%kproba(i)%arr) ) then
               leng = size(kp(iseqkp2)%kprobsub(1)%kproba(i)%arr)
               allocate ( kp(iseqkp1)%kprobsub(nprob)%kproba(i)%arr(leng) )
               kp(iseqkp1)%kprobsub(nprob)%kproba(i)%arr = &
                  kp(iseqkp2)%kprobsub(1)%kproba(i)%arr
            else
               nullify ( kp(iseqkp1)%kprobsub(nprob)%kproba(i)%arr )
            end if  ! ( associated(kp(iseqkp1)%kprobsub(nprob)%kproba(i)%arr) )

         end do  ! i = 1, length

      end if  ! ( kp(iseqkp2)%kprobsub(1)%kprobloc(14)>0 )

      if ( kp(iseqkp2)%kprobsub(1)%kprobloc(15)>0 ) then

!     --- kprobb

         length = size(kp(iseqkp2)%kprobsub(1)%kprobb)
         allocate ( kp(iseqkp1)%kprobsub(nprob)%kprobb(length) )
         kp(iseqkp1)%kprobsub(nprob)%kprobb = kp(iseqkp2)%kprobsub(1)%kprobb
         kprobloc(15) = length
         indprb = length

      end if  ! ( kp(iseqkp2)%kprobsub(1)%kprobloc(15)>0 )

      length = size(kp(iseqkp2)%kprobsub(1)%kprobc)
      allocate ( kp(iseqkp1)%kprobsub(nprob)%kprobc(length) )
      kp(iseqkp1)%kprobsub(nprob)%kprobc = kp(iseqkp2)%kprobsub(1)%kprobc

      if ( kp(iseqkp2)%kprobsub(1)%kprobloc(22)>0 ) then

!     --- kprobi

         length = size(kp(iseqkp2)%kprobsub(1)%kprobi)
         allocate ( kp(iseqkp1)%kprobsub(nprob)%kprobi(length) )
         kp(iseqkp1)%kprobsub(nprob)%kprobi = kp(iseqkp2)%kprobsub(1)%kprobi
         kprobloc(22) = length
         indpri = length

      end if  ! ( kp(iseqkp2)%kprobsub(1)%kprobloc(22)>0 )

      if ( kp(iseqkp2)%kprobsub(1)%kprobloc(19)>0 ) then

!     --- kprobf

         length = size(kp(iseqkp2)%kprobsub(1)%kprobf)
         allocate ( kp(iseqkp1)%kprobsub(nprob)%kprobf(length) )
         kp(iseqkp1)%kprobsub(nprob)%kprobf = kp(iseqkp2)%kprobsub(1)%kprobf
         kprobloc(19) = length
         indprf = length

      end if  ! ( kp(iseqkp2)%kprobsub(1)%kprobloc(19)>0 )

      if ( kp(iseqkp2)%kprobsub(1)%kprobloc(20)>0 ) then

!     --- kprobf

         length = size(kp(iseqkp2)%kprobsub(1)%kprobg,2)
         allocate ( kp(iseqkp1)%kprobsub(nprob)%kprobg(2,length) )
         kp(iseqkp1)%kprobsub(nprob)%kprobg = kp(iseqkp2)%kprobsub(1)%kprobg
         kprobloc(20) = length

      end if  ! ( kp(iseqkp2)%kprobsub(1)%kprobloc(20)>0 )

      if ( kp(iseqkp2)%kprobsub(1)%kprobloc(23)>0 ) then

!     --- kprobj

         length = size(kp(iseqkp2)%kprobsub(1)%kprobj,1)
         allocate ( kp(iseqkp1)%kprobsub(nprob)%kprobj(length,2) )
         kp(iseqkp1)%kprobsub(nprob)%kprobj = kp(iseqkp2)%kprobsub(1)%kprobj
         kprobj => kp(iseqkp1)%kprobsub(nprob)%kprobj
         kprobloc(23) = length

      end if  ! ( kp(iseqkp2)%kprobsub(1)%kprobloc(23)>0 )

      if ( kp(iseqkp2)%kprobsub(1)%kprobloc(35)>0 ) then

!     --- kprobp

         leng = size(kp(iseqkp2)%kprobsub(1)%kprobp,1)
         length = size(kp(iseqkp2)%kprobsub(1)%kprobp,2)
         allocate ( kp(iseqkp1)%kprobsub(nprob)%kprobp(leng,length) )
         kp(iseqkp1)%kprobsub(nprob)%kprobp= kp(iseqkp2)%kprobsub(1)%kprobp
         kprobp => kp(iseqkp1)%kprobsub(nprob)%kprobp
         kprobloc(35) = length

      end if  ! ( kp(iseqkp2)%kprobsub(1)%kprobloc(35)>0 )

      if ( kp(iseqkp2)%kprobsub(1)%kprobloc(37)>0 ) then

!     --- kprobr

         length = size(kp(iseqkp2)%kprobsub(1)%kprobr)
         allocate ( kp(iseqkp1)%kprobsub(nprob)%kprobr(length) )
         kp(iseqkp1)%kprobsub(nprob)%kprobr= kp(iseqkp2)%kprobsub(1)%kprobr
         kprobr => kp(iseqkp1)%kprobsub(nprob)%kprobr
         kprobloc(37) = length

      end if  ! ( kp(iseqkp2)%kprobsub(1)%kprobloc(37)>0 )

      if ( kp(iseqkp2)%kprobsub(1)%kprobloc(38)>0 ) then

!     --- kprobs

         length = size(kp(iseqkp2)%kprobsub(1)%kprobs)
         allocate ( kp(iseqkp1)%kprobsub(nprob)%kprobs(length) )
         kp(iseqkp1)%kprobsub(nprob)%kprobs= kp(iseqkp2)%kprobsub(1)%kprobs
         kprobs => kp(iseqkp1)%kprobsub(nprob)%kprobs
         kprobloc(38) = length
         length = size(kp(iseqkp2)%kprobsub(1)%rprobs,1)
         allocate ( kp(iseqkp1)%kprobsub(nprob)%rprobs(length,2) )
         kp(iseqkp1)%kprobsub(nprob)%rprobs= kp(iseqkp2)%kprobsub(1)%rprobs
         rprobs => kp(iseqkp1)%kprobsub(nprob)%rprobs

      end if  ! ( kp(iseqkp2)%kprobsub(1)%kprobloc(23)>0 )

      if ( kp(iseqkp2)%kprobsub(1)%kprobloc(50)>0 ) then

!     --- kprobh

         length = size(kp(iseqkp2)%kprobsub(1)%kprobh)
         allocate ( kp(iseqkp1)%kprobsub(nprob)%kprobh(length), &
                    kp(iseqkp1)%kprobsub(nprob)%kprobhinv(length) )
         kp(iseqkp1)%kprobsub(nprob)%kprobh = kp(iseqkp2)%kprobsub(1)%kprobh
         kprobh => kp(iseqkp1)%kprobsub(nprob)%kprobh
         kp(iseqkp1)%kprobsub(nprob)%kprobhinv = &
            kp(iseqkp2)%kprobsub(1)%kprobhinv
         kprobhinv => kp(iseqkp1)%kprobsub(nprob)%kprobhinv
         kprobloc(50) = length

      else
         nullify ( kp(iseqkp1)%kprobsub(nprob)%kprobh, &
                   kp(iseqkp1)%kprobsub(nprob)%kprobhinv )
      end if  ! ( kp(iseqkp2)%kprobsub(1)%kprobloc(50)>0 )

      if ( debug ) then
         write(irefwr,1) 'lastkprob, lastkprob2', lastkprob, lastkprob2
      end if  ! ( debug )
      call seprmprobinfo ( kprob2 )

      iseqkp = iseqkp1
      nprob = kp(iseqkp)%nprob
      kprobpart => kp(iseqkp)%kprobsub(nprob)
      kprobloc => kprobpart%kprobloc
      iprob = nprob
      kp(iseqkp)%kprobsub(nprob)%kprobloc(80) = 0
      kprobb => kp(iseqkp1)%kprobsub(nprob)%kprobb
      kprobc => kp(iseqkp1)%kprobsub(nprob)%kprobc
      kprobi => kp(iseqkp1)%kprobsub(nprob)%kprobi
      kprobf => kp(iseqkp1)%kprobsub(nprob)%kprobf

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End probmerge'

      end if  ! ( debug )
      call erclos ( 'probmerge' )

      end
