      subroutine procopred ( kprob1, kprob2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-05-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy some parts of kprob1 to kprob2
!     Only those parts needed for refine are copied
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kprob1, kprob2

!     kprob1     i    kprob array to be copied
!     kprob2    i/o   Copy of array kprob
!                     All positions must have been initialized before
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: maxstore=100
      integer i, indprfloc(maxprob), indprhloc(maxprob), icount, &
              indprploc(maxprob), indpreloc(maxstore), &
              kelgrp, ivec, iadres, iseqkp1, nprob1, indprflocict(maxprob)
      logical :: filled
      integer, pointer :: kprobloc1(:), kprobe1(:)

!     filled         If true kprob2 has been filled
!     i              Counting variable
!     iadres         Address in array
!     icount         Counter
!     indpreloc      Array to store sequence numbers of parts e
!     indprfloc      indicates if kprobfloc is filled (>0) or not (0)
!     indprflocict   Array to store sequence numbers of parts f
!     indprhloc      Array to store sequence numbers of parts h
!     indprploc      Array to store sequence numbers of parts p
!     iseqkp1        Sequence number of kprob1
!     ivec           Sequence number of vector of special structure
!     kelgrp         sum over all standard elements of number of points in
!                    standard elements
!     kprobe1        kprobe corresponding to kprob1
!     kprobloc1      kprobloc corresponding to kprob1
!     maxstore       Length of array indpre
!     nprob1         nprob corresponding to kprob1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     SEPPROCOPY1    Save parts of array kprob2
!     SEPPROCOPY2    Create kprob2 and copy parts of kprob1
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Check length of array kprob2
!    Save memory management positions of old arrays of kprob2
!    Copy parts of array kprob
!    Copy kprob part i (co-ordinates)
!    Copy kprob part c (topology)
!    Copy kprob part m (curve description)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'procopred' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from procopred'
         call ertrace
         write(irefwr,1) 'kprob1, kprob2', kprob1, kprob2
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check length of array kprob2

      iseqkp1 = kprob1
      nprob1 = kp(iseqkp1)%nprob

      filled = kprob2>0 .and. kprob2<=lastprob

      if ( debug ) then
         write(irefwr,1) 'iseqkp1, nprob1', iseqkp1, nprob1
         write(irefwr,*) 'filled ', filled
      end if  ! ( debug )

!     --- Save memory management positions of old arrays of kprob2

      if ( filled ) &
         call sepprocopy1 ( kprob2, indprfloc, indprhloc, &
                            indprploc, indpreloc, &
                            indprflocict, maxstore )

!     --- Create kprob2

      call  sepprocopy2 ( kprob2, nprob1, iseqkp1 )

      if ( .not. filled ) then
         do iprob = 1, nprob1
            kprobpart => kp(iseqkp)%kprobsub(iprob)
            kprobloc => kprobpart%kprobloc
            kprobloc(19:21) = 0
            kprobloc(46) = 0
            kprobloc(47) = 0
            kprobloc(50) = 0
            ndtvec = kprobloc(6)
            indpre = kprobloc(18)
            kprobe2 => kprobpart%kprobe2
            kprobe3 => kprobpart%kprobe3
            do ivec = 1, ndtvec
               if ( kprobe2(ivec)>0 ) &
                  deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobe3(ivec)%arr )
            end do
         end do
      end if
      do iprob = 1, nprob1
         kprobpart => kp(iseqkp)%kprobsub(iprob)
         kprobloc => kprobpart%kprobloc
         kprobloc(20) = 0
         kprobloc(22) = 0
         kprobloc(24:28) = 0
         kprobloc(36:37) = 0
         kprobloc(41:42) = 0
         kprobloc(50) = 0
         kprobloc(53) = 0
         kprobloc(55:58) = 0
         kprobloc(80) = 0
      end do

      do iprob = 1, nprob1
         kprobpart => kp(iseqkp)%kprobsub(iprob)
         kprobloc => kprobpart%kprobloc
         kprobloc1 => kp(iseqkp1)%kprobsub(iprob)%kprobloc

         if ( kprobloc1(18)>0 ) then

!        --- Array kprob part e has been filled
!            Check if there are parts stored in ibuffr and copy those parts

            ndtvec = kprobloc1(6)
            indpre = kprobloc1(18)
            kprobe3 => kprobpart%kprobe3
            kprobe2 => kprobpart%kprobe2
            kprobe1 => kp(iseqkp1)%kprobsub(iprob)%kprobe2
            do ivec = 1, ndtvec
               if ( kprobe2(ivec)>0 ) then

!              --- part is stored in ibuffr, copy

                  if ( associated(kprobe3(ivec)%arr) ) &
                     deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobe3(ivec)%arr )
                  length = size(kp(iseqkp)%kprobsub(iprob)%kprobe3(ivec)%arr)
                  allocate ( kp(iseqkp)%kprobsub(iprob)%kprobe3(ivec)% &
                             arr(length)  )
                  kp(iseqkp)%kprobsub(iprob)%kprobe3(ivec)%arr = &
                     kp(iseqkp1)%kprobsub(iprob)%kprobe3(ivec)%arr

               end if

            end do

         end if

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'kprob1, kprob2', kprob1, kprob2
         write(irefwr,*) 'End procopred'

      end if  ! ( debug )
      call erclos ( 'procopred' )

      end subroutine procopred
