      subroutine prstorevec ( iseqcoor, iseqvec, ivec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 04-09-2016 New parameter list
!        version  2.0  date 12-02-2015 Extra parameter numvecmn
!        version  1.1  date 10-04-2013 Correction for series of vectors
!
!   copyright (c) 2012-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt vector sequence number and copy coor if necessary
!     in new vector
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     copy
!     coordinate
!     solution_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemnsub
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqcoor, iseqvec, ivec

!     iseqcoor       o    Sequence number of coordinates vector in isolut
!                         This number is equal to numvecmn+2
!                         The vector must be removed at the end of the calling
!                         subroutine
!     iseqvec        o    Sequence number of input vector in isolut
!     ivec           i    Sequence number of input vector in isolut
!                         as stored in input array, i.e. before transformation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal, inivectr

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     INIREPSCAL     Corrects integer value in case of scalars
!     INIVECTR       Return with sequence number of vector
!     PRCOPCOOR      Copy coordinates temporarily in solution vector stored
!                    in isolut
!     PRSTOREVEC1    Create new vector by computing a vector expression
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3162   Option not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prstorevec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prstorevec'
         write(irefwr,1) 'ivec, iseqcoor, maxvc1', &
                          ivec, iseqcoor, maxvc1
  1      format ( a, 1x, (10i10) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ivec>200000000 ) then

!     --- Vector corresponds to vector expression
!         Create new vector

         call prstorevec1 ( ivec, iseqvec )
         go to 1000

      end if  ! ( iseqvec>200000000 )

      iseqvec = inirepscal(ivec)
      if ( iseqvec>0 ) iseqvec = inivectr(iseqvec)

      if ( debug ) write(irefwr,1) 'iseqvec, maxvc1', iseqvec, maxvc1

      if ( iseqvec>=maxvc1+1 .and. iseqvec<=maxvc1+1+ndim ) then

!     --- iseqvec=maxvc1+1, hence coor

         if ( iseqcoor==0 ) &
            call prcopcoor ( iseqcoor, iseqvec )
            iseqvec = iseqcoor

      else if ( iseqvec>maxvc1+1 .and. iseqvec<=maxvc1+1+ndim ) then

!     --- iseqvec>maxvc1+1, hence x_coor, not yet implemented

         call errsub ( 3162, 0, 0, 0 )
         go to 1000

      else if ( abs(iseqvec)>=1000 .and. iseqvec<=5000 ) then

!     --- iseqvec>=1000, Copy coor in a vector

         if ( iseqcoor==0 ) &
            call prcopcoor ( iseqcoor, iseqvec )

         iseqvec = iseqcoor

      end if  ! ( abs(inpsep(9))>=1000 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iseqvec, iseqcoor', iseqvec, iseqcoor
         write(irefwr,*) 'End prstorevec'

      end if  ! ( debug )
      call erclos ( 'prstorevec' )

      end
