      function prvalnode ( isol, node )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 05-11-2009
!
!   copyright (c) 2009-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Get the value of the solution vector in a given node
!
! **********************************************************************
!
!                       KEYWORDS
!
!     value
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*), node
      double precision prvalnode

!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     node           i    Node number
!     prvalnode      o    value of the solution vector in node
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3037   More than one degree of freedom in point
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prvalnode' )
      debug = .false. .and. ioutp>=0
      prvalnode = 0d0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prvalnode'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate array isol and corresponding arrays

      call sepactsolbf2 ( isol )
      if ( ierror/=0 ) go to 1000
      usoltemp => ks(isol(1))%sol

      if ( indprfi==0 ) then

!     --- indprfi = 0, hence constant number of unknowns

         if ( nunkpi/=1 ) then

!        --- Error: more than one degree of freedom in node

            call errint ( node, 1 )
            call errsub ( 3037, 1, 0, 0 )
            go to 1000

         end if  ! ( nunkpi/=1 )
         prvalnode = usoltemp(node)

      else

!     --- indprfi>0, hence non-constant number of unknowns

         nunkpi = kprobfi(node+1)-kprobfi(node)

         if ( nunkpi/=1 ) then

!        --- Error: more than one degree of freedom in node

            call errint ( node, 1 )
            call errsub ( 3037, 1, 0, 0 )
            go to 1000

         end if  ! ( nunkpi/=1 )

         prvalnode = usoltemp(kprobfi(node)+1)

      end if  ! ( indprfi==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prvalnode'

      end if  ! ( debug )
      call erclos ( 'prvalnode' )

      end function prvalnode
