      subroutine readb4 ( numarr, isol )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  6.1  date 15-11-2012 Long integers
!        version  6.0  date 14-04-1999 Extra parameters ibuffr/buffer
!        version  5.4  date 24-07-1998 include commons
!
!   copyright (c) 1982-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   readb4 reads array of isol type from backing storage
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     backing_storage
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecpack
      use sepmodulekprob
      use sepmodulevecs
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numarr, isol

!     isol    o    Standard SEPRAN array containing information of the solution
!     numarr  i    Refers to solution vector or vector of special structure
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical compl
      integer index, leng, itypvc, ivec
      integer, allocatable :: iwork(:)

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     index          Index array to be used if 1000<=ind(number)<2000 and
!                    ichois = 2
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     iwork          integer work array to store records read
!     leng           Help variable to compute the length of some arrays
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     READFL         Read integer array from a direct access SEPRAN file
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPSCATTER     Distribute global vector over various nodes in parallel
!                    case. The global vector is stored on the host (node 1)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readb4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readb4'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      leng = iwork3 ( 2*numarr )
      index  = iwork3 ( 2*numarr-1 )

      allocate ( iwork(leng), stat = error )
      if ( error/=0 ) call eralloc ( error, leng, 'iwork' )
      if ( ierror/=0 ) go to 1000

      if ( parallel ) then

!     --- parallel case, use scatter to write solution to all nodes
!         Read all information from file 2

         if ( inodenr==1 ) then

!        --- Reading is only done at node 1 (host)

            call readfl ( index, 1, leng, 1, 2, iwork )
            if ( isol>0 ) then
               if ( associated(ks(isol)%solglob) ) deallocate(ks(isol)%solglob)
            end if  ! ( isol>0
            iprob = iwork(7)
            iseqkp = iwork(8)
            call sepcreasolbf1 ( isol, iwork(2), iwork(3), 'isol', iwork(1) )
            leng = (leng-10)/2
            allocate ( ks(isol)%solglob(leng) )
            usoltemp => ks(isol)%solglob
            call dcopy ( leng, iwork(11), 1, usoltemp, 1 )
         end if  ! ( inodenr==1 )

!        --- distribute solution over all nodes

         call sepscatter ( isol )

      else

!     --- serial case
!         Read all information from file 2

         call readfl ( index, 1, leng, 1, 2, iwork )

         iprob = iwork(7)
         iseqkp = iwork(8)
         call sepcreasolbf1 ( isol, iwork(2), iwork(3), 'isol', iwork(1) )
         if ( ierror/=0 ) go to 1000
         leng = leng-10
         call dcopy(length, iwork(11), 1, usoltemp, 1)

      end if  ! ( parallel )

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readb4'

      end if  ! ( debug )
      call erclos ( 'readb4' )

      end subroutine readb4
