      subroutine readconame ( iseq, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 20-04-2012 Indexed vectors
!        version  1.1  date 06-02-2012 Allow arrays
!        version  1.0  date 26-02-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read part vector names of input block CONSTANTS
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     constpstant
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmoduleread
      use sepmodulecpack
      use sepmoduleplot
      use sepmodulepost
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseq, iwork(*)

!     iseq           i    Sequence numberof vector in VECOLD/ISLOLD
!     iwork         i/o   integer work array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq1, numbvec, i

!     i              Counting variable
!     iseq1          Shifted sequence number
!     numbvec        Number of entries in vector array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer readarrindex

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READARRINDEX   Check if the name of a constpstant vector or varibale is
!                    followed by an index
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2068   sequence number of the vector read is incorrect
!    2143   Sequence number already in use
!    3100   Name of constpstant, variable or vector too long
!    3101   Index too large
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readconame' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readconame'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iseq==0 ) iseq = numvec+1
      iseq1 = iseq+maxints+maxreals+maxcls

      if ( iwork(iseq1)/=0 ) then

!     --- error, sequence number already in use

         call errint ( iseq, 1 )
         call errchr ( 'vector_names' , 1 )
         call errsub ( 2143, 1, 0, 1 )

      end if

      iwork(iseq1) = 1
      numvec = max(numvec,iseq)

      if ( numvec>=maxvc1 ) then

!     --- Too many vector_names

         call errint ( numvec, 1 )
         call errint ( maxvc1-1, 2 )
         call errsub ( 2068, 2, 0, 0 )
         call errcrd
         go to 1000

      end if  ! ( numvec>=maxvc1 )

!     --- store name into names

      names(iseq) = stringim(1:nchar)
      if ( debug ) then
         write(irefwr,1) 'lastitem', lastitem
         write(irefwr,1) 'iseq, nchar', iseq, nchar
         write(irefwr,*) 'string ', string(1:nchar)
      end if  ! ( debug )

!     --- Check if vectorname is indexed

      numbvec = max(1,readarrindex())
      numvec = max(numvec,iseq+numbvec-1)

      if ( numvec>=maxvc1 ) then

!     --- Too many vector_names

         call errint ( numvec, 1 )
         call errint ( maxvc1-1, 2 )
         call errsub ( 2068, 2, 0, 0 )
         call errcrd
         go to 1000

      end if  ! ( numvec>=maxvc1 )

      dimnames(iseq) = numbvec   ! dimension of vector name
      ncharnames(iseq) = nchar   ! number of characters of constpstant

      if ( numbvec>1 ) then

!     --- numbvec>1, hence indexed real

         if ( nchar>29 ) then

!        --- Too many characters

            call errchr ( string, 1 )
            call errsub ( 3100, 0, 0, 1 )
            go to 1000

         end if  ! ( nchar>29 )
         do i = 1, min(numbvec,9)
            write ( names(iseq+i-1)(1:nchar+3), 100 ) &
                string(1:nchar), i
100         format ( a,'(',i1,')' )
         end do  ! i = 1, min(numbvec,9)

         if ( numbvec>9 ) then

            if ( nchar>28 ) then

!           --- Too many characters

               call errchr ( string, 1 )
               call errsub ( 3100, 0, 0, 1 )
               go to 1000

            end if  ! ( nchar>29 )
            do i = 10, min(numbvec,99)
               write ( names(iseq+i-1)(1:nchar+4), 110 ) &
                string(1:nchar), i
110            format ( a, '(',i2,')' )
            end do  ! i = 10, min(numbvec,99)

         end if  ! ( numbvec>9 )

         if ( numbvec>=100 ) then

!        --- The upper bound for index is 99 at this moment

            call errint ( numbvec, 1 )
            call errint ( 99, 2 )
            call errsub ( 3101, 2, 0, 0 )
            go to 1000

         end if  ! ( numbvec>100 )

      end if  ! ( numbvec>1 )

      do i = 1, numbvec-1

         if ( iwork(iseq1+i)/=0 ) then

!        --- error, sequence number already in use

            call errint ( iseq+i, 1 )
            call errchr ( 'vector_names' , 1 )
            call errsub ( 2143, 1, 0, 1 )

         end if
         iwork(iseq1+i) = 1
         names(iseq+i)(1:nchar) = stringim(1:nchar)
         if ( i+1<10 ) then

!        --- array number between 2 and 9

            if ( nchar<30 ) &
               write( names(iseq+i)(nchar+1:nchar+3),200 ) i+1
200            format ( '(',i1,')' )
         else

!        --- array number between 10 and 99

            if ( nchar<29 ) &
               write( names(iseq+i)(nchar+1:nchar+4),210 ) i+1
210            format ( '(',i2,')' )

         end if  ! ( i+1<10 )

      end do  ! i = 1, numbvec-1

      if ( nchar==1 .and. string(1:1)=='V' ) then

!     --- Vector of name V found

         namevfound = .true.

      else if ( nchar>1 .and. string(1:1)=='V' .and. &
                stringim(2:2)>='0' .and. &
                stringim(2:2)<='9' ) then

!     --- Vector of name Vi found

         namevfound = .true.

      end if  ! ( nchar==1 .and. stringim(1:1)=='V' )

1000  call erclos ( 'readconame' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readconame'

      end if  ! ( debug )

      end
