      subroutine readm2 ( dbreal )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 02-04-2008 Fortran 90
!        version  2.0  date 02-01-2003 Complete revision
!        version  1.4  date 28-12-2000 Length of records extended to 240
!        version  1.3  date 10-02-1994 New comments
!        version  1.2  date 06-11-1992 Remove text in case of incorrect file
!        version  1.1  date 27-12-1991 New type of reading, extra message
!        version  1.0  date 03-11-1990
!
!   copyright (c) 1990-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    With the help of READM2 a real is read from the next line of
!    the environment file and stored in the parameter dbreal
!    subroutine READM2 is a help subroutine of READMD
! **********************************************************************
!
!                       KEYWORDS
!
!     environment_file
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision dbreal

!     dbreal   o  Real that has been read
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical found

!     found          If true already a possibility has been found
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     READCD  Read a record from the standard input file
! **********************************************************************
!
!                       I/O
!
!    Read one record of the environment file
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     No colon found
!     No number found after colon
! **********************************************************************
!
!                       PSEUDO CODE
!
!      Read next record (comments excluded):  READCD
!      Find colon
!      Read number after colon into dbreal
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Read record

      call readcd

!     --- Find colon

      found = .false.
      do while ( .not. found .and. lastitem<nitems-1 )
         lastitem = lastitem+1
         if ( typeitems(lastitem)==16 ) found =.true.
      end do
      if ( .not. found ) then

!     --- No colon found:

         write(*,100) cardimage(1)(1:lastcolumn(1))
100      format ( ' Line in the environment file does not contain a', &
                  ' colon(:). '//' Execution is terminated'// &
                  ' The line read is:'/1x,a )
         stop

      end if

      lastitem = lastitem+1

      if ( typeitems(lastitem)==1 .or. typeitems(lastitem)==2 ) then

!     --- number found after colon

         dbreal = realitems(lastitem)

      else

!     --- No number found after colon whereas number is expected

         write(*,110) cardimage(1)(1:lastcolumn(1))
110      format ( ' Line in the environment file does not contain a', &
                  ' number after the colon', &
                  ' (:). '//' Execution is terminated'// &
                  ' The line read is:'/1x,a )
         stop

      end if

      end
