      subroutine readnamevec
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.6  date 09-05-2014 Extra debug
!        version  1.5  date 22-12-2013 New definition of sequence numbers
!        version  1.4  date 20-04-2012 Indexed vectors
!
!   copyright (c) 2008-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read name of vector in common cpost5
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecons
      use sepmoduleread
      use sepmoduleplot
      use sepmoduletexts
      use sepmodulepost
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nch, iseq
      character (len=80) namecheck

!     i              General loop variable
!     iseq           sequence number resultig from readarrindex
!     namecheck      Help parameter to store a name
!     nch            Number of characters in a string
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer readarrindex

!     ASKUPC         Translate all letters to upper case
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     READARRINDEX   Check if the name of a constpstant vector or varibale is
!                    followed by an index
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readnamevec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readnamevec'
         write(irefwr,1) 'numvec, nchar', numvec, nchar
         write(irefwr,1) 'ityprd, lastitem/1', ityprd, lastitem
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
         call prinin ( ncharnames, numvec, 'ncharnames' )
         do i = 1, numvec
            write(irefwr,*) 'vecname i', i, names(i)
         end do  ! i = 1, numvec
  1      format ( a, 1x, (10i9) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, numvec
         nch = ncharnames(i)
         if ( nchar==nch ) then
            namecheck = names(i)(1:nchar)
            call askupc ( namecheck )
            if ( string(1:nchar)==namecheck(1:nchar) ) then

!           --- name of vector found

               if ( iprogram==3 ) then
                  realrd = i-1
               else
                  realrd = i
               end if
               index_rdarr(1) = nint(realrd)
               iseq = readarrindex()
               if ( debug ) write(irefwr,1) 'iseq', iseq
               if ( iseq>0 ) then

!              --- iseq>0, hence index vector with constpstant index

                  realrd = realrd+iseq-1d0  ! raise sequence numbe
                  index_rdarr(1) = nint(realrd)

               else if ( iseq<0 ) then

!              --- iseq<0, hence index vector with variable index

                  index_rdarr(3) = abs(iseq)
                  realrd = realrd-10000*iseq  ! raise sequence number
                  index_rdarr(1) = nint(realrd)

               else if ( dimnames(i)>1 ) then

!              --- iseq = 0, but dimension of array more than 1
!                  The whole set of arrays is required
!                  Raise realrd

                  realrd = realrd+5000d0
                  index_rdarr(1) = nint(realrd)

               end if  ! ( iseq>0 )

               ityprd = 2
               go to 1000

            end if
         end if
      end do  ! i = 1, numvec

      if ( string(1:nchar)=='COOR' ) then

!     --- Special vector coor found (indicate by -1000)

         realrd = -1000d0
         ityprd = 2
         index_rdarr(1) = maxvc1+1

      else if ( string(1:nchar)=='X_COOR' .or. string(1:nchar)=='X' ) then

!     --- Special vector x_coor found (indicate by -1001)

         realrd = -1001d0
         ityprd = 2
         index_rdarr(1) = maxvc1+2

      else if ( string(1:nchar)=='Y_COOR' .or. string(1:nchar)=='Y' ) then

!     --- Special vector y_coor found (indicate by -1002)

         realrd = -1002d0
         ityprd = 2
         index_rdarr(1) = maxvc1+3

      else if ( string(1:nchar)=='Z_COOR' .or. string(1:nchar)=='Z' ) then

!     --- Special vector z_coor found (indicate by -1003)

         realrd = -1003d0
         ityprd = 2
         index_rdarr(1) = maxvc1+4

      end if  ! ( string(1:nchar)=='COOR' )

1000  call erclos ( 'readnamevec' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'realrd, ityprd', nint(realrd), ityprd
         write(irefwr,*) 'End readnamevec'

      end if  ! ( debug )

      end
