      subroutine readrealcons
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-05-2008
!
!   copyright (c) 2008-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if last string read corresponds to constant
!     If so return with ityprd = 2 or 3 else 4
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     constant
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ispeclsav

!     ispeclsav      Help parameter to save the value of ispecl
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     READSTRCONST   Check if name read corresponds to one of the integer or
!                    real constants
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readrealcons' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readrealcons'
         write(irefwr,*) 'string ', string(1:nchar)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      lastitem = lastitembef  ! reset lastitem in order to reread string
                              ! This allows digits to be part of the string
      ispeclsav = ispecl
      ispecl = 9
      call asknxt
      ispecl = ispeclsav

      if ( debug ) then
         write(irefwr,1) 'ityprd', ityprd
         write(irefwr,*) 'string ', string(1:nchar)
      end if  ! ( debug )

      call readstrconst  ! constant

      if ( ityprd/=2 .and. ityprd/=3 ) then

!     --- Not a constant found, reset everything

         ityprd = 4
         lastitem = lastitembef+1

      end if  ! ( ityprd/=2 .and. ityprd/=3 )

1000  call erclos ( 'readrealcons' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ityprd', ityprd
         write(irefwr,*) 'End readrealcons'

      end if  ! ( debug )

      end
