      subroutine readrecconts
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.6  date 12-09-2013 recognize & and |
!        version  1.5  date 29-02-2008 Interpret ** as ^
!        version  1.4  date 06-02-2008 Extension with boolean operators
!
!   copyright (c) 2002-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret each item stored in cardimage and store information in
!     common cseprdnew
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     input
!     record
!     text
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer irecord, endcolum, itypechar
      logical numberfound

!     endcolum       Last column with information in record
!     irecord        Record number in cardimage
!     itypechar      Indicates type of character found
!     numberfound    if true a number has been found
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      logical asklet

!     ASKLET         Check if character is a capital letter
!     ASKUPC         Translate all letters to upper case
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     READNUMBER     read one number from the input file
!     READSTRING     Read a string from the input file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     nitems := 0
!     for all records irecord in cardimageage do
!       icolum := firstcolumn(irecord)
!       while ( there is still information in record ) do
!          Find next item in record and raise icolum and nitems
!          Store item
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readrecconts' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readrecconts'
         write(irefwr,1) 'nitems, icolum', nitems, icolum
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- nitems := 0

      nitems = 0

!     --- for all records irecord in cardimage do

      do irecord = 1, numrecstored

!     --- icolum := firstcolumn(irecord)

         icolum = firstcolumn(irecord)
         endcolum = lastcolumn(irecord)

!        --- Store record in cards with all letters translated in upper case

         cards = cardimage(irecord)
         call askupc ( cards(icolum:endcolum) )

!        --- while ( there is still information in record ) do

         do while ( icolum<=endcolum )

!        --- Find next item in record and raise icolum and nitems
!            Store item

            numberfound = .false.
            if ( icolum<endcolum ) then

!           --- number if .is followed by digit

                numberfound = index('.',cards(icolum:icolum))>0 &
                              .and. cards(icolum+1:icolum+1)>='0'.and. &
                                    cards(icolum+1:icolum+1)<='9'
            end if
            if ( cards(icolum:icolum)==' ' ) then

!           --- blank found, raise icolum

               icolum = icolum + 1

            else if ( asklet(cards(icolum:icolum)) ) then

!           --- Letter found, read keyword

               call readstring ( irecord, endcolum )
               if ( debug ) then
                  write(irefwr,*) 'string found'
                  write(irefwr,*) 'string ', string(1:nchar)
                  write(irefwr,1) 'nitems', nitems
               end if  ! ( debug )

            else if ( numberfound ) then

!           --- . + or - followed by digit found, read number

               call readnumber ( irecord, endcolum )
               if ( debug ) then
                  write(irefwr,*) 'number found'
                  write(irefwr,*) 'number ', realrd
                  write(irefwr,1) 'nitems', nitems
               end if  ! ( debug )

            else if ( cards(icolum:icolum)>='0' .and. &
                      cards(icolum:icolum)<='9' ) then

!           --- Digit found, read number

               call readnumber ( irecord, endcolum )
               if ( debug ) then
                  write(irefwr,*) 'number found/2'
                  write(irefwr,*) 'number ', realrd
                  write(irefwr,1) 'nitems', nitems
               end if  ! ( debug )

            else

!           --- other type of character found, detect and raise icolum

               itypechar = ichar(cards(icolum:icolum))

               if ( debug ) then
                  write(irefwr,*) 'character ', cards(icolum:icolum), &
                                  ' itypechar ', itypechar
               end if  ! ( debug )

               select case ( itypechar )

                  case(39:43)

!                 --- char = ' ( ) * or + &

                     itypechar = itypechar-35
                     if ( itypechar==7 .and. &
                          cards(icolum+1:icolum+1)=='*' ) then

!                    --- ** found, same as ^

                        icolum = icolum+1  ! skip next item
                        itypechar = 19     ! ^

                     end if  ! ( itypechar==7 .and. ... )

                  case(36, 37)

!                 --- char = $ or %

                     itypechar = itypechar-27

                  case(38)

!                 --- char = &

                     if ( cards(icolum+1:icolum+1)=='&' ) &
                        icolum = icolum+1  ! && found

                     itypechar = 30  ! & or && found

                  case(44)

!                 --- char = , &

                     itypechar = 26

                  case(45)

!                 --- char = - &

                     itypechar = 11

                  case(46)

!                 --- char = .

                     itypechar = 27

                  case(47)

!                 --- char = / &

                     if ( cards(icolum+1:icolum+1)=='=' ) then
                        itypechar = 24    !/=found
                        icolum = icolum+1 ! raise icolum
                     else
                        itypechar = 12
                     end if  ! ( cards(icolum+1:icolum+1)=='=' )

                  case(59)

!                 --- char = ;

                     itypechar = 29

                  case(61)

!                 --- char = = &

                     if ( cards(icolum+1:icolum+1)=='=' ) then
                        itypechar = 25    !==found
                        icolum = icolum+1 ! raise icolum
                     else
                        itypechar = 13
                     end if  ! ( cards(icolum+1:icolum+1)=='=' )

                  case(63)

!                 --- char = ?

                     itypechar = 28

                  case(92)

!                 --- char = \

                     itypechar = 14

                  case(126)

!                 --- char = ~

                     itypechar = 15

                  case(58)

!                 --- char = &

                     itypechar = 16

                  case(123)

!                 --- char = {

                     itypechar = 17

                  case(125)

!                 --- char = }

                     itypechar = 18

                  case(94)

!                 --- char = ^

                     itypechar = 19

                  case(62)

!                 --- char = >

                     if ( cards(icolum+1:icolum+1)=='=' ) then
                        itypechar = 23    !>=found
                        icolum = icolum+1 ! raise icolum
                     else
                        itypechar = 22    !>found
                     end if  ! ( cards(icolum+1:icolum+1)=='=' )

                  case(60)

!                 --- char = <

                     if ( cards(icolum+1:icolum+1)=='=' ) then
                        itypechar = 21    !<=found
                        icolum = icolum+1 ! raise icolum
                     else
                        itypechar = 20
                     end if  ! ( cards(icolum+1:icolum+1)=='=' )

                  case(124)

!                 --- char = |

                     if ( cards(icolum+1:icolum+1)=='|' ) &
                        icolum = icolum+1  ! || found
                     itypechar = 31  ! | or || found

                 case default

!                --- char = unknown

                     itypechar = 0

               end select  ! case ( itypechar )
               if ( debug ) write(irefwr,1) 'itypechar', itypechar

               if ( itypechar>0 ) then

!              --- Character found

                  nitems = nitems+1
                  typeitems(nitems) = itypechar
                  firstcolitems(nitems) = icolum+maxcolum*(irecord-1)
                  lastcolitems(nitems) = icolum+maxcolum*(irecord-1)
                  if ( debug ) write(irefwr,1) 'nitems', nitems

               end if
               icolum = icolum + 1

            end if  ! checking of next item
            if ( nitems>maxitems ) then

!           --- maxitems too small

               call errint ( nitems, 1 )
               call errint ( maxitems, 2 )
               call errchr ( 'typeitems', 1 )
               call errsub ( 550, 2, 0, 1 )

            end if

         end do  ! while ( icolum<=endcolum )

      end do  ! irecord = 1, numrecstored

1000  call erclos ( 'readrecconts' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nitems, icolum, irecord', &
                          nitems, icolum, irecord
         call prinin ( typeitems, nitems, 'typeitems' )
         write(irefwr,*) 'End readrecconts'

      end if

      end
