      subroutine sepbuilddalloc
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     De-allocate temporary arrays in for subroutine build
!
! **********************************************************************
!
!                       KEYWORDS
!
!     allocate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmoduleelm
      use sepmodulebuild
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     i              general loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepbuilddalloc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepbuilddalloc'
         call ertrace

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      error = 0

      IM => intmt(intmat)

      if ( debug ) write(irefwr,*) 'deallocate elemms'
      if ( associated(elemms) ) deallocate ( elemms, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'elemms' )
      if ( debug ) write(irefwr,*) 'deallocate coeffarr'
      if ( associated(coeffarr) ) deallocate ( coeffarr, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'coeffarr' )
      if ( debug ) write(irefwr,*) 'deallocate icoeffarr'
      if ( associated(icoeffarr) ) deallocate ( icoeffarr, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'icoeffarr' )
      if ( debug ) write(irefwr,*) 'deallocate index1sub'
      if ( associated(index1sub) ) deallocate ( index1sub, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'index1sub' )
      if ( debug ) write(irefwr,*) 'deallocate indexren'
      if ( associated(indexren) ) deallocate ( indexren, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'indexren' )
      if ( debug ) write(irefwr,*) 'deallocate index5'
      if ( associated(index5) ) deallocate ( index5, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'index5' )
      if ( debug ) write(irefwr,*) 'deallocate index2'
      if ( associated(index2) ) deallocate ( index2, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'index2' )
      if ( debug ) write(irefwr,*) 'deallocate index3'
      if ( associated(index3) ) deallocate ( index3, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'index3' )
      if ( debug ) write(irefwr,*) 'deallocate index4'
      if ( associated(index4) ) deallocate ( index4, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'index4' )
      if ( associated(index4tmp) ) deallocate ( index4tmp, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'index4tmp' )
      nullify(elemms, coeffarr, icoeffarr, index1sub, indexren)
      nullify(index5,index2,index3,index4)

      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,*) 'deallocate elemvc2d'
      if ( associated(elemvc2d) ) deallocate ( elemvc2d, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'elemvc2d' )
      if ( debug ) write(irefwr,*) 'deallocate elemmt2d'
      if ( associated(elemmt2d) ) deallocate ( elemmt2d, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'elemmt2d' )
      do i = 1, numtypesols
         nullify (indoldsols(i)%index3)
         nullify (indoldsols(i)%index4)
      end do  ! i = 1, numtypesols

      if ( IM%simple ) then

!     --- In case of block preconditioners for the Navier-Stokes elements
!         or shifted Laplace preconditioner, extra space
!         we need extra positions to store some extra matrices

         if ( debug ) write(irefwr,*) 'deallocate dpressmmat'
         if ( associated(E%dpressmmat) ) deallocate ( E%dpressmmat, stat=error )
         if ( error/=0 ) call erdealloc ( error, 'dpressmmat' )
         if ( debug ) write(irefwr,*) 'deallocate wpressmmat'
         if ( associated(E%wpressmmat) ) deallocate ( E%wpressmmat, stat=error )
         if ( error/=0 ) call erdealloc ( error, 'wpressmmat' )
         if ( debug ) write(irefwr,*) 'deallocate dvelmmat'
         if ( associated(E%dvelmmat) ) deallocate ( E%dvelmmat, stat=error )
         if ( error/=0 ) call erdealloc ( error, 'dvelmmat' )

      end if  ! ( IM%simple ... )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepbuilddalloc'

      end if  ! ( debug )
      call erclos ( 'sepbuilddalloc' )

      end subroutine sepbuilddalloc
