      subroutine sepchkstring ( notfound )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if right-hand side corresponds to a string
!     Old subroutine intxchkstring
! **********************************************************************
!
!                       KEYWORDS
!
!     vector_name
!     name
!     add
!     string
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical :: notfound

!     notfound       o    If true the right-hand side is not an expression
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseq, lastitemsav, i

!     i              General loop variable
!     iseq           Sequence number
!     lastitemsav    Help parameter to save the value of lastitem
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepgetstring

!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPGETSTRING   Get the sequence number corresponding to the string in
!                    the block constants with name stringname
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepchkstring' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepchkstring'
         write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd
         call prinin ( typeitems, nitems, 'typeitems' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      notfound = .true.
      lastitemsav = lastitem

!     --- First check if there are quotes on the right-hand side

      do i = 3, nitems

         if ( typeitems(i)==4 ) then

!        --- quote found, set notfound and return

            notfound = .false.
            exit

         end if  ! ( typeitems(i)==4 )

      end do  ! i = 3, nitems

      if ( debug ) write(irefwr,*) 'notfound ', notfound

      if ( notfound ) then

!     --- not yet a quote found, check if there are strings in the rhs

         do i = 3, nitems

            if ( typeitems(i)==3 ) then

!           --- string found, check if it is a known string

               lastitem = i-1

               if ( debug ) &
                  write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd

               call asknxt

               if ( debug ) then
                  write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd
                  write(irefwr,*) 'string ', string(1:nchar)
               end if  ! ( debug )

               iseq = sepgetstring ( string(1:nchar), chstrings, numstrings )

               if ( iseq>0 ) then

!              --- string found, set notfound and return

                  if ( debug ) write(irefwr,1) 'i, iseq', i, iseq
                  notfound = .false.
                  exit

               end if  ! ( iseq>0 )

            end if  ! ( typeitems(i)==3 )

         end do  ! i = 3, nitems

      end if  ! ( notfound )

      lastitem = lastitemsav

1000  call erclos ( 'sepchkstring' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'notfound ', notfound
         write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd
         write(irefwr,*) 'End sepchkstring'

      end if  ! ( debug )

      end subroutine sepchkstring
