      subroutine sepcopsepvec ( isol, idegfd, func )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy one degree of freedom of sepran vector in user vector
!     All nodes in the mesh are used
!     Old subroutine prcopsepvec
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     vector
!     sepran_vector
!     user_vector
!     solution_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: idegfd, isol
      double precision, intent(out) :: func(*)

!     func           o    Contains the function values
!     idegfd         i    Degree of freedom sequence number
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRCOPSEPVEC1   Copy one degree of freedom of sepran vector in user vector
!     PRCOPY2D2      Copy two-dimensional array of size ndim1 x npoint into
!                    two-dimensional array of size ndim2 x npoint.
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcopsepvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcopsepvec'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate array isol and related arrays

      call sepactsolbf2 ( isol )

!     --- Compute starting addresses

      usol => ks(isol)%sol

!     --- Copy usol

      if ( indprfi==0 .and. nunkpi==1 ) then

!     --- 1 unknown per point

         func(1:npoint) = usol(1:npoint)

      else if ( indprfi==0 ) then

!     --- more unknowns per point

         call prcopy2d2 ( npoint, usol, nunkpi, func, 1, idegfd )

      else

!     --- variable degrees of freedom per point

         call prcopsepvec1 ( func, usol, idegfd )

      end if  ! ( indprfi==0 .and. nunkpi==1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcopsepvec'

      end if  ! ( debug )
      call erclos ( 'sepcopsepvec' )

      end
