      subroutine sepcopydf ( ivec1, ivec2, kprob1, kprob2, idf1, idf2 )
! ======================================================================
!
!        programmer    Frans van de Vosse
!        version  1.4  date 29-05-2014 remove call to time subroutine
!        version  1.3  date 09-11-2012 Long length ibuffr
!        version  1.2  date 28-06-2006 iniget --> seppointer
!
!   copyright (c) 1998-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     sepcopydf copies the contents of one degree of freedom in array ivec1
!     belonging to kprob1 into another degree of freedom in array ivec2
!     belonging to kprob2
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivec1, ivec2, idf1, idf2
      integer, target :: kprob1, kprob2

!     idf1       i    degree of freedom in ivec1 to be copied
!     idf2       i    degree of freedom in ivec2 to be filled
!     ivec1      i    old array (must be an excisting SEPRAN array)
!     ivec2     i/o   new array (must be an excisting SEPRAN array)
!     kprob1     i    kprob of old array
!     kprob2     i    kprob of new array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer itypv1, itypv2

!     itypv1         Type number of vector 1
!     itypv2         Type number of vector 2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCPDF01       Copies the contents of one degree of freedom in array vec1
!                    belonging to kprob1 into another degree of freedom in
!                    array vec2
!                    belonging to kprob2
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPACTSOLBF1   compute pointers to degrees of freedom of vectors
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      13   array iold has not been filled.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcopydf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcopydf'
         call prinin ( ivec1, 5, 'ivec1' )
         call prinin ( ivec2, 5, 'ivec2' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      itypv1 = ks(ivec1)%typevec
      if ( itypv1/=110 .and. itypv1/=115 .and. itypv1/=116 ) then
         call errint ( itypv1, 1 )
         call errsub ( 13, 1, 0, 0 )
      end if
      itypv2 = ks(ivec2)%typevec
      if ( itypv2/=110 .and. itypv2/=115 .and. itypv2/=116 ) then
         call errint ( itypv2, 1 )
         call errsub ( 13, 1, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000

!     --- End tests

      kprob => kprob1
      call sepactsolbf1 ( ivec1 )
      kprobfo => kprobfi
      kprobho => kprobhi
      indprfo = indprfi
      nphyso = nphysi

      kprob => kprob2
      call sepactsolbf1 ( ivec2 )

      vectin => ks(ivec1)%sol
      vectin2 => ks(ivec2)%sol

      call ercpdf01 ( vectin, kprobfo, indprfo,  nphyso, idf1, &
                      vectin2, kprobfi, indprfi, nphysi, idf2, npoint )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcopydf'

      end if  ! ( debug )
      call erclos ( 'sepcopydf' )

      end
