      subroutine sepcreasol110 ( isol, namear, icompl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-10-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     General memory management subroutine which simplifies the computation
!     of the starting addresses of a new or existing SEPRAN solution array
!     Only for vectors of type 110
!     Old subroutine sepcreasolbf
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol, icompl
      character (len=*) namear

!     icompl         i    0 = real, 1 = complex
!     isol           o    sequence number of solution array
!     namear         i    Name of array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     PRINRL         Print 1d real vector
!     SEPCRSOLINFO   Create a new vector in ks
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcreasol110' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcreasol110'
         call ertrace
         write(irefwr,1) 'isol', isol
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Put ISOL into IBUFFR

      length = kprobloc(5)  ! nusol
      length  = length * (icompl+1)
      call sepcrsolinfo ( isol, length, 0  )

      ks(isol)%iseqkp = iseqkp
      ks(isol)%typevec = 110
      ks(isol)%jcart = 0
      ks(isol)%ivec = 0
      ks(isol)%nphys = kprobloc(33)
      ks(isol)%nusol = kprobloc(5)
      ks(isol)%iprob = iprob
      ks(isol)%compl = icompl
      usoltemp => ks(isol)%sol

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( usoltemp, size(usoltemp), 'usoltemp' )
         write(irefwr,*) 'End sepcreasol110'

      end if
      call erclos ( 'sepcreasol110' )

      end subroutine sepcreasol110
