      subroutine sepcreastruc ( number, itypvc, ivec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find the structure of the vector with exactly NUMBER unknowns per point
!     If this type does not exist an error message is given
!     Old subroutine inistrnew
! **********************************************************************
!
!                       KEYWORDS
!
!     find
!     structure
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer number, itypvc, ivec

!     itypvc        i/o   Indication of the type of solution vector
!                         If itypvc = 110 at input then the output vector may be
!                         a vector of type 110 or 115 else of type 115 only
!                         Output:
!                         110: solution vector
!                         115: vector of special structure
!     ivec           o    Sequence number of vector of special structure
!     number         i    The required number of unknowns per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PROBEXTEND     Extend problem definition with vectors of special structure
!                    of exactly number degrees of freedom
!     SEPCREASTRUC1  Find the structure of the vector with exactly
!                    NUMBER unknowns per point
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     If ( solution vector contains exactly NUMBER unknowns per point ) then
!        itypvc := 110
!     else
!        ivec := 0
!        For i := 1 (1) NDTVEC, while ( ivec=0 ) do
!           If ( vector of special structure i has exactly NUMBER unknowns
!                per point then )
!              ivec := i
!              itypvc := 115
!        If ( ivec=0 ) then
!           Error: no vector found
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcreastruc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcreastruc'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000
      call sepcreastruc1 ( itypvc, ivec, number )

      if ( debug ) &
         write(irefwr,1) 'itypvc, ivec, iprob', itypvc, ivec, iprob
      if ( itypvc==115 .and. ivec==0 ) then

!     --- No vector found

         call probextend ( number )
         ivec = kprobloc(6)
         iprob = 1

      end if  ! ( ivec==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcreastruc'

      end if
      call erclos ( 'sepcreastruc' )

      end
