      subroutine sepcrsolinfo ( ivectlc, lengrl, lengin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-04-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     create a new array in ks
!
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmoduleprobinf
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: ivectlc
      integer, intent(in) :: lengrl, lengin

!     ivectlc      i/o   Defines vector structure by integer sequence number
!                        At output its value is iseqsols
!     lengin        i    Length of integer part of vector
!     lengrl        i    Length of real part of vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i
      logical :: old, remove

!     i         Loop variable
!     old       if true it concerns an old vector
!     remove    if true the old contents must be removed
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3223   all arrays in use
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcrsolinfo' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcrsolinfo'
!         call ertrace
         write(irefwr,1) 'ivectlc, lastsol, maxsoluts, nextfreesol', &
                          ivectlc, lastsol, maxsoluts, nextfreesol
         write(irefwr,1) 'lengrl, lengin', lengrl, lengin
  1      format ( a, 1x, (10i7) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      old = ivectlc>0 .and. ivectlc<=lastsol
      if ( old ) old = solarr(ivectlc)==1
      remove = old

      if ( debug ) write(irefwr,*) 'old ', old
      if ( old .and. lengrl>0 ) old = size(ks(ivectlc)%sol)==lengrl
      if ( old .and. lengin>0 ) old = size(ks(ivectlc)%isol)==lengin
      if ( old ) go to 1000  ! No action necessary

      if ( debug ) write(irefwr,*) 'remove ', remove

      if ( remove ) call seprmsolinfo ( ivectlc )

      if ( nextfreesol<=maxsoluts ) then

!     --- nextfreesol in proper range

         iseqsols = nextfreesol
         do i = iseqsols+1, maxsoluts
            if ( solarr(i)==0 ) then
               nextfreesol = i
               exit
            end if  ! ( solarr(i)==0 )
         end do  ! i = iseqsols+1, maxsoluts
         lastsol = max(lastsol, iseqsols)

      else

!     --- no free entries left

         call errchr ( 'isol', 1 )
         call errsub ( 3223, 0, 0, 1 )
         go to 1000

      end if  ! ( lastsol<maxsoluts )

      ks(iseqsols)%iseqkp = iseqkp
      solarr(iseqsols) = 1
      lastsol = max(lastsol, iseqsols)

      nullify ( ks(iseqsols)%sol, ks(iseqsols)%isol, ks(iseqsols)%solglob )
      if ( lengrl>0 ) allocate ( ks(iseqsols)%sol(lengrl) )
      if ( lengin>0 ) allocate ( ks(iseqsols)%isol(lengin) )
      ks(iseqsols)%changed = .true.

      if ( debug ) write(irefwr,1) 'iseqsols', iseqsols

      ivectlc = iseqsols

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iseqsols, lastsol, ivectlc, nextfreesol', &
                          iseqsols, lastsol, ivectlc, nextfreesol
         write(irefwr,*) 'End sepcrsolinfo'

      end if  ! ( debug )
      call erclos ( 'sepcrsolinfo' )

      end subroutine sepcrsolinfo
