      subroutine sepfillkmeshm
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make relation between kmeshm and Curvesub
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iwork(0:ncurvs)
      integer :: nstar3, nstar4, iend, i, nnodes

!     i              curve number
!     iend           Last position used at a part
!     iwork          integer work array to mark inner curvs (1)
!     nnodes         Number of nodes in curve
!     nstar3         Starting address of KMESHM part 3 in KMESHM containing the
!                    node numbers of the curves
!     nstar4         Starting address of KMESHM part 4 in KMESHM containing the
!                    inner boundaries
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillkmeshm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillkmeshm'
         call ertrace
         write(irefwr,1) 'ncurvs', ncurvs
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ninner = kmeshm(1)
      nstar3 = kmeshm(2)
      nstar4 = kmeshm(3)

      if ( debug) &
         write(irefwr,1) 'ninner, nstar3, nstar4', ninner, nstar3, nstar4

      iwork = 0
      iwork(abs(kmeshm(nstar4:nstar4+ninner-1))) = 1

      do i = 1, ncurvs
         nnodes = kmeshm(i+5)-kmeshm(i+4)
         Curvesub(i)%nnodes = nnodes
         iend = nstar3+Curvesub(i)%nnodes-1
         Curvesub(i)%nodes => kmeshm(nstar3:iend)
         Curvesub(i)%inner_curve = iwork(i)==1
         nstar3 = nstar3+nnodes
      end do  ! i = 1, ncurvs

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillkmeshm'

      end if  ! ( debug )
      call erclos ( 'sepfillkmeshm' )

      end subroutine sepfillkmeshm
