      subroutine sepfillkprobpc
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-10-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill parameters wrt kprobpar in module sepmodulekprob
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecpack
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillkprobpc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillkprobpc'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ninterface = kprobloc(67)
      nusolglob = nrusol
      ipstkprobpar = kprobloc(65)

      nrusolmax = nrusol

      if ( ipstkprobpar>0 ) then

!     --- ipstkprobpar>0, hence parallel case

         kprobpar => kprobpart%kprobpar
         nusolglob = kprobpar(2)

         blockinfor => kprobpart%blockinfor
         kprobhdd => kprobpart%kprobhdd
         kprobhddinv => kprobpart%kprobhddinv
         kprobhglob => kprobpart%kprobhglob
         ninterfdeg => kprobpart%ninterfdeg
         nrusollocs => kprobpart%nrusollocs
         onnz => kprobpart%onnz
         nnz => kprobpart%nnz
         nrusolmax = nrusollocs(inodenr)

         if ( allocated(kprobhglobinv) .and. &
              size(kprobhglobinv)/=nusolglob) deallocate(kprobhglobinv)
         if ( .not. allocated(kprobhglobinv) ) then
            allocate ( kprobhglobinv(nusolglob), stat = error )
            if ( error/=0 )  &
               call eralloc ( error, nusolglob, 'kprobhglobinv' )
         end if  ! ( .not. allocated(kprobhglobinv) )

      end if  ! ( ipstkprobpar>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillkprobpc'

      end if  ! ( debug )
      call erclos ( 'sepfillkprobpc' )

      end subroutine sepfillkprobpc
