      subroutine sepfillnmeshf
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nmesh f containing elements corresponding to surfaces
!     Old subroutine topfillnmeshf
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     element
!     surface
!     face
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lowvolelm, melped, lnnmeshf, melpef, i

!     i              Counting variable
!     lnnmeshf       length of array nmeshf
!     lowvolelm      Element number of volume element with lowest number
!     melped         maximum number of elements connected to one edges
!     melpef         maximum number of elements connected with one face
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFA05        creates nmesh part f
!     SEPFILLNMESHA  Fill array nmesha
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Activate old arrays nmesha, nmeshb, nmeshc, kmeshd
!     Create temporary space for nmeshf
!     Fill nmesh part f (nmeshf) with actual data
!     Make nmeshf permanent
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillnmeshf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillnmeshf'
         write(irefwr,1) 'ncurvs, nsurfs, nvolms', ncurvs, nsurfs, nvolms
         write(irefwr,1) 'kelmt', kelmt
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nullify ( nmeshf )
      if ( associated(km(iseqkm)%nmeshf) ) deallocate(km(iseqkm)%nmeshf)

      if ( nvolms==0 .or. kelmt==0 ) then

!     --- No volume elements, set lowvolelm to 1

         lowvolelm = 1

      else

!     --- There are volume elements, compute lowvolelm

         lowvolelm = km(iseqkm)%kmesh(9)+1
         do i = 1, nvolms
            lowvolelm = min(lowvolelm,kmesht(1,i))
            lowvolelm = min(lowvolelm,kmesht(2,i))
         end do

      end if

!     --- Check if nmesha/b exist, if not create

      if ( km(iseqkm)%nmesh(31)==0 ) call sepfillnmesha

!     --- Create temporary space for nmeshf (lnnmeshf) and nmeshf (nfaces)

      melped = km(iseqkm)%nmesh(15)
      lnnmeshf = (melped+2)*nfaces

      allocate ( nmeshf(lnnmeshf), stat = error )
      if ( error/=0 ) call eralloc ( error, lnnmeshf, 'nmeshf' )
      if ( ierror/=0 ) go to 1000

!     --- Fill nmesh part f (nmeshf) with actual data

      call sepfa05 ( melpef, lnnmeshf, lowvolelm )

!     --- Make nmeshf permanent

      allocate ( km(iseqkm)%nmeshf(lnnmeshf) )
      km(iseqkm)%nmeshf = nmeshf(1:lnnmeshf)
      km(iseqkm)%nmesh(36) = lnnmeshf
      km(iseqkm)%nmesh(16) = melpef

      deallocate ( nmeshf, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nmeshf' )

      nmeshf => km(iseqkm)%nmeshf

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillnmeshf'

      end if  ! ( debug )
      call erclos ( 'sepfillnmeshf' )

      end subroutine sepfillnmeshf
