      subroutine sepfillnmeshg
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nmesh g containing node sequence numbers of faces
!     Old subroutine topfillnmeshg
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     element
!     node
!     face
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: mndpel, mndpfa, lpnmpg

!     lpnmpg         length of array nmeshg
!     mndpel         maximum number of nodes in one element
!     mndpfa         maximum number of nodes in one face
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINNMESHX     Print the contents of an integer array with structure of
!                    sub array of nmesh
!     SEPFA06        creates the faces part 1 pointers and initialization
!     SEPFILLNMESHA  Fill array nmesha
!     SEPFILLNMESHE  Fill array nmeshe
!     SEPFILLNMESHF  Fill array nmeshf
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   Estimated length of array too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Activate old arrays nmesha, nmeshe, nmeshf
!     Create temporary space for nmeshg
!     Fill nmesh part g (nmeshg) with actual data
!     Make nmeshg permanent
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillnmeshg' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillnmeshg'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      nullify ( nmeshg )
      if ( associated(km(iseqkm)%nmeshg) ) deallocate(km(iseqkm)%nmeshg)

!     --- if nmesha, nmeshb do not exist: create

      if ( nmesh(31)==0 ) call sepfillnmesha

!     --- if nmeshe does not exist: create

      if ( nmesh(35)==0 ) call sepfillnmeshe

!     --- if nmeshf does not exist: create

      if ( nmesh(36)==0 ) call sepfillnmeshf

      if ( debug ) then

!     --- debug information

         write(irefwr,*) 'nmesha: face sequence numbers of elements'
         call prinnmeshx ( nelem, nmesha )
         write(irefwr,*) 'nmeshe: node sequence numbers of elements'
         call prinnmeshx ( nelem, nmeshe )
         write(irefwr,*) 'nmeshf: elements connected to faces'
         call prinnmeshx ( nfaces, nmeshf )

      end if

!     --- Create temporary space for nmeshg

      mndpel = npelm
      mndpfa = 0
      lpnmpg = (mndpel+2)*(nfaces)

      allocate ( nmeshg(lpnmpg), stat = error )
      if ( error/=0 ) call eralloc ( error, lpnmpg, 'nmeshg' )
      if ( ierror/=0 ) go to 1000

!     --- Fill nmesh part g (nmeshg) with actual data

      call sepfa06 ( lpnmpg, mndpfa, mndpel )
      if ( ierror/=0 ) go to 1000

      if ( lpnmpg>(mndpel+2)*nfaces ) then

!     --- Estimated length too short

         call errint ( (mndpel+2)*nfaces, 1 )
         call errint ( lpnmpg, 2 )
         call errchr ( 'nmeshg', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

!     --- Make nmeshg permanent

      allocate ( km(iseqkm)%nmeshg(lpnmpg) )
      km(iseqkm)%nmeshg = nmeshg(1:lpnmpg)
      nmesh(37) = lpnmpg
      nmesh(24) = mndpfa

      deallocate ( nmeshg, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nmeshg' )

      nmeshg => km(iseqkm)%nmeshg

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillnmeshg'

      end if  ! ( debug )
      call erclos ( 'sepfillnmeshg' )

      end subroutine sepfillnmeshg
