      subroutine sepfillnmeshh
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create nmesh part h
!     Old subroutine iniactnmeshh
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lpnmph, melpnd, melped

!     lpnmph         length of array nmeshh
!     melped         maximum number of elements connected to one edges
!     melpnd         maximum number of elements connected to a node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INICOMPMELPND  Compute melpnd, i.e. maximimum number of elements per node
!     PRINNMESHX     Print the contents of an integer array with structure of
!                    sub array of nmesh
!     SEPEDG05       creates the nmesh part h
!     SEPFILLKMESHAA Fill kmesh part aa
!     SEPFILLKMESHD  Fill kmesh part d
!     SEPFILLNMESHC  Fill nmesh part c
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     create kmesh part d if it does not exist
!     create nmeshc, nmeshd if it does not exist
!     Create temporary space for nmeshh
!     Compute maximum number of elements per node
!     Compute starting addresses
!     Fill nmesh part h (nmeshh) with actual data
!     Make nmeshh permanent
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillnmeshh' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillnmeshh'
  1      format ( a, 1x, (10i10) )

      end if
      nullify ( nmeshh )
      if ( associated(km(iseqkm)%nmeshh) ) deallocate(km(iseqkm)%nmeshh)

      if ( kelmaa==0 ) call sepfillkmeshaa
      if ( kelmd==0 ) call sepfillkmeshd

!     --- Array nmeshh does not exist, create
!         First check if nmeshc, nmeshd, kmeshd exist

      if ( nmesh(33)==0 ) call sepfillnmeshc

!     --- Create temporary space for nmeshh

      nedges = nmesh(4)   ! number of edges
      melpnd = nmesh(14)  ! maximimum number of elements per node
      if ( debug ) write(irefwr,1) 'melpnd', melpnd
      if ( melpnd==0 ) then

!     --- melpnd = 0, compute

         if ( debug ) write(irefwr,*) 'before inicompmelpnd'
         call inicompmelpnd ( kmshd1, melpnd, npoint )
         nmesh(14) = melpnd

      end if
      lpnmph = (melpnd+2)*nedges   ! overestimation of length of nmeshh
      if ( debug ) write(irefwr,1) 'melpnd, nedges, lpnmph', &
                                    melpnd, nedges, lpnmph

      allocate ( nmeshh(lpnmph), stat = error )
      if ( error/=0 ) call eralloc ( error, lpnmph, 'nmeshh' )
      if ( ierror/=0 ) go to 1000

!     --- Fill nmesh part h (nmeshh) with actual data

      if ( debug ) write(irefwr,*) 'before topedg05'
      call sepedg05 ( melped, lpnmph )

!     --- Check length of array

      call inichkarlen ( lpnmph, (melpnd+2)*nedges, 'nmeshh' )
      if ( ierror/=0 ) go to 1000

!     --- Make nmeshh permanent

      allocate ( km(iseqkm)%nmeshh(lpnmph) )
      km(iseqkm)%nmeshh = nmeshh(1:lpnmph)
      nmesh(38) = lpnmph
      nmesh(15) = melped  ! maximimum number of elements per edge

      deallocate ( nmeshh, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nmeshh' )
      if ( ierror/=0 ) go to 1000

      nmeshh => km(iseqkm)%nmeshh

      if ( debug ) then

!     --- Debug = true, print nmeshh

         write(irefwr,*) 'Elements connected to edges'
         call prinnmeshx ( nedges, nmeshh )
         write(irefwr,*) 'End sepfillnmeshh'

      end if

1000  call erclos ( 'sepfillnmeshh' )

      end
