      subroutine sepfillnmeshi1 ( lennmeshi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nmeshi with the face numbers of the faces connected to
!     the edges
!     Old subroutine topfaceedge1
! **********************************************************************
!
!                       KEYWORDS
!
!     face
!     edge
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: lennmeshi

!     lennmeshi     i/o   At input declared length of nmeshi
!                         At output actual length
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iface, ipstart, i, iedge, numedges
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     iedge          edge sequence number
!     iface          Face sequence number
!     ipstart        Starting address in array
!     iwork          integer work array of length nedges to fill
!                    number of faces connected or to store starting addresses
!     numedges       number of edges
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array iwork(1:nedges)
!     For all faces do
!        For all edges in face do
!           Raise iwork(edge)
!     Fill nmeshi part 1 using array iwork
!     Copy nmeshi in iwork
!     For all faces do
!        For all edges in face do
!           Raise iwork(edge)
!           Store edge in nmeshi
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillnmeshi1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillnmeshi1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(nedges), stat = error )
      if ( error/=0 ) call eralloc ( error, nedges, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork(1:nedges)

      iwork = 0

!     --- For all faces do
!            For all edges in face do
!               Raise iwork(edge)

      do iface = 1, nfaces
         ipstart = nmeshb(iface)
         numedges = nmeshb(ipstart)
         do i = 1, numedges
            iedge = abs(nmeshb(ipstart+i))
            iwork(iedge) = iwork(iedge)+1
         end do  ! i = 1, numedges
      end do  ! i = 1, nfaces

!     --- Fill nmeshi part 1 using array iwork

      ipstart = nedges+1
      do iedge = 1, nedges

!     --- nmeshi(iedge) := ipstart; nmeshi(ipstart) := iwork(iedge)
!         ipstart := ipstart+iwork(iedge)+1

         nmeshi(iedge) = ipstart
         nmeshi(ipstart) = iwork(iedge)
         ipstart = ipstart+iwork(iedge)+1
         if ( ipstart>lennmeshi ) then

!        --- Error: array nmeshi too large

            call inichkarlen ( ipstart, lennmeshi, 'nmeshi' )
            go to 1000

         end if  ! ( ipstart>lennmeshi )

      end do  ! iedge = 1, nedges
      lennmeshi = ipstart-1

!     --- Copy nmeshi in iwork

      iwork(1:nedges) = nmeshi(1:nedges)

!     --- For all faces do
!            For all edges in face do
!               Raise iwork(edge)
!               Store edge in nmeshi

      do iface = 1, nfaces

         ipstart = nmeshb(iface)
         numedges = nmeshb(ipstart)

         do i = 1, numedges

            iedge = abs(nmeshb(ipstart+i))
            iwork(iedge) = iwork(iedge)+1
            nmeshi(iwork(iedge)) = iface

         end do  ! i = 1, numedges

      end do  ! i = 1, nfaces

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillnmeshi1'

      end if  ! ( debug )
      call erclos ( 'sepfillnmeshi1' )

      end
