      subroutine sepfillnmeshl
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     creates nmesh part l
!     Old subroutine iniactnmeshl
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lpnmpl, lpnmplest, i, medpel, melpnd

!     i              Counting variable
!     lpnmpl         length of array nmeshl
!     lpnmplest      estimated length of array nmeshl
!     medpel         maximum number of edges in one element
!     melpnd         maximum number of elements connected to a node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPEDG06       creates nmesh part l
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLKMESHD  Fill kmesh part d
!     SEPFILLNMESHC  Fill array nmeshc
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     First check if nmeshc, nmeshd, kmeshd exist
!     Activate old arrays nmeshc, nmeshd, kmeshd
!     Create temporary space for nmeshl, do worst case estimate
!     compute starting addresses
!     Fill nmesh part l (nmeshl) with actual data
!     Make nmeshl permanent
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillnmeshl' )
      nullify ( nmeshl )
      if ( associated(km(iseqkm)%nmeshl) ) deallocate(km(iseqkm)%nmeshl)

!     --- First check if nmeshc, nmeshd, kmeshd exist

      if ( kelmd==0 ) call sepfillkmeshd
      if ( kelmaa==0 ) call sepfillkmeshaa
      if ( nmesh(33)==0 ) call sepfillnmeshc

      melpnd = nmesh(14)

      if ( melpnd==0 ) then

!     --- melpnd = 0, compute

         do i = 1, npoint
            melpnd = max(melpnd,kmshd1(i+1)-kmshd1(i))
         end do
         nmesh(15) = melpnd

      end if
      medpel = nmesh(22)

!     --- Create temporary space for nmeshl, do worst case estimate
!         number of nodes * maximum number of elements connected to one
!         node *
!         maximum number of edges connected to one element

      lpnmplest = npoint*melpnd*medpel + 2*npoint
      lpnmpl = lpnmplest

      allocate ( nmeshl(lpnmpl), stat = error )
      if ( error/=0 ) call eralloc ( error, lpnmpl, 'nmeshl' )
      if ( ierror/=0 ) go to 1000

!     --- Fill nmesh part l (nmeshl) with actual data

      call sepedg06 ( lpnmpl )

      if ( lpnmpl>lpnmplest ) then

!     --- Estimated length too short

         call errint ( lpnmplest, 1 )
         call errint ( lpnmpl, 2 )
         call errchr ( 'nmeshl', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

!     --- Make nmeshl permanent

      allocate ( km(iseqkm)%nmeshl(lpnmpl) )
      km(iseqkm)%nmeshl = nmeshl(1:lpnmpl)
      nmesh(42) = lpnmpl

      deallocate ( nmeshl, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nmeshl' )

      nmeshl => km(iseqkm)%nmeshl

1000  call erclos ( 'sepfillnmeshl' )

      end
