      subroutine sepfillvalue ( vector, idegfd, const )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     set one degree of freedom in each nodal point of vector
!     equal to constant
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: idegfd
      double precision, intent(in) :: const
      double precision, intent(inout) :: vector(nusol)

!     const          i    value of the constant
!     idegfd         i    only the idegfd-th unknown in each nodal point with
!                         respect to vector must be considered
!     vector        i/o   output vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ipos, inunkp

!     i              Counting variable
!     inunkp         Number of degrees of freedom in point
!     ipos           Position in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillvalue' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillvalue'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( indprh==0 ) then

!     --- unknowns are not renumbered

         if ( indprf==0 ) then

!        --- constant number of unknowns per point

            do i = 1, npoint
               vector((i-1)*nunkp+idegfd) = const
            end do  ! i = 1, npoint

         else

!        --- variable number of unknowns per point

            do i = 1, npoint
               ipos = kprobf(i)
               inunkp = kprobf(i+1)-ipos

               if ( idegfd>=inunkp ) vector(ipos+idegfd) = const
            end do  ! i = 1, npoint

         end if  ! ( indprf==0 )

      else

!     --- unknowns are renumbered

         if ( indprf==0 ) then

!        --- constant number of unknowns per point

            do i = 1, npoint
               vector(kprobh((i-1)*nunkp+idegfd)) = const
            end do  ! i = 1, npoint

         else

!        --- variable number of unknowns per point

            do i = 1, npoint
               ipos = kprobf(i)
               inunkp = kprobf(i+1)-ipos
               if ( idegfd<=inunkp ) vector(kprobh(ipos+idegfd)) = const
            end do  ! i = 1, npoint

         end if  ! ( indprf==0 )

      end if  ! ( indprh==0 )

1000  call erclos ( 'sepfillvalue' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillvalue'

      end if  ! ( debug )

      end subroutine sepfillvalue
