      subroutine sepgetinpfinf ( isubr, iseqnr, ipos, maxcall )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 26-08-2016 use sepmoduleinput
!        version  1.0  date 23-02-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find maxcall and ipos from sepmoduleinput as function of
!     isubr and iseqnr
! **********************************************************************
!
!                       KEYWORDS
!
!     extract
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleinput
      use sepmodulecons
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: isubr, iseqnr
      integer, intent(out) :: ipos, maxcall

!     ipos           o    Position of input arrays in sepmoduleinput
!     iseqnr         i    Sequence number with respect to the storage in array
!                         ISEPIN
!     isubr          i    Indicates the type of calling subroutine
!                         Possible values
!                          1:  START
!                          2:  PROBDF
!                          3:  COMMAT
!                          4:  CREATE
!                          5:  PRESDF
!                          6:  FILCOF
!                          7:  CHNCOF
!                          8:  SOLVE
!                          9:  OUTPUT
!                         10:  SEPCOMP
!                         11:  DERIV
!                         12:  NLNPRB
!                         13:  INTEGR
!                         14:  INTBND
!                         15:  TIMPRB
!                         16:  ADAPBOUN
!                         17:  ADAPMESH
!                         18:  STATFREE
!                         19:  INSTFREE
!                         20:  CONTACT
!                         21:  LOOP
!                         22:  CHANGE_COORDINATES
!                         23:  EIGENVAL
!                         24:  CAPACITY
!                         25:  INVERSE_PROBLEM
!                         26:  ENTHALPY_INTEGRATION
!                         27:  New problem
!     maxcall         o    Maximum number of items read
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: icall, istart, iseqnrcp, i, j

!                    they are not
!     i              Counting variable
!     icall          Call sequence number for present subroutine
!     iseqnrcp       Copy of iseqnr
!                    In case maxcall=1, iseqnrcp is set to 1
!     istart         First position used at a part
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1883   Sequence number more than maximum
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Activate isepin
!   Compute starting addresses of block2 and block 3 in IBUFFR
!   Compute ICALL, maxcall and ISTART
!   If ( ICALL>maxcall ) then
!      give warning
!      ICALL := maxcall
!   Fill IINPUT and IRNPUT
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepgetinpfinf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepgetinpfinf'
         call ertrace
         write(irefwr,1) 'isubr, iseqnr', isubr, iseqnr
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Compute certain parameters

      maxcall = iinputarrs(isubr)%maxcall
      istart = iinputarrs(isubr)%istart
      icall = iinputarrs(isubr)%icall
      if ( debug ) &
         write(irefwr,1) 'icall, maxcall, istart', icall, maxcall, istart

!     --- Set iseqnrcp equal to iseqnr

      if ( maxcall==1 ) then

!     --- maxcall = 1, so maximal 1 input found
!         in this case we set iseqnrcp equal to 1

         iseqnrcp = 1

      else

!     --- Standard case

         iseqnrcp = iseqnr

      end if  ! ( maxcall==1 )

      if ( maxcall>0 .and. iseqnrcp>0 ) then

         if ( iseqnr>100000000 ) then
            icall = iseqnr-100000000
            icall = nint(scalars(icall))
         else
            icall = iseqnrcp
         end if  ! ( iseqnr>100000000 )
         if ( icall>maxcall .and. (isubr/=3 .or. iseqnrcp/=1) ) then

            select case ( isubr )
               case(1)
                  call errchr ( 'START', 1 )
               case(2,27)
                  call errchr ( 'PROBDF (problem definition)', 1 )
               case(3)
                  call errchr ( 'COMMAT (structure of matrix)', 1 )
               case(4)
                  call errchr ( 'CREATE (create vector)', 1 )
               case(5)
                  call errchr ( 'PRESDF(essential boundary conditions)', &
                                1 )
               case(6)
                  call errchr ( 'FILCOF (coefficients)', 1 )
               case(7)
                  call errchr ( 'CHNCOF (change coefficients)', 1 )
               case(8)
                  call errchr ( 'SOLVE (solve linear system)', 1 )
               case(9)
                  call errchr ( 'OUTPUT', 1 )
               case(10)
                  call errchr ( 'SEPCOMP', 1 )
               case(11)
                  call errchr ( 'DERIVS (derivatives)', 1 )
               case(12)
                  call errchr ( 'NLNPRB (non-linear problem)', 1 )
               case(13)
                  call errchr ( 'INTEGR (integral)', 1 )
               case(14)
                  call errchr ( 'INTBND (boundary integral)', 1 )
               case(15)
                  call errchr ( 'TIMPRB (time integration)', 1 )
               case(16)
                  call errchr ( 'ADAPBOUN (adapt boundary)', 1 )
               case(17)
                  call errchr ( 'ADAPMESH (adapt mesh)', 1 )
               case(18)
                  call errchr ( 'STATFREE (stationary free surface)', &
                                1 )
               case(19)
                  call errchr ( 'INSTFREE (instationary free surface)', &
                        1 )
               case(20)
                  call errchr ( 'CONTACT', 1 )
               case(23)
                  call errchr ( 'EIGENVAL (eigenvalues)', 1 )
               case(24)
                  call errchr ( 'CAPACITY', 1 )
               case(25)
                  call errchr ( 'INVERSE_PROBLEM', 1 )
               case(26)
                  call errchr ( 'ENTHALPY_INTEGRATION', 1 )
            end select  ! case ( isubr )

            call errint ( icall, 1 )
            call errint ( maxcall, 2 )
            call errsub ( 1883, 2, 0, 1 )
            icall = maxcall

         end if

      end if
      if ( debug ) write(irefwr,1) 'maxcall', maxcall
      if ( ierror/=0 ) go to 1000

      ipos =  istart - 1 + icall

      if ( maxcall>0) then

         if ( in(ipos)%used ) then
            iinputarrs(isubr)%icall = min ( maxcall, icall+1 )
            if ( associated(inf(ipos)%iinputf) ) then

!           --- array iinputinf has been filled
!               adapt values in rinput

               do i = 1, size(re(ipos)%rinput)
                  j = inf(ipos)%iinputf(i)
                  if ( j>0 ) re(ipos)%rinput(i) = scalars(j)
               end do

            end if  ! ( associated(inf(ipos)%iinputf) )
         end if  ! ( in(ipos)%used )

      end if  ! ( maxcall>0)

1000  if ( debug) then

!     --- Debug information

         write(irefwr,1) 'ipos, maxcall', ipos, maxcall
         write(irefwr,*) 'End sepgetinpfinf'

      end if  ! ( debug)
      call erclos ( 'sepgetinpfinf' )

      end
