      integer function sepkeywordstr()
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-03-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if one of the keywords is String
!     In that case it might be a string name
!     Old subroutine intxstring
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      logical ::  stringfound

!     i              Counting variable
!     stringfound    If true string is a keyword
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READSTRINGS    Check if name read corresponds to one of the strings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2951   Sequence number of vectors out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepkeywordstr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepkeywordstr'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icode = 0

      do i = 1, nncode

         stringfound = ncodmx(i)==6 .and. code(i)=='STRING'
         jcode = i
         if ( stringfound ) exit

      end do  ! i = 1, nncode

      if ( stringfound ) then

!     --- The keyword string is present

         call readstrings
         if ( ityprd==16 ) then

!        --- string name found

            icode = jcode
            ivalrd = nint(realrd-100000000d0)
            ivalrd = mod(ivalrd,10000)
            ivalrd = mod(ivalrd,5000)

            valmax = numstrings
            if ( ivalrd<0 .or. ivalrd>valmax ) then

!           --- string number out of range

               call errint ( ivalrd, 1 )
               call errint ( valmax, 2 )
               call errsub ( 2951, 2, 0, 0 )
               call errcrd

            end if  ! ( ivalrd<=0 .or. ivalrd>numstrings )

         end if  ! ( ityprd==2 )

      end if  ! ( stringfound )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepkeywordstr'

      end if  ! ( debug )
      call erclos ( 'sepkeywordstr' )

      sepkeywordstr = icode

      end function sepkeywordstr
