      subroutine sepprocopy1 ( kprob2, indprfloc, indprhloc, &
                               indprploc, indpreloc, &
                               indprflocict, maxstore )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-12-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Save parts of array kprob2
!     Sub part of copy prob subroutines
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: kprob2, indprfloc(maxprob), indprhloc(maxprob), &
                 indprploc(maxprob), maxstore, indpreloc(maxstore), &
                 indprflocict(maxprob)

!     indpreloc      o    ?
!     indprfloc      o    indicates if kprobfloc is filled (>0) or not (0)
!     indprflocict   o    ?
!     indprhloc      o    ?
!     indprploc      o    ?
!     kprob2        i/o   Contains copy of old problem
!     maxstore       i    Length of array indpre
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: icount, kelgrp, ivec, iadres

!     iadres         Address in array
!     icount         Counter
!     ivec           Sequence number of vector of special structure
!     kelgrp         sum over all standard elements of number of points in
!                    standard elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     432   Too many problems submitted
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprocopy1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprocopy1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkp = kprob2
      nprob = kp(iseqkp)%nprob
      if ( nprob>maxprob ) then
         call errint ( nprob, 1 )
         call errint ( maxprob, 1 )
         call errsub ( 432, 2, 0, 0 )
         go to 1000
      end if
      icount = 0

      deallocate ( kp(iseqkp)%iinputkp )
      if ( associated(kp(iseqkp)%rinputkp) ) &
         deallocate ( kp(iseqkp)%rinputkp )

      do iprob = 1, nprob

         kprobpart => kp(iseqkp)%kprobsub(iprob)
         kprobloc => kprobpart%kprobloc
         indprfloc(iprob) = kprobloc(19)
         indprploc(iprob) = kprobloc(35)
         indprhloc(iprob) = kprobloc(50)
         if ( kprobloc(51)>0 ) then

!        --- Fictitous unknowns

            indprflocict(iprob) = kprobloc(kprobloc(51)+3)

         else

!        --- No fictitous unknowns

            indprflocict(iprob) = 0

         end if
         ndtvec = kprobloc(6)
         indpre = kprobloc(18)
         kelgrp = kprobloc(39)
         kprobe => kprobpart%kprobe
         kprobe2 => kprobpart%kprobe2
         do ivec = 1, ndtvec
            if ( kprobe2(ivec)>0 ) then
               icount = icount+1
               if ( icount<=maxstore ) then
                  indpreloc(icount) = kprobe2(ivec)
               else
                  deallocate (kprobpart%kprobe3(ivec)%arr )
               end if
            end if
         end do

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprocopy1'

      end if  ! ( debug )
      call erclos ( 'sepprocopy1' )

      end subroutine sepprocopy1
