      subroutine sepreadexpr ( finished )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Check if string read corresponds to expression
!      if so evaluate and return with number
!      Old subroutine intx02expr
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical :: finished

!     finished      i/o   Indicates if a main keyword has been found (true)
!                         or not (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical ::  openfound

!     openfound      Indicates if an openings bracket is found before
!                    reading the number or expression
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READREAL       Read a real or real expression from the input file
!     READSTRCONST   Check if name read corresponds to one the integer or
!                    real constants
!     READSTRSCALS   Check if name read corresponds to one the scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1468   Array tabout too small
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadexpr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadexpr'
         write(irefwr,*) 'string ', string(1:nchar)
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check if the string corresponds to a constant

      ityprd = 4
      call readstrconst

      if ( debug ) write(irefwr,1) 'ityprd/1', ityprd

      if ( ityprd==2 .or. ityprd==3 ) then

!     --- string = constant, check if we are dealing with an expression
!         All open brackets before the keyword are also used

         lastitem = lastitembef ! decrease lastitem, since readreal
                                ! increases it by 1
         do while ( lastitem>0 .and. (typeitems(lastitem)==5 .or. &
                    typeitems(lastitem)==11) )
            lastitem = lastitem-1
         end do  ! while ( lastitem>0 .and. typeitems(lastitem)==5 )
         if ( debug ) write(irefwr,1) 'lastitem/1', lastitem
         openfound = .false.
         if ( debug ) write(irefwr,1) 'lastitem/2', lastitem
         call readreal ( openfound, .false. )
         if ( debug ) write(irefwr,2) 'realrd', realrd

         iposut = iposut+1
         tabout(iposut) = realrd
         iretur = iretur-1
         nseqkeyw = nseqkeyw+1
         iseqkeyw(nseqkeyw) = -iposut
         if ( iposut>nout ) then

!        --- iposut>nout

            call errint ( nout, 1 )
            call errsub ( 1468, 1, 0, 0 )
            go to 1000

         end if  ! ( iposut>nout )
         finished = .true.

      end if  ! ( ityprd==2 .or. ityprd==3 )

!     --- not constant found, check if it is a variable

      call readstrscals

      if ( debug ) write(irefwr,1) 'ityprd/2', ityprd

      if ( ityprd==11 .and. &
           (itype(1)==16 .or. itype(1)==26 ) ) then

!     --- Special case of a scalar found in combination with a
!         coefficient

         table(1) = nint(realrd)
         nseqkeyw = nseqkeyw+1
         iseqkeyw(nseqkeyw) = 0
         readnx = .true.
         finished = .true.

      end if  ! ( ityprd==11 .and. ... )

1000  call erclos ( 'sepreadexpr' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepreadexpr'

      end if  ! ( debug )

      end
