      subroutine sepreadnextit ( finished )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-03-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read next item and interpret
!     Old subroutine readnextitem
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical :: finished

!     finished      i/o   Indicates if a main keyword has been found (true)
!                         or not (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ityprdsav, lastitemsav, lastitemold, lenstr
      double precision realrdsav
      logical :: openfound, equalfound, closefound, vectorfound, &
                 notfound, vecfound
      character (len=6) name

!     closefound     If true a closing bracket has been found, which
!                    corresponds to a closing bracket read earlier
!     equalfound     If true an equals sign has been found
!     ityprdsav      Help parameter to save the value of ityprd
!     lastitemold    Help parameter to save the old value oflastitem
!     lastitemsav    Help parameter to save the value of lastitem
!     lenstr         Length of string
!     name           Name of string
!     notfound       Dummy
!     openfound      Indicates if an openings bracket is found before
!                    reading the number or expression
!     realrdsav      Help parameter to save the value ofrealrd
!     vecfound       If true the expression contains vectors
!     vectorfound    If true the last item read is a vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     READEXPRESS    Interpret part of input as number or expression
!     READREALNOSCAL Read a real or real expression from the input file
!                    scalars are not permitted
!     SEPMAINKEYW2   Check if string read is (main) keyword
!     SEPREADCHKRHS  Check if right-hand side is an expression containing
!                    only existing scalars and vectors, operators and numbers
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Read next item in the usual way by asknxt
!     if ( item = string ) then
!        if ( item=keyword ) then
!           return
!     reset reading
!     if ( item is expression without scalars ) then
!        read expression
!        return
!     else
!        go back to standard reading
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadnextit' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadnextit'
         write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
  1      format ( a, 1x, (10i10) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read next item in the usual way by asknxt

      lastitemold = lastitem   ! necessary for reset
      equalfound = .false.

      if ( nitems>lastitem+1 ) then

!     --- At least 2 extra items can be found

         if ( typeitems(lastitem+1)==13 ) then
            equalfound = .true.
            lastitem = lastitem+1
         end if  ! ( typeitems(lastitem+1)==13 )

      end if  ! ( nitems>lastitem+1 )

      if ( debug ) write(irefwr,*) 'equalfound ', equalfound

      ityprd = 4               ! assume a string
      call asknxt              ! read
      if ( debug ) &
         write(irefwr,1) 'ityprd, lastitem/1', ityprd, lastitem

!     --- Save values in case this is what we need

      ityprdsav = ityprd
      realrdsav = realrd
      lastitemsav = lastitem

      if ( ityprd==4 ) then

!     --- item = string, check if it is a keyword

         call  sepmainkeyw2 ( finished )
         if ( debug ) &
            write(irefwr,1) 'ityprd, icode, lastitem', &
                             ityprd, icode, lastitem


!        --- if ( item=keyword ) then
!               return

         if ( ityprd==2 .and. icode>0 ) then

!        --- ityprd = 2 and icode>0, check if icode corresponds to = vector

            if ( icode>0 ) then
               if ( iseqkeywitem(icode)==0 ) &
                  iseqkeywitem(icode) = lastitem  ! store position
            end if  ! ( icode>0 )
            lenstr = ncodmx(icode)
            name = code(icode)(1:lenstr)
            vectorfound = name(1:lenstr)=='V' .or. &
                          name(1:lenstr)=='VECTOR'

            if ( debug ) then
               write(irefwr,1) 'icode, lenstr', icode, lenstr
               write(irefwr,*) 'equalfound, vectorfound ', &
                                equalfound, vectorfound
            end if  ! ( debug )

            if ( equalfound .and. vectorfound ) then

!           --- = vector found

               icode = 0

            end if  ! ( equalfound ... )

            if ( icode>0 .and. index_rdarr(3)>0 ) then

!           --- indexed vector found

               ityprd = 100
               go to 1000

            end if  ! ( index_rdarr(3)>0 )
         end if  ! ( ityprd==2 .and. icode>0 ) then
         if ( icode>0 ) go to 900

      end if  ! ( ityprd==4 )
      lastitem = lastitemold

      if ( equalfound ) then

!     --- equals sign found, check if it is a vector expression

         vectorfound = .false.
         call sepreadchkrhs ( notfound, vectorfound )
         if ( debug ) then
            write(irefwr,*) 'vectorfound', vectorfound
            write(irefwr,1) 'ityprd', ityprd
         end if  ! ( debug )
         lastitem = lastitemold

      end if  ! ( equalfound )

!     --- reset reading
!         Check if an expression or real is found
!         if true interpret and read expression and return

      openfound = .false.

      if ( .not. vectorfound ) then

!     --- Not a vector found

         call readrealnoscal ( openfound )
         if ( ityprd==2 .or. ityprd==3 ) go to 1000

      end if  ! ( .not. vectorfound )

      if ( equalfound .and. vectorfound ) then

!     --- Vector expression found, read

         closefound = .false.
         if ( debug ) write(irefwr,1) 'lastitem', lastitem
         call readexpress ( openfound, closefound, .true., .false., &
                            .true., vecfound )

         if ( debug ) write(irefwr,1) 'ityprd', ityprd

         if ( ityprd==19 ) go to 1000

      end if  ! ( equalfound )

!     --- go back to standard reading

900   ityprd = ityprdsav
      realrd = realrdsav
      lastitem = lastitemsav
      icode = 0

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icode', icode
         write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
         write(irefwr,2) 'realrd', realrd
         write(irefwr,*) 'End sepreadnextit'

      end if  ! ( debug )
      call erclos ( 'sepreadnextit' )

      end subroutine sepreadnextit
