      subroutine sepreadnokeyw ( finished )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Treat the situation in sepreadsetkeyw that no keyword is found
!     Old subroutine intx02nokeyw
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical :: finished

!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRSUB         Error messages
!     SEPREADEXPR    Check if string read corresponds to expression
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     648   string could not be recognized
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadnokeyw' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadnokeyw'
         write(irefwr,*) 'keywordfound', keywordfound
         write(irefwr,1) 'iretur, lastitem', iretur, lastitem
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( .not. keywordfound .and. iretur==-10000 ) then

!     --- iretur = -10000, so we must return with iretur = -10001
!         minus sign must be read again

         if ( minusg==-1 ) lastitem = lastitem-1
         lastitem = lastitembef  ! last item must be reread
         iretur = -10001
         finished = .true.

      else if ( iretur<=-10 .and. nchar<=32 ) then

!     --- iretur<=-10, check if it is an expression or constant

         if ( debug ) &
            write(irefwr,1) 'nout, iposut, nitems, lastitem', &
                             nout, iposut, nitems, lastitem

         call sepreadexpr ( finished )
         if ( ierror/=0 ) go to 1000
         if ( debug ) &
            write(irefwr,1) 'ityprd, minusg', ityprd, minusg

         if ( (ityprd==2 .or. ityprd==3) .and. minusg==-1 ) then

!        --- Number found, provided with minus sign

            minusg = 1
            tabout(-iseqkeyw(nseqkeyw)) = -realrd

         end if  ! ( ityprd==2 .or. ityprd==3 .and. minusg==-1 )

         if ( debug ) then
            write(irefwr,*) 'after intx02expr'
            write(irefwr,1) 'iretur, lastitem', iretur, lastitem
            write(irefwr,1) 'ityprd, nseqkeyw', ityprd, nseqkeyw
            write(irefwr,1) 'nout, iposut', nout, iposut
            write(irefwr,2) 'realrd', realrd
            if ( nseqkeyw>0 ) then
              write(irefwr,1) '-iseqkeyw(nseqkeyw)', -iseqkeyw(nseqkeyw)
              if ( iseqkeyw(nseqkeyw)<0 ) &
                 write(irefwr,2) 'tabout', tabout(-iseqkeyw(nseqkeyw))
            end if  ! ( nseqkeyw>0 )
            write(irefwr,*) 'finished', finished
         end if  ! ( debug )
         if ( ierror/=0 ) go to 1000

         if ( finished ) then

!        --- End of expression

            if ( lastitem>=nitems ) then

!           --- end of record found

               ityprd = 1

            else

!           --- There is more to read

               finished = .false.

            end if  ! ( lastitem>=nitems ) then

         else

!        --- When we reach this stage, the word is not recognized

           if ( errorcode>0 ) then
              errorcode = -1
              go to 1000  ! jump in case errorcode>0
           end if  ! ( errorcode>0 )

           call errchr ( string(1:nchar), 1 )
           do i = 1, abs(nmain)
              call errchr ( codemain(i), 1+i )
           end do
           do i = 1, nncode
              call errchr ( code(i), 1+i+abs(nmain) )
           end do
           call errsub ( 648, 0, 0, 1+abs(nmain)+nncode )
           call errcrd
           iretur = -1
           finished = .true.

         end if  ! ( finished )

      end if  ! ( iretur<=-10 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iretur', iretur
         write(irefwr,*) 'End sepreadnokeyw'

      end if  ! ( debug )
      call erclos ( 'sepreadnokeyw' )

      end subroutine sepreadnokeyw
