      subroutine sepreadprobseq ( iprob, iseqnr, nprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read sequence number iseqnr and problem number iprob from record
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: iseqnr, iprob
      integer, intent(in) :: nprob

!     iprob         i/o   Actual problem number
!     iseqnr        i/o   Sequence number with respect to the storage in
!                         sepmoduleinput
!     nprob          i    Number of various problems
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadprobseq' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadprobseq'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      code(1) = 'PROBLEM'
      code(2) = 'SEQUENCE_NUMBER'
      ncode(1) = 4
      ncode(2) = 3
      itype(1:2) = 0
      tabmin(1:2) = 1d0
      ncodmx(1) = 7
      ncodmx(2) = 15

      table(1) = iprob
      table(2) = iseqnr
      tabmax(1) = nprob
      tabmax(2) = 100d0
      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      iprob  = nint(table(1))
      iseqnr = nint(table(2))

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepreadprobseq'

      end if  ! ( debug )
      call erclos ( 'sepreadprobseq' )

      end subroutine sepreadprobseq
