      subroutine sepreadsetkeyw
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-03-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read rest of record and check if keywords from the code table followed
!     by a number are available
!     It is assumed that either the record has alrady been read or that ityprd
!     is set equal to 1
!     The subroutine starts with reading the next item but reads only one record
!     If a keyword from the code table is found, the value following it is
!     read and put in the right position of table
!     It is checked if the value read is within a given range
!     If an item is not recognized an error message is given
!     If a main keyword is found or end of record the subroutine returns
!     Old subroutine intx02ext1
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical finished
      integer ispecs, ihelp(lnc), lastitemsav, icount

!     finished       Indicates if the loop has been finished (true) or not
!                    (false)
!     icount         Counter, number of keywords with itype = 8 or 9
!     ihelp          Help array to store number of appearances of keywords that
!                    may be used more than once or number of numbers read
!     ispecs         Help variable to save the value of ispecl
!     lastitemsav    Help parameter to save the value of lastitem
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPMAINKEYW2   Check if string read is (main) keyword
!     SEPREADCOFSCAL pecial case: coefficient is a scalar
!     SEPREADFILLTAB Fill array tabmax for subroutine sepreadsetkeyw
!     SEPREADKEYINIT Initialize some parameters and perform some checks for
!                    sepreadsetkeyw
!     SEPREADMINUS   Check for the presence of minus sign and closing brackets
!     SEPREADNEXT    Keyword found, read next item
!     SEPREADNEXTIT  Read next item and interpret
!     SEPREADNOKEYW  Treat the situation in sepreadsetkeyw that no keyword is
!                    found
!     SEPREADNUMBER  Read number in case iretur = -10
!     SEPREADSETERR  Give error message that keyword is not recognized
!     SEPREADSETSKIP Read next item in case iretur#11
!     SEPREADTYPE89  There are keywords of type 8 and 9
!                    Fill information read into table, tabmax and tabout
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    finished := false
!    Clear array ihelp
!    While ( not finished ) do
!       read next item
!       if ( end of file found ) then
!          iretur := -2
!          finished := true
!       else if ( end of record found ) then
!          iretur := 0
!          finished := true
!       else if ( item # keyword ) then
!          Error message
!          iretur := -1
!          finished := true
!       else if ( item = main keyword ) then
!          set iretur
!          finished := true
!       else if ( item = keyword from code ) then
!          icode := sequence number of keyword in code table
!          if ( itype(icode) # 2 and # -1 ) then
!             read next item
!             if ( itype(icode) = 0, 1, 3 or 4 and next item is a number ) then
!                Check type of number
!                Fill number in table
!             else if ( itype(icode) = 5 or 13 and next item is a text ) then
!                Check type of text
!                Fill number in table
!                if ( itype(icode) = 13 ) then
!                   If ( second keyword should be present ) then
!                      Read next keyword
!                      Check type of text
!                      Fill number in table
!             else if ( itype(icode) = 6 ) then
!                Read and store text
!             else if ( itype(icode) = 3 or 4 and no number found ) then
!                skip
!             else if ( itype(icode) = 10, 12, 30 and 34
!                       and next item is integer number ) then
!                Check sequence number
!                Read next item
!                Check if this is a number in the correct range
!                Fill number in table
!             else if ( itype(icode) = 11 or 28 or 29 ) then
!                Read and check series of integers between brackets
!                Store integers in tabout
!                Set table(i) and tabmax(i)
!             else if ( itype(icode) = 14, 18 or 31 ) then
!                if ( itype(icode) = 18 or 31 ) then
!                   Set keyword to subkeyword
!                While ( next item = keyword with number icode ) do
!                   Read next item (must be integer) and store in tabout
!                   Read next item
!                Set table(i) and tabmax(i)
!             else if ( itype(icode) = 15 ) then
!                if ( ihelp(icode)=0 ) then
!                   Fill table(icode)
!                Read next item (must be integer) and store in tabout
!                Raise ihelp
!             else if ( itype(icode) = 16 ) then
!                Read next item
!                if ( item = 'value' or 'func' or 'time_func' ) then
!                   read next item
!                Fill table and if necessary tabout
!             else if ( itype(icode) = 17 ) then
!                Read series of numbers until new item is found
!                Fill table and tabout
!             else if ( itype(icode) = 19 ) then
!                Read series of numbers until new item is found
!                Fill table and tabout
!             else if ( itype(icode) = 20 ) then
!                Check if keyword is followed by special keyword
!                If ( not special keyword found ) then
!                   Set table(icode)
!                else
!                   Read and store integers
!             else if ( itype(icode) = 21 ) then
!                Read ax+by+sz=d
!                Store result in posout
!             else if ( itype(icode) = 22 or 24 ) then
!                Read next item
!                if ( next item = special keyword ) then
!                   read next item
!                if ( next item = special keyword ) then
!                   read next item
!                Read numbers and store in posout
!             else if ( itype(icode) = 23 ) then
!                Read next item
!                If ( next item is special keyword ) then
!                   skip
!                Set table(icode)
!             else if ( itype(icode) = 25 and next item is integer number ) then
!                Check sequence number
!                Read next item
!                if ( item = 'value', 'func' or 'time_func' ) then
!                   read next item
!                Fill table and if necessary tabout
!             else if ( itype(icode) = 26 ) then
!                Repeat until different item is found
!                   Read next item
!                   if ( item = 'value' or 'func' or 'time_func' ) then
!                      read next item
!                   Fill tabout and tabmax
!             else if ( itype(icode) = 32 ) then
!                Read information after keyword
!                Set table(icode) and fill tabmax and tabout
!             else
!                error message
!                iretur := -1
!                finished := true
!          else if ( itype(icode) = 2 ) then
!             Put a 1 in the table
!       else ( keyword not recognized )
!          iretur := -1
!          finished := true
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadsetkeyw' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadsetkeyw'
         write(irefwr,1) 'iretur, nncode, ityprd, lastitem', &
                          iretur, nncode, ityprd, lastitem
         write(irefwr,1) 'istop, nout, lnc', istop, nout, lnc
         call prinin ( itype, nncode, 'itype' )
         call prinin ( typeitems, nitems, 'typeitems' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d18.10) )

      end if
      if ( ierror/=0 ) go to 1000

      finished = .false.
      readnx = .false.
      keywordfound = .false.
      lastitemsav = lastitem

!     --- keyitems(1:nitems) = 0            ! mark as not found

      if ( iretur/=-11 ) call sepreadsetskip  ! read next item

      if ( debug ) then
         write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
      end if  ! ( debug )

!     --- Initialize some arrays and parameters
!         Perform some checks
!         Compute icount

      call sepreadkeyinit ( ihelp, ispecs, icount )

!     --- Loop over all items in record

      do while ( .not. finished )

!     --- read next item

         if ( ierror/=0 ) go to 1000
         minusg = 1
         ispecl = 4
         if ( debug ) then
            write(irefwr,*) 'readnx ', readnx
            write(irefwr,1) 'istop, lastitem/1a', istop, lastitem
            write(irefwr,1) 'iposut, ityprd', iposut, ityprd
            if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
         end if  ! ( debug )

         if ( readnx .or. ityprd==20 ) then

!        --- Read next item, might be an expression
!            However keywords have higher priority than reals

            call sepreadnextit ( finished )
            if ( debug ) then

               write(irefwr,*) 'after sepreadnextit'
               write(irefwr,1) 'ityprd, istop, lastitem/2', &
                                ityprd, istop, lastitem
               if ( ityprd==4 ) &
                  write(irefwr,*) 'string ', string(1:nchar)

            end if  ! ( debug )

            if ( ityprd==19 .and. errorcode==1 ) then

!           --- Expression found (not allowed), set errorcode negative

               errorcode = -1
               go to 1000

            end if  ! ( ityprd==19 )

         end if  ! ( readnx .or. ityprd==20 )
         if ( ierror/=0 ) go to 1000

         if ( ityprd==11 .and. (itype(1)==16 .or. itype(1)==26 ) ) then

!        --- Special case of a scalar found in combination with a
!            coefficient

            if ( debug ) write(irefwr,*) 'before sepreadcofscal'
            call sepreadcofscal
            if ( debug ) write(irefwr,*) 'coefficient is scalar'
            go to 1000

         else if ( ityprd==100 ) then

!        --- Special case of a vector found

            if ( debug ) write(irefwr,*) 'vector found'
            ityprd = 2
            call sepreadnext ( finished, ihelp, lastitemsav )
            go to 1000

         end if  ! ( ityprd==11 .and. (itype(1)==16 .or. itype(1)==26 ) )

!        --- Check for the presence of a minus sign or closing brackets
!            The last ones are skipped and a new item is read

         if ( debug ) write(irefwr,*) 'before sepreadminus'
         call sepreadminus
         if ( ierror/=0 ) go to 1000
         if ( debug ) then
            write(irefwr,1) 'iretur, ityprd, minusg, lastitem/3', &
                             iretur, ityprd, minusg, lastitem
            if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)

         end if  ! ( debug )

         if ( ityprd==0 ) then

!        --- end of file found< stop reading

            if ( iretur>-10 ) iretur = -2
            finished = .true.

         else if ( ityprd==1 ) then

!        --- end of record found, stop reading

            if ( iretur>-10 ) iretur = 0
            finished = .true.

         else if ( iretur<=-10 .and. iretur>-10000 .and. &
                 (ityprd==2 .or. ityprd==3 .or. ityprd==11 .or. &
                  ityprd==19) ) then

!        --- iretur = -10 and number found, fill tabout

            if ( debug ) write(irefwr,*) 'before sepreadnumber'
            call sepreadnumber
            if ( ierror/=0 ) go to 1000
            if ( debug ) write(irefwr,1) 'iretur', iretur

         else if ( ityprd/=4 .and. ityprd/=100 ) then

!        --- item # keyword: give error message or special return

            call sepreadseterr ( finished )
            go to 1000

         else

!        --- Keyword found, check if it is a main keyword

            if ( debug ) write(irefwr,*) 'before sepmainkeyw2'
            call sepmainkeyw2 ( finished )
            if ( debug ) then
               write(irefwr,*) 'after sepmainkeyw2'
               write(irefwr,1) 'icode, lastitem, iretur', &
                                icode, lastitem, iretur
               write(irefwr,1) 'ityprd, iposut, index_rdarr(3)', &
                                ityprd, iposut, index_rdarr(3)
               write(irefwr,2) 'realrd', realrd
               write(irefwr,*) 'finished', finished
            end if  ! ( debug )
            if ( ierror/=0 ) go to 1000

            if ( .not. finished ) then

!           --- not main keyword found

               if ( icode>0 ) then

!              --- keyword found

                  if ( debug ) write(irefwr,1) 'itype(icode)', itype(icode)
                  if ( itype(icode)==44 ) then

!                 --- itype(icode) = 44, keyword found, return

                     table(icode) = 1d0
                     go to 1000

                  end if  ! ( itype(icode)==44 )

                  if ( debug ) write(irefwr,*) 'before sepreadnext'
                  call sepreadnext ( finished, ihelp, lastitemsav )
                  if ( debug ) then
                     write(irefwr,*) 'after sepreadnext'
                     write(irefwr,1) 'iretur, lastitem', &
                                      iretur, lastitem
                  end if  ! ( debug )

               else

!              --- keyword not recognized
!                  might be an expression or some special case,
!                  but also an error

                  if ( debug ) write(irefwr,*) 'before sepreadnokeyw'
                  call sepreadnokeyw ( finished )
                  if ( ierror/=0 .or. errorcode/=0 ) go to 1000

               end if  ! ( icode>0 )

            end if  ! ( .not. finished )

         end if  ! ( ityprd==0 )

         lastitemsav = lastitem

      end do  ! while ( .not. finished )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'icount', icount
      if ( icount>0 ) then

!     --- icount>0, hence there are keywords of type 8 and 9
!         Fill information read into table, tabmax and tabout

         call sepreadtype89 ( icount )

      end if  ! ( icount>0 )

!     --- Fill tabmax

      call sepreadfilltab ( ihelp )

1000  if ( ierror/=0 ) iretur = -1
      ispecl = ispecs
      if ( debug ) then
         write(irefwr,1) 'istop, lastitem, ityprd', &
                          istop, lastitem, ityprd
         write(irefwr,*) 'End sepreadsetkeyw'
      end if  ! ( debug )
      call erclos ( 'sepreadsetkeyw' )

      end subroutine sepreadsetkeyw
