      subroutine sepreadtype05 ( finished )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword followed by one or two keywords
!     for subroutine sepreadsetkeyw
!     Old subroutine intx0205
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical finished

!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, imax, imin, iminsv, istart, ispeclsav

!     i              Counting variable
!     imax           Maximum position in table
!     imin           Minimum position in table
!     iminsv         Help variable to store the previous value of imin
!     ispeclsav      Help parameter to save the value of ispecl
!     istart         Starting address of information about second keyword in
!                    itype
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     648   string could not be recognized
!    1628   Keyword found more than one for the same problem
!    1879   Not a text found after the keyword
!    1914   Not a second keyword found after keyword = &
!    2811   ncodesub(i) has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadtype05' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadtype05'
         write(irefwr,1) 'icode, itype(icode), ityprd', &
                          icode, itype(icode), ityprd
         write(irefwr,1) 'istop', istop
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
  1      format ( a, 1x, (10i6) )
         write(irefwr,*) 'readnx ', readnx

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ispeclsav = ispecl
      ispecl = 9
      call asknxt
      ispecl = ispeclsav
      if ( debug ) then
         write(irefwr,1) 'ityprd', ityprd
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
      end if  ! ( debug )

      if ( itype(icode)==5 .and. icountkey(icode)/=0 ) then
          call errchr ( code(icode), 1 )
          call errsub ( 1628, 0, 0, 1 )
      else if ( itype(icode)==5 ) then
         icountkey(icode) = icountkey(icode)+1
      end if
      if ( ityprd/=4 ) then

!     --- Not a keyword found after this keyword

         call errchr ( code(icode), 1 )
         call errsub ( 1879, 0, 0, 1 )
         call errcrd

      else

!     --- keyword found, check if it is in the table

         imin = nint(tabmin(icode))
         imax = nint(tabmax(icode))
         if ( debug ) write(irefwr,1) 'imin, imax', imin, imax
         do i = imin, imax
            if ( string(1:ncodesub(i))== &
                 codesub(i)(1:ncodesub(i)) ) then

!           --- Text found

               table(icode) = i-imin+1

               if ( debug ) then
                  write(irefwr,1) 'i, table', i, i-imin+1
                  write(irefwr,*) 'string ', string(1:ncodesub(i))
               end if  ! ( debug )

               if ( itype(icode)==5 ) then
                  go to 1000
               else
                  go to 200
               end if

            end if

         end do

!        --- keyword not recognized

         call errchr ( string(1:nchar), 1 )
         do i = imin, imax
            call errchr ( codesub(i), 2+i-imin )
         end do
         call errsub ( 648, 0, 0, 2+imax-imin )
         call errcrd
         iretur = -1
         finished = .true.

!        --- itype(icode) = 13, check if second keyword should be
!            present

200      iminsv = imin
         istart = itype(icode+1)
         imin = itype(istart+(i-iminsv)*2)
         if ( imin==0 ) go to 1000
         imax = itype(istart+(i-iminsv)*2+1)
         if ( debug ) then
            write(irefwr,1) 'istart, iminsv, i', istart, iminsv, i
            write(irefwr,1) 'imin, imax, ityprd', imin, imax, ityprd
         end if  ! ( debug )

!        --- Read next keyword

         call asknxt
         if ( debug ) write(irefwr,1) 'ityprd', ityprd

         if ( ityprd/=4 ) then

!        --- Not a keyword found after this keyword

            call errchr ( code(icode), 1 )
            iminsv = nint(table(icode))+iminsv-1
            call errchr ( codesub(iminsv), 2 )
            call errsub ( 1914, 0, 0, 2 )
            call errcrd

         else

!        --- keyword found, check if it is in the table

            do i = imin, imax

               if ( ncodesub(i)<1 .or. ncodesub(i)>1000 ) then

!              --- ncodesub(i) out of range

                  call errint ( i, 1 )
                  call errint ( ncodesub(i), 2 )
                  call errint ( 1, 3 )
                  call errint ( 1000, 4 )
                  call errsub ( 2811, 4, 0, 0 )
                  go to 1000

               end if

               if ( string(1:ncodesub(i))== &
                    codesub(i)(1:ncodesub(i)) ) then

!              --- Text found

                  table(icode+1) = i-imin+1
                  if ( debug ) then
                     write(irefwr,1) 'table =', i-imin+1
                     write(irefwr,1) 'i, ncodesub(i)', i, ncodesub(i)
                     write(irefwr,*) 'string ', string(1:ncodesub(i))
                     write(irefwr,*) 'codesub ', codesub(i)(1:ncodesub(i))
                  end if  ! ( debug )
                  go to 1000

               end if

            end do

!           --- keyword not recognized

            if ( debug ) write(irefwr,1) 'istop', istop
            if ( istop==2 ) then

!           --- keyword may be the next keyword

               readnx = .false.
               go to 1000

            else

!           --- Error

               call errchr ( string(1:nchar), 1 )
               do i = imin, imax
                  call errchr ( codesub(i), 2+i-imin )
               end do
               call errsub ( 648, 0, 0, 2+imax-imin )
               call errcrd
               iretur = -1
               finished = .true.
               go to 1000

            end if

         end if

      end if  ! ( ityprd/=4 ) then

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'readnx ', readnx
         write(irefwr,*) 'End sepreadtype05'

      end if
      call erclos ( 'sepreadtype05' )

      end subroutine sepreadtype05
