      subroutine sepreadtype15 ( finished, ihelp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword followed by an integer, repeatedly
!     for subroutine sepreadsetkeyw
!     Old subroutine intx0215
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ihelp(*)
      logical finished

!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
!     ihelp         i/o   Help array to store number of appearances of keywords
!                         that may be used more than once or number of numbers
!                         read
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxval

!     maxval         Maximum value allowed for the integer read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READREALSCAL   Read integer, real or scalar
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     574   Not an integer number found after the keyword
!    1468   Array tabout too small
!    1870   Integer number read out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadtype15' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadtype15'
         write(irefwr,1) 'icode, ihelp(icode)', icode, ihelp(icode)
         write(irefwr,1) 'lastitem, ityprd, minusg', &
                          lastitem, ityprd, minusg
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ihelp(icode)==0 ) then

!     --- ihelp(icode) = 0, keyword has been found for the
!         first time

         table(icode) = iposut + 1
         iposut = iposut + nint(tabmin(icode))
         if ( iposut>nout ) go to 1000

      end if

      if ( ityprd==4 ) call asknxt  ! read item only if preceded by string
      if ( debug ) write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd

      if ( ityprd==4 ) then

!     --- String found, check if it is a vector, scalar or constant

         if ( debug ) then
            write(irefwr,*) 'before readrealscal'
            write(irefwr,*) 'string ', string(1:nchar)
         end if  ! ( debug )

         lastitem = lastitembef
         call readrealscal
         if ( ityprd==102 ) ityprd = 2  ! vector found

         if ( debug ) then
            write(irefwr,*) 'after readrealscal'
            write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd
         end if  ! ( debug )

      end if  ! ( ityprd==4 )

      if ( ityprd==2 .or. ityprd==11 ) then

!     --- Integer number found

         ivalrd = nint(realrd)
         if ( ityprd==11 ) ivalrd = ivalrd - 100000000
         maxval = nint(tabmax(icode))

         if ( debug ) write(irefwr,1) 'ivalrd, maxval', ivalrd, maxval

         if ( ivalrd>-1000 .and. ivalrd<1 .or. ivalrd>maxval ) then

!        --- Error integer out of range

            call errint ( ivalrd, 1 )
            call errint ( 1, 2 )
            call errint ( maxval, 3 )
            call errchr ( code(icode), 1 )
            call errsub ( 1870, 3, 0, 1 )
            call errcrd

         end if

         ivalrd = abs(ivalrd)  ! Make ivalrd positive (necessary for coor)

         if ( nint(table(icode))+ihelp(icode)>nout ) then
            call errint ( nout, 1 )
            call errsub ( 1468, 1, 0, 0 )
            go to 1000
         end if  ! (      nint(table(icode))+ihelp(icode)>nout )

         tabout(nint(table(icode))+ihelp(icode)) = &
            minusg*ivalrd
         ihelp(icode) = ihelp(icode) + 1

      else

!     --- Not an integer number found after the keyword

         call errchr ( code(icode), 1 )
         call errsub ( 574, 0, 0, 1 )
         call errcrd
         finished = .true.

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icode, ihelp(icode)', icode, ihelp(icode)
         write(irefwr,*) 'End sepreadtype15'

      end if
      call erclos ( 'sepreadtype15' )

      end subroutine sepreadtype15
