      subroutine sepreadtype17 ( finished, ihelp, nmax )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword followed by a series of series of numbers
!     for subroutine sepreadsetkeyw
!     Old subroutine intx0217
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ihelp(*), nmax
      logical finished

!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
!     ihelp         i/o   Help array to store number of appearances of keywords
!                         that may be used more than once or number of
!                         numbers read
!     nmax           i    maximum number of numbers allowed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ndim, inumlc, num, maxnum
      logical close

!     close          Indicates if the end of a sequence of numbers has been
!                    found (true) or not (false)
!     inumlc         Number of numbers read in a record
!     maxnum         Maximum number of numbers to be read
!     ndim           Dimension of the space.
!     num            Number of numbers read between brackets
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     SEPREADSERNUMB Read a series of numbers until ) or test is found
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1459   not a series of numbers found after keyword
!    1460   number of numbers between brackets incorrect
!    1462   too much series of numbers read
!    1468   Array tabout too small
!    1628   Keyword found more than one for the same problem
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadtype17' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadtype17'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ndim = nint(tabmin(icode))
      nmax = nint(tabmax(icode))

      if ( debug ) write(irefwr,1) 'ndim, nmax', ndim, nmax
      num = 0
      close = .false.
      table(icode) = iposut+1
      readnx = .false.
      if ( icountkey(icode)/=0 ) then
          call errchr ( code(icode), 1 )
          call errsub ( 1628, 0, 0, 1 )
      else
         icountkey(icode) = icountkey(icode)+1
      end if

      do while ( .not. close )

!     --- end of series has not yet been found
!         Read next series

         if ( ndim==1 ) then
            maxnum = 1        ! maximum number of numbers to be read
         else
            maxnum = ndim+1   ! one extra for check on brackets
         end if  ! ( ndim==1 )

         if ( iposut+1>nout ) then
            call errint ( nout, 1 )
            call errsub ( 1468, 1, 0, 0 )
            go to 1000
         end if  ! ( iposut>nout )

         call sepreadsernumb ( tabout(iposut+1), maxnum, inumlc, &
                               iposut+1 )
         if ( debug ) call prinrl ( tabout(iposut+1), inumlc, 'tabout' )

         if ( inumlc==0 ) then

!        --- end of series found

            if ( num==0 ) then

!           --- No series found

               call errchr ( code(icode), 1 )
               call errsub ( 1459, 0, 0, 1 )
               call errcrd
               ihelp(icode) = 0

            else

!           --- Fill table

               tabmax(icode) = num

            end if
            close = .true.

         else

!        --- next series found

            if ( inumlc/=ndim ) then

!           --- Not exactly ndim numbers between brackets

               call errint ( inumlc, 1 )
               call errint ( ndim, 2 )
               call errsub ( 1460, 2, 0, 0 )
               call errcrd

            end if
            iposut = iposut + ndim
            if ( iposut>nout ) go to 1000
            num = num + 1
            if ( num>nmax ) then

!           --- Too much series of numbers read

               call errchr ( code(icode), 1 )
               call errint ( num, 1 )
               call errint ( nmax, 2 )
               call errsub ( 1462, 2, 0, 1 )
               call errcrd
               finished = .true.

            end if

         end if

      end do  ! while ( .not. close )
      readnx = .false.

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepreadtype17'

      end if
      call erclos ( 'sepreadtype17' )

      end subroutine sepreadtype17
