      subroutine sepreadtype18 ( jump, ihelp, finished )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret part of input record for subroutine sepreadsetkeyw for the
!     possibilities itype = 14, 18 and 31
!     Old subroutine intx0218
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: jump, ihelp(*)
      logical finished

!     finished       o    Indicates if the loop has been finished (true) or not
!                         (false)
!     ihelp          o    Help array to store number of appearances of keywords
!                         that may be used more than once or number of numbers
!                         read
!     jump           o    if 0 no error has been found, if 1 array tabout
!                         is too small
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ih, num, numkey, ispeclsav
      logical ::  readnext
      character (len=80) subkey

!     ih             Help variable to store a constant temporarily
!     ispeclsav      Help parameter to save the value of ispecl
!     num            Number of numbers read between brackets
!     numkey         Number of significant characters in sub keyword
!     readnext       If true the next item must be read
!     subkey         Sub (second) keyword
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READNAMEVEC    Read name of vector in common cpost5
!     READSPLITKEYW  check whether the keyword contains numbers
!                    If so split the keyword into a part without number and
!     SEPREADCURVES  Read part ( Ci, Cj, ... Cl ) in input record
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1461   incorrect keyword found after keyword
!    1467   keyword has been found more than once
!    1468   Array tabout too small
!    1914   Not a second keyword found after keyword = &
! **********************************************************************
!
!                       PSEUDO CODE
!
!    itype(i) =14, special situation of keyword followed
!                  by number, followed by same keyword followed
!                  by number and so on, for example:
!                  p1, p3, p5, p8, p2
!                  The keywords may be preceded by a minus sign, &
!                  hence c1, c3, -c5, c8 etc., in which the
!                  sequence number gets the minus sign
!                  input:  code(i) keyword
!                          tabmin(i) maximum number of integers
!                          tabmax(i) maximum value of integers
!                  output: table(i) = iposut
!                          tabmax(i) = m (number of integers read)
!                          tabout(iposut to iposut+m-1) = values
!    itype(i) =18, special situation of keyword followed by
!                  followed by a series of the same keywords
!                  followed by sequence numbers
!                  hence we have the situation keyword followed
!                  by possibility 14
!                  input:  code(i) keyword
!                          table(i) -iposmain
!                          codesub(iposmain) special sub keyword
!                          tabmin(i) maximum number of numbers
!                                    that may be read
!                          tabmax(i) maximum value of |numbers|
!                  output: table(i)  iposut
!                          tabout(iposut to iposut+m-1) values
!                          tabmax(i) m
!    itype(i) =31, special situation of keyword followed by
!                  possibly an integer followed by
!                  followed by a series of the same keywords
!                  followed by sequence numbers
!                  hence we have the situation keyword
!                  possibly followed by an integer followed
!                  by possibility 18
!                  input:  code(i) keyword
!                          table(i) -iposmain
!                          codesub(iposmain) special sub keyword
!                          tabmin(i) maximum number of numbers
!                                    that may be read
!                          tabmax(i) maximum value of |numbers|
!                  output: table(i)  iposut
!                          tabout(iposut to iposut+m-1) values
!                          tabmax(i) m
!                          tabmin(i) integer after keyword
!                                    if not found: 0
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadtype18' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadtype18'
         write(irefwr,1) 'icode, itype(icode)', icode, itype(icode)
         write(irefwr,1) 'lastitem/1, ityprd', lastitem, ityprd
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      jump = 0
      readnext = .true.
      ispeclsav = ispecl
      if ( itype(icode)==18 .or. itype(icode)==31 ) then

!     --- the keyword must be followed by the next keyword
!         possibly provided with minus sign
!         In the case 31 it may be followed first by an integer

         ih = nint(-table(icode))
         if ( ih<0 ) then
            call errchr ( code(icode), 1 )
            call errsub ( 1467, 0, 0, 1 )
            call errcrd
            go to 1000
         end if
         subkey = codesub(ih)
         numkey = ncodesub(ih)

         call asknxt
         if ( debug ) then
            write(irefwr,1) 'ityprd/1, lastitem', ityprd, lastitem
         end if  ! ( debug )

         if ( ityprd==4 ) then

!        --- ityprd = 4, include digits if necessary

            lastitem = lastitembef
            ispecl = 9
            call asknxt
            ispecl = ispeclsav
            if ( debug ) write(irefwr,*) 'string ', string(1:nchar)

         end if  ! ( ityprd==4 )

         ivalrd = 0
         if ( itype(icode)==31 .and. ityprd==2 .or. ityprd==11 ) then

!        --- integer found

            ivalrd = nint(realrd)
            if ( debug ) write(irefwr,1) 'ivalrd', ivalrd
            call asknxt
            if ( debug ) then
               write(irefwr,1) 'ityprd/2, lastitem', ityprd, lastitem
               if ( ityprd==4 ) &
                  write(irefwr,*) 'string ', string(1:nchar)
            end if  ! ( debug )

         end if
         if ( ityprd==6 ) then

!        --- minus sign found

            minusg = -1
            ispecl = 9
            call asknxt
            ispecl = ispeclsav
            if ( debug ) then
               write(irefwr,*) 'minus sign found'
               write(irefwr,1) 'ityprd/3, lastitem', ityprd, lastitem
               if ( ityprd==4 ) &
                  write(irefwr,*) 'string ', string(1:nchar)
            end if  ! ( debug )

         end if

         if ( debug ) &
            write(irefwr,*) 'subkey(1:numkey) ', subkey(1:numkey)

         if ( subkey(1:numkey)=='V' ) then

!        --- Special case, second keyword is V, so it may also be
!            followed by a vectorname

            if ( ityprd==4 ) then

               if ( string(1:numkey)/=subkey(1:numkey) ) then

!              --- String found is not V, check if it is a vectorname

                  call readnamevec
                  if ( debug ) &
                     write(irefwr,1) 'ityprd/3a, lastitem', &
                                      ityprd, lastitem
                  if ( ityprd==2 ) then

!                 --- Vectorname found

                     readnext = .false.
                     if ( debug ) write(irefwr,1) 'realrd', nint(realrd)

                  else

!                 --- No vectorname found

                     call errchr ( code(icode), 1 )
                     call errchr ( string(1:nchar), 2 )
                     call errchr ( subkey(1:numkey), 3 )
                     call errsub ( 1461, 0, 0, 3 )
                     call errcrd
                     finished = .true.

                  end if  ! ( ityprd==2 )

               end if  ! ( string(1:numkey)/=subkey(1:numkey) )

            else if ( ityprd==2 ) then

!           --- Number found

               readnext = .false.

            else

!           --- Not a keyword found

               call errchr ( code(icode), 1 )
               call errchr ( subkey(1:numkey), 2 )
               call errsub ( 1914, 0, 0, 2 )
               call errcrd
               finished = .true.

            end if  ! ( ityprd==4 ) then

         else if ( ityprd/=4 ) then

!        --- Not a keyword found

            call errchr ( code(icode), 1 )
            call errchr ( subkey(1:numkey), 2 )
            call errsub ( 1914, 0, 0, 2 )
            call errcrd
            finished = .true.

         else if ( string(1:numkey)/=subkey(1:numkey) ) then

!        --- incorrect keyword found

            call errchr ( code(icode), 1 )
            call errchr ( string(1:nchar), 2 )
            call errchr ( subkey(1:numkey), 3 )
            call errsub ( 1461, 0, 0, 3 )
            call errcrd
            finished = .true.

         end if
         if ( readnext .and. .not. finished ) then

!        --- Check if string contains numbers, use name only

            call readsplitkeyw ( numchar, keyword )
            lastitem = lastitembef+1   ! Only string is used
            nchar = numchar

         end if  ! ( readnext .and. .not. finished )

      else

!     --- icode = 14, set subkey

         subkey = code(icode)
         numkey = ncode(icode)

      end if

      if ( iposut>nout ) then
         call errint ( nout, 1 )
         call errsub ( 1468, 1, 0, 0 )
         go to 1000
      end if
      table(icode) = iposut+1

!     --- Read part (Ci, Cj, ... ) and store in tabout

      if ( iposut>nout ) then
         call errint ( nout, 1 )
         call errsub ( 1468, 1, 0, 0 )
         go to 1000
      end if  ! ( iposut>nout )

      call sepreadcurves ( finished, readnext, ispeclsav, subkey, &
                           numkey, jump, ncode(icode), num )

      if ( itype(icode)==31 ) tabmin(icode) = ivalrd

      ihelp(icode) = num
      tabmax(icode) = num

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
         write(irefwr,*) 'End sepreadtype18'

      end if
      call erclos ( 'sepreadtype18' )

      end subroutine sepreadtype18
