      subroutine sepreadtype19 ( ihelp, nmax )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword followed by a series of numbers
!     for subroutine sepreadsetkeyw
!     Old subroutine intx0219
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: ihelp(*)
      integer, intent(out) :: nmax

!     ihelp         i/o   Help array to store number of appearances of keywords
!                         that may be used more than once or number of
!                         numbers read
!     nmax           o    maximum number of numbers found
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inumlc

!     inumlc         Number of numbers read in a record
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPREADSERNUMB Read a series of numbers until ) or test is found
!     SEPREADSETSUB  Interpret series of numbers read
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1468   Array tabout too small
!    1628   Keyword found more than one for the same problem
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadtype19' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadtype19'
         call prinin ( typeitems, nitems, 'typeitems' )
         write(irefwr,1) 'lastitem', lastitem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nmax = nint(tabmin(icode))
      if ( debug ) write(irefwr,1) 'nout, nmax', nout, nmax
      table(icode) = iposut+1
      if ( icountkey(icode)/=0 ) then
          call errchr ( code(icode), 1 )
          call errsub ( 1628, 0, 0, 1 )
      else
         icountkey(icode) = icountkey(icode)+1
      end if
      if ( nout<nmax+1 ) then

         call errint ( nout, 1 )
         call errsub ( 1468, 1, 0, 0 )
         go to 1000

      end if  ! ( nout<nmax+1 )

      call sepreadsernumb ( tabout(iposut+1), nmax+1, inumlc, iposut+1 )

!     --- interpret series of numbers

      call sepreadsetsub ( inumlc, nmax, ihelp )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ityprd', ityprd
         write(irefwr,*) 'readnx', readnx
         write(irefwr,*) 'End sepreadtype19'

      end if
      call erclos ( 'sepreadtype19' )

      end subroutine sepreadtype19
