      subroutine sepreadtype20 ( ihelp, subkey )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read single keyword inwhich case the table must be set to -100000
!     or keyword followed by agiven other keyword and a series of
!     integers
!     for subroutine sepreadsetkeyw
!     Old subroutine intx0220
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ihelp(*)
      character (len=*) :: subkey

!     ihelp         i/o   Help array to store number of appearances of keywords
!                         that may be used more than once or number of
!                         numbers read
!     subkey         i    Sub (second) keyword
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numkey, ih

!     ih             Help variable to store a constant temporarily
!     numkey         Number of significant characters in sub keyword
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRSUB         Error messages
!     SEPREADTYPE11  Read keyword followed by a series of positive integers
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1467   keyword has been found more than once
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadtype20' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadtype20'

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

!     --- Read next item

      call asknxt
      if ( ityprd.ne.4 ) then

!     --- not a keyword found, hence single keyword situation

         table(icode) = -100000d0
         readnx = .false.
         go to 1000

      else

!     --- keyword found, check if it is the special one

         ih = nint(-table(icode))
         if ( ih.lt.0 ) then
            call errchr ( code(icode), 1 )
            call errsub ( 1467, 0, 0, 1 )
            call errcrd
            go to 1000
         end if
         subkey = codesub(ih)
         numkey = ncodesub(ih)
         if ( string(1:numkey).ne.subkey(1:numkey) ) then

!        --- not special keyword found, hence single keyword situation

            table(icode) = -100000d0
            readnx = .false.
            go to 1000

         else

!        --- Special keyword found, read integers
!            This is the situation of itype(icode) = 11, except
!            for positiveness
!            Jump to itype(icode) = 11

            call sepreadtype11 ( ihelp )

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepreadtype20'

      end if
      call erclos ( 'sepreadtype20' )

      end subroutine sepreadtype20
