      subroutine sepreadtype21
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword followed by (ax+by+cz=d)
!     for subroutine sepreadsetkeyw
!     Old subroutine intx0221
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iseq
      double precision value
      logical :: found

!     found          Indicates if already a text or number has been found
!                    after plane (true) or not (false)
!     i              Counting variable
!     iseq           Sequence number read after keyword
!     value          Array containing the real or complex value read
!                    (2 positions)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1281   No character found after number in the part following keyword
!    1282   Character could not be recognized in the part following keyword
!    1465   Letter found in the part ax+by+cz=d more than once
!    1468   Array tabout too small
!    1628   Keyword found more than one for the same problem
!    1868   Text not recognized
!    1869   No number found after keyword
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadtype21' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadtype21'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read next item, the equals sign is treated as special symbol

      table(icode) = iposut+1
      found = .false.

      if ( iposut+4>nout ) then
         call errint ( nout, 1 )
         call errsub ( 1468, 1, 0, 0 )
         go to 1000
      end if  ! ( iposut>nout )

      do i = 1, 4
         tabout(iposut+i) = 0d0
      end do
      if ( icountkey(icode)/=0 ) then
          call errchr ( code(icode), 1 )
          call errsub ( 1628, 0, 0, 1 )
      else
         icountkey(icode) = icountkey(icode)+1
      end if

720   if ( found ) ispecl = 8
      call asknxt
      if ( ityprd==6 ) then
         value = -1d0
      else
         value = 1d0
      end if
      if ( ityprd==2 .or. ityprd==3 ) then

!     --- number found, read next item (must be text)

         value = realrd
         found = .true.
         ispecl = 8
         call asknxt

      end if
      if ( ityprd/=4 .and. ityprd/=9 ) then

!     --- next item not text or equals sign

         call errchr ( code(icode), 1 )
         call errsub ( 1281, 0, 0, 1 )
         call errcrd
         go to 1000

      else if ( ityprd==4 ) then

         if ( string(1:1)=='X' ) then
            iseq = 1
         else if ( string(1:1)=='Y' ) then
            iseq = 2
         else if ( string(1:1)=='Z' ) then
            iseq = 3
         else

!        --- text not recognized

            call errchr ( string(1:nchar), 1 )
            call errchr ( code(icode), 2 )
            call errsub ( 1282, 0, 0, 2 )
            call errcrd
            go to 1000

         end if
         found = .true.
         ispecl = 8

         if ( iposut+iseq>nout ) then
            call errint ( nout, 1 )
            call errsub ( 1468, 1, 0, 0 )
            go to 1000
         end if  ! ( iposut>nout )

         if ( nint(tabout(iposut+iseq))/=0 ) then

!        --- text is used repeatedly

            call errchr ( code(icode), 1 )
            call errchr ( string(1:1), 2 )
            call errsub ( 1465, 0, 0, 2 )
            call errcrd
            go to 1000

         end if
         tabout(iposut+iseq) = value
         go to 720

      else if ( ityprd==9 ) then

!     --- equals sign found, hence one number remains to be read

         ispecl = 4
         call asknxt
         if ( ityprd/=2 .and. ityprd/=3 ) then

!        --- Not a number found

            call errchr ( code(icode), 1 )
            call errsub ( 1869, 0, 0, 1 )
            call errcrd

         else

!        --- Last number read

            tabout(iposut+4) = realrd
            iposut = iposut+ 4
            if ( iposut>nout ) go to 1000

         end if
         go to 1000

      else

!     --- text not recognized

         call errsub ( 1868, 0, 0, 0 )
         call errcrd
         go to 1000

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepreadtype21'

      end if
      call erclos ( 'sepreadtype21' )

      end subroutine sepreadtype21
