      subroutine sepreadtype22 ( subkey, ihelp, nmax )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword followed by possibly one or two special keywords
!     that are skipped and then by a series of numbers
!     for subroutine sepreadsetkeyw
!     Old subroutine intx0222
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ihelp(*), nmax
      character (len=*) :: subkey

!     ihelp         i/o   Help array to store number of appearances of keywords
!                         that may be used more than once or number of
!                         numbers read
!     nmax           i    maximum number of numbers allowed
!     subkey         i    Sub (second) keyword
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ih, inumlc, irep, numkey

!     ih             Help variable to store a constant temporarily
!     inumlc         Number of numbers read in a record
!     irep           Number of local repetitions
!     numkey         Number of significant characters in sub keyword
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPREADSERNUMB Read a series of numbers until ) or test is found
!     SEPREADSETSUB  Interpret series of numbers read
!     SEPREADTYPE11  Read keyword followed by a series of positive integers
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1461   incorrect keyword found after keyword
!    1467   keyword has been found more than once
!    1468   Array tabout too small
!    1628   Keyword found more than one for the same problem
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadtype22' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadtype22'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call asknxt
      irep = 0
      if ( itype(icode)==22 .and. icountkey(icode)/=0 ) then
          call errchr ( code(icode), 1 )
          call errsub ( 1628, 0, 0, 1 )
      else if ( itype(icode)==22  ) then
         icountkey(icode) = icountkey(icode)+1
      end if
760   if ( ityprd==4 ) then

!     --- keyword found, check if it is the special one

         ih = nint(-table(icode))
         if ( ih<0 ) then
            call errchr ( code(icode), 1 )
            call errsub ( 1467, 0, 0, 1 )
            call errcrd
            go to 1000
         end if
         subkey = codesub(ih+irep)
         numkey = ncodesub(ih+irep)
         if ( string(1:numkey)/=subkey(1:numkey) ) then

!        --- string not recognized

            call errchr ( code(icode), 1 )
            call errchr ( string(1:nchar), 2 )
            call errchr ( subkey, 3 )
            call errsub ( 1461, 0, 0, 3 )
            call errcrd
            go to 1000

         end if
         irep = irep + 1
         call asknxt
         if ( irep==1 ) go to 760

      end if

!     --- The rest of the record should contain the numbers to be
!         read. In fact this is the same situation as for
!         itype(icode) = 19, so we jump to that part of the code
!         or in the case 24 as 11, so we jump to that part of the code

      if ( itype(icode)==24 ) then

         readnx = .false.
         call sepreadtype11 ( ihelp )
         go to 1000

      end if
      nmax = nint(tabmin(icode))
      table(icode) = iposut+1

      if ( iposut+1>nout ) then
         call errint ( nout, 1 )
         call errsub ( 1468, 1, 0, 0 )
         go to 1000
      end if  ! ( iposut>nout )

      tabout(iposut+1) = realrd
      infotabout(iposut+1) = ityprd
      if ( ityprd==2 .or. ityprd==3 ) then
         call sepreadsernumb ( tabout(iposut+2), nmax, inumlc, &
                               iposut+2 )
         inumlc = inumlc + 1
      else
         inumlc = 0
      end if

!     --- interpret series of numbers

      call sepreadsetsub ( inumlc, nmax, ihelp )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepreadtype22'

      end if
      call erclos ( 'sepreadtype22' )

      end subroutine sepreadtype22
