      subroutine sepreadtype32 ( finished, ihelp, subkey, m )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keyword like:
!                  surfaces (i1, i2, ... in ) = s1, s2, s3, .. or
!                  surfaces = s1, s2, s3, .. nodes (i1, i2, ... in )
!     for subroutine sepreadsetkeyw
!     Old subroutine intx0232
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ihelp(*), m
      character (len=*) :: subkey
      logical finished

!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
!     ihelp         i/o   Help array to store number of appearances of keywords
!                         that may be used more than once or number of
!                         numbers read
!     m              i    number of integration points
!     subkey         i    Sub (second) keyword
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ih, num, numkey, inumlc, iposloc, ispeclsav, jump
      double precision tablocal(100)
      logical ::  readnext

!     ih             Help variable to store a constant temporarily
!     inumlc         Number of numbers read in a record
!     iposloc        Position in local part of array
!     ispeclsav      Help parameter to save the value of ispecl
!     jump           if 0 no error has been found, if 1 array tabout
!                    is too small
!     num            Number of numbers read between brackets
!     numkey         Number of significant characters in sub keyword
!     readnext       If true the next item must be read
!     tablocal       Local array to be filled with numbers read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     SEPREADCURVES  Read part ( Ci, Cj, ... Cl ) in input record
!     SEPREADSERINT  Read a series of integers
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1042   nodes found but also integers after keyword
!    1467   keyword has been found more than once
!    1468   Array tabout too small
!    1914   Not a second keyword found after keyword = &
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadtype32' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadtype32'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d13.5) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ispeclsav = ispecl
      iposloc = 1

      m = 0
      num = 0
      ih = nint(-table(icode))
      if ( ih<0 ) then
         call errchr ( code(icode), 1 )
         call errsub ( 1467, 0, 0, 1 )
         call errcrd
         go to 1000
      end if
      subkey = codesub(ih)
      numkey = ncodesub(ih)
      inumlc = nint(tabmin(icode))
      table(icode) = iposut+1

      if ( debug ) then
         write(irefwr,*) 'subkey ', subkey
         write(irefwr,1) 'numkey, inumlc, iposut', &
                          numkey, inumlc, iposut
      end if  ! ( debug )

      call asknxt
      if ( debug ) write(irefwr,1) 'ityprd/1',ityprd
      readnx = .false.
      if ( ityprd==7 ) then

!     --- Closing bracket found, read next item

         call asknxt
         if ( num>0 ) then

!        --- already surfaces found, so no surfaces left

            if ( ityprd==4 .and. string(1:5)=='NODES' ) go to 910
            readnx = .false.
            go to 920

         end if

      end if

      if ( ityprd==2 ) then

!     --- integer number found
!         read integer numbers until closing bracket or
!         other type of separator is found

         call sepreadserint ( tablocal, inumlc, m, &
                              .true., inumlc, code(icode), iposloc )
         if ( debug ) call prinrl ( tablocal, m, 'tablocal' )

      end if

      if ( readnx ) call asknxt
      if ( debug ) write(irefwr,1) 'ityprd/2',ityprd
      if ( ityprd==7 ) call asknxt
      if ( debug ) write(irefwr,1) 'ityprd/3, num, m',ityprd, num, m

      if ( num==0 .and. ityprd/=4 ) then

!     --- Not a keyword read

         call errchr ( code(icode), 1 )
         call errchr ( subkey, 2 )
         call errsub ( 1914, 0, 0, 2 )
         call errcrd

      else if ( ityprd==4 ) then

!     --- keyword found

         if ( string(1:numkey)==subkey(1:numkey) ) then

!        --- Sub keyword found, read set

            readnext = .true.
            minusg = 1
            if ( debug ) write(irefwr,1) 'iposut/1', iposut
            call sepreadcurves ( finished, readnext, ispeclsav, subkey, &
                                 numkey, jump, 1, num )
            if ( ierror/=0 ) go to 1000
            if ( debug ) then
               write(irefwr,1) 'num_surfs, iposut', num, iposut
               call prinrl ( tabout(iposut-num), num, 'tabout' )
            end if  ! ( debug )

         end if  ! ( string(1:numkey)==subkey(1:numkey) )
         if ( ityprd==4 .and. string(1:5)=='NODES' ) go to 910
         readnx = .false.
         go to 920

      else if ( num>0 .and. string(1:5)=='NODES' ) then

!     --- keyword nodes found

         go to 910

      else

!     --- Other keyword found

         readnx = .false.
         go to 920

      end if

!     --- keyword nodes found, read integers

910   if ( m>0 ) then

!     --- error nodes found but also integers after keyword

         call errchr ( code(icode), 1 )
         call errsub ( 1042, 0, 0, 1 )
         call errcrd
         finished = .true.
         go to 1000

      end if
      readnx = .true.
      call sepreadserint ( tablocal, inumlc, m, &
                           .true., inumlc, code(icode), iposloc )

920   if ( m>0 ) then

         if ( iposut+m>nout ) then
            call errint ( nout, 1 )
            call errsub ( 1468, 1, 0, 0 )
            go to 1000
         end if  ! ( iposut>nout )

         tabout(iposut+1:iposut+m) = tablocal(1:m)
      end if  ! ( m>0 )
      iposut = iposut+m
      ihelp(icode) = m+10000*num
      tabmax(icode) = m+10000*num
      if ( debug ) then
         write(irefwr,1) 'm, num, icode', m, num, icode
         write(irefwr,2) 'tabmax(icode)', tabmax(icode)
      end if  ! ( debug )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iposut, m, num', iposut, m, num
         call prinrl ( tabout, iposut, 'tabout' )
         write(irefwr,*) 'End sepreadtype32'

      end if
      call erclos ( 'sepreadtype32' )

      end subroutine sepreadtype32
