      subroutine sepreadtype38 ( lastitemsav )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read number or vector name
!     for subroutine sepreadsetkeyw
!     Old subroutine intx0238
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmodulepost
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: lastitemsav

!     lastitemsav   i/o   Help parameter to save the value of lastitem
!                         This is used in case an expression is found, to
!                         restart with the begin of the expression
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ispeclsav
      logical ::  openfound, vectorexpr, closefound, vecfound

!     closefound     If true a closing bracket has been found, which
!                    corresponds to a closing bracket read earlier
!     ispeclsav      Help parameter to save the value of ispecl
!     openfound      Indicates if an openings bracket is found before
!                    reading the number or expression
!     vecfound       If true the expression contains vectors
!     vectorexpr     If true a vector expression has been found
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ADDVEC         Add vector name to the list of vectors
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
!     READEXPRESS    Interprete part of input as number or expression
!     READNAMEVEC    Read name of vector in common cpost5
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1628   Keyword found more than one for the same problem
!    2070   Name found after the percent symbol contains more than 32 characters
!    2950   No vector name found
!    2951   Sequence number of vectors out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadtype38' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadtype38'
         write(irefwr,1) 'icode, itype(icode), lastitem, ityprd', &
                          icode, itype(icode), lastitem, ityprd
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
         write(irefwr,2) 'realrd', realrd
         call prinin ( typeitems, nitems, 'typeitems' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read next item

      if ( ityprd/=2 ) call asknxt
      if ( debug ) then
         write(irefwr,1) 'ityprd, lastitem/1', ityprd, lastitem
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
      end if  ! ( debug )

100   if ( icountkey(icode)/=0 .and. icode==38 ) then
          call errchr ( code(icode), 1 )
          call errsub ( 1628, 0, 0, 1 )
      else
         icountkey(icode) = icountkey(icode)+1
      end if

      if ( debug ) then
         write(irefwr,1) 'ityprd, lastitem/2', ityprd, lastitem
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
      end if  ! ( debug )

      if ( ityprd==4 .and. nchar==1 .and. string(1:1)=='V' .and. &
           typeitems(lastitem+1)==10 ) then

!     --- V% found, read next item

         call asknxt

      else if ( ityprd==4 ) then

!     --- Name found, check if it belongs to the set of vector names
!         The name may contain numbers, hence the name is reread
!         including possible numbers

         lastitem = lastitem-1  ! decrease lastitem for reading
         ispeclsav = ispecl     ! store value of ispecl
         ispecl = 9             ! include numbers in names
         call asknxt            ! read name
         ispecl = ispeclsav     ! reset ispecl
         if ( debug ) write(irefwr,*) 'string/3 ', string(1:nchar)

         if ( nchar>32 ) then

!        --- If number of characters more than 32, reduce to 32 and give
!            warning

            call errchr ( string(1:nchar), 1 )
            call errint ( nchar, 1 )
            call errwar ( 2070, 1, 0, 1 )
            call errcrd
            nchar = 32

         end if  ! ( nchar>32 )

         call readnamevec    ! check if name belongs to set of vector names
         if ( debug ) write(irefwr,1) 'ityprd/3', ityprd

         if ( ityprd/=2 ) then

!        --- Not a vector name, reread name, without number

            lastitem = lastitembef  ! decrease lastitem for reading
            call asknxt             ! read name
            if ( nchar==1 .and. string(1:1)=='V' ) call asknxt

         end if  ! ( ityprd/=2 )

      end if  ! ( ityprd==4 )

      if ( debug ) then
         write(irefwr,1) 'ityprd, lastitem/4', ityprd, lastitem
         if ( ityprd==4 ) write(irefwr,*) 'string/4 ', string(1:nchar)
      end if  ! ( debug )

      if ( ityprd==4 .and. string(1:nchar)/='V' .and. &
           string(1:nchar)/='VECTOR' .and. typeitems(lastitem-1)==13 ) &
         then

!     --- xxx = vector name found, add to list of vectors

         call addvec
         ityprd = 2

      end if  ! ( ityprd==4 )

      if ( debug ) write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem

      if ( ityprd==2 ) then

!     --- Integer found or %name_of_vector or name_of_vector
!         First check if it is a vector expression

         vectorexpr = .false.
         if ( typeitems(lastitem)==3 ) then

!        --- It concerns a vector name hence possibly an expression

            if ( lastitem>1 ) then

!           --- Not last item, check if it is preceded by an operator

               if ( typeitems(lastitem-1)==8 .or. &
                    typeitems(lastitem-1)==11 ) vectorexpr = .true.

            end if  ! ( lastitem>1 )

            if ( .not. vectorexpr .and. lastitem<nitems ) then

!           --- Not preceded by operator, check if it is followed by one

               if ( typeitems(lastitem-1)==8 .or. &
                    typeitems(lastitem-1)==7 .or. &
                    typeitems(lastitem-1)==12 .or. &
                    typeitems(lastitem-1)==19 .or. &
                    typeitems(lastitem-1)==11 ) vectorexpr = .true.

            end if  ! ( .not. vectorexpr )

         end if  ! ( typeitems(lastitem)==3 )

         if ( debug ) write(irefwr,*) 'vectorexpr', vectorexpr

         if ( vectorexpr ) then

!        --- Vector expression found, read

            lastitem = lastitemsav  ! reset lastitem
            openfound = .false.

            call readexpress ( openfound, closefound, .true., &
                               .false., .true., vecfound )

            if ( debug ) write(irefwr,2) 'realrd', realrd

         else

!        --- Standard case

            ivalrd = mod(nint(realrd),10000)
            ivalrd = mod(ivalrd,5000)   ! correction for a series of vectors
            if ( debug ) write(irefwr,1) 'ivalrd', ivalrd
            valmax = maxvc1
            valmin = 1

            if ( mainsubr==4 ) then

!           --- In case of call by seppost we have to decrease the values by 1

               valmax = numvec-1
               valmin = 0

            end if  ! ( mainsubr==4 )

            if ( ivalrd>-1000 .and. ivalrd<valmin .or. ivalrd>valmax ) &
               then

!           --- Vector number out of range

               call errint ( ivalrd, 1 )
               call errint ( valmax, 2 )
               call errsub ( 2951, 2, 0, 0 )
               call errcrd

            end if  ! ( ivalrd<=0 .or. ivalrd>numvec )

         end if  ! ( vectorexpr )

         if ( itype(icode)==38 ) then

!        --- icode = 38, only once

            table(icode) = realrd

         else

!        --- icode = 40, more than once

            iposut = iposut+1
            call inichkarlen ( iposut, nout, 'tabout' )
            if ( ierror/=0 ) go to 1000
            if ( table(icode)<=0d0 ) table(icode) = iposut
            tabout(iposut) = realrd
            tabmax(icode) = icountkey(icode)
            if ( debug ) then
               write(irefwr,1) 'icountkey(icode), ityprd', &
                                icountkey(icode), ityprd
               write(irefwr,2) 'realrd', realrd
               write(irefwr,*) 'string ', string(1:nchar)
            end if  ! ( debug )
            call asknxt
            if ( debug ) write(irefwr,1) 'ityprd', ityprd
            if ( ityprd==4 ) go to 100

         end if  ! ( icode==38 )

      else if ( itype(icode)==40 .and. icountkey(icode)>1 ) then

!     --- icode = 40 and at least 1 has been read
!         end of set, reset everything

         readnx = .false.
         icountkey(icode) = icountkey(icode)-1
         if ( debug ) &
            write(irefwr,1) 'icountkey(icode)', icountkey(icode)

      else

!     --- item not recognized

         call errsub ( 2950, 0, 0, 0 )
         call errcrd

      end if  ! ( ityprd==2 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ivalrd, lastitem', ivalrd, lastitem
         write(irefwr,2) 'realrd', realrd
         write(irefwr,*) 'End sepreadtype38'

      end if  ! ( debug )
      call erclos ( 'sepreadtype38' )

      end subroutine sepreadtype38
