      subroutine sepreadtype39
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-04-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read number or scalar name
!     for subroutine sepreadsetkeyw
!     Old subroutine intx0239
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     keyword
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: maxval, numread, lastitemsav

!     lastitemsav    Help parameter to save the value of lastitem
!     maxval         Highest vector sequence number allowed
!     numread        number of scalars read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ADDSCAL        Add scalar name to the list of scalars
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     READSTRSCALS   Check if name read corresponds to one the scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1628   Keyword found more than one for the same problem
!    2070   Name found after the percent symbol contains more than 32 characters
!    2952   Sequence number of scalar out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadtype39' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadtype39'
         write(irefwr,1) 'icode', icode
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d16.9) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read next item

      if ( ityprd/=11 ) call asknxt
      if ( debug ) then
         write(irefwr,1) 'ityprd, lastitem/1', ityprd, lastitem
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)
      end if  ! ( debug )
      numread = 0
      lastitemsav = lastitem

100   if ( icountkey(icode)/=0 .and. icode==39 ) then
          call errchr ( code(icode), 1 )
          call errsub ( 1628, 0, 0, 1 )
      else
         icountkey(icode) = icountkey(icode)+1
      end if

      if ( ityprd==4 ) then

!     --- Name found, check if it belongs to the set of scalar names

         if ( nchar>32 ) then

!        --- If number of characters more than 32, reduce to 32 and give
!            warning

            call errchr ( string(1:nchar), 1 )
            call errint ( nchar, 1 )
            call errwar ( 2070, 1, 0, 1 )
            call errcrd
            nchar = 32

         end if  ! ( nchar>32 )
         call readstrscals

      end if  ! ( ityprd==4 )

      if ( ityprd==2 ) then

!     --- Integer found

         ivalrd = nint(realrd)

      else if ( ityprd==11 ) then

!     --- name_of_scalar or name_of_scalar

         ivalrd = nint(realrd-100000000d0)

      else if ( numread==0 ) then

!     --- new scalar, create by addscal

         call addscal
         ivalrd = mod(nint(realrd),100000000)

      else if ( ityprd==4 ) then

!     --- New keyword found, reset parameters

         lastitem = lastitemsav
         go to 1000

      end if  ! ( ityprd==2 )
      numread = numread+1

      maxval = numscals
      if ( maxval==0 ) maxval = maxcls

      ivalrd = mod(ivalrd,5000)   ! correction for indexed scalars

      if ( ivalrd<=0 .or. ivalrd>maxval ) then

!     --- Scalar number out of range

         call errint ( ivalrd, 1 )
         call errint ( maxval, 2 )
         call errsub ( 2952, 2, 0, 0 )
         call errcrd

      end if  ! ( ivalrd<=0 .or. ivalrd>numvec )

      if ( itype(icode)==39 ) then

!     --- icode = 39, only once

         table(icode) = realrd

      else

!     --- icode = 45, more than once

         iposut = iposut+1
         call inichkarlen ( iposut, nout, 'tabout' )
         if ( ierror/=0 ) go to 1000
         if ( table(icode)<=0d0 ) table(icode) = iposut
         tabout(iposut) = realrd
         tabmax(icode) = icountkey(icode)
         if ( debug ) then
            write(irefwr,1) 'icountkey(icode), ityprd', &
                             icountkey(icode), ityprd
            write(irefwr,2) 'realrd', realrd
            write(irefwr,*) 'string ', string(1:nchar)
         end if  ! ( debug )

         lastitemsav = lastitem  ! save lastitem

         call asknxt
         if ( debug ) write(irefwr,1) 'ityprd', ityprd
         if ( ityprd==4 ) go to 100

      end if  ! ( icode==39 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ivalrd', ivalrd
         write(irefwr,2) 'realrd', realrd
         write(irefwr,*) 'End sepreadtype39'

      end if  ! ( debug )
      call erclos ( 'sepreadtype39' )

      end subroutine sepreadtype39
