      subroutine seprmprobinfo ( kproblc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-06-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Remove array with sequence number kproblc from kprob
!
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     remove
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: kproblc

!     kproblc      i/o   Defines problem structure by integer sequence number
!                        At output its value is 0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     SEPRMINTMTINFO Remove set of intmat arrays corresponding to iseqkp
!     SEPRMMATINFO   Remove set of matrix arrays corresponding to iseqim
!     SEPRMPROBINFO1 remove subarrays of kprob
!     SEPRMSOLSINFO  Remove set of solution arrays corresponding to iseqkp
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seprmprobinfo' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seprmprobinfo'
         call ertrace
         write(irefwr,1) 'lastprob', lastprob
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkp = kproblc

      if ( debug ) write(irefwr,1) 'iseqkp', iseqkp

      call seprmprobinfo1

      if ( iseqkp==lastprob ) then

!     --- iseqkp = lastprob, decrease lastprob

         lastprob = lastprob-1

         do i = lastprob, 1, -1
            if ( probarr(lastprob)==0 ) then
               lastprob = lastprob-1
            else
               exit
            end if  ! ( probarr(lastprob)==0 )
         end do  ! i = lastprob, 1, -1

      end if  ! ( iseqkp==lastprob )
      probarr(iseqkp) = 0
      nextfreekp = min(nextfreekp,iseqkp)

      deallocate ( kp(iseqkp)%kprobsub )

!     ---remove vectors corresponding to iseqkp

      if ( removesol ) call seprmsolsinfo

!     --- remove intmat and matrix arrays corresponding to iseqkp

      if ( removeintmat ) call seprmintmtinfo
      call seprmmatinfo

      kproblc = 0

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iseqkp, lastprob', iseqkp, lastprob
         write(irefwr,*) 'End seprmprobinfo'

      end if  ! ( debug )
      call erclos ( 'seprmprobinfo' )

      end subroutine seprmprobinfo
