      subroutine seprmprobinfo1
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-06-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Remove array with sequence number iseq from kprob
!
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmoduleprobinf
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seprmprobinfo1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seprmprobinfo1'
         call ertrace
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( associated(kp(iseqkp)%iinputkp) ) &
         deallocate ( kp(iseqkp)%iinputkp )
      if ( associated(kp(iseqkp)%rinputkp) ) &
         deallocate ( kp(iseqkp)%rinputkp )
      nprob = kp(iseqkp)%nprob
      do iprob = 1, nprob

         kprobpart => kp(iseqkp)%kprobsub(iprob)
         kprobloc => kprobpart%kprobloc
         if ( associated(kprobpart%iinputloc) ) &
            nullify( kprobpart%iinputloc )
         if ( associated(kprobpart%rinputloc) ) &
            nullify( kprobpart%rinputloc )
         if ( associated(kprobpart%intarprb) ) then
            deallocate ( kp(iseqkp)%kprobsub(iprob)%intarprb )
            nullify( kprobpart%intarprb )
         end if  ! ( associated(kprobpart%intarprb) ) then
         kproba => kprobpart%kproba

         if ( debug ) write(irefwr,1) 'iprob, indpra', iprob, kprobloc(14)
         if ( kprobloc(14)>0 ) then

!        --- Remove kprob part a

            numnat = kprobloc(12)+kprobloc(8)
            if ( debug ) write(irefwr,1) 'numnat', numnat
            do i = 1, numnat
               if ( associated(kproba(i)%arr) ) &
                  deallocate ( kp(iseqkp)%kprobsub(iprob)%kproba(i)%arr )
            end do  ! i = 1, size(kproba)
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kproba )

         end if  ! ( kprobloc(14)>0 )
         if ( associated(kprobpart%kprobb) ) then
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobb )
            nullify( kprobpart%kprobb )
         end if  ! ( associated(kprobpart%kprobb) ) then
         if ( associated(kprobpart%kprobc) ) then
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobc )
            nullify( kprobpart%kprobc )
         end if  ! ( associated(kprobpart%kprobc) ) then
         if ( associated(kprobpart%kprobd) ) then
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobd )
            nullify( kprobpart%kprobd )
         end if  ! ( associated(kprobpart%kprobd) ) then
         if ( associated(kprobpart%kprobe) ) then
            ndtvec = kprobloc(6)
            do i = 1,ndtvec
               if ( kprobpart%kprobe2(i)>0 ) &
                  deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobe3(i)%arr )
               nullify ( kp(iseqkp)%kprobsub(iprob)%kprobe3(i)%arr )
            end do  ! i = 1,ndtvec
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobe, &
                         kp(iseqkp)%kprobsub(iprob)%kprobe2, &
                         kp(iseqkp)%kprobsub(iprob)%kprobe3 )
            nullify( kprobpart%kprobe, kprobpart%kprobe2, kprobpart%kprobe3 )
         end if  ! ( associated(kprobpart%kprobe) ) then
         if ( associated(kprobpart%kprobf) ) then
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobf )
            nullify( kprobpart%kprobf )
         end if  ! ( associated(kprobpart%kprobf) )
         if ( debug ) write(irefwr,*) 'before kprobg'
         if ( associated(kprobpart%kprobg) ) then
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobg )
            nullify( kprobpart%kprobg )
         end if  ! ( associated(kprobpart%kprobg) )
         if ( debug ) write(irefwr,*) 'before kprobi'
         if ( associated(kprobpart%kprobi) ) then
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobi )
            nullify( kprobpart%kprobi )
         end if  ! ( associated(kprobpart%kprobi) )
         if ( associated(kprobpart%kprobj) ) then
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobj )
            nullify( kprobpart%kprobj )
         end if  ! ( associated(kprobpart%kprobj) ) then
         if ( associated(kprobpart%kprobn) ) then
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobn )
            nullify( kprobpart%kprobn )
         end if  ! ( associated(kprobpart%kprobn) ) then
         if ( associated(kprobpart%coortr) ) then
            deallocate ( kp(iseqkp)%kprobsub(iprob)%coortr )
            nullify( kprobpart%coortr )
         end if  ! ( associated(kprobpart%coortr) ) then
         if ( associated(kprobpart%kprobo) ) then
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobo )
            nullify( kprobpart%kprobo )
         end if  ! ( associated(kprobpart%kprobo) ) then
         if ( associated(kprobpart%kprobp) ) then
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobp )
            nullify( kprobpart%kprobp )
         end if  ! ( associated(kprobpart%kprobp) )
         if ( associated(kprobpart%kprobs) ) then
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobs, &
                         kp(iseqkp)%kprobsub(iprob)%rprobs)
            nullify( kprobpart%kprobs, kprobpart%rprobs )
         end if  ! ( associated(kprobpart%kprobs) )
         if ( associated(kprobpart%kprobh) ) then
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobh, &
                         kp(iseqkp)%kprobsub(iprob)%kprobhinv )
            nullify( kprobpart%kprobh, kprobpart%kprobhinv )
         end if  ! ( associated(kprobpart%kprobh) )
         if ( associated(kprobpart%kprobr) .and. kprobloc(37)>0 ) then
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobr )
            nullify( kprobpart%kprobr )
         end if  ! ( associated(kprobpart%kprobr) )
         if ( associated(kprobpart%kprobx) ) then
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobx )
            nullify( kprobpart%kprobx )
         end if  ! ( associated(kprobpart%kprobx) )
         if ( associated(kprobpart%kproby) ) then
            kproby => kprobpart%kproby
            if ( kproby(4)>0 ) then
               if ( associated(kprobpart%kprobfict) ) &
                  deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobfict )
            end if  ! ( kproby(4)>0 )
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kproby )
            nullify( kprobpart%kproby )
         end if  ! ( associated(kprobpart%kproby) )
         if ( associated(kprobpart%kprobz) ) then
            kprobz => kprobpart%kprobz
            do i = 1, size(kprobz)
               if ( kprobpart%kprobz(i)>0 ) &
                  deallocate (kp(iseqkp)%kprobsub(iprob)%probzarr(i)%arr )
               nullify(kp(iseqkp)%kprobsub(iprob)%probzarr(i)%arr)
            end do  ! i = 1, size(kprobz)
            deallocate ( kp(iseqkp)%kprobsub(iprob)%probzarr )
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobz )
            nullify( kprobpart%kprobz )
         end if  ! ( associated(kprobpart%kprobz) ) then
         if ( associated(kprobpart%kprobaa) ) then
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobaa )
            nullify( kprobpart%kprobaa )
         end if  ! ( associated(kprobpart%kprobaa) ) then
         if ( associated(kprobpart%kprobab) ) then
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobab )
            nullify( kprobpart%kprobab )
         end if  ! ( associated(kprobpart%kprobab) ) then
         if ( associated(kprobpart%kprobac) ) then
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobac )
            nullify( kprobpart%kprobac )
         end if  ! ( associated(kprobpart%kprobac) ) then
         if ( associated(kprobpart%kprobad) ) then
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobad )
            nullify( kprobpart%kprobad )
         end if  ! ( associated(kprobpart%kprobad) ) then
         if ( associated(kprobpart%kprobper) ) then
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobper )
            nullify( kprobpart%kprobper )
         end if  ! ( associated(kprobpart%kprobper) ) then
         if ( associated(kprobpart%kprobpar) ) then
            if ( associated(kprobpart%nnz)) &
               deallocate(kprobpart%nnz)
            if ( associated(kprobpart%onnz)) &
               deallocate(kprobpart%onnz)
            if ( associated(kprobpart%ninterfdeg)) &
               deallocate(kprobpart%ninterfdeg)
            if ( associated(kprobpart%blockinfor)) &
               deallocate(kprobpart%blockinfor)
            if ( associated(kprobpart%kprobhglob)) &
               deallocate(kprobpart%kprobhglob)
            if ( associated(kprobpart%nrusollocs)) &
               deallocate(kprobpart%nrusollocs)
            if ( associated(kprobpart%kprobhdd)) &
               deallocate(kprobpart%kprobhdd, kprobpart%kprobhddinv )
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobpar )
            nullify( kprobpart%kprobpar )

            if ( associated(kprobpart%blockunkinfo)) &
               deallocate(kprobpart%blockunkinfo )
            if ( associated(kprobpart%innercontr)) &
               deallocate(kprobpart%innercontr )
            if ( associated(kprobpart%sendinfo)) &
               deallocate(kprobpart%sendinfo )
            if ( associated(kprobpart%sendinfopar)) &
               deallocate(kprobpart%sendinfopar )
            if ( associated(kprobpart%innercontru)) &
               deallocate(kprobpart%innercontru )
            if ( associated(kprobpart%sendinfou)) &
               deallocate(kprobpart%sendinfou )
            if ( associated(kprobpart%innercontrp)) &
               deallocate(kprobpart%innercontrp )
            if ( associated(kprobpart%sendinfop)) &
               deallocate(kprobpart%sendinfop )
         end if  ! ( associated(kprobpart%kprobpar) )
         if ( associated(kprobpart%kprobfglobal) ) then
            ndtvec = kprobloc(6)
            do i = 0, ndtvec
               if ( associated(kprobpart%kprobfglobal(i)%arr ) ) &
                  deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobfglobal(i)%arr )
            end do  ! i = 0, ndtvec
            deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobfglobal )
            nullify( kprobpart%kprobfglobal )
         end if  ! ( associated(kprobpart%kprobfglobal) ) then

      end do  ! iprob = 1, nprob

      do i = 1, kp(iseqkp)%nprob
         deallocate ( kp(iseqkp)%kprobsub(i)%kprobloc )
      end do  ! i = 1, kp(iseqkp)%nprob

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seprmprobinfo1'

      end if  ! ( debug )
      call erclos ( 'seprmprobinfo1' )

      end subroutine seprmprobinfo1
