      subroutine seprmsolarrs ( meshnrlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-07-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Remove all solution arrays corresponding to a specific mesh
!
! **********************************************************************
!
!                       KEYWORDS
!     mesh
!     memory_management
!     remove
!     solution_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: meshnrlc

!     meshnrlc     i    all solution arrays corresponding to meshnrlc must
!                       be removed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPRMSOLINFO   Remove array with sequence number ivectlc from ks
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seprmsolarrs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seprmsolarrs'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, numvecmn
         if ( isolut3d(i,meshnrlc)>0 ) then
            call seprmsolinfo ( isolut3d(i,meshnrlc) )
         end if  ! ( isolut3d(i,meshnrlc)>0 )
      end do  ! i = 1, numvecmn

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seprmsolarrs'

      end if  ! ( debug )
      call erclos ( 'seprmsolarrs' )

      end subroutine seprmsolarrs
