      subroutine sepstart ( jstart, i1, i2, i3 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-01-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Start SEPRAN
!     store machine dependent quantities in common cmcdpi and cmcdpr
!     open files 1, 2, 3 and 4
!     initialise common blocks
!     set time to zero
! **********************************************************************
!
!                       KEYWORDS
!
!     initialization
!     start
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulexxx
      use sepmodulecons
      use sepmoduleplot
      use sepmodulecomio
      use sepmoduledebug
      use sepmodulemach
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jstart, i1, i2, i3

!     i1         i    irotat   see programmers guide
!     i2         i    ioutp    see programmers guide
!     i3         i    itime    see programmers guide
!     jstart     i    choice parameter consisting of four parts: jfiles, jrefs, &
!                     jsepbc and iintmd according to the formula
!                        JSTART = 10000*IINTMD+100*JFILES+10*JREFS+JSEPBC
!                        when >=0 and
!                        JSTART =-10000*IINTMD-100*JFILES-10*JREFS-JSEPBC
!                        when <0 with:
!                        JSEPBC indicates if file 2 must be opened or not.
!                               Possible values:
!                               0   File 2 does not have to be opened
!                               1   File 2 must be opened, all preceding
!                                   information on the file is destroyed, i.e.
!                                   iwork3 is set to 0
!                              -1   File 2 must be opened, all preceding
!                                   information on the file is kept
!                        JREFS indicates if the special files in the common
!                        block cmachn must be opened (>0) or not (=0)
!                        JFILES indicates if the files 1 and 4 must be opened.
!                               Possible values:
!                               0   Files 1, 4 and 17 are opened
!                               1   The files 1 and 4 are not opened
!                               2   Files 1 and 4 are opened, 17 not
!                        IINTMD indicates if subroutines INITCB and INITMD have
!                        been called before or not
!                        Possible values:
!                        0: INITCB nor INITMD have been called
!                        1: INITMD has been called, but INITCB not
!                        2: Both INITCB and INITMD have been called
!                       10-12: See 1-2, file 3 must not be opened
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, iintmd, jrefs, jsepbc, jfiles, istri2, iopfil3
      logical check

!    check     Indicates if file 1 has been opened (true) or not (false)
!    iintmd    See input parameter JSTART
!    iopfil3   If 0 file 3 must be opened otherwise not
!    istart    See input parameter JSTART
!    istri2    Help variable to store i2 at input
!    jfiles    indicates if the files 1 and 4 must be opened or not.
!              Possible values:
!              0   Files 1 and 4 have to be opened
!              1   Files 1 and 4 do not have to be opened
!    jrefs     indicates if the special files in the common block
!              cmachn must be opened (>0) or not (=0)
!    jsepbc    indicates if file 2 must be opened or not.
!              Possible values:
!              0   File 2 does not have to be opened
!              1   File 2 must be opened, all preceding information
!                  on the file is destroyed, i.e. iwork3 is set to 0
!             -1   File 2 must be opened, all preceding information
!                  on the file is kept
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICLK         initialize the clock
!     INIFIL         Open files
!     INIRMF         Remove file if it exists
!     INITCB         Initializes SEPRAN machine-independent common blocks
!     INITGS         Initialize special common block
!     INITMD         Set machine-dependent quantities
!     READB1         Read first record of SEPRANBACK
!     SEPINITREAD    Initialize module sepmoduleinput
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    80   ioutp incorrect
!    81   itime incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Set timer equal to begin time

      iopfil3 = abs(jstart)/100000
      iintmd = mod(abs(jstart)/10000,10)
      if ( iintmd<2 ) call initcb

      call eropen ( 'sepstart' )

!     --- initialize cinout

      istri2 = i2
      irotat = i1
      ioutp  = i2
      itime  = i3

!     --- Set machine dependent parameters in common blocks

      if ( iintmd==0 ) then
         call initmd
         ioutp  = istri2
         call inifil(5)
         call inifil(6)
      end if

      istart = jstart - sign ( 10000*iintmd+100000*iopfil3, jstart )

      call iniclk

!     --- Open SEPRAN files

      if ( istart>0 ) then

!     --- jsuper>0

         jfiles = istart / 100
         jrefs  = ( istart - 100 * jfiles ) / 10
         jsepbc = ( istart - 100 * jfiles ) - 10 * jrefs

      else

!     --- istart<0

         jfiles = abs (istart) / 100
         jrefs = abs (istart+100*jfiles) / 10
         jsepbc = istart + 10 * jrefs + 100 * jfiles

      end if

      if ( jfiles/=1 ) then

!     --- JFILES = 0, Open files 1 and 4

         call inifil ( 1 )
         call inifil ( 4 )

      end if

      if ( jrefs>0 )then

!     --- Open files corresponding to common cmachn

         call inifil ( -10 )
         call inirmf ( 73 )
         call inirmf ( 74 )
         call inifil ( 74 )

      end if

!     --- Open file 2

      if ( jsepbc>0 ) then
         call inifil ( 2 )
      else if ( jsepbc<0 ) then
         call inifil ( -2 )
      end if

!     --- Open scratch file

      if ( iopfil3==0 ) call inifil ( 3 )

!     --- Fill common with Gauss points and read some extra information in
!         commons

      inquire ( unit=iref1, opened=check )
      if ( check ) call initgs

      if ( machin==2 .or. machin==8 ) then

!     --- Cyber or Cray version  single precision

         intlen = 1

      else

!     --- Other computers  (double precision)

         intlen = 2

      end if

!     --- Test input

      if ( ioutp<-1 .or. ioutp>3 ) then

!     --- Error 80, ioutp has wrong value

         call errint ( ioutp, 1 )
         call errsub ( 80, 1, 0, 0 )

      end if

      if ( itime<0 .or. itime>1 ) then

!     --- Error 81, itime has wrong value

         call errint ( itime, 1 )
         call errsub ( 81, 1, 0, 0 )

      end if

      if ( istart<=-1 ) call readb1

!     --- Initialize module sepmoduleinput

      call sepinitread

      call erclos ( 'sepstart' )

      end
