      subroutine sepstorenewsol ( number, nummax, inodes, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-11-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find all degrees of freedom from the solution array or array of special
!     structure as indicated by array inodes and store in output array
!     Real case
!     Old subroutine er0080
! **********************************************************************
!
!                       KEYWORDS
!
!     selection
!     solution_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: number, nummax, nnodes, inodes(nnodes)

!     inodes         i    Infor entry with respect to the array containing the
!                         nodal point numbers at output
!     nnodes         i    Number of nodes in inodes
!     number         i    The number-th unknown in each nodal point of the array
!                         is considered. If the array is complex number=2*i-1
!                         corresponds to the real part of array i and
!                         number=2*i to the imaginary part of array i
!     nummax         i    Maximum number of unknowns to be extracted in case
!                         number>0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inunkp, inodp, k, ih, istart, lstart

!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     inodp          Nodal point number
!     inunkp         Number of degrees of freedom in point
!     istart         Starting address of node in array USOL
!     k              Counting variable
!     lstart         Starting address of node in array SOLNEW
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepstorenewsol' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepstorenewsol'
         call prinin ( inodes, nnodes, 'inodes' )
         write(irefwr,1) &
            'number, indprfi, nummax, nunkpi', &
             number, indprfi, nummax, nunkpi
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( number>0 ) then

!     --- number>0

         if ( indprfi==0 ) then

!        --- indprfi = 0

            if ( nummax==1 ) then

!           --- nummax = 1

               do i = 1, nnodes
                  inodp = inodes(i)
                  unew(i) = usol(nunkpi*(inodp-1)+number)
               end do

            else

!           --- nummax>1

               do i = 1, nnodes
                  inodp = inodes(i)
                  istart = nunkpi*(inodp-1)+number-1
                  do k = 1, nummax
                     unew((i-1)*nummax+k) = usol(istart+k)
                  end do
               end do

            end if

         else

!        --- indprfi>0

            if ( nummax==1 ) then

!           --- nummax = 1

               do i = 1, nnodes
                  inodp = inodes(i)
                  unew(i) = usol(kprobfi(inodp)+number)
               end do

            else

!           --- nummax>1

               do i = 1, nnodes
                  inodp = inodes(i)
                  istart = kprobfi(inodp)+number-1
                  do k = 1, nummax
                     unew((i-1)*nummax+k) = usol(istart+k)
                  end do
               end do

            end if
         end if

      else

!     --- number = 0, all unknowns

         if ( indprfi==0 ) then

!        --- indprfi = 0, constant number of degrees of freedom

            do i = 1, nnodes
               inodp = inodes(i)
               do k = 1, nunkpi
                  unew((i-1)*nunkpi+k) = usol((inodp-1)*nunkpi+k)
               end do
            end do

         else

!        --- indprfi>0, variable number of degrees of freedom

            lstart = 0
            do i = 1, nnodes
               inodp = inodes(i)
               istart = kprobfi(inodp)
               inunkp = kprobfi(inodp+1)-istart
               do k = 1, inunkp
                  unew(lstart+k) = usol(istart+k)
               end do
               istore(i) = lstart
               lstart = lstart+inunkp
            end do
            istore(nnodes+1) = lstart

         end if
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepstorenewsol'

      end if  ! ( debug )
      call erclos ( 'sepstorenewsol' )

      end subroutine sepstorenewsol
