      subroutine to0019 ( nunkp, ivectr, index1, kprbef, indprf, &
                          indexv, indexj )
! ======================================================================
!
!        programmer    Jaap van der Zanden
!        version  2.4  date 01-03-2002 Layout
!        version  2.3  date 03-01-1999 Layout
!        version  2.2  date 22-12-1992 improved err. rep.; expl. decl.
!
!   copyright (c) 1984-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      fill indexv with positions of degrees of freedom in element ielem
!                  as stored in vector denoted by ivectr
!                  for ivectr (ivctr(2)=110,115,116)
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nunkp, ivectr, index1(*), kprbef(*), indprf, &
              indexv(*), indexj(*)

!     index1     i    Array of length inpelm containing the point numbers
!                     of the nodal points in the element
!     indexj     o    cumulative number of degrees of freedom in current
!                     element for vector denoted by ivectr, indexj(1)=0
!                     indexj(k+1)-indexj(k)=number of degrees of freedom
!                     in k'th nodal point of current element ielem
!     indexv     o    array with position of degrees of freedom in vector
!                     denoted by ivectr for nodal points of current element
!                     ielem
!     indprf     i    indication if kprob part f has been filled (>0) or not (0)
!     ivectr     i    standard sepran array denoting usol or vector of special
!                     structure defined in nodal points
!     kprbef     i
!     nunkp      i    Maximum number of unknowns in a point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ichois, iknpt, inunkp, inusol, j, jcount

!     i          Counting variable
!     ichois     Choice parameter
!     iknpt      Nodal point number
!     inunkp     Number of degrees of freedom in point
!     inusol     Position of unknown in array USOL
!     j          Counting variable
!     jcount     Counter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     200   First input vector is of illegal type
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- test if vector exists (itypvc=110,115,116 allowed)

      jcount=0
      ichois=ks(ivectr)%typevec-109
      indexj(1)=0
      goto (100,1,1,1,1,100,600),ichois

!     --- error : illegal type of vector (.ne.110,115,116)

1        call eropen ( 'to0019' )
         call errint ( ks(ivectr)%typevec,1)
         call errsub ( 200, 1, 0, 0)
         call erclos ( 'to0019' )
         go to 1000
100   if (indprf.ne.0) goto 150

!     --- number of degrees of freedom is constant for itypvc=110,115

      if (nunkp.gt.1) goto 120

!     --- simplest case : 1 degree of freedom per nodal point

      do i=1,inpelm
         indexv(i)=index1(i)
         indexj(i+1)=i
      end do
      jcount=inpelm
      goto (180,1,1,1,1,1000,600),ichois

!     --- number of degrees of freedom is constant but > 1

120   inunkp=nunkp
      do i=1,inpelm
         iknpt=index1(i)
         inusol=(iknpt-1)*inunkp
         do j=1,inunkp
            indexv(jcount+j)=inusol+j
         end do
         jcount=jcount+inunkp
         indexj(i+1)=jcount
      end do
      goto (180,1,1,1,1,1000,600),ichois

!     --- number of degrees of freedom not constant

150   do i=1,inpelm
         iknpt=index1(i)
         inusol=kprbef(iknpt)
         inunkp=kprbef(iknpt+1)-inusol
         if (inunkp.eq.0) goto 165
         do j=1,inunkp
            indexv(jcount+j)=inusol+j
         end do
         jcount=jcount+inunkp
165      indexj(i+1)=jcount
      end do
180   go to 1000

!     --- vector of special structure is defined per element , itypvc = 116
!         number of degrees of freedom per element is also in ks(ivectr)%n

600   inusol=(ielem-1)*ks(ivectr)%n
      do j=1,ks(ivectr)%n
         indexv(j)=inusol+j
      end do
      indexj(2)=ks(ivectr)%n

1000  end
