      subroutine to0022 ( ivec1, ivec2, iseq1, iseq2 )
! ======================================================================
!
!        programmer    Jaap van der Zanden
!        version  3.1  date 30-08-2006 New definition of cpost5
!        version  3.0  date 30-11-2002 Extra parameters iseq, iseq2
!        version  2.4  date 08-11-1999 Allow type 127
!
!   copyright (c) 1984-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     test if vectors are of equal type
!
! **********************************************************************
!
!                       KEYWORDS
!
!     compare
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulecons
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivec1, ivec2, iseq1, iseq2

!     iseq1      i    Sequence number of first input vector in isol
!     iseq2      i    Sequence number of second input vector in isol
!     ivec1      i    vectors to be compared (must be of equal type)
!     ivec2      i    vectors to be compared (must be of equal type)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nch, ih1, ih2, itypvc, jtypvc

!     ih1            Help variable to store a constant temporarily
!     ih2            Help variable to store a constant temporarily
!     nch            Number of characters in a string
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     201   Two SEPRAN vectors are not of equal type
!     202   One of the input vectors is real the other one complex
!     759   Second input vector is of illegal type
!    2103   Length of two input vectors is not the same
!    2370   First input vector is of illegal type
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0022' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0022'
         call ertrace
         write(irefwr,1) 'ivec1, ivec2', ivec1, ivec2
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- tests

      itypvc = ks(ivec1)%typevec
      if ( itypvc/=110 .and. itypvc/=115 .and. &
           itypvc/=116 .and. itypvc/=119 .and. &
           itypvc/=127 ) then
         nch = index(names(iseq1),' ')-1
         if ( nch<=0 ) nch = len(names(iseq1))
         call errchr ( names(iseq1)(1:nch), 1 )
         call errint ( iseq1, 1 )
         call errint ( itypvc, 2 )
         call errsub ( 2370, 2, 0, 1 )
      end if
      jtypvc = ks(ivec2)%typevec
      if ( jtypvc/=110 .and. jtypvc/=115 .and. &
           jtypvc/=116 .and. jtypvc/=119 .and. &
           itypvc/=127 ) then
         nch = index(names(iseq2),' ')-1
         if ( nch<=0 ) nch = len(names(iseq2))
         call errchr ( names(iseq2)(1:nch), 1 )
         call errint ( iseq2, 1 )
         call errint ( jtypvc, 2 )
         call errsub ( 759, 2, 0, 1 )
      end if
      if ( itypvc==116 .and. jtypvc/=116 .or. &
           itypvc/=116 .and. jtypvc==116 .or. &
           itypvc==119 .and. jtypvc/=119 .or. &
           itypvc/=119 .and. jtypvc==119 .or. &
           itypvc==127 .and. jtypvc/=127 .or. &
           itypvc/=127 .and. jtypvc==127 ) then
         nch = index(names(iseq1),' ')-1
         if ( nch<=0 ) nch = len(names(iseq1))
         call errchr ( names(iseq1)(1:nch), 1 )
         call errint ( iseq1, 1 )
         nch = index(names(iseq2),' ')-1
         if ( nch<=0 ) nch = len(names(iseq2))
         call errchr ( names(iseq2)(1:nch), 2 )
         call errint ( iseq2, 2 )
         call errint ( itypvc, 3 )
         call errint ( jtypvc, 4 )
         call errsub ( 201, 4, 0, 2 )
      end if

      if ( ks(ivec1)%compl/=ks(ivec2)%compl ) then
         if ( ks(ivec1)%compl==0 ) then
            ih1 = 1
            ih2 = 3
            call errchr ( 'real', 2 )
            call errchr ( 'complex', 4 )
         else
            ih1 = 3
            ih2 = 1
            call errchr ( 'real', 4 )
            call errchr ( 'complex', 2 )
         end if
         nch = index(names(iseq1),' ')-1
         if ( nch<=0 ) nch = len(names(iseq1))
         call errchr ( names(iseq1)(1:nch), ih1 )
         call errint ( iseq1, ih1 )
         nch = index(names(iseq2),' ')-1
         if ( nch<=0 ) nch = len(names(iseq2))
         call errchr ( names(iseq2)(1:nch), ih2 )
         call errint ( iseq2, ih2 )
         call errsub ( 202, 2, 0, 4 )
      end if
      if ( ks(ivec1)%nusol/=ks(ivec2)%nusol ) then
         nch = index(names(iseq1),' ')-1
         if ( nch<=0 ) nch = len(names(iseq1))
         call errchr ( names(iseq1)(1:nch), 1 )
         call errint ( iseq1, 1 )
         nch = index(names(iseq2),' ')-1
         if ( nch<=0 ) nch = len(names(iseq2))
         call errchr ( names(iseq2)(1:nch), 2 )
         call errint ( iseq2, 3 )
         call errint ( ks(ivec1)%nusol, 2 )
         call errint ( ks(ivec2)%nusol, 4 )
         call errsub ( 2103, 4, 0, 2 )
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End to0022'

      end if  ! ( debug )
      call erclos ( 'to0022' )

      end subroutine to0022
