      subroutine topqu4 ( number, idegfd, solutn )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 23-03-1992 New layout
!        version  2.0  date 09-06-1989 New call, less parameters
!        version  1.0  date 20-06-1988 Developed at Toshiba PC
!
!   copyright (c) 1988-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy the solution array usol (which may also be an array of special
!     structure) into the new organised solution array solutn
!     The new array has length number x npoint.
!     Array solutn contains the unknowns ordered corresponding to the nodes
!     Depending on the value of ichois, points are computed by interpolation
!     or not
!     Both usol and solutn must be real, for the complex equivalent see TOPQU5
! **********************************************************************
!
!                       KEYWORDS
!
!     extract_unknown
!     interpolation
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer number, idegfd
      double precision :: solutn(number,npoint)

!     idegfd  i   The first degree of freedom to be used is number idegfd in
!                 each node
!     number  i   Number of degrees of freedom per point that must be copied
!     solutn  o   Array of length number x npoint in which the reordered
!                 solution array is stored in the sequence:
!                 u (x ) = solutn ( i,k )  (i=1(1)number; k=1(1)npoint)
!                  i  k
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, inusol, inunkp, jnusol

!    i        Loop variable
!    inunkp   Number of unknowns in a node
!    inusol   Position in array USOL.
!             If zero, no solution is available
!    j        Loop variable
!    jnusol   Renumbered position in array USOL
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topqu4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topqu4'
         call ertrace
         write(irefwr,1) 'number, idegfd, npoint', number, idegfd, npoint
         write(irefwr,1) 'indprpi, indprfi, nunkpi', indprpi, indprfi, nunkpi
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear array solutn

      solutn = 0d0

!     --- Copy usol into solutn

      if ( indprpi>0 ) then

!     --- indprpi > 0, use array kprob part p

         do j = 1,number
            do i = 1, npoint
               inusol = kprobpi ( i, idegfd + j - 1 )
               if ( inusol>0 ) solutn ( j,i ) = usol ( inusol )
            end do
         end do

      else if ( indprfi>0 ) then

!     --- indprfi > 0, use array kprob part f  ( not constant number of degrees
!                                               of freedom )

         do i = 1, npoint
            inusol = kprobfi ( i )
            inunkp = kprobfi ( i + 1 ) - inusol
            do j = 1, min ( number, inunkp - idegfd + 1 )
               jnusol = inusol + j + idegfd - 1
               solutn ( j,i ) = usol ( jnusol )
            end do
         end do

      else

!     --- indprfi = 0 and indprpi = 0  Constant number of degrees of freedom

  !      nunkpi = size(usol)/npoint
         do j = 1, min ( number, nunkpi - idegfd + 1 )
            do i = 1, npoint
               inusol = ( i - 1 ) * nunkpi + j + idegfd - 1
               solutn ( j,i ) = usol ( inusol )
            end do
         end do

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topqu4'

      end if  ! ( debug )
      call erclos ( 'topqu4' )

      end subroutine topqu4
