      subroutine askprigen ( inpsep, istart, rinsep, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 22-03-2014 Use local inpsep
!        version  1.5  date 14-01-2013 Extension with print surface
!        version  1.4  date 22-08-2012 Extension with print mesh
!
!   copyright (c) 2008-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret the options after the keyword print
!     and store in inpsep and rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduletexts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(0:*), istart, istrin
      double precision rinsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         The input depends on inpsep(0)
!                         See subroutines askpri and askprisca
!     istart        i/o   At input first position of inpsep to be filled
!                         At output next position of inpsep to be filled
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     rinsep        i/o   Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ispeclsav, iseq, istartsav, number

!     iseq           Sequence number of constant
!     ispeclsav      Help parameter to save the value of ispecl
!     istartsav      Help variable to save the value of istart
!     number         Scalar sequence number read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer sepgetstring, getnameint, getnamereal

!     ASKNXT         Read next item from the standard input file
!     ASKPRINMESH    Interpret the options after the keyword print mesh
!     ASKPRINSURF    Read the keywords followig print surface and store
!                    information in inpsep
!     ASKPRISCA      Interpret options after print_scalar
!     ASKPRITIMHIS   Interpret part following the keywords PRINT TIME_HISTORY
!     ASKPRIVEC      Interpret options after print_vector or print text
!     ASKPRIZEROLEV  Interpret the options after the keyword print zero_levelset
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRSUB         Error messages
!     GETNAMEINT     Return with sequence number of integer in incons
!     GETNAMEREAL    Return with sequence number of real in rlcons
!     PRININ         print 1d integer array
!     READNAMEVEC    Read name of vector in common cpost5
!     SEPGETSTRING   Get the sequence number corresponding to the string in
!                    the block constants with name stringname
! **********************************************************************
!
!                       I/O
!
!    The following options are allowed:
!       PRINT text
!       PRINT scalar
!       PRINT vector
!       PRINT TIME
!       PRINT TIME_HISTORY, VECTOR = v
!       PRINT zero_levelset i
!       All may be followed by the options:
!       FILE = 'name_of_file'
!       append
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1487   Text not recognzied
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'askprigen' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askprigen'
         write(irefwr,1) 'istart, lastitem', istart, lastitem
         write(irefwr,1) 'typeitems(lastitem+1)', typeitems(lastitem+1)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istartsav = istart
      istart = 0

!     --- Interpret item following print

      if ( typeitems(lastitem+1)==4 ) then

!     --- quote found, hence read text

         icode = 15  ! print text
         if ( debug ) write(irefwr,*) 'print text found'
         call askprivec ( inpsep, istart, rinsep, istrin )

      else if ( typeitems(lastitem+1)==3 ) then

!     --- String found, interpret

         ispeclsav = ispecl
         ispecl = 19
         call asknxt
         ispecl = ispeclsav
         if ( nchar>=8 .and. string(1:8)=='TIME_HIS' ) then

!        --- print time_history found, read rest of input

            call askpritimhis ( inpsep, istart )

         else if ( nitems==2 .and. string(1:nchar)=='TIME' ) then

!        --- print time found

            if ( debug ) write(irefwr,*) 'print time found'
            inpsep(0) = 54
            istart = istart + 1

         else if ( string(1:10)=='ZERO_LEVEL' ) then

!        --- print zero_levelset found

            if ( debug ) write(irefwr,*) 'print zero_levelset found'
            call askprizerolev ( inpsep, istart )

         else if ( string(1:nchar)=='MESH' ) then

!        --- print mesh xxx found

            if ( debug ) write(irefwr,*) 'print mesh found'
            call askprinmesh ( inpsep, istart )

         else if ( string(1:7)=='SURFACE' .or. string(1:4)=='CURV' ) &
            then

!        --- print curve or surface found

            if ( debug ) write(irefwr,*) 'print curve/surface found'
            call askprinsurf ( inpsep, istart )

         else

!        --- not print time, check if next item is scalar or vector

            iseq = getnameint ( string(1:nchar) )
            if ( debug ) write(irefwr,1) 'iseq/1', iseq
            if ( iseq>0 ) then

!           --- iseq>0, integer found

               inpsep(0) = 10   ! print constant
               inpsep(1) = 2  ! integer found
               inpsep(2) = iseq ! sequence number
               istart = 3
               go to 1000

            end if  ! ( iseq>0 )

            iseq = getnamereal ( string(1:nchar) )
            if ( debug ) write(irefwr,1) 'iseq/2', iseq
            if ( iseq>0 ) then

!           --- iseq>0, real found

               inpsep(0) = 10   ! print constant
               inpsep(1) = 3  ! real found
               inpsep(2) = iseq ! sequence number
               istart = 3
               go to 1000

            end if  ! ( iseq>0 )

            iseq = sepgetstring ( string(1:nchar), chstrings, numstrings )
            if ( debug ) write(irefwr,1) 'iseq/3', iseq
            if ( iseq>0 ) then

!           --- iseq>0, string found

               inpsep(0) = 10   ! print constant
               inpsep(1) = 5  ! string found
               inpsep(2) = iseq ! sequence number
               istart = 3
               go to 1000

            end if  ! ( iseq>0 )

            call readnamevec       ! if name is vector return with ityprd=2
            if ( ityprd==2 ) then

!           --- vector found

               if ( debug ) write(irefwr,*) 'print vector found'
               icode = 10  ! print vector
               call askprivec ( inpsep, istart, rinsep, istrin )

            else

!           --- scalar found

               lastitem = lastitembef ! restart reading
               icode = 7  ! print scalar
               if ( debug ) write(irefwr,*) 'print scalar found'
               call askprisca ( inpsep, istart, icode )

            end if  ! ( ityprd==2 )

         end if  ! ( nitems==2 .and. string(1:nchar)=='TIME' )

      else

!     --- Item not recognized

         call errsub ( 1487, 0, 0, 0 )
         call errcrd
         go to 1000

      end if  ! ( typeitems(lastitem)==4 )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpsep(0), istart, 'inpsep' )
         write(irefwr,1) 'istart', istartsav+istart
         write(irefwr,*) 'End askprigen'

      end if  ! ( debug )
      call erclos ( 'askprigen' )
      istart = istartsav+istart

      end subroutine askprigen
