      subroutine askpritimhis ( inpsep, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 28-03-2015 Extension with scalars
!        version  2.1  date 16-04-2014 Extension with idegfd
!        version  2.0  date 22-03-2014 Use local inpsep
!
!   copyright (c) 2010-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret part following the keywords PRINT TIME_HISTORY
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     print
!     time_history
!     scalar
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(0:*), istart

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         Contents
!                         Pos. 0    86 to indicate print time_history
!                              1    vector sequence number
!                                   In case of scalars: -10000-num
!                                   with num the number f scalars
!                              2    position of file name in array texts
!                              3    appendmode
!                                   0: not
!                                   1: append
!                              4    idegfd
!                                   In case of scalars: iksipzero
!                                   Indicates if the first time step must
!                                   be skipped (1) or not
!                              5-4+num:  Only in case of scalars:
!                                   scalar sequence numbers
!     istart        i/o   Next free position in inpsep
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncod1=6
      integer ncode1(nncod1), ncod1m(nncod1), num
      character (len=9) code1(nncod1)

!     code1          Table with keywords
!     ncod1m         Table containing the number of maximum characters in
!                    the keyword table code1
!     ncode1         Table containing the number of significant letters in
!                    code1
!     nncod1         Number of entries in code1
!     num            Number of scalars read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     INTXSTORETEXT  Fill reference to text and number of characters in inpsep
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!    The following options are allowed:
!       PRINT TIME_HISTORY, VECTOR = v, FILE = 'name_of_file', APPEND
!       PRINT TIME_HISTORY, set of scalars, FILE = 'name_of_file', APPEND, &
!             SKIP_ZERO
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2549   Both scalars and points in vectors not allowed in time history
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'VECTOR', 'FILE', 'DEGFD', 'APPEND', 'SCALAR', &
                   'SKIP_ZERO' /
      data ncode1 / 1, 4, 3, 3, 3, 4 /
      data ncod1m / 6, 4, 5, 6, 6, 9 /
! ======================================================================
!
      call eropen ( 'askpritimhis' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askpritimhis'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepcopycode ( code1, nncod1, ncode1, ncod1m )
      table(1:nncode) = 0d0
      itype(1) = 38   ! Vector or vector name
      itype(2) = 6    ! File = file_name
      itype(3) = 0    ! degfd
      itype(4) = 2    ! append
      itype(5) = 45   ! Set of scalars or scalar names
      itype(6) = 2    ! Skip_zero
      tabmin(3) = 1d0  ! minimum value
      tabmax(3) = 1d1  ! largest value
      nmain = 0

      call sepreadsetkeyw
      if ( debug ) call prinrl ( table, nncod1, 'table' )

!     --- File name

      call intxstoretext ( inpsep(2), table(2), tabmax(2), &
                           strout )

      inpsep(0) = 86   ! print time_history
      inpsep(1) = mod(nint(table(1)),100000000)  ! vector sequence number
      inpsep(3) = nint(table(4))  ! append mode
      inpsep(4) = nint(table(3))  ! idegfd
      num = 0
      if ( table(5)>0d0 .and. table(1)<=0d0 ) then

!     --- scalars found

         iposut = nint(table(5))
         num = nint(tabmax(5))  ! number of scalars
         inpsep(1) = -num-10000
         inpsep(4) = nint(table(6))
         inpsep(5:4+num) = &
            mod(nint(tabout(iposut:iposut+num-1)),100000000)

      else if ( table(5)>0d0 ) then

!     --- Error both scalars and vectors found (not allowed)

         call errsub ( 2549, 0, 0, 0 )

      end if  ! ( table(5)>0d0 .and. table(1)<=0d0 )

!     --- Raise istart

      istart = istart+5+num

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpsep(0), 5+num, 'inpsep' )
         write(irefwr,*) 'End askpritimhis'

      end if  ! ( debug )
      call erclos ( 'askpritimhis' )

      end subroutine askpritimhis
