      subroutine askprivec ( inpsep, istart, rinsep, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 16-04-2014 New length inpsep
!        version  2.0  date 22-03-2014 Use local inpsep
!        version  1.1  date 26-05-2011 New call to askpri
!
!   copyright (c) 2008-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information following print_vector
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(0:*), istart, istrin
      double precision rinsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!     istart        i/o   At input first position of inpsep to be filled
!                         At output next position of inpsep to be filled
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     rinsep        i/o   Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: number

!     number         Scalar sequence number read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKPRI         Interpret options after print_vector
!     ASKSCL         Read and interpret integer sequence number after keyword
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'askprivec' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askprivec'
         write(irefwr,1) 'icode, istart', icode, istart
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( icode<15 .or. icode==114 ) then

!     --- icode<15, hence print vector

         call askscl ( number, 1, maxcls )
         if ( ierror/=0 ) go to 1000
         if ( icode==114 ) then
            inpsep(0) = icode+3   ! 114 => 117
         else
            inpsep(0) = 9
         end if  ! ( icode==114 )
         if ( debug ) then
            write(irefwr,*) 'After askscl'
            write(irefwr,1) 'number', number
         end if  ! ( debug )

      else

!     --- icode = 15, hence print scalar

         inpsep(0) = 15

      end if

      call askpri ( inpsep(1), rinsep, number, istrin )
      if ( ierror/=0 ) go to 1000

      if ( icode==15 ) then
         istart = istart+6
      else
         istart = istart+23
         istart = istart + inpsep(20)
      end if

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpsep(1), 20, 'inpsep' )
         write(irefwr,1) 'istart', istart
         write(irefwr,*) 'End askprivec'

      end if  ! ( debug )
      call erclos ( 'askprivec' )

      end subroutine askprivec
