      subroutine askvec03 ( inpsep, table, tabout, &
                            tabmax, code, ncodmx )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 17-01-2015 New parameter list
!        version  1.0  date 22-12-2013
!
!   copyright (c) 2013-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store degree of freedom, curves and skip element groups in inpsep
!     Help subroutine for askvec
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), ncodmx(*)
      double precision table(*), tabout(*), tabmax(*)
      character (len=*) code(*)

!     code           i    Table with keywords
!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         See subroutine askvec
!     ncodmx         i    Table containing the maximal number of  characters in
!                         the keyword table code
!     table         i/o   Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabmax        i/o   Contains maximum values allowed in table
!     tabout        i/o   Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer idegfd

!     idegfd         Degree of freedom sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKVEC25       Store information about skip element groups
!     ASKVECCURVS    Interpret information about curves
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRKEYWEXCLUS  Check if two keywords are mutually exclusive
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'askvec03' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askvec03'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Store degree of freedom

      idegfd = nint(table(11))
      if ( idegfd>0 ) inpsep(5) = idegfd

!     --- The keywords element groups and curves are mutually exclusive

      call errkeywexclus ( table, code, ncodmx, 22, 23 )
      call errkeywexclus ( table, code, ncodmx, 22, 27 )
      call errkeywexclus ( table, code, ncodmx, 23, 27 )
      call errkeywexclus ( table, code, ncodmx, 22, 40 )
      call errkeywexclus ( table, code, ncodmx, 23, 40 )
      call errkeywexclus ( table, code, ncodmx, 27, 40 )

!     --- information about skip element groups

      if ( table(22)>0d0 .or. table(23)>0d0 ) then

!     --- information about skip element groups

         call askvec25 ( inpsep, length, table, nelgrp, tabout, &
                         tabmax )

      end if  ! ( table(22)>0d0 .or. table(23)>0d0 )

      if ( table(27)>0d0 ) then

!     --- information about curves

         length = length+1
         inpsep(16) = 2          ! iplace (curves)
         call askveccurvs ( inpsep, table(27), tabmax(27), ncurvs, &
                            length, tabout, 15 )

      else if ( table(40)>0d0 ) then

!     --- information about surfaces

         length = length+1
         inpsep(16) = 3          ! iplace (surfaces)
         call askveccurvs ( inpsep, table(40), tabmax(40), nsurfs, &
                            length, tabout, 15 )

      end if  ! ( table(27)>0d0 ) then

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End askvec03'

      end if  ! ( debug )
      call erclos ( 'askvec03' )

      end
