      subroutine askvec24 ( inpsep, found, tabmx2, nexpected, ichois, &
                            namekeyw )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-05-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret part following mesh_velocity in subroutine askvec
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), nexpected, ichois
      double precision tabmx2(*)
      character (len=*) namekeyw
      logical found

!     found         i/o   Indicates if one of the possibilities modulus, length, &
!                         phase or phase or subtract has been found (true) or
!                         not (false)
!     ichois         i    Choice parameter (inpsep(4))
!     inpsep         o    The integer input read is stored in array INPSEP
!                         See subroutine askvec for a description
!     namekeyw       i    Name of keyword
!     nexpected      i    Number of vectors expected
!     tabmx2         i    Table containing the maximum values of the quantities
!                         given in code2
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer n

!     n              Number of occurences of keyword found
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1903   From the possibilities: PHASE, MODULUS and LENGTH at least two are
!           found in the record
!    2918   Number of vectors following mesh_velocity is not 2
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'askvec24' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askvec24'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(4) = ichois
      if ( found ) call errsub ( 1903, 0, 0, 0 )
      found = .true.
      n = nint(tabmx2(3))
      if ( n/=nexpected ) then

!     --- N#nexpected, hence not nexpected vectors found following keyword

         call errint ( n, 1 )
         call errint ( nexpected, 2  )
         call errchr ( namekeyw, 1 )
         call errsub ( 2918, 2, 0, 1 )
         go to 1000

      end if  ! ( n/=2 )

1000  call erclos ( 'askvec24' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End askvec24'

      end if  ! ( debug )

      end
