      subroutine askvecstscal ( inpsep, minussign, iscal1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 22-04-2013 Correction in case of indexed vectors
!        version  1.0  date 03-05-2010
!
!   copyright (c) 2010-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store scalar numbers for subroutine askvec
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmodulecons
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), minussign, iscal1

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         See subroutine askvec for a description
!     iscal1         o    Sequence number of first scalar
!     minussign      i    Sign to be used for first scalar
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer n, istart, iscal2, iscal

!     iscal          Sequence number of a scalar
!     iscal2         Sequence number of second scalar
!     istart         Starting address in tabout
!     n              Number of occurences of keyword found
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1893   Scalar has not yet been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'askvecstscal' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from askvecstscal'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      n = nint(tabmax(8))
      istart = nint(table(8))
      iscal1 = 0
      iscal2 = 0
      if ( debug ) write(irefwr,1) 'n', n

      if ( n>=1 ) then

         iscal1  = minussign*mod(nint(tabout(istart)),100000)
         if ( debug ) write(irefwr,1) 'iscal1', iscal1
         iscal = mod(iscal1,5000)
         if ( iscalars(abs(iscal))/=1 ) then
            call errint ( iscal1, 1 )
            call errchr ( 'scalar', 1 )
            call errchr ( 'scalar', 2 )
            call errsub ( 1893, 1, 0, 2 )
         end if

      end if

      if ( n>=2 ) then

         iscal2  = mod(nint(tabout(istart+1)),100000)
         if ( debug ) write(irefwr,1) 'iscal2', iscal2
         iscal = mod(iscal2,5000)
         if ( iscalars(iscal)/=1 ) then
            call errint ( abs(iscal2), 1 )
            call errchr ( 'scalar', 1 )
            call errchr ( 'scalar', 2 )
            call errsub ( 1893, 1, 0, 2 )
         end if

      end if

      inpsep(11) = iscal1
      inpsep(12) = iscal2

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End askvecstscal'

      end if  ! ( debug )
      call erclos ( 'askvecstscal' )

      end
