      subroutine eradapms
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 26-08-2016 use sepmoduleinput
!        version  2.3  date 26-08-2011 Simplification of subroutine
!        version  2.2  date 09-07-2001 Include common cseqnr
!
!   copyright (c) 1996-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Read input concerning the adaptation of the mesh from the
!    standard input file
!    Store the result in sepmoduleinput
! **********************************************************************
!
!                       KEYWORDS
!
!     adapt_mesh
!     change_mesh
!     free_boundary
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleinput
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nparm, ncall, nrlparm, n1, n2
      parameter ( nparm=40, nrlparm=1, ncall=25 )
      integer inpmsh(nparm+ncall), ireadbn(ncall)
      double precision rinmsh(nrlparm+ncall)

!     inpmsh         User input array for subroutine ADAPMESH
!     ireadbn        Array of length 25 containing the sequence numbers
!                    with respect to adapt boundary
!     n1             Number of elements along the  south  side
!                    of the current quadrilateral
!     n2             Number of elements along the  east  side
!                    of the current quadrilateral
!     ncall          Number of calls to a specific under subroutine
!     nparm          Number of parameters for a specific element group
!     nrlparm        Number of entries in rinada
!     rinmsh         User input array for subroutine ADAPMESH
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOPYINPUT    Copy input read by a read subroutine into buffer array
!     EROPEN         Produces concatenated name of local subroutine
!     READADMS       Read input for subroutine ADAPMESH from the standard input
!                    file
! **********************************************************************
!
!                       I/O
!
!    Input may be read from the standard input file
!    The first record is interpreted by this subroutine, the reading
!    of the rest of the records is performed by subroutine READADMS
!    The first record must have the following shape:
!    ADAPT_MESH [SEQUENCE_NUMBER = k]
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   reserve space for input arrays
!   Read rest of input until END has been found
!   Make input arrays permanent and store references in isepi3
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eradapms' )
      if ( ierror/=0 ) go to 1000

!     --- Read input and store in arrays inpmsh and rinmsh

      iseqadapms = iseqadapms + 1

      call readadms ( inpmsh, rinmsh, 2, iseqadapms, &
                      ireadbn, nparm )
      if ( ierror/=0 ) go to 1000

      n1 = inpmsh(1)
      n2 = inpmsh(2)
      length = n1+n2
      inpmsh(n1+1:n1+n2) = ireadbn(1:n2)

!     --- Make input arrays permanent

      call ercopyinput ( 17, iseqadapms, 'ADAPT_MESH', inpmsh, &
                         rinmsh, inpmsh, length, nrlparm, 0 )

1000  call erclos ( 'eradapms' )
      end
