      subroutine ercompposlev ( xcoor, funcx, ncoor, fval, xval, &
                                intersect )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-08-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find x-position of function corresponding to f = c
!
! **********************************************************************
!
!                       KEYWORDS
!
!     1d
!     interpolation
!     level
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncoor
      double precision xcoor(*), funcx(*), fval, xval
      logical intersect

!     funcx          i    Array of function values along line
!     fval           i    level for which the x-value must be computed
!     intersect      o    if true fval is in the reach of the function
!     ncoor          i    Number of nodes along 1D line
!     xcoor          i    array of x coordinates along line
!     xval           o    computed x-value
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision x1, x2, f1, f2, eps
      logical found

!     eps            Local accuracy
!     f1             value of f in left-hand point of interval
!     f2             value of f in right-hand point of interval
!     found          If true an intersection has been found
!     i              Counting variable
!     x1             x-coordinate in left-hand point of interval
!     x2             x-coordinate in right-hand point of interval
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercompposlev' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercompposlev'
         write(irefwr,*) 'fval ', fval
         call prinrl ( xcoor, ncoor, 'xcoor' )
         call prinrl ( funcx, ncoor, 'funcx' )

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

      found = .false.
      i = 1
      eps = sqreps
      if ( abs(fval).gt.1d-20 ) eps = abs(fval)*sqreps
      xval = 0d0

      do while ( .not. found .and. i.le.ncoor-1 )
         x1 = xcoor(i)
         x2 = xcoor(i+1)
         f1 = funcx(i)
         f2 = funcx(i+1)
         if ( f2.gt.f1 ) then
            found = fval.ge.f1-eps .and. fval.le.f2+eps
         else
            found = fval.ge.f2-eps .and. fval.le.f1+eps
         end if  ! ( f2.gt.f1 )

         if ( found ) then

!        --- intersection found, compute x

            found = .true.
            if ( abs(f2-f1).lt.eps ) then

!           --- f is constant, set xval to x1

               xval = x1

            else

!           --- f is not constant, compute xval

               xval = x1 + (fval-f1)/(f2-f1)*(x2-x1)

            end if  ! ( f2-f1.lt.eps ) then

         else

!        --- intersection not found, raise i

            i = i+1

         end if  ! ( f1.ge.fval-eps .and. f2.le.fval+eps )

      end do  ! while ( not found )
      intersect = found

1000  call erclos ( 'ercompposlev' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'xval, intersect ',xval, intersect
         write(irefwr,*) 'End ercompposlev'

      end if

      end
