      subroutine ercontct
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 26-08-2016 use sepmoduleinput
!        version  1.3  date 22-11-2012 Long integers
!        version  1.2  date 09-07-2001 Include common cseqnr
!
!   copyright (c) 1997-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Read input concerning contact from the standard input file
!    Store the result in sepmoduleinput
! **********************************************************************
!
!                       KEYWORDS
!
!     contact
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleinput
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ipos
      integer, parameter :: nparm=9
      integer :: iinder(nparm)
      double precision :: rinput

!     iinder         User input array for subroutine DERIV
!     ipos           Position of input arrays in sepmoduleinput
!     nparm          Number of entries in inpconct
!     rinput         Dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOPYINPUT    Copy input read by a read subroutine into sepmoduleinput
!     EROPEN         Produces concatenated name of local subroutine
!     READCONT       Read information for contact problems
! **********************************************************************
!
!                       I/O
!
!    Input may be read from the standard input file
!    The first record is interpreted by this subroutine, the reading
!    of the rest of the records is performed by subroutine READCONT
!    The first record must have the following shape:
!    CONTACT [SEQUENCE_NUMBER = k]
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   reserve space for input arrays
!   Read rest of input until END has been found
!   Make input arrays permanent and store references in isepi3
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercontct' )

!     --- Read input and store in array inpconct
!         reserve space in buffer array for the array inpconct

      iseqcontct = iseqcontct + 1

      call readcont ( iinder, 2, iseqcontct, nparm )

!     --- Make input arrays permanent

      call ercopyinput ( 20, iseqcontct, 'CONTACT', iinder, &
                         rinput, iinder, nparm, 0, 0 )

      call erclos ( 'ercontct' )
      end
