      subroutine erfillcf220
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iincof/rincof for boundary elements of type 220
!     Simple beam element
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     fill
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulestrings
      use sepmodulecoeffs
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLCFRULE   Fill array iincof/rincof part integration rule and
!                    coordinate system
!     ERFILLDEFCOF1  Check if keyword is found in list of constants, scalars or
!                    vectors
!                    If so fill value in iincof/rincof
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The default input is:
!     coef 6 = ELASTIC_MODULUS (E)
!     coef 7 = SECOND_MOMENT   (I)
!     coef 8 = load
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfillcf220' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillcf220'
         write(irefwr,1) 'iposin, iposin', iposin, iposin
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 6, 8

         if ( imark(i)==0 ) then

!        --- Coefficient i has not been filled

            icoef = i
            call erfilldefcof1 ( codemech(i-5) )

         end if  ! ( imark(6)==0 )

      end do  ! i = 6, 8

!     --- integration rule and coordinate system

      call erfillcfrule

      iincof(iposin+1) = -2        ! end of information

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iposin, iposin', iposin, iposin
         write(irefwr,*) 'End erfillcf220'

      end if  ! ( debug )
      call erclos ( 'erfillcf220' )

      end subroutine erfillcf220
