      subroutine erfillcf250
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 22-02-2015 Extension with E_X ...
!        version  1.0  date 15-12-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iincof/rincof for elements of type 250
!
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     fill
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulestrings
      use sepmodulecoeffs
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ndimsav, icoefforce

!     i              Counting variable
!     icoefforce     Coefficient sequence number w.r.t. force vector
!     ndimsav        Help variable to save value of ndim
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLCFFORCE  Fill array iincof/rincof part force
!     ERFILLDEFCOF1  Check if keyword is found in list of constants, scalars or
!                    vectors
!                    If so fill value in iincof/rincof
!     ERFILLDEFCOF2  Check if keyword is found in list of vectors
!                    If so fill value in iincof/rincof
!     ERFILLDEFCOF3  Check if keyword is found in list of constants or scalars
!     ERFILLDEFCOF4  Check if keyword is found in list of constants or strings
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfillcf250' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillcf250'
         write(irefwr,1) 'iposin, iposrn', iposin, iposrn
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, 2

         icoef = i+5  ! ELASTIC_MODULUS, POISSONS_RATIO
         if ( imark(icoef)==0 ) then

!        --- Coefficient icoef has not been filled

            call erfilldefcof1 ( codestress(i)(1:ncodestress(i)) )

         end if  ! ( imark(icoef)==0 )

      end do  ! icoef = 1, 2

      icoef = 27   ! THICKNESS
      i = 3
      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         iposin_h = iposin
         call erfilldefcof1 ( codestress(i)(1:ncodestress(i)) )
         if ( iposin_h==iposin ) iposin_h = 0

      end if  ! ( imark(icoef)==0 )

      icoef = 2  ! STRESS_STRAIN_RELATION
      i = 4

      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof4 ( codestress(i)(1:ncodestress(i)) )

      end if  ! ( imark(icoef)==0 )

      icoef = 3  ! INTEGRATION_RULE
      i = 5

      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof3 ( codestress(i)(1:ncodestress(i)) )

      end if  ! ( imark(icoef)==0 )

!     --- Force vector

      icoefforce = 28
      call erfillcfforce ( icoefforce )

      icoef = 31  ! INITIAL_STRAIN
      i = 10

      ndimsav = ndim
      if ( ndim==2 ) then
         ndim = 3
      else
         ndim = 6
      end if  ! ( ndim==2 )

      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof2 ( codestress(i)(1:ncodestress(i)) )

      end if  ! ( imark(icoef)==0 )
      ndim = ndimsav

      do i = 11, 16

         icoef = 20+i  ! xx_INITIAL_STRAIN

         if ( imark(icoef)==0 ) then

!        --- Coefficient icoef has not been filled

            call erfilldefcof1 ( codestress(i)(1:ncodestress(i)) )

         end if  ! ( imark(icoef)==0 )

      end do  ! i = 7, 12

      icoef = 37  ! INITIAL_STRESS
      i = 17

      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof2 ( codestress(i)(1:ncodestress(i)) )

      end if  ! ( imark(icoef)==0 )

      do i = 18, 23

         icoef = 19+i  ! xx_INITIAL_STRESS

         if ( imark(icoef)==0 ) then

!        --- Coefficient icoef has not been filled

            call erfilldefcof1 ( codestress(i)(1:ncodestress(i)) )

         end if  ! ( imark(icoef)==0 )

      end do  ! i = 14, 19

      icoef = 43   ! DENSITY
      i = 24
      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof1 ( codestress(i)(1:ncodestress(i)) )

      end if  ! ( imark(icoef)==0 )

      do i = 25, 29

         icoef = i-19  ! ELASTICITY parts

         if ( imark(icoef)==0 ) then

!        --- Coefficient icoef has not been filled

            call erfilldefcof1 ( codestress(i)(1:ncodestress(i)) )

         end if  ! ( imark(icoef)==0 )

      end do  ! i = 21, 25

      do i = 30, 37

         icoef = i-24  ! ELASTICITY parts

         if ( imark(icoef)==0 ) then

!        --- Coefficient icoef has not been filled

            call erfilldefcof1 ( codestress(i)(1:ncodestress(i)) )

         end if  ! ( imark(icoef)==0 )

      end do  ! i = 26, 33

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iincof, iposin, 'iincof' )
         call prinrl ( rincof, iposrn, 'rincof' )
         write(irefwr,*) 'End erfillcf250'

      end if  ! ( debug )
      call erclos ( 'erfillcf250' )

      end subroutine erfillcf250
