      subroutine erfillcf420
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-12-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iincof/rincof for elements of type 420
!
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     fill
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulestrings
      use sepmodulecoeffs
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: icoefforce

!                    they are not
!     icoefforce     Coefficient sequence number w.r.t. force vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLCFFORCE  Fill array iincof/rincof part force
!     ERFILLDEFCOF1  Check if keyword is found in list of constants, scalars or
!                    vectors
!                    If so fill value in iincof/rincof
!     ERFILLDEFCOF4  Check if keyword is found in list of constants or strings
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The default input is:
!  codenavstoks      contains keywords for navier stokes
!                 1: MODEL
!                 2: INTEGRATION_RULE
!                 3: COORDINATE_SYSTEM
!                 4: LINEARIZATION
!                 5: PENALTY
!                 6: DENSITY
!                 7: ANGULAR_VELOCITY
!                 8: FORCE
!                 9: X_FORCE
!                10: Y_FORCE
!                11: Z_FORCE
!                12: VISCOSITY
!                13: POWER_N
!                14: LAMBDA
!                15: VOLUME_EXPANSION_COEFFICIENT
!                16: REF_TEMPERATURE
!                17: HEAT_CAPACITY
!                18: THERMAL_CONDUCTIVITY
!                19: HEAT_SOURCE
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfillcf420' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillcf420'
         write(irefwr,1) 'iposin, iposin', iposin, iposin
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First fill model

      icoef = 12  ! MODEL

      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

        call erfilldefcof4 ( codenavstoks(1) )

      end if  ! ( imark(icoef)==0 )

      icoef = 3  ! LINEARIZATION

      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

        call erfilldefcof4 ( codenavstoks(4) )

      end if  ! ( imark(icoef)==0 )


      do icoef = 1, 2

         if ( imark(icoef)==0 ) then

!        --- Coefficient icoef has not been filled

            call erfilldefcof1 ( codenavstoks(icoef+4) )

         end if  ! ( imark(icoef)==0 )

      end do  ! icoef = 1, 2

!     --- Coefficient 4 (angular_velocity)

      icoef = 4
      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof1 ( codenavstoks(7) )

      end if  ! ( imark(icoef)==0 )

!     --- Coefficient 4 (angular_velocity)

      icoef = 5
      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof1 ( codenavstoks(15) )

      end if  ! ( imark(icoef)==0 )

      do icoef = 6, 8

         if ( imark(icoef)==0 ) then

!        --- Coefficient icoef has not been filled

            call erfilldefcof1 ( codenavstoks(icoef+10) )

         end if  ! ( imark(icoef)==0 )

      end do  ! icoef = 6,

!     --- Force vector

      icoefforce = 9
      call erfillcfforce ( icoefforce )

      iincof(iposin+1) = -2        ! end of information

      icoef = 11
      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof1 ( codenavstoks(19) )

      end if  ! ( imark(icoef)==0 )
!                 8: FORCE
!                 9: X_FORCE
!                10: Y_FORCE
!                11: Z_FORCE
!                12: VISCOSITY
!                13: POWER_N
!                14: LAMBDA
!                19: HEAT_SOURCE

!     --- Coefficients 13 to 15

      do icoef = 13, 15

         if ( imark(icoef)==0 ) then

!        --- Coefficient icoef has not been filled

            call erfilldefcof1 ( codenavstoks(icoef-1) )

         end if  ! ( imark(icoef)==0 )

      end do  ! icoef = 13, 15

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iposin, iposin', iposin, iposin
         call prinin ( iincof, iposin, 'iincof' )
         write(irefwr,*) 'End erfillcf420'

      end if  ! ( debug )
      call erclos ( 'erfillcf420' )

      end subroutine erfillcf420
