      subroutine erfillcf801
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iincof/rincof for boundary elements of type 801
!
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     fill
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmodulecomio
      use sepmodulestrings
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!                    they are not
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLCFRULE   Fill array iincof/rincof part integration rule and
!                    coordinate system
!     ERFILLDEFCOF1  Check if keyword is found in list of constants, scalars or
!                    vectors
!                    If so fill value in iincof/rincof
!     ERFILLDEFCOF3  Check if keyword is found in list of constants or scalars
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The default input is:
!     icoef1 = typenatbnd
!     icoef3 = integration_rule
!     icoef4 = coordinate_system
!     coef 6 = diff_sigma
!     coef 7 = diff_flux
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfillcf801' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillcf801'
         write(irefwr,1) 'iposin, iposin', iposin, iposin
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icoef = 1  ! TYEPNATBND

      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         i = 1
         call erfilldefcof3 ( codenatbnd(i)(1:ncodenatbnd(i)) )

      end if  ! ( imark(icoef)==0 )

      do i = 2, 3

!     --- Coefficient 6, 7 (diff_sigma, diff_flux)

         icoef = i+4
         if ( imark(icoef)==0 ) then

!        --- Coefficient icoef has not been filled

            call erfilldefcof1 ( codenatbnd(i)(1:ncodenatbnd(i)) )

         end if  ! ( imark(icoef)==0 )

      end do  ! i = 2, 3

!     --- integration rule and coordinate system

      call erfillcfrule

      iincof(iposin+1) = -2        ! end of information

1000  call erclos ( 'erfillcf801' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iposin, iposin', iposin, iposin
         write(irefwr,*) 'End erfillcf801'

      end if  ! ( debug )

      end subroutine erfillcf801
