      subroutine erfillcfconv ( icoeflow )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iincof/rincof for velocity part of elements of type 800
!
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     fill
!     element_group
!     velocity
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulestrings
      use sepmodulecoeffs
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: icoeflow

!     icoeflow       i    Lowest coefficient for velocity
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLDEFCOF1  Check if keyword is found in list of constants, scalars or
!                    vectors
!                    If so fill value in iincof/rincof
!     ERFILLDEFCOF2  Check if keyword is found in list of vectors
!                    If so fill value in iincof/rincof
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfillcfconv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillcfconv'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( imark(icoeflow)==0 ) then

!     --- Coefficient icoef has not been filled

         if ( ndim==1 ) then

!        --- ndim = 1, only one component

            icoef = icoeflow
            call erfilldefcof1 ( codevel(1) )

         else

!        --- ndim>1, more components

            icoef = icoeflow
            call erfilldefcof2 ( codevel(1) )

         end if  ! ( ndim==1 )
         if ( debug ) write(irefwr,1) 'iposin, iposin/2', iposin, iposin

      end if  ! ( imark(icoeflow)==0 )

!     --- Coefficient icoeflow (u_velocity)

      icoef = icoeflow
      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof1 ( codevel(2) )
         if ( debug ) write(irefwr,1) 'iposin, iposin/3', iposin, iposin

      end if  ! ( imark(icoef)==0 )

      if ( ndim>1 ) then

!     --- Coefficient 13 (v_velocity)

         icoef = icoeflow+1
         if ( imark(icoef)==0 ) then

!        --- Coefficient icoef has not been filled

            call erfilldefcof1 ( codevel(3) )

         end if  ! ( imark(icoef)==0 )

      end if  ! ( ndim>1 )
      if ( debug ) write(irefwr,1) 'iposin, iposin/4', iposin, iposin

      if ( ndim>2 ) then

!     --- Coefficient 14 (w_velocity)

         icoef = icoeflow+2
         if ( imark(icoef)==0 ) then

!        --- Coefficient icoef has not been filled

            call erfilldefcof1 ( codevel(4) )

         end if  ! ( imark(icoef)==0 )

      end if  ! ( ndim>2 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erfillcfconv'

      end if  ! ( debug )
      call erclos ( 'erfillcfconv' )

      end subroutine erfillcfconv
