      subroutine erfillcfrhocp ( icoefdens )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iincof/rincof for rho cp part of elements of type 800/7xx
!     2 consecutive positions are filled
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     fill
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmodulecomio
      use sepmodulestrings
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: icoefdens

!     icoefdens      i    Coefficient sequence number for density
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLDEFCOF1  Check if keyword is found in list of constants, scalars or
!                    vectors
!                    If so fill value in iincof/rincof
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfillcfrhocp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillcfrhocp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Coefficient icoefdens (density)

      icoef = icoefdens
      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof1 ( codegen(5)(1:ncodegen(5)) )

      end if  ! ( imark(icoef)==0 )

!     --- Coefficient icoefdens+1 (heat_capacity)

      icoef = icoef+1
      if ( imark(icoef)==0 ) then

!     --- Coefficient icoef has not been filled

         call erfilldefcof1 ( codegen(8)(1:ncodegen(8)) )

      end if  ! ( imark(icoef)==0 )

1000  call erclos ( 'erfillcfrhocp' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erfillcfrhocp'

      end if  ! ( debug )

      end subroutine erfillcfrhocp
