      subroutine erfillcofs
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 24-07-2015 Extension with type 220
!        version  1.3  date 05-03-2015 Extension with type 912
!        version  1.2  date 16-02-2015 Extension with type 814
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays iincof/rincof with default values for standard
!     elements
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     fill
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmoduleactual
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLCF152    Fill array iincof/rincof for elements of type 152
!     ERFILLCF220    Fill array iincof/rincof for elements of type 220 (beam)
!     ERFILLCF250    Fill array iincof/rincof for elements of type 250
!     ERFILLCF251    Fill array iincof/rincof for elements of type 251
!     ERFILLCF304    Fill array iincof/rincof for elements of type 304
!     ERFILLCF325    Fill array iincof/rincof for elements of type 325
!     ERFILLCF420    Fill array iincof/rincof for elements of type 420
!     ERFILLCF700    Fill array iincof/rincof for elements of type 700 (Laplace)
!     ERFILLCF701    Fill array iincof/rincof for elements of type 701 (Poisson)
!     ERFILLCF702    Fill array iincof/rincof for elements of type 702
!     ERFILLCF703    Fill array iincof/rincof for elements of type 703
!     ERFILLCF705    Fill array iincof/rincof for elements of type 705
!     ERFILLCF801    Fill array iincof/rincof for elements of type 801
!     ERFILLCF802    Fill array iincof/rincof for elements of type 802
!     ERFILLCF803    Fill array iincof/rincof for elements of type 803
!     ERFILLCF814    Fill array iincof/rincof for elements of type 814
!     ERFILLCF816    Fill array iincof/rincof for elements of type 816
!     ERFILLCF900    Fill array iincof/rincof for elements of type 900
!     ERFILLCF910    Fill array iincof/rincof for elements of type 910
!     ERFILLCF912    Fill array iincof/rincof for elements of type 912
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      43   Type number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfillcofs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillcofs'
         write(irefwr,1) 'typenr, ndim, iposin, iposrn, ielgrp', &
                          typenr, ndim, iposin, iposrn, ielgrp
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case (typenr)

         case(152)

            call erfillcf152

         case(220)

            call erfillcf220

         case(250)

            call erfillcf250

         case(251)

            call erfillcf251

         case(304)

            call erfillcf304

         case(325)

            call erfillcf325

         case(420)

            call erfillcf420

         case(700)

            call erfillcf700

         case(701)

            call erfillcf701

         case(702)

            call erfillcf702

         case(703)

            call erfillcf703

         case(705, 800)

            call erfillcf705

         case(801)

            call erfillcf801

         case(802)

            call erfillcf802

         case(803)

            call erfillcf803

         case(814)

            call erfillcf814

         case(816)

            call erfillcf816

         case(900:904)

            call erfillcf900

         case(910, 911)

            call erfillcf910

         case(912)

            call erfillcf912

         case default

!        --- Not yet implemented

            call errint ( typenr, 1 )
            call errint ( ielgrp, 2 )
            call errsub ( 43, 2, 0, 0 )

      end select  ! case ( typenr )

1000  call erclos ( 'erfillcofs' )
      if ( debug ) then

!     --- Debug information

         call prinin ( iincof, iposin, 'iincof' )
         call prinrl ( rincof, iposrn, 'rincof' )
         write(irefwr,*) 'End erfillcofs'

      end if  ! ( debug )

      end subroutine erfillcofs
