      subroutine erfilldefcof1 ( namecof )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if keyword is found in list of constants, scalars or vectors.
!     If so fill value in iincof/rincof
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     default
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=* ) namecof

!     icoef          i    Coefficient sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseq
      double precision value
      logical :: found

!                    they are not
!     found          If true the name has been found
!     iseq           Sequence number
!     value          value of the coefficient
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      logical :: getconstseq
      integer :: getnamevar

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETCONSTSEQ    Check if namecof belongs to the set of constants, &
!                    put the result in getconstseq
!     GETNAMEVAR     Return with sequence number of variable in scalars
!     GETVECNAMESEQ  Get sequence number of vector with specific name
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfilldefcof1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfilldefcof1'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check if the name is in the set of constants

      found = getconstseq ( namecof, value )

      if ( debug ) write(irefwr,*) 'found ', found
      if ( found ) then

!     --- Constant found, fill iincof/rincof

         iincof(iposin+1) = 2        ! Type input = real
         iincof(iposin+2) = icoef    ! Sequence number coefficient
         iposrn = iposrn+1
         rincof(iposrn) = value
         iincof(iposin+3) = iposrn   ! Position in rincof
         iposin = iposin+3

      else

!     --- Not a constant found, check if it is a variable

         iseq = getnamevar ( namecof )
         if ( debug ) write(irefwr,1) 'iseq', iseq
         if ( iseq>0 ) then

!        --- variable found, fill iincof

            iincof(iposin+1) = 3        ! Type input = real
            iincof(iposin+2) = icoef    ! Sequence number coefficient
            iincof(iposin+3) = iseq+100000000    ! Sequence number variable
            iposin = iposin+3

         else

!        --- not a variable found, check if it is a vector

            call getvecnameseq ( namecof, iseq )
            if ( debug ) write(irefwr,1) 'iseq/2', iseq

            if ( iseq>0 ) then

!           --- Vector found, fill iincof

               iincof(iposin+1) = 6        ! Type input = vector
               iincof(iposin+2) = icoef    ! Sequence number coefficient
               iincof(iposin+3) = iseq     ! Sequence number vector
               iincof(iposin+4) = 0        ! Degree of freedom
               iposin = iposin+4

            end if  ! ( iseq>0 )

         end if  ! ( iseq>0 )

      end if  ! ( iseq>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erfilldefcof1'

      end if  ! ( debug )
      call erclos ( 'erfilldefcof1' )

      end subroutine erfilldefcof1
