      subroutine erfilldefcof3 ( namecof )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-12-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if keyword is found in list of constants or scalars
!     If so fill value in iincof
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     default
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=* ) namecof

!     namecof        i    Name of Coefficient
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseq
      double precision value
      logical :: found

!                    they are not
!     found          If true the name has been found
!     iseq           Sequence number
!     value          value of the coefficient
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      logical :: getconstseq
      integer :: getnamevar

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETCONSTSEQ    Check if namecof belongs to the set of constants, &
!                    put the result in getconstseq
!     GETNAMEVAR     Return with sequence number of variable in scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfilldefcof3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfilldefcof3'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check if the name is in the set of constants

      found = getconstseq ( namecof, value )
      if ( found ) then

!     --- Constant found, fill iincof/rincof

         iincof(iposin+1) = 1        ! Type input = integer
         iincof(iposin+2) = icoef    ! Sequence number coefficient
         iincof(iposin+3) = nint(value)
         iposin = iposin+3

      else

!     --- Not a constant found, check if it is a variable

         iseq = getnamevar ( namecof )
         if ( iseq>0 ) then

!        --- variable found, fill iincof

            iincof(iposin+1) = 1        ! Type input = integer
            iincof(iposin+2) = icoef    ! Sequence number coefficient
            iincof(iposin+3) = iseq+100000000    ! Sequence number variable
            iposin = iposin+3

         end if  ! ( iseq>0 )

      end if  ! ( found )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erfilldefcof3'

      end if  ! ( debug )
      call erclos ( 'erfilldefcof3' )

      end subroutine erfilldefcof3
