      subroutine erinti
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 26-08-2016 use sepmoduleinput
!        version  3.0  date 26-08-2011 Remove kprob
!        version  2.4  date 08-04-2002 Extension of iinput
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read input concerning the integrals from the standard input file
!     Store the result in sepmoduleinput
! **********************************************************************
!
!                       KEYWORDS
!
!     integral
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleinput
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nparm
      integer, pointer :: iinint(:)
      parameter ( nparm=6)
      double precision :: dummy

!     dummy          Dummy parameter
!     iinint         Integer input array
!     nparm          Number of entries in iinint IHELP excepted
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOPYINPUT    Copy input read by a read subroutine into buffer array
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     READIN         Read the input for the subroutine INTEGR
! **********************************************************************
!
!                       I/O
!
!    Input may be read from the standard input file
!    The first record is interpreted by this subroutine, the reading
!    of the rest of the records is performed by subroutine READ03
!    The first record must have the following shape:
!    INTEGRAL [,PROBLEM = j], [SEQUENCE_NUMBER = k]
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   reserve space for input arrays
!   Read rest of input until END has been found
!   Make input arrays permanent and store references in isepi3
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erinti' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erinti'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read input and store in arrays iinint

      allocate ( iinint(nparm+nelgrp), stat=error )
      if ( error/=0 ) call eralloc ( error, nparm+nelgrp, 'iinint' )
      if ( ierror/=0 ) go to 1000

      iseqinti = iseqinti + 1

      call readin ( iinint, 2, iseqinti, nelgrp )
      if ( debug ) call prinin ( iinint, nparm+nelgrp, 'iinint' )

!     --- Make input arrays permanent

      call ercopyinput ( 13, iseqinti, 'INTEGRALS', iinint, &
                         dummy, iinint, nparm+nelgrp, 0, 0 )

      deallocate ( iinint, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'iinint' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erinti'

      end if  ! ( debug )
      call erclos ( 'erinti' )

      end
