      subroutine eroutp
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 26-08-2016 use sepmoduleinput
!        version  2.9  date 09-10-2013 Extension of array iinput
!        version  2.8  date 09-10-2013 Extension of array iinput
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Read input concerning the module output from the standard input file
!    Store the result in sepmoduleinput
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleinput
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nlist=26
      integer nprob, niinp, numvec, list(nlist)
      integer, allocatable :: iinput(:)
      double precision :: dummy
      save list

!     dummy          Dummy parameter
!     iinput         integer input array
!     list           Contains the sequence numbers of the arrays to be stored
!     niinp          Maximal permitted number of items to be stored in IINPUT
!                    The actual length of IINPUT is 7 x niinp
!     nlist          Length of array list
!     nprob          Number of various problems
!     numvec         Number of vectors stored in isol
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERCOPYINPUT    Copy input read by a read subroutine into sepmoduleinput
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     READ04         Read card input for subroutine OUTPUT
! **********************************************************************
!
!                       I/O
!
!    Input may be read from the standard input file
!    The first record is interpreted by this subroutine, the reading
!    of the rest of the records is performed by subroutine READ03
!    The first record must have the following shape:
!    SOLVE [,PROBLEM = j], [SEQUENCE_NUMBER = k]
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1872   Sequence number used more than once
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Interpret first record
!   Check IPROB and ISEQ
!   reserve space for input arrays
!   Read rest of input until END has been found
!   Make input arrays permanent and store references in isepi3
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data list /nlist*0/
! ======================================================================
!
      call eropen ( 'eroutp' )

!     --- Read input and store in array iinput
!         reserve space in buffer array for array IINPUT

      nprob  = 0
      niinp  = 1000
      iseqoutp = iseqoutp + 1

      allocate ( iinput(15*niinp), stat = error )
      if ( error/=0 ) call eralloc ( error, 15*niinp, 'iinput' )
      if ( ierror/=0 ) go to 1000

      call read04 ( iinput, niinp, nprob, 2, list, nlist, &
                    iseqoutp )
      numvec = iinput(1)

!     --- Make input arrays permanent

      call ercopyinput ( 9, iseqoutp, 'ichangecr', iinput, &
                         dummy, iinput, 15*(numvec+1), 0, 0 )

      deallocate ( iinput, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iinput' )

1000  call erclos ( 'eroutp' )
      end
