      subroutine erreadcr ( isubr, iseqnr, istart, inpcre, inforinpcre, &
                            rincre, leninp, lenrin, nstart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 26-08-2016 use sepmoduleinput
!        version  1.1  date 22-11-2012 Long integers
!        version  1.0  date 26-08-2011
!
!   copyright (c) 2011-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Reads the standard input file for subroutine CREATE and stores the
!     information in the arrays INPCRE and RINCRE
!     These arrays are stored in ibuffr/buffer
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     essential
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isubr, iseqnr, leninp, &
              lenrin, nstart, istart(nstart), inpcre(leninp), &
              inforinpcre(lenrin)
      double precision rincre(lenrin)

!     inforinpcre    i    Array of length of rincre that contains a 0 if the
!                         number stored in rincre is a real and a reference
!                         to scalars if it is a scalar
!     inpcre         i    The integer input read is stored in array INPCRE from
!                         position 1. Hence in the call INPCRE(ISTART), with
!                         ISTART the starting position must be used.
!                         INPCRE will be filled in correspondence with array
!                         INPCRE for subroutine CREATV
!     iseqnr         i    Sequence number with respect to the storage in array
!                         ISEPIN
!     istart         i    First position used at a part
!     isubr          i    Indicates the type of calling subroutine
!                         Possible values
!                         1:  CREATE
!     leninp         i    Computed length of integer input array
!     lenrin         i    Computed length of array user
!     nstart         i    Number of positions in array ISTART
!     rincre         i    Double precision array containing the values of the
!                         constant if KFUN<0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numveclc, i
      integer, allocatable :: inpcrecop(:)
      logical renum

!                    they are not
!     i              Counting variable
!     inpcrecop      Contains copy of inpcre
!     numveclc         Number of vectors stored in isol
!     renum          Indicates if array INPCRE must be renumbered (true)
!                    or not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0049         Reorder the input array INPCRE of subroutine CREATE
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     READCR         Read card input for subroutine CREATE
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erreadcr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erreadcr'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read input and store in arrays INPCRE and RINCRE

      call readcr ( numveclc, istart, inpcre, rincre, nstart, &
                    leninp, lenrin, isubr, iseqnr, inforinpcre )
      if ( ierror/=0 ) go to 1000

!     --- Reorder array INPCRE if necessary

      if ( numveclc>1 .and. lenrin>0 ) then

!     --- numveclc>1, hence possibly incorrect sequence

         renum = .false.
         do i = 1, numveclc-1
            if ( istart(i)>istart(i+1) ) renum = .true.
         end do

         if ( renum ) then

!        --- Renumbering of sequence necessary

            allocate ( inpcrecop(leninp), stat=error )
            if ( error/=0 ) call eralloc ( error, leninp, 'inpcrecop' )
            if ( ierror/=0 ) go to 1000

            call er0049 ( inpcre, istart, numveclc, inpcrecop )
            inpcre(1:leninp) = inpcrecop

            deallocate ( inpcrecop, stat=error )
            if ( error/=0 ) call erdealloc ( error, 'inpcrecop' )

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erreadcr'

      end if  ! ( debug )
      call erclos ( 'erreadcr' )

      end subroutine erreadcr
