      subroutine ersrnod2 ( ifirst, ilast, nodes, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 27-09-2010 Make work arrays local
!        version  1.0  date 12-07-1999
!
!   copyright (c) 1999-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store all nodes in the surfaces ifirst to ilast in array nodes
!     The number of nodes in nodes is stored in nnodes
!     No volume elements may be defined (kmesh(13)==0)
!     Information is retrieved from kmesh part c and t
!     Compare with ersurnod
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ifirst, ilast, nodes(*), nnodes

!     ifirst     i    First surface number
!     ilast      i    Last surface number
!     nnodes     o    Number of nodes in nodes
!     nodes      o    Contains the node numbers in the surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isurnr, istart, inode, iend
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     iend
!     inode          Node sequence number
!     istart         First position of a surface in kmeshn minus 1
!     isurnr         Surface sequence number
!     iwork          integer work array of length npoint
!                    iwork(i) = 0, means node i has not yet been filled
!                    iwork(i) = 1, means node i has been filled
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ersrnod2' )

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      iwork = 0

      nnodes = 0

!     --- Loop over surfaces

      do isurnr = ifirst, ilast

!     --- For all nodes inode in surface

         istart = kmesht(1,isurnr)-1
         iend = kmesht(2,isurnr)

         do i = istart*npelm+1, iend*npelm
            inode = kmeshc(i)
            if ( iwork(inode)==0 ) then

!           --- iwork(inode)=0

               iwork(inode)=1
               nnodes = nnodes+1
               nodes(nnodes) = inode

            end if
         end do

      end do

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'ersrnod2' )

      end
