      subroutine getintarr ( intname, intarray, lenarr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 22-11-2004 Fortran 90
!        version  1.0  date 28-04-2000
!
!   copyright (c) 2000-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Get the values of an integer array in the block constants with name intname
!    The indices are given by a sequence number behind the name
!    for example intname1 intname2 intname4
!    If the name does not exist an error message is given
! **********************************************************************
!
!                       KEYWORDS
!
!     name
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*) intname
      integer intarray(*), lenarr

!     intarray      o    Array with integer values
!     intname       i    Name of integer
!     lenarr        o    largest index in array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nch, nch1, intseqnm, j, intind, ndigits
      character (len=7) formt1
      character (len=80) iname1, iname2

!     formt1         Character string to store the format necessary for reading
!                    of the number with an internal read
!     i              Counting variable
!     iname1         Name of integer translated to capitals
!     iname2         Name of integer translated to capitals
!     intind         Array index of integer
!     intseqnm       Sequence number of integer with respect to array
!                    chincons
!     j              Counting variable
!     nch            Number of characters in a string
!     nch1           Number of characters in first word
!     ndigits        Nuber of digits
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKUPC         Translate all letters to upper case
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2155   integer name not found
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find position of name in chincons
!     If ( name in chincons ) then
!        getint := value
!     else
!        error message
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      intseqnm = 0
      lenarr = 0
      nch1 = index ( intname, ' ' ) -1
      if ( nch1==-1 ) nch1 = len(intname)
      iname1 = intname(1:nch1)
      call askupc(iname1)
      do i = 1, numints
         nch = index ( chincons(i), ' ' ) -1
         if ( nch<0 ) nch = 32
         if ( nch>nch1 ) then
            iname2 = chincons(i)(1:nch)
            call askupc(iname2)
            if ( iname1(1:nch1)==iname2(1:nch1) ) then

!           --- name of integer found, now check if it is followed by a number

               ndigits = 0
               do j = nch1+1, nch
                  if ( iname2(j:j)>='0' .and. iname2(j:j)<='9' ) then
                     ndigits = ndigits+1
                  else
                     go to 100
                  end if
               end do
100            if ( ndigits>0 ) then
                  intseqnm = i
                  formt1(1:4) = '(i )'
                  write(formt1(3:3),110) ndigits
110               format(i1)
                  read ( iname2(nch1+1:nch), fmt=formt1 ) intind
                  intarray(intind) = incons(i)
                  lenarr = max(lenarr,intind)
               end if

            end if
         end if
      end do
      if ( intseqnm==0 ) then
         call eropen ( 'getintarr' )
         call errchr ( intname, 1 )
         call errwar ( 2155, 0, 0, 1 )
         call erclos ( 'getintarr' )
      end if
      end
