      function getnameint ( intname )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 10-04-2013 Loop of indexed reals
!        version  1.2  date 20-04-2012 Indexed integers
!        version  1.1  date 22-11-2004 Fortran 90
!
!   copyright (c) 2000-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Get the sequence number corresponding to the integer in the block constants
!    with name intname
!    If the name does not exist getnameint gets the value 0
! **********************************************************************
!
!                       KEYWORDS
!
!     name
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulecomio
      use sepmoduletexts
      use sepmoduleinput
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*) intname
      integer getnameint

!     getnameint    o   Sequence number of integer with respect to array
!                       chincons
!     intname       i   Name of integer
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nch, nch1
      character (len=80) iname1, iname2

!     i              General loop variable
!     iname1         Name of integer translated to capitals
!     iname2         Name of integer translated to capitals
!     nch            Number of characters in a string
!     nch1           Number of characters in a string
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKUPC         Translate all letters to upper case
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3126   loop variable in integers array zero
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For i := 1 (1) numints do
!        if ( intname = chincons(i) ) then
!           getnameint := i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getnameint' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getnameint'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      getnameint = 0
      nch1 = index ( intname, ' ' ) -1
      if ( nch1==-1 ) nch1 = len(intname)
      iname1 = intname(1:nch1)
      call askupc(iname1)
      do i = 1, numints
         nch = ncharcons(i)
         if ( nch1==nch ) then
            iname2 = chincons(i)(1:nch1)
            call askupc(iname2)
            if ( iname1(1:nch1)==iname2(1:nch1) ) then

!           --- name of integer found

               getnameint = i

               if ( dimcons(i)>0 ) then

!              --- Integer is an array

                  if ( iseqarray>0 ) then

!                 --- constants array

                     getnameint = i+iseqarray-1

                  else if ( iseqarray<0 ) then

!                 --- constants array, but no sequence number given
!                     in this case an array is permitted because iseqarray<0

                     getnameint = i+5000

                  else

!                 --- error loop varibale is 0

                     call errchr ( iname1(1:nch1), 1 )
                     call errsub ( 3126, 0, 0, 1 )
                     go to 1000

                  end if  ! ( iseqarray>0 )

               end if  ! ( dimcons(i)>1 )
               exit

            end if
         end if
      end do

1000  call erclos ( 'getnameint' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End getnameint'

      end if  ! ( debug )

      end
