      subroutine getrealarr ( realname, realarray, lenarr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 10-04-2013 Complete new version
!        version  1.1  date 22-11-2004 Fortran 90
!        version  1.0  date 28-04-2000
!
!   copyright (c) 2000-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Get the values of a real array in the block constants with name realname
!    The indices are given by a sequence number behind the name
!    for example realname1 realname2 realname4
!    If the name does not exist an error message is given
! **********************************************************************
!
!                       KEYWORDS
!
!     name
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulecomio
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*) realname
      double precision realarray(*)
      integer lenarr

!     lenarr        o    largest index in array
!     realarray     o    Array with real values
!     realname      i    Name of real
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nch, nch1
      character (len=80) iname1, iname2

!     i              Counting variable
!     iname1         Name of integer translated to capitals
!     iname2         Name of integer translated to capitals
!     nch            Number of characters in a string
!     nch1           Number of characters in first word
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKUPC         Translate all letters to upper case
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2156   real name not found
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find position of name in chrlcons
!     If ( name in chrlcons ) then
!        getint := value
!     else
!        error message
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'getrealarr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from getrealarr'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      lenarr = 0
      nch1 = index ( realname, ' ' ) -1
      if ( nch1==-1 ) nch1 = len(realname)
      iname1 = realname(1:nch1)
      call askupc(iname1)

      if ( debug ) write(irefwr,*) 'iname1 ', iname1(1:nch1)

      i = 1
      do while ( i<=numreals )

         nch = index ( chrlcons(i), ' ' ) -1
         if ( nch<0 ) nch = 32

         if ( debug ) write(irefwr,1) 'i, nch, nch1', i, nch, nch1

         if ( nch>nch1 ) then
            iname2 = chrlcons(i)(1:nch)
            call askupc(iname2)

            if ( debug ) then
               write(irefwr,*) 'iname2 ', iname2(1:nch)
            end if  ! ( debug )

            if ( iname1(1:nch1)==iname2(1:nch1) ) then

!           --- name of real found, now check if it is followed by a number

               lenarr = max(1,dimcons(maxints+i))
               realarray(1:lenarr) = rlcons(i:i+lenarr-1)
               i = numreals+1

            else

               i = i+max(1,dimcons(maxints+i))

            end if  ! ( iname1(1:nch1)==iname2(1:nch1) )

         else

            i = i+max(1,dimcons(maxints+i))

         end if  ! ( nch>nch1 )

      end do  ! while ( i<=numreals )

      if ( lenarr==0 ) then
         call errchr ( realname, 1 )
         call errwar ( 2156, 0, 0, 1 )
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'lenarr', lenarr
         write(irefwr,*) 'End getrealarr'

      end if  ! ( debug )
      call erclos ( 'getrealarr' )

      end
