      subroutine incv00
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.1  date 08-12-2006 New call to incv02
!        version  4.0  date 06-10-2006 New call to incv01
!
!   copyright (c) 1993-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    program incv00 is an interactive program to make a conversion
!    of SEPRAN files from ASCII to binary or vice-versa
!    At this moment SEPCONV is able to handle the following files:
!    "meshoutput"
!    "sepcomp.out"
!    "sepplot.*"
! **********************************************************************
!
!                       KEYWORDS
!
!     conversion
!     interactive
!     mesh
!     file
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduleread
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical finished

!     finished   Indicates if the loop has been finished (true) or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTEXT         Produce menu
!     FINISH         Stop SEPRAN
!     INCV01         Convert meshoutput file from binary to ASCII and vice-versa
!     INCV02         Convert sepcomp.out file from binary to ASCII and
!                    vice-versa
!     INCV03         Convert sepplot.*** files from binary to ASCII and
!                    vice-versa
!     INICLS         Clear screen
!     INIFIL         Open files
!     INITCB         Initializes SEPRAN machine-independent common blocks
!     INITMD         Set machine-dependent quantities
!     SEPSTART       Standard SEPRAN starting subroutine
!     STARTSEPRAN    Create buffer array and fill part of sepmodulebuf
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Initialise machine dependent common blocks
!   open files
!   Set irefre and irefwr for interactive input and output
!   Start SEPRAN
!   finished := false
!   While (not finished ) do
!      print main menu
!      read answer
!      if ( answer = a ) then
!         Transform binary meshoutput file to ASCII file
!      else if ( answer = b ) then
!         Transform ASCII meshoutput file to binary file
!      else if ( answer = c ) then
!         Transform binary sepcomp.out file to ASCII file
!      else if ( answer = d ) then
!         Transform ASCII sepcomp.out file to binary file
!      else if ( answer = e ) then
!         Transform binary sepplot.*** files to ASCII file
!      else if ( answer = f ) then
!         Transform ASCII sepplot.*** files to binary file
!      else if ( answer = q ) then
!         finished := true
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      kmesh => kmesh2d(1)
      kprob => kprob2d(1)
      if ( startcalled/=99991 ) call startsepran
      call initcb
      call initmd

!     --- Open read and write files

      irefre = 5
      irefwr = 6

!     --- Open standard elements file and error message file

      call inifil(1)
      call inifil(4)
      call inifil(17)

!     --- Standard SEPRAN start subroutine

      call sepstart ( 10100, 1, 0, 0 )
      ioutp = -1
      call eropen ( 'incv00' )

!     --- While loop finished

      finished = .false.
100   if ( .not. finished ) then

!     --- Write main menu

         ierror = 0
         nerror = 0
         call inicls
         call ertext ( 13, 0, 0, 0 )
         if ( ierror/=0 ) go to 1000

!        --- Read answer

         ityprd = 1
         call asknxt
         if ( ityprd/=4 ) go to 100

!        --- Character found, detect

         if ( string(1:1)=='A' ) then

!        --- Translate binary meshoutput file to ASCII

            call incv01 ( 0 )

         else if ( string(1:1)=='B' ) then

!        --- Translate ASCII meshoutput file to binary

            call incv01 ( 1 )

         else if ( string(1:1)=='C' ) then

!        --- Translate binary sepcomp.out file to ASCII

            call incv02 ( 0 )

         else if ( string(1:1)=='D' ) then

!        --- Translate ASCII sepcomp.out file to binary

            call incv02 ( 1 )

         else if ( string(1:1)=='E' ) then

!        --- Translate binary sepplot.*** files to ASCII

            call incv03 ( 0 )

         else if ( string(1:1)=='F' ) then

!        --- Translate ASCII sepplot.*** files to binary

            call incv03 ( 1 )

         else if ( string(1:1)=='Q' ) then

!        --- Quit

            finished = .true.

         else

!        --- unrecognized option, repeat

           go to 100

         end if

         go to 100

      end if
1000  call erclos ( 'incv00' )
      call finish ( 0 )
      end subroutine incv00
