      subroutine initmx ( ndim, npoint, coor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 04-01-2001 Set ymin and zmin to 0 if not used
!        version  1.3  date 21-01-1999 Include commons
!        version  1.2  date 21-12-1991 Set zeyerf for 2D
!
!   copyright (c) 1991-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   subroutine initmx is used to initialise xmin, xmax, etc
!   with the aid of array kmesh
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, npoint
      double precision coor(ndim,npoint)

!     coor    i   Co-ordinates
!     ndim    i   Dimension of space
!     npoint  i   Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision xmean, ymean, zmean

!    i      General loop variable
!    xmean  X-coordinate of centroid
!    ymean  Y-coordinate of centroid
!    zmean  Z-coordinate of centroid
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      xminrf = rinfin
      xmaxrf = -rinfin
      yminrf = rinfin
      ymaxrf = -rinfin
      zminrf = rinfin
      zmaxrf = -rinfin
      if ( ndim.le.2 ) then
         zminrf = 0d0
         zmaxrf = 0d0
      end if
      if ( ndim.le.1 ) then
         yminrf = 0d0
         ymaxrf = 0d0
      end if
      xmean = 0d0
      ymean = 0d0
      zmean = 0d0
      do i = 1, npoint
         xminrf = min ( xminrf, coor(1,i) )
         xmaxrf = max ( xmaxrf, coor(1,i) )
         xmean  = xmean + coor(1,i)
      end do
      xeyerf = xmean/npoint + ( xmaxrf-xminrf )

      if ( ndim.gt.1 ) then
         do i = 1, npoint
            yminrf = min ( yminrf, coor(2,i) )
            ymaxrf = max ( ymaxrf, coor(2,i) )
            ymean  = ymean + coor(2,i)
         end do
         yeyerf = ymean/npoint - ( ymaxrf-yminrf )
      end if

      if ( ndim.gt.2 ) then
         do i = 1, npoint
            zminrf = min ( zminrf, coor(3,i) )
            zmaxrf = max ( zmaxrf, coor(3,i) )
            zmean  = zmean + coor(3,i)
         end do
         zeyerf = zmean/npoint + ( zmaxrf-zminrf )
      else
         zeyerf = 0d0
      end if

      end
