      subroutine insp00
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.7  date 25-11-2012 Long integers
!        version  2.6  date 30-08-2006 New definition of cpost5
!        version  2.5  date 18-05-2005 Fortran 90
!
!   copyright (c) 1990-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    The task of INSP00 is to plot a mesh or quantities corresponding to
!    a mesh, for example curves or user points
!    subroutine INSP00 is a help subroutine of subroutine SEPPS1
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     plot_mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icolsv

!     i              Counting variable
!     icolsv         Help parameter to store the value of dcolor
!                    temporarily
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLAFP7         Plot text using absolute co-ordinates
!     PLFR20         Plot curves and eventually one of the surfaces
!     PLLEGN         Plot bottom text and axis
!     SEPPLOTCU      Plot curves and user points
!     SEPPLOTMS      plot mesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      67   ielgrp has wrong value
!     288   volume number has incorrect value
!    2204   All element groups are skipped
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'insp00' )

!     --- Manipulate the parameter jtimes in order to get the correct legenda

      icolsv = dcolor
      dcolor = max(dcolor,1)

!     --- Perform actions depending on iinputpst(1)

      if ( iinputpst(1)==101 .or. iinputpst(1)==104 ) then

!     --- iinputpst(1)   = 101  Subroutine PLOTMS

         iconstpst(1)    = 20
         iconstpst(2)    = iinputpst(33)
         iconstpst(3)    = iinputpst(34)
         iconstpst(4)    = iinputpst(35)
         iconstpst(5)    = iinputpst(29)
         iconstpst(6)    = iinputpst(31)
         if ( iinputpst(1)==104 ) iconstpst(6) = 1
         iconstpst(7)    = iinputpst(32)
         iconstpst(8)    = iinputpst(36)
         if ( iinputpst(1)==101 ) then
            iconstpst(9) = 0
         else
            iconstpst(9) = 1
         end if
         iconstpst(10)   = iinputpst(40)
         iconstpst(11)   = iinputpst(53)
         iconstpst(12)   = iinputpst(26)
         iconstpst(13)   = iinputpst(27)
         iconstpst(14)   = iinputpst(83)
         iconstpst(15)   = iinputpst(84)
         iconstpst(16)   = iinputpst(85)
         iconstpst(17)   = iinputpst(94)
         iconstpst(18)   = iinputpst(95)
         iconstpst(19)   = iinputpst(96)
         iconstpst(20)   = iinputpst(97)
         constpst(1)     = rinputpst(1)
         constpst(2)     = rinputpst(2)
         do i = 1, 6
            constpst(2+i) = rinputpst(9+i)
         end do

!        --- Fill iconstpst with information of element groups
!            if elements must be skipped

         if ( iinputpst(32)>=1 ) then

!        --- elements must be skipped

            if ( associated(iskiparr) ) deallocate(iskiparr)
            allocate ( iskiparr(nelgrp), stat = error )
            if ( error/=0 ) call eralloc ( error, nelgrp, 'iskiparr' )
            if ( ierror/=0 ) go to 1000

            iskiparr = 1
            do i = 1, iinputpst(32)

               if ( igroup(i)>0 .and. igroup(i)<=nelgrp ) then

!              --- skip element group

                 iskiparr(igroup(i)) = 0

               else

!              --- wrong element group

                  call errint ( igroup(i), 1 )
                  call errint ( nelgrp, 2 )
                  call errsub ( 67, 2, 0, 0 )

               end if

            end do
            if ( iinputpst(32)==nelgrp ) then

!           --- all element groups are skipped

               call errsub ( 2204, 0, 0, 0 )

            end if

         else if ( iinputpst(85)>=1 ) then

!        --- volumes must be used

            if ( associated(iskiparr) ) deallocate(iskiparr)
            allocate ( iskiparr(nvolms), stat = error )
            if ( error/=0 ) call eralloc ( error, nvolms, 'iskiparr' )
            if ( ierror/=0 ) go to 1000

            iskiparr = 0
            do i = 1, iinputpst(85)
               if ( igroup(i)>0 .and. igroup(i)<=nvolms ) then

!              --- add volume

                 iskiparr(igroup(i)) = 1

               else

!              --- wrong volume

                  call errint ( igroup(i), 1 )
                  call errint ( nvolms, 2 )
                  call errsub ( 288, 2, 0, 0 )

               end if

            end do

         else

            allocate ( iskiparr(1) )

         end if

         call sepplotms ( iconstpst, constpst, iskiparr )

         deallocate ( iskiparr, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iskiparr' )
         if ( ierror/=0 ) go to 1000

      else if ( iinputpst(1)==102 ) then

!     --- iinputpst(1) = 102  Subroutine PLOTCU (plot user points)

         iconstpst(1) = 1

         ntextx = iinputpst(37)
         ntexty = iinputpst(38)
         call sepplotcu ( iconstpst, rinputpst )

      else if ( iinputpst(1)==103 ) then

!     --- iinputpst(1) = 103  Subroutine PLOTCU (plot curves)

         iconstpst(1) = 2
         iconstpst(2) = iinputpst(31)
         iconstpst(3) = iinputpst(30)*3
         iconstpst(4) = iinputpst(41)
         iconstpst(5) = iinputpst(35)

         ntextx = iinputpst(37)
         ntexty = iinputpst(38)
         call sepplotcu ( iconstpst, rinputpst )

      else if ( iinputpst(1)==105 ) then

!     --- iinputpst(1) = 105  Subroutine PLFR20 (plot a sequence
!                          of all the curves of the mesh and eventually
!                          a surface as wire frame or as coloured solid)

         constpst(1) = rinputpst(1)
         constpst(2) = rinputpst(2)

         do i = 1, 6
            constpst(2+i) = rinputpst(9+i)
         end do

         iconstpst(1) = 8
         iconstpst(2) = iinputpst(53)
         iconstpst(3) = iinputpst(26)
         iconstpst(4) = iinputpst(27)
         iconstpst(5) = iinputpst(60)
         iconstpst(6) = iinputpst(61)
         iconstpst(7) = 0
         iconstpst(8) = iinputpst(62)

         call plfr20 ( iconstpst, constpst )

         iinputpst(62) = iconstpst(8)

         do i = 1, 6
            rinputpst(9+i) = constpst(2+i)
         end do

      end if

!     --- Plot bottom text and axis if necessary

      if ( jtimes==11 ) then
         if ( iinputpst(1)/=101 ) then
            ntextx = iinputpst(37)
            ntexty = iinputpst(38)
            ntext  = iinputpst(39)
            if ( ntext==0 ) then
               ntext = 1
               textid(1)(1:1) = ' '
            end if
            call pllegn ( rinputpst(16) )
         end if
         if ( iinputpst(44)/=0 ) then
            if ( iinputpst(44)==1 ) then
               iinputpst(44) = 2
            end if
            call plafp7 ( xorgid, yorgid, identf, 0d0, .25d0, &
                          iinputpst(45) )
         end if

      end if

!     --- Update jtimes

      jtimes = jtimst
      if ( jtimes==11 .and. iinputpst(1)>=101 .and. &
           iinputpst(1)<130 ) jtimes = 12
      if ( jtimes==10 .and. iinputpst(1)>=101 .and. &
         iinputpst(1)<130 ) call plafep (2, 0d0, 0d0, 0d0, 0d0, 0)
1000  call erclos ( 'insp00' )
      dcolor = icolsv
      end subroutine insp00
