      subroutine insp02sub ( user, iuser, iinput, rinput, &
                             iconst, const, ivec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.8  date 25-11-2012 Long integers
!        version  1.7  date 25-04-2010 New call to plaf05
!        version  1.6  date 07-03-2007 New call to plotvcb1
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     perform the print and plot commands for
!     solution vectors and derived quantities
!     subroutine INSP02 is a help subroutine of subroutine SEPPS1
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     post_processing
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulepost
      use sepmodulekmesh
      use sepmodulemain
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(105), iinput(*), iconst(*), ivec
      double precision user(105), rinput(*),  const(*)

!     const          i    Array of length 10 to store real constants
!     iconst         i    Array of length 105 to store integer constants
!     iinput         i    integer input array of length 100
!     iuser          i    Help array for integer coefficients (particle tracking
!                         and plot field)
!     ivec           i    Sequence number of array to be processed
!     rinput         i    Real input array
!     user           i    Help array for real coefficients (particle tracking
!                         and plot field)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer idummy(1), ichois, ivalue(1), i, number, ncntln, &
              jchois(20), jindcl, icplot(24), icompt, inppri(9), &
              lengt, lengy, jtimessav, itypvc
      double precision multfact, height, dummy(1), rcplot(14), save1, &
                       save2, rinpri(2)
      save icplot, rcplot

!     dummy          Dummy parameter
!     height         Height of a plot, when <0 then in cm, else dimensionless
!     i              Counting variable
!     ichois         Standard choice parameter for several subroutines
!     icompt         indicates how the contour levels must be
!                    computed from NCNTLN in the case NCNTLN<=0
!                    0:  The minimum and maximum level are computed
!                    1:  The minimum level is given in contln(6), the
!                        maximum level must be computed
!                    2:  The maximum level is given in contln(7), the
!                        minimum level must be computed
!                    3:  The minimum level is given in contln(6), the
!                        maximum level in contln(7)
!     icplot         Integer input array for subroutine PLOTCN
!     idummy         Dummy parameter
!     inppri         Integer input for subroutine PRINBN
!     ivalue         Not yet used
!     jchois         choice parameter array for PTRACK:
!                      1)  length of array JCHOIS to be used
!                      2)  type of velocity field given
!                           1  Velocity vector is given on elements
!                           2  Velocity vector is given in points  (default)
!                           3  Fluxes through the edges are given  (elmtyp=650)
!                       3) print indicator
!                           0  Make no print of trajectories       (default)
!                          >0  Make print of particle trajectories on file with
!                               reference number JCHOIS(3)
!                       4) plot indicator
!                           0  Make no plots
!                           1  Make plot of particle trajectories
!                           2  Make plot of mesh and trajectories  (default)
!                       5) direction indicator for parallel projection
!                               (only used for 3-D)
!                           1  Projection on (x,z) plane from y is +inf default
!                           2  Projection on (y,z) plane from x is +inf
!                           3  Projection on (x,z) plane from y is -inf
!                           4  Projection on (y,z) plane from x is -inf
!     jindcl         indicates how array indcol must be filled for routine
!                    plotc2.
!                    Possible values:
!                    0: The colours with numbers 1 to 1+ncntln are used
!                    1: The user defines the colour numbers explicitly
!                    2: The user defines the minimum colour
!                    3: The user defines the minimum and maximum colour
!                    See indcol
!     jtimessav      Help variable to save the value of jtimes
!     lengt          Length of function array "t"
!     lengy          Length of function array "y"
!     multfact       Multiplication factor
!     ncntln         Parameter to compute the number of contour levels
!     number         Parameter indicating which coefficient in common block
!                    celiar must be considered
!     rcplot         Real input array for subroutine PLOTCN
!     rinpri         dummy
!     save1          Help variable to store a constant
!     save2          Help variable to store a constant
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     PLAF05         Plot polygon
!     PLOTF3         Make 3D plot of function defined on 2D mesh using hidden
!                    lines
!     PLOTFN1        Plot function
!     PLOTLN1        Plot interpolated function
!     PLOTVCB1       Plot vectors
!     PLTEXT         Plot text
!     PLTEXTBOX      Plot a text in the little box
!     PLTRAC         compute, print and or plot the trace of a number
!                    of particles in a velocity field
!     PRIAVS         Write solution array or array of special structure to file
!                    sepavs_xxx.yyy (AVS file)
!     PRININ         print 1d integer array
!     PRINPTRACE     Print values of time and coordinates of particle trace
!                    Also compute and print corresponding values of solution
!                    arrays
!     PRINRL         Print 1d real vector
!     PRINTVECT      Print the values of a SEPRAN solution vector or vector of
!                    special structure
!     SEPPFIELD      Plot field line(s)
!     SEPPLOT3C      Make 3D plot of function defined on 2D mesh using colored
!                    faces
!     SEPPLOTCN      General plot contour subroutine
!     SEPVECMAN      Sepran vector manipulations
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'insp02sub' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from insp02sub'
         write(irefwr,1) 'iinput(1)', iinput(1)
         write(irefwr,1) 'jtimes', jtimes
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iinput(1)==2 .or. iinput(1)==3 ) then

!     --- iinput(1) = 2  Subroutine PRINVC

         inppri(1) = 9              ! number of entries filled
         inppri(2) = 1-iinput(66)   ! ICHOICE, indicates the type of print
         if ( iinput(91)/=0 ) then

!        --- map to rectangular grid, use whole mesh

            inppri(3) = 0           ! IDIM

         else

!        --- standard, use idim

            inppri(3) = iinput(67)  ! IDIM

         end if  ! ( iinput(91)/=0 )
         inppri(4) = 0              ! IQUANTITY
         if ( inppri(3)==2 .or. inppri(3)==3) then
            if ( iinput(64)==1 ) then
               inppri(4) = 1
            else if ( iinput(65)==1 ) then
               inppri(4) = 2
            end if
         end if
         inppri(5) = iinput(8)      !  NUMBER
         inppri(6) = max ( min (iinput(10), ndim), 1 )  ! ISEQ
         inppri(7) = jmax           ! IREGION
         inppri(8) = iinput(70)     ! IHEADER
         inppri(9) = iinput(74)     ! ICOOR
         inppri(2) = inppri(2)+10*iinput(103)  ! suppress_nodes
         call printvect ( inppri, isolut, icurvs, names(ivec), rinpri )

      else if ( iinput(1)==10 ) then

!     --- iinput(1) = 10  Subroutine PRIAVS

         call priavs ( itimst, ivec, time(itimst), 0 )

      else if ( iinput(1)==112 .or. iinput(1)==113 ) then

!     --- iinput(1) = 112, 113  Contour plots (PLOTCN)

         ncntln = iinput(6)
         icompt = 0
         jindcl = 0
         if ( iinput(14)==0 ) then

!        --- iinput(14) = 0, the user has not given the contour
!                            levels explicitly. Check if minimum or
!                            maximum level is given.

            if ( iinput(12)==1 ) then

!           --- iinput(12) = 1  Minimum level is stored in rinput(4)
!                               Raise iinput(7)

               contln(6) = rinput(4)
               icompt = 1

            end if

            if ( iinput(13)==1 ) then

!           --- iinput(13) = 1  Maximum level is stored in rinput(5)
!                               Raise iinput(7)

               contln(7) = rinput(5)
               icompt = icompt + 2

            end if

            if ( iinput(6)>0 .and. iinput(12)==0 .and. &
                 iinput(13)==0 ) then

!           --- iinput(6)>0, hence the number of levels is given.
!               Furthermore iinput(12) = 0 and iinput(13) = 0, &
!               which means that not the minimum and maximum values
!               are given. Set ncntln equal to -ncntln-1000

               ncntln = -iinput(6) - 1000

            end if

         end if
         icplot(1) = 24
         icplot(2) = iinput(1)-111
         if ( debug ) &
           write(irefwr,1) 'ncntln, icplot', &
                            ncntln, icplot(1), icplot(2)

         if ( iinput(1)==113 ) then

!        --- iinput(1) = 113,  subroutine PLOTC2

            if ( icompt>=1 ) then

               if ( ncntln>0 ) then
                  ncntln = ncntln - 1
               else if ( ncntln<0 ) then
                  ncntln = ncntln + 1
               end if

            end if
            if ( iinput(62)==1 ) then

!           --- mincolour is given

               jindcl = 2
               indcol(6) = iinput(26)

               if ( iinput(63)==1 ) then

!              --- maxcolour is given as well

                  jindcl = 3
                  indcol(7) = iinput(27)

               end if

            else

!           --- Standard situation

               jindcl = 1

            end if

         end if
         if ( ierror/=0 ) go to 1000

         number = max(1,iinput(8))
         if ( iinput(41)==0 .and. icplot(2)==1 ) icplot(2) = 4
         icplot(3) = number
         icplot(4) = ncntln
         icplot(5) = jindcl
         icplot(6) = iinput(53)
         icplot(7) = iinput(7)
         icplot(8) = iinput(31)
         icplot(9) = iinput(30)
         icplot(10) = 0
         if ( iinput(71)==1 ) then
            icplot(11) = 1
         else
            icplot(11) = 2
         end if
         if ( iinput(72)==1 ) then
            icplot(12) = 2
         else
            icplot(12) = 1
         end if
         icplot(13) = 0
         icplot(14) = 2
         icplot(15) = 0
         icplot(16) = 0
         icplot(17) = 0
         icplot(18) = 0
         icplot(19) = iinput(92)
         icplot(20) = 0
         icplot(21) = icompt
         icplot(22) = 1
         icplot(23) = iinput(78)
         icplot(24) = iinput(79)
         rcplot(1)  = rinput(1)
         rcplot(2)  = rinput(2)
         rcplot(3)  = 1d0
         rcplot(4)  = rinput(10)
         rcplot(5)  = rinput(11)
         rcplot(6)  = rinput(12)
         if ( debug ) then
            call prinin ( icplot,24, 'icplot' )
            call prinrl ( rcplot, 6, 'rcplot' )
         end if  ! ( debug )

         call sepplotcn ( icplot, rcplot, contln, &
                          indcol, textid, idummy, dummy )

      else if ( iinput(1)==110 ) then

!     --- iinput(1) = 110  Subroutine PLOTVC

         number = max(1,iinput(8))
         if ( iinput(72)==1 ) jtimes = mod(jtimes,10)+20
         icplot(1) = 4
         icplot(2) = number
         icplot(3) = iinput(9)
         icplot(4) = 1
         if ( iinput(30)>0 ) icplot(4) = 3
         call plotvcb1 ( icplot, isolut, isolut, rinput, textid )

      else if ( iinput(1)==115 ) then

!     --- iinput(1) = 115  subroutine PLOTF3

         icplot(1) = 7
         do i = 1, 12
            rcplot(i) = 0d0
         end do
         icplot(2) = iinput(24)
         icplot(3) = max(1,iinput(25))
         icplot(4) = iinput(68)
         icplot(5) = max(1,iinput(8))
         icplot(6) = 0
         icplot(7) = iinput(69)
         rcplot(1) = rinput(9)
         rcplot(2) = rinput(23)
         rcplot(3) = rinput(24)
         rcplot(4) = rinput(1)
         rcplot(5) = rinput(1)
         rcplot(10) = rinput(25)
         rcplot(11) = rinput(27)
         rcplot(12) = rinput(26)
         rcplot(13) = rinput(2)
         call plotf3 ( icplot, rcplot, idummy, textid(1) )

      else if ( iinput(1)==116 ) then

!     --- iinput(1) = 116  subroutine PLOT3C

         iconst(1) = max ( 1, iinput(8) )
         iconst(2) = iinput(53)
         iconst(3) = 1
         iconst(4) = iinput(26)
         if ( iinput(27)>0 ) then
            iconst(4) = iinput(27)
         else if ( iinput(6)>0 ) then
            iconst(3) = iconst(4) + iinput(6)
         else
            iconst(3) = iconst(4) + 20
         end if
         if ( iinput(62)==1 ) iconst(3) = iinput(26)
         if ( iinput(63)==1 ) iconst(4) = iinput(27)
         iconst(5) = iinput(28)
         iconst(6) = 0
         const(1) = rinput(1)
         const(2) = rinput(2)
         do i = 3, 8
            const(i) = rinput(i+7)
         end do
         call sepplot3c ( iconst, const )

      else if ( iinput(1)==118 ) then

!     --- iinput(1) = 118, Plot field lines
!         Fill array JCHOIS
!         1 : ichois
!         2 : first user point of boundary part
!         3 : last user point of boundary part

         jchois(1) = iinput(5)
         if ( iinput(5)==2 .or. iinput(5)==3 ) then
            jchois(2) = iinput(18)
            jchois(3) = iinput(19)
         else
            jchois(2) = 0
            jchois(3) = 0
         end if

!        --- do not return lengths of field line(s)

         iconst(1) = 0

!        --- Fill array CONST
!             1 : plotfm
!             2 : yfact
!             3 : flux-cut
!             4 : x-coordinate
!             5 : y-coordinate

         const(1)  = rinput(1)
         const(2)  = rinput(2)
         if ( iinput(49)==0 ) then
            const(3)  = rinput(6)
         else
            if ( jchois(1)<=3 ) jchois(1) = jchois(1) + 3
            const(4)  = rinput(51)
            const(5)  = rinput(52)
         end if

         call seppfield ( user, iuser, isolut, jchois, const, const, iconst(1) )

      else if ( iinput(1)==119 ) then

!     --- iinput(1) = 119, Particle tracking
!         Fill array iinpar

         if ( iinput(50)==1 ) iinput(50) = irefwr

!        --- Positions:
!            1: length
!            2: npart
!            3: iprint
!            4: iplot
!            5: nview
!            6: iflux
!            7: imark
!            8: ioutput
!            9: use_fnpartic
!            10: use_tstepprint

         icplot(1) = 12
         icplot(2) = iinput(49)/ndim
         icplot(3) = iinput(50)
         if ( icplot(3)==0 ) icplot(3) = -1
         icplot(4) = 1 + iinput(48)
         icplot(5) = max(1,iinput(52))
         icplot(6) = 0
         if ( iinput(51)>0 ) then
            icplot(7) = 1
         else
            icplot(7) = 0
         end if
         icplot(8) = 0
         icplot(9) = 0
         icplot(10) = iinput(87)
         if ( icplot(10)>0 ) icplot(3) = irefwr
         icplot(11) = iinput(98)
         icplot(12) = 1
         if ( iinput(85)>0 ) then

!        --- values = ( vi, vj, ... ) given, separate print

            icplot(3) = -1
            icplot(10) = 0
            if ( rinput(30)>0d0 ) then
               icplot(8) = 2
               rcplot(7) = rinput(30)
            else
               icplot(8) = 1
            end if
         end if

         itypvc = ks(isolut)%typevec
         if ( itypvc==115 .or. itypvc==116 ) then
             icplot(6) = 1
         else if ( itypvc==110 ) then
             if ( iinput(76)==1 ) then
                icplot(6) = 3
             end if
         end if

         rcplot(1) = rinput(1)
         rcplot(2) = rinput(2)
         rcplot(3) = 0d0
         rcplot(4) = rinput(21)
         rcplot(5) = 0d0
         rcplot(5) = 0d0
         if ( iinput(51)>0 ) rcplot(5) = rinput(21)/iinput(51)
         rcplot(6) = rinput(30)
         rcplot(8) = rinput(34)
         if ( icplot(2)>100 ) then

!        --- maximum number of tracks too large

            call errint ( 100, 1 )
            call errint ( icplot(2), 2 )
            call errchr ( 'ioutput', 1 )
            call errwar ( 550, 2, 0, 1 )
            icplot(2) = 100

         end if

         call pltrac ( icplot, rcplot, rinput(51), iuser, user )

         if ( associated(particinf) ) then

!        --- mmioutput>0, print values

            call prinptrace
            deallocate(particinf, partics)

         end if

      else if ( iinput(1)==120 .or. iinput(1)==130 ) then

!     --- iinput(1) = 120, 130 Subroutine PLOTFN, height<0 is in cm

         number = max ( 1,iinput(8) )

         if ( rinput(2)<0d0 ) then

!        --- |rinput(2)| is height in cm

            height = rinput(2)

         else

!        --- rinput(2)>=0 is the relative height

            height = -rinput(2) * rinput(1)
            if ( abs(height)<epsmac ) height = -rinput(1)

         end if

         if ( iinput(71)==1 ) then
            jtimes = 1
            jtimessav = jtimes
         end if  ! ( iinput(71)==1 )

         multfact = rinput(3)
         if ( ivec<=1000 .and. abs(multfact)>0d0 .and. &
              abs(multfact-1d0)>sqreps ) then

!        --- multfact # 1, multiply solution by multfact

            const(1)  = multfact
            const(2)  = 0d0
            iconst(1) = 1
            call sepvecman ( 3, 1, isolut, isolut, isolut, const, iconst )
         end if
         if ( ierror/=0 ) go to 1000

         ntextx = iinput(37)
         ntexty = iinput(38)
         if ( ivec<=1000 ) then

!        --- standard situation

            if ( iinput(73)==1 ) then
               save1 = funcx(1)
               save2 = funcx(2)
               funcx(1) = rinput(28)
               funcx(2) = rinput(29)
            end if
            icplot(1) = 8
            icplot(2) = iinput(22)
            icplot(3) = iinput(23)
            icplot(4) = 5*iinput(73)
            icplot(5) = iinput(21)
            icplot(6) = 0
            icplot(7) = number
            icplot(8) = iinput(77)
            rcplot(1) = rinput(1)
            rcplot(2) = height
            call plotfn1 ( icplot, rcplot, isolut, icurvs, &
                           funcx, funcy )
            if ( iinput(73)==1 ) then
               funcx(1) = save1
               funcx(2) = save2
            end if
            if ( ierror/=0 ) go to 1000

            if ( abs(multfact)>0d0 .and. abs(multfact-1d0)>sqreps ) then

!           --- multfact # 1, reset solution

               const(1)  = 1d0 / multfact
               const(2)  = 0d0
               iconst(1) = 1
               call sepvecman ( 3, 1, isolut, isolut, isolut, const, iconst)
            end if

         else

!        --- ivec>1000, special function information stored in ifuncs
!            Copy contents of t and func to the end of buffer

            jtimes = 10
            icplot(1) = 8
            icplot(2) = iinput(22)
            icplot(3) = iinput(23)
            icplot(4) = 1
            icplot(5) = iinput(21)
            icplot(6) = 0
            icplot(7) = number
            icplot(8) = iinput(77)
            rcplot(1) = rinput(1)
            rcplot(2) = height
            call plotfn1 ( icplot, rcplot, isolut, icurvs, &
                           ifuncs(0)%arr, ifuncs(ivec-1000)%arr )

         end if
         if ( iinput(71)==1 ) jtimes = jtimessav

      else if ( iinput(1)==121 ) then

!     --- iinput(1) = 121 Subroutine PLOTLN, height>0 is dimensionless factor

         ichois = 1
         number = max( 1,iinput(8) )
         ichois = 1
         do i = 1,3
            values(i) = rinput(6+i)
         end do
         if ( rinput(2)<0d0 ) then

!        --- |rinput(2)| is in cm

            height = abs(rinput(2)) / rinput(1)
         else

!        --- rinput(2)>=0 is the relative height

            height = rinput(2)
            if ( height<epsmac ) height = 1d0
         end if

         multfact = rinput(3)
         if ( abs(multfact)>0d0 .and. abs(multfact-1d0)>sqreps ) then

!        --- multfact # 1, multiply solution

            ichois = 2
            values(4) = multfact
         end if

         if ( iinput(71)==1 ) then
            jtimes = 1
            jtimessav = jtimes
         end if  ! ( iinput(71)==1 )

         ntextx = iinput(37)
         ntexty = iinput(38)
         icplot(1) = 8
         icplot(2) = iinput(22)
         icplot(3) = iinput(23)
         icplot(4) = ichois
         icplot(5) = iinput(21)
         icplot(6) = 0
         icplot(7) = number
         icplot(8) = iinput(77)
         rcplot(1) = rinput(1)
         rcplot(2) = height
         call plotln1 ( icplot, rcplot, ivalue, values, &
                        textx(1:ntextx),texty(1:ntexty) )
         if ( iinput(71)==1 ) jtimes = jtimessav

      else if ( iinput(1)==125 ) then

!     --- iinput(1) = 125 Subroutine PLTEXT

         ntext = iinput(39)
         if ( ntext==0 ) then
            ntext = 1
            textid(1)(1:1) = ' '
         end if
         call pltext ( rinput(7), rinput(8), textid(1)(1:ntext), &
                       rinput(9), rinput(43) )

      else if ( iinput(1)==128 ) then

!     --- iinput(1) = 128 Subroutine PLTEXT (plot box text)

         ntext = iinput(39)
         if ( ntext==0 ) then
            ntext = 1
            textid(1)(1:1) = ' '
         end if
         call pltextbox ( rinput(7), rinput(8), textid(1)(1:ntext), &
                          rinput(43) )

      else if ( iinput(1)==126 ) then

!     --- iinput(1) = 126 Subroutine PLAFPO

         call plaf05 ( 3, 1, iinput(43), rinput(51), 2 )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'jtimes', jtimes
         write(irefwr,*) 'End insp02sub'

      end if
      call erclos ( 'insp02sub' )

      end subroutine insp02sub
