      subroutine linprbinp ( iinsol, inpsol, ictime )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 06-02-2014 Extra debug
!        version  1.3  date 24-10-2007 Extension of iinsol
!        version  1.2  date 23-03-2007 Adaptation in case of reaction force
!
!   copyright (c) 2005-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array inpsol with information stored in iinsol and
!     extended with defaults
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     input
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinsol(*), inpsol(*), ictime

!     ictime         o    Indication if information must be read from standard
!                         input file ( 0 ) or that information read before is
!                         used ( 1 ).  See to0089
!     iinsol         i    Contains information concerning the linear solver
!                         See the Programmers Guide for a description
!     inpsol         o    Integer input array for subroutine to0089
!                         See to0089 for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iread, iseqcf, iseqsl, iprob, maxvc1, idefcr, idefls, &
              ivectr, isubr, k

!     idefcr         If 1 defect correction is applied, if 0 not
!     idefls         If 1 linear subelements must be applied, otherwise 0
!     iprob          Actual problem number
!     iread          Defines how the input must be read
!                    Possible values:
!                    0:  All SEPRAN input has been read by
!                        subroutine SEPSTN until END_OF_SEPRAN_INPUT
!                        or end of file has been found this input is used
!                    1:  The input is read as described for LINSOL
!                    2:  The last input read as described for LINSOL
!                        is used (equivalent with LINSTM ICTIME=1)
!     iseqcf         Sequence number corresponding to the coefficients
!     iseqsl         Sequence number corresponding to the linear solver
!     isubr          Indicates the type of calling subroutine
!                    Possible values
!                    1:  LINSOL
!                    2:  LINSTM
!                    3:  LINPRB
!     ivectr         Sequence number of the solution vector with respect to
!                    array isol
!     k              General loop variable
!     maxvc1         Number of vectors stored in isol
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'linprbinp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from linprbinp'
         call prinin ( iinsol, iinsol(1), 'iinsol' )

      end if
      if ( ierror/=0 ) go to 1000
      iread  = 0
      iseqcf = 1
      iseqsl = 1
      iprob  = 1
      maxvc1 = 1
      idefcr = 0
      idefls = 0
      if ( iinsol(1)>=2 ) iread  = iinsol(2)
      if ( iinsol(1)>=3 ) iseqcf = iinsol(3)
      if ( iinsol(1)>=4 ) iseqsl = iinsol(4)
      if ( iinsol(1)>=5 ) iprob  = iinsol(5)
      ivectr = iprob
      if ( iinsol(1)>=6 ) ivectr = iinsol(6)
      if ( iinsol(1)>=7 ) maxvc1 = iinsol(7)
      if ( iinsol(1)>=8 ) idefcr = iinsol(8)
      if ( iinsol(1)>=9 ) idefls = iinsol(9)
      if ( iread==0 ) then
         isubr  = 3
         ictime = 0
      else if ( iread==1 ) then
         isubr  = 2
         ictime = 0
      else
         isubr  = 2
         ictime = 1
      end if
      if ( iinsol(1)>=13 ) then
         if ( iinsol(13)==1 ) then

!        --- new and keep

            ictime = 2

         else if ( iinsol(13)==2 ) then

!        --- old and destroy

            ictime = 7

         else if ( iinsol(13)==3 ) then

!        --- old and keep

            ictime = 6

         end if

      end if
      inpsol(1) = isubr
      inpsol(2) = iread
      inpsol(3) = iseqcf
      inpsol(4) = iseqsl
      inpsol(5) = iprob
      inpsol(6) = ivectr
      inpsol(7) = maxvc1
      inpsol(8) = idefcr
      inpsol(9) = idefls
      inpsol(10:18) = 0
      inpsol(19) = 1   !  iseqintmat
      do k = 10, 12
         if ( iinsol(1)>=k ) inpsol(k) = iinsol(k)
      end do ! k = 10, 12
      do k = 13, 19
         if ( iinsol(1)>=k+1 ) inpsol(k) = iinsol(k+1)
      end do ! k = 13, 18

!     --- If the reaction force must be computed, subtract the contribution
!         of slip

      if ( inpsol(10)>0 ) inpsol(14) = 1

      if ( inpsol(15)>1 ) then

!     --- Keep matrix

         ictime = 6

      end if  ! ( iinsol(1)>=16 )

1000  call erclos ( 'linprbinp' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End linprbinp'

      end if

      end
