      subroutine mshadapsurf ( curves, numcurves, iseqvel )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-11-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt coordinates of mesh and coordinates of curves
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     curve
!     change
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: numcurves, curves(numcurves), iseqvel

!     curves         i    array containing the coordinates of the curves
!     iseqvel        i    sequence number of velocity in isolut2dut
!     numcurves      i    Number of curves in array curves
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpada(20), iread, iseqvecs(1), numvecs, iinput(1), &
              iinplt(1), idummy
      double precision rinada(20), rinput(1), rinplt(2)
      logical changed, changeinput

!     changed        If true the number of nodes along the boundary has been
!                    changed
!     changeinput    If true the input arrays iinput and rinput have been
!                    changed
!     idummy         Dummy parameter
!     iinplt         Input array for subroutine PLOTMS
!     iinput         integer input array (dummy)
!     inpada         User input array for subroutine ADAPBOUN
!     iread          Defines how the input must be read (-1: no reading)
!     iseqvecs       Contains sequence numbers of vectors with respect to
!                    isolut2d
!     numvecs        Number of vectors in iseqvecs
!     rinada         User input array for subroutine ADAPBOUN
!     rinplt         Input array for subroutine PLOTMS
!     rinput         Real input array (dummy)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ADAPBNDB       Adapt the boundary of a mesh
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINTCURVE     Print the coordinates of a set of curves
!     SEPMESHSUB     General mesh generator
!     SEPPLOTMS      plot mesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshadapsurf' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshadapsurf'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Change boundary

      inpada = 0
      inpada(1) = 20             ! Length of the array
      inpada(2) = numcurves      ! number of curves in curves
      inpada(4) = 5              ! use normal velocity to move boundary
      inpada(8) = 1              ! Cartesian coordinates
      inpada(9) = 3              ! Keep first point
      inpada(10) = 3             ! Keep last point
      inpada(11) = 2             ! multiplication factor in rinada(1)
      inpada(18) = 1             ! maxiter
      rinada(1) = 1d0            ! relaxation parameter
         inpada(16) = 1             ! plot boundary
      if ( debug ) then
         inpada(16) = 1             ! plot boundary
         inpada(19) = 1             ! print boundary
      end if  ! ( debug )
      iread = -1                 ! no reading
      numvecs = 1                ! number of vectors to be used
      iseqvecs(1) = iseqvel      ! velocity vector
      call adapbndb ( inpada, rinada, curves, iread, changed, &
                      changeinput, numvecs, iseqvecs )

      if ( debug ) &
         call printcurve ( curves, numcurves, 'adapted interface' )

!     --- Compute new mesh, without changing the topology

      call sepmeshsub ( 11, iinput, rinput )

      if ( debug ) then

!     --- Plot the mesh

         iinplt(1) = 0
         rinplt(1) = 15d0
         rinplt(2) = 1d0
         call sepplotms ( iinplt, rinplt, idummy )

      end if  ! ( debug )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshadapsurf'

      end if  ! ( debug )
      call erclos ( 'mshadapsurf' )

      end subroutine mshadapsurf
