      subroutine plotsl ( text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 07-03-2007 New call to plotvcb1
!        version  2.2  date 12-03-2004 replace plotvcbf by plotvcb1
!        version  2.1  date 01-10-1999 New call to plotvcbf
!
!   copyright (c) 1993-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Plot a SEPRAN array if possible
!   If the field is a scalar a contour plot is made, otherwise a vector plot
!   At this moment only NDIM = 2 and number of unknowns is 1 or 2 is
!   implemented
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len = *) text(*)

!     text     i  Text for plot output
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iplots(4), k, input(1), ncoorc(1)
      double precision rplots(3), contln(1), coorcn(1)

!     contln      Dummy
!     coorcn      dummy
!     input       dummy
!     iplots      input array for actual plotting subroutines
!     k           Counting variable
!     ncoorc      dummy
!     rplots      input array for actual plotting subroutines
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PLOTVCB1       Plot vectors
!     SEPPLOTCN      General plot contour subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      90   array isol has not been filled, or the boundary conditions have
!           not been stored in the solution vector
!    1471   NUNKP>2, not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotsl' )

!     --- tests

      if ( ks(isolut)%typevec/=110 ) then
         call errint ( ks(isolut)%typevec, 1 )
         call errsub ( 90, 1, 0, 0 )
      end if
      iprob = ks(isolut)%iprob
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      nunkp = kprobloc(4)
      if ( nunkp==1 ) then

!     --- nunkp = 1, contour plot

         iplots(1) = 0
         call sepplotcn ( iplots, rplots, contln, input, text, ncoorc, coorcn )

      else if ( nunkp==2 ) then

!     --- nunkp = 2, vector plot

         iplots(1) = 4
         iplots(2) = 1
         iplots(3) = 2
         iplots(4) = 1
         rplots(1) = 15d0
         rplots(2) = 0d0
         rplots(3) = 0d0
         call plotvcb1 ( iplots, isolut, isolut, rplots, text )
      else

!     --- nunkp>2: warning and return

         call errint ( nunkp, 1 )
         call errwar ( 1471, 1, 0, 0 )

      end if
      call erclos ( 'plotsl' )
      end subroutine plotsl
