      subroutine plottimehis1 ( rinsep, text, inpsep, colours, idegfd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 16-04-2014 Extra parameters idegfd, nphysi
!        version  1.2  date 14-10-2012 Extra debug
!        version  1.1  date 25-02-2011 Correction copy array
!
!   copyright (c) 2010-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot contents of time history array
!     At present the array is printed instead of plotted
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     time_history
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletimhis
      use sepmodulecons
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer colours(*), inpsep(*), idegfd
      double precision rinsep(*)
      character (len=*) text

!     colours        i    Contains the set of colors to be used for each
!                         function
!     idegfd         i    Degree of freedom sequence number
!     inpsep         i    The integer input read is stored in array INPSEP
!                         For a description see subroutine readscplottim
!     rinsep         i    Array containing the real input for this subroutine
!                         For a description see subroutine readscplottim
!     text           i    String variable containing the text along the y-axis
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ipos
      double precision funcx(istephis), funcy(istephis,npoint_hist)

!     funcx          Work array to store the time parameter
!     funcy          Work array to store the values in each time history
!                    point per time step
!     i              Counting variable
!     ipos           Position in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLOTFUNC1      Plot one or more functions in a function plot
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plottimehis1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plottimehis1'
         write(irefwr,1) 'istephis, npoint_hist', istephis, npoint_hist
         write(irefwr,1) 'nstephis', nstephis
         call prinrl1 ( rvectim, istephis, npoint_hist+1, 'rvectim' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Copy rvectim in funcx and funcy
!         This is necessary, since the sequence is changed
!         The first and second index are interchanged

      funcx(1:istephis) = rvectim(1,1:istephis)

      if ( debug ) call prinrl ( funcx, istephis, 'funcx' )

      do i = 1, npoint_hist
         ipos = 1+(i-1)*nphysi+idegfd
         funcy(1:istephis,i) = rvectim(ipos,1:istephis)
      end do  ! i = 1, npoint_hist

      if ( debug ) &
         call prinrl1 ( funcy, npoint_hist, istephis, 'funcy' )

      call plotfunc1 ( istephis, funcx, funcy, npoint_hist, &
                       rinsep, text, inpsep, colours )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plottimehis1'

      end if  ! ( debug )
      call erclos ( 'plottimehis1' )

      end
