      subroutine prbreak ( istart, inpsep, ioutpsav, &
                           iseqlp, niter, timefinished, icalltimstr, &
                           loopready )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-09-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Handle the break option depending on the type of loop
!
! **********************************************************************
!
!                       KEYWORDS
!
!     loop
!     break
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleloop
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*), ioutpsav, iseqlp, &
              niter, icalltimstr
      logical timefinished, loopready

!     icalltimstr   i/o   Number of calls of loop start
!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscbreak
!     ioutpsav       i    Help parameter to save the value of ioutp
!     iseqlp        i/o   Present level of loop
!     istart        i/o   Next position in inpsep
!     loopready     i/o   If true the loop has finished
!     niter         i/o   Iteration sequence number
!     timefinished  i/o   If true the time loop has finished
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icode

!     icode          Defines type of loop
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2917   Option not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prbreak' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prbreak'
         write(irefwr,1) 'istart', istart
         call prinin ( inpsep, istart+2, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icode = inpsep(istart+1)
      istart = inpsep(istart+2)
      if ( debug ) write(irefwr,1) 'icode, istart/2', icode, istart

      select case ( icode )

         case (22)

!        --- End while loop

            istart = inpsep(istart+2)

         case (38)

!        --- isubr = 38, End for loop

            istart = inpsep(istart+5)
            iseqfor = iseqfor - 1

         case (45)

!        --- isubr = 45, call subroutine TOLOOP and check on convergence

            niter  = 1
            ifirstlp(iseqlp) = 0
            if ( isollp(1,iseqlp)>0 ) &
               call seprmsolinfo ( isollp(1,iseqlp) )
            iseqlp = iseqlp-1
            istart = inpsep(istart+2)
            loopready = .false.

         case (47)

!        --- isubr = 47, End time loop

            istart = inpsep(istart+2)
            itimloop = 0
            icalltimstr = 0
            timefinished = .false.
            maxcalltim = 0
            icalltim = 0

         case default

!        --- icode not implemented

            call errint ( icode, 1 )
            call errsub ( 2917, 1, 0, 0 )

      end select  ! case ( icode )
      ioutp = ioutpsav

1000  call erclos ( 'prbreak' )
      if ( debug ) then

!     --- Debug information

         if ( debug ) write(irefwr,1) 'istart/end', istart
         write(irefwr,*) 'End prbreak'

      end if  ! ( debug )

      end
