      subroutine prcompcurve ( kmeshm2, inodes1, inodes2, same )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-06-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compare curves in two meshes
!     nodes may be different
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     compare
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshm2(*), inodes1(*), inodes2(*)
      logical same

!     inodes1        i    Ordering of coor1 for increasing coordinates
!     inodes2        i    Ordering of coor2 for increasing coordinates
!     kmeshm2        i    kmesh part m for mesh 2
!     same           o    If true the arrays are the same
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istar3, icurnr, inode1, inode2, i, numnods1, numnods2

!     i              Counting variable
!     icurnr         Curve sequence number
!     inode1         Node in curve of first mesh
!     inode2         Node in curve of second mesh
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     numnods1       Number of nodes along curve of first mesh
!     numnods2       Number of nodes along curve of second mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcompcurve' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcompcurve'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istar3=kmeshm(2)-1 ! starting address of node numbers (-1)
      same = .true.

      do icurnr = 1, ncurvs

         numnods1 = kmeshm(5+icurnr)-kmeshm(4+icurnr)  ! number of nodes
         numnods2 = kmeshm2(5+icurnr)-kmeshm2(4+icurnr)  ! number of nodes
         if ( numnods1/=numnods2 ) then

!        --- The number of nodes along both curves not the same

            same = .false.
            write(irefwr,*) 'The number of nodes along curve ', icurnr, &
                            ' is different for both meshes'
            go to 1000

         end if  ! ( numnods1/=numnods2 )

         do i = 1, numnods1

            inode1 = kmeshm(istar3+i)  ! node on first mesh
            inode2 = kmeshm2(istar3+i)  ! node on second mesh

            if ( inode1>0 .and. inode2>0 ) then

!           --- Both kmeshm and kmeshm2 have been filled

               inode1 = inodes1(inode1)
               inode2 = inodes2(inode2)
               if ( inode1/=inode2 ) then

!              --- nodes have different sequence number

                  write(irefwr,*) 'node ', i, ' on curve ', icurnr, &
                                  ' is different for both meshes'
                  same = .false.
                  go to 1000

               end if  ! ( inode1/=inode2 )

            else if ( inode1>0 .or. inode2>0 ) then

!           --- Only one of the points has been filled

               same = .false.
               write(irefwr,*) 'node ', i, ' on curve ', icurnr, &
                           ' has been filled for only one of the meshes'
               go to 1000

            end if  ! ( inode1>0 .and. inode2>0 )

         end do  ! i = 1, numnods1

      end do  ! icurnr = 1, ncurvs

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcompcurve'

      end if  ! ( debug )
      call erclos ( 'prcompcurve' )

      end
