      subroutine prcompelem ( kmeshc2, inodes1, inodes2, lenkmeshc, same )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-05-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compare elements in two meshes
!     node and element numbers may be different
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     compare
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lenkmeshc, kmeshc2(lenkmeshc), inodes1(*), inodes2(*)
      logical same

!     inodes1        i    Ordering of coor1 for increasing coordinates
!     inodes2        i    Ordering of coor2 for increasing coordinates
!     kmeshc2        i    kmesh part c for mesh 2
!     lenkmeshc      i    length of array kmeshc
!     same           o    If true the arrays are the same
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp, inpelm, ikelmc
      integer, allocatable :: kmshc1(:), kmshc2(:)

!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     kmshc1         Copy of kmshc1 renumbered according to inodes1
!     kmshc2         Copy of kmshc2 renumbered according to inodes2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PR0003         Print KMESH parts c and g
!     PRCOMPELEM1    Compare elements in two meshes for one element group
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcompelem' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcompelem'
         write(irefwr,*) 'kmeshc'
         call pr0003 ( kmesh, kmeshc )
         write(irefwr,*) 'kmeshc2'
         call pr0003 ( kmesh, kmeshc2 )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Renumber kmeshc and kmeshc2 and store in new arrays kmshc1, kmshc2

      allocate ( kmshc1(lenkmeshc), kmshc2(lenkmeshc), stat = error )
      if ( error/=0 ) call eralloc ( error, 2*lenkmeshc, 'kmshc1' )
      if ( ierror/=0 ) go to 1000

      kmshc1 = inodes1(kmeshc)
      kmshc2 = inodes2(kmeshc2)

      if ( debug ) then
         write(irefwr,*) 'kmshc1'
         call pr0003 ( kmesh, kmshc1 )
         write(irefwr,*) 'kmshc1'
         call pr0003 ( kmesh, kmshc2 )
      end if  ! ( debug )

!     --- Loop over element groups

      ikelmc = 1         ! starting address in kmeshc

      do ielgrp = 1, nelgrp

         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm
         call prcompelem1 ( kmshc1(ikelmc), kmshc2(ikelmc), inpelm, &
                            inelem, ielgrp, same )
         ikelmc = ikelmc+inelem*inpelm
         if ( .not. same ) go to 1000

      end do  ! ielgrp = 1, nelgrp

      deallocate ( kmshc1, kmshc2, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'inodes1' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcompelem'

      end if  ! ( debug )
      call erclos ( 'prcompelem' )

      end
