      function prevalbool ( reals, boolop )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 17-05-2010 Correction on reals 0
!        version  1.0  date 15-02-2008
!
!   copyright (c) 2008-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Evaluate the expression real1 boolean_operator real2
!     Return with the result
! **********************************************************************
!
!                       KEYWORDS
!
!     boolean_expression
!     boolean
!     evaluate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision reals(2)
      integer boolop
      logical prevalbool

!     boolop         i    Value of boolean operator
!                         1: operator is <
!                         2: operator is <= &
!                         3: operator is >
!                         4: operator is >= &
!                         5: operator is /= &
!                         6: operator is == &
!     prevalbool     o    Result of operation
!     reals          i    Contains the reals before and after the operator
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision rmax, eps
      integer int1, int2

!     eps            Local accuracy
!     int1           integer value of first real
!     int2           integer value of second real
!     rmax           maximum of absolute value of the 2 reals
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      logical prevalboolint

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PREVALBOOLINT  Evaluate the expression int1 boolean_operator int2
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2939   Boolean operator has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prevalbool' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prevalbool'
         write(irefwr,1) 'boolop', boolop
         write(irefwr,2) 'reals', reals
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check if both reals are actually integers

      if ( abs(reals(1))<1d8 .and. abs(reals(2))<1d9 ) then

!     --- Both reals are in the range of integers

         int1 = nint(reals(1))
         int2 = nint(reals(2))
         if ( debug ) write(irefwr,1) 'int1, int2', int1, int2
         if ( abs(int1-reals(1))<=1d-10*int1 .and. &
              abs(int2-reals(2))<=1d-10*int2 ) then

!        --- Both reals are integers, do integer comparison

            prevalbool = prevalboolint ( int1, int2, boolop )
            go to 1000

         end if  ! ( abs(int1-reals(1))<1d-10*int1 ... )

      end if  ! ( abs(reals(1))<1d8 .and. abs(reals(2))<1d8 )

!     --- Compute accuracy

      rmax = max(abs(reals(1)),abs(reals(2)))
      eps = 1d-10*rmax

      select case ( boolop )

         case(1)

!        --- boolop = 1, i.e. r1<r2

            prevalbool = reals(1)<reals(2)

         case(2)

!        --- boolop = 2, i.e. r1<=r2

            prevalbool = reals(1)<reals(2) .or. &
                         abs(reals(1)-reals(2))<eps

         case(3)

!        --- boolop = 3, i.e. r1>r2

            prevalbool = reals(1)>reals(2)

         case(4)

!        --- boolop = 4, i.e. r1>=r2

            prevalbool = reals(1)>reals(2) .or. &
                         abs(reals(1)-reals(2))<eps

         case(5)

!        --- boolop = 5, i.e. r1/=r2

            prevalbool = abs(reals(1)-reals(2))>eps

         case(6)

!        --- boolop = 1, i.e. r1==r2

            prevalbool = abs(reals(1)-reals(2))<eps

         case default

!        --- error: boolop has wrong value

            call errint ( boolop, 1 )
            call errsub ( 2939, 1, 0, 0 )

      end select  ! case ( boolop )

1000  call erclos ( 'prevalbool' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'prevalbool ', prevalbool
         write(irefwr,*) 'End prevalbool'

      end if  ! ( debug )

      end
