      function prevalexpreal ( ival, rinsep, text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 10-01-2015 Allow ival = 0
!        version  3.0  date 14-11-2013 Extra parameter text
!        version  2.1  date 14-04-2013 New call to erscalval
!
!   copyright (c) 2008-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Evaluate real, scalar or real expression
!
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ival
      double precision prevalexpreal, rinsep(*)
      character (len=*) text

!     ival           i    Information about the quantity to be computed
!                        >200000000, expression with sequence number
!                                      ival-200000000
!                        >100000000, scalar with sequence number
!                                      ival-100000000
!                         else         position in rinsep
!     prevalexpreal  o    Result of the evaluation
!     rinsep         i    Array containing the real input for this subroutine
!     text           i    String variable containing some text for error message
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivalue
      double precision realrd, rvalue
      logical integerfound

!     integerfound   If true an integer has been found
!     ivalue         Integer value
!     realrd         Result og evaluation of expression
!     rvalue         Real value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERSCALVAL      Fill value of scalar defined by ipos into either rvalue or
!                    ivalue
!     PREVALEXPR     Evaluate expression and fill actual values of scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prevalexpreal' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prevalexpreal'
         write(irefwr,1) 'ival', ival
  1      format ( a, 1x, (10i9) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ival>200000000 ) then

!     --- ival>200000000, hence expression found

         if ( debug ) write(irefwr,*) 'before prevalexpr'
         call prevalexpr ( ival, realrd, text )
         prevalexpreal = realrd

      else if ( ival>100000000 ) then

!     --- ival>100000000, hence scalar found, that is stored in scalars

         if ( debug ) write(irefwr,*) 'before erscalval'
         call erscalval ( ival-100000000, rvalue, ivalue, integerfound )
         if ( integerfound ) then
            prevalexpreal = ivalue
         else
            prevalexpreal = rvalue
         end if  ! ( integerfound )

      else if ( ival>0 ) then

!     --- ival>0, hence real found, that is stored in rinsep

         if ( debug ) write(irefwr,*) 'before rinsep'
         prevalexpreal = rinsep(ival)

      else

!     --- ival=0, set to zero

         prevalexpreal = 0d0

      end if  ! ( ival>200000000 )

1000  call erclos ( 'prevalexpreal' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ival', ival
         write(irefwr,2) 'prevalexpreal', prevalexpreal
         write(irefwr,*) 'End prevalexpreal'

      end if  ! ( debug )

      end
