      subroutine prfil2 ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.5  date 25-11-2012 Long integers
!        version  2.4  date 06-02-2012 Allow indexed vectors
!        version  2.3  date 30-08-2006 New definition of cpost5
!
!   copyright (c) 1995-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Call subroutines WRITBS or READBS to write or read vectors from
!    backing storage (file 2)
! **********************************************************************
!
!                       KEYWORDS
!
!     backing_storage
!     read
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecpack
      use sepmodulesol
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduletime
      use sepmoduleplot
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: inpsep(*)

!     inpsep     i    The integer input read is stored in array INPSEP
!                     1  ICODE      Code indicating the action to be carried out
!                                   17: write vector
!                                   18: read vector
!                     2  IVECTOR    Vector sequence number
!                     3  ISEQUENCE  Sequence number with respect to reading or
!                                   writing
!                                   If -1, the last or the next one is used
!                     4  ISAVE      If 1 the administration must be saved
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqlc, ivec, ivectr, icode, numarr, nch
      character (len=10) :: name
      logical :: skippar

!     icode          Return code with respect to subroutine INTX01
!                    Indicates position in array code or code1
!     iseqlc         Sequence number of array in file 2
!     ivec           Sequence number of vector of special structure
!     ivectr         Sequence number of first vector
!     name           Character string to store a text
!     nch            Number of characters in a string
!     numarr         Contains the sequence number of the array that has been
!                    written at output of writbs
!     skippar        If true writing must be skipped, since this is a
!                    parallel computation at a node with number>1 (slave)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inivectr

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     INIVECTR       Return with sequence number of vector
!     READBS         Read arrays from the backing storage file 2
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
!     SEPWRITBS      Write arrays to the backing storage file 2
!     WRITB1         Save administration to the backing storage file 2
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1582   Array read from backing storage does not correspond to problem
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ivectr /0/
! ======================================================================
!
      call eropen ( 'prfil2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfil2'
         call ertrace

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      skippar = parallel .and. inodenr>1 ! slave
      ivectr = min ( ivectr + 1, maxvc1 )
      icode  = inpsep(1)
      ivec   = inivectr(inpsep(2))
      iseqlc   = inpsep(3)
      if ( ivec/=-1 ) ivectr = ivec
      write(name,100) ivec
100   format('vector ',i3)
      nch = index(names(ivectr),' ')-1
      if ( nch<=0 ) nch = len(names(ivectr))
      if ( icode==17 ) then

!     --- icode = 17, write vector

         if ( skippar ) go to 1000
         if ( ioutp>=0 .and. itimloop<=1 ) &
            write(irefwr,110) names(ivectr)(1:nch), ivectr
110      format ( ' Write vector ', a, ' with sequence number ', &
                  i3, ' to backing storage' )
         call sepwritbs ( 0, iseqlc, numarr, name, isolut2d(ivectr), &
                          kprob, 110 )

!        --- Save administration if necessary

         if ( inpsep(4)==1 ) call writb1

      else

!     --- icode = 18, read vector

         if ( iseqlc==-1 ) iseqlc = inew-1
         if ( ioutp>=0 .and. itimloop<=1 ) &
            write(irefwr,120) names(ivectr)(1:nch), ivectr, iseqlc
120      format ( ' Read vector ', a, ' with sequence number ', &
                  i3, ' from backing storage'/ &
                  'Sequence number on backing storage is ', i3 )
         call readbs ( iseqlc, isolut2d(ivectr), kprob )

!        --- Check the array and activate pointers

         call sepactsolbf2 ( isolut2d(ivectr) )
         if ( indprfi==0 ) then
            nusoli = nunkpi*npoint
         else
            nusoli = kprobfi(1+npoint)
         end if
         if ( nusoli/=ks(isolut2d(ivectr))%nusol ) then

!        --- nusol incorrect

            call errint ( ivectr, 1 )
            call errint ( iseqlc, 2 )
            call errsub ( 1582, 2, 0, 0 )

         end if

      end if
      numvecmn = max(numvecmn,ivectr)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfil2'

      end if  ! ( debug )
      call erclos ( 'prfil2' )

      end subroutine prfil2
