      subroutine prfilltext ( loctext, kmesh, inpsep, curves, numcurves, &
                              numvecs, seqvecs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 08-01-2015 Correction length text
!        version  1.0  date 10-05-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array loctext with the text to be used along axis
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     text
!     axis
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmodulecons
      use sepmoduleplot
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: kmesh(*), inpsep(4), numcurves, &
                             curves(numcurves), numvecs, &
                             seqvecs(numvecs)
      character (len=*) :: loctext(2)

!     curves         i    contains the curve numbers
!     inpsep         i    The integer input read is stored in array INPSEP
!                         Contents
!                         1: Position of text along x-axis in text array
!                         2: Position of text along y-axis in text array
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     loctext        o    Contains the texts to be plotted along the axis
!     numcurves      i    Number of curves in array curves
!     numvecs        i    Number of vectors in seqvecs
!     seqvecs        i    Contains the sequence numbers of the vectors in isol
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ndim, jtext(1), i

!     i              Counting variable
!     jtext          Number of characters in a text
!     ndim           Dimension of the space.
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfilltext' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfilltext'
         call prinin ( curves, numcurves, 'curves' )
         call prinin ( inpsep, 5, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Initializations

      ndim = kmesh(6)

      if ( debug ) write(irefwr,1) 'ndim', ndim

      if ( inpsep(1)>0 ) then

!     --- text given by user

         loctext(1) = texts(inpsep(1))
         ntextx = inpsep(2)

      else if ( ndim==1 ) then

!     --- One dimension, use default x

         loctext(1) = 'x'
         ntextx = 1

      else

!     --- more dimensions, use curve numbers

         loctext(1)  = 'curves '
         jtext(1) = 7
         write(loctext(1)(jtext(1)+1:jtext(1)+3),100) curves(1)
100      format( i3 )
         jtext(1) = jtext(1)+3
         do i = 2, numcurves
            write(loctext(1)(jtext(1)+1:jtext(1)+4),110) curves(i)
110         format( ',', i3 )
            jtext(1) = jtext(1)+4
            if ( jtext(1)>76 ) exit
         end do  ! i = 1, curves(1)
         ntextx = jtext(1)

      end if  ! ( inpsep(1)>0 ) then

      if ( inpsep(3)>0 ) then

!     --- text given by user

         loctext(2) = texts(inpsep(3))
         ntexty = inpsep(4)

      else if ( numvecs>1 ) then

!     --- more vectors, use default value

         loctext(2) = 'value'
         ntexty = 5

      else

!     --- one vector, use vector name

         loctext(2) = names(seqvecs(1))
         ntexty = index(loctext(2),' ')-1

      end if  ! ( inpsep(1)>0 ) then

1000  call erclos ( 'prfilltext' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'loctext(1) ', loctext(1)(1:ntextx)
         write(irefwr,*) 'loctext(2) ', loctext(2)(1:ntexty)
         write(irefwr,*) 'End prfilltext'

      end if  ! ( debug )

      end
