      subroutine prfindclosn3d1 ( cells, nactive, markcells, &
                                  nx, ny, nz, lencells )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 18-09-2012 Extra parameter lencells
!        version  1.0  date 29-02-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array cells with neighbors of old cells which have not been
!     treated yet
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     cell
!     find
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nactive, lencells, cells(3,lencells), nx, ny, nz, &
              markcells(nx,ny,nz)

!     cells         i/o   At input: all active cells
!                         At output all non-marked neighbors of these cells
!     lencells       i    Declared length of array cells
!     markcells     i/o   Array of size nx * ny to mark cells that have
!                         already been used
!     nactive       i/o   Number of active cells
!                         At input the old number
!                         At output the new one
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     nz             i    number of points in z direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nactcp, cellscop(3,nactive), indi, indj, indk, k, i, j, l

!     cellscop       Copy of the input array cells
!     i              Counting variable
!     indi           i-index
!     indj           j-index
!     indk           k-index
!     j              Counting variable
!     k              Counting variable
!     l              Counting variable
!     nactcp         copy of input value of nactive
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfindclosn3d1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfindclosn3d1'
         write(irefwr,1) 'nx, ny, nz', nx, ny, nz
         write(irefwr,1) 'nactive', nactive
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Copy old array cells in cellscop

      nactcp = nactive
      cellscop(1:3,1:nactive) = cells(1:3,1:nactive)

!     --- Fill array cells with neighbors of old cells which have not been
!         treated yet

      nactive = 0

      do k = 1, nactcp

         indi = cellscop(1,k)
         indj = cellscop(2,k)
         indk = cellscop(3,k)

         do i = max(1,indi-1), min(nx,indi+1)
            do j = max(1,indj-1), min(ny,indj+1)
               do l = max(1,indk-1), min(nz,indk+1)

                  if ( markcells(i,j,l)==0 ) then

!                 --- New cell found, store

                     markcells(i,j,l) = 1
                     nactive = nactive+1
                     call inichkarlen ( nactive, lencells, 'cells' )
                     if ( ierror/=0 ) go to 1000
                     cells(1,nactive) = i
                     cells(2,nactive) = j
                     cells(3,nactive) = l

                  end if  ! ( markcells(i,j)==0 )

               end do  ! l = max(1,indk-1), min(nz,indk+1)
            end do  ! j = max(1,indj-1), min(ny,indj+1)
         end do  ! i = max(1,indi-1), min(nx,indi+1)

      end do  ! k = 1, nactcp

1000  call erclos ( 'prfindclosn3d1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfindclosn3d1'

      end if  ! ( debug )

      end
