      subroutine prfindclosnod3 ( nnodes, istore, dist, nodefound, &
                                  coor, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-11-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find node with shortest distance to (x,y) in istore
!     corresponding to cell (i,j)
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     cell
!     find_node
!     find
!     distance
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, istore(nnodes), nodefound
      double precision dist, coor(2,*), x, y

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     dist          i/o   Shortest distance found
!     istore         i    Contains the set of nodes
!     nnodes         i    Number of nodes in istore
!     nodefound      o    Node number of node with shortest distance
!     x              i    x co-ordinate of point
!     y              i    y co-ordinate of point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, node
      double precision h

!     h              distance between two points
!     i              Counting variable
!     node           Node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfindclosnod3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfindclosnod3'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, nnodes

         node = istore(i)
         h = sqrt((x-coor(1,node))**2+(y-coor(2,node))**2)
         if ( h< dist ) then

!        --- new node found

            nodefound = node
            dist = h

         end if  ! ( sqrt((x-coor(1,node))**2+(y-coor(2,node))**2)< dist )

      end do  ! i = 1, nnodes

1000  call erclos ( 'prfindclosnod3' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfindclosnod3'

      end if  ! ( debug )

      end
