      subroutine printg ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 04-09-2016 New parameter list
!        version  4.12 date 28-12-2014 Extension with COORDINATE_SYSTEM
!        version  4.11 date 24-12-2014 New definition clinsl
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Call subroutine INTEGR to compute integrals
!     Fill array IININT with the values stored in INPSEP
! **********************************************************************
!
!                       KEYWORDS
!
!     integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulevecs
      use sepmodulekmesh
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduletime
      use sepmodulecons
      use sepmoduleplot
      use sepmodulepost
      use sepmoduletexts
      use sepmoduleinput
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscintegral from position 2
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq, iseqcf, ivectr, iseqin, iprob, isubr, &
              lenius, lenusr, idummy(1), iseqarrsav, isolcp, &
              iseqarrmin, iseqarrmax, dimscal, numvecmnsav, jdegfd, &
              iscal1, iscal2, iscal3, nch, nch1, &
              numarrays, numvecsav, maxcal, ipos, numelgrp, &
              leninpvol
      integer, allocatable :: inpvol(:)
      double precision outval(3)
      logical newvector, newvec

!     dimscal        Dimension of scalar
!     idummy         Dummy parameter
!     inpvol         Integer array in which the user indicates what
!                    computations must be performed by subroutine INTEGSBF
!     ipos           Position of input arrays in sepmoduleinput
!     iprob          Actual problem number
!     iscal1         Sequence number of first scalar
!     iscal2         Sequence number of second scalar
!     iscal3         Sequence number of third scalar
!     iseq           Sequence number of vector in VECOLD/ISLOLD
!     iseqarrmax     Maximum value of iseqarray in loop
!     iseqarrmin     Minimum value of iseqarray in loop
!     iseqarrsav     Help parameter to store old value of iseqarray
!     iseqcf         Sequence number corresponding to the coefficients
!     iseqin         Sequence number of vector in VECOLD/ISLOLD
!     isolcp         Standard SEPRAN array containing information of the
!                    new solution array with only one unknown per point
!     isubr          Indicates the type of calling subroutine for erlins
!     ivectr         Sequence number of first vector
!     jdegfd         Sequence of degree of freedom to be extracted
!     leninpvol      Computed length of array inpvol
!     lenius         Computed length of array IUSER
!     lenusr         Computed length of array USER
!     maxcal         Maximum number of coefficient inputs read
!     nch            Number of characters in a string
!     nch1           Number of characters in a string
!     newvec         If true isolcp is a new vector otherwise
!                    isolcp contains exactly isol, which means that
!                    removing the array corresponding to isolcp
!                    also implies removing the array corresponding to isol
!     newvector      If true a new vector is made, which should be
!                    removed at the end of the calling subroutine
!     numarrays      Number of indexed vectors in vector array
!     numelgrp       Number of element groups stored
!     numvecmnsav    Help variable to store the value of numvecmn
!     numvecsav      Help variable to store the value of numvec
!     outval         Contains the values of the integrals computed:
!                    1:  integral over domain
!                    2:  Minimum value over the elements
!                    3:  Maximum value over the elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getintparm, getscalind, getdimens

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     ERLINS         Help subroutine to call FILCOF, which fills coefficients in
!                    IUSER and USER
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     GETDIMENS      Return with dimension of constpstant or scalar corresponding
!                    to iseq
!     GETINTPARM     Return with real/scalar defined by inpsep/rinsep
!     GETSCALIND     Return with actual scalar number, where the original number
!                    is the modified one stored in the input array
!     GETVECTOR      Return with sequence number of vector in array isol
!                    If vector consists of a vector expression a new vector is
!                    created and a new sequence number made
!     INTEGS1        ?
!     PREXTRACTUNK   Extract degree of freedom jdegfd from the vector
!                    corresponding to isol and store in the vector corresponding
!                    to isolcp
!     PRFILLUSERDEF  Fill arrays iuser and user with default values for
!                    integration
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3128   dimension of scalr and vector not the same
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data iseqcf /0/, iprob /0/
! ======================================================================
!
      call eropen ( 'printg' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printg'
         write(irefwr,1) 'numvecmn, istartmn', numvecmn, istartmn
         call prinin ( inpsep, 14, 'inpsep' )
  1      format ( a, 1x, (10i8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqin = 1
      iseqcf = inpsep(2)
      iseq   = inpsep(3)
      if ( iseq/=-1 ) iseqin = iseq

      isolcp = 0
      newvec = .false.

      newvector = .false.
      numvecmnsav = numvecmn  ! save numvecmn
      numvecsav = numvec

      iseqarrsav = iseqarray
      if ( iseqarray==0 ) then

!     --- iseqarray = 0, check if loop is necessary

         call getvector ( inpsep(4), ivectr, numarrays, newvector )
         iseqarrmin = 1
         iseqarrmax = numarrays

      else

!     --- iseqarray # 0, use iseqarray

         iseqarrmin = iseqarray
         iseqarrmax = iseqarray

      end if  ! ( iseqarray==0 )

      iscal1 = inpsep(5)
      dimscal = getdimens(iscal1)
      iscal1 = getscalind ( iscal1 )
      iscal2 = inpsep(6)
      iscal3 = inpsep(7)
      if ( iscal2>0 ) iscal2 = getscalind ( iscal2 )
      if ( iscal3>0 ) iscal3 = getscalind ( iscal3 )

      if ( debug ) then
          write(irefwr,1) 'iseqarrmin, iseqarrmax, numarrays', &
                           iseqarrmin, iseqarrmax, numarrays
          write(irefwr,1) 'iscal1, iscal2, iscal3, dimscal', &
                           iscal1, iscal2, iscal3, dimscal
      end if  ! ( debug )

      allocate ( inpvol(8+nelgrp), stat = error )
      if ( error/=0 ) call eralloc ( error, 8+nelgrp, 'inpvol' )
      if ( ierror/=0 ) go to 1000

      inpvol = 0
      inpvol(1) = 8+nelgrp

      if ( numarrays>1 ) then

!     --- A series of vectors must be integrated

         if ( dimscal/=numarrays ) then

!        --- error dimension of scalars and vector not the same

            call errint ( numarrays, 1 )
            call errint ( dimscal, 2 )
            call errchr ( names(ivectr), 1 )
            call errchr ( chscalars(iscal1), 2 )
            call errsub ( 3128, 2, 0, 2 )
            go to 1000

         end if  ! ( dimscal/=numarrays )

      end if  ! ( numarrays>1 )

      isubr  = 3
      ivectr = max(1,ivectr)          ! to prevent zero index
      iprob  = ks(isolut2d(ivectr))%iprob  ! problem number
      numelgrp = 0

      if ( debug ) then
         write(irefwr,1) 'iseqin, ivectr, iprob, iseqcf', &
                          iseqin, ivectr, iprob, iseqcf
         write(irefwr,1) 'iscal1, iscal2, iscal3', &
                          iscal1, iscal2, iscal3
         write(irefwr,1) 'iseqarrmin, iseqarrmax, ivectr', &
                          iseqarrmin, iseqarrmax, ivectr
      end if  ! ( debug )

      if ( ioutp>=0 .and. itimloop<=1 ) then
         nch = index(chscalars(iscal1),' ')-1
         if ( nch<=0 ) nch = len(chscalars(iscal1))
         write(irefwr,120) chscalars(iscal1)(1:nch), iscal1
120      format ( ' Compute scalar ', a, &
                  ' with sequence number ', i3, ' as integral' )
      end if

!     --- Check if input for integral can be found

      call sepgetinpfinf ( 13, iseqin, ipos, maxcal )
      if ( debug ) write(irefwr,1) 'iseqin, maxcal', iseqin, maxcal

      if ( maxcal<=0 .or. inpsep(8)>0 ) then

!     --- No input found, fill array inpvol

         iseqin = -1  ! indicate that inpvol is filled
         jdegfd = max(1,getintparm(inpsep(11)))
         leninpvol = 8
         inpvol(2) = getintparm(inpsep(8))   ! icheli
         inpvol(3) = 1                       ! idegfd
         inpvol(4:6) = 0         ! MAXMIN, ALLELEMENTS, NUMVEC
         inpvol(5) = getintparm(inpsep(9))   ! ichoice
         inpvol(7) = 1           ! Vector sequence number
         inpvol(8) = getintparm(inpsep(10))   ! level set number

         if ( inpsep(14)>0 ) then

!        --- Element groups found

            inpvol(5) = 7
            inpvol(9:8+nelgrp) = 0
            numelgrp = inpsep(15)
            inpvol(8+inpsep(16:15+numelgrp)) = 1
            leninpvol = leninpvol+nelgrp
            numelgrp = numelgrp+1

         end if  ! ( inpsep(14)>0 )

         inpvol(1) = leninpvol           ! number of entries in inpvol
         if ( debug ) call prinin ( inpvol, leninpvol, 'inpvol' )
         numvecmn = 1

!        --- Copy degree jdegfd in isolcp

         call prextractunk ( isolut2d(ivectr), jdegfd, isolcp, newvec )
         if ( ierror/=0 ) go to 1000

      else if ( numvecmn==1 ) then

!     --- Copy vector into isolcp

         call sepcopyvc ( isolut2d(ivectr), isolcp )

      end if  ! ( maxcal<=0 )

      do iseqarray = iseqarrmin, iseqarrmax

         if ( iseqcf/=0 ) then

!        --- Fill coefficients

            if ( ioutp>=0 .and. itimloop<=1 ) write(irefwr,110) iseqcf
110         format ( ' Fill coefficients with sequence number ', i3 )
            call erlins ( isubr, iprob, iseqcf )

         end if

         if ( associated(iuser1(iprob)%iuser) ) then

!        --- Copy arrays IUSER and USER to end of IBUFFR.  In this way INTEGR
!            cannot overwrite the positions used by these arrays

            iuser => iuser1(iprob)%iuser
            user => iuser1(iprob)%user

         else

!        --- Do not copy, just set length of arrays to 1

            lenius = 6+5*nelgrp
            lenusr = 1

            allocate ( iuser(lenius), user(lenusr), stat = error )
            if ( error/=0 ) call eralloc ( error, lenius+lenusr, 'iuser' )
            if ( ierror/=0 ) go to 1000

         end if  ! ( iuser1(iprob)>0 )

         if ( inpsep(2)==0 ) then

!        --- Fill iuser with default values

            call prfilluserdef ( iuser, inpsep, nelgrp, inpvol )

         end if  ! ( iuser1(iprob)>0 .and. iseqcf>0 )

!        --- Actual call to integration subroutine

         outval(2) = 0d0
         outval(3) = 0d0
         if ( debug ) &
            write(irefwr,1) 'before integs: ivectr, numvecmn', &
                            ivectr, numvecmn
         if ( numvecmn==1 ) then

!        --- Only one vector

            call integs1 ( inpvol, outval, isolcp, &
                           iseqin, 1, numvecmn )

         else

            call integs1 ( inpvol, outval, isolut2d, &
                           iseqin, ivectr, numvecmn )

         end if  ! ( numvecmn==1 )
         if ( debug ) then
            write(irefwr,1) 'after integs: ivectr', ivectr
            call prinrl ( outval, 3, 'outval' )
         end if  ! ( debug )

         scalars(iscal1) = outval(1)
         numscals = max(numscals,iscal1)
         if ( abs(outval(2))>0d0 .and. abs(outval(3))>0d0 ) then

!        --- outval has been filled with minimum and maximum values

            if ( ioutp>=0 .and. itimloop<=1 ) then
               nch = index(chscalars(iscal2),' ')-1
               if ( nch<=0 ) nch = len(chscalars(iscal2))
               nch1 = index(chscalars(iscal3),' ')-1
               if ( nch1<=0 ) nch1 = len(chscalars(iscal3))
               write ( irefwr, 130 ) chscalars(iscal2)(1:nch), iscal2, &
                                     chscalars(iscal3)(1:nch1), iscal3
130            format ( ' Compute scalar ', a, &
                        ' with sequence number ', i3, &
                        ' as minimum integral over element' / &
                        ' Compute scalar ', a, &
                        ' with sequence number ', i3, &
                        ' as maximum integral over element' )
            end if
            if ( iscal2>0 ) scalars(iscal2) = outval(2)
            if ( iscal3>0 ) scalars(iscal3) = outval(3)
            numscals = max(numscals,iscal2)
            numscals = max(numscals,iscal3)

         end if

         if ( .not. associated(iuser1(iprob)%iuser) ) then

            deallocate ( iuser, user, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'iuser' )

         end if  ! ( .not. associated(iuser1(iprob)%iuser) )

         if ( iseqarray<iseqarrmax ) then

!        --- raise indices

            ivectr = ivectr+1
            iscal1 = iscal1+1
            if ( iscal2>0 ) iscal2 = iscal2+1
            if ( iscal3>0 ) iscal3 = iscal3+1

            if ( newvec ) then

!           --- Copy degree jdegfd in isolcp

               call prextractunk ( isolut2d(ivectr), jdegfd, isolcp, newvec )
            else

!           --- Copy vector into isolcp

               call sepcopyvc ( isolut2d(ivectr), isolcp )

            end if  ! ( newvec )

         end if  ! ( iseqarray<iseqarrmax )

      end do  ! iseqarray = iseqarrmin, iseqarrmax

      iseqarray = iseqarrsav  ! reset iseqarray

      istartmn = istartmn+14+numelgrp

      if ( debug ) write(irefwr,*) 'newvector ', newvector

      if ( newvector ) then

!     --- A new vector has been created, remove from isolut

         call seprmsolinfo ( isolut2d(ivectr) )
         if ( numvecmn==ivectr ) numvecmn = numvecmn-1

      end if  ! ( newvector )

      numvecmn = numvecmnsav  ! restore numvecmn
      numvec = numvecsav

!     --- Remove temporary array

      if ( newvec ) call seprmsolinfo ( isolcp )

      deallocate ( inpvol, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'inpvol' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ivectr, numvecmn, istartmn', ivectr, numvecmn, istartmn
         write(irefwr,*) 'End printg'

      end if  ! ( debug )
      call erclos ( 'printg' )

      end subroutine printg
