      subroutine prinversemap ( map1, map2, kmesh1, kmesh2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-11-2012 Long integers
!        version  1.0  date 10-02-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill reverse mapping array in case mapping array is given
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mapping_array
!     map
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh1, kmesh2
      type (mapping) :: map1, map2

!     kmesh1         i    Standard SEPRAN array containing information of mesh 1
!     kmesh2         i    Standard SEPRAN array containing information of mesh 1
!     map1           i    Contains mapping information of the map from mesh 2
!                         to mesh 1
!     map2           o    At output it is filled with mapping information
!                         of the map from mesh 1 to mesh 2
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer npoint1, npoint2, iseqkm1, iseqkm2

!     ipimap1        Starting address of array imap1
!     ipimap2        Starting address of array imap2
!     iprmap2        Starting address of array rmap2
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     npoint1        number of points in first mesh
!     npoint2        number of points in second mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINMAP        Print the contents of the mapping array map
!     PRINVERSEMAP1  Fill reverse mapping array in case mapping array is given
!     SEPCREATEIBUF  Create permanent space in array IBUFFR
!     SEPCREATERBUF  Create permanent space in array BUFFER
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPFILLKMESHD  Fill kmesh part d
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1656   Array map has not been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinversemap' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinversemap'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkm1 = kmesh1
      iseqkm2 = kmesh2

!     --- get constants

      npoint1 = km(iseqkm1)%kmesh(8)
      npoint2 = km(iseqkm2)%kmesh(8)

!     --- Activate arrays

      call sepfillkmesh(kmesh1)

      if ( kelmd==0 ) call sepfillkmeshd
      if ( ierror/=0 ) go to 1000

!     --- Create array map2

      if ( associated(map2%imap) ) deallocate ( map2%imap, map2%rmap )
      call sepclear ( map2 )
      allocate ( map2%imap(npoint2,ndim+3), map2%rmap(npoint2,ndim+1) )
      map2%iseqkm1 = iseqkm1
      map2%iseqkm2 = iseqkm2
      imap => map1%imap
      rmap => map1%rmap
      imap2 => map2%imap
      rmap2 => map2%rmap

      call prinversemap1 ( npoint1, npoint2, coor, km(iseqkm2)%coor )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( map2, 5, 'map2' )
         call prinmap ( map2, kmesh2 )
         write(irefwr,*) 'End prinversemap'

      end if  ! ( debug )
      call erclos ( 'prinversemap' )

      end subroutine prinversemap
