      subroutine prinversemap1 ( npoint1, npoint2, coor1, coor2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-02-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill reverse mapping array in case mapping array is given
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mapping_array
!     map
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint1, npoint2
      double precision coor1(ndim,npoint1), coor2(ndim,npoint2)

!     coor1          i    Array containing the coordinates of the first mesh
!     coor2          i    Array containing the coordinates of the second mesh
!     npoint1        i    number of points in first mesh
!     npoint2        i    number of points in second mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer node, icount, i

!     i              Counting variable
!     icount         Counter
!     node           Node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINVERSEMAP1D Interpolate node in new mesh from nodes in old mesh
!                    1d case (linear elements)
!     PRINVERSEMAP2D Interpolate node in new mesh from nodes in old mesh
!                    2d case (linear triangles)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     419   NDIM incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinversemap1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinversemap1'
         write(irefwr,1) 'npoint1, npoint2', npoint1, npoint2
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      imap2(1:npoint2,1) = 0

      icount = 0
      do i = 1, npoint1

         if ( imap(i,1)==1 ) then

!        --- Mapping consists of 1 point, hence nodes are equal

            node = imap(i,2)
            imap2(node,1) = 1
            imap2(node,2) = i
            rmap2(node,1) = 1d0
            icount = icount+1

         end if  ! ( imap(i,1)==1 )

      end do  ! i = 1, npoint1
      if ( debug ) write(irefwr,1) 'icount', icount

      if ( icount<npoint2 ) then

!     --- Mapping consists of several points, interpolation is necessary

         select case ( ndim )

         case(1)

!        --- ndim = 1

            call prinversemap1d ( npoint1, npoint2, coor1, coor2 )

         case(2, 3)

!        --- ndim = 2

            call prinversemap2d ( npoint1, npoint2, coor1, coor2 )

         case default

!        --- error: not yet available for this dimension

            call errint ( ndim, 1 )
            call errint ( 2, 2 )
            call errint ( 2, 3 )
            call errsub ( 419, 3, 0, 0 )
            go to 1000

         end select  ! case ( ndim )

      end if  ! ( icount<npoint2 ) then

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinversemap1'

      end if  ! ( debug )
      call erclos ( 'prinversemap1' )

      end
