      subroutine prmatstruc ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 17-12-2015 New call to incommat
!        version  1.2  date 13-01-2015 Extension with degfd
!        version  1.1  date 29-03-2014 Extension with mumps
!
!   copyright (c) 2011-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the structure of the large matrix
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemnsub
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine READSCMATRIX
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer,  parameter :: lniincommat=25
      integer iincop(lniincommat), inpseploc(2), iseqlc, &
              iprobmin, iprobmax, leniincop, lastentryused

!     iincop         Copy of input array extended with defaults
!     inpseploc      Input array for subroutine PRCOMM
!     iprobmax       maximum value of iprob in loop
!     iprobmin       minimum value of iprob in loop
!     iseqlc           Sequence number
!     lastentryused  last entry used in inpsep
!     leniincop      Length of array iincop
!     lniincommat    Declared length of iincommt
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHINPSOL    Adapt input array with respect to variables
!     PRCOMM         New call to COMMAT
!     PRININ         print 1d integer array
!     SEPCOMMAT      Actual body of COMMAT and MATSTRUC
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmatstruc' )
      debug = .false.
      lastentryused = 15 ! last entry for standard storage
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmatstruc'
         call prinin ( inpsep, lastentryused, 'inpsep' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ioutp>=0 ) write(irefwr,*) 'Define matrix structure'

      iseqlc = inpsep(1)
      if ( iseqlc>0 ) then

!     --- iseqlc>0, sequence number is given, get information

         inpseploc(1) = iseqlc
         inpseploc(2) = 1
         call prcomm ( inpseploc )
         leniincop = lastentryused

      else

!     --- iseqlc<=0, sequence number is not given, information in inpsep

         leniincop = lastentryused+inpsep(10)
         iincop(1:leniincop) = inpsep(1:leniincop)
         call inichinpsol ( iincop, leniincop )
         iseqkp = kprob
         nprob  = kp(iseqkp)%nprob

         if ( iincop(5)==0 ) then

!        --- iprob not given

            iprobmin = 1
            iprobmax = nprob

         else

!        --- iprob given

            iprobmin = iincop(5)
            iprobmax = iprobmin

         end if  ! ( iincop(5)==0 )

         do iprob = iprobmin, iprobmax

            iincop(5) = iprob
            intmat => intmat1d(iprob)
            call sepcommat ( iincop )

         end do  ! iprob = iprobmin, iprobmax

      end if  ! ( iseqlc>0 )

1000  istartmn = istartmn + leniincop+1

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmatstruc'

      end if  ! ( debug )
      call erclos ( 'prmatstruc' )

      end
