      subroutine prplotcn ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.12 date 20-01-2015 Use subroutine prcomplevels
!        version  1.11 date 20-05-2013 Loop in case of array
!        version  1.10 date 21-11-2012 Extension with icolor
!
!   copyright (c) 2000-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Help subroutine of TO0094/tofreesb to call the contour plot routine
!     PLOTCN
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     color
!     driving_subroutine
!     plot
!     plot_contour
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesymm
      use sepmodulemain
      use sepmodulemnsub
      use sepmodulekmesh
      use sepmoduleplot
      use sepmoduletexts
      use sepmoduleinput
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)
      double precision rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscplotcn
!     rinsep         i    Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: lencontln=46
      integer iplots(21), jmaxsave, ishift, input(1), ncoorc(1), &
              lentxt, numarrays, ivecseq
      double precision rplots(3), contln(lencontln), coorcn(1)
      character (len=120) textloc

!     contln         User array of length 7 (if used) in which the
!                    user is supposed to have stored the contour levels
!                    from position 6
!     coorcn         Dummy parameter
!                    they are not
!     input          Dummy parameter
!     iplots         Integer input array for plot subroutine
!     ishift         Shift in mesh sequence number for plotting
!     ivecseq        Sequence number of vector to be considered with
!                    respect to the solution array (isol or isolut)
!     jmaxsave       Help variable to save the value of jmax
!     lencontln      Declared length of array contln
!     lentxt         length of text parameter, with trailing blanks deleted
!     ncoorc         Dummy parameter
!     numarrays      Number of indexed vectors in vector array
!     rplots         Real input array for plot subroutine
!     textloc        String to store a local text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIREDUCEVEC   Compute the reduced value of ivecin which indicates the
!                    first sequence number of a (series of) vectors, as well as
!                    the dimension of the series
!     PLOTCHECKREG   Compute number of sub-arrays and ivecseq
!     PRCOMPLEVELS   Compute levels for contour plots and store in array contln
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPCOPSOLSYMM  Copy solution from original mesh to extended
!                    (axi-symmetric) mesh
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPPLOTCN      General plot contour subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill iplots/rplots using inpsep and rinsep
!     Call plot contour subroutine
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prplotcn' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prplotcn'
         call prinin ( inpsep, 13, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Compute number of sub arrays and ivecseq

      call inireducevec ( inpsep(1), ivecseq, numarrays )

      if ( debug ) write(irefwr,1) 'ivecseq', ivecseq
      if ( inpsep(13)==1 ) then

!     --- Use extended mesh and extend solution to new mesh

         call sepcopsolsymm ( ivecseq )
         isolut => isolsymm
         kprob => kprobsymm

      else

!     --- Use original mesh

         isolut => isolut2d(ivecseq)

      end if  ! ( inpsep(13)==1 )

      jmaxsave = jmax  ! save value of jmax

!     --- Compute region to be plotted

      call plotcheckreg ( inpsep, rinsep )

      iplots = 0

      iplots(1) = 21
      if ( isubrmn==40 ) then

!     --- Contour lines

         iplots(2) = 1

      else

!     --- colored levels

         iplots(2) = 2

      end if
      iplots(3) = inpsep(3)
      iplots(4) = 0
      iplots(21) = 0
      rplots(1) = 15d0
      rplots(2) = 1d0

!     --- Fill contln

      call prcomplevels ( contln, iplots(4), iplots(21), lencontln, &
                          inpsep, rinsep, iplots(2) )
      if ( inpsep(7)>0 ) rplots(2) = rinsep(inpsep(7))
      if ( kelmu/=0 ) then
         ishift = 1
         if ( inpsep(13)==1 ) then
            kmesh => kmesh2d(meshsymm+ishift)
         else
            kmesh => kmesh2d(meshnr+ishift)
         end if  ! ( inpsep(13)==1 ) then
         call sepfillkmesh ( kmesh )

      else
         ishift = 0
      end if

      iplots(6) = 10
      iplots(9) = inpsep(10)
      iplots(11) = 2
      iplots(12) = 1
      iplots(20) = 1

      if ( isubrmn==40 ) iplots(5) = inpsep(12) ! color

      if ( debug ) then

!     --- Debug information

         call prinin ( iplots, 21, 'iplots' )
         call prinrl ( rplots, 2, 'rplots' )

      end if  ! ( debug )

      do iseqarray = 1, numarrays

         if ( inpsep(8)>0 ) then

!        --- inpsep(8)>0, hence text found

            textloc = texts(inpsep(8))  ! text
            lentxt = inpsep(9)  ! number of characters of text

         else

!        --- inpsep(8)=0, no text found, use default

            textloc = names(ivecseq)
            lentxt = len_trim(textloc)

         end if  ! ( inpsep(8)>0 )

         if ( iseqarray>1 ) then

            if ( inpsep(13)==1 ) then

!           --- Use extended mesh and extend solution to new mesh

               call sepcopsolsymm ( ivecseq )

            else

               isolut => isolut2d(ivecseq)

            end if  ! ( inpsep(13)==1 )

         end if  ! ( iseqarray>1 )

!        --- Actual plotting

         call sepplotcn ( iplots, rplots, contln, input, &
                          textloc(1:lentxt), ncoorc, coorcn )

         ivecseq = ivecseq+1

      end do  ! iseqarray = 1, numarrays
      iseqarray = 0

      if ( inpsep(2)>0 ) jmax = jmaxsave  ! reset jmax

      kmesh => kmesh2d(meshnr)
      call sepfillkmesh ( kmesh )
      kprob => kprob2d(meshnr)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prplotcn'

      end if
      call erclos ( 'prplotcn' )

      end
