      subroutine prplotvc ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 06-02-2012 Allow indexed vectors
!        version  1.3  date 17-05-2011 plot boundary zero levelset
!        version  1.2  date 07-03-2007 plot text
!
!   copyright (c) 2000-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     PLot vector or tensor depending on array inpsep
!     This is a help subroutine for sepcomp and sepfree
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     vector
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulesymm
      use sepmodulemain
      use sepmodulemnsub
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)
      double precision rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscplotvc
!     rinsep         i    Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision rplots(3)
      integer iplots(10), icode, ishift, ivecseq, jmaxsave, jtimes1, &
              jtimes2, jtimessav, ktimes
      character (len=80) textloc(1)
      integer, pointer :: isol

!     icode          Return code with respect to subroutine INTX01
!                    Indicates position in array code or code1
!     iplots         Integer input array for plot subroutine
!     ishift         Shift in mesh sequence number for plotting
!     isol           Standard SEPRAN array containing information of the
!                    solution array
!     ivecseq        Sequence number of vector to be considered with respect
!                    to the solution array (isol or isolut)
!     jmaxsave       Help variable to save the value of jmax
!     jtimes1        jtimes for second call of plotvc
!     jtimes2        jtimes/10
!     jtimessav      Help variable to save the value of jtimes
!     ktimes         mod(itimes,10)
!     rplots         Real input array for plot subroutine
!     textloc        String to store a local text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inivectr

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIVECTR       Return with sequence number of vector
!     PLOTVCB1       Plot vectors
!     SEPCOPSOLSYMM  Copy solution from original mesh to extended
!                    (axi-symmetric) mesh
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
! **********************************************************************
!
!                       I/O
!
!    A plot of the vector or tensor is made
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     initializations
!     if ( icode=39 ) then
!        PLot vector
!     else
!        PLot tensor
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prplotvc' )

!     --- initializations

      icode = inpsep(1)
      ivecseq = inivectr(inpsep(2))
      if ( inpsep(3)>0 ) then
         jmaxsave = jmax
         jmax = 1
         xmin = rinsep(inpsep(3))
         xmax = rinsep(inpsep(3)+1)
         ymin = rinsep(inpsep(3)+2)
         ymax = rinsep(inpsep(3)+3)
      end if
      rplots(1) = 15d0
      rplots(2) = 1d0
      rplots(3) = 0d0
      if ( inpsep(6)>0 ) rplots(2) = rinsep(inpsep(6))
      if ( inpsep(7)>0 ) rplots(3) = rinsep(inpsep(7))

      if ( inpsep(11)==1 ) then

!     --- Use extended mesh and extend solution to new mesh

         call sepcopsolsymm ( ivecseq )
         isol => isolsymm
         kprob => kprobsymm

      else

!     --- Use original mesh

         isol => isolut2d(ivecseq)

      end if  ! ( inpsep(13)==1 )
      if ( kelmu/=0 ) then
         ishift = 1
         if ( inpsep(11)==1 ) then
            kmesh => kmesh2d(meshsymm+ishift)
         else
            kmesh => kmesh2d(meshnr+ishift)
         end if  ! ( inpsep(13)==1 )
         call sepfillkmesh ( kmesh )
      else
         ishift = 0
      end if

      textloc(1) = names(ivecseq)

      if ( icode==39 ) then

!     --- Plot vector

         iplots(1) = 6
         iplots(2) = inpsep(4)
         iplots(3) = inpsep(5)
         iplots(4) = inpsep(10)
         iplots(5) = 0
         iplots(6) = 1
         call plotvcb1 ( iplots, isol, isol, rplots, textloc )
         if ( inpsep(3)>0 ) jmax = jmaxsave

      else

!     --- Plot tensor
!         First jtimes is adapted so that two vector plots can be made in
!         one picture
!         The factor used by the first plot is reused in the second one

         jtimessav = jtimes
         jtimes2 = jtimes/10
         ktimes = mod(jtimes,10)
         if ( ktimes==0 ) then

!        --- jtimes = 0, only one plot is made

            jtimes = 1+10*jtimes2
            jtimes1 = 3+10*jtimes2

         else if ( ktimes==1 .or. ktimes==2 ) then

!        --- jtimes = 1, 2 first or next plot in a row

            jtimes1 = 2+10*jtimes2

         else if ( ktimes==3 ) then

!        --- jtimes = 3, last plot in a row

            jtimes = 2+10*jtimes2
            jtimes1 = jtimes

         end if

!        --- First the first two components are plotted as vector

         iplots(1) = 6
         iplots(2) = inpsep(4)
         iplots(3) = inpsep(5)
         iplots(4) = inpsep(10)
         if ( inpsep(7)>0 ) then
            iplots(5) = 0
         else
            iplots(5) = 1
         end if
         iplots(6) = 1

         call plotvcb1 ( iplots, isol, isol, rplots, textloc )

!        --- Next the last two components are plotted as vector

         iplots(1) = 3
         iplots(2) = inpsep(8)
         iplots(3) = inpsep(9)
         jtimes = jtimes1
         call plotvcb1 ( iplots, isol, isol, rplots, textloc )
         if ( inpsep(3)>0 ) jmax = jmaxsave
         jtimes = jtimessav

      end if

      kmesh => kmesh2d(meshnr)
      kprob => kprob2d(meshnr)
      call sepfillkmesh ( kmesh )

      call erclos ( 'prplotvc' )
      end subroutine prplotvc
