      subroutine prprintints ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-04-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute intersection of line with 2d mesh and print the results
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     vector
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduletime
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*)
      double precision, intent(in) :: rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine readscplotint
!     rinsep         i    The real input read is stored in array RINSEP
!                         See subroutine readscplotint
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ivectr, i
      double precision values(3)

!     i              Counting variable
!     ivectr         standard sepran array denoting USOL or vector of
!                    special structure defined in nodal points
!     values         Double precision or complex array in which the real
!                    or complex information concerning the line is stored
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: inivectr
      double precision :: getparm

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETPARM        Return with real defined by inpsep/rinsep
!     INIVECTR       Return with sequence number of vector
!     PRININLARGE    Print 1d integer vector (large numbers)
!     PRINRL         Print 1d real vector
!     PRINTINTERSEC  Compute intersection of line with 2d mesh
!                    and print the results
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prprintints' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prprintints'
         write(irefwr,1) 'istartmn', istartmn
         call prininlarge ( inpsep(1), 8, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( itimloop/=0 .and. ioutact/=1 ) go to 900 ! Only at certain time steps

!     --- initializations

      ivectr = inivectr(inpsep(1))   ! sequence number of vector in isolut2d
      do i = 1, 3
         values(i) = getparm ( inpsep, rinsep, i+4, 'values' )
      end do  ! i = 1, 3

      if ( debug ) call prinrl ( values, 3, 'values' )

      isolut => isolut2d(ivectr)
      call printintersec ( inpsep, values, rinsep )

!     --- Raise istartmn

900   istartmn = istartmn+11

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istartmn', istartmn
         write(irefwr,*) 'End prprintints'

      end if  ! ( debug )
      call erclos ( 'prprintints' )

      end
