      function prsclrminmax ( inpsep, rinsep, itype, numminmax, text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 14-11-2013 Extra parameter text
!        version  2.0  date 21-12-2011 Extra parameter kmesh
!        version  1.0  date 30-04-2008
!
!   copyright (c) 2008-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Evaluate scalar i = min/max ( ... )
!
! **********************************************************************
!
!                       KEYWORDS
!
!     fill
!     fill_scalar
!     minimum
!     maximum
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), itype, numminmax
      double precision prsclrminmax, rinsep(*)
      character (len=*) text

!     inpsep         i    The integer input read is stored in array INPSEP
!                         This array contains information of the
!                         values that must be computed from position 1
!                         These values are used to compute minimum or maximum
!                         Per value the rules of PREVALEXPREAL are used.
!                         Information per value is stored sequentially
!                         See subroutine asksca for definition
!     itype          i    Indicates if a minimum (3) or maximum (4) must be
!                         computed
!     numminmax      i    Number of quantities from which minimum or maximum
!                         must be computed
!     prsclrminmax   o    Result of the operation
!     rinsep         i    Array containing the real input for this subroutine
!     text           i    String variable containing some text for error message
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipos, i
      double precision value1, value2

!     i              Counting variable
!     ipos           Position in array
!     value1         Intermediate value of one variable
!     value2         Intermediate value of the operation after evaluation
!                    of the next variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision prevalexpreal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PREVALEXPREAL  Evaluate real, scalar or real expression
!     PRININLARGE    Print 1d integer vector (large numbers)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsclrminmax' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsclrminmax'
         write(irefwr,1) 'itype, numminmax', itype, numminmax
         call prininlarge ( inpsep, numminmax, 'inpsep' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill first item in value2

      ipos = 1
      value2 = prevalexpreal ( inpsep(ipos), rinsep, text )
      ipos = ipos+1

      if ( debug ) then
         write(irefwr,1) 'ipos', ipos
         write(irefwr,2) 'value2',value2
      end if  ! ( debug )

!     --- Fill other items in value1 and the result of the operation
!         in value2

      do i = 2, numminmax

         value1 = prevalexpreal ( inpsep(ipos), rinsep, text )
         ipos = ipos+1

         if ( debug ) then
            write(irefwr,1) 'i, ipos', i, ipos
            write(irefwr,2) 'value1',value1
         end if  ! ( debug )

         if ( itype==3 ) then

!        --- minimum value

            value2 = min ( value1, value2 )

         else

!        --- maximum value

            value2 = max ( value1, value2 )

         end if  ! ( itype==3 )

      end do  ! i = 2, numminmax
      prsclrminmax = value2

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'value2',value2
         write(irefwr,*) 'End prsclrminmax'

      end if  ! ( debug )
      call erclos ( 'prsclrminmax' )

      end
