      subroutine prsetarrcons ( isol, constant, inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 31-12-2013 New parameter list
!        version  1.1  date 25-11-2012 Long integers
!        version  1.0  date 02-02-2008
!
!   copyright (c) 2008-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Set the values of an existing vector equal to a constant
!
! **********************************************************************
!
!                       KEYWORDS
!
!     constant
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol, inpsep(*)
      double precision constant

!     constant       i    Value of constant
!     inpsep         i    The integer input read is stored in array INPSEP
!                         For a description see subroutine ASKVEC
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: itypvc, ivec

!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPCREASOL110  Activate or create arrays of type 110
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      60   Input solution array has not been filled or is of the wrong type
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsetarrcons' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsetarrcons'
         call prinin ( inpsep, 4, 'inpsep' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check array

      if ( isol==0 ) then

!     --- It concerns a new vector
!         Create this vector

         iprob = max(1,inpsep(2))
         call sepcreasol110 ( isol, names(inpsep(3)), 0 )

      else if ( ks(isol)%typevec/=110 .and. ks(isol)%typevec/=115 .and. &
                ks(isol)%typevec/=116 ) then

!     --- Error wrong type of array

         call errint ( ks(isol)%typevec, 1)
         call errsub ( 60, 1, 0, 0 )
         go to 1000

      else

         call sepcreasol110 ( isol, names(inpsep(3)), 0 )

      end if  ! ( isol== )

!     --- Set constant

      usoltemp => ks(isol)%sol
      usoltemp = constant

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsetarrcons'

      end if  ! ( debug )
      call erclos ( 'prsetarrcons' )

      end subroutine prsetarrcons
