      function prstdevval ( usol, nodes, nnodes, nunkp, idegfd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-12-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find standard deviation of vector
!
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     standard_deviation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, nodes(nnodes), nunkp, idegfd
      double precision usol(nunkp,*), prstdevval

!     idegfd         i    Degree of freedom sequence number
!     nnodes         i    Number of nodes
!     nodes          i    Contains the node numbers that must be used
!     nunkp          i    Maximum number of unknowns in a point
!     prstdevval     o    standard deviation
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision meanval
      double precision, allocatable :: diff(:)

!     diff           Difference vector
!     error          Return value of allocate or deallocate
!     meanval        mean value of usol in nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prstdevval' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prstdevval'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Only one degree of freedom

      allocate ( diff(nnodes), stat = error )
      if ( error/=0 ) call eralloc ( error, nnodes, 'diff' )
      if ( ierror/=0 ) go to 1000
      meanval = sum(usol(idegfd,nodes(1:nnodes)))/nnodes

      diff = usol(idegfd,nodes(1:nnodes))-meanval

      prstdevval = sqrt(dot_product(diff,diff)/nnodes)

      deallocate ( diff, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'diff' )

1000  call erclos ( 'prstdevval' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prstdevval'

      end if  ! ( debug )

      end
