      subroutine prstocsolve ( iseqcoeff, iprobw, iseqw )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-11-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve linear system for stochastic investment problem
!
! **********************************************************************
!
!                       KEYWORDS
!
!     stochastic
!     investment
!     linear_solver
!     solve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqcoeff, iprobw, iseqw

!     iprobw         i    ?
!     iseqcoeff      i    Array containing sequence numbers of input for
!                         coefficients
!     iseqw          i    ?
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinsol(7)

!     iinsol         Contains information concerning the linear solver
!                    See the Programmers Guide for a description
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     LINPRBNEW      Build system of equations and solves linear system of
!                    equations
!     PRININ         print 1d integer array
!     SEPPRINTV      Print solution vector or vector of special structure
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prstocsolve' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prstocsolve'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iinsol(1) = 7
      iinsol(2) = 0
      iinsol(3) = iseqcoeff                  ! seq coeff
      iinsol(4) = iprobw                        ! seq solve
      iinsol(5) = iprobw                        ! iprob
      iinsol(6) = iseqw                         ! seq vector
      iinsol(7) = numvecmn                        ! maximum number of vectors
      if ( debug ) call prinin ( iinsol, 7, 'iinsol' )

      intmat => intmat1d(iprobw)
      call linprbnew ( iinsol )

1000  if ( debug ) then

!     --- Debug information

         call sepprintv ( isolut2d(iseqw), 'w' )
         write(irefwr,*) 'End prstocsolve'

      end if  ! ( debug )
      call erclos ( 'prstocsolve' )

      end
