      subroutine prtavs ( ichprt, kmeshlc, kproblc, isol, ntime, &
                          iseqvc, time, scale )
! ======================================================================
!
!        programmer    Guus Segal / Martien Hulsen
!        version  2.5  date 25-11-2012 Long integers
!        version  2.4  date 22-07-2001 New call to topqu0
!        version  2.3  date 14-11-1997 New call to TOPQU0
!
!   copyright (c) 1993-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Write solution array or array of special structure to file sepavs.xxx
!   This possibility is meant for a SEPRAN-AVS interface
!   The 2D SEPRAN mesh is mapped onto a mesh of linear triangles.
!   The 2D mesh is repeated ntime times to contruct a 3D mesh consisting
!   of prisms.
! **********************************************************************
!
!                       KEYWORDS
!
!     avs
!     output
!     vector
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemain
      use sepmodulemach
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichprt, ntime, iseqvc
      integer, target :: kmeshlc, kproblc, isol(ntime)
      double precision time(ntime), scale

!     ichprt         i    choise parameter:
!                         0:  Only time axis is scaled. Scaling according
!                             to the minimum of xmax-xmin, ymax-ymin.
!                         1:  time axis and either x or y is scaled. The
!                             coordinate direction with minimum width and
!     iseqvc         i    Sequence number of vector to be written
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     kmeshlc        i    Standard SEPRAN array containing information
!                         of the mesh
!     kproblc        i    Contains information of the problem definition
!     ntime          i    Number of time sequence numbers
!     scale          i    scale factor for ichprt=1.
!     time           i    Array of actual times
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      character (len=14) filenm
      integer ichois, iresul(5,3), ifill, iskip, iskpar(1), number(4), &
              ndimlc, i, ivec, iref, npelmlc, nelemlc, iprobt, &
              itypvc, itimelc, itypvt, nprev, special, ilevel, &
              nelgrpsav, nelemsav
      logical compl

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     filenm         Name of file
!     i              Counting variable
!     ichois         Choice parameter
!     ifill          Parameter for subroutine TOPQU0 indicating if the extra
!                    array iwork1 must be filled (>0) or not (0)
!     ilevel         dummy
!     iprobt         ?
!     iref           File reference number
!     iresul         Output array of length 5,1 if ichois=2 and otherwise of
!                    length 5,3
!                    The array is filled as follows:
!                    Pos .,1 corresponds to the "vertex" mesh array (ichois=2,3)
!                    Pos .,2 corresponds to the "triangle" mesh array
!                            (ichois=1,3)
!                    Pos .,3 corresponds to the solution array (ichois=1,3)
!                    Pos. 1,1  INDVER sequence number with respect to INFOR
!                                     for the "vertex" mesh array
!                    Pos. 2,1  NPELM  number of points per element for this mesh
!                    Pos. 3,1  NELEM  number of points per element for this mesh
!                    Pos. 4,1  NEWVER indicates if the mesh is new (1) or is
!                                     in fact the standard mesh (0)
!                    Pos. 5,1  - &
!                    Pos. 1,2  INDTRI sequence number with respect to INFOR
!                                     for the "triangle" mesh array
!                    Pos. 2,2  NPELM  number of points per element for this mesh
!                    Pos. 3,2  NELEM  number of points per element for this mesh
!                    Pos. 4,2  NEWTRI indicates if the mesh is new (1) or is
!                                     in fact the standard mesh (0) or the mesh
!                                     created by the "vertex" mesh
!                    Pos. 5,2  NPHYS  Max. number of degrees of freedom
!                    Pos. 1,3  INDSOL sequence number with respect to INFOR
!                                     for the new solution array
!                    Pos. 2,3  NEWSOL indicates if the solution array is new (1)
!                                     or is in fact the standard solution
!                                     array (0)
!                    Pos. 3,3  INDPRF sequence number of KPROBF
!                    Pos. 4,3  INDPRH sequence number of KPROBH
!                    Pos. 5,3  INPPRP sequence number of KPROBP
!     iskip          Indicates if element groups must be skipped (>0) or not (0)
!     iskpar         Dummy parameter
!     itimelc        ?
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     itypvt         ?
!     ivec           Sequence number of vector of special structure
!     ndimlc         Local dimension of space
!     nelemlc        Number of elements
!     nelemsav       Help parameter to save the old value of nelem
!     nelgrpsav      Help parameter to save the old value of nelgrp
!     npelmlc        Number of points per element
!     nprev          number of solution arrays written by PRAVS2.
!     number         Array containing the sequence numbers of the unknowns to be
!                    considered.
!                    Position 1 contains the number of unknowns
!     special        Indicates which special situation is present
!                    Possible values:
!                    0     standard
!                    1     solution vector is of type 116 and a coloured
!                          2d plot is required
!                    2     solution vector is of type 110 or 115 and a colored
!                          2d plot is required, whereas there are no unknowns
!                          in the vertices
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRAVS1         Writing of mesh for AVS
!     PRAVS2         Writing of solution array for AVS
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     TOPQU0         Subdivide mesh if necessary. Map unknowns into an organized
! **********************************************************************
!
!                       I/O
!
!    Output is written to the file sepavs.xxx
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     222   input vector is complex:  not yet allowed.
!     832   Vectors defined per element not permitted in this subroutine
!     833   The solution vector does not correspond to a scalar or a simple
!           vector
!     834   ndim=3, whereas ndimlc=2 and nunkp>1
!     835   Only one-dimensional elements available
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Open file sepavs.xxx
!   Map mesh on appropriate mesh and map solution on this mesh
!   Write solution and mesh information to sepavs.xxx
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prtavs' )

!     --- Open file sepavs.xxx

      iref = abs(iref74)+1
      write ( filenm, 100 ) iseqvc
100   format ( 'sepavs.',i3.3 )
      open ( unit=iref, file=filenm )
      kmesh2d(1) = kmeshlc
      kmesh => kmesh2d(1)

      kprob => kproblc
      nelgrpsav = nelgrp
      nelemsav = nelem

      isolut2d => isol
      isolut => isolut2d(1)

      if ( nsurfs==0 .and. nvolms==0 ) then

!     --- Only one-dimensional elements

         call errsub ( 835, 0, 0, 0 )
         go to 1000

      end if
      if ( nvolms==0 ) then
         ichois = 1
         ndimlc = 2
      else
         write(irefwr,140)
 140     format( /' Volume elements not allowed for writing '/ &
                  ' (x,y,t) data to an AVS file.'/ )
         goto 1000
      end if
      if ( ntime<2 ) then
         write(irefwr,150)
 150     format( /' Number of vectors in a time sequence must', &
                  ' be larger than 1 '/ &
                  ' for writing (x,y,t) data to an AVS file.'/ )
         goto 1000
      end if
      if ( ichprt<0 .or. ichprt>1 ) then
         write(irefwr,170) ichprt
 170     format( /' Invalid ichprt in prtavs ',i5 )
         goto 1000
      end if

!     --- Compute IPROB and offset for array KPROB

      iprob  = ks(isolut2d(1))%iprob
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      indpre = kprobloc(18)
      if ( indpre>0 ) then
         ndtvec = kprobloc(6)
         kprobe => kprobpart%kprobe
         kprobe2 => kprobpart%kprobe2
      end if  ! ( kprobloc(18)>0 )

      itypvc = ks(isolut2d(1))%typevec
      if ( itypvc==110 ) then

!     --- ISOL is of type 110:  solution vector

         nphys = kprobloc(4)

      else if ( itypvc==115 ) then

!     --- ISOL is of type 115:  array of special structure

         ivec   = ks(isolut2d(1))%ivec
         nphys  = kprobe2(ivec)
         if ( nphys<0 ) then

!        --- nphys<0, constant number of degrees of freedom per point

            nphys = -nphys

         else

!        --- nphys>0, number of degrees of freedom per point not constant

            nphys = maxval(kprobe(:,ivec))

         end if

      else

!     --- Wrong type of solution array

         call errsub ( 832, 0, 0, 0 )
         go to 1000

      end if
      if ( nphys==1 ) then
         number(1) = 1
         number(2) = 1
      else if ( nphys==ndimlc ) then
         number(1) = ndimlc
         number(2) = 1
         number(3) = 2
         number(4) = 3
      else
         call errint ( nphys, 1 )
         call errint ( ndimlc, 2 )
         call errsub ( 833, 2, 0, 0 )
         go to 1000
      end if
      compl = ks(isolut2d(1))%compl==1
      if ( compl ) then
         call errsub ( 222, 0, 0, 0 )
         go to 1000
      end if
      iskip = 0
      ifill = 0
      if ( ndim/=ndimlc .and. nphys>1 ) then

!     --- dimension of space = 3, but no volume elements

         call errsub ( 834, 0, 0, 0 )
         go to 1000

      end if

!     --- Check remaining solutions vectors

      do 130 itimelc = 2, ntime
         iprobt = ks(isolut2d(itimelc))%iprob
         itypvt = ks(isolut2d(itimelc))%typevec
         if ( iprobt/=iprob .or. itypvt/=itypvc ) then
            write(irefwr,160)
 160        format( /' Solution vectors should be identical for '/ &
                     ' writing (x,y,t) data to an AVS file.'/ )
            goto 1000
         end if
 130  continue

      nprev = 0
      do 120 itimelc = 1, ntime
         special = 0
         ilevel = 1
         isolut => isol(itimelc)
         call topqu0 ( ichois, number, iskip, iskpar, &
                       iresul, compl, ifill, special, ilevel )

         if ( itimelc==1 ) then

!        --- Write mesh information to sepavs.xxx

            npelmlc = iresul(2,2)
            nelemlc = iresul(3,2)
            call pravs1 ( ichprt, scale, iref, coor, ndim, &
                          npoint, nelemlc, npelmlc, kmeshc, nphys, &
                          iseqvc, time, ntime )

         end if

!        --- Write solution to sepavs.xxx

         call pravs2 ( nprev, iref, npoint, nphys, usolt )
         nprev = nprev + 1
         if ( iresul(2,3)==1 ) call seprmsolinfo ( iresul(1,3) )

 120  continue

      close ( iref )

      kmeshc => km(iseqkm)%kmeshc
      nelgrp = nelgrpsav
      nelem = nelemsav
      inelem = nelem

1000  call erclos ( 'prtavs' )
      end subroutine prtavs
