      subroutine prtecplot ( isol, jtime, nametec, one_file, maxvec1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 16-04-2014 Extra parameter maxvec1
!        version  2.0  date 10-04-2014 Complete new version
!        version  1.3  date 20-06-2013 New definition file ref numbers
!
!   copyright (c) 2002-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write solution array or array of special structure to file septec.yyy
!     This possibility is meant for a SEPRAN-tec interface
!     The SEPRAN mesh is mapped onto a mesh of linear triangles or tetrahedrons
!     only
! **********************************************************************
!
!                       KEYWORDS
!
!     tecplot
!     output
!     vector
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulemach
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*), jtime, nametec, one_file, maxvec1

!     isol         i    Standard SEPRAN array containing information of the
!                         solution array
!     jtime          i    Integer time sequence number
!     maxvec1        i    ?
!     nametec        i    Sequence number of name of tec file in array
!                         file_names
!     one_file       i    Indicates if all time steps must be stored in one file
!                         (1) or not (0)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      character (len=104) filenm
      character (len=100) filename
      integer nunkp, iref, i, npointsav, &
              nchar, ifirst, kmeshtmp, numveccomps(100), &
              numvc1
      logical :: new_mesh
      save iref
      double precision, allocatable :: usol(:)

!     filename       Name of file
!     filenm         Name of file
!     i              Counting variable
!     ifirst         If ifirst = 0, the file is new and all information
!                    must be stored.
!                    If ifirst = 1, aready some information has been
!                    stored in the file and mesh information is not
!                    repeated
!     iref           File reference number
!     kmeshtmp       Contains information of the old mesh
!     nchar          Number of characters in a word
!     new_mesh       If true a new mesh is created
!     npointsav      Help parameter to save the old value of npoint
!     numvc1         Number of vectors stored in isol
!     numveccomps    Array to store the number of degrees of freedom per
!                    point for each vector to be written
!     nunkp          Maximum number of unknowns in a point
!     usol           temporary solution array organized per node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inisetnewref

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INIFILENAME    Create filename from indfile and namefile
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     INISETNEWREF   Create new reference number
!     PRININ         print 1d integer array
!     PRTECPLOT0     Actual writing of solution array for TECPLOT
!     PRTOAVSDIM     Define number of components and number of vectors for
!                    paraview output
!     PRTOAVSEXT0    Create array of length npoint x nunkp
!                    Copy sepran solutions to this array organized per unknown
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPRMMESHINFO  Remove mesh information
!     TOTRANSMESH    Transform quadratic mesh to linear mesh
! **********************************************************************
!
!                       I/O
!
!    Output is written to the file septec_xxx.yyy
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Open file septec_xxx.yyy
!     Map mesh on appropriate mesh and map solution on this mesh
!     Write solution and mesh information to septec_xxx.yyy
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prtecplot' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prtecplot'
         write(irefwr,1) 'nametec, jtime', nametec, jtime
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( jtime==1 .or. one_file==0 ) then

!     --- Open file septec.yyy

         iref = inisetnewref()
         call inifilename ( nametec, 'septec', filename, nchar )
         write ( filenm, 100 ) filename(1:nchar), jtime
100      format ( a,'.',i3.3 )
         open ( unit=iref, file=filenm )
         ifirst = 0

      else

!     --- Reuse file

         ifirst = 1
         iref = abs(iref74)+1

      end if

!     --- Transform mesh into linear mesh

      call totransmesh ( kmeshtmp, new_mesh )
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,*) 'new_mesh ', new_mesh

!     --- The number of unknowns depends on what is to be written
!         Adapt nunkp and numvc1
!         Fill numveccomps

      call prtoavsdim ( isol, nunkp, numvc1, numveccomps, &
                        maxvec1, ndim )

      if ( debug ) then
         write(irefwr,1) 'nunkp, numvc1, maxvec1', nunkp, numvc1, maxvec1
         call prinin ( numveccomps, numvc1, 'numveccomps' )
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- write solution
!         The solution must be written in the form:
!         node number, components
!         So it is necessary to create an array with nunkp components per point
!         This array is put at the back of ibuffr temporarily

      if ( new_mesh ) then

!     --- New mesh is used

         length = nunkp*npoint
         if ( debug ) write(irefwr,1) 'npoint, length', npoint, length

         allocate ( usol(length), stat = error )
         if ( error/=0 ) call eralloc ( error, length, 'usol' )
         if ( ierror/=0 ) go to 1000

         call prtoavsext0 ( isol, usol, maxvec1 )

      else

!     --- Old mesh is used

         length = nunkp*npoint
         if ( debug ) write(irefwr,1) 'npoint, length', npoint, length

         allocate ( usol(length), stat = error )
         if ( error/=0 ) call eralloc ( error, length, 'usol' )
         if ( ierror/=0 ) go to 1000

         call prtoavsext0 ( isol, usol, maxvec1 )

      end if  ! ( new_mesh )
      if ( debug ) write(irefwr,*) 'after prtoavsext0'
      if ( ierror/=0 ) go to 1000

!     --- Write solution and mesh information to septec_xxx.yyy

      if ( debug ) &
         write(irefwr,1) 'ndim, npoint, nelem, npelm, nunkp', &
                          ndim, npoint, nelem, npelm, nunkp

      call prtecplot0 ( iref, coor, ndim, npoint, &
                        nelem, npelm, kmeshc, usol, &
                        nunkp, ifirst, isol, maxvec1 )

      if ( one_file==0 ) then
         close ( iref )
         call inirmref ( iref )
      end if  ! ( one_file==0 )

      if ( new_mesh ) then

!     --- Remove new mesh and velocity

         call seprmmeshinfo ( kmesh )

!        --- Copy old mesh from kmeshtmp into kmesh

         kmesh = kmeshtmp
         call sepfillkmesh(kmesh)

      end if  ! ( new_mesh )

      deallocate ( usol, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'usol' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prtecplot'

      end if  ! ( debug )
      call erclos ( 'prtecplot' )

      end subroutine prtecplot
