      subroutine prtimehis ( inpsep, rinsep, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 05-12-2013 Correction call to prtimehis5
!        version  1.2  date 25-11-2012 Long integers
!        version  1.1  date 14-10-2012 Allow array coor
!
!   copyright (c) 2010-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Information concerning the storage of time history is
!     stored in common block ctimhis
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_history
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletimhis
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), istart
      double precision rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         For a description see subroutine readsctimhis
!     istart        i/o   Starting address in array INPSEP
!     rinsep        i/o   Array containing the real input for this subroutine
!                         For a description see subroutine readsctimhis
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivect, istrin, nch1, &
              itypenodes, i, ilow, itop
      character (len=80) namear

!                    they are not
!     i              Counting variable
!     ilow           Lower boundary of i
!     istrin         Starting address of information for this subroutine
!                    in RINSEP
!     itop           Upper boundary of i
!     itypenodes     Defines how coordinates are found
!                    1: coordinates are given
!                       the closest point is used
!                    2: user points are given
!     ivect          Vector sequence number
!     namear         Name of array
!     nch1           Number of characters in name
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRTIMEHIS1     Fill integer array corresponding to time_history partly
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2764   Integer parameter has wrong value
!    2951   Sequence number of vectors out of range
!    3050   Vector for time history already in use
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prtimehis' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prtimehis'
         call prinin ( inpsep, 6, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- get parameters

      ivect = inpsep(1)      ! vector sequence number
      itypenodes = inpsep(2) ! type of nodes
      istrin = inpsep(3)     ! starting address in rinsep
      npoint_hist = inpsep(4)    ! number of points

      if ( debug ) then
         write(irefwr,1) 'ivect, itypenodes, istrin, npoint_hist', &
                          ivect, itypenodes, istrin, npoint_hist
      end if  ! ( debug )

      if ( ivect>0 ) then

!     --- ivect>0

         nch1  = index(names(ivect),' ')-1

         ivecttimlast = max(ivecttimlast,ivect)
         if ( vecttim(ivect)%npoint_hist/=0 ) then

!        --- The time history for this vector has already been stored

            if ( nch1<=0 ) nch1 = len(names(ivect))
            call errint ( ivect, 1 )
            call errchr ( names(ivect)(1:nch1), 1 )
            call errsub ( 3050, 1, 0, 1 )

         end if  ! ( vecttim(ivect)/=0 )

!        --- Define integer vector for time history

         namear = names(ivect)(1:nch1)//'_timehis'
         vecttim(ivect)%npoint_hist = npoint_hist
         vecttim(ivect)%zerolev = .false.
         allocate ( vecttim(ivect)%point_nrs(npoint_hist) )

      else if ( ivect<=-1000 .and. ivect>-1004 ) then

!     --- coor found:-1000 =coor -1001: xcoor etc

         ilow = max(-ivect-1000,1)        ! lower bound
         itop = min(-ivect-1000,ndim) ! upper bound
         if ( itop==0 ) itop = ndim

         if ( debug ) &
           write(irefwr,1) 'ilow, itop, maxvc1', ilow, itop, maxvc1

         do i = ilow+maxvc1, itop+maxvc1
            if ( vecttim(i)%npoint_hist/=0 ) then
               call errint ( i, 1 )
               call errchr ( 'coor', 1 )
               call errsub ( 3050, 1, 0, 1 )
            end if  ! ( vecttim(i)/=0 )
            if ( debug ) write(irefwr,1) 'i', i
            vecttim(i)%npoint_hist = npoint_hist
            allocate ( vecttim(i)%point_nrs(npoint_hist) )
         end do  ! i = 1, ndim

         if ( ivect==-1000 ) then
            ivect = maxvc1+1
         else
            ivect = maxvc1-ivect-1000
         end if  ! ( ivect==-1000 )

      else

!     --- value of ivect incorrect

         call errint ( ivect, 1 )
         call errint ( maxvc1, 2 )
         call errsub ( 2951, 2, 0, 0 )

      end if  ! ( ivect>0 )
      if ( ierror/=0 ) go to 1000

      select case (itypenodes)
      case(1)

!     --- itypenodes=1 Coordinates found
!         Get starting addresses

         ivecttim => vecttim(ivect)%point_nrs

!        --- Fill integer array partly by finding nodes close to coordinates

         call prtimehis1 ( rinsep(istrin) )

!        --- Raise istart

         istart = istart+5

      case(2)

!     --- Points found
!         Get starting address

         ivecttim => vecttim(ivect)%point_nrs

!        --- Fill integer array partly

         ivecttim(1:npoint_hist) = kmeshl(inpsep(5:npoint_hist+4))
         istart = istart+5+npoint_hist

      case(3)

!     --- zero-level set found

         ivecttim => vecttim(ivect)%point_nrs
         vecttim(ivect)%zerolev = .true.

!        --- Fill integer array partly

         ivecttim(1:npoint_hist) = inpsep(3:npoint_hist+2)
         istart = istart+5

      case default

!     --- Not yet implemented

         call errint ( itypenodes, 1 )
         call errchr ( 'itypenodes', 1 )
         call errsub ( 2764, 1, 0, 1 )

      end select  ! case (itypenodes)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ivecttimlast', ivecttimlast
         write(irefwr,*) 'End prtimehis'

      end if  ! ( debug )
      call erclos ( 'prtimehis' )

      end
