      subroutine prtimehis1 ( rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-05-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill integer array corresponding to time_history partly
!
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_history
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletimhis
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision rinsep(*)

!     rinsep         i    Array containing the real input for this subroutine
!                         The starting address must correspond to the first
!                         coordinate of the first point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, idim1, idim2, j

!     i              Counting variable
!     idim1          ndim-1
!     idim2          max(0,ndim-2)
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     FINDXY         Find nodal point closest to a specific point
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prtimehis1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prtimehis1'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ndim, npoint, npoint_hist', ndim, npoint, npoint_hist
         call prinrl1 ( rinsep, npoint_hist, ndim, 'rinsep' )

      end if  ! ( debug )

!     --- Loop over points

      idim1 = min(1,ndim-1)
      idim2 = min(2,ndim-1)
      if ( debug) write(irefwr,1) 'idim1, idim2', idim1, idim2
      do i = 1, npoint_hist
         j = (i-1)*ndim+1
         call findxy ( coor, rinsep(j), rinsep(j+idim1), &
                       rinsep(j+idim2), npoint, ivecttim(i), ndim, 0 )
      end do  ! i = 1, npoint_hist

1000  if ( debug ) then

!     --- Debug information

         call prinin ( ivecttim, npoint_hist, 'ivecttim' )
         write(irefwr,*) 'End prtimehis1'

      end if  ! ( debug )
      call erclos ( 'prtimehis1' )

      end
