      subroutine prtimehis6 ( indim, ipoints )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-10-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill one time step in time history array for coordinates array
!     Mark that istephis must have been raised before the call of this
!     subroutine
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_history
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletimhis
      use sepmoduletime
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer indim, ipoints(npoint_hist)

!     indim          i    Sequence number of coordinate direction
!     ipoints        i    Contains the nodal points where the time history
!                         must be stored
!     ndim           i    Dimension of the space.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inode

!     i              Counting variable
!     inode          Node sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prtimehis6' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prtimehis6'
         call prinin ( ipoints, npoint_hist, 'ipoints' )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ipoints(1)==0 ) go to 1000  ! jump in case no nodes

      if ( istephis>1 ) then

!     --- istephis>1, check if t is equal to previous value
!         if so overwrite previous time step

         if ( abs(rvectim(1,istephis-1)-t)<=epsmac*t ) &
            istephis = istephis-1

      end if  ! ( istephis>1 )

!     --- Store time step

      rvectim(1,istephis) = t

!     --- Store solution

      do i = 1, npoint_hist
         inode = ipoints(i)
         rvectim(i+1,istephis) = coor(indim,inode)
      end do  ! i = 1, npoint_hist

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'rvectim', rvectim(1:npoint_hist+1,1)
         write(irefwr,*) 'End prtimehis6'

      end if  ! ( debug )
      call erclos ( 'prtimehis6' )

      end
