      subroutine prtimpresscor2 ( igq, iproblc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 25-11-2012 Long integers
!        version  1.1  date 02-12-2005 Minor change
!        version  1.0  date 10-10-2005
!
!   copyright (c) 2005-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill zero boundary conditions into vector Gq
!
! **********************************************************************
!
!                       KEYWORDS
!
!     essential_boundary_condition
!     time_integration
!     navier_stokes_equation
!     pressure_correction
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemain
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer igq(*), iproblc

!     igq            i    Sepran array of length containing information
!                         concerning the array Gq
!                         (has same structure as velocity vector)
!     iproblc       i    Actual problem number for pressure
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRTIMPRESSCOR3 Fill zero boundary conditions into vector Gq
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     activate arrays
!     get starting addresses
!     fill boundary conditions
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prtimpresscor2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prtimpresscor2'

      end if
      if ( ierror/=0 ) go to 1000

      iprob = iproblc
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc

!     --- Jump in case nbound = 0
!         That implies no essential boundary conditions

      if ( kprobloc(10)==0 ) go to 1000

!     --- get constants

      nusol = kprobloc(5)
      nrusol = kprobloc(66)

!     --- get starting addresses

      usoltemp => ks(igq(1))%sol
      kprobhinv => kprobpart%kprobhinv

!     --- fill boundary conditions

      call prtimpresscor3 ( usoltemp )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prtimpresscor2'

      end if
      call erclos ( 'prtimpresscor2' )

      end subroutine prtimpresscor2
