      subroutine read02_420
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-12-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read coefficients in case of type number 420
!
! **********************************************************************
!
!                       KEYWORDS
!
!     type_number
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmoduleactual
      use sepmodulekmesh
      use sepmodulestrings
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, icoefforce, iforce

!     i              Counting variable
!     icoefforce     Coefficient sequence number w.r.t. force vector
!     iforce         Sequence number of FORCE in code
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     READ02_FORCE   Read coefficients for force vector part
!     READ02FILLC    Read and fill information concerning coefficient
!                    into iincof/rincof
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'read02_420' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read02_420'
         write(irefwr,1) 'ndim, lastitem', ndim, lastitem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepcopycode ( codenavstoks, nncodenavstoks, nscodenavstoks, &
                         ncodenavstoks )
      call sepcopycodesub ( codemodel, 6, nscodemodel )
      nncodesub=13

      codesub(7:9) = codecsys
      ncodmxsub(7:9) = nscodecsys
      codesub(10:13) = codelinear
      ncodmxsub(10:13) = nscodelinear

      itype(1:nncode) = 44
      table(1:nncode) = -1d20
      tabmin(1:nncode) = -1d20
      tabmax(1:nncode) = 1d20
      nmain = 0
      iretur = -11  ! keyword has been read

      tabmin(5) = 0d0        ! penalty
      tabmin(12:14) = 1d-20  ! viscosity parameters must be positive

      itype(1) = 5   ! MODEL
      tabmin(1) = 1d0
      tabmax(1) = tabmin(1)+5d0

      itype(2) = 0   ! INTEGRATION_RULE
      tabmin(2) = 0d0
      tabmax(2) = 4d0

      itype(3:4) = 5   ! COORDINATE_SYSTEM
      tabmin(3) = tabmax(1)+1d0
      tabmax(3) = tabmin(3)+2d0

      tabmin(4) = tabmax(3)+1d0   ! LINEARIZATION
      tabmax(4) = tabmin(4)+3d0

      call sepreadsetkeyw
      if ( debug ) call prinrl ( table, nncode, 'table' )

      lastitem = 1   ! indicates that one keyword has been read
      ityprd = 4

      icoef = 0

      if ( table(4)>0d0 ) then

!     --- LINEARIZATION  found

         icoef = 3
         iincof(iposin+1) = 1      ! integer found
         iincof(iposin+2) = icoef
         iincof(iposin+3) = nint(table(4))-1
         iposin = iposin+3
         go to 1000

      end if  ! ( table(4)>0d0 )

      do i = 5, 6

         if ( table(i)>0d0 ) then

!        --- penalty, density

            icoef = i-4
            call read02fillc
            go to 1000

         end if  ! ( table(i)>0d0 )

      end do  ! i = 5, 6

      if ( table(7)>-1d19 ) then

!     --- ANGULAR_VELOCITY found

         icoef = 4
         call read02fillc
         go to 1000

      end if  ! ( table(7)>0d0 )

      do i = 15, 18

         if ( table(i)>0d0 ) then

!        --- VOLUME_EXPANSION_COEFFICIENT ... THERMAL_CONDUCTIVITY

            icoef = i-10
            call read02fillc
            go to 1000

         end if  ! ( table(i)>0d0 )

      end do  ! i = 15, 18

!     --- Force vector

      iforce = 8
      icoefforce = 9
      call read02_force ( iforce, icoefforce )
      if ( icoef>0 ) go to 1000

      if ( table(19)>-1d19 ) then

!     --- HEAT_SOURCE found

         icoef = 11
         call read02fillc
         go to 1000

      end if  ! ( table(19)>0d0 )

      do i = 12, 14

         if ( table(i)>0d0 ) then

!        --- viscosity, power_n, lambda

            icoef = i+1
            call read02fillc
            go to 1000

         end if  ! ( table(i)>0d0 )

      end do  ! i = 12, 14

      if ( table(1)>0d0 ) then

!     --- MODEL  found

         icoef = 12
         iincof(iposin+1) = 1      ! integer found
         iincof(iposin+2) = icoef
         iincof(iposin+3) = nint(table(1))
         iposin = iposin+3
         go to 1000

      end if  ! ( table(1)>0d0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End read02_420'

      end if  ! ( debug )
      call erclos ( 'read02_420' )

      end subroutine read02_420
