      subroutine read02_702
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 16-02-2015 Correction irule
!        version  1.0  date 30-10-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read coefficients in case of type number 702
!     diffusion equation
! **********************************************************************
!
!                       KEYWORDS
!
!     type_number
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmoduleactual
      use sepmodulekmesh
      use sepmodulestrings
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ih, ihint, ihdens, ihsour, ihcoor

!     i              Counting variable
!     ih             Start address in code1
!     ihcoor         Start address of coordinate_system in code
!     ihdens         Start address of density in code
!     ihint          Start address of integration_rule in code
!     ihsour         Start address of source in code
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     READ02_DIFF    Store coefficients for diffusion and source term for type
!                    800
!     READ02FILLC    Read and fill information concerning coefficient
!                    into iincof/rincof
!     READ02FILLI    Read and fill information concerning integer coefficient
!                    into iincof
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!     icoef3 = integration_rule
!     icoef4 = coordinate_system
!     coeff6-11: diffusion input
!     coef12 = source
!     coef13 = density
!     coef14 = heat_capacity
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
!     code       contains keywords recognized for coefficients type 703
!                 1: DIFFUSION
!                 2: DIFF_X
!                 3: DIFF_Y
!                 4: DIFF_Z
!                 5: DIFF_XX
!                 6: DIFF_XY
!                 7: DIFF_XZ
!                 8: DIFF_YY
!                 9: DIFF_YZ
!                10: DIFF_ZZ
!                11: SOURCE
!                12: INTEGRATION_RULE
!                13: COORDINATE_SYSTEM
!                14: DENSITY
!                15: HEAT_CAPACITY
!     code1      contains subkeywords
!                 1: CARTESIAN
!                 2: AXI_SYMMETRIC
!                 3: POLAR
! ======================================================================
!
      call eropen ( 'read02_702' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read02_702'
         write(irefwr,1) 'ndim, lastitem', ndim, lastitem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nncode=15

      ih = nncodediffus
      ihsour = ih+1      !SOURCE
      ihint = ihsour+1
      ihcoor = ihint+1   ! COORDINATE_SYSTEM
      ihdens = ihcoor+1  ! DENSITY

      code(1:nncodediffus) = codediffus(1:nncodediffus)
      ncode(1:nncodediffus) = nscodediffus(1:nncodediffus)
      ncodmx(1:nncodediffus) = ncodediffus(1:nncodediffus)
      code(ih+1:ih+3) = codegen(1:3)
      ncode(ih+1:ih+3) = nscodegen(1:3)
      ncodmx(ih+1:ih+3) = ncodegen(1:3)
      code(ihdens) = codegen(5)
      ncode(ihdens) = nscodegen(5)
      ncodmx(ihdens) = ncodegen(5)
      code(ihdens+1) = codegen(8)
      ncode(ihdens+1) = nscodegen(8)
      ncodmx(ihdens+1) = ncodegen(8)
      call sepcopycodesub ( codecsys, nncodecsys, nscodecsys )

      itype(1:nncode) = 44
      table(1:nncode) = -1d20
      tabmin(1:nncode) = -1d20
      tabmax(1:nncode) = 1d20
      nmain = 0
      iretur = -11  ! keyword has been read
      ityprd = 4    ! indicates that the first item is a string

      tabmin(1:nncodediffus) = 0d0

      ihint = ihsour+1
      itype(ihint) = 0
      tabmin(ihint) = 0d0
      tabmax(ihint) = 4d0

      ihcoor = ihint+1   ! COORDINATE_SYSTEM
      itype(ihcoor) = 5
      tabmin(ihcoor) = 1d0
      tabmax(ihcoor) = tabmin(ihcoor)+nncodecsys-1

      call sepreadsetkeyw
      if ( debug ) call prinrl ( table, nncode, 'table' )

      lastitem = 1   ! indicates that one keyword has been read
      ityprd = 4

      icoef = 0

!     --- Store diffusion part

      call read02_diff
      if ( icoef>0 ) go to 1000

      if ( table(ihsour)>-1d19 ) then

!     --- SOURCE found

         icoef = 12
         call read02fillc
         go to 1000

      end if  ! ( table(ihsour)>0d0 )

      if ( table(ihint)>0d0 ) then

!     --- table(ihint)> 0, INTEGRATION_RULE

         icoef = 3
         call read02filli ( table(ihint), 0 )
         go to 1000

      end if  ! ( table(ihint)>0d0 )

      if ( table(ihcoor)>0d0 ) then

!     --- table(ihcoor)> 0, COORDINATE_SYSTEM

         icoef = 4
         call read02filli ( table(ihcoor), 1 )
         go to 1000

      end if  ! ( table(ihcoor)>0d0 )

      if ( table(ihdens)>=0d0 ) then

!     --- table(14)> 0, DENSITY

         icoef = 13
         call read02fillc
         go to 1000

      end if  ! ( table(ihdens)>0d0 )

      if ( table(ihdens+1)>=0d0 ) then

!     --- table(15)> 0, heat_capacity

         icoef = 14
         call read02fillc
         go to 1000

      end if  ! ( table(ihdens)>0d0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icoef', icoef
         write(irefwr,*) 'End read02_702'

      end if  ! ( debug )
      call erclos ( 'read02_702' )

      end subroutine read02_702
