      subroutine read02_703
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 16-02-2015 Correction irule
!        version  1.0  date 30-10-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read coefficients in case of type number 703
!
! **********************************************************************
!
!                       KEYWORDS
!
!     type_number
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmoduleactual
      use sepmodulekmesh
      use sepmodulestrings
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ih, ihvel, ihint, ihconv, ihdens, ihheat, &
                 ihsour, ihsupg, ihcoor, ihupwind

!     i              Counting variable
!     ih             Start address in codemain
!     ihconv         Start address of convection in code
!     ihcoor         Start address of coordinate_system in code
!     ihdens         Start address of density in code
!     ihheat         Start address of heat_capacity in code
!     ihint          Start address of integration_rule in code
!     ihsour         Start address of source in code
!     ihsupg         Start address of supg in code
!     ihupwind       Start address of APPLY_UPWIND in code
!     ihvel          Start address of velocity in code
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     READ02_DIFF    Store coefficients for diffusion and source term for type
!                    800
!     READ02_VEL     Store coefficients for velocity term for type 800
!     READ02FILLC    Read and fill information concerning coefficient
!                    into iincof/rincof
!     READ02FILLI    Read and fill information concerning integer coefficient
!                    into iincof
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!     icoef1 = typediff
!     icoef2 = supg
!     icoef3 = integration_rule
!     icoef4 = coordinate_system
!     icoef5 = type_convection
!     coeff6-11: diffusion input
!     coeff12-14: convection input
!     coef15 = source
!     coef16 = density
!     coef17 = heat_capacity
!     icoef20= applyupw
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
!     code       contains keywords recognized for coefficients type 703
!                 1: DIFFUSION
!                 2: DIFF_X
!                 3: DIFF_Y
!                 4: DIFF_Z
!                 5: DIFF_XX
!                 6: DIFF_XY
!                 7: DIFF_XZ
!                 8: DIFF_YY
!                 9: DIFF_YZ
!                10: DIFF_ZZ
!                11: SOURCE
!                12: VELOCITY
!                13: U_VELOCITY
!                14: V_VELOCITY
!                15: W_VELOCITY
!                16: INTEGRATION_RULE
!                17: COORDINATE_SYSTEM
!                18: SUPG
!                19: DENSITY
!                20: HEAT_CAPACITY
!                21: TYPE_CONVECTION
!                22: APPLY_UPWIND
!     codemain      contains subkeywords
!                 1: CARTESIAN
!                 2: AXI_SYMMETRIC
!                 3: POLAR
!                 4: NONE
!                 5: CLASIC
!                 6: ILIN
!                 7: DOUBLY_ASSYMPTOTIC
!                 8: CRITICAL
!                 9: HUGHES
!                10: TIME_DEPENDENT
!                11: DC1
!                12: DC2
!                13: MIZUKAMI_MAX
!                14: MIZUKAMI_ACT
!                15: MIZUKAMI_USE
!                16: TORNBERG_TIME
! ======================================================================
!
      call eropen ( 'read02_703' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read02_703'
         write(irefwr,1) 'ndim, lastitem', ndim, lastitem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nncode=nncodediffus+nncodevel+8
      nncodesub=nncodeupw+nncodecsys+nncodecompr+nncodeapply

      itype(1:nncode) = 44
      table(1:nncode) = -1d20
      tabmin(1:nncode) = -1d20
      tabmax(1:nncode) = 1d20
      nmain = 0
      iretur = -11  ! keyword has been read
      ityprd = 4    ! indicates that the first item is a string

      ihvel = nncodediffus+1
      ihsour = ihvel+nncodevel      !SOURCE
      ih = ihsour-1
      ihint = ihsour+1    ! INTEGRATION_RULE
      ihcoor = ihint+1    ! COORDINATE_SYSTEM
      ihsupg = ihcoor+1   ! SUPG
      ihdens = ihsupg+1   ! DENSITY
      ihconv = ihdens+1   ! TYPE_CONVECTION
      ihheat = ihconv+1   ! HEAT_CAPACITY
      ihupwind = ihheat+1 ! APPLY_UPWIND

      code(1:nncodediffus) = codediffus(1:nncodediffus)
      ncode(1:nncodediffus) = nscodediffus(1:nncodediffus)
      ncodmx(1:nncodediffus) = ncodediffus(1:nncodediffus)
      code(ihvel:ihsour-1) = codevel(1:nncodevel)
      ncode(ihvel:ihsour-1) = nscodevel(1:nncodevel)
      ncodmx(ihvel:ihsour-1) = ncodevel(1:nncodevel)
      code(ih+1:ih+6) = codegen(1:6)
      ncode(ih+1:ih+6) = nscodegen(1:6)
      ncodmx(ih+1:ih+6) = ncodegen(1:6)
      code(ih+7:ih+8) = codegen(8:9)
      ncode(ih+7:ih+8) = nscodegen(8:9)
      ncodmx(ih+7:ih+8) = ncodegen(8:9)

      codesub(1:nncodecsys) = codecsys
      ncodesub(1:nncodecsys) = nscodecsys
      ih = nncodecsys
      codesub(ih+1:ih+nncodeupw) = codeupw
      ncodesub(ih+1:ih+nncodeupw) = nscodeupw
      ih = ih+nncodeupw
      codesub(ih+1:ih+nncodecompr) = codecompr
      ncodesub(ih+1:ih+nncodecompr) = nscodecompr
      ih = ih+nncodecompr
      codesub(ih+1:ih+nncodeapply) = codeapply
      ncodesub(ih+1:ih+nncodeapply) = nscodeapply

      tabmin(1:nncodediffus) = 0d0

      itype(ihint) = 0
      tabmin(ihint) = 0d0
      tabmax(ihint) = 4d0
      itype(ihcoor) = 5
      tabmin(ihcoor) = 1d0
      tabmax(ihcoor) = tabmin(ihcoor)+nncodecsys-1
      ih = nncodecsys

      ihsupg = ihcoor+1   ! SUPG
      itype(ihsupg) = 5
      tabmin(ihsupg) = ih+1
      tabmax(ihsupg) = ih+nncodeupw

      call sepreadsetkeyw
      if ( debug ) call prinrl ( table, nncode, 'table' )

      lastitem = 1   ! indicates that one keyword has been read
      ityprd = 4

      icoef = 0

!     --- Store diffusion part

      call read02_diff
      if ( icoef>0 ) go to 1000

!     --- Store velocity part

      call read02_vel ( ihvel, 12 )
      if ( icoef>0 ) go to 1000

      if ( table(ihsour)>-1d19 ) then

!     --- SOURCE found

         icoef = 15
         call read02fillc
         go to 1000

      end if  ! ( table(ihsour)>0d0 )

      if ( table(ihint)>0d0 ) then

!     --- table(ihint)> 0, INTEGRATION_RULE

         icoef = 3
         call read02filli ( table(ihint), 0 )
         go to 1000

      end if  ! ( table(ihint)>0d0 )

      if ( table(ihcoor)>0d0 ) then

!     --- table(ihcoor)> 0, COORDINATE_SYSTEM

         icoef = 4
         call read02filli ( table(ihcoor), 1 )
         go to 1000

      end if  ! ( table(ihcoor)>0d0 )

      if ( table(ihsupg)>0d0 ) then

!     --- table(ihsupg)> 0, SUPG

         icoef = 2
         call read02filli ( table(ihsupg), 1 )
         go to 1000

      end if  ! ( table(ihsupg)>0d0 )

      if ( table(ihdens)>=0d0 ) then

!     --- table(ihdens)> 0, DENSITY

         icoef = 16
         call read02fillc
         go to 1000

      end if  ! ( table(ihdens)>0d0 )

      if ( table(ihheat)>=0d0 ) then

!     --- table(ihheat)> 0, heat_capacity

         icoef = 17
         call read02fillc
         go to 1000

      end if  ! ( table(ihheat)>0d0 )

      if ( table(ihconv)>0d0 ) then

!     --- table(ihconv)> 0, TYPE_CONVECTION

         icoef = 5
         call read02filli ( table(ihconv), 1 )
         go to 1000

      end if  ! ( table(ihconv)>0d0 )

      if ( table(ihupwind)>0d0 ) then

!     --- table(ihupwind)> 0, APPLY_UPWIND

         icoef = 20
         call read02filli ( table(ihupwind), 1 )
         go to 1000

      end if  ! ( table(ihupwind)>0d0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icoef', icoef
         write(irefwr,*) 'End read02_703'

      end if  ! ( debug )
      call erclos ( 'read02_703' )

      end subroutine read02_703
