      subroutine read02_814
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-02-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read coefficients in case of type number 814
!
! **********************************************************************
!
!                       KEYWORDS
!
!     type_number
!     coefficient
!     read
!     minimal_surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecoeffs
      use sepmoduleactual
      use sepmodulekmesh
      use sepmodulestrings
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
!     code       contains keywords recognized for coefficients type 304
!                1: INTEGRATION_RULE
!                2: COORDINATE_SYSTEM
!                3: LINEARIZATION
!     codemain      contains subkeywords
!                 1: CARTESIAN
!                 2: AXI_SYMMETRIC
!                 3: POLAR
!                 4: PICARD
!                 5: NEWTON
! ======================================================================
!
      call eropen ( 'read02_814' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read02_814'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nncode=3

      code(1:3) = codenavstoks(2:4)
      ncode(1:3) = nscodenavstoks(2:4)
      ncodmx(1:3) = ncodenavstoks(2:4)
      call sepcopycodesub ( codecsys, nncodecsys, nscodecsys )
      nncodesub = 5
      codesub(4:5) = codelinear(2:3)
      ncodesub(4:5) = nscodelinear(2:3)

      table(1:nncode) = -1d20
      nmain = 0
      iretur = -11  ! keyword has been read

      itype(1) = 0   ! INTEGRATION_RULE
      tabmin(1) = 0d0
      tabmax(1) = 4d0

      itype(2:3) = 5   ! COORDINATE_SYSTEM / LINEARIZATION
      tabmin(2) = 1d0
      tabmax(2) = tabmin(2)+2d0

      tabmin(3) = tabmax(2)+1d0
      tabmax(3) = tabmin(3)+1d0

      call sepreadsetkeyw
      if ( debug ) call prinrl ( table, nncode, 'table' )

      if ( table(1)>0d0 ) then

!     --- INTEGRATION_RULE found

         icoef = 3
         iincof(iposin+1) = 1      ! integer found
         iincof(iposin+2) = icoef
         iincof(iposin+3) = nint(table(1))
         iposin = iposin+3
         go to 1000

      end if  ! ( table(1)>0d0 )

      do i = 2, 3

         if ( table(i)>0d0 ) then

!        --- COORDINATE_SYSTEM, LINEARIZATION found

            icoef = i+3
            iincof(iposin+1) = 1      ! integer found
            iincof(iposin+2) = icoef
            iincof(iposin+3) = nint(table(i))-1
            iposin = iposin+3
            go to 1000

         end if  ! ( table(i)>0d0 )

      end do  ! i = 2, 3

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End read02_814'

      end if  ! ( debug )
      call erclos ( 'read02_814' )

      end subroutine read02_814
