      subroutine read02_diff
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 18-02-2015 Removal ielgrp
!        version  1.0  date 07-12-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store coefficients for diffusion term for type 800
!
! **********************************************************************
!
!                       KEYWORDS
!
!     type_number
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecoeffs
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     READ02FILLC    Read and fill information concerning coefficient
!                    into iincof/rincof
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'read02_diff' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read02_diff'
         write(irefwr,1) 'iposin, iposrn, ndim', iposin, iposrn, ndim
         call prinrl ( table, 9, 'table' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( table(1)>0d0 ) then

!     --- DIFFUSION found

         icoef = 6
         call read02fillc

!        --- Set other coefficients corresponding to Diffusion

         do i = 2, ndim
            icoef = 9+(i-2)*2
            iincof(iposin+2) = icoef  ! Sequence number coefficient
            iincof(iposin+3) = 6      ! refers to coef 6
            iincof(iposin+1) = 9      ! coefficient is the same as coef 6
            iposin = iposin + 3
         end do  ! i = 2, ndim
         imark(6:11) = 1
         imark(icoef) = 0             ! prevents warning

      else if ( imark(6)==0 .and. (table(2)>0d0 .or. table(5)>0d0) ) then

!     --- DIFF_X found

         icoef = 6
         call read02fillc

      else if ( imark(9)==0 .and. ( table(3)>0d0 .or. table(8)>0d0) ) then

!     --- DIFF_Y found

         icoef = 9
         call read02fillc

      else if ( imark(11)==0 .and. ( table(4)>0d0 .or. table(10)>0d0) ) then

!     --- DIFF_Z found

         icoef = 11
         call read02fillc

      else if ( imark(7)==0 .and. table(6)>0d0 ) then

!     --- DIFF_XY found

         icoef = 7
         call read02fillc

      else if ( imark(8)==0 .and. table(7)>0d0 ) then

!     --- DIFF_XZ found

         icoef = 8
         call read02fillc

      else if ( imark(10)==0 .and. table(9)>0d0 ) then

!     --- DIFF_YZ found

         icoef = 10
         call read02fillc

      end if  ! ( table(9)>0d0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icoef', icoef
         write(irefwr,*) 'End read02_diff'

      end if  ! ( debug )
      call erclos ( 'read02_diff' )

      end subroutine read02_diff
