      subroutine read02_vel ( ihvel, icoeflow )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-12-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store coefficients for velocity term for type 800
!
! **********************************************************************
!
!                       KEYWORDS
!
!     type_number
!     coefficient
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecoeffs
      use sepmoduleactual
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ihvel, icoeflow

!     icoeflow       i    Coefficient sequence number for first velocity
!                         componeny
!     ihvel          i    Start address of velocity in code
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     PRINRL         Print 1d real vector
!     READ02FILLC    Read and fill information concerning coefficient
!                    into iincof/rincof
!     READ02FILLV    Fill information concerning coefficient into iincof/rincof
!                    Special case of a vector of coefficients
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1865   Coefficient has been filled more than once
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'read02_vel' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read02_vel'
         call prinrl ( table(ihvel), 3, 'table' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( table(ihvel)>0d0 ) then

!     --- VELOCITY found

         icoef = icoeflow
         call read02fillv ( ndim )
         imark(icoef) = 0
         imark(icoef+1:icoef+2) = 1
         go to 1000

      else if ( imark(icoeflow)==0 ) then

!     --- not VELOCITY found, check for U_VELOCITY ...

         do i = 2, 1+ndim

            if ( table(ihvel+i-1)>0d0 ) then

!           --- Uxx_VELOCITY found

               icoef = icoeflow+i-1
               call read02fillc
               exit

            end if  ! ( table(13)>0d0 )

         end do ! i = 2, 1+ndim

      end if  ! ( table(ihvel)>0d0 )

      if ( icoef>0 ) then

         if ( imark(icoef)==0 ) then

!        --- imark(icoef) = 0, Coefficient has not been filled

            imark(icoef) = 1

         else

!        --- imark(icoef)>0, Coefficient has been filled before

            call errint ( icoef, 1 )
            call errint ( ielgrp, 2 )
            call errwar ( 1865, 2, 0, 0 )

         end if  ! ( imark(icoef)==0 )

      end if  ! ( icoef>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End read02_vel'

      end if  ! ( debug )
      call erclos ( 'read02_vel' )

      end subroutine read02_vel
