      subroutine read04 ( iinput, niinp, nprob, isubr, list, nlist, &
                          iseqnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.14 date 19-04-2014 Extension with append
!        version  4.13 date 09-10-2013 Extension with SEPARATE_ELGRPS
!        version  4.12 date 09-03-2013 Correction length code
!
!   copyright (c) 1988-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   This subroutine reads the input for subsequent calls of subroutine DERIVA.
!   The input is filled in the array IINPUT
!   The input must be enclosed between OUTPUT and END
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer niinp, iinput(15,niinp), nprob, isubr, nlist, iseqnr, &
              list(0:nlist-1)

!     iinput         o    Contains the actual integer information from
!                         the standard input file
!                         Is stored in ibuffr from position ipiinp
!                         Length: 8 x (numveclc+2) integers
!                         iinput is filled as follows:
!                         iinput(1,1):    numveclc, i.e. number of calls of
!                                         DERIVA that must be performed
!                         iinput(2,1):    nprob, i.e. number of arrays to be
!                                         written to sepcomp.out
!                         iinput(3,1):    Indicates which kind of output must be
!                                         created.
!                                         This position consists of three parts:
!                                         TOSEPPOST, TOAVS and TODX according to
!                                         TOSEPPOST+2*TOAVS+4*TODX+8*TOTECPLOT
!                                         Possible values for TOSEPPOST
!                                         1:  Create output for SEPPOST only in
!                                             the form of file sepcomp.out
!                                         0:  Do not create output for SEPPOST
!                                         Possible values for TOAVS
!                                         1:  Create output for AVS only in the
!                                             form of a file sepavs_xxx.yyy
!                                             With xxx a time sequence number of
!                                             the shape 001, 002 etc
!                                             and yyy a vector sequence number
!                                             of the shape 001, 002 etc
!                                         0:  Do not create output for avs
!                                         Possible values for TODX
!                                         1:  Create output for OPEN_DX only in
!                                             the form of a file sepodx_xxx.yyy
!                                             With xxx a time sequence number of
!                                             the shape 001, 002 etc
!                                             and yyy a vector sequence number
!                                             of the shape 001, 002 etc
!                                         0:  Do not create output for open dx
!                                         Possible values for TOTECPLOT
!                                         1:  Create output for TECPLOT only in
!                                             the form of a file septec_xxx.yyy
!                                             With xxx a time sequence number of
!                                             the shape 001, 002 etc
!                                             and yyy a vector sequence number
!                                             of the shape 001, 002 etc
!                                         0:  Do not create output for open dx
!                         iinput(4,1):    Sequence number of first vector
!                         iinput(5,1):    sequence number file_name avs
!                         iinput(6,1):    sequence number file_name opendx
!                         iinput(7,1):    sequence number file_name tecplot
!                         iinput(8,1):    Indicates if all time steps must be
!                                         stored in a separate file (0) or
!                                         not (1)
!                                         Option only available for tecplot
!                                         output
!                         iinput(9,1):    sequence number file_name paraview
!                         iinput(10,1):   Indicates if all element groups
!                                         must be used as a part (0) or each
!                                         one separately (1)
!                         iinput(11,1):   Append mode (0 = not, 1=append)
!                         iinput(12-15,1):   - &
!                         iinput(1,i+1):  Reference number of array for user
!                               (2,i+1):  icheld
!                               (3,i+1):  ix
!                               (4,i+1):  jdegfd
!                               (5,i+1):  ivec
!                               (6,i+1):  solution number inpvc0
!                               (7,i+1):  solution number inpvc1
!                               (8,i+1):  sequence number coefficients
!                         icheld, ix, jdegfd and ivec are standard
!                         parameters for the SEPRAN subroutine DERIVA (See the
!                         SEPRAN Programmer's Guide).
!                         i refers to ith call of DERIVA
!     iseqnr        i/o   Sequence number with respect to the storage in
!                         array ISEPIN
!     isubr          i    Defines the calling subroutine
!                         Possible values:
!                         1:  PROUTP
!                         2:  EROUTP
!     list          i/o   In this array it is indicated if a sequence number is
!                         already used (list(i)=1) or not list(i)=0)
!     niinp          i    Length parameter for iinput
!     nlist          i    Length of array list
!     nprob         i/o   At input number of problems, at output number of
!                         arrays to be written to the file sepcomp.out during
!                         each time-step, the calls of DERIVA excluded
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncod2=16, nncod3=7, nncod4=1
      double precision tabmn3(nncod3), tabmx3(nncod3), &
                       tabmn4(nncod4), tabmx4(nncod4)
      integer ncode2(nncod2), ncode3(nncod3), itype3(nncod3), i, &
              numveclc, iseq, one_file, &
              ncode4(nncod4), itype4(nncod4), ncod2m(nncod2), &
              ncod3m(nncod3), ncod4m(nncod4)
      character (len=28) code2(nncod2), code4(nncod4)
      character (len=16) code3(nncod3)
      logical finished, seppst, avs, opendx, tecplot, paraview

!     avs            Indicates if avs output must be generated (true)
!                    or not (false)
!     code2          Table with keywords
!     code3          Table with keywords
!     code4          Table with keywords
!     finished       Indicates if the loop has been finished (true) or not
!                    (false)
!     i              Counting variable
!     iseq           Sequence number
!     itype3         Indicates the type of action corresponding to keywords
!                    in code3
!     itype4         Indicates the type of action corresponding to keywords
!                    in code4
!     ncod2m         Table containing the maximal number of  characters in
!                    the keyword table code2
!     ncod3m         Table containing the number of maximum characters in
!                    the keyword table code3
!     ncod4m         Table containing the maximal number of  characters in
!                    the keyword table code4
!     ncode2         Table containing the number of significant characters
!                    in the keyword table code2
!     ncode3         Table containing the number of significant letters in
!                    code3
!     ncode4         Table containing the number of significant letters in
!                    code4
!     nncod2         Number of entries in code2
!     nncod3         Number of entries in code3
!     nncod4         Number of entries in code4
!     numveclc         Number of calls of DERIVA that must be performed
!     one_file       Indicates if all time steps must be stored in one file (1)
!                    or not (0)
!     opendx         Indicates if open_dx output must be generated (true)
!                    or not (false)
!     paraview       Indicates if paraview must be generated (true)
!                    or not (false)
!     seppst         Indicates if seppost output must be generated (true) or not
!     tabmn3         Minimum values allowed in table (equivalent of tabmin)
!     tabmn4         Minimum values allowed in table (equivalent of tabmin)
!     tabmx3         Maximum values allowed in table (equivalent of tabmax)
!     tabmx4         Maximum values allowed in table (equivalent of tabmax)
!     tecplot        Indicates if tecplot output must be generated (true)
!                    or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!     READ04 requires input from the standard input file
!     The following input is recognised:
!     OUTPUT, sequence_number = i         start of input
!      optional records:
!       NOT_SEPPOST
!       TO_AVS
!       TO_OPEN_DX
!       TO_TECPLOT
!       TO_PARAVIEW
!       WRITE j SOLUTIONS
!       AVS_FILE_NAME = 'file name'
!       OPENDX_FILE_NAME = 'file name'
!       TECPLOT_FILE_NAME = 'file name'
!       PARAVIEW_FILE_NAME = 'file name'
!       FILE_PER_TIME_STEP
!       ALL_TIME_STEPS_IN_ONE_FILE
!       APPEND
!       Repeatedly:
!       Vi = ICHELD=k1, options
!        Where the following options are recognised:
!        IX     = k2
!        JDEGFD = k3
!        INPVC0 = k4
!        INPVC1 = k5
!        IVEC   = k6
!        SEQ_COEFFICIENTS = s
!        SEPARATE_ELGRPS
!     END   (mandatory):  end of input
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     653   First record in input for subroutine OUTPUT is not OUTPUT
!     656   Vector number out of range
!     658   No number found in record with NAME V ... = &
!     819   Not an integer number found after WRITE
!     829   No output for SEPPOST nor for AVS
!    1454   Sequence number is already in use for an output vector
!    1628   Keyword found more than one for the same problem
!    1868   Text not recognized
!    1879   Not a text found after the keyword
!    1884   NUMVEC too large
!    2184   Too many file names
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initialize parameters with default values
!    Find next free position in list
!    If ( isubr=1 ) then
!       Read first record
!       Analyse the first word (output)
!    finished := false
!    While ( not finished ) do
!       Read next record
!       Analyse first item
!       If ( item # keyword ) then
!          error
!       else
!          analyse keyword
!          if ( keyword = END ) then
!             finished := true
!          else if ( keyword = WRITE ) then
!             analyse rest of keyword
!          else if ( keyword = TO_AVS) then
!             analyse rest of keyword
!          else if ( keyword = NOT_SEPPOST) then
!             Set iinput(3,1)
!          else if ( keyword = ICHELD ) then
!             analyse rest of keyword
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'WRITE', 'V', 'END', 'NOT_SEPPOST', 'TO_AVS', &
                  'TO_OPEN_DX', 'AVS_FILE_NAME', 'OPENDX_FILE_NAME', &
                  'TO_TECPLOT', 'TECPLOT_FILE_NAME', &
                  'FILE_PER_TIME_STEP', &
                  'ALL_TIME_STEPS_IN_ONE_FILE', &
                  'TO_PARAVIEW', 'PARAVIEW_FILE_NAME', &
                  'SEPARATE_ELGRPS', 'APPEND' /
      data ncode2 / 3, 1, 3, 7, 6,   10, 7,10, 6,10, &
                   4, 3, 7, 5, 5,    3 /
      data ncod2m/ 5, 1, 3,11, 6,   10,13,16,10,17, &
                  18,26,11,28,15,    6 /
      data code3 / 'ICHELD', 'IX', 'JDEGFD', 'IVEC', 'INPVC', &
                   'INPVC', 'SEQ_COEFFICIENTS' /
      data ncode3 / 4, 2, 4, 4, 3, 3, 8 /
      data ncod3m / 6, 2, 6, 4, 5, 5, 16 /
      data itype3 / 0, 0, 0, 0, 10, 0, 0 /
      data tabmn3 / 1d0, 1d0, 1d0, 1d0, 0d0, 0d0, 1d0 /
      data tabmx3 / 1d4, 1d2, 1d2, 1d3, 1d0, 1d3,25d0 /
      data code4 / 'SEQUENCE_NU' /
      data ncode4 / 3 /, itype4 / 0 /, tabmn4 / 1d0 /, tabmx4 / 1d1 /, &
           ncod4m / 15 /

!     code2    Table with main keywords
!             Code is filled as follows:
!             1:   WRITE
!             2:   V
!             3:   END
!             4:   NOT_SEPPOST
!             5:   TO_AVS
!             6:   TO_OPEN_DX
!             7:   AVS_FILE_NAME
!             8:   OPENDX_FILE_NAME
!             9:   TO_TECPLOT
!            10:   TECPLOT_FILE_NAME
!            11:   FILE_PER_TIME_STEP
!            12:   ALL_TIME_STEPS_IN_ONE_FILE
!            13:   TO_PARAVIEW
!            14:   PARAVIEW_FILE_NAME
!            15:   SEPARATE_ELGRPS
!            16:   APPEND
!     code3   Table with keyword recognised after vi= &
!             code3 is filled as follows:
!             1:   ICHELD
!             2:   IX
!             3:   JDEGFD
!             4:   IVEC
!             5:   INPVC   (0)
!             6:   INPVC   (1)
!             7:   SEQ_COEFFICIENTS
!     code4   Table with keyword recognised after output
!             code4 is filled as follows:
!             1:   SEQUENCE_NUMBER
! ======================================================================
!
      call eropen ( 'read04' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read04'
         write(irefwr,1) 'isubr', isubr
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Initialize parameters

      seppst = .true.
      avs    = .false.
      opendx = .false.
      tecplot = .false.
      paraview = .false.
      numveclc = 0
      one_file = -1

      iinput = 0
      iinput(6:7,2:niinp) = 1

!     --- Find the first free sequence number

      if ( isubr==1 ) then

!     --- isubr = 1, read first record (OUTPUT)

         ityprd = 1
         call asknxt
         if ( ityprd/=4 ) then
            call errsub ( 653, 0, 0, 0 )
            call errcrd
         else if ( string(1:4)/='OUTP' ) then
            call errsub ( 653, 0, 0, 0 )
            call errcrd
         end if

      end if

!     --- read rest of record

      table(1) = iseqnr
      call sepcopycode ( code4, nncod4, ncode4, ncod4m )
      itype(1:nncod4) = itype4
      tabmin(1:nncod4) = tabmn4
      tabmax(1:nncod4) = tabmx4
      iretur = 0
      call sepreadsetkeyw
      iseqnr = nint(table(1))

      finished = .false.

!     --- While loop finished

      do while ( .not. finished )

!     --- Read next record

         ityprd = 1
         call asknxt
         if ( debug ) write(irefwr,1) 'ityprd', ityprd
         if ( ityprd==0 ) go to 500
         if ( ityprd/=4 ) then
            do i = 1, nncod2
               call errchr ( code2(i), i )
            end do
            call errsub ( 1868, 0, 0, nncod2 )
            call errcrd
            icode = 0

         else

            if ( debug ) write(irefwr,*) 'string ', string(1:nchar)
            ierpri = 1
            call sepcopycode ( code2, nncod2, ncode2, ncod2m )
            icode = sepsubkeyw()
            if ( debug ) write(irefwr,1) 'icode, ityprd', icode, ityprd

         end if  ! ( ityprd/=4 )
         select case ( icode )

            case ( 0 )

!           --- icode = 0, skip

            case ( 1 )

!           --- icode = 1, WRITE found, read alpha

               call asknxt
               if ( ityprd/=2 ) then
                  call errsub ( 819, 0, 0, 0 )
                  call errcrd
               end if
               nprob = max ( nint(realrd), nprob )

            case ( 2 )

!           --- icode = 2, V found, read number

               if ( ityprd/=2 ) call asknxt
               numveclc = numveclc + 1

               if ( numveclc>niinp-1 ) then

!              --- Too few space reserved

                  call errint ( numveclc, 1 )
                  call errint ( min(niinp-1,nlist-1), 1 )
                  call errsub ( 1884, 2, 0, 0 )

               end if

               if ( ityprd/=2 ) then
                  call errsub ( 658, 0, 0, 0 )
                  call errcrd
               else
                  iseq = nint(realrd)
                  if ( list(iseq)/=0 ) then
                     call errint ( iseq, 1 )
                     call errsub ( 1454, 1, 0, 0 )
                  end if
                  list(iseq) = 1
                  iinput(1,numveclc+1) = iseq
                  if ( iinput(1,numveclc+1)<nprob .or. &
                       iinput(1,numveclc+1)>1000 ) then
                     call errint ( iinput(1,numveclc+1), 1 )
                     call errint ( nprob, 2 )
                     call errint ( 1000, 3 )
                     call errsub ( 656, 3, 0, 0 )
                     call errcrd
                  end if
               end if

!              --- Read rest of record until end

               table(1:nncod3) = 0d0
               table(5) = 1d0
               table(6) = 1d0
               call sepcopytype ( nncod3, itype3, tabmn3, tabmx3 )
               call sepcopycode ( code3, nncod3, ncode3, ncod3m )
               iretur = 0
               call sepreadsetkeyw
               iinput(2:8,numveclc+1) = nint(table(1:7))

            case ( 3 )

!           --- icode = 3, END found

               finished = .true.

            case ( 4 )

!           --- icode = 4, NOT_SEPPOST found

               seppst = .false.

            case ( 5 )

!           --- icode = 5, TO_AVS found

               avs = .true.

            case ( 6 )

!           --- icode = 6, TO_OPEN_DX found

               opendx = .true.

            case ( 7, 8, 10, 14 )

!           --- icode = 7, 8, 10, 14 xxx file_name found, read name

               call asknxt
               if ( ityprd/=5 ) then

!              --- Not a file name found

                  call errchr ( code2(icode), 1 )
                  call errsub ( 1879, 0, 0, 1 )

               else

!              --- file name found

                  numfilenames = numfilenames+1
                  if ( numfilenames>maxfilenames ) then

!                 --- Too many file names

                     call errint ( numfilenames, 1 )
                     call errsub ( 2184, 1, 0, 0 )

                  end if
                  file_names(numfilenames) = string(1:nchar)

               end if

               if ( icode==14 ) then

!              --- paraview file name

                  iinput(9,1) = numfilenames

               else if ( icode==10 ) then

!              --- tecplot file name

                  iinput(7,1) = numfilenames

               else

!              --- avs or open_dx file name

                  iinput(icode-2,1) = numfilenames

               end if

            case ( 9 )

!           --- icode = 9, TO_TECPLOT found

               tecplot = .true.

            case ( 11 )

!           --- icode = 11, FILE_PER_TIME_STEP

               if ( one_file==0 ) then

!              --- keyword has been found more once

                  call errchr ( code2(11)(1:ncod2m(11)), 1 )
                  call errwar ( 1628, 0, 0, 1 )

               else if ( one_file==1 ) then

!              --- keywords are mutually exclusive

                  call errkeywexcl ( code2, ncod2m, 11, 12 )

               end if
               one_file = 0

            case ( 12 )

!           --- icode = 12, ALL_TIME_STEPS_IN_ONE_FILE

               if ( one_file==0 ) then

!              --- keywords are mutually exclusive

                  call errkeywexcl ( code2, ncod2m, 11, 12 )

               else if ( one_file==1 ) then

!              --- keyword has been found more once

                  call errchr ( code2(12)(1:ncod2m(12)), 1 )
                  call errwar ( 1628, 0, 0, 1 )

               end if
               one_file = 1

            case ( 13 )

!           --- icode = 13, TO_PARAVIEW found

               paraview = .true.

            case ( 15 )

!           --- icode = 15, SEPARATE_ELGRPS found

               iinput(10,1) = 1

            case ( 16 )

!           --- icode = 16, APPEND found

               iinput(11,1) = 1

         end select  ! case ( icode )

      end do  ! while ( .not. finished )

 500  iinput(1,1) = numveclc
      iinput(2,1) = nprob
      iinput(3,1) = 0
      iinput(8,1) = max(0,one_file)
      if ( .not. avs .and. .not. seppst .and. .not. opendx .and. &
           .not. tecplot .and. .not. paraview ) then

!     --- seppost nor avs nor opendx nor tecplot output required

         call errsub ( 829, 0, 0, 0 )

      else

!     --- at least one output medium found

         if ( seppst ) iinput(3,1) = 1
         if ( avs ) iinput(3,1) = iinput(3,1)+2
         if ( opendx ) iinput(3,1) = iinput(3,1)+4
         if ( tecplot ) iinput(3,1) = iinput(3,1)+8
         if ( paraview ) iinput(3,1) = iinput(3,1)+16

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iinput, 15, 'iinput' )
         write(irefwr,*) 'End read04'

      end if  ! ( debug )
      call erclos ( 'read04' )

      end
