      subroutine read07 ( kcode )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.20 date 10-01-2015 New call to readreal
!        version  3.19 date 09-03-2013 Correction texts
!        version  3.18 date 16-06-2010 extra debug
!
!   copyright (c) 1989-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Analyse first keyword for subroutine read05
!     This is the first keyword in a post processing record
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmoduleread1
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kcode

!    kcode   o   Return code for subroutine READ05
!                Possible values:
!               -1:  Error:  no known key word found
!              115:  Keyword 3D PLOT
!              116:  Keyword 3D COLOURED PLOT
!               -2:  Keyword COMPUTE
!               -3:  Keyword COMPUTE_VECTOR or V
!                0:  Keyword END
!               20:  Keyword READ_MESH
!               21:  Keyword PRESENT_MESH
!               30:  Keyword INTERPOLATE
!               81:  Keyword FILL COEFFICIENTS
!              120:  Keyword PLOT BOUNDARY
!              113:  Keyword PLOT COLOURED CONTOUR
!              112:  Keyword PLOT CONTOUR
!              103:  Keyword PLOT CURVES
!              118:  Keyword PLOT FIELD
!              120:  Keyword PLOT FUNCTION
!              127:  Keyword PLOT IDENTIFICATION
!              121:  Keyword PLOT INTERSECTION
!              101:  Keyword PLOT MESH
!              104:  Keyword PLOT COLOURED_MESH
!              105:  Keyword PLOT WIRE_FRAME
!              102:  Keyword PLOT POINTS
!              126:  Keyword PLOT POLYGON
!              125:  Keyword PLOT TEXT
!              128:  Keyword PLOT BOX_TEXT
!              119:  Keyword PLOT TRACK
!              110:  Keyword PLOT VECTOR
!              121:  Keyword PLOT VELOCITY PROFILE
!                2:  Keyword PRINT
!                3:  Keyword PRINT BOUNDARY FUNCTION
!                4:  Keyword WRITE_MESH
!                5:  Keyword INTERSECT_MESH
!               10:  Keyword PUT_IN_AVS_FILE
!              -10:  Keyword TIME HISTORY
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncod3=4, nncod2=20
      character (len=13) code3(nncod3)
      character (len=15) code2(nncod2)
      integer ivecnm, ncode3(nncod3), ncode2(nncod2), ifrom, i, &
              ih, lcode, inum, &
              ncod3m(nncod3), ncod2m(nncod2), icount
      double precision t
      logical three, openfound

!     code2          Table with other main keywords
!     code3          Table with keywords after 3
!     i              Counting variable
!     icount         Counter
!     ifrom          Number read after define colour table from
!     ih             Help variable to store a constpstant temporarily
!     inum           Number of numbers read in a record
!     ivecnm         Vector number read
!     lcode          return code produced by intx00
!     ncod2m         Table containing the maximal number of  characters in
!                    the keyword table code2
!     ncod3m         Table containing the number of maximum characters in
!                    the keyword table code3
!     ncode2         Table containing the number of significant letters in
!                    code2
!     ncode3         Table containing the number of significant letters in
!                    code3
!     nncod2         Number of entries in code2
!     nncod3         Number of entries in code3
!     openfound      Indicates if an openings bracket is found before
!                    reading the number or expression
!     t              actual time
!     three          Indication if the first character is the symbol 3 (true)
!                    or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INITDF         Set defaults for program SEPPOST
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PRBISC         computes the time step itime that is closest to the
!                    actual time t by the method of bisection
!     PRESET         Reset some of the defaults
!     PRININ         print 1d integer array
!     PRSET          Set reference default values for SEPPOST
!     READ06         Read part options in record for SEPPOST
!     READ071        Interpret keywords following PLOT for program seppost
!     READ072        Interpret keywords following PRINT or PUT
!     READ073        Interpret keywords following WRITE_MESH
!     READINTEGER    Read an integer from the input file
!     READREAL       Read a real or real expression from the input file
!     READSR         Read a series of numbers from the input (texts are
!                    neglected)
!     READVECNAME    Read name of vector following keyword
!     READVECNUM     Read the vector number following the letter V
!     SEPCOPYCODE    Copy code2 and related arrays into code
! **********************************************************************
!
!                       I/O
!
!    This subroutine interprets the first part of the records for seppost
!    The following items are recognized:
!    3D PLOT
!    3D COLOURED PLOT
!    CLOSE PLOT
!    COMPUTE
!    DEFINE COLOUR TABLE [FROM]
!    DEFINE PLOT PARAMETERS
!    END
!    FILL COEFFICIENTS
!    INTERPOLATE
!    NAME
!    OPEN PLOT
!    PLOT BOUNDARY
!    PLOT COLOURED LEVEL
!    PLOT CONTOUR
!    PLOT CURVES
!    PLOT FIELD
!    PLOT FUNCTION
!    PLOT IDENTICFICATION
!    PLOT INTERSECTION
!    PLOT MESH
!    PLOT COLOURED_MESH
!    PLOT WIRE_FRAME
!    PLOT POINTS
!    PLOT POLYGON
!    PLOT TEXT
!    PLOT TRACK
!    PLOT VECTOR
!    PLOT VELOCITY PROFILE
!    PLOT Vi MESH
!    PLOT Vi CURVES
!    PRESENT_MESH
!    PRINT BOUNDARY
!    PRINT V
!    WRITE_MESH
!    INTERSECT_MESH
!    PUT_IN_AVS_FILE V
!    READ_MESH
!    RESET COLOUR TABLE
!    RESET PLOT PARAMETERS
!    TIME t0 [t1] [tstep]
!    TIME HISTORY
!    ITERATION i0 [i1] [istep]
!    ADD_TO_COOR
!    V...
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     433   Open plot found, whereas the plot has already been opened
!     684   Wrong character found after the keyword TIME.
!     689   Wrong word found.
!    1456   number read is not 3
!    1457   text not recognized as keyword
!    1458   end of file detected for record end
!    1472   Plot file has been opened and closed without making any plot.
!    1473   The plot file has not been opened  before, whereas jtimes = 10.
!    1633   Time step not integer
!    1868   Text not recognized
!    1879   Not a text found after the keyword
! **********************************************************************
!
!                       PSEUDO CODE
!
!    exit := false
!    While ( not exit ) do
!       Read record and analyse first item
!       if ( item = integer ) then
!          Check if integer = 3
!          3D := true
!       else if ( item not keyword ) then
!          error
!       else
!          3d := false
!       if ( 3d ) then
!          Check presence of 3D coloured plot or 3D plot
!          set kcode
!          exit := true
!       else ( not 3d )
!          Determine main keyword
!          if ( main keyword = closee ) then
!             close plot
!          if ( main keyword = define ) then
!             determine next keyword
!             if ( next keyword = plot parameters ) then
!                set plot parameters
!             else if ( next keyword = colour table ) then
!                set colour table
!          else if ( main keyword = name ) then
!             set name
!          else if ( main keyword = plot ) then
!             determine next keyword
!             set kcode
!             exit := true
!          else if ( main keyword = print ) then
!             determine next keyword
!             set kcode
!             exit := true
!          else if ( main keyword = reset ) then
!             determine next keyword
!             if ( next keyword = plot ) then
!                reset plot parameters
!             if ( next keyword = colour ) then
!                reset colour table
!          else if ( main keyword = time ) then
!             determine next keyword
!             if ( next keyword = history ) then
!                set kcode
!                exit := true
!             else
!                determine t0 t1 and tstep
!          else if ( main keyword = open ) then
!             open plot
!          else if ( main keyword = iteration ) then
!             determine i0 i1 and istep
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code3 / 'DCOLOUREDPLOT', 'DPLOT', 'DCOLOURED', 'D' /
      data ncode3 / 11, 3, 4, 1 /
      data ncod3m / 13, 5, 9, 1 /
      data code2 / 'CLOSE', 'COMPUTE', 'DEFINE', 'END', 'FILL', &
                   'NAME', 'OPEN', 'PLOT', 'PRINT', 'RESET', 'TIME', &
                   'ITERATION', 'PUT_IN_AVS_FILE', 'READ_MESH', &
                   'PRESENT_MESH', 'INTERPOLATE', 'ADD_TO_COOR', &
                   'WRITE_MESH', 'INTERSECT_MESH', 'V' /
      data ncode2 / 3, 4, 3, 3, 3,    4, 4, 4, 4, 4, &
                    4, 4, 3, 8,10,    8, 9, 8, 8,1 /
      data ncod2m / 5, 7, 6, 3, 4,    4, 4, 4, 5, 5, &
                    4, 9,15, 9,12,   11,11,10,14, 1 /

!     code3   Table containing all keywords after 3
!             Contents
!              1:  DCOLOUREDPLOT
!              2:  DPLOT
!              3:  DCOLOURED
!              4:  D
!     code2   Table containing all other main keywords
!             Contents
!              1:  CLOSE
!              2:  COMPUTE_VECTOR
!              3:  DEFINE
!              4:  END
!              5:  FILL
!              6:  NAME
!              7:  OPEN
!              8:  PLOT
!              9:  PRINT
!             10:  RESET
!             11:  TIME
!             12:  ITERATION
!             13:  PUT_IN_AVS_FILE
!             14:  READ_MESH
!             15:  PRESENT_MESH
!             16:  INTERPOLATE
!             17:  ADD_TO_COOR
!             18:  WRITE_MESH
!             19:  INTERSECT_MESH
!             20:  COMPUTE
!             21:  V
! ======================================================================
!
      call eropen ( 'read07' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read07'
         write(irefwr,1) 'jtimes', jtimes
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- read next record

100   call initdf
      kcode = -1
      ierpri = 1

!     --- Read record and analyse first word

      ityprd = 1
      call asknxt
      if ( ityprd==0 ) then
         call errsub ( 1458, 0, 0, 0 )
         go to 1000
      end if
      if ( debug ) write(irefwr,1) 'ityprd/1', ityprd

!     --- Error end of file found, whereas not the keyword end has been
!         found
!         First check if first symbol = 3, because 3 is not a letter

      three = .false.
      if ( ityprd==2 ) then

!     --- integer found, check if it is number 3

         ih = nint(realrd)
         if ( ih==3 ) then
            three = .true.
         else
            call errint ( ih, 1 )
            call errsub ( 1456, 1, 0, 0 )
            call errcrd
         end if
         call asknxt

      end if
      if ( debug ) write(irefwr,1) 'ityprd/2', ityprd

110   if ( ityprd==1 ) go to 900
      if ( ityprd/=4 ) then

!     --- Not a keyword found

         if ( three ) then
            do i = 1, nncod3
               call errchr ( code3(i), i )
            end do
            call errsub ( 1868, 0, 0, nncod3 )
         else
            do i = 1, nncod2
               call errchr ( code2(i), i )
            end do
            call errsub ( 1868, 0, 0, nncod2 )
         end if
         call errcrd
         call asknxt
         go to 110

      end if

!     --- Check string

      if ( three ) then

!     --- 3D found, check rest of string

         call sepcopycode ( code3, nncod3, ncode3, ncod3m )
         lcode = sepsubkeyw()
         select case ( lcode )
            case ( :0 )

!           --- lcode<=0, jump

            case (1)

!           --- lcode = 1, 3D coloured plot

               kcode = 116
               call readvecname  ! read vector name

            case (2)

!           --- lcode = 2, 3D plot

               kcode = 115
               call readvecname  ! read vector name

            case (3)

!           --- lcode = 3, 3D coloured

               kcode = 116
               call asknxt       ! read plot
               call readvecname  ! read vector name

            case (4)

!           --- lcode = 4, 3D found, read next word

               call asknxt
               if ( ityprd/=4 ) then
                  call errsub ( 1868, 0, 0, 0 )
                  call errcrd
               else if ( string(1:1)=='C' ) then
                  kcode = 116
                  call asknxt
               else if ( string(1:1)=='P' ) then
                  kcode = 115
               else
                  call errchr ( string(1:nchar), 1 )
                  call errsub ( 1457, 0, 0, 1 )
                  call errcrd
               end if
               call readvecname  ! read vector name

         end select ! case ( lcode )

      else

!     --- Other keyword found, interpret

         call sepcopycode ( code2, nncod2, ncode2, ncod2m )
         lcode = sepsubkeyw()
         if ( debug ) write(irefwr,1) 'lcode, ityprd', lcode, ityprd

         select case ( lcode )

            case ( :0, 21: )

!           --- lcode<=0, jump

            case ( 1 )

!           --- CLOSE PLOT found
!               Check value of jtimes, and set this value to 3

               if ( jtimes==11 ) then

!              --- Error 1472:  Plot file is opened and closed, without
!                              making any plot

                  call errwar ( 1472, 0, 0, 0 )
                  jtimes = 10

               else if ( jtimes==10 ) then

!              --- Error 1473:  Plot file has not been opened

                  call errwar ( 1473, 0, 0, 0 )

               else

!              --- Close Plot dataset

                  jtimes = 13
                  call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )
                  jtimes = 10

               end if
               go to 100

            case ( 2 )

!           --- COMPUTE found

               kcode = -2

            case ( 3 )

!           --- DEFINE found, read next keyword

               call asknxt
               if ( debug ) write(irefwr,1) 'case 3: ityprd', ityprd
               if ( ityprd/=4 ) then
                  call errchr ( 'PLOT PARAMETERS', 1 )
                  call errchr ( 'COLOUR TABLE', 2 )
                  call errsub ( 1868, 0, 0, 2 )
                  call errcrd
                  go to 900
               end if
               if ( debug ) write(irefwr,*) 'string', string(1:nchar)
               if ( string(1:3)=='PLO' ) then

!              --- DEFINE PLOT PARAMETERS

                  kcode = -4
                  call asknxt
                  ivecnm = -1
                  call read06 ( ivecnm )
                  if ( iretur/=-1 ) call prset
                  go to 100

               else if ( string(1:3)=='COL' ) then

!              --- DEFINE COLOUR TABLE

                  call asknxt
                  if ( debug ) write(irefwr,1) 'ityprd:3/1', ityprd
                  if ( ityprd==4 ) then
                     if ( string(1:3)=='TAB' ) call asknxt
                  end if
                  if ( debug ) write(irefwr,1) 'ityprd:3/2', ityprd
                  if ( ityprd==4 ) then
                     if ( string(1:2)=='FR' ) then

!                    --- DEFINE COLOUR TABLE FROM found, read number

                        call asknxt
                        if ( ityprd==2 ) then
                           ifrom = nint(realrd)
                           do i = 1, 100
                              indcol(i+5) = ifrom+i-1
                           end do
                        end if

                     else

!                    --- string not recognized

                        call errchr ( string(1:nchar), 1 )
                        call errsub ( 1457, 0, 0, 1 )
                        call errcrd
                        go to 900

                     end if

                  else if ( ityprd==2 ) then

!                 --- number found, i.e DEFINE COLOUR TABLE = i
!                     Read rest of numbers

                     lastitem = lastitem-2
                     if ( typeitems(lastitem)==5 ) &
                        lastitem = lastitem-1  ! reset until (
                     if ( debug ) then
                        write(irefwr,1) 'lastitem', lastitem
                        call prinin ( typeitems, nitems, 'typeitems' )
                     end if  ! ( debug )
                     call readsr ( getal, 100, inum )
                     inum = inum + 1
                     do i = 1, inum
                        indcol(i+5) = nint(getal(i))
                     end do
                     iinputpst(62) = 0
                     iinputpst(63) = 0

                  else

!                 --- text not recognized

                     call errchr ( 'PLOT PARAMETERS', 1 )
                     call errchr ( 'COLOUR TABLE', 2 )
                     call errsub ( 1868, 0, 0, 2 )
                     call errcrd
                     go to 900

                  end if

               end if
               call prset
               go to 100

            case ( 4 )

!           --- END found

               kcode = 0

            case ( 5 )

!           --- FILL found

               kcode = 81
               call asknxt
               go to 900

            case ( 6 )

!           --- NAME found, read number (NAME VI = ...)

               ih = ispecl
               ispecl = 0
               call asknxt
               call readvecnum ( ivecnm )
               if ( ierror/=0 ) go to 900

!              --- Find equals sign

               ispecl = 8
               call asknxt
               ispecl = ih
               if ( ityprd==1 ) go to 100

!              --- Equals sign found, next 30 positions are used as name

               names(ivecnm) = ' '
               lastitem = lastitem+1
               if ( lastitem<=nitems ) then

!              --- There is some text after the equals sign

                  firstcol = firstcolitems(lastitem)
                  numrecord = firstcol/240+1
                  firstcol = mod(firstcol,240)
                  lastcol = mod(lastcolitems(nitems),240)

                  ih = min(30,lastcol-firstcol+1)
                  names(ivecnm) (1:ih) = &
                        cardimage(numrecord)(firstcol:lastcol)
                  ncharnames(ivecnm) = ih

               end if
               icount = 0
280            if ( icount<=ih-1 .and. names(ivecnm)(1:1)==' ' ) then
                  icount = icount + 1
                  names(ivecnm)(1:ih-icount) = &
                     names(ivecnm)(2:ih+1-icount)
                  names(ivecnm)(ih-icount+1:ih-icount+1) = ' '
                  ncharnames(ivecnm) = ih-icount
                  go to 280
               end if
               if ( names(ivecnm)(1:1)=="'" ) then
                  icount = icount + 1
                  names(ivecnm)(1:ih-icount) = &
                     names(ivecnm)(2:ih+1-icount)
                  ih = ih-icount
                  ncharnames(ivecnm) = ih
                  names(ivecnm)(ih+1:ih+1) = ' '
                  if ( names(ivecnm)(ih:ih)=="'" ) then
                     names(ivecnm)(ih:ih) = ' '
                  end if
               end if
               goto 100

            case ( 7 )

!           --- OPEN PLOT found
!               Check value of jtimes, and set this value to 1

               if ( jtimes/=10 ) then

!              --- Error 433:  Plot file has already been opened before

                  call errwar ( 433, 0, 0, 0 )

               else

                  jtimes = 11

               end if
               go to 100

            case ( 8 )

!           --- PLOT found, interpret next keyword

               call read071 (  kcode )

            case ( 9, 13 )

!           --- PRINT or PUT found, interpret next keyword

               call read072 ( kcode, lcode )

            case ( 10 )

!           --- RESET found, interpret next keyword

               call asknxt
               if ( ityprd/=4 ) then
                  call errsub ( 1868, 0, 0, 0 )
                  call errcrd
               else if ( string(1:3)=='COL' ) then

!              --- RESET COLOUR TABLE found

                  call preset(2)
                  go to 100

               else if ( string(1:3)=='PLO' ) then

!              --- RESET PLOT PARAMETERS found

                  call preset(1)
                  go to 100

               else

                  call errchr ( string(1:nchar), 1 )
                  call errsub ( 1457, 0, 0, 1 )
                  call errcrd

               end if

            case ( 11 )

!           --- TIME found, interpret next item

               call asknxt
               if ( ityprd==4 ) then

!              --- keyword found after time (must be history)

                  if ( string(1:3)=='HIS' ) then

!                 --- TIME HISTORY found

                     kcode = -10

                  else
                     call errchr ( string(1:nchar), 1 )
                     call errsub ( 1457, 0, 0, 1 )
                     call errcrd
                  end if
                  go to 900

               else if ( ityprd<2 .or. ityprd>3 ) then

!              --- Not a number found after time

                  call errsub ( 684, 0, 0, 0 )
                  call errcrd
                  go to 900

               else

!              --- Number found, i.e. TIME = t0, t1, tstep

                  t = realrd

!                 --- Find the corresponding time step itime

                  call prbisc ( t, time, iltime(1), ntime )

                  if ( debug ) then
                     write(irefwr,2) 't, time', t, time
                     write(irefwr,1) 'iltime(1), ntime', &
                                      iltime(1), ntime
                  end if  ! ( debug )

!                 --- read rest of record; at most two numbers

                  openfound = .true.
                  call readreal ( openfound, .false. )
                  inum = 0
                  if ( ityprd==2 .or. ityprd==3 ) then

!                 --- real found

                     inum = 1
                     getal(1) = realrd
                     call readreal ( openfound, .false. )
                     if ( ityprd==2 .or. ityprd==3 ) then

!                    --- another real found

                        inum = 2
                        getal(2) = realrd

                     end if  ! ( ityprd==2 .or. ityprd==3 )

                  end if  ! ( ityprd==2 .or. ityprd==3 )
                  if ( debug ) write(irefwr,1) 'inum', inum
                  if ( inum==0 ) then

!                 --- inum = 0, so only one number:  TIME  = t0

                     iltime(2) = iltime(1)
                     iltime(3) = 1

                  else

                     t = getal(1)
                     call prbisc ( t, time, iltime(2), ntime )
                     if ( debug ) then
                        write(irefwr,2) 't, time', t, time
                        write(irefwr,1) 'iltime(2), ntime', &
                                         iltime(2), ntime
                     end if  ! ( debug )
                     if ( inum==1 ) then

!                    --- inum = 1, so two numbers:  TIME  = t0,t1

                        iltime(3) = 1

                     else

!                    --- inum = 2, so three numbers:  TIME  = (t0,t1,iltime)

                        iltime(3) = nint(getal(2))
                        if ( abs(iltime(3)-getal(2))>1d-3 ) then

!                       --- iltime is not integer

                           call errwar ( 1633, 0, 0, 0 )
                           iltime(3) = 1

                        end if

                     end if

                  end if

!                 --- Read next record

                  go to 100

               end if

            case ( 12 )

!           --- ITERATION found, read at most three numbers

               call readsr ( getal, 3, inum )
               iltime(1) = 1
               iltime(2) = 1
               iltime(3) = 1
               if ( inum>=1 ) iltime(1) = nint(getal(1))
               if ( inum>=2 ) iltime(2) = nint(getal(2))
               if ( inum>=3 ) iltime(3) = nint(getal(3))
               go to 100

            case ( 14 )

!           --- READ_MESH found, read sequence number and file name

               call readinteger ( iinputpst(88), 1, nummesh, &
                                  'filename' )
               call asknxt
               code(1) = 'FILE'
               ncodefix = 4
               nncode = 1
               ierpri = 1
               i = sepkeyword3()
               call asknxt
               if ( ityprd==5 ) then

!              --- String between quotes found

                  textid(1) = string(1:nchar)

               else

!              --- Not a string between quotes found

                  call errchr ( 'read_mesh', 1 )
                  call errsub ( 1879, 0, 0, 1 )
                  call errcrd
                  go to 900

               end if
               kcode = 20

            case ( 15 )

!           --- PRESENT_MESH found, read sequence number

               call readinteger ( iinputpst(90), 1, nummesh, &
                                  'PRESENT_MESH' )
               kcode = 21

            case ( 16 )

!           --- INTERPOLATE found

               kcode = 30

            case ( 17 )

!           --- ADD_TO_COOR found

               kcode = 40

            case ( 18 )

!           --- WRITE_MESH found

               call read073 ( kcode )

            case ( 19 )

!           --- INTERSECT_MESH found

               kcode = 5

            case ( 20 )

!           --- V found, i.e. V%vector = alpha V%vector + beta

               kcode = -3

         end select  ! case ( lcode )

      end if  ! ( three )

!     --- Give error message if kcode = -1

 900  if ( kcode==-1 ) then

!     --- error 689

         call errchr ( string(1:nchar), 1 )
         call errsub ( 689, 0, 0, 1 )
         call errcrd
         ityprd = 1
         go to 100

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'jtimes', jtimes
         write(irefwr,1) 'kcode, ityprd/end', kcode, ityprd
         write(irefwr,*) 'End read07'

      end if  ! ( debug )
      call erclos ( 'read07' )

      end subroutine read07
