      subroutine read071 ( kcode )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 26-05-2009 Read name of vector immediately
!        version  1.4  date 12-03-2008 New way of defining scalars
!        version  1.3  date 01-09-2006 Extra plot box_text
!        version  1.2  date 01-08-2006 Initialize parameter
!        version  1.1  date 16-06-2006 Fortran 90
!        version  1.0  date 11-02-2005
!
!   copyright (c) 2005-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret keywords following PLOT for program seppost
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     post_processing
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmoduleread1
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kcode

!    kcode   o   Return code for subroutine READ05
!                Possible values:
!               -1:  Error:  no known key word found
!              115:  Keyword 3D PLOT
!              116:  Keyword 3D COLOURED PLOT
!               -2:  Keyword COMPUTE
!                0:  Keyword END
!               20:  Keyword READ_MESH
!               21:  Keyword PRESENT_MESH
!               30:  Keyword INTERPOLATE
!               81:  Keyword FILL COEFFICIENTS
!              120:  Keyword PLOT BOUNDARY
!              113:  Keyword PLOT COLOURED CONTOUR
!              112:  Keyword PLOT CONTOUR
!              103:  Keyword PLOT CURVES
!              118:  Keyword PLOT FIELD
!              120:  Keyword PLOT FUNCTION
!              127:  Keyword PLOT IDENTIFICATION
!              121:  Keyword PLOT INTERSECTION
!              101:  Keyword PLOT MESH
!              104:  Keyword PLOT COLOURED_MESH
!              105:  Keyword PLOT WIRE_FRAME
!              102:  Keyword PLOT POINTS
!              126:  Keyword PLOT POLYGON
!              125:  Keyword PLOT TEXT
!              128:  Keyword PLOT BOX_TEXT
!              119:  Keyword PLOT TRACK
!              110:  Keyword PLOT VECTOR
!              121:  Keyword PLOT VELOCITY PROFILE
!                2:  Keyword PRINT
!                3:  Keyword PRINT BOUNDARY FUNCTION
!               10:  Keyword PUT_IN_AVS_FILE
!              -10:  Keyword TIME HISTORY
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nncod3
      parameter ( nncod3=19 )
      integer ncode3(nncod3), ncod3m(nncod3), lcode, ivecnm
      character (len=14) code3(nncod3)
      logical ::  readvectnam

!     code3          Table with sub keywords corresponding to CHANGE_COEFF
!     ivecnm         vector sequencer number with respect to isol
!     lcode          return code produced by intx00
!     ncod3m         Table containing the number of maximum characters in
!                    the keyword table code3
!     ncode3         Table containing the number of significant characters in
!                    the keyword table code3
!     nncod3         Number of entries in code3
!     readvectnam    If true a vector name is expected
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRSUB         Error messages
!     READVECNAME    Read name of vector following keyword
!     READVECNUM     Read the vector number following the letter V
!     SEPCOPYCODE    Copy code2 and related arrays into code
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1457   text not recognized as keyword
!    1868   Text not recognized
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code3 / 'BOUNDARY', 'COLOURED_MESH', &
                   'COLOURED', 'CONTOUR', 'CURVES', 'FIELD', &
                   'FUNCTION', 'IDENTIFICATION', 'INTERSECTION', 'MESH', &
                   'POINTS', 'POLYGON', 'TEXT', 'TRACK', 'VECTOR', &
                   'VELOCITY', 'WIRE_FRAME', 'V', 'BOX_TEXT' /
      data ncode3 / 4,11, 3, 4, 3,   4, 4, 4, 5, 4, &
                    4, 3, 3, 3, 3,   3, 7, 1, 5 /
      data ncod3m / 8,13, 8, 7, 6,   5, 8,14,12, 4, &
                    6, 7, 4, 5, 6,   8,10, 1, 8 /

!     code3   Table containing all keywords after plot
!             Contents
!              1:  BOUNDARY
!              2:  COLOURED_MESH
!              3:  COLOURED
!              4:  CONTOUR
!              5:  CURVES
!              6:  FIELD
!              7:  FUNCTION
!              8:  IDENTIFICATION
!              9:  INTERSECTION
!             10:  MESH
!             11:  POINTS
!             12:  POLYGON
!             13:  TEXT
!             14:  TRACK
!             15:  VECTOR
!             16:  VELOCITY
!             17:  WIRE_FRAME
!             18:  V
!             19:  BOX_TEXT
! ======================================================================
!
      call eropen ( 'read071' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read071'
         write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      call asknxt
      if ( debug ) then
         write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
         write(irefwr,*) 'string ', string
      end if  ! ( debug )

      ierpri = 1
      call sepcopycode ( code3, nncod3, ncode3, ncod3m )
      lcode = sepsubkeyw()
      readvectnam = .false.

      if ( debug ) write(irefwr,1) 'lcode', lcode

      select case ( lcode )

         case ( :0 )

!        --- lcode<=0, jump

         case ( 1 )

!        --- PLOT BOUNDARY found

            kcode = 120

         case ( 2 )

!        --- PLOT COLOURED_MESH found

            kcode = 104

         case ( 3 )

!        --- PLOT COLOURED LEVEL found

            kcode = 113
            ispecl = 9
            call asknxt
            if ( debug ) then
               write(irefwr,1) 'ityprd', ityprd
               write(irefwr,*) 'string ', string
            end if  ! ( debug )

            if ( ityprd==4 .and. &
                 (string(1:nchar)=='LEVEL' .or. &
                  string(1:nchar)=='LEVELS' .or. &
                  string(1:nchar)=='CONTOUR') ) call asknxt
            if ( debug ) then
               write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
               write(irefwr,*) 'string ', string
            end if  ! ( debug )

!           --- reset lastitem, since asknxt is called in readvecname

            lastitem = lastitembef

            readvectnam = .true.
            if ( debug ) then
               write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
            end if  ! ( debug )

         case ( 4 )

!        --- PLOT CONTOUR found

            kcode = 112
            readvectnam = .true.

         case ( 5 )

!        --- PLOT CURVES found

            kcode = 103

         case ( 6 )

!        --- PLOT FIELD found

            kcode = 118
            iinputpst(5) = 1
            readvectnam = .true.

         case ( 7 )

!        --- PLOT FUNCTION found

            kcode = 120
            readvectnam = .true.

         case ( 8 )

!        --- PLOT IDENTIFICATION found

            kcode = 127

         case ( 9 )

!        --- PLOT INTERSECTION found

            kcode = 121
            readvectnam = .true.

         case ( 10 )

!        --- PLOT MESH found

            kcode = 101

         case ( 11 )

!        --- PLOT POINTS found

            kcode = 102

         case ( 12 )

!        --- PLOT POLYGON found

            kcode = 126

         case ( 13 )

!        --- PLOT TEXT found

            kcode = 125

         case ( 14 )

!        --- PLOT TRACK found

            kcode = 119
            readvectnam = .true.

         case ( 15 )

!        --- PLOT VECTOR found

            kcode = 110
            readvectnam = .true.

         case ( 16 )

!        --- PLOT VELOCITY PROFILE found

            kcode = 121
            ispecl = 9
            call asknxt
            if ( debug ) then
               write(irefwr,1) 'ityprd', ityprd
               write(irefwr,*) 'string ', string
            end if  ! ( debug )

            if ( ityprd==4 .and. &
                 (string(1:nchar)=='PROFILE') ) call asknxt
            if ( debug ) then
               write(irefwr,1) 'ityprd', ityprd
               write(irefwr,*) 'string ', string
            end if  ! ( debug )

!           --- reset lastitem, since asknxt is called in readvecname

            lastitem = lastitembef
            readvectnam = .true.

         case ( 17 )

!        --- PLOT WIRE_FRAME found

            kcode = 105

         case ( 18 )

!        --- PLOT Vi found, read number

            call asknxt
            call readvecnum ( ivecnm )
            if ( ierror/=0 ) go to 1000
            iinputpst(2) = ivecnm
            if ( ierror/=0 ) go to 1000

!           --- read rest of record (MESH or CURVES)

            call asknxt
            if ( ityprd/=4 ) then
               call errchr ( 'MESH', 1 )
               call errchr ( 'CURVES', 2 )
               call errsub ( 1868, 0, 0, 2 )
               call errcrd
            else if ( string(1:3)=='MES' ) then

!           --- PLOT Vi MESH found

               kcode = 101

            else if ( string(1:3)=='CUR' ) then

!           --- PLOT Vi CURVES found

               kcode = 103

            else
               call errchr ( string(1:nchar), 1 )
               call errsub ( 1457, 0, 0, 1 )
               call errcrd
            end if

         case ( 19 )

!        --- PLOT BOX_TEXT found

            kcode = 128

      end select  ! case ( lcode )

      if ( readvectnam ) then

!     --- Next item should be vector name, read and store in iinputpst(2)

         call readvecname

      end if  ! ( readvectnam )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End read071'

      end if
      call erclos ( 'read071' )

      end
