      subroutine read072 ( icode, lcode )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 12-03-2008 New way of defining scalars
!        version  1.1  date 30-08-2006 New definition of cpost5
!        version  1.0  date 12-06-2006
!
!   copyright (c) 2006-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret keywords following PRINT or PUT for program seppost
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     post_processing
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmoduleread
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icode, lcode

!    icode   o   Return code for subroutine READ05
!                Possible values:
!               -1:  Error:  no known key word found
!                2:  Keyword PRINT
!                3:  Keyword PRINT BOUNDARY FUNCTION
!               10:  Keyword PUT_IN_AVS_FILE
!    lcode   i   Return code from subroutine READ07
!                9:  Print
!               13:  put
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivecnm, numchar

!     ivecnm         vector sequencer number with respect to isol
!     numchar        Number of characters in keyword
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRSUB         Error messages
!     READNAMEVEC    Read name of vector in common cpost5
!     READSPLITKEYW  check whether the keyword contains numbers
!                    If so split the keyword into a part without number and
!     READVECNUM     Read the vector number following the letter V
! **********************************************************************
!
!                       I/O
!
!     PRINT BOUNDARY, file = 'filename'
!     PRINT V, file = 'filename'
!     PRINT MESH, file = 'filename'
!     PRINT Vi MESH, file = 'filename'
!     PUT_IN_AVS_FILE V
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1457   text not recognized as keyword
!    1868   Text not recognized
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'read072' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from read072'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read text, may also contain numbers

      ispecl = 9
      call asknxt
      ispecl = 0

!     --- First we have to check whether the keyword contains numbers
!         If so we have to split the keyword into a part without number
!         and a number
!         This is necessary to check if it is a keyword, since keywords do
!         not contain numbers

      call readsplitkeyw ( numchar, keyword )
      if ( debug ) then
         write(irefwr,1) 'ityprd', ityprd
         write(irefwr,*) 'string ', string(1:nchar)
         write(irefwr,*) 'keyword ', keyword(1:numchar)
      end if  ! ( debug )

      if ( ityprd/=4 ) then

!     --- No text found after print

         call errchr ( 'V', 1 )
         call errchr ( 'BOUNDARY', 2 )
         call errsub ( 1868, 0, 0, 2 )
         call errcrd
         go to 1000

      else if ( keyword(1:numchar)=='V' .or. &
                keyword(1:numchar)=='VECTOR' ) then

!     --- PRINT V found

         if ( lcode==9 ) then
            icode = 2
         else
            icode = 10
         end if
         lastitem = lastitembef+1

!        --- Read number

         call asknxt
         if ( ityprd==4 .and. string(1:2)=='V ' ) call asknxt

         call readvecnum ( ivecnm )
         iinputpst(2) = ivecnm
         if ( ierror/=0 ) go to 1000

      else if ( keyword(1:numchar)=='BOUNDARY' .and. lcode==9 ) then

!     --- PRINT BOUNDARY found

         icode = 3

      else

!     --- Other text found, check if it concerns the name of a vector

         call readnamevec

         if ( debug ) then
            write(irefwr,1) 'ityprd', ityprd
            write(irefwr,2) 'realrd', realrd
         end if  ! ( debug )

         if ( ityprd==2 ) then

!        --- vector name found

            if ( lcode==9 ) then
               icode = 2
            else
               icode = 10
            end if

            call readvecnum ( ivecnm )
            iinputpst(2) = ivecnm

         else

!        --- Text not recognized

            call errchr ( string(1:nchar), 1 )
            call errsub ( 1457, 0, 0, 1 )
            call errcrd

         end if  ! ( ityprd==2 )

      end if

1000  call erclos ( 'read072' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End read072'

      end if  ! ( debug )

      end
