      subroutine readc0interpr ( nread, nreal, inforinpcre, icompl, &
                                 rincre, ndegfd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 03-04-2012 Extension with series of surfaces
!        version  1.1  date 10-02-2012 use sepgetinpelem
!        version  1.0  date 06-07-2011
!
!   copyright (c) 2011-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpret all information read by readc0
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekmesh
      use sepmoduleread1
      use sepmoduletexts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nread, nreal, icompl, ndegfd
      integer :: inforinpcre(*)
      double precision rincre(*)

!     icompl         i    Indicates if vector is real (0) or complex (1)
!     inforinpcre    i    Array of length of rincre that contains a 0 if the
!                         number stored in rincre is a real and a reference
!                         to scalars if it is a scalar
!     ndegfd         i    Number of degrees of freedom per point
!     nread         i/o   Number of items read
!     nreal          i    Number of reals stored in RINCRE
!     rincre         i    Double precision array containing the values of the
!                         constant if KFUN<0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielgrp, inum, m, inpelm

!     i              Counting variable
!     ielgrp         element group number
!     inpelm         number of points per element
!     inum           Number of numbers read in a record
!     m              number of integers read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READC0CURV     Interpret all curve information read by readc0
!     READC0QUAD     Fill information concerning boundary conditions of the type
!                    (half)-quadratic into itypebc and rincre
!     READC0SURF     Interpret all surface information read by readc0
!     READC0VOLM     Interpret all volume information read by readc0
!     READC301       Check the numrel numbers stored in array relnods
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     891   In one record more than one of the combinations:
!           POINTS, CURVES, SURFACES, NODES, ELEMENTS or GROUP is found
!    1043   SEQ_VECTORS may only be used in combination with OLD_VECTOR
!    1044   Not the keyword POINTS found after USER
!    1045   Too much points found after keyword
!    1067   MAX found but not in combination with QUADRATIC
!    1104   Second nodal point number after NODES = incorrect
!    1106   VALUE = appears more than once in a record
!    1107   Both VALUE = and FUNC = found in record
!    1112   FUNC appears more than once in a record
!    1887   Number of texts read too large
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readc0interpr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readc0interpr'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( table(1)>0d0 ) then

!     --- VALUE found, read number

         if ( iwork(1)/=0 ) then
            call errsub ( 1107, 0, 0, 0 )
            call errcrd
         end if
         if ( nread>=1 ) then
            call errsub ( 1106, 0, 0, 0 )
            call errcrd
         end if
         iposut = nint(table(1))
         if ( infotabout(iposut)==11 ) then
            inforinpcre(nreal+1) = nint(tabout(iposut))-100000000
            rincre(nreal+1) = scalars(inforinpcre(nreal+1))
         else
            inforinpcre(nreal+1) = 0
            rincre(nreal+1) = tabout(iposut)
         end if
         if ( debug ) then
            write(irefwr,*) 'value found'
            write(irefwr,1) 'iposut, infotabout(iposut), nreal', &
                             iposut, infotabout(iposut), nreal
            write(irefwr,2) 'rincre(nreal+1)', rincre(nreal+1)
         end if  ! ( debug )

         if ( icompl==0 ) then

!        --- icompl = 0, real vector

            nread = 1

         else

!        --- icompl = 1, Complex vector
!                        Find next item

            nread = 2
            if ( tabmax(1)>1d0 ) then
               if ( infotabout(iposut+1)==11 ) then
                  inforinpcre(nreal+2) = &
                     nint(tabout(iposut+1))-100000000
                  rincre(nreal+2) = scalars(inforinpcre(nreal+2))
               else
                  inforinpcre(nreal+2) = 0
                  rincre(nreal+2) = tabout(iposut+1)
               end if
            else
               inforinpcre(nreal+2) = 0
               rincre(nreal+2) = 0d0
            end if

         end if

      end if
      if ( table(2)>0 ) then

!     --- FUNCTION found

         if ( iwork(1)/=0 ) then
            call errsub ( 1112, 0, 0, 0 )
            call errcrd
         end if
         if ( nread/=0 ) then
            call errsub ( 1107, 0, 0, 0 )
            call errcrd
         end if
         iwork(1) = nint(table(2))
         if ( debug ) then
            write(irefwr,*) 'function found'
            write(irefwr,1) 'iwork(1)', iwork(1)
         end if  ! ( debug )

      end if
      if ( table(3)>0 ) then

!     --- SPECIAL_FUNCTION found

         if ( iwork(1)/=0 ) then
            call errsub ( 1112, 0, 0, 0 )
            call errcrd
         end if
         if ( nread/=0 ) then
            call errsub ( 1107, 0, 0, 0 )
            call errcrd
         end if
         iwork(1) = nint(table(3))+1000
         if ( debug ) then
            write(irefwr,*) 'SPECIAL_FUNCTION found'
            write(irefwr,1) 'iwork(1)', iwork(1)
         end if  ! ( debug )

      end if
      if ( table(4)>0 ) then

!     --- OLD_FUNCTION found

         if ( iwork(1)/=0 ) then
            call errsub ( 1112, 0, 0, 0 )
            call errcrd
         end if
         if ( nread/=0 ) then
            call errsub ( 1107, 0, 0, 0 )
            call errcrd
         end if
         iwork(1) = nint(table(4))+2000
         if ( debug ) then
            write(irefwr,*) 'OLD_FUNCTION found'
            write(irefwr,1) 'iwork(1)', iwork(1)
         end if  ! ( debug )

      end if
      if ( table(5)>0 ) then

!     --- OLD_VECTOR found

         if ( iwork(1)/=0 ) then
            call errsub ( 1112, 0, 0, 0 )
            call errcrd
         end if
         if ( nread/=0 ) then
            call errsub ( 1107, 0, 0, 0 )
            call errcrd
         end if
         iwork(1) = nint(table(5))+3000
         if ( table(6)>0d0 ) then
            iposut = nint(table(6))
            m = nint(tabmax(6))
            iwork(111) = m
            do i = 0, m-1
               iwork(112+i) = nint(tabout(iposut+i))
            end do
         else
            iwork(111) = 1
            iwork(112) = 1
         end if
         if ( debug ) then
            write(irefwr,*) 'OLD_VECTOR found'
            write(irefwr,1) 'iwork(1), iwork(111), iwork(112)', &
                             iwork(1), iwork(111), iwork(112)
         end if  ! ( debug )

      else if ( table(6)>0d0 ) then

!     --- SEQ_VECTORS may only be used in combination with OLD_VECTOR

         call errsub ( 1043, 0, 0, 0 )
         call errcrd

      end if
      if ( table(38)>0 ) then

!     --- VECTOR found

         if ( iwork(1)/=0 ) then
            call errsub ( 1112, 0, 0, 0 )
            call errcrd
         end if
         if ( nread/=0 ) then
            call errsub ( 1107, 0, 0, 0 )
            call errcrd
         end if
         iwork(1) = nint(table(38))+4000
         if ( debug ) then
            write(irefwr,*) 'VECTOR found'
            write(irefwr,1) 'iwork(1)', iwork(1)
         end if  ! ( debug )

      end if
      if ( table(7)>0 ) then

!     --- DEGFD found

         iposut = nint(table(7))
         ndegfd = nint(tabmax(7))
         do i = 1, ndegfd
            iwork(99+i) = nint(tabout(iposut+i-1))
         end do
         if ( debug ) then
            write(irefwr,*) 'DEGFD found'
            write(irefwr,1) 'ndegfd', ndegfd
            call prinin ( iwork(100), ndegfd, 'iwork' )
         end if  ! ( debug )

      end if
      if ( table(8)>0d0 .and. table(9)<0d0 ) then

!     --- POINTS found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 1
         iposut = nint(table(8))
         m = nint(tabmax(8))
         iwork(3) = m
         do i = 1, m
            work(99+i) = tabout(iposut+i-1)
         end do
         if ( debug ) then
            write(irefwr,*) 'POINTS found'
            write(irefwr,1) 'm', m
            call prinin ( iwork(100), m, 'iwork' )
         end if  ! ( debug )

      end if
      if ( table(9)>0d0 ) then

!     --- USER POINTS found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         if ( table(8)<0d0 ) then
            call errsub ( 1044, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 8
         iposut = nint(table(8))
         m = nint(tabmax(8))
         iwork(4) = nint(tabout(iposut))
         if ( m>2 ) then
            call errchr ( code(9), 1 )
            call errint ( m, 1 )
            call errint ( 2, 2 )
            call errsub ( 1045, 2, 0, 1 )
            call errcrd
         end if
         if ( m==2 ) then
            iwork(5) = nint(tabout(iposut+1))
         else
            iwork(5) = iwork(4)
         end if
         if ( table(10)>-1d20 ) iwork(3) = nint(table(10))
         if ( debug ) then
            write(irefwr,*) 'USER POINTS found'
            write(irefwr,1) 'm', m
            call prinin ( iwork, 5, 'iwork' )
         end if  ! ( debug )

      end if
      if ( table(11)>0d0 .or. table(32)>0 ) then

!     --- CURVES found

         call readc0curv ( iwork, table, tabout, tabmin, tabmax, &
                           code )
         if ( debug ) write(irefwr,*) 'CURVES found'

      end if
      if ( table(13)>0d0 .or. table(33)>0 ) then

!     --- SURFACES RN1, RN2, ... (S1, S2, ... ) found

         call readc0surf ( iwork, table, tabout, tabmax, work )
         if ( debug ) write(irefwr,*) 'SURFACES found'

      end if

      if ( table(14)>0d0 ) then

!     --- VOLUMES RN1, RN1, ... (V1 to V2) found

         call readc0volm ( iwork, table, tabout, tabmax, work )
         if ( debug ) write(irefwr,*) 'VOLUMES found'

      end if

      if ( table(15)>0d0 ) then

!     --- NODES found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 4
         iwork(3) = 1
         if ( table(10)>-1d20 ) iwork(3) = nint(table(10))
         iposut = nint(table(15))
         m = nint(tabmax(15))
         iwork(4) = nint(tabout(iposut))
         if ( m==2 ) then
            iwork(5) = nint(tabout(iposut+1))
            if ( iwork(5)<iwork(4) ) then
               call errint ( iwork(5), 1 )
               call errint ( iwork(4), 2 )
               call errint ( npoint, 3 )
               call errsub ( 1104, 3, 0, 0 )
            end if
         else
            iwork(5) = iwork(4)
         end if
         if ( debug ) then
            write(irefwr,*) 'nodes found'
            write(irefwr,1) 'm', m
            call prinin ( iwork, 5, 'iwork' )
         end if  ! ( debug )

      end if

      if ( table(16)>0d0 ) then

!     --- ELEMENT i, j (RN1, RN1, ... ) found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 5
         iposut = nint(table(16))
         m = nint(tabmax(16))-2
         iwork(3) = m
         iwork(4) = nint(tabout(iposut))
         inum = nint(tabout(iposut+1))
         if ( inum==0 ) inum = iwork(4)
         iwork(5) = inum
         do i = 1, m
            work(99+i) = tabout(iposut+1+i)
         end do
         inpelm = sepgetinpelem ( iwork(4) )
         if ( m>0 ) then

!        --- numrel>0,  copy contents of work numrel+1 positions forwards
!            clear work from position 101+numrel

            call readc301 ( m, inpelm, work(100), work(101+m) )

         end if
         if ( debug ) then
            write(irefwr,*) 'elements found'
            write(irefwr,1) 'm', m
            call prinin ( iwork, 5, 'iwork' )
            call prinin ( iwork(100), m, 'iwork/1' )
         end if  ! ( debug )

      end if

      if ( table(17)>0d0 ) then

!     --- GROUP ielgrp ( RN1, RN2, RN, ... ) found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 6
         iposut = nint(table(17))
         m = nint(tabmax(17))
         iwork(4) = nint(tabout(iposut))
         m = m - 1
         do i = 1, m
            work(99+i) = tabout(iposut+i)
         end do

!        --- Compute inpelm

         inpelm = sepgetinpelm ( iwork(4) )
         if ( m>0 ) then

!        --- numrel>0,  copy contents of work numrel+1 positions forwards
!            clear work from position 101+numrel

            call readc301 ( m, inpelm, work(100), work(101+m) )

         end if
         if ( debug ) then
            write(irefwr,*) 'group found'
            write(irefwr,1) 'm', m
            call prinin ( iwork, 5, 'iwork' )
            call prinin ( iwork(100), m, 'iwork/1' )
         end if  ! ( debug )

      end if

      if ( table(18)>0d0 ) then

!     --- OBSTACLE i found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 9
         iwork(3) = nint(table(18))
         if ( debug ) then
            write(irefwr,*) 'obstacle found'
            call prinin ( iwork, 3, 'iwork' )
         end if  ! ( debug )

      end if

      if ( table(19)>0d0 ) then

!     --- CONTACT i found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 10
         iwork(3) = nint(table(19))
         if ( debug ) then
            write(irefwr,*) 'CONTACT found'
            call prinin ( iwork, 3, 'iwork' )
         end if  ! ( debug )

      end if
      if ( table(20)>0d0 ) then

!     --- NO_CONTACT i found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 11
         iwork(3) = nint(table(20))
         if ( debug ) then
            write(irefwr,*) 'NO_CONTACT found'
            call prinin ( iwork, 3, 'iwork' )
         end if  ! ( debug )

      end if

      if ( table(23)>0d0 .or. table(24)>0d0 .or. &
           table(26)>0d0 ) then

!     --- FILE_NODAL_VALUE or FILE_ELEMENT_VALUE or FILE_CAPACITY_VALUE found
!         Fill iwork as follows:
!         iwork(2) = 12/13/14
!         iwork(3) = nextxt (refers to texts)

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         if ( table(23)>0d0 ) then
            iwork(2) = 12
            iposut = nint(table(23))
            nchar = nint(tabmax(23))
         else if ( table(24)>0d0 ) then
            iwork(2) = 13
            iposut = nint(table(24))
            nchar = nint(tabmax(24))
         else
            iwork(2) = 14
            iposut = nint(table(26))
            nchar = nint(tabmax(26))
         end if
         if ( nextxt>ntexts ) then

!        --- Too many texts stored in texts

            call errint ( ntexts, 1 )
            call errsub ( 1887, 1, 0, 0 )
            call errcrd
            go to 1000

         end if
         iwork(3) = nextxt
         texts(nextxt) = strout(iposut) (1:nchar)
         nextxt = nextxt+1
         if ( debug ) then
            write(irefwr,*) 'FILE_NODAL_VALUE found'
            call prinin ( iwork, 3, 'iwork' )
         end if  ! ( debug )

      end if

      if ( table(27)>0d0 ) then

!     --- GLOBAL_GROUP iglgrp found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 15
         iwork(3) = nint(table(27))
         if ( debug ) then
            write(irefwr,*) 'GLOBAL_GROUP found'
            call prinin ( iwork, 3, 'iwork' )
         end if  ! ( debug )

      end if

      if ( table(28)>0d0 ) then

!     --- in_all_obstacle i found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 16
         iwork(3) = nint(table(28))
         if ( debug ) then
            write(irefwr,*) 'in_all_obstacle found'
            call prinin ( iwork, 3, 'iwork' )
         end if  ! ( debug )

      end if

      if ( table(29)>0d0 ) then

!     --- in_inn_obstacle i found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 17
         iwork(3) = nint(table(29))
         if ( debug ) then
            write(irefwr,*) 'in_inn_obstacle found'
            call prinin ( iwork, 3, 'iwork' )
         end if  ! ( debug )

      end if

      if ( table(30)>0d0 ) then

!     --- in_boun_obstacle i found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 18
         iwork(3) = nint(table(30))
         if ( debug ) then
            write(irefwr,*) 'in_boun_obstacle found'
            call prinin ( iwork, 3, 'iwork' )
         end if  ! ( debug )

      end if

      if ( table(31)>0d0 ) then

!     --- on_boun_obstacle i found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 19
         iwork(3) = nint(table(31))
         if ( debug ) then
            write(irefwr,*) 'on_boun_obstacle found'
            call prinin ( iwork, 3, 'iwork' )
         end if  ! ( debug )

      end if

      if ( table(37)>0d0 ) then

!     --- zero_levelset i found

         if ( iwork(2)/=0 ) then
            call errsub ( 891, 0, 0, 0 )
            call errcrd
         end if
         iwork(2) = 20
         iwork(3) = nint(table(37))
         if ( debug ) then
            write(irefwr,*) 'zero_levelset found'
            call prinin ( iwork, 3, 'iwork' )
         end if  ! ( debug )

      end if

      if ( table(21)>0d0 .or. table(25)>0d0 ) then

!     --- (HALF_)QUADRATIC found

         if ( iwork(1)/=0 ) then
            call errsub ( 1112, 0, 0, 0 )
            call errcrd
         end if
         if ( nread/=0 ) then
            call errsub ( 1107, 0, 0, 0 )
            call errcrd
         end if
         call readc0quad ( iwork, table, rincre, nreal, inforinpcre, &
                           code, ncodmx )
         if ( debug ) then
            write(irefwr,*) 'QUADRATIC found'
            call prinrl ( rincre, nreal+3, 'rincre' )
         end if  ! ( debug )

      else if ( table(22)>-1d20 ) then

!     --- MAX found, but not quadratic

         call errsub ( 1067, 0, 0, 0 )
         call errcrd

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readc0interpr'

      end if  ! ( debug )
      call erclos ( 'readc0interpr' )

      end subroutine readc0interpr
