      subroutine readevalfunc ( ifunc, funarg, funresult )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 26-10-2015 Extra debug
!        version  1.4  date 13-12-2013 Error message negative sqrt
!        version  1.3  date 27-10-2011 Extension with erfc
!
!   copyright (c) 2005-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Evaluate function defined by ifunc
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ifunc
      double precision funarg, funresult

!     funarg         i    argument of function
!     funresult      o    result of function
!     ifunc          i    Choice parameter defining the type of function
!                          10:  sqrt
!                          11:  sin
!                          12:  cos
!                          13:  exp
!                          14:  tan
!                          15:  asin
!                          16:  acos
!                          17:  atan
!                          18:  log (natural logarithm)
!                          19:  abs
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2798   Value of iwork not yet implemented
!    3160   Argument of sqrt in expression negative
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readevalfunc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readevalfunc'
         write(irefwr,1) 'ifunc', ifunc
         write(irefwr,2) 'funarg', funarg
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000
      select case ( ifunc )

         case(10)

!        --- ifunc=10: sqrt

            if ( funarg<0d0 ) then

!           --- argument of sqrt negative

               call erreal ( funarg, 1 )
               call errsub ( 3160, 0, 1, 0 )
               go to 1000
            end if  ! ( funarg<0d0 )

            funresult = sqrt(funarg)

         case(11)

!        --- ifunc=11: sin

            funresult = sin(funarg)

         case(12)

!        --- ifunc=12: cos

            funresult = cos(funarg)

         case(13)

!        --- ifunc=13: exp

            funresult = exp(funarg)

         case(14)

!        --- ifunc=14: tan

            funresult = tan(funarg)

         case(15)

!        --- ifunc=15: asin

            funresult = asin(funarg)

         case(16)

!        --- ifunc=16: acos

            funresult = acos(funarg)

         case(17)

!        --- ifunc=17: atan

            funresult = atan(funarg)

         case(18)

!        --- ifunc=18: log

            funresult = log(funarg)

         case(19)

!        --- ifunc=19: abs

            funresult = abs(funarg)

         case(20)

!        --- ifunc=20: erfc

            funresult = erfc(funarg)

         case default

!        --- ifunc>19: Not yet implemented

            call errint ( ifunc, 1 )
            call errsub ( 2798, 1, 0, 0 )

      end select  ! case ( ifunc )

1000  call erclos ( 'readevalfunc' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'funresult', funresult
         write(irefwr,*) 'End readevalfunc'

      end if

      end
