      subroutine readexprstore1 ( istore, lenistore )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-12-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store the information as stored in istore/store into the commons
!     cexpression and cseprd
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulexpr
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lenistore, istore(lenistore)

!     istore         i    Contains information of the expression
!                         See subroutine READEXPR, array iwork
!     lenistore      i    Number of positions used in istore
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart

!     istart         First position used at a part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
!     PRININLARGE    Print 1d integer vector (large numbers)
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readexprstore1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readexprstore1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istart = istexpress(numexpress)  ! last position used
      numexpress = numexpress+1  ! raise sequence number of expression

!     --- Check array length

      call inichkarlen ( numexpress, maxexpression, 'istexpress' )
      call inichkarlen ( istart+lenistore, lenintexpr, 'intexpress' )
      if ( ierror/=0 ) go to 1000

      istexpress(numexpress) = istart+lenistore ! end position used
      intexpress(istart+1:istart+lenistore) = istore(1:lenistore)

      realrd = numexpress+200000000  ! indicates expression
      ityprd = 19                    ! expression

1000  if ( debug ) then

!     --- Debug information

         call prinin ( istexpress, numexpress+1, 'istexpress' )
         call prininlarge ( intexpress, istart+lenistore, 'intexpress' )
         call prinrl ( realexpress, lastrealexpr, 'realexpress' )
         write(irefwr,*) 'End readexprstore1'

      end if  ! ( debug )
      call erclos ( 'readexprstore1' )

      end
