      subroutine readheading ( ios, ihelp, nlines, nglunk, nfictunk )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-11-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read second part of information of solution array from sepcomp.out
!     Return with number of lines
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulemach
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ios, ihelp(8), nlines, nglunk, nfictunk

!     ihelp          o    integer work array of length 8 to store
!                         quantities read
!                         1: compl
!                         2: iprob
!                         3: ivec
!                         4: jcart
!                         5: nphys
!                         6: nusol
!                         7: typevec
!                         8: iseqkp
!     ios            o    IOSTAT variable, returns from reading or writing
!     nfictunk       o    Total number of fictitious unknowns
!     nglunk         o    total number of global unknowns
!     nlines         o    number of lines corresponding to the solution array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, i
      character (len=80) text
      character (len=340) textloc

!     i              Counting variable
!     k              Counting variable
!     text           String variable containing some text
!     textloc        String to store a local text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     747   End of file reached
!     748   Error in reading from file
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readheading' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readheading'

      end if
      if ( ierror/=0 ) go to 1000

      if ( iref74>0 ) then
         read ( iref74,*, iostat=ios ) ihelp
         read (iref74,*,iostat=ios) nlines, k, nglunk, nfictunk
         read (iref74,110,iostat=ios) text(1:5), textloc
110      format ( a5,a )
      else
         read ( -iref74, iostat=ios ) ihelp
         read(-iref74,iostat=ios) nlines, k, nglunk, nfictunk
         read(-iref74,iostat=ios) text(1:5), textloc(1:16*k)
      end if
      if ( ios<0 ) then

!     --- ios<0, end of file reached

         call errsub ( 747, 0, 0, 0 )
         go to 1000

      else if ( ios>0 ) then

!     --- ios>0, Read error

         call errint ( ios, 1 )
         call errsub ( 748, 1, 0, 0 )
         go to 1000

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readheading'

      end if
      call erclos ( 'readheading' )

      end
