      subroutine readmeshextra1 ( inpsep, leninpsep, iinput1, ireadseq1, &
                                  iinput2, ireadseq2, nelgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-01-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Rearrange input as stored in iinput1 and iinput2 and put into array inpsep
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     reordering
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), leninpsep, iinput1(*), ireadseq1(*), &
              iinput2(*), ireadseq2(0:*), nelgrp

!     iinput1        i    Input array as created by readstouter
!     iinput2        i    Input array as created by readsubelms
!     inpsep         o    The integer input read is stored in array INPSEP
!     ireadseq1      i    Array ireadseq corresponding to readstouter
!     ireadseq2      i    Array ireadseq corresponding to readsubelms
!     leninpsep     i/o   Length of array inpsep
!                         At input: declared length
!                         At output: computed length
!     nelgrp         i    Number of element groups
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nseq, nmain, iseq, istart1, istart, jstart, istart2, &
              numgroups

!     i              Counting variable
!     iseq           Sequence number
!     istart         First position used at a part
!     istart1        First position used at a part
!     istart2        First position used at a part
!     jstart         starting position of curve i in kmeshm
!     nmain          Number of main keywords in inpsep
!     nseq           Largest sequence number used in a part
!     numgroups      Number of element groups
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    2493   Outer boundary does not exist
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readmeshextra1' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readmeshextra1'
         write(irefwr,*) 'nelgrp ', nelgrp
         call prinin ( ireadseq1, nelgrp, 'ireadseq1' )

      end if
      if ( ierror.ne.0 ) go to 1000
      nmain = 2
      inpsep(1) = nmain
      istart = 2*nmain+2

!     --- Fill information concerning outer boundaries
!         First extract nseq

      nseq = 0
      do i = 1, nelgrp
         if ( ireadseq1(i).gt.0 ) nseq = i
      end do
      inpsep(2) = nseq
      inpsep(3) = istart
      jstart = istart+nseq
      if ( debug ) &
         write(irefwr,*) 'nseq, istart, jstart ', nseq, istart, jstart

      do iseq = 1, nseq

         istart1 = ireadseq1(iseq)
         inpsep(istart+iseq-1) = jstart
         if ( debug ) write(irefwr,*) 'iseq, istart1, istart, jstart ', &
            iseq, istart1, istart, jstart
         if ( istart1.eq.0 ) then

!        --- istart1 = 0

            inpsep(jstart) = 0
            jstart = jstart+1

         else

!        --- istart1 > 0

            numgroups = iinput1(istart1+1)
            inpsep(jstart) = numgroups
            do i = 1, numgroups
               inpsep(jstart+i) = iinput1(istart1+1+i)
            end do
            jstart = jstart+numgroups+1
            if ( jstart.gt.leninpsep ) then

!           --- Array length of array inpsep too small

               call errint ( jstart, 1 )
               call errint ( leninpsep, 2 )
               call errchr ( 'inpsep', 1 )
               call errsub ( 550, 2, 0, 1 )
               go to 1000

            end if

         end if  ! ( istart1.eq.0 )

      end do  ! iseq = 1, nseq

!     --- Next fill information concerning subdivide

      istart = jstart
      inpsep(4) = 0
      inpsep(5) = istart

      do i = 0, nelgrp

         istart2 = ireadseq2(i)
         if ( istart2.gt.0 ) then

!        --- information found

            inpsep(4) = inpsep(4)+1
            iseq = iinput2(istart2+1)
            if ( iseq.gt.0 ) then

!           --- Outer boundary iseq, check if it exists

               if ( ireadseq1(iseq).le.0 ) then

!              --- Outer boundary does not exist, error

                  call errint ( iseq, 1 )
                  call errsub ( 2493, 1, 0, 0 )
                  go to 1000

               end if  ! ( ireadseq1(iseq).le.0 )

            end if  ! ( iseq.gt.0 )
            inpsep(jstart) = iinput2(istart2)
            inpsep(jstart+1) = iseq
            jstart = jstart+2
            if ( jstart.gt.leninpsep ) then

!           --- Array length of array inpsep too small

               call errint ( jstart, 1 )
               call errint ( leninpsep, 2 )
               call errchr ( 'inpsep', 1 )
               call errsub ( 550, 2, 0, 1 )
               go to 1000

            end if

         end if  ! ( ireadseq2(i).gt.0 )

      end do  ! i = 1, nelgrp
      leninpsep = jstart-1

1000  call erclos ( 'readmeshextra1' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readmeshextra1'

      end if

      end
