      subroutine readplotboun ( inpsep, code, ncodmx, table )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-11-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information on how boundary curves must be plotted
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     curve
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep, ncodmx(3)
      double precision table(3)
      character (len=*) code(3)

!     code           i    Table with keywords
!                         See subroutine readscplotmesh, from position 8
!     inpsep         o    The integer input read is stored in array INPSEP
!                         See subroutine readscplotmesh, inpsep(8)
!     ncodmx         i    Table containing the maximal number of  characters in
!                         the keyword table code, see code
!     table          i    Contains the numbers read for the various keywords
!                         See code
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readplotboun' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readplotboun'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First check on mutual exclusive keywords

      if ( table(1)>0d0 .and. table(3)>0d0 ) &
         call errkeywexcl ( code, ncodmx, 1, 3 ) ! inner_curves and no_curves
      if ( table(2)>0d0 .and. table(3)>0d0 ) &
         call errkeywexcl ( code, ncodmx, 2, 3 ) ! outer_curves and no_curves

!     --- Next fill inpsep

      inpsep = 3  ! default value (plot all curves)
      if ( table(1)>0d0 ) then

!     --- inner curves found

         if ( table(2)<=0d0 ) then

!        --- not outer curves found

            inpsep = 2  ! only inner curves

         end if  ! ( table(2)<=0d0 )

      else if ( table(2)>0d0 ) then

!     --- only outer curves found

         inpsep = 1  ! only outer curves

      end if  ! ( table(1)>0d0 )

      if ( table(3)> 0d0 ) inpsep = 4  ! no curves

1000  call erclos ( 'readplotboun' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readplotboun'

      end if  ! ( debug )

      end
