      subroutine readpostscal
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-11-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information following keyword V.. = &
!     for seppost
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivecnm
      double precision alpha, beta, minsign

!     alpha          multiplication factor
!     beta           addition constpstant
!     ivecnm         vector sequencer number with respect to isol
!     minsign        1 if sign = + and -1 if negative
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     READVECNUM     Read the vector number following the letter V
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2855   Text not recognized
!    2856   Vector numbers not equal
!    2857   Text not recognized
!    2858   Text not recognized
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readpostscal' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readpostscal'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iinputpst(1) = -3    ! set type of subroutine

!     --- Read first item, i.e. vector number (name)

      call asknxt
      call readvecnum ( ivecnm )
      iinputpst(2) = ivecnm
      if ( debug ) write(irefwr,1) 'ivecnm', ivecnm

!     --- Read next item, i.e. number or v

      call asknxt
      if ( ityprd==2 .or. ityprd==3 ) then

!     --- Number found, set alpha and read next item

         alpha = realrd
         call asknxt

      else

!     --- No number found, set alpha to 1

         alpha = 1d0

      end if  ! ( ityprd==2 )

      if ( ityprd/=4 ) then

!     --- Not a text found at this position

         call errchr ( 'V', 1 )
         call errsub ( 2855, 0, 0, 1 )
         call errcrd
         go to 1000

      else

!     --- Read next item, i.e.  vector number (name)

         call asknxt
         call readvecnum ( ivecnm )
         if ( ivecnm/=iinputpst(2) ) then
            call errint ( ivecnm, 1 )
            call errint ( iinputpst(2), 2 )
            call errsub ( 2856, 2, 0, 0 )
            call errcrd
            go to 1000

         end if  ! ( ivecnm/=iinputpst(2) )

      end if  ! ( ityprd/=4 )

!     --- Detect minus sign

      ispecl = 4
      call asknxt
      if ( ityprd==6 ) then

!     --- minus sign found

         minsign = -1d0
         call asknxt

      else if ( ityprd==3 ) then

!     --- number found

         minsign = 1d0

      end if  ! ( ityprd==6 )
      ispecl = 0
      if ( ityprd==1 ) then

!     --- End of line found, beta = 0

         beta = 0d0

      else if ( ityprd==2 .or. ityprd==3 ) then

!     --- Number found, set beta

         beta = realrd*minsign
         call asknxt
         if ( ityprd/=1 ) then

!        --- Text not recognized

            call errwar ( 2858, 0, 0, 0 )
            call errcrd

         end if  ! ( ityprd/=1 )

      else

!     --- Text not recognized

         call errsub ( 2857, 0, 0, 0 )
         call errcrd

      end if  ! ( ityprd==1 )
      rinputpst(46) = alpha
      rinputpst(47) = beta
      if ( debug ) then
         write(irefwr,2) 'alpha, beta', alpha, beta
         write(irefwr,1) 'ityprd', ityprd
      end if  ! ( debug )

1000  call erclos ( 'readpostscal' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ivecnm', ivecnm
         write(irefwr,2) 'alpha, beta', alpha, beta
         write(irefwr,*) 'End readpostscal'

      end if  ! ( debug )

      end
