      subroutine readrealnoscal ( openfound )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 08-12-2013 Extension with integer scalars
!        version  1.3  date 14-11-2013 New call to prevalexpr
!        version  1.2  date 21-12-2011 New call to prevalexpr
!
!   copyright (c) 2008-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read a real or real expression from the input file
!     scalars are not permitted
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical openfound

!     openfound     i/o   Indicates if an openings bracket is found before
!                         reading the number or expression
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lastitemsav
      logical ::  closefound, finished

!     closefound     If true a closing bracket has been found, which
!                    corresponds to a closing bracket read earlier
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     lastitemsav    Help parameter to save the value of lastitem
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     READSREXPR     Interprete part of input as number or expression
!     READSTRSCALS   Check if name read corresponds to one of the scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readrealnoscal' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readrealnoscal'
         write(irefwr,*) 'openfound ', openfound
         write(irefwr,1) 'lastitem', lastitem
         call prinin ( typeitems, nitems, 'typeitems' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( lastitem>=nitems ) then

!     --- End of record reached

         ityprd = 1
         go to 1000

      end if  ! ( lastitem>=nitems )

!     --- Skip = or , &

      if ( debug ) &
         write(irefwr,1) 'typeitems(lastitem+1)', typeitems(lastitem+1)

      if ( typeitems(lastitem+1)==13 .or. &
           typeitems(lastitem+1)==26 ) lastitem = lastitem+1

      if ( debug ) &
         write(irefwr,1) 'lastitem, typeitems(lastitem+1)', &
                          lastitem, typeitems(lastitem+1)

      if ( lastitem>=nitems ) then

!     --- End of record reached

         ityprd = 1
         go to 1000

      end if  ! ( lastitem>=nitems )

!     --- First check if we are dealing with a scalar or a real expression

      if ( typeitems(lastitem+1)==10 ) then

!     --- % found, i.e. scalar

         ityprd = 20
         go to 1000

      else if ( typeitems(lastitem+1)==3 ) then

!     --- String found, check if it is a scalar

         lastitemsav = lastitem
         call asknxt
         call readstrscals
         if ( debug ) write(irefwr,1) &
            'after readstrscals: ityprd, lastitem, lastitemsav', &
                                 ityprd, lastitem, lastitemsav
         if ( ityprd==11 .or. ityprd==14 ) then

!        --- scalar found, jump

            lastitem = lastitemsav
            ityprd = 20
            go to 1000

         else

!        --- no scalar found, reset lastitem

            lastitem = lastitemsav

         end if  ! ( ityprd==11 .or.ityprd==14 ... )

      end if  ! ( typeitems(lastitem+1)==10 )
      closefound = .false.
      finished = .false.

      call readsrexpr ( openfound, closefound, .false., finished )

      if ( debug ) then
         write(irefwr,*) 'openfound, closefound ', &
                          openfound, closefound
      end if  ! ( debug )

      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,2) 'realrd', realrd

      if ( ityprd==3 .and. abs(realrd)<1d7 ) then

!     --- Real less than 10000000, check if it is an integer

         if ( abs(realrd)<=0d0 ) then

!        --- real = 0

            ityprd = 2

         else if ( abs(realrd)>=0.8d0 ) then

!        --- real = 0

            if ( abs(realrd-nint(realrd))<1d-8*abs(realrd) ) ityprd = 2

         end if  ! ( abs(realrd)<=0d0 )

      end if  ! ( abs(realrd)<1d7 )

1000  call erclos ( 'readrealnoscal' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
         write(irefwr,2) 'realrd', realrd
         write(irefwr,*) 'End readrealnoscal'

      end if  ! ( debug )

      end
