      subroutine readscadpintbn ( inpsep, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 07-03-2012 New call to readseqvector
!        version  1.0  date 02-11-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keywords after the keyword ADAPT_INTERFACE_BOUNDARY
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     level_set
!     mesh_velocity
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         See subroutine readsc
!                         positions 1-7 are used:
!                         1:  ICODE
!                         2:  SEQCONC Sequence number of concentration
!                                     at interface
!                         3:  ILEV Sequence number of level_set
!     istart        i/o   Position in inpsep
!                         See subroutine readsc
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     READSEQVECTOR  Put the value of the sequence number read in iinput
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       ADAPT_INTERFACE_BOUNDARY V1, options
!           where options may have the following contents
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readscadpintbn' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscadpintbn'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(istart) = icode  ! icode (ADAPT_INTERFACE_BOUNDARY)

!     --- Input for subroutine intx02

      nncode = 2
      code(1) = 'V'
      code(2) = 'LEVELSET'
      ncode(1) = 1
      ncode(2) = 4
      ncodmx(1) = 1
      ncodmx(2) = 8

      itype(1:nncode) = 0
      itype(1) = 38  ! vectors

      tabmin(1:nncode) = 1d0

      tabmax(1:nncode) = 1d2

      table(1:nncode) = 0d0
      table(2) = 1d0

      nmain = 0
      call sepreadsetkeyw
      if ( debug ) then
         call prinrl ( table, nncode, 'table' )
         call prinrl ( tabout, 1, 'tabout' )
         call prinin ( infotable, nncode, 'infotable' )
      end if  ! ( debug )

      call readseqvector ( table(1), 'concentration', inpsep(istart+1), &
                           1 )
      inpsep(istart+2) = nint(table(2))
      istart = istart+3

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readscadpintbn'

      end if  ! ( debug )
      call erclos ( 'readscadpintbn' )

      end subroutine readscadpintbn
