      subroutine readscchcoef ( istart, inpsep, nprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 13-12-2014 Use local inpsep
!        version  1.1  date 22-11-2004 Fortran 90
!        version  1.0  date 29-11-2002
!
!   copyright (c) 2002-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input following the following the keyword
!     CHANGE_COEFFICIENTS
!     with respect to subroutine readsc
!     The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     coefficient
!     change
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(0:*), nprob

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                         2: coefficient sequence number
!                         3: change coefficient sequence number
!                         4: iteration sequence number
!                         5: problem sequence number (if 0, the last one)
!     istart        i/o   Next free position in inpsep
!     nprob          i    Number of various problems
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncod1=4
      integer ncode1(nncod1), ncod1m(nncod1)
      character (len=20) code1(nncod1)

!     code1          Table with keywords
!     ncod1m         Table containing the number of maximum characters in
!                    the keyword table code1
!     ncode1         Table containing the number of significant letters in
!                    code1
!     nncod1         Number of entries in code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       CHANGE_COEFFIENTS, options (all in one line)
!           where options may have the following contents
!           SEQ_COEF = i, &
!           SEQ_CHANGE = j, &
!           AT_ITERATION = i
!           PROBLEM = p
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'SEQ_COEF', 'SEQ_CHANGE', 'ITERATION', 'PROBLEM' /
      data ncode1 /  7, 7, 4, 4 /
      data ncod1m /  8,10, 9, 7 /

!     code1   Table with sub keywords corresponding to keyword CHANGE_COEFS
!             Values:
!             1:  SEQ_COEF
!             2:  SEQ_CHANGE
!             3:  ITERATION
!             4:  PROBLEM
! ======================================================================
!
      call eropen ( 'readscchcoef' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscchcoef'
         write(irefwr,1) 'istart', istart
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(0) = icode
      call sepcopycode ( code1, nncod1, ncode1, ncod1m )

      itype(1:nncode) = 0
      tabmin(1:nncode) = 1d0
      tabmin(2) = 0d0
      tabmax(1) =25d0
      tabmax(2) =50d0
      tabmax(3) = 1d1
      tabmax(4) = 1d2
      table(1) = 1d0
      table(2) = 1d0
      table(3) = 0d0
      table(4) = 0d0
      tabmax(4) = nprob
      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      if ( ierror/=0 ) go to 1000
      inpsep(1) = nint(table(1))
      inpsep(2) = nint(table(2))
      inpsep(3) = nint(table(3))
      inpsep(4) = nint(table(4))

      if ( debug ) call prinin ( inpsep(0), 5, 'inpsep' )
      istart = istart + 5

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istart', istart
         write(irefwr,*) 'End readscchcoef'

      end if  ! ( debug )
      call erclos ( 'readscchcoef' )

      end subroutine readscchcoef
