      subroutine readsccompmesh ( inpsep, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-05-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read keywords after the keyword COMPARE_MESH
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     mesh
!     compare
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         See subroutine readsc
!                         positions 1-3 are used:
!                          1:  ICODE
!                          2:  sequence number first mesh
!                          3:  sequence number second mesh
!     istart        i/o   Position in inpsep
!                         See subroutine readsc
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!        COMPARE_MESH, options
!           where options may have the following contents
!           mesh1   default: 1
!           mesh2   default: 2
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readsccompmesh' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readsccompmesh'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(1) = icode  ! icode (COMPARE_MESH)

      nncode = 1
      code(1) = 'xxx'
      ncode(1) = 3
      ncodmx(1) = 3
      itype(1) = 0
      iretur = -10  ! allow numbers
      nmain = 0
      call sepreadsetkeyw

      if ( debug ) then
         write(irefwr,1) 'iretur', iretur
         call prinrl ( tabout, -iretur-10, 'tabout' )
      end if  ! ( debug )

      inpsep(2) = 1    ! mesh 1
      inpsep(3) = 2    ! mesh 2

      if ( iretur<-10 ) then

!     --- At least one mesh number read

         inpsep(istart+1) = nint(tabout(1))
         if ( iretur<-11) inpsep(istart+2) = nint(tabout(2))

      end if  ! ( iretur<-10 )
      istart = istart+3  ! raise istart

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpsep, 3, 'inpsep' )
         write(irefwr,*) 'End readsccompmesh'

      end if  ! ( debug )
      call erclos ( 'readsccompmesh' )

      end subroutine readsccompmesh
