      subroutine readscdefmesh ( istart, inpsep, istrin, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 13-11-2010 error 1488 --> errkeywexcl
!        version  1.2  date 22-11-2004 Fortran 90
!        version  1.1  date 12-12-2002 Use V or VECTOR
!
!   copyright (c) 2002-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read the input following the following the keyword
!   DEFORM_MESH
!   with respect to subroutine readsc
!   The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     mesh
!     deformation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*), istrin
      double precision rinsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                         2: Sequence number of the vector to add
!                         3: SCALE
!     istart        i/o   Next free position in inpsep
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     rinsep        i/o   Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncod1=3
      integer ncode1(nncod1), itype1(nncod1), ncod1m(nncod1)
      double precision tabmn1(nncod1), tabmx1(nncod1)
      character (len=20) code1(nncod1)

!     code1          Table with keywords
!     itype1         Indicates the type of action corresponding to keywords
!                    in code1
!     ncod1m         Table containing the number of maximum characters in
!                    the keyword table code1
!     ncode1         Table containing the number of significant letters in
!                    code1
!     nncod1         Number of entries in code1
!     tabmn1         Minimum value of items read corresponding to code1
!     tabmx1         Maximum value of items read corresponding to code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       DEFORM_MESH, options (all in one line)
!          VECTOR = s
!          SCALE = s     (Default 1)
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1  / 'VECTOR', 'SCALE', 'V' /
      data ncod1m / 6, 5, 1 /
      data ncode1 / 6, 3, 1 /
      data itype1 / 0, 1, 0 /
      data tabmn1 / 1d0, -1d3, 1d0 /
      data tabmx1 / 1d2, 1d3, 1d2 /

!     code1   Table with sub keywords corresponding to keyword DEFORM_MESH
!             Values:
!             1:  VECTOR
!             2:  SCALE
!             3:  V
! ======================================================================
!
      call eropen ( 'readscdefmesh' )

      inpsep(istart) = icode

      call sepcopycode ( code1, nncod1, ncode1, ncod1m )
      call sepcopytype ( nncod1, itype1, tabmn1, tabmx1 )

      table(1) = -1d0
      table(2) = 1d0
      table(3) = -1d0
      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      if ( ierror/=0 ) go to 1000
      if ( table(3)>0d0 .and. table(2)>0d0 ) &
         call errkeywexcl ( code1, ncod1m, 3, 2)

      table(2) = max(table(3),table(2))
      inpsep(istart+1) = nint(table(1))
      inpsep(istart+2) = istrin
      rinsep(istrin) = table(2)
      istrin = istrin + 1
      istart = istart + 3

1000  call erclos ( 'readscdefmesh' )

      end subroutine readscdefmesh
