      subroutine readsceigval ( istart, inpsep, nprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 20-04-2012 Indexed scalars
!        version  1.4  date 13-11-2010 error 1488 --> errkeywexcl
!        version  1.3  date 12-03-2008 New way of defining scalars
!
!   copyright (c) 2002-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read the input following the following the keyword
!   COMPUTE_EIGENVALUES
!   with respect to subroutine readsc
!   The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     eigenvalue
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*), nprob

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                         2: contains the sequence number for the input
!                         3: IVECTOR
!                         4: ISCALAR
!                         5: IPROB
!                         6: NEIGV
!     istart        i/o   Next free position in inpsep
!     nprob          i    Number of various problems
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncod1=6
      integer ncode1(nncod1), ncod1m(nncod1), &
              neigv, ivec, i, iscal
      character (len=20) code1(nncod1)

!     code1          Table with keywords
!     i              Counting variable
!     iscal          Sequence number of scalar
!     ivec           Sequence number of vector with respect to array ISOL
!     ncod1m         Table containing the number of maximum characters in
!                    the keyword table code1
!     ncode1         Table containing the number of significant letters in
!                    code1
!     neigv          Number of eigenvalues
!     nncod1         Number of entries in code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       COMPUTE_EIGENVALUES, options (all in one line)
!           where options may have the following contents
!               SEQUENCE_NUMBER = s  (Default 1)
!               PROBLEM = s          (Default 1)
!               VECTOR = s           (Default 0)
!               SCALAR = s           (Default 1)
!               NUM_EIGVal = s       (Default 1)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2052   Too many eigenvectors
!    2053   Too many eigenvalues
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1/ 'SEQUENCE_NUMBER', 'VECTOR', 'SCALAR', 'PROBLEM', &
                  'NUM_EIGVAL', 'V' /
      data ncod1m /15, 6, 6, 7,10, 1 /
      data ncode1 / 6, 6, 4, 4, 7, 1 /

!     code1   Table with sub keywords corresponding to keyword COMPUTE_EIGENVAL
!             Values:
!             1:  SEQUENCE_NUMBER
!             2:  VECTOR
!             3:  SCALAR
!             4:  PROBLEM
!             5:  NUM_EIGVAL
!             6:  V
! ======================================================================
!
      call eropen ( 'readsceigval' )

      inpsep(istart) = icode

      call sepcopycode ( code1, nncod1, ncode1, ncod1m )

      itype(1:nncode) = 0
      itype(2) = 38
      itype(3) = 39
      table(1:nncode) = 1d0
      table(2) =-1d0
      table(6) =-1d0
      tabmin(1:nncode) = 1d0
      tabmax(1) = 1d1
      tabmax(2) = dble(maxvec)
      tabmax(3) = dble(maxcls)
      tabmax(4) = nprob
      tabmax(5) = dble(maxcls)
      tabmax(6) = tabmax(2)
      iretur = 0
      nmain = 0
      call sepreadsetkeyw

      if ( ierror/=0 ) go to 1000
      if ( table(2)>0d0 .and. table(6)>0d0 ) &
         call errkeywexcl ( code1, ncod1m, 2, 6 )

      table(2) = max(table(2),table(6))
      inpsep(istart+1) = nint(table(1))
      inpsep(istart+2) = nint(table(2))
      inpsep(istart+3) = mod(nint(table(3)),100000)
      inpsep(istart+4) = nint(table(4))
      inpsep(istart+5) = nint(table(5))
      ivec = nint(table(2))
      iscal = mod(mod(nint(table(3)),100000),5000) ! correction indexed scalars
      neigv =  nint(table(5))
      if ( ivec>0 ) then

!     --- Eigenvectors must be computed

         numvec = min ( max(numvec,ivec+neigv-1), maxvec )
         do i = ivec, min(maxvec,ivec+neigv-1)
            ivectors(i) = 1
         end do
         if ( ivec+neigv-1>maxvec ) then

!        --- Too much eigenvectors to be stored

            call errint ( neigv, 1 )
            call errint ( ivec, 2 )
            call errint ( maxvec, 3 )
            call errsub ( 2052, 3, 0, 0 )

         end if

      end if
      do i = iscal, min(maxcls,iscal+neigv-1)
         iscalars(i) = 1
      end do
      if ( iscal+neigv-1>maxcls ) then

!     --- Too much eigenvalues to be stored

         call errint ( neigv, 1 )
         call errint ( iscal, 2 )
         call errint ( maxcls, 3 )
         call errsub ( 2053, 3, 0, 0 )

      end if
      istart = istart + 6

1000  call erclos ( 'readsceigval' )

      end subroutine readsceigval
