      subroutine readscendfloop ( istart, inpsep, icode, iseqfr, instat, &
                                  timefnd, ifreest )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 22-11-2004 Fortran 90
!        version  1.0  date 29-11-2002
!
!   copyright (c) 2002-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read the input following the following the keywords
!   END_STATIONARY_FREE_BOUNDARY, END_INSTATIONARY_FREE_BOUNDARY, &
!   with respect to subroutine readsc
!   The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     loop
!     stationary
!     free_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*), icode, ifreest, iseqfr
      logical instat, timefnd

!     icode          i    Indicates which of the options in readsc is found
!     ifreest       i/o   ifreest = starting address of stationary free boundary
!                         in inpsep
!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                         2:  starting address in for loop
!     instat        i/o   If true an instationary free boundary loop is active
!     iseqfr        i/o   Sequence number for input of subroutine STATFREE
!     istart        i/o   Next free position in inpsep
!     timefnd       i/o   If true, time integration has been found in loop
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
!       END_STATIONARY_FREE_BOUNDARY_LOOP
!       END_INSTATIONARY_FREE_BOUNDARY_LOOP
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1926   END_FREE_BOUNDARY found, but no start
!    1937   No keyword TIME_INTEGRATION found in instationary loop active
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readscendfloop' )

      if ( iseqfr>0 ) then
         inpsep(istart) = icode
         inpsep(istart+1) = ifreest+3
         istart = istart + 2
         inpsep(ifreest+2) = istart
         iseqfr = iseqfr - 1
      else
         call errsub ( 1926, 0, 0, 0 )
      end if
      if ( icode==27 .and. .not. timefnd ) then

!     --- No keyword time integration has been found in the loop

         call errsub ( 1937, 0, 0, 0 )

      end if
      instat = .false.
      timefnd = .false.

      call erclos ( 'readscendfloop' )

      end
