      subroutine readscinterpol ( istart, inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 22-11-2004 Fortran 90
!        version  1.0  date 29-01-2002
!
!   copyright (c) 2002-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read the input following the following the keyword
!   INTERPOLATE
!   with respect to subroutine readsc
!   The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     interpolation
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                         2: Sequence number V1
!                         3: MESH_IN
!                         4: MESH_OUT
!     istart        i/o   Next free position in inpsep
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncod1=3
      integer ncode1(nncod1), ncod1m(nncod1)
      character (len=20) code1(nncod1)

!     code1          Table with keywords
!     ncod1m         Table containing the number of maximum characters in
!                    the keyword table code1
!     ncode1         Table containing the number of significant letters in
!                    code1
!     nncod1         Number of entries in code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       INTERPOLATE V1,  options
!           where options may have the following contents
!           MESH_IN = 1 (Default 1)
!           MESH_OUT = 2 (Default 2)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     996   mesh_in and mesh_out are equal
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'V', 'MESH_IN', 'MESH_OUT' /
      data ncode1 /  1, 7, 7 /
      data ncod1m /  1, 7, 8 /

!     code1   Table with sub keywords corresponding to keyword INTERPOLATE
!             Values:
!             1:  V
!             2:  MESH_IN
!             3:  MESH_OUT
! ======================================================================
!
      call eropen ( 'readscinterpol' )

      inpsep(istart) = icode

      call sepcopycode ( code1, nncod1, ncode1, ncod1m )

      itype(1:nncode) = 0
      tabmin(1:nncode) = 1d0
      tabmax(1:nncode) = 5d0

      table(1) = 1d0
      table(2) = 1d0
      table(3) = 2d0
      tabmax(1) = dble(maxvec)
      iretur = 0
      nmain = 0
      call sepreadsetkeyw

      if ( ierror/=0 ) go to 1000
      inpsep(istart+1) = nint(table(1))
      inpsep(istart+2) = nint(table(2))
      inpsep(istart+3) = nint(table(3))
      if ( inpsep(istart+2)==inpsep(istart+3) ) then

!     --- mesh_in and mesh_out are equal

         call errint ( inpsep(istart+1), 1 )
         call errsub ( 996, 1, 0, 0 )

      end if
      istart = istart+4

1000  call erclos ( 'readscinterpol' )

      end subroutine readscinterpol
