      subroutine readscobstmesh ( istart, inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 15-12-2017 New storage of intmat
!        version  1.3  date 22-11-2004 Fortran 90
!        version  1.2  date 27-04-2004 Improve computation of istart
!
!   copyright (c) 2002-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read the input following the following the keyword
!   MAKE_OBSTACLE_MESH, REMOVE_OBSTACLE_MESH
!   with respect to subroutine readsc
!   The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     mesh
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                         2: MESH_ORIG Sequence number of mesh to be adapted
!                         3: MESH_OBST Sequence number of adapted mesh
!                         4: IREAD_commat parameter iread for commat
!     istart        i/o   Next free position in inpsep
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncod1=3
      integer ncode1(nncod1), itype1(nncod1), ncod1m(nncod1), icodesav
      double precision :: tabmn1(nncod1), tabmx1(nncod1)
      character (len=20) code1(nncod1)

!     code1          Table with keywords
!     icodesav       Help parameter to save the value of icode
!     itype1         Indicates the type of action corresponding to keywords
!                    in code1
!     ncod1m         Table containing the number of maximum characters in
!                    the keyword table code1
!     ncode1         Table containing the number of significant letters in
!                    code1
!     nncod1         Number of entries in code1
!     tabmn1         Minimum value of items read corresponding to code1
!     tabmx1         Maximum value of items read corresponding to code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       MAKE_OBSTACLE_MESH, options (all in one line)
!           where options may have the following contents
!           MESH_ORIG (Default 1)
!           MESH_OBST (Default 2)
!           SEQ_STRUCTURE (Default -1)
!           Present mesh becomes MESH_OBST
!       REMOVE_OBSTACLE_MESH, options (all in one line)
!           where options may have the following contents
!           MESH_ORIG (Default 1)
!           MESH_OBST (Default 2)
!           Present mesh becomes MESH_ORIG
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     996   mesh_in and mesh_out are equal
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'MESH_ORIG', 'MESH_OBST', 'SEQ_STRUCTURE' /
      data ncode1 /  7, 7, 7 /
      data ncod1m /  9, 9, 13 /
      data itype1 /  0, 0, 0 /
      data tabmn1 / 1d0, 1d0, -1d0 /
      data tabmx1 / 5d0, 5d0,  1d2 /

!     code1   Table with sub keywords corresponding to keyword OBSTACLE_MESH
!             Values:
!             1:   MESH_ORIG
!             2:   MESH_OBST
!             3:   SEQ_STRUCTURE
! ======================================================================
!
      call eropen ( 'readscobstmesh' )

      icodesav = icode

      call sepcopycode ( code1, nncod1, ncode1, ncod1m )
      call sepcopytype ( nncod1, itype1, tabmn1, tabmx1 )
      table(1) = 1d0
      table(2) = 2d0
      table(3) = -1d0
      iretur = 0
      nmain = 0
      call sepreadsetkeyw

      icode = icodesav
      inpsep(istart) = icode
      inpsep(istart+1) = nint(table(1))
      inpsep(istart+2) = nint(table(2))
      if ( inpsep(istart+1)==inpsep(istart+2) ) then

!     --- mesh_orig and mesh_obst are the same

         call errint ( inpsep(istart+1), 1 )
         call errsub ( 996, 1, 0, 0 )

      end if
      if ( icode==59 ) then
         inpsep(istart+3) = nint(table(3))
         istart = istart+4
      else
         istart = istart+3
      end if

      call erclos ( 'readscobstmesh' )

      end
