      subroutine readscplotfn ( istart, inpsep, rinsep, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 09-01-2015 change of inpsep
!        version  1.2  date 12-05-2014 Extension of inpsep
!        version  1.1  date 03-07-2012 Use subroutine intxstoretext
!
!   copyright (c) 2007-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read options for keyword plot function
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     plot_function
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(0:*), istrin
      double precision :: rinsep(*)

!     inpsep         o    The integer input read is stored in array INPSEP
!                         INPSEP is filled as follows
!                          0: icode
!                          1: NUMVECTORS, number of vectors
!                          2: ITARTVECTORS, start address of vectors
!                          3: DEGFD, sequence number of degree of freedom
!                          4: NUMCURVES: number of curves that are used
!                          5: ISTARTCURVS:  Start address of curve numbers in
!                                         this array
!                             In case itype=1, start address of coordinates
!                             in rinsep
!                          6: ITEXTX: sequence number of x-text in TEXT array
!                          7: NTEXTX: number of characters in x-text
!                          8: ITEXTY: sequence number of y-text in TEXT array
!                          9: NTEXTY: number of characters in y-text
!                         10: NUMLEGENDA
!                         11: ISTARTLEGENDA
!                         12: NUMCOLORS
!                         13: ISTARTCOLORS
!                         14: itype
!                             0: curves
!                             1: points and surface
!                          Followed by vector sequence numbers and curve numbers
!     istart        i/o   Next position in array inpsep
!                         The parameter is raised in this subroutine
!     istrin         i    Starting address of information for this subroutine
!                         in RINSEP
!     rinsep         i    Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode2=8, nncode1=3
      character (len=7) code2(nncode2)
      character (len=10) code1(nncode1)
      character (len=13) namemain
      double precision :: tabmin2(nncode2), tabmax2(nncode2)
      integer ncode2(nncode2), ncodmx2(nncode2), itype2(nncode2), &
              ncode1(nncode1), &
              istartcur, nread, iseqv, istartvec, numvectors

!     code1          Table with keywords
!     code2          Table with keywords
!     iseqv          Sequence number of keyword V in array code1
!     istartcur      starting position of curves in inpsep
!     istartvec      starting position of vector numbers in inpsep
!     itype2         Table containing information about the numbers behind
!                    the keywords in the table code2
!     namemain       Name of main keyword
!     ncode1         Table containing the number of significant letters in
!                    code1
!     ncode2         Table containing the number of significant characters
!                    in the keyword table code2
!     ncodmx2        Table containing the maximal number of  characters in
!                    the keyword table code
!     nncode1        Number of entries in code1
!     nncode2        Number of entries in code2
!     nread          Number of items read
!     numvectors     number of vectors read
!     tabmax2        Contains maximum values allowed in table2
!     tabmin2        Contains minimum values allowed in table2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRKEYWEXCLUS  Check if two keywords are mutually exclusive
!     ERRSUB         Error messages
!     INTXSTORETEXT  Fill reference to text and number of characters in inpsep
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2227   Only one of a combination of keywords found
!    3214   Number of points found incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The following text is recognized
!
!     PLOT_FUNCTION, VECTORnames, options
!        with options:
!        curves ( cj1, cj2, ... )    Default: c1
!        degfd i                     Default:  1
!        textx                       Default: curve numbers or x (1d)
!        texty                       Default: name_vector
!        line ((x1,y1,z1),(x2,y2,z2))
!        surface (s1)
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'VECTOR', 'DEGFD', 'CURVES', 'TEXTX', 'TEXTY', 'V', &
                   'LINE', 'SURFACE' /
      data ncode2  /  4, 4, 4, 5, 5,   1, 2, 2 /
      data ncodmx2 /  6, 5, 6, 5, 5,   1, 4, 7 /
      data itype2  / 40, 0,18, 6, 6,  40,17,18 /
      data tabmin2 / 1d0, 1d0, 1d2, 0d0, 0d0,   1d0, 2d0, 1d0 /
      data tabmax2 / 1d0, 1d2, 1d1, 0d0, 0d0,   1d0, 1d0, 1d0 /
      data code1/ 'C', 'P', 'S' /
      data ncode1  / 1, 1, 1 /
      data namemain  / 'PLOT_FUNCTION' /

!     code2     Table with keywords:
!               1: VECTOR
!               2: DEGFD
!               3: CURVES
!               4: TEXTX
!               5: TEXTY
!               6: V
!               7: LINE
!               8: SURFACE
! ======================================================================
!
      call eropen ( 'readscplotfn' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscplotfn'
         write(irefwr,1) 'istart', istart
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(0) = icode     ! code with respect to tosepcomsub

      istartvec = 15  ! starting position of vectors in inpsep
      iseqv = 6       ! position of keyword v in table

!     --- Set tabmax for variable quantities

      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )
      call sepcopytype ( nncode2, itype2, tabmin2, tabmax2 )
      call sepcopycodesub ( code1, nncode1, ncode1 )
      tabmax(1) = 0             ! number of vectors
      tabmax(3) = ncurvs        ! maximum of curves numbers
      tabmax(iseqv) = dble(maxvc1)  ! maximum number of vectors

!     --- Initialize table

      table(1:nncode) = -1d0
      table(2) = 1d0
      tabmin(7) = ndim      ! ndim (number of coordinates per point)
      tabmax(7) = 2             ! maximum number of points
      table(8) = -3d0           ! reference to s for surface
      tabmax(8) = nsurfs        ! maximum of surface numbers

!     --- Read input

      iretur = 0
      nmain = 0
      call sepreadsetkeyw

      if ( debug ) then
         call prinrl ( table, nncode, 'table' )
         call prinrl ( tabmax, nncode, 'tabmax' )
         call prinrl ( tabout, 2, 'tabout' )
      end if  ! ( debug )

!     --- Fill array inpsep

      inpsep(10:14) = 0

      iposut = nint(table(1))
      if ( iposut>0 ) then

!     --- iposut>0, hence vectors found

         numvectors = nint(tabmax(1))
         inpsep(istartvec:istartvec+numvectors-1) = &
            nint(tabout(iposut:iposut+numvectors-1))

      else if ( table(iseqv)>0d0 ) then

!     --- table(iseqv)>0, hence v... found

         numvectors = 1
         inpsep(istartvec) = nint(table(iseqv))

      else

!     --- iposut=0, no vectors found
!         Error: no vector sequence number found

         call errchr ( namemain, 1 )
         call errchr ( code(1)(1:ncodmx(1)), 2 )
         call errsub ( 2227, 0, 0, 2 )
         numvectors = 0

      end if  ! ( numvectors<1 )
      istartcur = istartvec+numvectors
      inpsep(5) = istartcur  ! start of curves

      inpsep(1) = numvectors
      inpsep(2) = istartvec
      inpsep(3) = nint(table(2))   ! degfd

      nread = 0
      if ( table(3)>0d0 ) then

!     --- table(3)>0, curves found

         iposut = nint(table(3))  ! position in tabout
         nread = nint(tabmax(3))  ! number of curves read

!        --- Store curve numbers

         inpsep(istartcur:istartcur+nread-1) = &
            nint(tabout(iposut:iposut+nread-1))

!        --- Check on appearance of keywords 7 and 8

         call errkeywexclus ( table, code, ncodmx, 3, 7 )
         call errkeywexclus ( table, code, ncodmx, 3, 8 )

      else if ( table(7)>0d0 ) then

!     --- table(7)>0, line found, hence there must be 2 points and possibly
!         a surface

         iposut = nint(table(7))  ! position in tabout
         nread = nint(tabmax(7))  ! number of points read
         inpsep(4) = 1            ! number of curves read

         if ( nread/=2 ) then

!        --- Not 2 points found after line

            call errchr ( code(7), 1 )
            call errint ( nread, 1 )
            call errint ( 2, 2 )
            call errsub ( 3214, 2, 0, 1 )
            go to 1000

         end if  ! ( nread/=2 )

         inpsep(14) = 1           ! itype

!        --- Store starting address in rinsep and coordinates

         inpsep(5) = istrin
         nread = 2*ndim-1
         rinsep(istrin:istrin+nread) = tabout(iposut:iposut+nread)
         istrin = istrin+nread+1
         nread = 1

         if ( table(8)>0d0 ) then

!        --- Surface found

            iposut = nint(table(8))  ! position in tabout
            inpsep(istartcur) = nint(tabout(iposut))

         else

!        --- No surface found, use 1

            inpsep(istartcur) = 1 ! default

         end if  ! ( table(8)>0d0 )
         nread = 1

      else

!     --- No curves found, use default

         nread = 1
         inpsep(istartcur) = 1

      end if  ! ( table(3)>0d0 )
      inpsep(4) = nread      ! number of curves read

!     --- textx

      call intxstoretext ( inpsep(6), table(4), tabmax(4), &
                           strout )

!     --- texty

      call intxstoretext ( inpsep(8), table(5), tabmax(5), &
                           strout )

!     --- Raise parameters

      istart = istart+15+nread+numvectors+inpsep(10)+inpsep(12)

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpsep(1), istartcur+nread-1, 'inpsep' )
         write(irefwr,1) 'istart', istart
         write(irefwr,*) 'End readscplotfn'

      end if  ! ( debug )
      call erclos ( 'readscplotfn' )

      end subroutine readscplotfn
