      subroutine readscsettime ( istart, inpsep, istrin, rinsep, icode )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 28-05-2011 Correction scalars
!        version  1.0  date 04-02-2004
!
!   copyright (c) 2004-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information found after the keyword set_time and store in inpsep
!     and rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     interpret
!     read
!     set_time
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), istrin, istart, icode
      double precision rinsep(*)

!     icode          i    Return code with respect to subroutine INTX01
!                         Indicates position in array code or code1
!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         See tosepcomsub
!     istart        i/o   Next free position in inpsep
!     istrin         i    Starting address of information for this subroutine
!                         in RINSEP
!     rinsep        i/o   Array containing the real input for this subroutine
!                         See tosepcomsub
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRSUB         Error messages
!     READREALSCAL   Read integer, real or scalar
! **********************************************************************
!
!                       I/O
!
!     READ_TIME t
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1869   No number found after keyword
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readscsettime' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscsettime'

      end if
      if ( ierror/=0 ) go to 1000
      inpsep(istart) = icode

!     --- Read next item

      call readrealscal
      if ( ityprd==4 ) then

!     --- string found, check if it is a variable

      end if  ! ( ityprd==4 )
      if ( ityprd==2 .or. ityprd==3 ) then

!     --- Integer or real found

         rinsep(istrin) = realrd
         inpsep(istart+1) = istrin
         istrin = istrin + 1

      else if ( ityprd==11 ) then

!     --- scalar found

         inpsep(istart+1) = nint(realrd)

      else

!     --- Not a number found

         call errchr ( 'SET_TIME', 1 )
         call errsub ( 1869, 0, 0, 1 )
         call errcrd

      end if

      istart = istart+2

1000  call erclos ( 'readscsettime' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readscsettime'

      end if

      end
