      subroutine readscuserout ( istart, inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 04-06-2008 Debug information
!        version  1.2  date 22-11-2004 Fortran 90
!        version  1.1  date 05-08-2004 Extend maximum sequence numbers
!
!   copyright (c) 2002-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Read the input following the following the keyword
!   USER_OUTPUT
!   with respect to subroutine readsc
!   The input is stored in array inpsep and the real array rinsep
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     output
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*)

!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         This part contains:
!                         2:  Sequence number USEROUT
!                         3:  nextra is number of extra integer positions
!                             If this number is larger than 0, nextra
!                             positions are used after position 2
!     istart        i/o   Next free position in inpsep
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncod1=3
      integer ncode1(nncod1), ncod1m(nncod1), i, m, n
      character (len=20) code1(nncod1)

!     code1          Table with keywords
!     i              Counting variable
!     m              number of items read
!     n              number of items read
!     ncod1m         Table containing the number of maximum characters in
!                    the keyword table code1
!     ncode1         Table containing the number of significant letters in
!                    code1
!     nncod1         Number of entries in code1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       USER_OUTPUT, sequence_number = i, extra_integers(i_1,i_2,..), &
!                    extra_scalars(r_1, r_2,..)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1612   Too much variables read
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code1 / 'SEQUENCE_NUMBER', 'EXTRA_INTEGERS', &
                   'EXTRA_SCALARS' /
      data ncode1 / 3,  7,  7 /
      data ncod1m /15, 14, 13 /

!     code1  Table with sub keywords corresponding to keyword USER_OUTPUT
!             Values:
!             1:  SEQUENCE_NUMBER
!             2:  EXTRA_INTEGERS
!             3:  EXTRA_SCALARS
! ======================================================================
!
      call eropen ( 'readscuserout' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readscuserout'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(istart) = icode          ! code with respect to tosepcomsub

      call sepcopycode ( code1, nncod1, ncode1, ncod1m )

      itype(1) = 0
      itype(2:3) = 28
      tabmin(1) = 1d0
      tabmin(2) = 1d1
      tabmin(3) = 5d0
      tabmax(1) = 1d3
      tabmax(2:3) = 1d9
      table(1:3) = 0d0
      iretur = 0
      nmain = 0
      call sepreadsetkeyw
      if ( ierror/=0 ) go to 1000
      inpsep(istart+1) = nint(table(1))
      if ( table(2)>0 ) then
         n = nint(tabmax(2))
      else
         n = 0
      end if
      inpsep(istart+2) = n
      if ( n>0 ) then

!     --- Extra integers found, store in inpsep

         iposut = nint(table(2))
         do i = 1, n
            inpsep(istart+3+i) = nint(tabout(iposut-1+i))
         end do
         if ( debug ) &
            call prinin ( inpsep(istart+4), n, 'inpsep_integers' )

      end if

!     --- Check for extra scalars

      if ( table(3)>0 ) then
         m = nint(tabmax(3))
      else
         m = 0
      end if
      if ( m>maxcls ) then

!     --- Too much variables

         call errint ( m, 1 )
         call errint ( maxcls, 2 )
         call errsub ( 1612, 2, 0, 0 )
         call errcrd

      else

         inpsep(istart+3) = m
         if ( m>0 ) then

!        --- Extra scalars found, store in inpsep

            iposut = nint(table(3))
            do i = 1, m
               inpsep(istart+3+n+i) = &
                    mod(nint(tabout(iposut-1+i)),100000000)
            end do
            if ( debug ) &
               call prinin ( inpsep(istart+4+n), n, 'inpsep_scals' )

         end if
         istart = istart + 4 + n + m

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinin ( inpsep(istart-4-n-m), 4+n+m, 'inpsep' )
         write(irefwr,*) 'End readscuserout'

      end if  ! ( debug )
      call erclos ( 'readscuserout' )

      end subroutine readscuserout
