      subroutine readsoluts ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 18-12-2013 Extra parameters inpsep, istartmn
!        version  1.1  date 04-03-2011 New call to readoutp
!        version  1.0  date 28-11-2006
!
!   copyright (c) 2006-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read all solution arrays from sepcomp.in for one time step
!     Adapt time if necessary
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduletime
      use sepmodulemach
      use sepmoduletexts
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)

!     inpsep         i    Integer input array
!                         1:  -
!                         2:  sequence number of file
!                         3:  -
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqlc, iter, ios, i, irefsav, ndtvecold, iref
      double precision tread
      logical endfile, finished, tfound

!     endfile        if true end of file has been reached
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     i              Counting variable
!     ios            IOSTAT variable, returns from reading or writing
!     iref           File reference number
!     irefsav        Help parameter to save the value of iref74
!     iseqlc           Sequence number of vector in isolut2d
!     iter           Iteration sequence number
!     ndtvecold      Value of ndtvec at start of subroutine
!     tfound         If true the actual time has been reached
!     tread          Time read from sepcomp.out
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inisetnewref

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     INISETNEWREF   Create new reference number
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     READ74HEADING  Read first part of file sepcomp.out (no storage)
!     READ74PART2    Read second part of file sepcomp.out (no storage)
!     READ74SUB1     Read first part of information of solution array from
!                    sepcomp.out
!     READOUTP       read real/complex sepran solution array from the
!                    file sepcomp.out
!     READSOLDUM     Read lines with respect to solution and skip
!                    these results
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     747   End of file reached
! **********************************************************************
!
!                       PSEUDO CODE
!
!     finished := false;    tfound := false
!     while ( not finished ) do
!        read heading part
!        if ( end_of_file ) then
!           finished := true
!        else if ( tread<t ) then
!           skip solution in file
!        else if ( not tfound ) then
!           t := tread
!           read solution
!           tfound := true
!        else if ( tread>t ) then
!           rewind heading part
!           finished := true
!        else
!           read solution
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data endfile /.false./
! ======================================================================
!
      call eropen ( 'readsoluts' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readsoluts'
         call prinin ( inpsep, 3, 'inpsep' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      irefsav = iref74
      if ( inpsep(2)>0 ) then

!     --- inpsep(2)>0, file name found
!         Find new reference number and open file

         iref = inisetnewref ()
         open ( unit=iref, file=texts(inpsep(2)) )
         iref74 = iref

         if ( debug ) write(irefwr,*) 'file_name ', texts(inpsep(2))

!        --- Read first part of file

         call read74heading
         call read74part2

      else

!     --- Use standard sepcomp.out

         iref74 = iref73

      end if  ! ( inpsep(2)>0 )

      if ( endfile ) then

!     --- No more input

         call errsub ( 747, 0, 0, 0 )
         go to 1000

      end if  ! ( endfile )

!     --- finished := false;    tfound := false

      finished = .false.
      tfound = .false.
      ndtvecold = ndtvec

!     --- while ( not finished ) do

      do while ( .not. finished )

!     --- read heading part

         call read74sub1 ( iseqlc, iter, tread, ios )
         if ( ios>=0 ) iseqlc = iseqlc+1
         if ( debug ) then
            write(irefwr,1) 'iseqlc, iter, ios', iseqlc, iter, ios
            write(irefwr,2) 'tread, t', tread, t
         end if  ! ( debug )

         if ( ios<0 ) then

!        --- end_of_file found
!            finished := true

            finished = .true.
            endfile = .true.

         else if ( tread<t ) then

!        --- tread<t
!            skip solution in file

            if ( debug ) write(irefwr,*) 'readsoldum'
            call readsoldum

         else if ( .not. tfound ) then

!        --- not tfound
!            t := tread
!            read solution
!            tfound := true

            t = tread
            if ( debug ) write(irefwr,2) 'read solution: time =', t
            call readoutp ( isolut2d(iseqlc), ndtvecold )
            tfound = .true.

         else if ( tread>t ) then

!        --- tread>t
!            rewind heading part
!            finished := true

            if ( debug ) write(irefwr,*) 'rewind'
            do i = 1, 7
               backspace ( unit=abs(iref74) )
            end do  ! i = 1, 7
            finished = .true.

         else

!        --- read solution

            if ( debug ) write(irefwr,2) 'read solution/2: time =', t
            call readoutp ( isolut2d(iseqlc), ndtvecold )

         end if  ! ( ios<0 )

      end do  ! while ( .not. finished )

      istartmn = istartmn+4

1000  if ( inpsep(2)>0 ) then

!     --- remove new reference number and close file

         close ( unit=iref )
         call inirmref ( iref )

      end if  ! ( inpsep(2)>0 )
      iref74 = irefsav

      if ( debug ) then

!     --- Debug information

         call prinin1 ( isolut2d, iseqlc, 5, 'solution' )
         write(irefwr,*) 'End readsoluts'

      end if
      call erclos ( 'readsoluts' )

      end subroutine readsoluts
