      subroutine readtmtstep ( ndt, iintim, rintim, leniintim, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information concerning tstep in iintim/rintim
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ndt
      integer, intent(inout) :: leniintim
      integer, intent(inout) :: istrin, iintim(*)
      double precision, intent(inout) :: rintim(*)

!     iintim        i/o   Contains the input for time integration
!                         See subroutine readtm1
!     istrin        i/o   Starting address in array RINSEP
!     leniintim     i/o   Computed length of array iintim
!     ndt            i    Number of different time steps
!     rintim        i/o   Contains real input for time integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseq

!     iseq           Sequence number of scalar
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: getnamevar, getnamereal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     GETNAMEREAL    Return with sequence number of real in rlcons
!     GETNAMEVAR     Return with sequence number of variable in scalars
!     READSERVAR     Store numitems reals or variables in iinput
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readtmtstep' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readtmtstep'
         call ertrace
         write(irefwr,1) 'leniintim', leniintim
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,2) 'table(5)', table(5)

      leniintim = leniintim+1
      if ( table(5)>0d0 ) then

!     --- tstep given

         call readservar ( table(5), iintim(leniintim), rintim, ndt, istrin )

      else

!     --- tstep not given

         iseq = getnamevar ( 'tstep' )   ! check for variable
         if ( iseq>0 ) then
            iseq = iseq+maxints+maxreals+100000000  ! scalar found
         else
            iseq = getnamereal ( 'tstep' )   ! check for real
            if ( iseq>0 ) iseq = iseq+maxints+100000000  ! real found
         end if  ! ( iseq>0 )
         if ( iseq>0 ) then

!        --- iseq>0, variable name found

            iintim(leniintim:leniintim+ndt-1) = iseq

         else

            iintim(leniintim:leniintim+ndt-1) = 0

         end if  ! ( iseq>0 )

      end if

      iintim(52) = leniintim
      leniintim = leniintim+ndt-1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readtmtstep'

      end if  ! ( debug )
      call erclos ( 'readtmtstep' )

      end subroutine readtmtstep
