      subroutine readvalcons ( value, name, rinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 29-07-2010 fortran 90
!        version  1.0  date 28-06-2004
!
!   copyright (c) 2004-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Put the value of the parameter read in rinput
!     If no parameter is found, check if the name is known
!     Otherwise give an error message
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision value, rinput
      character *(*) name

!     name           i    Name of parameter to be filled
!     rinput         o    Real entry of input array in which value must
!                         be stored
!     value          i    value of parameter
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision getconst

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     GETCONST       Get the value of a constant in the input block constants
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2658   Parameter has not been given a value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'readvalcons' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from readvalcons'

      end if
      if ( ierror/=0 ) go to 1000

      if ( value>-1d19 ) then

!     --- constant is given

         rinput = value

      else

!     --- constant is not given

         iwarn = 0
         rinput = getconst ( name )
         if ( iwarn==2156 ) then

!        --- error: constant is not given

            call errchr ( name, 1 )
            call errsub ( 2658, 0, 0, 1 )

         end if  ! ( rinput<=0d0 ) then

      end if  ! ( value>0d0 )

1000  call erclos ( 'readvalcons' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End readvalcons'

      end if

      end
