      subroutine sepcompmain
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-01-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Solve linear and nonlinear stationary problems by SEPRAN, where the
!   mesh has been created by SEPMESH and postprocessing is performed by SEPPOST
!   The mesh must be stored in the file meshoutput or its new name defined
!   in the sepran.env file
!   If the adaptive mesh strategy is demanded (in the input data section
!   SOLVE ... END) then the file meshoutput will be overwritten
!   by the file corresponding to the last of the adaptive meshes.
!   The output is written to the files sepcompmainp.inf and sepcompmainp.out
!   (or their new names as defined in the sepran.env file)
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemain
      use sepmoduleinput
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, isubr, iread, istop, maxcall, ipos

!     i              Counting variable
!     ipos           Position of input arrays in sepmoduleinput
!     iread         Indicates if all SEPRAN input until end
!                   of file or END_OF_SEPRAN_INPUT is read (1)
!                   or that only the input as described for SEPSTR is read (0)
!     istop         Indication if file 2 must be closed and saved (#0)
!                   or not (0)
!     isubr         Defines from which subroutine TO0092 is called
!                   Possible values:
!                   1:  SEPSTR, nbuf does not have a value
!                   2:  SEPSTL, nbuf has a value
!                   3:  SEPSTN, nbuf may have a value
!                    All SEPRAN input is read in this subroutine
!                    provided iread = 1
!     maxcall        Maximum value of icall in sepmoduleinp
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     FINISH         Stop SEPRAN
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
!     STARTSEPRAN    Create buffer array and fill part of sepmodulebuf
!     TO0092         Body of subroutine SEPSTR or SEPSTL
!     TO0094         Executes all work of SEPCOM except for the reading of mesh
!                    and problem
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Clear all arrays
!   Open SEPRAN, read mesh (SEPSTL)
!   Read rest of input and store in ibuffr
!   Fill essential boundary conditions if required
!   The remainder of the tasks is carried out by TO0094
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Initialize all arrays

      call startsepran

!     --- Open SEPRAN and read mesh

      iread = 1
      isubr = 4
      call to0092 ( isubr, iread, maxmesh )
      if ( ierror/=0 ) go to 1000
      mainsubr = 2
      call eropen ( 'sepcompmain' )
      if ( ierror/=0 ) go to 1000

      removesol = .true.
      removeintmat = .true.

!     --- Rest of the tasks are carried out by TO0094
!         First compute starting address of arrays inpsep and rinsep
!         Next copy contents to end of ibuffr

      call sepgetinpfinf ( 10, 1, ipos, maxcall )

      call to0094 ( in(ipos)%iinput, re(ipos)%rinput )

      print *, 'sepcomp has finished'
1000  call erclos ( 'sepcompmain' )

!     --- Close all files, including file 2 if necessary

      istop = 0
      if ( inew/=0 ) istop = 1
      call finish ( istop )

      end
