      subroutine sepcopcurvesym
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy curves of axi-symmetric mesh and fill a part of new kmesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     mesh
!     curve
!     symmetric
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesymm
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ncurvsnew, i, nnodes, lenpart3, nstar3, nstar4
      integer, pointer :: nodes(:)

!     i              Counting variable
!     lenpart3       Length of kmeshm part 3
!     ncurvsnew      New number of curves
!     nnodes         Number of nodes
!     nodes          Contains the node numbers
!     nstar3         Starting address of KMESHM part 3 in KMESHM containing the
!                    node numbers of the curves
!     nstar4         Starting address of KMESHM part 4 in KMESHM containing the
!                    inner boundaries
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPFINDCURVSYM Find all curves that are positioned on the symmetry axis
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcopcurvesym' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcopcurvesym'
         nstar4 = kmeshm(3)
         call prinin ( kmeshm, nstar4+ninner-1, 'kmeshm' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Find all curves that are on the symmetry axis

      call sepfindcurvsym

!     --- Compute length of new array kmeshm

      ncurvsnew = 2*ncurvs-numcurvsymm
      km(iseqkm)%kmesh(11) = ncurvsnew

      lenpart3 = kmeshm(3)-kmeshm(2)

      if ( debug ) &
         write(irefwr,1) 'ncurvs, numcurvsymm, ncurvsnew, lenpart3', &
                          ncurvs, numcurvsymm, ncurvsnew, lenpart3

      do i = 1, ncurvs
         if ( iworkcur(i)==0 ) lenpart3 = lenpart3+Curvesub(i)%nnodes
      end do  ! i = 1, ncurvs

      length = 4+ncurvsnew+1+lenpart3+2*ninner+numcurvsymm

      if ( debug ) &
         write(irefwr,1) 'ninner, lenpart3. length', ninner, lenpart3, length

!     --- Create new array kmeshm

      allocate ( kmeshmtmp(length), stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'kmeshmtmp' )
      if ( ierror/=0 ) go to 1000

!     --- Fill new kmeshmtmp

      kmeshmtmp(1) = 2*ninner+numcurvsymm
      kmeshmtmp(2) = 6+ncurvsnew
      kmeshmtmp(3) = kmeshmtmp(2)+lenpart3
      kmeshmtmp(4) = kmeshmtmp(1)
      nstar3 = kmeshmtmp(2)
      nstar4 = kmeshmtmp(3)
      lenpart3 = kmeshm(3)-kmeshm(2)
      kmeshmtmp(5:5+ncurvs) = kmeshm(5:5+ncurvs)
      kmeshmtmp(nstar3:nstar3+lenpart3-1) = kmeshm(kmeshm(2):kmeshm(3)-1)
      kmeshmtmp(nstar4:nstar4+ninner-1) = kmeshm(kmeshm(3):kmeshm(3)+ninner-1)
      nstar3 = nstar3+lenpart3
      nstar4 = nstar4+ninner

      ncurvsnew = ncurvs
      do i = 1, ncurvs

         if ( iworkcur(i)==0 ) then

!        --- New curve found

            nnodes = Curvesub(i)%nnodes
            nodes => Curvesub(i)%nodes
            ncurvsnew = ncurvsnew+1
            kmeshmtmp(nstar3:nstar3+nnodes-1) = nodesnew(nodes)
            nstar3 = nstar3+nnodes
            kmeshmtmp(5+ncurvsnew) = nstar3+1-kmeshmtmp(2)

            if ( Curvesub(i)%inner_curve ) then

!           --- inner curve found

               kmeshmtmp(nstar4) = ncurvsnew
               nstar4 = nstar4+1

            end if  ! ( Curvesub(i)%inner_curve )

         end if  ! ( iworkcur(i)==0 )

      end do  ! i = 1, ncurvs

      kmeshmtmp(nstar4:nstar4+numcurvsymm-1) = listsymmc(1:numcurvsymm)

      km(iseqkm)%kmesh(11) = ncurvsnew

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kmeshmtmp, length, 'kmeshmtmp' )
         write(irefwr,*) 'End sepcopcurvesym'

      end if  ! ( debug )
      call erclos ( 'sepcopcurvesym' )

      end subroutine sepcopcurvesym
