      subroutine sepexecif ( inpsep, rinsep, isubr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-03-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Execute if statement in sepcomsub
!     Return with correct value of istartmn
!     Old subroutine prexecif
! **********************************************************************
!
!                       KEYWORDS
!
!     compute_all
!     if_statement
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemnsub
      use sepmodulecomp
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), isubr
      double precision rinsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         Contents
!                         0:   83 to indicate that this is an if statement
!                         1:   if >0 parameter k in call to userbool
!                              if <0, -length of boolean expression in
!                              array inpsep (no use of userbool)
!                         2:   Next address in inpsep if result of
!                              expression is false
!                         Rest of positions, information of boolean expression
!                         as described in readboolexpr
!     isubr          i    defines type of action
!                          83:  if
!                         115: else_if
!     rinsep         i    Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      logical userbool, prevalboolexpr

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PREVALBOOLEXPR Evaluate boolean expression and return with result
!     PRININ         print 1d integer array
!     USERBOOL       User subroutine corresponding to boolean_expr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepexecif' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepexecif'
         write(irefwr,1) 'istartmn, iseqif', istartmn, iseqif
         call prinin ( inpsep, 5, 'inpsep' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( isubr==83 ) iseqif = iseqif+1

      if ( checkif(iseqif) ) then

!     --- If statement was true, so skip

         istartmn = inpsep(2)

      else if ( inpsep(1)<0 ) then

!     --- inpsep(1)<0, hence a boolean expression given
!         evaluate this expression

         if ( debug ) write(irefwr,*) 'evaluate boolean expression'
         if ( prevalboolexpr ( inpsep(3), -inpsep(1), rinsep, &
              'if statement' ) ) then

!        --- If must be executed, go to next step

            istartmn = istartmn-inpsep(1)+3
            checkif(iseqif) = .true.

         else

!        --- If must not be executed, skip loop

            istartmn = inpsep(2)

         end if  ! ( prevalboolexpr ( inpsep(1), rinsep ) )

      else

!     --- inpsep(1)>0, hence use userbool

         if ( userbool ( inpsep(1) ) ) then

!        --- If must be executed, go to next step

            istartmn = istartmn + 3
            checkif(iseqif) = .true.

         else

!        --- If must not be executed, skip loop

            istartmn = inpsep(2)

         end if

      end if  ! ( checkif(iseqif) )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istartmn, iseqif', istartmn, iseqif
         write(irefwr,*) 'checkif(iseqif)', checkif(iseqif)
         write(irefwr,*) 'End sepexecif'

      end if  ! ( debug )
      call erclos ( 'sepexecif' )

      end subroutine sepexecif
