      function sepfilldefint ( nameint, defval )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-09-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Give sepfilldefint either the default value defval or the value of
!     the variable nameint if it exists
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: sepfilldefint
      integer, intent(in) :: defval
      character (len=*), intent(in) :: nameint

!     defval         i    Default value
!     nameint        i    Name of integer
!     sepfilldefint  o    Value of integer at output
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseq

!     iseq           Sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepgetnamevar
      double precision :: sepgetvar

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPGETNAMEVAR  Get sequence number of scalar, real or integer
!     SEPGETVAR      Get value of scalar, real or integer
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfilldefint' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfilldefint'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseq = sepgetnamevar ( nameint )

      if ( iseq>0 ) then

!     --- iseq > 0, hence it concerns a variable or constant

         sepfilldefint = nint(sepgetvar(iseq))

      else

!     --- iseq = 0, use default

         sepfilldefint = defval

      end if  ! ( iseq>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfilldefint'

      end if  ! ( debug )
      call erclos ( 'sepfilldefint' )

      end function sepfilldefint
