      subroutine sepfillnlevset ( iobst, nodes, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill nodal points at boundray of obstacle or zero-level set
!     Old subroutine prfillnlevset0
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     boundary
!     obstacle
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iobst, nodes(*), nnodes

!     iobst          i    Obstacle sequence number
!     nnodes         o    Number of nodes in nodes
!     nodes          o    Contains the node numbers that must be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart,  numbndobst

!                    they are not
!     istart         First position used at a part
!     numbndobst     Number of nodal points on the boundary of the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIZEROLEV     Compute number of nodes in zero level set and starting
!                    position in ipointobst
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillnlevset' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillnlevset'
         write(irefwr,1) 'iobst', iobst
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call inizerolev ( ipointobst, istart, numbndobst, iobst )
      nodes(1:numbndobst) = ipointobst(istart+1:istart+numbndobst)

      nnodes = numbndobst

1000  call erclos ( 'sepfillnlevset' )
      if ( debug ) then

!     --- Debug information

         call prinin ( nodes, nnodes, 'nodes' )
         write(irefwr,*) 'End sepfillnlevset'

      end if  ! ( debug )

      end subroutine sepfillnlevset
