      function sepgetsubstr ( stringname, substrings, numsubstr, &
                              numsign, maxnumlet )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-09-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Get sequence number of stringname in set of substrings
!     If not found return with 0
! **********************************************************************
!
!                       KEYWORDS
!
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: sepgetsubstr
      integer, intent(in) :: numsubstr, numsign(numsubstr), &
                             maxnumlet(numsubstr)
      character (len=*), intent(in) :: substrings(numsubstr)
      character (len=*), intent(in) :: stringname

!     maxnumlet      i    Maximum number of characters in sub-keywords
!     numsign        i    Number of siginificant characters in sub-keywords
!     numsubstr      i    Number of sub-keywords
!     sepgetsubstr   o    Value of the sequence number at output
!     stringname     i    Name of string
!     substrings     i    Set of sub-keywords
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, nch, lenstring
      character (len=132) :: vname1, vname2

!     i              Counting variable
!     lenstring      Help parameter to indicate the length of a string
!     nch            Number of characters in a string
!     vname1         Name of variable translated to capitals
!     vname2         Name of variable translated to capitals
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKUPC         Translate all letters to upper case
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepgetsubstr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepgetsubstr'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      sepgetsubstr = 0

      lenstring = index ( stringname, ' ' ) -1
      if ( lenstring==-1 ) lenstring = len(stringname)
      vname1 = stringname(1:lenstring)
      call askupc(vname1)

!     --- Check if stringname is part of the set substrings

      do i = 1, numsubstr


         if ( lenstring<=maxnumlet(i) ) then

            length = numsign(i)
            vname2 = substrings(i)(1:length)
            call askupc(vname2)
            if ( vname1(1:length)==vname2(1:length) ) then

!           --- name of string found

               sepgetsubstr = i
               go to 1000

            end if
         end if

      end do  ! i = 1, numsubstr

1000  call erclos ( 'sepgetsubstr' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepgetsubstr'

      end if  ! ( debug )

      end function sepgetsubstr
