      subroutine seppresscorr1 ( finished, first, isubrlc, iseqpresscor, &
                                 iinpresscor, rinpresscor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-11-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     solves Navier-Stokes by pressure correction
!     Input may be from the standard input file or through the parameter list
!     Old subroutine prtimefr04
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
!     navier_stokes_equation
!     pressure_correction
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulebuild
      use sepmodulecons
      use sepmoduletime
      use sepmoduleplot
      use sepmodulepost
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: isubrlc, iseqpresscor, iinpresscor(*)
      double precision, intent(in) :: rinpresscor(*)
      logical, intent(inout) :: first, finished

!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
!     first         i/o   If true it concerns the first step
!     iinpresscor    i    Integer input array for the pressure correction
!                         process
!                         See subroutine readpresscor
!     iseqpresscor   i    Sequence number of input for pressure correction
!     isubrlc        i    Indicates the type of calling subroutine
!     rinpresscor    i    Real input array for the pressure correction process
!                         See subroutine readpresscor
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: eps
      double precision :: timebefore(2)

!     eps            Local accuracy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERSETTIME      Get the CPU time at start of subroutine
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     SEPTIMOUTP     Perform output for subroutine septimprb during a time loop
!     SEPTIMPRESSCR  Compute one time step for the incompressible
!                    Navier-Stokes equations, pressure correction is applied
!     SEPTIMSET      Set some time parameters in the time loop
!     SEPTIMSTART    Start part of subroutine septimprb
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   -
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill some constpstants
!     Write initial condition depending on toutin
!     While t<tend do
!        compute tout
!        call time stepping subroutine and raise t
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppresscorr1' )
      if ( itime>0 ) call ersettime ( timebefore )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppresscorr1'
         write(irefwr,100) 'iseqpresscor', iseqpresscor
         write(irefwr,110) 't, tout, tend, tstep', t, tout, tend, tstep
         write(irefwr,*) 'complrun, finished ', complrun, finished
100      format ( a, 1x, (10i6) )
110      format ( a, 1x, (5d12.4) )
         call prinin ( iinpresscor, 5, 'iinpresscor' )
         call prinin1 ( isolut2d, 2, 5, 'isolut2d' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Start part of subroutine seppresscorr1
!         Fill input arrays and constpstants and write initial condition

      call septimstart ( first, eps, isubrlc )
      if ( ierror/=0 ) go to 1000

!     --- While loop time-method

200   if ( itimloop>=1 ) then

!     --- First set tstep and tout

         call septimset ( first, eps )
         if ( ierror/=0 ) go to 1000

!        --- Call time integration subroutine and raise t

         call septimpresscr ( iinpresscor, first )
         if ( ierror/=0 ) go to 1000

!        --- Perform output during a time loop

         call septimoutp ( eps )
         if ( ierror/=0 ) go to 1000
         if ( complrun ) go to 200

      end if
      if ( complrun ) itimloop = 0

      if ( t>=tend-0.00001*tstep ) then

!     --- t = tend   reset all arrays

         finished = .true.

      end if  ! ( t>=tend-0.00001*tstep )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'finished ', finished
         write(irefwr,*) 'End seppresscorr1'

      end if
      call erclmn ( 'seppresscorr1', timebefore )

      end
