      subroutine sepps2interp
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.1  date 30-08-2006 New definition of cpost5
!        version  1.0  date 26-06-2006
!
!   copyright (c) 2006-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpolate solution from one mesh to another
!     in case of postprocessing
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     post_processing
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmodulemain
      use sepmodulecons
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer mesh_in, mesh_out, ivec, indsol, jndsol

!     indsol         Sequence number of vector in isolut2d
!     ivec           Sequence number of array to be processed
!     jndsol         Sequence number of vector in isolut2d
!     mesh_in        Sequence number of input mesh
!     mesh_out       Sequence number of output mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPINTMSH      Interpolate solution from one mesh to another
!     SEPPRINTV      Print solution vector or vector of special structure
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2231   solution array has not been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepps2interp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepps2interp'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      mesh_in = iinputpst(88)
      mesh_out = iinputpst(89)
      ivec = iinputpst(2)
      indsol = imapsol(ivec,mesh_out)
      if ( indsol==0 ) then

!     --- Create new entry in isolut2d

         lastmapsol = lastmapsol+1
         imapsol(ivec,mesh_out) = lastmapsol
         indsol = lastmapsol

      end if
      jndsol = imapsol(ivec,mesh_in)
      if ( jndsol==0 ) then

!     --- Array has not been filled

         call errint ( ivec, 1 )
         call errint ( mesh_in, 2 )
         call errsub ( 2231, 2, 0, 0 )
         go to 1000

      end if

      call sepintmsh ( 0, mesh_in, mesh_out, isolut2d(jndsol), &
                       isolut2d(indsol) )

      if ( debug ) then
         write(irefwr,1) 'mesh_in, mesh_out, indsol, jndsol', &
                          mesh_in, mesh_out, indsol, jndsol
         call sepprintv ( isolut2d(jndsol),'before interpolation' )
         call sepprintv ( isolut2d(indsol), 'after interpolation' )
      end if  ! ( debug )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepps2interp'

      end if  ! ( debug )
      call erclos ( 'sepps2interp' )

      end
