      subroutine sepps2meshpl ( iactmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 30-08-2006 New definition of cpost5
!        version  1.0  date 26-06-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot mesh quantities
!     in case of postprocessing
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     post_processing
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmodulemain
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iactmesh

!     iactmesh      i/o   Actual mesh sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ivec, imesh, indsol, lastmapbef

!     imesh          Actual mesh and problem number
!     indsol         Sequence number of vector in isol
!     ivec           Sequence number array to be processed
!     lastmapbef     Help variable to store the last value of lastmapsol
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     INSP00         Plot mesh
!     INSP01         Compute quantities given by a compute statement
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     671   The vector from which output is required does not exist
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepps2meshpl' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepps2meshpl'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'iinputpst(2)', iinputpst(2)
      if ( iinputpst(2)<=1 ) then

!     --- Standard mesh, i.e. imesh = 1

         imesh = iactmesh

      else

!     --- ivec>1, i.e. Special mesh

         ivec = iinputpst(2)
         imesh = ispec(ivec,3)
         itimst = 1
         indsol = imapsol(ivec,itimst)
         lastmapbef = lastmapsol
         if ( indsol==0 ) then

!        --- Create new entry in isol

            lastmapsol = lastmapsol+1
            imapsol(ivec,itimst) = lastmapsol
            indsol = lastmapsol

         end if
         if ( isol(indsol)==0 ) then

!        --- isol(ivec,itimst) = 0, so array isol has not been filled
!            Check if array must be computed

            call insp01

         end if
         if ( isol(indsol)==0 ) then

!        --- Array has not been filled

            if ( lastmapsol>lastmapbef ) then

!           --- A new array has been tried to create, reset

               lastmapsol = lastmapsol-1
               imapsol(ivec,itimst) = 0

            end if
            call errint ( ivec-1, 1 )
            call errint ( ispec(ivec,1), 2 )
            call errchr ( names(ivec), 1 )
            call errwar ( 671, 2, 0, 1 )
            jtimes = jtimst
            go to 1000

         end if

      end if
      if ( ierror/=0 ) go to 1000

      kmesh => kmesh2d(imesh)
      call sepfillkmesh ( kmesh )
      call insp00

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepps2meshpl'

      end if  ! ( debug )
      call erclos ( 'sepps2meshpl' )

      end subroutine sepps2meshpl
