      subroutine sepreadscelse ( inpsep, istart, iseqif, iarrif )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-03-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the input due to the keyword
!     END_IF
!     with respect to subroutine readsc
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     if_statement
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*), iseqif, iarrif(*)

!     iarrif        i/o   Array of size 10 in which information of the if
!                         statements is stored in the following way:
!                         iarrif(i) = starting address of if part in inpsep
!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         See subroutine READSCWHILE for a description
!                         The only difference is that pos 1 is (code) has value
!                         114
!                         Pos. 2 not used (0)
!                         Pos. 3 for the jump
!     iseqif        i/o   Present level of if loop
!     istart        i/o   Next free position in inpsep
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepreadscelse' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadscelse'
         write(irefwr,1) 'lastitem', lastitem
         write(irefwr,1) 'icode, istart, iseqif', icode, istart, iseqif
         write(irefwr,1) 'iarrif(iseqif)', iarrif(iseqif)
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpsep(istart) = icode+3  ! 114
      inpsep(iarrif(iseqif)) = istart  ! position of else
      iarrif(iseqif) = istart+2        ! store position of return value
      istart = istart+1
      inpsep(istart:istart+1) = 0
      istart = istart+2

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istart, iseqif', istart, iseqif
         write(irefwr,1) 'iarrif(iseqif)', iarrif(iseqif)
         write(irefwr,1) 'inpsep(iarrif(iseqif))', inpsep(iarrif(iseqif))
         call prinin ( inpsep, istart, 'inpsep' )
         write(irefwr,*) 'End sepreadscelse'

      end if  ! ( debug )
      call erclos ( 'sepreadscelse' )

      end subroutine sepreadscelse
