      subroutine sepreadscif ( istart, inpsep, iseqif, iarrif, rinsep, &
                               leninpsep, istrin )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-03-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read the input following the keyword
!     IF
!     with respect to subroutine readsc
!     The input is stored in array inpsep and the real array rinsep
!     Old subroutine readscif
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     if_statement
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istart, inpsep(*), &
              iseqif, iarrif(*), leninpsep, istrin
      double precision rinsep(*)

!     iarrif        i/o   Array of size 10 in which information of the if
!                         statements is stored in the following way:
!                         iarrif(i) = starting address of if part in inpsep
!     inpsep        i/o   The integer input read is stored in array INPSEP
!                         See subroutine READSCWHILE for a description
!                         The only difference is that pos 0 is (code) has value
!                         83
!                         Pos. 1 is used for the length
!                         Pos. 2 for the jump
!                         Pos. 3 to length-2 contains the information
!     iseqif        i/o   Present level of if loop
!     istart        i/o   Next free position in inpsep
!     istrin        i/o   Starting address of information for this subroutine
!                         in RINSEP
!     leninpsep      i    Length of array inpsep
!     rinsep        i/o   Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: istartsav, icodesav
      logical :: booleanfound

!     booleanfound   If true a boolean or boolean expression has been found
!     icodesav       Help parameter to save the value of icode
!     istartsav      Help variable to save the value of istart
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRININLARGE    Print 1d integer vector (large numbers)
!     READBOOLEXPR   Read boolean expression between opening and closing
!                    brackets
!     READWR         Read keyword
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!       IF ( boolean_expr(k) ) THEN
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2916   Too much nested if statements
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!

!     code2   Table with sub keywords corresponding to keyword IF
!             Values:
!             1:  BOOLEAN_EXPR
!             2:  THEN
! ======================================================================
!
      call eropen ( 'sepreadscif' )
      debug = .false.
      icodesav = icode
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepreadscif'
         write(irefwr,1) 'lastitem', lastitem
         write(irefwr,1) 'icode, istart, iseqif', icode, istart, iseqif
         istartsav = istart
  1      format ( a, 1x, (10i6) )
         call prinin ( typeitems, nitems, 'typeitems' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( icode==83 ) then

!     --- if statement

         inpsep(istart) = icode        ! if in inpsep
         iseqif = iseqif+1        ! raise number of if statement

      else

!     --- else_if statement

         if ( debug ) write(irefwr,1) 'iarrif(iseqif)', iarrif(iseqif)
         inpsep(istart) = 115         ! else_if in inpsep
         inpsep(iarrif(iseqif)) = istart  ! position of else if

      end if  ! ( icode==83 )

      if ( iseqif>10 ) then
         call errint ( iseqif, 1 )
         call errint ( 10, 2 )
         call errsub ( 2916, 2, 0, 0 )
      end if

      iarrif(iseqif) = istart+2  ! store position of return value
      istart = istart+1          ! raise istart

!     --- Raise lastitem in case ( is found

      if ( typeitems(lastitem+1)==5 ) lastitem = lastitem+1

!     --- Read next item

      booleanfound = .true.  ! defines if a boolean or boolean_expression is
                             ! present
      if ( typeitems(lastitem+1)==3 ) then

!     --- String found, check if it is BOOLEAN_EXPR

         lastitem = lastitem+1
         call readwr
         if ( nchar>=4 .and. string(1:4)=='BOOL' ) then

!        --- Boolean expression found

            booleanfound = .false.

         end if
         lastitem = lastitem-1

      end if  ! ( typeitems(lastitem)==4 )

      if ( debug ) write(irefwr,*) 'booleanfound', booleanfound
      if ( booleanfound ) then

!     --- New method, interpret boolean expression
!         The opening bracket has already been found
!         information from inpsep(istart)

         call readboolexpr ( inpsep(istart+1), rinsep, leninpsep-3, istrin )
         if ( ierror/=0 ) go to 1000
         length = inpsep(istart+1)
         if ( debug ) write(irefwr,1) 'istart, length', istart, length
         inpsep(istart) = -length   ! Set negative length to distinguish
                               ! with old method

      else

!     --- Old method using boolean_expression(k)
!         Read k

         code(1) = 'BOOLEAN_EXPR'
         code(2) = 'THEN'
         ncode(1) = 4
         ncode(2) = 2
         ncodmx(1) = 12
         ncodmx(2) = 4
         itype(1) = 0
         itype(2) = -1
         tabmin(1) = 1d0
         tabmin(2) = 0d0
         tabmax(1) = 1d3
         tabmax(2) = 0d0
         nmain = 0
         iretur = 0
         call sepreadsetkeyw
         if ( ierror/=0 ) go to 1000
         inpsep(istart) = nint(table(1))    ! store k
         length = 0

      end if  ! ( booleanfound )

      icode = icodesav

      if ( icode==83 ) inpsep(istart+1) = 0  ! return position
      istart = istart + 2 + length

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istart, iseqif', istart, iseqif
         write(irefwr,1) 'iarrif(iseqif)', iarrif(iseqif)
         call prininlarge ( inpsep(istartsav), 2+length, 'inpsep' )
         write(irefwr,*) 'End sepreadscif'

      end if  ! ( debug )
      call erclos ( 'sepreadscif' )

      end subroutine sepreadscif
