      subroutine septextvec ( iseqnr, textloc, lentxt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-11-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute textloc and lentxt as function of iseqnr
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplot
      use sepmodulecons
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iseqnr, lentxt
      character (len=120) :: textloc

!     iseqnr         Sequence number with respect to the storage in array
!                    ISEPIN
!     lentxt         length of text parameter, with trailing blanks deleted
!     textloc        String to store a local text
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2147   Parameter out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septextvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septextvec'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iseqnr<=maxvc1 ) then

!     --- Standard vector

         textloc = names(iseqnr)
         lentxt = index(textloc,'          ')-1

      else if ( iseqnr==maxvc1+1 ) then

!     --- coor

         textloc = 'coor'
         lentxt = 4

      else if ( iseqnr<=maxvc1+ndim+1 ) then

!     --- x_coor ...

         select case(iseqnr-maxvc1-1)
            case(1)
               textloc = 'x_coor'
            case(2)
               textloc = 'y_coor'
            case(3)
               textloc = 'z_coor'
         end select  ! case(iseqnr-maxvc1-1)
         lentxt = 6

      else

!     --- option not available

         call errchr ( 'iseqnr', 1 )
         call errint ( iseqnr, 1 )
         call errint ( 1, 2 )
         call errint ( maxvc1+ndim+1, 3 )
         call errsub ( 2147, 3, 0, 1 )
         go to 1000

      end if  ! ( iseqnr<=maxvc1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septextvec'

      end if  ! ( debug )
      call erclos ( 'septextvec' )

      end subroutine septextvec
