      subroutine to0089text1 ( iinsol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 24-12-2014 New definition clinsl
!        version  1.2  date 17-12-2009 Less output
!        version  1.1  date 30-08-2006 New definition of cpost5
!
!   copyright (c) 2005-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write some text information for subroutine to0089 (solve)
!     depending on output parameters
!     Fill iread for solver
! **********************************************************************
!
!                       KEYWORDS
!
!     solve
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmoduletime
      use sepmodulesolve
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iinsol(*)

!     iinsol         i    Input array defined by the user, see to0089
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nch, nch1, isubr, iseqsl, ictime, iprecfem, ivectr, &
                 ivecreac, iread

!     ictime         Indication if information must be read from standard
!                    input file ( 0 ) or that information read before is
!                    used ( 1 ).  See to0089
!     iprecfem       Indicates if a FEM preconditioning must be applied
!                    in case of a spectral mesh
!                    Possible values:
!                    0: no FEM preconditioning
!                    1: FEM preconditioning
!     iread          Defines how the input must be read for subroutine
!                    solve. See solvelbf
!     iseqsl         Sequence number corresponding to the linear solver
!     isubr          Indicates the type of calling subroutine
!                    Possible values
!                    1:  CREATE
!     ivecreac       Sequence number of the reaction force vector with
!                    respect to array isol (if >0)
!     ivectr         Sequence number of the solution vector with respect to
!                    array isol
!     nch            Number of characters in the name of vector "ivectr"
!     nch1           Number of characters in the name of vector "reac"
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0089text1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0089text1'

      end if
      if ( ierror/=0 ) go to 1000

      isubr = iinsol(1)
      iseqsl = iinsol(4)
      ictime = iinsol(20)
      iprecfem = iinsol(12)
      ivectr = iinsol(6)
      ivecreac = iinsol(10)
      iread = iinsol(2)

      nch = index(names(ivectr),' ')-1
      if ( nch<=0 ) nch = len(names(ivectr))

      if ( isubr==3 ) then

!     --- isubr = 3, call by LINPRB

         iread = iseqsl

      else if ( isubr==2 ) then

!     --- isubr = 2, call by LINSTM

         if ( ictime==0 .or. ictime==2 ) then
            iread = -2
         else
            iread = -1
         end if

      else

!     --- isubr = 1, call by LINSOL

         iread = 0

      end if

      if ( ioutp>=0 .and. nitercom<=1 .and. itimloop<=1 ) then

!     --- Print some output for the first time step or iteration

         if ( iprecfem==0 ) then

!        --- Standard finite element method

            write ( irefwr, 170 ) names(ivectr)(1:nch)
170         format ( ' Compute vector ', a, &
                     ' by solution of linear', &
                     ' system of equations ' )

         else

!        --- Spectral element method

            write ( irefwr, 180 ) names(ivectr)(1:nch)
180         format ( ' Compute start vector ', a, &
                     ' by solution of linear', &
                     ' system of equations ' )
         end if
         if ( ivecreac/=0 ) then

!        --- Print information about reaction force

            nch1 = index(names(ivecreac),' ')-1
            if ( nch1<=0 ) nch1 = len(names(ivecreac))
            write(irefwr,190) names(ivecreac)(1:nch1)
190         format( ' Compute vector ', a, &
                    ' as reaction force' )

         end if  ! ( ivecreac/=0 )

      end if
      ireadinpsol = iread

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End to0089text1'

      end if
      call erclos ( 'to0089text1' )

      end
