      subroutine writprobiinp
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 25-11-2012 Long integers
!        version  1.1  date 20-03-2006 New output format
!        version  1.0  date 21-06-2002
!
!   copyright (c) 2002-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write arrays iinput and rinput corresponding to kprob to file
!     sepcomp.out
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemach
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ihelp(10), lenrinput
      character (len=80) textloc

!     i              Counting variable
!     ihelp          integer work array to store some variables
!     textloc        String to store a local text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Activate arrays iinput and rinput
!     Compute starting addresses of iinput and rinput
!     Write arrays
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'writprobiinp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from writprobiinp'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Write arrays

      if ( debug ) write(irefwr,1) 'iref74', iref74
      lenrinput = 0
      if ( associated(kp(iseqkp)%rinputkp) ) lenrinput = size(rinputkp)
      if ( iref74>0 ) then

!     --- iref74>0, formatted output

         write(iref74,150) size(iinputkp), lenrinput
150      format ( '//Version:  version_number, isubr, leniinput', &
                  ', lenrinput'/ &
                  '5   3', i6/ i6/ &
                  '//Iinput' )
         do i = 1, 4
            ihelp(i) = 0
         end do
         write(iref74,200) (ihelp(i),i=1,4), nprob, iinputkp
200      format(5i8/(10i8))
         if ( lenrinput>0 ) then
            write(iref74,210)
210         format ( '//Rinput' )
            write(iref74,220) rinputkp
220         format ( 5d16.8 )
         end if

      else

!     --- iref74<0, unformatted output

         textloc(1:45) = '//Version:  version_number, isubr, leniinput,'
         textloc(46:80) = ' lenrinput'
         write(-iref74) textloc
         ihelp(1) = 5
         ihelp(2) = 3
         write(-iref74) (ihelp(i),i=1,2), size(iinputkp)
         write(-iref74) lenrinput
         textloc = '//Iinput'
         write(-iref74) textloc
         do i = 1, 4
            ihelp(i) = 0
         end do
         write(-iref74) (ihelp(i),i=1,4), nprob, iinputkp
         if ( lenrinput>0 ) then
            textloc = '//Rinput'
            write(-iref74) textloc
            write(-iref74) rinputkp
         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End writprobiinp'

      end if
      call erclos ( 'writprobiinp' )

      end subroutine writprobiinp
