      subroutine inicur01 ( infcur, ifirst, npoint, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 23-07-1999 Extra parameter ndim
!        version  1.0  date 07-03-1997
!
!   copyright (c) 1997-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute number of user points and position of first user point
!    in infcur for one specific curve
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer infcur(*), ifirst, npoint, ndim

!     ifirst         o    First position of user point in curve
!     infcur         i    Array containing the information of one curve
!     ndim           i    Dimension of the space.
!     npoint         o    Number of user points in curve
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icurve

!     icurve     Type of curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1011   Curves of this type have not yet been implemented in this subroutine
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inicur01' )
      icurve = infcur(1)
      if ( icurve.lt.0 .or. icurve.gt.19 ) then

!     --- Not yet implemented

         call errint ( icurve, 1 )
         call errsub ( 1011, 1, 0, 0 )
         go to 300
      end if
      if ( icurve.eq.0 ) then
         npoint = 0
         ifirst = 0
         go to 300
      end if
      go to ( 110, 120, 130, 140, 110,   130, 170, 110, 110, 200, &
              200, 140, 200, 170, 110,   110, 210, 220, 230 ), icurve

!     --- icurve = 1, 5, 8, 9, 15, 16

110      npoint = 2
         ifirst = 5
         go to 300

!        --- icurve = 2

120      npoint = infcur(3)-1
         ifirst = 5
         go to 300

!        --- icurve = 3, 6

130      npoint = 3
         ifirst = 5
         go to 300

!        --- icurve = 4, 12

140      npoint = infcur(6)
         ifirst = 7
         go to 300

!        --- icurve = 7, 14

170      npoint = 0
         ifirst = 0
         go to 300

!        --- icurve = 10, 11, 13

200      npoint = infcur(4)
         ifirst = 6
         go to 300

!        --- icurve = 17

210      npoint = ndim
         ifirst = 5
         go to 300

!        --- icurve = 18

220      npoint = 3
         ifirst = 6
         go to 300

!        --- icurve = 19

230      npoint = 3
         ifirst = 7
300   call erclos ( 'inicur01' )
      end
