      subroutine msh002 ( iinput, icurvs, rhinp, jnew, curstr, &
                          irelcurves, istartcur, istartrel )
! ======================================================================
!
!        programmer    Guus Segal
!        version 20.0  date 11-12-2014 Extra parameters
!        version 19.0  date 02-09-2010 Remove intarmsh
!        version 18.0  date 12-10-2002 Extra parameter kmeshq
!
!   copyright (c) 1982-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array icurvs
!     Compute length of the work arrays tau and work
!     Compute length of array curves
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinput(*)
      integer, intent(out) :: icurvs(*)
      integer, intent(inout) :: irelcurves(*), istartcur, istartrel
      double precision, intent(in) :: rhinp(*)
      double precision, intent(out) :: curstr(*)
      logical, intent(in) :: jnew

!     curstr        i/o   Contains real information of the curves
!                         See subroutine mshcrtcurv
!     icurvs         o    array containing the number of points in the curves
!                         accumulated
!     iinput         i    standard input array for subroutine mesh
!                         it is supposed that the first five positions have been
!                         deleted
!     irelcurves    i/o   Contains integer information of the curves
!                         See subroutine mshcrtcurv
!     istartcur     i/o   next free position in curstr
!     istartrel     i/o   next free position in irelcurves
!     jnew           i    indication whether the mesh is new (jnew=true) or not
!                         (jnew=false)
!     rhinp          i    Part of array rinput containing information about
!                         coarseness and factors
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isum, iinp, icurve, nelm, npnts, jcurnr

!     icurve         Type of curve
!     iinp           last position used in array iinput
!     isum           Integer variable to compute a sum
!     jcurnr         Curve sequence number
!     nelm           number of elements along a curve
!     npnts          number of elements along a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH002SB       For all double curves with one element on it, double the
!                    number of elements
!     MSHCURNE       Compute number of elements along a curve
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     136   number of elements along curve is incorrect.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh002' )

      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh002'
         write(irefwr,1) 'istartcur, istartrel', istartcur, istartrel
  1      format ( a, 1x, (10i6) )

      end if

      do jcurnr = 1, ncurvs

         icurnr = iseqcurvs(jcurnr)
         iinp = ipointcurv(icurnr)
         icurve = iinput(iinp)
         npnts = 0
         if ( icurve==0 ) go to 250
         ishape = iinput(iinp+1)
         if ( debug ) &
            write(irefwr,1) 'jcurnr, icurnr, ishape, icurve, nelm', &
                             jcurnr, icurnr, ishape, icurve, iinput(2)
         if ( jnew ) then

!        --- jnew = true   new mesh

            call mshcurne ( iinput, icurvs, rhinp, curstr, icurve, &
                            iinp, irelcurves, istartcur, istartrel )

         end if
         nelm = iinput(iinp+2)

         if ( nelm<=0 ) then
            call errint ( nelm, 1 )
            call errint ( icurnr, 2 )
            call errsub ( 136, 2, 0, 0)
         end if
         npnts = ishape*nelm+1

250      icurvs(icurnr) = npnts

      end do

!     --- Accumulate icurvs

      if ( debug ) then

!     --- Debug information

         call prinin ( icurvs, ncurvs, 'number of nodes along curves' )

      end if

      isum = 0
      do icurnr = 1, ncurvs
         isum = isum+icurvs(icurnr)
         icurvs(icurnr) = isum
      end do

!     --- Check if there are double curves with only one element on it.
!         If so make the number of elements equal to 2

      call msh002sb ( iinput, icurvs )

      lencrv = icurvs(ncurvs)

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'lencrv, ncurvs', lencrv, ncurvs
         write(irefwr,1) 'istartcur, istartrel', istartcur, istartrel
         call prinin ( irelcurves, istartrel, 'irelcurves' )
         write(irefwr,*) 'End msh002'

      end if
      call erclos ( 'msh002' )

      end
