      subroutine msh013_7 ( iinput, iinp, istar2, icurve, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-02-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    initialize kmesh part m
!    Special case icurve = 7
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinput(*), iinp, istar2, icurve
      integer, intent(inout) :: iwork(*)

!     icurve         i    Type of curve
!     iinp           i    last position used in array iinput
!     iinput         i    integer input array with respect to the curves
!     istar2         i    Starting address of KMESHM part 2 in KMESHM
!     iwork         i/o   integer work array of length ncurvs to store all the
!                         subcurve numbers of a composite curve
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ncrvs, nsubcv, icount, j, jcurve, kcurve, i, ic, &
                 ifirst, ilast, n, istart

!     i              Counting variable
!     ic             Relative subcurve number
!     icount         Counter
!     ifirst         First curve (surface) number read
!     ilast          Last curve (surface) number read
!     istart         address in array KMESH part m
!     j              Counting variable
!     jcurve         Curve number
!     kcurve         abs(jcurve)
!     n              parameter for mesh generation
!     ncrvs          Number of curves
!     nsubcv         Number of subcurves of a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh013_7' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh013_7'
  1      format ( a, 1x, (10i6) )
      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check if curve is a complete curve

      ncrvs = iinput(iinp+3)
      nsubcv = kelmq
      istart = istar2+kmeshm(4+icurnr)
      icount = 0
      do j = 1, ncrvs
         if ( icurve==7 ) then
            jcurve = iinput(iinp+3+j)
         else
            jcurve = iinput(iinp+6+j)
         end if
         kcurve = abs(jcurve)
         if ( kelmq/=0 ) nsubcv = kmeshq(kcurve+1)-kmeshq(kcurve)
         if ( nsubcv==0 ) then

!        --- No subcurves of subcurve

            icount = icount+1
            iwork(icount) = jcurve

         else

!        --- Subcurve contains subcurves

            if ( jcurve>0 ) then

!           --- Positive direction

               do i = kmeshq(kcurve)+1, kmeshq(kcurve+1)
                  icount = icount+1
                  iwork(icount) = kmeshq(i)
               end do

            else

!           --- Negative direction

               do i = kmeshq(kcurve+1), kmeshq(kcurve)+1,-1
                  icount = icount+1
                  iwork(icount) = -kmeshq(i)
               end do

            end if

         end if

      end do

      if ( icount>ncurvs ) then

!     --- array length too short

         call errint ( icount, 1 )
         call errint ( ncurvs, 1 )
         call errchr ( 'iwork', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

      do ic = 1, icount
         jcurve = iwork(ic)
         kcurve = abs(jcurve)
         if ( debug ) then

!        --- Debug information

            write(irefwr,*) 'ic, jcurve', ic, jcurve

         end if

         if ( jcurve>0 ) then

!        --- jcurve>0

            ifirst = kmeshm(istar2+kmeshm(4+kcurve))
            ilast = kmeshm(istar2+kmeshm(5+kcurve)-1)
            if ( debug ) then

!           --- Debug information

               write(irefwr,1) 'kmeshm(4,5+kcurve)', &
                   kmeshm(4+kcurve), kmeshm(5+kcurve)
               write(irefwr,1) 'ifirst, ilast',ifirst, ilast

            end if

         else

!        --- jcurve<0

            ifirst = kmeshm(istar2+kmeshm(5+kcurve)-1)
            ilast = kmeshm(istar2+kmeshm(4+kcurve))
            if ( debug ) then

!           --- Debug information

               write(irefwr,1) 'kmeshm(4,5+kcurve)', &
                   kmeshm(4+kcurve), kmeshm(5+kcurve)
               write(irefwr,1) 'ifirst, ilast',ifirst, ilast

            end if

         end if
         n = kmeshm(kcurve+5) - kmeshm(kcurve+4)
         if ( icurve==7 ) then

!        --- icurve = 7: curve of curves

            kmeshm(istart) = ifirst
            kmeshm(istart+n-1) = ilast
            istart = istart+n

         else

!        --- iurve = 14, spline curve of curves

            istart = istart+n-1

         end if

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh013_7'

      end if  ! ( debug )
      call erclos ( 'msh013_7' )

      end subroutine msh013_7
