      subroutine msh019 ( iinput, lengpm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  8.1  date 12-03-2015 Extra debug
!        version  8.0  date 02-09-2010 Remove intarmsh
!        version  7.5  date 20-10-2008 Allow isurf = 24
!
!   copyright (c) 1982-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill inner boundaries in kmesh part m
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: lengpm, iinput(*)

!     iinput     i    integer input array corresponding to the surfaces
!     lengpm     i    computed length of kmesh part m
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iinp, nsubcv, ndefsf, j, jcurve, ic, icurve, &
              istar4, istart, notused
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     ic             Relative subcurve number
!     icurve         Type of curve
!     iinp           last position used in array iinput
!     istar4         Starting address of kmeshm part 4 in KMESH part M
!     istart         Starting address of kmeshm part 4 part notused
!     iwork          integer work array
!                    (j):  0  curve is not yet used
!                          1  curve is used once
!                         -1  curve is used more than once
!     j              Counting variable
!     jcurve         Curve number
!     ndefsf         Number of curves in surface
!     notused        Number of curves that are not used in a surface
!     nsubcv         Number of subcurves of a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSH046         Raise iinp as function of icurve
!     MSH049         Compute starting address of specific surface in array
!                    IINPUT
!     PR0008         Print information of curves
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear iwork
!     For all surfaces do
!        Fill iwork with 1 for curves used once and -1 with curves used twice
!     Mark all composite curves as internal and count number of curves not used
!     Fill part 4
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh019' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh019'
         write(irefwr,1) 'ncurvs, nsurfs', ncurvs, nsurfs
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(ncurvs), stat = error )
      if ( error/=0 ) call eralloc ( error, ncurvs, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear iwork

      iwork(1:ncurvs) = 0
      ninner=0
      iinp=iinp3
      nsubcv = 0

!     --- For all surfaces do

      do i = 1, nsurfs

         isurf=iinput(iinp)

         if ( debug ) write(irefwr,1) 'i, isurf', i, isurf
         if ( isurf==24 ) isurf = 7    ! compound surface
         if ( isurf/=0 .and. isurf/=7 .and. isurf/=11 .and. &
              isurf/=12 ) then

!        --- Surface not empty, nor composite surface

            ndefsf=mod(iinput(iinp+2),10000)

            if ( debug ) write(irefwr,1) 'ndefsf', ndefsf
            do j = 1, ndefsf
               jcurve=abs(iinput(iinp+2+j))
               if ( kelmq/=0 ) nsubcv = kmeshq(jcurve+1) - kmeshq(jcurve)

               if ( debug ) &
                  write(irefwr,1) 'j, jcurve, nsubcv', j, jcurve, nsubcv

!              --- Loop over subcurves

               do ic = 1,max(1,nsubcv)
                  if ( nsubcv==0 ) then
                     icurve = jcurve
                  else
                     icurve = abs ( kmeshq ( kmeshq(jcurve)+ic ) )
                  end if

!                 --- Fill iwork with 1 for curves used once and -1
!                     with curves used twice

                  if ( iwork(icurve)==0 ) then

!                 --- iwork(icurve) = 0  curve not yet used

                     iwork(icurve)=1

                  else if ( iwork(icurve)>0 ) then

!                 --- iwork(icurve)>0  curve used before

                     iwork(icurve)=-1
                     ninner=ninner+1

                  end if

               end do

            end do

         end if

         call msh049 ( iinput, iinp, isurf )

      end do

!     --- Mark all composite curves as internal and count number of curves not
!         used

      iinp = iinp1
      notused = 0
      do i = 1, ncurvs

         icurve = iinput(iinp)
         if ( icurve==7 ) then

!        --- composite curve

            iwork(i) = -2
            ninner = ninner + 1

         else if ( iwork(i)==0 ) then

!        --- curve not used

            notused = notused+1

         end if

         call msh046(iinput,iinp,icurve)

      end do

      if ( debug ) write(irefwr,1) 'ninner, notused', ninner, notused

!     --- Fill part 4

      kmeshm(1) = ninner+notused
      kmeshm(4) = ninner
      if ( kmeshm(1)>0 ) then
         istar4 = kmeshm(3)
         istart = istar4+ninner
         do i = 1, ncurvs

            if ( iwork(i)==0 ) then

!           --- Curve is not used

               kmeshm(istart) = i
               istart = istart+1

            else if ( iwork(i)==-1 ) then

!           --- Internal curve

               kmeshm(istar4) = i
               istar4 = istar4+1

            else if ( iwork(i)==-2 ) then

!           --- Composite curve

               kmeshm(istar4) = -i
               istar4 = istar4+1

            end if

         end do

      end if

      lengpm = kmeshm(3)+kmeshm(1)-1

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         call pr0008 ( kmeshm, ncurvs )
         write(irefwr,*) 'End msh019'

      end if  ! ( debug )
      call erclos ( 'msh019' )

      end subroutine msh019
