      subroutine msh028 (  iinput, leng, kmeshnlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.5  date 01-08-2001 new definition kmesh part h
!        version  2.4  date 24-07-1999 Extension for spheres
!        version  2.3  date 07-08-1995 Fill kmeshnlc if isurf=0
!
!   copyright (c) 1982-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     initialize kmeshnlc and compute the length of this array
!     iinput must contain information of the surfaces from position 1
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     surface
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), kmeshnlc(nsurfs,2), leng

!     iinput    i    array iinput containing the user input
!                    with respect to the surfaces (iinp3)
!     kmeshnlc  o    KMESH part n, first part
!     leng      o    Help variable to compute the length of some arrays
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iposi, isurnr, isurf, inpelm, lengi, &
              ielgeg(5), isurnrnew
      logical spherefound

!     ielgeg         array with number of nodes for each element shape
!     inpelm         number of points per element
!     iposi          next free position of iinput
!     isurf          Type of surface with respect to mesh generator
!     isurnr         Surface sequence number
!     isurnrnew      New surface sequence number
!     lengi          local length
!     spherefound    If true a sphere must be created
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH049         Compute starting address of specific surface in array
!                    IINPUT
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ielgeg / 3,6,4,9,7 /
! ======================================================================
!
      call eropen ( 'msh028' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh028'
         write(irefwr,1) 'nsurfs', nsurfs
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      iposi = 1
      leng = 2*nsurfs
      spherefound = .false.
      do isurnr = 1, nsurfs
         isurf = iinput(iposi)
         if ( isurf.eq.20 ) then

!        --- isurf = 20, sphere, do later on

            spherefound = .true.

         else if ( isurf.gt.0 ) then

!        --- isurf > 0

            ishape = iinput(iposi+1)
            inpelm = ielgeg(ishape-2)
            inelem = kmeshh(2,isurnr+1)
            lengi = inpelm*inelem
            kmeshnlc(isurnr,1) = inpelm
            kmeshnlc(isurnr,2) = inelem
            leng = leng+lengi

         else

!        --- isurf = 0

            kmeshnlc(isurnr,1) = 0
            kmeshnlc(isurnr,2) = 0

         end if
         call msh049 ( iinput, iposi, isurf )

      end do
      iposi = 1
      if ( spherefound ) then

!     --- a sphere has been found, perform the copying

         do isurnr = 1, nsurfs
            isurf = iinput(iposi)
            if ( isurf.eq.20 ) then

!           --- isurf = 20, sphere, do later on

               isurnrnew = iinput(iposi+6)

               kmeshnlc(isurnr,1) = kmeshnlc(isurnrnew,1)
               kmeshnlc(isurnr,2) = kmeshnlc(isurnrnew,2)
               leng = leng+kmeshnlc(isurnrnew,1)*kmeshnlc(isurnrnew,2)

            end if
            call msh049 ( iinput, iposi, isurf )

         end do

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'leng', leng
         write(irefwr,*) 'End msh028'

      end if  ! ( debug )
      call erclos ( 'msh028' )
      end subroutine msh028
