      subroutine msh030arc1 ( ndim, p1, p2, x1, x2, xz, dx, c2, &
                              coorusp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-02-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays x2, xz, dx and c2 for a circle
!
! **********************************************************************
!
!                       KEYWORDS
!
!     create_point
!     curve
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ndim, p1, p2
      double precision, intent(in) :: coorusp(ndim,*)
      double precision, intent(inout) :: x2(ndim)
      double precision, intent(out) :: x1(ndim), xz(ndim), dx(ndim), c2

!     c2             o    Helping constant (dx,dx)
!     coorusp        i    array containing on the first nuspnt*ndim positions
!                         coordinates , on position nuspnt+1 the unit edge
!                         length and on the next nuspnt positions the defined
!                         coarseness of the user points
!     dx             o    length of vector (x2-x1)
!     ndim           i    Dimension of the space.
!     p1             i    user point number of first point
!     p2             i    user point number of second point
!     x1             o    coordinates of first point
!     x2            i/o   coordinates of last point
!     xz             o    coordinates of centroid
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: v(ndim), w(ndim), alpha

!     alpha          (w,dx)/c2
!     v              x1+x2
!     w              (x1+x2)/2-x2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh030arc1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh030arc1'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      x1 = x2
      x2 = coorusp(:,p2)
      dx = x2-x1
      v = x1+x2

      c2 = dot_product(dx,dx)
      w = 0.5d0*v-x2
      alpha = dot_product(w,dx)/c2
      xz = x2+alpha*dx

1000  call erclos ( 'msh030arc1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'p1, p2', p1, p2
         write(irefwr,2) 'x1', x1
         write(irefwr,2) 'x2', x2
         write(irefwr,2) 'xz', xz
         write(irefwr,2) 'alpha', alpha
         write(irefwr,*) 'End msh030arc1'

      end if  ! ( debug )

      end subroutine msh030arc1
