      subroutine msh030ellips ( iinput, iinp, np, rinput, nuspnt, &
                                x1, x2, curstr, icurnr, sleng )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 11-12-2014 Remove parameter irstr
!        version  1.0  date 31-07-2012
!
!   copyright (c) 2012-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     determine number of elements to be created along
!     ellips with given coarseness
! **********************************************************************
!
!                       KEYWORDS
!
!     create_point
!     curve
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, np, nuspnt, icurnr
      double precision rinput(2,nuspnt), x1(2), x2(2), curstr(*), sleng

!     curstr         o    in this array some geometrical information concerning
!                         the curves is stored
!                         for each curve 10 positions are available
!                         storage:  pos. 1  length of curve
!                         pos. 2-5: phi, a, b, t0
!     icurnr         i    Curve sequence number
!     iinp           i    starting position of curve input in array iinput
!     iinput         i    array containing the user input concerning the
!                         curve from position iinp
!     np             o    Defines whether the number of points is odd (2) or
!                         even (3)
!     nuspnt         i    Number of user points
!     rinput         i    array containing on the first nuspnt*ndim positions
!                         coordinates , on position nuspnt+1 the unit edge
!                         length and on the next nuspnt positions the defined
!                         coarseness of the user points
!     sleng          o    length of the curve
!     x1             i    coordinates of first point
!     x2             i    coordinates of second point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j3
      double precision xt(2), a, b, t0, phi

!     a              Parameter defining the ellips
!                    (length of first direction)
!     b              Parameter defining the ellips
!                    (length of second direction)
!     j3             Index of center point
!     phi            angle
!     t0             Value of t corresponding to x1
!     xt             coordinates of centroid
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision mshelllength

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHELLLENGTH   Return with approximate length of ellips
!     MSHELLPARMS    Compute defining parameters for ellips
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1783   User point number out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh030ellips' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh030ellips'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      j3 = iinput(iinp+6)
      if ( j3==0 .or. abs(j3)>nuspnt ) then

         call errint ( j3, 1 )
         call errint ( nuspnt, 2 )
         call errint ( icurnr, 3 )
         call errsub ( 1783, 3, 0, 0 )
         go to 1000

      end if
      np = iinput(iinp+7)
      if ( debug ) write(irefwr,1) 'j3', j3
      xt = rinput(:,abs(j3))
      if ( debug ) write(irefwr,2) 'xt, yt', xt

!     --- Next compute essential parameters of ellips part
!         i.e. a, b, t0, phi

      call mshellparms ( x1(1), x1(2), x2(1), x2(2), xt(1), xt(2), a, b, &
                         t0, phi, j3 )
      sleng = mshelllength ( a, b, t0, phi, 100 )  ! length

      curstr(2) = phi
      curstr(3) = a
      curstr(4) = b
      curstr(5) = t0

1000  call erclos ( 'msh030ellips' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh030ellips'

      end if  ! ( debug )

      end
