      subroutine msh030line ( iinput, iinp, np, sleng, x1, x2, ndim, &
                              icurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-07-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     determine number of elements to be created along
!     line with given coarseness
! **********************************************************************
!
!                       KEYWORDS
!
!     create_point
!     curve
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, np, ndim, icurnr
      double precision x1(ndim), x2(ndim), sleng

!     icurnr         i    Curve sequence number
!     iinp           i    starting position of curve input in array iinput
!     iinput         i    array containing the user input concerning the
!                         curve from position iinp
!     ndim           i    Dimension of the space.
!     np             o    Defines whether the number of points is odd (2) or
!                         even (3)
!     sleng          o    length of the curve
!     x1             i    coordinates of first point
!     x2             i    coordinates of second point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision v(ndim)

!     v              x2 - x1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     470   initial and end point of line have the same co-ordinates
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh030line' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh030line'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      np = iinput(iinp+6)
      if ( ndim==1 ) then

!     --- ndim = 1

         sleng = abs(x2(1)-x1(1))
         if ( sleng<=epsmac*abs(x1(1)+x2(1)) ) then
            call errint ( icurnr, 1 )
            call erreal ( sleng, 1 )
            call errsub ( 470, 1, 1, 0 )
         end if

      else

!     --- ndim = 2, 3

         v = x2-x1
         sleng = sqrt(dot_product(v,v))
         v = abs(x1)+abs(x2)
         if ( sleng**2<=epsmac*dot_product(v,v) ) then
            call errint ( icurnr, 1 )
            call erreal ( sleng, 1 )
            call errsub ( 470, 1, 1, 0 )
         end if

      end if  ! ( ndim==1 )

1000  call erclos ( 'msh030line' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh030line'

      end if  ! ( debug )

      end
