      subroutine msh030spline ( rinput, nuspnt, iinput, rincor, nelm, &
                                ndim, icurnr, edge, stotal, iinp, np )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-07-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     determine number of elements to be created along
!     spline with given coarseness
! **********************************************************************
!
!                       KEYWORDS
!
!     create_point
!     curve
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nuspnt, iinput(*), nelm, ndim, icurnr, iinp, np
      double precision rinput(*), rincor(*), edge, stotal

!     edge           i    Unit length defined for coarseness
!     icurnr         i    Curve sequence number
!     iinp           i    starting position of curve input in array iinput
!     iinput         i    array containing the user input concerning the
!                         curve from position iinp
!     ndim           i    Dimension of the space.
!     nelm           i    number of elements along a curve
!     np             o    Defines whether the number of points is odd (2) or
!                         even (3)
!     nuspnt         i    Number of user points
!     rincor         i    Part of array rinput refering to coarseness
!     rinput         i    array containing on the first nuspnt*ndim positions
!                         coordinates , on position nuspnt+1 the unit edge
!                         length and on the next nuspnt positions the defined
!                         coarseness of the user points
!     stotal         o    total length of all segments together
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer number, itype, iline

!     iline          Starting address in iinput
!                    Defines position where the nodes of the spline start
!     itype          Defines type of spline
!     number         number of defining points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHC05         determine number of elements to be created along
!                    polygon-track in case of splines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh030spline' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh030spline'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      number = iinput(iinp+5)
      itype  = iinput(iinp+4)
      if ( itype==4 ) number = number - 2
      np    = iinput(iinp+5+number)
      iline = iinp + 6
      call mshc05 ( rinput, nuspnt, iinput(iline), number, &
                    rincor(2), nelm, ndim, icurnr, edge, stotal )

1000  call erclos ( 'msh030spline' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh030spline'

      end if  ! ( debug )

      end
