      subroutine msh032 ( coarse, ndim, curves, iinput, rinput, tau, n, &
                          nuspnt, sleng, icurnr, ncurvs, jcoars, &
                          iratio, curvefacts )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  6.1  date 23-09-2009 Extension with maxratio
!        version  6.0  date 22-08-2001 Extra parameter curvefacts
!        version  5.5  date 27-04-2000 Improvement of check
!        version  5.4  date 16-06-1999 Layout
!        version  5.3  date 05-02-1994 Saver call to funccv
!
!   copyright (c) 1984-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill array curves , in case of a parameter function
!
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical coarse
      integer iinput(*), ndim, n, nuspnt, icurnr, ncurvs, jcoars, iratio
      double precision curves(ndim,*), rinput(*), tau(*), sleng, &
                       curvefacts(*)

!     coarse         i    indication whether sleng has been computed before
!                         (true) or not (false)
!     curvefacts     o    Array containing the factors t along the curves
!     curves         o    array that must be filled with the coordinates
!                         of the curves
!     icurnr         i    curve number
!     iinput         i    standard input array for subroutine mesh
!                         it is supposed that the information for the curve
!                         is stored from position 1
!     iratio         i    indication of the parameter ratio in the input records
!                         Is only used for the function param
!     jcoars         i    indication whether coarseness is used (1) or not (0)
!     n              i    number of points on curve
!     ncurvs         i    Number of curves in mesh
!     ndim           i    Dimension of the space.
!     nuspnt         i    Number of user points
!     rinput         i    standard input arrays for subroutine mesh
!                         it is supposed that the first five
!                         positions have been deleted
!     sleng          i    length of curve (only when coarse is true)
!     tau            i    array containing the relative arc lengths for the
!                         elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j1, j2, ih, k, itel, i, j
      double precision x1, x2, y1, y2, z1, z2, y, z, t0, x, t1, dt, xp, &
                       yp, zp, t, sprec, tprec, si, s, tp, sp, e1, e2, &
                       e3, tfuncv

!     dt      ?
!     e1      ?
!     e2      ?
!     e3      ?
!     i       Counting variable
!     ih      Help variable to store a constant temporarily
!     itel    ?
!     j       Counting variable
!     j1      ?
!     j2      ?
!     k       Counting variable
!     s
!     si      ?
!     sp      ?
!     sprec   ?
!     t       Parameter along the curve
!     t0      Starting value of t
!     t1      End value of t
!     tfuncv  Help variable to store the actual value of t
!     tp      ?
!     tprec   ?
!     x
!     x1      x-coordinate of first point in curve
!     x2      x-coordinate of last point in curve
!     xp      x-coordinate in plotting units
!     y       y co-ordinate of point
!     y1      y-coordinate of first point in curve
!     y2      y-coordinate of last point in curve
!     yp      y-coordinate in plotting units
!     z       z co-ordinate of point
!     z1      z-coordinate of first point in curve
!     z2      z-coordinate of last point in curve
!     zp      ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERREAL  Put real in error message
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     FUNCCV  User function to define the curve in parameter form
!     INSTOP  Stop the program
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     1720    ?
!     1721    ?
!     1722    ?
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh032' )
      j1 = iinput(1)
      j2 = iinput(2)
      if ( j1<=0 .or. j1>nuspnt ) then
         call errint ( j1, 1 )
         call errint ( nuspnt, 2 )
         call errint ( icurnr, 3 )
         call errsub ( 1722, 3, 0, 0 )
      end if
      if ( j2<=0 .or. j2>nuspnt ) then
         call errint ( j2, 1 )
         call errint ( nuspnt, 2 )
         call errint ( icurnr, 3 )
         call errsub ( 1722, 3, 0, 0 )
      end if
      x1 = rinput(ndim*(j1-1)+1)
      x2 = rinput(ndim*(j2-1)+1)
      y1 = 0d0
      y2 = 0d0
      z1 = 0d0
      z2 = 0d0
      if ( ndim/=1) then
         y1 = rinput(ndim*(j1-1)+2)
         y2 = rinput(ndim*(j2-1)+2)
         if ( ndim==3 ) then
            z1 = rinput(ndim*(j1-1)+3)
            z2 = rinput(ndim*(j2-1)+3)
         end if
      end if
      if ( jcoars==1 ) then
         ih = nuspnt*(ndim+1)+2+ncurvs  !  change due to extra maxratio
      else
         ih = nuspnt*ndim+ncurvs
      end if

!     --- Check if t0 and t1 correspond to initial and end points

      y = 0d0
      z = 0d0
      t0 = rinput(ih+icurnr*2-1)
      tfuncv = t0
      call funccv ( icurnr, tfuncv, x, y, z )
      e1 = 1d-5*(abs(x1)+abs(x))
      e2 = 1d-5*(abs(y1)+abs(y))
      e3 = 1d-5*(abs(z1)+abs(z))
      if ( e1<epsmac .or. abs(x1)<epsmac ) e1=1d-5
      if ( e2<epsmac .or. abs(y1)<epsmac ) e2=1d-5
      if ( e3<epsmac .or. abs(z1)<epsmac ) e3=1d-5
      if ( abs(x1-x)>e1 .or. abs(y1-y)>e2 .or. &
           abs(z1-z)>e3 ) then
         call erreal ( x1, 1 )
         call erreal ( y1, 2 )
         call erreal ( z1, 3 )
         call erreal ( x, 4 )
         call erreal ( y, 5 )
         call erreal ( z, 6 )
         call errint ( icurnr, 1 )
         call errsub ( 1720, 1, 6, 0 )
      end if
      t1 = rinput(ih+icurnr*2)
      tfuncv = t1
      call funccv ( icurnr, tfuncv, x, y, z )
      e1 = 1d-5*(abs(x2)+abs(x))
      e2 = 1d-5*(abs(y2)+abs(y))
      e3 = 1d-5*(abs(z2)+abs(z))
      if ( e1<epsmac .or. abs(x2)<epsmac ) e1=1d-5
      if ( e2<epsmac .or. abs(y2)<epsmac ) e2=1d-5
      if ( e3<epsmac .or. abs(z2)<epsmac ) e3=1d-5
      if ( abs(x2-x)>e1 .or. abs(y2-y)>e2 .or. &
           abs(z2-z)>e3 ) then
         call erreal ( x2, 1 )
         call erreal ( y2, 2 )
         call erreal ( z2, 3 )
         call erreal ( x, 4 )
         call erreal ( y, 5 )
         call erreal ( z, 6 )
         call errint ( icurnr, 1 )
         call errsub ( 1721, 1, 6, 0 )
      end if
      if (  iratio>=0 .and. .not. coarse ) then

!     --- not coarse   compute length of curve

         dt = 0.1d0*(t1-t0)/n
         xp = x1
         yp = y1
         zp = z1
         sleng = 0d0
         do k = 1, 10*n
            t = t0+k*dt
            tfuncv = t
            call funccv ( icurnr, tfuncv, x, y, z )
            goto ( 100, 110, 120 ), ndim
100            sleng = sleng+abs(x-xp)
               goto 140
110            sleng = sleng+sqrt((x-xp)**2+(y-yp)**2)
               goto 130
120            sleng = sleng+sqrt((x-xp)**2+(y-yp)**2+(z-zp)**2)
               zp = z
130            yp = y
140            xp = x
         end do
         goto ( 160, 170, 180 ), ndim
160         sleng = sleng+abs(x2-xp)
            goto 200
170         sleng = sleng+sqrt((x2-xp)**2+(y2-yp)**2)
            goto 200
180         sleng = sleng+sqrt((x2-xp)**2+(y2-yp)**2+(z2-zp)**2)
      end if

!     --- store first and last point in curves

200   curves(1,1) = x1
      curves(1,n) = x2
      if ( ndim>1 ) then
         curves(2,1) = y1
         curves(2,n) = y2
         if ( ndim==3 ) then
            curves(3,1) = z1
            curves(3,n) = z2
         end if
      end if
      curvefacts(1) = t0
      curvefacts(n) = t1

!     --- find rest of co-ordinates

      if ( n==2 ) go to 1000

      if ( iratio>=0 ) then

!     --- iratio>0,  standard situation

         itel = 2
         sprec = 0d0
         tprec = t0
         si = tau(2)*sleng
         xp = x1
         yp = y1
         zp = z1
         do i = 1, n
            dt = (t1-t0)*0.1d0*(tau(itel)-tau(itel-1))
            s = sprec
            t = tprec
            do j = 1, 1000*n
               tp = t
               sp = s
               if ( t1>t0 ) then
                  t = min(t+dt, t1)
               else
                  t = max(t+dt, t1)
               end if
               tfuncv = t
               call funccv ( icurnr, tfuncv, x, y, z )
               goto ( 210, 220, 230 ), ndim
210               s = s+abs(x-xp)
                  goto 250
220               s = s+sqrt((x-xp)**2+(y-yp)**2)
                  goto 250
230               s = s+sqrt((x-xp)**2+(y-yp)**2+(z-zp)**2)
250            xp = x
               yp = y
               zp = z
               if ( s>si ) then

!              --- point found between tp and t ( in fact sp and s )
!                  search for corresponding t value by linear interpolation

                  tprec = t
                  sprec = s
270               t = ((s-si)*tp+(si-sp)*t)/(s-sp)
                  tfuncv = t
                  call funccv ( icurnr, tfuncv, x, y, z )
                  curves(1,itel) = x
                  if ( ndim>1 ) then
                     curves(2,itel) = y
                     if ( ndim==3 ) then
                        curves(3,itel) = z
                     end if
                  end if
                  curvefacts(itel) = t
                  itel = itel+1
                  if ( itel>=n ) go to 1000
                  si = tau(itel)*sleng
                  if ( si<s ) then
                     goto 270
                  else
                     goto 350
                  end if
               end if
            end do
350      end do

      else

!     --- iratio<0, the distribution of t is completely determined by tau

         dt = t1 - t0
         do i = 2, n-1
            t = tau(i) * dt
            call funccv ( icurnr, t, x, y, z )
            curves ( 1, i ) = x
            if ( ndim>1 ) then
               curves ( 2,i ) = y
               if ( ndim==3 ) curves ( 3,i ) = z
            end if
            curvefacts(i) = t
         end do

      end if

1000  call erclos ( 'msh032' )
      if ( ierror/=0 ) call instop
      end
