      subroutine msh034 ( iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  8.3  date 30-11-2014 Change isurf 23 in 99
!        version  8.2  date 08-03-2013 Correction dimension kbndpt
!        version  8.1  date 25-11-2012 Long integers
!
!   copyright (c) 1985-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill array kmeshn for the positions corresponding to curves
!     with the negative values of the corresponding positions in kmeshm
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iinput(*)

!     iinput         i    integer input array for subroutine mesh
!                         The starting address must refer to the surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, istar3, ns, kstart, ndefsf, nsubsf, &
              i, ngen, nd, kgen, icurv, jcurv, numnodes, &
              nodp, lengc, iprec, index, &
              iposi, iuserp, iposiinp, iinp
      integer, allocatable :: iwork(:), kmeshcloc(:), kbndpt(:)

!     i              Counting variable
!     icurv          Curve number with direction
!     iinp           position used in array iinput
!     index          Position in array kbndpt
!     iposi          next position of iinput
!     iposiinp       Starting address in array iinput with respect to the
!                    extra user points (internal points)
!     iprec          Surface number of surface to be copied etc.
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         First position used at a part
!     iuserp         User point sequence number
!     iwork          integer work array of length numnodes
!     jcurv          Absolute value of curve number
!     kbndpt         local array containing the nodal point numbers
!                    of the boundary of the surface
!     kgen           Loop variable
!     kmeshcloc      local array kmeshc corresponding to the surface
!                    number
!     kstart         Start value of parameter
!     lengc          computed length of kmesh part c
!     nd             Number of curves in special surface
!     ndefsf         Number of curves in surface
!     ngen           Number of separate defining curves
!     nodp           Nodal point sequence number
!     ns             Starting address of information of specific surface in
!                    array KMESHS
!     nsubsf         Number of subsurfaces
!     numnodes       Number of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSH034SUB      fill array iwork for the positions corresponding to curves
!                    with the negative values of the corresponding positions in
!                    kmeshm
!     MSH049         Compute starting address of specific surface in array
!                    IINPUT
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh034' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh034'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istart = 2*nsurfs
      istar3 = kmeshm(2)-1
      iposi = 1     ! start position of surface in iinput

      do isurnr = 1, nsurfs

         lengc = 0
         ns = nsurfs + 2 + (isurnr-1)*8
         isurf = kmeshs(ns)
         if ( isurf==24 ) isurf = 7    ! compound surface
         nextrapnts = 0
         nextracrvs = 0

         if ( debug ) write(irefwr,1) 'isurnr, isurf', isurnr, isurf

         if ( isurf==8 .or. isurf==9 .or. isurf==14 ) then

!        --- Surface is copied, use previous type number and reference

            iprec = kmeshs(ns+3)
            ns = nsurfs+2+(iprec-1)*8
            isurf = kmeshs(ns)

         else if ( isurf==22 .or. isurf==99 ) then

!        --- isurf = 22 or 99, there may be internal points

            ndefsf = mod(iinput(iposi+2),10000)
            iinp = iposi+3+ndefsf      ! starting address internal parts
            nextrapnts = iinput(iinp)  ! number of extra points
            nextracrvs = iinput(iinp+1)  ! number of extra curves
            iposiinp = iinp+1  ! address in iinput for userpoints (minus 1)
            if ( debug ) then
               write(irefwr,1) 'ndefsf, iinp, nextrapnts, iposiinp', &
                                ndefsf, iinp, nextrapnts, iposiinp
               write(irefwr,1) 'nextracrvs', nextracrvs
            end if  ! ( debug )

         end if  ! ( isurf==8 .or. isurf==9 .or. isurf==14 )

         kstart = kmeshs(isurnr)
         ndefsf = kmeshs(isurnr+1)-kstart

!        --- Raise iposi

         call msh049 ( iinput, iposi, iinput(iposi) )

         if ( debug ) then

!        --- Debug information

            write(irefwr,*) 'in msh034 '
            write(irefwr,1) 'isurnr, isurf, ndefsf', &
                             isurnr, isurf, ndefsf

         end if

         if ( kelmr/=0 ) then

!        --- kelmr # 0:  there are surfaces of subsurfaces
!                         These surfaces must be skipped

            nsubsf = kmeshr(isurnr+1)-kmeshr(isurnr)
            if ( nsubsf>0 ) then
               istart = istart + kmeshn(isurnr) * kmeshn(isurnr+nsurfs)
               goto 500
            end if

         end if
         numnodes = abs(kmeshh(1,isurnr+1))
         lengc = kmeshn(isurnr)*kmeshn(isurnr+nsurfs)
         if ( kmeshh(4,isurnr+1)==0 .or. isurf==20 ) then
            nbndpt = 0
            go to 500
         end if
         nbndpt = kmeshh(4,isurnr+1)+1

         allocate ( iwork(numnodes), kbndpt(nbndpt), kmeshcloc(lengc), &
                    stat = error )
         if ( error/=0 ) &
            call eralloc ( error, numnodes+nbndpt+lengc, 'iwork' )
         if ( ierror/=0 ) go to 1000

!        --- Correct due to new definition of kbndpt

         nbndpt = nbndpt-1

         kbndpt(1:nbndpt) = kmhsur(isurnr)%kbndptsur(1:nbndpt)

         if ( debug ) then

!        --- Debug information

            call prinin ( kbndpt, nbndpt, 'kbndpt' )

         end if

!        --- clear array iwork

         iwork(1:numnodes) = 0

!        --- put information of curves in iwork

         index = 1
         if ( isurf/=10 ) then
            ngen = 1
            nd = ndefsf
         else
            ngen = 4
         end if
         do kgen = 1, ngen

            if ( ngen>1 ) nd = kmeshs(ns+1+kgen)
            if ( debug ) &
               write(irefwr,1) 'kgen, ngen, nd', kgen, ngen, nd

            do i = 1, nd

               icurv = kmeshs(kstart+i)
               jcurv = abs(icurv)
               if ( debug ) write(irefwr,1) 'i, icurv', i, icurv

               call msh034sub ( iwork, icurv, jcurv, index, kmeshm, &
                                istar3, kbndpt, i, ngen )

            end do
            index = index + 1
            kstart = kstart + nd

         end do
         if ( debug ) &
            write(irefwr,1) 'nextrapnts, nextracrvs', &
                             nextrapnts, nextracrvs

         if ( nextrapnts>0 ) then

!        --- nextrapnts>0, fill reference to kmeshl in iwork
!            To distinguish this number with a reference to curves
!            it is set to -1000000000-user point number

            do i = 1, nextrapnts
               nodp = kbndpt(nbndpt-nextrapnts+i)  ! node number
               iuserp = iinput(iposiinp+i)         ! user point number
               if ( iwork(nodp)==0 ) iwork(nodp) = -1000000000-iuserp
               if ( debug ) &
                  write(irefwr,1) 'i, nodp, iuserp, numnodes', &
                                   i, nodp, iuserp, numnodes
            end do  ! i = 1, nextrapnts

         end if  ! ( nextrapnts>0 )

         if ( nextracrvs>0 ) then

!        --- nextracrvs>0, fill reference to kmeshm in iwork

            iposiinp = iposiinp+nextrapnts
            do i = 1, nextracrvs

               icurv = iinput(iposiinp+i)         ! curve number
               jcurv = abs(icurv)
               if ( debug ) write(irefwr,1) 'i, icurv', i, icurv

               call msh034sub ( iwork, icurv, jcurv, index, kmeshm, &
                                istar3, kbndpt, i, nextracrvs )
               index = index+1  ! raise index after each extra curve

            end do  ! i = 1, nextracrvs

         end if  ! ( nextracrvs>0 )

!        --- fill array kmeshn

         if ( kmhsur(isurnr)%filled ) then

!        --- mmkmeshc>0

            kmeshcloc = kmhsur(isurnr)%kmeshcsur(1:lengc)
            if ( debug ) then

!           --- Debug information

               call prinin ( iwork, numnodes, 'iwork' )
               call prinin ( kmeshcloc, lengc, 'kmeshc' )

            end if
            kmeshn(istart+1:istart+lengc) = iwork(kmeshcloc(1:lengc))

         else

!        --- index = 0

            kmeshn(istart+1:istart+lengc) = 0

         end if

         if ( debug ) then

!        --- Debug information

            call prinin ( kmeshn(istart+1), lengc, 'kmeshn' )

         end if

         deallocate ( iwork, kbndpt, kmeshcloc, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iwork' )
         if ( ierror/=0 ) go to 1000

500      istart = istart+lengc

      end do  ! isurnr = 1, nsurfs

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh034'

      end if  ! ( debug )
      call erclos ( 'msh034' )

      end subroutine msh034
