      subroutine msh036 ( mapar, inodp, jnodp, npoint, isurnr, ivolnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.2  date 14-11-2002 Extra debug statements
!        version  4.1  date 18-01-2002 Debug statements
!        version  4.0  date 29-04-1997 Extra parameters isurnr, ivolnr
!
!   copyright (c) 1985-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill array mapar position inodp with jnodp
!     check if array was already filled, and if so check values
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inodp, jnodp, mapar(*), npoint, isurnr, ivolnr

!     inodp      i    nodal point number in mapar (local number)
!     isurnr     i    Surface sequence number
!     ivolnr     i    Volume sequence number
!     jnodp      i    global nodal point number
!                     must be filled in mapar(inodp)
!     mapar     i/o   contains the new numbers of the nodal points
!     npoint     i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer m, l

!     l
!     m
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     300   common nodal points in surfaces have different co-ordinates.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh036' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh036'

      end if
      m = mapar(inodp)
      if ( debug ) then

!     --- debug = true, print debug information

         write(irefwr,*) 'msh036: inodp, jnodp, m ', inodp, jnodp, m

      end if
      if ( m==0 .or. m<-npoint ) then

!     --- m = 0  mapar was not filled

         mapar(inodp) = jnodp

      else if ( m<0 ) then

!     --- m < 0  point is identical to point -m

         l = mapar(-m)
         if ( l==0) then

!        --- l = 0 corresponding point was not filled

            mapar(-m) = jnodp

         else
            if ( l/=jnodp ) then
               call errint ( l, 1 )
               call errint ( jnodp, 2 )
               call errint ( isurnr, 3 )
               call errint ( ivolnr, 4 )
               call errsub ( 300, 4, 0, 0 )
            end if

         end if
         mapar(inodp) = jnodp

      else

!     --- m > 0  check

         if ( m/=jnodp ) then
            call errint ( m, 1 )
            call errint ( jnodp, 2 )
            call errint ( isurnr, 3 )
            call errint ( ivolnr, 4 )
            call errsub ( 300, 4, 0, 0 )
         end if

      end if

      if ( debug ) write(irefwr,*) 'End msh036'
      call erclos ( 'msh036' )

      end
