      subroutine msh046 ( iinput, iinp, jcurve )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.13 date 10-02-2015 Extension with ccircle
!        version  4.12 date 14-05-2014 Change of error message
!        version  4.11 date 15-08-2009 Extension with ellips
!
!   copyright (c) 1986-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     raise iinp as function of icurve
!     is used to find information in icurvs
!     When this subroutine will be changed, also subroutine msh048 must
!     be changed !!!!!!!!!!!!!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), jcurve, iinp

!    iinp    i/o  at input iinp gives the last position used in iinput
!                 at output the next free position is given
!    iinput   i   standard input array for subroutine mesh
!                 it is supposed that the first five positions have been
!                 deleted
!    jcurve   i   type of curve to be considered
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icurve

!     icurve         type of curve to be considered
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     124   code for generation of curves (icurve) incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh046' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh046'
         write(irefwr,1) 'iinp, jcurve', iinp, jcurve
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icurve = abs(jcurve)

      if ( icurve>=10000 .and. icurve<=10003 ) then

!     --- Special subcurve for free surfaces

         iinp = iinp+5
         go to 1000

      end if

      if ( icurve==0 ) then
         iinp = iinp + 1
         go to 1000
      end if

      iinp=iinp+4

      select case ( icurve )

         case(1, 8, 20)

!        --- icurve = 1, 8

           iinp=iinp+2

         case(2)

!        --- icurve = 2

            iinp=iinp+1+iinput(iinp-2)

         case(5)

!        --- icurve = 5

            iinput(iinp-1) = max(2,iinput(iinp-1))
            iinp=iinp+1+iinput(iinp-1)

         case(3, 9, 15, 22)

!        --- icurve = 3, 9, 15, 22

           iinp=iinp+3

         case(4, 12, 21)

!        --- icurve = 4, 12: splines
!            icurve = 21:    framecurve

           iinp=iinp+2+iinput(iinp+1)
           if ( icurve>4 ) iinp = iinp+1

         case(6, 16, 18, 23)

!        --- icurve = 6, 16, 18, 23

           iinp=iinp+4

         case(7)

!        --- icurve = 7

           iinp=iinp+iinput(iinp-1)

         case(10, 11, 13)

!        --- icurve = 10, 11 or 13

           iinp=iinp+iinput(iinp-1)+1

         case(14)

!        --- icurve = 14

           iinp=iinp+3+iinput(iinp+2)

         case(17)

!        --- icurve = 17

           iinp=iinp+iinput(1)

         case(19)

!        --- icurve = 19

           iinp=iinp+5

         case(24)

!        --- icurve = 24, ccircle

            iinp=iinp+1+iinput(iinp-1)

         case default

!        --- Curve type not recognized

            call errint ( icurve, 1 )
            call errint ( 23, 2 )
            call errsub ( 124, 2, 0, 0 )

      end select  ! case ( icurve )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iinp', iinp
         write(irefwr,*) 'End msh046'

      end if  ! ( debug )
      call erclos ( 'msh046' )

      end
