      subroutine msh052 ( iinput, rinput, iinp, iposc, n, icurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 11-08-2005 Adaptation accuracy
!        version  3.0  date 14-10-2002 Extra parameters kmesh, kmeshq
!        version  2.2  date 20-03-1992 Introduction of negative end point
!
!   copyright (c) 1987-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    compute co-ordinates of curve by translating the co-ordinates of
!    curve iprec
!    The distance is given by the difference between the first user points
!    on both curves
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
!     translate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, iposc, n, icurnr
      double precision rinput(ndim,*)

!     icurnr       i    curve number
!     iinp         i    starting position in array iinput with respect to new
!                       curve
!     iinput       i    new array iinput; The first 5 positions have been
!                       skipped
!     iposc        i    starting position of co-ordinates in curve
!     n            i    number of basis functions for each component
!     rinput      i/o   new array rinput
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical zero, errfnd
      double precision  dx(ndim), x1(ndim), x2(ndim), eps, sum1, sum2
      integer i, j, ipos, iprec, jposc, ic, npointlc, nuser, ipc1, ipc2, &
              k, nparts, nsubcv, isubcv

!     dx             length of vector in x-direction
!     eps            Local accuracy
!     errfnd
!     i              Counting variable
!     ic             Relative subcurve number
!     ipc1
!     ipc2
!     ipos
!     iprec
!     isubcv         Subcurve sequence number with direction
!     j              Counting variable
!     jposc
!     k              Counting variable
!     nparts         number of nodes in a subcurve
!     npointlc         Number of nodal points
!     nsubcv         Number of subcurves of a curve
!     nuser
!     sum1           Parameter to compute a sum
!     sum2           Parameter to compute a sum
!     x1             x-coordinate of first point in curve
!     x2             x-coordinate of last point in curve
!     zero
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     522   Number of user points on curve with TRANSLATE, SIMILAR or ROTATE is
!           incorrect
!     523   Co-ordinates of user point on curve with TRANSLATE or ROTATE not
!           the same as the corresponding user point on the curve to be copied
!    1772   Negative user point number found, not the end point
!    2368   Curve to be copied is not coniguous
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh052' )

!     --- Find starting positions of curve iprec in array curvesmsh (jposc)
!         iprec is array to be copied

      nsubcv = 0
      iprec = iinput(iinp)

      if ( kelmq>0 ) nsubcv = kmeshq(iprec+1)-kmeshq(iprec)

      if ( iprec==1 ) then
         jposc = 0
         n = icurvsmsh(iprec)
      else
         jposc = icurvsmsh(iprec-1)
         n = icurvsmsh(iprec) - jposc
      end if

!     --- Correct n if there are more subcurves due to double points

      if ( nsubcv>1 ) n = n-nsubcv+1

!     --- Characteristics of curve iprec
!         nuser is number of user points
!         ipc1 is first user point in curve iprec

      nuser = curveusrp(iprec+1)-curveusrp(iprec)
      ipc1 = curveusrp(curveusrp(iprec)+1)

!     --- Curve icurnr:  first point: ipc2
!                      npointlc: number of user points
!                      translation is stored in array dx

      ipc2 = iinput(iinp+1)
      npointlc = iinput(iinp-1)
      if ( npointlc>nuser ) then

!     --- Error 522: Number of user points too large

         call errint ( npointlc, 1 )
         call errint ( nuser, 2 )
         call errsub ( 522, 2, 0, 0 )

      end if

      dx(:) = rinput(:,ipc2)-rinput(:,ipc1)

!     --- Copy or check co-ordinates of user points on the curve icurnr

      do ic = 2, min(npointlc, nuser)
         ipc1 = curveusrp(curveusrp(iprec)+ic)
         ipc2 = iinput(iinp+ic)
         if ( ipc2<0 ) then
            if ( ic/=npointlc ) then

!           --- negative point in translate found, which is not the end point

               call errint ( icurnr, 1 )
               call errint ( ipc2, 2 )
               call errsub ( 1772, 2, 0, 0 )

            else

               ipc1 = curveusrp(curveusrp(iprec)+nuser)

            end if
            ipc2 = - ipc2

         end if

!        --- Put co-ordinates of user points in x1 resp. x2

         x1(:) = rinput(:,ipc1)
         x2(:) = rinput(:,ipc2)

!        --- Check if x2 equals zero

         zero = .true.
         do i = 1, ndim
            if ( x2(i)<0d0 .or.x2(i)>0d0 ) then
               zero = .false.
               goto 140
            end if
         end do

140      if ( zero ) then

!        --- zero = true    copy co-ordinates

            rinput(:,ipc2) = x1+dx

         else

!        --- zero = false   check co-ordinates

            errfnd = .false.
            sum1 = 0d0
            sum2 = 0d0
            do j = 1, ndim
               sum1 = sum1+x1(j)**2
               sum2 = sum2+x2(j)**2
            end do ! j = 1, ndim
            eps = sqreps*sqrt(max(sum1,sum2))
            do i = 1, ndim
               if ( abs(x2(i)-(x1(i)+dx(i)))>eps ) errfnd = .true.
            end do
            if ( errfnd ) then

!           --- Error 523:  Incorrect co-ordinates

                call errint ( ic, 1 )
                call errint ( icurnr, 2 )
                do i = 1, 3
                   call erreal ( x2(i), i )
                   call erreal ( x1(i)+dx(i), i+3 )
                end do
                call errsub ( 523, 2, 6, 0 )

            end if

         end if

      end do

!     --- Copy co-ordinates of curve iprec into curve icurnr

      if ( nsubcv<=1 ) then

!     --- there is at most one subcurve

         ipos = iposc-1
         do i = 1, n
            curvesmsh(:,ipos+i) = curvesmsh(:,jposc+i)+dx
         end do

      else

!     --- there are more subcurves, skip double points

         ipos = iposc-1

!        --- For all subcurves do

         do k = 1, nsubcv

!        --- Find subcurve isubcv and number of nodes along isubcv

            isubcv = abs(kmeshq(kmeshq(iprec)+k))
            if ( abs(isubcv)==1 ) then
               nparts = icurvsmsh(isubcv)
            else
               nparts = icurvsmsh(isubcv)-icurvsmsh(isubcv-1)
            end if
            if ( k>1 ) then

!           --- not the first subcurve, check if first points is equal to
!               last one in previous subcurve
!               Store begin coordinates in x1

               errfnd = .false.
               sum1 = 0d0
               sum2 = 0d0
               do j = 1, ndim
                  sum1 = sum1+x1(j)**2
                  sum2 = sum2+x2(j)**2
               end do ! j = 1, ndim
               eps = sqreps*sqrt(max(sum1,sum2))

               x1 = curvesmsh(:,jposc+1)
               do j = 1, ndim
                  if ( abs(x2(j)-x1(j))>eps ) errfnd = .true.
               end do
               if ( errfnd ) then

!              --- The curve is not contiguous

                  call errint ( iprec, 1 )
                  call errint ( icurnr, 2 )
                  call errint ( k, 3 )
                  call errsub ( 2368, 3, 0, 0 )
                  call prinrl ( x1, ndim, 'first node' )
                  call prinrl ( x2, ndim, 'second node' )

               end if

            end if

!           --- Store end coordinates in x2

            x2 = curvesmsh(:,jposc+nparts)

!           --- Copy coordinates

            do i = 1, nparts
               curvesmsh(:,ipos+i) = curvesmsh(:,jposc+i)+dx
            end do

!           --- Raise pointers, take double point into account

            jposc = jposc+nparts
            ipos = ipos+nparts-1

         end do

      end if

      call erclos ( 'msh052' )
      end
