      subroutine msh060 ( iinput, rpin, iseqsurfs, ipointsurf, maxncursur )
! ======================================================================
!
!        programmer    Guus Segal
!        version  9.2  date 10-02-2015 Extra debug
!        version  9.1  date 02-11-2012 Debug statements
!        version  9.0  date 02-09-2010 Remove intarmsh
!
!   copyright (c) 1987-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    fill array KMESHQ, KMESHR, KMESHS and KMESHAB depending on array IINPUT
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduledebug
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iseqsurfs(*), ipointsurf(*), maxncursur
      double precision  rpin(*)

!     iinput         i    Array containing information of the mesh
!                         See Prog. Guide 3.1.2
!     ipointsurf     i    Array containing the relative starting addresses
!                         of all surfaces in the part of iinput referring
!                         to surfaces
!     iseqsurfs      i    Contains new sequence of surface numbers
!     maxncursur     i    Estimate of the length of the last part of iwork
!     rpin          i/o   Array containing the thicknesses and factors
!                         of certain curves, might be changed by msh063
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH061         fill array KMESHQ with curve numbers of the subsequent
!                    curves
!     MSH062         fill array KMESHR with surface numbers of the subsequent
!                    surfaces
!     MSH063         fill array KMESHS with boundaries of surfaces
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh060' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh060'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( associated(kmeshq) ) nullify(kmeshq)
      if ( associated(kmeshr) ) nullify(kmeshr)
      allocate ( kmeshq(100*ncurvs), kmeshr(100*nsurfs), stat = error )
      if ( error/=0 ) call eralloc ( error, 100*ncurvs+100*nsurfs, 'kmeshq' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) &
         write(irefwr,1) 'ncurvs, nsurfs, kelmq, iinp3', &
                          ncurvs, nsurfs, kelmq, iinp3

!     --- First fill array kmesh part q with curves

      call msh061 ( iinput )

      if ( nsurfs>0 ) then

!     --- nsurfs>0, fill kmesh parts r and s
!         First fill kmesh part r with surfaces

         call msh062 ( iinput(iinp3), iseqsurfs, ipointsurf )

!        --- Next fill kmesh part s with boundaries

         call msh063 ( iinput(iinp3), rpin, &
                       iseqsurfs, ipointsurf, maxncursur )
         if ( ierror/=0 ) go to 1000

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh060'

      end if  ! ( debug )
      call erclos ( 'msh060' )

      end subroutine msh060
