      subroutine msh061 ( iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.1  date 20-02-2004 Extra error message
!        version  4.0  date 08-08-1999 Extra parameters
!        version  3.0  date 10-07-1999 Extra parameter iinp1
!
!   copyright (c) 1987-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    fill array kmeshq with curve numbers of the subsequent curves
!    Help subroutine for subroutine MSH060
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iinput(*)

!     iinput         i    Array containing information of the curves
!                         See Prog. Guide 3.1.2
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinp, i, istart, ic, icurve, ncrvs, jcurnr, kcurnr, &
              lengav, nsubsf, k, jstart, lengq

!     i              Counting variable
!     ic             Relative subcurve number
!     icurve         Type of curve
!     iinp           last position used in array iinput
!     istart         First position used at a part
!     jcurnr         Curve number
!     jstart         starting position of curve i in kmeshm
!     k              Counting variable
!     kcurnr         abs(jcurnr)
!     lengav         Available length for array kmeshq
!     lengq          Computed length of kmeshq
!     ncrvs          Number of curves
!     nsubsf         Number of subcurves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     377   The computed length of array KMESH part Q is more than estimated
!     602   curve contains curves with curve numbers larger than itself
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh061' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh061'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Fill array kmeshq

      lengav = 100*ncurvs
      lengq = 0
      istart = ncurvs + 1
      kmeshq(1) = istart

!     --- First fill number of subcurves into kmeshq part1
!         Loop over curves

      do i = 1, ncurvs
         icurnr = iseqcurvs(i)
         iinp = ipointcurv(icurnr)
         icurve = iinput(iinp)
         nsubsf = 0
         if ( icurve==7 ) then

!        --- curve consisting of subcurves

            ncrvs = iinput(iinp+3)

!           --- Loop over subcurves

            do ic = 1, ncrvs
               jcurnr = iinput(iinp+ic+3)
               kcurnr = abs(jcurnr)
               nsubsf = nsubsf+max(1,kmeshq(kcurnr))
            end do
         end if
         kmeshq(icurnr) = nsubsf

      end do
      if ( debug ) call prinin ( kmeshq, ncurvs, 'kmeshq/1' )

!     --- Construct kmeshq part a positions 1 to ncurvs+1

      istart = ncurvs + 1
      do i = 1, ncurvs
         nsubsf = kmeshq(i)
         kmeshq(i) = istart
         istart = istart+nsubsf
      end do
      kmeshq(ncurvs+1) = istart
      if ( debug ) call prinin ( kmeshq, ncurvs+1, 'kmeshq/2' )
      if ( istart>lengav ) then

!     --- Error 377:  Not enough space available

         call eropen ( 'msh061' )
         call errint ( istart, 1 )
         call errint ( lengav, 2 )
         call errsub ( 377, 2, 0, 0 )
         call erclos ( 'msh061' )
         go to 1000

      end if

!     --- Fill second part of kmeshq

      do i = 1, ncurvs
         icurnr = iseqcurvs(i)
         iinp = ipointcurv(icurnr)
         icurve = iinput(iinp)
         istart = kmeshq(icurnr)

         if ( debug ) &
            write(irefwr,*) 'i, icurnr, icurve, istart ', &
                             i, icurnr, icurve, istart

         if ( icurve==7 ) then

!        --- curve consisting of subcurves

            ncrvs = iinput(iinp+3)

!           --- Loop over subcurves

            do ic = 1, ncrvs
               jcurnr = iinput(iinp+ic+3)
               kcurnr = abs(jcurnr)

               if ( kcurnr==0 .or. kcurnr>ncurvs ) then

!              --- Subcurve number out of range

                  call errint ( icurnr, 1 )
                  call errint ( ncurvs, 2 )
                  call errint ( jcurnr, 3 )
                  call errsub ( 602, 3, 0, 0 )
                  go to 1000

               end if

!              --- Check if curve kcurnr consists of subcurves

               jstart = kmeshq(kcurnr)
               nsubsf = kmeshq(kcurnr+1) - jstart
               if ( debug ) &
                  write(irefwr,*) 'ic, jcurnr, jstart, nsubsf ', &
                                   ic, jcurnr, jstart, nsubsf
               if ( nsubsf==0 ) then

!              --- Curve kcurnr does not consist of subcurves

                  istart = istart + 1
                  kmeshq(istart) = jcurnr

               else

!              --- Curve kcurnr contains subcurves

                  if ( jcurnr>0 ) then

!                 --- jcurnr > 0   Copy forwards

                     do k = 1, nsubsf
                        kmeshq(istart+k) = kmeshq(jstart+k)
                     end do

                  else

!                 --- jcurnr < 0   Copy backwards

                     do k = 1, nsubsf
                        kmeshq(istart+k) = -kmeshq(jstart+nsubsf-k+1)
                     end do
                  end if
                  istart = istart + nsubsf
               end if
            end do
         end if

      end do
      lengq = kmeshq(ncurvs+1)
      if ( debug ) write(irefwr,1) 'lengq', lengq

!     --- Put kmeshq into ibuffr

      if ( lengq>ncurvs+1 ) then
         allocate ( km(iseqkm)%kmeshq(lengq) )
         km(iseqkm)%kmeshq = kmeshq(1:lengq)
         kelmq = lengq
         if ( ierror/=0 ) go to 1000
      else
         kelmq = 0
      end if  ! ( nsurfs>0 .or. lengq>ncurvs+1 )

      deallocate ( kmeshq, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'kmeshq' )
      kmeshq => km(iseqkm)%kmeshq

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kmeshq, lengq, 'kmeshq' )
         write(irefwr,*) 'End msh061'

      end if
      call erclos ( 'msh061' )

      end
