      subroutine msh066a ( icurvsl, ncurvsl )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.0    date   10-09-1997
!
!
!   copyright (c) 1997-1997 "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!   Mark curves at free boundary that must not be redistributed
!
! **********************************************************************
!
!                       KEYWORDS
!
!   adapt_boundary
!   change_boundary
!   curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncurvsl, icurvsl(ncurvsl)

!     icurvsl    i    Array of length ncurvs that is filled with all
!                     stand alone curves that must be adapted in the
!                     correct sequence and provided with a sign
!     ncurvsl    i    Number of curves stored in icurvsl
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icurnr
      integer, allocatable :: iworkcv(:)

!     i          Counting variable
!     icurnr     Curve sequence number
!     iworkcv    integer work array of length ncurvs
!                iworkcv(i) = 1 means curve i is in the boundary
!                iworkcv(i) = 0 means curve i is not in the boundary
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERDEALLOC      Produce error message in case deallocate went wrong
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
      allocate ( iworkcv(ncurvs), stat = error )
      if ( error/=0 ) call eralloc ( error, ncurvs, 'iworkcv' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iworkcv and mark curves at free boundary

      iworkcv = 0
      do i = 1, ncurvsl
         iworkcv(abs(icurvsl(i))) = 1
      end do

!     --- loop over all curves

      do icurnr = 1, ncurvs
         if ( iworkcv(icurnr)==1 ) then

!        --- Curve belongs to free boundary, do not redistribute

            kmeshz(icurnr) = 3

         end if

      end do

      deallocate ( iworkcv, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iworkcv' )

1000  end
