      subroutine msh068 ( iref )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.4  date 30-01-2016 New call to msh059
!        version  5.3  date 29-05-2014 Extension of timebefore
!        version  5.2  date 25-11-2012 Long integers
!
!   copyright (c) 1991-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    write information of the mesh to the file indicated by iref
!    If iref>=0, then formatted output is used with at most 80 columns
!    If iref<0 unformatted output
!    This subroutine is an extension to subroutine meshwr, because it
!    writes some extra information using array ihelp
!    This extra information is meant for finite difference grids only
! **********************************************************************
!
!                       KEYWORDS
!
!     finite_volume
!     write_mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iref

!     iref       i    dataset reference number for the output
!                     when iref = 0  the standard output file is assumed
!                     when iref>0, the output is written to the file with
!                     reference number iref with formatted output
!                     When iref<0, the output is written to the file with
!                     reference number -iref with unformatted output
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer krenum(11), ihelp(9)
      double precision :: timebefore(2)

!     ihelp          Input array for subroutine msh059
!     krenum         Renumbering array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERSETTIME      Get the CPU time at start of subroutine
!     MSH069         Find parameters NX and NY
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     SEPMSH059      Write information of the mesh to the file indicated by iref
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh068' )
      if ( itime>0 ) call ersettime ( timebefore )

      call sepfillkmesh(kmesh)

!     --- Activate array "IINPUT"

      ihelp(2) = 0
      ihelp(3) = 0
      if ( ispecial==0 ) then

!        --- Fill array ihelp with "NX" and "NY"

         call msh069 ( iinputkm, ihelp, iinp3 )

      end if
      call sepmsh059 ( iref, ihelp )
      call erclmn ( 'msh068', timebefore )
      end
