      subroutine msh471 ( ifile , coorh , npx   , npy   , kelemh, &
                          nelemh, npelm , funkh  )
! =======================================================================
!
!         programmer   Niek Praagman
!
!         version 2.0    date  19-04-1994 redesign of TESOCE
!         version 1.0    date  25-07-1991
!
!   copyright (c) 1991-1994  "ingenieursbureau sepra"
!   permission to copy or distribute this software or documentation
!   in hard or soft copy granted only by written license obtained
!   from "ingenieursbureau sepra".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system (e.g. , in memory, disk or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     Subroutine MSH471 constructs a triangular or quadrilateral grid
!     if the nodes in a domain are given in a structured way.
!
!     It is assumed that the points are given in NPY rows of NPX points
!     the first row in the x-direction contains point 1 to point NPX, &
!     the second row NPX+1 till 2*NPX and so on.
!
! ***********************************************************************
!
!     INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer ifile, npx, npy, kelemh(*), nelemh, npelm
      double precision funkh(*), coorh(*)

!     coorh   o    array with the coordinates of the nodes
!     funkh   o    array with the function values in the nodes
!     ifile   i    unit number of file with measurements
!     kelemh  o    array containing the nodes of the triangles.
!                  length of this array npelm * nelem.
!     nelemh  o    number of elements created.
!     npelm   i    number of points in element
!                  3 : linear triangle
!                  4 : linear quadrangle
!     npx     i    number of points on lines in x-direction
!     npy     i    number of points on lines in y-direction
!************************************************************************
!
!     COMMON BLOCKS
!
!  **********************************************************************
!
!     LOCAL PARAMETERS
!
      integer i1, i2, i3, i4, ielem, jc, jlm, jr, npoint
      double precision xc, yc, fnk

!     fnk          function value
!     i1           local node number
!     i2           local node number
!     i3           local node number
!     i4           local node number
!     ielem        temporary number of elements
!     jc           column number considered
!     jlm          position in array kelemh to be filled
!     jr           row number considered
!     npoint       number of points with data in structured mesh
!     xc           x-coordinate
!     yc           y-coordinate
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
!     ERCLOS     Deconcatenate name from calling sequence
!     EROPEN     Concatenate name to calling sequence
! ***********************************************************************
!
!     ERROR MESSAGES
!
! ***********************************************************************
!
!     PSEUDO CODE
!
!     Run through all nodes, read and place coordinates
!
!     Run through lines parallel x-direction
!
!     Create triangles or quadrangles in kelemh
! =======================================================================

      call eropen( 'msh471' )

      npoint = 0

!     Run through all rows and all columns and read and place the
!     coordinates

      do 200 jr = 1 , npy

        do 100 jc = 1 , npx

          read( ifile,* ) xc, yc, fnk

          npoint = npoint + 1

          coorh( 2*npoint - 1 ) = xc
          coorh( 2*npoint     ) = yc

          funkh( npoint )  =  fnk

100     continue

200   continue

!     Create all the elements

      ielem = 0

!     Run through all rows

      do 400 jr = 1 , npy - 1

!         Run through all columns

          do  300 jc = 1 , npx - 1

!              Determine four corner nodes

               i1 = ( jr-1 ) * npx + jc
               i2 = i1 + 1
               i3 = i1 + npx
               i4 = i3 + 1

!              Fill elements in kelemh

               if ( npelm.eq.3 ) then

                    jlm = 3 * ielem

                    kelemh( jlm + 1 ) = i1
                    kelemh( jlm + 2 ) = i2
                    kelemh( jlm + 3 ) = i3
                    kelemh( jlm + 4 ) = i2
                    kelemh( jlm + 5 ) = i4
                    kelemh( jlm + 6 ) = i3

                    ielem = ielem + 2

               else if ( npelm.eq.4 ) then

                    jlm = 4 * ielem

                    kelemh( jlm + 1 ) = i1
                    kelemh( jlm + 2 ) = i2
                    kelemh( jlm + 3 ) = i4
                    kelemh( jlm + 4 ) = i3

                    ielem = ielem + 1

               end if

300       continue

400   continue

      nelemh = ielem

      call erclos( 'msh471' )

      end
