      subroutine msh472 ( ifile , coorh , npc   , npr   , kelemh, &
                          nelemh, funkh  )
! =======================================================================
!
!         programmer   Niek Praagman
!         version 1.0    date  19-04-1994
!
!   copyright (c) 1994-1994  "ingenieursbureau sepra"
!   permission to copy or distribute this software or documentation
!   in hard or soft copy granted only by written license obtained
!   from "ingenieursbureau sepra".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system (e.g. , in memory, disk or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     Subroutine MSH472 constructs a triangular or quadrilateral grid
!     if only the nodes in a domain are given in a structured way.
!
!     It is assumed that the points are given in NPR rows of NPC points
!     the first row in the circular direction and situated on the
!     largest circle of a number of concentric circles contains point 1
!     to point NPC, the next circle the points NPC+1 till 2*NPC and so on.
!     Finally as the last point the coordinates of the center of all
!     circles is expected.
!
! ***********************************************************************
!
!     INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer ifile, npc, npr, kelemh(*), nelemh
      double precision funkh(*), coorh(*)

!     coorh   o    array with the coordinates of the nodes
!     funkh   o    array with the function values in the nodes
!     ifile   i    unit number of file with measurements
!     kelemh  o    array containing the nodes of the triangles.
!                  length of this array 3 * nelem.
!     nelemh  o    number of elements created.
!     npc     i    number of points on lines in circular-direction
!     npr     i    number of points on lines in radial-direction
!************************************************************************
!
!     COMMON BLOCKS
!
!  **********************************************************************
!
!     LOCAL PARAMETERS
!
      integer i, i1, i2, i3, i4, ielem, jc, jlm, jr, npoint, &
              jstart, jend
      double precision xc, yc, fnk

!     fnk          function value
!     i            loop variable
!     i1           local node number
!     i2           local node number
!     i3           local node number
!     i4           local node number
!     ielem        temporary number of elements
!     jc           column number considered
!     jend         end position in loop
!     jlm          position in array kelemh to be filled
!     jr           row number considered
!     jstart       start position in loop
!     npoint       number of given data points
!     xc           x-coordinate
!     yc           y-coordinate
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
!     ERCLOS     Deconcatenate name from calling sequence
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
! ***********************************************************************
!
!     PSEUDO CODE
!
!     Run through all circles and finally the center of all concentric
!     circles and read and next place the coordinates of the nodes
!
!     Run through circles and create triangles in kelemh
!
!     Run through last circle and create triangles of the last segments
!
! =======================================================================

      call eropen( 'msh472' )

      npoint = 0

!     Run through all circles and read and place the
!     coordinates

      do 200 jr = 1 , npr

        do 100 jc = 1 , npc

          read( ifile,* ) xc, yc, fnk

          npoint = npoint + 1

          coorh( 2*npoint - 1 ) = xc
          coorh( 2*npoint     ) = yc

          funkh( npoint )  =  fnk

100     continue

200   continue

!     Finally read the last point : the center of all circles

      read( ifile,* ) xc, yc, fnk

      npoint = npoint + 1

      coorh( 2*npoint - 1 ) = xc
      coorh( 2*npoint     ) = yc

      funkh( npoint )  =  fnk

!     Create all the elements

      ielem = 0

!     Run through all circles

      do 400 jr = 1 , npr - 1

!         Run through all "circular points"

          do  300 jc = 1 , npc - 1

!              Determine four corner nodes

               i1 = ( jr-1 ) * npc + jc
               i2 = i1 + 1
               i3 = i1 + npc
               i4 = i3 + 1

!              Fill elements in kelemh

               jlm = 3 * ielem

               kelemh( jlm + 1 ) = i1
               kelemh( jlm + 2 ) = i2
               kelemh( jlm + 3 ) = i3
               kelemh( jlm + 4 ) = i2
               kelemh( jlm + 5 ) = i4
               kelemh( jlm + 6 ) = i3

               ielem = ielem + 2

300       continue

!         Finally find last four points for this circle

          i2 = ( jr-1 ) * npc + 1
          i4 =   i2  +  npc
          i1 =   i4  -  1
          i3 =   i1  +  npc

!         Fill the last elements of this circle in in kelemh

          jlm = 3 * ielem

          kelemh( jlm + 1 ) = i1
          kelemh( jlm + 2 ) = i2
          kelemh( jlm + 3 ) = i3
          kelemh( jlm + 4 ) = i2
          kelemh( jlm + 5 ) = i4
          kelemh( jlm + 6 ) = i3

          ielem = ielem + 2

400   continue

!     Finally : the last parts of all segments

      jstart = ( npr - 1 ) * npc + 1
      jend   =   npr       * npc

      do 500 i = jstart, jend-1

          kelemh( 3*ielem + 1 ) = i
          kelemh( 3*ielem + 2 ) = i + 1
          kelemh( 3*ielem + 3 ) = npoint

          ielem = ielem + 1

500   continue

      kelemh( 3*ielem+1 ) = jend
      kelemh( 3*ielem+2 ) = jstart
      kelemh( 3*ielem+3 ) = npoint

      nelemh = ielem + 1

      call erclos( 'msh472' )

      end
