      subroutine msh483 ( kelem, inode, niedge, nisurf )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.2  date 25-02-2004 Correction reordering
!        version  1.1  date 27-10-1995 no ordering change if ndim=2
!        version  1.0  date 05-04-1995
!
!   copyright (c) 1995-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Refill kmesh part c with a new spectral quadrilateral element
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     spectral
!     quadrilateral
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kelem(*), inode(*), niedge, nisurf

!     inode    i    help array with all the important nodes for the
!                   new element
!     kelem    o    array with ordered nodes of new element
!     niedge   i    number of internal nodes on line part
!     nisurf   i    number of internal nodes on surface part
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jpos, k, kk, kl, line, newplm, npt

!     jpos      pointer in element array
!     k         loop variable
!     kk        loop variable
!     kl        loop variable
!     line      help variable
!     newplm    number of nodes in new element
!     npt       variable to store temporarily the internal node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2623   The option surface elements without internal points has not yet been
!           implemented
!    2624   The option spectral elements of different shapes has not yet been
!           implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check whether situation is in accordance with what can be handled
!     Create new spectral quadrilateral elements
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'msh483' )

      newplm = 4 + 4 * niedge + nisurf
      line   = 2 + niedge
      if ( nisurf.eq.0 ) then

!     --- error: nisurf = 0

         call errsub ( 2623, 0, 0, 0 )
         go to 1000

      else if ( nisurf.ne.niedge * niedge ) then

!     --- error: nisurf#niedge * niedge

         call errsub ( 2624, 0, 0, 0 )
         go to 1000

      else

!     --- Place the four corners

         kelem(                 1 )  = inode(1)
         kelem( line              )  = inode(2)
         kelem( newplm - line + 1 )  = inode(4)
         kelem( newplm            )  = inode(3)

!        --- Place the line nodes

         do k = 1 , niedge

!        --- Side inode(1) - inode(2)

          if ( inode(1).lt.inode(2) ) then
             kelem( 1 + k ) = inode(5) + k - 1
          else
             kelem( 1 + k ) = inode(5) + niedge - k
          end if

!         --- Side inode(4) - inode(3)

          if ( inode(4).lt.inode(3) ) then
             kelem( newplm - line + 1 + k ) = inode(7) + k - 1
          else
             kelem( newplm - line + 1 + k ) = inode(7) + niedge - k
          end if

!         --- Side inode(1) - inode(4)

          if ( inode(1).lt.inode(4) ) then
             kelem( k * line + 1 ) = inode(8) + k - 1
          else
             kelem( k * line + 1 ) = inode(8) + niedge - k
          end if

!         --- Side inode(2) - inode(3)

          if ( inode(2).lt.inode(3) ) then
             kelem( ( k+1 ) * line ) = inode(6) + k - 1
          else
             kelem( ( k+1 ) * line ) = inode(6) + niedge - k
          end if
         end do

!        --- Finally place the internal node numbers

         jpos = 2 + niedge
         npt  = inode(9)
         do kl = 1 , niedge
            jpos = jpos + 2
            do kk = 1 , niedge
               kelem( jpos ) = npt
               npt           = npt  + 1
               jpos          = jpos + 1
            end do
         end do

      end if

1000  call erclos( 'msh483' )

      end
