      subroutine msh491( inode, jnode )
! =======================================================================
!
!
!         programmer   niek praagman
!         version 1.0  date  06-04-1995
!
!
!
!   copyright (c) 1995 - 1995  "ingenieursbureau sepra"
!   permission to copy or distribute this software or documentation
!   in hard or soft copy granted only by written license obtained
!   from "ingenieursbureau sepra".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system (e.g. , in memory, disk or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     Determine the correct ordering of the internal nodal points of a
!     spectral surface. (Row by row, start in the node with smallest
!     node number, and in the direction of the neighbour with the
!     smallest node number)
!
! ***********************************************************************
!
!     KEYWORDS
!
!     mesh_generation
!     quadrilateral
!     spectral
!     ordering
!
! ***********************************************************************
!
!     INPUT / OUTPUT PARAMETERS
!
      implicit none

      integer inode(9), jnode(9)

!     inode  i   node array quadrilateral
!     jnode  o   ordered node array for quadrilateral
!
! ***********************************************************************
!
!     COMMON BLOCKS
!
! ***********************************************************************
!
!     LOCAL PARAMETERS

      integer imn

!     imn     node number of corner where numbering of internal nodes starts
!
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
! ***********************************************************************
!
!     METHOD
!
!     Determine first the smallest node number
!
!     Determine the neighbour with smallest node number
!
!     Arrange the nodes of the corners and of the lines (edges)
!
! =======================================================================

!     Determine which situation and fill jnode accordingly

      imn = min( inode(1), inode(2), inode(3), inode(4) )

      if ( imn.eq.inode(1) ) then

!        To the right or to the left

         if ( inode(2).lt.inode(4) ) then

              jnode(1) = inode(1)
              jnode(2) = inode(2)
              jnode(3) = inode(3)
              jnode(4) = inode(4)
              jnode(5) = inode(5)
              jnode(6) = inode(6)
              jnode(7) = inode(7)
              jnode(8) = inode(8)
              jnode(9) = inode(9)

         else

              jnode(1) = inode(1)
              jnode(2) = inode(4)
              jnode(3) = inode(3)
              jnode(4) = inode(2)
              jnode(5) = inode(8)
              jnode(6) = inode(7)
              jnode(7) = inode(6)
              jnode(8) = inode(5)
              jnode(9) = inode(9)

         end if

      else if ( imn.eq.inode(2) ) then

!        To the right or to the left

         if ( inode(3).lt.inode(1) ) then

              jnode(1) = inode(2)
              jnode(2) = inode(3)
              jnode(3) = inode(4)
              jnode(4) = inode(1)
              jnode(5) = inode(6)
              jnode(6) = inode(7)
              jnode(7) = inode(8)
              jnode(8) = inode(5)
              jnode(9) = inode(9)

         else

              jnode(1) = inode(2)
              jnode(2) = inode(1)
              jnode(3) = inode(4)
              jnode(4) = inode(3)
              jnode(5) = inode(5)
              jnode(6) = inode(8)
              jnode(7) = inode(7)
              jnode(8) = inode(6)
              jnode(9) = inode(9)

         end if

      else if ( imn.eq.inode(3) ) then

!        To the right or to the left

         if ( inode(4).lt.inode(2) ) then

              jnode(1) = inode(3)
              jnode(2) = inode(4)
              jnode(3) = inode(1)
              jnode(4) = inode(2)
              jnode(5) = inode(7)
              jnode(6) = inode(8)
              jnode(7) = inode(5)
              jnode(8) = inode(6)
              jnode(9) = inode(9)

         else

              jnode(1) = inode(3)
              jnode(2) = inode(2)
              jnode(3) = inode(1)
              jnode(4) = inode(4)
              jnode(5) = inode(6)
              jnode(6) = inode(5)
              jnode(7) = inode(8)
              jnode(8) = inode(7)
              jnode(9) = inode(9)

         end if

      else if ( imn.eq.inode(4) ) then

!        To the right or to the left

         if ( inode(1).lt.inode(3) ) then

              jnode(1) = inode(4)
              jnode(2) = inode(1)
              jnode(3) = inode(2)
              jnode(4) = inode(3)
              jnode(5) = inode(8)
              jnode(6) = inode(5)
              jnode(7) = inode(6)
              jnode(8) = inode(7)
              jnode(9) = inode(9)

         else

              jnode(1) = inode(4)
              jnode(2) = inode(3)
              jnode(3) = inode(2)
              jnode(4) = inode(1)
              jnode(5) = inode(7)
              jnode(6) = inode(6)
              jnode(7) = inode(5)
              jnode(8) = inode(8)
              jnode(9) = inode(9)

         end if

      end if

      end
