      subroutine msh493( kelem, i, niedge, nisurf )
! ======================================================================
!
!         programmer   Niek Praagman
!         version 1.1  date  26-05-1997 Keep counter-clockwise direction
!         version 1.0  date  10-04-1995
!
!
!
!   copyright (c) 1995 - 1997  "Ingenieursbureau sepra"
!   permission to copy or distribute this software or documentation
!   in hard or soft copy granted only by written license obtained
!   from "ingenieursbureau sepra".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system (e.g. , in memory, disk or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     Refill kmesh part c with a new spectral triangular element
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     spectral
!     quadrilateral
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer kelem(*), i(*), niedge, nisurf

!     i        i    helparray with all the important nodes for the
!                   new element
!     kelem    o    array with ordered nodes of new element
!     niedge   i    number of internal nodes on line part
!     nisurf   i    number of internal nodes on surface part
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, newplm

!     j         loop variable
!     newplm    number of nodes in new element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS     Deconcatenate subroutine name from calling string
!     EROPEN     Concatenate subroutine name to calling string
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check whether situation is in accordance with what can be handled
!
!     Determine the ordering of the triangular element
!
!     Create new spectral triangular elements
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'msh493' )

      newplm = 3 + 3 * niedge + nisurf

!     --- Place the three nodes on the edges

      if ( niedge.eq.1 ) then

!     --- Only one point between, standaard sequence

         do j = 1, 6
            kelem(j) = i(j)
         end do

      else

!     --- More points between, check sequence

         kelem(1) = i(1)

         if ( i(3).gt.i(1) ) then

!        --- From low to high

            do j = 1, niedge
               kelem(1+j) = i(2)-1+j
            end do

         else

!        --- From low to high

            do j = 1, niedge
               kelem(1+j) = i(2)-j+niedge
            end do

         end if

         kelem(2+niedge) = i(3)

         if ( i(5).gt.i(3) ) then

!        --- From low to high

            do j = 1, niedge
               kelem(2+niedge+j) = i(4)-1+j
            end do

         else

!        --- From low to high

            do j = 1, niedge
               kelem(2+niedge+j) = i(4)-j+niedge
            end do

         end if

         kelem(3+2*niedge) = i(5)

         if ( i(1).gt.i(5) ) then

!        --- From low to high

            do j = 1, niedge
               kelem(3+2*niedge+j) = i(6)-1+j
            end do

         else

!        --- From low to high

            do j = 1, niedge
               kelem(3+2*niedge+j) = i(6)-j+niedge
            end do

         end if

      end if

      if ( nisurf.gt.0 ) then

!     --- Place the internal points

         do j = 1 , nisurf
            kelem(newplm-nisurf+j) = i(7)-1+j
         end do

      end if

      call erclos( 'msh493' )

      end
