      subroutine mshc02 ( solve, inum, ndim, arc, amat, rhs1, rhs2, &
                          rhs3, iinput, coor, alpha, icurnr )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.0  date 02-02-1997 Extra parameter icurnr
!        version  2.1  date 16-04-1996 Update by Sander de Snoo
!        version  2.0  date 17-11-1995 Extra parameter alpha
!        version  1.1  date 04-02-1994 Check on double points
!        version  1.0  date 30-06-1989
!
!   copyright (c) 1989-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Determine spline through points given in array IINPUT
!
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision solve(*),arc(*),amat(*),rhs1(*),rhs2(*),rhs3(*), &
                       coor(*), alpha
      integer inum, ndim, iinput(*), icurnr

!     alpha   i    spline parameter
!     amat    o    matrix array
!     arc     o    arclength array
!     coor   i/o   array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     icurnr  i    Curve sequence number
!     iinput  i    integer input array
!     inum    i    number of basis points for spline-curve
!     ndim    i    Dimension of the space.
!     rhs1    o    right hand side array
!     rhs2    o    right hand side array
!     rhs3    o    right hand side array
!     solve   o    solution array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision dx,dy,dz,x,y,z,xe,ye,ze,hs,hi1,hi,c1,c2,cf, &
                       x1,x2,x3,y1,y2,y3,z1,z2,z3,cx,cy,cz,bb,d1, &
                       afax,afay,afaz,afbx,afby,afbz
      integer i,ipoint,itip, jh

!     afax    derivative in begin point
!     afay    derivative in begin point
!     afaz    derivative in begin point
!     afbx    derivative in end point
!     afby    derivative in end point
!     afbz    derivative in end point
!     bb      local bbar value
!     c1      coefficient
!     c2      coefficient
!     cf      coefficient
!     cx      help-coefficient
!     cy      help-coefficient
!     cz      help-coefficient
!     d1      array of length modecl x modecl contaning the values of the
!             derivatives of the basisfunctions in the x-direction
!     dx      distance
!     dy      distance
!     dz      distance
!     hi      local arclength
!     hi1     local arclength
!     hs      steplength
!     i       Counting variable
!     ipoint  nodal point number
!     itip    indicator which type of spline has to be created
!     jh      corresponds to h,  see jsigma
!     x       array containing the coordinates of the integration points
!     x1      local x-coordinates
!     x2      local x-coordinates
!     x3      local x-coordinates
!     xe      local x-coordinates
!     y       local y-coordinates
!     y1      local y-coordinates
!     y2      local y-coordinates
!     y3      local y-coordinates
!     ye      local y-coordinates
!     z       z co-ordinate of point
!     z1      local z-coordinates
!     z2      local z-coordinates
!     z3      local z-coordinates
!     ze      local z-coordinates
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1484   Distance between two points at spline is zero
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Determine the system of equations, in all internal points and next
!    the extra equations in the boundary points
!    Solve system by straightforward Gaussian elimination of the three- &
!    diagonal system of equations and find the coefficients of the Spline
!    on each subininterval.
!    See for the complete method the technical description.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Determine type of Spline

      itip = mod(iinput(1),10)

      if ( itip.eq.4 ) then

!     --- Last two points give derivative value in begin and endpoint

           inum = inum - 2

      end if

!     --- Place coordinates in array solve

      do 100 i = 1, inum

          ipoint = iinput(i+2) - 1

          solve(        i ) = coor( ndim * ipoint + 1 )
          solve( 4*inum+i ) = coor( ndim * ipoint + 2 )
          if ( ndim.eq.3 ) solve( 8*inum+i ) = coor(ndim*ipoint+3)

100   continue

      hs = 1000d0

      do 200 i = 1, inum - 1

          x = solve(            i )
          y = solve( 4 * inum + i )
          if ( ndim.eq.3 ) z = solve( 8 * inum + i )

          xe = solve(            i + 1 )
          ye = solve( 4 * inum + i + 1 )
          if ( ndim.eq.3 ) ze = solve( 8 * inum + i + 1 )

          dx = x - xe
          dy = y - ye
          if ( ndim.eq.3 ) then
             dz = z - ze
          else
             dz = 0d0
          end if

          arc(i) = sqrt( dx*dx + dy*dy + dz*dz )

          if ( arc(i).lt.hs ) hs = arc(i)
          if ( arc(i).le.0d0 ) then

!         --- Error arc(i) = 0

             call eropen ( 'mshc02' )
             call errint ( i, 1 )
             call errint ( i-1, 2 )
             call errint ( icurnr, 3 )
             call errsub ( 1484, 3, 0, 0 )
             call erclos ( 'mshc02' )

          end if

200   continue
      if ( ierror.ne.0 ) go to 1000

      hs = 1d-3*hs**alpha

      if ( itip.eq.4 ) then

!     --- Determine derivatives

         ipoint = iinput( 3 + inum ) - 1
         afax   = coor( ndim * ipoint + 1 )
         afay   = coor( ndim * ipoint + 2 )
         if ( ndim.eq.3 ) afaz = coor( ndim * ipoint + 3 )

         ipoint = iinput( 4 + inum ) - 1
         afbx   = coor( ndim * ipoint + 1 )
         afby   = coor( ndim * ipoint + 2 )
         if ( ndim.eq.3 ) afbz = coor( ndim * ipoint + 3 )

      end if

!     --- Determine system of equations

      if ( itip.eq.1 ) then

         amat( 1 ) = 0d0
         amat( 2 ) = 1d0
         amat( 3 ) = 0d0

         rhs1( 1 ) = 0d0
         rhs2( 1 ) = 0d0
         if (ndim.eq.3) rhs3( 1 ) = 0d0

         amat(3*inum-2) = 0d0
         amat(3*inum-1) = 1d0
         amat(3*inum  ) = 0d0

         rhs1(  inum  ) = 0d0
         rhs2(  inum  ) = 0d0
         if (ndim.eq.3) rhs3(  inum  ) = 0d0

      else if ( itip.eq.2 ) then

         amat( 1 ) =  0d0
         amat( 2 ) =  1d0
         amat( 3 ) = -1d0

         rhs1( 1 ) =  0d0
         rhs2( 1 ) =  0d0
         if (ndim.eq.3) rhs3( 1 ) =  0d0

         amat(3*inum-2) =  -1d0
         amat(3*inum-1) =   1d0
         amat(3*inum  ) =   0d0

         rhs1(  inum  ) =   0d0
         rhs2(  inum  ) =   0d0
         if (ndim.eq.3) rhs3(  inum  ) =   0d0

      else if ( itip.eq.4 ) then

         hi      = arc(1)**alpha

         amat(1) =   0d0
         amat(2) = ( 2d0*hi*hi + hs*hs ) / ( 3d0*hi )
         amat(3) = (   hi*hi - hs*hs ) / ( 3d0*hi )

         jh      =   0
         rhs1(1) = ( solve(2+jh)-solve(1+jh) ) / hi - afax
         jh      =   4 * inum
         rhs2(1) = ( solve(2+jh)-solve(1+jh) ) / hi - afay
         jh      =   8 * inum
         if (ndim.eq.3) rhs3(1) = ( solve(2+jh)-solve(1+jh) ) / hi - afaz

         hi      = arc( inum-1 )**alpha

         amat( 3*inum-2 ) = (   hi*hi-hs*hs ) / ( 3d0*hi )
         amat( 3*inum-1 ) = ( 2d0*hi*hi+hs*hs ) / ( 3d0*hi )
         amat( 3*inum   ) =   0d0

         jh           =      inum
         rhs1( inum ) = ( solve(jh-1)-solve(jh) ) / hi + afbx
         jh           =  5 * inum
         rhs2( inum ) = ( solve(jh-1)-solve(jh) ) / hi + afby
         jh           =  9 * inum
         if (ndim.eq.3) rhs3( inum ) = ( solve(jh-1)-solve(jh) ) / hi + afbz

      end if

      do 300 i = 2 , inum - 1

         hi1 = arc(i-1)**alpha
         hi  = arc(i  )**alpha

         c1 = ( hi1*hi1 - hs*hs ) / ( 3d0*hi1 )
         c2 = ( hi *hi  - hs*hs ) / ( 3d0*hi  )
         cf =   hi1 + hi - c1 - c2

         amat(3*i-2) = c1
         amat(3*i-1) = cf
         amat(3*i  ) = c2

         x1  =  solve( i - 1 )
         x2  =  solve( i     )
         x3  =  solve( i + 1 )

         rhs1(i) = (x3-x2)/hi - (x2-x1)/hi1

         y1  =  solve( 4*inum + i - 1 )
         y2  =  solve( 4*inum + i     )
         y3  =  solve( 4*inum + i + 1 )

         rhs2(i) = (y3-y2)/hi - (y2-y1)/hi1

         if ( ndim.eq.3 ) then

            z1  =  solve( 8*inum + i - 1 )
            z2  =  solve( 8*inum + i     )
            z3  =  solve( 8*inum + i + 1 )

            rhs3(i) = (z3-z2)/hi - (z2-z1)/hi1

         end if

300   continue

!     --- Solve sytem of equations in two sweeps

      do 400 i = 2 , inum

         cf = amat(3*i-2) / amat(3*i-4)

         amat(3*i-2) = 0d0
         amat(3*i-1) = amat(3*i-1)-cf*amat(3*i-3)

         rhs1(i)     = rhs1(i) - cf*rhs1(i-1)
         rhs2(i)     = rhs2(i) - cf*rhs2(i-1)
         if(ndim.eq.3) rhs3(i) = rhs3(i) - cf*rhs3(i-1)

400   continue

      cx = 0d0
      cy = 0d0
      cz = 0d0

      do 500 i = inum , 1 , -1

         cx = ( rhs1(i) - amat(3*i)*cx ) / amat(3*i-1)
         cy = ( rhs2(i) - amat(3*i)*cy ) / amat(3*i-1)
         if ( ndim.eq.3 ) cz = ( rhs3(i) - amat(3*i)*cz ) / amat(3*i-1)

         solve( 2*inum+i) = cx
         solve( 6*inum+i) = cy
         if ( ndim.eq.3 ) solve(10*inum+i) = cz

500   continue

!     --- Compute other coefficients

      do 600 i = 2 , inum

         hi = arc(i-1)**alpha
         solve( 3*inum+i-1)=(solve(2*inum+i)-solve(2*inum+i-1))/(3d0*hi)
         solve( 7*inum+i-1)=(solve(6*inum+i)-solve(6*inum+i-1))/(3d0*hi)
         if ( ndim.eq.3 ) &
         solve(11*inum+i-1)=(solve(10*inum+i)-solve(10*inum+i-1))/(3d0*hi)

600   continue

!     --- Determine bbar and finally the b values

      do 700 i = 2 , inum

         hi = arc(i-1)**alpha

         c1 = solve(2*inum+i-1)
         d1 = solve(3*inum+i-1)
         dx = solve(i)-solve(i-1)

         bb = dx / hi - c1*(hi-hs) - d1*(hi*hi-hs*hs)

         solve(  inum+i-1) = bb - hs*c1 - hs*hs*d1

         c1 = solve(6*inum+i-1)
         d1 = solve(7*inum+i-1)
         dy = solve(4*inum+i)-solve(4*inum+i-1)

         bb = dy / hi - c1*(hi-hs) - d1*(hi*hi-hs*hs)

         solve(5*inum+i-1) = bb - hs*c1 - hs*hs*d1

         if ( ndim.eq.3 ) then

            c1 = solve(10*inum+i-1)
            d1 = solve(11*inum+i-1)
            dz = solve( 8*inum+i)-solve( 8*inum+i-1)

            bb = dz / hi - c1*(hi-hs) - d1*(hi*hi-hs*hs)

            solve(9*inum+i-1) = bb - hs*c1 - hs*hs*d1

         end if

700   continue

1000  continue

      end
