      subroutine mshc03 ( solve, ntal, ndim, arc, tau, curves, n, &
                          alpha )
! ======================================================================
!
!        programmer     Niek Praagman
!        version 2.1    date   16-04-1996 Sander de Snoo: update
!        version 2.0    date   17-11-1995 Sander de Snoo: complete revision
!        version 1.0    date   26-06-1989
!
!
!   copyright (c) 1989-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     Determine coordinates of nodal points on spline curve
!
! **********************************************************************
!
!                       KEYWORDS
!
!     spline
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      double precision solve(*), arc(*), tau(*), curves(*), alpha
      integer ntal, ndim, n

!     alpha   i    spline parameter
!     arc     o    arc length array
!     curves  o    array containing the coordinates of the curves
!     n       i    number of points to be created
!     ndim    i    Dimension of the space.
!     ntal    i    number of defining points
!     solve   i    array containing for each spline interval the spline
!                  coefficients
!     tau     i    array containing the relative distribution of the
!                  points to be created
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision total, tbegin, tend, t, st, cz, dz, &
              x, y, z, ax, bx, cx, dx, ay, by, cy, dy, az, bz, &
              xb, xe, yb, ye, zb, ze, l, l1, l2, tlocal
      integer i, j, in

!     ax      x-spline coefficient
!     ay      y-spline coefficient
!     az      z-spline coefficient
!     bx      x-spline coefficient
!     by      y-spline coefficient
!     bz      z-spline coefficient
!     cx      x-spline coefficient
!     cy      y-spline coefficient
!     cz      z-spline coefficient
!     dx      x-spline coefficient
!     dy      y-spline coefficient
!     dz      z-spline coefficient
!     i       Counting variable
!     in      number of new nodes
!     j       Counting variable
!     l       relative distance from begin of piece
!     l1      distance from begin of piece
!     l2      distance from end of piece
!     st      help variable for scaling purposes
!     t       arc length of point considered
!     tbegin  begin of new piece
!     tend    end of piece
!     tlocal
!     total   length
!     x       x co-ordinate
!     xb      x co-ordinate of begin piece
!     xe      x co-ordinate of end piece
!     y       y co-ordinate
!     yb      y co-ordinate of begin piece
!     ye      y co-ordinate of end piece
!     z       z co-ordinate
!     zb      z co-ordinate of begin piece
!     ze      z co-ordinate of end piece
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Compute total length of curve

      total   =  0

      do 100 i = 1, ntal-1
          total  = total + arc(i)
100   continue

!     --- Compute distribution of points over curve according to tau

      in = 1
      j = 2
      tend = 0
      t = tau(2) * total

      do 300 i = 1, ntal-1

          tbegin = tend
          tend = tbegin + arc(i)

          if (t.gt.tend) go to 300

          ax = solve(          i )
          bx = solve(   ntal + i )
          cx = solve( 2*ntal + i )
          dx = solve( 3*ntal + i )

          xb = ax
          xe = solve(          i + 1 )

          ay = solve( 4*ntal + i )
          by = solve( 5*ntal + i )
          cy = solve( 6*ntal + i )
          dy = solve( 7*ntal + i )

          yb = ay
          ye = solve( 4*ntal + i + 1)

          if ( ndim.eq.3 ) then
             az = solve( 8*ntal + i )
             bz = solve( 9*ntal + i )
             cz = solve(10*ntal + i )
             dz = solve(11*ntal + i )

             zb = az
             ze = solve( 8*ntal + i + 1)
          end if

200       continue

!         --- Scale back

            st = (t - tbegin)*(arc(i)**(-1+alpha))

!           --- first estimate of point

            x = ax + bx*st + cx*st*st + dx*st*st*st
            y = ay + by*st + cy*st*st + dy*st*st*st

            if ( ndim.lt.3 ) then

              l1 = sqrt((x-xb)**2+(y-yb)**2)
              l2 = sqrt((x-xe)**2+(y-ye)**2)

            else

              z = az + bz*st + cz*st*st + dz*st*st*st
              l1 = sqrt((x-xb)**2+(y-yb)**2+(z-zb)**2)
              l2 = sqrt((x-xe)**2+(y-ye)**2+(z-ze)**2)

            end if

!           --- linear interpolation on relative distance
!               for better position point

            l = l1/(l1+l2)
            tlocal = (t-tbegin)/arc(i)

            if (abs(l-tlocal).gt.1e-6) then
               if (l.gt.tlocal) then
                  t = tbegin + tlocal**2/l*arc(i)
               else
                  t = tend - (1-tlocal)**2/(1-l)*arc(i)
               end if

               st = (t - tbegin)*(arc(i)**(-1+alpha))

               x = ax + bx*st + cx*st*st + dx*st*st*st
               y = ay + by*st + cy*st*st + dy*st*st*st

               if ( ndim.eq.3 ) z = az + bz*st + cz*st*st + dz*st*st*st

            end if

!           --- put x,y in curve array

            curves(ndim*in+1) = x
            curves(ndim*in+2) = y
            if ( ndim.eq.3 ) curves(ndim*in+3) = z

            in = in + 1
            j = j + 1
            if (j.eq.n) go to 400

            t = tau(j) * total

          if (t.lt.tend) go to 200

300   continue

400   continue

      curves(1) = solve(            1 )
      curves(2) = solve( 4 * ntal + 1 )

      curves(ndim*(n-1)+1) = solve(     ntal )
      curves(ndim*(n-1)+2) = solve( 5 * ntal )

      if ( ndim.eq.3 ) then

         curves(3  ) = solve( 8 * ntal + 1 )
         curves(3*n) = solve( 9 * ntal     )

      end if

      end
