      subroutine mshcheckelms1
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-04-2001
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Check if all edges in the two-dimensional mesh correspond to 2 elements
!    Edges may only be connected to one element if both nodes are on the
!    boundary
! **********************************************************************
!
!                       KEYWORDS
!
!     check
!     element
!     node
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, istar3, iedge, nelem_edge, inode1, &
              inode2, ipstart
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     iedge          edge sequence number
!     inode1         First node in edge
!     inode2         Second node in edge
!     ipstart        Starting address in array
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     iwork          integer work array of length npoint
!                    iwork(i) = 0 means that a node is not on a curve
!                    iwork(i) = 1 means that a node is on a curve
!     j              Counting variable
!     nelem_edge     number of elements corresponding to edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2235   Hanging node found
!    2237   Edge connected to more than two elements
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Initialize parameters
!     Clear iwork
!     Set iwork = 1 for all boundary points
!     For all edges do
!        nelem_edge := number of elements connected to the edge
!        if ( nelem_edge>2 ) then
!           error: too many elements
!        else if ( nelem_edge=1 ) then
!           if ( not both nodes on the boundary ) then
!              error: hanging node found
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcheckelms1' )

!     --- Initialize parameters

      istar3 = kmeshm(2) - 1

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear iwork

      iwork = 0

!     --- Set iwork = 1 for all boundary points

      do i = 1, ncurvs
         do j = kmeshm(4+i), kmeshm(5+i)-1
          if ( kmeshm(istar3+j).gt.0 ) iwork(kmeshm(istar3+j)) = 1
         end do
      end do

!     --- Loop over the edges
!         if ( nelem_edge>2 ) then
!           error: too many elements
!         else if ( nelem_edge=1 ) then
!           if ( not both nodes on the boundary ) then
!              error: hanging node found

      do iedge = 1, nedges

!     --- nelem_edge := number of elements connected to the edge

         ipstart = nmeshh(iedge)
         nelem_edge = nmeshh(ipstart)
         if ( nelem_edge.eq.1 ) then

!        --- nelem_edge=1, find both nodes

            ipstart = nmeshc(iedge)
            inode1 = nmeshc(ipstart+1)
            inode2 = nmeshc(ipstart+2)
            if ( iwork(inode1).eq.0 .or. iwork(inode2).eq.0 ) then

!           --- Not both nodes on the boundary

               call errint ( iedge, 1 )
               call errint ( inode1, 2 )
               call errint ( inode2, 3 )
               if ( iwork(inode1).eq.0 ) then

!              --- node1 is hanging

                  call errint ( inode1, 4 )

               else

!              --- node2 is hanging

                  call errint ( inode2, 4 )

               end if
               call errsub ( 2235, 4, 0, 0 )

            end if

         else if ( nelem_edge.gt.2 ) then

!        --- nelem_edge > 2: error

            call errint ( iedge, 1 )
            do i = 1, nelem_edge
               call errint ( nmeshh(ipstart+i), 1+i )
            end do
            call errsub ( 2237, 1+nelem_edge, 0, 0 )

         end if

      end do

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'mshcheckelms1' )
      end
