      subroutine mshcrv ( new, iinput, rinput, curstr, irelcurves, istartrel )
! ======================================================================
!
!        programmer    Guus Segal
!        version 24.1  date 12-03-2015 Extra debug
!        version 24.0  date 11-12-2014 Extra parameters
!        version 23.0  date 02-09-2010 Remove intarmsh
!
!   copyright (c) 1984-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the local arrays icurvsmsh and curvesmsh with information of the
!     curves
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     curve
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleconsts
      use sepmoduleplot
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical new
      integer iinput(*), irelcurves(*), istartrel
      double precision rinput(*), curstr(*)

!     curstr         i    array of length 10 * ncurvs to store some
!                         geometrical information of the curves
!     iinput         i    integer input array   see subroutine mesh
!     irelcurves      o    Array with the same length as the first part of
!                         iinput (until surfaces)
!                         Relative node numbers of the user points in
!                         framecurv are filled at the same positions as
!                         the user points in iinput
!     istartrel     i/o   next free position in irelcurves
!     new            i    indication whether the mesh is new (true) or old
!                         (false)
!     rinput         i    real input array   see subroutine mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iscolr
      integer, allocatable :: iwork(:)
      double precision plotfm, yfact

!     iscolr         Help variable to store the old value of dcolor
!     iwork          Work array of length ncurvs to indicate if a curve is
!                    composite or empty (1) or otherwise (0)
!     plotfm         Length of plot
!     yfact          Multiplication factor in width direction of plot
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSH003         Fill array CURVES
!     MSH013         initialize kmesh part m
!     MSHCHACR       Change coordinates in array coor by calls to FUNCCOOR
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLAFP7         Plot text using absolute co-ordinates
!     PLSCAL         Plot scales
!     PLTBN1         Plot curves and user points for 1-D
!     PLTBN2         Plot boundary of two-dimensional mesh
!     PLTBN3         Plot curves and user points for 3-D
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcrv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcrv'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(ncurvs), stat = error )
      if ( error/=0 ) &
         call eralloc (  error, ncurvs, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- fill array curves

      if ( debug ) write(irefwr,*) 'before msh003'
      call msh003 ( iinput, rinput, curstr, &
                    iwork, irelcurves, istartrel )
      if ( ierror/=0 ) go to 1000

!     --- plot curves

      if ( new .and. iplot/=0 ) then

         plotfm = rinput(1)
         yfact = rinput(2)
         if ( abs(plotfm)<=0.1d0 ) plotfm = defplo
         if ( abs(yfact)<=0d0 ) yfact = 1d0
         iscolr = dcolor
         if ( ndim==1 ) then

!        --- ndim = 1

            if ( juserp>0 ) then

!           --- juserp>0, plot user points

               call pltbn1 ( 1+100*juserp, icurvsmsh, curvesmsh, ncurvs, &
                             plotfm, yfact, nuspnt, rinput(3) )
               if ( ierror/=0 ) go to 1000

               if ( iscolr>0 ) dcolor = 1+iscolr
               if ( jtimes==2 ) then

!              --- jtimes = 2, plot the text "user points" and close the plot

                  call plscal
                  call plafp7 ( ale*0.49d0, 0.01d0, &
                                'USER POINTS', 0d0, 0.5d0, 11 )
                  jtimes = 3
                  call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )
                  jtimes = 1

               end if
               if ( ierror/=0 ) go to 1000

            end if
            dcolor = iscolr
            if ( jcurvp>0 ) call pltbn1 ( jcurvp+10*jcurvn, &
                 icurvsmsh, curvesmsh, ncurvs, plotfm, yfact, nuspnt, &
                 rinput(3) )

         else if ( ndim==2 ) then

!        --- ndim = 2

            if ( juserp>0 ) then

!           --- juserp>0, plot user points

               call pltbn2 ( 1+100*juserp, icurvsmsh, curvesmsh, ncurvs, &
                             plotfm, yfact, nuspnt, rinput(3), iwork )
               if ( ierror/=0 ) go to 1000

               if ( iscolr>0 ) dcolor = 1+iscolr
               if ( jtimes==1 ) then

!              --- jtimes = 1, plot the text "user points" and close the plot

                  call plscal
                  call plafp7 ( ale*0.49d0, 0.01d0, &
                                'USER POINTS', 0d0, 0.5d0, 11 )
                  jtimes = 3
                  call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )
                  jtimes = 1

               end if
               if ( ierror/=0 ) go to 1000

            end if

            dcolor = iscolr
            if ( jcurvp>0 ) call pltbn2 ( jcurvp+10*jcurvn, &
                 icurvsmsh, curvesmsh, ncurvs, plotfm, yfact, nuspnt, &
                 rinput(3), iwork )

         else if ( ndim==3 .and. jcurvp>0 ) then

!        --- ndim = 3

            if ( juserp>0 ) then

!           --- juserp>0, plot user points

               call pltbn3 ( 1+100*juserp, icurvsmsh, curvesmsh, ncurvs, &
                             plotfm, yfact, nuspnt, rinput(3) )

               if ( iscolr>0 ) dcolor = 1+iscolr
               if ( jtimes==2 ) then

!              --- jtimes = 2, plot the text "user points" and close the plot

                  call plscal
                  call plafp7 ( ale*0.49d0, 0.01d0, &
                                'USER POINTS', 0d0, 0.5d0, 11 )
                  jtimes = 3
                  call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )
                  jtimes = 1

               end if

            end if

            dcolor = iscolr
            if ( jcurvp>0 ) call pltbn3 ( jcurvp+10*jcurvn, &
                 icurvsmsh, curvesmsh, ncurvs, plotfm, yfact, nuspnt, &
                 rinput(3) )

         end if
         if ( ierror/=0 ) go to 1000
         if ( jcurvp>0 .and. jtimes==2 ) then

!        --- jtimes = 1, plot the text "curves" and close the plot

            if ( iscolr>0 ) dcolor = 1+iscolr
            call plscal
            call plafp7 ( ale*0.49d0, 0.01d0, 'CURVES', 0d0, &
                          0.5d0, 6 )
            jtimes = 3
            call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )
            jtimes = 1

         end if
         dcolor = iscolr

      end if
      if ( ierror/=0 ) go to 1000

!     --- initialize array kmeshm

      if ( debug ) write(irefwr,*) 'before msh013'
      if ( new .or. nsurvl==0 ) &
         call msh013 ( iinput, irelcurves )
      if ( ierror/=0 ) go to 1000

      if ( iinp13/=0 ) then

!     --- iinp13 # 0   change coordinates of curves

         isurnr = 0
         call mshchacr ( iinput(iinp13), curvesmsh, iwork, icurvsmsh, 2 )

      end if

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcrv'

      end if  ! ( debug )
      call erclos ( 'mshcrv' )

      end subroutine mshcrv
