      subroutine mshd01 ( lpoly, jpoly, ipoly, jconc, jpoln, ipoln, &
                          coor, nr, iview, npt, coarse, rim, angl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.1  date 12-08-1996 Minor corrections
!        version  5.0  date 28-12-1995 Complete update by Sander de Snoo
!        version  4.2  date 18-08-1992 remove float
!
!   copyright (c) 1983-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    subroutine mshd01 divides a concave polygon in two new but
!    smaller polygons
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     polygon
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleconsts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lpoly(*), jpoly(*), ipoly(*), nr(*), iview(*), &
              jconc, npt, ipoln, jpoln
      double precision coor(2,*), coarse(*), rim, angl(*)

!     angl       i
!     coarse     i    Array containing the coarseness per point
!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     ipoln      i    length of ipoly
!     ipoly      i
!     iview      i
!     jconc      i
!     jpoln      i    length of jpoly
!     jpoly      i
!     lpoly      i
!     npt        i    total number of points in the mesh
!     nr         i
!     rim        i    length of smallest edge along the boundary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer n, i, i1,i2,i3,jw,ih,iconc,nz,j,k,jl,jl1,jl2, &
              jmog,n1,jllast,jc,j1,j2
      double precision distcn,wa,w,xi,yi,xi1,yi1,xi2,yi2, &
                       xj,yj,xj1,yj1,xj2,yj2,dx,dy,alpha,afwijk,sum, &
                       c1,c2,c3,c4,af1,af2,af3,af4,rimog,rjmog,slang,a, &
                       s,ah,angli,angi1x,anglj
      logical bbest, bforty, bvisib

!     a          Help variable to store quantity
!     af1
!     af2
!     af3
!     af4
!     afwijk
!     ah
!     alpha
!     angi1x
!     angli
!     anglj
!     bbest
!     bforty
!     bvisib
!     c1
!     c2
!     c3
!     c4
!     distcn
!     dx         length of vector in x-direction
!     dy         length of vector in y-direction
!     i          Counting variable
!     i1
!     i2
!     i3
!     iconc
!     ih         Help variable to store a constant temporarily
!     j          Counting variable
!     j1
!     j2
!     jc
!     jl
!     jl1
!     jl2
!     jllast
!     jmog
!     jw
!     k          Counting variable
!     n
!     n1
!     nz         number of points in z direction
!     rimog
!     rjmog
!     s
!     slang
!     sum        Help variable to compute a sum
!     w
!     wa
!     xi
!     xi1
!     xi2
!     xj
!     xj1
!     xj2
!     yi
!     yi1
!     yi2
!     yj
!     yj1
!     yj2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer eround

!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     EROUND    Round real to integer with minimum relative error
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     ERRWAR    Print warnings
!     INSTOP    Stop the program
!     MSHG01    Compute the cosine of an angle
!     MSHG02    Check whether angle is greater or smaller than 180 degrees
!     MSHG10    Determine whether lines have an intersection point
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     350   Number of points in boundary too small
!     351   mesh generator general terminated the execution of the program
!     375   the difference in coarseness in two locations is too large in to
!           relation their mutual distance
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshd01' )

!     --- reference distance

      distcn=1.d-4*rim

!     --- first determine whether there are any "concave" points at all

      n=lpoly(jconc)

      if ( n.lt.3 ) then

!     --- n < 3 wrong input , error

        call errint ( n, 1 )
        call errint ( 3, 2 )
        call errsub ( 350, 2, 0, 0 )
        call instop

      end if

!     --- fill array nr containing the global numbering

      do 100 i = 1, n
        nr(i+1) = lpoly(i+jconc)
100   continue

      nr(1)=nr(n+1)
      nr(n+2)=nr(2)

!     --- check for each point whether it is "concave"

      wa=-1.0d0
      jw=0
      iconc=0

      do 110 i=1,n
        i1=nr(i)
        i2=nr(i+1)
        i3=nr(i+2)
        call mshg02(coor(1,i1),coor(2,i1),coor(1,i2),coor(2,i2), &
                    coor(1,i3),coor(2,i3),ih,w)
        if ( ih.eq.1 ) then

!       --- ih is 1 hence point is concave

          iconc=iconc+1
          if ( w.gt.wa ) then
            jw=i
            wa=w
          end if
        end if
110   continue

      if ( jw.eq.0 ) then

!     --- jw = 0 no concave points in this polygon , place in ipoly

        ipoln=ipoln+1
        ipoly(ipoln)=n

        do 120 i=1,n
          ipoln=ipoln+1
          ipoly(ipoln)=lpoly(i+jconc)
120     continue
        ipoly(ipoln+1)=0

      else

!     --- jw > 0 hence concave points and in jw extremal .
!         determine which points are "visible" for jw
!         ( use array iview )
!         if only one concave point than all points are visible

         if ( iconc.eq.1 ) then

            nz=0
            j=jw+2
            if ( j.gt.n ) j=j-n
            i1=jw-1
            if ( i1.eq.0 ) i1=n

!           --- do while (j.ne.i1)

200         continue
               nz=nz+1
               iview(nz)=j
               j=j+1
               if ( j.gt.n ) j=j-n
               if (j.ne.i1) goto 200

!              --- array iview contains nz (=n-3) points , &

         else

!        --- iconc > 1 : there are several concave points
!            determine which points are really visible

            bvisib = .false.

            i=nr(jw+1)
            xi=coor(1,i)
            yi=coor(2,i)
            i1=nr(jw)
            xi1=coor(1,i1)
            yi1=coor(2,i1)
            nz=n-3

!           --- all points initially invisible

            do 210 k=1,nz
              iview(k)=0
210         continue

!           --- angle of i1 - i - x_axis

            angi1x = atan2(yi1-yi,xi1-xi)

!           --- calculate angles with line i1-i (angle in interval [0,2pi])

            do 220 k=1,n+2
              xj=coor(1,nr(k))
              yj=coor(2,nr(k))

!             --- are points coinciding ?

              s=(xj-xi)**2+(yj-yi)**2

              if ( s.gt.distcn**2 ) then
                angl(k) = angi1x - atan2(yj-yi,xj-xi)
                if (angl(k).lt.0) angl(k)=angl(k)+2*pi
              else

!             --- coinciding point, treated as invisible

                angl(k)=0
              end if
220         continue

!           --- angle of i1-i-i2

            angli = angl(jw+2)

            do 250 k=1,nz
               j=mod(jw+k,n)+1
               xj=coor(1,nr(j+1))
               yj=coor(2,nr(j+1))

!              --- is j visible
!                  1) line i-j inside concave angle
!                     visible if j inside angle i2-i-i1 minus 6 degrees on
!                     each side

               anglj = angl(j+1)
               if ( (anglj.lt.0.1d0) .or. &
                    (anglj.gt.angli-0.1d0) ) goto 250

!              --- 2) point on (at least one) line with decreasing angle
!                     (i1-i-j)?

               if ( (angl(j).lt.angl(j+1)).and. &
                    (angl(j+1).lt.angl(j+2)) ) goto 250

!              --- 3) point hidden by sume line of polygon?
!                     loop through all the lines with decreasing angle
!                     positioned inside an angle of 20 degrees on each
!                     side of j
!                     except neigbours of j and jw

               jl=jw+1
               jllast = jw-1
               if ( jllast.lt.1) jllast = n
240            continue
               jl=jl+1
               if ( jl.gt.n ) jl=jl - n
               if ( jl.ne.j ) then
                  if ( (angl(jl).gt.angl(jl+1)) .and. &
                       (angl(jl).gt.angl(j+1)-0.35d0) .and. &
                       (angl(jl+1).lt.angl(j+1)+0.35d0) ) then
                    jl1=nr(jl+1)
                    jl2=nr(jl)
                    call mshg10(xi,yi,xj,yj,coor(1,jl1),coor(2,jl1), &
                                 coor(1,jl2),coor(2,jl2),ih)
                    if ( ih.le.0 ) goto 250
                  end if

               else

!              --- skip lines with j

                 jl=jl+1
                 if ( jl.gt.n ) jl=jl - n

               end if

               if ( jl.ne.jllast ) goto 240

!              --- all tests past: point is visible

               bvisib = .true.
               iview(k)=j

250         continue

            if ( .not. bvisib ) then

!           --- bvisib = false , no visible points at all

              call errsub ( 351, 0, 0, 0 )
              call instop

            end if

         end if

!        --- run through iview to determine best splitting line possibility

         bbest  = .false.
         bforty = .true.
         afwijk = 5.0d0

         i  = nr(jw+1)
         xi = coor(1,i)
         yi = coor(2,i)
         i1 = nr(jw)
         xi1= coor(1,i1)
         yi1= coor(2,i1)
         i2 = nr(jw+2)
         xi2= coor(1,i2)
         yi2= coor(2,i2)

310      continue

         do 350 k=1,nz
            j=iview(k)
            if ( j.eq.0 ) goto 350

            jc =nr(j+1)
            xj =coor(1,jc)
            yj =coor(2,jc)
            j1 =nr(j)
            xj1=coor(1,j1)
            yj1=coor(2,j1)
            j2 =nr(j+2)
            xj2=coor(1,j2)
            yj2=coor(2,j2)

!           --- angle 1 :  i1,i,j

            call mshg01(xi1,yi1,xi,yi,xj,yj,c1)

!           --- angle 2 : i2,i,j

            call mshg01(xi2,yi2,xi,yi,xj,yj,c2)

!           --- angle 3 : j1,j,i

            call mshg01(xj1,yj1,xj,yj,xi,yi,c3)
            if ( c3.gt.0.99d0 ) goto 350

!           --- angle 4 : j2,j,i

            call mshg01(xj2,yj2,xj,yj,xi,yi,c4)
            if ( c4.gt.0.99d0 ) goto 350

!           --- check all new angles if bforty = true

            if ( bforty .and. &
                 (( c1.gt.0.766d0 ) .or. &
                  ( c2.gt.0.766d0 ) .or. &
                  ( c3.gt.0.766d0 ) .or. &
                  ( c4.gt.0.766d0 )) )goto 350

!           --- all angles between 40 and 320 degrees .
!               compute sum of differences with 60, 120 and 180 degrees

            af1=min( abs(c1-0.5d0),abs(c1+0.5d0),abs(c1+1.0d0) )
            af2=min( abs(c2-0.5d0),abs(c2+0.5d0),abs(c2+1.0d0) )
            af3=min( abs(c3-0.5d0),abs(c3+0.5d0),abs(c3+1.0d0) )
            af4=min( abs(c4-0.5d0),abs(c4+0.5d0),abs(c4+1.0d0) )
            sum=af1+af2+af3+af4

            if ( sum.lt.afwijk ) then

!           --- candidate found

              afwijk = sum
              bbest = .true.
              jmog = j

            end if

350      continue

         if ( .not. bbest ) then
            if ( bforty ) then

!           --- bbest= .false. , up to now no candidate splittingline
!               drop requirement concerning angles

               bforty= .false.
               goto 310

            else

!           --- no splitting line found

               call errsub ( 351, 0, 0, 0 )
               call instop

           end if

         end if

!        --- create best splittingline  i-jmog
!            length, coarseness:

         xj=coor(1,nr(jmog+1))
         yj=coor(2,nr(jmog+1))
         slang=sqrt((xj-xi)**2+(yj-yi)**2)

         rimog=coarse(i)
         rjmog=coarse(nr(jmog+1))

!        --- determine number of elements

         if (abs(rimog/rjmog-1d0).ge.1d-8) then
            a = slang/(rimog-rjmog)*log(rimog/rjmog)
         else
            a = 2*slang/(rimog+rjmog)
         end if

         if (afwijk.lt.0.6d0) then
           n1=eround(a)
         else
           n1=max(1,nint(a))
         end if

!        --- determine growth factor alpha

         alpha=(rjmog/rimog)**(1d0/n1)

!        --- correct n1 if alpha is large

         if ( alpha.gt.2.5d0 .or. alpha.lt.0.4d0 ) then
            n1=n1+1
            alpha=(rjmog/rimog)**(1d0/n1)
         end if

         if ( alpha.gt.3.3d0 .or. alpha.lt.0.3d0 ) &
                call errwar ( 375, 0, 0, 0 )

         if ( n1.gt.1 ) then

!        --- n1 > 1 ,  hence  ( n1-1 ) new points
!            compute length of first element

            if (abs(alpha-1d0).gt.1d-8) then
              s=(1d0-alpha**n1)/(1d0-alpha)
            else
              s=n1*(1d0+0.5d0*(n1-1)*(alpha-1d0))
            end if

            dx=(xj-xi)/s
            dy=(yj-yi)/s

!           --- place points in right positions

            s=1d0
            ah=alpha

            do 400 i=1,n1-1
                npt=npt+1
                coor(1,npt)=xi+s*dx
                coor(2,npt)=yi+s*dy
                coarse(npt)=rimog*ah
                ah=ah*alpha
                s=alpha*s+1d0
400         continue

         end if

!        --- fill array jpoly

         jpoln=jpoln+1
         jpoly(jpoln)=mod(jmog-jw+n-1,n)+1 +n1

         ih=jw-1

!        --- do while (ih.ne.jmog)

500      continue
            ih=ih+1
            if ( ih.gt.n ) ih=ih-n
            jpoln=jpoln+1
            jpoly(jpoln)=nr(ih+1)
            if (ih.ne.jmog) goto 500

         do 510 i=1,n1-1
            jpoln=jpoln+1
            jpoly(jpoln)=npt+1-i
510      continue

         jpoln=jpoln+1
         jpoly(jpoln)=mod(jw-jmog+n-1,n)+1 +n1

         ih=jmog-1

!        --- do while (ih.ne.jw)

520      continue
            ih=ih+1
            if ( ih.gt.n ) ih=ih-n
            jpoln=jpoln+1
            jpoly(jpoln)=nr(ih+1)
            if (ih.ne.jw) goto 520

         do 530 i=1,n1-1
            jpoln=jpoln+1
            jpoly(jpoln)=npt+i-(n1-1)
530      continue

      end if
      call erclos ( 'mshd01' )

      end
