      subroutine mshd6 ( acoor, kmeshc, nelem, npoint, npt, npti, &
                         iwork, jwork, ibuurp, jbuure )
! ======================================================================
!
!        programmer     Niek Praagman
!        version 2.0    date  08-06-1996 Removal of nbndpt
!        version 1.4    date  09-07-1992 adjust npt if necessary
!        version 1.3    date  29-06-1990 extra check for boundary points
!        version 1.2    date  03-05-1990 new programming norms
!        version 1.1    date  17-02-1989 Deletion of superfl. parms
!
!
!
!   copyright (c) 1984-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard or soft copy granted only by written license obtained
!   from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system (e.g. , in memory, disk or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make 6-node triangles out of 3-node triangles
!     Create all internal mid-nodal points.
! **********************************************************************
!
!                       KEYWORDS
!
!   mesh
!   mesh_generation
!   surface
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      double precision acoor(2,*)
      integer kmeshc(*), nelem, npoint, npt, npti, &
              iwork(*), jwork(*), ibuurp(*), jbuure(*)

!     acoor     i/o   array containing the coordinates of the points
!     ibuurp     i    array with neighbour points sequentially
!     iwork      i    pointer array for neighbouring nodes
!     jbuure     i    array with neighbour elements sequentially
!     jwork      i    pointer array for neighbour elements
!     kmeshc    i/o   at input array containing 3-node element-information
!                     at output 6-node element information
!                     to the elements
!     nelem      i    Number of elements
!     npoint     i    number of points in mesh before creation of six- &
!                     node elements
!     npt        i    at input npoint
!                     at output total number of points in mesh after creation
!                     of six-node elements
!     npti       i    number of boundary points + 1
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i1, i2, i3, i4, i5, i6, iapp, ieind, iel, ielem, &
              ikn, ipunt, istart, itotal, j, jeind, jh, jkn, jstart

!     i          Counting variable
!     i1         node number
!     i2         node number
!     i3         node number
!     i4         node number
!     i5         node number
!     i6         node number
!     iapp       number of appearings
!     ieind      end pointer
!     iel        loop variable for elements
!     ielem      Element number
!     ikn        loop for node number
!     ipunt      node number
!     istart     start pointer
!     itotal     total number of
!     j          loop variable
!     jeind      end pointer
!     jh         help pointer
!     jkn        node pointer
!     jstart     start pointer
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Check situation

      if ( npt.lt.npoint ) npt = npoint

!     --- Run through all elements and construct new places

      do 100 i = 1 , nelem

           j   = 3 * (nelem+1-i)
           i1  =   kmeshc(j-2)
           i2  =   kmeshc(j-1)
           i3  =   kmeshc(j)

           j   = 2 * j
           kmeshc(j-5) = i1
           kmeshc(j-4) =  0
           kmeshc(j-3) = i2
           kmeshc(j-2) =  0
           kmeshc(j-1) = i3
           kmeshc(j  ) =  0

100   continue

      itotal = npoint - 1

      do 200 ikn = 1 , itotal

!     --- Run through all neighbour points of node ikn

          jstart = 0

          if ( ikn.ne.1 ) jstart = iwork(ikn-1)

          jstart = jstart + 1
          jeind  = iwork(ikn)

      do 200 i = jstart , jeind

          jkn = ibuurp(i)

          if ( jkn.le.ikn ) goto 200

          ipunt  = 0
          istart = jwork(jkn-1) + 1
          ieind  = jwork(jkn)

!         --- Run through all adjacent elements

          iapp = 0

          do 105 iel = istart , ieind

              ielem = jbuure(iel)

              j     = 6 * ielem

              i1    = kmeshc(j-5)
              i2    = kmeshc(j-3)
              i3    = kmeshc(j-1)

              if ((ikn.eq.i1).and.(jkn.eq.i2)) iapp = iapp + 1
              if ((ikn.eq.i1).and.(jkn.eq.i3)) iapp = iapp + 1
              if ((ikn.eq.i2).and.(jkn.eq.i1)) iapp = iapp + 1
              if ((ikn.eq.i2).and.(jkn.eq.i3)) iapp = iapp + 1
              if ((ikn.eq.i3).and.(jkn.eq.i2)) iapp = iapp + 1
              if ((ikn.eq.i3).and.(jkn.eq.i1)) iapp = iapp + 1

105       continue

          do 110 iel = istart , ieind

              ielem = jbuure(iel)

              j     = 6 * ielem

              i1    = kmeshc(j-5)
              i2    = kmeshc(j-3)
              i3    = kmeshc(j-1)

              jh = 0

              if ((ikn.eq.i1).and.(jkn.eq.i2)) jh = j-4
              if ((ikn.eq.i1).and.(jkn.eq.i3)) jh = j
              if ((ikn.eq.i2).and.(jkn.eq.i1)) jh = j-4
              if ((ikn.eq.i2).and.(jkn.eq.i3)) jh = j-2
              if ((ikn.eq.i3).and.(jkn.eq.i2)) jh = j-2
              if ((ikn.eq.i3).and.(jkn.eq.i1)) jh = j

!             --- if jh = 0 line ikn - jkn not in this triangle

              if ( jh.eq.0 ) goto 110

!             --- kmeshc(jh) has to be filled

              if ( jkn.ge.npti  ) then
                   ipunt = 1
                   kmeshc(jh) = npt + 1
              else if ( jkn.eq.ikn+2 ) then
                   kmeshc(jh) = ikn + 1
              else

!             --- Special situation check whether line is boundary line

                   if ( iapp.eq.1 ) then
                        kmeshc(jh)  = jkn + 1
                   else if ( iapp.eq.2 ) then
                        ipunt = 1
                        kmeshc(jh) = npt + 1
                   end if

              end if

110           continue

          if ( ipunt.eq.1 ) npt = npt + 1

200   continue

!     --- Run through all elements and fill midpoint coordinates

      do 300 i = 1 , nelem

          j          = 6 * i

          i1         = kmeshc(j-5)
          i2         = kmeshc(j-4)
          i3         = kmeshc(j-3)
          i4         = kmeshc(j-2)
          i5         = kmeshc(j-1)
          i6         = kmeshc(j  )

          acoor(1,i2) = (acoor(1,i1)+acoor(1,i3)) * 0.5d0
          acoor(2,i2) = (acoor(2,i1)+acoor(2,i3)) * 0.5d0
          acoor(1,i4) = (acoor(1,i3)+acoor(1,i5)) * 0.5d0
          acoor(1,i6) = (acoor(1,i5)+acoor(1,i1)) * 0.5d0
          acoor(2,i4) = (acoor(2,i3)+acoor(2,i5)) * 0.5d0
          acoor(2,i6) = (acoor(2,i5)+acoor(2,i1)) * 0.5d0

300   continue

      end
