      subroutine mshfillshapes ( ichoice )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill default shapes depending on ichoice
!
! **********************************************************************
!
!                       KEYWORDS
!
!     shape
!     mesh
!     default
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ichoice

!     ichoice        i    code to detect text that has been read
!                          1: TRIANGLE
!                          2: QUADRILATERAL
!                          3: TETRAHEDRON
!                          4: HEXAHEDRON
!                          5: LINEAR_TRIANGLE
!                          6: QUADRATIC_TRIANGLE
!                          7: LINEAR_TETRAHEDRON
!                          8: QUADRATIC_TETRAHEDRON
!                          9: LINEAR_HEXAHEDRON
!                         10: QUADRATIC_HEXAHEDRON
!                         11: LINEAR_QUADRILATERAL
!                         12: QUADRATIC_QUADRILATERAL
!                         13: LINEAR
!                         14: QUADRATIC
!                         15: EXTENDED_LINEAR_TRIANGLE
!                         16: EXTENDED_LINEAR_QUADRILATERAL
!                         17: EXTENDED_QUADRATIC_TRIANGLE
!                         18: EXTENDED_TETRAHEDRON14
!                         19: EXTENDED_TETRAHEDRON15
!                         20: EXTENDED_LINEAR_HEXAHEDRON
!                         21: EXTENDED_LINEAR_TETRAHEDRON
!                         22: LINEAR_WEDGE
!                         23: QUADRATIC_WEDGE
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2917   Option not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfillshapes' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfillshapes'
         write(irefwr,1) 'ichoice', ichoice
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case ( ichoice )

      case(1, 3, 5, 7, 13)

!     --- LINEAR (default values)

      case(6, 8, 14)

!     --- QUADRATIC

         shape_cur = 2
         shape_sur = 4
         shape_vol = 12

      case(2, 4, 9, 11, 22)

!     --- QUADRILATERAL

         shape_sur = 5

      case(10, 12, 23)

!     --- QUADRATIC_HEXAHEDRON

         shape_cur = 2
         shape_sur = 6
         shape_vol = 14

      case(15)

!     --- EXTENDED_LINEAR_TRIANGLE

         shape_cur = 1
         shape_sur = 10
         shape_vol = 18

      case(16)

!     --- EXTENDED_LINEAR_QUADRILATERAL

         shape_cur = 1
         shape_sur = 9
         shape_vol = 17

      case(17,18)

!     --- EXTENDED_QUADRATIC_TRIANGLE

         shape_cur = 2
         shape_sur = 7
         shape_vol = 15

      case(19)

!     --- EXTENDED_QUADRATIC_TRIANGLE

         shape_cur = 2
         shape_sur = 7
         shape_vol = 16

      case default

!     --- Option not available

         call errint ( ichoice, 1 )
         call errsub ( 2917, 1, 0, 0 )
         call errcrd

      end select  ! case ( ichoice )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshfillshapes'

      end if  ! ( debug )
      call erclos ( 'mshfillshapes' )

      end subroutine mshfillshapes
