      subroutine mshfilsr ( isurnr, irefc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 15-11-2012 Long integers
!        version  2.1  date 20-10-2008 Allow isurf = 24
!        version  2.0  date 01-08-2001 Extra parameter ibuffr,
!                                      new definition kmesh part h
!
!   copyright (c) 1996-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Store contents of surface in file 3 and references in kmesh part h
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     fill
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isurnr, irefc

!     irefc      i    dataset reference number for reading (>0 formatted, &
!                     <0 unformatted)
!     isurnr     i    Surface sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, isurf, npointlc, ielem, &
              inpelm, istart, iend
      double precision, allocatable :: work(:,:)
      integer, allocatable :: iwork(:), mapar(:), maparinv(:)

!     i              Counting variable
!     ielem          Element number
!     iend           Last position used in array
!     inpelm         number of points per element
!     istart         First position in array
!     isurf          Type of surface with respect to mesh generator
!     iwork          integer work array of length inpelm*inelem
!     k              Counting variable
!     mapar          mapping array: contains the mapping from local to
!                    global numbering of the nodal points (locally used)
!     maparinv       Work array of length npoint (local) containing the
!                    inverse of mapar
!     npointlc       Local number of points
!     work           Work array to store the local co-ordinates
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DCOPY          Copy double precision dx to double precision dy
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     SEPMSHEXTKT    Compute starting and ending address of elements in kmeshc
!                    by using the information stored in kmesh and kmesht
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     isurf := kmeshs(nsurfs+1+(isurnr-1)*8)
!     if ( isurf>0 ) then
!        Read array mapar
!        Compute the inverse of mapar
!        Store local co-ordinates in work
!        Write local co-ordinates to file 3
!        If ( nvolms=0 ) then
!           ielem1 := kmesht(1,isurnr)
!           ielem2 := kmesht(2,isurnr)
!           Compute starting address of element ielem1 in kmeshc
!           Compute end address of element ielem2 in kmeshc
!           Store local element numbers in iwork
!        else ( nvolms>0 )
!           Store local element numbers in iwork using array kmeshn
!        write local kmeshc to file 3
!        write array mapar to file 3
!        if ( isurf=2 ) then
!           Read extra information
!           Write extra information
!        else if ( isurf=7, 11 or 12 ) then
!           Read extra information
!           Write extra information
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfilsr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfilsr'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      isurf = kmeshs(nsurfs+2+(isurnr-1)*8)
      if ( isurf==24 ) isurf = 7    ! compound surface
      npointlc = kmeshh(1,isurnr+1)

      if ( debug ) write(irefwr,1) 'isurf, npointlc', isurf, npointlc

      allocate ( iwork(max(nsurfs,nelem)*npelm), mapar(npointlc), &
                 maparinv(npoint), work(ndim,npointlc), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, npointlc*2+npoint+nelem, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Read array mapar

      if ( irefc>0 ) then
         read(irefc,*) (mapar(i),i=1,npointlc)
      else
         read(-irefc) (mapar(i),i=1,npointlc)
      end if

!     --- Compute the inverse of mapar

      if ( mapar(1)>0 ) then

!     --- Standard situation, the surface has been filled

         do i = 1, npointlc
            maparinv(mapar(i)) = i
         end do

      else

!     --- The surface has not been filled

         do i = 1, npointlc
            maparinv(i) = i
            mapar(i) = i
         end do

      end if

!     --- Store local co-ordinates in work

      do i = 1, npointlc
         do k = 1, ndim
            work(k,i) = coor(k,mapar(i))
         end do
      end do

!     --- Write local co-ordinates to file 3

      call dcopy ( ndim*npointlc, work, 1, kmhsur(isurnr)%coorsur, 1 )
      if ( nvolms==0 ) then

!     --- Only surface elements, use array kmesht

         call sepmshextkt ( isurnr, istart, iend )

!        --- Store local element numbers in iwork

         do i = istart, iend
            iwork(i-istart+1) = maparinv(kmeshc(i))
         end do

      else

!     --- nvolms>0
!         Store local element numbers in iwork using array kmeshn

         istart = 2*nsurfs
         ielem  = 0
         do i = 1, isurnr-1
            inpelm = kmeshn(i)
            inelem = kmeshn(i+nsurfs)
            istart = istart+inelem*inpelm
            ielem  = ielem+inelem
         end do
         inpelm = kmeshn(isurnr)
         inelem = kmeshn(isurnr+nsurfs)
         istart = istart+1
         iend = istart+inelem*inpelm-1

         if ( debug ) &
            write(irefwr,1) 'nsurfs, isurnr, inpelm, inelem, istart, iend', &
                             nsurfs, isurnr, inpelm, inelem, istart, iend

!        --- Store local element numbers in iwork

         if ( kmeshn(istart)>0 ) then

!        --- Standard situation, the surface has been filled

            do i = istart, iend
               iwork(i-istart+1) = maparinv(kmeshn(i))
            end do

         else

!        --- the surface has not been filled

            do i = istart, iend
               iwork(i-istart+1) = i-istart+1
            end do

         end if

      end if

!     --- write local kmeshc to file 3

      k = iend-istart+1
      kmhsur(isurnr)%kmeshcsur(1:k) = iwork(1:k)

!     --- write array mapar to file 3

      kmhsur(isurnr)%maparsur(1:npointlc) = mapar(1:npointlc)

      if ( isurf==2 ) then

!     --- Read extra information

         if ( irefc>0 ) then
            read(irefc,*) npointlc
            read(irefc,*) (mapar(i),i=1,npointlc)
         else
            read(-irefc) npointlc
            read(-irefc) (mapar(i),i=1,npointlc)
         end if

!        --- Write extra information

         kmhsur(isurnr)%renumsur(1:npointlc) = mapar(1:npointlc)

      else if ( isurf==7 .or. isurf==11 .or. isurf==12 ) then

!     --- Read extra information

         if ( irefc>0 ) then
            read(irefc,*) ncurvs
            read(irefc,*) (mapar(i),i=1,ncurvs)
         else
            read(-irefc) ncurvs
            read(-irefc) (mapar(i),i=1,ncurvs)
         end if

!        --- Write extra information

         kmhsur(isurnr)%renumsur(1:ncurvs) = mapar(1:ncurvs)

      end if

      deallocate ( iwork, work, mapar, maparinv, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshfilsr'

      end if  ! ( debug )
      call erclos ( 'mshfilsr' )

      end subroutine mshfilsr
