      subroutine mshg03 ( x1, y1, x2, y2, xi, yi, xj, yj, iresul )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  7.1  date 14-02-2010 Count also if crossing point is
!                                      end of line
!        version  7.0  date 17-02-2005 Only real intersections!
!        version  6.0  date 30-06-1993 complete redesign
!        version  5.0  date 10-10-1990 complete new version
!        version  1.0  date 17-04-1983
!
!   copyright (c) 1983-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Determine whether lines  1 - 2  and  I - J have an intersection
!     point.
! **********************************************************************
!
!                       KEYWORDS
!
!     intersection
!     2d
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision x1, y1, x2, y2, xi, yi, xj, yj
      integer iresul

!     iresul     o    indicator of result
!                         iresul = 0 : there is an intersection point
!                         iresul = 1 : there is no intersection point
!     x1         i    x-coordinate point 1
!     x2         i    x-coordinate point 2
!     xi         i    x-coordinate point i
!     xj         i    x-coordinate point j
!     y1         i    y-coordinate point 1
!     y2         i    y-coordinate point 2
!     yi         i    y-coordinate point i
!     yj         i    y-coordinate point j
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision eps,xs,ys,r1,r2,r3,r4

!     eps        accuracy
!     r1         direction coefficient
!     r2         direction coefficient
!     r3         direction coefficient
!     r4         direction coefficient
!     xs         x-coordinate intersection point
!     ys         y-coordinate of intersection point of two lines
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'mshg03' )

      eps = 10 * epsmac

      iresul = 1

      if (abs(xi-xj)>(eps*(abs(xi)+abs(xj)))) then

!     --- xi # xj

         if (abs(x1-x2)>(eps*(abs(x1)+abs(x2)))) then

!        --- xi # xj  and  x1 # x2
!            Determine x of intersection point

            r1  =  (y1*x2-y2*x1)  /  (x2-x1)
            r2  =  (yi*xj-yj*xi)  /  (xj-xi)
            r3  =  (yj-yi)        /  (xj-xi)
            r4  =  (y2-y1)        /  (x2-x1)

            if ( abs(r3-r4)>(eps*(abs(r3)+abs(r4))) ) then

               xs  = (r1-r2) / (r3-r4)

!              --- Check whether xs of intersection is between i and j
!                  or j and i

               if ( xi<xs .and. xj>xs .or. &
                    xj<xs .and. xi>xs ) then

!              --- Check xs between x1 and x2 or between x2 and x1

                  if ((x1<xs).and.(x2>xs)) then
                     iresul=0
                  else if ((x2<xs).and.(x1>xs)) then
                     iresul=0
                  end if

               end if

            end if

         else if (abs(yi-yj)<=(eps*(abs(yi)+abs(yj)))) then

!        --- x1 = x2  line x = constant
!            yi = yj  line y = constant
!            Check position intersection point

!            --- Coordinates intersection are

            xs = x1
            ys = yi

            if ( xi<xs .and. xj<xs ) then

!           --- Linepiece i-j is "left" from xs

            else if ( xi>xs .and. xj>xs ) then

!           --- Linepiece i-j is "right" from xs

            else if ( y1<ys .and. y2<ys ) then

!           --- Linepiece 1-2 is "under" ys

            else if ( y1>ys .and. y2>ys ) then

!           --- Linepiece 1-2 is "above" ys

            else

!           --- Intersection

               iresul=0
            end if

         else

!        --- Determine y of intersection point

            ys = ((yj-yi)*x1+yi*xj-yj*xi)/(xj-xi)

            if ( y1<ys .and. y2>ys  .or. &
                 y2<ys .and. y1>ys )  then

               if ( yi<ys .and. yj>ys ) then
                  iresul=0
               else if ( yj<ys .and. yi>ys ) then
                  iresul=0
               end if

            end if

         end if

      else if ( abs(x1-x2)>(eps*(abs(x1)+abs(x2))) ) then

!     --- xi = xj however x1#x2

         if ( abs(y1-y2)<=(eps*(abs(y1)+abs(y2))) ) then

!        --- xi = xj and y1 = y2

            xs = xi
            ys = y1

            if ( yi<ys .and. yj<ys ) then
            else if ( yi>ys .and. yj>ys ) then
            else if ( x1<xs .and. x2<xs ) then
            else if ( x1>xs .and. x2>xs ) then
            else
               iresul =  0
            end if

         else

!        --- Determine y of common point

            ys=((y2-y1)*xi+y1*x2-y2*x1)/(x2-x1)
            if ( yi<ys .and. yj>ys .or. &
                 yi>ys .and. yj<ys ) then

               if ( y1<=ys .and. y2>=ys ) then
                  iresul=0
               else if ( y2<=ys .and. y1>=ys ) then
                  iresul=0
               end if

            end if

         end if

      else

!     --- line x1 = x2  also of type x = constant : no intersection
!         possible, leave routine

      end if

      call erclos( 'mshg03' )

      end
