      subroutine mshg22( ski, eta, x1, x2, x3, ndim, xn )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.0  date 14-10-1994
!
!   copyright (c) 1994-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute coordinates of internal point ski,eta in case of spectral
!     quadrilateral element for Gauss-Legendre-Lobatto points while
!     along the boundary the cornerpoints and the midpoints have been
!     given and also the centroid.
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     mesh_generation
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision ski, eta, x1(9), x2(9), x3(9), xn(3)
      integer ndim

!     eta       i     second coordinate local system
!     ndim      i     dimension (2D or 3D)
!     ski       i     first coordinate local system
!     x1        i     first coordinates of nine basis points
!     x2        i     second coordinates of nine basis points
!     x3        i     third coordinates of nine basis points
!     xn        o     coordinates of transformed point (ski,eta)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision phi(9)

!     i          loop variable
!     phi        array to store values of the basis functions
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Deconcatenate name of subroutine from string
!     EROPEN  Concatenate name to string of calling subroutines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Run through all basis functions and determine the value for
!     point (ski,eta)
!     Compute the coordinates of point (ski,eta)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshg22' )

!     --- Run through all basis functions and determine value

      phi(1) = 0.25d0 * ski * (ski-1d0) * eta * (eta-1d0)
      phi(2) = 0.50d0 * (1d0-ski)*(1d0+ski)*eta*(eta-1d0)
      phi(3) = 0.25d0 * ski * (ski+1d0) * eta * (eta-1d0)
      phi(4) = 0.50d0 * ski * (ski+1d0) * (1d0-eta)*(1d0+eta)
      phi(5) = 0.25d0 * ski * (ski+1d0) * eta * (eta+1d0)
      phi(6) = 0.50d0 * (1d0-ski)*(1d0+ski)*eta*(eta+1d0)
      phi(7) = 0.25d0 * ski * (ski-1d0) * eta * (eta+1d0)
      phi(8) = 0.50d0 * ski * (ski-1d0) * (1d0-eta)*(1d0+eta)
      phi(9) =    (ski-1d0)*(ski+1d0)*(eta-1d0)*(eta+1d0)

!     --- Compute the coordinates of the point (ski,eta) in the original
!         setting

      xn(1) = 0d0
      xn(2) = 0d0
      xn(3) = 0d0

      do 100 i = 1 , 9

               xn(1) = xn(1) + x1(i) * phi(i)
               xn(2) = xn(2) + x2(i) * phi(i)

          if ( ndim.eq.3 ) xn(3) = xn(3) + x3(i) * phi(i)

100   continue

      call erclos ( 'mshg22' )

      end
