      subroutine msho72 ( npx, npy, npz, kelem, npelm )
! ======================================================================
!
!        programmer     Niek Praagman
!        version 1.0    date   28-06-1996
!
!
!
!   copyright (c) 1996-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     MSHO72 constructs a grid of tetrahedrons or hexahedrons if only
!     the nodes in a 3d domain are given in a structured way.
!     It is assumed that the points are given in NPZ planes of NPY rows
!     each of NPX points
!     So the first plane contains
!       the first row in the x-direction is point 1 to point NPX, &
!       the second row is NPX+1 till 2*NPX and so on until the last
!       row in the first plane which is (NPY-1) * NPX + 1 to NPY * NPX.
!     This goes on for in total NPZ planes.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     3d
!     structured
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer npx, npy, npz, kelem(*), npelm

!     kelem   o    array containing the nodes of the triangles.
!     npelm   i    number of points in element
!                  4 : linear tetrahedron
!                  8 : linear hexahedron
!     npx     i    number of points on lines in x-direction
!     npy     i    number of points on lines in y-direction
!     npz     i    number of points on lines in z-direction
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i1, i2, i3, i4, i5, i6, i7, i8, ielem, &
              jp, jc, jlm, jr, ivlako, ivlakp

!     i1           first node number of local hexahedron
!     i2           second node number of local hexahedron
!     i3           third node number of local hexahedron
!     i4           fourth nodal number of local hexahedron
!     i5           fifth nodal number of local hexahedron
!     i6           sixth nodal number of local hexahedron
!     i7           seventh nodal number of local hexahedron
!     i8           eigth nodal number of local hexahedron
!     ielem        temporary number of elements
!     ivlako       number of points used in all planes before
!                  present plane
!     ivlakp       number of points used in all planes present
!                  plane included
!     jc           column number considered
!     jlm          position in array kelem to be filled
!     jp           plane number considered
!     jr           row number considered
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Run through all planes in z-direction
!
!     Run through all lines in y-direction
!
!     Run through all points in x-direction
!
!     Create tetrahedrons or hexahedrons in kelem
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      ielem = 0

!     --- Run through all planes

      do 300 jp = 1 , npz - 1

         ivlako = ( jp - 1 ) * npx * npy
         ivlakp =              npx * npy

!        --- Run through all rows

         do 200 jr = 1 , npy - 1

!        --- Run through all columns

            do  100 jc = 1 , npx - 1

!           --- Determine eight corner nodes

                i1 = ( jr-1 ) * npx + jc
                i2 = i1 + 1
                i3 = i1 + npx
                i4 = i3 + 1

                i1 = i1 + ivlako
                i2 = i2 + ivlako
                i3 = i3 + ivlako
                i4 = i4 + ivlako

                i5 = i1 + ivlakp
                i6 = i2 + ivlakp
                i7 = i3 + ivlakp
                i8 = i4 + ivlakp

!               --- Fill elements in kelem

                if ( npelm.eq.4 ) then

                     jlm = 4 * ielem

                     kelem( jlm +  1 )   =   i1
                     kelem( jlm +  2 )   =   i2
                     kelem( jlm +  3 )   =   i4
                     kelem( jlm +  4 )   =   i8

                     kelem( jlm +  5 )   =   i1
                     kelem( jlm +  6 )   =   i6
                     kelem( jlm +  7 )   =   i2
                     kelem( jlm +  8 )   =   i8

                     kelem( jlm +  9 )   =   i1
                     kelem( jlm + 10 )   =   i5
                     kelem( jlm + 11 )   =   i6
                     kelem( jlm + 12 )   =   i8

                     kelem( jlm + 13 )   =   i1
                     kelem( jlm + 14 )   =   i4
                     kelem( jlm + 15 )   =   i3
                     kelem( jlm + 16 )   =   i8

                     kelem( jlm + 17 )   =   i1
                     kelem( jlm + 18 )   =   i3
                     kelem( jlm + 19 )   =   i7
                     kelem( jlm + 20 )   =   i8

                     kelem( jlm + 21 )   =   i1
                     kelem( jlm + 22 )   =   i7
                     kelem( jlm + 23 )   =   i5
                     kelem( jlm + 24 )   =   i8

                     ielem = ielem + 6

                else if ( npelm.eq.8 ) then

                    jlm = 8 * ielem

                    kelem( jlm + 1 ) = i1
                    kelem( jlm + 2 ) = i2
                    kelem( jlm + 3 ) = i4
                    kelem( jlm + 4 ) = i3
                    kelem( jlm + 5 ) = i5
                    kelem( jlm + 6 ) = i6
                    kelem( jlm + 7 ) = i8
                    kelem( jlm + 8 ) = i7

                    ielem = ielem + 1

                end if

100         continue

200      continue

300   continue

      end
