      subroutine msho86 ( kstap , kstapl, coor , npoint, nexpnt, &
                          klaar , jb    )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.1  date 11-09-2000 remove random searches
!        version  3.0  date 29-04-1998 random searches, try every
!                                      point (BvR)
!        version  2.0  date 25-02-1997 extra searches internal points
!        version  1.0  date 15-09-1996
!
!   copyright (c) 1996-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Determine the triangles and nodes of the multihedron that can not
!     be divided in tetrahedrons. If possible give the coordinates of
!     the internal point that is added to the nodal set.
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     mesh_generation
!     tetrahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kstap, kstapl( 3, kstap ), npoint, nexpnt, &
              klaar(*), jb
      double precision coor(3,*)

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     jb      i    indicator whether in meshcreation (MSHG3D, jb = 0) or
!                  in pointset (MSH476, jb = 1)
!     klaar        workarray of length npoint for detecting that
!                  all work is done
!     kstap  i,o   number of triangles constituting the actual
!                  boundary : at output this variable should be 0
!     kstapl  i    helparray for actual boundary of region, &
!                  at input containing all boundary triangles, &
!                  and during the process the actual boundary hence
!                  at output empty
!     nexpnt i,o   number of extra points added to set of points
!     npoint  i    total number of points in mesh.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ia, ib, ic, ja, ih, inodnr, itrinr
      double precision deel, xmin, xc, xp, yc, yp, zc, zp

!     deel      helpvariable to store number of nodes
!     i         loop-variable
!     ia        first node of triangle
!     ib        second node of triangle
!     ic        third node of triangle
!     ih        helpnode
!     inodnr    node number used to generate new node with
!     itrinr    triangle in kstap of which inodnr indicates the node nr
!     ja        indicator for internal points
!     xc        x-coordinate of centre of unsolvable region
!     xmin      smallest x-coordinate
!     xp        x-coordinate of point
!     yc        y-coordinate of centre of unsolvable region
!     yp        y-coordinate of point
!     zc        z-coordinate of centre of unsolvable region
!     zp        z-coordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    deconcatenate string of calling subroutines
!     EROPEN    concatenate string of calling subroutines
!     ERREAL    Put real in error message
!     ERRSUB    Error messages
!     ERRWAR    Print warnings
!     INSTOP    stop execution of SEPRAN
!     MSH607    check whether point is inside or outside volume
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     997   triangles form a surface for which no division in tetrahedrons is
!           possible
!     998   An extra point is added to the set of nodes
!     999   Routine could not determine a way to find an adequate node position
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Run through all nodes and determine the number of internal
!     nodes and the smallest x-value
!     Check whether new point is inside area
!     Give nodes of triangles and coordinates of nodes
!     If possible give indication for new node and if allowed add
!     point to dataset to start again
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'msho86' )

!     --- Run through all nodes and determine the number of internal
!         nodes

      deel   = 0d0
      xc     = 0d0
      yc     = 0d0
      zc     = 0d0

      xmin   = rinfin

      do i = 1 , npoint + nexpnt

         if ( klaar(i).ne.0 ) then

              deel   = deel   + 1d0

              xc = xc + coor( 1, i )
              yc = yc + coor( 2, i )
              zc = zc + coor( 3, i )

              if ( coor( 1, i ).lt.xmin ) xmin = coor(1,i)

         end if

      end do

      deel = 1d0 * deel

      xc  =  xc / deel
      yc  =  yc / deel
      zc  =  zc / deel

      xmin = xmin - 100d0

!     --- Check whether new point is inside area

      call msh607( xc, yc, zc, xmin, coor, kstapl, kstap, ja )

      if ( ioutp.ge.1 .and. jb.eq.1 ) then
         call errwar ( 997, 0, 0, 0 )

         do i = 1 , kstap

            ia =  kstapl( 1, i ) - 8
            ib =  kstapl( 2, i ) - 8
            ic =  kstapl( 3, i ) - 8

            write(irefwr,302) ia,ib,ic
302         format(1x,3i8)

         end do

         write(irefwr,310)
310      format ( ' The coordinates of the multihedron nodes are '/ &
                  '   Node                   co-ordinates' )

         do i = 1 , npoint + nexpnt

            if ( klaar(i).gt.0 ) then

               xp = coor( 1, i )
               yp = coor( 2, i )
               zp = coor( 3, i )

               write(irefwr,352) i-8,xp,yp,zp
352            format(1x,i6,3f20.4)

            end if

         end do

      end if

      if ( ja.ne.0 ) goto 500

      do itrinr = 1, kstap

         do inodnr=1,3

            if ( ja.eq.0 ) then

!           --- Try in a different way with first node

               ih     = kstapl(inodnr,itrinr)
               deel   = 0d0
               xc     = 0d0
               yc     = 0d0
               zc     = 0d0

               do i=1,kstap

                  ia = kstapl(1,i)
                  ib = kstapl(2,i)
                  ic = kstapl(3,i)

                  if ( ia.eq.ih .or. ib.eq.ih .or. ic.eq.ih ) then

!                 --- Neighbour triangle

                     deel   = deel   + 1d0

                     xc = xc + coor(1,ia) + coor(1,ib) + coor(1,ic)
                     yc = yc + coor(2,ia) + coor(2,ib) + coor(2,ic)
                     zc = zc + coor(3,ia) + coor(3,ib) + coor(3,ic)

                  end if

               end do

               deel = 3d0 * deel

               xc = xc / deel
               yc = yc / deel
               zc = zc / deel

!              --- Check whether this point is inside area

               call msh607( xc, yc, zc, xmin, coor, kstapl, kstap, ja )
               if ( ja.ne.0 ) go to 500

            end if
         end do
      end do

 500  if ( ioutp.ge.1 .and.  ja.eq.1 .and. jb.eq.1 ) then

         call erreal ( xc, 1 )
         call erreal ( yc, 2 )
         call erreal ( zc, 3 )
         call errwar ( 998, 0, 3, 0 )

      end if

      if ( ja.eq.0 ) then

         call errsub ( 999, 0, 0, 0 )
         go to 1000

      end if

      nexpnt = nexpnt + 1

      coor( 1, npoint + nexpnt ) = xc
      coor( 2, npoint + nexpnt ) = yc
      coor( 3, npoint + nexpnt ) = zc

1000  call erclos( 'msho86' )
      if ( ierror.ne.0 ) call instop

      end
