      subroutine mshobsfledge2 ( iobgrid, nx, ny, i, numcells, &
                                 icurpart, iwork, beta1, beta2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-10-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute intersection of part of obstacle with x = ....
!     Part parallel to y-axis
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, iobgrid(0:nx-1,0:ny-1), i, numcells(2,*), &
              iwork(2,*), icurpart
      double precision beta1, beta2

!     beta1          i    parameter beta1 to define a line
!     beta2          i    parameter beta2 to define a line
!     i              i    Column number of cells that is considered
!     icurpart       i    curve part sequence number
!     iobgrid       i/o   Is filled with the number of curve parts that
!                         intersect cell i,j
!     iwork         i/o   Contains the cell numbers (i,j) in the sequence
!                         of the curve parts
!                         Must be used in connection with numcells
!     numcells      i/o   numcells(1,k) contains the number of cells curvepart
!                         k intersects
!                         numcells(1,k) contains the starting address of
!                         the cell numbers in iwork
!     nx             i    number of points of grid in x direction
!     ny             i    number of points of grid in y direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j1, j2, k, ksign

!     j1             Lower index number along second direction of a face
!     j2             Upper index number along second direction of a face
!     k              Counting variable
!     ksign          Step in k loop
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSFLEDGE3  Compute intersection of part of obstacle with x = ....
!                    Part parallel to y-axis
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobsfledge2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobsfledge2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      j1     = min(int(max(beta1,0d0)),ny-1)
      j2     = min(int(max(beta2,0d0)),ny-1)
      if ( j2>j1 ) then
         ksign = 1
      else
         ksign = -1
      end if

      do k = j1, j2, ksign

         call mshobsfledge3 ( iobgrid, nx, ny, i, k, numcells, &
                              icurpart, iwork )
         if ( ierror/=0 ) go to 1000

      end do  ! k = j1, j2, ksign

1000  call erclos ( 'mshobsfledge2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobsfledge2'

      end if  ! ( debug )

      end
