      subroutine mshobstac3 ( nspecialpnts, specialpoints, istore, &
                              isequence, icurvobs, curves, icurnr, &
                              work, iwork, icurvs, nskip )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-08-1999
!
!   copyright (c) 1999-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Find the natural sequence of the special points along the obstacle
!   Skip superfluous special points along the obstacle
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nspecialpnts, specialpoints(7,*), istore(*), nskip, &
              isequence(*), icurvobs(2,*), icurnr, iwork(*), icurvs(*)
      double precision work(*), curves(2,*)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inext, iseq, ivalue, istart, indspecialpnt, jcount, i, &
              inext1, inext2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Reorder special points so that they are stored along the curve in
!     natural sequence
!
!     First store the corresponding indices with respect to the curve
!     in array istore
!     Sort indices with the same values using the coordinates
!     inext := 2
!     iseq := isequence(1)
!     ivalue := istore(iseq)
!     jcount := 0
!     istart := 1
!     while ( inext<=nspecialpnts ) do
!        Perform the loop until all points are ordered
!        iseq := isequence(inext)
!        if ( istore(iseq)>ivalue ) then
!           if ( jcount>0 ) then
!              Sort the values istart to inext-1
!           ivalue := istore(iseq)
!           istart := inext
!           jcount := 0
!        else ( istore(iseq)=ivalue )
!           jcount := 1
!        inext := inext+1
!     if ( jcount>0 ) then
!        Sort the values istart to nspecialpnts
!
!     Next skip superfluous points
!     A point is defined superfluous if it is a special point with an
!     active obstacle on both sides
!
!     inext := 1
!     while ( inext<=nspecialpnts ) do
!        iseq := isequence(inext)
!        inext1 := specialpoints(3,iseq)
!        inext2 := specialpoints(5,iseq)
!        if ( inext1=0 and inext2=0 ) then
!           inext := inext+1
!        else if ( inext1#0 and isequence(inext1)#iseq+1 or
!                  inext2#0 and isequence(inext2)#iseq+1 ) then
!           inext := inext+1
!        else (following point corresponds to this one )
!           check if points may be skipped
!           raise inext
!     Remove parts to be skipped
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Reorder special points so that they are stored along the curve in
!         natural sequence
!         First store the corresponding indices with respect to the curve
!         in array istore

      do i = 1, nspecialpnts
         indspecialpnt = specialpoints(1,i)
         indspecialpnt = icurvobs(2,indspecialpnt)
         istore(i) = indspecialpnt
      end do

!     --- Next store the indices in increasing sequence in isequence

      call chsort ( istore, isequence, nspecialpnts )

!     --- Sort indices with the same values using the coordinates

      inext = 2
      iseq = isequence(1)
      ivalue = istore(iseq)
      jcount = 0
      istart = 1
      do while ( inext.le.nspecialpnts )

!     --- Perform the loop until all points are ordered

         iseq =  isequence(inext)
         if ( istore(iseq).gt.ivalue ) then

!        --- The new value is larger, hence the list of equal values has been
!            found

            if ( jcount.gt.0 ) then

!           --- jcount > 0, the values istart to inext-1 must be sorted

               call mshobstac4 ( isequence, istart, inext-1, &
                                 specialpoints, icurvobs, curves, &
                                 icurnr, work, iwork, icurvs  )

            end if

!           --- raise inext and recompute ivalue

            ivalue = istore(iseq)
            istart = inext
            jcount = 0

         else

!        --- The new value is the same, raise inext

            jcount = 1

         end if
         inext = inext+1

      end do
      if ( jcount.gt.0 ) then

!     --- jcount > 0, the values istart to nspecialpnts must be sorted

         call mshobstac4 ( isequence, istart, nspecialpnts, &
                           specialpoints, icurvobs, curves, icurnr, &
                           work, iwork, icurvs  )

      end if

!     --- Next skip superfluous points
!         A point is defined superfluous if it is a special point with an
!         active obstacle on both sides

      inext = 1
      do while ( inext.le.nspecialpnts )

!     --- Loop over all entries

         iseq = isequence(inext)
         inext1 = specialpoints(3,iseq)
         inext2 = specialpoints(6,iseq)
         if ( inext1.eq.0 .and. inext2.eq.0 ) then

!        --- inext1=0 and inext2=0

            inext = inext+1

         else if ( inext1.ne.0 ) then

!        --- inext1 # 0

            if ( isequence(inext1).eq.inext+1 ) then

!           --- Next point corresponds to same part, check the following points

               call mshobstac5 ( inext, isequence, specialpoints, &
                                 nspecialpnts )

            else if ( inext2.ne.0 ) then

               if ( isequence(inext2).eq.inext+1 ) then

!              --- Next point corresponds to same part, check the following
!                  points

                  call mshobstac5 ( inext, isequence, specialpoints, &
                                    nspecialpnts )

               else

!              --- Next point does not correspond to same part

                  inext = inext+1

               end if

            else

!           --- Next point does not correspond to same part

               inext = inext+1

            end if

         else

!        --- inext2 # 0

            if ( isequence(inext2).eq.inext+1 ) then

!           --- Next point corresponds to same part, check the following points

               call mshobstac5 ( inext, isequence, specialpoints, &
                                 nspecialpnts )

            else if ( inext1.ne.0 ) then

               if ( isequence(inext1).eq.inext+1 ) then

!              --- Next point corresponds to same part, check the following
!                  points

                  call mshobstac5 ( inext, isequence, specialpoints, &
                                    nspecialpnts )

               else

!              --- Next point does not correspond to same part

                  inext = inext+1

               end if

            else

!           --- Next point does not correspond to same part

               inext = inext+1

            end if

         end if

      end do

!     --- Remove points to be skipped

      jcount = 0
      do i = 1, nspecialpnts
         iseq = isequence(i)
         if ( iseq.ne.0 ) then
            jcount = jcount+1
            isequence(jcount) = iseq
         end if
      end do
      nskip = nspecialpnts-jcount

      end
