      subroutine mshobstact ( nspecialpnts, &
                              specialpoints, newcurves, nextracurves, &
                              infocurve, iinput, ipointcurv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 08-11-2000 Clear arrar istnewcurves
!        version  1.0  date 03-04-1999
!
!   copyright (c) 1999-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Subroutine to detect which actions must be undertaken with respect to
!    subdivision of curvesmsh and recreation of new surfaces in case of obstacles
!    that have become active
! **********************************************************************
!
!                       KEYWORDS
!
!     obstacle
!     boundary
!     curve
!     free_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nextracurves, nspecialpnts, &
              specialpoints(7,*), newcurves(*), infocurve(*), &
              ipointcurv(*), iinput(*)

!     iinput         i    integer input array
!     infocurve      i    Array of length ncurvs containing information about
!                         which curve is on a free surface and also
!                         part of an obstacle
!                         infocurve(i) = 0: standard curve
!                         infocurve(i) = iobst(>0): curve on free surface and
!                              also new part of obstacle iobst. The curve is not
!                              of type >= 10000
!                         infocurve(i) = -1: curve on obstacle that is made
!                         active
!     ipointcurv     i    Array containing the relative starting addresses
!                         of all curvesmsh in the part of iinput referring
!                         to curvesmsh
!     newcurves      o    Contains information about the new curvesmsh to be
!                         created by splitting old curvesmsh
!                         The starting addresses of the information per curve is
!                         stored in array istnewcurves
!                         For each curve that is stored in newcurves 2*nparts+2
!                         positions are used, where nparts is the number of
!                         parts the curve is subdivided into
!                         The first position contains nparts, the following
!                         2*nparts positions the couples relative starting
!                         addresses of the parts in the curve and corresponding
!                         special points
!     nextracurves   o    Number of extra curvesmsh to be created
!     nspecialpnts   o    Number of special points
!     specialpoints  o    This array of size 7 x nspecialpnts contains
!                         information about the special points on the curve
!                         specialpoint(1,i) contains the position in curvesmsh
!                                           of special point i
!                         specialpoint(2,i) contains the curve number in the
!                                           free surface connected to point i
!                         specialpoint(3,i) contains the corresponding special
!                                           point on the the curve
!                                           The space between is obstacle
!                                           If 0 the point is stand alone
!                         specialpoint(4,i) contains the corresponding special
!                                           point on the the second curve
!                                           If 0 there is no second curve
!                         specialpoint(5,i) contains the second curve number in
!                                           the free surface connected to point
!                         specialpoint(6,i) contains the corresponding special
!                                           point on the the curve
!                         specialpoint(7,i) contains the corresponding user
!                                           point number
!                                           if 0 no user point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icurnr, iobst, i, &
              newcurvesstrt, nskip, isub, nsubcv, isubcv, &
              istartsubcv, lencurvs
      double precision, allocatable :: work(:)
      integer, allocatable :: specialusp(:), isequence(:), iwork(:)

!     i              Counting variable
!     icurnr         Curve sequence number
!     iobst          Obstacle sequence number
!     isequence
!     istartsubcv    ?
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isubcv         Subcurve sequence number with direction
!     iwork          integer work array
!     lencurvs       Length of array curvesmsh
!     newcurvesstrt  Defines the next free position in newcurves
!     nskip          ?
!     nsubcv         Number of subcurvesmsh of a curve
!     specialusp
!     work           Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSTAC1     Split curve on free surface that is part of obstacle into
!                    several curvesmsh
!     MSHOBSTAC2     ?
!     MSHOBSTAC3     ?
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    For all obstacles iobst that are also part of a surface do
!       For all curvesmsh icurnr that are in a free surface do
!          If ( infocurve(icurnr)=iobst ) then
!             Curve found that must be subdivided
!             Store information of curve and special points
!                A special point is defined as the start or end point of a
!                part of the free surface that is on the obstacle
!       For all special points do
!          Find position and subcurve number on obstacle
!          Subdivide subcurves on obstacle
!    redefine boundary of outer surfaces
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstact' )

!     --- For all obstacles iobst that are also part of a surface do

      nextracurves  = 0
      newcurvesstrt = 1
      nspecialpnts  = 0

      lencurvs = icurvsmsh(ncurvs+1)

      allocate ( work(lencurvs), specialusp(nuspnt), &
                 isequence(2*lencurvs), iwork(2*lencurvs), stat = error )
      if ( error/=0 ) call eralloc ( error, lencurvs+nuspnt, 'work' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array specialusp

      specialusp(1:nuspnt) = 0

!     --- Clear array istnewcurves

      do i = 1, ncurvs
         newcurves(i) = 0
      end do

      do iobst = 1, numobst

         icurnr = kmeshx(2,iobst)
         if ( kelmq==0 ) then
            ishape = iinput(ipointcurv(icurnr)+1)
         else
            istartsubcv = kmeshq(icurnr)
            nsubcv = kmeshq(icurnr+1)-istartsubcv
            ishape = 10000
            do isub = 1, nsubcv
               isubcv = abs(kmeshq(istartsubcv+isub))
               if ( ipointcurv(isubcv)>0 ) then
                  ishape = min(ishape,iinput(ipointcurv(isubcv)+1))
               end if
            end do
         end if
         if ( kmeshx(6,iobst)>0 ) then

!        --- Obstacle iobst is part of surface
!            For all curvesmsh icurnr that are in a free surface do

            do icurnr = 1, ncurvs
               if ( infocurve(icurnr)==iobst ) then

!              --- infocurve(icurnr)=iobst
!                  Curve found that must be subdivided
!                  Store information of curve and special points
!                  A special point is defined as the start or end point of a
!                  part of the free surface that is on the obstacle

                  call mshobstac1 ( iwork, icurvsmsh, icurvsobst2d, &
                                    nspecialpnts, specialpoints, &
                                    newcurves, newcurves(ncurvs+1), &
                                    newcurvesstrt, nextracurves, icurnr, &
                                    iobst, curveusrp, specialusp )

!                 --- Check newcurvesstrt and nspecialpnts

               end if

            end do

!           --- Find the natural sequence of the special points along the
!               obstacle
!               Skip superfluous special points along the obstacle

            call mshobstac3 ( nspecialpnts, specialpoints, &
                              iwork(nspecialpnts+1), &
                              isequence, icurvsobst2d, curvesmsh, icurnr, &
                              work, iwork, icurvsmsh, nskip )

!           --- For all special points do
!               Find position and subcurve number on obstacle
!               Subdivide subcurvesmsh on obstacle

            call mshobstac2 ( iobst, nspecialpnts, &
                              nskip, specialpoints, isequence, icurvsobst2d, &
                              newcurvesstrt, nextracurves, &
                              newcurves(ncurvs+1), newcurves, iwork )

         end if

      end do

      deallocate ( work, isequence, specialusp, iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'work' )

1000  call erclos ( 'mshobstact' )
      end
