      subroutine mshobstiin2 ( iinput, rinput, iinputnew, rinputnew, &
                               specialpoints, iposrinp, iposiinp, &
                               infocurve, &
                               istnewcurves, nextracurves, newcurves, &
                               icurvsnew, curvesnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 02-09-2010 Remove intarmsh
!        version  1.0  date 15-08-1999
!
!   copyright (c) 1999-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Copy parts of iinput and rinput with respect to curves from input into
!   iinputnew and from rinput to rinputnew
!   Change contents of curves if necessary and create new curves
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer specialpoints(7,*), iinput(*), &
              iinputnew(*), iposrinp, nextracurves, &
              infocurve(*), iposiinp, &
              istnewcurves(*), newcurves(*), icurvsnew(*)
      double precision rinputnew(*), rinput(*), curvesnew(2,*)

!     curvesnew      i    ?
!     icurvsnew      i    ?
!     iinput         i    integer input array
!     iinputnew      i    ?
!     infocurve      i    Array of length ncurvs containing information about
!                         which curve is on a free surface and also
!                         part of an obstacle
!                         infocurve(i) = 0: standard curve
!                         infocurve(i) = iobst(>0): curve on free surface and
!                              also new part of obstacle iobst. The curve is not
!                              of type >= 10000
!                         infocurve(i) = -1: curve on obstacle that is made
!                              active
!     iposiinp       i    Starting address in array iinput with respect to the
!                         extra user points (internal points)
!     iposrinp       i    ?
!     istnewcurves   i    Array of length ncurvs containing the starting
!                         addresses of all curves in array NEWCURVES
!                         if istnewcurves(i)=0, no information of the curve is
!                         stored, hence the curve is not changed
!                         if istnewcurves(i)>0, the value is the starting
!                         address of curve i
!     newcurves      i    Contains information about the new curves to be
!                         created by splitting old curves
!                         The starting addresses of the information per curve is
!                         stored in array istnewcurves
!                         For each curve that is stored in newcurves nparts+1
!                         positions are used, where nparts is the number of
!                         parts the curve is subdivided into
!                         The first position contains nparts, the following
!                         nparts positions the relative starting addresses
!                         of the parts in the curve
!     nextracurves   i    Number of extra curves to be created
!     rinput         i    Real input array
!     rinputnew      i    ?
!     specialpoints  i    This array of size 3 x nspecialpnts contains
!                         information about the special points on the curve
!                         specialpoints(1,i) contains the position in curves
!                                           of special point i
!                         specialpoints(2,i) contains first curve number in the
!                                           free surface connected to point i
!                         specialpoints(3,i) contains second curve number in the
!                                           free surface connected to point i
!                                           If 0, there is only one curve
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icurnrnew, irnp, &
              iuserp1, iuserp2, ncopy, istartold, &
              istart, nsubcv, itype, inpncv, nparts, j, idirec, &
              iposcurves, istartcurv, nelm, iwork(5,nextracurves), &
              ipointcurvnw(ncurvs+nextracurves)

!     i              Counting variable
!     icurnrnew      New curve sequence number
!     idirec         Defines in which direction the curves must be
!                    considered
!                    idirec = 1, from low to high, idirec = -1: reversed
!                    order
!     inpncv         Number of points in curve
!     ipointcurvnw   ?
!     iposcurves     ?
!     irnp           position in array rinput
!     istart         First position used at a part
!     istartcurv     ?
!     istartold      Old starting address in array
!     itype          Table containing information about the numbers behind
!                    the keywords in the table code
!     iuserp1        First user point
!     iuserp2        Second user point
!     iwork          integer work array
!     j              Counting variable
!     ncopy          ?
!     nelm           number of elements along a curve
!     nparts         number of parts the boundary consists of
!     nsubcv         Number of subcurves of a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    For all curves icurnr in iinput do
!       if ( istnewcurves(icurnr) = 0 ) then
!          Copy information from iinput to iinputnew and rinputnew
!       else
!          Compute number of new curves and store these curves in iinputnew
!          Store information of new curves into iwork
!    Copy all information from iwork into iinputnew
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- initializations

      ncurvs = iinput(3)
      istartold = iinp1
      iposiinp = istartold
      icurnrnew = 0

!     --- Fill first part of arrays icurvsnew and curvesnew

      do i = 1, ncurvs
         icurvsnew(i) = icurvsmsh(i)
      end do
      do i = 1, icurvsmsh(ncurvs)
         curvesnew(1,i) = curvesmsh(1,i)
         curvesnew(2,i) = curvesmsh(2,i)
      end do
      iposcurves = icurvsmsh(ncurvs)

!     --- For all curves icurnr in iinput do

      do icurnr = 1, ncurvs

         istartold = ipointcurv(icurnr)
         ipointcurvnw(icurnr) = iposiinp
         if ( istnewcurves(icurnr)==0 ) then

!        --- istnewcurves(icurnr) = 0
!            Copy information from iinput to iinputnew and rinputnew

            if ( icurnr<ncurvs ) then
               ncopy = ipointcurv(icurnr+1)-istartold
            else
               ncopy = iinp3-istartold
            end if
            do i = 0, ncopy-1
               iinputnew(iposiinp+i) = iinput(istartold+i)
            end do
            iposiinp = iposiinp+ncopy

         else

!        --- Compute number of new curves and store these curves in iinputnew
!            Store information of new curves into iwork

            if ( icurnr==1 ) then
               inpncv = icurvsmsh(1)
            else
               inpncv = icurvsmsh(icurnr)-icurvsmsh(icurnr-1)
            end if
            istart = abs(istnewcurves(icurnr))
            nparts = newcurves(istart)

!           --- Curve is composite curve (type 7)

            iinputnew(iposiinp) = 7
            iinputnew(iposiinp+1) = 1
            iinputnew(iposiinp+2) = 0
            if ( newcurves(istart+2*nparts-1)<inpncv ) then
               nsubcv = nparts
            else
               nsubcv = nparts-1
            end if
            iinputnew(iposiinp+3) = nsubcv

            if ( istnewcurves(icurnr)>0 ) then

!           --- Subcurves are stored in forward direction

               do i = 1, nsubcv
                  iinputnew(iposiinp+3+i) = icurnrnew+i+ncurvs
               end do

            else

!           --- Subcurves are stored in backward direction

               do i = 1, nsubcv
                  iinputnew(iposiinp+4+nsubcv-i) = -icurnrnew-i-ncurvs
               end do

            end if
            iposiinp = iposiinp+4+nsubcv

!           --- Fill information of new curves into iwork

            if ( infocurve(icurnr)==0 ) then
               itype = 10000
            else if ( infocurve(icurnr)==-1 ) then
               itype = 10002
            else
               itype = 10001
            end if
            ishape = iinput(istartold+1)
            if ( istnewcurves(icurnr)>0 ) then

!           --- Curve is used in positive direction

               iuserp1 = curveusrp(curveusrp(icurnr)+1)
               iuserp2 = curveusrp(curveusrp(icurnr+1))
               if ( icurnr==1 ) then
                  istartcurv = 0
               else
                  istartcurv = icurvsmsh(icurnr-1)
               end if
               idirec = 1

            else

!           --- Curve is used in negative direction

               iuserp2 = curveusrp(curveusrp(icurnr)+1)
               iuserp1 = curveusrp(curveusrp(icurnr+1))
               istartcurv = icurvsmsh(icurnr)+1
               idirec = -1

            end if
            do i = 1, nparts-1
               iwork(1,icurnrnew+i) = itype
               iwork(2,icurnrnew+i) = ishape
               nelm = newcurves(istart+2*i+1)-newcurves(istart+2*i-1)
               icurvsnew(icurnrnew+ncurvs+i) = &
                  nelm+1+icurvsnew(icurnrnew+ncurvs+i-1)
               do j = 1, nelm+1
                  curvesnew(1,iposcurves+j) = &
                     curvesmsh(1,istartcurv+idirec*j)
                  curvesnew(2,iposcurves+j) = &
                     curvesmsh(2,istartcurv+idirec*j)
               end do
               iposcurves = iposcurves+nelm+1
               istartcurv = istartcurv+idirec*nelm
               nelm = nelm/ishape
               iwork(3,icurnrnew+i) = nelm
               iwork(4,icurnrnew+i) = iuserp1
               iuserp1 = specialpoints(7,newcurves(istart+2*i+2))
               iwork(5,icurnrnew+i) = iuserp1
            end do
            if ( nparts==nsubcv ) then

!           --- Curve to end of curve

               iwork(1,icurnrnew+nsubcv) = itype
               iwork(2,icurnrnew+nsubcv) = ishape
               nelm = inpncv-newcurves(istart+2*nparts-1)
               icurvsnew(icurnrnew+nsubcv+ncurvs) = &
                  nelm+1+icurvsnew(icurnrnew+ncurvs+nsubcv-1)
               do j = 1, nelm+1
                  curvesnew(1,iposcurves+j) = &
                     curvesmsh(1,istartcurv+idirec*j)
                  curvesnew(2,iposcurves+j) = &
                     curvesmsh(2,istartcurv+idirec*j)
               end do
               iposcurves = iposcurves+nelm+1
               istartcurv = istartcurv+idirec*nelm
               nelm = nelm/ishape
               iwork(3,icurnrnew+nsubcv) = nelm
               iwork(4,icurnrnew+nsubcv) = iuserp1
               iwork(5,icurnrnew+nsubcv) = iuserp2

            end if

!           --- Fill information into icurvsnew and curvesnew

            icurnrnew = icurnrnew+nsubcv

         end if

      end do

!     --- Correct type numbers 10001 in iwork that do not coincide with
!         curves of type 10002

      do i = 1, icurnrnew
         itype = iwork(1,i)
         if ( itype==10001 ) then

!        --- Curve on free surface, test if there is a curve on the obstacle
!            with the smae begin and end point

            iuserp1 = iwork(4,i)
            iuserp2 = iwork(5,i)
            iwork(1,i) = 10000
            do j = 1, icurnrnew
               itype = iwork(1,j)
               if ( itype==10002 ) then

!              --- Curve on obstacle

                  if ( iuserp1==iwork(4,j) .and. &
                       iuserp2==iwork(5,j) .or. &
                       iuserp1==iwork(5,j) .and. &
                       iuserp2==iwork(4,j) ) then

!                 --- Curve on obstacle equal to free surface curve

                     iwork(1,i) = 10001
                     iwork(1,j) = 10003
                     go to 100

                  end if
               end if
            end do
         end if
100   end do

!     --- Copy all information from iwork into iinputnew

      do i = 1, icurnrnew
         ipointcurvnw(ncurvs+i) = iposiinp+5*(i-1)
         do j = 1, 5
            iinputnew(iposiinp+(i-1)*5+j-1) = iwork(j,i)
         end do
      end do
      iposiinp = iposiinp+5*icurnrnew
      icurnrnew = icurnrnew+ncurvs
      iinputnew(3) = icurnrnew
      do i = 1, icurnrnew
         itype = iinputnew(ipointcurvnw(i))
         if ( itype==0 .or. itype==7 ) then
            icurvsnew(icurnrnew+i) = 1
         else
            icurvsnew(icurnrnew+i) = 0
         end if
      end do

!     --- Fill information in rinputnew
!         First factors

      irnp = irnp2
      do i = 0, ncurvs-1
         rinputnew(iposrinp+i) = rinput(irnp+i)
      end do
      do i = ncurvs, icurnrnew-1
         rinputnew(iposrinp+i) = 0d0
      end do
      iposrinp = iposrinp+icurnrnew
      irnp = irnp+ncurvs

!     --- Next tangents

      do i = 0, 2*ncurvs-1
         rinputnew(iposrinp+i) = rinput(irnp+i)
      end do
      do i = 2*ncurvs, 2*icurnrnew-1
         rinputnew(iposrinp+i) = 0d0
      end do
      iposrinp = iposrinp+2*icurnrnew
      irnp = irnp+2*ncurvs

!     --- Finally end of rinput

      nelm = irnp3-irnp
      do i = 0, nelm
         rinputnew(iposrinp+i) = rinput(irnp+i)
      end do
      iposrinp = iposrinp+nelm+1

      end
