      subroutine mshobstmarkel3 ( nextra, lstore, istore, nmeshh, iwork, &
                                  lenielemobst, ielemobstcp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 12-11-2004 Debug statements
!        version  1.0  date 22-03-2001
!
!   copyright (c) 2001-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store all elements that intersect the obstacle but do not have a node
!     inside the obstacle into ielemobstcp
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     element
!     obstacle
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nextra, lstore, istore(*), nmeshh(*), iwork(*), &
              lenielemobst, ielemobstcp(*)

!     ielemobstcp   i/o   Copy of ielemobst extended with extra elements
!                         In this subroutine the extra elements are added
!     istore         i    Work array of length nedges
!                         In this array all edges are stored that are candidates
!                         for a cross section with the obstacle, although their
!                         end points are not in the obstacle
!     iwork          i    integer work array of length nelem
!                         if ( iwork(ielem) = 0 element ielem may be outside
!                         the obstacle
!                         if ( iwork(ielem) = 1 element ielem is inside
!                         the obstacle
!     lenielemobst  i/o   Computed length of array ielemobstcp
!     lstore         i    Number of entries of istore
!     nextra         o    number of extra elements stored in this subroutine
!     nmeshh         i    nmesh part h
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iedge, jpstart, nelem_edge, j, jelem

!     i              Counting variable
!     iedge          edge sequence number
!     j              Counting variable
!     jelem          do loop variable with respect to the elements
!     jpstart        Starting address in array
!     nelem_edge     number of elements corresponding to edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstmarkel3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstmarkel3'
         call prinin ( istore, lstore, 'edges' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      nextra = 0

!     --- Loop over edges in istore

      do i = 1, lstore
         iedge = istore(i)
         jpstart = nmeshh(iedge)
         nelem_edge = nmeshh(jpstart)

!        --- Loop over the elements of the edge

         do j = 1, nelem_edge

!        --- element jelem

            jelem = nmeshh(jpstart+j)
            if ( iwork(jelem)==0 ) then

!           --- element outside the mesh, store edge in istore

               nextra = nextra+1
               lenielemobst = lenielemobst+1
               ielemobstcp(lenielemobst) = jelem
               iwork(jelem) = 1

            end if

         end do
      end do

1000  if ( debug ) then

!     --- Debug information

         call prinin ( ielemobstcp(lenielemobst-nextra+1), nextra, &
                       'extra elements' )
         write(irefwr,*) 'End mshobstmarkel3'

      end if
      call erclos ( 'mshobstmarkel3' )

      end
