      subroutine mshobstms01 ( ielemobst, numobst, numbnelmobst, &
                               icrossobst, numcross )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 11-10-2011 Extra debug
!        version  1.1  date 04-08-2001 Renamed mshobstmesh06
!        version  1.0  date 07-03-2001
!
!   copyright (c) 2001-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute some parameters for subroutine mshobstms
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ielemobst(*), numobst, numbnelmobst, icrossobst(*), &
              numcross

!     icrossobst     i    array containing information about the cross section
!                         of obstacles with edges
!                         The storage of icrossobst is as follows
!                         Pos. 1 to numobst (number of obstacles):
!                               icrossobst(iobst) = istartobst (starting address
!                               of obstacle iobst in array
!                         Per obstacle from position istartobst:
!                         Number of cross sections, followed by 2 positions for
!                         each cross section consisting of the edge sequence
!                         number and a reference to rcrossobst
!                         This part is followed by the number of elements
!                         that is intersected by the obstacle followed by
!                         for each element: element number, number of
!                         intersections and sequence numbers of intersections
!     ielemobst      i    Array containing information about the elements in an
!                         obstacle
!                         The first numobst positions contain the starting
!                         positions of each obstacle sequentially
!                         Per obstacle the first three positions contain
!                         1. The number of elements that are completely inside
!                            the obstacle
!                         2. The number of elements that are not completely
!                            inside the element but with all vertices inside
!                            the element
!                         3. The number of elements that are partly inside
!                            the obstacle
!                            elements that contain only boundary points of the
!                            obstacle are not considered
!                         These positions are followed by the element numbers
!                         in the sequence defined by the first three positions
!     numbnelmobst   o    Number of elements that are partly within the obstacle
!     numcross       o    Number of cross sections
!     numobst        i    number of obstacles
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iobst, istart

!     iobst          Obstacle sequence number
!     istart         First position used at a part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstms01' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstms01'
         write(irefwr,1) 'numobst', numobst
         call prinin ( ielemobst, numobst, 'ielemobst' )
         call prinin ( icrossobst, numobst, 'icrossobst' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- compute numbnelmobst

      numbnelmobst = 0
      do iobst = 1, numobst
         istart = ielemobst(iobst)
         numbnelmobst = numbnelmobst+ielemobst(istart+1)+ &
                        ielemobst(istart+2)
      end do
      if ( debug ) write(irefwr,1) 'numbnelmobst', numbnelmobst

!     --- compute numcross

      numcross = 0
      do iobst = 1, numobst
         istart = icrossobst(iobst)
         numcross = numcross+icrossobst(istart)
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'numcross', numcross
         write(irefwr,*) 'End mshobstms01'

      end if  ! ( debug )
      call erclos ( 'mshobstms01' )

      end
