      subroutine mshp00 ( curv1, curv2, curv3, curv4, linearlc, m, n )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  4.1  date 28-11-2014 Decrease accuracy
!        version  4.0  date 02-09-2010 Remove intarmsh
!        version  3.0  date 09-08-1999 Extra parameter intarmsh
!
!   copyright (c) 1988-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Check co-ordinates of boundary for pipe surface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision curv1(*), curv2(*), curv3(*), curv4(*)
      integer n, m, linearlc

!     curv1      i
!     curv2      i
!     curv3      i
!     curv4      i
!     linearlc   i    Indication if the elements to be generated
!                     are linear (1) or quadratic (2)
!     m          i    Parameter for mesh generation
!     n          i    Parameter for mesh generation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer n2, m2, mprow, npcol, k
      double precision dist, amax, eps

!     amax
!     dist       Distance between two points
!     eps        Local accuracy
!     k          Counting variable
!     m2
!     mprow
!     n2
!     npcol
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     592
!     593
!     630
!     631
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshp00' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshp00'
         write(irefwr,1) 'n, m, linearlc', n, m, linearlc
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set constants

      n2=linearlc*n
      m2=linearlc*m
      m2 = m2 + 1
      mprow=3*m2
      npcol=3*n2
      ivolnr = 0

      if ( debug ) then
         call prinrl ( curv1, mprow, 'curv1' )
         call prinrl ( curv2, mprow, 'curv2' )
         call prinrl ( curv3, 3+npcol, 'curv3' )
         call prinrl ( curv4, 3+npcol, 'curv4' )
      end if  ! ( debug )

!     --- Check if the input is correct
!         First point of curv3 must be equal to first point of curv1

      dist=0d0
      amax=0d0
      eps = 1d-3   ! check accuracy
      do k=1,3
         amax=max(amax,abs(curv3(k)-curv3(k+3)))
         dist=dist+abs(curv3(k)-curv1(k))
      end do
      if ( dist>eps*amax ) then
         call errint ( ivolnr, 1 )
         call errint ( isurnr, 2 )
         do k = 1, 3
            call erreal ( curv1(k), k )
            call erreal ( curv3(k), k+3 )
         end do
         call errsub ( 593, 2, 6, 0 )
      end if

!     --- Last point of curv3 must be equal to first point of curv2

      dist=0d0
      amax=0d0
      do k=1,3
         amax=max(amax,abs(curv2(k+3)-curv2(k)))
         dist=dist+abs(curv3(k+npcol)-curv2(k))
      end do
      if ( dist>eps*amax ) then
         call errint ( ivolnr, 1 )
         call errint ( isurnr, 2 )
         do k = 1, 3
            call erreal ( curv2(k), k )
            call erreal ( curv3(k+npcol), k+3 )
         end do
         call errsub ( 592, 2, 6, 0 )
      end if

!     --- First point of curv4 must be equal to last point of curv1

      dist=0d0
      amax=0d0
      mprow = mprow - 3
      do k=1,3
         amax=max(amax,abs(curv4(k)-curv4(k+3)))
         dist=dist+abs(curv4(k)-curv1(k+mprow))
      end do
      if ( dist>eps*amax ) then
         call errint ( ivolnr, 1 )
         call errint ( isurnr, 2 )
         do k = 1, 3
            call erreal ( curv4(k), k )
            call erreal ( curv1(k+mprow), k+3 )
         end do
         call errsub ( 630, 2, 6, 0 )
      end if

!     --- Last point of curv4 must be equal to last point of curv2

      dist=0d0
      amax=0d0
      do k=1,3
         amax=max(amax,abs(curv4(k+npcol)-curv4(k+npcol-3)))
         dist=dist+abs(curv4(k+npcol)-curv2(k+mprow))
      end do
      if ( dist>eps*amax ) then
         call errint ( ivolnr, 1 )
         call errint ( isurnr, 2 )
         do k = 1, 3
            call erreal ( curv4(k+npcol), k )
            call erreal ( curv2(k+mprow), k+3 )
         end do
         call errsub ( 631, 2, 6, 0 )
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshp00'

      end if  ! ( debug )
      call erclos ( 'mshp00' )

      end subroutine mshp00
