      subroutine mshp05 ( kmeshc, m, n, inpelm, nelem, ishape, idiag )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.2  date 22-05-2014 Debug statements
!        version  3.1  date 05-05-2002 Extension with 7-point elements
!        version  3.0  date 05-11-1997 Alternate diagonal direction (BvR)
!
!   copyright (c) 1988-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill elements in kmesh part c for pipe surface
!    Two generating curves
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(*), m, n, inpelm, nelem, ishape, idiag

!     idiag      i    Defines diagonal direction:
!                     0: standard (counter clockwise)
!                     1: clockwise
!                     2: alternating
!     inpelm     i    number of points per element
!     ishape     i    Shape number of elements
!     kmeshc    i/o   kmesh part c, contains the nodal points corresponding
!                     to the elements
!     m          i    Parameter for mesh generation
!     n          i    Parameter for mesh generation
!     nelem      i    Number of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, m1, ikelm, ih, j

!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     ikelm
!     j              Counting variable
!     m1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshp05' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshp05'
         write(irefwr,1) 'm, n, inpelm, idiag', m, n, inpelm, idiag
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      m1 = m+1
      if ( inpelm==3 .and. idiag==0 ) then

!     --- inpelm = 3   linear triangles, standard sequence
!         j+m+1 - j+m+2                   2m+1 -   2m+2
!         |   /    |        .....          |   /    |
!         j  --- j+1                      m  ---   m+1

         ikelm = 0
         do i = 1, n
            ih = (i-1)*m1
            do j = 1,m
               kmeshc(ikelm+1) = ih+j+m1
               kmeshc(ikelm+2) = ih+j
               kmeshc(ikelm+3) = ih+j+1+m1
               kmeshc(ikelm+4) = ih+j+1
               kmeshc(ikelm+5) = ih+j+1+m1
               kmeshc(ikelm+6) = ih+j
               ikelm = ikelm+6
            end do
         end do
         nelem = 2*m*n

      else if ( inpelm==3 .and. idiag==1 ) then

!     --- inpelm = 3   linear triangles, reversed sequence
!         j+m+1 - j+m+2                   2m+1 -   2m+2
!         |   \    |        .....          |   \    |
!         j  --- j+1                      m  ---   m+1

         ikelm = 0
         do i = 1, n
            ih = (i-1)*m1
            do j = 1,m
               kmeshc(ikelm+1) = ih+j+m1
               kmeshc(ikelm+2) = ih+j+1
               kmeshc(ikelm+3) = ih+j+1+m1
               kmeshc(ikelm+4) = ih+j+1
               kmeshc(ikelm+5) = ih+j+m1
               kmeshc(ikelm+6) = ih+j

               ikelm = ikelm+6
            end do
         end do
         nelem = 2*m*n

      else if ( inpelm==3 ) then

!     --- inpelm = 3   linear triangles, for mod(j,2) = 0
!         j+m+1 - j+m+2                   2m+1 -   2m+2
!         |   /    |        .....          |   /    |
!         j  --- j+1                      m  ---   m+1
!                                         for mod(j,2) = 1
!         j+m+1 - j+m+2                   2m+1 -   2m+2
!         |   \    |        .....          |   \    |
!         j  --- j+1                      m  ---   m+1

         ikelm = 0
         do i = 1, n
            ih = (i-1)*m1
            do j = 1,m
               if (mod(j,2)==0) then
                  kmeshc(ikelm+1) = ih+j+m1
                  kmeshc(ikelm+2) = ih+j
                  kmeshc(ikelm+3) = ih+j+1+m1
                  kmeshc(ikelm+4) = ih+j+1
                  kmeshc(ikelm+5) = ih+j+1+m1
                  kmeshc(ikelm+6) = ih+j
               else
                  kmeshc(ikelm+1) = ih+j+m1
                  kmeshc(ikelm+2) = ih+j+1
                  kmeshc(ikelm+3) = ih+j+1+m1
                  kmeshc(ikelm+4) = ih+j+1
                  kmeshc(ikelm+5) = ih+j+m1
                  kmeshc(ikelm+6) = ih+j

               end if
               ikelm = ikelm+6
            end do
         end do
         nelem = 2*m*n

      else if ( inpelm==4 ) then

!     --- inpelm = 4   linear quadrilaterals
!         j+m+1 - j+m+2                    2m+1 -  2m+2
!         |        |        .....          |        |
!         j  --- j+1                      m  ---   m+1

         ikelm = 0
         do i = 1, n
            ih = (i-1)*m1
            do j = 1,m
               kmeshc(ikelm+1) = ih+j
               kmeshc(ikelm+2) = ih+j+1
               kmeshc(ikelm+3) = ih+j+1+m1
               kmeshc(ikelm+4) = ih+j+m1
               ikelm = ikelm+4
            end do
         end do
         nelem = m*n

      else if ( (inpelm==6 .or. inpelm==7) .and. idiag==0 ) then

!     --- inpelm = 6   quadratic triangles, standard sequence
!         j+4m+2 - j+4m+3 - j+4m+4                    6m+1 -  6m+2  --  6m+3
!         |            /     |        .....          |            /    |
!         j+2m+1 j+2m+2    j+2m+3                    4m       4m+1     4m+2
!         |     /            |        .....          |   /             |
!         j   --  j+1   --  j+2                     2m-1 --  2m    --  2m+1

         ikelm = 0
         do i = 1, n
            ih = (i-1)*(m*4+2)
            do j = 1, 2*m-1,2
               kmeshc(ikelm+1) = ih+j+4*m+2
               kmeshc(ikelm+2) = ih+j+2*m+1
               kmeshc(ikelm+3) = ih+j
               kmeshc(ikelm+4) = ih+j+2*m+2
               kmeshc(ikelm+5) = ih+j+4*m+4
               kmeshc(ikelm+6) = ih+j+4*m+3
               kmeshc(ikelm+7) = ih+j+2
               kmeshc(ikelm+8) = ih+j+2*m+3
               kmeshc(ikelm+9) = ih+j+4*m+4
               kmeshc(ikelm+10) = ih+j+2*m+2
               kmeshc(ikelm+11) = ih+j
               kmeshc(ikelm+12) = ih+j+1
               ikelm = ikelm+12
            end do
         end do
         nelem = 2*m*n

      else if ( (inpelm==6 .or. inpelm==7) .and. idiag==1 ) then

!     --- inpelm = 6   quadratic triangles, reversed sequence
!         j+4m+2 - j+4m+3 - j+4m+4                    6m+1 -  6m+2  --  6m+3
!         |  \               |        .....          |    \             |
!         j+2m+1 j+2m+2    j+2m+3                    4m       4m+1     4m+2
!         |             \    |        .....          |              \   |
!         j   --  j+1   --  j+2                     2m-1 --  2m    --  2m+1

         ikelm = 0
         do i = 1, n
            ih = (i-1)*(m*4+2)
            do j = 1, 2*m-1,2
               kmeshc(ikelm+1) = ih+j+4*m+2
               kmeshc(ikelm+2) = ih+j+2*m+1
               kmeshc(ikelm+3) = ih+j
               kmeshc(ikelm+4) = ih+j+1
               kmeshc(ikelm+5) = ih+j+2
               kmeshc(ikelm+6) = ih+j+2*m+2
               kmeshc(ikelm+7) = ih+j+2
               kmeshc(ikelm+8) = ih+j+2*m+3
               kmeshc(ikelm+9) = ih+j+4*m+4
               kmeshc(ikelm+10) = ih+j+4*m+3
               kmeshc(ikelm+11) = ih+j+4*m+2
               kmeshc(ikelm+12) = ih+j+2*m+2
               ikelm = ikelm+12
            end do
         end do
         nelem = 2*m*n

      else if ( (inpelm==6 .or. inpelm==7) ) then

!     --- inpelm = 6   quadratic triangles, for mod(j,2) = 0
!         j+4m+2 - j+4m+3 - j+4m+4                    6m+1 -  6m+2  --  6m+3
!         |            /     |        .....          |            /    |
!         j+2m+1 j+2m+2    j+2m+3                    4m       4m+1     4m+2
!         |     /            |        .....          |   /             |
!         j   --  j+1   --  j+2                     2m-1 --  2m    --  2m+1
!                                           for mod(j,2) = 1
!         j+4m+2 - j+4m+3 - j+4m+4                    6m+1 -  6m+2  --  6m+3
!         |  \               |        .....          |    \             |
!         j+2m+1 j+2m+2    j+2m+3                    4m       4m+1     4m+2
!         |             \    |        .....          |              \   |
!         j   --  j+1   --  j+2                     2m-1 --  2m    --  2m+1

         ikelm = 0
         do i = 1, n
            ih = (i-1)*(m*4+2)
            do j = 1, 2*m-1,2
               if (mod(j,2)==0) then
                  kmeshc(ikelm+1) = ih+j+4*m+2
                  kmeshc(ikelm+2) = ih+j+2*m+1
                  kmeshc(ikelm+3) = ih+j
                  kmeshc(ikelm+4) = ih+j+2*m+2
                  kmeshc(ikelm+5) = ih+j+4*m+4
                  kmeshc(ikelm+6) = ih+j+4*m+3
                  kmeshc(ikelm+7) = ih+j+2
                  kmeshc(ikelm+8) = ih+j+2*m+3
                  kmeshc(ikelm+9) = ih+j+4*m+4
                  kmeshc(ikelm+10) = ih+j+2*m+2
                  kmeshc(ikelm+11) = ih+j
                  kmeshc(ikelm+12) = ih+j+1
               else
                  kmeshc(ikelm+1) = ih+j+4*m+2
                  kmeshc(ikelm+2) = ih+j+2*m+1
                  kmeshc(ikelm+3) = ih+j
                  kmeshc(ikelm+4) = ih+j+1
                  kmeshc(ikelm+5) = ih+j+2
                  kmeshc(ikelm+6) = ih+j+2*m+2
                  kmeshc(ikelm+7) = ih+j+2
                  kmeshc(ikelm+8) = ih+j+2*m+3
                  kmeshc(ikelm+9) = ih+j+4*m+4
                  kmeshc(ikelm+10) = ih+j+4*m+3
                  kmeshc(ikelm+11) = ih+j+4*m+2
                  kmeshc(ikelm+12) = ih+j+2*m+2
               end if
               ikelm = ikelm+12
            end do
         end do
         nelem = 2*m*n

      else if ( inpelm==9 ) then

!     --- inpelm = 9   quadratic quadrilaterals
!         j+4m+2 - j+4m+3 - j+4m+4                    6m+1 -  6m+2  --  6m+3
!         |                  |        .....          |                 |
!         j+2m+1 j+2m+2    j+2m+3                    4m       4m+1     4m+2
!         |                  |        .....          |                 |
!         j   --  j+1   --  j+2                     2m-1 --  2m    --  2m+1

         ikelm = 0
         do i = 1, n
            ih = (i-1)*(m*4+2)
            do j = 1, 2*m-1,2
               kmeshc(ikelm+1) = ih+j
               kmeshc(ikelm+2) = ih+j+1
               kmeshc(ikelm+3) = ih+j+2
               kmeshc(ikelm+4) = ih+j+2*m+3
               kmeshc(ikelm+5) = ih+j+4*m+4
               kmeshc(ikelm+6) = ih+j+4*m+3
               kmeshc(ikelm+7) = ih+j+4*m+2
               kmeshc(ikelm+8) = ih+j+2*m+1
               kmeshc(ikelm+9) = ih+j+2*m+2
               ikelm = ikelm+9
            end do
         end do
         nelem = m*n

      else

!     --- This type of element is not yet available

         call eropen ( 'mshp05' )
         call errint ( ishape, 1 )
         call errsub ( 800, 1, 0, 0 )
         call erclos ( 'mshp05' )

      end if

1000  call erclos ( 'mshp05' )
      if ( debug ) then

!     --- Debug information

         call prinin1 ( kmeshc, nelem, inpelm, 'kmeshc' )
         write(irefwr,*) 'End mshp05'

      end if  ! ( debug )

      end
