      subroutine mshr01pav ( nseqkeyw, iseqkeyw, table, tabmax, tabout, &
                             resul, isurnr, icurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 04-02-2008 Coreection icurvprev
!        version  1.0  date 17-01-2005
!
!   copyright (c) 2005-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information about curves in array resul for subroutine mshr01
!     Special case for paver
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     read
!     surface
!     paver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nseqkeyw, iseqkeyw(*), isurnr, icurvs(*)
      double precision table(*), tabmax(*), tabout(*), resul(*)

!     icurvs         i    Array indicating if curve i has been filled
!                         (icurvs(i)>0) or not (icurvs(i)=0)
!     iseqkeyw       i    Array of size nseqkeyw that contains the sequence in
!                         which keywords are found in a record
!                         isekeyw(i)>0, means i-th keyword found is the
!                                      iseqkeyw(i)-th keyword in the table
!                         isekeyw(i)<0, means keyword refers to number found
!                                      -isekeyw(i) refers to position in tabout
!     isurnr         i    Surface sequence number
!     nseqkeyw       i    Number of entries in iseqkeyw
!     resul         i/o   Array containing the output of this subroutine
!                         See subroutine mshr01
!     table          i    Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabmax         i    Contains maximum values allowed in table
!     tabout         i    Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iseqpresent, iposcur, numcur, iposf, numf, ipost, &
              numt, icount, icountf, icountt, icur, icurnr, icurprev

!     i              Counting variable
!     icount         Counter
!     icountf        Counter with respec to factor f
!     icountt        Counter with respec to thickness t
!     icur           sequence number
!     icurnr         Curve sequence number
!     icurprev       Previous curve number
!     iposcur        Position of curve number in tabout
!     iposf          Position of factor f in tabout
!     ipost          Position of thickness t in tabout
!     iseqpresent    Actual sequence number of keyword read
!     numcur         Number of curves read
!     numf           Number of values for f read
!     numt           Number of values for t read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     148   no curve numbers in surface record.
!    1023   cannot specify thickness before curve
!    1024   cannot specify factor before curve
!    1219   Curve number has not been defined and may therefore not be used
!    2754   f or t in worng sequence in paver record
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshr01pav' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshr01pav'
  1      format ( a, 1x, (10i6) )
         call prinin ( iseqkeyw, nseqkeyw, 'iseqkeyw' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Find first appearance of a curve number in the record

      iseqpresent = 0
      do i = 1, nseqkeyw
         if ( iseqkeyw(i)==17 ) then

!        --- the i-th sequence number is a curve number

            iseqpresent = i
            go to 200

         else if ( iseqkeyw(i)==14 ) then

!        --- the i-th sequence number is f
!            error: this number may not be read before curve number

            call errsub ( 1024, 0, 0, 0 )
            call errcrd

         else if ( iseqkeyw(i)==18 ) then

!        --- the i-th sequence number is t
!            error: this number may not be read before curve number

            call errsub ( 1023, 0, 0, 0 )
            call errcrd

         end if  ! ( iseqkeyw(i)==17 )

      end do  ! i = 1, nseqkeyw

200   if ( iseqpresent==0 ) then

!     --- Error: no curve number found

         call errint ( isurnr, 1 )
         call errsub ( 148, 1, 0, 0 )

      end if  ! ( iseqpresent==0 )

      iposcur = nint(table(17))  ! starting address of curve numbers in tabout
      numcur = nint(tabmax(17))  ! number of curves
      if ( table(14)>0d0 ) then

!     --- Factor f has been found

         iposf = nint(table(14))  ! starting address of f values in tabout
         numf = nint(tabmax(14))  ! number of f values

      else

!     --- Factor f has not been found

         iposf = 0
         numf = 0

      end if  ! ( table(14)>0d0 )

      if ( table(18)>0d0 ) then

!     --- Thickness t has been found

         ipost = nint(table(18))  ! starting address of t values in tabout
         numt = nint(tabmax(18))  ! number of t values

      else

!     --- Factor t has not been found

         ipost = 0
         numt = 0

      end if  ! ( table(18)>0d0 )
      icountf = 0
      icountt = 0
      if ( debug ) &
         write(irefwr,1) 'numcur, iposcur, numt, ipost, numf, iposf', &
                          numcur, iposcur, numt, ipost, numf, iposf

!     --- Fill information of curves

      do icur = 1, numcur

         icurnr = nint(tabout(iposcur+icur-1))  ! curve number
         if ( icurvs(abs(icurnr))==0 ) then

!        --- Error 1219:  Curve has not been filled

            call errint ( icurnr, 1 )
            call errint ( isurnr, 2 )
            call errsub ( 1219, 2, 0, 0 )
            call errcrd

         end if  ! ( icurvs(abs(icurnr))==0 )
         resul(24+5*icur) = icurnr
         iseqpresent = iseqpresent+1  ! raise iseqpresent
         if ( debug ) &
            write(irefwr,1) 'icur, icurnr, iseqpresent', &
                             icur, icurnr, iseqpresent

!        --- Fill default f and t values

         if ( icur==1 ) then

!        --- First curve number

            if ( icurnr>0 ) then

!           --- thickness and factor of first curve
!               use defaults, positive curve number

                  resul(27) = 1d0
                  resul(28) = 1d0

               else

!              --- thickness and factor of first curve
!                  use defaults, negative curve number

                  resul(25) = 1d0
                  resul(26) = 1d0

               end if  ! ( resul(29)>=0d0 )

         else if ( icurnr>=0 .and. icurprev>=0 ) then

!        --- icurnr>0 and icurprev>0, take the thickness of this curve
!            from the previous curve, positive curve number

            resul(24+5*icur-2) = resul(24+5*icur-7)

!           --- take the factor of this curve
!               from the previous curve

            resul(24+5*icur-1) = resul(24+5*icur-6)

         else if ( icurnr>=0 ) then

!        --- icurnr>0 and icurprev<0, take the thickness of this curve
!            from the previous curve, positive curve number

            resul(24+5*icur-2) = resul(24+5*icur-9)

!           --- take the factor of this curve
!               from the previous curve

            resul(24+5*icur-1) = resul(24+5*icur-8)

         else if ( icurnr<0 .and. icurprev<0 ) then

!        --- icurnr<0 and icurprev<0, take the thickness of this curve
!            from the previous curve, negative curve number

            resul(24+5*icur-4) = resul(24+5*icur-9)

!           --- take the factor of this curve
!               from the previous curve

            resul(24+5*icur-3) = resul(24+5*icur-8)

         else

!        --- icurnr<0 and icurprev>0, take the thickness of this curve
!            from the previous curve, negative curve number

            resul(24+5*icur-4) = resul(24+5*icur-7)

!           --- take the factor of this curve
!               from the previous curve

            resul(24+5*icur-3) = resul(24+5*icur-6)

         end if  ! ( icur==1 )

!        --- set previous curve number

         icurprev = icurnr

!        --- Now fill the factors if they have been read

         icount = 0
         if ( iseqkeyw(iseqpresent)==14 .or. &
              iseqkeyw(iseqpresent)==18 .and. &
              iseqkeyw(iseqpresent+1)==14 ) then

!        --- Next keyword is f, fill factor and raise iseqpresent and icountf

            if ( icurnr>0 ) then

!           --- Curve number positive

               resul(24+5*icur-1) = tabout(iposf+icountf)

            else

!           --- Curve number negative

               resul(24+5*icur-3) = tabout(iposf+icountf)

            end if  ! ( icurnr>0 )
            icount = 1
            icountf = icountf+1

         end if  ! ( iseqkeyw(iseqpresent)==14 ...  )

         if ( iseqkeyw(iseqpresent)==18 .or. &
              iseqkeyw(iseqpresent)==14 .and. &
              iseqkeyw(iseqpresent+1)==18 ) then

!        --- Next keyword is t, fill thickness and raise iseqpresent and icountt

            if ( icurnr>0 ) then

!           --- Curve number positive

               resul(24+5*icur-2) = tabout(ipost+icountt)

            else

!           --- Curve number negative

               resul(24+5*icur-4) = tabout(ipost+icountt)

            end if  ! ( icurnr>0 )
            icount = icount+1
            icountt = icountt+1

         end if  ! ( iseqkeyw(iseqpresent)==14 ...  )
         iseqpresent = iseqpresent+icount

      end do  ! icur = 1, numcur

      if ( icountf<numf ) then

!     --- icountf<numf, some f values are stored in the wrong sequence

         call errint ( isurnr, 1 )
         call errchr ( 'f', 1 )
         call errsub ( 2754, 1, 0, 1 )

      end if  ! ( icountf<numf )

      if ( icountt<numt ) then

!     --- icountf<numf, some t values are stored in the wrong sequence

         call errint ( isurnr, 1 )
         call errchr ( 't', 1 )
         call errsub ( 2754, 1, 0, 1 )

      end if  ! ( icountf<numf )
      resul(1) = numcur

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icountf, icountt', icountf, icountt
         write(irefwr,*) 'End mshr01pav'

      end if
      call erclos ( 'mshr01pav' )

      end subroutine mshr01pav
