      subroutine mshr17 ( x, y, idirec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 24-07-2003 New parameter list
!        version  1.3  date 18-08-1992 isign==>lsign
!        version  1.2  date 22-08-1991 Zdenek: no dimension, cdc, ce
!
!   copyright (c) 1982-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    detect direction of diagonal for triangular elements
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     rectangle
!     structured
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer idirec
      double precision x(6), y(6)

!     idirec         o    Defines in which direction the diagonal must be made
!                         0: from 1 to 3
!                         1: from 2 to 4
!                         2: all angles are 90 degrees
!     x             i/o   x co-ordinates of vertices of quadrilateral in
!                         positions 2 to 5
!                         The positions 1 and 6 are used as work space
!     y             i/o   y-coordinates of vertices of quadrilateral in
!                         positions 2 to 5
!                         The positions 1 and 6 are used as work space
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, l, lsign(4)
      double precision gamma, ph, s, u, u1, u2, v, v1, v2, phi(4)

!     gamma
!     k              Counting variable
!     l              General loop variable
!     lsign
!     ph             ?
!     phi            array containing the cosines of the 4 angles of the
!                    quadrilateral
!     s
!     u
!     u1
!     u2
!     v
!     v1
!     v2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Copy coordinates 5 to 2 and 2 to 6 in order to make the
!         everything cyclic

      x(1)=x(5)
      x(6)=x(2)
      y(1)=y(5)
      y(6)=y(2)

!     --- Store cosine of angles into phi and sign of innerproduct in lsign

      do k = 2, 5
         u1 = x(k+1)-x(k)
         u2 = y(k+1)-y(k)
         v1 = x(k)-x(k-1)
         v2 = y(k)-y(k-1)
         gamma = u1*v1+u2*v2
         s = u1*v2-u2*v1
         lsign(k-1) = 1
         if ( s.lt.0 ) lsign(k-1) = -1
         u = u1*u1+u2*u2
         v = v1*v1+v2*v2
         phi(k-1) = gamma/sqrt(u*v)

      end do

!     --- check whether there are angles larger than 180 degrees
!         This is the case if two subsequent entries of lsign have different
!         sign

      do k = 2, 4

         if ( lsign(k).ne.lsign(k-1) ) then

!        --- Signs are different, angle larger than 180 degrees

            if ( k.eq.2 .and. lsign(3).eq.lsign(2) ) then
               idirec = 1
            else
               idirec = mod(k,2)
            end if
            go to 1000

         end if

      end do

!     --- computation of largest angle

      ph = phi(1)
      l = 1
      do k = 2, 4
         if ( ph-sqreps.le.phi(k) ) then

!        --- phi(k) larger than all previous ones

            ph = phi(k)
            l = k

         end if

      end do

      if ( abs(ph).lt.sqreps ) then
         idirec = 2
      else
         idirec = mod(l,2)
      end if

1000  end
