      subroutine mshr20 ( kmeshc, coor, nelem, inpelm, ndim )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.1    date   21-12-1996 Extension with shape 10
!        version 1.0    date   22-08-1994
!
!
!
!   copyright (c) 1994-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill array coor for centroid points in 5-point elements
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer inpelm, nelem, kmeshc(inpelm, nelem), ndim
      double precision coor(ndim,*)

!     coor    o    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     inpelm  i    number of points per element
!     kmeshc  i    kmesh part c, contains the nodal points corresponding to the
!                  elements
!     ndim    i    Dimension of the space.
!     nelem   i    Number of elements
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielem
      double precision h

!     h       help variable to store a constant
!     i       Counting variable
!     ielem   Element number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!

      if ( inpelm.eq.5 ) then

!     --- quadrilateral with 5 points

         do ielem = 1, nelem
            do i = 1, ndim
               coor(i,kmeshc(5,ielem)) = 0.25d0*( &
                  coor(i,kmeshc(1,ielem)) + coor(i,kmeshc(2,ielem)) + &
                  coor(i,kmeshc(3,ielem)) + coor(i,kmeshc(4,ielem)) )
            end do
         end do

      else

!     --- triangle with 4 points

         h = 1d0/3d0
         do ielem = 1, nelem
            do i = 1, ndim
               coor(i,kmeshc(4,ielem)) = h*( &
                  coor(i,kmeshc(1,ielem)) + coor(i,kmeshc(2,ielem)) + &
                  coor(i,kmeshc(3,ielem)) )
            end do
         end do

      end if

      end
