      subroutine mshrd1shape ( iinput, iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-01-2015 New call to readreal
!        version  1.0  date 20-11-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read or set shape number of curve
!
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: iinput(*)
      integer, intent(in) :: iinp

!     iinp          i/o   last position used in array iinput
!     iinput        i/o   integer input array
!                         See Programmer's Guide
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical ::  openfound

!     openfound      Indicates if an openings bracket is found before
!                    reading the number or expression
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READREAL       Read a real or real expression from the input file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     128   code for generation of elements (ishape) is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrd1shape' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd1shape'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ishape = shape_cur

!     --- determine shape number

      openfound = .false.
      call readreal ( openfound, .false. )

      if ( debug ) write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
      if ( ityprd==2 ) then

!     --- shape number found

         ishape = nint(realrd)

         if ( ishape<=0 .or. ishape>3 ) then
            call errint ( ishape, 1 )
            call errsub ( 128, 1, 0, 0 )
            call errcrd
         end if

      else if ( typeitems(lastitem+1)==3 ) then

!     --- string found, check if it is linear or quadratic

         call asknxt
         if ( debug ) then
            write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
            write(irefwr,*) 'string ', string(1:nchar)
         end if  ! ( debug )

         if ( string(1:3)=='LIN' ) then

!        --- linear found

            ishape = 1

         else if ( string(1:3)=='QUA' ) then

!        --- quadratic found

            ishape = 2

         else

!        --- not linear or quadratic found, reset lastitem

            lastitem = lastitem-1

         end if  ! ( string(1:5)=='LINEA' )

      end if
      iinput(iinp+1) = ishape

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ishape, lastitem', ishape, lastitem
         write(irefwr,*) 'End mshrd1shape'

      end if  ! ( debug )
      call erclos ( 'mshrd1shape' )

      end subroutine mshrd1shape
