      subroutine mshrd2ordsur ( iinp, iinput, maxsrf, iwork, clcode, &
                                isurnr, composrf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-11-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read rest of surface for ordered surface of surfaces
!
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer,intent(in) :: maxsrf, isurnr
      integer,intent(inout) :: iinp, iinput(*), iwork(*)
      character (len=*) clcode(*)
      logical,intent(inout) :: composrf

!     clcode         i    In this table the significant characters of the
!                         various types of surfaces are stored.
!     composrf      i/o   If true composite elements have been found
!     iinp          i/o   last position used in array iinput
!     iinput        i/o   integer input array
!     isurnr         i    Surface sequence number
!     iwork         i/o   integer work array, see mshrd2
!     maxsrf         i    Maximal number of surfaces allowed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nrow, nsurf, maxnum, inum, i

!     i              Counting variable
!     inum           Number of numbers read in a record
!     maxnum         Maximal number of numbers to be read from the input
!     nrow           Number of rows of surfaces in ordered surface of
!                    surfaces
!     nsurf          Number of surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READSERKEYW    Read number of integers preceded by the same keyword
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     561   Surface consisting of subsurfaces contains too many subsurfaces
!     562   No subsurfaces found in record with ordered surface.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrd2ordsur' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd2ordsur'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nrow = 0
      nsurf = 0
      composrf = .true.

!     --- Read next item, check if it is surface
!         If not surface consider as not read

      call asknxt
      if ( debug ) &
         write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
      if ( ityprd/=4 .or. string(1:3)/='SUR' ) lastitem = lastitem-1
      if ( debug ) write(irefwr,1) 'lastitem', lastitem

!     --- New row of surfaces

250   maxnum = 100
      call readserkeyw ( iwork(nsurf+1), maxnum, inum, .false., &
                         maxsrf, clcode(11), 'S' )
      if ( debug ) &
         write(irefwr,1) 'inum, nrow, iinp', inum, nrow, iinp
      if ( inum>0 ) then
         nrow = nrow + 1
         iinput(iinp+2+nrow) = inum
         nsurf = nsurf + inum
         if ( nsurf>maxnum ) then
            call errint ( nsurf, 1 )
            call errint ( maxnum, 2 )
            call errint ( isurnr, 3 )
            call errsub ( 561, 3, 0, 0 )
            call errcrd
         end if
         go to 250
      end if

      if ( nrow==0 ) then

!     --- error 562: nrow = 0

         call errint ( isurnr, 1 )
         call errsub ( 562, 1, 0, 0 )
         call errcrd
         go to 1000

      end if

      iinput(iinp+2) = nrow
      iinp = iinp + 2 + nrow
      do i = 1, nsurf
         iinput(iinp+i) = iwork(i)
      end do
      iinp = iinp + 1 + nsurf

1000  call erclos ( 'mshrd2ordsur' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'inum, iinp', inum, iinp
         write(irefwr,*) 'End mshrd2ordsur'

      end if  ! ( debug )

      end subroutine mshrd2ordsur
