      subroutine mshrd5 ( iinput, rinput, lengin, &
                          inpmax, lengrl, iinpt1, lengintf, &
                          ichangecr, nextra, iobstacle, iinputdummy )
! ======================================================================
!
!        programmer    Guus Segal
!        version 19.2  date 21-11-2014 Correction iinput/rinput
!        version 19.1  date 28-03-2013 Cleaning of code
!        version 19.0  date 02-09-2010 Remove intarmsh
!
!   copyright (c) 1982-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      fill last part of arrays iinput and rinput
!      iinput must be filled from position 1
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduleconsts
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision rinput(*)
      integer inpmax, iinput(inpmax), lengin, lengrl, nextra, &
              iinpt1(*), lengintf, ichangecr(*), &
              iobstacle(*), iinputdummy(*)

!     ichangecr           integer work array of length 40 to store the
!                         information concerning change_coordinates
!                         For each data record 4 positions are required, at
!                         most 10 different cases are allowed
!                         Contents:
!                         (1,i):  ifunc_chan
!                         (2,i):  itype
!                         (3,i):  ifirst
!                         (4,i):  ilast
!     iinpt1         i    integer input array corresponding to interface
!                         elements
!     iinput        i/o   integer input array
!     iinputdummy    o    Integer array in which the number of nodes per dummy
!                         element are stored
!     inpmax         i    maximal space for iinput
!     iobstacle           integer work array of length 20 to store the
!                         information concerning the obstacles
!                         Contents
!                         (1,i) Type of obstacle (1=curve, rest not available)
!                         (2,i) Curve number
!     lengin        i/o   Length of array IINPUT. The actual length is 5 longer
!     lengintf       i    Length of part corresponding to interface elements
!     lengrl        i/o   Length of array RINPUT. The actual length is 5 longer
!     nextra         i    Number of extra entries in code to be used for
!                         subsubkeywords
!     rinput        i/o   complete real input array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinp, i, icurve, n, m, &
              j, jlast, nchancoor
      double precision plotfm, yfact
      logical renum, refine, interm, transf, checkl, plot, parallellc

!     checkl         If true check_level has been found
!     i              Counting variable
!     icurve         Type of curve with respect to mesh generator
!     iinp           last position used in array iinput
!     interm         If true intermediate has been found
!     j              Counting variable
!     jlast          iinput(20) refers to new part in iinput
!     m              number of points in 1-direction
!     n              number of points in 2-direction
!     nchancoor      number of data records for change coordinates
!     parallellc     If true parallel computing must be applied
!     plot           If true plot has been found
!     plotfm         Plotting factor. When PLOTFM<0, then the length of
!                    the plot is computed by - (xmax-xmin) * PLOTFM
!                    and the width by - (ymax-ymin) * YFACT * PLOTFM
!                    else the length of the plot is equal to PLOTFM cm
!                    When by some mistake the user has given | PLOTFM | a value
!                    smaller than 1 cm, then the length is chosen equal to
!                    the default length stored in common block consta
!     refine         If true refine has been found
!     renum          If true renumber has been found
!     transf         If true transformation has been found
!     yfact          Multiplication for the y-co-ordinates. With this
!                    parameter the user can influence the scale in y-direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     MSH046         Raise iinp as function of icurve
!     MSHR03         Interpret information in the part indicated by
!                    intermediate points
!     MSHR04         Interpret information following the keyword
!                    change_coordinates
!     MSHR05         Interpret the information following the keyword obstacles
!     MSHRDPARALLEL  Read input with respect to the keyword parallel
!     MSHRDPLOT      Read input with respect to the keyword plot
!     MSHRDRENUM     Read input with respect to the keyword renumber
!     MSHREACHECKL   Read input corresponding to check_levels
!     MSHREAREF      Read input corresponding to refine
!     MSHREATRANS    Read input corresponding to transformation
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
!    MSHRD5 reads input from the standard input file
!    The following records are recognized:
!      plot ( length=l, yfact=y, jmark=j, numsub=s, curve=c, nodes=no, &
!             userpoints=u, colour=dcl, suppress=su, rotate=r, &
!             eyepoint=(x,y,z), ren_plot, nomesh, nosubmesh, orientation=i )
!      renumber [, method=i]
!         or
!      renumber, method, preference, sequence, levels, print, start, not
!      with
!          method:    cuthill_mckee, sloan, best, element_groups
!          preference: profile, band, always
!          sequence: vertex_seq=j, mid_point_seq=j, centroid_seq=j
!          start:     start=pi, start=ci, start=si
!      norenumber
!      end
!      intermediate points
!         must be followed by extra information (See MSHR03)
!      refine [n times]
!      transform [ type = n ]
!      check_level = i
!      notopology
!      change_coordinates
!      obstacles
!      write_faces
!      parallel, method = x, num_processors = i
!      with
!         method = surfaces / volumes / layers
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1363   Main key words are not given in the correct sequence.
!    1552   keyword found more than once
!    2025   No volume elements connected to an element group
!    2026   No surface elements connected to an element group
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrd5' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd5'
         call prinin ( iinput, lengin, 'iinput' )
         write(irefwr,1) 'nelgrp, lengintf', nelgrp, lengintf
         write(irefwr,1) 'nintprop, nrealprop', nintprop, nrealprop
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      renum  = .false.
      refine = .false.
      interm = .false.
      transf = .false.
      checkl = .false.
      plot   = .false.
      parallellc = .false.
      iinput(6) = nlines
      iinput(7) = nsurel
      iinput(8) = nvolel
      iinput(9) = 0
      plotfm = defplo
      yfact = 1d0
      irefin = 0
      itrans = 0
      icheck = 0
      notopol = 0
      nchancoor = 0
      numobst = 0
      iwritfaces = 0

!     --- check if renumbering must be performed

      ncurvs = iinput(3)
      nsurfs = iinput(4)
      nvolms = iinput(5)
      if ( nelgrp>1 ) then

!     --- There are various element groups, check if the highest elements
!         are provided with an element group sequence number

         if ( nvolms>0 ) then

!        --- There are volume elements

            if ( nvolel==0 ) &
               call errsub ( 2025, 0, 0, 0 )

         else if ( nsurfs>0 ) then

!        --- There are surface elements but no volume elements

            if ( nsurel==0 ) &
               call errsub ( 2026, 0, 0, 0 )

         end if
      end if

      if ( nvolms/=0 .or. nsurfs>1 .or. &
          ( nsurfs>0 .and. nlines>0 ) .or. nlines>1 ) then

!     --- nvolms>0
!         or nsurfs>1
!         or nsurfs>0  and  nlines>0
!         or nlines>1
!         hence renumbering

         if ( nvolms==0 ) then
            iinput(10) = 2
         else
            iinput(10) = 9
         end if

      else if ( nlines==1 ) then

!     --- nlines = 1

         if ( ncurvs==1 ) then
            iinput(10) = 0
         else
            iinput(10) = 2
         end if

      else

!     --- nsurfs = 1, check type of mesh generator

         iinp = 16
         do i = 1, ncurvs
            icurve = iinput(iinp)
            call msh046 ( iinput, iinp, icurve )
         end do
         isurf = iinput(iinp)
         ishape = iinput(iinp+1)

         if ( isurf==1 .or. isurf==17 .and. &
              ( ishape==3 .or. ishape==5 ) )  then

!        --- rectangle with linear elements

            iinp = iinp + iinput(iinp+2)
            n = iinput(iinp+3)
            m = iinput(iinp+4)
            if ( n==0 .or. n>m ) then
               iinput(10) = 2
            else
               iinput(10) = 0
            end if
         else if ( isurf==2 .and. ( ishape==3 .or. ishape==5 ) ) then

!        --- general with linear elements

            iinput(10) = 0
         else
            iinput(10) = 2
         end if

      end if

      if ( debug ) write(irefwr,1) 'lengin, icode', lengin, icode

!     --- Perform actions depending on icode

200   select case ( icode )

      case (5)

!     --- icode = 5  renumber, read rest of record
!         initialize table1

         call mshrdrenum ( iinput, lengin, renum )

      case (6)

!     --- icode = 6 plot

         call mshrdplot ( iinput, lengin, rinput, ndim, lengrl, &
                          yfact, plot )

      case (7, 15)

!     --- case 7 or 15, jump

         go to 500

      case (9)

!     --- icode = 9  norenumber

         if ( renum ) then

!        --- renum has been found more than once

            call errchr ( string(1:nchar), 1 )
            call errwar ( 1552, 0, 0, 1 )

         end if
         renum = .true.
         iinput(10) = 0

      case (10)

!     --- icode = 10  intermediate points

         call mshr03 ( iinput, lengin, inpmax, interm )
         goto 200  ! repeat reading

      case (11)

!     --- icode = 11  refine, check if refine is followed by a number

         call mshrearef ( refine, renum, iinput, irefin )

      case (12)

!     --- icode = 12  transform, check if transform is followed by a type = t

         call mshreatrans ( transf, itrans )

      case (13)

!     --- icode = 13  check_level found

         call mshreacheckl ( checkl, icheck )

      case (14)

!     --- notopology found

         notopol = 1

      case (16)

!     --- change_coordinates found

         nmain = nncodemain
         call mshr04 ( ichangecr, nchancoor, iinput )
         istop = 3

      case (17)

!     --- obstacles found

         call mshr05 ( iobstacle, numobst, iinput )
         istop = 3

      case (18)

!     --- write_faces found

         iwritfaces = 1

      case (19)

!     --- parallel found

         parallellc = .true.
         call mshrdparallel ( iparallel )

      case default

!     --- icode = 0-4 or 8, wrong sequence of KEY words

        call errint ( icode, 1 )
        call errchr ( string(1:nchar), 1 )
        do i = 5, nncodemain
           call errchr ( codemain(i)(1:ncodmxmain(i)), i-3 )
        end do
        call errsub ( 1363, 1, 0, nncodemain-3 )
        call errcrd
        go to 1000

      end select  ! case

!     --- read next record

      if ( istop/=3 ) ityprd = 1
      call asknxt
      ierpri = 0
      icode = sepmainkeyw() - 1

      if ( debug ) write(irefwr,1) 'lengin, icode', lengin, icode

      go to 200

500   rinput(1) = plotfm
      rinput(2) = yfact

!     --- 20 positions at the present end of IINPUT are reserved for future
!         purposes. At this moment only the positions 1 (irefin) and 2 (itrans)
!         are used

      if ( icheck>0 .or. irefin>0 .or. itrans>0 .or. &
           notopol>0 .or. lengintf>0 .or. nintprop>0 .or. &
           nrealprop>0 .or. nchancoor>0 .or. numobst>0 .or. &
           iwritfaces>0 .or. iscale>0 .or. parallellc .or. &
           nelgrpdum>0 ) then
         jlast = lengin+1
         iinput(15) = jlast+5
         iinput(jlast) = irefin
         iinput(jlast+1) = itrans
         iinput(jlast+2) = icheck
         iinput(jlast+3) = notopol
         do i = 5, 20
            iinput(jlast+i-1) = 0
         end do
         if ( lengintf>0 ) iinput(jlast+4) = lengin+25
         lengin = lengin+20
         iinput(lengin:lengin+lengintf-1) = iinpt1(1:lengintf)
         lengin = lengin+lengintf
         if ( debug ) write(irefwr,1) 'lengin/1', lengin
         if ( nintprop>0 .or. nrealprop>0 ) then

!        --- properties found
!            The properties are stored in reversed order

            iinput(jlast+5) = lengin+6
            iinput(lengin+1) = nintprop
            iinput(lengin+2) = nrealprop
            lengin = lengin+2
            do i = 1, nelgrp
               do j = 1, nintprop
                  iinput(lengin+(i-1)*nintprop+j) = intprop(j,i)
               end do
            end do
            lengin = lengin+nelgrp*nintprop
            if ( debug ) write(irefwr,1) 'lengin/2', lengin

            do i = 1, nelgrp
               do j = 1, nrealprop
                  rinput(lengrl+(i-1)*nrealprop+j) = realprop(j,i)
               end do
            end do
            lengrl = lengrl+nelgrp*nrealprop

         end if
         if ( nchancoor>0 ) then

!        --- change coordinates found

            iinput(jlast+6) = lengin+6
            iinput(lengin+1) = nchancoor
            do i = 1, nchancoor*4
               iinput(lengin+i+1) = ichangecr(i)
            end do
            lengin = lengin+1+nchancoor*4

         end if
         if ( numobst>0 ) then

!        --- obstacles found

            iinput(jlast+7) = lengin+6
            iinput(lengin+1) = numobst
            do i = 1, numobst*2
               iinput(lengin+i+1) = iobstacle(i)
            end do
            lengin = lengin+1+numobst*2

         end if
         iinput(jlast+8) = iwritfaces
         iinput(jlast+9) = iscale
         iinput(jlast+10) = itype_scale
         if ( parallellc ) iinput(jlast+11) = iparallel
         if ( nelgrpdum>0 ) then

!        --- nelgrpdum>0, there are dummy element groups

            iinput(jlast+12) = lengin+6
            iinput(lengin+1) = nelgrpdum
            do i = 1, nelgrpdum
               iinput(lengin+1+2*i-1) = nelgrp-nelgrpdum+i
               iinput(lengin+1+2*i  ) = iinputdummy(i)
            end do
            lengin = lengin+1+nelgrpdum*2

         end if

      else

         iinput(15) = 0

      end if

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iinput, lengin, 'iinput' )
         write(irefwr,*) 'End mshrd5'

      end if  ! ( debug )
      call erclos ( 'mshrd5' )

      end subroutine mshrd5
