      subroutine mshrd6shape ( iinput, iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-01-2015 New call to readreal
!        version  1.0  date 21-11-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read or set shape number of volume
!
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: iinput(*)
      integer, intent(in) :: iinp

!     iinp          i/o   last position used in array iinput
!     iinput        i/o   integer input array
!                         See Programmer's Guide
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncodelc=10
      integer :: ncodelc(nncodelc), ncodmxlc(nncodelc)
      character (len=27) codelc(nncodelc)
      logical ::  openfound

!     codelc         Table with keywords
!     ncodelc        Table containing the number of significant characters
!                    in the keyword table codelc
!     ncodmxlc       Table containing the maximal number of  characters in
!                    the keyword table codelc
!     nncodelc       Number of entries in codelc
!     openfound      Indicates if an openings bracket is found before
!                    reading the number or expression
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READREAL       Read a real or real expression from the input file
!     SEPCOPYCODE    Copy code2 and related arrays into code
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     128   code for generation of elements (ishape) is incorrect
!    1169   Shape number of volume element incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data codelc / 'LINEAR_TETRAHEDRON', 'QUADRATIC_TETRAHEDRON', &
                  'LINEAR_HEXAHEDRON', 'QUADRATIC_HEXAHEDRON', &
                  'EXTENDED_TETRAHEDRON14', 'EXTENDED_TETRAHEDRON15', &
                  'EXTENDED_LINEAR_HEXAHEDRON', &
                  'EXTENDED_LINEAR_TETRAHEDRON', &
                  'LINEAR_WEDGE', 'QUADRATIC_WEDGE' /
      data ncodelc  / 10,13,10,13,22,  22,19,19,10,13 /
      data ncodmxlc / 18,21,17,20,22,  22,26,27,12,15 /

!     codelc           Table with keywords
!                    The following keywords are available:
!                     1: LINEAR_TETRAHEDRON
!                     2: QUADRATIC_TETRAHEDRON
!                     3: LINEAR_HEXAHEDRON
!                     4: QUADRATIC_HEXAHEDRON
!                     5: EXTENDED_TETRAHEDRON14
!                     6: EXTENDED_TETRAHEDRON15
!                     7: EXTENDED_LINEAR_HEXAHEDRON
!                     8: EXTENDED_LINEAR_TETRAHEDRON
!                     9: LINEAR_WEDGE
!                    10: QUADRATIC_WEDGE
! ======================================================================
!
      call eropen ( 'mshrd6shape' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd6shape'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepcopycode ( codelc, nncodelc, ncodelc, ncodmxlc )

      ishape =  shape_vol

      openfound = .false.
      if ( debug ) write(irefwr,1) 'lastitem', lastitem
      call readreal ( openfound, .false. )
      if ( debug ) &
         write(irefwr,1) 'lastitem, ityprd', lastitem, ityprd
      if ( ityprd==2 ) then

!     --- shape number found

         ishape = nint(realrd)

         if ( ishape<=10 .or. ishape==19 .or. ishape==20 .or. &
              ishape>22 ) then
            call errint ( ishape, 1 )
            call errsub ( 128, 1, 0, 0 )
            call errcrd
         end if

      else if ( typeitems(lastitem+1)==3 ) then

!     --- string found, check if it is one of the possible shapes

         call asknxt
         if ( debug ) then
            write(irefwr,1) 'ityprd, lastitem', ityprd, lastitem
            write(irefwr,*) 'string ', string(1:nchar)
         end if  ! ( debug )

         ierpri = 0
         iretur = sepsubkeyw()
         if ( iretur<=8 ) then

!        --- linear..

            ishape = iretur+10

         else if ( iretur>8 ) then

!        --- wedge elements

            ishape = iretur+20

         else

!        --- not shape found, reset lastitem

            lastitem = lastitem-1

         end if  ! ( iretur>7 )

      end if  ! ( ityprd==2 )

      if ( ishape<11 .or. ishape>18 .and. ishape<=20 .or. &
           ivol==4 .and. ishape>22 .or. &
           ivol/=4 .and. ishape>20 ) then

!     --- Shape number has not yet been implemented

           call errint ( ishape, 1 )
           call errint ( ivolnr, 2 )
           call errsub ( 1169, 2, 0, 0 )
           call errcrd

      end if

      iinput(iinp+1) = ishape

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrd6shape'

      end if  ! ( debug )
      call erclos ( 'mshrd6shape' )

      end subroutine mshrd6shape
