      subroutine mshrd9 ( input1, nelgrp, inpmax, iinput, iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 21-11-2014 New parameter list
!        version  2.0  date 03-03-2006 Extra parameter nsurfs
!        version  1.0  date 04-03-1997
!
!   copyright (c) 1997-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   read cards with interface elements until a card with
!   renumber, plot or end is found
!   fill array iinpt1 with the information of these interface elements
!   iinpt1 must be filled from position 1
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpmax, nelgrp, iinput(*), &
              input1(inpmax), iinp

!     iinp           i    last position used in array iinp1
!     iinput        i/o   integer input array
!                         See subroutine mshrea
!     inpmax         i    Available space for array iinp1
!     input1        i/o   array iinput with respect to interface elements
!     nelgrp        i/o   number of element groups
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nlcode=5
      character (len=8) clcode(nlcode)
      integer ipoint, icurv, isurf, inum, ielgrp, ifirst, ilast, &
              ishape, nscode(nlcode), i, &
              ipos, ncodlm(nlcode), ncurvs, nsurfs
      logical ::  check

!     check   Indicates if a new element group has been found, or still
!             the first one is investigated
!     clcode  Contains the texts that are recognized after the part IELM i = &
!     i       General loop variable
!     icurv   Number of curves that are interface
!     ielgrp  element group number
!     ifirst  First curve (surface) number read
!     ilast   Last curve (surface) number read
!     inum    Number of numbers read in a record
!     ipoint  Number of user points that are interface
!     ipos    Position in tabout
!     ishape  Shape of element
!     isurf   Number of surfaces that are interface
!     ncodlm  Table containing the number of maximum characters in
!             the keyword table clcode
!     ncurvs  Number of curves in mesh
!     nlcode  Number of entries in clcode
!     nscode  Number of significant characters in clcode
!     nsurfs  number of surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!    The following input of the standard input file is recognized
!    - main keywords, see icode
!    - Input for interface_elements:
!         IELMi = CURVES (C1,C2, SHAPE=j)
!         IELMi = SURFACES (S1,S2)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      68   element group number not equal to preceding element group number or
!           preceding element group number plus one
!     142   Internal error, too few space for integers
!     151   first curve number has incorrect value
!     152   last curve number has incorrect value
!     155   first surface number has incorrect value
!     156   last surface number has incorrect value
!     318   record with celm found, whereas already records with selm are read
!           for connected elements.
!    1167   Shape number of element incorrect
!    1184   No number found, whereas at these positions a number is expected
!    1348   Record does not start with a character
!    1362   Data record for MESHCONNECT does not start with the letter C.
!    1837   Present element group number is equal to preceding one, whereas a
!           new type of elements is introduced
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data clcode / 'CURVES', 'SHAPE', 'SURFACES', 'C', 'S'  /
      data nscode / 3, 3, 3, 1, 1 /
      data ncodlm / 6, 5, 8, 1, 1 /

!     clcode  Contains the texts that are recognized after the part IELM i = &
!             Values:
!             1:  CURVES
!             2:  SHAPE
!             3:  SURFACES
!             4:  C
!             5:  S
! ======================================================================
!
      call eropen ( 'mshrd9' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrd9'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      itype(1) = 2
      itype(2) = 0
      itype(3) = 2
      itype(4:5) = 14
      tabmin(1:3) = 0d0
      tabmin(4:5) = 2d0
      tabmax(1:5) = 0d0
      tabmax(2) = 59d0

!     --- Get parameters

      ncurvs = iinput(3)
      nsurfs = iinput(4)

      check = .false.
      iinp = 3
      ipoint = 0
      icurv = 0
      isurf = 0

!     --- read next record

200   ityprd = 1
      if ( iinp+4>inpmax ) then
         call errint ( inpmax, 1 )
         call errint ( iinp+4, 2 )
         call errsub ( 142, 2, 0, 0 )
         go to 1000
      end if
      call asknxt
      if ( ityprd/=4 ) then

!     --- error 1348: Record does not start with a character

         call errsub ( 1348, 0, 0, 0 )
         call errcrd
         go to 200

      else

!     --- Detect the type of record
!         If a main keyword has been found, return to calling subroutine

         ierpri = 0
         icode = sepmainkeyw() - 1
         if ( debug ) write(irefwr,1) 'icode', icode
         if ( icode>=0 ) go to 1000

      end if
      if ( string(1:1)/='I' ) then
         call errchr ( string(1:1), 1 )
         call errsub ( 1362, 0, 0, 1 )
         call errcrd
         go to 200
      end if

!     --- detect the interface card

      call asknxt
      if ( ityprd/=2 ) then

!     --- No integer number found after IELM

         call errsub ( 1184, 0, 0, 0 )
         call errcrd
         go to 200

      end if

      ielgrp = nint(realrd)
      input1(iinp+1) = ielgrp
      if ( .not. check ) then

!     --- No records with respect to interface elements read, so nelgrp must
!         be raised

         if ( ielgrp==nelgrp ) then

!        --- Error 1837: ielgrp = nelgrp

            call errint ( ielgrp, 1 )
            call errsub ( 1837, 1, 0, 0 )
            call errcrd

         end if
         nelgrp = nelgrp + 1

      else if ( ielgrp>nelgrp ) then

!     --- Records have been read and ielgrp>nelgrp, raise nelgrp

         nelgrp = nelgrp+1

      end if

      if ( ielgrp/=nelgrp ) then
         call errint ( ielgrp, 1 )
         call errint ( nelgrp-1, 2 )
         call errsub ( 68, 2, 0, 0 )
         call errcrd
      end if
      check = .true.

!     --- Read type of record

      call sepcopycode ( clcode, nlcode, nscode, ncodlm )

      nmain = 0
      table(1:nlcode) = 0d0
      tabmax(4) = ncurvs
      tabmax(5) = nsurfs
      tabout(1) = 0d0
      tabout(2) = 0d0
      iretur = 0
      call sepreadsetkeyw

      if ( debug ) then
         write(irefwr,1) 'iretur', iretur
         call prinrl ( table, nlcode, 'table' )
         call prinrl ( tabmin, nlcode, 'tabmin' )
         call prinrl ( tabmax, nlcode, 'tabmax' )
         call prinrl ( tabout, 2, 'tabout' )
       end if  ! ( debug )

      if ( iretur==-1 ) go to 200

      if ( nint(table(1))/=0 ) then

!     --- curves must be interface

         if ( isurf>0) then
            call errsub ( 318, 0, 0, 0 )
            goto 200
         end if
         ishape = nint(table(2))
         if ( ishape/=0 .and. ishape<50 ) then
            call errint ( ishape, 1 )
            call errint ( 50, 2 )
            call errsub ( 1167, 2, 0, 0 )
            call errcrd
         end if
         inum = nint(tabmax(4))
         if ( inum/=2 ) then
            call errint ( ipos, 1 )
            call errsub ( 1167, 2, 0, 0 )
         else
            ifirst = nint(tabout(1))
            if ( ifirst==0 .or. abs(ifirst)>ncurvs ) then
               call errint ( ifirst, 1 )
               call errint ( ncurvs, 2 )
               call errsub ( 151, 2, 0, 0 )
               call errcrd
            end if
            ilast = nint(tabout(2))
            if ( ilast==0 .or. abs(ilast)>ncurvs ) then
               call errint ( ilast, 1 )
               call errint ( ifirst, 2 )
               call errint ( ncurvs, 3 )
               call errsub ( 152, 3, 0, 0 )
               call errcrd
            end if

         end if

         input1(iinp+2) = ishape
         input1(iinp+3) = ifirst
         input1(iinp+4) = ilast
         iinp = iinp+4
         icurv = icurv+1
         goto 200

      else if ( nint(table(3))/=0 ) then

!     --- interface elements are surfaces

         inum = nint(tabmax(5))
         if ( inum/=2 ) then
            call errint ( ipos, 1 )
            call errsub ( 1167, 2, 0, 0 )
         else
            ifirst = nint(tabout(1))
            if ( ifirst<=0 .or. ifirst>nsurfs ) then
               call errint ( ifirst, 1 )
               call errint ( nsurfs, 2 )
               call errsub ( 155, 2, 0, 0 )
               call errcrd
            end if
            ilast = nint(tabout(2))
            if ( ilast<=0 .or. ilast>nsurfs ) then
               call errint ( ilast, 1 )
               call errint ( ifirst, 2 )
               call errint ( nsurfs, 3 )
               call errsub ( 156, 3, 0, 0 )
               call errcrd
            end if

         end if

         input1(iinp+2) = 70
         input1(iinp+3) = ifirst
         input1(iinp+4) = ilast
         iinp = iinp+4
         isurf = isurf+1
         goto 200

      end if

1000  input1(1) = ipoint
      input1(2) = icurv
      input1(3) = isurf

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrd9'

      end if
      call erclos ( 'mshrd9' )

      end
