      function mshz00 ( k1, k2, k3, npoint, Coor )
!
! **********************************************************************
!
!       programmer      Zdenek Vlasek
!       version 1.0     date 14-02-1990
!
!
!   copyright (c) 1990  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                   PURPOSE
!
!     Angle ( in radians ).  The three points are determined
!     by the three global nodenumbers k1, k2, k3.
!
!      k1                             k3
!        \                              \
!          \  this angle                  \  not this
!           k2 ----------- k3              k2 ------------ k1
!                                    this angle, it is >pi
!
!      k1 ------- k2 ----- k3      angle = pi
!
!      k1-------- k3 ----- k2      angle = 0
!
!      k1=k2 ------------- k3      error
!
!      k3=k2 ------------- k1      error
!
!     The angle is measured in the point k2, on the left, see diagrams.
!     The range of the angle: [ 0, 2*pi ).
!
! **********************************************************************
!
!                  INPUT / OUTPUT  PARAMETERS

      integer k1, k2, k3, npoint
      double precision Coor ( 2, npoint )
      double precision mshz00

!     k1,k2,k3  i    Global nodenumbers of the three points.
!                    They point to columns in Coor
!     Coor      i    Real array (2,npoint)
!                    Physical coordinates of all meshpoints
!     npoint    i    Current number of all meshpoints
!
! **********************************************************************
!
!                  LOCAL VARIABLES

      double precision, parameter :: twopi=2d0*3.14159265358979d0
      double precision x1, y1, x2, y2, x3, y3, x, y

!     x1, y1  Relative coordinates of the point k1 with respect to k2
!     x2, y2  Coordinates of the point k2
!     x3, y3  Relative coordinates of the point k3 with respect to k2
!     x,  y   x+i*y is the complex number  (x2+i*y2) * (x3-i*y3 ) , &
!             mshz00 is the argument of this complex number
!
! **********************************************************************
!
!                  SUBROUTINES CALLED
!
!     mshz04   To display an error message and to stop the run
!
! **********************************************************************
!
!                  ERROR MESSAGES are self-explaining
!
! **********************************************************************
!
!                  METHOD
!
!     Put the new origin into the point k2 : subtract x2+i*y2, &
!     direct the new x-ax to the point k3 :  multiply by new x3-i*y3, &
!     thus the point k2 becomes 0 and point k3 is on the positive x-ax, &
!     the angle is now determined solely by the angle of the final
!     position of the point k1.
!     Standard function "atan2" returns angle in the range (-pi,pi], &
!     transform it into the range [0,2*pi).
!
! **********************************************************************
!

!    *  components of point k2

      x2 = Coor ( 1, k2 )
      y2 = Coor ( 2, k2 )

!    *  after subtraction of x2+i*y2: new k1, new k3

      x1 = Coor ( 1, k1 ) - x2
      y1 = Coor ( 2, k1 ) - y2

      x3 = Coor ( 1, k3 ) - x2
      y3 = Coor ( 2, k3 ) - y2

!    *  after multiplication by x3-i*y3: new k1

      x = x1 * x3 + y1 * y3
      y = x3 * y1 - x1 * y3

!    *  check the length of arms

      if ( abs(x) + abs(y).le.0d0 ) then

!       *  at least one of arms has zero length

         call mshz04 ( 'mshz00: zero length of one of arms' )

      end if

      mshz00 = atan2 ( y, x )

!    *  atan2 ( y, x )  returns arg (x+i*y) in range  ( -pi, pi ], &
!    *  avoid a negative result, even if caused by a roundoff error

      if ( mshz00.lt.0d0 ) mshz00 = max ( mshz00 + twopi, 0d0 )

      end
