      subroutine mshz14 ( trian, mrec, iQ, iR, &
                          npelm, npedge, mpoint, melem, &
                          npoint, nelem, Coor, Ref, kMesh, &
                          nbcord, icurve, Tau, iblend )
! ======================================================================
!
!       programmer      Zdenek Vlasek
!       version 2.0     date 31-07-1990  extra variables n1..4
!       version 1.0     date 14-02-1990
!
!
!   copyright (c) 1990-1990  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!   Decomposition of a one-layer quadrilateral ( when n2=n4=1, n1=n3 )
!   into quadrilateral elements.
!   iQ must be positive oriented.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical trian
      integer  mrec, iQ ( mrec ), iR ( mrec ), &
               npelm, npedge, mpoint, melem, npoint, nelem, &
               kMesh ( npelm, melem ), nbcord, icurve, iblend
      double precision Coor ( 2, mpoint ), Ref ( 2, mpoint ), &
                       Tau ( 2, nbcord )

!     coor    i/o   Real array (2,mpoint).
!                   Physical coordinates of all meshpoints
!     iblend   i    User's choice, see mshzde
!     icurve   i    User's choice, see mshzde
!     iq       i    Integer array of length mrec.
!                   The record must be initialized by calling subroutine
!                   to describe boundary of a quadrilateral region
!                   ( see mshz21 ), the region will be subdivided here.
!                   Notice, the values  n1, n2, n3, n4  are in iQ (2..5)
!     ir       -    Integer scratch array of length mrec
!     kmesh   i/o   Integer array (npelm,melem). Global nodenumbers for
!                   all elements created.
!     melem    i    Upper bound for nelem
!     mpoint   i    Upper bound for npoint
!     mrec     i    Upper bound for length of iQ
!     nbcord   i    Number of user's boundary points, see mshzde
!     nelem   i/o   Current number of elements created
!     npedge   i    Number of points per one element edge, both ends
!                   included
!     npelm    i    Number of points per element
!     npoint  i/o   Current number of all meshpoints
!     ref     i/o   Real array (2,mpoint). Reference coordinates of all
!                   meshpoints, the inner-element points are not needed, &
!                   see mshz21.
!     tau      i    Real array (2,nbcord). Reference nonequidistant
!                   coordinates of user's boundary points, see mshz21.
!     trian    i    .true.  = triangular elements ( not here )
!                   .false. = quadrilateral elements
!                   ( the information is passed to called subroutines )
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iel, i, inew, incr, isg, &
              n1, n2, n3, n4, ibeg2, iend2, ibeg4, iend4, nbpelm
      double precision db

!     db         Step per one element in the reference plane
!     i          Counting variable
!     ibeg2      Pointers into iQ, where is in iQ edge 2 of iR
!     ibeg4      Pointers into iQ, where is in iQ edge 4 of iR
!     iel        Loop control variable, local element number
!     iend2      Pointers into iQ, where is in iQ edge 2 of iR
!     iend4      Pointers into iQ, where is in iQ edge 4 of iR
!     incr       Number of intervals along one element edge
!     inew       Current position when iR is filled
!     isg        Signum of iQ(10)-iQ(6), the loopstep in many loops
!     n1         Number of elements along the  south  side
!                of the current quadrilateral
!     n2         Number of elements along the  east  side
!                of the current quadrilateral
!     n3         Number of elements along the  north side
!                of the current quadrilateral
!     n4         Number of elements along the  west side
!                of the current quadrilateral
!     nbpelm
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     MSHZ03    From given boundary points of one element
!               creates the whole element
!     MSHZ04    To display an error message and to stop the run
!     MSHZ07    Creates all new intermediate points between two given
!               points, using the mapping chosen
!     MSHZ18    To change the inner pointers in the record like iQ, &
!               due to a choice of new first vertex, &
!               the orientation is not influenced
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   The corresponding points on the opposite sides of iQ are connected.
!   Each element whose boundary ( it is stored in iR ) becomes known, &
!   is processed by mshz03
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!

      if ( trian ) call mshz04 ( '# mshz14: called for triangles' )

      if ( iQ ( 14 ).lt.0 ) call mshz04 ( '# mshz14: negative oriented' )

!     --- historicaly: this was written for column of elements, &
!                      not for layer of elements, &
!         so we transpose iQ both now and before the return

      call mshz18 ( iQ, 4, n1, n2, n3, n4 )

      incr = npedge - 1
      nbpelm = 4 * incr

!     --- step per one element

      db = 1d0 / n2

      isg = sign ( 1, iQ ( 10 ) - iQ ( 6 ) )

!     --- pointers ( into iQ ) for the corners of the first element
!         ( edge 2 and 4 of iR is extracted from edge 2 and 4 of iQ )

      ibeg2 = iQ ( 7 )
      iend2 = ibeg2 + isg * incr

      iend4 = iQ ( 13 )
      ibeg4 = iend4 - isg * incr

!     --- loop over elements

      do 100 iel = 1, n2

!     --- the situation:
!              |---------------|
!              |               |
!              |               |
!              |               |
!              |               |
!              |               |
!            ibeg4            iend2    - \
!              |               |          \  the element to be done now
!              |               |          /
!            iend4------------ibeg2    - /
!              |               |
!              |    old iR     |
!              ----------------- &
!              |               |
!              |               |       the case iel=1
!            iQ(13)-----------iQ(7)
!            iQ(6)            iQ(10)

         inew = 15

!        --- edge 1 of iR, including the endpoints

         iR ( 6 ) = 15

         if ( iel.eq.1 ) then

!        --- edge 1 of iR is edge 1 of iQ

            do 10 i = iQ ( 6 ), iQ ( 10 ), isg
               iR ( inew ) = iQ ( i )
               inew = inew + 1
   10       continue

         else

!        --- edge 1 of iR is reverse edge 3 of old iR and
!            the two parts do  n o t  collide in iR

            do 20 i = iR ( 12 ), iR ( 8 ), -1
               iR ( inew ) = iR ( i )
               inew = inew + 1
   20       continue

         end if

         iR ( 10 ) = inew - 1

!        --- edge 2, without the first point, but including the last point

         iR ( 7 ) = iR ( 10 )

         do 30 i = ibeg2 + isg, iend2, isg
            iR ( inew ) = iQ ( i )
            inew = inew + 1
   30    continue

         iR ( 11 ) = inew - 1

!        --- edge 3, without endpoints

         iR ( 8 ) = iR ( 11 )

         if ( iel.lt.n2 ) then

!        --- inner points of edge 3 do not exist yet, do them

            if ( incr.gt.1 ) then
               call mshz07 ( 1d0, db * iel, 0d0, db * iel, mrec, iQ, &
                  incr, iR ( inew ), mpoint, npoint, Coor, &
                  iQ ( iend2 ), iQ ( ibeg4 ), Ref, nbcord, Tau, iblend )
               inew = inew + ( incr - 1 )
            end if

         else

!        --- edge 3 of iR is edge 3 of iQ, without endpoints

            do 40 i = iQ ( 8 ) + isg, iQ ( 12 ) - isg, isg
               iR ( inew ) = iQ ( i )
               inew = inew + 1
   40       continue

         end if

         iR ( 12 ) = inew

!        --- edge 4, including both endpoints

         iR ( 9 ) = iR ( 12 )

         do 50 i = ibeg4, iend4, isg
            iR ( inew ) = iQ ( i )
            inew = inew + 1
   50    continue

         iR ( 13 ) = inew - 1

         iR ( 1 ) = iR ( 13 )
         iR ( 2 ) = 1
         iR ( 3 ) = 1
         iR ( 4 ) = 1
         iR ( 5 ) = 1
         iR ( 14 ) = 1

!        --- nodenumbers in iR are in the natural order, &
!            Element  must not destroy iR !!

         call mshz03 ( trian, npelm, nbpelm, npedge, mrec, iR, &
            nelem, melem, kMesh, &
            mpoint, npoint, Coor, Ref, &
            nbcord, icurve, Tau, iblend )

!        --- prepare for the next element

         ibeg2 = ibeg2 + isg * incr
         iend2 = iend2 + isg * incr
         ibeg4 = ibeg4 - isg * incr
         iend4 = iend4 - isg * incr

  100 continue

!     --- reverse transposition

      call mshz18 ( iQ, 2, n1, n2, n3, n4 )

      end
