      subroutine sepfillkmad3
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmesh ad  part 3: icrossobst,rcrossobst
!     Old subroutine topfillkmad3
! **********************************************************************
!
!                       KEYWORDS
!
!     memory_management
!     topology
!     mesh
!     element
!     node
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ichoice

!     ichoice        Integer choice parameter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLKMAD2   Fill array kmesh ad  part 2: ielemobst
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLNMESHH  Fill array nmesh part h
!     SEPFLKMESHAD2  Fill array kmesh part ad as well as the subarrays
!                    ipointobst and ielemobst (R^2 only)
!     SEPFLKMESHAD3  Fill array kmesh part ad as well as the subarrays
!                    ipointobst and ielemobst (R^3 only)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillkmad3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillkmad3'
         write(irefwr,1) 'kelmaa', kelmaa
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- activate old arrays nmesha

      if ( kelmaa==0 ) then

!     --- kelmaa = 0, Initialize nmesha

         call sepfillkmeshaa

      else if ( nmesh(31)==0 ) then

!     --- nmesh(31) = 0, Initialize nmesha

         call sepfillkmeshaa

      end if  ! ( kelmaa==0 )

      if ( nmesh(38)==0 ) call sepfillnmeshh

!     --- Array kmesh ad 3,4 does not exist, create

      if ( kelmad==0 ) then

!     --- kmesh ad has not been created before

         ichoice = 3

      else if ( kmeshad(2)==0 .or. kmeshad(5)/=kmeshad(7) ) then

!     --- kmesh ad pos. 2 does not exist: create

         if ( debug ) write(irefwr,*) 'before sepfillkmad2'
         call sepfillkmad2
         if ( debug ) write(irefwr,*) 'after sepfillkmad2'
         ichoice = -3

      else

         ichoice = -3

      end if

      if ( debug ) write(irefwr,1) 'ndim, ichoice', ndim, ichoice

      if ( ndim==2 ) then

!     --- ndim = 2

         if ( debug ) write(irefwr,*) 'before sepflkmeshad2'
         call sepflkmeshad2 ( ichoice )
         if ( debug ) write(irefwr,*) 'after sepflkmeshad2'

      else

!     --- ndim = 3

         call sepflkmeshad3 ( ichoice )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillkmad3'

      end if
      call erclos ( 'sepfillkmad3' )

      end subroutine sepfillkmad3
