      subroutine sepmsh026 ( irefc, leniwork, imaplc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     write mesh arrays to file irefc
!     Old subroutine msh026
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecpack
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer irefc, leniwork, imaplc

!     imaplc         i    if 1 the arrays mapar must be written
!     irefc          i    dataset reference number for reading (>0 formatted, &
!                         <0 unformatted)
!     leniwork       i    Length of array iwork
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istar3, istar4, icurnr, iinner, inpncv, istart, j, &
              inpelm, n, lniinp, lnrinp, lenkmsq, lenkmsr, &
              lenkmss, isurnr, ivolnr, isurf, nelemlc, npointlc, lenkmsh, &
              ielgrp, ikelmc, irelem, nextra, iwork(leniwork)
      character (len=80) text

!     i              Counting variable
!     icurnr         Curve sequence number
!     ielgrp         element group number
!     iinner         Number of inner curves in mesh
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     inpncv         Number of points in curve
!     irelem         Relative element sequence number with respect to
!                    element group
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istar4         Starting address of KMESHM part 4 in KMESH part M
!     istart         First position used at a part
!     isurf          Type of surface with respect to mesh generator
!     isurnr         Surface sequence number
!     ivolnr         Volume sequence number
!     iwork          integer work array of length npoint
!     j              Counting variable
!     lenkmsh        Length of array kmeshh
!     lenkmsq        Length of array kmeshq
!     lenkmsr        Length of array kmeshr
!     lenkmss        Length of array kmeshs
!     lniinp         Available space for array IINPUT
!     lnrinp         Available space for array RINPUT
!     n              length
!     nelemlc        Local number of elements
!     nextra         Number of extra variables stored in kmeshh(5,.)
!     npointlc       Local number of points
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Determine the characteristic values of array kmesh
!     Write all information either formatted (irefc>0) or
!     unformatted (irefc<0) to the backing storage file
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmsh026' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmsh026'
         call ertrace
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( irefc>=0 ) then

!     --- Write all coordinates (formatted) to backing storage

         write(irefc,100) '//Coordinates'
100      format(a)
         do j = 1, npoint
            write(irefc,110) ( coor(i,j), i = 1, ndim )
         end do
110      format(1x, 3e19.10)

!        --- Write all elements (formatted) to backing storage

         write(irefc,100) '//Topology'
         ikelmc = 0
         do ielgrp = 1, nelgrp
            inpelm = Topgrp(ielgrp)%inpelm
            inelem = Topgrp(ielgrp)%inelem
            do irelem = 1, inelem
               write(irefc,120) (kmeshc(ikelmc+i), i = 1, inpelm)
               ikelmc = ikelmc+inpelm
            end do
         end do
120      format(10i10)

!        --- Write all user points (formatted) to backing storage

         if ( nuspnt>0 ) then
            write(irefc,100) '//User_Points'
            write(irefc,120) (kmeshl(i), i = 1,nuspnt)
         end if

      else

!     --- Write all coordinates, elements and user points unformatted to
!         backing storage

         text = '//Coordinates'
         write(-irefc) text
         write(-irefc) coor
         ikelmc = 0
         text = '//Topology'
         write(-irefc) text
         do ielgrp = 1, nelgrp
            inpelm = Topgrp(ielgrp)%inpelm
            inelem = Topgrp(ielgrp)%inelem
            if ( inelem>0 ) then
               write(-irefc) (kmeshc(ikelmc+i), i = 1, inpelm*inelem)
               ikelmc = ikelmc+inpelm*inelem
            end if
         end do
         if ( nuspnt>0 ) then
            text = '//User_Points'
            write(-irefc) text
            write(-irefc) (kmeshl(i), i = 1, nuspnt)
         end if

      end if

      if ( kelmm/=0 ) then

!     --- ncurvs>0  write KMESH part m

         istar3 = kmeshm(2)-1
         istar4 = kmeshm(3)
         ninner = kmeshm(1)
         if ( irefc>=0 ) then
            write(irefc,100) '//Curves'
         else
            text = '//Curves'
            write(-irefc) text
         end if

!        --- Set iwork equal to 1 for all curves

         iwork(1:ncurvs) = 1

!        --- Set iwork equal to 0 for all inner curves and
!            -1 for all composite curves

         do i = 1, ninner
            j = kmeshm(istar4+i-1)   ! curve number inner curve
            if ( j>0 ) then
               iwork(j) = 0       ! inner curve
            else
               iwork(-j) = -1     ! composite curve
            end if  ! ( j>0 )
         end do

         do icurnr = 1, ncurvs
            iinner = iwork(icurnr)
            inpncv = kmeshm(5+icurnr)-kmeshm(4+icurnr)
            if ( irefc>=0 ) then
               write(irefc,120) inpncv, iinner
               write(irefc,120) (kmeshm(istar3+i), i = 1, inpncv)
            else
               write(-irefc) inpncv, iinner
               write(-irefc) (kmeshm(istar3+i), i = 1, inpncv)
            end if
            istar3 = istar3+inpncv
         end do
      end if

      if ( kelmn/=0 ) then

!     --- kelmn/=0, write kmesh part n

         istart = 2*nsurfs
         if ( irefc>=0 ) then
            write(irefc,100) '//Surfaces'
         else
            text = '//Surfaces'
            write(-irefc) text
         end if
         do isurnr = 1, nsurfs
            inelem = kmeshn(isurnr+nsurfs)
            inpelm = kmeshn(isurnr)
            if ( irefc>=0 ) then
               write(irefc, 120) inelem, inpelm
            else
               write(-irefc) inelem, inpelm
            end if
            if ( irefc>=0 ) then
               do j = 1, inelem
                  write(irefc,120) (kmeshn(i+istart), i = 1, inpelm)
                  istart = istart+inpelm
               end do
            else if ( inelem>0 ) then
               n = inelem*inpelm
               write(-irefc) (kmeshn(i+istart), i = 1, n)
               istart = istart+n
            end if
         end do

      end if

!     --- Write KMESH part j if necessary

      if ( kelmj>0 ) then

!     --- Renumbering has been performed

         if ( irefc>=0 ) then
            write(irefc,100) '//Renumbering'
         else
            text = '//Renumbering'
            write(-irefc) text
         end if
         if ( irefc>=0 ) then
            write(irefc,120) (kmeshj(i), i = 1, npoint)
         else
            write(-irefc) (kmeshj(i), i = 1, npoint)
         end if

      end if

!     --- Write KMESH t if necessary

      if ( kelmt/=0 ) then

!     --- kelmt>0, write kmesh t

         n = nvolms
         if ( n==0 ) n = nsurfs
         if ( n==0 ) n = ncurvs
         if ( irefc>=0 ) then
            write(irefc,100) '//Kmesht'
         else
            text = '//Kmesht'
            write(-irefc) text
         end if
         if ( irefc>=0 ) then
            write(irefc,120) kmesht
         else
            write(-irefc)  kmesht
         end if

      end if

      if ( kelmu>0 ) then

!     --- Spectral elements, write contents of kmesh to backing storage

         if ( irefc>=0 ) then

!        --- Formatted

            write(irefc,100) '//Spectral_Elements'
            write(irefc,120) ( kmeshu(i), i = 1, 5 )

         else

!        --- Unformatted

            text =  '//Spectral_Elements'
            write(-irefc) text
            write(-irefc)  ( kmeshu(i), i = 1, 5 )

         end if

      end if

!     --- write arrays iinput and rinput

      lniinp  = 0
      lnrinp  = 0
      lenkmsh = 0
      lenkmsq = 0
      lenkmsr = 0
      lenkmss = 0

      if ( .not. parallel ) then

!     --- Serial computation

         lniinp  = size(iinputkm)
         lnrinp  = size(rinputkm)
         if ( lniinp==1 ) lniinp = 0
         if ( lnrinp==1 ) lnrinp = 0

      end if  ! ( .not. parallel )

      if ( kelmh>0 ) lenkmsh = size(kmeshh)
      if ( kelmq/=0 ) lenkmsq = size(kmeshq)
      if ( kelmr/=0 ) lenkmsr = size(kmeshr)
      if ( kelms/=0 ) lenkmss = size(kmeshs)
      if ( irefc>=0 ) then

!     --- Formatted

         write(irefc,100) '//Length_Info'
         write(irefc,120) lniinp, lnrinp, lenkmsh, lenkmsq, lenkmsr, &
                          lenkmss
         if ( lniinp>0 ) then
            write(irefc,100) '//Iinput'
            write(irefc,120) iinputkm
         end if
         if ( lnrinp>0 ) then
            write(irefc,100) '//Rinput'
            write(irefc,110) rinputkm
         end if
         if ( lenkmsq>0 ) then
            write(irefc,100) '//Kmeshq'
            write(irefc,120) kmeshq
         end if
         if ( lenkmsr>0 ) then
            write(irefc,100) '//Kmeshr'
            write(irefc,120) kmeshr
         end if
         if ( lenkmss>0 ) then
            write(irefc,100) '//Kmeshs'
            write(irefc,120) kmeshs
         end if
         if ( lenkmsh>0 ) then
            n = size(curvesmsh,2)
            write(irefc,100) '//Kmeshh'
            kmeshh(5,1) = n
            write(irefc,120) kmeshh
         end if

      else

!     --- Unformatted

         text = '//Length_Info'
         write(-irefc) text
         write(-irefc) lniinp, lnrinp, lenkmsh, lenkmsq, lenkmsr, &
                       lenkmss
         if ( lniinp>0 ) then
            text = '//Iinput'
            write(-irefc) text
            write(-irefc)  iinputkm
         end if
         if ( lnrinp>0 ) then
            text = '//Rinput'
            write(-irefc) text
            write(-irefc) rinputkm
         end if
         if ( lenkmsq>0 ) then
            text = '//Kmeshq'
            write(-irefc) text
            write(-irefc) kmeshq
         end if
         if ( lenkmsr>0 ) then
            text = '//Kmeshr'
            write(-irefc) text
            write(-irefc) kmeshr
         end if
         if ( lenkmss>0 ) then
            text = '//Kmeshs'
            write(-irefc) text
            write(-irefc) kmeshs
         end if
         if ( lenkmsh>0 ) then
            text = '//Kmeshh'
            write(-irefc) text
            n = size(curvesmsh,2)
            kmeshh(5,1) = n
            write(-irefc) kmeshh
         end if

      end if

!     --- Write contents of kmesh part h Section icurvs/curves

      if ( kelmh>0 ) then

!     --- kmesh part h exists
!         write icurvs

         if ( irefc>=0 ) then
            write(irefc,120) icurvsmsh
         else
            write(-irefc) icurvsmsh
         end if

!        --- write curves

         if ( irefc>=0 ) then
            write(irefc,110) curvesmsh
         else
            write(-irefc) curvesmsh
         end if

!        --- write curvefacts

         if ( irefc>=0 ) then
            write(irefc,110) curvefacts
         else
            write(-irefc) curvefacts
         end if

      end if

!     --- Write contents of kmesh part h Section mapar

      if ( kelmh>0 .and. imaplc>0 ) then

!     --- kmesh part h exists

         do isurnr = 1, nsurfs
            isurf = kmeshs(nsurfs+2+(isurnr-1)*8)
            if ( isurf==24 ) isurf = 7    ! compound surface
            if ( debug ) write(irefwr,1) 'isurnr, isurf', isurnr, isurf
            if ( isurf>0 .and. isurf/=7 .and. isurf/=11 .and. &
                 isurf/=12 .and. kmeshh(3,isurnr+1)>0 ) then

!           --- Actual surface
!               Read and write mapar

               nelemlc = size(kmhsur(isurnr)%kmeshcsur)
               npointlc = kmeshh(1,isurnr+1)
               if ( kmeshh(5,isurnr+1)>0 ) then
                  nextra = size(kmhsur(isurnr)%renumsur)
               else
                  nextra = 0
               end if
               iwork(1:npointlc) = kmhsur(isurnr)%maparsur(1:npointlc)
               if ( irefc>=0 ) then
                  write(irefc,120) npointlc, nelemlc, nextra
                  write(irefc,120) (iwork(i),i=1,npointlc)
               else
                  write(-irefc) npointlc, nelemlc, nextra
                  write(-irefc) (iwork(i),i=1,npointlc)
               end if
               if ( debug ) call prinin ( iwork, npointlc, 'iwork/1' )
               if ( isurf==2 ) then
                  npointlc = size(kmhsur(isurnr)%renumsur)
                  iwork(1:npointlc) = kmhsur(isurnr)%renumsur
                  if ( irefc>=0 ) then
                     write(irefc,120) npointlc
                     write(irefc,120) (iwork(i),i=1,npointlc)
                  else
                     write(-irefc) npointlc
                     write(-irefc) (iwork(i),i=1,npointlc)
                  end if
               else if ( isurf==7 .or. isurf==11 .or. &
                         isurf==12 ) then
                  npointlc = size(kmhsur(isurnr)%renumsur)
                  iwork(1:npointlc) = kmhsur(isurnr)%renumsur
                  if ( irefc>=0 ) then
                     write(irefc,120) npointlc
                     write(irefc,120) (iwork(i),i=1,npointlc)
                  else
                     write(-irefc) npointlc
                     write(-irefc) (iwork(i),i=1,npointlc)
                  end if
               end if
            end if

         end do

         do ivolnr = nsurfs+1, nsurfs+nvolms

!        --- Read and write mapar

            if ( kmeshh(3,ivolnr+1)>0 )then

               nelemlc = size(kmhsur(ivolnr)%kmeshcsur)
               npointlc =  kmeshh(1,ivolnr+1)
               iwork(1:npointlc) = kmhsur(ivolnr)%maparsur(1:npointlc)
               if ( irefc>=0 ) then
                  write(irefc,120) npointlc, nelemlc
                  write(irefc,120) (iwork(i),i=1,npointlc)
               else
                  write(-irefc) npointlc, nelemlc
                  write(-irefc) (iwork(i),i=1,npointlc)
               end if

            end if

         end do

      end if

      iwork(1:10) = 0
      if ( irefc>=0 ) then
         write(irefc,100) '//Dimension_Parameters'
         write(irefc,120) nlevel, nsplit, (iwork(i),i=1,8)
         write(irefc,120) (iwork(i),i=1,10)
      else
         text = '//Dimension_Parameters'
         write(-irefc) text
         write(-irefc) nlevel, nsplit, (iwork(i),i=1,8)
         write(-irefc) (iwork(i),i=1,10)
      end if

      if ( .not. parallel .and. kelmk>0 ) then

!     --- kmesh part k has been filled, write extra records

         if ( irefc>=0 ) then
            write(irefc,100) '//Kmeshk'
            write(irefc,120) (kmeshk(i),i=1,nlevel+nsplit)
         else
            text = '//Kmeshk'
            write(-irefc) text
            write(-irefc) (kmeshk(i),i=1,nlevel+nsplit)
         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmsh026'

      end if  ! ( debug )
      call erclos ( 'sepmsh026' )

      end subroutine sepmsh026
