      subroutine msh008 ( isubr, inpada, isol, &
                          icopad, nadapt, icurves )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 23-03-2001 New values of iadapt
!        version  1.3  date 03-08-1999 Extension of inpada
!        version  1.2  date 31-01-1997 Extension of inpada
!
!   copyright (c) 1996-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Fill array icopad with values from array inpada and extend with default
!   values.      Perform some checks
!   Help subroutine for adapb... subroutines
! **********************************************************************
!
!                       KEYWORDS
!
!     adapt_boundary
!     free_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulemain
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpada(*), isol, nadapt, icopad(*), &
              icurves(*), isubr

!     icopad     o    Copy of inpada extended with default values
!     icurves    i    Integer array containing the curve numbers for the
!                     computation of the boundary integral. A negative curve
!                     number indicates that the curve must be computed in
!                     the reversed direction.
!     inpada     i    Integer array to be filled by the user, indicating how the
!                     boundary must be adapted.
!                     INPADA must be filled as follows:
!                     Pos. 1   Position of last entry that has been filled by
!                              the user. If inpada(1) = 0 or 1, only defaults
!                              are used.
!                     Pos. 2   icrv1, Curve number of first curve to be adapted
!                              In case of call by adapboun: number of curves
!                              Default value: 1
!                     Pos. 3   icrv2, Curve number of last curve to be adapted
!                              Remark: the curve numbers icrv1 to icrv2 must
!                              be subsequent curves (Only for call by adapbn)
!                              Default value: icrv1
!                     Pos. 4   iadapt Indicates how the boundary must be adapted
!                              Possible values:
!                              1.  The curves are adapted by:
!                                  xnew(i) = xold(i) + alpha(i) n(i)
!                                  with xold(i) the old co-ordinates of the
!                                  i-th node and xnew(i) the new ones.
!                                  n(i) is the normal at the boundary of the
!                                  i-th node and alpha(i) the number-th degree
!                                  of freedom in this node
!                                  The normal is computed assuming a counter- &
!                                  clockwise direction of the boundary
!                              2.  The curves are adapted by the user provided
!                                  function subroutine FUNCCR
!                              3.  See 1, however, now alpha is computed
!                                  according to the 'film method'
!                                  See: Caswell and Viriyayuthakorn, &
!                                  Finite element simulation of die swell for a
!                                  Maxwell fluid, J. of Non-Newt. Fluid Mech.
!                                  Vol 12, 13-29(1983).
!                                  alpha(i) = factor*(psi(i)-psi(0))/|vt(i)|, &
!                                                   xi
!                                  psi(i)-psi(0) = /  v.n dx
!                                                 x0
!                                  The vector isol must be the velocity vector
!                                  with degrees of freedom number and number+1
!                                  from isol.  Cartesian coordinates.
!                                  Starting flow rate integration along free
!                                  boundary with flow rate zero in first point
!                                  of curves.
!                              4.  Other method according to:
!                                  xnew(i) = xold(i) + v(i) * factor
!                                  Time-dependent case if imult=1
!                                  (see inpada(11))
!                                  In the axisymmetric case we use:
!                                  (r,z)new(i) = (r,z)old(i) + &
!                                                 (vr,vz)(i) dt /(2 pi r(i))
!                              5.  See 1. Now alpha is computed according to
!                                  alpha=v.n * factor
!                              6.  See 1. Now alpha is computed according to
!                                  alpha=(grad c).n * factor
!                                  It is assumed that grad c is stored in
!                                  a vector of special structure defined per
!                                  element
!                              7.  See 6. The more accurate scheme described
!                                  in Segal, Vuik and Vermolen is used
!                                  Use omega, eps and maxiter as stored in
!                                  rinada(7), rinada(6) and inpada(18)
!                              8.  Constant velocity i.e.
!                                  xnew(i) = xold(i) + u * factor
!                                  where u is a given velocity vector
!                                  Time-dependent case if imult=1
!                                  (see inpada(11))
!                              9.  Using FUNCSOLCR
!                              Default value: 1
!                     Pos. 5   idirec Indicates the direction of the computation
!                              If idirec = 1 the computation is performed
!                              from first to last point, &
!                              if idirec = -1 from last to first point
!                              It is sometimes required to alternate between 1
!                              and -1 during iteration.
!                              This option is only used for iadapt = 3
!                              Default value: 1
!                     Pos. 6   number, the number-th degree of freedom of isol
!                              (and higher) are used for the computation of
!                              alpha.
!                              See iadapt.
!                              Default value: 1
!                     Pos. 7   linear. Indicates if the elements along the
!                              boundary must be treated as linear or as
!                              quadratic elements.
!                              Possible values:
!                              1: linear
!                              2: quadratic
!                              Default value: to be computed by the subroutine
!                              in the following way:
!                              If all internal elements are quadratic, the
!                              boundary elements are assumed quadratic, &
!                              otherwise linear
!                     Pos. 8   jcart defines the type of co-ordinate system to
!                              be used. This is of importance for
!                              iadapt = 3, 4 and 5
!                              Possible values:
!                              1:  Cartesian co-ordinates
!                              2:  Axi-symmetric co-ordinates
!                              Default value: 1
!                     Pos. 9   iexlude_1: indicates which of the co-ordinates of
!                              the first point of the first curve must be
!                              adapted.
!                              Possible values:
!                              0  Both co-ordinates must be adapted
!                              1  Only the second co-ordinate must be adapted, &
!                                 the first one remains unchanged
!                              2  Only the first co-ordinate must be adapted, &
!                                 the second one remains unchanged
!                              3  Both co-ordinates remain unchanged
!                              Default value: 0
!                     Pos. 10  iexlude_2: indicates which of the co-ordinates of
!                              the last point of the last curve must be adapted.
!                              Possible values:
!                              0  Both co-ordinates must be adapted
!                              1  Only the second co-ordinate must be adapted, &
!                                 the first one remains unchanged
!                              2  Only the first co-ordinate must be adapted, &
!                                 the second one remains unchanged
!                              3  Both co-ordinates remain unchanged
!                              Default value: 0
!                     Pos. 11  imult: indicates how the value of factor must be
!                              computed. (iadapt>2)
!                              Possible values:
!                              0  Multiplication factor is 1
!                              1  Multiplication factor is dt from common ctime
!                              2  Multiplication factor is stored in rinada(1)
!                              Default value: 0
!                     Pos. 12  ithres: indicates if a threshold value is used
!                              in Caswell's method and where to find it.
!                              Possible values:
!                              0  Threshold value = 0
!                              1  Threshold value is stored in rinada(2)
!                              Default value: 0
!                     Pos. 13  iangle_1: Indicates if angle of freesurface at
!                              first point is prescribed or not
!                              Possible values:
!                              0  angle is not given
!                              1  angle is stored in rinada(3)
!                              Default value: 0
!                     Pos. 14  iangle_2: Indicates if angle of freesurface at
!                              last point is prescribed or not
!                              Possible values:
!                              0  angle is not given
!                              1  angle is stored in rinada(4)
!                              Default value: 0
!                     Pos. 15  redistribute nodes
!                              Possible values:
!                              0  no node redistribution
!                              1  nodes are redistributed, nelm constant
!                              2  nodes are redistributed, nelm recomputed
!                                 according to coarseness
!                              Default value: 0
!                     Pos. 16  Plot_boun
!                              Possible values:
!                              0  do not plot new boundary
!                              1  plot new boundary
!                              Default value: 0
!                     Pos. 17  Numobst (Number of obstacles)
!                              Each obstacle must be closed and consist of
!                              one curve only (This may be a curve of curves)
!                              At most 10 obstacles may be used.
!                              The curve numbers of the Numobst obstacles are
!                              stored in array icurvs positions:
!                              ncurves+ncurves_redist+1 ... .+Numobst
!                              Default value: 0
!                     Pos. 18  Maxiter, maximum number of iterations in
!                              iteration process
!                              Default value: 10
!                     Pos. 19  Prin_boun
!                              Possible values:
!                              0  do not plot new boundary
!                              1  print new boundary
!                              2  print new boundary, use xmin, xmax, ymin
!                                 and ymax from rinada(8-11)
!                              Default value: 10
!                     Pos. 20  Check_dir
!                              Possible values:
!                              -1 inwards pointed normal direction
!                              0  no check
!                              1  outwards pointed normal direction
!                              Default value: 0
!     isol       i    Standard SEPRAN array containing information of the
!                     solution array
!     isubr      i    Defines the calling subroutine.
!                     Possible values:
!                     1:  Subroutine adapbn
!                     2:  Subroutine adapboun
!     nadapt     i    Length of array icopad
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer itypvc, i, ncurves, icurnr, &
              icrv1, icrv2, iadapt, idirec, jcart, iexclu, imult, iparm, &
              linear, maxiter

!     i       Counting variable
!     iadapt  Parameter iadapt as described in inpada(4)
!     icrv1   First curve number
!     icrv2   Last curve number
!     icurnr  Curve sequence number
!     idirec  Parameter idirec as described in inpada(5)
!     iexclu  Parameter idirec as described in inpada(9,10)
!     imult   Parameter imult as described in inpada(11)
!     iparm   Parameter in inpada
!     itypvc  Indication of the type of solution vector:
!             110: solution vector
!             115, 116: vectors of special structure
!     jcart   Parameter jcart as described in inpada(8)
!     linear  Parameter iadapt as described in inpada(7)
!     maxiter Maximum number of iterations
!     ncurves Number of curves in icurves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      60   Input solution array has not been filled or is of the wrong
!     151   first curve number has incorrect value
!     152   last curve number has incorrect value
!     419   NDIM # 2
!     473   Array KMESH is not the result of a call of subroutine MESH.
!    1475   inpada(i) out of range
!    1950   Curve number out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Perform standard tests
!    Fill icopad with default values
!    Copy contents from inpada into icopad
!    Check values
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Standard tests

      call eropen ( 'msh008' )
      if ( kelmh==0 ) call errsub ( 473, 0, 0, 0)
      if ( ndim/=2 ) then
         call errint ( ndim, 1 )
         call errint ( 2, 2 )
         call errint ( 2, 3 )
         call errsub ( 419, 3, 0, 0 )
      end if
      itypvc = ks(isol)%typevec
      if ( itypvc/=110 .and. itypvc/=115 ) then
         call errint ( ks(isol)%typevec, 1 )
         call errsub ( 60, 1, 0, 0)
      end if

!     --- Fill icopad with default values

      do i = 1, nadapt
         icopad(i) = 0
      end do
      icopad(2) = 1
      icopad(4) = 1
      icopad(5) = 1
      icopad(6) = 1
      icopad(8) = 1
      icopad(18) = 10

!     --- Copy contents from inpada into icopad
!         and fill last defaults

      do i = 1, min(inpada(1),nadapt)
         icopad(i) = inpada(i)
      end do
      if ( icopad(2)==0 ) icopad(2) = 1
      if ( icopad(3)==0 ) icopad(3) = icopad(2)
      if ( icopad(5)==0 ) icopad(5) = 1
      if ( icopad(6)==0 ) icopad(6) = 1
      if ( icopad(7)==0 ) then

!     --- The type of elements has not yet been detected
!         detect if all volume elements are quadratic or not

         linear = 2
         do i = 1, nelgrp
            ishape = kmeshf(i)
            if ( ishape==3 .or. ishape==5 ) linear = 1
         end do
         icopad(7) = linear
      end if

!     --- Check values

      if ( inpada(1)<0 .or. inpada(1)>nadapt ) then

!     --- inpada(1) out of range

         call errint ( 1, 1 )
         call errint ( inpada(1), 2 )
         call errint ( 0, 3 )
         call errint ( nadapt, 4 )
         call errsub ( 1475, 4, 0, 0 )
      end if
      if ( isubr==1 ) then

!     --- call by adapbn

         icrv1  = icopad(2)
         icrv2  = icopad(3)
         if ( icrv1<=0 .or. icrv1>ncurvs ) then

!        --- icrv1 out of range

            call errint ( icrv1, 1 )
            call errint ( ncurvs, 2 )
            call errsub ( 151, 2, 0, 0 )
         end if
         if ( icrv2<icrv1 .or. icrv2>ncurvs ) then

!        --- icrv2 out of range

            call errint ( icrv2, 1 )
            call errint ( icrv1, 2 )
            call errint ( ncurvs, 3 )
            call errsub ( 152, 3, 0, 0 )
         end if
      else

!     --- call by adapboun

         ncurves = icopad(2)
         if ( ncurves<1 .or. ncurves>ncurvs ) then

!        --- ncurves out of range

            call errint ( 2, 1 )
            call errint ( ncurves, 2 )
            call errint ( 1, 3 )
            call errint ( ncurvs, 4 )
            call errsub ( 1475, 4, 0, 0 )
         end if
         do i = 1, ncurves+icopad(15)/100+icopad(17)
            icurnr = abs(icurves(i))
            if ( icurnr<1 .or. icurnr>ncurvs ) then
               call errint ( i, 1 )
               call errint ( icurnr, 2 )
               call errint ( ncurvs, 3 )
               call errsub ( 1950, 3, 0, 0 )
            end if
         end do
      end if
      iadapt = icopad(4)
      if ( iadapt<1 .or. iadapt>9 ) then

!     --- iadapt out of range

         call errint ( 4, 1 )
         call errint ( iadapt, 2 )
         call errint ( 1, 3 )
         call errint ( 5, 4 )
         call errsub ( 1475, 4, 0, 0 )
      end if
      idirec = icopad(5)
      if ( idirec/=1 .and. idirec/=-1 ) then

!     --- idirec out of range

         call errint ( 5, 1 )
         call errint ( idirec, 2 )
         call errint ( -1, 3 )
         call errint ( 1, 4 )
         call errsub ( 1475, 4, 0, 0 )
      end if
      linear = icopad(7)
      if ( linear<1 .or. linear>2 ) then

!     --- linear out of range

         call errint ( 7, 1 )
         call errint ( linear, 2 )
         call errint ( 1, 3 )
         call errint ( 2, 4 )
         call errsub ( 1475, 4, 0, 0 )
      end if
      jcart = icopad(8)
      if ( jcart<1 .or. jcart>2 ) then

!     --- jcart out of range

         call errint ( 8, 1 )
         call errint ( jcart, 2 )
         call errint ( 1, 3 )
         call errint ( 2, 4 )
         call errsub ( 1475, 4, 0, 0 )
      end if
      do i = 8, 10
         iexclu = icopad(i)
         if ( iexclu<0 .or. iexclu>3 ) then

!        --- iexclu out of range

            call errint ( i, 1 )
            call errint ( iexclu, 2 )
            call errint ( 0, 3 )
            call errint ( 3, 4 )
            call errsub ( 1475, 4, 0, 0 )
         end if
      end do
      imult = icopad(11)
      if ( imult<0 .or. imult>2 ) then

!     --- imult out of range

         call errint ( 11, 1 )
         call errint ( imult, 2 )
         call errint ( 0, 3 )
         call errint ( 2, 4 )
         call errsub ( 1475, 4, 0, 0 )
      end if
      do i = 12, 14
         iparm = icopad(i)
         if ( iparm<0 .or. iparm>1 ) then

!        --- iparm out of range

            call errint ( i, 1 )
            call errint ( iparm, 2 )
            call errint ( 0, 3 )
            call errint ( 1, 4 )
            call errsub ( 1475, 4, 0, 0 )
         end if
      end do
      iparm = icopad(15)
      if ( iparm<0 .or. iparm>2 ) then

!     --- iparm out of range

         call errint ( 15, 1 )
         call errint ( iparm, 2 )
         call errint ( 0, 3 )
         call errint ( 2, 4 )
         call errsub ( 1475, 4, 0, 0 )
      end if
      iparm = icopad(16)
      if ( iparm<0 .or. iparm>1 ) then

!     --- iparm out of range

         call errint ( 16, 1 )
         call errint ( iparm, 2 )
         call errint ( 0, 3 )
         call errint ( 2, 4 )
         call errsub ( 1475, 4, 0, 0 )
      end if
      iparm = icopad(17)
      if ( iparm<0 .or. iparm>10 ) then

!     --- iparm out of range

         call errint ( 17, 1 )
         call errint ( iparm, 2 )
         call errint ( 0, 3 )
         call errint ( 10, 4 )
         call errsub ( 1475, 4, 0, 0 )
      end if
      maxiter = icopad(18)
      if ( maxiter<1 .or. maxiter>50 ) then

!     --- maxiter out of range

         call errint ( 18, 1 )
         call errint ( maxiter, 2 )
         call errint ( 1, 3 )
         call errint ( 50, 4 )
         call errsub ( 1475, 4, 0, 0 )
      end if
      call erclos ( 'msh008' )
      end
