      subroutine msh047 ( inpada, rinada, nunkp, kprobf, &
                          indprf, usol, iwork, &
                          lengcv, coorol, anorm, icurves, iwork1, &
                          rinput, ncurves, iellist, &
                          closed, nbefor, jcount, n, &
                          uold, maxphys, numvecs )
! ======================================================================
!
!        programmer    Guus Segal and Jaap van der Zanden
!        version 14.0  date 06-01-2009 Removal of work
!        version 13.0  date 29-12-2008 New body
!        version 12.0  date 05-11-2008 Remove kprobh
!
!   copyright (c) 1986-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     recompute co-ordinates of curves
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kprobf(*), iwork(*), iwork1(*), maxphys, &
              inpada(*), nunkp, indprf, icurves(*), &
              ncurves, iellist(*), &
              jcount, nbefor, n, numvecs, lengcv
      double precision usol(*), &
                       coorol(2,*), anorm(2,*), rinada(*), &
                       rinput(2,*), uold(*)
      logical closed

!     anorm      i    real work array containing the norms in the points
!                     on the curves in icurves
!     closed     i    If true the boundary is closed
!     coorol     i    array containing the old co-ordinates of the
!                     points on the curves in icurves
!     icurves    i    Integer array containing the curve numbers that
!                     must be adapted. It is assumed that these curves form
!                     one continuous curve. Compound curves must have
!                     been represented by their subcurves.
!     iellist         Array of length npoint-1 containing a
!                     list of elements that are adjacent to the boundary
!                     element iellist(i) contains both nodal points
!                     iwork(i) and iwork(i+1)
!     indprf     i    indication whether kprob part f is used or not
!     inpada     i    Integer array to be filled by the user indicating how the
!                     boundary must be adapted.
!                     INPADA must be filled as follows:
!                     Pos. 1   Position of last entry that has been filled by
!                              the user. If inpada(1) = 0 or 1 only defaults
!                              are used.
!                     Pos. 2   ncurves number of curves in icurves
!                     Pos. 3   - &
!                     Pos. 4   iadapt Indicates how the boundary must be adapted
!                              Possible values:
!                              1.  The curves are adapted by:
!                                  xnew(i) = xold(i) + alpha(i) n(i)
!                                  with xold(i) the old co-ordinates of the
!                                  i-th node and xnew(i) the new ones.
!                                  n(i) is the normal at the boundary of the
!                                  i-th node and alpha(i) the number-th degree
!                                  of freedom in this node
!                                  The normal is computed assuming a counter- &
!                                  clockwise direction of the boundary
!                              2.  The curves are adapted by the user provided
!                                  function subroutine FUNCCR
!                              3.  See 1 however now alpha is computed
!                                  according to the 'film method'
!                                  See: Caswell and Viriyayuthakorn &
!                                  Finite element simulation of die swell for a
!                                  Maxwell fluid J. of Non-Newt. Fluid Mech.
!                                  Vol 12 13-29(1983).
!                                  alpha(i) = factor*(psi(i)-psi(0))/|vt(i)| &
!                                                   xi
!                                  psi(i)-psi(0) = /  v.n dx
!                                                 x0
!                                  The vector isol must be the velocity vector
!                                  with degrees of freedom number and number+1
!                                  from isol.  Cartesian coordinates.
!                                  Starting flow rate integration along free
!                                  boundary with flow rate zero in first point
!                                  of curves.
!                              4.  Other method according to:
!                                  xnew(i) = xold(i) + v(i) * factor
!                                  Time-dependent case if imult=1
!                                  (see inpada(11))
!                                  In the axisymmetric case we use:
!                                  (rz)new(i) = (rz)old(i) + &
!                                                 (vrvz)(i) dt /(2 pi r(i))
!                              5.  See 1. Now alpha is computed according to
!                                  alpha=v.n * factor
!                              6.  See 1. Now alpha is computed according to
!                                  alpha=(grad c).n * factor
!                                  It is assumed that grad c is stored in
!                                  a vector of special structure defined per
!                                  element
!                              7.  See 6. The more accurate scheme described
!                                  in Segal Vuik and Vermolen is used
!                              8.  Constant velocity i.e.
!                                  xnew(i) = xold(i) + u * factor
!                                  where u is a given velocity vector
!                                  ( rinada(15-17) )
!                                  Time-dependent case if imult=1
!                                  (see inpada(11))
!                     Pos. 5   idirec Indicates the direction of the computation
!                              If idirec = 1 the computation is performed
!                              from first to last point &
!                              if idirec = -1 from last to first point
!                              It is sometimes required to alternate between 1
!                              and -1 during iteration.
!                              This option is only used for iadapt = 3
!                     Pos. 6   number the number-th degree of freedom of isol
!                              (and higher) are used for the computation of
!                              alpha. See iadapt.
!                     Pos. 7   linear. Indicates if the elements along the
!                              boundary must be treated as linear or as
!                              quadratic elements.
!                              Possible values:
!                              1: linear
!                              2: quadratic
!                     Pos. 8   jcart defines the type of co-ordinate system to
!                              be used. This is of importance for
!                              iadapt = 3 4 and 5
!                              Possible values:
!                              1:  Cartesian co-ordinates
!                              2:  Axi-symmetric co-ordinates
!                     Pos. 9   iexlude_1: indicates which of the co-ordinates
!                              of the first point of the first curve must be
!                              adapted.
!                              Possible values:
!                              0  Both co-ordinates must be adapted
!                              1  Only the second co-ordinate must be adapted &
!                                 the first one remains unchanged
!                              2  Only the first co-ordinate must be adapted &
!                                 the second one remains unchanged
!                              3  Both co-ordinates remain unchanged
!                     Pos. 10  iexlude_2: indicates which of the co-ordinates of
!                              the last point of the last curve must be adapted.
!                              Possible values:
!                              0  Both co-ordinates must be adapted
!                              1  Only the second co-ordinate must be adapted &
!                                 the first one remains unchanged
!                              2  Only the first co-ordinate must be adapted &
!                                 the second one remains unchanged
!                              3  Both co-ordinates remain unchanged
!                     Pos. 11  imult: indicates how the value of factor must be
!                              computed. (iadapt>2)
!                              Possible values:
!                              0  Multiplication factor is 1
!                              1  Multiplication factor is tstep from common
!                                 ctimen
!                              2  Multiplication factor is stored in rinada(1)
!                              3  Multiplication factor is dt * rinada(1)
!                     Pos. 12  ithres: indicates if a threshold value is used
!                              in Caswell's method and where to find it.
!                              Possible values:
!                              0  Threshold value = 0
!                              1  Threshold value is stored in rinada(2)
!                     Pos. 13  iangle_1: Indicates if angle of freesurface at
!                              first point is prescribed or not
!                              Possible values:
!                              0  angle is not given
!                              1  angle is stored in rinada(3)
!                             <0  The first point must be projected onto
!                                 the curve c |iangle_1|
!                     Pos. 14  iangle_2: Indicates if angle of freesurface at
!                              last point is prescribed or not
!                              Possible values:
!                              0  angle is not given
!                              1  angle is stored in rinada(4)
!                             <0  The last point must be projected onto
!                                 the curve c |iangle_2|
!     iwork      i    integer work array containing the nodal points of the
!                     curves in icurves at output
!     iwork1     i    integer work array of length npoint
!     jcount     i   Counter
!     kprobf     i    KPROB part f: contains the number of unknowns per
!                     point accumulated
!     lengcv     i
!     maxphys    i    Maximum number of unknowns per point
!     n          i    number of nodes in boundary
!     nbefor     i    number of oiginal nodes in boundary
!     ncurves    i    Number of curves in icurves
!     numvecs    i    Number of vectors in iseqvecs
!     nunkp      i    maximal number of degrees of freedom in nodal points
!     rinada     i    Real input array with data as described by inpada
!     rinput    i/o   Real input array corresponding to subroutine MESH
!                     Some co-ordinates in rinput may have been changed
!     uold       i    Array containing the old solutions
!     usol       i    solution array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inodp, j, jcurve, &
              icount, istart, icurve, idirec, icurnr, iadapt, nnodes
      double precision :: coornw(2,lengcv)

!     coornw         array containing the new co-ordinates of the
!                    points on the curves in icurves
!     i              Counting variable
!     iadapt         Indicates how the boundary must be adapted, see INPADA
!     icount         Counter
!     icurnr         Curve sequence number
!     icurve         Absolute value of jcurve
!     idirec         Defines in which direction the curves must be considered
!                    idirec = 1, from low to high, idirec = -1: reversed order
!     inodp          Nodal point number
!     istart         Starting address in array VECOLD
!     j              Counting variable
!     jcurve         Curve sequence number provided with direction sign
!     nnodes         Number of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH047UPD      recompute co-ordinates of curves
!     MSHCPCOM       Fill array curves in case of composite curves by copying
!                    from the single curves it consists of
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Make initializations
!     compute new co-ordinates and store in array coornw
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- initializations

      call eropen ( 'msh047' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information in msh047'
         call prinin ( inpada, 15, 'inpada' )

      end if

      iadapt = inpada(4)

!     --- Update the coordinates

      call msh047upd ( inpada, rinada, nunkp, curvesmsh, kprobf, &
                       indprf, usol, iwork, coornw, &
                       coorol, anorm, iellist, &
                       icurvsmsh, closed, n, uold, maxphys, numvecs )

!     --- fill array curves

      icount = 0

      do j = 1, ncurves

         jcurve = icurves(j)
         icurve = abs(jcurve)
         if ( icurve==1 ) then
            nnodes = icurvsmsh(icurve)
         else
            nnodes = icurvsmsh(icurve)-icurvsmsh(icurve-1)
         end if
         if ( jcurve>0 ) then
            if ( icurve==1 ) then
               istart = 1
            else
               istart = icurvsmsh(icurve-1)+1
            end if
            idirec = 1
         else
            istart = icurvsmsh(icurve)
            idirec = -1
         end if
         do i = 1, nnodes
            curvesmsh(:,istart+idirec*(i-1)) = coornw(:,icount+i)
         end do
         icount = icount+nnodes-1

      end do

!     --- refill array rinput for new co-ordinates

      if ( debug ) then

!     --- Debug information

         call prinin ( icurves, ncurves, 'icurves' )
         call prinin ( iwork, nbefor, 'iwork' )

      end if

      if ( iadapt/=8 ) then

!     --- Standard method

         do i = 1, nbefor
            iwork1(iwork(i)) = i
         end do
         do i = 1, nuspnt
            if ( kmeshl(i)>0 ) then

!           --- kmeshl(i)>0, user point is connected to node

               inodp = iwork1(kmeshl(i))
               if ( inodp/=0) then

!              --- inodp>0, nodal point number

                  rinput(1,i) = coornw(1,inodp)
                  rinput(2,i) = coornw(2,inodp)

               end if
            end if
         end do

      else

!     --- iadapt = 8, special treatment

         do j = 1, jcount, 2
            i = iwork(j)
            inodp = iwork(j+1)
            rinput(1,i) = coornw(1,inodp)
            rinput(2,i) = coornw(2,inodp)
         end do

      end if

!     --- Fill composite curves is necessary

      if ( kelmq/=0 ) then

!     --- kelmq/=0, hence there are composite curves

         do icurnr = 1, ncurvs
            iwork(icurnr) = 0
         end do
         do icurnr = 1, ncurves
            iwork(abs(icurves(icurnr))) = 1
         end do
         call mshcpcom ( iwork, .true. )

      end if

      if ( debug ) then
         call  prinrl1(coornw, n, 2, 'coornw end msh047')
         call prinin ( icurvsmsh, ncurvs+1, 'icurvsmsh' )
         call prinrl1 ( curvesmsh, size(curvesmsh,2), ndim, 'curvesmsh' )
         call prinrl1 ( rinput, nuspnt, 2, 'rinput' )
      end if  ! ( debug )
      call erclos ( 'msh047' )

      end
