      subroutine msh050 ( iinput, iinp, ivolm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.7  date 16-07-2008 Extension with internal surfaces
!        version  2.6  date 17-02-2006 Extension with old_general
!        version  2.5  date 04-05-2005 Extension with ivolm=10
!        version  2.4  date 29-12-1999 Extension with channel
!        version  2.3  date 31-03-1994 Extension with translate, &
!                                      rotate and reflect
!        version  2.2  date 27-12-1990 Extension ivolm = 0
!        version  2.1  date 07-12-1990 Explicit declarations
!        version  2.0  date 31-03-1988 Extension with general
!        version  1.0  date 12-01-1987
!
!   copyright (c) 1987-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     raise iinp as function of ivolm
!     is used to find information in array iinput
!     !!!!!!!! If this subroutine is changed also change msh027 !!!!!!!!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, ivolm

!    iinp   i/o   At input last position of preciding volume
!                 At output last position of present volume in iinput
!    iinput  i    standard input array for subroutine mesh
!                 it is supposed that the first five positions have been
!                 deleted
!    ivolm   i    type of volume to be considered
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS   Deconcatenate name from calling sequence
!     EROPEN   Concatenate name to calling sequence
!     ERRINT   Place integer in integer error array
!     ERRSUB   Submit error message
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     298   Wrong code for volume generator
! **********************************************************************
!
!                       PSEUDO CODE
!
!     trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh050' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh050'

      end if
      if ( ierror/=0 ) go to 1000

      if ( ivolm<0 .or. ivolm>11 ) then
         call errint ( ivolm, 1 )
         call errsub ( 298, 1, 0, 0 )
      end if
      if ( ivolm==0 ) then
         iinp = iinp + 1
         go to 1000
      end if

      iinp = iinp + 3 + iinput( iinp + 2 )

      select case ( ivolm )

         case(1)

!        --- ivolm = 1, brick

            iinp = iinp + 3

         case(2:5,9,11)

!        --- ivolm = 2-5, 9, do nothing

         case(6:8)

!        --- ivolm = 6, 7 or 9

            iinp = iinp + 1

         case(10)

!        --- ivolm = 10, general with internal points

            iinp = iinp+3+iinput(iinp)+iinput(iinp+1)+iinput(iinp+2)

      end select  ! case ( ivolm )

1000  call erclos ( 'msh050' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh050'

      end if

      end
