      subroutine msh310 ( iinput, nx, ny, nz, kbndpt, iwork, nsurfs )
! ======================================================================
!
!        programmer     Niek Praagman
!        version 2.0    date   07-03-1997 Complete revision
!        version 1.1    date   22-08-1991 Zdenek: no dimension, cdc, ce
!        version 1.0    date   10-01-1985
!
!
!   copyright (c) 1985-1997  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   fill kbndpt with the nodal point numbers (local) of the 6 boundaries
!   in the case that inpelm=8
! **********************************************************************
!
!                       KEYWORDS
!
!    mesh
!    mesh_generation
!    surface
!    volume
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer nx, ny, nz, kbndpt(4,*), iwork(*), iinput(*), nsurfs

!     iinput     i    integer input array for the mesh generator
!                     only the part referred to the surface numbers
!                     corresponding to the brick is used
!     iwork           integer work array of the same length as kbndpt
!     kbndpt     o    array containing the nodal point numbers of the boundary
!                     kbndpt is organized with surface elements
!     nsurfs     i    Number of surfaces in mesh
!     nx         i    number of points in x direction
!     ny         i    number of points in y direction
!     nz         i    number of points in z direction
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iorien, n1, n2,  i, j, k, ij, ijw, nxy, locsur, ih

!     i          Counting variable
!     ih         Help variable to store a constant temporarily
!     ij         help parameter
!     ijw        help parameter
!     iorien     Orientation of axis
!     j          Counting variable
!     k          Counting variable
!     locsur     i    Local surface number with respect to brick
!     n1         Number of nodes in the first direction of the surface
!     n2         Number of nodes in the second direction of the surface
!     nxy        nx * ny
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The sequence corresponding to iorien is as follows:
!
!                                             i               j
!     .-------.       .-------.         .-------X          X-------.
!     |       |       |       |         |       |          |       |
!  j  |       |       |       | i       |       | j      i |       |
!     |       |       |       |         |       |          |       |
!     X-------.       .-------X         .-------.          .-------.
!         i               j
!     iorien=0         iorien=1         iorien=2           iorien=3
!     standard
!
!                                             j               i
!     .-------.       .-------.         .-------X          X-------.
!     |       |       |       |         |       |          |       |
!  i  |       |       |       | j       |       | i      j |       |
!     |       |       |       |         |       |          |       |
!     X-------.       .-------X         .-------.          .-------.
!         j               i
!     iorien=4         iorien=5         iorien=6           iorien=7
!
!
!     For locsur = 1 (1) 6 do
!        Set n1 and n2 depending on locsur
!        Fill array iwork with local nodal point numbers
!        if ( iorien=1 ) then
!           kbndpt_new(n1-j+1,i) = kbndpt_old(i,j)  (j=1(1)n1,i=1(1)n2)
!        else if ( iorien=2 ) then
!           kbndpt_new(n1-i+1,n2-j+1) = kbndpt_old(i,j)  (i=1(1)n1,j=1(1)n2)
!        else if ( iorien=3 ) then
!           kbndpt_new(j,n2-i+1) = kbndpt_old(i,j)  (j=1(1)n1,i=1(1)n2)
!        else if ( iorien=4 ) then
!           kbndpt_new(j,i) = kbndpt_old(i,j)  (j=1(1)n1,i=1(1)n2)
!        else if ( iorien=5 ) then
!           kbndpt_new(n1-i+1,j) = kbndpt_old(i,j)  (i=1(1)n1,j=1(1)n2)
!        else if ( iorien=6 ) then
!           kbndpt_new(n1-j+1,n2-i+1) = kbndpt_old(i,j)  (j=1(1)n1,i=1(1)n2)
!        else if ( iorien=7 ) then
!           kbndpt_new(i,n2-j+1) = kbndpt_old(i,j)  (i=1(1)n1,j=1(1)n2)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      ih = 0
      do locsur = 1, 6

!     --- Set n1 and n2 depending on locsur

         iorien = (iinput(locsur)-1)/nsurfs
         if ( locsur.eq.1 .or. locsur.eq.6 ) then
            n1 = nx
            n2 = ny
         else if ( locsur.eq.2 .or. locsur.eq.4 ) then
            n1 = nx
            n2 = nz
         else
            n1 = ny
            n2 = nz
         end if
         nxy = nx*ny

!        --- Fill array iwork with local nodal point numbers

         go to ( 110, 120, 130, 140, 150, 160 ), locsur

!        --- locsur.eq.1

110      do i = 1, nxy
            iwork(i) = i
         end do
         go to 200

!        --- locsur = 2

120      do i = 1, n1
            do k = 1, n2
               iwork(i+(k-1)*n1) = i+(k-1)*nxy
            end do
         end do
         go to 200

!        --- locsur = 3

130      do j = 1, n1
            do k = 1, n2
               iwork(j+(k-1)*n1) = j*nx+(k-1)*nxy
            end do
         end do
         go to 200

!        --- locsur = 4

140      do i = 1, n1
            do k = 1, n2
               iwork(i+(k-1)*n1) = i+nx*(ny-1)+(k-1)*nxy
            end do
         end do
         go to 200

!        --- locsur = 5

150      do j = 1, n1
            do k = 1, n2
               iwork(j+(k-1)*n1) = 1+(j-1)*nx+(k-1)*nxy
            end do
         end do
         go to 200

!        --- locsur = 6

160      do i = 1, nxy
            iwork(i) = i+nxy*(nz-1)
         end do
         go to 200

!        --- Fill kbndpt depending on the orientation

200      go to ( 210, 220, 230, 240, 250, 260, 270, 280 ), iorien+1

!        --- iorien = 1

210      do i = 1, n1-1
            do j = 1, n2-1
               ij = i+(j-1)*(n1-1)+ih
               ijw = i+(j-1)*n1
               kbndpt(1,ij) = iwork(ijw)
               kbndpt(2,ij) = iwork(ijw+1)
               kbndpt(3,ij) = iwork(ijw+1+n1)
               kbndpt(4,ij) = iwork(ijw+n1)
            end do
         end do
         go to 300

!        --- iorien = 2

220      do i = 1, n2-1
            do j = 1, n1-1
               ij = i+(j-1)*(n2-1)+ih
               ijw = n1*i-j+1
               kbndpt(1,ij) = iwork(ijw)
               kbndpt(2,ij) = iwork(ijw+n1)
               kbndpt(3,ij) = iwork(ijw-1+n1)
               kbndpt(4,ij) = iwork(ijw-1)
            end do
         end do
         go to 300

!        --- iorien = 3

230      do i = 1, n1-1
            do j = 1, n2-1
               ij = i+(j-1)*(n1-1)+ih
               ijw = n1*(n2-j+1)-i+1
               kbndpt(1,ij) = iwork(ijw)
               kbndpt(2,ij) = iwork(ijw-1)
               kbndpt(3,ij) = iwork(ijw-1-n1)
               kbndpt(4,ij) = iwork(ijw-n1)
            end do
         end do
         go to 300

!        --- iorien = 4

240      do i = 1, n2-1
            do j = 1, n1-1
               ij = i+(j-1)*(n2-1)+ih
               ijw = n1*(n2-i)+j
               kbndpt(1,ij) = iwork(ijw)
               kbndpt(2,ij) = iwork(ijw-n1)
               kbndpt(3,ij) = iwork(ijw+1-n1)
               kbndpt(4,ij) = iwork(ijw+1)
            end do
         end do
         go to 300

!        --- iorien = 5

250      do i = 1, n2-1
            do j = 1, n1-1
               ij = i+(j-1)*(n2-1)+ih
               ijw = j+(i-1)*n1
               kbndpt(1,ij) = iwork(ijw)
               kbndpt(2,ij) = iwork(ijw+n1)
               kbndpt(3,ij) = iwork(ijw+1+n1)
               kbndpt(4,ij) = iwork(ijw+1)
            end do
         end do
         go to 300

!        --- iorien = 6

260      do i = 1, n1-1
            do j = 1, n2-1
               ij = i+(j-1)*(n1-1)+ih
               ijw = n1*j-i+1
               kbndpt(1,ij) = iwork(ijw)
               kbndpt(2,ij) = iwork(ijw-1)
               kbndpt(3,ij) = iwork(ijw+n1-1)
               kbndpt(4,ij) = iwork(ijw+n1)
            end do
         end do
         go to 300

!        --- iorien = 7

270      do i = 1, n2-1
            do j = 1, n1-1
               ij = i+(j-1)*(n2-1)+ih
               ijw = n1*(n2-i+1)-j+1
               kbndpt(1,ij) = iwork(ijw)
               kbndpt(2,ij) = iwork(ijw-n1)
               kbndpt(3,ij) = iwork(ijw-n1-1)
               kbndpt(4,ij) = iwork(ijw-1)
            end do
         end do
         go to 300

!        --- iorien = 8

280      do i = 1, n1-1
            do j = 1, n2-1
               ij = i+(j-1)*(n1-1)+ih
               ijw = i+(n2-j)*n1
               kbndpt(1,ij) = iwork(ijw)
               kbndpt(2,ij) = iwork(ijw+1)
               kbndpt(3,ij) = iwork(ijw-n1+1)
               kbndpt(4,ij) = iwork(ijw-n1)
            end do
         end do
         go to 300

300      ih = ih+(n1-1)*(n2-1)
      end do

      end
