      subroutine msh315 ( jnew, bcord, nx, ny, nz, nelem, coor, &
                          kmeshc, kbndpt, inpelm, npoint, ivolnr, &
                          iinput, iwork, nsurfs )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  5.0  date 26-08-2002 Extra parameters iinput, iwork, nsurfs
!        version  4.0  date 06-03-1999 Extra parameter
!        version  3.1  date 19-02-1993 New layout
!        version  3.0  date 17-07-1990 Extension for inpelm=15
!        version  2.1  date 17-02-1989 Deletion of superfluous
!                                      parameters
!        version  2.0  date 02-01-1986
!
!   copyright (c) 1984-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    coordinates of all internal points of a block domain are
!    computed and filled in in the right positions if inpelm=14
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision bcord(*),coor(3,*)
      integer nx, ny, nz, nelem, inpelm, kmeshc(inpelm,*), kbndpt(*), &
              npoint, ivolnr, iinput(*), iwork(*), nsurfs
      logical jnew

!     bcord      i    array containing the coordinates of the boundaries
!     coor       o    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     iinput     i    integer input array for the mesh generator
!                     only the part referred to the surface numbers
!                     corresponding to the brick is used
!     inpelm     i    number of points per element
!     ivolnr     i    volume sequence number
!     iwork           integer work array of the same length as kbndpt
!     jnew       i    indication whether the mesh is new (true) or old
!     kbndpt     o    array containing the nodal point numbers of the boundary
!     kmeshc     o    kmesh part c, contains the nodal points corresponding to
!                     the elements
!     nelem      o    Number of elements
!     npoint    i/o   Number of nodal points
!     nsurfs     i    Number of surfaces in mesh
!     nx         i    number of points in x direction
!     ny         i    number of points in y direction
!     nz         i    number of points in z direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ihelp, ii, ikn, ikn1, ikn10, ikn11, ikn12, ikn13, &
              ikn14, ikn15, ikn16, ikn17, ikn18, ikn19, ikn2, ikn20, &
              ikn21, ikn22, ikn23, ikn24, ikn25, ikn26, ikn27, ikn28, &
              ikn29, ikn3, ikn30, ikn31, ikn32, ikn33, ikn34, ikn35, &
              ikn36, ikn37, ikn38, ikn39, ikn4, ikn40, ikn41, ikn42, &
              ikn43, ikn44, ikn45, ikn5, ikn6, ikn7, ikn8, ikn9, &
              j, jr, jrep, k, kcht, kvlak1, kvlak2, kvlak3, n1, n2, n3, &
              n4, n5, n6, n7, n8, nel, nstepx, nx1, ny1, nz1
      double precision af, difmax, dist, dx, dy, dz, x, x1, x10, x2, x3, &
                       x4, x5, x6, xn, y, y1, y10, y2, y3, y4, y5, y6, &
                       yn, z, z1, z10, z2, z3, z4, z5, z6, zn

!     af
!     difmax
!     dist
!     dx      length of vector in x-direction
!     dy      length of vector in y-direction
!     dz      length of vector in z-direction
!     i       Counting variable
!     ihelp
!     ii
!     ikn
!     ikn1
!     ikn10
!     ikn11
!     ikn12
!     ikn13
!     ikn14
!     ikn15
!     ikn16
!     ikn17
!     ikn18
!     ikn19
!     ikn2
!     ikn20
!     ikn21
!     ikn22
!     ikn23
!     ikn24
!     ikn25
!     ikn26
!     ikn27
!     ikn28
!     ikn29
!     ikn3
!     ikn30
!     ikn31
!     ikn32
!     ikn33
!     ikn34
!     ikn35
!     ikn36
!     ikn37
!     ikn38
!     ikn39
!     ikn4
!     ikn40
!     ikn41
!     ikn42
!     ikn43
!     ikn44
!     ikn45
!     ikn5
!     ikn6
!     ikn7
!     ikn8
!     ikn9
!     j       Counting variable
!     jr
!     jrep
!     k       Counting variable
!     kcht
!     kvlak1
!     kvlak2
!     kvlak3
!     n1
!     n2
!     n3
!     n4
!     n5
!     n6
!     n7
!     n8
!     nel
!     nstepx
!     nx1
!     ny1
!     nz1
!     x
!     x1      x-coordinate of first point in curve
!     x10
!     x2      x-coordinate of last point in curve
!     x3
!     x4
!     x5
!     x6
!     xn
!     y       y co-ordinate of point
!     y1      y-coordinate of first point in curve
!     y10
!     y2      y-coordinate of last point in curve
!     y3
!     y4
!     y5
!     y6
!     yn
!     z       z co-ordinate of point
!     z1      z-coordinate of first point in curve
!     z10
!     z2      z-coordinate of last point in curve
!     z3
!     z4
!     z5
!     z6
!     zn
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     MSH304    determine distance of nodal points n and m
!     MSH316    check whether the input by bcord of msh315 is correct
!     MSH317    fill kbndpt with the boundary elements in case of inpelm=14
!     MSH321    places coordinates of the six surfaces in coor
!               in case of elements with inpelm = 15
!     MSH322    Fill centroids for elements of type 16
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh315' )

!     --- check if bcord is consistent in itself

      call msh316 ( bcord, nx, ny, nz, ivolnr )
      if ( ierror.ne.0 ) go to 1000

!     --- place all coordinates in coor

      call msh321(nx,ny,nz,bcord,coor)

!     --- all surfaces placed in coor , &
!         compute some general variables

      kcht=5*nx*ny-2*nx-2*ny+1
      kvlak1=nx*ny+(nx-1)*(ny-1)/2
      kvlak2=(nx-1)*ny/2+(ny-1)*nx/2+(nx-1)*(ny-1)/4
      kvlak3=nx*ny

!     --- if mesh is new , &
!         compute internal coordinates

      if (jnew) then

!     --- compute x , y , z of internal points using
!         coon's formula .
!         start with all "regular" points

         nx1=nx-1
         ny1=ny-1
         nz1=nz-1
         do i=3,nx-2,2
            do j=3,ny-2,2
               do k=3,nz-2,2

!              --- nodal point number

                   ii=i+(j-1)*(3*nx-1)/2+(k-1)*kcht/2

!                  --- determine neighbours

                   n1=i+(k-1)*kcht/2
                   n2=n1+kvlak1-nx
                   n3=ii-i+1
                   n4=n3+nx-1
                   n5=ii-(k-1)*kcht/2
                   n6=ii+(nz-k)*kcht/2

!                  --- x - coordinate

                   x1=coor(1,n1)
                   x2=coor(1,n2)
                   x3=coor(1,n3)
                   x4=coor(1,n4)
                   x5=coor(1,n5)
                   x6=coor(1,n6)
                   x=(x1+(j-1)*(x2-x1)/ny1+ &
                      x3+(i-1)*(x4-x3)/nx1+ &
                      x5+(k-1)*(x6-x5)/nz1)/3d0
                   coor(1,ii)=x

!                  --- y - coordinate

                   y1=coor(2,n1)
                   y2=coor(2,n2)
                   y3=coor(2,n3)
                   y4=coor(2,n4)
                   y5=coor(2,n5)
                   y6=coor(2,n6)
                   y=(y1+(j-1)*(y2-y1)/ny1+ &
                      y3+(i-1)*(y4-y3)/nx1+ &
                      y5+(k-1)*(y6-y5)/nz1)/3d0
                   coor(2,ii)=y

!                  --- z - coordinate

                   z1=coor(3,n1)
                   z2=coor(3,n2)
                   z3=coor(3,n3)
                   z4=coor(3,n4)
                   z5=coor(3,n5)
                   z6=coor(3,n6)
                   z=(z1+(j-1)*(z2-z1)/ny1+ &
                      z3+(i-1)*(z4-z3)/nx1+ &
                      z5+(k-1)*(z6-z5)/nz1)/3d0
                   coor(3,ii)=z
               end do
            end do
         end do
      end if

!     --- repositioning

      jrep=20

!     --- determine a stop-value

      n1=1
      n2=nx
      n3=kvlak1
      n4=kvlak1-nx+1
      ihelp=(nz-1)*kcht/2
      n5=n1+ihelp
      n6=n2+ihelp
      n7=n3+ihelp
      n8=n4+ihelp
      call msh304(n1,n2,coor,dist)
      af=dist/(nx-1)
      call msh304(n7,n8,coor,dist)
      af=af+dist/(nx-1)
      call msh304(n1,n4,coor,dist)
      af=af+dist/(ny-1)
      call msh304(n6,n7,coor,dist)
      af=af+dist/(ny-1)
      call msh304(n1,n5,coor,dist)
      af=af+dist/(nz-1)
      call msh304(n3,n7,coor,dist)
      af=af+dist/(nz-1)
      dist=af/6d0

!     --- dist is a mean value for the distance between two nodal points ;
!         use as a stop-value for repositioning dist/100

      dist=dist*0.01d0

!     --- repositioning

      do jr=1,jrep
          difmax=0d0
          nstepx=3*nx-1
          do i=3,nx-2,2
             do j=3,ny-2,2
                do k=3,nz-2,2
                    ikn=i+(j-1)*nstepx/2+(k-1)*kcht/2

!                   --- x - coordinate

                    xn=(coor(1,ikn+2)+coor(1,ikn-2)+ &
                        coor(1,ikn+nstepx)+coor(1,ikn-nstepx)+ &
                        coor(1,ikn+kcht)+coor(1,ikn-kcht))/6d0

!                   --- y - coordinate

                    yn=(coor(2,ikn+2)+coor(2,ikn-2)+ &
                        coor(2,ikn+nstepx)+coor(2,ikn-nstepx)+ &
                        coor(2,ikn+kcht)+coor(2,ikn-kcht))/6d0

!                   --- z - coordinate

                    zn=(coor(3,ikn+2)+coor(3,ikn-2)+ &
                        coor(3,ikn+nstepx)+coor(3,ikn-nstepx)+ &
                        coor(3,ikn+kcht)+coor(3,ikn-kcht))/6d0

!                   --- determine distance old - new

                    dx=coor(1,ikn)-xn
                    dy=coor(2,ikn)-yn
                    dz=coor(3,ikn)-zn
                    af=sqrt(dx*dx+dy*dy+dz*dz)
                    if (af.gt.difmax) difmax=af

!                   --- replace coordinates

                    coor(1,ikn)=xn
                    coor(2,ikn)=yn
                    coor(3,ikn)=zn
                end do
             end do
          end do
          if (jr.gt.2.and.difmax.lt.dist) goto 250
      end do

!     --- coor has been filled , repositioning performed .
!          if new mesh fill kmeshc

250   if (jnew) then
         nelem=0
         do k=1,nz-2,2
            do j=1,ny-2,2
               do i=1,nx-2,2
                  ii=i+(j-1)*nstepx/2+(k-1)*kcht/2

!                 --- boundary plane

                  ikn1=ii
                  ikn2=ikn1+1
                  ikn3=ikn2+1
                  ikn4=ikn1+nx-(i-1)/2
                  ikn5=ikn1+nstepx/2
                  ikn6=ikn5+1
                  ikn7=ikn6+1
                  ikn8=ikn5+nx-(i-1)/2
                  ikn9=ikn1+nstepx
                  ikn10=ikn9+1
                  ikn11=ikn10+1

!                 --- first sub-surface

                  ikn12=ikn1+kvlak1-(i-1)/2-(j-1)*(nx+1)/4
                  ikn13=ikn12+(nx-1)/2
                  ikn14=ikn13+(nx-1)/2+(i-1)
                  ikn15=ikn14+1
                  ikn16=ikn15+1
                  ikn17=ikn16+1
                  ikn18=ikn12+5*((nx-1)/2)+1

!                 --- middle-surface

                  ikn19=ikn1+kvlak1+kvlak2-(j-1)/2*(nx-1)
                  ikn20=ikn19+1
                  ikn21=ikn20+1
                  ikn22=ikn19+nx
                  ikn23=ikn22+1
                  ikn24=ikn23+1
                  ikn25=ikn19+2*nx
                  ikn26=ikn25+1
                  ikn27=ikn26+1

!                 --- second sub-surface

                  ikn28=ii+kvlak1+kvlak2+kvlak3-(i-1)/2-(j-1)*(nx+1)/4
                  ikn29=ikn28+(nx-1)/2+(i-1)
                  ikn30=ikn29+1
                  ikn31=ikn30+1
                  ikn32=ikn31+1
                  ikn33=ikn28+(nx-1)*2+1
                  ikn34=ikn33+(nx-1)/2

!                 --- upper-surface

                  ikn35=ikn1+kcht
                  ikn36=ikn2+kcht
                  ikn37=ikn3+kcht
                  ikn38=ikn4+kcht
                  ikn39=ikn5+kcht
                  ikn40=ikn6+kcht
                  ikn41=ikn7+kcht
                  ikn42=ikn8+kcht
                  ikn43=ikn9+kcht
                  ikn44=ikn10+kcht
                  ikn45=ikn11+kcht

!                 --- element 1

                  nelem = nelem + 1
                  kmeshc(1,nelem)=ikn1
                  kmeshc(2,nelem)=ikn2
                  kmeshc(3,nelem)=ikn3
                  kmeshc(4,nelem)=ikn7
                  kmeshc(5,nelem)=ikn11
                  kmeshc(6,nelem)=ikn6
                  kmeshc(7,nelem)=ikn23
                  kmeshc(8,nelem)=ikn24
                  kmeshc(9,nelem)=ikn27
                  kmeshc(10,nelem)=ikn45
                  kmeshc(11,nelem)=ikn4
                  kmeshc(12,nelem)=ikn13
                  kmeshc(13,nelem)=ikn17
                  kmeshc(14,nelem)=ikn16

!                 --- element 2

                  nelem = nelem + 1
                  kmeshc(1,nelem)=ikn1
                  kmeshc(2,nelem)=ikn20
                  kmeshc(3,nelem)=ikn37
                  kmeshc(4,nelem)=ikn21
                  kmeshc(5,nelem)=ikn3
                  kmeshc(6,nelem)=ikn2
                  kmeshc(7,nelem)=ikn23
                  kmeshc(8,nelem)=ikn41
                  kmeshc(9,nelem)=ikn24
                  kmeshc(10,nelem)=ikn45
                  kmeshc(11,nelem)=ikn12
                  kmeshc(12,nelem)=ikn31
                  kmeshc(13,nelem)=ikn32
                  kmeshc(14,nelem)=ikn13

!                 --- element 3

                  nelem = nelem + 1
                  kmeshc(1,nelem)=ikn1
                  kmeshc(2,nelem)=ikn19
                  kmeshc(3,nelem)=ikn35
                  kmeshc(4,nelem)=ikn36
                  kmeshc(5,nelem)=ikn37
                  kmeshc(6,nelem)=ikn20
                  kmeshc(7,nelem)=ikn23
                  kmeshc(8,nelem)=ikn40
                  kmeshc(9,nelem)=ikn41
                  kmeshc(10,nelem)=ikn45
                  kmeshc(11,nelem)=ikn28
                  kmeshc(12,nelem)=ikn30
                  kmeshc(13,nelem)=ikn38
                  kmeshc(14,nelem)=ikn31

!                 --- element 4

                  nelem = nelem + 1
                  kmeshc(1,nelem)=ikn1
                  kmeshc(2,nelem)=ikn6
                  kmeshc(3,nelem)=ikn11
                  kmeshc(4,nelem)=ikn10
                  kmeshc(5,nelem)=ikn9
                  kmeshc(6,nelem)=ikn5
                  kmeshc(7,nelem)=ikn23
                  kmeshc(8,nelem)=ikn27
                  kmeshc(9,nelem)=ikn26
                  kmeshc(10,nelem)=ikn45
                  kmeshc(11,nelem)=ikn8
                  kmeshc(12,nelem)=ikn16
                  kmeshc(13,nelem)=ikn18
                  kmeshc(14,nelem)=ikn15

!                 --- element 5

                  nelem = nelem + 1
                  kmeshc(1,nelem)=ikn1
                  kmeshc(2,nelem)=ikn5
                  kmeshc(3,nelem)=ikn9
                  kmeshc(4,nelem)=ikn25
                  kmeshc(5,nelem)=ikn43
                  kmeshc(6,nelem)=ikn22
                  kmeshc(7,nelem)=ikn23
                  kmeshc(8,nelem)=ikn26
                  kmeshc(9,nelem)=ikn44
                  kmeshc(10,nelem)=ikn45
                  kmeshc(11,nelem)=ikn14
                  kmeshc(12,nelem)=ikn15
                  kmeshc(13,nelem)=ikn34
                  kmeshc(14,nelem)=ikn33

!                 --- element 6

                  nelem = nelem + 1
                  kmeshc(1,nelem)=ikn1
                  kmeshc(2,nelem)=ikn22
                  kmeshc(3,nelem)=ikn43
                  kmeshc(4,nelem)=ikn39
                  kmeshc(5,nelem)=ikn35
                  kmeshc(6,nelem)=ikn19
                  kmeshc(7,nelem)=ikn23
                  kmeshc(8,nelem)=ikn44
                  kmeshc(9,nelem)=ikn40
                  kmeshc(10,nelem)=ikn45
                  kmeshc(11,nelem)=ikn29
                  kmeshc(12,nelem)=ikn33
                  kmeshc(13,nelem)=ikn42
                  kmeshc(14,nelem)=ikn30

!                 --- 6 elements placed in kmeshc

               end do
            end do
         end do
      end if
      do nel=1,nelem
         ikn1=kmeshc(1,nel)
         ikn2=kmeshc(2,nel)
         ikn3=kmeshc(3,nel)
         ikn4=kmeshc(4,nel)
         ikn5=kmeshc(5,nel)
         ikn6=kmeshc(6,nel)
         ikn7=kmeshc(7,nel)
         ikn8=kmeshc(8,nel)
         ikn9=kmeshc(9,nel)
         ikn10=kmeshc(10,nel)
         ikn11=kmeshc(11,nel)
         ikn12=kmeshc(12,nel)
         ikn13=kmeshc(13,nel)
         ikn14=kmeshc(14,nel)

!        --- coordinates of corner-points

         x1=coor(1,ikn1)
         y1=coor(2,ikn1)
         z1=coor(3,ikn1)
         x3=coor(1,ikn3)
         y3=coor(2,ikn3)
         z3=coor(3,ikn3)
         x5=coor(1,ikn5)
         y5=coor(2,ikn5)
         z5=coor(3,ikn5)
         x10=coor(1,ikn10)
         y10=coor(2,ikn10)
         z10=coor(3,ikn10)

!        --- place other coordinates (middle points and centroids)
!               point 2

         coor(1,ikn2)=(x1+x3)*0.5d0
         coor(2,ikn2)=(y1+y3)*0.5d0
         coor(3,ikn2)=(z1+z3)*0.5d0

!        --- point 4

         coor(1,ikn4)=(x3+x5)*0.5d0
         coor(2,ikn4)=(y3+y5)*0.5d0
         coor(3,ikn4)=(z3+z5)*0.5d0

!        --- point 6

         coor(1,ikn6)=(x1+x5)*0.5d0
         coor(2,ikn6)=(y1+y5)*0.5d0
         coor(3,ikn6)=(z1+z5)*0.5d0

!        --- point 7

         coor(1,ikn7)=(x1+x10)*0.5d0
         coor(2,ikn7)=(y1+y10)*0.5d0
         coor(3,ikn7)=(z1+z10)*0.5d0

!        --- point 8

         coor(1,ikn8)=(x3+x10)*0.5d0
         coor(2,ikn8)=(y3+y10)*0.5d0
         coor(3,ikn8)=(z3+z10)*0.5d0

!        --- point 9

         coor(1,ikn9)=(x5+x10)*0.5d0
         coor(2,ikn9)=(y5+y10)*0.5d0
         coor(3,ikn9)=(z5+z10)*0.5d0

!        --- point 11

         coor(1,ikn11)=(x1+x3+x5)/3d0
         coor(2,ikn11)=(y1+y3+y5)/3d0
         coor(3,ikn11)=(z1+z3+z5)/3d0

!        --- point 12

         coor(1,ikn12)=(x1+x3+x10)/3d0
         coor(2,ikn12)=(y1+y3+y10)/3d0
         coor(3,ikn12)=(z1+z3+z10)/3d0

!        --- point 13

         coor(1,ikn13)=(x3+x5+x10)/3d0
         coor(2,ikn13)=(y3+y5+y10)/3d0
         coor(3,ikn13)=(z3+z5+z10)/3d0

!        --- point 14

         coor(1,ikn14)=(x1+x5+x10)/3d0
         coor(2,ikn14)=(y1+y5+y10)/3d0
         coor(3,ikn14)=(z1+z5+z10)/3d0

!        --- all coordinates placed for this element

      end do

!     --- fill array kbndpt

      call msh317 ( kbndpt, nx, ny, nz, iinput, iwork, nsurfs )

!     --- replace boundary coordinates

      call msh321(nx,ny,nz,bcord,coor)

!     --- For elements with 15 points, the centroid must be filled:

      if ( inpelm.eq.15 ) call msh322 ( kmeshc, coor, nelem, npoint )
1000  call erclos ( 'msh315' )
      end
