      subroutine msh319( i1, i2, i3, i4, coor )

! =======================================================================
!
!
!        programmer     niek praagman
!
!        version 2.0    date   28-09-1993 New norms
!        version 1.2    date   23-06-1992 New common cmessg
!        version 1.1    date   22-08-1991 Zdenek: no dimension, cdc
!        version 1.0    date   10-11-1985
!
!
!     copyright (c) 1985,1993   " ingenieursbureau  SEPRA "
!     permission to copy or distribute this software or documentation
!     in hard copy or soft copy granted only by written license
!     obtained from  " ingenieursbureau  SEPRA ".
!     all rights reserved . no part of this publication may be repro- &
!     duced, stored in a retrieval system (e.g.,in memory,disk or
!     core) or be transmitted by any means ,electronic, mechanical
!     photocopy,recording, or otherwise , without written permission
!     from the publisher.
!
! ***********************************************************************
!
!     Compute volume of tetrahedron : if volume negative give message
!
! ***********************************************************************
!
!     KEYWORDS
!
!     3d
!     check
!     mesh_generation
!
! ***********************************************************************
!
!     INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer i1, i2, i3, i4
      double precision coor(3,*)

!     coor     i  coordinate array
!
!     i1       i  first node of tetrahedron
!
!     i2       i  second node
!
!     i3       i  third node
!
!     i4       i  fourth node of tetrahedron to be considered
!
! ***********************************************************************
!
!     COMMON BLOCKS
!
! ***********************************************************************
!
!     LOCAL PARAMETERS
!
      double precision a1, a2, a3, b1, b2, b3, c1, c2, c3, det, &
                       x1, x2, x3, x4, y1, y2, y3, y4, z1, z2, z3, z4

!     a1
!     a2
!     a3
!     b1
!     b2
!     b3
!     c1
!     c2
!     c3
!     det
!     x1
!     x2
!     x3
!     x4
!     y1
!     y2
!     y3
!     y4
!     z1
!     z2
!     z3
!     z4
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
!     614   Volume of tetrahedron is negative
!
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
!     ERCLOS   Deconcatenate name from callin string
!     EROPEN   Concatenate name to calling string
!     ERREAL   Place real value in error array
!     ERRINT   Place integer value in error array
!     ERRSUB   Submit error message
!
! ***********************************************************************
!
!     PSEUDO CODE
!
!     trivial
!
! =======================================================================

      call eropen( 'msh319' )

      x1 = coor( 1, i1 )
      x2 = coor( 1, i2 )
      x3 = coor( 1, i3 )
      x4 = coor( 1, i4 )

      y1 = coor( 2, i1 )
      y2 = coor( 2, i2 )
      y3 = coor( 2, i3 )
      y4 = coor( 2, i4 )

      z1 = coor( 3, i1 )
      z2 = coor( 3, i2 )
      z3 = coor( 3, i3 )
      z4 = coor( 3, i4 )

      a1 = x2 - x1
      a2 = y2 - y1
      a3 = z2 - z1

      b1 = x3 - x1
      b2 = y3 - y1
      b3 = z3 - z1

      c1 = x4 - x1
      c2 = y4 - y1
      c3 = z4 - z1

!     compute the " volume "

      det = a1 * b2 * c3 - a1 * b3 * c2 - a2 * b1 * c3 + &
            a2 * b3 * c1 + a3 * b1 * c2 - a3 * b2 * c1

      if ( det.le.0 ) then

           call erreal ( x1, 1 )
           call erreal ( y1, 2 )
           call erreal ( z1, 3 )
           call erreal ( x2, 4 )
           call erreal ( y2, 5 )
           call erreal ( z2, 6 )
           call erreal ( x3, 7 )
           call erreal ( y3, 8 )
           call erreal ( z3, 9 )
           call erreal ( x4, 10 )
           call erreal ( y4, 11 )
           call erreal ( z4, 12 )
           call erreal ( det, 13 )
           call errint ( i1, 1 )
           call errint ( i2, 2 )
           call errint ( i3, 3 )
           call errint ( i4, 4 )
           call errsub ( 614, 4, 13, 0 )

      end if

      call erclos( 'msh319' )

      end
