      subroutine msh320( bcord, coor, nx, ny, nz )
! ======================================================================
!
!        programmer    niek praagman
!        version  2.1  date 27-11-2012 Long integers
!        version  2.0  date 28-09-1993 New norms
!        version  1.1  date 22-08-1991 Zdenek: no dimension, cdc
!
!   copyright (c) 1985-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Place coordinates of all boundary nodes in the main coordinate
!     array
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     mesh_generation
!     hexahedron
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, nz
      double precision bcord(*), coor(*)

!     bcord    i    array containing the coordinates of all
!                   boundary points sequentially
!     coor     o    array where all coordinates are placed in
!     nx       i    number of nodal points in x-direction
!     ny       i    number of nodal points in y-direction
!     nz       i    number of nodal points in z-direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, kb, kc

!     i      loop variable
!     j      loop variable
!     kb     local node number in bcord
!     kc     global node number in coor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     MSH301    Copy point of boundary in coordinate array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- place all coordinates in coor
!         First surface 1  ( 1-2-3-4 )

      kb = 0
      do j = 1, ny
         do i = 1, nx
             kb = kb + 1
             kc =  i + (j-1) * nx
             call msh301( kb, kc, bcord, coor )
         end do
      end do

!     --- surface 2  ( 1-2-6-5 )

      do j = 1, nz
         do i = 1, nx
             kb = kb + 1
             kc = i  + (j-1) * nx * ny
             call msh301( kb, kc, bcord, coor )
         end do
      end do

!     --- surface 3  ( 2-3-7-6 )

      do j = 1, nz
         do i = 1, ny
             kb = kb + 1
             kc = i * nx + (j-1) * nx * ny
             call msh301( kb, kc, bcord, coor )
         end do
      end do

!     --- surface 4  ( 4-3-7-8 )

      do j = 1, nz
         do i = 1, nx
             kb = kb + 1
             kc = (ny-1) * nx + i + (j-1) * nx * ny
             call msh301( kb, kc, bcord, coor )
         end do
      end do

!     --- surface 5  ( 1-4-8-5 )

      do j = 1, nz
         do i = 1, ny
             kb = kb + 1
             kc = 1  + (i-1) * nx + (j-1) * nx * ny
             call msh301( kb, kc, bcord, coor )
         end do
      end do

!     --- surface 6 :  ( 5-6-7-8 )

      do j = 1, ny
         do i = 1, nx
             kb = kb + 1
             kc = (nz-1) * nx * ny + i + (j-1) * nx
             call msh301( kb, kc, bcord, coor )
         end do
      end do

!     --- all points of the six surfaces placed in coor

      end
