      subroutine msh321(nx,ny,nz,bcord,coor)
! ======================================================================
!
!        programmer    niek praagman
!        version  1.2  date 27-11-2012 Long integers
!        version  1.1  date 22-08-1991 Zdenek: no dimension, cdc
!        version  1.0  date 27-12-1985
!
!   copyright (c) 1985-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    subroutine places coordinates of the six surfaces in coor
!    in case of elements with inpelm = 15 .
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx,ny,nz
      double precision bcord(*), coor(*)

!     bcord          i    array containing the coordinates of six surfaces
!     coor           o    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!                         in this subroutine only
!                         the boundary points are substituted
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     nz             i    number of points in z direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, kb, kc, kcht, kvlak1, kvlak2, kvlak3, nhx, nhy, kch

!     i              Counting variable
!     j              Counting variable
!     kb             ?
!     kc             ?
!     kch            ?
!     kcht           ?
!     kvlak1         ?
!     kvlak2         ?
!     kvlak3         ?
!     nhx            ?
!     nhy            ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     MSH301         Place all points as given in BCORD in array COOR
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!


!     --- place all coordinates in coor
!             ( compute some helpvariables )

      kcht=5*nx*ny-2*nx-2*ny+1
      kvlak1=nx*ny+(nx-1)*(ny-1)/2
      kvlak2=(nx-1)*ny/2+(ny-1)*nx/2+(nx-1)*(ny-1)/4
      kvlak3=nx*ny
      nhx=(nx-1)/2
      nhy=(ny-1)/2


!     --- surface 1  ( 1-2-3-4 )

      kb=0
      do 12 j=1,ny
         do 10 i=1,nx
             kb=kb+1
             kc=i+(j-1)*(nx+nhx)
             call msh301(kb,kc,bcord,coor)
10       continue
         if (j/=ny) then
            do 11 i=1,nhx
                kb=kb+1
                kc=j*nx+(j-1)*nhx+i
                call msh301(kb,kc,bcord,coor)
11          continue
         end if
12    continue


!     --- surface 2  ( 1-2-6-5 )

      do 24 j=1,nz,2
         kch=(j-1)/2*kcht
         do 20 i=1,nx
             kb=kb+1
             kc=kch+i
             call msh301(kb,kc,bcord,coor)
20       continue
         if (j/=nz) then
             kch=kch+kvlak1
             do 21 i=1,nhx
                 kb=kb+1
                 kc=kch+i
                 call msh301(kb,kc,bcord,coor)
21           continue
             kch=kch+kvlak2
             do 22 i=1,nx
                 kb=kb+1
                 kc=kch+i
                 call msh301(kb,kc,bcord,coor)
22           continue
             kch=kch+kvlak3
             do 23 i=1,nhx
                 kb=kb+1
                 kc=kch+i
                 call msh301(kb,kc,bcord,coor)
23           continue
          end if
24    continue


!     --- surface 3  ( 2-3-7-6 )

      do 34 j=1,nz,2
          kch=(j-1)*kcht/2
          do 30 i=1,ny
              kb=kb+1
              kc=kch+i*nx+(i-1)*nhx
              call msh301(kb,kc,bcord,coor)
30        continue
          if (j/=nz) then
              kch=kch+kvlak1
              do 31 i=1,nhy
                  kb=kb+1
                  kc=kch+5*nhx*i+i
                  call msh301(kb,kc,bcord,coor)
31            continue
              kch=kch+kvlak2
              do 32 i=1,ny
                  kb=kb+1
                  kc=kch+i*nx
                  call msh301(kb,kc,bcord,coor)
32            continue
              kch=kch+kvlak3
              do 33 i=1,nhy
                  kb=kb+1
                  kc=kch+nhx*(5*i-1)+i
                  call msh301(kb,kc,bcord,coor)
33            continue
          end if
34    continue


!     --- surface 4  ( 4-3-7-8 )

      do 44 j=1,nz,2
          kch=(j-1)/2*kcht
          do 40 i=1,nx
              kb=kb+1
              kc=kch+kvlak1-nx+i
              call msh301(kb,kc,bcord,coor)
40        continue
          if (j/=nz) then
              kch=kch+kvlak1
              do 41 i=1,nhx
                  kb=kb+1
                  kc=kch-nhx+kvlak2+i
                  call msh301(kb,kc,bcord,coor)
41            continue
              kch=kch+kvlak2
              do 42 i=1,nx
                  kb=kb+1
                  kc=kch+kvlak3-nx+i
                  call msh301(kb,kc,bcord,coor)
42            continue
              kch=kch+kvlak3
              do 43 i=1,nhx
                  kb=kb+1
                  kc=kch+kvlak2-nhx+i
                  call msh301(kb,kc,bcord,coor)
43            continue
          end if
44    continue


!     --- surface 5  ( 1-4-8-5 )

      do 54 j=1,nz,2
          kch=(j-1)/2*kcht
          do 50 i=1,ny
              kb=kb+1
              kc=kch+(i-1)*(3*nhx+1)+1
              call msh301(kb,kc,bcord,coor)
50        continue
          if (j/=nz) then
              kch=kch+kvlak1
              do 51 i=1,nhy
                  kb=kb+1
                  kc=kch+(5*i-3)*nhx+i
                  call msh301(kb,kc,bcord,coor)
51            continue
              kch=kch+kvlak2
              do 52 i=1,ny
                  kb=kb+1
                  kc=kch+(i-1)*nx+1
                  call msh301(kb,kc,bcord,coor)
52            continue
              kch=kch+kvlak3
              do 53 i=1,nhy
                  kb=kb+1
                  kc=kch+nhx*(5*i-4)+i
                  call msh301(kb,kc,bcord,coor)
53            continue
          end if
54    continue


!     --- surface 6 :  ( 5-6-7-8 )

      kch=(nz-1)/2*kcht


      do 62 j=1,ny
          do 60 i=1,nx
              kb=kb+1
              kc=kch+(j-1)*(nx+nhx)+i
              call msh301(kb,kc,bcord,coor)
60        continue
          if (j/=ny) then
              do 61 i=1,nhx
                  kb=kb+1
                  kc=kc+1
                  call msh301(kb,kc,bcord,coor)
61            continue
          end if
62    continue


      end
