      subroutine msh453 ( kelem , nelem , kelemh, ibrpnt, istart, &
                          npnold, coorh )
! =======================================================================
!
!
!        programmer     Niek Praagman
!        version 3.1    date   27-04-1995 New basis for diagonal
!        version 3.0    date   23-02-1995 Determine centre using msh416
!        version 2.0    date   03-02-1995 Use neighbours for middle
!                                        points )
!        version 1.0    date   27-03-1992
!
!
!
!   copyright (c) 1992 - 1995  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     Place new point in barycentre of quadrilateral, store coordinates
!     in array coorh and create new quadratic element in array kelemh
!
! ***********************************************************************
!
!     KEYWORDS
!
!     2d
!     create_point
!     mesh
!
! ***********************************************************************
!
!     INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer kelem(*), ibrpnt(*), istart(*), nelem, &
              kelemh(*), npnold
      double precision coorh(*)

!     coorh  i,o  coordinate array
!     ibrpnt  i   neighbourarray
!     istart  i   starting positions array for ibrpnt
!     kelem   i   element array
!     kelemh  o   new element array
!     nelem   i   number of elements
!     npnold  i   number of points in old mesh
!
! ***********************************************************************
!
!     COMMON BLOCKS
!
! ***********************************************************************
!
!     LOCAL PARAMETERS
!
      integer i, i1, i2, i3, i4, i5, i6, i7, i8, i9, ie, it
!
!     i        loop variable
!     i1       node
!     i2       node
!     i3       node
!     i4       node
!     i5       node
!     i6       node
!     i7       node
!     i8       node
!     i9       node number
!     ie       extra node number
!     it       extra node number
!
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
!     MSH403   Determine node number of point in middle of line
!     MSH407   Place element in element array
!     MSH416   Determine diagonal nodes of preference
!
! ***********************************************************************
!
!     I / O
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
! ***********************************************************************
!
!     PSEUDO CODE
!
!     Run through all elements of MESHTP = 5
!        Check node numbers in middle of sides
!        Determine node number in barycenter
!        Create new element of MESHTP = 6
!
! =======================================================================

!     Check each element; place nodenumbers of new quadratic element in
!     array kelemh

      do 100 i = 1 , nelem

          i1 = kelem(4*i-3)
          i2 = kelem(4*i-2)
          i3 = kelem(4*i-1)
          i4 = kelem(4*i  )

!         Compute middle node numbers

          call msh403( npnold, i1, i2, ibrpnt, istart, i5 )
          call msh403( npnold, i2, i3, ibrpnt, istart, i6 )
          call msh403( npnold, i3, i4, ibrpnt, istart, i7 )
          call msh403( npnold, i4, i1, ibrpnt, istart, i8 )

!         Find node number of barycenter

          call msh416( i1, i2, i3, i4, ie, it )

!         Diagonal

          it = npnold * it

          call msh403( npnold, ie, it, ibrpnt, istart, i9 )

!         Fill coorh for barycentre point i9

          coorh(2*i9-1)=   ( coorh(2*i1-1) + coorh(2*i2-1) + &
                             coorh(2*i3-1) + coorh(2*i4-1) ) / 4d0
          coorh(2*i9  )=   ( coorh(2*i1  ) + coorh(2*i2  ) + &
                             coorh(2*i3  ) + coorh(2*i4  ) ) / 4d0

!         Create new quadratic element

          call msh407( kelemh, i, i1, i5, i2, i6, i3, i7, i4, i8, i9 )

100   continue

      end
