      subroutine msh454 ( kelem, nelem, kelemh, ibrpnt, istart, npoint )
! =======================================================================
!
!
!        programmer     Niek Praagman
!        version 2.0    date   15-02-1995 Use neighbours arrays
!        version 1.0    date   27-03-1992
!
!
!
!   copyright (c) 1992-1995  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     Create quadratic tetrahedrons from linear tetrahedrons
!     New points (on all edges) have already been stored in array ibrpnt
!
! ***********************************************************************
!
!     KEYWORDS
!
!     3d
!     create_quadratic_element
!     mesh
!
! ***********************************************************************
!
!     INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer kelem(*), ibrpnt(*), istart(*), nelem, kelemh(*), npoint

!     ibrpnt  i   neighbour array
!     istart  i   starting positions in ibrpnt
!     kelem   i   old linear element array
!     kelemh  o   new quadratic element array
!     nelem   i   number of elements
!     npoint  i   number of original nodes
!
! ***********************************************************************
!
!     COMMON BLOCKS
!
! ***********************************************************************
!
!     LOCAL PARAMETERS
!
      integer i, i1, i2, i3, i4, i5, i6, i7, i8, i9, i10
!
!     i        loop variable
!     i1       + &
!     i10      + &
!     i2       + &
!     i3       + &
!     i4       + &
!     i5       + &
!     i6       + &
!     i7       + &
!     i8       + &
!     i9       node numbers
!
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
!     ERCLOS   Deconcatenate subroutine name from calling string
!     EROPEN   Concatenate subroutine name to calling string
!     MSH403   Determine node number of middle point
!     MSH417   Place quadratic tetrahedron in element array
!
! ***********************************************************************
!
!     I / O
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
! ***********************************************************************
!
!     PSEUDO CODE
!
!     Run through all elements of MESHTP = 11
!        Check node numbers in middle of sides
!        Create new elements of MESHTP = 12
!
! =======================================================================

      call eropen( 'msh454' )

!     Check each element; place nodenumbers of new element in array
!     kelemh

      do 100 i = 1 , nelem

          i1 = kelem(4*i-3)
          i2 = kelem(4*i-2)
          i3 = kelem(4*i-1)
          i4 = kelem(4*i  )

!         Compute middle node numbers

          call msh403( npoint, i1, i2, ibrpnt, istart, i5 )
          call msh403( npoint, i2, i3, ibrpnt, istart, i6 )
          call msh403( npoint, i3, i1, ibrpnt, istart, i7 )
          call msh403( npoint, i1, i4, ibrpnt, istart, i8 )
          call msh403( npoint, i2, i4, ibrpnt, istart, i9 )
          call msh403( npoint, i3, i4, ibrpnt, istart, i10 )

!         Fill new element

          call msh417( kelemh, i, i1, i5, i2, i6, i3, i7, i8, &
                       i9, i10, i4 )

100   continue

      call erclos( 'msh454' )

      end
