      subroutine msh457 ( kelem , nelem , kelemh, coorh , ibrpnt, &
                          istart, npoint )
! =======================================================================
!
!
!        programmer     Niek Praagman
!        version 2.1    date   27-04-1995 New basis for diagonal point
!        version 2.0    date   14-02-1995 Use neighbour arrays
!        version 1.0    date   27-03-1992
!
!
!
!   copyright (c) 1992 - 1995  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     Create new quadratic elements (hexahedrons) and store in array
!     kelemh
!
! ***********************************************************************
!
!     KEYWORDS
!
!     3d
!     create_point
!     mesh
!
! ***********************************************************************
!
!     INPUT / OUTPUT  PARAMETERS
!
      implicit none
      integer kelem(*), ibrpnt(*), istart(*), nelem, kelemh(*), npoint
      double precision coorh(3,*)

!     coorh  i,o   new coordinate array
!     ibrpnt  i    neighbours array
!     istart  i    starting positions in ibrpnt
!     kelem   i    element array
!     kelemh  o    new element array
!     nelem   i    number of elements
!     npoint  i    number of nodes in original mesh
!
! ***********************************************************************
!
!     COMMON BLOCKS
!
! ***********************************************************************
!
!     LOCAL PARAMETERS
!
      integer i, idim, ip(27), ia, ib, nelhel
!
!     i         loop variable
!     ia        selected node
!     ib        selected node
!     idim      loop variable
!     ip        node numbers
!     nelhel    local element number
!
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
!     ERCLOS    Deconcatenate subroutine name from calling string
!     EROPEN    Concatenate subroutine name to calling string
!     MSH403    Determine node number of new middle point on line
!     MSH416    Determine nodes of diagonal
!     MSH419    Fill element in element array, quadratic hexahedron
!
! ***********************************************************************
!
!     I / O
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
! ***********************************************************************
!
!     PSEUDO CODE
!
!     Run through all elements
!
!       Determine for each element the node numbers of the points
!       in the middle of the edges, in the middle of the faces
!       and finally in the barycentre.
!       Compute the coordinates of the internal points          (MSH431)
!
!       Fill new element (one quadratic hexaheder)              (MSH419)
!
! =======================================================================

      call eropen( 'msh457' )

!     Check each element; place coordinates of new points in array coorh
!     and nodenumbers of quadratic elements in array kelemh

      nelhel = 0

      do 500 i = 1 , nelem

          ip(1) = kelem( 8 * i - 7 )
          ip(2) = kelem( 8 * i - 6 )
          ip(3) = kelem( 8 * i - 5 )
          ip(4) = kelem( 8 * i - 4 )
          ip(5) = kelem( 8 * i - 3 )
          ip(6) = kelem( 8 * i - 2 )
          ip(7) = kelem( 8 * i - 1 )
          ip(8) = kelem( 8 * i     )

!         Compute middle node numbers

          call msh403( npoint, ip(1), ip(2), ibrpnt, istart, ip( 9) )
          call msh403( npoint, ip(2), ip(3), ibrpnt, istart, ip(10) )
          call msh403( npoint, ip(3), ip(4), ibrpnt, istart, ip(11) )
          call msh403( npoint, ip(4), ip(1), ibrpnt, istart, ip(12) )
          call msh403( npoint, ip(1), ip(5), ibrpnt, istart, ip(13) )
          call msh403( npoint, ip(2), ip(6), ibrpnt, istart, ip(14) )
          call msh403( npoint, ip(3), ip(7), ibrpnt, istart, ip(15) )
          call msh403( npoint, ip(4), ip(8), ibrpnt, istart, ip(16) )
          call msh403( npoint, ip(5), ip(6), ibrpnt, istart, ip(17) )
          call msh403( npoint, ip(6), ip(7), ibrpnt, istart, ip(18) )
          call msh403( npoint, ip(7), ip(8), ibrpnt, istart, ip(19) )
          call msh403( npoint, ip(8), ip(5), ibrpnt, istart, ip(20) )

!         Barycenters of quadrilateral faces

          call msh416( ip(1), ip(2), ip(3), ip(4), ia, ib )
          ib = ib * npoint
          call msh403( npoint, ia, ib, ibrpnt, istart, ip(21) )

!         Fill coorh for point ip(21)

          do 110 idim = 1 , 3

             coorh( idim, ip(21) ) = &
              ( coorh( idim, ip(1) ) + coorh( idim, ip(2) ) + &
                coorh( idim, ip(3) ) + coorh( idim, ip(4) ) ) / 4d0

110       continue

          call msh416( ip(1), ip(2), ip(6), ip(5), ia, ib )
          ib = ib * npoint
          call msh403( npoint, ia, ib, ibrpnt, istart, ip(22) )

!         Fill coorh for point ip(22)

          do 120 idim = 1 , 3

             coorh( idim, ip(22) ) = &
              ( coorh( idim, ip(1) ) + coorh( idim, ip(2) ) + &
                coorh( idim, ip(5) ) + coorh( idim, ip(6) ) ) / 4d0

120       continue

          call msh416( ip(2), ip(3), ip(7), ip(6), ia, ib )
          ib = ib * npoint
          call msh403( npoint, ia, ib, ibrpnt, istart, ip(23) )

!         Fill coorh for point ip(23)

          do 130 idim = 1 , 3

             coorh( idim, ip(23) ) = &
              ( coorh( idim, ip(2) ) + coorh( idim, ip(3) ) + &
                coorh( idim, ip(7) ) + coorh( idim, ip(6) ) ) / 4d0

130       continue

          call msh416( ip(4), ip(3), ip(7), ip(8), ia, ib )
          ib = ib * npoint
          call msh403( npoint, ia, ib, ibrpnt, istart, ip(24) )

!         Fill coorh for point ip(24)

          do 140 idim = 1 , 3

             coorh( idim, ip(24) ) = &
              ( coorh( idim, ip(3) ) + coorh( idim, ip(4) ) + &
                coorh( idim, ip(7) ) + coorh( idim, ip(8) ) ) / 4d0

140       continue

          call msh416( ip(1), ip(4), ip(8), ip(5), ia, ib )
          ib = ib * npoint
          call msh403( npoint, ia, ib, ibrpnt, istart, ip(25) )

!         Fill coorh for point ip(25)

          do 150 idim = 1 , 3

             coorh( idim, ip(25) ) = &
              ( coorh( idim, ip(1) ) + coorh( idim, ip(4) ) + &
                coorh( idim, ip(5) ) + coorh( idim, ip(8) ) ) / 4d0

150       continue

          call msh416( ip(5), ip(6), ip(7), ip(8), ia, ib )
          ib = ib * npoint
          call msh403( npoint, ia, ib, ibrpnt, istart, ip(26) )

!         Fill coorh for point ip(26)

          do 160 idim = 1 , 3

             coorh( idim, ip(26) ) = &
              ( coorh( idim, ip(5) ) + coorh( idim, ip(6) ) + &
                coorh( idim, ip(7) ) + coorh( idim, ip(8) ) ) / 4d0

160       continue

!         Finally the centroid of the hexahedron

          ia = min( ip(1), ip(7) )
          ib = max( ip(1), ip(7) )

          ib = npoint * ( npoint + 1 ) + ib

          call msh403( npoint, ia, ib, ibrpnt, istart, ip(27) )

!         Fill coorh for point ip(27)

          do 400 idim = 1 , 3

             coorh( idim, ip(27) ) = &
              ( coorh( idim, ip(21) ) + coorh( idim, ip(22) ) + &
                coorh( idim, ip(23) ) + coorh( idim, ip(24) ) + &
                coorh( idim, ip(25) ) + coorh( idim, ip(26) ) ) / 6d0

400       continue

!         Fill new element

          nelhel = nelhel + 1

          call msh419( kelemh, nelhel, &
                       ip( 1), ip( 9), ip( 2), ip(12), ip(21), ip(10), &
                       ip( 4), ip(11), ip( 3), ip(13), ip(22), ip(14), &
                       ip(25), ip(27), ip(23), ip(16), ip(24), ip(15), &
                       ip( 5), ip(17), ip( 6), ip(20), ip(26), ip(18), &
                       ip( 8), ip(19), ip( 7) )

500   continue

      call erclos( 'msh457' )

      end
