      subroutine mshangle ( x1, x2, x3, angle, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-11-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the cosine of the angle between the nodes with coordinates
!     x1, x2 and x3, where the angle is defined as x3-x1-x2
!     This value makes only sense for -pi < angle < pi
! **********************************************************************
!
!                       KEYWORDS
!
!     angle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim
      double precision angle, x1(*), x2(*), x3(*)

!     angle          o    cosine of angle x3-x1-x2
!     ndim           i    Dimension of the space.
!     x1             i    coordinates of node "x1"
!     x2             i    coordinates of node "x2"
!     x3             i    coordinates of node "x3"
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision t1(ndim), t2(ndim), norm1, norm2, alpha
      integer i

!     alpha          inner product of t1 and t2
!     i              Counting variable
!     norm1          length of t1 (squared)
!     norm2          length of t2 (squared)
!     t1             tangential vector x3-x1
!     t2             tangential vector x2-x1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The angle is defined as:
!
!     cos(phi) = (a,b)/||a||||b||, &
!
!     with a = x3-x1 and b = x2-x1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshangle' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshangle'

      end if
      alpha = 0d0
      norm1 = 0d0
      norm2 = 0d0
      do i = 1, ndim
         t1(i) = x3(i)-x1(i)
         t2(i) = x2(i)-x1(i)
         alpha = alpha+t1(i)*t2(i)
         norm1 = norm1+t1(i)**2
         norm2 = norm2+t2(i)**2
      end do
      norm1 = norm1*norm2
      if ( norm1.gt.epsmac ) then
         angle = alpha/sqrt(norm1)
      else
         angle = -rinfin
      end if

      if ( ierror.ne.0 ) go to 1000

1000  call erclos ( 'mshangle' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshangle'

      end if

      end
