      subroutine mshasprat2 ( npoint, ndim, nelem, kmeshc, kmeshd, coor, &
                              markelm, smooth, indelm, inpelm )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.0  date 26-09-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       Check the elements for small angles
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     triangle
!     subdivision
!     smoothing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleconsts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, npoint, nelem, kmeshc(*), kmeshd(*), smooth(npoint), &
              indelm(nelem), inpelm
      double precision  coor(ndim,*), markelm(nelem)

!     coor           i    Array containing the new coordinates of the nodes
!     indelm         i    Array of length nelem in which elements are marked
!                         of which at least one node is shifted
!     kmeshc         i    kmesh part c, contains the nodal points corresponding
!                         to the elements
!     kmeshd         i    KMESH part d, contains the elements corresponding to
!                         the nodal points
!     markelm        o    Array of nelem long filled with the minimal
!                         angles per element
!     ndim           i    Dimension of space
!     nelem          i    Number of elements
!     npoint         i    Number of nodal points
!     smooth         i    Array of npoint long which indicates if nodes
!                         should be checked for small angles (1) or not (0)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  i,j,k,ii, ielem, idx(3,3), inode, jnode(4), nrelem, &
               knode, idx1(4)
      double precision  length(2,3), sum, minang, ang(3), veca(3,3), &
                        vecb(3,3), eps

!     ang            Angle
!     eps            Local accuracy
!     i              Counting variable
!     idx            Array for roulation of indices
!     idx1           Array for finding vertices
!     ielem          Element number
!     ii             Counting variable
!     inode          Node sequence number
!     inpelm         number of points per element
!     j              Counting variable
!     jnode          Array containing the node numbers of the vertices
!     k              Counting variable
!     knode          Node to be shifted
!     length         Help parameter to indicate the length of arrays
!     minang         Minimal angle
!     nrelem         Number of elements around node
!     sum            Help parameter to compute a sum
!     veca           Array containing 3 edges of an element which have
!                    the same node
!     vecb           Array containing the three normals to each couple
!                    of edges in veca
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data idx / 1,2,3, 2,3,1, 3,1,2  /
      data idx1 / 1,3,5,10 /
! ======================================================================
!
      call eropen ( 'mshasprat2' )

      eps=1d-13

      do i=1,nelem
         markelm(i)=0d0
      end do

!     --- loop over nodes

      do knode=1,npoint

!     --- if knode may be shifted

         if (smooth(knode).ne.0) then

!        --- determine the nr of elements around knode

            nrelem=kmeshd(knode+1) - kmeshd(knode)

!           --- loop over the elements around knode

            do k=1,nrelem
               ielem=kmeshd(npoint+1+kmeshd(knode)+k)
               indelm(ielem)=1

!              --- determine which nodes are involved in element

               if (markelm(ielem).gt.eps) then
                  minang=pi

!                 --- find the vertex node numbers

                  do j=1,4

!                 --- linear tetrahedron

                     if (inpelm.eq.4) then
                        jnode(j) = &
                            kmeshc((ielem-1)*inpelm+j)

!                       --- quadratic tetrahedron

                     else if (inpelm.eq.15) then
                        jnode(j) = kmeshc( &
                                   (ielem-1)*inpelm+idx1(j))
                     end if
                  end do

!                 --- determine the lengths of the elementsides
!                     first loop over nodes

                  do inode=1,4
                     ii=1

!                    --- second loop over nodes

                     do j=1,4
                        if(j.ne.inode) then
                           do i=1,ndim

!                          --- determine the edges of the element connected
!                              to nodeloop over nodes

                               veca(i,ii) = coor(i,jnode(j))- &
                                            coor(i,jnode(inode))
                           end do
                           ii=ii+1
                        end if
                     end do

!                    --- compute the normals to the plane and the lengths of
!                        these normals and the lengths of the edges

                     do j=1,3
                        do i=1,ndim
                           vecb(i,j)=veca(idx(i,2),j)* &
                                     veca(idx(i,3),idx(j,2))- &
                                     veca(idx(i,3),j)* &
                                     veca(idx(i,2),idx(j,2))
                        end do
                        length(1,j)= sqrt(veca(1,j)**2d0+ &
                                          veca(2,j)**2d0+ &
                                          veca(3,j)**2d0 )
                        length(2,j)= sqrt(vecb(1,j)**2d0+ &
                                          vecb(2,j)**2d0+ &
                                          vecb(3,j)**2d0 )
                     end do

!                    --- compute the angles

                     do j=1,3
                        sum=0d0
                        do i=1,ndim
                           sum=sum+veca(i,idx(j,3))*vecb(i,j)
                        end do
                        ang(j) = acos(sum/(length(1,idx(j,3))* &
                                      length(2,j)))

!                       --- compute angle depending on the direction of
!                           the normal

                        if (ang(j).le.0.5d0*pi) then
                           ang(j)=0.5d0*pi-ang(j)
                        else
                           ang(j)=ang(j)-0.5d0*pi
                        end if
                        minang=min(ang(j), minang)
                        markelm(ielem)=minang
                     end do

                  end do  ! end if first node loop

               end if

            end do  ! end element loop

         end if

      end do

      call erclos ( 'mshasprat2' )
      end
