      subroutine mshcrtvolm ( iinput, jnew, activatm, rinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.3  date 27-11-2012 Long integers
!        version  3.2  date 30-07-2012 New treatment of iinput/rinput
!        version  3.1  date 24-08-2010 Removal of work3
!
!   copyright (c) 2002-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create volumes for the mesh generator
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iinput(*)
      double precision  rinput(*)
      logical jnew, activatm

!     activatm       i    If true kmesh part m must be activated
!     iinput         i    integer input array
!     jnew           i    indication whether the mesh is new (true) or old
!     rinput         i    Real input array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iposi, icurvs, curves

!     curves         memory management sequence number of array curves
!                    they are not
!     icurvs         memory management sequence number of array icurvs
!     iposi          next free position of iinput
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSH012         write surface isurf to file 3; fill positions of kmesh
!                    part h
!     MSH043         initialize some parameters for mshvlm
!     MSHVLM         Generate one volume
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Initialize parameters
!     For all volumes ivolnr do
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcrtvolm' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcrtvolm'
  1      format ( a, 1x, (10i8) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Initialize parameters

      iposi = iinp5

      if ( debug ) &
         write(irefwr,1) 'ndim, nsurfs, nvolms, iinp3, iposi', &
                          ndim, nsurfs, nvolms, iinp3, iposi

!     --- For all volumes ivolnr do

      do ivolnr = 1, nvolms

         call msh043 ( iinput(iposi), iinput(iinp3), iposi )
         if ( ierror/=0 ) go to 1000

!        --- Activate some arrays with respect to mshvlm

         if ( iinput(iposi)==0 ) then

!        --- empty volume

            iposi = iposi+1
            go to 500

         end if
         if ( debug ) write(irefwr,1) 'ivolnr, iposi', ivolnr, iposi

         lnkbnd = max(lnkbnd,npointlc)
         if ( debug ) then
            write(irefwr,1) 'inpelm, npointlc, nelemlc', &
                             inpelm, npointlc, nelemlc
            write(irefwr,1) 'lnkbnd, lnbcor', lnkbnd, lnbcor
            write(irefwr,1) 'nextracrvs, nextrasrfs', &
                             nextracrvs, nextrasrfs
            write(irefwr,1) 'lnkbnd, length', &
                             lnkbnd, length
         end if  ! ( debug )
         allocate ( kmeshc(inpelm*nelemlc), coor3d(ndim,npointlc), &
                    kbndptlc(lnkbnd+1+nextracrvs), &
                    numnodextcurvs(nextracrvs), &
                    numelmextsurfs(nextrasrfs), &
                    stat = error )
         if ( lnbcor>0 ) allocate ( bcorold(ndim,lnbcor/3), stat = error )
         if ( error/=0 ) &
            call eralloc ( error, inpelm*nelemlc+ndim*npointlc+&
                           lnkbnd+lnbcor+nextracrvs, 'kmeshc' )
         if ( ierror/=0 ) go to 1000

!        --- Recompute pointers for real arrays

         if ( lnbcor==0 ) bcorold => coor3d
         kmeshclc => kmeshc
         call mshvlm ( iinput, iposi, jnew, bcorold, rinput )
         if ( ierror/=0 ) go to 1000

!        --- Make some local arrays permanent and store in kmeshh

         isurf = 1
         isurnr = ivolnr+nsurfs
         call msh012 ( jnew )

         deallocate ( kmeshc, coor3d, kbndptlc, numnodextcurvs, &
                      numelmextsurfs, stat = error )
         if ( lnbcor>0 ) deallocate ( bcorold, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshc' )
         if ( ierror/=0 ) go to 1000

500   end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcrtvolm'

      end if
      call erclos ( 'mshcrtvolm' )

      end subroutine mshcrtvolm
