      subroutine mshcurves2 ( istore, numpoints, kmeshmtmp, iworkmarknods, &
                              isurnr, jsurnr, numedges, &
                              istoreedges, iworkedges, iworkmarkedgs, &
                              istartm, ncurvsnew, ipoint, iworksurfs, &
                              curvstosurfs, lenkmeshmtmp, &
                              lncurvstosurfs, maxlen )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.1  date 29-04-2005 Combination open and closed curves
!        version  4.0  date 08-04-2005 Extra parameter maxlen
!        version  3.0  date 05-01-2005 Extra parameter lncurvstosurfs
!
!   copyright (c) 2004-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Sort nodes in curve in order to get a SEPRAN curve
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     curve
!     boundary
!     intersection
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istore(*), numpoints, kmeshmtmp(*), iworkmarknods(*), &
              isurnr, jsurnr, numedges, iworkmarkedgs(*), &
              istoreedges(*), iworkedges(*), istartm, ncurvsnew, ipoint(*), &
              iworksurfs(*), curvstosurfs(nsurfs,*), &
              lenkmeshmtmp, lncurvstosurfs, maxlen

!     curvstosurfs  i/o   Array of size nsurfs x nsurfs
!                         curvstosurfs(i,j) contains the j-th curve
!                         corresponding to surface i
!                         if 0 no more curves can be found
!     ipoint         i    Array of length nparts+1 containing the starting
!                         addresses in array inodes of the various parts
!                         the boundary consists of.
!                         ipoint(1) = 1, &
!                         number of nodes at part i = ipoint(i+1)-ipoint(i)
!     istartm       i/o   Starting address of next curve in kmeshm
!     istore        i/o   Work array of length numpoints to store the node
!                         numbers in the curve
!                         At input it contains all nodes of the curves
!                         (unsorted)
!     istoreedges   i/o   Work array of length nedges used to store remaining
!                         edge numbers along the curve
!                         At input it contains all edge numbers along the curve
!     isurnr         i    Surface sequence number of surface 1
!     iworkedges    i/o   Work array of length nedges
!                         iworkedge(i) = 0: edge is not in isurnr
!                         iworkedge(i) = 1: edge is in isurnr but nor jsurnr
!                         iworkedge(i) = 2: edge is in isurnr and jsurnr
!                         iworkedge(i) = 3: edge has been stored
!     iworkmarkedgs       integer work array of length nedges
!                         if iworkmarkedgs(i)>0, i is stored in istoreedges
!     iworkmarknods       integer work array of length npoint
!                         if iworkmarknods(i)>0, i is stored in
!                         ordered curve
!     iworksurfs    i/o   Work array of length nsurfs to store the number of
!                         curves that have been filled in curvstosurfs
!                         for each surface
!     jsurnr         i    Surface sequence number of surface 2
!     kmeshmtmp      o    KMESH part m, contains nodal point numbers of the
!                         curve
!     lenkmeshmtmp   i    Length of array kmeshmtmp (here kmeshm)
!     lncurvstosurfs i    Declared length of second dimension of array
!                         curvstosurfs
!     maxlen        i/o   Maximum number of curves that correspond to one
!                         surface
!     ncurvsnew     i/o   Last curve sequence number used
!     numedges       i    number of edges along curve
!     numpoints      i    number of points in istore (hence curve)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inode, icount, j, storenumedges(numpoints), &
              ifirst, nopen, nremaining, nnedges, jcount, &
              ipstart, jedge

!     i              Counting variable
!     icount         Counter number of open curves stored
!     ifirst         Position of firstnode in kmeshm
!     inode          Node sequence number
!     ipstart        Starting address in array
!     j              Counting variable
!     jcount         Counter number of closed curves stored
!     jedge          edge sequence number
!     nnedges        Number of edges
!     nopen          Number of open curves
!     nremaining     Number of remaining curves
!     storenumedges  Array of length numpoints to store the number of
!                    edges that are in the curve per point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     MSHCURVES2CLOS Store next closed boundary in kmeshm
!     MSHCURVES2OPEN Store next open boundary in kmeshm
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Store number of edges in curves coupled to all nodes in storenumedges
!     nopen := number of open curves
!     nremaining := numpoints;  ifirst := 1; icount := 0
!     while ( nremaining>0 ) do
!        if ( nopen>0 ) then
!           Store next open boundary in kmeshm
!        else
!           Store next closed boundary in kmeshm
!        Raise icount, ncurvsnew
!        Remove nodes from istore and storenumedges
!        Decrease nremaining, nopen
!        Raise ifirst
!        Store information in arrays ipoint, iworksurfs and  curvstosurfs
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcurves2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcurves2'
100      format ( a, 1x, (10i6) )
         write(irefwr,100) 'isurnr, jsurnr', isurnr, jsurnr
         write(irefwr,100) 'nedges, numpoints, numedges', &
                            nedges, numpoints, numedges
         write(irefwr,100) 'npoint', npoint
         call prinin ( istore, numpoints, 'node numbers in curve' )
         call prinin ( istoreedges, numedges, 'edge numbers in curve' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Mark edges stored in istoreedges

      do i = 1, nedges
         iworkmarkedgs(i) = 0
      end do  ! i = 1, nedges
      do i = 1, numedges
         iworkmarkedgs(istoreedges(i)) = i
      end do  ! i = 1, numpoints

!     --- Store number of edges in curves coupled to all nodes in storenumedges
!         nopen := number of open curves

      nopen = 0
      do j = 1, numpoints

         inode = istore(j)     ! next node in istore
         ipstart = nmeshl(inode)   ! starting address of node in nmeshl
         nnedges = 0               ! number of edges corresponding to node
         do i = 1, nmeshl(ipstart)

            jedge = nmeshl(ipstart+i)  ! edge corresponding to inode
            if ( iworkedges(jedge)==2 ) nnedges = nnedges+1

         end do  ! i = 1, nmeshl(ipstart)
         storenumedges(j) = nnedges
         if ( nnedges==1 ) then
            nopen = nopen + 1
            if ( debug ) then
               write(irefwr,100) 'Node number at end of curve', inode
               write(irefwr,100) 'nopen, j', nopen, j
            end if  ! ( debug )
         end if  ! ( nnedges==1 )

      end do  ! j = 1, numpoints
      nopen = nopen/2
      if ( debug ) then

!     --- Debug information

         write(irefwr,100) 'number of open curves is', nopen
         write(irefwr,100) 'number of points is', numpoints
         call prinin ( storenumedges, numpoints, 'storenumedges' )

      end if  ! ( debug )

!     --- nremaining := numpoints;  ifirst := 1; icount := 0

      nremaining = numpoints
      ifirst = 1
      icount = 1

      do while ( nremaining>0 )

!     --- nremaining>0, there are nodes left
!         Store refence to istore in iworkmarknods

         do i = 1, nremaining
            iworkmarknods(istore(i)) = i
         end do  ! i = 1, nremaining
         if (debug ) &
            write(irefwr,100) 'Number of remaining nodes', nremaining

         if ( nopen>0 ) then

!        --- nopen>0:  Store next open boundary in kmeshm

            call mshcurves2open ( istore, storenumedges, icount, &
                                  kmeshmtmp(ifirst), nmeshl, nmeshc, &
                                  iworkmarkedgs, iworkmarknods, &
                                  lenkmeshmtmp, ifirst+istartm )
            nopen = nopen-1  ! decrease nopen

         else

!        --- nopen=0:    Store next closed boundary in kmeshm

            call mshcurves2clos ( istore, icount, kmeshmtmp(ifirst), &
                                  nmeshl, nmeshc, iworkmarkedgs, &
                                  iworkmarknods, &
                                  lenkmeshmtmp, ifirst+istartm )

         end if  ! ( nopen>0 )
         if ( ierror/=0 ) go to 1000
         if ( debug ) then
            write(irefwr,100) 'Number of nodes stored', icount
            call prinin ( kmeshmtmp(ifirst), icount, 'kmeshm' )
         end if  ! ( debug )

!        --- Remove nodes from istore and storenumedges

         jcount = 0
         do i = 1, nremaining

            inode = istore(i)
            if ( iworkmarknods(inode)>0 ) then

!           --- node has not been treated yet

               jcount = jcount+1
               istore(jcount) = inode
               storenumedges(jcount) = storenumedges(i)

            end if  ! ( iworkmarknods(inode)>0 )

         end do  ! i = 1, nremaining
         nremaining = jcount

         if ( debug .and. jcount>0 ) then
            write(irefwr,100) 'number of remaining nodes is', jcount
            call prinin ( istore, jcount, 'istore' )
            call prinin ( storenumedges, jcount, 'storenumedges' )
         end if  ! ( debug )

!        --- Raise icount, ncurvsnew
!            Decrease nremaining, nopen
!            Raise ifirst
!            Store information in arrays ipoint, iworksurfs and  curvstosurfs

         ncurvsnew = ncurvsnew+1
         istartm = istartm+icount
         ipoint(ncurvsnew) = istartm
         ifirst = ifirst+icount
         if ( debug ) then
            write(irefwr,100) 'isurnr, jsurnr, icurnr', &
                            isurnr, jsurnr, ncurvsnew+ncurvs
            write(irefwr,100) 'iworksurfs(isurnr)', iworksurfs(isurnr)
            write(irefwr,100) 'iworksurfs(jsurnr)', iworksurfs(jsurnr)
         end if  ! ( debug )

!        --- add curve to surfaces

         iworksurfs(isurnr) = iworksurfs(isurnr)+1

         call inichkarlen ( iworksurfs(isurnr), lncurvstosurfs, &
                            'curvstosurfs' )
         maxlen = max(maxlen,iworksurfs(isurnr))
         if ( ierror/=0 ) go to 1000

         curvstosurfs(isurnr,iworksurfs(isurnr)) = ncurvsnew+ncurvs
         iworksurfs(jsurnr) = iworksurfs(jsurnr)+1

         call inichkarlen ( iworksurfs(jsurnr), lncurvstosurfs, &
                            'curvstosurfs' )
         maxlen = max(maxlen,iworksurfs(jsurnr))
         if ( ierror/=0 ) go to 1000

         curvstosurfs(jsurnr,iworksurfs(jsurnr)) = ncurvsnew+ncurvs

      end do  ! while ( nremaining>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,100) 'numpoints', numpoints
         call prinin ( kmeshmtmp, numpoints, 'kmeshm' )
         write(irefwr,*) 'End mshcurves2'

      end if
      call erclos ( 'mshcurves2' )

      end
