      subroutine mshdelsel ( kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 25-09-2006 Set km(iseqkm)%(7) = 0
!        version  1.2  date 22-06-2004 ini066 --> ini066text
!        version  1.1  date 01-10-2003 Remove km(iseqkm)% part aa
!
!   copyright (c) 2003-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Delete a selective part of array km(iseqkm)% and all of its corresponding arrays
!
! **********************************************************************
!
!                       KEYWORDS
!
!     delete
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh

!     kmesh     i/o   kmesh array to be deleted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshdelsel' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshdelsel'
         call ertrace
         write(irefwr,1) 'kmesh', kmesh
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkm = kmesh

      if ( debug ) write(irefwr,1) 'iseqkm', iseqkm
      km(iseqkm)%kmesh(7) = 0

      if ( associated(km(iseqkm)%rennested) ) then
         deallocate ( km(iseqkm)%rennested, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'rennested' )
         nullify(km(iseqkm)%rennested )
      end if  ! ( associated(km(iseqkm)%rennested) )
      if ( associated(km(iseqkm)%kmeshd) ) deallocate ( km(iseqkm)%kmeshd )
      km(iseqkm)%kmesh(18) = 0
      if ( associated(km(iseqkm)%kmeshe) ) deallocate ( km(iseqkm)%kmeshe )
      km(iseqkm)%kmesh(19) = 0
      if ( associated(km(iseqkm)%kmeshj) ) deallocate ( km(iseqkm)%kmeshj )
      km(iseqkm)%kmesh(24) = 0
      if ( associated(km(iseqkm)%kmeshk) ) deallocate ( km(iseqkm)%kmeshk )
      km(iseqkm)%kmesh(25) = 0
      if ( associated(km(iseqkm)%kmesho) ) deallocate ( km(iseqkm)%kmesho )
      km(iseqkm)%kmesh(29) = 0

!     --- km(iseqkm)% part aa

      if ( km(iseqkm)%kmesh(48)>0 ) then
         km(iseqkm)%nmesh = 0
         if ( associated(km(iseqkm)%nmesha) ) deallocate ( km(iseqkm)%nmesha )
         if ( associated(km(iseqkm)%nmeshb) ) deallocate ( km(iseqkm)%nmeshb )
         if ( associated(km(iseqkm)%nmeshc) ) deallocate ( km(iseqkm)%nmeshc )
         if ( associated(km(iseqkm)%nmeshd) ) deallocate ( km(iseqkm)%nmeshd )
         if ( associated(km(iseqkm)%nmeshe) ) deallocate ( km(iseqkm)%nmeshe )
         if ( associated(km(iseqkm)%nmeshf) ) deallocate ( km(iseqkm)%nmeshf )
         if ( associated(km(iseqkm)%nmeshg) ) deallocate ( km(iseqkm)%nmeshg )
         if ( associated(km(iseqkm)%nmeshh) ) deallocate ( km(iseqkm)%nmeshh )
         if ( associated(km(iseqkm)%nmeshi) ) deallocate ( km(iseqkm)%nmeshi )
         if ( associated(km(iseqkm)%nmeshj) ) deallocate ( km(iseqkm)%nmeshj )
         if ( associated(km(iseqkm)%nmeshk) ) deallocate ( km(iseqkm)%nmeshk )
         if ( associated(km(iseqkm)%nmeshl) ) deallocate ( km(iseqkm)%nmeshl )
         nullify ( nmesha, nmeshb, nmeshc, nmeshd, nmeshe, nmeshf, nmeshg )
         nullify ( nmesh, nmeshi, nmeshj, nmeshj, nmeshl )
      end if
      km(iseqkm)%kmesh(48) = 0

!     --- km(iseqkm)% part ac

      if ( associated(km(iseqkm)%kmeshac) ) deallocate ( km(iseqkm)%kmeshac )
      km(iseqkm)%kmesh(51) = 0
      if ( associated(km(iseqkm)%rmeshac) ) deallocate ( km(iseqkm)%rmeshac )
      if ( associated(km(iseqkm)%transinfo) ) &
         deallocate ( km(iseqkm)%transinfo )
      if ( associated(km(iseqkm)%blockinfo) ) &
         deallocate ( km(iseqkm)%blockinfo )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshdelsel'

      end if  ! ( debug )
      call erclos ( 'mshdelsel' )

      end subroutine mshdelsel
