      subroutine mshdirsurf ( kmeshc, inpelm, inelem, firstpnt, nextpnt, &
                              idirec, isurnr, icurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-03-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the direction of the outer boundary of a surface by
!     checking the direction of the first two nodes on the boundary
!     compared two the element it is lying in
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     direction
!     mesh
!     outer_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, inelem, kmeshc(inpelm,inelem), firstpnt, nextpnt, &
              idirec, isurnr, icurnr

!     firstpnt       i    First point of a curve
!     icurnr         i    Curve sequence number
!     idirec         o    Defines in which direction the curves must be
!                         considered
!                         idirec = 1, from low to high, idirec = -1: reversed
!                         order
!     inelem         i    Number of elements in a specific element group
!     inpelm         i    number of points per element
!     isurnr         i    Surface sequence number
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nextpnt        i    Next point on a curve
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, ncount, nodp, istore(2), j
      logical found

!     found          If true already a possibility has been found
!     ielem          Element number
!     istore         Contains the last level set
!     j              Counting variable
!     ncount         Counter
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2333   Inconsistency in surface, nodes on curve do not belong to surface
! **********************************************************************
!
!                       PSEUDO CODE
!
!     While ( element not found ) do
!        ncount := number of nodes in next element equal to firstpnt, nextpnt
!        if ( ncount = 2 ) element has been found
!     Find direction of element
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshdirsurf' )
      found = .false.
      debug = .false.

!     --- While ( element not found ) do

      ielem = 0
      do while ( .not. found .and. ielem.lt.inelem )

         ielem = ielem+1

!        --- ncount := number of nodes in next element equal to
!            firstpnt, nextpnt

         ncount = 0
         do j = 1, inpelm
            nodp = kmeshc(j,ielem)
            if ( nodp.eq.firstpnt ) then

!           --- Nodal point equal to first point

               ncount = ncount+1
               istore(1) = j

            else if ( nodp.eq.nextpnt ) then

!           --- Nodal point equal to next point

               ncount = ncount+1
               istore(2) = j

            end if

         end do
         if ( ncount.eq.2 ) found = .true.

      end do ! .not. found

!     --- Find direction of element

      if ( found ) then

!     --- found = true, element found

         if ( debug )  then

!        --- debugging statements

            write(irefwr,*) 'ielem, firstpnt, nextpnt, istore ', &
                             ielem, firstpnt, nextpnt, istore
            call prinin ( kmeshc(1,ielem), inpelm, 'element' )

         end if

         if ( istore(2).eq.istore(1)+1 ) then

!        --- two succesive nodes in positive direction

            idirec = 1

         else if ( istore(1).eq.istore(2)+1 ) then

!        --- two succesive nodes in negative direction

            idirec = -1

         else if ( istore(1).eq.1 ) then

!        --- first node is node 1, next node is last node
!            must be negative direction

            idirec = -1

         else

!        --- next node is node 1, first node is last node
!            must be positive direction

            idirec = 1

         end if

      else

!     --- error: no element found

         call errint ( isurnr, 1 )
         call errint ( icurnr, 2 )
         call errint ( firstpnt, 3 )
         call errint ( nextpnt, 4 )
         call errsub ( 2333, 4, 0, 0 )

      end if

      call erclos ( 'mshdirsurf' )
      end
