      subroutine mshfillkmeshs ( curvstosurfs, lncurvstosurfs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 27-11-2012 Long integers
!        version  1.3  date 11-01-2006 Extra debug information
!        version  1.2  date 08-04-2005 Correction for ncurvs>nsurfs
!                                      (per surface)
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information in array kmeshs using only information stored in
!     array kmeshn
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lncurvstosurfs, curvstosurfs(lncurvstosurfs)

!     curvstosurfs   i    Array of size nsurfs x nsurfs
!                         curvstosurfs(i,j) contains the j-th curve
!                         corresponding to surface i
!                         if 0 no more curves can be found
!     lncurvstosurfs i    Length of array curvstosurfs
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lencurvstosf

!     lencurvstosf   Length of second dimension of curvstosurfs
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHFILLKMESHS1 Fill kmesh part s from the information in kmesh part n
!     MSHFILLKMESHS2 Compute the length of array curvstosurfs
!     MSHFILLKMESHS3 Reorganize curves stored in kmeshs part 1 in order to get
!                    closed curves if possible
!     SEPFILLKMESHS  Make relation between kmeshs and kmeshs2
!     SEPPRINKMESHS  Print the contents of array kmeshs
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Create space for array kmeshs
!     Fill array kmeshs
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfillkmeshs' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfillkmeshs'

      end if
      if ( ierror/=0 ) go to 1000

      lencurvstosf = lncurvstosurfs/nsurfs  ! length of array
                                              ! divided by first dimension
      call mshfillkmeshs2 ( curvstosurfs, nsurfs, &
                            lncurvstosurfs, lencurvstosf )

!     --- Create space for array kmeshs

      length = nsurfs+1+8*nsurfs+lncurvstosurfs
      allocate ( km(iseqkm)%kmeshs(length))
      kmeshs => km(iseqkm)%kmeshs
      km(iseqkm)%kmesh(33) = length
      call sepfillkmeshs ( km(iseqkm)%kmeshs(nsurfs+2) )
      if ( ierror/=0 ) go to 1000

!     --- Fill array kmeshs

      call mshfillkmeshs1 ( nsurfs, kmeshn, kmeshs, &
                            kmeshs(nsurfs+2), curvstosurfs, lencurvstosf )

!     --- Reorder array kmeshs

      call mshfillkmeshs3

1000  if ( debug ) then

!     --- Debug information

         call  sepprinkmeshs
         write(irefwr,*) 'End mshfillkmeshs'

      end if
      call erclos ( 'mshfillkmeshs' )

      end
