      subroutine mshfillvert2d ( kmeshc_loc, ishape, nvertices, ivert )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-03-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the node numbers of the vertices of a 2d element in array ivert
!     and store number of vertices in nvertices
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     element
!     vertex
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc_loc(*), ishape, nvertices, ivert(*)

!     ishape         i    Shape number of elements
!     ivert          o    Array of size nvertices containing the node numbers
!                         of the vertices of an element
!     kmeshc_loc     i    Local array kmeshc for one element
!                         Contains the node numbers of the element
!     nvertices      o    Number of vertices
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfillvert2d' )

      if ( ishape.eq.3 .or. ishape.eq.10 ) then

!     --- Linear triangle

         nvertices = 3
         do i = 1, nvertices
            ivert(i) = kmeshc_loc(i)
         end do

      else if ( ishape.eq.5 .or. ishape.eq.9 ) then

!     --- Bi-linear quadrilateral

         nvertices = 4
         do i = 1, nvertices
            ivert(i) = kmeshc_loc(i)
         end do

      else if ( ishape.eq.4 .or. ishape.eq.7 ) then

!     --- quadratic triangle

         nvertices = 3
         do i = 1, nvertices
            ivert(i) = kmeshc_loc(2*i-1)
         end do

      else if ( ishape.eq.6 ) then

!     --- bi-quadratic quadrilateral

         nvertices = 4
         do i = 1, nvertices
            ivert(i) = kmeshc_loc(2*i-1)
         end do

      else

!     --- shape number not implemented

         call errint ( ishape, 1 )
         call errsub ( 800, 1, 0, 0 )

      end if
      call erclos ( 'mshfillvert2d' )
      end
