      subroutine mshfl1 ( icurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 02-09-2010 Remove intarmsh
!        version  3.3  date 20-10-2008 Allow isurf = 24
!        version  3.2  date 13-03-2006 Debug statements
!
!   copyright (c) 1987-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    fill array icurvs with starting positions in array kbndpt
!    with respect to composite surface
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icurvs(*)

!     icurvs     o    Array of length ncurvs containing information of the
!                     curves with respect to kbndpt
!                     icurvs is filled as follows:
!                     icurvs(i) = 0    Curve i is not present
!                     icurvs(i)>0    Curve i has not been used before
!                                      The value gives the starting position in
!                                      array kbndpt
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ich1, icurve, ifirst, ilast, istar3, jcurve, &
              jsurf, k, k1, k2, n, numc, ncursf, ns, ntot

!     i              Counting variable
!     ich1
!     icurve         Type of curve
!     ifirst         First curve (surface) number read
!     ilast          Last curve (surface) number read
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     jcurve         Curve number
!     jsurf
!     k              Counting variable
!     k1             First point of curve in KMESH part m
!     k2             Last point of curve in KMESH part m
!     n              Parameter for mesh generation
!     ncursf
!     ns             Starting address of information of specific surface in
!                    array KMESHS
!     ntot
!     numc           number of points along the curves
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfl1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfl1'

      end if
      if ( ierror/=0 ) go to 1000

!     --- First clear icurvs

      icurvs(1:ncurvs) = 0
      ich1 = kmeshs(isurnr)
      ncursf = kmeshs(isurnr+1) - ich1
      istar3 = kmeshm(2)-1

!     --- Loop over curves of surface

      nbndpt = 1
      ns = nsurfs + 2 + (isurnr-1) * 8
      jsurf = kmeshs(ns)
      if ( jsurf==7 .or. jsurf==11 .or. jsurf==24 ) then

!     --- jsurf = 7  surface of surfaces or
!         jsurf = 11  ordened surface of surfaces of type rectangle

         ilast = 0
         do i = 1, ncursf
            icurve = kmeshs(i+ich1)
            jcurve = abs(icurve)
            k1 = kmeshm(4+jcurve)
            k2 = kmeshm(5+jcurve)-1
            if ( icurve>0 ) then

!           --- icurve>0

              ifirst = kmeshm(istar3+k1)
              if ( ifirst==ilast ) nbndpt = nbndpt - 1
              ilast = kmeshm(istar3+k2)

             else

!            --- icurve<0

              ifirst = kmeshm(istar3+k2)
              if ( ifirst==ilast ) nbndpt = nbndpt - 1
              ilast = kmeshm(istar3+k1)

            end if
            icurvs(jcurve) = nbndpt
            n = k2-k1+1
            nbndpt = nbndpt + n

         end do

      else

!     --- jsurf = 12  ordened surface of surfaces of type pipesurface

         nbndpt = 1
         ntot = 0
         do k  = 1, 4
            numc = kmeshs(ns+k+1)
            if ( numc>0 ) then
               ntot = ntot + numc
               do i = 1, numc
                  icurve = abs ( kmeshs(i+ich1) )
                  if ( i==1 .and. icurvs(icurve)==0 ) &
                              icurvs(icurve) = nbndpt
                  n = kmeshm(icurve+5) - kmeshm(icurve+4)
                  nbndpt = nbndpt + n - 1
               end do
               nbndpt = nbndpt + 1
               ich1 = ich1 + numc
            end if
         end do
         do i = 1, ncursf-ntot
            icurve = abs ( kmeshs(i+ich1) )
            icurvs(icurve) = nbndpt
            n = kmeshm(icurve+5) - kmeshm(icurve+4)
            nbndpt = nbndpt + n
         end do
      end if
      nbndpt = nbndpt - 1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshfl1'

      end if
      call erclos ( 'mshfl1' )

      end
