      subroutine mshmakelistg ( listgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-01-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array listgroup with a 1 when an element is a volume element and
!     a zero otherwise
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     volume
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer listgrp(*)

!     listgrp        o    Array of size nelgrp indicating if an element
!                         group must be used (1) or not (0)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp

!     ielgrp         element group number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshmakelistg' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshmakelistg'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Check if mesh has volume elements
!         Fill listgrp for all element groups with elements of highest dimension

      if ( nvolms>0 ) then

!     --- There are volume elements

         do ielgrp = 1, nelgrp

            ishape = kmeshf(ielgrp)
            if ( ishape>10 .and. ishape<50 ) then

!           --- ishape > 10, hence volume element

               listgrp(ielgrp) = 1

            else

!           --- ishape <= 10, hence not a volume element

               listgrp(ielgrp) = 0

            end if  ! ( ishape>10 )

         end do  ! ielgrp = 1, nelgrp

      else

!     --- There are no volume elements

         do ielgrp = 1, nelgrp

            ishape = kmeshf(ielgrp)
            if ( ishape>2 .and. ishape<10 ) then

!           --- 3 <= ishape <= 10, hence surface element

               listgrp(ielgrp) = 1

            else

!           --- ishape <= 10, hence not a volume element

               listgrp(ielgrp) = 0

            end if  ! ( ishape>10 )

         end do  ! ielgrp = 1, nelgrp

      end if  ! ( nvolms>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshmakelistg'

      end if
      call erclos ( 'mshmakelistg' )

      end
