      subroutine mshmetis3 ( iprocessor, nprocessors, elements, &
                             points, numelms, numnodes, elperseg )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-03-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array elements with the elements corresponding to one block
!     and also the corresponding array points with the point numbers
!     In this case the elements are subdivided by metis
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     parallel
!     subdivision
!     metis
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iprocessor, nprocessors, elperseg(*)
      integer, intent(out) :: elements(*), points(*), numelms, numnodes

!     elements       o    Array of size nelemloc containing the element
!                         numbers corresponding to the local mesh
!     elperseg      i/o   Array of size nelem containing the sequence
!                         number of the processor to which it is added
!     iprocessor     i    Processor sequence number
!     nprocessors    i    Number of processors that is to be used for
!                         parallel processing
!                         Defines also the number of submeshes
!     numelms        o    Number of elements
!     numnodes       o    Number of nodal points in array nodes
!     points         o    Array of size npointloc containing the node
!                         numbers corresponding to the local mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, inpelm, ielem, istart, ielgrp, iielem

!     i              Counting variable
!     ielem          Element number
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     inpelm         number of points per element
!     istart         First position used at a part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHMETIS4      Add all elements of higher processor number that have all
!                    nodes in this block to present processor
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshmetis3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshmetis3'
         write(irefwr,1) 'nprocessors, npoint', nprocessors, npoint
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(npoint) )

!     --- Clear array iwork (1:npoint)

      iwork(1:npoint) = 0

!     --- numelms := 0

      numelms = 0

!     --- For all elements ielem do

      ielem  = 0
      istart = 0
      do ielgrp = 1, nelgrp

         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm
         do iielem = 1, inelem
            ielem = ielem + 1
            if ( elperseg(ielem)==iprocessor ) then

!           --- new element found

               numelms = numelms+1
               elements(numelms) = ielem
               do i = 1, inpelm
                  iwork(kmeshc(istart+i)) = 1
               end do

            end if
            istart = istart+inpelm

         end do  ! iielem = 1, inelem

      end do  ! ielgrp = 1, nelgrp

!     --- Fill array points using iwork

      numnodes = 0
      do i = 1, npoint

         if ( iwork(i)==1 ) then

!        --- iwork(i) = 1, hence node belongs to block

            numnodes = numnodes+1
            points(numnodes) = i

         end if

      end do  ! i = 1, npoint

!     --- Check if there are elements in other blocks with all nodes in this
!         block. If so add element to this block

      call mshmetis4 ( iprocessor, nprocessors, numelms, &
                       iwork, elements, elperseg )

      deallocate ( iwork )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( elements, numelms, 'elements' )
         call prinin ( points, numnodes, 'points' )
         write(irefwr,*) 'End mshmetis3'

      end if  ! ( debug )
      call erclos ( 'mshmetis3' )

      end subroutine mshmetis3
