      subroutine mshnumnodcrv ( icurnr, numnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 24-11-2004 Fortran 90
!        version  1.0  date 14-10-2002
!
!   copyright (c) 2002-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the number of nodes along the curve icurnr
!     Use array icurvsmsh and if is a compound curve also kmeshq
!     Composite curves must be contiguous in this case
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
!     compound_curve
!     number_of_nodes
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: icurnr, numnodes

!     icurnr         i    Curve sequence number
!     numnodes       o    Number of nodal points ialong curve
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nsubcv

!     nsubcv         Number of subcurves of a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     compute number of nodes as stored in icurvsmsh
!     Correct if the curve is a compound curve by subtracting the
!     number of double nodes
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshnumnodcrv' )

!     --- compute number of nodes as stored in icurvsmsh

      if ( icurnr==1 ) then
         numnodes = icurvsmsh(1)
      else
         numnodes = icurvsmsh(icurnr)-icurvsmsh(icurnr-1)
      end if

!     --- Correct if the curve is a compound curve by subtracting the
!         number of double nodes
!         This is only valid or contiguous curves

      if ( kelmq/=0 ) then

!     --- There are compound curves, correct number of nodes along curve

         nsubcv = max(1,kmeshq(icurnr+1)-kmeshq(icurnr))
         numnodes = numnodes+1-nsubcv

      end if
      call erclos ( 'mshnumnodcrv' )
      end
