      subroutine mshobstboun2d ( maploc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.7  date 15-11-2012 Long integers
!        version  1.6  date 20-07-2011 New call to plotobstacle
!        version  1.5  date 23-08-2008 activate coor
!
!   copyright (c) 2001-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if there are elements that have all nodes on the obstacle or on the
!     obstacle and an outer boundary
!     If this is the case subdivide these elements into three elements
!     by adding an extra point in the centroid
!     This subroutine can only be applied for linear triangles
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     2d
!     curve
!     element
!     obstacle
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmodulemesh
      use sepmoduledebug
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (mapping) :: maploc

!     maploc        i/o   Mapping array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iobst, lencurvs, lenin, lenkmeshc, npointsav, &
              lenrl, mmipointobst, mmnmeshc, mmnmeshd, &
              mmnmeshh, npointnew, numbounelms, &
              krenum(1), lenielemobst, nmap, npointmap, &
              mmioutercurvs, nmapmax, numobstelms
      integer, allocatable :: ielemobstlc(:), iworkedge(:), &
                              iworkshape(:), kmeshcnew(:), iwork(:), &
                              ibounelems(:)
      double precision, allocatable :: coornew(:,:)

!     coornew        Array containing the new coordinates of the nodes
!     ibounelems     Array of length numbounelms (<nelem) containing
!                    the element numbers of the elements that have all
!                    points on the boundary or on the obstacle
!     ielemobstlc      Array containing information about the elements in an
!                    obstacle
!                    The first numobst positions contain the starting
!                    positions of each obstacle sequentially
!                    Per obstacle the first three positions contain
!                    1. The number of elements that are completely inside
!                       the obstacle
!                    2. The number of elements that are not completely
!                       inside the element but with all vertices inside
!                       the element
!                    3. The number of elements that are partly inside
!                       the obstacle
!                       elements that contain only boundary points of the
!                       obstacle are not considered
!                    These positions are followed by the element numbers
!                    in the sequence defined by the first three positions
!     iobst          Obstacle sequence number
!     iwork          integer work array of length npoint
!                    iwork(i) is filled as follows (see pseudo code)
!                     0 for all points outside the obstacle and
!                       the boundaries
!                    -1 for all points inside the obstacle
!                     i for all points on the boundary of the obstacle i
!                    1001 for all points on the outer curves
!                         but not on an obstacle
!                    1002 for all points on the outer curves
!                         and on an obstacle
!     iworkedge      work array of length nedges to be used to mark
!                    edges
!                     0:  not marked yet
!                    -1:  boundary edge that has been used already
!                    -2:  diagonal of quadrilateral that may be removed
!     iworkshape     Work array of length nelem that is filled as follows
!                    iworkshape(ielem) contains the shape number of the
!                    element ielem
!     kmeshcnew      New kmesh part c
!     krenum         Parameter jrenum for TOPOL
!     lencurvs       Overestimate of the number of points along the new
!                    curves
!     lenielemobst   Computed length of array ielemobst
!     lenin          Number of integers in one row of kmeshx
!     lenkmeshc      length of array kmeshc
!     lenrl          Number of reals in one row of rmeshx
!     mmioutercurvs  memory management sequence number of array ioutercurvs
!     mmipointobst   memory management sequence number of array ipointobst
!     mmnmeshc       memory management sequence number of array nmeshc
!     mmnmeshd       memory management sequence number of array nmeshd
!     mmnmeshh       memory management sequence number of array nmeshh
!     nmap           length of second parameter of imap and rmap
!     nmapmax        Actual value of nmap
!     npointmap      Help variable to store the old value of npointnew
!     npointnew      Sequence number of last generated node
!     npointsav      Help parameter to save the old value of npoint
!     numbounelms    Number of elements that have all points on the
!                    boundary of the obstacle or a set of curves
!     numobstelms    number of elements that have all points on the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICOPINT2D    Copy two-dimensional integer array
!     INICOPREAL2D   Copy two-dimensional real array
!     MSHDELSEL      Delete a selected set of subarrays of kmesh
!     MSHOBSTBOUN2D1 Find all elements that have all nodes on the obstacle or
!                    on the obstacle and an outer boundary
!     MSHOBSTBOUN2D2 Create new points in mesh by subdividing some edges of
!                    elements that have all points on the boundary of the mesh
!                    and/or the obstacle
!     MSHOBSTBOUN2D3 create new elements in case new nodes have been made
!                    in combination with elements with all nodes on boundary
!                    and obstacle
!     MSHOBSTBOUN2D5 Create new points in mesh by subdividing some edges of
!                    elements that have all points on the boundary of obstacle
!     MSHOBSTBOUN2D6 Create new points in mesh by subdividing some edges of
!                    elements that have all points on the boundary of the mesh
!                    and/or the obstacle
!     MSHOBSTMARKELN Mark all elements in the mesh with information if they are
!                    within an obstacle or outside an obstacle
!     PLOTOBSTACLE   Print and plot information about the obstacle
!     PRINMAP1       Actual printing of array imap
!     SEPCREATEIBUF  Create permanent space in array IBUFFR
!     SEPCREATERBUF  Create permanent space in array BUFFER
!     SEPFILLKMESHC  Make relation between kmeshc and Topgr
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPFILLNMESHH  Fill array nmesh part h
!     SEPFILLOUTERB  Fill outer boundary of mesh
!     SEPFLPOINTOBST Fill array kmesh ad  part 1: ipointobst
!     TOPOL          Fill integer information corresponding to the mesh in KMESH
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2292   More than one obstacle
! **********************************************************************
!
!                       PSEUDO CODE
!
!     numbounelms := number of elements that have all nodes on the boundary
!     if ( numbounelms>0 ) then
!         Create new points and new elements
!         Adapt arrays ipointobst and ielemobst
!         Adapt arrays imap and rmap
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstboun2d' )
      debug = debug_mshobstmesh

!     --- numbounelms := number of elements that have all nodes on the boundary

      if ( numobst>1000 ) then

!     --- numobst>1000

         call errint ( numobst, 1 )
         call errsub ( 2292, 1, 0, 0 )
         go to 1000

      end if

!     --- Create or activate array ipointobst

      call sepflpointobst

!     --- Create or activate array outercurvs containing the outer curves

      if ( mmouter==0 ) call sepfillouterb

!     --- Next create the semi-permanent arrays iwork and ibounelems
!         these are deleted later on

      allocate ( iwork(npoint), ibounelems(3+nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint+3+nelem, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Find all elements that have all nodes on the obstacle or on the
!         obstacle and an outer boundary

      call mshobstboun2d1 ( iwork, numbounelms, ibounelems )

      if ( numbounelms>0 ) then

!     --- numbounelms>0
!         Create new points if necessary
!         First initialize some arrays and parameters

!        --- Activate or create some arrays

         if ( nmesh(33)==0 ) call sepfillnmeshc
         if ( nmesh(38)==0 ) call sepfillnmeshh

!        --- Over-estimate new number of points

         npointnew = npoint+nedges
         npointmap = npointnew
         nmap = 4

!        --- Create space for the coordinates array
!            Also for new arrays imap, rmap, iworkedge and iworkshape

         allocate ( coornew(2,npointnew), rmap2(npointnew,nmap), &
                    imap2(npointnew,nmap+1), iworkedge(nedges), &
                    iworkshape(nelem), stat = error )
         if ( error/=0 ) &
            call eralloc ( error, (2+nmap)*npointnew+nelem+nedges, 'coornew' )
         if ( ierror/=0 ) go to 1000

!        --- Clear arrays imap2 and rmap2

         imap2 = 0
         rmap2 = 0d0
         imap => maploc%imap
         rmap => maploc%rmap

!        --- initialize some arrays and parameters

         call mshobstboun2d2 ( iworkedge, npointnew, iworkshape )

         numobstelms = ibounelems(1)  ! number of elements on one
                                             ! obstacle

         if ( numobstelms>0 ) then

!        --- numobstelms>0
!            Create new points in mesh by subdividing some edges of
!            elements that have all points on the boundary of an obstacle
!            Activate some arrays

            lenin = 6
            lenrl = 6

!           --- Loop over obstacles

            nmapmax = 2
            do iobst = 1, numobst

!           --- Extract obstacle specific quantities

               nx = kmeshx(3,iobst)
               ny = kmeshx(4,iobst)

!              --- Set starting addresses

               lencurvs = size ( curvesmsh )
               iobgrid2d => kmobst(iobst)%iobgrid2d
               iobgrid2d1 => kmobst(iobst)%iobgrid2d1

!              --- Create new points in mesh by subdividing some edges of
!                  elements that have all points on the boundary of obstacle
!                  iobst

               call mshobstboun2d5 ( ibounelems, iworkedge, npointnew, &
                                     coornew, nx, ny,iwork, nmap, &
                                     lencurvs, iobst, nmapmax, npointmap )

            end do  ! iobst = 1, numobst

         else

!        --- numobstelms = 0, set nmapmax

            nmapmax = 2

         end if  ! ( numobstelms>0 )

!        --- Create new elements and points by subdividing elements
!            that have all nodes on the boundary or the boundary and various
!            obstacles or on different obstacles

         call mshobstboun2d6 ( ibounelems, iworkedge, npointnew, iworkshape, &
                               nmeshc, nmeshd, nmeshh, coor, coornew, &
                               iwork, npoint, nmap, nmapmax, npointmap )

         if ( ierror/=0 ) go to 1000

         if ( npointnew>npoint ) then

!        --- npointnew>npoint, create new elements
!            Adapt arrays kmeshc and ielemobst
!            Adapt arrays imap and rmap
!            First copy coor to coornew

            coornew(:,1:npoint) = coor(:,1:npoint)

!           --- Make new arrays imap and rmap permanent

            deallocate ( maploc%imap, maploc%rmap )
            allocate ( maploc%imap(npointnew,nmap+1), &
                       maploc%rmap(npointnew,nmap) )
            maploc%imap = imap2(1:npointnew,1:nmap+1)
            maploc%rmap = rmap2(1:npointnew,1:nmap)

!           --- Next replace coor by coornew and delete coornew

            if ( km(iseqkm)%kmesh(23)>0 ) deallocate(km(iseqkm)%coor)
            allocate(km(iseqkm)%coor(2,npointnew))
            coor => km(iseqkm)%coor
            coor = coornew(:,1:npointnew)
            km(iseqkm)%kmesh(23) = npointnew
            km(iseqkm)%kmesh(8) = npointnew

!           --- Create new kmeshc at the end of ibuffr

            lenkmeshc = size ( kmeshc )+(npointnew-npoint)*12

            allocate ( kmeshcnew(lenkmeshc), stat = error )
            if ( error/=0 ) call eralloc ( error, lenkmeshc, 'kmeshcnew' )
            if ( ierror/=0 ) go to 1000
            npointsav = npoint
            npoint = npointnew

!           --- create new elements

            call mshobstboun2d3 ( kmeshc, kmeshcnew, iworkedge, lenkmeshc )
            km(iseqkm)%kmesh(9) = nelem
            if ( ierror/=0 ) go to 1000

!           --- Next replace kmeshc by kmeshcnew

            if ( km(iseqkm)%kmesh(17)>0 ) deallocate(km(iseqkm)%kmeshc)
            allocate(km(iseqkm)%kmeshc(lenkmeshc))
            kmeshc => km(iseqkm)%kmeshc
            kmeshc = kmeshcnew(1:lenkmeshc)
            km(iseqkm)%kmesh(17) = lenkmeshc
            call sepfillkmeshc

            deallocate ( kmeshcnew, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'kmeshcnew' )
            if ( ierror/=0 ) go to 1000

!           --- Recompute topology after removing some subarrays

            call mshdelsel ( kmesh )

!           --- Mark elements that are on the obstacle

            lenielemobst = 6*numobst+nelem

            if ( associated(ielemobst) ) nullify(ielemobst)
            allocate ( ielemobst(lenielemobst), stat = error )
            if ( error/=0 ) call eralloc ( error, lenielemobst, 'ielemobst' )
            if ( ierror/=0 ) go to 1000

!           --- Fill array

            call mshobstmarkeln ( ipointobst, &
                                  ielemobst, kmeshc, lenielemobst )
            npoint = npointsav
            km(iseqkm)%kmesh(8) = npoint

!           --- Make array permanent

            kelmad = km(iseqkm)%kmesh(52)
            if ( associated(km(iseqkm)%ielemobst) ) &
               deallocate(km(iseqkm)%ielemobst)
            allocate ( km(iseqkm)%ielemobst(lenielemobst) )
            km(iseqkm)%ielemobst = ielemobst(1:lenielemobst)

            deallocate ( ielemobst, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'ielemobst' )
            if ( ierror/=0 ) go to 1000
            ielemobst => km(iseqkm)%ielemobst

            if ( debug .or. plotobst .or. printobst ) then

!           --- debug = true, plot nodes and elements with a colour
!               First print arrays

               call plotobstacle ( debug_mshobstmesh, 0 )

               if ( debug ) then

!              --- debug true, print map

                  call prinmap1 ( npointnew )

               end if

            end if

            call mshdelsel ( kmesh )
            krenum(1) = 9
            call topol ( krenum(1) )

         end if

         deallocate ( coornew, rmap2, imap2, iworkedge, &
                      iworkshape, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'coornew' )
         if ( ierror/=0 ) go to 1000

      end if

!     --- Delete semi-permanent arrays

      deallocate ( iwork, ibounelems, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )
      if ( ierror/=0 ) go to 1000

1000  call erclos ( 'mshobstboun2d' )

      end subroutine mshobstboun2d
