      subroutine mshobstboun2d3 ( kmeshcold, kmeshcnew, &
                                  iworkedge, lenkmeshc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 09-03-2013 Check on kmeshg
!        version  1.0  date 31-10-2001
!
!   copyright (c) 2001-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     create new elements in case new nodes have been made
!     Store new topology in kmeshc and kmeshg
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     subdivision
!     subdivision
!     triangle
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshcold(*), kmeshcnew(*), iworkedge(*),lenkmeshc

!     iworkedge      i    work array of length nedges, containing information
!                         whether an edge must be subdivided into two parts (>0)
!                         or not (0)
!                         if iworkedge(iedge)>0, the value refers to the node
!                         number of the new node on the edge
!     kmeshcnew      o    New kmesh part c
!     kmeshcold      i    Old kmesh part c
!     lenkmeshc     i/o   length of array kmeshcnew
!                         At input declared length
!                         At output computed length
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, ielgrp, ikelmcnew, ikelmcold, irelem, iextra, &
              ielem, i, iedge, istartedge, j, nedge_elem, nelemloc
      logical changed

!     changed        If true an issue has been changed
!     i              Counting variable
!     iedge          edge sequence number
!     ielem          Element number
!     ielgrp         element group number
!     iextra         Extra number of elements created
!     ikelmcnew      Starting address of next element in kmeshcnew
!     ikelmcold      Starting address of next element in kmeshcold
!     inpelm         number of points per element
!     irelem         Relative element sequence number with respect to
!                    element group
!     istartedge     Starting position of edge in array NMESHD
!     j              Counting variable
!     nedge_elem     number of edges corresponding to element
!     nelemloc       Number of new elements in the original one
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHOBSTBOUN2D4 Subdivide triangle into 2, 3 or 4 subtriangles, depending
!                    on the  number of edges with extra points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all element groups ielgrp do
!        For all elements ielem in ielgrp do
!           If ( ielem # triangle ) then
!              Copy old kmeshc to new kmeshc
!           else If ( all edges are unchanged ) then
!              Copy old kmeshc to new kmeshc
!           else ( element is triangle and at least one edge is changed ) then
!              Create new elements
!        Adapt kmeshg
!     Adapt kmesh(9) (nelem)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstboun2d3' )

      ikelmcold = 0
      ikelmcnew = 0
      ielem = 0
      nelem = 0

!     --- For all element groups ielgrp do

      do ielgrp = 1, nelgrp

         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm
         ishape = kmeshf(ielgrp)
         iextra = 0
         if ( ishape/=3 ) then

!        --- ishape # 3,  Copy old kmeshc into new kmeshc

            do i = 1, inelem*inpelm
               kmeshcnew(i+ikelmcnew) = kmeshcold(i+ikelmcold)
            end do
            ikelmcnew = ikelmcnew+inelem*inpelm
            ikelmcold = ikelmcold+inelem*inpelm
            ielem = ielem+inelem

         else

!        --- For all elements ielem in ielgrp do

            do irelem = 1, inelem

               ielem = ielem+1
               istartedge = nmeshd(ielem)
               nedge_elem = nmeshd(istartedge)

!              --- For all edges iedge corresponding to ielem do

               changed = .false.
               do j = 1, nedge_elem
                  iedge = abs(nmeshd(istartedge+j))
                  if ( iworkedge(iedge)>0 ) then

!                 --- edge is splitted into two parts

                     changed = .true.
                     go to 100

                  end if

               end do

100            if ( .not. changed ) then

!              --- all edges are unchanged
!                  Copy old kmeshc to new kmeshc

                  do i = 1, inpelm
                     kmeshcnew(i+ikelmcnew) = kmeshcold(i+ikelmcold)
                  end do
                  ikelmcnew = ikelmcnew+inpelm
                  ikelmcold = ikelmcold+inpelm

               else

!              --- element is triangle and at least one edge is changed
!                  Create new elements

                  call mshobstboun2d4 ( iworkedge, &
                                        kmeshcold(1+ikelmcold), &
                                        kmeshcnew(1+ikelmcnew), &
                                        nmeshd(istartedge+1), &
                                        nelemloc, coor )
                  ikelmcnew = ikelmcnew+inpelm*nelemloc
                  ikelmcold = ikelmcold+inpelm
                  iextra = iextra+nelemloc-1

               end if
               if ( kelmg>0 ) kmeshg(ielgrp) = inelem+iextra

            end do

         end if
         nelem = nelem + inelem+iextra
         if ( ikelmcnew>lenkmeshc ) then

!        --- Length too short

            call errint ( ikelmcnew, 1 )
            call errint ( lenkmeshc, 2 )
            call errchr ( 'kmeshcnew', 1 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         end if

      end do
      lenkmeshc = ikelmcnew

1000  call erclos ( 'mshobstboun2d3' )
      end
