      subroutine mshobstboun3d3 ( kmeshcnew, iworkedge, lenkmeshc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 09-03-2013 Check on kmeshg
!        version  1.3  date 19-05-2005 New calls to subroutines
!        version  1.2  date 10-11-2004 Replace error message
!
!   copyright (c) 2002-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     create new elements in case new nodes have been made
!     Store new topology in kmeshc and kmeshg
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     subdivision
!     subdivision
!     tetrahedron
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshcnew(*), iworkedge(*), lenkmeshc

!     iworkedge      i    work array of length nedges, containing information
!                         whether an edge must be subdivided into two parts (>0)
!                         or not (0)
!                         if iworkedge(iedge)>0, the value refers to the node
!                         number of the new node on the edge
!     kmeshcnew      o    New kmesh part c
!     lenkmeshc     i/o   length of array kmeshcnew
!                         At input declared length
!                         At output computed length
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, ielgrp, ikelmcnew, ikelmcold, irelem, iextra, &
              ielem, i, iedge, istartedge, j, nedge_elem, &
              nelemnew, edgenode(3,6), ipstart, nextranodes

!     edgenode       Array with all nodes on the crossed edge
!                    edgenode(1,1) first node on crossed edge
!                    edgenode(2,1) last node on crossed edge
!                    edgenode(3,1) new node on crossed edge
!     i              Counting variable
!     iedge          edge sequence number
!     ielem          Element number
!     ielgrp         element group number
!     iextra         Extra number of elements created
!     ikelmcnew      Starting address of next element in kmeshcnew
!     ikelmcold      Starting address of next element in kmeshc
!     inpelm         number of points per element
!     ipstart        Starting address in array
!     irelem         Relative element sequence number with respect to
!                    element group
!     istartedge     Starting position of edge in array NMESHD
!     j              Counting variable
!     nedge_elem     number of edges corresponding to element
!     nelemnew       Number of new elements in the original one
!     nextranodes    Number of extra nodes in element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     MSHOBSTMS3D01  Subdivide each tetrahedron into the right number of
!                    subtetrahedrons
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstboun3d3' )

      ikelmcold = 0
      ikelmcnew = 0
      ielem = 0
      nelem = 0

!     --- For all element groups ielgrp do

      do ielgrp = 1, nelgrp

         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm
         ishape = kmeshf(ielgrp)
         iextra = 0
         if ( ishape/=11 ) then

!        --- ishape # 11,  Copy old kmeshc into new kmeshc

            do i = 1, inelem*inpelm
               kmeshcnew(i+ikelmcnew) = kmeshc(i+ikelmcold)
            end do
            ikelmcnew = ikelmcnew+inelem*inpelm
            ikelmcold = ikelmcold+inelem*inpelm
            ielem = ielem+inelem

         else

!        --- For all elements ielem in ielgrp do

            do irelem = 1, inelem

               ielem = ielem+1
               istartedge = nmeshd(ielem)
               nedge_elem = nmeshd(istartedge)

!              --- For all edges iedge corresponding to ielem do

               nextranodes = 0
               do j = 1, nedge_elem
                  iedge = abs(nmeshd(istartedge+j))
                  if ( iworkedge(iedge)>0 ) then

!                 --- edge is splitted into two parts

                     nextranodes = nextranodes+1
                     ipstart = nmeshc(iedge)
                     edgenode(1,nextranodes) = nmeshc(ipstart+1)
                     edgenode(2,nextranodes) = nmeshc(ipstart+2)
                     edgenode(3,nextranodes) = iworkedge(iedge)

                  end if

               end do
               call mshobstms3d01 ( kmeshc(1+ikelmcold), &
                                    kmeshcnew(1+ikelmcnew), coor, &
                                    nextranodes, nelemnew, edgenode, &
                                    inpelm )

               ikelmcnew = ikelmcnew+inpelm*nelemnew
               ikelmcold = ikelmcold+inpelm
               iextra = iextra+nelemnew-1

               if ( kelmg>0 ) kmeshg(ielgrp) = inelem+iextra

            end do  ! irelem = 1, inelem

         end if
         nelem = nelem + inelem+iextra

!        --- Check array length

         call inichkarlen ( ikelmcnew, lenkmeshc, 'kmeshcnew' )
         if ( ierror/=0 ) go to 1000

      end do
      lenkmeshc = ikelmcnew

1000  call erclos ( 'mshobstboun3d3' )
      end
