      subroutine mshobstfaces3d ( nfaces, obstfaces, elemobst, nmeshf, &
                                  lenobstfaces )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-01-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Find all faces corresponding to one obstacle and store in array obstfaces
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     face
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nfaces, obstfaces(*), elemobst(*), nmeshf(*), lenobstfaces

!     elemobst       i    Array of length nelem containing the following
!                         information
!                         elemobst(i) = 1: element i is external
!                         elemobst(i) = 2: element i intersects the obstacle
!                         elemobst(i) = 3: element i is internal
!     lenobstfaces  i/o   Length of array obstfaces
!                         At input: estimated length
!                         At output: computed length
!     nfaces         i    Number of faces
!     nmeshf         i    nmesh part f
!     obstfaces      o    Array containing the face or edge numbers
!                         corresponding to the boundary of the obstacle
!                         Storage:
!                         Pos. 1 to numobst: starting addresses of obstacle
!                         Per obstacle:
!                         number of faces, followed by face numbers
!                         In this subroutine only part corresponding to one
!                         obstacle is used, i.e. without starting addresses
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nfaceobst, iface, ipstart, nface_elm, ielem, ielem1, &
              ielem2
      logical obstfacefound

!     ielem          Element number
!     ielem1         First element number in surface
!     ielem2         Last element number in surface
!     iface          Face sequence number
!     ipstart        Starting address in array
!     nface_elm      Number of faces per element
!     nfaceobst      Number of boundary faces of obstacle
!     obstfacefound  If true the face is on an obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     nfaceobst := 0
!     For all faces iface do
!        If ( iface corresponds to 1 element ielem ) then
!           if ( elemobst(ielem) = 3 ) then
!              nfaceobst := nfaceobst+1
!              obstfaces(nfaceobst+1) := iface
!        else ( iface corresponds to 2 elements ielem1/2 ) then
!           if ( elemobst(ielem1) = 3 and  elemobst(ielem2) < 3 or
!                elemobst(ielem1) < 3 and  elemobst(ielem2) = 3 ) then
!              nfaceobst := nfaceobst+1
!              obstfaces(nfaceobst+1) := iface
!     obstfaces(1) := nfaceobst
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstfaces3d' )

!     --- nfaceobst := 0

      nfaceobst = 0

!     --- For all faces iface do

      do iface = 1, nfaces
         ipstart = nmeshf(iface)
         nface_elm = nmeshf(ipstart)
         obstfacefound = .false.

!        --- If ( iface corresponds to 1 element ielem ) then
!               if ( elemobst(ielem) = 3 ) then
!                  nfaceobst := nfaceobst+1
!                  obstfaces(nfaceobst+1) := iface
!            else ( iface corresponds to 2 elements ielem1/2 ) then
!               if ( elemobst(ielem1) = 3 and  elemobst(ielem2) < 3 or
!                    elemobst(ielem1) < 3 and  elemobst(ielem2) = 3 ) then
!                  nfaceobst := nfaceobst+1
!                  obstfaces(nfaceobst+1) := iface

         if ( nface_elm.eq.1 ) then

!        --- Boundary face

            ielem = nmeshf(ipstart+1)
            obstfacefound = elemobst(ielem).eq.3

         else

!        --- Internal face

            ielem1 = nmeshf(ipstart+1)
            ielem2 = nmeshf(ipstart+2)
            obstfacefound = elemobst(ielem1).eq.3 .and. &
                            elemobst(ielem2).lt.3 .or. &
                            elemobst(ielem1).lt.3 .and. &
                            elemobst(ielem2).eq.3

         end if

         if ( obstfacefound ) then

!        --- obstacle face has been found

            nfaceobst = nfaceobst+1
            if ( nfaceobst.ge.lenobstfaces ) then

!           --- Array obstfaces too small

               call errint ( nfaceobst+1, 1 )
               call errint ( lenobstfaces, 2 )
               call errchr ( 'obstfaces', 1 )
               call errsub ( 550, 2, 0, 1 )
               go to 1000

            end if
            obstfaces(nfaceobst+1) = iface

         end if

      end do

!     --- obstfaces(1) := nfaceobst

      obstfaces(1) = nfaceobst
      lenobstfaces = nfaceobst+1

1000  call erclos ( 'mshobstfaces3d' )

      end
