      subroutine mshobstms3d05a ( tetnode, tetnodenew, edgenode, &
                                  isort, ledgenode, nelemnew )
! ======================================================================
!
!        programmer    Koos Meijerink
!        version  2.2  date 19-05-2005 Reset of edgenode
!        version  2.1  date 07-04-2004 Extension of edgenode (RvL)
!        version  2.0  date 21-01-2002 Remove coornew
!        version  1.0  date 11-07-2001
!
!   copyright (c) 2001-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide tetrahedron, with only two crossed edges which share a node, &
!     into three tetrahedrons.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     linear
!     tetrahedron
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer tetnode(4), tetnodenew(12), edgenode(3,3), isort(4), &
              ledgenode(2,3), nelemnew

!     edgenode       i    Array with all nodes on the crossed edge
!                         edgenode(1,1) first node on crossed edge
!                         edgenode(2,1) last node on crossed edge
!                         edgenode(3,1) new node on crossed edge
!     isort          i    if isort(i)>isort(j) then tetnode(i) is at least on
!                         the same number of crossed edges as tetnode(j)
!     ledgenode      i    for i=1,2 and 1<j< number of crossed edges
!                         ledgenode(i,j) is the local tetnode of edgenode(i,j), &
!                         in other words:
!                         edgnode(i,j) = tetnode(ledgenode(i,j))
!     nelemnew       i    Number of elements in new mesh
!     tetnode        i    Array of nodal point numbers of the corners of
!                         the original tetrahedron.
!     tetnodenew     o    Array of nodal point numbers of the corners of
!                         the new tetrahedrons.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i,j, &
              trans(4), transi(4), transtemp, transposj, &
              tetnodet(4), edgenodet(6)
      integer pedgenodet(4,4)
      logical oddpairintchan

!     edgenodet      Array of global nodal point numbers of X-point on edge i
!                    after transformation
!     i              Counting variable
!     j              Counting variable
!     oddpairintchan if true the transformation is an odd number of
!                    pair interchanges
!     pedgenodet     the local edge number = local edgenode number
!                    for the edge [local tetnode i, local tetnode j]
!     tetnodet       Array of global nodal point numbers of the corners of
!                    the transformed original tetrahedron
!     trans          Transformation array. trans(i) is the transformed
!                    local position of the i-th corner of the original
!                    tetrahedron.
!     transi         Inverse of trans
!     transposj      The transformed local edge node number of edge node j
!     transtemp      temporary storage
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!                   the new tetrahedrons
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Define transformation(= rotation) to move node isort(4) to node 1
!                                           and node isort(1) to node 4.
!
!     Transform tetrahedron tetnode to tetnodet and the position of the
!     intersection points edgenode(3,.) to edgenodet(.)
!
!     Split tetrahedron into subtetrahedrons.
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data pedgenodet /0,1,3,4, &
                       1,0,2,5, &
                       3,2,0,6, &
                       4,5,6,0/
! ======================================================================
!
      call eropen ( 'mshobstms3d05a' )

!     --- Define transformation to move node isort(4) to node 1
!                                and node isort(1) to node 4
!         Initialize

      oddpairintchan = .false.
      do i = 1,4
         trans(i) = i
      end do

!     --- move node isort(4) to 1

      if( isort(4)/=trans(1) ) then
         transtemp = trans(1)
         trans(1) = isort(4)
         trans(isort(4)) = transtemp
         oddpairintchan = .not.oddpairintchan
      end if

!     --- move node isort(1) to 2

      if (isort(1)/=trans(4) ) then
         transtemp = trans(4)
         trans(4) = isort(1)
         trans(trans(isort(1))) = transtemp
         oddpairintchan = .not.oddpairintchan
      end if

!     --- make pairinterchanges even

      if( oddpairintchan ) then
         transtemp = trans(2)
         trans(2) = trans(3)
         trans(3) = transtemp
      end if

!     --- transi := inverse of trans

      do i = 1,4
         transi(trans(i)) = i
      end do

!     --- Transform tetrahedron tetnode to tetnodet

      do i = 1,4
         tetnodet(i) = tetnode(trans(i))
      end do

!     --- Transform the intersection points to their transformed positions

      do j = 1,2
         transposj = pedgenodet( &
                       transi(ledgenode(1,j)),transi(ledgenode(2,j)))
         edgenodet(transposj) = edgenode(3,j)
      end do

      tetnodenew(1) = tetnodet(1)
      tetnodenew(2) = edgenodet(1)
      tetnodenew(3) = edgenodet(3)
      tetnodenew(4) = tetnodet(4)

      if( edgenodet(1)<edgenodet(3) ) then
         tetnodenew(5) = edgenodet(1)
         tetnodenew(6) = tetnodet(2)
         tetnodenew(7) = tetnodet(3)
         tetnodenew(8) = tetnodet(4)

         tetnodenew(9)  = edgenodet(1)
         tetnodenew(10) = tetnodet(3)
         tetnodenew(11) = edgenodet(3)
         tetnodenew(12) = tetnodet(4)
      else
         tetnodenew(5) = edgenodet(1)
         tetnodenew(6) = tetnodet(2)
         tetnodenew(7) = edgenodet(3)
         tetnodenew(8) = tetnodet(4)

         tetnodenew(9)  = tetnodet(2)
         tetnodenew(10) = tetnodet(3)
         tetnodenew(11) = edgenodet(3)
         tetnodenew(12) = tetnodet(4)
      end if

      nelemnew = 3

      call erclos ( 'mshobstms3d05a' )

      end
