      subroutine mshobstms3d06c ( tetnode, tetnodenew, edgenode, &
                                  isort, ledgenode, nelemnew, &
                                  nedgenode )
! ======================================================================
!
!        programmer    Koos Meijerink
!        version  3.0  date 19-05-2005 Extra parameter nedegenode
!        version  2.0  date 21-01-2002 Remove coornew
!        version  1.0  date 11-07-2001
!
!   copyright (c) 2001-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide a tetrahedron, with three crossed edges which form a curve
!     that is not closed, into four or five tetrahedrons.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     linear
!     tetrahedron
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer tetnode(4),tetnodenew(20), nedgenode, &
              edgenode(nedgenode,3), isort(4), &
              ledgenode(2,3), nelemnew

!     edgenode       i    Array with all nodes on the crossed edge
!                         edgenode(1,1) first node on crossed edge
!                         edgenode(2,1) last node on crossed edge
!                         edgenode(3,1) new node on crossed edge
!     isort          i    if isort(i)>isort(j) then tetnode(i) is at least on
!                         the same number of crossed edges as tetnode(j)
!     ledgenode      i    for i=1,2 and 1<j< number of crossed edges
!                         ledgenode(i,j) is the local tetnode of edgenode(i,j), &
!                         in other words:
!                         edgnode(i,j) = tetnode(ledgenode(i,j))
!     nedgenode      i    Length of first index of edgenode
!     nelemnew       o    Number of elements in new mesh
!     tetnode        i    Array of nodal point numbers of the corners of
!                         the original tetrahedron.
!     tetnodenew     o    Array of nodal point numbers of the corners of
!                         the new tetrahedrons.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, &
              trans(4), transi(4), transtemp, transposj, switch, &
              tetnodet(4), edgenodet(6), &
              tetnodettemp, edgenodettemp
      integer pedgenodet(4,4)
      logical oddpairintchan, edgenodetpos2

!     edgenodet      Array of global nodal point numbers of X-point on edge i
!                    after transformation
!     edgenodetpos2  one of the transformed edge nodes is at local position 2
!     edgenodettemp  temporary storage for a edgenodet element
!     i              Counting variable
!     j              Counting variable
!     oddpairintchan if true the transformation is an odd number of
!                    pair interchanges
!     pedgenodet     the local edge number = local edgenode number
!                    for the edge [local tetnode i, local tetnode j]
!     switch         if oddpairinterchanges .true. then switch = 1
!                    else switch = 0
!     tetnodet       Array of global nodal point numbers of the corners of
!                    the transformed original tetrahedron
!     tetnodettemp   temporary storage for a tetnodet element
!     trans          Transformation array. trans(i) is the transformed
!                    local position of the i-th corner of the original
!                    tetrahedron.
!     transi         Inverse of trans
!     transposj      The transformed local edge node number of edge node j
!     transtemp      temporary storage
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Define transformation to move node isort(3) to node 1
!                               and node isort(4) to node 2.
!
!     Transform tetrahedron tetnode to tetnodet and the position of the
!     intersection points edgenode(3,.) to edgenodet(.)
!
!     If intersection points at positions 1,3,5 transform tetrahedron such
!     that they get into positions 1,2,4
!
!     If node number of intersection point 4<node number of i.p. 2, &
!     interchange these 2
!
!     Split tetrahedron into four or five subtetrahedrons, taking into account
!     the determinant of the transformation and the node numbers of the
!     intersection points.
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data pedgenodet /0,1,3,4, &
                       1,0,2,5, &
                       3,2,0,6, &
                       4,5,6,0/
! ======================================================================
!
      call eropen ( 'mshobstms3d06c' )

!     --- Define transformation to move node isort(3) to node 1
!                                      and node isort(4) to node 2
!         -- Initialize

      oddpairintchan = .false.
      do i = 1,4
         trans(i) = i
      end do

!     --- move node isort(3) to node 1

      if( isort(3)/=trans(1) ) then
         transtemp = trans(1)
         trans(1) = isort(3)
         trans(isort(3)) = transtemp
         oddpairintchan = .not.oddpairintchan

       end if

!      --- move node isort(4) to 2

      if (isort(4)/=trans(2) ) then
         transtemp = trans(2)
         trans(2) = isort(4)
         trans(trans(isort(4))) = transtemp
         oddpairintchan = .not.oddpairintchan

      end if

!     --- transi := inverse of trans

      do i = 1,4
         transi(trans(i)) = i
      end do

!     --- Transform tetrahedron tetnode to tetnodet

      do i = 1,4
         tetnodet(i) = tetnode(trans(i))
      end do

!     --- Transform the intersection points to their transformed positions

      edgenodetpos2 = .false.
      do j = 1,3
         transposj = pedgenodet(transi(ledgenode(1,j)), &
                                transi(ledgenode(2,j)))
         if(transposj==2) edgenodetpos2 = .true.
         edgenodet(transposj) = edgenode(3,j)
      end do

!     --- if no intersection point at position 2, the intersection
!         points are at positions 1,3,5. Interchange edgenode 3 and 4
!         and 4 so that the intersection points are at positions 1,2,4

      if( .not.edgenodetpos2 ) then
         edgenodet(4) = edgenodet(3)
         edgenodet(2) = edgenodet(5)
         tetnodettemp = tetnodet(3)
         tetnodet(3) = tetnodet(4)
         tetnodet(4) = tetnodettemp
         oddpairintchan = .not.oddpairintchan
      end if

      if( edgenodet(4)<edgenodet(2) ) then
         edgenodettemp = edgenodet(2)
         edgenodet(2) = edgenodet(4)
         edgenodet(4) = edgenodettemp
         tetnodettemp = tetnodet(1)
         tetnodet(1) = tetnodet(2)
         tetnodet(2) = tetnodettemp
         tetnodettemp = tetnodet(3)
         tetnodet(3) = tetnodet(4)
         tetnodet(4) = tetnodettemp
      end if

      if( oddpairintchan) then
         switch = 1
      else
         switch = 0
      end if

!     --- Divide the tetrahedron

      if( edgenodet(1)<edgenodet(2) ) then
         tetnodenew(2-switch) = edgenodet(1)
         tetnodenew(3) = tetnodet(3)
         tetnodenew(4) = edgenodet(4)
         tetnodenew(1+switch) = tetnodet(1)
         tetnodenew(5+switch) = edgenodet(1)
         tetnodenew(6-switch) = tetnodet(3)
         tetnodenew(7) = edgenodet(4)
         tetnodenew(8) = tetnodet(4)
         tetnodenew(10-switch) = edgenodet(1)
         tetnodenew(11) = tetnodet(3)
         tetnodenew(12) = edgenodet(2)
         tetnodenew(9+switch) = tetnodet(4)

         tetnodenew(13+switch) = edgenodet(1)
         tetnodenew(14-switch) = tetnodet(2)
         tetnodenew(15) = edgenodet(2)
         tetnodenew(16) = tetnodet(4)
         nelemnew = 4
      else if ( edgenodet(1)>edgenodet(4)  ) then
         tetnodenew(2-switch) = edgenodet(2)
         tetnodenew(3) = edgenodet(4)
         tetnodenew(4) = edgenodet(1)
         tetnodenew(1+switch) = tetnodet(1)
         tetnodenew(5+switch) = edgenodet(2)
         tetnodenew(6-switch) = edgenodet(4)
         tetnodenew(7) = edgenodet(1)
         tetnodenew(8) = tetnodet(2)
         tetnodenew(10-switch) = edgenodet(2)
         tetnodenew(11) = edgenodet(4)
         tetnodenew(12) = tetnodet(4)
         tetnodenew(9+switch) = tetnodet(2)
         tetnodenew(13+switch) = edgenodet(2)
         tetnodenew(14-switch) = edgenodet(4)
         tetnodenew(15) = tetnodet(4)
         tetnodenew(16) = tetnodet(3)
         tetnodenew(18-switch) = edgenodet(2)
         tetnodenew(19) = edgenodet(4)
         tetnodenew(20) = tetnodet(1)
         tetnodenew(17+switch) = tetnodet(3)
         nelemnew = 5
      else
         tetnodenew(1+switch) = edgenodet(2)
         tetnodenew(2-switch) = edgenodet(4)
         tetnodenew(3) = tetnodet(1)
         tetnodenew(4) = edgenodet(1)
         tetnodenew(6-switch) = edgenodet(2)
         tetnodenew(7) = edgenodet(4)
         tetnodenew(8) = tetnodet(1)
         tetnodenew(5+switch) = tetnodet(3)

         tetnodenew(9+switch) = edgenodet(2)
         tetnodenew(10-switch) = edgenodet(4)
         tetnodenew(11) = tetnodet(4)
         tetnodenew(12) = tetnodet(3)
         tetnodenew(14-switch) = edgenodet(2)
         tetnodenew(15) = edgenodet(4)
         tetnodenew(16) = tetnodet(4)
         tetnodenew(13+switch) = edgenodet(1)
         tetnodenew(17+switch) = edgenodet(2)
         tetnodenew(18-switch) = tetnodet(2)
         tetnodenew(19) = tetnodet(4)
         tetnodenew(20) = edgenodet(1)
         nelemnew = 5
      end if

      call erclos ( 'mshobstms3d06c' )

      end
