      recursive subroutine mshparncorr4 ( istore, istore1, iloc, &
                                          numfacesurf, inpelm, iseq, &
                                          list )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-04-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Sort nodes in istore in increasing sequence
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     subdivision
!     2d
!     3d
!     parallel
!     sort
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numfacesurf, inpelm, istore(inpelm,numfacesurf), &
              istore1(inpelm,numfacesurf), iseq(numfacesurf), iloc, &
              list(numfacesurf)

!     iloc           i    Sequence number of local node in element
!     inpelm         i    number of points per element
!     iseq                work array for sorting algorithm
!                         Contains sequence of sorted set
!     istore        i/o   Contains the part of the set of faces to be considered
!     istore1       i/o   Help array to store sorted array istore
!     list          i/o   List of element numbers in kmeshncopy with respect to
!                         kmeshn
!     numfacesurf    i    Number of faces in istore to be considered
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, iend, i, ih
      integer, allocatable :: ihelp(:)

!     error          Return value of allocate or deallocate
!     i              Counting variable
!     iend           Last position to be used in istore
!     ih             Help variable to store a constant temporarily
!     ihelp          integer work array to store sorted list
!     istart         Next position to be used in istore
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHSORT         Sort integer array for increasing sequence
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRININ1        print 2d integer array
!     PRININ2        Print an integer two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3072   Double faces found
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparncorr4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparncorr4'
         write(irefwr,1) 'iloc, numfacesurf, inpelm', &
                          iloc, numfacesurf, inpelm
         call prinin1 ( istore, numfacesurf, inpelm, 'istore' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( ihelp(numfacesurf), stat = error )
      if ( error/=0 ) call eralloc ( error, numfacesurf, 'ihelp' )

!     --- Sort set

      ihelp(1:numfacesurf) = istore(iloc,1:numfacesurf)
      call chsort ( ihelp, iseq, numfacesurf )

      istore1(1:inpelm,1:numfacesurf) = &
         istore(1:inpelm,iseq(1:numfacesurf))
      ihelp(1:numfacesurf) = list(iseq(1:numfacesurf))

      istore = istore1
      list = ihelp

      if ( debug ) &
         call prinin1 ( istore1, numfacesurf, inpelm, 'istore1' )

      istart = 1

!     --- Repeat till end

100      iend = istart
         ih = istore1(iloc,istart)
         if ( debug ) write(irefwr,1) 'istart, ih', istart, ih

         do i = istart+1, numfacesurf

            if ( istore1(iloc,i)/=ih ) go to 110

!           --- Same first node found

            iend = i

         end do  ! i = istart, numfacesurf

!        --- The nodes istart till iend are the same

110      if ( istart==iend ) then

!        --- Only one node

         else if ( iloc<inpelm ) then

!        --- More nodes, sort with respect to other nodes

            call mshparncorr4 ( istore(1,istart), istore1(1,istart), &
                                iloc+1, iend-istart+1, inpelm, iseq, &
                                list(istart) )

         else

!        --- iloc = inpelm, should not appear

            call errsub ( 3072, 0, 0, 0 )
            go to 1000

         end if  ! ( istart==iend )

         istart = iend+1
         if ( istart<=numfacesurf ) go to 100

      deallocate ( ihelp, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'ihelp' )

1000  call erclos ( 'mshparncorr4' )
      if ( debug ) then

!     --- Debug information

         call prinin2 ( istore, numfacesurf, inpelm, 'istore' )
         write(irefwr,*) 'End mshparncorr4'

      end if  ! ( debug )

      end
