      subroutine mshparnumproc ( iinput, nprocessors, maxmsh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 28-03-2013 debug statements
!        version  1.1  date 27-11-2003 Extra option methodpar = 4
!        version  1.0  date 03-02-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute number of parallel processors to be used and hence the number
!     of subparts the mesh is subdivided into
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), nprocessors, maxmsh

!     iinput         i    integer input array with information concerning the
!                         parallelization
!                         Contents
!                         Pos. 1 Type of subdivision
!                                0:  If the mesh contains volumes the number of
!                                    subparts is equal to the number of volumes
!                                    otherwise equal to the number of surfaces
!                                1:  The mesh is subdivided in layers
!                         Pos. 2 If iinput(1) # 0: number of processor
!     maxmsh         i    Maximum number of meshes
!     nprocessors    o    Number of processors that is to be used for
!                         parallel processing
!                         Defines also the number of submeshes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer methodpar

!     methodpar      Defines the type of subdivision of the mesh to be made
!                    for parallel computing
!                    Possible values:
!                    0:  If the mesh contains volumes the number of
!                        subparts is equal to the number of volumes
!                        otherwise equal to the number of surfaces
!                    1:  The mesh is subdivided in layers
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2147   Parameter out of range
!    2399   Number of processors too large
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshparnumproc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshparnumproc'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nprocessors = 0
      methodpar = iinput(1)

      if ( debug ) write(irefwr,1) 'methodpar', methodpar

      if ( methodpar==1 ) then

!     --- Old method, the number of processors is equal to the number of
!         volumes (3d) or surfaces (2d)

         if ( nvolms>0 ) then

!        --- nvolms > 0, number of processors is number of volumes

            nprocessors = nvolms

         else

!        --- nvolms = 0, number of processors is number of surfaces

            nprocessors = nsurfs

         end if

      else if ( methodpar>=2 .or. methodpar<=4 ) then

!     --- methodpar = 2 - 4: layers or blocks

         nprocessors = iinput(2)

      else

!     --- Other values of methodpar have not yet been implemented

         call errint ( methodpar, 1 )
         call errint ( 1, 2 )
         call errint ( 4, 3 )
         call errchr ( 'methodpar', 1 )
         call errsub ( 2147, 3, 0, 1 )

      end if

      if ( nprocessors>maxmsh ) then

!     --- Number of processors more than allowed

         call errint ( nprocessors, 1 )
         call errint ( maxmsh, 2 )
         call errsub ( 2399, 2, 0, 0 )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'nprocessors', nprocessors
         write(irefwr,*) 'End mshparnumproc'

      end if  ! ( debug )
      call erclos ( 'mshparnumproc' )

      end
