      subroutine mshpav2layers ( crd, ncrd, kmeshc, nel, nowbnd, nloop, &
                                 nbndpt, maxbounds, layer, lnnowbnd, &
                                 maxlen, markboun, bndcoornew, bndangle, &
                                 bndcoorold, zero )
! ======================================================================
!
!        programmer    Bas van Rens
!        version  1.1  date 07-07-2004 new call to plotmeshinter
!        version  1.0  date 27-11-2003
!
!   copyright (c) 2003-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     The paving algorithm called by mshpav
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncrd, kmeshc(3,*), maxbounds, nel, nowbnd(maxbounds,*), &
              nbndpt, layer, lnnowbnd, nloop, &
              markboun(lnnowbnd,nloop,*)
      double precision crd(4,*), maxlen, bndcoorold(2*lnnowbnd,*), &
                       bndcoornew(4,lnnowbnd,*), bndangle(lnnowbnd,*)
      logical zero(nloop)

!     bndangle      i/o   Array containing the angles corresponding to all nodes
!                         of a boundary loop
!     bndcoornew    i/o   Contains the coordinates of the new boundary created
!                         by making one layer during the paver algorithm
!     bndcoorold     i    Contains the coordinates of the previous (old)
!                         boundary during the paver process
!     crd           i/o   array containing the coordinates of the nodes (1-2,.)
!                         as well as the present factor (3,.) and the
!                         multiplication factor (4,.)
!     kmeshc        i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!     layer          i    Present layer sequence number, only used for
!                         debugging
!     lnnowbnd       i    Declared length of second index of array nowbnd
!     markboun       i    Three diemsional array of size lnnowbnd x nloop x 3
!                         with the following contents
!                         markboun(i,j,k) refers to relative node i along the
!                         j-th boundary in nowbnd
!                         k has the following meaning.
!                         if Pos. 1 = 0 then the node is free otherwise (1)
!                         it is clcoe to another node.
!                         In that case position 2 contains the boundary number
!                         of that other node and position 3 the relative
!                         position  of the node along that boundary
!     maxbounds      i    Max number of loops in nowbnd
!     maxlen         i    Maximum length of edges in the surface
!     nbndpt         i    number of boundary points of the starting boundary
!     ncrd          i/o   number of nodes in crd
!     nel           i/o   number of elements in kmeshc
!     nloop         i/o   number of boundaries in nowbnd
!     nowbnd        i/o   array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (maxbounds,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
!     zero           i    If true all nodes along the boundary have thickness
!                         zero
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numnodes, ibndnr, lnbnd, nelprev, imark(0:lnnowbnd+1), &
              nodenrs(0:lnnowbnd+1), newbnd(0:lnnowbnd+1)
      logical plotdebug

!     ibndnr         i'th boundary in nowbnd
!     imark          Array indicating for each point in a boundary loop
!                    how it should be treated in the following way:
!                     -1: new node is skipped  (sharp corner)
!                      0: thickness = 0: reuse old node
!                      1: standard case: a new node is created
!                    n>1: n new node are created (concave case)
!     lnbnd           Number of nodes in array newbnd
!     nelprev        Number of elements stored in kmeshc at the start
!                    of the subroutine
!     newbnd         temp array containing the boundary nodes for one loop
!     nodenrs        Array containing for each point in a boundary loop, &
!                    the new node number after paving, if it has one
!     numnodes       Number of nodal points in array nodes
!     plotdebug      If true plots must be made for debugging
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCONTROL     Check control integers around integer array
!     MSHPAV2SETZERO Set thickness equal to zero for nodes in edges where one of
!                    the two nodes has already thickness zero
!     MSHPAVBNDCHK1  Check if lines from old boundary to new boundary intersects
!                    (or is close to ) other boundary
!     MSHPAVCOPBND   Copy boundary bndcoorold to bndcoornew and set thickness of
!                    bndcoornew equal to zero for one specific boundary
!     MSHPAVCOPY     Fill new coordinates in bndcoornew by copying from crd
!     MSHPAVMARK     Mark all nodes along the closed boundary part
!     MSHPAVMNMESH   Create a row of elements along a closed boundary using the
!                    paver algorithm
!     MSHPAVMNSMOOTH Smooth the layer of elements created by removing small
!                    elements if possible
!     MSHPAVUPDBND03 Update the boundary ibndnr corresponding stored in nowbnd, &
!                    using the information stored in newbnd
!     PLOTMESHINTER  Plot actual mesh during the process of creation of a mesh
!     PLOTNEWBND     Plot the new boundary during the creation of the mesh in a
!                    specific iteration
!     PLOTNOWBND     Plot the actual boundary during mesh generation which is
!                    defined in array nowbnd, used in mshpav
!     PRININ         print 1d integer array
!     PRINNOWBND     Print array nowbnd which is used in mshpav
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Set control integers
!     Loop over all the present boundary parts
!        Fill array imark with information of the number of new nodes to
!        be used in correspondence with the old ones
!        Check if new boundary does not intersect other boundary parts
!        Fill row of elements into mesh, create new points and
!        create new boundary
!        Smooth the elements and the boundary if possible
!        update the boundary in nowbnd
!     Set thickness equal to zero for nodes in edges where one of the
!     two nodes has already thickness zero
!     Check control integers
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpav2layers' )
      debug = .false.
      plotdebug = .false.
      if ( layer.ge.10000 ) debug = .true.
      if ( layer.ge.10000 ) plotdebug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpav2layers'
         write(irefwr,1) 'nel, ncrd, layer, nloop, nbndpt', &
                          nel, ncrd, layer, nloop, nbndpt
         write(irefwr,1) 'lnnowbnd',lnnowbnd
         call prinnowbnd ( nowbnd, nloop, maxbounds, layer )
  1      format ( a, 1x, (10i6) )

      end if
      if ( plotdebug ) then

!     --- Debug information

         jtimes = 1
         call plotnowbnd ( nowbnd, nloop, maxbounds, layer, crd, &
                           'mshpav2layers/1' )
         jtimes = 3
         call plotmeshinter ( nel, kmeshc, crd, layer, 4, nbndpt, &
                              'mshpav2layers/1' )
         jtimes = 0

      end if

!     --- Set control integers

      imark(0) = 1005
      imark(lnnowbnd+1) = 1005
      nodenrs(0) = 1006
      nodenrs(lnnowbnd+1) = 1006
      newbnd(0) = 1007
      newbnd(lnnowbnd+1) = 1007

!     --- Loop over all the present boundary parts

      do ibndnr = 1, nloop


         newbnd(1) = 0
         if ( .not. zero(ibndnr) ) then

!        --- Fill array imark with information of the number of new nodes to
!            be used in correspondence with the old ones

            numnodes = nowbnd(ibndnr,1)
            call mshpavmark ( numnodes, bndcoorold(1,ibndnr), &
                              bndcoornew(1,1,ibndnr), crd, &
                              bndangle(1,ibndnr), layer, nowbnd, &
                              maxbounds, ibndnr, imark(1), ncrd, &
                              nodenrs(1), nbndpt, nloop, kmeshc, nel, &
                              markboun, lnnowbnd )

!           --- Check if new boundary does not intersect other boundary parts

            call mshpavbndchk1 ( ibndnr, maxbounds, nowbnd, nloop, &
                                 bndcoornew, bndcoorold(1,ibndnr), &
                                 layer, lnnowbnd, nodenrs(1), imark(1), &
                                 crd )
            if ( debug ) then
               write(irefwr,1) 'layer, ibndnr', layer, ibndnr
               call prinin ( nodenrs(1), numnodes, &
                             'nodenrs after mshpavmark' )
            end if
            if ( plotdebug ) then

!           --- Debug information

               jtimes = 1
               call plotnowbnd ( nowbnd, nloop, maxbounds, layer, crd, &
                                 'mshpav2layers/2' )
               jtimes = 2
               call plotmeshinter ( nel, kmeshc, crd, layer, 4, nbndpt, &
                                    'mshpav2layers/2' )
               jtimes = 3
               call plotnewbnd ( nodenrs(1), numnodes, layer, &
                                 bndcoornew(1,1,ibndnr), &
                                 'mshpav2layers/2' )
               jtimes = 0

            end if
            if ( ierror.ne.0 ) go to 1000

!           --- Fill row of elements into mesh, create new points and
!               create new boundary

            nelprev = nel
            call mshpavmnmesh ( numnodes, bndcoorold(1,ibndnr), &
                                bndcoornew(1,1,ibndnr), crd, &
                                layer, nowbnd, maxbounds, ibndnr, &
                                imark(1), nel, kmeshc, ncrd, newbnd(1), &
                                lnbnd, nloop, maxlen, nodenrs(1) )
            if ( debug ) then
               write(irefwr,1) 'after mshpavmnmesh: nel', nel
               call prinin ( newbnd(2), newbnd(1), 'newbnd/1' )
            end if
            if ( plotdebug ) then

!           --- Debug information

               jtimes = 1
               call plotnowbnd ( nowbnd, nloop, maxbounds, &
                                 layer, crd, 'mshpav2layers/2a' )
               jtimes = 3
               call plotmeshinter ( nel, kmeshc, crd, &
                    layer, 4, nbndpt, 'mshpav2layers/2a' )

            end if

!           --- Smooth the elements and the boundary if possible

            call mshpavmnsmooth ( crd, layer, nowbnd, &
                                  maxbounds, ibndnr,  nel, kmeshc, &
                                  ncrd, newbnd(1), nelprev, nbndpt, &
                                  nloop )
            if ( debug ) then
               write(irefwr,1) 'after mshpavmnsmooth: nel', nel
               call prinin ( newbnd(2), newbnd(1), 'newbnd/2' )
            end if
            if ( ierror.ne.0 ) go to 1000

!           --- Copy information from crd into bndcoornew

            call mshpavcopy ( newbnd(1), bndcoornew(1,1,ibndnr), crd )

            if ( plotdebug ) then

!           --- Debug information

               jtimes = 1
               call plotnowbnd ( nowbnd, nloop, maxbounds, &
                                 layer, crd, 'mshpav2layers/2b' )
               jtimes = 3
               call plotmeshinter ( nel, kmeshc, crd, &
                    layer, 4, nbndpt, 'mshpav2layers/2b' )

            end if

!           --- update the boundary in nowbnd

            call mshpavupdbnd03 ( ibndnr, maxbounds, nowbnd, &
                                  newbnd(1) )
            if ( plotdebug ) then

!           --- Debug information

               jtimes = 1
               call plotnowbnd ( nowbnd, nloop, maxbounds, &
                                 layer, crd, 'mshpav2layers/3' )
               jtimes = 3
               call plotmeshinter ( nel, kmeshc, crd, &
                    layer, 4, nbndpt, 'mshpav2layers/3' )

            end if

            if ( ierror.ne.0 ) go to 1000

         else

!        --- Thickness of boundary is zero, copy bncoorold into bndcoornew
!            and set thickness equal to zero

            call mshpavcopbnd ( bndcoorold(1,ibndnr), &
                                bndcoornew(1,1,ibndnr), nowbnd(ibndnr,1) )

         end if  ! ( .not. zero )

      end do  ! ibndnr = 1, nloop

      if ( plotdebug ) then

!     --- Debug information

         jtimes = 0
         call plotnowbnd ( nowbnd, nloop, maxbounds, layer, crd, &
                           'mshpav2layers/7' )

      end if

!     --- Set thickness equal to zero for nodes in edges where one of the
!         two nodes has already thickness zero
!         This is to avoid too many edges ending in one point
!         The number of layers may increase only slowly

      call mshpav2setzero ( maxbounds, nowbnd, nloop, crd, layer )

!     --- Check control integers

      call errcontrol ( imark(0), imark(lnnowbnd+1), 1005, 'imark' )
      call errcontrol ( nodenrs(0), nodenrs(lnnowbnd+1), 1006, &
                        'nodenrs' )
      call errcontrol ( newbnd(0), newbnd(lnnowbnd+1), 1007, 'newbnd' )

1000  call erclos ( 'mshpav2layers' )

      if ( debug ) then

!     --- Debug information

         call prinnowbnd ( nowbnd, nloop, maxbounds, layer )
         write(irefwr,*) 'End mshpav2layers'

      end if

      end
