      subroutine mshpavfindel ( inode1, inode2, kmeshc, nelstart, ielem, &
                                nel )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-11-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find element in set of elements that contains both nodes inode1 and inode2
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
!     element
!     find
!     find_element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inode1, inode2, kmeshc(3,*), nelstart, ielem, nel

!     ielem          o    Element number
!     inode1         i    First node
!     inode2         i    Second node
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nel            i    number of elements in the topology
!     nelstart       i    First element from which the search must take place
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, icount

!     i              Counting variable
!     icount         Counter
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavfindel' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavfindel'

      end if
      if ( ierror.ne.0 ) go to 1000

      ielem = 0
      do i = nelstart, nel
         icount = 0
         do j = 1, 3
            if ( kmeshc(j,i).eq.inode1 .or. kmeshc(j,i).eq.inode2 ) &
               icount = icount+1
         end do
         if ( icount.eq.2 ) then

!        --- element found

            ielem = i
            go to 1000

         end if

      end do  !  i = nelstart, nel

1000  call erclos ( 'mshpavfindel' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavfindel'

      end if

      end
