      subroutine mshpavmninter5 ( numnodes, bndcoornew, imark, nodenrs, &
                                  changed, layer, bndcoorold, nbndpt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-08-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if 3 subsequent edges on the new boundary have distance less than
!     0.7 times the old distance
!     If so reduce the middle edge to zero by taking mean values
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numnodes, imark(*), nodenrs(*), layer, nbndpt
      double precision bndcoornew(4,*), bndcoorold(2,*)
      logical changed

!     bndcoornew     i    Contains the coordinates of the new boundary created
!                         by making one layer during the paver algorithm
!     bndcoorold     i    Contains the coordinates of the previous (old)
!                         boundary during the paver process
!     changed        o    If true double points have been created
!     imark          i    Array indicating for each point in a boundary loop
!                         how it should be treated in the following way:
!                          -1: new node is skipped  (sharp corner)
!                           0: thickness = 0: reuse old node
!                           1: standard case: a new node is created
!                         n>1: n new node are created (concave case)
!     layer          i    Present layer sequence number
!     nbndpt         i    Number of nodal points in boundary, length of array
!                         kbndpt
!     nodenrs       i/o   Array containing for each point in a boundary loop, &
!                         the new node number after paving, if it has one
!     numnodes       i    number of coordinates of nodes in bndcoorold
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ind, iprev
      double precision xnew, ynew, ratio(0:numnodes+1)

!     ind            Sequence number along boundary loop
!     iprev          Previous node number
!     ratio          Contains the ratio between edges on old and new
!                    boundary (squared)
!     xnew           New x-coordinate
!     ynew           New y-coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCONTREAL    Check control integer around a real work array
!     MSHPAVMNRATIO  Compute ratio between edges on old and new boundary
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Set control integers
!     Store ratio of subsequent distances in array ratio
!     Check if three subsequent ratios are less than 0.7 on new boundary
!     If so remove middle edge
!     Check control integers
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavmninter5' )
      debug = .false.
      if ( layer.ge.10000 ) debug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavmninter5'
         write(irefwr,*) 'layer ', layer
         call prinin ( nodenrs, numnodes, 'nodenrs' )
         call prinin ( imark, numnodes, 'imark' )

      end if

!     --- Set control integers

      ratio(0) = 1005d0
      ratio(numnodes+1) = 1005d0

!     --- Store ratio of subsequent distances in array ratio (squared)

      call mshpavmnratio ( bndcoorold,  bndcoornew, numnodes, &
                           nodenrs, imark, ratio(1) )
      if ( debug ) call prinrl ( ratio(1), numnodes, 'ratio' )

!     --- Check if three subsequent ratios are less than 0.7 on new boundary
!         If so remove middle edge

      ind = 0
      iprev = numnodes
      do while ( ind.lt.numnodes-2 )

!     --- While loop over boundary

         ind = ind+1
         if ( ratio(ind).lt.0.5d0 ) then

!        --- ratio(ind) < 0.5, hence ratio < 0.7, check next two

            if ( ratio(ind+1).lt.0.5d0 ) then

!           --- ratio(ind+1) < 0.5, hence ratio < 0.7, check next one

               if ( ratio(ind+2).lt.0.5d0 .and. &
                    nodenrs(ind).gt.nbndpt ) then

!              --- ratio(ind+2) < 0.5, hence ratio < 0.7, remove middle edge

                  xnew = 0.5d0*(bndcoornew(1,iprev)+bndcoornew(1,ind))
                  ynew = 0.5d0*(bndcoornew(2,iprev)+bndcoornew(2,ind))
                  nodenrs(ind) = min(nodenrs(iprev),nodenrs(ind))
                  if ( nodenrs(iprev).gt.nbndpt ) then
                     nodenrs(iprev)= nodenrs(ind)
                     bndcoornew(1,iprev) = xnew
                     bndcoornew(2,iprev) = ynew
                  end if
                  if ( nodenrs(ind+1).gt.nbndpt ) then
                     nodenrs(ind+1) = nodenrs(ind+2)
                     bndcoornew(1,ind) = xnew
                     bndcoornew(2,ind) = ynew
                  end if
                  xnew = 0.5d0*(bndcoornew(1,ind+1)+bndcoornew(1,ind+2))
                  ynew = 0.5d0*(bndcoornew(2,ind+1)+bndcoornew(2,ind+2))
                  if ( nodenrs(ind+1).gt.nbndpt ) then
                     bndcoornew(1,ind+1) = xnew
                     bndcoornew(2,ind+1) = ynew
                  end if
                  if ( nodenrs(ind+2).gt.nbndpt ) then
                     bndcoornew(1,ind+2) = xnew
                     bndcoornew(2,ind+2) = ynew
                  end if
                  changed = .true.
                  ind = ind+2

               end if  ! ( ratio(ind+2).lt.0.5d0 )

            end if  ! ( ratio(ind+1).lt.0.5d0 )

         end if  ! (  ratio(ind+1).lt.0.5d0 )
         iprev = ind

      end do  ! while ( ind.lt.numnodes-2 )

!     --- Check control integers

      call errcontreal ( ratio(0), ratio(numnodes+1), &
                         1005d0, 'ratio' )

      call erclos ( 'mshpavmninter5' )

      if ( debug ) then

!     --- Debug information

         call prinin ( nodenrs, numnodes, 'nodenrs' )
         write(irefwr,*) 'End mshpavmninter5'

      end if

      end
