      subroutine mshpavmninter7 ( numnodes, bndcoornew, nodenrs, &
                                  layer, nbndpt, nowbnd, maxbounds, &
                                  ibndnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-08-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Give all nodes with the same number the same coordinates
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numnodes, nodenrs(*), layer, nbndpt, maxbounds, &
              nowbnd(maxbounds,*), ibndnr
      double precision bndcoornew(4,*)

!     bndcoornew    i/o   Contains the coordinates of the new boundary created
!                         by making one layer during the paver algorithm
!     ibndnr         i    i'th boundary in nowbnd
!     layer          i    Present layer sequence number
!     maxbounds      i    Max number of loops in nowbnd
!     nbndpt         i    Number of nodal points in boundary, length of array
!                         kbndpt
!     nodenrs       i/o   Array containing for each point in a boundary loop, &
!                         the new node number after paving, if it has one
!     nowbnd         i    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (imxlps,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
!     numnodes       i    number of coordinates of nodes in bndcoorold
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ind, i, iwork(0:numnodes+1), icount, inode, j
      double precision x, y

!     i              Counting variable
!     icount         Counter
!     ind            Sequence number along boundary loop
!     inode          Node sequence number
!     iwork          integer work array of length numnodes
!                    is used to store the new sequence of the node numbers in
!                    nodenrs after sorting
!     j              Counting variable
!     x              x co-ordinate of point
!     y              y co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHSORT         Sort integer array for increasing sequence
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCONTROL     Check control integers around integer array
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Store ratio of subsequent distances in array ratio
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavmninter7' )
      debug = .false.
      if ( layer.ge.10000 ) debug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavmninter7'
         write(irefwr,*) 'layer, ibndnr, nbndpt ', layer, ibndnr, nbndpt
         call prinin ( nodenrs, numnodes, 'nodenrs' )
         call prinrl1 ( bndcoornew, numnodes, 4, 'bndcoornew' )

      end if

!     --- Set control integers

      iwork(0) = 1009
      iwork(numnodes+1) = 1009

!     --- Sort array nodenrs, store sequence in iwork

      call chsort ( nodenrs, iwork(1), numnodes )

      ind = 1
      do while ( ind.lt.numnodes )

         inode = nodenrs(iwork(ind))
         ind = ind+1
         if ( nodenrs(iwork(ind)).eq.inode ) then

!        --- At least two nodes with the same node numbers
!            Check if there are more nodes with the same node number

            icount = 2
            ind = ind+1
            do while ( ind.le.numnodes .and. &
                       nodenrs(iwork(ind)).eq.inode )

!           --- More nodes with the same number

               ind = ind+1
               icount = icount+1

            end do
            if ( inode.lt.nbndpt ) then

!           --- Node on original boundary, use original coordinates

               do i = 1, icount

                  j = iwork(ind-i)
                  if ( nowbnd(ibndnr,j+1).eq.inode ) then

!                 --- Old node and new node are the same

                     x = bndcoornew(1,j)
                     y = bndcoornew(2,j)

                  end if  ! ( nowbnd(ibndnr,j+1).eq.inode )

               end do  ! i = 1, icount

            else  ! ( inode.ge.nbndpt )

!           --- Node not on original boundary, adapt

               x = 0d0
               y = 0d0
               do i = 1, icount
                  x = x+bndcoornew(1,iwork(ind-i))
                  y = y+bndcoornew(2,iwork(ind-i))
               end do
               x = x/icount
               y = y/icount

            end if  ! ( inode.lt.nbndpt )

            do i = 1, icount
               bndcoornew(1,iwork(ind-i)) = x
               bndcoornew(2,iwork(ind-i)) = y
            end do

         end if  ! ( nodenrs(iwork(ind)).eq.inode )

      end do  ! while  ( ind.lt.numnodes )

!     --- Check control integers

      call errcontrol ( iwork(0), iwork(numnodes+1), 1009, &
                        'iwork' )

      call erclos ( 'mshpavmninter7' )

      if ( debug ) then

!     --- Debug information

         call prinrl1 ( bndcoornew, numnodes, 4, 'bndcoornew' )
         write(irefwr,*) 'End mshpavmninter7'

      end if

      end
