      subroutine mshpavmnsmoot5 ( i, remove, newbnd, crd, layer )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-11-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if edge is candidate for removing by checking the angle between
!     edges at other sides
!     If edge less than 150 degrees removing is not allowed
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
!     smoothing
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer i, newbnd(*), layer
      double precision crd(4,*)
      logical remove

!     crd            i    array containing the coordinates of the nodes of
!                         the boundary as well as the thickness and factor
!                         See subroutine mshpav
!     i              i    Counting variable
!     layer          i    Present layer sequence number
!     newbnd        i/o   temp array containing the boundary nodes for one loop
!     remove        i/o   if true element is removed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numnodes, inode, inodenext, inodeprev
      double precision x1(2), x2(2), x3(2), x4(2), angle

!     angle          angle between two edges
!     inode          Node sequence number
!     inodenext      Next node along boundary
!     inodeprev      Previous node along boundary
!     numnodes       number of coordinates of nodes in bndcoornew
!     x1             coordinate of last point in edge i-1
!     x2             coordinate of first point in edge i-1
!     x3             coordinate of first point in edge i
!     x4             coordinate of last point in edge i
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHANGLE       Compute the cosine of the angle between the nodes with
!                    coordinates x1, x2 and x3, where the angle is defined
!                    as x3-x1-x2
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     First we store the coordinates of edge i-1 into x2, x1
!     Next we store the coordinates of edge i+1 into x3-x4
!     Finally we map x3 onto x1
!     Compute angle
!     if angle > 30 degrees or angle < 150 degrees, do not remove edge
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavmnsmoot5' )
      debug = .false.
      if ( layer.ge.10000 ) debug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavmnsmoot5'
         write(irefwr,*) 'i ', i
         call prinin ( newbnd(2), newbnd(1), 'newbnd' )

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- First we store the coordinates of edge i-1 into x2, x1

      numnodes = newbnd(1)
      if ( i.eq.1 ) then
         inodeprev = newbnd(numnodes)
         inode = newbnd(numnodes+1)
      else if ( i.eq.2 ) then
         inodeprev = newbnd(numnodes+1)
         inode = newbnd(i)
      else
         inodeprev = newbnd(i-1)
         inode = newbnd(i)
      end if

      x2(1) = crd(1,inodeprev)
      x2(2) = crd(2,inodeprev)
      x1(1) = crd(1,inode)
      x1(2) = crd(2,inode)
      if ( debug ) then
         write(irefwr,*) 'inode, inodeprev ', inode, inodeprev
         write(irefwr,*) 'x1, x2 ', x1, x2
      end if

!     --- Next we store the coordinates of edge i+1 into x3-x4

      if ( i.eq.numnodes-1 ) then
         inodenext = newbnd(2)
         inode = newbnd(i+1)
      else if ( i.eq.numnodes ) then
         inodenext = newbnd(3)
         inode = newbnd(2)
      else
         inodenext = newbnd(i+2)
         inode = newbnd(i+1)
      end if

      x4(1) = crd(1,inodenext)
      x4(2) = crd(2,inodenext)
      x3(1) = crd(1,inode)
      x3(2) = crd(2,inode)
      if ( debug ) then
         write(irefwr,*) 'inode, inodenext ', inode, inodenext
         write(irefwr,*) 'x3, x4 ', x3, x4
      end if

!     --- Finally we map x3 onto x1

      x3(1) = x4(1)-x3(1)+x1(1)
      x3(2) = x4(2)-x3(2)+x1(2)

!     --- Compute angle

      call mshangle ( x1, x2, x3, angle, 2 )
      if ( debug ) write(irefwr,*) 'angle ', angle

!     --- if angle > 30 degrees or angle < 150 degrees, do not remove edge

      if ( angle.lt.0.85d0 .and. angle.gt.-0.85d0 ) remove = .false.

1000  call erclos ( 'mshpavmnsmoot5' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavmnsmoot5'

      end if

      end
