      subroutine mshpavtrian ( maxbounds, nowbnd, nloop, crd, ncrd, &
                               kmeshc, nel, npoint, work, iwork, &
                               lniwork, maxnpoint, maxnelem, isurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 22-01-2014 New call to msho2d
!        version  3.0  date 02-09-2010 Remove intarmsh
!        version  2.0  date 29-01-2010 Extra parameter intarmsh
!
!   copyright (c) 2003-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill remaining parts of mesh after application of the paver algorithm
!     by submesh generator triangle
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer maxbounds, nowbnd(maxbounds,*), nloop,  ncrd, kmeshc(3,*), &
              nel, npoint, iwork(*), lniwork, maxnpoint, maxnelem, &
              isurnr
      double precision crd(4,*), work(*)

!     crd           i/o   array containing the coordinates of the nodes of
!                         the boundary as well as the thickness and factor
!                         See subroutine mshpav
!     isurnr         i    Surface sequence number
!     iwork               integer work array to be used by msho2d
!                         The length is checked by this subroutine
!     kmeshc        i/o   kmesh part c, contains the nodal points
!                         corresponding to the elements
!     lniwork        i    Declared length of array iwork
!     maxbounds      i    Max number of loops in nowbnd
!     maxnelem       i    Maximum number of elements allowed
!     maxnpoint      i    Maximum number of nodal points allowed
!     ncrd           i    number of nodes in crd
!     nel           i/o   number of elements in the topology
!     nloop          i    number of boundaries in nowbnd
!     nowbnd         i    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (imxlps,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
!     npoint        i/o   Number of nodal points
!     work                Work array to be used by msho2d
!                         The length is checked by this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ibndnr, numnodes, i, inode, kbound(2,npoint), ipitri, &
              ipchelp, holeinfo(2), ipcube, ipicube, ipjcube, ipkstapl, &
              npunt, npointloc, lencube, j, nelemloc, icount
      double precision coorloc(2,npoint), thickness(npoint), coar(1)

!     coar           Dummy
!     coorloc        Array with the coordinates of the local mesh
!                    created by msho2d
!     holeinfo       In this subroutine used as dummy array
!     i              Counting variable
!     ibndnr         i'th boundary in nowbnd
!     icount         Counter
!     inode          Node sequence number
!     ipchelp        Starting address of array chelp
!     ipcube         Starting address of array cube
!     ipicube        Starting address of array icube
!     ipitri         Starting address of array itri
!     ipjcube        Starting address of array jcube
!     ipkstapl       Starting address of array kstapl
!     j              Counting variable
!     kbound         Array containing the node numbers of the boundary to be
!                    used as input for msho2d
!     lencube        Computed length of array cube
!     nelemloc       Number of new elements in the original one
!     npointloc      Number of points in local mesh
!     npunt          Overestimation of the number of nodes to be created by
!                    msho2d
!     numnodes       Number of nodal points in boundary
!     thickness      Array containing the thickness in the boundary nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     MSHO2D         driving subroutine to triangulate region
!     MSHPAVTRIAADAP Adapt mesh created by subroutine mshpavtrian to satisfy the
!                    thickness requirements of paver
!     MSHPAVTRIANCT  Check control integers around the work arrays in
!                    mshpavtrian
!     PLOTMESHINTER  Plot actual mesh during the process of creation of a mesh
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINNOWBNDCRD  Print the actual boundary during the creation of the mesh
!                    in a specific iteration
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For ibndnr := 1 (1) nloop do
!        numnodes := number of nodes in boundary ibndnr
!        if ( numnodes>0 ) then
!           Copy coordinates of boundary nodes to local array coor_loc
!           create triangular mesh for boundary part
!           Copy new coordinates from coor_loc to coor
!           Copy kmeshc_loc to kmeshc and renumber nodes
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavtrian' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavtrian'
         write(irefwr,*) 'nloop, npoint ', nloop, npoint
         write(irefwr,*) 'nel, ncrd ', nel, ncrd
         call prinnowbndcrd ( nowbnd, nloop, maxbounds, 1000, crd )

      end if
      if ( ierror/=0 ) go to 1000

!     --- For ibndnr := 1 (1) nloop do

      icount = 0
      do ibndnr = 1, nloop

!     --- numnodes := number of nodes in boundary ibndnr

         numnodes = nowbnd(ibndnr,1)
         if ( debug ) &
            write(irefwr,*) 'ibndnr, numnodes ', ibndnr, numnodes

         if ( numnodes>0 ) then

!        --- numnodes>0
!            Copy coordinates of boundary nodes to local array coor_loc

            do i = 1, numnodes
               inode = abs(nowbnd(ibndnr,i+1))
               coorloc(1,i+icount) = crd(1,inode)
               coorloc(2,i+icount) = crd(2,inode)
               thickness(i+icount) = crd(4,inode)
            end do ! i = 1, numnodes

!           --- Fill array kbound

            do i = 1, numnodes-1
               kbound(1,i+icount) = i+icount
               kbound(2,i+icount) = i+1+icount
            end do
            kbound(1,numnodes+icount) = numnodes+icount
            kbound(2,numnodes+icount) = 1+icount

         end if  ! ( numnodes>0 )
         icount = icount+numnodes

      end do ! ibndnr = 1, nloop

!     --- if ( debug ) call prinrl1 ( coorloc, icount, 2, 'coorloc' )

!         --- create triangular mesh for rest of region

      if ( icount==0 ) go to 1000
      numnodes = icount
      lencube = npoint

!     --- Set control numbers around subwork arrays

      ipchelp = 2
      ipcube = ipchelp+npoint+2
      ipicube = 2
      ipitri = ipicube+npoint+2
      ipjcube = ipitri+npoint+2
      ipkstapl = ipjcube+lencube+2
      npunt = npoint
      npointloc = numnodes
      work(1) = 1001d0
      work(ipcube-2) = 1001d0
      work(ipcube-1) = 1002d0
      work(ipcube+lencube) = 1002d0
      iwork(1) = 1001
      iwork(ipitri-2) = 1001
      iwork(ipitri-1) = 1002
      iwork(ipjcube-2) = 1002
      iwork(ipjcube-1) = 1003
      iwork(ipkstapl-2) = 1003
      iwork(ipkstapl-1) = 1004
      iwork(lniwork) = 1004
      if ( debug ) then
         write(irefwr,*) 'npoint ', npoint
      end if

      nelemloc = maxnelem - nel  ! estimate of nelem

      call msho2d ( coorloc, npointloc, kbound, numnodes, &
                    kmeshc(1,nel+1), nelemloc, iwork, 1, &
                    npunt, 0, holeinfo, 0, .true., &
                    kbound, 0, coar, 0, iwork, &
                    work(1), 0, iwork, 1, iwork, isurnr, &
                    work(1), 1, 0, work, work )
      if ( ierror/=0 ) go to 1000
      call inichkarlen ( npointloc+ncrd-numnodes, maxnpoint, &
                         'crd' )
      call inichkarlen ( nel+nelemloc, maxnelem, 'kmeshc' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         call prinrl1 ( coorloc, npointloc, 2, 'coorloc' )
         call prinin1 ( kmeshc(1,nel+1), nelemloc, 3, &
                        'kmeshcloc' )
         call plotmeshinter ( nelemloc, kmeshc(1,nel+1), coorloc, &
                              6, 2, npointloc, 'triangle' )
      end if

!     --- Check control numbers

      call mshpavtrianct ( iwork, work, ipcube, ipitri, ipjcube, &
                           ipkstapl, lniwork, lencube )
      if ( ierror/=0 ) go to 1000

!     --- Adapt mesh created by subroutine mshpavtrian to satisfy the
!         thickness requirements of paver

      call mshpavtriaadap ( kmeshc(1,nel+1), nelemloc, npointloc, &
                            icount, thickness, &
                            coorloc, npoint, maxnelem-nel )

      if ( debug ) then
         call prinrl1 ( coorloc, npointloc, 2, 'coorloc' )
         call prinin1 ( kmeshc(1,nel+1), nelemloc, 3, &
                        'kmeshcloc' )
         call plotmeshinter ( nelemloc, kmeshc(1,nel+1), coorloc, &
                              6, 2, npointloc, 'triangle' )
      end if

      call inichkarlen ( npointloc+ncrd-numnodes, maxnpoint, &
                         'crd' )
      call inichkarlen ( nel+nelemloc, maxnelem, 'kmeshc' )
      if ( ierror/=0 ) go to 1000

!     --- Copy new coordinates from coor_loc to coor

      do i = 1, npointloc-numnodes
         crd(1,ncrd+i) = coorloc(1,numnodes+i)
         crd(2,ncrd+i) = coorloc(2,numnodes+i)
      end do

!     --- Copy kmeshc_loc to kmeshc and renumber nodes
!         First store new numbering of nodes into iwork

      icount = 0
      do ibndnr = 1, nloop
         numnodes = nowbnd(ibndnr,1)
         do i = 1, numnodes
            iwork(i+icount) = abs(nowbnd(ibndnr,i+1))
         end do
         icount = icount+numnodes
      end do ! ibndnr = 1, nloop
      if ( debug ) call prinin ( iwork, icount, 'iwork' )

      do i = 1, npointloc-icount
         iwork(i+icount) = ncrd+i
      end do
      ncrd = ncrd+npointloc-icount

      do i = 1, nelemloc
         do j = 1, 3
            kmeshc(j,nel+i) = iwork(kmeshc(j,nel+i))
         end do
      end do
      nel = nel+nelemloc

1000  call erclos ( 'mshpavtrian' )

      if ( debug ) then

!     --- Debug information

         call prinrl1 ( crd, ncrd, 2, 'coor' )
         call prinin1 ( kmeshc, nel, 3, 'kmeshc' )
         write(irefwr,*) 'End mshpavtrian'

      end if

      end
