      subroutine mshpipparms ( ishape, inpelmbot, inpelmsur, trian, &
                               prism, inpelm, linear, ishapebot )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 22-05-2014 Extension with shape 22
!        version  1.0  date 11-12-2002
!
!   copyright (c) 2002-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute some parameters for a pipe as function of the shape number
!     of the elements
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     volume
!     pipe
!     parameter
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ishape, inpelmbot, inpelmsur, inpelm, linear, ishapebot
      logical trian, prism

!     inpelm         o    number of points per element
!     inpelmbot      o    Number of nodes per element of bottom surface and
!                         top surface of pipe
!     inpelmsur      o    Number of nodes per element of pipe surface
!     ishape         i    Shape number of elements
!     ishapebot      o    Shape number of elements in bottom and top surface
!                         of pipe
!     linear         o    Indicates whether the element is a linear simplex
!                         (true) or not (false)
!     prism          o    If true prismatic elements are used
!     trian          o    Indication whether triangles (true) or quadrilaterals
!                         are created (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpipparms' )
      trian = .false.
      linear = 1
      inpelmbot = 3

      if ( ishape==11 .or. ishape== 18 ) then

!     --- Linear tetrahedron (18: with extra point)

         if ( ishape==18 ) then
            inpelm = 5
         else
            inpelm = 4
         end if
         trian = .true.

      else if ( ishape==12 ) then

!     --- Quadratic tetrahedron

         inpelm = 10
         inpelmbot = 6
         linear = 2
         trian = .true.

      else if ( ishape==13 .or. ishape==17 ) then

!     --- Tri-linear hexahedron (17: with extra point)

         if ( ishape==17 ) then
            inpelm = 9
         else
            inpelm = 8
         end if
         inpelmbot = 4

      else if ( ishape==14 ) then

!     --- Tri-quadratic hexahedron

         inpelm = 27
         inpelmbot = 9
         linear = 2

      else if ( ishape==15 ) then

!     --- Quadratic tetrahedron with centroids of faces

         inpelm = 14
         inpelmbot = 7
         linear = 2
         trian = .true.

      else if ( ishape==16 ) then

!     --- Quadratic tetrahedron with centroids of faces + centroid of
!         tetrahedron

         inpelm = 15
         inpelmbot = 7
         linear = 2
         trian = .true.

      else if ( ishape==21 ) then

!     --- Linear prism

         inpelm = 6
         ishapebot = 3
         inpelmsur = 4

      else if ( ishape==22 ) then

!     --- Quadratic prism

         inpelm = 18
         linear = 2
         inpelmbot = 6
         ishapebot = 4
         inpelmsur = 9

      else

!     --- shape number not implemented

         call errint ( ishape, 1 )
         call errsub ( 800, 1, 0, 0 )

      end if
      if ( ishape<=20 ) then

!     --- Tetrahedron or hexahedron

         inpelmsur = inpelmbot
         prism = .false.
         ishapebot = ishape-8

      else

!     --- Prism

         prism = .true.

      end if
      call erclos ( 'mshpipparms' )
      end
