      subroutine mshrdplot ( iinput, lengin, rinput, ndim, lengrl, &
                             yfact, plot )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 28-03-2013 Extra parameter plot
!        version  1.2  date 09-03-2013 Correction length code
!        version  1.1  date 12-03-2008 New way of defining scalars
!
!   copyright (c) 2003-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read rest of input following the keyword plot
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     mesh
!     read
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      use sepmoduleconsts
      use sepmoduleplot
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), lengin, ndim, lengrl
      double precision rinput(*), yfact
      logical plot

!     iinput        i/o   integer input array
!     lengin        i/o   Length of integer array iinput
!     lengrl        i/o   Length of real array rinput
!                         (in double precision variables)
!     ndim           i    Dimension of the space.
!     plot          i/o   If true plot has been found
!     rinput        i/o   Real input array
!     yfact          i    Multiplication for the y-co-ordinates. With this
!                         parameter the user can influence the scale in
!                         y-direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nplot=17
      character (len=11) code2(nplot)
      integer itype2(nplot), ncode2(nplot), ncodm2(nplot), nomesh, &
              irenpl, iorien, nsurfs, &
              nvolms, i, icolor, jcurvn, jcurvp, juserp, numsub
      double precision plotfm, tabmn2(nplot), tabmx2(nplot)
      logical ::  threed

!     code2          In this table the significant characters of the various
!                    texts after plot are stored
!     i              Counting variable
!     icolor         Indicates if the plots must be made in one colour or not
!                    0:  One colour (i.e. default colour)
!                    >0: The texts are plotted int the default colour
!                        The boundaries etc. are plotted in colour icolor
!                        The marks are plotted in colour icolor+1
!                        The submeshes in the final mesh are plotted in the
!                        colour icolor+1+element group
!     iorien         Defines orientation of the base vectors in 3D plots
!                    1 x-y-z
!                    2 z-x-y
!                    3 y-z-x
!     irenpl         If 1 renumbered nodes must be plotted
!     itype2         Indicates the type of action corresponding to keywords in
!                    code2
!     jcurvn         Indicates if nodes along curves must be plotted
!                    0:  No plotting
!                    1:  Plot with crosses
!                    2:  Plot with symbol (value of symbol jcurvn-1)
!     jcurvp         Indicates if and how curves must be plotted
!                    0:  No plotting
!                    1:  Plot without numbers of curves
!                    2:  Plot with numbers of curves
!     juserp         Indicates if and how userpoints must be plotted
!                    0:  No plotting
!                    1:  Plot without numbers of user points
!                    2:  Plot with numbers of user points
!     ncode2         number of significant characters in table code2
!     ncodm2         maximum number of characters in table code2
!     nomesh         If 1 the mesh is not plotted
!     nplot          Number of entries in table code2
!     nsurfs         Number of surfaces in mesh
!     numsub         Number of submeshes to be skipped in plotting
!     nvolms         number of volumes
!     plotfm         Plotting factor. When PLOTFM<0, then the length of
!                    the plot is computed by - (xmax-xmin) * PLOTFM
!                    and the width by - (ymax-ymin) * YFACT * PLOTFM
!                    else the length of the plot is equal to PLOTFM cm
!                    When by some mistake the user has given | PLOTFM | a value
!                    smaller than 1 cm, then the length is chosen equal to
!                    the default length stored in common block consta
!     tabmn2         Table containing the minimum values of the quantities given
!                    in code2
!     tabmx2         Table containing the maximum values of the quantities given
!                    in code2
!     threed         Indication if eyepoint has been found (true) or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1391   Not enough numbers found after eyepoint.
!    1552   keyword found more than once
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'LENGTH', 'SCALE', 'YFACT', 'JMARK', 'NUMSUB', &
                   'CURVE', 'NODES', 'USERPOINTS', 'COLOUR', &
                   'ROTATE', 'SUPPRESS', 'EYEPOINT', 'REN_PLOT', &
                   'NOMESH', 'NOSUBMESH', 'PLOTFM', 'ORIENTATIO' /
      data itype2 / 1,  1, 1, 0, 0,  0, 0, 0, 0, 0, &
                    0, 19, 2, 2, 2,  1, 0 /
      data ncode2 / 4,  3, 2, 3, 4,  3, 3, 3, 3, 3, &
                    4,  3, 5, 5, 5,  4, 5 /
      data ncodm2 / 6,  5, 5, 5, 6,  6, 5,10, 6, 6, &
                    8,  9, 8, 6, 9,  6,11 /
      data tabmn2 / 3d0, 1d-8, 1d-8, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, 0d0, &
                    0d0,  3d0, 0d0,  0d0, 0d0, 3d0, 1d0 /
      data tabmx2 / 1d3,  1d8, 1d8,  5d0, 1d6, 2d0, 2d0, 2d0, 1d3, 2d0, &
                    1d0, 1d28, 0d0,  0d0, 0d0, 1d3, 3d0 /

!    code2     In this table the significant characters of the various
!              texts after plot are stored
!              The following types of texts are recognised:
!              1:   length
!              2:   scale
!              3:   yfact
!              4:   jmark
!              5:   numsub
!              6:   curve
!              7:   nodes
!              8:   userpoints
!              9:   colour
!             10:   rotate
!             11:   suppress
!             12:   eyepoint
!             13:   renumber_plot
!             14:   nomesh
!             15:   nosubmesh
!             16:   plotfm
!             17:   orientation
! ======================================================================
!
      call eropen ( 'mshrdplot' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrdplot'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )

      if ( plot ) then

!     --- plot has been found more than once

         call errchr ( string(1:nchar), 1 )
         call errwar ( 1552, 0, 0, 1 )

      end if
      plot = .true.

      threed = .false.
      nsurfs = iinput(4)
      nvolms = iinput(5)

      call sepcopycode ( code2, nplot, ncode2, ncodm2 )
      call sepcopytype ( nplot, itype2, tabmn2, tabmx2 )

      tabmax(12) = 1d28
      table(1:nncode) = -1d19

!     --- Remove main keyword PLOT temporarily

      nmain = nncodemain
      codemain(7) = 'xxxx'
      call sepreadsetkeyw
      codemain(7) = 'PLOT'

      if ( debug ) then
         call prinrl ( table, nncode, 'table' )
         write(irefwr,1) 'nncodemain, iretur', nncodemain, iretur
      end if  ! ( debug )

      jmark = 5
      numsub = 0
      jcurvp = 2
      jcurvn = 0
      juserp = 0
      icolor = 0
      nomesh = 0
      irenpl = 0
      iorien = 1
      if ( jkader<0 .and. jkader>-4 ) then
         jtimes = 1
      else
         jtimes = 0
      end if
      if ( ndim==3 ) then

!     --- NDIM = 3, set eyepoint into rinput

         rinput(lengrl+1) = 2d0*xmax-xmin
         rinput(lengrl+2) = 2d0*ymax-ymin
         rinput(lengrl+3) = 2d0*zmax-zmin
         lengrl = lengrl + 3

      end if
      if ( table(1)>0d0 ) plotfm = table(1)
      if ( table(16)>0d0 ) plotfm = table(16)
      if ( plotfm<=0d0 ) plotfm = defplo
      if ( table(2)>0d0 ) plotfm = -table(2)
      if ( table(3)>0d0 ) yfact = table(3)
      if ( table(4)>-1d0 ) jmark = nint(table(4))
      if ( table(5)>-1d0 ) numsub = nint(table(5))
      if ( table(6)>-1d0 ) jcurvp = nint(table(6))
      if ( table(7)>-1d0 ) jcurvn = nint(table(7))
      if ( table(8)>-1d0 ) juserp = nint(table(8))
      if ( table(9)>-1d0 ) icolor = nint(table(9))
      if ( table(10)>-1d0 ) irotat = nint(table(10))
      if ( table(11)>-1d0 ) jtimes = nint(table(11))
      if ( table(12)>-1d0 ) then

!     --- eye point found

         if ( nint(tabmax(12))/=3 ) &
              call errsub ( 1391, 0, 0, 0 )
         do i = 1, 3
            rinput(lengrl-3+i) = tabout(i)
         end do
         threed = .true.

      end if
      if ( table(13)>-1d0 ) irenpl = 1
      if ( table(14)>-1d0 ) nomesh = 1
      if ( table(15)>-1d0 ) numsub = nsurfs+nvolms+1
      if ( table(17)>-1d0 ) iorien = nint(table(17))
      iinput(9) = -lengin-6
      iinput(lengin+1) = jmark
      iinput(lengin+2) = numsub
      iinput(lengin+3) = jcurvp
      iinput(lengin+4) = jcurvn
      iinput(lengin+5) = juserp
      iinput(lengin+6) = icolor
      iinput(lengin+7) = 0
      iinput(lengin+8) = irenpl
      iinput(lengin+9) = nomesh
      iinput(lengin+10) = iorien
      if ( threed ) iinput(lengin+7) = 1
      lengin = lengin + 10

      call erclos ( 'mshrdplot' )
      if ( debug ) write(irefwr,*) 'End mshrdplot'

      end
