      subroutine mshreadtext ( irefc, text, filenm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 26-10-2005 Extra debug
!        version  1.0  date 20-03-2003
!
!   copyright (c) 2003-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read text from file and compare with text that is expected
!
! **********************************************************************
!
!                       KEYWORDS
!
!     read
!     compare
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer  irefc
      character *(*) text, filenm

!     filenm         i    Name of file from which reading takes place
!     irefc          i    dataset reference number for reading (>0 formatted, &
!                         <0 unformatted)
!     text           i    String variable containing the text that is supposed
!                         to be read
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      character *80 textread

!     textread       String variable to store the text that is read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2412   Text read is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshreadtext' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshreadtext'
         write(irefwr,*) text
         write(irefwr,*) 'irefc ', irefc

      end if

!     --- Read text from file and compare with input text

      if ( irefc.ge.0 ) then

!     --- irefc > 0, formatted

         read(irefc,100) textread
100      format(a)

      else

!     --- irefc < 0, unformatted

         read(-irefc) textread

      end if
      if ( debug ) write(irefwr,*) textread
      if ( text.ne.textread ) then

!     --- Text is incorrect

         call errchr ( filenm, 1 )
         call errchr ( textread, 2 )
         call errchr ( text, 3 )
         call errint ( abs(irefc), 1 )
         call errsub ( 2412, 1, 0, 3 )
         go to 1000

      end if

1000  if ( debug ) write(irefwr,*) 'End mshreadtext'
      call erclos ( 'mshreadtext' )

      end
