      subroutine mshreatrans ( transf, itrans )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-03-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read and interprete part of input corresponding to transformation
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer itrans
      logical transf

!     itrans        i/o   Indicates if the mesh must be transformed from a
!                         linear triangular mesh to a new type with type
!                         number itrans
!                         See mshrd5
!     transf        i/o   If true transformation has been found
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1328   The type number read after transform has incorrect value
!    1552   keyword found more than once
!    1762   The text after transform is not recognized
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshreatrans' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshreatrans'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( transf ) then

!     --- transformation has been found more than once

         call errchr ( string(1:nchar), 1 )
         call errwar ( 1552, 0, 0, 1 )

      end if
      transf = .true.
      call asknxt
      if ( ityprd==4 .and. string(1:4)=='TYPE' ) then

         call asknxt
         if ( ityprd==2 ) then

!        --- integer found after transform. Check this integer

            itrans = nint(realrd)
            if ( itrans<4 .or. itrans>7 ) then

!           --- itrans out of range

               call errint ( itrans, 1 )
               call errsub ( 1328, 1, 0, 0 )
               call errcrd

            end if

         else

!        --- Wrong character found after transform

            call errsub ( 1762, 0, 0, 0 )
            call errcrd

         end if

      else if ( ityprd==1 ) then

!     --- End of record found, i.e. no number

         itrans = 5

      else

!     --- Wrong character found after transform

         call errsub ( 1762, 0, 0, 0 )
         call errcrd

      end if

1000  call erclos ( 'mshreatrans' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshreatrans'

      end if  ! ( debug )

      end
