      subroutine mshrefcheck ( icheck )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 10-02-2012 use getnpelmelgrp
!        version  1.1  date 27-11-2011 use function getshapeelgrp
!        version  1.0  date 20-09-2003
!
!   copyright (c) 2003-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check volume of all elements corresponding to surfaces and volumes after
!     refinement
! **********************************************************************
!
!                       KEYWORDS
!
!     check
!     refine
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: icheck

!     icheck         i    Level of check of elements
!                         0:  no special checks (default)
!                         1:  check volume of elements
!                         2:  check volume of elements and print minimum and
!                             maximum value
!                             per surface/volume
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp, ikelmc, inpelm

!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHKSBM         Check volume of submesh
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     for ielgrp := 1 (1) nelgrp do
!        ishape := shape number of elements in group ielgrp
!        inelem := number of elements in ielgrp
!        if ( ishape>3 ) then
!           check surface or volume
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefcheck' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefcheck'

      end if
      if ( ierror/=0 ) go to 1000

      inpelm = npelm
      inelem = nelem

!     --- for ielgrp := 1 (1) nelgrp do

      ikelmc = 1
      do ielgrp = 1, nelgrp

!     --- ishape := shape number of elements in group ielgrp
!         inelem := number of elements in ielgrp

         ishape = kmeshf(ielgrp)
         if ( kelmg>0 ) inelem = kmeshg(ielgrp)
         if ( kelma>0 ) inpelm = kmesha(ielgrp+1)-kmesha(ielgrp)
         if ( ishape>0 ) then

!        --- ishape>0
!            check surface or volume

            call chksbm ( kmeshc(ikelmc), inelem, ishape, inpelm, &
                          ielgrp, coor, ndim, icheck, npoint )
         end if  ! ( ishape>0 )

         ikelmc = ikelmc+inelem*inpelm

      end do  ! ielgrp = 1, nelgrp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefcheck'

      end if
      call erclos ( 'mshrefcheck' )

      end
