      subroutine mshrefcoor9 ( kmeshclc, first_node, fillmap )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-06-2014 extra parameters
!        version  1.0  date 20-09-2003
!
!   copyright (c) 2003-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the coordinates for the "face" points in case of extended bilinear
!     quadrilaterals of shape 9
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     coordinate
!     node
!     refine
!     linear
!     quadrilateral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshclc(5), first_node
      logical :: fillmap

!     fillmap        o    If true array imap must be filled
!     first_node    i/o   Node number of next node to be created
!                                     required for the interpolation
!     kmeshclc       i    kmesh part c, contains the nodal points
!                         corresponding to the elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inode, inode1, inode2, inode3, inode4, inode5

!     inode          Node sequence number
!     inode1         Node sequence number
!     inode2         Node sequence number
!     inode3         Node sequence number
!     inode4         Node sequence number
!     inode5         Node sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The quadrilateral has 5 nodes according to
!     The edges are defined as: 1-2,  2-3,  3-4, 4-1
!     Let the corresponding nodes be e1, e2, e3 and e4
!
!
!            4     e3    3
!            x-----0-----x
!            |           |
!            |           |
!        e4  0     +     0  e2
!            |      5    |
!            |           |
!            x-----0-----x
!           1     e1     2
!
!     The face nodes are defined in the centres of the 4 quads:
!     f_1 in 1 e1 5 e4
!     f_2 in 2 e2 5 e1
!     f_3 in 3 e3 5 e2
!     f_4 in 4 e4 5 e3
!
!
!     inode := nodenumbers(ielem)
!     inodei := i-th node in element
!     Fill 4 face nodes
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefcoor9' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefcoor9'

      end if
      if ( ierror/=0 ) go to 1000

!     --- inode := nodenumbers(ielem)
!         inodei := i-th node in element
!         Fill 4 face nodes

      inode = first_node
      inode1 = kmeshclc(1)
      inode2 = kmeshclc(2)
      inode3 = kmeshclc(3)
      inode4 = kmeshclc(4)
      inode5 = kmeshclc(5)

!     --- f_1 in 1 e1 5 e4

      coor(1:ndim,inode) = 0.25d0*(2d0*coor(1:ndim,inode1)+ &
                                   coor(1:ndim,inode5)+ &
                      0.5d0*(coor(1:ndim,inode2)+coor(1:ndim,inode4)))

      if ( fillmap ) then
         imap(inode,1) = 4
         imap(inode,2) = inode1
         imap(inode,3) = inode5
         imap(inode,4) = inode2
         imap(inode,5) = inode4
         rmap(inode,1) = 0.5d0
         rmap(inode,2) = 0.25d0
         rmap(inode,3:4) = 0.125d0
      end if  ! ( fillmap )

!     --- f_2 in 2 e2 5 e1

      inode = inode+1
      coor(1:ndim,inode) = 0.25d0*(2d0*coor(1:ndim,inode2)+ &
                                   coor(1:ndim,inode5)+ &
                      0.5d0*(coor(1:ndim,inode1)+coor(1:ndim,inode3)))

      if ( fillmap ) then
         imap(inode,1) = 4
         imap(inode,2) = inode2
         imap(inode,3) = inode5
         imap(inode,4) = inode1
         imap(inode,5) = inode3
         rmap(inode,1) = 0.5d0
         rmap(inode,2) = 0.25d0
         rmap(inode,3:4) = 0.125d0
      end if  ! ( fillmap )

!     --- f_3 in 3 e3 5 e2

      inode = inode+1
      coor(1:ndim,inode) = 0.25d0*(2d0*coor(1:ndim,inode3)+ &
                                   coor(1:ndim,inode5)+ &
                      0.5d0*(coor(1:ndim,inode2)+coor(1:ndim,inode4)))

      if ( fillmap ) then
         imap(inode,1) = 4
         imap(inode,2) = inode3
         imap(inode,3) = inode5
         imap(inode,4) = inode2
         imap(inode,5) = inode4
         rmap(inode,1) = 0.5d0
         rmap(inode,2) = 0.25d0
         rmap(inode,3:4) = 0.125d0
      end if  ! ( fillmap )

!     --- f_4 in 4 e4 5 e3

      inode = inode+1
      coor(1:ndim,inode) = 0.25d0*(2d0*coor(1:ndim,inode4)+ &
                                   coor(1:ndim,inode5)+ &
                   0.5d0*(coor(1:ndim,inode1)+coor(1:ndim,inode3)))

      if ( fillmap ) then
         imap(inode,1) = 4
         imap(inode,2) = inode4
         imap(inode,3) = inode5
         imap(inode,4) = inode1
         imap(inode,5) = inode3
         rmap(inode,1) = 0.5d0
         rmap(inode,2) = 0.25d0
         rmap(inode,3:4) = 0.125d0
      end if  ! ( fillmap )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefcoor9'

      end if
      call erclos ( 'mshrefcoor9' )

      end
