      subroutine mshrefkmeshc10 ( kmeshc, kmeshcnew, nelem, nelprev, &
                                  nmeshd, nodenumbers, nedges )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-09-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the new array kmeshc for one element group in case that
!     the complete mesh is refined
!     Elements of shape 10, i.e. extended linear triangles
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(4,*), kmeshcnew(4,*), nelem, nodenumbers(*), &
              nelprev, nmeshd(*), nedges

!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshcnew      o    New kmesh part c
!     nedges         i    number of edges
!     nelem          i    Number of elements
!     nelprev        i    Number of elements stored in kmeshc at the start
!                         of the subroutine
!     nmeshd         i    nmesh part d
!     nodenumbers    i    Array containing the node numbers of the newly created
!                         nodes in the following way
!                         Pos. 1 to nedges node number of first node on edge
!                         Next nfaces positions node number of first node on
!                         face not stored before
!                         Next nelem positions node numbers of first extra node
!                         in element not stored before
!                         if elddimmax = 1 only the third part is used
!                         if elddimmax = 2 the second part is not used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielemnew, ielem, ipstart, iedge, edge(3), i, iface

!     edge           Unit length defined for coarseness
!     i              Counting variable
!     iedge          edge sequence number
!     ielem          Element number
!     ielemnew       Sequence number of element in new mesh
!     iface          Face sequence number
!     ipstart        Starting address in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The extended linear triangle has 4 nodes according to
!     The edges are defined as: 1-2,  2-3,  3-1
!     Let the corresponding nodes be e1, e2 and e3
!
!                  3
!                  x
!                 / \
!                /   \
!            e3 x     x e2
!              /   +   \
!             /     4   \
!            x-----x-----x
!           1     e1     2
!
!     The three face nodes are defined centres of the 3 triangles
!     f_1 in 1 e1 e3
!     f_2 in 2 e2 e1
!     f_3 in 3 e3 e2
!
!     Then the 4 new triangles become:
!     1 e1 e3 f1
!     2 e2 e1 f2
!     3 e3 e2 f3
!     e1 e2 e3 4
!
!     ielemnew := 0
!     For all elements ielem in old mesh do
!        ielemnew := ielemnew+1
!        fill first element
!        ielemnew := ielemnew+1
!        fill second element
!        ielemnew := ielemnew+1
!        fill third element
!        ielemnew := ielemnew+1
!        fill fourth element
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefkmeshc10' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefkmeshc10'
         write(irefwr,*) 'nelem, nelprev ', nelem, nelprev
         call prinin1 ( kmeshc, nelem, 4, 'kmeshc/surface' )

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- ielemnew := 0

      ielemnew = 0

!     --- For all elements ielem in old mesh do

      do ielem = 1, nelem

!     --- ielemnew := ielemnew+1
!         fill first element
!         ielemnew := ielemnew+1
!         fill second element
!         ielemnew := ielemnew+1
!         fill third element
!         ielemnew := ielemnew+1
!         fill fourth element

         ipstart = nmeshd(nelprev+ielem)
         do i = 1, 3
            iedge = abs(nmeshd(ipstart+i))
            edge(i) = nodenumbers(iedge)
         end do
         iface = nodenumbers(nedges+ielem+nelprev)  ! node number of first face
         ielemnew = ielemnew+1

!        --- 1 e1 e3 f1

         kmeshcnew(1,ielemnew) = kmeshc(1,ielem)
         kmeshcnew(2,ielemnew) = edge(1)
         kmeshcnew(3,ielemnew) = edge(3)
         kmeshcnew(4,ielemnew) = iface
         ielemnew = ielemnew+1

!        --- 2 e2 e1 f2

         kmeshcnew(1,ielemnew) = kmeshc(2,ielem)
         kmeshcnew(2,ielemnew) = edge(2)
         kmeshcnew(3,ielemnew) = edge(1)
         kmeshcnew(4,ielemnew) = iface+1
         ielemnew = ielemnew+1

!        --- 3 e3 e2 f3

         kmeshcnew(1,ielemnew) = kmeshc(3,ielem)
         kmeshcnew(2,ielemnew) = edge(3)
         kmeshcnew(3,ielemnew) = edge(2)
         kmeshcnew(4,ielemnew) = iface+2
         ielemnew = ielemnew+1

!        --- e1 e2 e3 4

         kmeshcnew(1,ielemnew) = edge(1)
         kmeshcnew(2,ielemnew) = edge(2)
         kmeshcnew(3,ielemnew) = edge(3)
         kmeshcnew(4,ielemnew) = kmeshc(4,ielem)

      end do  ! ielem = 1, nelem

1000  call erclos ( 'mshrefkmeshc10' )

      if ( debug ) then

!     --- Debug information

         call prinin1 ( kmeshcnew, ielemnew, 4, 'kmeshcnew/surface' )
         write(irefwr,*) 'End mshrefkmeshc10'

      end if

      end
