      subroutine mshrefkmeshc4e ( kmeshc, kmeshcnew, ielemnew, edge, &
                                  face  )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-09-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Refine one element in case of a complete refinement
!     Element of shape 4, i.e. quadratic triangles
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     topology
!     quadratic
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(6), kmeshcnew(6,*), ielemnew, edge(3,2), face(3)

!     edge           i    Contains the edge node numbers on the 3 edges
!                         (2 per edge)
!     face           i    Contains the face node numbers on the 3 faces
!     ielemnew       i    Sequence number of element in new mesh
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshcnew      o    New kmesh part c
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The quadratic triangle has 6 nodes according to
!
!                  5
!                  x
!                 / \
!                /   \
!             6 x     x 4
!              /       \
!             /         \
!            x-----x-----x
!           1      2     3
!
!     The three face nodes are defined as follows
!     f_1 between 6 and 2
!     f_2 between 2 and 4
!     f_3 between 4 and 6
!
!     The edges are defines as: 1-3,  3-5,  5-1
!     Let the corresponding nodes be e1, e2 and e3
!
!            5
!            x
!           / \
!       e3 0   0  e2
!         /     \
!        x---0---x
!        1   e1  3
!
!     On each edge we have two extra nodes: e1a/b etc
!     Then the 4 new triangles become:
!     1 e1a 2 f_1 6 e3b
!     3 e2a 4 f_2 2 e1b
!     5 e3a 6 f_3 4 e2b
!     2 f_2 4 f_3 6 f_1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefkmeshc4e' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefkmeshc4e'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- 1 e1a 2 f_1 6 e3b

      ielemnew = ielemnew+1
      kmeshcnew(1,ielemnew) = kmeshc(1)
      kmeshcnew(2,ielemnew) = edge(1,1)
      kmeshcnew(3,ielemnew) = kmeshc(2)
      kmeshcnew(4,ielemnew) = face(1)
      kmeshcnew(5,ielemnew) = kmeshc(6)
      kmeshcnew(6,ielemnew) = edge(3,2)

!     --- 3 e2a 4 f_2 2 e1b

      ielemnew = ielemnew+1
      kmeshcnew(1,ielemnew) = kmeshc(3)
      kmeshcnew(2,ielemnew) = edge(2,1)
      kmeshcnew(3,ielemnew) = kmeshc(4)
      kmeshcnew(4,ielemnew) = face(2)
      kmeshcnew(5,ielemnew) = kmeshc(2)
      kmeshcnew(6,ielemnew) = edge(1,2)

!     --- 5 e3a 6 f_3 4 e2b

      ielemnew = ielemnew+1
      kmeshcnew(1,ielemnew) = kmeshc(5)
      kmeshcnew(2,ielemnew) = edge(3,1)
      kmeshcnew(3,ielemnew) = kmeshc(6)
      kmeshcnew(4,ielemnew) = face(3)
      kmeshcnew(5,ielemnew) = kmeshc(4)
      kmeshcnew(6,ielemnew) = edge(2,2)

!     --- 2 f_2 4 f_3 6 f_1

      ielemnew = ielemnew+1
      kmeshcnew(1,ielemnew) = kmeshc(2)
      kmeshcnew(2,ielemnew) = face(2)
      kmeshcnew(3,ielemnew) = kmeshc(4)
      kmeshcnew(4,ielemnew) = face(3)
      kmeshcnew(5,ielemnew) = kmeshc(6)
      kmeshcnew(6,ielemnew) = face(1)

1000  call erclos ( 'mshrefkmeshc4e' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefkmeshc4e'

      end if

      end
