      subroutine mshrefkmeshc6e ( kmeshc, kmeshcnew, ielemnew, edge, &
                                  face  )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-09-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Refine one element in case of a complete refinement
!     Element of shape 6, i.e. bi-quadratic quadrilaterals
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     topology
!     quadratic
!     quadrilateral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(9), kmeshcnew(9,*), ielemnew, edge(4,2), face(8)

!     edge           i    Contains the edge node numbers on the 4 edges
!                         (2 per edge)
!     face           i    Contains the face node numbers on the 8 faces
!     ielemnew       i    Sequence number of element in new mesh
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the element
!     kmeshcnew      o    New kmesh part c
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The bi-quadratic quadrilateral has 9 nodes according to
!
!            7      6      5
!            x------x------x
!            |             |
!            |             |
!            |      9      |
!         8  x      x      x  4
!            |             |
!            |             |
!            |             |
!            x------x------x
!           1       2      3
!
!
!     The eight face nodes are defined as follows
!     f_1 between 2 and 9
!     f_2 between 4 and 9
!     f_3 between 6 and 9
!     f_4 between 8 and 9
!     The other 4 in the centers of the 4 quadrants:
!     f_5 in  1 - 2 - 9 - 8
!     f_6 in  2 - 3 - 9 - 2
!     f_7 in  4 - 5 - 6 - 9
!     f_8 in  6 - 7 - 8 - 9
!
!     The edges are defines as: 1-3,  3-5,  5-7, 7-1
!     Let the corresponding nodes be e1, e2, e3 and e4
!
!            4     e3    3
!            x-----0-----x
!            |           |
!            |           |
!        e4  0     +     0  e2
!            |           |
!            |           |
!            x-----0-----x
!           1     e1     2
!
!
!     On each edge we have two extra nodes: e1a/b etc
!     Then the 4 new quadrilaterals become:
!     1 e1a 2 f_1 9 f_4 8 e4b f_5
!     3 e2a 4 f_2 9 f_1 2 e1b f_6
!     5 e3a 6 f_3 9 f_2 4 e2b f_7
!     7 e4a 8 f_4 9 f_3 6 e3b f_8
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefkmeshc6e' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefkmeshc6e'
         call prinin ( kmeshc, 9, 'element_old' )
         call prinin1 ( edge, 2, 4, 'edge' )
         call prinin ( face, 8, 'face' )

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- 1 e1a 2 f_1 9 f_4 8 e4b f_5

      ielemnew = ielemnew+1
      kmeshcnew(1,ielemnew) = kmeshc(1)
      kmeshcnew(2,ielemnew) = edge(1,1)
      kmeshcnew(3,ielemnew) = kmeshc(2)
      kmeshcnew(4,ielemnew) = face(1)
      kmeshcnew(5,ielemnew) = kmeshc(9)
      kmeshcnew(6,ielemnew) = face(4)
      kmeshcnew(7,ielemnew) = kmeshc(8)
      kmeshcnew(8,ielemnew) = edge(4,2)
      kmeshcnew(9,ielemnew) = face(5)

!     --- 3 e2a 4 f_2 9 f_1 2 e1b f_6

      ielemnew = ielemnew+1
      kmeshcnew(1,ielemnew) = kmeshc(3)
      kmeshcnew(2,ielemnew) = edge(2,1)
      kmeshcnew(3,ielemnew) = kmeshc(4)
      kmeshcnew(4,ielemnew) = face(2)
      kmeshcnew(5,ielemnew) = kmeshc(9)
      kmeshcnew(6,ielemnew) = face(1)
      kmeshcnew(7,ielemnew) = kmeshc(2)
      kmeshcnew(8,ielemnew) = edge(1,2)
      kmeshcnew(9,ielemnew) = face(6)

!     --- 5 e3a 6 f_3 9 f_2 4 e2b f_7

      ielemnew = ielemnew+1
      kmeshcnew(1,ielemnew) = kmeshc(5)
      kmeshcnew(2,ielemnew) = edge(3,1)
      kmeshcnew(3,ielemnew) = kmeshc(6)
      kmeshcnew(4,ielemnew) = face(3)
      kmeshcnew(5,ielemnew) = kmeshc(9)
      kmeshcnew(6,ielemnew) = face(2)
      kmeshcnew(7,ielemnew) = kmeshc(4)
      kmeshcnew(8,ielemnew) = edge(2,2)
      kmeshcnew(9,ielemnew) = face(7)

!     --- 7 e4a 8 f_4 9 f_3 6 e3b f_8

      ielemnew = ielemnew+1
      kmeshcnew(1,ielemnew) = kmeshc(7)
      kmeshcnew(2,ielemnew) = edge(4,1)
      kmeshcnew(3,ielemnew) = kmeshc(8)
      kmeshcnew(4,ielemnew) = face(4)
      kmeshcnew(5,ielemnew) = kmeshc(9)
      kmeshcnew(6,ielemnew) = face(3)
      kmeshcnew(7,ielemnew) = kmeshc(6)
      kmeshcnew(8,ielemnew) = edge(3,2)
      kmeshcnew(9,ielemnew) = face(8)

1000  call erclos ( 'mshrefkmeshc6e' )

      if ( debug ) then

!     --- Debug information

         call prinin1 ( kmeshcnew, 4, 9, 'kmeshcnew' )
         write(irefwr,*) 'End mshrefkmeshc6e'

      end if

      end
