      subroutine mshrefkmeshcv ( kmeshcnew, nodenumbers, lenkmeshc, nelemnew, &
                                 fillmap )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 18-06-2014 Extra parameters
!        version  1.3  date 11-06-2014 Extension with shape 21
!        version  1.2  date 10-02-2012 use getnpelmelgrp
!
!   copyright (c) 2003-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the new array kmeshc case that the complete mesh is refined
!     In this case we have volume elements, surface elements and possibly
!     line elements
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     topology
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshcnew(*), nodenumbers(*),nelemnew, lenkmeshc
      logical fillmap

!     fillmap        i    If true array imap must be filled
!     kmeshcnew      o    New kmesh part c
!     lenkmeshc     i/o  length of array kmeshc
!     nelemnew       i    Number of elements in new mesh
!     nodenumbers    i    Array containing the node numbers of the newly created
!                         nodes in the following way
!                         Pos. 1 to nedges node number of first node on edge
!                         Next nfaces positions node number of first node on
!                         face not stored before
!                         Next nelem positions node numbers of first extra node
!                         in element not stored before
!                         if elddimmax = 1 only the third part is used
!                         if elddimmax = 2 the second part is not used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nelprev, ielgrp, ikelmc, ikelmcnew, inpelm, nsub

!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     ikelmcnew      Starting address of next element in kmeshcnew
!     inpelm         number of points per element
!     nelprev        Number of elements stored in kmeshc at the start
!                    of the subroutine
!     nsub           Number of elements an element is subdivided into
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHREFKMESHC11 Fill the new array kmeshc for one element group in case
!                    that the complete mesh is refined
!                    Elements of shape 11, i.e. linear tetrahedrons
!     MSHREFKMESHC12 Fill the new array kmeshc for one element group in case
!                    that the complete mesh is refined
!                    Elements of shape 12, i.e. quadratic tetrahedrons
!     MSHREFKMESHC13 Fill the new array kmeshc for one element group in case
!                    that the complete mesh is refined
!                    Elements of shape13, i.e. trilinear hexahedrons
!     MSHREFKMESHC14 Fill the new array kmeshc for one element group in case
!                    that the complete mesh is refined
!                    Elements of shape 14, i.e. quadratic hexahedrons
!     MSHREFKMESHC21 Fill the new array kmeshc for one element group in case
!                    that the complete mesh is refined
!                    Elements of shape 21, i.e. linear prisms
!     MSHREFKMESHC22 Fill the new array kmeshc for one element group in case
!                    that the complete mesh is refined
!                    Elements of shape 22, i.e. quadratic prisms
!     MSHREFKMESHCL1 Fill the new array kmeshc for one element group in case
!                    that the complete mesh is refined
!                    Elements of shape 1, i.e. linear line elements
!     MSHREFKMESHCL2 Fill the new array kmeshc for one element group in case
!                    that the complete mesh is refined
!                    Elements of shape 2, i.e. quadratic line elements
!     MSHREFKMESHCS3 Fill the new array kmeshc for one element group in case
!                    that the complete mesh is refined
!                    Elements of shape 3, i.e. linear triangles
!     MSHREFKMESHCS4 Fill the new array kmeshc for one element group in case
!                    that the complete mesh is refined
!                    Elements of shape 4, i.e. quadratic triangles
!     MSHREFKMESHCS5 Fill the new array kmeshc for one element group in case
!                    that the complete mesh is refined
!                    Elements of shape 5, i.e. bi-linear quadrilaterals
!     MSHREFKMESHCS6 Fill the new array kmeshc for one element group in case
!                    that the complete mesh is refined
!                    Elements of shape 6, i.e. bi-quadratic quadrilaterals
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefkmeshcv' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefkmeshcv'

      end if
      if ( ierror/=0 ) go to 1000

!     --- initializations

      nelemnew = 0
      nelprev = 0
      ikelmcnew = 1

      ikelmc = 1
      do ielgrp = 1, nelgrp

!     --- ishape := shape number of element

         ishape = kmeshf(ielgrp)
         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm

!        --- for all elements in group do
!            fill kmeshcnew

         select case ( ishape )

         case (1)

!        --- ishape = 1, linear line elements

            call mshrefkmeshcl1 ( kmeshc(ikelmc), kmeshcnew(ikelmcnew), &
                                  inelem, nelprev, nmeshd, nodenumbers )
            nsub = 2

         case (2)

!        --- ishape = 2, quadratic line elements

            call mshrefkmeshcl2 ( kmeshc(ikelmc), kmeshcnew(ikelmcnew), &
                                  inelem, nelprev, nmeshd, nodenumbers )
            nsub = 2

         case (3)

!        --- ishape = 3, linear triangles

            call mshrefkmeshcs3 ( kmeshcnew(ikelmcnew), inelem, nelprev, &
                                  nodenumbers, nmesha, nmeshb, &
                                  nmeshg )
            nsub = 4

         case (4)

!        --- ishape = 4, quadratic triangles

            call mshrefkmeshcs4 ( kmeshcnew(ikelmcnew), inelem, nelprev, &
                                  nodenumbers, nmesha, nmeshb, &
                                  nmeshg, nedges )
            nsub = 4

         case (5)

!        --- ishape = 5, bi-linear quadrilaterals

            call mshrefkmeshcs5 ( kmeshcnew(ikelmcnew), inelem, nelprev, &
                                  nodenumbers, nmesha, nmeshb, &
                                  nmeshg, nedges )
            nsub = 4

         case (6)

!        --- ishape = 6, bi-quadratic quadrilaterals

            call mshrefkmeshcs6 ( kmeshcnew(ikelmcnew), inelem, nelprev, &
                                  nodenumbers, nmesha, nmeshb, &
                                  nmeshg, nedges )
            nsub = 4

         case (11)

!        --- ishape = 11, linear tetrahedrons

            call mshrefkmeshc11 ( kmeshc(ikelmc), kmeshcnew(ikelmcnew), &
                                  inelem, nelprev, nmeshd, nodenumbers )
            nsub = 8

         case (12)

!        --- ishape = 12

            call mshrefkmeshc12 ( kmeshc(ikelmc), kmeshcnew(ikelmcnew), &
                                  nelprev, nodenumbers, fillmap )
            nsub = 8

         case (13)

!        --- ishape = 13

            call mshrefkmeshc13 ( kmeshc(ikelmc), kmeshcnew(ikelmcnew), &
                                  nelprev, nodenumbers, fillmap )
            nsub = 8

         case (14)

!        --- ishape = 14

            call mshrefkmeshc14 ( kmeshc(ikelmc), kmeshcnew(ikelmcnew), &
                                  nelprev, nodenumbers, fillmap )
            nsub = 8

         case (21)

!        --- ishape = 21, linear prisms

            call mshrefkmeshc21 ( kmeshc(ikelmc), kmeshcnew(ikelmcnew), &
                                  inelem, nelprev, nmeshd, nodenumbers, &
                                  nmesha, nedges )
            nsub = 8

         case (22)

!        --- ishape = 22

            call mshrefkmeshc22 ( kmeshc(ikelmc), kmeshcnew(ikelmcnew), &
                                  nelprev, nodenumbers, fillmap )
            nsub = 8

         case ( -10001 )

!        --- ishape = -10001, connection element

            nsub = 0

         case default

!        --- shape not implemented

            call errint ( ishape, 1 )
            call errsub ( 800, 1, 0, 0 )
            go to 1000

         end select  ! case ( ishape )

         ikelmc = ikelmc+inelem*inpelm
         ikelmcnew = ikelmcnew+inelem*nsub*inpelm
         nelemnew = nelemnew+nsub*inelem
         nelprev = nelprev+inelem
         if ( nelgrp>1 ) kmeshg(ielgrp) = nsub*inelem
         if ( ikelmcnew-1>lenkmeshc ) then

!        --- Array too short

            call errint ( ikelmcnew-1, 1 )
            call errint ( lenkmeshc, 1 )
            call errchr ( 'kmeshc_new', 1 )
            call errsub ( 550, 2, 0, 1 )
            go to 1000

         end if
         if ( ierror/=0 ) go to 1000

      end do  ! ielgrp = 1, nelgrp
      lenkmeshc = ikelmcnew-1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefkmeshcv'

      end if
      call erclos ( 'mshrefkmeshcv' )

      end
