      subroutine mshrefkmeshm ( kmeshmnew, lenkmeshm, &
                                nodenumbers, eldimmax, npoints_old )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 18-02-2004 Extra parameters
!        version  1.0  date 15-10-2002
!
!   copyright (c) 2002-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill nodes in kmesh part m (curves) in case of a mesh that is
!     refined
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     mesh_generation
!     curve
!     node
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshmnew(*), lenkmeshm, nodenumbers(*), &
              eldimmax, npoints_old

!     eldimmax       i    Maximum dimension of the elements, i.e.
!                         3 for volume elements
!                         2 for surface elements
!                         1 for line elements
!     kmeshmnew      o    New array kmeshm
!     lenkmeshm     i/o   Length of array kmeshm
!     nodenumbers    i    Array containing the node numbers of the newly created
!                         nodes in the following way
!                         Pos. 1 to nedges node number of first node on edge
!                         Next nfaces positions node number of first node on
!                         face not stored before
!                         Next nelem positions node numbers of first extra node
!                         in element not stored before
!                         if elddimmax = 1 only the third part is used
!                         if elddimmax = 2 the second part is not used
!     npoints_old    i    Number of nodes in old mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ist, ish, nsubcv, icurnr, ilold, &
              j, jcurnr, ipos3, i, iwork(ncurvs)

!     i              Counting variable
!     icurnr         Curve sequence number
!     ilold          old number of nodes on line
!     ipos3          position in curve array
!     ish            help variable to store node number
!     ist            node number where to start
!     iwork          work array of length ncurvs
!                    if curve i is filled iwork(i) = 1 ortherwise 0
!     j              Counting variable
!     jcurnr         |icurnr| i.e. absolute value of curve sequence number
!     nsubcv         Number of subcurves of a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH414         Determine whether curves are filled in
!     MSHFILLMSUB    Fill kmesh part m for composite curves
!     MSHREFKMESHM1D Copy information of old curve as stored in kmesh part m
!                    into new curve extended with intermediate point in case a
!                    mesh is refined completely (1D situation)
!     MSHREFKMESHMCR Copy information of old curve as stored in kmesh part m
!                    into new curve extended with intermediate point in case a
!                    mesh is refined completely
!     PR0008         Print information of curves
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefkmeshm' )
      debug = .false.

      kmeshmnew(1) = kmeshm(1)
      kmeshmnew(2) = kmeshm(2)
      kmeshmnew(4) = kmeshm(4)
      kmeshmnew(5) = 1
      ninner = kmeshm(1)
      ist = kmeshm(2)
      ish = kmeshmnew(2)
      nsubcv = 0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefkmeshm'
         call pr0008 ( kmeshm, ncurvs )

      end if

!     --- Fill array iwork with information of the curves
!         when curve i has been filled then iwork(i) = 0 else iwork(i) = 1

      call msh414 ( kmeshm, ncurvs, iwork )
      if ( debug ) call prinin ( iwork, ncurvs, 'iwork' )

!     --- Check whether coordinates of points on curves have to be
!         adjusted
!         First kmeshm part 2 must be filled

      do icurnr = 1, ncurvs
         if ( kelmq>0 ) nsubcv = kmeshq(icurnr+1)-kmeshq(icurnr)
         if ( nsubcv==0 ) then

!        --- No subcurves, hence single curve
!            Compute number of nodes along curves

            ilold = kmeshm(icurnr+5) - kmeshm(icurnr+4)
            if ( iwork(icurnr)/=1 ) ilold = max(ilold*2-1,0)

            kmeshmnew(icurnr+5) = ilold

         end if

      end do

      if ( kelmq>0 ) then

!     --- kelmq > 0, fill also the composite curves

         do icurnr = 1, ncurvs
            nsubcv = kmeshq(icurnr+1)-kmeshq(icurnr)
            if ( nsubcv>0 ) then

!           --- nsubcv > 0, there are subcurves
!               Fill by adding the subcurves

               ilold = 0
               do j = 1, nsubcv
                  jcurnr = abs(kmeshq(kmeshq(icurnr)+j))
                  ilold = ilold+kmeshmnew(jcurnr+5)
               end do
               kmeshmnew(icurnr+5) = ilold

            end if ! nsubcv>0

         end do  ! icurnr = 1, ncurvs

      end if  ! kelmq>0
      if ( debug ) then

!     --- Debug information

         call prinin ( kmeshmnew(6), ncurvs, &
           'Number of nodes in curves after refinement' )

      end if

!     --- Acumulate kmeshm part 2

      kmeshmnew(5) = 1
      do icurnr = 1, ncurvs
         kmeshmnew(5+icurnr) = &
            kmeshmnew(5+icurnr)+kmeshmnew(4+icurnr)
      end do
      if ( debug ) &
         call prinin ( kmeshmnew(5), ncurvs+1, 'kmeshm part 2' )
      length = kmeshmnew(5+ncurvs)+kmeshm(2)-1
      kmeshmnew(3) = length

      if ( length>lenkmeshm ) then

         call errint ( length, 1 )
         call errint ( lenkmeshm, 2 )
         call errchr ( 'kmeshmnew', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

      do icurnr = 1, ncurvs

         if ( kelmq>0 ) nsubcv = kmeshq(icurnr+1)-kmeshq(icurnr)

         if ( nsubcv==0 ) then

!        --- Single curve

            ilold = kmeshm(icurnr+5) - kmeshm(icurnr+4)
            ist = kmeshm(2)+kmeshm(icurnr+4)-1
            ish = kmeshm(2)+kmeshmnew(icurnr+4)-1
            if ( debug ) write(irefwr,*) 'icurnr, ilold, ist, ish ', &
               icurnr, ilold, ist, ish
            if ( iwork(icurnr)==1 ) then

!           --- icurnr has not been filled, special situation

               do i = 0, ilold-1
                  kmeshmnew(ish+i) = kmeshm(ist+i)
               end do

            else if ( eldimmax>1 ) then

!           --- iwork(icurnr) = 0 curve has been filled

               call mshrefkmeshmcr ( kmeshm(ist), kmeshmnew(ish), ilold, &
                                     nodenumbers, nmeshl, nmeshc, &
                                     nedges )

            else

!           --- iwork(icurnr) = 0 curve has been filled (1d case)

               call mshrefkmeshm1d ( kmeshd, kmeshm(ist), &
                                     kmeshmnew(ish), nelem, ilold, &
                                     nodenumbers, npoints_old )

            end if

         end if   ! single curve

      end do  ! icurnr = 1, ncurvs

!     --- Fill composite curves

      if ( kelmq>0 ) call mshfillmsub ( ncurvs, kmeshmnew, kmeshq )

!     --- Raise first positions and fill part 4

      length = kmeshmnew(3) + ninner

      if ( length>lenkmeshm ) then

         call errint ( length, 1 )
         call errint ( lenkmeshm, 2 )
         call errchr ( 'kmeshmnew', 1 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

      ipos3 = kmeshmnew(3)
      kmeshmnew(ipos3) = 0

      do i = 1, ninner
         kmeshmnew(ipos3+i-1) = kmeshm(kmeshm(3)+i-1)
      end do
      if ( debug ) call pr0008 ( kmeshmnew, ncurvs )

      lenkmeshm = length

1000  call erclos ( 'mshrefkmeshm' )

      end
