      subroutine mshrefkmeshn ( kmeshnnew, lenkmeshn, nodenumbers )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-09-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill nodes in kmesh part n (surfaces) in case of a mesh that is
!     refined
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     mesh_generation
!     surface
!     node
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshnnew(*), lenkmeshn, nodenumbers(*)

!     kmeshnnew      o    New array kmeshn
!     lenkmeshn     i/o   Length of array kmeshn
!     nodenumbers    i    Array containing the node numbers of the newly created
!                         nodes in the following way
!                         Pos. 1 to nedges node number of first node on edge
!                         Next nfaces positions node number of first node on
!                         face not stored before
!                         Next nelem positions node numbers of first extra node
!                         in element not stored before
!                         if elddimmax = 1 only the third part is used
!                         if elddimmax = 2 the second part is not used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isurnr, inpelm, istart, istartnew, k, &
              iwork(nfaces) , inode, ipstart, numfaces, j, iface, jface, &
              iielem, edge(8), iedge, numedges, idum(1)
      double precision dum(1)

!     dum            General double precision dummy variable
!     edge           Contains the edge node numbers on the 4 edges (2 per edge)
!     i              Counting variable
!     idum           General dummy variable
!     iedge          edge sequence number
!     iface          Face sequence number
!     iielem         Relative element number in element group
!     inode          Node sequence number
!     inpelm         number of points per element
!     ipstart        Starting address in array
!     istart         First position used at a part
!     istartnew      New starting address in array
!     isurnr         Surface sequence number
!     iwork          integer work array
!     j              Counting variable
!     jface          Face sequence number
!     k              Counting variable
!     numedges       number of edges
!     numfaces       number of faces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHREFKMESHNS  Fill part of kmeshn corresponding to one surface element
!     PR0009         Print information of surfaces
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
!    2463   No face found corresponding to surface element
! **********************************************************************
!
!                       PSEUDO CODE
!
!     for all surfaces isurnr do
!        inpelm := number of nodes in elements
!        inelem := number of elements in surface
!        for all elements in isurnr do
!           find face corresponding to element
!           copy new information to kmeshnnew
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefkmeshn' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefkmeshn'
         write(irefwr,*) 'lenkmeshn ', lenkmeshn
         call pr0009 ( 0, kmeshn, dum, idum, 0, 0, nsurfs )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Set constants

      istart = 2*nsurfs+1
      istartnew = istart

!     --- Clear array iwork

      do i = 1, nfaces
         iwork(i) = 0
      end do

!     --- for all surfaces isurnr do

      do isurnr = 1, nsurfs

!     --- inpelm := number of nodes in elements
!         inelem := number of elements in surface

         inpelm = kmeshn(isurnr)
         inelem = kmeshn(nsurfs+isurnr)
         kmeshnnew(isurnr) = inpelm
         kmeshnnew(nsurfs+isurnr) = 4*inelem

!        --- Clear part of kmeshnnew

         do i = 0, 4*inelem*inpelm-1
            kmeshnnew(istartnew+i) = 0
         end do

!        --- for all elements in isurnr do

         do iielem = 1, inelem

!        --- Check if element has been filled

            if ( kmeshn(istart)>0 ) then

!           --- element contains nodes
!               find face corresponding to element
!               copy new information to kmeshnnew

               iface = 0
               inode = kmeshn(istart)      ! first node in element
               ipstart = nmeshk(inode)     ! start position of inode in nmeshk
               numfaces = nmeshk(ipstart)  ! number of faces connected to inode

!              --- set iwork 1 for all faces connected to node

               do j = 1, numfaces
                  iwork(nmeshk(ipstart+j)) = 1
               end do

!              --- For all other nodes in the element raise iwork if not 0
!                  untill face number found

               do j = 2, inpelm

                  inode = kmeshn(istart+j-1) ! node in element
                  ipstart = nmeshk(inode)    ! start position of inode in nmeshk
                  numfaces = nmeshk(ipstart) ! number of faces connected to node

                  do k = 1, numfaces

                     jface = nmeshk(ipstart+k)
                     if ( iwork(jface)>=j-1 ) then

!                    --- face corresponds also to node 1

                        iwork(jface) = iwork(jface)+1
                        if ( j==inpelm ) iface = jface

                     end if

                  end do  ! j = 1, numfaces

               end do  ! j = 2, inpelm
               if ( debug ) write(irefwr,*) 'iface ', iface

               if ( iface==0 ) then

!              --- error no face found

                  call errint ( isurnr, 2 )
                  call errint ( iielem, 1 )
                  call errsub ( 2463, 2, 0, 0 )
                  go to 1000

               end if

!              --- Store edges in array edge

               ipstart = nmeshb(iface)
               numedges = nmeshb(ipstart)
               if ( inpelm<6 ) then

!              --- inpelm < 6, i.e. linear element

                  do i = 1, numedges
                     iedge = abs(nmeshb(ipstart+i))
                     edge(i) = nodenumbers(iedge)
                  end do

               else

!              --- inpelm >= 6, i.e. quadratic element, each edge contains two
!                  nodes

                  do i = 1, numedges
                     iedge = nmeshb(ipstart+i)
                     if ( iedge>0 ) then

!                    --- edge in positive direction

                        edge(i) = nodenumbers(iedge)
                        edge(i+numedges) = nodenumbers(iedge)+1

                     else

!                    --- edge in negative direction

                        edge(i) = nodenumbers(-iedge)+1
                        edge(i+numedges) = nodenumbers(-iedge)

                     end if

                  end do  ! i = 1, numedges

               end if
               ipstart = nmeshg(iface)
               iface = nodenumbers(nedges+iface)
               call mshrefkmeshns ( kmeshn(istart), nmeshg(ipstart+1), &
                                    edge, inpelm, numedges, &
                                    kmeshnnew(istartnew), iface )

               if ( debug ) then

!              --- debug information

                  write(irefwr,*) 'isurnr, iielem, iface ', &
                                   isurnr, iielem, iface
                  call prinin ( kmeshn(istart), inpelm, 'element' )
                  call prinin ( nmeshg(ipstart+1), inpelm, 'face' )
                  call prinin1 ( kmeshnnew(istartnew), 4, inpelm, &
                                 'element_new' )

               end if

!              --- clear iwork

               inode = kmeshn(istart)      ! first node in element
               ipstart = nmeshk(inode)     ! start position of inode in nmeshk
               numfaces = nmeshk(ipstart)  ! number of faces connected to inode
               do j = 1, numfaces
                  iwork(nmeshk(ipstart+j)) = 0
               end do

            end if  ! ( filled )
            istart = istart+inpelm
            istartnew = istartnew+4*inpelm

            if ( istartnew-1>lenkmeshn ) then

               call errint ( istartnew-1, 1 )
               call errint ( lenkmeshn, 2 )
               call errchr ( 'kmeshnnew', 1 )
               call errsub ( 550, 2, 0, 1 )
               go to 1000

            end if

         end do  ! iielem = 1, inelem

      end do  ! i = 1, nsurfs
      lenkmeshn = istartnew-1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'lenkmeshn ', lenkmeshn
         call pr0009 ( 0, kmeshnnew, dum, idum, 0, 0, nsurfs )
         write(irefwr,*) 'End mshrefkmeshn'

      end if
      call erclos ( 'mshrefkmeshn' )

      end
