      subroutine mshrefkmeshns ( element, face, edge, numnodes, &
                                 numedges, kmeshnnew, iface )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-09-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill part of kmeshn corresponding to one surface element
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     mesh_generation
!     surface
!     node
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numnodes, numedges, element(numnodes), face(numnodes), &
              edge(*), kmeshnnew(*), iface

!     edge           i    Contains the edge node numbers on the edges
!     element        i    Contains the node numbers in the original element
!     face           i    Contains the face node numbers on the faces
!     iface          i    Face sequence number
!     kmeshnnew      o    New array kmeshn
!     numedges       i    number of edges
!     numnodes       i    Number of nodal points in array nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inode1, inode2, i, ipos, idir, ind3(6), ind4(8), &
              edgecop(8), ielemnew, faces(8)

!     edgecop        Copy of array edge rotated so that face and element match
!     faces          Copy of array face rotated so that face and element match
!     i              Counting variable
!     idir           Direction of element compared to face
!     ielemnew       Sequence number of element in new mesh
!     ind3           Permutaion array for the numbers 1 2 3
!     ind4           Permutaion array for the numbers 1 2 3 4
!     inode1         First node
!     inode2         Second node
!     ipos           Position in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHREFKMESHC3E Refine one element in case of a complete refinement
!                    Element of shape 3, i.e. linear triangles
!     MSHREFKMESHC4E Refine one element in case of a complete refinement
!                    Element of shape 4, i.e. quadratic triangles
!     MSHREFKMESHC5E Refine one element in case of a complete refinement
!                    Element of shape 5, i.e. bi-linear quadrilaterals
!     MSHREFKMESHC6E Refine one element in case of a complete refinement
!                    Element of shape 6, i.e. bi-quadratic quadrilaterals
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ind3 / 1, 2, 3, 1, 2, 3 /, &
           ind4 / 1, 2, 3, 4, 1, 2, 3, 4 /
! ======================================================================
!
      call eropen ( 'mshrefkmeshns' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefkmeshns'
         write(irefwr,*) 'numnodes, numedges ', &
                          numnodes, numedges
         call prinin ( element, numnodes, 'element' )
         call prinin ( face, numnodes, 'face' )
         if ( numnodes.lt.6 ) then
            call prinin ( edge, numedges, 'edge' )
         else
            call prinin1 ( edge, 2, numedges, 'edge' )
         end if

      end if
      if ( ierror.ne.0 ) go to 1000
      ielemnew = 0

!     --- Find position of first node of element in face

      inode1 = element(1)
      do i = 1, numnodes
         if ( face(i).eq.inode1 ) then
            ipos = i
            go to 100
         end if
      end do

!     --- Find direction of face compared to element

100   inode2 = element(2)
      if ( ipos.lt.numnodes ) then

!     --- first node of element not the last node in face

         if ( face(ipos+1).eq.inode2 ) then

!        --- second node follows first node, same direction

            idir = 1

         else

!        --- second node does not follow first node, reversed direction

            idir = -1

         end if

      else

!     --- first node of element is the last node in face

         if ( face(1).eq.inode2 ) then

!        --- second node follows first node, same direction

            idir = 1

         else

!        --- second node does not follow first node, reversed direction

            idir = -1

         end if

      end if  ! ( ipos.lt.numnodes )
      if ( debug ) write(irefwr,*) 'ipos, idir ', ipos, idir

      if ( numnodes.eq.3 ) then

!     --- numnodes = 3, linear triangle
!         Rearrange edges depending on ipos and idir

         if ( idir.eq.1 ) then

!        --- standard direction

            do i = 1, 3
               edgecop(i) = edge(ind3(i+ipos-1))
            end do

         else  ! ( idir.eq.-1 )

!        --- reversed direction

            do i = 1, 3
               edgecop(4-i) = edge(ind3(i+ipos-1))
            end do

         end if  ! ( idir.eq.1 )
         if ( debug ) call prinin ( edgecop, numedges, 'edgecop' )

         call mshrefkmeshc3e ( element, kmeshnnew, ielemnew, edgecop )

      else if ( numnodes.eq.6 ) then

!     --- numnodes=6, quadratic triangle, 3 face nodes
!         Rearrange edges depending on ipos and idir

         ipos = ipos/2+1
         if ( idir.eq.1 ) then

!        --- standard direction

            do i = 1, 3
               edgecop(i) = edge(ind3(i+ipos-1))
               edgecop(i+3) = edge(ind3(i+ipos-1)+3)
            end do
            faces(1) = iface+ind3(ipos)-1
            faces(2) = iface+ind3(ipos+1)-1
            faces(3) = iface+ind3(ipos+2)-1

         else  ! ( idir.eq.-1 )

!        --- reversed direction

            do i = 1, 3
               edgecop(4-i) = edge(ind3(i+ipos-1)+3)
               edgecop(4-i+3) = edge(ind3(i+ipos-1))
            end do
            faces(ind3(ipos)) = iface
            faces(ind3(ipos+1)) = iface+2
            faces(ind3(ipos+2)) = iface+1

         end if  ! ( idir.eq.1 )
         if ( debug ) then
            call prinin1 ( edgecop, 2, numedges, 'edgecop' )
            call prinin ( faces, 3, 'faces' )
         end if

         call mshrefkmeshc4e ( element, kmeshnnew, ielemnew, edgecop, &
                               faces )

      else if ( numnodes.eq.4 .and. numedges.eq.4 ) then

!     --- numnodes=4, numedges=4, quadrilateral, 1 face node
!         Rearrange edges depending on ipos and idir

         if ( idir.eq.1 ) then

!        --- standard direction

            do i = 1, 4
               edgecop(i) = edge(ind4(i+ipos-1))
            end do

         else  ! ( idir.eq.-1 )

!        --- reversed direction

            do i = 1, 4
               edgecop(5-i) = edge(ind4(i+ipos-1))
            end do

         end if  ! ( idir.eq.1 )
         if ( debug ) then
            call prinin ( edgecop, numedges, 'edgecop' )
            write(irefwr,*) 'iface, ielemnew ', iface, ielemnew
         end if

         call mshrefkmeshc5e ( element, kmeshnnew, ielemnew, edgecop, &
                               iface )

      else if ( numnodes.eq.9 ) then

!     --- numnodes=9, bi-quadratic quadrilateral, 8 face nodes
!         Rearrange edges depending on ipos and idir

         ipos = ipos/2+1
         if ( idir.eq.1 ) then

!        --- standard direction

            do i = 1, 4
               edgecop(i) = edge(ind4(i+ipos-1))
               edgecop(i+4) = edge(ind4(i+ipos-1)+4)
            end do
            faces(1) = iface+ind4(ipos)-1
            faces(2) = iface+ind4(ipos+1)-1
            faces(3) = iface+ind4(ipos+2)-1
            faces(4) = iface+ind4(ipos+3)-1
            faces(5) = iface+ind4(ipos)+3
            faces(6) = iface+ind4(ipos+1)+3
            faces(7) = iface+ind4(ipos+2)+3
            faces(8) = iface+ind4(ipos+3)+3

         else  ! ( idir.eq.-1 )

!        --- reversed direction

            do i = 1, 4
               edgecop(5-i) = edge(ind4(i+ipos-1)+4)
               edgecop(5-i+4) = edge(ind4(i+ipos-1))
            end do
            faces(ind4(ipos+3)) = iface
            faces(ind4(ipos+2)) = iface+1
            faces(ind4(ipos+1)) = iface+2
            faces(ind4(ipos)) = iface+3
            faces(ind4(ipos)+4) = iface+4
            faces(ind4(ipos+3)+4) = iface+5
            faces(ind4(ipos+2)+4) = iface+6
            faces(ind4(ipos+1)+4) = iface+7

         end if  ! ( idir.eq.1 )
         if ( debug ) then
            call prinin1 ( edgecop, 2, numedges, 'edgecop' )
            call prinin1 ( faces, 2, 4, 'faces' )
         end if

         call mshrefkmeshc6e ( element, kmeshnnew, ielemnew, edgecop, &
                               faces )

      else if ( numnodes.eq.7 ) then

!     --- numnodes=7, extended quadratic triangle, 3 face nodes

         call errint ( 7, 1 )
         call errsub ( 800, 1, 0, 0 )
         go to 1000

      else if ( numnodes.eq.10 ) then

!     --- numnodes=10, not yet implemented

         call errint ( 8, 1 )
         call errsub ( 800, 1, 0, 0 )
         go to 1000

      else if ( numnodes.eq.5 ) then

!     --- numnodes=5, extended quadrilateral, 4 face nodes

         call errint ( 9, 1 )
         call errsub ( 800, 1, 0, 0 )
         go to 1000

      else

!     --- numnodes=4, extended triangle, 3 face nodes

         call errint ( 10, 1 )
         call errsub ( 800, 1, 0, 0 )
         go to 1000

      end if  ! ( numnodes.eq.3 )

1000  call erclos ( 'mshrefkmeshns' )

      if ( debug ) then

!     --- Debug information

         call prinin1 ( kmeshnnew, 4, numnodes, 'elements/new' )
         write(irefwr,*) 'End mshrefkmeshns'

      end if

      end
