      subroutine mshrefnumnods ( npoints_new, nodenumbers, eldimmax )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 11-06-2014 Extension with shape 21
!        version  1.0  date 14-09-2003
!
!   copyright (c) 2003-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the number of nodes in a mesh that is refined exactly one
!     times
!     Fill array nodenumbers with the first new node number along the edge
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     node
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoints_new, nodenumbers(*), eldimmax

!     eldimmax       i    Maximum dimension of the elements, i.e.
!                         3 for volume elements
!                         2 for surface elements
!                         1 for line elements
!     nodenumbers    o    Array containing the node numbers of the newly created
!                         nodes in the following way
!                         Pos. 1 to nedges node number of first node on edge
!                         Next nfaces positions node number of first node on
!                         face not stored before
!                         Next nelem positions node numbers of first extra node
!                         in element not stored before
!                         if elddimmax = 1 only the third part is used
!                         if elddimmax = 2 the second part is not used
!     npoints_new    i    Number of nodes in new mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer npoints_old, first_node, iedge, ipstart, numnodes, ielem, &
              ielgrp, iface, iielem, numedges, numfacenods

!     first_node     Node number of next node to be created
!     iedge          edge sequence number
!     ielem          Element number
!     ielgrp         element group number
!     iface          Face sequence number
!     iielem         Relative element number in element group
!     ipstart        Starting address in array
!     npoints_old    Number of nodes in old mesh
!     numedges       number of edges
!     numfacenods    Number of face nodes
!     numnodes       Number of nodal points in array nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     first_node := npoints_old+1
!     if ( eldimmax = 1 ) then
!        For all element groups do
!           ishape := shape number of element
!           for all elements in group do
!              ielem := element sequence number
!              nodenumbers(ielem) := first_node
!              if ( ishape=1 ) then
!                 first_node := first_node+1
!              else if ( ishape=2 ) then
!                 first_node := first_node+2
!              else if ( ishape#-10001 ) then
!                 error
!     else ( eldimmax>1 )
!        for iedge := 1 (1) number_of_edges do
!           numnodes := number of nodes along edge iedge
!           nodenumbers(iedge) := first_node
!           first_node := first_node+numnodes-1
!        if ( eldimmax = 2 ) then
!           For all element groups do
!              ishape := shape number of element
!              if ( ishape>2 ) then
!                 for all elements in group do
!                    ielem := nedges + element sequence number
!                    nodenumbers(ielem) := first_node
!                    if ( ishape=4 ) then
!                       first_node := first_node+3
!                    else if ( ishape=5 ) then
!                       first_node := first_node+1
!                    else if ( ishape=6 ) then
!                       first_node := first_node+8
!                    else if ( ishape=7 ) then
!                       first_node := first_node+6
!                    else if ( ishape=8 ) then
!                       error
!                    else if ( ishape=9 ) then
!                       first_node := first_node+4
!                    else if ( ishape=10 ) then
!                       first_node := first_node+3
!        else ( eldimmax = 3 )
!           for iface := 1 (1) nfaces do
!              numedges := number of edges in iface
!              numnodes := number of nodes in iface
!              ielem := iface+nedges
!              nodenumbers(ielem) := first_node
!              if ( numnodes=6 ) then
!                 first_node := first_node+3
!              else if ( numnodes=4 and numedges=4 ) then
!                 first_node := first_node+1
!              else if ( numnodes=9 ) then
!                 first_node := first_node+8
!              else if ( numnodes=7 ) then
!                 first_node := first_node+6
!              else if ( numnodes=10 ) then
!                 error
!              else if ( numnodes=5 ) then
!                 first_node := first_node+4
!              else if ( numnodes=4 ) then
!                 first_node := first_node+3
!           For all element groups do
!              ishape := shape number of element
!              if ( ishape>10 ) then
!                 for all elements in group do
!                    ielem := nedges+nfaces + element sequence number
!                    nodenumbers(ielem) := first_node
!                    if ( ishape=11 ) then
!                    else if ( ishape=12 ) then
!                       first_node := first_node+1
!                    else if ( ishape=13 ) then
!                       first_node := first_node+1
!                    else if ( ishape=14 ) then
!                       first_node := first_node+26
!                    else if ( ishape=15 ) then
!                    else if ( ishape=16 ) then
!                    else if ( ishape=17 ) then
!                    else if ( ishape=18 ) then
!                    else if ( ishape=21 ) then
!                    else if ( ishape=22 ) then
!                    else if ( ishape<50 ) then
!                       error
!     npoints_new := first_node-1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefnumnods' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefnumnods'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- first_node := npoints_old+1

      npoints_old = npoint
      inelem = nelem
      ielem = 0
      first_node = npoints_old+1
      if ( debug ) write(irefwr,1) 'npoints_old, eldimmax', &
                                    npoints_old, eldimmax

      if ( eldimmax==1 ) then

!     --- eldimmax = 1, hence line elements only
!         For all element groups do

         do ielgrp = 1, nelgrp

!        --- ishape := shape number of element

            ishape = kmeshf(ielgrp)
            if ( kelmg>0 ) inelem = kmeshg(ielgrp)

            if ( ishape>=1 .and. ishape<=2 ) then

!           --- ishape = 1 or 2 linear or quadratic line element
!               for all elements in group do

               do iielem = 1, inelem

!              --- ielem := element sequence number
!                  nodenumbers(ielem) := first_node

                  ielem = ielem+1
                  nodenumbers(ielem) = first_node
                  first_node = first_node+ishape

               end do  ! iielem = 1, inelem

            else if ( ishape/=-10001 ) then

!           --- ishape has other value: not implemented

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )
               go to 1000

            end if  ! ( ishape==1 ) then

         end do  ! ielgrp = 1, nelgrp

      else  ! ( eldimmax>1 )

!     --- eldimmax>1, i.e. surface and/or volume elements
!         for iedge := 1 (1) number_of_edges do

         do iedge = 1, nedges

!        --- numnodes := number of nodes along edge iedge
!            nodenumbers(iedge) := first_node
!            first_node := first_node+numnodes-1

            ipstart = nmeshc(iedge)
            numnodes = nmeshc(ipstart)
            nodenumbers(iedge) = first_node
            first_node = first_node+numnodes-1

         end do  ! iedge = 1, nedges

         ielem = nedges
         if ( debug ) &
            write(irefwr,*) 'nedges, first_node ',nedges, first_node

         if ( eldimmax==2 ) then

!        --- eldimmax = 2, surface elements
!            For all element groups do

            do ielgrp = 1, nelgrp

!           --- ishape := shape number of element

               ishape = kmeshf(ielgrp)
               if ( kelmg>0 ) inelem = kmeshg(ielgrp)

               if ( debug ) &
                  write(irefwr,1) 'ielgrp, ishape', ielgrp, ishape

               select case (ishape)
               case(-10001:3)

!              --- ishape=3, linear triangle, 0 face nodes

                  numfacenods = 0

               case(4)

!              --- ishape=4, quadratic triangle, 3 face nodes

                  numfacenods = 3

               case(5)

!              --- ishape=5, quadrilateral, 1 face node

                  numfacenods = 1

               case(6)

!              --- ishape=6, bi-quadratic quadrilateral, 8 face nodes

                  numfacenods = 8

               case(7)

!              --- ishape=7, extended quadratic triangle, 6 face nodes

                  numfacenods = 6

               case(9)

!              --- ishape=9, extended quadrilateral, 4 face nodes

                  numfacenods = 4

               case(10)

!              --- ishape=10, extended triangle, 3 face nodes

                  numfacenods = 3

               case default

!              --- ishape=8 not implemented

                  call errint ( ishape, 1 )
                  call errsub ( 800, 1, 0, 0 )
                  go to 1000

               end select  ! case (ishape)

!              --- for all elements in group do

               do iielem = 1, inelem

!              --- ielem := element sequence number
!                  nodenumbers(ielem) := first_node

                  ielem = ielem+1
                  nodenumbers(ielem) = first_node
                  first_node = first_node+numfacenods

               end do  ! iielem = 1, inelem

            end do  ! ielgrp = 1, nelgrp

         else  ! ( eldimmax==3 )

!        --- eldimmax, volume elements
!            for iface := 1 (1) nfaces do

            do iface = 1, nfaces

!           --- numedges := number of edges in iface
!               numnodes := number of nodes in iface
!               ielem := iface+nedges
!               nodenumbers(ielem) := first_node

               ipstart = nmeshb(iface)
               numedges = nmeshb(ipstart)
               ipstart = nmeshg(iface)
               numnodes = nmeshg(ipstart)
               ielem = ielem+1
               nodenumbers(ielem) = first_node
               if ( numnodes==6 ) then

!              --- numnodes=6, quadratic triangle, 3 face nodes

                  first_node = first_node+3

               else if ( numnodes==4 .and. numedges==4 ) then

!              --- numnodes=4, numedges=4, quadrilateral, 1 face node

                  first_node = first_node+1

               else if ( numnodes==9 ) then

!              --- numnodes=9, bi-quadratic quadrilateral, 8 face nodes

                  first_node = first_node+8

               else if ( numnodes==7 ) then

!              --- numnodes=7, extended quadratic triangle, 3 face nodes

                  first_node = first_node+3

               else if ( numnodes==10 ) then

!              --- numnodes=10, not yet implemented

                  call errint ( 8, 1 )
                  call errsub ( 800, 1, 0, 0 )
                  go to 1000

               else if ( numnodes==5 ) then

!              --- numnodes=5, extended quadrilateral, 4 face nodes

                  first_node = first_node+4

               else if ( numnodes==4 ) then

!              --- numnodes=4, extended triangle, 3 face nodes

                  first_node = first_node+3

               end if  ! ( numnodes==6 )

            end do  ! iface = 1, nfaces

            ielem = nedges+nfaces

!           --- For all element groups do

            do ielgrp = 1, nelgrp

!           --- ishape := shape number of element

               ishape = kmeshf(ielgrp)
               if ( kelmg>0 ) inelem = kmeshg(ielgrp)

!              --- for all elements in group do

               do iielem = 1, inelem

!              --- ielem := element sequence number
!                  nodenumbers(ielem) := first_node

                  ielem = ielem+1
                  nodenumbers(ielem) = first_node
                  select case ( ishape )
                  case (-10001:11, 21)

!                 --- ishape=11, linear tetrahedron or prism no action

                  case (12:13)

!                 --- ishape=12, quadratic tetrahedron, or
!                     ishape=13, tri-linear hexahedron, 1 volume node
!                     first_node := first_node+1

                     first_node = first_node+1

                  case (14)

!                 --- ishape=14, tri-quadratic hexahedron, 26 volume nodes

                     first_node = first_node+26

                  case (22)

!                 --- ishape=22, quadratic prism, 9 volume nodes

                     first_node = first_node+9

                  case default

!                 --- ishape>14, not yet implemented

                     call errint ( ishape, 1 )
                     call errsub ( 800, 1, 0, 0 )
                     go to 1000

                  end select  ! case ( ishape )

               end do  ! iielem = 1, inelem

            end do  ! ielgrp = 1, nelgrp

         end if  ! ( eldimmax==2 )

      end if  ! ( eldimmax==1 )

!     --- npoints_new := first_node-1

      npoints_new = first_node-1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefnumnods'

      end if
      call erclos ( 'mshrefnumnods' )

      end
