      subroutine mshreftopol ( kmeshcnew, eldimmax, &
                               nodenumbers, lenkmeshc, lenkmeshm, &
                               lenkmeshn, kmeshmnew, kmeshnnew, &
                               npoints_old, fillmap )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 18-02-2004 Extra parameters
!        version  1.0  date 20-09-2003
!
!   copyright (c) 2003-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill new topology of mesh into arrays kmeshcnew, kmeshmnew and kmeshnnew
!     in the case that the mesh is refined in one global refine step
!     It concerns the elements, curves and surfaces
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
!     topology
!     curve
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshcnew(*), eldimmax, &
              nodenumbers(*), lenkmeshc, lenkmeshm, lenkmeshn, &
              kmeshmnew(*), kmeshnnew(*),npoints_old
      logical fillmap

!     eldimmax       i    Maximum dimension of the elements, i.e.
!                         3 for volume elements
!                         2 for surface elements
!                         1 for line elements
!     fillmap        i    If true array imap must be filled
!     kmeshcnew      o    New kmesh part c
!     kmeshmnew      o    New array kmeshm
!     kmeshnnew      o    New array kmeshn
!     lenkmeshc     i/o   length of array kmeshc
!     lenkmeshm     i/o   Length of array kmeshm
!     lenkmeshn     i/o   Length of array kmeshn
!     nodenumbers    i    Array containing the node numbers of the newly created
!                         nodes in the following way
!                         Pos. 1 to nedges node number of first node on edge
!                         Next nfaces positions node number of first node on
!                         face not stored before
!                         Next nelem positions node numbers of first extra node
!                         in element not stored before
!                         if elddimmax = 1 only the third part is used
!                         if elddimmax = 2 the second part is not used
!     npoints_old    i    Number of nodes in old mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHREFKMESHC   Fill array kmeshc after refinement
!     MSHREFKMESHM   Fill array kmeshm after refinement
!     MSHREFKMESHN   Fill array kmeshn after refinement
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Refine kmeshc, put in kmeshcnew
!     Fill nelem and refill kmesh part g
!     Recompute kmeshm
!     Recompute kmeshn
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshreftopol' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshreftopol'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Refine kmeshc, put in kmeshcnew
!         Fill nelem and refill kmesh part g

      call mshrefkmeshc ( kmeshcnew, eldimmax, nodenumbers, lenkmeshc, &
                          fillmap )
      if ( ierror/=0 ) go to 1000

!     --- Recompute kmeshm

      call mshrefkmeshm ( kmeshmnew, lenkmeshm, &
                          nodenumbers, eldimmax, npoints_old )
      if ( ierror/=0 ) go to 1000

      if ( eldimmax>2 ) then

!     --- Recompute kmeshn

         call mshrefkmeshn ( kmeshnnew, lenkmeshn, nodenumbers )
         if ( ierror/=0 ) go to 1000

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshreftopol'

      end if
      call erclos ( 'mshreftopol' )

      end
