      subroutine mshshiftnod2 ( iwork, istarticross, marknod, &
                                eps, shifted )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  3.1  date 22-11-2011 Debug information
!        version  3.0  date 23-04-2003 Change in parameters
!        version  2.0  date 26-11-2002 Extra parameters
!                                      loop over element groups
!
!   copyright (c) 2002-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark nodes that are potentially shifted onto the obstacle
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     triangle
!     subdivision
!     adaptation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), istarticross, marknod(*), shifted(*)
      double precision eps

!     eps            i    Local accuracy
!     istarticross   i    Starting address of obstacle in array icrossobst
!     iwork          i    Integer work array of size nelem
!                         iwork(ielem) contains the starting address of element
!                         ielem in array icrossobst
!                         If 0, the element is not present in icrossobst
!     marknod       i/o   Mark nodes that will be shifted
!     shifted        i    Array of npoint long in which the nodes are marked
!                         which are shifted onto the obstacle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  istart, icross, inode, ipstart, i, ielgrp, &
               iedge, ireledge, iielem, inpelm, ielem
      double precision fact

!     fact           interpolation factor
!     i              Counting variable
!     icross         position of iedge in array icrossobst
!     iedge          edge sequence number
!     ielem          Sequence number of element
!     ielgrp         Sequence number of element group
!     iielem         Sequence number of element per specific element group
!     inode          Node number
!     inpelm         Number of nodes per element
!     ipstart        Starting address in array
!     ireledge       relative edge sequence number
!     istart         First position used at a part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshshiftnod2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshshiftnod2'
         write(irefwr,2) 'eps', eps
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ielem  = 0
      istart = 0

!     --- loop over element groups

      do ielgrp = 1, nelgrp

         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         ishape = kmeshf(ielgrp)

!             --- if shape is triangular or tetrahedron

         if (ishape==3 .or. ishape==7 .or. &
             ishape==11.or. ishape==16)  then

!        --- loop over elements

            do iielem=1, inelem

               ielem=ielem+1

               if (iwork(ielem)/=0) then
                  istart = iwork(ielem)

!                 --- loop over crossed edges

                  do i = 1,icrossobst(istart)
                     ireledge = icrossobst(istart+i)
                     icross  = istarticross+1+2*(ireledge-1)
                     iedge = icrossobst(icross)
                     fact  = rcrossobst(icrossobst(icross+1))
                     ipstart = nmeshc(iedge)

!                    --- mark potential nodes for shifting

                     if ( fact<=eps ) then

                        inode = nmeshc(ipstart+1)
                        marknod(inode)=1

                     else if ( fact>1d0-eps ) then

!                    --- linear or quadratic elements

                        if (inpelm==3 .or. inpelm==4) then
                           inode = nmeshc(ipstart+2)
                        else
                           inode = nmeshc(ipstart+3)
                        end if

                        marknod(inode)=1

                     end if

                  end do

               end if

            end do

         else

!        --- if element is not quadratic triangle

            do iielem=ielem+1,ielem+inelem
               ipstart=nmeshe(iielem)
               do i=1,inpelm

!              --- node may not be shifted

                  if (shifted(nmeshe(ipstart+i))==1) &
                     shifted(nmeshe(ipstart+i))=0
               end do
            end do

!           --- update ielem for next element group

            ielem=ielem+inelem

         end if

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshshiftnod2'

      end if  ! ( debug )
      call erclos ( 'mshshiftnod2' )

      end
