      subroutine mshspecial ( iinput, rinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 27-11-2012 Long integers
!        version  1.0  date 28-07-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Creates special rectangular grid
!
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*)
      double precision rinput(*)

!     iinput         i    integer input array for subroutine mesh
!                         It must be filled as described from position 6
!     rinput         i    Real input array
!                         It must be filled as described from position 6
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ndimlc, npointlc

!     ndimlc         Dimension of the space.
!     npointlc       Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCHKCR       Check if there are double points given by the user
!     MSHCRTSP       Create the mesh in the case ispecial = 1
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshspecial' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshspecial'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iinput(2)<2 ) then

!     --- Grid is non-rectangular, check uniqueness of co-ordinates

         ndimlc = iinput(1)
         if ( iinput(2)==0 ) then

!        --- Unstructured grid

            npointlc = iinput(3)+iinput(4)

         else

!        --- Structured grid

            if ( ndimlc==1 ) then
               npointlc = iinput(3)
            else if ( ndimlc==2 ) then
               npointlc = iinput(3)*iinput(4)
            else
               npointlc = iinput(3)*iinput(4)*iinput(5)
            end if

         end if
         call mshchkcr ( ndimlc, npointlc, rinput(6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Create mesh

      call mshcrtsp ( iinput, rinput )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshspecial'

      end if  ! ( debug )
      call erclos ( 'mshspecial' )

      end subroutine mshspecial
