      subroutine mshspecreorder ( kmeshcnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-01-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Reorder elements of shape 5 and 13 in case of spectral elements
!     without fill-in
! **********************************************************************
!
!                       KEYWORDS
!
!     spectral
!     reordering
!     quadrilateral
!     hexahedron
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: kmeshcnew(*)

!     kmeshcnew     i/o   New kmesh part c
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, ielgrp, ikelmc

!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHSPECREORD05 Reorder elements of shape 5 in case of spectral elements
!                    without fill-in (one element group only)
!     MSHSPECREORD13 Reorder elements of shape 13 in case of spectral elements
!                    without fill-in (one element group only)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     nelgrp := number of element groups in mesh
!     ikelmc := 1
!     For ielgrp := 1 (1) nelgrp do
!        ishape := shape number of elements in ielgrp
!        inpelm := number of nodes in elements in ielgrp
!        inelem := number of elements in ielgrp
!        if ( ishape = 5 or ishape = 13 ) then
!           reorder the inelem elements
!         raise ikelmc
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshspecreorder' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshspecreorder'

      end if
      if ( ierror/=0 ) go to 1000

!     --- nelgrp := number of element groups in mesh
!         ikelmc := 1

      ikelmc = 1
      inpelm = npelm
      inelem = nelem

!     --- For ielgrp := 1 (1) nelgrp do

      do ielgrp = 1, nelgrp

!     --- ishape := shape number of elements in ielgrp
!         inpelm := number of nodes in elements in ielgrp
!         inelem := number of elements in ielgrp

         ishape = kmeshf(ielgrp)
         if ( kelmg>0 ) inelem = kmeshg(ielgrp)
         if ( kelma>0 ) inpelm = kmesha(ielgrp+1)-kmesha(ielgrp)

!        --- if ( ishape = 5 or ishape = 13 ) then
!               reorder the inelem elements

         if ( ishape==5 ) then

!        --- ishape = 5, reorder quadrilaterals

            call mshspecreord05 ( kmeshcnew(ikelmc), inelem )

         else if ( ishape==13 ) then

!        --- ishape = 13, reorder hexahedrons

            call mshspecreord13 ( kmeshcnew(ikelmc), inelem )

         end if ! ( ishape==5 )

!        --- raise ikelmc

         ikelmc = ikelmc + inelem*inpelm

      end do  ! ielgrp = 1, nelgrp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshspecreorder'

      end if
      call erclos ( 'mshspecreorder' )

      end
