      subroutine mshsurfaces1 ( surffaces, extsurffaces, &
                                leniwork2, lenextsurffac, nsurfsold, &
                                nsurfsnew, iworksurfs, ipoint, &
                                leniworksurfs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 24-11-2004 Fortran 90
!        version  1.1  date 04-08-2004 Extra debug information
!        version  1.0  date 03-03-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      If there are faces that belong to more surfaces then
!      split surfaces in subsurfaces and make connected surfaces
! **********************************************************************
!
!                       KEYWORDS
!
!     face
!     mesh
!     compound_surface
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulework
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer surffaces(*), extsurffaces(*), leniwork2, &
              nsurfsold, nsurfsnew, iworksurfs(*), &
              ipoint(*), lenextsurffac, leniworksurfs

!     extsurffaces   o    Array of length lenextsurffac containing the
!                         face sequence numbers of the new surfaces to be
!                         created
!     ipoint         o    Array of estimated length 10*nsurfsold that is used to
!                         store the end addresses of each new surface in array
!                         extsurffaces
!                         The positions 1 to nsurfs get the value 0
!     iworksurfs     o    Output array of length nsurfs x nsurfs= &
!                         Is initially filled with zeros
!                         If a surface contains subsurfaces (i.e. the extra
!                         surfaces) then the sequence numbers of these
!                         subsurfaces are stored in iworksurfs(surface,.)
!                         sequentially
!     lenextsurffac i/o   Length of array extsurfface
!                         At input estimated length
!                         At output computed length
!     leniwork2     i/o   Length of array iwork2
!                         At input estimated length
!                         At output computed length
!     leniworksurfs i/o   Length of array iworksurfs
!                         At input estimated length
!                         At output computed length
!     nsurfsnew      i    Number of surfaces after extension with new surfaces
!                         corresponding to outer surfaces (if any)
!     nsurfsold      i    Number of surfaces in the mesh read
!     surffaces      i    Array containing the face numbers of the elements
!                         stored in array kmeshn
!                         Exactly the sequence of kmeshn is used
!                         In this case it refers to kmeshn before extension
!                         with extra outer surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isurnr, iielem, istart, iface, ifile
      logical doublefound

!     doublefound    If true faces have been found that are in more than one
!                    surface
!     i              Counting variable
!     iface          Face sequence number
!     ifile          Input parameter for subroutine prinsurfaces
!     ihelp          integer work array of length nsurfs
!                    If surface i is single ihelp(i) = 0
!                    If surface i is possibly compound ihelp(i) = -1
!     iielem         Relative element number in element group
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     iwork          integer work array of length nfaces
!                    iwork(iface) contains the number of surfaces
!                    face iface is stored in
!     iwork1         integer work array of length nfaces
!                    contains for each face iface with iwork(iface)>1 a
!                    starting address in iwork2
!     iwork2         integer work array of length leniwork2
!                    contains for each face iface with iwork(iface)>1 the
!                    sequence numbers of the surfaces iface is lying in
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     MSHSURFACES13  Write information with respect to elements that are in
!                    several surfaces at one time
!     MSHSURFACES5   Check which surfaces are single and which are
!                    compound surfaces
!     MSHSURFACES6   Fill arrays iwork1 and iwork2 with surface numbers
!                    corresponding to faces
!     PRININ         print 1d integer array
!     PRINSURFACES   Print the contents of array kmeshn either to the standard
!                    output file a specified file or a set of specified files
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2627   Composite surfaces not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!     nsurfsnew := nsurfs
!     Clear array ipoint(1:nsurfs)
!     Fill work with number of surfaces coupled to a face
!     For all surfaces stored in kmeshn and surffaces do
!        For all faces iface in surface do
!        iwork(iface) := iwork(iface)+1
!     if ( double face found ) then
!        Fill array iwork1 with starting addresses in array iwork2
!        Fill array iwork2 with surface numbers corresponding to faces
!        Fill compound surfaces and create new surfaces
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsurfaces1' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsurfaces1'
         ifile = 2
         call prinsurfaces ( nsurfs, kmeshn, ifile, 's' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- nsurfsnew := nsurfs

      nsurfsnew = nsurfs

!     --- Clear array ipoint(1:nsurfs)

      ipoint(1:nsurfs) = 0

!     --- Clear array iwork(1:nfaces)

      iwork = 0

      doublefound = .false.

!     --- Fill work with number of surfaces coupled to a face

      istart = 0

!     --- For all surfaces stored in kmeshn and surffaces do

      do isurnr = 1, nsurfsold

         inelem = kmeshn(isurnr+nsurfs)

!        --- For all faces iface in surface do

         do iielem = 1, inelem

!        --- iwork(iface) := iwork(iface)+1

            iface = surffaces(istart+iielem)
            if ( iwork(iface)>0 ) then
               doublefound = .true.
               if ( debug ) &
                  write(irefwr,*)'isurnr, iielem, iface, iwork(iface) ', &
                  isurnr, iielem, iface, iwork(iface)
            end if  ! ( iwork(iface)>0 )
            iwork(iface) = iwork(iface)+1

         end do  ! iielem = 1, inelem
         istart = istart+inelem

      end do  ! isurnr = 1, nsurfsold

      if ( doublefound ) then

!     --- Double faces found
!         Fill array iwork1 with starting addresses in array iwork2
!         Fill array iwork2 with surface numbers corresponding to faces

         call mshsurfaces6 ( extsurffaces, nfaces, leniwork2, &
                             nsurfsold, nsurfs, kmeshn, surffaces )

!        --- Fill compound surfaces and create new surfaces

         call mshsurfaces5 ( kmeshn, surffaces, nsurfsold, &
                             nsurfs, nsurfsnew, &
                             extsurffaces, lenextsurffac, ipoint, &
                             iworksurfs, leniworksurfs )

      else

!     --- Not a double surface found

         lenextsurffac = 0

      end if  ! ( doublefound )

1000  if ( debug ) then

!     --- Debug information

         if ( doublefound ) then

!        --- There are compound surfaces

            write(irefwr,*) 'nsurfsold, nsurfs ', nsurfsold, nsurfs
            write(irefwr,*) 'lenextsurffac, nsurfsnew ', &
                             lenextsurffac, nsurfsnew
            call prinin ( ipoint, nsurfsnew, 'ipoint' )
            call prinin ( extsurffaces, ipoint(nsurfsnew), &
                          'extsurffaces' )
            call prinin ( ihelp, nsurfs, 'ihelp' )

         else

!        --- No extra surfaces created

            write(irefwr,*) 'No compound surfaces'

         end if
         write(irefwr,*) 'End mshsurfaces1'

      end if
      call erclos ( 'mshsurfaces1' )

      end
