      subroutine mshtrakmeshm ( kmeshmnew, istart, ibrpnt, npnold, lenkmeshmnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-10-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill nodes on kmesh part m (curves) in case of a mesh that is transformed
!     by transf
!     Remark: if this subroutine is changed also update mshrefkmeshm and
!             mshspckmeshm !!!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh_generation
!     spectral
!     curve
!     coordinate
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulework
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshmnew(*), istart(*), ibrpnt(*), npnold, lenkmeshmnew

!     ibrpnt         i    neighbour array
!     istart         i    start addresses neighbour array
!     kmeshmnew      o    New array kmeshm
!     lenkmeshmnew   i    Length of kmeshmnew
!     npnold         i    old number of nodes in mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ist, ish, nsubcv, icurnr, ilold, &
              j, inew, ibeg, iend, jcurnr, ipos3, kkrom, i

!     i              Counting variable
!     ibeg           node number of begin node
!     icurnr         Curve sequence number
!     iend           end node number
!     ilold          old number of nodes on line
!     inew           help variable to store node number
!     ipos3          position in curve array
!     ish            help variable to store node number
!     ist            node number where to start
!     iwork          array of length ncurvs containing for each
!                    curve a 0 when i has been filled and a 1 when
!                    the curve has not been filled
!     j              Counting variable
!     jcurnr         |icurnr| i.e. absolute value of curve sequence number
!     kkrom          length of new curves array
!     nsubcv         Number of subcurves of a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH403         Determine number refinement point in line piece
!     MSH414         Determine whether curves are filled in
!     MSHFILLMSUB    Fill kmesh part m for composite curves
!     PR0008         Print information of curves
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     940   Not enough positions declared for help array
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshtrakmeshm' )
      debug = .false.

      kmeshmnew(1) = kmeshm(1)
      kmeshmnew(2) = kmeshm(2)
      kmeshmnew(4) = kmeshm(4)
      kmeshmnew(5) = 1
      ninner = kmeshm(1)
      ist = kmeshm(2)
      ish = kmeshmnew(2)
      nsubcv = 0
      if ( debug ) call pr0008 ( kmeshm, ncurvs )

      allocate ( iwork(ncurvs), stat = error )
      if ( error/=0 ) call eralloc ( error, ncurvs, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Check whether curves have been filled or not

      call msh414 ( kmeshm, ncurvs, iwork )

!     --- Check whether coordinates of points on curves have to be
!         adjusted
!         First kmeshm part 2 must be filled

      do icurnr = 1, ncurvs
         if ( kelmq>0 ) nsubcv = kmeshq(icurnr+1)-kmeshq(icurnr)
         if ( nsubcv==0 ) then

!        --- No subcurves, hence single curve
!            Compute number of nodes along curves

            ilold = kmeshm(icurnr+5) - kmeshm(icurnr+4)
            ist = kmeshm(2)+kmeshm(icurnr+4)-1
            if ( iwork(icurnr)==0 ) then

!           --- Curve has been filled, raise ilold

               do i = 1, ilold-1

                  ibeg = kmeshm(ist+i-1)
                  iend = kmeshm(ist+i  )

                  call msh403 ( npnold, ibeg, iend, ibrpnt, istart, &
                                inew )

                  if ( inew/=0 ) ilold = ilold + 1

               end do

            end if

            kmeshmnew(icurnr+5) = ilold

         end if

      end do

      if ( kelmq>0 ) then

!     --- kelmq > 0, fill also the composite curves

         do icurnr = 1, ncurvs
            nsubcv = kmeshq(icurnr+1)-kmeshq(icurnr)
            if ( nsubcv>0 ) then

!           --- nsubcv > 0, there are subcurves
!               Fill by adding the subcurves

               ilold = 0
               do j = 1, nsubcv
                  jcurnr = abs(kmeshq(kmeshq(icurnr)+j))
                  ilold = ilold+kmeshmnew(jcurnr+5)
               end do
               kmeshmnew(icurnr+5) = ilold

            end if ! nsubcv>0

         end do  ! icurnr = 1, ncurvs

      end if  ! kelmq>0
      if ( debug ) then

!     --- Debug information

         call prinin ( kmeshmnew(6), ncurvs, &
           'Number of nodes in curves after transformation' )

      end if

!     --- Acumulate kmeshm part 2

      kmeshmnew(5) = 1
      do icurnr = 1, ncurvs
         kmeshmnew(5+icurnr) = &
            kmeshmnew(5+icurnr)+kmeshmnew(4+icurnr)
      end do
      if ( debug ) &
         call prinin ( kmeshmnew(5), ncurvs+1, 'kmeshm part 2' )

      do icurnr = 1, ncurvs

         if ( kelmq>0 ) nsubcv = kmeshq(icurnr+1)-kmeshq(icurnr)

         if ( nsubcv==0 ) then

!        --- Single curve

            ilold = kmeshm(icurnr+5) - kmeshm(icurnr+4)
            ist = kmeshm(2)+kmeshm(icurnr+4)-1
            ish = kmeshm(2)+kmeshmnew(icurnr+4)-1
            if ( debug ) write(irefwr,*) 'icurnr, ilold, ist, ish ', &
               icurnr, ilold, ist, ish
            if ( iwork(icurnr)==1 ) then

!           --- icurnr has not been filled, special situation

               do i = 0, ilold-1
                  kmeshmnew(ish+i) = kmeshm(ist+i)
               end do

            else

!           --- iwork(icurnr) = 0 curve has been filled

               kmeshmnew(ish) = kmeshm(ist)
               ish = ish + 1

               do i = 1, ilold-1

                  ibeg = kmeshm(ist+i-1)
                  iend = kmeshm(ist+i  )

                  call msh403 ( npnold, ibeg, iend, ibrpnt, istart, &
                                inew )

                  if ( inew/=0 ) then

                       kmeshmnew(ish)  = inew
                       ish = ish + 1

                  end if

                  kmeshmnew(ish) = iend
                  ish = ish + 1

              end do

            end if

         end if   ! single curve

      end do  ! icurnr = 1, ncurvs

!     --- Fill composite curves

      if ( kelmq>0 ) call mshfillmsub ( ncurvs, kmeshmnew, kmeshq )

!     --- Raise first positions and fill part 4

      kmeshmnew(3) = kmeshmnew(5+ncurvs)+kmeshm(2)-1
      ipos3 = kmeshmnew(3)

      kmeshmnew(ipos3) = 0

      do i = 1, ninner
          kmeshmnew(ipos3+i-1) = kmeshm(kmeshm(3)+i-1)
      end do
      if ( debug ) call pr0008 ( kmeshmnew, ncurvs )

      kkrom = kmeshmnew(3) + ninner

      if ( kkrom>lenkmeshmnew ) then

         call errint ( kkrom, 1 )
         call errint ( lenkmeshmnew, 2 )
         call errsub ( 940, 2, 0, 0 )
         go to 1000

      end if

      lenkmeshmnew = kkrom

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'mshtrakmeshm' )

      end
