      subroutine mshtriancent2 ( kmeshcnew, kmeshcold, inpelm, inelem, &
                                 npoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-11-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy contents of array kmeshcold into kmeshcnew
!     Extend with center
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     centre
!     node
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, inelem, kmeshcnew(inpelm+1,inelem), &
              kmeshcold(inpelm,inelem), npoint

!     inelem         i    Number of elements in element group
!     inpelm         i    number of points per element in old mesh
!     kmeshcnew      o    New kmesh part c (7 points, one element group)
!     kmeshcold      i    Old kmesh part c (6 points, one element group)
!     npoint        i/o   Number of nodal points
!                         At input old number of nodes
!                         At output extended with centers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, i

!     i              Counting variable
!     ielem          Element number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all elements do
!        Copy old element
!        Add extra point and raise npoint
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshtriancent2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshtriancent2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- For all elements do

      do ielem = 1, inelem

!     --- Copy old element

         do i = 1, inpelm
            kmeshcnew(i,ielem) = kmeshcold(i,ielem)
         end do  ! i = 1, inpelm

!        --- Add extra point and raise npoint

         npoint = npoint+1
         kmeshcnew(inpelm+1,ielem) = npoint

      end do  ! ielem = 1, inelem

1000  call erclos ( 'mshtriancent2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshtriancent2'

      end if  ! ( debug )

      end
