      subroutine mshvolbouni ( iinput, iposi, kbndpt, ndefsf, kmeshslc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 02-09-2010 Remove intarmsh
!        version  2.0  date 14-11-2002 Extra parameter kmeshslc
!        version  1.0  date 14-01-2002
!
!   copyright (c) 2002-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Reorders array KBNDPT in case of negative subsurfaces
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     volume
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iposi, kmeshslc(8,*), &
              kbndpt(*), ndefsf

!     iinput         i    integer input array    see subroutine mesh
!                         iinput contains only the part with respect to the
!                         definition of the volumes
!     iposi          i    next free position of iinput
!     kbndpt        i/o   array containing the nodal point numbers of the
!                         elements of the boundary
!     kmeshslc       i    Kmesh part s, contains information about the various
!                         surfaces. See the PG 13.1
!     ndefsf         i    Number of curves in surface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, nsubsf, ikelmc, &
              istart, isubsf, isur, leng2, nelloc, jsur, npoloc

!     ikelmc         position in kmesh part c
!     istart         First position used at a part
!     isubsf         Sequence number of subsurface
!     isur           surface number
!     j              Counting variable
!     jsur           surface number provided with sign
!     leng2          length indicator elements
!     nelloc         Local number of elements
!     npoloc         Number of nodes in subsurface
!     nsubsf         Number of subsurfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inidln

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIDLN         Compute length of array in BUFFER
!     MSHNEGORDSURF  Reorder array kbndpt for negative subsurfaces of a
!                    composite ordered surface
!     MSHNEGSURF     Reorder array kbndpt for negative subsurfaces of a
!                    composite surface
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshvolbouni' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshvolbouni'

      end if
      ishape = ishape-8

!     --- Read kmeshc of boundary

      ikelmc = 0
      do j = 1, ndefsf
         isurnr = mod(iinput(iposi+2+j)-1,nsurfs)+1
         nsubsf = kmeshr(isurnr+1)-kmeshr(isurnr)
         isurf = kmeshslc(1,isurnr)
         length = kmeshh(2,isurnr+1)*kmeshh(6,isurnr+1)
         if ( debug ) write(irefwr,*) 'j, isurnr, nsubsf, isurf ', &
            j, isurnr, nsubsf, isurf

         if ( nsubsf==0 ) then

!        --- nsubsf = 0, No subsurfaces

            ikelmc = ikelmc+length

         else if ( isurf==11 .or. isurf==12 ) then

!        --- Ordered surface of subsurfaces

            istart = kmeshr(isurnr)
            do isubsf = 1, nsubsf

               jsur = kmeshr(istart+isubsf)
               isur = abs(jsur)
               nelloc = kmeshh(2,isur+1)
               leng2 = nelloc*inplmt
               if ( debug ) write(irefwr,*) 'isubsf, jsur, leng2 ', &
                  isubsf, jsur, leng2
               if ( jsur<0 ) then

!              --- reversed direction, change sequence in kbndpt

                  npoloc = kmeshh(1,isur+1)
                  if ( debug ) &
                      write(irefwr,*) 'nelloc, npoloc, ikelmc ', &
                                       nelloc, npoloc, ikelmc
                  call mshnegordsurf ( isurnr, isubsf, nsurfs, npoloc, &
                                       nelloc, kmeshr, ishape, &
                                       kbndpt(ikelmc+1), inplmt )

               end if

            end do
            ikelmc = ikelmc+length

         else

!        --- Surface consists of subsurfaces

            istart = kmeshr(isurnr)
            do isubsf = 1, nsubsf
               jsur = kmeshr(istart+isubsf)
               isur = abs(jsur)
               nelloc = kmeshh(2,isur+1)
               leng2 = nelloc*inplmt
               if ( jsur>0 ) then

!              --- standard direction, raise ikelmc

                  ikelmc = ikelmc+leng2

               else

!              --- reversed direction, change sequence in kbndpt

                  call mshnegsurf ( nelloc, ishape, inplmt, kbndpt, &
                                    ikelmc )

               end if

            end do

         end if

      end do
      ishape = ishape+8

      if ( debug ) write(irefwr,*) 'End mshvolbouni'
      call erclos ( 'mshvolbouni' )

      end
