      subroutine mshwrcurves ( iinput, rinput, irefmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 02-09-2010 Remove intarmsh
!        version  1.0  date 31-01-2003
!
!   copyright (c) 2003-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Write parts of arrays iinput and rinput corresponding to curves
!   in original form back to an input file with reference number irefmesh
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     input
!     write
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), irefmesh
      double precision rinput(*)

!     iinput         i    integer input array
!     irefmesh       i    Reference number of file to which the mesh description
!                         must be written
!     rinput         i    Real input array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncurs
      parameter ( ncurs=19 )
      integer ilast, ntxtcur(ncurs), j, indent, i, iinp, &
              icurve, ifirst
      character *1000 wrtrecord
      character *11 txtcurves(ncurs)

!     i              Counting variable
!     icurve         Type of curve
!     ifirst         First curve (surface) number read
!     iinp           last position used in array iinput
!     ilast          Last position used in wrtrecord
!     indent         number of spaces for the indent
!     j              Counting variable
!     ncurs          Number of curve generators
!     ntxtcur        Help array to store the length of the names of the
!                    curve generators
!     txtcurves      Help array to store the names of the curve generators
!     wrtrecord      Contains record to be written
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICUR01       Compute number of user points and position of first user
!                    point
!     INIPUTIN       Put integer number in wrtrecord from position ilast
!     INIPUTRL       Put real number in wrtrecord from position ilast
!     INIPUTXT       Put text in wrtrecord from position ilast
!     INIWRRCD       Write record to file irefmesh in sepran input format
!     MSH046         Raise iinp as function of icurve
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data txtcurves / 'line', 'user', 'arc', 'spline', 'cline', &
                       'carc', 'curves', 'param', 'cparam', &
                       'translate c', 'rotate c', 'cspline', &
                       'reflect c', 'spcurve', 'profile', 'cprofile', &
                       'circle', 'circle_part', 'sphere_arc'/
      data ntxtcur / 4, 4, 3, 6, 5,   4, 6, 5, 6, 11, &
                     8, 7, 9, 7, 7,   8, 6,11,10/

!     txtcurves   Help array to store the names of the curve generators
!                  1:  line
!                  2:  user
!                  3:  arc
!                  4:  spline
!                  5:  cline
!                  6:  carc
!                  7:  curves
!                  8:  param
!                  9:  cparam
!                 10:  translate
!                 11:  rotate
!                 12:  cspline
!                 13:  reflect
!                 14:  spcurve
!                 15:  profile
!                 16:  cprofile
!                 17:  circle
!                 18:  circle_part
!                 19:  sphere_arc
! ======================================================================
!
      call eropen ( 'mshwrcurves' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshwrcurves'

      end if

      indent = 3
      ilast = 0

!     --- write record with curves

      call iniputxt ( 'curves', wrtrecord, ilast )
      call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )

!     --- write curves

      indent = 6
      iinp = iinp2
      do i = 1, ncurvs
         icurve = iinput(iinp)
         if ( icurve/=0 ) then

!        --- Non-empty curve
!            First c i = name

            call iniputxt ( 'c', wrtrecord, ilast )
            call iniputin ( i, wrtrecord, ilast )
            call iniputxt ( ' = ', wrtrecord, ilast )
            call iniputxt ( txtcurves(icurve)(1:ntxtcur(icurve)), &
                            wrtrecord, ilast )
            call iniputxt ( ' ', wrtrecord, ilast )
            if ( icurve==10 .or. icurve==11 .or. icurve==13 ) then

!           --- translate, rotate or reflect

               call iniputin ( iinput(iinp+4), wrtrecord, ilast )

            else if ( icurve/=7 .and. icurve/=14 ) then

!           --- not curves of curves

               call iniputin ( iinput(iinp+1), wrtrecord, ilast )

            end if
            call iniputxt ( ' ( ', wrtrecord, ilast )
            if ( icurve==7 .or. icurve==14 ) then

!           --- curves of curves, write curves

               npoint = iinput(iinp+3)
               do j = 1, npoint
                  if ( iinput(iinp+3+j)<0 ) &
                     call iniputxt ( '-', wrtrecord, ilast )
                  call iniputxt ( 'c', wrtrecord, ilast )
                  call iniputin ( abs(iinput(iinp+3+j)), wrtrecord, &
                                  ilast )
                  call iniputxt ( ', ', wrtrecord, ilast )
               end do
               ilast = ilast-2

            else

!           --- not curves of curves, write points

               call inicur01 ( iinput(iinp), ifirst, npoint, ndim )
               if ( icurve==4 .or. icurve==12 ) npoint = npoint-2
               if ( icurve==13 ) then
                  call iniputxt ( 'axis = ', wrtrecord, ilast )
               end if
               do j = 1, npoint
                  if ( iinput(iinp+ifirst+j-2)<0 ) &
                     call iniputxt ( '-', wrtrecord, ilast )
                  call iniputxt ( 'p', wrtrecord, ilast )
                  call iniputin ( abs(iinput(iinp+ifirst+j-2)), &
                                  wrtrecord, ilast )
                  call iniputxt ( ', ', wrtrecord, ilast )
               end do
               ilast = ilast-2

            end if
            if ( icurve<=4 .or. icurve==8 .or. icurve==17 ) then

!           --- line, user, arc, spline, parameter, circle

               call iniputxt ( ', nelm = ', wrtrecord, ilast )
               call iniputin ( iinput(iinp+2), wrtrecord, ilast )
               if ( iinput(iinp+3)>0 ) then
                  call iniputxt ( ', ratio = ', wrtrecord, ilast )
                  call iniputin ( iinput(iinp+3), wrtrecord, ilast )
                  call iniputxt ( ', factor = ', wrtrecord, ilast )
                  call iniputrl ( rinput(irnp2+i-1), wrtrecord, ilast )
               end if

            else if ( icurve<=6 .or. icurve==9 ) then

               if ( iinput(iinp+ifirst+npoint-1)>1 ) then
                  call iniputxt ( ', nodd = ', wrtrecord, ilast )
                  call iniputin ( iinput(iinp+ifirst+npoint-1), &
                                  wrtrecord, ilast )
               end if

            else if ( icurve==18 ) then

!           --- circle part

               call iniputxt ( ', nelm = ', wrtrecord, ilast )
               call iniputin ( iinput(iinp+2), wrtrecord, ilast )
               call iniputxt ( ', c', wrtrecord, ilast )
               call iniputin ( iinput(iinp+3), wrtrecord, ilast )
               call iniputxt ( ', subseq = ', wrtrecord, ilast )
               call iniputin ( iinput(iinp+4), wrtrecord, ilast )

            else if ( icurve==19 ) then

!           --- arc on sphere

               call iniputxt ( ', nelm = ', wrtrecord, ilast )
               call iniputin ( iinput(iinp+2), wrtrecord, ilast )
               call iniputxt ( ', c', wrtrecord, ilast )
               call iniputin ( iinput(iinp+3), wrtrecord, ilast )
               call iniputxt ( ', c', wrtrecord, ilast )
               call iniputin ( iinput(iinp+4), wrtrecord, ilast )
               call iniputxt ( ', idirec = ', wrtrecord, ilast )
               call iniputin ( iinput(iinp+5), wrtrecord, ilast )

            end if

!           --- Rest has not yet been implemented

            call iniputxt ( ' )', wrtrecord, ilast )
            call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )

         end if

         call msh046 ( iinput, iinp, icurve )

      end do

      if ( debug ) write(irefwr,*) 'End mshwrcurves'
      call erclos ( 'mshwrcurves' )

      end
