      subroutine  mshwrrenum ( irenum, irefmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-02-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Write parts of arrays iinput and rinput corresponding to renumber
!   in original form back to an input file with reference number irefmesh
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     input
!     write
!     renumbering
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer irenum(*), irefmesh

!     irefmesh       i    Reference number of file to which the mesh description
!                         must be written
!     irenum         i    Array of length 10 with information about the
!                         renumbering.
!                         Array must be filled as follows:
!                          1: if >= 0, only position 1 is filled.
!                             This position contains the parameter method
!                             according to the old SEPRAN definition, i.e.
!                             jrenum = 9: Use default depending on mesh
!                             else:
!                             jrenum = ispecial*100 + method + 3 * ipref
!                             ispecial = 0: standard
!                                        1: ivertx=1, imidp=1, icentr=3
!                             If < 0, -jrenum(1) contains the sequence number
!                             of the last entry filled by the user
!                             All other values are filled with defaults
!                          2: method Indication of the renumbering method used
!                                    The algorithms are compared to the original
!                                    numbering:
!                                    0  No renumbering
!                                    1  Cuthill-Mckee algorithm
!                                    2  Sloan algorithm
!                                    3  Both algorithms,  the best is used
!                                    9  Choose depending on mesh
!                             Default: 9
!                          3: ipref  Indication of the criterion to be used
!                                    to find the best algorithm:
!                                    0  Choose the smallest profile
!                                    1  Choose the smallest band width
!                                    2  Use only the renumbered algorithm
!                                       (method=1 or 2)
!                             Default: 0
!                          4: iprint Indicates if information about the
!                                    renumbering must always be printed
!                                    (1) or only depending on ioutp (0)
!                             Default: 0
!                          5: ivertx Priority for vertex nodes (1-3)
!                             Default: 3
!                          6: imidp  Priority for mid point nodes (1-3)
!                             Default: 3
!                          7: icentr Priority for centroids (1-3)
!                             Default: 3
!                          8: ilevel Indicates if renumbering of specia nodes
!                                    must be performed per level (1) or not (0)
!                             Default: 0
!                          9: istmet Indicates how the start of the renumbering
!                                    algorithm must be performed
!                                    0  Automatic
!                                    1  Start user point given
!                                    2  Start curve given
!                                    3  Start surface given
!                             Default: 0
!                         10: istval Value of start point, curve f surface
!                             Default: 1
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nmethod, npref
      parameter ( nmethod = 3, npref = 3 )
      integer indent, ilast, nxtmethod(nmethod), method, ipref, &
              nxtpref(npref)
      character *13 txtmethod(nmethod)
      character *7 txtpref(npref)
      character *1 txtstart(3)
      character *1000 wrtrecord

!     ilast          Last position used in wrtrecord
!     indent         number of spaces for the indent
!     ipref          Sequence number of preference
!                    0:   profile
!                    1:   band
!                    2:   best
!     method         Defines type of renumbering algorithm, see above
!     nmethod        Number of various renumbering algorithms
!     npref          Number of various options for preference part
!     nxtmethod      Number of characters in various renumbering algorithms
!     nxtpref        Number of characters in various options for preference part
!     txtmethod      Names of various renumbering algorithms
!     txtpref        Names of various options for preference part
!     txtstart       Names of options for starting  methods
!     wrtrecord      Contains record to be written
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIPUTIN       Put integer number in wrtrecord from position ilast
!     INIPUTXT       Put text in wrtrecord from position ilast
!     INIWRRCD       Write record to file irefmesh in sepran input format
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data txtmethod / 'Cuthill_McKee', 'Sloan', 'Best' /
      data nxtmethod / 13, 5, 4 /
      data txtpref / 'profile', 'band', 'always' /
      data nxtpref / 7, 4, 6 /
      data txtstart / 'p', 'c', 's' /
! ======================================================================
!
      call eropen ( ' mshwrrenum' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from  mshwrrenum'

      end if
      if ( irenum(1).eq.9 ) go to 1000

      indent = 3
      ilast = 0

!     --- write record with renumber

      if ( irenum(1).gt.0 ) then

!     --- irenum(1) > 0, old fashioned method

         call iniputxt ( 'renumber, ', wrtrecord, ilast )
         method = irenum(1)
         ipref = (method-1)/3
         method = method-3*ipref
         call iniputxt ( txtmethod(method)(1:nxtmethod(method)), &
                         wrtrecord, ilast )
         if ( ipref.gt.1 ) then
            call iniputxt ( ', ', wrtrecord, ilast )
            call iniputxt ( txtpref(ipref)(1:nxtpref(ipref)), &
                            wrtrecord, ilast )
         end if

      else

!     --- irenum(1) < 0, new method

         method = irenum(2)
         if ( method.eq.0 ) then
            call iniputxt ( 'renumber not', wrtrecord, ilast )
            go to 500
         else
            call iniputxt ( 'renumber ', wrtrecord, ilast )
         end if
         ipref  = irenum(3)+1
         if ( method.lt.9 ) then

!        --- Only in case mthod = 1 to 3 the method is printed

            call iniputxt ( ', ', wrtrecord, ilast )
            call iniputxt ( txtmethod(method)(1:nxtmethod(method)), &
                            wrtrecord, ilast )

         end if

         if ( ipref.gt.1 ) then
            call iniputxt ( ', ', wrtrecord, ilast )
            call iniputxt ( txtpref(ipref)(1:nxtpref(ipref)), &
                            wrtrecord, ilast )
         end if
         if ( irenum(4).gt.0 ) &
            call iniputxt ( ', print', wrtrecord, ilast )
         if ( irenum(5).ne.3 ) then

!        --- vertex priority

            call iniputxt ( ', vertex_seq = ', wrtrecord, ilast )
            call iniputin ( irenum(5), wrtrecord, ilast )

         end if
         if ( irenum(6).ne.3 ) then

!        --- mid_point priority

            call iniputxt ( ', mid_point_seq = ', wrtrecord, ilast )
            call iniputin ( irenum(6), wrtrecord, ilast )

         end if
         if ( irenum(7).ne.3 ) then

!        --- centre priority

            call iniputxt ( ', centroid_seq = ', wrtrecord, ilast )
            call iniputin ( irenum(7), wrtrecord, ilast )

         end if
         if ( irenum(8).gt.0 ) &
            call iniputxt ( ', levels', wrtrecord, ilast )
         if ( irenum(9).gt.0 ) then

!        --- start_method

            call iniputxt ( ', start = ', wrtrecord, ilast )
            call iniputxt ( txtstart(irenum(9)), wrtrecord, ilast )
            call iniputin ( irenum(10), wrtrecord, ilast )

         end if

      end if

500   call iniwrrcd ( wrtrecord, ilast, indent, irefmesh )

1000  call erclos ( ' mshwrrenum' )
      if ( debug ) write(irefwr,*) 'End  mshwrrenum'

      end
