      subroutine sepfillsurnods
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array surfacenodes with number of nodes and node numbers of surfaces
!     Old subroutine mshfillsurnods
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     surface
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemesh
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart

!     istart         First position used at a part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRINNMESHX     Print the contents of an integer array with structure of
!                    sub array of nmesh
!     SEPSURNOD      Store all nodes in the surfaces ifirst to ilast in array
!                    nodes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     istart := nsurfs+1
!     For all surfaces isurnr do
!        surfnodes(isurnr) := istart
!        Detect all nodes in surface
!        surfnodes(istart) := number_of_nodes_in_surface
!        Fill surfnodes(istart+1,...,istart+number_of_nodes_in_surface)
!        istart := istart+1+number_of_nodes_in_surface
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillsurnods' )
      debug = .false.

      length = size ( kmeshn )

      allocate ( surfnodes(length), stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'surfnodes' )
      if ( ierror/=0 ) go to 1000

!     --- istart := nsurfs+1

      istart = nsurfs+1

!     --- For all surfaces isurnr do

      do isurnr = 1, nsurfs

!     --- surfnodes(isurnr) := istart

         surfnodes(isurnr) = istart

!        --- Detect all nodes in surface
!            surfnodes(istart) := number_of_nodes_in_surface
!            Fill surfnodes(istart+1,...,istart+number_of_nodes_in_surface)

         nnodes = npoint
         call sepsurnod ( isurnr, isurnr, surfnodes(istart+1), nnodes )
         surfnodes(istart) = nnodes

!        --- istart := istart+1+number_of_nodes_in_surface

         istart = istart+1+nnodes

      end do !isurnr = 1, nsurfs

!     --- make array surfnodes a permanent array
!         the length of surfnodes can be computed from its contents

      call inichkarlen ( istart-1, length, 'surfnodes' )
      if ( ierror/=0 ) go to 1000

      allocate ( km(iseqkm)%surfnodes(istart-1) )
      km(iseqkm)%surfnodes = surfnodes(1:istart-1)
      km(iseqkm)%kmesh(60) = istart-1

      deallocate ( surfnodes, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'surfnodes' )

      surfnodes => km(iseqkm)%surfnodes

      if ( debug ) then

!     --- debug true, debug statements

         write(irefwr,*) 'Nodes corresponding to surfaces'
         call prinnmeshx ( nsurfs, surfnodes )

      end if

1000  call erclos ( 'sepfillsurnods' )
      end
