      double precision function tetvolume(x1,x2,x3,x4)
! ======================================================================
!
!        programmer    Koos Meijerink
!        version  1.0  date 13-09-2001
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute the volume of a tetrahedron
!
! **********************************************************************
!
!                       KEYWORDS
!
!     determinant
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision x1(3),x2(3),x3(3),x4(3)

!     tetvolume      o    Volume of tetrahedron
!     x1             i    coordinates of first vertex
!     x2             i    coordinates of second vertex
!     x3             i    coordinates of third vertex
!     x4             i    coordinates of fourth vertex
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision v(3,3)
      integer i

!     i              Counting variable
!     v              array containing the vectors from vertex 1 to others
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    det = v1(1) * ( v2(2)*v3(3) - v3(2)*v2(3) ) + &
!          v2(1) * ( v3(2)*v1(3) - v1(2)*v3(3) ) + &
!          v3(1) * ( v1(2)*v2(3) - v2(2)*v1(3) )
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      do i = 1,3
         v(1,i) = x2(i) - x1(i)
         v(2,i) = x3(i) - x1(i)
         v(3,i) = x4(i) - x1(i)
      end do

!     --- calculate determinant of y

      tetvolume = v(1,1) * (v(2,2)*v(3,3)-v(2,3)*v(3,2)) - &
                  v(1,2) * (v(2,1)*v(3,3)-v(2,3)*v(3,1)) + &
                  v(1,3) * (v(2,1)*v(3,2)-v(2,2)*v(3,1))

      end
