      subroutine msh058conn ( iinput, iinp, ielgeg )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 02-09-2010 Remove intarmsh
!        version  1.0  date 10-03-2006
!
!   copyright (c) 2006-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmesh part f for connection elements
!     Adapt iinp
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     connect
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, ielgeg(*)

!     ielgeg         i    array with number of nodes for each element shape
!     iinp          i/o   last position used in array iinput
!     iinput        i/o   integer input array for subroutine MESH
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipoint, icurvs, isurfs, i, ielgrp, ifirst, ilast

!     i              Counting variable
!     icurvs         Number of parts where curve connection elements are given
!     ielgrp         element group number
!     ifirst         First surface
!     ilast          Second surface
!     ipoint         Number of parts where surface connection elements are given
!     isurfs         Number of parts where surface connection elements are given
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH058SURFS    Find shape number and number of points of element of a
!                    series of surfaces
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     323   element group number for connected element is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh058conn' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh058conn'

      end if
      if ( ierror/=0 ) go to 1000

      iinp = iinp7
      nelgrp = nelgrp+1
      ipoint = iinput(iinp7)
      icurvs = iinput(iinp7+1)
      isurfs = iinput(iinp7+2)
      iinp = iinp+3

      do i = 1, ipoint
         ielgrp = iinput(iinp)
         if ( ielgrp<nelgrp ) then
           call errint ( ielgrp, 1 )
           call errint ( nelgrp, 2 )
           call errsub ( 323, 2, 0, 0)
         else
            if ( ielgrp/=nelgrp ) nelgrp = nelgrp+1
            if ( ielgrp/=nelgrp ) then
               call errint ( ielgrp, 1 )
               call errint ( nelgrp, 2 )
               call errsub ( 323, 2, 0, 0)
            end if
            inpelm = 2
            if ( inpelm/=npelm ) jnodp = 0
            npelm = max(inpelm,npelm)
            kmeshf(ielgrp) = -10001
         end if
         iinp = iinp+3
      end do
      do i = 1, icurvs
         ielgrp = iinput(iinp)
         ishape = mod(iinput(iinp+1),100)
         if ( ielgrp<nelgrp ) then
            call errint ( ielgrp, 1 )
            call errint ( nelgrp, 2 )
            call errsub ( 323, 2, 0, 0)
         else
            if ( ielgrp/=nelgrp ) nelgrp = nelgrp+1
            if ( ielgrp/=nelgrp ) then
               call errint ( ielgrp, 1 )
               call errint ( nelgrp, 2 )
               call errsub ( 323, 2, 0, 0)
            end if
            if ( ishape==0 ) then
               inpelm = 2
            else
               inpelm = 2*(ishape+1)
            end if
            if ( inpelm/=npelm) jnodp = 0
            npelm = max(inpelm,npelm)
            kmeshf(ielgrp) = -10001-ishape
         end if
         iinp = iinp+4
      end do

      do i = 1, isurfs
         ielgrp = iinput(iinp)
         ishape = iinput(iinp+1)
         if ( ielgrp<nelgrp ) then
            call errint ( ielgrp, 1 )
            call errint ( nelgrp, 2 )
            call errsub ( 323, 2, 0, 0)
         else
            if ( ielgrp/=nelgrp ) nelgrp = nelgrp+1
            if ( ielgrp/=nelgrp ) then
               call errint ( ielgrp, 1 )
               call errint ( nelgrp, 2 )
               call errsub ( 323, 2, 0, 0)
            end if
            if ( ishape==0 ) then

!           --- ishape = 0, point elements

               inpelm = 2

            else

!           --- ishape = 1, shape number should be used from surface

               ifirst = iinput(iinp+2)
               ilast  = iinput(iinp+3)
               call msh058surfs ( iinput, ielgeg, ifirst, ilast, ielgrp )

            end if
            if ( inpelm/=npelm ) jnodp = 0
            npelm = max(inpelm,npelm)
            kmeshf(ielgrp) = -10001-ishape
         end if
         iinp = iinp+4
         iinp = iinp+1+max(0,iinput(iinp))

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh058conn'

      end if
      call erclos ( 'msh058conn' )

      end
