      subroutine msh076curvs ( iinput, iinp, icurvs, inodpbef, ikelmc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-03-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill array kmeshc for interface elements
!     curve elements only
! **********************************************************************
!
!                       KEYWORDS
!
!     interface_element
!     mesh
!     mesh_generation
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, icurvs, inodpbef, ikelmc

!     icurvs         i    Number of parts where curve interface elements are given
!     iinp          i/o   last position used in array iinput with respect to
!                         interface elements
!     iinput         i    integer input array with respect to the mesh
!     ikelmc        i/o   last position used in kmesh part c
!     inodpbef       i    First nodal point to be used for extra points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istar3, ielgrp, ileft, idir1, iright, idir2, n, &
              istar1, istar2, inpelm, j, k, ipos, inodp, l, i

!     i              Counting variable
!     idir1          Direction of first curve (1=positive, -1=negative)
!     idir2          Direction of second curve (1=positive, -1=negative)
!     ielgrp         element group number
!     ileft          Left curve in special element
!     inodp          Nodal point number
!     inpelm         number of points per element
!     ipos           Position in array
!     iright         Right curve in special element
!     istar1         Starting position of first curve in KMESHM
!     istar2         Starting address of KMESHM part 2 in KMESHM
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     j              Counting variable
!     k              Counting variable
!     l              General loop variable
!     n              Number of elements created by mesh interface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh076curvs' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from msh076curvs'

      end if
      if ( ierror/=0 ) go to 1000

      istar3 = kmeshm(2)-1
      do i = 1, icurvs
         ielgrp = iinput(iinp)
         ishape = iinput(iinp+1)-49
         ileft = iinput(iinp+2)
         idir1 = sign(1,ileft)
         ileft = abs(ileft)
         iright = iinput(iinp+3)
         idir2 = sign(1,iright)
         iright = abs(iright)
         n = kmeshm(ileft+5)-kmeshm(ileft+4)
         n = (n-1)/ishape

         kmeshg(ielgrp) = kmeshg(ielgrp)+n
         if ( idir1>0) then
            istar1 = istar3+kmeshm(ileft+4)
         else
            istar1 = istar3-1+kmeshm(ileft+5)
         end if
         if ( idir2>0) then
            istar2 = istar3+kmeshm(iright+4)
         else
            istar2 = istar3-1+kmeshm(iright+5)
         end if

         inpelm = ishape + 1
         do j = 1, n
            do k = 1, inpelm
               ipos = istar1+(k-1)*idir1
               inodp = kmeshm(ipos)
               kmeshc(ikelmc+2*k-1) = inodp
               if ( inodp>=inodpbef ) then

!              --- New node, coordinates must be filled

                  coor(:,inodp) = curvesmsh(:,ipos-istar3)

               end if
               ipos = istar2+(k-1)*idir2
               inodp = kmeshm(ipos)
               kmeshc(ikelmc+2*k) = inodp
               if ( inodp>=inodpbef ) then

!              --- New node, coordinates must be filled

                  coor(:,inodp) = curvesmsh(:,ipos-istar3)

               end if
            end do
            ikelmc = ikelmc+2*inpelm
            istar1 = istar1+ishape*idir1
            istar2 = istar2+ishape*idir2

         end do
         inpelm = 2*inpelm

         iinp = iinp+4

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End msh076curvs'

      end if
      call erclos ( 'msh076curvs' )

      end
