      subroutine mshareasur ( kmeshc, nelem, coor, area, iprint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-07-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the area of a region divided into linear triangles
!     If necessary print area of each triangle
! **********************************************************************
!
!                       KEYWORDS
!
!     area
!     triangle
!     mesh_generation
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(3,*), nelem, iprint
      double precision coor(2,*), area

!     area           o    Area of region
!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     iprint         i    Print level
!                         0: no print
!                         1: print area of complete region
!                         2: print area of each triangle as well as accumulated
!                            area
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nelem          i    Number of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem
      double precision elemarea

!     elemarea       Area of element
!     ielem          Element number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHAREA        Compute the area of the triangle spanned by x1-x2-x3
!                    The sign of the are defines the direction
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshareasur' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshareasur'

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

      if ( iprint.ge.2 ) write(irefwr,*) 'ielem,  element area', &
       ' accumulated  area'

      area = 0d0
      do ielem = 1, nelem

         call msharea ( coor(1,kmeshc(1,ielem)), &
                        coor(1,kmeshc(2,ielem)), &
                        coor(1,kmeshc(3,ielem)), elemarea )
         area = area + elemarea
         if ( iprint.ge.2 ) write(irefwr,*) ielem, elemarea, area
         if ( elemarea.lt.0 ) then

!        --- arae of element negative, wrong direction?

            call prinin ( kmeshc(1,ielem), 3, 'element' )

         end if

      end do  ! ielem = 1, nelem
      if ( iprint.ge.1 ) write(irefwr,*) 'area is ',  area

1000  call erclos ( 'mshareasur' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshareasur'

      end if

      end
