      function mshareatria ( kmeshc, coor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-04-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Return with area of triangle, with nodes given in kmeshc
!    Linear triangles only
!    2d
! **********************************************************************
!
!                       KEYWORDS
!
!     area
!     2d
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(3)
      double precision mshareatria, coor(2,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     kmeshc         i    kmesh part c, contains the nodal point numers of the
!                         element
!     mshareatria    o    area of triangle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision area

!     area           Area of element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHAREA        Compute the area of the triangle spanned by x1-x2-x3
!                    The sign of the are defines the direction
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshareatria' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshareatria'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call msharea ( coor(1,kmeshc(1)), coor(1,kmeshc(2)), &
                     coor(1,kmeshc(3)), area )
      mshareatria = abs(area)

1000  call erclos ( 'mshareatria' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshareatria'

      end if  ! ( debug )

      end
