      subroutine mshbackgrid3d ( nx, ny, nz, ox, oy, oz, dx, dy, dz, &
                                 coor, kmeshc, kmeshm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-04-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create mesh for background rectangular grid in R^3
!     Arrays kmesh, kmeshc and coor are filled
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nx, ny, nz, kmeshc(8,(nx-1)*(ny-1)*(nz-1)), kmeshm(*)
      double precision coor(3,nx,ny,nz), ox, oy, oz, dx, dy, dz

!     coor           o    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     dx             i    step size in x-direction
!     dy             i    step size in y-direction
!     dz             i    step size in z-direction
!     kmeshc         o    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmeshm         o    KMESH part m, contains nodal point numbers of the
!                         curves
!     nx             i    number of points in x direction
!     ny             i    number of points in y direction
!     nz             i    number of points in z direction
!     ox             i    x-coordinate of origin
!     oy             i    y-coordinate of origin
!     oz             i    z-coordinate of origin
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, ielem, jshift, kshift

!     i              Counting variable
!     ielem          Element number
!     j              Counting variable
!     jshift         shift in j-direction
!     k              Counting variable
!     kshift         shift in k-direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHRECT_GRID3D Create coordinates for rectangular grid in R^3
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill coordinates
!     Fill kmeshc
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshbackgrid3d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshbackgrid3d'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Fill coordinates

      call mshrect_grid3d ( coor, nx, ny, nz, ox, oy, oz, dx, dy, dz )

!     --- Fill kmeshc

      ielem = 0
      kshift = 0
      do k = 1, nz-1  ! loop over layers

         jshift = kshift
         do j = 1, ny-1  ! loop over rows

            do i = 1, nx-1  ! loop over columns

               ielem = ielem+1
               kmeshc(1,ielem) = i+jshift
               kmeshc(2,ielem) = i+1+jshift
               kmeshc(3,ielem) = i+nx+1+jshift
               kmeshc(4,ielem) = i+nx+jshift
               kmeshc(5,ielem) = i+jshift+nx*ny
               kmeshc(6,ielem) = i+1+jshift+nx*ny
               kmeshc(7,ielem) = i+nx+1+jshift+nx*ny
               kmeshc(8,ielem) = i+nx+jshift+nx*ny

            end do  ! i = 1, nx-1

            jshift = jshift+nx

         end do  ! j = 1, ny-1

         kshift = kshift+ny*nx

      end do  ! k = 1, nz-1

!     --- Fill kmeshm

      kmeshm(1) = 0             ! ninner
      kmeshm(2) = 7             ! start part 3
      kmeshm(3) = 11            ! start part 4
      kmeshm(4) = 0             ! length part 4
      kmeshm(5) = 1             ! start of first curve
      kmeshm(6) = 6             ! end of first curve + 1

!     --- Fill first part of boundary

      kmeshm(7) = 1             ! first point of curve
      kmeshm(8) = nx            ! second point of curve
      kmeshm(9) = nx*ny         ! third point of curve
      kmeshm(10) = nx*ny*nz     ! foruth point of curve
1000  call erclos ( 'mshbackgrid3d' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshbackgrid3d'

      end if

      end
