      subroutine mshcheckcurves ( ipointcurv, iinput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-11-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if the single curves do not intersect
!     If they intersect an error message is printed
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ipointcurv(*), iinput(*)

!     iinput         i    integer input array, see mshrea
!     ipointcurv     i    Array containing the relative starting addresses
!                         of all curves in the part of iinput referring
!                         to curves
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iwork(ncurvs)
      double precision xbox(ndim,ncurvs,2)

!     iwork          integer work array of length ncurvs
!                    iwork(i) = -1: curve is empty
!                    iwork(i) = 0: curve is single
!                    iwork(i) = 1: curve is compound
!     xbox           Is used to store the minimum and maximum coordinates of
!                    all curves
!                    xbox(i,j,1)  minimum i^th coordinate of curve j
!                    xbox(i,j,2)  maximum i^th coordinate of curve j
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCURVBOX     Create a box around all single curves
!     MSHCURVINTERS  Give an error if two curves intersect
!     PRINCURV       Print coordinates of curves
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( ndim=1 ) exit
!     Compute box xmin,xmax, ymin,ymax, zmin,zmax around single curves
!     For all single curves icurnr do
!        for all single curve with sequence number>icurnr do
!           If ( possible intersection ) then
!              if ( subparts of curve intersect ) then
!                 give error message
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcheckcurves' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcheckcurves'
         write(irefwr,100) 'ndim, ncurvs', ndim, ncurvs
100      format ( a, 1x, (10i6) )
         call princurv ( 0, icurvsmsh, curvesmsh, ncurvs, ndim, iwork, &
                         'curves' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- if ( ndim=1 ) exit

      if ( ndim/=2 .or. ncurvs<2 ) go to 1000

!     --- Compute box xmin,xmax, ymin,ymax, zmin,zmax around single curves

      call mshcurvbox ( xbox, iwork, ipointcurv, iinput )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( xbox(1,1,1), ncurvs, 2, &
                        'minimum coordinates of curves' )
         call prinrl1 ( xbox(1,1,2), ncurvs, 2, &
                        'maximum coordinates of curves' )

      end if ! ( debug )

!     --- For all single curves icurnr do
!            for all single curve with sequence number>icurnr do
!               If ( possible intersection ) then
!                  if ( subparts of curve intersect ) then
!                     give error message

      call mshcurvinters ( ncurvs, iwork, xbox, icurvsmsh, curvesmsh )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcheckcurves'

      end if
      call erclos ( 'mshcheckcurves' )

      end
