      subroutine mshchecktets ( ishape, inpelm, kmeshc, nelem, ivolnr, &
                                coor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-02-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the minimal ratio between radius of inner sphere and
!     outer sphere of all tets in a volume
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     check
!     quality
!     tetrahedron
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ishape, inpelm, nelem, kmeshc(inpelm,nelem), ivolnr
      double precision coor(*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points  of the volume
!     inpelm         i    number of points per element
!     ishape         i    Shape number of elements
!     ivolnr         i    Volume sequence number
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements of the volume
!     nelem          i    Number of elements of the volume
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i1, i2, i3, i4, ielem
      double precision vermin, ratio

!     i1             Index of vertex in element
!     i2             Index of vertex in element
!     i3             Index of vertex in element
!     i4             Index of vertex in element
!     ielem          Element number
!     ratio          ratio of radius of inner sphere and outer sphere of element
!     vermin         minimum value of ratio
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH661         Compute ratio of inner and outer sphere of tetrahedron
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshchecktets' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshchecktets'
         write(irefwr,1) 'ishape, inpelm, nelem, ivolnr', &
                          ishape, inpelm, nelem, ivolnr
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Find positions of vertices depending on shape of elements

      select case ( ishape )

         case(:10, 13, 14, 17, 19: )

!        --- Not a tetrahedron, jump

            go to 1000

         case(11, 18)

!        --- Linear tetrahedron

            i1 = 1
            i2 = 2
            i3 = 3
            i4 = 4

         case(12, 15, 16)

!        --- Quadratic tetrahedron

            i1 = 1
            i2 = 3
            i3 = 5
            i4 = 10

      end select  ! case ( ishape )

      vermin = rinfin

      do ielem = 1, nelem

         call msh661( coor, kmeshc(i1,ielem), kmeshc(i2,ielem), &
                      kmeshc(i3,ielem), kmeshc(i4,ielem), ratio )

         vermin = min(ratio,vermin)
         if ( debug .and. ratio<0.1d0 ) then

            write(irefwr,*) '1: Element ',ielem,' has ratio ', ratio
            call prinin ( kmeshc(1,ielem), inpelm, 'nodes' )

         end if

      end do

      write(irefwr,100) ivolnr, vermin
100   format ( 'smallest ratio of elements of volume ', i3, &
               ' is ', ss, 1pe15.6e2 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshchecktets'

      end if
      call erclos ( 'mshchecktets' )

      end
