      subroutine mshcompdispl1 ( npoint, coorold, coornew, displacement, &
                                 kprobf, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-12-2008
!
!   copyright (c) 2008-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute displacement vector as difference of coordinates
!     indprf#0, hence kprobf must be used
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, kprobf(npoint+1), ndim
      double precision coorold(ndim,npoint), coornew(ndim,npoint), &
                       displacement(*)

!     coornew        i    Array containing the new coordinates of the nodes
!     coorold        i    Array containing the old coordinates of the nodes
!     displacement   o    Contains the computed displacement vector
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     ndim           i    Dimension of the space.
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart, nunkp

!     i              Counting variable
!     istart         First position used at a part
!     nunkp          number of unknowns in a point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcompdispl1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcompdispl1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istart = 0
      do i = 1, npoint

         nunkp = kprobf(i+1)-istart  ! number of degrees of freedom in displ

         if ( nunkp>=ndim ) then

!        --- nunkp>=ndim, hence there are at least ndim degrees of freedom

            displacement(istart+1:istart+ndim) = &
               coornew(:,i)-coorold(:,i)

         end if  ! ( nunkp>=ndim )

         istart = istart+nunkp  ! raise istart

      end do  ! i = 1, npoint

1000  call erclos ( 'mshcompdispl1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcompdispl1'

      end if  ! ( debug )

      end
