      subroutine mshcompdispl22 ( inpada, rinada, nunkp, coorold, &
                                  kprobf, indprf, usol, iwork, coornw, &
                                  coorol, anorm, kmeshm, &
                                  closed, n, uold, maxphys, numvecs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-01-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     recompute co-ordinates of curves
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kprobf(*), iwork(*), maxphys, inpada(*), nunkp, indprf, &
              kmeshm(*), n, numvecs
      double precision coornw(2,n), usol(*), coorold(2,*), &
                       coorol(2,n), anorm(2,n), rinada(*), uold(*)
      logical closed

!     anorm          i    real work array containing the norms in the points
!                         on the curves in icurves
!     closed         i    If true the boundary is closed
!     coornw        i/o   Array containing the new co-ordinates along the
!                         boundary
!     coorol         i    Array containing the old co-ordinates along the
!                         boundary
!     coorold        i    Contains the coordinates of the original mesh
!     indprf         i    indication whether kprob part f is used or not
!     inpada         i    Integer array to be filled by the user, indicating
!                         how the boundary must be adapted.
!     iwork          i    integer work array containing the nodal points of the
!                         curves in icurves at output
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     maxphys        i    Maximum number of unknowns per point
!     n              i    number of nodes in boundary
!     numvecs        i    Number of vectors in iseqvecs
!     nunkp          i    maximal number of degrees of freedom in nodal points
!     rinada         i    Real input array with data as described by inpada
!     uold           i    Array containing the old solutions
!     usol           i    solution array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer linear, iadapt, iellist

!     iadapt         Indicates how the boundary must be adapted, see INPADA
!     iellist        Dummy
!     linear         Parameter linear as described in inpada(7)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHADCURVS     Adapt coordinates at boundary (2d)
!     MSHCORRFREND   correct end points for free separation points by projection
!     MSHUPDQUAD     Adapt updated curve for quadratic elements
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2990   Value of iadapt not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Make initializations
!     compute new co-ordinates and store in array coornw
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- initializations

      call eropen ( 'mshcompdispl22' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information in mshcompdispl22'
         call prinin ( inpada, 15, 'inpada' )
         call prinrl1 ( coornw, n, 2, 'coornw/in' )

      end if
      if ( ierror/=0 ) go to 1000

      iadapt = inpada(4)
      linear = inpada(7)
      if ( iadapt==6 .or. iadapt==7 .or. iadapt==9 ) then

!     --- Value of iadapt not yet implemented (iellist)

         call errint ( iadapt, 1 )
         call errsub ( 2990, 1, 0, 0 )
         go to 1000

      end if  ! ( iadapt==6 .or. iadapt==7 )

!     --- Adapt coordinates at boundary

      call mshadcurvs ( usol, iwork, n, indprf, kprobf, &
                        nunkp, iadapt, rinada, coorol, &
                        coornw, anorm, inpada, closed, maxphys, &
                        numvecs, uold, iellist )
      if ( debug ) call prinrl1 ( coornw, n, 2, 'coornw' )

!     --- correct end points for free separation points

      call mshcorrfrend ( coorol, coornw, inpada, rinada, n, &
                          kmeshm, coorold )
      if ( debug ) &
         call prinrl1(coornw, n, 2, 'coornw after mshcorrfrend')

!     --- this part calculates the shortest distance between every
!         'mid' and the line between the endpoints of an element side.
!         this 'mid' nodal point is put back to the 'mid' shift
!         parallel to the line between the end-points.
!         meaning of some variables used in this part:
!                !                X  nodal point p
!                !      a        /
!         y-coor !      | \     /     X    new position of point p
!                !      |   \  /
!                !  dy  |     \--> line between a and b
!                !      |       \
!                !      |   dx    \
!                !      |___________b
!                0------------------------------- &
!                                           x-coor

      if ( iadapt/=8 .and. linear==2 ) then

!     --- quadratic elements

         call mshupdquad ( coornw, n )

      end if

1000  call erclos ( 'mshcompdispl22' )
      if ( debug ) then

!     --- Debug information

         call  prinrl1(coornw, n, 2, 'coornw end mshcompdispl22')
         write(irefwr,*) 'End debug in mshcompdispl22'

      end if

      end
