      subroutine mshcompdispl32 ( inpada, rinada, nunkp, kprobf, indprf, &
                                  usol, iwork, coornw, coorol, anorm, &
                                  n, psi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-01-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     recompute co-ordinates of surfaces
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kprobf(*), iwork(*), inpada(*), nunkp, indprf, n
      double precision coornw(3,n), usol(*), &
                       coorol(3,n), anorm(3,n), rinada(*), psi(*)

!     anorm          i    real work array containing the norms in the points
!                         on the curves in icurves
!     coornw        i/o   Array containing the new co-ordinates along the
!                         boundary
!     coorol         i    Array containing the old co-ordinates along the
!                         boundary
!     indprf         i    indication whether kprob part f is used or not
!     inpada         i    Integer array to be filled by the user, indicating
!                         how the boundary must be adapted.
!     iwork          i    integer work array containing the nodal points of the
!                         curves in icurves at output
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     n              i    number of nodes in boundary
!     nunkp          i    maximal number of degrees of freedom in nodal points
!     psi            i    Contains the "pseudo" stream function at the
!                         free surface
!     rinada         i    Real input array with data as described by inpada
!     usol           i    solution array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iadapt

!     iadapt         Indicates how the boundary must be adapted, see INPADA
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHADSURFS     Adapt coordinates at boundary (3d)
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2990   Value of iadapt not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Make initializations
!     compute new co-ordinates and store in array coornw
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- initializations

      call eropen ( 'mshcompdispl32' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information in mshcompdispl32'
         call prinin ( inpada, 15, 'inpada' )
         call prinrl1 ( coornw, n, 3, 'coornw/in' )

      end if
      if ( ierror/=0 ) go to 1000

      iadapt = inpada(4)
      if ( iadapt==6 .or. iadapt==7 .or. iadapt==9 ) then

!     --- Value of iadapt not yet implemented (iellist)

         call errint ( iadapt, 1 )
         call errsub ( 2990, 1, 0, 0 )
         go to 1000

      end if  ! ( iadapt==6 .or. iadapt==7 )

!     --- Adapt coordinates at boundary

      call mshadsurfs ( usol, iwork, n, indprf, kprobf, &
                        nunkp, iadapt, rinada, coorol, &
                        coornw, anorm, inpada, psi )
      if ( debug ) call prinrl1 ( coornw, n, 3, 'coornw' )

!     --- correct end points for free separation points

!         call mshcorrfrend ( coorol, coornw, inpada, rinada, n, &
!         +                    kmeshm, coorold )
!         if ( debug ) &
!         +   call prinrl1(coornw, n, 3, 'coornw after mshcorrfrend')

!         --- this part calculates the shortest distance between every
!         'mid' and the line between the endpoints of an element side.
!         this 'mid' nodal point is put back to the 'mid' shift
!         parallel to the line between the end-points.
!         meaning of some variables used in this part:
!                !                X  nodal point p
!                !      a        /
!         y-coor !      | \     /     X    new position of point p
!                !      |   \  /
!                !  dy  |     \--> line between a and b
!                !      |       \
!                !      |   dx    \
!                !      |___________b
!                0------------------------------- &
!                                           x-coor

!         if ( iadapt/=8 .and. linear==2 ) then

!         --- quadratic elements

!          call mshupdquad ( coornw, n )

!         end if

1000  call erclos ( 'mshcompdispl32' )
      if ( debug ) then

!     --- Debug information

         call  prinrl1(coornw, n, 3, 'coornw end mshcompdispl32')
         write(irefwr,*) 'End debug in mshcompdispl32'

      end if

      end
