      subroutine mshcopykmeshad ( kmesh1, kmesh2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-11-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy kmesh part ad from one mesh to another
!
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     mesh
!     obstacle
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh1, kmesh2

!     kmesh1         i    Standard SEPRAN array containing information of mesh 1
!     kmesh2        i/o   Contains copy of old mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart, iseqkm1, iseqkm2

!     i              Counting variable
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     istart         Starting position of kmesh part ad
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPINITKMESHAD Initialize kmesh ad
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcopykmeshad' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcopykmeshad'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkm1 = kmesh1
      iseqkm2 = kmesh2
      iseqkm = iseqkm2

      if ( km(iseqkm1)%kmesh(52)==0 ) go to 1000 ! kmesh ad has not been filled

      if ( km(iseqkm2)%kmesh(52)==0 ) call sepinitkmeshad
      kmeshad => km(iseqkm)%kmeshad
      kmeshad = 0

      if ( associated(km(iseqkm1)%ipointobst) ) then

!     --- Copy ipointobst

         if ( associated(km(iseqkm)%ipointobst) ) &
            deallocate(km(iseqkm)%ipointobst)
         length = size(km(iseqkm1)%ipointobst)
         allocate(km(iseqkm)%ipointobst(length))
         ipointobst => km(iseqkm)%ipointobst
         ipointobst = km(iseqkm1)%ipointobst
         kmeshad(1) = length

      end if  ! ( associated(km(iseqkm1)%ipointobst) )

      if ( associated(km(iseqkm1)%ielemobst) ) then

!     --- Copy ielemobst

         if ( associated(km(iseqkm)%ielemobst) ) &
            deallocate(km(iseqkm)%ielemobst)
         length = size(km(iseqkm1)%ielemobst)
         allocate(km(iseqkm)%ielemobst(length))
         ielemobst => km(iseqkm)%ielemobst
         ielemobst = km(iseqkm1)%ielemobst
         kmeshad(2) = length

      end if  ! ( associated(km(iseqkm1)%ielemobst) )

      if ( associated(km(iseqkm1)%icrossobst) ) then

!     --- Copy icrossobst

         if ( associated(km(iseqkm)%icrossobst) ) &
            deallocate(km(iseqkm)%icrossobst)
         length = size(km(iseqkm1)%icrossobst)
         allocate(km(iseqkm)%icrossobst(length))
         icrossobst => km(iseqkm)%icrossobst
         icrossobst = km(iseqkm1)%icrossobst
         kmeshad(3) = length

      end if  ! ( associated(km(iseqkm1)%icrossobst) )

      if ( associated(km(iseqkm1)%rcrossobst) ) then

!     --- Copy rcrossobst

         if ( associated(km(iseqkm)%rcrossobst) ) &
            deallocate(km(iseqkm)%rcrossobst)
         length = size(km(iseqkm1)%rcrossobst)
         allocate(km(iseqkm)%rcrossobst(length))
         rcrossobst => km(iseqkm)%rcrossobst
         rcrossobst = km(iseqkm1)%rcrossobst
         kmeshad(4) = length

      end if  ! ( associated(km(iseqkm1)%rcrossobst) )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshcopykmeshad'

      end if  ! ( debug )
      call erclos ( 'mshcopykmeshad' )

      end
