      subroutine mshcurves2open ( istore, storenumedges, icount, &
                                  kmeshm, nmeshl, nmeshc, &
                                  iworkmarkedgs, iworkmarknods, &
                                  lenkmeshmtmp, istartm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 29-04-2005 Fortran 90
!        version  1.1  date 29-01-2005 Correction error message
!        version  1.0  date 18-10-2004
!
!   copyright (c) 2004-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store next open boundary in kmeshm
!     This boundary is part of the intersection of two surfaces
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     curve
!     boundary
!     intersection
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer istore(*), storenumedges(*), icount, &
              kmeshm(*), nmeshl(*), nmeshc(*), iworkmarkedgs(*), &
              iworkmarknods(*), lenkmeshmtmp, istartm

!     icount         o    Number of nodes stored
!     istartm        i    Starting address of next curve in kmeshm
!     istore         i    Contains the set of nodes
!     iworkmarkedgs i/o   integer work array of length nedges
!                         if iworkmarkedgs(i)>0, i is stored in istoreedges
!     iworkmarknods i/o   integer work array of length npoint
!                         if iworkmarknods(i)>0, i is stored in
!                         ordered curve
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     lenkmeshmtmp   i    Length of array kmeshmtmp (here kmeshm)
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
!     nmeshl         i    nmesh part l, contains edge numbers corresponding
!                         to nodes
!     storenumedges  i    Array of length numpoints to store the number of
!                         edges that are in the curve per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, ipstart, inode, iedge, nnodes, i
      logical finished

!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     i              Counting variable
!     iedge          edge sequence number
!     inode          Node sequence number
!     ipstart        Starting address in array
!     istart         First position used at a part
!     nnodes         Number of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2737   node is not end point of edge
! **********************************************************************
!
!                       PSEUDO CODE
!
!     istart := first entry in storenumedges with 1 edge
!     inode := istore(istart); iworkmarknods(inode) := 0; icount := 1
!     kmeshm(icount) := inode
!     finished := false
!     do  while ( not finished )
!        iedge := free edge connected to inode
!        inode := other node connected to iedge
!        icount := icount+1;  kmeshm(icount) := inode
!        if ( storenumedges(iworkmarknods(inode)) = 1 ) then
!           finished = true
!        icount := icount+1;  kmeshm(icount) := inode;
!        iworkmarkedgs(iedge) := 0
!        iworkmarknods(inode)) := 0
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshcurves2open' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshcurves2open'
         write(irefwr,*) 'istartm, lenkmeshmtmp ', istartm, lenkmeshmtmp

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- istart := first entry in storenumedges with 1 edge

      finished = .false.
      istart = 0
      do while ( .not. finished )

!     --- first entry not yet found

         istart = istart+1
         if ( storenumedges(istart)==1 ) then

!        --- Node with 1 edge found

            finished = .true.

         end if  ! ( storenumedges(istart)==1 )

      end do  ! while ( .not. finished )

!     --- inode := istore(istart); iworkmarknods(inode) := 0; icount := 1
!         kmeshm(icount) := inode

      inode = istore(istart)      ! first node in kmeshm
      iworkmarknods(inode) = 0    ! node is not available anymore
      icount = 1                  ! number of nodes stored
      kmeshm(icount) = inode
      if ( debug ) write(irefwr,*) 'istart, inode ', istart, inode

!     --- finished := false

      finished = .false.
      do while ( .not. finished )

!     --- Last node not yet found
!         iedge := edge connected to inode

         ipstart = nmeshl(inode)   ! starting address of node in nmeshl
         do i = 1, nmeshl(ipstart)
            iedge = nmeshl(ipstart+i)   ! next edge coupled to node
            if ( iworkmarkedgs(iedge)>0 ) go to 100
         end do  ! i = 1, nmeshl(ipstart)

!        --- if the code comes here, no edge can be found

         if ( debug ) write(irefwr,*) 'no edge found'
         finished = .true.
         go to 1000

!        --- inode := other node connected to iedge
!            icount := icount+1;  kmeshm(icount) := inode

100      ipstart = nmeshc(iedge)
         nnodes = nmeshc(ipstart)

         if ( debug ) then
            write(irefwr,*) 'iedge ', iedge
            call prinin ( nmeshc(ipstart+1), nnodes, 'nodes' )
         end if  ! ( debug )

         call inichkarlen ( istartm+icount+nnodes, lenkmeshmtmp, &
                            'kmeshmtmp' )
         if ( ierror/=0 ) go to 1000

         if ( inode==nmeshc(ipstart+1) ) then

!        --- inode is first node of edge

            do i = 1, nnodes-1
               kmeshm(icount+i) = nmeshc(ipstart+i+1)
            end do  ! i = 1, nnodes-1

         else if ( inode==nmeshc(ipstart+nnodes) ) then

!        --- inode is last node of edge, reversed order

            do i = 2, nnodes
               kmeshm(icount+i-1) = nmeshc(ipstart-i+nnodes+1)
            end do  ! i = 1, nnodes-1

         else

!        --- inode is not an end node, error

            call errint ( inode, 1 )
            call errint ( iedge, 2 )
            call errsub ( 2737, 2, 0, 0 )
            go to 1000

         end if  ! ( inode==nmeshc(ipstart+1) )

!        --- icount := icount+1;  kmeshm(icount) := inode
!            iworkmarkedgs(iedge) := 0

         if ( debug ) &
            call prinin ( kmeshm(icount+1), nnodes-1, 'kmeshm' )
         icount = icount + nnodes-1
         inode = kmeshm(icount)
         iworkmarkedgs(iedge) = 0
         if ( debug ) then
            write(irefwr,*) 'inode, icount ', inode, icount
            write(irefwr,*) 'iworkmarknods(inode) ',iworkmarknods(inode)
            write(irefwr,*) 'storenumedges(iworkmarknods(inode)) ', &
               storenumedges(iworkmarknods(inode))
         end if  ! ( debug )

         if ( storenumedges(iworkmarknods(inode))==1 ) then

!        --- storenumedges(iworkmarknods(inode)) = 1, end point found
!            finished = true

            finished = .true.

         end if  ! ( storenumedges(iworkmarknods(inode))==1 ) then
         iworkmarknods(inode) = 0    ! node is not available anymore

!        --- iworkmarknods(inode) := 0

         do i = 1, nnodes-1
            iworkmarknods(kmeshm(icount-nnodes+1+i)) = 0
         end do  ! i = 1, nnodes-1

      end do  ! while ( .not. finished )

1000  call erclos ( 'mshcurves2open' )

      if ( debug ) then

!     --- Debug information

         call prinin ( kmeshm, icount, 'kmeshm' )
         write(irefwr,*) 'End mshcurves2open'

      end if

      end
