      subroutine mshdefline ( intlinedef, reallinedef, xstart, tangent )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-11-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Define line defined by intlinedef and reallinedef in terms of xstart
!     and tangent vector
! **********************************************************************
!
!                       KEYWORDS
!
!     line
!     coordinates
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer intlinedef
      double precision reallinedef(*), xstart(2), tangent(2)

!     intlinedef     i    Defines the way the line is given in reallinedef
!                         Possible values:
!                         1: the start point of the line (xstart,ystart) is
!                            given in reallinedef(1:2)
!                            the angle of rotation with respect to the positive
!                            x-axis is given in reallinedef(3)
!     reallinedef    i    Contains the values defining the line as defined
!                         by intlinedef
!     tangent        o    Contains the tangential vector
!     xstart         o    Starting point of line
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision angle

!     angle          angle of rotation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2147   Parameter out of range
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( intlinedef == 1 ) then
!        xstart given in realline(1:2)
!        angle given in realline(3)
!        Compute tangent
!     else
!        error possibility not implemented
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshdefline' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshdefline'

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

1000  call erclos ( 'mshdefline' )

      if ( intlinedef.eq.1 ) then

!     --- intlinedef == 1
!         xstart given in realline(1:2)
!         angle given in realline(3)

         xstart(1) = reallinedef(1)
         xstart(2) = reallinedef(2)
         angle = reallinedef(3)

!        --- Compute tangent

         tangent(1) = cos(angle)
         tangent(2) = sin(angle)

      else

!     --- error possibility not implemented

         call errchr ( 'intlinedef', 1 )
         call errint ( intlinedef, 1 )
         call errint ( 1, 2 )
         call errint ( 1, 3 )
         call errsub ( 2147, 3, 0, 1 )

      end if  ! ( intlinedef.eq.1 )

      if ( debug ) then

!     --- Debug information

         call prinrl ( xstart, 2, 'xstart' )
         call prinrl ( tangent, 2, 'tangent' )
         write(irefwr,*) 'ange ', angle
         write(irefwr,*) 'End mshdefline'

      end if

      end
