      subroutine mshfillcompcur ( ipointer, subcurvs, lensubcurvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-12-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create set of compound curves by combining curves in the
!     intersection of surfaces
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     surface
!     curve
!     compound_curve
!     intersection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ipointer(*), subcurvs(*), lensubcurvs

!     ipointer       o    pointer array of length nextracurvs
!                         contains starting addresses in array subcurvs
!     lensubcurvs    o    Extra length required for compound curves
!     subcurvs       o    Contains subcurve numbers of compound curves
!                         sequentially
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isurnr, numcurves, iwork(ncurvs), ipstart, &
              jpstart, jsurnr, ncount, numcrv, istart, nextracurvs

!     i              Counting variable
!     ipstart        Starting address in array
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     iwork          integer work array of length ncurvs
!                    iwork(i) = 0 if curve i not in isurnr
!                    iwork(i) = 1 if curve i in isurnr
!     jpstart        Starting address in array
!     jsurnr         Surface sequence number
!     ncount         Counter
!     nextracurvs    Number of extra curves to be created
!     numcrv         number of curves in surface jsurnr
!     numcurves      Number of curves in surface isurnr
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     nextracurvs := 0
!     istart := 0
!     Clear array iwork(1:ncurvs)
!     for isurnr := 1 (1) nsurfs do
!        Set iwork(i) 1 for all curves in isurnr
!        For jsurnr := isurnr+1 (1) nsurfs do
!           ncount := number of common curves of isurnr and jsurnr
!           if ( ncount > 1 ) then
!              nextracurvs := nextracurvs+1
!              Store ncount curves in subcurvs from istart+1 to istart+ncount
!              istart := istart+ncount
!              ipointer(nextracurvs) := istart
!        Clear iwork
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfillcompcur' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfillcompcur'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     nextracurvs := 0
!     istart := 0

      nextracurvs = 0
      istart = 0

!     --- Clear array iwork(1:ncurvs)

      do i = 1, ncurvs
         iwork(i) = 0
      end do  ! i = 1, ncurvs


!     --- for isurnr := 1 (1) nsurfs do

      do isurnr = 1, nsurfs

!     --- Set iwork(i) 1 for all curves in isurnr

         numcurves = kmeshs(isurnr+1)-kmeshs(isurnr)  ! number of curves in
                                                      ! surface isurnr
         ipstart = kmeshs(isurnr)  ! staring address of curves in isurnr
         do i = 1, numcurves
            iwork(abs(kmeshs(ipstart+i))) = 1
         end do  ! i = 1, numcurves

!        --- For jsurnr := isurnr+1 (1) nsurfs do

         do jsurnr = isurnr+1, nsurfs

!        --- ncount := number of common curves of isurnr and jsurnr

            ncount = 0
            numcrv = kmeshs(jsurnr+1)-kmeshs(jsurnr)  ! number of curves in
                                                      ! surface jsurnr
            jpstart = kmeshs(jsurnr)  ! staring address of curves in jsurnr
            do i = 1, numcrv
               if ( iwork(abs(kmeshs(jpstart+i)))==1 ) &
                  ncount = ncount+1
            end do  ! i = 1, numcrv

            if ( ncount>1 ) then

!           --- More than one curve found
!               nextracurvs := nextracurvs+1
!               ipointer(nextracurvs) := istart
!               Store ncount curves in subcurvs from istart+1 to istart+ncount
!               istart := istart+ncount

                nextracurvs = nextracurvs+1
                if ( debug ) then
                   write(irefwr,100) 'isurnr, jsurnr, ncount, numcrv', &
                                      isurnr, jsurnr, ncount, numcrv
100                format ( a, 1x, 10i6 )
                end if  ! ( debug )

                ncount = 0
                do i = 1, numcrv
                   if ( iwork(abs(kmeshs(jpstart+i)))==1 ) then

!                  --- Common curve found, store

                      ncount = ncount+1
                      subcurvs(istart+ncount) = kmeshs(jpstart+i)
                      if ( debug ) then
                         write(irefwr,100) 'i, ncount, istart, jpstart', &
                                            i, ncount, istart, jpstart
                         write(irefwr,100) 'kmeshs(jpstart+i)', &
                                            kmeshs(jpstart+i)
                      end if  ! ( debug )

                   end if  ! ( iwork(kmeshs(jpstart+i))==1 )

                end do  ! i = 1, numcrv
                istart = istart+ncount
                ipointer(nextracurvs) = istart

            end if  ! ( ncount>1 )

         end do  ! jsurnr = isurnr+1, nsurfs

!        --- Clear iwork

         do i = 1, numcurves
            iwork(abs(kmeshs(ipstart+i))) = 0
         end do  ! i = 1, numcurves

      end do  ! isurnr = 1, nsurfs
      lensubcurvs = istart

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,100) 'lensubcurvs, nextracurvs', &
                            lensubcurvs, nextracurvs
         call prinin ( ipointer, nextracurvs, 'ipointer' )
         call prinin ( subcurvs, lensubcurvs, 'subcurvs' )
         write(irefwr,*) 'End mshfillcompcur'

      end if
      call erclos ( 'mshfillcompcur' )

      end
