      subroutine mshfillextcrvs ( icurvs, curves, bcord, &
                                  iinput, fillnumnod, numnodextcrvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 02-09-2010 Remove intarmsh
!        version  1.0  date 30-06-2008
!
!   copyright (c) 2008-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill coordinates of extra curves in bcord
!
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icurvs(*), iinput(*), numnodextcrvs(*)
      double precision curves(ndim,*), bcord(ibcrds,*)
      logical fillnumnod

!     bcord          o    array containing the coordinates of the interanl
!                         curves
!                         So its starting address in the calling subroutine
!                         must be after the outer boudary part
!     curves         i    array containing the coordinates of the curves
!     fillnumnod     i    If true array numnodextcrvs must be filled
!     icurvs         i    array containing the number of points in the curves
!                         accumulated
!     iinput         i    integer input array, standard for subroutine MESH
!     numnodextcrvs o    Contains the number of nodes for each internal
!                         curve in the sequence of these curves
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, nbc, i, j, icstart, bcstart

!     bcstart        Starting address in array bcord
!     i              Counting variable
!     icstart        Starting address in array curves
!     istart         First position used at a part
!     j              Counting variable
!     nbc            Number of points along a side
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfillextcrvs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfillextcrvs'
         call prinin ( icurvs, ncurvs, 'icurvs' )
         call prinrl1 ( curves, icurvs(ncurvs), ndim, 'curves' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istart = ncurvstart  ! address in iinput for curve numbers
      bcstart = 0  ! starting address in bcord

!     --- Loop over extra curves

      do i = 0, nextracrvs-1

!     --- Find curve number icurnr

         icurnr = abs(iinput(istart+i))
         if ( debug ) write(irefwr,1) 'i, icurnr', i, icurnr

!        --- Compute number of nodes of icurnr

         if ( icurnr==1 ) then
            nbc = icurvs(1)
            icstart = 0
         else
            icstart = icurvs(icurnr-1)
            nbc = icurvs(icurnr)-icstart
         end if  ! ( icurnr==1 )
         if ( debug ) write(irefwr,1) 'nbc', nbc

!        --- Copy curve into bcord

         do j = 1, nbc
            bcord(1:ndim,j+bcstart) = curves(1:ndim,j+icstart)
         end do  ! j = i1, i2

!        --- Fill nbc in numnodextcrvs

         if ( fillnumnod ) numnodextcrvs(i+1) = nbc

!        --- Raise bcstart

         bcstart = bcstart+nbc

      end do  ! i = 1, nextracrvs

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( bcord, nbc, ibcrds, 'bcord_extra' )
         write(irefwr,*) 'End mshfillextcrvs'

      end if  ! ( debug )
      call erclos ( 'mshfillextcrvs' )

      end
