      subroutine mshfillkmeshq1 ( ncurvs, nextracurvs, kmeshq, ipointer, &
                                  subcurves, kmeshm, lenkmeshmext )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-12-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmeshq with information stored in the arrays ipointer
!     and subcurves
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     curve
!     compound_curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncurvs, nextracurvs, kmeshq(*), ipointer(*), subcurves(*), &
              kmeshm(*), lenkmeshmext

!     ipointer       i    pointer array of length nextracurvs
!                         contains starting addresses in array subcurves
!     kmeshm         i    KMESH part m, contains nodal point numbers of the
!                         curves
!     kmeshq         o    KMESH part q, contains the subcurve numbers of
!                         each curve
!     lenkmeshmext   o    Extra length of new array kmeshm
!     ncurvs         i    Number of curves in mesh
!     nextracurvs    i    Number of extra curves to be created
!     subcurves      i    Contains subcurve numbers of compound curves
!                         sequentially
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncurvsnew, i, icurnr

!     i              Counting variable
!     icurnr         Curve sequence number
!     ncurvsnew      New value of ncurvs
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     ncurvsnew := ncurvs+nextracurvs
!     Set kmeshq pos. 1 to ncurvs+1
!     Adapt kmeshq pos. ncurvs+2 to ncurvsnew+1 using ipointer
!     Copy subcurves in kmeshq from position ncurvsnew+2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfillkmeshq1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfillkmeshq1'

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

!     --- ncurvsnew := ncurvs+nextracurvs

      ncurvsnew = ncurvs+nextracurvs
      lenkmeshmext = nextracurvs*2

!     --- Set kmeshq pos. 1 to ncurvs+1

      do i = 1, ncurvs+1
         kmeshq(i) = ncurvsnew+1
      end do  ! i = 1, ncurvs+1

!     --- Adapt kmeshq pos. ncurvs+2 to ncurvsnew+1 using ipointer

      do i = 1, nextracurvs
         kmeshq(i+ncurvs+1) = ipointer(i)+ncurvsnew+1
      end do  ! i = 1, nextracurvs

!     --- Copy subcurves in kmeshq from position ncurvsnew+2

      do i = 1, ipointer(nextracurvs)
         kmeshq(ncurvsnew+1+i) = subcurves(i)
         icurnr = abs(subcurves(i))
         lenkmeshmext = lenkmeshmext + kmeshm(5+icurnr)-kmeshm(4+icurnr)
      end do  ! i = 1, ipointer(nextracurvs)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshfillkmeshq1'

      end if
      call erclos ( 'mshfillkmeshq1' )

      end
