      subroutine mshfillnumextc ( iinput, icurvs, numnodextcrvs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 02-09-2010 Remove intarmsh
!        version  1.0  date 30-06-2008
!
!   copyright (c) 2008-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array numnodextcrvs with number of nodes in all
!     extra curves
! **********************************************************************
!
!                       KEYWORDS
!
!     curve
!     mesh
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), icurvs(*), numnodextcrvs(*)

!     icurvs         i    array containing the number of points in the curves
!                         accumulated
!     iinput         i    integer input array, standard for subroutine MESH
!     numnodextcrvs o    Contains the number of nodes for each internal
!                         curve in the sequence of these curves
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, i, nbc

!     i              Counting variable
!     istart         First position used at a part
!     nbc            Number of points along a side
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshfillnumextc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshfillnumextc'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istart = ncurvstart  ! address in iinput for curve numbers

!     --- Loop over extra curves

      do i = 0, nextracrvs-1

!     --- Find curve number icurnr

         icurnr = abs(iinput(istart+i))
         if ( debug ) write(irefwr,1) 'i, icurnr', i, icurnr

!        --- Compute first and last position of icurnr in icurvs

         if ( icurnr==1 ) then
            nbc = icurvs(1)
         else
            nbc = icurvs(icurnr)-icurvs(icurnr-1)
         end if  ! ( icurnr==1 )
         if ( debug ) write(irefwr,1) 'nbc', nbc

!        --- Put number of nodes in numnodextcrvs

         numnodextcrvs(i+1) = nbc

      end do  ! i = 1, nextracrvs

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshfillnumextc'

      end if  ! ( debug )
      call erclos ( 'mshfillnumextc' )

      end
