      subroutine mshgenestimate ( n, npoint, nelem, nrand )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.0  date 12-11-2007 Use new formula, nrand included
!        version  1.0  date 24-09-2007
!
!   copyright (c) 2007-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute estimates for npoint and nelem in a three dimensional
!     problem for a characteristic value n. (Assume a cube with
!     dimension n and regular division of nodes and elements. Next
!     use extra tolerance to get good estimates.)
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     estimate
!     3d
!     mesh_generation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, npoint, nelem, nrand

!     n              i    characteristic value for nodes in directions
!                         constituting the total volume
!     nelem          o    estimate of number of elements in volume
!     npoint         o    estimate of number of nodes in volume
!     nrand          o    number of triangles in boundary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nipnt

!     debug          parameter to indicate debugging messages yes (true) or
!                    not (false)
!     nipnt          Number of points in boundary
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshgenestimate' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshgenestimate'

      end if
      if ( ierror/=0 ) go to 1000

      npoint = 1.5 * n * n * (n-1) + 100

      nipnt  = int((nrand+5)/2)

      nelem  = 3 * ( 2 * npoint - nipnt )

1000  call erclos ( 'mshgenestimate' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshgenestimate'

      end if

      end
