      subroutine mshintcurvelin ( icurnr, intlinedef, reallinedef, coors )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-11-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     intersect curve icurnr with a straight line
!     Store the coordinates and distance from origin into coors
! **********************************************************************
!
!                       KEYWORDS
!
!     intersection
!     curve
!     line
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icurnr, intlinedef
      double precision reallinedef(*), coors(*)

!     coors          o    Array containing the coordinates and distance
!                         of the intersection from origin
!                         Pos: 1  distance
!                              2-3: coordinates
!     icurnr         i    Curve sequence number
!     intlinedef     i    Defines the way the line is given in reallinedef
!                         Possible values:
!                         1: the start point of the line (xstart,ystart) is
!                            given in reallinedef(1:2)
!                            the angle of rotation with respect to the positive
!                            x-axis is given in reallinedef(3)
!     reallinedef    i    Contains real information of intersection curve
!                         depending on intlinedef
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision xstart(2), tangent(2)

!     tangent        Contains the tangential vector
!     xstart         Starting point of line
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHDEFLINE     Define line defined by intlinedef and reallinedef in terms
!                    of xstart and tangent vector
!     MSHINTCURVLINE intersect curve icurnr with a straight line
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     139   curve number does not exist
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Define line in the form x_s + alpha t
!     Get coordinates from curve
!     Compute intersection and return with coordinates and alpha
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshintcurvelin' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshintcurvelin'
         write(irefwr,*) 'icurnr, intlinedef ', icurnr, intlinedef
         call prinrl ( reallinedef, 3, 'reallinedef' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Define line in the form x_s + alpha t

      call mshdefline ( intlinedef, reallinedef, xstart, tangent )

!     --- Get coordinates from curve
!         Compute intersection and return with coordinates and alpha

      if ( icurnr<1 .or. icurnr>ncurvs ) then

!     --- Curve number out of range

         call errint ( icurnr, 1 )
         call errint ( ncurvs, 2 )
         call errsub ( 139, 2, 0, 0 )
         go to 1000

      end if  ! ( icurnr<1 .or. icurnr>ncurvs )

      call mshintcurvline ( xstart, tangent, coor, &
                            kmeshm, icurnr, coors )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshintcurvelin'

      end if
      call erclos ( 'mshintcurvelin' )

      end subroutine mshintcurvelin
