      subroutine mshmakeparmesh
! ======================================================================
!
!        programmer    Dragan Vidovic
!        version  2.0  date 30-01-2016 New parameter list
!        version  1.4  date 20-06-2013 New definition file ref numbers
!        version  1.3  date 03-08-2011 activate nmesg
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine reads the meshoutput file and splits the mesh into
!     meshoutput_par.??? files.
!     It is called by program sepmakeparmesh
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmodulemain
      use sepmodulemach
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, manag(5), ihelp(10), jstart, iref, &
              lastps, icount, iinput(3), iparallel, &
              kmeshlc(maxmesh)
      character (len=100) :: filename
      logical check

!     check          Indicates if the file exist (true) or not
!     filename       Name of file
!     i              Counting variable
!     icount         Counter
!     ihelp          Work array to store input from the input file
!     iinput         integer input array for subroutine MSHRDPARALLEL
!     iparallel      Indicates if the mesh must be subdivided for parallel
!                    processing or not. See MSHRDPARALLEL
!     iref           File reference number
!     jstart         starting position of curve i in kmeshm
!     kmeshlc       Subdivision of array KMESH
!     lastps         Last non-blank position in name10
!     manag          Manager array that sets the parameters in subroutine
!                    SEPSTR
!                    MANAG must be filled as follows:
!                    1: Parameter ISTART for subroutine START
!                    2: Parameter IROTAT for subroutine START
!                    3: Parameter IOUTP  for subroutine START
!                    4: Parameter ITIME  for subroutine START
!                    5: Parameter IRENUM for subroutine MESHRD
!                    6: Parameter ICHOIS for subroutine PROBDF
!                    7: Indication if the files sepcomp.inf and sepcomp.out
!                       must be filled (1) or not (0)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inisetnewref

!     FINISH         Stop SEPRAN
!     INIFIL         Open files
!     INISETNEWREF   Create new reference number
!     INITCB         Initializes SEPRAN machine-independent common blocks
!     INITMD         Set machine-dependent quantities
!     MSH071         Read information of the mesh from the file indicated by
!                    iref
!     MSHPARBF       Actual body of MSHPAR
!     MSHRDPARALLEL  Read input with respect to the keyword parallel
!     SEPFILLOUTERB  Fill outer boundary of mesh
!     SEPSTART       Standard SEPRAN starting subroutine
! **********************************************************************
!
!                       I/O
!
!     Reads meshoutput
!     Reads input file defined in mshrdparallel
!     Writes meshoutput_par.???
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Initialize array KMESH
!     Open the error message file
!     Start SEPRAN
!     Read meshoutput file
!     Write the mesh to the meshoutput_par.??? files
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Initialize array KMESH

      kmesh2d = 0
      kmesh => kmesh2d(1)

      manag(1) = 20000
      manag(2) = 0
      manag(3) = -1
      manag(4) = 0
      manag(5) = -1
      call initcb
      call initmd
      mainsubr = 6       ! program mshmakeparmesh

!     --- Open standard elements file and error message file

      call inifil(1)
      call inifil(4)

!     --- Standard SEPRAN start subroutine

      jstart = abs(manag(1))
      if ( mod(jstart/100,10)==0 ) jstart = jstart + 100
      if ( manag(1)<0 ) then
         manag(1) = -jstart
      else
         manag(1) = jstart
      end if
      call sepstart ( manag(1), manag(2), manag(3), manag(4) )

!     --- Open files meshoutput, sepcomp.out and read mesh

      call inifil ( -10 )
      call msh071 ( manag(5), iref10, ihelp )

!     --- Activate or create kmesh part ah (outer surfaces)

      if ( mmouter==0 ) call sepfillouterb

!     --- Remove files meshoutput_par.xxx if they exist

      iref = inisetnewref()
      lastps = index ( name10, ' ' )-1
      filename = name10
      filename(lastps+1:lastps+8) = '_par.000'
      icount = 0

!     --- Loop over all files meshoutput_par.xxx

100   inquire ( file=filename, exist=check )
      if ( check ) then

!     --- File exists, remove it

         open ( unit = iref, file = filename, status = 'unknown' )
         close ( unit = iref, status = 'delete' )

!        --- Raise icount and construct new filename

         icount = icount+1
         write(filename(lastps+1:lastps+8),110) icount
110      format('_par.',i3.3)
         go to 100

      end if

!     --- Read input

      call mshrdparallel ( iparallel )

      iinput(1) = mod(iparallel,100)   !  Type of subdivision
      iinput(2) = iparallel/100        !  Number of processors
      iinput(3) = 0                    !  Do not plot mesh

!     --- Write submeshes to the files meshoutput_par.xxx

      call mshparbf ( iinput, kmeshlc )

!     --- Close all files and stop program

      call finish(0)

      end
