      subroutine mshmarkelems ( iedges, elements, nmeshd, nelem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-10-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark all elements with the number of edges that are subdivided
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     element
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iedges(*), elements(*), nmeshd(*), nelem

!     elements       o    Work array of length nelem
!                         elements(ielem) edges of element ielem are subdivided
!     iedges         i    Work array of size nedges
!                         If iedges(i) > 0, edge i must be splitted and
!                         the node number will be iedges(i)+npoint
!     nelem          i    Number of elements
!     nmeshd         i    nmesh part d, contains edge numbers of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielem, ipstart, nintersect

!     i              Counting variable
!     ielem          Element number
!     ipstart        Starting address in array
!     nintersect     Number of intersections found
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array elements(1:nelem)
!     For all elements ielem do
!        nintersect := number of edges that are intersected
!        elements(ielem) := nintersect
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshmarkelems' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshmarkelems'

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

!     --- Clear array elements(1:nelem)

      do i = 1, nelem
         elements(i) = 0
      end do  ! i = 1, nelem

!     --- For all elements ielem do

      do ielem = 1, nelem

!     --- nintersect := number of edges that are intersected

         nintersect = 0
         ipstart = nmeshd(ielem)
         if ( debug ) then
            write(irefwr,*) 'ielem ', ielem
            call prinin ( nmeshd(ipstart+1), nmeshd(ipstart), 'edges' )
         end if  ! ( debug )
         do i = 1, nmeshd(ipstart)
            if ( iedges(abs(nmeshd(ipstart+i))).gt.0 ) &
               nintersect = nintersect+1
         end do  ! i = 1, nmeshd(ipstart)

!        --- elements(ielem) := nintersect

         elements(ielem) = nintersect

      end do  ! ielem = 1, nelem

1000  call erclos ( 'mshmarkelems' )

      if ( debug ) then

!     --- Debug information

         call prinin ( elements, nelem, 'elements' )
         write(irefwr,*) 'End mshmarkelems'

      end if

      end
