      subroutine mshobstms022 ( kmeshnew, iseqkm2, coornew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 23-04-2012 Use mshcoppart
!        version  1.4  date 11-10-2011 Extension with 1d
!        version  1.3  date 04-10-2006 Correction
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy part of kmesh to iseqkm2 in case of obstacles
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     copy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer  kmeshnew, iseqkm2
      double precision coornew(2,*)

!     coornew        o    Array containing the new coordinates of the nodes
!     iseqkm2       i/o   new array kmesh
!     kmeshnew      i/o   New array kmesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision dummy(1)
      integer :: iseqkm1

!     dummy          Dummy array
!     iseqkm1        Sequence number with respect to km of first mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     MSHCOPPART     Copy part of kmesh to kmeshnew
!     SEPINITKMESHAD Initialize kmesh ad
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstms022' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstms022'
         call ertrace
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- general copy

      iseqkm1 = iseqkm
      if ( debug ) write(irefwr,1) 'iseqkm1, iseqkm2', iseqkm1, iseqkm2
      call mshcoppart ( kmeshnew, coornew, iseqkm2 )

!     --- Set kmesh part ad equal to zero ( obstacles )

      iseqkm = iseqkm2
      call sepinitkmeshad
      km(iseqkm)%kmeshad = 0

      iseqkm = iseqkm1
      kmeshad => km(iseqkm)%kmeshad
      kelmad  => km(iseqkm)%kmesh(52)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshobstms022'

      end if
      call erclos ( 'mshobstms022' )

      end
