      subroutine mshobstms024 ( nmap, iworkpoint, &
                                ipointobstnew, npointnew, kmeshnew, &
                                ielemnew, kmeshcnew, iinput, ielemobstnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 25-10-2012 Extra debug
!        version  1.1  date 11-10-2011 Extension with 1d
!        version  1.0  date 20-05-2004
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt the arrays ipointobstnew and ielemobstnew for new mesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nmap, iworkpoint(nmap,*), ipointobstnew(*), &
              npointnew, kmeshnew, ielemnew, kmeshcnew(*), iinput(*), &
              ielemobstnew(*)

!     ielemnew       i    Sequence number of element in new mesh
!     ielemobstnew   o    New array ielemobst
!     iinput         o    Array to pass some constants to mshobstms
!                         Contents
!                         1:  length of new array kmeshc
!                         2:  length of new array ipointobst
!                         3:  length of new array ielemobst
!     ipointobstnew  o    New array ipointobst
!     iworkpoint     i    Work array of length npointnew*2
!                         This array is filled as follows:
!                         iworkpoint(i,1) = 0 if node i outside obstacle
!                         iworkpoint(i,1) = 1 if node i inside obstacle
!                         iworkpoint(i,1) = 2 if node i inside obstacle (near
!                                             boundary)
!                         iworkpoint(i,1) = 3 if node i on obstacle
!                         iworkpoint(i,2) contains the corresponding obstacle
!                         sequence number iobst
!     kmeshcnew      i    New kmesh part c
!     kmeshnew       i    New array kmesh
!     nmap           i    length of first parameter of imap and rmap
!     npointnew      i    Sequence number of last generated node
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iobst, numpntobst, numbndobst, istart, istartobstnew, &
              lenielemobst, i, npointsav, iseqkm1, iseqkm2

!     i              Counting variable
!     iobst          Obstacle sequence number
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     istart         First position used at a part
!     istartobstnew  Starting address of obstacle in array ipointobstnew
!     lenielemobst   Computed length of array ielemobst
!     npointsav      Help parameter to save the old value of npoint
!     numbndobst     Number of nodal points on the boundary of the obstacle
!     numpntobst     Number of nodal points inside the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOBSTMARKELN Mark all elements in the mesh with information if they are
!                    within an obstacle or outside an obstacle
!     PRININ2        Print an integer two-dimensional array
!     SEPPRINIPOBST  Print the contents of array ipointobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstms024' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstms024'
         write(irefwr,1) 'numobst, npointnew', numobst, npointnew
         call prinin2 ( iworkpoint, npointnew, 2, 'iworkpoint' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- adapt ipointobstnew

      iseqkm1 = iseqkm
      istartobstnew = numobst+1    ! starting address of first obstacle

!     --- Loop over obstacles

      do iobst = 1, numobst

         numpntobst = 0  ! number of points in obstacle iobst
         numbndobst = 0  ! number of points on boundary of obstacle iobst
         istart = istartobstnew+3  ! start address of nodes in ipointobstnew
         if ( debug ) &
            write(irefwr,1) 'iobst, istart', iobst, istart

!        --- For all nodes do

         do i = 1, npointnew

            if ( (iworkpoint(i,1)==1 .or. iworkpoint(i,1)==2) .and. &
                  iworkpoint(i,2)==iobst ) then

!           --- Node belongs to obstacle and is inside the obstacle
!               Store and raise counter

               numpntobst = numpntobst+1
               ipointobstnew(istart+numpntobst) = i

            end if  ! ( (iworkpoint(i,1)==1 .or. iworkpoint(i,1)==2) ...

         end do  ! i = 1, npointnew
         istart = istart+numpntobst
         if ( debug ) write(irefwr,1) 'numpntobst', numpntobst

!        --- For all nodes do

         do i = 1, npointnew

            if ( iworkpoint(i,1)==3 .and. iworkpoint(i,2)==iobst ) then

!           --- Node on boundary of obstacle
!               Store and raise counter

               numbndobst = numbndobst+1
               ipointobstnew(istart+numbndobst) = i

            end if  ! ( iworkpoint(i,1)==3 .and. iworkpoint(i,2)==iobst )

         end do  ! i = 1, npointnew
         if ( debug ) write(irefwr,1) 'numbndobst', numbndobst

         ipointobstnew(istartobstnew)   = numpntobst  ! points in the obstacle
         ipointobstnew(istartobstnew+1) = 0           ! there are no points near
                                                      ! the boundary anymore
         ipointobstnew(istartobstnew+2) = numbndobst  ! points on the boundary
                                                      ! of the obstacle
         ipointobstnew(istartobstnew+3) = 0           ! will be filled later on
         ipointobstnew(iobst) = istartobstnew
         istartobstnew = istart+numbndobst+1

      end do  ! iobst = 1, numobst

!     --- adapt ielemobstnew

      iseqkm2 = kmeshnew
      km(iseqkm2)%kmesh(8) = npointnew
      km(iseqkm2)%kmesh(9) = ielemnew
      npointsav = npoint
      npoint = npointnew

      iseqkm = iseqkm2
      call mshobstmarkeln ( ipointobstnew, ielemobstnew, &
                            kmeshcnew, lenielemobst )
      iseqkm = iseqkm1
      npoint = npointsav

      iinput(2) = istartobstnew-1
      iinput(3) = lenielemobst

1000  if ( debug ) then

!     --- Debug information

         iseqkm = iseqkm2
         call sepprinipobst ( ipointobstnew )
         iseqkm = iseqkm1
         write(irefwr,*) 'End mshobstms024'

      end if
      call erclos ( 'mshobstms024' )

      end
