      subroutine mshobstms05 ( kmeshnnew, lenkmsnnew, iedges, linear, &
                               ielmfaces, npointnew, kmeshnew, &
                               idimedges, kmeshmnew, lenkmsmnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 22-11-2011 New parameter list
!        version  3.0  date 20-09-2006 Extra parameters
!        version  2.1  date 15-09-2006 Correction for empty surfaces
!
!   copyright (c) 2004-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Update kmeshm and kmeshn if necessary
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     refine
!     curve
!     surface
!     mapping_array
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshnnew(*), lenkmsnnew, &
              iedges(*), linear, ielmfaces(*), npointnew, kmeshnew, &
              idimedges, lenkmsmnew, kmeshmnew(*)

!     idimedges      i    Length of first dimension of array iedges
!                         (linear:1, quadratic: 3)
!     iedges        i/o   Work array of size nedges
!     ielmfaces     i/o   Contains faces of elements
!     kmeshmnew     i/o   New array kmeshm
!     kmeshnew      i/o   New array kmesh
!     kmeshnnew     i/o   New array kmeshn
!     lenkmsmnew     i    Length of array kmshmnew
!     lenkmsnnew     i    Length of array kmshnnew
!     linear         i    Indicates whether the element is a linear simplex
!                         (true) or not (false)
!     npointnew      i    Sequence number of last generated node
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseqkm2

!     iseqkm2        Sequence number with respect to km of second mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICOPINT2D    Copy two-dimensional integer array
!     INICOPREAL2D   Copy two-dimensional real array
!     MSHOBSTCURV    Create new array kmeshm in the case that extra points at
!                    the surface are created when a new mesh is created due to
!                    intersection with an obstacle (2D only)
!     MSHOBSTSURFS   Create new array kmeshn in the case that extra points at
!                    the surface are created when a new mesh is created due to
!                    intersection with an obstacle (3D only)
!     PR0008         Print information of curves
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshobstms05' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshobstms05'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkm2 = kmeshnew

      if ( ndim>=2 ) then

!     --- Update kmeshm if necessary

         call mshobstcurv ( kmeshmnew, &
                            lenkmsmnew, iedges, linear, idimedges )

      end if  ! ( ndim>=2 )

      if ( ndim==3 ) then

!     --- Update kmeshn if necessary

         call mshobstsurfs ( kmeshnnew, lenkmsnnew, iedges, &
                             ielmfaces, idimedges )

      end if  ! ( ndim==3 )

!     --- Copy contents of imap and rmap to final positions

      npointnew = km(iseqkm2)%kmesh(8)

1000  if ( debug ) then

!     --- Debug information

         call pr0008 ( kmeshnnew, nsurfs )
         write(irefwr,*) 'End mshobstms05'

      end if
      call erclos ( 'mshobstms05' )

      end
