      subroutine mshobstms2d04q ( trianglenode, trianglenodenw, &
                                  edgenode, ifacenode, coornew )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.0  date 22-03-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide triangle into two triangles
!     Quadratic 7 point elements
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     quadratic
!     triangle
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer trianglenode(7), trianglenodenw(7,*), edgenode(5), &
              ifacenode(2)
      double precision coornew(2,*)

!     coornew        i    Array containing the new coordinates of the nodes
!     edgenode       i   Array with nodes on the crossed edge
!                        edgenode(1) first node on crossed edge
!                        edgenode(2) last node on crossed edge
!                        edgenode(3) new node on crossed edge
!     ifacenode      i    ?
!     trianglenode   i   Array of nodal point numbers of the corners of
!                        the original triangle
!     trianglenodenw o   Array of nodal point numbers of the corners of
!                        the new triangles.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, iseq, triangtemp(7), idx(3), &
              edgenodet(2), itran(7,3)

!     edgenodet      Array of global nodal point numbers of X-point on edge i
!                    after transformation
!     i              Counting variable
!     idx            Array containing the indices of the non-vertices
!     iseq           Sequence number
!     itran          Array used for transformation
!     j              Counting variable
!     triangtemp     Temporary array to store nodal numbers
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     inode := third node in triangle
!     triangle1 := ( inode,newpoint,inode1)
!     triangle2 := ( inode,newpoint,inode2)
!     Change direction if necessary to get counterclockwise ordering
!
!     We have the following situations
!
!   edgenode(2)                       edgenode(1)
!     |\                                |\
!     | \                               | \
!     |  \                              |  \
!     |   \                             |   \
!     |    \                            |    \
!     |     \ edgenode(3)               |     \ edgenode(3)
!     |    / \                          |    / \
!     |  /    \                         |  /    \
!     |/       \                        |/       \
!     ---------- ---------- &
!   inode       edgenode(1)           inode     edgenode(2)
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data idx / 1,3,5 /
      data itran /1, 2, 3, 4, 5, 6, 7, &
                  3, 4, 5, 6, 1, 2, 7, &
                  5, 6, 1, 2, 3, 4, 7/
! ======================================================================
!
!     --- inode := third node in triangle

      do i = 1, 3
         if ( trianglenode(idx(i)).ne.edgenode(1) .and. &
              trianglenode(idx(i)).ne.edgenode(2) ) then

!        --- Third node found

            iseq = i
            go to 100
         end if
      end do
100   continue

!     --- rotate triangle

      do i=1,7
         triangtemp(i)=trianglenode(itran(i,iseq))
      end do

      if ( triangtemp(3).eq.edgenode(1) ) then

!     --- inode to edgenode(1) is counterclockwise

         edgenodet(1)=edgenode(3)
         edgenodet(2)=edgenode(5)

      else

!     --- inode to edgenode(1) is clockwise

         edgenodet(1)=edgenode(5)
         edgenodet(2)=edgenode(3)

      end if

!     --- rotate triangle

      do i=1,7
         triangtemp(i)=trianglenode(itran(i,iseq))
      end do

!     --- element 1

      trianglenodenw(1,1) = triangtemp(1)
      trianglenodenw(2,1) = triangtemp(2)
      trianglenodenw(3,1) = triangtemp(3)
      trianglenodenw(4,1) = edgenodet(1)
      trianglenodenw(5,1) = triangtemp(4)
      trianglenodenw(6,1) = triangtemp(7)
      trianglenodenw(7,1) = ifacenode(1)

!     --- element 1

      trianglenodenw(1,2) = triangtemp(1)
      trianglenodenw(2,2) = triangtemp(7)
      trianglenodenw(3,2) = triangtemp(4)
      trianglenodenw(4,2) = edgenodet(2)
      trianglenodenw(5,2) = triangtemp(5)
      trianglenodenw(6,2) = triangtemp(6)
      trianglenodenw(7,2) = ifacenode(2)

!     --- update coordinates

      do i=1,2
         do j=1,2
            coornew(j,trianglenodenw(2,i)) = &
               (coornew(j,trianglenodenw(1,i))+ &
                coornew(j,trianglenodenw(3,i)))*0.5d0
            coornew(j,trianglenodenw(4,i)) = &
               (coornew(j,trianglenodenw(3,i))+ &
                coornew(j,trianglenodenw(5,i)))*0.5d0
            coornew(j,trianglenodenw(6,i)) = &
               (coornew(j,trianglenodenw(5,i))+ &
                coornew(j,trianglenodenw(1,i)))*0.5d0
            coornew(j,trianglenodenw(7,i)) = &
               (coornew(j,trianglenodenw(1,i))+ &
                coornew(j,trianglenodenw(3,i))+ &
                coornew(j,trianglenodenw(5,i)) )/3d0
         end do
      end do

      end
