      subroutine mshobstms3dq5a ( tetnode, tetnodenew, edgenode, &
                                  isort, ledgenode, nelemnew, centroid, &
                                  midfacenodes, facenode, fluidnd )
! ======================================================================
!
!        programmer    Raoul van Loon
!        version  1.0  date 06-04-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide tetrahedron, with only two crossed edges which share a node, &
!     into three tetrahedrons. Quadratic case
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     quadratic
!     tetrahedron
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer tetnode(15), tetnodenew(15,3), edgenode(5,3), isort(4), &
              ledgenode(2,3), nelemnew, centroid(6), midfacenodes(4), &
              facenode(4,4), fluidnd(*)

!     centroid       i    ?
!     edgenode       i    Array with all nodes on the crossed edge
!                         edgenode(1,1) first node on crossed edge
!                         edgenode(2,1) last node on crossed edge
!                         edgenode(3,1) new node on crossed edge
!     facenode       i    ?
!     fluidnd        i    Array containing the fluidnodes that lie on the
!                         obstacle
!     isort          i    if isort(i)>isort(j) then tetnode(i) is at least on
!                         the same number of crossed edges as tetnode(j)
!     ledgenode      i    for i=1,2 and 1<j< number of crossed edges
!                         ledgenode(i,j) is the local tetnode of edgenode(i,j), &
!                         in other words:
!                         edgnode(i,j) = tetnode(ledgenode(i,j))
!     midfacenodes   i    ?
!     nelemnew       i    Number of elements in new mesh
!     tetnode        i    Array of nodal point numbers of the corners of
!                         the original tetrahedron.
!     tetnodenew     o    Array of nodal point numbers of the corners of
!                         the new tetrahedrons.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i,j, itran, transtot(15,12), &
              trans(4), transi(4), transtemp, transposj, facenodet(4,4), &
              tetnodet(15), edgenodet(2,6), facetrans(4), idx(2), &
              idx1(6)
      integer pedgenodet(4,4)
      logical oddpairintchan, flip(6)

!     edgenodet      Array of global nodal point numbers of X-point on edge i
!                    after transformation
!     facenodet      ?
!     facetrans      ?
!     flip           ?
!     i              Counting variable
!     idx            Array containing the indices of the non-vertices
!     idx1           Array for renumbering of the linear basis functions
!     itran          ?
!     j              Counting variable
!     oddpairintchan if true the transformation is an odd number of
!                    pair interchanges
!     pedgenodet     the local edge number = local edgenode number
!                    for the edge [local tetnode i, local tetnode j]
!     tetnodet       Array of global nodal point numbers of the corners of
!                    the transformed original tetrahedron
!     trans          Transformation array. trans(i) is the transformed
!                    local position of the i-th corner of the original
!                    tetrahedron.
!     transi         Inverse of trans
!     transposj      The transformed local edge node number of edge node j
!     transtemp      temporary storage
!     transtot       ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!                   the new tetrahedrons
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Define transformation(= rotation) to move node isort(4) to node 1
!                                           and node isort(1) to node 4.
!
!     Transform tetrahedron tetnode to tetnodet and the position of the
!     intersection points edgenode(3,.) to edgenodet(.)
!
!     Split tetrahedron into subtetrahedrons.
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data pedgenodet /0,1,3,4, &
                       1,0,2,5, &
                       3,2,0,6, &
                       4,5,6,0/
      data transtot / 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, &
                      3, 4, 5, 6, 1, 2, 8, 9, 7, 10, 11, 13, 14, 12, 15, &
                      5, 6, 1, 2, 3, 4, 9, 7, 8, 10, 11, 14, 12, 13, 15, &
                      1, 7, 10, 8, 3, 2, 6, 9, 4, 5, 12, 14, 13, 11, 15, &
                      3, 2, 1, 7, 10, 8, 4, 6, 9, 5, 12, 11, 14, 13, 15, &
                      10, 8, 3, 2, 1, 7, 9, 4, 6, 5, 12, 13, 11, 14, 15, &
                      1, 6, 5, 9, 10, 7, 2, 4, 8, 3, 14, 11, 13, 12, 15, &
                      5, 9, 10, 7, 1, 6, 4, 8, 2, 3, 14, 13, 12, 11, 15, &
                      10, 7, 1, 6, 5, 9, 8, 2, 4, 3, 14, 12, 11, 13, 15, &
                      3, 8, 10, 9, 5, 4, 2, 7, 6, 1, 13, 12, 14, 11, 15, &
                      5, 4, 3, 8, 10, 9, 6, 2, 7, 1, 13, 11, 12, 14, 15, &
                      10, 9, 5, 4, 3, 8, 7, 6, 2, 1, 13, 14, 11, 12, 15/
      data idx /2, 1/
      data idx1 /1,3,1,1,3,10/
! ======================================================================
!
      call eropen ( 'mshobstms3dq5a' )

!     --- Define transformation to move node isort(4) to node 1
!                                and node isort(1) to node 4
!         Initialize

      oddpairintchan = .false.
      do i = 1,4
         trans(i) = i
      end do

!     --- move node isort(4) to 1

      if( isort(4).ne.trans(1) ) then
         transtemp = trans(1)
         trans(1) = isort(4)
         trans(isort(4)) = transtemp
         oddpairintchan = .not.oddpairintchan
      end if

!     --- move node isort(1) to 2

      if (isort(1).ne.trans(4) ) then
         transtemp = trans(4)
         trans(4) = isort(1)
         trans(trans(isort(1))) = transtemp
         oddpairintchan = .not.oddpairintchan
      end if

!     --- make pairinterchanges even

      if( oddpairintchan ) then
         transtemp = trans(2)
         trans(2) = trans(3)
         trans(3) = transtemp
      end if

!     --- transi := inverse of trans

      do i = 1,4
         transi(trans(i)) = i
      end do

!     --- Transform tetrahedron tetnode to tetnodet

      if (trans(4).eq.4) then
         if (trans(1).eq.1) itran=1
         if (trans(1).eq.2) itran=2
         if (trans(1).eq.3) itran=3
      else if (trans(4).eq.3) then
         if (trans(1).eq.1) itran=4
         if (trans(1).eq.2) itran=5
         if (trans(1).eq.4) itran=6
      else if (trans(4).eq.2) then
         if (trans(1).eq.1) itran=7
         if (trans(1).eq.3) itran=8
         if (trans(1).eq.4) itran=9
      else if (trans(4).eq.1) then
         if (trans(1).eq.2) itran=10
         if (trans(1).eq.3) itran=11
         if (trans(1).eq.4) itran=12
      end if

      do i = 1,15
         tetnodet(i) = tetnode(transtot(i,itran))
      end do

!     --- Transform the intersection points to their transformed positions

      do j = 1, 4
         flip(j)=.false.
      end do
      do j = 1,2
         transposj = pedgenodet( &
                       transi(ledgenode(1,j)),transi(ledgenode(2,j)))
         if (edgenode(1,j).eq.tetnodet(idx1(transposj))) then
            edgenodet(1,transposj) = edgenode(3,j)
            edgenodet(2,transposj) = edgenode(5,j)
         else
            edgenodet(1,transposj) = edgenode(5,j)
            edgenodet(2,transposj) = edgenode(3,j)
            if (transposj.eq.1) flip(2)=.true.
            if (transposj.eq.3) flip(4)=.true.
         end if
      end do

!     --- face1 = bottom face
!         face2 = front face
!         face3 = back face

      do j=1,4
         do i=1,4
            if (midfacenodes(i).eq.tetnodet(10+j)) facetrans(j)=i
         end do
      end do

!     --- Transform faces

      do j=1,4
         do i=1,2
            if (.not.flip(j)) facenodet(i,j)=facenode(i,facetrans(j))
            if (flip(j)) facenodet(i,j)=facenode(idx(i),facetrans(j))
         end do
         if (j.eq.1) facenodet(3,j)=facenode(3,facetrans(j))
         if (j.eq.1) facenodet(4,j)=facenode(4,facetrans(j))
      end do

      tetnodenew(1,1) = tetnodet(1)
      tetnodenew(2,1) = edgenodet(1,1)
      tetnodenew(3,1) = tetnodet(2)
      tetnodenew(4,1) = tetnodet(11)
      tetnodenew(5,1) = tetnodet(6)
      tetnodenew(6,1) = edgenodet(1,3)
      tetnodenew(7,1) = tetnodet(7)
      tetnodenew(8,1) = tetnodet(12)
      tetnodenew(9,1) = tetnodet(14)
      tetnodenew(10,1) = tetnodet(10)
      tetnodenew(11,1) = facenodet(1,1)
      tetnodenew(12,1) = facenodet(1,2)
      tetnodenew(13,1) = tetnodet(15)
      tetnodenew(14,1) = facenodet(1,4)
      tetnodenew(15,1) = centroid(1)

      if ( tetnodet(2).lt.tetnodet(6) ) then

         tetnodenew(1,2) = tetnodet(2)
         tetnodenew(2,2) = edgenodet(2,1)
         tetnodenew(3,2) = tetnodet(3)
         tetnodenew(4,2) = tetnodet(4)
         tetnodenew(5,2) = tetnodet(5)
         tetnodenew(6,2) = facenodet(4,1)
         tetnodenew(7,2) = tetnodet(12)
         tetnodenew(8,2) = tetnodet(8)
         tetnodenew(9,2) = tetnodet(9)
         tetnodenew(10,2) = tetnodet(10)
         tetnodenew(11,2) = facenodet(3,1)
         tetnodenew(12,2) = facenodet(2,2)
         tetnodenew(13,2) = tetnodet(13)
         tetnodenew(14,2) = centroid(4)
         tetnodenew(15,2) = centroid(3)

         tetnodenew(1,3) = tetnodet(2)
         tetnodenew(2,3) = facenodet(4,1)
         tetnodenew(3,3) = tetnodet(5)
         tetnodenew(4,3) = edgenodet(2,3)
         tetnodenew(5,3) = tetnodet(6)
         tetnodenew(6,3) = tetnodet(11)
         tetnodenew(7,3) = tetnodet(12)
         tetnodenew(8,3) = tetnodet(9)
         tetnodenew(9,3) = tetnodet(14)
         tetnodenew(10,3) = tetnodet(10)
         tetnodenew(11,3) = facenodet(2,1)
         tetnodenew(12,3) = centroid(4)
         tetnodenew(13,3) = facenodet(2,4)
         tetnodenew(14,3) = tetnodet(15)
         tetnodenew(15,3) = centroid(2)

      else

         tetnodenew(1,2) = tetnodet(2)
         tetnodenew(2,2) = edgenodet(2,1)
         tetnodenew(3,2) = tetnodet(3)
         tetnodenew(4,2) = facenodet(4,1)
         tetnodenew(5,2) = tetnodet(6)
         tetnodenew(6,2) = tetnodet(11)
         tetnodenew(7,2) = tetnodet(12)
         tetnodenew(8,2) = tetnodet(8)
         tetnodenew(9,2) = tetnodet(14)
         tetnodenew(10,2) = tetnodet(10)
         tetnodenew(11,2) = facenodet(2,1)
         tetnodenew(12,2) = facenodet(2,2)
         tetnodenew(13,2) = centroid(4)
         tetnodenew(14,2) = tetnodet(15)
         tetnodenew(15,2) = centroid(2)

         tetnodenew(1,3)  = tetnodet(3)
         tetnodenew(2,3)  = tetnodet(4)
         tetnodenew(3,3)  = tetnodet(5)
         tetnodenew(4,3)  = edgenodet(2,3)
         tetnodenew(5,3)  = tetnodet(6)
         tetnodenew(6,3)  = facenodet(4,1)
         tetnodenew(7,3)  = tetnodet(8)
         tetnodenew(8,3)  = tetnodet(9)
         tetnodenew(9,3)  = tetnodet(14)
         tetnodenew(10,3)  = tetnodet(10)
         tetnodenew(11,3)  = facenodet(3,1)
         tetnodenew(12,3)  = tetnodet(13)
         tetnodenew(13,3)  = facenodet(2,4)
         tetnodenew(14,3)  = centroid(4)
         tetnodenew(15,3)  = centroid(3)

      end if

      nelemnew = 3

      fluidnd(1)=tetnodet(2)
      fluidnd(2)=tetnodet(11)
      fluidnd(3)=tetnodet(6)
      fluidnd(4)=tetnodet(14)
      fluidnd(5)=tetnodet(10)
      fluidnd(6)=tetnodet(12)
      fluidnd(7)=tetnodet(15)

      call erclos ( 'mshobstms3dq5a' )

      end
