      subroutine mshori3d12 ( kmeshc, inpelm, inelem, coor, ichnge, &
                              jac )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-05-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make the orientation of the elements such that the "volume" is always
!     positive
!     Quadratic tetrahedrons
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     mesh
!     orientation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, inelem, kmeshc(inpelm,inelem), ichnge
      double precision coor(3,*), jac

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     ichnge     i    indicator how many elements were changed
!     inelem     i    Number of elements in a specific element group
!     inpelm     i    number of points per element
!     jac        o    jacobian of element
!     kmeshc    i/o   kmesh part c, contains the nodal points corresponding
!                     to the elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielem, ihelp(5), ind(4), k, ind1(4)
      double precision e(3), delta(3,3)

!     delta          Work array to compute the Jacobian
!     e              Work array to compute the Jacobian
!     i              Counting variable
!     ielem          Element number
!     ihelp          integer work array to store the node numbers of one element
!     ind            Defines the cyclic permutation of modulo 4
!     ind1           Work array to define the 4 nodes that must be used to
!                    compute the sign of the jacobian
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ind / 2,3,1,2 /
! ======================================================================
!
      call eropen ( 'mshori3d12' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshori3d12'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ind1(1) = 1
      ind1(2) = 3
      ind1(3) = 5
      ind1(4) =10
      do ielem = 1, inelem

         do k = 1, 3
            do i = 1, 3
               delta(i,k) = coor(k,kmeshc(ind1(i+1),ielem)) - &
                            coor(k,kmeshc(ind1(i),ielem))
            end do
         end do
         do k = 1, 3
            e(k) = - ( delta(2,ind(k)) * delta(3,ind(k+1)) - &
                       delta(2,ind(k+1)) * delta(3,ind(k)) )
         end do

         jac = - delta(1,1)*e(1) - delta(1,2)*e(2) - delta(1,3)*e(3)
         if ( jac<0d0 ) then

!        --- Jacobian has negative direction, change the node numbers

            ichnge = ichnge + 1

            do i = 2, 6
               ihelp(i-1) = kmeshc(i,ielem)
            end do
            do i = 2, 6
               kmeshc(i,ielem) = ihelp(7-i)
            end do
            do i = 8, 9
               ihelp(i-7) = kmeshc(i,ielem)
            end do
            do i = 8, 9
               kmeshc(i,ielem) = ihelp(10-i)
            end do

         end if

      end do

1000  call erclos ( 'mshori3d12' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshori3d12'

      end if  ! ( debug )

      end
