      subroutine mshpavbnquad4 ( inode1, inode, nmeshl, iwork, iedge5, &
                                 itrian1, itrian2, iedge1, iedge3, &
                                 nmeshh, nmeshd, markedges, iworkedge, &
                                 numnodes1, nodeopposite1, nmeshc, &
                                 coor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 24-03-2009 Extra debug
!        version  1.0  date 07-07-2004
!
!   copyright (c) 2004-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Given the nodes inode1 and inode find edge iedge5 connecting both
!     and the 2 triangles itrian1 and itrian2 connected to edge5
!     Find edge3 on the oppsoite boundary and check if it is connected to node 1
!     If not return with iedge3 = 0
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     edge
!     triangle
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inode1, inode, nmeshl(*), iwork(*), iedge5, &
              itrian1, itrian2, iedge1, iedge3, &
              nmeshh(*), nmeshd(*), markedges(*), iworkedge(*), &
              numnodes1, nodeopposite1(*), nmeshc(*)
      double precision coor(2,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     iedge1         i    edge sequence number of boundary edge
!     iedge3         o    edge sequence number of opposite boundary edge
!                         if not found or this edge is not connected to node1
!                         by another edge, iedge3 is set to 0
!     iedge5         o    edge sequence number of "diagonal" edge
!     inode          i    Node sequence number of second node on iedge5
!     inode1         i    First node in edge1
!     itrian1        o    Sequence number of first triangle
!     itrian2        o    Sequence number of second triangle
!     iwork         i/o   integer work array of length nedges
!                         At start iwork must be set to zero
!                         After each call iwork is reset to 0
!     iworkedge      i    work array of length nedges to be used to mark
!                         edges
!                          0:  not marked yet
!                         -1:  boundary edge that has been used already
!                         -2:  diagonal of quadrilateral that may be removed
!     markedges      i    array of length nedges indicating whether a
!                         edge is internal or on the boundary
!                         -2:  internal edge between 2 boundary nodes
!                         -1:  boundary edge
!                          0:  standard internal edge
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
!     nmeshd         i    nmesh part d, contains edge numbers of elements
!     nmeshh         i    nmesh part h, contains element numbers corresponding
!                         to edges
!     nmeshl         i    nmesh part l, contains edge numbers corresponding
!                         to nodes
!     nodeopposite1  i    Number of nodes on opposite boundary connected
!                         to node1
!     numnodes1      i    Number of nodes in nodeopposite1
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inode4, i, inode2, ipstart
      logical found, acceptquad

!     acceptquad     Check if quadrilateral is acceptable (true) or not
!                    (false)
!     found          If true node4 and node1 are connected
!     i              Counting variable
!     inode2         First node in second edge iedge2
!     inode4         Second node in second boundary edge iedge3
!     ipstart        Starting address in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer iniedge

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIEDGE        Returns with edge number corresponding to 2 nodes
!     ININODONEDGE   Find other end node on edge if one of the end nodes and
!                    the ege number are given
!     MSHPAVBNQUAD3  Find triangle corresponding to two edges as well as
!                    opposite triangle corresponding to "diagonal" edge
!     MSHPAVCHKQUAD  Check if quadrilateral to be subdivided is acceptable
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     iedge5 is edge from inode to inode1
!     itrian1 := triangle spanned by e1 and e5
!     itrian2 := other triangle connected to edge 5
!     e3 := boundary edge in itrian2
!     if ( iedge3 is not a new edge ) then
!        iedge3 := 0
!     if ( iedge3 # 0 ) then
!        inode4 := other end point of iedge3
!        if ( inode4 not coupled to inode1 ) then
!           iedge3 := 0
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavbnquad4' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavbnquad4'
         write(irefwr,1) 'iedge1, inode, inode1', iedge1, inode, inode1
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- iedge5 is edge from inode to inode1

      iedge5 = iniedge ( inode1, inode, nmeshl, iwork )
      if ( ierror/=0 ) go to 1000

!     --- itrian1 := triangle spanned by e1 and e5
!         itrian2 := other triangle connected to edge 5
!         e3 := boundary edge in itrian2

      call mshpavbnquad3 ( itrian1, itrian2, iedge1, iedge5, &
                           iedge3, nmeshh, nmeshd, markedges )
      if ( ierror/=0 ) go to 1000
      if ( debug ) write(irefwr,*) 'iedge5, iedge3, itrian1, itrian2 ', &
          iedge5, iedge3, itrian1, itrian2

      if ( iedge3/=0 ) then

!     --- iedge3 # 0, so possibly a boundary edge has been found
!         check if it is a new edge

         if ( iworkedge(iedge3)==-1 ) iedge3 = 0

      end if  ! ( iedge3/=0 )

      if ( iedge3/=0 ) then

!     --- iedge3 # 0, so a boundary edge has been found
!         check if this edge is coupled to 1-2
!         Node inode is on this edge, hence the other node must be inode4
!         Check if inode4 is connected to node 1

         call ininodonedge ( iedge3, inode, inode4, nmeshc )
         if ( debug ) write(irefwr,*) 'iedge3, inode, inode4 ', &
            iedge3, inode, inode4
         if ( ierror/=0 ) go to 1000
         found = .false.
         do i = 1, numnodes1
            if ( inode4==nodeopposite1(i) ) found = .true.
         end do  ! i = 1, numnodes1
         if ( .not. found ) iedge3 = 0

      end if  ! ( iedge3/=0 )

      if ( iedge3/=0 ) then

!     --- iedge3 # 0, so a boundary edge has been found
!         Check if quadrilateral found is acceptable

         ipstart = nmeshc(iedge1)
         inode2 = nmeshc(ipstart+1)
         if ( inode2==inode1 ) inode2 = nmeshc(ipstart+2)
         call mshpavchkquad ( inode1, inode2, inode, inode4, &
                              acceptquad, coor )
         if ( .not. acceptquad ) iedge3 = 0

      end if  ! ( iedge3/=0 )

1000  call erclos ( 'mshpavbnquad4' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavbnquad4'

      end if

      end
