      subroutine mshpavbntrian ( iedge1, iedge2, iedge4, inode1, inode2, &
                                 nmeshc, nmeshh, iworkelem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 18-02-2009 Correction zero triangle
!        version  1.0  date 07-07-2004
!
!   copyright (c) 2004-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark triangle constructed by nodes of 3 edges as a boundary triangle
!
! **********************************************************************
!
!                       KEYWORDS
!
!     edge
!     node
!     boundary
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iedge1, iedge2, iedge4, inode1, inode2, nmeshc(*), &
              nmeshh(*), iworkelem(*)

!     iedge1         i    edge sequence number
!     iedge2         i    edge sequence number
!     iedge4         i    edge sequence number
!     inode1         i    First node in edge iedge1
!     inode2         i    Second node in edge iedge1
!     iworkelem     i/o   Work array of length nelem to be used to mark
!                         elements
!                         0:  element is not special
!                         1:  element is triangle with all nodes on the boundary
!                         2:  elements is triangle, that is part of a
!                             quadrilateral with all 4 points on the boundary
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
!     nmeshh         i    nmesh part h, contains element numbers corresponding
!                         to edges
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inode3, inode4, itriangle

!     inode3         Second node in iedge2
!     inode4         Second node in iedge4
!     itriangle      Triangle sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer initrianedge

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ININODONEDGE   Find other end node on edge if one of the end nodes and
!                    the ege number are given
!     INITRIANEDGE   Find triangle corresponding to 2 given edges
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Given iedge1, iedge2 and iedge4, as well as nodes inode1 and inode2
!     on iedge1
!     Check if iedge2 and iedge4 have the same end point (otherwise error)
!     Find triangle corresponding to these nodes
!     Mark triangle as boundary triangle
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavbntrian' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavbntrian'
         write(irefwr,1) 'iedge1, iedge2, iedge4, inode1, inode2', &
                          iedge1, iedge2, iedge4, inode1, inode2
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Find second node on iedge2 and on iedge4

      call ininodonedge ( iedge2, inode2, inode3, nmeshc )
      call ininodonedge ( iedge4, inode1, inode4, nmeshc )
      if ( debug ) write(irefwr,1) 'inode3, inode4', inode3, inode4
      if ( ierror/=0 ) go to 1000
      if ( inode3/=inode4 ) then

!     --- Error nodes 3 and 4 are not equal, so not a triangle found: jump

         go to 1000

      end if  ! ( inode3/=inode4 )

!     --- Find triangle corresponding to three edges

      itriangle = initrianedge ( iedge1, iedge2, nmeshh )
      if ( ierror/=0 ) go to 1000

!     --- Mark element as boundary triangle

      if ( itriangle>0 ) iworkelem(itriangle) = 1

1000  call erclos ( 'mshpavbntrian' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'itriangle ', itriangle
         write(irefwr,*) 'End mshpavbntrian'

      end if

      end
