      subroutine mshpavedges ( kmeshcloc, npointloc, nelemloc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 12-06-2015 Use markelem
!        version  1.2  date 12-07-2012 New call to toplm2
!        version  1.1  date 18-02-2009 Extra debug
!
!   copyright (c) 2004-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill edges of the mesh and compute edges corresponding to elements
!     and nodes
! **********************************************************************
!
!                       KEYWORDS
!
!     edge
!     element
!     node
!     mesh
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: npointloc, nelemloc
      integer, target :: kmeshcloc(*)

!     kmeshcloc      i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nelemloc       i    Number of elements
!     npointloc      i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lengd2, ishift, &
              lenkmesh, melped, medpel, melpnd, i, ncone, mndped
      integer, allocatable :: markelem(:)
      double precision dum(1)

!     dum            General double precision dummy array
!     i              Counting variable
!     ishift         Shift in array iwork
!     lengd2         length of second part of array KMESH part d
!     lenkmesh       Size of array kmesh2d
!     markelem       Work array to mark elements that must be used
!                    0: element is skipped
!                    1: element is used
!     medpel         maximum number of edges in one element
!     melped         maximum number of elements connected to one edges
!     melpnd         maximum number of elements connected to a node
!     mndped         maximum number of nodes in one edge
!     ncone          number of connecting edges
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR0012         Print accumulated array
!     SEPFILLNMESHH  Fill array nmesh part h
!     SEPFILLNMESHL  Fill array nmesh part l
!     SEPPRINNMESHC  print nmesh part c
!     SEPPRINNMESHD  print nmesh part d
!     SEPRELKMESH1   Make the relation between kmref and the pointers
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill array kmesh
!     Fill array kmeshd part 1
!     Fill array kmeshd part 2
!     Fill first part of kmeshaa
!     Fill first part of array nmeshd with pointers and compute length
!     Fill rest of array nmeshd as well as array nmeshc
!     Fill array nmeshh with elements connected to edges
!     Fill array nmeshl with nodes connected to edges
!     Remove kmeshd from iwork and compute length used by the edge arrays
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavedges' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavedges'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Fill array kmesh

      km(iseqkm)%kmesh(4) = 3  ! npelm
      km(iseqkm)%kmesh(5) = 1  ! nelgrp
      km(iseqkm)%kmesh(8) = npointloc
      km(iseqkm)%kmesh(9) = nelemloc
      allocate ( km(iseqkm)%kmeshf(1) )
      km(iseqkm)%kmeshf(1) = 3   ! ishape
      km(iseqkm)%kmesh(20) = 1
      if ( .not. associated(pointkmsc) ) allocate(pointkmsc(2))
      km(iseqkm)%kmeshc => kmeshcloc(1:3*nelemloc)
      km(iseqkm)%kmesh(17) = -nelemloc*3

      call seprelkmesh1

      if ( debug ) then

         write ( irefwr, '(/a/)') &
               ' elements corresponding to nodal points'
         call pr0012 ( npointloc, kmeshd, kmshd2 )

      end if  ! ( debug )

!     --- Fill array nmeshh with elements connected to edges

      call sepfillnmeshh
      if ( ierror/=0 ) go to 1000

!     --- Fill array nmeshl with nodes connected to edges

      call sepfillnmeshl

      if ( debug ) then

!     --- Debug information

         call sepprinnmeshc
         call sepprinnmeshd

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavedges'

      end if
      call erclos ( 'mshpavedges' )

      end
