      subroutine mshpavmarkbedg ( nedges, markedges, nmeshh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-07-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark all boundary edges by a -1 and all internal edges by a 0
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     edge
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nedges, markedges(*), nmeshh(*)

!     markedges      o    array of length nedges indicating whether a
!                         edge is internal (0) or on the boundary (-1)
!     nedges         i    number of edges
!     nmeshh         i    nmesh part h, contains elements connected to edges
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iedge, ipstart

!     iedge          edge sequence number
!     ipstart        Starting address in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array markedges (1:nedges)
!     For iedge := 1 (1) nedges do
!        if ( number of elements connected to edge i = 1 ) then
!           markedge(iedge) := -1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshpavmarkbedg' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshpavmarkbedg'

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- Clear array markedges (1:nedges)

      do iedge = 1, nedges
         markedges(iedge) = 0
      end do  ! iedge = 1, nedges

!     --- For iedge := 1 (1) nedges do
!           if ( number of elements connected to edge i = 1 ) then
!              markedge(iedge) := -1

      do iedge = 1, nedges

         ipstart = nmeshh(iedge)
         if ( nmeshh(ipstart).eq.1 ) markedges(iedge) = -1

      end do  ! iedge = 1, nedges

1000  call erclos ( 'mshpavmarkbedg' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshpavmarkbedg'

      end if

      end
