      subroutine mshposjac3 ( kmeshc, coor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-03-2007
!
!   copyright (c) 2007-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute Jacobian of quadratic tetrahedron
!     If Jacobian negative, change sequence of nodes
! **********************************************************************
!
!                       KEYWORDS
!
!     determinant
!     jacobian
!     element
!     positive
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(10)
      double precision coor(3,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     kmeshc        i/o   kmesh part c, contains the nodal points
!                         corresponding to one element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istore(9)
      double precision tetvolume, jacob

!     i              Counting variable
!     istore         Help array to store the node numbers that must be reordered
!     jacob          Jacobian of element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     TETVOLUME      Compute the volume of a tetrahedron
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshposjac3' )
      debug = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshposjac3'
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      jacob = tetvolume ( coor(1,kmeshc(1)), coor(1,kmeshc(3)), &
                          coor(1,kmeshc(5)), coor(1,kmeshc(10)) )
      if ( debug ) write(irefwr,2) 'jacob', jacob

      if ( jacob<0d0 ) then

!     --- Jacobian negative, change sequence of the first 3 nodes
!         This makes the Jacobian positive

         if ( debug ) call prinin ( kmeshc, 10, 'old sequence' )
         do i = 1, 9
            istore(i) = kmeshc(i)
         end do  ! i = 1, 9
         do i = 1, 5
            kmeshc(i) = istore(6-i)
         end do  ! i = 1, 5
         kmeshc(9) = istore(7)
         kmeshc(7) = istore(9)
         if ( debug ) then

!        --- Debug information

            jacob = tetvolume ( coor(1,kmeshc(1)), coor(1,kmeshc(3)), &
                                coor(1,kmeshc(5)), coor(1,kmeshc(10)) )
            call prinin ( kmeshc, 10, 'new sequence' )
            write(irefwr,2) 'jacob', jacob

         end if  ! ( debug )

      end if  ! ( jacob<0d0 )

1000  call erclos ( 'mshposjac3' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshposjac3'

      end if  ! ( debug )

      end
