      subroutine mshrdkmeshlmn ( irefc, ivers, ihelp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 09-06-2008 Adaptation lenth temp arrays
!        version  1.0  date 26-09-2007
!
!   copyright (c) 2007-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read kmesh parts l m and n from the file meshoutput
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer irefc, ivers, ihelp(10)

!     ihelp          i    Contains the input written by MSH059 on the second
!                         record after nvolms
!     irefc          i    dataset reference number for reading (>0 formatted,
!                         <0 unformatted)
!     ivers          i    Version sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ios
      character (len=80) text
      integer, allocatable :: kmeshmlc(:), kmeshnlc(:)

!     ios            IOSTAT variable, returns from reading or writing
!     kmeshmlc       KMESH part m, contains nodal point numbers of the
!                    curves
!     kmeshnlc       KMESH part n, contains nodal point numbers of the
!                    surfaces
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSH023         Read KMESH part l from a file
!     MSH024         Read KMESH part m from a file
!     MSH025         Read KMESH part n from a file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     195   error in reading of co-ordinate records
!     196   end of file found when reading co-ordinate records
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrdkmeshlmn' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrdkmeshlmn'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
100   format ( a )

      if ( nuspnt>0 ) then

!     --- nuspnt>0: kmesh part l

         if ( associated(km(iseqkm)%kmeshlglob) ) &
            deallocate(km(iseqkm)%kmeshlglob)
         allocate(km(iseqkm)%kmeshlglob(nuspnt))
         kmeshlglob => km(iseqkm)%kmeshlglob
         km(iseqkm)%kmeshpar(3) = nuspnt
         if ( ivers>=7 ) then

!        --- Read and skip comment record

            if ( irefc>=0 ) then
               read (irefc,100,iostat=ios) text
            else
               read(-irefc,iostat=ios) text
            end if
            if ( ios<0 ) then
               call errsub ( 196, 0, 0, 0 )
            else if ( ios>0 ) then
               call errint ( ios, 1 )
               call errsub ( 195, 1, 0, 0 )
            end if
            if ( debug ) write(irefwr,*) text
         end if
         call msh023 ( irefc, npoint, nuspnt, kmeshlglob )
      end if
      if ( ierror/=0 ) go to 1000

      if ( ncurvs>0 .and. (ihelp(1)==0 .or. nsurfs>0 .or. &
           ivers>=1 .and. ihelp(6)>0) ) then

!     --- ncurvs>0  read KMESH part m

         length = max(10*ncurvs+10000,npoint+10000)
         allocate ( kmeshmlc(length), stat = error )
         if ( error/=0 ) call eralloc ( error, length, 'kmeshmlc' )
         if ( ierror/=0 ) go to 1000

         if ( ivers>=7 ) then

!        --- Read and skip comment record

            if ( irefc>=0 ) then
               read (irefc,100,iostat=ios) text
            else
               read(-irefc,iostat=ios) text
            end if
            if ( ios<0 ) then
               call errsub ( 196, 0, 0, 0 )
            else if ( ios>0 ) then
               call errint ( ios, 1 )
               call errsub ( 195, 1, 0, 0 )
            end if
            if ( debug ) write(irefwr,*) text
         end if
         call msh024 ( irefc, npoint, kmeshmlc, ncurvs, length )
         if ( associated(km(iseqkm)%kmeshmglob) ) &
            deallocate(km(iseqkm)%kmeshmglob)
         allocate(km(iseqkm)%kmeshmglob(length))
         kmeshmglob => km(iseqkm)%kmeshmglob
         km(iseqkm)%kmeshpar(4) = length
         kmeshmglob = kmeshmlc(1:length)

         deallocate ( kmeshmlc, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshmlc' )
         if ( ierror/=0 ) go to 1000

      end if

      if ( nsurfs>0 .and. (ihelp(1)==0 .or. nvolms>0 .or. &
           ivers>=1 .and. ihelp(7)>0) ) then

!     --- nsurfs>0   read KMESH part n

         length = nsurfs*2+nelem*3+25000
         allocate ( kmeshnlc(length), stat = error )
         if ( error/=0 ) call eralloc ( error, length, 'kmeshnlc' )
         if ( ierror/=0 ) go to 1000

         if ( ivers>=7 ) then

!        --- Read and skip comment record

            if ( irefc>=0 ) then
               read (irefc,100,iostat=ios) text
            else
               read(-irefc,iostat=ios) text
            end if
            if ( ios<0 ) then
               call errsub ( 196, 0, 0, 0 )
            else if ( ios>0 ) then
               call errint ( ios, 1 )
               call errsub ( 195, 1, 0, 0 )
            end if
            if ( debug ) write(irefwr,*) text
         end if
         call msh025 ( irefc, npoint, kmeshnlc, nsurfs, length )
         if ( associated(km(iseqkm)%kmeshnglob) ) &
            deallocate(km(iseqkm)%kmeshnglob)
         allocate(km(iseqkm)%kmeshnglob(length))
         kmeshnglob => km(iseqkm)%kmeshnglob
         km(iseqkm)%kmeshpar(5) = length
         kmeshnglob = kmeshnlc(1:length)

         deallocate ( kmeshnlc, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'kmeshnlc' )
         if ( ierror/=0 ) go to 1000

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrdkmeshlmn'

      end if  ! ( debug )
      call erclos ( 'mshrdkmeshlmn' )

      end
