      subroutine mshrdparmsh3 ( irefc, elgrp, nelgrp, kmeshc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 09-05-2012 Remove ivers, extra kmeshc
!        version  1.0  date 26-09-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read topology array
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer irefc, nelgrp, elgrp(2,nelgrp), kmeshc(*)

!     elgrp          i    Array of size 2 x nelgrp
!                         (1,i) contains number of nodes in element group i
!                         (2,i) contains number of elements of element group i
!     irefc          i    dataset reference number for reading (>0 formatted, &
!                         <0 unformatted)
!     kmeshc         o    Global array kmeshc
!     nelgrp         i    Number of element groups
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inelem, inpelm, ios, ielgrp, ipstart
      character (len=80) text

!     ielgrp         element group number
!     inelem         Number of elements in a specific element group
!     inpelm         number of points per element
!     ios            IOSTAT variable, returns from reading or writing
!     ipstart        Starting address in kmeshc
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     197   error in reading of element records
!     198   end of file found when reading element records
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrdparmsh3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrdparmsh3'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
100   format ( a )

      ipstart = 1
      if ( irefc>=0 ) then

!     --- Formatted case

         read (irefc,100,iostat=ios) text
         if ( debug ) write(irefwr,*) text(1:10)

         do ielgrp = 1, nelgrp
            inpelm = elgrp(1,ielgrp)
            inelem = elgrp(2,ielgrp)
            if ( inelem>0 ) then
               read(irefc,*,iostat = ios) &
                  kmeshc(ipstart:ipstart+inelem*inpelm-1)
            end if
            ipstart = ipstart+inelem*inpelm
         end do

         if ( ios<0 ) then
            call errsub ( 198, 0, 0, 0 )
         else if ( ios>0 ) then
            call errint ( ios, 1 )
            call errsub ( 197, 1, 0, 0 )
         end if

      else

!     --- Unformatted case

         read (-irefc,iostat=ios) text
         if ( debug ) write(irefwr,*) text(1:10)
         do ielgrp = 1, nelgrp
            inpelm = elgrp(1,ielgrp)
            inelem = elgrp(2,ielgrp)
            if ( inelem>0 ) then
              read(-irefc,iostat = ios)  &
              kmeshc(ipstart:ipstart+inelem*inpelm-1)
            end if
            ipstart = ipstart+inelem*inpelm
         end do

         if ( ios<0 ) then
            call errsub ( 198, 0, 0, 0 )
         else if ( ios>0 ) then
            call errint ( ios, 1 )
            call errsub ( 197, 1, 0, 0 )
         end if

      end if

1000  call erclos ( 'mshrdparmsh3' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrdparmsh3'

      end if  ! ( debug )

      end
