      subroutine mshrefinenew ( refinenew, refmapnew, nelemnew )
! ======================================================================
!
!        programmer    John Brusche
!        version  1.0  date 21-04-2005
!
!   copyright (c) 2005-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array refinenew with information obtained from the new mapping
!     as stored in refmapnew
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer refmapnew(2,*), refinenew(*), nelemnew

!     nelemnew       i    Number of elements in new mesh
!     refinenew     i/o   Reference array containing the required level
!                         of refinement per element of the new mesh
!     refmapnew      i    Mapping array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefinenew' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefinenew'

      end if
      if ( ierror/=0 ) go to 1000

      do i = 1, nelemnew
         refinenew(i) = refmapnew(2,i)
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefinenew'

      end if
      call erclos ( 'mshrefinenew' )

      end
