      subroutine mshreflocal ( kmeshcnew, coornew, iedges, kmeshnew, &
                               elements, &
                               refarray, lenkmscnew, newblockinfo, &
                               newtransinfo, lennewtrans, istorest, &
                               leng2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 25-04-2012 Extra parameters for parallel
!        version  1.4  date 05-03-2012 Clear kmesh part h
!        version  1.3  date 22-11-2011 Extra debug
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Refine an existing mesh locally and store the result in a
!     new array kmesh
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulekmesh
      use sepmodulecpack
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshcnew(*), iedges(*), kmeshnew, elements(*), &
              lenkmscnew, refarray(*), lennewtrans, leng2, &
              newtransinfo(lennewtrans), istorest(4,*), &
              newblockinfo(leng2)
      double precision coornew(*)

!     coornew        o    Array containing the new coordinates of the nodes
!     elements            Work array of length nelem
!                         elements(ielem) edges of element ielem are subdivided
!     iedges        i/o   Work array of size nedges
!                         If iedges(i)>0, edge i must be splitted and
!                         the node number will be iedges(i)+npoint
!     istorest       o    array of length (4,iacnodes) containing information
!                         for the various blocks
!                         istorest(1,iblock):  starting address of block iblock
!                         istorest(2,iblock):  number of edges of block iblock
!                         istorest(3,iblock):  starting address in transportinfo
!                         istorest(4,iblock):  sequence number of neighbor
!     kmeshcnew      o    New kmesh part c
!     kmeshnew       o    New array kmesh
!     leng2          i    Declared length of array newblockinfo
!     lenkmscnew     o    Length of array kmshcnew
!     lennewtrans   i/o   Length of new array transinfo
!                         At input declared length
!                         At output actual length
!     newblockinfo  i/o   New array blockinfo
!     newtransinfo   o    New array transportinfo
!     refarray       i    Array of size nelem that is filled with a 0
!                         if the element ielem must not be subdivided and a 1
!                         if it must
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer npointnew, nsubdiv, i, idummy(1), iseqkm1, iseqkm2
      integer, allocatable :: refedges(:)

!     i              Counting variable
!     idummy         Dummy parameter
!     iseqkm1        Sequence number with respect to km of first mesh
!     iseqkm2        Sequence number with respect to km of second mesh
!     npointnew      Sequence number of last generated node
!     nsubdiv        number of extra nodes that are created
!     refedges       Contains the edge numbers of the edges that
!                    must be subdivided
!                    Length: nsubdiv
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCOORREF     Extend array coornew with the coordinates of the extra
!                    points that are created by subdividing the edges
!     MSHCOPPART     Copy part of kmesh to kmeshnew
!     MSHELEMREF     Create a new topology constisting of the set kmeshc/g/t
!                    in case of a local refinement
!     MSHMARKEDGES   Mark all edges that must be sudivided
!     MSHMARKEDGPAR  make sure that edges on common sides of blocks in parallel
!                    case are subdivided in both blocks
!     MSHMARKELEMS   Mark all elements with the number of edges
!                    that are subdivided
!     PR0003         Print KMESH parts c and g
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
!     SEPPRINKMESHC  print kmesh part c
!     SEPPRINNMESHC  print nmesh part c
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Copy part of kmesh to kmeshnew
!     Copy coordinates from coor to coornew
!     Mark all edges of the elements to be subdivided
!     Mark all elements that must be subdivided
!     Create new nodes and put in array coornew
!     Create new elements
!     Fill arrays imap and rmap
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshreflocal' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshreflocal'
         call sepprinnmeshc
         write(irefwr,1) 'lennewtrans', lennewtrans
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkm1 = iseqkm

      allocate ( refedges(nedges), stat = error )
      if ( error/=0 ) call eralloc ( error, nedges, 'refedges' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then
         call prinrl1 ( coor, npoint, ndim, 'coor' )
         call prinin ( refarray, nelem, 'refarray' )
      end if  ! ( debug )

!     --- Copy part of kmesh to kmeshnew
!         Copy coordinates from coor to coornew
!         It concerns the parts a, f-h, p-x, ag and ak

      if ( debug ) write(irefwr,*) 'before mshcoppart'

      call mshcoppart ( kmeshnew, coornew, iseqkm2 )
      km(iseqkm2)%kmesh(22) = 0  ! clear kmesh part h
      km(iseqkm2)%kmesh(29) = 0  ! clear kmesh part o
      if ( ierror/=0 ) go to 1000

!     --- Fill first part of imap and rmap with 1

      do i = 1, npoint
         imap(i,1) = 1
         imap(i,2) = i
         imap(i,3) = 0
         rmap(i,1) = 1d0
         rmap(i,2) = 0d0
      end do

!     --- Mark all edges of the elements to be subdivided

      if ( debug ) write(irefwr,*) 'before mshmarkedges'
      call mshmarkedges ( iedges, nedges, refarray, nmeshd, nelem, &
                          nsubdiv, npoint, 1, refedges )
      if ( ierror/=0 ) go to 1000

      if ( parallel ) then

!     --- parallel case, make sure that edges on common sides of blocks
!         are subdivided in both blocks

         if ( debug ) write(irefwr,*) 'before mshmarkedgpar'
         call mshmarkedgpar ( refedges, nsubdiv, &
                              iedges, newtransinfo, newblockinfo, &
                              lennewtrans, istorest, leng2 )
         if ( debug ) write(irefwr,1) 'lennewtrans', lennewtrans

      end if  ! ( parallel )

!     --- Mark all elements that must be subdivided

      if ( debug ) write(irefwr,*) 'before mshmarkelems'
      call mshmarkelems ( iedges, elements, nmeshd, nelem )
      if ( ierror/=0 ) go to 1000
      if ( debug ) call prinin ( elements, nelem, 'elements' )

!     --- Create new nodes and put in array coornew

      call mshcoorref ( iedges, nedges, coornew, nmeshc, &
                        ndim, .true. )
      if ( ierror/=0 ) go to 1000
      npointnew = npoint+nsubdiv
      if ( debug ) then
         call prinrl1 ( coornew, npointnew, ndim, 'coornew' )
         call  sepprinkmeshc
      end if  ! ( debug )

!     --- Create new elements

      call mshelemref ( elements, iedges, iseqkm2, &
                        kmeshcnew, coornew, lenkmscnew, idummy, idummy, &
                        idummy, idummy, idummy(1), refarray, &
                        idummy, .false., idummy(1), idummy, idummy(1) )
      if ( ierror/=0 ) go to 1000
      km(iseqkm2)%kmesh(8) = npointnew

      deallocate ( refedges, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'refedges' )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kmeshnew, 80, 'kmeshnew' )
         call pr0003 ( kmeshnew, kmeshcnew )
         write(irefwr,1) 'lennewtrans', lennewtrans
         write(irefwr,*) 'End mshreflocal'

      end if
      iseqkm = iseqkm1
      call erclos ( 'mshreflocal' )

      end
