      subroutine mshrefpar12 ( iseqkm2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-05-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy global arrays coor_glob, kmeshm/n_glob from node 1 to other nodes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     parallel
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqkm2

!     iseqkm2        i    Sequence number with respect to km of second mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lencoor, lenkmeshm, lenkmeshn, inode, icontext
      integer, pointer :: send_buffer(:)
      integer, save :: send_buffer_size

!     icontext       Tag for sending and receiving, is used as check
!     inode          Node sequence number
!     lencoor        Length of array COOR
!     lenkmeshm      Length of array kmeshm
!     lenkmeshn      Length of array kmeshn
!     send_buffer    send buffer
!     send_buffer_size  Length of the send buffer
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIBSENDMPI    Send information to other node (mpi buffered)
!     INIBUFATTMPI   Attach mpi buffer if it exists
!     INIBUFDETMPI   Detach mpi buffer if it exists
!     INIBUFSIZEMPI  Extend send_buffer_size with mpi overhead
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefpar12' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefpar12'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      lencoor = size ( km(iseqkm2)%coorglob )
      lenkmeshm = size ( km(iseqkm2)%kmeshmglob )

      if ( debug ) &
         write(irefwr,1) 'ndim, lencoor, lenkmeshm', &
                          ndim, lencoor, lenkmeshm

      send_buffer_size = (iacnodes-1)*(8*lencoor+4*lenkmeshm)

      if ( ndim>2 ) then
         lenkmeshn = size ( km(iseqkm2)%kmeshnglob )
         send_buffer_size = send_buffer_size+(iacnodes-1)*4*lenkmeshn
      end if  ! ( ndim>2 )

!     --- Compute actual buffer size using mpi information
!         allocate array and attach buffer

      call inibufsizempi ( send_buffer_size, iacnodes )

      allocate(send_buffer(send_buffer_size))

      call inibufattmpi ( send_buffer, send_buffer_size )

!     --- send coor to all nodes

      do inode = 2, iacnodes
         icontext = 27
         call inibsendmpi ( km(iseqkm2)%coorglob, lencoor, SPDOUBLE, inode, &
                            icontext )
      end do  ! inode = 2, iacnodes
      if ( debug ) write(irefwr,*) 'after sending coor'

!     --- send kmeshm to all nodes

      do inode = 2, iacnodes
         icontext = 28
         call inibsendmpi ( km(iseqkm2)%kmeshmglob, lenkmeshm, SPINT, inode, &
                            icontext )
      end do  ! inode = 2, iacnodes
      if ( debug ) write(irefwr,*) 'after sending kmeshm'

      if ( ndim>2 ) then

!     --- ndim>2, send kmeshn

         lenkmeshn = size ( km(iseqkm2)%kmeshnglob )

         do inode = 2, iacnodes
            icontext = 29
            call inibsendmpi ( km(iseqkm2)%kmeshnglob, lenkmeshn, SPINT, inode, &
                               icontext )
         end do  ! inode = 2, iacnodes
         if ( debug ) write(irefwr,*) 'after sending kmeshn'

      end if  ! ( ndim>2 )

      if (associated(send_buffer)) then

!     --- deattach send_buffer and deallocate

         call inibufdetmpi ( send_buffer, send_buffer_size )
         deallocate(send_buffer)

      end if  ! (associated(send_buffer))

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefpar12'

      end if  ! ( debug )
      call erclos ( 'mshrefpar12' )

      end subroutine mshrefpar12
