      subroutine mshrefpar4 ( coornew, coorloc, imapping, npoint, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy coordinates from local coordinates array into global
!     coordinates array using array imapping
! **********************************************************************
!
!                       KEYWORDS
!
!     refine
!     parallel
!     coordinate
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
       integer npoint, ndim, imapping(npoint)
       double precision coornew(ndim,*), coorloc(ndim,npoint)

!     coorloc        i    Array with the coordinates of the local mesh
!     coornew        o    Array containing the new coordinates of the nodes
!                         global mesh
!     imapping       i    Contains the global node numbers of the mesh
!     ndim           i    Dimension of the space.
!     npoint         i    Number of nodal points in local mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshrefpar4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshrefpar4'
         call prinrl1 ( coorloc, npoint, ndim, 'coorloc' )
         call prinin ( imapping, npoint, 'imapping' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      coornew(:,imapping(1:npoint)) = coorloc(:,1:npoint)

1000  call erclos ( 'mshrefpar4' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshrefpar4'

      end if  ! ( debug )

      end
