      subroutine mshshift2d1 ( iworkpoint, skipnod )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 22-05-2013 Correction nodes on curves
!        version  1.0  date 01-12-2011
!
!   copyright (c) 2011-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark nodes that ly on curves
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     triangle
!     subdivision
!     adaptation
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iworkpoint(*), skipnod(*)

!     iworkpoint     i    Work array of length npointnew
!                         This array is filled as follows:
!                         iworkpoint(i) = 0 if node i outside obstacle
!                         iworkpoint(i) = 1 if node i inside obstacle
!                         iworkpoint(i) = 2 if node i inside obstacle (near
!                                           boundary)
!                         iworkpoint(i) = 3 if node i on obstacle
!     skipnod       i/o   Array of npoint long indicating if a node has been
!                         moved or not
!                         See subroutine mshshift2d
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, inpelm, inode, nodenr, icurv

!     icurv          Loop variable
!     inode          Node sequence number
!     inpelm         number of points per element
!     istart         First position used at a part
!     nodenr         Node sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshshift2d1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshshift2d1'
         call prinin ( iworkpoint, npoint, 'iworkpoint' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istart = kmeshm(2)
      inpelm = 0

!     --- loop over curves

      do icurv = 1, ncurvs

         istart = istart+inpelm
         inpelm = kmeshm(5+icurv)-kmeshm(5+icurv-1)

!        --- loop over nodes

         do inode = 1, inpelm

            nodenr = kmeshm(istart-1+inode)
            if ( nodenr>0 ) then

!           --- Node exists

               if ( inode==1 .or. inode==inpelm ) then

!              --- node is end point and may not be moved

                   skipnod(nodenr) = 4

               else if ( iworkpoint(nodenr)==3 ) then

!              --- node on curve and obstacle

                  skipnod(nodenr) = 2

               else

!              --- node on curve

                  skipnod(nodenr) = -icurv

               end if  ! ( iworkpoint(nodenr)==3 )

            end if  ! ( nodenr>0 ) then

         end do  ! inode = 1, inpelm

      end do  ! icurv = 1, ncurvs

1000  if ( debug ) then

!     --- Debug information

         call prinin ( skipnod, npoint, 'skipnod' )
         write(irefwr,*) 'End mshshift2d1'

      end if  ! ( debug )
      call erclos ( 'mshshift2d1' )

      end
