      subroutine mshshift2d3 ( skipnod, iwork, edgemoved )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 10-02-2012 use getnpelmelgrp
!        version  1.0  date 01-12-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark edges of which one of the nodes has been moved
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     intersection
!     triangle
!     subdivision
!     adaptation
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer skipnod(*), iwork(nelem,*), edgemoved(*)

!     edgemoved     i/o   Array of size nedges indicating if nodes on an edge
!                         have been moved (1) or not (0)
!     iwork          i    integer work array
!                         See subroutine mshobstms2d
!     skipnod       i/o   Array of npoint long indicating if a node has been
!                         moved or not
!                         See subroutine mshshift2d
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, inpelm, istart, i, ireledge, &
              istarticross, ipstart, iedge, ielgrp, icross, iielem, &
              iobst, nodebegin, nodeend, jump
      logical linear

!     i              Counting variable
!     icross         Position of iedge in array icrossobst
!     iedge          edge sequence number
!     ielem          Element number
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     inpelm         number of points per element
!     iobst          Obstacle sequence number
!     ipstart        Starting address in array
!     ireledge       relative edge sequence number
!     istart         First position used at a part
!     istarticross   Starting address of obstacle in array icrossobst
!     jump           2 in case linear elements are used, 3 if quadratic
!     linear         Indicates whether the element is a linear simplex
!                    (true) or not (false)
!     nodebegin      Node at the begin of a crossed edge
!     nodeend        Node at the end of a crossed edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshshift2d3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshshift2d3'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initialize parameters

      ielem=0
      istart=0

!     --- Loop over element groups

      do ielgrp = 1, nelgrp

         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         linear = inpelm<=4
         if ( linear ) then
            jump = 2
         else
            jump = 3
         end if  ! ( linear )

!        --- loop over elements

         do iielem = 1, inelem

            ielem=ielem+1

            if ( debug ) &
               write(irefwr,1) 'ielem, iwork(ielem,1)', &
                                ielem, iwork(ielem,1)

            if ( iwork(ielem,1)/=0 ) then

!           --- iwork(ielem,1)#0, hence element is intersected

               istart = iwork(ielem,1)  ! starting address in icrossobst
               iobst = iwork(ielem,2)
               istarticross = icrossobst(iobst)

!              --- loop over crossed edges

               do i = 1, icrossobst(istart)

                  ireledge = icrossobst(istart+i)
                  icross  = istarticross+1+2*(ireledge-1)
                  iedge = icrossobst(icross)
                  if ( edgemoved(iedge)==0 ) then
                     ipstart = nmeshc(iedge)
                     nodeend = nmeshc(ipstart+1)  ! end node
                     nodebegin = nmeshc(ipstart+jump)  ! start end
                     if ( skipnod(nodeend)==3 .or. &
                          skipnod(nodebegin)==3 ) edgemoved(iedge) = 1

                  end if  ! ( edgemoved(iedge)==0 )

               end do  ! i = 1, icrossobst(istart)

            end if  ! ( iwork(ielem)/=0 )

         end do  ! iielem = 1, inelem

      end do  ! ielgrp = 1, nelgrp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshshift2d3'

      end if  ! ( debug )
      call erclos ( 'mshshift2d3' )

      end
