      subroutine mshsmoothadcr1 ( coornew, coorold, nodes, nnodes, &
                                  kprobf, usol, idim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-02-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt coordinates of surface using the displacement vector
!
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     smoothing
!     free_surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, nodes(nnodes), kprobf(*), idim
      double precision coornew(3,*), coorold(3,*), usol(nnodes)

!     coornew       i/o   Array containing the new coordinates of the nodes
!     coorold        i    Array containing the old coordinates of the nodes
!     idim           i    dimension of coordinate we want to use
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated (refers to usol)
!     nnodes         i    Number of nodes in nodes
!     nodes          i    Contains the node numbers that must be filled
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nodp, ipos

!     i              Counting variable
!     ipos           Position of unknown in usol
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsmoothadcr1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsmoothadcr1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, nnodes
         nodp = nodes(i)         ! node in surface
         ipos = kprobf(nodp)+1   ! position of node in usol
         coornew(idim,nodp) = usol(ipos)+coorold(idim,nodp)
      end do  ! i = 1, nnodes

1000  call erclos ( 'mshsmoothadcr1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshsmoothadcr1'

      end if  ! ( debug )

      end
