      subroutine mshsmoothbc ( idim, isol, coornw, nodepnts, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-02-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill boundary conditions due to displacement of boundary
!     in solution vector
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     smoothing
!     free_surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer idim, isol, nnodes, nodepnts(nnodes)
      double precision coornw(3,*)

!     coornw        i    Array containing the new coordinates of the nodes
!     idim           i    direction of coordinate
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution array
!                         This array is filled with the boundary conditions
!                         for surface isurnr
!     nnodes         i    Number of nodes
!     nodepnts       i    Contains the node numbers of the surface
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer itypvc, ivec

!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHSMOOTHBC1   Fill boundary conditions due to displacement of boundary
!                    in solution vector
!     SEPCREASOL110  Activate or create arrays of type 110
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsmoothbc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsmoothbc'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Activate arrays

      call sepcreasol110 ( isol, 'surfdisp', 0 )

!     --- Fill array usol with displacement of the boundary in dir idim

      call mshsmoothbc1 ( coornw, coor, nodepnts, nnodes, &
                          kprobf, usoltemp, idim )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshsmoothbc'

      end if  ! ( debug )
      call erclos ( 'mshsmoothbc' )

      end subroutine mshsmoothbc
