      subroutine mshsrfboun ( rinput, holeinfo, &
                              maxholes, nholes, boundary, &
                              numcurvboun, doublecurvs, trans, &
                              iprec, iinput, iposi, bcord3d )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 30-11-2014 Change isurf 23 in 99
!        version  3.0  date 02-09-2013 Extra parameter bcord3d
!        version  2.1  date 25-11-2012 New call to msh011
!
!   copyright (c) 2008-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays bcord with coordinates of the boundary of a surface
!     Fill array boundary with information of the boundary curves
!     Compute nbndpt
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer holeinfo(2,0:*), maxholes, nholes, iprec, iinput(*), &
              iposi, boundary(*), numcurvboun
      double precision trans(*), rinput(*), bcord3d(ndim,*)
      logical doublecurvs

!     bcord3d        o    array containing the coordinates of the boundaries
!     boundary       o    Array of size 2 x numcurvboun containing
!                         information of the boundary of the surface
!                         (1,i) contains the i-th curve number (including sign)
!                         (2,i) contains the starting address in kbndpt
!     doublecurvs    o    If true the boundary contains double curves
!     holeinfo       o    Integer array of size 2 x (nholes+2)
!                         Contains information about holes in surfaces and where
!                         a new part of the boundary starts
!                         Pos (1,i-1) contains the local sequence number of the
!                            first curve of part i provided with a sign
!                            If the sign is positive the boundary is
!                            created counter clockwise, otherwise clockwise
!                         Pos (2,i-1) contains the local sequence number of the
!                            first node of of part i
!                         Pos (1,nholes+1) contains the number of curves of the
!                         boundary
!     iinput         i    integer input array containing information of the mesh
!     iposi          i    position in iinput
!     iprec          o    Surface number of surface to be copied etc.
!     maxholes       i    Maximum number of holes found in a boundary of a
!                         surface
!     nholes         o    Number of holes in the boundary of a surface
!     numcurvboun    o    Number of curves in the boundary
!     rinput         i    Real input array containing information of the mesh
!     trans          o    Contains information of the tranformation 3D-->2D
!                         Pos. 1-3:  t1
!                         Pos. 4-6:  t2
!                         Pos. 7-9:  t3
!                         Pos.10-18:  trans (inverse of transformation)
!                         Pos.19-21:  bcorut (coordinates of first point)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer is, nbnold

!     is             help parameter to store value of isurnr
!     nbnold         Help parameter to store value of nbndpt
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     MSH009         Compute transformation and inverse transformation for a
!                    plane in 3D
!     MSH011         fill local array bcord with the coordinates
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsrfboun' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsrfboun'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Initialize parameters

      irnp4  = max(1,irnp4)

      if ( debug ) &
         write(irefwr,1) 'isurf, isurnr, ndim', isurf, isurnr, ndim

      if ( ndim==3 .and. (isurf>0 .and. isurf<5 .or. &
           isurf==17 .or. isurf==19 .or. isurf==99) ) then

!     --- ndim = 3,  and  isurf = 1, 2, 3 or 4  map bcord into r2


         allocate ( bcordtmp(ibcrds,lnbcor), stat = error )
         if ( error/=0 ) call eralloc ( error, ibcrds*lnbcor, 'bcordtmp' )
         if ( ierror/=0 ) go to 1000
         if ( debug ) write(irefwr,1) 'nbndpt before msh011', nbndpt
         call msh011 ( rinput(irnp4), &
                       holeinfo, maxholes, nholes, .true., boundary, &
                       numcurvboun, doublecurvs, iinput )
         if ( debug ) write(irefwr,1) 'nbndpt before msh009', nbndpt
         nbndpt = nbndpt+lenextracrvs
         call msh009 ( bcordtmp, bcord2d, trans, trans(4), trans(7), &
                       trans(10), trans(19), bcord3d )
         nbndpt = nbndpt-lenextracrvs

         deallocate ( bcordtmp, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'bcordtmp' )

      else if ( isurf==21 ) then

!     --- isurf = 21, copy surface

         iprec = iinput(iposi+iinput(iposi+2)+3)
         nbndpt = 0
         nholes = 0

      else

!     --- other possibilities

         bcordtmp => bcord2d
         call msh011 ( rinput(irnp4), &
                       holeinfo, maxholes, nholes, .true., boundary, &
                       numcurvboun, doublecurvs, iinput )
         nullify(bcordtmp)
         if ( isurf==8 .or. isurf==9 .or. isurf==14 .or. &
              isurf==16 ) then

!        --- isurf = 8, 9, 14 or 16   compute old bcord and store in bcorold

            iprec = iinput(iposi+iinput(iposi+2)+3)
            is = isurnr
            isurnr = iprec
            nbnold = nbndpt
            bcordtmp => bcorold
            call msh011 ( rinput(irnp4), &
                          holeinfo, maxholes, nholes, .false., &
                          boundary, numcurvboun, doublecurvs, iinput )
            nbndpt = nbnold
            isurnr = is
            nullify(bcordtmp)

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshsrfboun'

      end if  ! ( debug )
      call erclos ( 'mshsrfboun' )

      end
