      subroutine mshsrftria ( jnew, bcord, kbndpt, boundary, numcurvboun, &
                              holeinfo,  nholes, trans, &
                              iinput, rinput, bcord3d )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 12-03-2015 Extra debug
!        version  3.0  date 22-01-2014 Extra parameter bcord3d
!        version  2.1  date 02-11-2010 New call to mshoce (work arrays removed)
!
!   copyright (c) 2005-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create surface mesh using surface generator mshoce
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kbndpt(*), holeinfo(2,0:*), nholes, iinput(*), &
              numcurvboun, boundary(2,numcurvboun)
      double precision bcord(*), trans(*), rinput(*), bcord3d(*)
      logical jnew

!     bcord          i    array containing the coordinates of the boundaries
!     bcord3d        i    Contains the coordinates of the 3d boundary
!                         Is only used for error messages
!     boundary       i    array of size 2 x numcurvboun containing
!                         information of the boundary of the surface
!                         (1,i) contains the i-th curve number (including sign)
!                         (2,i) contains the starting address in kbndpt
!     holeinfo       o    Integer array of size 2 x (nholes+2)
!                         Contains information about holes in surfaces and where
!                         a new part of the boundary starts
!                         Pos (1,i-1) contains the local sequence number of the
!                            first curve of part i provided with a sign
!                            If the sign is positive the boundary is
!                            created counter clockwise, otherwise clockwise
!                         Pos (2,i-1) contains the local sequence number of the
!                            first node of of part i
!                         Pos (1,nholes+1) contains the number of curves of the
!                         boundary
!     iinput         i    integer input array, see subroutine MESHBODY
!     jnew           i    indication whether the mesh is new (true) or old
!     kbndpt         o    array containing the nodal point numbers of the
!                         boundary
!     nholes         o    Number of holes in the boundary of a surface
!     numcurvboun    i    number of curves in the boundary of the surface
!     rinput         i    Real input array, see subroutine MESHBODY
!     trans          i    Contains information of the tranformation 3D-->2D
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer npoh, nelh, iposiinp, iposiinpcrv
      double precision coorusp(3,nextrapnts)

!     coorusp        Work array of size 3 x coorusp to store the coordinates
!                    of the transformed extra user points and coarsenesses
!     iposiinp       Starting address in array iinput with respect to the
!                    extra user points (internal points)
!     iposiinpcrv    Starting address in array iinput with respect to the
!                    extra user curves (internal curves)
!     nelh           Estimate of the number of elements
!     npoh           Estimate of the number of points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     MSHOCE         Submesh generator TRIANGLE
!     MSHSRFTRIACP   Copy coordinates of user points from rinput into
!                    array trans
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     141   The actual generated number of nodal points is more than the
!           estimated number
!     157   The actual generated number of elements is more than the
!           estimated number
!     910   Mesh generator Triangle is only available for triangular elements
!           with 3 or 6 nodes
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsrftria' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsrftria'
         write(irefwr,1) 'isurnr, nbndpt', isurnr, nbndpt
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      npoh = npointlc           ! estimate of number of points
      nelh = nelemlc            ! estimate of number of elements
      iposiinp = max(1,ncoarstart)  ! address in iinput for userpoints
      iposiinpcrv = max(1,ncurvstart)  ! address in iinput for curve numbers
      if ( ishape>=9 ) inpelm = inpelm-1

      if ( inpelm/=3 .and. inpelm/=6 .and. inpelm/=7 ) then

           call errsub ( 910, 0, 0, 0 )
           go to 1000

      end if

      if ( debug ) write(irefwr,1) 'nextracrvs', nextracrvs
      if ( nextrapnts>0 ) then

!     --- nextrapnts>0, so there are extra points, fill coordinates in coorusp

         call mshsrftriacp ( rinput(irnp1), coorusp, iinput(iposiinp), &
                             nextrapnts, ndim, rinput(irnp1+ndim*nuspnt), &
                             jcoars, nuspnt, trans(10), trans(19) )

      end if  ! ( nextrapnts>0 )

      call mshoce ( jnew, coor2d, kmeshclc, inpelm, nbndpt, bcord, &
                    kbndpt, boundary, numcurvboun, npointlc, nelemlc, &
                    holeinfo, nholes, nextrapnts, coorusp, &
                    iinput(iposiinp), isurnr, nextracrvs, &
                    kbndpt(lnkbnd+1), iinput(iposiinpcrv), &
                    rinput, nuspnt, ndim, trans, bcord3d )

      if ( debug ) then

!     --- debug information

         if ( nholes>0 ) &
            call prinin1 ( holeinfo, nholes+2, 2, 'holeinfo' )

      end if

      if ( npoh<npointlc ) then
         call errint ( isurnr, 1 )
         call errint ( npointlc, 2 )
         call errint ( npoh  , 3 )
         call errsub ( 141, 3, 0, 0 )
      end if
      if ( nelh<nelemlc ) then
         call errint ( isurnr, 1 )
         call errint ( nelemlc , 2 )
         call errint ( nelh  , 3 )
         call errsub ( 157, 3, 0, 0 )
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'lnkbnd', lnkbnd
         call prinin1 ( kmeshclc, nelemlc, inpelm, 'kmeshclc' )
         call prinrl1 ( coor2d, npointlc, 2, 'coor2d' )
         call prinin ( kbndpt, nbndpt, 'kbndpt' )
         write(irefwr,*) 'End mshsrftria'

      end if
      call erclos ( 'mshsrftria' )

      end
