      subroutine mshsrftriacp ( rinput, coorusrp, iinput, ncoar, ndim, &
                                rinputcoarse, jcoars, nuspnt, trans, &
                                bcord )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 11-06-2008 Extra parameters
!        version  1.1  date 05-05-2005 Minor change in comments
!        version  1.0  date 24-01-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy coordinates of user points from rinput into array coorusrp
!     In case of R^3 apply transformation
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     triangle
!     copy
!     user_point
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), ncoar, ndim, nuspnt, jcoars
      double precision rinput(ndim,*), coorusrp(3,*), rinputcoarse(*), &
                       trans(3,3), bcord(3)

!     bcord          i    array containing the coordinates of the first
!                         node in the boundary (3d)
!                         This is required for the transformation
!     coorusrp       o    array containing the coordinates as well as the
!                         local coarseness of internal points to be placed
!                         in the surface
!                         Length 3 x ncoar
!                         (1,i):  x-coordinate of point i
!                         (2,i):  y-coordinate of point i
!                         (3,i):  coarseness of point i
!     iinput         i    integer input array containing the user point numbers
!                         of the extra internal points
!     jcoars         i    If 1 coarseness is given, if 0 not
!     ncoar          i    Number of internal points in surface
!     ndim           i    Dimension of the space.
!     nuspnt         i    Number of user points
!     rinput         i    Real input array, see MESHBODY
!                         This part contains the coordinates of the user points
!     rinputcoarse   i    Real input array
!                         This part contains the coarseness of the user points
!     trans          i    array containing the inverse of the transformation
!                         See subroutine msh009
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iuserp, j, k
      double precision unitlength, sum

!     i              Counting variable
!     iuserp         User point sequence number
!     j              Counting variable
!     k              Counting variable
!     sum            Help parameter to compute a sum
!     unitlength     Unit length for coarseness
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsrftriacp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsrftriacp'
         call prinin ( iinput, ncoar, 'user points' )
         write(irefwr,1) 'jcoars, nuspnt', jcoars, nuspnt
  1      format ( a, 1x, (10i6) )
         if ( jcoars==1 ) &
            call prinrl ( rinputcoarse(2), nuspnt, 'coarseness' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( jcoars==1 ) unitlength = rinputcoarse(1)

      if ( ndim==2 ) then

!     --- ndim = 2, just copy

         do i = 1, ncoar
            iuserp = iinput(i)
            coorusrp(1:2,i) = rinput(1:2,iuserp)
            if ( jcoars==1 ) then
               coorusrp(3,i) = rinputcoarse(iuserp+1)*unitlength
            else
               coorusrp(3,i) = 0d0
            end if  ! ( jcoars==1 )
         end do  ! i = 1, ncoar

      else

!     --- ndim = 3, transform

         do i = 1, ncoar

            iuserp = iinput(i)

!           --- compute transformation

            do j = 1, 2

               sum = 0d0
               do k = 1, 3
                  sum = sum + (rinput(k,iuserp)-bcord(k))*trans(j,k)
               end do  ! k = 1, 3
               coorusrp(j,i) = sum

            end do  ! j = 1, 2

!           --- store coarseness

            if ( jcoars==1 ) then
               coorusrp(3,i) = rinputcoarse(iuserp+1)*unitlength
            else
               coorusrp(3,i) = 0d0
            end if  ! ( jcoars==1 )

         end do  ! i = 1, ncoar
         go to 1000

      end if  ! ( ndim==2 )

1000  call erclos ( 'mshsrftriacp' )

      if ( debug ) then

!     --- Debug information

         call prinrl1 ( coorusrp, ncoar, 3, &
                        'coordinates of user points' )
         write(irefwr,*) 'End mshsrftriacp'

      end if

      end
