      subroutine mshstoreangle ( coor, numpoints, curvenodes, ndim, &
                                 angles )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-10-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store the angles corresponding to the closed curve with nodes
!     stored in curvenodes, into array angles
!     It is supposed that first and end point have different numbers
! **********************************************************************
!
!                       KEYWORDS
!
!     angle
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numpoints, curvenodes(*), ndim
      double precision coor(ndim,*), angles(*)

!     angles         o    Array containing the angles along the closed curve in
!                         the sequence defined by curvenodes
!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     curvenodes     i    Contains the nodes of the boundary curve
!     ndim           i    Dimension of the space.
!     numpoints      i    Number of points in curvenodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ibef, inow, next, i

!     i              Counting variable
!     ibef           Previous node
!     inow           Actual node
!     next           next node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHANGLE       Compute the cosine of the angle between the nodes with
!                    coordinates x1, x2 and x3, where the angle is defined
!                    as x3-x1-x2
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshstoreangle' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshstoreangle'
         call prinin ( curvenodes, numpoints, 'curvenodes' )

      end if  ! ( debug )
      if ( ierror.ne.0 ) go to 1000

!     --- Store first angle

      ibef = curvenodes(numpoints)
      inow = curvenodes(1)
      next = curvenodes(2)
      if ( debug ) then
         write(irefwr,*) 'ibef, inow, next ', ibef, inow, next
         call prinrl ( coor(1,ibef), 3, 'coor-Bef' )
         call prinrl ( coor(1,inow), 3, 'coor-now' )
         call prinrl ( coor(1,next), 3, 'coor-next' )
      end if  ! ( debug )
      call mshangle ( coor(1,ibef), coor(1,inow), coor(1,next), &
                      angles(1), ndim )

!     --- Store angles 2 to numpoints-1

      do i = 2, numpoints-1
         ibef = curvenodes(i-1)
         inow = curvenodes(i)
         next = curvenodes(i+1)
         if ( debug ) then
            write(irefwr,*) 'i, ibef, inow, next ',i, ibef, inow, next
            call prinrl ( coor(1,ibef), 3, 'coor-Bef' )
            call prinrl ( coor(1,inow), 3, 'coor-now' )
            call prinrl ( coor(1,next), 3, 'coor-next' )
         end if  ! ( debug )
         call mshangle ( coor(1,ibef), coor(1,inow), coor(1,next), &
                         angles(i), ndim )
      end do  ! i = 2, numpoints-1

!     --- Store last angle

      ibef = curvenodes(numpoints-1)
      inow = curvenodes(numpoints)
      next = curvenodes(1)
      if ( debug ) then
         write(irefwr,*) 'ibef, inow, next ', ibef, inow, next
         call prinrl ( coor(1,ibef), 3, 'coor-Bef' )
         call prinrl ( coor(1,inow), 3, 'coor-now' )
         call prinrl ( coor(1,next), 3, 'coor-next' )
      end if  ! ( debug )
      call mshangle ( coor(1,ibef), coor(1,inow), coor(1,next), &
                      angles(numpoints), ndim )

1000  call erclos ( 'mshstoreangle' )

      if ( debug ) then

!     --- Debug information

         call prinrl ( angles, numpoints, 'angles' )
         write(irefwr,*) 'End mshstoreangle'

      end if

      end
