      subroutine mshsurfaces11 ( kmeshnold, nsurfsnew, ipoint, extsurffaces )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-04-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Extend array kmeshn with extra surfaces created in case of double surfaces
!
! **********************************************************************
!
!                       KEYWORDS
!
!     face
!     mesh
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshnold(*), nsurfsnew, &
              ipoint(*), extsurffaces(*)

!     extsurffaces   i    Array of length lenextsurffac containing the
!                         face sequence numbers of the new surfaces to be
!                         created
!     ipoint         i    Array of estimated length 11*nsurfsold that is used to
!                         store the end addresses of each new surface in array
!                         extsurffaces
!                         The positions 1 to nsurfs get the value 0
!     kmeshnold      i    Old array kmeshn
!     nsurfsnew      i    number of surfaces in mesh after creating composite
!                         surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, isurnr, inpelm, ipstart, j, i, iface, &
              istartface, iielem

!     i              Counting variable
!     iface          Face sequence number
!     iielem         Relative element number in element group
!     inpelm         number of points per element
!     ipstart        Starting address in array
!     istart         First position used at a part
!     istartface     Starting address of surface in array extsurffaces (-1)
!     isurnr         Surface sequence number
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Copy old part of kmeshn to new positions
!     For all new surfaces do
!        Put all elements in kmeshn
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsurfaces11' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsurfaces11'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Copy old kmeshn to new positions

      istart = 2*nsurfsnew
      do i = 1, nsurfs
         kmeshn(i) = kmeshnold(i)                  ! number of nodes per elem.
         kmeshn(i+nsurfsnew) = kmeshnold(i+nsurfs) ! number of elements
         istart = istart+kmeshnold(i)*kmeshnold(i+nsurfs)
      end do  ! i = 1, nsurfs

      do i = 2*nsurfs+1, 2*nsurfs+istart
         kmeshn(i+2*(nsurfsnew-nsurfs)) = kmeshnold(i)
      end do

!     --- For all new surfaces do

      do isurnr = nsurfs+1, nsurfsnew

!     --- Put all elements in kmeshn, start by first part

         inelem = ipoint(isurnr)-ipoint(isurnr-1) ! number of elements
         istartface = ipoint(isurnr-1)            ! start in extsurffaces (-1)
         iface = extsurffaces(istartface+1)       ! first face number
         ipstart = nmeshg(iface)                  ! start address in nmeshg
         inpelm = nmeshg(ipstart)                 ! number of nodes per elem.
         kmeshn(isurnr) = inpelm
         kmeshn(isurnr+nsurfsnew) = inelem

!        --- Now fill faces

         do iielem = 1, inelem

            iface = extsurffaces(istartface+iielem)
            ipstart = nmeshg(iface)
            do j = 1, inpelm
               kmeshn(istart+j) = nmeshg(ipstart+j)
            end do  ! j = 1, inpelm
            istart = istart+inpelm
            if ( debug ) &
               call prinin ( nmeshg(ipstart+1), inpelm, 'face' )

         end do  ! iielem = 1, inelem

      end do  ! isurnr = nsurfs+1, nsurfsnew

      nsurfs = nsurfsnew

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshsurfaces11'

      end if
      call erclos ( 'mshsurfaces11' )

      end
