      subroutine mshsurfaces12 ( ihelp, nsurfs, nsurfsold, kmeshr, &
                                 iworksurfs, leniworksurfs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 24-11-2004 Fortran 90
!        version  1.0  date 28-04-2004
!
!   copyright (c) 2004-2004  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kmeshr for compound surfaces
!     The information stored in ihelp and iworksurfs is used
! **********************************************************************
!
!                       KEYWORDS
!
!     surface
!     compound_surface
!     fill
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ihelp(*), nsurfs, nsurfsold, kmeshr(*), &
              iworksurfs(*), leniworksurfs

!     ihelp          i    integer work array of length nsurfs
!                         If surface i is single ihelp(i) = 0
!                         If surface i is possibly compound ihelp(i) = -1
!     iworksurfs     i    Array in which the subsurfaces corresponding to a
!                         surface are stored
!     kmeshr         i    Kmesh part r, see the PG 13.1
!     leniworksurfs  i    Length of array iworksurfs
!     nsurfs         i    number of surfaces
!     nsurfsold      i    Number of surfaces in the mesh read
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iwork(nsurfs), isequence(nsurfsold), i, istart, iprev, &
              isurnr, numsurfs

!     i              Counting variable
!     iprev          Previous value used
!     isequence      Work array of length nsurfsold to store the sorted
!                    increasing sequence of array ihelp
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     iwork          integer work array of length nsurfs to store the
!                    number of subsurfaces per surface
!     numsurfs       number of surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHSORT         Sort integer array for increasing sequence
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Store number of subsurfaces in array iwork
!     For all old surfaces do
!       Fill kmeshr
!     Extend kmeshr for new surfaces
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshsurfaces12' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshsurfaces12'
         write(irefwr,*) 'nsurfs, nsurfsold ', nsurfs, nsurfsold
         call prinin ( ihelp, nsurfsold, 'ihelp' )

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- Sort array ihelp

      call chsort ( ihelp, isequence, nsurfsold )
      if ( debug ) call prinin ( isequence, nsurfsold, 'isequence' )

!     --- Store number of subsurfaces in array iwork

      do i = 1, nsurfs
         iwork(i) = 0
      end do  ! i = 1, nsurfs

      istart = 0
      iprev = isequence(1)

      do i = 1, nsurfsold
         isurnr = isequence(i)
         if ( ihelp(isurnr).gt.0 ) then

!        --- New starting address found

            if ( istart.gt.0 ) then

!           --- Next compound surface

               iwork(iprev) = ihelp(isurnr)-istart
            end if  ! ( istart.eq.0 )
            istart = ihelp(isurnr)

         end if
         iprev = isurnr

      end do  ! i = 1, nsurfsold
      iwork(isurnr) = leniworksurfs+1-istart
      if ( debug ) call prinin ( iwork, nsurfsold, 'iwork' )

!     --- For all old surfaces do

      istart = nsurfs+1
      kmeshr(1) = istart
      do isurnr = 1, nsurfsold

!     --- Fill kmeshr

         numsurfs = iwork(isurnr)
         do i = 0, numsurfs-1
            kmeshr(istart+i+1) = iworksurfs(ihelp(isurnr)+i)
         end do  ! i = 1, numsurfs
         istart = istart+numsurfs
         kmeshr(isurnr+1) = istart

      end do  ! i = 1, nsurfsold

!     --- Extend kmeshr for new surfaces

      do isurnr = nsurfsold+1, nsurfs
         kmeshr(isurnr+1) = kmeshr(isurnr)
      end do  ! isurnr = nsurfsold+1, nsurfs

1000  call erclos ( 'mshsurfaces12' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End mshsurfaces12'

      end if

      end
