      subroutine mshtransinfonw ( refedges, lenrefedges, iedges, &
                                  transportinfo, newtransinfo )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill new array transinfo from old array transinfo and array iedges
!
! **********************************************************************
!
!                       KEYWORDS
!
!     edge
!     parallel
!     node
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer  lenrefedges, refedges(lenrefedges), iedges(*), &
               transportinfo(*), newtransinfo(*)

!     iedges         i    Work array of size nedges
!                         If iedges(i)>0, edge i must be splitted and
!                         the node number will be iedges(i)+npoint
!     lenrefedges    i    length of array refedges
!     newtransinfo   o    New array transinfo
!                         In this case it contains the part for one block only
!     refedges       i    Contains the edge numbers of the edges that
!                         must be subdivided
!     transportinfo  i    Array containing information of the transport of
!                         unknowns between neighboring blocks
!                         See Programmers Guide Section 24.2 for a description
!                         In this case it contains the part for one block only
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nneigh

!     nneigh         Number of nodes in transportinfo
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Copy old transportinfo in new transinfo
!     For all edges in refedges do
!        add new number to new transinfo
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshtransinfonw' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshtransinfonw'
         call prinin ( transportinfo, transportinfo(1)+1, &
                       'transportinfo' )
         call prinin ( refedges, lenrefedges, 'refedges' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Copy old transportinfo in new transinfo

      nneigh = transportinfo(1)
      newtransinfo(1) = nneigh+lenrefedges
      newtransinfo(2:nneigh+1) = transportinfo(2:nneigh+1)

!     --- For all edges in refedges do
!           add new number to new transinfo

      newtransinfo(nneigh+2:nneigh+1+lenrefedges) = &
         iedges(refedges(1:lenrefedges))

1000  call erclos ( 'mshtransinfonw' )
      if ( debug ) then

!     --- Debug information

         call prinin ( newtransinfo, newtransinfo(1)+1, 'newtransinfo' )
         write(irefwr,*) 'End mshtransinfonw'

      end if  ! ( debug )

      end
