      subroutine mshupdate ( refine, nelem_basis, &
                             reflvl, rewind, iprint )
! ======================================================================
!
!        programmer    John Brusche
!        version  1.2  date 01-11-2005 Interface outside refinement:
!                                      condition fixed
!        version  1.1  date 15-09-2005 Rewind added
!        version  1.0  date 30-05-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Determine whether a mesh update is required or not.
!     It is also checked whether the interface moves outside of the
!     band of refinement. If this occurs, rewind is set to true and the
!     time level will be set to the previous level and the mesh refined.
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     refine
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical refine, rewind
      integer nelem_basis, reflvl, iprint

!     iprint         i    Print level, 0 is no printing
!     nelem_basis    i    Number of elements of the basis mesh
!     refine         o    If true, the mesh is to be refined
!     reflvl         i    Level of refinement
!     rewind         i    When true, the current time level is rewinded
!                         to a previous time level
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshupdate' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshupdate'
         write(irefwr,1) 'nelem_basis, reflvl', nelem_basis, reflvl
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      i = 0
      do while (.not. rewind .and. i<nelem_basis)
         i = i + 1
         if (debug) write(irefwr,*) 'i, refineold(i), refinenew(i)', &
              i, refineold(i), refinenew(i)
         if (refineold(i)/=refinenew(i)) refine = .true.
         if (refineold(i)<reflvl .and. refinenew(i)>reflvl ) then
            if ( iprint>1 ) write(irefwr,*) &
               'Interface outside refinement, element = ', i
            rewind = .true.
         end if
      end do

!     --- When rewinding, the old refinement and the new refinement will
!         be combined. Therefore, refining the mesh is required.

      if (rewind) refine = .true.

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'refine', refine
         write(irefwr,*) 'End mshupdate'

      end if
      call erclos ( 'mshupdate' )

      end
