      subroutine mshupdmeshinp2 ( iinput, iinputnew, nvolinsurfs, &
                                  sumextrasurfs, lengin, &
                                  rinput, rinputnew, lengrl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 30-07-2012 New treatment of iinput/rinput
!        version  2.0  date 02-09-2010 Remove intarmsh
!        version  1.1  date 20-10-2008 Make isurf = 24
!
!   copyright (c) 2008-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Fill array iinputnew with information of extra surfaces
!      These surfaces are combinations of internal surfaces and the
!      external surface corresponding to volumes of type 10
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     input
!     adapt_mesh
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinputnew(*), nvolinsurfs, sumextrasurfs, &
              lengin, lengrl
      double precision rinput(*), rinputnew(*)

!     iinput         i    integer input array with respect to mesh
!                         This array is considered from position 6
!     iinputnew      o    Adapted integer input array with respect to mesh
!                         This array is considered from position 6
!     lengin        i/o   Length of integer array iinput
!     lengrl        i/o   Length of real array rinput
!                         (in double precision variables)
!     nvolinsurfs    i    Number of volumes with internal surfaces
!     rinput         i    Real input array with respect to mesh
!                         This array is considered from position 6
!     rinputnew      o    Adapted double input array with respect to mesh
!                         This array is considered from position 6
!     sumextrasurfs  i    sum of number of extra surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart, jstart, kstartvol, &
              lenextra, nsurfsold, iinp

!     i              Counting variable
!     iinp           position in array iinput
!     istart         First position used at a part
!     jstart         starting position
!     kstartvol      starting position for volumes in iinput
!     lenextra       extra space needed in iinputnew
!     nsurfsold      Number of surfaces in the mesh read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSH050         Raise iinp as function of ivolm
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'mshupdmeshinp2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from mshupdmeshinp2'
         call prinin ( iinput, lengin, 'iinput' )
         call prinrl ( rinput, lengrl, 'rinput' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill constants

      ndim   = iinput(1)   ! dimension of space
      nuspnt = iinput(2)   ! number of user points
      ncurvs = iinput(3)   ! number of curves
      nsurfs = iinput(4)   ! number of surfaces
      nvolms = iinput(5)   ! number of volumes
      jcoars = iinput(11)  ! defines coarsness

!     --- Copy iinput to iinputnew in the correct positions

      iinputnew(1:iinp5-1) = iinput(1:iinp5-1)  ! part until surfaces
      lenextra = nvolinsurfs*4+sumextrasurfs    ! extra space needed
      kstartvol = iinp5+lenextra
      iinputnew(kstartvol:lengin+lenextra) = iinput(iinp5:lengin)  ! end part

      iinputnew(4) = iinputnew(4)+nvolinsurfs  ! new number of surfaces
      if ( iinputnew(9)<0 ) iinputnew(9) = iinputnew(9)-lenextra
      if ( iinputnew(15)>0 ) iinputnew(15) = iinputnew(15)+lenextra

!     --- For all surfaces do

      istart = iinp5-1   ! starting address for new surfaces
      iinp = iinp5
      do i = 1, nvolms

!     --- Find type of volume

         ivol = iinput(iinp)
         if ( debug ) write(irefwr,1) 'i, ivol', i, ivol
         if ( ivol==10 ) then

!        --- Volume with possibly internal surfaces found

            nextrasrfs = iinput(iinp+6)
            if ( debug ) write(irefwr,1) 'nextrasrfs', nextrasrfs

            if ( nextrasrfs>0 ) then

!           --- Internal surfaces found, fill iinputsurfextr for this surface

               iinputnew(istart+1) =24  ! surface of surfaces
               iinputnew(istart+2) = 0  ! ishape
               iinputnew(istart+3) = nextrasrfs+1  ! number of subsurfaces
               iinputnew(istart+4) = iinput(iinp+3) ! outer surface
               istart = istart+4  ! raise istart partly

!              --- Compute starting address of internal surfaces in iinput
!                  This is the position of the outer surface + 3 + &
!                  number of internal points and curves
!                  Decrease by minus 1 for simplicity

               jstart = iinp+6+iinput(iinp+4)+iinput(iinp+5) ! start
               iinputnew(istart+1:istart+nextrasrfs) = &
                  iinput(jstart+1:jstart+nextrasrfs) ! internal surfaces
               istart = istart+nextrasrfs  ! raise istart finally

               nsurfs = nsurfs+1   ! raise nsurf

!              --- The old surface number is replaced by the new one

               iinputnew(iinp+lenextra+3) = nsurfs

            end if  ! ( nextrasrfs>0 )

         end if  ! ( ivol==10 )

!        --- Raise iinp

         call msh050 ( iinput, iinp, ivol )

      end do
      length = 2+ndim*nuspnt
      nsurfsold = iinput(4)
      if ( jcoars==1 ) length = length+1+nuspnt+3*ncurvs+4*nsurfsold
      rinputnew(1:length) = rinput(1:length)
      rinputnew(length+1:length+4*nvolinsurfs) = 0
      rinputnew(length+4*nvolinsurfs+1:lengrl+4*nvolinsurfs) = &
         rinput(length+1:lengrl)

!     --- New length of arrays

      lengin = lengin+lenextra
      lengrl = lengrl+4*nvolinsurfs

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iinputnew, lengin, 'iinputnew' )
         call prinrl ( rinputnew, lengrl, 'rinputnew' )
         write(irefwr,*) 'End mshupdmeshinp2'

      end if  ! ( debug )
      call erclos ( 'mshupdmeshinp2' )

      end
