      subroutine seplevcross2d ( iseqlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill arrays icrossobst and rcrossobst (2d and 3d)
!     Old subroutine mshlevcross
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     element
!     node
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmodulekmesh
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iseqlc

!     iseqlc           o    Sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lenicrossobst, lenrcrossobst, iobst

!     iobst          Obstacle sequence number
!     lenicrossobst  Computed length of array icrossobst
!     lenrcrossobst  Computed length of array rcrossobst
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPLEVELCREL2D Compute all information concerning kmesh ad parts 3 and 4, &
!                    i.e. information of the cross sections of elements with
!                    zero level set
!     SEPLEVELCREL3D Compute all information concerning kmesh ad parts 3 and 4, &
!                    i.e. information of the cross sections of elements with
!                    level set function 0 and store in arrays icrossobst
!                    and rcrossobst
!                    3d version
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     419   NDIM incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seplevcross2d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seplevcross2d'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000


!     --- define work space to store rcrossobst and icrossobst

      allocate ( icrossobst(3*nedges+2*numobst), rcrossobst(nedges), &
                 stat = error )
      if ( error/=0 ) &
         call eralloc ( error, 4*nedges+2*numobst, 'icrossobst' )
      if ( ierror/=0 ) go to 1000

      lenicrossobst = 0
      lenrcrossobst = 0
      iobst = 1
      usoltemp => ks(isolut)%sol

      select case ( ndim )

      case(1,2)

!     --- ndim = 2

         call seplevelcrel2d ( usoltemp, &
                               lenicrossobst, lenrcrossobst, iobst )

      case(3)

!     --- ndim = 3

         call seplevelcrel3d ( usoltemp, &
                               lenicrossobst, lenrcrossobst, iobst )

      case default

!     --- ndim not implemented

         call errint ( ndim, 1 )
         call errint ( 2, 2 )
         call errint ( 3, 3 )
         call errsub ( 419, 3, 0, 0 )
         go to 1000

      end select  ! case ( ndim )

!     --- Make the arrays rcrossobst, icrossobst permanent

      if ( lenrcrossobst>0 ) then

!     --- Array may be created

         length = lenrcrossobst
         if ( associated(km(iseqkm)%rcrossobst) ) &
            deallocate(km(iseqkm)%rcrossobst)
         allocate ( km(iseqkm)%rcrossobst(length), stat = error )
         if ( error/=0 ) call eralloc ( error, length, 'rcrossobst' )
         if ( ierror/=0 ) go to 1000

         km(iseqkm)%rcrossobst = rcrossobst(1:length)
         deallocate ( rcrossobst )

         rcrossobst => km(iseqkm)%rcrossobst
         kmeshad(4) = length

      else

!     --- No intersections found, delete previous array if it exists

         if ( associated(km(iseqkm)%rcrossobst) ) &
            deallocate(km(iseqkm)%rcrossobst)
         nullify ( rcrossobst )
         kmeshad(4) = 0

      end if

      length = lenicrossobst
      allocate ( km(iseqkm)%icrossobst(length), stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'icrossobst' )
      if ( ierror/=0 ) go to 1000

      km(iseqkm)%icrossobst = icrossobst(1:length)
      deallocate ( icrossobst )

      icrossobst => km(iseqkm)%icrossobst
      kmeshad(3) = length

      kmeshad(8) = iseqlc

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seplevcross2d'

      end if  ! ( debug )
      call erclos ( 'seplevcross2d' )

      end subroutine seplevcross2d
