      function volmesh()
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-12-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the volume of a region
!
! **********************************************************************
!
!                       KEYWORDS
!
!     integral
!     volume
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision volmesh

!     volmesh        o    Volume of mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, ielgrp, ikelmc
      double precision volume, valmin, valmax, value

!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     valmax         Maximum value of computed integral
!     valmin         Minimum value of computed integral
!     value          volume of one element
!     volume         Volume of mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHSUBVOL      Compute volume of submesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     volume := 0
!     For all element groups of highest dimension do
!        Compute integral over element group
!        Add to volume
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'volmesh' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from volmesh'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ikelmc = 1

!     --- volume := 0

      volume = 0d0

!     --- For all element groups of highest dimension do

      do ielgrp = 1, nelgrp

!     --- Compute integral over element group

         inpelm = Topgrp(ielgrp)%inpelm
         ishape = kmeshf(ielgrp)
         inelem = Topgrp(ielgrp)%inelem
         if ( debug ) &
            write(irefwr,1) 'ielgrp, inpelm, ishape, inelem', &
                             ielgrp, inpelm, ishape, inelem
         if ( ndim==2 .and. ishape>2 .or. ndim==3 .and. ishape>10 ) then

!        --- Volume can be computed

            call mshsubvol ( kmeshc(ikelmc), inelem, ishape, inpelm, &
                             ielgrp, coor, ndim, valmin, &
                             valmax, value, npoint )

!           --- Add to volume

            volume = volume+value
            if ( debug ) write(irefwr,2) 'value', value

         end if  ! ( ndim==2 .and. ishape<3 .or. ndim==3 .and. ishape<11 )

         ikelmc = ikelmc+inpelm*inelem

      end do  ! ielgrp = 1, nelgrp

      volmesh = volume

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End volmesh'

      end if  ! ( debug )
      call erclos ( 'volmesh' )

      end function volmesh
