      subroutine er0029 ( inppri, inodes, nnodes, icurvs, text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.10 date 20-06-2013 Remove cyclop stuff
!        version  4.9  date 14-07-2011 Extra debug
!        version  4.8  date 09-06-2010 Change of layout
!
!   copyright (c) 1985-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     print values of vector in nodal points (real case)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inppri(*), nnodes, inodes(nnodes), icurvs(*)
      character (len=*) text

!     icurvs  i   Integer array containing the curve or surface numbers for the
!                 that must be printed.
!                 Pos. 1 indicates how array icurvs is filled.
!                 Possible values:
!                  0   only one curve or surface is used. The curve of
!                      surface number is stored in icurvs(2)
!                 -1   The solution is printed along the curves or surfaces
!                      icrv1 to icrv2. icrv1 is stored in icurvs(2) and
!                      icrv2 in icurvs(3).
!                 -2   See -1, however, the sequence of the printing is
!                      reversed. In case of curves this means that the
!                      curves are printed from the end point of icrv2 to
!                      the initial point of icrv2, then the end point of
!                      icrv2-1 to the begin point of icrv2-1 and so on.
!                      In case of surfaces ichoice=-2 is treated as ichoice=-1
!                 n>0  The curve numbers or surface numbers are stored in
!                      icurvs(2) to icurvs(n+1). That sequence is used.
!                 Remark: a negative curve number means that the curve is
!                      printed from end point to initial point, hence in
!                      reversed order. Negative surface numbers are not yet
!                      available.
!     inodes  i   Array containing the nodal point numbers at the boundary
!     inppri  i   Integer array in which the user indicates which quantities
!                 must be printed by subroutine PRINBN
!                 The entries of INPPRI must be filled as follows
!                 1:  Indicates calling subroutine:
!                     1: PRINVC
!                     2: PRINCR
!                 2:  ichoice, indicates the type of print
!                     Choice parameter consisting of 3 parts:
!                     ichoice_print, isuppress and suppress_zeros according to
!                     ichoice = ichoice_print+10*isuppress+100*suppress_zeros
!                     with:
!                     ichoice_print
!                     0:   The nodal point number is printed followed by the
!                          values in the nodal point
!                     1:   The nodal point number is printed followed by the
!                          coordinates and then the values in the nodal point
!                     2:   The nodal point number is printed followed by the
!                          coordinates
!                     3:   See 2, however, preceded by the number of nodal
!                          points to be printed
!                     isuppress:
!                     if 1 the nodal point number is suppressed if 0 not
!                     suppress_zeros
!                     if 1 points in which the solution is zero are skipped, &
!                     if 0 not
!                 3:  IDIM, indicates if user points (idim=1), curves (idim=2)
!                     or surfaces (idim=3) must be printed.
!                     The user point, curve or surface numbers must be stored
!                     in array ICURVS
!                 4:  IQUANTITY, indicates which quantity must be printed
!                     Possible values:
!                     0:  The contents of the vector are printed as stored in
!                         the vector corresponding to IVECTR i.e. without
!                         transformation.
!                     1:  The normal component of a vector along the curve
!                         or surface is printed.
!                     2:  The tangential component(s) of a vector along the
!                         curve or surface is printed.
!                 5:  NUMBER, indicates which degree of freedom in each point
!                     must be printed. This parameter must be seen in connection
!                     with IQUANTITY.
!                     Possible values:
!                     0:  If iquantity=0, all degrees of freedom are printed
!                         in the nodes defined by icurvs, &
!                         if iquantity>0, the vector from which the normal or
!                         tangential components must be computed is supposed
!                         to be a n-dimensional vector with first degree of
!                         freedom 1, second degree of freedom 2 and third
!                         degree of freedom 3. In this case it is necessary
!                         that IDIM = NDIM (dimension of space)
!                     i>0: If iquantity=0, the degree of freedom i is printed
!                         if iquantity>0, the vector from which the normal or
!                         tangential components must be computed is supposed
!                         to be a n-dimensional vector with first degree of
!                         freedom i, second degree of freedom i+1 and third
!                         degree of freedom i+2. In this case it is necessary
!                         that IDIM = NDIM (dimension of space)
!                 6:  ISEQ, indicates the sequence in which the coordinates
!                     along the surface must be ordered in case idim=3
!                     Possible values:
!                     1:  increasing x, for fixed x increasing y, &
!                         for fixed y increasing z
!                     2:  increasing y, for fixed y increasing z, &
!                         for fixed z increasing x
!                     3:  increasing z, for fixed z increasing x, &
!                         for fixed x increasing y
!                 7:  IREGION, indicates if the complete region must be printed
!                     (0) or that the printing is restricted to a sub region, &
!                     defined by the parameters xmin, xmax, ymin, ymax, zmin, &
!                     zmax in the common block CPLOT.
!                     Possible values:
!                     0:  The complete region is printed
!                     1:  The region is restricted to xmin<=x<=xmax, &
!                         ymin<=y<=ymax, zmin<=z<=zmax
!                    -1:  The region is restricted to x=xmin
!                    -2:  The region is restricted to y=ymin
!                    -3:  The region is restricted to z=zmin
!                    -11: The region is restricted to x>=xmin
!                    -12: The region is restricted to y>=ymin
!                    -13: The region is restricted to z>=zmin
!                    -21: The region is restricted to x<=xmin
!                    -22: The region is restricted to y<=ymin
!                    -23: The region is restricted to z<=zmin
!                 8:  IHEADER, indicates if the header must be printed (0) or
!                     not (1)
!                 9:  ICOOR, type of coordinate system
!                     1: Cartesian (x,y)
!                     3: Polar (r, phi)
!                 10: IFORMAT  Format used for printing
!                     1: e13.5
!                     2: e16.8
!                     3: e13.5, one node per line
!                     4: e16.8, one node per line
!     nnodes      Number of nodes in inodes
!     text    i   String variable containing some text
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ichoice, idim, iquant, number, iformat
      logical heading

!     heading        If true a heading must be printed
!     i              Counting variable
!     ichoice        Choice parameter
!     idim           dimension of the space
!     iformat        Format used for printing
!                    1: e13.5
!                    2: e16.8
!     iquant         See array inppri
!     number         See array inppri
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRSW09         Prints the actual array
!     PRSW093D       Prints real arrays in 3D, tangential or normal vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Extract constants from inppri
!    Write heading
!    if ( idim=2 ) then
!       print values along curve
!    else if ( idim=3 ) then
!       sort values along surface
!       print values along surface
!    else if ( idim=1 ) then
!       print values along user points
!    else ( idim=0 )
!       sort values
!       print values
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'er0029' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from er0029'
         call prinin ( inppri, 10, 'inppri' )
         call prinin ( inodes, nnodes, 'inodes' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Extract constants from inppri

      ichoice = inppri(2)
      idim    = inppri(3)
      iquant  = inppri(4)
      number  = inppri(5)
      heading = inppri(8)==0
      iformat = inppri(10)

!     --- Write heading

      if ( heading ) then
         write(irefwr,100) text
100      format(/' vector ',a/)
      else
         write(irefwr,*)
      end if

      if ( idim==2 ) then

!     --- idim=2, print values along curve

         if ( icurvs(1)==0 ) then
            if ( heading ) write(irefwr,110) icurvs(2)
110         format(/' Curve number ', i5/)
         else if ( icurvs(1)==-1 ) then
            if ( heading ) write(irefwr,120) icurvs(2), icurvs(3)
120         format(/' Curve numbers ', i5,'  to ',i5/)
         else if ( icurvs(1)==-2 ) then
            if ( heading ) write(irefwr,120) icurvs(3), icurvs(2)
         else if ( heading ) then
            write(irefwr,130) (icurvs(i), i=2,icurvs(1)+1)
130         format(/' Curve numbers ', 10i5/(15x,10i5)/)
         end if

         if ( iquant==1 .and. ndim==2 ) then

!        --- iquant=1, compute normal component and store in temp

            allocate ( usoltemp(nnodes) )
            do i = 1, nnodes
               usoltemp(i) = normalvec(1,i)*unew(1+(i-1)*2) + &
                             normalvec(2,i)*unew(2+(i-1)*2)
            end do

         else if ( iquant==2 .and. ndim==2 ) then

!        --- iquant=2, compute tangential component and store in temp

            allocate ( usoltemp(nnodes) )
            do i = 1, nnodes
               usoltemp(i) = tang(1,i)*unew(1+(i-1)*2) + &
                             tang(2,i)*unew(2+(i-1)*2)
            end do

         end if
         if ( iquant==0 ) then

!        --- Standard case

            usoltemp => unew
            call prsw09 ( ichoice, nnodes, inodes, number, &
                          heading, inppri(9), iformat )

         else if ( ndim==2 ) then

!        --- iquant = 1, 2, print normal or tangential component (R^2)

            if ( iquant==1 ) then
               if ( heading ) write(irefwr,260) 'normal components'
260            format(/a/)
            else
               if ( heading ) write(irefwr,260) 'tangential components'
            end if
            call prsw09 ( ichoice, nnodes, inodes, 1, &
                          heading, inppri(9), iformat )
            deallocate ( usoltemp )

         else

!        --- iquant = 1, 2, print normal vector or tangential component (R^3)

            call prsw093d ( nnodes, inodes, max(1,number), nunkpo, unew, &
                            coor, heading, inppri, tang )

         end if

      else if ( idim==3 ) then

!     --- idim=3, print values along surface

         if ( icurvs(1)==0 ) then
            if ( heading ) write(irefwr,510) icurvs(2)
510         format(/' Surface number ', i5/)
         else if ( icurvs(1)<0 ) then
            if ( heading ) write(irefwr,520) icurvs(2), icurvs(3)
520         format(/' Surface numbers ', i5,'  to ',i5/)
         else if ( heading ) then
            write(irefwr,530) (icurvs(i), i=2,icurvs(1)+1)
530         format(/' Surface numbers ', 10i5/(15x,10i5)/)
         end if

         if ( iquant==1 ) then

!        --- iquant=1, compute normal component and store in temp

            allocate ( usoltemp(nnodes) )
            do i = 1, nnodes
               usoltemp(i) = normalvec(1,i)*unew(1+(i-1)*3) + &
                             normalvec(2,i)*unew(2+(i-1)*3) + &
                             normalvec(3,i)*unew(3+(i-1)*3)
            end do

         else if ( iquant==2 ) then

!        --- iquant=2, compute tangential component and store in temp

            allocate ( usoltemp(nnodes) )
            do i = 1, nnodes
               usoltemp(i) = tang(1,i)*unew(1+(i-1)*3) + &
                             tang(2,i)*unew(2+(i-1)*3) + &
                             tang(3,i)*unew(3+(i-1)*3)
            end do

         end if

         if ( iquant==0 ) then

!        --- Standard case

            usoltemp => unew
            call prsw09 ( ichoice, nnodes, inodes, number, &
                          heading, inppri(9), iformat )

         else

!        --- iquant = 1, 2, print normal or tangential component

            if ( iquant==1 ) then
               if ( heading ) write(irefwr,260) 'normal components'
            else if ( heading ) then
               write(irefwr,260) 'tangential components'
            end if
            call prsw09 ( ichoice, nnodes, inodes, 1, &
                          heading, inppri(9), iformat )
            deallocate ( usoltemp )

         end if

      else if ( idim==4 ) then

!     --- idim=4, print values along Volume

         if ( icurvs(1)==0 ) then
            if ( heading ) write(irefwr,610) icurvs(2)
610         format(/' Volume number ', i5/)
         else if ( icurvs(1)<0 ) then
            if ( heading ) write(irefwr,620) icurvs(2), icurvs(3)
620         format(/' Volume numbers ', i5,'  to ',i5/)
         else if ( heading ) then
            write(irefwr,630) (icurvs(i), i=2,icurvs(1)+1)
630         format(/' Volume numbers ', 10i5/(15x,10i5)/)
         end if
         usoltemp => unew
         call prsw09 ( ichoice, nnodes, inodes, number, &
                       heading, inppri(9), iformat )

      else if ( idim==1 ) then

!     --- idim=1, print values in user points

         if ( icurvs(1)==0 ) then
            if ( heading ) write(irefwr,810) icurvs(2)
810         format(/' User point number ', i5/)
         else if ( icurvs(1)<0 ) then
            if ( heading ) write(irefwr,820) icurvs(2), icurvs(3)
820         format(/' User point numbers ', i5,'  to ',i5/)
         else if ( heading ) then
            write(irefwr,830) (icurvs(i), i=2,icurvs(1)+1)
830         format(/' User point numbers ', 10i5/(15x,10i5)/)
         end if

            usoltemp => unew
            call prsw09 ( ichoice, nnodes, inodes, number, &
                          heading, inppri(9), iformat )

      else

!     --- idim = 0, print complete region

         if ( debug ) write(irefwr,*) 'before prsw09'
         usoltemp => unew
         call prsw09 ( ichoice, nnodes, inodes, number, &
                       heading, inppri(9), iformat )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End er0029'

      end if  ! ( debug )
      call erclos ( 'er0029' )

      end
