      subroutine er0039 ( icurvs, n, number, vector, funcx, funcy )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 30-11-2012 Long integers
!        version  1.3  date 27-11-1992 Removal of superfluous parameters
!        version  1.2  date 22-08-1991 Zdenek: no dimension, cdc, ce
!
!   copyright (c) 1986-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute values of vector in nodal points of curves (real case)
!     values are stored in funcy, co-ordinates in funcx
!     special case:  number<0
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icurvs(*), n, number
      double precision vector(*), funcx(*), funcy(*)

!     funcx          o    in this array the co-ordinates of the points are
!                         stored
!     funcy          o    in this array the values in the points are stored
!                         of the points are stored
!     icurvs         i    array containing the nodal point numbers of the curves
!     n             i/o   at input: number of nodal points in icurvs
!                         at output: number of nodal points in icurvs where
!                         degrees of freedom are present
!     number         i    only number<0 is permitted in this subroutine
!                         possibilities:
!                         -1:  compute  u . n
!                         -2:  compute  u . t  (2D only)
!                         -3:  compute  n . sigma . n  (2D only)
!                         -4:  compute  n . sigma . t  (2D only)
!                         at this moment only ndim = 2,real case is permitted
!     vector         i    array containing the vector from which the values
!                         must be computed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ih, iknpt, inunkp, inusol, itel, m
      double precision dx, dy, f, h, t1, t2, u1, u2, u3, v1, v2, x, &
                       xnext, xprec, y, ynext, yprec

!     dx
!     dy
!     f
!     h
!     i
!     ih
!     iknpt
!     inunkp
!     inusol
!     itel
!     m
!     t1
!     t2
!     u1
!     u2
!     u3
!     v1
!     v2
!     x
!     xnext
!     xprec
!     y
!     ynext
!     yprec
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      inunkp=nphysi
      itel=0
      m=2
      if(number<-2) m=3
      do i=1,n
         iknpt=icurvs(i)
         if(indprfi==0) then
            inusol=(iknpt-1)*nphysi
         else
            inusol=kprobfi(iknpt)
            inunkp=kprobfi(iknpt+1)-inusol
         end if
         if(i==1) then
            x=coor(1,iknpt)
            y=coor(2,iknpt)
            xprec=x
            yprec=y
         else
            x=xnext
            y=ynext
         end if
         if(i==n) then
            xnext=x
            ynext=y
         else
            ih=icurvs(i+1)
            xnext=coor(1,ih)
            ynext=coor(2,ih)
         end if
         if(inunkp>=m) then
            u1=vector(inusol+1)
            u2=vector(inusol+2)
            if(m==3) u3=vector(inusol+3)
            funcx(itel*ndim+1)=x
            funcx(itel*ndim+2)=y
            itel=itel+1
            dx=xnext-xprec
            dy=ynext-yprec
            h=1d0/sqrt(dx*dx+dy*dy)
            t1=dx*h
            t2=dy*h
            goto ( 110, 120, 130, 130 ), -number

!           --- number = -1   u . n

110            f=u1*t2-u2*t1
               goto 200

!              --- number = -2   u . t

120            f=u1*t1+u2*t2
               goto 200

!              --- number = -3,-4  n . sigma .

130            v1=u1*t2-u2*t1
               v2=u2*t2-u3*t1
               if(number==-3) then
                  f=v1*t2-v2*t1
               else
                  f=v1*t1+v2*t2
               end if
200         funcy(itel)=f
         end if
         xprec=x
         yprec=y
      end do
      n=itel
      end
