      subroutine er0078 ( m, number, iwork, n, usol, usolbn )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 17-06-1991
!
!   copyright (c) 1991-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array usolbn with the solution corresponding to the points at
!     the boundary as stored in iwork
!     Complex equivalent of er0059
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer m, number, iwork(*), n
      double complex usol(*),usolbn(*)

!     iwork          i    array containing the nodal points of the boundary
!     m              i    number of unknowns per point that must be stored in
!                         usolbn
!     n              i    number of points in iwork
!     number         i    the number th  unknown in each nodal point of usol
!                         is used
!     usol           i    Solution vector
!     usolbn         o    array containing the values from usol at the boundary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, iknpt, inusol

!     i              Counting variable
!     iknpt          Nodal point number
!     inusol         Position of unknown in array USOL
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if(indprpi/=0) then

!     --- indprpi>0 hence array kprob part p must be used

         do i=1,n
            iknpt=iwork(i)
            do j=1,m
               usolbn((i-1)*m+j)=usol(kprobpi(iknpt,number+j-1))
            end do
         end do
      else if (indprfi/=0) then

!     --- indprpi=0, whereas indprfi#0
!         Not in all points the same number of unknowns

         do i=1,n
            iknpt=iwork(i)
            inusol=kprobfi(iknpt)+number-1
            do j=1,m
               usolbn((i-1)*m+j)=usol(inusol+j)
            end do
         end do
      else

!     --- indprfi=0 and indprpi=0
!         No renumbering, constant number of degrees of freedom per point

         do i=1,n
            iknpt=iwork(i)
            inusol=(iknpt-1)*nphysi+number-1
            do j=1,m
               usolbn((i-1)*m+j)=usol(inusol+j)
            end do
         end do
      end if
      end
