      subroutine erfillbck119 ( isol, numsol, numsubcurvs, inpvec, numnodes, &
                                ientry )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-05-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array isol of type 119 with the contents of array inpvec
!
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: numsol, numsubcurvs, numnodes, ientry
      integer, intent(inout) :: isol(numsol)
      double precision, intent(in) :: inpvec(numnodes,numsol+1)

!     ientry         i    The last entry of work that is used
!     inpvec         i    Contains the input vector
!                         (.,1) the x-coordinates
!                         (.,2..) the values
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution arrays
!     numnodes       i    Number of nodal points in array inpvec
!     numsol         i    Total number of solution vectors in isol
!     numsubcurvs    i    Number of subcurves
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, istart, isubcv, n, ipos
      integer, pointer :: infsubc(:)

!     i              Counting variable
!     infsubc        Contains number of nodes per segment
!     ipos           Starting position in array usol
!     istart         First position used at a part
!     isubcv         Subcurve sequence number
!     n              number of nodes in subcurve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ1        print 2d integer array
!     PRINRL1        Print 2d real vector
!     SEPCRSOLINFO   Create a new vector in ks
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3186   More than 1 subcurve
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfillbck119' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillbck119'
         call prinin1 ( isol, numsol, 5, 'isol' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( numsubcurvs>1 ) then

!     --- Error not yet implemented

         call errint ( numsubcurvs, 1 )
         call errsub ( 3186, 1, 0, 0 )
         go to 1000

      end if  ! ( numsubcurvs>1 )

      do i = 1, numsol
         call sepcrsolinfo ( isol(i), 2*ientry, numsubcurvs )
      end do  ! i = 1, numsol
      if ( ierror/=0 ) go to 1000

!     --- Loop over vectors

      do i = 1, numsol

         infsubc => ks(isol(i))%isol
         usol => ks(isol(i))%sol

!        --- Loop over subcurves

         istart = 1
         ipos = 1
         do isubcv = 1, numsubcurvs

            n = ientry
            infsubc(isubcv) = n
            if ( i==1 ) usol(1:n) = inpvec(istart:istart+n-1,1) ! coordinates
            ipos = ipos+n  ! raise ipos

!           --- Copy function values

            usol(ipos:ipos+n-1)  = inpvec(istart:istart+n-1,i+1)
            ipos = ipos+n  ! raise ipos
            istart = istart+n

         end do  ! isubcv = 1, numsubcurvs

      end do  ! i = 1, numsol

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( inpvec, numsol+1, numnodes, 'inpvec' )
         write(irefwr,*) 'End erfillbck119'

      end if  ! ( debug )
      call erclos ( 'erfillbck119' )

      end subroutine erfillbck119
