      subroutine ersolselect ( usol, iwork, npoint, indprf, kprobf, &
                               number, ncomp, uloc, nunkp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 16-01-2009 Extra debug
!        version  2.0  date 05-11-2008 Remove kprobh
!        version  1.0  date 13-03-2003
!
!   copyright (c) 2003-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Extract ncomp components from array usol in each point stored in iwork
!     and put these components in array uloc
! **********************************************************************
!
!                       KEYWORDS
!
!     extract
!     component
!     node
!     solution_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), npoint, indprf, kprobf(*), &
              number, ncomp, nunkp
      double precision usol(*), uloc(npoint,ncomp)

!     indprf         i    indication if kprob part f has been filled (>0)
!                         or not (0)
!     iwork          i    integer work array of size npoint containing the
!                         nodes of the set of which the solution must be
!                         selected
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     ncomp          i    Number of components per point required
!     npoint         i    Number of points in the set stored in iwork
!     number         i    The number-th unknown in each nodal point of the array
!                         is considered. If the array is complex number=2*i-1
!                         corresponds to the real part of array i and
!                         number=2*i to the imaginary part of array i
!     nunkp          i    Maximum number of unknowns in a point
!     uloc           o    Array of size npoint*ncomp containing the
!                         solution in the points of the set in iwork
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inodp, inunkp, inusol, k

!     i              Counting variable
!     inodp          Nodal point number
!     inunkp         Number of degrees of freedom in isol
!     inusol         Position of unknown in array USOL
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ersolselect' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ersolselect'
         call prinin ( iwork, npoint, 'iwork' )
         write(irefwr,1) 'npoint, nunkp, ncomp, indprf', &
                          npoint, nunkp, ncomp, indprf
  1      format ( a, 1x, (10i6) )

      end if
      inunkp = nunkp
      if ( ncomp==1 ) then

!     --- ncomp = 1

         if ( indprf==0 ) then

!        --- indprf = 0, constant number of degrees of freedom

            do i = 1, npoint
               inodp = iwork(i)
               inusol = (inodp-1)*inunkp+number
               uloc(i,1) = usol(inusol)
            end do

         else

!        --- indprf>0, variable number of degrees of freedom

            do i = 1, npoint
               inodp = iwork(i)
               inusol = kprobf(inodp)+number
               uloc(i,1) = usol(inusol)
            end do

         end if  ! ( indprf==0 )

      else

!     --- ncomp>1

         if ( indprf==0 ) then

!        --- indprf = 0, constant number of degrees of freedom

            do i = 1, npoint
               inodp = iwork(i)
               inusol = (inodp-1)*inunkp+number
               do k = 1, ncomp
                  uloc(i,k) = usol(inusol+k-1)
               end do
            end do

         else

!        --- indprf>0, variable number of degrees of freedom

            do i = 1, npoint
               inodp = iwork(i)
               do k = 1, ncomp
                  inusol = kprobf(inodp)+number+k-1
                  uloc(i,k) = usol(inusol)
               end do
            end do

         end if  ! ( indprf==0 )

      end if  ! ( ncomp==1 )

      call erclos ( 'ersolselect' )
      if ( debug ) write(irefwr,*) 'End ersolselect'

      end
