      subroutine incmrr ( n, intdi1, intmt1, amat1, amat2, ishift, eps, &
                          text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-04-1991
!
!   copyright (c) 1991-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       incmrr compares the matrices AMAT1 and AMAT2, &
!       and prints the difference between the two, if any
!       The definition of the matrix is given by the arrays INTMT1 and INTDI1
!       Restriction:  The matrix must have method number 9, or must
!                     correspond to essential boundary conditions
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, intdi1(n+1), intmt1(*), ishift
      double precision amat1(*), amat2(*), eps
      character *(*) text

!     amat1   i   First matrix to be compared. The storage is defined by
!                 INTDI1 and INTMT1
!     amat2   i   Second matrix to be compared. The storage is defined by
!                 INTDI1 and INTMT1
!     eps     i   All differences between the matrix that are larger than
!                 eps are printed
!     intdi1  i   First part of INTMT1. Gives number of columns per row
!     intmt1  i   Second part of INTMT1. Gives columns numbers per row
!     ishift  i   Shift for printing of row number. So the actual row number
!                 printed is i+ishift
!     n       i   Number of rows in INTMT1/2
!     text    i   Text to be printed in case of a difference
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical headin
      integer i, j, icol, ncol, isum
      double precision elmat1, elmat2

!    elmat1    Element of matrix 1
!    elmat2    Element of matrix 2
!    headin    Indicates if a heading must be printed (true) or not (false)
!    i         General loop variable
!    icol      Column number in INTMT1
!    isum      Counting variable
!    j         General loop variable
!    ncol      Number of columns in a row
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      headin = .true.
      isum = 0
      do 200 i = 1, n
         ncol = intdi1(i+1) - intdi1(i)
         do 150 j = 1, ncol
            icol = intmt1(intdi1(i)+j)
            isum = isum + 1
            elmat1 = amat1(isum)
            elmat2 = amat2(isum)
            if ( abs(elmat1-elmat2).gt.eps ) then
               if ( headin ) then
                  write(irefwr,100) text
100               format(//' Differences found in'/1x,a//)
                  headin = .false.
               end if
               write(irefwr,120) i+ishift, j, icol, abs(elmat1-elmat2)
120            format('Row', i6,'  Column ',i6, &
                      '  difference in matrix:',ss, 1pe12.4e2 )
            end if
150      continue
200   continue
      end
