      subroutine incpmt ( n, intdi1, intmt1, intdi2, intmt2, metod1, &
                          amat1, amat2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 09-03-2013 Correction declaration
!        version  2.0  date 07-02-1992 New parameter list, metod1, intmt2
!        version  1.0  date 07-04-1991
!
!   copyright (c) 1991-2013  "Ingenieursbureau  SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       incpmt is used to copy the contents of an array of type
!       MATR with one data storage type into another one
!       Restriction: the following mappings are allowed:
!                    method 5 ==> method 9
!                    method 1 ==> method 5
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, intdi1(n+1), intmt1(*), intdi2(n+1), metod1, &
              intmt2(*)
      double precision amat1(*), amat2(*)

!     amat1   i   Lower triangle of matrix to be copied
!                 If metod1 = 1, this part is not used
!     amat2   o   Lower and upper triangle of copied matrix
!     intdi1  i   Array to be transformed (INTMAT part 1, first part,
!                 method = 5 )
!     intdi2  i   Transformed array (INTMAT part 1, first part, method = 6 )
!     intmt1  i   (INTMAT1 part 1, second part, method = 5 )
!     intmt2  i   (INTMAT2 part 1, second part, method = 5 )
!     metod1  i   Storage method for first matrix
!     n       i   Number of degrees of freedom corresponding to intmt1/2
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icol, j, k, icount, iwork(n)

!    icol     Column number
!    icount   Counter
!     iwork   Work array of length n
!    j        General loop variable
!    k        General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    If ( METOD1 = 5 ) then
!       Fill IWORK for each row with number of columns in under triangle by
!       copying information from INTMT1:
!               IWORK(k) := INTDI1(k+1)-INTDI1(k) (k=1,n)
!       Copy contents of DIAG1 into DIAG2
!       Copy contents of lower triangle of AMAT1 into AMAT2
!         For k := 2, n do
!            For j := 1, INTDI1(k+1) - INTDI1(k)
!               AMAT2 ( INTDI2(k)+j ) := AMAT1 ( INTDI1(k)+j)
!      Fill upper triangle of matrix AMAT2:
!         For k := 2, n do
!            For j := 1, INTDI1(k+1) - INTDI1(k)
!               column number := INTMT1( INTDI1(k)+j )
!               IWORK(column number) := IWORK(column number)  + 1
!               AMAT2( INTDI2(column number)+IWORK(column number) ) := &
!                      AMAT1(INTDI1(k)+j)
!    else ( METOD1 = 1 )
!       counter := 0
!       Copy diagonal of AMAT1 into AMAT2:
!          DIAG2(k) := AMAT1(INTDI1(k))
!       For k := 2 (1) n do
!          For j := 1 (1) INTDI2(k+1) - INTDI2(k) do
!              column_number := INTMT2( INTDI2(k) + j )
!              counter := counter+1
!              AMAT2(counter) := AMAT1( INTDI1(k) - k + column_number)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( metod1==5 ) then

!     --- metod1 = 5, map method 5 to method 9
!         Fill IWORK for each row with number of columns in under triangle

         iwork(1) = 0
         do k = 1, n
            iwork(k) = intdi1(k+1)-intdi1(k)
         end do

!        --- Fill upper triangle of matrix AMAT2:

         do k = 2, n
            do j = 1, intdi1(k+1)-intdi1(k)-1
               icol = intmt1(intdi1(k)+j)
               iwork(icol) = iwork(icol) + 1
               amat2( intdi2(icol)+iwork(icol) ) = amat1(intdi1(k)+j)
            end do
         end do

      else

!     --- metod1 = 1, copy method1 to method 5
!         copy matrix

         icount = 0
         do k = 1, n
            do j = 1, intdi2(k+1) - intdi2(k)
               icol = intmt2(intdi2(k)+j)
               icount = icount + 1
               amat2(icount) = amat1( intdi1(k)-k+icol )
            end do
         end do
      end if
      end
