      subroutine intcopcompl00 ( iunk, usol, usolcop, indprp, kprobf, &
                                 kprobp, minnode, maxnode, npoint, &
                                 iwork, icount )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-10-2001 Complex variant of intcop00
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Copy usol into usolcop for the points in the range minnode, maxnode
!   Set array iwork(i) = 1 for copied points and the number in icount
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iunk, indprp, kprobf(*), npoint, &
              kprobp(npoint,*), minnode, maxnode, iwork(*), icount
      double complex usol(*), usolcop(*)

!     icount     o    number of points that have been filled in iwork
!     indprp     i    Indication if array kprob part p has been filled (1)
!                     or not (0)
!     iunk       i    Sequence number of unknown
!     iwork      o    integer work array of length: number of nodes in new
!                     mesh
!                     If iwork(i) = 1, node has been treated, &
!                     if iwork(i) = 0 it is free
!     kprobf     i    KPROB part f: contains the number of unknowns per point
!                     accumulated
!     kprobp     i    SEPRAN array KPROB part p containing the positions of the
!                     the unkowns per point
!     maxnode    i    Maximum nodal point number in part
!     minnode    i    Minimum nodal point number in part
!     npoint     i    Number of nodal points
!     usol       i    Solution vector
!     usolcop    o    Copy of solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nodp, ih, ih1

!     ih         Help variable to store a constant temporarily
!     ih1        Help variable to store a constant temporarily
!     nodp       Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( indprp.gt.0 ) then

!     --- Array kprobp has been filled

         do nodp = minnode, maxnode
            ih = kprobp(nodp,iunk)
            if ( ih.ne.0 ) then
               usolcop(nodp) = usol(ih)
               iwork(nodp) = 1
               icount = icount + 1
            end if
         end do

      else

!     --- Array kprobp has not been filled

         ih = kprobf(minnode)
         do nodp = minnode, maxnode
            ih1 = kprobf(nodp+1)
            if ( ih1-ih.ne.0 ) then
               ih = ih+iunk
               usolcop(nodp) = usol(ih)
               iwork(nodp) = 1
               icount = icount + 1
               ih = ih1
            end if
         end do
      end if
      end
