      subroutine intmshm1 ( iseqkm1, iseqkm2, map )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 19-04-2013 Debug statements
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 02-01-2001
!
!   copyright (c) 2000-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make a formal interpolation from one mesh to another.
!     In fact the mapping between the meshes is computed, &
!     without knowing the function (1D version)
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     mesh
!     1d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemeshinf
      use sepmodulemap
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqkm1, iseqkm2
      type (mapping) :: map

!     iseqkm1     i    Standard SEPRAN array containing information of mesh 1
!     iseqkm2     i    Standard SEPRAN array containing information of mesh 2
!     map        i/o   contains information of the mapping
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer npoint1, npoint2

!     npoint1        number of points in first mesh
!     npoint2        number of points in second mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INTMSHM11      Interpolate solution form one mesh to another (1D)
!     SEPCREATEIBUF  Create permanent space in array IBUFFR
!     SEPCREATERBUF  Create permanent space in array BUFFER
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Create arrays imap, rmap
!     Clear arrays imap, rmap
!     Activate arrays
!     Reserve space for temporary arrys kgrad1/2
!     Compute starting addresses
!     Perform the actual interpolation
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'intmshm1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from intmshm1'
         call ertrace
         write(irefwr,1) 'iseqkm1, iseqkm2', iseqkm1, iseqkm2
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Create arrays imap, rmap

      npoint1 = km(iseqkm1)%kmesh(8)
      npoint2 = km(iseqkm2)%kmesh(8)
      if ( associated(map%imap) ) deallocate ( map%imap, map%rmap )
      call sepclear ( map )
      allocate ( map%imap(npoint2,3), map%rmap(npoint2,2) )
      imap => map%imap
      rmap => map%rmap

!     --- Clear arrays imap, rmap

      imap = 0
      rmap = 0d0
      map%iseqkm1 = iseqkm1
      map%iseqkm2 = iseqkm2

!     --- Perform the actual interpolation

      call intmshm11 ( npoint1, km(iseqkm1)%coor, npoint2, &
                       km(iseqkm2)%coor )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End intmshm1'

      end if  ! ( debug )
      call erclos ( 'intmshm1' )

      end subroutine intmshm1
