      subroutine pr0009 ( ichois, kmeshn, coor, list, nlist, ndim, &
                          nsurfs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 10-03-2006 Debug information
!        version  1.3  date 27-05-2002 Format
!        version  1.2  date 27-12-2001 Print number of elements
!        version  1.1  date 06-04-2001 Correction of counter
!        version  1.0  date 25-06-1993
!
!   copyright (c) 1993-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print (selected) information of surfaces
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois, kmeshn(*), list(*), nlist, ndim, nsurfs
      double precision coor(ndim,*)

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     ichois  i    Choice parameter
!                  Possible values:
!                  0:   print nodal point numbers of all surfaces
!                  1:   print nodal point numbers of all surfaces
!                       and print the co-ordinates also
!                  10: See 0, 1, however in this case only the surfaces
!                       with surf sequence number between list(1)
!                       and list(2) are printed
!                  20: See 0, 1, however in this case only the surfaces
!                       with surface sequence number in list(1) to
!                       list(nlist) are printed
!     kmeshn  i    KMESH part n, contains nodal point numbers of the surfaces
!     list    i    See ichois
!     ndim    i    Dimension of the space.
!     nlist   i    Length of array list in case ichois>19
!     nsurfs  i    number of surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical pricor, chslst
      integer i, iunder, iupper, istart, j, ilist, k, &
              isurf, isurfp, ielem, inpelm, inelem, iielem

!     chslst         Indicate if the nodal point numbers must be found from
!                    list (true) or not (false)
!     i              Counting variable
!     ielem          element sequence number
!     iielem         relative element sequence number with respect to surface
!     ilist          ichois/20
!     inelem         number of elements in surface
!     inpelm         number of points in element
!     istart         Starting address of element in KMESH part n
!     isurf          Surface number
!     isurfp         Previous surface number
!     iunder         lower bound for loop variable
!     iupper         upper bound for loop variable
!     j              Counting variable
!     k              Counting variable
!     pricor         Indicates if co-ordinates must be printed
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Print heading
!   pricor := co-ordinates must be printed
!   choose_list := ichois>=20
!   Define range of loop (iunder, iupper)
!   For i := iunder (1) iupper do
!      if ( choose_list ) then
!         isurf := list(i)
!      else ( not choose_list )
!         isurf := i
!      ifirst := index of first node in surface with respect to kmeshn
!      ilast  := index of last node in surface with respect to kmeshn
!      if ( pricor ) then
!         write nodal point numbers and co-ordinates
!      else ( not pricor )
!         write nodal point numbers
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pr0009' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pr0009'
         write(irefwr,1) 'ichoice, nsurfs', ichois, nsurfs
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Print heading

      pricor = mod(ichois,10)==1
      write ( irefwr,100 )
100   format ( ' Information of surfaces' )

!     --- Set choose_list, iunder, iupper

      ilist = ichois/10
      chslst = ilist==2
      if ( ilist==0 ) then
         iunder = 1
         iupper = nsurfs
      else if ( ilist==1 ) then
         iunder = list(1)
         iupper = list(2)
      else
         iunder = 1
         iupper = nlist
      end if
      istart = 2*nsurfs
      ielem  = 0
      isurfp = 0
      if ( debug ) write(irefwr,1) 'iunder, iupper', iunder, iupper

!     --- loop over surfaces

      do i = iunder, iupper
         if ( chslst ) then
            isurf = list(i)
         else
            isurf = i
         end if
         if ( isurf/=isurfp+1 ) then

!        --- surface number is not the next one, recompute istart

            istart = 2*nsurfs
            ielem  = 0
            do j = 1, isurf-1
               inpelm=kmeshn(j)
               inelem=kmeshn(j+nsurfs)
               istart = istart + inelem*inpelm
               ielem  = ielem + inelem
            end do
         end if
         inpelm=kmeshn(isurf)
         inelem=kmeshn(isurf+nsurfs)
         write(irefwr,120 ) isurf, inelem
120      format ( ' Surface number', i5/ &
                  ' Number of elements is ',i8 )
         if ( pricor ) then

!        --- Print co-ordinates and nodal point numbers

            write(irefwr,130 )
130         format ( ' Element number', ' nodal point numbers', &
                     ' followed by co-ordinates' )
            do iielem = 1, inelem
               ielem = ielem + 1
               do k = 1, inpelm
                  if ( k==1 ) then

!                 --- k=1, print element number

                     write(irefwr,140) ielem, kmeshn(k+istart), &
                            (coor(j,kmeshn(k+istart)),j=1,ndim)
140                  format ( i8, ':', i8, ss, 3(1pe13.4e2) )
                  else

!                 --- k>1, do not print element number

                     write(irefwr,150) kmeshn(k+istart), &
                            (coor(j,kmeshn(k+istart)),j=1,ndim)
150                  format ( 8x, i8, ss, 3(1pe13.4e2) )
                  end if
               end do
               istart = istart+inpelm
            end do

         else

!        --- Print nodal point numbers only

            write(irefwr,210 )
210         format ( ' Element number', ' nodal point numbers')
            do iielem = 1, inelem
               ielem = ielem + 1
               write(irefwr,240) ielem, (kmeshn(k+istart),k=1,inpelm)
240            format ( i7, ':', 10i7/(7x,10i7) )
               istart = istart+inpelm
            end do

         end if

      end do

1000  call erclos ( 'pr0009' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pr0009'

      end if

      end
