      subroutine pr0010 ( npltra, ndim, maxtrn, itrans, coortr, &
                          iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 03-05-2006 Extra parameter iwork
!        version  3.3  date 28-02-2005 Formats
!        version  3.2  date 01-09-2000 Correction maxtrn=9 (CvO)
!        version  3.1  date 20-11-1999 Correction maxtrn=18
!        version  3.0  date 24-10-1998 Complete revision
!        version  2.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  2.0  date 31-03-1991 Deletion of ndim
!        version  1.0  date 12-05-1985
!
!   copyright (c) 1985-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    print kprob part n
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     local_transform
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npltra, ndim, maxtrn, itrans(ndim,*), iwork(4,*)
      double precision coortr(maxtrn,*)

!     coortr         i    array containing the factors for the local
!                         transformation
!     itrans         i    array containing the degrees of freedom that must
!                         be transformed
!     iwork          i    integer work array of size 4*nusol
!                         In this array per unknown the following information
!                         is stored
!                         (1,.) node corresponding to unknown in standard seq
!                         (2,.) local degree of freedom in node corresponding
!                               to unknown in standard sequence
!                         (3,.) node corresponding to unknown if numbering
!                               with respect to matrix is used
!                         (4,.) local degree of freedom in node corresponding
!                               to unknown if numbering
!                               with respect to matrix is used
!     maxtrn         i    Maximum number of real variables per point with
!                         respect to the local transformations
!     ndim           i    Dimension of the space.
!     npltra         i    Number of local transformations
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i          Counting variable
!     j          Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      write(irefwr,*) 'integer part of kprob n'
      write(irefwr,*) 'local_node  node             unknowns'
      do i = 1, npltra
         write(irefwr,90 ) i, iwork(3,itrans(1,i)), &
                           (itrans(j,i),j=1,ndim)
 90      format ( i6, i10, 3i8 )
      end do  ! i = 1, npltra

      if ( ndim==2 ) then

!     --- ndim = 2

         if ( maxtrn==2 ) then

!        --- maxtrn = 2, only nx and ny are stored

            write(irefwr,100) (i,coortr(1,i),coortr(2,i),i = 1, npltra)
100         format(/' factors for local transformations'/ &
                    ' seq        n1         n2 '/ &
                   (i6,2x,ss,2(1pe11.3e2)) )
         else if ( maxtrn==4 ) then

!        --- maxtrn = 4, the whole matrix is stored

            write(irefwr,110) (i,(coortr(j,i),j=1, 4),i = 1,npltra)
110         format(/' factors for local transformations'/ &
                    ' seq      R_11      R_12     R_21      R_22 '/ &
                   (i6,2x,ss,4(1pe11.3e2)) )

         else

!        --- maxtrn = 8, both matrices R and V are stored

            write(irefwr,120) (i,(coortr(j,i),j=1,8,2), &
                                 (coortr(j,i),j=2,8,2),i = 1,npltra)
120         format(/' factors for local transformations'/ &
                    ' seq          R_matrix      V_matrix'/ &
                   (i6,2x,ss,2(1pe11.3e2),5x,2(1pe11.3e2)/ &
                       8x,ss,2(1pe11.3e2),5x,2(1pe11.3e2)) )

         end if

      else

!     --- ndim = 3

         if ( maxtrn==9 ) then

!        --- maxtrn = 9, the whole matrix is stored

            write(irefwr,210) (i,(coortr(j,i),j=1,3), &
                                 (coortr(j,i),j=4,6), &
                                 (coortr(j,i),j=7,9),i = 1,npltra)
210         format(/' factors for local transformations'/ &
                    ' seq         R_matrix '/ &
                   (i6,2x,ss,3(1pe11.3e2)/ &
                       8x,ss,3(1pe11.3e2)/ &
                       8x,ss,3(1pe11.3e2)) )

         else

!        --- maxtrn = 18, both matrices R and V are stored

            write(irefwr,220) (i,(coortr(j,i),j=1,18,3), &
                                 (coortr(j,i),j=2,18,3), &
                                 (coortr(j,i),j=3,18,3),i = 1,npltra)
220         format(/' factors for local transformations'/ &
                    ' seq          R_matrix           V_matrix'/ &
                   (i6,2x,ss,3(1pe11.3e2),5x,3(1pe11.3e2)/ &
                       8x,ss,3(1pe11.3e2),5x,3(1pe11.3e2)/ &
                       8x,ss,3(1pe11.3e2),5x,3(1pe11.3e2)) )

         end if

      end if

      end
