      subroutine pr2025 ( ijcoor, minnode, maxnode, npntij, nx, ny, &
                          pntclij )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.0    date   06-12-1996
!
!
!   copyright (c) 1996-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!    compute number of nodes for each ij combination and store in array
!    npntij. This array is accumulated
! **********************************************************************
!
!                       KEYWORDS
!
!    2d
!    interpolation
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer ijcoor(2,*), minnode, maxnode, npntij(*), nx, ny, &
              pntclij(*)

!     ijcoor     i    Contains the ij cell number of the co-ordinates
!                     ijcoor(1,k) contains the i number of coor_k
!                     ijcoor(2,k) contains the j number of coor_k
!     maxnode    i    Maximum nodal point number in part
!     minnode    i    Minimum nodal point number in part
!     npntij     i    Contains number of coordinates in ij-cell accumulated
!                     npntij(1) = 0
!                     npntij((i-1)*ny+j+1) - npntij((i-1)*ny+j) is number
!                     of points in cell ij
!     nx         i    number of points in x direction
!     ny         i    number of points in y direction
!     pntclij    o    Contains the node numbers per cell
!                     The sequence is defined by npntij
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, nodp, ind, ij, iwork(nx*ny+1)

!     i          Counting variable
!     ij         ij index (one-dimensional storage)
!     ind        Help variable to store an index
!     iwork      integer work array of length nx*ny
!                iwork(i) contains the number of nodes already stored in
!                cell i
!     j          Counting variable
!     nodp       Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Clear array iwork
!    For all nodes nodp in range do
!       (i,j) := ijcoor(.,nodp)
!       ind := iwork((i-1)*ny+j)
!       iwork((i-1)*ny+j) := ind+1
!       pntclij(npntij((i-1)*ny+j)+ind) := nodp
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Clear array iwork

      do i = 1, nx*ny+1
         iwork(i) = 0
      end do

!     --- For all nodes in range do

      do nodp = minnode, maxnode

!     --- (i,j) := ijcoor(.,nodp)

         i = ijcoor(1,nodp)
         j = ijcoor(2,nodp)
         ij = (i-1)*ny+j
         ind = iwork(ij)

!        --- iwork((i-1)*ny+j) := ind+1

         iwork(ij) = ind+1
         pntclij(npntij(ij)+ind+1) = nodp

      end do

      end
