      subroutine pr2026res1 ( lenkmtri, ielem1, ielem2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-04-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Raise length of array kmeshtri, for one surface only
!     The surface is marked by begin and end element
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     subdivision
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lenkmtri, ielem1, ielem2

!     ielem1         i    First element number in surface
!     ielem2         i    Last element number in surface
!     lenkmtri      i/o   Length of array kmeshtri
!                         Is raised in this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp, ielem
      logical subdiv

!     ielem          Element number
!     ielgrp         element group number
!     subdiv         If true the elements must be subdivided
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR2026SUB      Raise lenkmtri as function of shape number and
!                    Set subdiv if necessary
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     ielem := 0
!     For all element groups ielgrp do
!        inelem := number of elements in element group
!        ielem := ielem+inelem
!        if ( ielem>=ielem2 ) then
!           surface in group
!           raise lenkmtri
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pr2026res1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pr2026res1'

      end if
      if ( ierror/=0 ) go to 1000

!     --- ielem := 0

      ielem = 0             ! element number

!     --- For all element groups ielgrp do

      do ielgrp = 1, nelgrp

!     --- inelem := number of elements in element group

         inelem = Topgrp(ielgrp)%inelem

!        --- ielem := ielem+inelem

         ielem = ielem+inelem  ! last element number in this group
         if ( ielem2<= ielem ) then

!        --- ielem2<=ielem hence all elements are in this group

            ishape = kmeshf(ielgrp)  ! shape number
            inelem = ielem2-ielem1+1        ! number of elements

!           --- Raise lenkmtri

            call pr2026sub ( ishape, lenkmtri, subdiv, inelem )
            go to 1000  ! jump because finished

         end if  ! ( ielem2<= ielem )

      end do  ! ielgrp = 1, nelgrp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pr2026res1'

      end if
      call erclos ( 'pr2026res1' )

      end
