      subroutine pr2026sub ( ishape, lenkmtri, subdiv, inelem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-04-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Raise lenkmtri as function of shape number
!     Set subdiv if necessary
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ishape, lenkmtri, inelem
      logical subdiv

!     inelem         i    Number of elements in a specific element group
!     ishape         i    Shape number of elements
!     lenkmtri      i/o   Length of array kmeshtri
!                         The length is extended for this group elements
!     subdiv        i/o   If true the elements must be subdivided
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1586   Subdivision of elements of this shape have not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pr2026sub' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pr2026sub'

      end if
      if ( ierror/=0 ) go to 1000

      if ( ishape==3 ) then

!     --- inpelm=3 Copy kmeshc to kmeshtri

         lenkmtri = lenkmtri+3*inelem

      else if ( ishape==10 ) then

!     --- ishape=10 Skip centroid

         lenkmtri = lenkmtri+3*inelem
         subdiv = .true.

      else if ( ishape==5 .or. ishape==9 ) then

!     --- inpelm=4 Subdivide elements into two triangles

         lenkmtri = lenkmtri+6*inelem
         subdiv = .true.

      else if ( ishape==4 .or. ishape==7 ) then

!     --- inpelm=6, 7 Subdivide elements into four triangles

         lenkmtri = lenkmtri+12*inelem
         subdiv = .true.

      else if ( ishape==6 ) then

!     --- inpelm=9 Subdivide elements into 8 triangles

         lenkmtri = lenkmtri+24*inelem
         subdiv = .true.

      else if ( ishape<3 ) then

!     --- Line elements are skipped

         subdiv = .true.

      else

!     --- Other types have not yet been implemented

         call errint ( ishape, 1 )
         call errsub ( 1586, 1, 0, 0 )

      end if

1000  call erclos ( 'pr2026sub' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pr2026sub'

      end if

      end
