      subroutine pr2027 ( kmeshtri, minnode, maxnode, inelemlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 18-04-2005 Replace body by new subroutine
!        version  2.1  date 14-02-1998 Correction ishape=10
!        version  2.0  date 12-01-1997 Extra parameters
!
!   copyright (c) 1996-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide a mesh consisting of triangles and/or quadrilaterals
!     into three-point triangles
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshtri(*), minnode, maxnode, inelemlc

!     inelemlc       i    Local value of inelem
!     kmeshtri       o    Kmesh part c for triangular mesh
!     maxnode        i    Highest node number in part of mesh
!     minnode        i    Lowest node number in part of mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp, ikelmc, ikelmctri, inpelm

!     ielgrp     element group number
!     ikelmc     First position in kmesh part c with respect to element group
!     ikelmctri  Last position used in kmeshtri
!     inpelm     number of points per element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR2027SUB      Subdivide a mesh consisting of triangles and/or
!                    quadrilaterals into three-point triangles for one
!                    element group
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    For all element groups do
!       inpelm := number of nodes in element group
!       inelem := number of elements in element group
!       if ( inpelm=3 ) then
!          Copy kmeshc to kmeshtri
!       else
!          Subdivide kmeshc into kmeshtri
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pr2027' )

!     --- For all element groups do

      ikelmc = 1
      ikelmctri = 0
      do ielgrp = 1, nelgrp

!     --- inpelm := number of nodes in element group
!         inelem := number of elements in element group

         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         ishape = kmeshf(ielgrp)
         if ( kmeshc(ikelmc+1)>=minnode .and. &
              kmeshc(ikelmc+1)<=maxnode ) then

!        --- element group belongs to sub part, subdivide

            call pr2027sub ( ishape, inelem, kmeshc(ikelmc), kmeshtri, &
                             ikelmctri, inpelm )

         end if

         ikelmc = ikelmc+inpelm*inelem

      end do
      inelemlc = ikelmctri/3
      call erclos ( 'pr2027' )

      end
