      subroutine pr2028 ( npointnw, ijcrnew, &
                          npntij, pntclij, minnodnw, maxnodnw, &
                          coorol, coornw, nx, ny, npointold )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 15-06-2002 Extension of imap with element numbers
!        version  1.0  date 31-12-1996
!
!   copyright (c) 1996-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Find all points in imap that have not been treated yet and interpolate
!    those points
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npointnw, ijcrnew(2,*), npointold, &
              npntij(nx*ny+1), pntclij(max(npointnw,npointold)), minnodnw, &
              maxnodnw, nx, ny
      double precision coorol(2,npointold), &
                       coornw(2,npointnw)

!     coornw     i    Array containing the new co-ordinates
!     coorol     i    Array containing the old co-ordinates
!     ijcrnew    i    Contains the ij cell number of the co-ordinates of mesh2
!                     ijcrnew(1,k) contains the i number of coornw_k
!                     ijcrnew(2,k) contains the j number of coornw_k
!     maxnodnw   i    Maximum nodal point number in part (new mesh)
!     minnodnw   i    Minimum nodal point number in part (new mesh)
!     npntij     i    Contains number of coordinates in ij-cell accumulated
!                     npntij(1) = 0
!                     npntij((i-1)*nx+j+1) - npntij((i-1)*nx+j) is number
!                     of points in cell ij
!     npointnw   i    Number of nodes in new mesh
!     npointold  i    Number of nodes in old mesh
!     nx         i    number of points in x direction
!     ny         i    number of points in y direction
!     pntclij    i    Contains the node numbers per cell
!                     The sequence is defined by npntij
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, inode, icell, jcell, istep, nnodes, ih, ij, n, &
              ilast
      double precision xh(3), yh(3), dphidx(3), dphidy(3), xgauss(3), &
                       x, y, xc, yc, delta, dx, dy
      integer, allocatable :: iwork(:), ihelp(:)
      double precision, allocatable :: work(:)

!     delta      jacobian of element
!     dphidx     Contains the x-derivatives of the basis functions
!     dphidy     Contains the y-derivatives of the basis functions
!     dx         distance in x-direction
!     dy         distance in y-direction
!     i          Counting variable
!     icell      i-index of cell
!     ih         Help variable to store a constant temporarily
!     ihelp      integer work array of length npointnw
!                It contains the sorted sequence of the nodes with smallest
!                distance to point to be interpolated
!     ij         help parameter to compute address of ij-cell in 1-d array
!     ilast      Last node used
!     inode      Node number
!     istep      Sequence number of step
!     iwork      integer work array of length npointnw
!                iwork contains the nodal point numbers of the nodes
!                with shortest distance to the point to be interpolated
!     j          Counting variable
!     jcell      j-index of cell
!     k          Counting variable
!     n          number of nodes in cell
!     nnodes     Number of nodes in isolnw
!     work       Work array of length npointnw
!                Contains the distances of the points stored in iwork
!                with respect to the point to be interpolated
!     x          x co-ordinate of point
!     xc         x-coordinate of point to be interpolated
!     xgauss     baricentric co-ordinates of the point
!     xh         x-coordinates of vertices
!     y          y-coordinate of point
!     yc         x-coordinate of point to be interpolated
!     yh         y-coordinates of vertices
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     PRSORT         Sort real array for increasing sequence
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    For inode = 1, npointnw
!       if ( imap(inode,1)=0 ) then
!          i,j := cell index of inode
!          Find smallest surrounding of i,j containing at least 3 old points
!          Find three points in this new region with smallest distance to inode
!          Interpolate and compute mapping
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!

      ih = max(npointnw,npointold)
      allocate ( iwork(ih), work(ih), ihelp(ih), stat = error )
      if ( error/=0 ) call eralloc ( error, ih, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- For all nodes

      do inode = minnodnw, maxnodnw
         if ( imap(inode,1).eq.0 ) then

!        --- Node has not been treated yet
!            i,j := cell index of inode

            xc = coornw(1,inode)
            yc = coornw(2,inode)
            icell = ijcrnew(1,inode)
            jcell = ijcrnew(2,inode)

!           --- Find smallest surrounding of i,j containing at least 3 old
!               points

            do istep = 1, max(nx,ny)
               nnodes = 0
               do i = max(1,icell-istep), min(icell+istep,nx)
                  ih = (i-1)*ny
                  do j = max(1,jcell-istep), min(jcell+istep,ny)
                     ij = ih+j
                     n = npntij(ij+1) - npntij(ij)
                     do k = 1, n
                        iwork(nnodes+k) = pntclij(npntij(ij)+k)
                     end do
                     nnodes = nnodes + n
                  end do
               end do
               if ( nnodes.ge.6 ) then

!              --- Find three points in this new region with smallest distance
!                  to inode
!                  Interpolate and compute mapping

                  do i = 1, nnodes
                     x = coorol(1,iwork(i))
                     y = coorol(2,iwork(i))
                     work(i) = sqrt ( (x-xc)**2+(y-yc)**2 )
                  end do
                  call prsort ( work, ihelp, nnodes )
                  do i = 1, 3
                     ih = iwork(ihelp(i))
                     xh(i) = coorol(1,ih)
                     yh(i) = coorol(2,ih)
                  end do
                  ilast = 3
100               dphidy(1) = xh(3) - xh(2)
                  dphidy(3) = xh(2) - xh(1)
                  dphidx(1) = yh(2) - yh(3)
                  dphidx(3) = yh(1) - yh(2)
                  delta = dphidx(3)*dphidy(1)-dphidx(1)*dphidy(3)
                  if ( abs(delta).le.0.1d-6*work(1) ) then

!                 --- The three points are lying at one line, &
!                     Remove point last point and add next point

                      ilast = ilast+1
                      if ( nnodes.lt.ilast ) go to 200
                      ih = iwork(ihelp(ilast))
                      xh(3) = coorol(1,ih)
                      yh(3) = coorol(2,ih)
                      go to 100
                  end if
                  dphidy(1) = dphidy(1) / delta
                  dphidy(3) = dphidy(3) / delta
                  dphidy(2) = -dphidy(1) - dphidy(3)
                  dphidx(1) = dphidx(1) / delta
                  dphidx(3) = dphidx(3) / delta
                  dphidx(2) = -dphidx(1) - dphidx(3)

!                 --- compute baricentric co-ordinates of inode

                  dx = coornw(1,inode) - xh(3)
                  dy = coornw(2,inode) - yh(3)
                  xgauss(1) = dphidx(1)*dx + dphidy(1)*dy
                  xgauss(2) = dphidx(2)*dx + dphidy(2)*dy
                  xgauss(3) = 1d0 - xgauss(1) - xgauss(2)
                  imap(inode,1) = 3
                  do i = 1, 3
                     imap(inode,i+1) = iwork(ihelp(i))
                     rmap(inode,i) = xgauss(i)
                  end do
                  imap(inode,5) = 0
                  go to 200
               end if
            end do
200      end if
      end do

      deallocate ( iwork, work, ihelp, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  end
