      subroutine pr2033 ( coor, xmin, ymin, zmin, dx, dy, dz, ijkcoor, &
                          minnode, maxnode, nx, ny, nz )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.0    date   14-02-1998
!
!   copyright (c) 1998-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill array ijkcoor with the ijk-cell number of the co-ordinates
!
! **********************************************************************
!
!                       KEYWORDS
!
!    3d
!    interpolation
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      double precision coor(3,*), xmin, ymin, zmin, dx, dy, dz
      integer ijkcoor(3,*), minnode, maxnode, nx , ny, nz

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     dx         i    Distance between x-coordinates of cells
!     dy         i    Distance between y-coordinates of cells
!     dz         i    Distance between z-coordinates of cells
!     ijkcoor    i    Contains the ijk cell number of the co-ordinates
!                     ijkcoor(1,l) contains the i number of coor_l
!                     ijkcoor(2,l) contains the j number of coor_l
!                     ijkcoor(3,l) contains the k number of coor_l
!     maxnode    i    Maximum nodal point number in part
!     minnode    i    Minimum nodal point number in part
!     nx         i    number of points in x direction
!     ny         i    number of points in y direction
!     nz         i    number of points in z direction
!     xmin       i    Minimum x-value
!     ymin       i    Minimum y-value
!     zmin       i    Minimum z-value
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nodp

!     nodp       Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   For all nodes do
!      i := (x-xmin)/dx+1
!      j := (y-ymin)/dy+1
!      k := (z-zmin)/dz+1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
!     --- Loop over nodes

      do nodp = minnode, maxnode
         ijkcoor(1,nodp) = min(int((coor(1,nodp)-xmin)/dx)+1,nx)
         ijkcoor(2,nodp) = min(int((coor(2,nodp)-ymin)/dy)+1,ny)
         ijkcoor(3,nodp) = min(int((coor(3,nodp)-zmin)/dz)+1,nz)
      end do
      end
