      subroutine pr2039 ( xh, yh, zh, dphidx, eps, singul )
! ======================================================================
!
!        programmer     Guus Segal
!        version 1.0    date   14-02-1998
!
!   copyright (c) 1998-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute gradient of the linear basis functions phi (dphidx) for a
!   tetrahedron with vertices xh, yh and zh
! **********************************************************************
!
!                       KEYWORDS
!
!    3d
!    interpolation
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      double precision xh(4), yh(4), zh(4), dphidx(4,3), eps
      logical singul

!     dphidx     o    Contains the derivatives of the basis functions
!     eps        i    Mininmum value of Jacobian
!     singul     o    If true the Jacobian is smaller than eps
!     xh         i    x-coordinates of vertices
!     yh         i    y-coordinates of vertices
!     zh         i    z-coordinates of vertices
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision delta(4,3), djacob
      integer i, j, iven, ind(5), ind1(4)

!     delta      Help array to compute the jacobian of element
!     djacob     jacobian of element
!     i          Counting variable
!     ind        Integer array containing the cyclic permutation of 1,2,3,4,5
!     ind1       Array of length 4 to store the cyclic permutation of 1 - 3
!     iven       help parameter to compute (-1)^k
!     j          Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!        To compute the derivatives of the basis functions, the jacobian
!        of the element and the weights, the following definitions are used:
!
!             ij     i+1     i
!        delta   =  x    -  x       cyclic;  i = 1,2,3,4;  j =  1,2,3
!                    j       j
!
!         ij        i        i+1,j+1     i+2,j+2        i+1,j+2     i+1,j+2
!        e   =  (-1)  [ delta       delta        - delta       delta        ]
!
!                        11  11        12  12       13  13
!        djacob = - delta   e   - delta   e  - delta   e
!
!                          ij
!        d phi  / dx   =  e   /  djacob
!             i     j
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data ind / 2,3,4,1,2 /, ind1 / 2,3,1,2 /
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
      do i = 1, 4
         delta(i,1) = xh(ind(i)) - xh(i)
         delta(i,2) = yh(ind(i)) - yh(i)
         delta(i,3) = zh(ind(i)) - zh(i)
      end do
      iven = -1
      do i = 1, 4
         do j = 1, 3
            dphidx(i,j) = iven* &
               (delta(ind(i),ind1(j)) * &
                delta(ind(i+1),ind1(j+1)) - &
                delta(ind(i),ind1(j+1)) * &
                delta(ind(i+1),ind1(j)))
         end do
         iven = - iven
      end do

      djacob = - delta(1,1) * dphidx(1,1) &
               - delta(1,2) * dphidx(1,2) &
               - delta(1,3) * dphidx(1,3)
      if ( abs(djacob).lt.eps ) then

!     --- Jacobian too small

         singul = .true.

      else

!     --- Correct jacobian

         singul = .false.
         do i = 1, 4
            do j = 1, 3
               dphidx(i,j) = dphidx(i,j)/djacob
            end do
         end do

      end if
      end
