      subroutine prblockmatlab
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 23-02-2016 New parameters list
!        version  3.4  date 20-06-2013 New definition file ref numbers
!        version  3.3  date 11-12-2012 Long integers
!
!   copyright (c) 2007-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print matrices for block preconditioner to files that can
!     be read by matlab
! **********************************************************************
!
!                       KEYWORDS
!
!     matlab
!     file
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulekprob
      use sepmodulematr
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
!                         See Programmers Guide for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer irefwrsav, method
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     irefwrsav      Help variable to store the value of irefwr
!     method         jmethodq
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inisetnewref

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     INISETNEWREF   Create new reference number
!     PRMAT2MATLAB   Print compact matrix in a form suitable to be read by
!                    matlab
!     PRMATLABRMAT   print a rectangular compact matrix in a form suitable
!                    to be read by matlab
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prblockmatlab' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prblockmatlab'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      IM => intmt(intmat)

      if ( .not. IM%Qsymmetric ) then
         method = 6
      else
         method = 5
      end if  ! ( jmethod==14 .or. jmethod==16 ... )

!     --- Write information concerning dimensions and storage to parms.dat

      irefwrsav = irefwr
      irefwr = inisetnewref()
      open ( unit = irefwr, file = 'parms.dat' )
      write(irefwr,*) nrusolvel
      write(irefwr,*) nrusolpress
      write(irefwr,*) jmethod
      close ( unit = irefwr )
      call inirmref ( irefwr )
      irefwr = irefwrsav

!     --- Write Q matrix and storage to qmat.dat

      call prmat2matlab ( method, nrusolvel, S_Q, irowsQ, 'qmat.dat' )

!     --- Write D matrix and storage to dmat.dat

      call prmatlabrmat ( nrusolpress, S_DIV, irowsG, icolsG, 'dmat.dat' )

!     --- Write G matrix and storage to gmat.dat

      call prmatlabrmat ( nrusolpress, S_G, irowsG, icolsG, 'gmat.dat' )


!     --- Write C matrix and storage to cmat.dat

      call prmat2matlab ( method, nrusolpress, S_C, irowsPM, 'cmat.dat' )

!     --- Write pressure mass matrix and storage to spmmat.dat

      call prmat2matlab ( 5, nrusolpress, S_PM, irowsPM, 'spmmat.dat' )

!     --- print diagonal pressure mass matrix

      call prmat2matlab ( 0, nrusolpress, S_PMD, irowsPM, 'dpmmat.dat' )

!     --- print diagonal velocity mass matrix

      call prmat2matlab ( 0, nrusolvel, S_UD, irowsii, 'vmmat.dat' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prblockmatlab'

      end if  ! ( debug )
      call erclos ( 'prblockmatlab' )

      end
