      subroutine prcpcoef ( isolpres, isolcpcoef, ipost, rpost )
! ======================================================================
!
!        programmer    Ivo Wenneker
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 15-06-1999
!
!   copyright (c) 1998-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Computes pressure-coefficient in all vertices
!
! **********************************************************************
!
!                       KEYWORDS
!
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: isolpres, isolcpcoef, ipost(*)
      double precision rpost(*)

!     ipost      i    Integer array containing information required for
!                     seppost
!                     See the Programmers Guide for a description
!     isolcpcoef i    Array of the structure of isol referring to the
!                     pressure coefficient
!     isolpres  i/o   Array of the structure of isol referring to pressure
!     rpost      i    Real array containing information required for
!                     seppost
!                     See the Programmers Guide for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     PRCPCOEF1      Actual computation of the pressure coefficient
!     SEPCREASOLVEC  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1185   Parameter for computing scaled pressure is equal to zero
!    1186   gamma has value 1
!    1187   error found, whereas profile flow has been solved
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcpcoef' )

!     --- Test if all reference parameters have been filled

      nwarn = 0
      if ( abs(rpost(1))<=0d0 ) then

!     --- Error: spgamma = 0

         call errchr ( 'gamma', 1 )
         call errwar ( 1186, 0, 0, 1 )

      end if
      if ( abs(1d0-rpost(1))<=epsmac ) then

!     --- Error: spgamma = 1

         call errchr ( 'gamma', 1 )
         call errwar ( 1186, 0, 0, 1 )

      end if
      if ( ipost(6)==0 .and. abs(rpost(2))<0d0 ) then

!     --- Error: machref = 0

         call errchr ( 'machnr_ref', 1 )
         call errwar ( 1185, 0, 0, 1 )

      end if
      if ( abs(rpost(4))<=0d0 ) then

!     --- Error: enthinf = 0

         call errchr ( 'enthalpy_infinty', 1 )
         call errwar ( 1185, 0, 0, 1 )

      end if
      if ( abs(rpost(5))<=0d0 ) then

!     --- Error: momlinf = 0

         call errchr ( 'momentum_infinty', 1 )
         call errwar ( 1185, 0, 0, 1 )

      end if
      if ( nwarn/=0 ) then

!     --- Error found: return

         if ( ipost(3)==1 ) then

!        --- Error found, whereas profile flow is computed

             call errsub ( 1187, 0, 0, 0 )

         else

!        --- error is not considered fatal

            iwarn = 0
            nwarn = 0

         end if
         go to 1000

      end if

!     --- Create or activate cpcoef array

      pressure => ks(isolpres)%sol

      call sepcreasolbf1 ( isolcpcoef, ks(isolpres)%typevec, &
                           ks(isolpres)%ivec,  'cpcoef', ks(isolpres)%compl )

!     --- Compute pressure coefficient in all vertices

      call prcpcoef1 ( npoint, pressure, usoltemp, ipost, rpost )

1000  call erclos ( 'prcpcoef' )

      end
