      subroutine prgathercore1 ( usol, usolpart, map, kprobf, npoint )
! ======================================================================
!
!        programmer    Dragan Vidovic
!        version  1.1  date 30-08-2006 New definition of cpost5
!        version  1.0  date 13-10-2005
!
!   copyright (c) 2005-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copies a local solution vector into a global solution vector by using
!     the mapping between the local and the global node numbers. The number
!     of degrees of freedom per point is given by array kprobf.
! **********************************************************************
!
!                       KEYWORDS
!
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer map(*), kprobf(*), npoint
      double precision usol(*), usolpart(*)

!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     map            i    Mapping array that is filled if ichoice = 2 or must
!                         be filled if ichoice = 3
!                         map is an integer array of length 5 with the following
!                         contents:
!                         1: mmimap  memory management reference to array imap
!                         2: 130 to indicate the type of array
!                         3: mmrmap  memory management reference to array rmap
!                         4: mmcoor1  memory management reference to array coor1
!                         5: mmcoor2  memory management reference to array coor2
!     npoint         i    Number of nodal points
!     usol           o    Solution vector
!     usolpart       i    Part of solution vector on one processor
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer node, lnod, lpos, ndeg, i

!     i              Counting variable
!     lnod           local node number
!     lpos           Starting position in usolpart array
!     ndeg           Number of degrees of freedom
!     node           Node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prgathercore1' )

      debug = .false.
      if (debug) print*, 'prgathercore1'

      lpos = 0
      do lnod = 1,npoint
         node = map(lnod)
         ndeg = kprobf(node+1)-kprobf(node)
         do i=1,ndeg
            usol(kprobf(node)+i) = usolpart(lpos+i)
            if (debug) print*, lnod, i, ' copying', node, &
                              usolpart(lpos+i)
         end do
         lpos = lpos + ndeg
      end do
      call erclos ( 'prgathercore1' )

      end
