      subroutine prgatherread ( iinputlc, rtime, niter, lastpart )
! ======================================================================
!
!        programmer    Dragan Vidovic
!        version  2.0  date 30-01-2016 New parameter list
!        version  1.3  date 30-11-2012 Long integers
!        version  1.2  date 28-11-2006 extension of manag
!
!   copyright (c) 2005-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       SEPST1 starts the SEPRAN program SEPPOST.
!       It must be called beore all other subroutines.
!       The mesh is supposed to be made by program SEPMESH and stored in the
!       file meshoutput
!       The input for subroutine PROBDF is stored in array iinputlc and read
!       from the file sepcomp.inf
!       The files meshoutput, sepcomp.inf and sepcomp.out are opened, &
!       the file meshoutput is closed.
!       The solution vectors are read from the file sepcomp.out and stored
!       in memory with the aid of subroutine READSL. The information of
!       the storage is placed in array ISOL.
!       The machine dependent reference numbers iref10, iref10 and iref74 get
!       a value
!       SEPST1 replaces the calls of the following SEPRAN subroutines:
!              START:  General SEPRAN starting subroutine
!              MESHRD: Reads mesh.
!              PROBDF: Reads problem definition.
!              READSL: Read SEPRAN solution vectors
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     start
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulecons
      use sepmodulemeshinf
      use sepmodulekprob
      use sepmodulemain
      use sepmodulevecs
      use sepmodulemach
      use sepmoduleplot
      use sepmodulepost
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinputlc(*), niter, lastpart
      double precision :: rtime(*)

!     iinputlc            Work array of length iinputlc(1) to store the input
!                         from file 10 temporarily.
!                         It contains the user numbers of the solution
!                         vectors to be read
!     lastpart       o    Number of solution vectors in all time steps read from
!                         a sepcomp_par.xxx file
!     niter          o    Maximum number of iterations
!     rtime          o    Array of length 100 containing the actual time at each
!                         time-step in increasing sequence
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer manag(lenmanag), i, ihelp(10), j, numneigh, &
              lenblock, idum, istartinfo, &
              isubmsh, itypvc, ivec, iref, indsol, &
              npointloc, nphysloc(maxsoluts), indprfloc(maxsoluts), &
              lenname10, lenname74, iref10sav, iproblc, iseqkp1
      character (len=80) filenm
      logical check
      character (len=80) text1
      character (len=100) name10sav, name74sav

!     check          Indicates if the file exist (true) or not
!     filenm         Name of file
!     i              Counting variable
!     idum           General dummy variable
!     ihelp          integer work array
!     indprfloc      Array to store the values of indprf for all vectors
!     indsol         Sequence number of vector in isol
!     iproblc        Local value of iprob
!     iref           File reference number
!     iref10sav      Help variable to store value of iref10
!     iseqkp1        Sequence number of kprob1
!     istartinfo     Starting address to put information in array
!                    infotabout
!     isubmsh        Sequence number of a submesh
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     j              Counting variable
!     lenblock       Dummy variable
!     lenname10      Number of characters in name10
!     lenname74      Number of characters in name74
!     manag          Manager array that sets the parameters in subroutine
!                    SEPSTR
!                    See subroutine readsp
!     name10sav      Help variable to store value of name10
!     name74sav      Help variable to store value of name74
!     nphysloc       Array to store the values of nphys for all vectors
!     npointloc      Number of points in local mesh
!     numneigh       number of neighbours of a point
!     text1          a string to read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INIFIL         Open files
!     MSH071         Read information of the mesh from the file indicated by
!                    iref
!     MSHREADTEXT    Read text from file and compare with text that is expected
!     PRGATHERCORE1  Maps local to global solution arrays
!     PRGATHERCORE2  Maps local to global solution arrays
!     PRGATHERSUB1   Perform the first part of subroutine prgather
!     READ74         Read solution vectors from sepcomp.out
!     READPROBIINP   Read information concerning the input for probdfbf from
!                    file sepcomp.out and call subroutine probdfbf
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPCRMESHINFO  Create permanent space for mesh
!     SEPRELKMESH    Make the relation between kmref and the pointers
!                    in sepmodulekmeshp
!     SEPRELKMESH1   Make the relation between kmref and the pointers
!                    in sepmodulekmeshp
! **********************************************************************
!
!                       I/O
!
!    The files meshoutput, sepcomp.inf and sepcomp.out and if it exists
!    sepcomp.freq are read
!    For a description of meshoutput: see the Programmers Guide Section 3.3
!    For a description of the file sepcomp.inf, see subroutine PROUTP
!    For a description of the file sepcomp.out, see subroutine PRINSL
!    For a description of the file sepcomp.freq, see subroutine FVWRITE
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1669   file does not exist
!    2782   gathering has not been tested for this type of array
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill common blocks
!     Open files
!     Read array manag
!     Start SEPRAN
!     Open files with mesh and solution
!     Read mesh
!     Read problem definition from sepcomp.inf
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Perform first part, i.e. start sepran
!         read mesh and problem definition

      kmesh => kmesh2d(1)
      call seprelkmesh ( kmesh )
      kprob => kprob2d(1)
      call prgathersub1 ( iinputlc, .true., manag )

      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prgatherread'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ierror/=0 ) go to 1000

      name10sav = name10        ! save name of file 10
      lenname10 = index ( name10, ' ' ) -1  ! length of name10
      name10 = name10sav(1:lenname10) // '_par.000' ! meshoutput_par.000
      call inifil ( -10 )       ! open file 10
      text1 = '//number of blocks'
      if ( debug ) write(irefwr,*) text1
      call mshreadtext ( iref10, text1, name10 )
      iref10sav = iref10       ! save value of iref10

!     --- Read number of blocks from file meshoutput_par.000

      if ( iref10>0 ) then

!     --- formatted file

         read(iref10,*) iacnodes

      else

!     --- unformatted file

         read(-iref10) iacnodes

      end if

      name74sav = name74     ! save value of name74
      lenname74 = index ( name74, ' ' ) -1         ! length of name74
      filenm = name74sav(1:lenname74-4) // '_par'  ! sepcomp_par

!     --- Loop over submeshes

      do isubmsh=1, iacnodes

         write (filenm,20) filenm(1:lenname74), isubmsh  ! sepcomp_par.xxx
20       format (a,'.',i3.3)
         inquire ( file=filenm, exist=check )
         if ( .not. check ) then

!        --- File sepcomp_par.xxx does not exist

            call errchr ( filenm, 1 )
            call errsub ( 1669, 0, 0, 1 )
            go to 1000

         end if  ! ( .not. check )

         name74 = filenm

         write (name10,20) name10(1:lenname10+4), isubmsh
         inquire ( file=name10, exist=check )
         if ( .not. check ) then

!        --- File meshoutput_par.xxx does not exist

            call errchr ( filenm, 1 )
            call errsub ( 1669, 0, 0, 1 )
            go to 1000

         end if  ! ( .not. check )

         if ( debug ) write(irefwr,*) 'reading ', filenm
         iref10 = sign(81,iref10)    ! temporary file number
         call inifil ( -10 )
         kmesh => kmesh2d(2)
         if ( kmesh<=0 .or. kmesh>lastmesh ) kmesh = 0
         if ( kmesh==0 ) call sepcrmeshinfo ( kmesh )
         call seprelkmesh1
         call msh071 ( manag(5), iref10, ihelp )
         iref = abs(iref10)
         close ( unit = iref )
         if ( ierror/=0 ) go to 1000

         if ( debug ) write(irefwr,*) 'reading problem definition from ', &
               name74
         call inifil ( -74 )
         kprob => kprob2d(2)
         call readprobiinp
         if ( ierror/=0 ) go to 1000

!        --- Read solution from file 74

         if ( debug ) write(irefwr,*) 'reading ', name74
         call read74 ( rtime, niter, manag )
         kmesh => kmesh2d(1)
         call seprelkmesh ( kmesh )
         kprob => kprob2d(1)
         iref = abs(iref74)
         close ( unit = iref )
         name74 = name74sav
         if ( ierror/=0 ) go to 1000

!        --- Skip lines in meshoutput_par.000 until global node numbers section
!            and then read this section

         iref10 = iref10sav
         if ( iref10>0 ) then

!        --- formatted file

            read(iref10,100) text1
100         format(a)
            text1 = '//number of neighbours'
            call mshreadtext ( iref10, text1, name10 )
            if ( ierror/=0 ) go to 1000
            read(iref10,*) numneigh
            text1 = '//blockinfo (first length)'
            call mshreadtext ( iref10, text1, name10 )
            if ( ierror/=0 ) go to 1000
            read(iref10,*) lenblock
            read(iref10,*) (idum,j=1,lenblock)
            text1 = '//transportinfo (first length)'
            call mshreadtext ( iref10, text1, name10 )
            if ( ierror/=0 ) go to 1000
            read(iref10,*) istartinfo
            read(iref10,*) (idum,j=1,istartinfo)
            text1 = '//global node numbers (first number)'
            call mshreadtext ( iref10, text1, name10 )
            if ( ierror/=0 ) go to 1000
            read(iref10,*) npointloc
            allocate ( imap1(npointloc) )
            read(iref10,*) imap1

         else

!        --- unformatted file

            read(-iref10) text1
            text1 = '//number of neighbours'
            call mshreadtext ( iref10, text1, name10 )
            if ( ierror/=0 ) go to 1000
            read(-iref10) numneigh
            text1 = '//blockinfo (first length)'
            call mshreadtext ( iref10, text1, name10 )
            if ( ierror/=0 ) go to 1000
            read(-iref10) idum
            read(-iref10) idum
            text1 = '//transportinfo (first length)'
            call mshreadtext ( iref10, text1, name10 )
            if ( ierror/=0 ) go to 1000
            read(-iref10) istartinfo
            read(-iref10) idum
            text1 = '//global node numbers (first number)'
            call mshreadtext ( iref10, text1, name10 )
            if ( ierror/=0 ) go to 1000
            read(-iref10) npointloc
            allocate ( imap1(npointloc) )
            read(-iref10) imap1

         end if
         if ( ierror/=0 ) go to 1000
         if ( debug ) write(irefwr,*) 'end of reading meshoutput'

         if (isubmsh==1) then

!        --- First submesh; make new solution arrays

            lastpart = lastmapsol
            do indsol = 1, lastpart

               lastmapsol = lastmapsol+1

!              --- Check array length

               call inichkarlen ( lastmapsol, maxsoluts, 'isol' )
               if ( ierror/=0 ) go to 1000

               isolut2d(lastmapsol) = 0
               itypvc = ks(isolut2d(indsol))%typevec
               if ( itypvc/=110 .and. itypvc/=115 .and. &
                    itypvc/=116 ) then

!              --- Gathering has not yet been tested for this type of arrays

                  call errint ( itypvc, 1 )
                  call errsub ( 2782, 1, 0, 0 )
                  go to 1000

               end if
               iprob  = ks(isolut2d(indsol))%iprob
               ivec = ks(isolut2d(indsol))%ivec
               iseqkp = kprob2d(1)
               call sepcreasolbf1 ( isolut2d(lastmapsol), itypvc, ivec, &
                                    names(mod(indsol-1,numvec)+1), &
                                    ks(isolut2d(indsol))%compl )
               indprfloc(indsol) = indprfo
               nphysloc(indsol) = nphyso
               iseqkp1 = iseqkp
            end do
         end if

         do indsol = 1,lastpart

            indprf = indprfloc(indsol)
            usolt => ks(isolut2d(indsol))%sol
            usoltemp => ks(isolut2d(indsol+lastpart))%sol

            if ( indprf/=0 ) then

!           --- indprf # 0, non-constant number of unknowns per point

               iproblc = ks(isolut2d(indsol))%iprob
               itypvc = ks(isolut2d(indsol))%typevec
               if ( itypvc==115 ) then
                  ivec = ks(isolut2d(indsol))%ivec
                  kprobfi => kp(iseqkp1)%kprobsub(iproblc)%kprobe3(ivec)%arr
               else if ( indprf>0 .and. itypvc==110 ) then
                  iproblc = ks(isolut2d(indsol))%iprob
                  kprobfi => kp(iseqkp1)%kprobsub(iproblc)%kprobf
               end if  ! ( indprf>0 .and. itypvc==115 )
               call prgathercore1 ( usoltemp, usolt, &
                                    imap1, kprobfi, npointloc )
            else

!           --- indprf = 0, constpstant number of unknowns per point

               call prgathercore2 ( usoltemp, usolt, &
                                    imap1, nphysloc(indsol), npointloc )
            end if

         end do

      end do

      iref = abs(iref10)
      close ( unit = iref )

!     --- Delete some arrays

      deallocate ( imap1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prgatherread'

      end if  ! ( debug )
      call erclos ( 'prgatherread' )

      end subroutine prgatherread
