      subroutine prgathersub1 ( iinputlc, posint, manag )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 30-01-2016 New parameter list
!        version  1.1  date 28-11-2006 extension of manag
!        version  1.0  date 22-10-2005
!
!   copyright (c) 2005-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform the first part of subroutine prgather
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     start
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulecpack
      use sepmodulemach
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iinputlc(*),  manag(lenmanag)
      logical posint

!     iinputlc            Work array of length iinputlc(1) to store the input
!                         from file 73 temporarily.
!                         It contains the user numbers of the solution
!                         vectors to be read
!     manag          o    Manager array that sets the parameters in subroutine
!                         SEPSTL
!                         See subroutine readsp
!     posint         o    true if seppost is called interactively
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iref, ihelp(10)
      logical meshonly
      character (len=100) name74sav

!     ihelp          Work array to store input from the input file
!     iref           Help variable to store a reference number
!     meshonly       If true only the mesh is plotted
!     name74sav      Help variable to store value of name74
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     INIFIL         Open files
!     INITCB         Initializes SEPRAN machine-independent common blocks
!     INITMD         Set machine-dependent quantities
!     MSH071         Read information of the mesh from the file indicated by
!                    iref
!     READPROBIINP   Read information concerning the input for probdfbf from
!                    file sepcomp.out and call subroutine probdfbf
!     SEPSTART       ?Standard SEPRAN starting subroutine
! **********************************************************************
!
!                       I/O
!
!    The files meshoutput, sepcomp.inf and sepcomp.out and if it exists
!    sepcomp.freq are read
!    For a description of meshoutput: see the Programmers Guide Section 3.3
!    For a description of the file sepcomp.inf, see subroutine PROUTP
!    For a description of the file sepcomp.out, see subroutine PRINSL
!    For a description of the file sepcomp.freq, see subroutine FVWRITE
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1669   file does not exist
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill common blocks
!     Open files
!     Read array manag
!     Start SEPRAN
!     Open files with mesh and solution
!     Read mesh
!     Read problem definition
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      debug = .false.

!     --- Start the program

      call initcb

      cputiming = .true.
      iprogram = 3

      call initmd

!     --- Open read and write files

      if ( posint ) then
         irefre = 5
         irefwr = 6
      else
         call inifil(5)
         call inifil(6)
      end if

!     --- Open standard elements file and error message file

      call inifil(1)
      call inifil(4)

!     --- Read array manag (defaults)

      manag(5) = -1
      manag(1) = 20000
      manag(2) = 1
      manag(3) = -1
      manag(4) = 0
      manag(6) = 102
      manag(7) = 1
      manag(8) = 1
      manag(9) = 1
      manag(10) = 1000

!     --- Standard SEPRAN start subroutine

      if ( mod(manag(1),100)==0 ) manag(1) = manag(1) + 100
      call sepstart ( manag(1), manag(2), manag(3), manag(4) )
      mainsubr = 4
      mxwarn = 0
      call eropen ( 'sepcombineoutexe' )
      call eropen ( 'prgather' )
      call eropen ( 'prgatherread' )
      call eropen ( 'prgathersub1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prgathersub1'

      end if
      if ( ierror/=0 ) go to 1000

      name74sav = name74
      length = index ( name74, ' ' ) -1
      name74 = name74sav(1:length-4) // '_par.001'

!     --- Open files meshoutput and sepcomp_par.001 and read mesh

      call inifil ( -10 )
      inquire ( file=name74, exist=meshonly )
      meshonly = .not. meshonly
      if ( meshonly ) then

!     --- File sepcomp_par.001 does not exist

         call errchr ( name74, 1 )
         call errsub ( 1669, 0, 0, 1 )
         go to 1000

      end if  ! ( meshonly )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,*) 'before msh071'
      call msh071 ( manag(5), iref10, ihelp )
      iref = abs(iref10)
      close ( unit = iref )
      if ( debug ) write(irefwr,*) 'after msh071'
      if ( ierror/=0 ) go to 1000

!     --- Read problem definition

      call inifil ( -74 )
      call readprobiinp
      if ( debug ) write(irefwr,*) 'after readprobiinp'

      iref = abs(iref74)
      close ( unit = iref )
      name74 = name74sav

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prgathersub1'

      end if
      call erclos ( 'prgathersub1' )

      end
