      subroutine princompl ( array, n, text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 20-08-2010 New format
!        version  1.0  date 16-03-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Auxilary help subroutine only meant for debugging purposes.
!   The task of princompl is to print a complex array in the
!   form   seq. number:  number etc.
! **********************************************************************
!
!                       KEYWORDS
!
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      complex * 16 array(*)
      integer n
      character (len=*) text

!    array    i  complex array to be printed
!    n        i  Number of entries of the integer array to be printed
!    text     i  Text to be printed in the heading
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, m, icount, ind(2), j
      double precision d1(2), d2(2)
      character (len=1) chsign(2)

!     chsign         Contains the signs of the imaginary parts of numbers
!     d1             real parts of numbers
!     d2             imaginary parts of numbers
!     i              Counting variable
!     icount         Counter
!     ind            Contains sequence numbers
!     j              Counting variable
!     m              Help variable to store number of elements
!                    to be printed in a line
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINCOMPLNUM   Split a complex number in a real part and the absolute
!                    value of the imaginary part
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'princompl' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from princompl'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      write ( irefwr, 100 ) text
100   format(a)
      icount = 0
      do i = 1, n, 2
         m = min(2,n-icount)
         do j = 1, m
            ind(j) = icount+j
            call princomplnum ( array(ind(j)), d1(j), d2(j), chsign(j) )
         end do  ! j = 1, m
         write ( irefwr, 110 ) (ind(j), d1(j), chsign(j), d2(j),j=1,m)
110      format((2(1x,i4,':',ss,1pe10.3e2," ",a,1pe10.3e2,"i")))
         icount = icount + 2
      end do  ! i = 1, n, 2

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End princompl'

      end if  ! ( debug )
      call erclos ( 'princompl' )

      end
