      subroutine prinedge2nodes ( inode1, inode2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 06-12-2004
!
!   copyright (c) 2004-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the edge corresponding to nodes inode1 and inode2
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     edge
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: inode1, inode2, mmnmeshl

!     inode1         i    First node in edge
!     inode2         i    Second node in edge
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iedge

!     iedge          edge sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIEDGE2NODES  Find the edge corresponding to nodes inode1 and inode2
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLNMESHL  Fill array nmesh part l
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Create nmeshl if it does not exist
!     Compute intersection of edges corresponding to inode1 and inode2
!     Print
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinedge2nodes' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinedge2nodes'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Create nmeshl if it does not exist

      if ( kelmaa==0 ) call sepfillkmeshaa
      if ( nmesh(42)==0 ) call sepfillnmeshl
      if ( ierror/=0 ) go to 1000

!     --- Compute intersection of edges corresponding to inode1 and inode2

      call iniedge2nodes ( nmeshl, inode1, inode2, nmesh, iedge )
      if ( ierror/=0 ) go to 1000

!     --- Print

      write(irefwr,100) iedge, inode1, inode2
100   format ( 'edge ', i8, ' corresponds to nodes ', i8, ' and ', i8 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinedge2nodes'

      end if
      call erclos ( 'prinedge2nodes' )

      end
