      subroutine prinedgenodes ( edgenodes, nodelist, lennodelist )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-07-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print edge numbers corresponding to nodes in nodelist
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     edge
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer edgenodes(2,*), nodelist(*), lennodelist

!     edgenodes      i    Array of size 2 x lennodelist containing the edge
!                         numbers connected to the nodes stored in nodelist
!                         in the sequence of nodelist
!                         Only the edges on the boundary are taken into
!                         account, so each node has at most 2 edge numbers
!                         if node with sequence number j has only one edge
!                         connected to it edgenodes(2,j) = 0
!     lennodelist    i    Number of nodes in nodelist
!     nodelist       i    Array of size lennodelist containing node numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinedgenodes' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinedgenodes'

      end if
      write(irefwr,*) 'edges corresponding to the next nodes'
      write(irefwr,*) ' '
      write(irefwr,*) 'node nr    edges '
      do i = 1, lennodelist
         if ( edgenodes(2,i)==0 ) then

            write(irefwr,100) nodelist(i), edgenodes(1,i)
100         format ( i6, 5x, 2i6)

         else

            write(irefwr,100) nodelist(i), edgenodes(1,i), &
                              edgenodes(2,i)

         end if

      end do

      if ( debug ) write(irefwr,*) 'End prinedgenodes'
      call erclos ( 'prinedgenodes' )

      end
