      subroutine pringrid01 ( coor, solint, ndim, npoint, nphys, text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 09-06-2010 Change of layout
!        version  1.0  date 04-01-2001
!
!   copyright (c) 2001-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print solution and coordinates defined on a rectangular grid
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     regular
!     grid
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, npoint, nphys
      double precision coor(ndim,npoint), solint(nphys,npoint)
      character *(*) text

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     ndim           i    Dimension of the space.
!     nphys          i    Number of physical unknowns, or maximal degrees of
!                         freedom per point
!     npoint         i    Number of nodal points
!     solint         i    Array of size nphys x npoint containing the solution
!                         to be printed
!     text           i    String variable containing the text for the heading
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Write heading

      write(irefwr,100) text
100   format(/' vector ',a/)

      if ( ndim==1 ) then

!     --- ndim = 1

         write(irefwr,110) text
110      format ( '    x        ', a )
         do i = 1, npoint
            write(irefwr,120) coor(1,i),(solint(j,i),j=1,nphys)
120         format(ss,50(1pe13.5e2))
         end do

      else if ( ndim==2 ) then

!     --- ndim = 2

         write(irefwr,200) text
200      format ( '     x             y     ', a )
         do i = 1, npoint
            write(irefwr,120) &
               coor(1,i),coor(2,i),(solint(j,i),j=1,nphys)
         end do

      else

!     --- ndim = 3

         write(irefwr,300) text
300      format ( '     x             y             z     ', a )
         do i = 1, npoint
            write(irefwr,120) &
               coor(1,i),coor(2,i),coor(3,i),(solint(j,i),j=1,nphys)
         end do

      end if

      end
