      subroutine prinintmat ( ichois )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 11-12-2017 New storage of intmat
!        version  2.0  date 21-04-2015 New parameter list
!        version  1.6  date 11-04-2015 Remove ibcmat
!
!   copyright (c) 1999-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Print contents of standard sepran integer array intmat or parts of it
!
! **********************************************************************
!
!                       KEYWORDS
!
!     array
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ichois

!     ichois  i    choice parameter, values:
!                  -1: Print a selection of the complete integer array.
!                      This possibility is only allowed for intmat equal to
!                      kprob, KPROB and INTMAT
!                      If the complete array kprob must be printed, then
!                      both arrays KMPRB and intmat are assumed to be kprob
!                      If the complete array KPROB must be printed, then
!                      array KMPRB is assumed to be kprob and intmat is
!                      assumed to be KPROB
!                   0: Print the complete integer array.
!                      This possibility is only allowed for intmat equal to
!                      kprob, KPROB and INTMAT
!                      If the complete array kprob must be printed, then
!                      both arrays KMPRB and intmat are assumed to be kprob
!                      If the complete array KPROB must be printed, then
!                      array KMPRB is assumed to be kprob and intmat is
!                      assumed to be KPROB
!                  20: print intmat part a
!                  21: print intmat part b
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ndepen
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     ndepen         Number of periodical boundary conditions
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR0005         Print INTMAT corresponding to boundary conditions
!     PRININ         print 1d integer array
!     PRININTMAT1    Print array storeinfo
!     PRININTMAT2    Print contents of array sendunknowns
!     PRINTMATSIMPLE Print the structure of the various matrices in case of a
!                    compact storage for the SIMPLE method
!     SEPFILLMATSTR  Fill parameters in module sepmodulematstr
!     SEPPRINCOMPSTR print part of array intmat corresponding to internal
!                    unknowns. Compact storage
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinintmat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinintmat'
         write(irefwr,1) 'ichois', ichois
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      call sepfillmatstr

!     --- get parameters

      jmethod = IM%jmethod
      nrusol = IM%nrusol

      if ( debug ) write(irefwr,1) 'jmethod, nrusol', jmethod, nrusol

      if ( ichois==0 .or. ichois==20 ) then

!     --- ichois = 0 or 20, print intmat part 1

         if ( IM%simple ) then

!        --- jmethod = 13-16, special storage for simple

            call printmatsimple ( IM )

         else

!        --- standard storage

            if ( size(IM%colsii)>1 ) &
               call  sepprincompstr ( nrusol, IM%rowsii, IM%colsii )
            if ( associated(IM%diag) ) &
               call prinin ( IM%diag, nrusol, &
                             'position of diagonal elements' )

         end if

      end if
      if ( ichois==0 .or. ichois==21 ) then

!     --- ichois = 0 or 21, print intmat part 2

         nbound = IM%nbound
         ndepen = IM%npboun
         ninterface = IM%ninterface
         if ( nbound>0 ) then

            write(irefwr,*) 'boundary conditions i, coupled with', &
                            ' internal unknowns'
            call pr0005 ( nbound, IM%rowsip, &
                          IM%colsip, nrusol+ndepen+ninterface )
            write(irefwr,*) ' boundary conditions i,', &
                 ' coupled with boundary conditions'
            call pr0005 ( nbound, IM%rowspp, IM%colspp, &
                          nrusol+ndepen+ninterface )

         end if

         if ( ndepen>0 ) then

            write(irefwr,*) 'periodical boundary conditions i,', &
                            ' coupled with internal unknowns'
            call pr0005 ( ndepen, IM%rowsic, IM%colsic, nrusol )

         end if

         if ( debug ) &
            write(irefwr,*) 'rowsif', associated(IM%rowsif),  size(IM%rowsif)
         if ( size(IM%rowsif)>1 ) then

!        --- Parallel case, part of equations at other blocks

            write(irefwr,*) 'structure of matrix S_fi'
            call pr0005 ( ninterface, IM%rowsfi, IM%colsfi, nrusol )

            write(irefwr,*) 'structure of matrix S_if'
            call pr0005 ( nrusol, IM%rowsif, IM%colsif, 0 )

         end if  ! ( size(IM%rowsif)>1 )

         if ( debug ) &
            write(irefwr,*) 'rowsout', associated(IM%rowsout),  size(IM%rowsout)
         if ( size(IM%rowsout)>1 ) then

!        --- Parallel case, part of equations at other blocks

            write(irefwr,*) 'structure of matrix Siout'
            call pr0005 ( nrusol, IM%rowsout, IM%colsout, 0 )

         end if  ! ( size(IM%rowsout)>1 )

         if ( debug ) &
            write(irefwr,*) 'storeinfo', &
            associated(IM%storeinfo),  size(IM%storeinfo)
         if ( size(IM%storeinfo)>1 ) then

!        --- Parallel case, storeinfo

            write(irefwr,*) 'structure of storeinfo'
            call prinintmat1 ( IM%storeinfo )

         end if  ! ( size(IM%storeinfo)>0 )

         if ( size(IM%sendunknowns)>1 ) then

!        --- Parallel case, sendunknowns

            write(irefwr,*) 'structure of sendunknowns'
            call prinintmat2 ( IM%sendunknowns )

         end if  ! ( size(IM%sendunknowns)>0 )

         if ( size(IM%recunknowns)>1 ) then

!        --- Parallel case, recunknowns

            write(irefwr,*) 'structure of recunknowns'
            call prinin ( IM%recunknowns, size(IM%recunknowns), &
                          'recunknowns' )

         end if  ! ( size(IM%recunknowns)>0 )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinintmat'

      end if  ! ( debug )
      call erclos ( 'prinintmat' )

      end subroutine prinintmat
