      subroutine prinmap ( map, kmesh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 21-11-2012 Long integers
!        version  1.1  date 05-08-2004 Correction rmap
!        version  1.0  date 07-03-2001
!
!   copyright (c) 2001-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the contents of the mapping array map
!
! **********************************************************************
!
!                       KEYWORDS
!
!     map
!     mapping_array
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulekmeshv
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh
      type (mapping) :: map

!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!                         kmesh must refer to the new mesh
!     map            i    contains information of the mapping
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer npoint

!     ipimap         Starting address of array imap in array IBUFFR
!     iprmap         Starting address of array rmap in buffer
!     npoint         Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINMAP1       Actual printing of array imap
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1656   Array map has not been filled
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinmap' )
      if ( ierror/=0 ) go to 1000

!     --- Perform some tests

      iseqkm = kmesh
      npoint = km(iseqkm)%kmesh(8)
      imap => map%imap
      rmap => map%rmap

!     --- Actual printing

      call prinmap1 ( npoint )
1000  call erclos ( 'prinmap' )
      end subroutine prinmap
