      subroutine prinmarkboun ( nowbnd, maxbounds, markboun, nloop, &
                                lnnowbnd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-11-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the contents of array markboun
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     surface
!     paver
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer maxbounds, nloop, nowbnd(maxbounds,*), lnnowbnd, &
              markboun(lnnowbnd,nloop,*)

!     lnnowbnd       i    Declared length of second index of array nowbnd
!     markboun       i    Three diemsional array of size lnnowbnd x nloop x 3
!                         with the following contents
!                         markboun(i,j,k) refers to relative node i along the
!                         j-th boundary in nowbnd
!                         k has the following meaning.
!                         if Pos. 1 = 0 then the node is free otherwise (1)
!                         it is clcoe to another node.
!                         In that case position 2 contains the boundary number
!                         of that other node and position 3 the relative
!                         position  of the node along that boundary
!     maxbounds      i    Max number of loops in nowbnd
!     nloop          i    number of boundaries in nowbnd
!     nowbnd         i    array containing the boundary nodes for each closed
!                         part
!                         This array is two-dimensional of size (imxlps,*), &
!                         where implxs defines the maximum number of loops
!                         and * is at most the number of nodes in the boundary+1
!                         Pos. (i,1) contains the number of nodes in the closed
!                            boundary part i
!                         Pos. (i,j+1) contains the j-th node number along this
!                            boundary
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ibndnr, i, j

!     i              Counting variable
!     ibndnr         i'th boundary in nowbnd
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinmarkboun' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinmarkboun'

      end if
      if ( ierror.ne.0 ) go to 1000
      write(irefwr,*) 'markboun '
      do ibndnr = 1, nloop
         write(irefwr,*) 'boundary ', ibndnr
         do i = 1, nowbnd(ibndnr,1)
            if ( markboun(i,ibndnr,1).gt.0 ) &
               write(irefwr,*) i, ': ', (markboun(i,ibndnr,j),j=2,3)
         end do  ! i = 1, nowbnd(ibndnr,1)
      end do  ! ibndnr = 1, nloop

1000  call erclos ( 'prinmarkboun' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinmarkboun'

      end if

      end
