      subroutine prinmtdir ( amat )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-04-2015 New definition intmat
!        version  1.0  date 27-02-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     print large matrix in case of a direct storage
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulekprob
      use sepmodulematr
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in ) :: amat(*)

!     amat          i    matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ipmat1, i, leftcl, nel, nelsav
      logical :: unsym, compl
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     compl          If true the matrix is complex
!     i              Counting variable
!     ipmat1         Starting address of first part of matrix
!     leftcl         Left column number
!     nel            number of elements in the topology
!     nelsav         help parameter to save the value of nel
!     unsym          indicates if the matrix is symmetrical (false) or
!                    not (true)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR0006         print row of real matrix
!     PR0015DIR      print profile matrix as compact matrix, skip zeros
!     PR0016DIR      print complex profile matrix as compact matrix, skip zeros
!     PR0019         print row of complex matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinmtdir' )
      debug = debug_prinmtbf
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinmtdir'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      unsym = .not. IM%symmetric
      compl = IM%complexm

      write(irefwr,110)
110   format(' matrix first part,  profile storage:')

      if ( debug ) then

!     --- debug = true, write in debug format

         if ( compl ) then

!        --- Complex case

            call pr0016dir ( amat )

         else

!        --- Real case

            call pr0015dir

         end if  ! ( compl )

      else

!     --- debug = false, standard case

         ipmat1 = 1
         do i = 1, nrusol
            leftcl = icolsii(i)
            nel = i-leftcl
            if(unsym) nel = 2*nel
            nel = nel+1
            nelsav = nel
            if(compl) then
               call pr0019(amat(ipmat1), nel, i)
               ipmat1 = ipmat1+2*nelsav
            else
               call pr0006(amat(ipmat1), nel, i)
               ipmat1 = ipmat1+nelsav
            end if
         end do

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinmtdir'

      end if  ! ( debug )
      call erclos ( 'prinmtdir' )

      end
