      subroutine prinoutsrfs ( ioutersurfs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-02-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the contents of array ioutersurfs
!
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     surface
!     print
!     boundary
!     outer_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ioutersurfs(*)

!     ioutersurfs    i    Contains information about the surfaces in the outer
!                         boundary (3D) in the following way
!                         Pos. 1  nparts  Number of parts the outer boundary
!                                         consists of
!                         Pos. 2-1+nparts Starting addresses of each part
!                         Per part: (first address: ioutersurfs(1+i)
!                         Pos. 1: nouter  Number of surfaces in part
!                         followed by the surface numbers (provided with a sign)
!                         The part itself is closed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nparts, ipart, istart, nouter, i

!     i              Counting variable
!     ipart          Sequence number of part
!     istart         First position used at a part
!     nouter         Number of points outside the obstacle
!     nparts         number of parts the boundary consists of
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     nparts := ioutersurfs(1)
!     for ipart := 1 (1) nparts do
!        istart := ioutersurfs(ipart+1)
!        nouter := ioutersurfs(istart)
!        write outer surfaces for part ipart
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinoutsrfs' )

!     --- nparts := ioutersurfs(1)

      nparts = ioutersurfs(1)

!     --- for ipart := 1 (1) nparts do

      do ipart = 1, nparts

!     --- istart := ioutersurfs(ipart+1)
!         nouter := ioutersurfs(istart)
!         write outer surfaces for part ipart

         istart = ioutersurfs(ipart+1)
         nouter = ioutersurfs(istart)
         write ( irefwr,100 ) ipart, (ioutersurfs(istart+i),i=1,nouter)
100      format ( 'Outer surfaces corresponding to part ', i3 / &
                  (10i6) )

      end do
      call erclos ( 'prinoutsrfs' )
      end
