      subroutine prinpropersur ( iinputsrf, numsurfs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 14-01-2013 correction surfaces
!        version  1.0  date 11-06-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the contents of array iinputsrf for periodical boundary
!     conditions
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     periodical
!     surface
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinputsrf(*), numsurfs

!     iinputsrf      i    Array containing the input for the surfaces
!                         with respect to periodical boundary conditions
!                         See subroutine prordperbc for a description
!     numsurfs       i    number of surfaces
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart, numexcl

!     i              Counting variable
!     istart         Last position used in iinputsrf
!     numexcl        number of curves to be excluded
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinpropersur' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinpropersur'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istart = 0
      do i = 1, numsurfs

         write(irefwr,1) 'ifirst, ilast, iexclu', &
                         iinputsrf(istart+1:istart+3)
         istart = istart+3
         numexcl = iinputsrf(istart)

         if ( numexcl>0 ) then

!        --- There are numexcl curves to be excluded

            call prinin ( iinputsrf(istart+1), numexcl, &
                          'curves to be excluded' )
            istart = istart+numexcl

         end if  ! ( numexcl>0 )

      end do  ! i = 1, numsurfs

1000  call erclos ( 'prinpropersur' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinpropersur'

      end if  ! ( debug )

      end
