      subroutine prinproprerinp ( iinput, rinput, istart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 14-01-2013 correction surfaces
!        version  1.0  date 11-06-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     print the contents of array iinput for periodical boundary conditions
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     periodical
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), istart
      double precision rinput(*)

!     iinput         i    integer input array to be printed
!                         It concerns the part of periodical boundary conditions
!                         corresponding to one pair of points, curves or surfs
!     istart         o    Relative last position used in this part of iinput
!     rinput         i    Real input array
!                         It may contain factor and constant depending on iinput
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numpoints, numcurves, numsurfs, i, numexcl

!     i              Counting variable
!     numcurves      Number of curves
!     numexcl        number of curves to be excluded
!     numpoints      Number of points
!     numsurfs       number of surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINPROPRERIN1 print the contents of array iinput for periodical boundary
!                    conditions
!                    it concerns the part degrees of freedom for one case only
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinproprerinp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinproprerinp'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      write(irefwr,1) 'length of this part', iinput(1)

      numpoints = iinput(2)
      numcurves = iinput(3)
      numsurfs = iinput(4)

!     --- write information of first part

      write(irefwr,1) 'numpoints, numcurves, numsurfs', iinput(2:4)
      call prinin ( iinput(5), numpoints, 'starting addresses points' )
      istart = 5+numpoints
      call prinin ( iinput(istart), numcurves, &
                    'starting addresses curves' )
      istart = istart+numcurves
      call prinin ( iinput(istart), numsurfs, &
                    'starting addresses surfaces' )
      istart = istart+numsurfs

!     --- write information of points

      if ( numpoints>0 ) write(irefwr,*) 'information of points'
      do i = 1, numpoints

         write(irefwr,1) 'ifirst, ilast, nparts', &
                          iinput(istart:istart+2)
         istart = istart+3
         call prinproprerin1 ( iinput, istart, rinput )

      end do  ! i = 1, numpoints

!     --- write information of curves

      if ( numcurves>0 ) write(irefwr,*) 'information of curves'
      do i = 1, numcurves

         write(irefwr,1) 'ifirst, ilast, l, nparts', &
                          iinput(istart:istart+3)
         istart = istart+4
         call prinproprerin1 ( iinput, istart, rinput )

      end do  ! i = 1, numcurves

!     --- write information of surfaces

      if ( numsurfs>0 ) write(irefwr,*) 'information of surfaces'
      do i = 1, numsurfs

         write(irefwr,1) 'ifirst, ilast, iexlcu', &
                          iinput(istart:istart+2)
         istart = istart+2
         numexcl = iinput(istart)

         if ( numexcl>0 ) then

!        --- There are numexcl curves to be excluded

            call prinin ( iinput(istart+1), numexcl, &
                          'curves to be excluded' )
            istart = istart+numexcl

         end if  ! ( numexcl>0 )
         call prinproprerin1 ( iinput, istart, rinput )

      end do  ! i = 1, numsurfs

1000  call erclos ( 'prinproprerinp' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinproprerinp'

      end if  ! ( debug )

      end
