      subroutine prinrl1nonzero ( array, n, m, text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-10-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     print a real array in the form
!     form   seq. number:  m numbers etc.
!     Rows with zero elements are skipped
! **********************************************************************
!
!                       KEYWORDS
!
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, m
      double precision array(m,n)
      character *(*) text

!    array    i  double precision array to be printed
!    m        i  Number of entries of the array to be printed per line
!    n        i  Number of lines of the array to be printed
!    text     i  Text to be printed in the heading
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision amax

!     amax           Absolute maximum value in row
!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prinrl1nonzero' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prinrl1nonzero'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      write ( irefwr, 100 ) text
100   format(1x,a)

      do i = 1, n

!     --- Compute maximum in row

         amax = 0d0
         do j = 1, m
            amax = max ( amax, abs(array(j,i)) )
         end do  ! j = 1, m

         if ( amax>0d0 ) then
            write(irefwr,110) i,(array(j,i),j=1,min(6,m))
110         format(i4,':',ss,6(1pe12.3e2))
            if ( m>6 ) write(irefwr,120) (array(j,i),j=7,m)
120         format((5x,ss,6(1pe12.3e2)))
         end if  ! ( amax>0d0 )

      end do  ! i = 1, n

1000  call erclos ( 'prinrl1nonzero' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prinrl1nonzero'

      end if  ! ( debug )

      end
