      subroutine printcurve ( curves, numcurves, text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 18-11-2009
!
!   copyright (c) 2009-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the coordinates of a set of curves
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: numcurves, curves(numcurves)
      character (len=*) text

!     curves         i    Contains the curve numbers provided with a sign
!     numcurves      i    Number of curves in array curves
!     text           i    String variable containing the heading to be printed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i
      integer, allocatable :: iwork(:)

!     error          Return value of allocate or deallocate
!     i              Counting variable
!     iwork          integer work array of length ncurvs
!                    indicates if curve i must be printed (1) or not (0)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPACTIVATE   Reactivate existing array in IBUFFR
!     PRINCURV       Print coordinates of curves
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2910   Not enough space
!    2911   Not possible to deallocate
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printcurve' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printcurve'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill array iwork

      allocate ( iwork(ncurvs), stat = error )
      if ( error/=0 ) then

!     --- error#0, so not enough space

         call errint ( ncurvs, 1 )
         call errint ( error, 2 )
         call errchr ( 'iwork', 1 )
         call errsub ( 2910, 2, 0, 1 )
         go to 900

      end if  ! ( error/= 0 )

      iwork = 0      ! set iwork to 0

      do i = 1, numcurves
         iwork(abs(curves(i))) = 1      ! mark curve to be printed
      end do  ! i = 1, numcurves

!     --- Print curves

      call princurv ( 2, icurvsmsh, curvesmsh, ncurvs, ndim, iwork, text )

900   deallocate ( iwork, stat = error )
      if ( error/= 0 ) then

!     --- error#0, problem with deallocate

         call errint ( error, 1 )
         call errchr ( 'iwork', 1 )
         call errsub ( 2911, 1, 0, 1 )

      end if  ! ( error/= 0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printcurve'

      end if  ! ( debug )
      call erclos ( 'printcurve' )

      end subroutine printcurve
