      subroutine printerscoor2d ( coor, coor2d, nnodes, iplane, trans )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 28-12-2006  Extra parameter trans
!        version  1.0  date 29-06-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Fill coordinates array for intersection of 3d mesh with plane
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     plane
!     intersection
!     coordinate
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, iplane
      double precision coor(3,*), coor2d(2,*), trans(3,3)

!     coor           i    array of length 3 x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     coor2d         o    Coordinates of local 2d mesh with local numbering
!                         Size 2 x nnodes
!     iplane         i    Defines direction of plane
!                         1:  plane x = constant
!                         2:  plane y = constant
!                         3:  plane z = constant
!                         4:  general plane
!     nnodes         i    Number of nodes in 2d mesh
!     trans          i    In this array the Transformation matrix from x,y,z
!                         co-ordinates to ksi-eta co-ordinates is stored as
!                         well as the origin in trans(.,3)
!                         The transformation of x,y,z to ksi,eta is defined by
!                         ksi = sum_i trans(i,1) ( x(i) - trans(i,3) )
!                         eta = sum_i trans(i,2) ( x(i) - trans(i,3) )
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, node1, node2, j
      double precision fact, xi, eta, x3(3)

!     eta            first local coordinate
!     fact           interpolation factor (rmap(i))
!     i              Counting variable (local node)
!     j              Counting variable (local node)
!     node1          First global node
!     node2          Second global node
!     x3             Contains coordinates of node
!     xi             second local coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printerscoor2d' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printerscoor2d'
         write(irefwr,1) 'iplane', iplane
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( iplane==1 ) then

!     --- iplane = 1, plane x constant

         do i = 1, nnodes
            node1 = imap(i,1)
            node2 = imap(i,2)
            fact = rmap(i,1)
            coor2d(1,i) = fact*coor(2,node1)+(1d0-fact)*coor(2,node2)
            coor2d(2,i) = fact*coor(3,node1)+(1d0-fact)*coor(3,node2)
         end do  ! i = 1, nnodes

      else if ( iplane==2 ) then

!     --- iplane = 2, plane y constant

         do i = 1, nnodes
            node1 = imap(i,1)
            node2 = imap(i,2)
            fact = rmap(i,1)
            coor2d(1,i) = fact*coor(1,node1)+(1d0-fact)*coor(1,node2)
            coor2d(2,i) = fact*coor(3,node1)+(1d0-fact)*coor(3,node2)
         end do  ! i = 1, nnodes

      else if ( iplane==3 ) then

!     --- iplane = 3, plane z constant

         do i = 1, nnodes
            node1 = imap(i,1)
            node2 = imap(i,2)
            fact = rmap(i,1)
            coor2d(1,i) = fact*coor(1,node1)+(1d0-fact)*coor(1,node2)
            coor2d(2,i) = fact*coor(2,node1)+(1d0-fact)*coor(2,node2)
         end do  ! i = 1, nnodes

      else

!     --- iplane = 4, arbitrary plane, compute transformation
!         Transform to ksi-eta and store in coornw

         do j = 1, nnodes

            node1 = imap(j,1)
            node2 = imap(j,2)
            fact = rmap(j,1)
            do i = 1, 3
               x3(i) = fact*coor(i,node1)+(1d0-fact)*coor(i,node2)
            end do
            xi = 0d0
            eta = 0d0
            do i = 1, 3
               xi = xi + trans(i,1) * (x3(i) - trans(i,3))
               eta= eta+ trans(i,2) * (x3(i) - trans(i,3))
            end do
            coor2d(1,j) = xi
            coor2d(2,j) = eta

         end do  ! j = 1, nnodes

      end if  ! ( iplane==1 )

1000  call erclos ( 'printerscoor2d' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( coor2d, nnodes, 2, 'coor2d' )
         write(irefwr,*) 'End printerscoor2d'

      end if  ! ( debug )

      end
