      subroutine printersedgnod ( inode, iedge, icount, marknode, &
                                  markedge )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 30-08-2006 Correction imap(2,.)
!        version  1.0  date 29-06-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill markedge, marknode, imap and rmap in case node
!     of edge is on plane
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     plane
!     intersection
!     edge
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemap
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inode, iedge, icount, marknode(*), markedge(*)

!     icount        i/o   Number of nodes in intersection mesh
!                         If new node is found, icount is raised
!     iedge          i    edge sequence number
!     inode          i    Node sequence number
!     markedge       i    Array of size nedges
!                         If edge i is not intersected markedge(i) = 0, &
!                         otherwise markedge(i)>0 contains the sequence number
!                         of the new node in the local mesh
!     marknode       i    Array of size npoint
!                         If node i is not on plane marknode(i) = 0, &
!                         otherwise marknode(i)>0 contains the sequence number
!                         of the new node in the local mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( marknode(inode)=0 ) then
!        icount := icount+1
!        marknode(inode) := icount
!        markedge(iedge) := icount
!        Fill imap and rmap (icount)
!     else
!        markedge(iedge) := marknode(inode)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printersedgnod' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'iedge, inode, icount', iedge, inode, icount
         write(irefwr,1) 'Debug information from printersedgnod'
  1      format ( a, 1x, (10i8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( marknode(inode)==0 ) then

!     --- Node has not been marked yet, so new intersection

         icount = icount+1        ! raise icount
         markedge(iedge) = icount ! fill markedge
         marknode(inode) = icount ! fill marknode
         imap(icount,1) = inode   ! fill node in imap
         imap(icount,2) = inode   ! dummy node
         rmap(icount,1) = 1d0     ! interpolation strictly node 1

      else

!     --- Node has been marked, fill markedge

         markedge(iedge) = marknode(inode) ! fill markedge

      end if  ! ( marknode(inode)==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printersedgnod'

      end if  ! ( debug )
      call erclos ( 'printersedgnod' )

      end
