      subroutine printimehis2 ( work, num, iwork, iskip )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-03-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write contents of scalar time history array to file
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     time_history
!     scalar
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletimhis
      use sepmodulecons
      use sepmodulecomio
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: num, iwork(num), iskip
      double precision, intent(in) :: work(num+1,*)

!     iskip          i    Indicates if element groups must be skipped (>0)
!                         or not (0)
!     iwork          i    integer work array with the sequence numbers of the
!                         scalars
!     num            i    Number of scalars
!     work           i    Work array with the time history of all scalars
!                         to be printed/plotted as well as the time
!                         work(1,:) contains the time
!                         work(1+i,:) contains the i^th scalar
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, iscal, istart
      character (len=200) :: line

!     i              Counting variable
!     iscal          Sequence number of scalar
!     istart         First position of new scalar in line
!     line           text array to store the heading
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printimehis2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printimehis2'
         write(irefwr,1) 'num, iskip', num, iskip
         call prinin ( iwork, num, 'iwork' )
         call prinrl1 ( work, istephiscal-iskip, num+1, 'work' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      line = ' '

      line(1:8) = '   time'
      istart = 21

      do i = 1, num
         iscal = iwork(i)
         length = min(18,ncharscals(iscal))
         line(istart:istart+length-1) = chscalars(iscal)(1:length)
         istart = istart+20
      end do  ! i = 1, num

      write(irefwr,*) line(1:istart-1)
      write(irefwr,*) ''
      do i = 1, istephiscal-iskip
         write(irefwr,100) work(1:num+1,i)
      end do  ! i = 1, istephiscal-iskip
100   format ( d12.4, 9d20.5 )

1000  call erclos ( 'printimehis2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printimehis2'

      end if  ! ( debug )

      end subroutine printimehis2
