      subroutine printkprobe1 ( ielshape )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-02-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print bounday elements
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ielshape(2,*)

!     ielshape       i    Array of length 2*(nelgrp+numnat) containing:
!                         (1,i) number of nodes in element group i
!                         (2,i) shape number of elements in element group i
!                         if i>nelgrp: i-nelgrp = boundary group number
!                         otherwise it is the element group number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ikelm, itype, ivec, inpelm, ielgrp, i

!     i              Counting variable
!     ielgrp         element group number
!     ikelm          Last position used in kmeshc
!     inpelm         number of points per element
!     itype          Type number of equation
!     ivec           Sequence number of vector of special structure
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printkprobe1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printkprobe1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kprobc => kprobpart%kprobc

      do ivec = 1, ndtvec

         write(irefwr,250) ivec

         ikelm = 0
         do ielgrp = 1, nelgrp+numnat
            itype = kprobc(ielgrp)
            inpelm = ielshape(1,ielgrp)
            if ( itype>=0 ) write(irefwr,210) ielgrp, &
                       (i,kprobe(ikelm+i,ivec),i = 1,inpelm)
            ikelm = ikelm+inpelm

         end do  ! ielgrp = 1, nelgrp+numnat

      end do  ! ivec = 1, ndtvec

210   format(i6, 6(i6, ':', i4)/(6x, 6(i6, ':', i4)))
250   format(/' number of degrees of freedom in nodal points of', &
              ' standard elements' / &
              ' of array of special structure', i3)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printkprobe1'

      end if  ! ( debug )
      call erclos ( 'printkprobe1' )

      end
