      subroutine printmatsimple ( IM )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 21-04-2009 New storage for block matrices
!        version  2.0  date 22-12-2008 Complete revision
!        version  1.1  date 22-10-2008 Extension with jmethod = 35-38
!
!   copyright (c) 2003-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the structure of the various matrices in case of a compact
!     storage for the SIMPLE method
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     matrix_structure
!     simple
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer np, nu, jmethod
      character (len=11) text
      logical qsymmetric, dgsymmetric

!     dgsymmetric    If true the pressure matrix is symmetric
!     jmethod        Indicates type of storage scheme for the matrix
!                    See Programmers Guide for a description
!     np             Number of pressure unknowns
!     nu             Number of velocity unknowns
!     qsymmetric     If true the velocity matrix is symmetric
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR0004         Print INTMAT corresponding to internal matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Write information concerning dimensions
!     Write structure of Q matrix
!     Write structure of D matrix
!     Write structure of DG matrix
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printmatsimple' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printmatsimple'
  1      format ( a, 1x, (10i6) )

      end if

!     --- Write information concerning dimensions

      nu = IM%nu
      np = IM%np
      jmethod = IM%jmethod
      write(irefwr,1) 'number of "velocity" unknowns =', nu
      write(irefwr,1) 'number of "pressure" unknowns =', np
      write(irefwr,*) 'number of of enries in matrix Q = ', &
                       size(IM%colsQ)
      write(irefwr,*) 'number of of enries in matrix D = ', size(IM%colsG)
      write(irefwr,*) 'number of of enries in pressure matrix = ', &
                       size(IM%colsPM)

!     --- The DG matrix is constructed

      write(irefwr,*) 'number of of enries in DG matrix = ', &
                      size(IM%colsDG)

!     --- Write structure of Q matrix

      qsymmetric = IM%Qsymmetric
      dgsymmetric = IM%Gsymmetric
      if ( debug ) &
         write(irefwr,*) 'qsymmetric, dgsymmetric ', &
                          qsymmetric, dgsymmetric
      if ( qsymmetric ) then

!     --- jmethod = 13, 15: Q is symmetric

         text = 'symmetric'

      else

!     --- jmethod = 14, 16: Q is unsymmetric

         text = 'unsymmetric'

      end if
      write(irefwr,*) 'The matrix Q is ', text(1:11)
      write(irefwr,*) 'Structure of matrix Q'
      call pr0004 ( jmethod, nu, IM%rowsQ, IM%colsQ )

!     --- Write structure of D matrix

      write(irefwr,*) 'Structure of matrix D'
      call pr0004 ( jmethod, np, IM%rowsG, IM%colsG )

!     --- Write structure of DG matrix

      if ( dgsymmetric ) then

!     --- jmethod = 13, 14: DG is symmetric

         text = 'symmetric'

      else

!     --- jmethod = 15, 16: DG is unsymmetric

         text = 'unsymmetric'

      end if

      write(irefwr,*) 'The matrix DG is ', text(1:11)
      write(irefwr,*) 'Structure of matrix DG'

      call pr0004 ( jmethod, np, IM%rowsDG, IM%colsDG )

!     --- Write structure of PM matrix

      write(irefwr,*) 'Structure of symmetric Pressure Mass matrix'

      call pr0004 ( jmethod, np, IM%rowsPM, IM%colsPM  )

      if ( debug ) write(irefwr,*) 'End printmatsimple'
      call erclos ( 'printmatsimple' )

      end
