      subroutine printvccurv ( ivectr, icurnr, text )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-03-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the values of a vector along the curve icurnr
!     This subroutine is just meant to simplify the call to prinvc
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     curve
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivectr(*), icurnr
      character *(*) text

!     icurnr         i    Curve sequence number of the curve along which the
!                         solution must be printed
!     ivectr         i    standard sepran array denoting USOL or vector of
!                         special structure defined in nodal points
!     text           i    String variable containing some text to be printed
!                         in the heading of the ouput
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inppri(3), icurvs(2)
      double precision :: rinpri(2)

!     icurvs         Input array with respect to PRINVCBF (see PRINVC)
!     inppri         Input array with respect to PRINVCBF (see PRINVC)
!     rinpri         dummy
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINTVECT      Print the values of a SEPRAN solution vector or vector of
!                    special structure
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printvccurv' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printvccurv'

      end if

!     --- Fill arrays inppri and icurvs with information to print the curve

      inppri(1) = 3
      inppri(2) = 0
      inppri(3) = 2
      icurvs(1) = 0
      icurvs(2) = icurnr

!     --- Perform the printing

      call printvect ( inppri, ivectr, icurvs, text, rinpri )

      if ( debug ) write(irefwr,*) 'End printvccurv'
      call erclos ( 'printvccurv' )

      end
