      subroutine printzerolev2 ( inpsep, iref )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 03-07-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the edges (2d) or faces (3d) of the zero level set
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*), iref

!     inpsep         i    The integer input read is stored in array INPSEP
!                         The first position refers to this subroutine
!                         See subroutine askprizerolev
!     iref           i    File reference number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ilev

!     ilev           Sequence number of level being considered
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CHECKZERLEV3D  Check the faces in the zero level set definition
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINTZEROLEV2D Print the edges (2d) of the zero level set
!     PRINTZEROLEV3D Print the faces (3d) of the zero level set
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPFILLNMESHG  Fill array nmeshg
!     SEPFILLOBFACES Fill kmesh part ad subpart 9: iobstfaces
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printzerolev2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printzerolev2'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ilev = inpsep(1)

      write(iref,2) 'time', t

      if ( debug ) write(irefwr,1) 'kelmaa, kelmad', kelmaa, kelmad

!     --- Create or activate nmeshc (nodes of edges)

      if ( ndim==2 ) then
         if ( nmesh(33)==0 ) call sepfillnmeshc
      else
         if ( nmesh(37)==0 ) call sepfillnmeshg
         if ( debug ) call checkzerlev3d ( ilev )
      end if  ! ( ndim==2 )

      if ( kmeshad(9)==0 ) call sepfillobfaces

      if ( ndim==2 ) then

!     --- ndim = 2

         call printzerolev2d ( coor, ilev, nmeshc, obstfaces, iref )

      else if ( ndim==3 ) then

!     --- ndim = 3

         call printzerolev3d ( coor, ilev, nmeshg, obstfaces, iref )

      end if  ! ( ndim==2 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printzerolev2'

      end if  ! ( debug )
      call erclos ( 'printzerolev2' )

      end subroutine printzerolev2
