      subroutine prmatlabmatpc ( jmetod, n, amat, intdia, intpro )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-08-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    print complex profile matrix in a form suitable to be read by matlab
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     print
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduledebug
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jmetod, n, intdia(*), intpro(*)
      complex *16 amat(*)

!     amat           i    large matrix (profile storage)
!     intdia         i    Contains the positions of the diagonal of the large
!                         matrix
!     intpro         i    intmat part 1b
!     jmetod         i    Indicates type of storage scheme for the matrix
!                         See Programmers Guide for a description
!     n              i    number of degrees of freedom
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      complex * 16 am

!     am             Help parameter to store a matrix element
!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmatlabmatpc' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmatlabmatpc'
         write(irefwr,1) 'n, format_matlab, jmetod', &
                          n, format_matlab, jmetod
         call prinrl ( amat, 10, 'amatu' )
         call prinin ( intdia, 10, 'intdia' )
         call prinin ( intpro, 10, 'intpro' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

100   format(i9,1x,i9,ss,1pe25.16e2, 3x, 1pe25.16e2)

!     --- first write diagonal of matrix

      write(irefwr,100) (i,i,amat(intdia(i)),i=1,n)
      write(irefwr,110)
110   format('')

!     --- Next write lower tri-diagonal part of matrix

      do i = 2, n
         do j = intpro(i), i-1
            am = amat(intdia(i)-i+j)
            if ( abs(am)>0d0 ) write(irefwr,100) i,j,am
         end do
      end do

!     --- Finally write upper tri-diagonal part of matrix

      if ( jmetod/=4 ) go to 1000
      write(irefwr,110)

      do i = 2, n
         do j = intpro(i), i-1
            am = amat(intdia(i)+i-j)
            if ( abs(am)>0d0 ) write(irefwr,100) j,i,am
         end do
      end do

1000  call erclos ( 'prmatlabmatpc' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmatlabmatpc'

      end if  ! ( debug )

      end
