      subroutine proutpraise ( icounter, iavs, nameavs, nametec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-04-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Raise icounter if files already exist
!     Only in case of tecplot and avs
! **********************************************************************
!
!                       KEYWORDS
!
!     file
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iavs, nameavs, nametec
      integer, intent(inout) :: icounter

!     iavs           i    Indicates which kind of output must be created.
!                         Possible values:
!                         0:  Create output for SEPPOST only in the form of file
!                             sepcomp.out
!                         1:  Create output for AVS only in the form of a file
!                             sepavs_xxx.yyy
!                             With xxx a time sequence number of the shape 001, &
!                             002 etc and yyy a vector sequence number of the
!                             shape 001, 002 etc
!                         2:  Create output for SEPPOST in the form of file
!                             sepcomp.out
!                             and for AVS in the form of a file sepavs_xxx.yyy
!     icounter      i/o   Counter for output files
!                         At input 1
!                         If files exist the counter is raised
!     nameavs        i    Sequence number of name of avs file in array
!                         file_names
!     nametec        i    Sequence number of name of tecplot file in array
!                         file_names
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nchar
      character (len=110) filenm
      character (len=100) filename
      logical :: avs, tecplot, check

!     avs            Indicates if avs output must be generated (true)
!                    or not (false)
!     check          Indicates if the file exists (true) or not
!     filename       Name of file
!     filenm         Name of file
!     i              Counting variable
!     nchar          Number of characters in a string
!     tecplot        Indicates if tecplot output must be generated (true)
!                    or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIFILENAME    Create filename from indfile and namefile
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'proutpraise' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from proutpraise'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      avs    = mod(iavs/2,2)==1
      tecplot = mod(iavs/8,2)==1

      if ( avs ) then

!     --- avs file format

         call inifilename ( nameavs, 'sepavs', filename, nchar )
         nchar = nchar+1
         filename(nchar:nchar) = '_'

         do i = 1, 999

            write ( filenm, 100 ) filename(1:nchar), icounter
100         format ( a,i3.3,'.001' )
            inquire ( file=filenm, exist=check )
            if ( .not. check ) exit
            icounter = icounter+1
         end do  ! i = 1, 999

      else if ( tecplot ) then

!     --- tecplot file format

         call inifilename ( nametec, 'septec', filename, nchar )

         do i = 1, 999
            write ( filenm, 110 ) filename(1:nchar), icounter
110         format ( a,'.',i3.3 )
            inquire ( file=filenm, exist=check )
            if ( .not. check ) exit
            icounter = icounter+1
         end do  ! i = 1, 999

      end if  ! ( avs )

1000  call erclos ( 'proutpraise' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End proutpraise'

      end if  ! ( debug )

      end
