      subroutine prsepblock ( nbuf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-01-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Main part of program makeblock
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     read
!     write
!     input
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecons
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nbuf

!    nbuf   i   If nbuf<0, all output by sepmesh is suppressed.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer manag(lenmanag)

!     manag          Manager array that sets the parameters in subroutine
!                    SEPSTR
!                    See subroutine readsp
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRSEPBLOCK1    Make mesh input file for a set of blocks
!     PRSEPBLOCK2    Make problem input file for a set of blocks
!     STARTSEP       General start routine for sepran
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Start SEPRAN

      call startsep ( nbuf, manag )

!     --- Run program

      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsepblock'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call eropen ( 'prsepblock' )

!     --- Create and write mesh input

      irefwr = 10
      open ( unit=irefwr, file = 'block.msh' )
      call prsepblock1
      close ( irefwr )

!     --- Create and write problem input

      irefwr = 10
      open ( unit=irefwr, file = 'block.prb' )
      call prsepblock2
      close ( irefwr )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsepblock'

      end if  ! ( debug )
      call erclos ( 'prsepblock' )

      end subroutine prsepblock
