      subroutine prsepblockcurs ( numlayers )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-01-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write part curves to output file
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numlayers

!     numlayers      i    Number of blocks that are created
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icount, jcount
      character (len=80) text

!     i              Counting variable
!     icount         Counter
!     jcount         Counter
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsepblockcurs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsepblockcurs'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      write(irefwr,*) 'curves'

      write(irefwr,*) '   c1  = line shape_cur ( p1,p2,nelm = nx )'
      write(irefwr,*) '   c2  = line shape_cur ( p2,p3,nelm = ny )'
      write(irefwr,*) '   c3  = line shape_cur ( p3,p4,nelm = nx )'
      write(irefwr,*) '   c4  = line shape_cur ( p4,p1,nelm = ny )'
      write(irefwr,*) '   c5  = line shape_cur ( p5,p6,nelm = nx )'
      write(irefwr,*) '   c6  = line shape_cur ( p6,p7,nelm = ny )'
      write(irefwr,*) '   c7  = line shape_cur ( p7,p8,nelm = nx )'
      write(irefwr,*) '   c8  = line shape_cur ( p8,p5,nelm = ny )'
      write(irefwr,*) '   c9  = line shape_cur ( p1,p5,nelm = nz )'
      write(irefwr,*) '   c10 = line shape_cur ( p2,p6,nelm = nz )'
      write(irefwr,*) '   c11 = line shape_cur ( p3,p7,nelm = nz )'
      write(irefwr,*) '   c12 = line shape_cur ( p4,p8,nelm = nz )'

      icount = 12
      jcount = 8
      do i = 2, numlayers

         write(text,100) icount+1, jcount+1, jcount+2, 'nx'
100      format ( '   c', i3, ' = line shape_cur ( p', i3, ',p', &
                  i3, ',nelm = ', a, ' )' )
         write(irefwr,*) text
         write(text,100) icount+2, jcount+2, jcount+3, 'ny'
         write(irefwr,*) text
         write(text,100) icount+3, jcount+3, jcount+4, 'nx'
         write(irefwr,*) text
         write(text,100) icount+4, jcount+4, jcount+1, 'ny'
         write(irefwr,*) text

         icount = icount+4
         write(text,100) icount+1, jcount-3, jcount+1, 'nz'
         write(irefwr,*) text
         write(text,100) icount+2, jcount-2, jcount+2, 'nz'
         write(irefwr,*) text
         write(text,100) icount+3, jcount-1, jcount+3, 'nz'
         write(irefwr,*) text
         write(text,100) icount+4, jcount, jcount+4, 'nz'
         write(irefwr,*) text
         icount = icount+4
         jcount = jcount+4

      end do  ! i = 2, numlayers

1000  call erclos ( 'prsepblockcurs' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsepblockcurs'

      end if  ! ( debug )

      end
