      subroutine prsetname ( vectorname, ivecnm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 20-03-2015 ivecnm is raised
!        version  1.2  date 20-04-2012 Indexed vectors
!        version  1.1  date 30-08-2006 New definition of cpost5
!
!   copyright (c) 1999-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Get the sequence number corresponding to the vector with vector name
!     If the name does not exist create a new sequence number and adapt that
!     name to this new sequence number
! **********************************************************************
!
!                       KEYWORDS
!
!     name
!     vector_name
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulecons
      use sepmodulecomio
      use sepmoduleplot
      use sepmodulepost
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character (len=*), intent(in) :: vectorname
      integer, intent(inout) :: ivecnm

!    ivecnm      i/o  Sequence number of vector with respect to array names - 1
!    vectorname   i   Name of vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, nch

!     i              General loop variable
!     nch            Number of characters in a string
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRGETNAME      Get sequence number of vector with specific name
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     672   The number of vectors or constpstants per time step to be created
!           is too large
! **********************************************************************
!
!                       PSEUDO CODE
!
!     ivecnm := -1
!     if ( vectorname is in names ) then
!        return with correct ivecnm
!     if ( ivecnm = -1 ) then
!        if ( numvec<maxvc1 ) then
!           numvec := numvec+1
!           ivecnm := numvec
!           names(ivecnm+1) := vectorname
!        else ( numvec = maxvc1 )
!           if ( there is a free name ) then
!              ivecnm := free position
!              names(ivecnm+1) := vectorname
!           else
!              error: no free position found
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsetname' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsetname'
         write(irefwr,*) 'vectorname ', vectorname
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- ivecnm := -1

      ivecnm = -1

!     --- if ( vectorname is in names ) then
!            return with correct ivecnm

      call prgetname ( vectorname, ivecnm )

      if ( debug ) &
         write(irefwr,1) 'ivecnm, numvec, maxvc1', &
                          ivecnm, numvec, maxvc1

      if ( ivecnm<0 ) then

!     --- ivecnm = -1

         if ( numvec<maxvc1 ) then

!        --- numvec<maxvc1

            numvec = numvec+1
            if ( mainsubr==4 ) then
               ivecnm = numvec-1
            else
               ivecnm = numvec
            end if  ! ( mainsubr==4 )
            names(numvec) = vectorname
            ncharnames(numvec) = len(vectorname)   ! number of characters

         else

!        --- numvec = maxvc1
!            if ( there is a free name ) then
!               ivecnm := free position
!               names(ivecnm) := vectorname

            do i = 1, numvec
               if ( names(i)(1:8)=='vector_v' ) then
                  ivecnm = i-1
                  names(ivecnm+1) = vectorname
                  nch = index ( vectorname, ' ' ) -1
                  if ( nch<0 ) nch = 32
                  ncharnames(ivecnm+1) = len(vectorname)
                  go to 1000
               end if
            end do

            if ( ivecnm<0 ) then

!           --- error: no free position found

               call errint ( numvec, 1 )
               call errint ( maxvc1, 2 )
               call errsub ( 672, 2, 0, 0 )
               call errcrd

            end if

         end if

      end if

1000  call erclos ( 'prsetname' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ivecnm', ivecnm
         write(irefwr,*) 'End prsetname'

      end if  ! ( debug )

      end
