      subroutine prsw03 ( idim, ndim, npoint, iseq, coor, istore, iwork, &
                          work )
! ======================================================================
!
!        programmer     Guus Segal
!        version 3.2    date   25-02-1996 New call to prsort
!        version 3.1    date   28-01-1995 Correction
!        version 3.0    date   03-06-1994 Extra parameter istore
!        version 2.0    date   01-02-1991 Complete new subroutine, preceding one
!                                         made by Jaap v.d. Zanden
!        version 1.1    date   07-11-1989 Correction lt --> le
!        version 1.0    date   05-09-1985
!
!
!
!   copyright (c) 1985-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!   Rearrange the second and third co-ordinate of coor such that an increasing
!   sequence is created for nodex with equal first co-ordinate
!
! ********************************************************************
!
!                      KEYWORDS
!
!    print
!    sort
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer idim, ndim, npoint, iseq(npoint), iwork(*), istore(*)
      double precision coor(ndim,*), work(npoint)

!     coor     i  Array containing the co-ordinates of the nodes in the original
!                 sequence
!     idim     i  Indicates the first dimension that must be used for the
!                 renumbering
!                 If idim<0, array istore is used
!     iseq    i/o In this array the numbering of the nodes is stored
!                 At input ISEQ is ordered with increasing IDIM co-ordinate
!                 At output points with equal IDIM co-ordinate are ordered
!                 with respect to equal IDIM+1 co-ordinate and for 3D problems
!                 this process is repeated
!     istore  i/o In this array the numbering in the solution array is stored
!                 At input it contains the numbering as computed before
!                 at output this numbering is rearranged
!     iwork       Work array of length 2*npoint
!     ndim     i  Dimension of the space
!     npoint   i  Number of nodes that must be sorted. NPOINT may be smaller
!                 than the actual number of available nodes
!     work        Work array of length npoint
! ********************************************************************
!
!                      COMMON BLOCKS
!
! ********************************************************************
!
!                    LOCAL PARAMETERS
!
      integer k, icount, jcount, kcount, lcount, jdim, n, kdim, ldim
      double precision eps, y, ybegin, z, zbegin

!    eps      Accuracy to determine whether two co-ordinates are equal
!    icount   Counting variable for first Outer loop
!    jcount   Counting variable for first inner loop
!    jdim     IDIM+1 modulo ndim
!    k        General loop variable
!    kcount   Counting variable for second Outer loop
!    kdim     IDIM+2 modulo ndim
!    lcount   Counting variable for second inner loop
!    ldim     abs(idim)
!    n        number of items with equal co-ordinate
!    y        y-co-ordinate
!    ybegin   First y-co-ordinate in a loop of equal y-co-ordinates
!    z        z-co-ordinate
!    zbegin   First z-co-ordinate in a loop of equal z-co-ordinates
! ********************************************************************
!
!             SUBROUTINES CALLED
!
!     PRSORT  Sort real array for increasing sequence
! ********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                    ERROR MESSAGES
!
!   - &
! ********************************************************************
!
!                    PSEUDO CODE
!
!    jdim := mod(idim,3)+1
!    icount := 1
!    eps := 1d-5
!    while ( icount <= npoint ) do
!       ybegin := coor(idim,iseq(icount))
!       lcount := icount
!       while ( lcount<=npoint and |ybegin-coor(idim,iseq(lcount))|<eps ) do
!          work(lcount) := coor(idim,iseq(lcount))
!          lcount := lcount + 1
!       lcount := lcount-1
!       Sort work(i), i := icount (1) lcount, store new sequence in iwork
!       Reorder iseq(i), i := icount (1) lcount, using iwork
!       icount := jcount + 1
!
!       If ( ndim=3 ) then
!          kcount := icount
!          while ( kcount <= jcount ) do
!             zbegin := coor(jdim,iseq(kcount))
!             lcount := kcount
!             while ( lcount<=jcount and |zbegin-coor(jdim,iseq(lcount))|<eps )
!                work(lcount) := coor(jdim,iseq(lcount))
!                lcount := lcount + 1
!             lcount := lcount-1
!             Sort work(i), i := kcount (1) lcount, store new sequence in iwork
!             Reorder iseq(i), i := kcount (1) lcount, using iwork
!             kcount := lcount + 1
!
!       icount := jcount + 1
!
! ======================================================================
!
      ldim = abs(idim)
      jdim = mod(ldim,ndim) + 1
      kdim = mod(jdim,ndim) + 1
      icount = 1
      eps = 1d-5

!     --- While loop icount <= npoint

100   if ( icount.le.npoint ) then

         ybegin = coor(ldim,iseq(icount))
         jcount = icount
         y = ybegin

!        --- While loop |ybegin-y| < eps

150      if ( abs(y-ybegin).le.eps ) then

            work(jcount) = coor(jdim,iseq(jcount))
            jcount = jcount + 1
            if ( jcount.le.npoint ) then

!           --- jcount <= npoint, while loop not yet finished

               y = coor(ldim,iseq(jcount))
               go to 150

            end if

         end if
         jcount = jcount - 1

!        --- Sort work(i), i = icount(1)jcount if jcount > icount

         if ( jcount.gt.icount ) then

!        --- jcount > icount

            n = jcount-icount+1
            call prsort ( work(icount), iwork, n )

!           --- Store new sequence of iseq into iwork from position n+1

            do 160 k = 1, n
               iwork(n+k) = iseq(icount-1+iwork(k))
160         continue

!           --- Copy back

            do 170 k = 1, n
               iseq(k+icount-1) = iwork(k+n)
170         continue

            if ( idim.lt.0 ) then

!              --- Store new sequence of istore into iwork from position n+1

               do 180 k = 1, n
                  iwork(n+k) = istore(icount-1+iwork(k))
180            continue

!              --- Copy back

               do 190 k = 1, n
                  istore(k+icount-1) = iwork(k+n)
190            continue

            end if
            if ( ndim.eq.3 ) then

!           --- NDIM = 3, sort third co-ordinate for equal second co-ordinate

               kcount = icount

!              --- While loop kcount <= jcount

200            if ( kcount.le.jcount ) then

                  zbegin = coor(jdim,iseq(kcount))
                  lcount = kcount
                  z = zbegin

!                 --- While loop |zbegin-z| < eps

250               if ( abs(z-zbegin).le.eps ) then

                     work(lcount) = coor(kdim,iseq(lcount))
                     lcount = lcount + 1
                     if ( lcount.le.jcount ) then

!                    --- lcount <= jcount, while loop not yet finished

                        z = coor(jdim,iseq(lcount))
                        go to 250

                     end if

                  end if
                  lcount = lcount - 1

!                 --- Sort work(i), i = kcount(1)lcount if lcount > kcount

                  if ( lcount.gt.kcount ) then

!                 --- lcount > kcount

                     n = lcount-kcount+1
                     call prsort ( work(kcount), iwork, n )

!                    --- Store new sequence of iseq into iwork from position n+1

                     do 260 k = 1, n
                        iwork(n+k) = iseq(kcount-1+iwork(k))
260                  continue

!                    --- Copy back

                     do 270 k = 1, n
                        iseq(k+kcount-1) = iwork(k+n)
270                  continue

                     if ( idim.lt.0 ) then

!                       --- Store new sequence of istore into iwork from
!                           position n+1

                        do 280 k = 1, n
                           iwork(n+k) = istore(kcount-1+iwork(k))
280                     continue

!                       --- Copy back

                        do 290 k = 1, n
                           istore(k+kcount-1) = iwork(k+n)
290                     continue

                     end if

                  end if

                  kcount = lcount + 1
                  go to 200

               end if

            end if

         end if
         icount = jcount + 1
         go to 100

      end if

      end
