      subroutine prsw06c14 ( iformat, inodes, nnodes, usol, heading, &
                             coor, ndim, kprobf, number, nunkp, indprf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-12-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the solution for subroutine PRINVC
!     complex case
!     Print coordinates
!     Do not print node numbers
!     Transformation to polar coordinates
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inodes(*), nnodes, iformat, ndim, kprobf(*), number, &
              nunkp, indprf
      double precision coor(ndim,*)
      complex * 16 usol(*)
      logical heading

!     coor           i    array of length ndim x npoint containing the
!                         coordinates of the nodal points with respect to the
!                         global numbering
!     heading        i    If true a heading must be printed
!     iformat        i    Format used for printing
!                         1: e13.5
!                         2: e16.8
!                         3: e13.5, one node per line
!                         4: e16.8, one node per line
!     indprf         i    indication if kprob part f has been filled (>0)
!                         or not (0)
!     inodes         i    Array containing the nodal point numbers for which
!                         printing must be performed
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     ndim           i    Dimension of the space.
!     nnodes         i    Number of nodes in inodes
!     number         i    The number-th unknown in each nodal point of the array
!                         is considered. If the array is complex number=2*i-1
!                         corresponds to the real part of array i and
!                         number=2*i to the imaginary part of array i
!     nunkp          i    Maximum number of unknowns in a point
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision x, y, r, phi
      integer i, j, inunkp, istart

!     i              Counting variable
!     inunkp         Number of degrees of freedom in first point
!     istart         First position used at a part
!     j              Counting variable
!     phi            Angle
!     r              Radius
!     x              x coordinate
!     y              y coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!     Print of vector
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsw06c14' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsw06c14'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( number>0 .or. nunkp==1 ) then

!     --- only one unknown

         if ( heading ) write(irefwr,350)
350      format('      x            y         value')
         do i = 1, nnodes
            x = coor(1,inodes(i))
            y = coor(2,inodes(i))
            r = sqrt(x**2+y**2)
            if ( x>0d0 ) then
               phi = atan(y/x)
            else
               phi = 0d0
            end if
            write(irefwr,360) r, phi, usol(i)
360         format((1x,ss,1pe13.5e2,1pe13.5e2, &
                   2('(',1pe13.5e2,',',1pe13.5e2,1x,')',2x)))

         end do

      else if ( indprf<=0 ) then

!     --- more unknowns

         if ( heading ) write(irefwr,400)
400      format('      x            y         values'/)
         do i = 1, nnodes
            x = coor(1,inodes(i))
            y = coor(2,inodes(i))
            r = sqrt(x**2+y**2)
            if ( x>0d0 ) then
               phi = atan(y/x)
            else
               phi = 0d0
            end if
            if ( iformat==1 ) then
               write(irefwr,440) r, phi, &
                  (usol(j+(i-1)*nunkp), j=1,min(3,nunkp))
               if ( nunkp>3 ) write(irefwr,450) &
                  (usol(j+(i-1)*nunkp), j=4,nunkp)
            else if ( iformat==2 ) then
               write(irefwr,441) r, phi, &
                  (usol(j+(i-1)*nunkp), j=1,min(3,nunkp))
               if ( nunkp>3 ) write(irefwr,451) &
                  (usol(j+(i-1)*nunkp), j=4,nunkp)
            else if ( iformat==3 ) then
               write(irefwr,442) r, phi, &
                  (usol(j+(i-1)*nunkp), j=1,nunkp)
            else
               write(irefwr,443) r, phi, &
                  (usol(j+(i-1)*nunkp), j=1,nunkp)
            end if
         end do

      else

!     --- number of unknowns not constant

         if ( heading ) write(irefwr,400)
         do i = 1, nnodes
            istart = kprobf(i)
            inunkp = kprobf(i+1)-istart
            x = coor(1,inodes(i))
            y = coor(2,inodes(i))
            r = sqrt(x**2+y**2)
            if ( x>0d0 ) then
               phi = atan(y/x)
            else
               phi = 0d0
            end if
            if ( iformat==1 ) then
               write(irefwr,440) r, phi, &
                  (usol(j+istart), j=1,min(2,inunkp))
               if ( inunkp>2 ) write(irefwr,450) &
                  (usol(j+istart), j=3,inunkp)
            else if ( iformat==2 ) then
               write(irefwr,441) r, phi, &
                  (usol(j+istart), j=1,min(2,inunkp))
               if ( inunkp>2 ) write(irefwr,451) &
                  (usol(j+istart), j=3,inunkp)
            else if ( iformat==3 ) then
               write(irefwr,442) r, phi, &
                  (usol(j+istart), j=1,inunkp)
            else
               write(irefwr,443) r, phi, &
                  (usol(j+istart), j=1,inunkp)
            end if
         end do

      end if
440   format((1x,ss,2(1pe13.5e2), &
             2('(',1pe13.5e2,',',1pe13.5e2,1x,')',2x)))
441   format((1x,ss,2(1pe16.8e2), &
             2('(',1pe16.8e2,',',1pe16.8e2,1x,')',2x)))
442   format(1x,ss, &
             99('(',1pe13.5e2,',',1pe13.5e2,1x,')',2x))
443   format(1x,ss, &
             99('(',1pe16.8e2,',',1pe16.8e2,1x,')',2x))
450   format((27x,ss, &
             2('(',1pe13.5e2,',',1pe13.5e2,1x,')',2x)))
451   format((27x,ss, &
             2('(',1pe16.8e2,',',1pe16.8e2,1x,')',2x)))

1000  call erclos ( 'prsw06c14' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsw06c14'

      end if  ! ( debug )

      end
