      subroutine prsw06c9 ( iformat, inodes, nnodes, usol, heading, &
                            coor, ndim, nunkp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-12-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the solution for subroutine PRINVC
!     complex case
!     Print coordinates
!     more than 1 unknown per point
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inodes(*), nnodes, iformat, ndim, nunkp
      double precision coor(ndim,*)
      complex * 16 usol(*)
      logical heading

!     coor           i    array of length ndim x npoint containing the
!                         coordinates of the nodal points with respect to the
!                         global numbering
!     heading        i    If true a heading must be printed
!     iformat        i    Format used for printing
!                         1: e13.5
!                         2: e16.8
!                         3: e13.5, one node per line
!                         4: e16.8, one node per line
!     inodes         i    Array containing the nodal point numbers for which
!                         printing must be performed
!     ndim           i    Dimension of the space.
!     nnodes         i    Number of nodes in inodes
!     nunkp          i    Maximum number of unknowns in a point
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPHEADING1    Print heading: coordinates values
! **********************************************************************
!
!                       I/O
!
!     Print of vector
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsw06c9' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsw06c9'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( heading ) call sepheading1(1)
      do i = 1, nnodes
         if ( ndim==1 ) then
            if ( iformat==1 ) then
               write(irefwr,420) inodes(i), coor(1,inodes(i)), &
                  (usol(j+(i-1)*nunkp), j=1,min(2,nunkp))
            else if ( iformat==2 ) then
               write(irefwr,421) inodes(i), coor(1,inodes(i)), &
                  (usol(j+(i-1)*nunkp), j=1,min(2,nunkp))
            else if ( iformat==3 ) then
               write(irefwr,422) inodes(i), coor(1,inodes(i)), &
                  (usol(j+(i-1)*nunkp), j=1,nunkp)
            else
               write(irefwr,423) inodes(i), coor(1,inodes(i)), &
                  (usol(j+(i-1)*nunkp), j=1,nunkp)
            end if
420         format(1x,i5,ss,1pe13.5e2, &
                   2('(',1pe13.5e2,',',1pe13.5e2,1x,')',2x))
421         format(1x,i5,ss,1pe16.8e2, &
                   2('(',1pe16.8e2,',',1pe16.8e2,1x,')',2x))
422         format(1x,i5,ss,1pe13.5e2, &
                   98('(',1pe13.5e2,',',1pe13.5e2,1x,')',2x))
423         format(1x,i5,ss,1pe16.8e2, &
                   98('(',1pe16.8e2,',',1pe16.8e2,1x,')',2x))
            if ( iformat==1 ) then
               if ( nunkp>2 ) write(irefwr,430) &
                  (usol(j+(i-1)*nunkp), j=3,nunkp)
            else if ( iformat==2 ) then
               if ( nunkp>2 ) write(irefwr,431) &
                  (usol(j+(i-1)*nunkp), j=3,nunkp)
            end if
430         format((19x,ss, &
                   2('(',1pe13.5e2,',',1pe13.5e2,1x,')',2x)))
431         format((19x,ss, &
                   2('(',1pe16.8e2,',',1pe16.8e2,1x,')',2x)))

         else if ( ndim==2 ) then

            if ( iformat==1 ) then
               write(irefwr,440) inodes(i), &
                  (coor(j,inodes(i)),j=1,2), &
                  (usol(j+(i-1)*nunkp), j=1,min(2,nunkp))
            else if ( iformat==2 ) then
               write(irefwr,441) inodes(i), &
                  (coor(j,inodes(i)),j=1,2), &
                  (usol(j+(i-1)*nunkp), j=1,min(2,nunkp))
            else if ( iformat==3 ) then
               write(irefwr,442) inodes(i), &
                  (coor(j,inodes(i)),j=1,2), &
                  (usol(j+(i-1)*nunkp), j=1,nunkp)
            else
               write(irefwr,443) inodes(i), &
                  (coor(j,inodes(i)),j=1,2), &
                  (usol(j+(i-1)*nunkp), j=1,nunkp)
            end if
440         format((1x,i5,ss,2(1pe13.5e2), &
                   2('(',1pe13.5e2,',',1pe13.5e2,1x,')',2x)))
441         format((1x,i5,ss,2(1pe16.8e2), &
                   2('(',1pe16.8e2,',',1pe16.8e2,1x,')',2x)))
442         format(1x,i5,ss, &
                   99('(',1pe13.5e2,',',1pe13.5e2,1x,')',2x))
443         format(1x,i5,ss, &
                   99('(',1pe16.8e2,',',1pe16.8e2,1x,')',2x))
            if ( iformat==1 ) then
               if ( nunkp>2 ) write(irefwr,450) &
                  (usol(j+(i-1)*nunkp), j=3,nunkp)
            else if ( iformat==2 ) then
               if ( nunkp>2 ) write(irefwr,451) &
                  (usol(j+(i-1)*nunkp), j=3,nunkp)
            end if
450         format((32x,ss, &
                   2('(',1pe13.5e2,',',1pe13.5e2,1x,')',2x)))
451         format((32x,ss, &
                   2('(',1pe16.8e2,',',1pe16.8e2,1x,')',2x)))

         else

            if ( iformat==1 ) then
               write(irefwr,444) inodes(i), &
                  (coor(j,inodes(i)),j=1,3), &
                  (usol(j+(i-1)*nunkp), j=1,min(1,nunkp))
               if ( nunkp>1 ) write(irefwr,460) &
                  (usol(j+(i-1)*nunkp), j=2,nunkp)
            else if ( iformat==2 ) then
               write(irefwr,445) inodes(i), &
                  (coor(j,inodes(i)),j=1,3), &
                  (usol(j+(i-1)*nunkp), j=1,min(1,nunkp))
               if ( nunkp>1 ) write(irefwr,461) &
                  (usol(j+(i-1)*nunkp), j=2,nunkp)
            else if ( iformat==3 ) then
               write(irefwr,446) inodes(i), &
                  (coor(j,inodes(i)),j=1,3), &
                  (usol(j+(i-1)*nunkp), j=1,nunkp)
            else
               write(irefwr,447) inodes(i), &
                  (coor(j,inodes(i)),j=1,3), &
                  (usol(j+(i-1)*nunkp), j=1,nunkp)
            end if
444         format((1x,i5,ss,3(1pe13.5e2), &
                   '(',1pe13.5e2,',',1pe13.5e2,1x,')',2x))
445         format((1x,i5,ss,3(1pe16.8e2), &
                   '(',1pe16.8e2,',',1pe16.8e2,1x,')',2x))
446         format(1x,i5,ss, &
                   99('(',1pe13.5e2,',',1pe13.5e2,1x,')',2x))
447         format(1x,i5,ss, &
                   99('(',1pe16.8e2,',',1pe16.8e2,1x,')',2x))
460         format((45x,ss,('(',1pe13.5e2,',',1pe13.5e2,1x,')',2x)))
461         format((45x,ss,('(',1pe16.8e2,',',1pe16.8e2,1x,')',2x)))
         end if
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsw06c9'

      end if  ! ( debug )
      call erclos ( 'prsw06c9' )

      end
