      subroutine prsw06sup ( ichoice_print, number, nunkp, isuppress, &
                             inodes, nnodes, usol, indprf, kprobf, coor, &
                             iformat, heading, ndim, icoor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 21-10-2007 Correction format
!        version  1.0  date 18-04-2005
!
!   copyright (c) 2005-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the solution for subroutine PRINVC
!     Nodes with a zero solution are skipped
!     real case
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichoice_print, number, nunkp, isuppress, inodes(*), &
              nnodes, indprf, kprobf(*), iformat, ndim, icoor
      double precision usol(*), coor(ndim,*)
      logical heading

!     coor           i    array of length ndim x npoint containing the
!                         coordinates of the nodal points with respect to the
!                         global numbering
!     heading        i    If true a heading must be printed
!     ichoice_print  i    0:   The nodal point number is printed followed by the
!                                 values in the nodal point
!                         1:   The nodal point number is printed followed by the
!                                 coordinates and then the values in the
!                                 nodal point
!                         2:   The nodal point number is printed followed by the
!                                 coordinates
!                         3:   See 2, however, preceded by the number of nodal
!                                 points to be printed
!     icoor          i    Type of coordinate system
!                         1: Cartesian
!                         2: Cylindrical
!                         3: Polar
!     iformat        i    Format used for printing
!                         1: e13.5
!                         2: e16.8
!                         3: e13.5, one node per line
!                         4: e16.8, one node per line
!     indprf         i    indication if kprob part f has been filled (>0)
!                         or not (0)
!     inodes         i    Array containing the nodal point numbers for which
!                         printing must be performed
!     isuppress      i    If 0 the node numbers are printed, &
!                         if 1 they are suppressed
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     ndim           i    Dimension of the space.
!     nnodes         i    Number of nodes in inodes
!     number         i    The number-th unknown in each nodal point of the array
!                         is considered. If the array is complex number=2*i-1
!                         corresponds to the real part of array i and
!                         number=2*i to the imaginary part of array i
!     nunkp          i    Maximum number of unknowns in a point
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ind(nnodes)

!     ind            Integer array of length icount containing the
!                    indices of the nodes to be printed
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRSW06SUP01    Print the non-zero part of the solution
!                    Special case ichoice_print=0, number>0 or nunkp=1
!     PRSW06SUP02    Print the non-zero part of the solution
!                    Special case ichoice_print=0, indprf<=0, number=0 and
!                    nunkp>1
!     PRSW06SUP03    Print the non-zero part of the solution
!                    Special case ichoice_print=0, number of unknowns not
!                    constant
!     PRSW06SUP04    Print the non-zero part of the solution
!                    Special case ichoice_print=1, only one unknown
!     PRSW06SUP05    Print the non-zero part of the solution
!                    Special case ichoice_print=1, more unknowns
!     PRSW06SUP06    Print the non-zero part of the solution
!                    Special case ichoice_print=1, number of unknowns not
!                    constant
!     PRSW06SUP07    Print the non-zero part of the solution
!                    Special case ichoice_print=1,transform to polar coordinates
!                    only one unknown
!     PRSW06SUP08    Print the non-zero part of the solution
!                    Special case ichoice_print=1,transform to polar coordinates
!                    more unknowns
!     PRSW06SUP09    Print the non-zero part of the solution
!                    Special case ichoice_print=1,transform to polar coordinates
!                    number of unknowns not constant
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsw06sup' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsw06sup'
         write(irefwr,1) 'ichoice_print, ndim, icoor', &
                          ichoice_print, ndim, icoor
         write(irefwr,1) 'number, nunkp, iformat', &
                          number, nunkp, iformat
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( ichoice_print==0 ) then

!     --- ichoice_print = 0, do not print coordinates

         if ( number>0 .or. nunkp==1 ) then

!        --- only one unknown

            call prsw06sup01 ( isuppress, inodes, usol, nnodes, &
                               iformat, ind )

         else if ( indprf<=0 ) then

!        --- more unknowns

            call prsw06sup02 ( nunkp, isuppress, inodes, nnodes, usol, &
                               iformat, heading )

         else

!        --- number of unknowns not constant

            call prsw06sup03 ( isuppress, inodes, nnodes, usol, kprobf, &
                               iformat, heading )

         end if  ! ( number>0 .or. nunkp==1 )

      else if ( icoor<=2 .or. ndim/=2 ) then

!     --- ichoice_print = 1, print coordinates

         if ( number>0 .or. nunkp==1 ) then

!        --- only one unknown
!            First store the indices of non-zero entries

            call prsw06sup04 ( inodes, nnodes, usol, coor, &
                               iformat, heading, ndim )

         else if ( indprf<=0 ) then

!        --- more unknowns

            call prsw06sup05 ( nunkp, inodes, nnodes, usol, coor, &
                               iformat, heading, ndim )

         else

!        --- number of unknowns not constant

            call prsw06sup06 ( inodes, nnodes, usol, kprobf, coor, &
                               iformat, heading, ndim )

         end if

      else

!     --- ichoice_print = 1, print coordinates and transform to polar ones

         if ( number>0 .or. nunkp==1 ) then

!        --- only one unknown

            call prsw06sup07 ( inodes, nnodes, usol, coor, &
                               heading, ndim )

         else if ( indprf<=0 ) then

!        --- more unknowns

            call prsw06sup08 ( nunkp, inodes, nnodes, usol, coor, &
                               iformat, heading, ndim )

         else

!        --- number of unknowns not constant

            call prsw06sup09 ( inodes, nnodes, usol, kprobf, coor, &
                               iformat, heading, ndim )

         end if

      end if

1000  call erclos ( 'prsw06sup' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsw06sup'

      end if

      end
