      subroutine prsw06sup05 ( nunkp, inodes, nnodes, usol, coor, &
                               iformat, heading, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-12-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the solution for subroutine PRINVC
!     Nodes with a zero solution are skipped
!     real case
!     Special case ichoice_print=1, more unknowns
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nunkp, inodes(*), nnodes, iformat, ndim
      double precision usol(*), coor(ndim,*)
      logical heading

!     coor           i    array of length ndim x npoint containing the
!                         coordinates of the nodal points with respect to the
!                         global numbering
!     heading        i    If true a heading must be printed
!     iformat        i    Format used for printing
!                         1: e13.5
!                         2: e16.8
!                         3: e13.5, one node per line
!                         4: e16.8, one node per line
!     inodes         i    Array containing the nodal point numbers for which
!                         printing must be performed
!     ndim           i    Dimension of the space.
!     nnodes         i    Number of nodes in inodes
!     nunkp          i    Maximum number of unknowns in a point
!     usol           i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      logical zero

!     i              Counting variable
!     j              Counting variable
!     zero           If true all unknowns in the node are zero
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPHEADING1    Print heading: coordinates values
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsw06sup05' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsw06sup05'

      end if
      if ( ierror/=0 ) go to 1000

      if ( heading ) call sepheading1(0)

      do i = 1, nnodes

!     --- Check if solution is zero in node i

         zero = .true.
         do j = 1, nunkp
            if ( abs(usol(j+(i-1)*nunkp))>0d0 ) zero = .false.
         end do  ! j = 1, nunkp
         if ( .not. zero ) then

!        --- Solution is not equal to zero: print

            if ( ndim==1 ) then
               if ( iformat==1 ) then
                  write(irefwr,420) inodes(i), coor(1,inodes(i)), &
                     (usol(j+(i-1)*nunkp), j=1,min(4,nunkp))
               else if ( iformat==2 ) then
                  write(irefwr,421) inodes(i), coor(1,inodes(i)), &
                     (usol(j+(i-1)*nunkp), j=1,min(4,nunkp))
               else if ( iformat==3 ) then
                  write(irefwr,422) inodes(i), coor(1,inodes(i)), &
                     (usol(j+(i-1)*nunkp), j=1,nunkp)
               else
                  write(irefwr,423) inodes(i), coor(1,inodes(i)), &
                     (usol(j+(i-1)*nunkp), j=1,nunkp)
               end if
               if ( iformat==1 ) then
                  if ( nunkp>4 ) write(irefwr,430) &
                     (usol(j+(i-1)*nunkp), j=5,nunkp)
               else if ( iformat==2 ) then
                  if ( nunkp>4 ) write(irefwr,431) &
                     (usol(j+(i-1)*nunkp), j=5,nunkp)
               end if
            else if ( ndim==2 ) then
               if ( iformat==1 ) then
                  write(irefwr,440) inodes(i), &
                     (coor(j,inodes(i)),j=1,2), &
                     (usol(j+(i-1)*nunkp), j=1,min(3,nunkp))
               else if ( iformat==2 ) then
                  write(irefwr,441) inodes(i), &
                     (coor(j,inodes(i)),j=1,2), &
                     (usol(j+(i-1)*nunkp), j=1,min(3,nunkp))
               else if ( iformat==3 ) then
                  write(irefwr,442) inodes(i), &
                     (coor(j,inodes(i)),j=1,2), &
                     (usol(j+(i-1)*nunkp), j=1,nunkp)
               else
                  write(irefwr,443) inodes(i), &
                     (coor(j,inodes(i)),j=1,2), &
                     (usol(j+(i-1)*nunkp), j=1,nunkp)
               end if
               if ( iformat==1 ) then
                  if ( nunkp>3 ) write(irefwr,450) &
                     (usol(j+(i-1)*nunkp), j=4,nunkp)
               else if ( iformat==2 ) then
                  if ( nunkp>3 ) write(irefwr,451) &
                     (usol(j+(i-1)*nunkp), j=4,nunkp)
               end if
            else
               if ( iformat==1 ) then
                  write(irefwr,440) inodes(i), &
                     (coor(j,inodes(i)),j=1,3), &
                     (usol(j+(i-1)*nunkp), j=1,min(2,nunkp))
                  if ( nunkp>2 ) write(irefwr,460) &
                     (usol(j+(i-1)*nunkp), j=3,nunkp)
               else if ( iformat==2 ) then
                  write(irefwr,441) inodes(i), &
                     (coor(j,inodes(i)),j=1,3), &
                     (usol(j+(i-1)*nunkp), j=1,min(2,nunkp))
                  if ( nunkp>2 ) write(irefwr,461) &
                     (usol(j+(i-1)*nunkp), j=3,nunkp)
               else if ( iformat==3 ) then
                  write(irefwr,442) inodes(i), &
                     (coor(j,inodes(i)),j=1,3), &
                     (usol(j+(i-1)*nunkp), j=1,nunkp)
               else
                  write(irefwr,443) inodes(i), &
                     (coor(j,inodes(i)),j=1,3), &
                     (usol(j+(i-1)*nunkp), j=1,nunkp)
               end if
            end if

         end if  ! ( .not. zero )

      end do
420   format(1x,i9,ss,5(1pe13.5e2))
421   format(1x,i9,ss,5(1pe16.8e2))
422   format(1x,i9,ss,99(1pe13.5e2))
423   format(1x,i9,ss,99(1pe16.8e2))
430   format((19x,ss,4(1pe13.5e2)))
431   format((22x,ss,4(1pe16.8e2)))
440   format(1x,i9,ss,5(1pe13.5e2))
441   format(1x,i9,ss,5(1pe16.8e2))
442   format(1x,i9,ss,99(1pe13.5e2))
443   format(1x,i9,ss,99(1pe16.8e2))
450   format((36x,ss,3(1pe13.5e2)))
451   format((42x,ss,3(1pe16.8e2)))
460   format((49x,ss,2(1pe13.5e2)))
461   format((58x,ss,2(1pe16.8e2)))

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsw06sup05'

      end if
      call erclos ( 'prsw06sup05' )

      end
