      subroutine prtoavsdim ( isol, inunkp, numveclc, numveccomps, &
                              maxvec1, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 09-04-2014 Extra parameter ndim
!        version  1.0  date 13-04-2007
!
!   copyright (c) 2007-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Define number of components and number of vectors for
!     paraview output
! **********************************************************************
!
!                       KEYWORDS
!
!     avs
!     output
!     paraview
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*), inunkp, numveclc, numveccomps(*), &
              maxvec1, ndim

!     inunkp         o    Number of degrees of freedom per point to be written
!     isol           i    Standard SEPRAN array containing information of the
!                         solution arrays
!     maxvec1        i    Maximum number of vectors in isol to be written
!     ndim           i    Dimension of the space.
!     numveclc         o    Number of vectors stored in isol
!     numveccomps    o    Array to store the number of degrees of freedom per
!                         point for each vector to be written
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     SEPACTSOLBF2   compute pointers to degrees of freedom of vectors
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2867   Value of inunkp not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Initialize parameters
!     For all vectors 1 to maxvec1 do
!        If ( vector is filled ) then
!           Determine type of vector, inunkp and number of vectors
!           Raise inunkp, numveclc and fill numveccomps
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prtoavsdim' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prtoavsdim'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Initialize parameters

      inunkp = 0
      numveclc = 0

!     --- For all vectors 1 to maxvec1 do

      do i = 1, maxvec1

         if ( ks(isol(i))%typevec==110 .or. ks(isol(i))%typevec==115 ) then

!        --- Vector has been filled and is of type 110 or 115
!            Determine type of vector, inunkp and number of vectors
!            Raise inunkp, numveclc and fill numveccomps

            call sepactsolbf2 ( isol(i) )

            if ( nphysi==1 ) then

!           --- One degree of freedom per point

               inunkp = inunkp+1
               numveccomps(numveclc+1) = 1
               numveclc = numveclc+1

            else if ( nphysi==ndim ) then

!           --- Three degrees of freedom per point
!               This must be a vector

               inunkp = inunkp+ndim
               numveccomps(numveclc+1) = ndim
               numveclc = numveclc+1

            else if ( nphysi==ndim+1 ) then

!           --- Four degrees of freedom per point
!               This must be a vector consisting of a velocity part
!               and a pressure part (type 903)

               inunkp = inunkp+ndim+1
               numveccomps(numveclc+1) = ndim
               numveccomps(numveclc+2) = 1
               numveclc = numveclc+2

            else if ( nphysi==6 ) then

!           --- Six degrees of freedom per point
!               This must be a tensor

               inunkp = inunkp+6
               numveccomps(numveclc+1) = 6
               numveclc = numveclc+1

            else if ( nphysi==7 ) then

!           --- Four degrees of freedom per point
!               This must be a vector consisting of a velocity part
!               and a pressure part (type 902)
!               Only the velocity part is written

               inunkp = inunkp+3
               numveccomps(numveclc+1) = 3
               numveclc = numveclc+1

            else

!           --- Situation not yet implemented

               call errint ( nphysi, 1 )
               call errsub ( 2867, 1, 0, 0 )
               go to 1000

            end if  ! ( nphysi==1 )

         end if  ! ( isol(i)>0 )

      end do  ! i = 1, maxvec1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'inunkp, numveclc', inunkp, numveclc
         call prinin ( numveccomps, numveclc, 'numveccomps' )
         write(irefwr,*) 'End prtoavsdim'

      end if  ! ( debug )
      call erclos ( 'prtoavsdim' )

      end
