      subroutine prtotenth1 ( npoint, veloc, enth, totenth, ipost, &
                              rpost )
! ======================================================================
!
!        programmer    Ivo Wenneker
!        version  1.0  date 31-05-1999
!
!   copyright (c) 1999-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute the total enthalpy in all points of the mesh, from the given
!    velocity and enthalpy
! **********************************************************************
!
!                       KEYWORDS
!
!     post_processing
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer          npoint, ipost(*)
      double precision veloc(2,npoint), enth(npoint), totenth(npoint), &
                       rpost(*)

!     enth           i    Array containing the enthalpy at every nodal point
!     ipost          i    Integer array containing information required for
!                         seppost
!                         See the Programmers Guide for a description
!     npoint         i    Number of nodal points
!     rpost          i    Real array containing information required for
!                         seppost
!                         See the Programmers Guide for a description
!     totenth        o    Array containing the total enthalpy at every
!                         nodal point
!     veloc          i    Array containing the velocity
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer          i, idimchoice
      double precision rconst, velocsq, spgamma, machref

!     i              Counting variable
!     idimchoice     Dimensionless (0) or dimensionful computation (1)
!     machref        Reference Mach number
!     rconst         Helping constant
!     spgamma        Specific heat ratio
!     velocsq        Squared velocity in a nodal point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The total enthalpy, using dimensionless quantities, is defined by
!
!     H = h + 0.5*(gamma-1)*machref^2*velocity^2
!
!     The total enthalpy, using dimensionful quantities, is defined by
!
!     H = h + 0.5*velocity^2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      idimchoice = ipost(6)

!     --- Dimensionless input parameters

      if ( idimchoice.eq.0 ) then
         spgamma = rpost(1)
         machref = rpost(2)

         rconst = 0.5d0*(spgamma-1.0d0)*machref**2

      else

!     --- Dimensionful input pararameters

         rconst = 0.5d0

      end if

      do i = 1, npoint
         velocsq    = veloc(1,i)**2 + veloc(2,i)**2
         totenth(i) = enth(i) + rconst*velocsq
      end do

      end
