      subroutine prvec0r ( vector, inppri, text, infovec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 09-06-2010 Change of layout
!        version  1.1  date 28-07-2001 Extra option iformat
!        version  1.0  date 17-03-1999
!
!   copyright (c) 1999-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Print vector of special structure defined per element per integration point
!   It concerns vectors of type 129, real case
! **********************************************************************
!
!                       KEYWORDS
!
!     real
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inppri(*), infovec(4,*)
      double precision vector(*)
      character *(*) text

!     infovec    i    Contains first integer part of vector of structure 129
!                     This part has length 4*nelgrp
!                     contents:
!                     (1,ielgrp): number of integration points
!                     (2,ielgrp): number of unknowns per point
!                     (3,ielgrp): number of elements in group
!                     (4,ielgrp): starting address of first element in
!                                 real part minus 1
!     inppri     i    Input array with respect to PRINVC (see PRINVC)
!     text       i    String variable containing some text
!     vector     i    Array of type 126 to be printed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, i, ikelmc, ielem, irelem, &
              ielgrp, nunkp, m, iformat
      logical heading

!     heading       If true a heading must be printed
!     i             Counting variable
!     ielem         Element number
!     ielgrp        element group number
!     iformat       Format used for printing
!                   1: e13.5
!                   2: e16.8
!     ikelmc        position in kmesh part c
!     irelem        Relative element sequence number with respect to element
!                   group
!     k             Counting variable
!     m             number of integration points
!     nunkp         Maximum number of unknowns in a point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    For all element groups do
!       For all elements do
!          print part of vector
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      heading = inppri(8)==0
      iformat = inppri(10)

!     --- Write heading

      if ( heading ) then
         write(irefwr,100) text
100      format(/' vector ',a/)
      else
         write(irefwr,*)
      end if
      if ( heading ) then
         write(irefwr,110)
110      format ( 'element integrat. point                values ' )
      end if
      ikelmc = 0
      ielem = 0

!     --- For all element groups do

      do ielgrp = 1, nelgrp

         inelem = Topgrp(ielgrp)%inelem
         nunkp = infovec(2,ielgrp)
         m = infovec(1,ielgrp)

!        --- For all elements do

         do irelem = 1, inelem

            ielem = ielem + 1

!           --- print part of vector

            if ( iformat==1 ) then
               write(irefwr,200) ielem, 1, &
                  (vector(ikelmc+k),k=1,min(5,nunkp))
               if ( nunkp>5 ) write(irefwr,210) &
                  (vector(ikelmc+k),k=6,nunkp)
            else
               write(irefwr,201) ielem, 1, &
                  (vector(ikelmc+k),k=1,min(5,nunkp))
               if ( nunkp>5 ) write(irefwr,211) &
                  (vector(ikelmc+k),k=6,nunkp)
            end if
200         format(i6, i4, ss,5(1pe13.5e2))
201         format(i6, i4, ss,5(1pe16.8e2))
210         format((10x, ss,5(1pe13.5e2)))
211         format((10x, ss,5(1pe16.8e2)))
            ikelmc = ikelmc + nunkp
            do i = 2, m
               if ( iformat==1 ) then
                  write(irefwr,220) i, &
                     (vector(ikelmc+k),k=1,min(5,nunkp))
                  if ( nunkp>5 ) write(irefwr,210) &
                     (vector(ikelmc+k),k=6,nunkp)
               else
                  write(irefwr,221) i, &
                     (vector(ikelmc+k),k=1,min(5,nunkp))
                  if ( nunkp>5 ) write(irefwr,211) &
                     (vector(ikelmc+k),k=6,nunkp)
               end if
220            format(6x, i4, ss,5(1pe13.5e2))
221            format(6x, i4, ss,5(1pe16.8e2))
               ikelmc = ikelmc + nunkp
            end do
         end do

      end do
      end
