      subroutine sep2odx ( isol, time, &
                           jtime, iseqlc, ifstodx, ilstodx, namedx )
! ======================================================================
!
!        programmer    Bas van Rens
!        version  1.4  date 20-06-2013 New definition file ref numbers
!        version  1.3  date 30-11-2012 Long integers
!        version  1.2  date 10-02-2012 use getnpelmelgrp
!
!   copyright (c) 2000-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write solution array or array of special structure to file sepodx_xxx.yyy
!     This possibility is meant for a SEPRAN-OPENDX interface
!     The SEPRAN mesh is mapped onto a mesh of linear triangles or tetrahedrons
!     only
! **********************************************************************
!
!                       KEYWORDS
!
!     open_dx
!     output
!     vector
!     write
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulefiles
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*), jtime, iseqlc, &
              ifstodx, ilstodx, namedx
      double precision  time

!     ifstodx        i    First solution vector to be printed to odx
!     ilstodx        i    Last solution vector to be printed to odx
!     iseqlc           i    Sequence number
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     jtime          i    Integer time sequence number
!     namedx         i    Sequence number of name of open dx file in array
!                         file_names
!     time           i    Actual time
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp, ncell, nucdcell, nchar, ipc, &
              icell, iref, i, j, ielem, &
              nnod, loctop(27), ncellnod, cellmap(27), &
              iodxvec, iphys, ipos
      double precision lsol(100)
      character *(40) filenm
      character *(100) filename
      character *(12) cellname, oldname

!     cellmap        Matrix containing the mapping of one element to
!                    one or more linear cells.  E.g. a the mapping of
!                    a quadratic brick into 8 linear bricks
!     cellname       Name of the cells to be written. E.g. 'quad'
!                    global numbering
!     filename       Name of file
!     filenm         Name of file
!     i              Counting variable
!     icell          Cell number of cell under consideration
!     ielem          Element number
!     ielgrp         element group number
!     iodxvec        index of the current solution vector to be written
!                    to odx
!     ipc            pointer in kmesh part c
!     iphys          Number of physical variables
!     ipos           Position in array
!     iref           File reference number
!     j              Counting variable
!     loctop         local topology of an element
!     lsol           solution vector associated with one point
!     ncell          number of cells in an element
!     ncellnod       number of nodes in a cell
!     nchar          Number of characters in a word
!     nnod           number of nodes in an element
!     nucdcell       total number of cells to be written
!     oldname        old name of a cell
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: getucdnr
      integer :: inisetnewref

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     GETUCDINF      compute cell info for a given element
!     GETUCDNR       compute number of cells in an element
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     INISETNEWREF   Create new reference number
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     720   Array ISLOLD has not been filled.
!    2185   Not all converted cells of same type
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sep2odx' )

!     --- Open file sepodx_xxx_yyy.dx

      iref = inisetnewref()
      if ( namedx==0 ) then

!     --- namedx = 0, make file name default

         filename = 'sepodx_'
         nchar = 7

      else

!     --- namedx>0, use file name from cfilenames

         filename = file_names(namedx)
         nchar = index ( filename, ' ' )
         filename(nchar:nchar) = '_'

      end if
      write ( filenm, 50 ) filename(1:nchar), jtime, iseqlc
 50   format ( a,i3.3,'_',i3.3,'.dx' )
      open ( unit=iref, file=filenm )

!     --- deal with solution vector

      do i = ifstodx, ilstodx
         if ( ks(isol(i))%typevec/=110 .and. &
              ks(isol(i))%typevec/=115 .and. &
              ks(isol(i))%typevec/=116 ) then

!        --- Error input vector not filled

            call errint ( i, 1)
            call errint ( ks(isol(i))%typevec, 2 )
            call errsub ( 720, 2, 0, 0 )
            go to 1000

         end if

      end do

!     --- compute number of cells

      nucdcell = 0
      ishape = kmeshf ( 1 )
      call getucdinf(ishape, 1, cellmap, ncellnod,cellname)
      oldname = cellname
      do ielgrp = 1, nelgrp
         inelem = Topgrp(ielgrp)%inelem
         ishape = kmeshf ( ielgrp )
         call getucdinf(ishape, 1, cellmap, ncellnod,cellname)
         if ( cellname/=oldname ) then

!        --- Error Not all converted cells of same type

            call errchr ( oldname, 1)
            call errchr ( cellname, 2 )
            call errsub ( 2185, 0, 0, 2 )
            go to 1000

         end if
         oldname = cellname
         if (ishape==-10001)ishape=1
         ncell = getucdnr(ishape)
         nucdcell = nucdcell + inelem*ncell
      end do

      write(iref,100) time
100   format ( '#   SEPRAN - ODX OUTPUT FILE'/ &
               '#'/ &
               '#    time level ', f12.3/ &
               '#' )

!     --- Write co-ordinates to file

      write(iref,200)'object "coordinates"'
      write(iref,200)'   class         array'
      write(iref,200)'   type          float'
      write(iref,205)'   rank       ',1
      write(iref,205)'   shape      ',ndim
      write(iref,205)'   items      ',npoint
      write(iref,205)'   data        follows'
      do i = 1, npoint
         write(iref,210)'       ',coor(:,i)
      end do

!     --- Write topology
!         determine shape based on first group
!         assume each group has same type of elements
!         (this can be improved)

      ishape = kmeshf ( 1 )
      call getucdinf(ishape, 1, cellmap, ncellnod,cellname)

      write(iref,200)'object "elements"'
      write(iref,200)'   class         array'
      write(iref,200)'   type            int'
      write(iref,205)'   rank       ',1
      write(iref,205)'   shape      ',ncellnod
      write(iref,205)'   items      ',nucdcell
      write(iref,215)'   attribute  "element type" string ', &
                     cellname
      write(iref,205)'   data        follows'
      nucdcell = 0
      ipc = 1
      do ielgrp = 1, nelgrp

!     --- initialize

         inelem = Topgrp(ielgrp)%inelem
         ishape = kmeshf ( ielgrp )
         nnod = Topgrp(ielgrp)%inpelm
         if (ishape==-10001)ishape=1
         ncell = getucdnr(ishape)
         call getucdinf(ishape, 1, cellmap, ncellnod, &
                         cellname)
         do ielem = 1, inelem
            loctop(1:nnod) = kmeshc(ipc:ipc+nnod-1)
            ipc = ipc+nnod
            do icell =1, ncell
               nucdcell = nucdcell+1
               call getucdinf(ishape, icell, cellmap, ncellnod, &
                         cellname)
               write(iref,220)'              ', &
                              (loctop(cellmap(i))-1,i=1,ncellnod)
            end do
         end do
      end do

!     --- Write information of solution to file

      do iodxvec = ifstodx, ilstodx

         call sepactsolbf2 ( isol(iodxvec) )

         write(iref,225)'object "solution',iodxvec,'"'
         write(iref,200)'   class          array'
         write(iref,200)'   type          double'
         write(iref,205)'   rank       ',1
         write(iref,205)'   shape      ',nphysi
         write(iref,205)'   items      ',npoint
         if (ks(isol(iodxvec))%typevec==116) then
            write(iref,200)'   attribute     "dep" string "connections"'
         else
            write(iref,200)'   attribute     "dep" string "positions"'
         end if
         write(iref,205)'   data         follows'
         usoltemp => ks(isol(iodxvec))%sol

         do i = 1, npoint
            do iphys = 1, nphysi

               if (indprpi/=0) then
                  ipos = kprobpi ( i, iphys )
               else
                  ipos = (i-1) * nphysi + iphys
               end if
               if (ipos/=0) then
                  lsol(iphys) = usoltemp(ipos)
               else
                  lsol(iphys) = 0.0d0
               end if
            end do

            write(iref,230)'           ',(lsol(iphys),iphys=1,nphysi)
         end do
      end do

      if (ilstodx<ifstodx) then

!     --- no solution fields exist, create a dummy one

         write(iref,225)'object "solution',iodxvec,'"'
         write(iref,200)'   class constantarray'
         write(iref,200)'   type         double'
         write(iref,205)'   rank       ',1
         write(iref,205)'   shape      ',1
         write(iref,205)'   items      ',npoint
         write(iref,200)'   attribute     "dep"     string "positions"'
         write(iref,205)'   data        follows'
         write(iref,230)'       ',0.0d0
      end if

!     --- construct the fields

      do iodxvec = ifstodx, max(ifstodx, ilstodx)
         write(iref,225)'object "field',iodxvec,'"'
         write(iref,200)'   class         field'
         write(iref,200)'   component "positions"   value "coordinates"'
         write(iref,200)'   component "connections" value "elements"'
         write(iref,225)'   component "data"        value "solution', &
                            iodxvec,'"'
      end do

!     --- put everything in a group

      write(iref,225)'object "everything"'
      write(iref,225)'   class         group'
      do iodxvec = ifstodx, max(ifstodx, ilstodx)
         write(iref,235)'   member    "sol',iodxvec, &
                        '"        value "field',iodxvec,'"'
      end do

      write(iref,200)'end'

      close ( unit=iref )
      call inirmref ( iref )

 200  format (A)
 205  format (A,I8)
 210  format (A,3E15.7)
 215  format (A,A)
 220  format (A,27i8)
 225  format (A,I1,A)
 230  format (A,30E15.7)
 235  format (A,I1,A,I1,A)

1000  call erclos ( 'sep2odx' )

      end
