      subroutine sepassemstreac
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-11-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Reserve space in array IBUFFR with respect to subroutine sepcompreac
!     Fill array ipoint in common CSUBWK
!     Old subroutine to0103
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     integral
!     memory_management
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulesmf
      use sepmodulemain
      use sepmodulecpack
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduleelm
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, mcount, marray, miarry, liarry, larray, mindex

!     i           Counting variable
!     larray      length of array array in element subroutines
!     liarry      length of the integer array iarray
!     marray      Maximum of larray over all element groups
!     mcount      Maximum of icount over all element groups
!     miarry      Maximum of liarry over all element groups
!     mindex      Help parameter for the length of array index3
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELMGRINF       Fill general information for a specific element group
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     TO0063         Compute NINDEX
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepassemstreac' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepassemstreac'
         write(irefwr,1) 'numold', numold
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      oldsols(1)%nunkp = nunkp

      mindex = 0
      mcount = 0
      marray = 0
      miarry = 0

!     --- Loop over element groups

      do ielgrp = 1, nelgrp

         itype = kprobc(ielgrp)
         inelem = sepgetinelem(ielgrp)
         inpelm = sepgetinpelm(ielgrp)
         if ( itype>0 .and. skipelgrps(ielgrp)>0 ) then

!        --- element group does not have to be skipped
!            compute number of elements in element vector

            icount = sepfillcountvc ( isolut2d(iseqi1), ielgrp )
            mcount = max ( mcount, icount )

            call to0063 ( isolut2d )
            mindex = max ( mindex, nindex )

            call elmgrinf ( larray, liarry, kmeshm, ndim )
            marray = max ( marray, larray )
            miarry = max ( miarry, liarry )

            if ( debug ) &
               write(irefwr,1) 'icount, mindex, marray, miarry', &
                                icount, mindex, marray, miarry

         end if

      end do

!     --- Store values of mindex and mcount

      nindex = mindex
      icount = mcount

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepassemstreac'

      end if  ! ( debug )
      call erclos ( 'sepassemstreac' )

      end subroutine sepassemstreac
