      subroutine sepcompderint
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-12-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute integer quantities per element using one or more
!     input vectors
!     Old subroutine to0047int
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduleelm
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     SEPASSEMSTREAC Reserve space in array IBUFFR with respect to
!                    subroutine sepcompreac
!                    Fill array ipoint in common CSUBWK
!     SEPDERINTBODY  Body of subroutine sepcompderint
!     SEPDERINTSTART Perform some tests for subroutine sepcompderint
!     SEPREACSTART1  Start of assembly process
!                    Make all initializations, fill modules and declare arrays
!     SEPRMMATSTR2   nullify arrays in module sepmodulematstr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcompderint' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcompderint'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Perform some checks
!         Make initializations

      call sepderintstart
      if ( ierror/=0 ) go to 1000

!     --- If there are no unknowns the matrix must not be filled and
!         everything is skipped

      if ( nusol-nbound-npboun==0 ) go to 1000

!     ---  Start of assembly process
!          Make some initializations, fill modules and declare arrays

      call sepreacstart1
      if ( ierror/=0 ) go to 1000

!     --- Initialize some parameters for the element groups

      call sepassemstreac (  E )

!     --- compute the output vector

      call sepderintbody

      deallocate ( istold, istold1, oldsols, indoldsols, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'istoldlc' )

      call seprmmatstr2  ! nullify arrays in modulematstr

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcompderint'

      end if  ! ( debug )
      call erclos ( 'sepcompderint' )

      end subroutine sepcompderint
