      subroutine sepderaverage
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute average of derived quantities by dividing the
!     output vector by the weights
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     average
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     TO0020C        apply averaging by division by weights (Complex version)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepderaverage' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepderaverage'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( complo ) then

!     --- Complex case

         call to0020c ( nusolo, vectout, weight )

      else

!     --- Real case

         do i = 1, nusolo
            if ( weight(i)>epsmac ) vectout(i)=vectout(i)/weight(i)
         end do

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepderaverage'

      end if  ! ( debug )
      call erclos ( 'sepderaverage' )

      end subroutine sepderaverage
