      subroutine sepderbody
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-12-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Body of subroutine sepcompder
!     Compute quantities per element using one or more
!     input vectors
!     Old subroutine to0047
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulemain
      use sepmodulesmf
      use sepmodulebuild
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecpack
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ikelmc, ipkproba, ih
      logical :: fillweight

!     fillweight     if true the weight vector must be filled
!     ih             Contains last address used in output vector
!     ikelmc         position in kmesh part c
!     ipkproba       start address in kprob part a
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPCOPYELEM    Copy information of elements in type element
!     SEPDERASSEM    add element vector to large vector of derived quantities
!     SEPDERAVER     Interpolate values in global vertices in region by using
!                    the mean value of two neighboring nodes
!     SEPDERAVERAGE  Compute average of derived quantities by dividing the
!                    output vector by the weights
!     SEPDERELGRP    Initialize some parameters for one element group
!     SEPDERPAR       Add vector and weight for common sides in parallel case
!     SEPDERPERIOD   Adapt the derivative for periodic boundary conditions
!     SEPDERVDALLOC  De-allocate temporary arrays in for subroutine to0112
!     SEPDERVINDEX   Fill arrays index2, index3 and index4 for subroutine deriv
!     SEPELDERV      Compute element vector and weights for derivatives
!     SEPNULLIFYBASE Nullify all pointers associated with B
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     211   notvec = 1  for relative element number more than 1
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepderbody' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepderbody'
         write(irefwr,1) 'nelgrp, SMF%ioutvc', nelgrp, SMF%ioutvc
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ielem  = 0
      ikelmc = 0
      posout = 0
      ipkproba = 1
      icompl = 1

      iweivert = 0

      B => B1
      call sepnullifybase  ! nullify all pointers in B

      if ( complo ) icompl = 2

!     --- loop over element groups

      do ielgrp = 1, nelgrp

         if ( indpra>0 ) kprobatmp => kproba(ielgrp)%arr
         inelem = sepgetinelem(ielgrp)
         inpelm = sepgetinpelm(ielgrp)
         itype = kprobc(ielgrp)
         if ( itype<=0 .or. skipelgrps(ielgrp)==0 ) then

!        --- element group ielgrp must be skipped
!            Raise parameters

            ikelmc = ikelmc+inpelm*inelem
            ielem = ielem+inelem

         else

            ifirst = 0
            vector = .true.
            fillweight = ichderiv<=1 .and. &
                         itype/=615 .and. itype/=616 .and. &
                         itype/=604 .and. itype/=603 .and. &
                         itype/=607 .and. itype/=608

!           --- Compute number of unknowns in element vector

            icouno = sepfillcountvc ( SMF%ioutvc, ielgrp )

            if ( debug ) &
               write(irefwr,1) 'ielgrp, ikelmc, icouno, inpelm', &
                                ielgrp, ikelmc, icouno, inpelm

!           --- Fill information for element group in E and allocate some
!               element arrays

            call sepderelgrp
            if ( debug ) write(irefwr,1) 'inelem', inelem

!           --- loop over all elements of this group

            do irelem = 1, inelem

               ielem  = ielem + 1
               E%ielem = ielem
               E%irelem = irelem
               E%first = ifirst

               index1 => kmeshc(ikelmc+1:ikelmc+inpelm)

!              --- Fill arrays index2/3/4/5

               call sepdervindex ( ikelmc, ipkproba, SMF%ioutvc )

!              --- Copy information into type elem

               call sepcopyelem

!              --- Create element vector as well as weight vector

               if ( irelem==inelem ) E%last=1  ! mark last element
               call sepelderv
               if ( ierror/=0 ) go to 1000

               if ( notvec/=0 ) then

!              --- skip other elements of this group since the element vector
!                  is zero

                  if ( irelem>1 ) then
                     call errint ( ielgrp, 1 )
                     call errint ( irelem, 2 )
                     call errsub ( 211, 2, 0, 0 )
                     go to 1000
                  end if
                  ikelmc = ikelmc+inpelm*inelem
                  ielem = ielem+inelem-1
                  go to 500

               end if

!              --- add element element vector and weight to large vectors

               elemvc2 => elemvc1
               vectout2 => vectout
               call sepderassem ( complo )
               elemvc2 => elemwg
               vectout2 => weight
               if ( fillweight ) call sepderassem ( .false. )
               ikelmc = ikelmc+inpelm
               if ( ichtst/=7 ) then
                  posout = ikelmc
               else
                  posout = posout+m
               end if  ! ( ichtst/=7 )
            end do  ! irelem = 1, inelem

         end if  ! ( itype<=0 .or. skipelgrps(ielgrp)==0 )

!        --- Deallocate temporary arrays

500      if ( debug ) write(irefwr,*) 'before sepdervdalloc'
         call sepdervdalloc
         if ( ierror/=0 ) go to 1000
         ipkproba = ipkproba+inpelm

      end do  ! ielgrp = 1, nelgrp

!     --- special care for periodical boundaries when itypvc = 115 (ichderiv=1)

      if ( ichderiv==1 .and. nelgrp>1 ) call sepderperiod

!     --- In case of parallel computing we have to add common quantities
!         at common sides of parallele blocks

      if ( parallel ) call sepderpar ( fillweight )

!     --- apply weights : averaging for itypvo = 110, 115
!         i.e. vectout := vectout/weight
!         In case a weight is 0, no division

      if ( debug ) write(irefwr,*) 'before sepderaverage'
      if ( fillweight ) call sepderaverage

!     --- If iweivert=1, special situation, the global vertex nodes must be
!         interpolated from their neighbors

      if ( debug ) write(irefwr,*) 'before sepderaver'
      if ( iweivert==1 ) call sepderaver

      if ( debug ) write(irefwr,*) 'before deallocate'
      if ( associated(weight) ) deallocate ( weight, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'weight' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepderbody'

      end if  ! ( debug )
      call erclos ( 'sepderbody' )

      end subroutine sepderbody
