      subroutine sepderpar ( fillweight )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Add vector and weight for common sides in parallel case
!
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     average
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulebuild
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical, intent(in) :: fillweight

!     fillweight     i    if true the weight vector must be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRPARADDVEC    Add common parts of a SEPRAN solution vector or vector of
!                    special structure in case of parallel program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2339   The combination complex and parallel has not yet been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepderpar' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepderpar'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( complo ) then

!     --- Complex case has not yet been implemented

         call errsub ( 2339, 0, 0, 0 )
         go to 1000

      else

!     --- add solution vector of common sides of blocks

         call prparaddvec ( vectout, indprfo, kprobfo, &
                            nunkpo, nusolo, npoint, &
                            blockinfo, transinfo )
         if ( fillweight ) &
            call prparaddvec ( weight, indprfo, &
                               kprobfo, nunkpo, nusolo, &
                               npoint, blockinfo, transinfo )

      end if

1000  call erclos ( 'sepderpar' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepderpar'

      end if  ! ( debug )

      end subroutine sepderpar
