      double precision function sepelint ( )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-06-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute integral over element
!
! **********************************************************************
!
!                       KEYWORDS
!
!     integral
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

!     sepelint       o    Result of integration in element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, pointer :: iuser(:), islold(:)
      double precision, pointer :: user(:)

!     islold         User input array in which the user puts information
!                    of all preceding solutions
!                    Integer array of length 5 x numold
!     iuser          Integer user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
!     user           Real user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision :: sepelisubs

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELISUBS     Compute value of integral for one element
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelint' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelint'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iuser => E%iuser
      user => E%user
      islold => E%islold

      if ( debug ) write(irefwr,*) 'before sepelisubs'

      sepelint = sepelisubs ( )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelint'

      end if  ! ( debug )
      call erclos ( 'sepelint' )

      end function sepelint
