      subroutine sepfillfuncy ( iseqvecs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array funcy  with function values for a series of sepran vectors
!
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     vector
!     sepran_vector
!     user_vector
!     solution_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplotfn
      use sepmodulemain
      use sepmoduleint2d
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iseqvecs(numvecs)

!     iseqvecs       i    Contains sequence numbers of vectors with respect to
!                         isolut
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPMAPSOL1D    Interpolate one degree of freedom of sepran vector
!                    in user vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillfuncy' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillfuncy'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, numvecs
         call sepmapsol1d ( isolut2d(iseqvecs(i)), funcy(1,i) )
      end do  ! i = 1, numvecs

1000  call erclos ( 'sepfillfuncy' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( funcx, numint, 'funcx' )
         call prinrl1 ( funcy, numint, numvecs, 'funcx' )
         write(irefwr,*) 'End sepfillfuncy'

      end if  ! ( debug )

      end subroutine sepfillfuncy
