      subroutine sepfilltext ( loctext, inpsep, rinsep, &
                               numvecs, seqvecs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array loctext with the text to be used along axis
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     text
!     axis
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmodulekmesh
      use sepmoduleplot
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*), numvecs, seqvecs(numvecs)
      double precision, intent(in) :: rinsep(*)
      character (len=*) :: loctext(2)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         Contents
!                         1: Position of text along x-axis in text array
!                         2: Position of text along y-axis in text array
!     loctext        o    Contains the texts to be plotted along the axis
!     numvecs        i    Number of vectors in seqvecs
!     rinsep         i    Array containing the real input for this subroutine
!     seqvecs        i    Contains the sequence numbers of the vectors in isol
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: n, numchar, istart, lentext, k

!                    they are not
!     istart         First position used at a part
!     k              Counting variable
!     lentext        Length of loctext
!     n              Position in string
!     numchar        Number of characters
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPNUMCHAR     Translate number into text using as few positions as
!                    possible
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfilltext' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfilltext'
         call prinin ( inpsep, 5, 'inpsep' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      lentext = len(loctext(1))

      if ( inpsep(6)>0 ) then

!     --- text given by user

         loctext(1) = texts(inpsep(6))
         ntextx = inpsep(2)

      else

!     --- more dimensions, use line

         n = 7
         loctext(1)(1:n)  = 'line (('

         istart = inpsep(5)

         do k = 1, 2

            call sepnumchar ( rinsep(istart), loctext(1)(n+1:lentext), &
                              numchar )
            n = n+numchar+1
            loctext(1)(n:n) = ','

            call sepnumchar ( rinsep(istart+1),  loctext(1)(n+1:lentext), &
                              numchar )
            n = n+numchar+1

            if ( ndim==3 ) then

!           --- ndim = 3

               loctext(1)(n:n) = ','

               call sepnumchar ( rinsep(istart+2),  loctext(1)(n+1:lentext), &
                                 numchar )
               n = n+numchar+1

            end if  ! ( ndim==3 )
            loctext(1)(n:n) = ')'

            n = n+1

            if ( k==1 ) then

!           --- First set of coordinates

               istart = istart+ndim
               loctext(1)(n:n) = ','
               n = n+1

            end if  ! ( k==1 )

         end do  ! k = 1, 2

         loctext(1)(n:n) = ')'
         ntextx = n

      end if  ! ( inpsep(1)>0 ) then

      if ( inpsep(9)>0 ) then

!     --- text given by user

         loctext(2) = texts(inpsep(9))
         ntexty = inpsep(4)

      else if ( numvecs>1 ) then

!     --- more vectors, use default value

         loctext(2) = 'value'
         ntexty = 5

      else

!     --- one vector, use vector name

         loctext(2) = names(seqvecs(1))
         ntexty = index(loctext(2),' ')-1

      end if  ! ( inpsep(1)>0 ) then

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'loctext(1) ', loctext(1)(1:ntextx)
         write(irefwr,*) 'loctext(2) ', loctext(2)(1:ntexty)
         write(irefwr,*) 'End sepfilltext'

      end if  ! ( debug )
      call erclos ( 'sepfilltext' )

      end
