      subroutine sepintbody ( outval, iworkel )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-06-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Body of subroutine to0085
!     Compute quantities per element using one or more
!     input vectors
!     Old subroutine to0085
! **********************************************************************
!
!                       KEYWORDS
!
!     integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulesmf
      use sepmodulebuild
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecpack
      use sepmoduleoldvecs
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision :: outval(*)
      integer :: iworkel(*)

!     iworkel        i    Work array of length nelem to indicate if an element
!                         may be used (0) or must be skipped (1)
!     outval         o    In this array the computed integral is stored as well
!                         as the minimal and maximum values per element if
!                         MAXMIN>0:
!                         OUTVAL(1): integral
!                         OUTVAL(2): minimum value of the integrals per element
!                         OUTVAL(3): maximum value of the integrals per element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ikelmc, ipkproba
      double precision value, elvlmn, elvlmx, volint, help(2)

!     elvlmn         Minimum value of the integral over one element
!     elvlmx         Maximum value of the integral over one element
!     help           Work array to store some variables to be sent globally
!     ikelmc         position in kmesh part c
!     ipkproba       start address in kprob part a
!     value          value of the integral over one element
!     volint         value of the complete integral
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision :: sepelint

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRPARGLMAX     Performs a global maximum of a quantitity
!                    or vector across all processes
!     PRPARGLSUM     This function performs a global sum of a quantitity
!                    or vector across all processes
!     SEPCOPYELEM    Copy information of elements in type element
!     SEPELINT       Compute integral over element
!     SEPINTDALLOC   De-allocate temporary arrays in for subroutine sepintbody
!     SEPINTELGRP    Initialize some parameters for one element group
!     SEPINTINDEX    Fill arrays index2, index3 and index4 for integr
!     SEPNULLIFYBASE Nullify all pointers associated with B
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepintbody' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepintbody'
         write(irefwr,1) 'nelgrp', nelgrp
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kproba => kprobpart%kproba
      indpra = kprobloc(14)
      kprobc => kprobpart%kprobc

!     --- initialize parameters

      volint = 0d0
      elvlmn =  rinfin
      elvlmx = -rinfin

      ielem  = 0
      ikelmc = 0
      ipkproba = 1
      icompl = 1
      E%imas = 0

      B => B1
      call sepnullifybase  ! nullify all pointers in B
      call sepintdalloc

      if ( complo ) icompl = 2

      iseqprevsol = iseqin
      iseqi1 = iseqprevsol
      iseqi2 = iseqprevsol

!     --- loop over element groups

      do ielgrp = 1, nelgrp

         if ( indpra>0 ) kprobatmp => kproba(ielgrp)%arr
         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm
         itype = kprobc(ielgrp)

         if ( debug ) &
            write(irefwr,1) 'ielgrp, inelem, inpelm, skipelgrps(ielgrp)', &
                             ielgrp, inelem, inpelm, skipelgrps(ielgrp)
         if ( itype<=0 .or. skipelgrps(ielgrp)==0 ) then

!        --- element group ielgrp must be skipped
!            Raise parameters

            ikelmc = ikelmc+inpelm*inelem
            ielem = ielem+inelem

         else

            ifirst = 0

!           --- Fill information for element group in E and allocate some
!               element arrays

            call sepintelgrp
            if ( debug ) write(irefwr,1) 'inelem', inelem

!           --- loop over all elements of this group

            do irelem = 1, inelem

               ielem  = ielem + 1
               if ( iworkel(ielem)==1 ) go to 400

               E%ielem = ielem
               E%irelem = irelem
               E%first = ifirst

               index1 => kmeshc(ikelmc+1:ikelmc+inpelm)

!              --- Fill arrays index2/3/4

               call sepintindex ( ikelmc, ipkproba )

!              --- Copy information into type elem

               call sepcopyelem

!              --- Create element vector as well as weight vector

               if ( irelem==inelem ) E%last=1  ! mark last element
               value = sepelint ( )
               if ( ierror/=0 ) go to 1000

!              --- add value to integral

               volint = volint+value
               if ( maxmin==2 ) value = abs(value)
               if ( maxmin>=1 ) then

!              --- maxmin = 1,2 compute minimum and maximum value

                  elvlmn = min ( elvlmn, value )
                  elvlmx = max ( elvlmx, value )

               end if
               ifirst = 1

               if ( debug ) write(irefwr,2) 'value, volint', value, volint

400            ikelmc = ikelmc+inpelm

            end do  ! irelem = 1, inelem

         end if  ! ( itype<=0 .or. skipelgrps(ielgrp)==0 )

!        --- Deallocate temporary arrays

         if ( debug ) write(irefwr,*) 'before sepintdalloc'
         call sepintdalloc
         if ( ierror/=0 ) go to 1000

         ipkproba = ipkproba+inpelm

      end do  ! ielgrp = 1, nelgrp

!     --- Fill array outval

      if ( parallel ) then

!     --- Parallel computing: perform some global operations
!         First sum all local integrals

         call prparglsum ( volint, outval(1), 1, SPDOUBLE )
         if ( maxmin>0 ) then

!        --- maxmin>0
!            Compute global maxima

            help(1) = elvlmn
            help(2) = elvlmx
            call prparglmax ( help, outval(2), 2, SPDOUBLE )

         end if

      else

!     --- Serial computing: standard

         outval(1) = volint
         if ( maxmin>0 ) then

!        --- maxmin>0

            outval(2) = elvlmn
            outval(3) = elvlmx

         end if

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepintbody'

      end if  ! ( debug )
      call erclos ( 'sepintbody' )

      end subroutine sepintbody
