      subroutine sepintelgrp
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-06-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Initialize some parameters for one element group
!     Help subroutine for integr
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulebuild
      use sepmoduleelm
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: larray, lenindex3, lenarray

!     larray         length of array coeffarray in element subroutines
!     lenarray       Length of array coeffarr
!     lenindex3      Length of array index3
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELIGRINF       determine ielrou, larray for integration element routines
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPINTELINIT   initialize some parameters for one element group in
!                    subroutine integr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepintelgrp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepintelgrp'
         write(irefwr,1) 'icouno', icouno
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Initialize some parameters for one element group

      call sepintelinit

      if ( itype<=0 ) go to 1000  ! skip element group

!     --- allocate arrays for element subroutines
!         In this case icount is correct

      larray = 1

!     --- Compute some parameters depending on the type number

      call eligrinf ( larray )
      lenarray = larray+7425

      allocate ( indoldsols(1)%index3(icount), stat=error )
      if ( error/=0 ) call eralloc ( error, inpelm, 'index4' )
      if ( ierror/=0 ) go to 1000

      error = 0
      allocate ( coeffarr(lenarray), stat=error )
      if ( error/=0 )  call eralloc ( error, lenarray, 'coeffarr' )


      allocate ( index2(icount), stat=error )
      if ( error/=0 )  call eralloc ( error, icount, 'index2' )
      allocate ( index4(max(1,numold),inpelm+1), stat=error )
      if ( error/=0 )  call eralloc ( error, icount, 'index4' )
      if ( ierror/=0 ) go to 1000

      indoldsols(1)%index3 => index2

      lenindex3 = max(nindex+inpelm, inpelm*(nunkp+2))

      if ( debug ) write(irefwr,1) 'lenindex3', lenindex3
      allocate ( index3(max(1,numold),lenindex3), stat=error )
      if ( error/=0 ) call eralloc ( error, lenindex3, 'index3' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepintelgrp'

      end if  ! ( debug )
      call erclos ( 'sepintelgrp' )

      end subroutine sepintelgrp
