      subroutine sepintmsh ( ichois, meshin, meshout, isol1, isol2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-11-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine interpolates the solution as stored in array isol1
!     from nodal points in array kmesh1 onto nodal points in array kmesh2
!     and stores the result in array isol2
!     Old subroutine intmsh
! **********************************************************************
!
!                       KEYWORDS
!
!     interpolation
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulemain
      use sepmoduletexts
      use sepmodulekmesh
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer meshin, meshout, isol1, isol2, ichois

!    ichois   i    Choice parameter
!                  Possible values:
!                  0,1: All points are interpolated; Points outside the mesh are
!                       extrapolated
!                  2:   Only points inside the old mesh or very close to the
!                       old mesh are interpolated, the other points are left
!                       unchanged
!    isol1     i   Standard SEPRAN array containing information of the vector to
!                  be interpolated
!    isol2     o   Standard SEPRAN array containing information of the
!                  interpolated vector
!     meshin         i    Sequence number of first mesh to be used
!     meshout        i    Sequence number of last mesh to be used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer itypvc
      double precision :: timebefore(2)

!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     INTMSH2D       Interpolate between 2D meshes
!     INTMSH3D       Interpolate between 3D meshes
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       1   ICHOIS is incorrect
!      60   Input solution array has not been filled or is of the wrong type
!     419   NDIM incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Perform some tests

      call eropen ( 'sepintmsh' )
      if ( itime>0 ) call ersettime ( timebefore )
      ndimlc = ndim
      if ( ichois<0 .or. ichois>2) then
         call errint ( ichois, 1 )
         call errsub ( 1, 1, 0, 0 )
      end if
      itypvc = ks(isol1)%typevec
      if ( itypvc/=110 .and. itypvc/=115) then
         call errint ( itypvc, 1 )
         call errsub ( 60, 1, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000

      call sepfillkmesh ( kmesh2d(meshin) )

      if ( ndim==2 ) then

!     --- ndim = 2

         call intmsh2d ( meshin, meshout, isol1, isol2, ichois )

      else if ( ndim==3 ) then

!     --- ndim = 3

         call intmsh3d ( meshin, meshout, isol1, isol2, ichois )

      else

!     --- ndim = 1

         call errint ( ndim, 1 )
         call errint ( 2, 2 )
         call errint ( 3, 3 )
         call errsub ( 419, 3, 0, 0 )

      end if
1000  call erclmn ( 'sepintmsh', timebefore )
      ierror = 0
      end
