      subroutine sepintsect3dch ( plane, iplane, ichoicearr, trans, values )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-05-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform checks for subroutine sepintsec3dmsh
!     Old subroutine intersect3dchk
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     3d
!     plane
!     intersection
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iplane, ichoicearr(3)
      double precision plane(4), trans(3,3), values(*)

!     ichoicearr     i    Array of size 3 containing the three parts of ichoice
!                         1:  ichoice_plane
!                         2:  ichoice_tang
!                         3:  ichoice_orig
!     iplane         o    Defines direction of plane
!                         1:  plane x = constant
!                         2:  plane y = constant
!                         3:  plane z = constant
!                         4:  general plane
!     plane          i    Array of length 4 containing the parameters a, b, c
!                         and d describing the plane: ax=by+cz+d = 0
!                         plane(1) = a; plane(2) = b; plane(3) = c; plane(4) = d
!     trans          o    In this array the Transformation matrix from x,y,z
!                         co-ordinates to ksi-eta co-ordinates is stored as
!                         well as the origin in trans(.,3)
!                         The transformation of x,y,z to ksi,eta is defined by
!                         ksi = sum_i trans(i,1) ( x(i) - trans(i,3) )
!                         eta = sum_i trans(i,2) ( x(i) - trans(i,3) )
!     values         i    Array containing information concerning the plane, &
!                         tangential vectors and origin
!                         See subroutine intp3dbf
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kdir

!     kdir           Coordinate direction of largest component
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INTERSECT3DCH1 Fill origin in trans
!     INTERSECT3DCH2 Fill transformation matrix in array trans
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     419   NDIM incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepintsect3dch' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepintsect3dch'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ndim/=3 ) then

!     --- ndim # 3, not available

         call errint ( ndim, 1 )
         call errint ( 3, 2 )
         call errint ( 3, 3 )
         call errsub ( 419, 3, 0, 0 )

      end if  ! ( ndim/=3 )

      iplane = ichoicearr(1)
      if ( iplane==4 ) then

!     --- iplane = 4, general case
!         check if this can be reduced to a simple case

         if ( abs(plane(2))<=0d0 .and. abs(plane(3))<=0d0 ) then

!        --- Plane in x-direction

            iplane = 1

         else if ( abs(plane(1))<=0d0 .and. abs(plane(3))<=0d0 ) then

!        --- Plane in y-direction

            iplane = 2

         else if ( abs(plane(1))<=0d0 .and. abs(plane(2))<=0d0 ) then

!        --- Plane in z-direction

            iplane = 3

         end if  ! ( abs(plane(2))<=0d0 .and. abs(plane(3))<=0d0 )

      end if  ! ( iplane==4)

      if ( ichoicearr(2)>0 .or. iplane==4 .or. ichoicearr(3)>0 ) then

!     --- ichoice_tang>0, tangential vectors given, or general plane
!         Fill transformation matrix and set iplane to 4

         iplane = 4
         call intersect3dch1 ( coor, plane, trans, ichoicearr(2), &
                               values, npoint, kdir )

      end if  ! ( ichoicearr(2)>0 .or. iplane==4 )

      if ( iplane==4 ) then

!     --- ichoice_orig>0, or general case
!         Fill origin in trans

         call intersect3dch2 ( ichoicearr(3), trans, values, plane, &
                               kdir )

      end if  ! ( ichoicearr(2)>0 .or. iplane==4 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepintsect3dch'

      end if  ! ( debug )
      call erclos ( 'sepintsect3dch' )

      end
