      function seplenglocvec ( iblock, ivec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-11-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Return with length of local vector corresponding to isol
!     Only for parallel computations
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     solution_vector
!     vector
!     global
!     length
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: seplenglocvec
      integer, intent(in) :: iblock, ivec

!     iblock         i    Block sequence number
!     ivec           i    Sequence number of vector of special structure
!     seplenglocvec  o    Result of computation of length
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: npointsav, inunkp, i, inode
      integer, pointer :: kprobfglob(:)

!     i              Counting variable
!     inode          Node sequence number
!     inunkp         Number of degrees of freedom in point
!     kprobfglob     Global array kprobf
!     npointsav      Help parameter to save the old value of npoint
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seplenglocvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seplenglocvec'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      npointsav = npoint
      npoint = km(iseqkm)%mappar(0)%arr(iblock)
      kprobfglob => kprobpart%kprobfglobal(ivec)%arr

      if ( debug ) &
         write(irefwr,1) 'iblock, npoint, indprf, nunkp', &
                          iblock, npoint, indprf, nunkp

      if ( indprf==0 ) then

!     --- Constant number of unknowns

         seplenglocvec = nunkp*npoint

      else

!     --- Variable number of unknowns

         length = 0
         do i = 1, npoint
            inode = nodes(i)
            inunkp = kprobfglob(inode+1)-kprobfglob(inode)
            length = length+inunkp
         end do  ! i = 1, npoint
         seplenglocvec = length

      end if  ! ( indprf==0 )
      npoint = npointsav

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'seplenglocvec', seplenglocvec
         write(irefwr,*) 'End seplenglocvec'

      end if  ! ( debug )
      call erclos ( 'seplenglocvec' )

      end function seplenglocvec
