      subroutine sepmapsol1d1 ( sepvector, nunkp, func )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interpolate one degree of freedom of sepran vector in user vector
!     Array imapfn and mapfn are used
!     Constant number of degrees per point
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     sepran_vector
!     user_vector
!     solution_vector
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplotfn
      use sepmoduleint2d
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nunkp
      double precision, intent(in) :: sepvector(nunkp,*)
      double precision, intent(out) :: func(*)

!     func           o    contains computed function values
!     nunkp          i    Maximum number of unknowns in a point
!     sepvector      i    sepran vector consisting of nunkp degrees of freedom
!                         per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, k, inode

!     i              Counting variable
!     inode          Node sequence number
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmapsol1d1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmapsol1d1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, numint

         func(i) = 0d0
         do k = 1, imapfn(1,i)

            inode = imapfn(k+1,i)
            func(i) = func(i)+mapfn(k,i)*sepvector(idegfd,inode)

         end do  ! k = 1, imapfn(1,i)

      end do  ! i = 1, numint

1000  call erclos ( 'sepmapsol1d1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmapsol1d1'

      end if  ! ( debug )

      end subroutine sepmapsol1d1
