      subroutine sepnumchar ( number, text, numchar )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Translate number into text using as few positions as possible
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     text
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: number
      integer, intent(out) :: numchar
      character (len=11) text

!     number         i    number to be translated to string
!     numchar        i    Number of characters in text
!     text          i/o   String translation of number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k

!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepnumchar' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepnumchar'
         write(irefwr,2) 'number', number
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( abs(number)<=0d0 ) then

!     --- number = 0, integer with 1 digit

         text = '0'
         numchar = 1

      else if ( abs(number)>=1d4 .or. abs(number)<=1d-4 ) then

!     --- number with exponent
!         write number into text in e format

         write(text,100) number
100      format(e10.4)
         text(7:7) = 'e'  ! change capital E into e
         numchar = 10

         if ( text(9:9)=='0' ) then

!        --- Remove superfluous zero in exponent

            text(9:9)=text(10:10)
            numchar = 9

         end if  ! ( text(9:9)=='0' )

         if ( text(8:8)=='+' ) then

!        --- Remove superfluous + in exponent

            text(8:9)=text(9:10)
            numchar = numchar-1

         end if  ! ( text(8:8)=='+' )

!        --- Remove superfluous zeros in part following dot

         do k = 6, 4, -1
            if ( text(k:k)=='0' ) then
               text(k:numchar-1) = text(k+1:numchar)
               numchar = numchar-1
            else
               exit
            end if  ! ( text(k:k)=='0' )
         end do  ! k = 6, 4, -1

      else

!     --- number without exponent
!         write as float in f format

         write(text,110) number
110      format(f11.5)
         numchar = 11

!        --- Remove trailing zeros

         do k = 11, 7, -1
            if ( text(k:k)=='0' ) then
               text(k:numchar-1) = text(k+1:numchar)
               numchar = numchar-1
            else
               exit
            end if  ! ( text(k:k)=='0' )
         end do  ! k = 11, 7, -1

!        --- If last character is dot remove this dot

         if ( text(numchar:numchar)=='.' ) numchar = numchar-1

!        --- Remove superfluous spaces

         do k = 1, 4
            if ( text(1:1)==' ' ) then
               numchar = numchar-1
               text(1:numchar) = text(2:numchar+1)
            else
               exit
            end if  ! ( text(1:1)==' ' )
         end do  ! k = 1, 4

      end if  ! ( abs(number)<=0d0 ) then

1000  call erclos ( 'sepnumchar' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'text', text(1:numchar)
         write(irefwr,*) 'End sepnumchar'

      end if  ! ( debug )

      end subroutine sepnumchar
