      subroutine sepprinelmnod1 ( ielem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print all nodes corresponding to an element
!     Old subroutine sepprinelmnod1
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     element
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ielem

!     ielem          i    Element number of element to be printed
!                         If ielem < 0 then also the coordinates of the
!                         nodes are printed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ikelmc, i, j, nodp, inpelm

!     i              Counting variable
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     j              Counting variable
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Find starting address in kmeshc
!     Actual printing
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprinelmnod1' )

!     --- Find starting address in kmeshc

      ikelmc = sepstkmeshc ( abs(ielem), inpelm )-1
      if ( ierror/=0 ) go to 1000

!     --- Actual printing

      write(irefwr,110) ielem, (kmeshc(ikelmc+i),i=1,inpelm)
110   format ( 'Nodal point numbers of element ', i8, ' are'/ &
               (10i8) )
      if ( ielem<0 ) then

!     --- ielem < 0, print coordinates

         write(irefwr,*) 'Co-ordinates of these nodes'
         do i = 1, inpelm
            nodp = kmeshc(ikelmc+i)
            write(irefwr,120) nodp, (coor(j,nodp), j = 1, ndim)
120         format(i8,' : ',3d14.5)
         end do

      end if  ! ( ielem<0 )

1000  call erclos ( 'sepprinelmnod1' )
      end
