      subroutine sepprinkmesha
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-04-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print kmesh parts a, b and f if they exist
!     Old subroutine prinkmesha
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, i

!     i              Counting variable
!     inpelm         number of points per element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprinkmesha' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprinkmesha'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( kelma==0 ) then

!     --- kelma = 0, constant number of nodes per group

         write(irefwr,1) 'number of nodes per element', npelm

      else

!     --- kelma>0, variable number of nodes per group

         write(irefwr,*) 'number of nodes per element group'
         do i = 1, nelgrp
            inpelm = Topgrp(i)%inpelm
            write(irefwr,100) i, inpelm
100         format ( i4, ':', i4 )
         end do  ! i = 1, nelgrp

      end if  ! ( kelma==0 )

      if ( kelmb>0 ) &
         call prinin ( kmeshb, nelgrp, &
                       'number of boundary nodes per group' )

      call prinin ( kmeshf, nelgrp, 'shape numbers per group' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprinkmesha'

      end if  ! ( debug )
      call erclos ( 'sepprinkmesha' )

      end
