      subroutine sepprinkmeshal
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-04-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the contents of array kmeshal
!     Old subroutine prinkmeshal
! **********************************************************************
!
!                       KEYWORDS
!
!     array
!     print
!     mesh
!     outer_boundary
!     boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseq, istart, nseq, iend

!     iend           Last position used at a part
!     iseq           Sequence number
!     istart         First position used at a part
!     nseq           Number of parts stored in kmeshal
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPPRINKMESHBS Print array ioutercurvs or ioutersurfs,
!                    depending on the dimension
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprinkmeshal' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprinkmeshal'
         call prinin ( iouterbounseq, 10, 'iouterbounseq' )

      end if
      if ( ierror/=0 ) go to 1000
      nseq = iouterbounseq(1)
      istart = 2

      do iseq = 1, nseq
         iend = iouterbounseq(istart)
         istart = istart+1
         if ( debug ) then
            write(irefwr,*) 'iseq, iend, istart ', iseq, iend, istart
            call prinin ( iouterbounseq(istart), 10, 'iouterbounseq/1' )
         end if  ! ( debug )
         call sepprinkmeshbs ( iouterbounseq(istart), iseq )
         istart = iend+1
      end do  ! iseq = 1, nseq

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprinkmeshal'

      end if
      call erclos ( 'sepprinkmeshal' )

      end
