      subroutine sepprinkmeshbs ( ioutercurvs, iseq )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-04-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print array ioutercurvs or ioutersurfs, depending on the dimension
!     Old subroutine prinkmeshal1
! **********************************************************************
!
!                       KEYWORDS
!
!     array
!     print
!     outer_boundary
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ioutercurvs(*), iseq

!     ioutercurvs    i    Contains information about the curves in the outer
!                         boundary (2D) or surfaces (3D) in the following way
!                         Pos. 1  nparts  Number of parts the outer boundary
!                                         consists of
!                         Pos. 2-1+nparts Starting addresses of each part
!                         Per part: (first address: ioutercurvs(1+i)
!                         Pos. 1: nouter  Number of curves in part
!                         followed by the curve numbers (provided with a sign)
!                         These curve numbers are consecutive, which means
!                         the end of one curve is the start of the new one
!                         The part itself is closed
!     iseq           i    Sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nparts, ipart, istart, nouter
      character (len=33) text

!     ipart          Sequence number of part
!     istart         First position used at a part
!     nouter         Number of outer curves in a part
!     nparts         number of parts the boundary consists of
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprinkmeshbs' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprinkmeshbs'

      end if
      if ( ierror/=0 ) go to 1000
      if ( ndim==2 ) then

!     --- ndim = 2, outer curves

         text = 'outer curves sequence number'
         write(text(29:31),100) iseq
100      format(i3)

      else

!     --- ndim = 3, outer surfaces

         text = 'outer surfaces sequence number'
         write(text(31:33),100) iseq

      end if
      nparts = ioutercurvs(1)

!     --- Loop over various parts

      do ipart = 1, nparts
         write(irefwr,*) 'part ', ipart
         istart = ioutercurvs(1+ipart)
         nouter = ioutercurvs(istart)
         call prinin ( ioutercurvs(istart+1), nouter, text )
      end do  ! ipart = 1, nparts

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprinkmeshbs'

      end if
      call erclos ( 'sepprinkmeshbs' )

      end
