      subroutine sepprinkprob1 ( ichois )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-11-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print contents of array kprobloc or parts of it
!     Old subroutine prinkprob
! **********************************************************************
!
!                       KEYWORDS
!
!     array
!     print
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulecpack
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ichois

!     ichois  i    choice parameter, values:
!                  -1: Print a selection of the complete integer array.
!                      This possibility is only allowed for kprob equal to
!                      KMESH, KPROB and INTMAT
!                      If the complete array KMESH must be printed, then
!                      both arrays KMPRB and kprob are assumed to be KMESH
!                      If the complete array KPROB must be printed, then
!                      array KMPRB is assumed to be KMESH and kprob is
!                      assumed to be KPROB
!                   0: Print the complete integer array.
!                      This possibility is only allowed for kprob equal to
!                      KMESH, KPROB and INTMAT
!                      If the complete array KMESH must be printed, then
!                      both arrays KMPRB and kprob are assumed to be KMESH
!                      If the complete array KPROB must be printed, then
!                      array KMPRB is assumed to be KMESH and kprob is
!                      assumed to be KPROB
!                  11: print kprob parts b and i
!                  12: print kprob part f
!                  13: print kprob part e
!                  14: print kprob part h
!                  15: print kprob part g
!                  16: print kprob part k
!                  17: print kprob part l
!                  18: print kprob part m
!                  19: print kprob part n
!                  22: print kprob part o
!                  23: print kprob part p
!                  24: print kprob part r
!                  25: print kprob part s
!                  26: print kprob part v and w
!                  27: print kprob part x
!                  28: print kprob part y
!                  29: print kprob part aa
!                  51: print kprob parts parinfo, sendinfo and innercontr
!                  52: print kprob part ab
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ih, k, i, istart, nfictpnt, mmkprobaa

!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     istart         First position used at a part
!     k              Counting variable
!     mmkprobaa      If>0, memory management sequence number of kprob aa
!     nfictpnt       Total number of points with fictitious unknowns
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR0000         print kprob parts b and i
!     PR0020         print Kprob part p
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINKPROBAB    print kprob part ab
!     PRINKPROBN     print kprob part n
!     PRINKPROBPAR   print parallel subarrays of kprob
!     PRINKPROBPAR1  Print array kprob par
!     PRINKPROBV     Print contents of array kprobv
!     PRINKPROBW     Print contents of array kprob
!     PRINSKIPELMS   Print elements that must be skipped
!     SEPPRINKPROBE  print kprob part e
!     SEPPRINKPROBF  print kprob part f
!     SEPPRINKPROBH  print kprob part h
!     SEPPRINPOSDEGF Print the numbering of nodes corresponding to degrees of
!                    freedom for one problem only
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprinkprob1' )

      if ( ichois==0 ) then

!     --- ichois = 0, print array KPROB

         call prinin ( kprobloc, kprobloc(3), 'kprob' )

      end if

      if ( ichois<=0 .or. ichois==11 ) then

!     --- ichois = 0 or 11, print kprob parts b and i

         if ( kprobloc(12)/=0 ) call pr0000

      end if
      if ( ichois<=0 .or. ichois==12 ) then

!     --- ichois = 0 or 12, print kprob part f

         if ( indprf/=0 ) call sepprinkprobf

!        --- Print also the numbering of nodes corresponding to degrees of
!            freedom

         call sepprinposdegf

      end if
      if ( ichois<=0 .or. ichois==13 ) then

!     --- ichois = 0 or 13, print kprob part e

         call sepprinkprobe

      end if
      if ( ichois<=0 .or. ichois==14 ) then

!     --- ichois = 0 or 14, print kprob part h

         ih = kprobloc(50)
         if ( ih>0 ) then
            write ( irefwr, 235 )
235         format(' internal positions of degrees of freedom'/ &
                   ' without the effect of periodical boundary', &
                   ' conditions' )
            if ( indprf==0 ) then
               write ( irefwr, 900 ) (k, kprobh(k), k = 1,nusol )
            else
               call sepprinkprobh
            end if
         end if
      end if
      if ( ichois<=0 .or. ichois==15 ) then

!     --- ichois = 0 or 15, print kprob part g

         ih = kprobloc(50)
         if ( ih>0 ) then
            if ( kprobloc(20)==0 ) then
               write ( irefwr,* ) 'kprob g is empty'
            else
               write ( irefwr, 236 )
236            format('essential boundary conditions' )
               call prinin1 ( kprobg, kprobloc(9), 2, &
                              '           node  bit_pattern' )
            end if
         end if
      end if
      if ( ichois==0 .or. ichois==19 ) then

!     --- ichois = 0 or 19, print kprob part n

         call prinkprobn ( npoint, ndim )

      end if

      if ( ichois<=0 .or. ichois==22 ) then

!     --- ichois = 0 or 22, print kprob part o

         if ( kprobloc(28)>0 ) then
            nbcons = kprobloc(32)
            kprobo => kprobpart%kprobo
            write(irefwr,*) 'Degrees of freedom corresponding', &
                            ' to boundary conditions of the type', &
                            ' u equals unknown constant:'
            write(irefwr,901) ( kprobo(k),k = 1,nbcons )
         end if

      end if

      if ( ichois<=0 .or. ichois==23 ) then

!     --- ichois = 0 or 23, print kprob part p

         if ( indprp>0 ) then
            nphys = max ( kprobloc(4), kprobloc(33))
            kprobp => kprobpart%kprobp
            npoint = size(kprobp,1)
            call pr0020
         end if

      end if

      if ( ichois<=0 .or. ichois==24 ) then

!     --- ichois = 0 or 24, print kprob part r

         npboun = kprobloc(34)
         if ( npboun>0 ) then
            ih = kprobloc(37)
            write ( irefwr, 320 )
320         format(' internal positions of degrees of freedom'/ &
                   ' The effect of periodical boundary conditions' &
                   ,' has been taken into account')
            write ( irefwr, 900 ) (k, kprobr(k), k = 1,nusol )
         end if

      end if

      if ( ichois<=0 .or. ichois==25 ) then

!     --- ichois = 0  or 25, print kprob part s

         npboun = kprobloc(34)
         if ( kprobloc(38)>0 ) then
            write ( irefwr, 330 )
330         format(' Positions of periodical boundary conditions,', &
                   ' followed by 2 constants c_1 and c_2')
            write ( irefwr, 903 ) (k, kprobs(k), &
                    rprobs(k,1), rprobs(k,2), k = 1,npboun )
         end if

      end if

      if ( ichois<=0 .or. ichois==26 ) then

!     --- ichois = 0 or 26, print kprob parts v and w

         if ( kprobloc(46)>0 ) call prinkprobv ( nsurfs )
         if ( kprobloc(47)>0 ) call prinkprobw ( nsurfs )

      end if

      if ( ichois<=0 .or. ichois==27 ) then

!     --- ichois = 0 or 27, print kprob part x

         indprx = kprobloc(11)
         if ( indprx>0 ) then
            write ( irefwr, 420 ) (kprobx(k), k = 0,2)
420         format(' nglgrp = ', i3, '  nglunk = ', i5, &
                   ' nglcoupl = ', i6 )

            indprx = 2
            do i = 1,  nglgrp
               write( irefwr, 430 ) (kprobx(indprx+k), k = 1,6)
430            format (' itype = ', i6/ &
                       ' inunkpgr = ', i6/ &
                       ' ityperegion = ', i6/ &
                       ' ishape = ', i6/ &
                       ' icrv1 = ', i6/ &
                       ' icrv2 = ', i6 )
               indprx = indprx+6
            end do
         end if

      end if

      if ( ichois<=0 .or. ichois==28 ) then

!     --- ichois = 0 or 28, print kprob part y

         indpry = kprobloc(51)
         if ( indpry>0 ) then
            nfictgrp = kproby(1)
            nfictpnt = kproby(3)
            kprobfict => kprobpart%kprobfict
            write ( irefwr, 440 ) (kproby(k), k = 1,3)
440         format(' nfictgrp = ', i3, ' nfictunk  = ', i5, &
                   ' nfictpnt = ', i6 )
            do i = 1, nfictgrp
               istart = kproby(10+i)
               write( irefwr, 450 ) (kproby(istart+k), k = 0, 11)
450            format (' itype = ', i6/ &
                       ' inunkpgr = ', i6/ &
                       ' iregionlam = ', i6/ &
                       ' ishape = ', i6/ &
                       ' icurve1 = ', i6/ &
                       ' icurve2 = ', i6/ &
                       ' istructgrp = ', i6/ &
                       ' ifluidgrp1 = ', i6/ &
                       ' ifluidgrp2 = ', i6/ &
                       ' inunkp_elm = ', i6/ &
                       ' nelem = ', i6/ &
                       ' ishape_curve = ', i6 )
               write ( irefwr, 460 )
460            format ( 'Elements corresponding to lambda points'/ &
                        ' Point no   structural elm.  fluid elm.' )
               do k = 1, nfictpnt
                  write(irefwr,470) k, kprobfict(:,k)
               end do
470            format ( 1x, i6, i12, i12 )

            end do

         end if

      end if

      if ( ichois<=0 .or. ichois==29 ) then

!     --- ichois = 0 or 29, print kprob part aa

         if ( kprobloc(53)>0 ) then

!        --- mmkprobaa>0, skip elements

            kprobaa => kprobpart%kprobaa
            call prinskipelms

         end if

      end if

      if ( parallel .and. (ichois<=0 .or. ichois==51) ) then

!     --- ichois = 0 or 51, print kprob parallel part

         if ( associated(kprobpart%blockunkinfo) ) call prinkprobpar

         call prinkprobpar1

      end if  ! ( parallel .and. (ichois<=0 .or. ichois==51) )

      if ( ichois<=0 .or. ichois==52 ) then

!     --- ichois = 0 or 52, print kprob part ab

         indprab = kprobloc(58)
         if ( indprab>0 ) then

!        --- indprab>0, periodical pairs

            kprobab => kprobpart%kprobab
            call prinkprobab

         end if

      end if  ! ( ichois<=0 .or. ichois==52 )

900   format( ( 1x, 6(i7,':',i7) ) )
901   format ( 1x, 10i7 )
903   format ( 1x, 2i7, 2d13.5 )

      call erclos ( 'sepprinkprob1' )
      end subroutine sepprinkprob1
