      subroutine sepprinmat2 ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-10-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print inverse matrix that is stored as profile real matrix
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulemat   ! definition of matrices
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(in) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, istart, leftcl, nel, ii
      logical :: unsym
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     i              Counting variable
!     istart         First position used at a part
!     leftcl         smallest column number in row
!     nel            number of elements in a row
!     unsym          indicates if the matrix is symmetrical (false) or
!                    not (true)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR0006         print row of real matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprinmat2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprinmat2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)
      unsym = .not. IM%symmetric

      istart = 1
      do i = 1, A%neq
         ii = A%Mrows(i)
         nel = ii-istart
         if(unsym) nel = 2*nel
         nel = nel+1
         call pr0006(A%MD(istart), nel, i)
         istart = istart+nel
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprinmat2'

      end if  ! ( debug )
      call erclos ( 'sepprinmat2' )

      end subroutine sepprinmat2
