      subroutine sepprinmatcomp ( threshold, A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-03-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     print large matrix in case of a real compact storage
!     Old subroutine prinmtrecomp
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulekprob
      use sepmodulematr
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: threshold
      type (rmatrix), intent(in) :: A

!     A              i    Structure of type matrix containing the necessary
!                         information of the matrix
!     threshold      i    threshold value, only elements larger than this value
!                         are printed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPPRINMAT6    print compact matrix
!     SEPPRINMATDBG  print compact matrix, skip zeros
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprinmatcomp' )
      debug = debug_prinmtbf
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprinmatcomp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) then

!     --- debug = true, write in debug format

         call sepprinmatdbg ( A, threshold )

      else

!     --- debug = false, standard case

         call sepprinmat6 ( A )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprinmatcomp'

      end if  ! ( debug )
      call erclos ( 'sepprinmatcomp' )

      end
