      subroutine sepprinmatdc6 ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-12-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print matrix that is stored as row-compact real matrix
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulemat   ! definition of matrices
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(in) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, istart, ncols, jstart, m, jend, j, ih
      integer :: posU(A%neq)
      double precision :: work(A%neq)
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     istart         start address
!     j              Counting variable
!     jend           end address
!     jstart         starting position
!     m              number of items to be printed
!     ncols          number of non-zero A%Mcols in row
!     posU           Contains next non-used column number per row
!     work           Work array to store elements to be printed in a row
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprinmatdc6' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprinmatdc6'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(A%iseqim)

!     --- Row compact storage

      write(irefwr,110) (i,A%MD(A%diag(i)),i=1,A%neq)
110   format((4(1x,i6,':',ss,1pe12.5e2)))
      istart = 0
      do i = 1, A%neq
         ncols = A%diag(i)-istart-1
         if ( ncols>0 ) then
            jstart = istart+1
            jend = istart+ncols
            m = min(jstart+3,jend)
            write(irefwr,200) i, ( A%Mcols(j), A%MD(j), j = jstart, m )
200         format(1x,i6,':',4(i5,':',ss,1pe12.5e2))
            if ( m<jend ) &
               write(irefwr,210) ( A%Mcols(j), A%MD(j), j = m+1, jend )
210         format((8x,4(i5,':',ss,1pe12.5e2)))
         end if  ! ( ncols>0 )

         istart = A%Mrows(i+1)

      end do  ! i = 1, A%neq

      write(irefwr,250)
250   format(/' upper triangular matrix with column and row numbers'/)

      posU = A%diag+1

      istart = 0
      do i = 1, A%neq
         ncols = A%diag(i)-istart-1
         if ( ncols>0 ) then
            jstart = istart+1
            jend = istart+ncols
            m = min(4,ncols)
            do j = 1, ncols
               ih = A%Mcols(j+istart)
               work(j) = A%MD(posU(ih))
               posU(ih) = posU(ih)+1
            end do  ! j = 1, ncols
            write(irefwr,200) i, ( A%Mcols(j+istart), work(j), j = 1, m )
            if ( m<ncols ) &
               write(irefwr,210) ( A%Mcols(j+istart), work(j), j = m+1, ncols )
         end if  ! ( ncols>0 )

         istart = A%Mrows(i+1)

      end do  ! i = 1, A%neq

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprinmatdc6'

      end if  ! ( debug )
      call erclos ( 'sepprinmatdc6' )

      end subroutine sepprinmatdc6
