      subroutine sepprintmatb ( matr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-03-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print part of matrix corresponding to boundary conditions
!     Old subroutine printmatr
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulematr
      use sepmodulematstr
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: matr

!     matr    i   Standard SEPRAN array containing information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR0017         print matrix corresponding to boundary conditions
!     PR0018         print matrix corresponding to boundary conditions
!                    complex case
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprintmatb' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprintmatb'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      write(irefwr,510)
510   format(' matrix corresponding to boundary conditions (Sup)')

      if ( realmt(matr)%complexmat ) then

         call pr0018 ( S_ip )

      else

         S_ip => realmt(matr)%S_ip
         call pr0017 ( S_ip, irowsip, icolsip, nbound, nusol )
         write(irefwr,520)
520      format(' matrix corresponding to boundary', &
                ' conditions (Spu)')

         S_pi => realmt(matr)%S_pi
         call pr0017 ( S_pi, irowsip, icolsip, nbound, nusol )
         write(irefwr,530)
530      format(' matrix corresponding to boundary', &
                ' conditions (Spp)')

!        --- ipmatb is a real matrix so adjust for integer length

         S_pp => realmt(matr)%S_pp
         call pr0017 ( S_pp, irowsp, icolsp, nbound, nusol )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprintmatb'

      end if  ! ( debug )
      call erclos ( 'sepprintmatb' )

      end subroutine sepprintmatb
