      subroutine sepprintmatp ( matr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-03-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print part of matrix corresponding to parallel part
!     Old subroutine printmatr
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulekprob
      use sepmodulematr
      use sepmodulematstr
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: matr

!     matr    i   Standard SEPRAN array containing information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (intmatrix), pointer :: IM
      integer :: nrusol1

!     IM             Refers to intmt(intmat)
!     nrusol1        Local value of nrusol
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PR0017         print matrix corresponding to boundary conditions
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprintmatp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprintmatp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(realmt(matr)%intmatref)

      if ( ninterface>0 .and. size(irowsfi)>1 ) then

!     --- Parallel case matrices S_if and S_fi

         if ( .not. IM%petsc ) then

!        --- Matrix S_fi exists

            write(irefwr,540)
540         format(' matrix S_fi (interface-internal)' )
            call pr0017 ( S_fi, irowsfi, icolsfi, ninterface, &
                          ninterface+nrusol )

         end if  ! ( .not. IM%petsc )

         if ( .not. IM%petsc ) then

!        --- Matrix S_if exists

            write(irefwr,550)
550         format(' matrix S_if (internal-interface)' )
            call pr0017 ( S_if, irowsif, icolsif, nrusol, nrusol )

         end if  ! ( .not. IM%petsc )

      end if  ! ( ninterface>0  )

      if ( size(irowsout)>1 ) then

!     --- Matrix S_iout exists

         write(irefwr,560)
560      format(' matrix S_iout (internal-interface global)' )
         nrusol1 = realmt(matr)%S_i%neq
         call pr0017 ( S_iout, irowsout, icolsout, nrusol1, nrusol1 )

      end if  ! ( size(irowsout)>1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprintmatp'

      end if  ! ( debug )
      call erclos ( 'sepprintmatp' )

      end subroutine sepprintmatp
