      subroutine sepprirvec ( iformat, inodes, nnodes, ichoice )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-11-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print real vector usoltemp
!
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: nnodes, inodes(nnodes), iformat, ichoice

!     ichoice        i    Integer choice parameter
!                         0: do not use node numbers
!                         1: use node numbers
!     iformat        i    Format used for printing
!                         1: e13.5
!                         2: e16.8
!                         3: e13.5, one node per line
!                         4: e16.8, one node per line
!     inodes         i    Array containing the nodes to be printed
!     nnodes         i    Number of nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      character (len=35) :: format1, format2
      integer :: lentext1,  lentext2, num, i, istart, m1, inunkp

!     format1        Contains the format for the first line per point
!                    to be printed
!     format2        Contains the format for the next lines for these points
!     i              Counting variable
!     inunkp         Number of degrees of freedom in point
!     istart         First position used at a part
!     lentext1       number of characters in format1
!     lentext2       number of characters in format2
!     m1             Help variable
!     num            Help variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprirvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprirvec'
         call ertrace
         write(irefwr,1) 'iformat, ichoice, nnodes', iformat, ichoice, nnodes
         write(irefwr,1) 'indprfo, nunkpo', indprfo, nunkpo
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      format1 = ' '
      format2 = ' '
      if ( ichoice==0 ) then

!     --- ichoice = 0, no node numbers

         lentext1 = 7
         format1(1:lentext1) = '(1x,ss,'

      else

!     --- ichoice = 0, use node numbers

         lentext1 = 10
         format1(1:lentext1) = '(1x,i9,ss,'

      end if  ! ichoice==0 )

      select case ( iformat )

      case(1)

!     --- iformat = 1, e13.5

         format1(lentext1+1:lentext1+24) = '5(1pe13.5e2))'
         lentext1 = lentext1+24
         num = ndim*13+10
         lentext2 = 9
         write(format2(1:lentext2),100) '((', num, 'x,ss,'
100      format(a,i2,a)
         num = 5-ndim
         write(format2(lentext2+1:lentext2+1),110) num
110      format(i1)
         format2(lentext2+2:lentext2+14) = '(1pe13.5e2)))'
         lentext2 = lentext2+14

      case(2)

!     --- iformat = 2, e16.8

         format1(lentext1+1:lentext1+24) = '5(1pe16.8e2))'
         lentext1 = lentext1+24
         num = ndim*16+10
         lentext2 = 9
         write(format2(1:lentext2),100) '((', num, 'x,ss,'
         num = 5-ndim
         write(format2(lentext2+1:lentext2+1),110) num
         format2(lentext2+2:lentext2+14) = '(1pe16.8e2)))'
         lentext2 = lentext2+14

      case(3)

!     --- iformat = 3, e13.5 all in one line

         format1(lentext1+1:lentext1+25) = '99(1pe13.5e2))'
         lentext1 = lentext1+25
         lentext2 = 0

      case(4)

!     --- iformat = 4, e16.8 all in one line

         format1(lentext1+1:lentext1+25) = '99(1pe16.8e2))'
         lentext1 = lentext1+25
         lentext2 = 0

      end select  ! case ( iformat )

      if ( ichoice==0 ) then

!     --- ichoice = 0, no node numbers

         if ( indprfo<=0 ) then

!        --- indprfo = 0, hence constant number of unknowns per point

            istart = 0
            if ( iformat<3 ) then
               m1 = min(nunkpo,5-ndim)
            else
               m1 = nunkpo
            end if  ! ( iformat<3 )
            do i = 1, nnodes
               write(irefwr,fmt=format1) coor(:,inodes(i)), &
                                            usoltemp(istart+1:istart+m1)
               if ( nunkpo>m1 ) &
                  write(irefwr,fmt=format2) usoltemp(istart+m1+1:istart+nunkpo)
               istart = istart+nunkpo
            end do  ! i = 1, nnodes

         else

!        --- indprfo > 0, hence variable number of unknowns per point

            istart = kprobfo(1)
            do i = 1, nnodes
               inunkp = kprobfo(i+1)-istart
               if ( iformat<3 ) then
                  m1 = min(inunkp,5-ndim)
               else
                  m1 = inunkp
               end if  ! ( iformat<3 )
               if ( m1>0 ) &
                  write(irefwr,fmt=format1) coor(:,inodes(i)), &
                                            usoltemp(istart+1:istart+m1)
               if ( inunkp>m1 ) &
                  write(irefwr,fmt=format2) usoltemp(istart+m1+1:istart+inunkp)
               istart = istart+inunkp
            end do  ! i = 1, nnodes

         end if  ! ( indprfo<=0 )

      else

!     --- ichoice = 0, use node numbers

         if ( indprfo<=0 ) then

!        --- indprfo = 0, hence constant number of unknowns per point

            istart = 0
            if ( iformat<3 ) then
               m1 = min(nunkpo,5-ndim)
            else
               m1 = nunkpo
            end if  ! ( iformat<3 )
            do i = 1, nnodes
               write(irefwr,fmt=format1) inodes(i), coor(:,inodes(i)), &
                                         usoltemp(istart+1:istart+m1)
               if ( nunkpo>m1 ) &
                  write(irefwr,fmt=format2) usoltemp(istart+m1+1:istart+nunkpo)
               istart = istart+nunkpo
            end do  ! i = 1, nnodes

         else

!        --- indprfo > 0, hence variable number of unknowns per point

            istart = kprobfo(1)
            do i = 1, nnodes
               inunkp = kprobfo(i+1)-istart
               if ( iformat<3 ) then
                  m1 = min(inunkp,5-ndim)
               else
                  m1 = inunkp
               end if  ! ( iformat<3 )
               if ( m1>0 ) &
                  write(irefwr,fmt=format1) inodes(i), coor(:,inodes(i)), &
                                            usoltemp(istart+1:istart+m1)
               if ( inunkp>m1 ) &
                  write(irefwr,fmt=format2) usoltemp(istart+m1+1:istart+inunkp)
               istart = istart+inunkp
            end do  ! i = 1, nnodes

         end if  ! ( indprfo<=0 )

      end if  ! ichoice==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprirvec'

      end if  ! ( debug )
      call erclos ( 'sepprirvec' )

      end subroutine sepprirvec
