      subroutine sepremdouble
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Remove double points in array iseqx
!
! **********************************************************************
!
!                       KEYWORDS
!
!     triangle
!     intersection
!     line
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplotfn
      use sepmoduleint2d
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i1, i2, i
      double precision :: dx, xprev, eps

!     dx             distance between 2 points
!     eps            Local accuracy
!     i              Counting variable
!     i1             sequence number of edge in element
!     i2             element number in element
!     xprev          x co-ordinate of previous point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepremdouble' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepremdouble'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numint = 1                ! number of interpolation points
      i1 = 2-mod(iseqx(1),2)    ! sequence number of node in element
      i2 = (iseqx(1)+1)/2       ! element number
      x(1) = listx(i1,i2)       ! coordinates first node

      i1 = 2-mod(iseqx(2*numelm),2)  ! sequence number of node in element
      i2 = (iseqx(2*numelm)+1)/2     ! element number
      x(2) = listx(i1,i2)       ! coordinates last node
      xprev = x(1)

      dx = x(2)-x(1)            ! distance
      eps = dx*sqreps           ! accuracy

      do i = 2, 2*numelm

         i1 = 2-mod(iseqx(i),2)  ! sequence number of first node in element
         i2 = (iseqx(i)+1)/2     ! element number
         x(1) = listx(i1,i2)     ! coordinates of node

         if ( x(1)-xprev>eps ) then

!        --- New coordinate found, store in iseqx and raise numint

            numint = numint+1
            iseqx(numint) = iseqx(i)
            xprev = x(1)

         end if  ! ( abs(x(1)-xprev)>eps )

      end do  ! i = 2, 2*numelm

1000  call erclos ( 'sepremdouble' )
      if ( debug ) then

!     --- Debug information

         call prinin ( iseqx, numint, 'iseqx' )
         write(irefwr,*) 'End sepremdouble'

      end if  ! ( debug )

      end subroutine sepremdouble
