      subroutine sepstream3 ( ichois, numelm, ielhlp, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-11-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     mark elements that must be skipped by setting iwork = -2
!     compute number elements (numelm) remaining
! **********************************************************************
!
!                       KEYWORDS
!
!     compute
!     incompressible
!     stream_function
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ichois, ielhlp(*)
      integer, intent(out) :: numelm
      integer, intent(inout) :: iwork(*)

!     ichois         i    If 1 ielhlp is used
!     ielhlp         i    This array of length nelgrp+5 is only used if
!                         allelements = 1
!                         ielhlp(ielgrp+5) = 0 means elements of element group
!                         ielgrp are skipped, ielhlp(ielgrp+5) = 1, means that
!                         they are assembled
!     iwork         i/o   integer work array, see sepstream
!     numelm         o    Number of elements remaining
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iielem, j, inpelm, itype

!     iielem         Relative element number in element group
!     inpelm         number of points per element
!     itype          Table containing information about the numbers behind
!                    the keywords in the table code
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     215   Expected two-dimensional problem on two-dimensional mesh does
!           not exist or starting point is not lying in an accessible element
!           group.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepstream3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepstream3'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numelm = nelem    ! number of elements that may be used
      iielem = 1

      do j = 1, nelgrp

         inelem = Topgrp(j)%inelem
         inpelm = Topgrp(j)%inpelm
         ishape = kmeshf ( j )
         itype = kprobc(j)

!        --- meshconnect elements (for periodicity conditions) are allowed
!            only for ichois=1, ielhlp(5+ielgrp)/=0

         if ( (ichois==0 .and. &
              (ishape<=2 .or.  itype==0)) .or. &
               ichois==1 .and. ielhlp(j+5)==0 .or. &
               ichois==1 .and. ielhlp(j+5)==1  .and. &
              (ishape<=2 .and. ishape/=-10001 .or. &
               itype==0 ) ) then

!        element must be skipped, set iwork to -2

            numelm=numelm-inelem
            iwork(iielem:iielem+inelem-1)=-2

         end if
         if ( ishape>10 ) then

!        --- three-dimensional elements in the problem not yet allowed

            call errsub ( 215, 0, 0, 0 )
            go to 1000
         end if
         iielem=iielem+inelem

      end do
      if ( numelm<=0 ) call errsub ( 215, 0, 0, 0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepstream3'

      end if  ! ( debug )
      call erclos ( 'sepstream3' )

      end subroutine sepstream3
