      subroutine sepstream4 ( knpelm, lstep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-11-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute knpelm and lstep as function of ielgrp
!
! **********************************************************************
!
!                       KEYWORDS
!
!     compute
!     incompressible
!     stream_function
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(out) :: knpelm, lstep

!     knpelm         o    Last vertex in element
!     lstep          o    Step in element for use of vertices only
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepstream4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepstream4'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpelm = Topgrp(ielgrp)%inpelm

      if ( inpelm==5 .or. ishape==10 ) then

!     --- Element with point in center

         knpelm=inpelm-1
         lstep = 1

      else if ( inpelm==7 ) then

!     --- Extended quadratic element

         knpelm=inpelm-2
         lstep=2

      else if ( inpelm>5 ) then

!     --- Quadratic element

         lstep=2
         knpelm=inpelm-1

      else

!     --- Linear element

         knpelm = inpelm
         lstep = 1

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepstream4'

      end if  ! ( debug )
      call erclos ( 'sepstream4' )

      end subroutine sepstream4
