      subroutine to0068 ( ndim, ichois, partout, nsteps )
! ======================================================================
!
!        programmer    Arno van Duin
!        version  2.1  date 25-04-2010 New call to plaf05
!        version  2.0  date 06-04-1996 Complete new subroutine
!        version  1.0  date 14-11-1990
!
!   copyright (c) 1990-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   The plotfile is opened, the boundaries are drawn, the pen is moved to
!   the starting point, and the starting point is written to stdout
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     print
!     plot
!     curve
!     particle_tracking
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois(*), ndim, nsteps
      double precision partout(ndim+1,*)

!     ichois     i    Choice parameter array:
!                     ichois(1)  =  length of array ichois to be used
!                     ichois(2)  =  type of velocity field given
!                          3 : Fluxes through the edges are given (elmtyp=650)
!                     ichois(3)  =  print indicator
!                          0 : Make no print of trajectories       (default)
!                         >0 : Make print of particle trajectories on file with
!                              referencenumber ichois(3)
!                     ichois(4)  =  plot indicator
!                          0 : Make no plots
!                          1 : Make plot of particle trajectories
!                          2 : Make plot of mesh and trajectories  (default)
!                    ichois(5) = direction indicator for parallel projection
!                              (only used for 3-D)
!                         1 : Projection on (x,z) plane from y is +inf (default)
!                         2 : Projection on (y,z) plane from x is +inf
!                         3 : Projection on (x,z) plane from y is -inf
!                         4 : Projection on (y,z) plane from x is -inf
!     ndim       i    the dimension of space
!     nsteps     i    Contains the number of positions stored in partout
!     partout    i    Array containing the actual information concerning the
!                     particle trajectories
!                     partout(1,i) contains t_i
!                     partout(2-4,i) contains x_i, y_i and z_i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, i, nview

!     i           General loop variable
!     k           General loop variable
!     nview       see ichois(5)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     PLAF05    Plot polygon
!     PLTR15    Plot part of 3-D track
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     If ( print output is required ) then
!        Write heading of print-out
!        Write co-ordinates
!     If ( plot output is required ) then
!        Make plot
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0068' )
      if ( ichois(3)/=0 ) then

!     --- Print output required

         if ( ndim==2 ) then
            write ( ichois(3),100 )
100         format ( /'          t              x              y' )
            write ( ichois(3),110) ((partout(k,i),k=1,3),i=1,nsteps)
110         format((ss,3(1pe16.6e2)))
         else if ( ndim==3 ) then
            write ( ichois(3),200 )
200         format ( /'          t              x              y', &
                      '          z' )
            write ( ichois(3),210) ((partout(k,i),k=1,4),i=1,nsteps)
210         format((ss,4(1pe16.6e2)))
         end if
      end if
      if ( ichois(4)/=0 ) then

!     --- plot output

         if ( ndim==2 ) then
            call plaf05 ( 3, dcolor, nsteps, partout(2,1), 3 )
         else if ( ndim==3 ) then
            if ( ichois(5)>4  .or. ichois(5)<1 ) then
               nview = 1
            else
               nview = ichois(5)
            end if
            call pltr15 ( nsteps, partout(2,1), nview, 4 )
         end if
      end if
      call erclos ( 'to0068' )
      end
