      subroutine to0074 ( inpelm, index1, index2, fluxe, flux, ndim )
! ======================================================================
!
!        programmer    Arno van Duin
!        version 1.2   date   06-04-1996 Layout
!        version 1.1   date   21-03-1991 adjusted for NDIM = 3
!        version 1.0   date   14-11-1990
!
!
!   copyright (c) 1990-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute the translation vector of the mapping to a reference element
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer inpelm, index1(*), index2(*), ndim
      double precision flux(*), fluxe(*)

!     flux     i  global real array, containing the fluxes through the edges
!                 the positive direction is from a lower numbered nodal point
!                 to a higher numbered one
!     fluxe    o  real array, containing the outward fluxes through the edges
!     index1   i  integer array, containing the nodal point numbers of
!                 one element
!     index2   i  integer array, containing the index numbers of flux
!                 in the midths of the edges of one element
!     inpelm   i  integer, the number of nodal points per element
!     ndim     i  the dimension of space
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     fluxe = flux
!          i      index2
!                       i
!     fluxe = -fluxe  if inward
!          i        i
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( ndim.eq.2 ) then

         fluxe(1) = flux(index2(1))
         fluxe(2) = flux(index2(2))
         fluxe(3) = flux(index2(3))

!        --- get sign fluxe
!                          1

         if ( index1(1).gt.index1(3) ) then
            fluxe(1) = -fluxe(1)
         end if

!        --- get sign fluxe
!                          2

         if ( index1(3).gt.index1(5) ) then
            fluxe(2) = -fluxe(2)
         end if

         if ( inpelm.eq.6 ) then

!        --- get fluxe in case of a six point triangle
!                     3

            if ( index1(5).gt.index1(1) ) then
               fluxe(3) = -fluxe(3)
            end if
         else

!        --- get fluxe in case of an eight point quadrilateral
!                  3

            if ( index1(5).gt.index1(7) ) then
               fluxe(3) = -fluxe(3)
            end if

!           --- get fluxe in case of an eight point quadrilateral
!                        4

            fluxe(4) = flux(index2(4))

            if ( index1(7).gt.index1(1) ) then
               fluxe(4) = -fluxe(4)
            end if
         end if
      else if ( ndim.eq.3 ) then

         fluxe(1) = flux(index2(1))
         fluxe(2) = flux(index2(2))
         fluxe(3) = flux(index2(3))
         fluxe(4) = flux(index2(4))

!        --- get sign fluxe
!                          1

         if ( index1(5).lt.index1(10) ) then
            if ( ( index1(3).lt.index1(10) ) .and. &
                 ( index1(3).gt.index1(5) ) ) then
               fluxe(1) = -fluxe(1)
            end if
         else
            if ( ( index1(3).lt.index1(10) ) .or. &
                 ( index1(3).gt.index1(5) )  ) then
               fluxe(1) = -fluxe(1)
            end if
         end if

!        --- get sign fluxe
!                          2

         if ( index1(1).lt.index1(10) ) then
            if ( ( index1(5).lt.index1(10) ) .and. &
                 ( index1(5).gt.index1(1) ) ) then
               fluxe(2) = -fluxe(2)
            end if
         else
            if ( ( index1(5).lt.index1(10) ) .or. &
                 ( index1(5).gt.index1(1) )  ) then
               fluxe(2) = -fluxe(2)
            end if
         end if

!        --- get sign fluxe
!                          3

         if ( index1(3).lt.index1(10) ) then
            if ( ( index1(1).lt.index1(10) ) .and. &
                 ( index1(1).gt.index1(3) ) ) then
               fluxe(3) = -fluxe(3)
            end if
         else
            if ( ( index1(1).lt.index1(10) ) .or. &
                 ( index1(1).gt.index1(3) )  ) then
               fluxe(3) = -fluxe(3)
            end if
         end if

!        --- get sign fluxe
!                          4

         if ( index1(1).lt.index1(5) ) then
            if ( ( index1(3).lt.index1(5) ) .and. &
                 ( index1(3).gt.index1(1) ) ) then
               fluxe(4) = -fluxe(4)
            end if
         else
            if ( ( index1(3).lt.index1(5) ) .or. &
                 ( index1(3).gt.index1(1) )  ) then
               fluxe(4) = -fluxe(4)
            end if
         end if

      end if

      end
