      subroutine to0079 ( iloc, index1, index2, array, nelem, &
                          inpelm, ielem, ndim )
!
! ********************************************************************
!
!
!        programmer    Arno van Duin
!        version 1.2   date   91-03-21 adjusted for NDIM = 3
!        version 1.1   date   91-03-15 Adjusted to PTRACK
!        version 1.0   date   90-11-14
!
!
!   copyright (c) 1990  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
!
! ********************************************************************
!
!   Get the entered element
!
! ********************************************************************
!
!
!                INPUT / OUTPUT PARAMETERS

      implicit none
      integer array(*), ielem, iloc, index1(*), index2(*), inpelm, &
       ndim, nelem

!
!     array    i  integer array, containing the nodal point numbers of
!                 all elements
!                 standard SEPRAN array, KMESH part c (see PG 13.1.3)
!
!     ielem    o  integer, the element number
!
!     iloc     io integer, at start: 0 if there is a sink in this element
!                 otherwise the number of the edge through which the former
!                 element was left
!                          at end : 0 if no element could be found, in the
!                 calling routine end_of_area is set to true
!                 otherwise not used
!
!     index1   io integer array, containing the nodal point numbers of
!                 one element. At start of the left element, at end of the
!                 entered element
!
!     index2   o  integer array, containing the index numbers of flux
!                 in the midths of the edges of one element
!
!     inpelm   i  integer, number of nodal points per element
!
!     ndim     i  the dimension of space
!
!     nelem    i  integer, number of elements
!
! *********************************************************************
!
!               LOCAL PARAMETERS

      integer iplace, j
      logical found

!
!     iplace      local point number
!
!     j           integers, loop variables
!
!     found       boolean, tells whether the entered element is found or not
!
! ********************************************************************
!
!                              METHOD
!
!     The entered element is the element that has side ILOC in common
!     with the left element but no other sides
!
! ********************************************************************
!
!               SUBROUTINES CALLED
!
!     TO0072    adjust INDEX1 to the new element
!     TO0073    adjust INDEX2 to the new element
!
! ********************************************************************

      call eropen ( 'to0079' )

!     initialise some parameters

      found = .false.
      ielem = 1

!     check if there is a sink in this element if so end_of_area = true

      if ( iloc.eq.0 ) ielem = nelem + 1

!     while not found and there still elements left

10    if ( ( .not. found ) .and. ( ielem.le.nelem ) ) then

!        check iloc edge with next element

         if ( ndim.eq.2 ) then
            do 20 j = 2, inpelm, 2
               if ( array((ielem-1)*inpelm+j).eq.index1(iloc*2) ) &
                  found = .true.
20          continue
         else if ( ndim.eq.3 ) then
            if ( iloc.eq.1 ) iplace = 13
            if ( iloc.eq.2 ) iplace = 14
            if ( iloc.eq.3 ) iplace = 12
            if ( iloc.eq.4 ) iplace = 11
            do 30 j = 11, 14
               if ( array((ielem-1)*inpelm+j).eq.index1(iplace) ) &
                  found = .true.
30          continue
         end if


!        if iloc edge is an edge of this element then

         if ( found ) then

!           check if this element differs from the left element

            found = .false.
            do 40 j = 1, inpelm
               found = (index1(j).ne.array((ielem-1)*inpelm+j)) .or. &
                       found
40          continue

!           if this element is different then fill index1 and index2

            if ( found ) then
               call to0072 ( index1, array, inpelm, ielem )
               call to0073 ( index2, index1, ndim )
            end if
         end if

         if ( .not. found ) ielem = ielem + 1

         go to 10
      end if

!     if no element was found end_of_area = true

      if ( ielem.gt.nelem ) iloc = 0

      call erclos ( 'to0079' )

      end
