      subroutine to0112clear ( vector, inodes, indprf, kprobf, nunkp, &
                               leninodes, npoint )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-11-2009
!
!   copyright (c) 2009-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Clear part of output array not corresponding to nodes in inodes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     reaction_force
!     clear
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inodes(*), indprf, kprobf(*), nunkp, leninodes, npoint
      double precision vector(*)

!     indprf         i    indication if kprob part f has been filled (>0)
!                         or not (0)
!     inodes         i    Contains nodes that must be kept
!     kprobf         i    KPROB part f: contains the number of unknowns per
!                         point accumulated
!     leninodes      i    length of array inodes
!     npoint         i    Number of nodal points
!     nunkp          i    Maximum number of unknowns in a point
!     vector        i/o   vector to be updated
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iwork(npoint), i, istart, inunkp

!     i              Counting variable
!     inunkp         Number of degrees of freedom in point
!     istart         First position used at a part
!     iwork          integer work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININNONZERO  Print the non-zero contents of an integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0112clear' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0112clear'
         call prinin ( inodes, leninodes, 'inodes' )
         write(irefwr,1) 'indprf, npoint', indprf, npoint
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill iwork with 1 for all nodes that must be used

      iwork = 0   ! Clear array iwork
      do i = 1, leninodes
         if ( inodes(i)>0 ) iwork(inodes(i)) = 1
      end do  ! i = 1, leninodes

      if ( debug ) call prininnonzero ( iwork, npoint, 'iwork' )

!     --- Clear vector for points with iwork(i) = 0

      if ( indprf==0 ) then

!     --- Constant number of degrees of freedom

         istart = 0       !last position used in vector
         do i = 1, npoint

            if ( iwork(i)==0 ) then

!           --- iwork(i) = 0, Clear vector

               vector(istart+1:istart+nunkp) = 0d0

            end if  ! ( iwork(i)==0 )
            istart = istart+nunkp

         end do  ! i = 1, npoint

      else

!     --- Non-constant number of degrees of freedom

         do i = 1, npoint

            if ( iwork(i)==0 ) then

!           --- iwork(i) = 0, Clear vector

               istart = kprobf(i)
               inunkp = kprobf(i+1)-istart
               vector(istart+1:istart+inunkp) = 0d0

            end if  ! ( iwork(i)==0 )

         end do  ! i = 1, npoint

      end if  ! ( indprf==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End to0112clear'

      end if  ! ( debug )
      call erclos ( 'to0112clear' )

      end
