      subroutine topmaptetra1 ( kmeshnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 30-11-2012 Long integers
!        version  1.0  date 29-11-2006
!
!   copyright (c) 2006-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map 3d mesh onto a mesh consisting of linear tetrahedrons
!     and hexahedrons only
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     interpolation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshnew

!     kmeshnew       o    New array kmesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nelemnew, ielgrp, i, inelemnew, inpelm, &
              inpelmnew, ishapenew, kelgrpnew, &
              kelmnewend, lenkmeshc, nelgrpnew, npelmnew
      double precision volume, dum(1)

!     dum            dummy array
!     i              Counting variable
!     ielgrp         element group number
!     inelemnew      inelem for new mesh
!     inpelm         number of points per element
!     inpelmnew      number of points per element for new mesh
!     ishapenew      Shape number of elements for new mesh
!     kelgrpnew      sum over all standard elements of number of points in
!                    standard elements
!     kelmnewend     last position used in kmeshnew
!     lenkmeshc      length of array kmeshc
!     nelemnew       Number of elements in new mesh
!     nelgrpnew      Number of element groups (new mesh)
!     npelmnew       Maximum number of points per element in new mesh
!     volume         Volume of mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision volmesh

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     TOPMAPTETRA2   Map 3d array kmeshc onto new array kmesh consisting of
!                    linear tetrahedrons only
!     VOLMESH        Compute the volume of a region
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute length of new array kmeshc
!     Create new array kmeshc
!     Fill new array kmesh
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topmaptetra1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topmaptetra1'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d16.8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) &
         write(irefwr,1) 'nelgrp, inelem', nelgrp, inelem
      if ( nelgrp>1 ) then

!     --- More than one element group
!         Skip line elements

         nelgrpnew = 0
         do ielgrp = 1, nelgrp
            ishape = kmeshf(ielgrp)
            if ( ishape>10 ) nelgrpnew = nelgrpnew+1
         end do  ! ielgrp = 1, nelgrp

      else

!     --- Only one element group

         nelgrpnew = 1

      end if  ! ( nelgrp>1 ) then
      km(iseqkm)%kmesh(5) = nelgrpnew
      allocate ( km(iseqkm)%kmeshf(nelgrpnew) )
      km(iseqkm)%kmesh(20) = nelgrpnew
      if ( kelma/=0 ) then
         allocate ( km(iseqkm)%kmesha(nelgrpnew+1) )
         km(iseqkm)%kmesh(15) = nelgrpnew+1
         km(iseqkm)%kmesha(1) = 0
      end if  ! ( kelma/=0 )
      if ( kelmg/=0 ) then
         allocate ( km(iseqkm)%kmeshg(nelgrpnew) )
         km(iseqkm)%kmesh(21) = nelgrpnew
      end if  ! ( kelmg>0 )

!     --- Compute length of new array kmeshc

      nelemnew = 0
      lenkmeshc = 0
      nelgrpnew = 0
      npelmnew = 0
      kelgrpnew = 0
      do ielgrp = 1, nelgrp
         ishape = kmeshf(ielgrp)
         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm
         if ( debug ) &
            write(irefwr,1) 'ielgrp, ishape, inelem, inpelm', &
                             ielgrp, ishape, inelem, inpelm

         if ( ishape<11 ) then

!        --- shape<11, hence not 3d element, skip

            go to 100

         end if  ! ( ishape<11 )
         nelgrpnew = nelgrpnew+1
         if ( debug ) &
            write(irefwr,1) 'ielgrp, ishape, inelem', &
                             ielgrp, ishape, inelem
         select case ( ishape )

            case ( 19, 20, 22: )

!           --- Shape number not available

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )
               go to 1000

           case(11, 18)

!          --- Linear tetrahedron

               inelemnew = inelem
               inpelmnew = 4
               ishapenew = 11

            case(12, 15, 16)

!           --- Quadratic tetrahedron

               inelemnew = 8*inelem
               inpelmnew = 4
               ishapenew = 11

            case(13, 17)

!           --- Trilinear hexahedron

               inelemnew = inelem
               inpelmnew = 8
               ishapenew = 13

            case(14)

!           --- Quadratic hexahedron

               inelemnew = 8*inelem
               inpelmnew = 8
               ishapenew = 13

            case(21)

!           --- Linear prism

               inelemnew = inelem
               inpelmnew = 6
               ishapenew = 21

         end select  ! case ( ishape )
         km(iseqkm)%kmeshf(nelgrpnew) = ishapenew
         if ( kelma/=0 ) &
            km(iseqkm)%kmesha(nelgrpnew+1) = &
               km(iseqkm)%kmesha(nelgrpnew)+inpelmnew
         if ( kelmg>0 ) km(iseqkm)%kmeshg(nelgrpnew) = inelemnew
         nelemnew = nelemnew+inelemnew
         lenkmeshc = lenkmeshc+inelemnew*inpelmnew
         npelmnew = max(npelmnew, inpelmnew)
         kelgrpnew = kelgrpnew+inpelmnew

100   end do  ! ielgrp = 1, nelgrp
      if ( debug ) write(irefwr,1) 'nelem, nelemnew', nelem, nelemnew

!     --- Create new array kmeshc

      allocate ( km(iseqkm)%kmeshc(lenkmeshc) )
      allocate ( km(iseqkm)%coor(ndim,npoint) )
      km(iseqkm)%coor = coor

      call topmaptetra2 ( km(iseqkm)%kmeshc )

!     --- Fill new array kmesh

      km(iseqkm)%kmesh(4) = npelmnew
      km(iseqkm)%kmesh(6) = 3           ! ndim
      km(iseqkm)%kmesh(8) = npoint
      km(iseqkm)%kmesh(9) = nelemnew
      km(iseqkm)%kmesh(13) = nvolms
      km(iseqkm)%kmesh(14) = kelgrpnew
      km(iseqkm)%kmesh(23) = npoint*ndim

1000  if ( debug ) then

!     --- Debug information

         volume = volmesh()
         write(irefwr,2) 'volume old mesh', volume
         call sepfillkmesh ( kmeshnew )
         volume = volmesh()
         call sepfillkmesh ( kmesh )
         write(irefwr,2) 'volume new mesh', volume
         write(irefwr,*) 'End topmaptetra1'

      end if  ! ( debug )
      call erclos ( 'topmaptetra1' )

      end subroutine topmaptetra1
