      subroutine tranpl ( ichois, kmeshlc, kproblc, isol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 29-05-2014 remove call to time subroutine
!        version  1.2  date 30-11-2012 Long integers
!        version  1.1  date 12-08-1999 New call to erclmn
!
!   copyright (c) 1999-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Transform from polar co-ordinates to cartesian co-ordinates
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulemain
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois, kmeshlc, kproblc, isol

!     ichois    i    choice parameter: Not yet used
!     isol      i    Standard SEPRAN array containing information of the solution
!     kmeshlc   i    Standard SEPRAN array containing information of the mesh
!     kproblc   i    Standard SEPRAN array containing information of the problem
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERTRAN         Transform from polar co-ordinates to cartesian
!                    co-ordinates
!     SEPACTSOLBF2   compute pointers to degrees of freedom of vectors
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      90   array isol has not been filled, or the boundary conditions have
!           not been stored in the solution vector
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tranpl' )
      if ( ks(isol)%typevec/=110 ) then
         call errint ( ks(isol)%typevec, 1 )
         call errsub ( 90, 1, 0, 0 )
      end if

      kmesh2d(1) = kmeshlc
      kmesh => kmesh2d(1)
      call sepfillkmesh(kmesh)
      kprob2d(1) = kproblc
      kprob => kprob2d(1)

!     --- reserve space for standard arrays

      call sepactsolbf2 ( isol )
      usoltemp => ks(isol)%sol
      if ( ierror/=0 ) go to 1000
      call ertran ( usoltemp )
1000  call erclos ( 'tranpl' )
      end subroutine tranpl
