      subroutine msh2dq ( bcord, coorlc1, kmeshclc1, plotfm, yfact, &
                          nbspec, nispec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  8.0  date 02-09-2010 Remove intarmsh
!        version  7.1  date 21-05-2001 New call to pl0001
!        version  7.0  date 02-02-1997 Extra parameter intarmsh
!
!   copyright (c) 1984-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     plot one surface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplot
      use sepmodulekmesh
      use sepmodulemesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coorlc1(*), bcord(*), plotfm, yfact
      integer kmeshclc1(*), nbspec, nispec

!     bcord      i    array containing the coordinates of the nodal points
!                     in the boundary
!     coorlc1    i    array containing the coordinates of the surface
!     kmeshclc1  i    kmesh part c
!     nbspec     i    Number of points at the boundary of one side in the case
!                     of spectral elements
!     nispec     i    Number of internal points in the case
!                     of spectral elements
!     plotfm     i    length of the plot in centimeters
!     yfact      i    multiplication factor for the y-coordinates
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ikader, imark

!     ikader     helpvariable to store jkader temporarily
!     imark      helpvariable to store jmark temporarily
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS   Deconcatenate name from calling sequence
!     EROPEN   Concatenate name to calling sequence
!     PL0000   Compute length and width of plot
!     PL0001   Compute extreme values of mesh submitted
!     PLAFEP   Standard SEPRAN plot routine
!     PLELEM   Plot elements
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Compute xmin, xmax, ymin, ymax
!     Compute aleng and width
!     Start picture
!     Plot elements
!     Close plot dataset
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'msh2dq' )

      aleng = plotfm
      width = plotfm*yfact

!     --- compute xmin, xmax, ymin, ymax, aleng and width

      call pl0001 ( bcord, 2, nbndpt, xmin, xmax, ymin, ymax )
      if(jmax==0) call pl0000 ( plotfm, yfact )

!     --- Store kader value

      ikader = jkader
      if ( ikader==0 ) jkader = 10 + isurnr

!     --- Start picture

      call plafep(1, aleng, width, 0d0, 0d0, 0)
      jkader =  ikader
      imark  =  jmark
      jmark  =  3
      if ( nspec>0 ) then

!     --- nspec>0, Spectral elements

         if ( nbspec>0 ) then

!        --- Spectral element with extra boundary points found

            if ( ishape==3 ) then

!           --- Triangles, delete internal points

                nboun = inpelm - nispec
            else

!           --- Quadrilaterals

                if ( nispec==0 ) then
                     nboun = -1
                else
                     nboun = -2
                end if
            end if
         end if
      end if

!     --- Plot all the elements

      call plelem ( .true., coorlc1, kmeshclc1, 1, nelemlc, inpelm, &
                    nboun, .true., 2, .false. )

!     --- Reset jmark

      jmark  =  imark

!     --- Close plot dataset

      call plafep(2, 0d0, 0d0, 0d0, 0d0, 0)
      call erclos( 'msh2dq' )
      end
