      subroutine pl0000 ( plotfm, yfact )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 22-08-1996 Layout
!        version  3.0  date 12-04-1988 Reduction of parameter list
!        version  2.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!
!   copyright (c) 1983-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    compute the parameters aleng and width indicating the length
!    and the width of a plot respectively
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleconsts
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision plotfm, yfact

!    plotfm    i   Plotting factor. When PLOTFM < 0, then the length of
!                  the plot is computed by - (xmax-xmin) * PLOTFM
!                  and the width by - (ymax-ymin) * YFACT * PLOTFM
!                  else the length of the plot is equal to PLOTFM cm
!                  When by some mistake the user has given | PLOTFM | a value
!                  smaller than 1 cm, then the length is chosen equal to
!                       COMMON BLOCKS
!    yfact     i   Multiplication for the y-co-ordinates. With this
!                  parameter the user can influence the scale in y-direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision dx, dy, pltemp, yftemp

!    dx          Maximal difference between x- co-ordinates i.e. xmax - xmin
!    dy          Maximal difference between y- co-ordinates i.e. ymax - ymin
!                If yfact > 0 multiplied by YFACT
!    pltemp      Temporary value of plotfm.
!                If PLOTFM is too small pltemp gets the value of defplo
!    yftemp      Temporary value of yfact.
!                If YFACT is too small yftemp gets the value 1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!      First it is checked if the input of YFACT and PLOTFM is correct.
!      Two common errors are a value 0 given and a single precision
!      value given. In that case the actual value is smaller than 1d-30
!      (in absolute value)
!      If necessary the error is corrected and the new values are stored
!      in PLTEMP and YFTEMP
!      The values of ALENG and WIDTH are computed by the following formulas
!        dx = xmax - xmin
!        if yfact <= 0  then  dy = ymax - ymin  else  dy = yfact * (ymax-ymin)
!        if plotfm < 0  then  aleng = -dx * plotfm;  width = -dy * plotfm
!        else if dx >= dy  then  aleng = plotfm;  width = dy * plotfm/dx
!             if dy >= dx  then  width = plotfm;  aleng = dx * plotfm/dy
!        Finally a minimum length and width of 1 cm is assumed
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- In order to exclude errors of the user it is checked if
!         yfact and plotfm are larger than 1d-30 in absolute value
!         If not, default values are used

      if ( abs(plotfm).gt.1d-30 ) then
         pltemp = plotfm
      else if ( plotfm.gt.0 ) then
         pltemp = defplo
      else
         pltemp = -defplo
      end if
      if ( abs(yfact).gt.1d-30 ) then
         yftemp = yfact
      else if ( yfact.gt.0 ) then
         yftemp = 1d0
      else
         yftemp = -1d0
      end if

!     --- Compute dx and dy

      dx = xmax-xmin
      if( yftemp.le.0d0 ) then
         dy = ymax-ymin
      else
         dy = (ymax-ymin) * yftemp
      end if

!     --- Compute aleng and width

      if ( pltemp.lt.0d0 ) then

!     --- pltemp < 0   scale given

         aleng = -dx*pltemp
         width = -dy*pltemp
      else

!     --- pltemp > 0  length given

         if ( dx.le.dy ) then

!        --- dx <= dy

            aleng = dx * pltemp / dy
            width = pltemp
         else

!        --- dx > dy

            aleng = pltemp
            width = dy * pltemp / dx
         end if
      end if

!     --- Check if aleng and width are not too small.
!         If so correct

      aleng = max ( 1d0, aleng )
      width = max ( 1d0, width )
      end
