      subroutine plaf05 ( ichois, icolor, npolyg, polyg, ndimpol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.1  date 18-09-2013 Set dcolor for ichois = 2
!        version  5.0  date 25-04-2010 Remove parameter work
!        version  4.1  date 01-09-2006 Allow plot coordinates
!
!   copyright (c) 1986-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot polygon, or fill polygon with color depending on the
!     value of ichois
! **********************************************************************
!
!                       KEYWORDS
!
!     clipping
!     color
!     plot
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmodulecons
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois, icolor, npolyg, ndimpol
      double precision polyg(ndimpol,npolyg)

!     ichois  i  Choice parameter. Possibilities:
!                1:  The interior of the polygon gets the color icolor
!                2:  The boundary of the polygon gets the color icolor
!                3:  See 2, however in this case the boundary is not closed
!     icolor  i  number indicating the type of color to be used
!     ndimpol i  Declared first parameter of polyg (>= 2)
!     npolyg  i  number of points in the polygon
!     polyg   i  array of length 2*npolyg containing the co-ordinates
!                of the points in the polygon in the sequence:
!                x1, y1, x2, y2, ... , xnpolyg, ynpolyg
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icount, iside, npoint
      double precision xp, yp
      real work(2,2*npolyg)

!     i              Counting variable
!     icount         Counter for created polygon
!     iside          Indicates at which side the cross section is positioned
!                    Possible values:
!                    0: There is no cross section point computed in this call
!                    1: The cross section point is at side y=ymint
!                    2: The cross section point is at side x=xmaxt
!                    3: The cross section point is at side y=ymaxt
!                    4: The cross section point is at side x=xmint
!     npoint         Number of points in curve
!     work       real single precision work aray of length 4*npolyg to
!                store the co-ordinates of the points of the polygon
!                in plot co-ordinates
!     xp             x-coordinate of vertex of polygon in plotting units
!     yp             y-coordinate of vertex of polygon in plotting units
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAF051        Fill array work with plot coordinates of polygon
!                    Coordinates are given in user coordinates
!     PLAFP6         Standard machine dependent plotting
!     PLAFP8         Machine dependent plotting of filling of polygons
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   In case of filled colors:
!   icount := 0
!   For i := 1 (1) npolyg+1 do
!      if ( i=npolyg+1 ) then
!         j := 1
!      else
!         j := i
!      Compute position of point j
!      If ( both new and old point within rectangle ) then
!         icount := icount + 1
!         Store point in work(icount)
!      else if ( new point in region and old point outside ) then
!         xpboun := cross section of line new_point - old_point with region
!         if ( icount>0 and work(icount) not on same boundary ) then
!            if ( there is no boundary between the two boundary points) then
!               icount := icount + 1
!               Store common vertex in work(icount)
!         icount := icount + 1
!         work(icount) := xpboun
!      else if ( old point in region and new point outside ) then
!         xp := cross section of line new_point - old_point with region
!         if ( j=1 and work(icount) not on same boundary ) then
!            if ( there is no boundary between the two boundary points) then
!               icount := icount + 1
!               Store common vertex in work(icount)
!         icount := icount + 1
!         work(icount) := xp
!      else if ( last point and at least two points found and the two
!                last points both outside the region ) then
!         if ( work(1) and work(icount) not on same boundary ) then
!            if ( there is no boundary between the two boundary points) then
!               icount := icount + 1
!               Store common vertex in work(icount)
!         icount := icount + 1
!         work(icount) := work(1)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plaf05' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plaf05'
         write(irefwr,1) 'iinputpst(102)', iinputpst(102)
         call prinrl1 ( polyg, npolyg, 2, 'polyg' )
  1      format ( a, 1x, (10i6) )
  3      format ( 2f12.4 )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icount = 0
      if ( jtest>=1 ) then

!     --- Set parameters for testing if jtest = 1

         jtest  = 2
         ioutsd = -1
      else
         ioutsd = 0
      end if
      if ( ichois<=2 ) then
         npoint = npolyg+1
      else
         npoint = npolyg
      end if

      if ( ichois==2 ) dcolor = icolor

      if ( iinputpst(102)==0 ) then

!     --- User coordinates

         call plaf051 ( npoint, npolyg, work, polyg, ndimpol, iside, &
                        jtest, ioutsd, icount )

      else

!     --- Plot coordinates, copy polyg into work

         do i = 1, npolyg
            work(1,i) = sngl(polyg(1,i))
            work(2,i) = sngl(polyg(2,i))
         end do ! i = 1, npolyg

         if ( npoint>npolyg ) then

!        --- npoint>npolyg, closed curve

            work(1,npoint) = work(1,1)
            work(2,npoint) = work(2,1)

         end if  ! ( npoint>npolyg )
         icount = npoint

      end if  ! ( iinputpst(102)>0 )

      if ( debug ) then
         write(irefwr,1) 'npoint, npolyg', npoint, npolyg
         write(irefwr,*) 'work'
         write(irefwr,3) (work(1,i), work(2,i), i = 1, icount)
      end if  ! ( debug )

      if ( icount>1 ) then

!     --- Check last co-ordinates

         if ( abs(work(1,icount)-work(1,icount-1))<sqreps .and. &
              abs(work(2,icount)-work(2,icount-1))<sqreps ) &
         icount = icount - 1
         if ( ichois==1 ) then
            call plafp8 ( ichois, icolor, icount, work )
        else
            xp = work(1,1)
            yp = work(2,1)
            call plafp6 ( 3, xp, yp )
            do i = 2, icount
               xp = work(1,i)
               yp = work(2,i)
               call plafp6 ( 2, xp, yp )
            end do
         end if
      end if
      jtest = min (jtest, 1)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plaf05'

      end if  ! ( debug )
      call erclos ( 'plaf05' )

      end
