      subroutine plbc03 ( ibrpnt, ibrelm, npoint, kmeshc, &
                          lbrpnt, nbndpt )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.0  date 27-09-2005 Make ineigh local
!        version  1.2  date 20-01-1996 New layout
!        version  1.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  1.0  date 10-08-1986
!
!   copyright (c) 1986-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   subroutine to fill array nbndpt with all the boundary elements, &
!   these are computed by the subroutine.
!   Also all internal bounds are registered
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     element
!     plot_contour
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibrpnt(*), ibrelm(*), kmeshc(3,*), nbndpt(2,*), &
              npoint, lbrpnt

!     ibrelm     i    pointer array for ibrelm
!     ibrpnt     i    helparray with for each node all elements it
!                     belongs to, sequentially
!     ineigh          set of new neighbours
!     kmeshc     i    kmesh part c, contains the nodal points corresponding
!                     to the elements
!     lbrpnt     o    number of boundary elements
!     nbndpt     o    array with the nodes of each boundary element sequentially
!     npoint     i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, istart, iend, j, ib, jelem, ielem, i1, i2, i3, is, &
              ielem1, ielem2, ineigh(50)

!     i          Counting variable
!     i1
!     i2
!     i3
!     ib
!     ielem      Element number
!     ielem1
!     ielem2
!     iend       Last position used at a part
!     is
!     istart     First position used at a part
!     j          Counting variable
!     jelem
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PLBC05         determine the, at most 2, elements where both nodes i1 and
!                    i2 belong to
!     PLBC07         fill a neighbour node of node i in array ineigh
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plbc03' )
      lbrpnt=0
      do i=1,npoint

!     --- determine elements of point i

         istart=ibrpnt(i)
         iend  =ibrpnt(i+1)-1
         do j = 1, 50
            ineigh(j) = 0
         end do
         ib=0
         do jelem=istart,iend
            ielem=ibrelm(jelem)
            i1=kmeshc(1,ielem)
            i2=kmeshc(2,ielem)
            i3=kmeshc(3,ielem)
            if ( i<i1 ) call plbc07(ineigh,ib,i1)
            if ( i<i2 ) call plbc07(ineigh,ib,i2)
            if ( i<i3 ) call plbc07(ineigh,ib,i3)
            call inichkarlen ( ib, 50, 'ineigh' )
         end do
         if ( ib>0 ) then
            do j=1,ib
               is=ineigh(j)
               call plbc05(ibrelm,ibrpnt,i,is,ielem1,ielem2)
               if ( ielem/=0 .and. ielem2==0 ) then
                  lbrpnt=lbrpnt+1
                  nbndpt(1,lbrpnt)=i
                  nbndpt(2,lbrpnt)=is
               end if
            end do
         end if
      end do
      call erclos ( 'plbc03' )
      end
