      subroutine plbc11 ( inodes, coorp, istep, npoint, ichois, icolor )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  6.0  date 21-11-2012 Extra parameter icolor
!        version  5.0  date 22-01-1997 Extra parameter ichois
!        version  4.0  date 20-01-1996 Extra parameter npoint
!
!   copyright (c) 1986-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     plot contour with points coorp, length of parts is given in
!     array inodes
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     plot
!     plot_contour
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inodes(*), istep, npoint, ichois, icolor
      double precision coorp(*)

!     coorp      i    see plbc10
!     ichois     i    if ichois=4, the labels are not plotted
!     icolor     i    Sequence number of colour
!     inodes     i    see plbc10
!     istep      i    number of level considered
!     npoint     i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision dr, dx, dy, factx, facty, r1, r2, x, xcentr1, &
                       xold, y, ycentr1, yold
      integer icoor, ikn, ital, jkn, jnbr, dcolorsav

!     dcolorsav      Help parameter to save the value of dcolor
!     dr
!     dx             length of vector in x-direction
!     dy             length of vector in y-direction
!     factx
!     facty
!     icoor
!     ikn
!     ital
!     jkn
!     jnbr
!     r1
!     r2
!     x              x co-ordinate of point
!     xcentr1
!     xold
!     y              y co-ordinate of point
!     ycentr1
!     yold
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRCHR    Put character in error message
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     PLAFEP    General SEPRAN low level plotting subroutine
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1699   Array too small
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plbc11' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plbc11'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      dcolorsav = dcolor

      icoor=0
      jkn=1
      ital=inodes(1)
200   if ( ital/=0 ) then
         x=coorp(2*icoor+1)
         y=coorp(2*icoor+2)
         call plafep(4, x, y, 0d0, 0d0, 0)
         if ( ital<0 ) then
            ital=-ital
         end if
         jnbr=(ital+1)/2
         if ( jnbr<3 ) jnbr=0
         do ikn=1, ital
            icoor=icoor+1
            if ( icoor>2*npoint+50 ) then
               call eropen ( 'plbc11' )
               call errint ( 2*icoor, 1 )
               call errint ( 4*npoint+100, 2 )
               call errchr ( 'coorp', 1 )
               call errsub ( 1699, 2, 0, 1 )
               call erclos ( 'plbc11' )
               go to 1000
            end if
            x=coorp(2*icoor-1)
            y=coorp(2*icoor  )
            if ( ichois/=4 .and. ikn==jnbr ) then

!           --- plot number

               xold=coorp(2*icoor-3)
               yold=coorp(2*icoor-2)
               dy=y-yold
               dx=x-xold
               dr=sqrt(dx*dx+dy*dy)
               if ( dr<=0d0 ) then
                  jnbr=jnbr+1
               else
                  factx=(xmax-xmin)/2.5d2
                  facty=(ymax-ymin)/2.5d2
                  r1=(dx/dr)*facty
                  r2=(dy/dr)*factx
                  if ( xold<x ) then
                     xcentr1=(x+xold)/2d0-r2
                     ycentr1=(y+yold)/2d0+r1
                  else
                     xcentr1=(x+xold)/2d0+r2
                     ycentr1=(y+yold)/2d0-r1
                  end if
                  call plafep ( 10, xcentr1, ycentr1, 0d0, 0d0, istep )
                  call plafep ( 4, xold, yold, 0d0, 0d0, 0 )
               end if
            end if

            dcolor = icolor
            call plafep ( 5, x, y, 0d0, 0d0, 0 )

         end do
         jkn=jkn+1
         ital=inodes(jkn)
         goto 200

      end if
      dcolor = dcolorsav

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plbc11'

      end if  ! ( debug )
      call erclos ( 'plbc11' )

      end
