      subroutine plbc24 ( usol, ihelp, icont, coorp, inodes, w, coor, &
                          ncoorc, coorcn, kncoor, kcoorn, npoint )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  3.0  date 20-01-1996 New layout, extra parameter
!        version  2.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  2.0  date 30-03-1987
!
!   copyright (c) 1986-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   subroutine to determine the coordinates of the contourlines
!
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     element
!     plot_contour
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ihelp(*), inodes(*), ncoorc(*), npoint, kncoor, kcoorn, &
              icont
      double precision usol(*), coorp(*), coor(*), coorcn(*), w

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     coorcn     o    At output this array contains the co-ordinates of the
!                     points on the contours in the sequnce as given in NCOORC.
!                     These co-ordinates are stored from position 6 in the
!                     sequence (x1,y1), (x2,y2) etc. COORCN is a variable length
!                     array, hence the first position must be filled by the user
!     coorp      o    coordinates of points on contour
!     icont      i    length of ihelp
!     ihelp      i    array containing the line elements which the
!                     contour crosses
!     inodes     o    array with the number of points on each part of
!                     the contour ( if negative then contour is closed
!     kcoorn     i    ?
!     kncoor     i    ?
!     ncoorc     i    This variable length array contains at output for every
!                     contour the following information from position 6
!                     For any contour level n+1 positions are used, where n is
!                     the number of parts the contour levels contain. A part is
!                     a continuous line. The contours are stored sequentially, &
!                     starting with contour level 1.
!                     pos. 1  contains the number n for the contour level (n>=1)
!                     pos. 2- n+1 contain of each part the numbe of points in
!                     the contour level part of which the co-ordinates are
!                     stored in array COORCN.
!                     If the contour part is a closed contour, then minus the
!                     number of points on the contour is stored.
!                     So the information of contour level 1 is stored in NCOORC
!                     from position 6 to position 6 + n1 etc.
!     npoint     i    Number of nodal points
!     usol       i    array containing the function values in the nodes
!     w          i    contour level value
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ikn, jkn, i1, icnt, icirc, ital, i2, i
      double precision eps, x, u1, u2, al, x1, x2, y1, y2, y

!     al
!     eps        Local accuracy
!     i          Counting variable
!     i1
!     i2
!     icirc      indicator whether curve is open (icirc=0) or
!                closed (icirc=1)
!     icnt
!     ikn
!     ital
!     jkn
!     u1
!     u2
!     x          x co-ordinate of point
!     x1
!     x2
!     y          y-coordinate
!     y1
!     y2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRCHR    Put character in error message
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1699   Array too small
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      eps=10d0*epsmac
      ikn=1
      jkn=1
      i1=ihelp(1)
      icnt=2
      if ( icont.le.2 ) then
         inodes(jkn)=0
         go to 1000
      end if
100   if ( i1.eq.0 ) then
         icirc=1
      else
         icirc=-1
      end if
      ital=0
200   i1=ihelp(icnt+1)
      i2=ihelp(icnt+2)
      icnt=icnt+2
      if ( icnt.gt.4*npoint+100 ) then
         call eropen ( 'plbc24' )
         call errint ( icnt, 1 )
         call errint ( 4*npoint+100, 2 )
         call errchr ( 'ihelp', 1 )
         call errsub ( 1699, 2, 0, 1 )
         call erclos ( 'plbc24' )
         go to 1000
      end if
      if ( i1.le.0 ) then
         inodes(jkn)=icirc*ital
         jkn=jkn+1
         goto 100
      else
         u1=usol(i1)
         u2=usol(i2)

!        --- interpolation

         if ( abs(u1).lt.eps .and. abs(u2).lt.eps ) then
            al=0.5d0
         else if ( abs(u1-u2).lt.eps ) then
            al=0.5d0
         else
            al=(w-u1)/(u2-u1)
         end if
         if ( al.lt.eps ) then
            al=eps
         end if
         if ( al.gt.( 1d0-eps ) ) then
            al=1d0-eps
         end if
         x1=coor(2*i1-1)
         x2=coor(2*i2-1)
         y1=coor(2*i1  )
         y2=coor(2*i2  )
         x=x1+al*(x2-x1)
         y=y1+al*(y2-y1)
         ital=ital+1
         coorp(2*ikn-1)=x
         coorp(2*ikn  )=y
         ikn=ikn+1
         if ( icnt.lt.2*icont ) goto 200
         inodes(jkn)=icirc*ital
         inodes(jkn+1)=0
      end if
      ncoorc(kncoor)=jkn
      do 300 i=1,jkn
         ncoorc(kncoor+i)=inodes(i)
300   continue
      kncoor=kncoor+jkn+1
      do 310 i=1,ikn*2
         coorcn(i+kcoorn)=coorp(i)
310   continue
      kcoorn=kcoorn+2*ikn
1000  continue
      end
