      subroutine plbc34 ( usol, contln, ncntln, &
                          number, nphys, indcol, special, elgrps )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.4  date 25-04-2010 New call to plaf05
!        version  2.3  date 31-10-2003 Correction nboun
!        version  2.2  date 05-04-2001 Correct number of levels
!        version  2.1  date 05-04-1996 New call to plaf05
!
!   copyright (c) 1995-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make coloured contour plot of a function defined per element in 2D
!
! **********************************************************************
!
!                       KEYWORDS
!
!     coordinate
!     fill
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ncntln, nphys, number, &
              indcol(ncntln+1), special, elgrps(*)
      double precision usol(nphys,*), contln(ncntln)

!     contln  i    User array of length ncntln in which the
!                  user is supposed to have stored the contour levels
!     elgrps  i    Array of length nelgrp that is only used if special=2
!                  In that case elgrps(ielgrp)=0 means skip element
!                  and elgrps(ielgrp)=1 is treat element
!     indcol  i    array containing the colour numbers of the various
!                  contour levels
!     ncntln  i    Parameter to compute the number of contour levels
!     nphys   i    Number of physical unknowns, or maximal degrees of freedom
!                  per point
!     number  i    The number-th unknown in each nodal point of the array
!                  is considered. If the array is complex number=2*i-1
!                  corresponds to the real part of array i and number=2*i
!                  to the imaginary part of array i
!     special i    If special=2 array elgrps may be used, otherwise
!                  special=0 not
!     usol    i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inpelm, ikelmc, nboun, &
              ielem, ielgrp, irelem, nodp, icolor
      double precision polyg(2,9), value

!     i       Counting variable
!     icolor  Sequence number of colour
!     ielem   Element number
!     ielgrp  element group number
!     ikelmc  position in kmesh part c
!     inpelm  number of points per element
!     irelem  Relative element sequence number with respect to element group
!     nboun   Number of boundary points in an element
!     nodp    Nodal point sequence number
!     polyg   Array containing the co-ordinates of the point in the polygon
!             in the following way:
!             polyg(1,i) contains the x-coordinate of the first point
!             polyg(2,i) contains the y-coordinate of the first point
!     value   value in nodal point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     PLAF05  Plot polygon
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      ikelmc = 0
      ielem  = 1
      do ielgrp = 1, nelgrp
         inpelm = Topgrp(ielgrp)%inpelm
         nboun  = inpelm
         if ( kelmb>0 ) nboun  = kmeshb(ielgrp)
         inelem = Topgrp(ielgrp)%inelem
         ishape = kmeshf(ielgrp)
         if ( ishape<3 .or. ishape>9 ) then
            ielem = ielem+inelem
            ikelmc = ikelmc+inpelm*inelem
            inelem = 0
         else if ( special==2 ) then
            if ( elgrps(ielgrp)==0 ) then
               ielem = ielem+inelem
               ikelmc = ikelmc+inpelm*inelem
               inelem = 0
            end if
         end if
         do irelem = 1, inelem
            value = usol(number,ielem)

!           --- Determine colour

            icolor = 1
            if ( value>contln(ncntln) ) then
               icolor = ncntln
            else if ( value>contln(1) ) then
               do i = 2, ncntln-1
                  if ( value>contln(i) ) icolor = icolor+1
               end do
            end if
            do i = 1, nboun
               nodp = kmeshc(ikelmc+i)
               polyg(1,i) = coor(1,nodp)
               polyg(2,i) = coor(2,nodp)
            end do
            polyg(1,nboun+1) = polyg(1,1)
            polyg(2,nboun+1) = polyg(2,1)
            call plaf05 ( 1, indcol(icolor), nboun+1, polyg, 2 )
            ikelmc = ikelmc + inpelm
            ielem  = ielem  + 1
         end do

      end do

      end
