      subroutine plbc35 ( coor, v, procor, xmin, xmax, ymin, ymax, &
                          plotfm, kmeshc, nelem, icolor, nprcor )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.1  date 21-11-1994 Norms
!        version  2.0  date 30-12-1992 Extra parameter nprcor
!        version  1.2  date 27-06-1992 Initialise tst and ten
!
!   copyright (c) 1990-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot background lines in case of a 3-D projection
!     Go from xmin to xmax with a small discrete constant step and
!     plot all vertical lines which are not hidden by the projection
!     of the total mesh.
! **********************************************************************
!
!                       KEYWORDS
!
!     background
!     plot
!     line
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(*), nelem, icolor, nprcor
      double precision coor(*), v(3), procor(nprcor,*), plotfm,  &
                       xmin, xmax, ymin, ymax

!     coor    i    array of length 3 x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     icolor  i    colour of lines in plot
!     kmeshc  i    kmesh part c, contains the nodal points corresponding to the
!                  elements
!     nelem   i    Number of elements
!     nprcor  i    Dimension of first index of procor
!                  If nprcor = 3, pl0014 is called by a colour subroutine, &
!                  which implies that also the distance to the view-point is
!                  computed, if nprcor = 2, only the projection is computed
!     plotfm  i    Plotting factor. When PLOTFM < 0, then the length of
!                  the plot is computed by - (xmax-xmin) * PLOTFM
!                  and the width by - (ymax-ymin) * YFACT * PLOTFM
!                  else the length of the plot is equal to PLOTFM cm
!                  When by some mistake the user has given | PLOTFM | a value
!                  smaller than 1 cm, then the length is chosen equal to
!                  the default length stored in common block consta
!     procor  o    coordinate array for projected points
!                  procor(1,i) contains the x-coordinate of the projected point
!                  procor(2,i) contains the y-coordinate of the projected point
!                  procor(3,i) contains the distance of point to the view-point
!                  i is the point number
!     v       i    array containing the co-ordinates of the view point
!     xmax    o    Maximum x-value
!     xmin    o    Minimum x-value
!     ymax    o    Maximum y-value
!     ymin    o    Minimum y-value
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nhor, i, ivis, iview, idr, iv, kp, lp, mp
      double precision dx, tst, ten, xa, ya, xb, yb, eps, xp, xl, xm, &
                       xmi, xma, xao, yao, zao, xbo, ybo, zbo, &
                       xst, yst, xen, yen, view(100)

!     dx      length of vector in x-direction
!     eps     local accuracy
!     i       loop variable
!     idr     loop variable for triangles
!     iv      loop variable for array view
!     iview   number of visible parts of line in array view
!     ivis    indicator whether parts are visible or not
!     kp      first node of triangle
!     lp      second node of triangle
!     mp      third node of triangle
!     nhor    number of background lines to be considered
!     ten     end of line part
!     tst     start of line part
!     view    help array for visibility of 100 positions
!     xa      x-coordinate first point of line in projected coordinates
!     xao     x-coordinate in fysical plane
!     xb      x-coordinate second point of line in projected coordinates
!     xbo     x-coordinate in fysical plane
!     xen     x-coordinate end point in projection plane
!     xl      x-coordinate point lp
!     xm      x-coordinate point mp
!     xma     maximum value of x-coordinates of triangle
!     xmi     minimum value of x-coordinates of triangle
!     xp      ?
!     xst     x-coordinate start point
!     ya      y-coordinate first point of line in projection plane
!     yao     y-coordinate first point in fysical plane
!     yb      y-coordinate second point of line in projection plane
!     ybo     y-coordinate second point in fysical plane
!     yen     y-coordinate end point in projection plane
!     yst     y-coordinate start point
!     zao     z-coordinate first point line in fysical plane
!     zbo     z-coordinate second point line in fysical plane
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Deconcatenate name of subroutine from calling string
!     EROPEN  Concatenate name of subroutine to calling string
!     ERRINT  Put integer in error message
!     ERRSUB  Submit error message
!     PLAFEP  General SEPRAN low level plotting subroutine
!     PLBC26  Subroutine to determine whether part of a line is covered by
!             a triangle in projection-plane
!     PLT352  Cancel part of a segment of line that is not visible
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     935     Declared length of array view too small
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Run through all background curves
!     Check whether curve is (partly) visible
!     Plot pieces that are visible of curve
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('plbc35')
      eps = 10 * epsmac

!     --- Plot all enveloping lines if wanted

      nhor = nint( 10 * plotfm )
      tst  = 0d0
      ten  = 0d0
      dx   = ( xmax - xmin ) / ( nhor + 1d0 )
      do 200 i = 1 , nhor

!     --- Clear array VIEW

          ivis = 0
          call plt352( ivis, tst, ten, view, iview )

!         --- Beginpoint line

          xa = xmin + i * dx + 3 * eps
          ya = ymin

!         --- Endpoint line

          xb = xa
          yb = ymax
          do 180 idr = 1 , nelem
              kp = kmeshc( 3 * idr - 2 )
              lp = kmeshc( 3 * idr - 1 )
              mp = kmeshc( 3 * idr     )

!             --- Determine extreme x-values of triangle

              xp  = procor( 1, kp )
              xl  = procor( 1, lp )
              xm  = procor( 1, mp )
              xmi = min( xp, xl, xm )
              xma = max( xp, xl, xm )
              if ( xmi.lt.xa .and. xma.gt.xa ) then

!             --- Interesting case , look for intersection points

                   call plbc26( v, xa, ya, xao, yao, zao, &
                                xb, yb, xbo, ybo, zbo, &
                                kp, lp, mp, coor, procor, &
                                ivis, tst, ten, 1, nprcor )

!                  --- Cancel non-visible part

                   if ( ivis.eq.2 ) then

!                  --- Nothing is visible

                        goto 200
                   else if ( ivis.gt.2 ) then

!                  --- Line is partly visible

                        call plt352(ivis,tst,ten,view,iview)
                   end if
              end if
180       continue
          if ( iview.gt.49 ) then

!         --- Declared length too small

               call errint( iview, 1 )
               call errsub ( 935, 1, 0, 0 )
          end if

!         --- Run through all pieces that are visible and plot

          do 190 iv = 1 , iview
              tst = view( 2 * iv - 1 )
              ten = view( 2 * iv     )
              if ( (ten -tst).gt.5.d-3 ) then

!             --- Compute starting point of this piece

                   xst = xa + tst * ( xb-xa )
                   yst = ya + tst * ( yb-ya )

!                  --- Move pen to starting point

                   call plafep( 4, xst, yst, 0d0, 0d0, 0 )

!                  --- Compute coordinates end point of this piece

                   xen = xa + ten * ( xb - xa )
                   yen = ya + ten * ( yb - ya )

!                  --- Plot from begin to end point for this piece

                   call plafep( 5, xen, yen, 0d0, 0d0, icolor )
             end if
190       continue
200   continue
      call erclos('plbc35')
      end
