      subroutine plfillviewp ( irotation, jv, viewp, viewpoint, iorien, &
                               numrotations )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-10-2001
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array viewpoint with the viewpoint depending on the rotation
!     sequence number
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     fill
!     rotation
!     view_point
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleconsts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer irotation, jv, iorien, numrotations
      double precision viewp(3), viewpoint(3)

!     iorien         i    Defines orientation of the base vectors in 3D plots
!                         1 x-y-z
!                         2 z-x-y
!                         3 y-z-x
!     irotation      i    Loop variable for rotation
!     jv             i    Indicates if the view point has been filled (1)
!                         or not (0)
!     numrotations   i    Number of rotations
!     viewp         i/o   Array containing the first view point
!     viewpoint      o    Array containing actual view point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision arcphi

!     arcphi         Angle of rotation
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( irotation.eq.1 .and. jv.eq.1 ) then

!     --- First rotation, set viewpoint

         do i = 1, 3
            viewp(i) = viewpoint(i)
         end do

      else if ( irotation.gt.1 ) then

!     --- Next rotation, adapt viewp

         jv = 1
         arcphi = 2d0*pi*(irotation-1)/numrotations
         if ( iorien.eq.1 ) then

!        --- Rotation in x-y plane

            viewpoint(1) = cos(arcphi)*viewp(1)- &
                           sin(arcphi)*viewp(2)
            viewpoint(2) = sin(arcphi)*viewp(1)+ &
                           cos(arcphi)*viewp(2)
            viewpoint(3) = viewp(3)

         else if ( iorien.eq.2 ) then

!        --- Rotation in y-z plane

            viewpoint(1) = viewp(1)
            viewpoint(2) = cos(arcphi)*viewp(2)- &
                           sin(arcphi)*viewp(3)
            viewpoint(3) = sin(arcphi)*viewp(2)+ &
                           cos(arcphi)*viewp(3)

         else

!        --- Rotation in z-x plane

            viewpoint(1) = sin(arcphi)*viewp(3)+ &
                           cos(arcphi)*viewp(1)
            viewpoint(2) = viewp(2)
            viewpoint(3) = cos(arcphi)*viewp(3)- &
                           sin(arcphi)*viewp(1)

         end if

      end if
      end
