      subroutine plfl04 ( xp, yp, ep, labda, x1, y1, e1, eps, f )
! ======================================================================
!
!        programmer    Arno van Duin
!        version  1.0  date 12-07-1991
!
!   copyright (c) 1991-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision e1, ep, eps, f, labda, x1, xp, y1, yp

!     e1      i    electric field in the point (x1,y1)
!     ep     i/o   electric field in point (xp,yp)
!     eps     i    dielectric constant
!     f       i    required flux-cut
!     labda   o    distance between (xp,yp) and the new point
!     x1      i    x-coordinate of the point on the equal potential line
!     xp     i/o   i: x-coordinate of starting point on the equal potential
!                     line
!                  o: x-coordinate of point with flux-cut F from the old (xp,yp)
!     y1      i    y-coordinate of the point on the equal potential line
!     yp     i/o   i: y-coordinate of starting point on the equal potential
!                     line
!                  o: y-coordinate of point with flux-cut F from the old (xp,yp)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision a, b, c, e1abs, epabs, error, length
      parameter ( error = 1d-9 )

!     a       coefficient of labda^2 of the quadratic equation
!     b       coefficient of labda^1 of the quadratic equation
!     c       coefficient of labda^0 of the quadratic equation
!     e1abs
!     epabs
!     error   allowed error for a = 0
!     length  distance between (xp,yp) and (x1,y1)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!         Approximate electric field with straight line
!         E = (E1 - Ep) * labda/length + Ep     0 <= labda <= length
!         Then
!         F = eps * labda * 1/2 * ( Ep + Enew ) = &
!           = eps * labda * 1/2 * ( Ep + (E1 - Ep) * labda/length + Ep )   <=>
!         (E1-Ep)/length * labda^2 + 2*Ep * labda - 2*F/eps = 0
!         Solve quadratic equation of labda.
!         PSEUDO CODE
!         Equation : a * labda^2 + b * labda + c = 0
!         Where a = (E1 - Ep) / length
!               b = 2 * Ep
!               c = -2 * F / eps
!               length = sqrt( (x1-xp)^2 + (y1-yp)^2 )
!         If a = 0 then
!            Labda = -c / b
!         Else
!            Labda = ( -b + sqrt ( b^2 - 4ac ) ) / 2a
!         Endif
!         xnew = ( x1 - xp ) * labda / length + xp
!         ynew = ( y1 - yp ) * labda / length + yp
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      length = sqrt( (xp-x1) * (xp-x1) + (yp-y1) * (yp-y1) )
      E1abs  = abs(E1)
      Epabs  = abs(Ep)
      if ( abs(E1abs - Epabs).lt.error * Epabs ) then
         labda = F / (Epabs * eps)
      else
         a     = (E1abs - Epabs) / length
         b     = 2 * Epabs
         c     = -2 * F / eps
         labda = (-b + sqrt(b*b - 4*a*c)) / (2 * a)
      end if
      xp = (x1 - xp) * labda / length + xp
      yp = (y1 - yp) * labda / length + yp
      end
