      subroutine plfr20 ( iplots, rplots )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.1  date 25-11-2012 Long integers
!        version  2.0  date 09-11-1999 Extra parameters ibuffr, buffer
!        version  1.5  date 05-06-1998 Include commons
!
!   copyright (c) 1992-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine plfr20 is used to plot all the curves of a 3D SEPRAN
!     mesh. At the users wish also all elements in a surface can be
!     plotted. It is possible to specify that one of the curves is plotted
!     with a different colour or that the surface is plotted either
!     as a wireframe or as a solid.
! **********************************************************************
!
!                       KEYWORDS
!
!     projection
!     wire_frame
!     solid_plot
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iplots(*)
      double precision rplots(*)

!     iplots  i   Integer array in which the following quantities must be
!                 stored:
!                 1:  Highest position that has been filled by the user
!                 2:  choice parameter for type of projection
!                     Possible values:
!                      0 : parallel projection plot
!                      1 : perspective projection plot
!                     10 : parallel projection plot (View point automatic)
!                     11 : perspective projection plot (View point automatic)
!                     Default value: 1
!                 3:  First number of colour
!                 4:  Second number of colour
!                 5:  Number of curve or surface that has to be plotted with
!                     the special colour (second). The other curves/surfaces
!                     are plotted with the first colour
!                 6:  Indicator whether curves only (0) or curves and
!                     surfaces (1) have to be plotted
!                 7:  Indicator whether KMESH part N has to be used, or
!                     parts C and T.
!                 8:  Indicator for basic coordinate system
!                        1 :   x - y - z
!                        2 :   z - x - y
!                        3 :   y - z - x
!     rplots  i   Double precision input array that must be filled as follows:
!                 1:    plotfm
!                 2:    yfact
!                 3-5:  eye-point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer indica

!     indica  Helpvariable to store iplots(6)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     PLFR21  Do the actual plotting as required
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    581     dimension not 3
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Check some parameters
!     Activate permanent arrays
!     Call plfr21 to do all the plotting
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plfr20' )

      if ( ndim/=3 ) then
         call errint ( ndim, 1 )
         call errsub ( 581, 1, 0, 0 )
      end if

      if ( ierror/=0 ) go to 1000

      if ( iplots(1)<6 ) then

         indica = 0

      else

         indica = iplots(6)

      end if

      if ( indica/=0 ) then

!     --- Determine number of surfaces and activate either KMESH part N
!         or KMESH part T

         if ( kelmn/=0 ) then

            iplots(7) = 0

         else if ( kelmt/=0 ) then

            if ( iplots(1)<7 ) iplots(1) = 7

            iplots(7) = 1

         end if

      else

         nsurfs = 0

      end if

!     --- Run through all the curves and plot the curves and run through all
!         the surfaces and plot (eventually) the surfaces

      call plfr21 ( iplots, rplots )

1000  if ( ierror/=0 ) then

!     --- ierror # 0  Reset ierror and nerror
!                     Return to main program

         ierror = 0
         nerror = 0
         if ( jtimes==11 ) jtimes = 1
         go to 1010

      end if

      if ( iwarn/=0 ) then

!     --- iwarn # 0  Reset iwarn and nwarn
!                    Return to main program

         iwarn = 0
         nwarn = 0

      end if

1010  call erclos ( 'plfr20' )

      end
