      subroutine plfr22 ( kelem, nelemh, npelmlc, xmean, vact, &
                          ski, eta, ipl6, isitua )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.2  date 25-04-2010 New call to plaf05
!        version  2.1  date 05-04-1996 New call to plaf05
!        version  2.0  date 06-04-1995 Spectral elements and basic
!                                      coordinate system added
!
!   copyright (c) 1993-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot all the elements of a surface of a SEPRAN mesh in 3-D either
!     as a solid or as wireframe (spectral elements are allowed)
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     projection
!     3d
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ipl6, npelmlc, nelemh, kelem(npelmlc,nelemh), &
              isitua
      double precision xmean(3), vact(3), ski(3), eta(3)

!     eta     i   Second basis vector of projection plane
!     ipl6    i   Indicator whether elements must be plotted as solids (=2)
!                 or as wireframes (=1)
!     isitua  i   Indicator for coordinate basis
!                   1  :  x - y - z
!                   2  :  z - x - y
!                   3  :  y - z - x
!     kelem   i   Standard SEPRAN array containing the elements
!     nelemh  i   Number of elements in the mesh
!     npelmlc i   Number of nodes in the elements
!     ski     i   First basis vector of projection plane
!     vact    i   Coordinates of actual viewpoint
!     xmean   i   Coordinates of center of projection plane
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, in(4), nisrf, is(3)
      double precision etac(4), skic(4), polyg(2,4), &
                       x(4), xao, y(4), yao, z(4), zao

!     etac       coordinates (first) in projection plane
!     i          loop variable
!     in         node numbers
!     is         array with temporary ordering (isitua)
!     nisrf      help variable to store the number of surface points
!     polyg      array with coordinates of plotting points
!     skic       coordinates (second) in projection plane
!     x          x-coordinates of nodes
!     xao        help variable ( not really used here)
!     y          y-coordinates of nodes
!     yao        coordinate (not really used here)
!     z          z-coordinates of nodes
!     zao        coordinate (not really used here)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     PLAF05  Plot polygon
!     PLT351  Compute coordinates in projection plane
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Run through all surfaces and plot the projection of all the
!     elements of the surface
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plfr22' )
      if ( isitua==2 ) then
           is(1) = 3
           is(2) = 1
           is(3) = 2
      else if ( isitua==3 ) then
           is(1) = 2
           is(2) = 3
           is(3) = 1
      else
           is(1) = 1
           is(2) = 2
           is(3) = 3
      end if

!     --- Run through all elements

      do 100 i = 1 , nelemh
          if ( kelmu>0 ) then

!         --- Spectral elements

               ishape = kmeshf(1)
               if ( ishape==11 .and. nisurf==-1 ) then
                    if ( niedge==0 ) then
                         nisrf = 0
                    else
                         nisrf = 1
                    end if
               else if ( ishape==13 .and. nisurf==-1 ) then
                    nisrf = niedge * niedge
               else
                    nisrf = nisurf
               end if
               if ( npelmlc==3*(1+niedge) .or. &
                    npelmlc==3*(1+niedge)+nisrf ) then

!              --- Triangle

                  in(1) = kelem( 1              , i )
                  in(2) = kelem( 2 +     niedge , i )
                  in(3) = kelem( 3 + 2 * niedge , i )
                  in(4) = in(1)
               else if ( npelmlc==4*(1+niedge) .or. &
                         npelmlc==(2+niedge)*(2+niedge) ) then

!              --- Quadrilateral, not filled (nisurf = 0 ) or filled

                  in(1) = kelem( 1                  , i )
                  in(2) = kelem( 2 + niedge         , i )
                  in(3) = kelem( npelmlc              , i )
                  in(4) = kelem( npelmlc - 1 - niedge , i )
               end if
          else if ( npelmlc==3 .or. npelmlc==4 ) then

!         --- Linear case

               in(1) = kelem ( 1, i )
               in(2) = kelem ( 2, i )
               in(3) = kelem ( 3, i )
          else if ( npelmlc==6 .or. npelmlc==7 .or. &
                    npelmlc==9 ) then

!         --- Quadratic case

               in(1) = kelem ( 1, i )
               in(2) = kelem ( 3, i )
               in(3) = kelem ( 5, i )
          end if

!         --- Compute projection of the three points

          x(1) = coor(is(1),in(1)) - xmean(1)
          y(1) = coor(is(2),in(1)) - xmean(2)
          z(1) = coor(is(3),in(1)) - xmean(3)
          call plt351( vact, x(1), y(1), z(1), ski, eta, &
                       xao, yao, zao, skic(1), etac(1) )
          x(2) = coor(is(1),in(2)) - xmean(1)
          y(2) = coor(is(2),in(2)) - xmean(2)
          z(2) = coor(is(3),in(2)) - xmean(3)
          call plt351( vact, x(2), y(2), z(2), ski, eta, &
                       xao, yao, zao, skic(2), etac(2) )
          x(3) = coor(is(1),in(3)) - xmean(1)
          y(3) = coor(is(2),in(3)) - xmean(2)
          z(3) = coor(is(3),in(3)) - xmean(3)
          call plt351( vact, x(3), y(3), z(3), ski, eta, &
                       xao, yao, zao, skic(3), etac(3) )
          if ( npelmlc==3 .or. npelmlc==6 .or. &
               npelmlc==7 ) then

!         --- Triangle no extra point

             skic(4) = skic(3)
             etac(4) = etac(3)
          else

!         --- Quadrilateral

             if ( kelmu>0 ) then

!            --- Do nothing, in(4) has been filled already

             else if ( npelmlc==4 ) then
                in(4) = kelem( 4, i )
             else if ( npelmlc==9 ) then
                in(4) = kelem( 7, i )
             end if

!            --- Compute projection of fourth point

             x(4) = coor(is(1),in(4)) - xmean(1)
             y(4) = coor(is(2),in(4)) - xmean(2)
             z(4) = coor(is(3),in(4)) - xmean(3)
             call plt351( vact, x(4), y(4), z(4), ski, eta, &
                          xao, yao, zao, skic(4), etac(4) )
          end if

!         --- Plot the projection of the element

          polyg(1,1) = skic(1)
          polyg(2,1) = etac(1)
          polyg(1,2) = skic(2)
          polyg(2,2) = etac(2)
          polyg(1,3) = skic(3)
          polyg(2,3) = etac(3)
          polyg(1,4) = skic(4)
          polyg(2,4) = etac(4)

!         --- Determine whether as solid or as wireframe

          if ( ipl6==1 ) then
             call plaf05( 2, 2, 4, polyg, 2 )
          else if ( ipl6==2 ) then
             call plaf05( 1, 2, 4, polyg, 2 )
          end if
100   continue
      call erclos('plfr22')
      end
