      subroutine plotbounnodes ( inbndnod, coor, icolor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-08-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Plot boundary nodes
!      Meant for test purposes only
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     boundary
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inbndnod, icolor
      double precision coor(4,*)

!     coor           i    array of length 4 x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering positions (3-4,*) are not used
!     icolor         i    Sequence number of colour
!     inbndnod       i    number of coordinates of nodes in coor
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PL0000         Compute length and width of the plot
!     PLAFEP         General SEPRAN low level plotting subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotbounnodes' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotbounnodes'
         write(irefwr,*) 'jtimes ', jtimes

      end if

!     --- Open plot

      call pl0000 ( 15d0, 1d0 )
      if ( ierror.ne.0 ) go to 1000
      call plafep( 1, aleng, width, 1d0, 15d0, 1 )
      if ( ierror.ne.0 ) go to 1000

      call plafep ( 4, coor(1,inbndnod), coor(2,inbndnod), 0d0, 0d0, &
                    icolor )
      if ( ierror.ne.0 ) go to 1000
      do i = 1, inbndnod
         call plafep ( 5, coor(1,i), coor(2,i), 0d0, 0d0, icolor )
      end do
      if ( ierror.ne.0 ) go to 1000

      do i = 1, inbndnod
         call plafep ( 6, coor(1,i), coor(2,i), 0d0, 0d0, icolor )
      end do
      if ( ierror.ne.0 ) go to 1000

      call plafep( 2, 0d0, 0d0, 0d0, 0d0, 0 )
      if ( ierror.ne.0 ) go to 1000

1000  call erclos ( 'plotbounnodes' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotbounnodes'

      end if

      end
