      subroutine plotbox
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-08-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot small box below the picture
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     frame
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepost
      use sepmodulecons
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      character (len=80) charac
      double precision xs, ys, amax, hx, dx, hy, dy, h, dx1
      integer nchar

!     amax           Maximum value
!     charac         String variable to store a text
!     dx             width
!     dx1            width
!     dy             length
!     h              length
!     hx             length
!     hy             length
!     nchar          Number of characters in a string
!     xs             Scale factor in x-direction
!     ys             Scale factor in y-direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAF03         Convert real number to character format
!     PLAFP6         Standard machine dependent plotting
!     PLAFP7         Plot text using absolute co-ordinates
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotbox' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotbox'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set scale factors

      xs=1d0/xfct
      ys=1d0/yfct
      amax = 1d2

!     --- Define some distances

      hx = 25d0
      hy = 5d0
      dy = hy/3d0
      h = 2d0*dy
      dx = hx

!     --- plot first block

      call plafp6 ( 3, 0d0, 0d0 )
      call plafp6 ( 2, hx, 0d0 )
      call plafp6 ( 2, hx, hy )
      call plafp6 ( 2, 0d0, hy )
      call plafp6 ( 2, 0d0, 0d0 )

!     --- horizontal line (at 1/3 of small box)

      call plafp6 ( 3, 0d0, dy )
      call plafp6 ( 2, hx, dy )

!     --- vertical line

      dx1 = dx-5d0
      call plafp6 ( 3, dx1, 0d0 )
      call plafp6 ( 2, dx1, hy )

!     --- 1 horizontal small line

      call plafp6 ( 3, dx1, h )
      call plafp6 ( 2, hx, h )

!     --- 2 vertical small lines

      dx1 = dx-2.5d0
      call plafp6 ( 3, dx1, 0d0 )
      call plafp6 ( 2, dx1, dy )
      call plafp6 ( 3, dx1, 2*dy )
      call plafp6 ( 2, dx1, hy )

      if ( iinputpst(72)==0 ) then

!     --- plot xscale,  yscale

         dx1 = dx - 4.5d0
         call plafp7 ( dx1, 4.3d0, 'SCALEX', 0d0, 0.25d0, 6 )
         nchar = 0
         call plaf03 ( xs, charac, 3, nchar, amax )
         call plafp7 ( dx1, 3.7d0, charac, 0d0, 0.25d0, nchar )
         dx1 = dx - 2d0
         call plafp7 ( dx1, 4.3d0, 'SCALEY', 0d0, 0.25d0, 6 )
         nchar = 0
         call plaf03 ( ys, charac, 3, nchar, amax )
         call plafp7 ( dx1, 3.7d0, charac, 0d0, 0.25d0, nchar )

      end if  ! ( iinputpst(72)> 0 )

!     --- plot sepran  and special texts

      call plafp7 ( 0.4d0, 0.4d0, 'INGENIEURSBUREAU SEPRA', 0d0, &
                    0.4d0, 22 )
      call plafp6 ( 3, 0d0, 4d0 )
      call plafp6 ( 2, 3.35d0, 4d0 )
      call plafp6 ( 2, 3.35d0, 5d0 )
      call plafp7 ( .3d0, 4.3d0, 'SEPRAN', 0d0, 0.3d0, 6 )

!     --- change origin

      call plafp6 ( 4, 0d0, hy )
      call plafp6 ( 3, 0d0, 0d0 )

!     --- plot last frame

      hy = 20d0
      call plafp6 ( 2, hx, 0d0 )
      call plafp6 ( 2, hx, hy )
      call plafp6 ( 2, 0d0, hy )
      call plafp6 ( 2, 0d0, 0d0 )

1000  call erclos ( 'plotbox' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotbox'

      end if  ! ( debug )

      end
