      subroutine plotcompscal3 ( minout, maxout, step )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-05-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Round minout and maxout to round values, related to step
!
! **********************************************************************
!
!                       KEYWORDS
!
!     rounding
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision minout, maxout, step

!     maxout        i/o   Scaled valmax (will be rounded)
!     minout        i/o   Scaled valmin (will be rounded)
!     step           i    Step along axis
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision quotient, remainder, diff

!     diff           maxout-maxin
!     quotient       quotient of diff and step
!     remainder      remainder of diff and quotient times step
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotcompscal3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotcompscal3'
         write(irefwr,2) 'minout, maxout, step', minout, maxout, step
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( abs(minout)>0d0 ) then

!     --- minout # 0, try to round if necessary

         quotient = nint(minout/step)
         remainder = minout-quotient*step
         if ( debug ) &
            write(irefwr,2) 'quotient, remainder', quotient, remainder
         if ( abs(remainder)>0.01d0*step ) then

!        --- remainder>1% of step, rounding is necessary

            if ( remainder<0d0 ) then

!           --- remainder<0, make minout equal to quotient*step

               minout = (quotient-1d0)*step

            else

!           --- remainder>0, make minout equal to (quotient-1)*step

               minout = quotient*step

            end if  ! ( remainder<0d0 )

         end if  ! ( abs(remainder)>0.01d0*step )

      end if  ! ( abs(minout)>0d0 )

      diff = maxout-minout
      quotient = nint(diff/step)
      remainder = diff-quotient*step
      if ( debug ) &
         write(irefwr,2) 'quotient, remainder', quotient, remainder

      if ( abs(remainder)>0.01d0*step ) then

!     --- remainder>1% of step, rounding is necessary

         if ( remainder<0d0 ) then

!        --- remainder<0, make maxout equal to minout+quotient*step

            maxout = minout+quotient*step

         else

!        --- remainder>0, make maxout equal to minout+(quotient+1)*step

            maxout = minout+(quotient+1d0)*step

         end if  ! ( remainder<0d0 )

      end if  ! ( abs(remainder)>0.01d0*step )

1000  call erclos ( 'plotcompscal3' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'minout, maxout, step', minout, maxout, step
         write(irefwr,*) 'End plotcompscal3'

      end if  ! ( debug )

      end
