      subroutine plotedgenr ( coor, nmeshc, nedges )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-05-2001
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Plot the edge sequence numbers in a mesh where the elements have been
!    plotted already (2d only)
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     edge
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor(2,*)
      integer nmeshc(*), nedges

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     nedges         i    number of edges
!     nmeshc         i    nmesh part c
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision x, y
      integer iedge, nodp1, nodp2, istart, npoint

!     iedge          edge sequence number
!     istart         First position used at a part
!     nodp1          Node sequence number
!     nodp2          Node sequence number
!     npoint         Number of nodal points in edge
!     x              x co-ordinate of point
!     y              y co-ordinate of point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAFEP         General SEPRAN low level plotting subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all edges do
!        Find begin and end point of edge
!        Compute centre of edge
!        PLot edge sequence number
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotedgenr' )

!     --- For all edges do

      do iedge = 1, nedges

!     --- Find begin and end point of edge

         istart = nmeshc(iedge)
         npoint = nmeshc(istart)
         nodp1 = nmeshc(istart+1)
         nodp2 = nmeshc(istart+npoint)

!        --- Compute centre of edge

         x = 0.5d0*(coor(1,nodp1)+coor(1,nodp2))
         y = 0.5d0*(coor(2,nodp1)+coor(2,nodp2))

!        --- PLot edge sequence number

         call plafep (7, x, y, -0.06d0, 0.06d0, iedge )

      end do

      call erclos ( 'plotedgenr' )

      end
