      subroutine plotfn1 ( iplots, rplots, ivectr, &
                           icurvs, funcx, funcy )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.5  date 29-05-2014 Extension of timebefore
!        version  2.4  date 25-11-2012 Long integers
!        version  2.3  date 26-10-2009 Debug statements
!
!   copyright (c) 1997-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    plot function along curves or user function
!    the function either has to be a sepran array of type 100, 115 or 119
!    or a set of user arrays of type 117
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      use sepmoduleplot
      use sepmoduleplotfun
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iplots(*), ivectr(*), icurvs(*)
      double precision rplots(*), funcx(*), funcy (*)

!    funcx    i   user array of type 117 to be used when ichois=1 (coor)
!    funcy    i   user array of type 117 to be used when ichois=1 (func)
!    icurvs   i   definition of the curves by the user.
!                 position 1 gives the type of indication
!                 the next positions contain the curve numbers
!                 values for icurvs(1):
!                   0       only one curve is used
!                  -1       the function is plotted along curves icrv1 to icrv2
!                  -2       see -1, however in reversed order
!                  >0       the function is plotted along the curves icrv1, &
!                           icrv2, . . .  , icrvn
!    iplots   i   Integer input array for subroutine PLOTFNBF
!                 IPLOTS must be filled as follows:
!                 1:   Highest entry that has been filled by the user
!                 2:   ipict  sequence number of the function to be plotted
!                             Must be between 1 and npict (inclusive end-points)
!                      Default value: 1
!                 3:   npict  number of functions to be plotted in one picture
!                      Default value: 1
!                 4:   ichfun  Indication how the function is given.
!                             Possibilities:
!                             0 standard situation
!                               the input vector may be of type 110, 115 or 119
!                               when itype=110 or 115 array icurvs is used
!                             1 the input is stored in the arrays funcx and
!                               funcy of type 117 (user arrays)
!                             2 see ichfun=0, however, the minimum and maximum x
!                               and y values xmin,xmax,ymin and ymax are not
!                               computed from the functions to be plotted, but
!                               from the mesh boundaries    (itype=119 only)
!                             3 see 2, boundaries of the mesh are also plotted
!                             4 see 3, the initial- and end-points of curves
!                               to be plotted are connected by a straight line.
!                             5 See 0, the user gives the min and maximum values
!                               of s in funcx(1) resp funcx(2)
!                      Default value: 0
!                 5:   isymbl  Choice parameter indicating with what symbols the
!                             functions are plotted. Isymbl may be different
!                             for every call of plotfn1. Possibilities are:
!                              0     Only straight lines are plotted
!                              1-10  Points will be connected with a line and
!                                    marked with a symbol, system depended
!                             11-20  Only the same symbols are used, no lines
!                      Default value: 0
!                 6:   jpicho   Indicator to use a second y-scale along the
!                             right-hand side
!                              0  Only one y-scale is used.
!                             >0  A second scale will be made
!                      Default value: 0
!                 7:   number  the number th degree of freedom in each nodal
!                             point is plotted
!                      Default value: 1
!                 8:   itypescales  type of scales used along each of the
!                             axis. Is subdivided into 2 or 3 parts according
!                             to itypescales = itype1+10*itype2
!                             where itype1 refers to the x-axis, itype2 to the
!                             y-axis
!                             The following values for itypex are available
!                              0  Linear scale
!                              1  Logarithmic scale (base 10)
!                      Default value: 0
!    ivectr   i   array containing information of the function to be
!                 plotted
!    rplots   i   Real user input array for subroutine PLOTFNBF
!                 RPLOTS must be filled as follows:
!                 1:  plotfm   length of the plot in the x-direction in cm.
!                              when plotfm<0 the length is equal to
!                              -plotfm * (xmax-xmin)
!                 2:  yfact    scale factor in y-direction
!                              when yfact>0 the height of the plot is equal to
!                              |plotfm| x yfact x (xmax-xmin) / (ymax - ymin)
!                              when yfact<0 the height of the plot is equal to
!                              -yfact cm
!                              when yfact=0, yfact=1 is assumed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kiesy1, kiesy2, isigy1, isigy2, nstpy1, nstpy2, &
              jpicho, isymbl, ichfun, kx, ky, ipwork, &
              ntext, kader, kchois, jtestr, ipic, icolsv, &
              n, indexl, i, m, ileft, modifys, ipict, &
              npict, itypescales, itypescalx, itypescaly, number, &
              ntextloc
      double precision valext(4), xpmin, xpmax, ypmin, ypmax, ay1min, &
                       ay1max, ay2min, ay2max, dx, dy, dy2, y2min, &
                       y2max, width2, y1min, y1max, ddy, &
                       plotfm, yfact
      character (len=20) texty1, texty2
      character (len=78) text
      character (len=3) label
      double precision :: timebefore(2)
      save xpmin, xpmax, ypmin, ypmax, ay1min,ay1max,ay2min, &
           ay2max,texty1,texty2,kiesy1,kiesy2,isigy1,isigy2,nstpy1, &
           nstpy2

!     ay1max         storage of aymax for first scale
!     ay1min         storage of aymin for second scale
!     ay2max         storage of aymax for first scale
!     ay2min         storage of aymin for second scale
!     ddy            temporary margin while computing ymin etc.
!     dx             temporary margin while computing xmin etc.
!     dy             temporary margin while computing ymin etc.
!     dy2            temporary margin while computing ymin etc.
!     i              general purpose loop and counter variable
!     ichfun         See input/output parameters ICHOIS
!     icolsv         Help variable to store the present colour
!     ileft          indication if left (0) or right y-scale (1) should be used
!     indexl         Infor entry with respect to one curve to be plotted
!     ipic           local counter of pictures, used during plotting
!     ipict          sequence number of the function to be plotted
!                    Must be between 1 and npict (inclusive end-points)
!     ipwork         Starting address of real information of curves in IBUFFR
!     isigy1         local storage of common variable isigy (first y-scale)
!     isigy2         local storage of common variable isigy (second y-scale)
!     isymbl         See input/output parameters ICHOIS
!     itypescales    type of scales used along each of the axis.
!                    Is subdivided into 2 according
!                    to itypescales = itype1+10*itype2+100*itype3
!                    where itype1 refers to the x-axis, itype to the
!                    y-axis
!                    The following values for itypex are available
!                     0  Linear scale
!                     1  Logarithmic scale (base 10)
!     itypescalx     Type of scale in x-direction, see itypescales
!     itypescaly     Type of scale in y-direction, see itypescales
!     jpicho         See input/output parameters ICHOIS
!     jtestr         help variable to remember old value of jtest
!     kader          temporary storage of common variable jkader
!     kchois         choice parameter for plscls routine
!     kiesy1         Local storage of common variable kiesy (first scale)
!     kiesy2         Local storage of common variable kiesy (second scale)
!     kx             local storage of kiesx
!     ky             local storage of kiesy
!     label          Label to be printed
!     m              Number of points along a curve
!     modifys        If 1 the minimum and maximum s coordinates are stored in
!                    funcx
!     n              Number of points along a curve
!     npict          number of functions to be plotted in one picture
!     nstpy1         Local storage of common variable nstepy (first scale)
!     nstpy2         Local storage of common variable nstepy (second scale)
!     ntext          flag for plscls if text is to be plotted
!     ntextloc       Number of characters in local text
!     number         the number th degree of freedom in each nodal point
!                    is plotted
!     plotfm         length of the plot in the x-direction in cm.
!                    when plotfm<0 the length is equal to
!                    -plotfm * (xmax-xmin)
!     text           Text to be plotted at bottom line
!     texty1         Temporary storage of text to be plotted along first y-axis
!     texty2         Temporary storage of text to be plotted along second y-axis
!     valext         In this array the extreme values of function and
!                    co-ordinates are stored:
!                    Pos. 1 xmin
!                    Pos. 2 xmax
!                    Pos. 3 ymin
!                    Pos. 4 ymax
!     width2         width for second y-scale
!     xpmax          maximum x-value
!     xpmin          minimum x-value
!     y1max          storage of ymax for first y-scale
!     y1min          storage of ymin for first y-scale
!     y2max          storage of ymax for second y-scale
!     y2min          storage of ymin for second y-scale
!     yfact          scale factor in y-direction
!                    when yfact>0 the height of the plot is equal to
!                    |plotfm| x yfact x (xmax-xmin) / (ymax - ymin)
!                    when yfact<0 the height of the plot is equal to
!                    -yfact cm
!                    when yfact=0, yfact=1 is assumed
!     ypmax          maximum y-value
!     ypmin          minimum y-value
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLMN         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERSETTIME      Get the CPU time at start of subroutine
!     PL0002         Compute the parameters xmin, xmax, ymin and ymax
!     PL0004         Round minimum and maximum values
!     PL0009         Plot curve
!     PL0017         Fill function values and co-ordinates
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLAFP7         Plot text using absolute co-ordinates
!     PLMORE         User subroutine to plot extra information in the plot
!     PLOTTIME       Plot the actual time in a plot depending on the parameters
!                    itimdep and t in common ctimen
!     PLSCLS         Subroutine to plot scales and axis at a picture
!     PRINRL         Print 1d real vector
!     PRVCMATLAB     Print to matlab file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     858   ichfun out of range
!     859   isymbl out of range
!    2129   xmin<0 not allowed for logarithmic scale
!    2130   ymin<0 not allowed for logarithmic scale
! **********************************************************************
!
!                       PSEUDO CODE
!
!      determine choice values
!      Store co-ordinates and function values in IBUFFR
!      Compute minima and maxima
!      Store text parameters
!      if ( ipict=npict ) then
!         if ( two scales are used ) then
!            store text information about second scale
!         plot axis and compute sizes of plot
!         if ( jtimes>10 ) then
!            plot bottom text
!         plot all curves
!         Delete all locally defined arrays
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotfn1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotfn1'
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      if ( itime>0 ) call ersettime ( timebefore )

!     --- Set default values for parameters in iplots

      ipict = 1
      npict = 1
      ichfun = 0
      isymbl = 0
      jpicho = 0
      number = 1
      itypescales = 0
      ntextloc = min(3,len(textx))
      label = textx(1:ntextloc)

!     --- start with separating all variables given by iplots

      if ( iplots(1)>=2 ) ipict = iplots(2)
      if ( iplots(1)>=3 ) npict = iplots(3)
      if ( iplots(1)>=4 ) ichfun = iplots(4)
      if ( iplots(1)>=5 ) isymbl = iplots(5)
      if ( iplots(1)>=6 ) jpicho = iplots(6)
      if ( iplots(1)>=7 ) number = iplots(7)
      if ( iplots(1)>=8 ) itypescales = iplots(8)
      itypescalx = mod(itypescales,10)
      itypescaly = itypescales/10
      plotfm = rplots(1)
      yfact = rplots(2)
      if ( ichfun==5 ) then
         modifys = 1
         ichfun = 0
      else
         modifys = 0
      end if
      if (ichfun<0 .or. ichfun>4) then
         call errint ( 4, 1 )
         call errint ( ichfun, 2 )
         call errint ( 0, 3 )
         call errint ( 4, 4 )
         call errsub ( 858, 4, 0, 0 )
      end if
      if (isymbl<0 .or. isymbl>20)  then
         call errint ( 5, 1 )
         call errint ( isymbl, 2 )
         call errint ( 0, 3 )
         call errint ( 20, 4 )
         call errsub ( 859, 4, 0, 0 )
      end if

!     --- store locally data of pictures

      if (ipict==1) then
         xpmin = rinfin
         xpmax = -rinfin
         ypmin = rinfin
         ypmax = -rinfin
         if (kiesx==-1) then
            axmin = rinfin
            axmax = -rinfin
         end if
         if (kiesy==-1) then
            aymin = rinfin
            aymax = -rinfin
            ay2min = rinfin
            ay2max = -rinfin
         end if

      end if

      if ( ipict==1 ) then

!     --- First picture, allocate ilocal

         allocate ( ilocal(npict), stat = error )
         if ( error/=0 ) call eralloc ( error, npict, 'ilocal' )
         if ( ierror/=0 ) go to 1000

      end if  ! ( ipict==1 )

!     --- Store co-ordinates and function values in IBUFFR
!         compute extreme values

      ileft = min ( jpicho, 1 )
      call pl0017 ( ivectr, ipict, &
                    npict, isymbl, ichfun, funcx, funcy, number, icurvs, &
                    valext, ileft, modifys )

      if ( debug ) then
         call prinrl ( valext, 4, 'valext' )
         write(irefwr,1) 'kiesx, kiesy', kiesx, kiesy
      end if  ! ( debug ) then

      xpmin = min(valext(1),xpmin)
      xpmax = max(valext(2),xpmax)
      ypmin = min(valext(3),ypmin)
      ypmax = max(valext(4),ypmax)
      if (kiesx==-1) then
         axmin = min(xpmin,axmin)
         axmax = max(xpmax,axmax)
      end if
      if ( ipict==jpicho .and. kiesy==-1 ) then
         ay2min = min(ypmin,ay2min)
         ay2max = max(ypmax,ay2max)
         if ( ipict==1 ) then
            aymin = ay2min
            aymax = ay2max
         end if
      else if ( kiesy==-1 ) then
         aymin = min(ypmin,aymin)
         aymax = max(ypmax,aymax)
      end if

!     --- save values of first scale

      if (ipict==1) then
         ay1min = aymin
         ay1max = aymax
         kiesy1 = kiesy
         isigy1 = isigy
         nstpy1 = nstepy
         texty1 = texty
      end if
      icolsv = dcolor
      if ( ierror/=0 ) then
         ierror = 0
         go to 1000
      end if

      if (ipict==npict) then

!     --- ipict = npict   plot values
!         save values of last plot (second y scale)

         if (jpicho>0) then
            ay2min = aymin
            ay2max = aymax
            kiesy2 = kiesy
            isigy2 = isigy
            nstpy2 = nstepy
            texty2 = texty
            aymin = ay1min
            aymax = ay1max
            kiesy = kiesy1
            isigy = isigy1
            nstepy = nstpy1
            texty = texty1
         end if

!        --- plot axis

         kx = kiesx
         ky = kiesy

         if ( debug ) then
            write(irefwr,1) 'kiesx, kiesy, ichfun', kiesx, kiesy, ichfun
         end if  ! ( debug ) then

         if (ichfun>=2) then
            if ( jmax==0 ) then
               call pl0002
            end if
            ntext = 1
            if (kiesx==-1) then
               axmin = xmin
               axmax = xmax
            end if
            if (kiesy==-1) then
               aymin = ymin
               aymax = ymax
            end if

         else

            if ( debug ) write(irefwr,1) 'itypescalx', itypescalx
            if ( itypescalx==1 ) then

!           --- Use logarithmic scale in x-direction

               if ( axmin>0d0 ) then

!              --- xmin>0, standard

                  axmin = log10(axmin)
                  i = nint(axmin)
                  if ( axmin<i ) then
                     axmin = i-1
                  else
                     axmin = i
                  end if
                  axmax = log10(axmax)
                  i = nint(axmax)
                  if ( axmax>i ) then
                     axmax = i+1
                  else
                     axmax = i
                  end if
                  nstepx = nint(axmax)-nint(axmin)

               else

!              --- xmin<=0, error

                  call erreal ( axmin, 1 )
                  call errsub ( 2129, 0, 1, 0 )

               end if

            else

!           --- Use linear scale in x-direction

               if (kiesx==-1) call pl0004(axmin,axmax,kiesx,isigx)
               if ( debug ) then
                  write(irefwr,1) 'kiesx, isigx', kiesx, isigx
                  write(irefwr,2) 'axmin,axmax', axmin,axmax
               end if  ! ( debug )

            end if

            if ( itypescaly==1 ) then

!           --- Use logarithmic scale in y-direction

               if ( aymin>0d0 ) then

!              --- ymin>0, standard

                  aymin = log10(aymin)
                  i = nint(aymin)
                  if ( aymin<i ) then
                     aymin = i-1
                  else
                     aymin = i
                  end if
                  aymax = log10(aymax)
                  i = nint(aymax)
                  if ( aymax>i ) then
                     aymax = i+1
                  else
                     aymax = i
                  end if
                  nstepy = nint(aymax)-nint(aymin)

               else

!              --- ymin<=0, error

                  call erreal ( aymin, 1 )
                  call errsub ( 2130, 0, 1, 0 )

               end if

            else

!           --- Use linear scale in y-direction

               if (kiesy==-1) then
                  call pl0004(aymin,aymax,kiesy,isigy)
                  if ( debug ) then
                     write(irefwr,1) 'kiesy, isigy ', kiesy, isigy
                     write(irefwr,2) 'aymin,aymax', aymin,aymax
                  end if  ! ( debug )

                  if (jpicho/=0.and.kiesy2==-1) &
                    call pl0004(ay2min,ay2max,kiesy2,isigy2)
               end if

            end if
            dx = (axmax-axmin)*.1d0
            dy = (aymax-aymin)*.1d0
            if (jpicho/=0) dy2 = (ay2max-ay2min)*.1d0
            if (jkader==-1 .or. jkader==-3 .or.jkader==-5) then
               xmax = axmax+dx*0.1d0
               ymax = aymax+dy*0.1d0
               xmin = axmin-dx*0.3d0
               ymin = aymin-dy*0.3
               if (jpicho/=0) then
                  y2min = ay2min-dy2*0.3d0
                  y2max = ay2max+dy2*0.1d0
               end if
            else
               xmin = axmin-dx
               ymin = aymin-dy
               xmax = axmax+dx
               ymax = aymax+dy
               if (jpicho/=0) then
                  y2min = ay2min-dy2
                  y2max = ay2max+dy2
               end if
            end if
            ntext = -1

         end if

!        --- compute length and width of the plot

         if (plotfm>0) then
            aleng = plotfm
         else
            aleng = -plotfm*(axmax-axmin)
         end if
         if (yfact>=0d0) then
            width = aleng*(aymax-aymin)/(axmax-axmin)*yfact
            if (jpicho/=0) width2 = aleng*(ay2max-ay2min)/ &
                                      (axmax-axmin)*yfact
         else
            width = -yfact
            if (jpicho/=0) width2 = -yfact
         end if
         aleng = max(aleng,2.5d0)
         width = max(width,2.5d0)
         if (jpicho/=0) width2 = max(width2,2.5d0)

!        --- plot start and axis

         dcolor = 0
         kader = jkader
         if (jkader==0) jkader = 1
         if (jpicho==0) then
            call plafep(1,aleng,width,1d0,0d0,1)
            jkader = kader
            kchois = 1
            y1min = ymin
            y1max = ymax
         else
            y1min = ymin
            y1max = ymax
            ymin = y2min
            ymax = y2max
            width2 =  width
            call plafep(1,aleng,width,1d0,0d0,1)
            jkader = kader
            aymin = ay2min
            aymax = ay2max
            yfct = (y2max-y2min)/width2
            kiesy = kiesy2
            isigy = isigy2
            nstepy = nstpy2
            texty = texty2
            kchois = 3
            call plscls(ntext,kchois)
            width = width2
            ymin = y1min
            ymax = y1max
            yfct = (y1max-y1min)/width
            aymin = ay1min
            aymax = ay1max
            kiesy = kiesy1
            isigy = isigy1
            nstepy = nstpy1
            texty = texty1
            kchois = 2
         end if
         kchois = kchois+10*itypescales
         if ( debug ) &
            write(irefwr,1) 'kiesx, isigx, kiesy, isigy', &
                             kiesx, isigx, kiesy, isigy
         call plscls(ntext,kchois)

         if ( jtimes>=10 ) then

!        --- Plot bottom line with information of minimum and maximum values

            text(1:7) = 'xmin = '
            write( text(8:18),110 ) xpmin
110         format(ss,1pe11.3e2)
            text(19:27) = ', xmax = '
            write( text(28:38),110 ) xpmax
            text(39:47) = ', ymin = '
            write( text(48:58),110 ) ypmin
            text(59:67) = ', ymax = '
            write( text(68:78),110 ) ypmax

            if ( idirec==1 ) then

!           --- idirec = 1, normal direction

               call plafp7 ( 0d0, 0d0, text, 0d0, 0.15d0, 78 )

            else

!           --- idirec = -1, rotated direction

               call plafp7 ( 0d0, wid+2.5d0, text, -90d0, 0.15d0, 78 )

            end if

!           --- Plot the actual time if time dependent

            call plottime

         end if

!        --- plot the curves from end curve to initial one

         jtestr = jtest
         jtest = 1
         dx = 0.0001d0*(xmax-xmin)
         xmint = xmin-dx
         xmaxt = xmax+dx
         do ipic = npict, 1, -1
            n      = ilocal(ipic)%n
            isymbl = ilocal(ipic)%isymbl
            dcolor = ilocal(ipic)%dcolor
            ileft  = ilocal(ipic)%ileft
            if ( ileft>0 ) then
               ymin = y2min
               ymax = y2max
               yfct = (y2max-y2min)/width2
            else
               ymin = y1min
               ymax = y1max
               yfct = (y1max-y1min)/width
            end if
            ddy = 1d-4*(ymax-ymin)
            ymint = ymin-ddy
            ymaxt = ymax+ddy

            if ( n>0 ) then

!           --- n>0, standard situation

               call pl0009 ( isymbl, ilocal(ipic)%work, &
                             ilocal(ipic)%work(1+n), &
                             n, itypescales )
               deallocate ( ilocal(ipic)%work )

            else

!           --- n<0, special situation of types 119

               ipwork = 1
               do i = 1, -n
                  m = ilocal(ipic)%iwork(i)
                  call pl0009 ( isymbl, ilocal(ipic)%work(ipwork), &
                                ilocal(ipic)%work(ipwork+m),&
                                m, itypescales )
                  if ( mainsubr==104 ) then

!                 --- mainsubr = 104, call by cycpostexe

                     write(irefwr,*) '(plotfn1) calling prvcmatlab'
                     call prvcmatlab ( ilocal(ipic)%work(ipwork), &
                                       ilocal(ipic)%work(ipwork+m),&
                                       m, label )

                  end if  ! ( mainsubr==104 )
                  ipwork = ipwork+2*m

               end do

               deallocate (ilocal(ipic)%work, ilocal(ipic)%iwork )

            end if

         end do

         kiesx = kx
         kiesy = ky
         jtest = jtestr
         call plmore
         call plafep(2,0d0,0d0,0d0,0d0,0)

      end if

      if ( ipict==npict ) then

!     --- Last picture, deallocate ilocal

         deallocate ( ilocal, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'ilocal' )

      end if  ! ( ipict==npict )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotfn1'

      end if  ! ( debug )
      call erclmn ( 'plotfn1', timebefore )

      end subroutine plotfn1
