      subroutine plotfuncleg ( nlegenda, colours, rnumbers )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-05-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot legenda for function plot
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     legenda
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nlegenda, colours(*)
      double precision rnumbers(nlegenda)

!     colours        i    Contains the color numbers of the colours
!                         corresponding to the legenda
!     nlegenda       i    Number of items in legenda
!     rnumbers       i    Numbers to be plotted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iform, nchar
      double precision dx, dy, yund, xleft, xright, x, y, xmaxval, &
                       sizetxt, lenmarker, x1, ytop

!     dx             shift in x-direction
!     dy             shift in y-direction
!     i              Counting variable
!     iform          Defines format to be used
!                    -1:  fixed format, non-significant zeros are removed
!                     0:  fixed format
!                     1:  floating format
!     lenmarker      Length of marker
!     nchar          Number of characters in a word
!     sizetxt        Size of text in cm's
!     x              x co-ordinate of point
!     x1             x co-ordinate of point
!     xleft          shift to the left
!     xmaxval        Maximum value in x-direction
!     xright         shift to the right
!     y              y co-ordinate of point
!     ytop           Top y coordinate
!     yund           Lowest y coordinate
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLOTNUMBER     Plot a real number
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For all values do
!        Find representation
!        Plot small coloured line followed by number
!        Compute most right position
!        Move downwards
!     Plot box
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotfuncleg' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotfuncleg'
         write(irefwr,2) 'axmin, axmax, aymin, aymax', &
                          axmin, axmax, aymin, aymax
         write(irefwr,2) 'xfct, yfct', xfct, yfct
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      sizetxt = 0.2d0
      lenmarker = 0.2d0

!     --- Left upper part of starting position of legenda
!         May be imput in the future

      xleft = axmax+sizetxt*xfct
      ytop = aymax

!     --- Define steps

      dy = sizetxt*yfct*1.5d0
      dx = sizetxt*xfct
      y = aymax-dy
      if ( debug ) &
         write(irefwr,2) 'xleft, ytop, dx, dy', xleft, ytop, dx, dy

!     --- For all values do

      x = xleft+0.5d0*dx
      xmaxval = x
      do i = 1, nlegenda

!     --- Plot small coloured line followed by number
!         Compute most right position

         y = ytop-i*dy
         dcolor = colours(i)
         if ( debug ) write(irefwr,2) 'x, y, lenmarker', x, y, lenmarker

         call plafep ( 4, x, y, 0d0, 0d0, 0 )
         call plafep (16, x, y, lenmarker, 0d0, 0 )

         x1 = x+dx*1.5d0

!        --- Plot number

         dcolor = 1
         if ( abs(rnumbers(i))>0d0 .and. abs(rnumbers(i))<1d-3 .or. &
              abs(rnumbers(i))>1d4 ) then
            iform = 3
         else
            iform = -1
         end if  ! ( abs(rnumbers(i))>0d0 ... )
         if ( debug ) write(irefwr,1) 'iform', iform

         if ( debug ) write(irefwr,2) 'x1, y', x1, y

         call plotnumber ( x1, y, rnumbers(i), 1, sizetxt, 2, 0d0, &
                           -0.4d0*sizetxt, iform, nchar )
         xmaxval = max(xmaxval,x1+nchar*dx)

      end do  ! i = 1, nlegenda

!     --- Plot box

      dcolor = 1
      yund = ytop-(nlegenda+0.5d0)*dy
      xright = xmaxval+0.5d0*dx

      call plafep ( 4, xleft, ytop, 0d0, 0d0, 0 )  !Pen up
      call plafep ( 5, xright, ytop, 0d0, 0d0, 0 )  ! pen down
      call plafep ( 5, xright, yund, 0d0, 0d0, 0 )  ! pen down
      call plafep ( 5, xleft, yund, 0d0, 0d0, 0 )  ! pen down
      call plafep ( 5, xleft, ytop, 0d0, 0d0, 0 )  !Pen up

1000  call erclos ( 'plotfuncleg' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotfuncleg'

      end if  ! ( debug )

      end
