      subroutine plotmeshinter ( nelem, kmeshc, coor, icount, idim, &
                                 npoint, text1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 02-09-2010 New call to plelm1
!        version  2.0  date 07-07-2004 extra parameter npoint
!        version  1.0  date 24-02-2003
!
!   copyright (c) 2003-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot actual mesh during the process of creation of a mesh by msho2d
!     Subroutine meant for checking purposes
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     mesh_generation
!     plot
!     check
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem, kmeshc(3,*), icount, idim, npoint
      double precision coor(idim,*)
      character *(*) text1

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     icount         i    Counter
!     idim           i    Defines the first dimension for array coor
!                         Must be at least equal to 2
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     nelem          i    Number of elements
!     npoint         i    Number of nodal points
!     text1          i    Text to be printed in heading
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jtimessav, lentext, jmarksav
      character (len=100) text
      logical plotpoints, plotelements

!     jmarksav       Help variable to save the value of jmark
!     jtimessav      Help variable to save the value of jtimes
!     lentext        Number of characters in text
!     plotelements   If true element numbers must be plotted
!     plotpoints     if true node numbers must be plotted
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PL0000         Compute length and width of the plot
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLAFP7         Plot text using absolute co-ordinates
!     PLELEM         Plot elements for one element group (2d)
!     PLPOIN         plot nodal points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Open plot
!     For i := 1, nelem do
!        Plot element
!     Close plot
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotmeshinter' )
      debug = .false.
      plotpoints = .true.
      plotelements = .true.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotmeshinter'

      end if
      jtimessav = jtimes
      jmarksav = jmark

!     --- Open plot

      jtimes = 0
      call pl0000 ( 15d0, 1d0 )
      call plafep ( 1, aleng, width, 1d0, 15d0, 1 )

      jmark = 0
      if ( plotpoints ) call plpoin ( coor, npoint, kmeshc, 2 )

      if ( .not. plotelements ) jmark = 3
      call plelem ( .true., coor, kmeshc, 1, nelem, 3, &
                    3, .true., 2, .false. )

!     --- Close plot

      lentext = len(text1)+1
      text = text1
      text(lentext+1:lentext+18) = '        loop '
      write(text(lentext+19:lentext+24),100) icount
100   format(i6)
      call plafp7 ( 1d0, 1d0, text, 0d0, 0.25d0, lentext+24 )
      call plafep( 2, 0d0, 0d0, 0d0, 0d0, 0 )

      call erclos ( 'plotmeshinter' )
      if ( debug ) write(irefwr,*) 'End plotmeshinter'
      jtimes = jtimessav
      jmark = jmarksav

      end
