      subroutine plotzerolev2 ( inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 15-11-2013 Extension with curve plot info
!        version  1.2  date 12-09-2013 Add plottime
!        version  1.1  date 25-11-2012 Long integers
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot the edges (2d) or faces (3d) of the zero level set
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpsep(*)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         The first position refers to this subroutine
!                         See subroutine readscplotzerl
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ikader, ilev
      double precision plotfm, yfact
      logical bouncurv

!     bouncurv       if true the 2D boundaries are plotted using icurvs/curves
!                    otherwise using kmeshm and coor
!     ikader         Help variable to store jkader temporarily
!     ilev           Sequence number of level being considered
!     plotfm         Plotting factor. When PLOTFM<0, then the length of
!                    the plot is computed by - (xmax-xmin) * PLOTFM
!                    and the width by - (ymax-ymin) * YFACT * PLOTFM
!                    else the length of the plot is equal to PLOTFM cm
!                    When by some mistake the user has given | PLOTFM | a
!                    value smaller than 1 cm, then the length is chosen
!                    equal to the default length stored in common block
!                    consta
!     yfact          Multiplication for the y-co-ordinates. With this
!                    parameter the user can influence the scale in
!                    y-direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLOTTIME       Plot the actual time in a plot depending on the parameters
!                    itimdep and t in common ctimen
!     PLOTZEROLEV2D  Plot the edges (2d) of the zero level set
!     PLOTZEROLEV3D  Plot the faces (3d) of the zero level set
!     PROPENPLOT     Open plot and plot boundary (2d)
!     SEPFILLKMESHAA Fill first part of array KMESH part aa
!     SEPFILLKMESHD  Fill kmesh part d
!     SEPFILLNMESHC  Fill array nmeshc
!     SEPFILLNMESHG  Fill array nmeshg
!     SEPFILLOBFACES Fill kmesh part ad subpart 9: iobstfaces
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plotzerolev2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plotzerolev2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ilev = inpsep(1)

!     --- Create or activate nmeshc (nodes of edges)

      if ( kelmaa==0 ) call sepfillkmeshaa
      if ( ndim==2 ) then
         if ( nmesh(33)==0 ) call sepfillnmeshc
      else
         if ( nmesh(37)==0 ) call sepfillnmeshg
      end if  ! ( ndim==2 )

      if ( kelmad==0 ) call sepfillkmeshd
      if ( kmeshad(9)==0 ) call sepfillobfaces

      if ( ndim==2 ) then

!     --- ndim = 2
!         Open plot

         bouncurv = .true.
         yfact = 1d0
         plotfm = 15d0
         call propenplot ( bouncurv, plotfm, yfact, ikader, inpsep(2) )

!        --- Plot the zero level set

         call plotzerolev2d ( coor, ilev, nmeshc, obstfaces )

         call plottime  ! plot time label if time dependent

!        --- close plot dataset

         call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )

      else if ( ndim==3 ) then

!     --- ndim = 3

         call plotzerolev3d ( coor, ilev, nmeshg, obstfaces )

      end if  ! ( ndim==2 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plotzerolev2'

      end if  ! ( debug )
      call erclos ( 'plotzerolev2' )

      end
