      subroutine plpos6 ( ichois, sx, sy, xminbox, yminbox, xmaxbox, &
                          ymaxbox )
! ======================================================================
!
!        programmer    Jac Janssen
!        version  2.3  date 22-08-2013 Save file ref numbers
!        version  2.2  date 20-06-2013 New definition file ref numbers
!        version  2.1  date 16-05-2003 Extra moveto in case of change colour
!
!   copyright (c) 1991-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    PostScript dependent calls of plotting subroutines
!    except for texts, special symbols and colours
!    machine dependent subroutine
! **********************************************************************
!
!                       KEYWORDS
!
!     machine_dependent
!     plot
!     postscript
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulemach
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois
      real sx, sy, xminbox, yminbox, xmaxbox, ymaxbox

!     ichois         i    Choice parameter
!                         Possibilities:
!                         1 Call plot start subroutine
!                           When X=1 small plotting paper is used, when X=2
!                           wide plotting paper is used.
!                           (not supported in some versions)
!                           The length of the plot is given in Y (centimeters)
!                         2 Plot with pen down to point (x,y)
!                         3 Plot with pen up to point (x,y)
!                         4 Change origin
!                           (will disappear in next major revision!)
!                           (x,y) are the values to be added to the current
!                           origin
!                         5 Close plot data set
!                           When X=-1 the number for the next (w or u) output
!                           file will be reset to 1 again
!     sx             i    Single precision value of x
!     sy             i    Single precision value of y
!     xmaxbox        i    Maximum x-value in box around object
!     xminbox        i    Minimum x-value in box around object
!     ymaxbox        i    Maximum y-value in box around object
!     yminbox        i    Minimum y-value in box around object
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer pltnum, irefsv, ios, mcount, mcent, lastcol
      real size, xps, yps, lastxm, lastym, xoffset, yoffset
      logical check
      parameter ( size = 72/3.0, mcent = 100, xoffset = 15.0, &
                  yoffset = 20.0 )
      character*30 filenm
      save pltnum, mcount, lastcol, lastxm, lastym

!     check          Indicates if the file exist (true) or not
!     filenm         character field to build filename in
!     ios            File i/o status parameter
!     irefsv         Help variable to store a reference number temporarily
!     lastcol        Last active colour
!     lastxm         x-coordinate of last move command
!     lastym         y-coordinate of last move command
!     mcent          max number of 'moves' (m); before the mcent'th 'move', &
!                    there is written a 'stroke' (s) to the output file
!     mcount         count for the number of 'moves' (m) (moving with pen up)
!     pltnum         file number of the plot last made, initialized to 0
!     size           Parameter to scale from program units to cm's on the paper
!                    This parameter is defined by a parameter statement
!                    One postscript unit = 1/72 inch
!     xoffset        offset in x direction
!     xps            X value in postscript coordinates
!     yoffset        offset in y direction
!     yps            Y value in postscript coordinates
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inisetnewref

!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDUMP         Print error message directly to screen without substitution
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     INISETNEWREF   Create new reference number
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1297   Value of ichois not supported
!    1317   Error in opening of the plot
!    3148   Too many plot files
! **********************************************************************
!
!                       PSEUDO CODE
!
!    If ( ichois=1 and first picture ) then
!       delete all preceding plot files
!    if ( ichois=1 ) then
!       open plot
!       write heading of file
!    else if ( ichois=2 ) then
!       compute to where the "pen" must be drawn ("pen down") to
!       write this position to the file
!    else if ( ichois=3 ) then
!       compute to where the "pen" must be moved ("pen up") to
!       write this position to the file
!    else if ( ichois=5 ) then
!       close the plot
!                          METHOD
!     All measures are in Postscript units (PS units,1 unit is 1/72 inch) in
!     the PS default user coordinate system
!     The values are defined such that the maximal plotting paper is used, &
!     given by addressable paper size
!     In order to get an eps file (i.e. encapsulated postscript file), &
!     a bounding box is given
!     The addressable paper size has been found by trial and error:
!        560.368*806.144 user units = 19.768538 * 28.438969 cm = &
!        7.7828889 * 11.196444 inch
!        ( A4 paper size is 210 * 297 mm = 8.267717 * 11.69291 inch = &
!          595.27559 * 841.88976 user units )
!     The centered symbols have a fixed size independent of the scaling
!     The file gets the following heading:
!     %!PS-Adobe-2.0 EPSF-2.0
!     %%BoundingBox:    0.000     0.000   560.368   806.144
!     %%EndComments
!     /m /moveto load def
!     /l /lineto load def
!     /rm /rmoveto load def
!     /rl /rlineto load def
!     /s /stroke load def
!     /rfill {dup neg 0 exch rl exch 0 rl 0 exch rl closepath fill} def
!     1 setlinecap
!     1 setlinejoin
!        0.00 setlinewidth
!     /master /Times-Roman findfont def
!     /haldix     4.252  def
!     /haldiy     4.252  def
!     /halfx      3.682  def
!     /halfy      2.455  def
!     /centre { m } def
!     /plus { haldix neg  0.0  rm
!             2 haldix mul  0.0  rl
!             haldix neg  haldiy neg  rm
!             0.0  2 haldiy mul  rl } def
!     /asterisk { 0.0  haldiy neg  rm
!                 0.0  2 haldiy mul  rl
!                 halfx neg  haldiy neg 2 div  rm
!                 2 halfx mul  haldiy neg  rl
!                 0.0  haldiy  rm
!                 2 neg halfx mul  haldiy neg  rl } def
!     /hexagon { 0.0  haldiy neg  rm
!                halfx neg  haldiy 2 div  rl
!                0.0  haldiy  rl
!                halfx  haldiy 2 div  rl
!                halfx  haldiy neg 2 div  rl
!                0.0  haldiy neg  rl
!                halfx neg  haldiy neg 2 div  rl } def
!     /cross { haldix neg  haldiy neg  rm
!              2 haldix mul  2 haldiy mul  rl
!              2 neg haldix mul  0.0  rm
!              2 haldix mul  2 neg haldiy mul  rl } def
!     /delta { 0.0  2 halfy mul  rm
!              haldix neg  3 neg halfy mul  rl
!              2 haldix mul  0.0  rl
!              haldix neg  3 halfy mul  rl } def
!     /square { haldix neg  haldiy neg  rm
!               2 haldix mul  0.0  rl
!               0.0  2 haldiy mul  rl
!               2 neg haldix mul  0.0  rl
!               0.0  2 neg haldiy mul  rl } def
!     /diamond { haldix neg  0.0  rm
!                haldix  haldiy neg  rl
!                haldix  haldiy  rl
!                haldix neg  haldiy  rl
!                haldix neg  haldiy neg  rl } def
!     /4square { plus  centre  square } def
!     /nabla { 0.0  2 neg halfy mul  rm
!              haldix neg  3 halfy mul  rl
!              2 haldix mul  0.0  rl
!              haldix neg  3 neg halfy mul  rl } def
!     /star { delta  centre  nabla } def
!     /polygon { % pen up to polyg(1),polyg(2):
!                m
!                % pen down to polyg(i),polyg(i+1),(i=3(2)2*numofpol-1)
!                numofpol 1 sub { l } repeat
!                % pen down to polyg(1),polyg(2):
!                closepath
!              } def
!       18.016    18.256  translate
!        1.000     1.000  scale
!     % end prologue
!     Explanation of some variables:
!     halfdix/y: fixed, half maximum height and width of centered symbols in cm
!                The symbols here defined are used by plpos3
!     numofcol:  maximum number of colours. Colours are used by plpos8, &
!                which uses dithering to convert it to black and white
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data lastcol /0/
      data pltnum /0/
! ======================================================================
!
      call eropen ( 'plpos6' )

!     --- Make a choice depending on ichois

      go to ( 100, 200, 300, 1000, 500 ), ichois

!     --- Incorrect value of ichois

         call errint ( ichois, 1 )
         call errsub ( 1297, 1, 0, 0 )
         go to 1000

!        --- ichois = 1, open plot

100      if ( pltnum==0 ) then

!        --- First call, *  delete existing files

            irefps = inisetnewref()
110         pltnum=pltnum+1

            if ( pltnum<1000 ) then

!           --- pltnum<1000, hence format i3.3 is possible

               write ( filenm, 120 ) 'sepposc', pltnum
120            format ( a, '.', i3.3, '.eps' )
               inquire ( file=filenm, exist=check )

            else

!           --- pltnum>999, hence format i4.4 is necessary

               check = .false.

            end if  ! ( pltnum<1000 )

            if ( .not. check ) then

!           --- The file format is not i3.3 hence it must be i4.4
!               or there is no file left

               write ( filenm, 130 ) 'sepposc', pltnum
130            format ( a, '.', i4.4, '.eps' )
               inquire ( file=filenm, exist=check )

            end if  ! ( .not. check )

            if ( check ) then
               open ( unit=irefps, file=filenm, status='old' )
               close ( unit=irefps, status='delete' )
               go to 110
            end if
            pltnum = 0
         end if

         pltnum=pltnum+1

         if ( pltnum>9999 ) then

!        --- the number of plot files is more than 9999
!            However the format does not allow such number

            call errint ( pltnum, 1 )
            call errwar ( 3148, 1, 0, 0 )
            go to 1000

         end if  ! ( pltnum>9999 )

         oldfac = 0
         write ( filenm, 130 ) 'sepposc', pltnum
         open ( unit=irefps, file=filenm, form='formatted', iostat=ios )
         if ( ios/=0 ) then

!        --- ios # 0

            call errint ( ios, 1 )
            call errchr ( filenm, 1 )
            call errsub ( 1317, 1, 0, 1 )
            go to 1000

         end if

!        --- Write first part of Postscript definition to the file
!            The text has been stored in the error message file, through the
!            ASCII file error.dat

         irefsv = irefwr
         irefwr = irefps
         write(irefwr,150) xminbox, yminbox, xmaxbox, ymaxbox
150      format ('%!PS-Adobe-2.0 EPSF-2.0'/ &
                 '%%BoundingBox: ',4f10.3)
         call erdump ( 1828 )
         if ( posdev(2:2)=='l' .or. posdev(2:2)=='w' ) then

!        --- Landscape mode

            write (irefwr,*) ' 90 rotate 0 -560 translate'
         end if
         irefwr = irefsv
         mcount = 1
         go to 1000

!        --- ichois = 2,  draw line:
!            calculate position from program units to user units (PostScript
!               units):

200      if ( curcol/=lastcol ) then

!        --- New colour found while plotting line
!            This means that the moveto command is followed by a setrgbcolor
!            command.
!            Before giving a lineto we have to repeat the last moveto

            write ( irefps, 310 ) lastxm, lastym
            lastcol = curcol

         end if

         xps = size * sx + xoffset
         yps = size * sy + yoffset
         write ( irefps, 210 ) xps, yps
210      format ( 2(1x,f8.3),' l' )
         if ( mcount>=mcent ) then
            write ( irefps, '('' s'')' )
            write ( irefps, 310 ) xps, yps
            mcount = 1
         else
            mcount = mcount + 1
         end if
         go to 1000

!        --- ichois = 3,  move pen:
!            calculate position from program units to user units (PostScript
!               units):

300      xps = size * sx + xoffset
         yps = size * sy + yoffset
         lastcol = curcol
         lastxm = xps
         lastym = yps
         write ( irefps, 310 ) xps, yps
310      format ( 2(1x,f8.3),' m' )
         go to 1000

!        --- ichois = 5,  end of plot:

500      write (irefps, 510 )
510      format ( ' 0 0 m' / ' ( ) show' / ' s' / ' showpage' )
         close (irefps)
         call inirmref ( irefps )

         go to 1000
1000  call erclos ( 'plpos6' )
      end
