      subroutine plpos7 ( x, y, text, angle, height, nchar )
! ======================================================================
!
!        programmer    Jac Janssen
!        version  1.8  date 22-08-2013 New definition file ref numbers
!        version  1.7  date 26-01-2007 Correction angles
!        version  1.6  date 11-09-2006 Correction angle of 0 degrees
!
!   copyright (c) 1991-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     subroutine to plot a text starting on (x,y)
!     machine dependent subroutine
!     PostScript
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     postscript
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulemach
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      real x, y, angle, height
      integer nchar
      character (len=*) text

!     angle     i   The text is plotted under an angle of angle degrees.
!     height    i   The height of the text is equal to HEIGHT centimeters.
!     nchar     i   Number of characters in the text to be plotted
!     text      i   Text to be plotted. Character variable
!                   The text may contain at most 80 characters
!     x         i   give the co-ordinates of the at most left under point of
!                   the first character. The pen moves with pen up to (X,Y).
!     y         i   give the co-ordinates of the at most left under point of
!                   the first character. The pen moves with pen up to (X,Y).
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iangle, ntext, i
      real size, fontsz, xoffset, yoffset
      character (len=1) c
      character (len=360) textcp
      parameter ( size = 72/3.0, xoffset = 15.0, yoffset = 20.0 )

!     c              Copied character
!     fontsz         Font size of characters in cm
!     i              General loop variable
!     iangle         Integer rounded value of the angle
!     ntext          Length of textcp
!     size           Parameter to scale from program units to cm's on the paper
!                    This parameter is defined by a parameter statement
!                    One postscript unit = 1/72 inch
!     textcp         Copy of text provided with \ for brackets
!     xoffset        offset in x direction
!     yoffset        offset in y direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLCOL0         Routine to look at color and set it if changed since last
!                    call
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plpos7' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plpos7'
         write(irefwr,1) 'nchar, irefpl', nchar, irefpl
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call plcol0 ( nchar/100 )

!     --- Copy text in textcp, provide brackets with \

      ntext = 0
      do i = 1, mod(nchar,100)
         c = text(i:i)
         if ( c=='(' .or. c==')' ) then
            if ( ntext>=359 ) go to 20
            textcp(ntext+1:ntext+2) = '\\'
            textcp(ntext+2:ntext+2) = c
            ntext = ntext + 2
         else
            if ( ntext>=360 ) go to 20
            textcp(ntext+1:ntext+1) = c
            ntext = ntext + 1
         end if
      end do

!     --- calculate size of characters
!         a correction factor of 1.52 is required to get capitals in
!         Times-Roman font at the given amount of centimeters
!         The font size is restricted between 1 and 9999

20    fontsz = size * height * 3.0/2.54
      fontsz = max ( 1., fontsz )
      fontsz = min ( 9999., fontsz )

!     --- The postscript command /stroke is written
!         In plpos6 this command is defined as s
!         The "pen" is moved to x,y

      write ( irefps, 100 ) x*size+xoffset, y*size+yoffset
100   format ( ' s' / 2(1x,f8.3),' m' )

!     --- if font number or character height is changed
!         create font in new character size:

      if ( abs(fontsz-oldfac)>0.001 ) then
         write ( irefps, 130 ) fontsz
130      format ( ' master ', f8.3, ' scalefont setfont' )
         oldfac = fontsz
      end if

!     --- rotate by given angle:

      if ( abs(angle)<1d0 ) then
         iangle = 0
      else
         iangle= nint(angle)
      end if
      if ( iangle/=0 ) then
         write ( irefps, 160 ) iangle
160      format ( 1x, i4, ' rotate' )
      end if

!     --- place text between brackets and append a "show" command:

      write ( irefps, 170 ) textcp(1:ntext)
170   format ( 1x, '(', a, ') show' )

!     --- rotate back, otherwise the rest of the plot will be rotated:

      if ( iangle/=0 ) then
         iangle = -iangle
         write ( irefps, 180 ) iangle
180      format ( 1x, i4, ' rotate' )
      end if

1000  call erclos ( 'plpos7' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plpos7'

      end if  ! ( debug )

      end
