      subroutine plscls ( ntext, kchois )
! ======================================================================
!
!        programmer    Guus Segal
!        version 11.15 date 26-10-2009 Debug statements
!        version 11.14 date 07-03-2007 Move number along y-axis
!        version 11.13 date 29-08-2002 Correction numbers along axis
!
!   copyright (c) 1984-2009  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    service routine to plot all scaling, markers and other text in
!    a plot made by plotfn
!    Depending on the value of jmark axises are plotted or not
!    Possibilities: jmark = jmarkold + 100 * jplxs + 1000 * jplys
!                                     + 10000 * jply2s
!    if jplxs  = 1 then the x-axis is not plotted
!    if jplys  = 1 then the first y-axis is not plotted
!    if jply2s = 1 then the second y-axis is not plotted
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     plot_axis
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ntext, kchois

!     kchois     i    integer to choose between different y-scale options.
!                     kchois consists of two parts kchoice and itypescales
!                     according to kchois = kchoice+10*itypescales
!                     Possibilities for kchoice are:
!                     1   Only one y-scale will be made, values and scale
!                         markers depend on jplys only
!                     2   There will be 2 y-scales, this is the call for the
!                         left one.
!                         Values and markers on the left depend on jplys
!                         The x-scale will also be made in this call
!                     3   This is call for right y-scale, depending on jply2s
!                         Currently only markers for x on top will be made also
!                     itypescales    type of scales used along each of the axis.
!                                    Is subdivided into 2 parts according
!                                    to itypescales = itype1+10*itype2
!                                    where itype1 refers to the x-axis, &
!                                    itype2 and to the y-axis
!                     The following values for itypex are available
!                      0  Linear scale
!                      1  Logarithmic scale (base 10)
!     ntext      i    Number of characters in a text
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ikomma, ix, iy, j, j5, jplxs, jplys, jply2s, &
              itypescales, kmark, komma, kx, ky, jtstsv, itypescalx, &
              itypescaly, kchoice
      double precision axhulp, ayhulp, dx, dy, f, f1, f2, f3, f4, &
                       f5, f6, x, xdown, xj, xlow, xtop, y, ydown, yj, &
                       ylow,ytop

!     axhulp         step between 2 successive scale markers
!     ayhulp         step between 2 successive scale markers
!     dx             step between sub-markers
!     dy             step between sub-markers
!     f              dummy value for plafep calls
!     f1             size of marker
!     f2             size of characters in x-direction
!     f3             same in y-direction
!     f4             distance before y-axis to start plotting values
!     f5             distance below x-axis to put values
!     f6             size of sub-marker
!     i              loop variable, counting steps along the axis
!     ikomma         parameters for plafep, given kind of values
!     itypescales    type of scales used along each of the axis.
!                    Is subdivided into 2 according
!                    to itypescales = itype1+10*itype2+100*itype3
!                    where itype1 refers to the x-axis, itype to the
!                    y-axis
!                    The following values for itypex are available
!                     0  Linear scale
!                     1  Logarithmic scale (base 10)
!     itypescalx     Type of scale in x-direction, see itypescales
!     itypescaly     Type of scale in y-direction, see itypescales
!     ix             integer version of number to be put along axis
!     iy             integer version of number to be put along axis
!     j              loop variable, counting sub-markers between steps
!     j5             number of sub-markers to be made, normal 5 except last
!     jplxs          flag to switch off drawing of scale values
!     jply2s         flag to switch off drawing of scale values
!     jplys          flag to switch off drawing of scale values
!     jtstsv         Saves value of jtest at input
!     kchoice        See kchois
!     kmark          lower order of jmark, without scale options
!     komma          function code of plafep call to be used for values
!     kx             temporary storage of kiesx,kiesy values
!     ky             temporary storage of kiesx,kiesy values
!     x              fixed coordinate for markers
!     xdown          maximum distance to test for legality of coordinates
!     xj             other (changing) coordinate for markers
!     xlow           window of coordinates for scales
!     xtop           window of coordinates for scales
!     y              fixed coordinate for markers
!     ydown          maximum distance to test for legality of coordinates
!     yj             other (changing) coordinate for markers
!     ylow           window of coordinates for scales
!     ytop           window of coordinates for scales
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PL0004         Round minimum and maximum values
!     PLAF04         Routine to plot arrows with text along the axises
!     PLAFEP         General SEPRAN low level plotting subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    plscls: seq
!      isolate scale-markers from jmark common variable
!      calculate several help-variables
!      factor: sel (kiesx<0 or kiesy<0)
!        determine minimum/maximum of scale and number of digits
!      factor: alt (else)
!      factor: end
!      ayhulp := value for 1 step of scale y
!      dy := value for 1/5 of a step
!      xdown := 1/3 * xfct
!      ydown := 1/5 * yfct
!      tekst: sel (ntext>=0)
!        window set from (xmin -xdown),(ymin - ydown)-(xmax + xdown), &
!                                                         (ymax + ydown)
!      tekst: alt (else)
!        window is (axmin,aymin)-(axmax,aymax)
!      tekst: end
!      chois: sel (1 scale or under-left for 2 scales)
!        calculate number of digits for scale-values along x-axis
!        y := low y-value of window
!        draw x-under-axis
!        x-scale: iter (nstepx+1 times)
!          determine x-coordinate
!          aantal: sel (last time)
!            j5 := 1
!          aantal: alt (else)
!            j5 := 5
!          aantal: end
!          line: iter (j5 times)
!            determine x-coordinate
!            draw short scale-marker
!          line: end
!          step: sel (x coordinate within window)
!            draw longer scale-marker for value
!          step: alt (else)
!          step: end
!          value: sel (even numbered value and enough space)
!            put text of value below marker
!          value: alt (else)
!          value: end
!        x-scale: end
!        calculate number of digits for scale-values along y-axis
!        x := low x-value of window
!        draw y-left-axis
!        y-scale: iter (nstepy+1 times)
!          determine y-coordinate
!          aantal: sel (last time)
!            j5 := 1
!          aantal: alt (else)
!            j5 := 5
!          aantal: end
!          line: iter (j5 times)
!            determine y-coordinate
!            draw short scale-marker
!          line: end
!          step: sel (y coordinate within window)
!            draw longer scale-marker for value
!          step: alt (else)
!          step: end
!          value: sel (enough space)
!            put text of value left of marker
!          value: alt (else)
!          value: end
!        y-scale: end
!        put arrows and texts along x- and (first) y-axis
!      choice: alt (1 scale or above-right for 2 scales)
!        calculate number of digits for scale-values along x-axis
!        y := high y-value of window
!        draw x-upper-axis
!        x-scale: iter (nstepx+1 times)
!          determine x-coordinate
!          aantal: sel (last time)
!            j5 := 1
!          aantal: alt (else)
!            j5 := 5
!          aantal: end
!          line: iter (j5 times)
!            determine x-coordinate
!            draw short scale-marker
!          line: end
!          step: sel (x coordinate within window)
!            draw longer scale-marker for value
!          step: alt (else)
!          step: end
!        x-scale: end
!        calculate number of digits for scale-values along y-axis
!        x := high x-value of window
!        draw y-right-axis
!        y-scale: iter (nstepy+1 times)
!          determine y-coordinate
!          aantal: sel (last time)
!            j5 := 1
!          aantal: alt (else)
!            j5 := 5
!          aantal: end
!          line: iter (j5 times)
!            determine y-coordinate
!            draw short scale-marker
!          line: end
!          step: sel (y coordinate within window)
!            draw longer scale-marker for value
!          step: alt (else)
!          step: end
!          value: sel (enough space)
!            put text of value right of marker
!          value: alt (else)
!          value: end
!        y-scale: end
!        text-2: sel (2 y-scales selected)
!          put arrows and texts along x- and second y-axis
!        text-2: alt (else)
!        text-2: end
!      choice: end
!    plscls: end
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'plscls' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from plscls'
         write(irefwr,1) 'kiesx, kiesy', kiesx, kiesy
         write(irefwr,1) 'isigx, isigy', isigx, isigy
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- first get all scale options out of jmark

      jtstsv = jtest
      jplxs=0
      jplys=0
      jply2s=0
      kmark=mod(jmark,100000)
      if (kmark>=10000) then
         jply2s=1
         kmark=mod(kmark,10000)
      end if
      if (kmark>=1000) then
         jplys=1
         kmark=mod(kmark,1000)
      end if
      if (kmark>=100) then
         jplxs=1
      end if
      if ( debug ) &
         write(irefwr,1) 'jply2s, jplys, jplxs', jply2s, jplys, jplxs

!     --- Next kchoice and itypescales

      kchoice = mod(kchois,10)
      itypescales = kchois/10
      itypescalx  = mod(itypescales,10)
      itypescaly  = itypescales/10
      if ( debug ) &
         write(irefwr,1) 'kchois, itypescales, itypescalx, itypescaly', &
                          kchois, itypescales, itypescalx, itypescaly

!     --- call pl0004 ? Yes if kiesx=-1 or kiesy=-1

      kx=kiesx
      ky=kiesy
      f=0
      f1=-0.16d0
      f2=.2d0
      f3=.5d0
      f4=xfct*.4d0
      f5=yfct*.4d0
      f6=-.5d0*f1
      if ( kx==-1 .and. itypescalx==0 ) &
         call pl0004(axmin,axmax,kx,isigx)
      if ( ky==-1 .and. itypescaly==0 ) &
         call pl0004(aymin,aymax,ky,isigy)
      if ( debug ) &
         write(irefwr,1) 'kx, ky, isigx, isigy', kx, ky, isigx, isigy

!     --- definition of window

      axhulp=(axmax-axmin)/nstepx
      dx=axhulp*.2d0
      ayhulp=(aymax-aymin)/nstepy
      dy=ayhulp*.2d0
      xdown=xfct*.3333d0
      ydown=yfct*.3333d0
      if (ntext>=0) then
         xlow=xmint-xdown
         ylow=ymint-ydown
         xtop=xmaxt+xdown
         ytop=ymaxt+ydown
      else
         xlow=axmin
         ylow=aymin
         xtop=axmax
         ytop=aymax
         xmint = axmin
         xmaxt = axmax
         ymint = aymin
         ymaxt = aymax
      end if
      if ( kchoice==3 ) go to 100

!     --- draw lower x-axis
!         floating or fixed point notation in x-direction ?
!         kx=0 : floating   kx>0 : fixed

      jtest=0
      y=ylow
      if ( ntext>=0 ) then
         call plafep(4,xmaxt,y,f,f,0)
         call plafep(5,xmint,y,f,f,0)
      else
         call plafep(4,axmax,y,f,f,0)
         call plafep(5,axmin,y,f,f,0)
      end if
      if (jplxs==1) go to 25
      komma=14
      if (kx>0) komma=12
      ikomma=-isigx
      if (ikomma==0) komma=10
      if (ntext<0) komma=komma+1
      if ( debug ) &
         write(irefwr,1) 'ikomma, komma, ntext, nstepx', &
                          ikomma, komma, ntext, nstepx

!     --- draw values and markers along the x-axis

      do i=0,nstepx
         x=axmin+i*axhulp
         if (i>0) then
            if ( itypescalx==1 ) then
               j5=10
            else
               j5=5
            end if
         else
            j5=1
         end if
         do j=j5,1,-1
            if ( itypescalx==1 ) then

!           --- logarithmic scale

               if ( i==0 ) then
                  xj = x
               else
                  xj=x-(1d0-log10(11d0-j))*axhulp
               end if

            else

!           --- linear scale

               xj=x-(j-1)*dx

            end if
            if ( xj>=xmin-xdown .and. xj<=xmax+xdown ) then
               call plafep(4,xj,y,f,f,0)
               call plafep(16,xj,y,f,f6,0)
            end if
         end do
         if (x<xmin-xdown .or. x>xmax+xdown) go to 20
         call plafep(4,x,y,f,f,0)
         call plafep(16,x,y,f,f1,0)
         if ( debug ) &
            write(irefwr,1) 'itypescalx, komma, ikomma', &
                             itypescalx, komma, ikomma
         if (i/2*2==i .or. itypescalx==1 ) go to 15
         if (aleng<50d0) go to 20
         if ((aleng<2d0).and.(kx==0)) go to 20
15       if ( itypescalx==1 ) then

!        --- logarithmic scale

            call plafep(24,x,y-2d0*f5,x,f2,nint(x))

         else if (komma>=12) then
            call plafep(komma,x,y-f5,x,f2,ikomma)
         else
            ix=nint(x)
            call plafep(komma,x,y-f5,-1d0,f2,ix)
         end if
20    end do

!     --- draw left y-axis
!         floating or fixed point notation in y-direction (first y-scale) ?

25    x=xlow
      if ( ntext>=0 ) then
         call plafep(4,x,ymaxt,f,f,0)
         call plafep(5,x,ymint,f,f,0)
      else
         call plafep(4,x,aymax,f,f,0)
         call plafep(5,x,aymin,f,f,0)
      end if
      if (jplys==1) go to 45
      komma=14
      if (ky>0) komma=12
      ikomma=isigy
      if (ikomma==0) komma=10
      if (ntext<0) komma=komma+1

!     --- draw values and markers along the first y-axis

      do i=0,nstepy
         y=aymin+i*ayhulp
         if (i>0) then
            if ( itypescaly==1 ) then
               j5=10
            else
               j5=5
            end if
         else
            j5=1
         end if
         do j=j5,1,-1
            if ( itypescaly==1 ) then

!           --- logarithmic scale

               if ( i==0 ) then
                  yj = y
               else
                  yj=y-(1d0-log10(11d0-j))*ayhulp
               end if

            else

!           --- linear scale

               yj=y-(j-1)*dy

            end if
            if (yj>=ymin-ydown .and. yj<=ymax+ydown) then
               call plafep(4,x,yj,f,f,0)
               call plafep(16,x,yj,f6,f,0)
            end if
         end do
         if (y<ymin-ydown .or. y>ymax+ydown) go to 40
         call plafep(4,x,y,f,f,0)
         call plafep(16,x,y,f1,f,0)
         if (i/2*2==i .or. itypescaly==1 ) go to 35
         if (width<1d0) go to 40
         if ((width<2d0).and.(ky==0)) go to 40
35       if ( itypescaly==1 ) then

!        --- logarithmic scale

            call plafep(24,x-f4,y,y,f3,nint(y))
         else if (komma>=12) then

!        --- Plot standard scale

            call plafep(komma,x-f4,y,y,f3,ikomma)

         else
            iy=nint(y)
            call plafep(komma,x-f4,y,1d0,f3,iy)
         end if
40    end do
45    jtest=1
      if (ntext/=0) call plaf04(5,0d0,0d0)

!     --- draw upper x-axis and scale-markers
!         only in possibilities 1 and 3

      if (kchoice==2) go to 1000
100   jtest=0
      y=ytop
      if ( ntext>=0 ) then
         call plafep(4,xmaxt,y,f,f,0)
         call plafep(5,xmint,y,f,f,0)
      else
         call plafep(4,axmax,y,f,f,0)
         call plafep(5,axmin,y,f,f,0)
      end if
      if (jplxs==1) go to 65
      do i=0,nstepx
         x=axmin+i*axhulp
         if (i>0) then
            j5=5
         else
            j5=1
         end if
         do j=j5,1,-1
            xj=x-(j-1)*dx
            if (xj>=xmin-xdown .and. xj<=xmax+xdown) then
               call plafep(4,xj,y,f,f,0)
               call plafep(16,xj,y,f,-f6,0)
            end if
         end do
         if (x<xmin-xdown .or. x>xmax+xdown) go to 60
         call plafep(4,x,y,f,f,0)
         call plafep(16,x,y,f,-f1,0)
60    end do

!     --- draw right y-axis
!         floating or fixed point notation in second y-direction ?

65    x=xtop
      if ( ntext>=0 ) then
         call plafep(4,x,ymaxt,f,f,0)
         call plafep(5,x,ymint,f,f,0)
      else
         call plafep(4,x,aymax,f,f,0)
         call plafep(5,x,aymin,f,f,0)
      end if
      if ( (kchoice==1.and.jplys==1) .or. &
           (kchoice==3.and.jply2s==1) ) go to 85
      komma=14
      if (ky>0) komma=12
      ikomma=isigy
      if (ikomma==0) komma=10
      if (ntext<0) komma=komma+1

!     --- draw values and markers along the second y-axis

      do i=0,nstepy
         y=aymin+i*ayhulp
         if (i>0) then
            j5=5
         else
            j5=1
         end if
         do j=j5,1,-1
            yj=y-(j-1)*dy
            if (yj>=ymin-ydown .and. yj<=ymax+ydown) then
               call plafep(4,x,yj,f,f,0)
               call plafep(16,x,yj,-f6,f,0)
            end if
         end do
         if (y<ymin-ydown .or. y>ymax+ydown) go to 80
         call plafep(4,x,yj,f,f,0)
         call plafep(16,x,yj,-f1,f,0)
         if (kchoice==1) go to 80
         if (i/2*2==i) go to 75
         if (width<1d0) go to 80
         if ((width<2d0).and.(ky==0)) go to 80
75       if (komma>=12) then
            call plafep(komma,x+3.5d0*f4,y,y,f3,ikomma)
         else
            iy=nint(y)
            call plafep(komma,x+3.5d0*f4,y,1d0,f3,iy)
         end if
80    end do
85    if (kchoice/=1) then
         jtest=1
         if (ntext/=0) call plaf04(6,0d0,0d0)
      end if

1000  jtest = jtstsv
      call erclos ( 'plscls' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End plscls'

      end if  ! ( debug )

      end
