      subroutine plt354( v, coor, kp, lp, mp, ipoint, xa, ya, za, &
                         jes, in )
! =======================================================================
!
!
!         programmer     Niek Praagman
!         version 4.0    date  21-04-1995 Coordinate basis added, &
!                                        extra parameter)
!         version 3.1    date  22-08-1991 Zdenek: no dimension, cdc, ce
!         version 3.0    date  10-01-1989
!
!   copyright (c) 1984 - 1995  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ***********************************************************************
!
!     DESCRIPTION
!
!     Determine whether point is visible in relation to triangular surface
!     from given viewpoint
!
! ***********************************************************************
!
!     KEYWORDS
!
!     visibility
!     hidden_line
!
! ***********************************************************************
!
!     INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer kp, lp, mp, ipoint, jes, in(3)
      double precision v(3), coor(3,*), xa, ya, za

!     coor      i   coordinate array
!     in        i   ordering of basis considered
!     ipoint    i   if ipoint = 0 coordinates are already given
!                   else determine (xa,ya,za)
!     jes       o   indicator for visibility (1=visible, 0=not visible)
!     kp        i   first node of triangle to be considered
!     lp        i   second node of triangle to be considered
!     mp        i   third node of triangle to be considered
!     v         i   coordinates of viewpoint
!     xa        i   x-coordinate of node to be checked
!     ya        i   y-coordinate of node to be checked
!     za        i   z-coordinate of node to be checked
!
! ***********************************************************************
!
!     COMMON BLOCKS
!
! ***********************************************************************
!
!     LOCAL PARAMETERS
!
      double precision a1, a2, a3, b1, b2, b3, c1, c2, c3, &
                       x1, x2, x3, y1, y2, y3, z1, z2, z3, &
                       deta, detv, pro

!    a1    x-distance point 1 and viewpoint
!    a2    y-distance point 1 and viewpoint
!    a3    z-distance point 1 and viewpoint
!    b1    x-distance point 2 and viewpoint
!    b2    y-distance point 2 and viewpoint
!    b3    z-distance point 2 and viewpoint
!    c1    x-distance point 3 and viewpoint
!    c2    y-distance point 3 and viewpoint
!    c3    z-distance point 3 and viewpoint
!    deta  determinant value
!    detv  determinant value
!    pro   product of two determinant values
!    x1    x-coordinate node 1
!    x2    x-coordinate node 2
!    x3    x-coordinate node 3
!    y1    y-coordinate node 1
!    y2    y-coordinate node 2
!    y3    y-coordinate node 3
!    z1    z-coordinate node 1
!    z2    z-coordinate node 2
!    z3    z-coordinate node 3
!
! ***********************************************************************
!
!     SUBROUTINES CALLED
!
! ***********************************************************************
!
!     ERROR MESSAGES
!
! ***********************************************************************
!
!     PSEUDO CODE
!
!     Determine coordinates of the three nodes
!
!     Check whether point and viewpoint are in the same half space or
!     not. If so then point is visible else point is not visible
!
! =======================================================================

      if ( ipoint.gt.0 ) then

           xa = coor( in(1), ipoint )
           ya = coor( in(2), ipoint )
           za = coor( in(3), ipoint )

      end if

      x1 = coor( in(1), kp )
      x2 = coor( in(1), lp )
      x3 = coor( in(1), mp )

      y1 = coor( in(2), kp )
      y2 = coor( in(2), lp )
      y3 = coor( in(2), mp )

      z1 = coor( in(3), kp )
      z2 = coor( in(3), lp )
      z3 = coor( in(3), mp )

      a1 = x1 - v(1)
      a2 = y1 - v(2)
      a3 = z1 - v(3)

      b1 = x2 - v(1)
      b2 = y2 - v(2)
      b3 = z2 - v(3)

      c1 = x3 - v(1)
      c2 = y3 - v(2)
      c3 = z3 - v(3)

      detv = a1*b2*c3-a1*c2*b3-a2*b1*c3+a3*b1*c2+a2*b3*c1-c1*a3*b2

      a1 = x1 - xa
      a2 = y1 - ya
      a3 = z1 - za

      b1 = x2 - xa
      b2 = y2 - ya
      b3 = z2 - za

      c1 = x3 - xa
      c2 = y3 - ya
      c3 = z3 - za

      deta = a1*b2*c3-a1*c2*b3-a2*b1*c3+a3*b1*c2+a2*b3*c1-c1*a3*b2

      jes = 0

      pro = deta * detv

      if ( pro.ge.0d0 ) jes = 1

      end
