      subroutine plt358 ( v, ski, eta )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.2  date 01-12-1991 New layout
!        version  1.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  0.1  date 27-06-1987
!
!   copyright (c) 1987-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Subroutine to determine two vectors in the plane through the
!    origin and perpendicular to vector v.
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision v(3), ski(3), eta(3)

!     eta     o   second basisvector in perpendicular plane
!     ski     o   first basisvector in perpendicular plane
!     v       i   vector of viewpoint
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision eps, skw
      integer i

!     eps        Local accuracy
!     i          Counting variable
!     skw
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      eps  =  epsmac

      if ( abs(v(1)).lt.eps .and. abs(v(2)).lt.eps ) then
           ski(1) = v(3)
           ski(2) = 0.d0
           ski(3) = 0.d0
      else
           ski(1) = - v(2)
           ski(2) =   v(1)
           ski(3) = 0.d0
      end if

!     --- Normalize

      skw = ski(1)*ski(1) + ski(2)*ski(2) + ski(3)*ski(3)
      skw = sqrt(skw)

      do 100 i=1, 3
          ski(i) = ski(i) / skw
100   continue

!     --- Determine second vector eta

      eta(1) = v(2)*ski(3) - v(3)*ski(2)
      eta(2) = v(3)*ski(1) - v(1)*ski(3)
      eta(3) = v(1)*ski(2) - v(2)*ski(1)

!     --- Normalize

      skw = eta(1)*eta(1) + eta(2)*eta(2) + eta(3)*eta(3)
      skw = sqrt(skw)

      do 200 i=1, 3
          eta(i) = eta(i) / skw
200   continue

      end
