      subroutine pltextbox ( x, y, text, height )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-08-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot a text in the little box
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     text
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision x, y, height
      character (len=*) text

!     height    i   The height of the text is equal to HEIGHT centimeters.
!                   Double precision parameter
!     text      i   Text to be plotted. Character variable
!                   The text may contain at most 80 characters
!     x         i   x-co-ordinate of the at most left under point of
!                   the first character. The pen moves with pen up to (X,Y).
!                   Double precision parameters.
!                   x,y are given in user co-ordinates
!     y         i   y-co-ordinate see x
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nchar
      real xorig_sav, yorig_sav

!     nchar          Number of characters in a word
!     xorig_sav      Stores original value of xorig
!     yorig_sav      Stores original value of yorig
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLAFP7         Plot text using absolute co-ordinates
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pltextbox' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pltextbox'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nchar = len ( text )

      xorig_sav = xorig
      yorig_sav = yorig
      xorig = 0e0
      yorig = 0e0
      call plafp7 ( x, y, text, 0d0, height, nchar )
      xorig = xorig_sav
      yorig = yorig_sav

1000  call erclos ( 'pltextbox' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pltextbox'

      end if  ! ( debug )

      end
