      subroutine pltjmr ( stadr1, stadr2, xcoor, zcoor )
! ======================================================================
!
!        programmer    Jos van Kan
!        version  1.1  date 21-11-1995 Better error handling
!        version  1.0  date 20-01-1995
!
!   copyright (c) 1995-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       pltjmr merges two lists sorting to xcoordinates only. Is an auxiliary
!       routine to newsk1 and the construction of the visible segment of an
!       intersection. It is assumed, that the two lists donot overlap, but this
!       is not checked.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulepltj
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer stadr1, stadr2, pltjca, pltjcd
      double precision xcoor(*), zcoor(*)

!     stadr1 i/o   startadress of the first list and on output
!                  startaddress of the merged list.
!     stadr2  i    startadress of the second list
!     xcoor   i    xcoordinates of the lists
!     zcoor   i    z coordinates of the lists
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer seghd1, seghd2, pred1, gbgcan, endcel, cell0, cell1

!     cell0   cell number
!     cell1   cell number
!     endcel  ?
!     gbgcan  garbagecan (everything that gets in here is thrown away).
!     pred1   predecessor cell of segment to be moved. If this
!             is nil the first segment is moved and stadr1 is made
!             to point at the second segment. (If any).
!     seghd1  ?
!     seghd2  ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     PLTJAD  to change the segment pointer
!     PLTJCA  to get the next segment
!     PLTJCD  To find cellpointer to next cell
!     PLTJDC  to change the cell pointer of the freelist.
!     PLTJEN  to find end of segment
!     PLTJFS  to return a segment to the freelist
!     PLTJMS  to append a created segment to an existing list
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       Get start address of both lists.
!       While first list not exhausted
!          While {Second list Nonempty} and
!                {Xcoordinate of SegdHdr (second list) < Xcoordinate of SegHdr
!                (first list)} do
!             {Move this segment from second list to first list;
!             Get next segment of second list}
!          End While;
!          Get next segment of first list
!       End While;
!       Put remainder of second list after first list.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Exception handling : one or both lists empty

      if (stadr1.eq.nil) then
         stadr1 = stadr2
         stadr2 = nil
         return
      else if (stadr2.eq.nil) then
         return
      end if
      call eropen ('pltjmr')
      seghd1 = stadr1
      pred1 = nil
100   if (seghd1.eq.nil) then
         goto 120

!        --- 120: FIRST_LIST_EXHAUSTED

      end if
110   if (stadr2.eq.nil) then
         goto 130

!        --- 130: Second_list_exhausted

      end if
      call pltjen (stadr2, endcel)
      if (xcoor (seghd1).ge.xcoor (endcel)) then
         call pltjms (stadr2, nil, stadr1, pred1)
         if (pred1.eq. nil) then
            seghd1 = stadr1
         else
            seghd1 = pltjca (pred1)
         end if
         go to 110
      else if (pred1.eq.nil) then
         pred1 = stadr1
      else
         pred1 = pltjca (pred1)
      end if
      seghd1 = pltjca (seghd1)
      goto 100

!     --- first list exhausted. remainder of second list to be added to
!         first

120   call pltjad (pred1, stadr2, gbgcan)

!     --- second list exhausted : ready

130   continue

!     --- Now merge continuous segments

      seghd2 = stadr1
      if (seghd2.eq.nil) go to 150

!     --- EMPTY LIST

140   seghd1 = seghd2
      seghd2 = pltjca (seghd2)
145   if (seghd2.eq.nil)  goto 150
      call pltjen (seghd1, endcel)
      if (sqrt ((xcoor (seghd2) - xcoor (endcel))**2 + (zcoor &
            (seghd2) - zcoor (endcel))**2).lt.1d-6) then

!     --- these two segments are continuous and have to be merged
!         seghd2 cell may be dropped

         call pltjdc (endcel, pltjcd (seghd2), gbgcan)
         call pltjdc (seghd2, nil, gbgcan)
         call pltjfs (seghd2)
         call pltjad (seghd1, seghd2, gbgcan)
         if ( ierror/=0 ) goto 99999
         goto 145
      else
         goto 140
      end if
150   continue

!     --- Now remove all double points

      seghd1 = stadr1
160   if (seghd1.eq.nil) goto 200

!     --- 200: End -of- List

      cell0 = seghd1
170   cell1 = pltjcd (cell0)
      if (cell1.eq.nil) then

!     --- Next segment

         seghd1 = pltjca (seghd1)
         goto 160
      end if
      if (xcoor (cell1) - xcoor (cell0).lt.1d-6) then
         if (abs (zcoor (cell1) - zcoor (cell0)).lt.1d-6) then

!        --- Double point. Remove cell1

            call pltjdc (cell0, pltjcd (cell1), gbgcan)
            call pltjdc (cell1, nil, gbgcan)
            call pltjfs (cell1)
            if ( ierror/=0 ) goto 99999
            goto 170
         else
            cell0 = cell1
            goto  170

!           --- Next cell

         end if
      else
         cell0 = cell1
         goto 170

!        --- Next cell

      end if
200   continue
99999 continue
      call erclos ('pltjmr')
      end
