      subroutine pltjms ( stadr1, pred1, stadr2, pred2 )
! ======================================================================
!
!        programmer    Jos van Kan
!        version  1.0  date 20-01-1995
!
!   copyright (c) 1995-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       pltjms moves a segment of the list starting at stadr1 to the list
!       starting at stadr2. The segment to be moved is pointed to by
!       pltjca (pred1). If pred1 = nil the first segment has to be moved.
!       (Of course stadr1 points to this segment). The segment is placed
!       AFTER pred2 so that pltjca (pred2) points to the segment on output.
!       If pred2 = nil stadr2 will get the value pltjca (pred1).
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulepltj
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer stadr1, pred1, stadr2, pred2

!     pred1   i    predecessor cell of segment to be moved. If this
!                  is nil the first segment is moved and stadr1 is made
!                  to point at the second segment. (If any).
!     pred2   i    predecessor cell of segment after the move. If this
!                  is nil stadr2 is made to point at the segment.
!     stadr1  o    startaddres of first list. if pred1 = nil
!                  this will be updated.
!     stadr2  o    startaddress of second list. If pred2 = nil
!                  this will be updated.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer seghdr, gbgcan, optr

!     gbgcan  garbage can. Information that gets there is no longer of use
!     optr    auxiliary variable to temporarily store a pointer
!     seghdr  cellnumber of first cell of a segment
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer pltjca

!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     PLTJAD  to change the segment pointer
!     PLTJCA  to get the next segment
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ('pltjms')
      if (pred1.eq.nil) then

!     --- The first segment has to be moved. stadr1 now must point
!         to second segment. The segment header is the old start address.

      seghdr = stadr1
      stadr1 = pltjca (seghdr)
      else

!     --- the address register of pred1 must be updated
!         former contents are placed in segment header

      call pltjad (pred1, pltjca (pltjca (pred1)), seghdr)
      end if

!     --- Now this segment must be inserted in the second list.

      if (pred2.eq.nil) then

!     --- The segment is placed on top of the list. Address register of
!         the segment header is made to point to former first segment. Old
!         pointer
!         is thrown in the garbage can.

      call pltjad (seghdr, stadr2, gbgcan)
      stadr2 = seghdr
      else

!     --- The segment is placed after pred2. address registers of pred2
!         and seghdr must be updated. first pred2, then the old contents are
!         placed in seghdr. optr acts as temporary variable to store the old
!         contents of pltjca (pred2). Old contents of adrg (seghdr) are thrown
!         away.

      call pltjad (pred2, seghdr, optr)
      call pltjad (seghdr, optr, gbgcan)
      end if
      call erclos ('pltjms')
      end
