      subroutine pltjss ( stadr, point )
! ======================================================================
!
!        programmer    Jos van Kan
!        version  1.1  date 21-11-1995 Better error handling
!        version  1.0  date 20-01-1995
!
!   copyright (c) 1995-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    pltjss separates segments with different visibility property and as such
!    is an auxiliary routine to pltjsl, which accept
!    lists that are preprocessed by pltjss. On exit all cells of any segment
!    have the same visibility property. (hence either all cells of a segment
!    are visible or invisible).
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulepltj
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer stadr, point(*)

!     point   i    array containing the visibility property of the cells
!     stadr   i    start address of the list
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer seghd0, cell0, cell1

!     cell0   cell number
!     cell1   cell number
!     seghd0  first cell of a segment
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer pltjca, pltjcd

!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     PLTJCA  to get the next segment
!     PLTJCD  To find cellpointer to next cell
!     PLTJCU  to cut a segment into a visible and invisible part
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ('pltjss')

!     --- Get first segment.

      seghd0 = stadr

!     --- While {there is a new segment}
!             get new segment
!             treat segment
!         End While {segment}

100   if (seghd0.eq.nil) goto 125

!     --- Get cellnumbers of first two cells (if any)

      cell0 = seghd0
      cell1 = pltjcd (cell0)

!     --- TREAT SEGMENT:
!         While {there is another point in this segment}
!              compare visibility of this point and previous point
!              if they differ, then cut the segment
!         EndWhile {Point}

105   if (cell1.eq.nil) goto 120
      if (point(cell1).ne.point (cell0)) then
         call pltjcu (seghd0, cell0)

!        --- move segment pointer to segment just created

         if ( ierror/=0 ) goto 99999
         seghd0 = pltjca (seghd0)
         cell1 = seghd0
      end if
      cell0 = cell1
      cell1 = pltjcd (cell0)
      goto 105
120   continue

!     --- End While {Point}
!         This is the end of a segment

      seghd0 = pltjca (seghd0)
      go to 100
125   continue

!     --- End While {Segment}
!         This is the end of the list

99999 continue
      call erclos ('pltjss')
      end
