      subroutine pltjvi ( skystr, nlnstr, xcoor, ycoor, point, chckmd )
! ======================================================================
!
!        programmer    Jos van Kan
!        version  1.1  date 21-11-1995 Better error handling
!        version  1.0  date 20-01-1995
!
!   copyright (c) 1995-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    pltjvi performs a visibility check of the list starting at
!    nlnstr (startaddres of the new line) against the list starting
!    at skystr (start address of the sky line). If a point is visible
!    the entry in point is so marked. See for data structure listpack.txt
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulepltj
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision xcoor(*), ycoor(*)
      integer skystr, nlnstr, point(*), chckmd

!     chckmd  i    high (=0) : check against high sky
!                  low (=1) : check against low sky
!     nlnstr  i    start address of the new line
!     point   o    array containing the visibility of the points of sky &
!                  new line
!     skystr  i    start address of the sky
!     xcoor   i    array containing the x coordinates of sky & new line
!     ycoor   i    array containing the y coordinates of sky & new line
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer skyst0, nlnst0, skycl0, skycl1, nlncl0, nlncl1

!     nlncl0  consecutive cell of new line
!     nlncl1  consecutive cell of new line
!     nlnst0  first cells of segments of new line
!     skycl0  consecutive cell of the sky line
!     skycl1  consecutive cell of the sky line
!     skyst0  first cells of segments of sky
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer pltjca, pltjcd

!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     PLTJCA  to get the next segment
!     PLTJCD  To find cellpointer to next cell
!     PLTJMK  to make visible the rest of a segment
!     PLTJPN  to check the visibility of one point and add an intersecting
!             cell if the visibility changes.
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       Both lists are run off simultaneously checking points of one list that
!       are between two consecutive points of the other. If a point has lower x
!       coordinate than first point of the other list it is ipso facto visible.
!       formally:
!       WHILE {neither skylist nor newline list are exhausted}
!          WHILE {neither the current sky segment nor newline segment is
!          exhausted}
!             IF (xcoor (sky cel 1) < xcoor (new line cel 1) THEN
!                Check visibility of skycel 1
!                Get next skycel
!             ELSE
!                Check visibility of new line cel 1
!                Get next new line cell
!             ENDIF
!          End While {segments not exhausted}
!          IF (current sky segment exhausted) THEN
!             Add intersection cell to the line segment runout if last
!             sky point in this segment was visible (if necessary)
!             Get next sky segment
!          ELSE
!             Add intersection cell to the skysegment runout if last new
!             line point in this segment was visible
!             Get next new line segment
!          ENDIF
!       END WHILE {lists not exhausted}
!       IF (skylist exhausted) THEN
!          Remainder of new line list is visible
!       ELSE
!          Remainder of sky list is visible
!       ENDIF
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ('pltjvi')

!     --- STARTUP
!         get first new line and sky segment.

      skyst0 = skystr
      nlnst0 = nlnstr

!     --- Get first cell of both segments

      nlncl0 = nil
      nlncl1 = nlnst0
      skycl0 = nil
      skycl1 = skyst0

!     --- While {neither list exhausted}
!          ..... (see algorithm description)

100   if (skyst0.eq.nil .or. nlnst0.eq.nil) goto 130

!     --- 130: END-OF-LIST
!         While {neither segment exhausted}
!         ..... (see algorithm description)

110   if (skycl1.eq.nil .or. nlncl1.eq.nil) goto 120

!     --- 120: END-OF-SEGMENT

      if (xcoor (skycl1).lt.xcoor (nlncl1)) then
         call pltjpn (skycl0, skycl1, nlncl0, nlncl1, xcoor, ycoor, &
                     point, chckmd)
         if ( ierror/=0 ) goto 99999
         skycl0 = skycl1
         skycl1 = pltjcd (skycl1)
      else
         call pltjpn (nlncl0, nlncl1, skycl0, skycl1, xcoor, ycoor, &
                     point, chckmd)
         if ( ierror/=0 ) goto 99999
         nlncl0 = nlncl1
         nlncl1 = pltjcd (nlncl1)
      end if
      goto 110
120   continue

!     --- End While {segments not exhausted}

      if (skycl1.eq.nil) then

!     --- Sky segment exhausted.
!         Get new sky segment.

         skyst0 = pltjca (skyst0)

!        --- Add intersection point to the new line segment if
!            1. last skypoint was visible.
!            2. There is a discontinuity in the sky or the sky is exhausted

      if (skyst0.eq.nil) then
      call pltjpn (nlncl0, nlncl1, skycl0, skycl1, xcoor, ycoor, point, &
       chckmd)
         if ( ierror/=0 ) goto 99999
      else if (sqrt ((xcoor (skycl0) - xcoor (skyst0)) ** 2 + (ycoor &
                        (skycl0) - ycoor (skyst0)) ** 2).gt.1d-3) then
      call pltjpn (nlncl0, nlncl1, skycl0, skycl1, xcoor, ycoor, point, &
       chckmd)
         if ( ierror/=0 ) goto 99999
      skycl0 = nil
      end if

!     --- Initialize first point of new sky segment

         skycl1 = skyst0
      else

!     --- New line segment exhausted.
!         Get new segment.

         nlnst0 = pltjca (nlnst0)

!        --- Add intersection point to the sky segment if
!            1. last new line point was visible.
!            2. There is a discontinuity in the line segment. or segment
!               exhausted

      if (nlnst0.eq.nil) then
      call pltjpn (skycl0, skycl1, nlncl0, nlncl1, xcoor, ycoor, point, &
       chckmd)
         if ( ierror/=0 ) goto 99999
      else if (sqrt ((xcoor (nlncl0) - xcoor (nlnst0)) ** 2 + (ycoor &
                        (nlncl0) - ycoor (nlnst0)) ** 2).gt.1d-3) then
      call pltjpn (skycl0, skycl1, nlncl0, nlncl1, xcoor, ycoor, point, &
       chckmd)
         if ( ierror/=0 ) goto 99999
      nlncl0 = nil
      end if

!     --- Initialize first point of new segment

         nlncl1 = nlnst0
      end if
      go to 100
130   continue

!     --- End While {lists not exhausted}
!         Now clean up the mess. The remainder of the list that was
!         not exhausted is visible.

      if (skyst0.eq.nil) then

!     --- Sky list exhausted. Remainder of new line list is visible.

         call pltjmk (nlnst0, nlncl1, point)
      else

!     --- New line list exhausted. Remainder of skylist is visible.

         call pltjmk (skyst0, skycl1, point)
      end if
99999 continue
      call erclos ('pltjvi')
      end
