      subroutine pltr01( x, y, iresul, xl, yl, ski, eta, det )
! ======================================================================
!
!        programmer    niek praagman
!        version  2.0  date 22-01-1993 redesign
!        version  1.0  date 08-03-1991
!
!   copyright (c) 1991-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subroutine to determine whether point (x,y) is inside or outside a
!     triangular element. Local coordinates (ski,eta) are computed.
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     location_determination
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision x, y, xl(3), yl(3), ski, eta, det
      integer iresul

!     det      o       surface triangle
!     eta      o       local coordinate
!     iresul   o       indicator : point inside  => iresul = 1
!                                  point outside => iresul = 0
!     ski      o       local coordinate
!     x        i       x-coordinate of point to be considered
!     xl       i       x-coordinates triangle
!     y        i       y-coordinate of point to be considered
!     yl       i       y-coordinates triangle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision eps

!     eps    accuracy parameter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Determine accuracy

      eps = 10 * epsmac

!     --- Compute ski and eta

      det = (xl(2)-xl(1))*(yl(3)-yl(1)) - (xl(3)-xl(1))*(yl(2)-yl(1))
      ski =   ((yl(3)-yl(1))*x - (xl(3)-xl(1))*y - &
                yl(3)*xl(1) + xl(3)*yl(1)) / det
      eta = - ((yl(2)-yl(1))*x - (xl(2)-xl(1))*y - &
                yl(2)*xl(1) + xl(2)*yl(1)) / det
      iresul = 0
      if ( ski     .lt.-eps ) goto 1000
      if ( ski        .gt.1d0+eps ) goto 1000
      if ( eta     .lt.-eps ) goto 1000
      if ( eta        .gt.1d0+eps ) goto 1000
      if ( 1d0-ski-eta.lt.   -eps ) goto 1000

!     --- Point of this triangle

      iresul = 1
1000  continue
      end
