      subroutine pltr15 ( npoint, coor, nview, ndim )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.2  date 25-04-2010 New call to plaf05
!        version  2.1  date 29-12-2000 Layout
!        version  2.0  date 06-04-1996 Complete update, plot complete array
!
!   copyright (c) 1991-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Make a plot of a 3D curve with a projection depending on nview
!
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     plot
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, nview, ndim
      double precision coor(ndim,npoint)

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     ndim       i    First dimension of coor
!     npoint     i    Number of nodal points
!     nview      i    indicator for eye-position/direction
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision :: work(2,npoint)

!     i          Counting variable
!     work       Work array of length 2*npoint
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     PLAF05    Plot polygon
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Compute projection of the arrays

      if ( nview==1 ) then
         do i = 1, npoint
            work(1,i) = coor(1,i) + 0.77d0*coor(2,i)
            work(2,i) = coor(3,i) + 0.45d0*coor(2,i)
         end do
      else if ( nview==2 ) then
         do i = 1, npoint
            work(1,i) = coor(2,i) - 0.77d0*coor(1,i)
            work(2,i) = coor(3,i) - 0.45d0*coor(1,i)
         end do
      else if ( nview==3 ) then
         do i = 1, npoint
            work(1,i) =-coor(1,i) - 0.77d0*coor(2,i)
            work(2,i) = coor(3,i) - 0.45d0*coor(2,i)
         end do
      else if ( nview==4 ) then
         do i = 1, npoint
            work(1,i) =-coor(2,i) + 0.77d0*coor(1,i)
            work(2,i) = coor(3,i) + 0.45d0*coor(1,i)
         end do
      end if

!     --- Make plot

      call plaf05 ( 3, dcolor, npoint, work, 2 )
      end
