      subroutine prcalk ( elkey, diael, coor, kmeshc, mxdia, nelem )
! ======================================================================
!
!
!        programmer     Jos van Kan
!        version 1.0    date   20-01-1995
!
!
!
!   copyright (c) 1995-1995 "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!    prcalk calculates a sortkey for the elements in kmeshc
!    with respect to the y coordinate. The sortkey is the lowest y coor- &
!    dinate of an element. If subroutine rotate is called first the
!    sortkey with respect to any given direction can be calculated.
! ********************************************************************
!
!                      KEYWORDS
!
!
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer kmeshc(*), nelem
      double precision elkey(nelem), diael(nelem), coor(*), mxdia

!     coor    i    array of length ndim x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!     diael   o    element diameter y coordinate
!     elkey   o    element sortkey y coordinate
!     kmeshc  i    nodal point numbers of elements
!     mxdia   o    max element diam  y coordinate
!     nelem   i    Number of elements
! ********************************************************************
!
!                      COMMON BLOCKS
!
! ********************************************************************
!
!                      LOCAL PARAMETERS
!
      integer i, i1, i2, i3

!     i       Counting variable
!     i1      auxiliary variables containg indices of y-coordinates
!             in coor of element under consideration.
!     i2      See i1
!     i3      See i1
! ********************************************************************
!
!                      SUBROUTINES CALLED
!
! ********************************************************************
!
!                      I/O
!
! ********************************************************************
!
!                      ERROR MESSAGES
!
! ********************************************************************
!
!                      PSEUDO CODE
!
!   trivial
! ======================================================================
!
      mxdia = 0d0
!     --- For all elements ....

      do 200 i = 1, nelem
!     --- calculate indices of y-coordinates in coor for this element

         i1 = 2 * kmeshc (3 * i - 2)
         i2 = 2 * kmeshc (3 * i - 1)
         i3 = 2 * kmeshc (3 * i)
!        --- Store minimum of these coordinates in elkey

         elkey (i) = min (coor (i1), coor (i2), coor (i3))
!        --- Store maximum difference of these coordinates in diael

         diael (i) = max (coor (i1), coor (i2), coor (i3)) - elkey (i)
!        --- calculate maximum "diameter"

         if (mxdia.lt.diael (i)) then
            mxdia = diael (i)
         end if
200   continue
      end
