      subroutine prcompncntln ( ncntlnus, ncntln, ichois, iplots )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute number of contour lines ncntln from ncntlnus
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot_contour
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ncntlnus, ichois, iplots(*)
      integer, intent(out) :: ncntln

!     ichois         i    Choice parameter
!                         1: standard contour plot
!                         2: coloured levels
!     iplots         i    Integer input array for plot subroutine
!                         See subroutine plbc01
!     ncntln         o    number of contour levels
!     ncntlnus       i    Parameter to compute the number of contour levels
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     116   NCONT incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcompncntln' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcompncntln'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ncntln = ncntlnus
      if ( ncntlnus==-10000 .or. ncntlnus==-10001 ) then

!     --- The number of positive and negative levels is given in iplots(23-24)

          ncntln = iplots(23)+iplots(24)+1

      else if ( ncntln<-2000 .or. ncntln>1000 .or. &
                ncntln==-1001 .or. ncntln==-1 ) then
         call errint ( ncntln, 1 )
         call errwar ( 116, 1, 0, 0 )
         ncntln = 0
      end if
      if ( ncntln>=0 ) then
         if ( ncntln==0 ) then
            if ( ichois==2 ) then
               ncntln = 21
            else
               ncntln = 11
            end if
         end if
      else if ( ncntln>=-1000 ) then
         ncntln = abs(ncntln)
      else if ( ncntln>-10000 ) then
         ncntln = abs(ncntln+1000)
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcompncntln'

      end if  ! ( debug )
      call erclos ( 'prcompncntln' )

      end subroutine prcompncntln
