      subroutine prelin0 ( slevel, elkey, diael, kgrad, mxdia, nelem, &
                          niout )
! ======================================================================
!
!        programmer     Jos van Kan
!        version 1.0    date 21-12-1998
!
!
!
!   copyright (c) 1998-1998 "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!    prelin0 finds in a list of sorted elements all elements that
!    cut the line y = slevel. The number of elements is stored in niout.
!    kgrad is
!    the array that contains the permutation indices to sort the
!    elements in ascending order, using elkey as sortkey.
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      double precision mxdia, elkey (*), diael(*), slevel
      integer kgrad(*), nelem, niout

!     diael   i    element diameter, y coordinate.
!     elkey   i    element sortkey, y coordinate
!     kgrad   i    integer array containing the permutation
!                  that sorts elkey. (ascending). Use sort
!                  (see sortpack.f) to get kgrad.
!     mxdia   i    maximum element diameter (y coordinate)
!     nelem   i    Number of elements
!     niout   o    number of elements in iout.
!     slevel  i    value of y coordinate to intersect the elements
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ind1, index

!       ind1      index in sortkey of element under consideration
!       index     index in kgrad of element under consideration
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     PRBINS  ?
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       WHILE {list in srtkey lower than slevel is nonempty}
!        if (highest entry intersects the line y = slevel)
!           add this element to the outputlist;
!           remove this element from the srtkey list;
!        else
!           check if any element in th remaining srtkey list can
!           intersect the line y = slevel;
!           if not, remove all elements from the remaining srtkey list;
!           if so, remove this element from the sortkey list;
!       END WHILE
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- search for highest sortkey lower than or equal to slevel

      call prbins (elkey, kgrad, slevel, index, nelem)

!     --- Initialize number of elements on output

      niout = 0

!     --- WHILE start

100   if (index.eq.0) return

!     --- Get index of highest element in sortkey

      ind1 = kgrad (index)
      if (elkey (ind1) + diael (ind1).gt.slevel) then

!     --- Add to ouputlist

         niout = niout + 1

!        --- Remove from inputlist

         index = index - 1
      else if (slevel - elkey (ind1).gt.mxdia) then

!     --- Remove all elements from input list

         index = 0
      else

!     --- Remove from inputlist

         index = index - 1
      end if
      goto 100
      end
