      subroutine prplotminmax ( nfunc, funcx, funcy, ny )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-05-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute minimum and maximum x and y values for a plot
!
! **********************************************************************
!
!                       KEYWORDS
!
!     plot
!     boundary
!     minimum
!     maximum
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nfunc, ny
      double precision, intent(in) :: funcx(nfunc), funcy(nfunc,ny)

!     funcx          i    Set of values in x-direction
!     funcy          i    ny sets of values in y-direction
!     nfunc          i    Number of values per function
!     ny             i    number of functions to be plotted
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision dx, dy

!     dx             10% of difference in x-direction
!     dy             10% of difference in y-direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLOTFUNC1MINMX Compute minimum and maximum along x and y-axis for
!                    function plots
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prplotminmax' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prplotminmax'
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( kiesx==-1 .or. kiesy==-1 ) then

!     --- Compute minimum and maximum x and y-values
!         In this part also xmin, xmax, ymin and ymax are set

         call plotfunc1minmx ( nfunc, funcx, funcy, ny )
         if ( ierror/=0 ) go to 1000

      else

!     --- Set xmin, xmax, ymin, ymax

         dx = (axmax-axmin)*.1d0
         dy = (aymax-aymin)*.1d0

         xmax = axmax+dx*5d0
         ymax = aymax+dy
         xmin = axmin-dx
         ymin = aymin-dy

      end if  ! ( kiesx==-1 .or. kiesy==-1 )

1000  call erclos ( 'prplotminmax' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'axmin, axmax, aymin, aymax', &
                          axmin, axmax, aymin, aymax
         write(irefwr,*) 'End prplotminmax'

      end if  ! ( debug )

      end
