      subroutine prtecplot1 ( isol, ndim, varline, lenvar, maxvec1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-04-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill string varline with the containing variables
!
! **********************************************************************
!
!                       KEYWORDS
!
!     tecplot
!     output
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: isol(5,*), ndim, maxvec1
      integer, intent(out) :: lenvar
      character (len=*) :: varline

!     isol           i    Standard SEPRAN array containing information of the
!                         solution arrays
!     lenvar         o    Length of text in varline
!     maxvec1        i    Maximum number of vectors
!     ndim           i    Dimension of the space.
!     varline        o    Is used to store the line with variables
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, lenname, j
      character (len=4) :: namehead

!     i              Counting variable
!     j              Counting variable
!     lenname        Length of name
!     namehead       String to store text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ASKUPC         Translate all letters to upper case
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prtecplot1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prtecplot1'
         write(irefwr,1) 'ndim, maxvec1', ndim, maxvec1
         call prinin1 ( isol, maxvec1, 5, 'isol' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      varline = 'VARIABLES = "X" "Y"'
      lenvar = 19
      if ( ndim==3 ) then
         varline(lenvar+1:lenvar+4) = ' "Z"'
         lenvar = lenvar+4
      end if  ! ( ndim==3 )

!     --- For all vectors 1 to maxvec1 do

      do i = 1, maxvec1

         if ( isol(2,i)==110 .or. isol(2,i)==115 ) then

!        --- Vector has been filled and is of type 110 or 115
!            Determine type of vector, inunkp and number of vectors

            call sepactsolbf2 ( isol(1,i) )

            lenname = index(names(i),' ')-1
            if ( lenname<0 ) lenname = len(names(i))
            if ( nphysi==1 ) then

!           --- 1 unknown per point

               varline(lenvar+1:lenvar+lenname+3) = &
                  ' "'//names(i)(1:lenname)//'"'
               lenvar = lenvar+lenname+3

            else if ( nphysi==ndim ) then

!           --- nphysi = ndim

               namehead = names(i)(1:4)
               call askupc ( namehead)
               if ( namehead=='VELO' ) then

!              --- velocity found

                  varline(lenvar+1:lenvar+8) = ' "U" "V"'
                  lenvar = lenvar + 8
                  if ( ndim==3 ) then
                     varline(lenvar+1:lenvar+4) = ' "W"'
                     lenvar = lenvar + 4
                  end if  ! ( ndim==3 )

               else

!              --- unknown vector found

                  do j = 1, nphysi
                     varline(lenvar+1:lenvar+lenname+2) = &
                       ' "'//names(i)(1:lenname)
                     lenvar = lenvar+lenname+2
                     write(varline(lenvar+1:lenvar+3),100)
100                  format('_',i1,'"' )
                     lenvar = lenvar + 3

                  end do  ! i = 1, nphysi

               end if  ! ( namehead=='VELO' )

            else if ( nphysi==ndim+1 ) then

!           --- nphysi = ndim+1

               namehead = names(i)(1:4)
               call askupc ( namehead)
               if ( namehead=='VELO' ) then

!              --- velocity pressure found

                  varline(lenvar+1:lenvar+8) = ' "U" "V"'
                  lenvar = lenvar + 8
                  if ( ndim==3 ) then
                     varline(lenvar+1:lenvar+4) = ' "W"'
                     lenvar = lenvar + 4
                  end if  ! ( ndim==3 )
                  varline(lenvar+1:lenvar+4) = ' "P"'
                  lenvar = lenvar + 4

               else

!              --- unknown vector found

                  do j = 1, nphysi
                     varline(lenvar+1:lenvar+lenname+2) = &
                       ' "'//names(i)(1:lenname)
                     lenvar = lenvar+lenname+2
                     write(varline(lenvar+1:lenvar+3),100)
                     lenvar = lenvar + 3

                  end do  ! i = 1, nphysi

               end if  ! ( namehead=='VELO' )

            else

!           --- nphysi has other value

               do j = 1, nphysi
                  varline(lenvar+1:lenvar+lenname+2) = &
                    ' "'//names(i)(1:lenname)
                  lenvar = lenvar+lenname+2
                  write(varline(lenvar+1:lenvar+3),100)
                  lenvar = lenvar + 3

               end do  ! i = 1, nphysi

            end if  ! ( nphysi==1 )

         end if  ! ( isol(2,i)==110 .or. isol(2,i)==115 )

      end do  ! i = 1, maxvec1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prtecplot1'

      end if  ! ( debug )
      call erclos ( 'prtecplot1' )

      end
