      subroutine sepplelemobst
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Plot elements that are in or partly in an obstacle
!     Two-dimensional case only
!     Old subroutine plelemobst
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     obstacle
!     plot
!     plot_mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iscolr, inpelm, ielgrp, nboun, ikelmc, ielem, iielem, &
              iobst, istart, numinelmobst, numbnelmobst
      integer, allocatable :: iwork(:)

!     ielem          Element number
!     ielgrp         element group number
!     iielem         Relative element number in element group
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     iobst          Obstacle sequence number
!     iscolr         Help variable to store the value of dcolor temporarily
!     istart         First position used at a part
!     iwork          Work array of length nelem
!                    if ( iwork(i)> 0, element i is completely inside
!                    the obstacle
!                    if ( iwork(i)< 0, element i is partly inside
!                    the obstacle
!     nboun          Number of boundary points at an element
!     numbnelmobst   Number of elements that are partly within the obstacle
!     numinelmobst   Number of elements that are completely within the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getnpelmelgrp, getnumelmelgrp

!     ERALLOC        Produce error message in case allocate went wrong
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     GETNPELMELGRP  Fetch number of points per element in element group ielgrp
!     GETNUMELMELGRP Determine number of elements in element group
!     PLAFP7         Plot text using absolute co-ordinates
!     PLOTELEM       Plot one element (2d) with color dcolor
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      iscolr = dcolor

!     --- allocate work array

      allocate ( iwork(nelem), stat = error )
      if ( error/=0 ) call eralloc ( error, inelem, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      iwork = 0

      do iobst = 1, numobst

!     --- Fill array iwork

         istart = ielemobst(iobst)
         numinelmobst = ielemobst(istart)+ielemobst(istart+1)
         numbnelmobst = ielemobst(istart+2)
         istart = istart+2
         iwork(ielemobst(istart+1:istart+numinelmobst)) = 1
         istart = istart+numinelmobst
         iwork(ielemobst(istart+1:istart+numbnelmobst)) = -1

      end do  ! iobst = 1, numobst

!     --- First the elements outside the obstacle are plotted

      ikelmc = 1
      ielem  = 0
      do ielgrp = 1, nelgrp

!     --- Determine number of elements and number of nodes in element

         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         nboun = inpelm
         if ( kelmb/=0 ) then
            if ( kmeshb(ielgrp)>0 ) nboun = kmeshb(ielgrp)
         end if
         do iielem = 1, inelem

            ielem = ielem+1
            if ( iwork(ielem)==0 ) &
               call plotelem ( kmeshc(ikelmc), nboun, coor, 0, .true. )
            ikelmc = ikelmc+inpelm

         end do

      end do

!     --- Then the elements near the boundary are plotted

      ikelmc = 1
      ielem  = 0
      do ielgrp = 1, nelgrp

!     --- Determine number of elements and number of nodes in element

         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         nboun = inpelm
         if ( kelmb/=0 ) then
            if ( kmeshb(ielgrp)>0 ) nboun = kmeshb(ielgrp)
         end if
         do iielem = 1, inelem

            ielem = ielem+1
            if ( iwork(ielem)<0 ) &
               call plotelem ( kmeshc(ikelmc), nboun, coor, &
                               64+max(-40,iobst), .true. )
            ikelmc = ikelmc+inpelm

         end do

      end do

!     --- Then the internal elements

      ikelmc = 1
      ielem  = 0
      do ielgrp = 1, nelgrp

!     --- Determine number of elements and number of nodes in element

         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         nboun = inpelm
         if ( kelmb/=0 ) then
            if ( kmeshb(ielgrp)>0 ) nboun = kmeshb(ielgrp)
         end if
         do iielem = 1, inelem

            ielem = ielem+1
            if ( iwork(ielem)>0 ) &
               call plotelem ( kmeshc(ikelmc), nboun, coor, &
                               44+iobst, .true. )
            ikelmc = ikelmc+inpelm

         end do

      end do

      dcolor = iscolr
      call plafp7 ( 15d0, 19.5d0, 'elements covering the obstacle', 0d0, &
                    0.25d0, 30 )
      dcolor = 44
      call plafp7 ( 15d0, 19d0, '-- internal', 0d0, 0.25d0, 11 )
      dcolor = 65
      call plafp7 ( 15d0, 18.5d0, '-- boundary', 0d0, 0.25d0, 11 )
      dcolor = iscolr

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  end
