      subroutine sepplot2dobst ( ichoice )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     plot information about a 2d obstacle
!     Old subroutine plot2dobst
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     obstacle
!     plot
!     print
!     debug
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichoice

!     ichoice        i    Integer choice parameter
!                         See subroutine mshlevelkm2ad
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ikader, iscolr

!     ikader         Help variable to store jkader temporarily
!     iscolr         Help variable to store the value of dcolor temporarily
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PL0003         Open plot, plots frame work and computes plotting
!     PLAFEP         General SEPRAN low level plotting subroutine
!     SEPPLELEMOBST  Plot elements that cover obstacles
!     SEPPLOTBN      Plot boundary of region
!     SEPPLPOINOBST  Plot nodes of the mesh in obstacles
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepplot2dobst' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepplot2dobst'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- open plot

      jkader = -1
      textx = 'x'
      texty = 'y'
      call pl0003 ( ikader, 1, 15d0, 1d0, 10 )

!     --- plot elements

      if ( ichoice==0 .or. ichoice>=2 ) call sepplelemobst

!     --- plot boundary

      iscolr = dcolor
      dcolor = 1
      call sepplotbn ( 0, 3, .true. )
      dcolor = iscolr

!     --- plot nodes in obstacle

      call sepplpoinobst

!     --- close plot dataset

      call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepplot2dobst'

      end if  ! ( debug )
      call erclos ( 'sepplot2dobst' )

      end subroutine sepplot2dobst
