      subroutine sepplotm1 ( ichois, iuser, plotfm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 29-05-2014 Extension of timebefore
!        version  1.4  date 12-09-2013 Add plottime
!        version  1.3  date 25-11-2012 Long integers
!
!   copyright (c) 1999-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Plot one-dimensional mesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     1d
!     plot
!     plot_mesh
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois, iuser(*)
      double precision plotfm

!    ichois  i   parameter to distinguish between the following
!                possibilities
!                0    standard method
!                1    only a part of the mesh is plotted.
!                     array iuser is used.  see programmers guide
!                2    The elements are contracted
!    iuser   i   Integer user array to indicate if element group IELGRP must
!                be skipped (iuser(ielgrp+5)=0) or not (iuser(ielgrp+5)=1)
!    plotfm  i   Plotting factor. When PLOTFM<0, then the length of
!                the plot is computed by - (xmax-xmin) * PLOTFM
!                and the width by - (ymax-ymin) * YFACT * PLOTFM
!                else the length of the plot is equal to PLOTFM cm
!                When by some mistake the user has given | PLOTFM | a value
!                smaller than 1 cm, then the length is chosen equal to
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical ncontr, part, plboun, plrenm, fillcl
      integer ikader, imark, iscolr
      double precision :: timebefore(2)

!     fillcl         Indicates if the elements must be filled with a colour
!                    (true) or that only the boundaries must be plotted (false)
!                    If fillcl = true, the colour is given by dcolor, the
!                    boundaries get the default colour. Otherwise the colour of
!                    the boundaries is defined by dcolor
!     ikader         Help variable to store jkader temporarily
!     imark          ?
!     iscolr         Help variable to store the value of dcolor temporarily
!     ncontr         indication whether the elements are contracted (false)
!                    or not (true)
!     part           indication whether all elements are plotted (false)
!                    or that some are skipped (true)
!     plboun         Indicates if the boundaries must be plotted (true) or not
!     plrenm         Indicates if the renumbered nodes must be plotted (true)
!                    or not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERSETTIME      Get the CPU time at start of subroutine
!     INITS5         Compute parameters for the plotting subroutine
!     PL0000         Compute length and width of the plot
!     PLAFEP         General SEPRAN low level plotting subroutine
!     PLELM1         Plot elements in 2D mesh
!     PLOTTIME       Plot the actual time in a plot depending on the parameters
!                    itimdep and t in common ctimen
!     PLPOIN         plot nodal points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepplotm1' )
      if ( itime>0 ) call ersettime ( timebefore )
      iscolr = dcolor
      ikader = jkader
      call inits5 ( iuser, ncontr, jmark, imark, &
                    part, plboun, ichois, plrenm, fillcl )
      fillcl = .false.

!     --- Compute minimum and maximum values of co-ordinates

      xmin = minval(coor(1,1:npoint))
      xmax = maxval(coor(1,1:npoint))
      ymax = 0.25d0 * ( xmax - xmin )
      ymin = - ymax

!     --- Compute the parameters aleng and width

      call pl0000 ( plotfm, 1d0 )

!     --- start picture

      call plafep ( 1, aleng, width, 0d0, 0d0, 0 )

!     --- plot nodal points

      if ( plrenm ) npoint = -npoint

      if ( iscolr>0 ) dcolor = iscolr + 1
      if(imark<2) call plpoin ( coor, npoint, kmeshj, 1 )

!     --- plot elements

      call plelm1 ( iuser(6), coor, ncontr, part, fillcl )

      call plottime  ! plot time label if time dependent

!     --- close plot dataset

      call plafep ( 2, 0d0, 0d0, 0d0, 0d0, 0 )
      call erclmn ( 'sepplotm1', timebefore )
      jkader = ikader
      dcolor = iscolr
      end subroutine sepplotm1
