      subroutine to0028 ( itypvc, ich1, vectr1, coor, coorc, iwork, &
                          index1, index2, index3, amin, almax, avmax )
! ======================================================================
!
!        programmer    jaap van der zanden
!        version  3.0  date 27-11-1992 Removal of ichoic
!        version  2.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  2.0  date 12-03-1991 addition itypvc=116; revision
!
!   copyright (c) 1984-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   scale factor amin is determined for real vectors vectr1,2
!     for vector to be plotted (ich1 from vectr1 together with
!     imaginary part of vectr1 form the vector)
!   amin=min(local grid size/length of vector)
!   almax,avmax are updated if required
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduleplot
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer itypvc,ich1, &
              iwork(*), index1(*), index2(*), index3(*)
      double precision vectr1(*), coor(*), coorc(*), &
             amin, almax, avmax

!     almax      i    maximum distance between neighbouring nodal points
!     amin       i    min(
!                     distance between nodal point and its nearest neighbour/
!                     length of vector (vectr1(ich1),vectr2(ich2))
!     avmax      i    maximum length of vector
!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     coorc      i    array with coordinates of centroid of element
!     ich1       i    degree of freedom ich1 from vectr1 together with
!                     imaginary part from vectr1 form
!                     the vector to be plotted
!     index1     i    Array of length inpelm containing the point numbers
!                     of the nodal points in the element
!     index2     i    arrays with positions in vectr1 of degrees
!                     of freedom in nodal points of actual element, &
!                     or degrees of freedom per element
!     index3     i    arrays with positions in vectr1 of degrees
!                     of freedom in nodal points of actual element, &
!                     or degrees of freedom per element
!     itypvc     i    Indication of the type of solution vector:
!                     110: solution vector
!                     115, 116: vectors of special structure
!     iwork      i    integer work array
!                     if iwork(iknpt)=0 nodal point iknpt should be considered
!                                    =1 nodal point has already been 'seen'
!                                    =-1 nodal point is outside range
!                                        or does not contain degree of freedom
!     vectr1     i    standard sepran vectors (type 110,115) defined
!                     in any nodal point, or (type=116) defined per element
!            ( distance between nodal point and its nearest neighbour )
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ih, ihi, indloc(2), j, jcoun1, num1
      double precision al, av, sch, v1, v2, x, xc, x1, y, yc, y1

!     al
!     av
!     i          Counting variable
!     ih         Help variable to store a constant temporarily
!     ihi
!     ind
!     j          Counting variable
!     jcoun1
!     num1
!     sch
!     v1
!     v2
!     x
!     x1
!     xc
!     y
!     y1
!     yc
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if (itypvc.eq.116) then
         jcoun1=index3(1)
         num1  =index3(2)
         xc=0.0d0
         yc=0.0d0
      end if
      do 10 i=1,inpelm

!     --- test if point has already been "seen" before

         ihi=index1(i)
         if (iwork(ihi).ne.0) goto 10
         if (itypvc.eq.110 .or. itypvc.eq.115) then
            jcoun1=index3(i)
            num1  =index3(i+1)
         end if

!        --- test if degree of freedom is present in nodal point

         if (ich1.gt.num1-jcoun1) then
            iwork(ihi)=-1
         else

!        --- get coordinates and test if they are within range

            ih=2*ihi
            x=coor(ih-1)
            y=coor(ih)
            if ((x.ge.xmin).and.(x.le.xmax).and.(y.ge.ymin) .and. &
               (y.le.ymax)) then

!           --- determine length of vector

               ih=2*index2(jcoun1+ich1)
               v1=vectr1(ih-1)
               v2=vectr1(ih)
               av=v1*v1+v2*v2

!              --- determine distance between neigbouring nodal points

               indloc(1)=i+1
               if (i.eq.inpelm) indloc(1)=1
               indloc(2)=i-1
               if (i.eq.1) indloc(2)=inpelm
               do 7 j=1,2
                  ih=2*index1(indloc(j))
                  x1=coor(ih-1)
                  y1=coor(ih)
                  if ((x1.ge.xmin).and.(x1.le.xmax).and. &
                     (y1.ge.ymin).and.(y1.le.ymax).and. &
                     (av.gt.epsmac)) then
                     al=(x1-x)*(x1-x)+(y1-y)*(y1-y)
                     sch=al/av
                     if (sch.lt.amin) amin=sch
                     if (al.gt.almax) almax=al
                     if (av.gt.avmax) avmax=av
                  end if
7              continue

!              --- reset iwork for this point

               iwork(ihi)=1
            else
               iwork(ihi)=-1
            end if
         end if
10    continue
      if (itypvc.eq.116) then

!     --- store coordinates of centroid of element ielem

         do 20 i=1,inpelm
            ihi=index1(i)
            ih=2*ihi
            xc=xc+coor(ih-1)
            yc=yc+coor(ih)
20       continue
         xc=xc/inpelm
         yc=yc/inpelm
         coorc(ielem*2-1)=xc
         coorc(ielem*2  )=yc
      end if
      end
