      subroutine topsubdivtri ( skipelgrps )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Subdivide mesh into triangles depending on skipelgrps
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     subdivision
!     surface
!     2d
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: skipelgrps(2,*)

!     skipelgrps     i    array of size 2 x nelgrp containing information
!                         for the subdivision of the elements in triangles
!                         (1,i) mult, i.e. number of triangles each element
!                               of element group i must be subdivided
!                         (2,i) typesubdiv, defines special subdivisions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ielgrp, inpelm, ikelmc, mult, ikelmctmp

!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     ikelmctmp      position in kmeshctmp
!     inpelm         number of points per element
!     mult           Number of triangles an element is subdivided in
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     TOPSUBDIVTRI1  Subdivide mesh into triangles for one element group
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topsubdivtri' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topsubdivtri'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ikelmc = 1
      ikelmctmp = 1

!     --- For all element groups do

      do ielgrp = 1, nelgrp

         ishape = kmeshf(ielgrp)   ! shape of element
         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem

         mult = skipelgrps(1,ielgrp)

         if ( mult>0 ) then

!        --- subdivide element

            call topsubdivtri1 ( kmeshc(ikelmc), kmeshctmp(ikelmctmp), &
                                 inelem, mult, ishape, inpelm )
            ikelmctmp = ikelmctmp+inelem*3*mult  ! raise ikelmctmp

         end if  ! ( mult>0 )

         ikelmc = ikelmc+inelem*inpelm  ! raise ikelmc

      end do  ! ielgrp = 1, nelgrp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topsubdivtri'

      end if  ! ( debug )
      call erclos ( 'topsubdivtri' )

      end subroutine topsubdivtri
