      subroutine creatv116 ( isol, nphys, itypvc, kfill, kfun, &
                             kdeg, icompl, inpcre, rincre, jpstin, &
                             kvec, istartvc, maxphys, mvec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 05-12-2012 Long integers
!        version  3.0  date 03-01-2012 New parameter list
!        version  2.0  date 11-08-2009 New parameter list
!
!   copyright (c) 1999-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays of type 116 or 126
!     Help subroutine for creatvbf
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
!     vector_defined_per_element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulevecs
      use sepmoduletexts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*), nphys, &
              itypvc, kfill, kfun, kdeg, icompl, inpcre(*), jpstin, &
              kvec, istartvc, maxphys, mvec
      double precision rincre(*)

!     icompl         i    Multiplication factor for complex numbers
!                         Real: icompl=1,   Complex: icompl=2
!     inpcre         i    Array containing the integer input
!                         See subroutine creatv01
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution array
!     istartvc       i    Starting position of sequence numbers of vectors
!                         in inpcre
!     itypvc         i    Indication of the type of solution vector:
!                         110: solution vector
!                         115, 116: vectors of special structure
!     jpstin         i    Help variable to compute new value of ipstin partly
!     kdeg           i    Indication which degrees of freedom must be filled
!                         Possibilities:
!                         0:    All degrees of freedom
!                         >0:   Only the KDEGth degree of freedom
!                         <0:   The -KDEG degrees of freedom stored in array
!                               IARDEG must be filled
!     kfill          i    Indication how the nodal points must be filled into
!                         array NODES
!                         See subroutine prfillnodes
!     kfun           i    Indication of how the degrees of freedom must be
!                         filled.
!                         Possibilities:
!                         0:  Not filled
!                         <0:  Constant value stored in RINCRE(-KFUN)
!                         >0:  Function subroutine
!     kvec           i    Loop variable over the vectors
!     maxphys        i    Maximum number of unknowns per point
!     mvec           i    Number of old vectors to be used in FUNCVECT
!     nphys          i    Number of physical unknowns, or maximal degrees of
!                         freedom per point
!     rincre         i    Double precision array containing the values of the
!                         constant if KFUN<0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipos
      character (len=80) filename

!     filename       Name of file
!     ipos           Position in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CREATV116CONST Fill vector of type 116 with a constant
!     CREATV116GROUP Fill vector of type 116 for specific element group
!     CREATVSPEC116  Fill array usol with values in the case that 1000<fkun<2000
!     ER0064         Fill constant value in vector of special structure 116
!                     real case
!     ER0073         Fill constant value in vector of special structure 116
!                     complex case
!     ER0091         Fill elements values from a file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     218   Degree of freedom does not exist
!    1675   Wrong value of reference to texts
!    1678   For vectors of special structure defined per element, only the
!           filling of a constant over a complete region has been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'creatv116' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from creatv116'
         write(irefwr,1) 'itypvc, kfill, kfun', itypvc, kfill, kfun
         write(irefwr,1) 'jpstin, kvec, istartvc, maxphys, mvec', &
                          jpstin, kvec, istartvc, maxphys, mvec
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )

!     --- Compute starting address of vector to be created

      usolt => ks(isol(kvec))%sol

      if ( itypvc==116 .and. kfill==13 ) then

!     --- kfill = 13, read information from file
!         Only implemented for type 116

         ipos = inpcre(jpstin)
         if ( ipos<=0 .or. ipos>=nextxt ) then

!        --- ipos out of range

            call errint ( jpstin, 1 )
            call errint ( ipos, 2 )
            call errint ( nextxt, 3 )
            call errsub ( 1675, 3, 0, 0 )
            go to 1000

         end if
         filename = texts(ipos)
         call er0091 ( filename, usolt, nphys )

      else if ( itypvc==116 .and. kfill==5 ) then

!     --- kfill = 5, fill per element
!         Only implemented for type 116

         call creatv116const ( usolt, inpcre, rincre, jpstin, &
                               kdeg, kfun, nphys )

      else if ( itypvc==116 .and. kfill==6 ) then

!     --- kfill = 6, fill per element group
!         Only implemented for type 116

         call creatv116group ( inpcre, kfun, &
                               rincre, kdeg, nphys, maxphys, mvec, &
                               isol, jpstin, istartvc, kvec )

      else if ( kfill>0 ) then

!     --- Only kfill = 0 and kfill = 13 has been implemented

         call errsub ( 1678, 0, 0, 0 )

      else if ( itypvc==116 .and. kfill==0 .and. kfun>1000 .and. &
                kfun<2000 .and. kdeg==0 ) then

!     --- kfill = 0, itypvc = 116 and 1000<kfun<2000, special function

         call creatvspec116 ( kfun, nphys, usolt, icompl )

      else if ( icompl==0 ) then

!     --- real case

         if ( itypvc==126 .and. kfun>0 ) then

!        --- Function in case of type 126 has not yet been implemented

            call errsub ( 1678, 0, 0, 0 )
            go to 1000

         end if
         if ( kdeg>nphys ) then

!        --- kdeg too large

            call errint ( kdeg, 1 )
            call errint ( nphys, 2 )
            call errsub ( 218, 2, 0, 0 )
            go to 1000

         end if

!        --- Actual call

         call er0064 ( usolt, ks(isol(kvec))%nusol, rincre, &
                       kfun, nphys, kdeg )

      else if ( kfun<0 ) then

!     --- complex case

         call er0073 ( usolt, ks(isol(kvec))%nusol, rincre, &
                       kfun )

      else

!     --- Other options have not yet been implemented

         call errsub ( 1678, 0, 0, 0 )

      end if
1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End creatv116'

      end if  ! ( debug )
      call erclos ( 'creatv116' )

      end subroutine creatv116
