      subroutine creatv116group ( inpcre, kfun, &
                                  rincre, kdeg, nphys, maxphys, mvec, &
                                  islold, jpstin, istartvc, kvec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 05-12-2012 Long integers
!        version  2.1  date 10-02-2012 use getnpelmelgrp
!        version  2.0  date 27-12-2011 New parameter list
!
!   copyright (c) 2009-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill vector of type 116 for specific element group
!     Not all possibilities have been programmed yet
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
!     vector_defined_per_element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpcre(*), kfun, kdeg, nphys, kvec, &
              maxphys, mvec, islold(*), jpstin, istartvc
      double precision rincre(*)

!     inpcre         i    Array containing the integer input
!                         See subroutine creatv01
!     islold         i    User input array in which the user puts information
!                         of all preceding solutions
!                         Integer array of length 5 x numold
!     istartvc       i    Starting position of sequence numbers of vectors
!                         in inpcre
!     jpstin         i    Position in inpcre (see creatv01)
!     kdeg           i    Indication which degrees of freedom must be filled
!                         Possibilities:
!                         0:    All degrees of freedom
!                         >0:   Only the KDEGth degree of freedom
!                         <0:   The -KDEG degrees of freedom stored in array
!                               IARDEG must be filled
!     kfun           i    Indication of how the degrees of freedom must be
!                         filled.
!                         Possibilities:
!                         0:  Not filled
!                         <0:  Constant value stored in RINCRE(-KFUN)
!                         >0:  Function subroutine
!     kvec           i    Loop variable over the vectors to be created
!     maxphys        i    Maximum number of unknowns per point
!     mvec           i    Number of old vectors to be used in FUNCVECT
!     nphys          i    Number of physical unknowns, or maximal degrees of
!                         freedom per point
!     rincre         i    Double precision array containing the values of the
!                         constant if KFUN<0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielgrp, i, ielem

!     i              Counting variable
!     ielem          Element number
!     ielgrp         element group number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CREATV116FUNC  Fill special array of type 116 with special function
!     CREATV116GRCON Fill vector of type 116 for specific element group
!                    Special case a constant is filled
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      67   ielgrp has wrong value
!     218   Degree of freedom does not exist
!    3020   kfun has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'creatv116group' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from creatv116group'
         write(irefwr,1) 'kfun, kdeg, nphys, maxphys', &
                          kfun, kdeg, nphys, maxphys
         write(irefwr,1) 'mvec,  jpstin, istartvc', &
                          mvec,  jpstin, istartvc
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- compute starting address of vector to be filled

      usolt => ks(islold(kvec))%sol

      ielem = 0
      ielgrp = inpcre(jpstin)
      if ( ielgrp<=0 .or. ielgrp>nelgrp ) then
         call errint ( ielgrp, 1 )
         call errint ( nelgrp, 2 )
         call errsub ( 67, 2, 0, 0 )
      end if
      if ( debug ) write(irefwr,1) 'ielgrp', ielgrp

      if ( kdeg>nphys ) then

!     --- kdeg too large

         call errint ( kdeg, 1 )
         call errint ( nphys, 2 )
         call errsub ( 218, 2, 0, 0 )

      end if

      if ( ierror/=0 ) go to 1000

!     --- compute inpelm and number of elements
!         compute the number of elements  inelem

      do i = 1, ielgrp-1
         inelem = sepgetinelem ( i )
         ielem = ielem+inelem
      end do

      inelem = sepgetinelem ( ielgrp )
      if ( debug ) write(irefwr,1) 'ielem, inelem', ielem, inelem

      if ( kfun<0 ) then

!     --- kfun<0, store constant value

         call creatv116grcon ( usolt, rincre, kdeg, kfun, &
                               nphys, ielem, inelem )

      else if ( kfun>3000 .and. kfun<4000 ) then

!     --- kfun>0, function depends on coordinates and old solutions through
!         subroutine funcvect
!         Compute starting addresses

         usoltemp => ks(islold(inpcre(istartvc)))%sol

!        --- Actual call

         call creatv116func ( usoltemp, ielem+1, &
                              ielem+inelem, maxphys, mvec, kfun, usolt )

      else

!     --- Other values of kfun have not yet been implemented

         call errint ( kfun, 1 )
         call errsub ( 3020, 1, 0, 0 )

      end if  ! ( kfun<0 )
      if ( ierror/=0 ) go to 1000

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End creatv116group'

      end if  ! ( debug )
      call erclos ( 'creatv116group' )

      end
