      subroutine creatv801 ( vector, nphys, kfill, kfun, &
                             kdeg, icompl, inpcre, rincre, jpstin, &
                             nelectrodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 27-12-2011 New parameter list
!        version  1.0  date 06-11-1999
!
!   copyright (c) 1999-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Fill arrays of type 801
!      Help subroutine for creatvbf
! **********************************************************************
!
!                       KEYWORDS
!
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletexts
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nphys, nelectrodes, kfill, kfun, &
              kdeg, icompl, inpcre(*), jpstin
      double precision vector(nphys,*), rincre(*)

!     icompl         i    Multiplication factor for complex numbers
!                         Real: icompl=1,   Complex: icompl=2
!     inpcre         i    The integer input read is stored in array INPCRE from
!                         position 1. Hence in the call INPCRE(ISTART), with
!                         ISTART the starting position must be used.
!                         INPCRE will be filled in correspondence with array
!                         INPCRE for subroutine CREATV
!     jpstin         i    Help variable to compute new value of ipstin partly
!     kdeg           i    Indication which degrees of freedom must be filled
!                         Possibilities:
!                         0:    All degrees of freedom
!                         >0:   Only the KDEGth degree of freedom
!                         <0:   The -KDEG degrees of freedom stored in array
!                               IARDEG must be filled
!     kfill          i    Indication how the nodal points must be filled into
!                         array NODES
!                         Possible values:
!                          0:  The nodes are filled in all points
!                          1:  The nodes are filled in n user points
!                          2:  The nodes are filled in the curves C1 to C2
!                          3:  The nodes are filled in the surfaces S1 to S2
!                          5:  The nodes are filled in the elements i to j
!                          6:  The nodes are filled in the element group ielgrp
!                          7:  The nodes are filled in the volumes V1 to V2
!                          8:  The nodes are filled in the user points P1 to P2
!                          9:  The nodes are filled in the active points of
!                              obstacle i
!                         10:  The nodes are filled in the active points of
!                              contact i
!                         11:  The nodes are filled in the non-active points
!                              of contact i
!                         12:  The nodes are filled using a file with node
!                              numbers
!                         13:  The nodes are filled using a file with elements
!                         14:  The nodes are filled using a file with capacities
!     kfun           i    Indication of how the degrees of freedom must be
!                         filled.
!                         Possibilities:
!                         0:  Not filled
!                         <0:  Constant value stored in RINCRE(-KFUN)
!                         >0:  Function subroutine
!     nelectrodes    i    Number of electrodes
!     nphys          i    Number of physical unknowns, or maximal degrees of
!                         freedom per point
!     rincre         i    Double precision array containing the values of the
!                         constant if KFUN<0
!     vector         i    In this array all preceding solutions are stored, i.e.
!                         all solutions that have been computed before and
!                         have been carried to system or deriva by the parameter
!                         islold in the parameter list of these main subroutines
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision consta
      integer ipos, i, k, nelem
      character *80 filename

!     consta         If true the number of unknowns per point is constant
!     filename       Name of file
!     i              Counting variable
!     ipos           Position in array
!     k              Counting variable
!     nelem          Number of elements in capacity vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     READFILE801    Read capacity numbers and values from file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     218   Degree of freedom does not exist
!    1675   Wrong value of reference to texts
!    1678   For vectors of special structure defined per element, only the
!           filling of a constant over a complete region has been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'creatv801' )
      nelem = (nelectrodes*(nelectrodes-1))/2
      if ( kfill==14 ) then

!     --- kfill = 14, read information from file

         ipos = inpcre(jpstin)
         if ( ipos<=0 .or. ipos>=nextxt ) then

!        --- ipos out of range

            call errint ( jpstin, 1 )
            call errint ( ipos, 2 )
            call errint ( nextxt, 3 )
            call errsub ( 1675, 3, 0, 0 )
            go to 1000

         end if
         filename = texts(ipos)
         call readfile801 ( filename, vector, nelectrodes, nphys )

      else if ( kfun<0 .and. icompl==0 ) then

!     --- real case

         if ( kdeg>nphys ) then

!        --- kdeg too large

            call errint ( kdeg, 1 )
            call errint ( nphys, 2 )
            call errsub ( 218, 2, 0, 0 )
            go to 1000

         end if

!        --- kfun<0, function is constant

         consta = rincre(-kfun)
         if ( kdeg==0 ) then

!        --- All degrees of freedom

            do i = 1, nelem
               do k = 1, nphys
                  vector(k,i) = consta
               end do
            end do

         else

!        --- One degree of freedom

            do i = 1, nelem
               vector(kdeg,i) = consta
            end do
         end if

      else

!     --- Other options have not yet been implemented

         call errsub ( 1678, 0, 0, 0 )

      end if
1000  call erclos ( 'creatv801' )
      end
