      subroutine elmgrinf ( larray, liarry, kmeshm, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.17 date 03-02-2015 extension for type 704
!        version  3.16 date 14-11-2014 Extension with 700
!        version  3.15 date 07-08-2013 change length 260
!
!   copyright (c) 1998-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill general information for a specific element group, depending
!     on the type number. Compute ielrou, larray and liarry
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer larray, liarry, kmeshm(*), ndim

!     kmeshm     i    KMESH part m, contains nodal point numbers of the curves
!     larray     o    length of array array in element subroutines
!     liarry     o    length of the integer array iarray
!     ndim       i    Dimension of the space.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nspec
      double precision help, rdim

!    help   Help variable to compute sqrt of inpelm
!    nspec  Number of points at the boundary of a spectral element
!    rdim   Help variable to transfer the ndim into real
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      43   Type number not available
!    1698   nspec and inpelm do not match
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elmgrinf' )
      larray=1
      select case ( itype )

         case (0)

!        --- Type 0, skip

         case (1:99)

!        --- Type 1-99, user elements

         case (502:505, 509, 510)

!        --- Type 100, 101, 104, 105, 108:110, 120, 124, 502:505, 509, 510
!            second order elliptic equation  ( Two-dimensional )

            larray = 235

         case (506, 507)

!        --- Type 113, 115, 506, 507
!            second order elliptic equation 3-d

            larray = 792*2

         case (150, 151, 154, 155, 158, 159, 170, 174)

!        --- Type 150, 151, 154, 155, 158, 159, 170, 174
!            Helmholtz equation (Two-dimensional)

            larray = 1

         case (152, 153)

!        --- Type 152, 153
!            complex boundary conditions for helmholtz equation )
!            (linear line element)

            larray = 26

         case (162)

!        --- Type 162
!            complex boundary conditions for helmholtz equation )

            larray = 50

         case (163, 165)

!        --- Type 163, 165
!            Helmholtz equation (Three-dimensional)

            larray = 792

         case (190)

!        --- Type 190

            larray = 30

         case (200:202)

!        --- Type 200, 201, 202: general nonlinear solid element

            larray = 69696

         case (203)

!        --- Type  203: 2-phase nonlinear solid element

            larray = 69696

         case (204, 205)

!        --- Type  204, 205: 4-phase nonlinear solid element

            larray = 71830

         case (250, 252)

!        --- Type 250, linear elasticity element

            larray = 36558

         case (251, 253)

!        --- Type 208, 209, 251, boundary conditions for linear elasticity
!                                element

            larray = 90

         case (210, 211, 212)

!        --- Type 210: boundary element for element 200-204
!            Type 211, 212: contact element for element 200

            larray = 13800

         case (220)

!        --- Type 220: simple beam element
!

            larray = 1

         case (230, 232)

!        --- Type 230,232: general VIp shell element

            larray = 1

         case (255)

!        --- Type 255: plate element

            larray = 304

         case (256)

!        --- Type 256: shell element

            larray = 420

         case (260, 261)

!        --- Type 260, 261, rubber element

            larray = 15000

         case (300)

!        --- Type 300
!            Reynolds equation (in)compressible 1D/2D

            larray = 126

         case (302)

!        --- Type 302
!            elasticty-flow interaction (incompressible bearing )
!            (linear line element)

            larray = 126

         case (303)

!        --- Type 303
!            elasticty-flow interaction (incompressible bearing )
!            point element

            larray = 1

         case (304)

!        --- Type 304

            larray = 1

         case (350)

!        --- Type 350
!            Elements for RTMSIM

            larray = 1

         case (400, 402, 404)

!        --- Type 400, 402, 404
!            navier-stokes (penalty equation) extended quadratic element

            larray = 392

         case (401, 403, 405, 407, 423, 424, 425, 437)

!        --- Type 401, 403, 405, 407, 423, 424, 425, 437
!            line element for navier-stokes quadratic element

            larray = 1

         case (406)

!        --- Type 406,  navier-stokes equation axi-symmetric with swirl

            larray = 800

         case (408)

!        --- Type 408  line element for navier-stokes  (Free boundary)
!                      quadratic element

            larray = 50

         case (410)

!        --- Type 410

            larray = 36387

         case (414:416)

!        --- Type 414:416

            larray = 1400+3*npointcac
            liarry=200+kmeshm(3)-kmeshm(2)

         case (420:422)

!        --- Type 420:422  boussinesq-stokes equation

            larray = 1600

         case (426:428, 438)

!        --- Type 426:428, 438
!            line element for navier-stokes  Temperature part quadratic element

            larray = 50

         case (429)

!        --- Type 429  navier-stokes (penalty equation) Temperature dependent
!                      including swirl extended quadratic element

            larray = 2054

         case (430, 431)

!        --- Type 430, 431 navier-stokes (penalty equation)  mixing length model
!                          extended quadratic element

            larray = 950+3*npointcac
            liarry=200+kmeshm(3)-kmeshm(2)

         case (433)

!        --- Type 433  navier-stokes (penalty equation)  mixing length model
!                      extended quadratic element; axi-symm with swirl

            larray = 1500+3*npointcac
            liarry = 200+kmeshm(3)-kmeshm(2)

         case (440:442)

!        --- Type 440:442  decoupled navier-stokes ((v-p)-method; penalty
!                          equation)  extended quadratic element

            larray = 1350+3*npointcac
            liarry = 200+kmeshm(3)-kmeshm(2)

         case (446:448)

!        --- Type 446:448  decoupled energy eq./convection-diff eq.
!                          extended quadratic element

            larray = 1100+3*npointcac
            liarry = 200+kmeshm(3)-kmeshm(2)

         case (450)

!        --- Type 450

            larray = 1100

         case (500)

!        --- Type 500 heat conduction problem with freezing front

            larray = 42

         case (501)

!        --- Type 501 mass matrix  ( linear triangle  )

            larray = 21

         case (600)

!        --- Type 600
!            second order elliptic equation General using spectral elements

            help = inpelm
            rdim = ndim
            nspec = int(help**(1d0/rdim)+.5d0)
            larray = (34+ndim+2*ndim**2)*inpelm+2*nspec+nspec*nspec

         case (601)

!        --- Type 601

            larray = 70*inpelm+2*inpelm*inpelm

         case (603)

!        --- Type 603
!            Spectral elements, boundary element for a scalar

            larray = 7*inpelm+2*inpelm*inpelm

         case (606)

!        --- Type 606
!            Navier-Stokes equation (2D) with spectral elements

            help = inpelm
            nspec = nint(sqrt(help))
            larray = 26*inpelm+4*nspec+4*inpelm**2

         case (607)

!        --- Type 607
!            Spectral elements, Convection-diffusion of a scalar; 3-D case

            help = inpelm
            rdim = ndim
            nspec = int(help**(1d0/rdim)+.5d0)
            if ( inpelm/=nspec**ndim ) then
               call errint ( nspec, 1 )
               call errint ( inpelm, 2 )
               call errsub ( 1698, 2, 0, 0 )
               go to 1000
            end if
            larray = (15+ndim)*inpelm+2*nspec+nspec*nspec

         case (608)

!        --- Type 608
!            Spectral elements, boundary element for a scalar; 3-D case

            help = inpelm
            nspec = nint(sqrt(help))
            larray = 7*inpelm+2*nspec

         case (609)

!        --- Type 609

            help = inpelm
            nspec = nint(sqrt(help))
            larray = 26*inpelm+4*nspec+4*inpelm**2

         case (612)

!        --- Type 612

            help = inpelm
            nspec = nint(sqrt(help))
            larray = 60*inpelm+8*nspec+11*inpelm**2

         case (615)

!        --- Type 615

            help = inpelm
            nspec = nint(sqrt(help))
            larray = 50*inpelm+4*nspec+6*inpelm**2

         case (650:652)

!        --- Type 650,651652 mixed element

            larray = 1

         case (665, 661)

!        --- Type 665, 661 mixed boundary element

            larray = 1

         case ( 700:705, 800, 803, 807, 810, 813, 815:816, 325)

!        --- Type 800 : General second order elliptic equation or
!                 803 : Special nonlinear second order elliptic equation
!                 325 : Reynolds equation (in)compressible 1D/2D

            larray = 12285

         case (801, 907)

!        --- Type 801 : Boundary conditions for general second order elliptic
!                       equation

            larray = 72

         case (802)

!        --- Type 802 : Delta function

            larray = 1

         case (804)

!        --- Type 804 : Boundary conditions for general second order elliptic
!                       equation discontinuous

            larray = 99

         case (805)

!        --- Type 805 : General second order elliptic equation with two unknowns
!                       per point

            larray = 8910

         case (806)

!        --- Type 806 : Boundary conditions for general second order elliptic
!                       equation with two unknowns per point

            larray = 144

         case (808)

!        --- Type 805 : General second order elliptic equation with two unknowns
!                       per point

            larray = 19000

         case (811, 812)

!        --- Type 811 : Compute psi at surface
!                 812 : Smoothing of surface

            larray = 1

         case (814)

!        --- Type 814 : minimal surface

            larray = 4374

         case (850)

!        --- Type 850 : Stochastic investment

            larray = 4401

         case (900:902, 904)

!        --- Type 900, 901, 902
!            laminar, isothermal flow, penalty method and integrated method

            larray = 36414

         case (903, 905)

!        --- Type 903  laminar, isothermal flow, integrated method
!                      Taylor Hood

            larray = 36414+729

         case (906)

!        --- Type 906  pressure matrix for Taylor Hood

            larray = 192

         case (910, 911)

!        --- Type 910, 911  laminar, isothermal flow, boundary conditions

            larray = 54

         case (912)

!        --- Type 912  Special line element for given mass flow

            larray = inpelm

         case (914)

!        --- Type 914  laminar, isothermal flow, discontinuous boundary
!                      conditions

            larray = 18+12*12

         case (915)

!        --- Type 915  special element for free surface computation
!                      restricted to die-swell, 2d, quadratic

            larray = 50

         case (916:918)

!        --- Type 916-918
!            laminar, isothermal flow, penalty method and integrated method
!            axi-symmetric and swirl

            larray = 36387

         case (919)

!        --- Type 919  laminar, isothermal flow, integrated method
!                      Taylor Hood
!            axi-symmetric and swirl

            larray = 36387

         case (922)

!        --- Type 922  laminar, isothermal flow, discontinuous boundary
!                      conditions

            larray = 1

         case (924, 928)

!        --- Type 924, 928  special fluid-structure coupling element

            larray = 1

         case (995, 996)

!        --- Type 995, 996    special elements to copy reaction force

            larray = 1

         case (997)

!        --- Type 997  special contact element

            larray = 1

         case (998)

!        --- Type 998  special friction element

            larray = 1

         case default

!        --- Type<0 or>999
!            These types are not available

            call errint ( itype, 1 )
            call errint ( ielgrp, 2 )
            call errsub ( 43, 2, 0, 0)

      end select  ! case ( itype )

1000  call erclos ( 'elmgrinf' )

      end
