      subroutine er0037 ( usol, ichoice, ielem1, &
                          ielem2, idegfd, value, irelnp, compl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.2  date 05-12-2012 Fortran 90
!        version  3.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  3.0  date 17-06-1987
!
!   copyright (c) 1986-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual filling of essential boundary conditions in the solution vector
!     for elements     one component only
!     this subroutine is called by bvalue
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical compl
      integer ichoice, ielem1, ielem2, irelnp, idegfd
      double precision usol(*), value

!     compl          i    indication whether the arithmetic is real (false)
!                         or complex (true)
!     ichoice        i    Choice parameter, see bvalue
!     idegfd         i    Degree of freedom sequence number, see bvalue
!     ielem1         i    First element number in surface
!     ielem2         i    Last element number in surface
!     irelnp         i    Relative nodal point number in elements
!     usol          i/o   Solution vector
!     value          i    see bvalue
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, ielgrp, ikelmc, nodenr, inpelm, j, jnelem, k, inusol

!     ielem          Element number
!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     inusol         Position in usol
!     j              Counting variable
!     jnelem         number of elements in an element group for one super
!                    element
!     k              Counting variable
!     nodenr         Nodal point number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPINIBVL      actual filling of essential boundary condition for one point
!                    and one degree of freedom only
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( ielem1<=0 .or. ielem1>nelem ) then
         call errint ( ielem1, 1 )
         call errint ( nelem, 2 )
         call errsub ( 446, 2, 0, 0 )
         go to 1000
      end if  ! ( ielem1<=0 .or. ielem1>nelem )
      if ( ielem2<ielem1 .or. ielem2>nelem ) then
         call errint ( ielem2, 1 )
         call errint ( ielem1, 2 )
         call errint ( nelem, 3 )
         call errsub ( 447, 3, 0, 0 )
         go to 1000
      end if  ! ( ielem2<ielem1 .or. ielem2>nelem )

      ikelmc=0
      inelem=0
      ielgrp=1
      jnelem=nelem
      if(nelgrp==1) goto 120
      do k=1,nelgrp
         ielgrp=k
         jnelem=kmeshg(k)
         inelem=inelem+jnelem
         if(inelem>ielem1) goto 110
         inpelm=Topgrp(k)%inpelm
         ikelmc=ikelmc+inpelm*jnelem
      end do
110   inelem=inelem-jnelem
120   ielem=ielem1
      do k=ielgrp,nelgrp
         inpelm=Topgrp(k)%inpelm
         jnelem=Topgrp(k)%inelem
         if(k==ielgrp) then
            ikelmc=ikelmc+(ielem1-inelem-1)*inpelm
            jnelem=jnelem-ielem1+1
         end if
         if(irelnp>inpelm) call ertrap('bvalue  (er0037) ',448,3, &
            ielgrp,irelnp,0)
         do j=1,jnelem
            nodenr=kmeshc(ikelmc+irelnp)
            ikelmc=ikelmc+inpelm
            inusol = nodenr
            if ( indprp>0 ) inusol = kprobp(inusol,idegfd)
            if ( inusol>0 ) &
               call sepinibvl ( ichoice, nodenr, usol, inusol, idegfd, &
                                value, compl )
            ielem=ielem+1
            if(ielem>ielem2) return
            end do
         end do
1000  end
