      subroutine er0038 ( usol, ichoice, inode1, &
                          inode2, idegfd, value, jstap, ielgrp, compl )
! ======================================================================
!
!        programmer   Guus Segal
!        version  2.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  2.0  date 17-06-1987
!
!   copyright (c) 1986,1987  "Ingenieursbureau sepra"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "ingenieursbureau sepra".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual filling of essential boundary conditions in the solution vector
!     for element groups   one component only
!     this subroutine is called by bvalue
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical compl
      double precision usol(*), value
      integer :: ichoice, inode1, inode2, idegfd, jstap, ielgrp

!     compl          i    indication whether the arithmetic is real (false)
!                         or complex (true)
!     ichoice        i    Choice parameter
!     idegfd         i    Degree of freedom sequence number
!     ielgrp         i    element group number
!     inode1         i    First node
!     inode2         i    Second node
!     jstap          i    Step
!     usol          i/o   Solution vector
!     value          i    see bvalue
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ikelmc, nodenr, j, k, inpelm, kstap, node1, node2, inusol

!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     inusol         Position in usol
!     j              Counting variable
!     k              Counting variable
!     kstap          Number of pairs in kstapl
!     node1          Node
!     node2          Node
!     nodenr         Nodal point number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPINIBVL      actual filling of essential boundary condition for one point
!                    and one degree of freedom only
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      ikelmc=0
      do k=1,ielgrp-1
         inelem=Topgrp(k)%inelem
         inpelm=Topgrp(k)%inpelm
         ikelmc=ikelmc+inpelm*inelem
      end do ! k=1,ielgrp-1
      inpelm=Topgrp(ielgrp)%inpelm
      inelem=Topgrp(ielgrp)%inelem
!
      if ( inode1<0 .or. inode1>inpelm ) then
         call errint ( inode1, 1 )
         call errint ( inpelm, 2 )
         call errsub ( 448, 2, 0, 0 )
         go to 1000
      end if  ! ( inode1<0 .or. inode1>inpelm )
      if(inode1==0) then
         kstap=1
         node1=1
         node2=inpelm
         goto 120
      else
         kstap=max(1,jstap)
         node1=inode1
         node2=inode2
      end if
      if ( inode2<inode1 .or. inode2>inpelm ) then
         call errint ( inode2, 1 )
         call errint ( inode1, 2 )
         call errint ( inpelm, 3 )
         call errsub ( 449, 3, 0, 0 )
         go to 1000
      end if  ! (inode2<inode1 .or. inode2>inpelm  )
      if ( jstap<=0 .or. jstap>inpelm ) then
         call errint ( jstap, 1 )
         call errint ( inpelm, 2 )
         call errsub ( 450, 2, 0, 0 )
         go to 1000
      end if  ! ( jstap<=0 .or. jstap>inpelm )
120   do j=1,inelem
         do k=node1,node2,kstap
            nodenr=kmeshc(ikelmc+k)
            inusol = nodenr
            if ( indprp>0 ) inusol = kprobp(inusol,idegfd)
            if ( inusol>0 ) &
               call sepinibvl ( ichoice, nodenr, usol, inusol, idegfd, &
                                value, compl )
            end do
            ikelmc=ikelmc+inpelm
         end do
1000  end
