      subroutine er0046 ( usol, ichois, jelgrp, compl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.5  date 24-07-2015 Replace to0000 by to00002
!        version  4.4  date 25-03-2011 New call to to0000
!        version  4.3  date 03-06-2006 New call to to0000
!
!   copyright (c) 1986-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual filling of essential boundary conditions in the solution vector
!     for element groups   one component only
!     this subroutine is called by bvalue
!     special case   |ichois|>1000
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     essential_boundary_condition
!     fill
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical compl
      integer :: ichois, jelgrp
      double precision usol(*)

!     compl   i    indication whether the arithmetic is real (false)
!                  or complex (true)
!     ichois  i    Choice parameter, see bvalue
!     jelgrp  i    element group number
!     usol   i/o   Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ikelmc, k, jchois
      integer, allocatable :: index1(:), index2(:)

!     ikelmc      position in kmesh part c
!     index1      work array index1  for subroutine cfun1b or func1b
!     index2      work array index2  for subroutine cfun1b or func1b
!     jchois      ?
!     k           Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     CFUN1B         User subroutine to fill boundary conditions
!     ERALLOC        Produce error message in case allocate went wrong
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     FUNC1B         User subroutine to fill boundary conditions
!     TO00002        Fill arrays index1 and index2
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      ikelmc=0
      do k=1,jelgrp-1
         inelem=Topgrp(k)%inelem
         inpelm=Topgrp(k)%inpelm
         ikelmc=ikelmc+inpelm*inelem
      end do
      inpelm=Topgrp(ielgrp)%inpelm
      inelem=Topgrp(ielgrp)%inelem

      allocate ( index1(inpelm), index2(inpelm*nunkp), stat = error )
      if ( error/=0 ) call eralloc ( error, inpelm*nunkp, 'index1' )
      if ( ierror/=0 ) go to 1000

      if(nunkp==1) then
         if(indprf==0) jchois=1
         if(indprf>0) jchois=2
      else
         if(indprf==0) jchois=3
         if(indprf>0) jchois=4
      end if
      do ielem=1,inelem
         call to00002 ( jchois, kmeshc, ikelmc, &
                        index1, index2, nelgrp, 0, 1, 1, 0 )
         ikelmc = ikelmc + inpelm
         if ( compl ) then
             call cfun1b ( -ichois, index1, index2, usol, coor )
         else
             call func1b ( ichois, index1, index2, usol, coor )
         end if
      end do

      deallocate ( index1, index2, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'index1' )

1000  end
