      subroutine ercrea129i ( ivecint, nunkploc, leng1 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.1  date 29-06-2011 Correction periodical bc's
!        version  1.0  date 17-01-2001
!
!   copyright (c) 2001-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill integer part of a vector of type 129
!
! **********************************************************************
!
!                       KEYWORDS
!
!     fill
!     vector
!     vector_defined_per_element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivecint(4,*), nunkploc, leng1

!     ivecint        o    Integer part of a vector of structure 129
!     leng1          o    Number of reals or complex quantities stored in
!                         vector of special structure 129
!     nunkploc       i    Number of physical unknowns, or maximal degrees of
!                         freedom per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELDGRINT       determine number of integration points for element routines
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     initialize parameters
!     leng1 := 0
!     for ielgrp := 1 (1) nelgrp do
!        compute number of integration points m
!        raise leng1
!        fill part of ivecint
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercrea129i' )

!     --- initialize parameters

      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      kprobc => kprobpart%kprobc

!     --- leng1 := 0

      leng1 = 0

!     --- for ielgrp := 1 (1) nelgrp do

      do ielgrp = 1, nelgrp

!     --- compute number of integration points m

         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         ishape = kmeshf(ielgrp)
         itype = kprobc(ielgrp)
         if ( itype<=0) then

!        --- Empty or periodical element

            m = 0

         else

!        --- Standard element

            call eldgrint

         end if  ! ( itype==-1)

!        --- fill part of ivecint

         ivecint(1,ielgrp) = m
         ivecint(2,ielgrp) = nunkploc
         ivecint(3,ielgrp) = inelem
         ivecint(4,ielgrp) = leng1

!        --- raise leng1

         leng1 = leng1+m*nunkploc*inelem

      end do
      call erclos ( 'ercrea129i' )
      end
