      subroutine ercreatstand4 ( nnodes, inpcre, ipstin, kdeg, kvec, isolut, &
                                 islold, ivecold )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 04-03-2011 Correction ipkprfold
!        version  1.1  date 05-12-2012 Long integers
!        version  1.0  date 09-01-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill one vector for one option only
!     kfun>4000, vector
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: nnodes, inpcre(*), ipstin, &
                 kdeg, kvec, isolut(*), islold(*), ivecold

!     inpcre         i    Array containing the integer input
!                         See subroutine creatv01
!     ipstin         i    Starting address in array INPCRE
!     islold         i    User input array in which the user puts information
!                         of all preceding solutionss
!                         Integer array of length 5 x numold
!     isolut        i/o   Standard SEPRAN array containing information of
!                         the solution vectors (same as isol)
!     ivecold        i    ivec for old vector
!     kdeg           i    Indication which degrees of freedom must be filled
!                         Possibilities:
!                         0:    All degrees of freedom
!                         >0:   Only the KDEGth degree of freedom
!                         <0:   The -KDEG degrees of freedom stored in array
!                               IARDEG must be filled
!     kvec           i    Loop variable over the vectors to be created
!     nnodes         i    Number of nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer indprfsav, indprpsav, nphyssav

!     indprfsav      Help parameter to save the value of indprf
!     indprpsav      Help parameter to save the value of indprp
!     nphyssav       Help parameter to save the value of nphys
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERFILLCOPVEC   Copy the contents of array uold into array unew for
!                    all nodes stored in array nodes
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercreatstand4' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercreatstand4'
         write(irefwr,1) 'ivecold', ivecold
         call prinin ( islold(ivecold), 5, 'isol' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      indprfsav = indprfi
      indprpsav = indprpi
      nphyssav = nphysi

!     --- Activate array and corresponding arrays

      call sepactsolbf2 ( islold(ivecold) )

!     --- Get starting addresses

      usolt => ks(islold(ivecold))%sol
      usoltemp => ks(isolut(kvec))%sol

      call erfillcopvec ( nodes, kdeg, inpcre(ipstin+2), &
                          usolt, usoltemp, &
                          indprfi, indprf, kprobfi, &
                          kprobf, nphysi, nphys, nnodes )
      indprfi = indprfsav
      indprpi = indprpsav
      nphysi = nphyssav

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End ercreatstand4'

      end if  ! ( debug )
      call erclos ( 'ercreatstand4' )

      end
