      subroutine ercreatsub7 ( inpcre, ipstin, jpstin, kfill, n )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 03-04-2012 Extension with series of surfaces
!        version  1.0  date 29-12-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Raise ipstin as function of kfill and jpstin
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     create
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpcre(*), ipstin, jpstin, kfill, n

!     inpcre         i    Array containing the integer input
!                         See subroutine creatv01
!     ipstin        i/o   Starting address in array INPCRE
!     jpstin         i    Help variable to compute new value of ipstin partly
!     kfill          i    Indication how the nodal points must be filled into
!                         array NODES
!                         See subroutine prfillnodes
!     n              i    number of commands
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ercreatsub7' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ercreatsub7'
         write(irefwr,1) 'ipstin, kfill', ipstin, kfill
         call prinin ( inpcre(ipstin), 6, 'inpcre' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- raise ipstin

      ipstin = jpstin - 1

!     --- i+4

      if ( kfill==1 ) then
         ipstin = ipstin + 1 + n

!        --- i+6

         ipstin = ipstin - 1

!        --- i+5, 5 items per function when n=1, are other cases OK ?

      else if ( kfill==2 .or.kfill==4 .or.kfill==8 ) then
         ipstin = ipstin + 3
      else if ( kfill==3 .or.kfill==5 .or.kfill==7 ) then
         ipstin = ipstin + 3 + inpcre(ipstin+3)
      else if ( kfill==6 ) then
         ipstin = ipstin + 2 + inpcre(ipstin+2)
      else if ( kfill==22 ) then
         ipstin = ipstin + 2 + inpcre(ipstin+2)
      else if ( kfill==24 .or. kfill==25 ) then

!     --- Special case of set of surfaces or volumes

         ipstin = ipstin + 2 + inpcre(ipstin+1) + inpcre(ipstin+2)

      else if ( kfill>=9 ) then
         ipstin = ipstin + 1
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ipstin', ipstin
         write(irefwr,*) 'End ercreatsub7'

      end if  ! ( debug )
      call erclos ( 'ercreatsub7' )

      end
