      subroutine erfillrad ( nnodes, nodes, coor, origin, radius, &
                             numpoints, curvenodes, isurnr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 06-04-2005 Extra debug information
!        version  2.1  date 11-02-2005 New call to mshchplane
!        version  2.0  date 15-10-2004 Extension with part of circle
!        version  1.0  date 17-03-2004
!
!   copyright (c) 2004-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute centroid and radius of circle in 3d
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     centre
!     radius
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnodes, nodes(*), numpoints, curvenodes(*), isurnr
      double precision coor(3,*), origin(3), radius

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     curvenodes     i    Contains the nodes of the boundary curve
!     isurnr         i    Surface sequence number
!     nnodes         i    Number of nodes in nodes
!     nodes          i    Contains the node numbers that must be filled
!     numpoints      i    Number of points in icurnr
!     origin         o    Contains the coordinates of the origin of the circle
!     radius         o    Radius of sphere or circle
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision normal(3)
      integer indcol(3)
      logical planefound

!     indcol         Array containing the column index for subroutine
!                    PRINRL1_IND (in this case it is just 1, 2, 3)
!     normal         Contains normal on surface isurnr
!     planefound     Input parameter for subroutine mshchplane
!                    false means that it is checked if coordinates are in a
!                    plane. If not an error message is given.
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHCENTRECIRC  Compute centroid of (part of a) circle as well as radius
!     MSHCHPLANE     Check if all nodes in surface are lying in a plane
!     MSHNORMALPLANE Compute the normal on a set of coordinates in a plane
!     PRININ         print 1d integer array
!     PRINRL1_IND    Print parts of a full element matrix by indicating rows and
!                    columns
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erfillrad' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erfillrad'
110      format ( a, 1x, (5d12.4) )
         call prinin ( nodes, nnodes, 'nodes' )
         call prinin ( curvenodes, numpoints, 'boundary nodes' )
         indcol(1) = 1
         indcol(2) = 2
         indcol(3) = 3
         call prinrl1_ind ( coor, 3, nnodes, nodes, indcol, nnodes, 3, &
                            'coor' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Check if all nodes are in a plane
!         First compute normal on plane

      call mshnormalplane ( nodes, nnodes, coor, normal, isurnr )

!     --- Then check

      planefound = .false.
      call mshchplane ( nnodes, nodes, coor, isurnr, normal, &
                        planefound )
      if ( ierror/=0 ) go to 1000

!     --- (o_x,o_y,o_z) :=  centroid of circle
!         Radius := radius of circle
!         Check if region is a circle

      call mshcentrecirc ( normal, coor, curvenodes, numpoints, &
                           origin, radius, isurnr )

1000  call erclos ( 'erfillrad' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,110) 'radius', radius
         write(irefwr,110) 'origin', origin
         write(irefwr,*) 'End erfillrad'

      end if

      end
