      subroutine ervl12 ( iseqlc, n, multnr, w, y, &
                          stiff, amas, eigvec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.1  date 04-07-1999 New type of normalization
!        version  5.0  date 27-11-1992 Removal of intmas, new call to ervl14
!        version  4.0  date 12-09-1992 Complete revision
!
!   copyright (c) 1986-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Reorder and normalize the eigenvectors of the original matrix
!    Help subroutine for EIGVAL
! **********************************************************************
!
!                       KEYWORDS
!
!     eigenvalue
!     eigenvector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      use sepmodulematstr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqlc, n, multnr
      double precision w(multnr), y(n,multnr), amas(*), eigvec(*)
      type (rmatrix) :: stiff

!     amas           i  mass matrix
!     eigvec         o  eigenvector
!     iseqlc         i  sequence number of eigenvector
!     multnr         i  Number of eigenvalues that have been computed, &
!                       including double ones
!     n              i  number of degrees of freedom
!     stiff          i  stiffness matrix
!     w              i  Array of length multnr containing the computed
!                       eigenvalues including double ones
!     y                 Array of length n x multnr containing the multnr
!                       unreordered eigenvectors of the matrix.
!                       Output of subroutine ervl13
!                       INPUT / OUTPUT PARAMETERS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nboundall
      double precision :: work(n,2), zn

!     i              Counting variable
!     nboundall      nbound+npboun
!     work           Work array of length n x 2
!     zn
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision dnrm2
      integer idamax

!     DAXPY          Overwrite double precision dy with double precision
!                    da*dx + dy
!     DCOPY          Copy double precision dx to double precision dy
!     DNRM2          Blas routine to compute the 2-norm of a vector
!     DSCAL          Blas routine to scale a vector
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERVL11         solve   sqrt(m)^(-1) s sqrt(m)^(-1)    a  =  b
!     ERVL14         compute eigenvector of (generalized) eigenvalue problem
!     IDAMAX         Blas subroutine defining the index of the maximum
!                    element in an array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ervl12' )

      nboundall = nbound+npboun

!     --- Set boundary conditions equal to zero

      do i = 1,nboundall
         eigvec(i+n) = 0d0
      end do
      if ( w(iseqlc).le.1d-9 * w(multnr) ) then

!     --- Eigenvalue is zero, set eigenvector constant, except for the
!         boundary conditions

         do i = 1,n
            eigvec(i) = 1d0
         end do

      else

!     --- shifting vector to work and normalizing

         call dcopy(n,y(1,iseqlc),1,work(1,1),1)
         zn=dnrm2(n,work(1,1),1)
         call dscal(n,1d0/zn,work(1,1),1)
         call dcopy(n,work(1,1),1,y(1,iseqlc),1)
         call ervl11(work(1,1),eigvec,n,stiff,amas, &
                     work(1,1))
         call dcopy(n,y(1,iseqlc),1,work(1,1),1)
         call daxpy(n,-w(iseqlc),work(1,1),1,eigvec,1)
         call dcopy(n,work(1,1),1,work(1,2),1)

!        --- call for backtransf. x=l(-t)*y  and fill in of zeros in b.c.

         call ervl14(work(1,2),eigvec,amas,n,nboundall)
         i = idamax ( n, eigvec,1)
         zn=1d0/eigvec(i)
         call dscal(n,zn,eigvec,1)

      end if
      call erclos ( 'ervl12' )

      end subroutine ervl12
