      subroutine ervl23 ( alpha, beta, j, x, del, ni )
! ======================================================================
!
!        programmer    Zdenek Vlasek
!        version  1.1  date 05-12-2012 Fortran 90
!        version  1.0  date 19-02-1991
!
!   copyright (c) 1991-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     evaluate functions  delta(j,x), ni(j,x),  see P+R  (3.1), (3.2), &
!                   using old values  x,  delta(j-1,x),  ni(j-1,x)
!     to be called from ervl20 and ervl22 only
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer j, ni
      double precision alpha(j), beta(j), x, del

!     alpha          i    alpha(1..j)
!     beta           i    beta(1..j-1)
!     del           i/o
!     j              i    >=1
!     ni            i/o
!                         variable i
!     x              i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INSTOP         Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1403   An internal error in SEPRAN, in the eigenvalue package ervl20
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( j<2 ) then
        call errint ( 11, 1 )
        call errsub ( 1403, 1, 0, 0 )
        call instop
      end if
      if ( abs(del)<=0d0 ) then
        call errint ( 12, 1 )
        call errsub ( 1403, 1, 0, 0 )
        call instop
      end if

!     --- ... notice:  abs(ni(x)) is equal to " 1 + original ni(x) "

      del = alpha(j) - x - beta(j-1)**2 / del

      if ( del<0d0 ) then

!     --- ... change ni

        if ( ni>=0 ) then
          ni = ni + 1
        else

!       --- ... preserve the minus sign

          ni = ni - 1
        end if
      end if

      end
