      subroutine ervl24 ( nnzer, tol, alpha, beta, j, &
                          zer, anorm, erbnd, deb, ier )
! ======================================================================
!
!        programmer    Zdenek Vlasek
!        version  1.1  date 05-12-2012 Long integers
!        version  1.0  date 19-02-1991
!
!   copyright (c) 1986-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   to be called from ervl20 only
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nnzer, j, ier
      double precision tol, alpha(j), beta(j), zer, anorm, erbnd
      logical deb

!     alpha      i    see subr ervl20
!     anorm      i    estimation of norm2(A)
!     beta       i    see subr ervl20
!     deb        i    logical, it allows a debug prints
!     erbnd      o    the errorbound
!     ier        o    ier = 0,  normal return
!                     ier = 1,  value  tol  is greater then
!                               the roundoff error estimation, &
!                               no result in  erbnd
!     j          i    it should be  j>=3
!     nnzer      i    approximate number of nonzero elements in the matrix A
!     tol        i    see subr ervl20
!     zer        i    the point for which the errorbound should be calculated, &
!                     it is close to a  Ritz(j)  value
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k
      double precision sj, bet, w, w1, w2, w3, vjg, znorm, &
                       z, z1, bz, sumw, sumz, abswk, psi, erb
      logical left, right

!     abswk
!     bet
!     bz
!     erb
!     i          Counting variable
!     k          Counting variable
!     left
!     psi
!     right
!     sj
!     sumw
!     sumz
!     vjg
!     w
!     w1
!     w2
!     w3
!     z
!     z1
!     znorm
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     INSTOP    Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1403   An internal error in SEPRAN, in the eigenvalue package ervl20
! **********************************************************************
!
!                       PSEUDO CODE
!
!    errorbound calculation, method: P+R, section 4
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( j<=2 ) then
        call errint ( 13, 1 )
        call errsub ( 1403, 1, 0, 0 )
        call instop
      end if

      ier = 0

!     --- at present: instead of the P+R's
!                            beta = norm2 ( abs(A) ) / norm2 (A)
!                        a voluntary constant  beta = 2  is used here

      bet = 2d0

!     --- initialize  erbnd,  k,  w2=w(k),  w1=w(k+1), &
!            sumw  from  1  to  k-1,  left,  right,  abswk

      erbnd = 1d30
      k = 1

!     --- P+R (4.12a),  w(1)

      w2 = 1d0

!     --- (4.12b),  w(2)

      w1 = - ( alpha(1) - zer ) / beta(1)
      sumw = 0d0
      left = .true.
      right = abs(w2)>abs(w1)
      abswk = 0d0

!     --- test for local maximum of abs(w)

      if ( right ) then

!     --- local maximum in  k=1
!         it will be  beta(k-1)=0  in expression (4.15) for  psi

        goto 30
      end if

!------ forward recurrence, it can be restarted by  goto 10

   10 continue

!     --- P+R (4.12c)

        do 20 i = k+2, j

!       --- w2 is  w(i-2),  w1 is w(i-1),  set  w = w(i)

          w = ( - ( alpha(i-1) - zer ) * w1 - beta(i-2) * w2 ) &
               / beta(i-1)

!         --- add  w(i-2)

          sumw = sumw + w2**2

!         --- update  left,  right

          left = .not. right
          right = abs(w1)>abs(w)

!         --- store  w(i-2)  in  w3  for  psi

          w3 = w2

!         --- prepare  w2,  w1  for next turn
!                ( or for restarting this loop )

          w2 = w1
          w1 = w

!         --- now  w2 is w(i-1),  w1 is w(i)
!             test for local maximum of abs(w) in i-1

          if ( left .and. right ) then

!         --- local maximum in  new k = i-1
!                test whether it is higher then  abswk

            if ( abs(w2)>abswk ) then

!           --- new k  is  i-1,  leave this loop

              k = i - 1
              goto 30
            end if
          end if
   20   continue

!       --- here  w3 is w(j-2),  w2 is w(j-1),  w1 is w(j)
!                    sumw  is from  1  to  j-2
!                    left  is abs(w(j-2))<abs(w(j-1))
!           last local maximum can be for  k = j-1  (if w(j) ignored), &

        if ( abs(w2)<=abswk ) then

!       --- no maximum here, return with the last  erbnd

          if ( abswk<=0d0 ) then
            call errint ( 14, 1 )
            call errsub ( 1403, 1, 0, 0 )
            call instop
          end if
          goto 5000
        end if

!       --- local maximum for  k = j-1

         k = j - 1

   30   continue

!------ reverse recurrence,  it is  k<j

!       --- use only  k, w2, w1, sumw
!           w2 is w(k),  w1 is w(k+1),  sumw is from  1  to  k-1
!           w3 is w(k-1), if only k>=2
!           P+R (4.13a),  z(j)

        z1 = 1d0

!       --- P+R (4.13b),  z(j-1)

        z = - ( alpha(j) - zer ) / beta(j-1)

!       --- sumz  form  j-1  to  j

        sumz = 1d0 + z**2
        do 40 i = j-1, k+1, -1

!       --- z is z(i),  z1 is z(i+1), &
!              set  bz =  beta(i-1)*z(i-1)

          bz = - ( alpha(i) - zer ) * z - beta(i) * z1

!         --- prepare  z,  z1  for next turn

          z1 = z
          z = bz / beta(i-1)

!         --- now  z is z(i-1),  z1 is z(i)
!             add  z(i-1)

          sumz = sumz + z**2
   40   continue

!       --- sumz  is now from  k  to  j
!           z is z(k),  z1 is  z(k+1)

!------ error bound

!           add  z(1..k-1),  use (4.14)

        sumz = sumw * ( z / w2 ) ** 2 + sumz
        znorm = sqrt (sumz )

!       --- P+R (4.15)

        if ( k==1 ) then

!       --- first term is zero

          psi = 0d0
        else

!       --- k>=2,  z(k-1)  is hidden in  w(k-1), see (4.14)

          psi = beta(k-1) *  ( w3 * ( z / w2 ) )
        end if

        psi = psi + (alpha(k)-zer) * z + beta(k) * z1

!       --- P+R (4.18)

        vjg = abs (psi) / znorm

!       --- sj  is the j-th component of  s(*) = z(*)/znorm, z(j)=1d0

        sj = 1d0 / znorm

!       --- candidate for  erbnd  for this  k

        erb = 1.25d0 * ( 7d0  + nnzer * bet ) &
                   * sqrt(dble(j)) * epsmac * anorm
        if ( erb>tol ) then

!       --- round-off error estimation is greater then  tol

          ier = 1
          goto 5000
        end if

        erb = erb + 1.25d0 *  abs( beta(j) ) * sj
        erbnd = min ( erbnd, erb )

!       --- P+R (4.19), was this  k  the last one ?

        if ( vjg<0.2d0 * erb ) then

!       --- vjg  is small with respect to reziduum and roundoff, &
!              no chance to improve  erbnd  significantly for other  k

          goto 5000
        end if

!       --- search for next  k  with 10 times higher abswk

        abswk = 10 * abs(w2)

!       --- restart the forward recurrence loop from  i = k+2, &
!              because during the last turn it was  i = k+1  there

        if ( deb ) write ( irefwr, * ) '    restart search for  k'
      goto 10

 5000 continue
      end
