      subroutine presidual ( matr, isol, irhsdlc, iresid, itrans )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 21-09-2001 Replace loctrn by loctrnbf
!        version  2.0  date 17-08-2000 New parameter list
!        version  1.0  date 03-07-2000
!
!   copyright (c) 2000-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute residual of system of linear equations given a solution vector
!
! **********************************************************************
!
!                       KEYWORDS
!
!     residual
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer matr, isol, iresid, irhsdlc, itrans

!     iresid         o    Array containing information about the residual vector
!     irhsdlc          i    Standard SEPRAN array, containing information of the
!                         large vector
!     isol           i    Standard SEPRAN array containing information of the
!                         solution array
!     itrans         i    Indicates if transformation from standard vector (i.e.
!                         usual degrees of freedom, for example Cartesian) to
!                         transformed vector (i.e. with transformed degrees of
!                         freedom in points with local transform, for example
!                         normal and tangential components) is defined (1) or
!                         that the backwards transformation to standard vector
!                         must be computed (2)
!     matr           i    Standard SEPRAN array, containing information of the
!                         large matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinvec(3), idummy(5)
      double precision rinvec(2)

!     idummy         Dummy parameter
!     iinvec         User input array for subroutine MANVEC
!     rinvec         Real user input array for subroutine MANVEC
!                    The contents depend on iinvec
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPLOCTRN      Compute (back-)transformation in case of local transform
!     SEPMANVEC      Vector manipulation subroutine
!     SEPMAVER       perform matrix vector multiplication
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'presidual' )

!     --- First perform local (back) transformation on solution

      if ( itrans>0 ) call seploctrn ( itrans, isol )

!     --- Next res := Ax

      call sepmaver ( matr, isol, iresid, 4 )

!     --- (Back) transformation on solution

      if ( itrans>0 ) call seploctrn ( 3-itrans, isol )

!     --- Next res := b - Ax

      iinvec(1) = 3
      iinvec(2) = 27
      iinvec(3) = 0
      rinvec(1) = 1d0
      rinvec(2) = -1d0

      idummy(2) = 100
      call sepmanvec ( iinvec, rinvec, irhsdlc, iresid, iresid )

!     --- (Back) transformation on residual

      if ( itrans>0 ) call seploctrn ( 3-itrans, iresid )

      call erclos ( 'presidual' )
      end
