      subroutine prreducenodes ( idegfd, nodes, nnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-01-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Reduce set of nodes to those nodes where the unknown is present
!
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     reduction
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: idegfd
      integer, intent(inout) :: nodes(*), nnodes

!     idegfd         i    Degree of freedom sequence number
!     nnodes        i/o   Number of nodes in array nodes
!     nodes         i/o   Contains the node numbers
!                         At input set of nodes to be reduced
!                         At output reduced set
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer ::text

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRREDUCENODES1 Reduce set of nodes to those nodes where the unknown
!                    is present (use kprobp)
!     PRREDUCENODES2 Reduce set of nodes to those nodes where the unknown
!                    is present (use kprobf)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prreducenodes' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prreducenodes'
         write(irefwr,1) 'idegfd, indprpo, nnodes', &
                          idegfd, indprpo, nnodes
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( indprpo>0 ) then

!     --- indprpo>0, reduce set of nodes

         call prreducenodes1 ( nodes, nnodes, max(1,idegfd) )

      else

!     --- indprfo>0 reduce set of nodes

         call prreducenodes2 ( nodes, nnodes, max(1,idegfd) )

      end if  ! ( indprpo>0 )

1000  call erclos ( 'prreducenodes' )
      if ( debug ) then

!     --- Debug information

         call prinin ( nodes, nnodes, 'nodes' )
         write(irefwr,*) 'End prreducenodes'

      end if  ! ( debug )

      end subroutine prreducenodes
