      subroutine prsdia ( amat, rhsd, usol, nusol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 26-10-2011 Jump in case of error
!        version  3.0  date 03-01-1999 Extra parameter iprint
!        version  2.0  date 12-01-1998 Extra parameters nusol, nbound, kprobh
!                                      Unknowns are not longer renumbered
!
!   copyright (c) 1986-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    solve system of equations amat u = f, where amat is a
!    diagonal matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     diagonal_matrix
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nusol
      double precision amat(*), rhsd(*), usol(*)

!     amat       i    matrix to be inverted
!     nusol      i    Total number of unknowns
!     rhsd       i    Right-hand-side vector
!     usol       o    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ih, imin, imax
      double precision a, amin, amax, rcond

!     a          Help variable to store quantity
!     amax       Maximum value in solution array
!     amin       Minimum value in solution array
!     i          Counting variable
!     ih         Help variable to store a constant temporarily
!     imax       Index for which maximum value is reached
!     imin       Index for which minimum value is reached
!     rcond      Inverse of condition of matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   1535   matrix singular
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsdia' )

      if ( ierror/=0 ) go to 1000
      do i = 1, nusol
         if ( abs(amat(i))<=0d0 ) then
            call errint ( i, 1 )
            call errsub ( 1535, 1, 0, 0 )
            go to 1000
         end if
         usol(i) = rhsd(i) / amat(i)

      end do
1000  call erclos ( 'prsdia' )

      end
