      subroutine prsfullgg ( amat, usol, rhsd, nrow )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-10-1999
!
!   copyright (c) 1999-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     solve system of linear equations by gaussian elimination
!     where the matrix has been decomposed by subroutine prfullgg
!     This implies that the matrix is supposed to be a real symmetric
!     positive definite full matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     positive_definite
!     real
!     symmetric
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nrow
      double precision amat(nrow,nrow), usol(nrow), rhsd(nrow)

!     amat           i    in this array the parts of the matrix to be decomposed
!                         are stored
!     nrow           i    Number of rows of the matrix
!     rhsd           i    Right-hand-side vector
!     usol           o    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k
      double precision sum

!     i              Counting variable
!     k              Counting variable
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!      The G G^T  decomposition of the matrix A has been computed by
!      subroutine PRFULLGG
!      Now the system of equations:   A x = f must be solved, hence:
!         G G^T  x = f   or:     G^T x = y
!                                G   y = f
!      First:  G y = f
!      fi  =  sum (k = 1 to i )  gik yk
!      yi = ( fi - sum (k=1 to i-1) gik yk ) / gii
!      Next:  G^T x = y
!      yi  =  sum (k = i to n )  g^Tik xk or
!      yi  =  sum (k = i to n )  gki xk
!      xi = ( yi - sum (k=i+1 to n) gki xk ) / gii
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- First forward substitution:
!         yi = ( fi - sum (k=1 to i-1) gik yk ) / gii

      do i = 1, nrow
         sum = 0d0
         do k = 1, i-1
            sum = sum+amat(i,k)*usol(k)
         end do
         usol(i) = (rhsd(i)-sum)/amat(i,i)
      end do

!     --- Next:  backward substitution
!         xi = ( yi - sum (k=i+1 to n) gki xk ) / gii

      do i = nrow, 1, -1
         sum = 0d0
         do k = i+1, nrow
            sum = sum+amat(k,i)*usol(k)
         end do
         usol(i) = (usol(i)-sum)/amat(i,i)
      end do

      end
