      subroutine sepadapinpsol ( ieqn, first )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 18-10-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt array inpsoltim
!     Fill array inpsol from inpsoltim
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
!     input
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulesolve
      use sepmodulematr
      use sepmoduletime
      use sepmodulematstr
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ieqn
      logical, intent(in) :: first

!     first          i    If true it concerns the first time step
!     ieqn           i    Equation sequence number
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCOPINPSOL   Copy arrays inpsl1 and rinsl1 into inpsol and rinsol
!                    into sepmodulesolve
!     SEPPETSCMAT    Transform sepran matrix into petsc matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepadapinpsol' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepadapinpsol'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(realmt(iseqmt)%intmatref)

!     --- Adapt inpsoltim

      if ( inpsoltim(3)==0 ) inpsoltim(3) = 1
      inpsoltim(9)  = 1

      if ( first ) then

!     --- First call, set some parameters in inpsoltim

         if ( (.not. IM%direct .or. IM%nested) .and. &
               Teq(ieqn)%imasstot/=0 ) inpsoltim(10) = 1

      else if ( associated(realmt(iseqmt)%matinv) ) then

!     --- next call, reuse preconditioning if possible

         if ( Teq(ieqn)%imasstot/=0 ) inpsoltim(10) = 2

      end if

      iseqsol = ieqn
      iseqreac = 0
      iseqintmat = 1
      keepmat = 0
      iseqsolut = Teq(1)%iseqfr

!     --- Initialize inpsol/rinsol and copy inpsoltim/rinsoltim

      call sepcopinpsol ( inpsoltim, rinsoltim )

      if ( IM%petsc .and. .not. IM%nosepmat ) then

!     --- Store sepran matrix into petsc matrix
!         in case of jmethod = 56, this is done in build

         call seppetscmat
         if ( ierror/=0 ) go to 1000

      end if  ! ( IM%petsc .and. .not. IM%nosepmat )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepadapinpsol'

      end if  ! ( debug )
      call erclos ( 'sepadapinpsol' )

      end subroutine sepadapinpsol
