      subroutine sepassemstart
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-02-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Start of assembly process
!     Make all initializations, fill modules and declare arrays
!     If changed, also change sepreacstart1
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     matrix
!     right_hand_side
!     assembly
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulemain
      use sepmoduleintmat
      use sepmodulesmf
      use sepmodulebuild
      use sepmoduleelm
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     TO0052         Compute starting addresses of all permanent arrays that
!                    are used in subroutine TO0050
!     TO0066         Compute length of temporary arrays for subroutine TO0050
!                    and reserve space.
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepassemstart' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepassemstart'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%nuser = 0
      E%nelem = nelem
      E%nelgrp = nelgrp
      E%npoint = npoint
      E%imas = imas

!     --- initialize some parameters (used by several subroutines)

      skipbounels = iinbld(6)==-1
      skipinnels = iinbld(6)==-2
      matexi = addmat .or. clrmat
      if ( .not. matexi ) matexi = stiffmat(1)>0
      symm = .true.

!     --- Reorder right-hand sides and start create petsc matrix (jmethod=56)

      call to0052 ( SMF%irhsd )

!     --- Compute starting positions of temporary arrays

      call to0066 ( SMF%islold )

      if ( iinbld(2)==6 ) then

!     --- Special case of multiplication of boundary element
!         matrices by solution vector
!         Loop over boundary elements only

         matrix = .true.
         vector = .false.

      end if  ! ( iinbld(2)==6 )

      if ( nintprop==0 .and. nrealprop==0 ) then

!     --- nintprop = 0, no integer or real properties given
!         Create array realprop

         fillkmeshw = .true.
         realprop = 0d0

      else

!     --- nintprop > 0, integer or real properties given

         fillkmeshw = .false.

      end if  ! ( nintprop==0 )

!     --- Allocate work arrays

      allocate ( indoldsols(numtypesols), stat=error )
      if ( error/=0 ) call eralloc ( error, numtypesols, 'iworktr' )

      do i = 1, numtypesols
         nullify (indoldsols(i)%index3)
         nullify (indoldsols(i)%index4)
      end do  ! i = 1, numtypesols

      sigma  = 0d0

      E%user => user                  ! contains real user input
      E%iuser => iuser                ! contains integer user input
      E%islold => SMF%islold          ! information of old vectors

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepassemstart'

      end if  ! ( debug )
      call erclos ( 'sepassemstart' )

      end subroutine sepassemstart
