      subroutine sepbuildactsol
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 15-12-2017 New storage of intmat
!        version  1.0  date 18-12-2016
!
!   copyright (c) 2016-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Activate array usol and renumber if necessary
!
! **********************************************************************
!
!                       KEYWORDS
!
!     right_hand_side
!     stiffness_matrix
!     renumbering
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulexxx
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulemain
      use sepmodulesmf
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, allocatable :: work(:)

!     work           Work array to store the renumbered solution temporary
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
!     TOORDC         Map array of solution type from vector sequence to matrix
!                    sequence. Complex version
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepbuildactsol' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepbuildactsol'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      usol => ks(SMF%iusol)%sol
      compl = ks(SMF%iusol)%compl==1   ! solution is real or complex
      icompl = 1
      if ( compl ) icompl = 2     ! multiplication factor for complex
      kmesh => kmeshloc
      bufchanged = .false.
      call sepfillkmesh ( kmesh )
      if ( indprr>0 .and. iinbld(2)==6 ) then

!     --- usol must be renumbered

         allocate ( work(icompl*nusol), stat = error )
         if ( error/=0 ) call eralloc ( error, icompl*nusol, 'work' )
         if ( ierror/=0 ) go to 1000

         if ( compl ) then

!        --- Complex case

            call toordc ( kprobr, usol, nusol, work )

         else

!        --- Real case

            work(kprobr(1:nusol)) = usol(1:nusol)
            usol = work

         end if  ! ( compl )

         deallocate ( work, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iwork' )

      end if  ! ( indprh>0 .and. iinbld(2)==6

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepbuildactsol'

      end if  ! ( debug )
      call erclos ( 'sepbuildactsol' )

      end subroutine sepbuildactsol
