      subroutine sepbuildchintm
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 15-12-2017 New storage of intmat
!        version  1.0  date 18-12-2016
!
!   copyright (c) 2016-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check second array intmat
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mass_matrix
!     right_hand_side
!     stiffness_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmoduleintmat
      use sepmodulesmf
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1777   INTMAT(2,2) has not been filled
!    1778   INTMAT(2,1) corresponds to the wrong problem number
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepbuildchintm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepbuildchintm'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Check second array intmat

      imas = iinbld(4)

      if ( imas==3 ) then

!     --- Test second array intmat

         iseqim = intmatar(2)

         if ( intmt(iseqim)%iprob/=iprob ) then

            call errint ( 2, 1 )
            call errint ( intmt(iseqim)%iprob, 2 )
            call errint ( iprob, 3 )
            call errsub ( 1778, 3, 0, 0 )

         end if

      end if  ! ( imas==3 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepbuildchintm'

      end if  ! ( debug )
      call erclos ( 'sepbuildchintm' )

      end subroutine sepbuildchintm
