      subroutine sepccgaccur ( res, rferr, A, iter, parvers )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     rhotest is smaller than maximal accuracy possible
!     It is useless to continue the process, print
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecmat
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix), intent(inout) :: A
      integer, intent(in) :: iter
      double precision, intent(in) :: rferr
      double complex, intent(in) :: res(A%neq)
      logical, intent(in) :: parvers

!     A              i    Matrix A
!     iter           i    Iteration number
!     parvers        i    If true it concerns the parallel version
!     res            i    Residual
!     rferr          i    Maximal accuracy possible for this matrix, &
!                         due to round-off
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: nwarnsave
      double precision :: reprint

!                    they are not
!     nwarnsave      Variable to store the value of nwarn
!     reprint        Auxiliary variable to print a real number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     677   The maximal accuracy with respect to the condition of the
!           matrix is reached
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepccgaccur' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepccgaccur'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( A%iprint==1 ) then

         if ( parvers ) then
            reprint = glnorm(res,A%neq)
         else
            reprint = cnorm(res,A%neq)
         end if  ! ( parvers )
         write ( irefwr, 150 ) iter-1, reprint
150      format(' The number of executed iterations in the', &
                ' CG-process is', i5/ &
                ' The 2-norm of the residual is:', ss,1pe12.2e2 )
      end if

      if ( A%iprint>=0 ) then
         if ( parvers ) then
            call erreal ( rferr, 1 )
         else
            call erreal ( sqrt(rferr), 1 )
         end if  ! ( parvers )
         if ( A%iprint>=2 ) then
            nwarnsave = nwarn
            nwarn = -100
            call errwar ( 677, 0, 1, 0 )
            nwarn = nwarnsave + 1
         else
            call errwar ( 677, 0, 1, 0 )
         end if

      end if

      A%numiter = iter-1
      A%itererr = 4

1000  call erclos ( 'sepccgaccur' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepccgaccur'

      end if  ! ( debug )

      end subroutine sepccgaccur
