      subroutine sepccgprint1 ( res, iter, A, bnorm, r0norm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform extended print for subroutine prcg during iteration
!     iprint>=2
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
!     print
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecmat
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix), intent(in) :: A
      integer, intent(in) :: iter
      double precision, intent(in) :: bnorm, r0norm
      double complex, intent(in) :: res(A%neq)

!     A              i    Matrix A
!     bnorm          i    Norm of the right-hand side (b,b)
!     iter           i    Iteration sequence number
!     r0norm         i    cnorm of start residual
!     res            i    real matrix containing A  - A  B ( B A  B )  B A
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: reprint

!                    they are not
!     reprint        Auxiliary variable to print a real number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepccgprint1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepccgprint1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( A%iprint>1 .and. iter==1 ) &
         write(irefwr,*) 'Iteration number  residual'

      reprint = cnorm(res,A%neq)
      if ( A%iaccuracy==0 .or. A%irelerr==3 ) then

         write ( irefwr, 201 ) iter, reprint/sqrt(bnorm)

      else if ( A%irelerr==1 ) then

         write ( irefwr, 201 ) iter, reprint/sqrt(r0norm)

      else

         write ( irefwr, 201 ) iter, reprint

      end if

201   format ( i7, ss, 1pe10.2e2 )

1000  call erclos ( 'sepccgprint1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepccgprint1'

      end if  ! ( debug )

      end
