      subroutine sepccgprint2 ( res, parvers, A, nmax )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-06-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Perform printing CG process when iteration has not converged
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
!     print
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecmat
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix), intent(inout) :: A
      integer, intent(in) :: nmax
      double complex, intent(in) :: res(A%neq)
      logical, intent(in) :: parvers

!     A              i    Matrix A
!     nmax           i    maximum number of iterations that may be performed by
!                         the CG-process. In any way no more than the number of
!                         degrees of freedom iterations are carried out.
!     parvers        i    If true it concerns the parallel version
!     res            i    residual
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: reprint

!                    they are not
!     reprint        Auxiliary variable to print a real number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     676   The CG-process has terminated because the number of iterations
!           exceeds the maximum value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepccgprint2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepccgprint2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( A%iprint==1 ) then

!     --- iprint = 1  Print information

         if ( parvers) then
            reprint = glnorm(res,A%neq)
         else
            reprint = cnorm(res,A%neq)
         end if  ! ( parvers)

         write ( irefwr, 350 ) nmax, reprint
350      format(' The number of executed iterations in the', &
                ' CG-process is', i5/ &
                ' The 2-norm of the residual is:', ss,1pe12.2e2 )

      end if

      call errint ( nmax, 1 )
      if ( A%aterror==1) then
         A%numiter = nmax
         call errwar ( 676, 1, 0, 0 )
      else
         call errsub ( 676, 1, 0, 0 )
      end if

1000  call erclos ( 'sepccgprint2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepccgprint2'

      end if  ! ( debug )

      end subroutine sepccgprint2
