      subroutine sepccopymatcs ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 06-01-2016 Use sepmodulebuf
!        version  1.0  date 02-04-2015
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy complex symmetric compact matrix in type matrix
!
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     matrix
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulekprob
      use sepmodulecmat
      use sepmodulematr
      use sepmodulesolve
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix) :: A

!     A              o    Structure of type matrix containing the necessary
!                         information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepccopymatcs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepccopymatcs'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      if ( .not. IM%direct ) then

!     --- Compact matrix

         A%nglob = 0
         A%scaling = 1          ! matrix is scaled (standard)
         A%deflation = .false.
         A%prlump = inpsol(33,1)

      end if  ! ( .not. IM%direct )

!     --- Store type of matrix

      A%type_prec = 0
      A%typesolv = typesolv
      A%type_storage = IM%jmethod
      A%symmetric = IM%symmetric
      A%neq = IM%nrusol
      A%nproj = 0
      A%iprint = inpsol(6,1)     ! print level
      A%factor = rinsol(3,1)
      A%adapt = rinsol(2,1)

      if ( parallel ) then

!     --- Parallel case, store matrix S_i out and storage scheme

         A%nglob = nusolglob

      end if  ! ( parallel )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( A%S, nrusol*2, 'diag' )
         write(irefwr,*) 'End sepccopymatcs'

      end if  ! ( debug )
      call erclos ( 'sepccopymatcs' )

      end subroutine sepccopymatcs
