      subroutine sepcgrferr ( A, x, b, res, rferr, iter, work )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     iter is multiple of sqrt(n)
!     Check error level, i.e. compute new value for rferr
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(in) :: A
      double precision, intent(in) :: x(A%neq), b(A%neq), res(A%neq)
      integer, intent(in) :: iter
      double precision, intent(inout) :: rferr
      double precision, intent(inout) :: work(A%neq)

!     A              i    Matrix A
!     b              i    Right-hand side
!     iter           i    Iteration number
!     res            i    Residual
!     rferr         i/o   Maximal accuracy possible for this matrix,
!                         due to round-off
!     work          i/o   Work array of length n to store the help vectors
!     x              i    Solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: h

!     h              Help variable to store a real
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcgrferr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcgrferr'
         write(irefwr,1) 'iter, nglob', iter, A%nglob
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      rferr = iter
      rferr = exp ( (rferr/A%nglob) ** 2 )
      if ( debug) write(irefwr,2) 'rferr', rferr

!     --- Compute Ax, store in work

      work = -residual( A, x, b )
      work = work + res

      h = norm(work, A)**2

      rferr = rferr * h
      if ( debug) write(irefwr,2) 'rferr, h', rferr, h

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcgrferr'

      end if  ! ( debug )
      call erclos ( 'sepcgrferr' )

      end subroutine sepcgrferr
