      subroutine sepckrylov ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve system of linear equations
!     Complex, serial case
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
!     solve
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulesolve
      use sepmodulecmat
      use sepmodulematr
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double complex, allocatable :: scalmat(:)
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     scalmat        Diagonal of scaling matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPCCOPYINPSOL Put information of inpsol and rinsol in matrix structure
!     SEPCSOLVEMETH  Solve system of linear equations
!     SEPZMAKEPREC   Create preconditioning matrix if necessary
!                    Complex matrix
!     SEPZRENUMSOLBC Renumber solution vector back to original form
!                    Complex case
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepckrylov' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepckrylov'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

!     --- Put information of inpsol and rinsol in matrix structure

      call sepccopyinpsol ( A, 1 )

!     --- Allocate arrays

      allocate ( scalmat(nrusol), stat = error )
      if ( error/=0 ) call eralloc ( error, nrusol, 'usol' )
      if ( ierror/=0 ) go to 1000

!     --- Create preconditioning matrix if necessary

      if ( .not. IM%direct ) call sepzmakeprec ( A )

      if ( debug ) call prinrl ( A%MD, nrusol, 'diag_pre' )

!     --- Solve system of equations

      call sepcsolvemeth ( A, usol, rhsd )

!     --- Copy solution array back into buffer

      call sepzrenumsolbc ( usol, 1 )

!     --- Deallocate arrays

      deallocate ( usol, rhsd, scalmat, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'usol' )

      if ( itotit==0 ) then
         itotit = itotit + A%numiter
      else
         itotit = itotit + A%numiter
         if ( A%iprint>=1 ) write(irefwr,210) itotit
210      format ( ' Accumulated number of iterations ', i8 )
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepckrylov'

      end if  ! ( debug )
      call erclos ( 'sepckrylov' )

      end subroutine sepckrylov
