      subroutine sepcopmatmulc ( A, B, diag, amatout, lapshiftd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-08-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy information from A with type mat into B with type cmatrix
!     Complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     complex
!     copy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulekprob
      use sepmodulematr
      use sepmodulemat
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (globmat) :: A
      type ( cmatrix ) :: B
      double complex, target, intent(in) :: diag(*), amatout(*), &
                                            lapshiftd(*)

!     A              i    matrix of type mat to be used in multiplication
!     B              o    internal matrix A_ii stored as type rmatrix
!     amatout        i    Matrix part i out i.e. internal unknowns coupled
!                         to unknowns  at other blocks
!     diag           i    Diagonal of the large matrix
!     lapshiftd      i    diagonal matrix for shifted laplace
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ipos
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     ipos           Position in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcopmatmulc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcopmatmulc'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(A%intmatref)

      B%neq = A%nrow
      irowsii => IM%rowsii
      icolsii => IM%colsii

      B%diag => IM%diag
      B%Mdiag => IM%diag

!     --- Row-Compact matrix

      length = irowsii(nrusol+1)
      B%S => diag(1:length)

      B%DLshift => lapshiftd(1:nrusol)
      if ( size(IM%diag)> 0 ) then
         B%diag => IM%diag
      end if  ! ( size(IM%diag)> 0 )

!     --- Store type of matrix

      B%rows => irowsii
      B%cols => icolsii
      B%type_storage = IM%jmethod
      B%nglob = B%neq
      B%symmetric = IM%symmetric

      if ( parallel ) then

!     --- Parallel case, store matrix S_i out and storage scheme

call ertrace
call instop

!         B%rowsout => irowsout
!         B%colsout => icolsout
!         B%nglob = nusolglob
!         B%innercontr => innercontr
!         length = irowsout(nrusol+1)
!         B%Aout => amatout(1:length)

!         if ( typecoupl==0 ) then

 !           B%sendinfo => sendinfo
!         else
!            B%sendinfo =>  sendinfopar
!         end if  ! ( typecoupl==0 )

      end if  ! ( parallel )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcopmatmulc'

      end if  ! ( debug )
      call erclos ( 'sepcopmatmulc' )

      end subroutine sepcopmatmulc
