      subroutine sepcopymatcs ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 23-02-2016 New parameters list
!        version  1.0  date 02-04-2015
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy symmetric matrix in type matrix
!
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulevecs
      use sepmoduleproj
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A

!     A              o    Structure of type matrix containing the necessary
!                         information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lengL

!     lengL          Length of lower tridiagonal of matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPCOPPROJ     Copy information of projection vectors in type matrix
!     SEPPARGLOB     Send information to neighboring processors in order to give
!                    all local arrays the global value in the common points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcopymatcs' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcopymatcs'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      A%decomp = .false.

      if ( parallel .and. typecoupl==0 ) then

         length = size(A%sendinfo)
         call sepparglob ( A, rhsd, A%neq ) ! make global rhsd

      end if  ! ( parallel )

!     --- If there are projection vectors, store information in A

      if ( nproj>0 ) call sepcopproj ( A )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( A%S, nrusol, 'diag' )
         write(irefwr,*) 'End sepcopymatcs'

      end if  ! ( debug )
      call erclos ( 'sepcopymatcs' )

      end subroutine sepcopymatcs
