      subroutine sepcopymt ( matr1, matr2 )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  2.0  date 23-02-2016 complete new subroutine
!        version  1.4  date 29-05-2014 remove call to time subroutine
!        version  1.3  date 15-11-2012 Long integers
!
!   copyright (c) 1999-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     copy matrix corresponding to matr1 into matrix corresponding to
!     matr2
! **********************************************************************
!
!                       KEYWORDS
!
!     copy
!     matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecpack
      use sepmodulematr
      use sepmoduleintmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: matr1, matr2

!     matr1      i    matrix to be copied
!     matr2      o    copy of matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: matrsav
      logical :: matrdiag, bouncond
      type (globmat), pointer :: S1, S2
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     S1             pointer to realmt(matr1)
!     S2             pointer to realmt(matr2)
!     bouncond       If true there are essential boundary conditions
!     matrdiag       if true matr2 at input is a diagonal matrix
!     matrsav        Saves value of matr1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCRDIAGMATR  create a new diagonal matrix in realmt
!     SEPCRMATRIX    create a new array in realmt
!     SEPFILLMAT     Fill information of matrix with sequence number iseqmt
!                    structure of type mat
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcopymt' )

      matrsav = matr1
      S1 => realmt(matr1)

      intmat = S1%intmatref
      IM => intmt(intmat)
      matrdiag = S1%diagonal

      if ( matrdiag ) then

!     --- matr1 is a diagonal matrix
!         Reserve space and activate arrays

         call sepcrdiagmatr ( matr2 )
         S2 => realmt(matr2)

!        --- Copy matrix

         S2%S_ii = S1%S_ii
         S2%diagonal = .true.

      else

!     --- matr1 is a non-diagonal matrix stored completely in core
!         Reserve space and activate arrays

         length = size(S1%S_ii)
         call sepcrmatrix ( matr2 )
         S2 => realmt(matr2)
         S2%intmul = .true.

!        --- Copy matrix part 1

         S2%S_ii = S1%S_ii
         call sepfillmat

      end if
      iseqmt = matrsav
      iseqmt2 = matr2

      call erclos ( 'sepcopymt' )

      end subroutine sepcopymt
