      subroutine sepcpreceissym ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the preconditioning matrix for the CG-process (complex matrix)
!     iprec = 2,  Eisenstat preconditioning
!     Old subroutine prcg01prec2
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmodulecmat
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, k, kpo, k1
      double complex :: sum1
      integer, pointer :: point(:), rowhor(:), colhor(:)

!     colhor         contains column numbers per horizontal row
!     i              Counting variable
!     k              Counting variable
!     k1             column number
!     kpo            position of element k1 in preconditioning matrix
!     point          pointer array referring to next element in a row
!     rowhor         contains starting addresses (-1) of horizontal rows in
!                    colhor
!     sum1           Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     SEPCOMMATREV   Fill arrays irowsnew and icolsnew with upper triangular
!                    part if icolsii and irowsii contain the lower triangular
!                    part and vice versa
!     SEPPRINMAT6    print compact matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2236   The desired preconditioning matrix does not exist, lumping is tried
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcpreceissym' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcpreceissym'
         call sepprinmat6 ( A )

      end if
      if ( ierror/=0 ) go to 1000

      allocate ( point(A%neq+1), rowhor(A%neq+1), colhor(size(A%cols)) )

!     --- Store row structure of matrix in row/colhor

      call sepcommatrev ( rowhor, colhor, A%rows, A%cols, nrusol )


      A%MD = A%S(A%diag)
      point = A%rows+2

      do i = 1, nrusol

         sum1 = A%MD(i)

         do k = rowhor(i)+1, rowhor(i+1)-1

            k1 = colhor(k)
            kpo = point(k1)
            sum1 = sum1 - ( A%S(kpo) / A%MD( k1 ) ) ** 2
            point(k1) = point(k1)+1

         end do  ! k = rowhor(i)+1, rowhor(i+1)-1

         if ( abs(sum1)<=0d0 ) then

!        --- If sum becomes too small or too big make prlump = 1
!            and start again.

             call errsub ( 2236, 0, 0, 0)
             go to 1000

         end if  ! ( A%prlump/=1 .and. ... )

         A%MD (i) = sqrt ( sum1 )

      end do  ! i = 1, nrusol

      deallocate ( point, rowhor, colhor )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcpreceissym'

      end if
      call erclos ( 'sepcpreceissym' )

      end subroutine sepcpreceissym
