      subroutine sepcscmatdivL ( A, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-04-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute y = A^(-1) x, where A is a preconditioning matrix
!     a symmetric compact storage is used, hence only D and L
!     The preconditioning matrix has been created before
!     In other words a part of the ILU is executed (The U part)
!     This routine is meant for eistenstat preconditioning
!     complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecmat   ! definition of matrices
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix), intent(in) :: A
      double complex, intent(in) :: x(A%neq)
      double complex :: y(A%neq)

!     A              i    Matrix A
!     x              i    Solution vector
!     y              o    Result  y = A^(-1) x
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, jp, k
      double complex :: yi
      double complex :: sum1

!     i              General loop variable
!     j              General loop variable
!     jp             Loop over elements in row
!     k              Column number
!     sum1           help variable to compute a sum
!     yi             help variable to store y(i)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector stored
!     SEPPRINMAT6    print compact matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcscmatdivL' )

      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcscmatdivL'
         call prinrl ( x, A%neq, 'x' )
         call sepprinmat6 ( A )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First multiplication by diagonal:  Y = D x

      y = x
      do i = 1, A%neq

         yi = y(i)
         do jp = A%rows(i)+2, A%rows(i+1)
            k = A%cols(jp)
            y(k) = y(k)-A%S(jp)*yi
         end do  ! k = row(i)+1, diag(i)-1

      end do

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( y, A%neq, 'y' )
         write(irefwr,*) 'End sepcscmatdivL'

      end if  ! ( debug )
      call erclos ( 'sepcscmatdivL' )

      end subroutine sepcscmatdivL
