      subroutine sepdmatvec ( u, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-04-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Matrix vector multiplication D u = y (simple)
!     D is divergence matrix and u a velocity vector
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     matrix_vector_multiplication
!     real
!     simple
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesimple
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: u(nu)
      double precision :: y(np)

!     u              i    vector to be multiplied
!     y              o    result Du
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k
      double precision :: sum1

!     i              Counting variable
!     j              Counting variable
!     k              Column sequence number
!     sum1           Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPPARGLOB     Send information to neighboring processors in order to give
!                    all local arrays the global value in the common points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepdmatvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepdmatvec'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, np

         sum1 = 0d0

!        --- for all non-zero elements d_ij do
!               y_i = y_i + d_ij x_j

         do j = D%rows(i)+1, D%rows(i+1)
            k = D%cols(j)
            sum1 = sum1 + D%DIV(j)*u(k)
         end do  ! j = D%rows(i)+1, D%rows(i+1)
         y(i) = sum1

      end do  ! i = 1, np

      if ( parallel .and. typecoupl==0 ) then
         length = size(C%sendinfo)
         call sepparglob ( C, y, C%neq )
      end if  ! ( parallel .and. typecoupl==0 ) then

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepdmatvec'

      end if  ! ( debug )
      call erclos ( 'sepdmatvec' )

      end subroutine sepdmatvec
