      subroutine sepfillicount
! ======================================================================
!
!        programmer    Guus Segal
!        version 10.0  date 24-07-2015 Use sepmodulebuild
!        version  9.1  date 08-08-2014 Extra debug
!        version  9.0  date 23-03-2011 Extra parameter kprob
!
!   copyright (c) 1984-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill arrays index1sub and index2 for the element subroutines
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     matrix
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: jcount, j, kchois, inodp, inusol, inunkp, k, i

!     i              Counting variable
!     inodp          Nodal point number
!     inunkp         Number of degrees of freedom in first point
!     inusol         Position of unknown in array USOL
!     j              Counting variable
!     jcount         Help parameter to compute the centroid of the element
!     k              Counting variable
!     kchois
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillicount' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillicount'
         write(irefwr,1) 'inpelm, nunkp', inpelm, nunkp
         write(irefwr,1) 'ichnun, indprf, indpra', ichnun, indprf, indpra
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      jcount=0
      inunkp = nunkp

      kchois=ichnun
      if ( itype>0 .and. ielgrp<=nelgrp ) then
         kchois=ichnun+4
      end if

      allocate ( index4tmp(inpelm), stat=error )
      if ( error/=0 )  call eralloc ( error, inpelm, 'index4tmp' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'kchois', kchois
      if ( itype==-2 ) then

!     --- type number = -2, boundary conditions of the type u equals
!                           unknown constant

         jcount = inpelm

      else

!     --- other type numbers

         select case ( kchois )

         case(1, 5)

!        --- ichnun = 1

            jcount=inpelm
            do j = 1,inpelm
               index4tmp(j) = j
            end do  ! j = 1,inpelm

         case(2)

!        --- ichnun = 2

            do j = 1,inpelm
               inodp=index1(j)
               inusol=kprobf(inodp)
               inunkp=kprobf(inodp+1)-inusol
               if ( inunkp/=0 ) jcount=jcount+1
               index4tmp(j) = jcount
            end do  ! j = 1,inpelm

         case(3)

!        --- ichnun = 3

            do j = 1,inpelm
               inodp=index1(j)
               jcount=jcount+nunkp
               index4tmp(j) = jcount
            end do  ! j = 1,inpelm

         case(4)

!        --- ichnun = 4

            do j = 1,inpelm
               inodp=index1(j)
               inusol=kprobf(inodp)
               inunkp=kprobf(inodp+1)-inusol
               jcount=jcount+inunkp
               index4tmp(j) = jcount
            end do  ! j = 1,inpelm

         case(6)

!        --- ichnun = 2   itype>0

            if ( indprf>0 ) then

!           --- indprf>0

               do j = 1,inpelm
                  inunkp=kprobatmp(j)
                  if ( inunkp>0 ) jcount=jcount+1
                  index4tmp(j) = jcount
               end do  ! j = 1,inpelm

            else

!           --- indprf = 0

               do j = 1,inpelm
                  if ( indpra>0 ) inunkp=kprobatmp(j)
                  if ( inunkp>0 ) jcount=jcount+1
                  index4tmp(j) = jcount
               end do  ! j = 1,inpelm

            end if

         case(7)

!        --- ichnun = 7

            do j = 1, inpelm
               if ( indpra>0 ) inunkp=kprobatmp(j)
               jcount=jcount+inunkp
               index4tmp(j) = jcount
            end do  ! j = 1,inpelm

         case(8)

!        --- ichnun = 8   itype>0

            if ( indprf>0 ) then

!           --- indprf>0

               do j = 1,inpelm
                  inunkp=kprobatmp(j)
                  jcount=jcount+inunkp
                  index4tmp(j) = jcount
               end do  ! j = 1,inpelm

            else

!           --- indprf = 0

               do j = 1,inpelm
                  if ( indpra>0 ) inunkp=kprobatmp(j)
                  jcount=jcount+inunkp
                  index4tmp(j) = jcount
               end do  ! j = 1,inpelm

            end if

         end select  ! case ( kchois )

      end if

1000  icount=jcount

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icount', icount
         write(irefwr,*) 'End sepfillicount'

      end if  ! ( debug )
      call erclos ( 'sepfillicount' )

      end
