      subroutine sepfillindex34 ( islold, ikelmc, lenindex3 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  7.4  date 06-08-2015 New definition istold
!        version  7.3  date 06-02-2014 Correction itype<0
!        version  7.2  date 17-04-2013 Extra debug
!
!   copyright (c) 1988-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array index3 with addresses of unknowns for one element.
!     Fill array index4 starting position for node in index3
!     Help subroutine for subroutine TO0050 (SYSTEM)
! **********************************************************************
!
!                       KEYWORDS
!
!     fill
!     topology
!     matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulebuild
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: islold(numold), ikelmc, lenindex3

!     ikelmc         i    position in kmesh part c
!     islold         i    This array is used if ichois<0, or numold>0
!                         When ichois<0, it is supposed to consist of one vector
!                         only, which must be of the same structure as the
!                         solution vector
!                         When numold>0, islold contains numold vectors which may
!                         be solution vectors of several problems or vectors of
!                         special structure (type 115) resulting of several
!                         problems
!     lenindex3      i    length of index3
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k, irefbf, j, itypvc, jcount, inunrf, &
                 inusol, i, inunkp, isum, ndegfd, m1, indprfloc, &
                 mark(numtypesols), iseqlc, iproblc, ivec
      logical :: consta

!     consta         If true the number of unknowns per point is constant
!                    they are not
!     i              Counting variable
!     indprfloc       Local value of indprf
!     inunkp         Number of degrees of freedom in point
!     inunrf         Reference value for inunkp
!     inusol         Position of unknown in array USOL
!     iproblc        Local value of iprob
!     irefbf         Reference number for type of vector (istold(k,2))
!     iseqlc         Sequence number of vector in set
!     isum           Integer variable to compute a sum
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     j              Counting variable
!     jcount         Help parameter to compute the centroid of the element
!     k              Counting variable
!     m1             number of integration points
!     mark           marker array
!                    if mark(ivectr) = 1, array ivectr has been filled
!                    in indoldsols
!     ndegfd         Number of degrees of freedom per point
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERTRACE        Prints trace back of subroutines
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1585   SEPRAN arrays of this type  are not yet allowed in this subroutine
!    2189   Length of index3>index2
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillindex34' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillindex34'
         call ertrace
         write(irefwr,1) 'numold, itype, ielgrp, nelgrp', &
                          numold, itype, ielgrp, nelgrp
         write(irefwr,1) 'lenindex3', lenindex3
         call prinin ( index1, inpelm, 'index1' )
         call prinin ( index2, icount, 'index2' )

  1      format ( a, 1x, (10i8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Loop over all old solution vectors

      mark = 0      ! all types are marked as not filled
      do k = 1, numold
         irefbf = istold1(1,k)
         if ( islold(k)<=0 ) go to 900
         iproblc = ks(islold(k))%iprob
         indprfloc = istold1(2,k)
         if ( ks(islold(k))%typevec==115 .and. indprfloc>0 ) then

!        --- Type 115

            ivec = ks(islold(k))%ivec
            kprobflc => kp(iseqkp)%kprobsub(iproblc)%kprobe3(ivec)%arr

         else if ( indprfloc>0 ) then

!        --- Type 110

            kprobflc => kp(iseqkp)%kprobsub(iproblc)%kprobf

         end if  ! ( islold(2)==115 .and. indprfloc>0 )

         iseqlc = istold1(5,k)

         if ( debug ) then
            write(irefwr,1) 'k, irefbf, indprfloc', &
                             k, irefbf, indprfloc
            write(irefwr,1) 'ielgrp', ielgrp
         end if  ! ( debug )

         if ( irefbf==0 ) then

!        --- irefbf = 0, so index3 = index2

            index3(k,1:icount) = index2(1:icount)
            jcount = icount
            if ( debug ) write(irefwr,1) 'jcount', jcount

            if ( indprfloc<=0 ) then

!           --- indprfloc = 0, number of unknowns constant

               inunkp = icount/inpelm
               do j = 1, inpelm
                  index4(k,j) = j*inunkp
               end do
               if ( inunkp==1 ) then
                  index4(k,inpelm+1) = 1
               else
                  index4(k,inpelm+1) = 2
               end if

            else

!           --- indprfloc>0, number of unknowns not constant

               isum = 0
               consta = .true.
               do j = 1, inpelm
                  inusol = kprobflc(index1(j))
                  inunkp = kprobflc(index1(j)+1)-inusol
                  if ( j==1 ) then
                     inunrf = inunkp
                  else
                     if ( inunrf/=inunkp ) then
                        consta = .false.
                        inunrf = min(inunrf,inunkp)
                     end if
                  end if
                  isum = isum + inunkp
                  index4(k,j) = isum
               end do

               if ( debug ) &
                  write(irefwr,1) 'isum, jcount, itype', &
                                   isum, jcount, itype
               if ( isum>icount .and. itype>0 ) then

!              --- isum>icount, special situation, refill index4 if
!                  possible

                  if ( ielgrp>nelgrp ) then

!                 --- error, this situation has not yet been programmed

                     call errsub ( 2189, 0, 0, 0 )
                     go to 1000

                  end if
                  consta = .true.
                  isum = 0

                  do j = 1, inpelm
                     inusol = kprobflc(index1(j))

                     inunkp = kprobatmp(j)
                     if ( j==1 ) then
                        inunrf = inunkp
                     else
                        if ( inunrf/=inunkp ) then
                           consta = .false.
                           inunrf = min(inunrf,inunkp)
                        end if
                     end if
                     isum = isum+inunkp
                     index4(k,j) = isum
                  end do
                  jcount = isum
               end if
               if ( consta .and. inunkp==1 ) then
                  index4(k,inpelm+1) = 1
               else if ( consta ) then
                  index4(k,inpelm+1) = 2
               else if ( isum>0 ) then
                  index4(k,inpelm+1) = -1
               else
                  index4(k,inpelm+1) = -2
               end if
               if ( .not. consta .and. icount==inunrf*inpelm .and. &
                    (ishape==1 .or. ishape==3 .or. ishape==5 .or. &
                     ishape==11 .or. ishape==13 .or. ishape==21) ) then

!              --- not constant number of degrees of freedom, but linear element
!                  This means that we have a coupled boundary element
!                  Therefore we take the minimum value of inunkp and store
!                  only the corresponding unknowns

                  isum = 0
                  do j = 1, inpelm
                     isum = isum + inunrf
                     index4(k,j) = isum
                  end do
                  jcount = icount
                  index4(k,inpelm+1) = 2

               end if

            end if
            if ( debug ) write(irefwr,1) 'jcount', jcount

            if ( debug ) &
               write(irefwr,1) 'k, iseqlc, mark(iseqlc)', k, iseqlc, mark(iseqlc)

            if ( mark(iseqlc)==0 ) then

!           --- vectors of type iseqlc have not been filled yet

               mark(iseqlc) = 1  ! mark as filled

               indoldsols(iseqlc)%index4(1:inpelm+1) => index4(k,1:inpelm+1)

               indoldsols(iseqlc)%lenindex3 = icount

            end if  ! ( imark(iseqlc)==0 )

         else if ( irefbf==-1 ) then

!        --- irefbf = -1, i.e. this array has not been filled:  set index4

            do j = 1, inpelm+1
               index4(k,j) = -1
            end do
            index4(k,inpelm+1) = -2

         else if ( irefbf<k ) then

!        --- 0<irefbf<k, copy from preceding index3/4

            do j = 1, inpelm+1
               index4(k,j) = index4(irefbf,j)
            end do
            do j = 1, index4(k,inpelm)
               index3(k,j) = index3(irefbf,j)
            end do
            jcount = index4(k,inpelm)
            if ( debug ) write(irefwr,1) 'jcount', jcount

         else

!        --- irefbf = k, new array index3/4

            itypvc = ks(islold(k))%typevec

            if ( debug ) write(irefwr,1) 'itypvc, indprfloc', itypvc, indprfloc

            if ( itypvc==110 .or. itypvc==115 ) then

!           --- itypvc = 110, Array is of the type isol,
!               or itypvc = 115, array is an array of special structure
!               Compute starting address of KPROB for specific problem
!               as well as local nunkp

               jcount = 0
               if ( indprfloc<=0 ) then

!              --- indprfloc = 0, constant number of unknowns per node

                  inunkp = ks(islold(k))%nusol/npoint
                  if ( debug ) write(irefwr,1) 'inunkp', inunkp
                  if ( inunkp==1 ) then

!                 --- inunkp = 1

                     index3(k,1:inpelm) = index1(1:inpelm)
                     jcount = inpelm

                  else

!                 --- indprfloc = 0, nunkp>1
!                                 fixed number of degrees of freedom per point

                     do j = 1, inpelm
                        inusol = (index1(j)-1)*inunkp
                        do i = 1, inunkp
                           index3(k,jcount+i) = inusol + i
                        end do
                        jcount = jcount + inunkp
                     end do

                  end if
                  do j = 1, inpelm
                     index4(k,j) = j*inunkp
                  end do
                  if ( inunkp==1 ) then
                     index4(k,inpelm+1) = 1
                  else
                     index4(k,inpelm+1) = 2
                  end if
                  if ( debug ) write(irefwr,1) 'jcount', jcount

               else

!              --- indprfloc>0, number of degrees of freedom per node not
!                              constant.

                  consta = .true.

                  if ( debug ) then
                     write(irefwr,1) 'inpelm', inpelm
                     call prinin ( index1, inpelm, 'index1' )
                  end if  ! ( debug )

                  do j = 1, inpelm

                     inusol = kprobflc(index1(j))

                     if ( itypvc==110 .and. istold1(3,k)==istold1(4,k) ) &
                        then

!                    --- vector is of type solution vector corresponding
!                        to the same problem as we are solving
!                        inunkp can be found in kprobatmp

                        inunkp = kprobatmp(j)

                     else

!                    --- vector is of not of that type
!                        get inunkp from  kprobf

                        inunkp = kprobflc(index1(j)+1)-inusol

                     end if  ! ( itypvc==110 .and. istold1(3,k)==istold1(4,k) )

                     if ( debug ) then

!                    --- Debug information

                        write(irefwr,1) 'itypvc, inusol', itypvc, inusol
                        write(irefwr,1) 'iprob, jprob', &
                                         istold1(3,k), istold1(4,k)
                        write(irefwr,1) 'inunkp, jcount', inunkp, jcount

                     end if  ! ( debug )
                     do i = 1, inunkp
                        index3(k,jcount+i) = inusol+i
                     end do
                     jcount = jcount + inunkp
                     index4(k,j) = jcount
                     if ( j==1 ) then
                        inunrf = inunkp
                     else
                        if ( inunrf/=inunkp ) then
                           consta = .false.
                           inunrf = min(inunrf,inunkp)
                        end if
                     end if
                     if ( debug ) write(irefwr,1) 'jcount', jcount

                  end do

                  if ( debug ) call prinin ( index3, jcount, 'index3' )

                  if ( consta .and. inunkp==1 ) then
                     index4(k,inpelm+1) = 1
                  else if ( consta ) then
                     index4(k,inpelm+1) = 2
                  else if ( jcount>0 ) then
                     index4(k,inpelm+1) = -1
                  else
                     index4(k,inpelm+1) = -2
                  end if
                   if ( .not. consta .and. &
                       (ishape==1 .or. ishape==3 .or. &
                        ishape==5 .or. ishape==11 .or. &
                        ishape==13 .or. ishape==21) ) then

!                 --- not constant number of degrees of freedom,
!                     but linear element
!                     This means that we have a coupled boundary element
!                     Therefore we take the minimum value of inunkp and store
!                     only the corresponding unknowns

                     jcount = 0
                     do j = 1, inpelm
                        inusol = kprobflc(index1(j))
                        inunkp = kprobflc(index1(j)+1)-inusol
                        do i = 1, inunrf
                           index3(k,i+jcount) = inusol+i
                        end do
                        jcount = jcount+inunrf
                        index4(k,j) = jcount
                     end do
                     index4(k,inpelm+1) = 2

                  end if
                  if ( debug ) write(irefwr,1) 'jcount', jcount

               end if

            else if ( itypvc==116 ) then

!           --- itypvc = 116,  Array of special structure per element
!               shift in  vector required, reset jcount with total degrees
!               of freedom

               index4(k,1) = ks(islold(k))%n
               do j = 2, inpelm
                  index4(k,j) = index4(k,1)
               end do
               index4(k,inpelm+1) = 116
               jcount=(ielem-1)*index4(k,1)
               do j = 1, index4(k,1)
                  index3(k,j) = j+jcount
               end do
               jcount=index4(k,1)
               if ( debug ) write(irefwr,1) 'jcount', jcount

            else if ( itypvc==126 ) then

!           --- itypvc = 126,  Array of special structure per element per node
!               shift in  vector required, reset jcount with total degrees
!               of freedom

               ndegfd = ks(islold(k))%n
               index4(k,1) = ndegfd
               do j = 2, inpelm
                  index4(k,j) = ndegfd+index4(k,j-1)
               end do
               index4(k,inpelm+1) = 126
               jcount=ikelmc*ndegfd
               do j = 1, ndegfd*inpelm
                  index3(k,j) = j+jcount
               end do
               jcount = ndegfd*inpelm
               if ( debug ) write(irefwr,1) 'jcount', jcount

            else if ( itypvc==129 ) then

!           --- itypvc = 129,  Array of special structure per element per
!               integration point
!               shift in  vector required, reset jcount with total degrees
!               of freedom

               ndegfd = ks(islold(k))%n
               index4(k,1) = ndegfd
               if ( ielgrp<=nelgrp ) then

!              --- Standard case not a boundary element

                  isoltemp => ks(islold(k))%isol
                  m1 = isoltemp((ielgrp-1)*4+1)
                  jcount=isoltemp((ielgrp-1)*4+4)+(irelem-1)*m1
                  do j = 1, ndegfd*m1
                     index3(k,j) = j+jcount
                  end do
                  jcount=index4(k,1)*m1
                  if ( debug ) write(irefwr,1) 'jcount', jcount

               else

!              --- Boundary element, no information of integration points
!                  available

                  m1 = 1
                  do j = 1, ndegfd*m1
                     index3(k,j) = j
                  end do
                  jcount = ndegfd*m1
                  if ( debug ) write(irefwr,1) 'jcount', jcount

               end if
               index4(k,2) = m1
               do j = 3, inpelm
                  index4(k,j) = index4(k,1)
               end do
               index4(k,inpelm+1) = 129

            else if ( itypvc==801 ) then

!           --- itypvc = 801,  Array corresponding to electrodes,
!               there is no relation to nodes or elements

               index4(k,1) = 1
               index4(k,2) = 1
               do j = 3, inpelm
                  index4(k,j) = index4(k,1)
               end do
               index4(k,inpelm+1) = 801
               index3(k,1) = 1
               jcount = 1

            else

!           --- itypvc is of unknown type, give error message

               call errint ( itypvc, 1 )
               call errsub ( 1585, 1, 0, 0 )
               go to 1000

            end if

            iseqlc = istold1(5,k)

            if ( debug ) &
               write(irefwr,1) 'k, iseqlc, mark(iseqlc)', k, iseqlc, mark(iseqlc)

            if ( mark(iseqlc)==0 ) then

!           --- vectors of type iseqlc have not been filled yet

               mark(iseqlc) = 1  ! mark as filled
               if ( .not. associated(indoldsols(iseqlc)%index3) ) then

!              --- Allocate index3

                  inunkp = oldsols(k)%nunkp
                  length = max(inunkp*npelm, jcount, lenindex3 )

                  allocate ( indoldsols(iseqlc)%index3(length), &
                             stat = error )
                  if ( error/=0 ) call eralloc ( error, nunkp*inpelm, 'index3' )
                  if ( ierror/=0 ) go to 1000

               end if  ! ( .not. associated(indoldsols(iseqlc)%index3) )
               if ( debug ) write(irefwr,1) 'jcount', jcount
               indoldsols(iseqlc)%index3(1:jcount) = index3(k,1:jcount)
               indoldsols(iseqlc)%index4(1:inpelm+1) => index4(k,1:inpelm+1)
               indoldsols(iseqlc)%lenindex3 = jcount


            end if  ! ( imark(iseqlc)==0 )

         end if  ! ( irefbf==0 )

900   end do

1000  if ( debug ) then

!     --- Debug information

         call prinin1 ( index3, jcount, numold, 'index3' )
         call prinin ( index4, inpelm+1, 'index4' )
         write(irefwr,*) 'End sepfillindex34'

      end if  ! ( debug )
      call erclos ( 'sepfillindex34' )

      end subroutine sepfillindex34
