      subroutine sepfillmat
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-08-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information of matrix with sequence number iseqmt
!     structure of type mat
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmoduleintmat
      use sepmodulekprob
      use sepmodulematr
      use sepmodulemat
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (globmat), pointer :: S
      integer :: icompl, intmatref, leng1, ipos, &
                 ipstart, lastp
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     S              pointer to realmt(iseqmt)
!     icompl         Multiplication factor for complex numbers
!                    Real: icompl=1,   Complex: icompl=2
!     intmatref      sequence number of intmat array corresponding to S
!     ipos           Position in array
!     ipstart        Starting address in array
!     lastp          last position in array
!     leng1          Length of local co-ordinates array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     SEPCOPMATMULC  Copy information from A with type mat into B with
!                    type cmatrix (complex)
!     SEPCOPMATMULR  Copy information from A with type mat into B with
!                    type rmatrix
!     SEPCOPMATMULRP Copy information from A with type mat into B with
!                    type pmatrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillmat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillmat'
         call ertrace
         write(irefwr,1) 'iseqmt, ninterface', iseqmt, ninterface
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      S => realmt(iseqmt)

      S%intmul = .true.
      length = size(S%S_ii)
      intmatref =  S%intmatref
      IM => intmt(intmatref)
      icompl = 1
      if ( S%complexmat ) icompl = 2
      S%nrow = IM%nrusol
      nbound = IM%nbound

      if ( debug ) &
         write(irefwr,1) 'nrow, nbound, length', &
                          S%nrow, nbound, length

      if ( ninterface>0 ) then

!     --- There are interface unknowns

         S_if => S%S_if
         S_fi => S%S_fi
         if ( debug ) &
            write(irefwr,1) 'size(S_if), size(S_fi)', &
                             size(S_if), size(S_fi)

      end if  ! ( ninterface>0 )

      S_iout => S%S_iout
      if ( debug ) write(irefwr,1) 'size(S_iout)', size(S_iout)

      if ( parallel ) then

         if ( typecoupl==0 ) then
            S%sendinfo => sendinfo
         else
            S%sendinfo => sendinfopar
         end if  ! ( typecoupl==0 )

         S%S_i%sendinfo => S%sendinfo

      end if  ! ( parallel )

      if ( S%complexmat ) then
         call sepcopmatmulc ( S, S%Sc_i, S%S_ii, S%S_iout, S%Dlapshift )
      else
         call sepcopmatmulr ( S, S%S_i )
         call sepcopmatmulrp ( S, S%S_p )
      end if  ! ( S%complexmat ) then

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillmat'

      end if  ! ( debug )
      call erclos ( 'sepfillmat' )

      end subroutine sepfillmat
