      subroutine sepfillstelm ( jkelmc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-02-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Initialize some parameters for one element
!     Fill index arrays
!     Help routine for matrix assembly
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     matrix
!     right_hand_side
!     assembly
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmodulecpack
      use sepmoduleelm
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: jkelmc

!     jkelmc         i    position in kmesh part c
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, liarry

!     i              Counting variable
!     liarry         length of the integer array icoeffarray
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCOPYELEM    Copy information of elements in type element
!     SEPFILLINDEX34 Fill array index3 with addresses of unknowns for one
!                    element.
!                    Fill array index4 starting position for node in index3
!     TO0000         Fill arrays index1 and index2
!     TO00641        Fill array index4
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillstelm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillstelm'
         write(irefwr,1) 'numold', numold
  1      format ( a, 1x, (10i8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      skip = .false.  ! do not skip element

!     --- fill array index1

      index1 => kmeshc(jkelmc+1:jkelmc+inpelm)

!     --- fill arrays index2

      if ( ielgrp<=nelgrp ) ilocelem = ilocelem + 1
      ilocelem = max(1,ilocelem)

      call to0000 ( ichnun )  ! Fill array index2

      if ( parallel .and. ielgrp>nelgrp ) then

!     --- parallel computation, a boundary element is used
!         Check if element contains nodal points with sequence
!         number 0.
!         In that case the element must be skipped

         do i = 1, inpelm
            if ( index1(i)==0 ) then
               skip = .true.
               go to 1000
            end if  ! ( index1(i)==0 )
         end do

      end if  ! ( parallel .and. ielgrp>nelgrp )

!     --- Fill arrays index3 and index4 if necessary

      if ( numol0>0  ) then

!     --- numol0>0, there are several vectors

         call sepfillindex34 ( E%islold, jkelmc,nindex+inpelm )

      else

!     --- numol0 = 0, there is only one vector

         index3(1,1:icount) = index2(1:icount)
         indoldsols(1)%lenindex3 = icount
         indoldsols(1)%index4 => index4(1,:)
         index4(1,inpelm+1) = 110
         call to00641  ! Fill array index4

      end if
      if ( ierror/=0 ) go to 1000

!     --- Copy information into type elem

      call sepcopyelem

!     --- compute element matrix and element vector

      elemvc2d = 0d0
      if ( itype/=996 ) then

!     --- Type number 996 is special, it has no element matrix
!         nor a mass matrix

         elemmt2d = 0d0

      end if  ! ( itype/=996 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillstelm'

      end if  ! ( debug )
      call erclos ( 'sepfillstelm' )

      end subroutine sepfillstelm
