      subroutine sepidrshadow ( sparam, nrusol, pshadow )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 03-04-2013 New parameter list
!        version  2.1  date 11-12-2012 Long integers
!        version  2.0  date 10-11-2008 New parameter list
!
!   copyright (c) 2007-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Compute shadow vectors p for IDR process
!      Store in pshadow
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     iterative
!     linear_solver
!     preconditioning
!     parallel
!     idr
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulecpack
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: sparam, nrusol
      double precision, intent(out) :: pshadow(nrusol,sparam)

!     nrusol         i    Number of free unknowns
!     pshadow        o    Array of size n x s
!                         Contains all shadow vectors of size n each
!                         So pshadow(i,j) contains i^th component of p_j
!     sparam         i    Defines the parameters s in IDR(s)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRANDOM        generates a random vector in parallel
!     PRINRL1        Print 2d real vector
!     PRSINGVALDEC   Compute singular value decomposition of array a
!                    Returns with orthogonal basis of space spanned by A
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepidrshadow' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepidrshadow'
         write(irefwr,1) 'nrusol', nrusol
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- The vectors in P are chosen randomly

      do i = 1, sparam
     !    if ( testversion ) then
            call prandom ( nrusol, pshadow(:,i) )
    !     else
    !        call random_number ( pshadow(:,i) )
   !      end if  ! ( .not. parallel )
!         pshadow(:,i) = 1d0-pshadow(:,i)
      end do  ! i = 1, sparam

      if ( debug ) call prinrl1 ( pshadow, sparam, nrusol, 'Pshadow' )

!     --- P will be defined as an orthonormal basis in the
!         space spanned by the column vectors
!         This is done by applying svdcmp

      call prsingvaldec ( pshadow, nrusol, sparam )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( pshadow, sparam, nrusol, 'Pshadow' )
         write(irefwr,*) 'End sepidrshadow'

      end if  ! ( debug )
      call erclos ( 'sepidrshadow' )

      end
