      subroutine seplinsinp ( inpsol, inpsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-12-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array inpsol with information for building the matrix as
!     well as for the linear solver
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulekprob
      use sepmodulemnsub
      use sepmodulemain
      use sepmoduleinput
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inpsep(*)
      integer, intent(out) :: inpsol(20)

!     inpsep         i    The integer input read is stored in array INPSEP
!                         See subroutine READSCLINSOL
!     inpsol         o    Input array defined by the user
!                         With this array the user may indicate which actions
!                         he wants to be performed by LINPRB
!                         See subroutine LINPRBBF
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqlc, ivec, iseqcf, ivectr, iseqsl, iproblc, &
              iprobr, ipos, maxcall

!     ipos           Position in array
!     iproblc        Actual problem number
!     iprobr         Problem number read
!     iseqcf         Sequence number corresponding to the coefficients
!     iseqlc         Sequence number read
!     iseqsl         Sequence number corresponding to the linear solver
!     ivec           Sequence number of vector of special structure
!     ivectr         Sequence number of first vector
!     maxcall        Maximum value of icall in sepmoduleinp
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inivectr

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIVECTR       Return with sequence number of vector
!     PRININ         print 1d integer array
!     SEPGETINPFINF  Find maxcall and ipos from sepmoduleinput as function of
!                    isubr and iseqnr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! *********************iseq*************************************************
!
!                       DATA STATEMENTS
!
      data iseqcf /0/, ivectr /0/, iseqsl /0/, iproblc /0/
! ======================================================================
!
      call eropen ( 'seplinsinp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seplinsinp'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkp = kprob
      nprob = kp(iseqkp)%nprob

      iseqcf = 1  ! default sequence number of coefficients
      iseqsl = 1
      iproblc  = min(iproblc+1, nprob)

!     --- in case more than one problem we set the default sequence number to 0

      if ( nprob>1 ) iseqcf = 0
      iseqlc   = inpsep(1)
      if ( iseqlc/=-1 ) iseqcf = iseqlc
      iseqlc = iinputarrs(6)%maxcall
      iseqcf = min ( iseqcf, iseqlc )
      ivectr = iproblc
      ivec   = inivectr(inpsep(3))
      if ( ivec/=-1 ) ivectr = ivec
      iseqlc   = inpsep(4)
      if ( iseqlc/=-1 ) iseqsl = iseqlc

      if ( debug ) &
         write(irefwr,1) 'ivectr, iseqcf, iseqsl', &
                          ivectr, iseqcf, iseqsl

      isolut => isolut2d(ivectr)

      if ( isolut>=1 .and. isolut<=lastsol ) then
         if ( ks(isolut)%typevec==110 ) iproblc = ks(isolut)%iprob
      end if  ! ( isolut>=1 .and. isolut<=lastsol )
      iprobr = inpsep(2)
      if ( iprobr/=-1 ) iproblc = iprobr

      if ( iseqcf>0 ) then

!     --- iseqcf>0, check if there is an input block with this sequence number

         call sepgetinpfinf ( 6, iseqcf, ipos, maxcall )
         if ( .not. in(ipos)%used ) iseqcf = 0  ! input block does not exist

      end if  ! ( iseqcf>0 )

      inpsol(16) = inpsep(10) ! keep_matrix
      inpsol(17) = inpsep(11) ! keep_rhside
      inpsol(18) = inpsep(12) ! iseq_test_force
      inpsol(19) = inpsep(13) ! iseq_rhs
      inpsol(20) = iproblc    ! iseq_intmat1d

      inpsol(1) = 3  !  isubr
      inpsol(2) = 0  !  iread
      inpsol(3) = iseqcf
      inpsol(4) = iseqsl
      inpsol(5) = iproblc
      inpsol(6) = ivectr
      inpsol(7) = max(0,numvecmn)
      inpsol(8:12) = inpsep(5:9)
      inpsol(13:14) = 0
      inpsol(15:18) = inpsep(10:13)
      inpsol(19) = iproblc   !  iseqintmat

!     --- If the reaction force must be computed, subtract the contribution
!         of slip

      if ( inpsol(10)>0 ) inpsol(14) = 1


      inpsol(20) = 0  ! ictime
      if ( inpsol(15)>1 ) inpsol(20) = 6  ! keep matrix

1000  if ( debug ) then

!     --- Debug information

        call prinin ( inpsol, 20, 'inpsol' )
        write(irefwr,*) 'End seplinsinp'

      end if  ! ( debug )
      call erclos ( 'seplinsinp' )

      end subroutine seplinsinp
