      subroutine sepmatvec1 ( A, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-10-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Matrix vector multiplication for internal matrix
!     Real matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_vector_multiplication
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      use sepmodulecpack
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix), intent(in) :: A
      double precision, intent(in) :: x(A%neq)
      double precision :: y(A%neq)

!     A              i    Matrix A
!     x              i    vector to be multiplied
!     y              o    result Ax
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPMATOUTVEC   Compute y = A x, where A is the part of the matrix
!                    consisting of the off-diagonal block (parallel)
!     SEPMATVEC      Matrix vector multiplication for serial matrices
!     SEPSOLCOPSOL   Send parts of local solution array to neighbor blocks
!                    Receive these solution arrays and store in usolglob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmatvec1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmatvec1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( parallel .and. typecoupl==1 ) then

!     --- parallel case
!         Copy xglob from other blocks and send x to other blocks

         call sepsolcopsol ( x, xglob )

!        --- Standard part matrix vector multiplication

         call sepmatvec ( A, x, y )

!        --- Extra part, due to parts of vector stored in neighbor blocks

         call sepmatoutvec ( A, xglob, y )

      else

!     --- serial case

         call sepmatvec ( A, x, y )

      end if  ! ( parallel )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmatvec1'

      end if  ! ( debug )
      call erclos ( 'sepmatvec1' )

      end subroutine sepmatvec1
