      subroutine sepmaver1 ( A, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Matrix vector multiplication y_i = S_ii*x_i
!     In case of A%inclbc: y_i = y_i + S_pi*x_p
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_vector_multiplication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematr
      use sepmodulekprob
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (globmat), intent(in) :: A
      double precision, intent(in) :: x(nusol)
      double precision :: y(nusol)

!     A              i    matrix of type mat to be used in multiplication
!     x              i    vector to be multiplied
!     y              o    result vector of multiplication
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPMATVEC1     Matrix vector multiplication for internal matrix (real)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmaver1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmaver1'
         write(irefwr,*) 'A%diagonal, A%intmul', A%diagonal, A%intmul
         write(irefwr,1) 'indprh, nbound, nrow', indprh, nbound, A%nrow
  1      format ( a, 1x, (10i11) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( .not. A%intmul ) then

!     --- No internal multiplication

         y(1:A%nrow) = 0d0

      else if ( A%diagonal ) then

!     --- Matrix is diagonal matrix, treat separately

         y(1:A%nrow) = A%S_ii(1:A%nrow)*x(1:A%nrow)

      else

!     --- Matrix is standard matrix

         call sepmatvec1 ( A%S_i, x, y )

      end if  ! ( A%diagonal )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( y, A%nrow, 'y' )
         write(irefwr,*) 'End sepmaver1'

      end if  ! ( debug )
      call erclos ( 'sepmaver1' )

      end subroutine sepmaver1
