      subroutine sepmaver2 ( A, x, y )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Matrix vector multiplication y_i = S_ii*x_i
!     In case of A%inclbc: y_i = y_i + S_pi*x_p
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_vector_multiplication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulematr
      use sepmodulemat
      use sepmodulekprob
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (globmat), intent(inout) :: A
      double complex, intent(in) :: x(nusol)
      double complex :: y(nusol)

!     A              i    matrix of type mat to be used in multiplication
!     x              i    vector to be multiplied
!     y              o    result vector of multiplication
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPMULTBCOND2  Compute y = y + S_ip x_p in complex case
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmaver2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmaver2'
         write(irefwr,*) 'A%diagonal, A%intmul, A%inclbc', &
                          A%diagonal, A%intmul, A%inclbc
         write(irefwr,1) 'indprh, nbound', indprh, nbound
  1      format ( a, 1x, (10i11) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( A%diagonal ) then

!     --- Matrix is diagonal matrix, treat separately

         y(1:A%nrow) = A%S_ii(1:A%nrow)*x(1:A%nrow)

      else

!     --- Matrix is standard matrix

         if ( A%intmul ) then
            A%S_i%S =>A%S_ii
            y(1:A%nrow) = A%S_i*x(1:A%nrow)
         else
            y(1:A%nrow) = 0d0
         end if  ! ( A%intmul )
         if ( A%inclbc .and. nbound>0 ) call sepmultbcond2 ( A, x, y )

         if ( parallel ) then

            deallocate ( xglob, sendwork, stat = error )
            if ( error/=0 ) call erdealloc ( error, 'sendwork' )

         end if  ! ( parallel )

      end if  ! ( A%diagonal )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( y, A%nrow, 'y' )
         write(irefwr,*) 'End sepmaver2'

      end if  ! ( debug )
      call erclos ( 'sepmaver2' )

      end subroutine sepmaver2
