      subroutine sepmaverp1 ( matr, ix, iy, ichoice )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     First part of Matrix vector multiplication
!     Find compl and compvc
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_vector_multiplication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulemain
      use sepmodulematr
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer matr, ix, iy, ichoice

!     ichoice   i choice parameter. See sepmaver
!     ix       i  standard sepran array containing information of the
!                 vector x
!     iy       o  sepran array containing information of the vector y
!     matr     i  standard sepran array containing information
!                 concerning the matrix
!                 matr may be of type isol.
!                 then the matrix is supposed to be diagonal.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ipos
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     ipos           Position in array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCREASOL110  Activate or create arrays of type 110
!     SEPMAKECOMPL   Define vcompl as complex variant of input vector
!     SEPPARALLOC    Allocate some global arrays for parallel computing
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepmaverp1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepmaverp1'
  1      format ( a, 1x, (10i11) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Create array iy if it does not exist (otherwise activate)

      iprob = ks(ix)%iprob
      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(iprob)
      kprobloc => kprobpart%kprobloc
      nusol  = kprobloc(5)
      nbound = kprobloc(10)
      indprh = kprobloc(50)
      nrusol = kprobloc(66)

      call sepcreasol110 ( iy, 'iy', ks(ix)%compl )

      Smat => realmt(matr)
      jmethod = Smat%jmethod
      iseqim = Smat%intmatref

      if ( debug ) &
         write(irefwr,1) 'matr, jmethod, iseqim', matr, jmethod, iseqim

      IM => intmt(iseqim)
      typecoupl = IM%typecoupl
      if ( parallel .and. typecoupl==1 ) then
         kprobhinv => kprobpart%kprobhddinv
      else
         kprobhinv => kprobpart%kprobhinv
      end if  ! ( parallel .and. typecoupl==1 )

      if ( ks(ix)%compl==1 ) then
         call sepmakecompl ( ks(ix)%sol, nusol )
         xcompl => vcompl
         call sepmakecompl ( ks(iy)%sol, nusol )
         ycompl => vcompl
         if ( indprh==0 ) then
            xci => xcompl
         else
            allocate( xci(nusol), yci(nusol))
            xci = xcompl(kprobhinv)
            yci = 0d0
         end if  ! ( indprh==0 )

      else
         xreal => ks(ix)%sol
         yreal => ks(iy)%sol
         if ( indprh==0 ) then
            xri => xreal
            yri => yreal
         else
            allocate( xri(nusol), yri(nusol) )
            xri = xreal(kprobhinv(1:nusol))
            if ( ichoice==7 ) then
               yri = yreal(kprobhinv(1:nusol))
            else
               yri = 0d0
               if ( ichoice==8 ) yri(1:nrusol) = yreal(kprobhinv(1:nrusol))
            end if  ! ( ichoice==7 )
            xrp => xri(nusol-nbound+1:)
            yrp => yri(nusol-nbound+1:)
         end if  ! ( indprh==0 )

      end if  ! ( ks(ix)%compl )

      if ( parallel ) call sepparalloc

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepmaverp1'

      end if  ! ( debug )
      call erclos ( 'sepmaverp1' )

      end subroutine sepmaverp1
