      subroutine sepoverrlupp ( A, maxlevel )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-12-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array maxlevel for overrelaxation
!
! **********************************************************************
!
!                       KEYWORDS
!
!     constraint
!     linear_solver
!     overrelaxation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      use sepmodulekprob
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      double precision :: maxlevel(nusol)

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     maxlevel       o    Work array to store the constraint b(x)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: maxv

!     maxv           Constant constraint corresponding to maximum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     SEPFILLVALUE   set one degree of freedom in each nodal point of vector
!                    equal to constant
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1237   Variable coefficients not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepoverrlupp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepoverrlupp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill array maxlevel

      maxv = 0d0
      if ( A%imaximum>0 ) maxv = A%maxlev
      if ( A%imaximum>=0 ) then


         if ( A%idegfd==0 .or. nphys==1 .and. indprf==0 ) then

!        --- Constant value

            maxlevel = maxv

         else

!        --- Only one degree of freedom is applied

            maxlevel = rinfin
            call sepfillvalue ( maxlevel, A%idegfd, maxv )

         end if  ! ( A%idegfd==0 .or. nphys==1 .and. indprf==0 )

      else

!     --- imaximum < 0, not implemented

         call errsub ( 1237, 0, 0, 0 )

      end if  ! ( A%imaximum>=0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepoverrlupp'

      end if  ! ( debug )
      call erclos ( 'sepoverrlupp' )

      end subroutine sepoverrlupp
