      subroutine seppjfillA ( A, nlocdgfd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-04-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill information of submatrix into structure A of type rmatrix
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
 !     use sepmodulekprob
      use sepmoduleproj
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      integer, intent(in) :: nlocdgfd

!     A              i    Structure of type matrix containing the necessary
!                         information of the matrix
!     nlocdgfd       i    Number of degrees of freedom in the subsystem.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, icount, ncols, ishift
      integer, pointer :: rows(:), cols(:)
      double precision, pointer :: mat(:)

!     cols           Temporary array with column numbers
!     i              Counting variable
!     icount         Counter
!     ishift         Shift of specific variable with respect to diagonal
!     mat            temporary array with matrix
!     ncols          number of non-zero columns in row
!     rows           Temporary array with row numbers
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPCOMMATREV   Fill arrays irowsnew and icolsnew with upper triangular
!                    part if icolsii and irowsii contain the lower triangular
!                    part and vice versa
!     SEPMATREV      Copy matrix from rowwise to columnwise storage
!     SEPPRINMAT6    print compact matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppjfillA' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppjfillA'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      length = nlocdgfd+locintmt(1+nlocdgfd)
      A%neq = nlocdgfd
      A%nglob = A%neq
      allocate ( A%rows(nlocdgfd+1),A%cols(length), &
                 A%diag(A%neq), A%S(length),&
                 rows(nlocdgfd+1), cols(length), mat(length) )
      A%Mrows => A%rows
      A%Mcols => A%cols
      A%Mdiag => A%diag
      A%MD => A%S

!     --- Fill A%rows, A%cols and A%diag

      icount = 0
      ishift = nlocdgfd+1
      do i = 1, nlocdgfd
         ncols = locintmt(i+1)-locintmt(i)  ! number of columns
         rows(i) = icount
         cols(icount+1:icount+ncols) = &
            locintmt(ishift+locintmt(i)+1:ishift+locintmt(i+1))
         mat(icount+1:icount+ncols) = &
            locmat(locintmt(i)+1+nlocdgfd:locintmt(i+1)+nlocdgfd)
         icount = icount+ncols+1
         cols(icount) = i         !diagonal
         mat(icount) = locmat(i)
      end do
      rows(nlocdgfd+1) = icount

!     --- Change to columnwise storage

      call sepcommatrev ( A%rows, A%cols, rows, cols, nlocdgfd )
      call sepmatrev ( A%rows, A%S, rows, cols, mat, nlocdgfd )
      A%diag = A%rows(1:size(rows)-1)+1
      deallocate ( rows, cols, mat )

1000  if ( debug ) then

!     --- Debug information

         call sepprinmat6 ( A )
         write(irefwr,*) 'End seppjfillA'

      end if  ! ( debug )
      call erclos ( 'seppjfillA' )

      end subroutine seppjfillA
