      subroutine sepprecsymgs ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the preconditioning matrix for the CG-process
!     iprec = 4,  Symmetric Gauss-Seidel
!     Old subroutine prcg01prec4
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, k, id

!     i              Counting variable
!     id             Position of diagonal element i
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     674   The desired preconditioning matrix does not exist
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprecsymgs' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprecsymgs'

      end if
      if ( ierror/=0 ) go to 1000
      do i = 1, A%neq
         id = A%diag(i)
         if ( A%adapt<=0d0 .and. A%S(id)<=0d0 ) then
            call errint ( i, 1 )
            call erreal ( max(A%adapt,A%S(id)), 1 )
            call errsub ( 674, 1, 1, 0)
            go to 1000
         end if

!        --- Pre en post multiply the preconditioning matrix by sqrt(D^-1)
!            which implies  sqrt(D^-1) L U sqrt(D^-1) gets diagonal element 1
!            (scaling)

         A%MD(id) = sqrt ( max(A%S(id),A%adapt) )
         do k = A%rows(i)+2, A%rows(i+1)
            A%MD(k) = A%S(k) / A%MD(id)
         end do
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprecsymgs'

      end if
      call erclos ( 'sepprecsymgs' )

      end
