      subroutine seppreczeis10 ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     computes the preconditioning matrix for the CGS-process
!     ipreco = 2, Eisenstat preconditioning
!     Old subroutine pzcg02_2
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     preconditioning
!     complex
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmodulecmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, jp, point(A%neq), iposu
      double complex :: sum1
      integer, pointer :: diag(:), col(:)

!     col            Column numbers of matrix
!     diag           Positions of diagonal elements of the large matrix
!     i              Counting variable
!     iposu          point(j)
!     j              Counting variable
!     jp             position of element i,j in matrix
!     point          Array with positions of specific matrix elements
!     sum1           Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     674   The desired preconditioning matrix does not exist
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppreczeis10' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppreczeis10'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      diag => A%diag
      col => A%cols

!     --- Initialize array point, with next position referring to U

      do i = 1, A%neq
         point(i) = diag(i)+1
      end do

!     --- ipreco = 2,  Eisenstat preconditioning

      do i = 1, A%neq

         sum1 = A%S(diag(i))
         do jp = A%rows(i)+1, diag(i)-1

            j = col(jp)  ! column number
            iposu = point(j)
            point(j) = point(j)+1

            sum1 = sum1 - A%S(jp) * A%S(iposu) / A%MD(j)**2

         end do

         if ( abs(sum1)<=0d0 ) then
            call errint ( i, 1 )
            call erreal ( abs(sum1), 1 )
            call errsub ( 674, 1, 1, 0)
            go to 1000
         end if

         A%MD (i) = sqrt ( sum1 )

      end do

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( A%MD, A%neq, 'A%MD' )
         write(irefwr,*) 'End seppreczeis10'

      end if  ! ( debug )
      call erclos ( 'seppreczeis10' )

      end subroutine seppreczeis10
