      subroutine sepprofforwgg ( A, usol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-01-2021
!
!   copyright (c) 2021-2021  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     solve system of linear equations by gaussian elimination
!     where the matrix has been decomposed by subroutine sepprofgg
!     Forward solve, hence Gy = f
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      double precision, intent(inout) :: usol(A%neq)

!     A              i    Matrix A
!     usol          i/o   At input: right-hand-side vector
!                         At output: solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: pi, i, ii
      double precision :: som

!     i              Counting variable
!     ii             Position of diagonal element ii
!     pi             Column number of at most left non-zero element in row i
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ddot

!     DDOT           (CDOT)     Inner product of two vectors
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!

      do i = minzerorhs, A%rowmax
         pi=A%Mcols(i)
         ii=A%Mrows(i)
         som=ddot(i-pi,A%MD(ii-i+pi),1,usol(pi),1)
         usol(i)=(usol(i)-som)/A%MD(ii)
      end do
      end subroutine sepprofforwgg
