      subroutine sepprofilesol ( A, usol, rhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     solve system of linear equations by gaussian elimination
!     where the matrix has been decomposed by subroutine sepprofun
!     This implies that the matrix is supposed to be a real
!     profile matrix
!     Old subroutine prspun
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulesolve
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      double precision :: usol(A%neq)
      double precision, intent(in) :: rhsd(A%neq)

!     A              i    Matrix A
!     rhsd           i    right-hand-side vector
!     usol           o    solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (intmatrix), pointer :: IM
      double precision :: timebefore(2)

!     IM             Refers to intmt(intmat)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPPROFSOLGG   solve system of linear equations by gaussian elimination
!                    where the matrix has been decomposed by sepprofgg
!     SEPPROFSOLSM   solve system of linear equations by gaussian elimination
!                    where the matrix has been decomposed by sepprofsm
!     SEPPROFSOLUN   solve system of linear equations by gaussian elimination
!                    where the matrix has been decomposed by sepprofun
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprofilesol' )
      if ( itime>0 ) call ersettime ( timebefore )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprofilesol'
         call prinrl ( rhsd, A%neq, 'rhsd' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(iseqim)
      if ( IM%symmetric ) then

!     --- symmetric matrix

         if ( ipos==1 ) then

!        --- Matrix positive definite

            A%rowmin = 1
            A%rowmax = A%neq
            minzerorhs = 1
            usol = rhsd
            do while ( abs(usol(minzerorhs))<=0d0 )
               minzerorhs = minzerorhs+1
            end do  ! while ( abs(usol(minzerorhs))<=0d0 )

            call sepprofforwgg ( A, usol )
            call sepprofbackgg ( A, usol )

         else

!        --- General symmetric matrix

            call sepprofsolsm ( A, usol, rhsd )

         end if  ! ( ipos==1 )

      else

!     --- asymmetric matrix

         call sepprofsolun ( A, usol, rhsd )

      end if  ! ( IM%symmetric )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( usol, A%neq, 'usol/1' )
         write(irefwr,*) 'End sepprofilesol'

      end if  ! ( debug )
      call erclmn ( 'sepprofilesol', timebefore )
      end subroutine sepprofilesol
