      subroutine sepsimplestart ( rnorm, rferr, normrhsd, A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-04-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute initial norm, adapted eps and print start
!     Produce matlab output if required
!     Old subroutine prsimplestart
! **********************************************************************
!
!                       KEYWORDS
!
!     simple
!     linear_solver
!     matrix
!     real
!     krylov
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmodulesimple
      use sepmodulemat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(out) :: rnorm, rferr
      double precision, intent(in) :: normrhsd
      type (rmatrix) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     normrhsd       i    Norm of right-hand side vector
!     rferr          o    Maximal accuracy possible for this matrix, due to
!                         round-off
!     rnorm          o    Norm of initial residual
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, iref
      double precision :: normu
      logical :: matlab

!                    they are not
!     i              Counting variable
!     iref           File reference number
!     matlab         If true matlab input is generated
!     normu          norm of u-vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: inisetnewref

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     INIRMREF       Remove entry iref from the set of reference numbers used
!     INISETNEWREF   Create new reference number
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsimplestart' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsimplestart'
         call ertrace
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      matlab = .false.
      if ( matlab ) then
         iref = inisetnewref()
         open ( unit=iref, file = 'fu.dat' )
         do i = 1, nu
            write(iref,10) ru(i)
 10         format ( e15.8 )
         end do
         close ( unit=iref )
         open ( unit=iref, file = 'fp.dat' )
         do i = 1, np
            write(iref,10) rp(i)
         end do
         close ( unit=iref )
         call inirmref ( iref )
      end if

      rnorm = norm ( resup, A )
      normu = norm ( ru, Q )
      rferr = 1d3*normrhsd*epsmac
      if ( debug ) write(irefwr,2) 'norm, normu', rnorm, normu

!     --- Check if accuracy has been reached

      if ( debug ) write(irefwr,*) 'ireler, eps', A%irelerr, A%eps
      if ( A%irelerr==3 ) then

!     --- Relative error compared to rhs

         A%eps = A%eps * normrhsd

      end if

      if ( A%iprint==1 ) then

!     --- iprint = 1,  Print information

         write ( irefwr, 110 ) rnorm
110      format ( ' The 2-norm of the initial residual', ss, 1pe17.2e2 )

      else if ( A%iprint>=2 ) then

!     --- iprint>=2,  Maximal output

         write ( irefwr, 120 )
120      format ( '  Iter    residual  res_u     res_p     P^-1 res ', &
                  '   iter_u   iter_p '/ &
                  ' number   |r_k|_2   |ru_k|_2  |rp_k|_2  |P^-1r_k|_2')

      end if

      A%nmax = inpsol(5,1)
      if ( A%nmax==0 ) A%nmax = A%neq

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepsimplestart'

      end if  ! ( debug )
      call erclos ( 'sepsimplestart' )

      end
