      subroutine sepsolvedefcor
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-10-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     apply defect correction
!     Old subroutine solveldefcor
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematr
      use sepmodulevecs
      use sepmodulemain
      use sepmodulesolve
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: residu
      double precision, pointer :: resid(:), ucor(:), rinsl(:)
      logical :: finished
      type (rmatrix), pointer :: A2
      integer, pointer :: inpsl(:)

!     A2             Structure of type matrix containing the necessary
!                    information of the second matrix
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     inpsl          Local vector inpsol(:,1)
!     resid          residual vector
!     residu         Norm of residual
!     rinsl          Local vector rinsol(:,1)
!     ucor           correction vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPCOPYMATCS   Copy symmetric compact matrix in type matrix
!     SEPLINSOLREAL  Solve system of linear equations (real matrix)
!     SEPMATVEC      Matrix vector multiplication for serial matrices
!     SEPPRINTMAT    Print matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsolvedefcor' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsolvedefcor'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      inpsl => inpsol(:,1)
      rinsl => rinsol(:,1)

      if ( inpsl(6)>=4 ) then

!     --- inpsol(6) (iprint)>=4, print matrix and right-hand side

         call sepprintmat ( iseqmt2 )
         call prinrl ( rhsdbuf2, size(rhsdbuf2), 'right-hand side 2' )

      end if
      A2 => realmt(iseqmt2)%S_i

      call sepcopymatcs ( A2 )
      if ( ierror/=0 ) go to 1000

      niter = 0
      finished = .false.
      rinsl(1) = rinsl(10)

      allocate ( resid(A2%neq), ucor(A2%neq) )

400   if ( .not. finished ) then

!     --- While loop defect correction

         niter = niter + 1

!        --- Compute residual -(matr usol - rhsd(2)), store in iresid

         call sepmatvec ( A2, usol, resid )
         resid =  rhsd2(1:A2%neq) - resid

!        --- Solve: A2 ucorr = residual

         if ( inpsl(3)>0 ) then

!        --- inpsol(3)>0, iterative cg-type method

            inpsl(1) = 0
            inpsl(22) = 2

         end if
         call seplinsolreal ( A2, ucor, resid )
         if ( inpsl(6)>=1 ) write(irefwr,210) itotit
210      format ( ' Accumulated number of iterations ', i8 )

!        --- Compute norm of correction

         residu = norm2(ucor)

!        --- Add ucorr to usol

         usol = usol+ucor

         if ( inpsl(6)>=2 ) then
            write(irefwr,410) niter, residu
410         format(' Iteration ', i4,'  Correction is ', ss, &
                     1pe11.2e2 )
         end if
         if ( inpsl(21)>1 .and. residu<rinsl(9) ) then
            if ( inpsl(6)>=1 ) write(irefwr,420) niter
420         format(' Defect correction finished after ',i4, &
                   ' iterations' )
            finished = .true.
         else if ( niter>=inpsl(21) ) then
            finished = .true.
         end if
         go to 400

      end if

!     --- Delete temporary arrays

      deallocate ( resid, ucor )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepsolvedefcor'

      end if  ! ( debug )
      call erclos ( 'sepsolvedefcor' )

      end subroutine sepsolvedefcor
