      subroutine sepsolvepar ( A, usol, rhsd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 23-02-2016 New calls to several subroutines
!        version  1.0  date 20-05-2015
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve system of linear equations in a parallel environment
!
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
!     solve
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulematr
      use sepmodulemat
      use sepmodulekprob
      use sepmodulesolve
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: A
      double precision :: usol(A%neq)
      double precision, intent(in) :: rhsd(A%neq)

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
!     rhsd           i    right-hand-side vector
!     usol           o    solution vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPMAKEPREC    Create preconditioning matrix if necessary
!     SEPPARGCR      Solve linear system of equation by the preconditioned CGR
!                    method. Parallel case (block jacobi)
!     SEPSOLCOPSOL   Send parts of local solution array to neighbor blocks
!                    Receive these solution arrays and store in usolglob
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepsolvepar' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepsolvepar'
         write(irefwr,1) 'nusol, nrusol, ninterface', &
                          nusol, nrusol, ninterface
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'nrusol', nrusol

!     --- Copy xglob from other blocks and send usol to other blocks

      call sepsolcopsol ( usol, xglob )

!     --- Select a standard (=default) preconditioner:
!                  Eisenstat when symmetric
!                  ILU when asymmetric or parallel

      if ( debug ) then
         call prinrl ( usol, nrusol, 'usol_start' )
         call prinrl ( xglob, nglob, 'xglob' )
      end if  ! ( debug )

!     --- Create preconditioning matrix if necessary

      if ( keep<=1 ) call sepmakeprec ( A )

!     --- Store type of preconditioner in type matrix

      if ( A%type_prec>0 .and. debug ) call prinrl ( A%S, nrusol, 'diag_pre' )

!     --- Solve system of equations

      if ( debug ) write(irefwr,1) 'typeparsolv', typeparsolv

!     --- typeparsolv = 2:  gcr method

      call seppargcr ( A, usol, rhsd )

!     --- Extend usol to xglob

      call sepsolcopsol ( usol, xglob )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepsolvepar'

      end if  ! ( debug )
      call erclos ( 'sepsolvepar' )

      end subroutine sepsolvepar
