      subroutine sepzrenumsolbc ( solin, ichoice )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-03-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Renumber solution vector back to original form
!     Complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     complex
!     reordering
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ichoice
      double complex :: solin(nusol)

!     ichoice        i    If > 0 it concerns the solution vector
!     solin         i/o   Solution vector to be transformed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPLOCTRNCM    Compute (back-)transformation in case of local
!                    transformations ( complex case )
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepzrenumsolbc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepzrenumsolbc'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      npboun = kprobloc(34)
      if ( npboun>0 ) then

!     --- npboun>0; compute effect of periodical boundary conditions

         kprobs => kprobpart%kprobs
         rprobs => kprobpart%rprobs
         solin(nrusol+1:nrusol+npboun) = rprobs(:,2)*solin(kprobs)+rprobs(:,1)

      end if  ! ( npboun>0 )

!     --- Transform essential boundary conditions in case of local transform

      if ( npltra>0 .and. itrans==0 ) then
         indprh = 0
         call seploctrncm ( 2, solin )
         indprh = kprobloc(50)
      end if  ! ( npltra>0 .and. itrans==0 )

      if ( indprh>0 ) solin(kprobhinv) = solin  ! reorder solin back

      if ( indprh>0 ) then

!     --- indprh > 0, hence renumbering. Copy usol back

         do i = 1, nusol
            usolbuf(2*i-1) = real(solin(i), kind=8)
            usolbuf(2*i) = dimag(solin(i))
         end do  ! i = 1, nusol

      end if  ! ( indprh>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepzrenumsolbc'

      end if  ! ( debug )
      call erclos ( 'sepzrenumsolbc' )

      end subroutine sepzrenumsolbc
