      subroutine sepzsolve ( A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 23-02-2016 New calls to several subroutines
!        version  1.0  date 02-04-2015
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve linear system of equations
!     Complex matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     solve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      use sepmodulematstr
      use sepmodulecmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (cmatrix) :: A

!     A             i/o   Structure of type matrix containing the necessary
!                         information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPCKRYLOV     Solve system of linear equations by a krylov method
!     SEPZPROFILE    Solve system of complex linear equations by
!                    Gaussian elimination
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2987   Iterative solver not available for real matrices
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepzsolve' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepzsolve'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case (typesolv)

      case (0)

!     --- typesolv =0, profile method

         call sepzprofile ( A )

      case (1:4, 7, 9, 10, 17, 31 )

!     --- typesolv = 1-4, 7, 9, 10, 17, standard krylov method

         call sepckrylov ( A )

      case default

!     --- Solver not available

         call errint ( typesolv, 1 )
         call errsub ( 2987, 1, 0, 0 )

      end select  ! case  (typesolv)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepzsolve'

      end if  ! ( debug )
      call erclos ( 'sepzsolve' )

      end
