      subroutine solveldefcor ( matr, irhsdlc, isol, A )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 12-08-2014 New call to prcong
!        version  1.2  date 03-04-2013 New call to prcong
!        version  1.1  date 05-12-2012 Long integers
!
!   copyright (c) 2008-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     apply defect correction
!
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematr
      use sepmodulevecs
      use sepmodulemain
      use sepmodulesolve
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*), irhsdlc(*), matr(*)
      type (rmatrix) :: A

!     A              i    Structure of type matrix containing the necessary
!                         information of the matrix
!     irhsdlc        i    Standard SEPRAN array, containing information of the
!                         large vector
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution array
!     matr           i    Standard SEPRAN array, containing information of the
!                         large matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iretur, iwork(4), iinvec(6), idummy, icor, iresid
      double precision :: work(2), rinvec(2), residu
      logical :: finished

!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     icor           Refers to correction vector
!     idummy         Dummy parameter
!     iinvec         User input array for subroutine MANVEC
!                    For a description see subroutine MANVEC
!     iresid         Refers to residual vector
!     iretur         Return value of subroutine INTX02
!                    See subroutine INTX02
!     iwork          integer work array
!     residu         Computed residual
!     rinvec         Real user input array for subroutine MANVEC
!                    The contents depend on iinvec
!     work           Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRESIDUAL      Compute residual of system of linear equations
!     PRINRL         Print 1d real vector
!     SEPCOPYVC      Copy one SEPRAN vector into another one
!     SEPLINSOLREAL  Solve system of linear equations (real matrix)
!     SEPMANVEC      Vector manipulation subroutine
!     SEPPRINTMAT    Print matrix
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
!     SEPSOLDIR      Actual body of subroutine SOLVE
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'solveldefcor' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from solveldefcor'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icor = 0
      iresid = 0

      if ( inpsol(6,1)>=4 ) then

!     --- inpsol(6) (iprint)>=4, print matrix and right-hand side

         call sepprintmat ( matr(2) )
         usoltemp =>  ks(irhsdlc(2))%sol
         call prinrl ( usoltemp, size(usoltemp), 'right-hand side 2' )

      end if

      niter = 0
      finished = .false.
      rinsol(1,1) = rinsol(10,1)

400   if ( .not. finished ) then

!     --- While loop defect correction

         niter = niter + 1

!        --- Compute residual -(matr usol - rhsd(2)), store in iresid

         call presidual ( matr(2), isol(iseqsol), &
                          irhsdlc(2), iresid, 0 )
         if ( icor==0 ) call sepcopyvc ( irhsdlc, icor )

!        --- Solve: matr ucorr = residual

         if ( inpsol(3,1)==0 ) then

!        --- inpsol(3)=0, profile method

            call sepsoldir ( iseqmt, icor, iresid )

         else if ( inpsol(3,1)>0 ) then

!        --- inpsol(3)>0, iterative cg-type method

            inpsol(1,1) = 0
            inpsol(22,1) = 2
            call  seplinsolreal ( A, icor )
            if ( inpsol(6,1)>=1 ) write(irefwr,210) itotit
210         format ( ' Accumulated number of iterations ', i8 )

         end if

!        --- Compute norm of correction

         iinvec(1) = 6
         iinvec(2) = 5
         iinvec(3) = 0
         iinvec(4) = 1
         iinvec(5) = 0
         iinvec(6) = 2

         call sepmanvec ( iinvec, rinvec, icor, icor, icor )

!        --- Add ucorr to usol

         iinvec(1) = 3
         iinvec(2) = 27
         iinvec(3) = 0
         residu = rinvec(1)
         rinvec(1) = 1d0
         rinvec(2) = 1d0

         call sepmanvec ( iinvec, rinvec, isol(iseqsol), icor, isol )
         if ( inpsol(6,1)>=2 ) then
            write(irefwr,410) niter, residu
410         format(' Iteration ', i4,'  Correction is ', ss, &
                     1pe11.2e2 )
         end if
         if ( inpsol(21,1)>1 .and. residu<rinsol(9,1) ) then
            if ( inpsol(6,1)>=1 ) write(irefwr,420) niter
420         format(' Defect correction finished after ',i4, &
                   ' iterations' )
            finished = .true.
         else if ( niter>=inpsol(21,1) ) then
            finished = .true.
         end if
         go to 400

      end if

!     --- Delete temporary arrays

      call seprmsolinfo ( icor )
      call seprmsolinfo ( iresid )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End solveldefcor'

      end if  ! ( debug )
      call erclos ( 'solveldefcor' )

      end subroutine solveldefcor
