      subroutine to0000 ( ichois )
! ======================================================================
!
!        programmer    Guus Segal
!        version 10.0  date 24-07-2015 Use sepmodulebuild
!        version  9.1  date 08-08-2014 Extra debug
!        version  9.0  date 23-03-2011 Extra parameter kprob
!
!   copyright (c) 1984-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill arrays index1sub and index2 for the element subroutines
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     matrix
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ichois

!     ichois         i    Choice parameter, possibilities
!                         1  nunkp=1, number of degrees of freedom is constant
!                         2  nunkp=1, number of degrees of freedom in the nodal
!                            points not constant
!                         3  nunkp>1,  number of degres of freedom is constant
!                         4  nunkp>1,  number of degrees of freedom not constant
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: jcount, j, kchois, inodp, inusol, inunkp, k, i

!     i              Counting variable
!     inodp          Nodal point number
!     inunkp         Number of degrees of freedom in first point
!     inusol         Position of unknown in array USOL
!     j              Counting variable
!     jcount         Help parameter to compute the centroid of the element
!     k              Counting variable
!     kchois
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0000' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0000'
         write(irefwr,1) 'inpelm, icount, nunkp', inpelm, icount, nunkp
         write(irefwr,1) 'ichois, indprf, indpra', ichois, indprf, indpra
         call prinin ( kprobatmp, inpelm, 'kprobatmp' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      jcount=0
      inunkp = nunkp

      kchois=ichois
      if ( itype>0 .and. ielgrp<=nelgrp ) then
         kchois=ichois+4
      end if

      if ( debug ) write(irefwr,1) 'kchois', kchois
      if ( itype==-2 ) then

!     --- type number = -2, boundary conditions of the type u equals
!                           unknown constant

         jcount = inpelm
         if ( nunkp==1 ) then
            index2(1:inpelm)=index1(1:inpelm)
         else if ( ichois==3) then
            index2(1:inpelm)=(index1(1:inpelm)-1)*nunkp+iboun
         else
            index2(1:inpelm)=kprobf(index1(1:inpelm))+iboun
         end if

      else

!     --- other type numbers

         select case ( kchois )

         case(1, 5)

!        --- ichois = 1

            jcount=inpelm
            index2(1:inpelm)=index1(1:inpelm)

         case(2)

!        --- ichois = 2

            do j = 1,inpelm
               inodp=index1(j)

               inusol=sepgetinusol(inodp, inunkp)
               if ( inunkp/=0 ) then
                  jcount=jcount+1
                  index2(jcount)=inusol+1
               end if
            end do

         case(3)

!        --- ichois = 3

            do j = 1,inpelm
               inodp=index1(j)
               inusol=sepgetinusol(inodp, inunkp)
               do k=1,nunkp
                  index2(jcount+k)=inusol+k
               end do
               jcount=jcount+nunkp
            end do

         case(4)

!        --- ichois = 4

            do j = 1,inpelm
               inodp=index1(j)
               inusol=sepgetinusol(inodp, inunkp)

               do k=1,inunkp
                  index2(jcount+k)=inusol+k
               end do
               jcount=jcount+inunkp
            end do

         case(6)

!        --- ichois = 2   itype>0

            if ( indprf>0 ) then

!           --- indprf>0

               do j = 1,inpelm
                  inodp=index1(j)
                  inusol=kprobf(inodp)
                  inunkp=kprobatmp(j)
                  if ( inunkp>0 ) then
                     jcount=jcount+1
                     index2(jcount)=inusol+1
                  end if
               end do

            else

!           --- indprf = 0

               do j = 1,inpelm
                  inodp=index1(j)
                  inusol=inodp-1
                  if ( indpra>0 ) inunkp=kprobatmp(j)
                  if ( inunkp>0 ) then
                     jcount=jcount+1
                     index2(jcount)=inusol+1
                  end if
               end do

            end if

         case(7)

!        --- ichois = 7

            do j = 1,inpelm
               inodp=index1(j)
               inusol=(inodp-1)*nunkp
               if ( indpra>0 ) inunkp=kprobatmp(j)
               do k=1,inunkp
                  index2(jcount+k)=inusol+k
               end do
               jcount=jcount+inunkp
            end do

         case(8)

!        --- ichois = 8   itype>0

            if ( indprf>0 ) then

!           --- indprf>0

               do j = 1,inpelm
                  inodp=index1(j)
                  inusol=kprobf(inodp)
                  inunkp=kprobatmp(j)
                  do k=1,inunkp
                     index2(jcount+k)=inusol+k
                  end do
                  jcount=jcount+inunkp
               end do

            else

!           --- indprf = 0

               do j = 1,inpelm
                  inodp=index1(j)
                  inusol=(inodp-1)*nunkp
                  if ( indpra>0 ) inunkp=kprobatmp(j)
                  do k=1,inunkp
                     index2(jcount+k)=inusol+k
                  end do
                  jcount=jcount+inunkp
               end do

            end if

         end select  ! case ( kchois )

      end if

1000  icount=jcount

      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icount', icount
         call prinin ( index2, jcount, 'index2' )
         write(irefwr,*) 'End to0000'

      end if  ! ( debug )
      call erclos ( 'to0000' )

      end
