      subroutine to0023 ( ichois, a, b, c, ipoint, &
                          nusol, nbound, indprh, kprobh, iconst )
! ======================================================================
!
!        programmer    Jaap van der Zanden / Guus Segal
!        version 10.8  date 17-05-2008 Remove ichois = 3, 24
!        version 10.7  date 10-02-2004 Debug statements
!        version 10.6  date 14-03-2002 Extension for parallel
!
!   copyright (c) 1984-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     algebraic manipulations with real vectors
!
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulecomio
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois, ipoint, nusol , nbound, indprh, kprobh(*), &
              iconst(*)
      double precision a, b, c

!     a             i/o   Input output constant (see ichois)
!     b             i/o   Input output constant (see ichois)
!     c             i/o   Input output constant (see ichois)
!     ichois         i    Choice parameter. Possible values
!                          1  subtract : u3 = u1-a
!                          2  determine average of u1 = =>a
!                          3  linear combination u3 = a*u1+b*u2
!                          6  a = min and b = max of u1
!                          7  inner product sum(u1*u2), ipoint = nusol
!                         10  inner product without nbound
!                         11  Compute the minimum and maximum of u1 and the
!                             corresponding co-ordinates.
!                             In this case only one degree of freedom per point
!                             is permitted. Furthermore some parameters have the
!                             following special meaning:
!                             u1:  i vector from which min and max must be
!                                      computed
!                             u2:  i array containing the co-ordinates
!                             u3:  o Output array of length 6, containing the
!                                      co-ordinates xmin, ymin, zmin, xmax, &
!                                      ymax, zmax
!                             nusol: i Number of nodal points
!                             ipoint:i Dimension of the space ( 1, 2 or 3)
!                         12  See 11, however the absolute value of u1 is used
!                         15  Compute norm of VEC1-VEC2
!                         16  Compute norm of VEC1
!                         17,18  reserved in to0035
!                         19  Compute u3 as function of u1 and u2.
!                             The SEPRAN subroutine FUNADA is called
!                         24  Linear combination
!                             u3 = a * u1 + b * u2
!                             with constraint: u3(i)>=c
!                         25  "Product of two vectors"
!                             u3 = a * u1 * u2
!                         26  Point-wise inner-product of two vectors:
!                             u out(i) = sum_j u1(i)_j + u2(i)_j
!                         27  Limit u1 between rinput(1) (minimum) and
!                             rinput(2) (maximum)
!                         43  u3 = function of u1
!                             The type of functions is stored in iconst(1)
!                             Possible values:
!                             1: log
!     iconst         o    Input array for ICHOIS = 15, 16
!                         ICONST(1) contains the type of norm to be computed
!                         Possible values:
!                         1       l1-norm
!                         2       l2-norm
!                         3       max-norm
!                         4       l1-norm / number of degrees of freedom
!                         5       l2-norm / number of degrees of freedom
!     indprh         i    indication if kprob part h has been filled (>0)
!                         or not (0)
!     ipoint        i/o   nusol for ichois = 2, 7, 10
!     kprobh         i    KPROB part h: contains the new numbering of the
!                         unknowns
!     nbound         i    Number of essential boundary conditions
!     nusol          i    Total number of unknowns
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, imin, imax, j, nusolglob
      double precision vv, aglob

!     aglob          Global value of a
!     i              Counting variable
!     imax           Nodal point number for which the maximum value is reached
!     imin           Nodal point number for which the minimum value is reached
!     j              Help variable
!     nusolglob      Global value of nusol
!     vv             Value of vector in a node
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ddot, dasum

!     DASUM          Blas subroutine computing the sum of absolute values of
!                    vector
!     DDOT           (CDOT)     Inner product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     FUNADA         User function to perform algebraic manipulations (real)
!     FUNALG         User function to perform algebraic manipulations (real)
!     PRPARGLMAX     This function performs a global max of a quantitity
!                    or vector across all processes
!     PRPARGLSUM     This function performs a global sum of a quantitity
!                    or vector across all processes
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       1   ICHOIS is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0023' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0023'
         write(irefwr,1) 'indprh, ichois, nusol', indprh, ichois, nusol
         write(irefwr,2) 'a, b, c', a, b, c
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( indprh==0 ) then

!     --- No renumbering required

         select case ( ichois )
         case(1)

!        --- subtract a constant from u1

            u3 = u1 - a

         case(2)

!        --- determine average of u1 = =>a

            a = sum(u1)/nusol
            ipoint = nusol

         case(6)

!        --- determine min and max of u1 = =>a = min, b = max

            a = minval(u1)
            b = maxval(u1)

         case(7)

!        --- inner product of u1 and u2  (ichois = 7)

            a = ddot(nusol, u1, 1, u2, 1)
            ipoint = nusol

         case(10)

!        --- inner product of u1 and u2, without effect of boundary
!            conditions

            a = ddot(nusol, u1, 1, u2, 1)
            ipoint = nusol

         case(11, 12)

!        --- Ichois = 11, 12, compute minimum and maximum, plus corresponding
!                              co-ordinates
!            First compute minimum, maximum and corresponding nodal points

            a =  rinfin
            b = -rinfin
            imin = 0
            imax = 0

            do i = 1, nusol
               if ( ichois==11 ) then
                  vv = u1(i)
               else
                  vv = abs( u1(i) )
               end if
               if ( vv<a ) then

!              --- vv<a, new minimum found

                  a    = vv
                  imin = i

               end if

               if ( vv>b ) then

!              --- vv>b, new maximum found

                  b    = vv
                  imax = i
               end if

            end do

!           --- Next store the corresponding co-ordinates

            do i = 1, ipoint
               u3(i)   = u2 ( (imin-1) * ipoint + i )
               u3(i+3) = u2 ( (imax-1) * ipoint + i )
            end do

         case(13)

!        --- ichois = 13,  u3 is a function of u1 and u2 through the
!                          subroutine funalg

            do i = 1, nusol
               call funalg ( u1(i), u2(i), u3(i) )
            end do

         case(15)

!        --- ichois = 15,  a = || VEC1-VEC2 ||

            if ( debug ) write(irefwr,1) ' iconst(1), nusol', iconst(1), nusol
            a = 0d0
            if ( iconst(1)==1 .or. iconst(1)==4 ) then

!           --- iconst(1) = 1 or 4 L1 norm, i.e. a = sum |VEC1(i)-VEC2(i)|

               do i = 1, nusol
                  a = a + abs ( u1(i) - u2(i) )
               end do

            else if ( iconst(1)==2 .or. iconst(1)==5 ) then

!           --- iconst(1) = 2 or 5 L2 norm, i.e. a = sum (VEC1(i)-VEC2(i)) ** 2

               do i = 1, nusol
                  a = a + ( u1(i) - u2(i) ) ** 2
               end do

            else

!           --- iconst(1) = 3 max norm, i.e. a = max (VEC1(i)-VEC2(i))

               do i = 1, nusol
                  a = max ( a , abs( u1(i) - u2(i) ) )
               end do

            end if
            if ( parallel ) then

!           --- parallel case, perform global sum or global max

               if ( iconst(1)==3 ) then

!              --- Parallel max

                  call prparglmax ( a, aglob, 1, SPDOUBLE )

               else

!              --- Parallel sum

                  call prparglsum ( a, aglob, 1, SPDOUBLE )

               end if
               a = aglob
               if ( iconst(1)==4 .or. iconst(1)==5 ) then

!              --- Compute parallel nusol

                  call prparglsum ( nusol, nusolglob, 1, SPINT )
                  a = a/nusolglob

               end if
               if ( iconst(1)==2 .or. iconst(1)==5 ) a = sqrt(a)

            else

!           --- standard case

               if ( iconst(1)==4 .or. iconst(1)==5 ) a = a/nusol
               if ( iconst(1)==2 .or. iconst(1)==5 ) a = sqrt(a)

            end if

         case(16)

!        --- ichois = 16,  a = || VEC1 ||

            if ( iconst(1)==1 .or. iconst(1)==4 ) then

!           --- iconst(1) = 1 or 4 L1 norm, i.e. a = sum |VEC1(i)|

               a = dasum ( nusol, u1, 1 )

            else if ( iconst(1)==2 .or. iconst(1)==5 ) then

!           --- iconst(1) = 2 or 5 L2 norm, i.e. a = sum (VEC1(i)) ** 2

               a = ddot ( nusol, u1, 1, u1, 1 )

            else

!           --- iconst(1) = 3 max norm, i.e. a = max (VEC1(i))

               a = 0d0
               do i = 1, nusol
                  a = max ( a , abs( u1(i) ) )
               end do

            end if
            if ( parallel ) then

!           --- parallel case, perform global sum or global max

               if ( iconst(1)==3 ) then

!              --- Parallel max

                  call prparglmax ( a, aglob, 1, SPDOUBLE )

               else

!              --- Parallel sum

                  call prparglsum ( a, aglob, 1, SPDOUBLE )

               end if
               a = aglob
               if ( iconst(1)==4 .or. iconst(1)==5 ) then

!              --- Compute parallel nusol

                  call prparglsum ( nusol, nusolglob, 1, SPINT )
                  a = a/nusolglob

               end if
               if ( iconst(1)==2 .or. iconst(1)==5 ) a = sqrt(a)

            else

!           --- standard case

               if ( iconst(1)==4 .or. iconst(1)==5 ) a = a/nusol
               if ( iconst(1)==2 .or. iconst(1)==5 ) a = sqrt(a)

            end if

         case(19)

!        --- ichois = 19,  u3 is a function of u1 and u2 through the
!                          subroutine FUNADA

            do i = 1, nusol
               call funada ( u1(i), u2(i), u3(i) )
            end do

         case(25)

!        --- "Product of two vectors"

            if ( abs(a-1d0)<=epsmac ) then
               do i = 1, nusol-nbound
                  u3(i) = u1(i)*u2(i)
               end do
            else
               do i = 1, nusol-nbound
                  u3(i) = a*u1(i)*u2(i)
               end do
            end if

         case(27)

!        --- "Limit vector"

            do i = 1, nusol
               u1(i) = min(b,max(a,u1(i)))
            end do

         case(43)

!        --- "u3 = function of u1"

            if ( iconst(1)==1 ) then

!           --- logarithm

               do i = 1, nusol
                  u3(i) = log10(abs(u1(i)))
               end do

            else

!           --- Possibility not available

               call eropen ( 'to0023' )
               call errint ( ichois, 1 )
               call errsub ( 1, 1, 0, 0 )
               call erclos ( 'to0023' )

            end if

         case default

!        --- Possibility not available

            call eropen ( 'to0023' )
            call errint ( ichois, 1 )
            call errsub ( 1, 1, 0, 0 )
            call erclos ( 'to0023' )

         end select  ! case ( ichois )

      else

!     --- Renumbering required

         select case ( ichois )
         case(1)

!        --- subtract a constant from u1

            do i = 1, nusol
               j = kprobh(i)
               u3(j) = u1(j)-a
            end do

         case(2)

!        --- determine average of u1 = =>a

            a = 0.0d0
            do i = 1, nusol
               j = kprobh(i)
               a = a+u1(j)
            end do
            ipoint = nusol
            a = a/ipoint

         case(6)

!        --- determine min and max of u1 = =>a = min, b = max

            a = u1(kprobh(1))
            b = a
            do i = 2, nusol
               j = kprobh(i)
               a = min (u1(j), a)
               b = max (u1(j), b)
            end do

         case(7)

!        --- inner product of u1 and u2  (ichois = 7)

            a = 0d0
            do i = 1, nusol
               j = kprobh(i)
               a = a+u1(j)*u2(j)
            end do
            ipoint = nusol

         case(10)

!        --- inner product of u1 and u2, without effect of boundary
!            conditions

            a = 0d0
            do i = 1, nusol-nbound
               j = kprobh(i)
               a = a+u1(j)*u2(j)
            end do
            ipoint = nusol-nbound

         case(11, 12)

!        --- Ichois = 11, 12, compute minimum and maximum, plus corresponding
!                              co-ordinates
!            First compute minimum, maximum and corresponding nodal points

            a =  rinfin
            b = -rinfin
            imin = 0
            imax = 0

            do i = 1, nusol
               if ( ichois==11 ) then
                  vv = u1( kprobh(i) )
               else
                  vv = abs ( u1( kprobh(i) ) )
               end if

               if ( vv<a ) then

!              --- vv<a, new minimum found

                  a    = vv
                  imin = i

               end if

               if ( vv>b ) then

!              --- vv>b, new maximum found

                  b    = vv
                  imax = i
               end if

            end do

!           --- Next store the corresponding co-ordinates

            do i = 1, ipoint
               j = kprobh(i)
               u3(i)   = u2 ( (imin-1) * ipoint + j )
               u3(i+3) = u2 ( (imax-1) * ipoint + j )
            end do

         case(13)

!        --- ichois = 13,  u3 is a function of u1 and u2 through the
!                          subroutine funalg

            do i = 1, nusol
               j = kprobh(i)
               call funalg ( u1(j), u2(j), u3(j) )
            end do

         case(15)

!        --- ichois = 15,  a = || VEC1-VEC2 ||

            a = 0d0
            if ( iconst(1)==1 .or. iconst(1)==4 ) then

!           --- iconst(1) = 1 or 4 L1 norm, i.e. a = sum |u1(j)-u2(j)|

               do i = 1, nusol
                  j = kprobh(i)
                  a = a + abs ( u1(j) - u2(j) )
               end do

            else if ( iconst(1)==2 .or. iconst(1)==5 ) then

!           --- iconst(1) = 2 or 5 L2 norm, i.e. a = sum (u1(j)-u2(j)) ** 2

               do i = 1, nusol
                  j = kprobh(i)
                  a = a + ( u1(j) - u2(j) ) ** 2
               end do

            else

!           --- iconst(1) = 3 max norm, i.e. a = max (u1(j)-u2(j))

               do i = 1, nusol
                  j = kprobh(i)
                  a = max ( a , abs( u1(j) - u2(j) ) )
               end do

            end if
            if ( iconst(1)==4 .or. iconst(1)==5 ) a = a/(nusol)
            if ( iconst(1)==2 .or. iconst(1)==5 ) a = sqrt(a)

         case(16)

!        --- ichois = 16,  a = || VEC1 ||

            if ( iconst(1)==1 .or. iconst(1)==4 ) then

!           --- iconst(1) = 1 or 4 L1 norm, i.e. a = sum |u1(j)|

               a = 0d0
               do i = 1, nusol
                  j = kprobh(i)
                  a = abs(u1(j))
               end do

            else if ( iconst(1)==2 .or. iconst(1)==5 ) then

!           --- iconst(1) = 2 or 5 L2 norm, i.e. a = sum (u1(j)) ** 2

               a = 0d0
               do i = 1, nusol
                  j = kprobh(i)
                  a = a+u1(j)**2
               end do

            else

!           --- iconst(1) = 3 max norm, i.e. a = max (u1(j))

               a = 0d0
               do i = 1, nusol
                  j = kprobh(i)
                  a = max ( a , abs( u1(j) ) )
               end do

            end if
            if ( iconst(1)==4 .or. iconst(1)==5 ) a = a/(nusol)
            if ( iconst(1)==2 .or. iconst(1)==5 ) a = sqrt(a)

         case(19)

!        --- ichois = 19,  u3 is a function of u1 and u2 through the
!                          subroutine FUNADA

            do i = 1, nusol
               j = kprobh(i)
               call funada ( u1(j), u2(j), u3(j) )
            end do

         case(25)

!        --- "Product of two vectors"

            if ( abs(a-1d0)<=epsmac ) then

               do i = 1, nusol
                  j = kprobh(i)
                  u3(j) = u1(j)*u2(j)
               end do

            else

               do i = 1, nusol
                  j = kprobh(i)
                  u3(j) = a*u1(j)*u2(j)
               end do

            end if

         case(43)

!        --- "u3 = function of u1"

            if ( iconst(1)==1 ) then

!           --- logarithm

               do i = 1, nusol
                  j = kprobh(i)
                  u3(i) = log(abs(u1(j)))
               end do

            else

!           --- Possibility not available

               call eropen ( 'to0023' )
               call errint ( ichois, 1 )
               call errsub ( 1, 1, 0, 0 )
               call erclos ( 'to0023' )

            end if

         case default

!        --- Possibility not available

            call eropen ( 'to0023' )
            call errint ( ichois, 1 )
            call errsub ( 1, 1, 0, 0 )
            call erclos ( 'to0023' )

         end select  ! case ( ichois )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End to0023'

      end if
      call erclos ( 'to0023' )

      end
