      subroutine to0024 ( ichois, vec1, vec2, vec3, a, b, ipoint, nusol, &
                          nbound, indprh, iconst, const, kprobh )
! ======================================================================
!
!        programmer    Jaap van der Zanden / Guus Segal
!        version 10.1  date 02-07-1999 Error correction for l2-norm
!        version 10.0  date 02-01-1998 The solution arrays are not longer
!                                      renumbered, newparameter list
!        version  9.5  date 15-04-1995 Extension with ICHOIS=25
!
!   copyright (c) 1984-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     algebraic manipulations with complex vectors
!          (do not take npboun more than once !)
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double complex vec1(*), vec2(*), vec3(*), a, b
      double precision const
      integer ichois, ipoint, nusol, nbound, indprh, iconst(*), &
              kprobh(*)

!     a             i/o   Input output constant (see ichois)
!     b             i/o   Input output constant (see ichois)
!     const          o    Contains norm at output (ICHOIS = 15 or 16)
!     ichois         i    Choice parameter. Possible values
!                          1  subtract : vec3 = vec1-a
!                          2  determine average of vec1 = = > a
!                          3  linear combination vec3 = a*vec1+b*vec2
!                          7  inner product sum(vec1*vec2), ipoint = nusol
!                         10  inner product without nbound
!                         13  Compute vec3 as function of vec1 and vec2.
!                             The user subroutine funalc is called
!                         15  Compute norm of VEC1-VEC2
!                         16  Compute norm of VEC1
!                         18  Compute complex conjugate of vector
!                         25  "Product of two vectors"
!                             vec3 = a * vec1 * vec2
!     iconst         o    Input array for ICHOIS = 15, 16
!                         ICONST(1) contains the type of norm to be computed
!                         Possible values:
!                         1       l1-norm
!                         2       l2-norm
!                         3       max-norm
!                         4       l1-norm / number of degrees of freedom
!                         5       l2-norm / number of degrees of freedom
!     indprh         i    indication if kprob part h has been filled (>0) or
!                         not (0)
!     ipoint        i/o   nusol for ichois = 2, 7, 10
!     kprobh         i    KPROB part h
!     nbound         i    Number of essential boundary conditions
!     nusol          i    Total number of unknowns
!     vec1           i    first input vector
!     vec2           i    second input vector
!     vec3           o    output vector with result (ichois = 1, 3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double complex zdotc
      double precision dzasum, ddot

!     DDOT           (CDOT)     Inner product of two vectors
!     DZASUM         (CSASUM)   Sum of magnitudes of vector components
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     FUNALC         User function to perform algebraic manipulations (complex)
!     ZDOTC          (CDOTC)     Inner product of two vectors
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       1   ICHOIS is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( indprh.eq.0 ) then

!     --- No renumbering required

         if ( ichois.eq.1 ) then

!        --- subtract a constant from vec1

            do i = 1, nusol
               vec3(i) = vec1(i)-a
            end do

         else if ( ichois.eq.2 ) then

!        --- determine average of vec1 = = > a

            a = ( 0d0, 0d0 )
            do i = 1, nusol
               a = a+vec1(i)
            end do
            ipoint = nusol
            a = a/ipoint

         else if ( ichois.eq.3 .or. ichois.eq.24) then

!        --- linear combination of vec1 and vec2

            do i = 1, nusol
               vec3(i) = vec1(i)*a+vec2(i)*b
            end do

         else if ( ichois.eq.7 ) then

!        --- inner product of vec1 and vec2  (ichois = 7)

            a = zdotc(nusol, vec1, 1, vec2, 1)
            ipoint = nusol

         else if ( ichois.eq.10 ) then

!        --- inner product of vec1 and vec2, without effect of boundary
!            conditions

            a = zdotc(nusol, vec1, 1, vec2, 1)
            ipoint = nusol

         else if ( ichois.eq.13 ) then

!        --- ichois = 13,  vec3 is a function of vec1 and vec2 through the
!                          subroutine funalg

            do i = 1, nusol
               call funalc ( vec1(i), vec2(i), vec3(i) )
            end do

         else if ( ichois.eq.15 ) then

!        --- ichois = 15,  const = || VEC1-VEC2 ||

            const = 0d0
            if ( iconst(1).eq.1 .or. iconst(1).eq.4 ) then

!           --- iconst(1) = 1 or 4 L1 norm, i.e. const = sum |VEC1(i)-VEC2(i)|

               do i = 1, nusol
                  const = const + abs ( vec1(i) - vec2(i) )
               end do

            else if ( iconst(1).eq.2 .or. iconst(1).eq.5 ) then

!           --- iconst(1) = 2 or 5 L2 norm, i.e. a = sum (VEC1(i)-VEC2(i)) ** 2

               do i = 1, nusol
                  const = const + (dble(vec1(i))-dble(vec2(i)))**2 + &
                          (dimag(vec1(i))-dimag(vec2(i)) )**2
               end do

            else

!           --- iconst(1) = 3 max norm, i.e. const = max (VEC1(i)-VEC2(i))

               do i = 1, nusol
                  const = max ( const , abs( vec1(i) - vec2(i) ) )
               end do

            end if
            if ( iconst(1).eq.4 .or. iconst(1).eq.5 ) const = const / &
                 nusol
            if ( iconst(1).eq.2 .or. iconst(1).eq.5 ) &
               const = sqrt(const)

         else if ( ichois.eq.16 ) then

!        --- ichois = 16,  const = || VEC1 ||

            if ( iconst(1).eq.1 .or. iconst(1).eq.4 ) then

!           --- iconst(1) = 1 or 4 L1 norm, i.e. a = sum |VEC1(i)|

               const = dzasum ( nusol, vec1, 1 )

            else if ( iconst(1).eq.2 .or. iconst(1).eq.5 ) then

!           --- iconst(1) = 2 or 5 L2 norm, i.e. a = sum (VEC1(i)) ** 2

               const = ddot ( 2*nusol, vec1, 1, vec1, 1 )

            else

!           --- iconst(1) = 3 max norm, i.e. const = max (VEC1(i))

               const = 0d0
               do i = 1, nusol
                  const = max ( const , abs( vec1(i) ) )
               end do

            end if
            if ( iconst(1).eq.4 .or. iconst(1).eq.5 ) const = const / &
                 nusol
            if ( iconst(1).eq.2 .or. iconst(1).eq.5 ) &
               const = sqrt(const)

         else if ( ichois.eq.18 ) then

!        --- ichois = 18, complex conjugate of input vector

            do i = 1, nusol
               vec3(i) = conjg(vec1(i))
            end do

         else if ( ichois.eq.25 ) then

!        --- "Product of two vectors"

            do i = 1, nusol
               vec3(i) = a*vec1(i)*vec2(i)
            end do

         else

!        --- Possibility not available

            call eropen ( 'to0024' )
            call errint ( ichois, 1 )
            call errsub ( 1, 1, 0, 0 )
            call erclos ( 'to0024' )

         end if

      else

!     --- Renumbering required

         if ( ichois.eq.1 ) then

!        --- subtract a constant from vec1

            do i = 1, nusol-nbound
               j = kprobh(i)
               vec3(j) = vec1(j)-a
            end do
            do i = nusol-nbound+1, nusol
               j = kprobh(i)
               vec3(j) = vec1(j)-a
            end do

         else if ( ichois.eq.2 ) then

!        --- determine average of vec1 = = > a

            a = ( 0d0, 0d0 )
            do i = 1, nusol-nbound
               j = kprobh(i)
               a = a+vec1(j)
            end do
            do i = nusol-nbound+1, nusol
               j = kprobh(i)
               a = a+vec1(j)
            end do
            ipoint = nusol
            a = a/ipoint

         else if ( ichois.eq.3 .or. ichois.eq.24)  then

!        --- linear combination of vec1 and vec2

            do i = 1, nusol-nbound
               j = kprobh(i)
               vec3(j) = vec1(j)*a+vec2(j)*b
            end do
            do i = nusol-nbound+1, nusol
               j = kprobh(i)
               vec3(j) = vec1(j)*a+vec2(j)*b
            end do

         else if ( ichois.eq.7 ) then

!        --- inner product of vec1 and vec2  (ichois = 7)

            a = (0d0,0d0)
            do i = 1, nusol-nbound
               j = kprobh(i)
               a = a + conjg(vec1(j))*vec2(j)
            end do
            do i = nusol-nbound+1, nusol
               j = kprobh(i)
               a = a + conjg(vec1(j))*vec2(j)
            end do
            ipoint = nusol

         else if ( ichois.eq.10 ) then

!        --- inner product of vec1 and vec2, without effect of boundary
!            conditions

            a = (0d0,0d0)
            do i = 1, nusol-nbound
               j = kprobh(i)
               a = a + conjg(vec1(j))*vec2(j)
            end do
            ipoint = nusol-nbound

         else if ( ichois.eq.13 ) then

!        --- ichois = 13,  vec3 is a function of vec1 and vec2 through the
!                          subroutine funalg
!              First internal unkowns

            do i = 1, nusol-nbound
               j = kprobh(i)
               call funalc ( vec1(j), vec2(j), vec3(j) )
            end do

!           --- Next boundary unkowns

            do i = nusol-nbound+1, nusol
               j = kprobh(i)
               call funalc ( vec1(j), vec2(j), vec3(j) )
            end do

         else if ( ichois.eq.15 ) then

!        --- ichois = 15,  const = || VEC1-VEC2 ||

            const = 0d0
            if ( iconst(1).eq.1 .or. iconst(1).eq.4 ) then

!           --- iconst(1) = 1 or 4 L1 norm, i.e. a = sum |vec1(j)-vec2(j)|

               do i = 1, nusol - nbound
                  j = kprobh(i)
                  const = const + abs ( vec1(j) - vec2(j) )
               end do
               do i = nusol-nbound+1, nusol
                  j = kprobh(i)
                  const = const + abs ( vec1(j) - vec2(j) )
               end do

            else if ( iconst(1).eq.2 .or. iconst(1).eq.5 ) then

!           --- iconst(1) = 2 or 5 L2 norm, i.e. a = sum (vec1(j)-vec2(j)) ** 2

               do i = 1, nusol-nbound
                  j = kprobh(i)
                  const = const + (dble(vec1(j))-dble(vec2(j))) ** 2 &
                                + (dimag(vec1(j))-dimag(vec2(j))) ** 2
               end do
               do i = nusol-nbound+1, nusol
                  j = kprobh(i)
                  const = const + (dble(vec1(j))-dble(vec2(j))) ** 2 &
                                + (dimag(vec1(j))-dimag(vec2(j))) ** 2
               end do

            else

!           --- iconst(1) = 3 max norm, i.e. const = max (VEC1(i)-VEC2(i))

               do i = 1, nusol - nbound
                  j = kprobh(i)
                  const = max ( const , abs( vec1(j) - vec2(j) ) )
               end do
               do i = nusol-nbound+1, nusol
                  j = kprobh(i)
                  const = max ( const , abs( vec1(j) - vec2(j) ) )
               end do

            end if
            if ( iconst(1).eq.4 .or. iconst(1).eq.5 ) const = const / &
                 nusol
            if ( iconst(1).eq.2 .or. iconst(1).eq.5 ) &
               const = sqrt(const)

         else if ( ichois.eq.16 ) then

!        --- ichois = 16,  const = || VEC1 ||

            const = 0d0
            if ( iconst(1).eq.1 .or. iconst(1).eq.4 ) then

!           --- iconst(1) = 1 or 4 L1 norm, i.e. a = sum |vec1(j)|

               do i = 1, nusol - nbound
                  j = kprobh(i)
                  const = const + abs(vec1(j))
               end do
               do i = nusol-nbound+1, nusol
                  j = kprobh(i)
                  const = const + abs(vec1(j))
               end do

            else if ( iconst(1).eq.2 .or. iconst(1).eq.5 ) then

!           --- iconst(1) = 2 or 5 L2 norm, i.e. a = sum (vec1(j)) ** 2

               do i = 1, nusol - nbound
                  j = kprobh(i)
                  const = const + abs(vec1(j))
               end do
               do i = nusol-nbound+1, nusol
                  j = kprobh(i)
                  const = const + dble(vec1(j))**2+dimag(vec1(j))**2
               end do

            else

!           --- iconst(1) = 3 max norm, i.e. const = max (VEC1(i))

               do i = 1, nusol - nbound
                  j = kprobh(i)
                  const = max ( const , abs( vec1(j) ) )
               end do
               do i = nusol-nbound+1, nusol
                  j = kprobh(i)
                  const = max ( const , abs( vec1(j) ) )
               end do

            end if
            if ( iconst(1).eq.4 .or. iconst(1).eq.5 ) const = const / &
                 nusol
            if ( iconst(1).eq.2 .or. iconst(1).eq.5 ) &
               const = sqrt(const)

         else if ( ichois.eq.18 ) then

!        --- ichois = 18, complex conjugate of input vector

            do i = 1, nusol-nbound
               j = kprobh(i)
               vec3(j) = conjg(vec1(j))
            end do
            do i = nusol-nbound+1, nusol
               j = kprobh(i)
               vec3(j) = conjg(vec1(j))
            end do

         else if ( ichois.eq.25 ) then

!        --- "Product of two vectors"

            do i = 1, nusol-nbound
               j = kprobh(i)
               vec3(j) = a*vec1(j)*vec2(j)
            end do
            do i = nusol-nbound+1, nusol
               j = kprobh(i)
               vec3(j) = a*vec1(j)*vec2(j)
            end do

         else

!        --- Possibility not available

            call eropen ( 'to0024' )
            call errint ( ichois, 1 )
            call errsub ( 1, 1, 0, 0 )
            call erclos ( 'to0024' )
         end if

      end if

      end
