      subroutine to0053 ( relemmt, relemvc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.1  date 23-03-2011 Debug statements
!        version  4.0  date 27-10-2010 Remove ihelp
!        version  3.2  date 28-02-2005 Fortran 90
!
!   copyright (c) 1985-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     transformation of element matrix and element vector if necessary
!
! **********************************************************************
!
!                       KEYWORDS
!
!     transformation
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(inout) :: relemmt(icount,icount), relemvc(icount)

!     relemmt    i/o   Element matrix
!     relemvc    i/o   Element vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: isum, i, ih, i1, i2, i3, j, ihelp(2,icount)
      double precision :: anx, any, s1, s2, s3, f1, f2, f3, &
                          r11, r12, r13, r21, r22, r23, r31, r32, r33, &
                          v11, v12, v13, v21, v22, v23, v31, v32, v33

!     anx            x-component of normal vector
!     any            y-component of normal vector
!                    they are not
!     f1             1-th component of right-hand side vector to be transformed
!     f2             2-th component of right-hand side vector to be transformed
!     f3             3-th component of right-hand side vector to be transformed
!     i              Counting variable
!     i1             1-th component of point in index5
!     i2             2-th component of point in index5
!     i3             3-th component of point in index5
!     ih             Position of point in coortr
!     ihelp          integer work array of size 2*icount
!                    Position (1,i) contains the position of the i^th
!                    unknown to be transformed with respect to the
!                    element vector
!                    Position (2,i) contains the corresponding position in
!                    coortr
!     isum           Integer variable to compute a sum
!     j              Counting variable
!     r11            R(1,1) component of matrix R
!     r12            R(1,2) component of matrix R
!     r13            R(1,3) component of matrix R
!     r21            R(2,1) component of matrix R
!     r22            R(2,2) component of matrix R
!     r23            R(2,3) component of matrix R
!     r31            R(3,1) component of matrix R
!     r32            R(3,2) component of matrix R
!     r33            R(3,3) component of matrix R
!     s1             1-th component of element matrix to be transformed
!     s2             2-th component of element matrix to be transformed
!     s3             3-th component of element matrix to be transformed
!     v11            V(1,1) component of matrix V
!     v12            V(1,2) component of matrix V
!     v13            V(1,3) component of matrix V
!     v21            V(2,1) component of matrix V
!     v22            V(2,2) component of matrix V
!     v23            V(2,3) component of matrix V
!     v31            V(3,1) component of matrix V
!     v32            V(3,2) component of matrix V
!     v33            V(3,3) component of matrix V
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ1        print 2d integer array
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                                             T
!       The tranformation matrix is given by V  S  R, &
!                                    T
!       the tranformation vector by V  f
!       with R and V depending on maxtrn:
!       In R^2:
!                | nx   -ny |              | r11   r12 |
!       2:   R = |          |   4,8:   R = |           |
!                | ny    nx |              | r21   r22 |
!                  T                  | v11   v12 |
!       2,4:  R = R       8:      V = |           |
!                                     | v21   v22 |
!       this means:
!      2:
!        T      |  nx s11 + ny s21       nx s12 + ny s22 |
!       R  S  = |                                        |
!               | -ny s11 + nx s21      -ny s12 + nx s22 |
!               |  s11 nx + s12 ny      -s11 ny + s12 nx |
!       S R   = |                                        |
!               |  s21 nx + s22 ny      -s21 ny + s22 nx |
!        T      |  nx f1 + ny f2 |
!       R f   = |                |
!               | -ny f1 + nx f2 |
!      4:
!        T      |  r11 s11 + r21 s21       r11 s12 + r21 s22 |
!       R  S  = |                                            |
!               |  r12 s11 + r22 s21       r12 s12 + r22 s22 |
!               |  s11 r11 + s12 r21       s11 r12 + s12 r22 |
!       S R   = |                                            |
!               |  s21 r11 + s22 r21       s21 r12 + s22 r22 |
!        T      |  r11 f1 + r21 f2 |
!       R f   = |                  |
!               |  r12 f1 + r22 f2 |
!   and similar expressions for maxtrn = 8
!       In R^3:
!       See 4:, but now extended to three dimensions
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0053' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0053'
         write(irefwr,1) 'maxtrn, ndim, icount', maxtrn, ndim, icount
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ndim==2 ) then

!     --- ndim = 2

         isum = 0
         do i = 1, icount
            ih = iworktr(index5(i))
            if ( ih/=0 ) then
               isum = isum+1
               ihelp(1,isum) = i
               ihelp(2,isum) = (ih+1)/2
            end if
         end do
         if ( isum==0 ) go to 1000

         if ( maxtrn==2 ) then

!        --- Standard transformation

            do i = 2, isum, 2
               i1 = ihelp(1,i-1)
               i2 = ihelp(1,i)
               ih = ihelp(2,i-1)
               anx = coortr(1,ih)
               any = coortr(2,ih)
               if ( matrix ) then

!              --- matrix = true:  compute R^T S R
!                  First the R^T S part

                  do j = 1, icount
                     s1 = relemmt(j,i1)
                     s2 = relemmt(j,i2)
                     relemmt(j,i1) = anx*s1+any*s2
                     relemmt(j,i2) = -any*s1+anx*s2
                  end do

!                 --- Finally the S(new) R part

                  do j = 1, icount
                     s1 = relemmt(i1,j)
                     s2 = relemmt(i2,j)
                     relemmt(i1,j) = anx*s1+any*s2
                     relemmt(i2,j) =-any*s1+anx*s2
                  end do

               end if

               if ( vector ) then

!              --- vector = true, compute R^T f

                  f1 = relemvc(i1)
                  f2 = relemvc(i2)
                  relemvc(i1) = anx*f1+any*f2
                  relemvc(i2) =-any*f1+anx*f2

               end if

            end do

         else if ( maxtrn==4 ) then

!        --- Symmetric transformation

            do i = 2, isum, 2
               i1 = ihelp(1,i-1)
               i2 = ihelp(1,i)
               ih = ihelp(2,i-1)
               r11 = coortr(1,ih)
               r12 = coortr(2,ih)
               r21 = coortr(3,ih)
               r22 = coortr(4,ih)
               if ( matrix ) then

!              --- matrix = true:  compute R^T S R
!                  First the R^T S part

                  do j = 1, icount
                     s1 = relemmt(j,i1)
                     s2 = relemmt(j,i2)
                     relemmt(j,i1) = r11*s1+r21*s2
                     relemmt(j,i2) = r12*s1+r22*s2
                  end do

!                 --- Finally the S(new) R part

                  do j = 1, icount
                     s1 = relemmt(i1,j)
                     s2 = relemmt(i2,j)
                     relemmt(i1,j) = r11*s1+r21*s2
                     relemmt(i2,j) = r12*s1+r22*s2
                  end do

               end if

               if ( vector ) then

!              --- vector = true, compute R^T f

                  f1 = relemvc(i1)
                  f2 = relemvc(i2)
                  relemvc(i1) = r11*f1+r21*f2
                  relemvc(i2) = r12*f1+r22*f2

               end if

            end do

         else

!        --- Non-symmetric transformation

            do i = 2, isum, 2
               i1 = ihelp(1,i-1)
               i2 = ihelp(1,i)
               ih = ihelp(2,i-1)
               r11 = coortr(1,ih)
               r12 = coortr(2,ih)
               r21 = coortr(3,ih)
               r22 = coortr(4,ih)
               v11 = coortr(5,ih)
               v12 = coortr(6,ih)
               v21 = coortr(7,ih)
               v22 = coortr(8,ih)
               if ( matrix ) then

!              --- matrix = true:  compute V^T S R
!                  First the V^T S part

                  do j = 1, icount
                     s1 = relemmt(j,i1)
                     s2 = relemmt(j,i2)
                     relemmt(j,i1) = v11*s1+v21*s2
                     relemmt(j,i2) = v12*s1+v22*s2
                  end do

!                 --- Finally the S(new) R part

                  do j = 1, icount
                     s1 = relemmt(i1,j)
                     s2 = relemmt(i2,j)
                     relemmt(i1,j) = r11*s1+r21*s2
                     relemmt(i2,j) = r12*s1+r22*s2
                  end do

               end if

               if ( vector ) then

!              --- vector = true, compute V^T f

                  f1 = relemvc(i1)
                  f2 = relemvc(i2)
                  relemvc(i1) = v11*f1+v21*f2
                  relemvc(i2) = v12*f1+v22*f2

               end if

            end do

         end if

      else

!     --- ndim = 3

         isum = 0
         do i = 1, icount
            ih = iworktr(index5(i))
            if ( ih/=0 ) then
               isum = isum+1
               ihelp(1,isum) = i
               ihelp(2,isum) = (ih+2)/3
            end if
         end do
         if ( isum==0 ) go to 1000
         if ( debug ) call prinin1 ( ihelp, isum, 2, 'ihelp' )

         if ( maxtrn==9 ) then

!        --- Symmetric transformation

            do i = 3, isum, 3
               i1 = ihelp(1,i-2)
               i2 = ihelp(1,i-1)
               i3 = ihelp(1,i)
               ih = ihelp(2,i)
               r11 = coortr(1,ih)
               r12 = coortr(2,ih)
               r13 = coortr(3,ih)
               r21 = coortr(4,ih)
               r22 = coortr(5,ih)
               r23 = coortr(6,ih)
               r31 = coortr(7,ih)
               r32 = coortr(8,ih)
               r33 = coortr(9,ih)
               if ( debug ) then
                  write(irefwr,1) 'i, i1, i2, i3, ih', i, i1, i2, i3, ih
                  call prinrl ( coortr(1,ih), 9, 'coortr' )
               end if  ! ( debug )
               if ( matrix ) then

!              --- matrix = true:  compute R^T S R
!                  First the R^T S part

                  do j = 1, icount
                     s1 = relemmt(j,i1)
                     s2 = relemmt(j,i2)
                     s3 = relemmt(j,i3)
                     relemmt(j,i1) = r11*s1+r21*s2+r31*s3
                     relemmt(j,i2) = r12*s1+r22*s2+r32*s3
                     relemmt(j,i3) = r13*s1+r23*s2+r33*s3
                  end do

!                 --- Finally the S(new) R part

                  do j = 1, icount
                     s1 = relemmt(i1,j)
                     s2 = relemmt(i2,j)
                     s3 = relemmt(i3,j)
                     relemmt(i1,j) = r11*s1+r21*s2+r31*s3
                     relemmt(i2,j) = r12*s1+r22*s2+r32*s3
                     relemmt(i3,j) = r13*s1+r23*s2+r33*s3
                  end do

                  if ( debug ) &
                     call prinrl1 ( relemmt, icount, icount, 'relemmt' )

               end if

               if ( vector ) then

!              --- vector = true, compute R^T f

                  f1 = relemvc(i1)
                  f2 = relemvc(i2)
                  f3 = relemvc(i3)
                  relemvc(i1) = r11*f1+r21*f2+r31*f3
                  relemvc(i2) = r12*f1+r22*f2+r32*f3
                  relemvc(i3) = r13*f1+r23*f2+r33*f3
                  if ( debug ) call prinrl ( relemvc, icount, 'relemvc' )

               end if

            end do

         else

!        --- Non-symmetric transformation

            do i = 3, isum, 3
               i1 = ihelp(1,i-2)
               i2 = ihelp(1,i-1)
               i3 = ihelp(1,i)
               ih = ihelp(2,i)
               r11 = coortr(1,ih)
               r12 = coortr(2,ih)
               r13 = coortr(3,ih)
               r21 = coortr(4,ih)
               r22 = coortr(5,ih)
               r23 = coortr(6,ih)
               r31 = coortr(7,ih)
               r32 = coortr(8,ih)
               r33 = coortr(9,ih)
               v11 = coortr(10,ih)
               v12 = coortr(11,ih)
               v13 = coortr(12,ih)
               v21 = coortr(13,ih)
               v22 = coortr(14,ih)
               v23 = coortr(15,ih)
               v31 = coortr(16,ih)
               v32 = coortr(17,ih)
               v33 = coortr(18,ih)
               if ( matrix ) then

!              --- matrix = true:  compute V^T S R
!                  First the V^T S part

                  do j = 1, icount
                     s1 = relemmt(j,i1)
                     s2 = relemmt(j,i2)
                     s3 = relemmt(j,i3)
                     relemmt(j,i1) = v11*s1+v21*s2+v31*s3
                     relemmt(j,i2) = v12*s1+v22*s2+v32*s3
                     relemmt(j,i3) = v13*s1+v23*s2+v33*s3
                  end do

!                 --- Finally the S(new) R part

                  do j = 1, icount
                     s1 = relemmt(i1,j)
                     s2 = relemmt(i2,j)
                     s3 = relemmt(i3,j)
                     relemmt(i1,j) = r11*s1+r21*s2+r31*s3
                     relemmt(i2,j) = r12*s1+r22*s2+r32*s3
                     relemmt(i3,j) = r13*s1+r23*s2+r33*s3
                  end do

               end if

               if ( vector ) then

!              --- vector = true, compute V^T f

                  f1 = relemvc(i1)
                  f2 = relemvc(i2)
                  f3 = relemvc(i3)
                  relemvc(i1) = v11*f1+v21*f2+v31*f3
                  relemvc(i2) = v12*f1+v22*f2+v32*f3
                  relemvc(i3) = v13*f1+v23*f2+v33*f3

               end if

            end do

         end if

      end if

1000  call erclos ( 'to0053' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End to0053'

      end if  ! ( debug )

      end
