      subroutine to0063 ( islold )
! ======================================================================
!
!        programmer    Guus Segal
!        version  7.0  date 06-01-2016 New parameter list
!        version  6.0  date 15-11-2012 Extra parameters
!        version  5.0  date 05-04-2002 Replace intarbld by nindex
!
!   copyright (c) 1988-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute nindex, i.e. maximal number of degrees of freedom per solution
!     vector as well as maxunk.
!     nindex is used to dimension array index3 (numold, nindex) and is
!     completely determined by kprob and islold
!     TO0063 is a help subroutine for TO0050 (BUILD)
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     topology
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmoduleelm
      use sepmodulebuild
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: islold(*)

!     islold         i    Standard SEPRAN array containing information of the
!                         various old solution vectors that must be used
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, itypvc, inunkp, irefbf, iielem, inusol, &
              jcount, j, indprfloc

!     iielem         Relative element number in element group
!     indprfloc       indication if kprob part f has been filled (>0) or not (0)
!     inunkp         Number of degrees of freedom in first point
!     inusol         Position of unknown in array USOL
!     irefbf         Reference number for type of vector (istold(k,2))
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     j              Counting variable
!     jcount         Counter
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0063' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0063'
         write(irefwr,1) 'itype, numold, nunkp', itype, numold, nunkp
         write(irefwr,1) 'icount', icount
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- compute nindex

      nindex = 0
      if ( itype<0 ) go to 1000

!     --- type number>0   standard
!         Loop over all solution vectors

      do k = 1, numold

         irefbf = istold1(1,k)
         indprfloc = istold1(2,k)

         if ( debug ) &
            write(irefwr,1) 'k, irefbf, indprf', k, irefbf, indprfloc

         if ( irefbf==0 ) then

!        --- irefbf = 0, so index3 = index2

            nindex = max ( nindex, icount )
            maxunk = max(maxunk,nunkp)

            if ( debug ) write(irefwr,1) 'icount, nunkp', icount, nunkp

         else if ( irefbf==k ) then

!        --- irefbf = k, new array index3/4

            itypvc = ks(islold(k))%typevec
            if ( itypvc/=116 ) then

!           --- itypvc = 110, Array is of the type isol,
!               or itypvc = 115, array is an array of special structure
!               Compute starting address of KPROB for specific problem
!               as well as local nunkp

               jcount = 0
               if ( indprfloc<=0 ) then

!              --- indprfloc = 0, constant number of unknowns per node

                  inunkp = ks(islold(k))%nusol/npoint
                  nindex = max ( nindex, inpelm*inunkp)
                  maxunk = max(maxunk,inunkp)
                  if ( debug ) &
                     write(irefwr,1) 'nindexl, inunkp', inpelm*inunkp, inunkp

               else

!              --- indprfloc>0, number of degrees of freedom per node not
!                              constant.

                  inunkp = sepgetinunel ( islold(k), ielgrp, jcount )
                  nindex = max ( nindex,jcount )
                  maxunk = max(maxunk,inunkp)
                  if ( debug ) &
                     write(irefwr,1) 'jcount, inunkp', jcount, inunkp

               end if

            else

!           --- itypvc = 116,  Array of special structure per element

               nindex = max ( nindex, ks(islold(k))%ivec )
               maxunk = max(maxunk,ks(islold(k))%ivec)
               if ( debug ) &
                  write(irefwr,1) 'ks(islold(k))%ivec', ks(islold(k))%ivec

            end if

         end if

      end do
      E%maxunk = maxunk

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'maxunk, nindex', maxunk, nindex
         write(irefwr,*) 'End to0063'

      end if  ! ( debug )
      call erclos ( 'to0063' )

      end subroutine to0063
