      subroutine to0105 ( vec1, vec2, vec3, npoint, ncomp, ndegfd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 27-12-2007 Extra parameter ndegfd
!        version  1.1  date 09-12-2005 Comments
!        version  1.0  date 14-04-1999
!
!   copyright (c) 1999-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the point-wise inner product of two vectors, each which ncomp
!     degrees of freedom per point
! **********************************************************************
!
!                       KEYWORDS
!
!     algebraic_manipulation
!     inner_product
!     pointwise
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, ncomp, ndegfd
      double precision vec1(ncomp,*), vec2(ncomp,*), vec3(*)

!     ncomp          i    number of components of the vectors vec1 and vec2
!     ndegfd         i    Number of components of the vectors that must be
!                         used in the inner product
!     npoint         i    Number of nodal points
!     vec1           i    first input vector with ncomp component per point
!     vec2           i    second input vector with ncomp component per point
!     vec3           o    output vector with one component per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision sum

!     i          Counting variable
!     j          Counting variable
!     sum        Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      do i = 1, npoint
         sum = 0d0
         do j = 1, ndegfd
            sum = sum + vec1(j,i)*vec2(j,i)
         end do
         vec3(i) = sum
      end do
      end
