      subroutine to0108 ( ichois, idegfd, ivec1, ivec2, ivec3, iconst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.8  date 31-03-2015 Replace er0061/2 by inicopindarr1/2
!        version  2.7  date 09-03-2013 Correction ipgath
!        version  2.6  date 05-12-2012 Long integers
!
!   copyright (c) 1995-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    TO0108 is a vector manipulation subroutine
!    Only the possibilities 14, 21 and 26 are used
!    It is called by the SEPRAN subroutines MANVEC, VECMAN, ALGEBR, ANORM,
!    DIFFVC and MANIVC
!    TO0108 is able to manipulate vectors of common type 110, 115, 116
! **********************************************************************
!
!                       KEYWORDS
!
!     algebraic_manipulation
!     manipulation
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulevecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichois, idegfd(*), ivec1, ivec2, ivec3, &
              iconst(*)

!    ichois    i   Choice parameter indicating what type of function
!                  is chosen by the user.  Possible values:
!                   14:   VEC3(i) = sqrt ( VEC1(idegfd(1) **2 +
!                                                        i
!                                          VEC1(idegfd(2) **2  )
!                                                        i
!                         The dimension must be stored in ICONST(1)
!                         The type of output vector is found by the
!                         subroutine
!                   21:   VEC3 is a vector of one unknown in each point
!                         The IDEGFD-th degree of freedom from VEC1 is
!                         used as input
!                         It is supposed that this degree of freedom is
!                         available in each point
!                   26:   Point-wise inner-product of two vectors:
!                         u out(i) = sum_j u1(i)_j + u2(i)_j
!                         if ( idegfd(1)>0 ) then j is from 1 to idegfd(1)
!    iconst   i/o  In- and output integer array containing constants for the
!                  manipulation depending on ichois
!    idegfd    i   In this array given by the user, he must store the degrees
!                  of freedom that are requested for the special purpose.
!                  In most applications only one degree of freedom per point
!                  variable but not by a constant.
!                  If the variable has value 0, then all degrees of freedom
!                  are considered.
!                  If the variable has value -1, then all degrees of freedom
!                  are considered, except those corresponding to essential
!                  boundary conditions
!                  The length of the array depends on ichois:
!                  If ichois =   14:         2 or 3 (See iconst(1))
!    ivec1     i   First input vector to be manipulated
!                  ivec1 must be a SEPRAN array of length 5, which may be
!                  of type 110, 115, 116
!                  If ichois = 17, IVEC1 is supposed to be a more dimensional
!                  array of size 5 x NVEC, otherwise IVEC1 contains only
!                  one array
!    ivec2     i   Second input vector to be manipulated. Must have the same
!                  structure as ivec1
!    ivec3    i/o  Third input vector to be manipulated (if necessary)
!                  If only one or two arrays are needed, ivec3 may be replaced
!                  by a dummy
!                  ivec3 may also correspond to the output vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, number, itypvc, lenhlp, idum, icompl, &
              nbnout, npointlc, nusout, ivec, ndegfd
      logical compli, gather
      integer, allocatable :: iwork(:), igathr(:)

!     compli         Indication if IVEC1 corresponds to a complex array
!                    (true) or to a real one (false)
!     gather         indication whether gathering has been carried out
!                    (true) or not (false)
!     i              Counting variable
!     idum           General dummy variable
!     igathr         Integer array containing the indices of the positions
!                    in USOL corresponding to IDEGFD
!                    If nboundlc>0 the prescribed degrees of freedom are
!                    positioned at the rear. Periodical boundary conditions
!                    are counted for only once
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     iwork          Array IWORK is filled by subroutine ER0063 as follows:
!                    IWORK(i)>0, gives start position of unknown in point i
!                    IWORK(i) = 0, unknown is not available in point i
!                    IWORK(i)<0, point is second periodical point
!                    -IWORK(i) refers to first point
!                    Array IWORK is only filled if gather is true
!     lenhlp         Length of a compressed input vector in words
!     nbnout         Computed number of prescribed degrees of freedom after the
!                    compress (gather) operation
!     ndegfd         Number of components of the vectors that must be
!                    used in the inner product
!     npointlc       Number of nodal points or number of elements in the case of
!                    an array of special structure defined per element
!     number         The number-th unknown in each nodal point of the array
!                    is considered. If the array is complex number=2*i-1
!                    corresponds to the real part of array i and
!                    number=2*i to the imaginary part of array i
!     nusout         Number of degrees of freedom in a solution vector
!                    after the  compress operation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ER0063         Fill index array for gather operation
!     ER0065         General gathering subroutine (complex case)
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICOPINDARR1  Copy old indexed arrayold into new array arraynew
!     PRININ         print 1d integer array
!     SEPACTSOLBF1   compute pointers to degrees of freedom of vectors
!     SEPCREASOLBF1  Create sepran vector of solution type or special structure
!                    Activate corresponding subarrays of kprob
!     SEPCREASTRUC   find structure of vector
!     SEPPRINTV      Print solution vector or vector of special structure
!     TO0045         Compute length of complex vector
!     TO0049         Compute length of real vector
!     TO0105         Compute component-wise inner product of two vectors
!     TO0105NONCONS  Compute the point-wise inner product of two vectors,
!                    each which ncomp degrees of freedom per point
!                    in some points
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1287   The output vector does not have one unknown per point
!    1441   The degree of freedom is not present in all nodes of the mesh
!    2054   Input vector is of incorrect type
!    2794   The two input vectors to compute the point-wise inner product
!           do not have the same structure
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'to0108' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from to0108'
         call ertrace
         write(irefwr,1) 'ichois', ichois
  1      format ( a, 1x, (10i6) )
         write(irefwr,1) 'ivec1, ivec2, ivec3', ivec1, ivec2, ivec3
         call sepprintv ( ivec1,'input vector' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Check type of ivec1

      compli = ks(ivec1)%compl/=0
      vectin => ks(ivec1)%sol

!     --- Reserve space

      itypvc = ks(ivec1)%typevec
      if ( itypvc ==116 ) then

!     --- Vector is of type 116, special structure defined per element

         nphysi = ks(ivec1)%ivec

         if ( ichois==26 ) then

!        --- For ichois = 26, this is not allowed

            call errint ( itypvc, 1 )
            call errsub ( 2054, 1, 0, 0 )
            go to 1000

         end if
         if ( debug ) write(irefwr,1) 'nhpys, nelem ', nphysi, nelem

      else

!     --- Vector is of type 110 or 115, i.e. solution vector or
!         special structure defined per node

         call sepactsolbf1 ( ivec1 )
         if ( ks(ivec1)%typevec==110 ) indprhi = kprobloc(50)
         if ( indprhi>0 ) then
            kprobhi => kprobpart%kprobh
            kprobhinvi => kprobpart%kprobhinv
         end if  ! ( indprfi>0 )
         iprob = ks(ivec1)%iprob
         if ( ichois==26 ) then

!        --- ichois = 26, inner product of two vectors
!            Check if both vectors have the same length

            if ( ks(ivec1)%nusol/=ks(ivec2)%nusol ) then

!           --- ivec1 and ivec2 do not have the same length

               call errsub ( 2794, 0, 0, 0 )
               go to 1000

            end if  ! ks(ivec1)%nusol/=ks(ivec2)%nusol

         end if  ! ( ichois==26 )

      end if
      if ( itypvc==116 ) then

!     --- Special case, vector defined per element, set npointlc equal
!         to number of elements

         npointlc = nelem

      else

!     --- Standard case

         npointlc = npoint

      end if
      if ( itypvc==110 ) then
         if ( ichois/=10 ) npbouni = 0
      else
         nboundi = 0
         npbouni = 0
      end if
      nusoli  = size ( vectin )
      if ( compli ) nusoli = nusoli / 2
      itypvc = 110
      number = 1  ! number of unknowns per point
      call sepcreastruc ( number, itypvc, ivec )
      if ( ierror/=0 ) go to 1000

!     --- Define output vector (if ichois=14, 26, always real)

      if ( ichois==14 .or. ichois==26 .or. .not. compli ) then
         icompl = 0
      else
         icompl = 1
      end if

      if ( ichois==21 .and. ks(ivec1)%typevec==116 ) then

!     --- Extract unknown from array defined per element
!         In this case the output vector is also defined per element
!         with one degree of freedom

         ivec = 1
         itypvc = 116

      end if
      if ( debug ) write(irefwr,1) 'itypvc, ivec, npointlc', &
                                    itypvc, ivec, npointlc
      call sepcreasolbf1 ( ivec3, itypvc, ivec, 'vector', icompl )
      vectout => usoltemp
      if ( ichois/=26 .and. (ks(ivec3)%nusol/=npointlc .or. indprfo/=0) ) then

!     --- The output vector does not contain exactly one unknown per point

         call errint ( ks(ivec3)%nusol, 1 )
         call errint ( npointlc, 2 )
         call errsub ( 1287, 2, 0, 0 )

      end if

!     --- Clear ivec3

      vectout = 0d0

!     --- Create temporary space

      gather = .false.

      length = max(nusoli,npointlc)
      allocate ( iwork(length), igathr(length), stat = error )
      if ( error/=0 ) call eralloc ( error, length, 'iwork' )
      if ( ierror/=0 ) go to 1000

      if ( (indprfi>0 .or. nphysi>1) .and. ichois==21 ) then

!     --- idegfd(1)>0;
!         ichois = 21
!         For this value of ichois a gather operation is required

         gather = .true.
         lenhlp = min ( nusoli, npointlc ) * (ks(ivec1)%compl+1)

      end if
      if ( ierror/=0 ) go to 1000

!     --- Set some constants depending on the type of vector

      if ( ichois==26 ) vectin2 => ks(ivec2)%sol

      if ( ichois==14 ) then

!     --- ICHOIS = 14, Compute length of vector

         if ( compli ) then

!        --- Complex case

            call to0045 ( vectin, vectout, idegfd, iconst(1) )
         else

!        --- Real case

            call to0049 ( vectin, vectout, idegfd, iconst(1) )
         end if

      else if ( ichois==21 ) then

!     --- ichois = 21

         if ( indprfi>0 .or. nphysi>1 ) then

!        --- First apply gather operation

            gather = .true.
            nboundi = 0

            call er0063 ( npointlc, max(1,idegfd(1)), &
                          iwork, igathr, nusout, nbnout, 0, idum )

            if ( debug ) then
               call prinin ( igathr, npointlc, 'igather' )
               write(irefwr,*) 'nusout'
            end if

!           --- Recompute nusoli, npbouni and npbouni

            nusoli  = nusout
            nboundi = nbnout
            npbouni = 0
            if ( nusout/=npointlc ) then

!           --- error: Unknown is not present in each node

               call errint ( idegfd(1), 1 )
               call errsub ( 1441, 1, 0, 0 )
            end if
            if ( compli ) then

!           --- complex case

               call er0065 ( vectin, igathr, &
                             vectout, nusoli )
            else

!           --- real case

               call inicopindarr1 ( nusoli, vectout, &
                                    vectin, igathr )
            end if
         else
            gather = .false.
         end if

      else

!     --- ichois = 26, inner product per component

         if ( idegfd(1)==0 ) then
            ndegfd = nphysi
         else
            ndegfd = min(idegfd(1),nphysi)
         end if  ! ( idegfd(2)==0 )
         if ( indprfi==0 ) then

!        --- Constant number of degrees of freedom per point

            call to0105 ( vectin, vectin2, &
                          vectout, npointlc, nphysi, ndegfd )

         else

!        --- Variable number of degrees of freedom per point

            call to0105noncons ( vectin, vectin2, &
                                 vectout, npointlc, indprfi, &
                                 indprfo, kprobfi, &
                                 kprobfo, ndegfd )

         end if  ! ( indprfi==0 )

      end if

      deallocate ( iwork, igathr, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End to0108'

      end if
      call erclos ( 'to0108' )

      end subroutine to0108
