      subroutine toaddpetsc ( relemmt, nrow )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 24-07-2015 Use sepmodulebuild
!        version  1.0  date 11-07-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Add part of element matrix corresponding to real unknowns
!     to petsc matrux
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     matrix
!     real
!     petsc
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecpack
      use sepmodulebuild
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(out) :: nrow
      double precision, intent(in) :: relemmt(icount,icount)

!     nrow           o    Number of non-prescribed unknowns in element
!     relemmt        i    Element matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, jcount, rowindex(icount), inusol, nrusolglob

!     i              Counting variable
!     inusol         Position of unknown in array USOL
!     jcount         Counter
!     nrusolglob     Global number of unknowns
!     rowindex       Contains petsc numbering of free unknowns in element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     TOADDPETSC1    Adds element matrix to petsc matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toaddpetsc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from toaddpetsc'
         write(irefwr,1) 'nrusol, ninterface', nrusol, ninterface
         call prinin ( indexren, icount, 'indexren' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute nrow and store rowindex

      jcount = 0
      nrusolglob = nrusol+ninterface
      do i = 1, icount

         inusol=indexren(i)  ! sequence number of row
         if ( parallel ) then
            if ( inusol<=nrusolglob ) then
               inusol = kprobhglob(inusol)
               jcount = jcount+1
               rowindex(jcount) = inusol-1  ! start with zero numbering
            end if  ! ( inusol<=nrusolglob )
         else if ( inusol<=nrusol ) then

!        --- Real unknown

            jcount = jcount+1
            rowindex(jcount) =  inusol-1  ! start with zero numbering

         end if  ! ( parallel )

      end do  ! i = 1, icount
      nrow = jcount

!     --- Store in petsc matrix

      call toaddpetsc1 ( rowindex, nrow, relemmt )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End toaddpetsc'

      end if  ! ( debug )
      call erclos ( 'toaddpetsc' )

      end subroutine toaddpetsc
