      subroutine tofillpresmmat
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 24-07-2015 Extra debug
!        version  2.2  date 21-09-2011 Extra debug
!        version  2.1  date 04-09-2009 Extra debug
!
!   copyright (c) 2007-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Add pressure element mass matrix to pressure mass matrix
!
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     matrix
!     simple
!     mass_matrix
!     pressure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulematstr
      use sepmodulebuild
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, inusol, nhelp, itel

!                    they are not
!     i              Counting variable
!     inusol         Position of unknown in array USOL
!     itel           Counting variable
!     nhelp          nrusol-nrusolvel
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tofillpresmmat' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from tofillpresmmat'
         write(irefwr,1) 'nrusolvel, icount, ielem, npsi', &
                          nrusolvel, icount, ielem, npsi
         call prinin ( indexren, icount, 'indexren' )
         call prinrl ( E%dpressmmat, icount, 'E%dpressmmat' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      nhelp = nrusol-nrusolvel

      if ( debug ) write(irefwr,1) 'nhelp', nhelp

      itel = 0
      do i = 1, icount

!     --- inusol := indexren(i)

         inusol = indexren(i)-nrusolvel

         if ( debug ) &
            write(irefwr,1) 'i, inusol, itel', i, inusol, itel+1

         if ( inusol>0 .and. inusol<=nhelp ) then

!        --- inusol>0, hence a pressure unknown is found
!            Add diagonal element of element matrix to diagonal matrix

            itel = itel+1
            S_PMD(inusol) = S_PMD(inusol)+E%dpressmmat(itel)

         end if  ! ( inusol>0 )

      end do  ! i = 1, icount

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( S_PMD, nhelp, 'Pressure Mass matrix' )
         write(irefwr,*) 'End tofillpresmmat'

      end if  ! ( debug )
      call erclos ( 'tofillpresmmat' )

      end
