      subroutine toglob ( iinbld, ielhlp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 24-07-2015 Use sepmodulebuild
!        version  2.1  date 05-12-2012 Long integers
!        version  2.0  date 03-10-2005 Extra parameters
!
!   copyright (c) 2000-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill matrix and right-hand side with respect to global unknowns
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     right_hand_side
!     global_unknown
!     assembly
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinbld(*), ielhlp(*)

!     ielhlp         i    This array of length nelgrp+5 is only used if
!                         allelements = 1
!                         ielhlp(ielgrp+5) = 0 means elements of element group
!                         ielgrp are skipped, &
!                         ielhlp(ielgrp+5) = 1, means that they are assembled
!     iinbld         i    User input array for subroutine BUILD, see buildbf
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, lnelemmt, lnelemvc, lnindex2
      double precision, allocatable :: elemmtloc(:)

!     elemmtloc      work array to store local element matrix
!     i              Counting variable
!     lnelemmt       Length of array elemmt
!     lnelemvc       Length of array elemvc
!     lnindex2       Length of array index2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     TOGLOB00       Compute parameters for the standard elements
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2778   More than one right-hand side not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toglob' )

      if ( numrhs>1 ) then

!     --- More than one right-hand side not yet implemented

         call errint ( numrhs, 1 )
         call errsub ( 2778, 1, 0, 0 )
         go to 1000

      end if  ! ( numrhs>1 )

      lnindex2 = npelm*nunkp
      lnelemmt = lnindex2*nglunk
      lnelemvc = nglunk

      allocate ( iwork(nusol), elemmtloc(lnelemmt), &
                 elemvc(lnelemvc), index1(npelm), index2(lnindex2), &
                 index5(lnindex2), index4(1,npelm+1), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, length+lnelemmt+lnelemvc, 'iwork' )
      if ( ierror/=0 ) go to 1000

      call toglob00 ( iinbld, ielhlp, elemmtloc )

      deallocate ( iwork, elemmtloc, index1, elemvc, index2, &
                   index5, index4, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  call erclos ( 'toglob' )
      end
