      subroutine toglob02 ( inunkpgr, inusolgr, elemmtloc, icolinv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 03-10-2005 Extra parameters
!        version  2.0  date 25-12-1998 Extra parameters, extension for storage
!                                      schemes 5 and 6
!        version  1.0  date 12-09-1998
!
!   copyright (c) 1998-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Add element matrix and element vector to large matrix and vector in case
!    of global unknowns
!    It is assumed that the new rows must be added both for rows as
!    for columns, depending on the matrix storage
!    The main diagonal is not updated since the global unknowns are assumed
!    to be related to the other unknowns, but not to the global ones
!    This may be changed in the future
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     matrix
!     right_hand_side
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulevecs
      use sepmodulebuild
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inunkpgr, inusolgr, icolinv(inunkpgr,nusol)
      double precision, intent(inout) :: elemmtloc(icount,*)

!     elemmtloc      i    Element matrix
!     icolinv        i    Integer array of length inunkpgr * nusol containing
!                         the positions of the columns for the rows
!                         corresponding to inusolgr ... inusolgr+inunkpgr-1
!                         in the lower and upper triangular matrix.
!     inunkpgr       i    Number of global unknowns in a group
!     inusolgr       i    Position of first global unknown in a group with
!                         respect to solution array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: imat, i, j, idiag, ishift, inusol
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     i          Counting variable
!     idiag      Position of diagonal element in matrix
!     imat       Position in array amat
!     inusol     Position of unknown in array USOL
!     ishift     Shift of specific variable with respect to diagonal
!     j          Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    add the element right-hand side to the right-hand-side vector
!    add a part of the element matrix to the internal matrix
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toglob02' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from toglob02'
         write(irefwr,1) 'inunkpgr, inusolgr', inunkpgr, inusolgr
         call prinin ( index2, icount, 'index2' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(iseqim)

!     --- First add the element right-hand side to the right-hand-side vector

      do j = 1, inunkpgr
         rhsd(inusolgr+j-1) = rhsd(inusolgr+j-1)+elemvc(j)
      end do

!     --- Next add a part of the element matrix to the internal matrix

      S_ii => Smat%S_ii

      if ( IM%symmetric ) then

!     --- Symmetrical compact matrix, store in rows only

         do i = 1, icount
            inusol = index2(i)

            if ( inusol<=inusolgr ) then

!           --- Internal unknown, lower part of matrix

               do j = irowsii(inusol)+1, irowsii(inusol+1)
                  iwork(icolsii(j)) = j
               end do
               do j = 1, inunkpgr
                  imat = iwork(inusolgr+j-1)
                  S_ii(imat) = S_ii(imat)+elemmtloc(i,j)
               end do

            else if ( inusol<=nrusol ) then

!           --- Internal unknown, upper part of matrix

               do j = 1, inunkpgr
                  imat = icolinv(j,index2(i))
                  S_ii(imat) = S_ii(imat)+elemmtloc(i,j)
               end do


            end if

         end do

      else

!     --- Non-symmetrical compact matrix, store in row and column

         do i = 1, icount
            inusol = index2(i)

            if ( debug ) write(irefwr,1) 'i, inusol', i, inusol
            if ( inusol<=nrusol ) then

!           --- Internal unknown

               do j = irowsii(inusol)+1, irowsii(inusol+1)
                  iwork(icolsii(j)) = j
               end do
               do j = 1, inunkpgr

!              --- upper part of matrix

                  imat = icolinv(j,index2(i))
                  if ( debug ) then
                     write(irefwr,1) 'i, j, imat/1', i, j, imat
                     write(irefwr,2) 'elemmtloc(i,j)', elemmtloc(i,j)
                  end if  ! ( debug )
                  S_ii(imat) = S_ii(imat)+elemmtloc(i,j)

!                 --- lower part of matrix

                  imat = iwork(inusolgr+j-1)
                  if ( debug ) write(irefwr,1) 'i, j, imat/2', i, j, imat
                  S_ii(imat) = S_ii(imat)+elemmtloc(i,j)

               end do

            end if

         end do

      end if  ! ( IM%symmetric )

      if ( effbc ) then

!     --- Subtract effect of essential boundray conditions from rhs

         do i = 1, icount
            if ( index2(i)>nrusol ) then
               do j = 1, inunkpgr
                  rhsd(inusolgr+j-1) = rhsd(inusolgr+j-1)- &
                                       elemmtloc(i,j)*usol(index5(i))
               end do  ! j = 1, inunkpgr
            end if  ! ( index2(i)>nrusol )
         end do  ! i = 1, icount

      end if  ! ( effbc )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End toglob02'

      end if  ! ( debug )
      call erclos ( 'toglob02' )

      end subroutine toglob02
