      subroutine toglob03 ( icolinv, inunkpgr, inusolgr )
! ======================================================================
!
!        programmer    Guus Segal
!        version 1.0   date 25-12-1998
!
!
!   copyright (c) 1998-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill array icolinv with the positions of the columns for the rows
!    inusolgr ... inusolgr+inunkpgr-1 in the lower and upper part of the matrix
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulematstr
      use sepmodulebuild
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: inunkpgr, inusolgr
      integer, intent(out) :: icolinv(inunkpgr,nusol)

!     icolinv    o    Integer array of length inunkpgr * nusol containing
!                     the positions of the columns for the rows
!                     corresponding to inusolgr ... inusolgr+inunkpgr-1
!                     in the lower and upper triangular matrix.
!     inunkpgr   i    Number of global unknowns in a group
!     inusolgr   i    Position of first global unknown in a group with respect
!                     to solution array
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, inusol

!     i          Counting variable
!     inusol     Position of unknown in array USOL
!     j          Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!      For  i := 1 (1) inunkpgr do
!         Store inverse of icolsii into icolinv
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toglob03' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from toglob03'
         write(irefwr,1) 'inunkpgr, inusolgr', inunkpgr, inusolgr
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- For  i := 1 (1) inunkpgr do

      do i = 1, inunkpgr

         inusol = inusolgr+i-1
         if ( debug ) then
            write(irefwr,1) 'i, inusol', i, inusol
            call prinin ( irowsii(inusol), 2, 'irowsii' )
            call prinin ( icolsii(irowsii(inusol)+1), &
                          irowsii(inusol+1)-irowsii(inusol), 'icolsii' )
         end if  ! ( debug )
         do j = irowsii(inusol)+1, irowsii(inusol+1)
            icolinv(i,icolsii(j)) = j

         end do

      end do
1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End toglob03'

      end if  ! ( debug )
      call erclos ( 'toglob03' )

      end subroutine toglob03

