      subroutine tomapsolelm ( vecout, vecin, kmeshc, ndegfd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-10-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Map vector of type 110 or 115 onto vector of type 116, i.e. map a vector
!     defined per node onto a vector defined per element for one element
!     Restricted to linear triangles
! **********************************************************************
!
!                       KEYWORDS
!
!     manipulation
!     vector
!     map
!     vector_defined_per_element
!     element
!     real
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmeshc(3), ndegfd
      double precision vecin(ndegfd,*), vecout(ndegfd)

!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to one element
!     ndegfd         i    Number of degrees of freedom per point
!     vecin          i    Input vector,to be mapped
!     vecout         o    Output vector, result of the mapping
!                         In this case restricted to one element only
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, inode

!     i              Counting variable
!     inode          Node sequence number
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     for i := 1 (1) 3 do
!        inode := i-th node in element
!        for j := 1 (1) ndegfd
!           vecout(j) := vecout(j)+vecin(j,inode)
!     for j := 1 (1) ndegfd
!        vecout(j) := vecout(j)/3
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tomapsolelm' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from tomapsolelm'
         write(irefwr,*) 'ndegfd ', ndegfd
         call prinin ( kmeshc, 3, 'kmeshc' )

      end if
      if ( ierror.ne.0 ) go to 1000

!     --- for i := 1 (1) 3 do

      do i = 1, 3

!     --- inode := i-th node in element
!         for j := 1 (1) ndegfd
!            vecout(j) := vecout(j)+vecin(j,inode)

         inode = kmeshc(i)
         do j =1, ndegfd
            vecout(j) = vecout(j)+vecin(j,inode)
            if ( debug ) &
               write(irefwr,*) 'j,inode,vecin ',j,inode,vecin(j,inode)
         end do  ! j =1, ndegfd

      end do  ! i = 1, 3

!     --- for j := 1 (1) ndegfd
!            vecout(j) := vecout(j)/3

      do j = 1, ndegfd
         vecout(j) = vecout(j)/3d0
      end do  ! j =1, ndegfd

1000  call erclos ( 'tomapsolelm' )

      if ( debug ) then

!     --- Debug information

         call prinrl ( vecout, ndegfd, 'vecout' )
         write(irefwr,*) 'End tomapsolelm'

      end if

      end
