      subroutine tost01 ( ilow, ihigh, relemmt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 24-07-2015 Use sepmodulebuild
!        version  3.1  date 21-04-2015 Extra debug
!        version  3.0  date 27-10-2010 remove jwork
!
!   copyright (c) 1989-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill part of element matrix corresponding to periodical boundary
!     conditions to second part of large matrix.
!     For this subroutine we need index5 instead of indexren, since we only
!     need the connection with respect to the "right" unknowns. Real case
!     It is assumed that an element does not contain both "left" and
!     "right" unknowns corresponding to one point, which means that
!     periodical boundary conditions should be separated by at least
!     two elements.
!     Compare with to0007.
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     matrix
!     periodical
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulebuild
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ilow, ihigh
      double precision, intent(in) :: relemmt(icount,icount)

!     ihigh          i    Highest position number in USOL with respect to
!                         "right" periodical boundary conditions
!                         ihigh = nusol-nbound
!     ilow           i    Lowest position number in USOL with respect to "right"
!                         periodical boundary conditions
!                         ilow = nusol-nbound-ndepen+1
!     relemmt        i    Element matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: j, k, irel, iamat, jwork(nusol)

!     iamat          Position in array Smat%S_ic
!     irel           Relative number of degrees of freedom with respect to
!                    periodical boundary conditions
!     j              Counting variable
!     jwork          Work array of length nusol
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Clear array jwork
!     for j := 1(1)icount do
!        if (index5(j)>=ilow and<=ihigh) then
!           fill jwork with sequence numbers of unknowns in row index5(j)
!           with respect to matrix
!           for k := 1(1)icount do
!              if (index5(k)<=ihigh) then
!                 iamat := jwork(index5(k))
!                 Smat%S_ic(iamat) := Smat%S_ic(iamat)+Ski
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'tost01' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from tost01'
  1      format ( a, 1x, (10i6) )
         write(irefwr,1) 'ilow, ihigh', ilow, ihigh
         call prinin ( index5, icount, 'index5' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Clear array jwork

      do j = 1, icount
         if ( index5(j)<=ihigh ) jwork(index5(j)) = 0
      end do

!     --- Loop over degrees of freedom

      do j=1,icount
        if (index5(j)>=ilow .and. index5(j)<=ihigh) then

!       --- ilow<=index5(j)<=ihigh, degrees of freedom is "right"
!           periodical boundary condition
!           fill array jwork

           irel = index5(j)-ilow+1
           do k = irowsic(irel)+1,irowsic(irel+1)
              jwork(icolsic(k)) = k
           end do
           do k = 1,icount
              if (index5(k)<=ihigh) then

!             --- index5(k)<=ihigh, non-prescribed degrees of freedom

                 iamat = jwork(index5(k))
                 if ( debug ) write(irefwr,1) 'k, iamat', k, iamat
                 if ( iamat>0 ) &
                    Smat%S_ic(iamat) = Smat%S_ic(iamat)+relemmt(j,k)
              end if
           end do
         end if
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End tost01'

      end if  ! ( debug )
      call erclos ( 'tost01' )

      end
