      subroutine inineighmark2 ( nset1, set1, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-05-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Reduce set of nodes to those nodes that satisfy
!     grad(phi) not close to 1
! **********************************************************************
!
!                       KEYWORDS
!
!     node
!     interface
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nset1, set1(nset1), iwork(*)

!     iwork          i    integer work array containing the marked nodes
!                         iwork(i) = 10: node has ||grad phi -1|| < eps
!                                  < 10, other node
!     nset1         i/o   at output: Number of nodes of set1
!                         at input:  Number of nodes of set1
!     set1          i/o   Set of nodes
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nset, i, inode

!     i              Counting variable
!     inode          Node sequence number
!     nset           Counter for number of nodes in set1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'inineighmark2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from inineighmark2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nset = 0
      do i = 1, nset1

         inode = set1(i)
         if ( iwork(inode)<10 ) then

!        --- iwork(inode)<10, hence grad(phi) not close to 1

            nset = nset+1
            set1(nset) = inode

         end if  ! ( iwork(inode)<10 )

      end do ! i = 1, nset1
      nset1 = nset

1000  call erclos ( 'inineighmark2' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End inineighmark2'

      end if  ! ( debug )

      end
