      subroutine prandom ( n, usol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 07-10-2014 New parameter list
!        version  1.2  date 11-12-2012 Long integers
!        version  1.1  date 13-05-2010 Fortran 90 random
!
!   copyright (c) 2008-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This function generates a random vector in parallel
!
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmodulecomio
      use sepmodulecpack
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n
      double precision usol(n)

!     n              i    Size of the vector
!     usol          i/o   Random vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ipblock, ipsend, j

!     ipblock        Starting address of array blockunkinfo
!     ipsend         Starting address of array sendinfo
!     j              Loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision :: prandm

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRANDM         returns random number between [0,1)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prandom' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prandom'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( testversion ) then

         do j = 1, n
            usol(j) = prandm ( 0 )
         end do

      else
         call random_number ( usol )
      end if  ! ( testversion )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prandom'

      end if  ! ( debug )
      call erclos ( 'prandom' )

      end
