      subroutine prbearmasscon6 ( nodescav, npoint, ncav, nodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-09-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kprob ac with points that have prescribed pressure
!     due to cavity
! **********************************************************************
!
!                       KEYWORDS
!
!     lubrication
!     mass_conservation
!     incompressible
!     bearing
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nodescav(0:*), npoint, ncav, nodes(*)

!     ncav           o    Number of nodes with pressure equal to cavity
!                         pressure
!     nodes          i    integer work array of length npoint that is filled
!                         as follows:
!                         0: degree of freedom is unknown (belongs to set 1a)
!                         1: degree of freedom is known,belongs to set 1b
!                         2: degree of freedom is known,belongs to set 2
!                         3: degree of freedom is known,belongs to set 3
!     nodescav       o    Kprob ad with information of nodes in which we
!                         have a cavity boundary condition
!                         Pos 0 contains the number of nodes
!                         The next positions the node numbers
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prbearmasscon6' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prbearmasscon6'
         call prinin ( nodes, npoint, 'nodes' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ncav = 0

      do i = 1, npoint

         if ( nodes(i)==2 ) then
            ncav = ncav+1
            nodescav(ncav) = i
         end if  ! ( nodes(i)==2 )

      end do  ! i = 1, npoint
      nodescav(0) = ncav

1000  if ( debug ) then

!     --- Debug information

         call prinin ( nodescav(1), ncav, 'nodescav' )
         write(irefwr,*) 'End prbearmasscon6'

      end if  ! ( debug )
      call erclos ( 'prbearmasscon6' )

      end
