      function prcg17 ( i, alpha, beta, x )
! ======================================================================
!
!        programmer    Rik Kaasschieter
!        version  1.1  date 27-09-2000 layout
!
!   copyright (c) 2000-2000  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     The logical function prcg17 is true if psi(j;x) > 0 for j=1,...,i.
!
! **********************************************************************
!
!                       KEYWORDS
!
!     eigenvalue
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision alpha(*), beta(*), x
      logical prcg17
      integer i

!     alpha          i    array containing the factors alpha (see method)
!     beta           i    array containing the factors beta (see method)
!     i              i    Counting variable
!     prcg17         o    Result of computation
!     x              i    number x, see method
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision phi, psi
      integer j

!     j              Counting variable
!     phi            value of polynomial
!     psi            value of polynomial
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The logical function prcg17 is true if psi(j;x) > 0 for j=1,...,i.
!     The value of the polynomial psi(i) of degree i for the number x is
!     computed according to the recursive formula:
!        phi(0;x) = 1;
!        psi(1;x) = 1 - alpha(1) * x;
!        phi(j;x) = psi(j;x) + beta(j) * phi(j-1;x), &
!        psi(j+1;x) = psi(j;x) - alpha(j+1) * x * phi(j;x)
!           for j = 1,...,i-1.
!     This function is required by cg for estimating the smallest
!     effective eigenvalue of the product of the inverse of the
!     preconditioning matrix M and the matrix A (see [1], section 5).
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      phi=1d0
      psi=1d0-alpha(1)*x
      prcg17=(psi.gt.0d0)
      do j=1,i-1
         phi=psi+beta(j)*phi
         psi=psi-alpha(j+1)*x*phi
         if (psi.le.0d0) then
            prcg17=.false.
            return
         end if
      end do
      end
