      subroutine prchangediagmt
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.3  date 11-12-2017 New storage of intmat
!        version  1.2  date 21-04-2015 New definition intmat
!        version  1.1  date 11-12-2012 Long integers
!
!   copyright (c) 2007-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Change zero diagonal elements of matrix by epsilon
!      Process is meant for incompressible materials, where the pressure
!      mass matrix is 0
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     change
!     diagonal
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      use sepmodulematr
      use sepmoduleintmat
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      type (intmatrix), pointer :: IM
      type (rmatrix), pointer :: A
      double precision :: maxdiag
      integer :: i

!     A              Structure of type rmatrix containing the necessary
!                    information of the real matrix
!     IM             Refers to intmt(intmat)
!     i              Counting variable
!     maxdiag        Maximal diagonal element of matrix
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3227   Complex matrices not yet implemented in this subroutine
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prchangediagmt' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prchangediagmt'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      A => realmt(totmat)%S_i

      if ( ierror/=0 ) go to 1000

!     --- Change matrix

      if ( IM%complexm ) then

!     --- matrix is complex or the type has not been implemented

         call errsub ( 3227, 0, 0, 0 )
         go to 1000

      else

!     --- Real matrix
!         Compute largest diagonal element of matrix

         maxdiag = 0d0
         do i = 1, A%neq
            maxdiag = max ( maxdiag, abs(A%S(A%diag(i))) )
         end do  ! i = 1, A%neq
         maxdiag = maxdiag*epsmac    ! multiply by machine precision

!        --- Replace diagonal by maxdiag*epsmac

         do i = 1, A%neq
            if ( abs(A%S(A%diag(i)))<= 0d0 ) A%S(A%diag(i)) = maxdiag
         end do  ! i = 1, A%neq

      end if  ! ( IM%complexm )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prchangediagmt'

      end if  ! ( debug )
      call erclos ( 'prchangediagmt' )

      end subroutine prchangediagmt
