      subroutine prcompdist1 ( nodes, nnodes, coor, ndim, nx, origin, &
                               dx, grid, markintf, nmeshc, nmeshl, &
                               ninterf, phi, isigphi, &
                               nedginterf, coorinter )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 26-04-2013 Extra parameters nedginterf, lenmarkedg
!        version  1.1  date 18-09-2012 Jump in case of errors
!        version  1.0  date 23-08-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute distance of set of nodes to interface
!     Store in array phi with sign isigphi
!     It concerns the case of a non-adapted mesh
!     This subroutine is the equivalent of prcompdist
! **********************************************************************
!
!                       KEYWORDS
!
!     level_set
!     distance
!     interface
!     2d
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim, nx(ndim), grid(*), nedginterf, nmeshc(*), &
              nmeshl(*), ninterf, nnodes, nodes(nnodes), &
              isigphi, markintf(nedginterf)
      double precision coor(ndim,*), origin(ndim), dx(ndim), phi(*), &
                       coorinter(ndim,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     coorinter      i    Contains coordinates of the interface nodes
!                         in the local numbering of the interface
!     dx             i    contains the difference between xmax, xmin etc
!     grid           i    The domain is subdivided in nx(1) x nx(2) cells
!                         of size dx(1) x dx(2)
!                         In grid the number of interface nodes per cell is
!                         stored as well as the node numbers in the folwoing way
!                         Pos. 1 starting address of node numbers
!                         (nx(1)*nx(2)+1)))
!                         The number of nodes in cell ij is defined by
!                         grid(start+1)-grid(start). where
!                         start = (j)-1)*nx(1)+i
!                         The corresponding node numbers are stored in grid
!                         from position start
!     isigphi        i    sign of phi (1 if phi>0), (-1 if phi<0)
!     markintf      i/o   Array of size nedges or nfaces to mark the edges or
!                         faces of the interface (1) else 0
!     ndim           i    Dimension of the space.
!     nedginterf     i    Declared length of array markintf
!     ninterf        i    Number of nodes in interface
!     nmeshc         i    nmesh part c, contains local node numbers
!                         corresponding to local edges of interface
!     nmeshl         i    nmesh part l, contains edge numbers corresponding
!                         to local nodes of interface
!     nnodes         i    Number of nodes in array nodes
!     nodes          i    Contains the node numbers for which we have to
!                         compute the distance
!     nx             i    number of points in ndim directions of the grid
!     origin         i    Contains the coordinates of the origin of the region
!     phi           i/o   contains the level set function phi
!                         (distance)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inode

!     i              Counting variable
!     inode          Node sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRCOMPDISTNOD1 Compute distance of node to interface
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcompdist1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcompdist1'
         write(irefwr,1) 'isigphi', isigphi
         call prinin ( nodes, nnodes, 'nodes' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, nnodes

         inode = nodes(i)

!        --- compute distance

         call prcompdistnod1 ( inode, coor, ndim, nx, origin, dx, &
                               grid, markintf, nmeshc, nmeshl, &
                               ninterf, phi, isigphi, coorinter )
         if ( ierror/=0 ) go to 1000

         if ( debug ) then
            write(irefwr,1) 'i, inode', i, inode
            write(irefwr,2) 'phi(inode)', phi(inode)
         end if  ! ( debug )

      end do  ! i = 1, nnodes

1000  call erclos ( 'prcompdist1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcompdist1'

      end if  ! ( debug )

      end
