      subroutine prcompinvmt0 ( smatrix, cielem, c0, c1, constant, &
                                volumes, ielem, irelem, inelem, &
                                lengcvec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-10-1999
!
!   copyright (c) 1999-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill matrix S for the inverse problem with one row
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ielem, irelem, inelem, lengcvec
      double precision smatrix(inelem,lengcvec), cielem(*), c0(*), &
                       c1(*), constant, volumes(*)

!     c0             i    Vector c_0
!     c1             i    Vector c_1
!     cielem         i    Vector c_ielem
!     constant       i    Value of constant
!                         In this case the constant is equal to
!                         Amax/(eps_1-eps_0)
!                         with Amax the maximum capacity and eps_1 and eps_0
!                         the values of epsilon used in the computation
!     ielem          i    Element number
!     inelem         i    Number of elements in a specific element group
!     irelem         i    Relative element sequence number with respect to
!                         element group
!     lengcvec       i    Length of the capacity vectors
!     smatrix        o    Array of size inelem x lengcvec containing
!                         the matrix to be used for the inverse problem
!     volumes        i    Array of length nelem containing the volumes of all
!     element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision cons1
      integer j

!     cons1          Help variable to store a constant
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The matrix S is a matrix of size inelem x N with inelem the
!    number of elements in the specific element group and N the size of the
!    Capacity-vectors
!    N = (nelectrodes-1)*nelectrodes/2
!    The first index refers to the element number, the second refers to the
!    Capacity vector
!
!    The matrix is defined as:
!
!    S(i,j) = Amax/A(i) (C_ielem(j)-C_0(j))/(C_1(j)-C_0(j)) / (eps_1-eps_0)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      cons1 = constant/volumes(ielem)
      do j = 1, lengcvec
         smatrix(irelem,j) = cons1*(cielem(j)-c0(j))/(c1(j)-c0(j))
      end do
      end
