      subroutine prcomppsistart ( surfaces, curves, numcurvs, numsurfs )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 17-12-2015 New call to incommat
!        version  1.0  date 16-02-2009
!
!   copyright (c) 2009-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Create problem definition for psi and create array psi
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     mesh
!     stream_function
!     start
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekprob
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numcurvs, numsurfs, &
              surfaces(numsurfs), curves(numcurvs)

!     curves         i    array containing the curve numbers
!     numcurvs       i    Number of curves in array curves
!     numsurfs       i    number of surfaces in array surfaces
!     surfaces       i    array containing the surface numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iincommt(20), itypvc, ivec
      integer, pointer :: kproblc, kprobtmp

!     iincommt       Integer input array containing information concerning
!                    the structure of the large matrix
!                    See subroutine read00
!     itypvc         Indication of the type of solution vector:
!                    110: solution vector
!                    115, 116: vectors of special structure
!     ivec           Sequence number of vector of special structure
!     kproblc        Local array kprob
!     kprobtmp       Temporary reference to kprob
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRCOMPPSIPRB   Fill array kprob with respect to computation of
!                    pseudo stream function
!     PRININ         print 1d integer array
!     PROBMERGE      Merge arrays kprob1 and kprob2 into a new array kprob1
!     SEPCOMMAT      Actual body of COMMAT and MATSTRUC
!     SEPCREASOL110  Activate or create arrays of type 110
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prcomppsistart' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prcomppsistart'
         call prinin ( kprob, 40, 'kprob' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kproblc => kprob2d(maxmesh+1)
      kprobtmp => kprob
      kprob => kproblc
      call prcomppsiprb ( surfaces, curves, numcurvs, numsurfs )
      kprob => kprobtmp
      if ( ierror/=0 ) go to 1000

!     --- Merge array kproblc and array kprob to one new kprob

      call probmerge ( kprob, kproblc )
      if ( ierror/=0 ) go to 1000

!     --- Create corresponding array intmat

      iprob = nprob  ! problem number
      iincommt = 0
      iincommt(2) = 6        ! non-symmetric compact matrix
      iincommt(5) = iprob    ! iprob
      intmat => intmat1d(iprob)
      call sepcommat ( iincommt )

!     --- Create array psi

      call sepcreasol110 ( isolut, 'psi', 0 )

!     --- Clear array psi, i.e. essential boundary conditions are 0

      usoltemp = 0d0

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prcomppsistart'

      end if  ! ( debug )
      call erclos ( 'prcomppsistart' )

      end
