      subroutine prenthrefrdin ( iseqtemp, tm, reflvl, &
                                 maxreflvl, iprint, iinenthalpy, &
                                 rinenthalpy )
! ======================================================================
!
!        programmer    John Brusche
!        version  2.0  date 09-03-2007 Extra parameters iinenthalpy, rinenthalpy
!        version  1.0  date 09-09-2005
!
!   copyright (c) 2005-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read some parameters from the input file
!
! **********************************************************************
!
!                       KEYWORDS
!
!     refine
!     read
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iseqtemp, reflvl, maxreflvl, &
              iprint, iinenthalpy(*)
      double precision tm, rinenthalpy(*)

!     iinenthalpy    i    Integer input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
!     iprint         o    Print level, 0 is no printing
!     iseqtemp       o    Sequence number of temperature in solution vector
!     maxreflvl      i    Maximum value of number of refinements
!     reflvl         o    Level of refinement
!     rinenthalpy    i    Real input array with respect to enthalpy
!                         computations
!                         For a definition of this array see subroutine
!                         readenthalpy (array iinput)
!     tm             o    Melting temperature
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqnrref, leninp, lenrnp, nrinpref, niinpref, niinpent, &
              nrinpent
      parameter ( nrinpref=1, niinpref=10, niinpent=10, nrinpent=10)
      integer iinputref(niinpref)
      double precision rinputref(nrinpref)

!     iinputref      integer input array in which information is stored
!     iseqnrref      sequence number of refine input
!     leninp         Computed length of integer input array
!     lenrnp         Computed length of real input array
!     niinpent       Declared length of array iinenthalpy
!     niinpref       Declared length of array iinputref
!     nrinpent       Declared length of array rinenthalpy
!     nrinpref       Declared length of array rinputref
!     rinputref      Real input array in which information is stored
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     IINPUTSEQ      Read contents of input block with sequence number iseqnr
!                    corresponding to main keyword name
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1378   Parameter has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Read some parameters from input
!    Produce error when reflvl has incorrect value
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prenthrefrdin' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prenthrefrdin'
100      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Read the input for the refinement and store in arrays iinputref and
!         rinputref

      leninp = niinpent
      lenrnp = nrinpent
      iseqnrref = iinenthalpy(9)  ! sequence number refine input
      call iinputseq ( 28, iseqnrref, iinputref, &
                       rinputref, leninp, lenrnp, 'REFINE' )
      if ( ierror/=0 ) go to 1000

!     --- Set some parameters extracted from the input

      iseqtemp = iinenthalpy(3)      ! sequence number of temperature
      iprint = iinenthalpy(8)        ! print level
      tm = rinenthalpy(7)            ! melting temperature
      reflvl = iinputref(1)       ! number of times we have to refine

!     --- the maximum required level of refinement is given by the user and
!         is stored in variable %TIMES  (iinputref(1))
!         Give an error in case reflvl is prohibited of incorrect

      if ( reflvl<1 .or. reflvl>maxreflvl ) then

!     --- reflvl out of range

         call errint ( reflvl, 1 )
         call errint ( 1, 2 )
         call errint ( 3, 3 )
         call errchr ( 'reflvl', 1 )
         call errsub ( 1378, 3, 0, 1 )
         go to 1000

      end if  ! ( reflvl<1 .or. reflvl>maxreflvl )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,100) 'reflvl', reflvl
         write(irefwr,*) 'End prenthrefrdin'

      end if
      call erclos ( 'prenthrefrdin' )

      end
