      subroutine prfilliintim ( inpsep, rinsep )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 25-04-2013 Use getparm
!        version  1.0  date 26-01-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iintim with information stored in array inpsep
!     Fill common ctimen with information stored in rinsep
!     Help routine for prmakedist
! **********************************************************************
!
!                       KEYWORDS
!
!     time_integration
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecomio
      use sepmodulecons
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: inpsep(*)
      double precision rinsep(*)

!     inpsep         i    Integer input array for subroutine prmakedist
!                         See subroutine readscmakedist for a description
!     rinsep         i    Real input array for subroutine prmakedist
!                         See subroutine readscmakedist for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision getparm

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETPARM        Return with real defined by inpsep/rinsep
!     SEPCOMPTIMCOF  Allocate some arrays for module sepmoduletime depending on
!                    the parameter ireadcftim
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfilliintim' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfilliintim'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      itimbc = 1
      ichtim = inpsep(4)
      lineartim =  1
      ncopeqtim = 1
      ireadbctim = -1
      ifilltim = 0
      itimcf = 0
      numoldtim = 1
      nfrac = 1
      ndt = 1
      icall_outtim = 0
      iseqouttim = 1
      ireturtim = 0
      iprinttim = 0
      keeptim = 0
      nhist = 0
      iseqfrtim = 1

      call sepcomptimcof ( 1, 1 )

      Teq(1)%iconsstiff = 0
      Teq(1)%iconsmass = 1
      Teq(1)%iconsrhs = 0
      Teq(1)%imasstot = 0
      Teq(1)%ireadsl = max(1,inpsep(5))
      Teq(1)%imas = 1
      Teq(1)%iseqaddrhs = 0
      Teq(1)%isubdiv = 0
      Teq(1)%iseqfr = 1
      Teq(1)%lenextra = 0
      Teq(1)%limit = 0
      Teq(1)%iseqreac = 0

      usectimen = .true.

!     --- Store values of t, tstep and tend in ctimen

      t = 0d0             ! actual time
      t0 = 0d0            ! t0
      tstep = getparm ( inpsep, rinsep, 7, 'tstep' )  ! dt
      tout = 1d10
      tend = inpsep(2)*tstep  ! end time
      theta = 1d0         ! euler implicit
      toutinit = 1d10
      toutend = 1d10
      toutstep = 1d10
      toutnext = 1d10

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prfilliintim'

      end if  ! ( debug )
      call erclos ( 'prfilliintim' )

      end
