      subroutine prfillmasslevs ( massmt, coor, nmeshc, nedges, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 23-08-2005
!
!   copyright (c) 2005-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill mass matrix for all edges with zero level set
!
! **********************************************************************
!
!                       KEYWORDS
!
!     mass_matrix
!     level_set
!     2d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nmeshc(*), nedges, iwork(*)
      double precision massmt(*), coor(2,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     iwork          i    integer work array of size npoint
!                         if iwork(i) = 0 point is not on the interface, &
!                         if 1 it is
!     massmt         o    mass matrix defined over the interface
!     nedges         i    number of edges
!     nmeshc         i    nmesh part c, contains node numbers corresponding to
!                         edges
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iedge, ipstart, numnodes, inode1, inode2
      double precision x1(2), x2(2), h

!     h              length of edge / 2
!     iedge          edge sequence number
!     inode1         First node in edge
!     inode2         Second node in edge
!     ipstart        Starting address in array
!     numnodes       Number of nodal points in edge
!     x1             coordinates of first point in edge
!     x2             coordinate of last point in edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINRL         Print 1d real vector
!     SEPPRINNMESHC  print nmesh part c
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2740   more than 2 nodes in edge
! **********************************************************************
!
!                       PSEUDO CODE
!
!     for all edges in mesh do
!        if ( both nodes have iwork = 1 ) then
!           create element mass matrix for edge
!           Add to large matrix
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillmasslevs' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillmasslevs'
         write(irefwr,1) 'nedges', nedges
         call sepprinnmeshc
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- for all edges in mesh do

      do iedge = 1, nedges

         ipstart = nmeshc(iedge)
         numnodes = nmeshc(ipstart)
         if ( numnodes>2 ) then

!        --- Not all edges are linear

            call errint ( numnodes, 1 )
            call errint ( iedge, 2 )
            call errint ( 2, 3 )
            call errsub ( 2740, 3, 0, 0 )
            go to 1000

         end if  ! ( numnodes>2 )

         inode1 = nmeshc(ipstart+1)
         inode2 = nmeshc(ipstart+2)
         if ( iwork(inode1)>0 .and. iwork(inode2)>0 ) then

!        --- both nodes have iwork = 1
!            create element mass matrix for edge
!            Add to large matrix

            x1(1) = coor(1,inode1)
            x1(2) = coor(2,inode1)
            x2(1) = coor(1,inode2)
            x2(2) = coor(2,inode2)
            h = sqrt((x2(1)-x1(1))**2+(x2(2)-x1(2))**2)*0.5d0
            massmt(inode1) = massmt(inode1)+h
            massmt(inode2) = massmt(inode2)+h

         end if  ! ( iworkinode1()>0 .and. iwork(inode2)>0 )

      end do  ! iedge = 1, nedges

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( massmt, 322, 'massmt' )
         write(irefwr,*) 'End prfillmasslevs'

      end if
      call erclos ( 'prfillmasslevs' )

      end
