      subroutine prfillrefvec2 ( refinenew, update, refmapold, &
                                 nelem_basis, nelemold )
! ======================================================================
!
!        programmer    John Brusche
!        version  1.0  date 23-08-2005
!
!   copyright (c) 2005-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill array refinenew of length nelemold with required levels of
!    refinement based on the refinement data collected and stored in
!    array update. Array refmapold contains the information for the mapping
!    between the basis mesh and the refined mesh.
!    The refinement level of each of element of the basis mesh is set
!    equal the to maximum level of refinement as found in the subelements
!    for the specific element
! **********************************************************************
!
!                       KEYWORDS
!
!     interface
!     phase_change
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nelem_basis, nelemold, update(nelemold), &
              refmapold(2*nelemold), refinenew(nelem_basis)

!     nelem_basis    i    Number of elements of the basis mesh
!     nelemold       i    Number of elements of the previously refined mesh
!     refinenew      i    Required refinement levels per element of the basis
!                         mesh
!     refmapold      i    Mapping array
!     update         i    Required refinement levels per element of the
!                         currently refinement mesh
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, elem_basis

!     elem_basis     Parent element number of a subelement
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prfillrefvec2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prfillrefvec2'
         call prinin(refmapold, 2*nelemold, 'refmapold')
         call prinin(update, nelemold, 'update')

      end if
      if ( ierror/=0 ) go to 1000

!     --- Initialize refinenew

      refinenew = 0

!     --- Actual filling of refinenew

      do i = 1, nelemold
         elem_basis = refmapold(2*i-1)
         refinenew(elem_basis) = max(refinenew(elem_basis), update(i))
      end do ! i = 1, nelemold

1000  if ( debug ) then

!     --- Debug information


         call prinin ( refinenew, nelem_basis, 'refinenew' )
         write(irefwr,*) 'End prfillrefvec2'

      end if
      call erclos ( 'prfillrefvec2' )

      end
