      subroutine prmakedistmark ( nnodes, nodes, ninterf, interf, iwork, &
                                  npoint, phi, phibarrier, phiold )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-08-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark nodes in interface and outside interface
!     Fill arrays nodes, interf and array phiold
! **********************************************************************
!
!                       KEYWORDS
!
!     level_set
!     distance
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npoint, nnodes, nodes(npoint), ninterf, interf(npoint), &
              iwork(npoint)
      double precision phi(npoint), phibarrier, phiold(npoint)

!     interf         o    Contains the node numbers of the interface
!     iwork          o    integer work array to mark the nodes
!                         -10:  node with phi<0 outside the band
!                          -1:  node with phi<0 inside the band
!                           0:  node on interface
!                           1:  node with phi>0 inside the band
!                          10:  node with phi>0 outside the band
!     ninterf        o    number of nodes in array interf
!     nnodes         o    Number of nodes in array nodes
!     nodes          o    Contains the node numbers of the band outside the
!                         interface
!     npoint         i    Number of nodal points
!     phi            i    Contains the distance function phi
!     phibarrier     i    Defines the region where the accuracy is tested
!                         Only nodes close to the interface defined by
!                         |phi|<phibarrier are used
!     phiold         o    Contains copy of phi
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmakedistmark' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmakedistmark'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      nnodes = 0
      ninterf = 0
      do i = 1, npoint

         if ( phi(i)>0d0 ) then

!        --- phi positive

            if ( phi(i)<phibarrier ) then

!           --- node in band around interface

               nnodes = nnodes+1
               nodes(nnodes) = i
               iwork(i) = 1

            else

!           --- node outside band around interface

               iwork(i) = 10

            end if  ! ( phi(i)<phibarrier )

         else if ( phi(i)<0d0 ) then

!        --- phi negative

            if ( -phi(i)<phibarrier ) then

!           --- node in band around interface

               nnodes = nnodes+1
               nodes(nnodes) = i
               iwork(i) = -1

            else

!           --- node outside band around interface

               iwork(i) = -10

            end if  ! ( phi(i)<phibarrier )

         else

!        --- phi zero

            iwork(i) = 0
            ninterf = ninterf+1
            interf(ninterf) = i

         end if  ! ( phi(i)>0d0 )

      end do  ! i = 1, npoint

      phiold = phi

1000  call erclos ( 'prmakedistmark' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmakedistmark'

      end if  ! ( debug )

      end
