      subroutine prmarkinterf ( markintf, iobstfaces, lenmarkintf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 26-04-2013 Extra parameter lenmarkintf
!        version  1.0  date 22-08-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark all edges or faces in interface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     level_set
!     interface
!     edge
!     face
!     mark
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iobstfaces(*), lenmarkintf, markintf(lenmarkintf)

!     iobstfaces     i    contains edge sequence numbers of boundary
!                         of levelset
!     lenmarkintf    i    Declared length of array markintf
!     markintf       o    Array of size nedges or nfaces to mark the edges or
!                         faces of the interface (1) else 0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ilev, kstart, numedges

!     ilev           Sequence number of level set being considered
!     kstart         Starting address in iobstfaces
!     numedges       number of edges
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmarkinterf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmarkinterf'
         write(irefwr,1) 'lenmarkintf', lenmarkintf
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ilev = 1  ! level set sequence number
      kstart = iobstfaces(ilev)      ! start of edges in iobstfaces
      numedges = iobstfaces(kstart)  ! number of edges in interface

      if ( debug ) then
         write(irefwr,1) 'ilev, kstart, numedges', &
                          ilev, kstart, numedges
         call prinin ( iobstfaces(kstart+1), numedges, 'iobstfaces' )
      end if  ! ( debug )

      markintf(iobstfaces(kstart+1:kstart+numedges)) = 1

1000  call erclos ( 'prmarkinterf' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmarkinterf'

      end if  ! ( debug )

      end
