      subroutine prmarklev1 ( ipointobst, iwork, interf, ninterf, isig, &
                              iobst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-10-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays iwork and interf with interface points
!
! **********************************************************************
!
!                       KEYWORDS
!
!     interface
!     level_set
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ipointobst(*), iwork(*), interf(*), ninterf, isig, iobst

!     interf         o    Contains node numbers of interface
!     iobst          i    Obstacle sequence number
!     ipointobst     i    Array containing information about the nodes in an
!                         obstacle
!                         See PG Section 24.2 for a description
!     isig           i    Sign of level set inside obstacle
!     iwork          o    integer work array of length npoint
!                          -1:  node with phi<0
!                           0:  node on interface
!                           1:  node with phi>0
!     ninterf        o    Number of nodes in interf
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, numbndobst, numinnobst, numpntobst

!     istart         Starting address in ipointobst
!     numbndobst     Number of nodal points on the boundary of the obstacle
!     numinnobst     Number of points that are near the boundary of the obstacle
!     numpntobst     Number of nodal points inside the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmarklev1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmarklev1'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- get parameters

      istart = ipointobst(iobst)
      if ( debug ) write(irefwr,1) 'iobst, istart', iobst, istart

      numpntobst = ipointobst(istart)
      numinnobst = ipointobst(istart+1)
      numbndobst = ipointobst(istart+2)
      istart = istart+3

      if ( debug ) &
          write(irefwr,1) 'numpntobst, numinnobst, numbndobst', &
                           numpntobst, numinnobst, numbndobst

!     --- Points inside obstacle

      iwork(ipointobst(istart+1:istart+numpntobst)) = isig
      istart = istart+numpntobst+numinnobst

!     --- Points on obstacle

      ninterf = numbndobst
      interf(1:ninterf) = ipointobst(istart+1:istart+numbndobst)
      iwork(interf(1:ninterf)) = 0

1000  call erclos ( 'prmarklev1' )
      if ( debug ) then

!     --- Debug information

         call prinin ( interf, ninterf, 'interf' )
         write(irefwr,*) 'End prmarklev1'

      end if  ! ( debug )

      end
