      subroutine prmarkneigh1 ( reflvl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-04-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array refinenew for all neighbours of elements that are
!     intersected by the melting front
! **********************************************************************
!
!                       KEYWORDS
!
!     interface
!     phase_change
!     neighbor
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer reflvl

!     reflvl         i    Level of refinement
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inpelm, ikelmc, &
              ielem, ielgrp, irelem, node, istart, nneigh, k, &
              neighbor, i

!     i              Counting variable
!     ielem          Element number
!     ielgrp         element group number
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     irelem         Relative element sequence number with respect to
!                    element group
!     istart         First position used at a part
!     k              Counting variable
!     neighbor       Neighbor element
!     nneigh         Number of neighbours of node
!     node           Node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmarkneigh1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmarkneigh1'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Get constants

      ikelmc = 0           ! start address in kemshc
      ielem = 0            ! last element number used

!     --- Loop over element groups

      do ielgrp = 1, nelgrp

!     --- Compute some element group dependent quantities

         inelem = Topgrp(ielgrp)%inelem
         inpelm = Topgrp(ielgrp)%inpelm
         ishape = kmeshf(ielgrp)  ! shape number
         if ( ndim==1 .and. ishape<1 .or. &
              ndim==2 .and. ishape<3 .or. &
              ndim==3 .and. ishape<10 ) then

!        --- Element of lower dimension
!            raise ikelmc and ielem

            ikelmc = ikelmc+inelem*inpelm
            ielem = ielem+inelem

         else

!        --- Not an element of lower dimension
!            Loop over elements

            do irelem = 1, inelem

!           --- Raise ielem

               ielem = ielem+1
               if ( refinenew(ielem)==reflvl + 1 ) then

!              --- This element is marked as to be refined
!                  Mark all neighbouring elements with at least reflvl

                  do k = 1, inpelm
                     ikelmc = ikelmc+1
                     node = kmeshc(ikelmc)    ! node in element

!                    --- Find neighbouring elements of node

                     istart = kmshd1(node)
                     nneigh = kmshd1(node+1)-istart

                     if (debug) then
                        print *, 'node, istart, nneigh ', &
                                  node, istart, nneigh
                     end if

                     do i = 1, nneigh
                        neighbor = kmshd2(istart+i)
                        refinenew(neighbor) = &
                           max(reflvl, refinenew(neighbor))
                     end do  ! i = 1, nneigh

                  end do  ! k = 1, inpelm

               else

!              --- Not intersected

                  ikelmc = ikelmc+inpelm

               end if  ! (refinenew(ielem)==reflvl + 1)

            end do  ! iielem = 1, inelem

         end if  ! ( ndim==1 .and. ishape<1 )

      end do  ! ielgrp = 1, nelgrp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmarkneigh1'

      end if
      call erclos ( 'prmarkneigh1' )

      end
