      subroutine prmat2matlab ( jmetod, nrusol, amat, intmt1, filename )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 15-02-2010 Extension with complex vectors
!        version  1.0  date 02-01-2008
!
!   copyright (c) 2008-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print a sepran matrix in a form suitable for reading
!     in matlab
!     reading in matlab may be done in the following way:
!     sepget make_matrix.m
!     give the command make_matrix in the matlab command window
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     real
!     complex
!     print
!     matlab
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jmetod, nrusol, intmt1(*)
      double precision amat(*)
      character (len=*) filename

!     amat           i    matrix to be written
!     filename       i    Name of file to which the matlab output must be
!                         written
!     intmt1         i    first part of array intmat corresponding to amat
!     jmetod         i    Indicates type of storage scheme for the matrix
!                         See subroutine prcong
!     nrusol         i    Number of free unknowns
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer irefwrsav, ishift
      logical compl

!     compl          indication whether the arithmetic is real (false)
!                    or complex (true)
!     irefwrsav      Help variable to store the value of irefwr
!     ishift         Shift from the lower to the upper triangular part
!                    in amat(*)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRMATLABMAT    Print compact matrix in a form suitable to be read by
!                    matlab
!     PRMATLABMATC   Print compact complex matrix in a form suitable to be
!                    read by matlab
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmat2matlab' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmat2matlab'
         write(irefwr,1) 'jmetod, nrusol', jmetod, nrusol
         call prinin ( intmt1, 10, 'intmt1' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      compl = jmetod==7 .or. jmetod==8
      if ( debug ) write(irefwr,*) 'compl ', compl

      if ( jmetod==5 .or. jmetod==7 ) then

!     --- symmetric matrix

         ishift = 0

      else if ( jmetod<30 ) then

!     --- nonsymmetric matrix

         ishift = intmt1(1+nrusol)

      end if

      if ( debug ) write(irefwr,1) 'ishift', ishift

!     --- open file

      irefwrsav = irefwr    ! save irefwr
      irefwr = 19           ! new definition (temporary)
      open ( unit = irefwr, file = filename )  ! open file

!     --- Write matrix

      if ( compl ) then

!     --- complex case

         call prmatlabmatc ( jmetod, nrusol, amat, amat(2*nrusol+1), &
                             amat(2*(nrusol+ishift)+1), intmt1, &
                             intmt1(nrusol+2) )

      else

!     --- real case

         call prmatlabmat ( jmetod, nrusol, amat, amat(nrusol+1), &
                            amat(nrusol+ishift+1), intmt1, &
                            intmt1(nrusol+2) )

      end if  ! ( compl )

!     --- close file

      close( unit = irefwr )  ! close file
      irefwr = irefwrsav      ! reset irefwr

1000  call erclos ( 'prmat2matlab' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmat2matlab'

      end if  ! ( debug )

      end
