      subroutine prmatlabmat ( jmetod, n, diag, amatl, amatu, intmt1, &
                               intmt2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 04-02-2014 Also print non-symmetrical part
!        version  1.3  date 19-09-2007 Extension with jmethod 0
!        version  1.2  date 29-03-2007 Correction format
!
!   copyright (c) 1998-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    print compact matrix in a form suitable to be read by matlab
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jmetod, n, intmt1(*), intmt2(*)
      double precision diag(*), amatl(*), amatu(*)

!     amatl      i    lower triangular part of the large matrix
!     amatu      i    upper triangular part of the large matrix
!     diag       i    Diagonal of the large matrix
!     intmt1     i    first part of array intmat
!     intmt2     i    part 2 of array intmat
!     jmetod     i    Indicates type of storage scheme for the matrix
!                     if 0, only the diagonal is printed
!     n          i    number of degrees of freedom
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, jend, j, jstart, ii

!     i              Counting variable
!     ii             address of matrix entry in 1D form
!     j              Counting variable
!     jend           end position of a row
!     jstart         starting position of row
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmatlabmat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmatlabmat'
100      format(i9,1x,i9,ss,1pe22.12e2)
110      format(i9,ss,1pe22.12e2)

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( format_matlab==0 ) then

!     --- format_matlab = 0, default format defined by Kees Vuik

         write(irefwr,100) (i,i,diag(i),i=1,n)
         if ( jmetod==0 ) go to 1000
         do i = 1, n
            jstart=intmt1(i)+1
            jend=intmt1(i+1)

            do j = jstart, jend
               if ( abs(amatl(j))>0d0 ) then
                  write(irefwr,100) i,intmt2(j),amatl(j)
               end if  ! ( abs(amatl(j)>0d0 ) )
            end do  ! j = jstart, jend

         end do  ! i = 1, n
         if ( jmetod==6 ) then

!        --- Non symmetric version

            do i = 1, n
               jstart=intmt1(i)+1
               jend=intmt1(i+1)

               do j = jstart, jend
                  if ( abs(amatu(j))>0d0 ) then
                     write(irefwr,100) intmt2(j), i, amatu(j)
                  end if  ! ( abs(amatl(j)>0d0 ) )
               end do  ! j = jstart, jend

            end do  ! i = 1, n

         end if  ! ( jmetod==5 )

      else if ( format_matlab==1 ) then

!     --- format_matlab = 1, compact format defined by Mehfooz ur Rehman

         do i=1,n
            ii=i+(i-1)*n
            write(irefwr,110) ii, diag(i)
         end do
         if ( jmetod==0 ) go to 1000
         do i=1,n
            jstart=intmt1(i)+1
            jend=intmt1(i+1)
            do j=jstart,jend
               ii= i+(intmt2(j)-1)*n
               write(irefwr,110)  ii, amatl(j)
            end do
         end do
         if(jmetod/=6) go to 1000

         do i=1,n
            jstart=intmt1(i)+1
            jend=intmt1(i+1)
            do j=jstart,jend
               ii=intmt2(j)+(i-1)*n
               write(irefwr,110) ii, amatu(j)
            end do

         end do

      end if  ! ( format_matlab==0 )

1000  call erclos ( 'prmatlabmat' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmatlabmat'

      end if  ! ( debug )

      end
