      subroutine prparddots ( resvec, icount, n, dx, dy, innercontr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-03-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine calculates a number of ddots if the code is not parallel.
!     For a parallel code the local set of ddots is calculated, then a global
!     summation takes place.
!     res(1:icount) = inner_product ( dx(:,1:icount), dy(:) )
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer n, icount, innercontr(*)
      double precision resvec(icount), dx(1:n,1:icount), &
                       dy(1:n)

!     dx             i    Arrays to compute the inner products
!     dy             i    Second array to compute the inner products
!     icount         i    Number of vectors
!     innercontr     i    Array of size nrusol indicating if an unknown in the
!                         block must be used in a global inner product or not
!                         innercontr(i) = 1: take unknown i into account
!                         innercontr(i) = 0: take unknown i not into account
!                         The unknown in the block with smallest number is taken
!                         into accoount
!     n              i    Length of the vectors
!     resvec         o    Contains the icount inner products
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision sum, res(icount)

!     i              Counting variable
!     j              Counting variable
!     res            Intermediate array to store the local inner products
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     PRPARGLSUM     This function performs a global sum of a quantitity
!                    or vector across all processes
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( parallel ) then

!     --- Parallel case, store all local sums into res

         do i = 1, icount

            sum = 0d0
            do j = 1, n
               sum = sum + innercontr(j)*(dx(j,i)*dy(j))
            end do  ! j = 1, n
            res(i) = sum

         end do  ! i = 1, icount

!        --- Global summation

         call prparglsum ( res, resvec, icount, SPDOUBLE )

      else

!     --- serial case

         do i = 1, icount
            resvec(i) = dot_product ( dx(:,i), dy(:) )
         end do  ! i = 1, icount

      end if  ! ( parallel )

      end
