      subroutine prreacnodes ( isol, iseqreac )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 13-02-2006 New call to prdividereac
!        version  1.0  date 23-08-2005
!
!   copyright (c) 2005-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Replace the integrated reaction force by the nodal force
!     This is done by subdivision by a diagonal mass matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     level_set
!     mass_matrix
!     reaction_force
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isol(*), iseqreac

!     iseqreac       i    Sequence number of reaction vector in array ISOL
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution arrays
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRDIVIDEREAC   Subdivide the reaction force vector by the diagonal
!                    mass matrix defined by the surface
!     PRMASSLEVSET   Fill mass matrix for all nodes that are on the set of
!                    points with level set 0
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prreacnodes' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prreacnodes'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Compute mass matrix (matrixm)

      call prmasslevset

!     --- subdivide the reaction force

      call prdividereac ( isol(iseqreac), 1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prreacnodes'

      end if
      call erclos ( 'prreacnodes' )

      end subroutine prreacnodes
