      subroutine prsimplepar2 ( inumblocks, inumblocksu, inumblocksp, &
                                mu, mp, nu )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-11-2008
!
!   copyright (c) 2008-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays inumblocks for u and p unknowns
!
! **********************************************************************
!
!                       KEYWORDS
!
!     parallel
!     simple
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mu, mp, inumblocks(2,mu+mp), inumblocksu(2,mu), &
              inumblocksp(2,mp), nu

!     inumblocks     i    See Programmers Guide, Section 24.3
!                         Array innercontr part 2
!     inumblocksp    i    Array inumblocks with respect to pressure
!     inumblocksu    i    Array inumblocks with respect to velocity
!     mp             i    number of pressure unknowns at interfaces
!     mu             i    number of velocity unknowns at interfaces
!     nu             i    Number of velocity unknowns
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsimplepar2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsimplepar2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inumblocksu(:,1:mu) = inumblocks(:,1:mu)
      inumblocksp(1,1:mp) = inumblocks(1,mu+1:mu+mp)-nu
      inumblocksp(2,1:mp) = inumblocks(2,mu+1:mu+mp)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsimplepar2'

      end if  ! ( debug )
      call erclos ( 'prsimplepar2' )

      end
