      subroutine prsingvaldec ( a, m, n )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-09-2007
!
!   copyright (c) 2007-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute singular value decomposition of array a
!     Returns with orthogonal basis of space spanned by A
! **********************************************************************
!
!                       KEYWORDS
!
!     singular_value_decomposition
!     orthogonal_basis
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer m, n
      double precision a(m,n)

!     a             i/o   Matrix of size m x n
!                         Contains the orthogonal basis at output
!     m              i    number of rows
!     n              i    number of columns
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nmax
      parameter (nmax=500)
      integer i, its, j, jj, k, l, nm
      double precision v(n,n), w(n), anorm, c, f, g, h, s, scale, &
                       x, y, z, rv1(nmax)

!     anorm          ?
!     c              ?
!     f              ?
!     g              ?
!     h              ?
!     i              Counting variable
!     its            ?
!     j              Counting variable
!     jj             ?
!     k              Counting variable
!     l              General loop variable
!     nm             ?
!     nmax           ?
!     rv1            ?
!     s              ?
!     scale          ?
!     v              ?
!     w              ?
!     x              ?
!     y              ?
!     z              ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision prpythagoras

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRPYTHAGORAS   finds sqrt(a^2+b^2) without overflow
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsingvaldec' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsingvaldec'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      g=0d0
      scale=0d0
      anorm=0d0
      do 25 i=1,n
        l=i+1
        rv1(i)=scale*g
        g=0d0
        s=0d0
        scale=0d0
        if(i<=m)then
          do 11 k=i,m
            scale=scale+abs(a(k,i))
11        continue
          if(abs(scale)>0d0)then
            do 12 k=i,m
              a(k,i)=a(k,i)/scale
              s=s+a(k,i)*a(k,i)
12          continue
            f=a(i,i)
            g=-sign(sqrt(s),f)
            h=f*g-s
            a(i,i)=f-g
            do 15 j=l,n
              s=0d0
              do 13 k=i,m
                s=s+a(k,i)*a(k,j)
13            continue
              f=s/h
              do 14 k=i,m
                a(k,j)=a(k,j)+f*a(k,i)
14            continue
15          continue
            do 16 k=i,m
              a(k,i)=scale*a(k,i)
16          continue
          end if
        end if
        w(i)=scale *g
        g=0d0
        s=0d0
        scale=0d0
        if((i<=m).and.(i/=n))then
          do 17 k=l,n
            scale=scale+abs(a(i,k))
17        continue
          if(abs(scale)>0d0)then
            do 18 k=l,n
              a(i,k)=a(i,k)/scale
              s=s+a(i,k)*a(i,k)
18          continue
            f=a(i,l)
            g=-sign(sqrt(s),f)
            h=f*g-s
            a(i,l)=f-g
            do 19 k=l,n
              rv1(k)=a(i,k)/h
19          continue
            do 23 j=l,m
              s=0d0
              do 21 k=l,n
                s=s+a(j,k)*a(i,k)
21            continue
              do 22 k=l,n
                a(j,k)=a(j,k)+s*rv1(k)
22            continue
23          continue
            do 24 k=l,n
              a(i,k)=scale*a(i,k)
24          continue
          end if
        end if
        anorm=max(anorm,(abs(w(i))+abs(rv1(i))))
25    continue
      do 32 i=n,1,-1
        if(i<n)then
          if(abs(g)>0d0)then
            do 26 j=l,n
              v(j,i)=(a(i,j)/a(i,l))/g
26          continue
            do 29 j=l,n
              s=0d0
              do 27 k=l,n
                s=s+a(i,k)*v(k,j)
27            continue
              do 28 k=l,n
                v(k,j)=v(k,j)+s*v(k,i)
28            continue
29          continue
          end if
          do 31 j=l,n
            v(i,j)=0d0
            v(j,i)=0d0
31        continue
        end if
        v(i,i)=1d0
        g=rv1(i)
        l=i
32    continue
      do 39 i=min(m,n),1,-1
        l=i+1
        g=w(i)
        do 33 j=l,n
          a(i,j)=0d0
33      continue
        if( abs(g)>0d0 )then
          g=1d0/g
          do 36 j=l,n
            s=0d0
            do 34 k=l,m
              s=s+a(k,i)*a(k,j)
34          continue
            f=(s/a(i,i))*g
            do 35 k=i,m
              a(k,j)=a(k,j)+f*a(k,i)
35          continue
36        continue
          do 37 j=i,m
            a(j,i)=a(j,i)*g
37        continue
        else
          do 38 j= i,m
            a(j,i)=0d0
38        continue
        end if
        a(i,i)=a(i,i)+1d0
39    continue
      do 49 k=n,1,-1
        do 48 its=1,30
          do 41 l=k,1,-1
            nm=l-1
            if ( abs((abs(rv1(l))+anorm)-anorm)<=0d0 )  goto 2
            if ( abs((abs(w(nm))+anorm)-anorm)<=0d0 )  goto 1
41        continue
1         c=0d0
          s=1d0
          do 43 i=l,k
            f=s*rv1(i)
            rv1(i)=c*rv1(i)
            if ( abs((abs(f)+anorm)-anorm)<=0d0 ) goto 2
            g=w(i)
            h=prpythagoras(f,g)
            w(i)=h
            h=1d0/h
            c= (g*h)
            s=-(f*h)
            do 42 j=1,m
              y=a(j,nm)
              z=a(j,i)
              a(j,nm)=(y*c)+(z*s)
              a(j,i)=-(y*s)+(z*c)
42          continue
43        continue
2         z=w(k)
          if(l==k)then
            if(z<0d0)then
              w(k)=-z
              do 44 j=1,n
                v(j,k)=-v(j,k)
44            continue
            end if
            goto 3
          end if
          if ( its==30 ) then
!             write(irefwr,*) 'no convergence in svdcmp'
             go to 1000
          end if  ! ( its==30 )
          x=w(l)
          nm=k-1
          y=w(nm)
          g=rv1(nm)
          h=rv1(k)
          f=((y-z)*(y+z)+(g-h)*(g+h))/(2.0*h*y)
          g=prpythagoras(f,1d0)
          f=((x-z)*(x+z)+h*((y/(f+sign(g,f)))-h))/x
          c=1d0
          s=1d0
          do 47 j=l,nm
            i=j+1
            g=rv1(i)
            y=w(i)
            h=s*g
            g=c*g
            z=prpythagoras(f,h)
            rv1(j)=z
            c=f/z
            s=h/z
            f= (x*c)+(g*s)
            g=-(x*s)+(g*c)
            h=y*s
            y=y*c
            do 45 jj=1,n
              x=v(jj,j)
              z=v(jj,i)
              v(jj,j)= (x*c)+(z*s)
              v(jj,i)=-(x*s)+(z*c)
45          continue
            z=prpythagoras(f,h)
            w(j)=z
            if(abs(z)>0d0)then
              z=1d0/z
              c=f*z
              s=h*z
            end if
            f= (c*g)+(s*y)
            x=-(s*g)+(c*y)
            do 46 jj=1,m
              y=a(jj,j)
              z=a(jj,i)
              a(jj,j)= (y*c)+(z*s)
              a(jj,i)=-(y*s)+(z*c)
46          continue
47        continue
          rv1(l)=0d0
          rv1(k)=f
          w(k)=x
48      continue
3       continue
49    continue

1000  call erclos ( 'prsingvaldec' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prsingvaldec'

      end if  ! ( debug )

      end
