      subroutine prsolveoverrl ( inpcop, rincop, irhsd, isol )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 11-11-2005 New call to prov00
!        version  1.1  date 18-06-2004 Correction call prov00
!        version  1.0  date 02-05-2003
!
!   copyright (c) 2003-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve system of linear equations by the overrelaxation method
!
! **********************************************************************
!
!                       KEYWORDS
!
!     linear
!     linear_solver
!     solve
!     overrelaxation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesolve
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpcop(*), irhsd(*), isol(5,*)
      double precision rincop(*)

!     inpcop         i    Copy of integer input array extended with defaults
!                         Input array for overrelaxation subroutines
!                         Is filled in subroutine prsolveinput
!     irhsd          i    Standard SEPRAN array, containing information of the
!                         large vector
!     isol          i/o   Standard SEPRAN array containing information of the
!                         solution array
!     rincop         i    This array contains the copy of array rinsol
!                         Input array for overrelaxation subroutines
!                         Is filled in subroutine prsolveinput
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer  iwork(4), ichois, iretur
      double precision work(2), eps

!                    they are not
!     eps            Local accuracy
!     ichois         Choice parameter for the overrelaxation process
!     iretur         Defines if the process must be terminated if the
!                    number of iterations exceeds the maximum (0) or
!                    continues (1)
!     iwork          integer work array containing input for overrelaxation
!     work           Work array containing input for overrelaxation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PROV00ENTHAL   Solve system of non-linear equations for the enthalpy
!                    equation by overrelaxation
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prsolveoverrl' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prsolveoverrl'
         call prinin ( inpcop, 40, 'inpcop' )
         call prinin ( irhsd, 5, 'irhsd' )

      end if
      if ( inpcop(15)/=0 ) niter1 = inpcop(15)
      if ( inpcop(16)/=0 ) niter2 = inpcop(16)
      if ( inpcop(15)==-3 ) niter1 = 0
      if ( rincop(3)>0d0 ) alpha = rincop(3)
      if ( rincop(4)>0d0 ) beta  = rincop(4)
      if ( rincop(5)>0d0 ) omega = rincop(5)
      if ( rincop(6)>0d0 ) alam  = rincop(6)
      nmax = inpcop(5)
      if ( nmax==0 ) nmax = irhsd(5)
      iretur = inpcop(22)

!     --- Special case of overrelaxation for enthalpy equation

      eps = rincop(1)
      if ( inpcop(11)==2 ) eps = -eps
      ichois = inpcop(9)
      If ( inpcop(6)>0 ) ichois = ichois + 10
      iwork(1) = 0
      iwork(2) = 0
      call prov00enthal ( ichois, irhsd, isol, &
                          eps, iwork, inpcop(39) )

      if ( debug ) write(irefwr,*) 'End prsolveoverrl'
      call erclos ( 'prsolveoverrl' )

      end subroutine prsolveoverrl
