      subroutine prtimenthaladp ( rewind, iprint, tstep_user, &
                                  finished, irewind, ioutactsav, &
                                  toutnextsav, islold, again )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-06-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt time step for enthalpy integration if necessary
!
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     enthalpy
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmoduletime
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iprint, irewind, ioutactsav, islold(*)
      double precision tstep_user, toutnextsav
      logical rewind, finished, again

!     again          o    If true repeat step
!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
!     ioutactsav     i    Help variable to store old value of ioutact
!     iprint         i    If 1 information must be printed
!     irewind       i/o   Counting variable that keeps tracks of the number of
!                         time steps that is taken for the current time step
!                         size
!     islold         i    User input array in which the user puts information
!                         of all preceding solutions
!                         Integer array of length 5 x numold
!     rewind        i/o   When true, the current time level is rewinded
!                         to a previous time level
!     toutnextsav    i    Help varariable to store value of toutnext
!     tstep_user     i    Initial and maximum time step size as set by the user
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2791   time step halved>16 times, program halted
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prtimenthaladp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prtimenthaladp'
         write(irefwr,*) 'rewind', rewind
         write(irefwr,1) 'irewind', irewind
         write(irefwr,2) 'tstep', tstep
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      again = .false.

      if (rewind) then

!     --- rewind = true, halving of time step

         if ( iprint>0 ) write(irefwr,*) 'rewinding time at t>0'
         t = t - tstep
         tstep = tstep*0.5d0

         if ( debug ) write(irefwr,2) 'new time step', tstep

         if (tstep<1.525878d-05*tstep_user) then

!        --- time step halved>16 times, program halted

            call errsub ( 2791, 0, 0, 0 )
            go to 1000

         end if
         finished = .false.

!        --- reset to old values

         ioutact = ioutactsav
         toutnext = toutnextsav
         irewind = 0

!        --- Copy temperature to temperature_new

         usoltemp => ks(islold(1))%sol
         usoltemp = uold

         again = .true.

      else if ( irewind>2 ) then

!     --- irewind>0

         if ( iprint>1 ) then

!        --- iprint>1, maximal printing

            write(irefwr,*) 'doubling time step size'
            write(irefwr,*) 'but not larger than tstep as set by user'

         end if  ! ( iprint>1 )

         irewind = 0
         if (tstep*2d0<=tstep_user+1d-5) tstep = tstep*2d0

      else

         irewind = irewind + 1

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prtimenthaladp'

      end if  ! ( debug )
      call erclos ( 'prtimenthaladp' )

      end
