      subroutine prvec2matlab ( nrusol, rhsd, filename )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-01-2008
!
!   copyright (c) 2008-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print a sepran vector in a form suitable for reading
!     in matlab
!     reading in matlab may be done in the following way:
!     sepget make_matrix.m
!     give the command make_matrix in the matlab command window
! **********************************************************************
!
!                       KEYWORDS
!
!     vector
!     real
!     print
!     matlab
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nrusol
      double precision rhsd(nrusol)
      character (len=*) filename

!     filename       i    Name of file to which the matlab output must be
!                         written
!     nrusol         i    Number of free unknowns
!     rhsd           i    vector to be written
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer irefwrsav

!     irefwrsav      Help variable to store the value of irefwr
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prvec2matlab' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prvec2matlab'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- open file

      irefwrsav = irefwr    ! save irefwr
      irefwr = 19           ! new definition (temporary)
      open ( unit = irefwr, file = filename )  ! open file

!     --- Write vector

      write(irefwr,100) rhsd
100   format(1pe17.8e2)

!     --- close file

      close( unit = irefwr )  ! close file
      irefwr = irefwrsav      ! reset irefwr

1000  call erclos ( 'prvec2matlab' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prvec2matlab'

      end if  ! ( debug )

      end
