      subroutine sepconvsubstep ( finished, iinnavstokes )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-11-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Solve one step of the time-dependent Navier-Stokes equations
!     Use an explicit method for the convection with smaller time steps
!     (Substepping)
!     The method applied is a form of operator splitting
!     Old subroutine prconvsubstep
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     navier_stokes_equation
!     time_integration
!     convection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinnavstokes(*)
      logical, intent(inout) :: finished

!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
!     iinnavstokes   i    Integer input array for Navier-Stokes
!                         See subroutine readnavstokes for a description
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iseqnr, iseqvel, isubr
      logical first

!     first          If true it concerns the first time step
!     iseqnr         Sequence number with respect to the storage in array
!                    ISEPIN
!     iseqvel        Array containing the positions of the velocities in
!                    the element vector in the sequence required by el2005
!     isubr          Indicates the type of calling subroutine
!                    Possible values
!                    1:  CREATE
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPCONVSUBSTP1 Solve one step of the time-dependent Navier-Stokes
!                    equations
!                    Use an explicit method for the convection with
!                    smaller time steps (Substepping)
!     SEPFILLTIMINP  Fill module sepmoduletime with information stored in
!                    inpsep/rinsep and ibuffr
!     SEPTIMEFR02    Adapt time parameters at end of time step for timefrbf
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data first /.true./
! ======================================================================
!
      call eropen ( 'sepconvsubstep' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepconvsubstep'
         call prinin ( iinnavstokes, 5, 'iinnavstokes' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Put parameters in model sepmoduletime

      iseqnr = iinnavstokes(2)
      iseqvel = iinnavstokes(3)
      call sepfilltiminp ( iseqnr, isubr, iseqvel )
      if ( ierror/=0 ) go to 1000

!     --- Actual call to time subroutine

      isubr = 3
      call sepconvsubstp1 ( finished, first, isubr, iinnavstokes )
      if ( ierror/=0 ) go to 1000

!     --- Adapt time parameters

      call septimefr02

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepconvsubstep'

      end if
      call erclos ( 'sepconvsubstep' )

      end
