      subroutine sepdiffmain1 ( iinputlc, niter )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 30-01-2016 New parameter list
!        version  1.2  date 11-12-2012 Long integers
!        version  1.1  date 28-11-2006 extension of manag
!
!   copyright (c) 2005-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!       sepdiffmain1 starts the SEPRAN program SEPDIFFOUT
!       It must be called before all other subroutines.
!       The mesh is supposed to be made by program SEPMESH and stored in the
!       file meshoutput
!       The input for subroutine PROBDF is stored in array iinputlc and read
!       from the file sepcomp.inf
!       The files meshoutput, sepcomp.inf and sepcomp.out are opened, &
!       the file meshoutput is closed.
!       The solution vectors are read from the file sepcomp.out and stored
!       in memory with the aid of subroutine READSL. The information of
!       the storage is placed in array ISOL.
!       The machine dependent reference numbers iref10, iref73 and iref74 get
!       a value
!       sepdiffmain1 replaces the calls of the following SEPRAN subroutines:
!              START:  General SEPRAN starting subroutine
!              MESHRD: Reads mesh.
!              PROBDF: Reads problem definition.
!              READSL: Read SEPRAN solution vectors
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     start
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulesol
      use sepmodulemain
      use sepmodulemach
      use sepmodulepost
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iinputlc(*), niter

!     iinputlc            Work array of length iinputlc(1) to store the input
!                         from file 73 temporarily.
!                         It contains the user numbers of the solution
!                         vectors to be read
!     niter          o    Maximum number of iterations
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer manag(lenmanag), i, j, iinvec(2), &
              isubr, iter
      double precision rinvec(2), timeloc

!     i              Counting variable
!     iinvec         User input array for subroutine MANVEC
!                    For a description see subroutine MANVEC
!     isubr          Indicates the type of calling subroutine
!     iter           Iteration sequence number
!     j              Counting variable
!     manag          Manager array that sets the parameters in subroutine
!                    SEPSTL
!                    See subroutine readsp
!     rinvec         Real user input array for subroutine MANVEC
!                    The contents depend on iinvec
!     timeloc        Actual time (local parameter)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     PROUTPNOINPUT  writes the solution sas stored in the solution arrays
!                    corresponding to ISOL to the file sepcomp.out
!     READ74         Read solution vectors from sepcomp.out
!     READPROBIINP   Read information concerning the input for probdfbf from
!                    file sepcomp.out and call subroutine probdfbf
!     SEPDIFFMAIN2   Perform the first part of subroutine sepdiffmain1
!     SEPMANVEC      Vector manipulation subroutine
!     WRITPROBIINP   Write arrays iinput and rinput corresponding to kprob to
!                    file sepcomp.out
! **********************************************************************
!
!                       I/O
!
!    The files meshoutput, sepcomp.inf and sepcomp.out and if it exists
!    sepcomp.freq are read
!    For a description of meshoutput: see the Programmers Guide Section 3.3
!    For a description of the file sepcomp.inf, see subroutine PROUTP
!    For a description of the file sepcomp.out, see subroutine PRINSL
!    For a description of the file sepcomp.freq, see subroutine FVWRITE
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill common blocks
!     Open files
!     Read array manag
!     Start SEPRAN
!     Open files with mesh and solution
!     Read mesh
!     Read problem definition from sepcomp.inf
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      debug = .false.

!     --- Perform first part


      call sepdiffmain2 ( iinputlc, manag )
      if ( ierror/=0 ) go to 1000

!     --- Read solution from file 74

      call read74 ( time, niter, manag )

      if ( debug ) write(irefwr,*) 'after read74'
      if ( ierror/=0 ) go to 1000

!     --- Store solution read in isolut2d from number 1000

      do i = 1, lastmapsol
         call sepcopyvc ( isolut2d(i), isolut2d(i+999) )
         call seprmsolinfo ( isolut2d(i) )
      end do  ! i = 1, lastmapsol

      close(iref74)

!     --- Read second file

      open(iref74,file=nameout2)
      kprob => kprob2d(2)
      kprob = 10000
      call readprobiinp
      call read74 ( time, niter, manag )
      close(iref74)

!     --- Compute differences

      iinvec(1) = 2
      iinvec(2) = 27
      rinvec(1) = 1d0
      rinvec(2) = -1d0
      do i = 1, lastmapsol
         call sepmanvec ( iinvec, rinvec, isolut2d(i), &
                          isolut2d(i+999), isolut2d(i) )
      end do  ! i = 1, lastmapsol

      open(iref74,file=namediff)

!     --- write first part to file

      call writprobiinp
      timeloc = 0d0
      iter = 1
      isubr = 3
      call proutpnoinput ( isubr, timeloc, iter )

      close(iref74)

1000  call erclos ( 'sepdiffmain1' )

      end subroutine sepdiffmain1
