      subroutine sepenthbldrhs ( iprob, ireadcf, irhsdlc )
! ======================================================================
!
!        programmer    John Brusche
!        version  1.0  date 12-11-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Build right-hand side with respect to linesearch in
!     Newton method for phase change
!     Old subroutine prenthalbldrhs
! **********************************************************************
!
!                       KEYWORDS
!
!     right_hand_side
!     enthalpy
!     newton
!     phase_change
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleuser
      use sepmodulemain
      use sepmoduleintmat
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iprob, ireadcf
      integer, intent(inout) :: irhsdlc(2)

!     iprob          i    actual problem number
!     ireadcf        i    Sequence number of input for coefficients
!     irhsdlc       i/o   Right-hand side vectors R1 and R2
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, pointer :: temperature

!     temperature    Temperature in the nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERLINS         Help subroutine to call FILCOF, which fills coefficients in
!                    IUSER and USER
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPBUILD       Build matrix and right-hand side
!     SEPFILLSMF     Fill all input for subroutine SEPBUILD in SMF
!     SEPPRINTV      Print solution vector or vector of special structure
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepenthbldrhs' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepenthbldrhs'

      end if
      if ( ierror/=0 ) go to 1000

!     --- Compute iuser1 and iuser2, indicating references to start addresses
!         of iuser and user

      call erlins ( 3, iprob, ireadcf )
      iuser => iuser1(iprob)%iuser
      user => iuser1(iprob)%user
      temperature => isolut

!     --- Build right-hand side R_2

      iinbldtim(1) = 13    ! number of entries filled
      iinbldtim(2) = 2     ! build right-hand side
      iinbldtim(3) = 1     ! do not use essential boundary conditions
      iinbldtim(4) = 0     ! no mass matrix
      iinbldtim(5) = 1     ! not important
      iinbldtim(6) = 0     ! allelements (default)
      iinbldtim(7) = 0     ! addmatvec (default)
      iinbldtim(8) = 0     ! CLEARLASTRHSD (default)
      iinbldtim(9) = iprob ! problem definition number
      iinbldtim(10) = numoldtim  ! number of old solutions
      iinbldtim(11) = 0    ! ISPECTRAL (default)
      iinbldtim(12) = 1    ! number of righthand sides
      iinbldtim(13) = 1 !  ISEQIN, sequence number of temperature
      if ( debug ) call prinin ( iinbldtim, 13, 'iinbldtim' )

      call sepfillsmf ( iinbldtim, irhsdlc, temperature, temperature )
      call sepbuild

      if (debug) then
         call sepprintv ( irhsdlc(1), '(sepenthbldrhs) irhsdlc(1)' )

         call sepprintv ( irhsdlc(2), '(sepenthbldrhs) irhsdlc(2)' )
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepenthbldrhs'

      end if
      call erclos ( 'sepenthbldrhs' )

      end
