      subroutine sepfilltimdef ( ndim, ivector )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-10-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Find input parameters for time integration in structure block
!     and extend with default values
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
!     input
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulestrings
      use sepmoduletime
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ivector, ndim

!     ivector        i    Sequence number of first vector
!     ndim           i    Dimension of space
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode=2
      integer :: i, numcop, ncode(nncode), ncodmx(nncode)
      character (len=10) :: code(nncode)

!     code           Table with texts following type_mass
!     i              Loop variable
!     ncode          Table containing the number of significant characters
!                    in the keyword table code
!     ncodmx         Table containing the maximal number of  characters in
!                    the keyword table code
!     nncode         Number of entries in code
!     numcop         maximal number of coupled equations
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepfilldefint, sepfilldefstr
      double precision :: sepfilldefreal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRGETNAME      Get sequence number of vector with specific name
!     SEPCOMPTIMCOF  Allocate some arrays for module sepmoduletime depending on
!                    the parameter ireadcftim
!     SEPFILLDEFINT  Give sepfilldefint either the default value defval or the
!                    value of the variable nameint if it exists
!     SEPFILLDEFREAL Give sepfilldefreal either the default value defval or the
!                    value of the variable namereal if it exists
!     SEPFILLDEFSTR  Give sepfilldefstr either the default value defval or the
!                    value of the variable namestring if it exists
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code / 'DIAGONAL', 'CONSISTENT' /
      data ncodmx / 8, 10 /
      data ncode  / 3, 4 /
! ======================================================================
!
      call eropen ( 'sepfilltimdef' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfilltimdef'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numcop = 1  ! temporary

      keeptim = sepfilldefint ( 'TIM_KEEP', 0 )
      keeptim = sepfilldefint ( 'TIM_REUSE', keeptim-1 )+1

      ichtim = sepfilldefstr ( 'TIM_METHOD', 6, codetimmeth, nncodetimmeth, &
                                nscodetimmeth, ncodetimmeth )-1
      lineartim = 0
      ifilltim = 0
      ncopeqtim = 1
      itimcf = 1
      numoldtim = 1
      nfrac = sepfilldefint ( 'TIM_FRAC', 0 )
      ndt = 1
      icall_outtim = 0
      iseqouttim = 1
      ireturtim = 0
      iprinttim = sepfilldefint ( 'TIM_PRINT_LEVEL', 0 )

      usectimen = .false.

      nhist = 0
      minitertim = 1
      maxoldtim = 1
      maxitertim = 1

      stationarysol = .false.
      nocomputationtim = sepfilldefint ( 'TIM_NOCOMPUTATION', 0 )>0

      epstim = sepfilldefreal ( 'TIM_ACCURACY', 1d-2 )

      if ( debug ) write(irefwr,1) 'keeptim', keeptim

      if ( keeptim<2 ) then

!      --- Not reusetime

         t0 = sepfilldefreal ( 'TINIT', 0d0 )
         tend = sepfilldefreal ( 'TEND', 1d0 )
         tstep = sepfilldefreal ( 'TSTEP', 1d0 )
         toutinit = sepfilldefreal ( 'TOUTINIT', t0 )
         if ( toutinit<t0 ) toutinit = t0
         toutend = sepfilldefreal ( 'TOUTEND', tend )
         toutstep = sepfilldefreal ( 'TOUTSTEP', tstep )
         arrtimestep(1) = tstep
         arrendtime(1) = tend

      end if  ! ( keeptim<2 )

!     --- Allocate some arrays for module sepmoduletime depending on
!         the parameter ireadcftim

      call sepcomptimcof ( ndim, numcop )

      itimbc = sepfilldefstr ( 'TIM_TYPEBC', 1, codetimbc, nncodetimbc, &
                                nscodetimbc, ncodetimbc )-1
      ireadbctim = sepfilldefint ( 'TIM_SEQ_BC', 0 )

      if ( debug ) write(irefwr,1) 'ireadbctim, itimbc', ireadbctim, itimbc

      if ( ireadbctim==0 ) then

!     --- ireadbctim = 0, hence no sequence number for the boundary conditions
!         has been found.
!         Check if the string TIM_BC = vector_name can be found and read
!         sequence number of vector_name

         call prgetname ( 'TIM_BC', ireadbctim )
         if ( ireadbctim>0 ) ireadbctim = -ireadbctim-10

      end if  ! ( ireadbctim==0 )

      if ( debug ) write(irefwr,1) 'ireadbctim', ireadbctim

      Teq(1)%iconsstiff = sepfilldefstr ( 'STIFFNESS_MATRIX', 1, codevar, &
                              nncodevar, nscodevar, ncodevar )-1
      Teq(1)%iconsmass = sepfilldefstr ( 'MASS_MATRIX', 1, codevar, &
                              nncodevar, nscodevar, ncodevar )-1
      Teq(1)%iconsrhs = sepfilldefstr ( 'RIGHT_HAND_SIDE', 1, codevar, &
                              nncodevar, nscodevar, ncodevar )-1
      Teq(1)%ireadsl = sepfilldefint ( 'TIM_TYPESOL', 0 )
      Teq(1)%imas = sepfilldefstr ( 'TYPE_MASS', 2, code, &
                                    nncode, ncode, ncodmx )
      Teq(1)%isubdiv = 0
      Teq(1)%iseqreac = 0
      iseqreactim = 0
      Teq(1)%iseqveloc = 0
      Teq(1)%iseqaccel = 0
      Teq(1)%iseqmt = 0
      Teq(1)%iseqaddrhs = 0
      Teq(1)%lenextra = 0
      Teq(1)%limit = 0

      Teq(1)%imasstot = 1
      if ( Teq(1)%iconsstiff/=1 .or. Teq(1)%iconsmass/=1 ) &
         Teq(1)%imasstot = 0

      do i = 1, ncopeqtim
         Teq(i)%iseqfr = ivector+i-1
      end do  ! i = 2, ncopeqtim
      iseqfrtim = Teq(1)%iseqfr

      if ( ichtim==14 .or. ichtim==15 ) then
         Teq(2)%iseqfr = iseqfrtim+1
         Teq(3)%iseqfr = iseqfrtim+2
         Teq(4)%iseqfr = iseqfrtim+1
      end if  ! ( ichtim==14 .or. ichtim==15 )

      if ( ichtim==5 .or. ichtim==11 .or. ichtim==14 .or. ichtim==15 ) then
         theta = 1d0
      else if ( ichtim==6 ) then
         theta = 0.5d0
      end if
      arrtheta(1) = theta
      icalltim = max(1,icalltim)
      ireadcftim(1) = sepfilldefint ( 'TIM_SEQ_COEF', 0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfilltimdef'

      end if  ! ( debug )
      call erclos ( 'sepfilltimdef' )

      end subroutine sepfilltimdef
