      subroutine sepnewmarkupdr ( irhsdold )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-11-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute a^n+1 and v^n+1
!     as part of the generalized alpha method
!     Update the mesh if necessary
!     Old subroutine pr1114updrhs
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
!     newmark
!     generalized_alpha
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmoduleuser
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: irhsdold(2)

!     irhsdold      i/o   Sepran array to store the vector
!                         alpha_f F^n - alpha_m M a^n  - alpha_f S u^n
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision factor

!     factor         Multiplication factor
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPDEFMSHNEW   Deform mesh for solid computation
!     SEPPRINTV      Print solution vector or vector of special structure
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepnewmarkupdr' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepnewmarkupdr'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Update v^n by v^n := v^n + dt(1-gamma_g) a^n
!         This is the first step to compute v^n+1

      u1 => ks(isolut2dred(Teq(2)%iseqfr))%sol
      u2 => ks(isolut2dred(Teq(3)%iseqfr))%sol
      u1 = u1+tstep*(1d0-gamma_g)*u2

!     --- Compute a^n+1 = ( u^n+1 - h^n ) / (beta_g dt^2)
!         In case of updated Lagrange u^n+1 is in fact u^n+1-u^n
!         h^n contains h^n - u^n, so this update is correct

      factor = 1d0/(beta_g*tstep**2)
      u3 => ks(isolut2dred(iseqfrtim))%sol
      rhsdcp => ks(irhsdold(2))%sol
      u2 = factor*(u3-rhsdcp)
      if ( debug ) call sepprintv ( isolut2dred(Teq(3)%iseqfr), 'a^n+1' )
      if ( ierror/=0 ) go to 1000

!     --- Compute v^n+1 = v^n + dt gamma_g a^n+1 (updated v^n)
!         Store in isolut2dred

      u1 = u1 + tstep*gamma_g*u2
      if ( debug ) call sepprintv ( isolut2dred(Teq(2)%iseqfr), 'v^n+1' )

      if ( lineartim==3 ) then

!     --- Updated Lagrage, Deform mesh

         call sepdefmshnew ( isolut2dred(iseqfrtim), 1d0 )

      end if  ! ( lineartim==3 )

      if ( lineartim==3 .or. lineartim==4 ) then

!     --- un := un + u

         u2 => ks(isolut2dred(Teq(4)%iseqfr))%sol
         u2 = u3+u2

      end if  ! ( lineartim==3 .or. lineartim==4 )
      if ( ierror/=0 ) go to 1000

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepnewmarkupdr'

      end if  ! ( debug )
      call erclos ( 'sepnewmarkupdr' )

      end
