      subroutine seppjparmain
! ======================================================================
!
!        programmer    Guus Segal and Fred Vermolen
!        version  1.0  date 29-06-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Initializes the call for the construction of the projection vectors.
!     The projection vectors are computed in seppjparmain2.
!     Old subroutine prpjparmain
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     projection
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleproj
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numneigh
      double precision, allocatable :: projvc1(:)

!     numneigh       number of neighbors of a point
!     projvc1        See sepmoduleproj: projU
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPPJPARMAIN2  Creates projection vectors for the parallel deflated
!                    iccg algorithm
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppjparmain' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppjparmain'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Initializing data

      nproj  = iacnodes

      if ( debug ) write(irefwr,1) 'nproj', nproj

!     --- Reserve temporary space

      numneigh = blockunkinfo(1)

      allocate ( projcolstmp(nproj*npoint),  projcolsatmp(nproj*npoint),&
                 projrows(numneigh+1), projrowsa(numneigh+1),&
                 projvc1(nproj*npoint),&
                 projE(nproj*nproj), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'projcolstmp' )
      if ( ierror/=0 ) go to 1000

      call seppjparmain2 ( projvc1, projE )

      allocate ( projcols(projrows(numneigh+1)), &
                 projcolsa(projrowsa(numneigh+1)), stat = error )

      projcols = projcolstmp(1:projrows(numneigh+1))
      projcolsa = projcolsatmp(1:projrowsa(numneigh+1))
      deallocate ( projcolstmp, projcolsatmp, stat = error )

      allocate ( projU(nrusol+1+projrows(numneigh+1)), stat = error )
      projU = projvc1(1:nrusol+1+projrows(numneigh+1))
      deallocate ( projvc1, stat = error )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( projrows, numneigh+1, 'projrows' )
         call prinin ( projcols, projrows(numneigh+1), 'projcols' )
         call prinin ( projrowsa, numneigh+1, 'projrowsa' )
         call prinin ( projcolsa, projrowsa(numneigh+1), 'projcolsa' )
         write(irefwr,*) 'End seppjparmain'

      end if  ! ( debug )
      call erclos ( 'seppjparmain' )

      end subroutine seppjparmain
