      subroutine seppjpost
! ======================================================================
!
!        programmer    Gjalt Wijma
!        version  1.0  date 27-07-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     In this subroutine contour plots are made of the projection
!     vectors
!     Old subroutine prpjpost
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmodulemain
      use sepmoduleproj
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lnplot
      parameter (lnplot=22)
      double precision coorcn(1), rplots(3)
      character *1 text(1)
      integer :: indcol(1), inodp, idegfd, &
              iproj, i, number, ncntln, ichois1, ichois2, &
              isubr, ncoorc(1), iplots(lnplot), j
      integer, target :: isolvct
      double precision contln(1), plotfm, yfact
      double precision, allocatable :: projvec(:)

!     contln      User array of length ncntln + 5 (if used) in which the
!                  user is supposed to have stored the contour levels from
!                  position 6
!     coorcn      Dummy parameter
!     i           General loop variable.
!     ichois1     Choice parameter consisting of 4 parts:
!                  ichois = jsmoot + 10 x idoubl + 100 x iinner + 1000 x icompt
!                  with jsmoot, idoubl, iinner and icompt as defined in common
!                  jsmoot  indication which kind of smoothing is used
!                       possibilities:
!                       0:  no smoothing
!                       1:  shuman smoothing
!                       2:  |
!                       3:  |  all spline smoothing : the higher the value the
!                       4:  |  more smoothing and the less interpolation of
!                       5:  |  the values.
!                       If PLOTC is called by subroutine PLOTC2 then jsmoot is
!                       the parameter JINDCOL indicating if array INDCOL is
!                        filled by the user (1) or not (0)
!                  idoubl  indication whether the outer boundary is plotted
!                       as a single line (0) or as double line (1)
!                  iinner  indication whether inner boundaries must be plotted
!                       (1) or not (0)
!                  icompt  Parameter indicating how contour levels should be
!                       computed. Possibilities:
!                       0:  Standard situation as described previously
!                       1:  The minimum level is given in contln(6), the
!                           maximum level must be computed
!                       2:  The maximum level is given in contln(7), the
!                           minimum level must be computed
!                       3:  The minimum level is given in contln(6), the
!                           maximum level in contln(7)
!     ichois2     Choice parameter consisting of 4 parts:
!                  ichois = jindcl + 10 x idoubl + 100 x iinner + 1000 x icompt
!                  with jindcl, idoubl, iinner and icompt as defined in common
!                  jindcl  indicates how array indcol must be filled.
!                       Possible values:
!                       0: The colours with numbers 1 to 1+ncntln are used
!                       1: The user defines the colour numbers explicitly
!                       2: The user defines the minimum colour
!                       3: The user defines the minimum and maximum colour
!                       See indcol
!                  idoubl  indication whether the outer boundary is plotted
!                       as a single line (0), as a double line (1) or is not
!                       plotted at all (2)
!                  iinner  indication whether inner boundaries must be plotted
!                       (1) or not (0)
!                  icompt  Parameter indicating how contour levels should be
!                       computed. Possibilities:
!                       0:  Standard situation as described previously
!                       1:  The minimum level is given in contln(6), the
!                           maximum level must be computed
!                       2:  The maximum level is given in contln(7), the
!                           minimum level must be computed
!                       3:  The minimum level is given in contln(6), the
!                           maximum level in contln(7)
!     idegfd      Dummy variable, degree of freedom
!     indcol      This array is only used when jindcl>0.
!                  If JINDCL=1 it is a variable length array of length:
!                  number of contour levels plus 5
!                  indcol must be filled by the user with the colours
!                  of the various contour levels in increasing sequence.
!                  indcol must be filled from position 6
!                  If JINDCL=2 it is a variable length array of length 6
!                  In INDCOL(6) the user must store the minimum colour required
!                  If JINDCL=3 it is a variable length array of length 7
!                  The minimum colour must be stored in INDCOL(6) the maximum
!                  one in INDCOL(7)
!                  If the range of the colours is less than the available range
!                  of colours the colour numbering is repeated
!     inodp       Loop variable, nodal point number.
!     iplots      Communication array for PLCN00, filled by PLOTC0
!                 For a description see these subroutines
!     iproj       Loop variable, number of a projection vector.
!     isolvct     Standard SEPRAN array containing information of a temporary
!                  solution vector.
!     isubr       Indicates by which subroutine PLOTC0 has been called
!                 Possible values:
!                 1:  PLOTC1
!                 2:  PLOTC2
!                 3:  PLOTC3
!     j              Counting variable
!     lnplot      Length of array ilplot
!     ncntln      Parameter indicating the number of contour levels to be
!                  used. Possible values of ncntln are:
!                  > 0    : There are ncntln contour levels that must stored in
!                           array contln from position 6
!                  = 0    : The program computes the contour levels by
!                           dividing the range of the function between maximum
!                           and minimum value into 10 parts
!              < 0 > -1000: The program computes the contour levels by
!                           dividing the range of the function between maximum
!                           and minimum value into |NCNTLN| - 1 parts. The
!                           values are chosen such that rounded values of the
!                           contour levels appear
!                  < -1000: The program computes the contour levels by
!                           dividing the range of the function between maximum
!                           and minimum value into |NCNTLN + 1000| - 1 parts.
!                           The values are chosen such that rounded values of
!                           the contour levels appear
!                         th
!     ncoorc      Dummy parameter
!     number      The number   unknown in each nodal point of the array
!                  is considered.
!     plotfm         Plotting factor. When PLOTFM < 0, then the length of
!                    the plot is computed by - (xmax-xmin) * PLOTFM
!                    and the width by - (ymax-ymin) * YFACT * PLOTFM
!                    else the length of the plot is equal to PLOTFM cm
!                    When by some mistake the user has given | PLOTFM | a
!                    value smaller than 1 cm, then the length is chosen
!                    equal to the default length stored in common block
!                    consta
!     projvec        Contains teh complete projection vector
!     rplots         Real input array for plot subroutine
!     text           String variable containing some text
!     yfact          Multiplication for the y-co-ordinates. With this
!                    parameter the user can influence the scale in
!                    y-direction
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PLCN00         Actual contour plotting subroutine
!     PLOTC0         Actual contour subroutine
!     SEPCREASOL110  Activate or create arrays of type 110
!     SEPRMSOLINFO   Remove information of vector in ks (including deallocate)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppjpost' )

!     --- Initializing variables

      number = 1
      isolvct = 0
      contln(1) = 0d0
      ncntln = -18
      plotfm = 10d0
      yfact = 1d0
      ichois1 = 0
      ichois2 = 110

      call sepcreasol110 ( isolvct, 'solvect', 0 )

      allocate ( projvec(nrusol), stat = error )
      if ( error/=0 ) call eralloc ( error, nrusol, 'projvec' )
      if ( ierror/=0 ) go to 1000

      do iproj = 1, nproj

         usoltemp = 0d0
         projvec = 0d0

!        --- Extend projection vector into projvec

         do j = projrows(iproj)+1, projrows(iproj+1)
            projvec(projcols(j)) = projU(j)
         end do  ! j = projrows(iproj)+1, projrows(iproj+1)

         do idegfd = 1, nrusol
            inodp = kprobhinv(idegfd)
            usoltemp(inodp) = projvec(idegfd)
         end do

         isubr = 1
         call plotc0 ( number, ncntln, plotfm, yfact, ichois1, isubr, &
                       iplots, rplots, lnplot )
         text(1) = ' '
         isolut=>isolvct
         call plcn00 ( iplots, rplots, contln, indcol, text, ncoorc, coorcn )

         isubr = 2
         call plotc0 ( number, ncntln, plotfm, yfact, ichois2, isubr, &
                       iplots, rplots, lnplot )
         text(1) = ' '
         call plcn00 ( iplots, rplots, contln, indcol, text, ncoorc, coorcn )

      end do

      call seprmsolinfo ( isolvct )

      deallocate ( projvec, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'projvec' )

1000  call erclos ( 'seppjpost' )
      end subroutine seppjpost
