      subroutine seppjspar3
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-06-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Creates projection vectors for the deflated iccg algorithm.
!     New method, using real properties
!     Old subroutine prpjspar3
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleproj
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     SEPPJSPAR5     Assemble projection vectors on base of real properties
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppjspar3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppjspar3'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nproj = nelgrp
      nnproj = npoint*3

      if ( debug ) write(irefwr,1) 'nproj, nnproj', nproj, nnproj

      allocate ( projrows(nproj+1), projcolstmp(nnproj), projUtmp(nnproj), &
                 stat = error )
      if ( error/=0 ) call eralloc ( error, nproj+nnproj, 'projrows' )
      if ( ierror/=0 ) go to 1000

!     --- actual construction of vectors

      call seppjspar5

!     --- Make arrays permanent

      allocate ( projcols(nnproj), projU(nnproj), stat = error )
      if ( error/=0 ) call eralloc ( error, nproj+nnproj, 'projrows' )
      if ( ierror/=0 ) go to 1000

      projcols(1:nnproj) = projcolstmp(1:nnproj)
      projU(1:nnproj) = projUtmp(1:nnproj)

      deallocate ( projcolstmp, projUtmp, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'projcolstmp' )

1000  call erclos ( 'seppjspar3' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seppjspar3'

      end if  ! ( debug )

      end
