      subroutine seppjsubb ( ishale, isand, iproj, &
                             nlocdgfd, ninbound, frstproj, &
                             idxprojv, calcssol )
! ======================================================================
!
!        programmer     Gjalt Wijma
!        version  1.0  date 14-06-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This routine checks if for the current shale and current sand
!     set a subsolution has to be constructed. If thats the case a
!     right hand side is assembled or a previous solution is reused.
!     Old subroutine prpjsubb
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulekmesh
      use sepmoduleproj
      use sepmodulekprob
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer ishale, isand, iproj, &
              nlocdgfd, ninbound, frstproj, idxprojv
      logical calcssol

!     calcssol    o     Logical variable, if true then the right hand side is
!                       filled and a sub-solutions needs to be computed for
!                       the combination of the current shale set and
!                       the current sand set.
!     frstproj    i     First projection vector for set iSHALE.
!     idxprojv   i/o    Index into array RTSPROJV.
!     iproj       i     Number of the current projection vector.
!     isand       i     Dummy variable, set number of a sand set.
!     ishale      i     Dummy variable, set number of a shale set.
!     ninbound    i     Total number of degrees of freedom on internal
!                       boundaries of the current shale set.
!     nlocdgfd    i     Number of degrees of freedom in the subsystem.
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i3, inodp, idegfd, iinbound, iset, &
              jnodp, jdegfd, jlocdgfd, jset, i4, searchcl, collmn, &
              jstart
      double precision coeff
      type (globmat), pointer :: S
      type (intmatrix), pointer :: IM
      integer, pointer :: rows(:), cols(:)
      double precision, pointer :: mat(:)

!     IM             Refers to intmt(intmat)
!     S              Refers to global matrix
!     coeff          contains a coefficient from the large matrix
!     collmn         column number in the large matrix.
!     cols           array with column numbers
!     i3             Loop variable.
!     i4             Loop variable.
!     idegfd         number of a degree of freedom.
!     iinbound       index into DMAP.
!     inodp          nodal point number.
!     iset           set number.
!     jdegfd         degree of freedom.
!     jlocdgfd       local degree of freedom.
!     jnodp          nodal point number.
!     jset           set number.
!     jstart         index into array RTSPROJV.
!     mat            elements of matrix
!     rows           contains pointers to cols
!     searchcl       column number.
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     SEPCOMMATREV   Fill arrays irowsnew and icolsnew with upper triangular
!                    part if icolsii and irowsii contain the lower triangular
!                    part and vice versa
!     SEPMATREV      Copy matrix from rowwise to columnwise storage
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Goal: Checking if for the current shale and current sand
!           set a subsolution has to be constructed. If thats
!           the case a right hand side is assembled or a
!           previous solution is reused.
!
!     Note that the solve routine is called in the higher
!     subroutine prpjSUBP.
!
!     Note that the large matrix is stored in sparse symmetrical
!     form.
!
!     Semi PSD: prpjSUBB
!
!     IF the current sand set has no projection vector associated
!     to it THEN
!        Nothing happens
!     ELSE if the current shale set has two sand neighbours AND
!     for both of them a projection vector has to be made AND
!     this is the second projection vector THEN
!        The current solution is 1 - the previous solution. The
!        previous solution is stored in arrays ITSPROJV and
!        RTSPROJV.
!     ELSE
!        A right hand side has to be assembled:
!        LOOP over all nodes that lie on the interface that
!        surrounds the current shale set, see DMAP.
!           If the node, with degree of freedom iDEGFD, lies in the
!           current sand set THEN
!              LOOP over all neighbor nodes
!                 IF a neighbor node, with degree of freedom
!                 jDEGFD, lies in the current shale set
!                 THEN
!                    The neighbor node has a local degree of
!                    freedom associated to it and for that local
!                    degree of freedom we get a contribution in
!                    the right hand side of the sub-system. This
!                    contribution is equal to minus the coefficient
!                    in the large matrix at position (iDEGFD,jDEGFD).
!                    See subroutine prpjSUBM for how this coefficient
!                    can be found.
!                 END IF
!              END LOOP
!           END IF
!        END LOOP
!     END IF
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppjsubb' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppjsubb'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      S => realmt(iseqmt)
      IM => intmt(S%intmatref)
      irowsii => IM%rowsii
      icolsii => IM%colsii

      allocate ( rows(size(irowsii)), cols(size(icolsii)), &
                 mat(size(S%S_ii)) )

!     --- Change to rowwise storage

      call sepcommatrev ( rows, cols, irowsii, icolsii, nrusol )
      call sepmatrev ( rows, mat, irowsii, icolsii, S%S_ii, nrusol )

!     --- There is no right hand side for the current sand set yet

      calcssol = .false.

      if ( iproj==0 ) then

!     --- nothing happens!

      else if ( nsol(ishale)==2 .and. iproj/=frstproj ) then

!     --- reuse previous solution!

         itsprojv( iproj, ishale ) = -(idxprojv+1)
         jstart = abs( itsprojv(frstproj,ishale) )

!        --- loop over all degrees of freedom in the shale layer

         do i3 = nsetnode(ishale)+1, nsetnode(ishale+1)
            inodp = setnode(i3)
            idegfd = kprobh(inodp)

!           --- Fill part of the projection vector

            idxprojv = idxprojv+1
            rtsprojv(idxprojv) = 1d0 - rtsprojv(jstart)
            jstart = jstart+1
         end do

!        --- Update estimated length of all projection vectors

         nnproj = nnproj+nlocdgfd+ninbound

      else

!     --- Calculate a new solution

         calcssol = .true.

!        --- Clear right hand side

         locrhsd = 0d0

!        --- Loop over all points which are on an internal border
!            of the shale set

         do iinbound = 1, ninbound
            idegfd = dmap(nusol+nlocdgfd+iinbound)
            inodp = kprobhinv(idegfd)
            iset = abs( nodeset(inodp) )

            if ( iset==isand ) then

!           --- Loop over all connected nodes

               do i3 = kmshe1(inodp)+1, kmshe1(1+inodp)
                  jnodp = kmshe2(i3)
                  jdegfd = kprobh(jnodp)
                  jlocdgfd = dmap(jdegfd)
                  jset = nodeset(jnodp)

                  if ( jset==ishale ) then

                     if ( idegfd<jdegfd ) then
                        i4 = rows(jdegfd)
                        searchcl = idegfd
                     else
                        i4 = rows(idegfd)
                        searchcl = jdegfd
                     end if

                     collmn = 0
100                  if ( collmn/=searchcl ) then
                        i4 = i4+1
                        collmn = cols(i4)
                     go to 100
                     end if

                     coeff = - mat(i4)
                     locrhsd(jlocdgfd) = locrhsd(jlocdgfd)+coeff
                  end if
               end do

            else

!           --- This node lies on a interface with a different sand
!               set.

               nnproj = nnproj+1

            end if

         end do

!        --- Update estimated length of all projection vectors

         nnproj = nnproj+nlocdgfd

      end if
      deallocate ( rows, cols, mat )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'calcssol', calcssol
         if ( calcssol ) call prinrl ( locrhsd, nlocdgfd, 'locrhsd' )
         write(irefwr,*) 'End seppjsubb'

      end if  ! ( debug )
      call erclos ( 'seppjsubb' )

      end subroutine seppjsubb
