      subroutine sepprojmk
! ======================================================================
!
!        programmer    Gjalt Wijma
!        version  1.0  date 14-06-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Creates projection vectors for the deflated iccg algorithm. The
!     space spanned by the projection vectors approximates the space
!     spanned by the small eigenvectors.
!     Old subroutine prprojmk
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     projection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulekprob
      use sepmodulesolve
      use sepmodulecpack
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

!                    they are not
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     SEPPJPARMAIN   Creates projection vectors for the deflated iccg algorithm
!                    Parallel case
!     SEPPROJMAIN    Creates projection vectors for the deflated iccg algorithm
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1660   The number of element groups is less than 2
!    1661   number of degrees of freedom per point is not equal to 1
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Read some information about the mesh and the problem from KMESH
!     and KPROB. Call the actual routine PRPJMAIN to create the
!     projection vectors.
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepprojmk' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepprojmk'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'nusol, nrusol', nusol, nrusol

      if ( .not.parallel .and. nelgrp<2 ) then

!     --- For the deflation technique more than one element groups
!         are required or the method is used in parallel

         call errint ( nelgrp, 1 )
         call errwar ( 1660, 1, 0, 0 )
         go to 1000

      end if

      if ( nusol/=npoint .or. nunkp/=1 ) then

!     --- Assumed is that each node has 1 degree of freedom!

         call errwar ( 1661, 0, 0, 0 )
         go to 1000

      end if

      if ( imetproj<=1 ) then

!     --- Compute new projection vectors

         if ( parallel ) then

!        --- Parallel computation

            call seppjparmain

         else

!        --- Serial computation

            call sepprojmain

         end if

      end if

1000  call erclos ( 'sepprojmk' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepprojmk'

      end if  ! ( debug )

      end subroutine sepprojmk
