      subroutine septimehis4
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-11-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill time step for time history
!     If necessary enlarge arrays
!     Old subroutine prtimehis4
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_history
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulevecs
      use sepmoduletimhis
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulecons
      use sepmoduleplot
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: lenpoints=100
      integer :: i, nch1, icoor, ipoints(lenpoints), lencop
      character (len=40) :: namear
      double precision, allocatable :: vectim2(:,:)

!     i              Counting variable
!     icoor          If>0 a timehistory of coordinates must be made
!     ipoints        Contains the nodal points where the time histor. &
!                    must be stored
!     lencop         Length of part to be copied
!     lenpoints      Declared length of array ipoints
!     namear         Name of array
!     nch1           Number of characters in name
!     vectim2        Real array to store copy of array corresponding to
!                    vectim(2,i)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     PRTIMEHIS3     Fill one time step in time history array for one specific
!                    solution array
!     PRTIMEHIS6     Fill one time step in time history array for coor array
!     PRTIMEHIS8     Fill one time step in time history array for zero level
!                    set
!     SEPACTSOLBF2   Activate sepran solution vector and related arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimehis4' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimehis4'
         write(irefwr,1) 'istephis, nstephis', istephis, nstephis
         call ertrace
  1      format ( a, 1x, (10i10) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icoor = maxval(vecttim(maxvc1+1:maxvc1+3)%npoint_hist)
      if ( ivecttimlast==0 .and. icoor==0 ) go to 1000  ! no time hist required

!     --- Raise time step

      istephis = istephis+1
      if ( istephis>nstephis ) then

!     --- Number of time steps too large, enlarge real arrays

         if ( debug ) then
            write(irefwr,*) 'Number of time steps too large'
            write(irefwr,*) 'Real array is extended'
            write(irefwr,1) 'nstephis', nstephis
         end if  ! ( debug )

         nstephis = nstephis*2  ! double nstephis

!        --- Loop over all vectors

         do i = 1, ivecttimlast

            npoint_hist = vecttim(i)%npoint_hist
            if ( npoint_hist>0 ) then

!           --- Time history of vector must be stored

               nch1  = index(names(i),' ')-1
               namear = names(i)(1:nch1)//'_timehis'

!              --- Activate arrays

               call sepactsolbf2 ( isolut2d(i) )

               ivecttim => vecttim(i)%point_nrs
               zerolev = vecttim(i)%zerolev
               call prtimehis8 ( ipoints, lenpoints )
               if ( ierror/=0 ) go to 1000

               if ( debug ) then
                  write(irefwr,1) 'i, npoint_hist, nstephis', &
                                   i, npoint_hist, nstephis
               end if  ! ( debug )

!              --- Enlarge array

               lencop = size(vecttim(i)%arr,2)
               if ( lencop<nstephis ) then

                  allocate ( vectim2(nphysi*npoint_hist+1,lencop), &
                             stat = error )
                  if ( error/=0 ) call eralloc ( error, lencop, 'vectim2' )
                  if ( ierror/=0 ) go to 1000

                  vectim2(:,1:lencop) = vecttim(i)%arr

                  deallocate ( vecttim(i)%arr )
                  allocate ( vecttim(i)%arr(nphysi*npoint_hist+1,nstephis) )
                  rvectim => vecttim(i)%arr
                  rvectim(1,:) = 0d0
                  rvectim(:,1:lencop) = vectim2

                  deallocate ( vectim2, stat = error )
                  if ( error/=0 ) call erdealloc ( error, 'vectim2' )
                  if ( ierror/=0 ) go to 1000

               end if ! ( lencop<nstephis )

            end if  ! ( vecttim(i)%npoint_hist>0 )

         end do  ! i = 1, ivecttimlast

         do i = maxvc1+1, maxvc1+ndim

            npoint_hist = vecttim(i)%npoint_hist
            if ( npoint_hist>0 ) then

!           --- Time history of vector must be stored

               namear = 'coor_timehis'

!              --- Activate arrays

               ivecttim => vecttim(i)%point_nrs
               zerolev = vecttim(i)%zerolev
               call prtimehis8 ( ipoints, lenpoints )
               if ( ierror/=0 ) go to 1000

               if ( debug ) then
                  write(irefwr,1) 'i, npoint_hist, nstephis', &
                                   i, npoint_hist, nstephis
               end if  ! ( debug )

!              --- Enlarge array

               lencop = size(vecttim(i)%arr,2)
               if ( lencop<nstephis ) then

                  allocate ( vectim2(npoint_hist+1,lencop), &
                             stat = error )
                  if ( error/=0 ) call eralloc ( error, lencop, 'vectim2' )
                  if ( ierror/=0 ) go to 1000

                  deallocate ( vecttim(i)%arr )
                  allocate ( vecttim(i)%arr(npoint_hist+1,nstephis) )
                  rvectim => vecttim(i)%arr
                  rvectim(:,1:lencop) = vectim2

                  deallocate ( vectim2, stat = error )
                  if ( error/=0 ) call erdealloc ( error, 'vectim2' )
                  if ( ierror/=0 ) go to 1000

               end if ! ( lencop<nstephis )

            end if  ! ( vecttim(i)%npoint_hist>0 )

         end do  ! i = maxvc1+1, maxvc1+ndim

      end if  ! ( istephis>nstephis )

!     --- Fill time step
!         Loop over all vectors

      do i = 1, ivecttimlast

         npoint_hist = vecttim(i)%npoint_hist

         if ( debug ) write(irefwr,1) 'i, npoint_hist', i, npoint_hist
         if ( npoint_hist>0 ) then

!        --- Time history of vector must be stored

            nch1  = index(names(i),' ')-1
            namear = names(i)(1:nch1)//'_timehis'

!           --- Activate arrays

            ivecttim => vecttim(i)%point_nrs
            zerolev = vecttim(i)%zerolev
            call prtimehis8 ( ipoints, lenpoints )
            if ( ierror/=0 ) go to 1000

            call sepactsolbf2 ( isolut2d(i) )

!           --- Compute starting positions

            rvectim => vecttim(i)%arr
            vectin => ks(isolut2d(i))%sol

!           --- Fill time step

            call prtimehis3 ( ipoints )

         end if  ! ( vecttim(i)%npoint_hist>0 )

      end do  ! i = 1, ivecttimlast

      do i = maxvc1+1, maxvc1+ndim

         npoint_hist = vecttim(i)%npoint_hist
         if ( npoint_hist>0 ) then

!        --- Time history of coor vector must be stored

            namear = 'coor_timehis'

!           --- Activate arrays

            ivecttim => vecttim(i)%point_nrs
            zerolev = vecttim(i)%zerolev
            call prtimehis8 ( ipoints, lenpoints )
            if ( ierror/=0 ) go to 1000

!           --- Fill first time step

            rvectim => vecttim(i)%arr
            call prtimehis6 ( i-maxvc1, ipoints )

         end if  ! ( vecttim(i)%npoint_hist>0 )

      end do  ! i = maxvc1+1, maxvc1+ndim

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septimehis4'

      end if  ! ( debug )
      call erclos ( 'septimehis4' )

      end subroutine septimehis4
