      subroutine septimprb ( finished, first, isubr )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 13-12-2017 New storage of intmat
!        version  1.0  date 28-10-2015
!
!   copyright (c) 2015-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     solves time-dependent problems.
!     Old subroutine timprbnew
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduletime
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: isubr
      logical, intent(inout) :: finished, first

!     finished      i/o   Indicates if the loop has finished (true) or not
!                         (false)
!     first         i/o   If true it concerns the first step
!     isubr          i    Indicates the type of calling subroutine
!                         Possible values:
!                         1:  timprb
!                         2:  prtimd
!                         3:  timefrbf
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: eps
      logical :: finishedloc
      double precision :: timebefore(2)

!     eps            Local accuracy
!     finishedloc    Indicates if the loop has finished (true) or not
!                    (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLMN         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERSETTIME      Get the CPU time at start of subroutine
!     SEPTIMOUTP     Perform output for subroutine septimprb during a time loop
!     SEPTIMROUT     General time integration of a discretized partial
!                    differential equation of first or second order in time
!     SEPTIMSET      Set some time parameters in the time loop
!     SEPTIMSTART    Start part of subroutine septimprb
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimprb' )
      if ( itime>0 ) call ersettime ( timebefore )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimprb'
         write(irefwr,2) 't, t0, tstep, tout', t, t0, tstep, tout
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      massmat => massmt(1)
      stiffmat => stiffm
      totmat => mattot(1)

!     --- Start part of subroutine septimprb
!         Fill input arrays and constants and write initial condition

      call septimstart ( first, eps, isubr )
      if ( ierror/=0 ) go to 1000

!     --- while loop performing the time steps

      finishedloc = itimloop<1
      do while ( .not. finishedloc .and. itimloop>=1 )

!     --- First set tstep and tout

         call septimset ( first, eps )

!        --- Call time integration subroutine

         isolut2dred => isolut2d
         call septimrout ( first, finished )
         if ( ierror/=0 ) go to 1000

         call septimoutp ( eps )
         if ( ierror/=0 ) go to 1000

         if ( .not. complrun .or. oldmethod ) finishedloc = .true.

         if ( debug ) then
            write(irefwr,*) 'finishedloc', finishedloc
            write(irefwr,1) 'itimloop', itimloop
            write(irefwr,2) 'time', t
         end if  ! ( debug )

      end do  ! ( while .not. finishedloc .and. itimloop>=1 )

      if ( complrun ) itimloop = 0

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septimprb'

      end if  ! ( debug )
      call erclmn ( 'septimprb', timebefore )

      end subroutine septimprb
