      subroutine septimset ( first, eps )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Set some time parameters in the time loop
!     Old subroutine prtimprb02
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical, intent(in) :: first
      double precision, intent(in) :: eps

!     eps            i    Local accuracy
!     first          i    If true it concerns the first step
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPTIMEHIS2    Create real arrays for time history and fill partly
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimset' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimset'
         write(irefwr,2) 't, tout, eps', t, tout, eps
         write(irefwr,*) 'first, usectimen', first, usectimen
         write(irefwr,1) 'itiminterval, keeptim', itiminterval, keeptim
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if
      if ( ierror/=0 ) go to 1000

      if ( .not. first .and. keeptim<=0 ) then

!     --- first = false, not the first time step

         tendnext = arrendtime(itiminterval)
         if ( debug ) write(irefwr,2) 'tendnext', tendnext
         if ( tendnext<=t+eps ) then

!        --- tendnext>t, raise itiminterval

            itiminterval = min(ndt,itiminterval+1)

         end if

      end if

      if ( first ) then

!     --- First call in a time loop
!         Set t, t0, tend and so on

         itiminterval = 1
         if ( toutstep<=0d0 ) toutstep = tstep
         toutnext = min(max(toutinit,t+toutstep),toutend)

         call septimehis2

      end if  ! ( first )

      if ( keeptim<=1 ) then

!     --- Not reuse_t found

         if ( .not. usectimen ) then
            tstep  = arrtimestep(itiminterval)
            tendnext = arrendtime(itiminterval)
            if ( ichtim==9 ) theta  = arrtheta(itiminterval)
         end if

         if ( .not. oldmethod ) then

!        --- standard method, adapt tout

            tout = min ( tendnext, toutnext )
            tout = max(t,tout)

         end if  ! ( .not. oldmethod )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,2) 't, tstep, tendnext', t, tstep, tendnext
         write(irefwr,2) 'toutnext, tout, theta', toutnext, tout, theta
         write(irefwr,*) 'End septimset'

      end if
      call erclos ( 'septimset' )

      end
