      subroutine septimtheta2 ( tsteps, ieqn )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-11-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      Set some parameters during a time loop
!      Old subroutine pr110502
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ieqn
      double precision, intent(inout) :: tsteps

!     ieqn           i    Equation sequence number
!     tsteps         o    help variable to save the value of tstep
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimtheta2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimtheta2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nitertim = nitertim + 1
      if ( nfrac<=1 ) then

!     --- standard situation

         dt = theta*tstep
         if ( abs(tstep-tsteps)>100d0*epsmac ) then
            Teq(ieqn)%imasstot = 0
            tsteps = tstep
         end if

      else

!     --- fractional step method

         Teq(ieqn)%imasstot = 0
         dt = arrtheta(ifrac+1)*tstep
         theta = arrtheta(ifrac+1)/(arrtheta(ifrac+1)+arrtheta(ifrac))

      end if

      if ( (complrun .or. keeptim/=1) .and. ieqn==1 )  t = t+dt

      if ( .not. stationarysol ) diffsl(1:ncopeqtim,1:3) = 0d0

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septimtheta2'

      end if  ! ( debug )
      call erclos ( 'septimtheta2' )

      end subroutine septimtheta2
