      subroutine septimthetafrc ( islold, isolprec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-11-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Extrapolate the solution from old and intermediate time step to
!     next time step in case of theta method
!     Old subroutine: pr1105frac
! **********************************************************************
!
!                       KEYWORDS
!
!     time_dependent
!     time_integration
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmoduletime
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: islold(*)
      integer, intent(in) :: isolprec(*)

!     islold        i/o   User input array in which the user puts information
!                         of all preceding solutions
!                         Integer array of length 5 x numold
!                         At output the new solutions are stored in this array, &
!                         at input the initial conditions must have been stored
!                         in islold
!     isolprec       i    Solution at preceding level
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iinvec(2), ieqn
      double precision :: rinvec(2)

!     ieqn           Equation sequence number
!     iinvec         User input array for subroutine MANVEC
!                    For a description see subroutine MANVEC
!     rinvec         Real user input array for subroutine MANVEC
!                    The contents depend on iinvec
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPMANVEC      Vector manipulation subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     u^n+1 := 1/theta u^(n+theta) - (1-theta)/theta u^n
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'septimthetafrc' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from septimthetafrc'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iinvec(1) = 2     ! number of entries filled
      iinvec(2) = 27    ! linear combination

!     --- old solution is stored in isolprec
!         u^n+1 := 1/theta u^(n+theta) -(1-theta)/theta u^n

      rinvec(1) = 1d0/theta              !first factor
      rinvec(2) = -(1d0-theta)*rinvec(1) !second factor
      do ieqn = 1, ncopeqtim
         call sepmanvec ( iinvec, rinvec, &
                          islold(Teq(ieqn)%iseqfr), isolprec(ieqn), &
                          islold(Teq(ieqn)%iseqfr) )
      end do

      if ( nfrac<=1 ) then

!     --- standard situation

         t = t + (1d0-theta)*tstep

      else

!     --- fractional step method

         t = t + arrtheta(ifrac)*tstep
         ifrac = mod(ifrac+2,nfrac)

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End septimthetafrc'

      end if  ! ( debug )
      call erclos ( 'septimthetafrc' )

      end subroutine septimthetafrc
