      subroutine toaddmatif ( relemmt )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-05-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Assemble one element matrix in the matrix part S_fi
!     real matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     assembly
!     matrix
!     real
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulebuild
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: relemmt(icount, icount)

!     relemmt        i    element matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, nsum, k, inusol, iamat, knusol

!                    they are not
!     i              Counting variable
!     iamat          Position in array AMAT
!     inusol         Position of unknown in array USOL
!     k              Counting variable
!     knusol         Position of unknown in array USOL
!     nsum           nrusol+ninterface+1, i.e. position of first prescribed
!                    unknown
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toaddmatif' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from toaddmatif'
         call prinin ( indexren, icount, 'indexren' )
         call prinrl1 ( relemmt, icount, icount, 'relemmt' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      nsum = nrusol+ninterface+1

      if ( debug ) &
         write(irefwr,1) 'nrusol, ninterface, nsum', &
                          nrusol, ninterface, nsum

!     --- for every interface d.o.f

      do i = 1, icount

!     --- inusol := global seguence number of local
!         precribed degree of freedom i

         inusol = indexren(i)
         if ( inusol<=nrusol ) then

            if ( debug ) write(irefwr,1) 'i, inusol', i, inusol

!           --- fill iwork with addresses in amat of all free degrees of
!               freedom coupled to inusol

            do k = irowsif(inusol)+1, irowsif(inusol+1)
               iwork(icolsif(k)) = k
            end do

!           --- for every free d.o.f coupled to inusol
!               construct Sfi

            do k = 1, icount

               knusol = indexren(k)

               if ( knusol>nrusol .and. knusol<nsum ) then

!              --- only interface unknowns

                  iamat = iwork(knusol)

                  if ( debug ) &
                     write(irefwr,1) 'k, knusol, iamat', &
                                      k, knusol, iamat

!                 --- add the entry to S_fi

                  Smat%S_if(iamat) = Smat%S_if(iamat)+relemmt(k,i)

               end if  ! ( knusol>nrusol .and. knusol<nsum )

            end do  ! k = 1, icount

         end if  ! ( inusol<=nrusol )

      end do  ! i = 1, icount

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End toaddmatif'

      end if  ! ( debug )
      call erclos ( 'toaddmatif' )

      end subroutine toaddmatif
