      subroutine chseq1 ( imidp, ivertx, icentr, ilevel, levels )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 18-12-2002 Extension of shapes
!        version  1.0  date 03-01-1995
!
!   copyright (c) 1995-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Renumber nodes such that vertices, mid points and centroids get a
!     sequence depending on the priority
! **********************************************************************
!
!                       KEYWORDS
!
!     renumbering
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: imidp, ivertx, icentr, ilevel, levels(nlevel)

!     icentr  i    priority number corresponding to centroid
!     ilevel  i    If 1 renumbering must be performed per level, if 0 not
!     imidp   i    priority number corresponding to mid point
!     ivertx  i    priority number corresponding to vertex
!     levels  i    Contains the levels in the mesh
!                  Level 1 consists of all nodes from 1 to levels(1)
!                  Level i consists of all nodes from levels(i-1) to
!                  levels(i)  (i>1)
!                  If the mesh is renumbered then
!                  Level 1 consists of all nodes from kmeshj(1) to
!                  kmeshj(levels(1))
!                  Level i consists of all nodes from kmeshj(levels(i-1)) to
!                  kmeshj(levels(i))  (i>1)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, icount, ielgrp, ikelmc, inodp, inpelm, &
              ivert(8), j, iupp, ilow, ilev, istore(npoint), iwork(npoint)

!     i       Counting variable
!     icount  number of unknowns per element
!     ielgrp  element group number
!     ikelmc  position in kmesh part c
!     ilev    Level sequence number
!     ilow    Lower bound of loop
!     inodp   Nodal point number
!     inpelm  number of points per element
!     istore        integer work array of length npoint
!     iupp    Upper bound of loop
!     ivert   Array containing the local positions of the vertices in
!             the standard elements
!     iwork        integer work array of length npoint
!     j       Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Give array iwork the priority of the mid points
!     Find all vertices and give iwork the corresponding priority
!     Find all centroids and give iwork the corresponding priority
!     Copy kmeshj into istore
!     Copy all nodes from istore in the sequence given by the priority
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'chseq1' )

!     --- Give array iwork the priority of the mid points

      iwork = imidp

!     --- Find all vertices and give iwork the corresponding priority
!         Find all centroids and give iwork the corresponding priority

      ikelmc = 0
      do ielgrp = 1, nelgrp
         ishape = kmeshf(ielgrp)
         inpelm = Topgrp(ielgrp)%inpelm
         inelem = Topgrp(ielgrp)%inelem
         if ( ishape==1 .or. ishape==3 .or. ishape.eq.5 .or. &
              ishape==11 .or. ishape==13 .or. ishape.eq.21 ) then

!        --- Linear element

            do i = 1, inelem
               do j = 1, inpelm
                  iwork(kmeshc(ikelmc+j+(i-1)*inpelm)) = ivertx
               end do
            end do
         else if ( ishape==2 .or. ishape==4 .or. ishape.eq.6 .or. &
                   ishape==7 ) then

!        --- Quadratic line or surface element

            do i = 1, inelem
               do j = 1, inpelm, 2
                  iwork(kmeshc(ikelmc+j+(i-1)*inpelm)) = ivertx
               end do
            end do
            if ( ishape==6 .or. ishape==7 ) then

!           --- ishape = 6 or 7, centroid present

               do i = 1, inelem
                  iwork(kmeshc(ikelmc+i*inpelm)) = icentr
               end do
            end if
         else if ( ishape==8 ) then

!        --- Special surface element

            do i = 1, inelem
               do j = 1, 7, 3
                  iwork(kmeshc(ikelmc+j+(i-1)*inpelm)) = ivertx
               end do
            end do
            do i = 1, inelem
               iwork(kmeshc(ikelmc+i*inpelm)) = icentr
            end do
         else if ( ishape==12 .or. ishape==15 .or. ishape.eq.16 ) then

!        --- Quadratic tetrahedron

            ivert(1) = 1
            ivert(2) = 3
            ivert(3) = 5
            ivert(4) = 10
            do i = 1, inelem
               do j = 1, 4
                  iwork(kmeshc(ikelmc+ivert(j)+(i-1)*inpelm)) = ivertx
               end do
            end do
            if ( ishape==16 ) then

!           --- ishape = 16, centroid present

               do i = 1, inelem
                  iwork(kmeshc(ikelmc+i*inpelm)) = icentr
               end do
            end if
         else if ( ishape==14 ) then

!        --- Quadratic hexahedron

            ivert(1) = 1
            ivert(2) = 3
            ivert(3) = 7
            ivert(4) = 9
            ivert(5) = 19
            ivert(6) = 21
            ivert(7) = 25
            ivert(8) = 27
            do i = 1, inelem
               do j = 1, 8
                  iwork(kmeshc(ikelmc+ivert(j)+(i-1)*inpelm)) = ivertx
               end do
            end do
            do i = 1, inelem
               iwork(kmeshc(ikelmc+14+(i-1)*inpelm)) = icentr
            end do
         else

!        --- Shape number not implemented

            call errint ( ishape, 1 )
            call errsub ( 800, 1, 0, 0 )
         end if
         ikelmc = ikelmc + inpelm*inelem
      end do

!     --- Copy kmeshj into istore

      istore = kmeshj

!     --- Copy all nodes from istore in the sequence given by the priority

      icount = 0
      if ( ilevel==0 ) then

!     --- Global renumbering

         iupp = npoint
         nlevel = 1
      end if
      ilow = 1
      do ilev = 1, nlevel
         if ( ilevel/=0 ) iupp = levels(ilev)
         do i = ilow, iupp
            inodp = istore(i)
            if ( iwork(inodp)==1 ) then
               icount = icount + 1
               kmeshj(icount) = inodp
            end if
         end do
         do i = ilow, iupp
            inodp = istore(i)
            if ( iwork(inodp)==2 ) then
               icount = icount + 1
               kmeshj(icount) = inodp
            end if
         end do
         do i = ilow, iupp
            inodp = istore(i)
            if ( iwork(inodp)==3 ) then
               icount = icount + 1
               kmeshj(icount) = inodp
            end if
         end do
         ilow = iupp + 1
      end do
      call erclos ( 'chseq1' )
      end
