      subroutine incm011 ( iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.0  date 06-01-2016 Use sepmodulematstr
!        version  4.0  date 17-12-2015 New parameter list
!
!   copyright (c) 1999-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Mark positions in array iwork corresponding to periodical boundary
!     conditions and creat chain of nodes
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulematstr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(3,*)

!     iwork          o    work array of length 3*npoint to indicate if a point
!                         is periodical or not
!                         If iwork(1,i) = 0 then point is not periodical
!                         For a chain of periodical points, iwork is
!                         filled as follows:
!                         iwork(1,first_node_in_chain) = sequence number in
!                         iwork(2,.)
!                         iwork(1,next_nodes in chain) = -first_node_in_chain
!                         iwork(2,.) contains for each
!                         chain of periodical points the number of points that
!                         are connected to the first point. These numbers are
!                         accumulated
!                         iwork(2,1) = 1
!                         iwork(2,i+1)-iwork(2,i) = number of points connected
!                         to nodal point i
!                         iwork(3,.) contains the nodal points
!                         connected to the first point in a chain in the
!                         sequence given by iwork(2,.)
!                         Hence iwork(3,.) positions iwork(2,i) to
!                         iwork(2,i+1)-1
!                         contain all nodes from the chain starting with node i
!                         iwork is used in subroutine incm02
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ielgrp, ikelmc, nodref, nodfir, isum, itype, nodp1, &
              ichain, nodalp, maxchn, nodp, iseqlc, iseqol, inpelm, &
              iseqlc1
      integer, allocatable :: ihelp(:)
      logical found

!     found          Indicates if changes have been found (true) or not (false)
!                    nor (second part) indicates if the starting point of a
!                    chain has been found (true) or not (false)
!     i              Counting variable
!     ichain         ?
!     ielgrp         element group number
!     ihelp          work array to compute the chain
!                    Futhermore it is used to indicate if a point belongs to
!                    the set of periodical points corresponding to a specific
!                    periodic point in the following way:
!                    ihelp(i) = 1, point i corresponds to the periodic point
!                    ihelp(i) = 0, it does not
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     iseqlc         Sequence number of vector
!     iseqlc1        Sequence number of vector in iwork(2,.)
!     iseqol         ?
!     isum           Integer variable to compute a sum
!     itype          Table containing information about the numbers behind
!                    the keywords in the table code
!     maxchn         maximum length of chain
!     nodalp         Nodal point sequence number
!     nodfir         ?
!     nodp           Nodal point sequence number
!     nodp1          Node sequence number
!     nodref         ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INCM011LOOP    Mark positions in array iwork corresponding to periodical
!                    boundary conditions and creat chain of nodes
!                    Sub case for one element group
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Fill periodical points into IWORK
!        First fill negative numbers in iwork using the same technique as in
!        topbv5:
!           max_chain := 1
!           For ielgrp := 1 (1) nelgrp do
!           if ( element = periodical ) then
!              For iielem := 1 (1) number of elements of group ielgrp do
!                 nodp_1 := first node in element
!                 nodp_2 := second node in element
!                 if ( iwork(1,nodp_2) = 0 ) then
!                    iwork(1,nodp_2) := nodp_1
!                 else if ( iwork(1,nodp_1) = 0 ) then
!                    iwork(1,nodp_1) := nodp_2
!                 else ( iwork(1,nodp_1)#0  and iwork(1,nodp_2)#0 )
!                    Reverse chain starting from nodp_2;
!                       store nodes of chain in ihelp:
!                       ihelp(1) := nodp_2
!                       ichain := 1
!                       nodalp := iwork(1,nodp_2)
!                       while ( nodalp#0 ) do
!                          ichain := ichain + 1
!                          ihelp(ichain) := nodalp
!                          nodalp := iwork(1,nodalp)
!                          For i := ichain (-1) 2 do
!                             iwork(1,ihelp(i)) := ihelp(i-1)
!                          iwork(1,ihelp(1)) := nodp_1
!                          max_chain := max ( max_chain, ichain )
!           Remove points between and make single references:
!              For ichain := 1 (1) max_chain do
!                 For nodp := 1 (1) npoint do
!                    nod_ref := iwork(1,nodp)
!                    while ( nod_ref # 0 ) do
!                       if ( iwork(1,nod_ref) # 0 ) then
!                          iwork(1,nodp) := nod_ref
!                          nod_ref := iwork(1,nod_ref)
!           Reverse signs in iwork
!           make references in chain to first node
!        Next fill array iwork(2,.):
!        iseqlc := 1
!        For i := 1 (1) npoint do
!           if ( iwork(1,i)<0 ) then
!              nodp_1 := -iwork(1,i)
!              if ( iwork(1,nodp_1)=0 ) then
!                 iwork(2,iseqlc+1) := 1
!                 iwork(1,nodalp) := iseqlc
!                 iseqlc := iseqlc+1
!              else ( iwork(1,nodp_1)>0 ) then
!                 iseqol := iwork(1,nodp_1)
!                 iwork(2,iseqol+1) := iwork(2,iseqol+1) + 1
!        Accumulate iwork(2,.)
!        Finally fill IWORK(3,.) using ihelp as work array
!        For i := 1 (1) iseqlc
!           ihelp(i) := 0
!        For i := 1 (1) npoint do
!           if ( iwork(1,i)<0 ) then
!              nodp_1 := -iwork(1,i)
!              iseqlc := iwork(1,nodp_1)
!              iwork(3,iwork(2,iseqlc)+ihelp(iseqlc)) := i
!              ihelp(iseqlc) := ihelp(iseqlc)+1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incm011' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incm011'
         write(irefwr,1) 'npoint, nelem', npoint, nelem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Mark positions in iwork corresponding to periodical boundary
!         conditions

      ikelmc = 1
      maxchn = 0
      allocate ( ihelp(npoint), stat=error )
      if ( error/=0 ) call eralloc ( error, npoint, 'ihelp' )
      if ( ierror/=0 ) go to 1000
      ihelp = 0

!     --- Loop over element groups

      do ielgrp = 1, nelgrp

         itype = kprobc(ielgrp)
         inpelm = sepgetinpelm(ielgrp)
         inelem = sepgetinelem(ielgrp)
         if ( debug ) &
            write(irefwr,1) 'ielgrp, itype, inpelm, inelem', &
                             ielgrp, itype, inpelm, inelem
         if ( itype==-1 ) then

!        --- Element with periodical boundary conditions

            call incm011loop ( kmeshc(ikelmc), iwork, ihelp, maxchn, inelem )

         end if
         ikelmc = ikelmc+inelem*inpelm

      end do

      if ( debug ) then
         write(irefwr,1) 'numpairs, nelgrp', numpairs, nelgrp
         call prinin1 ( iwork, npoint, 3, 'iwork/1' )
      end if  ! ( debug )

      if ( numpairs>0 ) then

!     --- numpairs (numpairs)>0, hence periodical boundary conditions

         call incm011loop ( kprobper, iwork, ihelp, maxchn, numpairs )

      end if  ! ( numpairs>0 )

      if ( debug ) write(irefwr,1) 'maxchn', maxchn

!     --- Remove points between and make single references:

      do ichain = 1, maxchn

         found = .false.

         do nodp = 1, npoint

            nodref = iwork(1,nodp)
            nodfir = 0

!           --- while loop

210         if ( nodref/=0 .and. nodfir/=nodref ) then

!           --- node with reference found

               if ( iwork(1,nodref)/=0 ) then

!              --- nodref refers to another node

                  nodfir = nodref
                  iwork(1,nodp) = iwork(1,nodref)
                  nodref = iwork(1,nodref)
                  found = .true.
                  go to 210

               end if

            end if

         end do

         if ( .not. found ) go to 260

      end do

!     --- Reverse sign in iwork and fill positive numbers

260   iwork(1,1:npoint) = -iwork(1,1:npoint)

      if ( debug ) call prinin1 ( iwork, npoint, 3, 'iwork' )

!     --- Make references in chain all to first node in chain

      do i = 1, npoint

         if ( iwork(1,i)==-i ) then
            iwork(1,i) = i
         else if ( iwork(1,i)<0 ) then

!        --- nodal point in chain found

            nodalp = -iwork(1,i)
            if ( iwork(1,nodalp)<0 ) then

!           --- Not the basis nodal point, search for this point

               found = .false.
               nodp1 = -iwork(1,nodalp)
280            if ( .not. found ) then

                  if ( iwork(1,nodp1)==0 ) then
                     found = .true.
                  else
                     nodp1 = -iwork(1,nodp1)
                  end if
                  go to 280

               end if

!              --- Make references to first point

               found = .false.
290            if ( .not. found ) then

                  iwork(1,nodalp) = -nodp1
                  nodalp = -iwork(1,nodalp)
                  if ( iwork(1,nodalp)==0 ) then
                     found = .true.
                  else
                     nodalp = -iwork(1,nodalp)
                  end if
                  go to 290

               end if

            end if

         end if

      end do
      if ( debug ) call prinin1 ( iwork, npoint, 3, 'iwork/2' )

!     --- Fill array iwork(2,.)

      iseqlc = 1
      do i = 1, npoint

         if ( iwork(1,i)<0 ) then

!        --- Point in a chain found

            nodp1 = abs(iwork(1,i))
            if ( debug ) &
               write(irefwr,1) 'i, nodp1, iwork(1,nodp1)', &
                                i, nodp1, iwork(1,nodp1)
            if ( iwork(1,nodp1)==0 ) then

!           --- First appearance of starting point of the sequence

               iwork(1,nodp1) = iseqlc
               iwork(2,iseqlc+1) = 1
               iseqlc = iseqlc + 1

            else

!           --- Next appearance of starting point of the chain

               do while ( iwork(1,nodp1)<0 )
                  nodp1 = -iwork(1,nodp1)
               end do  ! while ( iwork(1,nodp1)<0 )
               iseqol = abs(iwork(1,nodp1))
               if ( debug ) write(irefwr,1) 'iseqol', iseqol
               iwork(2,iseqol+1) = iwork(2,iseqol+1) + 1

            end if

         end if

      end do
      if ( debug ) call prinin1 ( iwork, npoint, 3, 'iwork/3' )

!     --- Accumulate iwork(2,.)

      isum = 1
      iwork(2,1) = 1
      do i = 2, iseqlc
         isum = isum + iwork(2,i)
         iwork(2,i) = isum
      end do
      if ( debug ) call prinin1 ( iwork, npoint, 3, 'iwork/4' )

!     --- Fill array iwork(3,.)

      ihelp(1:iseqlc) = 0
      do i = 1, npoint

         if ( iwork(1,i)<0 ) then

!        --- Point in chain found

            nodp1 = abs(iwork(1,i))
            do while ( iwork(1,nodp1)<0 )
               nodp1 = -iwork(1,nodp1)
            end do  ! while ( iwork(1,nodp1)<0 )
            iseqlc = abs(iwork(1,nodp1))
            iseqlc1 = iwork(2,iseqlc)
            if ( debug ) then
               write(irefwr,1) 'i, nodp1, iseqlc, iseqlc1', &
                                i, nodp1, iseqlc, iseqlc1
               write(irefwr,1) 'iwork(2,iseqlc), ihelp(iseqlc1)', &
                                iseqlc1, ihelp(iseqlc1)
            end if  ! ( debug )
            iwork(3,iseqlc1+ihelp(iseqlc1)) = i
            ihelp(iseqlc1) = ihelp(iseqlc1) + 1

         end if

      end do

      deallocate ( ihelp, stat=error )
      if ( error/=0 ) call erdealloc ( error, 'ihelp' )

1000  if ( debug ) then

!     --- Debug information

         call prinin1 ( iwork, npoint, 3, 'iwork' )
         write(irefwr,*) 'End incm011'

      end if  ! ( debug )
      call erclos ( 'incm011' )

      end
