      subroutine incm01imat2 ( nfdegmid, kfree, nfdeg, &
                               numnei, istore, numneimid, imark )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmodulematstr
!        version  2.0  date 17-12-2015 New parameter list
!
!   copyright (c) 2007-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array irowsii
!     Special case of decoupled unknowns
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nfdegmid, kfree(2,*), nfdeg, numnei, &
              istore(nusol,2), numneimid, imark(nunkp)

!     imark          i    Work array of length nunkp to indicate which degrees
!                         of freedom are coupled or decoupled per node
!                         imark(i) = 0, degree of freedom is coupled to all
!                                       others
!                         imark(i) = 1, degree of freedom is coupled to all
!                                       degrees of freedom with imark(i) = 0
!     istore         i    work array of length nusol to store the non-prescribed
!                         neighbour degrees of freedom of a point
!                         At input this array is 0
!                         At output it contains neighbours of periodical nodes
!     kfree          i    Array of length nunkp to store the non-prescribed
!                         degrees of freedom in a node
!     nfdeg          i    Number of free degrees of freedom in a point
!     nfdegmid       i    Number of free degrees of freedom corresponding to the
!                         first ndim+1 unknowns
!     numnei         i    Number of neighbors
!     numneimid      i    Number of standard neighbors
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, isum, knusol, j, iunk, junk

!     i              Counting variable
!     isum           Integer variable to compute a sum
!     iunk           Value of i-th degree of freedom in imark
!     j              Counting variable
!     junk           Value of j-th degree of freedom in imark
!     knusol         Position of unknown in array USOL
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incm01imat2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incm01imat2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = 1, nfdegmid

         isum   = 0
         knusol = kfree(1,i)
         iunk = kfree(2,i)
         if ( imark(iunk)==0 ) then

!        --- imark(iunk) = 0, degree of freedom is coupled to all others
!            Standard case

            do j = 1, nfdeg
               if ( kfree(1,j)<knusol ) isum = isum+1
            end do
            do j = 1, numneimid
               if ( istore(j,1)<knusol ) isum = isum+1
            end do

         else

!        --- imark(iunk) = 1, degree of freedom is not coupled to all others
!            Special case

            do j = 1, nfdeg

               junk = kfree(2,j)
               if ( iunk==junk .or. imark(junk)==0 ) then

!              --- Either i = j or imark(junk) = 0
!                  Hence unknown itself or junk is coupled to all

                  if ( kfree(1,j)<knusol ) isum = isum+1

               end if  ! ( iunk==junk .or. imark(junk)==0 )

            end do  ! j = 1, nfdeg

            do j = 1, numneimid

               junk = istore(j,2)
               if ( iunk==junk .or. imark(junk)==0 ) then

!              --- Either i = j or imark(junk) = 0
!                  Hence unknown itself or junk is coupled to all

                  if ( istore(j,1)<knusol ) isum = isum+1

               end if  ! ( iunk==junk .or. imark(junk)==0 )

            end do  ! j = 1, numneimid

         end if  ! ( imark(i)==0 ) then

         irowsii(knusol+1) = isum

      end do  ! i = 1, nfdegmid

      do i = nfdegmid+1, nfdeg

         isum   = 0
         knusol = kfree(1,i)
         iunk = kfree(2,i)
         if ( imark(iunk)==0 ) then

!        --- imark(iunk) = 0, degree of freedom is coupled to all others
!            Standard case

            do j = 1, nfdeg
               if ( kfree(1,j)<knusol ) isum = isum+1
            end do
            do j = 1, numnei
               if ( istore(j,1)<knusol ) isum = isum+1
            end do

         else

!        --- imark(iunk) = 1, degree of freedom is not coupled to all others
!            Special case

            do j = 1, nfdeg

               junk = kfree(2,j)
               if ( iunk==junk .or. imark(junk)==0 ) then

!              --- Either i = j or imark(junk) = 0
!                  Hence unknown itself or junk is coupled to all

                  if ( kfree(1,j)<knusol ) isum = isum+1

               end if  ! ( iunk==junk .or. imark(junk)==0 )

            end do  ! j = 1, nfdeg

            do j = 1, numnei

               junk = istore(j,2)
               if ( iunk==junk .or. imark(junk)==0 ) then

!              --- Either i = j or imark(junk) = 0
!                  Hence unknown itself or junk is coupled to all

                  if ( istore(j,1)<knusol ) isum = isum+1

               end if  ! ( iunk==junk .or. imark(junk)==0 )

            end do  ! j = 1, numnei

         end if  ! ( imark(i)==0 )

         irowsii(knusol+1) = isum

      end do  ! i = nfdegmid+1, nfdeg

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End incm01imat2'

      end if  ! ( debug )
      call erclos ( 'incm01imat2' )

      end
