      subroutine incm01period ( node, nodp1, icheck, iwork, icoupled, &
                                nfdeg, numnei, kfree, iseqlc, &
                                istore, nrep, ihelpunk )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmodulematstr
!        version  2.0  date 17-12-2015 New parameter list
!        version  1.0  date 12-10-2007
!
!   copyright (c) 2007-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt icheck, kfree and istore for periodical nodes
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematstr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer node, nodp1, icheck(*), &
              iwork(3,*), icoupled, nfdeg, numnei, &
              kfree(2,*), iseqlc, istore(nusol,2), nrep, ihelpunk(*)

!     icheck        i/o   work array of length nusol to indicate if a degree of
!                         is present in array istore (1) or not (0)
!     icoupled       i    If 0 the momentum equations are coupled, if 1 they
!                         are not
!     ihelpunk       i    Work array indicating if unknowns in coupled nodes are
!                         periodical (0) or not (1)
!     iseqlc           i    Sequence number
!     istore        i/o   work array of length nusol to store the non-prescribed
!                         neighbour degrees of freedom of a point
!     iwork          i    work array of length 3*npoint to indicate if a point
!                         is periodical or not
!                         See subroutine incm01basis
!     kfree         i/o   Array of length nunkp to store the non-prescribed
!                         degrees of freedom in a node
!     nfdeg          i    Number of free degrees of freedom in a point
!     node           i    Node number
!     nodp1          i    Node sequence number
!     nrep           i    ?
!     numnei        i/o   Number of neighbors
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inusol, knusol, i, j, inunkp

!     i              Counting variable
!     inunkp         Number of degrees of freedom in point
!     inusol         Position of unknown in array USOL
!     j              Counting variable
!     knusol         Position of unknown in array USOL
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incm01period' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incm01period'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nodp1 = node
      inusol = sepgetinusol ( nodp1, inunkp )
      do i = 1, inunkp
         icheck(kprobr(inusol+i)) = 1
      end do
      do j = iwork(2,iseqlc), iwork(2,iseqlc+1) - 1
         nodp1 = iwork(3,j)
         inusol = sepgetinusol ( nodp1, inunkp )
         do i = 1, inunkp
            icheck(kprobr(inusol+i)) = 1
         end do
      end do

      nrep = 1
      if ( icoupled==0 ) then
         nodp1 = node
      else
         nodp1 = iwork(3,iwork(2,iseqlc)+icoupled-1)
      end if
      icoupled = icoupled+1

!     --- Fill array kfree and compute nfdeg

      inusol = sepgetinusol ( nodp1, inunkp )
      nfdeg = 0
      numnei = 0
      do i = 1, inunkp
         knusol = kprobr(inusol+i)
         if ( knusol<=nrusol ) then

!        --- free unknown found

            if ( ihelpunk(i)>0 ) then

!           --- Unknown differs in periodical nodes, store in kfree

               nfdeg = nfdeg+1
               kfree(1,nfdeg) = knusol
               kfree(2,nfdeg) = i
               icheck(knusol) = 1

            else

!           --- Unknown is periodical, store in istore

               numnei = numnei+1
               istore(numnei,1) = knusol
               istore(numnei,2) = i
               icheck(knusol) = 1

            end if

         end if

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End incm01period'

      end if  ! ( debug )
      call erclos ( 'incm01period' )

      end
