      subroutine incm02icol ( nfdegmid, kfree, imat1, nfdeg, &
                              numnei, istore, numneimid, iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmodulematstr
!        version  2.1  date 30-03-2014 Extension with jmethod = 52/53
!        version  2.0  date 26-02-2014 Extra parameter iwork
!
!   copyright (c) 2007-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array icolsii with column numbers of non-zero entries
!     per row of the matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nfdegmid, kfree(2,*), imat1(*), nfdeg, numnei, &
              numneimid, istore(nusol,3), iwork(*)

!     imat1          i    Integer array of length n+1, containing the number of
!                         elements in each row of the lower triangular matrix
!                         accumulated. The first position contains zero, the
!                         last the number of non-zero elements in the
!                         lower-triangular matrix
!     istore         i    work array of length nusol to store the non-prescribed
!                         neighbour degrees of freedom of a point
!                         At input this array is 0
!                         At output it contains neighbours of periodical nodes
!     iwork               work array of length nrusol to be used for sorting
!     kfree          i    Array of length nunkp to store the non-prescribed
!                         degrees of freedom in a node
!     nfdeg          i    Number of free degrees of freedom in a point
!     nfdegmid       i    Number of free degrees of freedom corresponding to the
!                         first ndim+1 unknowns
!     numnei         i    Number of neighbors
!     numneimid      i    Number of standard neighbors
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, knusol, isum, j, ipt, nh, ksum
      logical :: islong
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     i              Counting variable
!     ipt
!     islong
!     isum           Starting address in icolsii
!     j              Counting variable
!     knusol         Position of unknown in array USOL
!     ksum           Starting address in icolsii
!     nh             numnei+nfdeg
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incm02icol' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incm02icol'
         write(irefwr,1) 'nusol, nfdeg', nusol, nfdeg
         write(irefwr,1) 'nfdegmid, numneimid', nfdegmid, numneimid
         write(irefwr,1) 'numnei', numnei
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      if ( IM%symmetric ) then

!     --- jmethod = 5-8:  symmetric data structure

         do i = 1, nfdegmid
            knusol = kfree(1,i)
            isum   = imat1(knusol)
            if ( debug) write(irefwr,1) 'i, knusol, isum', i, knusol, isum
            do j = 1, nfdeg
               if ( kfree(1,j)<knusol ) then
                  isum = isum+1
                  icolsii(isum) = kfree(1,j)
                  if ( debug ) write(irefwr,1) 'j, kfree(1,j)', j, kfree(1,j)
               end if
            end do
            do j = 1, numneimid
               if ( istore(j,1)<knusol ) then
                  isum = isum+1
                  icolsii(isum) = istore(j,1)
                  if ( debug)&
                     write(irefwr,1) 'j, isum, istore(j,1)',&
                                      j, isum, istore(j,1)
              end if
            end do
            istore(knusol,3) = isum
         end do
         do i = nfdegmid+1, nfdeg

            knusol = kfree(1,i)
            isum   = imat1(knusol)
            if ( debug) write(irefwr,1) 'i, knusol, isum', i, knusol, isum
            do j = 1, nfdeg
               if ( kfree(1,j)<knusol ) then
                  isum = isum+1
                  icolsii(isum) = kfree(1,j)
                  if ( debug ) write(irefwr,1) 'j, kfree(1,j)', j, kfree(1,j)
               end if
            end do
            do j = 1, numnei
               if ( istore(j,1)<knusol ) then
                  isum = isum+1
                  icolsii(isum) = istore(j,1)
                  if ( debug )&
                     write(irefwr,1) 'j, isum, icolsii(isum)',&
                                      j, isum, icolsii(isum)
               end if
            end do
            istore(knusol,3) = isum

         end do

      else

!     --- jmethod = 6, 8, 52:  row-compact storage (unsymmetric)

         do i = 1, nfdegmid
            knusol = kfree(1,i)
            isum   = imat1(knusol)
            if ( debug) write(irefwr,1) 'i, knusol, isum', i, knusol, isum
            do j = 1, nfdeg
               if ( kfree(1,j)<knusol ) then
                  isum = isum+1
                  icolsii(isum) = kfree(1,j)
                  if ( debug ) write(irefwr,1) 'j, kfree(1,j)', j, kfree(1,j)
               end if
            end do
            do j = 1, numneimid
               if ( istore(j,1)<knusol ) then
                  isum = isum+1
                  icolsii(isum) = istore(j,1)
                  if ( debug)&
                     write(irefwr,1) 'j, isum, istore(j,1)',&
                                      j, isum, istore(j,1)
              end if
            end do
            istore(knusol,3) = isum
         end do
         do i = nfdegmid+1, nfdeg

            knusol = kfree(1,i)
            isum   = imat1(knusol)
            if ( debug) write(irefwr,1) 'i, knusol, isum', i, knusol, isum
            do j = 1, nfdeg
               if ( kfree(1,j)<knusol ) then
                  isum = isum+1
                  icolsii(isum) = kfree(1,j)
                  if ( debug ) write(irefwr,1) 'j, kfree(1,j)', j, kfree(1,j)
               end if
            end do
            do j = 1, numnei
               if ( istore(j,1)<knusol ) then
                  isum = isum+1
                  icolsii(isum) = istore(j,1)
                  if ( debug )&
                     write(irefwr,1) 'j, isum, icolsii(isum)',&
                                      j, isum, icolsii(isum)
               end if
            end do
            istore(knusol,3) = isum

         end do

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End incm02icol'

      end if  ! ( debug )
      call erclos ( 'incm02icol' )

      end
