      subroutine incommatpar11 ( iwork, irowsiiext )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmodulematstr
!        version  2.0  date 17-12-2015 New parameter list
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array irowsiiext with number of extra elements per row
!     accumulated (only parallel)
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulecpack
      use sepmodulematstr
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(out) :: irowsiiext(0:nrusol)
      integer, intent(inout) :: iwork(nrusol)

!     irowsiiext     o    contains the number of extra elements required in
!                         the matrix S_ii per row (accumulated, first pos. 0)
!     iwork         i/o   integer work array to mark column numbers per row
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ilow, ihigh, istart, ninterf, istart1, k, ncols, &
                 istart2, icol, icount, iposout, j, irow
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     i              Counting variable
!     icol           Column sequence number
!     icount         Counter
!     ihigh          largest index of interface unknown+1
!     ilow           smallest index of interface unknown
!     iposout        position in array amatout
!     irow           Row sequence number
!     istart         First position used at a part
!     istart1        First position used at a part
!     istart2        First position used at a part
!     j              Counting variable
!     k              Counting variable
!     ncols          number of non-zero columns in row
!     ninterf        Number of unknowns in interf (interface)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incommatpar11' )
      debug = .false. .and. ioutp>=0
     if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incommatpar11'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      ilow = 0
      if ( inodenr>1 ) ilow = nrusollocs(inodenr-1)
      ihigh = nrusollocs(inodenr)

      iwork = 0  ! clear work array
      irowsiiext = 0

      if ( debug ) write(irefwr,1) 'ilow, ihigh', ilow, ihigh

!     --- First store number of extra elements in irowsiiext

      do i = 1, iacnodes

         istart = storeinfo(i)  ! start address of neighbor block information

         if ( debug ) write(irefwr,1) 'i, istart', i, istart

         if ( istart>0 ) then

!        --- istart>0, information available

            ninterf = storeinfo(istart)  ! number of interface unknowns
            istart1 = istart+ninterf+1

            if ( debug ) &
               write(irefwr,1) 'ninterf, istart1', ninterf, istart1

            do j = 1, ninterf

               irow = kprobhglobinv(storeinfo(istart+j))  !row number for ii
               istart2 = istart1+storeinfo(istart1+j-1)+ninterf
               ncols = storeinfo(istart1+j)-storeinfo(istart1+j-1)

!              --- Fill iwork with pointers to S_ii

               icount = irowsiiext(irow)
               do k = irowsii(irow)+1, irowsii(irow+1)
                  iwork(icolsii(k)) = k
               end do

               if ( debug ) then
                  write(irefwr,1) 'j, irow, ncols', j, irow, ncols
                  call prinin ( storeinfo(istart2+1), ncols, 'cols' )
               end if  ! ( debug )

               do k = 1, ncols

                  icol = storeinfo(istart2+k)  ! global column number
                  if ( icol>ilow .and. icol<=ihigh ) then

!                 --- Column number in present block, add to S_ii

                     icol = kprobhglobinv(icol)  ! local column number (ii)
                     if ( icol<irow .or. .not. IM%symmetric ) then

!                    --- Lower triangular matrix

                        iposout = iwork(icol)
                        if ( iposout==0 ) then

!                       --- icol is not stored in matrix: address

                           icount = icount+1
                           irowsiiext(irow) = icount

                        end if  ! ( iposut==0 )

                     end if  ! ( irow==icol )

                  end if  ! ( icol<=ilow .or. icol>ihigh )

               end do  ! k = 1, ncols

!              --- Clear iwork

               do k = irowsii(irow)+1, irowsii(irow+1)
                  iwork(icolsii(k)) = 0
               end do

            end do  ! j = 1, ninterf

         end if  ! ( istart>0 )

      end do  ! i = 1, iacnodes

      if ( debug ) call prinin ( irowsiiext(1), nrusol, 'irowsiiext' )

!     --- Accumulate irowsiiext

      do i = 1, nrusol
         irowsiiext(i) = irowsiiext(i)+irowsiiext(i-1)
      end do  ! i = 1, nrusol

1000  if ( debug ) then

!     --- Debug information

         call prinin ( irowsiiext(1), nrusol, 'irowsiiext' )
         write(irefwr,*) 'End incommatpar11'

      end if  ! ( debug )
      call erclos ( 'incommatpar11' )

      end subroutine incommatpar11
