      subroutine incommatpar12 ( iwork, irowsiiold, icolsiiold, irowsiiext )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmodulematstr
!        version  2.0  date 17-12-2015 New parameter list
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill new arrays irowsii and icolsii in case extar elements
!     are required in the parallel case
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulecpack
      use sepmodulematstr
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: irowsiiold(*), &
                             icolsiiold(*), irowsiiext(0:nrusol)
      integer, intent(inout) :: iwork(nrusol)

!     icolsiiold     i    starting address of icolsiiold
!     irowsiiext     i    contains the number of extra elements required in
!                         the matrix S_ii per row (accumulated, first pos. 0)
!     irowsiiold     i    old version of irowsii
!     iwork         i/o   integer work array to mark column numbers per row
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ilow, ihigh, istart, ninterf, istart1, k, ncols, &
                 istart2, icol, icount, iposout, j, irow, ipos, iposol, &
                 ihelp(nrusol)
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     i              Counting variable
!     icol           Column sequence number
!     icount         Counter
!     ihelp          Work of size nrusol
!                    In iwork (irow) the number of extra column numbers
!                    that have been stored in icolsii is stored
!     ihigh          Highest position number in USOL with respect to
!                    "right" periodical boundary conditions
!                    ihigh = nusol-nbound
!     ilow           elements are numbered from element number ilow
!     ipos           Position in array icolsii
!     iposol         Position in array icolsiiold
!     iposout        position in array amatout
!     irow           Row sequence number
!     istart         First position used at a part
!     istart1        First position used at a part
!     istart2        First position used at a part
!     j              Counting variable
!     k              Counting variable
!     ncols          number of non-zero columns in row
!     ninterf        Number of unknowns in interf (interface)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incommatpar12' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incommatpar12'
         call prinin ( irowsiiold, nrusol+1, 'irowsiiold' )
         call prinin ( icolsiiold, irowsiiold(nrusol), 'icolsiiold' )
         call prinin ( irowsiiext(1), nrusol, 'irowsiiext' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      ilow = 0
      if ( inodenr>1 ) ilow = nrusollocs(inodenr-1)
      ihigh = nrusollocs(inodenr)

      iwork = 0  ! clear work array
      ihelp = 0

      if ( debug ) write(irefwr,1) 'ilow, ihigh', ilow, ihigh

!     --- First copy irowsiiold and icolsiiold to new positions

      irowsii(1) = 0
      ipos = 0
      iposol = 0
      do i = 1, nrusol

         ncols = irowsiiold(i+1)-iposol
         icolsii(ipos+1:ipos+ncols) = icolsiiold(iposol+1:iposol+ncols)
         iposol = irowsiiold(i+1)
         ipos = ipos+ncols+irowsiiext(i)-irowsiiext(i-1)
         irowsii(i+1) = ipos

      end do  ! i = 1, nrusol

!     --- Now store extra elements

      do i = 1, iacnodes

         istart = storeinfo(i)  ! start address of neighbor block information

         if ( debug ) write(irefwr,1) 'i, istart', i, istart

         if ( istart>0 ) then

!        --- istart>0, information available

            ninterf = storeinfo(istart)  ! number of interface unknowns
            istart1 = istart+ninterf+1

            if ( debug ) &
               write(irefwr,1) 'ninterf, istart1', ninterf, istart1

            do j = 1, ninterf

               irow = kprobhglobinv(storeinfo(istart+j))  !row number for ii
               istart2 = istart1+storeinfo(istart1+j-1)+ninterf
               ncols = storeinfo(istart1+j)-storeinfo(istart1+j-1)

!              --- Fill iwork with pointers to S_ii

               icount = 0
               do k = irowsiiold(irow)+1, irowsiiold(irow+1)
                  iwork(icolsiiold(k)) = k
               end do

               icount = irowsiiold(irow+1)-irowsiiold(irow)
               ipos = irowsii(irow)+icount
               icount = ihelp(irow)

               if ( debug ) then
                  write(irefwr,1) 'j, irow, ncols, ipos', &
                                   j, irow, ncols, ipos
                  call prinin ( storeinfo(istart2+1), ncols, 'cols' )
               end if  ! ( debug )

               do k = 1, ncols

                  icol = storeinfo(istart2+k)  ! global column number

                  if ( icol>ilow .and. icol<=ihigh ) then

!                 --- Column number in present block, add to S_ii

                     icol = kprobhglobinv(icol)  ! local column number (ii)
                     if ( icol<irow .or. .not. IM%symmetric ) then

!                    --- Lower triangular matrix

                        iposout = iwork(icol)

                        if ( iposout==0 ) then

!                       --- icol is not stored in matrix: address

                           icount = icount+1
                           icolsii(ipos+icount) = icol

                        end if  ! ( iposut==0 )

                     end if  ! ( irow==icol )

                  end if  ! ( icol<=ilow .or. icol>ihigh )

               end do  ! k = 1, ncols

!              --- Clear iwork

               do k = irowsiiold(irow)+1, irowsiiold(irow+1)
                  iwork(icolsiiold(k)) = 0
               end do
               ihelp(irow) = icount

            end do  ! j = 1, ninterf

         end if  ! ( istart>0 )

      end do  ! i = 1, iacnodes

1000  if ( debug ) then

!     --- Debug information

         call prinin ( irowsii, nrusol+1, 'irowsii' )
         call prinin ( icolsii, irowsii(nrusol), 'icolsii' )
         write(irefwr,*) 'End incommatpar12'

      end if  ! ( debug )
      call erclos ( 'incommatpar12' )

      end subroutine incommatpar12
