      subroutine incommatpar3 ( leniwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmodulematstr
!        version  2.0  date 17-12-2015 New parameter list
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy information of istore into array storeinfo
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulecpack
      use sepmodulematstr
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(inout) :: leniwork

!     leniwork      i/o   Computed length of array iwork
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iblock, istart, istartstore, ninterf, lenif

!     iblock         Block sequence number
!     istart         First position used at a part
!     istartstore    Start position in array storeinfo
!     lenif          Length of array icolsif
!     ninterf        Number of unknowns at interface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'incommatpar3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from incommatpar3'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      storeinfo = 0

      istartstore = iacnodes+1
      do iblock = 1, iacnodes

         istart = istore(iblock)

         if ( debug ) write(irefwr,1) 'iblock, istart', iblock, istart

         if ( istart>0 ) then

!        --- block is coupled to this node

            ninterf = istore(istart)

            if ( debug ) then
               write(irefwr,1) 'ninterf, istartstore', &
                                ninterf, istartstore
               call prinin ( istore(istart+1), ninterf, 'istore' )
            end if  ! ( debug )

            if ( ninterf>0 ) then

!           --- Copy global unknown numbers of rows

               storeinfo(iblock) = istartstore
               storeinfo(istartstore) = ninterf
               storeinfo(istartstore+1:istartstore+ninterf) = &
                  istore(istart+1:istart+ninterf)
               istart = istart+ninterf
               istartstore = istartstore+ninterf

!              --- Copy irowsif

               storeinfo(istartstore+1:istartstore+ninterf+1) = &
                  istore(istart+1:istart+ninterf+1)
               lenif = istore(istart+ninterf+1)

!              --- Raise istartstore

               istartstore = istartstore+ninterf+2+lenif

               leniwork = max(leniwork, lenif)

            end if  ! ( ninterf>0 )

         end if  ! ( istart>0 )

      end do  ! iblock = 1, iacnodes

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'istartstore', istartstore
         call prinin ( storeinfo, istartstore, 'storeinfo' )
         write(irefwr,*) 'End incommatpar3'

      end if  ! ( debug )
      call erclos ( 'incommatpar3' )

      end subroutine incommatpar3
