      subroutine ini011reac ( intm1b )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.0  date 06-01-2016 Use sepmodulematstr
!        version  2.0  date 17-12-2015 New parameter list
!        version  1.0  date 14-04-2015
!
!   copyright (c) 2015-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute integer information of matrix corresponding to
!     essential boundary conditions
!     It concerns the part with respect to reaction forces
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulematstr
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer intm1b(*)

!     intm1b         o    1b part of integer information ( pointers )
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inusol, nhelp1, nhelp2, knusol, nfnbr, nbr, iknbr, isum, &
              icount, nodp, ibr, j, l, inunkp
      integer, allocatable :: iwork(:)

!     ibr            Loop variable for neighbours
!     icount         Counter for prescribed degrees of freedom, defines next one
!     iknbr          Nodal point number of neighbour
!     inunkp         Number of degrees of freedom in point
!     inusol         Position of unknown in array USOL
!     isum           Integer variable to compute a sum
!     iwork          work array
!     j              Counting variable
!     knusol         Position of unknown in array USOL
!     l              General loop variable
!     nbr            number of neighbours of nodal point
!     nfnbr          help parameter to compute nbr
!     nhelp1         number of prescribed degrees of freedom in node
!     nhelp2         number of free degrees of freedom coupled to
!                    the prescribed degrees of freedom in node
!     nodp           Nodal point number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INI011FICT     Fill part of imat1/2 corresponding to fictitious unknown
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ini011reac' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from ini011reac'
         call prinin ( kprobr(311), 10, 'kprobr' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icount = 0

      allocate ( iwork(nusol), stat = error )
      if ( error/=0 ) call eralloc ( error, nusol, 'iwork' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) &
         write(irefwr,1) 'nusol, nrusolloc', nusol, nrusolloc

      do nodp = 1, npoint

         inusol = sepgetinusol ( nodp, inunkp )
         nhelp1 = 0
         nhelp2 = 0
         do j = 1, inunkp
            knusol = kprobr(inusol+j)
            if ( knusol>nrusolloc ) then

!           --- knusol>nrusolloc

               nhelp1 = nhelp1+1
               iwork(nhelp1) = knusol

            end if
         end do

         if ( debug ) &
            write(irefwr,1) 'nodp, inunkp, nhelp1, nhelp2', &
                             nodp, inunkp, nhelp1, nhelp2

         if ( nhelp1/=0 ) then

!        --- nhelp1>0, so there is at least one prescribed unknown
!            in nodp

            nhelp2 = nhelp1

            nfnbr = kmshe1(nodp)
            nbr = kmshe1(nodp+1)-nfnbr
            do ibr = 1, nbr
               iknbr = kmshe2(nfnbr+ibr)
               inusol = sepgetinusol ( iknbr, inunkp )
               do j = 1,inunkp
                  knusol = kprobr(inusol+j)
                  if(knusol>nrusolloc) then

!                 --- knusol>nrusolloc

                    nhelp2 = nhelp2+1
                    iwork(nhelp2) = knusol

                  end if
               end do
            end do

            if ( debug ) &
               write(irefwr,1) 'nhelp2', nhelp2

!           --- put information in array intmat part 2

            do l = 1, nhelp1
               icount = icount+1
               isum = intm1b(icount)
               do j = 1, nhelp2
                  isum = isum+1
                  icolsp(isum) = iwork(j)
               end do
               intm1b(icount) = intm1b(icount)+nhelp2
            end do
         end if
      end do

      if ( indpry>0 ) then

!     --- indpry>0, fictitious unknowns

         call ini011fict ( intm1b, icolsp, kprobfict )

      end if

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( intm1b, icount, 'intm1a' )
         call prinin ( icolsp, isum, 'intm2a' )
         write(irefwr,*) 'End ini011reac'

      end if  ! ( debug )
      call erclos ( 'ini011reac' )

      end subroutine ini011reac
