      subroutine init07crv ( iinp, ielshape, numcrv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 15-03-2012 Extra parameter ivers
!        version  1.0  date 05-11-2007
!
!   copyright (c) 2007-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill number of nodes per element and element shapes for
!     boundary groups of type curve in array ielshape
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_element
!     element_shape
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp, ielshape(2,*), numcrv

!     ielshape      i/o   Array of length 2*(nelgrp+numnat) containing:
!                         (1,i) number of nodes in element group i
!                         (2,i) shape number of elements in element group i
!                         if i>nelgrp: i-nelgrp = boundary group number
!                         otherwise it is the element group number
!     iinp          i/o   last position used in array iinputkp
!     numcrv         i    number of parts where curves are given for generation
!                         of boundary elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, ibngrp, inpelm, i
      logical spectral, linear

!     i              Counting variable
!     ibngrp         Actual boundary element group number
!     inpelm         number of points per element
!     j              Counting variable
!     linear         Indicates whether the element is a linear simplex
!                    (true) or not (false)
!     spectral       If true spectral elements are used
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     128   code for generation of elements (ishape) is incorrect
!     271   boundary element group number has wrong value
!    1083   Shape number for boundary group different from before
!    1694   More than one spectral group not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'init07crv' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from init07crv'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- numcrv>0  curve boundary elements

      spectral = .false.
      if ( kelmu>0 ) then

!     --- Spectral elements are used

         spectral = .true.
         if ( kmeshu(1)/=1 ) then

!        --- Not yet prepared for various types of spectral elements

            call errsub ( 1694, 0, 0, 0 )
            go to 1000

         end if
         ishape = 1+kmeshu(2)

      else

!     --- detect if all volume elements are quadratic or not

         linear = .false.
         do i = 1, nelgrp
            ishape = kmeshf(i)
            if ( ishape==3 .or. ishape==5 .or. ishape==9 .or. &
                 ishape==10 ) linear = .true.
         end do

      end if

      do j = 1, numcrv
         ibngrp = iinputkp(iinp+1)
         if ( ibngrp<=0 .or. ibngrp>numnat ) then
            call errint ( ibngrp, 1 )
            call errint ( numnat, 2 )
            call errint ( j, 3 )
            call errchr ( 'curve', 1 )
            call errsub ( 271, 3, 0, 1 )
         end if
         if ( debug ) write(irefwr,*) 'j, ibngrp, iinp, spectral ', &
            j, ibngrp, iinp, spectral
         if ( .not.spectral ) then

!        --- Standard case

            ishape = iinputkp(iinp+2)
            if ( ishape==0 ) then
               if ( linear ) then
                  ishape = 1
               else
                  ishape = 2
               end if
               iinputkp(iinp+2) = ishape
            end if
            if ( ishape<1 .or. ishape>2 ) then
               call errint ( ishape, 1 )
               call errsub ( 128, 1, 0, 0 )
            end if

         end if
         inpelm = ishape+1
         if ( spectral ) ishape = 1
         if ( ielshape(2,ibngrp+nelgrp)/=0 ) then

!        --- check if shapes coincide

            if ( ielshape(2,ibngrp+nelgrp)/=ishape ) then
               call errint ( ibngrp, 1 )
               call errint ( ishape, 2 )
               call errint ( ielshape(2,ibngrp+nelgrp), 3 )
               call errint ( j, 4 )
               call errchr ( 'curve', 1 )
               call errsub ( 1083, 4, 0, 1 )
            end if

         end if
         ielshape(1,ibngrp+nelgrp) = inpelm
         ielshape(2,ibngrp+nelgrp) = ishape
         iinp = iinp+3+iinputkp(iinp+3)

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End init07crv'

      end if  ! ( debug )
      call erclos ( 'init07crv' )

      end
