      subroutine initsp ( ielshape, istore, type922found )
! ======================================================================
!
!        programmer    Guus Segal
!        version  9.0  date 06-01-2016 Use sepmodulecomio
!        version  8.1  date 03-04-2013 extra positions in case of parallel
!        version  8.0  date 27-03-2013 Extra parameter kprobloc
!
!   copyright (c) 1984-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     subroutine initsp is used to process general information from
!     array iinputkp
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulecpack
      use sepmoduletexts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ielshape(2,*), istore(*)
      logical type922found

!     ielshape       i    Array of length 2*(nelgrp+numnat) containing:
!                         (1,i) number of nodes in element group i
!                         (2,i) shape number of elements in element group i
!                         if i>nelgrp: i-nelgrp = boundary group number
!                         otherwise it is the element group number
!     istore         i    work array
!     type922found   i    If true elements of type 922 have been found, which
!                         means that Lagrangian multipliers are defined by means
!                         of boundary elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, ibuf
      double precision dummy(1)

!     dummy          Dummy parameter
!     ibuf           Next position in istore to be used
!     istart         Next position in array iinputkp
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIT03         the input in array iinputkp is processed and stored
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   -
! **********************************************************************
!
!                       PSEUDO CODE
!
!      array istore is filled as follows:
!      For each element group   (nelgrp standard elements):
!       pos. 1-ntype   ntype type numbers
!       next           inunkp, indtvc, iphys
!       next           if inunkp<=0: inpelm pos. with degrees of freedom
!       next           indtvc*inpelm pos. with degrees of freedom for arrays
!                      of special structure
!       next           iphys*inpelm pos. with information of physical degrees
!                      of freedom
!
!
!      Make some initializations
!      if ( kprob(3)#101 ) then
!         Clear first part of kprob
!      read information of types
!      put temporary information of arrays istore and istfl1 at the end of
!      ibuffr
!      reserve space for kprob part p when nphys>0
!      Put information in kprob and inialize a part of kprob p
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'initsp' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from initsp'
         call prinin ( iinputkp, 50, 'iinputkp' )
  1      format ( a, 1x, (10i10) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Make some initializations

      nbcons = iinputkp(13)
      numnat = iinputkp(2)+nbcons
      if ( debug ) &
         write(irefwr,1) 'nelgrp, nbcons, numnat, lenkprobfp', &
                          nelgrp, nbcons, numnat, lenkprobfp

      call prinlg ( 'kprobloc ', lenkprobfp, dummy, kprobloc )
      kprobloc(3) = lenkprobfp

!     --- read information of types
!         put temporary information in array istore

      call init03 ( istore, istart, ibuf, ielshape, type922found )
      if ( type922found ) kprobloc(54) = 1

      if ( parallel ) then

!     --- Parallel case, 4 positions are used, starting address kprob(65)

         if ( .not. associated(kp(iseqkp)%kprobsub(iprob)%kprobpar) ) &
            allocate ( kp(iseqkp)%kprobsub(iprob)%kprobpar(4) )
         kprobpar => kp(iseqkp)%kprobsub(iprob)%kprobpar
         kprobpar = 0
         kprobloc(65) = 4

      end if  ! ( parallel )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'kelgrp, type922found ', &
                          kelgrp, type922found
         call prinin ( intarprb, 20, 'intarprb' )
         call prinin1 ( ielshape, nelgrp+numnat, 2, 'ielshape' )

      end if  ! ( debug )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kprobloc, kprobloc(3), 'kprobloc' )
         write(irefwr,*) 'End initsp'

      end if
      call erclos ( 'initsp' )

      end
