      subroutine printprobinp01 ( iinput, iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 13-10-2013 Extension with leniinpfp
!        version  1.1  date 18-06-2012 Extra input description
!        version  1.0  date 05-04-2012
!
!   copyright (c) 2012-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the contents of the first part of array iinput with respect
!     to problem definition
!     Raise iinp
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     problem_definition
!     input
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp

!     iinp          i/o   next position used in array iinput
!     iinput         i    integer input array to be printed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer leniinpfp

!     leniinpfp      Length of first part of iinput for probdf
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINTPOSINT    Print value of positive integer if>0
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printprobinp01' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printprobinp01'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call printposint ( iinput(1), 'ntype' )
      call printposint ( iinput(2), 'numnatbnd' )
      call printposint ( iinput(3), 'numnatbntp' )
      call printposint ( iinput(4), 'numnatbcrv' )
      call printposint ( iinput(5), 'numnatbsrf' )
      call printposint ( iinput(6), 'numesspnt' )
      call printposint ( iinput(7), 'numesscrv' )
      call printposint ( iinput(8), 'numesssrf' )
      call printposint ( iinput(9), 'jlocaltransform' )
      call printposint ( iinput(10), 'numessnodes' )
      call printposint ( iinput(11), 'numesselements' )
      call printposint ( iinput(12), 'numessgroups' )
      call printposint ( iinput(13), 'numconsgroups' )
      call printposint ( iinput(14), 'maxnumrenum' )
      call printposint ( iinput(15), 'ivers' )
      call printposint ( iinput(16), 'numessobstacles' )
      call printposint ( iinput(17), 'numnatobstacles' )
      call printposint ( iinput(18), 'numlocobstacles' )
      call printposint ( iinput(19), 'nescontact' )
      call printposint ( iinput(20), 'numnatcontactelements' )
      call printposint ( iinput(21), 'numloccontacts' )
      call printposint ( iinput(22), 'nesnocontact' )
      call printposint ( iinput(23), 'numnatcontactpoints' )
      call printposint ( iinput(24), 'numnatnocontactelements' )
      call printposint ( iinput(25), 'numnatnocontactpoints' )
      call printposint ( iinput(26), 'numnatcurponts' )
      call printposint ( iinput(27), 'numnatsurponts' )
      call printposint ( iinput(28), 'nesnodal' )
      call printposint ( iinput(29), 'nglgrp' )
      call printposint ( iinput(30), 'numglcrv' )
      call printposint ( iinput(31), 'numglsrf' )
      call printposint ( iinput(32), 'numglvlm' )
      call printposint ( iinput(33), 'numglall' )
      call printposint ( iinput(34), 'numnatcrossobst' )
      call printposint ( iinput(35), 'print_level' )
      call printposint ( iinput(36), 'nfictgrp' )
      call printposint ( iinput(37), 'numfictcrv' )
      call printposint ( iinput(38), 'numfictsrf' )
      call printposint ( iinput(39), 'numessvolm' )
      call printposint ( iinput(40), 'numfictobst' )
      call printposint ( iinput(41), 'numessinnobst' )
      call printposint ( iinput(42), 'numessbounnobst' )
      call printposint ( iinput(43), 'numessonbounnobst' )
      call printposint ( iinput(44), 'iskipelem' )
      call printposint ( iinput(45), 'nesallobst' )
      call printposint ( iinput(46), 'nesinonobst' )
      call printposint ( iinput(47), 'neszerolev' )
      call printposint ( iinput(48), 'numlevelset' )
      call printposint ( iinput(49), 'nescav' )
      call printposint ( iinput(50), 'natcomcrv' )
      call printposint ( iinput(51), 'natcomsrf' )
      call printposint ( iinput(52), 'numperbounconds' )
      call printposint ( iinput(53), '' )
      call printposint ( iinput(54), '' )
      call printposint ( iinput(55), 'natlevset' )
      call printposint ( iinput(56), 'nesintersect' )
      call printposint ( iinput(59), 'newstyle' )
      leniinpfp = 60
      iinp = iinp+leniinpfp

1000  call erclos ( 'printprobinp01' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printprobinp01'

      end if  ! ( debug )

      end
