      subroutine printprobinp03 ( iinput, iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 18-06-2012 Extra input description
!        version  1.0  date 05-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the contents of the natural boundary coditions part of array
!     iinput with respect to problem definition
!     Raise iinp
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     problem_definition
!     input
!     natural_boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp

!     iinp          i/o   next position used in array iinput
!     iinput         i    integer input array to be printed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numnatbnd, ibngrp, ntype, typemax, inpelm, i, jinp

!     i              Counting variable
!     ibngrp         Actual boundary element group number
!     inpelm         number of points per element
!     jinp           last position used in array iinput with respect to
!                    iprec
!     ntype          Number of different type numbers to be used for one
!                    group
!     numnatbnd      Number of natural boundary conditions
!     typemax        Maximum type number found
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINTPOSINT    Print value of positive integer if>0
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printprobinp03' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printprobinp03'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ntype = iinput(1)
      numnatbnd = iinput(2)

      if ( numnatbnd==0 ) go to 1000

      write(irefwr,*) 'natural boundary conditions'
      do ibngrp = 1, numnatbnd

         call prinin ( iinput(iinp), ntype, 'type numbers' )
         typemax = maxval(iinput(iinp:iinp+ntype-1))
         iinp = iinp+ntype

         if ( typemax>0 .and. typemax<100 ) then

!        --- All type numbers between 1 and 99

            jinp = iinp
            call printposint ( iinput(iinp), 'inpelm' )
            inpelm = iinput(iinp)
            call printposint ( iinput(iinp+1), 'inunkp' )
            call printposint ( iinput(iinp+2), 'indtvc' )
            call printposint ( iinput(iinp+3), 'isear' )
            iinp = iinp+4

            if ( iinput(jinp+1)==0 ) then
               call prinin ( iinput(iinp), inpelm, 'ndegfd per point' )
               iinp = iinp+inpelm
            end if  ! ( iinput(jinp)==0 )
            do i = 1, iinput(jinp+2)
               write(irefwr,1) 'array of special structure', i
               call prinin ( iinput(iinp), inpelm, 'ndegfd per point' )
               iinp = iinp+inpelm
            end do  ! i = 1, iinput(jinp+1)
            do i = 1, iinput(jinp+3)
               write(irefwr,1) 'isear', i
               call prinin ( iinput(iinp), inpelm, 'nphys per point' )
               iinp = iinp+inpelm
            end do  ! i = 1, iinput(jinp+1)

         end if  ! ( typemax>0 .and. typemax<100 )

      end do  ! ibngrp = 1, numnatbnd

!     --- Next information of boundary elements
!         Point boundary elements

      do i = 1, iinput(3)
         call printposint ( iinput(iinp), 'ibngrp' )
         call prinin ( iinput(iinp+2), iinput(iinp+1), 'user points' )
         iinp = iinp+iinput(iinp+1)+2
      end do  ! i = 1, iinput(3)

!     --- Curve boundary elements

      do i = 1, iinput(4)
         call printposint ( iinput(iinp), 'ibngrp' )
         call printposint ( iinput(iinp+1), 'ishape' )
         call prinin ( iinput(iinp+3), iinput(iinp+2), 'curves' )
         iinp = iinp+iinput(iinp+2)+3
      end do  ! i = 1, iinput(4)

!     --- Surface boundary elements

      do i = 1, iinput(5)
         call printposint ( iinput(iinp), 'ibngrp' )
         call prinin ( iinput(iinp+2), iinput(iinp+1), 'surfaces' )
         iinp = iinp+iinput(iinp+1)+2
      end do  ! i = 1, iinput(5)

!     --- Obstacle boundary elements

      do i = 1, iinput(17)
         call printposint ( iinput(iinp), 'ibngrp' )
         call printposint ( iinput(iinp+1), 'ishape' )
         call printposint ( iinput(iinp+2), 'iobst' )
         iinp = iinp+3
      end do  ! i = 1, iinput(17)

!     --- Contact boundary elements

      do i = 1, iinput(20)
         call printposint ( iinput(iinp), 'ibngrp' )
         call printposint ( iinput(iinp+1), 'ishape' )
         call printposint ( iinput(iinp+2), 'icontact' )
         iinp = iinp+3
      end do  ! i = 1, iinput(20)

!     --- No Contact boundary elements

      do i = 1, iinput(24)
         call printposint ( iinput(iinp), 'ibngrp' )
         call printposint ( iinput(iinp+1), 'ishape' )
         call printposint ( iinput(iinp+2), 'inocontact' )
         iinp = iinp+3
      end do  ! i = 1, iinput(24)

!     --- Contact Point boundary elements

      do i = 1, iinput(24)
         call printposint ( iinput(iinp), 'ibngrp' )
         call printposint ( iinput(iinp+2), 'icontact' )
         iinp = iinp+3
      end do  ! i = 1, iinput(24)

!     --- No Contact Point boundary elements

      do i = 1, iinput(25)
         call printposint ( iinput(iinp), 'ibngrp' )
         call printposint ( iinput(iinp+2), 'icontact' )
         iinp = iinp+3
      end do  ! i = 1, iinput(25)

!     --- Curve points boundary elements

      do i = 1, iinput(26)
         call printposint ( iinput(iinp), 'ibngrp' )
         call prinin ( iinput(iinp+2), iinput(iinp+1), 'curves' )
         iinp = iinp+iinput(iinp+1)+2
      end do  ! i = 1, iinput(26)

!     --- Surface points boundary elements

      do i = 1, iinput(27)
         call printposint ( iinput(iinp), 'ibngrp' )
         call prinin ( iinput(iinp+2), iinput(iinp+1), 'surfaces' )
         iinp = iinp+iinput(iinp+1)+2
      end do  ! i = 1, iinput(27)

!     --- Cross obstacle boundary elements

      do i = 1, iinput(34)
         call printposint ( iinput(iinp), 'ibngrp' )
         call printposint ( iinput(iinp+1), 'iobst' )
         call printposint ( iinput(iinp+2), 'iexclude' )
         call printposint ( iinput(iinp+3), 'exclude curves' )
         call prinin ( iinput(iinp+5), iinput(iinp+4), 'curves' )
         iinp = iinp+5+iinput(iinp+4)
      end do  ! i = 1, iinput(34)

!     --- complete curve boundary elements

      do i = 1, iinput(50)
         call printposint ( iinput(iinp), 'ibngrp' )
         call prinin ( iinput(iinp+2), iinput(iinp+1), 'curves' )
         iinp = iinp+2+iinput(iinp+1)
      end do  ! i = 1, iinput(50)

!     --- complete surface boundary elements

      do i = 1, iinput(51)
         call printposint ( iinput(iinp), 'ibngrp' )
         call prinin ( iinput(iinp+2), iinput(iinp+1), 'surfaces' )
         iinp = iinp+2+iinput(iinp+1)
      end do  ! i = 1, iinput(51)

!     --- zero level set boundary elements

      do i = 1, iinput(55)
         call printposint ( iinput(iinp), 'ibngrp' )
         call printposint ( iinput(iinp+1), 'itype' )
         call printposint ( iinput(iinp+2), 'seq number level set' )
         call printposint ( iinput(iinp+3), 'iexclude' )
         call printposint ( iinput(iinp+4), 'type of exclusion' )
         call prinin ( iinput(iinp+6), iinput(iinp+5), 'curves' )
         iinp = iinp+6+iinput(iinp+5)
      end do  ! i = 1, iinput(55)

1000  call erclos ( 'printprobinp03' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printprobinp03'

      end if  ! ( debug )

      end
