      subroutine printprobinp06 ( iinput, iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 13-10-2013 Extension with intersection
!        version  1.1  date 03-07-2012 Correction cavity
!        version  1.0  date 05-04-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Print the contents of the essential boundary coditions part of array
!     iinput with respect to problem definition
!     Raise iinp
! **********************************************************************
!
!                       KEYWORDS
!
!     print
!     problem_definition
!     input
!     essential_boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp

!     iinp          i/o   next position used in array iinput
!     iinput         i    integer input array to be printed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numbpn, i, numcrv, inpelm, numsrf, iskipbn, numvlm, &
              numnod, numelm, numgrp, ielgrp, nesallobst, nesinnobst, &
              nesbnobst, nesonbnobst, nesinonbnobst, nesconct, &
              nesnoconct, nesnodal, numnodal, neszerolev, numobstlc, &
              iinpbef, nescav, nesintersect

!     i              Counting variable
!     ielgrp         element group number
!     iinpbef        ?
!     inpelm         number of points per element
!     iskipbn        Defines if the outer boundary must be skipped
!                    Possible values:
!                    0:  the outer boundary must not be skipped
!                    1:  the complete outer boundary must be skipped
!                    2:  the inner boundary must be skipped as well as some
!                        explicitly given outer curves
!                    3:  the outer boundary must be skipped except some
!                        explicitly given outer curves
!     nesallobst     Number of obstacles where all nodes are prescribed
!     nesbnobst      Number of obstacles where all but the inner nodes are
!                    prescribed
!     nescav         Number of parts where cavity bondary conditions are given
!     nesconct       Number of contacts at which essential boundary
!                    conditions are given
!     nesinnobst     Number of obstacles where all inner nodes are prescribed
!     nesinonbnobst  Number of obstacles where all he nodes on the
!                    obstacle which are only part of elements completely
!                    inside the obstacle are prescribed
!     nesintersect   Number of intersections where essential boundary
!                    conditions are given
!     nesnoconct     Number of no contacts at which essential boundary
!                    conditions are given
!     nesnodal       Number of groups of nodal points with essential
!                    boundary conditions
!     nesonbnobst    Number of obstacles where all the nodes on the boundary of
!                    the obstacle are prescribed
!     neszerolev     Number of parts where essential boundary conditions of
!                    the type zero_levelset i are given
!     numbpn         Number of parts where point boundary elements are
!                    created
!     numcrv         number of parts where curves are given for generation
!                    of boundary elements
!     numelm         Number of elements
!     numgrp         Number of groups
!     numnod         Number of points
!     numnodal       Number of nodal points
!     numobstlc      number of obstacles
!     numsrf         number of parts where surfaces are given for
!                    generation of boundary elements
!     numvlm         number of parts where volumes are given for
!                    generation of boundary conditions
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getnpelmelgrp

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     GETNPELMELGRP  Fetch number of points per element in element group ielgrp
!     PRININ         print 1d integer array
!     PRINTPOSINT    Print value of positive integer if>0
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'printprobinp06' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from printprobinp06'
         write(irefwr,1) 'iinp', iinp
         call prinin ( iinput(iinp), 4, 'iinput' )
         iinpbef = iinp
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      write(irefwr,*) 'essential boundary conditions'

      numbpn = iinput(6)

      do i = 1, numbpn
         write(irefwr,1) 'bitpattern', iinput(iinp)
         call prinin ( iinput(iinp+2), iinput(iinp+1), 'user points' )
         iinp = iinp+2+iinput(iinp+1)
         if ( debug ) write(irefwr,1) 'iinp_loc', iinp-iinpbef+1
      end do  ! i = 1, numbpn

      numcrv = iinput(7)

      do i = 1, numcrv
         write(irefwr,1) 'bitpattern', iinput(iinp)
         call printposint ( iinput(iinp+1), 'jstep' )
         call prinin ( iinput(iinp+3), iinput(iinp+2), 'curves' )
         iinp = iinp+3+iinput(iinp+2)
         if ( debug ) write(irefwr,1) 'iinp_loc', iinp-iinpbef+1
      end do  ! i = 1, numcrv

      if ( debug ) write(irefwr,1) 'iinp/1a', iinp

      numsrf = iinput(8)
      if ( numsrf>0 ) write(irefwr,*) 'surfaces'

      do i = 1, numsrf
         write(irefwr,1) 'i', i
         write(irefwr,1) 'bitpattern', iinput(iinp)
         if ( iinput(iinp+1)==-1 ) then

!        --- Outer surfaces

            write(irefwr,*) 'Outer surfaces'
            iinp = iinp+3

         else

!        --- Standard case

            inpelm = iinput(iinp+1)
            call printposint ( inpelm, 'inpelm' )
            call prinin ( iinput(iinp+3), iinput(iinp+2), 'surfaces' )
            iinp = iinp+3+iinput(iinp+2)
            call prinin ( iinput(iinp), inpelm, 'use local nodes' )
            iinp = iinp+inpelm
            iskipbn = iinput(iinp)
            numcrv = iinput(iinp+1)
            call printposint ( iskipbn, 'iskipbn' )
            call printposint ( numcrv, 'ncurves' )
            iinp = iinp+2
            if ( iskipbn>=1 ) then
               call prinin ( iinput(iinp), numcrv, 'curves' )
               iinp = iinp+numcrv
            end if  ! ( iskipbn==1 )

         end if  ! ( iinput(iinp+1)==-1 )
         if ( debug ) write(irefwr,1) 'iinp_loc', iinp-iinpbef+1

      end do  ! i = 1, numsrf

      numvlm = iinput(39)
      if ( numvlm>0 ) write(irefwr,*) 'volumes'

      do i = 1, numvlm

         write(irefwr,1) 'bitpattern', iinput(iinp)
         inpelm = iinput(iinp+1)
         call printposint ( inpelm, 'inpelm' )
         call prinin ( iinput(iinp+3), iinput(iinp+2), 'volumes' )
         iinp = iinp+3+iinput(iinp+2)
         call prinin ( iinput(iinp), inpelm, 'use local nodes' )
         iinp = iinp+inpelm
         numsrf = iinput(iinp+1)
         call printposint ( iinput(iinp), 'iskipbn' )
         call printposint ( numsrf, 'nsurfaces' )
         iinp = iinp+2
         if ( numsrf>0 ) then
            call prinin ( iinput(iinp), numsrf, 'skip surfaces' )
            iinp = iinp+numsrf
         end if  ! ( numsrf>0 )
         if ( debug ) write(irefwr,1) 'iinp_loc', iinp-iinpbef+1

      end do  ! i = 1, numvlm

      numnod = iinput(10)
      if (numnod >0 ) write(irefwr,*) 'nodes'

      do i = 1, numnod

         write(irefwr,1) 'bitpattern', iinput(iinp)
         call printposint ( iinput(iinp+1), 'ifirst' )
         call printposint ( iinput(iinp+2), 'ilast' )
         iinp = iinp+2
         if ( debug ) write(irefwr,1) 'iinp_loc', iinp-iinpbef+1

      end do  ! i = 1, numnod

      numelm = iinput(11)
      if ( numelm>0 ) write(irefwr,*) 'elements'

      do i = 1, numelm

         write(irefwr,1) 'bitpattern', iinput(iinp)
         call printposint ( iinput(iinp+1), 'ifirst' )
         call printposint ( iinput(iinp+2), 'ilast' )
         call printposint ( iinput(iinp+3), 'rn1' )
         iinp = iinp+3
         if ( debug ) write(irefwr,1) 'iinp_loc', iinp-iinpbef+1

      end do  ! i = 1, numelm

      numgrp = iinput(12)
      if ( numgrp>0 ) write(irefwr,*) 'element goups'

      do i = 1, numgrp

         write(irefwr,1) 'bitpattern', iinput(iinp)
         ielgrp = iinput(iinp+1)
         call printposint ( ielgrp, 'ielgrp' )
         inpelm = Topgrp(ielgrp)%inpelm
         call prinin ( iinput(iinp+2), inpelm, 'skip nodes' )
         iinp = iinp+2+inpelm
         if ( debug ) write(irefwr,1) 'iinp_loc', iinp-iinpbef+1

      end do  ! i = 1, numgrp

      numobstlc = iinput(16)
      if ( numobstlc>0 ) write(irefwr,*) 'element goups'

      do i = 1, numobstlc

         write(irefwr,1) 'bitpattern', iinput(iinp)
         call printposint ( iinput(iinp+1), 'jstep' )
         call printposint ( iinput(iinp+2), 'iobst' )
         iinp = iinp+3
         if ( debug ) write(irefwr,1) 'iinp_loc', iinp-iinpbef+1

      end do  ! i = 1, numobstlc

      nesallobst = iinput(45)
      if ( nesallobst>0 ) write(irefwr,*) 'all obstacles'

      do i = 1, nesallobst

         write(irefwr,1) 'bitpattern', iinput(iinp)
         call printposint ( iinput(iinp+1), 'iobst' )
         iinp = iinp+2
         if ( debug ) write(irefwr,1) 'iinp_loc', iinp-iinpbef+1

      end do  ! i = 1, nesallobst

      nesinnobst = iinput(41)
      if ( nesinnobst>0 ) write(irefwr,*) 'in obstacles'

      do i = 1, nesinnobst

         write(irefwr,1) 'bitpattern', iinput(iinp)
         call printposint ( iinput(iinp+1), 'iobst' )
         iinp = iinp+2
         if ( debug ) write(irefwr,1) 'iinp_loc', iinp-iinpbef+1

      end do  ! i = 1, nesinnobst

      nesbnobst = iinput(42)

      if ( nesbnobst>0 ) write(irefwr,*) 'boundary obstacles'

      do i = 1, nesbnobst

         write(irefwr,1) 'bitpattern', iinput(iinp)
         call printposint ( iinput(iinp+1), 'iobst' )
         iinp = iinp+2
         if ( debug ) write(irefwr,1) 'iinp_loc', iinp-iinpbef+1

      end do  ! i = 1, nesbnobst

      nesonbnobst  = iinput(43)
      if ( nesonbnobst>0 ) write(irefwr,*) 'in obstacles'

      do i = 1, nesonbnobst

         write(irefwr,1) 'bitpattern', iinput(iinp)
         call printposint ( iinput(iinp+1), 'iobst' )
         iinp = iinp+2
         if ( debug ) write(irefwr,1) 'iinp_loc', iinp-iinpbef+1

      end do  ! i = 1, nesonbnobst

      nesinonbnobst = iinput(46)
      if ( nesinonbnobst>0 ) write(irefwr,*) 'boundary and in obstacles'

      do i = 1, nesinonbnobst

         write(irefwr,1) 'bitpattern', iinput(iinp)
         call printposint ( iinput(iinp+1), 'iobst' )
         iinp = iinp+2
         if ( debug ) write(irefwr,1) 'iinp_loc', iinp-iinpbef+1

      end do  ! i = 1, nesinonbnobst

      nesconct = iinput(19)
      if ( nesconct>0 ) write(irefwr,*) 'contact elements'

      do i = 1, nesconct

         write(irefwr,1) 'bitpattern', iinput(iinp)
         call printposint ( iinput(iinp+1), 'jstep' )
         call printposint ( iinput(iinp+2), 'icontact' )
         iinp = iinp+3
         if ( debug ) write(irefwr,1) 'iinp_loc', iinp-iinpbef+1

      end do  ! i = 1, nesconct

      nesnoconct = iinput(22)
      if ( nesnoconct>0 ) write(irefwr,*) 'no-contact elements'

      do i = 1, nesnoconct

         write(irefwr,1) 'bitpattern', iinput(iinp)
         call printposint ( iinput(iinp+1), 'jstep' )
         call printposint ( iinput(iinp+2), 'icontact' )
         iinp = iinp+3
         if ( debug ) write(irefwr,1) 'iinp_loc', iinp-iinpbef+1

      end do  ! i = 1, nesnoconct

      neszerolev = iinput(47)
      if ( neszerolev>0 ) write(irefwr,*) 'zero levelset'

      do i = 1, neszerolev

         write(irefwr,1) 'bitpattern', iinput(iinp)
         call printposint ( iinput(iinp+1), 'level set' )
         iinp = iinp+2
         if ( debug ) write(irefwr,1) 'iinp_loc', iinp-iinpbef+1

      end do  ! i = 1, neszerolev

      nesnodal = iinput(28)
      if ( nesnodal>0 ) write(irefwr,*) 'nodal points'

      do i = 1, nesnodal

         write(irefwr,1) 'bitpattern', iinput(iinp)
         numnodal = iinput(iinp+1)  ! number of nodes
         if ( numnodal>0 ) then
            call prinin ( iinput(iinp+2), numnodal, 'nodes' )
            iinp = iinp+2+numnodal
         else
            call printposint ( iinput(iinp+2), 'ipos' )
            iinp = iinp+3
         end if  ! ( numnodal>0 )
         if ( debug ) write(irefwr,1) 'iinp_loc', iinp-iinpbef+1

      end do  ! i = 1, nesnodal

      nescav = iinput(49)
      if ( nescav>0 ) write(irefwr,*) 'cavity'
      do i = 1, nescav

         write(irefwr,1) 'bitpattern', iinput(iinp)
         call printposint ( iinput(iinp+1), 'cavity' )
         iinp = iinp+2

      end do  ! i = 1, nescav

      nesintersect = iinput(56)
      if ( nesintersect>0 ) write(irefwr,*) 'intersection'
      do i = 1, nesintersect

         if ( debug ) write(irefwr,1) 'i, iinp', i, iinp
         write(irefwr,1) 'bitpattern', iinput(iinp)
         call printposint ( iinput(iinp+1), 'type of intersection' )
         numsrf = iinput(iinp+2)
         call prinin ( iinput(iinp+3), min(numsrf,100), 'surfaces' )
         iinp = iinp+3+numsrf

      end do  ! i = 1, nesintersect

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End printprobinp06'

      end if  ! ( debug )
      call erclos ( 'printprobinp06' )

      end
