      subroutine prob01
! ======================================================================
!
!        programmer    Guus Segal
!        version 15.0  date 05-04-2012 Remove ielshape
!        version 14.4  date 15-03-2012 New type of input
!        version 14.3  date 01-05-2011 Extension with level set
!
!   copyright (c) 1984-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill information concerning the natural boundary conditions
!     in kprob
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     boundary_element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numpnt, numcrv, numsrf, ikprbi, iinp, numobstlc, &
              i, natconct, mmindex, &
              natconpt, natnocel, natnocpt, natcrpnt, &
              natsrpnt, ih, ikprbisv, jinp, &
              natobstcross, natlevset, nintersect
      integer, allocatable :: iposgrp(:), kcrosselem(:), iworkcross(:)

!     i              Counting variable
!     ih             Help variable to store a constant temporarily
!     iinp           last position used in array iinputkp
!     ikprbi         last position used in array kprob part i
!     ikprbisv       Help variable to store ikprbi
!     iposgrp        work array of length numnat in which the starting
!                    addresses of the element groups in KPROB part i are
!                    stored
!     iworkcross     integer work array of length natobstcross that
!                    contains the number of elements per set of boundary
!                    elements corresponding to the intersections
!     jinp            position used in array iinputkp
!     kcrosselem     array of size nfictunk x 2 with the following contents
!                    kcrosselem(i,1) contains the first node of the element
!                    kcrosselem(i,2) contains the second node of the
!                    element
!     mmindex        memory management sequence number of an array
!     natconct       Number of contacts where natural boundary elements
!                    are given
!     natconpt       number of parts where contact points are defined as
!                    bound els.
!     natcrpnt       Number of curves where point boundary elements
!                    are given
!     natlevset      number of parts where zero level set elements
!                    are defined
!     natnocel       number of parts where no_contact elements are defined
!     natnocpt       number of parts where no_contact points are defined as
!                    boundary elements
!     natobstcross   number of parts where cross_section_obstacle elements
!                    are defined
!     natsrpnt       Number of surfaces where point boundary elements are
!                    given
!     nintersect     Number of intersections found
!     numcrv         number of parts where curves are given for generation
!                    of boundary elements
!     numobstlc      Local number of obstacles
!     numpnt         number of parts where points are given for generation
!                    of boundary elements
!     numsrf         number of parts where surfaces are given for
!                    generation of boundary elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PR0000         print kprob parts b and i
!     PRININ1        print 2d integer array
!     PROB01CONST    Fill constants for subroutine prob01
!     PROB01CROBST1  Fill the elements corresponding to cross sections with an
!                    obstacle into kprobi and update kprobz
!     PROB02         Fill KPROB part i for point boundary elements
!     PROB03         Fill KPROB part i for curve boundary elements  New input
!     PROB06         Compute length of KPROB part i with respect to point
!                    boundary elements
!     PROB07         Fill KPROB part i for surface boundary elements  New input
!     PROB08         Compute length of KPROB part i with respect to curve
!                    boundary elements
!     PROB11         Compute length of KPROB part i with respect to surface
!                    boundary elements
!     PROB12         Compute length of KPROB part i with respect to boundary
!                    elements corresponding to u equals unknown constant
!     PROB15         Fill KPROB part i for boundary elements corresponding to u
!                    equals unknown constant
!     PROB17         Compute length of KPROB part i with respect to obstacle
!                    boundary elements
!     PROB18         Fill KPROB part i for obstacle boundary elements
!     PROB24         Compute length of KPROB part i with respect to contact
!                    boundary elements
!     PROB25         Fill KPROB part i for contact boundary elements
!     PROB26         Compute length of KPROB part i with respect to contact
!                    point boundary elements
!     PROB27         Fill KPROB part i for contact point boundary elements
!     PROB28         Compute length of KPROB part i with respect to contact
!                    boundary elements
!     PROB29         Compute length of KPROB part i with respect to contact
!                    point boundary elements
!     PROB30         Fill KPROB part i for contact boundary elements
!     PROB31         Fill KPROB part i for contact point boundary elements
!     PROB32         Compute length of KPROB part i with respect to point
!                    boundary elements along curves
!     PROB33         Compute length of KPROB part i with respect to point
!                    boundary elements along surfaces
!     PROB34         Fill KPROB part i for point boundary elements along curves
!                    New input
!     PROB35         Fill KPROB part i for point boundary elements along
!                    surfaces  New input
!     PROBNATBLEVEL  Fill kprob part b and iposgrp with information of elements
!                    for natural boundary conditions defined on zero levelset
!                    points
!     PROBNATBLEVS   Fill kprob part i with elements for natural
!                    boundary conditions defined on zero levelset points
!     PROBNATOBSTCR  Fill KPROB part b with respect to cross-sections with
!                    an obstacle natural boundary conditions
!     SEPFILLKMAD3   Fill array kmesh part ad as well as the subarrays
!                    ipointobst and ielemobst (R^3 only)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob01' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob01'
  1      format ( a, 1x, (10i6) )

      end if

!     --- Set constants

      call prob01const ( numpnt, numcrv, numsrf, &
                         numobstlc, natconct, natconpt, &
                         natnocel, natnocpt, natcrpnt, natsrpnt, &
                         natobstcross, natlevset )

      if ( debug ) then
         call prinin1 ( kprobj, 2, size(kprobj,1), 'kprobj' )
         write(irefwr,1) 'nelgrp, npoint, numnat, numobstlc', &
                          nelgrp, npoint, numnat, numobstlc
      end if  ! ( debug )

      ikprbi = 0
      iinp   = intarprb(3)-1

      if ( numnat>0 ) then

!     --- numnat>0
!         Create temporary array iposgrp to store the starting addresses of
!         all boundary groups in kprob part i
!         Clear this array

         allocate ( iposgrp(numnat), stat = error )
         if ( error/=0 ) call eralloc ( error, numnat, 'iposgrp' )
         if ( ierror/=0 ) go to 1000

         iposgrp = 0

      end if  ! ( numnat>0 )

!     --- Fill the arrays KPROB part b and IWORK

      if ( numpnt>0 ) call prob06 ( numpnt, ikprbi, iinp, iposgrp )

      if ( numcrv>0 ) call prob08 ( numcrv, ikprbi, iinp, iposgrp )

      if ( numsrf>0 ) call prob11 ( numsrf, ikprbi, iinp, iposgrp )

      if ( numobstlc>0 .and. kelmh>0 ) &
         call  prob17 ( ikprbi, iinp, iposgrp, numobstlc )

      if ( kprobloc(48)>0 .and. natconct>0 .and. kprobloc(47)>0 ) &
         call  prob24 ( natconct, ikprbi, iinp, iposgrp )

      if ( kprobloc(48)>0 .and. natnocel>0 .and. kprobloc(47)>0 ) &
         call prob28 ( natnocel, ikprbi, iinp, iposgrp )

      if ( kprobloc(48)>0 .and. natconpt>0 .and. kprobloc(46)>0 ) &
         call prob26 ( natconpt, ikprbi, iinp, iposgrp )

      if ( kprobloc(48)>0 .and. natnocpt>0 .and. kprobloc(46)>0 ) &
         call prob29 ( natnocpt, ikprbi, iinp, iposgrp )

      if ( natcrpnt>0 ) &
         call prob32 ( natcrpnt, ikprbi, iinp, iposgrp )

      if ( natsrpnt>0 ) &
         call prob33 ( natsrpnt, ikprbi, iinp, iposgrp )

      if ( debug ) write(irefwr,1) 'natlevset', natlevset
      if ( natlevset>0 ) then

!     --- natlevset>0  zero level set boundary elements

         call probnatblevel ( natlevset, iinp, iposgrp, ikprbi )

      end if  ! ( natlevset>0 )

      if ( debug ) write(irefwr,1) 'natobstcross', natobstcross
      if ( natobstcross>0 ) then

!     --- natobstcross>0 boundary elements due to cross-sections with
!         an obstacle
!         activate arrays

         if ( kelmad==0 ) then
            call sepfillkmad3
         else if ( kmeshad(3)==0 .or. kmeshad(5)/=kmeshad(8) ) then
            call sepfillkmad3
         end if  ! ( kelmad==0 )

         nintersect = 0
         if ( kmeshad(4)>0 ) nintersect = size ( rcrossobst )
         if ( nintersect>0 ) then

!        --- Intersections have been found

            allocate ( kcrosselem(2*nintersect), &
                       iworkcross(natobstcross), stat = error )
            if ( error/=0 ) &
               call eralloc ( error, 2*nintersect+natobstcross, &
                              'kcrosselem' )
            if ( ierror/=0 ) go to 1000

            call probnatobstcr ( natobstcross, ikprbi, iinp, iposgrp, &
                                 kcrosselem, iworkcross, nintersect )

         end if  ! ( nintersect>0 )

      end if  ! ( natobstcross>0 )

      ikprbisv = ikprbi

      if ( nbcons>0 ) then
         jinp = intarprb(6)
         call prob12 ( ikprbi, jinp )
      end if  ! ( nbcons>0 )

      if ( ikprbi>0 ) then

!     --- Boundary elements are present
!         KPROB part i has length ikprbi
!         Reserve space for kprob part i
!         Accumulate all information in iwork one position

         allocate ( kp(iseqkp)%kprobsub(iprob)%kprobi(ikprbi) )
         kprobi => kprobpart%kprobi
         kprobloc(22) = ikprbi

         ikprbi = 0
         do i = 1, numnat
            ih  = ikprbi
            ikprbi = ikprbi+iposgrp(i)
            iposgrp(i) = ih
         end do

!        --- Fill KPROB part i

         iinp = intarprb(3)-1
         if ( numpnt>0 ) call prob02 ( numpnt, iposgrp, iinp )

         if ( numcrv>0 ) call prob03 ( numcrv, iinp, iposgrp )

         if ( numsrf>0 ) call prob07( numsrf, iinp, iposgrp )

         if ( numobstlc>0 .and. kelmh>0 ) &
            call prob18 ( numobstlc, iinp, iposgrp )

         if ( kprobloc(48)>0 .and. natconct>0 .and. kprobloc(47)>0 ) &
            call prob25 ( natconct, iinp, iposgrp )

         if ( kprobloc(48)>0 .and. natnocel>0 .and. kprobloc(47)>0 ) &
            call prob30 ( natnocel, iinp, iposgrp )

         if ( kprobloc(48)>0 .and. natconpt>0 .and. kprobloc(46)>0 ) &
            call prob27 ( natconpt, iinp, iposgrp )

         if ( kprobloc(48)>0 .and. natnocpt>0 .and. kprobloc(46)>0 ) &
            call prob31 ( natnocpt, iinp, iposgrp )

         if ( natcrpnt>0 ) &
            call prob34 ( natcrpnt, iinp, iposgrp )

         if ( natsrpnt>0 ) &
            call prob35 ( natsrpnt, iinp, iposgrp )

         if ( natlevset>0 ) then

!        --- natlevset>0  zero level set boundary elements

            call probnatblevs ( natlevset, iinp, iposgrp, ikprbi )

         end if  ! ( natlevset>0 )

         if ( natobstcross>0 ) then

!        --- natobstcross>0 boundary elements due to cross-sections with
!            an obstacle

            if ( debug ) write(irefwr,1) 'nintersect', nintersect
            if ( nintersect>0 ) then

!           --- Intersections have been found

               call prob01crobst1 ( kcrosselem, natobstcross, iposgrp, &
                                    iworkcross, nintersect, iinp )

               deallocate ( kcrosselem, iworkcross, stat = error )
               if ( error/=0 ) call erdealloc ( error, 'kcrosselem' )

            end if  ! ( nintersect>0 )

         end if  ! ( natobstcross>0 )

      end if

      ikprbi = ikprbisv
      if ( nbcons>0 ) then
         jinp = intarprb(6)
         call prob15 ( ikprbi, jinp )
      end if  ! ( nbcons>0 )

      if ( iinputkp(2)>0 ) then

!     --- numnat>0

         deallocate ( iposgrp, stat = error )
         if ( error/=0 ) call erdealloc ( error, 'iposgrp' )

      end if  ! ( numnat>0 )


1000  if ( debug ) then

!     --- Debug information

         call pr0000
         write(irefwr,*) 'End of prob01'

      end if
      call erclos ( 'prob01' )

      end subroutine prob01
