      subroutine prob04elem ( iinput, iinp, nkbnd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the number of nodal points with prescribed boundary
!     conditions for elements
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, nkbnd

!     iinp          i/o   last position used in array iinput
!     iinput         i    integer input array
!                         See subroutine prord3
!     nkbnd         i/o   Number of points with essential boundary conditions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, j, ifirst, ilast, inpelm, jnelem, &
              k, l, ielgrp, irelnp, numelm

!     ielem          Element number
!     ielgrp         element group number
!     ifirst         First element number
!     ilast          Last element number
!     inpelm         number of points per element
!     irelnp         Relative nodal point number
!     j              Counting variable
!     jnelem         number of elements in an element group for one super
!                    element
!     k              Counting variable
!     l              General loop variable
!     numelm         Number of elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     440   first element number in element record incorrect
!     441   last element number in element record incorrect
!     442   relative nodal point number in element incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob04elem' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob04elem'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inpelm = npelm
      numelm = iinput(11)
      do l = 1,numelm
         ifirst = iinput(iinp+2)
         ilast = iinput(iinp+3)
         if ( ilast==0 ) ilast = ifirst
         if ( ifirst<=0 .or. ifirst>nelem ) then
            call errint ( ifirst, 1 )
            call errint ( nelem, 2 )
            call errsub ( 440, 2, 0, 0 )
         end if
         if ( ilast<ifirst .or. ilast>nelem ) then
            call errint ( ilast, 1 )
            call errint ( ifirst, 2 )
            call errint ( nelem, 3 )
            call errsub ( 441, 3, 0, 0 )
         end if
         irelnp = iinput(iinp+4)
         iinp = iinp+4
         if ( ierror/=0 ) goto 1000
         inelem = 0
         ielgrp = 1
         jnelem = nelem
         if ( nelgrp>1 ) then
            do k = 1,nelgrp
               ielgrp = k
               jnelem = kmeshg(k)
               inelem = inelem+jnelem
               if ( inelem>=ifirst ) exit
               inpelm = Topgrp(k)%inpelm
            end do
            inelem = inelem-jnelem

         end if

         ielem = ifirst
         do k = ielgrp, nelgrp
            if ( kelma/=0 ) inpelm = Topgrp(k)%inpelm
            if ( nelgrp>1 ) jnelem = kmeshg(k)
            if ( k==ielgrp ) then
               jnelem = jnelem-ifirst+1
            end if
            if ( irelnp<=0 .or. irelnp>inpelm ) then
               call errint ( irelnp, 1 )
               call errint ( inpelm, 2 )
               call errint ( ielgrp, 3 )
               call errsub ( 442, 3, 0, 0 )
            end if
            do j = 1,jnelem
               nkbnd = nkbnd+1
               ielem = ielem+1
               if ( ielem>ilast ) goto 1000
            end do

         end do

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob04elem'

      end if  ! ( debug )
      call erclos ( 'prob04elem' )

      end
