      subroutine prob04nodp ( iinput, iinp, nkbnd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the number of nodal points with prescribed boundary
!     conditions for points
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp, nkbnd

!     iinp          i/o   last position used in array iinput
!     iinput         i    integer input array
!                         See subroutine prord3
!     nkbnd         i/o   Number of points with essential boundary conditions
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numnod, j, ifirst, ilast

!     ifirst         First node number
!     ilast          Last node number
!     j              Counting variable
!     numnod         Number of points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     438   first nodal point number in nodes record incorrect
!     439   last nodal point number in nodes record incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob04nodp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob04nodp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numnod = iinput(10)
      do j = 1,numnod
         ifirst = iinput(iinp+2)
         ilast = iinput(iinp+3)
         if ( ilast==0 ) ilast = ifirst
         if ( ifirst<=0 .or. ifirst>npoint ) then
            call errint ( ifirst, 1 )
            call errint ( npoint, 2 )
            call errsub ( 438, 2, 0, 0 )
         end if
         if ( ilast<ifirst .or. ilast>npoint ) then
            call errint ( ilast, 1 )
            call errint ( ifirst, 2 )
            call errint ( npoint, 3 )
            call errsub ( 439, 3, 0, 0 )
         end if
         iinp = iinp+3
         nkbnd = nkbnd + ilast-ifirst+1
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob04nodp'

      end if  ! ( debug )
      call erclos ( 'prob04nodp' )

      end
