      subroutine prob05cvstep ( jstep, iexclu, npointscurve, &
                                nodes, inunkp, ibit, iinp, kstart )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     perform the jstep part of prob05cvold/new
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jstep, iexclu, npointscurve, nodes(*), &
              inunkp, ibit, iinp, kstart

!     ibit           i    Actual bit pattern
!     iexclu         i    Indicates which points are excluded
!     iinp           i    last position used in array iinputkp
!     inunkp         i    Number of degrees of freedom in point
!     jstep          i    Step in loop
!     kstart         i    Start value of parameter
!     nodes         i/o   Contains the node numbers that must be filled
!     npointscurve   i    number of points along a curve
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer kend, k, kstep, inodp, j, ibit2, ih

!     ibit2          Maximal bit pattern
!     ih             Help variable to store a constant temporarily
!     inodp          Nodal point number
!     j              Counting variable
!     k              Counting variable
!     kend           End value of parameter
!     kstep          Step in loop
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inibt0, inibt1

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     INIBT0         Compute bit pattern
!     INIBT1         Compute maximum unknown corresponding to bit pattern
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1861   Wrong unknown prescribed at curve
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob05cvstep' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob05cvstep'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kstep = abs(jstep)+1
      ibit2  = inibt0(inunkp)

      if ( jstep>=0 ) then

!     --- jstep>=  0   end points of intervals
!         Correct for initial and end point in the case that exclude>0

         if ( iexclu==0 ) then

!        --- iexclu  =  0  No excluding of nodal points

            kstart = 1
            kend = npointscurve

         else if ( iexclu==1 ) then

!        --- iexclu  =  1 Exclude initial point

            kstart = 2
            kend = npointscurve

         else if ( iexclu==2 ) then

!        --- iexclu  =  2 Exclude end point

            kstart = 1
            kend = npointscurve-1

         else if ( iexclu==3 ) then

!        --- iexclu  =  3 Exclude initial and end point

            kstart = 2
            kend = npointscurve-1

         end if

         do k = kstart, kend, kstep

            inodp = nodes(k)
            if ( inodp>0 ) then

!           --- nodal point found

               if ( indprf>0 ) then

!              --- indprf>0  kprob part f contains the number of
!                  unknowns

                  inunkp = kprobf(inodp+1)-kprobf(inodp)
                  ibit2 = inibt0(inunkp)

               end if

               if ( ibit>ibit2 .or. ibit<0 ) then
                  call errint ( inibt1(ibit), 1 )
                  call errint ( k-kstart+1, 2 )
                  call errint ( iinputkp(iinp+3), 3 )
                  call errint ( iinputkp(iinp+3), 4 )
                  call errint ( inunkp, 5 )
                  call errwar ( 1861, 5, 0, 0 )
               end if
               ih = ibit
               if ( ih==0 ) ih = ibit2
               nkbnd = nkbnd+1
               kprobg(1,nkbnd) = inodp
               kprobg(2,nkbnd) = ih
               if ( debug ) &
                  write(irefwr,1) 'nkbnd, inodp/1', nkbnd, inodp

            end if

         end do

      else

!     --- jstep <0      points between intervals

         npointscurve = npointscurve-1
         do k = 1, npointscurve, kstep
            do j = 1, kstep-1
               inodp = nodes(k+j)
               if ( indprf>0 ) then

!              --- indprf>0  kprob part f contains the number of
!                  --- unknowns

                  inunkp = kprobf(inodp+1)-kprobf(inodp)
                  ibit2 = inibt0(inunkp)

               end if

               if ( ibit>ibit2 .or. ibit<0 ) then
                  call errint ( inibt1(ibit), 1 )
                  call errint ( k+j, 2 )
                  call errint ( iinputkp(iinp+3), 3 )
                  call errint ( iinputkp(iinp+3), 4 )
                  call errint ( inunkp, 5 )
                  call errwar ( 1861, 5, 0, 0 )
               end if
               nkbnd = nkbnd+1
               ih = ibit
               if ( ih==0 ) ih = ibit2
               kprobg(1,nkbnd) = inodp
               kprobg(2,nkbnd) = ih
               if ( debug ) &
                  write(irefwr,1) 'nkbnd, inodp/2', nkbnd, inodp

            end do  ! j = 1, kstep-1

         end do

      end if

1000  call erclos ( 'prob05cvstep' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob05cvstep'

      end if  ! ( debug )

      end
