      subroutine prob05elem ( iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill KPROB part g with information concerning the essential boundary
!     conditions for elements
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     user_point
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp

!     iinp          i/o   last position used in array iinputkp
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, j, ifirst, ilast, inpelm, jnelem, &
              k, l, ielgrp, irelnp, numelm, &
              inunkp, inodp, ibit, ibit2, ih, ikelmc, nodalp

!     ibit           Actual bit pattern
!     ibit2          Maximal bit pattern
!     ielem          Element number
!     ielgrp         element group number
!     ifirst         First curve (surface) number read
!     ih             Help variable to store a constant temporarily
!     ikelmc         position in kmesh part c
!     ilast          Last curve (surface) number read
!     inodp          Nodal point number
!     inpelm         number of points per element
!     inunkp         Number of degrees of freedom in point
!     irelnp         Relative nodal point number
!     j              Counting variable
!     jnelem         number of elements in an element group for one super
!                    element
!     k              Counting variable
!     l              General loop variable
!     nodalp         Nodal point sequence number
!     numelm         Number of elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inibt0

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     INIBT0         Compute bit pattern
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      77   You try to prescribe the wrong degree of freedom
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob05elem' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob05elem'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inunkp = nunkp
      ibit2  = inibt0(inunkp)

      inpelm = npelm
      numelm = iinputkp(11)
      do l = 1,numelm
         ibit = iinputkp(iinp+1)
         ifirst = iinputkp(iinp+2)
         ilast = iinputkp(iinp+3)
         if ( ilast==0 ) ilast = ifirst
         irelnp = iinputkp(iinp+4)
         iinp = iinp+4
         inelem = 0
         ikelmc = 0
         ielgrp = 1
         jnelem = nelem
         if ( nelgrp>1 ) then

!        --- nelgrp>1

            do k = 1,nelgrp
               ielgrp = k
               jnelem = Topgrp(k)%inelem
               inelem = inelem+jnelem
               if ( inelem>=ifirst ) exit
               inpelm = Topgrp(k)%inpelm
               ikelmc = ikelmc+inpelm*jnelem
            end do
            inelem = inelem-jnelem

         end if

         ielem = ifirst
         do k = ielgrp, nelgrp
            inpelm = Topgrp(k)%inpelm
            jnelem = Topgrp(k)%inelem
            if ( k==ielgrp ) then
               jnelem = jnelem-ifirst+1
               ikelmc = ikelmc+(ifirst-inelem-1)*inpelm
            end if
            do j = 1,jnelem
               nodalp = kmeshc(ikelmc+irelnp)
               ikelmc = ikelmc+inpelm
               if ( indprf>0 ) then

!              --- indprf>0  kprob part f contains the number of unknowns

                  inunkp = kprobf(nodalp+1)-kprobf(nodalp)
                  ibit2 = inibt0(inunkp)

               end if

               if ( ibit>ibit2 .or. ibit<0 ) then
                  call errint ( ibit, 1 )
                  call errint ( ibit2, 2 )
                  call errint ( inodp, 3 )
                  call errwar ( 77, 3, 0, 0 )
               end if

               nkbnd = nkbnd+1
               ih = ibit
               if ( ih==0 ) ih = ibit2
               kprobg(1,nkbnd) = nodalp
               kprobg(2,nkbnd) = ih
               ielem = ielem+1
               if ( ielem>ilast) goto 400
            end do
         end do
400   end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob05elem'

      end if  ! ( debug )
      call erclos ( 'prob05elem' )

      end
