      subroutine prob05nodal ( iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill KPROB part g with information concerning the essential boundary
!     conditions for nodes
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     node
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp

!     iinp          i/o   last position used in array iinputkp
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nesnodal, i, numnodal, ibit

!     i              Counting variable
!     ibit           Actual bit pattern
!     nesnodal       Number of groups of nodal points with essential
!                    boundary conditions
!     numnodal       Number of nodal points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PROB05NODAL1   Fill essential boundary conditions for nodes
!     PROB05NODAL2   Fill essential boundary conditions for nodes in a file
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob05nodal' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob05nodal'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nesnodal = iinputkp(28)

      do i = 1, nesnodal
         ibit = iinputkp(iinp+1)
         numnodal = iinputkp(iinp+2)
         iinp = iinp+2
         if ( numnodal>0 ) then

!        --- numnodal>0, point numbers given

            call prob05nodal1 ( iinp, ibit, numnodal )

         else if ( numnodal<0 ) then

!        --- numnodal<0, point numbers in file
!            Read number from file

            call prob05nodal2 ( ibit, numnodal )
            iinp = iinp+1

         end if

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob05nodal'

      end if  ! ( debug )
      call erclos ( 'prob05nodal' )

      end
