      subroutine prob05nodp ( iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-10-2013
!
!   copyright (c) 2013-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill KPROB part g with information concerning the essential boundary
!     conditions for user points
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     user_point
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp

!     iinp          i/o   last position used in array iinputkp
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, i, ibit, ibit2, ifirst, ilast, numnod, &
              ih, inunkp, inodp

!     i              Counting variable
!     ibit           Actual bit pattern
!     ibit2          Maximal bit pattern
!     ifirst         First user point
!     ih             Help variable to store a constant temporarily
!     ilast          Last user point
!     inodp          Nodal point number
!     inunkp         Number of degrees of freedom in point
!     j              Counting variable
!     numnod         Number of points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inibt0

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     INIBT0         Compute bit pattern
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      77   You try to prescribe the wrong degree of freedom
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob05nodp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob05nodp'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      numnod = iinputkp(10)
      inunkp = nunkp
      ibit2  = inibt0(inunkp)

      if ( debug ) &
         write(irefwr,1) 'numnod, inunkp, indprf, ibit2', &
                          numnod, inunkp, indprf, ibit2

      do j = 1,numnod
         ibit = iinputkp(iinp+1)
         ifirst = iinputkp(iinp+2)
         ilast = iinputkp(iinp+3)
         if ( ilast==0 ) ilast = ifirst
         iinp = iinp+3

         if ( debug ) &
            write(irefwr,1) 'j, ibit, ifirst, ilast, iinp', &
                             j, ibit, ifirst, ilast, iinp

         do i = ifirst,ilast

            if ( indprf>0 ) then

!           --- indprf>0  kprob part f contains the number of unknowns

               inunkp = kprobf(i+1)-kprobf(i)
               ibit2 = inibt0(inunkp)

            end if

            if ( ibit>ibit2 .or. ibit<0 ) then
               call errint ( ibit, 1 )
               call errint ( ibit2, 2 )
               call errint ( inodp, 3 )
               call errwar ( 77, 3, 0, 0 )
            end if
            nkbnd = nkbnd+1
            ih = ibit
            if ( ih==0 ) ih = ibit2

            if ( debug ) write(irefwr,1) 'i, ih, nkbnd', i, ih, nkbnd
            kprobg(1,nkbnd) = i
            kprobg(2,nkbnd) = ih

         end do

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob05nodp'

      end if  ! ( debug )
      call erclos ( 'prob05nodp' )

      end
