      subroutine prob05obst ( iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 13-10-2013 New parameter list
!        version  1.3  date 29-11-2004 Correction for points in obstacle and
!                                      on boundary
!        version  1.2  date 07-01-2004 Debug statements
!
!   copyright (c) 2000-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill KPROB part g with information concerning the essential
!     conditions for nodes in obstacles
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     obstacle
!     essential_boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp

!     iinp          i/o   last position used in array iinputkp
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nesallobst, nesinnobst, iobst, ibit2, ibit, &
              nesbnobst, istart, numpntobst, numinnobst, numbndobst, &
              nesonbnobst, nesinonobst, numinonobst, numobstpnt, inunkp

!     i              Counting variable
!     ibit           Actual bit pattern
!     ibit2          Maximal bit pattern
!     inunkp         Number of degrees of freedom in point
!     iobst          Obstacle sequence number
!     istart         First position used at a part
!     nesallobst     Number of obstacles where all nodes are prescribed
!     nesbnobst      Number of obstacles where all but the inner nodes are
!                    prescribed
!     nesinnobst     Number of obstacles where all inner nodes are prescribed
!     nesinonobst    Number of obstacles where all special nodes on the
!                    boundary of an obstacle are prescribed
!     nesonbnobst    Number of obstacles where all the nodes on the boundary of
!                    the obstacle are prescribed
!     numbndobst     Number of nodal points on the boundary of the obstacle
!     numinnobst     Number of points that are near the boundary of the obstacle
!     numinonobst    Number of nodal points on the boundary of the obstacle
!                    which are only in elements completely inside the obstacle
!     numobstpnt     Number of points on obstacle to be treated
!     numpntobst     Number of nodal points inside the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inibt0

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INIBT0         Compute bit pattern
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     PROB05OBST1    Fill array kprobg with information of essential boundary
!                    conditions w.r.t. nodes stored in ipointobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!      nesallobst  := iinputkp(45)
!      nesinnobst  := iinputkp(41)
!      nesbnobst   := iinputkp(42)
!      nesbonnobst := iinputkp(43)
!      nesinonobst := iinputkp(46)
!      for i := 1 (1) nesallobst do
!         iobst := obstacle number
!         for all points in obstacle do
!            fill kprobg
!            nkbnd := nkbnd+1
!      for i := 1 (1) nesinnobst do
!         iobst := obstacle number
!         for all points in obstacle do
!            fill kprobg
!            nkbnd := nkbnd+1
!      for i := 1 (1) nesbnobst do
!         iobst := obstacle number
!         for all points near the boundary of the obstacle do
!            fill kprobg
!            nkbnd := nkbnd+1
!      for i := 1 (1) nesonbnobst do
!         iobst := obstacle number
!         for all points on the boundary of the obstacle do
!            fill kprobg
!            nkbnd := nkbnd+1
!      for i := 1 (1) nesinonobst do
!         iobst := obstacle number
!         for all points on the boundary of the obstacle do
!            fill kprobg
!            nkbnd := nkbnd+1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob05obst' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob05obst'

      end if
      if ( ierror/=0 ) go to 1000

!     --- nesallobst  := iinputkp(45)
!         nesinnobst  := iinputkp(41)
!         nesbnobst   := iinputkp(42)
!         nesbonnobst := iinputkp(43)

      nesallobst  = iinputkp(45)
      nesinnobst  = iinputkp(41)
      nesbnobst   = iinputkp(42)
      nesonbnobst = iinputkp(43)
      nesinonobst = iinputkp(46)
      if ( debug ) then
         write(irefwr,*) 'nesallobst, nesinnobst, nesbnobst', &
                         ' nesonbnobst, nesinonobst ', &
                          nesallobst, nesinnobst, nesbnobst, &
                          nesonbnobst, nesinonobst
      end if

      inunkp = nunkp
      ibit2  = inibt0(inunkp)

!     --- for i := 1 (1) nesallobst do

      do i = 1, nesallobst

!     --- iobst := obstacle number

         ibit = iinputkp(iinp+1)
         iobst = iinputkp(iinp+2)
         iinp = iinp+2
         istart = ipointobst(iobst)
         numpntobst = ipointobst(istart)
         numinnobst = ipointobst(istart+1)
         numbndobst = ipointobst(istart+2)
         numinonobst = ipointobst(istart+3)

         istart = istart+3

         if ( debug ) then
            write(irefwr,*) 'allpoints '
            write(irefwr,*) 'nesallobst, ibit, iobst ', &
                             nesallobst, ibit, iobst
            write(irefwr,*) 'i, numpntobst, numinnobst, numbndobst, ', &
                            'numinonobst ', &
                             i, numpntobst, numinnobst, numbndobst, &
                             numinonobst
            call prinin ( ipointobst(istart+1), &
                          numpntobst+numinnobst+numbndobst+numinonobst, &
                          'points' )
         end if

!        --- for j := 1 (1) npoint do

         numobstpnt = numpntobst+numinnobst+numbndobst+numinonobst
         call prob05obst1 ( ipointobst, istart, numobstpnt, &
                            ibit, ibit2, iobst )

      end do

!     --- for i := 1 (1) nesinnobst do

      do i = 1, nesinnobst

!     --- iobst := obstacle number

         ibit = iinputkp(iinp+1)
         iobst = iinputkp(iinp+2)
         iinp = iinp+2

!        --- internal nodes

         istart = ipointobst(iobst)
         numpntobst = ipointobst(istart)
         numinnobst = ipointobst(istart+1)
         numbndobst = ipointobst(istart+2)
         numinonobst = ipointobst(istart+3)
         istart = istart+3

         if ( debug ) then
            write(irefwr,*) 'points in obstacle '
            write(irefwr,*) 'nesinnobst, numinonobst, ibit, iobst ', &
                             nesinnobst, numinonobst, ibit, iobst
            write(irefwr,*) 'i, numpntobst ', i, numpntobst
            call prinin ( ipointobst(istart+1), numpntobst, 'points/1' )
            istart = istart+numpntobst+numinnobst+numbndobst
            call prinin ( ipointobst(istart+1), numinonobst, 'points/2')
         end if

         istart = ipointobst(iobst)
         istart = istart+3
         call prob05obst1 ( ipointobst, istart, numpntobst, &
                            ibit, ibit2, iobst )
         istart = istart+numpntobst+numinnobst+numbndobst
         call prob05obst1 ( ipointobst, istart, numinonobst, &
                            ibit, ibit2, iobst )

      end do

!     --- for i := 1 (1) nesbnobst do

      do i = 1, nesbnobst

!     --- iobst := obstacle number

         ibit = iinputkp(iinp+1)
         iobst = iinputkp(iinp+2)
         iinp = iinp+2

!        --- boundary nodes

         istart = ipointobst(iobst)
         numpntobst = ipointobst(istart)
         numinnobst = ipointobst(istart+1)
         numbndobst = ipointobst(istart+2)
         istart = istart+3+numpntobst

         if ( debug ) then
            write(irefwr,*) 'points in obstacle near boundary'
            write(irefwr,*) 'nesbnobst, ibit, iobst ', &
                             nesbnobst, ibit, iobst
            write(irefwr,*) 'i, numinnobst, numbndobst ', &
                             i, numinnobst, numbndobst
            call prinin ( ipointobst(istart+1), numinnobst+numbndobst, &
                          'points' )
         end if

         numobstpnt = numinnobst+numbndobst
         call prob05obst1 ( ipointobst, istart, numobstpnt, &
                            ibit, ibit2, iobst )

      end do

!     --- for i := 1 (1) nesonbnobst do

      do i = 1, nesonbnobst

!     --- iobst := obstacle number

         ibit = iinputkp(iinp+1)
         iobst = iinputkp(iinp+2)
         iinp = iinp+2

!        --- boundary nodes

         istart = ipointobst(iobst)
         numpntobst = ipointobst(istart)
         numinnobst = ipointobst(istart+1)
         numbndobst = ipointobst(istart+2)
         numinonobst = ipointobst(istart+3)
         istart = istart+3+numpntobst+numinnobst

         if ( debug ) then
            write(irefwr,*) 'points on obstacle boundary'
            write(irefwr,*) 'nesonbnobst, ibit, iobst ', &
                             nesonbnobst, ibit, iobst
            write(irefwr,*) 'i, numbndobst ', i, numbndobst
            call prinin ( ipointobst(istart+1), numbndobst, 'points' )
         end if

         call prob05obst1 ( ipointobst, istart, numbndobst, &
                            ibit, ibit2, iobst )

      end do

!     --- for i := 1 (1) nesinonobst do

      do i = 1, nesinonobst

!     --- iobst := obstacle number

         ibit = iinputkp(iinp+1)
         iobst = iinputkp(iinp+2)
         iinp = iinp+2

!        --- boundary nodes

         istart = ipointobst(iobst)
         numpntobst = ipointobst(istart)
         numinnobst = ipointobst(istart+1)
         numbndobst = ipointobst(istart+2)
         numinonobst = ipointobst(istart+3)
         istart = istart+3+numpntobst+numinnobst+numbndobst

         if ( debug ) then
            write(irefwr,*) 'special points on obstacle boundary'
            write(irefwr,*) 'nesinonobst, ibit, iobst ', &
                             nesinonobst, ibit, iobst
            write(irefwr,*) 'i, numinonobst ', i, numinonobst
            call prinin ( ipointobst(istart+1), numinonobst, 'points' )
         end if

         call prob05obst1 ( ipointobst, istart, numinonobst, &
                            ibit, ibit2, iobst )

      end do  ! i = 1, nesinonobst

1000  if ( debug ) then

!     --- Debug information

         call prinin1 ( kprobg, nkbnd, 2, 'kprobg' )
         write(irefwr,*) 'End prob05obst'

      end if
      call erclos ( 'prob05obst' )

      end
