      subroutine prob05sfels ( inpelm, isrf1, isrf2, &
                               istart, iinp, iwork, inunkp, ibit )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-04-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill KPROB part g with information concerning the essential
!     boundary conditions for set of elements in surfaces
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     problem_definition
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, isrf1, isrf2, &
              istart, iinp, iwork(*), inunkp, ibit

!     ibit           i    Actual bit pattern
!     iinp          i/o   last position used in array iinputkp
!     inpelm         i    number of points per element
!     inunkp        i/o   Number of degrees of freedom in point
!     isrf1          i    First surface number
!     isrf2          i    Last surface number
!     istart         i    First position used at a part
!     iwork         i/o   integer work array of length npoint
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, nodalp, ibit2, ih, inodp

!     ibit2          Maximal bit pattern
!     ielem          Element number
!     ih             Help variable to store a constant temporarily
!     inodp          Nodal point number
!     nodalp         Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inibt0, inibt1

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     INIBT0         Compute bit pattern
!     INIBT1         Compute maximum unknown corresponding to bit pattern
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1862   Wrong unknown prescribed at surface
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob05sfels' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob05sfels'
         write(irefwr,1) 'inunkp', inunkp
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ibit2  = inibt0(inunkp)

!     --- Loop over elements

      do ielem = 1, inelem

         do nodalp = 1, inpelm

            if ( iinputkp(iinp+nodalp)==1 ) then

!           --- iinputkp(iinp+nodalp) = 1:   prescribed nodal point

               if ( kelmn>0 ) then
                  inodp = kmeshn(istart+nodalp)
               else
                  inodp = kmeshc(istart+nodalp)
               end if

               if ( inodp==0 ) go to 100  ! Jump for empty nodes

               if ( iwork(inodp)==0 ) then

!              --- iwork(inodp) = 0:  new nodal point

                  iwork(inodp) = 1
                  if ( indprf>0 ) then

!                 --- indprf>0  kprob part f contains
!                     the number of unknowns

                     inunkp = kprobf(inodp+1)-kprobf(inodp)
                     ibit2 = inibt0(inunkp)

                  end if

                  if ( ibit>ibit2 .or. ibit<0 ) then
                     call errint ( inibt1(ibit), 1 )
                     call errint ( nodalp, 2 )
                     call errint ( ielem, 3 )
                     call errint ( isrf1, 4 )
                     call errint ( isrf2, 5 )
                     call errint ( inunkp, 6 )
                     call errwar ( 1862, 6, 0, 0 )
                     ibit = 0
                  end if
                  nkbnd = nkbnd+1
                  ih = ibit
                  if ( ih==0 ) ih = ibit2
                  kprobg(1,nkbnd) = inodp
                  kprobg(2,nkbnd) = ih
                  if ( debug ) &
                     write(irefwr,1) 'nkbnd, inodp', nkbnd, inodp
               end if
            end if

100      end do  ! nodalp = 1, inpelm

         istart = istart+inpelm

      end do  ! ielem = 1, inelem

1000  call erclos ( 'prob05sfels' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob05sfels'

      end if  ! ( debug )

      end
