      subroutine prob08new ( iinp, inpelm, ibngrp, &
                             ikprbi, iposgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 14-03-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute length of array kprob part i with respect to boundary
!     elements corresponding to curves
!     This subroutine is the counterpart of subroutine PROB03, &
!     if PROB08 is changed also PROB03 should be changed and vice versa
!     New input
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp, inpelm, ibngrp, &
              ikprbi, iposgrp(*)

!     ibngrp         i    Actual boundary element group number
!     iinp          i/o   last position used in array iinputkp
!     ikprbi        i/o   last position used in array kprob part i
!     inpelm         i    number of points per element
!     iposgrp       i/o   work array of length numnat in which the starting
!                         addresses of the element groups in KPROB part i are
!                         stored
!                         In this subroutine the length with respect to an
!                         element group is added to the existing array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icurnr, npointscurve, lenkmeshm3, i, inum

!     i              Counting variable
!     icurnr         Curve sequence number
!     inum           Number of curves
!     lenkmeshm3     Length of kmesh m part 3
!     npointscurve   number of points along a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PROBCURVNUMELM Compute number of elements along a curve
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     139   curve number does not exist
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob08new' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob08new'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      inum = iinputkp(iinp+3)
      if ( debug ) &
         write(irefwr,1) 'ibngrp, ishape, inpelm', &
                          ibngrp, ishape, inpelm
      lenkmeshm3 = kmeshm(3)-kmeshm(2)

!     --- Loop over the curves

      do i = 1, inum

         icurnr = iinputkp(iinp+3+i)
         if ( icurnr<1 .or. icurnr>ncurvs ) then

!        --- icurnr out of range

            call errint ( icurnr, 1 )
            call errint ( ncurvs, 2 )
            call errsub ( 139, 2, 0, 0 )
            go to 1000

         end if  ! ( icurnr<1 .or. icurnr>ncurvs )

!        --- Detect number of elements along curve

         call probcurvnumelm ( icurnr, npointscurve, ibngrp, lenkmeshm3 )
         if ( debug ) &
            write(irefwr,*) 'icurnr, npointscurve, ibngrp, ikprbi ', &
                             icurnr, npointscurve, ibngrp, ikprbi

!        --- Store in kprob part b

         kprobb(ibngrp) = kprobb(ibngrp)+npointscurve
         ikprbi = ikprbi+npointscurve*inpelm
         iposgrp(ibngrp) = iposgrp(ibngrp)+npointscurve*inpelm

      end do
      iinp = iinp+3+inum

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob08new'

      end if  ! ( debug )
      call erclos ( 'prob08new' )

      end
