      subroutine prob14ess ( iinput, iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.5  date 13-10-2013 Extension with intersection
!        version  1.4  date 03-07-2012 Correction cavity
!        version  1.3  date 03-04-2012 New definition of input
!
!   copyright (c) 2004-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Raise iinp for essential boundary conditions (subroutine PROBDF)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     initialization
!     essential_boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), iinp

!     iinp          i/o   last position used in array iinput
!     iinput         i    integer input array corresponding to subroutine probdf
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numbpn, k, inpelm, numnod, numelm, nesintersect, &
              numgrp, ielgrp, numnodal, nesnodal, nescav, numsurfs, numobstlc

!     ielgrp         element group number
!     inpelm         number of points per element
!     k              Counting variable
!     nescav         Number of parts where cavity bondary conditions are given
!     nesintersect   Number of intersections where essential boundary
!                    conditions are given
!     nesnodal       Number of groups of nodal points with essential
!                    boundary conditions
!     numbpn         Number of parts where point boundary elements are
!                    created
!     numelm         Number of groups of elements with essential boundary
!                    conditions
!     numgrp         Number of groups
!     numnod         Number of points
!     numnodal       Number of nodal points in group
!     numobstlc      Local number of obstacles
!     numsurfs       number of surfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PROB14ESSCURV  Raise iinp for essential boundary conditions
!                    (subroutine PROBDF) Only curve part
!     PROB14ESSSURF  Raise iinp for essential boundary conditions
!                    (subroutine PROBDF) Only surface part
!     PROB14ESSVOLM  Raise iinp for essential boundary conditions
!                    (subroutine PROBDF) Only volume part
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob14ess' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob14ess'
         write(irefwr,1) 'iinp', iinp
         call prinin ( iinput, 60, 'iinput' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Essential boundary conditions

      numbpn = iinput(6)
      do k = 1, numbpn
         iinp = iinp+2+iinput(iinp+1)
      end do  ! k = 1, numbpn
      if ( debug ) write(irefwr,1) 'iinp/1', iinp

!     --- Essential boundary conditions on curves

      call prob14esscurv ( iinput, iinp )
      if ( debug ) write(irefwr,1) 'iinp/2', iinp

!     --- Essential boundary conditions on surfaces

      call prob14esssurf ( iinput, iinp )
      if ( debug ) write(irefwr,1) 'iinp/3', iinp

!     --- Volumes

      call prob14essvolm ( iinput, iinp )
      if ( debug ) write(irefwr,1) 'iinp/4', iinp

      numnod = iinput(10)
      numelm = iinput(11)
      numgrp = iinput(12)
      if ( debug ) &
          write(irefwr,1) 'numnod, numelm, numgrp', &
                           numnod, numelm, numgrp
      do k = 1, numgrp
         ielgrp = iinput(iinp+1)
         inpelm = Topgrp(ielgrp)%inpelm
         iinp = iinp + inpelm + 2
         if ( debug ) &
          write(irefwr,1) 'k, ielgrp, kelma, inpelm', &
                           k, ielgrp, kelma, inpelm
      end do
      if ( debug ) write(irefwr,1) 'iinp/5 ', iinp
      numgrp = iinput(19)+iinput(22)

      numobstlc = iinput(16)
      iinp = iinp+numnod*3+numelm*4+numobstlc*3+numgrp*3
      if ( debug ) write(irefwr,1) 'iinp/6', iinp

!     --- Obstacles

      iinp = iinp+2* &
         (iinput(45)+iinput(41)+iinput(42)+iinput(43)+iinput(46))
      if ( debug ) write(irefwr,1) 'iinp/7', iinp

!     --- Level set

      iinp = iinp+2*iinput(47)
      if ( debug ) write(irefwr,1) 'iinp/8', iinp

!     --- Nodal points

      nesnodal = iinput(28)  ! number of part with nodal points given
      if ( debug )  write(irefwr,1) 'nesnodal', nesnodal
      do k = 1, nesnodal
         numnodal = iinput(iinp+1)  ! number of nodes
         if ( debug )  write(irefwr,1) 'k, numnodal', k, numnodal
         if ( numnodal>0 ) then
            iinp = iinp+2+numnodal
         else
            iinp = iinp+3   ! stored in file
         end if  ! ( numnodal>0 )
      end do  ! k = 1, nesnodal

      nescav = iinput(49)
      iinp = iinp+2*nescav

      if ( debug ) write(irefwr,1) 'iinp/9', iinp

!     --- intersection

      nesintersect = iinput(56) ! number of parts with intersections
      if ( debug )  write(irefwr,1) 'nesintersect', nesintersect
      do k = 1, nesintersect
         numsurfs = iinput(iinp+3)  ! number of surfaces
         if ( debug )  write(irefwr,1) 'k, numsurfs', k, numsurfs
         iinp = iinp+4+numsurfs
      end do  ! k = 1, nesintersect

      if ( debug ) write(irefwr,1) 'iinp/10', iinp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'End prob14ess'

      end if
      call erclos ( 'prob14ess' )

      end
