      subroutine prob15 ( ikprbi, iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  6.0  date 27-09-2010 Make work arrays local
!        version  5.0  date 14-10-2002 Extra parameter kmeshq
!        version  4.3  date 15-10-1999 New call to er0026
!
!   copyright (c) 1987-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   fill array kprob part i with boundary elements corresponding to
!   boundary conditions of the type u equals unknown constant
!   This subroutine is the counterpart of subroutine PROB12, &
!   if PROB12 is changed also PROB15 should be changed and vice versa
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_element
!     fill
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ikprbi, iinp

!     iinp          i/o   last position used in array iinputkp
!     ikprbi        i/o   last position used in array kprob part i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer numcrv, i, idegfd, icount, kcount, k, &
              jstep, ifirst, ilast, j, n, kstep, inpelm, &
              numsrf, isurf, istart
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     icount         Counter
!     idegfd         Degree of freedom
!     ifirst         First curve (surface) number read
!     ilast          Last curve (surface) number read
!     inpelm         number of points per element
!     istart         First position used at a part
!     isurf          Type of surface with respect to mesh generator
!     iwork          work array to store the nodes in the curves
!     j              Counting variable
!     jstep          Step in loop
!     k              Counting variable
!     kcount         Counter
!     kstep          Step in loop
!     n              number of points on the curves icrv1 to icrv2
!     numcrv         number of parts where curves are given for generation
!                    of boundary elements
!     numsrf         number of parts where surfaces are given for generation
!                    of boundary elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRININ1        print 2d integer array
!     SEPER0026      fill nodal points of curves icrv1 to icrv2 in array iwork
!     SEPER0084      fill nodal points of curves icrv1 to icrv2 in array iwork
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     For i := 1 (1) numcrv
!        fill nodal points of curves ifirst to ilast in iwork
!        if ( jstep<0 ) then
!           delete non-interesting points from iwork
!        else ( jstep>1 )
!           delete non-interesting points from iwork
!        Fill kprob part i
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob15' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob15'
  1      format ( a, 1x, (10i6) )

      end if

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

      numcrv = iinputkp(iinp)
      numsrf = iinputkp(iinp+1)
      iinp = iinp+2
      do i = 1, numcrv
         idegfd = iinputkp(iinp)
         kprobo(i) = idegfd
         jstep = iinputkp(iinp+1)
         ifirst = iinputkp(iinp+2)
         ilast = iinputkp(iinp+3)
         iinp  =  iinp + 4
         if ( debug ) &
            write(irefwr,*) 'i, idegfd, jstep, ifirst, ilast ', &
                             i, idegfd, jstep, ifirst, ilast

!        --- fill nodal points of curves ifirst to ilast in iwork

         if ( jstep<=0 ) then
            j = 1
         else
            j = jstep
         end if
         if ( kelmm>0 ) then
            call seper0026 ( ifirst, ilast, j, n, iwork, .true., kmeshm )
         else
            call seper0084 ( ifirst, ilast, j, n, iwork )
         end if
         if ( debug ) call prinin ( iwork, n, 'iwork' )

         if ( jstep<0 ) then

!        --- jstep<0  delete non-interesting points from iwork

            kstep = abs(jstep)
            inelem = (n-1)/kstep
            icount = 0
            kcount = 0
            do j = 1, inelem
               kcount = kcount+1
               do k = 2, kstep
                  icount = icount+1
                  kcount = kcount+1
                  iwork(icount) = iwork(kcount)
               end do
            end do
            n = icount
            inelem = n-1

         else if ( jstep>1 ) then

!        --- jstep>1 delete non-interesting points from iwork

            do j = 1, inelem
               iwork(j+1) = iwork(1+j*jstep)
            end do
            n = inelem+1

         end if

!        --- Fill kprob part i

         do j = 1, inelem
            kprobi(ikprbi+2*j-1) = iwork(j)
            kprobi(ikprbi+2*j) = iwork(j+1)
         end do
         if ( debug ) &
            call prinin1 ( kprobi(ikprbi+1), inelem, 2, 'kprobi' )
         ikprbi = ikprbi+2*inelem

      end do

      do i = 1, numsrf
         idegfd = iinputkp(iinp)
         ifirst = iinputkp(iinp+1)
         ilast  = iinputkp(iinp+2)
         inpelm = iinputkp(iinp+3)
         kprobo(i) = idegfd
         iinp  =  iinp + 4 + inpelm

!        --- fill nodal points of surfaces ifirst to ilast in KPROB part i

         if ( kelmn>0 ) then

!        --- kelmn>0, kmesh part n has been filled

            istart = 2*nsurfs
            do isurf = 1, ifirst-1
               istart = istart + kmeshn(isurf)*kmeshn(nsurfs+isurf)
            end do
            do isurf = ifirst, ilast
               inelem = kmeshn(nsurfs+isurf)
               do j = 1, inelem*inpelm
                  kprobi(ikprbi+j) = kmeshn(istart+j)
               end do
               istart = istart+inelem*inpelm
               ikprbi = ikprbi+inelem*inpelm
            end do

         end if

      end do

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'indpro', kprobloc(28)
         call prinin ( kprobo, nbcons, 'kprobo' )
         write(irefwr,*) 'End prob15'

      end if
      call erclos ( 'prob15' )

      end
