      subroutine prob17 ( ikprbi, iinp, iposgrp, numobstlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 14-10-2002 Extra parameter kmeshq
!        version  3.1  date 14-08-1999 New definition icurvsobst2d
!        version  3.0  date 06-05-1998 Remove error message, ibngrp
!                                      Extra parameter iposgrp
!
!   copyright (c) 1996-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute boundary elements corresponding to the obstacles
!   Fill in kprob
!   This subroutine is the counterpart of subroutine PROB18, &
!   if PROB18 is changed also PROB17 should be changed and vice versa
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     boundary_element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ikprbi, iinp, iposgrp(*), numobstlc

!     iinp          i/o   last position used in array iinputkp
!     ikprbi         i    last position used in array kprob part i
!     iposgrp       i/o   work array of length numnat in which the starting
!                         addresses of the element groups in KPROB part i are
!                         stored
!                         In this subroutine the length with respect to an
!                         element group is added to the existing array
!     kprobloc       i    Standard SEPRAN array, containing information of
!                         the problem
!     numobstlc      i    Local number of obstacles
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ibngrp, nsubcv, maxobst, &
              inpelm, i1, i2, n, k, iobst, icurnr

!     i              Counting variable
!     i1             First point of curve in icurvsobst2d
!     i2             Last point of curve in icurvsobst2d
!     ibngrp         Boundary element group number read
!     icurnr         Curve sequence number
!     inpelm         number of points per element
!     iobst          Obstacle sequence number
!     k              Counting variable
!     maxobst        maximum number of active obstacles
!     n              number of basis functions for each component
!     nsubcv         Number of subcurves of a curve
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1004   Composite curves have not yet been implemented for this subroutine
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initialize parameters
!    For all obstacles do
!       Find boundary group sequence number
!       Find shape number
!       n := 0
!       For all elements along the obstacle
!          If ( all nodes at element are active ) then
!             n := n+1
!       Store n
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob17' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob17'
         write(irefwr,*) 'numobst ', numobstlc
         call prinin ( iobscurv, ncurvs, 'iobscurv' )

      end if
      maxobst = numobstlc

!     --- Loop over obstacles

      do i = 1, min(numobstlc,maxobst)

!     --- Find boundary group sequence number

         ibngrp = iinputkp(iinp+1)

!        --- Find shape number

         ishape = iinputkp(iinp+2)
         inpelm = ishape+1

         iobst = iinputkp(iinp+3)
         iinp = iinp+3
         n = 0

         if ( debug ) &
            write(irefwr,*) 'i, ibngrp, ishape, inpelm, iobst ', &
                             i, ibngrp, ishape, inpelm, iobst

         do icurnr = 1, ncurvs

            if ( iobscurv(icurnr)==1 ) then

!           --- Curve icurnr contains at least two nodes at the obstacle

               if ( icurnr==1 ) then
                  i1 = 1
               else
                  i1 = icurvsmsh(icurnr-1)+1
               end if
               i2 = icurvsmsh(icurnr)
               nsubcv = 0
               if ( kelmq>0 ) nsubcv = kmeshq(icurnr+1)-kmeshq(icurnr)
               if ( nsubcv>1 ) then

!              --- Number of subcurves > 1 not yet implemented

                  call errsub ( 1004, 0, 0, 0 )
                  go to 1000

               end if
               if ( debug ) &
                  write(irefwr,*) 'icurnr, i1, i2 ',icurnr, i1, i2

!              --- Find the active nodes and elements along the obstacle
!                  Create boundary elements for these nodes

               if ( ishape==1 ) then

!              --- linear elements

                  do k = i1, i2-1
                     if ( icurvsobst2d(1,k)==iobst .and. &
                          icurvsobst2d(1,k+1)==iobst ) then
                        n = n+1
                     end if
                  end do
                  if ( debug ) write(irefwr,*) 'n ', n

               else

!              --- quadratic elements

                  do k = i1, i2-1, 2
                     if ( icurvsobst2d(1,k)==iobst .and. &
                          icurvsobst2d(1,k+1)==iobst .and. &
                          icurvsobst2d(1,k+2)==iobst ) then
                        n = n+1
                     end if
                  end do
                  if ( debug ) write(irefwr,*) 'n ', n

               end if

            end if

         end do

         kprobb(ibngrp) = kprobb(ibngrp)+n
         ikprbi = ikprbi+n*inpelm
         iposgrp(ibngrp) = iposgrp(ibngrp)+n*inpelm
         if ( debug ) &
            write(irefwr,*) 'ishape, n, inpelm ', ishape, n, inpelm

      end do

1000  call erclos ( 'prob17' )

      end
