      subroutine prob18 ( numobstlc, iinp, iposgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.1  date 14-08-1999 New definition icurvsobst2d
!        version  3.0  date 05-05-1998 Use array iposgrp
!        version  2.0  date 20-12-1997 New definition of obstacles
!
!   copyright (c) 1996-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    fill array kprob part i with boundary elements corresponding to
!    obstacles
!    This subroutine is the counterpart of subroutine PROB17, &
!    if PROB17 is changed also PROB18 should be changed and vice versa
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numobstlc, iinp, iposgrp(*)

!     iinp      i/o   last position used in array iinputkp
!     iposgrp   i/o   work array of length numnat in which the starting
!                     addresses of the element groups in KPROB part i are
!                     stored
!                     At input the last postions used for an element group
!                     are stored, at output they are updated
!     numobstlc  i    number of obstacles where boundary elements must be
!                     created
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istar3, i, i1, i2, k, iobst, icurnr, ibngrp, &
              ikprbi

!     i          Counting variable
!     i1         First point of curve in icurvsobst2d
!     i2         Last point of curve in icurvsobst2d
!     ibngrp     boundary element group number
!     icurnr     Curve sequence number
!     ikprbi     Last position used in kprobi for this group
!     iobst      Obstacle sequence number
!     istar3     Starting address of KMESHM part 3 in KMESH part M
!     k          Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      istar3 = kmeshm(2)-1
      do i = 1, min(numobstlc,numobst)
         ibngrp = iinputkp(iinp+1)
         ikprbi = iposgrp(ibngrp)
         ishape = iinputkp(iinp+2)
         if ( ishape==0 ) then
            iinp = iinp+3
            goto 200
         end if
         iobst = iinputkp(iinp+3)
         iinp = iinp+3

         do icurnr = 1, ncurvs

            if ( iobscurv(icurnr)==1 ) then

!           --- Curve icurnr contains at least two nodes at the obstacle

               if ( icurnr==1 ) then
                  i1 = 1
               else
                  i1 = icurvsmsh(icurnr-1)+1
               end if
               i2 = icurvsmsh(icurnr)

!              --- Find the active nodes and elements along the obstacle
!                  Create boundary elements for these nodes

               if ( ishape==1 ) then

!              --- linear elements

                  do k = i1, i2-1
                     if ( icurvsobst2d(1,k)==iobst .and. &
                        icurvsobst2d(1,k+1)==iobst ) then
                        kprobi(ikprbi+1) = kmeshm(istar3+k)
                        kprobi(ikprbi+2) = kmeshm(istar3+k+1)
                        ikprbi = ikprbi+2
                     end if
                  end do

               else

!              --- quadratic elements

                  do k = i1, i2-1, 2
                     if ( icurvsobst2d(1,k)==iobst .and. &
                          icurvsobst2d(1,k+1)==iobst .and. &
                          icurvsobst2d(1,k+2)==iobst ) then
                        kprobi(ikprbi+1) = kmeshm(istar3+k)
                        kprobi(ikprbi+2) = kmeshm(istar3+k+1)
                        kprobi(ikprbi+3) = kmeshm(istar3+k+2)
                        ikprbi = ikprbi+3
                     end if
                  end do

               end if
               iposgrp(ibngrp) = ikprbi

            end if

         end do

200   end do

      end
