      subroutine prob31 ( natnocpt, iinp, iposgrp )
! ======================================================================
!
!        programmer     Guus Segal
!        version 2.0    date   05-05-1998 Use array iposgrp
!        version 1.0    date   12-12-1997
!
!
!
!   copyright (c) 1997-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
! **********************************************************************
!
!                       DESCRIPTION
!
!    fill array kprob part i with boundary elements
!    corresponding to no_contact surfaces
!     surface
!    This subroutine is the counterpart of subroutine PROB29, &
!    if PROB29 is changed also prob31 should be changed and vice versa
! **********************************************************************
!
!                       KEYWORDS
!
!   problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer natnocpt,  iinp, iposgrp(*)

!     iinp      i/o   last position used in array iinputkp
!     iposgrp   i/o   work array of length numnat in which the starting
!                     addresses of the element groups in KPROB part i are
!                     stored
!                     At input the last postions used for an element group
!                     are stored, at output they are updated
!     natnocpt   i    Number of no_contacts where natural point boundary
!                     conditions are given
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, iconct, istart, nconctct, j, numconct, &
              nocontct, ibngrp, ikprbi

!     i          Counting variable
!     ibngrp     boundary element group number
!     iconct     Sequence number of contact
!     ikprbi     Last position used in kprobi for this group
!     istart     First position used at a part
!     j          Counting variable
!     nconctct   Number of points with contact
!     nocontct   Number of points that make no contact
!     numconct   Maximum number of contacts in problem
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Initializations
!    For all natural boundary contact elements do
!       iblgrp = boundary element group
!       iconct = contact sequence number
!       isurnr := contact surface
!       inpelm := number of nodes per element
!       nconctct := number of contact elements
!       nocontct = number of no_contact points
!       for j := 1 (1) nocontct do
!          Copy elements to kprobi
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
!     --- Initializations

      numconct = kprobloc(48)

!     --- For all natural boundary contact elements do

      do i = 1, natnocpt
         ibngrp = iinputkp(iinp+1)
         ikprbi = iposgrp(ibngrp)
         iconct = iinputkp(iinp+3)
         iinp = iinp+3
         if ( iconct<=numconct ) then

!        --- For all surfaces do

            istart = kprobvp1(iconct,3)
            nconctct = kprobvp1(iconct,1)
            nocontct = kprobvp1(iconct,2)
            if ( nocontct>0 ) then

!           --- Contact elements have been found

               istart = istart-1+nconctct

!              --- Copy elements to kprobi

               do j = 1, nocontct
                  kprobi(ikprbi+j) = kprobvp2(istart+j)
               end do
               ikprbi = ikprbi+nocontct

            end if

         end if
         iposgrp(ibngrp) = ikprbi

      end do

      end
