      subroutine prob33 ( numsrf, ikprbi, iinp, iposgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-03-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute length of array kprob part i with point boundary elements
!     corresponding to surfaces
!     This subroutine is the counterpart of subroutine PROB35, &
!     if prob33 is changed also PROB35 should be changed and vice versa
!     New input
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numsrf, ikprbi, iinp, iposgrp(*)

!     iinp       i    last position used in array iinputkp
!     ikprbi    i/o   last position used in array kprob part i
!     iposgrp   i/o   work array of length numnat in which the starting
!                     addresses of the element groups in KPROB part i are
!                     stored
!                     In this subroutine the length with respect to an
!                     element group is added to the existing array
!     numsrf     i    number of parts where surfaces are given for generation
!                     of boundary elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ibngrp, isurnr, inelembn, k, &
              inpelm, j, istart, inum, ipoint
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     ibngrp         Actual boundary element group number
!     inelembn       Number of boundary elements in a specific element group
!     inpelm         number of points per element
!     inum           Number of surfaces
!     ipoint         nodal point sequence number
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     iwork          integer work array of length npoint
!                    iwork is used to indicate the uniqueness of the nodes
!                    iwork(i) = 0, means node i not yet added to the list
!                    iwork(i)>0, means node i has been added to the list
!     j              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INIINFSURF     Find starting address of surface isurnr in array kmeshn
!                    Also find number of element and number of points per
!                    element in isurnr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob33' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob33'

      end if

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      iwork = 0

      do i = 1, numsrf

         ibngrp = iinputkp(iinp+1)
         inum = iinputkp(iinp+2)
         inelembn = 0

         do j = 1, inum

            isurnr = iinputkp(iinp+2+i)

!           --- Find information of surface in kmeshn

            call iniinfsurf ( kmeshn, istart, inpelm, inelem, nsurfs, &
                              isurnr )

            do k = 1, inelem*inpelm
               ipoint = kmeshn(istart+k)
               if ( iwork(ipoint)==0 ) then

!              --- New point found

                  iwork(ipoint)=1
                  inelembn = inelembn+1

               end if
            end do  ! k = 1, inelem*inpelm

         end do  ! j = 1, inum
         kprobb(ibngrp) = kprobb(ibngrp)+inelembn
         ikprbi = ikprbi+inelembn
         iposgrp(ibngrp) = iposgrp(ibngrp)+inelembn

      end do

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob33'

      end if
      call erclos ( 'prob33' )

      end
