      subroutine prob35 ( numsrf, iinp, iposgrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-03-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill array kprob part i with boundary elements corresponding to
!     surface
!     This subroutine is the counterpart of subroutine PROB11, &
!     if PROB11 is changed also PROB35 should be changed and vice versa
!     New input
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer numsrf, iinp, iposgrp(*)

!     iinp      i/o   last position used in array iinputkp
!     iposgrp   i/o   work array of length numnat in which the starting
!                     addresses of the element groups in KPROB part i are
!                     stored
!                     At input the last postions used for an element group
!                     are stored, at output they are updated
!     numsrf     i    number of parts where surfaces are given for generation
!                     of boundary elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, inum, k, inpelm, j, istart, &
              ibngrp, ikprbi, inelembn, ipoint, isurnr
      integer, allocatable :: iwork(:)

!     i              Counting variable
!     ibngrp         boundary element group number
!     ikprbi         Last position used in kprobi for this group
!     inelembn       Number of boundary elements in a specific element group
!     inpelm         number of points per element
!     inum           Number of surfaces
!     ipoint         nodal point sequence number
!     istart         First position used at a part
!     isurnr         Surface sequence number
!     iwork          Integer work array of length npoint
!                    iwork is used to indicate the uniqueness of the nodes
!                    iwork(i) = 0, means node i not yet added to the list
!                    iwork(i)>0, means node i has been added to the list
!     j              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     SEPIINFSURF    Find starting address of surface isurnr in array kmeshn
!                    Also find number of element and number of points per
!                    element in isurnr
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prob35' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prob35'

      end if

      allocate ( iwork(npoint), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint, 'iwork' )
      if ( ierror/=0 ) go to 1000

!     --- Clear array iwork

      iwork = 0

      do i = 1, numsrf
         ibngrp = iinputkp(iinp+1)
         ikprbi = iposgrp(ibngrp)
         inum = iinputkp(iinp+2)
         inelembn = 0

         do j = 1, inum

            isurnr = iinputkp(iinp+2+i)

!           --- Find information of surface in kmeshn

            call sepiinfsurf ( istart, inpelm, isurnr )
            do k = 1, inelem*inpelm
               ipoint = kmeshn(istart+k)
               if ( iwork(ipoint)==0 ) then

!              --- New point found

                  iwork(ipoint)=1
                  inelembn = inelembn+1
                  kprobi(ikprbi+inelembn) = ipoint

               end if
            end do  ! k = 1, inelem*inpelm
            istart = istart+inelem*inpelm
         end do  ! j = 1, inum
         ikprbi = ikprbi+inelembn
         iposgrp(ibngrp) = iposgrp(ibngrp)+inelembn

      end do  ! i = 1, numsrf

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prob35'

      end if
      call erclos ( 'prob35' )

      end
