      subroutine probextend ( number )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 03-12-2013 Correction part e not the last one
!        version  1.0  date 04-03-2011
!
!   copyright (c) 2011-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Extend problem definition with vectors of special structure
!     of exactly number degrees of freedom
! **********************************************************************
!
!                       KEYWORDS
!
!     structure
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer number

!     number         i    The required number of unknowns per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: istart, iend, istart1, ishift, ih1, i
      double precision dummy(1)

!     dummy          Dummy parameter
!     i              Counting variable
!     iend           Last position used at a part
!     ih1            First position used at a part
!     ishift         Shift of specific variable with respect to diagonal
!     istart         First position used at a part
!     istart1        First position used at a part
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probextend' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probextend'
         call ertrace
         write(irefwr,1) 'number', number
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iseqkp = kprob
      kprobpart => kp(iseqkp)%kprobsub(1)
      kprobloc => kprobpart%kprobloc
      if ( debug ) call prinin ( kprobloc, kprobloc(3), 'kprobloc' )

!     --- get some parameters

      ndtvec = kprobloc(6)
      indpre = kprobloc(18)
      kprelgrp = kprobloc(39)

      if ( debug ) &
         write(irefwr,1) 'ndtvec, nprob, indpre, kprelgrp', &
                          ndtvec, nprob, indpre, kprelgrp

      if ( indpre==0 ) then

!     --- Simple case, we put kprob part e at the back of kprob
!         Consider first problem

         ndtvec = 1
         allocate ( kp(iseqkp)%kprobsub(iprob)%kprobe(kprelgrp,ndtvec), &
                    kp(iseqkp)%kprobsub(iprob)%kprobe2(ndtvec), &
                    kp(iseqkp)%kprobsub(iprob)%kprobe3(ndtvec) )
         kprobe => kprobpart%kprobe
         kprobe2 => kprobpart%kprobe2
         kprobe3 => kprobpart%kprobe3
         nullify(kp(iseqkp)%kprobsub(iprob)%kprobe3(1)%arr)
         kprobe = 0  ! not important
         kprobe2 = -number     ! essential

      else

!     --- Complicated case, kprob part e already exists
!         Extend ndtvec by 1 and create new part e

         allocate ( kprobe(kprelgrp,ndtvec+1), kprobe2(ndtvec+1), &
                    kprobe3(ndtvec+1) )
         kprobe(:,1:ndtvec) = kp(iseqkp)%kprobsub(iprob)%kprobe
         kprobe2(1:ndtvec) = kp(iseqkp)%kprobsub(iprob)%kprobe2
         do i = 1, ndtvec
            nullify(kprobe3(i)%arr)
            if ( associated(kp(iseqkp)%kprobsub(iprob)%kprobe3(i)%arr) ) then
               length = size(kp(iseqkp)%kprobsub(iprob)%kprobe3(i)%arr)
               allocate ( kprobe3(i)%arr(length) )
               kprobe3(i)%arr = kp(iseqkp)%kprobsub(iprob)%kprobe3(i)%arr
               deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobe3(i)%arr )
            end if  ! ( associated(kp(iseqkp)%kprobsub(iprob)%kprobe3(i)%arr) )
         end do  ! i = 1, ndtvec
         deallocate ( kp(iseqkp)%kprobsub(iprob)%kprobe, &
                      kp(iseqkp)%kprobsub(iprob)%kprobe2 )
         ndtvec = ndtvec+1
         kprobe(:,ndtvec) = 0
         kprobe2(ndtvec) = -number
         nullify(kprobe3(ndtvec)%arr)
         allocate ( kp(iseqkp)%kprobsub(iprob)%kprobe(kprelgrp,ndtvec), &
                    kp(iseqkp)%kprobsub(iprob)%kprobe2(ndtvec), &
                    kp(iseqkp)%kprobsub(iprob)%kprobe3(ndtvec) )
         do i = 1, ndtvec
            nullify(kp(iseqkp)%kprobsub(iprob)%kprobe3(i)%arr)
            if ( associated(kprobe3(i)%arr) ) then
               allocate ( kp(iseqkp)%kprobsub(iprob)%kprobe3(i)%arr(length) )
               kp(iseqkp)%kprobsub(iprob)%kprobe3(i)%arr = kprobe3(i)%arr
               deallocate ( kprobe3(i)%arr )
            end if  ! ( associated(kprobe3(i))%arr )
         end do  ! i = 1, ndtvec
         kp(iseqkp)%kprobsub(iprob)%kprobe = kprobe
         kp(iseqkp)%kprobsub(iprob)%kprobe2 = kprobe2
         deallocate ( kprobe, kprobe2, kprobe3 )
         kprobe => kprobpart%kprobe
         kprobe2 => kprobpart%kprobe2
         kprobe3 => kprobpart%kprobe3

      end if  ! ( indpre==0 )
      indpre = ndtvec
      kprobloc(6) = ndtvec
      kprobloc(18) = ndtvec

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kprobloc, kprobloc(3), 'kprob' )
         write(irefwr,*) 'End probextend'

      end if  ! ( debug )
      call erclos ( 'probextend' )

      end
