      subroutine probfict02 ( iwork, iwork1, iwork2, newelm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.6  date 19-05-2003 Small 2D correction (RvL)
!        version  2.5  date 14-03-2003 Corrections for nvertices 3D (RvL)
!        version  2.4  date 25-11-2002 Reorganization (JdH)
!
!   copyright (c) 2000-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Find fluid elements in which the fictitious unknowns are stored
!    R^2 and R^3
! **********************************************************************
!
!                       KEYWORDS
!
!     fictitious_domain
!     2d
!     3d
!     fluid_element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iwork(*), iwork1(*), iwork2(*)
      logical newelm

!     iwork               integer work array of length npoint
!                         Is used by prfindelm2
!     iwork1              integer work array of length nelem
!                         Is used by prfindelm2
!     iwork2              integer work array of length nelem
!                         Is used by prfindelm2
!     newelm         o    Parameter indicating if different fluid elements are
!                         found that contain lambda points. Hence, a new call
!                         to commat should be made.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxlambda
      parameter (maxlambda=9)
      integer i1, i2, icurnr, icurve1, icurve2, igrp, iposkproby, &
              iregionlam, ishape_curve, istructgrp, &
              nfictpnt, i, ielemlow, ielemtop, &
              ifluidgrp1, ifluidgrp2, ikelmc, isplit, &
              j, istar3, jkelmc, jnpelm, lstep, &
              nvertices, nodp1, inunkp_elm, inpelmsurf, &
              istart, isurnr, ishapelam, inunkpgr
      double precision philam(81), w(81), xgauss(maxlambda*3)

!     i              Counting variable
!     i1             First point of curve in KMESH part m
!     i2             Last point of curve in KMESH part m
!     icurnr         Curve sequence number
!     icurve1        First curve number of a set
!     icurve2        Last curve number of a set
!     ielemlow       Smallest element number in group
!     ielemtop       Largest element number in group
!     ifluidgrp1     First fluid element group number
!     ifluidgrp2     Last fluid element group number
!     igrp           Boundary element group number read
!     ikelmc         position in kmesh part c
!     inpelmsurf     number of points per surface element
!     inunkp_elm     Number of unknowns per point
!     inunkpgr       Number of global unknowns in a group
!     iposkproby     Position in kprob y
!     iregionlam     type of region where the lambda's are defined:
!                     1:  curves
!                     2:  surfaces
!     ishape_curve   Shape of lambda basis functions
!     ishapelam      Shape of lambda basis functions
!     isplit         Sequence number of discontinuous part
!     istar3         Starting address of KMESHM part 3 in KMESH part M
!     istart         First position used at a part
!     istructgrp     Structural element group
!     isurnr         Surface sequence number
!     j              Counting variable
!     jkelmc         position in kmesh part c
!     jnpelm         Number of nodes in an element
!     lstep          Step in loop
!     maxlambda      Maximum number of lambda's allowed per element
!     nfictpnt       Total number of points with fictitious unknowns
!     nodp1          Node sequence number
!     nvertices      Number of vertices
!     philam         dummy
!     w              dummy
!     xgauss         array of length m x ndim containing the co-ordinates
!                    of the gauss points.  array xgauss is only filled when
!                    gauss integration is applied.
!                    xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                    zg_i = xgauss (i,3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELMFICTLAM     Compute the integration points and basis functions for the
!                    Lagrangian multipliers
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRFINDELM2     Find element number in which point with coordinates (x,y)
!                    is lying (R^2)
!     PRFINDELM3     Find element number in which point with coordinates (x,y,z)
!                    is lying (R^3)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
!    2149   Fluid elements do not all belong to the same set of elements
!    2152   number of elements in fluid groups not the same
! **********************************************************************
!
!                       PSEUDO CODE
!
!    nfictgrp := kproby(1)
!    nfictunk := 0
!    For igrp := 1 (1) nfictgrp do
!       ifluidgrp := fluid group corresponding to igrp
!       icurve1 := first curve number for this group
!       icurve2 := last curve number for this group
!       iregionlam := type of region where the lambda's are defined
!
!       if ( region is of type curve ) then
!          ishape := shape number of structural element
!          if ( ishape = 3 or 5 ) then
!             ishape_curve := 1
!          else if ( ishape = 4 or 6 ) then
!             ishape_curve := 2
!          else
!             error: not yet implemented
!          nelem := number of elements along the curves
!          Store relative coordinates of lambda points per element
!          for icurnr := icurve1 (1) icurve2 do
!             For all elements ielem along curve icurnr do
!                For all lambda points in element do
!                   find coordinates of point
!                   find fluid element corresponding to point
!                   Store fluid element in kproby
!       else if ( region is of type surface ) then
!          ishape := shape number of structural element
!          if ( ishape = 11 ) then
!             ishape_curve := 3
!          else if ( ishape = 12 ) then
!             ishape_curve := 4
!          else if ( ishape = 13 ) then
!             ishape_curve := 5
!          else if ( ishape = 14 ) then
!             ishape_curve := 6
!          else
!             error: not yet implemented
!          Store relative coordinates of lambda points per element
!          nelem := number of elements along the surfaces
!          for isurnr := icurve1 (1) icurve2 do
!             For all elements ielem along surface number isurnr
!                For all lambda points in element do
!                   find coordinates of point
!                   find fluid element corresponding to point
!                   Store fluid element in kproby
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probfict02' )
!     --- Make some initializations

      nfictgrp = kproby(1)
      nfictpnt = 0
      isplit   = 1
      newelm = .false.
      if ( ndim==2 ) then

!     --- ndim = 2

         istar3 = kmeshm(2)-1

      end if

!     --- Loop over fictitious groups

      do igrp = 1, nfictgrp

!     --- Set some relevant parameters

         iposkproby = kproby(10+igrp)-1
         inunkpgr   = kproby(iposkproby+2)
         iregionlam = kproby(iposkproby+3)
         ishapelam  = kproby(iposkproby+4)
         icurve1    = kproby(iposkproby+5)
         icurve2    = kproby(iposkproby+6)
         istructgrp = kproby(iposkproby+7)
         ifluidgrp1 = kproby(iposkproby+8)
         ifluidgrp2 = kproby(iposkproby+9)
         inunkp_elm = kproby(iposkproby+10)
         ielemlow = 0
         ikelmc = 0
         do i = 1, ifluidgrp1-1
            inpelm = Topgrp(i)%inpelm
            inelem = Topgrp(i)%inelem
            ikelmc = ikelmc+inelem*inpelm
            ielemlow = inelem+ielemlow
         end do
         ielemtop = ielemlow + kmeshg(ifluidgrp2)
         ielemlow = ielemlow+1
         jkelmc = ikelmc
         inpelm = Topgrp(ifluidgrp1)%inpelm
         if ( nsplit>1 ) then

!        --- The region consists of disconnected elements
!            Check to which part the fluid elements belong

            nodp1 = kmeshc(ikelmc+1)
            do i = 1, nsplit
               if ( kmeshk(nlevel+i)>=nodp1 ) then
                  isplit = i
                  go to 100
               end if
            end do
100         if ( ifluidgrp2>ifluidgrp1 ) then

!           --- ifluidgrp2 > ifluidgrp1, check if both groups belong
!               to the same part of the mesh

               do i = ifluidgrp1, ifluidgrp2-1
                  inpelm = Topgrp(i)%inpelm
                  if ( i==ifluidgrp1 ) then
                     jnpelm = inpelm
                  else if ( jnpelm/=inpelm ) then

!                 --- Not a constant number of nodes in the fluid groups

                     call errint ( i, 1 )
                     call errint ( jnpelm, 2 )
                     call errint ( ifluidgrp1, 3 )
                     call errint ( inpelm, 4 )
                     call errsub ( 2152, 4, 0, 0 )

                  end if
                  inelem = kmeshg(i)
                  jkelmc = jkelmc+inelem*inpelm
               end do
            else if ( kelma>0 ) then

!           --- Compute inpelm

               inpelm = Topgrp(ifluidgrp1)%inpelm

            end if
            ishape = kmeshf(ifluidgrp1)
            nodp1 = kmeshc(jkelmc+1)
            if ( kmeshk(nlevel+isplit)<nodp1 ) then

!           --- ifluidgrp2 does not belong to the same part

               call errint ( ifluidgrp1, 1 )
               call errint ( isplit, 2 )
               call errint ( ifluidgrp2, 3 )
               call errsub ( 2149, 3, 0, 0 )

            end if

         end if

         if ( iregionlam==1 ) then

!        --- region is of type curve
!            ishape := shape number of fluid element

            ishape = kmeshf(ifluidgrp1)
            if ( ishape==3 .or. ishape==5 .or. ishape==9 .or. &
                 ishape==10 ) then

!           --- linear elements

               lstep = 1

            else if ( ishape==4 .or. ishape==6 .or. ishape==7 ) then

!           --- quadratic elements

               lstep = 2

            else

!           --- Shape number not available

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )
               goto 1000

            end if
            if ( ishape<=4 .or. ishape==7  .or. ishape==10 ) then

!           --- triangles

               nvertices = 3

            else

!           --- quadrilaterals

               nvertices = 4

            end if

!           --- ishape := shape number of structural element

            ishape = kmeshf(istructgrp)
            if ( ishape==3 .or. ishape==5 ) then

!           --- ishape = 3 or 5

               ishape_curve = 1

            else if ( ishape==4 .or. ishape==6 ) then

!           --- ishape = 4 or 6

               ishape_curve = 2

            else

!           --- error: shape number not yet implemented

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )
               go to 1000

            end if

!           --- loop over curves

            ifirst = 0
            do icurnr = icurve1, icurve2

               i1 = kmeshm(4+icurnr)
               i2 = kmeshm(5+icurnr)-1

!              --- Find fluid elements corresponding to lambda points

               do i = i1, i2-1, ishape_curve

                  call elmfictlam ( coor, kmeshm(istar3+i), &
                                    ishape_curve, ndim, xgauss, philam, &
                                    w, ishapelam, .false.  )

                  if ( ierror/=0 ) go to 1000

                  do j = 1, ishapelam

!                 --- Find corresponding fluid element

                     call prfindelm2 ( xgauss, j, kmeshc(ikelmc+1), &
                                       iwork, iwork1, iwork2, isplit, &
                                       inpelm, lstep, nvertices, ielem, &
                                       ielemlow, ielemtop )
                     if ( ierror/=0 ) go to 1000
                     nfictpnt = nfictpnt+1

                     if ( .not. newelm ) then
                        if (kprobfict(2,nfictpnt)/=ielem) then
                           newelm = .true.
                        end if
                     end if

                     kprobfict(2,nfictpnt) = ielem

                  end do

!                 --- Raise parameters

                  ifirst = 1

               end do

            end do

         else

            ishape = kmeshf(ifluidgrp1)
            if ( ishape==11 .or. ishape==12 .or. &
                 ishape==16 ) then

!           --- ishape = 11, 12 or 16

               nvertices = 4

            else if ( ishape==13 .or. ishape==14 ) then

!           --- ishape = 13 or 14

               nvertices = 8

            else

!           --- error: shape number not yet implemented

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )
               go to 1000

            end if

!           --- region is of type surface
!               ishape := shape number of structural element

            ishape = kmeshf(istructgrp)
            if ( ishape==11 ) then

!           --- ishape = 11

               ishape_curve = 3

            else if ( ishape==12 ) then

!           --- ishape = 12

               ishape_curve = 4

            else if ( ishape==13 ) then

!           --- ishape = 13

               ishape_curve = 5

            else if ( ishape==14 ) then

!           --- ishape = 14

               ishape_curve = 6

            else

!           --- error: shape number not yet implemented

               call errint ( ishape, 1 )
               call errsub ( 800, 1, 0, 0 )
               go to 1000

            end if

!           --- Find starting address in kmesh part n

            istart = 2*nsurfs+1
            do isurnr = 1, icurve1-1
               istart = istart+kmeshn(nsurfs+isurnr)*kmeshn(isurnr)
            end do

!           --- loop over surfaces

            ifirst = 0
            do isurnr = icurve1, icurve2

               inelem = kmeshn(nsurfs+isurnr)
               inpelmsurf = kmeshn(isurnr)

!              --- Find structural and fluid element containing the lambdas
!                  Loop over elements along surface

               do i = 1, inelem

!              --- Find fluid elements corresponding to lambda points

                  call elmfictlam ( coor, kmeshn(istart), &
                                    ishape_curve, ndim, xgauss, philam, &
                                    w, ishapelam, .false. )
                  if ( ierror/=0 ) go to 1000
                  istart = istart+inpelmsurf

!                 --- Loop over lambda points in element
!                     Here 3 is changed into inunkpgr

                  do j = 1, inunkp_elm/inunkpgr

!                 --- Find corresponding fluid element

                     call prfindelm3 ( xgauss, j,kmeshc(ikelmc+1), &
                                       iwork, iwork1, iwork2, isplit, &
                                       inpelm, nvertices, ielem, &
                                       ielemlow, ielemtop )
                     if ( ierror/=0 ) go to 1000
                     nfictpnt = nfictpnt+1

!                    --- check if a new element contains the lambda point

                     if ( .not. newelm ) then
                        if ( kprobfict(2,nfictpnt)/=ielem ) then

                           newelm = .true.

                        end if
                     end if

                     kprobfict(2,nfictpnt) = ielem

                  end do

!                 --- Raise parameters

                  ifirst = 1

               end do

            end do

         end if

      end do

1000  call erclos ( 'probfict02' )

      end
