      subroutine probnatblevel ( natlevset, iinp, iposgrp, ikprbi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.2  date 24-02-2012 Extension with 3d
!        version  1.1  date 11-10-2011 Extension with 1d
!        version  1.0  date 02-05-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kprob part b and iposgrp with information of elements for
!     natural boundary conditions defined on zero levelset points
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     natural
!     problem_definition
!     level_set
!     natural_boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer natlevset, iinp, iposgrp(*), ikprbi

!     iinp          i/o   last position used in array iinputkp
!     ikprbi        i/o   last position used in array kprob part i
!     iposgrp       i/o   work array of length numnat in which the starting
!                         addresses of the element groups in KPROB part i are
!                         stored
!     natlevset      i    number of parts where zero level set elements
!                         are defined
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer igrp, ibngrp, inum

!     ibngrp         Actual boundary element group number
!     igrp           Boundary element group number read
!     inum           Number of numbers read in a record
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PROBNATBLEVEL1 Fill kprob part b and iposgrp with information of elements
!                    for natural boundary conditions defined on zero levelset
!                    points (1D)
!     PROBNATBLEVEL2 Fill kprob part b and iposgrp with information of elements
!                    for natural boundary conditions defined on zero levelset
!                    points (2D)
!     PROBNATBLEVEL3 Fill kprob part b and iposgrp with information of elements
!                    for natural boundary conditions defined on zero levelset
!                    points (3D)
!     SEPFILLOBFACES Fill kmesh part ad subpart 9: iobstfaces
!     SEPFLPOINTOBST Fill array kmesh ad  part 1: ipointobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     419   NDIM incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probnatblevel' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probnatblevel'
         write(irefwr,1) 'iinp', iinp
         write(irefwr,1) 'ndim, nlevelsets', ndim, nlevelsets
         call prinin ( iinputkp(iinp+1), 6, 'iinputkp' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( nlevelsets==0 ) then

!     --- nlevelsets = 0, no elements to be created

         do igrp = 1, natlevset

            ibngrp = iinputkp(iinp+1)
            kprobb(ibngrp) = 0
            if ( debug ) &
               write(irefwr,1) 'ibngrp, iinp', ibngrp, iinp

         end do  ! igrp = 1, natlevset

      else if ( ndim==1 ) then

!     --- ndim = 1

         call sepflpointobst
         call probnatblevel1 ( natlevset, iinp, ipointobst, iposgrp, ikprbi )

      else if ( ndim==2 ) then

!     --- ndim = 2

         call sepflpointobst
         if ( kmeshad(9)==0 ) call sepfillobfaces

!        --- Actual subroutine

         call probnatblevel2 ( natlevset, iinp, iposgrp, ikprbi, obstfaces )

      else if ( ndim==3 ) then

!     --- ndim = 3

         call sepflpointobst
         if ( kmeshad(9)==0 ) call sepfillobfaces

!        --- Actual subroutine

         call probnatblevel3 ( natlevset, iinp, iposgrp, ikprbi, obstfaces )

      else

!     --- ndim = >3, not yet implemented

         call errint ( ndim, 1 )
         call errint ( 1, 2 )
         call errint ( 3, 3 )
         call errsub ( 419, 3, 0, 0 )
         go to 1000

      end if  ! ( nlevelsets==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End probnatblevel'

      end if  ! ( debug )
      call erclos ( 'probnatblevel' )

      end
