      subroutine probnatblevel1 ( natlevset, iinp, &
                                  ipointobst, iposgrp, ikprbi )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-10-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kprob part b and iposgrp with information of elements for
!     natural boundary conditions defined on zero levelset points
!     1d only
! **********************************************************************
!
!                       KEYWORDS
!
!     1d
!     boundary_condition
!     natural
!     problem_definition
!     level_set
!     natural_boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer natlevset, iinp, &
              ipointobst(*), iposgrp(*), ikprbi

!     iinp          i/o   last position used in array iinputkp
!     ikprbi        i/o   last position used in array kprob part i
!     ipointobst     i    Array containing information about the nodes in an
!                         obstacle
!                         See PG Section 24.2 for a description
!     iposgrp       i/o   work array of length numnat in which the starting
!                         addresses of the element groups in KPROB part i are
!                         stored
!     natlevset      i    number of parts where zero level set elements
!                         are defined
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer igrp, ibngrp, inum, numbndobst, istart

!     ibngrp         Actual boundary element group number
!     igrp           Boundary element group number read
!     inum           Number of numbers
!     istart         Starting address
!     numbndobst     Number of nodal points on the boundary of the obstacle
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1093   NPELM incorrect for boundary group
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probnatblevel1' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probnatblevel1'
         write(irefwr,1) 'natlevset', natlevset
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      istart = ipointobst(1)
      numbndobst = ipointobst(istart+2)
      if ( debug ) write(irefwr,1) 'numbndobst', numbndobst

!     --- Loop over set of natural boundary conditions corresponding to
!         zero level set

      do igrp = 1, natlevset

         ibngrp = iinputkp(iinp+1)         ! boundary group
         inum = max(1,iinputkp(iinp+5) )   ! number of curves
         iinp = iinp+6+inum
         if ( debug ) &
            write(irefwr,1) 'ibngrp, inum, iinp', ibngrp, inum, iinp

!        --- Raise kprob part b and fill kprob part j

         kprobb(ibngrp) = kprobb(ibngrp)+numbndobst

         if ( kprobj(ibngrp,1)==0 ) then

!        --- kprob part j has not yet been filled

            kprobj(ibngrp,1) = 1         ! number of nodes
            kprobj(ibngrp,2) = 1  ! shape number

         else if ( kprobj(ibngrp,1)/=1 ) then

!        --- error kprob part j has been filled with other value

            call errint ( kprobj(ibngrp,1), 1 )
            call errint ( ibngrp, 2 )
            call errint ( 1, 3 )
            call errsub ( 1093, 3, 0, 0 )

         end if  ! ( kprobj(ibngrp,1)==0 )

!        --- Raise ikprbi and iposgrp

         ikprbi = ikprbi+numbndobst
         iposgrp(ibngrp) = iposgrp(ibngrp)+numbndobst
         if ( debug ) write(irefwr,1) 'numbndobst', numbndobst

      end do  ! igrp = 1, natlevset

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iinp, iposgrp(ibngrp)', iinp, iposgrp(ibngrp)
         call prinin ( kprobb, kprobloc(12), 'kprob part b' )
         write(irefwr,*) 'End probnatblevel1'

      end if  ! ( debug )
      call erclos ( 'probnatblevel1' )

      end
