      subroutine probparglob11
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-11-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Send information from node to host with respect to global
!     vectors of special structure
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     parallel
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulecpack
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ivec, leniwork, icontext
      integer, pointer :: send_buffer(:)
      integer, save :: sendbuffersize

!     icontext       Tag for sending and receiving, is used as check
!     ivec           Sequence number of vector of special structure
!     leniwork       Computed length of array iwork
!     send_buffer    Buffer for sending messages
!     sendbuffersize Size of send_buffer in bytes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     INIBSENDMPI    Send information to other node (mpi buffered)
!     INIBUFATTMPI   Attach mpi buffer if it exists
!     INIBUFDETMPI   Detach mpi buffer if it exists
!     INIBUFSIZEMPI  Extend send_buffer_size with mpi overhead
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probparglob11' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probparglob11'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      kprobe2 => kprobpart%kprobe2
      kprobe3 => kprobpart%kprobe3
      leniwork = 2+npoint*2
      if ( debug ) write(irefwr,1) 'npoint, leniwork', npoint, leniwork

!     --- allocate iwork

      allocate ( iwork(leniwork), stat = error )
      if ( error/=0 ) call eralloc ( error, leniwork, 'iwork' )
      if ( ierror/=0 ) go to 1000

      iwork(1) = npoint
      iwork(2:1+npoint) = mappingpar(1:npoint)!copy mapping from local to global

!     --- Loop over all vectors of special structure

      do ivec = 1, ndtvec

         if ( kprobe2(ivec)>0 ) then

            leniwork = 1+npoint

            iwork(leniwork+1:leniwork+npoint+1) = kprobe3(ivec)%arr
            leniwork = leniwork+npoint+2

            sendbuffersize = leniwork*4  ! size in bytes

!           --- Compute actual buffer size using mpi information
!               allocate array and attach buffer

            call inibufsizempi ( sendbuffersize, 1 )

            allocate(send_buffer(sendbuffersize))

            call inibufattmpi ( send_buffer, sendbuffersize )

            if ( debug ) write(irefwr,1) 'leniwork', leniwork

!           --- Send array iwork

            icontext = 41
            call inibsendmpi ( iwork, leniwork, SPINT, 1, icontext )

            if (associated(send_buffer)) then

!           --- deattach send_buffer and deallocate

               call inibufdetmpi ( send_buffer, sendbuffersize )
               deallocate(send_buffer)

            end if  ! (associated(send_buffer))

         end if  ! ( kprobe2(ivec)>0 )

      end do  ! ivec = 1, ndtvec

      deallocate ( iwork, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'iwork' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End probparglob11'

      end if  ! ( debug )
      call erclos ( 'probparglob11' )

      end subroutine probparglob11
