      subroutine probskipelms
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 26-10-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kprob part aa with information concerning the elements to be
!     skipped
!     kprob part aa must have been created and initialized
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     element
!     skip
!     plot
!     print
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmoduledebug
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PLELEMSKIP     Plot elements that must be skipped
!     PRININNONZERO  Print the non-zero contents of an integer array
!     PROBSKIPELM    Fill kprob part aa with information concerning the elements
!                    to be skipped
!     SEPFLIELEMOBST Fill or activate array kmesh ad  part 2: ielemobst
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Initializations
!     activate arrays
!     Create kprobaa
!     Define work space
!     Compute starting addresses
!     create kprob part aa
!     if ( debug ) then
!         print kprob aa
!         plot elements to be skipped
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'probskipelms' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from probskipelms'

      end if
      if ( ierror/=0 ) go to 1000

!     --- activate arrays

      call sepflielemobst  ! Fill array ielemobst if necessary
      if ( debug ) write(irefwr,*) 'after sepfillkmad2'

!     --- create kprob part aa

      if ( debug ) write(irefwr,*) 'before probskipelm'
      call probskipelm
      if ( debug .or. printskipel ) then

!     --- debug is true, print elements to be skipped

         call prininnonzero ( kprobaa, nelem, 'kprob part aa' )

      end if

      if ( ndim==2 .and. (debug .or. plotskipel) ) then

!     --- debug is true, plot elements to be skipped (2d only)

         call plelemskip

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End probskipelms'

      end if
      call erclos ( 'probskipelms' )

      end subroutine probskipelms
