      subroutine prord0copy ( iwork, iinput, iinp, iinputlc, &
                              ntype, rinput, lengrl, rinputlc, nncode3, &
                              lengrllc, numveclc, nsear )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-10-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy array iinputlc into iinput and rinputlc into rinput
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: iinputlc(*), ntype, nncode3, &
                             numveclc, nsear
      integer, intent(inout) :: iinp, iinput(*), lengrl, iwork(*), &
                                lengrllc
      double precision, intent(inout) :: rinput(*)
      double precision, intent(in) :: rinputlc(*)

!     iinp          i/o   Last position used in IINPUT
!     iinput         o    integer input array
!                         See Programmer's guide
!     iinputlc       i    See subroutine prord0
!     iwork         i/o   See subroutine prord0
!     lengrl        i/o   Length of real array rinput
!     lengrllc      i/o   Length of array rinputlc
!     nncode3        i    Number of items in array code3
!     nsear          i    Number of specific degrees of freedom
!     ntype          i    Number of different type numbers to be used for one
!                         group
!     numveclc         i    Number of vectors of special structure
!     rinput        i/o   Real input array
!     rinputlc       i    See subroutine prord0
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ielgrp, iinplc, ielgrplast, ielgrpfirst, i, &
                 icheck, k, itypnr, iextra, icount, istart, inpelm, &
                 indtvc, inum, isear, ivec

!     i              Counting variable
!     icheck         Indication if all type numbers are smaller than 100
!                    (0) or  not (1)
!     icount         Counter
!     ielgrp         element group number
!     ielgrpfirst    first element group number
!     ielgrplast     last element group number
!     iextra         Extra space needed at both sides of the array for
!                    control integers
!     iinplc         Last position used in array iinputlc
!     indtvc         Number of vectors of special structure
!     inpelm         number of points per element
!     inum           Number of numbers read in a record
!     isear          Number of specific degrees of freedom
!     istart         First position used at a part
!     itypnr         Type number of element
!     ivec           Sequence number of vector of special structure
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRWAR         Print warnings
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1085   No information read for element group number
!    1086   No information read for vector of special structure
!    1087   No information read for physical degree of freedom
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prord0copy' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prord0copy'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ielgrp = 1
      do while ( ielgrp<=nelgrp )

         iinplc = iwork(ielgrp)
         if ( iinplc==0 ) then

!        --- element group has not been found, warning

            call errint ( ielgrp, 1 )
            call errwar ( 1085, 1, 0, 0 )
            iinput(iinp+1:iinp+20) = 0
            iinp = iinp+20
            ielgrp = ielgrp+1

         else

!        --- element group has been filled

            ielgrpfirst = iinputlc(iinplc)
            ielgrplast = iinputlc(iinplc+1)

            do ielgrp = ielgrpfirst, ielgrplast

               iinplc = iwork(ielgrp)+2
               iinput(iinp+1:iinp+20) = iinputlc(iinplc:iinplc+19)
               iinplc = iinplc+20

!              --- check if all type numbers are larger than 99 or not

               icheck = 0
               do k = 1, ntype
                  itypnr = iinput(iinp+k)
                  if ( itypnr>=100 .or. itypnr<=0 .and. itypnr>=-3 ) &
                     icheck = 1
               end do
               iextra = itypnr/10000
               iinp = iinp+20
               if ( itypnr==-1 ) then

!              --- periodical boundary condition found

                  icount = iinputlc(iinplc)
                  iinplc = iinplc+1
                  iinput(iinp+1) = icount
                  iinp = iinp+1
                  do k = 1, icount

!                 --- loop over all the options

                     iinput(iinp+1) = iinputlc(iinplc)
                     iinput(iinp+2) = iinputlc(iinplc+1)
                     lengrllc = iinputlc(iinplc+1)
                     rinput(lengrl+1) = rinputlc(lengrllc  )
                     rinput(lengrl+2) = rinputlc(lengrllc+1)
                     lengrl = lengrl + 2
                     iinp = iinp+2
                     iinplc = iinplc+2

                  end do

                  if ( debug ) then

!                 --- Debug information

                     write(irefwr,*) 'global periodical boundary conds'
                     write(irefwr,1) 'lengrl, iinp, icount', &
                                      lengrl, iinp-2*icount, icount
                     call prinin ( iinput(iinp-2*icount), 1+2*icount, &
                                   'iinput' )
                     write(irefwr,*) 'Contents are: icount, followed ', &
                                     'by pairs of idegfd, start in ', &
                                     'rinput'
                     call prinrl ( rinput(lengrl-2*icount+1), 2*icount, &
                                   'rinput' )
                     write(irefwr,*) 'Contents are: pairs of constant,', &
                                     ' factor'

                  end if  ! ( debug )

               else if ( icheck==0 ) then

!              --- icheck = 0, user elements

                  inpelm = sepgetinpelm ( ielgrp )

!                 --- Find starting addresses of information

                  istart = iinplc
                  do k = 1, numveclc+nsear+1
                     iwork(nelgrp+k) = 0
                  end do
                  indtvc = 0
                  isear  = 0

!                 --- While loop until end of information

300               if ( iinputlc(istart)==1 ) then

!                 --- Type: numdegfd

                     iwork(nelgrp+1) = istart
                     inum = iinputlc(istart+1)
                     istart = istart+inum+2
                     go to 300

                  else if ( iinputlc(istart)==2 ) then

!                 --- Type: vectors of special structure

                     inum = iinputlc(istart+1)
                     ivec = iinputlc(istart+2)
                     iwork(nelgrp+1+ivec) = istart
                     istart = istart+inum+2
                     indtvc = max(indtvc,ivec)
                     go to 300

                  else if ( iinputlc(istart)==3 ) then

!                 --- Type: physical degrees of freedom

                     inum = iinputlc(istart+1)
                     ivec = iinputlc(istart+2)
                     isear = max(isear,ivec)
                     iwork(nelgrp+1+numveclc+ivec) = istart
                     istart = istart+inum+2
                     go to 300

                  end if

!                 --- Fill information, first numdegfd

                  iinput(iinp+1) = 1
                  iinput(iinp+2) = indtvc
                  iinput(iinp+3) = isear
                  istart = iwork(nelgrp+1)
                  if ( istart>0 ) then

!                 --- istart>0 information of degrees of freedom

                     inum = iinputlc(istart+1)
                     if ( inum==1 ) then

!                    --- inum = 1 constant number of degrees of freedom

                        iinput(iinp+1) = iinputlc(istart+2)
                        iinp = iinp+3

                     else

!                    --- inum>1 non-constant number of degrees of freedom

                        iinput(iinp+1) = 0
                        do k = 1, inpelm
                           iinput(iinp+k+3) = iinputlc(istart+1+k)
                        end do
                        iinp = iinp+3+inpelm

                     end if

                  else

!                 --- istart = 0 no information of degrees of freedom

                     iinp = iinp+3

                  end if

!                 --- Next vectors of special structure

                  do i = 1, indtvc
                     istart = iwork(nelgrp+1+i)
                     if ( istart==0 ) then

!                    --- istart = 0, no information found

                        call errint ( ivec, 1 )
                        call errint ( ielgrp, 2 )
                        call errwar ( 1086, 2, 0, 0 )
                        do k = 1, inpelm
                           iinput(iinp+k) = 0
                        end do
                        iinp = iinp+inpelm

                     else

!                    --- istart>0, information found

                        inum = iinputlc(istart+1)
                        if ( inum==2 ) then
                           do k = 1, inpelm
                              iinput(iinp+k) = iinputlc(istart+3)
                           end do
                        else
                           do k = 1, inpelm
                              iinput(iinp+k) = iinputlc(istart+2+k)
                           end do
                        end if
                        iinp = iinp+inpelm

                     end if

                  end do

!                 --- Finally physical degrees of freedom

                  do i = 1, isear
                     istart = iwork(nelgrp+1+numveclc+i)
                     if ( istart==0 ) then

!                    --- istart = 0, no information found

                        call errint ( ivec, 1 )
                        call errint ( ielgrp, 2 )
                        call errwar ( 1087, 2, 0, 0 )
                        do k = 1, inpelm
                           iinput(iinp+k) = 0
                        end do
                        iinp = iinp+inpelm

                     else

!                    --- istart>0, information found

                        inum = iinputlc(istart+1)
                        if ( inum==2 ) then
                           do k = 1, inpelm
                              iinput(iinp+k) = iinputlc(istart+3)
                           end do
                        else
                           do k = 1, inpelm
                              iinput(iinp+k) = iinputlc(istart+2+k)
                           end do
                        end if
                        iinp = iinp+inpelm

                     end if

                  end do

               else if ( iextra>0 ) then

!              --- iextra>0, extra information

                  iinput(iinp+1) = iextra
                  iinp = iinp+1
                  do i = 1, iextra
                     do k = 1, nncode3
                        iinput(iinp+k) = iinputlc(iinplc+k)
                     end do
                     iinplc = iinplc + nncode3
                     iinp = iinp + nncode3
                  end do

               end if

            end do  ! ielgrp = ielgrpfirst, ielgrplast
            ielgrp = ielgrplast+1

         end if

      end do  ! while ( ielgrp<=nelgrp )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prord0copy'

      end if  ! ( debug )
      call erclos ( 'prord0copy' )

      end subroutine prord0copy
