      subroutine prord1user ( iinplc, iinputlc, numveclc, nsear, jinp, ibngrp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Read information for boundary user elements
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ibngrp
      integer, intent(inout) :: iinplc, iinputlc(*), numveclc, nsear, jinp

!     ibngrp         i    boundary element group number
!     iinplc        i/o   Last position used in array iinputlc
!     iinputlc      i/o   See subroutine prord0
!     jinp          i/o   last position used in array iinput
!     nsear         i/o   Number of specific degrees of freedom
!     numveclc        i/o   Number of vectors of special structure
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode2=4
      integer itype2(nncode2), ncode2(nncode2), ncodmx2(nncode2)
      double precision tabmin2(nncode2), tabmax2(nncode2)
      character (len=8) code2(nncode2)
      integer :: inpelm, inum, i, inunkp, isear, ivec
      logical :: finished

!     code2      Table with keywords
!     finished       Indicates if the loop has finished (true) or not
!                    (false)
!     i          Counting variable
!     inpelm     Number of points in an element
!     inum       Number of numbers read in a record
!     inunkp     Number of degrees of freedom in first point
!     isear      Sequence number of specific degrees of freedom
!     itype2     Indicates the type of action corresponding to keywords
!                in code2
!     ivec       Sequence number of vector of special structure
!     ncode2     Table containing the number of significant letters in code2
!     ncodmx2    See ncodmx, now for code2
!     nncode2    Number of entries in code2
!     tabmax2    Contains maximum values allowed in table for code2
!     tabmin2    Contains minimum values allowed in table for code2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      22   number of degrees of freedom in nodal points not correct
!      31   Incorrect number of points with degrees of freedom are given
!           in element
!      32   Number of numbers found in record corresponding to vector
!           of special structure incorrect
!      33   sequence number of vector of special structure is incorrect
!      34   Incorrect number of points with degrees of freedom given in
!           element for array of special structure
!     540   sequence number of record corresponding to definition of physical
!           degrees of freedom is incorrect
!    1093   NPELM incorrect for boundray group
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'NUMDEGFD', 'VEC', 'NUMBER', 'NPELM' /
      data ncode2  /  5,  3,  5, 4 /
      data ncodmx2 /  8,  3,  6, 5 /
      data itype2  / 28,  33, 33, 0 /
      data tabmin2 / 1d2, 1d2, 1d2, 1d0 /
      data tabmax2 / 0d0, 0d0, 0d0, 100d0 /

! ======================================================================
!
      call eropen ( 'prord1user' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prord1user'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )
      call sepcopytype ( nncode2, itype2, tabmin2, tabmax2 )
      codemain(nncodemain+1) = codesub(1)
      ncodemain(nncodemain+1) = ncodesub(1)
      nmain = -nncodemain-1

!     -- all elements are user elements
!        Read rest of information until a main keyword or bngrp is found

      finished = .false.
      inpelm = -1

      do while ( .not. finished )

         table(1:nncode) = -1d20

         tabmax(1:4) = 100d0
         iretur = 0
         call sepreadsetkeyw
         icode = iretur

         if ( icode>0 ) then

!        --- icode>0, end of information

            iinplc = iinplc+1
            iinputlc(iinplc) = -1
            finished = .true.

         else if ( table(1)>0d0 ) then

!        --- numdegfd found

            iposut = nint(table(1))
            inum = nint(tabmax(1))
            iinputlc(iinplc+1) = 1
            iinputlc(iinplc+2) = inum
            iinplc = iinplc+2
            do i = 1, inum
               iinputlc(iinplc+i) = nint(tabout(iposut+i-1))
            end do
            if ( inum==1 ) then

!           --- inum  =  1

               inunkp = iinputlc(iinplc+1)
               if ( inunkp<=0 ) then
                  call errint ( inunkp, 1 )
                  call errsub ( 22, 1, 0, 0 )
               end if

            else if ( inpelm==-1 .and. inum>1 .and. inum<=100 ) then
               inpelm = inum

            else if ( inum/=inpelm ) then
               call errint ( inum, 1 )
               call errint ( inpelm, 2 )
               call errsub ( 31, 2, 0, 0 )

            end if
            iinplc = iinplc+inum

         else if ( table(2)>0d0 ) then

!        --- VEC found,  hence vector of special structure

            iposut = nint(table(2))
            inum = nint(tabmax(2))
            iinputlc(iinplc+1) = 2
            iinputlc(iinplc+2) = inum
            iinplc = iinplc+2
            do i = 1, inum
               iinputlc(iinplc+i) = nint(tabout(iposut+i-1))
            end do
            if ( inum<2 )  then
               call errint ( inum, 1 )
               call errsub ( 32, 1, 0, 0 )
            else if ( inpelm==-1 .and. inum<=100 ) then
               inpelm = inum-1
            else if ( inum>2 .and. inum/=inpelm+1 ) then
               call errint ( inum, 1 )
               call errint ( inpelm+1, 2 )
               call errsub ( 34, 2, 0, 0 )
               call errcrd
            end if
            ivec = iinputlc(iinplc+1)
            if ( ivec<=0 .or. ivec>100 ) then
               call errint ( ivec, 1 )
               call errint ( 100, 2 )
               call errsub ( 33, 2, 0, 0 )
               call errcrd
            else
               numveclc = max(numveclc,ivec)
            end if
            iinplc = iinplc+inum

         else if ( table(3)>0d0 ) then

!        --- NUMBER found,  hence physical degrees of freedom

            iposut = nint(table(3))
            inum = nint(tabmax(3))
            iinputlc(iinplc+1) = 3
            iinputlc(iinplc+2) = inum
            iinplc = iinplc+2
            do i = 1, inum
               iinputlc(iinplc+i) = nint(tabout(iposut+i-1))
            end do
            if ( inum<2 )  then
               call errint ( inum, 1 )
               call errsub ( 32, 1, 0, 0 )
            else if ( inpelm==-1 .and. inum<=100 ) then
               inpelm = inum-1
            else if ( inum>2 .and. inum/=inpelm+1 ) then
               call errint ( inum, 1 )
               call errint ( inpelm+1, 2 )
               call errsub ( 34, 2, 0, 0 )
               call errcrd
            end if
            isear = iinputlc(iinplc+1)
            if ( isear<=0 .or. isear>100 ) then
               call errint ( isear, 1 )
               call errint ( 100, 2 )
               call errsub ( 540, 2, 0, 0 )
               call errcrd
            else
               nsear = max(nsear,isear)
            end if
            iinplc = iinplc+inum

         else if ( table(4)>0d0 ) then

!        --- NPELM found

            inum = nint(table(4))
            if ( inpelm==-1 ) then
               inpelm = inum
            else if ( inpelm/=inum ) then
               call errint ( inum, 1 )
               call errint ( ibngrp, 2 )
               call errint ( inpelm, 3 )
               call errsub ( 1093, 3, 0, 0 )
            end if
         end if
         iinputlc(jinp) = inpelm

      end do  ! while ( .not. finished )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prord1user'

      end if  ! ( debug )
      call erclos ( 'prord1user' )

      end subroutine prord1user
