      subroutine prord2 ( iinput, maxlen, iinp, iinputlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.3  date 03-11-2014 Allow absence of natbouncond
!        version  5.2  date 15-03-2012 New type of input
!        version  5.1  date 22-11-2011 Extra debug
!
!   copyright (c) 1996-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read cards for natural boundary elements
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), maxlen, iinp, iinputlc(*)

!     iinp          i/o   Last position used in IINPUT
!     iinput        i/o   integer array to be filled with information concerning
!                         the natural boundary conditions
!     iinputlc            Work array in which the input read is stored before
!                         copying in iinput
!                         Per record read the following information is stored:
!                         Boundary element group number, followed by:
!                         Type of record:
!                            1: - &
!                            2: POINTS
!                            3: CURVES
!                            4: SURFACES
!                            5: - &
!                            6: OBSTACLE
!                            7: CONTACT_ELEMENTS
!                            8: CONTACT_POINTS
!                            9: NO_CONTACT_ELEMENTS
!                           10: NO_CONTACT_POINTS
!                           11: CURV_POINTS
!                           12: SURF_POINTS
!                           13: CROSS_SECTION_OBSTACLES
!                           14: ZERO_LEVEL_SET
!                         Followed by INUM = number of integers to follow, &
!                         followed by the integers itself
!     maxlen         i    Maximum amount of space for array iinput
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: ncodsb=3, nncode2=21
      integer ncode2(nncode2), ncodmx2(nncode2), nscods(ncodsb)
      character (len=22) codesb(ncodsb), code2(nncode2)
      integer inum, iinplc, numnat, istsrpnt, &
              ibngrp, lenpnt, lenobstcr, numnatinp, &
              i, jinp, natpnt, natcrv, natsrf, natobstcross, &
              natobst, natconct, natconpt, natnocel, natnocpt, natcrpnt, &
              natsrpnt, typrec, istpnt, istcrv, istsrf, istobstcross, &
              istobst, istconct, istconpt, istnocel, istnocpt, istcrpnt, &
              istartpnt, natlevset, lenlevset, istlevset, iinplcbef, &
              lencrv, lensrf, lencrvpt, lensrfpt

!     code2          Table with keywords
!     codesb         Table with subkeywords
!     i              Counting variable
!     ibngrp         Actual boundary element group number
!     iinplc         Last position used in array iinputlc
!     iinplcbef      Value of iinplc at start
!     inum           Number of numbers read in a record
!     istartpnt      Start position of keyword p in table
!     istconct       Starting address of contact elements in iinput
!     istconpt       Starting address of contact points in iinput
!     istcrpnt       Starting address of curve points in iinput
!     istcrv         Starting address of curves in iinput
!     istlevset      Starting address of level set in iinput
!     istnocel       Starting address of no-contact elements in iinput
!     istnocpt       Starting address of no-contact points in iinput
!     istobst        Starting address of obstacles in iinput
!     istobstcross   Starting address of cross_section_obstacles in iinput
!     istpnt         Starting address of points in iinput
!     istsrf         Starting address of surfaces in iinput
!     istsrpnt       Starting address of surface points in iinput
!     jinp           Help variable to store the value of iinp
!     lencrv         Length of curves part
!     lencrvpt       Length of curve_points part
!     lenlevset      Length of level set part
!     lenobstcr      Length of cross_section_obstacles part
!     lenpnt         Length of points part
!     lensrf         Length of surfaces part
!     lensrfpt       Length of surface_point part
!     natconct       Number of contacts where natural boundary elements
!                    are given
!     natconpt       number of parts where contact points are defined as
!                    bound els.
!     natcrpnt       Number of curves where point boundary elements
!                    are given
!     natcrv         number of parts where curve elements are defined
!                    for natural boundary conditions
!     natlevset      number of parts where zero level set elements
!                    are defined
!     natnocel       number of parts where no_contact elements are defined
!     natnocpt       number of parts where no_contact points are defined as
!                    boundary elements
!     natobst        number of parts where obstacle elements are defined
!     natobstcross   number of parts where cross_section_obstacle elements
!                    are defined
!     natpnt         number of parts where point elements are defined
!                    for natural boundary conditions
!     natsrf         number of parts where surface elements are defined
!                    for natural boundary conditions
!     natsrpnt       Number of surfaces where point boundary elements are
!                    given
!     ncode2         Table containing the number of significant letters in
!                    code2
!     ncodmx2        Table containing the maximal number of  characters in
!                    the keyword table code2
!     ncodsb         Number of entries in codesb
!     nncode2        Number of entries in code2
!     nscods         Number of sub keywords
!     numnat         Number of different boundary groups
!     numnatinp      numnat at input
!     typrec         Type of record found
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRORD2CONTCT   Fill array iinputlc with information of natural boundary
!                    conitions for contact records
!     PRORD2CONTCTP  Fill array iinputlc with information of natural boundary
!                    conitions for contact_points records
!     PRORD2CROSS    Fill array iinputlc with information of natural boundary
!                    conitions for cros_section_obstacle records
!     PRORD2CURVE    Fill array iinputlc with information of natural boundary
!                    conitions for curve records
!     PRORD2CURVEP   Fill array iinputlc with information of natural boundary
!                    conitions for curv_points records
!     PRORD2LEVEL    Fill array iinputlc with information of natural boundary
!                    conitions for zero_levelset records
!     PRORD2NOCONTCP Fill array iinputlc with information of natural boundary
!                    conitions for no_contact_points records
!     PRORD2NOCONTCT Fill array iinputlc with information of natural boundary
!                    conitions for no_contact_elements records
!     PRORD2OBST     Fill array iinputlc with information of natural boundary
!                    conitions for obstacle records
!     PRORD2POINT    Fill array iinputlc with information of natural boundary
!                    conitions for point records
!     PRORD2SURF     Fill array iinputlc with information of natural boundary
!                    conitions for surface records
!     PRORD2SURFP    Fill array iinputlc with information of natural boundary
!                    conitions for surf_points records
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!    input is read from the input file.
!    The following texts are recognized:
!       belm1 = points ( p3, p5, p7, ... )
!       belm2 = curves ( [shape = k,] c1, c2, c3, ... )
!       belm3 = surfaces ( s1, s2, s3, s4, ... )
!       belm4 = obstacle i
!       belm5 = contact_elements i
!       belm6 = contact_points i
!       belm7 = no_contact_elements i
!       belm8 = no_contact_points i
!       belmi = curv_points ( c1, c2, c3, ... )
!       belmj = surf_points ( s1, s2, s3, s4, ... )
!       belmj = crosssection_obstacle i, exclude_type = k, &
!               exclude_curves (c1, c2, c3, ... )
!       belmk = zero_levelset i, exclude_curves (c1, c2, c3, ... )
!    main keywords ( see prorea )
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      29   Too few space for standard problems in IINPUT
!     166   boundary element group number incorrect
!     550   array length too short
!    1894   boundary elements with group number larger than numnatinp
!    2917   Option not available
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Read all input until main keyword is found, store in iinputlc
!    Copy all input in the correct sequence into iinput
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2  / 'BELM', 'POINTS', 'CURV_POINTS', 'SURF_POINTS', &
                   'SHAPE', &
                   'OBSTACLE', 'CONTACT_ELEMENTS', 'CONTACT_POINTS', &
                   'NO_CONTACT_ELEMENTS', 'NO_CONTACT_POINTS', &
                   'CURVES', 'SURFACES', 'CROSS_SECTION_OBSTACLE', &
                   'EXCLUDE_TYPE', 'EXCLUDE_CURVES', 'COMPLETE_CURVES', &
                   'COMPLETE_SURFACES', 'ZERO_LEVELSET', &
                   'P', 'C', 'S' /
      data ncode2  / 3, 3, 7, 7, 3,  4, 10, 10, 13, 13, &
                    5, 6, 5,10, 10,11, 11,  8,  1,  1, &
                    1 /
      data ncodmx2 / 4, 6,11,11, 5,  8, 16, 15, 19, 18, &
                    6, 8,22,12,14, 15, 17, 13,  1,  1, &
                    1 /
      data codesb / 'BOUNELEMENT', 'BNGRP', 'TYPE' /
      data nscods /  4, 3, 3 /

!     code2    Table of local keywords
!              1: BELM
!              2: POINTS
!              3: CURV_POINTS
!              4: SURF_POINTS
!              5: SHAPE
!              6: OBSTACLE
!              7: CONTACT_ELEMENTS
!              8: CONTACT_POINTS
!              9: NO_CONTACT_ELEMENTS
!             10: NO_CONTACT_POINTS
!             11: CURVES
!             12: SURFACES
!             13: CROSS_SECTION_OBSTACLE
!             14: EXCLUDE_TYPE
!             15: EXCLUDE_CURVES
!             16: -
!             17: -
!             18: ZERO_LEVELSET
!             19: P
!             20: C
!             21: S
!     codesb   Table of local subkeywords
!              1: BOUNELEMENTS
!              2: BNGRP
!              3: TYPE
! ======================================================================
!
      call eropen ( 'prord2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prord2'
         write(irefwr,1) 'iinp', iinp
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Copy extra information into codemain

      call sepcopycodesub ( codesb, ncodsb, nscods )
      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )

      ityprd = 1
      istartpnt = 19    ! Position of keyword P in table

!     --- Read records for the generation of boundary elements

      numnat = iinput(2)
      numnatinp = numnat

      natpnt = 0
      natcrv = 0
      natsrf = 0
      natobst = 0
      natconct = 0
      natconpt = 0
      natnocel = 0
      natnocpt = 0
      natcrpnt = 0
      natsrpnt = 0
      natobstcross = 0
      natlevset = 0
      lenlevset = 0
      lenpnt = 0
      lencrv = 0
      lensrf = 0
      lencrvpt = 0
      lensrfpt = 0
      lenobstcr = 0
      iinplc = 0
      jinp = iinp

300   table(1:nncode) = -1d20

      itype(1:nncode) = 0
      itype(2:4) = 2  ! POINTS, CURV_POINTS, SURF_POINTS
      itype(11:12) = 2 ! CURVES, SURFACES
      itype(15:17) = 2 ! EXCLUDE_CURVES, COMPLETE_CURVES, COMPLETE_SURFACES
      itype(istartpnt:istartpnt+2) = 14  ! P, C, S

      tabmin(1:nncode) = 0d0
      tabmin(1) = 1d0
      tabmin(5:10) = 1d0
      tabmin(13:14) = 1d0
      tabmin(18) = 1d0

      tabmax(1:nncode) = 0d0
      tabmax(1) = 1d3
      tabmax(5) = 2d0
      tabmax(6) = 1d0
      tabmax(7:10) = 10d0
      tabmax(13) = numobst
      tabmax(14) = 2d0
      tabmax(18) = 10d0

      tabmax(istartpnt) = nuspnt
      tabmin(istartpnt) = nuspnt
      tabmax(istartpnt+1) = ncurvs
      tabmin(istartpnt+1) = ncurvs
      tabmax(istartpnt+2) = nsurfs
      tabmin(istartpnt+2) = nsurfs

      table(5) = 0d0
      table(14) = 1d0
      nmain = nncodemain
      iretur = icode

      call sepreadsetkeyw
      icode = iretur
      if ( ierror/=0 ) go to 1000
      if ( icode==0 ) then

!     --- Not a main keyword found, hence BELM = ...

         if ( table(1)>0 ) then
            ibngrp = nint(table(1))
         else
            ibngrp = 0
         end if
         numnat = max(0,ibngrp)
         if ( ibngrp<1 ) then

!        --- Error boundary group number out of range

            call errint ( ibngrp, 1 )
            call errint ( numnat, 2 )
            call errsub ( 166, 2, 0, 0 )

         end if
         iinputlc(iinplc+1) = ibngrp
         iinplc = iinplc+1
         if ( table(2)>0d0 ) then

!        --- point card found

            call prord2point ( iinputlc, iinplc, typrec, natpnt, table, &
                               istartpnt, lenpnt, tabmax, tabout )

         else if ( table(11)>0d0 ) then

!        --- curves card found

            call prord2curve ( iinputlc, iinplc, typrec, natcrv, table, &
                               istartpnt, tabmax, tabout, lencrv )

         else if ( table(12)>0d0 ) then

!        --- surface card found

            call prord2surf ( iinputlc, iinplc, typrec, natsrf, table, &
                              istartpnt, tabmax, tabout, lensrf )

         else if ( table(6)>0d0 ) then

!        --- obstacle card found

            call prord2obst ( iinputlc, iinplc, typrec, natobst, table )

         else if ( table(7)>0d0 ) then

!        --- contact_elements card found

            call prord2contct ( iinputlc, iinplc, typrec, natconct, &
                                table )

         else if ( table(9)>0d0 ) then

!        --- no_contact_elements card found

            call prord2nocontct ( iinputlc, iinplc, typrec, natnocel, &
                                  table )

         else if ( table(8)>0d0 ) then

!        --- contact_points card found

            call prord2contctp ( iinputlc, iinplc, typrec, natconpt, &
                                 table )

         else if ( table(10)>0d0 ) then

!        --- no_contact_points card found

            call prord2nocontcp ( iinputlc, iinplc, typrec, natnocpt, &
                                  table )

         else if ( table(3)>0d0 ) then

!        --- curv_points card found

            call prord2curvep ( iinputlc, iinplc, typrec, natcrpnt, &
                                istartpnt, table, tabmax, tabout, &
                                lencrvpt )

         else if ( table(4)>0d0 ) then

!        --- surf_points card found

            call prord2surfp ( iinputlc, iinplc, typrec, natsrpnt, &
                               istartpnt, table, tabmax, tabout, &
                               lensrfpt )

         else if ( table(13)>0d0 ) then

!        --- cros_section_obstacle card found

            iinplcbef = iinplc
            call prord2cross ( iinputlc, iinplc, typrec, natobstcross, &
                               table, istartpnt, tabmax, tabout, &
                               lenobstcr )

            if ( debug ) then
               write(irefwr,1) 'iinplcbef, iinplc', iinplcbef, iinplc
               call prinin ( iinputlc(iinplcbef), iinplc-iinplcbef+1, &
                             'iinputlc after prord2cross' )
            end if  ! ( debug )

         else if ( table(18)>0d0 ) then

!        --- zero_level set found

            iinplcbef = iinplc
            call prord2level ( iinputlc, iinplc, typrec, natlevset, &
                               table, istartpnt, tabmax, tabout, &
                               code, ncodmx )
            lenlevset = lenlevset+iinplc-iinplcbef

            if ( debug ) then
               write(irefwr,1) 'iinplcbef', iinplcbef
               call prinin ( iinputlc(iinplcbef), iinplc-iinplcbef, &
                             'iinputlc after prord2level' )
            end if  ! ( debug )

         end if

         if ( table(13)<=0d0 .and. table(15)>0d0 ) &
             call errkeywexcl ( code, ncodmx, 15, typrec )
         if ( ierror/=0 ) go to 1000
         go to 300

      end if  ! ( icode==0 ) then

      if ( iinplc>maxlen ) then

!     --- Too much information, i.e. maxlen too short

         call errchr ( 'iinputlc', 1 )
         call errint ( iinplc, 1 )
         call errint ( maxlen, 2 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

!     --- Copy all input in the correct sequence into iinput

      istpnt   = iinp
      istcrv   = istpnt + lenpnt
      istsrf   = istcrv + lencrv
      istobst  = istsrf + lensrf
      istconct = istobst + 3*natobst
      istnocel = istconct + 3*natconct
      istconpt = istnocel + 3*natnocel
      istnocpt = istconpt + 3*natconpt
      istcrpnt = istnocpt + 3*natnocpt
      istsrpnt = istcrpnt + lencrvpt
      istobstcross = istsrpnt + lensrfpt
      istlevset = istobstcross + lenobstcr
      iinp     = istlevset + lenlevset
      jinp = 0

      if ( debug ) then
         write(irefwr,1) 'iinp', iinp
         write(irefwr,1) 'istpnt, istcrv, istsrf, istobst, istconct', &
                          istpnt, istcrv, istsrf, istobst, istconct
         write(irefwr,1) 'istnocel, istconpt, istnocpt, istcrpnt', &
                          istnocel, istconpt, istnocpt, istcrpnt
         write(irefwr,1) 'istsrpnt, istobstcross, istlevset', &
                          istsrpnt, istobstcross, istlevset
      end if  ! ( debug )

400   if ( jinp<iinplc ) then

!     --- The last record has not yet been stored

         typrec = iinputlc(jinp+2)
         if ( debug ) &
            write(irefwr,1) 'jinp, iinplc, typrec', jinp, iinplc, typrec

         select case (typrec)
         case(2)

!        --- Point record found, variable length

            inum = iinputlc(jinp+3)
            iinput(istpnt+1) = iinputlc(jinp+1)
            iinput(istpnt+2) = inum
            iinput(istpnt+3:istpnt+inum+2) = &
               iinputlc(jinp+4:jinp+inum+3)
            istpnt = istpnt+inum+2
            jinp = jinp+inum+3

         case(3)

!        --- Curve record found, variable length

            iinput(istcrv+1) = iinputlc(jinp+1) ! boundary el group number
            inum = iinputlc(jinp+4)+2           ! number of curves + 2
            iinput(istcrv+2:istcrv+1+inum) = &
               iinputlc(jinp+3:jinp+inum+2)
            istcrv = istcrv+1+inum
            jinp = jinp+2+inum
            if ( debug ) write(irefwr,1) 'istcrv, jinp', istcrv, jinp

         case(4)

!        --- Surface record found, 3 positions

            iinput(istsrf+1) = iinputlc(jinp+1) ! boundary el group number
            inum = iinputlc(jinp+3)+2           ! number of surfaces + 2
            iinput(istsrf+2:istsrf+inum) = &
               iinputlc(jinp+3:jinp+inum+1)
            istsrf = istsrf+inum
            jinp = jinp+1+inum

         case(6)

!        --- Obstacle record found, 3 positions

            iinput(istobst+1) = iinputlc(jinp+1)
            iinput(istobst+2) = iinputlc(jinp+3)
            iinput(istobst+3) = iinputlc(jinp+4)
            istobst = istobst+3
            jinp = jinp+4

         case(7)

!        --- Contact elements record found, 3 positions

            iinput(istconct+1) = iinputlc(jinp+1)
            iinput(istconct+2) = iinputlc(jinp+3)
            iinput(istconct+3) = iinputlc(jinp+4)
            istconct = istconct+3
            jinp = jinp+4

         case(8)

!        --- Contact points record found, 3 positions

            iinput(istconpt+1) = iinputlc(jinp+1)
            iinput(istconpt+2) = iinputlc(jinp+3)
            iinput(istconpt+3) = iinputlc(jinp+4)
            istconpt = istconpt+3
            jinp = jinp+4

         case(9)

!        --- No-Contact elements record found, 3 positions

            iinput(istnocel+1) = iinputlc(jinp+1)
            iinput(istnocel+2) = iinputlc(jinp+3)
            iinput(istnocel+3) = iinputlc(jinp+4)
            istnocel = istnocel+3
            jinp = jinp+4

         case(10)

!        --- No-Contact points record found, 3 positions

            iinput(istnocpt+1) = iinputlc(jinp+1)
            iinput(istnocpt+2) = iinputlc(jinp+3)
            iinput(istnocpt+3) = iinputlc(jinp+4)
            istnocpt = istnocpt+3
            jinp = jinp+4

!           --- 11: CURV_POINTS
!               12: SURF_POINTS

         case(11)

!        --- Curve-points record found, 3 positions

            iinput(istcrpnt+1) = iinputlc(jinp+1) ! boundary el group number
            inum = iinputlc(jinp+3)+2           ! number of curves + 2
            iinput(istcrpnt+2:istcrpnt+inum) = &
               iinputlc(jinp+3:jinp+inum+1)
            istcrpnt = istcrpnt+inum
            jinp = jinp+1+inum

         case(12)

!        --- Surface points record found, 3 positions

            iinput(istsrpnt+1) = iinputlc(jinp+1) ! boundary el group number
            inum = iinputlc(jinp+3)+2           ! number of surfaces + 2
            iinput(istsrpnt+2:istsrpnt+inum) = &
               iinputlc(jinp+3:jinp+inum+1)
            istsrpnt = istsrpnt+inum
            jinp = jinp+2+inum

         case(13)

!        --- Cross_section_obstacles record found, 6 positions

            iinput(istobstcross+1) = iinputlc(jinp+1)
            inum = iinputlc(jinp+6)           ! number of curves
            do i = 1, 5+inum
               iinput(istobstcross+1+i) = iinputlc(jinp+2+i)
            end do
            if ( debug ) then
               write(irefwr,1) 'inum, istobstcross', inum, istobstcross
               call prinin ( iinput(istobstcross+1), 5+inum, 'iinput' )
            end if  ! ( debug )
            jinp = jinp+6+inum
            istobstcross = istobstcross+5+inum

         case(18)

!        --- Zero level set record found, 6 positions

            iinput(istlevset+1) = iinputlc(jinp+1)
            iinput(istlevset+2:istlevset+5) = iinputlc(jinp+3:jinp+6)
            inum = max(1,iinputlc(jinp+6))
            iinput(istlevset+6:istlevset+5+inum) = &
               iinputlc(jinp+7:jinp+6+inum)

            if ( debug ) then
               write(irefwr,1) 'istlevset', istlevset
               call prinin ( iinput(istlevset+1), 5+inum, &
                             'iinput/levset' )
            end if  ! ( debug )

            istlevset = istlevset+5+inum
            jinp = jinp+6+inum

         case default

            call errint ( typrec, 1 )
            call errsub ( 2917, 1, 0, 0 )
            go to 1000

         end select  ! case (typrec)

         go to 400

      end if

      if ( numnatinp>0 .and. numnat>numnatinp ) then

!     --- Error boundary elements with group number larger than numnatinp found

         call errint ( numnatinp, 1 )
         call errint ( numnat, 2 )
         call errsub ( 1894, 2, 0, 0  )
         call errcrd

      end if  ! ( numnatinp>0 .and. numnat>numnatinp )

      if ( iinp>=maxlen ) then

!     --- iinp>maxlen, too few space in IINPUT

         call errint ( iinp, 1 )
         call errint ( maxlen, 2 )
         call errsub ( 29, 2, 0, 0 )
         call errcrd

      end if
      iinput( 3) = natpnt
      iinput( 4) = natcrv
      iinput( 5) = natsrf
      iinput(17) = natobst
      iinput(20) = natconct
      iinput(23) = natconpt
      iinput(24) = natnocel
      iinput(25) = natnocpt
      iinput(26) = natcrpnt
      iinput(27) = natsrpnt
      iinput(34) = natobstcross
      iinput(55) = natlevset
      iinput( 2) = numnat

1000  if ( debug ) then

!     --- Debug information

         call prinin ( iinput, iinp, 'iinput' )
         write(irefwr,*) 'End prord2'

      end if  ! ( debug )
      call erclos ( 'prord2' )

      end
