      subroutine prord2curve ( iinputlc, iinplc, itype, natcrv, table, &
                               istartpnt, tabmax, tabout, lencrv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 15-03-2012 Extra parameter lencrv
!        version  1.0  date 14-04-2011
!
!   copyright (c) 2011-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iinputlc with information of natural boundary conditions
!     for curve records
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     natural_boundary_condition
!     curve
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinputlc(*), iinplc, itype, natcrv, istartpnt, &
              lencrv
      double precision table(*), tabmax(*), tabout(*)

!     iinplc        i/o   Last position used in array iinputlc
!     iinputlc      i/o   Work array in which the input read is stored before
!                         copying in iinput
!                         See subroutine prord2
!     istartpnt      i    Start position of keyword p in table
!     itype          o    Type of record found
!     lencrv        i/o   Length of curves part in iinput for boundary elements
!     natcrv        i/o   number of parts where curve elements are defined
!                         for natural boundary conditions
!     table          i    Contains the numbers read for the various keywords
!     tabmax         i    Contains maximum values allowed in table
!     tabout         i    Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inum, iposut, ishape

!     inum           Number of numbers read in a record
!     iposut         Position in array tabout
!     ishape         Shape number of elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     334   record with curves contains wrong number of curve numbers
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prord2curve' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prord2curve'
         write(irefwr,1) 'iinplc, lencrv', iinplc, lencrv
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- curves card found

      natcrv = natcrv+1
      itype = 3
      ishape = nint(table(5))

      inum = nint(tabmax(istartpnt+1))  ! number of curves read
      iposut = nint(table(istartpnt+1)) ! starting address in tabout

      if ( debug ) then
         write(irefwr,1) 'natcrv, itype, ishape', &
                          natcrv, itype, ishape
         write(irefwr,1) 'inum, iposut', inum, iposut
      end if  ! ( debug )

      if ( inum==0 ) then

!     --- inum = 0, no curves found

         call errint ( inum, 1 )
         call errint ( 2, 2 )
         call errsub ( 334, 2, 0, 0 )
         go to 1000

      end if  ! ( inum==0 )

      iinputlc(iinplc+1) = itype
      iinputlc(iinplc+2) = ishape
      iinputlc(iinplc+3) = inum
      iinplc = iinplc+3
      iinputlc(iinplc+1:iinplc+inum) = &
         nint(tabout(iposut:iposut+inum-1))
      if ( debug ) then
         write(irefwr,1) 'iinplc/curves', iinplc
         call prinin ( iinputlc(iinplc-2), inum+3, 'iinputlc' )
      end if  ! ( debug )
      iinplc = iinplc+inum
      lencrv = lencrv+3+inum

1000  call erclos ( 'prord2curve' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'inum, iinplc, lencrv', inum, iinplc, lencrv
         write(irefwr,*) 'End prord2curve'

      end if  ! ( debug )

      end
