      subroutine prord2level ( iinputlc, iinplc, itype, natlevset, &
                               table, istartpnt, tabmax, tabout, &
                               code, ncodmx )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 22-11-2011 Extra debug
!        version  1.0  date 14-04-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iinputlc with information of natural boundary conditions
!     for zero_levelset records
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     natural_boundary_condition
!     level_set
!     cross_section
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinputlc(*), iinplc, itype, natlevset, istartpnt, &
              ncodmx(*)
      double precision table(*), tabmax(*), tabout(*)
      character ( len=*) code(*)

!     code           i    Table with keywords
!     iinplc        i/o   Last position used in array iinputlc
!     iinputlc      i/o   Work array in which the input read is stored before
!                         copying in iinput
!                         See subroutine prord2
!     istartpnt      i    Start position of keyword p in table
!     itype          o    Type of record found
!     natlevset     i/o   number of parts where zero level set elements
!                         are defined
!     ncodmx         i    Table containing the maximal number of  characters in
!                         the keyword table code
!     table          i    Contains the numbers read for the various keywords
!     tabmax         i    Contains maximum values allowed in table
!     tabout         i    Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer inum, iposut, iexclude

!     iexclude       Type of exclusion method
!     inum           Number of numbers read in a record
!     iposut         Position in array tabout
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1463   The number of numbers found after the keyword too large
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prord2level' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prord2level'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      itype = 18
      natlevset = natlevset + 1
      iexclude = nint(table(14))
      iinputlc(iinplc+1) = itype             ! type of elements
      iinputlc(iinplc+2) = nint(table(18))   ! sequence number of level set
      iinputlc(iinplc+3) = iexclude          ! type of exclusion
      if ( table(15)>0d0 ) then

!     --- exclude_curves found

         inum = nint(tabmax(istartpnt+1))
         if ( inum<=0 ) then
            call errint ( inum, 1 )
            call errint ( 10, 2 )
            call errchr ( code(15)(1:ncodmx(15)), 1 )
            call errsub ( 1463, 2, 0, 1 )
            go to 1000
         end if ! ( inum<=0 )

         iposut = nint(table(istartpnt+1))

         iinputlc(iinplc+4) = 1
         iinputlc(iinplc+5) = inum
         iinputlc(iinplc+6:iinplc+5+inum) = &
              nint(tabout(iposut:iposut+inum-1))

      else

!     --- not, exclude_curves found; set equal to zero

         iinputlc(iinplc+4) = 0
         iinputlc(iinplc+5) = 0
         iinputlc(iinplc+6) = 0
         inum = 1

      end if

1000  call erclos ( 'prord2level' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'natlevset, iexclude, inum', &
                          natlevset, iexclude, inum
         call prinin ( iinputlc(iinplc), 6+inum, 'iinplc' )
         write(irefwr,*) 'End prord2level'

      end if  ! ( debug )

      iinplc = iinplc+5+inum

      end
