      subroutine prord3copy ( iinp, lenpnt, lensrf, lenvlm, &
                              nesnod, neselm, lengrp, nesobst, &
                              nesallobst, nesinnobst, nesbnobst, &
                              nesonbnobst, nesinonbnobst, nesconct, &
                              nesnocon, lennodal, neszerolev, nescav, &
                              iinput, iinplc, iinputlc, npoint, lencrv, &
                              leninters )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 13-10-2013 Extra parameter leninters
!        version  1.0  date 30-03-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Copy all input read by prord3 in the correct sequence into iinput
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     essential_boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinp, lenpnt, lensrf, lenvlm, nesnod, neselm, &
              lengrp, nesobst, nesallobst, nesinnobst, nesbnobst, &
              nesonbnobst, nesinonbnobst, nesconct, nesnocon, lennodal, &
              neszerolev, nescav, iinput(*), iinplc, iinputlc(*), &
              npoint, lencrv, leninters

!     iinp          i/o   Last position used in IINPUT
!     iinplc        i/o   Last position used in array iinputlc
!     iinput        i/o   integer array to be filled with information concerning
!                         the natural boundary conditions
!     iinputlc       i    integer work array where the information read is
!                         stored before processing
!     lencrv         i    Number of entries in array curves
!     lengrp         i    Length of group part
!     leninters      i    Length of intersection part
!     lennodal       i    Length of nodal_point part
!     lenpnt         i    Length of points part
!     lensrf         i    Length of surfaces part
!     lenvlm         i    Length of volumes part
!     nesallobst     i    Number of obstacles where all nodes are prescribed
!     nesbnobst      i    Number of obstacles where all but the inner nodes are
!                         prescribed
!     nescav         i    Number of parts where cavity bondary conditions are given
!     nesconct       i    Number of contacts at which essential boundary
!                         conditions are given
!     neselm         i    Number of parts where element essential boundary
!                         conditions are given
!     nesinnobst     i    Number of obstacles where all inner nodes are prescribed
!     nesinonbnobst  i    Number of obstacles where all he nodes on the
!                         obstacle which are only part of elements completely
!                         inside the obstacle are prescribed
!     nesnocon       i    Number of "no"-contacts with essential boundary
!                         conditions
!     nesnod         i    Number of groups of nodes with essential boundary
!                         conditions
!     nesobst        i    Number of obstacles with essential boundary
!                         conditions
!     nesonbnobst    i    Number of obstacles where all the nodes on the boundary of
!                         the obstacle are prescribed
!     neszerolev     i    Number of parts where essential boundary conditions of
!                         the type zero_levelset i are given
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istpnt, istcrv, istsrf, istvlm, istnod, istelm, istgrp, &
              istobst, istallobst, istinnobst, istbnobst, istonbnobst, &
              istinonbnobst, istconct, istnocon, istnodal, istzerolev, &
              istcav, jinp, ityperec, i, inum, istinters

!     i              Counting variable
!     inum           Number of numbers read in a record
!     istallobst     Starting address of obstacles where all nodes are
!                    prescribed in iinput
!     istbnobst      Starting address of obstacles where all but the inner nodes
!                    are prescribed in iinput
!     istcav         Starting address of cavity input part
!     istconct       Starting address of contact elements in iinput
!     istcrv         Starting address of curves in iinput
!     istelm         Starting address of elements in iinput
!     istgrp         Starting address of groups in iinput
!     istinnobst     Starting address of obstacles where all inner nodes
!                    are prescribed in iinput
!     istinonbnobst  Starting address of obstacles where all the nodes on the
!                    obstacle which are only part of elements completely
!                    inside the obstacle are prescribed in iinput
!     istinters      Starting address of intersection in iinput
!     istnocon       Starting address of no-contact points in iinput
!     istnod         Starting address of nodes in iinput
!     istnodal       Starting address of nodal points in iinput
!     istobst        Starting address of obstacles in iinput
!     istonbnobst    Starting address of obstacles where all the nodes on the
!                    obstacle are prescribed in iinput
!     istpnt         Starting address of points in iinput
!     istsrf         Starting address of surfaces in iinput
!     istvlm         Starting address of volumes in iinput
!     istzerolev     Starting address of zero_levelset in iinpu
!     ityperec       Type of record found
!     jinp           last position used in array iinput
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRORD3COPYSF   Copy all input read by prord3 in the correct sequence
!                    into iinput surface part
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1672   Nodal point number out of range
!    3098   Value of parameter not allowed
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prord3copy' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prord3copy'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      istpnt     = iinp
      istcrv     = istpnt + lenpnt
      istsrf     = istcrv + lencrv
      istvlm     = istsrf + lensrf
      istnod     = istvlm + lenvlm
      istelm     = istnod + 3*nesnod
      istgrp     = istelm + 4*neselm
      istobst    = istgrp + lengrp
      istallobst = istobst + 3*nesobst
      istinnobst = istallobst + 2*nesallobst
      istbnobst  = istinnobst + 2*nesinnobst
      istonbnobst = istbnobst + 2*nesbnobst
      istinonbnobst = istonbnobst + 2*nesonbnobst
      istconct   = istinonbnobst + 2*nesinonbnobst
      istnocon   = istconct + 3*nesconct
      istnodal   = istnocon + 3*nesnocon
      istzerolev = istnodal+lennodal
      istcav = istzerolev+2*neszerolev
      istinters = istcav+2*nescav
      iinp = istinters+leninters
      jinp = 0

      if ( debug ) then
         write(irefwr,1) 'istpnt, istcrv, istsrf, istvlm, istnod', &
                          istpnt, istcrv, istsrf, istvlm, istnod
         write(irefwr,1) 'istelm, istgrp, istobst', &
                          istelm, istgrp, istobst
         write(irefwr,1) 'istallobst, istinnobst, istbnobst', &
                          istallobst, istinnobst, istbnobst
         write(irefwr,1) 'istonbnobst, istinonbnobst, istconct', &
                          istonbnobst, istinonbnobst, istconct
         write(irefwr,1) 'istnocon, istnodal, istzerolev, istcav', &
                          istnocon, istnodal, istzerolev, istcav
         write(irefwr,1) 'istinters, iinp', &
                          istinters, iinp
      end if  ! ( debug )

      do while ( jinp<iinplc )

!     --- The last record has not yet been stored

         ityperec = iinputlc(jinp+2)
         select case ( ityperec )
         case(1)

!        --- Point record found, variable length

            inum = iinputlc(jinp+3)
            iinput(istpnt+1) = iinputlc(jinp+1)
            iinput(istpnt+2) = inum
            iinput(istpnt+3:istpnt+2+inum) = &
               iinputlc(jinp+4:jinp+3+inum)
            istpnt = istpnt+inum+2
            jinp = jinp+inum+3

         case(2)

!        --- Curve record found, 4 positions

            if ( debug )  write(irefwr,1) 'jinp, istcrv', jinp, istcrv
            iinput(istcrv+1) = iinputlc(jinp+1)
            iinput(istcrv+2:istcrv+3) = iinputlc(jinp+3:jinp+4)
            inum = iinputlc(jinp+4)
            if ( debug ) &
               call prinin ( iinput(istcrv+1), 3, 'iinput_cur/1' )
            istcrv = istcrv+3
            jinp = jinp+4
            if ( inum>0 ) then

!           --- inum>0: set of curves

               iinput(istcrv+1:istcrv+inum) = iinputlc(jinp+1:jinp+inum)
               if ( debug ) &
                  call prinin ( iinput(istcrv+1), inum, 'iinput_cur/2' )
               istcrv = istcrv+inum
               jinp = jinp+inum

            end if  ! ( inum>0 )

         case(3)

!        --- Surface record found, variable length

            call prord3copysf ( iinput, iinputlc, jinp, istsrf )

         case(4)

!        --- Nodes record found, 3 positions

            iinput(istnod+1) = iinputlc(jinp+1)
            iinput(istnod+2:istnod+3) = iinputlc(jinp+3:jinp+4)
            istnod = istnod+3
            jinp = jinp+4

         case(5)

!        --- Element record found, 5 positions

            iinput(istelm+1) = iinputlc(jinp+1)
            iinput(istelm+2:istelm+4) = iinputlc(jinp+3:jinp+5)
            istelm = istelm+4
            jinp = jinp+5

         case(6)

!        --- Group record found, variable length

            inum = iinputlc(jinp+3)
            iinput(istgrp+1) = iinputlc(jinp+1)
            jinp = jinp+4
            istgrp = istgrp+2
            iinput(istgrp:istgrp+inum) = iinputlc(jinp:jinp+inum)
            istgrp = istgrp+inum
            jinp = jinp+inum

         case(7)

!        --- Obstacle record found, 3 positions

            iinput(istobst+1) = iinputlc(jinp+1)
            iinput(istobst+2:istobst+3) = iinputlc(jinp+3:jinp+4)
            istobst = istobst+3
            jinp = jinp+4

         case(8)

!        --- Contact record found, 3 positions

            iinput(istconct+1) = iinputlc(jinp+1)
            iinput(istconct+2:istconct+3) = iinputlc(jinp+3:jinp+4)
            istconct = istconct+3
            jinp = jinp+4

         case(9)

!        --- No-contact record found, 3 positions

            iinput(istnocon+1) = iinputlc(jinp+1)
            iinput(istnocon+2:istnocon+3) = iinputlc(jinp+3:jinp+4)
            istnocon = istnocon+3
            jinp = jinp+4

         case(10)

!        --- Nodal point record found, variable length

            inum = iinputlc(jinp+3)
            iinput(istnodal+1) = iinputlc(jinp+1)
            iinput(istnodal+2) = inum
            if ( inum>0 ) then

!           --- Nodal point numbers found

               do i = 1, inum
                  iinput(istnodal+2+i) = iinputlc(jinp+3+i)
                  if ( iinputlc(jinp+3+i)<=0 .or. &
                       iinputlc(jinp+3+i)>npoint ) then

!                 --- nodal point number out of range

                     call errint ( i, 1 )
                     call errint ( iinputlc(jinp+3+i), 2 )
                     call errint ( npoint, 3 )
                     call errsub ( 1672, 3, 0, 0 )

                  end if

               end do
               istnodal = istnodal+inum+2
               jinp = jinp+inum+3

            else

!           --- File name with Nodal point numbers found

               istnodal = istnodal+2
               jinp = jinp+3

            end if

         case(11)

!        --- Volume record found, variable length

            call prord3copysf ( iinput, iinputlc, jinp, istvlm )

         case(12)

!        --- in_all_obstacles record found, 3 positions

            iinput(istallobst+1) = iinputlc(jinp+1)
            iinput(istallobst+2) = iinputlc(jinp+3)
            istallobst = istallobst+2
            jinp = jinp+3

         case(13)

!        --- in_inn_obstacles record found, 3 positions

            iinput(istinnobst+1) = iinputlc(jinp+1)
            iinput(istinnobst+2) = iinputlc(jinp+3)
            istinnobst = istinnobst+2
            jinp = jinp+3

         case(14)

!        --- in_boun_obstacles record found, 3 positions

            iinput(istbnobst+1) = iinputlc(jinp+1)
            iinput(istbnobst+2) = iinputlc(jinp+3)
            istbnobst = istbnobst+2
            jinp = jinp+3

         case(15)

!        --- on_boun_obstacles record found, 3 positions

            iinput(istonbnobst+1) = iinputlc(jinp+1)
            iinput(istonbnobst+2) = iinputlc(jinp+3)
            istonbnobst = istonbnobst+2
            jinp = jinp+3

         case(16)

!        --- inon_boun_obstacles record found, 3 positions

            iinput(istinonbnobst+1) = iinputlc(jinp+1)
            iinput(istinonbnobst+2) = iinputlc(jinp+3)
            istinonbnobst = istinonbnobst+2
            jinp = jinp+3

         case(17)

!        --- zero_levelset record found, 3 positions

            iinput(istzerolev+1) = iinputlc(jinp+1)
            iinput(istzerolev+2) = iinputlc(jinp+3)
            istzerolev = istzerolev+2
            jinp = jinp+3

         case(18)

!        --- cavity record found, 3 positions

            iinput(istcav+1) = iinputlc(jinp+1)
            iinput(istcav+2) = iinputlc(jinp+3)
            istcav = istcav+2
            jinp = jinp+3

         case(19)

!        --- intersection record found, inum+2 positions

            iinput(istinters+1) = iinputlc(jinp+1)
            iinput(istinters+2) = iinputlc(jinp+3)
            inum = iinputlc(jinp+4)
            iinput(istinters+3) = inum
            iinput(istinters+4:istinters+3+inum) = &
                iinputlc(jinp+5:jinp+4+inum)
            if ( debug ) &
               call prinin ( iinput(istinters+1), 3+inum, &
                             'iinput/intersection')
            istinters = istinters+3+inum
            jinp = jinp+4+inum
            if ( debug ) &
               write(irefwr,1) 'istinters, jinp', istinters, jinp

         case default

!        --- Error wrong value of ityperec

            call errint ( ityperec, 1 )
            call errchr ( 'ityperec', 1 )
            call errsub ( 3098, 1, 0, 1 )
            go to 1000

         end select  ! case ( ityperec )

      end do  ! while ( jinp<iinplc )

1000  call erclos ( 'prord3copy' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prord3copy'

      end if  ! ( debug )

      end
