      subroutine prord3curve ( iinputlc, iinplc, table, tabout, tabmin, &
                               tabmax, nescrv, lencrv )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-03-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Interprete curve information read by prord3 and store in iinputlc
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     curve
!     essential_boundary_condition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer  iinputlc(*), iinplc, nescrv, lencrv
      double precision table(*), tabout(*), tabmin(*), tabmax(*)

!     iinplc        i/o   Last position used in array iinputlc
!     iinputlc      i/o   integer work array where the information read is
!                         stored before processing
!     lencrv         i    Number of entries in array curves
!     nescrv         i    Number of parts where curve essential boundary
!                         conditions are given
!     table          i    Contains the numbers read for the various keywords
!                         Must be filled with the default values at input
!                         If keyword = text is considered, the number to be put
!                         in table is equal to the relative position in the
!                         table codemn, starting with 1
!                         Hence if the text is equal to codemn(tabmin(icode))
!                         table(icode) gets the value 1 etc.
!     tabmax         i    Contains maximum values allowed in table
!     tabmin         i    Contains minimum values allowed in table
!     tabout         i    Contains series of numbers
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iposut, inum, iexclu, lcur, iinplcbef

!     iexclu         Indicates which points are excluded
!     iinplcbef      Value of iinplc at start
!     inum           Number of numbers read in a record
!     iposut         Position in array tabout
!     lcur           Parameter l in curves record that has been read
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCRD         Prints the last record read and underlines the errorneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     558   The parameter l for the definition of essential boundary conditions
!           or connected elements has an incorrect value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prord3curve' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prord3curve'
         write(irefwr,1) 'iinplc', iinplc
         iinplcbef = iinplc
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nescrv = nescrv+1
      iinputlc(iinplc+2) = 2
      if ( table(3)>0d0 ) then

!     --- Standard case: curves ...

         iinputlc(iinplc+3) = nint(tabmin(3))
         lcur = iinputlc(iinplc+3)
         if ( lcur>0 ) then

!        --- lcur  (l in curves card)>0,  check exclude

            iexclu  =  lcur
            iexclu  =  iexclu / 100
            if ( iexclu>3 ) then
               call errint ( iexclu, 1 )
               call errint ( lcur, 2 )
               call errsub ( 558, 2, 0, 0 )
               call errcrd
               go to 1000
            end if

         end if  ! ( lcur>0 )
         iposut = nint(table(3))
         inum = nint(tabmax(3))
         iinputlc(iinplc+4) = inum
         iinputlc(iinplc+5:iinplc+4+inum) = &
            nint(tabout(iposut:iposut+inum-1))
         iinplc = iinplc+4+inum
         lencrv = lencrv+3+inum

      else  ! ( table(24)>0d0 )

!     --- Special case: outer_curves ...

         iinputlc(iinplc+3) = 0    ! all nodes are used
         iinputlc(iinplc+4) = -1   ! all outer curves
         iinplc = iinplc+4
         lencrv = lencrv+3

      end if  ! ( table(3)>0d0 )

1000  call erclos ( 'prord3curve' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iinplc', iinplc
         call prinin ( iinputlc(iinplcbef+1),iinplc-iinplcbef, &
                       'iinputlc' )
         write(irefwr,*) 'End prord3curve'

      end if  ! ( debug )

      end
