      subroutine prord4 ( iinput, maxlen, &
                          rinput, lengrl, maxrl, iinp, iinputlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version 12.2  date 10-02-2012 use getnpelmelgrp
!        version 12.1  date 13-11-2010 error 1488 --> errkeywexcl
!        version 12.0  date 27-09-2010 Remove work arrays
!
!   copyright (c) 1982-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read cards for local transformations
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), maxlen, lengrl, maxrl, iinp, iinputlc(*)
      double precision rinput(*)

!     iinp      i/o   Last position used in IINPUT
!     iinput    i/o   integer array to be filled with information concerning
!                     the local transformations
!                     iinput must have been filled before partly
!     iinputlc        integer work array where the information read is stored
!                     before processing
!     lengrl     i    Length of real array rinput
!                     (in double precision variables)
!     maxlen     i    Maximal space available for array IINPUT
!     maxrl      i    Maximal space available for array RINPUT
!     rinput     o    Real input array
!                     In this part information about the matrices MATRIXR
!                     and MATRIXV must be stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nloccv, nlocsf, ncount, i, jinp, &
              nodp, jnodp, ifunc, jtrans, ipos, &
              iinsign, nlocobs, nloccon, inpelm, jtype, idim2, istcrv, &
              istconct, istsrf, istobst,  lencrv, lenconct, lensrf, &
              lenobst, iinplc, jinplc, istc, nuserp, leng, &
              ncurvsact, iskipbn, numcurvs
      integer, parameter :: ncodsb=10, nncode2=19
      integer itype2(nncode2), ncode2(nncode2), ncodmx2(nncode2), &
              nscods(ncodsb)
      double precision tabmin2(nncode2), tabmax2(nncode2)
      character (len=15) codesb(ncodsb), code2(nncode2)

!     code2           Table with keywords
!     codesb         Table with subkeywords
!     i              Counting variable
!     idim2          ndim^2 i.e. 4 for ndim=2 and 9 for ndim=9
!     ifunc          Choice parameter with respect to function calls
!     iinplc         Last position used in array iinputlc
!     iinsign        Sign of curve number
!     inpelm         Number of points in an element
!     ipos           Position in array
!     iskipbn        If 1 the outer boundary of the surfaces must be
!                    skipped
!     istc           Starting address of C in table
!     istconct       Starting address of contact elements in iinput
!     istcrv         Starting address of curves in iinput
!     istobst        Starting address of obstacles in iinput
!     istsrf         Starting address of surfaces in iinput
!     itype2          Table containing information about the numbers behind
!                    the keywords in the table code2
!     jinp           position used in array iinput
!     jinplc         position used in array iinput
!     jnodp          Nodal point sequence number
!     jtrans         Type of local transformation
!                    1:  standard transformation
!                    2:  symmetric transformation
!                    3:  non-symmetric transformation
!     jtype          Type of record
!     lenconct       Length of contact part
!     lencrv         Length of curves part
!     leng           Length of array
!     lenobst        Length of obstacles part
!     lensrf         Length of surfaces part
!     ncode2          Table containing the number of significant characters in
!                    the keyword table code2
!     ncodmx2         Table containing the maximal number of  characters in
!                    the keyword table code2
!     ncodsb         Number of entries in codesb
!     ncount         Counter
!     ncurvsact      Actual number of curves to be used
!     nloccon        Number of contacts with local transforms
!     nloccv         Number of curves with local transforms
!     nlocobs        Number of obstacles with local transforms
!     nlocsf         Number of surfaces with local transforms
!     nncode2         Number of entries in code2
!     nodp           Nodal point sequence number
!     nscods         Table containing the number of significant characters in
!                    the subkeyword table codesb
!     numcurvs       Number of curves
!     nuserp         Number of user points
!     tabmax2         Contains maximum values allowed in table
!     tabmin2         Contains minimum values allowed in table
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRKEYWEXCL    Print error message concerning two keywords that are
!                    mutually exclusive
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     PRTS00         Check icurv1, icurv2
!     PRTS01         Check isurf1, isurf2
!     SEPCOPYCODE    Copy code2 and related arrays into code
!     SEPCOPYCODESUB Copy codemn2 and related arrays into codesub
!     SEPCOPYTYPE    Copy itype2 and related arrays into itype
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!    input is read from the input file.
!    The following texts are recognised:
!     [degrees of freedom] [location part] [transformation_information]
!     in arbitrary order
!     Degrees of freedom has the following shape
!     DEGFD k [,DEGFD l [, DEGFD m ....] ]
!     If omitted DEGFD 0 is assumed
!     The location part has one of the following structures:
!     CURVES [l] (Cj [to Cm] )
!     SURFACES [i1, i2, i3, ... ] (Sj [to Sm] ), SKIP_BOUNDARY//
!        SKIP_CURVES (Ci, Cj, ... ), INCLUDE_CURVES (Ci, Cj, ... )//
!        TANG_CURVE = (k_1, k_2, .. )
!     OBSTACLE i, JSTEP = j
!     CONTACT i, JSTEP = j
!     Transformation information has the following shape:
!     TRANSFORMATION = s, &
!     MATRIXR = ( r1, r2, r3, r4 ), or ( r1, r2, r3, r4, r5, r6, r7, r8, r9 )
!     MATRIXV = ( v1,v2,v3,v4)
!     TANG_DIR = LINE(P1,P2) or ZDIR or XDIR or YDIR
!     NORMAL = o
!     All these items must be in one line
!     s may have one of the following shapes:
!     STANDARD
!     SYMMETRIC
!     NON_SYMMETRIC
!     MIN_UN_UT
!     o may have one of the following shapes:
!     INWARD
!     OUTWARD
!     ri and vi may be a number, or VALUE=v of FUNC=i
!    main keywords ( see prorea )
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      29   Too few space for standard problems in IINPUT
!     269   Relative nodal point number in surface record incorrect
!     280   Double point found in surface record
!     332   record with curves contains too much degrees of freedom
!     333   record with surfaces contains too much degrees of freedom
!     424   first user point number (ifirst) has incorrect value
!     425   last point number (ilast) has incorrect value
!     550   array length too short
!     892   No number or more than two numbers found after CURVES [i] = (
!    1136   Number of surfaces found incorrect
!    1522   Both curves and surfaces found
!    1651   obstacles with curves or surfaces in one record found
!    1920   MATRIXR or MATRIXV found but not TRANSFORMATION
!    1921   MATRIXR or MATRIXV found in combination with
!           TRANSFORMATION = STANDARD
!    1922   MATRIXV found in combination with TRANSFORMATION = SYMMETRIC
!    1923   MATRIXR has not been found
!    1924   MATRIXV has not been found
!    2011   Tang found in combination with (non-)symmetric
!    2012   Not two user points found in combination with tang
!    2061   The same point is used for both parts of the tangential vector
!    2192   no curves found
!    2820   no curves found after tang_curve
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code2 / 'DEGFD', 'CURVES', 'SURFACES', 'TRANSFORMAT', &
                  'MATRIXR', 'MATRIXV', 'TO', 'OBSTACLE', &
                  'JSTEP', 'CONTACT', 'TANG_CURVE', 'SKIP_BOUNDARY', &
                  'NORMAL', 'SKIP_CURVES', 'INCLUDE_CURVES', &
                  'TANG_DIR', 'C', 'S', 'P' /
      data ncode2 /  3, 4, 4, 4, 7,    7, 2, 4, 4, 4, &
                    9, 8, 4, 8,10,    4, 1, 1, 1 /
      data ncodmx2/  5, 6, 8,14, 7,    7, 2, 8, 5, 7, &
                   11,13, 6,11,14,    8, 1, 1, 1 /
      data itype2 / 14, 4,11, 5,27,   27,-1, 0, 0, 0, &
                   11, 2, 5, 2, 2,    5,14,14,14 /
      data tabmin2 / 3d0, 0d0, 0d0, 0d0, 4d0,  4d0, 0d0, 1d0,-2d0, 1d0, &
                    1d0, 0d0, 0d0, 0d0, 0d0,  0d0, 1d2, 2d0, 2d0 /
      data tabmax2 / 1d2, 5d2, 0d0, 0d0, 4d0,  4d0, 0d0, 1d2, 2d0,10d0, &
                    1d0, 0d0, 0d0, 0d0, 0d0,  0d0, 1d3, 1d3, 1d3 /
      data codesb / 'STANDARD', 'SYMMETR', 'NON_SYMMETRIC', 'MIN_UN_UT', &
                    'LINE', 'XDIR', 'YDIR', 'ZDIR', 'OUTWARD', &
                    'INWARD' /
      data nscods /  4, 4, 7, 9, 4,  4, 4, 4, 4, 3 /

!     code2     Table of local keywords
!              1: DEGFD
!              2: CURVES
!              3: SURFACES
!              4: TRANSFORMATION
!              5: MATRIXR
!              6: MATRIXV
!              7: TO
!              8: OBSTACLE
!              9: JSTEP
!             10: CONTACT
!             11: TANG_CURVE
!             12: SKIP_BOUNDARY
!             13: NORMAL
!             14: SKIP_CURVES
!             15: INCLUDE_CURVES
!             16: TANG_DIR
!             17: C
!             18: S
!             19: P
!     codesb   Table of local subkeywords
!              1: STANDARD
!              2: SYMMETRIC
!              3: NON_SYMMETRIC
!              4: MIN_UN_UT
!              5: LINE
!              6: XDIR
!              7: YDIR
!              8: ZDIR
!              9: OUTWARD
!             10: INWARD
! ======================================================================
!
      call eropen ( 'prord4' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prord4'
         write(irefwr,1) 'iinp', iinp
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      jinp = iinp
      iinplc = 0
      nloccv = 0
      nlocsf = 0
      nlocobs = 0
      nloccon = 0
      lencrv = 0
      lensrf = 0
      lenconct = 0
      lenobst = 0

!     --- Copy information into codesub

      call sepcopycodesub ( codesb, ncodsb, nscods )
      call sepcopycode ( code2, nncode2, ncode2, ncodmx2 )
      call sepcopytype ( nncode2, itype2, tabmin2, tabmax2 )

      icode = 0

!     --- while loop (icode=0)

200   if ( icode==0 ) then
         ityprd = 1
         table(1:nncode) = -1d20
         table(9) = 0d0
         tabmax(1) = 100d0
         tabmin(3) = npelm
         tabmax(3) = npelm
         tabmin(4) = 1
         tabmax(4) = tabmin(4)+3d0
         tabmin(5) = ndim**2
         tabmin(6) = ndim**2
         tabmin(11) = ncurvs   ! at most ncurvs numbers
         tabmax(11) = ncurvs   ! maximumum number is ncurvs
         tabmin(16) = tabmax(4)+1d0
         tabmax(16) = tabmin(16)+3d0
         tabmin(13) = tabmax(16)+1d0
         tabmax(13) = tabmin(13)+1d0
         istc = 17
         table(istc) = 0d0
         table(istc+1) = 0d0
         table(istc+2) = 0d0
         tabmax(istc) = ncurvs
         tabmax(istc+1) = nsurfs
         tabmax(istc+2) = nuspnt
         nmain = nncodemain
         iretur = icode
         call sepreadsetkeyw
         icode = iretur
         if ( icode/=0 ) go to 200
         jinplc = iinplc

!        --- interpret curves part

         if ( table(4)>0 ) then
            jtrans = nint(table(4))
         else
            jtrans = 1
         end if
         if ( table(2)>-1d18 ) then

!        --- curves record found

            nloccv = nloccv+1
            jtype = 1
            iinplc = iinplc+1
            iinputlc(iinplc) = jtype
            iposut = nint(table(istc))
            ncount = nint(tabmax(istc))
            if ( ncount==0 .or.ncount>2 ) then

!           --- No curve numbers, or more than 2 found

               call errint ( ncount, 1 )
               call errsub ( 892, 1, 0, 0 )

            else

!           --- One or two curve numbers found

               iinputlc(iinplc+ndim+2) = nint(tabout(iposut))
               if ( ncount==1 ) then
                  iinputlc(iinplc+ndim+3) = iinputlc(iinplc+ndim+2)
               else
                  iinputlc(iinplc+ndim+3) = nint(tabout(iposut+1))
               end if
               if ( iinputlc(iinplc+ndim+2)<0 .and. &
                    iinputlc(iinplc+ndim+3)<0 ) then
                  iinputlc(iinplc+ndim+2) = -iinputlc(iinplc+ndim+2)
                  iinputlc(iinplc+ndim+3) = -iinputlc(iinplc+ndim+3)
                  iinsign = -1
               else
                  iinsign = 1
               end if
               call prts00 ( iinputlc(iinplc+ndim+2), &
                             iinputlc(iinplc+ndim+3), ncurvs )
               if ( iinsign<0 ) then
                  iinputlc(iinplc+ndim+2) = -iinputlc(iinplc+ndim+2)
                  iinputlc(iinplc+ndim+3) = -iinputlc(iinplc+ndim+3)
               end if

!              --- Fill default values in iinput

               iinputlc(iinplc+1) = 1
               iinputlc(iinplc+2) = 2
               if ( ndim==3 ) iinputlc(iinplc+3) = 3
               iinputlc(iinplc+4+ndim) = jtrans
               if ( table(2)>0 ) then

!              --- integer found after curves

                  iinputlc(iinplc+ndim+1) = nint(tabout(nint(table(2))))

               else

!              --- no integer found after curves

                  iinputlc(iinplc+ndim+1) = 0

               end if

            end if
            if ( ndim==2 ) then
               lencrv = lencrv+6
               if ( jtrans==2 .or. jtrans==3 ) &
                  lencrv = lencrv+4*(jtrans-1)
            else
               lencrv = lencrv+7
               if ( jtrans==2 .or. jtrans==3 ) &
                  lencrv = lencrv+9*(jtrans-1)
            end if

         end if

!        --- interpret surfaces part

         ncurvsact = 0
         if ( table(3)>-1d18 ) then

!        --- surfaces record found

            nlocsf = nlocsf+1
            jtype = 2
            iinplc = iinplc+1
            iinputlc(iinplc) = jtype
            if ( table(2)>-1d-18 ) then
               call errsub ( 1522, 0, 0, 0 )
            end if
            iposut = nint(table(istc+1))
            ncount = nint(tabmax(istc+1))
            if ( ncount==0 .or. ncount>2 ) then

!           --- No surface numbers, or more than 2 found

               call errint ( ncount, 1 )
               call errsub ( 1136, 1, 0, 0 )

            else

!           --- One or two surface numbers found

               iinputlc(iinplc+4) = nint(tabout(iposut))
               if ( ncount==1 ) then
                  iinputlc(iinplc+5) = iinputlc(iinplc+4)
               else
                  iinputlc(iinplc+5) = nint(tabout(iposut+1))
               end if
               call prts01 ( iinputlc(iinplc+4), iinputlc(iinplc+5), &
                             nsurfs )
               inpelm = sepgetinpelsurf ( iinputlc(iinplc+4), 0 )

!              --- Fill default values in iinputlc

               iinputlc(iinplc+1) = 1
               iinputlc(iinplc+2) = 2
               iinputlc(iinplc+3) = 3
               iinputlc(iinplc+6) = jtrans
               iinputlc(iinplc+7) = 0
               iinputlc(iinplc+8) = 0
               if ( table(12)>=0d0 ) then

!              --- skip_boundary found

                  iinputlc(iinplc+7) = 1
                  if ( table(14)>=0d0 ) &
                     call errkeywexcl ( code, ncodmx, 12, 14 )
                  if ( table(15)>=0d0 ) &
                     call errkeywexcl ( code, ncodmx, 12, 15 )

               else if ( table(14)>=0d0 ) then

!              --- skip_curves found

                  iinputlc(iinplc+7) = 2
                  if ( table(15)>=0d0 ) &
                     call errkeywexcl ( code, ncodmx, 14, 15 )

               else if ( table(15)>=0d0 ) then

!              --- include_curves found

                  iinputlc(iinplc+7) = 3

               end if

               if ( debug ) then
                  write(irefwr,1) 'iinplc', iinplc
                  call prinin ( iinputlc(iinplc+1), 8, 'iinputlc_sf/1' )
               end if  ! ( debug )

               iskipbn = iinputlc(iinplc+7)
               if ( table(13)>=0d0 ) iinputlc(iinplc+7) = &
                   iinputlc(iinplc+7)+10*(nint(table(13))-1)
               do i = 1, inpelm
                  iinputlc(iinplc+8+i) = 1
               end do
               if ( tabmax(3)>0 ) then

!              --- integers found after surfaces

                  iposut = nint(table(3))
                  ncount = nint(tabmax(3))
                  do i = 1,inpelm
                     iinputlc(iinplc+8+i) = 0
                  end do
                  do i = 1, ncount
                     nodp = nint(tabout(iposut+i-1))
                     if ( nodp<=0 .or. nodp>inpelm ) then
                        call errint ( nodp, 1 )
                        call errint ( inpelm, 2 )
                        call errsub ( 269, 2, 0, 0 )
                     end if
                     jnodp = iinputlc(iinplc+8+i)
                     if ( jnodp>0 ) then

!                    --- jnodp >0  double point

                        call errint ( i, 1 )
                        call errint ( nodp, 2 )
                        call errsub ( 280, 2, 0 , 0)

                     else

!                    --- jnodp   =  0

                        iinputlc(iinplc+8+i) = 1

                     end if

                  end do

               end if

            end if
            if ( debug ) then
               write(irefwr,1) 'iinplc', iinplc
               call prinin ( iinputlc(iinplc+1), 8+inpelm, &
                             'iinputlc_sf/2' )
            end if  ! ( debug )
            lensrf = lensrf+8+inpelm+9*(jtrans-1)
            if ( table(11)>0 ) then

!           --- TANG_CURVE found

               numcurvs = nint(tabmax(11))  ! number of curves read
               if ( numcurvs==0 ) then

!              --- error, no curves found after tang_curve

                  call errsub ( 2820, 0, 0, 0 )

               end if  ! ( numcurvs==0 )
               lensrf = lensrf + 1+numcurvs ! 1+numcurv extra positions required
                                  ! the first one to store the number of curves
                                  ! the other ones to store the curves
               if ( jtrans>1 ) then

!              --- Error jtrans # 1

                  call errsub ( 2011, 0, 0, 0 )

               end if  ! ( jtrans>1 )

               if  ( table(16)>0 ) call errkeywexcl ( code, ncodmx, 11, 16 )

!              --- Fill information

               iinputlc(iinplc+8) = 10   ! set TANG = 10
               iinputlc(iinplc+9+inpelm) = numcurvs   ! number of curves

!              --- Fill curve numbers

               iposut = nint(table(11))
               do i = 0, numcurvs-1
                  iinputlc(iinplc+10+i+inpelm) = nint(tabout(iposut+i))
               end do  ! i = 1, numcurvs
               if ( debug ) then
                  write(irefwr,1) 'iinplc, numcurvs', iinplc, numcurvs
                  call prinrl ( tabout(iposut), numcurvs, 'curvenrs' )
                  call prinin ( iinputlc(iinplc+1), 9+inpelm+numcurvs, &
                                'iinputlc_sf/3' )
               end if  ! ( debug )

            else if ( table(16)>0 ) then

!           --- TANG_DIR found

               if ( jtrans>1 ) then

!              --- Error jtrans # 1

                  call errsub ( 2011, 0, 0, 0 )

               else

!              --- TANG_DIR found, now look for p1, p2

                  lensrf = lensrf + 2  ! two extra positions required
                                       ! containing the user points
                  iinputlc(iinplc+8) = nint(table(16))  !  itang
                  iinputlc(iinplc+9+inpelm) = 0
                  iinputlc(iinplc+10+inpelm) = 0
                  if ( iinputlc(iinplc+8)==1 ) then

!                 --- itang = 1, hence line = p1, p2; find user points

                      iposut = nint(table(istc+2))
                      ncount = nint(tabmax(istc+2))
                      if ( ncount/=2 ) then

!                     --- Not two user points found

                         call errint ( ncount, 1 )
                         call errsub ( 2012, 1, 0, 0 )

                      else

!                     --- Two user points found

                         jnodp = nint(tabout(iposut))
                         if ( jnodp<=0 .or. jnodp>nuspnt ) then
                            call errint ( jnodp, 1 )
                            call errint ( nuspnt, 2 )
                            call errsub ( 424, 2, 0, 0 )
                         end if
                         iinputlc(iinplc+9+inpelm) = jnodp
                         jnodp = nint(tabout(iposut+1))
                         if ( jnodp<=0 .or. jnodp>nuspnt ) then
                            call errint ( jnodp, 1 )
                            call errint ( 1, 2 )
                            call errint ( nuspnt, 3 )
                            call errsub ( 425, 3, 0, 0 )
                         end if
                         if ( jnodp==iinputlc(iinplc+9+inpelm) ) then
                            call errint ( jnodp, 1 )
                            call errsub ( 2061, 1, 0, 0 )
                         end if
                         iinputlc(iinplc+10+inpelm) = jnodp

                      end if

                  end if  ! ( iinputlc(iinplc+8)==1 )

               end if

            end if
            if ( iskipbn>1 ) then

!           --- iskip_boundary>1, curves needed

               ncurvsact = nint(tabmax(istc))
               if ( ncurvsact<1 ) then

!              --- No curves found

                  ipos = iskipbn+12
                  call errchr ( code(ipos)(1:ncodmx(ipos)), 1 )
                  call errsub ( 2192, 0, 0, 1 )

               end if
               iposut = nint(table(istc))
               ipos = iinplc+8+inpelm+1 +9*(jtrans-1)
               if ( debug ) write(irefwr,1) 'ipos/1', ipos
               if ( table(16)>0d0 ) then
                  ipos = ipos+2  ! raise ipos in case of tangent
               else if ( table(11)>0d0 ) then
                  ipos = ipos+1+numcurvs  ! raise ipos in case of tangent
               end if  ! ( table(16)>0d0 )
               if ( debug ) write(irefwr,1) 'ipos/2', ipos
               iinputlc(ipos) = ncurvsact
               do i = 1, ncurvsact
                  iinputlc(ipos+i) = nint(tabout(iposut+i-1))
               end do
               lensrf = lensrf+ncurvsact+1
               if ( debug ) then
                  write(irefwr,1) 'ipos', ipos
                  call prinin ( iinputlc(iinplc+1), &
                                ipos+ncurvsact-jinplc, 'iinputlc_sf/4' )
               end if

            end if

         end if

!        --- interpret obstacles part

         if ( table(8)>-1d18 ) then

!        --- obstacles record found

            jtype = 3
            iinplc = iinplc+1
            iinputlc(iinplc) = jtype
            nlocobs = nlocobs+1
            if ( table(2)>-1d-18 .or. table(3)>-1d-18 ) then
               call errsub ( 1651, 0, 0, 0 )
            end if
            iinputlc(iinplc+5) = nint(table(8))
            iinputlc(iinplc+1) = 1
            iinputlc(iinplc+2) = 2
            iinputlc(iinplc+3) = 0
            iinputlc(iinplc+4) = nint(table(9))
            iinputlc(iinplc+6) = jtrans
            lenobst = lenobst+6
            if ( jtrans==2 .or. jtrans==3 ) lenobst = lenobst+4*(jtrans-1)

         end if

!        --- interpret contact part

         if ( table(10)>-1d18 ) then

!        --- contact record found

            jtype = 4
            iinplc = iinplc+1
            iinputlc(iinplc) = jtype
            nloccon = nloccon+1
            if ( table(2)>-1d-18 .or. table(3)>-1d-18 ) then
               call errsub ( 1651, 0, 0, 0 )
            end if
            iinputlc(iinplc+5) = nint(table(10))
            iinputlc(iinplc+1) = 1
            iinputlc(iinplc+2) = 2
            iinputlc(iinplc+3) = 0
            iinputlc(iinplc+4) = nint(table(9))
            iinputlc(iinplc+6) = jtrans
            lenconct = lenconct+6
            if ( jtrans==2 .or. jtrans==3 ) &
               lenconct = lenconct+4*(jtrans-1)

         end if

!        --- interpret degfd part

         if ( table(1)>0d0 ) then

!        --- degfd found

            iposut = nint(table(1))
            ncount = nint(tabmax(1))
            if ( ndim==2 ) then

!           --- curves record

               if ( ncount/=2 ) then
                  call errint ( ncount, 1 )
                  call errint ( 2, 2 )
                  call errint ( 2, 3 )
                  call errsub ( 332, 3, 0, 0 )
               else
                  iinputlc(iinplc+1) = nint(tabout(iposut))
                  iinputlc(iinplc+2) = nint(tabout(iposut+1))
               end if

            else

!           --- surfaces record

               if ( ncount/=3 ) then
                  call errint ( ncount, 1 )
                  call errint ( 3, 2 )
                  call errint ( 2, 3 )
                  call errsub ( 333, 3, 0, 0 )
               else
                  iinputlc(iinplc+1) = nint(tabout(iposut))
                  iinputlc(iinplc+2) = nint(tabout(iposut+1))
                  iinputlc(iinplc+3) = nint(tabout(iposut+2))
               end if

            end if

         end if

!        --- Raise iinplc in order to get to tranformation part

         jinplc = iinplc
         if ( jtype==1 ) then

!        --- Curve record is found

            iinplc = iinplc+4+ndim
            idim2 = ndim**2

         else if ( jtype==2 ) then

!        --- Surface record is found

            iinplc = iinplc+8+inpelm
            if ( table(16)>0d0 ) then
               iinplc = iinplc+2  ! 2 extra positions in case of tangent given
            else if ( table(11)>0d0 ) then
               iinplc = iinplc+1+numcurvs  ! 1+numcurvs extra positions
            end if  ! ( table(16)>0d0 ) then
            if ( ncurvsact>0d0 ) iinplc = iinplc+1+ncurvsact
            idim2 = 9

         else

!        --- Other type of record found

            iinplc = iinplc+6
            idim2 = ndim**2

         end if

!        --- interpret transformation part

         if ( table(4)>0 ) then

!        --- transformation found

            jtrans = nint(table(4))
            iinputlc(jinplc+6) = jtrans
            if ( ndim==2 ) then
               jinplc = jinplc+6
            else
               jinplc = jinplc+8+inpelm
            end if
            if ( jtrans==1 .or. jtrans==4 ) then

!           --- standard transformation

               if ( table(5)>0d0 .or. table(6)>0d0 ) then
                  call errsub ( 1921, 0, 0, 0 )
               end if

            else if ( jtrans>=2 ) then

!           --- (non)-symmetric transformation

               if ( table(5)>0d0 ) then
                  iposut = nint(table(5))
                  do i = 1, idim2
                     ifunc = nint(tabout(iposut+2*i-2))
                     if ( ifunc>0 ) then
                        iinputlc(jinplc+i) = -ifunc
                     else if ( ifunc<0 ) then
                        iinputlc(jinplc+i) = ifunc
                     else
                        lengrl = lengrl+1
                        iinputlc(jinplc+i) = lengrl
                        rinput(lengrl) = tabout(iposut+2*i-1)
                     end if
                  end do
                  iinplc = iinplc+idim2
                  jinplc = jinplc+idim2

               else
                  call errsub ( 1923, 0, 0, 0 )
               end if

               if ( jtrans==2 ) then

!              --- symmetric transformation

                  if ( table(6)>0d0 ) then
                     call errsub ( 1922, 0, 0, 0 )
                  end if

               else

!              --- non symmetric transformation

                  if ( table(6)>0d0 ) then
                     iposut = nint(table(6))
                     do i = 1, idim2
                        ifunc = nint(tabout(iposut+2*i-2))
                        if ( ifunc>0 ) then
                           iinputlc(jinplc+i) = -ifunc
                        else if ( ifunc<0 ) then
                           iinputlc(jinplc+i) = ifunc
                        else
                           lengrl = lengrl+1
                           iinputlc(jinplc+i) = lengrl
                           rinput(lengrl) = tabout(iposut+2*i-1)
                        end if
                     end do
                     iinplc = iinplc+idim2

                  else
                     call errsub ( 1924, 0, 0, 0 )
                  end if

               end if

            end if

         else if ( table(5)>0d0 .or. table(6)>0d0 ) then

!        --- MATRIXR/V found and not TRANSFORMATION

            call errsub ( 1920, 0, 0, 0 )

         end if
         go to 200

      end if

      if ( iinplc>maxlen ) then

!     --- Too much information, i.e. maxlen too short

         call errchr ( 'iinputlc', 1 )
         call errint ( iinplc, 1 )
         call errint ( maxlen, 2 )
         call errsub ( 550, 2, 0, 1 )
         go to 1000

      end if

!     --- Copy all input in the correct sequence into iinput

      istcrv   = iinp + 2
      istsrf   = istcrv + lencrv
      istobst  = istsrf + lensrf
      istconct = istobst + lenobst
      iinp     = istconct + lenconct
      jinplc = 1

400   if ( jinplc<iinplc ) then

!     --- The last record has not yet been stored

         jtype = iinputlc(jinplc)
         if ( jtype==1 ) then

!        --- curves record

            jtrans = iinputlc(jinplc+4+ndim)
            if ( ndim==2 ) then

               leng = 6
               if ( jtrans==2 .or. jtrans==3 ) leng = leng + 4*(jtrans-1)
            else

               leng = 7
               if ( jtrans==2 .or. jtrans==3 ) leng = leng + 9*(jtrans-1)

            end if
            do i = 1, leng
               iinput(istcrv+i) = iinputlc(jinplc+i)
            end do
            istcrv = istcrv+leng
            jinplc = jinplc+leng+1

         else if ( jtype==2 ) then

!        --- surface record

            jtrans = iinputlc(jinplc+6)
            inpelm = sepgetinpelsurf ( iinputlc(jinplc+4), 0 )
            iskipbn = mod(iinputlc(jinplc+7),10)
            if ( iinputlc(jinplc+8)>0 .and. iinputlc(jinplc+8)<10 ) then

!           --- 1<=itang<10, use two extra positions

               nuserp = 2

            else if ( iinputlc(jinplc+8)>9 ) then

!           --- itang>=10, use numcurvs+1 extra positions

               nuserp = 1+iinputlc(jinplc+9+inpelm)

            else

!           --- itang = 0, no extra positions required

               nuserp = 0

            end if  ! ( iinputlc(jinplc+8)>0 )
            if ( iskipbn>1 ) then

!           --- curves found, raise nuserp

               ncurvsact = &
                  iinputlc(jinplc+8+9*(jtrans-1)+nuserp+inpelm+1)
               nuserp = nuserp+ncurvsact+1

            end if
            do i = 1, 8+9*(jtrans-1)+nuserp+inpelm
               iinput(istsrf+i) = iinputlc(jinplc+i)
            end do
            istsrf = istsrf+8+9*(jtrans-1)+nuserp+inpelm
            jinplc = jinplc+9+9*(jtrans-1)+nuserp+inpelm

         else if ( jtype==3 ) then

!        --- obstacles record

            jtrans = iinputlc(jinplc+6)
            do i = 1, 6+4*(jtrans-1)
               iinput(istobst+i) = iinputlc(jinplc+i)
            end do
            istobst = istobst+6+4*(jtrans-1)
            jinplc = jinplc+7+4*(jtrans-1)

         else

!        --- contact record

            jtrans = iinputlc(jinplc+6)
            do i = 1, 6+4*(jtrans-1)
               iinput(istconct+i) = iinputlc(jinplc+i)
            end do
            istconct = istconct+6+4*(jtrans-1)
            jinplc = jinplc+7+4*(jtrans-1)

         end if

         go to 400

      end if

      if ( iinp>=maxlen ) then

!     --- iinp>maxlen, too few space in IINPUT

         call errint ( iinp, 1 )
         call errint ( maxlen, 2 )
         call errsub ( 29, 2, 0, 0 )
         call errcrd

      end if

      iinput(9) = 1
      iinput(18) = nlocobs
      iinput(jinp+1) = nloccv
      iinput(jinp+2) = nlocsf
      if ( lengrl>maxrl ) then
         call errint ( lengrl, 1 )
         call errint ( maxrl, 2 )
         call errsub ( 29, 2, 0, 0 )
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'iinp, jinp', iinp, jinp
         call prinin ( iinput, 20, 'iinput/1' )
         call prinin ( iinput(jinp+1), iinp-jinp, 'iinput/2' )
         call prinin ( iinputlc, iinplc, 'iinputlc' )
         write(irefwr,*) 'End prord4'

      end if
      call erclos ( 'prord4' )

      end
