      subroutine prord6 ( iinput, maxlen, iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.5  date 03-03-2008 adaptation to new readsr
!        version  4.4  date 27-02-2006 Use reorder instead of REORDER
!        version  4.3  date 27-06-2003 Correction for plast
!
!   copyright (c) 1989-2008  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     read reordering with respect to unknowns (icode=6)
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmoduleread1
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), maxlen, iinp

!     iinp      i/o   Last position used in IINPUT
!     iinput    i/o   integer array to be filled with information concerning
!                     the REORDERing
!                     iinput must have been filled before partly
!     maxlen     i    Maximal space available for array IINPUT
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, ih, inum, iprio, maxun
      logical levels, openbr

!     i              Loop variable
!     ih             Help variabale to store the number of the degree of freedom
!     inum           Number of numbers read in a record
!     iprio          Next priority for unknown
!     levels         If true levels has been read
!     maxun          Number of degrees of freedom read
!     openbr         If true, an open bracket has been found
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer intx01

!     ASKNXT         Read next item from the standard input file
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRCRD         Prints the last record read and underlines the erroneous
!                    part
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INTX01         Compare text in string with table (fixed length)
!     READSR         Read a series of numbers from the input (texts are
!                    neglected)
! **********************************************************************
!
!                       I/O
!
!    input is read from the input file.
!    The following texts are recognised:
!       REORDER [levels] 1,3, (4,2), 6, 5
!       or REORDER plast
!    main keywords ( see prorea )
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      29   Too few space for standard problems in IINPUT
!     839   Degree of freedom has incorrect value
!     840   Number of numbers read after REORDER is incorrect
!     841   The next recrd read is not END, whereas no input is expected after
!           REORDER
!    1868   Text not recognized
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prord6' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prord6'
         write(irefwr,1) 'ityprd/1, lastitem', ityprd, lastitem
         if ( ityprd==4 ) write(irefwr,*) 'string ', string(1:nchar)

  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( work(1500) )

      lastitem = max(lastitem,1)  ! prevent reading reorder twice

!     --- Clear array work positions 1 to 100
!         It is supposed that the maximal number of degrees of freedom is 100

      work(1:100) = 0d0

!     --- Check if levels is present in REORDER statement

      levels = .false.
      if ( nchar>10 ) then

!     --- REORDER followed by characters, for example REORDER_levels

         if ( string(1:11)=='REORDER_LEV' .or. &
              string(1:12)=='RENUMBER_LEV' ) levels = .true.

      else if ( lastitem<nitems ) then

!     --- There is at least one item following REORDER, check if text

         if ( typeitems(lastitem+1)==3 ) then

!        --- Text found, check if this is the text levels or plast

            call asknxt

            if ( debug ) then
               write(irefwr,1) 'ityprd/2', ityprd
               if ( ityprd==4 ) &
                  write(irefwr,*) 'string ', string(1:nchar)
            end if  ! ( debug )

            if ( string(1:3)=='LEV' ) then

!           --- Levels found, set levels true

               levels = .true.

            else if ( string(1:4)=='PLAS' ) then

!           --- plast found: set iinput(14) 1 and iinput(iinp+1) -1

               iinput(14) = 1
               iinput(iinp+1) = -1
               iinp = iinp+1
               maxun = 0
               go to 500

            else

!           --- Unrecognizable text found: error

               call errchr ( string(1:20), 1 )
               call errsub ( 1868, 0, 0, 1 )
               call errcrd

            end if

         end if

      end if

!     --- read numbers from record

      iprio = 1
      maxun = 0

!     --- First check if open bracket is found

200   ispecl = 5
      openbr = .false.
      call asknxt
      if ( ityprd==8 ) openbr = .true.   ! Open bracket found
      lastitem = lastitem-1  ! reset last item

      ispecl = 0
      call readsr ( work(101), 100, inum )
      if ( ierror/=0 ) go to 1000

      if ( inum/=0 ) then

!     --- New set of values found

         do i = 1, inum
            ih = nint ( work(100+i) )
            if ( ih<=0 .or. ih>100 ) then

!           --- ih<=0 or ih>100, which means that degrees of freedom
!               are out of range

               call errint ( ih, 1 )
               call errint ( 100, 2 )
               call errsub ( 839, 2, 0, 0 )
               call errcrd

            else

               work(ih) = iprio

            end if
            if ( .not. openbr ) iprio = iprio+1

            maxun = max ( maxun, ih )

         end do

         if ( openbr ) iprio = iprio + 1
         if ( lastitem<nitems ) go to 200

      end if

      if ( levels ) then
         iinput(14) = -maxun
      else
         iinput(14) = maxun
      end if
      if ( iinp+maxun>maxlen ) then
         call errint ( iinp+maxun, 1 )
         call errint ( maxlen, 2 )
         call errsub ( 29, 2, 0, 0 )
      end if
      if ( maxun==0 .or. maxun>100 ) then
         call errint ( maxun, 1 )
         call errint ( 100, 2 )
         call errsub ( 840, 2, 0, 0 )
      end if

!     --- Read new card (END)

500   ityprd = 1
      call asknxt

      icode = 0
      ierpri = 0
      if ( ityprd==4 ) icode = sepmainkeyw () ! check main keyword

      if ( icode==0 ) then

!     --- Wrong record found: error

         call errsub ( 841, 0, 0, 0 )
         call errcrd

      end if
      if ( ierror/= 0 ) go to 1000

!     --- Fill information in iinput

      do i = 1, maxun
         if ( work(i)>0 ) then

            iinput(iinp+i) = nint(work(i))

         else

            iinput(iinp+i) = iprio

         end if
      end do
      iinp = iinp + maxun

      deallocate ( work )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prord6'

      end if  ! ( debug )
      call erclos ( 'prord6' )

      end subroutine prord6
