      subroutine prord9 ( iinput, maxlen, iinp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 27-09-2010 Remove work arrays
!        version  1.0  date 28-12-1998
!
!   copyright (c) 1998-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   read cards for global renumbering
!
! **********************************************************************
!
!                       KEYWORDS
!
!     input
!     problem_definition
!     global_unknown
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleread1
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), maxlen, iinp

!     iinp      i/o   Last position used in IINPUT
!     iinput    i/o   integer array to be filled with information concerning
!                     the global elements
!     maxlen     i    Maximum amount of contiguous space in IBUFFR after a
!                     starting address
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, ideg, idegfd, ndeg, iwork(20)

!     i              Counting variable
!     ideg           Degree of freedom read
!     idegfd         Degree of freedom sequence number
!     iwork          integer work array of length 20
!     ndeg           Nuber of degrees of freedom
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPREADSETKEYW Interpret a record containing keywords followed by data
! **********************************************************************
!
!                       I/O
!
!    input is read from the input file.
!    The following texts are recognized:
!       degfdi, degfd_j, ...
!    main keywords ( see prorea )
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      29   Too few space for standard problems in IINPUT
!    2030   Degree of freedom found more than once
! **********************************************************************
!
!                       PSEUDO CODE
!
!    Read all input until main keyword is found, store in iinputlc
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prord9' )

      ityprd = 1
      nmain = nncodemain
      code(1) = 'DEGFD'
      ncode(1) = 3
      ncodmx(1) = 5
      itype(1) = 14
      nncode = 1

!     --- Read record with information about renumbering

      idegfd = 0
      iwork(1:20) = 0
100   table(1) = -1d20
      tabmin(1) = 20d0
      tabmax(1) = 20d0
      iretur = 0
      call sepreadsetkeyw
      icode = iretur
      if ( icode==0 ) then

!     --- Not a main keyword found, hence DEGFD = ...

         if ( table(1)>0 ) then
            ndeg = nint(tabmax(1))
            iposut = nint(table(1))
         else
            ndeg = 0
         end if
         do i = 1, ndeg
            ideg = nint(tabout(iposut+i-1))
            if ( iwork(ideg)>0 ) then

!           --- ideg found more than once, this one skipped

               call errint ( ideg, 1 )
               call errsub ( 2030, 1, 0, 0 )

            else

               idegfd = idegfd + 1

            end if
            iwork(ideg) = 1

         end do
         go to 100

      else

!     --- Main keyword found, put information in iinputlc

         iinp = iinp+1
         iinput(iinp) = idegfd
         do i = 1, 20
            if ( iwork(i)/=0 ) then
               iinp = iinp+1
               iinput(iinp) = i
            end if
         end do

      end if
      if ( iinp>maxlen ) then
         call errint ( iinp, 1 )
         call errint ( maxlen, 2 )
         call errsub ( 29, 2, 0, 0 )
      end if
      call erclos ( 'prord9' )

      end
