      subroutine sepbounlevset3
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array iobstfaces with faces corresponding to the boundary
!     of the region with elements with levelset>0
!     Old subroutine prbounlevset3d
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary
!     face
!     level_set
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: lenbndfaces
      integer, allocatable :: bndfaces(:)

!     bndfaces       contains face sequence numbers of boundary
!                    of levelset
!     lenbndfaces    Computed length of array iobstfaces
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     SEPBOUNLEVSET4 Fill array iobstfaces with faces corresponding to the
!                    boundary of the region with elements with levelset>0
!     SEPFILLNMESHA  Fill array nmesha
!     SEPFILLNMESHG  Fill array nmeshg
!     SEPPRINIELOBST Print the contents of array ielemobst
!     SEPPRINKMESHC  print kmesh part c
!     SEPPRINNMESHA  print nmesh part a
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepbounlevset3' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepbounlevset3'
         call sepprinkmeshc
         call sepprinnmesha

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( nlevelsets==0 ) nlevelsets = numobst  ! number of obstacles

      if ( nlevelsets==0 ) go to 1000  ! levelset not yet created

!     --- create faces

      if ( nmesh(31)==0 ) call sepfillnmesha

!     --- allocate temporary space to store obstfaces

      lenbndfaces = nlevelsets*2+nfaces
      allocate ( bndfaces(lenbndfaces), stat = error )
      if ( error/=0 ) &
         call eralloc ( error, lenbndfaces, 'bndfaces' )
      if ( ierror/=0 ) go to 1000

!     --- Activate or create arrays

      if ( nmesh(37)==0 ) call sepfillnmeshg

!     --- actual call

      if ( debug ) then

!     --- Debug information

         call sepprinielobst ( ielemobst )
         call sepprinnmesha

      end if  ! ( debug )

      call sepbounlevset4 ( bndfaces, lenbndfaces )

      if ( associated(km(iseqkm)%obstfaces) ) &
         deallocate(km(iseqkm)%obstfaces)
      allocate ( km(iseqkm)%obstfaces(lenbndfaces) )
      obstfaces =>  km(iseqkm)%obstfaces
      obstfaces = bndfaces(1:lenbndfaces)
      km(iseqkm)%kmeshad(9) = lenbndfaces

      deallocate ( bndfaces, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'bndfaces' )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepbounlevset3'

      end if  ! ( debug )
      call erclos ( 'sepbounlevset3' )

      end
