      subroutine sepcommatlowup ( irowsnew, icolsnew )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 27-10-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     store integer information of lower triangular part including diagonal
!     into irowsii and icolsii
!     Store upper part in irowsnew and icolsnew
!     The original storage is asymmetrical
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     nested_dissection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulekprob
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(out) :: irowsnew(nrusol+1), icolsnew(size(icolsii))

!     icolsnew       o    Work array to store the column numbers of the matrix
!                         upper triangular part
!     irowsnew       o    Work array to store starting addresses (-1) in
!                         icolsnew
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, icount, iprev
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     i              Counting variable
!     icount         Number of elements in row (upper or lower part)
!     iprev          Previous value in irowsii
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcommatlowup' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcommatlowup'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

!     --- First fill irowsnew and icolsnew (i.e. upper part)

      irowsnew(1) = 0
      iprev = 0
      do i = 1, nrusol
         icount = IM%rowsii(i+1)-IM%diag(i)+1
         irowsnew(i+1) = iprev+icount
         icolsnew(iprev+1:irowsnew(i+1)) = IM%colsii(IM%diag(i):IM%rowsii(i+1))
         iprev = irowsnew(i+1)
      end do  ! i = 1, nrusol

!     --- Next fill irowsii and icolsii (i.e. lower part)

      irowsii(1) = 0
      iprev = 0
      do i = 1, nrusol
         icount = IM%diag(i)-IM%rowsii(i)
         irowsii(i+1) = iprev+icount
         icolsii(iprev+1:irowsii(i+1)) = IM%colsii(IM%rowsii(i)+1:IM%diag(i))
         iprev = irowsii(i+1)
      end do  ! i = 1, nrusol

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepcommatlowup'

      end if  ! ( debug )
      call erclos ( 'sepcommatlowup' )

      end subroutine sepcommatlowup
