      subroutine sepcommatnest1 ( irowsnew, icolsnew, rows )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-06-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute length of array cols and fill array rows
!
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     nested_dissection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulematstr
      use sepmodulekprob
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: irowsnew(nrusol+1), icolsnew(irowsnew(nrusol+1))
      integer, intent(out) :: rows(nrusol+1)

!     icolsnew       i    Work array to store the column numbers of the matrix
!                         upper triangular part
!     irowsnew       i    Work array to store starting addresses (-1) in
!                         icolsnew
!     rows           o    Work array to store starting addresses (-1) in cols
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, icount, j, k, column, columnnew, ih
      integer, allocatable :: iwork(:), istore(:)

!     column         column number
!     columnnew      new column number
!     i              Counting variable
!     icount         Counter
!     ih             Help variable to store a constant temporarily
!     istore         contains column numbers in a row
!     iwork          integer work array to mark nodes in a row
!     j              Counting variable
!     k              Counting variable
!! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepcommatnest1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepcommatnest1'
         write(irefwr,1) 'nrusol', nrusol
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      allocate ( iwork(nrusol+1), istore(nrusol) )

!     --- Clear array iwork

      iwork = 0

      rows(1:2) = irowsnew(1:2)    ! Row 1 remains unchanged

!     --- Loop over other rows

      do i = 2, nrusol

         ih = irowsnew(i+1)-irowsnew(i) ! number of unknowns in row of icolsnew
         iwork(icolsnew(irowsnew(i)+1:irowsnew(i+1))) = 1 ! mark all nodes
         icount = 0                    ! number of unknowns in new row

!        --- Loop over all rows in icolsii corresponding to row i
!            only those with lower row number

         do j = irowsii(i)+1, irowsii(i+1)-1 ! skip last one

            column = icolsii(j)

            do k = irowsnew(column)+2, irowsnew(column+1) ! skip first one

               columnnew = icolsnew(k)  ! new columnsepcommatfill number
               if ( columnnew>i .and. iwork(columnnew)==0 ) then

!              --- New column number has not been stored yet
!                  Store in istore and mark in iwork

                  icount = icount+1
                  istore(icount) = columnnew
                  iwork(columnnew) = 1

               end if  ! ( iwork(columnnew)==0 )

            end do  ! k = irowsnew(column)+2, irowsnew(column+1)

         end do  ! j = irowsii(i)+1, irowsii(i)

         rows(i+1) = rows(i)+ih+icount  ! Store number in rows (accumulated)
         iwork(icolsnew(irowsnew(i)+1:irowsnew(i+1))) = 0
         iwork(istore(1:icount)) = 0

      end do  ! i = 2, nrusol

      deallocate ( iwork, istore ) !, irowsii, icolsii )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( rows, nrusol+1, 'rows' )
         write(irefwr,*) 'End sepcommatnest1'

      end if  ! ( debug )
      call erclos ( 'sepcommatnest1' )

      end subroutine sepcommatnest1
