      subroutine sepedg011 ( ielem, ipd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   creates the edges part 1 pointers and initialization
!   One element group only
!     Old subroutine topedg011
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     edge
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: ielem, ipd

!     ielem         i/o   Element number
!                         At input last one used
!                         At output last one used
!     ipd           i/o   pointer in kmeshaad
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iedge, irelem

!     iedge          edge sequence number
!     irelem         Relative element sequence number with respect to
!                    element group
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepedg011' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepedg011'
         write(irefwr,1) 'inelem, ielem ', inelem, ielem
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Loop over the elements

      do irelem = 1, inelem

         ielem = ielem + 1

!        --- Set pointer to second part of nmeshd

         nmeshd(ielem) = ipd
         nmeshd(ipd) = nedpel

!        --- Initialize second part of nmeshd

         do iedge = 1, nedpel
            nmeshd( ipd + iedge ) = 0
         end do

!        --- Update pointer ipd

         ipd = ipd + nedpel + 1

      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepedg011'

      end if  ! ( debug )
      call erclos ( 'sepedg011' )

      end subroutine sepedg011
