      subroutine sepfa04 ( edges, iface, ihelp )
! ======================================================================
!
!        programmer    Frans van de Vosse
!        version  1.0  date 10-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     creates the faces part 2 actual face labels
!     Old subroutine topfa04
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     face
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: iface, ihelp(*), edges(*)

!     edges      i    edges of the face to create
!     iface      i    Face sequence number of input face
!     ihelp      i    integer work array of length medpfa>=nedpfa
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ie1, iph, nelped, ielped, ielem, jedge, ipd, &
              iedpel, i, j, ipa, ielgrp, jface, &
              kedpfa, n1, nt, ndirec, nrotat, nfapellc

!     i              Counting variable
!     ie1            label of first edge on the face
!     iedpel         edge number in element
!     ielem          Element number
!     ielgrp         element group number
!     ielped         element in one edge
!     ipa            pointer in kmeshaaa
!     ipd            pointer in kmeshaad
!     iph            pointer in kmeshaah
!     j              Counting variable
!     jedge          edge number in element
!     jface          face number in element
!     kedpfa         number of edges in a face
!     n1             location first edge
!     ndirec         direction of the face found
!     nelped         number of elements connected to an edge
!     nfapellc       number of faces per element
!     nrotat         number of rotations needed
!     nt             counter for edges
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     TOPFA03        Determine local edge numbers for local face number
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfa04' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfa04'
    !     call prinin ( edges, nedpfa, 'edges' )
    !     call sepprinnmeshd
    !     write(irefwr,*) 'Elements connected to edges'
     !    call prinnmeshx ( nmesh(4), nmeshh )
  1      format ( a, 1x, (10i6) )

      end if

      ie1 = abs(edges(1))
      iph = nmeshh(ie1)
      nelped = nmeshh(iph)

!     --- Loop over all elements connected with this edge

      do ielped = 1, nelped

         ielem = nmeshh(iph+ielped)

!        --- Fetch number of edges in this element

         ipd = nmeshd(ielem)
         nedpel = nmeshd(ipd)
         if ( debug ) &
            write(irefwr,1) 'ielped, ielem, nedpel, nedpfa', &
                             ielped, ielem, nedpel, nedpfa

!        --- Loop over the (other) edges in this face (they all must be in
!            ielem)

         do i = 2, nedpfa

            jedge = abs(edges(i))

!           --- Check if also this edge is in the element

            do iedpel = 1, nedpel

               if ( abs(nmeshd(ipd+iedpel))==jedge ) then
                  goto 50
               end if
            end do

            goto 500

 50      end do

!        --- This element contains all edges in iedge fetch number of faces in
!            ielem

         ipa = nmesha(ielem)
         nfapellc = nmesha(ipa)/2
         ielgrp = sepelgrpelem(ielem)
         ishape = sepgetshape ( ielgrp )
         if ( debug ) &
            write(irefwr,1) 'nfapellc, ielgrp, ishape', &
                             nfapellc, ielgrp, ishape

!        --- Loop over the faces

         do jface = 1, nfapellc

!        --- Fetch the local edges of this face and store in ihelp

            call topfa03 ( jface, ishape, kedpfa, ihelp )
            if ( debug ) then

!           --- Debug information

               write(irefwr,1) 'jface, kedpfa', jface, kedpfa
               call prinin ( ihelp, kedpfa, 'ihelp' )

            end if

!           --- Try next face if number of edges do not correspond

            if ( kedpfa/=nedpfa ) goto 100

!           --- Store this face in ihelp for a while

            ipd = nmeshd(ielem)
            do i = 1, kedpfa
               j = abs(ihelp(i))
               if ( ihelp(i)>0 ) then
                  ihelp(i) = nmeshd(ipd+j)
               else
                  ihelp(i) = -nmeshd(ipd+j)
               end if
            end do
            if ( debug ) call prinin ( ihelp, kedpfa, 'ihelp' )

!           --- Compare this face with the input face stored in array edges

            n1 = 0
            nt = 0
            do i = 1, nedpfa
               if (abs(ihelp(i))==abs(edges(1)) ) n1 = i
               do j = 1, nedpfa
                  if ( abs(ihelp(i))==abs(edges(j)) ) nt = nt + 1
               end do
            end do
            if ( debug ) write(irefwr,1) 'n1, nt', n1, nt

!           --- Try next face if faces do not correspond

            if ( nt/=nedpfa ) then
               goto 100

            else

!           --- This elements contains the input face: store and try next
!               element

               nrotat = n1 - 1
               ndirec = 1
               if (ihelp(n1)/=edges(1)) ndirec = -1

               nmesha(ipa+jface) = ndirec*iface
               nmesha(ipa+jface+nfapellc) = nrotat
               if ( debug ) &
                  write(irefwr,1) 'jface, ndirec, iface, nrotat', &
                                   jface, ndirec, iface, nrotat
               goto 500
            end if
100      end do

500   end do

      if ( debug )  write(irefwr,*) 'End sepfa04'

      call erclos ( 'sepfa04' )

      end
