      subroutine sepfa06 ( lnmeshg, mndpfa, mndpel )
! ======================================================================
!
!        programmer    Bas van Rens
!        version  1.0  date 10-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     creates the faces part g pointers and initialization
!     Old subroutine topfa06
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer :: lnmeshg, mndpfa, mndpel

!     lnmeshg       i/o   input: declared length of nmeshg
!                         output: actual length of nmeshg
!     mndpel         i    maximum number of nodes in one element
!     mndpfa         i    maximum number of nodes in one face
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipa, ipe, ipf, ipg, iface, nndpfa, ielem, &
              ifaind, ielgrp, i, j, ifound, imax, &
              ihelp(mndpel)
      logical surface, volume

!     i              Counting variable
!     ielem          Element number
!     ielgrp         element group number
!     iface          Face sequence number
!     ifaind         face indicator
!     ifound
!     ihelp          integer work array
!     imax
!     ipa            pointer in kmeshaaa
!     ipe            pointer in kmeshaae
!     ipf            pointer in kmeshaaf
!     ipg            pointer in kmeshaag
!     j              Counting variable
!     nndpfa         number of nodes in one face
!     surface        if true one of the elements is a surface element
!     volume         Indication if the mesh contains volume elements (true)
!                    or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     PRININ         print 1d integer array
!     PRINNMESHX     Print the contents of an integer array with structure of
!                    sub array of nmesh
!     TOPFA07        Determine local node numbers for local face
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1940   Unacceptable user mesh
!    2241   two elements found that have faces pointing in the same direction
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfa06' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfa06'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- check if there volume and surface elements

      surface = .false.
      volume = .false.
      do ielgrp = 1, nelgrp
         ishape = sepgetshape ( ielgrp )
         if ( ishape>=3 .and. ishape<=10 ) then
            surface = .true.
         else if ( ishape>=11 ) then
            volume = .true.
         end if

      end do

      if ( debug ) write(irefwr,*) 'surface, volume ', surface, volume

!     --- get number of faces

      nfaces = nmeshf(1)-1

      ipg = nfaces + 1

      imax = 0
      do iface = 1, nfaces
         ipf = nmeshf(iface)
         if (nmeshf(ipf)>imax) then
            imax = nmeshf(ipf)
         end if
      end do

      if ( debug ) write(irefwr,1) 'nfaces, imax', nfaces, imax

!     --- loop over faces

      do iface = 1, nfaces

!     --- set pointer in nmeshg

         nmeshg(iface) = ipg
         nndpfa = 0

         ipf = nmeshf(iface)

         ifound = 0

         if ( debug ) write(irefwr,1) 'iface', iface

!        --- loop over elements connected to face

         do i = 1, nmeshf(ipf)

            ielem = nmeshf(ipf + i)

!           --- find face

            ipa = nmesha(ielem)

            nfapel = nmesha(ipa)/2

            if ( debug ) then
               write(irefwr,1) 'ielem, nfapel, ifound', &
                                ielem, nfapel, ifound
               call prinin ( nmesha(ipa+1), nfapel, 'nmesha' )
            end if  ! ( debug )
            ifaind = 0

!           --- use only the face that has a positive
!               index to the element.  By definition
!               this face was used first, the other
!               face will have a negative index.
!               There is one exception: If surface
!               elements have been created.  In this
!               case two elements can have a face
!               that points in the same direction, i.e.
!               they have a positive number in nmesha

            do j = 1, nfapel
               if (nmesha(ipa+j)==iface) ifaind = j
            end do

            if ( debug ) write(irefwr,1) 'ifaind', ifaind

            if ( ifaind/=0 .and. ifound==1 ) then
               if ( volume .and. surface ) then

!              --- volume elements combined with surface elements, no problem

               else

!              --- either volume elements without surface elements
!                  or surface elements only: error

                  call errint ( iface, 1 )
                  call errint ( ielem, 2 )
                  call errint ( iface, 3 )
                  do j = 1, nmeshf(ipf)
                     call errint ( nmeshf(ipf+j), j+3 )
                  end do
                  call errsub ( 2241, 3+nmeshf(ipf), 0, 0 )
                  go to 1000

               end if

            else if ( ifaind/=0 ) then

!           --- set ifound to 1 to prevent filling
!               nmeshg twice in case a face is connected
!               to a surface element and a volume element, &
!               both with normals in the same direction

               ifound = 1

               if ( nmesha(ipa+nfapel+ifaind)/=0 ) then

!              --- a face was found with a rotation and
!                  a positive index.  This is not possible
!                  due to the algorithm. All faces are created
!                  based on an element.  They have positive
!                  normal and no rotation for that element.
!                  If they are found again in a neighbouring
!                  element they will have a negative normal
!                  and possibly a rotation.  One exception:
!                  surface elements.  Here only positive normal
!                  faces are considered, they should have no rotation

                  call errint ( iface, 1 )
                  call errint ( ielem, 2 )
                  call errint ( iface, 3 )
                  do j = 1, nmeshf(ipf)
                     call errint ( nmeshf(ipf+j), j+3 )
                  end do
                  call errsub ( 1940, 3+nmeshf(ipf), 0, 0 )
                  go to 1000

               end if

               ielgrp = sepelgrpelem(ielem)
               ishape = sepgetshape ( ielgrp )

               call topfa07 ( ifaind, ishape, nndpfa, ihelp, nspec )

               ipe = nmeshe(ielem)

               do j = 1, nndpfa
                  nmeshg(ipg+j) = nmeshe(ipe+ihelp(j))
               end do

            end if

         end do
         mndpfa = max(mndpfa,nndpfa)
         nmeshg(ipg) = nndpfa
         ipg = ipg + nndpfa + 1
         call inichkarlen ( ipg-1, lnmeshg, 'nmeshg' )

      end do

      lnmeshg = ipg - 1

      if ( debug ) then

!     --- debug = true, print nmeshg

         write ( irefwr, 100 )
100      format ( ' Node numbers corresponding to the faces'/ )
         call prinnmeshx ( nfaces, nmeshg )

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfa06'

      end if  ! ( debug )
      call erclos ( 'sepfa06' )

      end
