      subroutine sepfaceedge
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-07-2012
!
!   copyright (c) 2012-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array nmeshi with the face numbers of the faces connected to
!     the edges
!     Old subroutine topfaceedge
! **********************************************************************
!
!                       KEYWORDS
!
!     face
!     edge
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lennmeshi

!     lennmeshi      (Estimated) length of array kmeshi
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFACEEDGE1   Fill array nmeshi with the face numbers of the faces
!                    connected to the edges
!     SEPFILLNMESHA  Fill array nmesha
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfaceedge' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfaceedge'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      lennmeshi = nedges*25

!     --- Activate or create arrays

      if ( nmesh(31)==0 ) call sepfillnmesha

      allocate ( nmeshi(lennmeshi), stat = error )
      if ( error/=0 ) call eralloc ( error, lennmeshi, 'nmeshi' )
      if ( ierror/=0 ) go to 1000

      call sepfaceedge1 ( lennmeshi )
      if ( ierror/=0 ) go to 1000

!     --- make array nmeshi permanent

      if ( km(iseqkm)%nmesh(39)>0 ) deallocate(km(iseqkm)%nmeshi)
      allocate ( km(iseqkm)%nmeshi(lennmeshi) )
      km(iseqkm)%nmesh(39) = lennmeshi
      km(iseqkm)%nmeshi = nmeshi(1:lennmeshi)

      deallocate ( nmeshi, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'nmeshi' )

      nmeshi => km(iseqkm)%nmesh

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfaceedge'

      end if  ! ( debug )
      call erclos ( 'sepfaceedge' )

      end
