      subroutine sepfillkmeshe
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-02-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kmesh e with nodal points connected to nodal points
!     Old subroutine topfillkmeshe
! **********************************************************************
!
!                       KEYWORDS
!
!     mesh
!     topology
!     node
!     neighbor
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxim

!     maxim          Computed length of array kmesh part e2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERALLOC        Produce error message in case allocate went wrong
!     ERCLOS         Resets old name of previous subroutine of higher level
!     ERDEALLOC      Produce error message in case deallocate went wrong
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFILLKMESHD  Fill kmesh part d
!     SEPFILLKMESHE1 Compute length of KMESH part E2
!     SEPFILLKMESHE2 Fill KMESH part E2
!     SEPPRINKMESH   Print array KMESH
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Extract constants from kmesh
!     Create kmesh part d if necessay
!     Reserve work space
!     Compute starting addresses
!     Fill kmesh part e1 and compute length of part e2
!     make kmesh part e permanent
!     Reserve work space
!     Compute starting addresses
!     Fill kmesh part e2
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillkmeshe' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillkmeshe'
  1      format ( a, 1x, (10i6) )

      end if

!     --- Reserve work space

      if ( associated(kmshe1) ) nullify(kmshe1)
      allocate ( kmshe1(npoint+1), stat = error )
      if ( error/=0 ) call eralloc ( error, npoint+1, 'kmshe1' )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,*) 'before sepfillkmeshd'
      if ( kelmd==0 ) call sepfillkmeshd

!     --- Fill kmesh part e1 and compute length of part e2

      if ( debug ) write(irefwr,*) 'before sepfillkmeshe1'
      call sepfillkmeshe1
      maxim = kmshe1(1+npoint)
      if ( debug ) write(irefwr,1) 'maxim', maxim

!     --- make kmesh part e permanent

      kelme = maxim+npoint+1
      allocate ( km(iseqkm)%kmeshe(kelme) )
      km(iseqkm)%kmshe1 => km(iseqkm)%kmeshe(1:npoint+1)
      km(iseqkm)%kmshe2 => km(iseqkm)%kmeshe(npoint+2:kelme)
      km(iseqkm)%kmshe1 = kmshe1
      kmeshe => km(iseqkm)%kmeshe

      deallocate ( kmshe1, stat = error )
      if ( error/=0 ) call erdealloc ( error, 'kmeshe1' )
      if ( ierror/=0 ) go to 1000

      kmshe1 => km(iseqkm)%kmshe1
      kmshe2 => km(iseqkm)%kmshe2

!     --- Fill kmesh part e2

      call sepfillkmeshe2

      if ( debug ) call sepprinkmesh ( 3 )

1000  if ( debug ) write(irefwr,*) 'End sepfillkmeshe'
      call erclos ( 'sepfillkmeshe' )
      end
