      subroutine sepfillobfaces
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-03-2018
!
!   copyright (c) 2018-2018  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill kmesh part ad subpart 9: iobstfaces
!     Old subroutine iniactm26
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     element
!     node
!     level_set
!     obstacle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPBOUNLEVSET2 Fill array iobstfaces with edges corresponding to the
!                    boundary of the region with elements with levelset>0
!     SEPBOUNLEVSET3 Fill array iobstfaces with faces corresponding to the
!                    boundary of the region with elements with levelset>0
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillobfaces' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillobfaces'
         write(irefwr,1) 'nlevelsets, ndim', nlevelsets, ndim
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) write(irefwr,1) 'kelmad', kelmad
      if ( kmeshad(9)==0 ) then

!     --- Array kmesh ad part 9 does not exist, create in case of levelset

         select case ( ndim )

         case(2)

!        --- ndim = 2, use edges

            call sepbounlevset2

         case(3)

!        --- ndim = 3, use faces

            call sepbounlevset3

         end select  ! case ( ndim )

      end if  ! ( kmeshad(9)==0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillobfaces'

      end if  ! ( debug )
      call erclos ( 'sepfillobfaces' )

      end
