      subroutine sepfillouterb
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 22-09-2007 Fortran 90
!        version  1.0  date 24-09-2003
!
!   copyright (c) 2003-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill outer boundary of mesh
!
! **********************************************************************
!
!                       KEYWORDS
!
!     2d
!     3d
!     boundary
!     outer_boundary
!     curve
!     surface
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MSHOUTERBOUN2D Compute the outer boundary curves of a region in R^2 and
!                    sort this boundary such that it is always counterclockwise
!     MSHOUTERBOUN3D Compute the outer boundary surfaces of a region in R^3 and
!                    sort this boundary such that the normal is directed
!                    outwards
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillouterb' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillouterb'

      end if
      if ( ierror/=0 ) go to 1000

      if ( ndim==2 .and. nsurfs>0 ) then

!     --- ndim = 2, and nsurfs>0, store outer boundary

         call mshouterboun2d

      else if ( ndim==3 .and. nvolms>0 .and. kelmu==0 ) then

!     --- ndim = 3, and nvolms>0 and no spectral elements,
!         store outer boundary

         call mshouterboun3d

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepfillouterb'

      end if
      call erclos ( 'sepfillouterb' )

      end subroutine sepfillouterb
