      subroutine sepfillsubset ( subset, middleset, lensubset )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-05-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill subset with nodes in level middleset that have at least
!     one node in common with level middleset+1
! **********************************************************************
!
!                       KEYWORDS
!
!     renumbering
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmesh
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(out) :: subset(npoint), lensubset
      integer, intent(in) ::  middleset

!     lensubset      o    Size of subset
!     middleset      i    Sequence number of "middle" set
!     subset         o    set of nodes in set middleset that have at least
!                         one node in common with level middleset+1
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: mark(npoint), i, node, j

!     i              Counting variable
!     j              Counting variable
!     mark           Work array to mark all nodes in level middleset+1
!     node           Node number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepfillsubset' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepfillsubset'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Mark all nodes in level middleset+1

      mark = 0
      mark(levelstruc(pointlevel(middleset+1):pointlevel(middleset+2)-1)) = 1
      lensubset = 0

!     --- loop pver all nodes in middleset

      do i = pointlevel(middleset), pointlevel(middleset+1)-1

         node = levelstruc(i)

!        --- Check if one of the neighbors is in level middleset+1

         do j = kmshe1(node)+1, kmshe1(node+1)

            if ( kmshe2(j)>0 ) then

!           --- neighbor in next level

               lensubset = lensubset+1
               subset(lensubset) = node
               exit

            end if  ! ( kmshe2(j)>0 )

         end do  ! j = kmshe1(node):kmshe1(node+1)-1

      end do  ! i = pointlevel(middleset), pointlevel(middleset+1)-1

1000  if ( debug ) then

!     --- Debug information

         call prinin ( subset, lensubset, 'subset' )
         write(irefwr,*) 'End sepfillsubset'

      end if  ! ( debug )
      call erclos ( 'sepfillsubset' )

      end subroutine sepfillsubset
