      subroutine sepintmatpm
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-01-2020
!
!   copyright (c) 2020-2020  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Transform symmetric compact storage scheme to row-compact storage
!     Special application PM-matrix for simple
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_structure
!     simple
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleintmat
      use sepmodulesimple
      use sepmodulematstr
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, allocatable :: rows(:), cols(:)
      type (intmatrix), pointer :: IM

!     IM             Refers to intmt(intmat)
!     cols           array wirh column npmbers
!     rows           Refers to array irowsii
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPCOMMATREV   Fill arrays irowsnew and icolsnew with upper triangular
!                    part if icolsii and irowsii contain the lower triangular
!                    part and vice versa
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepintmatpm' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepintmatpm'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      IM => intmt(intmat)

      allocate ( cols(size(IM%colsPM)), rows(np+1) )

!     --- Copy colsPM in cols and rowsPM in rows

      cols = IM%colsPM
      rows = IM%rowsPM

!     --- Fill arrays rowsPM and colsPM

      call sepcommatrev ( IM%rowsPM, IM%colsPM, rows, cols, np )
      IM%diagPM = IM%rowsPM(1:np)+1
      deallocate ( cols, rows )

1000  if ( debug ) then

!     --- Debug information

         call prinin ( IM%rowsPM, np+1, 'rowsPM' )
         call prinin ( IM%colsPM, IM%rowsPM(np+1), 'colsPM' )
         write(irefwr,*) 'End sepintmatpm'

      end if  ! ( debug )
      call erclos ( 'sepintmatpm' )

      end subroutine sepintmatpm
