      subroutine topbv2 ( iwork )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 15-06-2012 Remove ihelp from parameter list
!        version  1.4  date 04-08-2006 Debug statements
!        version  1.3  date 23-12-1996 Skip non-existing unknowns
!
!   copyright (c) 1983-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Fill positions in iwork corresponding to prescribed boundary conditions
!
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*)

!     iwork     i/o   Work array of length nusol in which the information
!                     about the essential boundary conditions is stored
!                     if iwork(i) = 0, then unknown i is not prescribed
!                     if iwork(i) = 1, then unknown i is prescribed
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, i2nun, ibitp, ih, ih1, inodp, inunkp, inusol, k, &
              ihelp(nunkp)

!     i              Counting variable
!     i2nun          i^2
!     ibitp          Bit pattern
!     ih             Help variable to store a constant temporarily
!     ih1            Help variable to store a constant temporarily
!     inodp          Nodal point number
!     inunkp         Number of degrees of freedom in first point
!     inusol         Position of unknown in array USOL
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ1        print 2d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      77   You try to prescribe the wrong degree of freedom
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topbv2' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topbv2'
         write(irefwr,1) 'nunkp, nusol, nkbnd', nunkp, nusol, nkbnd
         call prinin1 ( kprobg, nkbnd, 2, 'kprobg' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- array ihelp is filled

      i2nun = 1
      do i = 1, nunkp
         i2nun = i2nun*2
         ihelp(i) = i2nun
      end do

!     --- fill array iwork with numbers of prescribed unknowns

      inunkp = nunkp
      do i = 1, nusol
         iwork(i) = 0
      end do

      do i = 1, nkbnd
         inodp = kprobg(1,i)
         ibitp = kprobg(2,i)
         if ( debug ) write(irefwr,1) 'i, inodp, ibitp', i, inodp, ibitp
         if ( ibitp==0 ) go to 600
         if ( ibitp<0 ) then
            call errint ( ibitp, 1 )
            call errint ( ihelp(nunkp)-1, 2 )
            call errint ( inodp, 3 )
            call errsub ( 77, 3, 0, 0 )
            go to 600
         end if
         if ( indprf==0 ) then

!        --- indprf  =  0

            inusol = (inodp-1)*nunkp+1

         else

!        --- indprf>0

            inusol = kprobf(inodp)
            inunkp = kprobf(inodp+1)-inusol
            inusol = inusol+1
            if ( inunkp==0 ) go to 600
         end if

         i2nun = ihelp(inunkp)

!        --- fill iwork

         if ( ibitp>=8 ) then

!        --- ibitp>=  8

            ih = ibitp
            k = 0
100         ih1 = ih/2
            if ( ih/=ih1*2 ) then

!           --- bit = 1 found

               if ( inunkp>k ) iwork(inusol+k) = 1

            end if
            k = k+1
            ih = ih1
            if ( ih/=0 ) goto 100

         else

!        --- ibitp<8

            goto (510, 520, 530, 540, 550, 560, 570), ibitp

!           --- ibitp = 1

510            iwork(inusol) = 1
               goto 600

!              --- ibitp = 2

520            if ( inunkp>1 ) iwork(inusol+1) = 1
               goto 600

!              --- ibitp = 3

530            iwork(inusol) = 1
               goto 520

!              --- ibitp = 4

540            if ( inunkp>2 ) iwork(inusol+2) = 1
               goto 600

!              --- ibitp = 5

550            if ( inunkp>2 ) iwork(inusol+2) = 1
               goto 510

!              --- ibitp = 6

560            if ( inunkp>2 ) iwork(inusol+2) = 1
               goto 520

!              --- ibitp = 7

570            if ( inunkp>2 ) iwork(inusol+2) = 1
               goto 530
         end if

600   end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topbv2'

      end if  ! ( debug )
      call erclos ( 'topbv2' )

      end
