      subroutine topbv3 ( iwork, maxren, irenum )
! ======================================================================
!
!        programmer    Guus Segal
!        version  5.1  date 13-04-2015 Get nrusol from kprob(66)
!        version  5.0  date 15-06-2012 Remove ihelp from parameter list
!        version  4.6  date 17-12-2009 Less output
!
!   copyright (c) 1982-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill kprob part h and renumber kprob part p
!     Array KPROB P refers to the numbering of unknowns where the unknowns are
!     not renumbered!
! **********************************************************************
!
!                       KEYWORDS
!
!     renumbering
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      use sepmodulecpack
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iwork(*), irenum(*), maxren

!     irenum         i    This array contains the kmeshk of the renumbering of
!                         all unknowns per point
!                         So if iren(i)=j then unknown i is numbered in the j-th
!                         level
!                         Unknowns with sequence number larger than maxren
!                         correspond always to the largest level
!     iwork         i/o   Output of subroutine TOPBV2
!                         iwork(i)=1, means essential boundary condition for
!                                     unknown i
!                         iwork(i)>100, means i-th unknown on interface with
!                                       lower  block number (parallel case)
!                                       (+100)
!                         iwork(i)=0, means unknown i is free
!                         iwork(i)<0, means unknown i corresponds to
!                         periodical bc.
!     maxren         i    Maximum number of unknowns that have a special
!                         numbering
!                         If <0, the unknowns must be renumbered per level
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, nh, nusolclassic, nlevelsav
      logical jump

!     i              Counting variable
!     jump           if true nrusol should not be adapted after the call
!     nh             Help variable to store a constant
!     nlevelsav      Help variable to save old value of nlevel
!     nusolclassic   Number of free classical unknowns, i.e ficitious unknowns
!                    and global unknowns excluded
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     TOPBV3FREE     Fill kprobh for free unknowns
!     TOPBV3GLOB     Fill last part of kprobh for global unknowns
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   Clear KPROBh
!   Compute nbound and npboun
!   If ( nkbnd>0 ) then
!      Fill KPROBh with respect to boundary conditions
!   Print nkbnd, nbound and npboun
!   Fill part of KPROB
!   If ( no renumbering is used ) then
!      Fill KPROBh staight-forward for internal degrees of freedom
!   else if ( only the nodes are renumbered ) then
!      For i := 1 (1) npoint do
!         inodp := i-th nodal point
!         inunkp := number of unknowns in node inodp
!         Fill all free unknowns in node inode
!   else ( sequence of unknowns is renumbered )
!      If ( renumbering must be performed for all points ) then
!         For ilevel := 1 (1) maxren+1 do
!            nlevel := number of unknowns of level ilevel
!            if ( nlevel>0 ) then
!               Fill all unknowns of level ilevel in array ihelp
!               For i := 1 (1) npoint do
!                  inodp := i-th nodal point
!                  inunkp := number of unknowns in node inodp
!                  For j := 1 (1) inunkp do
!                     If ( ihelp(j)>0 ) then
!                        Fill unknown if it is free
!      else ( renumbering must be performed per level )
!          Perform the same actions per level
!   Renumber KPROB part p
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topbv3' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topbv3'
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- set constants

      nkbnd  = kprobloc(9)
      nphys = max(nphys,nunkp)
      nlevelsav = nlevel
      if ( debug ) then
         write(irefwr,1) 'nunkp, nusol, nkbnd, indprf, indprp', &
                          nunkp, nusol, nkbnd, indprf, indprp
         write(irefwr,1) 'nlevel, kelmj, nphys', nlevel, kelmj, nphys

!        --- if ( nlevel>0 ) call prinin ( kmeshk, nlevel, 'kmeshk' )

      end if  ! ( debug )

!     --- clear kprob part h

      kprobh(1:nusol) = 0

!     --- compute nbound

      if ( maxren>0 ) nlevel = 1
      nbound = 0
      npboun = 0
      do i = 1, nusol
         if ( iwork(i)==1 ) then
            nbound = nbound+1
         else if ( iwork(i)<0 ) then
            npboun = npboun+1
         end if
      end do
      kprobloc(10) = nbound

      if ( nkbnd>0 ) then

!     --- nkbnd>0  fill part of kprob part h

         nh = nusol-nbound+1
         do i = 1, nusol
            if ( iwork(i)==1 ) then

!           --- iwork(i) = 1   boundary condition found

               kprobh(i) = nh
               nh = nh+1
            end if
         end do

      end if  ! ( nkbnd>0 )
      nusolclassic = nusol-nglunk-nfictunk
      nrusol = nusolclassic-nbound-npboun-nglcoupl
      if ( kprobloc(51)>0 ) then

!     --- There are fictitious unknowns, fill corresponding part of kprobh

         do i = 1, nfictunk
            kprobh(nusolclassic+i) = nrusol+i
         end do

      end if

      nrusol = nrusol+nfictunk
      nusolclassic = nusolclassic+nfictunk
      jump = .false.

      if ( indprx>0 ) then

!     --- There are global unknowns, fill last part of kprobh

         call topbv3glob ( nusolclassic, iwork, jump )
         if ( ierror/=0 ) go to 1000

      end if  ! ( indprx>0 )

      if ( .not. jump ) nrusol = nusol-nbound-npboun

      if ( debug .or. ioutp>=1 ) &
          write(irefwr,200) nkbnd,nbound,npboun,nglunk,nrusol, kprobloc(67)
200   format ( 'number of points with prescribed boundary conditions'/ &
               '(including double points)  =  ',i6/ &
               'number of prescribed degrees of freedom  = ',i6/ &
               'number of periodical boundary conditions  = ',i6/ &
               'number of global unknowns  = ',i6/ &
               'number of free degrees of freedom = ',i8/ &
               'number of interface unknowns = ', i6 )
      kprobloc(34) = npboun

!     --- fill kprobloc part h for free unknowns

      call topbv3free ( maxren, irenum, iwork )

      if ( npboun>0 ) then

!     --- fill kprob part h for periodical boundary conditions

         do i = 1, nusol
            if ( iwork(i)<0 ) kprobh(i) = kprobh(-iwork(i))
         end do

      end if

      kprobloc(66) = nrusol
      nlevel = nlevelsav

1000  if ( debug ) then

!     --- Debug information

         call prinin ( kprobh, nusol, 'kprobh' )
         if ( indprac>0 ) call prinin ( kprobac, maxren, 'kprobac' )
         write(irefwr,*) 'End topbv3'

      end if
      call erclos ( 'topbv3' )

      end
