      subroutine topbv7011 ( iinput, idegper, rjumploc, rmultloc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 14-01-2013 extra debug
!        version  1.0  date 18-06-2012
!
!   copyright (c) 2012-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array idegper with degrees of freedom that are periodical
!     New version without the use of connection elements
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     periodical
!     problem_definition
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iinput(*), idegper(nunkp)
      double precision rjumploc(nunkp), rmultloc(nunkp)

!     idegper        o    Array of size nunkp containing a 1 if the
!                         corresponding unknown is periodical and a 0 if not
!     iinput         i    integer input array for subroutine probdfext
!                         The first position refers to the start of the
!                         information with respect to degrees of freedom
!     rjumploc       o    Array of size nunkp containing the jumps for each
!                         degree of freedom
!     rmultloc       o    Array of size nunkp containing the multiplication
!                         factors for each degree of freedom
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, idegfd, istart, nparts
      double precision rjump, rmult

!     i              Counting variable
!     idegfd         Degree of freedom sequence number
!     istart         First position used at a part
!     nparts         number of parts the degrees of freedom are given
!     rjump          Contains the jump constant
!     rmult          Multiplication factor for the solution
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
!     TOPBV5LOC      Update the arrays idegper, rjumploc and factorloc with
!                    information about jump and factors as defined by
!                    the bit pattern in idegfd
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topbv7011' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topbv7011'
         call prinin ( iinput, 4, 'iinput/loc' )
         call prinrl ( rinputkp, 2, 'rinputkp' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Fill array idegper with default values

      idegper = 0

      istart = 1
      nparts = iinput(istart)

      if ( debug ) write(irefwr,1) 'nparts', nparts

      do i = 1, nparts

         idegfd = iinput(istart+1)
         if ( iinput(istart+2)==0 ) then
            rjump = 0d0
         else
            rjump = rinputkp(iinput(istart+2))
         end if  ! ( iinput(istart+2)==0 )
         if ( iinput(istart+3)==0 ) then
            rmult = 1d0
         else
            rmult = rinputkp(iinput(istart+3))
         end if  ! ( iinput(istart+3)==0 )
         istart = istart+3
         if ( debug ) then
            write(irefwr,1) 'idegfd', idegfd
         end if  ! ( debug )

         if ( idegfd>0 ) then

!        --- Not all degrees of freedom are prescribed

            call topbv5loc ( idegfd, rjump, rmult, idegper, &
                             rjumploc, rmultloc, nunkp, 1 )

         else

!        --- All degrees of freedom are prescribed

            idegper(1:nunkp) = 1
            rjumploc = rjump
            rmultloc = rmult

         end if  ! ( idegfd>0 )

      end do  ! i = 1, nparts

1000  if ( debug ) then

!     --- Debug information

         call prinin ( idegper, nunkp, 'idegper' )
         write(irefwr,*) 'End topbv7011'

      end if  ! ( debug )
      call erclos ( 'topbv7011' )

      end
