      subroutine topbvperiod ( lenkprobab, istoreleft )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 02-04-2003
!
!   copyright (c) 2003-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array kprob part ab with pairs of periodical degrees of freedom
!
! **********************************************************************
!
!                       KEYWORDS
!
!     problem_definition
!     periodical
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer lenkprobab, istoreleft(*)

!     istoreleft     i    Contains all periodical right neighbour unknowns
!                         for all left periodical neighbour unknowns
!                         Pos. 1 to nusol+1 contain pointers to the second part
!                         istoreleft(1) contains nusol+1
!                         The number of neighbours of unknown i is stored in
!                         istoreleft(i+1)-istoreleft(i)
!                         The second part contains the neighbours
!     lenkprobab    i/o   Length of array kprobab
!                         At input reserved length
!                         At output computed length
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icount, istart, i, j, num, istartab

!     i              Counting variable
!     icount         Counter
!     istart         First position used at a part
!     istartab       Starting address in array kprob ab
!     j              Counting variable
!     num            Number of degrees of freedom in a "pair" minus 1
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRINKPROBAB    print kprob part ab
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!     First compute number of "pairs", store in kprobab(1)
!     icount := 0
!     istart := istoreleft(1)
!     for i := 1 (1) nusol do
!        num := istoreleft(i+1)-istart
!        if ( num > 0 ) then
!           icount := icount+1
!        istart := istart+num
!     kprobab(1) := icount
!     Next fill rest of kprobab
!     istartab := icount+2
!     icount := 0
!     istart := istoreleft(1)
!     for i := 1 (1) nusol do
!        num := istoreleft(i+1)-istart
!        if ( num > 0 ) then
!           icount := icount+1
!           kprobab(1+icount) := istartab
!           kprobab(istartab) := num+1
!           kprobab(istartab+1) := i
!           for j := 1 (1) num do
!              kprobab(istartab+1+j) := istoreleft(istart+j)
!           istartab := istartab+2+num
!        istart := istart+num
!     lenkprobab := istartab-1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topbvperiod' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topbvperiod'

      end if

!     --- First compute number of "pairs", store in kprobab(1)
!         icount := 0
!         istart := istoreleft(i)

      icount = 0
      istart = istoreleft(1)

!     --- for i := 1 (1) nusol do

      do i = 1, nusol

!     --- num := istoreleft(i+1)-istart

         num = istoreleft(i+1)-istart
         if ( num.gt.0 ) then

!        --- num > 0, i.e. unknown i is coupled with num other unknowns
!            icount := icount+1
!            istart := istart+num

            icount = icount+1
            istart = istart+num

         end if  ! ( num.gt.0 )

      end do  ! i = 1, nusol

!     --- kprobab(1) := icount

      kprobab(1) = icount

!     --- Next fill rest of kprobab
!         istartab := icount+2
!         icount := 0
!         istart := istoreleft(1)

      istartab = icount+2
      icount = 0
      istart = istoreleft(1)

      do i = 1, nusol

!     --- num := istoreleft(i+1)-istart

         num = istoreleft(i+1)-istart
         if ( num.gt.0 ) then

!        --- num > 0, i.e. unknown i is coupled with num other unknowns
!            icount := icount+1
!            kprobab(1+icount) := istartab
!            kprobab(istartab) := num+1
!            kprobab(istartab+1) := i

            icount = icount+1
            kprobab(1+icount) = istartab
            kprobab(istartab) = num+1
            kprobab(istartab+1) = i

!           --- for j := 1 (1) num do
!                  kprobab(istartab+1+j) := istoreleft(istart+j)

            do j = 1, num
               kprobab(istartab+1+j) = istoreleft(istart+j)
            end do

!           --- istartab := istartab+2+num
!               istart := istart+num

            istartab = istartab+2+num
            if ( istartab-1.gt.lenkprobab ) then

!           --- lenkprobab too small

               call errint ( istartab-1, 1 )
               call errint ( lenkprobab, 2 )
               call errchr ( 'kprobab', 1 )
               call errsub ( 550, 2, 0, 1 )
               go to 1000

            end if
            istart = istart+num

         end if  ! ( num.gt.0 )

      end do  ! i = 1, nusol

!     --- lenkprobab := istartab-1

      lenkprobab = istartab-1

1000  if ( debug ) then

!     --- Debug information

         call prinkprobab
         write(irefwr,*) 'End topbvperiod'

      end if
      call erclos ( 'topbvperiod' )

      end
