      subroutine topedg03 ( iedge, ishape, nndped, nodes, nspecp )
! ======================================================================
!
!        programmer    Frans van de Vosse
!        version  1.2  date 18-12-2002 Check of shapes
!        version  1.1  date 13-09-2000 Extension of shapes
!        version  1.0  date 23-08-1998 Renamed nmed03
!
!   copyright (c) 1998-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   return local node numbers for local edge iedge
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iedge, ishape, nndped, nodes(*), nspecp

!     iedge      i    local edge number
!     ishape     i    Type number of elements in the mesh or element group
!     nndped     o    number of nodes in one edge
!     nodes      o    Contains the node numbers that must be filled
!     nspecp     i    number of spectral points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, dln1(2), dln2(3), dln3(2,3), dln4(3,3), dln5(2,4), &
                 dln6(3,4), dln7(3,3), dln8(4,3), dln9(2,4), &
                 dln10(2,3), dln11(2,6), &
                 dln12(3,6), dln13(2,12), dln14(3,12), dln15(3,6), &
                 dln16(3,6), dln17(2,12), dln18(2,6), &
                 dnoped(18)

!     dln1           Data local nodes for element type 1
!     dln10          Data local nodes for element type 10
!     dln11          Data local nodes for element type 11
!     dln12          Data local nodes for element type 12
!     dln13          Data local nodes for element type 13
!     dln14          Data local nodes for element type 14
!     dln15          Data local nodes for element type 15
!     dln16          Data local nodes for element type 16
!     dln17          Data local nodes for element type 17
!     dln18          Data local nodes for element type 18
!     dln2           Data local nodes for element type 2
!     dln3           Data local nodes for element type 3
!     dln4           Data local nodes for element type 4
!     dln5           Data local nodes for element type 5
!     dln6           Data local nodes for element type 6
!     dln7           Data local nodes for element type 7
!     dln8           Data local nodes for element type 8
!     dln9           Data local nodes for element type 9
!     dnoped         Number of nodes per edge
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1692   type of spectral element has not been implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data dnoped / 2, 3, 2, 3, 2, 3, 3, 4, 2, 2, &
                    2, 3, 2, 3, 3, 3, 2, 2 /
      data dln1 / 1, 2 /
      data dln2 / 1, 2, 3 /
      data dln3 / 1, 2,        2, 3,        3, 1 /
      data dln4 / 1, 2, 3,     3, 4, 5,     5, 6, 1 /
      data dln5 / 1, 2,        2, 3,        3, 4,        4, 1 /
      data dln6 / 1, 2, 3,     3, 4, 5,     5, 6, 7,     7, 8, 1 /
      data dln7 / 1, 2, 3,     3, 4, 5,     5, 6, 1 /
      data dln8 / 1, 2, 3, 4,  4, 5, 6, 7,  7, 8, 9, 1 /
      data dln9 / 1, 2,        2, 3,        3, 4,        4, 1 /
      data dln10/ 1, 2,        2, 3,        3, 1 /
      data dln11/ 1, 2,        2, 4,        4, 1,        1, 3, &
                  2, 3,        4, 3 /
      data dln12/ 1, 2, 3,     3, 8,10,    10, 7, 1,     1, 6, 5, &
                  3, 4, 5,    10, 9, 5 /
      data dln13/ 1, 2,        2, 6,        6, 5,        5, 1, &
                  4, 3,        3, 7,        7, 8,        8, 4, &
                  1, 4,        2, 3,        6, 7,        5, 8 /
      data dln14/ 1, 2, 3,     3,12,21,    21,20,19,    19,10, 1, &
                  7, 8, 9,     9,18,27,    27,26,25,    25,16, 7, &
                  1, 4, 7,     3, 6, 9,    21,24,27,    19,22,25 /
      data dln15/ 1, 2, 3,     3, 8,10,    10, 7, 1,     1, 6, 5, &
                  3, 4, 5,    10, 9, 5 /
      data dln16/ 1, 2, 3,     3, 8,10,    10, 7, 1,     1, 6, 5, &
                  3, 4, 5,    10, 9, 5 /
      data dln17/ 1, 2,        2, 6,        6, 5,        5, 1, &
                  4, 3,        3, 7,        7, 8,        8, 4, &
                  1, 4,        2, 3,        6, 7,        5, 8 /
      data dln18/ 1, 2,        2, 4,        4, 1,        1, 3, &
                  2, 3,        4, 3 /
! ======================================================================
!
      call eropen  ('topedg03' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from topedg03'
         write(irefwr,1) 'ishape, nspecp, iedge', ishape, nspecp, iedge
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ishape== -10001 ) then
         nndped = 2
      else if ( nspecp==0 .and. ishape<=18 ) then
         nndped = dnoped(ishape)
      else if ( nspecp>0 .and. &
               (ishape==5 .or. ishape==13) ) then
         nndped = nspecp
      else

         call errint ( ishape, 1 )
         call errint ( nspecp, 2 )
         call errsub ( 1692, 2, 0, 0 )
         go to 1000

      end if

      if ( debug ) write(irefwr,1) 'nndped', nndped

      if (nspecp==0) then

         select case ( ishape )
            case (-10001)
               nodes(1:nndped) = dln1 (1:nndped)
            case(1)
               nodes(1:nndped) = dln1 (1:nndped)
            case(2)
               nodes(1:nndped) = dln2 (1:nndped)
            case(3)
               nodes(1:nndped) = dln3 (1:nndped,iedge)
            case(4)
               nodes(1:nndped) = dln4 (1:nndped,iedge)
            case(5)
               nodes(1:nndped) = dln5 (1:nndped,iedge)
            case(6)
               nodes(1:nndped) = dln6 (1:nndped,iedge)
            case(7)
               nodes(1:nndped) = dln7 (1:nndped,iedge)
            case(8)
               nodes(1:nndped) = dln8 (1:nndped,iedge)
            case(9)
               nodes(1:nndped) = dln9 (1:nndped,iedge)
            case(10)
               nodes(1:nndped) = dln10(1:nndped,iedge)
            case(11)
               nodes(1:nndped) = dln11(1:nndped,iedge)
            case(12)
               nodes(1:nndped) = dln12(1:nndped,iedge)
            case(13)
               nodes(1:nndped) = dln13(1:nndped,iedge)
            case(14)
               nodes(1:nndped) = dln14(1:nndped,iedge)
            case(15)
               nodes(1:nndped) = dln15(1:nndped,iedge)
            case(16)
               nodes(1:nndped) = dln16(1:nndped,iedge)
            case(17)
               nodes(1:nndped) = dln17(1:nndped,iedge)
            case(18)
               nodes(1:nndped) = dln18(1:nndped,iedge)

         end select  ! case ( ishape )

      else if(ishape==5) then

         select case (iedge)
            case(1)
               do i=1,nndped
                  nodes(i) = i
               end do
            case(2)
               do i=1,nndped
                  nodes(i) = i*nndped
               end do
            case(3)
               do i=1,nndped
                  nodes(i) = nndped*nndped-i+1
               end do
            case(4)
               do i=1,nndped
                  nodes(i) = nndped*nndped-i*nndped+1
               end do
         end select ! case (iedge)

      else if(ishape==13) then

         select case (iedge)
            case(1)
               do i=1,nndped
                  nodes(i) = i
               end do
            case(2)
               do i=1,nndped
                  nodes(i) = (i-1)*nndped*nndped+nndped
               end do
            case(3)
               do i=1,nndped
                  nodes(i) = (nndped-1)*nndped*nndped+nndped-i+1
               end do
            case(4)
               do i=1,nndped
                  nodes(i) = (nndped-1)*nndped*nndped+1- &
                             (i-1)*nndped*nndped
               end do
            case(5)
               do i=1,nndped
                  nodes(i) = (nndped-1)*nndped+i
               end do
            case(6)
               do i=1,nndped
                  nodes(i) = i*nndped*nndped
               end do
            case(7)
               do i=1,nndped
                  nodes(i) = nndped*nndped*nndped-i+1
               end do
            case(8)
               do i=1,nndped
                  nodes(i) = (nndped-1)*nndped+ &
                             (nndped-1)*nndped*nndped+1- &
                             (i-1)*nndped*nndped
               end do
            case(9)
               do i=1,nndped
                  nodes(i) = 1+(i-1)*nndped
               end do
            case(10)
               do i=1,nndped
                  nodes(i) = nndped+(i-1)*nndped
               end do
            case(11)
               do i=1,nndped
                  nodes(i) = (nndped-1)*nndped*nndped+nndped+ &
                             (i-1)*nndped
               end do
            case(12)
               do i=1,nndped
                  nodes(i) = (nndped-1)*nndped*nndped+1+(i-1)*nndped
               end do
         end select ! case (iedge)
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End topedg03'

      end if  ! ( debug )
      call erclos  ('topedg03' )

      end
