      subroutine topedges02 ( kmeshc, kmshd1, kmshd2, nmeshd, nmeshc, &
                              mnoped, nedges, nelem, nedpel, inpelm, &
                              ishape, length )
! ======================================================================
!
!        programmer    Frans van de Vosse
!        version  1.0  date 08-05-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   creates the edges part 2 actual edge labels and nmeshc
!   This subroutine is copied from topedg02
!   Restricted to one specific element group
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     edge
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, nelem, kmeshc(inpelm,nelem), nmeshd(*), &
              mnoped, nedges, kmshd1(*), kmshd2(*), nedpel, ishape, &
              nmeshc(*), length

!     inpelm         i    number of points per element
!     ishape         i    Shape number of elements
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmshd1         i    first part of kmesh part d, contains the number of
!                         elements corresponding to each nodal point accumulated
!                         kmshd1(1) = 0.
!                         Length: npoint+1
!     kmshd2         i    second part of kmesh part d
!                         Contains all the element numbers the nodal points are
!                         lying in sequentially
!     length        i/o   Help parameter to indicate the length of arrays
!                         At input declared length of array nmeshc
!                         At output computed length of array nmeshc
!     mnoped         i    maximum number of nodes per edge
!     nedges         o    number of edges
!     nedpel         i    number of edges in one element
!     nelem          i    Number of elements
!     nmeshc         o    nmesh part c
!     nmeshd        i/o   nmesh part d
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ielem, i, ipd, ihelp(mnoped), jedge, nndped, iedge, ipc, &
              locnodes(4,12)

!     i              Counting variable
!     iedge          edge sequence number
!     ielem          Element number
!     ihelp          integer work array of length mnoped
!                    In this array the node numbers of an edge are stored
!     ipc            pointer in nmeshn
!     ipd            pointer in kmeshaad
!     jedge          edge sequence number
!     locnodes       Array of size 4 x 12 containing the local node
!                    numbers of each edge of a volume element
!     nndped         number of nodes in one edge
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     TOPEDG03       Get local node numbers of an edge
!     TOPEDGES04     Find elements connected to an edge and store in nmeshd
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen  ('topedges02' )

      nedges = 0

!     --- Store local node numbers of edges in locnodes

      do jedge = 1, nedpel
         call topedg03 ( jedge, ishape, nndped, locnodes(1,jedge), 0 )
      end do

!     --- Loop over the elements in this group

      do ielem = 1, nelem
         ipd = nmeshd(ielem)

!        --- Loop over the edges

         do jedge = 1, nedpel

            if ( nmeshd(ipd+jedge).eq.0 ) then

!           --- New edge found

               nedges = nedges + 1
               nmeshc(nedges) = 0

!              --- Store this edge in ihelp for a while

               do i = 1, nndped
                  ihelp(i) = kmeshc(locnodes(i,jedge),ielem)
               end do

!              --- Find all elements with this edge and fill nmeshd part 2

               call topedges04 ( kmeshc, kmshd1, kmshd2, nmeshd, &
                                 ihelp, nndped, nedges, &
                                 inpelm, nedpel, locnodes )

            end if

         end do

      end do

!     --- Redo this but now fill nmeshc

      ipc = nedges + 1

!     --- Loop over the elements in this group

      do ielem = 1, nelem
         ipd = nmeshd(ielem)

!        --- Loop over the edges

         do jedge = 1, nedpel

!        --- Get the label of this edge from nmeshd

            iedge = abs(nmeshd(ipd+jedge))
            if ( nmeshc(iedge).eq.0 ) then

!           --- New edge found: store this edge in nmeshc

               nmeshc(iedge) = ipc
               nmeshc(ipc) = nndped
               do i = 1, nndped
                  nmeshc(ipc+i) = kmeshc(locnodes(i,jedge),ielem)
               end do
               ipc = ipc + nndped + 1
            end if

         end do

      end do

      length = ipc - 1

      call erclos  ('topedges02' )
      end
