      subroutine topedges04 ( kmeshc, kmshd1, kmshd2, nmeshd, inodes, &
                              nndped, iedge, inpelm, nedpel, locnodes )
! ======================================================================
!
!        programmer    Frans van de Vosse
!        version  1.0  date 8-05-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   creates the nmeshd part 2 actual edge labels
!   Restricted to one specific element group
!   This subroutine is a copy of subroutine topedg04
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     edge
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, kmeshc(inpelm,*), nmeshd(*), inodes(*), &
              nndped, iedge, kmshd1(*), kmshd2(*), nedpel, &
              locnodes(4,*)

!     iedge          i    edge sequence number
!     inodes         i    array with information about the edge nodes
!     inpelm         i    number of points per element
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmshd1         i    first part of kmesh part d, contains the number of
!                         elements corresponding to each nodal point accumulated
!                         kmshd1(1) = 0.
!                         Length: npoint+1
!     kmshd2         i    second part of kmesh part d
!                         Contains all the element numbers the nodal points are
!                         lying in sequentially
!     locnodes       i    Array of size 4 x 12 containing the local node
!                         numbers of each edge of a volume element
!     nedpel         i    number of edges in one element
!     nmeshd        i/o   nmesh part d
!     nndped         i    number of nodes in one edge
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer istart, j, ielem, jedge, kc, i, ipd

!     i              Counting variable
!     ielem          Element number
!     ipd            pointer in kmeshaad
!     istart         First position used at a part
!     j              Counting variable
!     jedge          edge sequence number
!     kc             Counter
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Loop over all elements connected with this node

      istart = kmshd1(inodes(1))
      do j = 1, kmshd1(inodes(1)+1) - istart

!     --- Fetch element label

         ielem = kmshd2(istart+j)

!        --- Loop over the edges of this element

         do jedge = 1, nedpel

            kc = 0
            do i = 1, nndped
               if ( kmeshc(locnodes(i,jedge),ielem).eq.inodes(i) ) &
                  kc = kc + 1
            end do
            if ( kc.ne.nndped ) then

!           --- Try the other way around

               kc = 0
               do i = 1, nndped
                  if ( kmeshc(locnodes(i,jedge),ielem).eq. &
                       inodes(nndped+1-i) ) kc = kc -1
               end do
            end if
            if ( abs(kc).eq.nndped ) then

!           --- Element found with edge the same as input edge

               ipd = nmeshd(ielem)
               nmeshd(ipd+jedge) = iedge*sign(1,kc)
               goto 400

            end if

         end do

400   end do
      end
