      subroutine topelnodes1 ( inpelm, kmshd1, kmeshc, length, npoint, &
                               nelem )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-05-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill kmesh part d1 with pointers to kmesh part d2
!     kmesh part d contains the elements corresponding to each node
!     The second part will be filled in topelnodes2
!     This is the same routine as toplm1, but now restricted to one
!     element group only
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
!     element
!     node
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer inpelm, kmshd1(*), kmeshc(inpelm,*), length, npoint, &
              nelem

!     inpelm         i    number of points per element
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     kmshd1         o    first part of kmesh part d, contains the number of
!                         elements corresponding to each nodal point accumulated
!                         kmshd1(1) = 0.
!                         Length: npoint+1
!     length         o    computed length of of part 2 of kmesh part d
!     nelem          i    Number of elements
!     npoint         i    Number of nodal points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, nodp, ih, ielem

!     i              Counting variable
!     ielem          Element number
!     ih             Help variable to store a constant temporarily
!     j              Counting variable
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     clear kmesh part d1
!     Store number of elements corresponding to nodes in kmesh d part 1
!     Fill kmesh d part 1 by addition
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- clear kmesh part d1

      do i = 1, npoint
         kmshd1(i) = 0
      end do

!     --- Store number of elements corresponding to nodes in kmesh d part 1
!         Loop over elements in element group

      do ielem = 1, nelem

!     --- Loop over nodes in element

         do j = 1, inpelm
            nodp = kmeshc(j,ielem)
            kmshd1(nodp) = kmshd1(nodp)+1
         end do

      end do

!     --- Fill kmesh d part 1 by addition

      length = 0
      do i = 1,npoint
         ih = kmshd1(i)
         kmshd1(i) = length
         length = length+ih
      end do

      kmshd1(npoint+1) = length

      end
