      subroutine topfa03 ( iface, ishape, nedpfa, edges )
! ======================================================================
!
!        programmer    Frans van de Vosse
!        version  1.3  date 22-05-2014 Extension with shape 22
!        version  1.2  date 24-12-2002 Extension with prisms
!        version  1.1  date 18-12-2002 Check of shapes
!
!   copyright (c) 1998-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     return local edge numbers for local face iface
!
! **********************************************************************
!
!                       KEYWORDS
!
!     edge
!     face
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iface, ishape, nedpfa, edges(*)

!     edges      o    node numbers
!     iface      i    Face sequence number
!     ishape     i    Type number of elements in the mesh or element group
!     nedpfa     o    number of edges per face
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, dle3(3,1), dle5(4,1), dle11(3,4), &
                 dle13(4,6), dle21(18), dedpfa(18), istart

!     dedpfa     data edges per face
!     dle11      data edges per face for element 11
!     dle13      data edges per face for element 13
!     dle21      data edges per face for element 21
!     dle3       data edges per face for element 3
!     dle5       data edges per face for element 5
!     i          Counting variable
!     istart     Starting address
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     800   Shape number not available
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data dedpfa / 0, 0, 3, 3, 4, 4, 3, 3, 4, 3, 3, 3, 4, 4, 3, 3, 4, &
                    3 /
      data dle3 / 1, 2, 3 /
      data dle5 / 1, 2, 3, 4 /
      data dle11/ 1, 2, 3,     5,-6,-2,    -4,-3, 6,     4,-5,-1 /
      data dle13/  1,  2,  3,  4,  -5, -8, -7, -6,  -1,  9,  5,-10, &
                  -3, 11,  7,-12,  -4, 12,  8, -9,  10,  6,-11, -2 /
      data dle21/ -3, -2, -1,   4, 5, 6,    1, 8, -4,-7, &
                  2, 9, -5,-8,   3, 7, -6, -9 /
! ======================================================================
!
      if ( ishape== -10001 ) then
         nedpfa = 0

      else if ( ishape>=3 .and. ishape<=18 ) then

!     --- Standard two or three-dimensional element

         nedpfa = dedpfa(ishape)

      else if ( ishape==21 .or. ishape==22 ) then

!     --- linear prism

         if ( iface<=2 ) then

!        --- First two faces are triangles

            nedpfa = 3

         else

!        --- Last three faces are quadrilaterals

            nedpfa = 4

         end if

      else

!     --- Shape number not implemented

            call eropen  ('topfa03' )
            call errint ( ishape, 1 )
            call errsub ( 800, 1, 0, 0 )
            call erclos  ('topfa03' )
            go to 1000

      end if

      select case ( ishape )

      case (1, 2, 19, 20)

!     --- ishape = 1, 2, 19, 20: skip

      case (3, 4, 7, 8, 10)

!     --- ishape = 3, linear, quadratic triangles

         edges(1:nedpfa) = dle3 (1:nedpfa,iface)

      case (5, 6, 9)

!     --- ishape = 5, bilinear, biquadratic quadrilaterals

         edges(1:nedpfa) = dle5 (1:nedpfa,iface)

      case (11, 12, 15, 16, 18)

!     --- ishape = 11, 12 linear or quadratic tetrahedron

         edges(1:nedpfa) = dle11 (1:nedpfa,iface)

      case (13, 14, 17)

!     --- ishape = 13, trilinear hexahedron

         edges(1:nedpfa) = dle13(1:nedpfa,iface)

      case (21,22)

!     --- ishape = 21, linear or quadratic prism

         if ( iface<=2 ) then

!        --- First two faces are triangles

            istart = 3*(iface-1)

         else

!        --- Last three faces are quadrilaterals

            istart = 6+4*(iface-3)

         end if
         do i = 1, nedpfa
            edges(i) = dle21(istart+i)
         end do

      end select  ! case ( ishape )

1000  end
