      subroutine toplph ( ivec, ielshape, constunk, kprobelc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.2  date 01-02-2012 Debug statements
!        version  4.1  date 19-02-1999 Minor Correction
!        version  4.0  date 05-02-1999 Extra parameter constunk
!
!   copyright (c) 1982-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     array kprob part e is filled when the number of degrees of
!     freedom in the nodal points is not constant
!     kmesh part c and kprob part e must be stored in-core
! **********************************************************************
!
!                       KEYWORDS
!
!     topology
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekprob
      use sepmodulekmesh
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ivec, ielshape(2,*), kprobelc(npoint+1)
      logical constunk

!     constunk   o    If true the number of unknowns per point is constant
!     ielshape   i    Array of length 2*(nelgrp+numnat) containing:
!                     (1,i) number of nodes in element group i
!                     (2,i) shape number of elements in element group i
!                     if i>nelgrp: i-nelgrp = boundary group number
!                     otherwise it is the element group number
!     ivec       i    Sequence number of vector of special structure
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, ielem, ipkpre, &
              ikelmc, inpelm, ielgrp, itype, iielem, nodp, &
              inunkp, ih, jnunkp

!     ielem          Element number
!     ielgrp         element group number
!     ih             Help variable to store a constant temporarily
!     iielem         Relative element number in element group
!     ikelmc         position in kmesh part c
!     inpelm         number of points per element
!     inunkp         Number of degrees of freedom in first point
!     ipkpre         Starting position of array kprob part e in array KPROB
!     itype          Type number of element
!     j              Counting variable
!     jnunkp         Number of degrees of freedom in a point
!     nodp           Nodal point sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     284   number of degrees of freedom in common nodal point in different
!           elements is not constant for array of special structure.
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'toplph' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from toplph'
  1      format ( a, 1x, (10i7) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) &
         write(irefwr,1) 'npoint, kelma, kelmf, kelmg, nelgrp', &
                          npoint, kelma, kelmf, kelmg, nelgrp

!     --- clear kprob part e

      kprobelc(1:npoint) = 0

      ielem = 0
      ipkpre = 0
      ikelmc = 0
      inpelm = npelm
      inelem = nelem
      numnat = kprobloc(12)

      if ( debug ) then
         write(irefwr,1) 'indprb, ipkpre', indprb, ipkpre
         write(irefwr,1) 'inpelm, inelem, numnat', &
                          inpelm, inelem, numnat
      end if  ! ( debug )

!     --- First fill kprobelc for internal elements

      do ielgrp = 1, nelgrp

         if ( nelgrp>1 ) then

!        --- nelgrp>1

            inelem = Topgrp(ielgrp)%inelem
            ishape = sepgetshape ( ielgrp )
            inpelm = Topgrp(ielgrp)%inpelm
            itype = kprobc(ielgrp)

            if ( debug ) &
               write(irefwr,1) 'ielgrp, inelem, ishape, itype', &
                                ielgrp, inelem, ishape, itype

            if ( itype==-1 ) then
               ikelmc = ikelmc+inelem*2
               ielem = ielem+inelem
               inelem = 0
            else if ( itype==0 .or. ishape<=-10001 ) then
               ikelmc = ikelmc+inelem*inpelm
               ielem = ielem+inelem
               inelem = 0
            end if

         end if
         do iielem = 1, inelem
            do j = 1, inpelm

               nodp = kmeshc(ikelmc+j)
               inunkp = kprobe(ipkpre+j,ivec)

               if ( kprobelc(nodp)/=0 ) then

!              --- kprobelc(nodp)#0

                  if ( inunkp/=0 .and. kprobelc(nodp)/=inunkp ) then

                     call errint ( inunkp, 1 )
                     call errint ( kprobelc(nodp), 2 )
                     call errint ( iielem+ielem, 3 )
                     call errint ( nodp, 4 )
                     call errint ( ivec, 5 )
                     call errsub ( 284, 5, 0, 0 )

                  end if

               else if ( inunkp/=0 ) then

!              --- kprobelc(nodp)  =  0

                  kprobelc(nodp) = inunkp

               end if

            end do
            ikelmc = ikelmc+inpelm
         end do
         ipkpre = ipkpre+inpelm
         ielem = ielem+inelem

      end do

!     --- Next fill for boundary elements

      ikelmc = 0
      do ielgrp = nelgrp+1, nelgrp+numnat
         inpelm = ielshape(1,ielgrp)
         inelem = kprobb(ielgrp-nelgrp)
         do iielem = 1, inelem
            do j = 1, inpelm
               nodp = kprobi(ikelmc+j)
               inunkp = kprobe(ipkpre+j,ivec)
               if ( inunkp>0 ) then
                  if ( kprobelc(nodp)/=0 ) then

!                 --- kprobelc(nodp)#0

                     if ( kprobelc(nodp)/=inunkp ) then
                        call errint ( inunkp, 1 )
                        call errint ( kprobelc(nodp), 2 )
                        call errint ( iielem+ielem, 3 )
                        call errint ( nodp, 4 )
                        call errint ( ivec, 5 )
                        call errsub ( 284, 5, 0, 0 )
                     end if

                  else

!                 --- kprobelc(nodp)  =  0

                     kprobelc(nodp) = inunkp

                  end if
               end if
            end do
            ikelmc = ikelmc+inpelm
         end do
         ipkpre = ipkpre+inpelm
      end do

!     --- accumulate part e

      ih = 0
      constunk = .true.

      inunkp = kprobelc(1)
      do j = 1, npoint
         jnunkp = kprobelc(j)
         kprobelc(j) = ih
         ih = ih+jnunkp
         if ( inunkp/=jnunkp ) constunk = .false.
      end do
      kprobelc(npoint+1) = ih

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End toplph'

      end if  ! ( debug )
      call erclos ( 'toplph' )

      end
