      subroutine eldervcheck ( elemvec, icouno )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 25-02-2010 Extra parameter icouno
!        version  1.0  date 01-10-2009
!
!   copyright (c) 2009-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Check if array elderv has been filled correctly in subroutine
!     eldervsubr
! **********************************************************************
!
!                       KEYWORDS
!
!     derivatives
!     vector
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icouno
      double precision elemvec(0:10*icouno)

!     elemvec        i    Local array elemvc that is filled by the user
!                         This array is extended with some positions to check
!                         overwriting by the user
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      logical zero, error1, error2

!     error1         If true warning 1 has already been printed
!     error2         If true warning 1 has already been printed
!     i              Counting variable
!     zero           If true all elements in the vector are zero
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3023   elemvec(0) # 0
!    3024   elemvec(i) # 0
!    3025   elemvc = 0
!    3026   elemvc = 0 from position inpelm+1
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data error1 /.false./, error2 /.false./
! ======================================================================
!
      call eropen ( 'eldervcheck' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eldervcheck'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( elemvec(0)/=0d0 ) then

!     --- Error elemvec(0) has non-zero value

         call errint ( ielem, 1 )
         call errint ( ielgrp, 2 )
         call errint ( itype, 3 )
         call errsub ( 3023, 3, 0, 0 )
         go to 1000

      end if  ! ( elemvec(0)/=0d0 )

      do i = icouno+1, 10*icouno

         if ( elemvec(i)/=0d0 ) then

!        --- Error elemvec(i) has non-zero value

            call errint ( i, 1 )
            call errint ( icouno, 2 )
            call errint ( ielem, 3 )
            call errint ( ielgrp, 4 )
            call errint ( itype, 5 )
            call errsub ( 3024, 5, 0, 0 )
            go to 1000

         end if  ! ( elemvec(i)/=0d0 )

      end do  ! i = icouno+1, 10*icouno

!     --- Check if element vector contains non-zero elements

      zero = .true.
      do i = 1, icouno
         if ( elemvec(i)/=0d0 ) zero = .false.
      end do  ! i = 1, icouno

      if ( .not. error1 .and. zero ) then

!     --- element vector is zero

         call errint ( ielem, 1 )
         call errint ( ielgrp, 2 )
         call errint ( itype, 3 )
         call errwar ( 3025, 3, 0, 0 )
         error1 = .true.
         go to 1000

      end if  ! ( zero )

      if ( icouno>inpelm ) then

!     --- icouno > inpelm, check if at least one of the entries is non-zero

         zero = .true.
         do i = inpelm+1, icouno
            if ( elemvec(i)/=0d0 ) zero = .false.
         end do  ! i = 1, icouno
         if ( .not. error2 .and. zero ) then

!        --- element vector is zero after entry inpelm

            call errint ( inpelm, 1 )
            call errint ( icouno/inpelm, 2 )
            call errint ( icouno, 3 )
            call errint ( ielem, 4 )
            call errint ( ielgrp, 5 )
            call errint ( itype, 6 )
            call errwar ( 3026, 6, 0, 0 )
            error2 = .true.
            go to 1000

         end if  ! ( zero )

      end if  ! ( icouno>inpelm )

1000  call erclos ( 'eldervcheck' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End eldervcheck'

      end if  ! ( debug )

      end
