      function elint ( jcheli, kdegfd, coorlc, iuser, user, vectorlc, &
                       index1, index2 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.4  date 07-12-2010 Change sequence
!        version  1.3  date 13-01-2004 Use iseqin to define vector
!        version  1.2  date 02-03-2003 Extra option jcheli = 999
!
!   copyright (c) 1998-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      User element subroutine to compute integrals to be used by SEPRAN
!      This is the general version, which produces an error message if it is
!      called except in the case of the Numerical Analysis Lab of Delft
!      University of Technology
! **********************************************************************
!
!                       KEYWORDS
!
!     integral
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmodulecactl
      use sepmodulecomio
      use sepmodulemach
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jcheli, kdegfd, iuser(*), index1(inpelm), &
              index2(numold,icount)
      double precision coorlc(ndimlc,*), elint, user(*), vectorlc(*)

!     coorlc       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     elint      o    The user must give elint the value of the integral over
!                     the element
!     index1     i    Array of length inpelm containing the point numbers
!                     of the nodal points in the element
!     index2     i    Array of length icount containing the positions of the
!                     degrees of freedom in the element with respect to usol
!     iuser      i    Integer user array to pass user information from
!                     main program to subroutine. See STANDARD PROBLEMS
!     jcheli     i    Defines the type of integral to be computed
!     kdegfd     i    The degree of freedom kdegfd in vector is used
!     user       i    Real user array to pass user information from
!                     main program to subroutine. See STANDARD PROBLEMS
!     vector     i    In this array all preceding solutions are stored, i.e.
!                     all solutions that have been computed before and
!                     have been carried to integral by the parameter
!                     islold in the parameter list of these main subroutines
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer lenhome, lenarray, inpelmcopy, i, &
              icountcopy, itypecopy, jchelicopy
      parameter ( lenarray=100 )
      double precision x(lenarray), u(lenarray), sum, y(lenarray)
      character * 80 filenm
      logical check

!     check       Indicates if the file exist (true) or not
!     filenm      Name of file
!     i           General loop variable
!     icountcopy  Copy of icount
!     inpelmcopy  Copy of inpelm
!     itypecopy   Copy of itype
!     jchelicopy  Copy of jcheli
!     lenarray    Length of arrays u and x
!     lenhome     Number of characters in the name of the SEPRAN home directory
!     ndimcopy    Copy of ndimlc
!     sum         Help parameter to compute a sum
!     u           array of length m containing the values of vector1 in the
!                 nodal points of the element
!     x           Contains the x-coorlcdinates of the nodes of the element using
!                 the local node numbering
!     y           Contains the y-coorlcdinates of the nodes of the element using
!                 the local node numbering
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision elintsubr

!     ELINTSUBR Special user element subroutine to be called by elint and
!               meant for the numerical analysis lab at Delft University
!               of Technology
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRCHR    Put character in error message
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
!     INSTOP    Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     349   Subroutine not available
!     550   array length too short
! **********************************************************************
!
!                       PSEUDO CODE
!
!    If ( file sphome/bin/update/wi211 exists ) then
!       prepare call to elintsubr
!       call elintsubr
!       give elint a value
!    else
!       give error message
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elint' )

      lenhome = index(sphome,' ')-1
      filenm = sphome(1:lenhome)//'/bin/update/wi211'
      inquire ( file=filenm, exist=check )

      if ( jcheli==999 ) then

!     --- Special case of jcheli = 999, add absolute values of solution

         do i = 1, icount
            u(i) = vectorlc(index2(iseqin,i))
         end do
         sum = 0d0
         do i = 1, icount
            sum = sum+abs(u(i))
         end do
         elint = sum

      else if ( check ) then

!     --- wi211 file found, Hence special situation for the "wi211-lab" at
!         Delft University
!         First check array boundaries

         if ( inpelm*ndimlc>lenarray ) then
            call errint ( inpelm*ndimlc, 1 )
            call errint ( lenarray, 2 )
            call errchr ( 'x', 1 )
            call errsub ( 550, 2, 0, 1 )
         end if
         if ( icount>lenarray ) then
            call errint ( icount, 1 )
            call errint ( lenarray, 2 )
            call errchr ( 'u', 1 )
            call errsub ( 550, 2, 0, 1 )
         end if
         if ( ierror/=0 ) go to 1000

!        --- Copy co-ordinates from coorlc into x

         x(1:inpelm) = coorlc(1,index1(1:inpelm))
         y(1:inpelm) = coorlc(2,index1(1:inpelm))

!        --- Copy old solution into u

         u(1:icount) = vectorlc(index2(iseqin,1:icount))
         inpelmcopy = inpelm
         icountcopy = icount
         itypecopy = itype
         jchelicopy = jcheli

         elint = elintsubr ( inpelmcopy, x, y, icountcopy, &
                             u, itypecopy, jchelicopy )
      else

!     --- Standard situation: give error message

         call errchr ( 'elint', 1 )
         call errsub ( 349, 0, 0, 1 )

      end if

1000  call erclos ( 'elint' )
      if ( ierror/=0 ) call instop

      end
