      double precision function fndetf ( ichoice, eps, detf, matpar )
! ======================================================================
!
!        programmer     Sander Gielen
!        version 2.0    date   20-01-1998 added ichoice
!        version 1.0    date   26-08-1996
!
!
!
!   copyright (c) 1996-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!  function derived for eld250
!
! **********************************************************************
!
!                       KEYWORDS
!
!      elasticity_equation
!      solid
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      double precision eps(6), detf, matpar(*)
      integer ichoice

!     detf       i    determinant of deformation gradient
!     eps        i    green-lagrange strain
!     fndetf     o    value for eld250
!     ichoice    i    choice variable: (icoef4)
!     matpar     i    parameters of incompressiblity relation
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision b, g, penal
      integer icomprs

!     b          exponent of compression function (type 3 and 4)
!     g
!     icomprs    compression function number
!     penal      penalty factor (1/bulk-modulus)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!
!     ichoice = icoef4 = imateri + 10*icomprs + 100*iadd
!
!     if a programmed model is chosen, then compute pressure according
!     to model, otherwise return detf.
!
!     See fncomprs for details.
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      icomprs = ichoice/10 - 10*(ichoice/100)

      penal = 1d0/matpar(6)
      b     = matpar(7)

      goto ( 110, 120, 130, 140, 150 ) icomprs

!     --- no compression model number found. Just return detf

      fndetf = detf
      goto 999

 110  g = detf-1d0
      goto 200

 120  g = log(detf)
      goto 200

 130  if ( b.eq.0d0 ) b = 1d0/6d0
      g = (1d0/(2d0*b)) * ( detf**(2d0*b)-1d0)
      goto 200

 140  if ( b.eq.0d0 ) b = 10d0
      g = (1d0/(b-1d0)) * ( (1d0/detf)-(1d0/detf**b))
      goto 200

 150  g = 5d-1*(detf*detf -1d0)*detf
      goto 200

 200  fndetf = - g/penal

 999  continue
      end
