      subroutine fnexpon1 ( eps, matpar, s, se, makese )
! ======================================================================
!
!        programmer     Marcel Rutten
!        version 1.0    date   21-01-1998
!
!
!
!   copyright (c) 1998-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!    fill estimate of 2nd Piola Kirchoff stress plus tangential stiffness
!    matrix for one integration point.
!    exponentially fitting version 1
! **********************************************************************
!
!                       KEYWORDS
!
!     non_linear
!     solid_mechanics
!     material_behaviour
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      logical makese
      double precision eps(6), matpar(21), s(6), se(6,6)

!     eps      i     Green-Lagrange strain
!     makese   i     flag for building tangential stiffness matrix
!     matpar   i     material properties
!     s        o     2nd Piola Kirchhoff stress estimate
!     se       o     tangential stiffness matrix
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision a0, a1, a2, a3, a4, a12, re, sc, f, df(6), &
                       ddf(6,6), we

!     a0
!     a1
!     a12
!     a2
!     a3
!     a4
!     ddf
!     df
!     f
!     i          Counting variable
!     j          Counting variable
!     re
!     sc
!     we
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data df /6* 0d0/
      data ddf /36*0d0/
! ======================================================================
!

!     --- exponential stiffening law:

!         w = a0 exp(f)

      a0 = matpar(1)
      a1 = matpar(2)
      a2 = matpar(3)
      a3 = matpar(4)
      a4 = matpar(5)
      a12= 2d0*a1 - a2

      re = eps(1)*eps(1) + eps(2)*eps(2) + eps(3)*eps(3)
      sc = eps(4)*eps(4) + eps(5)*eps(5) + eps(6)*eps(6)

      f =  a1*re + a3*eps(1)*eps(1) + &
           a12*(eps(1)*eps(2) + eps(2)*eps(3) + eps(3)*eps(1)) + &
           a2*sc + a4*(eps(4)*eps(4)+eps(6)*eps(6))

      we = a0* exp( f )

      df(1) = 2d0*(a1+a3)*eps(1) + a12*(eps(2)+eps(3))
      df(2) = 2d0*  a1*   eps(2) + a12*(eps(1)+eps(3))
      df(3) = 2d0*  a1*   eps(3) + a12*(eps(1)+eps(2))

!     --- because of symmetry, derivatives to 4..6 should be counted for 1/2

      df(4)= (a2+a4)*eps(4)
      df(5)=  a2 *   eps(5)
      df(6)= (a2+a4)*eps(6)

!     --- stress at end of increment

      do i=1,6
         s(i) = df(i)*we
      end do

      if ( makese ) then

        ddf(1,1) = 2d0*(a1+a3)
        ddf(1,2) = a12
        ddf(1,3) = a12
        ddf(2,1) = a12
        ddf(2,2) = 2d0*a1
        ddf(2,3) = a12
        ddf(3,1) = a12
        ddf(3,2) = a12
        ddf(3,3) = 2d0*a1
        ddf(4,4) = a2 + a4
        ddf(5,5) = a2
        ddf(6,6) = a2 + a4

!       --- tangential stiffness matrix

        do i = 1, 6
           do j = 1, 6
              se(i,j) = ( ddf(i,j) + df(i)*df(j) )*we
           end do
        end do

      end if

      end
