      subroutine fnorthot ( eps, matpar, s, se )
! ======================================================================
!
!        programmer     Marcel Rutten
!        version 1.0    date   21-01-1998
!
!   copyright (c) 1998-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
! fill estimate of 2nd Piola Kirchoff stress plus tangential stiffness
! matrix for one integration point. (orthotropic linear elastic)
! **********************************************************************
!
!                       KEYWORDS
!
!      elasticity_equation
!      solid
!      material_behaviour
!      orthotropic_linear_elastic
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      double precision eps(6), matpar(21), s(6), se(6,6)

!     eps      i     Green-Lagrange strain
!     matpar   i     material properties
!     s        o     2nd Piola Kirchhoff stress estimate
!     se       o     tangential stiffness matrix
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision e1, e2, nu1, nu2, g1, n, a, b, c1, c2, c3, c4, &
                       c5, c6

!     a
!     b
!     c1
!     c2
!     c3
!     c4
!     c5
!     c6
!     e1
!     e2
!     g1
!     i          Counting variable
!     j          Counting variable
!     n
!     nu1
!     nu2
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- orthotropic linear elastic material:
!         (transversely isotropic, really :-)
!         structure of the stiffness matrix:
!          c1  c2  c2  0   0   0         e11
!          c2  c3  c4  0   0   0         e22
!          c2  c4  c3  0   0   0         e33
!          0   0   0   c5  0   0         g12
!          0   0   0   0   c5  0         g13
!          0   0   0   0   0   c6        g23

      E1  = matpar(1)
      E2  = matpar(2)
      nu1 = matpar(3)
      nu2 = matpar(4)
      G1  = matpar(5)

      n = E2/E1
      A = 1d0 + nu2
      B = 1d0 - nu2 - 2d0*n*nu1*nu1

      c1 = ( 1d0 - nu2 ) * E1 / B
      c2 = nu1 * E2 / B
      c3 = ( 1d0 - n * nu1 * nu1 ) * E2 / ( A * B )
      c4 = ( nu2 + n * nu1 * nu1 ) * E2 / ( A * B )
      c5 = G1
      c6 = E2 / ( 2 * A )

      se(1,1) = c1
      se(1,2) = c2
      se(2,1) = c2
      se(1,3) = c2
      se(3,1) = c2
      se(2,2) = c3
      se(2,3) = c4
      se(3,2) = c4
      se(3,3) = c3
      se(4,4) = c5
      se(5,5) = c5
      se(6,6) = c6

      do i=1,3
         do j=1,3
            s(i) = s(i) + se(i,j)*eps(j)
         end do
         s(i+3) = se(i+3,i+3)*eps(i+3)
      end do

      end
