      subroutine el0100 ( iuser, user, jelp, kfirst, jheat, isub, &
                          isup, array, icheld, ix )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.14 date 19-05-2003 New definition of jelp
!        version  4.13 date 23-08-2001 New call to elusr3
!        version  4.12 date 29-09-1998 Include common
!
!   copyright (c) 1989-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initialisations for subroutine ELM100 or
!     ELD100 in the case that IFIRST = 0
!     The parameters in the common blocks celint, celiar and celp
!     get a start value
!     Array array is initialised with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*), array(*)
      integer iuser(*), jelp, kfirst, jheat, isub, isup, icheld, ix

!     array   o      Real work array in which the values of the coefficients
!                    in the integration points are stored in the sequence:
!                    1:   alpha 11
!                    2:   alpha 12
!                    3:   alpha 22
!                    4:   u 1
!                    5:   u 2
!                    6:   beta
!                    7:   f
!                    8:   rho cp
!                    Each coefficient requires exact m (is number of
!                    integration points) positions
!     icheld   i     Choice parameter to distinguish between the various
!                    possibilities of this subroutine
!                    0:   Call by ELM100
!                    >0:  Call by ELD100
!                    <0:  Call by ELI100
!                    1:   compute the derivative in ix direction
!                         ix = 1    du/dx
!                         ix = 2    du/dy
!                    2:   grad u
!                    3:   - grad u
!                    4:   elemvc(1)=du/dy   elemvc(2)=-du/dx  ( curl u )
!                  5-8:   See 1-4, however, now computed per element
!                    9:   Only the function given through user and iuser is
!                         stored in the output vector
!                         This possibility is only available for itype = 100
!                   10:  ........
!                   14:   compute the derivative in ix direction (per node)
!                         ix = 1    du/dx
!                         ix = 2    du/dy
!                   15:   - A grad u
!                         where A is the coefficient matrix for the second order
!                         part of the elliptic equation
!                   16:   - A grad u, now computed per element
!                   -1:   / f(x) dx
!                   -2:   / f(x) u(x) dx
!                   -3-5: / f(x) du(x)/dx  dx   i=-icheld-2
!                                        i
!     isub    o      Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup    o      Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     iuser    i     User array in which integer information of the
!                    coefficients is stored (See Standard Problems)
!     ix       i     indication what type of derivative must be computed
!                    See DERIVA
!     jelp    o      Indication of the type of basis function subroutine must
!                    be called by ELM100. Possible vaues:
!                    1:  Subroutine ELP620  (Linear triangle)
!                    2:  Subroutine ELP621  (Bilinear quadrilateral)
!                    3:  Subroutine ELP622  (Quadratic triangle)
!                    4:  Subroutine ELP623  (Biquadratic quadrilateral)
!     jheat   o      Integer parameter indicating if the mass matrix for the
!                    heat equation must be computed (>0) or not ( = 0)
!                    If jheat = 1, then the coefficient for the mass matrix is
!                    constant, otherwise (jheat = 2) it is variable
!     kfirst  o      This parameter indicates the smallest velocity component
!                    that is not equal to zero in the convective terms
!                    For example kfirst = 2, means that the y- &
!                    component of the velocity is unequal to zero
!     user    i      User array in which real information of the
!                    coefficients is stored (See Standard Problems)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jwcons, ncoef, i, j, isubcf, icoef, kchois
      logical funchk

!    funchk        Indicates whether there are coefficients that must be
!    i             Counting variable
!    icoef         Coefficient sequence number
!    isubcf        Under bound for coefficients check
!    j             Counting variable
!    jwcons        Indication if the weight function is constant (1) or not (2)
!    kchois        Indication what type of parameters must be computed
!                  Possible values:
!                  0:  Standard method, the calling subroutine is ELM100
!                      It is supposed that an element matrix must be computed
!                  1:  Subroutine ELD100 calls this subroutine, only
!                      derivatives must be computed
!                  2:  Subroutine ELD100 or ELI100 calls this subroutine, &
!                      a user function defined as first parameter in
!                      IUSER/USER must be filled in elemvc (icheld=9)
!                  3:  Subroutine ELD100 calls this subroutine, &
!                      alpha times the gradient must be stored in ELEMVC
!                      The first 3 parameters of IUSER/USER are used
!    ncoef         Number of coefficients in the equation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELCOMM  Initializes the commons celp and celint with standard values
!     ELUSR3  Extracts information concerning the coefficients from IUSER
!             and USER.
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Print error messages
!     ERRWAR  Print warnings
!     INSTOP  Stop the program
!     PRINLG  Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     1:  alpha 11
!     2:  alpha 12
!     3:  alpha 22
!     4:  u 1
!     5:  u 2
!     6:  beta
!     7:  f
!     8:  rho cp
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       3  Fatal error: INPELM has wrong value
!      42  Fatal error: IUSER(6) smaller than 7
!      70  Warning:     alpha11 = 0 or alpha22 = 0
!      74  Fatal error: ICOUNT has wrong value
!     616  Fatal error: METUPW has wrong value
!     771  Fatal error: Rho is non-positive
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Fill some standard values in the common blocks celp and celint

      call eropen ( 'el0100' )
      call elcomm
      ndimlc = 2

!     --- Compute kchois

      if ( icheld.eq.0 ) then

!     --- Call by elm100, all coefficients required

         kchois = 0

      else if ( icheld.eq.9 .or. icheld.lt.0 ) then

!     --- call by eld100, icheld = 9, one (new) coefficient required
!         or call by eli100, one (new) coefficient required

         kchois = 2

      else if ( icheld.le.8 .or. icheld.eq.14 ) then

!     --- call by eld100, icheld < 9, or icheld = 14, i.e. no coefficients
!         required

         kchois = 1

      else

!     --- call by eld100, 9 < icheld < 14, or icheld = 15
!         The first three standard coefficients are required

         kchois = 3

      end if

!     --- if derivatives must be computed, the weights are not necessary

      if ( icheld.gt.0 .and. icheld.lt.10 .or. icheld.gt.13 ) iweigh = 0

!     --- Set parameter upwind

      if ( itype.ge.120 .and. itype.le. 200 .or. itype.eq.510 ) then

         metupw = 1

      else

         metupw = 0

      end if

!     --- Compute jderiv ( if icheld#9 ) or ideriv ( if icheld=9 ), ndimlc

      if ( icheld.eq.1 .or. icheld.eq.5 .or. icheld.eq.14 ) then

!     --- icheld = 1 or 5

         jderiv = ix + 1
         if ( icheld.eq.14 ) jderiv = jderiv + 4
         ndimlc   = 1

      else if ( icheld.eq.9 .or. icheld.eq.-1 .or. icheld.eq.-2 ) then

!     --- icheld = -1, -2, or 9, no derivatives should be computed

         ideriv = 0

      else if ( icheld.lt.-2 ) then

!     --- icheld < -2, derivatives should be computed

         jderiv = -icheld+3

      else if ( icheld.le.8 .or. icheld.eq.15 .or. icheld.eq.16 ) then

!     --- icheld = 2, 3, 4, 6, 7, 8, 15, 16

         jderiv = 5

      else if ( icheld.ge.10 .and. icheld.le.13 ) then

!     --- icheld = 10, 11, 12, 13

         jderiv = 1

      end if

!     --- Set parameters depending on the type numbers

      if ( itype.eq.100 .or. itype.eq.101 .or. itype.eq.120 .or. &
           itype.eq.190 .or. itype.eq.300 .or. &
           itype.eq.502 .or. itype.eq.503 .or. itype.eq.510 ) then

!     --- itype = 100,101,120,190,502,503,510:  Linear triangle

         n = 3
         m = 3
         mn = 3
         jelp = 3
         jdercn = 0
         if ( itype.eq.101 .or. itype.eq.503 ) jcart = 2
         jwcons = jcart
         nvert = 3

      else if ( itype.eq.104 .or. itype.eq.105 .or. itype.eq.124 .or. &
                itype.eq.509 ) then

!     --- itype = 104,105,124,509  Quadratic triangle

         n = 6
         m = 6
         jelp = 4
         jwcons = 2
         if ( itype.eq.105 ) then

!        --- Axi-symmetric co-ordinates

            jcart = 2

         else if ( icheld.eq.0 .and. itype.ne.124 ) then

!        --- Cartesian co-ordinates

            kstep = 2

         end if
         mn = m*n
         nvert = 3

      else if ( itype.eq.108 .or. itype.eq.109 .or. itype.eq.504 &
               .or. itype.eq.505 ) then

!     --- itype = 108,109, 504, 505:  Bilinear quadrilateral

         n = 4
         m = 4
         mn = 16
         jelp = 5
         if ( itype.eq.109 .or. itype.eq.505 ) jcart = 2
         jwcons = 2
         nvert = 4

      else

!     --- itype = 110,111:   Biquadratic quadrilateral
!         For the computation of derivatives, always use irule = 1, i.e.
!         fill all values in nodes

         n = 9
         m = 9
         jelp = 6
         jwcons = 2
         if ( itype.eq.111 ) jcart = 2
         mn = m*n
         nvert = 4
      end if
      if ( kchois.eq.0 .and. itype.gt.500 ) then

!     --- itype > 500,  Heat equation

         ncoef = 8
         jheat = 1

      else if ( kchois.eq.1 ) then

!     --- kchois = 1, no coefficients required

         ncoef = 0
         jheat = 0
         metupw = 0

      else if ( kchois.eq.2 ) then

!     --- kchois = 2, one coefficient required

         ncoef = 1
         jheat = 0
         metupw = 0

      else if ( kchois.eq.3 ) then

!     --- kchois = 3, three coefficients required

         ncoef = 3
         jheat = 0
         metupw = 0

      else

!     --- itype < 500,  Elliptic equation

         ncoef = 7
         jheat = 0

      end if

!     --- Fill the arrays IND, MST and ARRAY with information of
!         the coefficients

      if ( ncoef.gt.0 ) then
         ipos = iuser ( ielgrp + 5 )
         if ( ipos.lt.7 ) then
            call errint ( ielgrp+5, 1 )
            call errint ( ipos, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub ( 42, 5, 0, 0 )
            go to 1000
         end if
         nuser = 0
         do i = 1 , ncoef
            call elusr3 ( i, array(1+(i-1)*m), iuser, user, i )
         end do
         if ( metupw.gt.0 ) then

!        --- metupw > 0, upwinding method

            metupw = iuser(ipos)
            if ( metupw.le.0 .or. metupw.gt.4 ) then
               call errint ( metupw, 1 )
               call errint ( ielem, 2 )
               call errint ( itype, 3 )
               call errint ( ielgrp, 4 )
               call errsub ( 616, 4, 0, 0 )
               go to 1000
            end if
            ipos = ipos + 1
            iphi = 1

         end if

!        --- Test array lengths, number of nodes and number of
!            degrees of freedom

         call prinlg ( 'iuser', ipos-1, user, iuser )
         call prinlg ( 'user', -nuser, user, iuser )

      end if

      if ( inpelm.ne.n ) then
         call errint ( inpelm, 1 )
         call errint ( n, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 3, 5, 0, 0 )
         go to 1000
      end if

!     --- icount is not used, only checked in this subroutine

      if ( icount.ne.n ) then
         call errint ( icount, 1 )
         call errint ( n, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 74, 5, 0, 0 )
         go to 1000
      end if

!     --- Next consider all terms carefully in order to get the cheapest
!         computation

      if ( kchois.eq.0 .and. (matrix .or. metupw.gt.0) .or. kchois.eq.3) &
           then

!     --- matrix = true  The element matrix must be computed.
!         or metupw > 0:  Upwinding
!                        Compute jsecnd, idim and jzero
!         --- First check if alpha  , and alpha   are unequal to zero
!                             11           22

         if ( ind(1).eq.0 .or. ind(3).eq.0 ) then

!        --- alpha 11 or alpha 22 are zero

             call errwar ( 70, 0, 0, 0)
          end if

!         --- Second derivative terms:
!             --- compute jsecnd

         if ( ind(2).eq.0 ) then

!        --- alpha 12 = 0

            if ( ind(1).eq.0 .and. ind(3).eq.0 ) then

!           --- alpha = 0

               jsecnd = 0

            else if ( ind(1).le.0 ) then

!           --- alpha 11 is constant

               if ( ind(3).eq.ind(1) ) then

!              --- alpha 11 = alpha 22

                  jsecnd = jwcons

               else if ( ind(3).le.0 ) then

!              --- alpha 22 is constant

                  if ( abs ( array(1)-array(1+2*m) ).gt.sqreps* &
                       abs(array(1))) then

!                 --- alpha 11 = alpha 22

                     jsecnd = 4

                  else

!                 --- alpha 11 # alpha 22

                     jsecnd = jwcons

                  end if

               else

!              --- alpha 22 is variable

                  jsecnd = 4

               end if

            else if ( ind(1).eq.ind(3) .and. mst(1).eq.mst(3) ) then

!           --- alpha 11 = alpha 22  variable

               jsecnd = 2

            else

!           --- general, except for alpha 12 = 0

               jsecnd = 4

            end if

         else

!        --- alpha 12 # 0

            if ( ind(1).le.0 ) then

!           --- alpha 11 = constant

               if ( ind(3).eq.ind(1) ) then

!              --- alpha 11 = alpha 22

                  jsecnd = 3

               else if ( ind(3).le.0 ) then

!              --- alpha 22 is constant

                  if ( abs( array(1)-array(1+2*m) ).gt.sqreps* &
                       abs(array(1))) then

!                 --- alpha 11 # alpha 22

                     jsecnd = 4

                  else

!                 --- alpha 11 = alpha 22

                     jsecnd = 3

                  end if

               else

!              --- alpha 22 is variable

                  jsecnd = 4

               end if

            else

!           --- alpha 11 is variable

               if ( ind(1).eq.ind(3) .and. mst(1).eq.mst(3) ) then

!              --- alpha 11 = alpha 22  variable

                  jsecnd = 3

               else

!              --- general

                  jsecnd = 4

               end if

            end if

         end if

!        --- Convective terms:
!            compute idim and first component idim

         kfirst = 1
         if ( ind(4).eq.0 ) then

!        --- u1 = 0

            if ( ind(5).eq.0 ) then

!           --- u1 = 0 and u2 = 0

               idim = 0

            else

!           --- u1 = 0, u2 # 0

               idim = 1
               kfirst = 2
            end if

         else if ( ind(5).eq.0 ) then

!        --- u2 = 0 and u1 # 0

            idim = 1

         else

!        --- General

            idim = 2

         end if

!        --- Constant part of the equation:
!            compute jzero

         if ( ind(6).eq.0 ) then

!        --- beta = 0

            jzero = 0

         else if ( ind(6).lt.0 .or. ind(6).eq.2002 ) then

!        --- beta is constant (per element)

            jzero = jwcons

         else

!        --- General

            jzero = 2

         end if

      end if

      if ( kchois.eq.0 .and. vector .or. kchois.eq.2 ) then

!     --- Vector = true, the element vector must be computed
!                        or kchois = 2, the first parameter must be computed
!         Compute jconsf

         if ( kchois.eq.0 ) then

            icoef = 7

         else

            icoef = 1

         end if
         if ( ind(icoef).eq.0 ) then

!        --- F = 0, no right-hand side

            notvec = 0

         else

!        --- General

            jconsf = 2

         end if

      end if

!     --- Check which parameters depend on the space co-ordinates
!         Compute isub and isup

      if ( matrix .and. kchois.eq.0 .or. kchois.ge.2 ) then
         isubcf = 1
      else
         isubcf = 7
      end if

      do i = isubcf , ncoef
         isub = i

!        --- If ind(i) > 0 then paramater i corresponds to a function

         if ( ind(i).gt.0 ) goto 170

      end do

      isub = ncoef + 1

170   isup = ncoef
      do i = ncoef, isub , -1
         isup = i
         if ( ind(i).gt.0 ) goto 190

      end do

!     --- Check parameter rho cp if the heat equation is solved

190   if ( jheat.gt.0 ) then

!     --- jheat > 0, i.e. compute mass matrix

         if ( ind(8).eq.0 .or. ind(8).lt.0 .and. array(8*m).le.0 ) then
            call errint ( ielem, 1 )
            call errint ( itype, 2 )
            call errint ( ielgrp, 3 )
            call errsub ( 771, 3, 0, 0 )
            go to 1000
         else if ( ind(8).lt.0 .or. ind(8).eq.2002 ) then
            jheat = jwcons
         else
            jheat = 2
         end if

      end if

!     --- Check for double parameters (functions) to avoid calls for
!         function subroutines
!         ist is used to store the references for the double parameters
!         Set ind(lengthmst)

      do i = isub, isup
         ist(i) = 0
      end do
      ind(lengthmst) = 0
      funchk = .false.
      do i = isub, isup
         if ( ind(i).gt.0 .and. ind(i).lt.2000 ) then

!        --- only the function calls are checked

            do j = i+1,isup
               if ( ind(j).eq.ind(i) ) then
                  ist(j) = i
                  ind(j) = 0
               end if
            end do
         end if
         if ( ind(i).gt.1000 .and. ind(i).le.2000 ) ind(lengthmst) = 1
         if ( ind(i).gt.10000 ) funchk = .true.
      end do

      if ( icheld.lt.-1 .or. icheld.gt.0 .and. icheld.ne.9 ) &
         ind(lengthmst) = 1

      if ( funchk ) ind(lengthmst) = ind(lengthmst) + 2

1000  if ( ierror.ne.0 ) call instop

      call erclos ( 'el0100' )

      end
