      subroutine el0609 ( iuser, user,  isub, isup, array, &
                          icheld, modelv )
! ======================================================================
!
!        programmer    Patrick Anderson
!        version  1.2  date 23-08-2001 New call to elusr3
!        version  1.1  date 03-09-2000 Extension for scalars
!        version  1.0  date 05-01-1997
!
!   copyright (c) 1997-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     this subroutine performs the initialisations for subroutine ELM609
!     or ELD609 in the case that IFIRST = 0
!     the parameters in the common blocks celint and celiar get a start
!     value
!     array array is initialised with the values of the coefficients
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*), isub, isup, icheld, modelv
      double precision user(*), array(*)

!     array      o    real work array in which the values of the coefficients
!                     in the integration points are stored in the sequence:
!                     1: eta
!                     2: f1
!                     3: f2
!                     4: modelv
!     icheld     i    choice parameter to distinguish between the various
!                     possibilities of this subroutine
!                     0:   Call by ELM609
!                     >0:  Call by ELD609 (not yet done)
!     isub       o    integer help parameter for do-loops to indicate the
!                     smallest coefficient number that is dependent on space
!     isup       o    integer help parameter for do-loops to indicate the
!                     largest coefficient number that is dependent on space
!     iuser      i    user array in which integer information of the
!                     coefficients is stored (See Standard Problems)
!     modelv     o    Defines the type of viscosity model
!                     Possible values:
!                       1:  newtonian liquid
!                       2:  power law liquid
!                       3:  carreau liquid
!                       4:  plastico viscous liquid
!                     100:  user provided model depending on grad u
!                     101:  user provided model depending on second invariant
!                     102:  user provided model depending on second invariant, &
!                           the co-ordinates and the velocity
!     user       i    user array in which real information of the
!                     coefficients is stored (See Standard Problems)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncoef, i, j, isubcf, kchois
      double precision rn

!     i             counting variable
!     isubcf        under bound for coefficients check
!     j             counting variable
!     kchois        indication what type of parameters must be computed
!                   possible values:
!                   0:  standard method, the calling subroutine is ELM609
!                       it is supposed that an element matrix must be computed
!                   1:  subroutine ELD609 calls this subroutine, only
!                       derivatives must be computed
!                   2:  subroutine ELD609 calls this subroutine, &
!                       a user function defined as first parameter in
!                       IUSER/USER must be filled in elemvc (icheld=9)
!                   3:  subroutine ELD609 calls this subroutine, &
!                       alpha times the gradient must be stored in ELEMVC
!                       the first 3 parameters of IUSER/USER are used
!     ncoef         number of coefficients in the equation
!     rn            double precision value of inpelm
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ELUSR3         Extracts information concerning the coefficients from IUSER
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREPSCAL     Replace ivalue by scalar if necessary
!     INSTOP         Stop the program
!     PRINLG         Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- fill some standard values in the common block celint

      call eropen ('el0609')
      call elcomm
      ncomp = 2

!     --- compute kchois

      if (icheld.eq.0) then

!     --- call by elm609, all coefficients required

         kchois = 0
      end if

!     --- set parameters depending on the type numbers

      n      = inpelm
      m      = inpelm
      rn     = n
      mone   = int(sqrt(rn))
      jdercn = 1
      nvert  = 4
      ndimlc   = 2

!     --- fill the arrays IND, MST and ARRAY with information of the
!     coefficient

      ncoef = 4
      if ( ncoef.gt.0 ) then
         ipos = iuser(ielgrp + 5)
         if (ipos.lt.7) then
            call errint ( ielgrp+5, 1 )
            call errint ( ipos, 2 )
            call errint ( ielem, 3 )
            call errint ( itype, 4 )
            call errint ( ielgrp, 5 )
            call errsub (42, 5, 0, 0)
            go to 1000
         end if
         nuser = 0
         do i = 1 , ncoef
            call elusr3 (i, array(1+(i-1)*n), iuser, user, i)
         end do

!        --- compute modelv

         modelv = inirepscal ( iuser(ipos) )

!        --- check value of method
!            --- test array lengths, number of nodes and number of degrees of
!            freedom

         call prinlg ( 'iuser', ipos-1, user, iuser)
         call prinlg ( 'user', -nuser, user, iuser)
      end if

!     --- next consider all terms carefully in order to get the cheapest
!         computation
!         check which parameters depend on the space coordinates
!         compute isub and isup

      if ( kchois.eq.0 .and. matrix ) then
         isubcf = 1
      else
         isubcf = 4
      end if
      do i = isubcf, ncoef
         isub = i

!        --- if ind(i) > 0 then paramater i corresponds to a function

         if (ind(i).gt.0) goto 300
      end do
300   isup = ncoef
      do i = ncoef, isub , -1
         isup = i
         if (ind(i).gt.0) goto 500
      end do

!     --- check for double parameters (functions) to avoid calls for
!         function subroutines
!         ist is used to store the references for the double parameters
!         set ind(lengthmst)

500   do i = isub, isup
         ist(i) = 0
      end do
      ind(lengthmst) = 0
      do i = isub, isup
         if ( ind(i).gt.0 .and. ind(i).lt.2000 ) then

!        --- only the function calls are checked

            do j = i+1, isup
               if (ind(j).eq.ind(i) ) then
                  ist(j) = i
                  ind(j) = 0
               end if
            end do
         end if
         if ( ind(i).gt.1000 .and. ind(i).le.2000 ) ind(lengthmst) = 1
      end do
1000  if (ierror.ne.0) call instop
      call erclos ('el0609')
      end
