      subroutine el0800elp ( linear, jelp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.2  date 03-02-2015 extension for type 704
!        version  1.1  date 22-05-2014 Extension with shape 22
!
!   copyright (c) 2009-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Set parameters depending on shape of element
!
! **********************************************************************
!
!                       KEYWORDS
!
!     elliptic_equation
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jelp
      logical linear

!     jelp           o    Indication of the type of basis function subroutine
!                         must be called by ELel0800inx.
!     linear         o    Indicates whether the element is a linear simplex
!                         (true) or not (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1771   Shape number not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0800elp' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0800elp'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set parameters depending on the shape numbers

      n = inpelm
      m = n

      jelp = ishape
      if ( itype==704 ) jelp = 8   ! non-conforming triangle

      select case ( jelp )

      case (1, 3, 11)

!     --- linear element

         linear = .true.
         mn = m
         jdercn = 0

      case(7)

!     --- extended quadratic element

         linear = .false.
         jdercn = 1
         n = inpelm-1
         m = n
         jelp = 4

      case(8)

!     --- non-conforming linear triangle

         linear = .false.
         jdercn = 1
         n = 3
         m = 3

      case (2, 4:6, 9:10, 12:14, 21:22 )

!     --- Bilinear or quadratic element

         linear = .false.
         jdercn = 1

      case(15)

!     --- Bilinear or quadratic element

         if ( icheld<0 ) then
            linear = .false.
            jdercn = 1
            irule = 1
            n = 10
            m = 10
         end if  ! ( icheld<0 )

      case(16)

!     --- quadratic element

         if ( icheld<0 ) then
            linear = .false.
            jdercn = 1
            irule = 1
         end if  ! ( icheld<0 )

      case(17, 18)

!     --- Extended elements

         if ( icheld<0 ) then
            linear = .false.
            jdercn = 1
            irule = 3
            m = inpelm-1
         end if  ! ( icheld<0 )

      case default

!     --- Error 1771: this type of element has not yet been implemented

         call errint ( ishape, 1 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errsub ( 1771, 4, 0, 0 )

      end select  ! case ( jelp )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el0800elp'

      end if  ! ( debug )
      call erclos ( 'el0800elp' )

      end
