      subroutine el0800filcof ( iuser, user, isub, array, ishift, &
                                ncoef )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-01-2012
!
!   copyright (c) 2012-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill all real coefficients in array
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*), array(m,*)
      integer iuser(*), isub, ishift, ncoef

!     array          o    Array of length m * number of coefficients
!                         Contains the coefficients in the integration points
!     ishift        i/o   Shift of specific variable with respect to diagonal
!     isub           i    Integer help parameter for do-loops to indicate the
!                         smallest coefficient number that is dependent on space
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     ncoef          i    Number of coefficients in the equation
!     user           i    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELUSR3         Extracts information concerning the coefficients from IUSER
!                    and USER.
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINLG         Print and checks length of user arrays
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0800filcof' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0800filcof'
         write(irefwr,1) 'isub, ncoef, ipos, ishift', &
                          isub, ncoef, ipos, ishift
         call prinrl1 ( array, ncoef, m, 'array' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do i = isub, ncoef
         call elusr3 ( i, array(1,i), iuser, user, i+ishift )
      end do

!     --- Test array lengths, number of nodes and number of degrees
!         of freedom

      call prinlg ( 'iuser', ipos-1, user, iuser )
      call prinlg ( 'user', -nuser, user, iuser )

1000  call erclos ( 'el0800filcof' )
      if ( debug ) then

!     --- Debug information

         call prinin ( ind, ncoef, 'ind' )
         call prinin ( ist, ncoef, 'ist' )
         call prinin ( ivecnums, ncoef, 'ivecnums' )
         call prinrl1 ( array, ncoef, m, 'array' )
         write(irefwr,*) 'End el0800filcof'

      end if  ! ( debug )

      end
