      subroutine el0800jheat ( array, jheat, jelp, jwcons )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 20-01-2012
!
!   copyright (c) 2012-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the parameter jheat
!     ishift and ipos are adapted
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jheat, jwcons, jelp
      double precision array(m,*)

!     array         i/o   Array of length m * number of coefficients
!                         Contains the coefficients in the integration points
!     jelp           i    Indication of the type of basis function subroutine
!                         must be called by ELxxxx.
!     jheat          o    Integer parameter indicating if the mass matrix for
!                         the heat equation must be computed (>0) or not ( = 0)
!                         If jheat = 1, then the coefficient for the mass matrix
!                         is constant, otherwise (jheat = 2) it is variable
!     jwcons         i    Indication if the weight function is constant
!                         (1) or not (2)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     771   Rho is non-positive
!    1541   The combination imas = 1 and ishape=4 is not permitted
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0800jheat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0800jheat'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( jheat==1 .and. jelp==4 .and. irule==1 ) then

!     --- Error: jelp = 4 and diagonal mass matrix
!         In this case the mass amtrix is singular so not allowed

         call errint ( ishape, 1 )
         call errint ( irule, 2 )
         call errint ( jheat, 3 )
         call errwar ( 1541, 3, 0, 0 )

      end if  ! ( jheat==1 .and. jelp==4 )

      if ( ind(12)<0 .and. array(1,12)<=0 ) then

!     --- Error rho non-positive

         call errint ( ielem, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 771, 3, 0, 0 )

      else if ( ind(12)==0 ) then

!     --- Warning, rho = 0.
!         rho is made equal to 1

         call errint ( ielem, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errwar ( 771, 3, 0, 0 )
         ind(12) = -1
         array(1:m,12) = 1d0
         jheat = jwcons + 10*(jheat-1)

      else if ( ind(12)<0 .or. ind(12)==2002 ) then

         jheat = jwcons + 10*(jheat-1)

      else

         jheat = 2 + 10*(jheat-1)

      end if

1000  call erclos ( 'el0800jheat' )
      if ( debug ) then

!     --- Debug information

          write(irefwr,1) 'jheat', jheat
         write(irefwr,*) 'End el0800jheat'

      end if  ! ( debug )

      end
