      subroutine el0800upw ( iuser, metup, jelp, ishift )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 18-12-2014 Use elgetintcoef
!        version  1.0  date 20-01-2012
!
!   copyright (c) 2012-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute parameter metup defining the type of upwinding
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*), metup, jelp, ishift

!     ishift        i/o   Shift of specific variable with respect to diagonal
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     jelp           i    Indication of the type of basis function subroutine
!                         must be called by ELxxxx.
!     metup          o    Upwind parameter
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icompr, metupm
      integer :: nncode
      parameter (nncode=13)
      integer :: ncode(nncode)
      character (len=18) :: code (nncode)

!     code           Table with keywords
!     icompr         If 0 the classical convective term is computed, otherwise
!                    (1) the compressible variant is used.
!     metupm         Upwind parameter with respect to mass matrix
!     ncode          Table containing the number of significant characters
!                    in the keyword table code
!     nncode         Number of entries in code
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer elgetintcoef

!     ELGETINTCOEF   Give elgetintcoef the actual integer value corresponding to
!                    iuser(ipos)
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     616   Fatal error: METUPW has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code / 'NONE', 'CLASIC', 'ILIN', 'DOUBLY_ASSYMPTOTIC', &
                  'CRITICAL', 'HUGHES', 'TIME_DEPENDENT', 'DC1', 'DC2', &
                  'MIZUKAMI_MAX', 'MIZUKAMI_ACT', 'MIZUKAMI_USE', &
                  'TORNBERG_TIME' /
      data ncode / 4, 6, 4,18, 8,   6,14, 3, 3,12, &
                  12,12,13 /

!     code       contains subkeywords
!                 1: NONE
!                 2: CLASIC
!                 3: ILIN
!                 4: DOUBLY_ASSYMPTOTIC
!                 5: CRITICAL
!                 6: HUGHES
!                 7: TIME_DEPENDENT
!                 8: DC1
!                 9: DC2
!                10: MIZUKAMI_MAX
!                11: MIZUKAMI_ACT
!                12: MIZUKAMI_USE
!                13: TORNBERG_TIME
! ======================================================================
!
      call eropen ( 'el0800upw' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0800upw'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      metupw = elgetintcoef ( iuser, ipos, ishift, code, nncode, &
                              ncode, 1, 'supg' )
      if ( debug ) write(irefwr,1) 'metupw', metupw
      icompr = metupw/1000
      metupm = mod(metupw,1000)/100
      metup  = mod(mod(metupw,1000),100)
      if ( metup<0 .or. metup>12 .and. metup<21 .or. &
           metup>32 .or. metupm<0 .or. &
           mod(metup,20)>=9 .and. mod(metup,20)<=12 .and. &
           jelp/=3 .and. jelp/=1 .and. jelp/=11 .or. &
           metupm>1 .or. icompr<0 .or. icompr>1 ) then
         call errint ( metupw, 1 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errsub ( 616, 4, 0, 0 )
      end if

1000  call erclos ( 'el0800upw' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el0800upw'

      end if  ! ( debug )

      end
