      subroutine el0850in ( iuser, mrule, ncoef, linear, &
                            jelp, ishift, jwcons )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 20-11-2009
!
!   copyright (c) 2009-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the number of integration points for this element group
!     using array iuser
!     Elliptic second order equations
! **********************************************************************
!
!                       KEYWORDS
!
!     elliptic_equation
!     initialization
!     stochastic
!     investment
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*), mrule, ncoef, jelp, ishift, jwcons
      logical linear

!     ishift        i/o   Shift in coefficient sequence number
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     jelp           o    Indication of the type of basis function subroutine
!                         must be called by ELel0850inx.
!     jwcons         o    Indication if the weight function is constant
!                         (1) or not (2)
!     linear         o    Indicates whether the element is a linear simplex
!                         (true) or not (false)
!     mrule          o    Type of integration rule as given by the user
!     ncoef          o    Number of coefficients in the equation
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     EL0800ELP      Set parameters depending on shape of element
!     EL0800INM      Compute the parameter m for an element
!     ELCOMM         Initializes the commons celp and celint with standard
!                    values
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERROR42        Print error message 42
!     ERRSUB         Error messages
!     INIREPSCAL     Corrects integer value in case of scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1766   Type of integration rule incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0850in' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0850in'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call elcomm
      mrule = 0

!     --- Set parameters depending on the shape numbers

      call el0800elp ( linear, jelp )

      ncoef = 1
      if ( ierror/=0 ) go to 1000

!     --- Fill the arrays IND, MST and ARRAY with information of the
!         coefficient

      ipos = iuser ( ielgrp + 5 )
      if ( ipos<7 ) call error42 ( ielgrp, ipos, ielem, itype )

      ipos = ipos + 2
      ishift = ishift+2

      mrule = inirepscal ( iuser(ipos) )
      if ( debug ) write(irefwr,1) 'mrule', mrule
      if ( mrule==0 ) then
         if ( ishape==9 ) then
            irule = 3
         else if ( ishape==10 ) then
            irule = 2
         else if ( ishape>=17 ) then
            irule = 3
         else
            irule = 1
         end if
      else
         irule = mrule
      end if
      if ( ishape==9 ) then
         if ( irule<3 .or. irule>4 ) irule = 3
      else if ( ishape==10 ) then
         if ( irule<2 .or. irule>3 ) irule = 2
      end if
      if ( debug ) write(irefwr,1) 'irule', irule
      ipos = ipos + 1
      ishift = ishift+1
      if ( mrule<0 .or. mrule>40 ) then
         call errint ( mrule, 1 )
         call errint ( 3, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 1766, 5, 0, 0 )
      end if

      ipos = ipos + 2
      ishift = ishift+2
      jcart = 1

      if ( linear ) then
         jwcons = min(jcart,2)
      else
         jwcons = 2
      end if

      if ( irule/=1 ) then

!     --- irule>1, Gauss rule

         call el0800inm ( m, jelp, jwcons, irule, igausp, jdiag, &
                          iphi )
         mn = m * n

      end if
      igausp = 1   ! always fill xgauss

1000  call erclos ( 'el0850in' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'irule, jdiag', irule, jdiag
         write(irefwr,*) 'End el0850in'

      end if  ! ( debug )

      end
