      subroutine el10017 ( dphidx, u, elemvc, ndim )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-08-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    actual filling of derivatives in elemvc
!    with respect to subroutine ELD800
!    icheld = 7: div u
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim
      double precision  u(m,ndim), dphidx(n,m,ndim), elemvc(n)

!     dphidx   i     Array of length n x m x ndim  containing the derivatives
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = dphidx (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = dphidx (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = dphidx (i,k,3);
!                         i        k
!                    If the element is a linear simplex, the
!                    derivatives are constant and only k = 1 is filled.
!     elemvc   o     element vector containing the derivatives for subroutine
!                    deriva
!     ndim     i     Dimension of the space.
!     u        i     Real m x ndim  array in which the values of the
!                    of the solution vector are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision divergence

!     divergence     Help paramter to compute the divergence
!     i              General loop variable
!     j              General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The divergence is defined as div u = sum_i du_i/dx_i
!     In case of constant derivatives this implies
!        sum_i ( sum_j u_j dphi_j/dx_i )
!     else
!        sum_i ( sum_j u_k dphi_j dx_i(j) )
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el10017' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el10017'
         call prinrl1 ( u, ndim, n, 'u' )
         call prinrl1 ( dphidx(1,1,1),m,n,'dphidx' )
         call prinrl1 ( dphidx(1,1,2),m,n,'dphidy' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( jdercn==0 ) then

!     --- Derivatives are constant

         divergence = 0d0
         do i = 1, ndim
            divergence = divergence+sum(u(1:n,i)*dphidx(1:n,1,i))
         end do  ! i = 1, ndim
         elemvc(1:n) = divergence

      else

!     --- Derivatives are variable

         elemvc = 0d0
         do i = 1, ndim
            do j = 1, n
               elemvc(j) = elemvc(j) + sum(u(1:n,i)*dphidx(1:n,j,i))
            end do  ! k = 1, n
         end do  ! i = 1, ndim

      end if  ! ( jdercn==0 )

1000  call erclos ( 'el10017' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el10017'

      end if  ! ( debug )

      end
