      subroutine el1011 ( elemmt, w, b, dalpdb, dcdx, dphidx, ndim )
! ======================================================================
!
!        programmer     Guus Segal
!        version  1.0   date 20-06-1994
!
!
!
!   copyright (c) 1994-1994  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!    Compute extra contribution for element matrix corresponding to
!    newton linearization of special nonlinear equation treated in SP 3.3.2
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!    element_group
!    element_matrix
! **********************************************************************
!
!                       COMMON BLOCKS
!
      use sepmodulecactl
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ndim
      double precision elemmt(n,n), b(m), dalpdb(m), dcdx(m,ndim), &
                       dphidx(n,m,ndim), w(m)

!     b       i    || grad c|| in integration points
!     dalpdb  i    d alpha / d (grad c) in integration points
!     dcdx    i    Real m x ndim x ncomp array in which the values of the
!                  derivatives of the solution vector in the integration
!                  points are stored in the sequence:
!                  dc / dx  (x ) = dcdx ( k, j, i )
!                    i    j   k
!     dphidx  i    Array of length n x m x ndim  containing the derivatives
!                  of the basis functions in the sequence:
!                  d phi / dx (xg ) = dphidx (i,k,1);
!                       i        k
!                  d phi / dy (xg ) = dphidx (i,k,2);
!                       i        k
!                  d phi / dz (xg ) = dphidx (i,k,3);
!                       i        k
!                  If the element is a linear simplex, the
!                  derivatives are constant and only k = 1 is filled.
!     elemmt i/o   Element matrix
!                  It is supposed that this matrix has been filled before
!                  It will be updated by this subroutine
!     ndim    i    Dimension of the space.
!     w       i    array of length mdecl containing the weights for integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      double precision c, fac, sum

!     c       coefficient
!     fac     coefficient
!     i       Counting variable
!     j       Counting variable
!     k       Counting variable
!     sum     Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                         n     -1       n                    n
!         S (i,j) = dalpha /dB B  (grad c , grad phi ) (grad c , grad phi )
!          extra                                    j                    i
!
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if ( ndim.eq.1 ) then

!     --- ndim = 1

         c = 0d0
         if ( n.eq.2 ) then

!        --- n = 2, linear elements, hence grad phi and grad c constant

            do k = 1, m
               if ( b(k).gt.0d0 ) c = c + dalpdb(k)/b(k)*w(k)
            end do
            do i = 1, n
               fac = c * dcdx(1,1)*dphidx(i,1,1)
               do j = 1, i
                  sum = fac * (dcdx(1,1)*dphidx(j,1,1))
                  elemmt(i,j) = elemmt(i,j) + sum
                  if ( i.ne.j ) elemmt(j,i) = elemmt(j,i) + sum
               end do
            end do

         else

!        --- n > 2, non-linear elements

            do i = 1, n
               do j = 1, i
                  sum = 0d0
                  do k = 1, m
                     if ( b(k).gt.0d0 ) then
                        sum = sum + dalpdb(k)/b(k)*w(k) * &
                                    dcdx(k,1)*dphidx(i,k,1)* &
                                    dcdx(k,1)*dphidx(j,k,1)
                     end if
                  end do
                  elemmt(i,j) = elemmt(i,j) + sum
                  if ( i.ne.j ) elemmt(j,i) = elemmt(j,i) + sum
               end do
            end do

         end if

      else if ( ndim.eq.2 ) then

!     --- ndim = 2

         if ( n.eq.3 ) then

!        --- n = 3, linear elements, hence grad phi and grad c constant

            c = 0d0
            do k = 1, m
               if ( b(k).gt.0d0 ) c = c + dalpdb(k)/b(k)*w(k)
            end do
            do i = 1, n
               fac = c * (dcdx(1,1)*dphidx(i,1,1)+ &
                          dcdx(1,2)*dphidx(i,1,2))
               do j = 1, i
                  sum = fac * (dcdx(1,1)*dphidx(j,1,1)+ &
                              dcdx(1,2)*dphidx(j,1,2))
                  elemmt(i,j) = elemmt(i,j) + sum
                  if ( i.ne.j ) elemmt(j,i) = elemmt(j,i) + sum
               end do
            end do

         else

!        --- n > 3, non-linear elements

            do i = 1, n
               do j = 1, i
                  sum = 0d0
                  do k = 1, m
                     if ( b(k).gt.0d0 ) then
                        sum = sum + dalpdb(k)/b(k)*w(k) * &
                                    (dcdx(k,1)*dphidx(i,k,1)+ &
                                     dcdx(k,2)*dphidx(i,k,2))* &
                                    (dcdx(k,1)*dphidx(j,k,1)+ &
                                     dcdx(k,2)*dphidx(j,k,2))
                     end if
                  end do
                  elemmt(i,j) = elemmt(i,j) + sum
                  if ( i.ne.j ) elemmt(j,i) = elemmt(j,i) + sum
               end do
            end do

         end if

      else

!     --- ndim = 3

         if ( n.eq.4 ) then

!        --- n = 4, linear elements, hence grad phi and grad c constant

            c = 0d0
            do k = 1, m
               if ( b(k).gt.0d0 ) c = c + dalpdb(k)/b(k)*w(k)
            end do
            do i = 1, n
               fac = c * (dcdx(1,1)*dphidx(i,1,1)+ &
                          dcdx(1,2)*dphidx(i,1,2)+ &
                          dcdx(1,3)*dphidx(i,1,3))
               do j = 1, i
                  sum = fac * (dcdx(1,1)*dphidx(j,1,1)+ &
                              dcdx(1,2)*dphidx(j,1,2)+ &
                              dcdx(1,3)*dphidx(j,1,3))
                  elemmt(i,j) = elemmt(i,j) + sum
                  if ( i.ne.j ) elemmt(j,i) = elemmt(j,i) + sum
               end do
            end do

         else

!        --- n > 4, non-linear elements

            do i = 1, n
               do j = 1, i
                  sum = 0d0
                  do k = 1, m
                     if ( b(k).gt.0d0 ) then
                        sum = sum + dalpdb(k)/b(k)*w(k) * &
                                    (dcdx(k,1)*dphidx(i,k,1)+ &
                                     dcdx(k,2)*dphidx(i,k,2)+ &
                                     dcdx(k,3)*dphidx(i,k,3))* &
                                    (dcdx(k,1)*dphidx(j,k,1)+ &
                                     dcdx(k,2)*dphidx(j,k,2)+ &
                                     dcdx(k,3)*dphidx(j,k,3))
                     end if
                  end do
                  elemmt(i,j) = elemmt(i,j) + sum
                  if ( i.ne.j ) elemmt(j,i) = elemmt(j,i) + sum
               end do
            end do

         end if

      end if
      end
