      subroutine el2004 ( jsec,a11,a12,a22,s,w,phix,phiy,nloc,mloc, &
                          lrowloc,work,econ )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.3  date 03-04-1999 Include commons
!        version  3.1  date 22-08-1991 Zdenek: no dimension, cdc, ce
!        version  3.0  date 08-10-1986
!
!   copyright (c) 1984-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    compute second derivative part for a 2-d elliptic equation
!    real matrix
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical econ
      double precision a11(*), a12(*), a22(*), s(*), w(*), phix(*), &
                       phiy(*), work(*)
      integer jsec,nloc,mloc,lrowloc

!     a11        i    array containing the values of a11
!                     in the integration points
!     a12        i    array containing the values of a12
!                     in the integration points
!     a22        i    array containing the values of a22
!                     in the integration points
!     econ       i    indication whether some integer parameters may be saved
!                     for the next calls of el2001 (true) or not (false)
!                     When el2001 is called more than once in an element
!                     subroutine econ = .false. should be used
!     jsec       i    choice parameter. Possibilities:
!                     jsec = jsecndloc + 100 * ispec
!                     ispec:
!                       0  normal case
!                       1  quadratic triangle with 3-point newton cotes rule
!                     jsecndloc:
!                       1  a12=0;  a11=a22   a11 and w constant
!                       2  a12=0   a11=a22
!                       3  a12#0   a11=a22
!                       4  general
!     lrowloc       isize of the element matrix
!     m          i    number of integration points
!     n          i    number of basis functions
!     phix       i    Array of length n x m  containing the x-derivatives
!                     of the basis functions in the sequence:
!                     d phi_i/ dx (xg_k) = phix((i-1)*m+k)
!     phiy       i    Array of length n x m  containing the y-derivatives
!                     of the basis functions in the sequence:
!                     d phi_i/ dy (xg_k) = phiy((i-1)*m+k)
!     s          o    element matrix (lrowloc x lrowloc)
!                     s(i,j)=s((i-1)*n+j)  1-D storage
!                     only the part i,j=1,...,nloc is filled
!     w          i    array of length m containing the weights for integration
!     work            Work array of length 3xm
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer jsecndloc,kbot,ksteploc, in, im, jm, i, j, k, m2
      double precision a11h, a11con, a12con, a22con, sum
      save jsecndloc,kbot,ksteploc,a11h

!     a11con     Constant value of a11
!     a11h
!     a12con     Constant value of a12
!     a22con     Constant value of a22
!     i          Counting variable
!     im         (i-1) x m
!     in         (i-1) x lrowloc
!     j          Counting variable
!     jm         (j-1) x m
!     jsecndloc
!     k          Counting variable
!     kbot
!     ksteploc
!     m2
!     sum        Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if(.not. econ .or. ifirst.eq.0) then

!     --- ifirst = 0  compute jsecndloc, kbot and ksteploc

         if(jsec.gt.100) then
            jsecndloc=jsec-100
            kbot=2
            ksteploc=2
         else
            jsecndloc=jsec
            kbot=1
            ksteploc=1
         end if
         a11h = 3d0 * a11(1)
      end if
      goto (100, 200, 300, 400), jsecndloc

!     --- jsecndloc = 1   a12=0; a11=a22  a11 and w constant

100   a11con=w(1)*a11h
      if(nloc.eq.3) then
         in=0
         do 130 i=1,nloc
            do 120 j=1,i
120            s(in+j)=a11con*(phix(i)*phix(j)+phiy(i)*phiy(j))
130         in=in+lrowloc
      else
         in=0
         im=0
         do 180 i=1,nloc
            jm=0
            do 170 j=1,i
               sum=0d0
               do 160 k=1,mloc
160               sum=sum+phix(im+k)*phix(jm+k)+phiy(im+k)*phiy(jm+k)
               s(in+j)=a11con*sum
170            jm=jm+mloc
            im=im+mloc
180         in=in+lrowloc
      end if
      goto 500

!     --- jsecndloc = 2   a12=0; a11=a22

200   in=0
      if(nloc.eq.3) then
         a11con=w(1)*a11(1)
         do 210 k=2,mloc
210         a11con=a11con+w(k)*a11(k)
         do 230 i=1,nloc
            do 220 j=1,i
220            s(in+j)=a11con*(phix(i)*phix(j)+phiy(i)*phiy(j))
230         in=in+lrowloc
      else
         do 250 k=kbot,mloc,ksteploc
250         work(k)=w(k)*a11(k)
         im=0
         do 280 i=1,nloc
            jm=0
            do 270 j=1,i
               sum=0d0
               do 260 k=kbot,mloc,ksteploc
260               sum=sum+work(k)*(phix(im+k)*phix(jm+k)+phiy(im+k)* &
                      phiy(jm+k))
               s(in+j)=sum
270            jm=jm+mloc
            im=im+mloc
280         in=in+lrowloc
      end if
      goto 500

!     --- jsecndloc = 3   a12#0; a11=a22

300   in=0
      if(nloc.eq.3) then
         a11con=w(1)*a11(1)
         a12con=w(1)*a12(1)
         do 310 k=2,mloc
            a11con=a11con+w(k)*a11(k)
310         a12con=a12con+w(k)*a12(k)
         do 330 i=1,nloc
            do 320 j=1,i
320            s(in+j)=a11con*(phix(i)*phix(j)+phiy(i)*phiy(j))+ &
                  a12con*(phix(i)*phiy(j)+phiy(i)*phix(j))
330         in=in+lrowloc
      else
         do 350 k=kbot,mloc,ksteploc
            work(k)=w(k)*a11(k)
350         work(k+mloc)=w(k)*a12(k)
         im=0
         do 380 i=1,nloc
            jm=0
            do 370 j=1,i
               sum=0d0
               do 360 k=kbot,mloc,ksteploc
360               sum=sum+work(k)*(phix(im+k)*phix(jm+k)+phiy(im+k)* &
                      phiy(jm+k))+work(k+mloc)*(phix(im+k)*phiy(jm+k)+ &
                      phiy(im+k)*phix(jm+k))
               s(in+j)=sum
370            jm=jm+mloc
            im=im+mloc
380         in=in+lrowloc
      end if
      goto 500

!     --- jsecndloc = 4   general

400   in=0
      if(nloc.eq.3) then
         a11con=w(1)*a11(1)
         a22con=w(1)*a22(1)
         a12con=w(1)*a12(1)
         do 410 k=2,mloc
            a11con=a11con+w(k)*a11(k)
            a22con=a22con+w(k)*a22(k)
410         a12con=a12con+w(k)*a12(k)
         do 430 i=1,nloc
            do 420 j=1,i
420            s(in+j)=a11con*phix(i)*phix(j)+a22con*phiy(i)*phiy(j)+ &
                  a12con*(phix(i)*phiy(j)+phiy(i)*phix(j))
430         in=in+lrowloc
      else
         m2=mloc+mloc
         do 450 k=kbot,mloc,ksteploc
            work(k)=w(k)*a11(k)
            work(k+mloc)=w(k)*a22(k)
450         work(k+m2)=w(k)*a12(k)
         im=0
         do 480 i=1,nloc
            jm=0
            do 470 j=1,i
               sum=0d0
               do 460 k=kbot,mloc,ksteploc
460               sum=sum+work(k)*phix(im+k)*phix(jm+k)+work(k+mloc)* &
                    phiy(im+k)*phiy(jm+k)+work(k+m2)*(phix(im+k)* &
                    phiy(jm+k)+phiy(im+k)*phix(jm+k))
               s(in+j)=sum
470            jm=jm+mloc
            im=im+mloc
480         in=in+lrowloc
      end if

!     --- the upper triangle of s is computed

500   in=0
      do 520 i=1,nloc
         do 510 j=i+1,nloc
510         s(in+j)=s((j-1)*lrowloc+i)
520      in=in+lrowloc
      end
