      subroutine el2201 ( elemvc, f, trans, mdiv, s, rhsdivtr )
! ======================================================================
!
!        programmer   Guus Segal
!        version 2.0    date 18-10-1998 Extra parameters
!        version 1.0    date 28-01-1995 Renamed subroutine el2201
!
!
!
!   copyright (c) 1995-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Eliminate the centroid point for the incompressible Navier-Stokes
!     equations in case of an extended quadratic triangle
!     Add contribution of divergence right-hand side
! **********************************************************************
!
!                       KEYWORDS
!
!   element_vector
!   navier_stokes_equation
! **********************************************************************
!
!                       COMMON BLOCKS
!
      use sepmodulecactl
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision elemvc((n-1)*ndimlc), rhsdivtr(*), &
                       f(n*ndimlc), trans(ndimlc*(n-1),ndimlc), &
                       s(n*ndimlc,n*ndimlc)
      integer mdiv

!     elemvc   o    New element vector
!     f        i    Element vector to be changed
!     mdiv     i    Defines if divergence right-hand side is used (1) or not (0)
!     rhsdivtr i    Contains values of transformed divergence right-hand side
!     s        i    Element matrix before elimination
!     trans    i    Transformation matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, n12, l, ncent
      double precision sum, rhs(3)

!     i         Counting variable
!     k         Counting variable
!     l         Counting variable
!     n12       Number of nt to be eliminated velocity unknowns
!     ncent     Number of nt to be eliminated velocity unknowns
!     rhs       Work array to store the factor s22 rhsdivtr
!     sum       Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     See for an explanation of the method subroutine EL2107
!
!
!                T
!      F = f  + R  f
!           1    1  2
!
!             new   1     T   2
!     Hence: f   = f   + R1  f
!             i     i      ik k
!
!     In case of mdiv = 1, the right-hand side must be updated by the following
!     term:
!
!
!               T             T
!     F = f  + R  f + ( S  + R   S   ) fd_tr ,
!          1    1  2     12   1   22
!
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      n12 = ndimlc*(n-1)

      do i = 1, n12
         sum = 0d0
         do k = 1, ndimlc
            sum = sum + trans(i,k)*f(k+n12)
         end do
         elemvc(i) = f(i) + sum

      end do

      if ( mdiv.eq.1 ) then

!     --- mdiv = 1, divergence right-hand side

         if ( ndimlc.eq.2 ) then

!        --- R2

            ncent = ndimlc*(n-1)

!           --- First the term S22 * fd_tr is stored in rhs

            rhs(1) = s(1+ncent,1+ncent)*rhsdivtr(1) + &
                     s(1+ncent,2+ncent)*rhsdivtr(2)
            rhs(2) = s(2+ncent,1+ncent)*rhsdivtr(1) + &
                     s(2+ncent,2+ncent)*rhsdivtr(2)

!           --- Next the complete term is added

            do i = 1, n12
               sum = 0d0
               do k = 1, ndimlc
                  sum = sum + s(i,k+ncent)*rhsdivtr(k)+trans(i,k)*rhs(k)
               end do
               elemvc(i) = elemvc(i) + sum
            end do

         else

!        --- R3

            ncent = ndimlc*13
            do k = 1, ndimlc
               sum = 0d0
               do l = 1, ndimlc
                  sum = sum + s(l+ncent,k+ncent)*rhsdivtr(k)
               end do
               rhs(k) = sum
            end do

!           --- Next the complete term is added

            do i = 1, ncent
               sum = 0d0
               do k = 1, ndimlc
                  sum = sum + s(i,k+ncent)*rhsdivtr(k)+trans(i,k)*rhs(k)
               end do
               elemvc(i) = elemvc(i) + sum
            end do

            do i = 43, 81
               sum = 0d0
               do k = 1, ndimlc
                  sum = sum + s(i,k+ncent)*rhsdivtr(k)+trans(i,k)*rhs(k)
               end do
               elemvc(i) = elemvc(i) + sum
            end do

         end if

      end if

      end
