      subroutine el6000 ( g, f, w, det, mdecl )
! ======================================================================
!
!        programmer     Patrick Anderson
!        version 1.1    date  25-01-1996 layout
!        version 1.0    date  07-04-1995 el6003 adapted to eld607
!
!
!
!   copyright (c) 1995-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     compute contributions to the spectral element vector
!     corresponding to the righthand side of the equation, i.e.
!
!         f  =  f   +  / g phi  de
!          i     i     e      i
!
!                              i
!            =  f   +  w(i) g(x )
!                i
!
!            i
!     where x  is the integration point
!
!     EL6000 is called by the element subroutine ELD607
!            it must be preceded by the subroutines ELP640 and ELP647
!
!***********************************************************************
!
!                      KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
!     spectral
! ********************************************************************
!
!                INPUT / OUTPUT PARAMETERS
!
      use sepmodulecactl
      implicit none
      integer mdecl
      double precision g(mdecl), f(mdecl), w(mdecl), det

!     det      i   transformation determinantc
!     f        o   array of length mdecl containing the spectral element
!                  vector
!     g        i   array of length mdecl containing the values of the
!                  fourth coefficient (righthand side) in the integration
!     node
!     mdecl    i   number of integration/interpolation points
!     w        i   array of length mdecl containing the weights for
!                  numerical integration
!***********************************************************************
!
!                    COMMON BLOCKS
!
!***********************************************************************
!
!              LOCAL PARAMETERS

      integer i

!     i     counting variable
!***********************************************************************
!
!             SUBROUTINES CALLED
!
! ********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                    ERROR MESSAGES
!
!    - &
! ********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
      if( jadd.eq.0 ) then

         do 100 i=1,mdecl
            f(i) = 0d0
100      continue

      end if

      do 200 i = 1, mdecl
         f(i) = g(i)*w(i)*det
200   continue

      end
