      subroutine el6015 ( f, old, w, det, mdecl )
! ======================================================================
!
!        programmer     Peter Minev/Patrick Anderson
!        version 1.2    date  24-07-1998  Include SPcommon/ctimen
!        version 1.1    date  25-01-1996  Layout
!        version 1.0    date  29-03-1995
!
!
!
!   copyright (c) 1995-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute contributions to spectral element residue vector
!     corresponding to the mass matrix
!     EL6015 is called by the element subroutine ELM607
!            it must be preceded by the subroutines ELP640 and ELP647
! **********************************************************************
!
!                       KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
!     spectral
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      use sepmoduletime
      implicit none
      integer mdecl
      double precision f(mdecl), old(mdecl), w(mdecl), det

!     det      i     jacobian of the transform
!     f       i/o    array of length mdecl containing the spectral
!                    element residue vector, at input f may already have
!                    been filled depending on the value of jadd in the common
!                    block celint
!     mdecl    i     number of integration/interpolation points
!     old      i     array of length mdecl containing the solution of
!                    the previous iteration step in the nodal points
!     w        i     array of length mdecl containing the weights for
!                    numerical integration
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision fac

!     fac factor variable
!     i   counting variables
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- set fac

      fac = 1.5d0
      if (icons.eq.0) fac = 1d0

!     --- add the mass contribution

      do i = 1, mdecl
         f(i) = f(i)*tstep + fac*w(i)*old(i)*det
      end do

      end
