      subroutine el6020(a, f, w, det, mone)
! ======================================================================
!
!        programmer     Peter Minev
!        version 1.1    date  25-01-1996  Layout
!        version 1.0    date  15-12-1994
!
!
!
!   copyright (c) 1994-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     compute contributions to the spectral element vector
!     corresponding to nonhomogeneous Neumann conditions
!     on 2-D surface in a 3-D space
!
!         f  =  f   +  / a phi  de
!          i     i     e      i
!
!                              i
!            =  f   +  w(i) a(x )
!                i
!
!            i
!     where x  is the integration point
!
!     EL6020 is called by the element subroutine ELM608
!            it must be preceded by the subroutines ELP640 and ELP649
!***********************************************************************
!
!                      KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
!     spectral
! ********************************************************************
!
!                    INPUT / OUTPUT PARAMETERS
      implicit none
      integer mone
      double precision a(mone,mone), f(mone,mone), w(mone), &
                       det(mone,mone)

!     a        i   array of length mdecl containing the values of the
!                  boundary condition in the integration points
!     det      i   the determinant of the transform, length m
!     f      i/o   array of length mdecl containing the spectral element
!                  vector
!     mone     i   number of points in 1 direction
!     w        i   array of length mdecl containing the weights for
!                  numerical integration
!***********************************************************************
!
!                     COMMON BLOCKS
!
!***********************************************************************
!
!                LOCAL PARAMETERS
!
      integer i, j

!     i     counting variable
!     j     counting variable
!***********************************************************************
!
!             SUBROUTINES CALLED
!
! ********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                    ERROR MESSAGES
!
!    - &
! ********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
      do 200 i = 1, mone
         do 100 j = 1, mone
            f(i,j) = a(i,j)*w(i)*w(j)*det(i,j)
100      continue
200   continue

      end
