      subroutine el6022(beta, s, w1, w2, det, modecl, mdecl)
!
!***********************************************************************
!
!     programmer Luc Timmermans
!     version 2.0   date  07-11-1991  change in parameterlist
!     version 1.0   date  19-09-1991  first draft
!
!***********************************************************************
!
!     compute contributions to spectral element stiffness matrix
!     corresponding to the Helmholtz part of the equation
!
!     EL6022 is called by the element subroutine ELM602
!            it must be preceded by the subroutines ELP640 and ELP642
!
!***********************************************************************
!
!     INPUT/OUTPUT PARAMETERS

      use sepmodulecactl
      implicit none

      integer modecl, mdecl
      double precision beta(mdecl), s(mdecl,mdecl), w1(modecl), &
                       w2(modecl), det(modecl,modecl)
!
!     beta     i     array of length mdecl containing the values of the
!                    third coefficient in the integration points
!     s       i/o    array of length mdecl x mdecl containing the spectral
!                    element stiffness matrix, at input s may already have
!                    been filled depending on the value of jadd in the common
!                    block celint
!                    since SEPRAN assumes an one-dimensional storage, s
!                    must be stored transposed
!     w1       i     array of length modecl containing the weights for
!                    numerical integration in the x-direction corres- &
!                    ponding to the reference element
!     w2       i     array of length modecl containing the weights for
!                    numerical integration in the y-direction corres- &
!                    ponding to the reference element
!     det      i     array of length modecl x modecl containing the
!                    transformation determinant
!     modecl   i     number of integration points in one dimension
!     mdecl    i     number of integration points in the element
!
!***********************************************************************
!
!     PARAMETERS IN COMMON BLOCKS
!
!***********************************************************************
!
!     LOCAL PARAMETERS
!
      integer i, j, ij
      double precision bcon, help

!     i, j    counting variables
!     ij      help parameter
!     bcon    value of beta in the case that it is a constant function
!     help    help parameter to store products
!
!***********************************************************************

!     * check if Helmholtz matrix must be added to an existing matrix
!       or not, if not set s equal to zero

      if (jadd.eq.0) then
!        * jadd = 0, set s equal to zero

         do 200 i = 1, mdecl
            do 100 j = 1, mdecl

               s(i,j) = 0d0
100         continue
200      continue

      end if

!     * add Helmholtz matrix to s

      if (jzero.eq.1) then
!        * beta is constant (per element)

         bcon = beta(1)
         do 400 i = 1, modecl
            help = bcon*w1(i)
            do 300 j = 1, modecl

               ij = i + (j - 1)*modecl
               s(ij,ij) = s(ij,ij) + help*w2(j)*det(i,j)
300         continue
400      continue

      else
!        * beta is variable

         do 600 i = 1, modecl
            do 500 j = 1, modecl

               ij = i + (j - 1)*modecl
               s(ij,ij) = s(ij,ij) + w1(i)*w2(j)*det(i,j)*beta(ij)
500         continue
600      continue

      end if

      end
