      subroutine el6032(beta, f, old, w1, w2, det, modecl, mdecl)
!
!***********************************************************************
!
!     programmer Luc Timmermans
!     version 1.0   date  14-04-1992  test version
!
!***********************************************************************
!
!     compute contributions to spectral element residue vector
!     corresponding to the Helmholtz part of the equation in the
!     case of finite element preconditionning
!
!     EL6032 is called by the element subroutine ELM602
!            it must be preceded by the subroutines ELP640 and ELP642
!
!***********************************************************************
!
!     INPUT/OUTPUT PARAMETERS

      use sepmodulecactl
      implicit none

      integer modecl, mdecl
      double precision beta(mdecl), f(mdecl), old(mdecl), w1(modecl), &
                       w2(modecl), det(modecl,modecl)
!
!     beta     i     array of length mdecl containing the values of the
!                    third coefficient in the integration points
!     f       i/o    array of length mdecl containing the spectral
!                    element residue vector, at input f may already have
!                    been filled depending on the value of jadd in the common
!                    block celint
!     old      i     array of length mdecl containing the solution of
!                    the previous iteration step in the nodal points
!     w1       i     array of length modecl containing the weights for
!                    numerical integration in the x-direction corres- &
!                    ponding to the reference element
!     w2       i     array of length modecl containing the weights for
!                    numerical integration in the y-direction corres- &
!                    ponding to the reference element
!     det      i     array of length modecl x modecl containing the
!                    transformation determinant
!     modecl   i     number of integration points in one dimension
!     mdecl    i     number of integration points in the element
!
!***********************************************************************
!
!     PARAMETERS IN COMMON BLOCKS
!
!***********************************************************************
!
!     LOCAL PARAMETERS
!
      integer i, j, ij
      double precision bcon, help

!     i       counting variable
!     j       counting variable
!     ij      help parameter
!     bcon    value of beta in the case that it is a constant function
!     help    help parameter to store products
!
!***********************************************************************

!     * check if Helmholtz part must be added to an existing vector
!       or not, if not set f equal to zero

      if (jadd.eq.0) then
!        * jadd = 0, set f equal to zero

         do 100 i = 1, mdecl

            f(i) = 0d0
100      continue
      end if

!     * add Helmholtz part to f

      if (jzero.eq.1) then
!        * beta is constant (per element)

         bcon = beta(1)
         do 400 i = 1, modecl
            ij = i
            help = bcon*w1(i)
            do 300 j = 1, modecl

               f(ij) = f(ij) + help*w2(j)*det(i,j)*old(ij)
               ij = ij + modecl
300         continue
400      continue

      else
!        * beta is variable

         do 600 i = 1, modecl

            ij = i
            do 500 j = 1, modecl

               f(ij) = f(ij) + w1(i)*w2(j)*det(i,j)*beta(ij)*old(ij)
               ij = ij + modecl
500         continue
600      continue

      end if

      end
