      subroutine el6038 ( vec, w1, det, old, mone )
! ======================================================================
!
!        programmer     Peter Minev/Patrick Anderson
!        version 1.2    date  24-07-1998  Include SPcommon/ctimen
!        version 1.1    date  25-01-1996 layout
!        version 1.0    date  29-03-1995
!
!
!
!   copyright (c) 1995-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the spectral element mass matrix residual - 2-D case
!     EL6038 is called by the element subroutine ELM607
!            it must be preceded by the subroutines ELP640 and ELP642
! **********************************************************************
!
!                       KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
!     spectral
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      use sepmoduletime
      implicit none
      integer mone
      double precision vec(mone,mone), w1(mone), det(mone,mone), &
                       old(mone,mone)

!     det      i   array of length mone x mone containing the transformation
!                  determinant
!     mone     i   number of integration points in one dimension
!     old      i   array of length mone x mone containing the old solution
!     vec      o   array of length mone x mone containing the spectral element
!                  mass-matrix residual
!     w1       i   array of length mone containing the weights for
!                  numerical integration in 1 direction correspon- &
!                  ding to the reference element
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision fac

!     fac        factor variable
!     i          Counting variable
!     j          Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- set fac

      fac = 1.5d0
      if ( icons.eq.0 ) fac = 1d0

      do i = 1, mone
         do j = 1, mone
            vec(i,j) = vec(i,j)*tstep + &
                       w1(i)*w1(j)*det(i,j)*old(i,j)*fac
         end do
      end do

      end
