      subroutine el6039(beta, f, old, w1, det, modecl, mdecl)
! ======================================================================
!
!        programmer     Patrick Anderson
!        version 1.1    date  25-01-1996 layout
!        version 1.0    date  02-01-1996
!
!
!
!   copyright (c) 1995-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     compute contributions to spectral element residue vector
!     corresponding to the Helmholtz part of the equation in the
!     case of finite element preconditionning
!
!     EL6039 is called by the element subroutine ELM600
!***********************************************************************
!
!                      KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
!     spectral
! ********************************************************************
!
!                INPUT / OUTPUT PARAMETERS
!
      use sepmodulecactl
      implicit none
      integer modecl, mdecl
      double precision beta(mdecl), f(mdecl), old(mdecl), w1(modecl), &
                       det(modecl,modecl,modecl)

!     beta     i     array of length mdecl containing the values of the
!                    third coefficient in the integration points
!     det      i     array of length modecl x modecl containing the
!                    transformation determinant
!     f       i/o    array of length mdecl containing the spectral
!                    element residue vector, at input f may already have
!                    been filled depending on the value of jadd in the common
!                    block celint
!     mdecl    i     number of integration points in the element
!     modecl   i     number of integration points in one dimension
!     old      i     array of length mdecl containing the solution of
!                    the previous iteration step in the nodal points
!     w1       i     array of length modecl containing the weights for
!                    numerical integration in the x-direction corres- &
!                    ponding to the reference element
!***********************************************************************
!
!                     COMMON BLOCKS
!
!***********************************************************************
!
!                           LOCAL PARAMETERS
!
      integer i, j, ijk, k, mtwo

!     i          Counting variable
!     ijk        help parameter
!     j          Counting variable
!     k          Counting variable
!     mtwo       mtwo = mone*mone
!***********************************************************************
!
!             SUBROUTINES CALLED
!
! ********************************************************************
!
!                    I/O
!
!    none
! ********************************************************************
!
!                    ERROR MESSAGES
!
!    - &
! ********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
!     --- check if Helmholtz part must be added to an existing vector
!         or not, if not set f equal to zero

      if ( jadd.eq.0 ) then

!     --- jadd = 0, set f equal to zero

         do 10 i = 1, mdecl
            f(i) = 0d0
10       continue

      end if

!     --- add Helmholtz part to f

      mtwo = mone*mone

      do 300 i = 1, mone
         do 200 j = 1, mone
            do 100 k = 1, mone
            ijk = i + (j-1)*mone + (k-1)*mtwo
            f(ijk)=f(ijk)+w1(i)*w1(j)*w1(k)* &
                         beta(ijk)*det(i,j,k)*old(ijk)
100         continue
200      continue
300   continue

      end
