      subroutine el6060 (ichois, tout, txxn, txyn, tyyn, u, v, ux, uy, &
                         vx, vy, w1, d1, det, p1, p2, p3, p4, &
                         work, workar, user, iuser, mone1, m1, modelv)
! ======================================================================
!
!        programmer     Patrick Anderson
!        version 1.0    date  20-01-1997
!
!   copyright (c) 1997-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       KEYWORDS
!
!   visco_elastic
!   maxwell
!     spectral
! **********************************************************************
!
!                       DESCRIPTION
!
!   In this routine the stresses are calculated for the
!   two-dimensional upper convected maxwell model
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      use sepmodulecactl
      implicit none
      integer mone1, m1, modelv, ichois, iuser(*)
      double precision tout(m1), txxn(m1),  txyn(m1),  tyyn(m1), &
                       u(m1), v(m1), ux(m1), uy(m1), vx(m1), vy(m1)
      double precision w1(mone1), d1(mone1,mone1), &
                       det(mone1,mone1), &
                       p1(mone1,mone1), p2(mone1,mone1), &
                       p3(mone1,mone1), p4(mone1,mone1), &
                       work(m1), workar(m1), user(*)

!     d1         i    array of length mone1 x mone1 contaning the values of the
!                     derivatives of the basis functions in the x-direction
!     det        i    array of length mone1 x mone1 containing the
!                     transformation determinant
!     ichois     i    Choice parameter
!     iuser      i    Integer user array to pass user information from
!                     main program to subroutine. See STANDARD PROBLEMS
!     m1         i
!     modelv     i    Defines the type of viscosity model
!                     Possible values:
!     mone1      i
!     p1         i
!     p2         i
!     p3         i
!     p4         i
!     tout       i
!     txxn       i
!     txyn       i
!     tyyn       i
!     u          i
!     user       i    Real user array to pass user information from
!                     main program to subroutine. See STANDARD PROBLEMS
!     ux         i
!     uy         i
!     v          i
!     vx         i
!     vy         i
!     w1         i    array of length m containing the weights for
!                     numerical integration in the x-direction corresponding
!                     to the reference element
!     work       i    Work array
!     workar     i
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, jddprv, jzprv
      double precision lambda, xi

!     i          loop parameter
!     jddprv     keep jadd
!     jzprv      keep jzero
!     lambda     parameter in consitutive equation
!     xi         parameter in consitutive equation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL6032    compute the spectral element Helmholtz residual
!     EL6043    compute the convection residual in the case of spectral elements
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       1   ICHOIS is incorrect
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! **********************************************************************
!
!                       MODULES USED
!
! ======================================================================
!
      call eropen ( 'el6060' )

!     --- Set lambda and xi

      lambda = user(80)
      xi     = user(81)

!     --- Keep jadd and jzero, just not to disturb diff. eqn

      jddprv = jadd
      jzprv  = jzero
      if (ichois.ne.1 .and. ichois.ne.2 .and. &
          ichois.ne.3) then
         call errint ( ichois, 1 )
         call errsub ( 1, 1, 0, 0 )
         go to 1000
      end if

      go to ( 100, 200, 300) ichois

!     --- Build S11 txxn : [ lambda(u ddx + v ddy) - &
!                            2 lambda ux  + 1 ] txxn
!         Build Helmholtz part

100   do i = 1, m1
         work(i) =  - 2 * lambda * ux(i) + 1.0d0
      end do

!     --- Add PTT part, if necesary

      if (modelv.eq.2) then
         do i = 1, m1
            work(i) =  work(i) + 2 * xi * ux(i)
         end do
      end if

!     --- Initialize tout

      jadd = 0

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      call el6032(work, tout, txxn, w1, w1, det, mone1, m1)

!     --- Add Convection term to tout

      jadd = 1
      call el6043(u, v, lambda,  tout, txxn, w1, &
                  d1, p1, p2, p3, p4, workar, mone1, m1)

!     --- Build and add S12 txyn to tout :  -2 lambda uy txyn
!         Build Helmholtz part

      do i = 1, m1
         work(i) = - 2 * lambda * uy(i)
      end do

!     --- Add PTT part, if necesary

      if (modelv.eq.2) then
         do i = 1, m1
            work(i) =  work(i) + xi * (uy(i) + vx(i))
         end do
      end if

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      jadd = 1
      call el6032(work, tout, txyn, w1, w1, det, mone1, m1)

      go to 1000

!     --- Build S21 txxn :  - lambda vx txxn
!         Build Helmholtz part

200   do i = 1, m1
         work(i) = - lambda *vx(i)
      end do

!     --- Add PTT part, if necesary

      if (modelv.eq.2) then
         do i = 1, m1
            work(i) =  work(i) + xi * ( uy(i) + vx(i) ) /2.0d0
         end do
      end if

!     --- Initialize tout

      jadd = 0

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      call el6032(work, tout, txxn, w1, w1, det, mone1, m1)

!     --- Build and add S22 txyn to tout  : [lambda(u ddx + v ddy) + 1] txyn
!         Build Helmholtz part

      do i = 1, m1
         work(i) =  1.0d0
      end do

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      jadd = 1
      call el6032(work, tout, txyn, w1, w1, det, mone1, m1)

!     --- Add Convection term to tout

      jadd = 1
      call el6043(u, v, lambda, tout, txyn, w1, &
                  d1, p1, p2, p3, p4, workar, mone1, m1)

!     --- Build S23 tyyn :  - lambda uy tyyn
!         Build Helmholtz part

      do i = 1, m1
         work(i) = - lambda * uy(i)
      end do

!     --- Add PTT part, if necesary

      if (modelv.eq.2) then
         do i = 1, m1
            work(i) =  work(i) + xi * ( uy(i) + vx(i) ) /2.0d0
         end do
      end if

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      jadd = 1
      call el6032(work, tout, tyyn, w1, w1, det, mone1, m1)
      go to 1000

!     --- Build S32 txyn :  - lambda vx txyn
!         Build Helmholtz part

300   do i = 1, m1
         work(i) = - 2.0d0 * lambda *vx(i)
       end do

!      --- Add PTT part, if necesary

      if (modelv.eq.2) then
         do i = 1, m1
            work(i) =  work(i) + xi *( uy(i) + vx(i) )
         end do
      end if

!     --- Initialize tout

      jadd = 0

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      call el6032(work, tout, txyn, w1, w1, det, mone1, m1)

!     --- Build S33 tyyn : [ lambda(u ddx + v ddy) - &
!                             2 lambda vy  + 1 ] tyyn
!         Build Helmholtz part

      do i = 1, m1
         work(i) =  - 2 * lambda *vy(i) + 1.0d0
      end do

!     --- Add PTT part, if necesary

      if (modelv.eq.2) then
         do i = 1, m1
            work(i) =  work(i) + 2.0d0 * xi * vy(i)
         end do
      end if

!     --- Beta in Helmholtz operator is not a constant

      jzero = 2
      jadd = 1
      call el6032(work, tout, tyyn, w1, w1, det, mone1, m1)

!     --- Add Convection term to tout

      jadd = 1
      call el6043(u, v, lambda,  tout, tyyn, w1, &
                  d1, p1, p2, p3, p4, workar, mone1, m1)

      go to 1000

!     --- Restore jadd and jzero to previous values

1000  jadd  = jddprv
      jzero = jzprv
      call erclos ( 'el6060' )

      end
