      subroutine el6081 ( f, old, wxp, pdxv, p1p, &
                          p2p, p3p, p4p, work, mpone, mp )
! ======================================================================
!
!        programmer     Peter Minev
!        version 1.1    date   28-01-1996 Layout
!        version 1.0    date   31-03-1995
!
!   copyright (c) 1995-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     compute the vorticity of a vector field  - 2-D spectral elements
!
!     EL6081 is called by the element subroutine ELD607
!            it must be preceded by the subroutines ELP640, ELP646
! ********************************************************************
!
!                      KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      use sepmodulecactl
      implicit none
      integer mpone, mp
      double precision f(mp), old(2*mp), wxp(mpone), &
                       pdxv(mpone,mpone), &
                       p1p(mpone,mpone), p2p(mpone,mpone), &
                       p3p(mpone,mpone), p4p(mpone,mpone), work(mp)

!     f       i/o  array of length mp containing the righthandside
!                  or the residual
!     mp       i   number of pressure points in the element
!     mpone    i   number of pressure points in the one dimension
!     old      i   array of length 2*mp containing the old velocity
!     p1p      i   arrays of length mpone x mpone containing the partial
!                  derivatives of the coordinates due to transformation
!                  for the pressure
!     p2p      i   See p1p
!     p3p      i   See p1p
!     p4p      i   See p1p
!     pdxv     i   array of length mpone x mpone contaning the values of the
!                  derivatives of the basisfunctions for the velocity in the
!                  pressure points in the x-direction
!     work     -   work array, length mp
!     wxp      i   array of length mpone containing the pressure weights for
!                  numerical integration in the x-direction corresponding
!                  to the reference element
!***********************************************************************
!
!                      COMMON BLOCKS
!
!***********************************************************************
!
!                      LOCAL PARAMETERS
!
      integer i, j, m1, l, ij, lj, im
      double precision sum

!     i            counting variable
!     ij           position in the vector f
!     im           position in the vector old
!     j            counting variable
!     l            counting variable
!     lj           position in the vector old
!     m1           counting variable
!     sum          help variable to compute a sum
!***********************************************************************
!
!             SUBROUTINES CALLED
!
! ********************************************************************
!
!                    I/O
!
! ********************************************************************
!
!                    ERROR MESSAGES
!
! ********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
!     --- initialize f to zero if necessary

      if (jadd.eq.0) then
         do 50 i = 1, mp
            f(i) = 0d0
50       continue
      end if

!     --- add to f dv/dx
!         store first help factor in work

      do 200 i = 1, mpone
         ij = i
         do 100 j = 1, mpone
            work(ij) = wxp(i)*wxp(j)*p4p(i,j)
            ij = ij + mpone
100      continue
200   continue

!     --- add first factor to f

      do 600 i = 1, mpone
         ij = i
         do 500 j = 1, mpone
            sum = 0d0
            do 300 l = 1, mpone
               lj = l + (j - 1)*mpone
               sum = sum + pdxv(i,l)*old(lj+mp)
300         continue
            f(ij) = f(ij) + work(ij)*sum
            ij = ij + mpone
500      continue
600   continue

!     --- store second help factor in work

      do 800 i = 1, mpone
         ij = i
         do 700 j = 1, mpone
            work(ij) = -wxp(i)*wxp(j)*p2p(i,j)
            ij = ij + mpone
700      continue
800   continue

!     --- add second factor to f

      do 1200 i = 1, mpone
         ij = i
         do 1100 j = 1, mpone
            sum = 0d0
            do 900 m1 = 1, mpone
               im = i + (m1 - 1)*mpone
               sum = sum + pdxv(j,m1)*old(im+mp)
900         continue
            f(ij) = f(ij) + work(ij)*sum
            ij = ij + mpone
1100     continue
1200  continue

!     --- subtract from f du/dy
!         store first help factor in work

      do 1400 i = 1, mpone
         ij = i
         do 1300 j = 1, mpone
            work(ij) = -wxp(i)*wxp(j)*p3p(i,j)
            ij = ij + mpone
1300     continue
1400  continue

!     --- add first factor to f

      do 1800 i = 1, mpone
         ij = i
         do 1700 j = 1, mpone
            sum = 0d0
            do 1500 l = 1, mpone
               lj = l + (j - 1)*mpone
               sum = sum - pdxv(i,l)*old(lj)
1500        continue
            f(ij) = f(ij) + work(ij)*sum
            ij = ij + mpone
1700     continue
1800  continue

!     --- store second help factor in work

      do 2000 i = 1, mpone
         ij = i
         do 1900 j = 1, mpone
            work(ij) = wxp(i)*wxp(j)*p1p(i,j)
            ij = ij + mpone
1900     continue
2000  continue

!     --- add second factor to f

      do 2400 i = 1, mpone
         ij = i
         do 2300 j = 1, mpone
            sum = 0d0
            do 2100 m1 = 1, mpone
               im = i + (m1-1)*mpone
               sum = sum - pdxv(j,m1)*old(im)
2100        continue
            f(ij) = f(ij) + work(ij)*sum
            ij = ij + mpone
2300     continue
2400  continue

      end
