      subroutine el6082 ( f, old, wxp, pdxv, p1p, p2p, p3p, p4p, &
                          work, mvone, mv, numr )
! ======================================================================
!
!        programmer     Peter Minev
!        version 1.1    date   28-01-1996 Layout
!        version 1.0    date   06-04-1995
!
!   copyright (c) 1995-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     compute the residual of the first derivative with respect to
!     the numr - th coordinate, numr =1, 2; 2-D case;
!     it is used to compute the weak pressure gradient with integration
!     by parts.
!
!     EL6082 is called by the element subroutine ELD607
!            it must be preceded by the subroutines ELP640, ELP642, &
! ********************************************************************
!
!                      KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      implicit none
      integer mvone, mv, numr
      double precision f(mv), old(mv), wxp(mvone), &
                       pdxv(mvone,mvone), &
                       p1p(mvone,mvone), p2p(mvone,mvone), &
                       p3p(mvone,mvone), p4p(mvone,mvone), work(mv)

!     f       i/o  array of length 2*mv containing the righthandside
!                  or the residual
!     mv       i   number of velocity points in the element
!     mvone    i   number of velocity points in the one dimension
!     numr     i   the numr-th component of the gradient is to be computed
!     old      i   array of length mv containing the old pressure
!     p1p      i   arrays of length mvone x mvone containing the partial
!                  derivatives of the coordinates due to transformation for
!                  the pressure
!     p2p      i   See p1p
!     p3p      i   See p1p
!     p4p      i   See p1p
!     pdxv     i   array of length mvone x mvone contaning the values of the
!                  derivatives of the basisfunctions for the pressure in the
!                  pressure points in the x-direction
!     work     -   work array, length mv
!     wxp      i   array of length mvone containing the pressure weights for
!                  numerical integration in the x-direction corresponding
!                  to the reference element
!***********************************************************************
!
!                      COMMON BLOCKS
!
!***********************************************************************
!
!                      LOCAL PARAMETERS
!
      integer i, j, k, ij, il, kj, l
      double precision sum

!     i          Counting variable
!     ij         position in the vector f
!     il         position in the vector f
!     j          Counting variable
!     k          Counting variable
!     kj         position in the vector f
!     l          General loop variable
!     sum        Help parameter to compute a sum
!***********************************************************************
!
!             SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! ********************************************************************
!
!                    I/O
!
! ********************************************************************
!
!                    ERROR MESSAGES
!
!    1619   Component of gradient can not be computed
! ********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
      do 100 i = 1, mv
         f(i) = 0d0
100   continue

      if ( numr.lt.1 .or. numr.gt.2 ) then
         call eropen ( 'el6082' )
         call errint ( numr, 1 )
         call errsub ( 1619, 1, 0, 0 )
         call erclos ( 'el6082' )
         go to 5000
      end if
      go to ( 1, 2) numr

!     --- store first help factor in work

  1   do 300 i = 1, mvone
         ij = i
         do 200 j = 1, mvone
            work(ij) = wxp(i)*wxp(j)*p4p(i,j)*old(ij)
            ij = ij + mvone
200      continue
300   continue

!     --- subtract first factor of derivative part from f

      do 700 i = 1, mvone
         ij = i
         do 600 j = 1, mvone
            sum = 0d0
            do 500 k = 1, mvone
               kj = k + (j-1)*mvone
               sum = sum + pdxv(k,i)*work(kj)
500         continue
            f(ij) = f(ij) - sum
            ij = ij + mvone
600      continue
700   continue

!     --- store second help factor in work

      do 900 i = 1, mvone
         ij = i
         do 800 j = 1, mvone
            work(ij) = - wxp(i)*wxp(j)*p2p(i,j)*old(ij)
            ij = ij + mvone
800      continue
900   continue

!     --- subtract second factor of divergence part from f

      do 1300 i = 1, mvone
         ij = i
         do 1200 j = 1, mvone
            sum  = 0d0
            il = i
            do 1000 l = 1, mvone
               sum = sum + pdxv(l,j)*work(il)
               il = il + mvone
1000        continue
            f(ij) = f(ij) - sum
            ij = ij + mvone
1200     continue
1300  continue

      go to 5000

!     --- calculate the second component
!         store first help factor in work

  2   do 1500 i = 1, mvone
         ij = i
         do 1400 j = 1, mvone
            work(ij) = - wxp(i)*wxp(j)*p3p(i,j)*old(ij)
            ij = ij + mvone
1400     continue
1500  continue

!     --- add first factor of derivative part to f

      do 1900 i = 1, mvone
         ij = i
         do 1800 j = 1, mvone
            sum = 0d0
            do 1700 k = 1, mvone
               kj = k + (j-1)*mvone
               sum = sum + pdxv(k,i)*work(kj)
1700        continue
            f(ij) = f(ij) - sum
            ij = ij + mvone
1800     continue
1900  continue

!     --- store second help factor in work

      do 2100 i = 1, mvone
         ij = i
         do 2000 j = 1, mvone
            work(ij) = wxp(i)*wxp(j)*p1p(i,j)*old(ij)
            ij = ij + mvone
2000     continue
2100  continue

!     --- add second factor of divergence part to f

      do 2500 i = 1, mvone
         ij = i
         do 2400 j = 1, mvone
            sum  = 0d0
            il = i
            do 2200 l = 1, mvone
               sum = sum + pdxv(l,j)*work(il)
               il = il + mvone
2200        continue
            f(ij) = f(ij) - sum
            ij = ij + mvone
2400     continue
2500  continue

5000  continue

      end
