      subroutine el6083 ( g, f, w1, det, modecl, mdecl )
! ======================================================================
!
!        programmer     Peter Minev
!        version 1.1    date   28-01-1996 Layout
!        version 1.0    date   29-09-1994
!
!   copyright (c) 1994-1996  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! ********************************************************************
!
!                         DESCRIPTION
!
!     compute contributions to the spectral element vector
!     corresponding to the righthand side of the equation in 3-D case
!
!     EL6083 is called by the element subroutine ELD607
!            it must be preceded by the subroutines ELP640 and ELP646
! ********************************************************************
!
!                      KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
! ********************************************************************
!
!                      INPUT / OUTPUT    PARAMETERS
!
      use sepmodulecactl
      implicit none
      integer modecl, mdecl
      double precision g(mdecl), f(mdecl), w1(modecl), &
                       det(modecl,modecl,modecl)

!     det      i   array of length modecl x modecl x modecl containing the
!                  transformation determinant
!     f        o   array of length mdecl containing the spectral element
!                  vector
!     g        i   array of length mdecl containing the values of the
!                  fourth coefficient (righthand side) in the integration
!     node
!     mdecl    i   number of integration points in the element
!     modecl   i   number of integration points in one dimension
!     w1       i   array of length modecl containing the weights for
!                  numerical integration in the x-direction corresponding
!                  to the reference element
!***********************************************************************
!
!                        COMMON BLOCKS
!
!***********************************************************************
!
!                         LOCAL PARAMETERS
!
      integer i, j, ijk, k, mtdecl

!     i        counting variable
!     ijk      counting variable
!     j        counting variable
!     k        counting variable
!     mtdecl
!***********************************************************************
!
!             SUBROUTINES CALLED
!
! ********************************************************************
!
!                    I/O
!
! ********************************************************************
!
!                    ERROR MESSAGES
!
! ********************************************************************
!
!                    PSEUDO CODE
!
!   trivial
! ======================================================================
!
      mtdecl = modecl*modecl
      if( jadd.eq.0) then
         do 3009 i=1,mdecl
            f(i) = 0d0
 3009    continue
      end if

!     --- g is variable

      do 500 i = 1, modecl
         do 600 j = 1, modecl
            do 700 k = 1, modecl
               ijk = i + (j-1)*modecl + (k-1)*mtdecl
               f(ijk) =f(ijk)+g(ijk)*w1(i)*w1(j)*w1(k)*det(i,j,k)
700         continue
600      continue
500   continue

      end
