      subroutine el6086 ( f, w1, det, old, mone )
! ======================================================================
!
!        programmer     Peter Minev/Patrick Anderson
!        version 1.2    date  24-07-1998  Include SPcommon/ctimen
!        version 1.1    date  28-01-1996  Layout
!        version 1.0    date  10-02-1995
!
!   copyright (c) 1995-1998  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the spectral element mass residual; 3-D case
!     EL6086 is called by the element subroutine ELM607
!            it must be preceded by the subroutines ELP640 and ELP646
! **********************************************************************
!
!                       KEYWORDS
!
!    element_matrix
!    element_vector
!    elliptic_equation
!     initialization
!    parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduletime
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      implicit none
      integer mone
      double precision f(mone,mone,mone), w1(mone), det(mone,mone,mone), &
                       old(mone,mone,mone)

!     det      i   array of length mone x mone x mone containing the
!                  determinant of the jacobian
!     f        o   array of length m containing the spectral element
!                  residual
!     mone     i   number of integration points in one dimension
!     old      i   array of length mone x mone x mone containing the
!                  old solution
!     w1       i   array of length mone containing the weights for
!                  numerical integration in 1 direction correspon- &
!                  ding to the reference element
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      double precision fac

!     fac       factor = 1. (first step) and 1.5 (afterwards)
!     i          Counting variable
!     j          Counting variable
!     k          Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- set fac

      fac = 1.5d0
      if (icons.eq.0) fac = 1d0

      do i = 1, mone
         do j = 1, mone
            do k = 1, mone
               f(i,j,k) = f(i,j,k)*tstep + &
                          w1(i)*w1(j)*w1(k)*det(i,j,k)*old(i,j,k)*fac
            end do
         end do
      end do

      end
