      subroutine el6221( coor, index1, x, inter, iinter, moneloc, mloc )
! ======================================================================
!
!        programmer    Patrick Anderson
!        version  1.0  date 30-10-1999
!
!   copyright (c) 1999-1999  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Determine element number for set of arbitrary points
!
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(*), moneloc, mloc,iinter(*)
      double precision coor(3,*), x(mloc,3), inter(6,*)

!     coor       i    array of length ndim x npoint containing the co-ordinates
!                     of the nodal points with respect to the global numbering
!     iinter     i    ?
!     index1     i    Array of length inpelm containing the point numbers
!                     of the nodal points in the element
!     inter      i    ?
!     m          i    number of integration points
!     moneloc       i    number of integration points in one dimension
!     x          i    x co-ordinate of point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k, ijk,indloc, m2
      double precision  lox,loy,loz,rbx,rby,rbz,xlocal(1,3)

!     i          Counting variable
!     ijk        Counting variable
!     indloc        indlocex of current point
!     j          Counting variable
!     k          Counting variable
!     l          General loop variable
!     lox        Minimum value for x in current element
!     loy        Minimum value for y in current element
!     loz        Minimum value for z in current element
!     m2         moneloc * moneloc
!     rbx        Maximum value for x in current element
!     rby        Maximum value for y in current element
!     rbz        Maximum value for z in current element
!     xlocal     Local coordinate array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     INSTOP         Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      if (moneloc.ne.2) then
         write (*,*) 'Only first order elements allowed'
         call instop
      end if
      m2 = moneloc * moneloc

!     --- store global integration points

      do i = 1, moneloc
         do j = 1, moneloc
            do k = 1, moneloc
               ijk = i+(j-1)*moneloc+(k-1)*m2
               x(ijk,1) = coor(1, index1(ijk))
               x(ijk,2) = coor(2, index1(ijk))
               x(ijk,3) = coor(3, index1(ijk))
            end do
         end do
      end do

!     --- determine bounding box of element

      lox  = min(x(1,1),x(2,1),x(3,1),x(4,1), &
                 x(5,1),x(6,1),x(7,1),x(8,1))
      rbx  = max(x(1,1),x(2,1),x(3,1),x(4,1), &
                 x(5,1),x(6,1),x(7,1),x(8,1))
      loy  = min(x(1,2),x(2,2),x(3,2),x(4,2), &
                 x(5,2),x(6,2),x(7,2),x(8,2))
      rby  = max(x(1,2),x(2,2),x(3,2),x(4,2), &
                 x(5,2),x(6,2),x(7,2),x(8,2))
      loz  = min(x(1,3),x(2,3),x(3,3),x(4,3), &
                 x(5,3),x(6,3),x(7,3),x(8,3))
      rbz  = max(x(1,3),x(2,3),x(3,3),x(4,3), &
                 x(5,3),x(6,3),x(7,3),x(8,3))
      indloc=10

!     --- check of any points have to be interpolated

   5  if ( (iinter(indloc).eq.0) .and. (iinter(1).gt.iinter(2)) &
           .and. ((indloc-10).lt.iinter(1)) ) &
      then
          xlocal(1,1)=inter(1,indloc)
          xlocal(1,2)=inter(2,indloc)
          xlocal(1,3)=inter(3,indloc)

!         --- check if point is in bounding box

          if (xlocal(1,1).ge.lox .and. &
              xlocal(1,1).le.rbx .and. &
              xlocal(1,2).ge.loy .and. &
              xlocal(1,2).le.rby .and. &
              xlocal(1,3).ge.loz .and. &
              xlocal(1,3).le.rbz) &
          then
             inter(4,indloc)=ielem
             iinter(indloc)=1
             iinter(2) = iinter(2) + 1
             indloc = indloc + 1
          else
          indloc = indloc + 1
          goto 5
          end if
       else
          indloc = indloc + 1
          if ( (indloc-10).lt.iinter(1) ) goto 5
       end if
       if ( (indloc-10).lt.iinter(1) ) goto 5
      end
