      subroutine elcheckcentr ( ishape, index1, coor )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-06-2002
!
!   copyright (c) 2002-2002  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Help routine, to check if the centroids in elements of shape 15 and 16
!     are created in the right way, i.e. if they are really positioned
!     in the centre
! **********************************************************************
!
!                       KEYWORDS
!
!     check
!     element
!     element
!     centre
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ishape, index1(*)
      double precision coor(3,*)

!     coor           i    array of length ndim x npoint containing the
!                         co-ordinates of the nodal points with respect to the
!                         global numbering
!     index1         i    Array of length inpelm containing the point numbers
!                         of the nodal points in the element
!     ishape         i    Shape number of elements
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nodc, indloc(3,4), i, j, nodp, ind1(4)
      double precision xc, yc, zc, x, y, z

!     i              Counting variable
!     ind            Integer array of size 3 x 4
!                    indloc(.,j) contains the local vertex numbers of the element
!                    corresponding to the j-th local face
!     ind1           Conaains the local vertex numbers of the element
!     j              Counting variable
!     nodc           Node number of centroid
!     nodp           Nodal point sequence number
!     x              x co-ordinate of point
!     xc             x co-ordinate of centre
!     y              y co-ordinate of point
!     yc             y-coordinate of centre
!     z              z co-ordinate of point
!     zc             z-coordinate of centre
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERREAL         Put real in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2426   Centre of face not in centroid
!    2427   Centre of element not in centroid
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data indloc / 1, 3, 5, &
                 1, 3,10, &
                 3, 5,10, &
                 1, 5,10 /, &
      ind1/ 1, 3, 5, 10 /
! ======================================================================
!
      call eropen ( 'elcheckcentr' )

!     --- First check centroids of faces

      do i = 1, 4
         nodc = index1(10+i)
         xc = coor(1,nodc)
         yc = coor(2,nodc)
         zc = coor(3,nodc)
         x = 0d0
         y = 0d0
         z = 0d0
         do j = 1, 3
            nodp = index1(indloc(j,i))
            x = x + coor(1,nodp)
            y = y + coor(2,nodp)
            z = z + coor(3,nodp)
         end do
         if ( abs(3d0*xc-x)+abs(3d0*yc-y)+abs(3d0*zc-z).gt.1d-10 ) then
            call errint ( 10+i, 1 )
            call errint ( ielem, 2 )
            call errint ( nodc, 3 )
            call erreal ( xc, 1 )
            call erreal ( yc, 2 )
            call erreal ( zc, 3 )
            do j = 1, 3
               nodp = index1(indloc(j,i))
               call errint ( nodp, 3+j )
               call erreal ( coor(1,nodp), 1+3*j )
               call erreal ( coor(2,nodp), 2+3*j )
               call erreal ( coor(3,nodp), 3+3*j )
            end do
            call erreal ( x/3d0, 13 )
            call erreal ( y/3d0, 14 )
            call erreal ( z/3d0, 15 )
            call errsub ( 2426, 6, 15, 0 )
         end if

      end do
      if ( ishape.eq.16 ) then
         nodc = index1(15)
         xc = coor(1,nodc)
         yc = coor(2,nodc)
         zc = coor(3,nodc)
         x = 0d0
         y = 0d0
         z = 0d0
         do j = 1, 4
            nodp = index1(ind1(j))
            x = x + coor(1,nodp)
            y = y + coor(2,nodp)
            z = z + coor(3,nodp)
         end do
         if ( abs(4d0*xc-x)+abs(4d0*yc-y)+abs(4d0*zc-z).gt.1d-10 ) then
            call errint ( ielem, 1 )
            call errint ( nodc, 2 )
            call erreal ( xc, 1 )
            call erreal ( yc, 2 )
            call erreal ( zc, 3 )
            do  j = 1, 4
               nodp = index1(ind1(j))
               call erreal ( coor(1,nodp), 1+3*j )
               call erreal ( coor(2,nodp), 2+3*j )
               call erreal ( coor(3,nodp), 3+3*j )
            end do
            call erreal ( x*0.25d0, 16 )
            call erreal ( y*0.25d0, 17 )
            call erreal ( z*0.25d0, 18 )
            call errsub ( 2427, 6, 18, 0 )
         end if
      end if

      call erclos ( 'elcheckcentr' )
      end
