      subroutine eld007(jelp)
! ======================================================================
!
!        programmer     Peter Minev
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.3  date 12-02-1999 Common cinforel
!        version  1.2  date 25-01-1996 Layout
!                                      (patrick anderson)
!
!   copyright (c) 1994-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     this subroutine performs the initialisations for subroutine ELD607
!     the parameters in the common blocks celint get a start value
! **********************************************************************
!
!                       KEYWORDS
!
!     spectral
!     derivative
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      use sepmoduleelem
      implicit none
      integer jelp

!     jelp     o     Indication of the type of basis function subroutine must
!                    be called by ELD607, possible vaues:
!                    1:  subroutine ELP640  (one-dimensional)
!                    (more values not yet done)
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELCOMM    Initializes the commons celp and celint with standard values
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     ERRINT    Put integer in error message
!     ERRSUB    Error messages
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     406   Value of icheld not allowed
!    1697   combination n and mone not correct
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       MODULES USED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- fill some standard values in the common block celint

      call eropen ('eld007')
      call elcomm
      ncomp = 1

!     --- set parameters depending on the type numbers

      if ( itype.eq.607 ) then

!     --- itype = 607, 1-D, 2-D and 3-D spectral element
!         one unknown per nodal point for a scalar

         n = inpelm
         m = inpelm
         mone = int(n**(1d0/ndim)+0.5d0)
         if ( n.ne.mone**ndim ) then
            call errint ( ndim, 1 )
            call errint ( n, 2 )
            call errint ( mone**ndim, 3 )
            call errsub ( 1697, 3, 0, 0 )
         end if

         jelp = 1
         jdercn = 1
         nvert = 2**ndim
         ndimlc = ndim
      end if

!     --- check value of icheld

      if ( icheld.lt.1 .or. icheld.gt.12 ) then
         call errint ( icheld, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errsub ( 406, 3, 0, 0)
      end if

!     --- load jconsf

      if (ind(2).eq.0) then

!     --- g = 0, no right-hand side function

         jconsf = 0

      else if ((ind(2).lt.0) .or. (ind(2).eq.2002)) then

!     --- g is constant (per element)

         jconsf = 1

      else

!     --- general

         jconsf = 2

      end if

      call erclos ('eld007')

      end
