      subroutine eld650 ( coor, elemvc, elemwg, iuser, user, vectr0, &
                          index1, index2 )
!
! ********************************************************************
!
!
!        programmer    Arno van Duin
!        version 1.1   date   91-03-21 adjusted for NDIM = 3
!        version 1.0   date   90-11-01
!
!
!   copyright (c) 1990  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
!
! ********************************************************************
!
! the outward flux through the edges of a mixed finite element triangle
! are computed on the element level
!
! ********************************************************************
!
!
!                INPUT / OUTPUT PARAMETERS

      use sepmodulecactl
      implicit none

      integer iuser(*), index1(*), index2(*)
      double precision coor(*), elemvc(*), elemwg(*), user(*), vectr0(*)

!
!     coor     i     array of length 2 x npoint containing the co-ordinates of
!                    the nodal points with respect to the global numbering
!
!                    x  = coor (1,i);  y  = coor (2,i);
!                     i                 i
!
!     elemvc   o     Element vector to be computed
!
!     elemwg   o     array that is filled with the weight factors of the
!                    averaging procedure
!
!     index1   i     Array of length inpelm containing the point numbers
!                    of the nodal points in the element
!
!     index2   i     Array of length icount containing the positions of the
!                    degrees of freedom in the element with respect to VECTR0
!                    (and VECTR1) sequentially
!
!     iuser    i     Integer user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
!
!     user     i     Real user array to pass user information from
!                    main program to subroutine. See STANDARD PROBLEMS
!
!     vectr0   i     In this array all preceding solutions are stored, i.e.
!                    all solutions that have been computed before and
!                    have been carried to system or deriva by the parameter
!                    islold in the parameter list of these main subroutines
!
! ********************************************************************
! ********************************************************************
!
!               LOCAL PARAMETERS

      double precision A(100), det, Amin1(100), B(10), Amin1B(10), &
       BTAmin(1), F2(10), x(56), elemmt(100), vecold(10), vhlp
      integer i, ndim

!     Meaning of local parameters
!     (see E.F. Kaasschieter: Preconditioned conjugate gradients and mixed- &
!                             hybrid finite elements for the solution of
!                             potential flow problems)
!
!     A           real matrix: A  = / (C v ) . v  dx, C is the inverse of the
!                               ij  S     i     j
!                 hydraulic conductivity tensor, v are the basis functions
!
!     det         real, determinant of the mapping to a reference triangle
!
!                                         -1
!     Amin1       real matrix containing A
!
!                                         -1
!     Amin1B      real vector containing A  B
!
!     B           real vector: B = - / nabla . v dx = -1
!                               i    S          i
!                                T -1
!     BTAmin      real equal to B A  B
!
!                                       -1   -1     T -1   -1 T -1
!     elemmt      real matrix containg A  - A  B ( B A  B )  B A
!
!     F2          real vector: F  = - / f dx = f * opp(S)
!                               2     S
!
!     i           loop variable
!
!     ndim        the dimension of space
!
!     vecold      real array containing the values of the old solution (VECTR0)
!                 in the midpoints of the edges of the element
!
!     vhlp        temporary variable for putting the solution vector in the
!                 right order
!
!     x           real array containing the coordinates of the midpoints of the
!                 edges of the element
!
! ********************************************************************
!
!                              METHOD
!
!
!       Differential equation:
!
!                    u = - A grad phi
!
!                    div u = f
!
!
!       leads to:
!                     -1
!              U   = A  ( F  - B Phi - C Labda )
!                          1
!
!                       T -1   -1   T -1
!              Phi = ( B A  B )  ( B A  ( F  - C Labda ) - F )
!                                          1                2
!
!              D Labda = F
!
!        the last equation has been solved and the solution stored in VECTR0
!        is used to compute U
!
! ********************************************************************
!
!                  USER INPUT
!
!     The user provides the coefficients through the arrays IUSER and USER
!     in the usual way
!     The sequence is:
!
!     ndim = 2                           ndim = 3
!     1:  alpha 11                       1:  alpha 11
!     2:  alpha 12                       2:  alpha 12
!     3:  alpha 22                       3:  alpha 31
!                                        4:  alpha 22
!                                        5:  alpha 23
!                                        6:  alpha 33
!     4:  u 1  ( = 0 )                   7:  u 1  ( = 0 )
!     5:  u 2  ( = 0 )                   8:  u 2  ( = 0 )
!     6:  beta ( = 0 )                   9:  beta ( = 0 )
!     7:  f                             10:  f
!     8:  rho cp ( not used )
!     8/9:  method for upwinding (8 if no heat equation must be computed, &
!                                 otherwise 9 )
!
! ********************************************************************
!
!               SUBROUTINES CALLED
!
!   ELM651   fills x
!   ELM652   computes A
!   ELM653   computes the inverse of A
!   ELM654   fills B (with -1s)
!   ELM655   performs matrix multiplications
!   ELM656   computes D
!   ELM657   computes F2
!   ELD651   adds F to the element vector
!
! ********************************************************************

      call eropen ( 'eld650' )

      if ( inpelm.eq.14 ) then
         ndim = 3
      else
         ndim = 2
      end if

      do 10 i = 1, icount
         elemwg(i) = 1
         vecold(i) = vectr0(index2(i))
10    continue

      if ( ndim.eq.3 ) then
         vhlp = vecold(1)
         vecold(1) = vecold(3)
         vecold(3) = vecold(2)
         vecold(2) = vecold(4)
         vecold(4) = vhlp
      end if

      call elm651 ( coor, inpelm, index1, x, ndim )

!     *  First compute A

      call elm652 ( ielgrp, user, iuser, x, inpelm, A, det, ndim )

!     *  Next compute inverse(A)

      call elm653 ( A, icount, Amin1 )

!                 -1
!     *  Compute A  B

      call elm654 ( icount, B )
      call elm655 ( Amin1, icount, icount, B, icount, 1, Amin1B )

!                 -1   -1   T -1  -1  -1  T
!     *  Compute A  - A  B(B A  B)  (A  B)

      call elm655 ( B, 1, icount, Amin1B, icount, 1, BTAmin )
      call elm656 ( icount, Amin1, Amin1B, BTAmin(1), elemmt )
!
!          -1   T -1  -1
!      *  A  B(B A  B)  F
!                        2
!
!      *  F  = - rhsf ( x, y ) * opp ( e )
!          2
!
!                -1   T -1  -1      -1   -1   T -1  -1  -1  T
!      *  flux = A  B(B A  B)  F - (A  - A  B(B A  B)  (A  B) ) vecold
!                               2
!
      call elm655 ( elemmt, icount, icount, vecold, icount, 1, elemvc )
      call elm657 ( ielgrp, iuser, user, det, icount, x, F2)
      call eld651 ( icount, Amin1B, BTAmin(1), F2, elemvc )

!     *   adjust sign to the direction which is defined to be in the positive
!         direction if the first nodal point number of the corresponding edge
!         is lower then the last in case of NDIM=2, when NDIM=3 the direction
!         is defined positive if the corners of the triangle are numbered in
!         a clockwise direction.

      if ( ndim.eq.2 ) then
         do 20 i = 1, inpelm, 2
            if ( index1(i).gt.index1(mod(i+2,inpelm)) ) &
               elemvc((i+1)/2) = -elemvc((i+1)/2)
20       continue
      else if ( ndim.eq.3 ) then
         if ( index1(5).lt.index1(10) ) then
            if ( ( index1(10).gt.index1(3) ) .and. &
                 ( index1(3) .gt.index1(5) ) ) then
               elemvc(1) = -elemvc(1)
            end if
         else
            if ( ( index1(10).gt.index1(3) ) .or. &
                 ( index1(3) .gt.index1(5) ) ) then
               elemvc(1) = -elemvc(1)
            end if
         end if
         if ( index1(1).lt.index1(10) ) then
            if ( ( index1(10).gt.index1(5) ) .and. &
                 ( index1(5).gt.index1(1) ) ) then
               elemvc(2) = -elemvc(2)
            end if
         else
            if ( ( index1(10).gt.index1(5) ) .or. &
                 ( index1(5).gt.index1(1) ) ) then
               elemvc(2) = -elemvc(2)
            end if
         end if
         if ( index1(3).lt.index1(10) ) then
            if ( ( index1(10).gt.index1(1) ) .and. &
                 ( index1(1) .gt.index1(3) ) ) then
               elemvc(3) = -elemvc(3)
            end if
         else
            if ( ( index1(10).gt.index1(1) ) .or. &
                 ( index1(1) .gt.index1(3) ) ) then
               elemvc(3) = -elemvc(3)
            end if
         end if
         if ( index1(1).lt.index1(5) ) then
            if ( ( index1(5).gt.index1(3) ) .and. &
                 ( index1(3).gt.index1(1) ) ) then
               elemvc(4) = -elemvc(4)
            end if
         else
            if ( ( index1(5).gt.index1(3) ) .or. &
                 ( index1(3).gt.index1(1) ) ) then
               elemvc(4) = -elemvc(4)
            end if
         end if
         vhlp = elemvc(1)
         elemvc(1) = elemvc(4)
         elemvc(4) = elemvc(2)
         elemvc(2) = elemvc(3)
         elemvc(3) = vhlp
      end if

      call erclos ( 'eld650' )

      end
