      subroutine elf001 ( icount, jcount, ifunc, func, ipos, iuser, &
                          user )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.2  date 15-08-2007 Allow coef %scalar
!        version  2.1  date 16-05-1991 Minor correction
!        version  2.0  date 26-03-1990 Delete name from parameter list
!        version  1.1  date 12-03-1990 Start of user at odd positions
!        version  1.0  date 10-01-1983
!
!   copyright (c) 1983-2007  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      fill a part of  arrays iuser and user corresponding to
!      one standard element and one coefficient
!      Complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     fill
!     coefficient
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*), icount, jcount, ifunc, ipos
      double precision user(*), func(*)

!      func    i    value of the coefficient when it is a constant
!                   func corresponds to the complex case
!                   func(1) contains the real and func(2) the imaginary part
!      icount i/o   last position used in iuser
!      ifunc   i    integer information of the coefficient
!                   when ifunc = 0  the coefficient is constant with value func
!                   when ifunc>0 the coefficient is a function
!      ipos    i    position of parameter in array iwork in subroutine fil...
!      iuser  i/o   Integer user array to pass user information from
!                   main program to subroutine. See STANDARD PROBLEMS
!      jcount i/o   last position used in user
!      user   i/o   Real user array to pass user information from
!                   main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inirepscal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREPSCAL     Corrects integer value in case of scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     115   ifunc<0
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elf001' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elf001'
         write(irefwr,1) 'ifunc', ifunc
  1      format ( a, 1x, (10i9) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      icount = icount+1
      if ( ifunc<0 ) then

!     --- error 115   ifunc<0

         call eropen ( 'elf001' )
         call errint ( ifunc, 1 )
         call errint ( ipos, 2 )
         call errsub ( 115, 2, 0, 0 )
         call erclos ( 'elf001' )

      else if ( ifunc==0 ) then

!     --- ifunc = 0   func is constant

        if ( abs(func(1))<=0d0 .and. abs(func(2))<=0d0 ) then

!       --- func = 0

          iuser(icount) = 0

        else

!       --- func # 0

          jcount = jcount+1
          if ( mod(jcount, 2)==0 ) jcount = jcount + 1
          iuser(icount) = -jcount
          user(jcount) = func(1)
          jcount = jcount+1
          user(jcount) = func(2)

        end if

      else

!     --- ifunc>0

        iuser(icount) = inirepscal(ifunc)

      end if
1000  call erclos ( 'elf001' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icount, jcount, iuser(icount)', &
                          icount, jcount, iuser(icount)
         write(irefwr,*) 'End elf001'

      end if  ! ( debug )

      end subroutine elf001
