      subroutine eli012 ( ichint, ichfun, mvec, usollc, coor2d, index1, &
                          value )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.5  date 04-07-2003 Remove common celwrk
!        version  1.4  date 08-03-2003 Enlarge qmat
!
!   copyright (c) 1991-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!      subroutine eli012 is used to compute the boundary integral over
!      a linear surface triangle
!      Complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_integral
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulework
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichint, mvec, index1(3), ichfun
      double precision coor2d(3,*)
      double complex usollc(mvec,3), value(*)

!     coor2d    i   Array containing the co-ordinates of the mesh
!     ichfun  i   indication of the value of f.
!                 Possibible values:
!                 0         f = 1
!                 >0        f = func ( ichfun, x, y, z ) (real case)
!                           or
!                           f = cfunc ( ichfun, x, y, z ) (complex
!                               case)
!                           at this moment only the combination f#1
!                           and ichint=1 is permitted
!     ichint  i   indicates the kind of integral to be computed.
!                 possibilities:
!                 1          /  f u  ds
!                 2          /  f  u.n  ds
!                 9          /  f ds
!     index1  i   Array containing the nodal point numbers of the element
!     mvec    i   Number of components of the solution vector
!     usollc  i   Local solution vector
!     value   o   Contains the value of the computed integralc
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double complex sum, f(3)
      double precision x(3,3), w(3), dphidx(9), xgauss(9), &
                       qmat(3,27)

!     dphidx         Array of length n x m x ndim  containing the
!     derivative
!                    of the basis functions in the sequence:
!                    d phi / dx (xg ) = dphidx (i,k,1);
!                         i        k
!                    d phi / dy (xg ) = dphidx (i,k,2);
!                         i        k
!                    d phi / dz (xg ) = dphidx (i,k,3);
!                         i        k
!                    If the element is a linear simplex, the
!                    derivatives are constant and only k = 1 is filled.
!     f              array of length m containing the rhs vector
!     i              Counting variable
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     sum            Help parameter to compute a sum
!     w              array of length m containing the weights for
!                    integration
!     x              x co-ordinate of point
!     xgauss         array of length m x ndim containing the co-ordinates
!                    of the gauss points.  array xgauss is only filled when
!                    gauss integration is applied.
!                    xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                    zg_i = xgauss (i,3)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double complex cfunc

!     CFUNC   Computes the value of the vector in a point as function of the
!             co-ordinates
!     ELP620  Compute basis functions and weights for linear triangle
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eli012' )

!     --- Compute the weight functions

      m = 3
      iweigh = 1
      ideriv = 0
      irule = 1
      igausp = 0
      icoor = 1
      call elp620 ( coor2d, x, w, dphidx, xgauss, work, index1, qmat )
      if ( ichfun.eq.0 ) then
         do i = 1, m
            f(i) = (1d0,0d0)
         end do
      else
         do i = 1, m
            f(i) = cfunc ( ichfun, x(i,1), x(i,2), x(i,3) )
         end do
      end if
      if ( ichint.eq.1 ) then
         sum = (0d0,0d0)
         do i = 1, m
            sum = sum + w(i)*f(i)*usollc(1,i)
         end do
         value(1) = sum
      else if ( ichint.eq.9 ) then
         sum = (0d0,0d0)
         do i = 1, m
            sum = sum + w(i)*f(i)
         end do
         value(1) = sum
      end if
      call erclos ( 'eli012' )
      end
