      function elic00 ( icheli, f, w, u, dudx, work )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.3  date 04-07-2003 Remove common celwrk
!        version  2.2  date 15-03-1996 icheli=8 introduced
!        version  2.1  date 09-11-1994 icheli=7 introduced
!        version  2.0  date 05-08-1993 Removal of mf
!        version  1.0  date 08-02-1991
!
!   copyright (c) 1991-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute a volume integral over one element depending on icheli
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icheli
      double precision  dudx(m), u(m), elic00, f(m), w(m), work(m)

!     dudx     i     Real m x ndim  array in which the values of the
!                    derivatives of the solution vector in the integration
!                    points are stored in the sequence:
!                    du / dx  (x ) = dudx ( k, j )
!                           j   k
!                    dudx is only filled if jderiv > 0
!                    At this moment dudx is used as one-dimensional array
!                    because, ndim is 1
!     elic00   o     Computed integral
!     f        i     Array containing the fucntion f to be integrated in the
!                    integration points
!     icheli   i     choice parameter. See subroutine integr. Possibilities:
!                    1:   / f(x) dx
!                    2:   / f(x) u(x) dx
!                    3-5: / f(x) du(x)/dx  dx   i=icheld-2
!                                        i
!                    7    / dx
!     u        i     Solution to be integrated in integration points
!     w        i     Contains the weights for the numerical integration
!     work           Work array of length m
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!     i              General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ddot

!     DDOT           (CDOT)     Inner product of two vectors
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elic00' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elic00'
         write(irefwr,1) 'icheli, jdercn', icheli, jdercn
         call prinrl ( f, m, 'f' )
  1      format ( a, 1x, (10i6) )
  2      format ( a, 1x, (5d12.4) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Store f, uf or f du/dx into work

      work = 0d0
      if ( icheli==1 ) then

!     --- icheli = 1, only f

         work = f

      else if ( icheli==2 ) then

!     --- icheli = 2, f u

            work = f*u

      else if ( icheli.lt.7 ) then

!     --- icheli = 3-5, f du/dx

         if ( debug ) call prinrl ( dudx, m, 'dudx' )

         if ( jdercn==0 ) then

!        --- Derivative constant

            work = f*dudx(1)

         else

!        --- Derivative variable

            do i = kstep, m, kstep
               work(i) = f(i) * dudx(i)
            end do

         end if

      else if ( icheli==8 ) then

!     --- icheli = 8, f u^2

         do i = kstep, m, kstep
            work(i) = f(i) * u(i)**2
         end do

      end if

      if ( debug ) call prinrl ( work, m, 'work' )

      if ( icheli==7 ) then

!     --- icheli = 7, compute integral

         elic00 = 0d0
         do i = kstep, m, kstep
            elic00 = elic00 + w(i)
         end do

      else if ( kstep==1 ) then

         elic00 = ddot ( m, work, 1, w, 1 )

      else

         elic00 = 0d0
         do i = kstep, m, kstep
            elic00 = elic00+work(i)*w(i)
         end do  ! i = kstep, m, kstep

      end if

1000  call erclos ( 'elic00' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,2) 'elic00', elic00
         write(irefwr,*) 'End elic00'

      end if  ! ( debug )

      end function elic00
