      subroutine elm501 ( array )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 23-08-2001 Correct call to prinlg
!        version  2.0  date 26-12-1994 New parameter list
!        version  1.5  date 22-12-1992 JvdZ: improved error report
!
!   copyright (c) 1984-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill element mass matrix and element mass vector for second order
!     equations
!     itype=501 :    cartesian coordinates   linear triangle
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmodulecomio
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision array(*)

!     array        Work array for the element subroutines
!                  contents:
!                  first part created by elp subroutines:
!                  ipx:     x(i)
!                  ipy:     y(i)
!                  ipw:     w(i)
!                  second part local parameters:
!                  ipc  :   c(i)    contains values of c
!                  ipwork:  work array of length 3m
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer idum(1), ipwork, jwcons, k
      double precision c, delta, dum(1), sign
      integer ipx,ipy,ipw,ichphi,indc,mstc,ipc,igaus,jfunc
      save ipx,ipy,ipw,ichphi,indc,mstc,ipc,igaus,jfunc

!     c
!     delta   jacobian of element
!     dum     General double precision dummy variable
!     ichphi  ?
!     idum    General dummy variable
!     igaus   ?
!     indc    ?
!     ipc
!     ipos
!     ipw     ?
!     ipwork  Starting address in array work of work array
!     ipx     ?
!     ipy     ?
!     jcart   indication whether the element is a triangle with
!             cartesian coordinates (1)
!             or a circular triangle with cylinder symmetric
!             coordinates (2)
!     jdiag   Indicates if the basis functions satisfy phi i(j) = 0,
!             if i#j (jdiag=1) or not (jdiag=2)
!     jfunc   ?
!     jwcons  Indication if the weight function is constant (1) or not (2)
!     k       Counting variable
!     m       number of integration points
!     mstc    ?
!     n       number of basis functions for each component
!     nuser   ?
!     sign    ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL2003  Compute right-hand side term for an element vector (one comp.)
!     ELFLR5  Get user-defined parameters (with funccf)
!     ELP210  Compute basis functions linear triangle
!     ELUSER  Get parameters from arrays user,E%iuser
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     INSTOP  Stop the program
!     PRINLG  Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       3   Element has incorrect number of nodal points for this type
!      42   Array E%iuser(.) has value which is too small
!      74   Incorrect number of degrees of freedom for this type
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen( 'elm501' )
      if ( ifirst.eq.0 ) then

!     --- ifirst = 0, initialize some parameters and arrays

         n = 3
         m = 3
         jcart = 1
         ichphi = 0
         jdiag = 1
         jwcons = jcart
         igaus = 1
         ipx = 1
         ipy = ipx+m
         ipw = ipy+m
         ipc = ipw+m
         ipwork = ipc+m
         ichphi = 0
         ipos = E%iuser(ielgrp+5)
         if(ipos.lt.7) then
            call errint(ielgrp+5, 1)
            call errint(ipos, 2)
            call errint(ielem, 3)
            call errint(itype, 4)
            call errint(ielgrp, 5)
            call errsub ( 42, 5, 0, 0)
         end if
         nuser = 0
         call eluser(indc, mstc, array(ipc), ipos, E%iuser, E%user, &
                     nelemcac, npointcac, nuser, m)
         call prinlg('E%iuser', ipos-1, E%user, E%iuser)
         call prinlg('user', -nuser, E%user, E%iuser)
         if(inpelm.ne.n) then
            call errint(inpelm, 1)
            call errint(n, 2)
            call errint(ielem, 3)
            call errint(itype, 4)
            call errint(ielgrp, 5)
            call errsub ( 3, 5, 0, 0)
         end if
         if(icount.ne.n) then
            call errint(icount, 1)
            call errint(n, 2)
            call errint(ielem, 3)
            call errint(itype, 4)
            call errint(ielgrp, 5)
            call errsub ( 74, 5, 0, 0)
         end if
         if(indc.lt.0 .or. indc.eq.2002) then
            jfunc = jwcons
         else
            jfunc = 2
         end if
         if(ierror.ne.0) call instop
      end if

!     --- computation of basis functions

      call elp210 ( ichphi, coor, sign, array(ipx), array(ipy), &
                    array(ipw), array, array, index1 )
      k = icount/inpelm
      if ( indc.gt.0 ) &
           call elflr5 ( igaus, indc, mstc, E%user, array(ipc), &
                         array(ipx), array(ipy), array, dum, n, m, k, &
                         index1, array, idum, dum )
      if ( vector ) then
           call el2003 ( jfunc, jdiag, array(ipc), E%vec, array(ipw), &
                         array, n, m, array(ipwork), .true. )
      end if
      if ( matrix ) then
         c = 2d0*(array(ipc)+array(ipc+1)+array(ipc+2))
         delta = array(ipw)/20d0
         E%mat(1,1) = delta*(c+array(ipc)*4d0)
         E%mat(2,1) = delta*(c-array(ipc+2))
         E%mat(3,1) = delta*(c-array(ipc+1))
         E%mat(1,2) = E%mat(2,1)
         E%mat(2,2) = delta*(c+array(ipc+1)*4d0)
         E%mat(3,2) = delta*(c-array(ipc))
         E%mat(1,3) = E%mat(3,1)
         E%mat(2,3) = E%mat(3,2)
         E%mat(3,3) = delta*(c+array(ipc+2)*4d0)
      end if
      call erclos( 'elm501' )
      end
