      subroutine elm612 ( swork )
! ======================================================================
!
!        programmer    Patrick Anderson
!        version  1.4  date 09-03-2013 Replace elflr7 by elflr71
!        version  1.3  date 12-12-2012 Long integers
!        version  1.2  date 18-09-2002 New call to el3007
!
!   copyright (c) 1996-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!
!
! **********************************************************************
!
!                       KEYWORDS
!
!     spectral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmodulecactl
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision swork(*)

!     swork          spectral element work array
!                    the work array contains several quantities of the high
!                    order spectral element discretization, the first position
!                    of each quantity is pointed at by an integer pointer
!                    total length:  m*m + 24*m + 4*mone
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer isub, isup, i, ichois, &
           iparry, ipxre, ipx, ipw, ipd1, &
           ipdet, ip, ipwork, ipwork1, ipwork2, &
           ipmt,ipgrd,ipvec1, ipvec2, ipvec3, ipvel, ipold, jcheld
      double precision rdum
      save  isub, isup, i, &
           iparry, ipxre, ipx, ipw, ipd1, &
           ipdet, ip, ipwork, ipwork1, ipwork2, &
           ipmt,ipgrd, ipvec1, ipvec2, ipvec3, ipvel, ipold

!     i              Counting variable
!     jcheld         Choice parameter to distinguish between the various
!                    possibilities of this subroutine
!     ichois         Choice parameter
!     ip             indicate partial derivatives of the coordinates due to
!                    the isoparametric transformation, length m
!     iparry         indicates array in which the coefficients are stored
!                    length 7 x m
!     ipd1           indicates the values of derivatives of the basisfunctions
!                    in the interpolation points in the x-direction correspon- &
!                    ding to the reference element, length mone x mone
!     ipdet          transformation determinant, length m
!     ipgrd          ?
!     ipmt           ?
!     ipold          ?
!     ipvec1         vector length m
!     ipvec2         vector length m
!     ipvec3         vector length m
!     ipvel          velocity vector length m
!     ipw            indicates the Legendre Gauss-Lobatto weights for
!                    numerical integration in the x-direction corresponding
!                    to the reference element, length mone
!     ipwork         work array, length m
!     ipwork1        work array, length m
!     ipwork2        work array, length m
!     ipx            indicates the coordinates of the corresponding points
!                    in the element, length m x 2
!     ipxre          indicates the Legendre Gauss-Lobatto points in the
!                    x-direction corresponding to the reference element [-1,1], &
!                    length mone
!     isub           integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     rdum           dummy variable
!                    (more values not yet done)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0612         ?
!     EL3007         Fills old solution and derivatives
!     EL6007         compute the spectral element mass matrix (diagonal)
!     EL6008         compute the spectral element explicit non-linear term
!                    (Newton)
!     EL6027         compute the spectral element mass matrix (diagonal)
!     EL6028         ?
!     EL6046         ?
!     EL6051         compute the strong gradient of a scalar in the case of
!                    spectral elements
!     EL6055         Compute strong gradients in case of spectral elements 3d
!     EL6074         Compute the weak gradient of a scalar function (pressure)
!                    for right-hand-side/residual of the time-dependent
!                    Navier-Stokes equations; 2-D case.
!     EL6099         compute the residual of the first derivative
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELP640         compute basis functions, derivatives and weights for
!                    numerical integration corresponding to the reference
!                    element (element independent)
!     ELP642         compute the integration points, determinant and
!                    partial derivatives, two-dimensional case
!                    (element dependent)
!     ELP646         compute transformation determinant and
!                    partial derivatives  in 3-D
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INSTOP         Stop the program
! **********************************************************************
!
!                       I/O
!
!     USER INPUT
!     the user provides the coefficients through the arrays E%iuser and
!     USER in the usual way, the sequence is:
!     1:  eta
!     2:  reserved
!     3:  reserved
!     4:  reserved
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1781   The possibility ifunc>10000 not yet available
! **********************************************************************
!
!                       PSEUDO CODE
!
!     differential equation:
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen('elm612')

      if (ifirst==0) then

!     --- ifirst = 0, compute element independent quantities
!         perform initialisations, set iparry

         iparry = 1
         jcheld = 0
         call el0612(E%iuser, E%user, isub, isup, swork(iparry), &
                     ichois, jcheld)

!        --- set pointers in the workarray

         ipxre   = iparry  + 4*m
         ipx     = ipxre   + mone
         ipw     = ipx     + m*ndimlc
         ipd1    = ipw     + mone
         ipdet   = ipd1    + m
         ip      = ipdet   + m
         ipwork  = ip      + ndimlc*ndimlc*m
         ipwork1 = ipwork  + m*ncomp
         ipwork2 = ipwork1 + m*ncomp
         ipmt    = ipwork2 + m*ncomp
         ipvec1  = ipmt    + ncomp*ncomp*m*m
         ipvec2  = ipvec1  + m
         ipvec3  = ipvec2  + m
         ipvel   = ipvec3  + m
         ipgrd   = ipvel   + m*ncomp
         ipold   = ipgrd   + ndimlc*ndimlc*m

!        --- fill array ich (common celiar) for preceeding solutions

         call elind0( E%islold, numold, isub, isup, E%iuser)

!        --- compute points and weights for numerical integration
!            compute derivatives of the basisfunctions
!            corresponding to the reference element

         call elp640(1, swork(ipxre), swork(ipw), swork(ipd1), &
                     swork(ipwork1), mone)
      end if
      if (ierror/=0) go to 1000

!     --- compute integration points, determinant and derivatives

      if ( ndimlc==2 ) then
         call elp642(coor, index1, swork(ipx), swork(ipd1), &
                  swork(ipdet), swork(ip), swork(ip+m), swork(ip+2*m), &
                  swork(ip+3*m), mone, m)
      else if ( ndimlc==3 ) then
         call elp646(coor,index1,swork(ipx),swork(ipx+m),swork(ipx+2*m), &
                     swork(ipd1), swork(ipdet), swork(ip), &
                     swork(ip+m),  swork(ip+2*m),   swork(ip+3*m), &
                     swork(ip+4*m),  swork(ip+5*m),   swork(ip+6*m), &
                     swork(ip+7*m),  swork(ip+8*m),   mone)
      else
         write (*,*) 'Error in elm612 !!'
         call instop
      end if

!     --- first fill old solution if necessary

      if (ind(lengthmst)==1 .or.numold>=1) then
         call el3007 ( swork(ipold), rdum, rdum, rdum, m, m, &
                       ndimlc, ncomp, index1, rdum, &
                       swork(ipold), jtrans, numold, 1 )
      end if
      do 100 i = isub, isup
         if ((ind(i)>0) .or. (ist(i)>0)) then

!        --- coefficient is variable

            if ( ind(i)>10000 ) then
               call errint ( i, 1 )
               call errsub ( 1781, 1, 0, 0 )
            end if
            call elflr71(i, E%user, swork(iparry), &
                        swork(ipx), swork(ipx), m, m, rdum, &
                         1, index1, E%iuser, swork(ipold), swork )
         end if
 100  continue
      if (matrix) then
       if (ndimlc==2) then

!      --- matrix = true, compute spectral element matrix
!          Calculate u_x, u_y, v_x, v_y
!          perform call for u_x

         call el6051( swork(ipgrd),   swork(ipwork1),swork(ipold), &
                      swork(ipd1),    swork(ip),     swork(ip+m), &
                      swork(ip+2*m),  swork(ip+3*m), swork(ipdet), &
                      swork(ipwork),  mone, 1)

!        --- perform call for u_y

         call el6051( swork(ipgrd+m), swork(ipwork1),swork(ipold), &
                      swork(ipd1),    swork(ip),     swork(ip+m), &
                      swork(ip+2*m),  swork(ip+3*m), swork(ipdet), &
                      swork(ipwork),  mone, 2)

!        --- perform call for v_x

         call el6051( swork(ipgrd+2*m), swork(ipwork1), swork(ipold+m), &
                      swork(ipd1),    swork(ip),   swork(ip+m), &
                      swork(ip+2*m),  swork(ip+3*m),   swork(ipdet), &
                      swork(ipwork),  mone, 1)

!        --- perform call for v_y

         call el6051( swork(ipgrd+3*m), swork(ipwork1), swork(ipold+m), &
                      swork(ipd1),    swork(ip),  swork(ip+m), &
                      swork(ip+2*m),  swork(ip+3*m),   swork(ipdet), &
                      swork(ipwork),  mone, 2)

!        --- calculate stress matrices

         call el6008(swork(ipmt), &
                 swork(ipold),  swork(ipold+m), swork(ipgrd), &
                 swork(ipgrd+m),swork(ipgrd+2*m),swork(ipgrd+3*m), &
                 swork(ipw),    swork(ipd1),   swork(ipdet), &
                 swork(ip),    swork(ip+m),    swork(ip+2*m), &
                 swork(ip+3*m),    swork(ipwork1), swork(ipwork2), &
                 E%user, E%iuser,    mone,   m, ichois)

!        --- compose full matrix

        call el6046(E%mat,swork(ipmt), m)
       else if (ndimlc==3) then

!      --- Calculate u_x, u_y, u_z
!                    v_x, v_y, v_z
!                    w_x, w_y, w_z
!          perform call for u_x

            call el6055 ( swork(ipgrd),  swork(ipvec3),  swork(ipold), &
                          swork(ipd1),   swork(ipdet), &
                          swork(ip),     swork(ip+m), &
                          swork(ip+2*m), swork(ip+3*m), &
                          swork(ip+4*m), swork(ip+5*m), &
                          swork(ip+6*m), swork(ip+7*m), &
                          swork(ip+8*m), mone, 1 )

!           --- perform call for u_y

            call el6055 ( swork(ipgrd+1*m), swork(ipvec3), swork(ipold), &
                          swork(ipd1),     swork(ipdet), &
                          swork(ip),       swork(ip+m), &
                          swork(ip+2*m),   swork(ip+3*m), &
                          swork(ip+4*m),   swork(ip+5*m), &
                          swork(ip+6*m),   swork(ip+7*m), &
                          swork(ip+8*m),   mone, 2 )

!           --- perform call for u_z

            call el6055 ( swork(ipgrd+2*m), swork(ipvec3), swork(ipold), &
                          swork(ipd1),    swork(ipdet), &
                          swork(ip),      swork(ip+m), &
                          swork(ip+2*m),  swork(ip+3*m), &
                          swork(ip+4*m),  swork(ip+5*m), &
                          swork(ip+6*m),  swork(ip+7*m), &
                          swork(ip+8*m),  mone, 3 )

!           --- perform call for v_x

            call el6055 (swork(ipgrd+3*m),swork(ipvec3),swork(ipold+m), &
                          swork(ipd1),    swork(ipdet), &
                          swork(ip),      swork(ip+m), &
                          swork(ip+2*m),  swork(ip+3*m), &
                          swork(ip+4*m),  swork(ip+5*m), &
                          swork(ip+6*ip+6*m), swork(ip+7*m), &
                          swork(ip+8*m),   mone, 1 )

!           --- perform call for v_y

            call el6055 ( swork(ipgrd+4*m),swork(ipvec3),swork(ipold+m), &
                          swork(ipd1),    swork(ipdet), &
                          swork(ip),      swork(ip+m), &
                          swork(ip+2*m),  swork(ip+3*m), &
                          swork(ip+4*m),  swork(ip+5*m), &
                          swork(ip+6*m),  swork(ip+7*m), &
                          swork(ip+8*m),  mone, 2 )

!           --- perform call for v_z

            call el6055(swork(ipgrd+5*m),swork(ipvec3),swork(ipold+m), &
                          swork(ipd1),   swork(ipdet), &
                          swork(ip),     swork(ip+m), &
                          swork(ip+2*m), swork(ip+3*m), &
                          swork(ip+4*m), swork(ip+5*m), &
                          swork(ip+6*m), swork(ip+7*m), &
                          swork(ip+8*m), mone, 3 )

!           --- perform call for w_x

            call el6055(swork(ipgrd+6*m),swork(ipvec3),swork(ipold+2*m), &
                          swork(ipd1),   swork(ipdet), &
                          swork(ip),     swork(ip+m), &
                          swork(ip+2*m), swork(ip+3*m), &
                          swork(ip+4*m), swork(ip+5*m), &
                          swork(ip+6*m), swork(ip+7*m), &
                          swork(ip+8*m), mone, 1 )

!           --- perform call for w_y

            call el6055(swork(ipgrd+7*m),swork(ipvec3),swork(ipold+2*m), &
                          swork(ipd1),   swork(ipdet), &
                          swork(ip),     swork(ip+m), &
                          swork(ip+2*m), swork(ip+3*m), &
                          swork(ip+4*m), swork(ip+5*m), &
                          swork(ip+6*m), swork(ip+7*m), &
                          swork(ip+8*m), mone, 2 )

!           --- perform call for w_z

            call el6055(swork(ipgrd+8*m),swork(ipvec3),swork(ipold+2*m), &
                          swork(ipd1),   swork(ipdet), &
                          swork(ip),     swork(ip+m), &
                          swork(ip+2*m), swork(ip+3*m), &
                          swork(ip+4*m), swork(ip+5*m), &
                          swork(ip+6*m), swork(ip+7*m), &
                          swork(ip+8*m), mone, 3 )
        end if
      end if

!     --- fill spectral element mass matrix if necessary

      if (notmas==0) then

!     --- notmas = 0, compute diagonal mass matrix

         call el6027(swork(ipvec1), swork(ipw), swork(ipdet), &
                        mone, m)

!        --- compose full mass matrix, from three components

         call el6028(E%mmat, swork(ipvec1), swork(ipvec1), &
                     swork(ipvec1), m)
      end if

!     --- fill spectral element vector if necessary

      if (vector) then
        if (ndimlc==2) then

!       --- first component  : 2 eta dudx (weak)
!           --- second component : eta ( dudy  + dvdx) (weak)
!           --- third component  : 2 eta dudx (weak)
!           --- Note : swork(ipgrd**) is here used to store
!                  the gradients weak !!
!           --- compute dudx (weak)

         jadd = 0
         call el6074( swork(ipgrd), swork(ipold),   swork(ipw), &
                      swork(ipd1),  swork(ip),     swork(ip+m), &
                      swork(ip+2*m),swork(ip+3*m), &
                      swork(ipwork2), mone, m,      1)

!        --- compute dudy (weak)

         jadd = 0
         call el6074( swork(ipgrd+m), swork(ipold),   swork(ipw), &
                      swork(ipd1),    swork(ip),     swork(ip+m), &
                      swork(ip+2*m),  swork(ip+3*m), &
                      swork(ipwork2), mone, m,      2)

!        --- compute dvdx (weak)

         jadd = 0
         call el6074( swork(ipgrd+2*m), swork(ipold+m), swork(ipw), &
                      swork(ipd1),    swork(ip),     swork(ip+m), &
                      swork(ip+2*m),  swork(ip+3*m), &
                      swork(ipwork2), mone, m,      1)

!        --- compute dvdy (weak)

         jadd = 0
         call el6074( swork(ipgrd+3*m), swork(ipold+m), swork(ipw), &
                      swork(ipd1),   swork(ip),     swork(ip+m), &
                      swork(ip+2*m),  swork(ip+3*m), &
                      swork(ipwork2), mone, m,      2)

!        --- compose full rhs vector, from four gradients
!            and the viscosity

         call el6007( E%vec, swork(ipgrd), swork(ipgrd+m), &
                      swork(ipgrd+2*m), swork(ipgrd+3*m), &
                      swork(iparry)   , m)
      else if (ndimlc==3) then

!     --- Note that now ipgrd is used to store weak gradients
!         perform call for u_x

            call el6099 ( swork(ipgrd), swork(ipold),  swork(ipw), &
                          swork(ipd1),  swork(ip),     swork(ip+m), &
                          swork(ip+2*m),swork(ip+3*m), swork(ip+4*m), &
                          swork(ip+5*m),swork(ip+6*m), swork(ip+7*m), &
                          swork(ip+8*m),mone, 1 )

!           --- perform call for u_y

            call el6099 ( swork(ipgrd+1*m),swork(ipold),swork(ipw), &
                          swork(ipd1),   swork(ip),     swork(ip+m), &
                          swork(ip+2*m), swork(ip+3*m), swork(ip+4*m), &
                          swork(ip+5*m), swork(ip+6*m), swork(ip+7*m), &
                          swork(ip+8*m), mone, 2 )

!           --- perform call for u_z

            call el6099 ( swork(ipgrd+2*m),swork(ipold),  swork(ipw), &
                          swork(ipd1),     swork(ip),     swork(ip+m), &
                          swork(ip+2*m),   swork(ip+3*m), swork(ip+4*m), &
                          swork(ip+5*m),   swork(ip+6*m), swork(ip+7*m), &
                          swork(ip+8*m),   mone, 3 )

!           --- perform call for v_x

            call el6099 ( swork(ipgrd+3*m),swork(ipold+1*m),swork(ipw), &
                          swork(ipd1),     swork(ip),     swork(ip+m), &
                          swork(ip+2*m),   swork(ip+3*m), swork(ip+4*m), &
                          swork(ip+5*m),   swork(ip+6*m), swork(ip+7*m), &
                          swork(ip+8*m),   mone, 1 )

!           --- perform call for v_y

            call el6099 ( swork(ipgrd+4*m),swork(ipold+1*m),swork(ipw), &
                          swork(ipd1),     swork(ip),     swork(ip+m), &
                          swork(ip+2*m),   swork(ip+3*m), swork(ip+4*m), &
                          swork(ip+5*m),   swork(ip+6*m), swork(ip+7*m), &
                          swork(ip+8*m),   mone, 2 )

!           --- perform call for v_z

            call el6099 ( swork(ipgrd+5*m),swork(ipold+1*m),swork(ipw), &
                          swork(ipd1),     swork(ip),     swork(ip+m), &
                          swork(ip+2*m),   swork(ip+3*m), swork(ip+4*m), &
                          swork(ip+5*m),   swork(ip+6*m), swork(ip+7*m), &
                          swork(ip+8*m),   mone, 3 )

!           --- perform call for w_x

            call el6099 ( swork(ipgrd+6*m),swork(ipold+2*m),swork(ipw), &
                          swork(ipd1),     swork(ip),     swork(ip+m), &
                          swork(ip+2*m),   swork(ip+3*m), swork(ip+4*m), &
                          swork(ip+5*m),   swork(ip+6*m), swork(ip+7*m), &
                          swork(ip+8*m),   mone, 1 )

!           --- perform call for w_y

            call el6099 ( swork(ipgrd+7*m),swork(ipold+2*m),swork(ipw), &
                          swork(ipd1),     swork(ip),     swork(ip+m), &
                          swork(ip+2*m),   swork(ip+3*m), swork(ip+4*m), &
                          swork(ip+5*m),   swork(ip+6*m), swork(ip+7*m), &
                          swork(ip+8*m),   mone, 2 )

!           --- perform call for w_z

            call el6099 ( swork(ipgrd+8*m),swork(ipold+2*m),swork(ipw), &
                          swork(ipd1),     swork(ip),     swork(ip+m), &
                          swork(ip+2*m),   swork(ip+3*m), swork(ip+4*m), &
                          swork(ip+5*m),   swork(ip+6*m), swork(ip+7*m), &
                          swork(ip+8*m),   mone, 3 )

!           --- compose full rhs vector, from nine gradients
!               and the viscosity
!               call el6006a( E%vec, swork(ipgrd), swork(ipgrd+m), &
!               +                   swork(ipgrd+2*m), swork(ipgrd+3*m), &
!               +                   swork(ipgrd+4*m), swork(ipgrd+5*m), &
!               +                   swork(ipgrd+6*m), swork(ipgrd+7*m), &
!               +                   swork(ipgrd+8*m), swork(iparry), m, 1)

       end if
      end if
1000  if (ierror/=0) call instop

      call erclos('elm612')

      end
