      subroutine elm654 ( upper, B )
!
! ********************************************************************
!
!
!        programmer    Arno van Duin
!        version 1.1   date   91-03-19 adjusted for NDIM = 3
!        version 1.0   date   90-11-01
!
!
!   copyright (c) 1990  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
!
! ********************************************************************
!
!   the vector B is filled with -1s
!
! ********************************************************************
!
!
!                INPUT / OUTPUT PARAMETERS

      implicit none

      integer upper
      double precision B(*)

!
!     upper   i     length of B
!
!     (see E.F. Kaasschieter: Preconditioned conjugate gradients and mixed- &
!                             hybrid finite elements for the solution of
!                             potential flow problems)
!
!     B       o   real vector: B = - / nabla . v dx = -1
!                               i    S          i
! *********************************************************************
!
!               LOCAL PARAMETERS

      integer i

!
!     i           loop variable
!
! ********************************************************************
!
!                              METHOD
!
!
!       Differential equation:
!
!                    u = - A grad phi
!
!                    div u = f
!
!
!       leads to:
!                     -1
!              U   = A  ( F  - B Phi - C Labda )
!                          1
!
!                       T -1   -1   T -1
!              Phi = ( B A  B )  ( B A  ( F  - C Labda ) - F )
!                                          1                2
!
!              D Labda = F
!
!       the vector B is computed
!
! ********************************************************************
!
!               SUBROUTINES CALLED
!
!       none
!
! ********************************************************************

      call eropen ( 'elm654' )

      do 10 i = 1, upper
         B(i) = -1d0
10    continue

      call erclos ( 'elm654' )

      end
