      subroutine elm656 ( icount, Amin1, Amin1B, BTAmin, res)
!
! ********************************************************************
!
!
!        programmer    Arno van Duin
!        version 1.1   date   91-03-20 adjusted for NDIM = 3
!        version 1.0   date   90-11-01
!
!
!   copyright (c) 1990  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
!
! ********************************************************************
!
!   the element matrix is filled
!
! ********************************************************************
!
!
!                INPUT / OUTPUT PARAMETERS

      implicit none

      integer icount
      double precision Amin1(*), Amin1B(*), BTAmin, res(*)

!
!     icount   i     number of unknowns per element
!
!     (see E.F. Kaasschieter: Preconditioned conjugate gradients and mixed- &
!                             hybrid finite elements for the solution of
!                             potential flow problems)
!
!                                         -1
!     Amin1    i  real matrix containing A
!
!                                         -1
!     Amin1B   i  real vector containing A  B
!
!                                T -1
!     BTAmin i  real equal to B A  B
!
!                                       -1   -1     T -1   -1 T -1
!     res      o  real matrix containg A  - A  B ( B A  B )  B A
!
! *********************************************************************
!
!               LOCAL PARAMETERS

      integer i, j

!
!     i, j        loop variable
!
! ********************************************************************
!
!                              METHOD
!
!
!       Differential equation:
!
!                    u = - A grad phi
!
!                    div u = f
!
!
!       leads to:
!                     -1
!              U   = A  ( F  - B Phi - C Labda )
!                          1
!
!                       T -1   -1   T -1
!              Phi = ( B A  B )  ( B A  ( F  - C Labda ) - F )
!                                          1                2
!
!              D Labda = F
!
!       the element matrix for the last equation is computed
!
! ********************************************************************
!
!               SUBROUTINES CALLED
!
!       none
!
! ********************************************************************

      call eropen ( 'elm656' )

      do 20 i = 1, icount
         do 10 j = 1, icount
           res((i-1)*icount+j) = Amin1((i-1)*icount+j) - &
                                   Amin1B(i) * Amin1B(j) / BTAmin
10       continue
20    continue

      call erclos ( 'elm656' )

      end
