      subroutine elm658 ( icount, Amin1B, btamin, F2, elemvc)
!
! ********************************************************************
!
!
!        programmer    Arno van Duin
!        version 1.2   date   03-07-1992 replace btamin1b by btamin
!        version 1.1   date   20-03-1991 adjusted for NDIM = 3
!        version 1.0.  date   01-11-1990
!
!
!   copyright (c) 1990,1992  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
!
! ********************************************************************
!
!   the element vector is filled
!
! ********************************************************************
!
!
!                INPUT / OUTPUT PARAMETERS

      implicit none

      integer icount
      double precision Amin1B(*), btamin, F2(*), elemvc(*)

!
!     icount   i     number of unknowns per element
!
!     (see E.F. Kaasschieter: Preconditioned conjugate gradients and mixed- &
!                             hybrid finite elements for the solution of
!                             potential flow problems)
!
!                                         -1
!     Amin1B   i  real vector containing A  B
!
!                                T -1
!     btamin   i  real equal to B A  B
!
!     F2       i  real vector: F  = - / f dx = f * opp(S)
!                               2     S
!
!     elemvc   o  real vector containing the element vector
!                   -1   T -1  -1
!                  A  B(B A  B)  F - elemvc
!                                 2        in
!
! *********************************************************************
!
!               LOCAL PARAMETERS

      integer i

!
!     i           loop variable
!
! ********************************************************************
!
!                              METHOD
!
!
!       Differential equation:
!
!                    u = - A grad phi
!
!                    div u = f
!
!
!       leads to:
!                     -1
!              U   = A  ( F  - B Phi - C Labda )
!                          1
!
!                       T -1   -1   T -1
!              Phi = ( B A  B )  ( B A  ( F  - C Labda ) - F )
!                                          1                2
!
!              D Labda = F
!
!       the element vector for the last equation is computed
!
! ********************************************************************
!
!               SUBROUTINES CALLED
!
!       none
!
! ********************************************************************

      call eropen ( 'elm658' )

      do 10 i = 1, icount
         elemvc(i) = Amin1B(i) / btamin * F2(i)
10    continue

      call erclos ( 'elm658' )

      end
