      subroutine elm663 ( i, k, a, b, c, d, ndim )
!
! ********************************************************************
!
!
!        programmer    Arno van Duin
!        version 1.1   date   91-03-21 adjusted for NDIM = 3
!        version 1.0   date   90-11-01
!
!
!   copyright (c) 1990  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
!
! ********************************************************************
!
!   the basis function is taken to be of the form: (v ) = a x + b y + cz + d
!                                                    i j
!   a, b, c, and d are returned depending on i and j
!
! ********************************************************************
!
!
!                INPUT / OUTPUT PARAMETERS

      implicit none

      integer i, k, ndim
      double precision a, b, c, d

!
!     ndim     i     the dimension of space
!
!     (see E.F. Kaasschieter: Preconditioned conjugate gradients and mixed- &
!                             hybrid finite elements for the solution of
!                             potential flow problems)
!
!     i        i     number of basisfunction
!                                                              th
!     k        i     number of element of basis function, the k   element of
!                         th
!                    the i   basis function is taken
!
!     a        i     coefficient of x
!
!     b        i     coefficient of y
!
!     c        i     coefficient of z
!
!     d        i     last coefficient
!
! *********************************************************************
!
!               LOCAL PARAMETERS
!
!     none
!
! ********************************************************************
!
!                              METHOD
!     ndim = 2
!
!        ^    ( x )     ^    ( x - 1 )     ^    (   x   )
!        v  = ( y )     v  = (   y   )     v  = ( y - 1 )
!         1              2                  3
!
!     ndim = 3
!
!        ^      ( x )   ^      ( x - 1 )   ^      (   x   )  ^      (   x   )
!        v  = 2 ( y )   v  = 2 (   y   )   v  = 2 ( y - 1 )  v  = 2 (   y   )
!         1     ( z )    2     (   z   )    3     (   z   )   4     ( z - 1 )
!
!     (see E.F. Kaasschieter: Preconditioned conjugate gradients and mixed- &
!                             hybrid finite elements for the solution of
!                             potential flow problems)
!
! ********************************************************************
!
!               SUBROUTINES CALLED
!
!       none
!
! ********************************************************************

      call eropen ( 'elm663' )

      if ( ndim.eq.2 ) then
         c = 0d0
         if ( i.eq.1) then
            if (k.eq.1) then
               a = 1d0
               b = 0d0
               d = 0d0
            else
               a = 0d0
               b = 1d0
               d = 0d0
            end if
         else if ( i.eq.2) then
            if (k.eq.1) then
               a = 1d0
               b = 0d0
               d = -1d0
            else
               a = 0d0
               b = 1d0
               d = 0d0
            end if
         else if ( i.eq.3) then
            if (k.eq.1) then
               a = 1d0
               b = 0d0
               d = 0d0
            else
               a = 0d0
               b = 1d0
               d = -1d0
            end if
         end if
      else
         if ( i.eq.1 ) then
            if ( k.eq.1 ) then
               a = 2d0
               b = 0d0
               c = 0d0
               d = 0d0
            else if ( k.eq.2 ) then
               a = 0d0
               b = 2d0
               c = 0d0
               d = 0d0
            else
               a = 0d0
               b = 0d0
               c = 2d0
               d = 0d0
            end if
         else if ( i.eq.2 ) then
            if ( k.eq.1 ) then
               a = 2d0
               b = 0d0
               c = 0d0
               d = -2d0
            else if ( k.eq.2 ) then
               a = 0d0
               b = 2d0
               c = 0d0
               d = 0d0
            else
               a = 0d0
               b = 0d0
               c = 2d0
               d = 0d0
            end if
         else if ( i.eq.3 ) then
            if ( k.eq.1 ) then
               a = 2d0
               b = 0d0
               c = 0d0
               d = 0d0
            else if ( k.eq.2 ) then
               a = 0d0
               b = 2d0
               c = 0d0
               d = -2d0
            else
               a = 0d0
               b = 0d0
               c = 2d0
               d = 0d0
            end if
         else if ( i.eq.4 ) then
            if ( k.eq.1 ) then
               a = 2d0
               b = 0d0
               c = 0d0
               d = 0d0
            else if ( k.eq.2 ) then
               a = 0d0
               b = 2d0
               c = 0d0
               d = 0d0
            else
               a = 0d0
               b = 0d0
               c = 2d0
               d = -2d0
            end if
         end if
      end if

      call erclos ( 'elm663' )

      end
