      subroutine elm665
!
! ********************************************************************
!
!
!        programmer    Arno van Duin
!        version 1.0.  date   91-04-22
!
!
!   copyright (c) 1990  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
!
!
!
! ********************************************************************
!
!     Fill element matrix and vector for natural boundaries for
!     potential flow equations using mixed hybrid FEMs
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      use sepmodulecactl
      implicit none
! ********************************************************************
!
!
!                INPUT / OUTPUT PARAMETERS
!
! ********************************************************************
!
!               LOCAL PARAMETERS
!
      double precision dx, dx1, dx2, dy, dy1, dy2, dz, dz1, dz2, gN, &
       xc, yc, zc
      integer istart
!
!     dx      length of vector in x-direction
!
!     dx1     x-coordinate of first spanning vector of triangle
!
!     dx2     x-coordinate of second spanning vector of triangle
!
!     dy      length of vector in y-direction
!
!     dy1     y-coordinate of first spanning vector of triangle
!
!     dy2     y-coordinate of second spanning vector of triangle
!
!     dz      length of vector in z-direction
!
!     dz1     z-coordinate of first spanning vector of triangle
!
!     dz2     z-coordinate of second spanning vector of triangle
!
!     gN      the value of the Neumann condition in this element
!
!     istart  see SEPRAN Standard Problems 2.1.1
!
!     xc      x-coordinate of the midth-point
!
!     yc      y-coordinate of the midth-point
!
!     zc      z-coordinate of the midth-point
!
! ********************************************************************
!
!                              METHOD
!
!     The returned element vector equals the Neumann value times the size
!     of the element.
!
!     PSEUDO CODE
!
!     If matrix then
!        E%mat(0) = 0d0
!     Endif
!     If vector then
!        If ndim = 3 then
!                        || ( x2 - x1 )   ( x3 - x1 ) ||
!           Size := .5 * || ( y2 - y1 ) X ( y3 - y1 ) ||
!                        || ( z2 - z1 )   ( z3 - z1 ) ||
!                                                       2
!        Else
!                   || ( x2 - x1 ) ||
!           Size := || ( y2 - y1 ) ||
!                                    2
!        Endif
!        E%vec(1) = -size * g
!                            N
!     Endif
!
! ********************************************************************
!
!                  USER INPUT
!
!     The user provides the coefficients through the arrays USER and E%iuser
!     The coefficients are stored in the following way:
!
!     1) sigma(x)         ( = 0d0  h.l. )
!     2) gN(x)
!
! ********************************************************************
!
!               SUBROUTINES CALLED
!
      double precision funccf
!
!     FUNCCF  returns the value of a coefficient
!
! ********************************************************************
!
      call eropen ( 'elm665' )


      istart = E%iuser(ielgrp + 5) + 1
      if ( E%iuser(istart).eq.0 ) then
         gN = 0d0
      else if ( E%iuser(istart).lt.-5 ) then
         gN = E%user(-E%iuser(istart))
      else
         if ( inpelm.eq.7 ) then
            xc = coor(1,index1(7))
            yc = coor(2,index1(7))
            zc = coor(3,index1(7))
         else
            xc = coor(1,index1(2))
            yc = coor(2,index1(2))
            zc = 0d0
         end if
         gN = funccf ( E%iuser(istart), xc, yc, zc )
      end if

      if ( matrix ) then
         E%mat(1,1) = 0d0
      end if

      if ( vector ) then

         if ( inpelm.eq.7 ) then

!           NDIM = 3

!           calculate the first spanning vector

            dx1 = coor(1,index1(1)) - coor(1,index1(3))
            dy1 = coor(2,index1(1)) - coor(2,index1(3))
            dz1 = coor(3,index1(1)) - coor(3,index1(3))

!           calculate the second spanning vector

            dx2 = coor(1,index1(1)) - coor(1,index1(5))
            dy2 = coor(2,index1(1)) - coor(2,index1(5))
            dz2 = coor(3,index1(1)) - coor(3,index1(5))

!           calculate the outer product between the two spanning vectors
!           and divide by two

            dx  = (dy1*dz2-dz1*dy2)/2
            dy  = (dz1*dx2-dx1*dz2)/2
            dz  = (dx1*dy2-dy1*dx2)/2

         else

!           NDIM = 2

!           calculate the difference vector

            dx = coor(1,index1(1)) - coor(1,index1(3))
            dy = coor(2,index1(1)) - coor(2,index1(3))
            dz = 0d0

         end if

!        fill E%vec

         E%vec(1) = -sqrt(dx*dx+dy*dy+dz*dz) * gN

      end if

      call erclos ( 'elm665' )

      end
