      subroutine elm800reset ( alpha, u, uconst, f, fconst, ah )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 02-10-2002
!
!   copyright (c) 2002-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Reset the diffusion tensor, velcoity vector and right-hand side
!     to their original values
!     They have been transformed in elm800
! **********************************************************************
!
!                       KEYWORDS
!
!     elliptic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision alpha(m,*), u(m,*), uconst(*), f(m), fconst, &
                       ah(3,3)

!     ah             i    Contains the original diffusion tensor
!     alpha          o    array of length m x 3 in which the values of
!                         alpha  , alpha  , alpha
!                              11       12       22
!                         are stored in the sequence:
!                                                 k
!                         alpha ( k,i ) = alpha (x )   i = 1(1)3, k=1(1)m
!                                              i
!     f              o    Contains the right-hand side
!     fconst         i    Contains the original right-hand side
!     u              o    array of length mxndim containing the values of the
!                         velcoty in the integration points
!     uconst         i    Array in which the original value of u are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer k, i

!     i              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm800reset' )

!     --- constant velocities, only computed once so put back in work

      do i = 1, 3
         if ( ind(i+5).le.0 ) then
            do k = kstep, m, kstep
               u(k,i) = uconst(i)
            end do
         end if
      end do

!     --- constant force, only computed once so put back

      if (ind(11).le.0) then
         do k = kstep, m, kstep
            f(k) = fconst
         end do
      end if

!     --- put back the diffusivity matrix

      if ( ndim.eq.2 ) then

         if ( ind(1).le.0 )  then
            do k = kstep, m, kstep
               alpha(k,1) = ah(1,1)
            end do
         end if
         if ( ind(2).le.0 )  then
            do k = kstep, m, kstep
               alpha(k,2) = ah(2,1)
            end do
         end if
         if ( ind(3).le.0 )  then
            do k = kstep, m, kstep
               alpha(k,3) = ah(2,2)
            end do
         end if

      else if ( ndim.eq.3 ) then

         if ( ind(1).le.0 )  then
            do k = kstep, m, kstep
               alpha(k,1) = ah(1,1)
            end do
         end if
         if ( ind(2).le.0 )  then
            do k = kstep, m, kstep
               alpha(k,2) = ah(2,1)
            end do
         end if
         if ( ind(3).le.0 )  then
            do k = kstep, m, kstep
               alpha(k,3) = ah(3,1)
            end do
         end if
         if ( ind(4).le.0 )  then
            do k = kstep, m, kstep
               alpha(k,4) = ah(2,2)
            end do
         end if
         if ( ind(5).le.0 )  then
            do k = kstep, m, kstep
               alpha(k,5) = ah(3,2)
            end do
         end if
         if ( ind(6).le.0 )  then
            do k = kstep, m, kstep
               alpha(k,6) = ah(3,3)
            end do
         end if
      end if

      call erclos ( 'elm800reset' )
      end
