      subroutine elm805rhsd ( wrk1, frhsd1, frhsd2, elemvc, &
                              phi, w, icounthalf )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-05-2006
!
!   copyright (c) 2006-2006  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element vector for elements of type 805, i.e.
!     general second order elliptic equation with two degreed of freedom
!     per point
!     Upwind is not implemented and also no special non-linear convection
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     elliptic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer icounthalf
      double precision wrk1(*), frhsd1(*), frhsd2(*), &
                       elemvc(*), phi(*), w(*)

!     elemvc         o    Element vector
!     frhsd1         i    Contains right-hand side vector for first eq
!     frhsd2         i    Contains right-hand side vector for second eq
!     icounthalf     i    icount/2, i.e. size of submatrix
!     phi            i    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     w              i    array of length m containing the weights for
!                         integration
!     wrk1                work space
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision elemvc1(icounthalf), elemvc2(icounthalf)

!     elemvc1        element vector for first eq.
!     elemvc2        element vector for second eq.
!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL3003         Compute integrals of the shape / f(x) phi (x) dx
!                                                   e         i
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm805rhsd' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm805rhsd'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      jadd = 0

!     --- Fill element vector for first component

      call el3003 ( frhsd1, elemvc1, w, phi, m, n )

!     --- Fill element vector for second component

      call el3003 ( frhsd2, elemvc2, w, phi, m, n )

!     --- Combine both element vectors into one large vector

      do i = 1, icounthalf
         elemvc(2*i-1) = elemvc1(i)
         elemvc(2*i  ) = elemvc2(i)
      end do  ! i = 1, icounthalf

1000  call erclos ( 'elm805rhsd' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elm805rhsd'

      end if  ! ( debug )

      end
