      subroutine elmdiffusion ( jelp, kappa, elemmt, w, dphidx )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-06-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the element diffusion matrix in case there is one
!     diffusion coefficient
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     diffusion
!     elliptic_equation
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jelp
      double precision elemmt(icount**2,*), w(*), dphidx(*), kappa(*)

!     dphidx         i    Array of length n x m x ndim  containing the
!                         derivatives
!                         of the basis functions in the sequence:
!                         d phi / dx (xg ) = dphidx (i,k,1);
!                              i        k
!                         d phi / dy (xg ) = dphidx (i,k,2);
!                              i        k
!                         d phi / dz (xg ) = dphidx (i,k,3);
!                              i        k
!                         If the element is a linear simplex, the
!                         derivatives are constant and only k = 1 is filled.
!     elemmt         o    Element matrix
!     jelp           i    Indication of the type of basis function subroutine
!                         must be called by ELxxxx.
!     kappa          i    Diffusion coefficient vector per integrtion point
!     w              i    array of length m containing the weights for
!                         integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision work(6*m)

!     work           Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL3004         Compute integrals of the shape
!                    /_e alpha grad phi_i . grad phi_j dx
!     EL3005         Compute contribution to element stiffness matrix
!                    corresponding to the second derivative part of a
!                    three-dimensional equation
!     EL3008         Compute contribution to element stiffness matrix
!                    corresponding to the second derivative part of a
!                    one-dimensional equation
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elmdiffusion' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elmdiffusion'

      end if
      if ( ierror/=0 ) go to 1000

      jsecnd = 2

      if ( jelp<=2 ) then

!     --- One-dimensional elements

         call el3008 ( kappa, elemmt, w, dphidx, n, n )

      else if ( jelp<=10 ) then

!     --- Two-dimensional elements

         call el3004 ( kappa, elemmt, w, dphidx, m, n, n )

      else

!     --- Three-dimensional elements

         call el3005 ( kappa, elemmt, w, dphidx, m, n, n, &
                       work )

      end if  ! ( jelp<=2 )

1000  call erclos ( 'elmdiffusion' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elmdiffusion'

      end if

      end
