      subroutine elp611first ( phi, wh, iweigo, jcartsav, phider, iqmat )
! ======================================================================
!
!        programmer    Guus Segal
!        version  4.0  date 06-01-2016 Use sepmoduleelem
!        version  3.5  date 04-09-2002 Correction dimension xloc
!        version  3.4  date 29-01-1999 common cinforel
!
!   copyright (c) 1992-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element independent quantities for a quadratic line element
!
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iweigo, jcartsav, iqmat
      double precision phi(3,m), wh(m), phider(3,m)

!     iqmat          o    if 1 the qmat matrix must be filled
!     iweigo         o    if 1 the weights must be filled
!     jcartsav       o    Help variable to store jcart temporarily
!     phi            o    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     phider         o    Contains the derivatives of the basis functions in the
!                         reference element
!                         phider(i,j) = d phi_i/ ds (x^j)
!     wh             o    Contains the weights in the reference element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision d, lam1(m), lambda(m)
      character (len=25) text

!     d              help parameter to store a constant
!     lam1           1-lambda (lin basis function corresponding to first node)
!     lambda         Array containing the linear basis functions in the
!                    integration points
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELPLAMBDA1D    Fill the one-dimensional linear basis functions in case of
!                    a non-Newton Cotes rule
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data text / 'Quadratic 1D line element' /
! ======================================================================
!
      call eropen ( 'elp611first' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp611first'

      end if  ! ( debug )

      if ( ierror/=0 ) go to 1000

      iweigo = mod(iweigh,10)
      iqmat  = iweigh/10

      jcartsav = jcart
      if ( jcart==2 .or. jcart==3 ) then

!     --- jcart = 2, 3  (Polar co-ordinates)

         jcartsav = 3

      else if ( jcart/=1 ) then

!     --- Error 1769: Co-ordinate system not available

         call errorelvar ( jcart-1, 1769, text, 0 )

      end if
      if ( irule<1 .or. irule>4 .or. irule==2 ) then

!     --- Error 1770: Integration rule not available

         call errorelvar ( irule, 1770, text, 1 )
         irule = 1

      end if
      if ( irule==1 .and. m/=3 .or. irule==2 .and. m/=1 .or. &
           irule==3 .and. m/=2 .or. irule==4 .and. m/=3 )then

!     --- Error 1424: m and integration rule do not match

         call errorelvar2 ( m, irule, 1424, text, 0 )

      end if

!     --- Fill array lambda and wh

      if ( irule==1 ) then

!     --- irule = 1  Newton Cotes rule

         lambda(1) = 0d0
         lambda(2) = 0.5d0
         lambda(3) = 1d0

         d = 1d0/6d0
         wh(1) = d
         wh(2) = 4d0*d
         wh(3) = d

      else

!     --- irule>2  one-point Gauss rule

         call elplambda1d ( lambda, wh, text )

      end if  ! ( irule==1 )

!     --- Fill array phi

      lam1(1:m) = 1d0-lambda(1:m)
      phi(1,1:m) = lam1(1:m)*(2d0*lam1(1:m)-1d0)
      phi(2,1:m) = 4d0*lambda(1:m)*lam1(1:m)
      phi(3,1:m) = lambda(1:m)*(2d0*lambda(1:m)-1d0)

      phider(1,1:m) = -(4d0*lam1(1:m)-1d0)
      phider(2,1:m) = 4d0*(lam1(1:m)-lambda(1:m))
      phider(3,1:m) = 4d0*lambda(1:m)-1d0

1000  call erclos ( 'elp611first' )

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp611first'

      end if  ! ( debug )

      end
