      subroutine elp620first ( iweigo, iqmat, jcartorg, jtransfo, &
                               ndimloc, phi, intrule, wh )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.0  date 12-06-2013
!
!   copyright (c) 2013-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element independent quantities for a linear triangle
!
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     linear
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iweigo, iqmat, jcartorg, jtransfo, ndimloc, intrule
      double precision phi(3,m), wh(m)

!     intrule        o    Actual integration rule
!     iqmat          o    if 1 the qmat matrix must be filled
!     iweigo         o    if 1 the weights must be filled
!     jcartorg       o    Parameter jcart_orig as defined in jcart
!     jtransfo       o    Parameter jtransform as defined in jcart
!     ndimloc        o    Local value of ndim
!                         (usually ndim, except is jtransfo>0: 2)
!     phi            o    array of length n * m, values of shape
!                         functions in integration points in the sequence:
!                         phi_i(xg^k) = phi (i,k)
!                         array phi is only filled when gauss integration is
!                         applied
!     wh             o    Contains the weights in the reference element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision h
      character (len=15) text

!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     text           String variable containing some text
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRORELVAR     Print error message errornr for element: wrong value of
!                    parameter
!     ERRORELVAR2    Print error message errornr for element: wrong value of
!                    2 parameters
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data text / 'Linear triangle' /
! ======================================================================
!
      call eropen ( 'elp620first' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elp620first'
         write(irefwr,1) 'n, m', n, m
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      iweigo = mod(iweigh,10)
      iqmat  = iweigh/10
      jcartorg = mod(jcart,10)
      jtransfo = jcart/10
      ndimloc = ndim
      if ( jtransfo>0 ) ndimloc = 2
      if ( jcartorg==2 ) then

!     --- jcartorg = 2

      else if ( jcartorg/=1 ) then

!     --- Error 1769: Co-ordinate system not available

         call errorelvar ( jcartorg-1, 1769, text, 0 )

      end if
      if ( intrule<1 .or. intrule>3 .and. intrule/=5 ) then

!     --- Error 1770: Integration rule not available

         call errorelvar ( intrule, 1770, text, 1 )
         intrule = 1

      end if
      if ( intrule==1 .and. m/=3 .or. &
           intrule==2 .and. m/=1 .or. &
           intrule==3 .and. m/=4 .or. &
           intrule==5 .and. m/=3 ) then

!     --- Error 1424: m and integration rule do not match

         call errorelvar2 ( m, intrule, 1424, text, 0 )

      end if

!     --- Fill array phi

      select case ( intrule )

         case(1)

!        --- intrule = 1  Newton Cotes rule

            phi = 0d0
            do i = 1, 3
               phi(i,i) = 1d0
            end do
            h = 1d0/6d0
            wh = h

         case(2)

!        --- intrule = 2  one-point Gauss rule

            phi(:,1) = alam1
            wh = wgs1

         case(3)

!        --- intrule = 3  four-point Gauss rule

            phi(:,1:4) = alam2(:,1:4)
            wh = wgs2

         case(5)

!        --- intrule = 5  second order Newton-Cotes rule

            phi(:,1:3) = 0.5d0
            phi(1,2) = 0d0
            phi(2,3) = 0d0
            phi(3,1) = 0d0
            h = 1d0/6d0
            wh = h

      end select  ! case ( intrule )

!     --- Set igausp to 1 in case of integration rules larger than 1

      if ( intrule>1 .and. iweigo==1 ) igausp = 1

1000  call erclos ( 'elp620first' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elp620first'

      end if  ! ( debug )

      end
