      subroutine eltemp2enthal ( elemvc, temperature, coefarr, m, &
                                 itypeeq )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 22-06-2005 Extra parameter itypeeq
!        version  1.1  date 13-10-2004 set elemvc = zero for itype # 810
!                                      (J.H. Brusche)
!        version  1.0  date 17-09-2004
!
!   copyright (c) 2004-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element vector with enthalpy values (isothermal with constant
!     physical parameters cs, cl)
! **********************************************************************
!
!                       KEYWORDS
!
!     enthalpy
!     temperature
!     element
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer m, itypeeq
      double precision elemvc(*), temperature(*), coefarr(m,*)

!     coefarr        i    Array containing the coefficients of the PDE
!                         Pos. (.,11)  rho
!                         Pos. (.,12)  capacity_s
!                         Pos. (.,13)  capacity_l
!                         Pos. (.,14)  latent_heat
!                         Pos. (.,15)  melt_temp
!     elemvc         o    Element vector, contains enthalpy in integration
!                         node
!     itypeeq        i    if 0 the standard enthalpy equation is solved
!                         if 1 the equation is restricted to a solid temperature
!                         equation
!     m              i    number of integration points
!     temperature    i    temperature vector in integration points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i
      double precision Tm, rho, cs, cl, L

!     cl             Specific heat of the liquid
!     cs             Specific heat of the solid
!     i              Counting variable
!     l              General loop variable
!     rho            density rho
!     tm             Melting temperature
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'eltemp2enthal' )
      debug = .false.

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from eltemp2enthal'

      end if
      if ( ierror/=0 ) go to 1000

!     --- get parameters

      rho = coefarr(1,12)
      cs  = coefarr(1,3)

      if (itypeeq==1) then

!     --- itype # 810, set vector to 0

         do i = 1, m
            elemvc(i) = rho*cs*temperature(i)
         end do

      else

!     --- get parameters

         cl  = coefarr(1,4)
         L   = coefarr(1,5)
         Tm  = coefarr(1,6)

         if ( debug) &
            write(irefwr,*) 'Tm, rho, cs, cl, L ', Tm, rho, cs, cl, L

         do i = 1, m

            if ( temperature(i)<=Tm ) then
               elemvc(i) = rho*cs*temperature(i)
            else
               elemvc(i) = rho*cl*temperature(i) + rho*L
            end if   ! ( temperature(i)<=Tm )

         end do  ! i = 1, m
      end if ! ( itype )

1000  call erclos ( 'eltemp2enthal' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End eltemp2enthal'

      end if

      end
