      subroutine elup01 ( s, alpha, dphidx, phi, psi, w )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 04-07-2003 Remove common celwrk
!        version  2.0  date 21-09-1993 Removal of dimension parameters
!        version  1.0  date 27-10-1992
!
!   copyright (c) 1992-2003  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Add contribution of second derivative part to matrix in the case of
!    upwinding (1D, qudratic element)
! **********************************************************************
!
!                       KEYWORDS
!
!     1d
!     element_matrix
!     elliptic_equation
!     upwinding
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision alpha(m), s(n,n), phi(n,m), psi(n,m), &
                       dphidx(n,m), w(m)

!     alpha    i     Array of length m containing the factors alpha
!                    in the integration points
!     dphidx   i     Array of length n x m containing the values of the
!                    derivatives of the basis functions in the integration
!     node
!     phi      i     Array of length n x m containing the values of the basis
!                    functions in the integration points
!     psi      o     Array of length n x m containing the values of the modified
!                    basis functions in the integration points
!     s       i/o    Array of length n x n containing the element
!                    matrix
!     w        i     Array of length m containing the weights for the
!                    numerical integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      double precision sum, dalpdx(3,6)

!     dalpdx         Work array to store d/dx alpha d phi(j)/dx in the
!                    integration points in the following way:
!                    dalpdx(j,k) = d/dx alpha d phi(j)/dx (xk)
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     sum            Parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     - &
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   - &
! **********************************************************************
!
!                       PSEUDO CODE
!
!                              METHOD
!        The contribution of the second derivative upwind terms is:
!       - /  d/dx ( alpha d phi / dx )  (psi -phi )   de
!         e                    j            i    i
!        In order to avoid the computation of second derivatives of basis
!        functions we apply the following approximation:
!                              n
!         alpha dphi / dx  ~  sum  alpha  d phi (x )/dx  phi
!                   j         k=1       k      j  k         k
!                                      n
!        So  d/dx alpha dphi / dx  ~  sum  alpha  d phi (x )/dx  d phi /dx
!                           j         k=1       k      j  k           k
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- First store d/dx alpha dphi(j)/dx in array dalpdx

      do j = 1, n
         do i = 1, m
            sum = 0d0
            do k = 1, n
               sum = sum + alpha(k)*dphidx(j,k)*dphidx(k,i)
            end do
            dalpdx(j,i) = sum
         end do
      end do

!     --- second derivative part

      do i = 1, n
         do j = 1, n
            sum = 0d0
            do k = 1, m
               sum = sum + dalpdx(j,k)*w(k)*(psi(i,k)-phi(i,k))
            end do
            s(j,i) = s(j,i) - sum
         end do
      end do
      end
