      subroutine sepadaprule700
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Adapt integration rule in case of elements for elliptic equations
!     See old subroutine el0800ruledef
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     numerical_integration
!! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: massparm, mrule

!     massparm       Indicates the largest value of massrule during the
!                    time integration
!     mrule          Type of integration rule as given by the user
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepadaprule700' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepadaprule700'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      massparm = max(E%imas,massrule)
      mrule = B%rule
      if ( debug ) then
        write(irefwr,1) 'mrule, ishape, n, ndim', B%rule, E%ishape, B%n, E%ndim
        write(irefwr,1) 'massrule', massrule
      end if  ! ( debug )

      if ( mrule==0 ) then

!     --- Default integration rule

         B%rule = 1
         if ( E%ishape==9 .or. E%ishape>=17) then
            B%rule = 3
         else if ( E%ishape==10 ) then
            B%rule = 2
         else if ( E%ishape>=17 ) then
            B%rule = 3
         else if ( E%itype>=700 .and. E%itype<=801 .or. E%itype==325 .or. &
                   E%itype==816 .or. E%itype>=150 .and. E%itype<=165 .or. &
                   E%itype==814 .or. E%itype==300 ) then
            B%rule = 1
         else if ( B%n==E%ndim+1 .or. E%ishape==21 ) then
            B%rule = 2
         end if

      end if  ! ( mrule==0 )

      if ( debug ) write(irefwr,1) 'B%rule/1', B%rule

!     --- Adapt rule for elements of shape 9 and 10

      if ( E%ishape==9 ) then
         if ( B%rule<3 .or. B%rule>4 ) B%rule = 3
      else if ( E%ishape==10 ) then
         if ( B%rule<2 .or. B%rule>3 ) B%rule = 2
      end if

      if ( debug ) then
         write(irefwr,1) 'B%rule, mrule, methodupw', B%rule, mrule, E%methodupw
         write(irefwr,1) 'E%imas', E%imas
      end if  ! ( debug )

!     --- Adapt rule for upwind

      if ( E%methodupw>=9 .and. E%methodupw<=11 ) then

!     --- special upwind (9: linear, satisfying maximum principle)
!         Always use 1 point Gauss rule

         B%rule = 2

      else if ( mrule==0 .and. E%methodupw>0 ) then

!     --- B%rule = 0, and upwind
!         A different integration rule is applied

         if ( E%imas==1 ) then
            B%rule = 1
         else if ( E%ishape==3 .and. E%methodupw>6 .or. &
                   E%ishape==11 .or. E%ishape==21 ) then
            B%rule = 2
         else if ( E%ishape>=3 .and. E%ishape<=7 ) then
            B%rule = 3
         end if

      else if ( E%imas==2 .and.  mrule==0 .and. &
                (E%ishape<=5 .or. E%ishape>=7) ) then

!     --- Non-diagonal mass matrix, shapes 1-5 and >=7
!         Integration rule must be at least 3 to avoid non-positive
!         definite mass matrices

         if ( E%ishape==2 ) then

!        --- Quadratic 1d, use B%rule = 4

            B%rule = 4

         else

!        --- Other cases, use B%rule = 3

            B%rule = max ( B%rule, 3 )

         end if

      else if ( E%imas==2 .and. E%ishape<=6 .and. mrule==0 ) then

!     --- Non-diagonal mass matrix, shape 6
!         Integration rule must be at least 2 to avoid non-positive
!         definite mass matrices

         B%rule = max ( B%rule, 2 )

      end if
      if ( massparm==2 .and.  mrule==0 .and. &
           (E%ishape<=5 .or. E%ishape>=7) ) then

!     --- Non-diagonal mass matrix, shapes 1-5 and >=7
!         Integration rule must be at least 3 to avoid non-positive
!         definite mass matrices

         if ( E%ishape==2 ) then

!        --- Quadratic 1d, use irule = 4

            B%rule = 4

         else

!        --- Other cases, use irule = 3

            B%rule = max ( B%rule, 3 )

         end if

      else if ( massparm==2 .and. E%ishape<=6 .and. mrule==0 ) then

!     --- Non-diagonal mass matrix, shape 6
!         Integration rule must be at least 2 to avoid non-positive
!         definite mass matrices

         B%rule = max ( B%rule, 2 )
         B%m = 8

      end if

      if ( E%ishape==17 ) B%rule = 3

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'B%rule', B%rule
         write(irefwr,*) 'End sepadaprule700'

      end if  ! ( debug )
      call erclos ( 'sepadaprule700' )

      end subroutine sepadaprule700
