      subroutine sepel0220
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-07-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element independent quantities for element type 220
!     Simple beam element
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     beam
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduleoldvecs
      use sepmodulestrings
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ndimlc

!     ndimlc         Dimension of element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININLARGE    Print 1d integer vector (large numbers)
!     SEPELALLOCCF   Allocate coeffs and related arrays for element
!     SEPELIPOSSET   Set ipos and initialize some other quantities for element
!     SEPELLOWUPP    Find lower and upper bound of variable coefficients
!                    Fill oldvectors if necessary
!     SEPGETCOORSYS  Get the coordinate system as stored in iuser
!     SEPGETRULE     Get the integration rule as stored in iuser
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1771   Shape number not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel0220' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel0220'
         write(irefwr,1) 'ielgrp', E%ielgrp
         call prininlarge ( E%iuser, 15, 'iuser' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initialize ipos and some other quantities

      call sepeliposset

      select case ( E%ishape )

      case(1)

!     --- linear element

         B%n = 4     ! number of basis functions
         B%m = 3     ! number of quadrature points
         B%type_basisfunc = 30  ! linear 1d element with w and w' as degfd
         ndimlc = 1  ! dimension of basis functions
         B%nd2 = 1   ! number of second derivatives

      case default

!     --- Error 1771: this type of element has not yet been implemented

         call errint ( E%ishape, 1 )
         call errint ( E%ielem, 2 )
         call errint ( E%itype, 3 )
         call errint ( E%ielgrp, 4 )
         call errsub ( 1771, 4, 0, 0 )

      end select  ! case ( E%ishape )

      B%fillpsi = .false.
      B%fillderiv2 = .true.
      B%nd = 1
      E%jsol = 1

!     --- First 2 coefficients are not used, so skip

      E%ipos = E%ipos+2
      E%ishift = E%ishift+2

      if ( debug ) write(irefwr,1) 'ipos, ishift/2', E%ipos, E%ishift

!     --- Next quadrature rule and coordinate system

      call sepgetrule
      if ( B%rule>1 ) B%m = B%m+B%rule-1
      call sepgetcoorsys

!     --- Skip one coefficient

      E%ipos = E%ipos+1
      E%ishift = E%ishift+1

      if ( debug ) write(irefwr,1) 'ipos, ishift/3', E%ipos, E%ishift

!     --- Fill all real coefficients

      E%ncoef = 3

      call sepelalloccf ( 1, 0 ) ! allocate coefficient arrays

      if ( E%ind(2)==0 ) E%coeffs(:,2) = 1d0
      E%coeffs(:,4) = E%coeffs(:,1)*E%coeffs(:,2)

!     --- Find lower and upper bound for space dependent coefficients
!         Fill old vectors in locvecs if necessary

      call sepellowupp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepel0220'

      end if  ! ( debug )
      call erclos ( 'sepel0220' )

      end subroutine sepel0220
