      subroutine sepel0300cf
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill real coefficients for elements of type 705
!
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
!     coefficient
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPELALLOCCF   Allocate coeffs and related arrays for element
!     SEPELLOWUPP    Find lower and upper bound of variable coefficients
!                    Fill oldvectors if necessary
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel0300cf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel0300cf'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%indvel = 7
      E%indbeta = E%indvel+3
      E%indsource = E%indbeta+1
      E%indrho = E%indsource+1

      E%indcp = E%indrho+1
      E%indgamma = E%indrho+2
      E%indrhocp = E%indgamma+3
      E%compold = 0

      E%ncoef = 15

      if ( debug ) &
         write(irefwr,1) 'indvel, indbeta, indsource, indrho, indgamma', &
                          E%indvel, E%indbeta, E%indsource, E%indrho, E%indgamma

      call sepelalloccf ( 3, 0 ) ! allocate coefficient arrays

!     --- Find lower and upper bound for space dependent coefficients
!         Fill old vectors in locvecs if necessary

      call sepellowupp

      if ( E%ncoef>6 ) then

!     --- More than 6 real coefficients

         E%sourceterm = E%ind(E%indsource)/=0
         E%zeroterm =  E%ind(E%indbeta)/=0

      end if  ! ( E%ncoef>6 ) then

1000  if ( debug ) then

!     --- Debug information

         call prinin ( E%ich, E%ncoef, 'ich' )
         call prinin ( E%ind, E%ncoef, 'ind' )
         write(irefwr,*) 'End sepel0300cf'

      end if  ! ( debug )
      call erclos ( 'sepel0300cf' )

      end subroutine sepel0300cf
