      subroutine sepel0801cf
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-02-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill real coefficients for elements of type 801
!
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
!     coefficient
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPELALLOCCF   Allocate coeffs and related arrays for element
!     SEPELLOWUPP    Find lower and upper bound of variable coefficients
!                    Fill oldvectors if necessary
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel0801cf' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel0801cf'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      call sepelalloccf ( 0, 0 ) ! allocate coefficient arrays

      do i = 1, E%ncoef
         E%coeffsnode(1:B%n,i) = E%coeffs(1,i)
      end do  ! i = 1, E%ncoef

!     --- Find lower and upper bound for space dependent coefficients
!         Fill old vectors in locvecs if necessary

      call sepellowupp

1000  if ( debug ) then

!     --- Debug information

         call prinin ( E%ich, E%ncoef, 'ich' )
         call prinin ( E%ind, E%ncoef, 'ind' )
         write(irefwr,*) 'End sepel0801cf'

      end if  ! ( debug )
      call erclos ( 'sepel0801cf' )

      end subroutine sepel0801cf
