      subroutine sepel0803
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-02-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element independent quantities for element type 803
!     Old subroutine el0803m
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmoduleoldvecs
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmodulestrings
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepelgetintcf

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     SEPADAPRULE700 Adapt integration rule in case of elements for
!                    elliptic equations
!     SEPEL0803CF    Fill real coefficients for elements of type 803
!     SEPELBASEFNCH  Define parameters for choice of basis functions
!     SEPELGETINTCF  Return with the actual integer value corresponding to
!                    iuser(ipos)
!     SEPELIPOSSET   Set ipos and initialize some other quantities for element
!     SEPELLOWUPP    Find lower and upper bound of variable coefficients
!                    Fill oldvectors if necessary
!     SEPGETCOORSYS  Get the coordinate system as stored in iuser
!     SEPGETRULE     Get the integration rule as stored in iuser
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel0803' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel0803'
         call prinin ( E%iuser, 25, 'iuser' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initialize ipos and some other quantities

      call sepeliposset
      if ( debug ) write(irefwr,1) 'ipos, ishift/0', E%ipos, E%ishift

!     --- Get information of coefficients

      E%ipos = E%ipos+2   ! skip 2 coefficients
      if ( debug ) write(irefwr,1) 'ipos, ishift/1', E%ipos, E%ishift

!     --- Next quadrature rule and coordinate system

      call sepgetrule
      if ( debug ) write(irefwr,1) 'ipos, ishift/2', E%ipos, E%ishift

      call sepgetcoorsys
      if ( debug ) write(irefwr,1) 'ipos, ishift/3', E%ipos, E%ishift
      E%typelinear = sepelgetintcf ( codelinear(2), 2, &
                                     ncodelinear(2), 0, codegen(10) )

!     --- Adapt integration rule for specific cases

      call sepadaprule700

!     --- Get information of basis functions

      call sepelbasefnch

!     --- Fill all real coefficients

      call sepel0803cf

!     --- Check if derivatives of basis functions are required in nodes

      B%fillderivn = B%type_basisfunc/=3 .and. B%type_basisfunc/=10 .and. &
                     B%type_basisfunc/=1 .and. B%type_basisfunc/=11

      E%filloldderiv = .true.  ! the derivative of the old vector must be
                               ! computed
      E%compold = 1            ! There old vectors
      indvector(iseqin) = 1    ! the input vector is an old vector

!     --- Find lower and upper bound for space dependent coefficients
!         Fill old vectors in locvecs if necessary

      call sepellowupp

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'isupcfvar, isubcfvar', E%isupcfvar, E%isubcfvar
         write(irefwr,1) 'B%m, B%n, B%cart', B%m, B%n, B%cart
         write(irefwr,1) 'B%rule', B%rule
         write(irefwr,*) 'diffusion, convection', E%diffusion, E%convection
         write(irefwr,*) 'End sepel0803'

      end if  ! ( debug )
      call erclos ( 'sepel0803' )

      end subroutine sepel0803
