      subroutine sepel0804
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-07-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine SEPELM804
!     in the case that IFIRST = 0
!     Old subroutine el0804
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elliptic_equation
!     initialization
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i

!    i             Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPELALLOCCF   Allocate coeffs and related arrays for element
!     SEPELBASEFNCH  Define parameters for choice of basis functions
!     SEPELIPOSSET   Set ipos and initialize some other quantities for element
!     SEPELLOWUPP    Find lower and upper bound of variable coefficients
!                    Fill oldvectors if necessary
!     SEPGETCOORSYS  Get the coordinate system as stored in iuser
!     SEPGETRULE     Get the integration rule as stored in iuser
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     In the case of a call by BUILD:
!     1:  Not yet in use:  set equal to 0
!     2:  Not yet in use:  set equal to 0
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         Possible values:
!         0: Cartesian co-orindates
!         1: Axi-symmetric co-orindates
!         2: Polar co-orindates
!     5:  Not yet in use:  set equal to 0
!     6:  sigma
!     7:  h
!     8-15: Not yet in use (must be zero)
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1771   Shape number not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel0804' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel0804'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initialize ipos and some other quantities

      call sepeliposset
      if ( debug ) write(irefwr,1) 'ipos, ishift/0', E%ipos, E%ishift

!     --- Set parameters depending on the shape numbers

      n = E%npelm/2
      m = n
      mn = m * n
      E%ishape = -ishape-10001
      if ( E%npelm==1 ) then

         E%ishape = 0

      else if ( E%ishape==1 .or. E%ishape.eq.3 ) then

!     --- linear element

         mn = m

      else if ( E%ishape==2 .or. E%ishape>=4 .and. E%ishape<=6 ) then

!     --- Bilinear or quadratic element

      else

!     --- Error 1771: this type of element has not yet been implemented

         call errint ( ishape, 1 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errsub ( 1771, 4, 0, 0 )

      end if

!     --- Compute the parameters jcart and irule

      E%ipos = E%ipos + 2
      E%ishift = E%ishift+2

      call sepgetrule     ! quadrature rule
      call sepgetcoorsys  ! coordinate system

      E%ipos = E%ipos + 1
      E%ishift = E%ishift+1

!     --- Get information of basis functions

      E%npelm = E%npelm/2
      call sepelbasefnch
      E%npelm = E%npelm*2
      n = B%n

!     --- Fill all real coefficients

      E%ncoef = 2
      call sepelalloccf ( 0, 0 ) ! allocate coefficient arrays

!     --- Find lower and upper bound for space dependent coefficients
!         Fill old vectors in locvecs if necessary

      call sepellowupp

      allocate ( E%submat(n,n), E%subvec(n) )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepel0804'

      end if  ! ( debug )
      call erclos ( 'sepel0804' )

      end subroutine sepel0804
