      subroutine sepelbearconv
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-02-201E%indvel+1
!
!   copyright (c) 201E%indvel+1-201E%indvel+1  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute convection part of Reynolds equation
!     Old subroutine elbearconv
! **********************************************************************
!
!                       KEYWORDS
!
!     bearing
!     compressible
!     element_matrix
!     lubrication
!     convection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleoldvecs
      use sepmodulecomio
      use sepmoduleelm
      use sepmodulebasefn
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: coeff(B%m,3), help(B%m)
      integer :: indh(3), m
      double precision, pointer :: h(:), u1(:), u2(:), diffusion(:), RT(:)
      double precision, pointer :: dpdx(:), f(:,:), pold(:)
      integer, pointer :: ind(:)

!     RT             Contains the constant RT from the ideal gas law
!     coeff          Help array to store the computed coefficient
!                    they are not
!     diffusion      Contains the parameter h^3/(12 mu)
!     dpdx           Contains dpdx in 2d mesh
!     f              Contains the function f in the equation (source)
!     h              Contains the function h in the equation (film thickness)
!     help           help variable to compute a product
!     ind            Refers to E%ind
!     indh           Index array for subroutine sepelconvect0
!     m              Number of integration points
!     pold           Contains the pressure in the prior iteration
!     u1             First component of velocity of lower surface
!     u2             Second component of velocity vector of lower surface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPELCONVECT0T Fill element matrix for convection equation (transpose)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelbearconv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelbearconv'
         write(irefwr,1) 'linear', E%typelinear
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      h => E%coeffs(:,E%indh)
      diffusion => E%coeffs(:,E%inddiff)
      f => E%coeffs(:,E%indsource:E%indsource+1)
      ind => E%ind
      pold => E%ugauss(:,1)
      m = B%m

      if ( debug ) then
         call prinrl ( h, m, 'h' )
         call prinrl ( diffusion, m, 'diffusion' )
         call prinrl ( f(1,1), m, 'f1' )
         call prinrl ( f(1,2), m, 'f2' )
         call prinrl ( pold, m, 'pold' )
      end if  ! ( debug )

!     --- Part -h/2 u

      coeff = 0d0

      if ( ind(E%indvel)/=0 .or. ind(E%induppvel)/=0 ) then

!     --- Store mean value of x-velocity into coeff(.,1)

         u1 => E%coeffs(:,E%indvel)
         u2 => E%coeffs(:,E%induppvel)
         coeff(:,1) = -0.5d0*h*(u1+u2)

      end if  ! ( ind(E%indvel)/=0 .or. ind(E%induppvel)/=0 )

      if ( ind(E%indvel+1)/=0 .or. ind(E%induppvel+1)/=0 ) then

!     --- Store mean value of y-velocity into coeff(.,1)

         u1 => E%coeffs(:,E%indvel+1)
         u2 => E%coeffs(:,E%induppvel+1)
         coeff(:,2) = -0.5d0*h*(u1+u2)

      end if  ! ( ind(E%indvel+1)/=0 .or. ind(E%induppvel+1)/=0 )

      if ( E%typelinear==1 ) then

!     --- Newton linearization
!         Part h^3/12mu grad p

         dpdx => gradoldvec(:,1,1)
         coeff(:,1) = coeff(:,1)+diffusion*dpdx
         if ( E%ndim>=2 ) then
            dpdx => gradoldvec(:,2,1)
            coeff(:,2) = coeff(:,2)+diffusion*dpdx
         end if

      end if  ! ( E%typelinear==1 )

      if ( ind(E%indsource)/=0 .or. ind(E%indsource+1)/=0 ) then

!     --- There is a source

         RT => E%coeffs(:,E%indrt)
         help = -diffusion*pold/RT
         if ( E%typelinear==1 ) help = 2d0*help

      end if  ! ( ind(E%indsource)/=0 .or. ind(E%indsource+1)/=0 )

!     --- Part -h^3/12mu fp/RT (*2)

      if ( ind(E%indsource)/=0 ) coeff(:,1) = coeff(:,1) + help*f(:,1)
      if ( ind(E%indsource+1)/=0 ) coeff(:,2) = coeff(:,2) + help*f(:,2)

!     --- The first derivative term is the transpose of the standard one

      indh = 0
      indh(1) = 1
      indh(2) = 2
      indh(3) = 3
      call sepelconvect0t ( coeff, indh )

1000  call erclos ( 'sepelbearconv' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( E%mat, B%n, B%n, 'elemmat')
         write(irefwr,*) 'End sepelbearconv'

      end if  ! ( debug )

      end subroutine sepelbearconv
