      subroutine sepelbeardiff
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-02-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute diffusion part of Reynolds equation
!     Old subroutine elbeardiff
! **********************************************************************
!
!                       KEYWORDS
!
!     bearing
!     compressible
!     incompressible
!     element_matrix
!     lubrication
!     diffusion
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: arrmu(:), rho(:), pold(:)
      double precision :: diffusion(B%m)
      integer :: m

!     arrmu          Contains the parameter h^3/(12 mu)
!     diffusion      Contains the diffusion array
!     m              number of quadrature points
!     pold           Contains the pressure in the prior iteration
!     rho            density rho
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     SEPELLAPLACE   Compute the element matrix for the Laplace equation
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelbeardiff' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelbeardiff'
         write(irefwr,1) 'E%typebear', E%typebear
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      arrmu => E%coeffs(:,E%inddiff)
      rho => E%coeffs(:,E%indrho)
      pold => E%ugauss(:,1)

      diffusion = arrmu

      m = B%m

      if ( debug ) then
         call prinrl ( diffusion, m, 'diffusion' )
         call prinrl ( rho, m, 'rho' )
      end if  ! ( debug )

      select case ( E%typebear)
      case(1)

!     --- compressible bearing

         diffusion = diffusion*pold

      case(2)

!     --- incompressible bearing, mass conserving
!         In this case we use rho h^3 / 12 mu

         diffusion = diffusion*rho

      end select  ! case ( E%typebear)
      if ( debug ) call prinrl ( diffusion, m, 'diffusion' )

      call sepellaplace ( diffusion )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( E%mat, B%n, B%n, 'elemmat')
         write(irefwr,*) 'End sepelbeardiff'

      end if  ! ( debug )
      call erclos ( 'sepelbeardiff' )

      end subroutine sepelbeardiff
