      subroutine sepelcconvectco
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element matrix for compressible convection equation
!     It is assumed that velocity coefficients are stored in coefficients
!     7 to 9
!     Complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     convection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELCCONVECT3 Fill element matrix for compressible convection equation
!                    Constant derivatives of basis functions (complex)
!     SEPELCCONVECT4 Fill element matrix for compressible convection equation
!                    Variable derivatives of basis functions (complex)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The incompressible form of the convection equation is
!
!     div ( rho cp u c)
!
!     In standard Galerkin this term becomes:
!
!     - ( rho cp u c ) grad phi_i
!
!     in case of upwind we have to add:
!       div ( rho cp u c) (psi-phi)_i
!     with phi the standard basis function and psi the updated (upwind) one
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelcconvectco' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelcconvectco'
         write(irefwr,*) 'B%derivconst ', B%derivconst
         write(irefwr,1) 'E%methodupw ', E%methodupw
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First part without upwind

      if ( B%derivconst ) then

!     --- Derivatives of basis functions are constant

         call sepelcconvect3

      else

!     --- Derivatives of basis functions are variable

         call sepelcconvect4

      end if  ! ( B%derivconst )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelcconvectco'

      end if  ! ( debug )
      call erclos ( 'sepelcconvectco' )

      end subroutine sepelcconvectco
