      subroutine sepelconvect10 ( veloc, ind )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-07-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill (sub) element matrix for convection equation
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     convection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ind(*)
      double precision :: veloc(B%m,*)

!     ind            i    indicates if the "new" velocity components are
!                         constant of variable
!     veloc          i    Contains the velocity components for this
!                         term.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELCONVECT1  Fill element matrix for convection equation
!                    The derivatives of the basis functions are constant
!     SEPELCONVECT2  Fill element matrix for convection equation
!                    The derivatives of the basis functions are variable
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelconvect10' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelconvect10'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( B%derivconst ) then

!     --- Derivatives of basis functions are constant

         call sepelconvect1 ( veloc, ind, B%jdiag, B%phi )

      else

!     --- Derivatives of basis functions are variable

         call sepelconvect2 ( veloc, ind, B%jdiag, B%phi )

      end if  ! ( B%derivconst )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelconvect10'

      end if  ! ( debug )
      call erclos ( 'sepelconvect10' )

      end subroutine sepelconvect10
