      subroutine sepelconvect7 ( veloc, ind, f )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-10-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill arrays veloc, f and ind in case of non-linear convection
!
! ***********************************a***********************************
!
!                       KEYWORDS
!
!     element_matrix
!     convection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulefunc
      use sepmoduleoldvecs
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(out) :: ind(3)
      double precision, intent(out) :: veloc(B%m,3), f(B%m)

!     f              o    right-hand side term due to convection
!     ind            o    indicates if the "new" velocity components are
!                         constant of variable
!     veloc          o    Contains the new velocity components for this
!                         term. They are -1 or 1 depending on the sign of the
!                         derivatives
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i

!     i              General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPFUNCC2      User function to compute g and dg/dgradc
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelconvect7' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelconvect7'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First fill g and grad g by calling the user subroutine sepfuncc2

      call sepfuncc2 ( f, veloc )

      ind(1:B%nd) = 1
      ind(B%nd+1:3) = 0

      f = -f

      do i = 1, B%nd
         f = f+gradprev(:,i)*veloc(:,i)
      end do  ! i = 1, B%nd

1000  call erclos ( 'sepelconvect7' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelconvect7'

      end if  ! ( debug )

      end subroutine sepelconvect7
