      subroutine sepelctypeconv
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill quantities with respect to convection
!     Complex version
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     convection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleoldvecs
      use sepmodulestrings
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepelgetintcf

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELGETINTCF  Return with the actual integer value corresponding to
!                    iuser(ipos)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelctypeconv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelctypeconv'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set logical convection

      E%convection = E%ind(E%indvel)/=0 .or. B%nd>1 .and. &
                     E%ind(E%indvel+1)/=0 .or. B%nd>2 .and. &
                     E%ind(E%indvel+2)/=0 .or. E%typeconv>1

      if ( B%fillqmat .and. .not. E%convection ) &
         E%convection = E%ind(E%indvel)/=0 .or. E%ind(E%indvel+1)/=0 .or. &
                        E%ind(E%indvel+2)/=0

      E%cveloc => E%ccoeffs(:,E%indvel:E%indvel+2)
      E%cuvel => E%ccoeffs(:,E%indvel)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelctypeconv'

      end if  ! ( debug )
      call erclos ( 'sepelctypeconv' )

      end subroutine sepelctypeconv
