      subroutine sepelcvecgamma
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Add term div( gamma) to element vector
!     Complex case
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i
      double complex :: work(B%m)
      double precision, pointer :: phix(:,:), phiy(:,:), phiz(:,:)

!     i              General loop variable
!     phix           x derivative of basis function phi
!     phiy           y derivative of basis function phi
!     phiz           z derivative of basis function phi
!     work           work array to store gamma . grad(phi)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The term div gamma is added to the right-hand side
!
!     The contribution to the element vector is
!
!     - / grad (phi_i) . gamma d Omega
!
!     and after numerical quadrature:
!
!     - sum_k w(k) grad(phi_i(x_k)) . gamma(x_k)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelcvecgamma' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelcvecgamma'
         write(irefwr,*) 'B%derivconst ', B%derivconst

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      phix => B%phider(:,:,1)
      phiy => B%phider(:,:,2)
      phiz => B%phider(:,:,3)

      if ( B%derivconst ) then

!     --- Derivative of phi is constant

         do i = 1, B%n
            work = 0d0
            if ( E%ind(E%indgamma)/=0 ) work = E%cgamma(:,1)*phix(i,1)
            if ( E%ind(E%indgamma+1)/=0 ) work = work+E%cgamma(:,2)*phiy(i,1)
            if ( E%ind(E%indgamma+2)/=0 ) work = work+E%cgamma(:,3)*phiz(i,1)
            E%cvec(i) = E%cvec(i) - sum(work*B%w)
         end do

      else

!     --- Derivative of phi is variable

         do i = 1, B%n
            work = 0d0
            if ( E%ind(E%indgamma)/=0 ) work = E%cgamma(:,1)*phix(i,:)
            if ( E%ind(E%indgamma+1)/=0 ) work = work+E%cgamma(:,2)*phiy(i,:)
            if ( E%ind(E%indgamma+2)/=0 ) work = work+E%cgamma(:,3)*phiz(i,:)
            E%cvec(i) = E%cvec(i) - sum(work*B%w)
         end do

      end if  ! ( B%derivconst )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelcvecgamma'

      end if  ! ( debug )
      call erclos ( 'sepelcvecgamma' )

      end subroutine sepelcvecgamma
