      subroutine sepeld0705
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element independent quantities for derivatives of element type 705
!     general elliptic equation (one unknown per point)
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleelem
      use sepmodulestrings
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: kchoice

!     kchoice        Defines type of coefficients required
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepelgetintcf

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPEL0705CF    Fill real coefficients for elements of type 705
!     SEPELALLOCVCOL Allocate locvecs in oldsols for one vector
!     SEPELBASEFNCH  Define parameters for choice of basis functions
!     SEPELGETINTCF  Return with the actual integer value corresponding to
!                    iuser(ipos)
!     SEPELGETIQMAT  Get the parameter iqmat as stored in iuser
!     SEPELIPOSSET   Set ipos and initialize some other quantities for element
!     SEPELRDTYPCONV Read information about type_convection
!                    Adapt methodupw if necessary
!     SEPGETCOORSYS  Get the coordinate system as stored in iuser
!     SEPGETRULE     Get the integration rule as stored in iuser
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeld0705' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeld0705'
         write(irefwr,1) 'icheld, itype', icheld, itype
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- In case of laplace or poisson, the diffusion matrix is the unit matrix

      if ( icheld==6 .and. (itype==700 .or. itype==701) ) icheld = 3

!     --- Compute kchoice as function of icheld

      select case ( icheld )
      case ( 23:25, 41:45 )
         kchoice = 0
         E%ncoef = 18
      case ( 21 )
         kchoice = 4
         E%ncoef = 1
      case ( 6, 8:10, 16, 26, 28, 29, 36 )
         kchoice = 3
         E%ncoef = 6
      case default
         kchoice = 1
         E%ncoef = 0
      end select  ! case ( icheld )

      if ( debug ) write(irefwr,1) 'kchoice, ncoef', kchoice, E%ncoef

      E%imas = 0
      massrule = 0
      E%methodupw = 0
      E%applyupw = 0
      B%cart = 1
      B%rule = 1
      E%compold = 0

      if ( icheld==10 .and. E%ndim>1 ) then
         E%filloldderiv = .false.
      else
         E%filloldderiv = .true.
      end if  ! ( icheld==10 .and. E%ndim>1 )

      if ( E%ncoef>0 ) call sepeliposset  ! set parameters for coeffs

      if ( kchoice==0 ) then

!     --- At least some coefficients must be filled
!         initialize ipos and some other quantities

         call sepelgetiqmat
         E%methodupw = sepelgetintcf ( codeupw, nncodeupw, &
                                       ncodeupw, 1, 'supg' )
         if ( debug ) write(irefwr,1) 'ipos, ishift/1', E%ipos, E%ishift

!        --- Next quadrature rule and coordinate system

         call sepgetrule

         call sepgetcoorsys

!        --- Read type_convection
!            Adapt methodupw if necessary

         call sepelrdtypconv

      else if ( E%ncoef>0 ) then
         E%ipos = E%ipos+5
      end if  ! ( E%ncoef>0 ) then
      if ( debug ) write(irefwr,1) 'ipos', E%ipos

!     --- Get information of basis functions

      call sepelbasefnch
      B%kstep = 1  ! to avoid jumping over vertices

      if ( E%ncoef>0 ) call sepel0705cf  ! fill coefficients

      if ( E%compold==0 ) then
         if ( icheld==10 .or. icheld==9 ) E%compold = 1
         call sepelallocvcol! allocate oldsols%locvecs
      end if  ! ( E%compold==0 )

      B%fillderivn = .true.  ! Compute derivatives also in vertices
      B%callbyderiv = .true.

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'icheld, ncoef', icheld, E%ncoef
         write(irefwr,1) 'irule, cart', B%rule, B%cart
         write(irefwr,*) 'End sepeld0705'

      end if  ! ( debug )
      call erclos ( 'sepeld0705' )

      end subroutine sepeld0705
