      subroutine sepeld11
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual filling of derivatives and weights in E%vec
!     with respect to subroutine sepeld705
!     icheld = 11: d c /d_ix, no weights
!     Old subroutine el100111
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleelem
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i
      double precision, pointer :: dudx(:), dphidx(:,:)

!                    they are not
!     dphidx         Reference to array B%phider
!                    (derivatives of basis functions in nodes)
!     dudx           Reference to array gradoldvec
!                    (derivatives of solution in nodes)
!     i              General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeld11' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeld11'
         write(irefwr,1) 'jdercn, n, ix', jdercn, n, ix
         call prinrl ( dudx, n, 'dudx' )
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      dudx => gradoldvec(:,ix,1)
      dphidx => B%phider(:,:,ix)
      n = B%n

      if ( B%derivconst ) then

!     --- Derivatives are constant

         E%vec = dudx(1)

      else if ( ndim==2 .and. n>=6 ) then

!     --- quadratic elements in R2

         do i = 1, n-1, 2
            E%vec(i) = dudx(i)
         end do
         do i = 2, n, 2
            E%vec(i) = dudx(i)
         end do
         E%vec(n) = dudx(n)

      else

!     --- General case

         E%vec = dudx(1:n)

      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeld11'

      end if  ! ( debug )
      call erclos ( 'sepeld11' )

      end subroutine sepeld11
