      subroutine sepeld23 ( elemvc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-02-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute flow vector for Reynolds
!     Old subroutine el1017
! **********************************************************************
!
!                       KEYWORDS
!
!     bearing
!     compressible
!     incompressible
!     element_vector
!     lubrication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision :: elemvc(E%ndim,B%n)

!     elemvc         o    Element vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: j, n
      double precision, pointer :: h(:), mu(:), diffusion(:), gradp(:,:)
      double precision, pointer :: rho(:), f(:,:), u1(:,:), u2(:,:)
      double precision, pointer :: RT(:), p(:), h2(:)

!     RT             Contains the constant RT from the ideal gas law
!                    they are not
!     diffusion      Contains h^3/(12 mu)
!     f              source vector
!     gradp          gradient of the prior pressure
!     h              layer thickness
!     h2             h/2
!     j              Counting variable
!     mu             viscosity
!     n              number of basis functions for each component
!     p              prior pressure
!     rho            density rho
!     u1             velocity of lower surface
!     u2             velocity of upper surface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeld23' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeld23'
         write(irefwr,1) 'm, n, nd', B%m, B%n, B%nd
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      n = B%n

      h => E%coeffs(:,1)
      if ( debug ) call prinrl ( h, n, 'h' )
      mu => E%coeffs(:,2)
      rho => E%coeffs(:,9)
      RT => E%coeffs(:,10)
      f => E%coeffs(:,11:12)
      u1 => E%coeffs(:,6:7)
      u2 => E%coeffs(:,15:16)
      if ( debug ) call prinrl ( mu, n, 'mu' )

      gradp => gradoldvec(:,:,1)
      p => uprev(:,1)

      diffusion => E%coeffs(:,18)
      h2 => E%coeffs(:,17)

      diffusion = h**3/(12d0*mu)
      h2 = h*0.5d0

      elemvc = 0d0

      if ( debug ) call prinrl ( diffusion, n, 'diffusion' )

!     --- Compute flow vector

      do j = 1, min(2,B%nd)
         elemvc(j,:) = -diffusion*(gradp(:,j)- rho*f(:,j))+ &
                       h2*(u1(:,j)+u2(:,j))
      end do  ! j = 1, min(2,B%nd)

      if ( debug ) write(irefwr,1) 'typebear', E%typebear

      if ( E%typebear/=0 ) then
         do j = 1, min(2,B%nd)
            elemvc(j,:) = p*elemvc(j,:)/RT
         end do  ! j = 1, min(2,B%nd)
      end if

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( elemvc, n, 'elemvc' )
         write(irefwr,*) 'End sepeld23'

      end if  ! ( debug )
      call erclos ( 'sepeld23' )

      end subroutine sepeld23
