      subroutine sepeld25 ( elemvc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 22-02-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute shear stress for top surface for Reynolds
!     Old subroutine el1017
! **********************************************************************
!
!                       KEYWORDS
!
!     bearing
!     compressible
!     incompressible
!     element_vector
!     lubrication
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision :: elemvc(E%ndim,B%n)

!     elemvc         o    Element vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: j, n, k, i
      double precision, pointer :: h(:), mu(:), gradp(:,:)
      double precision, pointer :: rho(:), f(:,:), u1(:,:), u2(:,:)
      double precision, pointer :: p(:), h2(:), muh(:)
      logical :: cavitat, cavstart

!     cavitat    If true we have cavitation in the element
!     cavstart   Starting value for cavitat
!     f              source vector
!     gradp          gradient of the prior pressure
!     h              layer thickness
!     h2             h/2
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     mu             viscosity
!     muh            mu/h
!     n              number of basis functions for each component
!     p              prior pressure
!     rho            density rho
!     u1             velocity of lower surface
!     u2             velocity of upper surface
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeld25' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeld25'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      h => E%coeffs(:,1)
      mu => E%coeffs(:,2)
      rho => E%coeffs(:,9)
      f => E%coeffs(:,11:12)
      u1 => E%coeffs(:,6:7)
      u2 => E%coeffs(:,15:16)

      muh => E%coeffs(:,18)
      h2 => E%coeffs(:,17)

      gradp => gradoldvec(:,:,1)
      p => uprev(:,1)

      h2 = h*0.5d0
      muh = mu/h

      n = B%n

      elemvc = 0d0

!     --- Compute upper shear stress
!         if pressure==0d0 for all elements asume no cavitation
!         else if pressure==0d0 AND dpdx==0d0 AND dpdy==0d0 then
!         assume cavitation in point and thus no shear stress
!         SHOULD BE REPLACED BY A CORRECT MASS-CONSERVATION ALGORITHM

      cavstart = .false.
      do k = 1, n
         if ( abs(p(k))>1d-8 ) then
            cavstart = .true.
         end if
      end do
      do j = 1, min(2,B%nd)
         do k = 1, n
            elemvc(j,k) = h2(k)*(gradp(k,j)-rho(k)*f(k,j))+ &
                          muh(k)*(u2(k,j)-u1(k,j))
            if ( abs(p(k))<=0d0 ) then
               cavitat = cavstart
            else
               cavitat = .false.
            end if
            do i = 1, B%nd
               if ( abs(gradp(k,i))>0d0 ) cavitat = .false.
            end do
            if ( cavitat ) elemvc(j,k) = 0d0
         end do
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeld25'

      end if  ! ( debug )
      call erclos ( 'sepeld25' )

      end subroutine sepeld25
