      subroutine sepeld810_45
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-12-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Determine whether an element should be refined (E%ielvec = 1) or not
!     (E%ielvec = 0). An element will NOT be refined only when the enthalpy
!     values (in the integration points) are all larger than rho*L or all
!     less than zero.
!     Old subroutine eldetrefelem
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
!     enthalpy
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      use sepmoduleelm
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, pos, neg
      double precision :: rho, L
      double precision, pointer :: enthalpy(:)


!     L              Latent heat
!     enthalpy       enthalpy vector in integration points
!     i              Counting variable
!     neg            Counting variable
!     pos            Counting variable
!     rho            density rho
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeld810_45' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeld810_45'
         write(irefwr,100) 'ielem, iseqenth', ielem, iseqenth
100      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000

!     --- Get parameters

      rho = E%coeffs(1,12)
      L   = E%coeffs(1,15)

!     --- Fill enthalpy

      enthalpy =>  oldsols(iseqenth)%locvecs(1:m,1)

!     --- Count the number of enthalpy values larger than rho*L
!         and less than zero

      neg = 0
      pos = 0

      do i = 1, m
         if ( enthalpy(i)<0d0 ) then
            neg = neg + 1
         else if ( enthalpy(i)>rho*L ) then
            pos = pos + 1
         end if
      end do

!     --- mark only those elements NOT to be refined when either
!         all enthalpy values are larger than rho*L or less than
!         zero

      if ( neg==m .or. pos==m ) then
         E%ielvec(1) = 0
      else
         E%ielvec(1) = 1
      end if

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,100) 'itype', itype
         call prinrl ( enthalpy, m, 'enthalpy' )
         call prinin ( E%ielvec, 1, 'E%ielvec' )
         write(irefwr,*) 'End sepeld810_45'

      end if
      call erclos ( 'sepeld810_45' )

      end
