      subroutine sepeld810int
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-12-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element derivative vector for second order elliptic
!     equations with an integer derivative vector
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPEL0810      initializations for subroutine sepelm810
!     SEPELBASEFUNC  Compute information about basis functions
!     SEPELD810_45   Fill element vector for integer derivative of type 810
!     SEPELOLDVECS   Fill old vectors for element
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     The sequence is:
!     1:  Not used
!     2:  not used
!     3:  Type of numerical integration
!         Possible values:
!         0: default value, defined by element
!     4:  Type of co-ordinate system
!         See type 800
!     5:  Type of equation
!         0: standard enthalpy with phase change
!         1: temperature equation, no phase change
!     6:  kappa_S
!     7:  kappa_L
!     8:  Capacity_S
!     9:  Capacity_L
!    10:  Latent_Heat
!    11:  Melt_Temp
!    12:  u 1       (1D/2D/3D)
!    13:  u 2       (2D/3D)
!    14:  u 3       (3D)
!    15:  beta      (1D/2D/3D)
!    16:  f         (1D/2D/3D)
!    17:  rho cp    (1D/2D/3D)
!    18:  Not yet used (must be 0)
!    19:  Not yet used (must be 0)
!    20:  Not yet used (must be 0)
!    21:  iseqtemp
!    22:  iseqenth
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     if ( ifirst=0 ) then
!        Define element independent quantities
!        Compute pointers in array WORK
!     fi
!     Compute basis functions and related quantities
!     Fill old solution into array UOLD and if necessary the derivatives
!     Compute variable coefficients if necessary
!     Compute derived quantities
!                              METHOD
!       Differential equation:
!         - div alpha grad phi + u + grad phi + beta phi = f
!                                -
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeld810int' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeld810int'
         write(irefwr,1) 'E%first', E%first
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%ielvec = 0

      if ( E%first==0 ) then

!     --- First element,  compute element independent quantities

         call sepel0810
         if ( ierror/=0 ) go to 1000

!     --- Next element, restore coeffs

         E%coeffs = E%coeffcp

      end if

!     --- Fill basis functions

      call sepelbasefunc

!     --- Fill old vectors in quadrature points if necessary

      call sepeloldvecs

!     --- icheld = 45

      call sepeld810_45

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeld810int'

      end if  ! ( debug )
      call erclos ( 'sepeld810int' )

      end
