      subroutine sepeldc06 ( elemvc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual filling of derivatives and weights in elemvc and elemwg
!     with respect to subroutine sepeld705
!     icheld = 6, 16, 26  Compute - A gradient
!     Old subroutine el100116
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleelem
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double complex :: elemvc(ndimlc,B%n)

!     elemvc         o    Element vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i
      double complex :: som
      double complex, pointer :: dudx(:,:)
      double precision, pointer :: dphidx(:,:,:)

!                    they are not
!     dphidx         Reference to array B%phider
!                    (derivatives of basis functions in nodes)
!     dudx           Reference to array gradoldvec
!                    (derivatives of solution in nodes)
!     i              Counting variable
!     som            Help variable to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELDC061     actual filling of derivatives and weights in elemvc
!                    with respect to subroutine sepeld705 (complex)
!                    icheld = 6, 16, 26  Compute - A gradient. ndim = 1
!     SEPELDC062     actual filling of derivatives and weights in elemvc
!                    with respect to subroutine sepeld705 (complex)
!                    icheld = 6, 16, 26  Compute - A gradient. ndim = 2
!     SEPELDC063     actual filling of derivatives and weights in elemvc
!                    with respect to subroutine sepeld705 (complex)
!                    icheld = 6, 16, 26  Compute - A gradient. ndim = 3
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeldc06' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeldc06'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case ( ndim )
      case(1)

!     --- NDIM = 1, 1 dimensional space

         call sepeldc061 ( elemvc )

      case(2)

!     --- NDIM = 2, 2 dimensional space

         call sepeldc062 ( elemvc )

      case(3)

!     --- NDIM = 3, 3 dimensional space

         call sepeldc063 ( elemvc )

      end select  ! case ( ndim )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeldc06'

      end if  ! ( debug )
      call erclos ( 'sepeldc06' )
      end
