      subroutine sepeldc12 ( elemvc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-01-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     actual filling of derivatives and weights in elemvc and elemwg
!     with respect to subroutine sepeld705
!     icheld = 12, 13, 15: gradient, no weights
!     Old subroutine el10012
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elliptic_equation
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmoduleelem
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double complex :: elemvc(ndimlc)

!     elemvc         o    Element vector
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j
      double complex :: h
      double complex, pointer :: dudx(:,:)
      double precision, pointer :: dphidx(:,:,:)

!                    they are not
!     dphidx         Reference to array B%phider
!                    (derivatives of basis functions in nodes)
!     dudx           Reference to array gradoldvec
!                    (derivatives of solution in nodes)
!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     j              General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeldc12' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeldc12'
         write(irefwr,1) 'n', B%n
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      dudx => gradoldcvec(:,:,1)
      dphidx => B%phider(:,:,:)
      n = B%n

      if ( B%derivconst ) then

!     --- Derivatives are constant

         do j = 1, ndimlc
            elemvc(j) = dudx(1,j)
         end do

      else

!     --- General element

         do j = 1, ndimlc
            elemvc(j) = sum(dudx(1:n,j))/n
         end do

      end if

      if ( icheld==13 ) elemvc = -elemvc  ! change sign

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeldc12'

      end if  ! ( debug )
      call erclos ( 'sepeldc12' )

      end subroutine sepeldc12
