      subroutine sepeldiff803
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-03-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Recompute the diffusion tensor for type 803
!     Depends on type of linearization
!     Old subroutine elm800special
! **********************************************************************
!
!                       KEYWORDS
!
!     element
!     coefficient
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleoldvecs
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, i2, i3
      double precision, pointer :: alpha(:), x(:,:), gradph(:), dalpdg(:), &
                                   dcdx(:,:)

!     alpha          diffusion coefficient
!     dalpdg         derivative of alpha with respect to || grad c ||
!     dcdx           derivatives of the old solution (c)
!     gradph         || grad c ||
!     i              Counting variable
!     i2             2 if ndim>=2, else 1
!     i3             3 if ndim>=1, else 1
!     x              co-ordinates of nodes in element
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     FUNCC2         User subroutine to compute alpha and d alpha/d ||grad c||
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeldiff803' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeldiff803'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      i = E%indrhocp
      alpha => E%coeffs(:,1)
      gradph => E%coeffs(:,i+1)
      dalpdg => E%coeffs(:,i+2)
      dcdx => gradoldvec(:,:,1)
      x => B%xyzg
      i2 = max(1,E%ndim)
      i3 = max(1,E%ndim)

      do i = 1, B%m
         gradph(i) = sqrt(sum(dcdx(i,1:E%ndim)**2))
         call funcc2 ( E%typelinear, x(i,1), x(i,i2), x(i,i3), &
                       gradph(i), alpha(I), dalpdg(i) )
      end do  ! i = 1, B%m

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeldiff803'

      end if  ! ( debug )
      call erclos ( 'sepeldiff803' )

      end subroutine sepeldiff803
