      function sepelgetintcf ( code, nncode, nscode, isubtract, keywordlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 07-01-2015 Change for non-existing string
!        version  1.0  date 15-12-2014
!
!   copyright (c) 2014-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Give sepelgetintcf the actual integer value corresponding to iuser(ipos)
!     Raise ipos and ishift
!     Old subroutine elgetintcoef
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     scalar
!     string
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulecomio
      use sepmodulecons
      use sepmoduleread
      use sepmoduletexts
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: nncode, nscode(nncode), isubtract
      character (len=*) :: code(nncode), keywordlc
      integer :: sepelgetintcf

!     code           i    Table with subkeywords corresponding to keywordlc
!     isubtract      i    Value which must be subtracted from the sequence
!                         number in code to get the correct parameter
!     keywordlc      i    contains the main keyword corresponding to the
!                         parameter
!     nncode         i    Number of entries in code
!     nscode         i    Table containing the number of significant characters
!                         in the keyword table code
!     sepelgetintcf  o    Value of integer parameter
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: iseq, i

!     i              Counting variable
!     iseq           Sequence number
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: inirepscal

!     ASKUPC         Translate all letters to upper case
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRSUB         Error messages
!     INIREPSCAL     Corrects integer value in case of scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2543   string not recognized as subkeyword
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelgetintcf' )
      debug = .false.! .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelgetintcf'
         write(irefwr,1) 'ipos, ishift', E%ipos, E%ishift
         write(irefwr,1) 'iuser(ipos)', E%iuser(E%ipos)
  1      format ( a, 1x, (10i10) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      sepelgetintcf = 0

      if ( E%iuser(E%ipos)>100100000 ) then

!     --- Parameter refers to string

         iseq = E%iuser(E%ipos)-100100000        ! sequence number
         string = texts(instring(iseq))      ! value of string
         nchar =  index ( string, ' ' )-1    ! number of characters

         if ( nchar==0 ) then

!        --- No string found, return with 0 because not yet filled

            sepelgetintcf = 0
            E%ipos = E%ipos + 1
            E%ishift = E%ishift+1
            go to 1000

         end if  ! ( nchar==0 )

         call askupc ( string(1:nchar) )     ! replace by capitals

         if ( debug ) then
            write(irefwr,1) 'iseq, nchar', iseq, nchar
            write(irefwr,*) 'string ', string(1:nchar)
         end if  ! ( debug )

!        --- Replace the string by a number

         iseq = 0
         do i = 1, nncode

            if ( code(i)(1:nscode(i))==string(1:min(nchar,nscode(i))) ) then

!           --- string is equal to code i

               iseq = i
               exit

            end if  ! ( code(i)(1:nscode(i))==string(1:nchar) )

         end do  ! i = 1, nncode

         if ( iseq==0 ) then

!        --- string has wrong value

            call errchr ( keywordlc, 1 )
            call errchr ( string(1:nchar), 2 )
            do i = 1, nncode
               call errchr ( code(i)(1:nscode(i)), i+2 )
            end do  ! i = 1, nncode
            call errsub ( 2543, 0, 0, nncode+2 )

         end if  ! ( iseq==0 )

         sepelgetintcf = iseq-isubtract

      else

!     --- Parameter refers to integer

         sepelgetintcf = inirepscal ( E%iuser(E%ipos) )

      end if  ! ( E%iuser(E%ipos)>100100000)

      E%ipos = E%ipos + 1
      E%ishift = E%ishift+1

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ipos, ishift', E%ipos, E%ishift
         write(irefwr,1) 'sepelgetintcf', sepelgetintcf
         write(irefwr,*) 'End sepelgetintcf'

      end if  ! ( debug )
      call erclos ( 'sepelgetintcf' )

      end function sepelgetintcf
