      subroutine sepelgetiqmat
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-11-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Get the parameter iqmat as stored in iuser
!     raise ipos and E%ishift
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     transformation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebasefn
      use sepmoduleelm
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: inirepscal

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     INIREPSCAL     Corrects integer value in case of scalars
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1378   Parameter has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelgetiqmat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelgetiqmat'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%iqmat = inirepscal ( E%iuser(E%ipos) )
      if ( debug ) write(irefwr,1) 'iqmat', E%iqmat
      E%ipos = E%ipos+1
      E%ishift = E%ishift+1

      if ( E%iqmat>1 ) B%fillqmat = .true.
      if  ( E%iqmat>=10 ) E%iqmat = E%iqmat/10

      if ( E%iqmat<0 .or. E%iqmat>4 ) then

!     --- iqmat has wrong value

         call errchr ( 'iqmat', 1 )
         call errint ( E%iqmat, 1 )
         call errint ( 0, 2 )
         call errint ( 4, 3 )
         call errsub ( 1378, 3, 0, 1 )

      end if  ! ( iqmat<0 .or. iqmat>4 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelgetiqmat'

      end if  ! ( debug )
      call erclos ( 'sepelgetiqmat' )

      end subroutine sepelgetiqmat
