      double precision function sepelic01 ( )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-06-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute a volume integral over one element depending on icheli
!     Old subroutine elic01
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     integral
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmoduleelem
      use sepmodulebasefn
      use sepmoduleoldvecs
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: dudx(:), f(:), w(:), u(:)
      double precision :: work(B%m)
      integer i

!    i      General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      double precision ddot

!    ddot    Computes inner product of two vectors (BLAS subroutine)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Store f, uf or f du/dx into work

      f => E%coeffs(:,1)
      w => B%w

      select case ( icheli )
      case (1)

!     --- icheli = 1, only f

         work = f

      case (2)

!     --- icheli = 2, f u

         u => uprev(:,jdegfd)
         work = f*u

      case (6)

!     --- icheli = 6, f |u|

         u => uprev(:,jdegfd)
         work = f*abs(u)

      case default

!     --- icheli = 3-5, f du/dx

         dudx => gradoldvec(:,jdegfd,1)
         if ( B%derivconst ) then

!        --- Derivative constant

            work = f * dudx(1)

         else

!        --- Derivative variable

            work = f * dudx

         end if

      end select  ! case ( icheli )

      sepelic01 = dot_product ( work, w )

      end function sepelic01
