      subroutine sepelinstrain
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute contribution to element vector due to initial strain
!     Old subroutine el8103
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: Dw(E%dimD,E%dimD,m)
      integer :: i, j

!     Dw             Dmat * w
!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELINSTRAIN1 Compute contribution to element vector due to
!                    initial strain (2d)
!     SEPELINSTRAIN2 Compute contribution to element vector due to
!                    initial strain (3d)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!                                 T
!   The contribution is given by B D eps0
!
!   So:
!         l
!   2D:  f (i) = sum w(k) b(ip,l,i,k) d(ip,iq,k) eps0(k,iq)
!
!                where the sum is taken over k, ip, iq
!
!         1
!   3D:  f (i) = sum  w(k) [ phi (i,k) d(1,j,k) + phi (i,k) d(4,j,k)+ &
!                k,j            x                    y
!
!                            phi (i,k) d(6,j,k) ] eps0(k,j)
!                               z
!
!         2
!        f (i) = sum  w(k) [ phi (i,k) d(2,j,k) + phi (i,k) d(4,j,k)+ &
!                k,j            y                    x
!
!                            phi (i,k) d(5,j,k) ] eps0(k,j)
!                               z
!
!         2
!        f (i) = sum  w(k) [ phi (i,k) d(3,j,k) + phi (i,k) d(5,j,k)+ &
!                k,j            z                    y
!
!                            phi (i,k) d(6,j,k) ] eps0(k,j)
!                               x
!
!         In the isotropic case (igprob=1) this reduces to:
!
!         1
!        f (i) = sum  w(k) [ sum  phi (i,k) d(1,j,k) eps0(k,j) ] + &
!                k         j=1,2,3   x
!
!                    phi (i,k) d(4,4,k)eps0(k,4)+phi (i,k) d(6,6,k)eps0(k,6)
!                       y                           z
!
!         2
!        f (i) = sum  w(k) [ sum  phi (i,k) d(2,j,k) eps0(k,j) ] + &
!                k         j=1,2,3   y
!
!                    phi (i,k) d(4,4,k)eps0(k,4)+phi (i,k) d(5,5,k)eps0(k,5)
!                       x                           z
!
!         3
!        f (i) = sum  w(k) [ sum  phi (i,k) d(3,j,k) eps0(k,j) ] + &
!                k         j=1,2,3   z
!
!                    phi (i,k) d(5,5,k)eps0(k,5)+phi (i,k) d(6,6,k)eps0(k,6)
!                       y                           x
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelinstrain' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelinstrain'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Multiply D by w and store in dw

      do i = 1, E%dimD
         do j = 1, E%dimD
            Dw(i,j,:) = B%w(:)*E%Dmat(i,j,:)
         end do  ! j = 1, E%dimD
      end do  ! i = 1, E%dimD

      select case (E%ndim)

      case(2)

!     --- ndim = 2

         call sepelinstrain1 ( E%coeffs(:,26), Dw )

      case(3)

!     --- ndim = 3

         call sepelinstrain2 ( E%coeffs(:,26), Dw )

      end select  ! case (E%ndim)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelinstrain'

      end if  ! ( debug )
      call erclos ( 'sepelinstrain' )

      end subroutine sepelinstrain
