      subroutine sepelinstress
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute contribution to element vector due to initial stress
!     Old subroutine el8104
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELINSTRESS1 Compute contribution to element vector due to
!                    initial stress (2d)
!     SEPELINSTRESS2 Compute contribution to element vector due to
!                    initial stress (3d)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     f =  f + B sigma
!
!          l    l
!     or  f  = f  + B(i,l) sigma
!
!     In 3D:
!
!      1       1
!     f (i) = f (i) + sum w (phi  (k) sigma(k,1)+phi  (k) sigma(k,4)+
!                      k   k    ix                  iy
!
!                            phi  (k) sigma(k,6) )
!                               iz
!
!      2       2
!     f (i) = f (i) + sum w (phi  (k) sigma(k,2)+phi  (k) sigma(k,4)+
!                      k   k    iy                  ix
!
!                            phi  (k) sigma(k,5) )
!                               iz
!
!      3       3
!     f (i) = f (i) + sum w (phi  (k) sigma(k,3)+phi  (k) sigma(k,5)+
!                      k   k    iz                  iy
!
!                            phi  (k) sigma(k,6) )
!                               ix
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelinstress' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelinstress'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      select case (E%ndim)

      case(2)

!     --- ndim = 2

         call sepelinstress1 ( E%coeffs(:,32) )

      case(3)

!     --- ndim = 3

         call sepelinstress2 ( E%coeffs(:,32) )

      end select  ! case (E%ndim)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelinstress'

      end if  ! ( debug )
      call erclos ( 'sepelinstress' )

      end subroutine sepelinstress
