      subroutine sepelinstress1 ( sigma0 )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute contribution to element vector due to initial stress (2d)
!     Old subroutine el8104
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     elasticity_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: sigma0(m,E%dimD)

!     sigma0         i    initial stress in quadrature points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: v(:)
      integer :: i, nd, ip, l
      double precision :: som, sigma(m,E%dimD)

!     i              Counting variable
!     ip             Loop variable
!     l              General loop variable
!     nd             Dimension of Matrix D
!     sigma          stress tensor
!     som            Help parameter to compute a sum
!     v              Sub element vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelinstress1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelinstress1'
         call prinrl1 ( E%vec, n, 2, 'vec' )
         call prinrl2 ( sigma0, m, E%dimD, 'sigma0' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      nd = E%dimD
      do i = 1, nd
         sigma(:,i) = B%w(:)*sigma0(:,i)
      end do  ! i = 1, nd

      if (  B%derivconst .and. nd==3 ) then

!     --- Derivatives of basis functions constant

         do l = 1, 2

            v => E%vec(l:2*n:2)

            do i = 1, n
               som = 0d0
               do ip = 1, nd
                  som = som+E%Bmat(ip,l,i,1)*sum(sigma(:,ip))
               end do  !
               v(i) = v(i)-som
            end do  ! i = 1, n

         end do  ! l = 1, 2

      else

!     --- Derivatives of basis functions not constant

         do l = 1, 2

            v => E%vec(l:2*n:2)

            do i = 1, n
               som = 0d0
               do ip = 1, nd
                  som = som+sum(E%Bmat(ip,l,i,:)*sigma(:,ip))
               end do  !
               v(i) = v(i)-som
            end do  ! i = 1, n

         end do  ! l = 1, 2

      end if  ! (  B%derivconst .and. nd==3 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( E%vec, n, 2, 'vec' )
         write(irefwr,*) 'End sepelinstress1'

      end if  ! ( debug )
      call erclos ( 'sepelinstress1' )

      end subroutine sepelinstress1
