      subroutine sepelmassmat
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 05-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the element mass matrix in case of one unknown per node
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     mass_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: applyupw
      double precision, pointer :: diagmass(:), rho(:)

!     applyupw       store E%applyupw temporarily
!     diagmass       Contains the element matrix in case of a diagonal mass
!                    matrix
!     rho            density rho cp
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELMASSMAT1  Fill the element mass matrix; one unknown per node
!                    Consistent mass matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelmassmat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelmassmat'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      rho => E%coeffs(:,E%indrhocp)

      if ( E%imas==1 ) then

!     --- imas = 1, diagonal mass matrix

         diagmass => E%mmat(:,1)
         diagmass = rho*B%w

      else if ( E%imas==2 ) then

!     --- imas = 2, non-diagonal mass matrix

         E%mmat = 0d0
         applyupw = E%applyupw
         if ( E%applyupw==1 ) E%applyupw = 0   ! use upwind
         E%submat => E%mmat
         call sepelmassmat1 ( rho )
         E%applyupw = applyupw

      end if  ! ( E%imas==1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelmassmat'

      end if  ! ( debug )
      call erclos ( 'sepelmassmat' )

      end subroutine sepelmassmat
