      subroutine sepelmassmat2 ( rho )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the element mass matrix in case of ndim unknowns per node
!     Consistent mass matrix
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     mass_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleelm
      use sepmodulebasefn
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: rho(B%m)

!     rho            i    density rho cp
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j, k
      double precision :: work(B%m), selem(B%n,B%n)

!     i              General loop variable
!     j              General loop variable
!     k              Counting variable
!     selem          Work array to store the local element matrix
!     work           work array to store rho*w
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The element mass matrix has elements
!
!     / rho phi_j phi_i d Omega
!
!     Hence m_ij = sum_k rho(x_k) w_k phi_j(x_k) phi_i(x_k)
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelmassmat2' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelmassmat2'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      work = rho*B%w

      if ( debug) call prinrl ( work, B%m, 'work' )

      if ( B%jdiag==1 ) then

!     --- jdiag=1, phi_ik = delta_ik

         do k = 1, E%ndim
            do i = 1, B%n
               j = k+E%ndim*(i-1)
               E%mmat(j,j) = work(i)
            end do  ! i = 1, B%n
         end do  ! k = 1, E%ndim

      else

!     --- jdiag#1, phi general

         do i = 1, B%n
            do j = 1, i
               selem(i,j) = sum ( work*B%phi(j,:)*B%phi(i,:) )
               selem(j,i) = selem(i,j)
            end do  ! j = 1, B%n
         end do  ! i = 1, B%n
         do k = 1, E%ndim
            do i = 1, B%n
               do j = 1, B%n
                  E%mmat(k+E%ndim*(i-1),k+E%ndim*(j-1)) = selem(i,j)
               end do  ! j = 1, B%n
            end do  ! i = 1, B%n
         end do  ! k = 1, E%ndim

      end if  ! ( B%jdiag==1 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelmassmat2'

      end if  ! ( debug )
      call erclos ( 'sepelmassmat2' )

      end subroutine sepelmassmat2
