      subroutine sepelmlinelas1 ( l1, Dw )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 03-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill subpart of element mass matrix for linear elastic problem
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     elasticity_equation
!     parabolic_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision :: Dw(E%dimD,E%dimD,B%m)
      integer :: l1

!     Dw             i    Dmat * w
!     l1             i    sequence number of sub matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: ip, iq, i, j, nd
      double precision, pointer :: s(:,:)
      double precision :: h, som

!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     ip             loop variable
!     iq             loop variable
!     j              Counting variable
!     nd             Dimension of D-matrix
!     s              Sub element matrix to be filled
!     som            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
!     PRINRL2        Print a real two-dimensional array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelmlinelas1' )
      debug = .false. .and. ioutp>=0
      nd = E%dimD

      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelmlinelas1'
         write(irefwr,1) 'l1, n, nd', l1, n, nd
         write(irefwr,*) 'derivconst', B%derivconst
         call prinrl ( B%w, m, 'w' )
         call prinrl1 ( E%Dmat, nd, nd, 'D')
         call prinrl1 ( Dw, nd, nd, 'Dw')
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      s => E%mat(l1:2*n:2,l1:2*n:2)

      if ( B%derivconst .and. nd==3 ) then

!     --- Derivatives of basis functions are constant

         do i = 1, n
            do j = 1, i
               som = 0d0
               do ip = 1, nd
                  do iq = 1, nd
                     h = sum(Dw(ip,iq,:))
                     som = som+h*E%Bmat(ip,l1,i,1)*E%Bmat(iq,l1,j,1)
                  end do  ! iq = 1, nd
               end do  ! ip = 1, nd
               s(i,j) = som
            end do  ! j = 1, i
         end do  ! i = 1, n

      else

!     --- Derivatives of basis functions are not constant

         do i = 1, n
            do j = 1, i
               som = 0d0
               do ip = 1, nd
                  do iq = 1, nd
                     h = sum(Dw(ip,iq,:))
                     som = som+sum(E%Bmat(ip,l1,i,:)*Dw(ip,iq,:)*&
                                   E%Bmat(iq,l1,j,:))
                  end do  ! iq = 1, nd
               end do  ! ip = 1, nd
               s(i,j) = som
            end do  ! j = 1, i
         end do  ! i = 1, n

      end if  ! ( B%derivconst )

1000  if ( debug ) then

!     --- Debug information

         call prinrl2 ( s, n, n, 's')
         write(irefwr,*) 'End sepelmlinelas1'

      end if  ! ( debug )
      call erclos ( 'sepelmlinelas1' )

      end subroutine sepelmlinelas1
