      subroutine sepelnumint
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Detect parameters n and m based on shape of element
!     as well as quadrature rule
!     Old routine: el0800inm
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulecomio
      use sepmoduleelm
      use sepmodulebasefn
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1497   Type of integration rule not yet available
!    1771   Shape number not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelnumint' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelnumint'
         write(irefwr,1) 'E%ishape, B%rule', E%ishape, B%rule
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( B%rule==1 ) B%jdiag = 1  ! Newton Cotes

      select case ( E%ishape )

      case(-10, 0)

!     --- point element

         B%n = 1
         B%m = 1

      case(1)

!     --- linear element

         B%n = 2     ! number of basis functions
         B%nd = 1    ! dimension of basis functions
         B%nvert = 2 ! number of vertices

         select case ( B%rule )

         case(1, 3)
            B%m = 2
         case(2)
            B%m = 1
         case(4, 5)
            B%m = 3
         case default
            iwarn = 1497
            B%m = 2
         end select  ! case ( B%rule )

      case(2)

!     --- Quadratic element

         B%n = 3     ! number of basis functions
         B%nd = 1    ! dimension of basis functions
         B%nvert = 2 ! number of vertices

         if ( B%rule==1 ) then
            B%m = 3
         else
            if ( B%rule==2 ) B%rule = 3
            B%m = B%rule-1
         end if  ! ( B%rule==1 )

      case(3)

!     --- linear triangle

         B%n = 3     ! number of basis functions
         B%nd = 2    ! dimension of basis functions
         B%nvert = 3 ! number of vertices

         select case ( B%rule )

         case(1, 5)
            B%m = 3
         case(2)
            B%m = 1
         case(3)
            B%m = 4
         case(4)
            B%m = 7
         case default
            iwarn = 1497
            B%m = 3
         end select  ! case ( B%rule )

      case(4)

!     --- Quadratic triangle

         B%n = 6     ! number of basis functions
         B%nd = 2    ! dimension of basis functions
         B%nvert = 3 ! number of vertices
         E%ishape = 4 ! set shape number to 4


         select case ( B%rule )

         case(1)
            B%m = 6
         case(2)
            B%m = 4
         case(3)
            B%m = 7
         case default
            iwarn = 1497
            B%m = 6
         end select  ! case ( B%rule )

      case(7)

!     --- Quadratic triangle

         if ( jtrans==0 .and. itype<900 ) then
            B%n = 6     ! number of basis functions
            E%ishape = 4 ! set shape number to 4
         else
            B%n = 7     ! number of basis functions
         end if  ! ( jtrans==0 )
         B%nd = 2    ! dimension of basis functions
         B%nvert = 3 ! number of vertices

         select case ( B%rule )

         case(1)
            B%m = B%n
         case(2)
            B%m = 4
         case(3)
            B%m = 7
         case default
            iwarn = 1497
            B%m = 6
         end select  ! case ( B%rule )

      case(5, 9)

!     --- Bi-linear quadrilateral

         B%n = E%npelm     ! number of basis functions
         B%nd = 2    ! dimension of basis functions
         B%nvert = 4 ! number of vertices

         if ( itype>=900 ) B%n = 4

         select case ( B%rule )

         case(1)
            B%m = B%n
         case(2)
            B%m = 1
         case(3)
            B%m = 4
         case(4)
            B%m = 9
         case default
            iwarn = 1497
            B%m = B%n
         end select  ! case ( B%rule )

      case(6)

!     --- Bi-quadratic quadrilateral

         B%n = 9     ! number of basis functions
         B%nd = 2    ! dimension of basis functions
         B%nvert = 4 ! number of vertices

         select case ( B%rule )

         case(2)
           B%m = 4
         case(1,3)
            B%m = 9
         case default
            iwarn = 1497
            B%m = 4
         end select  ! case ( B%rule )

      case(10)

!     --- Extended linear triangle

         B%n = 4     ! number of basis functions
         B%nd = 2    ! dimension of basis functions
         B%nvert = 3 ! number of vertices

         select case ( B%rule )

         case(1, 2)
            B%m = 4
         case(3)
            B%m = 7
         case default
            iwarn = 1497
            B%m = 4
         end select  ! case ( B%rule )

      case(11)

!     --- linear tetrahedron

         B%n = 4     ! number of basis functions
         B%nd = 3    ! dimension of basis functions
         B%nvert = 4 ! number of vertices

         select case ( B%rule )
         case(1, 3)
            B%m = 4
         case(2)
            B%m = 1
         case(4)
            B%m = 5
         case(5)
            B%m = 11
         case(6)
            B%m = 15
         case default
            iwarn = 1497
            B%m = 4
         end select  ! case ( B%rule )

      case(12, 15)

!     --- Quadratic tetrahedron

         B%n = 10    ! number of basis functions
         B%nd = 3    ! dimension of basis functions
         B%nvert = 4 ! number of vertices
         select case ( B%rule )

         case(1)
            B%m = 10
         case(2, 3)
            B%m = 11
         case(4)
            B%m = 15
         case default
            iwarn = 1497
            B%m = 10

         end select  ! case ( B%rule )

      case(13)

!     --- Tri-linear hexahedron

         B%n = 8     ! number of basis functions
         B%nd = 3    ! dimension of basis functions
         B%nvert = 8 ! number of vertices

         select case ( B%rule )

         case(1, 3)
            B%m = 8
         case(2)
            B%m = 1
         case default
            iwarn = 1497
            B%m = 8
         end select  ! case ( B%rule )

      case(14)

!     --- Tri-quadratic hexahedron

         B%n = 27    ! number of basis functions
         B%nd = 3    ! dimension of basis functions
         B%nvert = 8 ! number of vertices

         select case ( B%rule )

         case(1, 3)
            B%m = 27
         case(2)
            B%m = 8
         case default
            iwarn = 1497
            B%m = 27
         end select  ! case ( B%rule )

      case(16)

!     --- teterhedron with 15 points

         B%n = 15     ! number of basis functions
         B%nd = 3    ! dimension of basis functions
         B%nvert = 4 ! number of vertices

         select case ( B%rule )

         case(1, 3)
            B%m = 15
         case(2)
            B%m = 11
         case default
            iwarn = 1497
            B%m = 15
         end select  ! case ( B%rule )

      case(17)

!     --- hexahedron with 9 points

         B%n = 9     ! number of basis functions
         B%nd = 3    ! dimension of basis functions
         B%nvert = 8 ! number of vertices
         B%rule = 3
         if ( B%rule==3 ) B%m = 8

      case(18)

!     --- tetrahedron with 5 points

         B%n = 5     ! number of basis functions
         B%nd = 3    ! dimension of basis functions
         B%nvert = 4 ! number of vertices

         select case ( B%rule )
         case(3)
            B%m = 4
         case default
            B%m = 5
         end select  ! case ( B%rule )

      case(21)

!     --- linear wedge element

         B%n = 6     ! number of basis functions
         B%nd = 3    ! dimension of basis functions
         B%nvert = 6 ! number of vertices

         select case ( B%rule )

         case(1)
            B%m = 6
         case(2)
            B%m = 1
         case(3)
            B%m = 8
         case default
            iwarn = 1497
            B%m = 6
         end select  ! case ( B%rule )

      case(22)

!     --- quadratic wedge element

         B%n = 18     ! number of basis functions
         B%nd = 3    ! dimension of basis functions
         B%nvert = 6 ! number of vertices

         select case ( B%rule )

         case(1)
            B%m = 18
         case(2)
            B%m = 8
         case(3)
            B%m = 21
         case default
            iwarn = 1497
            B%m = 18
         end select  ! case ( B%rule )

      case default

!     --- Error 1771: this type of element has not yet been implemented

         call errint ( E%ishape, 1 )
         call errint ( E%ielem, 2 )
         call errint ( E%itype, 3 )
         call errint ( E%ielgrp, 4 )
         call errsub ( 1771, 4, 0, 0 )

      end select  ! case ( E%ishape )

      if ( iwarn==1497 ) then

!     --- Integration rule not yet implemented

         call errint ( B%rule, 1 )
         call errwar ( 1497, 1, 0, 0 )
         B%jdiag = 1
         B%rule = 1
         iwarn = 0

      end if  ! ( iwarn==1497 )

      if ( B%rule>1 ) B%fillgauss = .true.

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'm, n, nd, nvert', B%m, B%n, B%nd, B%nvert
         write(irefwr,1) 'rule', B%rule
         write(irefwr,*) 'End sepelnumint'

      end if  ! ( debug )
      call erclos ( 'sepelnumint' )

      end subroutine sepelnumint
