      subroutine sepeloldvecs1 ( ndegfd, vecloc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array vecloc in case of vectors of type 126
!     ndegfd is set to 0
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     coefficient
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(out) :: ndegfd
      double precision, intent(out) :: vecloc(m,*)

!     ndegfd         o    Number of degrees of freedom per point
!     vecloc         o    Local vector defined per element
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: j, k
      double precision :: work(n)

!     j              General loop variable (over degrees of freedom)
!     k              General loop variable (over integration points)
!     work           work array of length n in which one component of
!                    one vector in all nodes is stored
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeloldvecs1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeloldvecs1'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      ndegfd = index4n(1)
      do j = 1, ndegfd

         do k = 1, E%npelm
            work(k) = vecold(index3n((k-1)*ndegfd+j))
         end do

!        --- Copy information from array work into vecloc

         if ( B%jdiag==1 ) then

!        --- jdiag = 1, integration points are nodal points

            vecloc(1:E%npelm,j) = work(1:E%npelm)

         else

!        --- jdiag = 2, integration points and nodal points
!                       are different

            if ( n==E%npelm+1 ) then

               if ( n==7 ) then
                  work(7) = (-work(1)-work(3)-work(5) + &
                             4d0*(work(2)+work(4)+work(6)) )/ 9d0
               else if ( n==4 ) then
                  work(4) = (work(1)+work(2)+work(3))/3d0
               else if ( n==5 ) then
                  work(5) = (work(1)+work(2)+work(3)+work(4))* 0.25d0
               end if  ! ( n==7 )

            end if  ! ( n==E%npelm+1 )

            do k = 1, m
               vecloc(k,j) = dot_product ( work(1:n), B%phi(1:n,k))
            end do  ! k = 1, m

         end if  ! ( B%jdiag==1 )

      end do  ! j = 1, ndegfd
      ndegfd = 0

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeloldvecs1'

      end if  ! ( debug )
      call erclos ( 'sepeloldvecs1' )

      end subroutine sepeloldvecs1
