      subroutine sepeloldvecs5 ( ndegfd, ivec )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 24-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute ndegfd
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     coefficient
!     vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulekmeshv
      use sepmoduleelm
      use sepmodulebuild
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: ivec
      integer, intent(out) :: ndegfd

!     ivec           i    Sequence number of vector in set of old vectors
!     ndegfd         o    Number of degrees of freedom per point
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical :: centr, quadrat
      integer :: i

!     centr          If true the element contains a centroid
!     i              Counting variable
!     quadrat        If true the elements are quadratic
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     PRININ         print 1d integer array
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1774   Only constant numbers of degrees of freedom permitted
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeloldvecs5' )
      debug = .false.! .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeloldvecs5'
         write(irefwr,1) 'ishape', ishape
         call prinin ( index4n, E%npelm, 'index4n' )
  1      format ( a, 1x, (10i8) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      quadrat = ishape==2 .or. ishape==4 .or. ishape==6 .or. &
                ishape==7 .or. ishape==12 .or. &
                ishape>=14 .and. ishape<=16
      centr = ishape==2 .or. ishape>=6 .and. ishape<=10 .or. &
              ishape==14 .or. ishape>=16 .and. ishape<=18
      if ( debug ) write(irefwr,*) 'centr, quadrat ', centr, quadrat

      if ( quadrat ) then

!     --- Quadratic elements, check if one of the following three
!         options is present:
!         value is available in all but the centroid point
!         value is available in the vertices
!         not one of these options

         ndegfd = index4n(1)

      else if ( centr ) then

!     --- centroid is last point

         ndegfd = index4n(1)
         if ( ndegfd<=0 ) then
            call errint ( ivec, 1 )
            call errsub ( 1774, 1, 0, 0 )
            go to 1000
         end if  ! ( ndegfd<=0 )

      else

!     --- Error 1774:  Only constant number of unknowns per point are
!                      permitted

         call errint ( ivec, 1 )
         call errsub ( 1774, 1, 0, 0 )
         go to 1000

      end if  ! ( quadrat )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ndegfd', ndegfd
         write(irefwr,*) 'End sepeloldvecs5'

      end if  ! ( debug )
      call erclos ( 'sepeloldvecs5' )

      end subroutine sepeloldvecs5
