      subroutine sepelp04jacob ( curved )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 01-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute Jacobian and derivatives of basis functions
!     Quadratic triangle (6 nodes)
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     quadratic
!     triangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical, intent(out) :: curved

!     curved         indication if the element is curved (true) or not (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: x(:), y(:)

!     curved         indication if the element is curved (true) or not (false)
!     x              Contains x co-ordinates of nodes
!     y              Contains y co-ordinates of nodes
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
!     SEPELP04JACCUR Compute basis functions and related quantities for
!                    quadratic triangle (curved element)
!     SEPELP04JACSTR Compute basis functions and related quantities for
!                    quadratic triangle (straight element)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp04jacob' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp04jacob'
         write(irefwr,1) 'kstep', B%kstep
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      x => B%xyz(:,1)
      y => B%xyz(:,2)

!     --- check if element is straight or curved

      if ( E%ndim>2 .and. (E%iqmat==0 .or. E%iqmat==3) ) then

!     --- Standard 3D case. Treat as curved

         curved = .true.

      else

!     --- 2D case, compute curved

         curved = .false.
         if ( abs(x(2)+x(4)+x(6)-x(1)-x(3)-x(5)) > sqreps ) then
            curved = .true.
         else if ( abs(y(2)+y(4)+y(6)-y(1)-y(3)-y(5)) > sqreps ) then
            curved = .true.
         end if

      end if
      if ( debug ) write(irefwr,*) 'curved ', curved

      if ( .not. curved ) then

!     --- curved = false,  straight triangle

         call sepelp04jacstr

      else

!     --- curved = true,  curved triangle

         call sepelp04jaccur

      end if  ! ( .not. curved )

1000  call erclos ( 'sepelp04jacob' )
      if ( debug ) then

!     --- Debug information

         call prinrl1 ( B%phider(1,1,1), B%m, 6, 'dphidx' )
         call prinrl1 ( B%phider(1,1,2), B%m, 6, 'dphidy' )
         write(irefwr,*) 'End sepelp04jacob'

      end if  ! ( debug )

      end subroutine sepelp04jacob
