      subroutine sepelp12jacob ( jac, intern, imidp )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 15-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute Jacobian and derivatives of basis functions
!     Quadratic tetrahedrons
!     Old subroutine elp631
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     tetrahedron
!     quadratic
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: intern(10), imidp(6,2)
      double precision :: jac(*)

!     imidp          i    Contains the i,j numbering of the mid side points in
!                         the sequence of intern
!                         So basis function
!                         phi(intern(4+i)) = 4 lambda(imidp(i,1))
!                                              lambda(imidp(i,2))
!     intern         i    Internal numbering of the nodes, vertices first
!     jac            o    Array of length m containing the values of the
!                         Jacobian in the vertices or for the Gauss integration
!                         rules work array to store some help variables
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical :: curved

!     curved         indication if the element is curved (true) or not
!                    (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      logical :: sepelp12cur

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELP12CUR    Check if quadratic tetrahedron is curved
!     SEPELP12JACSTR Compute basis functions and related quantities for
!                    quadratic tetrahedron (straight element)
!     SEPELP13JACOB  Compute Jacobian and derivatives of basis functions
!                    3d Tri-linear hexahedron (8 nodes)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp12jacob' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp12jacob'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- check if element is straight or curved

      curved = sepelp12cur ( intern, imidp )
      if ( debug ) write(irefwr,*) 'curved ', curved

      if ( .not. curved ) then

!     --- curved = false,  straight triangle

         call sepelp12jacstr ( jac, intern, imidp )

      else

!     --- curved = true,  curved triangle

         call sepelp13jacob ( jac )

      end if  ! ( .not. curved )

1000  call erclos ( 'sepelp12jacob' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp12jacob'

      end if  ! ( debug )

      end subroutine sepelp12jacob
