      subroutine sepelp17phixi ( phixi, n, m, m1, theta, dtetds, indi, inds )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 25-06-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute dphi/dxi in quadrature points
!     3d Extended tri-linear hexahedron (9 nodes)
!     Old subroutine: elp636
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     hexahedron
!     linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: n, m, m1, indi(8,3), inds(m,3)
      double precision, intent(in) :: theta(m1,m1), dtetds(m1,m1)
      double precision, intent(out) :: phixi(n,m,3)

!     dtetds         i    Derivatives of theta in the integration points
!     indi           i    Array in which the triples indi, indj and indk are
!                         stored
!     inds           i    Array in which the triples inds1, inds2 and inds3 are
!                         stored
!     m              i    number of quadrature points
!     m1             i    number of nodes in 1 direction
!     n              i    number of basis functions
!     phixi          o    Array of length 8 x m x 3 containing the values of the
!                         derivatives of the basis functions in the Gauss points
!                         in the reference element.
!     theta          i    1D basis funtion in integration points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, k
      double precision :: hc(3), dpdxic

!     dpdxic         Value of dphi_c/dxi
!     hc             Help array to store a quantity temporarily
!     i              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelp17phixi' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelp17phixi'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000
      do k = 1, m
         dpdxic = (2d0*theta(1,inds(k,1))-1d0)* &
                  theta(1,inds(k,2))*theta(2,inds(k,2))* &
                  theta(1,inds(k,3))*theta(2,inds(k,3))
         hc(1) = 8d0*dpdxic
         phixi(9,k,1) = 64d0*dpdxic
         dpdxic = (2d0*theta(1,inds(k,2))-1d0)* &
                  theta(1,inds(k,3))*theta(2,inds(k,3))* &
                  theta(1,inds(k,1))*theta(2,inds(k,1))
         hc(2) = 8d0*dpdxic
         phixi(9,k,2) = 64d0*dpdxic
         dpdxic = (2d0*theta(1,inds(k,3))-1d0)* &
                  theta(1,inds(k,1))*theta(2,inds(k,1))* &
                  theta(1,inds(k,2))*theta(2,inds(k,2))
         hc(3) = 8d0*dpdxic
         phixi(9,k,3) = 64d0*dpdxic
         do i = 1, 8
            phixi(i,k,1) = dtetds(indi(i,1),inds(k,1))* &
                           theta(indi(i,2),inds(k,2))* &
                           theta(indi(i,3),inds(k,3)) - hc(1)
            phixi(i,k,2) = theta(indi(i,1),inds(k,1))* &
                           dtetds(indi(i,2),inds(k,2))* &
                           theta(indi(i,3),inds(k,3)) - hc(2)
            phixi(i,k,3) = theta(indi(i,1),inds(k,1))* &
                           theta(indi(i,2),inds(k,2))* &
                           dtetds(indi(i,3),inds(k,3)) - hc(3)
         end do
      end do

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelp17phixi'

      end if  ! ( debug )
      call erclos ( 'sepelp17phixi' )

      end subroutine sepelp17phixi
