      subroutine sepelpgauss
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-08-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill coordinates of Gauss points
!
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulebasefn
      use sepmodulecomio
      use sepmoduleelm
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, k

!     i              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelpgauss' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelpgauss'
         write(irefwr,*) 'fillgauss, first', B%fillgauss, E%first
         write(irefwr,1) 'n, m, jtrans, npelm', n, m, jtrans, E%npelm
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      n = min(B%n,E%npelm)
      if ( jtrans==1 ) n = B%n

      if ( B%fillgauss ) then

!     --- Gauss points must be filled

         do i = 1, E%ndim
            do k = 1, m
               B%xyzg(k,i) = dot_product ( B%xyz(1:n,i),B%philoc(1:n,k) )
            end do  ! k = 1, m
         end do  ! i = 1, ndim

      else

!     --- Gauss points are the standard nodes
!         The number of nodes may differ from the number of Gauss points
!         in case nodes are skipped

         B%xyzg(1:n,:) = B%xyz(1:n,:)

      end if  ! ( B%fillgauss )
      n = B%n

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( B%xyzg, m, E%ndim, 'xgauss')
         write(irefwr,*) 'End sepelpgauss'

      end if  ! ( debug )
      call erclos ( 'sepelpgauss' )

      end subroutine sepelpgauss
