      subroutine sepelphixphixa ( phix1, phix2, Dw, S )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 06-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill part of element matrix / C dphi_i/dx_k dphi_j/dx_l dOmega
!     Add to S-matrux
!     Symmetric matrix
!     Old subroutine el3011
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision, intent(in) :: Dw(m), phix1(n,m), phix2(n,m)
      double precision, intent(out) :: S(n,n)

!     Dw             i    Dmat * w
!     S              o    sub part of element matrix
!     phix1          i    Either dphi/dx or dphi/dy
!     phix2          i    Either dphi/dx or dphi/dy
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: h
      integer :: i, j

!     h              Help variable to store a constant temporarily
!     i              Counting variable
!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelphixphixa' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelphixphixa'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( B%derivconst ) then

!     --- Derivatives of basis functions constant

         h = sum(Dw(:))
         do i = 1, n
            do j = 1, i
               S(i,j) = S(i,j)+h*phix1(i,1)*phix2(j,1)
            end do  ! j = 1, i
         end do  ! i = 1, n

      else

!     --- Derivatives of basis functions are variable

         do i = 1, n
            do j = 1, i
               S(i,j) = S(i,j)+sum(Dw(:)*phix1(i,:)*phix2(j,:))
            end do  ! j = 1, i
         end do  ! i = 1, n

      end if  ! ( B%derivconst )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelphixphixa'

      end if  ! ( debug )
      call erclos ( 'sepelphixphixa' )

      end subroutine sepelphixphixa
