      subroutine sepelpup31 ( h, unorm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute distance h for modified basis functions for upwind elements
!     Three dimensional elements (linear tetrahedrons)
!     Old subroutine elpup31
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     upwind
!     3d
!     linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision h(B%m), unorm(B%m)

!     h              o    Contains the approximation of the distance
!                         h in the integration points
!     unorm          i    Length of velocity vector in the integration points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i
      double precision :: phimax
      double precision, pointer :: dlamdx(:,:,:), u(:,:)

!     dlamdx         Contains derivatives of linear basis functions
!     i              Counting variable
!     phimax         Max_j( |u_i . grad phi_j| )
!     u              Velocities in quadrature points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!       h = max distance in element in the direction of u
!                                                       -
!       For a tetrahedron:  h_i  = ||u_i ||_j / max (|u_i . grad_j lambda|)
!       in the vertices
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelpup31' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelpup31'
         call prinrl ( unorm, B%m, 'unorm' )
         call prinrl1 ( B%lamder, 3, 4, 'dlamdx' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      dlamdx => B%lamder
      u => E%veloc

      do i = 1, B%m
         phimax = max ( abs(u(i,1)*dlamdx(1,i,1)+ u(i,2)*dlamdx(1,i,2)+ &
                          u(i,3)*dlamdx(1,i,3)), &
                      abs(u(i,1)*dlamdx(2,i,1)+ u(i,2)*dlamdx(2,i,2)+ &
                          u(i,3)*dlamdx(2,i,3)), &
                      abs(u(i,1)*dlamdx(3,i,1)+ u(i,2)*dlamdx(3,i,2)+ &
                          u(i,3)*dlamdx(3,i,3)), &
                      abs(u(i,1)*dlamdx(4,i,1)+ u(i,2)*dlamdx(4,i,2)+ &
                          u(i,3)*dlamdx(4,i,3)) )
         if ( phimax.gt.epsmac ) then
            h(i) = unorm(i)/phimax
         else
            h(i) = 0d0
         end if
      end do

1000  call erclos ( 'sepelpup31' )
      if ( debug ) then

!     --- Debug information

         call prinrl ( h, B%m, 'h' )
         write(irefwr,*) 'End sepelpup31'

      end if  ! ( debug )

      end subroutine sepelpup31
