      subroutine sepelpupwdist ( h, unorm )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-09-2015
!
!   copyright (c) 2015-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute distance h for modified basis functions for upwind elements
!     Three dimensional elements
!     Old subroutine elpupwdist
! **********************************************************************
!
!                       KEYWORDS
!
!     basis_function
!     element
!     upwind
!     3d
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision h(B%m), unorm(B%m)

!     h              o    Contains the approximation of the distance
!                         h in the integration points
!     unorm          i    Length of velocity vector in the integration points
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: xcent(6,3)
      double precision, pointer :: x(:,:)
      integer :: j

!     j              Counting variable
!     x              Contains the coordinates of the nodes
!     xcent          Contains the coordinates of the vertices
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPELPUP3      Compute distance h for modified basis functions for upwind
!                    elements
!                    Three dimensional elements (linear tetrahedrons)
!     SEPELPUP31     Compute distance h for modified basis functions for upwind
!                    elements
!                    Three dimensional elements (linear tetrahedrons)
!     SEPELPUP4      Compute distance h for modified basis functions for upwind
!                    elements
!                    Three dimensional elements (trilinear hexahedrons)
!     SEPELPUP5      Compute distance h for modified basis functions for upwind
!                    elements
!                    Three dimensional elements, linear prisms
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelpupwdist' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelpupwdist'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      x => B%xyz

      select case ( B%type_basisfunc )

      case(11)

!     --- Linear tetrahedron

         call sepelpup3 ( h, unorm )

      case(12)

!     --- Quadratic tetrahedron

         call sepelpup31 ( h, unorm )

      case(13)

!     --- Trilinear hexahedron

         do j = 1, 3
            xcent(1,j) = 0.25d0 * ( x(1,j)+x(2,j)+x(3,j)+x(4,j) )
            xcent(2,j) = 0.25d0 * ( x(5,j)+x(6,j)+x(7,j)+x(8,j) )
            xcent(3,j) = 0.25d0 * ( x(1,j)+x(2,j)+x(6,j)+x(5,j) )
            xcent(4,j) = 0.25d0 * ( x(4,j)+x(3,j)+x(7,j)+x(8,j) )
            xcent(5,j) = 0.25d0 * ( x(1,j)+x(4,j)+x(8,j)+x(5,j) )
            xcent(6,j) = 0.25d0 * ( x(2,j)+x(3,j)+x(7,j)+x(6,j) )
         end do
         call sepelpup4 ( xcent, h, unorm )

      case(14)

!     --- Triquadratic hexahedron

         do j = 1, 3
            xcent(1,j) = x(5,j)
            xcent(2,j) = x(23,j)
            xcent(3,j) = x(11,j)
            xcent(4,j) = x(17,j)
            xcent(5,j) = x(13,j)
            xcent(6,j) = x(15,j)
         end do
         call sepelpup4 ( xcent, h, unorm )

      case(21)

!     --- Trilinear prism

         call sepelpup5 ( h, x, unorm )

      case(22)

!     --- Triquadratic prism

         do j = 1, 3
            xcent(1,j) = x(1,j)
            xcent(2,j) = x(3,j)
            xcent(3,j) = x(5,j)
            xcent(4,j) = x(13,j)
            xcent(5,j) = x(15,j)
            xcent(6,j) = x(17,j)
         end do
         call sepelpup5 ( h, xcent, unorm )

      case default

!     --- Value of type_basisfunc not yet implemented

         call errint ( B%type_basisfunc, 1 )
         call errsub ( 800, 1, 0, 0 )
         go to 1000

      end select  ! case ( B%type_basisfunc )

1000  call erclos ( 'sepelpupwdist' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelpupwdist'

      end if  ! ( debug )

      end subroutine sepelpupwdist
