      subroutine sepeltypeconv
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 17-11-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill quantities with respect to convection
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     convection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleoldvecs
      use sepmodulestrings
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepelgetintcf

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELGETINTCF  Return with the actual integer value corresponding to
!                    iuser(ipos)
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepeltypeconv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepeltypeconv'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Read apply_upwind

      E%applyupw = sepelgetintcf ( codeapply, nncodeapply, &
                                   ncodeapply, 1, 'apply_upwind' )

      if ( debug ) write(irefwr,1) 'ipos, applyupw', E%ipos, E%applyupw

      if ( E%methodupw>=7 .and. E%methodupw<=11 .or. E%typeconv>1 ) then

!     --- Non-linear upwind or non-linear convection,
!         the derivative of the prior vector is needed

         E%filloldvec = .true.  ! fill prior vector
         E%filloldderiv = .true.  ! fill derivative of prior vector
         E%compold = 1
         indvector(iseqprevsol) = 1

      end if  ! ( E%methodupw>=7 .and. E%methodupw<=8 )

!     --- Set logical convection

      E%convection = E%ind(E%indvel)/=0 .or. B%nd>1 .and. &
                     E%ind(E%indvel+1)/=0 .or. B%nd>2 .and. &
                     E%ind(E%indvel+2)/=0 .or. E%typeconv>1

      if ( B%fillqmat .and. .not. E%convection ) &
         E%convection = E%ind(E%indvel)/=0 .or. E%ind(E%indvel+1)/=0 .or. &
                        E%ind(E%indvel+2)/=0

      E%veloc => E%coeffs(:,E%indvel:E%indvel+2)
      E%uvel => E%coeffs(:,E%indvel)

!     --- Define when linear basis functions must be computed

      if ( E%methodupw>0 .and. E%applyupw<2 ) B%comppsi = .true.

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepeltypeconv'

      end if  ! ( debug )
      call erclos ( 'sepeltypeconv' )

      end subroutine sepeltypeconv
