      subroutine el0900 ( iuser, user, jelp, jtime, penalp, cn, clamb, &
                          mconv, mcont, modelv, isub, isup, &
                          coeffintpnts, thetdt, nveloc, npres, imesh, &
                          mdiv, mcoefconv, modelrhsd, ioldm, coeffnodes )
! ======================================================================
!
!        programmer    Guus Segal
!        version 13.0  date 06-01-2016 Use sepmoduleelem
!        version 12.0  date 27-02-2010 Extra parameters
!        version 11.0  date 21-12-2008 Remove parameter iarlagr
!
!   copyright (c) 1993-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     This subroutine performs the initializations for subroutine ELM900 or
!     ELD900 in the case that IFIRST = 0
!     The parameters in the common blocks celint, celiar and celp
!     get a start value
!     array coeffintpnts is initialized with the values of the coefficients
!     If this subroutine is changed, also adapt el0260
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision user(*), coeffintpnts(m,*), penalp, cn, clamb, &
                       thetdt, coeffnodes(n,*)
      integer iuser(*), jelp, jtime, isub, isup, mconv, modelv, &
              mcont, nveloc, npres, imesh, mdiv, &
              mcoefconv, modelrhsd, ioldm

!     clamb          o    Parameter lambda with respect to viscosity model
!     cn             o    Power in power law model
!     coeffintpnts   o    Real array in which the values of the
!                         coefficients in the integration points are stored
!                           in the sequence:
!                           In the case of eld900 or elm900:
!                                2D                  3D
!                           1:   rho              rho
!                           2:   omega            omega
!                           3:   f1               f1
!                           4:   f2               f2
!                           5:                    f3
!                           6:   eta              eta
!                           7:   rho in nodal points (mcont=1 only)
!                           7:   G_L              G_L (mcont=2 only)
!                           8:   kappa            kappa
!                           9:   fdiv             fdiv
!                            In the standard case 6 stress coefficients
!                          10:   fstress_xx       fstress_xx
!                          11:   fstress_yy       fstress_yy
!                          12:   fstress_zz       fstress_zz
!                          13:   fstress_xy       fstress_xy
!                          14:   fstress_yz       fstress_yz
!                          15:   fstress_zx       fstress_zx
!                            In the special case of the dimensionless equations:
!                          10:   fstress          fstress
!                          11:   c_div_1
!                          12:   c_div_2
!                          13:   -
!                          14:   c_grad
!                          15:   c_conv_1
!                          16:   c_conv_2
!                          17:   c_conv_3
!                          18:   c_conv_4
!                          19:   -
!                          20:   -
!                          21:   -
!                          22:   -
!                          23:   -
!                          24:   c_visc_1
!                          25:   c_visc_2
!                          26:   c_visc_3
!                          27:   c_visc_4
!                          28:   -
!                          29:   -
!                          30:   -
!                          31:   -
!                          32:   -
!                          33:   gamma for AL method
!                          34:   gamma_nu for AL method
!                           In the case of eli900:
!                           1:   f
!     coeffnodes     o    Real array in which the values of the
!                         coefficients in the nodal points are stored
!     imesh          o    Sequence number of mesh velocity in input vector
!                         If 0 no mesh velocity is present
!     ioldm          o    Sequence number of velocity at end of previous time
!                         step in input vector.
!                         If 0 no iterative procedure is adopted for dynamical
!                         analyses.
!     isub           o    Integer help parameter for do-loops to indicate the
!                         smallest coefficient number that is dependent on space
!     isup           o    Integer help parameter for do-loops to indicate the
!                         largest coefficient number that is dependent on space
!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
!     jelp           o    Indication of the type of basis function subroutine
!                         must be called by ELM100. Possible vaues:
!                         1:  Subroutine ELP610  (Linear line element)
!                         2:  Subroutine ELP611  (Quadratic line element)
!                         3:  Subroutine ELP620  (Linear triangle)
!                         4:  Subroutine ELP622  (Quadratic triangle)
!                         5:  Subroutine ELP621  (Bilinear quadrilateral)
!                         6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                        11:  Subroutine ELP630  (Linear tetrahedron)
!                        12:  Subroutine ELP631  (Quadratic tetrahedron)
!                        13:  Subroutine ELP632  (Trilinear brick)
!                        14:  Subroutine ELP633  (Triquadratic brick)
!     jtime          o    Integer parameter indicating if the mass matrix for
!                         the time-dependent equation must be computed (>0)
!                         or not (=0)
!                         Possible values:
!                           0:  no mass matrix
!                           1:  diagonal mass matrix with constant coefficient
!                           2:  diagonal mass matrix with variable coefficient
!                          11:  full mass matrix with constant coefficient
!                          12:  full mass matrix with variable coefficient
!                         101:  refers to the special possibility of the theta
!                               method directly applied
!                         102:  refers to the special possibility of the theta
!                               method directly applied with variable rho
!     mcoefconv      o    Defines how the convetive term is treated:
!                         0: standard case
!                         1: special case: the convective term is defined as
!                               c_conv_1 u du/dx + c_conv_2 v du/dy
!                               c_conv_3 u dv/dx + c_conv_4 v dv/dy
!     mcont          o    Defines the type of continuity equation
!                         Possible values:
!                         0: incompressible flow
!                         1: compressible flow: div (rho u ) = 0
!                         2: Goertler equations
!                         4: the continuity equation contains a pressure term
!                         5: special option for pressure mass matrix
!                            see elm900
!     mconv          o    Defines the treatment of the convective terms
!                         Possible values:
!                         0: convective terms are skipped (Stokes flow)
!                         1: convective terms are linearized by Picard iteration
!                         2: convective terms are linearized by Newton iteration
!                         3: convective terms are linearized by the incorrect
!                            Picard iteration
!                         4: convective terms are linearized by successive
!                            substitution
!     mdiv           o    Defines if divergence right-hand side is used (1)
!                         or not (0)
!     modelrhsd      o    Defines how the stress tensor is treated in the rhs
!                         Possible values:
!                         0:    if there is a given stress tensor the reference
!                               to this tensor is stored in position 19
!                         1:    It is supposed that there are 6 given stress
!                               tensor components stored in positions 19-24
!     modelv         o    Defines the type of viscosity model
!                         Possible values:
!                           1:  newtonian liquid
!                           2:  power law liquid
!                           3:  carreau liquid
!                           4:  plastico viscous liquid
!                           5:    special case: the viscous term is defined as
!                                 c_visc_1 d^2 u /dx^2 + c_visc_2 d^2 u /dy^2
!                                 c_visc_3 d^2 v /dx^2 + c_visc_4 d^2 v /dy^2
!                          10:  simplified newtonian liquid, &
!                               with constant viscosity
!                         100:  user provided model depending on grad u
!                         101:  user provided model depending on the second
!                               invariant
!                         102:  user provided model depending on the second
!                               invariant
!                               the co-ordinates and the velocity
!                         103:  user provided model depending on the second
!                               invariant
!                               the co-ordinates, the velocity, and old
!                               solutions
!     npres          o    Number of pressure parameters
!     nveloc         o    Number of velocity parameters
!     penalp         o    Parameter eps for the penalty function formulation
!     thetdt         o    1/(theta dt) for the case of the theta method
!                         immediately applied
!     user           i    Real user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ncoef, kchois, mrule, interpol, jcheld, ishift
      logical compress

!     compress       If true the pressure must be computed
!     interpol       If 0 the standard interpolation is applied, &
!                    if 1 the interpolation is restricted to linear subelements
!     ishift         Shift in coefficient sequence number
!     jcheld         Value of icheld corrected for different storage schemes
!     kchois         Indication what type of parameters must be computed
!                    Possible values:
!                    0:  Standard method, the calling subroutine is ELM900
!                        It is supposed that an element matrix must be computed
!                    1:  Subroutine ELD900 calls this subroutine, only
!                        derivatives must be computed
!                    2:  Subroutine ELI900 calls this subroutine, only
!                        integrals must be computed
!     mrule          Type of integration rule as given by the user
!     ncoef          Number of coefficients in the equation
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0900COEFF    Initialize coefficients for elements of type 900-902
!                    and 916-918
!     EL0900FLCOEFFS Initialize coefficients for elements of type 900-902
!                    and 916-918
!     ELM900PARMS1   Compute parameters irule, m and n for type elements of
!                    type 900
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRINRL         Print 1d real vector
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     See subroutines elm900, eld900 and eli900 for a description
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Fill some standard values in the common blocks celp and celint

      call eropen ( 'el0900' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0900'
         write(irefwr,1) 'imas, ishape', imas, ishape
  1      format ( a, 1x, (10i6) )
         call prinin ( iuser, 30, 'iuser' )
         call prinrl ( user, 10, 'user' )

      end if
      if ( ierror/=0 ) go to 1000

!     --- First part, compute parameters

      call elm900parms1 ( iuser, interpol, ioldm, ishift, jcheld, &
                          jelp, jtime, kchois, nveloc, mcont, &
                          modelrhsd, modelv, mrule, ncoef, &
                          thetdt, compress )

!     --- extra initializations

      mcoefconv = 0
      npres  = 1

      if ( itype==902 ) then
         if ( inpelm==27 ) jind = -2
         if ( ishape==6 .or. ishape==7 .or.ishape==14 .or. &
              ishape==16 ) &
            npres = ndim+1
         if ( icheld>0 .and. ncoef<=1 ) then
            kchois = 1
            ncoef = 0
         end if
      else
         jind = 1
      end if

!     --- Fill the arrays IND, MST and array with information of the
!         coefficients

      call el0900coeff ( ncoef, kchois, iuser, user, jtime, &
                         thetdt, jcheld, compress, penalp, &
                         ioldm, mcoefconv, mconv, mcont, &
                         modelrhsd, modelv, interpol, itype, ishift, &
                         mrule )
      if ( ierror/=0 ) go to 1000

!     --- Fill coefficients in coeffintpnts and do some checks

      call el0900flcoeffs ( ncoef, modelv, mcont, mcoefconv, &
                            coeffintpnts, jtime, jcheld, cn, clamb, &
                            kchois, iuser, modelrhsd, mconv, &
                            imesh, isub, isup, mdiv, &
                            user, ishift, coeffnodes )
      if ( ierror/=0 ) go to 1000

      irule = irule + 100*interpol

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'jelp, ishape, m, n, jcart, irule', &
                          jelp, ishape, m, n, jcart, irule
         write(irefwr,1) 'igausp, ndim, ncomp', igausp, ndim, ncomp
         write(irefwr,1) 'nveloc, npres', nveloc, npres
         write(irefwr,*) 'End el0900'

      end if
      call erclos ( 'el0900' )

      end
