      subroutine el0900cons ( jtime, jconsf, ncomp, ndim, kchois, &
                              modelv, modelrhsd, mcont, mrule, jcheld, &
                              icheld, compress )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 08-06-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Set some constants for subroutines el0900 and el0905
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer jtime, jconsf, ncomp, ndim, kchois, modelv, modelrhsd, &
              mcont, mrule, jcheld, icheld
      logical compress

!     compress       o    If true the pressure must be computed
!     icheld         i    Choice parameter to distinguish between the various
!                         possibilities of this subroutine
!                         0:   Call by ELM100
!                         >0:  Call by ELD100
!                         <0:  Call by ELI100
!     jcheld         o    if ( icheld<= 10 ) then icheld else icheld-20
!     jconsf         o    Information of the weight function w and the function
!                         to be integrated g. Possibilities:
!                         1   g and w constant
!                         2   general
!     jtime          o    Integer parameter indicating if the mass matrix for
!                         the time-dependent equation must be computed (>0)
!                         or not ( = 0)
!     kchois         o    Indication what type of parameters must be computed
!     mcont          o    Defines the type of continuity equation
!     modelrhsd      o    Defines how the stress tensor is treated in the rhs
!     modelv         o    Defines the type of viscosity model
!     mrule          o    Type of integration rule as given by the user
!     ncomp          o    Number of components per node
!     ndim           i    Dimension of the space.
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el0900cons' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el0900cons'

      end if
      if ( ierror/=0 ) go to 1000

      jtime  = 0
      jconsf = 2
      ncomp  = ndim
      kchois = 0
      modelv = 0
      modelrhsd = 0
      mcont = 0
      mrule = 0
      jcheld = icheld
      compress = .false.

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el0900cons'

      end if
      call erclos ( 'el0900cons' )

      end
