      subroutine el4041 (ichois,xg,uold,phi,index1, &
                         nloc,mloc,ncomploc,iuser,user,ktime,method,sigma,rho, &
                         mconv,irotat,omega,fg,modelv,etha,cn,cl,ct, &
                         temp1, indeth,work,beta,temp0, &
                         cp,cappa,indf1,indf2,indf3,indt0,indold,ialpha)
! ======================================================================
!
!        programmer    Jaap van der Zanden
!        version  2.7  date 13-01-2015 Extension with models
!        version  2.6  date 23-08-2001 Correct call to prinlg
!        version  2.5  date 26-12-1994 New commons cmessc and cmessg
!
!   copyright (c) 1986-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fetch parameters for boussinesq type problems
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer index1(*), iuser(*)
      double precision xg(*), uold(*), phi(*), user(*), fg(*), etha(*), &
             work(*), beta(*), temp0(*), cp(*), cappa(*)
      integer ichois,nloc,mloc,ncomploc,ktime,method, &
           mconv,irotat,modelv, &
           indeth,indf1,indf2,indf3,indt0,indold,ialpha
      double precision sigma,rho,omega,cn,cl,ct,temp1

!     beta    i
!     cappa   i    ?
!     cl      i    ?
!     cn      i    Power in power law model
!     cp      i    ?
!     ct      i    ?
!     etha    i    Array containing the viscosity in the integration points
!     fg      i    ?
!     ialpha  i
!     ichois  i    Choice parameter
!     indeth  i    ?
!     index1  i    Array of length inpelm containing the point numbers
!                  of the nodal points in the element
!     indf1   i    ?
!     indf2   i    ?
!     indf3   i    ?
!     indold  i    ?
!     indt0   i    ?
!     irotat  i
!     ktime   i    ?
!     iuser   i    Integer user array to pass user information from
!                  main program to subroutine. See STANDARD PROBLEMS
!     m       i    number of integration points
!     mconv   i
!     method  i    method = 0  solenoidal field, this parameter is not specified
!                           1  penalty method,  iwork(1)=0 , work(1)=1/sigma
!     modelv  i    Defines the type of viscosity model
!                  Possible values:
!                    1:  newtonian liquid
!                    2:  power law liquid
!                    3:  carreau liquid
!                    4:  plastico viscous liquid
!                  100:  user provided model depending on grad u
!                  101:  user provided model depending on the second invariant
!                  102:  user provided model depending on the second invariant, &
!                        the co-ordinates and the velocity
!     n       i    number of basis functions for each component
!     ncomploc   i    ?
!     omega   i    Relaxation parameter
!     phi     i    array of length n * m, values of shape
!                  functions in integration points in the sequence:
!                         k
!                  phi (xg ) = phi (i,k)
!                     i
!                  array phi is only filled when gauss integration is applied
!     rho     i    density rho
!     sigma   i    parameter sigma for boundary conditions of the type u is
!                  constant
!     temp0   i    ?
!     temp1   i    ?
!     uold    i    Array containing the previous iteration (old solution)
!     user    i    Real user array to pass user information from
!                  main program to subroutine. See STANDARD PROBLEMS
!     work    i    Work array
!     xg      i    Array of length 4x2 containing the co-ordinates of the Gauss
!                  points in the reference element for a four point Gauss rule
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer, parameter :: nncode=4, nncode1=6
      integer :: ncode(nncode), ncode1(nncode1)
      character (len=16) :: code (nncode)
      character (len=16) :: code1 (nncode1)
      integer indexn(21)
      integer mstaf1,mstaf2,mstaf3,mstaet,mstact,mstat1, &
              mstat0,mstacp,mstcap, &
              indcp,indcap,indct,indt1
      save    mstaf1,mstaf2,mstaf3,mstaet,mstact,mstat1, &
              mstat0,mstacp,mstcap, &
              indcp,indcap,indct,indt1
      integer i,ichel,ij,indbet,inddum,iposus, &
              jparm,mstadm,m2,nuserloc

!     code           Table with keywords corresponding to string linearization
!     code1          Table with keywords corresponding to string
!                    coordinate_system
!     i       Counting variable
!     ichel   ?
!     ij      help parameter
!     indbet  ?
!     indcap  ?
!     indcp   ?
!     indct   ?
!     inddum  ?
!     indexn  ?
!     indt1   ?
!     iposus  ?
!     jparm   Number of parameters in an element group
!     m2      ?
!     mstacp  ?
!     mstact  ?
!     mstadm  ?
!     mstaet  ?
!     mstaf1  ?
!     mstaf2  ?
!     mstaf3  ?
!     mstat0  ?
!     mstat1  ?
!     mstcap  ?
!     ncode          Table containing the number of significant characters
!                    in the keyword table code
!     ncode1         Table containing the number of significant characters
!                    in the keyword table code1
!     nncode         Number of entries in code
!     nncode1        Number of entries in code1
!     nuserloc   ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer elgetintcoef

!     ELFLR5  Get user-defined parameters (with funccf)
!     ELGETINTCOEF   Give elgetintcoef the actual integer value corresponding to
!                    iuser(ipos)
!     ELUSER  Get parameters from arrays user,iuser
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRCHR  Put character in error message
!     ERREAL  Put real in error message
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     INSTOP  Stop the program
!     PRINLG  Print and checks length of user arrays
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      42   Array IUSER(.) has value which is too small
!     231   Viscosity is less than or equal to zero
!     232   Power of viscosity model is negative or larger than 5
!     234   Computed effective viscosity is less than or equal to zero
!     235   j th parameter in iuser (iwork) must be a constant
!     236   j th parameter in iuser (iwork) must be larger than zero
!     237   Parameter modelv has illegal value
!    1305   Coefficient has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
!              1  penalty method        ? iwork(1)=0 , work(1)=1/sigma
!     rho                               ? iwork(2)=0 , work(2)=rho
!     mconv =  0  no convective terms   ? iwork(3)=mconv
!              1  picard linearization
!              2  newton linearization
!              3  picard linearization (erroneous one ?)
!                 +10  time dependent theta method
!                 +20  pressure correction scheme
!     irotat = 0  steady coordinate sytem
!              1  rotating system       ? iwork(4)=0 , work(4)=omega
!     beta                              ? iwork(5)=0,  work(5)=beta
!     temp0      initial temperature    ? iwork(6)=0,ich1 , work(6)=temp0
!     cp                                ? iwork(7)=0,ich1 , work(7)=cp
!     cappa                             ? iwork(8)=0,ich1 , work(8)=cappa
!     ktime = 0   steady state
!             1   instationary equations theta method
!            +-2  pressure correction scheme
!     external forces                   ?  iwork(9)=0,ichl , work(9)=f1
!                                       ?  iwork(10)=0,ichl , work(10)=f2
!                                       ?  iwork(11)=0,ichl , work(10)=ft
!         the external forces (f1,f2) must be active forces, while
!         forces that do not create a flow with p=0 must be taken into
!         the reduced pressure
!     modelv = 1  newtonian liquid      ? iwork(12)=modelv
!              2  power law liquid
!              3  carreau model liquid
!              4  plastico-viscous liquid
!            100  user defined effective viscosity (based on grad(v),t)
!            101  user defined effective viscosity (based on secinv,t)
!            102  user defined effective viscosity (x,u,secinv,t)
!     ialpha = 0  no effect of viscous dissipation
!            = 1  including viscous dissipation
!     other parameters
!     viscosity model    ?  a) modelv=1-4 : iwork(13)=0,ichl
!                                            work(13)=etha
!                                           iwork(14)=0, work(14)=ct
!                                           iwork(15)=0, work(15)=temp1
!                        ?  b) modelv=2 : work(16)=cn
!                        ?     modelv=3 : work(17)=cl
!                        ?  c) modelv=4 : work(16)=cn
!                        ?              : work(17)=cl
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data code / 'NONE', 'PICARD', 'NEWTON', 'INCORRECT_PICARD' /
      data ncode / 4, 6, 6,16 /
      data code1 / 'NEWTONIAN', 'POWER_LAW', 'CARREAU', &
                   'PLASTICO_VISCOUS', 'SPECIAL', 'DIAGONAL' /
      data ncode1 / 9, 9, 7,16, 7,   8 /
! ======================================================================
!
      if (ichois/=0) goto 100
      call eropen('el4041')
      ktime=0
      iposus=iuser(ielgrp+5)
      if(iposus<7) then
         call errint(ielgrp+5,1)
         call errint(iposus  ,2)
         call errint(ielem,3)
         call errint(itype,4)
         call errint(ielgrp,5)
         call errsub ( 42, 5, 0, 0)
      end if
      nuserloc=0
      jparm=0
      if (method==1) then
         jparm=jparm+1
         call eluser(inddum,mstadm,work(1),iposus,iuser,user,nelemcac, &
                     npointcac,nuserloc,1)
         if (inddum>0 .and. inddum<100000000) then
            call errint(jparm,1)
            call errint(inddum,2)
            call errint(ielem,3)
            call errint(itype,4)
            call errint(ielgrp,5)
            call errsub ( 235, 5, 0, 0)
         else
            if (work(1)<=0) then
               call erreal(work(1),1)
               call errint(jparm,1)
               call errint(ielem,2)
               call errint(itype,3)
               call errint(ielgrp,4)
               call errsub ( 236, 4, 1, 0)
            else
               sigma=1.0d0/work(1)
            end if
         end if
      end if
      jparm=jparm+1
      call eluser(inddum,mstadm,work(1),iposus,iuser,user,nelemcac, &
                    npointcac,nuserloc,1)
      if (inddum>0 .and. inddum<100000000) then
         call errint(jparm,1)
         call errint(inddum,2)
         call errint(ielem,3)
         call errint(itype,4)
         call errint(ielgrp,5)
         call errsub ( 235, 5, 0, 0)
      end if
      rho=work(1)
      if (rho<=0) then
         call errint(jparm,1)
         call erreal(rho,1)
         call errint(ielem,2)
         call errint(itype,3)
         call errint(ielgrp,4)
         call errsub ( 236, 4, 1, 0)
      end if
      mconv= elgetintcoef ( iuser, iposus, jparm, code, nncode, ncode, &
                            1, 'linearization' )
      if(mconv>=10) then
         mconv=mconv-10
         ktime=1
         if(mconv>=10) then
           mconv=mconv-10
           ktime=2
           if(mconv>=5) then
              mconv=mconv-5
              ktime=-2
           end if
         end if
      end if
      if ( mconv<0 .or. mconv>3 ) then
         call errint(jparm,1)
         call errchr('mconv',1)
         call errint(itype,2)
         call errint(ielgrp,3)
         call errint(ielem,4)
         call errint(mconv,5)
         call errsub ( 1305, 5, 0, 1)
      end if
      jparm=jparm+1
      call eluser(inddum,mstadm,work(1),iposus,iuser,user,nelemcac, &
                    npointcac,nuserloc,1)
      omega=work(1)
      if (inddum>0 .and. inddum<100000000) then
         call errint(jparm,1)
         call errint(inddum,2)
         call errint(ielem,3)
         call errint(itype,4)
         call errint(ielgrp,5)
         call errsub ( 235, 5, 0, 0)
      end if
      irotat=0
      if (abs(omega)>1.0d-16) irotat=1

!     --- parameters for Boussinesq part
!         in array beta:  - beta rho g  is stored

      jparm=jparm+1
      call eluser(indbet,mstadm,beta,iposus,iuser,user,nelemcac, &
                    npointcac,nuserloc,mloc)
      if (indbet>0 .and. indbet<100000000) then
         call errint(jparm,1)
         call errint(indbet,2)
         call errint(ielem,3)
         call errint(itype,4)
         call errint(ielgrp,5)
         call errsub ( 235, 5, 0, 0)
      end if
      do 50 i=1,mloc
 50   beta(i) = -rho * 9.81d0 * beta(i)
      jparm=jparm+6
      call eluser(indt0,mstat0,temp0,iposus,iuser,user,nelemcac, &
                    npointcac,nuserloc,mloc)
      call eluser(indcp,mstacp,cp,iposus,iuser,user,nelemcac, &
                    npointcac,nuserloc,mloc)
      call eluser(indcap,mstcap,cappa,iposus,iuser,user,nelemcac, &
                    npointcac,nuserloc,mloc)

!     --- read external forces and etha; these may be functions

      call eluser(indf1,mstaf1,fg,iposus,iuser,user,nelemcac, &
                    npointcac,nuserloc,mloc)
      call eluser(indf2,mstaf2,fg(mloc+1),iposus,iuser,user,nelemcac, &
                    npointcac,nuserloc,mloc)
      call eluser(indf3,mstaf3,fg(mloc+mloc+1),iposus,iuser,user,nelemcac, &
                    npointcac,nuserloc,mloc)

!     --- read model of viscosity and other parameters

      modelv = elgetintcoef ( iuser, iposus, jparm, code1, nncode1, &
                              ncode1, 0, 'model' )
      if ( modelv==0 ) then
         modelv = 1
      else if ( modelv==6 ) then
         modelv = 10
      end if  ! ( modelv==0 )
      if(modelv>=1000) then
         ialpha=1
         modelv=modelv-1000
      else
         ialpha=0
      end if
      if (modelv<=0) then
         call errint(modelv,1)
         call errint(ielem,2)
         call errint(itype,3)
         call errint(ielgrp,4)
         call errsub ( 237, 4, 0, 0)
      end if
      if ( (modelv>4 .and. modelv<100) .or. modelv>102 ) then
         call errint(modelv,1)
         call errint(ielem,2)
         call errint(itype,3)
         call errint(ielgrp,4)
         call errsub ( 237, 4, 0, 0)
      end if
      if ( modelv>0 .and. modelv<5 ) then
         jparm=jparm+1
         call eluser(indeth,mstaet,etha,iposus,iuser,user,nelemcac, &
                     npointcac,nuserloc,mloc)
         if (indeth==0) then
            call erreal(etha(1),1)
            call errint(ielem,1)
            call errint(itype,2)
            call errint(ielgrp,3)
            call errsub ( 231, 3, 1, 0)
         end if
         jparm=jparm+1
         call eluser(indct,mstact,work(1),iposus,iuser,user,nelemcac, &
                     npointcac,nuserloc,1)
         ct=work(1)
         if (indct>0) then
            call errint(jparm,1)
            call errint(indct,2)
            call errint(ielem,3)
            call errint(itype,4)
            call errint(ielgrp,5)
            call errsub ( 235, 5, 0, 0)
         end if
         if (ct<0) then
            call errint(jparm,1)
            call erreal(ct,1)
            call errint(ielem,2)
            call errint(itype,3)
            call errint(ielgrp,4)
            call errsub ( 236, 4, 1, 0)
         end if
         jparm=jparm+1
         call eluser(indt1,mstat1,work(1),iposus,iuser,user, &
                     nelemcac,npointcac,nuserloc,1)
         temp1=work(1)
         if (indt1>0) then
            call errint(jparm,1)
            call errint(indt1,2)
            call errint(ielem,3)
            call errint(itype,4)
            call errint(ielgrp,5)
            call errsub ( 235, 5, 0, 0)
         end if

!        --- for modelv>1 more parameters are necessary for the viscous part
!                    (except for modelv=100,101,102)

         if ( modelv>=2 .and. modelv<=4 ) then
            jparm=jparm+1
            call eluser(inddum,mstadm,work(1),iposus,iuser,user, &
                       nelemcac, npointcac,nuserloc,1)
            cn=work(1)
            if (inddum>0) then
               call errint(jparm,1)
               call errint(inddum,2)
               call errint(ielem,3)
               call errint(itype,4)
               call errint(ielgrp,5)
               call errsub ( 235, 5, 0, 0)
            end if
            if ( cn<0.0d0 .or. cn>5.0d0 ) then

!           --- error 232 : cn<=0 or cn>5

               call erreal(cn,1)
               call errint(ielem,1)
               call errint(itype,2)
               call errint(ielgrp,3)
               call errsub ( 232, 3, 1, 0)
            end if
         end if
         if (modelv==3) then

!        --- 3 parameters carreau model

            jparm=jparm+1
            call eluser(inddum,mstadm,work(1),iposus,iuser,user, &
                       nelemcac, npointcac,nuserloc,1)
            cl=work(1)
            if (inddum>0) then
               call errint(jparm,1)
               call errint(inddum,2)
               call errint(ielem,3)
               call errint(itype,4)
               call errint(ielgrp,5)
               call errsub ( 235, 5, 0, 0)
            end if
            if (cl<0.0d0) then
               call errint(jparm,1)
               call erreal(cl,1)
               call errint(ielem,2)
               call errint(itype,3)
               call errint(ielgrp,4)
               call errsub ( 236, 4, 1, 0)
            end if

!           --- error 233 : cl<=0

         end if
         if (modelv==4) then

!        --- plastico-viscous liquid

            jparm=jparm+1
            call eluser(inddum,mstadm,work(1),iposus,iuser,user, &
                       nelemcac, npointcac,nuserloc,1)
            cl=work(1)
            if (inddum>0) then
               call errint(jparm,1)
               call errint(inddum,2)
               call errint(ielem,3)
               call errint(itype,4)
               call errint(ielgrp,5)
               call errsub ( 235, 5, 0, 0)
            end if
            if (cl<0.0d0) then
               call errint(jparm,1)
               call erreal(cl,1)
               call errint(ielem,2)
               call errint(itype,3)
               call errint(ielgrp,4)
               call errsub ( 236, 4, 1, 0)
            end if

!           --- error 233 : cl<=0

         end if
      end if
      call prinlg ( 'iuser',iposus-1,user,iuser )
      call prinlg ( 'user',-nuserloc,user,iuser )
      if ((indf1>1000 .and. indf1<10000).or. &
          (indf2>1000 .and. indf2<10000).or. &
          (indf3>1000 .and. indf3<10000).or. &
          (indt0>1000 .and. indt0<10000).or. &
          (indcp>1000 .and. indcp<10000).or. &
          (indcap>1000 .and. indcap<10000).or. &
          (indeth>1000 .and. indeth<10000).or. &
          (indbet>1000 .and. indbet<10000)) then
         indold=1
      else
         indold=0
      end if
      if (ierror/=0) then
         call instop
      else
         call erclos('el4041')
         return
      end if

!     --- end preliminary reading
!         if not constant compute the external driving forces
!         and the viscosity as function of (x,y)
!         prepare for the case that ind...>1000

100   if (indold==1) then
         ij=ncomploc+1
         do 102 i=1,mloc
            work(ij)=uold(i)
            work(ij+1)=uold(mloc+i)
            work(ij+2)=uold(mloc+mloc+i)
            ij=ij+3
            indexn(i)=i*ncomploc
102      continue
      end if
      if (indf1>0 .and. indf1<10000) &
          call elflr5(ichel,indf1,mstaf1,user,fg, &
          xg,xg(mloc+1),xg(mloc+mloc+1),work(ncomploc+1),nloc,mloc,ncomploc,index1,phi, &
          indexn,work)
      if (indf2>0 .and. indf2<10000) &
          call elflr5(ichel,indf2,mstaf2,user,fg(mloc+1), &
          xg,xg(mloc+1),xg(mloc+mloc+1),work(ncomploc+1),nloc,mloc,ncomploc,index1,phi, &
          indexn,work)
      if (indf3>0 .and. indf3<10000) &
          call elflr5(ichel,indf3,mstaf3,user,fg(mloc+mloc+1), &
          xg,xg(mloc+1),xg(mloc+mloc+1),work(ncomploc+1),nloc,mloc,ncomploc,index1,phi, &
          indexn,work)
      if (indt0>0 .and. indt0<10000) &
          call elflr5(ichel,indt0,mstat0,user,temp0, &
          xg,xg(mloc+1),xg(mloc+mloc+1),work(ncomploc+1),nloc,mloc,ncomploc,index1,phi, &
          indexn,work)
      if (indcp>0 .and. indcp<10000) &
          call elflr5(ichel,indcp,mstacp,user,cp, &
          xg,xg(mloc+1),xg(mloc+mloc+1),work(ncomploc+1),nloc,mloc,ncomploc,index1,phi, &
          indexn,work)
      if (indcap>0 .and. indcap<10000) &
          call elflr5(ichel,indcap,mstcap,user,cappa, &
          xg,xg(mloc+1),xg(mloc+mloc+1),work(ncomploc+1),nloc,mloc,ncomploc,index1,phi, &
          indexn,work)
      if (indeth>0 .and. indeth<10000) &
          call elflr5(ichel,indeth,mstaet,user,etha, &
          xg,xg(mloc+1),xg(mloc+mloc+1),work(ncomploc+1),nloc,mloc,ncomploc,index1,phi, &
          indexn,work)

!     --- test viscosity

      if ( modelv>=1 .and. modelv<=4 .and. indeth<10000) then
         do 110 i=1,mloc
            if (etha(i)<=0) then
               call eropen('el4041')
               call erreal(etha(i),1)
               call errint(ielem,1)
               call errint(itype,2)
               call errint(ielgrp,3)
               call errsub ( 234, 3, 1, 0)
               call erclos('el4041')
            end if
110      continue
      end if

!     --- error 231 : etha(i)<=0
!         --- an extra internal node has been added (6-point triangle)

      if (indeth==2001) then
         if (mloc==7) then
            etha(7)=(-etha(1)-etha(3)-etha(5) &
                    +4*(etha(2)+etha(4)+etha(6)))/9
         end if
      end if
      if (indf1==2001) then
         if (mloc==7) then
            fg(7)=(-fg(1)-fg(3)-fg(5) &
                    +4*(fg(2)+fg(4)+fg(6)))/9
         end if
      end if
      if (indf2==2001) then
         if (mloc==7) then
            fg(mloc+7)=(-fg(mloc+1)-fg(mloc+3)-fg(mloc+5) &
                    +4*(fg(mloc+2)+fg(mloc+4)+fg(mloc+6)))/9
         end if
      end if
      if (indf3==2001) then
         if (mloc==7) then
            m2=mloc+mloc
            fg(m2+7)=(-fg(m2+1)-fg(m2+3)-fg(m2+5) &
                    +4*(fg(m2+2)+fg(m2+4)+fg(m2+6)))/9
         end if
      end if
      end
