      subroutine el8218 (f, b, sigma, w, nd)
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.2  date 13-04-2005 Extension for axi-symmetry (RvL)
!        version  1.1  date 01-11-2002 Extension for 2D shapes (RvL)
!
!   copyright (c) 1999-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Compute element vector from -B' * sigma
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nd
      double precision b(nd,ndim,n,m), w(m), f(ndim,n), &
                       sigma(nd,m)

!     b              i    B-matrix
!     f              i    Deformation gradient in integration points
!     nd             i    Formal dimension corresponding to n
!     sigma          i    stress tensor
!     w              i    array of length m containing the weights for
!                         integration
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, k, ip, idx(2,14), p, l1, istart
      double precision sum

!     i              Counting variable
!     idx            ?
!     ip
!     istart         beginposition of 2D part of array idx
!     k              Counting variable
!     l1             ?
!     p
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data idx / 1,1, 2,2, 3,3, 4,1, 5,2, 6,3, 7,2, 8,3, 9,1, &
                 1,1, 2,2, 3,1, 4,2, 5,1 /
! ======================================================================
!
      if (ndim==2) istart=9
      if (ndim==3) istart=0

      do i = 1, n
         do k = 1, ndim
            f(k,i) = 0d0
         end do
      end do

      do i = 1, n
         do p = 1, nd
            ip = idx(1,istart+p)
            l1 = idx(2,istart+p)
            sum = 0d0
            do k = 1, m
               sum= sum + w(k)*b(ip,l1,i,k)*sigma(ip,k)
            end do
            f(l1,i) = f(l1,i) - sum
         end do
      end do

      end
