      subroutine el8234 (elemmt, S, L, Cab, Kab, z, isequ, iseqp, &
                         iseqmu, iseqksi, npres)
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.1  date 19-05-2003  Some adjustments (RvL)
!        version  1.0  date 15-12-2000
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    Assemble element stiffness matrix for a quadriphasic mixture model, &
!    consisting of charged solid, fluid, cations and anions
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npres, isequ(*), iseqp(*), iseqmu(3,*), iseqksi(*)
      double precision elemmt(icount,icount), s(ndim*n,ndim*n), &
                       L(ndim*n,npres), Cab(npres,npres,6), &
                       Kab(npres,npres,6), z(*)

!     cab            i    ?
!     elemmt         o    Element matrix to be filled
!                         Only the part corresponding to div and grad is filled
!     iseqksi        i    ?
!     iseqmu         i    ?
!     iseqp          i    Sequence for pressure dof's
!     isequ          i    Sequence for displacement dof's
!     kab            i    ?
!     l              i    General loop variable
!     npres          i    Number of pressure parameters per element
!     s              i    Element matrix without the gradient and divergence
!                         part
!     z              i
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k

!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The following system is build:
!     | S       -L         0           0       |  |  du  |   |     R       |
!     |                                        |  |      | = |             |
!     |   T                                    |  |      |   |             |
!     | -L   -sum(Cab)    Cab     -sum(Cab)*z  |  |  dp  |   | U - sum(Q)  |
!     |                                        |  |      |   |             |
!     |                                        |  |      |   |             |
!     |  0      Cab     -Kab-Cab    Cab*z      |  | dmu  |   | Q + sum(T)  |
!     |                                        |  |      |   |             |
!     |                                        |  |      |   |             |
!     |  0  -sum(Cab)*z   Cab*z  -sum(z*Cab*z) |  | dksi |   | -sum(z*Cab) |
!     with z a scalar.
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Fill material stiffness part  S

      do i = 1, ndim*n
         do j = 1, ndim*n
            elemmt(isequ(i),isequ(j)) = -S(i,j)
         end do
      end do

!     --- Fill L

      do i = 1, npres
         do j = 1, ndim*n
            elemmt(isequ(j),iseqp(i)) = -L(j,i)
            elemmt(iseqp(i),isequ(j)) = L(j,i)
         end do
      end do

!     --- Fill -sum(Cab)

      do i = 1, npres
         do j = 1, npres
            elemmt(iseqp(i),iseqp(j)) = &
               -(Cab(i,j,1) + Cab(i,j,2)+ Cab(i,j,3) + &
                 2d0*Cab(i,j,4) + 2d0*Cab(i,j,5) + 2d0*Cab(i,j,6))
         end do
      end do

!     --- Fill  -Kab-Cab

      do i = 1, npres
         do j = 1, npres
            elemmt(iseqmu(1,i),iseqmu(1,j)) = -Kab(i,j,1)-Cab(i,j,1)
            elemmt(iseqmu(1,i),iseqmu(2,j)) = -Kab(i,j,4)-Cab(i,j,4)
            elemmt(iseqmu(1,i),iseqmu(3,j)) = -Kab(i,j,6)-Cab(i,j,6)
            elemmt(iseqmu(2,i),iseqmu(1,j)) = -Kab(i,j,4)-Cab(i,j,4)
            elemmt(iseqmu(2,i),iseqmu(2,j)) = -Kab(i,j,2)-Cab(i,j,2)
            elemmt(iseqmu(2,i),iseqmu(3,j)) = -Kab(i,j,5)-Cab(i,j,5)
            elemmt(iseqmu(3,i),iseqmu(1,j)) = -Kab(i,j,6)-Cab(i,j,6)
            elemmt(iseqmu(3,i),iseqmu(2,j)) = -Kab(i,j,5)-Cab(i,j,5)
            elemmt(iseqmu(3,i),iseqmu(3,j)) = -Kab(i,j,3)-Cab(i,j,3)
         end do
      end do

!     --- Fill  -sum(z*Cab*z)

      do i = 1, npres
         do j = 1, npres
            elemmt(iseqksi(i),iseqksi(j)) =-(z(1)*Cab(i,j,1)*z(1) + &
                                             z(2)*Cab(i,j,2)*z(2) + &
                                             z(3)*Cab(i,j,3)*z(3) + &
                                         2d0*z(1)*Cab(i,j,4)*z(2) + &
                                         2d0*z(2)*Cab(i,j,5)*z(3) + &
                                         2d0*z(3)*Cab(i,j,6)*z(1))
         end do
      end do

!     --- Fill in zeros

      do i = 1, npres
         do j = 1, ndim*n
            do k = 1, 3
               elemmt(iseqmu(k,i),isequ(j)) = 0d0
               elemmt(isequ(j),iseqmu(k,i)) = 0d0
            end do
            elemmt(iseqksi(i),isequ(j)) = 0d0
            elemmt(isequ(j),iseqksi(i)) = 0d0
         end do
      end do

!     --- Fill  Cab

      do i = 1, npres
         do j = 1, npres
            elemmt(iseqmu(1,i),iseqp(j)) = &
                                Cab(i,j,1) + Cab(i,j,4) + Cab(i,j,6)
            elemmt(iseqmu(2,i),iseqp(j)) = &
                                Cab(i,j,2) + Cab(i,j,4) + Cab(i,j,5)
            elemmt(iseqmu(3,i),iseqp(j)) = &
                                Cab(i,j,3) + Cab(i,j,5) + Cab(i,j,6)
            elemmt(iseqp(j),iseqmu(1,i)) = elemmt(iseqmu(1,i),iseqp(j))
            elemmt(iseqp(j),iseqmu(2,i)) = elemmt(iseqmu(2,i),iseqp(j))
            elemmt(iseqp(j),iseqmu(3,i)) = elemmt(iseqmu(3,i),iseqp(j))
         end do
      end do

!     --- Fill   -sum(Cab)*z

      do i = 1, npres
         do j = 1, npres
            elemmt(iseqksi(i),iseqp(j)) = &
                      -(z(1)*(Cab(i,j,1)+Cab(i,j,4)+Cab(i,j,6)) + &
                           z(2)*(Cab(i,j,2)+Cab(i,j,4)+Cab(i,j,5)) + &
                           z(3)*(Cab(i,j,3)+Cab(i,j,5)+Cab(i,j,6)))
            elemmt(iseqp(j),iseqksi(i)) = elemmt(iseqksi(i),iseqp(j))
         end do
      end do

!     --- Fill in   Cab*z

      do i = 1, npres
         do j = 1, npres
            elemmt(iseqksi(i),iseqmu(1,j)) = z(1)*Cab(i,j,1) + &
                                             z(2)*Cab(i,j,4) + &
                                             z(3)*Cab(i,j,6)
            elemmt(iseqksi(i),iseqmu(2,j)) = z(1)*Cab(i,j,4) + &
                                             z(2)*Cab(i,j,2) + &
                                             z(3)*Cab(i,j,5)
            elemmt(iseqksi(i),iseqmu(3,j)) = z(1)*Cab(i,j,6) + &
                                             z(2)*Cab(i,j,5) + &
                                             z(3)*Cab(i,j,3)
            elemmt(iseqmu(1,j),iseqksi(i)) = &
                                      elemmt(iseqksi(i),iseqmu(1,j))
            elemmt(iseqmu(2,j),iseqksi(i)) = &
                                      elemmt(iseqksi(i),iseqmu(2,j))
            elemmt(iseqmu(3,j),iseqksi(i)) = &
                                      elemmt(iseqksi(i),iseqmu(3,j))
         end do
      end do

      do i=1, icount
         do j=1, icount
            elemmt(i,j)=-elemmt(i,j)
         end do
      end do

      end
