      subroutine el8248 ( strn, Fd, Fn, xg0, xg )
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  5.0  date 06-01-2016 Use sepmoduleelem
!        version  4.0  date 13-04-2005 Add integer nsig for axi-symmetry (RvL)
!        version  3.0  date 18-12-2002 Remove nd
!                                      Extension for 2D shapes (RvL)
!
!   copyright (c) 2001-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   compute the strain for general nonlinear solid element
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision strn(6,m), Fd(m,ndim,ndim), &
                       Fn(m,ndim,ndim), xg0(m,ndim), xg(m,ndim)

!     Fd             i    Deformation gradient
!     Fn             i    Deformation gradient
!     strn           o    Left Cauchy Green strain
!     xg             i    Array of length 27x3 containing the coordinates of
!                         the Gauss points in the reference element for a
!                         27 point Gauss rule
!     xg0            i    ?
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j, k
      double precision Fd_wrk(ndim,ndim), Fn_wrk(ndim,ndim), &
                       strn_wrk(6), work1(ndim, ndim), &
                       work2(ndim, ndim)

!     Fd_wrk         Work array for deformation gradient
!     Fn_wrk         Work array for deformation gradient
!     i              Counting variable
!     j              Counting variable
!     k              Counting variable
!     strn_wrk       Work array for strain
!     work1          Work array
!     work2          Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DGEMM          BLAS routine for matrix multiplication general.
!                    (NAG: f06yaf)
!     DSYMM          BLAS routine for matrix multiplication with a symmetric
!                    matrix.  (NAG: f06ycf)
!     ELMUL01        multiply normal stored matrix and result is reduced stored
!     ELROTSYM       Rotate symmetrical matrix
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- Loop over the integration points

      do k = 1, m

!     --- compute deformation gradient in current integration point

         do j = 1, ndim
            do i = 1, ndim
               Fd_wrk(i,j) = Fd(k,j,i)
               Fn_wrk(i,j) = Fn(k,j,i)
            end do
         end do

!        --- Compute Finger strain
!                             c      c
!            B = Fd * (Fn * Fn ) * Fd

         call elmul01 (Fn_wrk, strn_wrk, 0, ndim)

         if (ndim==2) then

            work1(1,1) = strn_wrk(1)
            work1(2,2) = strn_wrk(2)
            work1(1,2) = strn_wrk(3)
            work1(2,1) = strn_wrk(3)

!           --- compute work2=Fd_wrk.work1

            call dsymm( 'r', 'u', 2, 2, 1d0, work1, 2, Fd_wrk, &
                         2, 0d0, work2, 2)

!           --- compute work=work2.Fd_wrk'

            call dgemm( 'n', 't', 2, 2, 2, 1d0, work2, 2, Fd_wrk, &
                         2, 0d0, work1, 2)

            strn_wrk(1) = work1(1,1)
            strn_wrk(2) = work1(2,2)
            strn_wrk(3) = work1(1,2)
            strn_wrk(4) = work1(2,1)

            if (jcart==2) then
               strn_wrk(4)=(xg(k,1)/xg0(k,1))**2
            end if

!           --- store strn_wrk

            do i = 1, 4
               strn(i,k) = strn_wrk(i)
            end do

         else if (ndim==3) then

            call elrotsym (strn_wrk, Fd_wrk, 1)

!           --- store strn_wrk

            do i = 1, 6
               strn(i,k) = strn_wrk(i)
            end do

         end if

      end do

      end
