      subroutine eld203 ( coor2d, iuser, user, islold, vecloc, work )
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  2.0  date 06-01-2016 Use sepmoduleelem
!        version  1.9  date 29-05-2014 remove call to time subroutine
!        version  1.8  date 09-03-2013 Replace elflr7 by elflr71
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element derivative vector and weights for nonlinear
!     solid elements.
!     ELD200 is a help subroutine for subroutine DERIV (TO0047)
!     it is called through the intermediate subroutine eldsubs
!     So:
!     DERIV
!     TO0047
!       -  Loop over elements
!          -  ELDSUBS
!             - ELD200
!    The subroutine is able to handle the following element shapes and cases:
!    3D: 27 node brick (Type 200)
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     element_vector
!     mechanics
!     non_linear
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmoduleelem
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      double precision coor2d(*), user(*), vecloc(*), work(*)
      integer iuser(*), islold(*)

!     coor2d    i    array of length ndim x npointcac containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!                  x  = coor2d (1,i);  y  = coor2d (2,i);  z  = coor2d (3,i);
!                   i                 i                 i
!     islold  i    User input array in which the user puts information
!                  of all preceding solutions
!                  Integer array of length 5 x numold
!     iuser   i    Integer user array to pass user information from
!                  main program to subroutine. See STANDARD PROBLEMS
!     user    i    Real user array to pass user information from
!                  main program to subroutine. See STANDARD PROBLEMS
!     vecloc  i    Work array in which all old solution vectors for the
!                  integration points are stored
!     work         work array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxnp, maxnu, maxcoef, iseqnr, maxusr
      parameter (maxnu = 27, maxnp = 8, maxcoef = 45, maxusr = 1000, &
                 iseqnr = 1)
      integer isub, isup, icounw, jelp, mult, nsig
      save isub, isup, icounw, jelp, mult
      integer i, j, imatlaw, iusrvc, iusrflg, ncoef, nd, &
              ndisp, npres, ifsi
      integer isequ(3*maxnu), iseqp(maxnp)
      save imatlaw, iusrvc, iusrflg, ncoef, nd, ndisp, npres, &
           isequ, iseqp
      double precision coor2d0(ndim,inpelm), coor2dn(ndim,inpelm), &
                       coor2d1(ndim,inpelm), qmat(3*inpelm), rdum
      integer ipx, ipxg, ipw, ipphix, ippsix, ipphi0, ipphin, ipphi, &
              ipdphi0, ipdphin, ipdphi, ippsi, ipdpsi, ipcoef, &
              ipu, ipug, ipFn, ichfib, ipend, ipxg0, ipxgn, &
              ipJn, ipFd, ipJd, ipp, ipp8, ippn, ippn8, ipusrvc, ipwork
      save ipx, ipxg, ipw, ipphix, ippsix, ipphi0, ipphin, ipphi, &
           ipdphi0, ipdphin, ipdphi, ippsi, ipdpsi, ipcoef, &
           ipu, ipug, ipFn, ichfib, ifsi, ipxg0, ipxgn, &
           ipJn, ipFd, ipJd, ipp, ipp8, ippn, ippn8, ipusrvc, ipwork
      double precision, pointer :: vecoldlc(:), vecoldlc1(:)
      integer, pointer :: index3n1(:)

!     coor2d0          Coordinates in undeformed configuration
!     coor2d1          Coordinates in current configuration
!     coor2dn          Coordinates in reference configuration
!     i              Counting variable
!     ichfib         Indicates which fiber subroutine has to be used
!                    See el0200
!     icounw         length of elemwg(*)
!     ifsi           Sequence number of old solution vector for
!                    fully coupled fluid-structure interaction
!     imatlaw        Type of material constitutive equation to be used
!     ipcoef         Pointer to coefficients
!     ipdphi         Pointer to dphi/dx on current configuration
!     ipdphi0        Pointer to dphi/dx on undeformed configuration
!     ipdphin        Pointer to dphi/dx on reference configuration
!     ipdpsi         Shape functions for the pressure
!     ipend          Last position in array
!     ipfd           Pointer to F_delta
!     ipfn           Pointer to F_n
!     ipjd           Pointer to J_delta
!     ipjn           Pointer to J_n
!     ipp            Pointer to pressures in the current configuration
!     ipp8           Pointer to pressures in the vertices
!     ipphi          Pointer to basisfunctions on current configuration
!     ipphi0         Pointer to basisfunctions on undeformed configuration
!     ipphin         Pointer to basisfunctions on reference configuration
!     ipphix         Pointer to phiksi
!     ippn           Pointer to hydrostatic pressure on reference configuration
!     ippn8          Pointer to hydrostatic pressure in the vertices
!     ippsi          Starting address in array work of psi
!     ippsix         Pointer to the derivatives of the pressure shape functions
!     ipu            Pointer to nodal displacements
!     ipug           Pointer to displacements in integration points
!     ipusrvc        Pointer to user vector
!     ipw            Pointer to weighing functions
!     ipwork         Pointer to work array
!     ipx            Pointer to element coordinates
!     ipxg           Pointer to integration point coordinates
!     ipxg0          Pointer to integration point coordinates on undeformed
!                    configuration
!     ipxgn          Pointer to integration point coordinates on reference
!                    configuration
!     iseqnr         Sequence number for first old solution vector
!     iseqp          Sequence for pressure dof's
!     isequ          Sequence for displacement dof's
!     isub           Integer help parameter for do-loops to indicate the
!                    smallest coefficient number that is dependent on space
!     isup           Integer help parameter for do-loops to indicate the
!                    largest coefficient number that is dependent on space
!     iusrflg        User flag
!     iusrvc         if>0, a user defined vector is used
!     j              Counting variable
!     jelp           Indication of the type of basis function subroutine
!                    must be called by ELxxxx. Possible vaues:
!                     1:  Subroutine ELP610  (Linear line element)
!                     2:  Subroutine ELP611  (Quadratic line element)
!                     3:  Subroutine ELP620  (Linear triangle)
!                     4:  Subroutine ELP624  (Extended quadratic triangle)
!                     5:  Subroutine ELP621  (Bilinear quadrilateral)
!                     6:  Subroutine ELP623  (Biquadratic quadrilateral)
!                    11:  Subroutine ELP630  (Linear tetrahedron)
!                    12:  Subroutine ELP631  (Quadratic tetrahedron)
!                    13:  Subroutine ELP632  (Trilinear brick)
!                    14:  Subroutine ELP633  (Triquadratic brick)
!     maxcoef        Maximum number of coefficients
!     maxnp          Maximum number of pressure dof's
!     maxnu          Maximum number of displacement dof's
!     maxusr         Maximum number of uservector dof's
!     mult           Number of dof's to be filled
!     ncoef          Number of coefficients in the equation
!     nd             Dimension of tangent stiffness matrix
!     ndisp          Number of displacement dof's
!     npres          Number of pressure dof's
!     nsig           Number of stress components
!     qmat           transformation matrix global -> local coordinates
!                    in case of a 3D element
!     rdum           dummy variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL0200         performs the initializations for subroutine ELM200/ELD200
!     EL1007         Fill element vector for vertices only
!     EL2603         Get old displacement vector and deformation gradient for
!                    general nonlinear solid
!     EL8246         compute the stress for general nonlinear solid element
!     EL8248         compute the stress for general nonlinear solid element
!     EL8249         compute deformation gradient for general nonlinear solid
!     EL8250         compute the fiber direction in case of composite elements
!     EL8251         Computes relative fluid velocity for biphase material
!     ELFLR71        Fill variable coefficients in array ARRAY
!     ELIND0         Fill array ICH in COMMON CELIAR for preceding solutions
!     ELM800BASEFN   Compute the basis functions phi, its derivatives and other
!                    related quantities for various element shapes
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
! **********************************************************************
!
!                       I/O
!
!     The user provides the coefficients through the arrays IUSER and USER
!     Usually they are filled by FILCOF
!     See subroutine el0200 for a description of the coefficients
!
!     The following derived quantitites can be computed:
!    +--------+-----------------------+----+------------------------+------+ &
!    | icheld | derived quantity      | ix |                        | ivec |
!    +--------+-----------------------+----+------------------------+------+ &
!    |    1   | du/dx                 |    |                        |      |
!    +--------+-----------------------+----+------------------------+------+ &
!    |    2   | grad u                |    |                        |      |
!    +--------+-----------------------+----+------------------------+------+ &
!    |    3   | -grad u               |    |                        |      |
!    +--------+-----------------------+----+------------------------+------+ &
!    |    4   | div u                 |    |                        |      |
!    +--------+-----------------------+----+------------------------+------+ &
!    |    5   | curl u                |    |                        |      |
!    +--------+-----------------------+----+------------------------+------+ &
!    |    6   | stress                                                 4   |
!    |        +-----------------------+----+------------------------+------+ &
!    |                                |  1 | sigma                  |      |
!    |                                |  2 | tau                    |      |
!    |                                |  3 | fiber stress (global)  |      |
!    |                                |  4 | fiber stress (local)   |      |
!    +--------+-----------------------+----+------------------------+------+ &
!    |    7   | pressure              |    |                        |  1   |
!    +--------+-----------------------+----+------------------------+------+ &
!    |    8   | strain                |    |                        |  4   |
!    +--------+-----------------------+----+------------------------+------+ &
!    |    9   | deformation gradient                                   13  |
!    |        +-----------------------+----+------------------------+------+ &
!    |                                |  1 | nodal points           |      |
!    |                                |  2 | integration points     |      |
!    +--------+-----------------------+----+------------------------+------+ &
!    |   10   | local direction       |    |                        |  3   |
!    +--------+-----------------------+----+------------------------+------+ &
!    |   11   | fluid velocity        |    |                        |  3   |
!    +--------+-----------------------+----+------------------------+------+ &
!
!     21-31 Only in vertices
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ('eld203')

      if (ifirst==0)  then

!     --- ifirst = 0,  compute element independent quantities

         call el0200 ( iuser, user, jelp, imatlaw, iusrvc, &
                       iusrflg, nd, isub, isup, work, npres, ndisp, &
                       isequ, iseqp, ncoef, ifsi, nsig, maxusr, &
                       ichfib )

!        --- Fill array ich (COMMON CELIAR) for preceding solutions

         call elind0 ( islold, numold, isub, isup, iuser)

!        --- Length of elemwg to be filled

         goto (599, 599, 599, 599, 599, 506, 507, 508, 509, 510, &
               511, 599, 599, 599, 599, 599, 599, 599, 599, 599, &
               599, 599, 599, 599, 599, 599, 527, 599, 599, 599, &
               599, 599, 599, 599, 599, 599, 599, 599, 599, 599) icheld

 506     mult = 6
         goto 599
 507     mult = 1
         goto 599
 508     mult = 6
         goto 599
 509     mult = 9
         goto 599
 510     mult = 3
         goto 599
 511     mult = 3
         goto 599
 527     mult = 1
         goto 599

 599     if (icheld<=20) then

!        --- unknowns in all nodes

            icounw = inpelm * mult
         else

!        --- unknowns in vertices only

            icounw = nvert * mult
         end if

!        --- Initialize weight vector, if any

         do i = 1, icounw
            elemvc1(i) = 0d0

!           --- no weight for icheld=9, because F is stored elementwise

            if (icheld/=9) then
               elemwg(i) = 1d0
            end if
         end do

!        --- Fill pointers in array work

         ipcoef  = 1
         ipx     = ipcoef  + maxnu*maxcoef
         ipxg0   = ipx     + maxnu*ndim
         ipxgn   = ipxg0   + maxnu*ndim
         ipxg    = ipxgn   + maxnu*ndim
         ipw     = ipxg    + maxnu*ndim
         ipphix  = ipw     + maxnu
         ippsix  = ipphix  + maxnu*maxnu*ndim
         ipphi0  = ippsix  + maxnp*maxnu*ndim
         ipphin  = ipphi0  + maxnu*maxnu
         ipphi   = ipphin  + maxnu*maxnu
         ipdphi0 = ipphi   + maxnu*maxnu
         ipdphin = ipdphi0 + maxnu*maxnu*ndim
         ipdphi  = ipdphin + maxnu*maxnu*ndim
         ippsi   = ipdphi  + maxnu*maxnu*ndim
         ipdpsi  = ippsi   + maxnp*maxnu
         ipu     = ipdpsi  + maxnp*maxnu*ndim
         ipug    = ipu     + maxnu*ndim
         ipFn    = ipug    + maxnu*ndim
         ipJn    = ipFn    + maxnu*ndim*ndim
         ipFd    = ipJn    + maxnu
         ipJd    = ipFd    + maxnu*ndim*ndim
         ipp     = ipJd    + maxnu
         ipp8    = ipp     + maxnu
         ippn    = ipp8    + maxnp
         ippn8   = ippn    + maxnu
         ipusrvc = ippn8   + maxnp
         ipwork  = ipusrvc + maxnu*maxusr
         ipend   = ipwork  + max((n+1)*m,n**2+n+m)

!        --- Check length of array work (declared in elmgrinf)

         call inichkarlen ( ipend, 69696, 'work' )

      end if

!     --- Copy coordinates of reference mesh

      iseqtype = oldsols(iseqnr+1)%iseqtype
      index3n1 => indoldsols(iseqtype)%index3
      vecoldlc1 => oldsols(iseqnr+1)%oldvecs
      iseqtype = oldsols(iseqnr)%iseqtype
      index3n => indoldsols(iseqtype)%index3
      vecoldlc => oldsols(iseqnr)%oldvecs
      do i = 1, n
         do j = 1 , ndim
            coor2dn(j,i) = coor2d(j+(index1(i)-1)*ndim)
            coor2d1(j,i) = coor2dn(j,i) + &
                         vecoldlc(index3n(isequ((i-1)*ndim+j)))
            coor2d0(j,i) = coor2dn(j,i) - &
                         vecoldlc1(index3n1(isequ((i-1)*ndim+j)))
         end do
      end do

!     --- Compute basis functions on reference and deformed mesh

      call elm800basefn ( coor2d0, work(ipx), work(ipw), work(ipdphi0), &
                          work(ipxg0), work(ipphi0), index1, qmat, rdum, &
                          jelp, work(ipphix), work(ippsi), work(ipdpsi), &
                          work(ippsix) )
      call elm800basefn ( coor2dn, work(ipx), work(ipw), work(ipdphin), &
                          work(ipxgn), work(ipphin), index1, qmat, rdum, &
                          jelp, work(ipphix), work(ippsi), work(ipdpsi), &
                          work(ippsix) )
      call elm800basefn ( coor2d1, work(ipx), work(ipw), work(ipdphi), &
                          work(ipxg), work(ipphi), index1, qmat, rdum, &
                          jelp, work(ipphix), work(ippsi), work(ipdpsi), &
                          work(ippsix) )
      if (ierror/=0) goto 1000

!     --- Get all old vectors

      call el2603 ( work(ipphi0), work(ipphin), work(ipphi), &
                    work(ipdphi0), work(ipdphin), work(ippsi), work(ipu), &
                    work(ipug), work(ipFd), work(ipJd), work(ipFn), &
                    work(ipJn), work(ipp), work(ipp8), work(ippn), &
                    work(ippn8), work(ipusrvc), iseqnr, iusrvc, isequ, &
                    iseqp, npres )

!     --- Fill variable coefficients if necessary

      do i = isub, isup
         if (ind(i)>0 .or. ist(i)>0) then

!        --- Coefficient is variable

            call elflr71 (i, user, work, work(ipx), &
                         work(ipxg), m, n, work(ipphi), 1, index1, &
                         iuser, work(ipug), vecloc )
         end if
      end do

!     --- Fill elemvc1. Content depends on icheld

      if (icheld<=20) then
         goto (1000,1000,1000,1000,1000, 206, 207, 208, 209, 210, &
                211,1000,1000,1000,1000,1000,1000,1000,1000,1000),icheld
      else
         goto (1000,1000,1000,1000,1000, 206, 207, 208, 209, 210, &
                211,1000,1000,1000,1000,1000,1000,1000,1000,1000), &
               icheld-20
      end if

!     --- icheld = 6
!         Compute the stress tensor and store in elemvc1

 206  call el8246 (work(ipwork), work(ipxg), coor2d0, work(ipFd), &
                   work(ipJd), work(ipFn), work(ipJn), work(ipp), &
                   work(ipusrvc), imatlaw, iusrvc, iusrflg, &
                   work(ipcoef), ncoef, index1, nd, work(ipxg0), &
                   ichfib)
      goto 999

!     --- icheld = 7
!         Compute the pressure and store in elemvc1

 207  do i = 1, n
         work(ipwork+i-1) = work(ipp+i-1)
      end do
      goto 999

!     --- icheld = 8
!         Compute the strain tensor and store in elemvc1

 208  call el8248 (work(ipwork), work(ipFd), work(ipFn), &
                   work(ipxg0), work(ipxg) )
      goto 999

!     --- icheld = 9
!         Compute the deformation tensor and store in elemvc1

 209  call el8249 (work(ipwork), work(ipFd), work(ipFn), &
                   work(ipxg0), work(ipxg), ndim+jcart-1 )
      goto 999

!     --- icheld = 10
!         Compute fiber direction in case of composite material law
!         and store in elemvc1

 210  call el8250 (work(ipwork), work(ipxg), coor2d0, work(ipFd), &
                   work(ipFn), imatlaw, work(ipusrvc), iusrvc, iusrflg, &
                   work(ipcoef), ncoef, index1)
      goto 999

!     --- icheld = 11
!         Compute fluid velocity and store in elemvc1

 211  call el8251 (work(ipwork), work(ipp8), work(ipdpsi), &
                   work(ipcoef), ncoef, npres)
      goto 999

!     --- If icheld<=20 then work(ipwork) is just copied, otherwise
!         only the unknowns in the vertices are copied

 999  if (icheld<=20) then
         do i = 1, n*mult
            elemvc1(i) = work(ipwork+i-1)
         end do
      else
         call el1007 (work(ipwork), elemvc1, mult, ishape)
      end if

1000  call erclos ('eld203')

      end
