      subroutine eld420(coorloc,iuser,user, &
                        sol,solold, array, vecloc)
! ======================================================================
!
!        programmer    jaap van der zanden
!        version  4.7  date 12-12-2012 Long integers
!        version  4.6  date 22-05-1995 Rename ctime in ctimen
!        version  4.5  date 26-12-1994 New commons cmessc and cmessg
!
!   copyright (c) 1986-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mlocechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     2-dimensional boussinesq equations
!     compute derived quantities
!     icheld = 1  pressure
!              2  du(j)/dx(i)
!              3  div v
!              4  curl v
!              5  extra stress : 11,12,22(,33)
!              6  grad v
!              7  elongation
!              8  shear rate
!     extended quadratic triangle for velocities (itype=420)
!              quadratic triangle for temperature
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmoduleoldvecs
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      double precision theta, deltat, t0, rtime
      integer nstep, ktime
      common /ctimen/ theta, deltat, t0, rtime(7), nstep, ktime(9)
      save /ctimen/

!                      /ctimen/
!     Contains parameters for the time discretization
!    theta    i    constant for theta method (0.5<theta<1)
!    deltat   i    Time step for the numerical integration
!    t0       i    Time at first call of soltim
!    rtime    -    Dummy, not yet used
!    nstep    i    Time step number
!    ktime    -    Dummy, not yet used
!  - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*)
      double precision coorloc(*), user(*), sol(*), &
             solold(*), array(*), vecloc(numold,maxunk,*)

!     array        Work array for the element subroutines
!     coorloc    i    array of length ndimloc x npoint containing the co-ordinates
!                  of the nodal points with respect to the global numbering
!                  deriva
!     iuser   i    Integer user array to pass user information from
!                  main program to subroutine. See STANDARD PROBLEMS
!     sol     i    ?
!     solold  i    ?
!     user    i    Real user array to pass user information from
!                  main program to subroutine. See STANDARD PROBLEMS
!     vecloc       Work array in which all old solution vectors for the
!                  integration points are stored
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer method, itcoorloc, jtcoorloc, ndimloc, jcomp, n7, n14, &
           nstart, lmh, lmd, ipx, ipde, ipdl, ipw, ipxg, ipphi, ipdpx, &
           iptr, ipt3, ipq1, ipfg, ipet, ipu, ipug, ipdux, ipmt, ipvc, &
           ippen, ipvsc, ipinv, ipdsc, ipmas, ipu0, &
           jtime, linear, irotat, modelv, &
           indeth, ippsi, ipdpsx, ipbeta, iptem0, ipcp, ipcapp, &
           ialpha, ipwrk, mloc, indf1, indf2, indf3, indt0, induv(18), &
           indold
      double precision sigmalc, rho, omega, cn, cl, ct, temp1
      save    method, itcoorloc, jtcoorloc, ndimloc, jcomp, n7, n14, &
           nstart, lmh, lmd, ipx, ipde, ipdl, ipw, ipxg, ipphi, ipdpx, &
           iptr, ipt3, ipq1, ipfg, ipet, ipu, ipug, ipdux, ipmt, ipvc, &
           ippen, ipvsc, ipinv, ipdsc, ipmas, ipu0, &
           jtime, linear, irotat, modelv, &
           indeth, ippsi, ipdpsx, ipbeta, iptem0, ipcp, ipcapp, &
           ialpha, ipwrk, mloc, indf1, indf2, indf3, indt0, induv, indold
      save             sigmalc, rho, omega, cn, cl, ct, temp1
      integer i,ih,ich1,ich220,ich221,ichh,ichint,ichiv,icurve,isign, &
              ill,junkp,k,numveclc
      double precision delta, pc, rhoh(1), data(7), ddatdx(14), &
             datag(7), funcc3
      save rhoh

!     cl      ?
!     cn      Power in power law model
!     ct      ?
!     data    ?
!     datag   ?
!     ddatdx  ?
!     delta   jacobian of element
!     i       Counting variable
!     ialpha
!     ich1    ?
!     ich220  ?
!     ich221  ?
!     ichh    ?
!     ichint  ?
!     ichiv   ?
!     icurve  Type of curve
!     ih      Help variable to store a constant temporarily
!     ill     ?
!     indeth  ?
!     indf1   ?
!     indf2   ?
!     indf3   ?
!     indold  ?
!     indt0   ?
!     induv   ?
!     ipbeta  ?
!     ipcapp  ?
!     ipcp    ?
!     ipde    ?
!     ipdl    ?
!     ipdpsx  ?
!     ipdpx   ?
!     ipdsc   ?
!     ipdux   ?
!     ipet    ?
!     ipfg    ?
!     ipinv   ?
!     ipmas   ?
!     ipmt    ?
!     ippen   ?
!     ipphi   ?
!     ippsi   Starting address in array work of psi
!     ipq1    ?
!     ipt3    ?
!     iptem0  ?
!     iptr    ?
!     ipu     ?
!     ipu0    ?
!     ipug    ?
!     ipvc    ?
!     ipvsc   ?
!     ipw     ?
!     ipwrk   ?
!     ipx     ?
!     ipxg    ?
!     irotat
!     isign   Help parameters to check the sign of the Jacobian in the
!             integration points
!     itcoorloc  ?
!     jtime   ?
!     jcomp   ?
!     jtcoorloc  ?
!     junkp   ?
!     k       Counting variable
!     linear  Indicates whether the element is a linear simplex (true)
!             or not (false)
!     lmd     ?
!     lmh     ?
!     m       number of integration points
!     method
!     modelv  Defines the type of viscosity model
!             Possible values:
!               1:  newtonian liquid
!               2:  power law liquid
!               3:  carreau liquid
!               4:  plastico viscous liquid
!             100:  user provided model depending on grad u
!             101:  user provided model depending on the second invariant
!             102:  user provided model depending on the second invariant, &
!                   the co-ordinates and the velocity
!     n14     ?
!     n7      ?
!     ncomp   ?
!     ndimloc    Dimension of the space.
!     nstart
!     numveclc  Number of vectors stored in isol
!     omega
!     pc      ?
!     rho     density rho
!     rhoh    ?
!     sigmalc
!     temp1   ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     EL2002  Compute part of matrix corresponding to zeroth order derivatives
!     EL2015  Compute matrix vector multiplication
!     EL4021  the old solution is retrieved from solold with help of index2;
!             uc,vc in the centroid with help of truc,trvc.
!     EL4022  Compute viscous matrix for 2-dim incompressible flow
!     EL4025  Compute (transformed) penalty vector
!     EL4031  Compute (driving) external forces for RHS vector
!     EL4032  Compute coriolis acceleration and centrifugal force
!     EL4033  Compute convective acceleration for RHS vector
!     EL4034  Compute pressure in centroid for penalty method
!     EL4035  Compute gradient of pressure and fill elemvc1 and elemwg
!     EL4040  Compute viscosity for a non-isothermal newtonian liquid
!     EL4041  Fetch parameters for boussinesq type problems
!     EL4042  Compute derived quantities of a vector
!     EL4402  extract old vectors from sol, solold and form derivatives
!     ELP220  Shape functions for 7-point triangle (7 integration points)
!     ELP221  Shape functions for 6 point triangle (7 integration points)
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     ERRINT  Put integer in error message
!     ERRSUB  Error messages
!     FUNCC3  User function subroutine to compute the value of a coefficient
!             in a point depending on a series of preceding solutions
!     INSTOP  Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!       3   Element has incorrect number of nodal points for this type
!      74   Incorrect number of degrees of freedom for this type
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data induv/1,7,13, 2,8,14, 3,9,15, 4,10,16, 5,11,17, 6,12,18/
! ======================================================================
!
      call eropen( 'eld420' )
      if (ifirst==1) goto 1000

!     --- ifirst = 0,  initialize some parameters and arrays
!         n7    number of points for velocity u1,u2 including centroid
!         nstart  number of test function to start with
!         itype=  420      itcoorloc = 0  cartesian coorlocdinates
!             422               2  polar coorlocdinates
!             421               1  axisymmetric coorlocdinates
!           ( all  with penalty function method (method=1) and )
!           (      stationary                                  )

      method=1
      itcoorloc=itype-420
      mloc=7
      ndimloc=2
      ncomp=3
      jcomp=2
      n7=inpelm+1
      n14=n7+n7
      nstart=n7

!     --- test if element is quadratic (triangle,...) for the mesh generator

      if (inpelm/=6) then
         call errint ( inpelm, 1 )
         call errint ( 6, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 3, 5, 0, 0)
      end if
      if (icount/=ncomp*inpelm) then
         call errint ( icount, 1 )
         call errint ( ncomp*inpelm, 2 )
         call errint ( ielem, 3 )
         call errint ( itype, 4 )
         call errint ( ielgrp, 5 )
         call errsub ( 74, 5, 0, 0)
      end if

!     --- array /  first part to be created by elp220 (total length ...)
!         ----- &

      lmh=n7*n7
      lmd=n14*n14
      ipx=1
      ipde=ipx+ndimloc*n7
      ipdl=ipde+6
      ipw=ipdl+6
      ipxg=ipw+3*n7
      ipphi=ipxg+ndimloc*n7
      ipdpx=ipphi+lmh
      iptr=ipdpx+ndimloc*lmh
      ipt3=iptr+ndimloc*ndimloc*inpelm
      ipq1=ipt3

!     --- isign : sign of the jacobian
!         icurve =0 : element boundary is straight; =2 : curved
!         delta = 2 * area of element
!         ipx      x(i) = x1-coorlocdinates of nodal points
!                  y(i) = y1-coorlocdinates of nodal points
!         ipde     e1(i) see lecture notes van kan/segal numerieke analyse c2
!                  e2(i) see lecture notes van kan/segal numerieke analyse c2
!         ipdl     dldx1(i)  derivative of linear basis function to x1
!                  dldx2(i)  derivative of linear basis function to x2
!         ipw      w(i) = weight factors for integration (any itcoorloc)
!         ipxg     x1g(i) x1 coorlocdinates in integration points
!                  x2g(i) x2 coorlocdinates in integration points
!         ipphi     phig(ij) basis functions in integration points
!         ipdpx    dphidx(ij) derivatives of basis functions to x1 (integr)
!                  dphidy(ij) derivatives to x2 (in integration points)
!         iptr     truc(i) transformation/elimination due to uc
!                  trvc(i) transformation/elimination due to vc (centroid)
!         ipt3     trpsut(i) (u,v,psi,ut)=trpsut*(u,v) : solenoidal basis
!         ipq1     qphixx(i,j) integrals on reference element
!                  qphixe(i,j) integrals on reference element
!                  qphiee(i,j) integrals on reference element
!                  second part local parameters:
!                  array(ipmt),array(ipvc) are used for intermediate storage
!                  as well as for element matrix and element vector

      ipfg=ipq1+3*lmh
      ipet=ipfg+ncomp*mloc
      ipu=ipet+mloc
      ipug=ipu+ncomp*n7
      ipdux=ipug+ncomp*mloc
      ipmt=ipdux+ndimloc*ncomp*mloc
      ipvc=ipmt+lmd
      ippen=ipvc+n14
      ipvsc=ippen+n14
      ipinv=ipvsc+mloc
      ipdsc=ipinv+mloc
      ippsi=ipdsc+mloc
      ipdpsx=ippsi+mloc*inpelm
      ipbeta=ipdpsx+ndimloc*inpelm*mloc
      iptem0=ipbeta+mloc
      ipcp=iptem0+mloc
      ipcapp=ipcp+mloc
      ipwrk=ipcapp+mloc
      ipmas=ipwrk+n14
      ipu0=ipmas+n7*n7

!     --- iplast=ipu0+ncomp*m-1
!         length of array is:  974
!         ipfg     f1(i)  external body force (to be multiplied with rho)
!                  f2(i)  external body force (must be non-driving)
!                  f3(i)  external 'heat' force
!         ipet     etha(i) viscosity (parameter) in integration points
!         ipu      u(i)  u1-velocity in nodal points
!                  v(i)  u2-velocity in nodal points
!                  t(i)  temperature in nodal points
!         ipug     ug(i)  u1-velocity in integration points
!                  vg(i)  u2-velocity in integration points
!                  tg(i)  temperature in integration points
!         ipdux    dux(i) derivative of u to x1 in integration points
!                  duy(i) derivative of u to x2 in integration points
!                  dvx(i) derivative of v to x1 in integration points
!                  dvy(i) derivative of v to x2 in integration points
!                  dtx(i) derivative of t to x1 in integration points
!                  dty(i) derivative of t to x2 in integration points
!         ipmt     s(ifjf) help array for element matrix, &
!                          (icount+2)*(icount+2)
!         ipvc     v(if) help array for vector (icount+2) or
!                        matrix((icount+2)*(icount+2))
!         ipvsc    etheff(i) effective viscosity in integration points
!         ipinv    secinv(i) second invariant of a1 in integration points
!         ipdsc    dvisd2(i) derivative of visc to secinv (integr. points)
!         ipmas    mass matrix for time derivative of momentum equation
!         ipu0     u0(i)  u1-velocity at previous time step
!                  v0(i)  u2-velocity at previous time step
!                  t0(i)  temperature at previous time step
!         fetch parameters (the constants only)

      call el4041(0,array(ipxg),array(ipug),array(ipphi), &
                  index1,n7,n7,ncomp,iuser,user, &
                  jtime,method,sigmalc,rho,linear,irotat,omega, &
                  array(ipfg),modelv,array(ipet),cn,cl,ct, &
                  temp1,indeth, &
                  array(ipvc),array(ipbeta),array(iptem0),array(ipcp), &
                  array(ipcapp),indf1,indf2,indf3,indt0,indold,ialpha)
       if (jtime>=1) rhoh(1)=rho/(theta*deltat)

!      --- jtcoorloc=3  cartesian coorlocdinates with Gauss integration

1000  if ((itcoorloc==0).and.( &
          (modelv/=1).or.(ct>0.0d0))) then
          jtcoorloc=3
      else
         jtcoorloc=itcoorloc
      end if
      ich220=jtcoorloc+30
      if (method==0) ich220=ich220+40

!     --- compute basis functions, derivatives of basis functions, etc.

      if (icheld/=1) ich220=ich220-jtcoorloc
      call elp220(ich220,ichint,coorloc,index1,isign,icurve, &
          delta,array(ipx),array(ipxg),array(ipde),array(ipdl), &
          array(ipw),array(ipphi),array(ipdpx), &
          array(iptr),array(ipt3),array(ipq1))

!     --- temperature : quadratic basis functions

      if (icheld==1) then
         ich221=10+jtcoorloc
      else
         ich221=10
      end if
      call elp221(ich221,ichh,inpelm,coorloc,index1,isign, &
          icurve,delta,array(ipx),array(ipxg), &
          array(ipw),array(ippsi),array(ipdpsx))

!     --- fetch solution at previous time if required, and
!           compute mass matrix

      if ((icheld==1).and.(jtime>=1)) then
         ich1=0
         call el4021(ich1,ncomp,solold,index2,induv, &
           array(ipu),array(ipu0),array(ipdux), &
           array(ipphi),array(ipdpx),array(ippsi),array(ipdpsx), &
           array(iptr),array(ipmt),n7,n7,inpelm)
         call el2002(-nstart,ichint+1,rhoh,array(ipmas),array(ipw), &
                     array(ipphi),n7,n7,n7,array(ipvc),array(ippsi), &
                     delta,.true.)
      end if

!     --- fetch solution and derivatives if required in
!           nodal points and/or integration points

      if ((icheld==1).and.(modelv==1).and.(linear==0)) then
         ich1=0
      else
         ich1=-1
      end if
      if ( .true. .or. numold<=1) then
         call el4021(ich1,ncomp,sol,index2,induv, &
           array(ipu),array(ipug),array(ipdux), &
           array(ipphi),array(ipdpx),array(ippsi),array(ipdpsx), &
           array(iptr),array(ipmt),n7,n7,inpelm)

!        --- get previous vectors if present

      else if (numold>1) then
         call el4021(ich1,ncomp,sol,index2,induv, &
           array(ipu),array(ipug),array(ipdux), &
           array(ipphi),array(ipdpx),array(ippsi),array(ipdpsx), &
           array(iptr),array(ipmt),n7,n7,inpelm)
         jdiag  = 2
         jderiv = 0
         jtrans = 0
         jdercn = 0
         jind   = 1
         jsol   =-1

         do 100 numveclc = 2, numold

!        --- temporarily: constant number of degrees of freedom (no warning)

            iseqtype = oldsols(numveclc)%iseqtype
            index4n => indoldsols(iseqtype)%index4
            junkp  = index4n(1)
            if (junkp>1) then
               write(*,'('' junkp>1 in ELM420 not implemented'')')
               call instop
            end if
            call el4402 ( data , ddatdx , &
                array(ippsi), array(ipdpsx), &
                n7, inpelm, ndimloc, jcomp, index1, array(iptr), &
                datag , jtrans, numold, numveclc, junkp, jdercn, &
                jderiv, jdiag, jind, jsol)
         do 90  i = 1, junkp
            do 89 k=1,n7
               ih=(i-1)*n7
               vecloc(numveclc,i,k)=datag((i-1)*n7+k)
89          continue
90       continue
100      continue
      end if
      if (.false. .or. numold>=2) then
         do 120  i = 1, ncomp
            ih = (i-1)*n7-1
            do 119 k=1,n7
               vecloc(1,i,k)=array(ipug+ih+k)
119         continue
120      continue
      end if

!     --- if not constant compute the external (driving+conservative) forces
!         and the viscosity as function of (x,y)

      if ((icheld==1).or.(icheld==5)) then
         call el4041(1,array(ipxg),array(ipug),array(ipphi), &
                     index1,n7,n7,ncomp,iuser,user, &
                     jtime,method,sigmalc,rho,linear,irotat,omega, &
                     array(ipfg),modelv,array(ipet),cn,cl,ct, &
                     temp1,indeth,array(ipvc),array(ipbeta), &
                     array(iptem0),array(ipcp),array(ipcapp),indf1, &
                     indf2,indf3,indt0,indold,ialpha)
         if ((indeth/=0).or.(ct>0.0d0)) then
            ichiv=1
         else
            ichiv=ichint
         end if
      end if
      if (indeth>10000 .and. indeth<100000000) then
         do 130 k = 0, n7-1
            array(ipet+k) = funcc3 ( indeth-10000, array(ipxg+k), &
                            array(ipxg+n7+k),array(ipxg+k), &
                            numold,maxunk, vecloc(1,1,k+1) )
130      continue
      end if
      if (indf1>10000 .and. indf1<100000000) then
         do 140 k = 0, n7-1
            array(ipfg+k) = funcc3 ( indf1-10000, array(ipxg+k), &
                            array(ipxg+n7+k),array(ipxg+k), &
                            numold,maxunk, vecloc(1,1,k+1) )
140      continue
      end if
      if (indf2>10000 .and. indf2<100000000) then
         do 150 k = 0, n7-1
            array(ipfg+k+n7) = funcc3 ( indf2-10000, array(ipxg+k), &
                            array(ipxg+n7+k),array(ipxg+k), &
                            numold,maxunk, vecloc(1,1,k+1) )
150      continue
      end if
      if (indf3>10000 .and. indf3<100000000) then
         do 160 k = 0, n7-1
            array(ipfg+k+2*n7) = funcc3 ( indf3-10000, array(ipxg+k), &
                            array(ipxg+n7+k),array(ipxg+k), &
                            numold,maxunk, vecloc(1,1,k+1) )
160      continue
      end if

!     --- compute effective viscosity

      if (icheld==1) then
         call el4040(0,itcoorloc,n7,n7,ndimloc, &
             modelv,array(ipet),cn,cl,ct,temp1,array(ipvc), &
             array(ipxg),array(ipug),array(ipdux), &
             array(ipvsc),array(ipmt),array(ipinv),array(ipdsc))
      else

!     --- compute derived quantities

         call el4042(icheld,ix,jdegfd,itcoorloc,n7,ndimloc, &
                     modelv,array(ipet),cn,cl,ct,temp1, &
                     array(ipvc),array(ipx), &
                     array(ipu),array(ipdux),array(ipmt), &
                     elemvc1,elemwg)
         call erclos( 'eld420' )
         return
      end if

!     --- compute ndimloc rows of viscous matrix and initialize element matrix
!             array(ipmt)
!         --- set matrix equal to zero

      ill=n14*n14-1
      do 1010 i=0,ill
         array(ipmt+i)=0.0d0
1010  continue
      call el4022(ichiv,itcoorloc,array(ipvsc),array(ipmt),array(ipw), &
          array(ipxg),array(ipphi),array(ipdpx), &
          nstart,n7,n7,n14,array(ippsi),array(ipdpsx),inpelm, &
          array(ipdl),array(ipq1),delta,array(ipvc))

!     --- compute penalty matrix if required, but do not add it

      if (method==1) &
          call el4025(itcoorloc,ichint,0,array(ipmt),array(ippen), &
          sigmalc,isign,delta,array(ipde),array(ipw),array(ipxg), &
          array(ipphi),array(ipdpx),n7,n7,jcomp,n14)

!     --- compute element vector
!         fill (driving) external forces into vector (initialize it)
!         --- add influence of buoyancy to second momentum equation
!         (minus sign and 1/rho due to action in el4041)

      do 1020 i=0,n7-1
         array(ipwrk+i)=array(ipfg+i)
         array(ipwrk+n7+i)=array(ipfg+n7+i) - array(ipbeta+i)* &
             (array(ipug+n14+i)-array(iptem0+i))/rho
1020  continue
      call el4031(ichint,array(ipwrk),array(ipphi), &
          rho,array(ipw),array(ipvc),nstart,n7,n7,n14)

!     --- compute influence of convective acceleration for linear>0

      if (linear>0) then
         call el4033(ichint,itcoorloc,-rho,array(ipug),array(ipdux), &
                      array(ipw),array(ipxg),array(ipphi),array(ipvc), &
                      nstart,n7,n7,ndimloc,array(ippsi),inpelm)
      end if

!     --- compute coriolis acceleration and centrifugal force if required

      if (irotat==1) then
         call el4032(ichint,itcoorloc,array(ipphi),rho,omega,array(ipug), &
            array(ipw),array(ipvc),nstart,n7,n7,array(ippsi),inpelm)
      end if

!     --- substract m(u-uold)

      if (jtime==1) then
         do 1030 i=1,n14
            array(ipu0+i-1)=array(ipu0+i-1)-array(ipug+i-1)
1030     continue
         call el2015(2,ichint+1,array(ipmas),array(ipu0),1d0, &
                 array(ipvc),n7,n7,n7)
         call el2015(2,ichint+1,array(ipmas),array(ipu0+n7),1d0, &
                 array(ipvc+n7),n7,n7,n7)
      end if

!     --- compute pressure in centroid for penalty method

      call el4034(pc,sigmalc,delta,array(ippen),array(ipu),n7,ndimloc)

!     --- compute gradient of pressure and fill elemvc1 and elemwg

       call el4035(ichint,itcoorloc,elemvc1,elemwg,pc,array(ipphi), &
          array(ipxg),array(ipx),array(ipu),array(ipmt), &
          array(ipvc),array(ipw),n7,n14,ndimloc)
       call erclos( 'eld420' )
       end
