      subroutine elfillpsipres ( psipres, npres, xgauss, x, icent )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 10-02-2010
!
!   copyright (c) 2010-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill the pressure basis functions in the integration points
!     Crouzeix Raviart elements
! **********************************************************************
!
!                       KEYWORDS
!
!     pressure
!     basis_function
!     crouzeix_raviart
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer npres, icent
      double precision psipres(npres,m), xgauss(m,*), x(n,*)

!     icent          i    Index of centroid
!     npres          i    Number of pressure parameters per element
!     psipres        o    Contains the pressure basis functions
!                         psipres(i,k) refers to i-th basis function in
!                         integration point k
!     x              i    array containing the coordinates of the nodal points
!                         according to:
!                         x  = x (i,1);  y  = x (i,2);  z  = x (i,3)
!                          i              i              i
!     xgauss         i    array of length m x ndim containing the co-ordinates
!                         of the gauss points.  array xgauss is only filled when
!                         gauss integration is applied.
!                         xg_i = xgauss (i,1);  yg_i = xgauss (i,2);
!                         zg_i = xgauss (i,3)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j

!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elfillpsipres' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elfillpsipres'
         write(irefwr,1) 'npres, m, n', npres, m, n
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      psipres(1,1:m) = 1d0
      if ( npres>1 ) then

!     --- npres>1

         do j = 1, npres-1
            psipres(1+j,1:m) = xgauss(1:m,j) - x(icent,j)
         end do  ! j = 1, ndimlc

      end if  ! ( npres>1 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( psipres, m, npres, 'psripres' )
         write(irefwr,*) 'End elfillpsipres'

      end if  ! ( debug )
      call erclos ( 'elfillpsipres' )

      end
