      subroutine elm900parms ( iuser )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.1  date 03-02-2015 change in comment
!        version  2.0  date 21-08-2013 New parameter list
!        version  1.0  date 20-02-2010
!
!   copyright (c) 2010-2015  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute parameters irule, m and n for type elements of type 900
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     initialization
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*)

!     iuser          i    Integer user array to pass user information from
!                         main program to subroutine. See STANDARD PROBLEMS
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer interpol, ioldm, ishift, jcheld, jelp, jtime, kchois, &
              nveloc, mcont, modelrhsd, modelv, mrule, ncoef, ipend
      double precision thetdt
      logical :: compress

!     compress       If true compressible has been read
!                    they are not
!     interpol       If 0 the standard interpolation is applied, &
!                    if 1 the interpolation is restricted to linear
!                    subelements
!     ioldm          Sequence number of velocity at end of previous time
!                    step in input vector.
!                    If 0 no iterative procedure is adopted for dynamical
!                    analyses.
!     ipend          Last position in array
!     ishift         Shift of specific variable with respect to diagonal
!     jcheld         if ( icheld<= 10 ) then icheld else icheld-20
!     jelp           Indication of the type of basis function subroutine
!                    must be called by ELxxxx.
!     jtime          Integer parameter indicating if the mass matrix for
!                    the time-dependent equation must be computed (>0)
!                    or not ( = 0)
!     kchois         Indication what type of parameters must be computed
!     mcont          Defines the type of continuity equation
!     modelrhsd      Defines how the stress tensor is treated in the rhs
!     modelv         Defines the type of viscosity model
!     mrule          Type of integration rule as given by the user
!     ncoef          Number of coefficients in the equation
!     nveloc         Number of velocity parameters
!     thetdt         1/(theta dt) for the case of the theta method
!                    immediately applied
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELM900PARMS1   Compute parameters irule, m and n for type elements of
!                    type 900
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elm900parms' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elm900parms'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First part, compute parameters

      if ( itype>500 ) then

!     --- Standard type numbers 900, 901, 902, 904

         call elm900parms1 ( iuser, interpol, ioldm, ishift, jcheld, &
                             jelp, jtime, kchois, nveloc, mcont, &
                             modelrhsd, modelv, mrule, ncoef, &
                             thetdt, compress )

      else

!     --- type numbers 410

         n = inpelm
         m = n
         npsi = 4
         ncomp = 3

      end if  ! ( itype>500 )

1000  call erclos ( 'elm900parms' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End elm900parms'

      end if  ! ( debug )

      end
