      subroutine elmatmultrect ( sp, pold, elemvc, nrow, ncol, jadd )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-04-2005
!
!   copyright (c) 2005-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Multiply rectangular matrix sp of size (nrow x ncol) by
!     vector of size ncol to get vector of size nrow
! **********************************************************************
!
!                       KEYWORDS
!
!     matrix_vector_multiplication
!     element
!     full_matrix
!     rectangle
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nrow, jadd, ncol
      double precision sp(nrow,ncol), pold(ncol), elemvc(nrow)

!     elemvc        i/o   Result of multiplication
!     jadd           i    if 0 no addition takes place
!     ncol           i    Number of rows in sp
!     nrow           i    Number of columns in sp
!     pold           i    old pressure
!     sp             i    Gradient of pressure matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j
      double precision sum

!     i              Counting variable
!     j              Counting variable
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'elmatmultrect' )
      debug = .false.
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from elmatmultrect'
         write(irefwr,1) 'jadd, nrow, ncol', jadd, nrow, ncol
         call prinrl ( pold, ncol, 'pold' )
         call prinrl1 ( sp, ncol, nrow, 'sp' )
  1      format ( a, 1x, (10i6) )

      end if
      if ( ierror/=0 ) go to 1000
      if ( jadd==0 ) then

!     --- jadd = 0, do not add to existing vector

         do i = 1, nrow
            sum = 0d0
            do j = 1, ncol
               sum = sum + sp(i,j)*pold(j)
            end do
            elemvc(i) = sum
         end do

      else

!     --- jadd # 0, add to existing vector

         do i = 1, nrow
            sum = 0d0
            do j = 1, ncol
               sum = sum + sp(i,j)*pold(j)
            end do
            elemvc(i) = elemvc(i)+sum
         end do

      end if  ! ( jadd==0 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl ( elemvc, nrow, 'elemvc' )
         write(irefwr,*) 'End elmatmultrect'

      end if
      call erclos ( 'elmatmultrect' )

      end
