      subroutine sepel900corr
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 12-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     compute the rotatio part of the element matrix for navier-stokes
!     and store in matrix s
!     Old subroutine el4914
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: i, j
      double precision :: work(m), S1(n,n)
      double precision, pointer :: S(:,:)

!     S              sub matrix
!     S1             sub matrix
!     i              Counting variable
!     j              Counting variable
!     work           Work array
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL         Print 1d real vector
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Computation of the rotation matrix results in a 3x3 block matrix, each
!     consisting of nxn matrices of the following shape:
!
!            |   0       S      0     |
!            |            uv          |
!            |                        |
!            |   S       0      0     |
!     S =    |    vu                  |
!            |                        |
!            |   0       0      0     |
!            |                        |
!
!
!     The submatrices have the following shape:
!
!     S uv =  -2 rho omega phi(j) phi(i)
!     S vu =   2 rho omega phi(j) phi(i)
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel900corr' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel900corr'
         call prinrl ( E%coeffs(:,E%indangvel), m, 'omega' )
         call prinrl1 ( E%submat, n*E%ndim, n*E%ndim, 'elemmt' )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Compute the matrix 2 rho omega phi(j)*phi(i)

      work = 2d0*B%w*E%coeffs(:,E%indangvel)*E%rho

      if ( B%jdiag==1 ) then

!     --- jdiag=1, phi_ik = delta_ik
!         Add matrix Suv

         S => E%submat(1:n*E%ndim:E%ndim,2:n*E%ndim:E%ndim)

         do i = 1, n
            S(i,i) = S(i,i)+work(i)
         end do  ! i = 1, n

!     --- Add matrix Svu

         S => E%submat(2:n*E%ndim:E%ndim,1:n*E%ndim:E%ndim)
         do i = 1, n
            S(i,i) = S(i,i)-work(i)
         end do  ! i = 1, n

      else

!     --- jdiag#1, phi general
!         Add matrix Suv

         S => E%submat(1:n*E%ndim:E%ndim,2:n*E%ndim:E%ndim)
         do i = 1, n
            do j = 1, n
               S1(i,j) = sum(work*B%phi(i,:)*B%phi(j,:))
            end do  ! j = 1, n
         end do  ! i = 1, n

         if ( debug ) call prinrl1 ( S1, n, n, 'S1' )

         do i = 1, n
            do j = 1, n
               S(i,j) = S(i,j)+S1(i,j)
            end do  ! j = 1, n
         end do  ! i = 1, n

!     --- Add matrix Svu

         S => E%submat(2:n*E%ndim:E%ndim,1:n*E%ndim:E%ndim)

         do i = 1, n
            do j = 1, n
               S(i,j) = S(i,j)-S1(i,j)
            end do  ! j = 1, n
         end do  ! i = 1, n

      end if  ! ( B%jdiag==1 )

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( E%submat, n*E%ndim, n*E%ndim, 'elemmt' )
         write(irefwr,*) 'End sepel900corr'

      end if  ! ( debug )
      call erclos ( 'sepel900corr' )

      end subroutine sepel900corr
