      subroutine sepel900rhscon
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute element vector for Navier-Stokes element due to convection
!     Old subroutine elm900rhsd
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     navier_stokes_equation
!     convection
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: adams

!     adams          Multiplication factor for the Adams-Bashfort method
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERTRACE        Prints trace back of subroutines
!     INSTOP         Stop the program
!     SEPEL900RHS1   compute the convective part of the element vector
!                    for navier-stokes and add in consisting element vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel900rhscon' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel900rhscon'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( E%typeconvect==4 ) then
         if ( E%typetime>0 ) then
            adams = -23d0/12d0
         else
            adams = -1d0
         end if
      else
         adams = 1d0
      end if

      if ( E%modelconv>0 ) then

!     --- compute convective terms where the coefficients are stored
!         in the special way

         call sepel900rhssp ( adams )

      else if ( E%typecontinuity==2 ) then

!     --- compute convective terms in case of Goertler equations

            call sepel900rhsg ( adams )

      else

!     --- compute convective terms in standard case

         call sepel900rhs1 ( adams )

      end if  ! ( E%modelconv>0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepel900rhscon'

      end if  ! ( debug )
      call erclos ( 'sepel900rhscon' )

      end subroutine sepel900rhscon
