      subroutine sepelm900mata
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 21-07-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Assemble element matrix using submatrices created before
!
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulecactl
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELM900MATI  Create element matrix for Navier-Stokes equations
!                    in case of integrated method using separate submatrices
!     SEPELM900MATP  Create element matrix for Navier-Stokes equations in case
!                    of penalty method using separate submatrices
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm900mata' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm900mata'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( itype==901 .or. itype==902 ) then

!     --- Type 901, 902, integrated method

         call sepelm900mati

      else

!     --- Type 900, penalty method

         call sepelm900matp

      end if  ! ( itype==901 .or. itype==902 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelm900mata'

      end if  ! ( debug )
      call erclos ( 'sepelm900mata' )

      end subroutine sepelm900mata
