      subroutine sepelmodconvec
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 31-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill modelconv, type_compress and typeconvect for
!     Navier-Stokes equation
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     navier_stokes_equation
!     convection
!     compressible
!     incompressible
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulecomio
      use sepmodulestrings
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer :: sepelgetintcf, sepfilldefstr

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPELGETINTCF  Return with the actual integer value corresponding to
!                    iuser(ipos)
!     SEPFILLDEFSTR  Give sepfilldefstr either the default value defval or the
!                    value of the variable namestring if it exists
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1305   Coefficient has wrong value
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelmodconvec' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelmodconvec'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      E%typeconvect = &
         sepelgetintcf ( codelinear, nncodelinear, nscodelinear, &
                         1, 'linearization' )
      E%methodupw = E%typeconvect/1000
      E%typeconvect = E%typeconvect-1000*E%methodupw
      E%modelconv = E%typeconvect/100
      E%typeconvect = E%typeconvect-100*E%modelconv
      E%typecontinuity = E%typeconvect/10
      E%typeconvect = E%typeconvect-10*E%typecontinuity

      if ( E%typecontinuity==0 ) then

!     --- E%typecontinuity = 0, check if there is string variable with value
!         continuity_eq

         E%typecontinuity = &
            sepfilldefstr ( 'CONTINUITY_EQ', 0, codecont, &
                            nncodecont, nscodecont, ncodecont )
         if ( E%typecontinuity>0 ) E%typecontinuity = E%typecontinuity-1

      end if  ! ( E%typecontinuity==0 )

      if ( debug ) then
         write(irefwr,1) 'methodupw, modelconv, typecontinuity, typeconvect', &
                          E%methodupw, E%modelconv, E%typecontinuity, &
                          E%typeconvect
      end if  ! ( debug )

      if ( E%typeconvect<0 .or. E%typeconvect>3 .or. E%typecontinuity>5 .or. &
           E%modelconv>1 .or. E%methodupw>3 ) then

!     --- E%typeconvect<0 or E%typeconvect>3 or E%typecontinuity>5 or
!         E%modelconv>1 or E%methodupw>3
!         Such values are not allowed

         call errint ( 5, 1 )
         call errint ( itype, 2 )
         call errint ( ielgrp, 3 )
         call errint ( ielem, 4 )
         call errint ( E%typeconvect+10*E%typecontinuity+100*E%modelconv+ &
                       1000*E%methodupw, 5 )
         call errchr ( 'typeconvect', 1 )
         call errsub ( 1305, 5, 0, 1 )

      end if

      E%compressible = E%typecontinuity>0

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelmodconvec'

      end if  ! ( debug )
      call erclos ( 'sepelmodconvec' )

      end subroutine sepelmodconvec
