      subroutine sepelnsdefor ( elemvc, gradv, u, nelemvc, ndcomp, mdecl )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 29-09-2016
!
!   copyright (c) 2016-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the deformation for the Navier-Stokes equations
!
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     navier_stokes_equation
!     viscosity
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
     integer, intent(in) :: nelemvc, ndcomp, mdecl
     double precision, intent(in) :: gradv(9,m), u(mdecl,*)
     double precision, intent(inout) :: elemvc(ndcomp,nelemvc)

!     elemvc         o    Element vector
!     gradv          i    Array containing the gradient vector in
!                         the quadrature points
!     mdecl          i    number of quadrature points in element,
!                         same as  m  in common /celint/
!     ndcomp         i    Indicates the number of quantities that must be filled
!                         in elemvc and elemwg per nodal point
!     nelemvc        i    Length parameter for elemvc
!     u              i    array of length m x ncomp in which the values of
!                         u in the integration points are stored the sequence:
!                         u  (x ) = u(j,i)  j = 1(1)m, i = 1 (1) ncomp
!                          i   j
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: k
      double precision :: u1, u2

!     k              Counting variable
!     u1             First component of velocity vector
!     u2             Second component of velocity vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelnsdefor' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelnsdefor'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( ndimlc==2 .and. jcart/=4 ) then

!     --- ndimlc = 2, except cylindrical co-ordinates with swirl

         do k = 1, nelemvc
            u1 = u(k,1)
            u2 = u(k,2)
            if ( abs(u1)<=sqreps .and. abs(u2)<=sqreps ) &
               then
               elemvc(1,k) = 0d0
            else
               elemvc(1,k) = (2d0*u1*u2*(-gradv(1,k)+ &
                              gradv(4,k)) + (u1**2-u2**2)* &
                              (gradv(2,k)+gradv(3,k)) ) / &
                             (u1**2+u2**2)
            end if
         end do

      else

!     --- ndimlc = 3 or cylindrical co-ordinates with swirl

         if ( itype==410 ) then

!        --- itype = 410, the sequence of the shear rate is defined
!            different from usual

            do k = 1, nelemvc
               elemvc(1,k) = gradv(2,k)+gradv(4,k)
               elemvc(3,k) = gradv(6,k)+gradv(8,k)
               elemvc(2,k) = gradv(3,k)+gradv(7,k)
            end do

         else

!        --- standard situation

            do k = 1, nelemvc
               elemvc(1,k) = gradv(2,k)+gradv(4,k)
               elemvc(2,k) = gradv(6,k)+gradv(8,k)
               elemvc(3,k) = gradv(3,k)+gradv(7,k)
            end do

         end if

      end if

1000  call erclos ( 'sepelnsdefor' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelnsdefor'

      end if  ! ( debug )

      end subroutine sepelnsdefor
