      subroutine sepelphipress
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 04-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array phipress with the pressure basis functions
!     Navier-Stokes Crouzeix-Raviart elements
!     Old Subroutine elfillpsipres
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     navier_stokes_equation
!     pressure
!     basis_function
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer :: j

!     j              Counting variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelphipress' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelphipress'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      do j = 1, npsi-1
         B%phipress(j+1,:) = B%xyzg(:,j)-B%xyz(E%icent,j)
      end do  ! j = 1, npress-1

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( B%phipress, B%m, npsi, 'phipress' )
         write(irefwr,*) 'End sepelphipress'

      end if  ! ( debug )
      call erclos ( 'sepelphipress' )

      end subroutine sepelphipress
