      subroutine sepelsecinv
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-09-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute second inverse in case of Navier-Stokes
!     old subroutine sepelnssecinv
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     navier_stokes_equation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: secinv(:), a1(:,:)

!     a1             tensor grad U + grad U ^T
!     secinv         Array containing the second invariant in the
!                    quadrature points
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!    The second inverse II is defined as II = 1/2 A_1:A_1
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelsecinv' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelsecinv'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      a1 => E%a1
      secinv => E%secinv

      if ( E%ndim==2 .and. (B%cart==1 .or. B%cart==3) ) then

!     --- E%ndim = 2, Cartesian or Polar co-ordinates

         secinv = 0.5d0*(a1(1,:)**2+a1(2,:)**2)+a1(4,:)**2

      else if ( E%ndim==2 .and. B%cart==2 ) then

!     --- E%ndim = 2, Cylinder co-ordinates

         secinv = 0.5d0*(a1(1,:)**2+a1(2,:)**2+a1(3,:)**2)+a1(4,:)**2

      else

!     --- E%ndim = 3

         secinv = 0.5d0*(a1(1,:)**2+a1(2,:)**2+a1(3,:)**2)+&
                         a1(4,:)**2+a1(5,:)**2+a1(6,:)**2


      end if  ! ( E%ndim==2 .and. (B%cart==1 .or. B%cart==3) )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelsecinv'

      end if  ! ( debug )
      call erclos ( 'sepelsecinv' )

      end subroutine sepelsecinv
