      subroutine el2730 (D, B, nd, B4)
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  3.0  date 06-01-2016 Use sepmoduleelem
!        version  2.0  date 13-04-2005 Extra parameters for axi-symmetry (RvL)
!        version  1.1  date 01-11-2002 Extension for 2D shapes (RvL)
!
!   copyright (c) 2000-2016  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Tangential stiffness matrix for delta(B)
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmoduleelem
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer nd
      double precision D(nd,nd), B(3*ndim-3), B4

!     b              i    Finger strain
!     b4             i    ?
!     d              o    D-matrix
!     nd             i    Dimension of tangent stiffness matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer idxlen1, idxlen2
      parameter (idxlen1 = 35, idxlen2 = 114)
      integer idx1(idxlen1), idx2(idxlen2), sum, i, j, p, q

!     i              Counting variable
!     idx1           Array used to fill the tangent stiffness matrix(2D)
!     idx2           Array used to fill the tangent stiffness matrix(3D)
!     idxlen1        Length of array idx1
!     idxlen2        Length of array idx2
!     j              Counting variable
!     p              ?
!     q              rotation matrix (local to global)
!     sum            Help parameter to compute a sum
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data idx1  / 4, 1,1, 1,1, 3,4, 4,4, &
                   4, 2,2, 2,2, 3,3, 4,3, &
                   8, 1,3, 1,3, 2,4, 2,4, &
                      3,1, 3,2, 4,1, 4,2 /
      data idx2  /  6, 1,1, 1,1, 4,7, 6,6, 7,7, 9,6 , &
                   6, 2,2, 2,2, 4,4, 5,8, 7,4, 8,8 , &
                   6, 3,3, 3,3, 5,5, 6,9, 8,5, 9,9 , &
                  12, 1,4, 1,4, 2,7, 2,7, 4,1, 4,2 , &
                      5,6, 6,8, 7,1, 7,2, 8,6, 9,8 , &
                  12, 2,5, 2,5, 3,8, 3,8, 4,9, 5,2 , &
                      5,3, 6,4, 7,9, 8,2, 8,3, 9,4 , &
                  12, 1,9, 1,9, 3,6, 3,6, 4,5, 5,7 , &
                      6,1, 6,3, 7,5, 8,7, 9,1, 9,3 /
! ======================================================================
!
      if (ndim==2) then

         sum = 1
         do i = 1, 3
            do j = 0, idx1(sum)-1
               p = idx1(sum+1+(j*2))
               q = idx1(sum+1+(j*2)+1)
               D(p,q) = D(p,q) + B(i)
            end do
            sum = sum+idx1(sum)*2+1
         end do

!        --- axisymmetric case

         if (jcart==2) then
            D(nd,nd)=2*B4
         end if

      else if (ndim==3) then

         sum = 1
         do i = 1, 6
            do j = 0, idx2(sum)-1
               p = idx2(sum+1+(j*2))
               q = idx2(sum+1+(j*2)+1)
               D(p,q) = D(p,q) + B(i)
            end do
            sum = sum+idx2(sum)*2+1
         end do

      end if

      end
