      subroutine el8107 ( elemvc, eps, mdecl, nd, icheld )
! ======================================================================
!
!        programmer    Guus Segal
!        version  2.0  date 26-06-2014 Extra parameter ichled
!        version  1.4  date 20-08-1999 Allow integration points
!        version  1.3  date 28-12-1998 Include common
!
!   copyright (c) 1992-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill array eps into elemvc
!     At this moment linear elements are assumed with one integration point
! **********************************************************************
!
!                       KEYWORDS
!
!     derivative
!     elasticity_equation
!     strain
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      use sepmodulecactl
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer mdecl, nd, icheld
      double precision elemvc(6,*), eps(mdecl,*)

!     elemvc   o     If icheld = 2 then elemvc contains the element vector at
!                    output
!     eps      i     Array containing the value of the strain epsilon in the
!                    integration points
!                               k
!                    epsilon  (x ) = eps(i,k)
!                            i
!     icheld   i     Choice parameter to distinguish between the various
!                    possibilities of this subroutine. See eld250
!     mdecl    i     Number of integration points in the element
!     nd       i     Dimension of elasticity matrix D
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer i, j

!     i              General loop variable
!     j              General loop variable
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!     837   integration rule not yet available
!    3189   option not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'el8107' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from el8107'
         write(irefwr,1) 'nd, m, n, mdecl', nd, m, n, mdecl
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( nd<=4 ) then

!     --- ndim = 2

         if ( m==1 ) then

!        --- m = 1    linear triangle   or  bilinear quadrilateral

            if ( nd==3 ) then

!           --- nd = 3, fill epsx, epsy and gammaxy

               do i = 1, n
                  elemvc(1,i) = eps(1,1)
                  elemvc(2,i) = eps(1,2)
                  elemvc(4,i) = eps(1,3)
               end do

            else

!           --- nd = 4, fill epsr, epsz, eps theta and gammarz

               do i = 1, n
                  do j = 1, nd
                     elemvc(j,i) = eps(1,j)
                 end do
               end do

            end if

         else if ( m/=1 .and. jdiag==1 ) then

!        --- Newton-Cotes integration, &

            if ( nd==3 ) then

!           --- nd = 3, fill epsx, epsy and gammaxy

               do i = 1, n
                  elemvc(1,i) = eps(i,1)
                  elemvc(2,i) = eps(i,2)
                  elemvc(4,i) = eps(i,3)
               end do

            else

!           --- nd = 4, fill epsr, epsz, eps theta and gammarz

               do i = 1, n
                  do j = 1, nd
                     elemvc(j,i) = eps(i,j)
                  end do
               end do

            end if
         else

!        --- Not available yet

            call errsub ( 837, 0, 0, 0 )

         end if

      else

!     --- ndim = 3

         if ( m==1 ) then

!        --- m = 1, copy eps in elemvc

            do j = 1, nd
               elemvc(j,1:n) = eps(1,j)
            end do  ! j = 1, nd

         else if ( m==n ) then

!        --- m = n, copy eps in elemvc

            do j = 1, nd
               elemvc(j,1:n) = eps(1:n,j)
            end do  ! j = 1, nd

         else if ( icheld==17 .or. icheld==47 ) then

!        --- special case, copy sigma in elemvc per integration point

            do j = 1, nd
               elemvc(j,1:m) = eps(1:m,j)
            end do  ! j = 1, nd

         else

!        --- other options not yet implemented

            call errint ( m, 1 )
            call errchr ( 'm', 1 )
            call errsub ( 3189, 1, 0, 1 )

         end if  ! ( m==1 )

      end if

1000  call erclos ( 'el8107' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End el8107'

      end if  ! ( debug )

      end
