      subroutine el8304 (Fd, Fn, p, coef, D_t, sigma, ichoice)
! ======================================================================
!
!        programmer    Chris van Oijen
!        version  1.1  date 21-11-2002 Minor change
!        version  1.0  date 28-05-2001
!
!   copyright (c) 2001-2005  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Incompressible Mooney-Rivlin constitutive law
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ichoice
      double precision D_t(9,9), Fd(3,3), Fn(3,3), &
                       p, coef(*), sigma(*)

!     coef           i    element coefficients
!     d_t            o    matrix to be filled
!     fd             i    Deformation gradient F(k,j,i) = F_ij(x^k)
!     fn             i    deformation gradient at start of timestep
!     ichoice        i    calculate tang. stifness matrix+stress(0)
!                         or just stress(1)
!     p              i
!     sigma          o    stress to be filled
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer idxlen
      parameter (idxlen = 114)
      integer idx(idxlen)
      integer i, j
      double precision B(6), B2(6), trB, c1, c2, D1(9,9), D2(9,9), &
                       D3(9,9), D4(9,9), F(3,3), b4

!     b              B-matrix
!     b2             constant
!     b4             ?
!     c1             Cash-Karp parameters (see Press et al. (1992))
!     c2             Helping constant
!     d1             array of length modecl x modecl containing the values
!                    of the derivatives of the basis functions in the
!                    x-direction
!     d2             array of length modecl x modecl containing the values
!                    of the derivatives of the basis functions in the
!                    y-direction
!     d3             ?
!     d4             ?
!     f              ?
!     i              Counting variable
!     idx            ?
!     idxlen         ?
!     j              Counting variable
!     trb            ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     DGEMM          BLAS routine for matrix multiplication general.
!                    (NAG: f06yaf)
!     EL2730         ?
!     EL2731         ?
!     ELMUL01        ?
!     ELMUL03        ?
!     ELMUL04        ?
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data idx  /  6, 1,1, 1,1, 4,7, 6,9, 7,4, 9,6 , &
                   6, 2,2, 2,2, 4,7, 5,8, 7,4, 8,5 , &
                   6, 3,3, 3,3, 5,8, 6,9, 8,5, 9,6 , &
                  12, 1,4, 1,7, 2,4, 2,7, 4,1, 4,2 , &
                      5,6, 6,5, 7,1, 7,2, 9,8, 8,9 , &
                  12, 2,5, 2,8, 3,5, 3,8, 4,6, 5,2 , &
                      5,3, 6,4, 7,9, 8,2, 8,3, 9,7 , &
                  12, 1,6, 1,9, 3,6, 3,9, 4,5, 5,4 , &
                      6,1, 6,3, 7,8, 8,7, 9,1, 9,3 /
! ======================================================================
!
!     --- Compute Finger strain
!                         c     c
!         B = Fd * Fn * Fn  * Fd

      call elmul03 (Fd, Fn, F, 0, 3)
      call elmul01 (F, B, 0, 3)

!     --- Compute B^2

      call elmul04 (B, B, B2, 0)

!     --- Compute tr(B)

      trB = B(1) + B(2) + B(3)

!     --- Get material parameters

      c1 = coef(10-5)
      c2 = coef(11-5)

!     --- Compute tangential stiffness

      if (ichoice==0) then

!     --- fill in D_t
!         D_t = (c1 + c2*tr(B))*delta(B) + c2*tr(delta(B))B - &
!                c2*(delta(B)*B + B*delta(B))

         do i = 1, 9
            do j = 1, 9
               D1(i,j) = 0d0
               D2(i,j) = 0d0
               D3(i,j) = 0d0
               D4(i,j) = 0d0
            end do
         end do

!        --- D1 = delta(B)

         B4 = 0d0
         call el2730 (D1, B, 9, B4)

!        --- second part

         do i = 1, 6
            do j = 1, 6
               D2(i,j) = 2d0*B(j)*B(i)
            end do
            do j = 7, 9
               D2(i,j) = 2d0*B(j-3)*B(i)
            end do
         end do
         do i = 7, 9
            do j = 1, 6
               D2(i,j) = 2d0*B(j)*B(i-3)
            end do
            do j = 7, 9
               D2(i,j) = 2d0*B(j-3)*B(i-3)
            end do
         end do

!        --- third part D4 = D3*D1

         call el2731 (D3, idx, 6, B)
         call dgemm ('n', 'n', 9, 9, 9, 1d0, D3, 9, D1, 9, 0d0, &
                     D4, 9)

!        --- assemble

         do i = 1,9
            do j = 1, 9
               D_t(i,j) = (c1+c2*trB)*D1(i,j) + c2*D2(i,j) - c2*D4(i,j)
            end do
         end do

      end if

!     --- Compute stress
!         sigma = -pI + c1*B + c2*tr(B)*B - c2*B^2

      do i = 1, 3
         sigma(i) = -p - (c1+2d0*c2) + c1*B(i) + c2*trB*B(i) - c2*B2(i)
      end do
      do i = 4, 6
         sigma(i) = c1*B(i) + c2*trB*B(i) - c2*B2(i)
      end do

      end
