      subroutine sepel0255
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 28-07-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element independent quantities for element type 255
!     Old subroutine el0255
! **********************************************************************
!
!                       KEYWORDS
!
!     element_matrix
!     element_vector
!     elasticity_equation
!     initialization
!     parabolic_equation
!     thick_plate
!     plate
!     bending
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     SEPEL0250CF    Fill real coefficients for elements of type 250
!     SEPELBASEFNCH  Define parameters for choice of basis functions
!     SEPELIPOSSET   Set ipos and initialize some other quantities for element
!     SEPFILLDMATS   ill elasticity matrix D^s for bending problem, depending on
!                    the values of the elasticity E and the poisson ratio nu
!                    The matrix D^s is the elasticity matrix for shear stiffness
!     SEPGETRULE     Get the integration rule as stored in iuser
!     SEPGETSTRESSRL Read stress-strain relations and store information in E
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1771   Shape number not yet implemented
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepel0255' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepel0255'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- initialize ipos and some other quantities

      call sepeliposset
      if ( debug ) write(irefwr,1) 'ipos, ishift/0', E%ipos, E%ishift

!     --- Get information of coefficients

      E%ipos = E%ipos+1          ! skip first position
      E%ishift = 1

!     --- Get information of coefficients

      call sepgetstressrl  ! stress-strain relation
      if ( ierror/=0 ) go to 1000

!     --- Next quadrature rule and coordinate system

      call sepgetrule
      if ( ierror/=0 ) go to 1000

!     --- make default integration rule 2-pt Gauss

      if ( B%rule==0 ) B%rule = 3
      B%cart = 1

      E%ipos = E%ipos+2          ! skip two positions
      E%ishift = E%ishift+2

!     --- Get information of basis functions

      call sepelbasefnch
      if ( ierror/=0 ) go to 1000
      n = B%n
      m = B%m

      if ( E%ishape/=5 ) then

!     --- Error 1771: this type of element has not yet been implemented

         call errint ( E%ishape, 1 )
         call errint ( ielem, 2 )
         call errint ( itype, 3 )
         call errint ( ielgrp, 4 )
         call errsub ( 1771, 4, 0, 0 )

      end if  ! ( E%ishape/=5 )

!     --- Allocate D-matrix

      allocate ( E%Dmat(3,3,m), E%Dmats(2,2,m), &
                 E%bb(3,3,n,m), E%bs(2,3,n,m) )
      E%Dmat = 0d0
      E%Dmats = 0d0
      E%bb = 0d0
      E%bs = 0d0

!     --- Fill all real coefficients

      E%ncoef = 40
      call sepel0250cf

!     --- Fill Ds-matrix if constant

      if ( E%dmatconst ) call sepfilldmats

      E%fillbmat = matrix .or. E%initial_strain .or. E%initial_stress

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepel0255'

      end if  ! ( debug )
      call erclos ( 'sepel0255' )

      end subroutine sepel0255
