      subroutine sepelm250nl
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 13-06-2019
!
!   copyright (c) 2019-2019  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!
! **********************************************************************
!
!                       KEYWORDS
!
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulebuild
      use sepmodulecactl
      use sepmoduleoldvecs
      use sepmodulebasefn
      use sepmoduleelm
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision :: uold(B%n,E%ndim), glstrn(6,B%m), glstrr(6,27)
      double precision :: rotat(3,3), pos(3), matpar(21)
      integer :: t1, t2

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelm250nl' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelm250nl'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- set options

      E%rdint = E%modelelastnl==2 .or. E%modelelastnl==4
      E%rmhy  = E%special==3
      E%souza = E%special==1 .or. E%special==2

      if ( B%type_basisfunc==13  .and. E%rdint ) then
         E%mr = 1
      else if ( B%type_basisfunc==14  .and. E%rdint ) then
         E%mr = 8
      else
         E%mr = 0
      end if
      if ( E%first==0 ) allocate ( E%ugaussr(B%n,3), E%xgaussr(B%m,3) )

      if ( debug ) then
          write(irefwr,1) 'jelp, m, n, ndim, mr, nd', &
                           B%type_basisfunc, B%m, B%n, E%ndim, E%mr, E%dimD
          write(irefwr,*) 'rdint', E%rdint
      end if  ! ( debug )

!     --- Fill variable coefficients if necessary
!         First fill uold if necessary

      if ( E%compold>0 ) then

!     --- Store derivative  of prior vector and store in gradoldvec
!         Used to be: dfgrad

         if (E%mr==8) then

!        --- mr = 8, compute basis functions using B%rule = 2

            B => B2
            E%b2filled = .true.
            if ( E%first==0 ) then
               B%m = E%mr
               B%n = B1%n
               B%cart = B1%cart
               B%jdiag = B1%jdiag
               B%rule = 2
               B%type_basisfunc = B1%type_basisfunc
               deallocate ( E%ugaussr, E%xgaussr )
               allocate ( E%ugaussr(B%n,3), E%xgaussr(B%m,3) )

            end if

            call sepelp13  ! compute new derivate of basis function
                           ! store in E%gradred

            jsol = 1
            jderiv = 1
            call el2005 ( uold, E%gradred, B%phi, &
                          B%phider, E%nodes, E%ugaussr, iseqin )

         end if

      end if  ! ( E%compold>0 )

      if ( E%first==0 ) allocate ( E%gradred(B%m,3,3), E%detf0(B%n,B%m,3) )

      E%wr(1:B%m) = B%w
      B => B1

!     --- Do general stuff for nonlinear computation
!         make dfgrad from dudx (computed with el2005)

      call eldu2dfg ( gradoldvec, E%detf, B%xyzg, E%ugauss, B%m, &
                      E%ndim, B%cart )

!     --- reduced data (rdint) and data for Souza method computed with
!         ELRI01

      if ( (E%mr==1) .or. E%souza ) then

         call sepelri01

      else if ( E%mr==8 ) then

        call eldu2dfg ( E%gradred, E%detfr, E%xgaussr, E%ugaussr, E%mr, &
                        E%ndim, B%cart )

      end if

      E%dfgrad => gradoldvec
      E%ugaussloc => E%ugauss
      E%xgaussloc => B%xyzg

      if ( E%souza ) then
         call elsgmgls ( glstrn, gradoldvec, E%detf, E%detfr, B%m )
      else
         call sepeldf2gls ( glstrn, B%m, E%ndim, B%cart )
      end if

!     --- setup the blwork matrix

      E%blloc => E%bl
      E%dphidx => B%phider
      call sepelmkbl( B%m )

!     --- reduced integration ?

      if ( E%rdint ) then

         E%dfgrad => E%gradred
         E%ugaussloc => E%ugaussr
         E%xgaussloc => E%xgaussr
         call sepeldf2gls ( glstrr, E%mr, E%ndim, B%cart )

!        --- bl matrix for RI point

         E%blloc => E%blr
         E%dfgrad => E%gradred(1:E%mr,:,:)
         E%dphidx => E%detf0
         E%xgaussloc => E%xgaussr
         E%ugaussloc => E%ugaussr

         call sepelmkbl( E%mr )

      end if

!     --- get stress and stiffness info for every integration point

      do t1 = 1, B%m

!     --- local axes?

         if ( B%rule==1 ) then
            pos(1:E%ndim) = B%xyz(t1,:)
         else
            pos(1:E%ndim) = B%xyzg(t1,:)
         end if

!        --- local axes?

         if ( E%modelelastnl>=3 ) then

            call fnlocdir ( pos, rotat, ielgrp )

!           --- rotate glstrn from global to local axes

            call elrotsym ( glstrn(1,t1), rotat, 0 )

         end if

!        --- get material parameters from array

         do t2 = 1, 21
            matpar(t2) = E%coeffs(t1,t2)
         end do

!        --- compute stress estimates and tangential matrix

         call fnmateri ( E%choicefnmater, E%pksest(1,t1), E%Dmat(1,1,t1), &
                         glstrn(1,t1), E%detf(t1), matpar, matrix )

!        --- remove hydrostatics

         if ( E%rmhy ) then

            call elrmhy ( E%pksest(1,t1), E%Dmat(1,1,t1) )

         end if

!        --- rotate E%pksest and E%Dmat back to global axes, if local rotations

         if ( E%modelelastnl>=3 ) then

            call elrotsym ( E%pksest(1,t1), rotat, 1 )
            call elrotstf ( E%Dmat(1,1,t1), rotat )

         end if

      end do

      if ( debug ) write(irefwr,*) 'after do-loop'

!     --- if RI get stress and stiffness by calling FNCOMPRS
!         reset pksr and ser

      if ( E%rdint ) then

         do t1 = 1, E%mr

           E%pksr(:,t1) = 0d0
           E%ser(:,:,t1) = 0d0

!          --- for now assume constant matpar's, use last set

           call fncomprs( E%choicefnmater, E%pksr(1,t1), E%ser(1,1,t1), &
                          glstrr(1,t1), E%detfr(t1), matpar, matrix )
         end do

      end if

      if ( E%last==1 ) deallocate ( E%ugaussr, E%xgaussr )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelm250nl'

      end if  ! ( debug )
      call erclos ( 'sepelm250nl' )

      end subroutine sepelm250nl
