      subroutine sepelpfibjac
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 16-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Compute the Jacobian for the integration of the shell element
!     Both in the zeta direction as in the area direction an integration is
!     applied
!     Old subroutine el8127
! **********************************************************************
!
!                       KEYWORDS
!
!     elasticity_equation
!     shell
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer j, k, l, i
      double precision xjxi(3,3), som, hh
      double precision, pointer :: xhat(:,:), qfib(:,:,:), dpdxsi(:,:), &
                                   dpdeta(:,:), jac(:,:), h(:), x(:,:)

!     dpdeta         contains derivatives in reference element for gauss
!                    rule.
!                    dpdeta(i,k) = d phi_i/ d eta (x^k)
!     dpdxsi         contains derivatives in reference element for gauss
!                    rule.
!                    dpdxsi(i,k) = d phi_i/ d xi (x^k)
!     h              height of shell element
!     hh             help variable to store a constant
!     i              Counting variable
!     j              Counting variable
!     jac            Jacobian in the quadrature points
!     k              Counting variable
!     l              General loop variable
!     qfib           Contains the fiber coordinate system
!     som            Help parameter to compute a sum
!     x              co-ordinates of points
!     xhat           normal vector in the nodes
!     xjxi           ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRINRL1        Print 2d real vector
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelpfibjac' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelpfibjac'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      xhat => B%xhat
      dpdxsi => B%phiderc(:,:,1)
      dpdeta => B%phiderc(:,:,2)
      jac => B%fibjac
      h => E%coeffs(:,22)
      x => B%xyz

!     --- Loop over integration points

      do l = 1, 2
         do k = 1, m
            hh = 0.5d0*h(k)*B%zeta(l)
            do j = 1, 3
               som = 0d0
               do i = 1, n
                  som = som + dpdxsi(i,k)*(x(i,j)+hh*xhat(i,j))
               end do
                xjxi(j,1) = som
               som = 0d0
               do i = 1, n
                  som = som + dpdeta(i,k)*(x(i,j)+hh*xhat(i,j))
               end do
               xjxi(j,2) = som
               som = 0d0
               do i = 1, n
                  som = som + B%phi(i,k)*xhat(i,j)
               end do
               xjxi(j,3) = som * 0.5d0*h(k)
            end do
            jac(k,l) = 0.25d0*B%wzeta(l)* &
               (xjxi(1,1)*(xjxi(2,2)*xjxi(3,3)-xjxi(2,3)*xjxi(3,2))+ &
                xjxi(1,2)*(xjxi(2,3)*xjxi(3,1)-xjxi(2,1)*xjxi(3,3))+ &
                xjxi(1,3)*(xjxi(2,1)*xjxi(3,2)-xjxi(2,2)*xjxi(3,1)))
         end do
      end do

1000  if ( debug ) then

!     --- Debug information

         call prinrl1 ( jac, m, 2, 'jac' )
         write(irefwr,*) 'End sepelpfibjac'

      end if  ! ( debug )
      call erclos ( 'sepelpfibjac' )

      end subroutine sepelpfibjac
