      subroutine sepelvplate
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 11-08-2017
!
!   copyright (c) 2017-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill element vector for plate problems
!     Two-dimensional elements
!     Old subroutine elm255
! **********************************************************************
!
!                       KEYWORDS
!
!     element_vector
!     elasticity_equation
!     parabolic_equation
!     thick_plate
!     plate
!     bending
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecactl
      use sepmoduleelm
      use sepmodulebasefn
      use sepmodulecomio
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!

! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      double precision, pointer :: vec(:)

!     vec            work array to store a vector
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     SEPELLOAD1     Fill the element vector with a load
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'sepelvplate' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from sepelvplate'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( E%ind(23)/=0 ) then

!     --- f1 # 0  store contribution temporary in array work

         E%subvec => E%vec(1:3*n:3)
         call  sepelload1 ( E%coeffs(:,23), B%jdiag, B%phi )

      end if  ! ( E%ind(23)/=0 )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End sepelvplate'

      end if  ! ( debug )
      call erclos ( 'sepelvplate' )

      end subroutine sepelvplate
