      subroutine erpres1 ( isepi2, isepi3, isubr, iseqnr, mmiinput,
     +                     mmrinput, mmiinputinf, iinput, rinput,
     +                     iinputinf, numint, nreal, inpsep, rinsep,
     +                     inpsepinf, iprob )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 07-07-2011
!
!   copyright (c) 2011-2011  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store input arrays into arrays in ibuffr such that they can be
!     seen by the standard SEPRAN read subroutines
! **********************************************************************
!
!                       KEYWORDS
!
!     boundary_condition
!     essential
!     read
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cmcdpi'
      include 'SPcommon/cconst'
      include 'SPcommon/cinout'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer isepi2(4,*), isepi3(3,*), isubr, iseqnr, mmiinput,
     +        mmrinput, mmiinputinf, iinput(*), iinputinf(*),
     +        numint, nreal, inpsep(*), inpsepinf(*), iprob
      double precision rinput(*), rinsep(*)

!     iinput         o    integer input array created from standard input and
!                         inpsep
!     iinputinf      o    integer input array with information of rinsep
!                         created from iinput
!     inpsep         i    The integer input read is stored in array INPSEP
!     inpsepinf      o    integer input array with information of rinsep
!     iprob          i    Actual problem number
!     isepi2        i/o   Block 2 of array ISEPIN
!                         See subroutine ini009 for a description
!                         It is extended with the temporary input
!     isepi3        i/o   Block 3 of array ISEPIN
!                         See subroutine ini016 for a description
!                         It is extended with the temporary input
!     iseqnr         i    Sequence number with respect to the storage in array
!                         ISEPIN
!     isubr          i    Indicates the type of calling subroutine
!                         Possible values
!                         4:  CREATE
!                         5:  PRESBC
!     mmiinput       i    Memory management sequence number with respect to
!                         iinput
!     mmiinputinf    i    Memory management sequence number with respect to
!                         iinputinf
!     mmrinput       i    Memory management sequence number with respect to
!                         rinput
!     nreal          i    Number of reals stored
!     numint         i    Number of integers stored
!     rinput         o    Real input array
!     rinsep         i    Array containing the real input for this subroutine
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer maxsub, icall, ipos
      logical debug

!     debug          If true debug statements are carried out otherwise
!                    they are not
!     icall          Call sequence number for present subroutine
!     ipos           Position in array
!     maxsub         maximum number of inputs that may be stored for calling
!                    subroutine
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRCHR         Put character in error message
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    1556   Sequence number exceeds maximum
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'erpres1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from erpres1'
         write(irefwr,1) 'mmiinput, mmrinput, mmiinputinf',
     +                    mmiinput, mmrinput, mmiinputinf
         write(irefwr,1) 'iprob', iprob
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- First fill array isepin

      maxsub = isepi2(1,isubr)
      icall = isepi2(3,isubr)+1
      iseqnr = icall
      if ( icall>maxsub ) then

!     --- Sequence number exceeds maximum

         call errint ( icall, 1 )
         call errint ( maxsub, 2 )
         call errint ( icall, 3 )
         if ( isubr==5 ) then
            call errchr ( 'ESSENTIAL BOUNDARY CONDITIONS', 1 )
         else
            call errchr ( 'CREATE VECTOR', 1 )
         end if  ! ( isubr==5 )
         call errsub ( 1556, 3, 0, 1 )
         go to 1000

      end if  ! ( icall>maxsub )
      isepi2(2,isubr) = icall
      isepi2(3,isubr) = icall
      ipos = isepi2(4,isubr)-1+icall
      isepi3(1,ipos) = mmiinput
      isepi3(2,ipos) = mmrinput
      isepi3(3,ipos) = mmiinputinf

      if ( debug ) write(irefwr,1) 'ipos, icall', ipos, icall

!     --- Fill input array with first part and copy rest

      iinput(1) = -(isubr-3)  ! -1: create;  -2 prescribe_bc
      iinput(2) = 1           ! number of vectors
      iinput(3) = 1           ! solution vector
      iinput(4) = iprob       ! problem number
      iinput(5) = 1           ! ivec
      iinput(6) = 0           ! icompl
      iinput(7) = 1           ! number of cases

      iinput(8:7+numint) = inpsep(1:numint)

      if ( nreal>0 ) then

!     --- There is real input

         rinput(1:nreal) = rinsep(1:nreal)
         iinputinf(1:nreal) = inpsepinf(1:nreal)

      end if  ! ( nreal>0 )

1000  call erclos ( 'erpres1' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End erpres1'

      end if  ! ( debug )

      end
