      subroutine ini016 ( ibuffr, kprob, isepi2, isepi3, nsubr, maxi3,
     +                    nooutput )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.6  date 13-11-2010 New definition boundary conditions
!        version  3.5  date 23-03-2007 Extension of lin solver input
!        version  3.4  date 21-02-2007 Extension of lin solver input
!
!   copyright (c) 1993-2010  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy,
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!   Fill default structure in ISEPIN in case the structure of the program
!   has not explicitly been given by the user
!   At this moment the default is linear problem except when non_linear
!   equations has been read, in which case non-linear problem is the default
!   or time-dependent when time integration has been found
! **********************************************************************
!
!                       KEYWORDS
!
!     program_structure
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/cconst'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kprob(*), nsubr, maxi3, isepi2(4,nsubr), isepi3(3,maxi3),
     +        ibuffr(*)
      logical nooutput

!     ibuffr        i/o   Integer buffer array in which all large data is stored
!     isepi2         i    Block 2 of array ISEPIN
!                         Contents:
!                         Pos. (1,i)  MAXSUB  maximum number of input arrays
!                                         that may be stored for subroutine i
!                         Pos. (2,i)  ICALL   sequence number of next call of
!                                         subroutine i
!                         Pos. (3,i)  MAXCALL maximum number of input arrays
!                                         that have been read for subroutine i
!                         Pos. (4,i)  ISTART  Contains lowest number j of infor
!                                         entries for subroutine i
!                         The subroutines have a fixed position:
!                            1:  START  (not yet implemented)
!                            2:  PROBDF
!                            3:  COMMAT
!                            4:  CREATE
!                            5:  PRESDF
!                            6:  FILCOF
!                            7:  CHNCOF
!                            8:  SOLVE
!                            9:  OUTPUT
!                           10:  SEPCOMP
!                           11:  DERIVS
!                           12:  NLNPRB
!                           13:  INTEGR
!                           14:  INTBND
!                           15:  TIMPRB
!     isepi3         o    Block 3 of array ISEPIN
!                         Contents:
!                         Pos. (1,j)  IREFIN   INFOR entry with respect to the
!                                              j-th integer input array
!                         Pos. (2,j)  IREFRL   INFOR entry with respect to the
!                                              j-th real input array
!                         Pos. (3,j)  IREFRLIN INFOR entry with respect to the
!                                              j-th real input array
!                                              (variable part)
!                              The arrays are stored in the sequence:
!                              First all arrays for subroutine 1, then for
!                              subroutine 2 etc.
!                              Per subroutine the sequence of the calls
!                              (See ICALL) is used
!     kprob          i    Standard SEPRAN array, containing information of the
!                         problem
!     maxi3          i    Maximum length of first parameter in two-dimensional
!                         array isepi3
!     nooutput       i    If true no output is written to the file sepcomp.out
!                         by the time integration subroutine
!     nsubr          i    Number of different subroutines that may store their
!                         information in ISEPIN
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer ipinsp, niinp, nprob, ipos, irefin, ipinsl, i, ipinspsav,
     +        lenlinsol

!     i              Counting variable
!     ipinsl         Starting address of array IINSOL
!     ipinsp         Starting address of array iinspc
!     ipinspsav      Help parameter to save starting value of ipinsp
!     ipos           Position in array ISEPI3
!     irefin         Infor entry for integer input array
!     lenlinsol      Number of entries in array iinsol
!     niinp          Length of integer input array
!     nprob          Number of various problems
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer inigettext

!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     INI060TEXT     Reactivate existing array in IBUFFR
!     INI062         reserve space for temporary arrays
!     INI083         Create new array in IBUFFR in the case that already a
!                    temporary copy has been stored in IBUFFR
!     INICHKARLEN    Check length of array and give error message 550 of
!                    array is too small
!     INIGETTEXT     Compute pointer of array in IBUFFR
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
!   -
! **********************************************************************
!
!                       PSEUDO CODE
!
!   if ( isepi2(3,10) = 0 ) then
!      Create space for array iinspc
!      Fill array iinspc with default values depending on the input read
!      Fill arrays isepi2 and isepi3 for structure
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'ini016' )
      if ( isepi2(3,10)==0 ) then

!     --- Structure has not yet been filled
!         Create temporary space

         nprob = kprob(40)
         ipos = isepi2(4,10)
         niinp = 100
         call ini062 ( ibuffr, niinp, ipinsp )

!        --- Fill array iinspc with defaults
!            numvec

         ibuffr(ipinsp) = nprob
         ipinspsav = ipinsp

         if ( isepi2(3,15)/=0 ) then

!        --- create

            ibuffr(ipinsp+1) = 4

         else

!        --- presbc

            ibuffr(ipinsp+1) = 1

         end if

         ibuffr(ipinsp+2) = -1
         ibuffr(ipinsp+3) = -1
         ibuffr(ipinsp+4) = -1
         ipinsp = ipinsp + 4

!        --- linprb or nlnprb

         if ( isepi2(3,12)==0 .and. isepi2(3,15)==0 ) then

!        --- nlnprb nor timprb found, hence linear problem

            ibuffr(ipinsp+1) = 2
            ibuffr(ipinsp+2) = -1
            ibuffr(ipinsp+3) = -1
            ibuffr(ipinsp+4) = -1
            ibuffr(ipinsp+5) = -1
            lenlinsol = 14
            do i = 6, lenlinsol      ! This number must be enlarged in case
                                     ! the same happens in readsclinsol
               ibuffr(ipinsp+i) = 0

            end do  ! i = 6, lenlinsol
            if ( isepi2(3,8)/=0 ) then

!           --- input for the linear solver has been found
!               Check if the defect correction should be applied

               irefin = isepi3(1,isepi2(4,8))
               if ( irefin>0 ) then

!              --- irefin>0, hence input found

                  call ini060text ( ibuffr, -irefin, 'irefin' )
                  ipinsl = inigettext ( irefin, 'irefin' )
                  if ( ibuffr(ipinsl+19)/=0 ) ibuffr(ipinsp+6) = 1

               end if

            end if
            ipinsp = ipinsp + lenlinsol

         else if ( isepi2(3,15)==0 ) then

!        --- nlnprb found, hence non-linear problem

            ibuffr(ipinsp+1) = 3
            ibuffr(ipinsp+2) = -1
            ibuffr(ipinsp+3) = -1
            ibuffr(ipinsp+4) = -1
            ibuffr(ipinsp+5) = 0
            ipinsp = ipinsp + 5

         else

!        --- timprb found, hence time-dependent problem

            ibuffr(ipinsp+1) = 19
            ibuffr(ipinsp+2) = -1
            ibuffr(ipinsp+3) = -1
            ibuffr(ipinsp+4) = -1
            ipinsp = ipinsp + 4

         end if

         if ( isepi2(3,15)==0 .or. nooutput ) then

!        --- outsol

            ibuffr(ipinsp+1) = 5
            ibuffr(ipinsp+2) = 1
            ibuffr(ipinsp+3) = 1
            ibuffr(ipinsp+4) = 1
            ipinsp = ipinsp + 4

         end if

!        --- end

         ibuffr(ipinsp+1) = -1

!        --- Fill arrays isepi2 and isepi3

         isepi2(3,10) = 1

!        --- Check length of array inputstru

         niinp = ipinsp+2-ipinspsav
         call inichkarlen ( niinp, 100, 'inputstru' )
         if ( ierror/=0 ) go to 1000

!        --- Make array permanent

         call ini083 ( ibuffr, isepi3(1,ipos), niinp, ipinspsav,
     +                 'inputstru' )

      end if

1000  call erclos ( 'ini016' )

      end
