      program hertz2

!      --- Main program for the Hertz-problem:
!          An infinitely long, elastic, half cylinder is pressed
!          on a flat surface.
!          A slice of this cylinder is analyzed.
!          This main program is necessary because of the variable boundary
!          conditions

      implicit none

      integer, allocatable, dimension (:) :: ibuffr
      integer (kind=8) pbuffr
      integer error
      parameter ( pbuffr=25000000)
      allocate(ibuffr(pbuffr), stat = error)
      if (error /= 0) then
         ! space for these arrays could not be allocated
         print *, "error: (hertz2) could not allocate space."
         stop
      end if  ! (error /= 0)

      call freebsub ( ibuffr, ibuffr, pbuffr )
      end

!     --- Function subroutine for the boundary conditions

      function funcbc( ichoice, x, y, z)
      implicit none

      integer ichoice
      double precision funcbc, x, y, z

      if (ichoice==1) then

!     --- ichoice = 1, boundary condition for the contact points
!         The z-displacement is made equal to -z
!         In this way points are moved back to z=0

        funcbc = -z

      end if

      end

!     --- Subroutine funcvect defines the contact distance and the
!         contact force

      subroutine funcvect ( ichoice, ndim, coor, numnodes,
     +                      uold, nuold, result, nphys )
      implicit none

      integer ichoice, ndim, numnodes, nuold, nphys
      double precision coor(ndim, numnodes),
     +                 uold( numnodes, nphys, nuold),
     +                 result( numnodes, *)

      integer k

      if ( ichoice==3 ) then

!     --- ichoice = 3, contact distance = u_z + z

         do k = 1, numnodes
            result(k,1) = coor(3,k) + uold(k,3,1)
         end do

      else if ( ichoice==4 ) then

!     --- ichoice = 4, contact force is third component of reaction force

         do k = 1, numnodes
            result(k,1) = uold(k,3,1)
         end do

      end if

      end

!     --- Function user bool is used to set the boolean
!         In this case the boolean is true if the contact region has been
!         changed

      function userbool( ichoice )
      implicit none
      logical userbool
      integer ichoice

      include 'SPcommon/ccontact'

      if ( ichoice==1 ) then

!     --- ichoice = 1, the only possible value in this program
!         set userbool equal to the value of contact_changed(1)
!         This indicates if the contact region corresponding to the first
!         (and in this case only) contact problem has been changed

         userbool = contact_changed(1)

      end if

      end
