      program roll

!      --- Main program for the Roll-problem:
!          An infinitely long, elastic, cylinder is pressed
!          between two flat surfaces.
!          A slice of this cylinder is analyzed.
!          This main program is necessary because of the variable boundary
!          conditions

      implicit none

      integer, allocatable, dimension (:) :: ibuffr
      integer (kind=8) pbuffr
      integer error
      parameter ( pbuffr=25000000)
      allocate(ibuffr(pbuffr), stat = error)
      if (error /= 0) then
         ! space for these arrays could not be allocated
         print *, "error: (roll) could not allocate space."
         stop
      end if  ! (error /= 0)

      call freebsub ( ibuffr, ibuffr, pbuffr )
      end

!     --- Function subroutine for the boundary conditions


      function funcbc( ichoice, x, y, z)
      implicit none

      integer ichoice
      double precision funcbc, x, y, z
      integer ifirst
      double precision Ru, dH
      double precision getconst
      save Ru, dH
      data ifirst /0/

      if ( ifirst==0 ) then

!     --- ifirst = 0, first call of funcbc
!         Get the values of some constants

         ifirst = 1
         Ru = getconst ( 'Ru' )
         dH = getconst ( 'dH' )

      end if

      if (ichoice==11) then

!     --- ichoice = 11, bottom contact surface
!         In order to restrict the solution to z = -Ru it is necessary
!         to set the displacement equal to -Ru - z

        funcbc = -Ru - z

      else if (ichoice==12) then

!     --- ichoice = 12, top contact surface
!         In order to restrict the solution to z = Ru-dH it is necessary
!         to set the displacement equal to Ru-dH - z

        funcbc =  Ru-dH - z

      end if

      end

!     --- Subroutine funcvect defines the contact distance and the
!         contact force

      subroutine funcvect ( ichoice, ndim, coor, numnodes,
     +                      uold, nuold, result, nphys )
      implicit none

      integer ichoice, ndim, numnodes, nuold, nphys
      double precision coor(ndim, numnodes),
     +                 uold( numnodes, nphys, nuold),
     +                 result( numnodes, *)

      integer k, ifirst
      double precision Ru, dH
      save Ru, dH
      double precision getconst
      data ifirst /0/

      if ( ifirst==0 ) then

!     --- ifirst = 0, first call of funcbc
!         Get the values of some constants

         ifirst = 1
         Ru = getconst ( 'Ru' )
         dH = getconst ( 'dH' )

      end if

      if (ichoice==31) then

!     --- ichoice = 31, contact distance = u_z + z + Ru

        do k = 1, numnodes
           result(k,1) = (coor(3,k) + uold(k,3,1)) + Ru
        end do

      else if (ichoice==32) then

!     --- ichoice = 32, contact distance =  + Ru-dH- (u_z + z)

        do k = 1, numnodes
           result(k,1) = Ru-dH - (coor(3,k) + uold(k,3,1))
        end do

      else if (ichoice==41) then

!     --- ichoice = 41, contact force is third component of reaction force

        do k = 1, numnodes
           result(k,1) = uold(k,3,1)
        end do

      else if (ichoice==42) then

!     --- ichoice = 42, contact force is minus third component of reaction force

        do k = 1, numnodes
           result(k,1) = -uold(k,3,1)
        end do

      end if

      end
