      program leafspring
      call startsepcomp
      end

      subroutine fnmateri ( ichoice, s, se, eps, detf, matpar, makese )
! ======================================================================
!
!
!
!                       DESCRIPTION
!
!     MATERI :  routine for ELM250. Material behaviour in nonlinear case
!
!     Compute 2nd-Piola-Kirchoff stress from given Green-Lagrange
!     strains. The determinant of the deformation gradient, can be
!     used in this relationship
!
!       EXAMPLE MATERIAL
!       linear elastic material model (isotropic hooke)
!       the large displacement/rotation are taken into account
!       in the Green-Lagrange strain, which is small.
!
! **********************************************************************
!
!                       KEYWORDS
!
!       elasticity
!       nonlinear
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      logical          makese
      double precision s(6), eps(6), detf, matpar(10),
     +                 se(6,6)
      integer ichoice

!     detf  i    determinant of deformation gradient
!     eps   i    Green-Lagrange strains
!                   eps(i)  : i: components (symmetric!);
!     ichoice i   material model number ( icoef 4 )
!                   0 : hookean
!                        coef6 = E
!                        coef7 = nu
!     makese i   se must be computed (true) or not (false)
!     matpar i   material parameters (User defines!) for every
!                integration point
!     nip   i    number of data points
!     s     o    2nd-PK-stresses: You only have to store symmetric
!                components!
!                   s(j) :    j: component (1=11, 2=22, 3=33,
!                                           4=12, 5=23, 6=31)
!     se    o    tangential matrix.
!                   se(i,j) : i,j components (symmetric!)
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS

      integer          i, j
      double precision a0, a1, a2, a3, E, nu

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
!     --- material parameters

      E  = matpar(1)
      nu = matpar(2)
      a0 = E / ( 1d0 + nu )
      a1 = a0 * ( 1d0 - nu ) /( 1d0 - 2d0*nu)
      a2 = a0 * nu /( 1d0 - 2d0*nu)
      a3 = a0 / 2d0

!     --- clear se and s

      do i = 1, 6
         s(i) = 0d0
         do j = 1, 6
            se(i,j) = 0d0
         end do
      end do

      do i=1,3
         se(i,i) = a1
         se(i+3,i+3) = a3
      end do

      se(1,2) = a2
      se(2,1) = a2
      se(2,3) = a2
      se(3,2) = a2
      se(1,3) = a2
      se(3,1) = a2

      do i=1,3
         do j=1,3
            s(i) = s(i) + se(i,j)*eps(j)
         end do
         s(i+3) = se(i+3,i+3)*eps(i+3)
      end do

      do i = 1, 3
         se(i,i) = a1
         se(i+3,i+3) = a3*0.5d0
      end do

      end

      subroutine fnlocdir ( pos, dir, ielgrp )

!     --- example for usage of fnlocdir

      implicit none
      double precision pos(3), dir(3,3)
      integer ielgrp

      double precision rct, x, y, z, len

      rct = 0.8d0
      x = pos(1)
      y = pos(2)
      z = pos(3) + rct
      len = sqrt( z*z + x*x )

!     --- local x-vector

      dir(1,1) = z/len
      dir(2,1) = 0d0
      dir(3,1) = -x/len

!     --- local y-vector (unchanged)

      dir(1,2) = 0d0
      dir(2,2) = 1d0
      dir(3,2) = 0d0

!     --- local z-vector

      dir(1,3) = -dir(3,1)
      dir(2,3) = 0d0
      dir(3,3) = dir(1,1)

      end
