      subroutine meshus ( isurf, new, coor, kmeshc, inpelm, nbndpt,
     .                    kbndpt, bcord, npoint, nelem )
      implicit none

!    *  Input / ouput parameters (See Porgrammer's Guide: 3.4.6)

      integer isurf, inpelm, kmeshc(inpelm,*), nbndpt, kbndpt(*),
     .        npoint, nelem
      logical new
      double precision coor(2,*), bcord(2,*)

!    *  Simple example of a subroutine meshus
!       This example defines a square mesh defined on the region (0,1) x (0,1)
!       The number of elements in all directions is 5
!       The mesh is defined equidistantly:
!
!
!       31         32         33         34         35          36
!        --------------------------------------------------------
!        |          |          |         |           |          |
!        |          |          |         |           |          |
!        |    21    |   22     |         |           |    25    |
!        |          |          |         |           |          |
!    25  -------------------------------------------------------- 30
!        |          |          |         |           |          |
!        |          |          |         |           |          |
!        |   16     |          |         |           |          |
!        |          |          |         |           |          |
!        |          |          |         |           |          |
!    19  -------------------------------------------------------- 24
!        |          |          |         |           |          |
!        |          |          |         |           |          |
!        |    11    |          |         |           |    15    |
!        |          |          |         |           |          |
!        |          |          |         |           |          |
!    13  -------------------------------------------------------- 18
!        |          |          |         |           |          |
!        |          |          |         |           |          |
!        |     6    |          |         |           |    10    |
!        |          |          |         |           |          |
!        |          |          |         |           |          |
!     7  -------------------------------------------------------- 12
!        |          |          |         |           |          |
!        |          |          |         |           |          |
!        |    1     |    2     |    3    |      4    |    5     |
!        |          |          |         |           |          |
!        |          |          |         |           |          |
!        --------------------------------------------------------
!        1          2          3         4           5          6
!
!
!     *  Local parameters

      integer i, j
      double precision dx, dy

!
!     *  Fill co-ordinates
!
      dx = 0.2d0
      dy = 0.2d0
      do i = 1, 6
         do j = 1, 6
            coor(1,(j-1)*6+i) = (i-1)*dx
            coor(2,(j-1)*6+i) = (j-1)*dy
         end do
      end do

!    *  Fill elements

      do j = 1, 5
         do i = 1, 5
            kmeshc(1,(j-1)*5+i) = i + (j-1)*6
            kmeshc(2,(j-1)*5+i) = i + (j-1)*6 + 1
            kmeshc(3,(j-1)*5+i) = i + (j-1)*6 + 7
            kmeshc(4,(j-1)*5+i) = i + (j-1)*6 + 6
         end do
      end do

!    *  Fill array kbndpt:

      do i = 1, 6
         kbndpt(i) = i
      end do

      do i = 2, 5
         kbndpt(i+5) = i*6
      end do

      do i = 1, 6
         kbndpt(i+10) = 37-i
      end do

      do i = 2, 5
         kbndpt(i+15) = 37-6*i
      end do

!    *  Fill nelem and npoint

      nelem = 25
      npoint = 36

      end
