      program examtimdep
! *******************************************************************
! *
! *   example for time dependent integration  (non-diagonal mass matrix)
! *   programmer Niek Praagman
! *   date 06-05-89
! *
! *******************************************************************
!
      use sepmoduleoldrouts
      use sepmoduletime
      integer kmesh, kprob, intmat, isol, iuser(100),
     +        iu1(10), ichois, ichvc, icount, itytim, iu2,
     +        ivec
      double precision user(100), u1(10), epsusr, u2

!    *  Standard SEPRAN initialisations

      kmesh = 0
      kprob = 0
      intmat = 0
      isol = 0
      user = 0
      iuser = 0
      user(1)  = 100
      iuser(1) = 100

!    *  Start SEPRAN  (read mesh etc.)

      call sepstr( kmesh, kprob, intmat )
!
! *******************************************************************
!
! *   fill initial conditions in isol :
!
! *******************************************************************
!
      t      = 0d0
      ichois = 0
      ichvc  = 1
      iu1(1) = 1
!
      call creavc ( ichois, ichvc, ivec, isol, kmesh, kprob,
     +              iu1, u1, iu2, u2 )

!    *  Write initial condition to file sepcomp.out

      call outtim ( t, kmesh, kprob, isol )
!
! *******************************************************************
!
! *   set parameters for subroutine soltim:
!
! *******************************************************************
!
      itytim = 1
      ichtim = 1
      epsusr = 1d-3
      oldmethod = .true.
!
! *******************************************************************
!
! *   compute solution for t = 0.1, 0.2, . . . , 1.0
!
! *******************************************************************
!
!
      tend = 1d0
      do 200 icount = 1, 10
         tout = icount*0.1d0
         call soltim ( itytim, ichtim, kmesh, kprob, intmat, isol,
     +                 epsusr, user, iuser)

!     *    write solution to file sepcomp.out

         call outtim ( t, kmesh, kprob, isol )

200   continue

!    *  Close SEPRAN

      call finish(0)
      end
! *******************************************************************
! *
! *   function func for the initial condition :
! *
! *******************************************************************
      function func ( ichois, x, y, z )
      double precision func, x, y, z
      integer ichois
!
      func = 16d0*y*(1d0-x)*(1d0-y)*x
!
      end
! *******************************************************************
! *
! *   function for essential boundary conditions :
! *
! *******************************************************************
      function funcbc ( ichois, x, y, z )
      use sepmoduletime
      double precision funcbc, x, y, z
      integer ichois
!
!
      if ( ichois.eq.1 ) then
         funcbc = (1d0-x*x)*(1d0-exp(-10d0*t))
      else if ( ichois.eq.2 ) then
         funcbc = (y*y)*(1d0-exp(-10d0*t))
      else if ( ichois.eq.3 ) then
         funcbc = (x*x)*(1d0-exp(-10d0*t))
      else if ( ichois.eq.4 ) then
         funcbc = (1d0-y*y)*(1d0-exp(-10d0*t))
      end if
!
      end
! **********************************************************************
! *
! *   subroutine filtim for the control of the time-dependent process
! *
! **********************************************************************
      subroutine filtim ( kmesh, kprob, intmat, isol, matrs, matrm,
     +                    irhsd, islold, user, iuser )
      use sepmoduleoldrouts
      use sepmoduletime
      double precision user(*)
      integer kmesh, kprob, intmat, isol, islold,
     +        matrs, matrm, irhsd, iuser(*)
      integer ictime, ichois, ielhlp, iprob
      save ictime
      data ictime /0/
!
! *******************************************************************
! *
! *   User written subroutine for the incorporation of boundary
! *   conditions and the determination of the matrices M and S
! *   Example : Heat equation on the unit square .
! *
! *******************************************************************
!
! *   essential boundary conditions :
!

      call prestm ( ictime, kmesh, kprob, isol )
!
! *******************************************************************
! *
! *   Fill matrices M and S and right-hand side F ( = 0)
! *   Since the matrices are time-independent this is only
! *   necessary at t = 0
! *
! *******************************************************************

      if ( ictime.eq.0 ) then

!      *  Information is only read the first step
!         The parameter ictime is used to distinguish between first and
!         next step

!      *  First fill the stiffness matrix and right-hand side
!         Standard Laplacian equations, so use itype = 700

         iprob = 1
         call filcof ( iuser, user, kprob, kmesh, iprob )
         ichois = 1
         call systm1 ( ichois, 1, matrs, intmat, kmesh, kprob,
     +                 irhsd, matrm, isol, iuser, user, islold, ielhlp)
         ictime = 1

      end if
      end
