 /*	
  *  Copyright (c) 1991  Philips Export B.V.
  * 
  *  Philips Research Laboratories
  *  Eindhoven, The Netherlands
  *
  *  Author    : G.F.M. Haenen
  *              F.W.G.M. Wouters
  *
  *  Date      : July, 1991
  *
  *  Function  : Translates plotting primitives to PostScript.
  *
  *  Comment   : 
  *
  *  Modified  : Kees Lemmens 
  *            - It now produces a real EPS file without header
  *              and with the correct boundingbox if the HEADER
  *              macro is set to 0.
  *
  * - Sander Gielen, 29 Dec 1997.
  *            - Changed layout
  *            - ps_text writes plain PostScript text, not lines
  */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/file.h>
#include <time.h>
#include <unistd.h>
#include "plotfun.h"

# define XPICSIZE (72.0 * 6.0)
# define YPICSIZE (72.0 * 6.0)
#define HEADER   0

#if HEADER == 1
# define XOFFSET  100.0
# define YOFFSET  100.0
# define X(x) ((x - usr->xmin) * (XPICSIZE / scale))
# define Y(y) ((y - usr->ymin) * (YPICSIZE / scale))
#else
# define XOFFSET  (15.0)
# define YOFFSET  (15.0)
# define X(x) ((x - usr->xmin) * scale)
# define Y(y) ((y - usr->ymin) * scale)
#endif

static FILE *fp;
static double scale;
static window_t *usr;

/* ps_header */

static void ps_header(char *psfile, window_t *dim)
{
   
#define SPACE 20.0
#define YUPPER 650.0 

  time_t timeloc;

  (void) time(&timeloc);

#if HEADER == 1
   fprintf(fp, "%%!PS-Adobe-2.0\n");
#else
   fprintf(fp, "%%!PS-Adobe-2.0 EPSF\n");
#endif
   fprintf(fp, "%%%%Title: Sepran viewer\n");
   fprintf(fp, "%%%%Creator: Sepview\n");
   fprintf(fp, "%%%%CreationDate: %.24s\n", ctime(&timeloc));
   fprintf(fp, "%%%%BoundingBox: %f %f %f %f\n", 
	   X(dim->xmin), Y(dim->ymin), 
	   X(dim->xmax)+2*XOFFSET, Y(dim->ymax)+2*YOFFSET);
   fprintf(fp, "%%%%EndComments\n");
   fprintf(fp, "/bdef {bind def} bind def\n");
   fprintf(fp, "1 setlinejoin 1 setlinecap 1 setlinewidth\n");
   fprintf(fp, "%3f %.3f translate\n", XOFFSET, YOFFSET);
   fprintf(fp, "/l {moveto lineto stroke} bdef\n");
   fprintf(fp, "/header\n{\n");
   fprintf(fp, "/Helvetica-Bold findfont 20 scalefont setfont\n");
   fprintf(fp, "%f %f moveto \n",0.0, YUPPER);
   fprintf(fp, "(SEPVIEW) show \n");
   fprintf(fp, "/Helvetica findfont 12 scalefont setfont\n");
   fprintf(fp, "%f %f moveto\n",0.0, YUPPER-2*SPACE);
   fprintf(fp, "(Userid) show\n");
   fprintf(fp, "%f %f moveto\n",0.0+70, YUPPER-2*SPACE);
   fprintf(fp, "(: %s) show\n",getenv("USER"));
   fprintf(fp, "%f %f moveto \n",0.0, YUPPER-3*SPACE);
   fprintf(fp, "(Filename) show\n");
   fprintf(fp, "%f %f  moveto\n",0.0+70,YUPPER-3*SPACE);
   fprintf(fp, "(: %s) show\n",psfile);
   fprintf(fp, "%f %f moveto \n",0.0, YUPPER-4*SPACE);
   fprintf(fp, "(Date/Time)show\n");
   fprintf(fp, "%f %f moveto\n",0.0+70,YUPPER-4*SPACE);
   fprintf(fp, "(: %.24s) show \n", ctime(&timeloc));
   fprintf(fp, "%f %f moveto\n",0.0, YUPPER-5*SPACE);
   fprintf(fp, "%f %f lineto stroke\n",XPICSIZE, YUPPER-5*SPACE);
   fprintf(fp, "} def\n");
   
   /* setup definition for textfont */
   fprintf(fp, "/textfont {/Helvetica findfont exch scalefont setfont} bdef\n");
#if HEADER == 1
   fprintf(fp, "%% to include this picture in TeX erase the next line\n");
   fprintf(fp, "header\n");
#endif
}

/* ps_trailer */

static void ps_trailer()
{
   fprintf(fp, "%%%%Trailer\n");
   fprintf(fp, "%% to include this picture in TeX erase the next line\n");
   fprintf(fp, "showpage\n");
   fprintf(fp, "%%%%EOF\n");
   fclose(fp);
   infoMessage("");
}

/* ps_finish */

void ps_finish()
{
  ps_trailer();
}

/* ps_init */

int  ps_init(window_t *dim,filnam_t *fileinfo)
{
   int  number = 0;
   char string[80], psfile[80];
   
   if(strlen(fileinfo->outname) == 0)
   {
      do
	sprintf(psfile, "%s%03d_%02d.eps", fileinfo->name, 
		fileinfo->version, ++number);
      while (access(psfile, R_OK) == F_OK && number < 100);
   }
   else
     strncpy(psfile,fileinfo->outname,79);
   
  if ((fp = fopen(psfile, "w")) == NULL)
    {
       sprintf(string, "Unable to open postscript file '%s'", psfile);
       errorMessage(string);
       return(FALSE);
    }
   sprintf(string, "Creating postscript file: %s...", psfile);
   infoMessage(string);
   
   scale = dim->height > dim->width ? dim->height : dim->width;

   usr = dim; /* remember it for clipping ! */

   ps_header(psfile,dim);
   
   return(TRUE);
}

/* ps_line */

void ps_line(double x1, double  y1, double x2, double y2)
{
   if (clip_line(&x1, &y1, &x2, &y2, usr))
     fprintf(fp, "%.2f %.2f %.2f %.2f l\n",  X(x1), Y(y1), X(x2), Y(y2));
}

/* ps_color */

void ps_color(int iclr)
{
   int R, G, B;
   
   getcolor(iclr, &R, &G, &B);
   fprintf(fp, "%.4f %.4f %.4f setrgbcolor\n", R/65535.0, G/65535.0, B/65535.0);
}

/* ps_polyline */
        
void ps_polyline(double x[],double y[],int n)
{
   int    i;
   
   if (clip_polygon(x, y, &n, usr))
   {
      fprintf(fp, "%.2f %.2f moveto\n",  X(x[0]), Y(y[0]));
      for (i = 1; i < n; i++)
	fprintf(fp, "%.2f %.2f lineto\n", X(x[i]), Y(y[i]));
      fprintf(fp, "stroke\n");
   }
}

/* ps_polyfill */
 
void ps_polyfill(double x[],double y[],int n)
{
   int    i;
   
   if (clip_polygon(x, y, &n, usr))
   {
      fprintf(fp, "%.2f %.2f moveto\n",  X(x[0]), Y(y[0]));
      for (i = 1; i < n; i++)
	fprintf(fp, "%.2f %.2f lineto\n", X(x[i]), Y(y[i]));
      fprintf(fp, "closepath fill\n");
   }
}

/* ps_text adapted to write PostScript text instead of line symbols */
 
void ps_text(double x,double y,double height,double angle,char * string)
{
   /* set textheight, factor 30 looks ok */
   fprintf(fp, "%.1f textfont\n", height*30 );
   /* move to location x,y */
   fprintf(fp, "%.2f %.2f moveto\n", X(x), Y(y));
   /* if angle!=0 then store graphics state put in rotate mode */
   if ( angle!=0 ) { 
      fprintf(fp, "gsave %.1f rotate\n", angle);
   }
   /* put the text */
   fprintf(fp, "(%s) show\n",string);
   /* angle? then restore the graphics state */
   if ( angle!=0 ) { fprintf(fp, "grestore\n"); }
}

/* ps_symbol */

void ps_symbol(double x,double y,int num)
{
  line_symbol(x, y, num);
}

