/*
 *  Author    : Sander Gielen
 *
 *  Date      : March, 1998
 *
 *  Modified  : Kees Lemmens, June 9, 1999
 *            - fixed compiler warnings and some small bugs
 *
 *  Function  : Translates plotting primitives to fig.
 *
 *  Comment   : derived from ps_plotfun.c
 *
 *  Modified  : 
*/

#include <stdio.h>
#include <string.h>
#include <sys/file.h>
#include <unistd.h>
#include "plotfun.h"

#define XPICSIZE (1200.0 * 6.0)
#define YPICSIZE (1200.0 * 6.0)
#define XOFFSET  120.0
#define YOFFSET  120.0
#define HEADER   1
#define PI       3.1415927

#define X(x) (XOFFSET  +(x - usr->xmin) * (XPICSIZE / scale))
#define Y(y) (YOFFSET  -(y - usr->ymax) * (YPICSIZE / scale))

#define SPACE 300.0
#define YUPPER 650.0 

static FILE *fp;
static double scale;
static int curcolor;
static window_t *usr;

/* fig_header */

static void fig_header(char *figfile,window_t *dim)
{     
  int R, G, B;
  int thecolor;
  

  fprintf(fp, "#FIG 3.1\n");
  fprintf(fp, "Portrait\nCenter\nMetric\n1200 2\n");

  /* colormap must be included into the header 
     before any other command */
  
  for( thecolor = 1; thecolor<=20; thecolor++ ) 
    {
      getcolor(thecolor, &R, &G, &B);
      fprintf(fp, "0 %d #%02x%02x%02x \n", thecolor+31, R/256, G/256, B/256 );
    }
}

/* fig_trailer */

static void fig_trailer()
{
  fprintf(fp, "#Trailer\n");
  fprintf(fp, "#EOF\n");
  fclose(fp);
  infoMessage("");
}

/* fig_finish */

void fig_finish()
{
  fig_trailer();
}

/* fig_init */

int fig_init(window_t *dim,filnam_t *fileinfo)
{
   int  number = 0;
   char string[80], figfile[80];
   
   if(strlen(fileinfo->outname) == 0)
   {
      do
	sprintf(figfile, "%s%03d_%02d.fig", fileinfo->name, 
		fileinfo->version, ++number);
      while (access(figfile, R_OK) == F_OK && number < 100);
   }
   else
     strncpy(figfile,fileinfo->outname,79);

   if ((fp = fopen(figfile, "w")) == NULL)
   {
      sprintf(string, "Unable to open fig file '%s'", figfile);
      errorMessage(string);
      return(FALSE);
   }
   sprintf(string, "Creating fig file: %s...", figfile);
   infoMessage(string);
   
   scale = dim->height > dim->width ? dim->height : dim->width;
   
   fig_header(figfile,dim);
   
   usr = dim; /* remember it for clipping ! */
   return(TRUE);
}

/* fig_line */

void fig_line(x1, y1, x2, y2)
     double x1, y1, x2, y2;
{
  if (clip_line(&x1, &y1, &x2, &y2, usr))
    {
      fprintf(fp, "2 1 0 1 -1 -1 0 0 20 0.000 0 0 -1 0 0 2\n");
      fprintf(fp, "%.0f %.0f %.0f %.0f \n",  X(x1), Y(y1), X(x2), Y(y2));
    }
}

/* fig_color */

void fig_color(iclr)
     int iclr;
{
  curcolor = iclr; /* move the color setting global */

/* for now keep old ps-definitions, may-be useful for future
  int R, G, B;
  
  getcolor(iclr, &R, &G, &B);
  fprintf(fp, "%.4f %.4f %.4f setrgbcolor\n", R/65535.0, G/65535.0, B/65535.0);

*/
}

/* fig_polyline */
        
void fig_polyline(x, y, n)
     double x[], y[];
     int    n;
{
  int    i;
  
  if (clip_polygon(x, y, &n, usr))
    {
      fprintf(fp, "2 1 0 1 %d %d 0 0 20 0.000 0 0 -1 0 0 %d\n", 
	      curcolor+31, curcolor+31, n);
      for (i = 0; i < n; i++)
	fprintf(fp, "%.0f %.0f \n", X(x[i]), Y(y[i]));
    }
}

/* fig_polyfill */
 
void fig_polyfill(x, y, n)
     double x[], y[];
     int    n;
{
  int    i;
  
  if (clip_polygon(x, y, &n, usr))
    {
      fprintf(fp, "2 3 0 1 %d %d 0 0 20 0.000 0 0 -1 0 0 %d\n", 
	      curcolor+31, curcolor+31, n);
      for (i = 0; i < n; i++)
	fprintf(fp, "%.0f %.0f \n", X(x[i]), Y(y[i]));
    }
}

/* fig_text adapted to write PostScript text instead of line symbols */
 
void fig_text(x, y, height, angle, string)
     double x, y, height, angle;
     char   *string;
{
  fprintf(fp, "4 0 -1 0 0 0 10.0 %.1f 2 %.0f %.0f ", 
	  2*PI*angle/180, 195.0, sizeof(string)*10.0 );
  fprintf(fp, "%.0f %.0f ", X(x), Y(y));
  fprintf(fp, string );
  fprintf(fp, "\\001\n");
}

/* fig_symbol */

void fig_symbol(x, y, num)
     double x, y;
     int    num;
{
  line_symbol(x, y, num);
}

