      subroutine fil102 ( iref, ielgrp, iuser, user, kproblc, nparm, &
                          iwork, work, kmeshlc )
! ======================================================================
!
!        programmer    Guus Segal
!        version  3.4  date 05-12-2012 Long integers
!        version  3.3  date 12-07-1998 Include commons
!        version  3.2  date 11-05-1991 New call of elf000
!
!   copyright (c) 1987-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     fill arrays iuser and user from arrays iwork and work
!     internal elements
!     In this case both iwork(k)=2001 and iwork(k)=2002 are permitted
!     They require the input to be read from file with reference number iref
!     This input is stored in the user array user
!     When iref<0 then the input is not read from the input file but
!     stored in array user by using subroutine funcfl
! **********************************************************************
!
!                       KEYWORDS
!
!     coefficient
!     fill
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemain
      use sepmodulekmesh
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer iuser(*), kproblc(*), iwork(*), kmeshlc, iref, ielgrp, &
              nparm
      double precision user(*), work(*)

!      ielgrp  i   standard element number
!      iref    i   When>0 it indicates the input reference number of the file
!                  when<0, it indicates that the coefficients to be stored in
!                  array user are created by the user subroutine funcfl
!                  iref<0 should only be used in combination with
!                  iwork(k) = 2001
!     iuser  i/o   Integer input array for subroutine SYSTEM or DERIVA,
!                  which contains information of the coefficients
!     iwork   i    User array of length NPARM to store information of the
!                  coefficients, together with array work
!                  iwork(i) contains the parameter ifunc
!                  with
!                  ifunc  Indication of the type of coefficients.
!                     Possible values:
!                     0:           coef i is a constant, stored in work(i)
!                     >0<1000:   coef i is given by the subroutine FUNCCF
!                     >1000<2000:   coef i is given by the subroutine FUNCC1
!                     2001:        coef i is given by the user in each point
!                                  The values are read from file iref
!                     2002:        coef i is given by the user in each element
!                                  The values are read from file iref
!                    <0:         Coefficient i is equal to coefficient j, &
!                                  where j = -iwork(1,i) and j<i
!     kmeshlc i    Standard SEPRAN array, containing information of the mesh
!     kproblc   i    Standard SEPRAN array, containing information of the problem
!     nparm   i    Number of parameters for a specific element group
!     user   i/o   Real input array for subroutine SYSTEM or DERIVA, &
!                  which contains information of the coefficients
!     work    i    Real work array of length NPARM in which values of
!                  coefficients are stored. See iwork
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer icount, jcount, mcount, &
              i, ifunc, k, kcount, ih
      double precision func

!     func       value of the coefficient when it is a constant
!     i          Counting variable
!     icount     number of unknowns per element
!     ifunc      Choice parameter with respect to function calls
!     ih         Help variable to store a constant temporarily
!     jcount     Counter
!     k          Counting variable
!     kcount     Counter
!     mcount     ?
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ELF000         Actual filling of IUSER and USER for the simple cases
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     FUNCFL         User written subroutine to fill coefficients
!     PRINLG         Print and checks length of user arrays
!     SEPFILLKMESH   Fill parameters in module sepmodulekmesh
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!      26   Array kprob has not been filled
!     415   element group number read larger than NELGRP
!     536   value of iwork is not permitted in subroutine
!     571   iref<0 is not allowed in combination with iwork(k) = 2002
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'fil102' )
      if ( kproblc(2)/=101 ) then
         call errint ( kproblc(1), 1 )
         call errsub ( 26, 1, 0, 0 )
      end if
      if ( ielgrp<=0 .or. ielgrp>kproblc(8) ) then
         call errint ( ielgrp, 1 )
         call errint ( kproblc(8), 2 )
         call errsub ( 415, 2, 0, 0 )
      end if
      if ( ierror/=0 ) go to 1000

      kmesh2d(1) = kmeshlc
      kmesh => kmesh2d(1)
      call sepfillkmesh(kmesh)

!     --- Set some constants

      if ( nelgrp>1 ) inelem = kmeshg(ielgrp)
      if ( ielgrp==1 .or. iuser(2)<=0 ) then

!     --- ielgrp  =  1

        icount = 5+kproblc(8)+kproblc(12)
        jcount = 5

      else

!     --- ielgrp>1

        icount = iuser(4)
        jcount = nint(user(4))

      end if

!     --- Store value of icount into mcount

      mcount = icount

!     --- fill arrays iuser and user

      iuser(ielgrp+5) = icount+1

      do i = 1, nparm
         ifunc = iwork(i)
         if ( ifunc>=0 .and. ifunc<=2000 ) then
            func = work(i)
            call elf000 ( icount, jcount, ifunc, func, i, iuser, user )
            if ( jcount==2001 .or. jcount==2002 ) jcount = 2003

         else if ( ifunc==2001 ) then

!        --- ifunc  =  2001   read npoint function values from file iref

            icount = icount+1
            iuser(icount) = 2001
            icount = icount+1
            iuser(icount) = jcount+1
            call prinlg( 'user', -jcount-npoint, user, iuser )
            if ( iref>0  ) then

!           --- iref>0  Read from file iref

               read(iref, *) (user(jcount+k), k = 1, npoint)

            else if ( iref<0 ) then

!           --- iref<0  Call subroutine funcfl

               call funcfl ( coor, npoint, -iref, &
                             user(jcount+1) )
            end if
            jcount = jcount+npoint
            if ( jcount==2001 .or. jcount==2002 ) jcount = 2003

         else if ( ifunc==2002 ) then

!        --- ifunc  =  2002   read nelem function values from file iref

            icount = icount+1
            iuser(icount) = 2002
            icount = icount+1
            iuser(icount) = jcount+1
            call prinlg ( 'user', -jcount-inelem, user, iuser )
            if ( iref<0 ) then
               call errint ( iref, 1 )
               call errint ( ielgrp, 2 )
               call errint ( i, 3 )
               call errsub ( 571, 7, 0, 0 )
            end if
            if ( iref>0 ) read(iref,*) (user(jcount+k), k = 1, inelem)
            jcount = jcount+inelem
            if ( jcount==2001 .or. jcount==2002) jcount = 2003

         else if ( ifunc<0 .and. ifunc>-i ) then

!        --- ifunc<0 and ifunc>-i  Coefficient  i is identical to
!                                      coefficient -iwork(i)

            kcount = mcount
            do k = 1, -ifunc
               ih = iwork(k)
               if ( ih>=0 .and. ih<=2000 ) then
                  kcount = kcount+1
               else if (  ih>=2001 .and. ih<=2002 ) then
                  kcount = kcount+2
               else
                  kcount=kcount+1
                  if( iuser(kcount)==2001 .or. iuser(kcount)==2002 ) then
                     ih = iuser(kcount)
                     kcount = kcount+1
                  end if
               end if
            end do
            if ( ih>=0 .and. ih<=2000 ) then
               icount = icount+1
               iuser(icount) = iuser(kcount)
            else if ( ih<=2002 ) then
               icount = icount+2
               iuser(icount-1) = ih
               iuser(icount) = iuser(kcount)
            end if

         else

!        --- ifunc has wrong value

            call errint ( ifunc, 1 )
            call errint ( ielgrp, 2 )
            call errint ( i, 3 )
            call errsub ( 536, 7, 0, 0 )
         end if
      end do

      iuser(4) = icount
      user(4)  = jcount
      iuser(2) = ielgrp
      call prinlg ( 'user', -jcount-1, user, iuser )
      call prinlg ( 'iuser', icount+1, user, iuser )
1000  call erclos ( 'fil102' )
      end
