      subroutine pcgrad ( iu0, iun, matfe, kmeshlc, kproblc, intmatlc, &
                          irhsdlc, iuserloc, userloc, eps )
! ======================================================================
!
!        programmer    Peter Minev
!        version  2.2  date 13-12-2017 New storage of intmat
!        version  2.1  date 30-12-2003 Replace pcgradbf by prpcgrad
!        version  2.0  date 08-11-1999 New body
!
!   copyright (c) 1995-2017  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     finite-element preconditioned conjugate-gradient solver
!
! **********************************************************************
!
!                       KEYWORDS
!
!     conjugate_gradients
!     preconditioning
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulematr
      use sepmodulesolve
      use sepmoduleuser
      use sepmodulemain
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, target :: kmeshlc, kproblc, iuserloc(*), intmatlc(*), &
                         irhsdlc, matfe(*)
      integer :: iu0, iun
      double precision eps
      double precision, target :: userloc(*)

!     eps            i  convergence criterium for the convergence loop
!     intmatlc       i  Contains information about the structure of the
!                       matrix
!     irhsdlc        i  Contains information about the right-hand-side vector
!     iu0            i  Contain information of the initial approximation
!     iun            o  Contain information of the solution
!     iuserloc        i  Communication array for the element subroutines
!     kmeshlc        i  Output of the mesh generator - spectral mesh
!     kproblc        i  Contains information of the problem definition
!     matfe          i  Contains information of the finite-element matrix
!     userloc         i  Double precision communication array for the element
!                       subroutines
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer iinsol(2)
      double precision rinsol1(1)

!     iinsol         Contains information concerning the linear solver
!                    See subroutine pcgradbf
!     rinsol1        Contains information concerning the linear solver
!                    See subroutine pcgradbf
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     PRININ         print 1d integer array
!     PRPCGRAD       finite-element preconditioned conjugate-gradient solver
!     SEPINITINPSOL  initialize arrays inpsol and rinsol for seplinsol
!     SEPLINSOLKPROB Fill relevant parameters into sepmodulekprob for
!                    subroutine linsol
!     SEPUPDINPSOL   Update inpsol for special situations
!                    Put parameters in sepmodulesolve
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'pcgrad' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from pcgrad'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Set parameters for global CG process using FEM preconditioner

      rinsol1(1) = eps
      iinsol(1) = 100
      iinsol(2) = 2

!     --- Set parameters for FEM preconditioner
!         direct solver

      call sepinitinpsol
      inpsol(1,1) = 2
      inpsol(2,1) = 1
      kmesh2d(1) = kmeshlc
      kmesh => kmesh2d(1)
      kprob2d(1) = kproblc
      kprob => kprob2d(1)

      intmat => intmatlc(1)
      intmatar =>intmatlc(1:2)
      irhsd(1) = irhsdlc
      iuser => iuserloc(1:iuserloc(1))
      user => userloc(1:nint(userloc(1)))
      stiffmat => matfe(1:2)

      if ( debug ) call prinin ( stiffmat, 2, 'stiffmat' )

      iseqmt = stiffmat(1)
      iseqintmat = 1
      call sepupdinpsol  ! Update inpsol for special situations
      call seplinsolkprob ( iu0 )

!     --- Actual solving

      call prpcgrad ( iu0, iun, iinsol, rinsol1 )
      kmeshlc = kmesh2d(1)
      kproblc = kprob2d(1)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End pcgrad'

      end if  ! ( debug )
      call erclos ( 'pcgrad' )

      end subroutine pcgrad
