      subroutine prmumpsinfo ( info )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 19-04-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Write error message or warning in case of a mumps error
!
! **********************************************************************
!
!                       KEYWORDS
!
!     linear_solver
!     solve
!     mumps
!     error_manipulation
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none 
! **********************************************************************
!
!                       COMMON BLOCKS
!

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer, intent(in) :: info(*)

!     info           i    Information from mumps
! **********************************************************************
!
!                       LOCAL PARAMETERS
!

! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRINT         Put integer in error message
!     ERRSUB         Error messages
!     ERRWAR         Print warnings
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    3181   Error in mumps
!    3182   Warning in mumps
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'prmumpsinfo' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from prmumpsinfo'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( info(1)==-10 ) then

!     --- error in mumps, matrix is singular

         call errsub ( 3191, 0, 0, 0 )

      else if ( info(1)<0 ) then

!     --- error in mumps

         call errint ( info(1), 1 )
         call errint ( info(2), 2 )
         call errsub ( 3181, 2, 0, 0 )

      else

!     --- waring in mumps

         call errint ( info(1), 1 )
         call errint ( info(2), 2 )
         call errwar ( 3182, 2, 0, 0 )
         go to 1000

      end if  ! ( info(1)<0 )

1000  call erclos ( 'prmumpsinfo' )
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End prmumpsinfo'

      end if  ! ( debug )

      end
