      subroutine seppetscfinmat
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 09-07-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Complete creation of petsc matrix
!     This routine should be called after completing all calls to
!     MatSetValues()
! **********************************************************************
!
!                       KEYWORDS
!
!     petsc
!     matrix
!     create
! **********************************************************************
!
!                       MODULES USED
!
      use seppetscmod   ! defines all petsc stuff
      use sepmodulecomio
      use sepmodulecpack
      implicit none

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     MATASSEMBLYBEGIN  Begins assembling the matrix.
!     MATASSEMBLYEND    Ends assembling the matrix.
!     MATVIEW        Visualizes a matrix object.
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppetscfinmat' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppetscfinmat'
         write(irefwr,1) 'iseqmat', iseqmat
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

!     --- Perform assembly process
!         These steps are necessary in petsc

      call MatAssemblyBegin (A(iseqmat), MAT_FINAL_ASSEMBLY, ierrpetsc)
      call MatAssemblyEnd ( A(iseqmat), MAT_FINAL_ASSEMBLY, ierrpetsc)

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,1) 'ierrpetsc', ierrpetsc
         call MatView ( A(iseqmat), PETSC_VIEWER_STDOUT_WORLD, &
                        ierrpetsc )
         write(irefwr,1) 'ierrpetsc', ierrpetsc
         write(irefwr,*) 'End seppetscfinmat'

      end if  ! ( debug )
      call erclos ( 'seppetscfinmat' )

      end subroutine seppetscfinmat
