      subroutine seppetscsol1 ( Amat )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 30-07-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Sets the type of iteration itermethod to be used by petsc
!
! **********************************************************************
!
!                       KEYWORDS
!
!     petsc
!     linear_solver
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulemat
      use sepmodulesolve
      use seppetscmod   ! defines all petsc stuff
      implicit none
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      type (rmatrix) :: Amat

!     Amat          i/o   Structure of type matrix containing the necessary
!                         information of the matrix
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     KSPCHEBYSHEVSETEIGENVALUES  Sets estimates for the extreme eigenvalues
!                    of the preconditioned problem
!     KSPGMRESSETRESTART  Sets number of iterations at which GMRES, FGMRES
!                    and LGMRES restarts.
!     KSPRICHARDSONSETSCALE  Set the damping factor;
!                    if this routine is not called, the factor defaults to 1.0.
!     KSPSETTYPE     Builds KSP for a particular solver
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'seppetscsol1' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppetscsol1'
  1      format ( a, 1x, (10i6) )

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( debug ) &
         write(irefwr,1) 'itermethod, printlev', itermethod, Amat%iprint

      select case ( itermethod )
         case(1)
            call KSPSetType ( sepksp, 'cg', ierrpetsc )
            if ( Amat%iprint>0 ) write(irefwr,*) 'cg iteration'
         case(2)
            call KSPSetType ( sepksp, 'cgs', ierrpetsc )
            if ( Amat%iprint>0 ) write(irefwr,*) 'cgs iteration'
         case(3)
            call KSPSetType ( sepksp, 'gmres', ierrpetsc )
            call KSPGMRESSetRestart ( sepksp, Amat%ninner, ierrpetsc )
            if ( Amat%iprint>0 ) write(irefwr,*) 'gmres iteration'
         case(4, 100)
            call KSPSetType ( sepksp, 'gcr', ierrpetsc )
            if ( Amat%iprint>0 ) write(irefwr,*) 'gcr iteration'
         case(17)
            call KSPSetType ( sepksp, 'bcgs', ierrpetsc )
            if ( Amat%iprint>0 ) write(irefwr,*) 'bcgs iteration'
         case(21)
            call KSPSetType ( sepksp, 'richardson', ierrpetsc )
            call KSPRichardsonSetScale ( sepksp, dampingfact, ierrpetsc )
            if ( Amat%iprint>0 ) write(irefwr,*) 'richardson iteration'
         case(22)
            call KSPSetType ( sepksp, 'chebyshev', ierrpetsc )
            call KSPChebyshevSetEigenvalues ( sepksp, emax, emin, ierrpetsc )
            if ( Amat%iprint>0 ) write(irefwr,*) 'chebyshev iteration'
         case(23)
            call KSPSetType ( sepksp, 'fgmres', ierrpetsc )
            if ( Amat%iprint>0 ) write(irefwr,*) 'fgmres iteration'
         case(24)
            call KSPSetType ( sepksp, 'dgmres', ierrpetsc )
            if ( Amat%iprint>0 ) write(irefwr,*) 'dgmres iteration'
         case(25)
            call KSPSetType ( sepksp, 'bicg', ierrpetsc )
            if ( Amat%iprint>0 ) write(irefwr,*) 'bicg iteration'
         case(26)
            call KSPSetType ( sepksp, 'tfqmr', ierrpetsc )
            if ( Amat%iprint>0 ) write(irefwr,*) 'tfqmr iteration'
         case(27)
            call KSPSetType ( sepksp, 'tcqmr', ierrpetsc )
            if ( Amat%iprint>0 ) write(irefwr,*) 'tcqmr iteration'
         case(28)
            call KSPSetType ( sepksp, 'cr', ierrpetsc )
            if ( Amat%iprint>0 ) write(irefwr,*) 'cr iteration'
         case(29)
            call KSPSetType ( sepksp, 'lsqr', ierrpetsc )
            if ( Amat%iprint>0 ) write(irefwr,*) 'lsqr iteration'
         case(30)
            call KSPSetType ( sepksp, 'preonly', ierrpetsc )
            if ( Amat%iprint>0 ) write(irefwr,*) 'preonly iteration'
      end select  ! case ( itermethod )

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seppetscsol1'

      end if  ! ( debug )
      call erclos ( 'seppetscsol1' )

      end subroutine seppetscsol1
