      subroutine seppetscstart
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.1  date 06-09-2014 Use mpistart
!        version  1.0  date 09-07-2014
!
!   copyright (c) 2014-2014  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Start petsc
!
! **********************************************************************
!
!                       KEYWORDS
!
!     start
!     petsc
! **********************************************************************
!
!                       MODULES USED
!
      use seppetscmod   ! defines all petsc stuff
      use sepmodulecomio
      use sepmodulecpack
      implicit none


! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical :: startpetsc

!     startpetsc     If false petsc has already been started
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     MPI_Comm_rank      Returns with rank of processor
!     MPI_COMM_SIZE      Returns with number of processors
!     PetscInitialize    Starts Petsc
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
! **********************************************************************
!
!                       DATA STATEMENTS
!
      data startpetsc /.true./
! ======================================================================
!
      call eropen ( 'seppetscstart' )
      debug = .false. .and. ioutp>=0
      if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'Debug information from seppetscstart'

      end if  ! ( debug )
      if ( ierror/=0 ) go to 1000

      if ( startpetsc ) then

!      --- First call of seppetscstart

         call PetscInitialize(PETSC_NULL_CHARACTER,ierrpetsc)

!        --- Get rank of processor

         call MPI_Comm_rank(PETSC_COMM_WORLD,inodenr,ierrpetsc)

!        --- Get number of processors

         call MPI_COMM_SIZE ( PETSC_COMM_WORLD, iacnodes, ierrpetsc )

!        --- Raise inodenr, so we start with sequence number 1

        inodenr = inodenr+1

!       --- define MPI types for communications

        SPINT = MPI_INTEGER
        SPCHAR = MPI_CHARACTER
        SPDOUBLE = MPI_DOUBLE_PRECISION
        SP_ANY = MPI_ANY_SOURCE

        startpetsc = .false.   ! prevent recalling this stuff

      end if  ! ( startpetsc )

      usepetsc = .true.

1000  if ( debug ) then

!     --- Debug information

         write(irefwr,*) 'End seppetscstart'

      end if  ! ( debug )
      call erclos ( 'seppetscstart' )

      end subroutine seppetscstart
