      subroutine topq12( kmesh, itrian, ihelp )
! ======================================================================
!
!        programmer    Niek Praagman
!        version  1.1  date 09-03-2013 Use getnumelmelgrp
!        version  1.0  date 31-03-1993
!
!   copyright (c) 1993-2013  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Fill helparray with information of numbers of elements
!
! **********************************************************************
!
!                       KEYWORDS
!
!     subdivision
!     element_group
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), itrian(*), ihelp(*)

!     ihelp    o  helparray to store the number of triangles/tetrahedrons
!                 for each group augmented
!     itrian   i  indicator whether and in which way elements
!                 have been divided in triangles or tetrahedrons
!                 itrian ( ielgrp ) corresponds to the ielgrp th element
!                 group and its value has the following meaning:
!                 -1:   Skip the element group
!                  0:   No subdivision required
!                  1:   Subdivision of a quadrilateral with 4 points into 2
!                       triangles
!                  2:   Subdivision of a triangle with 6 or 7 points into 4
!                       triangles
!                  3:   Subdivision of a triangle with 6 or 7 points into 1
!                       triangle
!                  4:   Subdivision of a quadrilateral with 9 points into 8
!                       triangles
!                  5:   Subdivision of a quadrilateral with 9 points into 8
!                       triangles. The centroid must be computed by
!                       interpolation
!                  6:   Subdivision of a quadrilateral with 9 points into 2
!                       triangles.
!                  7:   Subdivision of a tetrahedron with 10, 14 or 15 points
!                       into 8 tetrahedrons
!                  8:   Subdivision of a tetrahedron with 10, 14 or 15 points
!                       into 1 tetrahedron
!                  9:   Subdivision of a hexahedron with 8 points into 6
!                       tetrahedrons
!                 10:   Subdivision of a hexahedron with 27 points into 48
!                       tetrahedrons
!                 11:   Subdivision of a hexahedron with 27 points into
!                       6 tetrahedrons
!     kmesh    i  Standard SEPRAN mesh array
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer nelgrp, inelem, i

!     i        loop variable
!     inelem   number of elements in group
!     nelgrp   number of element groups in mesh
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer getnumelmelgrp

!     GETNUMELMELGRP Determine number of elements in element group
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     Run through all groups
!     Check in which way group will be subdivided
!     Add number of (new) triangles/tetrahedrons and place on correct
!     position in array ihelp
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      nelgrp = kmesh(5)
      ihelp(1) = 0
      do 100 i = 1 , nelgrp
          inelem = getnumelmelgrp ( kmesh, i )

!         --- store number of triangles

          if ( itrian(i)==-1 ) then

!         --- Element group should be skipped

               ihelp( i+1 ) = ihelp( i )
          else if ( itrian(i)==0 ) then

!         --- No subdivision

               ihelp( i+1 ) = ihelp(i) + inelem
          else if ( itrian(i)==1 ) then

!         --- From quadrilateral in two triangles

               ihelp( i+1 ) = ihelp(i) + 2 * inelem
          else if ( itrian(i)==2 ) then

!         --- Triangle with 6 or 7 points into 4 triangles

               ihelp( i+1 ) = ihelp( i ) + 4 * inelem
          else if ( itrian(i)==3 ) then

!         --- Triangle with 6 or 7 points into 1 triangle

               ihelp( i+1 ) = ihelp( i ) + inelem
          else if ( itrian(i)==4 .or. itrian(i)==5 ) then

!         --- Quadrilateral with 9 points into 8 triangles

               ihelp( i+1 ) = ihelp( i ) + 8 * inelem
          else if ( itrian(i)==6 ) then

!         --- Quadrilateral with 9 points into 2 triangles

               ihelp( i+1 ) = ihelp( i ) + 2 * inelem
          else if ( itrian(i)==7 ) then

!         --- Tetrahedron with 10, 14 or 15 points into 8 tetrahedrons

               ihelp( i+1 ) = ihelp(i) + 8 * inelem
          else if ( itrian(i)==8 ) then

!         --- Tetrahedron with 10, 14 or 15 points into 1 tetrahedron

               ihelp( i+1 ) = ihelp(i) + 2 * inelem
          else if ( itrian(i)==9 ) then

!         --- Hexahedron with 8 points into 6 tetrahedrons

               ihelp( i+1 ) = ihelp( i ) + 6 * inelem
          else if ( itrian(i)==10 ) then

!         --- Hexahedron with 27 points into 48 tetrahedrons

               ihelp( i+1 ) = ihelp( i ) + 48 * inelem
          else if ( itrian(i)==11 ) then

!         --- Hexahedron with 27 points into 6 tetrahedrons

               ihelp( i+1 ) = ihelp( i ) + 6 * inelem
          end if
100   continue
      end
