      subroutine topqu8obst ( kmesh, kmeshc, kelem, ntria, npelnw, &
                              nelext, vertex, kline, neleml, iskip, &
                              iskpar, elemskip, ilevel, itriacolour, &
                              elemobst )
! ======================================================================
!
!        programmer    Guus Segal
!        version  1.0  date 20-07-2001 Copied from topqu8
!
!   copyright (c) 2001-2001  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!     Store all triangles or quadrilaterals of a three-dimensional mesh
!     in array KELEM
!     If the mesh consists of triangles and quadrilaterals, the quadrilaterals
!     are subdivided into triangles
!     All elements must be either linear or quadratic
!     Surface elements that belong to two volume elements are stored twice
!     Later on they can be removed in order to get the outer surface only
! **********************************************************************
!
!                       KEYWORDS
!
!     3d
!     boundary
!     mesh
!     subdivision
! **********************************************************************
!
!                       MODULES USED
!
      use sepmodulecomio
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/ctyp3df90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer kmesh(*), kmeshc(*), kelem(*), ntria, npelnw, nelext, &
              kline(*), neleml, iskip, iskpar(*), elemobst(*), &
              elemskip(*), ilevel, itriacolour(*)

      logical vertex

!     elemobst       i    Array of length nelem containing the following
!                         information
!                         elemobst(i) = 1: element i is external
!                         elemobst(i) = 2: element i intersects the obstacle
!                         elemobst(i) = 3: element i is internal
!     elemskip       i    Is only used if iskipel#0
!                         elemskip(ielem) = 0 means do not skip element
!                         elemskip(ielem) = 1 means skip element
!     ilevel         i    Defines the level of elements to be plotted if
!                         kmesh(53)>0
!     iskip          i    Indicates if element groups must be skipped (>0)
!                         or not (0)
!                         If <0 volumes must be skipped
!     iskpar         i    array of length NELGRP that is only used if ISKIP > 0
!                         It indicates if element group ielgrp must be skipped
!                         iskpar(ielgrp) = 0 or not iskpar(ielgrp) = 1
!     itriacolour    o    Same as elemobst but now for the triangles
!     kelem          o    New KMESH part c with respect to the surface elements
!                         All elements have the same number of nodes
!     kline          o    Array for extra line elements
!     kmesh          i    Standard SEPRAN array containing information of the
!                         mesh
!     kmeshc         i    kmesh part c, contains the nodal points
!                         corresponding to the elements
!     neleml         i    Estimated number of elements by user (parsurf)
!     nelext         i    Number of boundary elements
!     npelnw         o    Number of points per element in KELEM
!     ntria          o    number of triangles or quadrilaterals after removal of
!                         double ones
!     vertex         i    indicates if only the vertices of the surface elements
!                         have to be filled (true) or all points (false)
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      logical triang, bound, sevenp, quadrt
      integer ilow, ihigh, inpelm, nelgrp, inelem, kelma, kelmf, &
              ielgrp, meshtp, ikelmc, ikelem, i, j, &
              nsurfe, istare, npelmc, istart, k, kelmg, ielem

!     bound          Indicates if boundary elements are present (true) or not
!                    (false)
!     i              Counting variable
!     ielem          Element number
!     ielgrp         element group number
!     ihigh          Indicates the highest dimensionality in the grid.
!                    If ihigh=0 only 2D elements are present, otherwise (1)
!                    there are also 3D elements
!     ikelem         position in array KELEM
!     ikelmc         position in kmesh part c
!     ilow           Indicates the lowest dimensionality in the grid.
!                    If ilow=0 only 3D elements are present, otherwise (1)
!                    there are also 2D elements
!     inelem         Number of elements in a specific element group
!     inpelm         number of points per element
!     istare         Starting address of a type of element in array IBOUN3
!     istart         First position used at a part
!     j              Counting variable
!     k              Counting variable
!     kelma          Indicates if KMESH part a has been filled (>0)
!                    or not (0)
!                    If kelma>0, kelma is the starting address of KMESH
!                    part a in KMESH
!     kelmf          Indicates if KMESH part f has been filled (>0) or
!                    not (0)
!                    If kelmf>0, kelmf is the starting address of KMESH
!                    part f in KMESH
!     kelmg          Indicates if KMESH part g has been filled (>0)
!                    or not (0)
!                    If kelmg>0, kelmg is the starting address of KMESH
!                    part g in KMESH
!     meshtp         Type number of elements in the mesh or element group
!     nelgrp         Number of element groups
!     npelmc         Number of points in the surface elements as stored
!                    in IBOUN3
!     nsurfe         Number of surfaces of a three-dimensional element as
!                    stored in IBOUN3
!     quadrt         Indicates if quadratic elements have been found
!                    (true) or not
!     sevenp         Indicates if elements with 7 points have been found
!                    (true) or not (false)
!     triang         Indicates if triangular elements have been found
!                    (true) or not (false)
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS         Resets old name of previous subroutine of higher level
!     EROPEN         Produces concatenated name of local subroutine
!     ERRSUB         Error messages
!     INSTOP         Stop the program
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
!    2204   All element groups are skipped
! **********************************************************************
!
!                       PSEUDO CODE
!
!     determine whether triangles or quadrilaterals are used
!     and check whether only surfaces or surfaces and volumes are
!     considered
!     run through all groups
!         if line elements
!           place nodes in special "line" array
!         else if ( meshtp.eq.3 .or. meshtp.eq.5 .or. .not. vertex .and. &$
!                 ( meshtp.ge.4 .and. meshtp.lt.8 ) ) then
!           linear triangles or quadrilaterals
!           or quadratic ones place all nodes in element array
!           (if spectral element use special values)
!         else if ( meshtp.ge.4 .and. meshtp.le.7 ) then
!           quadratic triangles or quadrilaterals (place vertices only)
!         else if ( meshtp.eq.11 .or. meshtp.eq.13 .or. .not. vertex
!             .and. meshtp.gt.10 ) then
!           linear tetrahedron or hexahedron or 3D element with all points
!           (if spectral element use special values)
!         else if ( meshtp.ge.12 ) then
!           3D elements, vertices only
!        end if
!     Do the same loop once again for surface elements if surfaces and
!     volumes are considered in order to cancel later the correct
!     elements
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ======================================================================
!
      call eropen ( 'topqu8obst' )

      ilow   = 0
      ihigh  = 0
      neleml = 0

      inpelm = kmesh(4)
      nelgrp = kmesh(5)
      inelem = kmesh(9)
      kelma  = kmesh(15)
      kelmf  = kmesh(20)
      kelmg  = kmesh(21)

!     --- determine whether triangles or quadrilaterals are used

      triang = .false.
      quadrt = .false.
      sevenp = .false.

      do ielgrp = 1, nelgrp

         meshtp = kmesh(kelmf+ielgrp-1)

         if ( meshtp.ge.3 .and. meshtp.lt.10 ) ilow  = 1
         if ( meshtp.ge.10 ) ihigh = 1

         if ( meshtp.eq. 3 .or. meshtp.eq. 4 .or. meshtp.eq. 7 .or. &
              meshtp.eq. 8 .or. meshtp.eq.11 .or. meshtp.eq.12 .or. &
              meshtp.ge.15 .or. meshtp.eq.18 ) triang = .true.
         if ( meshtp.eq. 4 .or. meshtp.eq. 6 .or. meshtp.eq. 7 .or. &
              meshtp.eq. 8 .or. meshtp.eq.12 .or. meshtp.ge.14 ) &
              quadrt = .true.
         if ( meshtp.eq. 7 .or. meshtp.eq.15 .or. meshtp.eq.16 ) &
              sevenp = .true.

      end do

      bound  = ilow + ihigh.eq.2
      ikelmc = 0
      ikelem = 0
      ntria  = 0
      nelext = 0

      if ( triang ) then
         if ( .not. vertex ) then
            if ( sevenp ) then
               npelnw = 7
            else if ( quadrt ) then
               npelnw = 6
            else
               npelnw = 3
            end if
         else
            npelnw = 3
         end if
      else if ( quadrt .and. .not. vertex ) then
         npelnw = 9
      else
         npelnw = 4
      end if

      ielem = 0
      if ( iskip.ge.0 ) then

!     --- Standard case, run through all groups

         do ielgrp = 1, nelgrp

            meshtp = kmesh(kelmf+ielgrp-1)
            if ( nelgrp.gt.1 ) inelem = kmesh(kelmg+ielgrp-1)
            if ( kelma.gt.0 )  inpelm = kmesh(kelma+ielgrp  ) - &
                                        kmesh(kelma+ielgrp-1)
            if ( iskip.gt.0 ) then

!           --- Skip element groups if necessary

               if ( iskpar(ielgrp).eq.0 ) then

!              --- Skip this group

                  ikelmc = ikelmc + inelem * inpelm
                  ielem = ielem+inelem
                  go to 100

               end if

            end if

            if ( meshtp.lt.3 ) then

!           --- line elements, some elements may be skipped
!               place nodes in special "line" array

               if ( meshtp.eq.1 ) then

!              --- Linear lines

                  do i = 1, inelem
                     ielem = ielem+1
                     if ( elemskip(ielem).eq.ilevel ) then

!                    --- Element must not be skipped

                        kline(2*neleml+1) = kmeshc(ikelmc+1)
                        kline(2*neleml+2) = kmeshc(ikelmc+2)
                        neleml = neleml+1

                     end if
                     ikelmc = ikelmc+2
                  end do

               else if ( meshtp.eq.2 ) then

!              --- Quadratic lines (vertices only)

                  do i = 1, inelem
                     ielem = ielem+1
                     if ( elemskip(ielem).eq.ilevel ) then

!                    --- Element must not be skipped

                        kline(2*neleml+1) = kmeshc(ikelmc+1)
                        kline(2*neleml+2) = kmeshc(ikelmc+3)
                        neleml = neleml+1

                     end if
                     ikelmc = ikelmc+3
                  end do

               end if

            else if ( meshtp.eq.3 .or. meshtp.eq.5 .or. .not. vertex &
                       .and.( meshtp.ge.4 .and. meshtp.lt.8 ) ) then

!           --- linear triangles or quadrilaterals
!               or quadratic ones all points

               do i = 1, inelem
                  ielem = ielem+1
                  if ( elemskip(ielem).eq.ilevel ) then

!                 --- Element must not be skipped

                     do j = 1, inpelm
                        kelem(ikelem+j) = kmeshc(ikelmc+j)
                     end do
                     ikelem = ikelem + npelnw
                     ntria  = ntria  + 1
                     if ( bound ) nelext = nelext + 1
                     itriacolour(ntria) = elemobst(ielem)

                  end if
                  ikelmc = ikelmc+inpelm
               end do

            else if ( meshtp.ge.4 .and. meshtp.le.7 ) then

!           --- quadratic triangles or quadrilaterals (vertices only)

               do i = 1, inelem
                  ielem = ielem+1
                  if ( elemskip(ielem).eq.ilevel ) then

!                 --- Element must not be skipped

                     do j = 1, npelnw
                        kelem(ikelem+j) = kmeshc(ikelmc+2*j-1)
                     end do
                     ikelem = ikelem + npelnw
                     ntria  = ntria  + 1
                     if ( bound ) nelext = nelext + 1
                     itriacolour(ntria) = elemobst(ielem)

                  end if
                  ikelmc = ikelmc+inpelm
               end do

            else if ( meshtp.eq.11 .or. meshtp.eq.13 .or. meshtp.ge.17 &
                      .or. .not. vertex .and. meshtp.gt.10 ) then

!           --- linear tetrahedron or hexahedron or 3D element with all points

               nsurfe = itypbs(2,meshtp-10)
               istare = itypbs(3,meshtp-10)-1

               do i = 1, inelem
                  ielem = ielem+1
                  if ( elemskip(ielem).eq.ilevel ) then

!                 --- Element must not be skipped

                     do j = 1, nsurfe * npelnw
                        kelem(ikelem+j) = &
                           kmeshc(ikelmc+iboun3(istare+j))
                     end do
                     ikelem = ikelem + nsurfe * npelnw
                     do  j = 1, nsurfe
                        itriacolour(ntria+j) = elemobst(ielem)
                     end do
                     ntria  = ntria  + nsurfe

                  end if
                  ikelmc = ikelmc+inpelm
               end do

            else if ( meshtp.ge.12 ) then

!           --- 3D elements, vertices only

               npelmc = itypbs(1,meshtp-10)
               nsurfe = itypbs(2,meshtp-10)
               istare = itypbs(3,meshtp-10)-1

               do i = 1, inelem
                  ielem = ielem+1
                  if ( elemskip(ielem).eq.ilevel ) then

!                 --- Element must not be skipped

                     istart = istare
                     do j = 1, nsurfe
                        do k = 1, npelnw
                           kelem(ikelem+k) = &
                           kmeshc(ikelmc+iboun3(istart+2*k-1))
                        end do
                        ikelem = ikelem + npelnw
                        ntria  = ntria  + 1
                        itriacolour(ntria) = elemobst(ielem)
                        istart = istart + npelmc
                     end do

                  end if
                  ikelmc = ikelmc + inpelm
               end do

            end if

100      end do

         if ( nelext.eq.0 ) go to 1000

         ikelmc = 0

!        --- extra for boundary elements

         do ielgrp = 1, nelgrp

            meshtp = kmesh(kelmf+ielgrp-1)
            inelem = kmesh(kelmg+ielgrp-1)
            inpelm = kmesh(kelma+ielgrp)-kmesh(kelma+ielgrp-1)

            if ( meshtp.eq.3 .or. meshtp.eq.5 .or. .not. vertex .and. &
                 ( meshtp.ge.4 .and. meshtp.lt.8 ) ) then

!           --- linear triangles or quadrilaterals
!               or quadratic ones all points

               do i = 1, inelem
                  ielem = ielem+1
                  if ( elemskip(ielem).eq.ilevel ) then

!                 --- Element must not be skipped

                     do j = 1, inpelm
                        kelem(ikelem+j) = kmeshc(ikelmc+j)
                     end do
                     ikelem = ikelem + npelnw
                     ntria  = ntria  + 1
                     itriacolour(ntria) = elemobst(ielem)

                  end if
                  ikelmc = ikelmc+inpelm
               end do

            else if ( meshtp.ge.4 .and. meshtp.le.7 ) then

!           --- quadratic triangles or quadrilaterals (vertices only)

               do i = 1, inelem
                  ielem = ielem+1
                  if ( elemskip(ielem).eq.ilevel ) then

!                 --- Element must not be skipped

                     do j = 1, npelnw
                        kelem(ikelem+j) = kmeshc(ikelmc+2*j-1)
                     end do
                     ikelem = ikelem + npelnw
                     itriacolour(ntria) = elemobst(ielem)
                     ntria  = ntria  + 1

                  end if
                  ikelmc = ikelmc+inpelm
               end do

            else

               ikelmc = ikelmc + inelem * inpelm

            end if

         end do
         if ( ntria.eq.0 ) then

!        --- Error no active element groups found

            call errsub ( 2204, 0, 0, 0 )

         end if

      end if

1000  if ( ierror.ne.0 ) call instop
      call erclos ( 'topqu8obst' )

      end
