      subroutine writb3 ( ibuffr, inum, numarr, namevc, kprob, &
                          kmesh, ihelp )
! ====================================================================== &
!
!        programmer    Niek Praagman
!        version  3.4  date 21-11-2012 Long integers
!        version  3.3  date 04-04-2001 Corrections for more problems
!        version  3.2  date 27-06-2000 New definitions
!
!   copyright (c) 1982-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!  array kprob is written to backing storage
!
! **********************************************************************
!
!                       KEYWORDS
!
!     write
!     backing_storage
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
      include 'SPcommon/comcons3f90'
      include 'SPcommon/carrayf90'

! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      integer ibuffr(*), inum, numarr, kprob(*), kmesh(*), ihelp(*)
      character *(*) namevc

!    ibuffr    i/o   Integer buffer array in which all large data is stored
!    ihelp           Work array of length NREC2 which is used as a buffer
!    inum       i    Choice parameter at the input of subroutine WRITBS
!                    When INUM<=0, KPROB is written on the next free positions
!                    of file 2. When INUM = -1, the subroutine prints a text, as
!                    described in the Programmer's Guide.
!                    When INUM>0, KPROB is written on positions of a preceding
!                    written array, with array number INUM.
!    kmesh      i    Standard SEPRAN array containing information of the mesh
!    kprob      i    Standard SEPRAN array containing information of the problem
!                    definition. See the Programmer's Guide for a complete
!                    description
!    namevc     i    Name of the array to be written to file 2.
!                    Character variable, is used for printing only
!    numarr     o    At output NUMARR gets the sequence number of KPROB in
!                    file 2.
!                    NUMARR must be a variable
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipkpre, ipkprm, ipstar
      integer ipkprb, length, ndtvec, iprio, nsuper, kprobe, kelgrp, &
              ivec, iprob, nprob, lenglc, lengkb, isuper, &
              k, iwork(2)

!     ipkprb      Starting address of part of KPROB with respect to
!                 a complete problem in array KPROB (minus one)
!     ipkpre      Starting address of kprob part e in array IBUFFR
!     ipkprm      Starting address of kprob part m in array IBUFFR
!     iprio       Help variable to store the old priority temporarily
!                 The new priority is made equal to the old one
!     iprob       Problem number
!     ipstar      Starting address of various parts of kprob in array IBUFFR
!     isuper      Super element number
!     ivec        Loop variable: gives vector number of array of special
!                 structure
!     iwork       Work array of length 2 in which extar information to be
!                 written is stored
!     k           General loop variable
!     kelgrp      Sum over the nodes per element group for all element groups
!     kprobe      Indication of vector of special sturcture has a constant
!                 number of unknowns per point (<0) or not (>0)
!     lengkb      Length of array KPROB
!     lenglc      Length of local array in KPROB
!     length      Computed total length of the arrays to be written
!     ndtvec      Number of vectors of special structure (type 115)
!     nprob       Number of problems stored in array KPROB
!     nsuper      Number of super elements
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
!     ERCLOS    Resets old name of previous subroutine of higher level
!     EROPEN    Produces concatenated name of local subroutine
!     INI060    Reactivate existing array in IBUFFR
!     INI062    reserve space for temporary arrays
!     READFL    Read integer array from a direct access SEPRAN file
!     WRIT00    writes array in buffered form to file 2
! **********************************************************************
!
!                       I/O
!
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!     The complete array is written to file 2
!     The subroutine finds which parts of KPROB have been filled and which
!     have not been filled.
!     The actual writing is performed by subroutine WRIT00, which
!     uses a buffer of length NREC2
!     In order that preceding versions of file 2 still can be read or written
!     for each part 2 extra positions are written. The contents of these
!     two positions are completely arbitrary.
!     These 2 positions are written before each part, except the first one, &
!     however, for part n only one extra position is written and for the
!     remaining parts no extra positions are used
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'writb3' )

!     --- compute length of array

      iwork(1) = 1
      nprob = max ( 1, kprob(40) )

!     --- Compute number of problems and store starting address of part of
!         kprob (minus one ) in ipkprb

      ipkprb = 0
      do iprob = 1, nprob
         ipkprb = ipkprb + kprob(3+ipkprb)
      end do
      lengkb  =  ipkprb

!     --- Compute complete length of arrays

      length = lengkb
      kelgrp = kmesh(14)

!     --- Loop over all problems

      ipkprb = 0
      do iprob = 1, nprob

         ndtvec = kprob(ipkprb+6)
         nsuper = kprob(ipkprb+29)
         if ( ndtvec>0 ) then

!        --- ndtvec>0  kprob part e

            do ivec = 1, ndtvec
               kprobe = kprob(ipkprb+kprob(ipkprb+18)-1+kelgrp*ndtvec &
                              +ivec)
               if ( kprobe>0 ) length = length+infor1(1,kprobe)+2
            end do

         end if

!        --- Kprob parts f, g

         do k = 19, 20

            if ( kprob(ipkprb+k)>0 ) length = length &
                 + infor1(1,kprob(ipkprb+k))+2
         end do

!        --- Kprob part i

         do k = 22, 22

            if ( kprob(ipkprb+k)>0 ) length = length &
                 + infor1(1,kprob(ipkprb+k))+2
         end do

!        --- Kprob parts k, l and m

         if ( nsuper>0 ) then

            do k = 24, 26
               length = length+infor1(1,kprob(ipkprb+k))
            end do
            iprio = infor1(2,kprob(ipkprb+26))
            call ini060 ( ibuffr, -kprob(ipkprb+26) )
            infor1(2,kprob(ipkprb+26)) = iprio
            ipkprm = infor(kprob(ipkprb+26))
            do isuper = 0, nsuper-1
               length = length+ibuffr(ipkprm+2*isuper)
           end do

         end if

!        --- Kprob parts n, p-s, iinput and rinput

         if ( kprob(ipkprb+27)>0 ) length = length + &
              infor1(1,kprob(ipkprb+27))+1
         if ( kprob(ipkprb+35)>0 ) length = length + &
              infor1(1,kprob(ipkprb+35))
         do k = 36, 38
            if ( kprob(ipkprb+k)>0 ) length = length + &
                 infor1(1,kprob(ipkprb+k))+1
         end do
         do k = 41, 44
            if ( kprob(ipkprb+k)>0 ) length = length + &
                 infor1(1,kprob(ipkprb+k))+1
         end do
         k = 46
         if ( kprob(ipkprb+k)>0 ) length = length + &
              infor1(1,kprob(ipkprb+k))+1
         do k = 50, 51
            if ( kprob(ipkprb+k)>0 ) length = length + &
                 infor1(1,kprob(ipkprb+k))+1
         end do

!        --- Raise ipkprb

         ipkprb = ipkprb + kprob(3+ipkprb)

      end do

!     --- write parts of arrays to backing storage
!         Array KPROB:
!         Put version number 1) into KPROB temporarily

      k = kprob(1)
      kprob(1) = 1
      call writ00( inum, numarr, namevc, kprob, length, 1, lengkb, &
                   ihelp )
      kprob(1) = k

!     --- Loop over the problems

      ipkprb = 0
      do iprob = 1, nprob

!     --- Kprob part e

         ndtvec = kprob(ipkprb+6)
         nsuper = kprob(ipkprb+29)

!        --- Kprob part e

         if ( ndtvec>0 ) then

!        --- ndtvec>0  kprob part e

            do ivec = 1, ndtvec
               kprobe = kprob(ipkprb+kprob(ipkprb+18)-1+kelgrp*ndtvec+ &
                              ivec)
               if ( kprobe>0 ) then

!              --- kprobe>0  hence part in ibuffr or on file 3

                  iprio = infor1(2,kprobe)
                  call ini060 ( ibuffr, -kprobe )
                  infor1(2,kprobe) = iprio
                  ipkpre = infor(kprobe)
                  lenglc = infor1(1,kprobe)
                  iwork(2) = lenglc
                  call writ00(inum, numarr, namevc, iwork, length, 2, &
                              2, ihelp)
                  call writ00(inum, numarr, namevc, ibuffr(ipkpre), &
                              length, 2, lenglc, ihelp)
               end if

            end do

         end if

!        --- Kprob parts f, g

         do k = 19, 20

            if ( kprob(ipkprb+k)>0 ) then

               iprio = infor1(2,kprob(ipkprb+k))
               call ini060 ( ibuffr, -kprob(ipkprb+k) )
               infor1(2,kprob(ipkprb+k)) = iprio
               ipstar = infor(kprob(ipkprb+k))
               lenglc = infor1(1,kprob(ipkprb+k))
               iwork(2) = lenglc
               call writ00(inum, numarr, namevc, iwork, length, 2, &
                           2, ihelp)
               call writ00 ( inum, numarr, namevc, ibuffr(ipstar), &
                             length, 2, lenglc, ihelp )
            end if

         end do

!        --- Kprob part i

         do k = 22, 22

            if ( kprob(ipkprb+k)>0 ) then

               iprio = infor1(2,kprob(ipkprb+k))
               call ini060 ( ibuffr, -kprob(ipkprb+k) )
               infor1(2,kprob(ipkprb+k)) = iprio
               ipstar = infor(kprob(ipkprb+k))
               lenglc = infor1(1,kprob(ipkprb+k))
               iwork(2) = lenglc
               call writ00(inum, numarr, namevc, iwork, length, 2, &
                           2, ihelp)
               call writ00 ( inum, numarr, namevc, ibuffr(ipstar), &
                             length, 2, lenglc, ihelp )
            end if

         end do

!        --- Kprob parts k, l and m

         if ( nsuper>0) then

            do k = 24, 26
               iprio = infor1(2,kprob(ipkprb+k))
               call ini060 ( ibuffr, -kprob(ipkprb+k) )
               if ( k/=26 ) infor1(2,kprob(ipkprb+k)) = iprio
               ipkprm = infor(kprob(ipkprb+k))
               lenglc = infor1(1,kprob(ipkprb+k))
               call writ00(inum, numarr, namevc, ibuffr(ipkprm), length, &
                           2, lenglc, ihelp )
            end do

            do isuper = 1, nsuper
               lenglc = ibuffr(ipkprm+2*isuper-2)
               call ini062 ( ibuffr, lenglc, ipstar )
               call readfl ( ibuffr(ipkprm+2*isuper-1), 1, lenglc, &
                             1, 3, ibuffr(ipstar) )
               call writ00(inum, numarr, namevc, ibuffr(ipstar), length, &
                           2, lenglc, ihelp)
            end do
            infor1(2,kprob(ipkprb+26)) = iprio

         end if

!        --- Kprob parts n, p-s, iinput and rinput

         if ( kprob(ipkprb+27)>0 ) then

            iprio = infor1(2,kprob(ipkprb+27))
            call ini060 ( ibuffr, -kprob(ipkprb+27) )
            infor1(2,kprob(ipkprb+27)) = iprio
            ipstar = infor(kprob(ipkprb+27))
            lenglc = infor1(1,kprob(ipkprb+27))
            iwork(1) = lenglc
            call writ00(inum, numarr, namevc, iwork, length, 2, &
                        1, ihelp)
            call writ00 ( inum, numarr, namevc, ibuffr(ipstar), &
                          length, 2, lenglc, ihelp )
         end if

         do k = 35, 38

            if ( kprob(ipkprb+k)>0 ) then

               iprio = infor1(2,kprob(ipkprb+k))
               call ini060 ( ibuffr, -kprob(ipkprb+k) )
               infor1(2,kprob(ipkprb+k)) = iprio
               ipstar = infor(kprob(ipkprb+k))
               lenglc = infor1(1,kprob(ipkprb+k))
               if ( k/=35 ) then
                  iwork(1) = lenglc
                  call writ00(inum, numarr, namevc, iwork, length, 2, &
                              1, ihelp)
               end if
               call writ00 ( inum, numarr, namevc, ibuffr(ipstar), &
                             length, 2, lenglc, ihelp )
            end if

         end do

         do k = 41, 44

            if ( kprob(ipkprb+k)>0 ) then

               iprio = infor1(2,kprob(ipkprb+k))
               call ini060 ( ibuffr, -kprob(ipkprb+k) )
               infor1(2,kprob(ipkprb+k)) = iprio
               ipstar = infor(kprob(ipkprb+k))
               lenglc = infor1(1,kprob(ipkprb+k))
               iwork(1) = lenglc

               call writ00(inum, numarr, namevc, iwork, length, 2, &
                           1, ihelp)
               call writ00 ( inum, numarr, namevc, ibuffr(ipstar), &
                             length, 2, lenglc, ihelp )
            end if

         end do

         k = 46
         if ( kprob(ipkprb+k)>0 ) then

            iprio = infor1(2,kprob(ipkprb+k))
            call ini060 ( ibuffr, -kprob(ipkprb+k) )
            infor1(2,kprob(ipkprb+k)) = iprio
            ipstar = infor(kprob(ipkprb+k))
            lenglc = infor1(1,kprob(ipkprb+k))
            iwork(1) = lenglc
            call writ00(inum, numarr, namevc, iwork, length, 2, &
                        1, ihelp)
            call writ00 ( inum, numarr, namevc, ibuffr(ipstar), &
                          length, 2, lenglc, ihelp )
         end if
         do k = 50, 51

            if ( kprob(ipkprb+k)>0 ) then

               iprio = infor1(2,kprob(ipkprb+k))
               call ini060 ( ibuffr, -kprob(ipkprb+k) )
               infor1(2,kprob(ipkprb+k)) = iprio
               ipstar = infor(kprob(ipkprb+k))
               lenglc = infor1(1,kprob(ipkprb+k))
               iwork(1) = lenglc

               call writ00(inum, numarr, namevc, iwork, length, 2, &
                           1, ihelp)
               call writ00 ( inum, numarr, namevc, ibuffr(ipstar), &
                             length, 2, lenglc, ihelp )
            end if

         end do

!        --- Raise ipkprb

         ipkprb = ipkprb + kprob(3+ipkprb)

      end do

!     --- close writing by writing nothing but using ifirst = 3 in writ00

      call writ00 ( inum, numarr, namevc, ibuffr(ipstar), length, 3, &
                    0, ihelp )
      call erclos ( 'writb3' )
      end
