      subroutine writb4 ( ibuffr, buffer, inum, numarr, namevc, isol, &
                          iwork, kprob )
! ====================================================================== &
!
!        programmer    Niek Praagman
!        version  5.1  date 21-11-2012 Long integers
!        version  5.0  date 03-04-1999 Extra parameters ibuffr, buffer
!        version  4.1  date 18-09-1995 JvdZ: improve iprob
!
!   copyright (c) 1984-2012  "Ingenieursbureau SEPRA"
!   permission to copy or distribute this software or documentation
!   in hard copy or soft copy granted only by written license
!   obtained from "Ingenieursbureau SEPRA".
!   all rights reserved. no part of this publication may be reproduced,
!   stored in a retrieval system ( e.g., in memory, disk, or core)
!   or be transmitted by any means, electronic, mechanical, photocopy
!   recording, or otherwise, without written permission from the
!   publisher.
! **********************************************************************
!
!                       DESCRIPTION
!
!    writes array of isol type to backing storage
!
! **********************************************************************
!
!                       KEYWORDS
!
!     backing_storage
!     write
! **********************************************************************
!
!                       MODULES USED
!
      implicit none
! **********************************************************************
!
!                       COMMON BLOCKS
!
! **********************************************************************
!
!                       INPUT / OUTPUT PARAMETERS
!
      character * (*) namevc
      integer ibuffr(*), inum, numarr, isol(*), iwork(*), kprob(*)
      double precision buffer(*)

!     buffer    i/o   Buffer array
!     ibuffr    i/o   Integer buffer array in which all large data is stored
!     inum       i    Choice parameter at the input of subroutine WRITBS
!                     When INUM<=0, ISOL is written on the next free positions
!                     of file 2. When INUM = -1, the subroutine prints a text, &
!                     as described in the Programmer's Guide.
!                     When INUM>0, ISOL is written on positions of a preceding
!                     written array, with array number INUM.
!     isol       i    Standard SEPRAN array containing information of the
!                     solution
!     iwork           Work array of length NREC2 which is used as a buffer
!     kprob      i    Standard SEPRAN array, containing information of the
!                     problem
!     namevc     i    Name of the array to be written to file 2.
!                     Character variable, is used for printing only
!     numarr     o    At output NUMARR gets the sequence number of ISOL in
!                     file 2.
!                     NUMARR must be a variable
! **********************************************************************
!
!                       LOCAL PARAMETERS
!
      integer (kind=8) ipusol, iptemp,  ipkprh
      integer ihelp(2), length, nusol, lensol, iprob, &
              ipkprb, k, indprh
      logical compl

!     compl   indication whether the arithmetic is real (false)
!             or complex (true)
!     ihelp     Help array to store two positions from infor into the backing
!               storage in order to have the same files for old and new
!               versions of the backing storage subroutines
!     indprh  indication if kprob part h has been filled (>0) or not (0)
!     ipkprb  Starting address of array KPROB for actual problem number
!     ipkprh  starting address of array kprob part h
!     iprob   Actual problem number
!     iptemp  Starting address of temporary array in IBUFFR
!     ipusol  Starting address of solution array in IBUFFR
!     k       Counting variable
!     length  Total length of array to be written to backing storage
!     lensol  Length of array usol
!     nusol   Total number of unknowns
! **********************************************************************
!
!                       SUBROUTINES CALLED
!
      integer (kind=8) inisgl, inidbl, iniget, inidgt
      integer inilen

!     ER0003  Reorder solution array into array with output sequence
!     ERCLOS  Resets old name of previous subroutine of higher level
!     EROPEN  Produces concatenated name of local subroutine
!     INI060  Reactivate existing array in IBUFFR
!     INI062  Create temporary space in IBUFFR or BUFFER
!     INIDBL  Transform pointer for IBUFFR to pointer for BUFFER
!     INIDGT  Compute pointer of array in BUFFER
!     INIGET  Compute pointer of array in IBUFFR
!     INILEN  Compute length of array in IBUFFR
!     INISGL  Transform pointer for BUFFER to pointer for IBUFFR
!     WRIT00  writes array in buffered form to file 2
! **********************************************************************
!
!                       I/O
!
!    none
! **********************************************************************
!
!                       ERROR MESSAGES
!
! **********************************************************************
!
!                       PSEUDO CODE
!
!   trivial
! **********************************************************************
!
!                       DATA STATEMENTS
!
! ====================================================================== &
!
      call eropen ( 'writb4' )

!     --- Compute length of array

      call ini060 ( ibuffr, -isol(1) )
      length = 7 + inilen(isol(1))

!     --- Get solution aray into ibuffr

      lensol = inilen(isol(1))
      ipusol = iniget(isol(1))
      nusol  = isol(5)
      if ( isol(2)==110 ) then

!     --- Array is of solution type, check if renumbering must be performed

         iprob = isol(4)/1000+1
         ipkprb = 1
         do 100 k = 2, iprob
            ipkprb = ipkprb + kprob ( 2+ipkprb )
100      continue
         indprh = kprob(ipkprb+20)
         isol(2) = -isol(2)
         if ( indprh>0 ) then

!        --- Solution is renumbered
!            renumber back and store this numbering in temporary array

            call ini060 ( ibuffr, -indprh )
            call ini062 ( ibuffr, lensol, iptemp )
            iptemp = inidbl(iptemp)
            ipusol = inidgt(isol(1))
            ipkprh = iniget(indprh)
            compl = mod(isol(3),10)==1
            call er0003 ( buffer(ipusol), ibuffr(ipkprh), nusol, &
                          buffer(iptemp), compl )
            ipusol = inisgl(iptemp)

         end if

      end if
      ihelp(1) = ipusol
      ihelp(2) = lensol

!     --- Write part of array to backing storage

      call writ00 ( inum, numarr, namevc, isol, length, 1, 5, iwork )
      if ( isol(2)<0 ) isol(2) = -isol(2)

      call writ00 ( inum, numarr, namevc, ihelp, length, 2, 2, iwork )

      call writ00 ( inum, numarr, namevc, ibuffr(ihelp(1)), length, 3, &
                    ihelp(2), iwork )

      call erclos ( 'writb4' )

      end
